// massdecayvevPQV.cc is part of the CHIRON ChPT at two loops program
// collection
// Copyright (C) 2015 Johan Bijnens, v1.0
// CHIRON is licenced under the GNU GPL version 2 or later,
// see COPYING for details.
// Please respect the Guidelines, see GUIDELINES for details.

// results for the finite volume correction
// to the partially quenched masses and decay constants
// derived in
// J.~Bijnens and T.~Rössler, to be published
//
// contains the finite volume corrections from the
// partially quenched  expressions for masses and decay
// constants for three sea-quark flavours 
// in terms of the lowest order masses and the chiral limit
// decay constant

#include <iostream>
#include <iomanip>
#include <string>

#include "inputsnf.h"
#include "Linf.h"
#include "oneloopintegrals.h"
#include "finitevolumeoneloopintegrals.h"
#include "finitevolumesunsetintegrals.h"
#include "massdecayvevPQV.h"

const double pi = M_PI;
const double pi2 = M_PI*M_PI;
const double pi16 = 1./(16.*pi*pi);
const double pi162 = pow(pi16,2);

// the code that produces results with Bessel method or theta method
#ifdef CHIRONTHETA
#define AbV AbVt
#define BbV BbVt
#define CbV CbVt
#define A23bV A23bVt
#define B23bV B23bVt
#define hhV hhVt
#define hh1V hh1Vt
#define hh21V hh21Vt
#define hh27V hh27Vt
#define hhdV hhdVt
#define hh1dV hh1dVt
#define hh21dV hh21dVt
#define hh27dV hh27dVt
#define mv1s1nf3p4V   mv1s1nf3p4Vt 
#define mv1s1nf3p6V   mv1s1nf3p6Vt 
#define mv1s1nf3p6LV  mv1s1nf3p6LVt 
#define mv1s1nf3p6RV  mv1s1nf3p6RVt 
#define mv1s2nf3p4V   mv1s2nf3p4Vt 
#define mv1s2nf3p6V   mv1s2nf3p6Vt 
#define mv1s2nf3p6LV  mv1s2nf3p6LVt 
#define mv1s2nf3p6RV  mv1s2nf3p6RVt 
#define mv1s3nf3p4V   mv1s3nf3p4Vt 
#define mv1s3nf3p6V   mv1s3nf3p6Vt 
#define mv1s3nf3p6LV  mv1s3nf3p6LVt 
#define mv1s3nf3p6RV  mv1s3nf3p6RVt 
#define mv2s1nf3p4V   mv2s1nf3p4Vt 
#define mv2s1nf3p6V   mv2s1nf3p6Vt 
#define mv2s1nf3p6LV  mv2s1nf3p6LVt 
#define mv2s1nf3p6RV  mv2s1nf3p6RVt 
#define mv2s2nf3p4V   mv2s2nf3p4Vt 
#define mv2s2nf3p6V   mv2s2nf3p6Vt 
#define mv2s2nf3p6LV  mv2s2nf3p6LVt 
#define mv2s2nf3p6RV  mv2s2nf3p6RVt 
#define mv2s3nf3p4V   mv2s3nf3p4Vt 
#define mv2s3nf3p6V   mv2s3nf3p6Vt 
#define mv2s3nf3p6LV  mv2s3nf3p6LVt 
#define mv2s3nf3p6RV  mv2s3nf3p6RVt 
#define fv1s1nf3p4V   fv1s1nf3p4Vt 
#define fv1s1nf3p6V   fv1s1nf3p6Vt 
#define fv1s1nf3p6LV  fv1s1nf3p6LVt 
#define fv1s1nf3p6RV  fv1s1nf3p6RVt 
#define fv1s2nf3p4V   fv1s2nf3p4Vt 
#define fv1s2nf3p6V   fv1s2nf3p6Vt 
#define fv1s2nf3p6LV  fv1s2nf3p6LVt 
#define fv1s2nf3p6RV  fv1s2nf3p6RVt 
#define fv1s3nf3p4V   fv1s3nf3p4Vt 
#define fv1s3nf3p6V   fv1s3nf3p6Vt 
#define fv1s3nf3p6LV  fv1s3nf3p6LVt 
#define fv1s3nf3p6RV  fv1s3nf3p6RVt 
#define fv2s1nf3p4V   fv2s1nf3p4Vt 
#define fv2s1nf3p6V   fv2s1nf3p6Vt 
#define fv2s1nf3p6LV  fv2s1nf3p6LVt 
#define fv2s1nf3p6RV  fv2s1nf3p6RVt 
#define fv2s2nf3p4V   fv2s2nf3p4Vt 
#define fv2s2nf3p6V   fv2s2nf3p6Vt 
#define fv2s2nf3p6LV  fv2s2nf3p6LVt 
#define fv2s2nf3p6RV  fv2s2nf3p6RVt 
#define fv2s3nf3p4V   fv2s3nf3p4Vt 
#define fv2s3nf3p6V   fv2s3nf3p6Vt 
#define fv2s3nf3p6LV  fv2s3nf3p6LVt 
#define fv2s3nf3p6RV  fv2s3nf3p6RVt 

#else
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#ifdef CHIRONBESSEL
#define AbV AbVb
#define BbV BbVb
#define CbV CbVb
#define A23bV A23bVb
#define B23bV B23bVb
#define hhV hhVb
#define hh1V hh1Vb
#define hh21V hh21Vb
#define hh27V hh27Vb
#define hhdV hhdVb
#define hh1dV hh1dVb
#define hh21dV hh21dVb
#define hh27dV hh27dVb
#define mv1s1nf3p4V   mv1s1nf3p4Vb 
#define mv1s1nf3p6V   mv1s1nf3p6Vb 
#define mv1s1nf3p6LV  mv1s1nf3p6LVb 
#define mv1s1nf3p6RV  mv1s1nf3p6RVb 
#define mv1s2nf3p4V   mv1s2nf3p4Vb 
#define mv1s2nf3p6V   mv1s2nf3p6Vb 
#define mv1s2nf3p6LV  mv1s2nf3p6LVb 
#define mv1s2nf3p6RV  mv1s2nf3p6RVb 
#define mv1s3nf3p4V   mv1s3nf3p4Vb 
#define mv1s3nf3p6V   mv1s3nf3p6Vb 
#define mv1s3nf3p6LV  mv1s3nf3p6LVb 
#define mv1s3nf3p6RV  mv1s3nf3p6RVb 
#define mv2s1nf3p4V   mv2s1nf3p4Vb 
#define mv2s1nf3p6V   mv2s1nf3p6Vb 
#define mv2s1nf3p6LV  mv2s1nf3p6LVb 
#define mv2s1nf3p6RV  mv2s1nf3p6RVb 
#define mv2s2nf3p4V   mv2s2nf3p4Vb 
#define mv2s2nf3p6V   mv2s2nf3p6Vb 
#define mv2s2nf3p6LV  mv2s2nf3p6LVb 
#define mv2s2nf3p6RV  mv2s2nf3p6RVb 
#define mv2s3nf3p4V   mv2s3nf3p4Vb 
#define mv2s3nf3p6V   mv2s3nf3p6Vb 
#define mv2s3nf3p6LV  mv2s3nf3p6LVb 
#define mv2s3nf3p6RV  mv2s3nf3p6RVb 
#define fv1s1nf3p4V   fv1s1nf3p4Vb 
#define fv1s1nf3p6V   fv1s1nf3p6Vb 
#define fv1s1nf3p6LV  fv1s1nf3p6LVb 
#define fv1s1nf3p6RV  fv1s1nf3p6RVb 
#define fv1s2nf3p4V   fv1s2nf3p4Vb 
#define fv1s2nf3p6V   fv1s2nf3p6Vb 
#define fv1s2nf3p6LV  fv1s2nf3p6LVb 
#define fv1s2nf3p6RV  fv1s2nf3p6RVb 
#define fv1s3nf3p4V   fv1s3nf3p4Vb 
#define fv1s3nf3p6V   fv1s3nf3p6Vb 
#define fv1s3nf3p6LV  fv1s3nf3p6LVb 
#define fv1s3nf3p6RV  fv1s3nf3p6RVb 
#define fv2s1nf3p4V   fv2s1nf3p4Vb 
#define fv2s1nf3p6V   fv2s1nf3p6Vb 
#define fv2s1nf3p6LV  fv2s1nf3p6LVb 
#define fv2s1nf3p6RV  fv2s1nf3p6RVb 
#define fv2s2nf3p4V   fv2s2nf3p4Vb 
#define fv2s2nf3p6V   fv2s2nf3p6Vb 
#define fv2s2nf3p6LV  fv2s2nf3p6LVb 
#define fv2s2nf3p6RV  fv2s2nf3p6RVb 
#define fv2s3nf3p4V   fv2s3nf3p4Vb 
#define fv2s3nf3p6V   fv2s3nf3p6Vb 
#define fv2s3nf3p6LV  fv2s3nf3p6LVb 
#define fv2s3nf3p6RV  fv2s3nf3p6RVb 
#else
// just some garbage to produce an error
x = 1./0.0.;
#endif
#endif
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


// auxiliary internal functions
inline double RN(const double m11,const double m22){
  return 1./(m11-m22);
}
inline double RS(const double m11,const double m22,const double m33){
  return (m11-m22)/(m11-m33);
}
inline double RS(const double m11,const double m22,const double m33,
		 const double m44,const double m55){
  return (m11-m22)*(m11-m33)/((m11-m44)*(m11-m55));
}
inline double RS(const double m11,const double m22,const double m33,
		 const double m44,const double m55,const double m66,
		 const double m77){
  return (m11-m22)*(m11-m33)*(m11-m44)/((m11-m55)*(m11-m66)*(m11-m77));
}
inline double RD(const double m11,const double m22,const double m33,
		 const double m44,const double m55,const double m66){
  return (m11-m22)*RS(m11,m33,m44,m55,m66);
}
inline double RD(const double m11,const double m22,const double m33,
		 const double m44){
  return (m11-m22)*RS(m11,m33,m44);
}
inline double RC(const double m11,const double m44,const double m66,
		 const double me2){
  return RS(m11,m44,me2)+RS(m11,m66,me2)-RS(m11,m44,m66,me2,me2);
}
inline double RC(const double m11,const double m44,const double m55,
		 const double m66,const double mp2,const double me2){
  return RS(m11,m44,m55,mp2,me2)+RS(m11,m44,m66,mp2,me2)
    +RS(m11,m55,m66,mp2,me2)-RS(m11,m44,m55,m66,mp2,me2,me2)
    -RS(m11,m44,m55,m66,mp2,mp2,me2);
}
//*********************************************************************
// masses
// 1 valence mass 1 sea mass *****************************************

double mv1s1nf3p4V(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 2) || (B0mq.size() != 2)){
    cout << "ERROR: mv2s1nf3p4V probably called with wrong number"
	 << "of quark masses\n";}
  // two masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  //double mu2 = mu0*mu0;
  double M12p4x11V =
       + AbV(m11,L) * (  - 1./3.*m11 );

      M12p4x11V +=  + BbV(m11,L) * ( 1./3.*m11*m44 - 1./3.*pow(m11,2) )
         ;

  return M12p4x11V/pow(f0,2);
}


double mv1s1nf3p6V(const quarkmassnf mass, const Linf Liin, const double L){
  return mv1s1nf3p6LV(mass,Liin,L)+mv1s1nf3p6RV(mass,L);
}


double mv1s1nf3p6LV(const quarkmassnf mass, const Linf Liin, const double L){
  double L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu;
  int nfLi;
  Liin.out(L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu,nfLi);
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if (mu0 != mu){
    cout << "ERROR: differing mu's in mv1s1nf3p6LV\n";}
  if ( (nq != 2) || (B0mq.size() != 2)){
    cout << "ERROR: mv1s1nf3p6LV probably called with wrong number"
	 << "of quark masses\n";}
  // two masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m14 = (m11+m44)/2.;
  //double mu2 = mu0*mu0;
  double M12p6x11LV =
       + AbV(m11,L) * (  - 32*m11*m44*L7r - 16*m11*m44*L6r - 16./3.*m11
         *m44*L5r + 16*m11*m44*L4r + 8*m11*m44*L3r + 8*m11*m44*L0r - 64.
         /3.*pow(m11,2)*L8r + 32*pow(m11,2)*L7r + 32*pow(m11,2)*L6r + 
         16*pow(m11,2)*L5r - 16*pow(m11,2)*L3r + 20*pow(m11,2)*L2r + 8*
         pow(m11,2)*L1r - 16*pow(m11,2)*L0r );

      M12p6x11LV +=  + AbV(m14,L) * ( 48*m11*m44*L8r - 24*m11*m44*L5r
          + 30*m11*m44*L3r + 12*m11*m44*L0r + 48*pow(m11,2)*L8r - 24*
         pow(m11,2)*L5r + 30*pow(m11,2)*L3r + 12*pow(m11,2)*L0r );

      M12p6x11LV +=  + AbV(m44,L) * ( 128*m11*m44*L6r - 128*m11*m44*L4r
          + 32*m11*m44*L2r + 128*m11*m44*L1r );

      M12p6x11LV +=  + BbV(m11,L) * ( 16./3.*m11*pow(m44,2)*L8r + 16*
         m11*pow(m44,2)*L7r + 32*m11*pow(m44,2)*L6r - 24*m11*pow(m44,2)
         *L4r + 64./3.*pow(m11,2)*m44*L8r - 32*pow(m11,2)*m44*L7r - 48*
         pow(m11,2)*m44*L6r - 16*pow(m11,2)*m44*L5r + 32*pow(m11,2)*m44
         *L4r + 8*pow(m11,2)*m44*L3r + 8*pow(m11,2)*m44*L0r - 32*pow(
         m11,3)*L8r + 16*pow(m11,3)*L7r + 56./3.*pow(m11,3)*L5r - 8*
         pow(m11,3)*L3r - 8*pow(m11,3)*L0r );

      M12p6x11LV +=  + CbV(m11,L) * ( 32*pow(m11,2)*pow(m44,2)*L6r - 16
         *pow(m11,2)*pow(m44,2)*L4r + 32./3.*pow(m11,3)*m44*L8r - 32*
         pow(m11,3)*m44*L6r - 16./3.*pow(m11,3)*m44*L5r + 16*pow(m11,3)
         *m44*L4r - 32./3.*pow(m11,4)*L8r + 16./3.*pow(m11,4)*L5r );

      M12p6x11LV +=  + A23bV(m11,L) * ( 8*m11*L3r - 12*m11*L2r - 24*m11
         *L1r + 8*m11*L0r );

      M12p6x11LV +=  + A23bV(m14,L) * (  - 36*m11*L3r - 72*m11*L0r );

      M12p6x11LV +=  + A23bV(m44,L) * (  - 96*m11*L2r );

      M12p6x11LV +=  + B23bV(m11,L) * (  - 8*m11*m44*L3r - 8*m11*m44*
         L0r + 8*pow(m11,2)*L3r + 8*pow(m11,2)*L0r );

  return M12p6x11LV/pow(f0,4);
}


double mv1s1nf3p6RV(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 2) || (B0mq.size() != 2)){
    cout << "ERROR: mv1s1nf3p6RV probably called with wrong number"
	 << "of quark masses\n";}
  // two masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m14 = (m11+m44)/2.;
  double mu2 = mu0*mu0;

  double M12p6x11RV =
       + Ab(m11,mu2)*AbV(m11,L) * ( 10./9.*m11 );

      M12p6x11RV +=  + Ab(m11,mu2)*BbV(m11,L) * (  - 1./9.*m11*m44 + 4./
         3.*pow(m11,2) );

      M12p6x11RV +=  + Ab(m11,mu2)*CbV(m11,L) * (  - 2./9.*pow(m11,2)*
         m44 + 2./9.*pow(m11,3) );

      M12p6x11RV +=  + Ab(m14,mu2)*AbV(m14,L) * (  - 9./2.*m11 );

      M12p6x11RV +=  + Ab(m14,mu2)*BbV(m11,L) * (  - 2*m11*m44 );

      M12p6x11RV +=  + Ab(m44,mu2)*BbV(m11,L) * ( 8./9.*m11*m44 );

      M12p6x11RV +=  + Bb(m11,mu2)*AbV(m11,L) * (  - 1./9.*m11*m44 + 4./
         3.*pow(m11,2) );

      M12p6x11RV +=  + Bb(m11,mu2)*AbV(m14,L) * (  - 2*m11*m44 );

      M12p6x11RV +=  + Bb(m11,mu2)*AbV(m44,L) * ( 8./9.*m11*m44 );

      M12p6x11RV +=  + Bb(m11,mu2)*BbV(m11,L) * ( 1./9.*m11*pow(m44,2)
          - 2./3.*pow(m11,2)*m44 + 5./9.*pow(m11,3) );

      M12p6x11RV +=  + Bb(m11,mu2)*CbV(m11,L) * ( 2./9.*pow(m11,2)*pow(
         m44,2) - 4./9.*pow(m11,3)*m44 + 2./9.*pow(m11,4) );

      M12p6x11RV +=  + Cb(m11,mu2)*AbV(m11,L) * (  - 2./9.*pow(m11,2)*
         m44 + 2./9.*pow(m11,3) );

      M12p6x11RV +=  + Cb(m11,mu2)*BbV(m11,L) * ( 2./9.*pow(m11,2)*pow(
         m44,2) - 4./9.*pow(m11,3)*m44 + 2./9.*pow(m11,4) );

      M12p6x11RV +=  + pow(AbV(m11,L),2) * ( 5./9.*m11 );

      M12p6x11RV +=  + AbV(m11,L)*BbV(m11,L) * (  - 1./9.*m11*m44 + 4./
         3.*pow(m11,2) );

      M12p6x11RV +=  + AbV(m11,L)*CbV(m11,L) * (  - 2./9.*pow(m11,2)*
         m44 + 2./9.*pow(m11,3) );

      M12p6x11RV +=  + AbV(m14,L)*pi16 * ( 3*m11*m44 );

      M12p6x11RV +=  + pow(AbV(m14,L),2) * (  - 9./4.*m11 );

      M12p6x11RV +=  + AbV(m14,L)*BbV(m11,L) * (  - 2*m11*m44 );

      M12p6x11RV +=  + AbV(m44,L)*BbV(m11,L) * ( 8./9.*m11*m44 );

      M12p6x11RV +=  + pow(BbV(m11,L),2) * ( 1./18.*m11*pow(m44,2) - 1./
         3.*pow(m11,2)*m44 + 5./18.*pow(m11,3) );

      M12p6x11RV +=  + BbV(m11,L)*CbV(m11,L) * ( 2./9.*pow(m11,2)*pow(
         m44,2) - 4./9.*pow(m11,3)*m44 + 2./9.*pow(m11,4) );

      M12p6x11RV +=  + hhV(1,m11,m11,m11,m11,L,mu2) * ( 5./9.*pow(
         m11,2) );

      M12p6x11RV +=  + hhV(1,m11,m14,m14,m11,L,mu2) * ( 1./4.*m11*m44
          - pow(m11,2) );

      M12p6x11RV +=  + hhV(1,m44,m14,m14,m11,L,mu2) * ( 2*m11*m44 );

      M12p6x11RV +=  + hhV(2,m11,m11,m11,m11,L,mu2) * (  - 4./9.*pow(
         m11,2)*m44 + 4./9.*pow(m11,3) );

      M12p6x11RV +=  + hhV(2,m11,m14,m14,m11,L,mu2) * ( 5./4.*pow(
         m11,2)*m44 - 5./4.*pow(m11,3) );

      M12p6x11RV +=  + hhV(5,m11,m11,m11,m11,L,mu2) * ( 2./9.*pow(
         m11,2)*pow(m44,2) - 4./9.*pow(m11,3)*m44 + 2./9.*pow(m11,4) );

      M12p6x11RV +=  + hh1V(2,m11,m14,m14,m11,L,mu2) * (  - 2*pow(
         m11,2)*m44 + 2*pow(m11,3) );

      M12p6x11RV +=  + hh21V(1,m11,m14,m14,m11,L,mu2) * ( 3./4.*pow(
         m11,2) );

      M12p6x11RV +=  + hh21V(1,m44,m14,m14,m11,L,mu2) * ( 6*pow(m11,2)
          );

      M12p6x11RV +=  + hh21V(2,m11,m14,m14,m11,L,mu2) * ( 3./4.*pow(
         m11,2)*m44 - 3./4.*pow(m11,3) );

      M12p6x11RV +=  + hh27V(1,m11,m14,m14,m11,L,mu2) * (  - 3./4.*m11
          );

      M12p6x11RV +=  + hh27V(1,m44,m14,m14,m11,L,mu2) * (  - 6*m11 );

      M12p6x11RV +=  + hh27V(2,m11,m14,m14,m11,L,mu2) * (  - 3./4.*m11*
         m44 + 3./4.*pow(m11,2) );

  return M12p6x11RV/pow(f0,4);
}


// 2 valence mass 1 sea mass *****************************************


double mv2s1nf3p4V(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 3) || (B0mq.size() != 3)){
    cout << "ERROR: mv1s1nf3p4V probably called with wrong number"
	 << "of quark masses\n";}
  // three masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double RD14 = m11-m44;
  double RD34 = m33-m44;
  double RS143 = RS(m11,m44,m33);
  double RS341 = RS(m33,m44,m11);
  //double mu2 = mu0*mu0;
  double M13p4x21V =
       + AbV(m11,L) * ( 1./18.*RD14 - 1./9.*m33*RS143 - 2./9.*m11*RS143
          );

      M13p4x21V +=  + AbV(m33,L) * ( 1./18.*RD34 - 2./9.*m33*RS341 - 1./
         9.*m11*RS341 );

  return M13p4x21V/pow(f0,2);
}

double mv2s1nf3p6V(const quarkmassnf mass, const Linf Liin, const double L){
  return mv2s1nf3p6LV(mass,Liin,L)+mv2s1nf3p6RV(mass,L);
}

double mv2s1nf3p6LV(const quarkmassnf mass, const Linf Liin, const double L){
  double L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu;
  int nfLi;
  Liin.out(L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu,nfLi);
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if (mu0 != mu){
    cout << "ERROR: differing mu's in mv2s1nf3p6L\n";}
  if ( (nq != 3) || (B0mq.size() != 3)){
    cout << "ERROR: mv2s1nf3p6LV probably called with wrong number"
	 << "of quark masses\n";}
  // three masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m13 = (m11+m33)/2.;
  double m14 = (m11+m44)/2.;
  double m34 = (m33+m44)/2.;
  //double RN13 = RN(m11,m33);
  double RD14 = m11-m44;
  double RD34 = m33-m44;
  double RS143 = RS(m11,m44,m33);
  double RS341 = RS(m33,m44,m11);
  //double mu2 = mu0*mu0;
  double M13p6x21LV =
       + AbV(m11,L) * (  - 8./3.*m44*L4r*RD14 - 8./9.*pow(m44,2)*L8r - 
         8./3.*pow(m44,2)*L7r - 8./3.*pow(m44,2)*L6r + 4./9.*m33*L5r*
         RD14 - 5./3.*m33*L3r*RD14 - 2./3.*m33*L0r*RD14 - 16./3.*m33*
         m44*L7r - 16./3.*m33*m44*L6r*RS143 + 16./3.*m33*m44*L4r*RS143
          - 16./9.*pow(m33,2)*L8r*RS143 + 4./9.*pow(m33,2)*L5r*RS143 - 
         5./3.*m11*L3r*RD14 - 2./3.*m11*L0r*RD14 - 16./3.*m11*m44*L7r
          + 8./3.*m11*m44*L6r - 32./3.*m11*m44*L6r*RS143 - 4./9.*m11*
         m44*L5r - 8./3.*m11*m44*L4r + 8./3.*m11*m44*L4r*RS341 + 40./3.
         *m11*m44*L4r*RS143 - 8./3.*m11*m33*L8r - 16./3.*m11*m33*L8r*
         RS143 + 16./3.*m11*m33*L7r + 4./3.*m11*m33*L5r + 8./3.*m11*m33
         *L5r*RS143 - 5./3.*m11*m33*L3r - 2./3.*m11*m33*L3r*RS143 - 2./
         3.*m11*m33*L0r - 8./3.*m11*m33*L0r*RS143 - 16./9.*pow(m11,2)*
         L8r - 32./9.*pow(m11,2)*L8r*RS143 + 8*pow(m11,2)*L7r + 8./9.*
         pow(m11,2)*L5r + 8./9.*pow(m11,2)*L5r*RS341 + 28./9.*pow(
         m11,2)*L5r*RS143 - 5./3.*pow(m11,2)*L3r - 2./3.*pow(m11,2)*L3r
         *RS143 );

      M13p6x21LV +=  + AbV(m11,L) * (  - 2./3.*pow(m11,2)*L0r - 8./3.*
         pow(m11,2)*L0r*RS143 );

      M13p6x21LV +=  + AbV(m13,L) * ( 8*pow(m33,2)*L6r + 5*pow(m33,2)*
         L2r + 2*pow(m33,2)*L1r + 16*m11*m33*L6r + 10*m11*m33*L2r + 4*
         m11*m33*L1r + 8*pow(m11,2)*L6r + 5*pow(m11,2)*L2r + 2*pow(
         m11,2)*L1r );

      M13p6x21LV +=  + AbV(m14,L) * ( 12*m33*m44*L8r - 6*m33*m44*L5r + 
         15./2.*m33*m44*L3r + 3*m33*m44*L0r + 12*m11*m44*L8r - 6*m11*
         m44*L5r + 15./2.*m11*m44*L3r + 3*m11*m44*L0r + 12*m11*m33*L8r
          - 6*m11*m33*L5r + 15./2.*m11*m33*L3r + 3*m11*m33*L0r + 12*
         pow(m11,2)*L8r - 6*pow(m11,2)*L5r + 15./2.*pow(m11,2)*L3r + 3*
         pow(m11,2)*L0r );

      M13p6x21LV +=  + AbV(m33,L) * (  - 8./3.*m44*L4r*RD34 + 8./9.*
         pow(m44,2)*L8r + 8./3.*pow(m44,2)*L7r + 8./3.*pow(m44,2)*L6r
          - 5./3.*m33*L3r*RD34 - 2./3.*m33*L0r*RD34 - 32./3.*m33*m44*
         L7r + 8./3.*m33*m44*L6r - 32./3.*m33*m44*L6r*RS341 - 16./3.*
         m33*m44*L6r*RS143 - 4./3.*m33*m44*L5r - 16./3.*m33*m44*L4r + 
         16*m33*m44*L4r*RS341 + 16./3.*m33*m44*L4r*RS143 - 16./9.*pow(
         m33,2)*L8r - 32./9.*pow(m33,2)*L8r*RS341 - 16./9.*pow(m33,2)*
         L8r*RS143 + 8*pow(m33,2)*L7r + 8./9.*pow(m33,2)*L5r + 28./9.*
         pow(m33,2)*L5r*RS341 + 16./9.*pow(m33,2)*L5r*RS143 - 5./3.*
         pow(m33,2)*L3r - 2./3.*pow(m33,2)*L3r*RS341 - 2./3.*pow(m33,2)
         *L0r - 8./3.*pow(m33,2)*L0r*RS341 + 4./9.*m11*L5r*RD34 - 5./3.
         *m11*L3r*RD34 - 2./3.*m11*L0r*RD34 - 32./3.*m11*m44*L7r - 32./
         3.*m11*m44*L6r*RS341 - 8./9.*m11*m44*L5r - 8./3.*m11*m44*L4r
          + 8*m11*m44*L4r*RS341 + 8./3.*m11*m44*L4r*RS143 - 8./3.*m11*
         m33*L8r - 16./3.*m11*m33*L8r*RS341 + 32./3.*m11*m33*L7r + 4./3.
         *m11*m33*L5r );

      M13p6x21LV +=  + AbV(m33,L) * ( 32./9.*m11*m33*L5r*RS341 + 8./9.*
         m11*m33*L5r*RS143 - 5./3.*m11*m33*L3r - 2./3.*m11*m33*L3r*
         RS341 - 2./3.*m11*m33*L0r - 8./3.*m11*m33*L0r*RS341 - 32./9.*
         pow(m11,2)*L8r*RS341 + 4./3.*pow(m11,2)*L5r*RS341 );

      M13p6x21LV +=  + AbV(m34,L) * ( 12*m33*m44*L8r - 6*m33*m44*L5r + 
         15./2.*m33*m44*L3r + 3*m33*m44*L0r + 12*pow(m33,2)*L8r - 6*
         pow(m33,2)*L5r + 15./2.*pow(m33,2)*L3r + 3*pow(m33,2)*L0r + 12
         *m11*m44*L8r - 6*m11*m44*L5r + 15./2.*m11*m44*L3r + 3*m11*m44*
         L0r + 12*m11*m33*L8r - 6*m11*m33*L5r + 15./2.*m11*m33*L3r + 3*
         m11*m33*L0r );

      M13p6x21LV +=  + AbV(m44,L) * ( 64*m33*m44*L6r - 64*m33*m44*L4r
          + 16*m33*m44*L2r + 64*m33*m44*L1r + 64*m11*m44*L6r - 64*m11*
         m44*L4r + 16*m11*m44*L2r + 64*m11*m44*L1r );

      M13p6x21LV +=  + BbV(m11,m33,L) * ( 16./9.*m33*pow(m44,2)*L8r + 
         16./3.*m33*pow(m44,2)*L7r + 16./3.*m33*pow(m44,2)*L6r - 16./3.
         *pow(m33,2)*m44*L7r - 16./3.*pow(m33,2)*m44*L6r*RS143 - 16./9.
         *pow(m33,3)*L8r*RS143 + 32./9.*m11*pow(m44,2)*L8r + 32./3.*m11
         *pow(m44,2)*L7r + 32./3.*m11*pow(m44,2)*L6r - 16*m11*m33*m44*
         L7r - 16./3.*m11*m33*m44*L6r*RS341 - 32./3.*m11*m33*m44*L6r*
         RS143 - 16./9.*m11*m33*m44*L5r - 16./3.*m11*m33*m44*L4r + 16./
         3.*m11*m33*m44*L4r*RS341 + 16./3.*m11*m33*m44*L4r*RS143 - 32./
         9.*m11*pow(m33,2)*L8r*RS143 + 16./3.*m11*pow(m33,2)*L7r + 16./
         9.*m11*pow(m33,2)*L5r*RS143 - 32./3.*pow(m11,2)*m44*L7r - 32./
         3.*pow(m11,2)*m44*L6r*RS341 - 8./9.*pow(m11,2)*m44*L5r - 8./3.
         *pow(m11,2)*m44*L4r + 8./3.*pow(m11,2)*m44*L4r*RS341 + 8./3.*
         pow(m11,2)*m44*L4r*RS143 - 16./9.*pow(m11,2)*m33*L8r*RS341 + 
         32./3.*pow(m11,2)*m33*L7r + 16./9.*pow(m11,2)*m33*L5r*RS341 + 
         8./9.*pow(m11,2)*m33*L5r*RS143 - 32./9.*pow(m11,3)*L8r*RS341
          + 8./9.*pow(m11,3)*L5r*RS341 );

      M13p6x21LV +=  + BbV(m11,L) * ( 8./3.*m11*m44*L6r*RD14 - 4./3.*
         m11*m44*L4r*RD14 - 8./3.*m11*m33*L8r*RD14 + 4./3.*m11*m33*L5r*
         RD14 - 5./3.*m11*m33*L3r*RD14 - 2./3.*m11*m33*L0r*RD14 - 16./3.
         *m11*m33*m44*L6r*RS143 + 8./3.*m11*m33*m44*L4r*RS143 - 16./9.*
         pow(m11,2)*L8r*RD14 + 8./9.*pow(m11,2)*L5r*RD14 - 5./3.*pow(
         m11,2)*L3r*RD14 - 2./3.*pow(m11,2)*L0r*RD14 - 32./3.*pow(
         m11,2)*m44*L6r*RS143 + 16./3.*pow(m11,2)*m44*L4r*RS143 - 16./9.
         *pow(m11,2)*m33*L8r*RS143 + 8./9.*pow(m11,2)*m33*L5r*RS143 - 
         32./9.*pow(m11,3)*L8r*RS143 + 16./9.*pow(m11,3)*L5r*RS143 );

      M13p6x21LV +=  + BbV(m33,L) * ( 8./3.*m33*m44*L6r*RD34 - 4./3.*
         m33*m44*L4r*RD34 - 16./9.*pow(m33,2)*L8r*RD34 + 8./9.*pow(
         m33,2)*L5r*RD34 - 5./3.*pow(m33,2)*L3r*RD34 - 2./3.*pow(m33,2)
         *L0r*RD34 - 32./3.*pow(m33,2)*m44*L6r*RS341 + 16./3.*pow(
         m33,2)*m44*L4r*RS341 - 32./9.*pow(m33,3)*L8r*RS341 + 16./9.*
         pow(m33,3)*L5r*RS341 - 8./3.*m11*m33*L8r*RD34 + 4./3.*m11*m33*
         L5r*RD34 - 5./3.*m11*m33*L3r*RD34 - 2./3.*m11*m33*L0r*RD34 - 
         16./3.*m11*m33*m44*L6r*RS341 + 8./3.*m11*m33*m44*L4r*RS341 - 
         16./9.*m11*pow(m33,2)*L8r*RS341 + 8./9.*m11*pow(m33,2)*L5r*
         RS341 );

      M13p6x21LV +=  + A23bV(m11,L) * ( m33*L3r + 2*m33*L3r*RS143 + 2*
         m33*L0r + m11*L3r + 2*m11*L3r*RS143 + 2*m11*L0r );

      M13p6x21LV +=  + A23bV(m13,L) * (  - 6*m33*L2r - 12*m33*L1r - 6*
         m11*L2r - 12*m11*L1r );

      M13p6x21LV +=  + A23bV(m14,L) * (  - 9*m33*L3r - 18*m33*L0r - 9*
         m11*L3r - 18*m11*L0r );

      M13p6x21LV +=  + A23bV(m33,L) * ( m33*L3r + 2*m33*L3r*RS341 + 2*
         m33*L0r + m11*L3r + 2*m11*L3r*RS341 + 2*m11*L0r );

      M13p6x21LV +=  + A23bV(m34,L) * (  - 9*m33*L3r - 18*m33*L0r - 9*
         m11*L3r - 18*m11*L0r );

      M13p6x21LV +=  + A23bV(m44,L) * (  - 48*m33*L2r - 48*m11*L2r );

      M13p6x21LV +=  + B23bV(m11,L) * ( m33*L3r*RD14 + 2*m33*L0r*RD14
          + m11*L3r*RD14 + 2*m11*L0r*RD14 );

      M13p6x21LV +=  + B23bV(m33,L) * ( m33*L3r*RD34 + 2*m33*L0r*RD34
          + m11*L3r*RD34 + 2*m11*L0r*RD34 );

  return M13p6x21LV/pow(f0,4);
}


double mv2s1nf3p6RV(const quarkmassnf mass,const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 3) || (B0mq.size() != 3)){
    cout << "ERROR: mv2s1nf3p6RV probably called with wrong number"
	 << "of quark masses\n";}
  // three masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m13 = (m11+m33)/2.;
  double m14 = (m11+m44)/2.;
  double m34 = (m33+m44)/2.;
  double RN13 = RN(m11,m33);
  double RD14 = m11-m44;
  double RD34 = m33-m44;
  double RS143 = RS(m11,m44,m33);
  double RS341 = RS(m33,m44,m11);
  double mu2 = mu0*mu0;

  double M13p6x21RV =
       + Ab(m11,mu2)*AbV(m11,L) * ( 1./270.*RD14 - 1./135.*RS143*RD14
          + 5./36.*m33 + 1./9.*m33*RS341 + 1./270.*m33*RS143 + 13./270.
         *m33*pow(RS143,2) + 1./9.*pow(m33,2)*RN13*RS341 + 11./108.*m11
          + 2./27.*m11*RS341 + 1./30.*m11*RS143 + 17./270.*m11*pow(
         RS143,2) );

      M13p6x21RV +=  + Ab(m11,mu2)*AbV(m13,L) * ( 1./324.*RD34 + 7./324.
         *RD14 - 1./54.*RS341*RD14 - 1./162.*RS143*RD34 - 2./81.*RS143*
         RD14 + 557./648.*m33 + 1./81.*m33*RS341 + 5./81.*m33*RS143 - 1.
         /27.*m33*RS143*RS341 - 4./81.*m33*pow(RS143,2) + pow(m33,2)*
         RN13 + 79./216.*m11 + 1./81.*m11*RS341 + 1./81.*m11*RS143 - 1./
         81.*m11*RS143*RS341 );

      M13p6x21RV +=  + Ab(m11,mu2)*AbV(m14,L) * (  - 29./180.*RD14 - 7./
         30.*m44 - 19./24.*m33 + 41./360.*m33*RS143 - 1./3.*m33*m44*
         RN13 - 2./3.*pow(m33,2)*RN13 - 47./120.*m11 + 109./360.*m11*
         RS143 );

      M13p6x21RV +=  + Ab(m11,mu2)*AbV(m33,L) * ( 1./540.*RD34 + 1./540.
         *RD14 - 1./270.*RS341*RD14 - 1./270.*RS143*RD34 - 1./72.*m33
          - 1./540.*m33*RS341 + 31./540.*m33*RS143 + 1./18.*m33*RS143*
         RS341 + 1./9.*pow(m33,2)*RN13*RS143 - 1./72.*m11 + 11./540.*
         m11*RS341 - 1./540.*m11*RS143 + 1./18.*m11*RS143*RS341 );

      M13p6x21RV +=  + Ab(m11,mu2)*AbV(m34,L) * (  - 1./60.*RD14 - 1./
         30.*m44 - 5./12.*m33 + 7./120.*m33*RS143 - 1./3.*m33*m44*RN13
          - 2./3.*pow(m33,2)*RN13 - 13./60.*m11 + 43./120.*m11*RS143 );

      M13p6x21RV +=  + Ab(m11,mu2)*AbV(m44,L) * ( 4./9.*m44 + 8./9.*m33
         *m44*RN13 );

      M13p6x21RV +=  + Ab(m11,mu2)*BbV(m11,m33,L) * ( 1./27.*m11*m33*
         RS341 + 2./27.*pow(m11,2)*RS341 );

      M13p6x21RV +=  + Ab(m11,mu2)*BbV(m11,L) * ( 1./540.*pow(RD14,2)
          + 1./72.*m33*RD14 + 1./9.*m33*RS341*RD14 + 1./540.*m33*RS143*
         RD14 + 1./9.*pow(m33,2)*RN13*RS341*RD14 - 5./216.*m11*RD14 + 2.
         /27.*m11*RS341*RD14 + 1./60.*m11*RS143*RD14 + 1./27.*m11*m33*
         RS143 + 2./27.*pow(m11,2)*RS143 );

      M13p6x21RV +=  + Ab(m11,mu2)*BbV(m33,L) * ( 1./540.*RD14*RD34 - 1.
         /72.*m33*RD34 + 31./540.*m33*RS143*RD34 + 1./9.*pow(m33,2)*
         RN13*RS143*RD34 - 1./72.*m11*RD34 - 1./540.*m11*RS143*RD34 );

      M13p6x21RV +=  + Ab(m13,mu2)*AbV(m11,L) * ( 1./324.*RD34 + 7./324.
         *RD14 - 1./54.*RS341*RD14 - 1./162.*RS143*RD34 - 2./81.*RS143*
         RD14 - 19./648.*m33 + 1./81.*m33*RS341 + 5./81.*m33*RS143 - 1./
         27.*m33*RS143*RS341 - 4./81.*m33*pow(RS143,2) + 7./216.*m11 + 
         1./81.*m11*RS341 + 1./81.*m11*RS143 - 1./81.*m11*RS143*RS341 )
         ;

      M13p6x21RV +=  + Ab(m13,mu2)*AbV(m13,L) * (  - 1./4.*m33 - 1./4.*
         m11 );

      M13p6x21RV +=  + Ab(m13,mu2)*AbV(m33,L) * ( 7./324.*RD34 + 1./324.
         *RD14 - 2./81.*RS341*RD34 - 1./162.*RS341*RD14 - 1./54.*RS143*
         RD34 + 55./216.*m33 + 1./81.*m33*RS341 + 1./81.*m33*RS143 - 1./
         81.*m33*RS143*RS341 + 125./648.*m11 + 5./81.*m11*RS341 - 4./81.
         *m11*pow(RS341,2) + 1./81.*m11*RS143 - 1./27.*m11*RS143*RS341
          );

      M13p6x21RV +=  + Ab(m13,mu2)*BbV(m11,m33,L) * ( 1./9.*pow(m33,2)
          + 5./9.*m11*m33 + 1./3.*pow(m11,2) );

      M13p6x21RV +=  + Ab(m13,mu2)*BbV(m11,L) * ( 1./324.*RD14*RD34 + 1.
         /324.*pow(RD14,2) - 5./324.*m33*RD14 + 1./81.*m33*RS341*RD14
          + 1./54.*m33*RS143*RD14 - 1./108.*m11*RD14 + 1./81.*m11*RS341
         *RD14 + 1./162.*m11*RS143*RD14 );

      M13p6x21RV +=  + Ab(m13,mu2)*BbV(m33,L) * ( 1./324.*pow(RD34,2)
          + 1./324.*RD14*RD34 - 1./108.*m33*RD34 + 1./162.*m33*RS341*
         RD34 + 1./81.*m33*RS143*RD34 - 5./324.*m11*RD34 + 1./54.*m11*
         RS341*RD34 + 1./81.*m11*RS143*RD34 );

      M13p6x21RV +=  + Ab(m14,mu2)*AbV(m11,L) * (  - 29./180.*RD14 - 1./
         90.*m44 - 1./8.*m33 + 41./360.*m33*RS143 - 7./120.*m11 + 1./9.
         *m11*RS341 + 109./360.*m11*RS143 );

      M13p6x21RV +=  + Ab(m14,mu2)*AbV(m33,L) * (  - 1./60.*RD34 + 7./
         90.*m44 - 13./60.*m33 + 43./120.*m33*RS341 - 1./12.*m11 - 19./
         360.*m11*RS341 );

      M13p6x21RV +=  + Ab(m14,mu2)*AbV(m34,L) * (  - 1./36.*RD34 - 1./
         36.*RD14 - 1./18.*m44 - 1./18.*m44*RS341 - 1./18.*m44*RS143 - 
         11./12.*m33 - 13./72.*m33*RS341 - 1./8.*m33*RS143 - 11./12.*
         m11 - 1./8.*m11*RS341 - 13./72.*m11*RS143 );

      M13p6x21RV +=  + Ab(m14,mu2)*BbV(m11,m33,L) * (  - 1./9.*m33*m44
          - 2./9.*m11*m44 - 1./3.*m11*m33 + 1./9.*m11*m33*RS341 - 1./3.
         *pow(m11,2) - 1./9.*pow(m11,2)*RS341 );

      M13p6x21RV +=  + Ab(m14,mu2)*BbV(m33,L) * ( 1./8.*m33*RD34 + 1./8.
         *m11*RD34 );

      M13p6x21RV +=  + Ab(m33,mu2)*AbV(m11,L) * ( 1./540.*RD34 + 1./540.
         *RD14 - 1./270.*RS341*RD14 - 1./270.*RS143*RD34 - 1./72.*m33
          - 61./540.*m33*RS341 - 1./60.*m33*RS143 + 1./18.*m33*RS143*
         RS341 - 1./9.*pow(m33,2)*RN13*RS341 - 1./72.*m11 - 29./540.*
         m11*RS341 - 1./540.*m11*RS143 + 1./18.*m11*RS143*RS341 );

      M13p6x21RV +=  + Ab(m33,mu2)*AbV(m13,L) * ( 7./324.*RD34 + 1./324.
         *RD14 - 2./81.*RS341*RD34 - 1./162.*RS341*RD14 - 1./54.*RS143*
         RD34 - 137./216.*m33 + 1./81.*m33*RS341 + 1./81.*m33*RS143 - 1.
         /81.*m33*RS143*RS341 - pow(m33,2)*RN13 - 91./648.*m11 + 5./81.
         *m11*RS341 - 4./81.*m11*pow(RS341,2) + 1./81.*m11*RS143 - 1./
         27.*m11*RS143*RS341 );

      M13p6x21RV +=  + Ab(m33,mu2)*AbV(m14,L) * (  - 1./60.*RD34 + 3./
         10.*m44 + 9./20.*m33 + 43./120.*m33*RS341 + 1./3.*m33*m44*RN13
          + 2./3.*pow(m33,2)*RN13 + 1./4.*m11 + 7./120.*m11*RS341 );

      M13p6x21RV +=  + Ab(m33,mu2)*AbV(m33,L) * ( 1./270.*RD34 - 1./135.
         *RS341*RD34 + 11./108.*m33 + 1./30.*m33*RS341 + 17./270.*m33*
         pow(RS341,2) - 1./9.*pow(m33,2)*RN13*RS143 + 5./36.*m11 + 1./
         270.*m11*RS341 + 13./270.*m11*pow(RS341,2) );

      M13p6x21RV +=  + Ab(m33,mu2)*AbV(m34,L) * (  - 29./180.*RD34 + 1./
         10.*m44 + 11./40.*m33 + 109./360.*m33*RS341 + 1./3.*m33*m44*
         RN13 + 2./3.*pow(m33,2)*RN13 - 1./8.*m11 + 41./360.*m11*RS341
          );

      M13p6x21RV +=  + Ab(m33,mu2)*AbV(m44,L) * (  - 4./9.*m44 - 8./9.*
         m33*m44*RN13 );

      M13p6x21RV +=  + Ab(m33,mu2)*BbV(m11,m33,L) * ( 1./27.*pow(m33,2)
         *RS143 + 2./27.*m11*m33*RS143 );

      M13p6x21RV +=  + Ab(m33,mu2)*BbV(m11,L) * ( 1./540.*RD14*RD34 - 1.
         /72.*m33*RD14 - 61./540.*m33*RS341*RD14 - 1./9.*pow(m33,2)*
         RN13*RS341*RD14 - 1./72.*m11*RD14 - 29./540.*m11*RS341*RD14 );

      M13p6x21RV +=  + Ab(m33,mu2)*BbV(m33,L) * ( 1./540.*pow(RD34,2)
          - 5./216.*m33*RD34 + 1./60.*m33*RS341*RD34 - 1./27.*m33*RS143
         *RD34 + 2./27.*pow(m33,2)*RS341 - 1./9.*pow(m33,2)*RN13*RS143*
         RD34 + 1./72.*m11*RD34 + 1./540.*m11*RS341*RD34 + 1./27.*m11*
         m33*RS341 );

      M13p6x21RV +=  + Ab(m34,mu2)*AbV(m11,L) * (  - 1./60.*RD14 + 17./
         90.*m44 + 5./36.*m33 - 19./360.*m33*RS143 - 19./180.*m11 + 89./
         360.*m11*RS143 );

      M13p6x21RV +=  + Ab(m34,mu2)*AbV(m14,L) * (  - 1./36.*RD34 - 1./
         36.*RD14 - 1./18.*m44 - 1./18.*m44*RS341 - 1./18.*m44*RS143 - 
         11./12.*m33 - 13./72.*m33*RS341 - 1./8.*m33*RS143 - 11./12.*
         m11 - 1./8.*m11*RS341 - 13./72.*m11*RS143 );

      M13p6x21RV +=  + Ab(m34,mu2)*AbV(m33,L) * (  - 29./180.*RD34 - 11.
         /90.*m44 - 101./360.*m33 + 109./360.*m33*RS341 + 1./9.*m33*
         RS143 - 17./72.*m11 + 41./360.*m11*RS341 + 1./9.*m11*RS143 );

      M13p6x21RV +=  + Ab(m34,mu2)*BbV(m11,m33,L) * (  - 1./9.*m33*m44
          - 1./9.*pow(m33,2) - 1./9.*pow(m33,2)*RS143 - 2./9.*m11*m44
          - 4./9.*m11*m33 - 1./9.*pow(m11,2) + 1./9.*pow(m11,2)*RS143 )
         ;

      M13p6x21RV +=  + Ab(m34,mu2)*BbV(m11,L) * ( 1./8.*m33*RD14 + 1./8.
         *m11*RD14 );

      M13p6x21RV +=  + Ab(m44,mu2)*AbV(m11,L) * (  - 4./27.*m44 );

      M13p6x21RV +=  + Ab(m44,mu2)*AbV(m33,L) * ( 4./27.*m44 );

      M13p6x21RV +=  + Ab(m44,mu2)*BbV(m11,m33,L) * ( 8./27.*m33*m44 + 
         16./27.*m11*m44 );

      M13p6x21RV +=  + Bb(m11,mu2)*AbV(m11,L) * ( 1./540.*pow(RD14,2)
          + 1./72.*m33*RD14 + 1./540.*m33*RS143*RD14 - 5./216.*m11*RD14
          + 1./60.*m11*RS143*RD14 + 1./27.*m11*m33*RS143 + 2./27.*pow(
         m11,2)*RS143 );

      M13p6x21RV +=  + Bb(m11,mu2)*AbV(m13,L) * ( 1./324.*RD14*RD34 + 1.
         /324.*pow(RD14,2) - 5./324.*m33*RD14 + 1./81.*m33*RS341*RD14
          + 1./54.*m33*RS143*RD14 - 1./108.*m11*RD14 + 1./81.*m11*RS341
         *RD14 + 1./162.*m11*RS143*RD14 );

      M13p6x21RV +=  + Bb(m11,mu2)*AbV(m33,L) * ( 1./540.*RD14*RD34 - 1.
         /72.*m33*RD14 - 1./540.*m33*RS341*RD14 - 1./72.*m11*RD14 + 11./
         540.*m11*RS341*RD14 );

      M13p6x21RV +=  + Bb(m11,mu2)*AbV(m34,L) * ( 1./8.*m33*RD14 + 1./8.
         *m11*RD14 );

      M13p6x21RV +=  + Bb(m11,mu2)*BbV(m11,m33,L) * ( 1./27.*m11*m33*
         RS341*RD14 + 2./27.*pow(m11,2)*RS341*RD14 );

      M13p6x21RV +=  + Bb(m11,mu2)*BbV(m11,L) * ( 1./72.*m33*pow(
         RD14,2) - 5./216.*m11*pow(RD14,2) + 2./27.*m11*m33*RS143*RD14
          + 4./27.*pow(m11,2)*RS143*RD14 );

      M13p6x21RV +=  + Bb(m11,mu2)*BbV(m33,L) * (  - 1./72.*m33*RD14*
         RD34 - 1./72.*m11*RD14*RD34 );

      M13p6x21RV +=  + Bb(m33,mu2)*AbV(m11,L) * ( 1./540.*RD14*RD34 - 1.
         /72.*m33*RD34 - 1./60.*m33*RS143*RD34 - 1./72.*m11*RD34 - 1./
         540.*m11*RS143*RD34 );

      M13p6x21RV +=  + Bb(m33,mu2)*AbV(m13,L) * ( 1./324.*pow(RD34,2)
          + 1./324.*RD14*RD34 - 1./108.*m33*RD34 + 1./162.*m33*RS341*
         RD34 + 1./81.*m33*RS143*RD34 - 5./324.*m11*RD34 + 1./54.*m11*
         RS341*RD34 + 1./81.*m11*RS143*RD34 );

      M13p6x21RV +=  + Bb(m33,mu2)*AbV(m14,L) * ( 1./8.*m33*RD34 + 1./8.
         *m11*RD34 );

      M13p6x21RV +=  + Bb(m33,mu2)*AbV(m33,L) * ( 1./540.*pow(RD34,2)
          - 5./216.*m33*RD34 + 1./60.*m33*RS341*RD34 + 1./27.*m33*RS143
         *RD34 + 2./27.*pow(m33,2)*RS341 + 1./72.*m11*RD34 + 1./540.*
         m11*RS341*RD34 + 1./27.*m11*m33*RS341 );

      M13p6x21RV +=  + Bb(m33,mu2)*BbV(m11,m33,L) * ( 1./27.*pow(m33,2)
         *RS143*RD34 + 2./27.*m11*m33*RS143*RD34 );

      M13p6x21RV +=  + Bb(m33,mu2)*BbV(m11,L) * (  - 1./72.*m33*RD14*
         RD34 - 1./72.*m11*RD14*RD34 );

      M13p6x21RV +=  + Bb(m33,mu2)*BbV(m33,L) * (  - 5./216.*m33*pow(
         RD34,2) + 4./27.*pow(m33,2)*RS341*RD34 + 1./72.*m11*pow(
         RD34,2) + 2./27.*m11*m33*RS341*RD34 );

      M13p6x21RV +=  + AbV(m11,L)*pi16 * ( 1./144.*m33*RD34 + 1./36.*
         m33*RD14 - 1./72.*m33*RS143*RD34 - 1./72.*m33*RS143*RD14 - 1./
         16.*m33*m44 + 1./8.*m33*m44*RS143 - 1./108.*pow(m33,2) - 1./54.
         *pow(m33,2)*RS341 + 1./72.*pow(m33,2)*RS143 + 1./27.*pow(
         m33,2)*RS143*RS341 + 1./108.*pow(m33,2)*pow(RS143,2) + 1./144.
         *m11*RD34 + 1./36.*m11*RD14 - 1./72.*m11*RS143*RD34 - 1./72.*
         m11*RS143*RD14 - 1./16.*m11*m44 + 1./8.*m11*m44*RS143 + 1./432.
         *m11*m33 - 5./216.*m11*m33*RS341 - 1./36.*m11*m33*RS143 + 5./
         108.*m11*m33*RS143*RS341 + 5./108.*m11*m33*pow(RS143,2) + 5./
         432.*pow(m11,2) - 1./216.*pow(m11,2)*RS341 - 1./24.*pow(m11,2)
         *RS143 + 1./108.*pow(m11,2)*RS143*RS341 + 1./27.*pow(m11,2)*
         pow(RS143,2) );

      M13p6x21RV +=  + pow(AbV(m11,L),2) * ( 1./540.*RD14 - 1./270.*
         RS143*RD14 + 5./72.*m33 + 1./540.*m33*RS143 + 13./540.*m33*
         pow(RS143,2) + 11./216.*m11 + 1./60.*m11*RS143 + 17./540.*m11*
         pow(RS143,2) );

      M13p6x21RV +=  + AbV(m11,L)*AbV(m13,L) * ( 1./324.*RD34 + 7./324.
         *RD14 - 1./54.*RS341*RD14 - 1./162.*RS143*RD34 - 2./81.*RS143*
         RD14 - 19./648.*m33 + 1./81.*m33*RS341 + 5./81.*m33*RS143 - 1./
         27.*m33*RS143*RS341 - 4./81.*m33*pow(RS143,2) + 7./216.*m11 + 
         1./81.*m11*RS341 + 1./81.*m11*RS143 - 1./81.*m11*RS143*RS341 )
         ;

      M13p6x21RV +=  + AbV(m11,L)*AbV(m14,L) * (  - 29./180.*RD14 - 1./
         90.*m44 - 1./8.*m33 + 41./360.*m33*RS143 - 7./120.*m11 + 1./9.
         *m11*RS341 + 109./360.*m11*RS143 );

      M13p6x21RV +=  + AbV(m11,L)*AbV(m33,L) * ( 1./540.*RD34 + 1./540.
         *RD14 - 1./270.*RS341*RD14 - 1./270.*RS143*RD34 - 1./72.*m33
          - 1./540.*m33*RS341 - 1./60.*m33*RS143 + 1./18.*m33*RS143*
         RS341 - 1./72.*m11 + 11./540.*m11*RS341 - 1./540.*m11*RS143 + 
         1./18.*m11*RS143*RS341 );

      M13p6x21RV +=  + AbV(m11,L)*AbV(m34,L) * (  - 1./60.*RD14 + 17./
         90.*m44 + 5./36.*m33 - 19./360.*m33*RS143 - 19./180.*m11 + 89./
         360.*m11*RS143 );

      M13p6x21RV +=  + AbV(m11,L)*AbV(m44,L) * (  - 4./27.*m44 );

      M13p6x21RV +=  + AbV(m11,L)*BbV(m11,m33,L) * ( 1./27.*m11*m33*
         RS341 + 2./27.*pow(m11,2)*RS341 );

      M13p6x21RV +=  + AbV(m11,L)*BbV(m11,L) * ( 1./540.*pow(RD14,2) + 
         1./72.*m33*RD14 + 1./540.*m33*RS143*RD14 - 5./216.*m11*RD14 + 
         1./60.*m11*RS143*RD14 + 1./27.*m11*m33*RS143 + 2./27.*pow(
         m11,2)*RS143 );

      M13p6x21RV +=  + AbV(m11,L)*BbV(m33,L) * ( 1./540.*RD14*RD34 - 1./
         72.*m33*RD34 - 1./60.*m33*RS143*RD34 - 1./72.*m11*RD34 - 1./
         540.*m11*RS143*RD34 );

      M13p6x21RV +=  + AbV(m13,L)*pi16 * ( 1./144.*pow(m33,2) - 1./72.*
         pow(m33,2)*RS341 + 1./144.*pow(m33,2)*pow(RS341,2) - 1./72.*
         pow(m33,2)*RS143 + 1./72.*pow(m33,2)*RS143*RS341 + 1./144.*
         pow(m33,2)*pow(RS143,2) + 1./72.*m11*m33 - 1./36.*m11*m33*
         RS341 + 1./72.*m11*m33*pow(RS341,2) - 1./36.*m11*m33*RS143 + 1.
         /36.*m11*m33*RS143*RS341 + 1./72.*m11*m33*pow(RS143,2) + 1./
         144.*pow(m11,2) - 1./72.*pow(m11,2)*RS341 + 1./144.*pow(m11,2)
         *pow(RS341,2) - 1./72.*pow(m11,2)*RS143 + 1./72.*pow(m11,2)*
         RS143*RS341 + 1./144.*pow(m11,2)*pow(RS143,2) );

      M13p6x21RV +=  + pow(AbV(m13,L),2) * (  - 1./8.*m33 - 1./8.*m11 )
         ;

      M13p6x21RV +=  + AbV(m13,L)*AbV(m33,L) * ( 7./324.*RD34 + 1./324.
         *RD14 - 2./81.*RS341*RD34 - 1./162.*RS341*RD14 - 1./54.*RS143*
         RD34 + 55./216.*m33 + 1./81.*m33*RS341 + 1./81.*m33*RS143 - 1./
         81.*m33*RS143*RS341 + 125./648.*m11 + 5./81.*m11*RS341 - 4./81.
         *m11*pow(RS341,2) + 1./81.*m11*RS143 - 1./27.*m11*RS143*RS341
          );

      M13p6x21RV +=  + AbV(m13,L)*BbV(m11,m33,L) * ( 1./9.*pow(m33,2)
          + 5./9.*m11*m33 + 1./3.*pow(m11,2) );

      M13p6x21RV +=  + AbV(m13,L)*BbV(m11,L) * ( 1./324.*RD14*RD34 + 1./
         324.*pow(RD14,2) - 5./324.*m33*RD14 + 1./81.*m33*RS341*RD14 + 
         1./54.*m33*RS143*RD14 - 1./108.*m11*RD14 + 1./81.*m11*RS341*
         RD14 + 1./162.*m11*RS143*RD14 );

      M13p6x21RV +=  + AbV(m13,L)*BbV(m33,L) * ( 1./324.*pow(RD34,2) + 
         1./324.*RD14*RD34 - 1./108.*m33*RD34 + 1./162.*m33*RS341*RD34
          + 1./81.*m33*RS143*RD34 - 5./324.*m11*RD34 + 1./54.*m11*RS341
         *RD34 + 1./81.*m11*RS143*RD34 );

      M13p6x21RV +=  + AbV(m14,L)*pi16 * (  - 1./16.*m33*RD34 + 21./32.
         *m33*m44 + 1./32.*m33*m44*RS341 + 1./32.*m33*m44*RS143 + 1./4.
         *pow(m33,2) - 1./16.*m11*RD34 + 21./32.*m11*m44 + 1./32.*m11*
         m44*RS341 + 1./32.*m11*m44*RS143 + 1./32.*m11*m33 + 1./32.*m11
         *m33*RS341 + 1./32.*m11*m33*RS143 - 7./32.*pow(m11,2) + 1./32.
         *pow(m11,2)*RS341 + 1./32.*pow(m11,2)*RS143 );

      M13p6x21RV +=  + AbV(m14,L)*AbV(m33,L) * (  - 1./60.*RD34 + 7./90.
         *m44 - 13./60.*m33 + 43./120.*m33*RS341 - 1./12.*m11 - 19./360.
         *m11*RS341 );

      M13p6x21RV +=  + AbV(m14,L)*AbV(m34,L) * (  - 1./36.*RD34 - 1./36.
         *RD14 - 1./18.*m44 - 1./18.*m44*RS341 - 1./18.*m44*RS143 - 11./
         12.*m33 - 13./72.*m33*RS341 - 1./8.*m33*RS143 - 11./12.*m11 - 
         1./8.*m11*RS341 - 13./72.*m11*RS143 );

      M13p6x21RV +=  + AbV(m14,L)*BbV(m11,m33,L) * (  - 1./9.*m33*m44
          - 2./9.*m11*m44 - 1./3.*m11*m33 + 1./9.*m11*m33*RS341 - 1./3.
         *pow(m11,2) - 1./9.*pow(m11,2)*RS341 );

      M13p6x21RV +=  + AbV(m14,L)*BbV(m33,L) * ( 1./8.*m33*RD34 + 1./8.
         *m11*RD34 );

      M13p6x21RV +=  + AbV(m33,L)*pi16 * ( 1./36.*m33*RD34 + 1./144.*
         m33*RD14 - 1./72.*m33*RS341*RD34 - 1./72.*m33*RS341*RD14 - 1./
         16.*m33*m44 + 1./8.*m33*m44*RS341 + 5./432.*pow(m33,2) - 1./24.
         *pow(m33,2)*RS341 + 1./27.*pow(m33,2)*pow(RS341,2) - 1./216.*
         pow(m33,2)*RS143 + 1./108.*pow(m33,2)*RS143*RS341 + 1./36.*m11
         *RD34 + 1./144.*m11*RD14 - 1./72.*m11*RS341*RD34 - 1./72.*m11*
         RS341*RD14 - 1./16.*m11*m44 + 1./8.*m11*m44*RS341 + 1./432.*
         m11*m33 - 1./36.*m11*m33*RS341 + 5./108.*m11*m33*pow(RS341,2)
          - 5./216.*m11*m33*RS143 + 5./108.*m11*m33*RS143*RS341 - 1./
         108.*pow(m11,2) + 1./72.*pow(m11,2)*RS341 + 1./108.*pow(m11,2)
         *pow(RS341,2) - 1./54.*pow(m11,2)*RS143 + 1./27.*pow(m11,2)*
         RS143*RS341 );

      M13p6x21RV +=  + pow(AbV(m33,L),2) * ( 1./540.*RD34 - 1./270.*
         RS341*RD34 + 11./216.*m33 + 1./60.*m33*RS341 + 17./540.*m33*
         pow(RS341,2) + 1./27.*m33*RS143 + 5./72.*m11 + 1./540.*m11*
         RS341 + 13./540.*m11*pow(RS341,2) );

      M13p6x21RV +=  + AbV(m33,L)*AbV(m34,L) * (  - 29./180.*RD34 - 11./
         90.*m44 - 101./360.*m33 + 109./360.*m33*RS341 + 1./9.*m33*
         RS143 - 17./72.*m11 + 41./360.*m11*RS341 + 1./9.*m11*RS143 );

      M13p6x21RV +=  + AbV(m33,L)*AbV(m44,L) * ( 4./27.*m44 );

      M13p6x21RV +=  + AbV(m33,L)*BbV(m11,m33,L) * ( 1./27.*pow(m33,2)*
         RS143 + 2./27.*m11*m33*RS143 );

      M13p6x21RV +=  + AbV(m33,L)*BbV(m11,L) * ( 1./540.*RD14*RD34 - 1./
         72.*m33*RD14 - 1./540.*m33*RS341*RD14 - 1./72.*m11*RD14 + 11./
         540.*m11*RS341*RD14 );

      M13p6x21RV +=  + AbV(m33,L)*BbV(m33,L) * ( 1./540.*pow(RD34,2) - 
         5./216.*m33*RD34 + 1./60.*m33*RS341*RD34 + 1./27.*m33*RS143*
         RD34 + 2./27.*pow(m33,2)*RS341 + 1./72.*m11*RD34 + 1./540.*m11
         *RS341*RD34 + 1./27.*m11*m33*RS341 );

      M13p6x21RV +=  + AbV(m34,L)*pi16 * (  - 1./16.*m33*RD14 + 21./32.
         *m33*m44 + 1./32.*m33*m44*RS341 + 1./32.*m33*m44*RS143 - 7./32.
         *pow(m33,2) + 1./32.*pow(m33,2)*RS341 + 1./32.*pow(m33,2)*
         RS143 - 1./16.*m11*RD14 + 21./32.*m11*m44 + 1./32.*m11*m44*
         RS341 + 1./32.*m11*m44*RS143 + 1./32.*m11*m33 + 1./32.*m11*m33
         *RS341 + 1./32.*m11*m33*RS143 + 1./4.*pow(m11,2) );

      M13p6x21RV +=  + AbV(m34,L)*BbV(m11,m33,L) * (  - 1./9.*m33*m44
          - 1./9.*pow(m33,2) - 1./9.*pow(m33,2)*RS143 - 2./9.*m11*m44
          - 4./9.*m11*m33 - 1./9.*pow(m11,2) + 1./9.*pow(m11,2)*RS143 )
         ;

      M13p6x21RV +=  + AbV(m34,L)*BbV(m11,L) * ( 1./8.*m33*RD14 + 1./8.
         *m11*RD14 );

      M13p6x21RV +=  + AbV(m44,L)*BbV(m11,m33,L) * ( 8./27.*m33*m44 + 
         16./27.*m11*m44 );

      M13p6x21RV +=  + BbV(m11,m33,L)*BbV(m11,L) * ( 1./27.*m11*m33*
         RS341*RD14 + 2./27.*pow(m11,2)*RS341*RD14 );

      M13p6x21RV +=  + BbV(m11,m33,L)*BbV(m33,L) * ( 1./27.*pow(m33,2)*
         RS143*RD34 + 2./27.*m11*m33*RS143*RD34 );

      M13p6x21RV +=  + BbV(m11,L)*pi16 * ( 1./144.*m33*RD14*RD34 + 1./
         144.*m33*pow(RD14,2) - 1./16.*m33*m44*RD14 - 1./108.*pow(
         m33,2)*RD14 - 1./54.*pow(m33,2)*RS341*RD14 - 1./216.*pow(
         m33,2)*RS143*RD14 + 1./144.*m11*RD14*RD34 + 1./144.*m11*pow(
         RD14,2) - 1./16.*m11*m44*RD14 + 1./432.*m11*m33*RD14 - 5./216.
         *m11*m33*RS341*RD14 - 5./216.*m11*m33*RS143*RD14 + 5./432.*
         pow(m11,2)*RD14 - 1./216.*pow(m11,2)*RS341*RD14 - 1./54.*pow(
         m11,2)*RS143*RD14 );

      M13p6x21RV +=  + pow(BbV(m11,L),2) * ( 1./144.*m33*pow(RD14,2) - 
         5./432.*m11*pow(RD14,2) + 1./27.*m11*m33*RS143*RD14 + 2./27.*
         pow(m11,2)*RS143*RD14 );

      M13p6x21RV +=  + BbV(m11,L)*BbV(m33,L) * (  - 1./72.*m33*RD14*
         RD34 - 1./72.*m11*RD14*RD34 );

      M13p6x21RV +=  + BbV(m33,L)*pi16 * ( 1./144.*m33*pow(RD34,2) + 1./
         144.*m33*RD14*RD34 - 1./16.*m33*m44*RD34 + 5./432.*pow(m33,2)*
         RD34 - 1./54.*pow(m33,2)*RS341*RD34 - 1./216.*pow(m33,2)*RS143
         *RD34 + 1./144.*m11*pow(RD34,2) + 1./144.*m11*RD14*RD34 - 1./
         16.*m11*m44*RD34 + 1./432.*m11*m33*RD34 - 5./216.*m11*m33*
         RS341*RD34 - 5./216.*m11*m33*RS143*RD34 - 1./108.*pow(m11,2)*
         RD34 - 1./216.*pow(m11,2)*RS341*RD34 - 1./54.*pow(m11,2)*RS143
         *RD34 );

      M13p6x21RV +=  + pow(BbV(m33,L),2) * (  - 5./432.*m33*pow(RD34,2)
          + 2./27.*pow(m33,2)*RS341*RD34 + 1./144.*m11*pow(RD34,2) + 1./
         27.*m11*m33*RS341*RD34 );

      M13p6x21RV +=  + A23bV(m11,L)*pi16 * (  - 1./72.*m33 + 1./72.*m33
         *RS341 + 1./24.*m33*RS143 - 1./36.*m33*RS143*RS341 - 1./36.*
         m33*pow(RS143,2) - 1./72.*m11 + 1./72.*m11*RS341 + 1./24.*m11*
         RS143 - 1./36.*m11*RS143*RS341 - 1./36.*m11*pow(RS143,2) );

      M13p6x21RV +=  + A23bV(m13,L)*pi16 * (  - 1./24.*m33 + 1./12.*m33
         *RS341 - 1./24.*m33*pow(RS341,2) + 1./12.*m33*RS143 - 1./12.*
         m33*RS143*RS341 - 1./24.*m33*pow(RS143,2) - 1./24.*m11 + 1./12.
         *m11*RS341 - 1./24.*m11*pow(RS341,2) + 1./12.*m11*RS143 - 1./
         12.*m11*RS143*RS341 - 1./24.*m11*pow(RS143,2) );

      M13p6x21RV +=  + A23bV(m14,L)*pi16 * ( 3./16.*m33 - 3./16.*m33*
         RS341 - 3./16.*m33*RS143 + 3./16.*m11 - 3./16.*m11*RS341 - 3./
         16.*m11*RS143 );

      M13p6x21RV +=  + A23bV(m33,L)*pi16 * (  - 1./72.*m33 + 1./24.*m33
         *RS341 - 1./36.*m33*pow(RS341,2) + 1./72.*m33*RS143 - 1./36.*
         m33*RS143*RS341 - 1./72.*m11 + 1./24.*m11*RS341 - 1./36.*m11*
         pow(RS341,2) + 1./72.*m11*RS143 - 1./36.*m11*RS143*RS341 );

      M13p6x21RV +=  + A23bV(m34,L)*pi16 * ( 3./16.*m33 - 3./16.*m33*
         RS341 - 3./16.*m33*RS143 + 3./16.*m11 - 3./16.*m11*RS341 - 3./
         16.*m11*RS143 );

      M13p6x21RV +=  + B23bV(m11,L)*pi16 * (  - 1./72.*m33*RD14 + 1./72.
         *m33*RS341*RD14 + 1./72.*m33*RS143*RD14 - 1./72.*m11*RD14 + 1./
         72.*m11*RS341*RD14 + 1./72.*m11*RS143*RD14 );

      M13p6x21RV +=  + B23bV(m33,L)*pi16 * (  - 1./72.*m33*RD34 + 1./72.
         *m33*RS341*RD34 + 1./72.*m33*RS143*RD34 - 1./72.*m11*RD34 + 1./
         72.*m11*RS341*RD34 + 1./72.*m11*RS143*RD34 );

      M13p6x21RV +=  + hhV(1,m11,m11,m13,m13,L,mu2) * ( 1./324.*pow(
         RD14,2) - 1./81.*m33*RS143*RD14 - 5./36.*pow(m33,2) + 1./18.*
         pow(m33,2)*RS143 - 7./162.*pow(m33,2)*pow(RS143,2) - 2./81.*
         m11*RS143*RD14 - 29./72.*m11*m33 + 1./9.*m11*m33*RS143 - 5./81.
         *m11*m33*pow(RS143,2) - 19./72.*pow(m11,2) + 1./18.*pow(m11,2)
         *RS143 - 1./162.*pow(m11,2)*pow(RS143,2) );

      M13p6x21RV +=  + hhV(1,m11,m13,m33,m13,L,mu2) * ( 1./162.*RD14*
         RD34 + 1./81.*m33*RS341*RD14 + 2./81.*m33*RS143*RD34 + 4./81.*
         pow(m33,2)*RS143*RS341 + 2./81.*m11*RS341*RD14 + 1./81.*m11*
         RS143*RD34 + 10./81.*m11*m33*RS143*RS341 + 4./81.*pow(m11,2)*
         RS143*RS341 );

      M13p6x21RV +=  + hhV(1,m11,m14,m34,m13,L,mu2) * ( 1./4.*pow(
         m33,2) - 1./4.*pow(m33,2)*RS143 + 5./8.*m11*m33 - 5./8.*m11*
         m33*RS143 + 3./8.*pow(m11,2) - 3./8.*pow(m11,2)*RS143 );

      M13p6x21RV +=  + hhV(1,m13,m11,m11,m13,L,mu2) * ( 5./144.*pow(
         m33,2) - 1./72.*pow(m33,2)*RS143 + 1./72.*pow(m33,2)*pow(
         RS143,2) + 5./72.*m11*m33 - 1./36.*m11*m33*RS143 + 1./36.*m11*
         m33*pow(RS143,2) + 5./144.*pow(m11,2) - 1./72.*pow(m11,2)*
         RS143 + 1./72.*pow(m11,2)*pow(RS143,2) );

      M13p6x21RV +=  + hhV(1,m13,m11,m33,m13,L,mu2) * ( 1./144.*pow(
         m33,2) - 1./72.*pow(m33,2)*RS341 - 1./72.*pow(m33,2)*RS143 + 1.
         /36.*pow(m33,2)*RS143*RS341 + 1./72.*m11*m33 - 1./36.*m11*m33*
         RS341 - 1./36.*m11*m33*RS143 + 1./18.*m11*m33*RS143*RS341 + 1./
         144.*pow(m11,2) - 1./72.*pow(m11,2)*RS341 - 1./72.*pow(m11,2)*
         RS143 + 1./36.*pow(m11,2)*RS143*RS341 );

      M13p6x21RV +=  + hhV(1,m13,m13,m13,m13,L,mu2) * ( 1./16.*pow(
         m33,2) + 1./8.*m11*m33 + 1./16.*pow(m11,2) );

      M13p6x21RV +=  + hhV(1,m13,m33,m33,m13,L,mu2) * ( 1./324.*pow(
         RD34,2) - 1./54.*m33*RD34 + 1./81.*m33*RS341*RD34 + 25./144.*
         pow(m33,2) + 1./216.*pow(m33,2)*RS341 + 17./648.*pow(m33,2)*
         pow(RS341,2) - 1./54.*m11*RD34 + 2./81.*m11*RS341*RD34 + 25./
         72.*m11*m33 - 1./36.*m11*m33*RS341 + 25./324.*m11*m33*pow(
         RS341,2) + 25./144.*pow(m11,2) - 7./216.*pow(m11,2)*RS341 + 41.
         /648.*pow(m11,2)*pow(RS341,2) );

      M13p6x21RV +=  + hhV(1,m14,m11,m34,m13,L,mu2) * ( 1./16.*m33*m44*
         RS143 + 1./16.*m11*m44*RS143 + 1./16.*m11*m33*RS143 + 1./16.*
         pow(m11,2)*RS143 );

      M13p6x21RV +=  + hhV(1,m14,m33,m34,m13,L,mu2) * (  - 1./6.*m33*
         RD34 - 19./48.*m33*m44 + 1./16.*m33*m44*RS341 + 1./12.*pow(
         m33,2) - 5./12.*pow(m33,2)*RS341 - 1./6.*m11*RD34 - 19./48.*
         m11*m44 + 1./16.*m11*m44*RS341 - 11./48.*m11*m33 - 7./16.*m11*
         m33*RS341 - 5./16.*pow(m11,2) - 1./48.*pow(m11,2)*RS341 );

      M13p6x21RV +=  + hhV(1,m33,m13,m33,m13,L,mu2) * (  - 1./8.*pow(
         m33,2) - 1./8.*m11*m33 );

      M13p6x21RV +=  + hhV(1,m33,m14,m34,m13,L,mu2) * ( 1./8.*pow(
         m33,2) - 1./8.*pow(m33,2)*RS341 + 1./8.*m11*m33 - 1./8.*m11*
         m33*RS341 );

      M13p6x21RV +=  + hhV(1,m34,m11,m14,m13,L,mu2) * (  - 1./16.*m33*
         m44 + 1./16.*m33*m44*RS143 - 1./16.*pow(m33,2) + 1./16.*pow(
         m33,2)*RS143 - 1./16.*m11*m44 + 1./16.*m11*m44*RS143 - 1./16.*
         m11*m33 + 1./16.*m11*m33*RS143 );

      M13p6x21RV +=  + hhV(1,m34,m14,m33,m13,L,mu2) * ( 1./16.*m33*m44*
         RS341 + 1./16.*pow(m33,2)*RS341 + 1./16.*m11*m44*RS341 + 1./16.
         *m11*m33*RS341 );

      M13p6x21RV +=  + hhV(1,m44,m14,m34,m13,L,mu2) * ( m33*m44 + m11*
         m44 );

      M13p6x21RV +=  + hhV(2,m11,m11,m13,m13,L,mu2) * (  - 1./36.*pow(
         m33,2)*RD14 + 1./18.*pow(m33,2)*RS143*RD14 - 1./18.*m11*m33*
         RD14 + 1./9.*m11*m33*RS143*RD14 - 1./36.*pow(m11,2)*RD14 + 1./
         18.*pow(m11,2)*RS143*RD14 );

      M13p6x21RV +=  + hhV(3,m13,m11,m11,m13,L,mu2) * ( 1./144.*pow(
         m33,2)*RD14 - 1./72.*pow(m33,2)*RS143*RD14 + 1./72.*m11*m33*
         RD14 - 1./36.*m11*m33*RS143*RD14 + 1./144.*pow(m11,2)*RD14 - 1.
         /72.*pow(m11,2)*RS143*RD14 );

      M13p6x21RV +=  + hhV(3,m13,m11,m33,m13,L,mu2) * ( 1./144.*pow(
         m33,2)*RD14 - 1./72.*pow(m33,2)*RS341*RD14 + 1./72.*m11*m33*
         RD14 - 1./36.*m11*m33*RS341*RD14 + 1./144.*pow(m11,2)*RD14 - 1.
         /72.*pow(m11,2)*RS341*RD14 );

      M13p6x21RV +=  + hhV(3,m13,m33,m33,m13,L,mu2) * (  - 23./1296.*
         m33*pow(RD34,2) + 5./144.*pow(m33,2)*RD34 + 1./216.*pow(m33,2)
         *RS341*RD34 - 23./1296.*m11*pow(RD34,2) + 5./72.*m11*m33*RD34
          - 1./36.*m11*m33*RS341*RD34 + 5./144.*pow(m11,2)*RD34 - 7./
         216.*pow(m11,2)*RS341*RD34 );

      M13p6x21RV +=  + hhV(3,m14,m11,m34,m13,L,mu2) * (  - 1./16.*m33*
         m44*RD14 - 1./16.*m11*m44*RD14 - 1./16.*m11*m33*RD14 - 1./16.*
         pow(m11,2)*RD14 );

      M13p6x21RV +=  + hhV(4,m13,m11,m33,m13,L,mu2) * ( 1./144.*pow(
         m33,2)*RD34 - 1./72.*pow(m33,2)*RS143*RD34 + 1./72.*m11*m33*
         RD34 - 1./36.*m11*m33*RS143*RD34 + 1./144.*pow(m11,2)*RD34 - 1.
         /72.*pow(m11,2)*RS143*RD34 );

      M13p6x21RV +=  + hhV(4,m13,m33,m33,m13,L,mu2) * (  - 1./1296.*m33
         *pow(RD34,2) - 1./1296.*m11*pow(RD34,2) );

      M13p6x21RV +=  + hhV(4,m34,m14,m33,m13,L,mu2) * (  - 1./16.*m33*
         m44*RD34 - 1./16.*pow(m33,2)*RD34 - 1./16.*m11*m44*RD34 - 1./
         16.*m11*m33*RD34 );

      M13p6x21RV +=  + hhV(5,m11,m11,m13,m13,L,mu2) * (  - 1./72.*pow(
         m33,2)*pow(RD14,2) - 1./36.*m11*m33*pow(RD14,2) - 1./72.*pow(
         m11,2)*pow(RD14,2) );

      M13p6x21RV +=  + hhV(7,m13,m11,m11,m13,L,mu2) * ( 1./288.*pow(
         m33,2)*pow(RD14,2) + 1./144.*m11*m33*pow(RD14,2) + 1./288.*
         pow(m11,2)*pow(RD14,2) );

      M13p6x21RV +=  + hhV(7,m13,m11,m33,m13,L,mu2) * ( 1./144.*pow(
         m33,2)*RD14*RD34 + 1./72.*m11*m33*RD14*RD34 + 1./144.*pow(
         m11,2)*RD14*RD34 );

      M13p6x21RV +=  + hhV(7,m13,m33,m33,m13,L,mu2) * ( 5./288.*pow(
         m33,2)*pow(RD34,2) + 5./144.*m11*m33*pow(RD34,2) + 5./288.*
         pow(m11,2)*pow(RD34,2) );

      M13p6x21RV +=  + hh1V(1,m11,m11,m13,m13,L,mu2) * (  - 1./27.*m33*
         RD14 + 2./27.*m33*RS143*RD14 + 5./9.*pow(m33,2) - 4./27.*pow(
         m33,2)*RS143 + 5./27.*pow(m33,2)*pow(RS143,2) - 1./27.*m11*
         RD14 + 2./27.*m11*RS143*RD14 + 10./9.*m11*m33 - 2./9.*m11*m33*
         RS143 + 2./9.*m11*m33*pow(RS143,2) + 5./9.*pow(m11,2) - 2./27.
         *pow(m11,2)*RS143 + 1./27.*pow(m11,2)*pow(RS143,2) );

      M13p6x21RV +=  + hh1V(1,m11,m14,m34,m13,L,mu2) * (  - 1./2.*pow(
         m33,2) + 1./2.*pow(m33,2)*RS143 - m11*m33 + m11*m33*RS143 - 1./
         2.*pow(m11,2) + 1./2.*pow(m11,2)*RS143 );

      M13p6x21RV +=  + hh1V(1,m13,m11,m33,m13,L,mu2) * ( 1./54.*m33*
         RD34 + 1./54.*m33*RD14 - 1./27.*m33*RS341*RD14 - 1./27.*m33*
         RS143*RD34 + 1./27.*pow(m33,2)*RS341 + 2./27.*pow(m33,2)*RS143
          - 1./9.*pow(m33,2)*RS143*RS341 + 1./54.*m11*RD34 + 1./54.*m11
         *RD14 - 1./27.*m11*RS341*RD14 - 1./27.*m11*RS143*RD34 + 1./9.*
         m11*m33*RS341 + 1./9.*m11*m33*RS143 - 2./9.*m11*m33*RS143*
         RS341 + 2./27.*pow(m11,2)*RS341 + 1./27.*pow(m11,2)*RS143 - 1./
         9.*pow(m11,2)*RS143*RS341 );

      M13p6x21RV +=  + hh1V(1,m13,m33,m33,m13,L,mu2) * ( 1./54.*m33*
         RD34 - 1./27.*m33*RS341*RD34 - 5./18.*pow(m33,2) + 1./27.*pow(
         m33,2)*RS341 - 1./54.*pow(m33,2)*pow(RS341,2) + 1./54.*m11*
         RD34 - 1./27.*m11*RS341*RD34 - 5./9.*m11*m33 + 1./9.*m11*m33*
         RS341 - 1./9.*m11*m33*pow(RS341,2) - 5./18.*pow(m11,2) + 2./27.
         *pow(m11,2)*RS341 - 5./54.*pow(m11,2)*pow(RS341,2) );

      M13p6x21RV +=  + hh1V(1,m14,m11,m34,m13,L,mu2) * (  - 1./6.*m33*
         RD14 - 1./3.*m33*m44 - 1./2.*pow(m33,2) + 1./6.*pow(m33,2)*
         RS143 - 1./6.*m11*RD14 - 1./3.*m11*m44 - 2./3.*m11*m33 - 1./6.
         *pow(m11,2) - 1./6.*pow(m11,2)*RS143 );

      M13p6x21RV +=  + hh1V(1,m14,m33,m34,m13,L,mu2) * ( 1./6.*m33*RD34
          + 1./3.*m33*m44 + 1./6.*pow(m33,2) + 1./6.*pow(m33,2)*RS341
          + 1./6.*m11*RD34 + 1./3.*m11*m44 + 2./3.*m11*m33 + 1./2.*pow(
         m11,2) - 1./6.*pow(m11,2)*RS341 );

      M13p6x21RV +=  + hh1V(1,m33,m14,m34,m13,L,mu2) * ( 1./6.*m33*RD34
          + 1./3.*m33*m44 - 1./3.*pow(m33,2) + 2./3.*pow(m33,2)*RS341
          + 1./6.*m11*RD34 + 1./3.*m11*m44 - 1./3.*m11*m33 + m11*m33*
         RS341 + 1./3.*pow(m11,2)*RS341 );

      M13p6x21RV +=  + hh1V(2,m11,m11,m13,m13,L,mu2) * (  - 1./54.*m33*
         pow(RD14,2) + 1./18.*pow(m33,2)*RD14 - 2./27.*pow(m33,2)*RS143
         *RD14 - 1./54.*m11*pow(RD14,2) + 1./9.*m11*m33*RD14 - 1./9.*
         m11*m33*RS143*RD14 + 1./18.*pow(m11,2)*RD14 - 1./27.*pow(
         m11,2)*RS143*RD14 );

      M13p6x21RV +=  + hh1V(3,m11,m11,m13,m13,L,mu2) * (  - 1./54.*m33*
         pow(RD14,2) + 1./18.*pow(m33,2)*RD14 - 2./27.*pow(m33,2)*RS143
         *RD14 - 1./54.*m11*pow(RD14,2) + 1./9.*m11*m33*RD14 - 1./9.*
         m11*m33*RS143*RD14 + 1./18.*pow(m11,2)*RD14 - 1./27.*pow(
         m11,2)*RS143*RD14 );

      M13p6x21RV +=  + hh1V(3,m13,m11,m33,m13,L,mu2) * ( 1./54.*m33*
         RD14*RD34 + 1./27.*pow(m33,2)*RS341*RD14 + 1./54.*m11*RD14*
         RD34 + 1./9.*m11*m33*RS341*RD14 + 2./27.*pow(m11,2)*RS341*RD14
          );

      M13p6x21RV +=  + hh1V(3,m13,m33,m33,m13,L,mu2) * ( 23./1296.*m33*
         pow(RD34,2) - 1./18.*pow(m33,2)*RD34 + 1./27.*pow(m33,2)*RS341
         *RD34 + 23./1296.*m11*pow(RD34,2) - 1./9.*m11*m33*RD34 + 1./9.
         *m11*m33*RS341*RD34 - 1./18.*pow(m11,2)*RD34 + 2./27.*pow(
         m11,2)*RS341*RD34 );

      M13p6x21RV +=  + hh1V(4,m13,m11,m33,m13,L,mu2) * ( 1./54.*m33*
         RD14*RD34 + 2./27.*pow(m33,2)*RS143*RD34 + 1./54.*m11*RD14*
         RD34 + 1./9.*m11*m33*RS143*RD34 + 1./27.*pow(m11,2)*RS143*RD34
          );

      M13p6x21RV +=  + hh1V(4,m13,m33,m33,m13,L,mu2) * ( 1./1296.*m33*
         pow(RD34,2) + 1./1296.*m11*pow(RD34,2) );

      M13p6x21RV +=  + hh1V(5,m11,m11,m13,m13,L,mu2) * ( 1./18.*pow(
         m33,2)*pow(RD14,2) + 1./9.*m11*m33*pow(RD14,2) + 1./18.*pow(
         m11,2)*pow(RD14,2) );

      M13p6x21RV +=  + hh1V(7,m13,m33,m33,m13,L,mu2) * (  - 1./36.*pow(
         m33,2)*pow(RD34,2) - 1./18.*m11*m33*pow(RD34,2) - 1./36.*pow(
         m11,2)*pow(RD34,2) );

      M13p6x21RV +=  + hh21V(1,m11,m11,m13,m13,L,mu2) * (  - 3./16.*
         pow(m33,2) - 3./8.*m11*m33 - 3./16.*pow(m11,2) );

      M13p6x21RV +=  + hh21V(1,m11,m14,m34,m13,L,mu2) * ( 3./16.*pow(
         m33,2) - 3./16.*pow(m33,2)*RS143 + 3./8.*m11*m33 - 3./8.*m11*
         m33*RS143 + 3./16.*pow(m11,2) - 3./16.*pow(m11,2)*RS143 );

      M13p6x21RV +=  + hh21V(1,m13,m11,m11,m13,L,mu2) * ( 5./48.*pow(
         m33,2) - 1./24.*pow(m33,2)*RS143 + 1./24.*pow(m33,2)*pow(
         RS143,2) + 5./24.*m11*m33 - 1./12.*m11*m33*RS143 + 1./12.*m11*
         m33*pow(RS143,2) + 5./48.*pow(m11,2) - 1./24.*pow(m11,2)*RS143
          + 1./24.*pow(m11,2)*pow(RS143,2) );

      M13p6x21RV +=  + hh21V(1,m13,m11,m33,m13,L,mu2) * ( 1./48.*pow(
         m33,2) - 1./24.*pow(m33,2)*RS341 - 1./24.*pow(m33,2)*RS143 + 1.
         /12.*pow(m33,2)*RS143*RS341 + 1./24.*m11*m33 - 1./12.*m11*m33*
         RS341 - 1./12.*m11*m33*RS143 + 1./6.*m11*m33*RS143*RS341 + 1./
         48.*pow(m11,2) - 1./24.*pow(m11,2)*RS341 - 1./24.*pow(m11,2)*
         RS143 + 1./12.*pow(m11,2)*RS143*RS341 );

      M13p6x21RV +=  + hh21V(1,m13,m13,m13,m13,L,mu2) * ( 3./16.*pow(
         m33,2) + 3./8.*m11*m33 + 3./16.*pow(m11,2) );

      M13p6x21RV +=  + hh21V(1,m13,m33,m33,m13,L,mu2) * ( 5./48.*pow(
         m33,2) - 1./24.*pow(m33,2)*RS341 + 1./24.*pow(m33,2)*pow(
         RS341,2) + 5./24.*m11*m33 - 1./12.*m11*m33*RS341 + 1./12.*m11*
         m33*pow(RS341,2) + 5./48.*pow(m11,2) - 1./24.*pow(m11,2)*RS341
          + 1./24.*pow(m11,2)*pow(RS341,2) );

      M13p6x21RV +=  + hh21V(1,m14,m11,m34,m13,L,mu2) * ( 3./16.*pow(
         m33,2)*RS143 + 3./8.*m11*m33*RS143 + 3./16.*pow(m11,2)*RS143 )
         ;

      M13p6x21RV +=  + hh21V(1,m14,m33,m34,m13,L,mu2) * (  - 3./16.*
         pow(m33,2) + 3./16.*pow(m33,2)*RS341 - 3./8.*m11*m33 + 3./8.*
         m11*m33*RS341 - 3./16.*pow(m11,2) + 3./16.*pow(m11,2)*RS341 );

      M13p6x21RV +=  + hh21V(1,m33,m13,m33,m13,L,mu2) * (  - 3./16.*
         pow(m33,2) - 3./8.*m11*m33 - 3./16.*pow(m11,2) );

      M13p6x21RV +=  + hh21V(1,m33,m14,m34,m13,L,mu2) * ( 3./16.*pow(
         m33,2) - 3./16.*pow(m33,2)*RS341 + 3./8.*m11*m33 - 3./8.*m11*
         m33*RS341 + 3./16.*pow(m11,2) - 3./16.*pow(m11,2)*RS341 );

      M13p6x21RV +=  + hh21V(1,m34,m11,m14,m13,L,mu2) * (  - 3./16.*
         pow(m33,2) + 3./16.*pow(m33,2)*RS143 - 3./8.*m11*m33 + 3./8.*
         m11*m33*RS143 - 3./16.*pow(m11,2) + 3./16.*pow(m11,2)*RS143 );

      M13p6x21RV +=  + hh21V(1,m34,m14,m33,m13,L,mu2) * ( 3./16.*pow(
         m33,2)*RS341 + 3./8.*m11*m33*RS341 + 3./16.*pow(m11,2)*RS341 )
         ;

      M13p6x21RV +=  + hh21V(1,m44,m14,m34,m13,L,mu2) * ( 3./2.*pow(
         m33,2) + 3*m11*m33 + 3./2.*pow(m11,2) );

      M13p6x21RV +=  + hh21V(3,m13,m11,m11,m13,L,mu2) * ( 1./48.*pow(
         m33,2)*RD14 - 1./24.*pow(m33,2)*RS143*RD14 + 1./24.*m11*m33*
         RD14 - 1./12.*m11*m33*RS143*RD14 + 1./48.*pow(m11,2)*RD14 - 1./
         24.*pow(m11,2)*RS143*RD14 );

      M13p6x21RV +=  + hh21V(3,m13,m11,m33,m13,L,mu2) * ( 1./48.*pow(
         m33,2)*RD14 - 1./24.*pow(m33,2)*RS341*RD14 + 1./24.*m11*m33*
         RD14 - 1./12.*m11*m33*RS341*RD14 + 1./48.*pow(m11,2)*RD14 - 1./
         24.*pow(m11,2)*RS341*RD14 );

      M13p6x21RV +=  + hh21V(3,m13,m33,m33,m13,L,mu2) * ( 1./48.*pow(
         m33,2)*RD34 - 1./24.*pow(m33,2)*RS341*RD34 + 1./24.*m11*m33*
         RD34 - 1./12.*m11*m33*RS341*RD34 + 1./48.*pow(m11,2)*RD34 - 1./
         24.*pow(m11,2)*RS341*RD34 );

      M13p6x21RV +=  + hh21V(3,m14,m11,m34,m13,L,mu2) * (  - 3./16.*
         pow(m33,2)*RD14 - 3./8.*m11*m33*RD14 - 3./16.*pow(m11,2)*RD14
          );

      M13p6x21RV +=  + hh21V(4,m13,m11,m33,m13,L,mu2) * ( 1./48.*pow(
         m33,2)*RD34 - 1./24.*pow(m33,2)*RS143*RD34 + 1./24.*m11*m33*
         RD34 - 1./12.*m11*m33*RS143*RD34 + 1./48.*pow(m11,2)*RD34 - 1./
         24.*pow(m11,2)*RS143*RD34 );

      M13p6x21RV +=  + hh21V(4,m34,m14,m33,m13,L,mu2) * (  - 3./16.*
         pow(m33,2)*RD34 - 3./8.*m11*m33*RD34 - 3./16.*pow(m11,2)*RD34
          );

      M13p6x21RV +=  + hh21V(7,m13,m11,m11,m13,L,mu2) * ( 1./96.*pow(
         m33,2)*pow(RD14,2) + 1./48.*m11*m33*pow(RD14,2) + 1./96.*pow(
         m11,2)*pow(RD14,2) );

      M13p6x21RV +=  + hh21V(7,m13,m11,m33,m13,L,mu2) * ( 1./48.*pow(
         m33,2)*RD14*RD34 + 1./24.*m11*m33*RD14*RD34 + 1./48.*pow(
         m11,2)*RD14*RD34 );

      M13p6x21RV +=  + hh21V(7,m13,m33,m33,m13,L,mu2) * ( 1./96.*pow(
         m33,2)*pow(RD34,2) + 1./48.*m11*m33*pow(RD34,2) + 1./96.*pow(
         m11,2)*pow(RD34,2) );

      M13p6x21RV +=  + hh27V(1,m11,m11,m13,m13,L,mu2) * ( 3./8.*m33 + 3.
         /8.*m11 );

      M13p6x21RV +=  + hh27V(1,m11,m14,m34,m13,L,mu2) * (  - 3./8.*m33
          + 3./8.*m33*RS143 - 3./8.*m11 + 3./8.*m11*RS143 );

      M13p6x21RV +=  + hh27V(1,m13,m11,m11,m13,L,mu2) * (  - 5./24.*m33
          + 1./12.*m33*RS143 - 1./12.*m33*pow(RS143,2) - 5./24.*m11 + 1.
         /12.*m11*RS143 - 1./12.*m11*pow(RS143,2) );

      M13p6x21RV +=  + hh27V(1,m13,m11,m33,m13,L,mu2) * (  - 1./24.*m33
          + 1./12.*m33*RS341 + 1./12.*m33*RS143 - 1./6.*m33*RS143*RS341
          - 1./24.*m11 + 1./12.*m11*RS341 + 1./12.*m11*RS143 - 1./6.*
         m11*RS143*RS341 );

      M13p6x21RV +=  + hh27V(1,m13,m13,m13,m13,L,mu2) * (  - 3./8.*m33
          - 3./8.*m11 );

      M13p6x21RV +=  + hh27V(1,m13,m33,m33,m13,L,mu2) * (  - 5./24.*m33
          + 1./12.*m33*RS341 - 1./12.*m33*pow(RS341,2) - 5./24.*m11 + 1.
         /12.*m11*RS341 - 1./12.*m11*pow(RS341,2) );

      M13p6x21RV +=  + hh27V(1,m14,m11,m34,m13,L,mu2) * (  - 3./8.*m33*
         RS143 - 3./8.*m11*RS143 );

      M13p6x21RV +=  + hh27V(1,m14,m33,m34,m13,L,mu2) * ( 3./8.*m33 - 3.
         /8.*m33*RS341 + 3./8.*m11 - 3./8.*m11*RS341 );

      M13p6x21RV +=  + hh27V(1,m33,m13,m33,m13,L,mu2) * ( 3./8.*m33 + 3.
         /8.*m11 );

      M13p6x21RV +=  + hh27V(1,m33,m14,m34,m13,L,mu2) * (  - 3./8.*m33
          + 3./8.*m33*RS341 - 3./8.*m11 + 3./8.*m11*RS341 );

      M13p6x21RV +=  + hh27V(1,m34,m11,m14,m13,L,mu2) * ( 3./8.*m33 - 3.
         /8.*m33*RS143 + 3./8.*m11 - 3./8.*m11*RS143 );

      M13p6x21RV +=  + hh27V(1,m34,m14,m33,m13,L,mu2) * (  - 3./8.*m33*
         RS341 - 3./8.*m11*RS341 );

      M13p6x21RV +=  + hh27V(1,m44,m14,m34,m13,L,mu2) * (  - 3*m33 - 3*
         m11 );

      M13p6x21RV +=  + hh27V(3,m13,m11,m11,m13,L,mu2) * (  - 1./24.*m33
         *RD14 + 1./12.*m33*RS143*RD14 - 1./24.*m11*RD14 + 1./12.*m11*
         RS143*RD14 );

      M13p6x21RV +=  + hh27V(3,m13,m11,m33,m13,L,mu2) * (  - 1./24.*m33
         *RD14 + 1./12.*m33*RS341*RD14 - 1./24.*m11*RD14 + 1./12.*m11*
         RS341*RD14 );

      M13p6x21RV +=  + hh27V(3,m13,m33,m33,m13,L,mu2) * (  - 1./24.*m33
         *RD34 + 1./12.*m33*RS341*RD34 - 1./24.*m11*RD34 + 1./12.*m11*
         RS341*RD34 );

      M13p6x21RV +=  + hh27V(3,m14,m11,m34,m13,L,mu2) * ( 3./8.*m33*
         RD14 + 3./8.*m11*RD14 );

      M13p6x21RV +=  + hh27V(4,m13,m11,m33,m13,L,mu2) * (  - 1./24.*m33
         *RD34 + 1./12.*m33*RS143*RD34 - 1./24.*m11*RD34 + 1./12.*m11*
         RS143*RD34 );

      M13p6x21RV +=  + hh27V(4,m34,m14,m33,m13,L,mu2) * ( 3./8.*m33*
         RD34 + 3./8.*m11*RD34 );

      M13p6x21RV +=  + hh27V(7,m13,m11,m11,m13,L,mu2) * (  - 1./48.*m33
         *pow(RD14,2) - 1./48.*m11*pow(RD14,2) );

      M13p6x21RV +=  + hh27V(7,m13,m11,m33,m13,L,mu2) * (  - 1./24.*m33
         *RD14*RD34 - 1./24.*m11*RD14*RD34 );

      M13p6x21RV +=  + hh27V(7,m13,m33,m33,m13,L,mu2) * (  - 1./48.*m33
         *pow(RD34,2) - 1./48.*m11*pow(RD34,2) );

  return M13p6x21RV/pow(f0,4);
}



// 1 valence mass 2 sea mass *****************************************

double mv1s2nf3p4V(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 3) || (B0mq.size() != 3)){
    cout << "ERROR: mv1s2nf3p4L probably called with wrong number"
	 << "of quark masses\n";}
  // three masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m66 = 2.*B0mq[2];
  double me2 = (m44+2.*m66)/3.;
  //double mu2 = mu0*mu0;
  double RC146e = RC(m11,m44,m66,me2);
  double RSe6611 = pow(RS(me2,m66,m11),2);
  double RD146e = RD(m11,m44,m66,me2);
  double M12p4x12V =
       + AbV(m11,L) * (  - 1./3.*m11*RC146e );

      M12p4x12V +=  + AbV(me2,L) * ( 2./3.*m11*RSe6611 );

      M12p4x12V +=  + BbV(m11,L) * (  - 1./3.*m11*RD146e );

  return M12p4x12V/pow(f0,2);
}

double mv1s2nf3p6V(const quarkmassnf mass, const Linf Liin, const double L){
  return mv1s2nf3p6LV(mass,Liin,L)+mv1s2nf3p6RV(mass,L);
}

double mv1s2nf3p6LV(const quarkmassnf mass, const Linf Liin, const double L){
  double L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu;
  int nfLi;
  Liin.out(L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu,nfLi);
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if (mu0 != mu){
    cout << "ERROR: differing mu's in mv1s2nf3p6LV\n";}
  if ( (nq != 3) || (B0mq.size() != 3)){
    cout << "ERROR: mv1s2nf3p6LV probably called with wrong number"
	 << "of quark masses\n";}
  // three masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m66 = 2.*B0mq[2];
  double m14 = (m11+m44)/2.;
  double m16 = (m11+m66)/2.;
  double m46 = (m44+m66)/2.;
  double me2 = (m44+2.*m66)/3.;
  //double mu2 = mu0*mu0;
  double RC146e = RC(m11,m44,m66,me2);
  double RSe61 = RS(me2,m66,m11);
  double RSe6611 = pow(RSe61,2);
  double RS14e = RS(m11,m44,me2);
  double RS16e = RS(m11,m66,me2);
  //double RS146ee = RS14e*RS16e;
  double RD146e = RD(m11,m44,m66,me2);
  //double RN1e = RN(m11,me2);

  double M12p6x12LV =
       + AbV(m11,L) * ( 8./3.*m11*L5r*RD146e - 8*m11*L3r*RD146e - 8*m11
         *L0r*RD146e - 32./3.*m11*m66*L7r*RS14e - 16./3.*m11*m66*L6r*
         RC146e - 8./9.*m11*m66*L5r*pow(RS14e,2) - 16./9.*m11*m66*L4r*
         pow(RS16e,2) - 8./9.*m11*m66*L4r*pow(RS14e,2) + 8*m11*m66*L4r*
         RC146e - 64./3.*m11*m44*L7r*RS16e - 32./3.*m11*m44*L6r*RC146e
          - 16./9.*m11*m44*L5r*pow(RS16e,2) - 32./9.*m11*m44*L4r*pow(
         RS16e,2) - 16./9.*m11*m44*L4r*pow(RS14e,2) + 16*m11*m44*L4r*
         RC146e - 64./3.*pow(m11,2)*L8r*RC146e + 32*pow(m11,2)*L7r + 32
         *pow(m11,2)*L6r + 40./3.*pow(m11,2)*L5r*RC146e - 8*pow(m11,2)*
         L3r*RC146e + 20*pow(m11,2)*L2r + 8*pow(m11,2)*L1r - 8*pow(
         m11,2)*L0r*RC146e );

      M12p6x12LV +=  + AbV(m14,L) * ( 32*m11*m44*L8r - 16*m11*m44*L5r
          + 20*m11*m44*L3r + 8*m11*m44*L0r + 32*pow(m11,2)*L8r - 16*
         pow(m11,2)*L5r + 20*pow(m11,2)*L3r + 8*pow(m11,2)*L0r );

      M12p6x12LV +=  + AbV(m16,L) * ( 16*m11*m66*L8r - 8*m11*m66*L5r + 
         10*m11*m66*L3r + 4*m11*m66*L0r + 16*pow(m11,2)*L8r - 8*pow(
         m11,2)*L5r + 10*pow(m11,2)*L3r + 4*pow(m11,2)*L0r );

      M12p6x12LV +=  + AbV(m44,L) * ( 48*m11*m44*L6r - 48*m11*m44*L4r
          + 12*m11*m44*L2r + 48*m11*m44*L1r );

      M12p6x12LV +=  + AbV(m46,L) * ( 32*m11*m66*L6r - 32*m11*m66*L4r
          + 8*m11*m66*L2r + 32*m11*m66*L1r + 32*m11*m44*L6r - 32*m11*
         m44*L4r + 8*m11*m44*L2r + 32*m11*m44*L1r );

      M12p6x12LV +=  + AbV(me2,L) * (  - 16./3.*m11*me2*L5r*RSe6611 - 8
         *m11*me2*L4r + 16*m11*me2*L3r*RSe6611 + 4*m11*me2*L2r + 16*m11
         *me2*L1r + 16*m11*me2*L0r*RSe6611 + 64./3.*m11*m66*L7r*RSe61
          + 32./3.*m11*m66*L6r + 32./3.*m11*m66*L6r*RSe6611 - 32./9.*
         m11*m66*L5r*pow(RSe61,2) + 32./9.*m11*m66*L5r*RS14e*RSe61 - 16.
         /3.*m11*m66*L4r - 16./3.*m11*m66*L4r*pow(RSe61,2) - 16*m11*m66
         *L4r*RSe6611 - 32./9.*m11*m66*L4r*RS16e*RSe61 + 32./9.*m11*m66
         *L4r*RS14e*RSe61 - 64./3.*m11*m44*L7r*RSe61 + 16./3.*m11*m44*
         L6r + 64./3.*m11*m44*L6r*RSe6611 - 16./9.*m11*m44*L5r*pow(
         RSe61,2) - 32./9.*m11*m44*L5r*RS16e*RSe61 - 8./3.*m11*m44*L4r
          - 32./3.*m11*m44*L4r*pow(RSe61,2) - 32*m11*m44*L4r*RSe6611 - 
         64./9.*m11*m44*L4r*RS16e*RSe61 + 64./9.*m11*m44*L4r*RS14e*
         RSe61 + 128./3.*pow(m11,2)*L8r*RSe6611 - 64./3.*pow(m11,2)*L5r
         *RSe6611 );

      M12p6x12LV +=  + BbV(m11,me2,L) * (  - 64./9.*m11*pow(m66,2)*L8r*
         RS14e*RSe61 - 64./9.*m11*pow(m66,2)*L7r*RS14e*RSe61 - 64./9.*
         m11*pow(m66,2)*L6r*RS14e*RSe61 - 128./9.*m11*m44*m66*L7r*RS16e
         *RSe61 + 64./9.*m11*m44*m66*L7r*RS14e*RSe61 + 64./9.*m11*m44*
         m66*L6r*RS16e*RSe61 - 128./9.*m11*m44*m66*L6r*RS14e*RSe61 + 64.
         /9.*m11*pow(m44,2)*L8r*RS16e*RSe61 + 128./9.*m11*pow(m44,2)*
         L7r*RS16e*RSe61 + 128./9.*m11*pow(m44,2)*L6r*RS16e*RSe61 + 64./
         3.*pow(m11,2)*m66*L7r*RSe61 + 64./3.*pow(m11,2)*m66*L6r*
         RSe6611 + 32./9.*pow(m11,2)*m66*L5r*RS14e*RSe61 - 32./3.*pow(
         m11,2)*m66*L4r*RSe6611 - 32./9.*pow(m11,2)*m66*L4r*RS16e*RSe61
          + 32./9.*pow(m11,2)*m66*L4r*RS14e*RSe61 - 64./3.*pow(m11,2)*
         m44*L7r*RSe61 + 128./3.*pow(m11,2)*m44*L6r*RSe6611 - 32./9.*
         pow(m11,2)*m44*L5r*RS16e*RSe61 - 64./3.*pow(m11,2)*m44*L4r*
         RSe6611 - 64./9.*pow(m11,2)*m44*L4r*RS16e*RSe61 + 64./9.*pow(
         m11,2)*m44*L4r*RS14e*RSe61 + 64./3.*pow(m11,3)*L8r*RSe6611 - 
         32./3.*pow(m11,3)*L5r*RSe6611 );

      M12p6x12LV +=  + BbV(m11,L) * (  - 16./3.*m11*m66*L6r*RD146e + 8*
         m11*m66*L4r*RD146e + 16./9.*m11*pow(m66,2)*L8r*pow(RS14e,2) + 
         16./9.*m11*pow(m66,2)*L7r*pow(RS14e,2) + 16./9.*m11*pow(m66,2)
         *L6r*pow(RS14e,2) - 32./3.*m11*m44*L6r*RD146e + 16*m11*m44*L4r
         *RD146e + 64./9.*m11*m44*m66*L7r*RS14e*RS16e + 32./9.*m11*m44*
         m66*L6r*pow(RS16e,2) + 32./9.*m11*m44*m66*L6r*pow(RS14e,2) + 
         32./9.*m11*pow(m44,2)*L8r*pow(RS16e,2) + 64./9.*m11*pow(m44,2)
         *L7r*pow(RS16e,2) + 64./9.*m11*pow(m44,2)*L6r*pow(RS16e,2) - 
         64./3.*pow(m11,2)*L8r*RD146e + 40./3.*pow(m11,2)*L5r*RD146e - 
         8*pow(m11,2)*L3r*RD146e - 8*pow(m11,2)*L0r*RD146e - 32./3.*
         pow(m11,2)*m66*L7r*RS14e - 32./3.*pow(m11,2)*m66*L6r*RC146e - 
         8./9.*pow(m11,2)*m66*L5r*pow(RS14e,2) - 16./9.*pow(m11,2)*m66*
         L4r*pow(RS16e,2) - 8./9.*pow(m11,2)*m66*L4r*pow(RS14e,2) + 16./
         3.*pow(m11,2)*m66*L4r*RC146e - 64./3.*pow(m11,2)*m44*L7r*RS16e
          - 64./3.*pow(m11,2)*m44*L6r*RC146e - 16./9.*pow(m11,2)*m44*
         L5r*pow(RS16e,2) );

      M12p6x12LV +=  + BbV(m11,L) * (  - 32./9.*pow(m11,2)*m44*L4r*pow(
         RS16e,2) - 16./9.*pow(m11,2)*m44*L4r*pow(RS14e,2) + 32./3.*
         pow(m11,2)*m44*L4r*RC146e - 32./3.*pow(m11,3)*L8r*RC146e + 16*
         pow(m11,3)*L7r + 16./3.*pow(m11,3)*L5r*RC146e );

      M12p6x12LV +=  + BbV(me2,L) * (  - 32./9.*m11*m66*me2*L5r*pow(
         RSe61,2) - 16./3.*m11*m66*me2*L4r*pow(RSe61,2) + 64./9.*m11*
         pow(m66,2)*L8r*pow(RSe61,2) + 64./9.*m11*pow(m66,2)*L7r*pow(
         RSe61,2) + 64./9.*m11*pow(m66,2)*L6r*pow(RSe61,2) - 16./9.*m11
         *m44*me2*L5r*pow(RSe61,2) - 32./3.*m11*m44*me2*L4r*pow(
         RSe61,2) - 128./9.*m11*m44*m66*L7r*pow(RSe61,2) + 160./9.*m11*
         m44*m66*L6r*pow(RSe61,2) + 32./9.*m11*pow(m44,2)*L8r*pow(
         RSe61,2) + 64./9.*m11*pow(m44,2)*L7r*pow(RSe61,2) + 64./9.*m11
         *pow(m44,2)*L6r*pow(RSe61,2) );

      M12p6x12LV +=  + CbV(m11,L) * (  - 32./3.*pow(m11,2)*m66*L6r*
         RD146e + 16./3.*pow(m11,2)*m66*L4r*RD146e - 64./3.*pow(m11,2)*
         m44*L6r*RD146e + 32./3.*pow(m11,2)*m44*L4r*RD146e - 32./3.*
         pow(m11,3)*L8r*RD146e + 16./3.*pow(m11,3)*L5r*RD146e );

      M12p6x12LV +=  + A23bV(m11,L) * ( 8*m11*L3r*RC146e - 12*m11*L2r
          - 24*m11*L1r + 8*m11*L0r*RC146e );

      M12p6x12LV +=  + A23bV(m14,L) * (  - 24*m11*L3r - 48*m11*L0r );

      M12p6x12LV +=  + A23bV(m16,L) * (  - 12*m11*L3r - 24*m11*L0r );

      M12p6x12LV +=  + A23bV(m44,L) * (  - 36*m11*L2r );

      M12p6x12LV +=  + A23bV(m46,L) * (  - 48*m11*L2r );

      M12p6x12LV +=  + A23bV(me2,L) * (  - 16*m11*L3r*RSe6611 - 12*m11*
         L2r - 16*m11*L0r*RSe6611 );

      M12p6x12LV +=  + B23bV(m11,L) * ( 8*m11*L3r*RD146e + 8*m11*L0r*
         RD146e );

  return M12p6x12LV/pow(f0,4);
}

double mv1s2nf3p6RV(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 3) || (B0mq.size() != 3)){
    cout << "ERROR: mv1s2nf3p6RV probably called with wrong number"
	 << "of quark masses\n";}
  // three masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m66 = 2.*B0mq[2];
  double m14 = (m11+m44)/2.;
  double m16 = (m11+m66)/2.;
  double m46 = (m44+m66)/2.;
  double me2 = (m44+2.*m66)/3.;
  double mu2 = mu0*mu0;
  double RC146e = RC(m11,m44,m66,me2);
  double RSe61 = RS(me2,m66,m11);
  double RSe6611 = pow(RSe61,2);
  double RS14e = RS(m11,m44,me2);
  double RS16e = RS(m11,m66,me2);
  //double RS146ee = RS14e*RS16e;
  double RD146e = RD(m11,m44,m66,me2);
  double RN1e = RN(m11,me2);


  double  M12p6x12RV =
       + Ab(m11,mu2)*AbV(m11,L) * ( m11 + 1./9.*m11*pow(RC146e,2) - 4./
         9.*pow(m11,2)*RN1e*RC146e*RSe6611 );

      M12p6x12RV +=  + Ab(m11,mu2)*AbV(m14,L) * ( 4./45.*RD146e + 4./45.
         *m44*RS16e - 44./45.*m11*RS16e + 8./9.*m11*RC146e - 4./9.*m11*
         m44*RN1e*RSe61 - 8./9.*pow(m11,2)*RN1e*RSe61 );

      M12p6x12RV +=  + Ab(m11,mu2)*AbV(m16,L) * ( 2./45.*RD146e + 2./45.
         *m66*RS14e - 22./45.*m11*RS14e + 4./9.*m11*RC146e + 4./9.*m11*
         m66*RN1e*RSe61 + 8./9.*pow(m11,2)*RN1e*RSe61 );

      M12p6x12RV +=  + Ab(m11,mu2)*AbV(m44,L) * ( 2./3.*m11*m44*RN1e*
         RS16e*RSe61 );

      M12p6x12RV +=  + Ab(m11,mu2)*AbV(m46,L) * (  - 2./27.*m11*pow(
         RS16e,2) + 4./27.*m11*RS14e*RS16e - 2./27.*m11*pow(RS14e,2) - 
         8./27.*m11*m66*RN1e*RS16e*RSe61 - 4./27.*m11*m66*RN1e*RS14e*
         RSe61 - 4./27.*m11*m44*RN1e*RS16e*RSe61 + 4./27.*m11*m44*RN1e*
         RS14e*RSe61 - 4./9.*pow(m11,2)*RN1e*RS16e*RSe61 + 4./9.*pow(
         m11,2)*RN1e*RS14e*RSe61 );

      M12p6x12RV +=  + Ab(m11,mu2)*AbV(me2,L) * (  - 2./9.*m11*RC146e*
         RSe6611 - 8./27.*m11*m66*RN1e*RS14e*RSe61 + 2./27.*m11*m44*
         RN1e*RS16e*RSe61 + 8./9.*pow(m11,2)*RN1e*pow(RSe6611,2) );

      M12p6x12RV +=  + Ab(m11,mu2)*BbV(m11,me2,L) * (  - 4./9.*pow(
         m11,2)*RC146e*RSe6611 );

      M12p6x12RV +=  + Ab(m11,mu2)*BbV(m11,L) * ( 1./9.*m11*RD146e*
         RC146e + pow(m11,2) + 2./9.*pow(m11,2)*pow(RC146e,2) - 4./9.*
         pow(m11,2)*RN1e*RD146e*RSe6611 );

      M12p6x12RV +=  + Ab(m11,mu2)*CbV(m11,L) * ( 2./9.*pow(m11,2)*
         RD146e*RC146e );

      M12p6x12RV +=  + Ab(m14,mu2)*AbV(m11,L) * ( 4./45.*RD146e + 4./45.
         *m44*RS16e - 44./45.*m11*RS16e + 8./9.*m11*RC146e );

      M12p6x12RV +=  + Ab(m14,mu2)*AbV(m14,L) * (  - 4./27.*RD146e + 2./
         27.*me2 + 8./27.*me2*RSe61 + 8./27.*me2*RSe6611 - 2./27.*m44
          + 8./27.*m44*RSe6611 - 4./27.*m44*RC146e - 47./27.*m11 + 2./
         27.*m11*RSe61 + 2./3.*m11*RSe6611 + 10./27.*m11*RS16e - 13./27.
         *m11*RC146e );

      M12p6x12RV +=  + Ab(m14,mu2)*AbV(m16,L) * (  - m11 );

      M12p6x12RV +=  + Ab(m14,mu2)*AbV(me2,L) * (  - 2./45.*me2 - 8./45.
         *me2*RSe61 - 8./45.*me2*RSe6611 + 2./45.*m44 + 4./45.*m44*
         RSe61 - 4./5.*m11*RSe61 - 8./5.*m11*RSe6611 );

      M12p6x12RV +=  + Ab(m14,mu2)*BbV(m11,me2,L) * (  - 4./9.*m11*m44*
         RSe61 - 8./9.*pow(m11,2)*RSe61 );

      M12p6x12RV +=  + Ab(m14,mu2)*BbV(m11,L) * ( 8./9.*m11*RD146e - 4./
         9.*m11*m44*RS16e - 8./9.*pow(m11,2)*RS16e );

      M12p6x12RV +=  + Ab(m16,mu2)*AbV(m11,L) * ( 2./45.*RD146e + 2./45.
         *m66*RS14e - 22./45.*m11*RS14e + 4./9.*m11*RC146e );

      M12p6x12RV +=  + Ab(m16,mu2)*AbV(m14,L) * (  - m11 );

      M12p6x12RV +=  + Ab(m16,mu2)*AbV(m16,L) * (  - 2./27.*RD146e + 4./
         27.*me2 - 8./27.*me2*RSe61 + 4./27.*me2*RSe6611 - 4./27.*m66
          + 4./27.*m66*RSe6611 - 2./27.*m66*RC146e - 10./27.*m11 - 2./
         27.*m11*RSe61 + 1./3.*m11*RSe6611 + 5./27.*m11*RS14e - 13./54.
         *m11*RC146e );

      M12p6x12RV +=  + Ab(m16,mu2)*AbV(me2,L) * (  - 4./45.*me2 + 8./45.
         *me2*RSe61 - 4./45.*me2*RSe6611 + 4./45.*m66 - 4./45.*m66*
         RSe61 + 4./5.*m11*RSe61 - 4./5.*m11*RSe6611 );

      M12p6x12RV +=  + Ab(m16,mu2)*BbV(m11,me2,L) * ( 4./9.*m11*m66*
         RSe61 + 8./9.*pow(m11,2)*RSe61 );

      M12p6x12RV +=  + Ab(m16,mu2)*BbV(m11,L) * ( 4./9.*m11*RD146e - 2./
         9.*m11*m66*RS14e - 4./9.*pow(m11,2)*RS14e );

      M12p6x12RV +=  + Ab(m44,mu2)*BbV(m11,me2,L) * ( 2./3.*m11*m44*
         RS16e*RSe61 );

      M12p6x12RV +=  + Ab(m44,mu2)*BbV(m11,L) * ( 1./3.*m11*m44*pow(
         RS16e,2) );

      M12p6x12RV +=  + Ab(m44,mu2)*BbV(me2,L) * ( 1./3.*m11*m44*pow(
         RSe61,2) );

      M12p6x12RV +=  + Ab(m46,mu2)*AbV(m11,L) * (  - 2./27.*m11*pow(
         RS16e,2) + 4./27.*m11*RS14e*RS16e - 2./27.*m11*pow(RS14e,2) );

      M12p6x12RV +=  + Ab(m46,mu2)*AbV(me2,L) * (  - 2./3.*m11*pow(
         RSe61,2) - 4./9.*m11*RS16e*RSe61 + 4./9.*m11*RS14e*RSe61 );

      M12p6x12RV +=  + Ab(m46,mu2)*BbV(m11,me2,L) * (  - 8./27.*m11*m66
         *RS16e*RSe61 - 4./27.*m11*m66*RS14e*RSe61 - 4./27.*m11*m44*
         RS16e*RSe61 + 4./27.*m11*m44*RS14e*RSe61 - 4./9.*pow(m11,2)*
         RS16e*RSe61 + 4./9.*pow(m11,2)*RS14e*RSe61 );

      M12p6x12RV +=  + Ab(m46,mu2)*BbV(m11,L) * ( 4./27.*m11*m66*RS14e*
         RS16e + 2./27.*m11*m66*pow(RS14e,2) + 2./27.*m11*m44*pow(
         RS16e,2) + 4./27.*m11*m44*RS14e*RS16e - 2./27.*pow(m11,2)*pow(
         RS16e,2) + 4./27.*pow(m11,2)*RS14e*RS16e - 2./27.*pow(m11,2)*
         pow(RS14e,2) );

      M12p6x12RV +=  + Ab(m46,mu2)*BbV(me2,L) * (  - 2./3.*m11*me2*pow(
         RSe61,2) - 2./9.*m11*m44*pow(RSe61,2) );

      M12p6x12RV +=  + Ab(me2,mu2)*AbV(m11,L) * (  - 2./9.*m11*RC146e*
         RSe6611 + 4./9.*pow(m11,2)*RN1e*RC146e*RSe6611 );

      M12p6x12RV +=  + Ab(me2,mu2)*AbV(m14,L) * (  - 2./45.*me2 - 8./45.
         *me2*RSe61 - 8./45.*me2*RSe6611 + 2./45.*m44 + 4./45.*m44*
         RSe61 - 4./5.*m11*RSe61 - 8./5.*m11*RSe6611 + 4./9.*m11*m44*
         RN1e*RSe61 + 8./9.*pow(m11,2)*RN1e*RSe61 );

      M12p6x12RV +=  + Ab(me2,mu2)*AbV(m16,L) * (  - 4./45.*me2 + 8./45.
         *me2*RSe61 - 4./45.*me2*RSe6611 + 4./45.*m66 - 4./45.*m66*
         RSe61 + 4./5.*m11*RSe61 - 4./5.*m11*RSe6611 - 4./9.*m11*m66*
         RN1e*RSe61 - 8./9.*pow(m11,2)*RN1e*RSe61 );

      M12p6x12RV +=  + Ab(me2,mu2)*AbV(m44,L) * (  - 2./3.*m11*m44*RN1e
         *RS16e*RSe61 );

      M12p6x12RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 2./3.*m11*pow(
         RSe61,2) - 4./9.*m11*RS16e*RSe61 + 4./9.*m11*RS14e*RSe61 + 8./
         27.*m11*m66*RN1e*RS16e*RSe61 + 4./27.*m11*m66*RN1e*RS14e*RSe61
          + 4./27.*m11*m44*RN1e*RS16e*RSe61 - 4./27.*m11*m44*RN1e*RS14e
         *RSe61 + 4./9.*pow(m11,2)*RN1e*RS16e*RSe61 - 4./9.*pow(m11,2)*
         RN1e*RS14e*RSe61 );

      M12p6x12RV +=  + Ab(me2,mu2)*AbV(me2,L) * ( 4./9.*m11*pow(
         RSe6611,2) + 8./27.*m11*m66*RN1e*RS14e*RSe61 - 2./27.*m11*m44*
         RN1e*RS16e*RSe61 - 8./9.*pow(m11,2)*RN1e*pow(RSe6611,2) );

      M12p6x12RV +=  + Ab(me2,mu2)*BbV(m11,me2,L) * (  - 8./27.*m11*m66
         *RS14e*RSe61 + 2./27.*m11*m44*RS16e*RSe61 + 8./9.*pow(m11,2)*
         pow(RSe6611,2) );

      M12p6x12RV +=  + Ab(me2,mu2)*BbV(m11,L) * (  - 2./9.*m11*RD146e*
         RSe6611 + 2./27.*m11*m66*pow(RS14e,2) + 1./27.*m11*m44*pow(
         RS16e,2) - 4./9.*pow(m11,2)*RC146e*RSe6611 + 4./9.*pow(m11,2)*
         RN1e*RD146e*RSe6611 );

      M12p6x12RV +=  + Ab(me2,mu2)*BbV(me2,L) * ( 8./27.*m11*m66*pow(
         RSe61,2) + 1./27.*m11*m44*pow(RSe61,2) );

      M12p6x12RV +=  + Ab(me2,mu2)*CbV(m11,L) * (  - 4./9.*pow(m11,2)*
         RD146e*RSe6611 );

      M12p6x12RV +=  + Bb(m11,mu2)*AbV(m11,L) * ( 1./9.*m11*RD146e*
         RC146e + pow(m11,2) + 2./9.*pow(m11,2)*pow(RC146e,2) );

      M12p6x12RV +=  + Bb(m11,mu2)*AbV(m14,L) * ( 8./9.*m11*RD146e - 4./
         9.*m11*m44*RS16e - 8./9.*pow(m11,2)*RS16e );

      M12p6x12RV +=  + Bb(m11,mu2)*AbV(m16,L) * ( 4./9.*m11*RD146e - 2./
         9.*m11*m66*RS14e - 4./9.*pow(m11,2)*RS14e );

      M12p6x12RV +=  + Bb(m11,mu2)*AbV(m44,L) * ( 1./3.*m11*m44*pow(
         RS16e,2) );

      M12p6x12RV +=  + Bb(m11,mu2)*AbV(m46,L) * ( 4./27.*m11*m66*RS14e*
         RS16e + 2./27.*m11*m66*pow(RS14e,2) + 2./27.*m11*m44*pow(
         RS16e,2) + 4./27.*m11*m44*RS14e*RS16e - 2./27.*pow(m11,2)*pow(
         RS16e,2) + 4./27.*pow(m11,2)*RS14e*RS16e - 2./27.*pow(m11,2)*
         pow(RS14e,2) );

      M12p6x12RV +=  + Bb(m11,mu2)*AbV(me2,L) * (  - 2./9.*m11*RD146e*
         RSe6611 + 2./27.*m11*m66*pow(RS14e,2) + 1./27.*m11*m44*pow(
         RS16e,2) - 4./9.*pow(m11,2)*RC146e*RSe6611 );

      M12p6x12RV +=  + Bb(m11,mu2)*BbV(m11,me2,L) * (  - 4./9.*pow(
         m11,2)*RD146e*RSe6611 );

      M12p6x12RV +=  + Bb(m11,mu2)*BbV(m11,L) * ( 1./9.*m11*pow(
         RD146e,2) + 4./9.*pow(m11,2)*RD146e*RC146e );

      M12p6x12RV +=  + Bb(m11,mu2)*CbV(m11,L) * ( 2./9.*pow(m11,2)*pow(
         RD146e,2) );

      M12p6x12RV +=  + Bb(me2,mu2)*AbV(m44,L) * ( 1./3.*m11*m44*pow(
         RSe61,2) );

      M12p6x12RV +=  + Bb(me2,mu2)*AbV(m46,L) * (  - 2./3.*m11*me2*pow(
         RSe61,2) - 2./9.*m11*m44*pow(RSe61,2) );

      M12p6x12RV +=  + Bb(me2,mu2)*AbV(me2,L) * ( 8./27.*m11*m66*pow(
         RSe61,2) + 1./27.*m11*m44*pow(RSe61,2) );

      M12p6x12RV +=  + Cb(m11,mu2)*AbV(m11,L) * ( 2./9.*pow(m11,2)*
         RD146e*RC146e );

      M12p6x12RV +=  + Cb(m11,mu2)*AbV(me2,L) * (  - 4./9.*pow(m11,2)*
         RD146e*RSe6611 );

      M12p6x12RV +=  + Cb(m11,mu2)*BbV(m11,L) * ( 2./9.*pow(m11,2)*pow(
         RD146e,2) );

      M12p6x12RV +=  + AbV(m11,L)*pi16 * (  - 1./4.*pow(m11,2) + 1./6.*
         pow(m11,2)*RS16e + 1./12.*pow(m11,2)*RS14e );

      M12p6x12RV +=  + pow(AbV(m11,L),2) * ( 1./2.*m11 + 1./18.*m11*
         pow(RC146e,2) );

      M12p6x12RV +=  + AbV(m11,L)*AbV(m14,L) * ( 4./45.*RD146e + 4./45.
         *m44*RS16e - 44./45.*m11*RS16e + 8./9.*m11*RC146e );

      M12p6x12RV +=  + AbV(m11,L)*AbV(m16,L) * ( 2./45.*RD146e + 2./45.
         *m66*RS14e - 22./45.*m11*RS14e + 4./9.*m11*RC146e );

      M12p6x12RV +=  + AbV(m11,L)*AbV(m46,L) * (  - 2./27.*m11*pow(
         RS16e,2) + 4./27.*m11*RS14e*RS16e - 2./27.*m11*pow(RS14e,2) );

      M12p6x12RV +=  + AbV(m11,L)*AbV(me2,L) * (  - 2./9.*m11*RC146e*
         RSe6611 );

      M12p6x12RV +=  + AbV(m11,L)*BbV(m11,me2,L) * (  - 4./9.*pow(
         m11,2)*RC146e*RSe6611 );

      M12p6x12RV +=  + AbV(m11,L)*BbV(m11,L) * ( 1./9.*m11*RD146e*
         RC146e + pow(m11,2) + 2./9.*pow(m11,2)*pow(RC146e,2) );

      M12p6x12RV +=  + AbV(m11,L)*CbV(m11,L) * ( 2./9.*pow(m11,2)*
         RD146e*RC146e );

      M12p6x12RV +=  + AbV(m14,L)*pi16 * (  - 1./6.*m11*RD146e + 1./12.
         *m11*me2 + 1./3.*m11*me2*RSe61 + 1./3.*m11*me2*RSe6611 + 1./2.
         *m11*m66 + 7./6.*m11*m44 + 1./6.*m11*m44*RSe61 + 1./6.*m11*m44
         *RSe6611 + 1./6.*m11*m44*RS16e - 1./12.*m11*m44*RC146e - 1./36.
         *pow(m11,2) + 1./18.*pow(m11,2)*RSe61 + 1./18.*pow(m11,2)*
         RSe6611 + 7./18.*pow(m11,2)*RS16e - 7./36.*pow(m11,2)*RC146e )
         ;

      M12p6x12RV +=  + pow(AbV(m14,L),2) * (  - 2./27.*RD146e + 1./27.*
         me2 + 4./27.*me2*RSe61 + 4./27.*me2*RSe6611 - 1./27.*m44 + 4./
         27.*m44*RSe6611 - 2./27.*m44*RC146e - 47./54.*m11 + 1./27.*m11
         *RSe61 + 1./3.*m11*RSe6611 + 5./27.*m11*RS16e - 13./54.*m11*
         RC146e );

      M12p6x12RV +=  + AbV(m14,L)*AbV(m16,L) * (  - m11 );

      M12p6x12RV +=  + AbV(m14,L)*AbV(me2,L) * (  - 2./45.*me2 - 8./45.
         *me2*RSe61 - 8./45.*me2*RSe6611 + 2./45.*m44 + 4./45.*m44*
         RSe61 - 4./5.*m11*RSe61 - 8./5.*m11*RSe6611 );

      M12p6x12RV +=  + AbV(m14,L)*BbV(m11,me2,L) * (  - 4./9.*m11*m44*
         RSe61 - 8./9.*pow(m11,2)*RSe61 );

      M12p6x12RV +=  + AbV(m14,L)*BbV(m11,L) * ( 8./9.*m11*RD146e - 4./
         9.*m11*m44*RS16e - 8./9.*pow(m11,2)*RS16e );

      M12p6x12RV +=  + AbV(m16,L)*pi16 * (  - 1./12.*m11*RD146e + 1./6.
         *m11*me2 - 1./3.*m11*me2*RSe61 + 1./6.*m11*me2*RSe6611 + 5./24.
         *m11*m66 - 1./6.*m11*m66*RSe61 + 1./12.*m11*m66*RSe6611 + 1./
         12.*m11*m66*RS14e - 1./24.*m11*m66*RC146e + 1./2.*m11*m44 - 1./
         72.*pow(m11,2) - 1./18.*pow(m11,2)*RSe61 + 1./36.*pow(m11,2)*
         RSe6611 + 7./36.*pow(m11,2)*RS14e - 7./72.*pow(m11,2)*RC146e )
         ;

      M12p6x12RV +=  + pow(AbV(m16,L),2) * (  - 1./27.*RD146e + 2./27.*
         me2 - 4./27.*me2*RSe61 + 2./27.*me2*RSe6611 - 2./27.*m66 + 2./
         27.*m66*RSe6611 - 1./27.*m66*RC146e - 5./27.*m11 - 1./27.*m11*
         RSe61 + 1./6.*m11*RSe6611 + 5./54.*m11*RS14e - 13./108.*m11*
         RC146e );

      M12p6x12RV +=  + AbV(m16,L)*AbV(me2,L) * (  - 4./45.*me2 + 8./45.
         *me2*RSe61 - 4./45.*me2*RSe6611 + 4./45.*m66 - 4./45.*m66*
         RSe61 + 4./5.*m11*RSe61 - 4./5.*m11*RSe6611 );

      M12p6x12RV +=  + AbV(m16,L)*BbV(m11,me2,L) * ( 4./9.*m11*m66*
         RSe61 + 8./9.*pow(m11,2)*RSe61 );

      M12p6x12RV +=  + AbV(m16,L)*BbV(m11,L) * ( 4./9.*m11*RD146e - 2./
         9.*m11*m66*RS14e - 4./9.*pow(m11,2)*RS14e );

      M12p6x12RV +=  + AbV(m44,L)*BbV(m11,me2,L) * ( 2./3.*m11*m44*
         RS16e*RSe61 );

      M12p6x12RV +=  + AbV(m44,L)*BbV(m11,L) * ( 1./3.*m11*m44*pow(
         RS16e,2) );

      M12p6x12RV +=  + AbV(m44,L)*BbV(me2,L) * ( 1./3.*m11*m44*pow(
         RSe61,2) );

      M12p6x12RV +=  + AbV(m46,L)*AbV(me2,L) * (  - 2./3.*m11*pow(
         RSe61,2) - 4./9.*m11*RS16e*RSe61 + 4./9.*m11*RS14e*RSe61 );

      M12p6x12RV +=  + AbV(m46,L)*BbV(m11,me2,L) * (  - 8./27.*m11*m66*
         RS16e*RSe61 - 4./27.*m11*m66*RS14e*RSe61 - 4./27.*m11*m44*
         RS16e*RSe61 + 4./27.*m11*m44*RS14e*RSe61 - 4./9.*pow(m11,2)*
         RS16e*RSe61 + 4./9.*pow(m11,2)*RS14e*RSe61 );

      M12p6x12RV +=  + AbV(m46,L)*BbV(m11,L) * ( 4./27.*m11*m66*RS14e*
         RS16e + 2./27.*m11*m66*pow(RS14e,2) + 2./27.*m11*m44*pow(
         RS16e,2) + 4./27.*m11*m44*RS14e*RS16e - 2./27.*pow(m11,2)*pow(
         RS16e,2) + 4./27.*pow(m11,2)*RS14e*RS16e - 2./27.*pow(m11,2)*
         pow(RS14e,2) );

      M12p6x12RV +=  + AbV(m46,L)*BbV(me2,L) * (  - 2./3.*m11*me2*pow(
         RSe61,2) - 2./9.*m11*m44*pow(RSe61,2) );

      M12p6x12RV +=  + pow(AbV(me2,L),2) * ( 2./9.*m11*pow(RSe6611,2) )
         ;

      M12p6x12RV +=  + AbV(me2,L)*BbV(m11,me2,L) * (  - 8./27.*m11*m66*
         RS14e*RSe61 + 2./27.*m11*m44*RS16e*RSe61 + 8./9.*pow(m11,2)*
         pow(RSe6611,2) );

      M12p6x12RV +=  + AbV(me2,L)*BbV(m11,L) * (  - 2./9.*m11*RD146e*
         RSe6611 + 2./27.*m11*m66*pow(RS14e,2) + 1./27.*m11*m44*pow(
         RS16e,2) - 4./9.*pow(m11,2)*RC146e*RSe6611 );

      M12p6x12RV +=  + AbV(me2,L)*BbV(me2,L) * ( 8./27.*m11*m66*pow(
         RSe61,2) + 1./27.*m11*m44*pow(RSe61,2) );

      M12p6x12RV +=  + AbV(me2,L)*CbV(m11,L) * (  - 4./9.*pow(m11,2)*
         RD146e*RSe6611 );

      M12p6x12RV +=  + BbV(m11,me2,L)*BbV(m11,L) * (  - 4./9.*pow(
         m11,2)*RD146e*RSe6611 );

      M12p6x12RV +=  + pow(BbV(m11,L),2) * ( 1./18.*m11*pow(RD146e,2)
          + 2./9.*pow(m11,2)*RD146e*RC146e );

      M12p6x12RV +=  + BbV(m11,L)*CbV(m11,L) * ( 2./9.*pow(m11,2)*pow(
         RD146e,2) );

      M12p6x12RV +=  + A23bV(m11,L)*pi16 * ( 3./4.*m11 - 1./2.*m11*
         RS16e - 1./4.*m11*RS14e );

      M12p6x12RV +=  + A23bV(m14,L)*pi16 * ( 1./6.*m11 - 1./3.*m11*
         RSe61 - 1./3.*m11*RSe6611 - 1./3.*m11*RS16e + 1./6.*m11*RC146e
          );

      M12p6x12RV +=  + A23bV(m16,L)*pi16 * ( 1./12.*m11 + 1./3.*m11*
         RSe61 - 1./6.*m11*RSe6611 - 1./6.*m11*RS14e + 1./12.*m11*
         RC146e );

      M12p6x12RV +=  + hhV(1,m11,m11,m11,m11,L,mu2) * ( 1./3.*pow(
         m11,2) + 2./9.*pow(m11,2)*pow(RC146e,2) );

      M12p6x12RV +=  + hhV(1,m11,m11,me2,m11,L,mu2) * (  - 8./9.*pow(
         m11,2)*RC146e*RSe6611 );

      M12p6x12RV +=  + hhV(1,m11,m14,m14,m11,L,mu2) * (  - 11./18.*m11*
         RD146e - 4./9.*m11*m44*RS16e + 7./9.*pow(m11,2)*RS16e - 5./6.*
         pow(m11,2)*RC146e );

      M12p6x12RV +=  + hhV(1,m11,m16,m16,m11,L,mu2) * (  - 11./36.*m11*
         RD146e - 2./9.*m11*m66*RS14e + 7./18.*pow(m11,2)*RS14e - 5./12.
         *pow(m11,2)*RC146e );

      M12p6x12RV +=  + hhV(1,m11,me2,me2,m11,L,mu2) * ( 8./9.*pow(
         m11,2)*pow(RSe6611,2) );

      M12p6x12RV +=  + hhV(1,m14,m14,me2,m11,L,mu2) * ( 1./27.*pow(
         me2,2) + 4./27.*pow(me2,2)*RSe61 + 4./27.*pow(me2,2)*RSe6611
          - 2./27.*m44*me2 - 4./27.*m44*me2*RSe61 + 1./27.*pow(m44,2)
          - 2./9.*m11*me2 - 16./27.*m11*me2*RSe61 - 8./27.*m11*me2*
         RSe6611 + 2./9.*m11*m44 + 4./27.*m11*m44*RSe61 - 8./9.*pow(
         m11,2)*RSe61 - 32./27.*pow(m11,2)*RSe6611 );

      M12p6x12RV +=  + hhV(1,m16,m16,me2,m11,L,mu2) * ( 2./27.*pow(
         me2,2) - 4./27.*pow(me2,2)*RSe61 + 2./27.*pow(me2,2)*RSe6611
          - 4./27.*m66*me2 + 4./27.*m66*me2*RSe61 + 2./27.*pow(m66,2)
          - 4./9.*m11*me2 + 16./27.*m11*me2*RSe61 - 4./27.*m11*me2*
         RSe6611 + 4./9.*m11*m66 - 4./27.*m11*m66*RSe61 + 8./9.*pow(
         m11,2)*RSe61 - 16./27.*pow(m11,2)*RSe6611 );

      M12p6x12RV +=  + hhV(1,m44,m14,m14,m11,L,mu2) * ( 3./4.*m11*m44 )
         ;

      M12p6x12RV +=  + hhV(1,m46,m14,m16,m11,L,mu2) * ( 1./2.*m11*m66
          + 1./2.*m11*m44 );

      M12p6x12RV +=  + hhV(1,me2,m14,m14,m11,L,mu2) * ( 1./12.*m11*me2
          + 1./3.*m11*me2*RSe61 + 1./3.*m11*me2*RSe6611 );

      M12p6x12RV +=  + hhV(1,me2,m16,m16,m11,L,mu2) * ( 1./6.*m11*me2
          - 1./3.*m11*me2*RSe61 + 1./6.*m11*me2*RSe6611 );

      M12p6x12RV +=  + hhV(2,m11,m11,m11,m11,L,mu2) * ( 4./9.*pow(
         m11,2)*RD146e*RC146e );

      M12p6x12RV +=  + hhV(2,m11,m11,me2,m11,L,mu2) * (  - 8./9.*pow(
         m11,2)*RD146e*RSe6611 );

      M12p6x12RV +=  + hhV(2,m11,m14,m14,m11,L,mu2) * (  - 5./6.*pow(
         m11,2)*RD146e );

      M12p6x12RV +=  + hhV(2,m11,m16,m16,m11,L,mu2) * (  - 5./12.*pow(
         m11,2)*RD146e );

      M12p6x12RV +=  + hhV(5,m11,m11,m11,m11,L,mu2) * ( 2./9.*pow(
         m11,2)*pow(RD146e,2) );

      M12p6x12RV +=  + hh1V(1,m11,m14,m14,m11,L,mu2) * ( 4./9.*m11*
         RD146e + 4./9.*m11*m44*RS16e - 16./9.*pow(m11,2)*RS16e + 4./3.
         *pow(m11,2)*RC146e );

      M12p6x12RV +=  + hh1V(1,m11,m16,m16,m11,L,mu2) * ( 2./9.*m11*
         RD146e + 2./9.*m11*m66*RS14e - 8./9.*pow(m11,2)*RS14e + 2./3.*
         pow(m11,2)*RC146e );

      M12p6x12RV +=  + hh1V(1,m14,m14,me2,m11,L,mu2) * ( 4./9.*m11*me2
          + 16./9.*m11*me2*RSe61 + 16./9.*m11*me2*RSe6611 - 4./9.*m11*
         m44 - 8./9.*m11*m44*RSe61 + 16./9.*pow(m11,2)*RSe61 + 32./9.*
         pow(m11,2)*RSe6611 );

      M12p6x12RV +=  + hh1V(1,m16,m16,me2,m11,L,mu2) * ( 8./9.*m11*me2
          - 16./9.*m11*me2*RSe61 + 8./9.*m11*me2*RSe6611 - 8./9.*m11*
         m66 + 8./9.*m11*m66*RSe61 - 16./9.*pow(m11,2)*RSe61 + 16./9.*
         pow(m11,2)*RSe6611 );

      M12p6x12RV +=  + hh1V(2,m11,m14,m14,m11,L,mu2) * ( 4./3.*pow(
         m11,2)*RD146e );

      M12p6x12RV +=  + hh1V(2,m11,m16,m16,m11,L,mu2) * ( 2./3.*pow(
         m11,2)*RD146e );

      M12p6x12RV +=  + hh21V(1,m11,m14,m14,m11,L,mu2) * ( pow(m11,2)*
         RS16e - 1./2.*pow(m11,2)*RC146e );

      M12p6x12RV +=  + hh21V(1,m11,m16,m16,m11,L,mu2) * ( 1./2.*pow(
         m11,2)*RS14e - 1./4.*pow(m11,2)*RC146e );

      M12p6x12RV +=  + hh21V(1,m44,m14,m14,m11,L,mu2) * ( 9./4.*pow(
         m11,2) );

      M12p6x12RV +=  + hh21V(1,m46,m14,m16,m11,L,mu2) * ( 3*pow(m11,2)
          );

      M12p6x12RV +=  + hh21V(1,me2,m14,m14,m11,L,mu2) * ( 1./4.*pow(
         m11,2) + pow(m11,2)*RSe61 + pow(m11,2)*RSe6611 );

      M12p6x12RV +=  + hh21V(1,me2,m16,m16,m11,L,mu2) * ( 1./2.*pow(
         m11,2) - pow(m11,2)*RSe61 + 1./2.*pow(m11,2)*RSe6611 );

      M12p6x12RV +=  + hh21V(2,m11,m14,m14,m11,L,mu2) * (  - 1./2.*pow(
         m11,2)*RD146e );

      M12p6x12RV +=  + hh21V(2,m11,m16,m16,m11,L,mu2) * (  - 1./4.*pow(
         m11,2)*RD146e );

      M12p6x12RV +=  + hh27V(1,m11,m14,m14,m11,L,mu2) * (  - m11*RS16e
          + 1./2.*m11*RC146e );

      M12p6x12RV +=  + hh27V(1,m11,m16,m16,m11,L,mu2) * (  - 1./2.*m11*
         RS14e + 1./4.*m11*RC146e );

      M12p6x12RV +=  + hh27V(1,m44,m14,m14,m11,L,mu2) * (  - 9./4.*m11
          );

      M12p6x12RV +=  + hh27V(1,m46,m14,m16,m11,L,mu2) * (  - 3*m11 );

      M12p6x12RV +=  + hh27V(1,me2,m14,m14,m11,L,mu2) * (  - 1./4.*m11
          - m11*RSe61 - m11*RSe6611 );

      M12p6x12RV +=  + hh27V(1,me2,m16,m16,m11,L,mu2) * (  - 1./2.*m11
          + m11*RSe61 - 1./2.*m11*RSe6611 );

      M12p6x12RV +=  + hh27V(2,m11,m14,m14,m11,L,mu2) * ( 1./2.*m11*
         RD146e );

      M12p6x12RV +=  + hh27V(2,m11,m16,m16,m11,L,mu2) * ( 1./4.*m11*
         RD146e );

  return M12p6x12RV/pow(f0,4);
}

// 2 valence mass 2 sea mass *****************************************

double mv2s2nf3p4V(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 4) || (B0mq.size() != 4)){
    cout << "ERROR: mv2s1nf3p4V probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m66 = 2.*B0mq[3];
  double me2 = (m44+2.*m66)/3.;
  double RD146e = RD(m11,m44,m66,me2);
  double RD346e = RD(m33,m44,m66,me2);
  double RS1463e = RS(m11,m44,m66,m33,me2);
  double RS3461e = RS(m33,m44,m66,m11,me2);
  double RSe6611 = RS(me2,m66,m66,m11,m11);
  double RSe6613 = RS(me2,m66,m66,m11,m33);
  double RSe6633 = RS(me2,m66,m66,m33,m33);
  //double mu2 = mu0*mu0;
  double M13p4x22V =
       + AbV(m11,L) * ( 1./18.*RD146e - 1./9.*m33*RS1463e - 2./9.*m11*
         RS1463e );

      M13p4x22V +=  + AbV(m33,L) * ( 1./18.*RD346e - 2./9.*m33*RS3461e
          - 1./9.*m11*RS3461e );

      M13p4x22V +=  + AbV(me2,L) * ( 2./9.*me2*RSe6613 - 1./9.*me2*
         RSe6633 - 1./9.*me2*RSe6611 + 2./9.*m33*RSe6613 + 1./9.*m33*
         RSe6633 + 2./9.*m11*RSe6613 + 1./9.*m11*RSe6611 );

  return M13p4x22V/pow(f0,2);
}

double mv2s2nf3p6V(const quarkmassnf mass, const Linf Liin, const double L){
  return mv2s2nf3p6LV(mass,Liin,L)+mv2s2nf3p6RV(mass,L);
}

double mv2s2nf3p6LV(const quarkmassnf mass, const Linf Liin, const double L){
  double L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu;
  int nfLi;
  Liin.out(L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu,nfLi);
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if (mu0 != mu){
    cout << "ERROR: differing mu's in mv2s2nf3p6LV\n";}
  if ( (nq != 4) || (B0mq.size() != 4)){
    cout << "ERROR: mv2s2nf3p6LV probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m66 = 2.*B0mq[3];
  double m13 = (m11+m33)/2;
  double m14 = (m11+m44)/2;
  double m16 = (m11+m66)/2;
  double m34 = (m33+m44)/2;
  double m36 = (m33+m66)/2;
  double m46 = (m44+m66)/2;
  double me2 = (m44+2.*m66)/3.;

  //double RN13 = RN(m11,m33);
  //double RN1e = RN(m11,me2);
  //double RN3e = RN(m33,me2);

  double RD146e = RD(m11,m44,m66,me2);
  double RD346e = RD(m33,m44,m66,me2);

  double RC146e = RC(m11,m44,m66,me2);
  double RC346e = RC(m33,m44,m66,me2);

  double RS14e = RS(m11,m44,me2);
  double RS16e = RS(m11,m66,me2);
  double RS34e = RS(m33,m44,me2);
  double RS36e = RS(m33,m66,me2);
  double RSe61 = RS(me2,m66,m11);
  double RSe63 = RS(me2,m66,m33);

  double RS1463e = RS(m11,m44,m66,m33,me2);
  double RS3461e = RS(m33,m44,m66,m11,me2);
  double RSe6611 = RS(me2,m66,m66,m11,m11);
  double RSe6613 = RS(me2,m66,m66,m11,m33);
  double RSe6633 = RS(me2,m66,m66,m33,m33);

  //double mu2 = mu0*mu0;

  double M13p6x22LV =
       + AbV(m11,L) * (  - 8./9.*m66*L4r*RD146e - 8./27.*pow(m66,2)*L8r
         *pow(RS14e,2) - 8./27.*pow(m66,2)*L7r*pow(RS14e,2) - 8./27.*
         pow(m66,2)*L6r*pow(RS14e,2) - 16./9.*m44*L4r*RD146e - 32./27.*
         m44*m66*L7r*RS14e*RS16e - 16./27.*m44*m66*L6r*pow(RS16e,2) - 
         16./27.*m44*m66*L6r*pow(RS14e,2) - 16./27.*pow(m44,2)*L8r*pow(
         RS16e,2) - 32./27.*pow(m44,2)*L7r*pow(RS16e,2) - 32./27.*pow(
         m44,2)*L6r*pow(RS16e,2) + 4./9.*m33*L5r*RD146e - 5./3.*m33*L3r
         *RD146e - 2./3.*m33*L0r*RD146e - 16./9.*m33*m66*L7r*RS14e - 16.
         /9.*m33*m66*L6r*RS1463e + 16./9.*m33*m66*L4r*RS1463e - 32./9.*
         m33*m44*L7r*RS16e - 32./9.*m33*m44*L6r*RS1463e + 32./9.*m33*
         m44*L4r*RS1463e - 16./9.*pow(m33,2)*L8r*RS1463e + 4./9.*pow(
         m33,2)*L5r*RS1463e - 5./3.*m11*L3r*RD146e - 2./3.*m11*L0r*
         RD146e - 16./9.*m11*m66*L7r*RS14e - 32./9.*m11*m66*L6r*RS1463e
          + 8./9.*m11*m66*L6r*RC146e + 16./27.*m11*m66*L5r*RS14e*RSe63
          - 8./27.*m11*m66*L5r*RS14e*RS34e - 16./27.*m11*m66*L5r*RS14e*
         RSe61 );

      M13p6x22LV +=  + AbV(m11,L) * ( 4./27.*m11*m66*L5r*pow(RS14e,2)
          + 8./9.*m11*m66*L4r*RS3461e + 40./9.*m11*m66*L4r*RS1463e - 16.
         /9.*m11*m66*L4r*RSe6613 + 16./9.*m11*m66*L4r*RSe6611 - 16./27.
         *m11*m66*L4r*RS16e*RSe63 - 16./27.*m11*m66*L4r*RS16e*RS36e + 
         16./27.*m11*m66*L4r*RS16e*RSe61 + 8./27.*m11*m66*L4r*pow(
         RS16e,2) + 16./27.*m11*m66*L4r*RS14e*RSe63 - 8./27.*m11*m66*
         L4r*RS14e*RS34e - 16./27.*m11*m66*L4r*RS14e*RSe61 + 4./27.*m11
         *m66*L4r*pow(RS14e,2) - 4./9.*m11*m66*L4r*RC146e - 32./9.*m11*
         m44*L7r*RS16e - 64./9.*m11*m44*L6r*RS1463e + 16./9.*m11*m44*
         L6r*RC146e - 16./27.*m11*m44*L5r*RS16e*RSe63 - 16./27.*m11*m44
         *L5r*RS16e*RS36e + 16./27.*m11*m44*L5r*RS16e*RSe61 + 8./27.*
         m11*m44*L5r*pow(RS16e,2) + 16./9.*m11*m44*L4r*RS3461e + 80./9.
         *m11*m44*L4r*RS1463e - 32./9.*m11*m44*L4r*RSe6613 + 32./9.*m11
         *m44*L4r*RSe6611 - 32./27.*m11*m44*L4r*RS16e*RSe63 - 32./27.*
         m11*m44*L4r*RS16e*RS36e + 32./27.*m11*m44*L4r*RS16e*RSe61 + 16.
         /27.*m11*m44*L4r*pow(RS16e,2) );

      M13p6x22LV +=  + AbV(m11,L) * ( 32./27.*m11*m44*L4r*RS14e*RSe63
          - 16./27.*m11*m44*L4r*RS14e*RS34e - 32./27.*m11*m44*L4r*RS14e
         *RSe61 + 8./27.*m11*m44*L4r*pow(RS14e,2) - 8./9.*m11*m44*L4r*
         RC146e - 16./3.*m11*m33*L8r*RS1463e - 8./3.*m11*m33*L8r*RC146e
          + 16./3.*m11*m33*L7r + 8./3.*m11*m33*L5r*RS1463e + 4./3.*m11*
         m33*L5r*RC146e - 2./3.*m11*m33*L3r*RS1463e - 5./3.*m11*m33*L3r
         *RC146e - 8./3.*m11*m33*L0r*RS1463e - 2./3.*m11*m33*L0r*RC146e
          - 32./9.*pow(m11,2)*L8r*RS1463e - 16./9.*pow(m11,2)*L8r*
         RC146e + 8*pow(m11,2)*L7r + 8./9.*pow(m11,2)*L5r*RS3461e + 28./
         9.*pow(m11,2)*L5r*RS1463e - 16./9.*pow(m11,2)*L5r*RSe6613 + 16.
         /9.*pow(m11,2)*L5r*RSe6611 + 8./9.*pow(m11,2)*L5r*RC146e - 2./
         3.*pow(m11,2)*L3r*RS1463e - 5./3.*pow(m11,2)*L3r*RC146e - 8./3.
         *pow(m11,2)*L0r*RS1463e - 2./3.*pow(m11,2)*L0r*RC146e );

      M13p6x22LV +=  + AbV(m13,L) * ( 8*pow(m33,2)*L6r + 5*pow(m33,2)*
         L2r + 2*pow(m33,2)*L1r + 16*m11*m33*L6r + 10*m11*m33*L2r + 4*
         m11*m33*L1r + 8*pow(m11,2)*L6r + 5*pow(m11,2)*L2r + 2*pow(
         m11,2)*L1r );

      M13p6x22LV +=  + AbV(m14,L) * ( 8*m33*m44*L8r - 4*m33*m44*L5r + 5
         *m33*m44*L3r + 2*m33*m44*L0r + 8*m11*m44*L8r - 4*m11*m44*L5r
          + 5*m11*m44*L3r + 2*m11*m44*L0r + 8*m11*m33*L8r - 4*m11*m33*
         L5r + 5*m11*m33*L3r + 2*m11*m33*L0r + 8*pow(m11,2)*L8r - 4*
         pow(m11,2)*L5r + 5*pow(m11,2)*L3r + 2*pow(m11,2)*L0r );

      M13p6x22LV +=  + AbV(m16,L) * ( 4*m33*m66*L8r - 2*m33*m66*L5r + 5.
         /2.*m33*m66*L3r + m33*m66*L0r + 4*m11*m66*L8r - 2*m11*m66*L5r
          + 5./2.*m11*m66*L3r + m11*m66*L0r + 4*m11*m33*L8r - 2*m11*m33
         *L5r + 5./2.*m11*m33*L3r + m11*m33*L0r + 4*pow(m11,2)*L8r - 2*
         pow(m11,2)*L5r + 5./2.*pow(m11,2)*L3r + pow(m11,2)*L0r );

      M13p6x22LV +=  + AbV(m33,L) * (  - 8./9.*m66*L4r*RD346e - 8./27.*
         pow(m66,2)*L8r*pow(RS34e,2) + 16./27.*pow(m66,2)*L8r*RS14e*
         RS34e - 8./27.*pow(m66,2)*L7r*pow(RS34e,2) + 16./27.*pow(
         m66,2)*L7r*RS14e*RS34e - 8./27.*pow(m66,2)*L6r*pow(RS34e,2) + 
         16./27.*pow(m66,2)*L6r*RS14e*RS34e - 16./9.*m44*L4r*RD346e - 
         32./27.*m44*m66*L7r*RS34e*RS36e + 32./27.*m44*m66*L7r*RS16e*
         RS34e + 32./27.*m44*m66*L7r*RS14e*RS36e - 16./27.*m44*m66*L6r*
         pow(RS36e,2) - 16./27.*m44*m66*L6r*pow(RS34e,2) + 32./27.*m44*
         m66*L6r*RS16e*RS36e + 32./27.*m44*m66*L6r*RS14e*RS34e - 16./27.
         *pow(m44,2)*L8r*pow(RS36e,2) + 32./27.*pow(m44,2)*L8r*RS16e*
         RS36e - 32./27.*pow(m44,2)*L7r*pow(RS36e,2) + 64./27.*pow(
         m44,2)*L7r*RS16e*RS36e - 32./27.*pow(m44,2)*L6r*pow(RS36e,2)
          + 64./27.*pow(m44,2)*L6r*RS16e*RS36e - 5./3.*m33*L3r*RD346e
          - 2./3.*m33*L0r*RD346e - 16./9.*m33*m66*L7r*RS34e - 16./9.*
         m33*m66*L7r*RS14e - 32./9.*m33*m66*L6r*RS3461e - 16./9.*m33*
         m66*L6r*RS1463e );

      M13p6x22LV +=  + AbV(m33,L) * ( 8./9.*m33*m66*L6r*RC346e - 16./27.
         *m33*m66*L5r*RS34e*RSe63 + 4./27.*m33*m66*L5r*pow(RS34e,2) + 
         16./27.*m33*m66*L5r*RSe61*RS34e - 16./27.*m33*m66*L5r*RS14e*
         RS34e + 16./3.*m33*m66*L4r*RS3461e + 16./9.*m33*m66*L4r*
         RS1463e - 16./9.*m33*m66*L4r*RSe6613 + 16./9.*m33*m66*L4r*
         RSe6633 + 16./27.*m33*m66*L4r*RS36e*RSe63 + 8./27.*m33*m66*L4r
         *pow(RS36e,2) - 16./27.*m33*m66*L4r*RS34e*RSe63 + 4./27.*m33*
         m66*L4r*pow(RS34e,2) - 4./9.*m33*m66*L4r*RC346e - 16./27.*m33*
         m66*L4r*RSe61*RS36e + 16./27.*m33*m66*L4r*RSe61*RS34e - 32./27.
         *m33*m66*L4r*RS16e*RS36e - 16./27.*m33*m66*L4r*RS14e*RS34e - 
         32./9.*m33*m44*L7r*RS36e - 32./9.*m33*m44*L7r*RS16e - 64./9.*
         m33*m44*L6r*RS3461e - 32./9.*m33*m44*L6r*RS1463e + 16./9.*m33*
         m44*L6r*RC346e + 16./27.*m33*m44*L5r*RS36e*RSe63 + 8./27.*m33*
         m44*L5r*pow(RS36e,2) - 16./27.*m33*m44*L5r*RSe61*RS36e - 32./
         27.*m33*m44*L5r*RS16e*RS36e + 32./3.*m33*m44*L4r*RS3461e + 32./
         9.*m33*m44*L4r*RS1463e );

      M13p6x22LV +=  + AbV(m33,L) * (  - 32./9.*m33*m44*L4r*RSe6613 + 
         32./9.*m33*m44*L4r*RSe6633 + 32./27.*m33*m44*L4r*RS36e*RSe63
          + 16./27.*m33*m44*L4r*pow(RS36e,2) - 32./27.*m33*m44*L4r*
         RS34e*RSe63 + 8./27.*m33*m44*L4r*pow(RS34e,2) - 8./9.*m33*m44*
         L4r*RC346e - 32./27.*m33*m44*L4r*RSe61*RS36e + 32./27.*m33*m44
         *L4r*RSe61*RS34e - 64./27.*m33*m44*L4r*RS16e*RS36e - 32./27.*
         m33*m44*L4r*RS14e*RS34e - 32./9.*pow(m33,2)*L8r*RS3461e - 16./
         9.*pow(m33,2)*L8r*RS1463e - 16./9.*pow(m33,2)*L8r*RC346e + 8*
         pow(m33,2)*L7r + 28./9.*pow(m33,2)*L5r*RS3461e + 16./9.*pow(
         m33,2)*L5r*RS1463e - 16./9.*pow(m33,2)*L5r*RSe6613 + 16./9.*
         pow(m33,2)*L5r*RSe6633 + 8./9.*pow(m33,2)*L5r*RC346e - 2./3.*
         pow(m33,2)*L3r*RS3461e - 5./3.*pow(m33,2)*L3r*RC346e - 8./3.*
         pow(m33,2)*L0r*RS3461e - 2./3.*pow(m33,2)*L0r*RC346e + 4./9.*
         m11*L5r*RD346e - 5./3.*m11*L3r*RD346e - 2./3.*m11*L0r*RD346e
          - 32./9.*m11*m66*L7r*RS34e - 32./9.*m11*m66*L6r*RS3461e - 8./
         27.*m11*m66*L5r*RS14e*RS34e );

      M13p6x22LV +=  + AbV(m33,L) * ( 8./3.*m11*m66*L4r*RS3461e + 8./9.
         *m11*m66*L4r*RS1463e - 16./27.*m11*m66*L4r*RS16e*RS36e - 8./27.
         *m11*m66*L4r*RS14e*RS34e - 64./9.*m11*m44*L7r*RS36e - 64./9.*
         m11*m44*L6r*RS3461e - 16./27.*m11*m44*L5r*RS16e*RS36e + 16./3.
         *m11*m44*L4r*RS3461e + 16./9.*m11*m44*L4r*RS1463e - 32./27.*
         m11*m44*L4r*RS16e*RS36e - 16./27.*m11*m44*L4r*RS14e*RS34e - 16.
         /3.*m11*m33*L8r*RS3461e - 8./3.*m11*m33*L8r*RC346e + 32./3.*
         m11*m33*L7r + 32./9.*m11*m33*L5r*RS3461e + 8./9.*m11*m33*L5r*
         RS1463e + 4./3.*m11*m33*L5r*RC346e - 2./3.*m11*m33*L3r*RS3461e
          - 5./3.*m11*m33*L3r*RC346e - 8./3.*m11*m33*L0r*RS3461e - 2./3.
         *m11*m33*L0r*RC346e - 32./9.*pow(m11,2)*L8r*RS3461e + 4./3.*
         pow(m11,2)*L5r*RS3461e );

      M13p6x22LV +=  + AbV(m34,L) * ( 8*m33*m44*L8r - 4*m33*m44*L5r + 5
         *m33*m44*L3r + 2*m33*m44*L0r + 8*pow(m33,2)*L8r - 4*pow(m33,2)
         *L5r + 5*pow(m33,2)*L3r + 2*pow(m33,2)*L0r + 8*m11*m44*L8r - 4
         *m11*m44*L5r + 5*m11*m44*L3r + 2*m11*m44*L0r + 8*m11*m33*L8r
          - 4*m11*m33*L5r + 5*m11*m33*L3r + 2*m11*m33*L0r );

      M13p6x22LV +=  + AbV(m36,L) * ( 4*m33*m66*L8r - 2*m33*m66*L5r + 5.
         /2.*m33*m66*L3r + m33*m66*L0r + 4*pow(m33,2)*L8r - 2*pow(
         m33,2)*L5r + 5./2.*pow(m33,2)*L3r + pow(m33,2)*L0r + 4*m11*m66
         *L8r - 2*m11*m66*L5r + 5./2.*m11*m66*L3r + m11*m66*L0r + 4*m11
         *m33*L8r - 2*m11*m33*L5r + 5./2.*m11*m33*L3r + m11*m33*L0r );

      M13p6x22LV +=  + AbV(m44,L) * ( 24*m33*m44*L6r - 24*m33*m44*L4r
          + 6*m33*m44*L2r + 24*m33*m44*L1r + 24*m11*m44*L6r - 24*m11*
         m44*L4r + 6*m11*m44*L2r + 24*m11*m44*L1r );

      M13p6x22LV +=  + AbV(m46,L) * ( 16*m33*m66*L6r - 16*m33*m66*L4r
          + 4*m33*m66*L2r + 16*m33*m66*L1r + 16*m33*m44*L6r - 16*m33*
         m44*L4r + 4*m33*m44*L2r + 16*m33*m44*L1r + 16*m11*m66*L6r - 16
         *m11*m66*L4r + 4*m11*m66*L2r + 16*m11*m66*L1r + 16*m11*m44*L6r
          - 16*m11*m44*L4r + 4*m11*m44*L2r + 16*m11*m44*L1r );

      M13p6x22LV +=  + AbV(me2,L) * ( 32./27.*m66*me2*L5r*pow(RSe63,2)
          - 16./27.*m66*me2*L5r*RS34e*RSe63 - 64./27.*m66*me2*L5r*RSe61
         *RSe63 + 16./27.*m66*me2*L5r*RSe61*RS34e + 32./27.*m66*me2*L5r
         *pow(RSe61,2) + 16./27.*m66*me2*L5r*RS14e*RSe63 - 16./27.*m66*
         me2*L5r*RS14e*RSe61 - 32./9.*m66*me2*L4r*RSe6613 + 16./9.*m66*
         me2*L4r*pow(RSe63,2) + 16./9.*m66*me2*L4r*RSe6633 + 16./27.*
         m66*me2*L4r*RS36e*RSe63 - 16./27.*m66*me2*L4r*RS34e*RSe63 - 32.
         /9.*m66*me2*L4r*RSe61*RSe63 - 16./27.*m66*me2*L4r*RSe61*RS36e
          + 16./27.*m66*me2*L4r*RSe61*RS34e + 16./9.*m66*me2*L4r*pow(
         RSe61,2) + 16./9.*m66*me2*L4r*RSe6611 - 16./27.*m66*me2*L4r*
         RS16e*RSe63 + 16./27.*m66*me2*L4r*RS16e*RSe61 + 16./27.*m66*
         me2*L4r*RS14e*RSe63 - 16./27.*m66*me2*L4r*RS14e*RSe61 - 32./27.
         *pow(m66,2)*L8r*pow(RSe63,2) + 32./27.*pow(m66,2)*L8r*RS34e*
         RSe63 + 64./27.*pow(m66,2)*L8r*RSe61*RSe63 - 32./27.*pow(
         m66,2)*L8r*RSe61*RS34e - 32./27.*pow(m66,2)*L8r*pow(RSe61,2)
          - 32./27.*pow(m66,2)*L8r*RS14e*RSe63 );

      M13p6x22LV +=  + AbV(me2,L) * ( 32./27.*pow(m66,2)*L8r*RS14e*
         RSe61 - 32./27.*pow(m66,2)*L7r*pow(RSe63,2) + 32./27.*pow(
         m66,2)*L7r*RS34e*RSe63 + 64./27.*pow(m66,2)*L7r*RSe61*RSe63 - 
         32./27.*pow(m66,2)*L7r*RSe61*RS34e - 32./27.*pow(m66,2)*L7r*
         pow(RSe61,2) - 32./27.*pow(m66,2)*L7r*RS14e*RSe63 + 32./27.*
         pow(m66,2)*L7r*RS14e*RSe61 - 32./27.*pow(m66,2)*L6r*pow(
         RSe63,2) + 32./27.*pow(m66,2)*L6r*RS34e*RSe63 + 64./27.*pow(
         m66,2)*L6r*RSe61*RSe63 - 32./27.*pow(m66,2)*L6r*RSe61*RS34e - 
         32./27.*pow(m66,2)*L6r*pow(RSe61,2) - 32./27.*pow(m66,2)*L6r*
         RS14e*RSe63 + 32./27.*pow(m66,2)*L6r*RS14e*RSe61 + 16./27.*m44
         *me2*L5r*pow(RSe63,2) + 16./27.*m44*me2*L5r*RS36e*RSe63 - 32./
         27.*m44*me2*L5r*RSe61*RSe63 - 16./27.*m44*me2*L5r*RSe61*RS36e
          + 16./27.*m44*me2*L5r*pow(RSe61,2) - 16./27.*m44*me2*L5r*
         RS16e*RSe63 + 16./27.*m44*me2*L5r*RS16e*RSe61 - 64./9.*m44*me2
         *L4r*RSe6613 + 32./9.*m44*me2*L4r*pow(RSe63,2) + 32./9.*m44*
         me2*L4r*RSe6633 );

      M13p6x22LV +=  + AbV(me2,L) * ( 32./27.*m44*me2*L4r*RS36e*RSe63
          - 32./27.*m44*me2*L4r*RS34e*RSe63 - 64./9.*m44*me2*L4r*RSe61*
         RSe63 - 32./27.*m44*me2*L4r*RSe61*RS36e + 32./27.*m44*me2*L4r*
         RSe61*RS34e + 32./9.*m44*me2*L4r*pow(RSe61,2) + 32./9.*m44*me2
         *L4r*RSe6611 - 32./27.*m44*me2*L4r*RS16e*RSe63 + 32./27.*m44*
         me2*L4r*RS16e*RSe61 + 32./27.*m44*me2*L4r*RS14e*RSe63 - 32./27.
         *m44*me2*L4r*RS14e*RSe61 + 64./27.*m44*m66*L7r*pow(RSe63,2) + 
         64./27.*m44*m66*L7r*RS36e*RSe63 - 32./27.*m44*m66*L7r*RS34e*
         RSe63 - 128./27.*m44*m66*L7r*RSe61*RSe63 - 64./27.*m44*m66*L7r
         *RSe61*RS36e + 32./27.*m44*m66*L7r*RSe61*RS34e + 64./27.*m44*
         m66*L7r*pow(RSe61,2) - 64./27.*m44*m66*L7r*RS16e*RSe63 + 64./
         27.*m44*m66*L7r*RS16e*RSe61 + 32./27.*m44*m66*L7r*RS14e*RSe63
          - 32./27.*m44*m66*L7r*RS14e*RSe61 - 80./27.*m44*m66*L6r*pow(
         RSe63,2) - 32./27.*m44*m66*L6r*RS36e*RSe63 + 64./27.*m44*m66*
         L6r*RS34e*RSe63 + 160./27.*m44*m66*L6r*RSe61*RSe63 + 32./27.*
         m44*m66*L6r*RSe61*RS36e );

      M13p6x22LV +=  + AbV(me2,L) * (  - 64./27.*m44*m66*L6r*RSe61*
         RS34e - 80./27.*m44*m66*L6r*pow(RSe61,2) + 32./27.*m44*m66*L6r
         *RS16e*RSe63 - 32./27.*m44*m66*L6r*RS16e*RSe61 - 64./27.*m44*
         m66*L6r*RS14e*RSe63 + 64./27.*m44*m66*L6r*RS14e*RSe61 - 16./27.
         *pow(m44,2)*L8r*pow(RSe63,2) - 32./27.*pow(m44,2)*L8r*RS36e*
         RSe63 + 32./27.*pow(m44,2)*L8r*RSe61*RSe63 + 32./27.*pow(
         m44,2)*L8r*RSe61*RS36e - 16./27.*pow(m44,2)*L8r*pow(RSe61,2)
          + 32./27.*pow(m44,2)*L8r*RS16e*RSe63 - 32./27.*pow(m44,2)*L8r
         *RS16e*RSe61 - 32./27.*pow(m44,2)*L7r*pow(RSe63,2) - 64./27.*
         pow(m44,2)*L7r*RS36e*RSe63 + 64./27.*pow(m44,2)*L7r*RSe61*
         RSe63 + 64./27.*pow(m44,2)*L7r*RSe61*RS36e - 32./27.*pow(
         m44,2)*L7r*pow(RSe61,2) + 64./27.*pow(m44,2)*L7r*RS16e*RSe63
          - 64./27.*pow(m44,2)*L7r*RS16e*RSe61 - 32./27.*pow(m44,2)*L6r
         *pow(RSe63,2) - 64./27.*pow(m44,2)*L6r*RS36e*RSe63 + 64./27.*
         pow(m44,2)*L6r*RSe61*RSe63 + 64./27.*pow(m44,2)*L6r*RSe61*
         RS36e );

      M13p6x22LV +=  + AbV(me2,L) * (  - 32./27.*pow(m44,2)*L6r*pow(
         RSe61,2) + 64./27.*pow(m44,2)*L6r*RS16e*RSe63 - 64./27.*pow(
         m44,2)*L6r*RS16e*RSe61 - 16./9.*m33*me2*L5r*RSe6613 - 8./9.*
         m33*me2*L5r*RSe6611 - 4*m33*me2*L4r + 4./3.*m33*me2*L3r*
         RSe6613 + 10./3.*m33*me2*L3r*RSe6633 + 10./3.*m33*me2*L3r*
         RSe6611 + 2*m33*me2*L2r + 8*m33*me2*L1r + 16./3.*m33*me2*L0r*
         RSe6613 + 4./3.*m33*me2*L0r*RSe6633 + 4./3.*m33*me2*L0r*
         RSe6611 + 32./9.*m33*m66*L7r*RSe63 + 64./9.*m33*m66*L7r*RSe61
          + 16./3.*m33*m66*L6r + 64./9.*m33*m66*L6r*RSe6613 - 16./9.*
         m33*m66*L6r*RSe6633 - 16./27.*m33*m66*L5r*pow(RSe63,2) + 16./
         27.*m33*m66*L5r*RS34e*RSe63 - 32./27.*m33*m66*L5r*RSe61*RSe63
          + 16./27.*m33*m66*L5r*RSe61*RS34e + 16./27.*m33*m66*L5r*RS14e
         *RSe63 - 8./3.*m33*m66*L4r - 16./3.*m33*m66*L4r*RSe6613 - 8./9.
         *m33*m66*L4r*pow(RSe63,2) - 8./3.*m33*m66*L4r*RSe6633 - 16./27.
         *m33*m66*L4r*RS36e*RSe63 + 16./27.*m33*m66*L4r*RS34e*RSe63 - 
         16./9.*m33*m66*L4r*RSe61*RSe63 );

      M13p6x22LV +=  + AbV(me2,L) * (  - 16./27.*m33*m66*L4r*RSe61*
         RS36e + 16./27.*m33*m66*L4r*RSe61*RS34e - 16./27.*m33*m66*L4r*
         RS16e*RSe63 + 16./27.*m33*m66*L4r*RS14e*RSe63 - 32./9.*m33*m44
         *L7r*RSe63 - 64./9.*m33*m44*L7r*RSe61 + 8./3.*m33*m44*L6r + 
         128./9.*m33*m44*L6r*RSe6613 - 32./9.*m33*m44*L6r*RSe6633 - 8./
         27.*m33*m44*L5r*pow(RSe63,2) - 16./27.*m33*m44*L5r*RS36e*RSe63
          - 16./27.*m33*m44*L5r*RSe61*RSe63 - 16./27.*m33*m44*L5r*RSe61
         *RS36e - 16./27.*m33*m44*L5r*RS16e*RSe63 - 4./3.*m33*m44*L4r
          - 32./3.*m33*m44*L4r*RSe6613 - 16./9.*m33*m44*L4r*pow(
         RSe63,2) - 16./3.*m33*m44*L4r*RSe6633 - 32./27.*m33*m44*L4r*
         RS36e*RSe63 + 32./27.*m33*m44*L4r*RS34e*RSe63 - 32./9.*m33*m44
         *L4r*RSe61*RSe63 - 32./27.*m33*m44*L4r*RSe61*RS36e + 32./27.*
         m33*m44*L4r*RSe61*RS34e - 32./27.*m33*m44*L4r*RS16e*RSe63 + 32.
         /27.*m33*m44*L4r*RS14e*RSe63 + 64./9.*pow(m33,2)*L8r*RSe6613
          + 32./9.*pow(m33,2)*L8r*RSe6633 - 8./3.*pow(m33,2)*L5r*
         RSe6613 );

      M13p6x22LV +=  + AbV(me2,L) * (  - 32./9.*pow(m33,2)*L5r*RSe6633
          - 16./9.*m11*me2*L5r*RSe6613 - 8./9.*m11*me2*L5r*RSe6633 - 4*
         m11*me2*L4r + 4./3.*m11*me2*L3r*RSe6613 + 10./3.*m11*me2*L3r*
         RSe6633 + 10./3.*m11*me2*L3r*RSe6611 + 2*m11*me2*L2r + 8*m11*
         me2*L1r + 16./3.*m11*me2*L0r*RSe6613 + 4./3.*m11*me2*L0r*
         RSe6633 + 4./3.*m11*me2*L0r*RSe6611 + 64./9.*m11*m66*L7r*RSe63
          + 32./9.*m11*m66*L7r*RSe61 + 16./3.*m11*m66*L6r + 64./9.*m11*
         m66*L6r*RSe6613 - 16./9.*m11*m66*L6r*RSe6611 - 32./27.*m11*m66
         *L5r*RSe61*RSe63 + 16./27.*m11*m66*L5r*RSe61*RS34e - 16./27.*
         m11*m66*L5r*pow(RSe61,2) + 16./27.*m11*m66*L5r*RS14e*RSe63 + 
         16./27.*m11*m66*L5r*RS14e*RSe61 - 8./3.*m11*m66*L4r - 16./3.*
         m11*m66*L4r*RSe6613 - 16./9.*m11*m66*L4r*RSe61*RSe63 - 16./27.
         *m11*m66*L4r*RSe61*RS36e + 16./27.*m11*m66*L4r*RSe61*RS34e - 8.
         /9.*m11*m66*L4r*pow(RSe61,2) - 8./3.*m11*m66*L4r*RSe6611 - 16./
         27.*m11*m66*L4r*RS16e*RSe63 - 16./27.*m11*m66*L4r*RS16e*RSe61
          + 16./27.*m11*m66*L4r*RS14e*RSe63 );

      M13p6x22LV +=  + AbV(me2,L) * ( 16./27.*m11*m66*L4r*RS14e*RSe61
          - 64./9.*m11*m44*L7r*RSe63 - 32./9.*m11*m44*L7r*RSe61 + 8./3.
         *m11*m44*L6r + 128./9.*m11*m44*L6r*RSe6613 - 32./9.*m11*m44*
         L6r*RSe6611 - 16./27.*m11*m44*L5r*RSe61*RSe63 - 16./27.*m11*
         m44*L5r*RSe61*RS36e - 8./27.*m11*m44*L5r*pow(RSe61,2) - 16./27.
         *m11*m44*L5r*RS16e*RSe63 - 16./27.*m11*m44*L5r*RS16e*RSe61 - 4.
         /3.*m11*m44*L4r - 32./3.*m11*m44*L4r*RSe6613 - 32./9.*m11*m44*
         L4r*RSe61*RSe63 - 32./27.*m11*m44*L4r*RSe61*RS36e + 32./27.*
         m11*m44*L4r*RSe61*RS34e - 16./9.*m11*m44*L4r*pow(RSe61,2) - 16.
         /3.*m11*m44*L4r*RSe6611 - 32./27.*m11*m44*L4r*RS16e*RSe63 - 32.
         /27.*m11*m44*L4r*RS16e*RSe61 + 32./27.*m11*m44*L4r*RS14e*RSe63
          + 32./27.*m11*m44*L4r*RS14e*RSe61 + 32./3.*m11*m33*L8r*
         RSe6613 + 16./3.*m11*m33*L8r*RSe6633 + 16./3.*m11*m33*L8r*
         RSe6611 - 16./3.*m11*m33*L5r*RSe6613 - 16./9.*m11*m33*L5r*
         RSe6633 - 16./9.*m11*m33*L5r*RSe6611 + 64./9.*pow(m11,2)*L8r*
         RSe6613 );

      M13p6x22LV +=  + AbV(me2,L) * ( 32./9.*pow(m11,2)*L8r*RSe6611 - 8.
         /3.*pow(m11,2)*L5r*RSe6613 - 32./9.*pow(m11,2)*L5r*RSe6611 );

      M13p6x22LV +=  + BbV(m11,m33,L) * ( 16./27.*m33*pow(m66,2)*L8r*
         RS14e*RS34e + 16./27.*m33*pow(m66,2)*L7r*RS14e*RS34e + 16./27.
         *m33*pow(m66,2)*L6r*RS14e*RS34e + 32./27.*m33*m44*m66*L7r*
         RS16e*RS34e + 32./27.*m33*m44*m66*L7r*RS14e*RS36e + 32./27.*
         m33*m44*m66*L6r*RS16e*RS36e + 32./27.*m33*m44*m66*L6r*RS14e*
         RS34e + 32./27.*m33*pow(m44,2)*L8r*RS16e*RS36e + 64./27.*m33*
         pow(m44,2)*L7r*RS16e*RS36e + 64./27.*m33*pow(m44,2)*L6r*RS16e*
         RS36e - 16./9.*pow(m33,2)*m66*L7r*RS14e - 16./9.*pow(m33,2)*
         m66*L6r*RS1463e - 32./9.*pow(m33,2)*m44*L7r*RS16e - 32./9.*
         pow(m33,2)*m44*L6r*RS1463e - 16./9.*pow(m33,3)*L8r*RS1463e + 
         32./27.*m11*pow(m66,2)*L8r*RS14e*RS34e + 32./27.*m11*pow(
         m66,2)*L7r*RS14e*RS34e + 32./27.*m11*pow(m66,2)*L6r*RS14e*
         RS34e + 64./27.*m11*m44*m66*L7r*RS16e*RS34e + 64./27.*m11*m44*
         m66*L7r*RS14e*RS36e + 64./27.*m11*m44*m66*L6r*RS16e*RS36e + 64.
         /27.*m11*m44*m66*L6r*RS14e*RS34e + 64./27.*m11*pow(m44,2)*L8r*
         RS16e*RS36e );

      M13p6x22LV +=  + BbV(m11,m33,L) * ( 128./27.*m11*pow(m44,2)*L7r*
         RS16e*RS36e + 128./27.*m11*pow(m44,2)*L6r*RS16e*RS36e - 16./9.
         *m11*m33*m66*L7r*RS34e - 32./9.*m11*m33*m66*L7r*RS14e - 16./9.
         *m11*m33*m66*L6r*RS3461e - 32./9.*m11*m33*m66*L6r*RS1463e - 16.
         /27.*m11*m33*m66*L5r*RS14e*RS34e + 16./9.*m11*m33*m66*L4r*
         RS3461e + 16./9.*m11*m33*m66*L4r*RS1463e - 32./27.*m11*m33*m66
         *L4r*RS16e*RS36e - 16./27.*m11*m33*m66*L4r*RS14e*RS34e - 32./9.
         *m11*m33*m44*L7r*RS36e - 64./9.*m11*m33*m44*L7r*RS16e - 32./9.
         *m11*m33*m44*L6r*RS3461e - 64./9.*m11*m33*m44*L6r*RS1463e - 32.
         /27.*m11*m33*m44*L5r*RS16e*RS36e + 32./9.*m11*m33*m44*L4r*
         RS3461e + 32./9.*m11*m33*m44*L4r*RS1463e - 64./27.*m11*m33*m44
         *L4r*RS16e*RS36e - 32./27.*m11*m33*m44*L4r*RS14e*RS34e - 32./9.
         *m11*pow(m33,2)*L8r*RS1463e + 16./3.*m11*pow(m33,2)*L7r + 16./
         9.*m11*pow(m33,2)*L5r*RS1463e - 32./9.*pow(m11,2)*m66*L7r*
         RS34e - 32./9.*pow(m11,2)*m66*L6r*RS3461e - 8./27.*pow(m11,2)*
         m66*L5r*RS14e*RS34e );

      M13p6x22LV +=  + BbV(m11,m33,L) * ( 8./9.*pow(m11,2)*m66*L4r*
         RS3461e + 8./9.*pow(m11,2)*m66*L4r*RS1463e - 16./27.*pow(
         m11,2)*m66*L4r*RS16e*RS36e - 8./27.*pow(m11,2)*m66*L4r*RS14e*
         RS34e - 64./9.*pow(m11,2)*m44*L7r*RS36e - 64./9.*pow(m11,2)*
         m44*L6r*RS3461e - 16./27.*pow(m11,2)*m44*L5r*RS16e*RS36e + 16./
         9.*pow(m11,2)*m44*L4r*RS3461e + 16./9.*pow(m11,2)*m44*L4r*
         RS1463e - 32./27.*pow(m11,2)*m44*L4r*RS16e*RS36e - 16./27.*
         pow(m11,2)*m44*L4r*RS14e*RS34e - 16./9.*pow(m11,2)*m33*L8r*
         RS3461e + 32./3.*pow(m11,2)*m33*L7r + 16./9.*pow(m11,2)*m33*
         L5r*RS3461e + 8./9.*pow(m11,2)*m33*L5r*RS1463e - 32./9.*pow(
         m11,3)*L8r*RS3461e + 8./9.*pow(m11,3)*L5r*RS3461e );

      M13p6x22LV +=  + BbV(m11,me2,L) * (  - 32./27.*m33*pow(m66,2)*L8r
         *RS14e*RSe63 - 32./27.*m33*pow(m66,2)*L7r*RS14e*RSe63 - 32./27.
         *m33*pow(m66,2)*L6r*RS14e*RSe63 - 64./27.*m33*m44*m66*L7r*
         RS16e*RSe63 + 32./27.*m33*m44*m66*L7r*RS14e*RSe63 + 32./27.*
         m33*m44*m66*L6r*RS16e*RSe63 - 64./27.*m33*m44*m66*L6r*RS14e*
         RSe63 + 32./27.*m33*pow(m44,2)*L8r*RS16e*RSe63 + 64./27.*m33*
         pow(m44,2)*L7r*RS16e*RSe63 + 64./27.*m33*pow(m44,2)*L6r*RS16e*
         RSe63 + 16./27.*m11*m66*me2*L5r*RS14e*RSe63 - 16./27.*m11*m66*
         me2*L5r*RS14e*RSe61 - 16./9.*m11*m66*me2*L4r*RSe6613 + 16./9.*
         m11*m66*me2*L4r*RSe6611 - 16./27.*m11*m66*me2*L4r*RS16e*RSe63
          + 16./27.*m11*m66*me2*L4r*RS16e*RSe61 + 16./27.*m11*m66*me2*
         L4r*RS14e*RSe63 - 16./27.*m11*m66*me2*L4r*RS14e*RSe61 - 64./27.
         *m11*pow(m66,2)*L8r*RS14e*RSe63 - 64./27.*m11*pow(m66,2)*L7r*
         RS14e*RSe63 - 64./27.*m11*pow(m66,2)*L6r*RS14e*RSe63 - 16./27.
         *m11*m44*me2*L5r*RS16e*RSe63 + 16./27.*m11*m44*me2*L5r*RS16e*
         RSe61 );

      M13p6x22LV +=  + BbV(m11,me2,L) * (  - 32./9.*m11*m44*me2*L4r*
         RSe6613 + 32./9.*m11*m44*me2*L4r*RSe6611 - 32./27.*m11*m44*me2
         *L4r*RS16e*RSe63 + 32./27.*m11*m44*me2*L4r*RS16e*RSe61 + 32./
         27.*m11*m44*me2*L4r*RS14e*RSe63 - 32./27.*m11*m44*me2*L4r*
         RS14e*RSe61 - 128./27.*m11*m44*m66*L7r*RS16e*RSe63 + 64./27.*
         m11*m44*m66*L7r*RS14e*RSe63 + 64./27.*m11*m44*m66*L6r*RS16e*
         RSe63 - 128./27.*m11*m44*m66*L6r*RS14e*RSe63 + 64./27.*m11*
         pow(m44,2)*L8r*RS16e*RSe63 + 128./27.*m11*pow(m44,2)*L7r*RS16e
         *RSe63 + 128./27.*m11*pow(m44,2)*L6r*RS16e*RSe63 + 32./9.*m11*
         m33*m66*L7r*RSe63 + 32./9.*m11*m33*m66*L6r*RSe6613 + 16./27.*
         m11*m33*m66*L5r*RS14e*RSe63 - 16./9.*m11*m33*m66*L4r*RSe6613
          - 16./27.*m11*m33*m66*L4r*RS16e*RSe63 + 16./27.*m11*m33*m66*
         L4r*RS14e*RSe63 - 32./9.*m11*m33*m44*L7r*RSe63 + 64./9.*m11*
         m33*m44*L6r*RSe6613 - 16./27.*m11*m33*m44*L5r*RS16e*RSe63 - 32.
         /9.*m11*m33*m44*L4r*RSe6613 - 32./27.*m11*m33*m44*L4r*RS16e*
         RSe63 );

      M13p6x22LV +=  + BbV(m11,me2,L) * ( 32./27.*m11*m33*m44*L4r*RS14e
         *RSe63 - 16./9.*pow(m11,2)*me2*L5r*RSe6613 + 16./9.*pow(m11,2)
         *me2*L5r*RSe6611 + 64./9.*pow(m11,2)*m66*L7r*RSe63 + 64./9.*
         pow(m11,2)*m66*L6r*RSe6613 + 16./27.*pow(m11,2)*m66*L5r*RS14e*
         RSe63 + 16./27.*pow(m11,2)*m66*L5r*RS14e*RSe61 - 16./9.*pow(
         m11,2)*m66*L4r*RSe6613 - 16./9.*pow(m11,2)*m66*L4r*RSe6611 - 
         16./27.*pow(m11,2)*m66*L4r*RS16e*RSe63 - 16./27.*pow(m11,2)*
         m66*L4r*RS16e*RSe61 + 16./27.*pow(m11,2)*m66*L4r*RS14e*RSe63
          + 16./27.*pow(m11,2)*m66*L4r*RS14e*RSe61 - 64./9.*pow(m11,2)*
         m44*L7r*RSe63 + 128./9.*pow(m11,2)*m44*L6r*RSe6613 - 16./27.*
         pow(m11,2)*m44*L5r*RS16e*RSe63 - 16./27.*pow(m11,2)*m44*L5r*
         RS16e*RSe61 - 32./9.*pow(m11,2)*m44*L4r*RSe6613 - 32./9.*pow(
         m11,2)*m44*L4r*RSe6611 - 32./27.*pow(m11,2)*m44*L4r*RS16e*
         RSe63 - 32./27.*pow(m11,2)*m44*L4r*RS16e*RSe61 + 32./27.*pow(
         m11,2)*m44*L4r*RS14e*RSe63 + 32./27.*pow(m11,2)*m44*L4r*RS14e*
         RSe61 );

      M13p6x22LV +=  + BbV(m11,me2,L) * ( 32./9.*pow(m11,2)*m33*L8r*
         RSe6613 - 16./9.*pow(m11,2)*m33*L5r*RSe6613 + 64./9.*pow(
         m11,3)*L8r*RSe6613 - 16./9.*pow(m11,3)*L5r*RSe6613 - 16./9.*
         pow(m11,3)*L5r*RSe6611 );

      M13p6x22LV +=  + BbV(m11,L) * ( 8./9.*m11*m66*L6r*RD146e - 4./9.*
         m11*m66*L4r*RD146e + 16./9.*m11*m44*L6r*RD146e - 8./9.*m11*m44
         *L4r*RD146e - 8./3.*m11*m33*L8r*RD146e + 4./3.*m11*m33*L5r*
         RD146e - 5./3.*m11*m33*L3r*RD146e - 2./3.*m11*m33*L0r*RD146e
          - 16./9.*m11*m33*m66*L6r*RS1463e + 8./9.*m11*m33*m66*L4r*
         RS1463e - 32./9.*m11*m33*m44*L6r*RS1463e + 16./9.*m11*m33*m44*
         L4r*RS1463e - 16./9.*pow(m11,2)*L8r*RD146e + 8./9.*pow(m11,2)*
         L5r*RD146e - 5./3.*pow(m11,2)*L3r*RD146e - 2./3.*pow(m11,2)*
         L0r*RD146e - 32./9.*pow(m11,2)*m66*L6r*RS1463e + 16./9.*pow(
         m11,2)*m66*L4r*RS1463e - 64./9.*pow(m11,2)*m44*L6r*RS1463e + 
         32./9.*pow(m11,2)*m44*L4r*RS1463e - 16./9.*pow(m11,2)*m33*L8r*
         RS1463e + 8./9.*pow(m11,2)*m33*L5r*RS1463e - 32./9.*pow(m11,3)
         *L8r*RS1463e + 16./9.*pow(m11,3)*L5r*RS1463e );

      M13p6x22LV +=  + BbV(m33,me2,L) * (  - 16./27.*m33*m66*me2*L5r*
         RS34e*RSe63 + 16./27.*m33*m66*me2*L5r*RSe61*RS34e - 16./9.*m33
         *m66*me2*L4r*RSe6613 + 16./9.*m33*m66*me2*L4r*RSe6633 + 16./27.
         *m33*m66*me2*L4r*RS36e*RSe63 - 16./27.*m33*m66*me2*L4r*RS34e*
         RSe63 - 16./27.*m33*m66*me2*L4r*RSe61*RS36e + 16./27.*m33*m66*
         me2*L4r*RSe61*RS34e - 64./27.*m33*pow(m66,2)*L8r*RSe61*RS34e
          - 64./27.*m33*pow(m66,2)*L7r*RSe61*RS34e - 64./27.*m33*pow(
         m66,2)*L6r*RSe61*RS34e + 16./27.*m33*m44*me2*L5r*RS36e*RSe63
          - 16./27.*m33*m44*me2*L5r*RSe61*RS36e - 32./9.*m33*m44*me2*
         L4r*RSe6613 + 32./9.*m33*m44*me2*L4r*RSe6633 + 32./27.*m33*m44
         *me2*L4r*RS36e*RSe63 - 32./27.*m33*m44*me2*L4r*RS34e*RSe63 - 
         32./27.*m33*m44*me2*L4r*RSe61*RS36e + 32./27.*m33*m44*me2*L4r*
         RSe61*RS34e - 128./27.*m33*m44*m66*L7r*RSe61*RS36e + 64./27.*
         m33*m44*m66*L7r*RSe61*RS34e + 64./27.*m33*m44*m66*L6r*RSe61*
         RS36e - 128./27.*m33*m44*m66*L6r*RSe61*RS34e + 64./27.*m33*
         pow(m44,2)*L8r*RSe61*RS36e );

      M13p6x22LV +=  + BbV(m33,me2,L) * ( 128./27.*m33*pow(m44,2)*L7r*
         RSe61*RS36e + 128./27.*m33*pow(m44,2)*L6r*RSe61*RS36e - 16./9.
         *pow(m33,2)*me2*L5r*RSe6613 + 16./9.*pow(m33,2)*me2*L5r*
         RSe6633 + 64./9.*pow(m33,2)*m66*L7r*RSe61 + 64./9.*pow(m33,2)*
         m66*L6r*RSe6613 + 16./27.*pow(m33,2)*m66*L5r*RS34e*RSe63 + 16./
         27.*pow(m33,2)*m66*L5r*RSe61*RS34e - 16./9.*pow(m33,2)*m66*L4r
         *RSe6613 - 16./9.*pow(m33,2)*m66*L4r*RSe6633 - 16./27.*pow(
         m33,2)*m66*L4r*RS36e*RSe63 + 16./27.*pow(m33,2)*m66*L4r*RS34e*
         RSe63 - 16./27.*pow(m33,2)*m66*L4r*RSe61*RS36e + 16./27.*pow(
         m33,2)*m66*L4r*RSe61*RS34e - 64./9.*pow(m33,2)*m44*L7r*RSe61
          + 128./9.*pow(m33,2)*m44*L6r*RSe6613 - 16./27.*pow(m33,2)*m44
         *L5r*RS36e*RSe63 - 16./27.*pow(m33,2)*m44*L5r*RSe61*RS36e - 32.
         /9.*pow(m33,2)*m44*L4r*RSe6613 - 32./9.*pow(m33,2)*m44*L4r*
         RSe6633 - 32./27.*pow(m33,2)*m44*L4r*RS36e*RSe63 + 32./27.*
         pow(m33,2)*m44*L4r*RS34e*RSe63 - 32./27.*pow(m33,2)*m44*L4r*
         RSe61*RS36e );

      M13p6x22LV +=  + BbV(m33,me2,L) * ( 32./27.*pow(m33,2)*m44*L4r*
         RSe61*RS34e + 64./9.*pow(m33,3)*L8r*RSe6613 - 16./9.*pow(
         m33,3)*L5r*RSe6613 - 16./9.*pow(m33,3)*L5r*RSe6633 - 32./27.*
         m11*pow(m66,2)*L8r*RSe61*RS34e - 32./27.*m11*pow(m66,2)*L7r*
         RSe61*RS34e - 32./27.*m11*pow(m66,2)*L6r*RSe61*RS34e - 64./27.
         *m11*m44*m66*L7r*RSe61*RS36e + 32./27.*m11*m44*m66*L7r*RSe61*
         RS34e + 32./27.*m11*m44*m66*L6r*RSe61*RS36e - 64./27.*m11*m44*
         m66*L6r*RSe61*RS34e + 32./27.*m11*pow(m44,2)*L8r*RSe61*RS36e
          + 64./27.*m11*pow(m44,2)*L7r*RSe61*RS36e + 64./27.*m11*pow(
         m44,2)*L6r*RSe61*RS36e + 32./9.*m11*m33*m66*L7r*RSe61 + 32./9.
         *m11*m33*m66*L6r*RSe6613 + 16./27.*m11*m33*m66*L5r*RSe61*RS34e
          - 16./9.*m11*m33*m66*L4r*RSe6613 - 16./27.*m11*m33*m66*L4r*
         RSe61*RS36e + 16./27.*m11*m33*m66*L4r*RSe61*RS34e - 32./9.*m11
         *m33*m44*L7r*RSe61 + 64./9.*m11*m33*m44*L6r*RSe6613 - 16./27.*
         m11*m33*m44*L5r*RSe61*RS36e - 32./9.*m11*m33*m44*L4r*RSe6613
          - 32./27.*m11*m33*m44*L4r*RSe61*RS36e );

      M13p6x22LV +=  + BbV(m33,me2,L) * ( 32./27.*m11*m33*m44*L4r*RSe61
         *RS34e + 32./9.*m11*pow(m33,2)*L8r*RSe6613 - 16./9.*m11*pow(
         m33,2)*L5r*RSe6613 );

      M13p6x22LV +=  + BbV(m33,L) * ( 8./9.*m33*m66*L6r*RD346e - 4./9.*
         m33*m66*L4r*RD346e + 16./9.*m33*m44*L6r*RD346e - 8./9.*m33*m44
         *L4r*RD346e - 16./9.*pow(m33,2)*L8r*RD346e + 8./9.*pow(m33,2)*
         L5r*RD346e - 5./3.*pow(m33,2)*L3r*RD346e - 2./3.*pow(m33,2)*
         L0r*RD346e - 32./9.*pow(m33,2)*m66*L6r*RS3461e + 16./9.*pow(
         m33,2)*m66*L4r*RS3461e - 64./9.*pow(m33,2)*m44*L6r*RS3461e + 
         32./9.*pow(m33,2)*m44*L4r*RS3461e - 32./9.*pow(m33,3)*L8r*
         RS3461e + 16./9.*pow(m33,3)*L5r*RS3461e - 8./3.*m11*m33*L8r*
         RD346e + 4./3.*m11*m33*L5r*RD346e - 5./3.*m11*m33*L3r*RD346e
          - 2./3.*m11*m33*L0r*RD346e - 16./9.*m11*m33*m66*L6r*RS3461e
          + 8./9.*m11*m33*m66*L4r*RS3461e - 32./9.*m11*m33*m44*L6r*
         RS3461e + 16./9.*m11*m33*m44*L4r*RS3461e - 16./9.*m11*pow(
         m33,2)*L8r*RS3461e + 8./9.*m11*pow(m33,2)*L5r*RS3461e );

      M13p6x22LV +=  + BbV(me2,L) * ( 16./27.*m66*pow(me2,2)*L5r*pow(
         RSe63,2) - 32./27.*m66*pow(me2,2)*L5r*RSe61*RSe63 + 16./27.*
         m66*pow(me2,2)*L5r*pow(RSe61,2) + 8./9.*m66*pow(me2,2)*L4r*
         pow(RSe63,2) - 16./9.*m66*pow(me2,2)*L4r*RSe61*RSe63 + 8./9.*
         m66*pow(me2,2)*L4r*pow(RSe61,2) - 32./27.*pow(m66,2)*me2*L8r*
         pow(RSe63,2) + 64./27.*pow(m66,2)*me2*L8r*RSe61*RSe63 - 32./27.
         *pow(m66,2)*me2*L8r*pow(RSe61,2) - 32./27.*pow(m66,2)*me2*L7r*
         pow(RSe63,2) + 64./27.*pow(m66,2)*me2*L7r*RSe61*RSe63 - 32./27.
         *pow(m66,2)*me2*L7r*pow(RSe61,2) - 32./27.*pow(m66,2)*me2*L6r*
         pow(RSe63,2) + 64./27.*pow(m66,2)*me2*L6r*RSe61*RSe63 - 32./27.
         *pow(m66,2)*me2*L6r*pow(RSe61,2) + 8./27.*m44*pow(me2,2)*L5r*
         pow(RSe63,2) - 16./27.*m44*pow(me2,2)*L5r*RSe61*RSe63 + 8./27.
         *m44*pow(me2,2)*L5r*pow(RSe61,2) + 16./9.*m44*pow(me2,2)*L4r*
         pow(RSe63,2) - 32./9.*m44*pow(me2,2)*L4r*RSe61*RSe63 + 16./9.*
         m44*pow(me2,2)*L4r*pow(RSe61,2) + 64./27.*m44*m66*me2*L7r*pow(
         RSe63,2) );

      M13p6x22LV +=  + BbV(me2,L) * (  - 128./27.*m44*m66*me2*L7r*RSe61
         *RSe63 + 64./27.*m44*m66*me2*L7r*pow(RSe61,2) - 80./27.*m44*
         m66*me2*L6r*pow(RSe63,2) + 160./27.*m44*m66*me2*L6r*RSe61*
         RSe63 - 80./27.*m44*m66*me2*L6r*pow(RSe61,2) - 16./27.*pow(
         m44,2)*me2*L8r*pow(RSe63,2) + 32./27.*pow(m44,2)*me2*L8r*RSe61
         *RSe63 - 16./27.*pow(m44,2)*me2*L8r*pow(RSe61,2) - 32./27.*
         pow(m44,2)*me2*L7r*pow(RSe63,2) + 64./27.*pow(m44,2)*me2*L7r*
         RSe61*RSe63 - 32./27.*pow(m44,2)*me2*L7r*pow(RSe61,2) - 32./27.
         *pow(m44,2)*me2*L6r*pow(RSe63,2) + 64./27.*pow(m44,2)*me2*L6r*
         RSe61*RSe63 - 32./27.*pow(m44,2)*me2*L6r*pow(RSe61,2) - 16./27.
         *m33*m66*me2*L5r*pow(RSe63,2) - 32./27.*m33*m66*me2*L5r*RSe61*
         RSe63 - 8./9.*m33*m66*me2*L4r*pow(RSe63,2) - 16./9.*m33*m66*
         me2*L4r*RSe61*RSe63 + 32./27.*m33*pow(m66,2)*L8r*pow(RSe63,2)
          + 64./27.*m33*pow(m66,2)*L8r*RSe61*RSe63 + 32./27.*m33*pow(
         m66,2)*L7r*pow(RSe63,2) + 64./27.*m33*pow(m66,2)*L7r*RSe61*
         RSe63 );

      M13p6x22LV +=  + BbV(me2,L) * ( 32./27.*m33*pow(m66,2)*L6r*pow(
         RSe63,2) + 64./27.*m33*pow(m66,2)*L6r*RSe61*RSe63 - 8./27.*m33
         *m44*me2*L5r*pow(RSe63,2) - 16./27.*m33*m44*me2*L5r*RSe61*
         RSe63 - 16./9.*m33*m44*me2*L4r*pow(RSe63,2) - 32./9.*m33*m44*
         me2*L4r*RSe61*RSe63 - 64./27.*m33*m44*m66*L7r*pow(RSe63,2) - 
         128./27.*m33*m44*m66*L7r*RSe61*RSe63 + 80./27.*m33*m44*m66*L6r
         *pow(RSe63,2) + 160./27.*m33*m44*m66*L6r*RSe61*RSe63 + 16./27.
         *m33*pow(m44,2)*L8r*pow(RSe63,2) + 32./27.*m33*pow(m44,2)*L8r*
         RSe61*RSe63 + 32./27.*m33*pow(m44,2)*L7r*pow(RSe63,2) + 64./27.
         *m33*pow(m44,2)*L7r*RSe61*RSe63 + 32./27.*m33*pow(m44,2)*L6r*
         pow(RSe63,2) + 64./27.*m33*pow(m44,2)*L6r*RSe61*RSe63 - 32./27.
         *m11*m66*me2*L5r*RSe61*RSe63 - 16./27.*m11*m66*me2*L5r*pow(
         RSe61,2) - 16./9.*m11*m66*me2*L4r*RSe61*RSe63 - 8./9.*m11*m66*
         me2*L4r*pow(RSe61,2) + 64./27.*m11*pow(m66,2)*L8r*RSe61*RSe63
          + 32./27.*m11*pow(m66,2)*L8r*pow(RSe61,2) + 64./27.*m11*pow(
         m66,2)*L7r*RSe61*RSe63 );

      M13p6x22LV +=  + BbV(me2,L) * ( 32./27.*m11*pow(m66,2)*L7r*pow(
         RSe61,2) + 64./27.*m11*pow(m66,2)*L6r*RSe61*RSe63 + 32./27.*
         m11*pow(m66,2)*L6r*pow(RSe61,2) - 16./27.*m11*m44*me2*L5r*
         RSe61*RSe63 - 8./27.*m11*m44*me2*L5r*pow(RSe61,2) - 32./9.*m11
         *m44*me2*L4r*RSe61*RSe63 - 16./9.*m11*m44*me2*L4r*pow(RSe61,2)
          - 128./27.*m11*m44*m66*L7r*RSe61*RSe63 - 64./27.*m11*m44*m66*
         L7r*pow(RSe61,2) + 160./27.*m11*m44*m66*L6r*RSe61*RSe63 + 80./
         27.*m11*m44*m66*L6r*pow(RSe61,2) + 32./27.*m11*pow(m44,2)*L8r*
         RSe61*RSe63 + 16./27.*m11*pow(m44,2)*L8r*pow(RSe61,2) + 64./27.
         *m11*pow(m44,2)*L7r*RSe61*RSe63 + 32./27.*m11*pow(m44,2)*L7r*
         pow(RSe61,2) + 64./27.*m11*pow(m44,2)*L6r*RSe61*RSe63 + 32./27.
         *m11*pow(m44,2)*L6r*pow(RSe61,2) );

      M13p6x22LV +=  + A23bV(m11,L) * ( 2*m33*L3r*RS1463e + m33*L3r*
         RC146e + 2*m33*L0r*RC146e + 2*m11*L3r*RS1463e + m11*L3r*RC146e
          + 2*m11*L0r*RC146e );

      M13p6x22LV +=  + A23bV(m13,L) * (  - 6*m33*L2r - 12*m33*L1r - 6*
         m11*L2r - 12*m11*L1r );

      M13p6x22LV +=  + A23bV(m14,L) * (  - 6*m33*L3r - 12*m33*L0r - 6*
         m11*L3r - 12*m11*L0r );

      M13p6x22LV +=  + A23bV(m16,L) * (  - 3*m33*L3r - 6*m33*L0r - 3*
         m11*L3r - 6*m11*L0r );

      M13p6x22LV +=  + A23bV(m33,L) * ( 2*m33*L3r*RS3461e + m33*L3r*
         RC346e + 2*m33*L0r*RC346e + 2*m11*L3r*RS3461e + m11*L3r*RC346e
          + 2*m11*L0r*RC346e );

      M13p6x22LV +=  + A23bV(m34,L) * (  - 6*m33*L3r - 12*m33*L0r - 6*
         m11*L3r - 12*m11*L0r );

      M13p6x22LV +=  + A23bV(m36,L) * (  - 3*m33*L3r - 6*m33*L0r - 3*
         m11*L3r - 6*m11*L0r );

      M13p6x22LV +=  + A23bV(m44,L) * (  - 18*m33*L2r - 18*m11*L2r );

      M13p6x22LV +=  + A23bV(m46,L) * (  - 24*m33*L2r - 24*m11*L2r );

      M13p6x22LV +=  + A23bV(me2,L) * (  - 4*m33*L3r*RSe6613 - 2*m33*
         L3r*RSe6633 - 2*m33*L3r*RSe6611 - 6*m33*L2r - 4*m33*L0r*
         RSe6633 - 4*m33*L0r*RSe6611 - 4*m11*L3r*RSe6613 - 2*m11*L3r*
         RSe6633 - 2*m11*L3r*RSe6611 - 6*m11*L2r - 4*m11*L0r*RSe6633 - 
         4*m11*L0r*RSe6611 );

      M13p6x22LV +=  + B23bV(m11,L) * ( m33*L3r*RD146e + 2*m33*L0r*
         RD146e + m11*L3r*RD146e + 2*m11*L0r*RD146e );

      M13p6x22LV +=  + B23bV(m33,L) * ( m33*L3r*RD346e + 2*m33*L0r*
         RD346e + m11*L3r*RD346e + 2*m11*L0r*RD346e );

  return M13p6x22LV/pow(f0,4);
}

double mv2s2nf3p6RV(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 4) || (B0mq.size() != 4)){
    cout << "ERROR: mv2s2nf3p6RV probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m66 = 2.*B0mq[3];
  double m13 = (m11+m33)/2;
  double m14 = (m11+m44)/2;
  double m16 = (m11+m66)/2;
  double m34 = (m33+m44)/2;
  double m36 = (m33+m66)/2;
  double m46 = (m44+m66)/2;
  double me2 = (m44+2.*m66)/3.;

  double RN13 = RN(m11,m33);
  double RN1e = RN(m11,me2);
  double RN3e = RN(m33,me2);

  double RD146e = RD(m11,m44,m66,me2);
  double RD346e = RD(m33,m44,m66,me2);

  double RC146e = RC(m11,m44,m66,me2);
  double RC346e = RC(m33,m44,m66,me2);

  double RS14e = RS(m11,m44,me2);
  double RS16e = RS(m11,m66,me2);
  double RS34e = RS(m33,m44,me2);
  double RS36e = RS(m33,m66,me2);
  double RSe61 = RS(me2,m66,m11);
  double RSe63 = RS(me2,m66,m33);

  double RS1463e = RS(m11,m44,m66,m33,me2);
  double RS3461e = RS(m33,m44,m66,m11,me2);
  double RSe6611 = RS(me2,m66,m66,m11,m11);
  double RSe6613 = RS(me2,m66,m66,m11,m33);
  double RSe6633 = RS(me2,m66,m66,m33,m33);

  double mu2 = mu0*mu0;

  double M13p6x22RV =
       + Ab(m11,mu2)*AbV(m11,L) * (  - 1./135.*RD146e*RS1463e + 1./270.
         *RD146e*RC146e + 1./8.*m33 + 13./270.*m33*pow(RS1463e,2) + 1./
         270.*m33*RC146e*RS1463e + 1./72.*m33*pow(RC146e,2) + 1./8.*m11
          + 17./270.*m11*pow(RS1463e,2) + 1./30.*m11*RC146e*RS1463e - 5.
         /216.*m11*pow(RC146e,2) - 2./27.*m11*m33*RN1e*RC146e*RSe6613
          + 1./27.*m11*m33*RN13*RC146e*RS3461e - 4./27.*pow(m11,2)*RN1e
         *RC146e*RSe6613 + 2./27.*pow(m11,2)*RN13*RC146e*RS3461e );

      M13p6x22RV +=  + Ab(m11,mu2)*AbV(m13,L) * (  - 1./162.*RD346e*
         RS1463e + 1./324.*RC146e*RD346e - 1./54.*RD146e*RS3461e - 2./
         81.*RD146e*RS1463e + 1./27.*RD146e*RSe6613 - 1./54.*RD146e*
         RSe6633 + 1./108.*RD146e*RC346e - 1./54.*RD146e*RSe6611 + 1./
         81.*RD146e*RC146e - 2./81.*me2*RSe6613*RS1463e + 1./81.*me2*
         RSe6633*RS1463e + 1./81.*me2*RSe6611*RS1463e + 1./81.*me2*
         RC146e*RSe6613 - 1./162.*me2*RC146e*RSe6633 - 1./162.*me2*
         RC146e*RSe6611 - 1./72.*m33 - 1./27.*m33*RS1463e*RS3461e - 4./
         81.*m33*pow(RS1463e,2) + 8./81.*m33*RSe6613*RS1463e - 5./81.*
         m33*RSe6633*RS1463e + 2./81.*m33*RC346e*RS1463e - 1./27.*m33*
         RSe6611*RS1463e + 1./81.*m33*RC146e*RS3461e + 1./27.*m33*
         RC146e*RS1463e - 1./27.*m33*RC146e*RSe6613 + 1./162.*m33*
         RC146e*RSe6633 + 5./162.*m33*RC146e*RSe6611 - 5./324.*m33*pow(
         RC146e,2) + 1./9.*pow(m33,2)*RN13 + 1./24.*m11 - 1./81.*m11*
         RS1463e*RS3461e + 2./81.*m11*RSe6613*RS1463e - 2./81.*m11*
         RSe6611*RS1463e );

      M13p6x22RV +=  + Ab(m11,mu2)*AbV(m13,L) * ( 1./81.*m11*RC146e*
         RS3461e + 1./81.*m11*RC146e*RS1463e - 2./81.*m11*RC146e*
         RSe6613 - 1./81.*m11*RC146e*RSe6633 + 1./162.*m11*RC146e*
         RC346e + 1./27.*m11*RC146e*RSe6611 - 5./324.*m11*pow(RC146e,2)
          + 5./9.*m11*m33*RN13 + 1./3.*pow(m11,2)*RN13 );

      M13p6x22RV +=  + Ab(m11,mu2)*AbV(m14,L) * (  - 29./270.*RD146e - 
         1./135.*m44*RS16e + 41./540.*m33*RS1463e - 1./12.*m33*RS16e - 
         2./27.*m33*m44*RN1e*RSe63 - 2./27.*m33*m44*RN13*RS36e + 2./27.
         *m11*RS3461e + 109./540.*m11*RS1463e - 4./27.*m11*RSe6613 - 2./
         27.*m11*RSe63 - 2./27.*m11*RS36e + 2./27.*m11*RSe61 + 4./27.*
         m11*RSe6611 + 19./540.*m11*RS16e - 4./27.*m11*me2*RN1e*RSe6613
          - 2./27.*m11*me2*RN1e*RSe63 + 2./27.*m11*me2*RN1e*RSe61 + 4./
         27.*m11*me2*RN1e*RSe6611 - 4./27.*m11*m44*RN1e*RSe63 - 4./27.*
         m11*m44*RN13*RS36e - 4./27.*m11*m33*RN1e*RSe63 + 2./27.*m11*
         m33*RN13*RS3461e - 2./9.*m11*m33*RN13*RS36e + 4./27.*pow(
         m11,2)*RN1e*RSe6613 - 2./9.*pow(m11,2)*RN1e*RSe63 - 2./27.*
         pow(m11,2)*RN1e*RSe61 - 4./27.*pow(m11,2)*RN1e*RSe6611 - 2./27.
         *pow(m11,2)*RN13*RS3461e - 2./9.*pow(m11,2)*RN13*RS36e );

      M13p6x22RV +=  + Ab(m11,mu2)*AbV(m16,L) * (  - 29./540.*RD146e - 
         1./270.*m66*RS14e + 41./1080.*m33*RS1463e - 1./24.*m33*RS14e
          + 2./27.*m33*m66*RN1e*RSe63 - 1./27.*m33*m66*RN13*RS34e + 1./
         27.*m11*RS3461e + 109./1080.*m11*RS1463e - 2./27.*m11*RSe6613
          + 2./27.*m11*RSe63 - 1./27.*m11*RS34e - 2./27.*m11*RSe61 + 2./
         27.*m11*RSe6611 + 19./1080.*m11*RS14e - 2./27.*m11*me2*RN1e*
         RSe6613 + 2./27.*m11*me2*RN1e*RSe63 - 2./27.*m11*me2*RN1e*
         RSe61 + 2./27.*m11*me2*RN1e*RSe6611 + 4./27.*m11*m66*RN1e*
         RSe63 - 2./27.*m11*m66*RN13*RS34e + 4./27.*m11*m33*RN1e*RSe63
          + 1./27.*m11*m33*RN13*RS3461e - 1./9.*m11*m33*RN13*RS34e + 2./
         27.*pow(m11,2)*RN1e*RSe6613 + 2./9.*pow(m11,2)*RN1e*RSe63 + 2./
         27.*pow(m11,2)*RN1e*RSe61 - 2./27.*pow(m11,2)*RN1e*RSe6611 - 1.
         /27.*pow(m11,2)*RN13*RS3461e - 1./9.*pow(m11,2)*RN13*RS34e );

      M13p6x22RV +=  + Ab(m11,mu2)*AbV(m33,L) * (  - 1./270.*RD346e*
         RS1463e + 1./540.*RC146e*RD346e - 1./270.*RD146e*RS3461e + 1./
         540.*RD146e*RC346e + 1./18.*m33*RS1463e*RS3461e - 1./60.*m33*
         RC346e*RS1463e - 1./540.*m33*RC146e*RS3461e - 1./72.*m33*
         RC146e*RC346e + 1./27.*pow(m33,2)*RN13*RC346e*RS1463e + 1./18.
         *m11*RS1463e*RS3461e - 1./540.*m11*RC346e*RS1463e + 11./540.*
         m11*RC146e*RS3461e - 1./72.*m11*RC146e*RC346e + 2./27.*m11*m33
         *RN13*RC346e*RS1463e );

      M13p6x22RV +=  + Ab(m11,mu2)*AbV(m34,L) * (  - 1./90.*RD146e + 17.
         /135.*m44*RS16e - 19./540.*m33*RS1463e + 1./108.*m33*RS16e + 1.
         /12.*m33*RC146e - 2./27.*m33*m44*RN13*RS16e - 2./27.*pow(
         m33,2)*RN13*RS1463e - 2./27.*pow(m33,2)*RN13*RS16e + 89./540.*
         m11*RS1463e - 83./540.*m11*RS16e + 1./12.*m11*RC146e - 4./27.*
         m11*m44*RN13*RS16e - 8./27.*m11*m33*RN13*RS16e + 2./27.*pow(
         m11,2)*RN13*RS1463e - 2./27.*pow(m11,2)*RN13*RS16e );

      M13p6x22RV +=  + Ab(m11,mu2)*AbV(m36,L) * (  - 1./180.*RD146e + 
         17./270.*m66*RS14e - 19./1080.*m33*RS1463e + 1./216.*m33*RS14e
          + 1./24.*m33*RC146e - 1./27.*m33*m66*RN13*RS14e - 1./27.*pow(
         m33,2)*RN13*RS1463e - 1./27.*pow(m33,2)*RN13*RS14e + 89./1080.
         *m11*RS1463e - 83./1080.*m11*RS14e + 1./24.*m11*RC146e - 2./27.
         *m11*m66*RN13*RS14e - 4./27.*m11*m33*RN13*RS14e + 1./27.*pow(
         m11,2)*RN13*RS1463e - 1./27.*pow(m11,2)*RN13*RS14e );

      M13p6x22RV +=  + Ab(m11,mu2)*AbV(m44,L) * (  - 1./18.*m44*pow(
         RS16e,2) + 1./9.*m33*m44*RN1e*RS16e*RSe63 + 1./9.*m33*m44*RN13
         *RS16e*RS36e + 2./9.*m11*m44*RN1e*RS16e*RSe63 + 2./9.*m11*m44*
         RN13*RS16e*RS36e );

      M13p6x22RV +=  + Ab(m11,mu2)*AbV(m46,L) * (  - 2./81.*m66*RS14e*
         RS16e - 1./81.*m66*pow(RS14e,2) - 1./81.*m44*pow(RS16e,2) - 2./
         81.*m44*RS14e*RS16e - 4./81.*m33*m66*RN1e*RS16e*RSe63 - 2./81.
         *m33*m66*RN1e*RS14e*RSe63 + 2./81.*m33*m66*RN13*RS16e*RS34e + 
         2./81.*m33*m66*RN13*RS14e*RS36e + 2./81.*m33*m66*RN13*RS14e*
         RS34e - 2./81.*m33*m44*RN1e*RS16e*RSe63 + 2./81.*m33*m44*RN1e*
         RS14e*RSe63 + 2./81.*m33*m44*RN13*RS16e*RS36e + 2./81.*m33*m44
         *RN13*RS16e*RS34e + 2./81.*m33*m44*RN13*RS14e*RS36e - 2./27.*
         m11*RS16e*RSe63 - 2./81.*m11*RS16e*RS36e + 2./81.*m11*RS16e*
         RS34e + 2./27.*m11*RS16e*RSe61 + 1./81.*m11*pow(RS16e,2) + 2./
         27.*m11*RS14e*RSe63 + 2./81.*m11*RS14e*RS36e - 2./81.*m11*
         RS14e*RS34e - 2./27.*m11*RS14e*RSe61 - 2./81.*m11*RS14e*RS16e
          + 1./81.*m11*pow(RS14e,2) - 2./27.*m11*me2*RN1e*RS16e*RSe63
          + 2./27.*m11*me2*RN1e*RS16e*RSe61 + 2./27.*m11*me2*RN1e*RS14e
         *RSe63 - 2./27.*m11*me2*RN1e*RS14e*RSe61 - 8./81.*m11*m66*RN1e
         *RS16e*RSe63 );

      M13p6x22RV +=  + Ab(m11,mu2)*AbV(m46,L) * (  - 4./81.*m11*m66*
         RN1e*RS14e*RSe63 + 4./81.*m11*m66*RN13*RS16e*RS34e + 4./81.*
         m11*m66*RN13*RS14e*RS36e + 4./81.*m11*m66*RN13*RS14e*RS34e - 4.
         /81.*m11*m44*RN1e*RS16e*RSe63 + 4./81.*m11*m44*RN1e*RS14e*
         RSe63 + 4./81.*m11*m44*RN13*RS16e*RS36e + 4./81.*m11*m44*RN13*
         RS16e*RS34e + 4./81.*m11*m44*RN13*RS14e*RS36e - 2./27.*m11*m33
         *RN1e*RS16e*RSe63 + 2./27.*m11*m33*RN1e*RS14e*RSe63 - 4./81.*
         m11*m33*RN13*RS16e*RS36e + 4./81.*m11*m33*RN13*RS16e*RS34e + 4.
         /81.*m11*m33*RN13*RS14e*RS36e - 4./81.*m11*m33*RN13*RS14e*
         RS34e - 2./27.*pow(m11,2)*RN1e*RS16e*RSe63 - 2./27.*pow(m11,2)
         *RN1e*RS16e*RSe61 + 2./27.*pow(m11,2)*RN1e*RS14e*RSe63 + 2./27.
         *pow(m11,2)*RN1e*RS14e*RSe61 - 2./81.*pow(m11,2)*RN13*RS16e*
         RS36e + 2./81.*pow(m11,2)*RN13*RS16e*RS34e + 2./81.*pow(m11,2)
         *RN13*RS14e*RS36e - 2./81.*pow(m11,2)*RN13*RS14e*RS34e );

      M13p6x22RV +=  + Ab(m11,mu2)*AbV(me2,L) * ( 1./135.*RD146e*
         RSe6613 - 1./270.*RD146e*RSe6633 - 1./270.*RD146e*RSe6611 - 2./
         135.*me2*RSe6613*RS1463e + 1./135.*me2*RSe6633*RS1463e + 1./
         135.*me2*RSe6611*RS1463e + 1./135.*me2*RC146e*RSe6613 - 1./270.
         *me2*RC146e*RSe6633 - 1./270.*me2*RC146e*RSe6611 - 1./81.*m66*
         pow(RS14e,2) - 1./162.*m44*pow(RS16e,2) - 13./135.*m33*RSe6613
         *RS1463e + 7./270.*m33*RSe6633*RS1463e - 1./270.*m33*RSe6611*
         RS1463e - 1./270.*m33*RC146e*RSe6613 + 17./540.*m33*RC146e*
         RSe6633 - 1./36.*m33*RC146e*RSe6611 - 4./81.*m33*m66*RN1e*
         RS14e*RSe63 + 2./81.*m33*m66*RN13*RS14e*RS34e + 1./81.*m33*m44
         *RN1e*RS16e*RSe63 + 1./81.*m33*m44*RN13*RS16e*RS36e - 2./27.*
         pow(m33,2)*RN13*RSe6633*RS1463e - 1./9.*m11*RSe6613*RS1463e + 
         1./270.*m11*RSe6633*RS1463e - 11./270.*m11*RSe6611*RS1463e - 
         11./270.*m11*RC146e*RSe6613 + 1./36.*m11*RC146e*RSe6633 + 1./
         20.*m11*RC146e*RSe6611 - 8./81.*m11*m66*RN1e*RS14e*RSe63 + 4./
         81.*m11*m66*RN13*RS14e*RS34e );

      M13p6x22RV +=  + Ab(m11,mu2)*AbV(me2,L) * ( 2./81.*m11*m44*RN1e*
         RS16e*RSe63 + 2./81.*m11*m44*RN13*RS16e*RS36e + 4./27.*m11*m33
         *RN1e*RSe6611*RSe6613 - 4./27.*m11*m33*RN13*RSe6633*RS1463e - 
         2./27.*m11*m33*RN13*RSe6611*RS3461e + 8./27.*pow(m11,2)*RN1e*
         RSe6611*RSe6613 - 4./27.*pow(m11,2)*RN13*RSe6611*RS3461e );

      M13p6x22RV +=  + Ab(m11,mu2)*BbV(m11,m33,L) * ( 1./27.*m11*m33*
         RC146e*RS3461e + 2./27.*pow(m11,2)*RC146e*RS3461e );

      M13p6x22RV +=  + Ab(m11,mu2)*BbV(m11,me2,L) * (  - 2./27.*m11*m33
         *RC146e*RSe6613 - 4./27.*pow(m11,2)*RC146e*RSe6613 );

      M13p6x22RV +=  + Ab(m11,mu2)*BbV(m11,L) * ( 1./540.*pow(RD146e,2)
          + 1./540.*m33*RD146e*RS1463e + 1./72.*m33*RD146e*RC146e + 1./
         60.*m11*RD146e*RS1463e - 5./216.*m11*RD146e*RC146e + 1./27.*
         m11*m33*RC146e*RS1463e - 2./27.*m11*m33*RN1e*RD146e*RSe6613 + 
         1./27.*m11*m33*RN13*RD146e*RS3461e + 2./27.*pow(m11,2)*RC146e*
         RS1463e - 4./27.*pow(m11,2)*RN1e*RD146e*RSe6613 + 2./27.*pow(
         m11,2)*RN13*RD146e*RS3461e );

      M13p6x22RV +=  + Ab(m11,mu2)*BbV(m33,L) * ( 1./540.*RD146e*RD346e
          - 1./60.*m33*RD346e*RS1463e - 1./72.*m33*RC146e*RD346e + 1./
         27.*pow(m33,2)*RN13*RD346e*RS1463e - 1./540.*m11*RD346e*
         RS1463e - 1./72.*m11*RC146e*RD346e + 2./27.*m11*m33*RN13*
         RD346e*RS1463e );

      M13p6x22RV +=  + Ab(m13,mu2)*AbV(m11,L) * (  - 1./162.*RD346e*
         RS1463e + 1./324.*RC146e*RD346e - 1./54.*RD146e*RS3461e - 2./
         81.*RD146e*RS1463e + 1./27.*RD146e*RSe6613 - 1./54.*RD146e*
         RSe6633 + 1./108.*RD146e*RC346e - 1./54.*RD146e*RSe6611 + 1./
         81.*RD146e*RC146e - 2./81.*me2*RSe6613*RS1463e + 1./81.*me2*
         RSe6633*RS1463e + 1./81.*me2*RSe6611*RS1463e + 1./81.*me2*
         RC146e*RSe6613 - 1./162.*me2*RC146e*RSe6633 - 1./162.*me2*
         RC146e*RSe6611 - 1./72.*m33 - 1./27.*m33*RS1463e*RS3461e - 4./
         81.*m33*pow(RS1463e,2) + 8./81.*m33*RSe6613*RS1463e - 5./81.*
         m33*RSe6633*RS1463e + 2./81.*m33*RC346e*RS1463e - 1./27.*m33*
         RSe6611*RS1463e + 1./81.*m33*RC146e*RS3461e + 1./27.*m33*
         RC146e*RS1463e - 1./27.*m33*RC146e*RSe6613 + 1./162.*m33*
         RC146e*RSe6633 + 5./162.*m33*RC146e*RSe6611 - 5./324.*m33*pow(
         RC146e,2) + 1./24.*m11 - 1./81.*m11*RS1463e*RS3461e + 2./81.*
         m11*RSe6613*RS1463e - 2./81.*m11*RSe6611*RS1463e + 1./81.*m11*
         RC146e*RS3461e );

      M13p6x22RV +=  + Ab(m13,mu2)*AbV(m11,L) * ( 1./81.*m11*RC146e*
         RS1463e - 2./81.*m11*RC146e*RSe6613 - 1./81.*m11*RC146e*
         RSe6633 + 1./162.*m11*RC146e*RC346e + 1./27.*m11*RC146e*
         RSe6611 - 5./324.*m11*pow(RC146e,2) );

      M13p6x22RV +=  + Ab(m13,mu2)*AbV(m13,L) * (  - 1./4.*m33 - 1./4.*
         m11 );

      M13p6x22RV +=  + Ab(m13,mu2)*AbV(m33,L) * (  - 2./81.*RD346e*
         RS3461e - 1./54.*RD346e*RS1463e + 1./27.*RD346e*RSe6613 - 1./
         54.*RD346e*RSe6633 + 1./81.*RD346e*RC346e - 1./54.*RSe6611*
         RD346e + 1./108.*RC146e*RD346e - 1./162.*RD146e*RS3461e + 1./
         324.*RD146e*RC346e - 2./81.*me2*RSe6613*RS3461e + 1./81.*me2*
         RSe6633*RS3461e + 1./81.*me2*RC346e*RSe6613 - 1./162.*me2*
         RC346e*RSe6633 + 1./81.*me2*RSe6611*RS3461e - 1./162.*me2*
         RSe6611*RC346e + 19./72.*m33 - 1./81.*m33*RS1463e*RS3461e + 2./
         81.*m33*RSe6613*RS3461e - 2./81.*m33*RSe6633*RS3461e + 1./81.*
         m33*RC346e*RS3461e + 1./81.*m33*RC346e*RS1463e - 2./81.*m33*
         RC346e*RSe6613 + 1./27.*m33*RC346e*RSe6633 - 5./324.*m33*pow(
         RC346e,2) - 1./81.*m33*RSe6611*RC346e + 1./162.*m33*RC146e*
         RC346e + 5./24.*m11 - 4./81.*m11*pow(RS3461e,2) - 1./27.*m11*
         RS1463e*RS3461e + 8./81.*m11*RSe6613*RS3461e - 1./27.*m11*
         RSe6633*RS3461e + 1./27.*m11*RC346e*RS3461e + 1./81.*m11*
         RC346e*RS1463e );

      M13p6x22RV +=  + Ab(m13,mu2)*AbV(m33,L) * (  - 1./27.*m11*RC346e*
         RSe6613 + 5./162.*m11*RC346e*RSe6633 - 5./324.*m11*pow(
         RC346e,2) - 5./81.*m11*RSe6611*RS3461e + 1./162.*m11*RSe6611*
         RC346e + 2./81.*m11*RC146e*RS3461e );

      M13p6x22RV +=  + Ab(m13,mu2)*AbV(me2,L) * ( 1./81.*RD346e*RSe6613
          - 1./162.*RD346e*RSe6633 - 1./162.*RSe6611*RD346e + 1./81.*
         RD146e*RSe6613 - 1./162.*RD146e*RSe6633 - 1./162.*RD146e*
         RSe6611 - 2./27.*me2*RSe6613*RS3461e - 2./27.*me2*RSe6613*
         RS1463e + 16./81.*me2*pow(RSe6613,2) + 1./27.*me2*RSe6633*
         RS3461e + 1./27.*me2*RSe6633*RS1463e - 16./81.*me2*RSe6633*
         RSe6613 + 4./81.*me2*pow(RSe6633,2) + 1./27.*me2*RC346e*
         RSe6613 - 1./54.*me2*RC346e*RSe6633 + 1./27.*me2*RSe6611*
         RS3461e + 1./27.*me2*RSe6611*RS1463e - 16./81.*me2*RSe6611*
         RSe6613 + 8./81.*me2*RSe6611*RSe6633 - 1./54.*me2*RSe6611*
         RC346e + 4./81.*me2*pow(RSe6611,2) + 1./27.*me2*RC146e*RSe6613
          - 1./54.*me2*RC146e*RSe6633 - 1./54.*me2*RC146e*RSe6611 + 2./
         27.*m33*RSe6613*RS3461e + 8./81.*m33*RSe6613*RS1463e - 16./81.
         *m33*pow(RSe6613,2) - 4./81.*m33*RSe6633*RS3461e - 5./81.*m33*
         RSe6633*RS1463e + 20./81.*m33*RSe6633*RSe6613 - 1./9.*m33*pow(
         RSe6633,2) );

      M13p6x22RV +=  + Ab(m13,mu2)*AbV(me2,L) * (  - 4./81.*m33*RC346e*
         RSe6613 + 4./81.*m33*RC346e*RSe6633 - 2./81.*m33*RSe6611*
         RS3461e - 1./27.*m33*RSe6611*RS1463e + 4./27.*m33*RSe6611*
         RSe6613 - 2./81.*m33*RSe6611*RSe6633 - 5./81.*m33*pow(
         RSe6611,2) - 1./27.*m33*RC146e*RSe6613 + 1./162.*m33*RC146e*
         RSe6633 + 5./162.*m33*RC146e*RSe6611 + 8./81.*m11*RSe6613*
         RS3461e + 2./27.*m11*RSe6613*RS1463e - 16./81.*m11*pow(
         RSe6613,2) - 1./27.*m11*RSe6633*RS3461e - 2./81.*m11*RSe6633*
         RS1463e + 4./27.*m11*RSe6633*RSe6613 - 5./81.*m11*pow(
         RSe6633,2) - 1./27.*m11*RC346e*RSe6613 + 5./162.*m11*RC346e*
         RSe6633 - 5./81.*m11*RSe6611*RS3461e - 4./81.*m11*RSe6611*
         RS1463e + 20./81.*m11*RSe6611*RSe6613 - 2./81.*m11*RSe6611*
         RSe6633 + 1./162.*m11*RSe6611*RC346e - 1./9.*m11*pow(
         RSe6611,2) - 4./81.*m11*RC146e*RSe6613 + 4./81.*m11*RC146e*
         RSe6611 );

      M13p6x22RV +=  + Ab(m13,mu2)*BbV(m11,m33,L) * ( 1./9.*pow(m33,2)
          + 5./9.*m11*m33 + 1./3.*pow(m11,2) );

      M13p6x22RV +=  + Ab(m13,mu2)*BbV(m11,L) * ( 1./324.*RD146e*RD346e
          + 1./324.*pow(RD146e,2) + 1./81.*me2*RD146e*RSe6613 - 1./162.
         *me2*RD146e*RSe6633 - 1./162.*me2*RD146e*RSe6611 + 1./81.*m33*
         RD146e*RS3461e + 1./54.*m33*RD146e*RS1463e - 1./27.*m33*RD146e
         *RSe6613 + 1./162.*m33*RD146e*RSe6633 + 5./162.*m33*RD146e*
         RSe6611 - 5./324.*m33*RD146e*RC146e + 1./81.*m11*RD146e*
         RS3461e + 1./162.*m11*RD146e*RS1463e - 2./81.*m11*RD146e*
         RSe6613 - 1./81.*m11*RD146e*RSe6633 + 1./162.*m11*RD146e*
         RC346e + 1./27.*m11*RD146e*RSe6611 - 5./324.*m11*RD146e*RC146e
          );

      M13p6x22RV +=  + Ab(m13,mu2)*BbV(m33,L) * ( 1./324.*pow(RD346e,2)
          + 1./324.*RD146e*RD346e + 1./81.*me2*RD346e*RSe6613 - 1./162.
         *me2*RD346e*RSe6633 - 1./162.*me2*RSe6611*RD346e + 1./162.*m33
         *RD346e*RS3461e + 1./81.*m33*RD346e*RS1463e - 2./81.*m33*
         RD346e*RSe6613 + 1./27.*m33*RD346e*RSe6633 - 5./324.*m33*
         RD346e*RC346e - 1./81.*m33*RSe6611*RD346e + 1./162.*m33*RC146e
         *RD346e + 1./54.*m11*RD346e*RS3461e + 1./81.*m11*RD346e*
         RS1463e - 1./27.*m11*RD346e*RSe6613 + 5./162.*m11*RD346e*
         RSe6633 - 5./324.*m11*RD346e*RC346e + 1./162.*m11*RSe6611*
         RD346e );

      M13p6x22RV +=  + Ab(m14,mu2)*AbV(m11,L) * (  - 29./270.*RD146e - 
         1./135.*m44*RS16e + 41./540.*m33*RS1463e - 1./12.*m33*RS16e + 
         2./27.*m11*RS3461e + 109./540.*m11*RS1463e - 4./27.*m11*
         RSe6613 - 2./27.*m11*RSe63 - 2./27.*m11*RS36e + 2./27.*m11*
         RSe61 + 4./27.*m11*RSe6611 + 19./540.*m11*RS16e );

      M13p6x22RV +=  + Ab(m14,mu2)*AbV(m33,L) * (  - 1./90.*RD346e + 7./
         135.*m44*RS36e + 43./180.*m33*RS3461e - 41./180.*m33*RS36e + 1.
         /12.*m33*RC346e - 19./540.*m11*RS3461e - 5./36.*m11*RS36e + 1./
         12.*m11*RC346e );

      M13p6x22RV +=  + Ab(m14,mu2)*AbV(m34,L) * (  - 1./54.*RD346e - 1./
         54.*RD146e + 1./27.*me2 + 2./27.*me2*RSe6613 + 2./27.*me2*
         RSe63 + 1./27.*me2*RSe6633 + 2./27.*me2*RSe61 + 1./27.*me2*
         RSe6611 - 1./27.*m44 - 1./27.*m44*RS3461e - 1./27.*m44*RS1463e
          + 2./27.*m44*RSe6613 + 1./27.*m44*RSe6633 - 1./54.*m44*RC346e
          + 1./27.*m44*RSe6611 - 1./54.*m44*RC146e - 47./108.*m33 - 13./
         108.*m33*RS3461e - 1./12.*m33*RS1463e + 1./6.*m33*RSe6613 - 1./
         36.*m33*RSe63 - 1./27.*m33*RSe6633 + 5./108.*m33*RS36e + 5./
         108.*m33*RSe61 + 1./27.*m33*RSe6611 + 5./108.*m33*RS16e - 1./
         54.*m33*RC146e - 47./108.*m11 - 1./12.*m11*RS3461e - 13./108.*
         m11*RS1463e + 1./6.*m11*RSe6613 + 5./108.*m11*RSe63 + 1./27.*
         m11*RSe6633 + 5./108.*m11*RS36e - 1./54.*m11*RC346e - 1./36.*
         m11*RSe61 - 1./27.*m11*RSe6611 + 5./108.*m11*RS16e );

      M13p6x22RV +=  + Ab(m14,mu2)*AbV(m36,L) * (  - 1./4.*m33 - 1./4.*
         m11 );

      M13p6x22RV +=  + Ab(m14,mu2)*AbV(me2,L) * (  - 1./45.*me2 - 44./
         135.*me2*RSe6613 - 19./135.*me2*RSe63 + 1./45.*me2*RSe6633 + 7.
         /135.*me2*RSe61 + 29./135.*me2*RSe6611 + 1./45.*m44 + 7./135.*
         m44*RSe63 - 1./135.*m44*RSe61 - 41./270.*m33*RSe6613 - 47./540.
         *m33*RSe63 - 17./90.*m33*RSe6633 - 1./12.*m33*RSe61 + 19./270.
         *m11*RSe6613 - 5./36.*m11*RSe63 - 1./6.*m11*RSe6633 - 49./540.
         *m11*RSe61 - 49./135.*m11*RSe6611 );

      M13p6x22RV +=  + Ab(m14,mu2)*BbV(m11,m33,L) * (  - 2./27.*m33*m44
         *RS36e - 4./27.*m11*m44*RS36e + 2./27.*m11*m33*RS3461e - 2./9.
         *m11*m33*RS36e - 2./27.*pow(m11,2)*RS3461e - 2./9.*pow(m11,2)*
         RS36e );

      M13p6x22RV +=  + Ab(m14,mu2)*BbV(m11,me2,L) * (  - 2./27.*m33*m44
         *RSe63 - 4./27.*m11*me2*RSe6613 - 2./27.*m11*me2*RSe63 + 2./27.
         *m11*me2*RSe61 + 4./27.*m11*me2*RSe6611 - 4./27.*m11*m44*RSe63
          - 4./27.*m11*m33*RSe63 + 4./27.*pow(m11,2)*RSe6613 - 2./9.*
         pow(m11,2)*RSe63 - 2./27.*pow(m11,2)*RSe61 - 4./27.*pow(m11,2)
         *RSe6611 );

      M13p6x22RV +=  + Ab(m14,mu2)*BbV(m33,L) * ( 1./12.*m33*RD346e + 1.
         /12.*m11*RD346e );

      M13p6x22RV +=  + Ab(m16,mu2)*AbV(m11,L) * (  - 29./540.*RD146e - 
         1./270.*m66*RS14e + 41./1080.*m33*RS1463e - 1./24.*m33*RS14e
          + 1./27.*m11*RS3461e + 109./1080.*m11*RS1463e - 2./27.*m11*
         RSe6613 + 2./27.*m11*RSe63 - 1./27.*m11*RS34e - 2./27.*m11*
         RSe61 + 2./27.*m11*RSe6611 + 19./1080.*m11*RS14e );

      M13p6x22RV +=  + Ab(m16,mu2)*AbV(m33,L) * (  - 1./180.*RD346e + 7.
         /270.*m66*RS34e + 43./360.*m33*RS3461e - 41./360.*m33*RS34e + 
         1./24.*m33*RC346e - 19./1080.*m11*RS3461e - 5./72.*m11*RS34e
          + 1./24.*m11*RC346e );

      M13p6x22RV +=  + Ab(m16,mu2)*AbV(m34,L) * (  - 1./4.*m33 - 1./4.*
         m11 );

      M13p6x22RV +=  + Ab(m16,mu2)*AbV(m36,L) * (  - 1./108.*RD346e - 1.
         /108.*RD146e + 2./27.*me2 + 1./27.*me2*RSe6613 - 2./27.*me2*
         RSe63 + 1./54.*me2*RSe6633 - 2./27.*me2*RSe61 + 1./54.*me2*
         RSe6611 - 2./27.*m66 - 1./54.*m66*RS3461e - 1./54.*m66*RS1463e
          + 1./27.*m66*RSe6613 + 1./54.*m66*RSe6633 - 1./108.*m66*
         RC346e + 1./54.*m66*RSe6611 - 1./108.*m66*RC146e - 5./54.*m33
          - 13./216.*m33*RS3461e - 1./24.*m33*RS1463e + 1./12.*m33*
         RSe6613 + 1./36.*m33*RSe63 - 1./54.*m33*RSe6633 + 5./216.*m33*
         RS34e - 5./108.*m33*RSe61 + 1./54.*m33*RSe6611 + 5./216.*m33*
         RS14e - 1./108.*m33*RC146e - 5./54.*m11 - 1./24.*m11*RS3461e
          - 13./216.*m11*RS1463e + 1./12.*m11*RSe6613 - 5./108.*m11*
         RSe63 + 1./54.*m11*RSe6633 + 5./216.*m11*RS34e - 1./108.*m11*
         RC346e + 1./36.*m11*RSe61 - 1./54.*m11*RSe6611 + 5./216.*m11*
         RS14e );

      M13p6x22RV +=  + Ab(m16,mu2)*AbV(me2,L) * (  - 2./45.*me2 - 22./
         135.*me2*RSe6613 + 19./135.*me2*RSe63 + 1./90.*me2*RSe6633 - 7.
         /135.*me2*RSe61 + 29./270.*me2*RSe6611 + 2./45.*m66 - 7./135.*
         m66*RSe63 + 1./135.*m66*RSe61 - 41./540.*m33*RSe6613 + 47./540.
         *m33*RSe63 - 17./180.*m33*RSe6633 + 1./12.*m33*RSe61 + 19./540.
         *m11*RSe6613 + 5./36.*m11*RSe63 - 1./12.*m11*RSe6633 + 49./540.
         *m11*RSe61 - 49./270.*m11*RSe6611 );

      M13p6x22RV +=  + Ab(m16,mu2)*BbV(m11,m33,L) * (  - 1./27.*m33*m66
         *RS34e - 2./27.*m11*m66*RS34e + 1./27.*m11*m33*RS3461e - 1./9.
         *m11*m33*RS34e - 1./27.*pow(m11,2)*RS3461e - 1./9.*pow(m11,2)*
         RS34e );

      M13p6x22RV +=  + Ab(m16,mu2)*BbV(m11,me2,L) * ( 2./27.*m33*m66*
         RSe63 - 2./27.*m11*me2*RSe6613 + 2./27.*m11*me2*RSe63 - 2./27.
         *m11*me2*RSe61 + 2./27.*m11*me2*RSe6611 + 4./27.*m11*m66*RSe63
          + 4./27.*m11*m33*RSe63 + 2./27.*pow(m11,2)*RSe6613 + 2./9.*
         pow(m11,2)*RSe63 + 2./27.*pow(m11,2)*RSe61 - 2./27.*pow(m11,2)
         *RSe6611 );

      M13p6x22RV +=  + Ab(m16,mu2)*BbV(m33,L) * ( 1./24.*m33*RD346e + 1.
         /24.*m11*RD346e );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(m11,L) * (  - 1./270.*RD346e*
         RS1463e + 1./540.*RC146e*RD346e - 1./270.*RD146e*RS3461e + 1./
         540.*RD146e*RC346e + 1./18.*m33*RS1463e*RS3461e - 1./60.*m33*
         RC346e*RS1463e - 1./540.*m33*RC146e*RS3461e - 1./72.*m33*
         RC146e*RC346e + 1./18.*m11*RS1463e*RS3461e - 1./540.*m11*
         RC346e*RS1463e + 11./540.*m11*RC146e*RS3461e - 1./72.*m11*
         RC146e*RC346e - 1./27.*m11*m33*RN13*RC146e*RS3461e - 2./27.*
         pow(m11,2)*RN13*RC146e*RS3461e );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(m13,L) * (  - 2./81.*RD346e*
         RS3461e - 1./54.*RD346e*RS1463e + 1./27.*RD346e*RSe6613 - 1./
         54.*RD346e*RSe6633 + 1./81.*RD346e*RC346e - 1./54.*RSe6611*
         RD346e + 1./108.*RC146e*RD346e - 1./162.*RD146e*RS3461e + 1./
         324.*RD146e*RC346e - 2./81.*me2*RSe6613*RS3461e + 1./81.*me2*
         RSe6633*RS3461e + 1./81.*me2*RC346e*RSe6613 - 1./162.*me2*
         RC346e*RSe6633 + 1./81.*me2*RSe6611*RS3461e - 1./162.*me2*
         RSe6611*RC346e + 19./72.*m33 - 1./81.*m33*RS1463e*RS3461e + 2./
         81.*m33*RSe6613*RS3461e - 2./81.*m33*RSe6633*RS3461e + 1./81.*
         m33*RC346e*RS3461e + 1./81.*m33*RC346e*RS1463e - 2./81.*m33*
         RC346e*RSe6613 + 1./27.*m33*RC346e*RSe6633 - 5./324.*m33*pow(
         RC346e,2) - 1./81.*m33*RSe6611*RC346e + 1./162.*m33*RC146e*
         RC346e - 1./9.*pow(m33,2)*RN13 + 5./24.*m11 - 4./81.*m11*pow(
         RS3461e,2) - 1./27.*m11*RS1463e*RS3461e + 8./81.*m11*RSe6613*
         RS3461e - 1./27.*m11*RSe6633*RS3461e + 1./27.*m11*RC346e*
         RS3461e );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(m13,L) * ( 1./81.*m11*RC346e*
         RS1463e - 1./27.*m11*RC346e*RSe6613 + 5./162.*m11*RC346e*
         RSe6633 - 5./324.*m11*pow(RC346e,2) - 5./81.*m11*RSe6611*
         RS3461e + 1./162.*m11*RSe6611*RC346e + 2./81.*m11*RC146e*
         RS3461e - 5./9.*m11*m33*RN13 - 1./3.*pow(m11,2)*RN13 );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(m14,L) * (  - 1./90.*RD346e + 7./
         135.*m44*RS36e + 43./180.*m33*RS3461e - 41./180.*m33*RS36e + 1.
         /12.*m33*RC346e + 2./27.*m33*m44*RN13*RS36e - 19./540.*m11*
         RS3461e - 5./36.*m11*RS36e + 1./12.*m11*RC346e + 4./27.*m11*
         m44*RN13*RS36e - 2./27.*m11*m33*RN13*RS3461e + 2./9.*m11*m33*
         RN13*RS36e + 2./27.*pow(m11,2)*RN13*RS3461e + 2./9.*pow(m11,2)
         *RN13*RS36e );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(m16,L) * (  - 1./180.*RD346e + 7.
         /270.*m66*RS34e + 43./360.*m33*RS3461e - 41./360.*m33*RS34e + 
         1./24.*m33*RC346e + 1./27.*m33*m66*RN13*RS34e - 19./1080.*m11*
         RS3461e - 5./72.*m11*RS34e + 1./24.*m11*RC346e + 2./27.*m11*
         m66*RN13*RS34e - 1./27.*m11*m33*RN13*RS3461e + 1./9.*m11*m33*
         RN13*RS34e + 1./27.*pow(m11,2)*RN13*RS3461e + 1./9.*pow(m11,2)
         *RN13*RS34e );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(m33,L) * (  - 1./135.*RD346e*
         RS3461e + 1./270.*RD346e*RC346e + 1./8.*m33 + 17./270.*m33*
         pow(RS3461e,2) + 1./30.*m33*RC346e*RS3461e + 2./27.*m33*RC346e
         *RS1463e - 5./216.*m33*pow(RC346e,2) - 4./27.*pow(m33,2)*RN3e*
         RC346e*RSe6613 - 1./27.*pow(m33,2)*RN13*RC346e*RS1463e + 1./8.
         *m11 + 13./270.*m11*pow(RS3461e,2) + 1./270.*m11*RC346e*
         RS3461e + 1./72.*m11*pow(RC346e,2) - 2./27.*m11*m33*RN3e*
         RC346e*RSe6613 - 2./27.*m11*m33*RN13*RC346e*RS1463e );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(m34,L) * (  - 29./270.*RD346e - 
         1./135.*m44*RS36e - 2./27.*m44*RS16e + 109./540.*m33*RS3461e
          + 2./27.*m33*RS1463e - 4./27.*m33*RSe6613 + 2./27.*m33*RSe63
          + 4./27.*m33*RSe6633 + 19./540.*m33*RS36e - 2./27.*m33*RSe61
          - 2./9.*m33*RS16e - 4./27.*m33*me2*RN3e*RSe6613 + 2./27.*m33*
         me2*RN3e*RSe63 + 4./27.*m33*me2*RN3e*RSe6633 - 2./27.*m33*me2*
         RN3e*RSe61 - 4./27.*m33*m44*RN3e*RSe61 + 2./27.*m33*m44*RN13*
         RS16e + 4./27.*pow(m33,2)*RN3e*RSe6613 - 2./27.*pow(m33,2)*
         RN3e*RSe63 - 4./27.*pow(m33,2)*RN3e*RSe6633 - 2./9.*pow(m33,2)
         *RN3e*RSe61 + 2./27.*pow(m33,2)*RN13*RS1463e + 2./27.*pow(
         m33,2)*RN13*RS16e + 41./540.*m11*RS3461e + 2./27.*m11*RS1463e
          - 1./12.*m11*RS36e - 2./27.*m11*RS16e - 2./27.*m11*m44*RN3e*
         RSe61 + 4./27.*m11*m44*RN13*RS16e - 4./27.*m11*m33*RN3e*RSe61
          + 8./27.*m11*m33*RN13*RS16e - 2./27.*pow(m11,2)*RN13*RS1463e
          + 2./27.*pow(m11,2)*RN13*RS16e );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(m36,L) * (  - 29./540.*RD346e - 
         1./270.*m66*RS34e - 1./27.*m66*RS14e + 109./1080.*m33*RS3461e
          + 1./27.*m33*RS1463e - 2./27.*m33*RSe6613 - 2./27.*m33*RSe63
          + 2./27.*m33*RSe6633 + 19./1080.*m33*RS34e + 2./27.*m33*RSe61
          - 1./9.*m33*RS14e - 2./27.*m33*me2*RN3e*RSe6613 - 2./27.*m33*
         me2*RN3e*RSe63 + 2./27.*m33*me2*RN3e*RSe6633 + 2./27.*m33*me2*
         RN3e*RSe61 + 4./27.*m33*m66*RN3e*RSe61 + 1./27.*m33*m66*RN13*
         RS14e + 2./27.*pow(m33,2)*RN3e*RSe6613 + 2./27.*pow(m33,2)*
         RN3e*RSe63 - 2./27.*pow(m33,2)*RN3e*RSe6633 + 2./9.*pow(m33,2)
         *RN3e*RSe61 + 1./27.*pow(m33,2)*RN13*RS1463e + 1./27.*pow(
         m33,2)*RN13*RS14e + 41./1080.*m11*RS3461e + 1./27.*m11*RS1463e
          - 1./24.*m11*RS34e - 1./27.*m11*RS14e + 2./27.*m11*m66*RN3e*
         RSe61 + 2./27.*m11*m66*RN13*RS14e + 4./27.*m11*m33*RN3e*RSe61
          + 4./27.*m11*m33*RN13*RS14e - 1./27.*pow(m11,2)*RN13*RS1463e
          + 1./27.*pow(m11,2)*RN13*RS14e );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(m44,L) * (  - 1./18.*m44*pow(
         RS36e,2) + 1./9.*m44*RS16e*RS36e + 2./9.*m33*m44*RN3e*RSe61*
         RS36e - 1./9.*m33*m44*RN13*RS16e*RS36e + 1./9.*m11*m44*RN3e*
         RSe61*RS36e - 2./9.*m11*m44*RN13*RS16e*RS36e );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(m46,L) * (  - 2./81.*m66*RS34e*
         RS36e - 1./81.*m66*pow(RS34e,2) + 2./81.*m66*RS16e*RS34e + 2./
         81.*m66*RS14e*RS36e + 2./81.*m66*RS14e*RS34e - 1./81.*m44*pow(
         RS36e,2) - 2./81.*m44*RS34e*RS36e + 2./81.*m44*RS16e*RS36e + 2.
         /81.*m44*RS16e*RS34e + 2./81.*m44*RS14e*RS36e + 2./27.*m33*
         RS36e*RSe63 + 1./81.*m33*pow(RS36e,2) - 2./27.*m33*RS34e*RSe63
          - 2./81.*m33*RS34e*RS36e + 1./81.*m33*pow(RS34e,2) - 2./27.*
         m33*RSe61*RS36e + 2./27.*m33*RSe61*RS34e - 4./81.*m33*RS16e*
         RS36e + 4./81.*m33*RS16e*RS34e + 4./81.*m33*RS14e*RS36e - 4./
         81.*m33*RS14e*RS34e + 2./27.*m33*me2*RN3e*RS36e*RSe63 - 2./27.
         *m33*me2*RN3e*RS34e*RSe63 - 2./27.*m33*me2*RN3e*RSe61*RS36e + 
         2./27.*m33*me2*RN3e*RSe61*RS34e - 8./81.*m33*m66*RN3e*RSe61*
         RS36e - 4./81.*m33*m66*RN3e*RSe61*RS34e - 2./81.*m33*m66*RN13*
         RS16e*RS34e - 2./81.*m33*m66*RN13*RS14e*RS36e - 2./81.*m33*m66
         *RN13*RS14e*RS34e - 4./81.*m33*m44*RN3e*RSe61*RS36e + 4./81.*
         m33*m44*RN3e*RSe61*RS34e );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(m46,L) * (  - 2./81.*m33*m44*
         RN13*RS16e*RS36e - 2./81.*m33*m44*RN13*RS16e*RS34e - 2./81.*
         m33*m44*RN13*RS14e*RS36e - 2./27.*pow(m33,2)*RN3e*RS36e*RSe63
          + 2./27.*pow(m33,2)*RN3e*RS34e*RSe63 - 2./27.*pow(m33,2)*RN3e
         *RSe61*RS36e + 2./27.*pow(m33,2)*RN3e*RSe61*RS34e - 2./81.*m11
         *RS16e*RS36e + 2./81.*m11*RS16e*RS34e + 2./81.*m11*RS14e*RS36e
          - 2./81.*m11*RS14e*RS34e - 4./81.*m11*m66*RN3e*RSe61*RS36e - 
         2./81.*m11*m66*RN3e*RSe61*RS34e - 4./81.*m11*m66*RN13*RS16e*
         RS34e - 4./81.*m11*m66*RN13*RS14e*RS36e - 4./81.*m11*m66*RN13*
         RS14e*RS34e - 2./81.*m11*m44*RN3e*RSe61*RS36e + 2./81.*m11*m44
         *RN3e*RSe61*RS34e - 4./81.*m11*m44*RN13*RS16e*RS36e - 4./81.*
         m11*m44*RN13*RS16e*RS34e - 4./81.*m11*m44*RN13*RS14e*RS36e - 2.
         /27.*m11*m33*RN3e*RSe61*RS36e + 2./27.*m11*m33*RN3e*RSe61*
         RS34e + 4./81.*m11*m33*RN13*RS16e*RS36e - 4./81.*m11*m33*RN13*
         RS16e*RS34e - 4./81.*m11*m33*RN13*RS14e*RS36e + 4./81.*m11*m33
         *RN13*RS14e*RS34e );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(m46,L) * ( 2./81.*pow(m11,2)*
         RN13*RS16e*RS36e - 2./81.*pow(m11,2)*RN13*RS16e*RS34e - 2./81.
         *pow(m11,2)*RN13*RS14e*RS36e + 2./81.*pow(m11,2)*RN13*RS14e*
         RS34e );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(me2,L) * ( 1./135.*RD346e*
         RSe6613 - 1./270.*RD346e*RSe6633 - 1./270.*RSe6611*RD346e - 2./
         135.*me2*RSe6613*RS3461e + 1./135.*me2*RSe6633*RS3461e + 1./
         135.*me2*RC346e*RSe6613 - 1./270.*me2*RC346e*RSe6633 + 1./135.
         *me2*RSe6611*RS3461e - 1./270.*me2*RSe6611*RC346e - 1./81.*m66
         *pow(RS34e,2) + 2./81.*m66*RS14e*RS34e - 1./162.*m44*pow(
         RS36e,2) + 1./81.*m44*RS16e*RS36e - 1./9.*m33*RSe6613*RS3461e
          - 11./270.*m33*RSe6633*RS3461e - 2./27.*m33*RSe6633*RS1463e
          - 11./270.*m33*RC346e*RSe6613 + 1./20.*m33*RC346e*RSe6633 + 1.
         /270.*m33*RSe6611*RS3461e + 1./36.*m33*RSe6611*RC346e - 8./81.
         *m33*m66*RN3e*RSe61*RS34e - 2./81.*m33*m66*RN13*RS14e*RS34e + 
         2./81.*m33*m44*RN3e*RSe61*RS36e - 1./81.*m33*m44*RN13*RS16e*
         RS36e + 8./27.*pow(m33,2)*RN3e*RSe6633*RSe6613 + 2./27.*pow(
         m33,2)*RN13*RSe6633*RS1463e - 13./135.*m11*RSe6613*RS3461e - 1.
         /270.*m11*RSe6633*RS3461e - 1./270.*m11*RC346e*RSe6613 - 1./36.
         *m11*RC346e*RSe6633 );

      M13p6x22RV +=  + Ab(m33,mu2)*AbV(me2,L) * (  - 13./270.*m11*
         RSe6611*RS3461e + 17./540.*m11*RSe6611*RC346e - 4./81.*m11*m66
         *RN3e*RSe61*RS34e - 4./81.*m11*m66*RN13*RS14e*RS34e + 1./81.*
         m11*m44*RN3e*RSe61*RS36e - 2./81.*m11*m44*RN13*RS16e*RS36e + 4.
         /27.*m11*m33*RN3e*RSe6633*RSe6613 + 4./27.*m11*m33*RN13*
         RSe6633*RS1463e + 2./27.*m11*m33*RN13*RSe6611*RS3461e + 4./27.
         *pow(m11,2)*RN13*RSe6611*RS3461e );

      M13p6x22RV +=  + Ab(m33,mu2)*BbV(m11,m33,L) * ( 1./27.*pow(m33,2)
         *RC346e*RS1463e + 2./27.*m11*m33*RC346e*RS1463e );

      M13p6x22RV +=  + Ab(m33,mu2)*BbV(m11,L) * ( 1./540.*RD146e*RD346e
          - 1./540.*m33*RD146e*RS3461e - 1./72.*m33*RD146e*RC346e + 11./
         540.*m11*RD146e*RS3461e - 1./72.*m11*RD146e*RC346e - 1./27.*
         m11*m33*RN13*RD146e*RS3461e - 2./27.*pow(m11,2)*RN13*RD146e*
         RS3461e );

      M13p6x22RV +=  + Ab(m33,mu2)*BbV(m33,me2,L) * (  - 4./27.*pow(
         m33,2)*RC346e*RSe6613 - 2./27.*m11*m33*RC346e*RSe6613 );

      M13p6x22RV +=  + Ab(m33,mu2)*BbV(m33,L) * ( 1./540.*pow(RD346e,2)
          + 1./60.*m33*RD346e*RS3461e + 1./27.*m33*RD346e*RS1463e - 5./
         216.*m33*RD346e*RC346e + 2./27.*pow(m33,2)*RC346e*RS3461e - 4./
         27.*pow(m33,2)*RN3e*RD346e*RSe6613 - 1./27.*pow(m33,2)*RN13*
         RD346e*RS1463e + 1./540.*m11*RD346e*RS3461e + 1./72.*m11*
         RD346e*RC346e + 1./27.*m11*m33*RC346e*RS3461e - 2./27.*m11*m33
         *RN3e*RD346e*RSe6613 - 2./27.*m11*m33*RN13*RD346e*RS1463e );

      M13p6x22RV +=  + Ab(m34,mu2)*AbV(m11,L) * (  - 1./90.*RD146e + 17.
         /135.*m44*RS16e - 19./540.*m33*RS1463e + 1./108.*m33*RS16e + 1.
         /12.*m33*RC146e + 89./540.*m11*RS1463e - 83./540.*m11*RS16e + 
         1./12.*m11*RC146e );

      M13p6x22RV +=  + Ab(m34,mu2)*AbV(m14,L) * (  - 1./54.*RD346e - 1./
         54.*RD146e + 1./27.*me2 + 2./27.*me2*RSe6613 + 2./27.*me2*
         RSe63 + 1./27.*me2*RSe6633 + 2./27.*me2*RSe61 + 1./27.*me2*
         RSe6611 - 1./27.*m44 - 1./27.*m44*RS3461e - 1./27.*m44*RS1463e
          + 2./27.*m44*RSe6613 + 1./27.*m44*RSe6633 - 1./54.*m44*RC346e
          + 1./27.*m44*RSe6611 - 1./54.*m44*RC146e - 47./108.*m33 - 13./
         108.*m33*RS3461e - 1./12.*m33*RS1463e + 1./6.*m33*RSe6613 - 1./
         36.*m33*RSe63 - 1./27.*m33*RSe6633 + 5./108.*m33*RS36e + 5./
         108.*m33*RSe61 + 1./27.*m33*RSe6611 + 5./108.*m33*RS16e - 1./
         54.*m33*RC146e - 47./108.*m11 - 1./12.*m11*RS3461e - 13./108.*
         m11*RS1463e + 1./6.*m11*RSe6613 + 5./108.*m11*RSe63 + 1./27.*
         m11*RSe6633 + 5./108.*m11*RS36e - 1./54.*m11*RC346e - 1./36.*
         m11*RSe61 - 1./27.*m11*RSe6611 + 5./108.*m11*RS16e );

      M13p6x22RV +=  + Ab(m34,mu2)*AbV(m16,L) * (  - 1./4.*m33 - 1./4.*
         m11 );

      M13p6x22RV +=  + Ab(m34,mu2)*AbV(m33,L) * (  - 29./270.*RD346e - 
         1./135.*m44*RS36e - 2./27.*m44*RS16e + 109./540.*m33*RS3461e
          + 2./27.*m33*RS1463e - 4./27.*m33*RSe6613 + 2./27.*m33*RSe63
          + 4./27.*m33*RSe6633 + 19./540.*m33*RS36e - 2./27.*m33*RSe61
          - 2./9.*m33*RS16e + 41./540.*m11*RS3461e + 2./27.*m11*RS1463e
          - 1./12.*m11*RS36e - 2./27.*m11*RS16e );

      M13p6x22RV +=  + Ab(m34,mu2)*AbV(me2,L) * (  - 1./45.*me2 - 44./
         135.*me2*RSe6613 + 7./135.*me2*RSe63 + 29./135.*me2*RSe6633 - 
         19./135.*me2*RSe61 + 1./45.*me2*RSe6611 + 1./45.*m44 - 1./135.
         *m44*RSe63 + 7./135.*m44*RSe61 + 19./270.*m33*RSe6613 - 49./
         540.*m33*RSe63 - 49./135.*m33*RSe6633 - 5./36.*m33*RSe61 - 1./
         6.*m33*RSe6611 - 41./270.*m11*RSe6613 - 1./12.*m11*RSe63 - 47./
         540.*m11*RSe61 - 17./90.*m11*RSe6611 );

      M13p6x22RV +=  + Ab(m34,mu2)*BbV(m11,m33,L) * (  - 2./27.*m33*m44
         *RS16e - 2./27.*pow(m33,2)*RS1463e - 2./27.*pow(m33,2)*RS16e
          - 4./27.*m11*m44*RS16e - 8./27.*m11*m33*RS16e + 2./27.*pow(
         m11,2)*RS1463e - 2./27.*pow(m11,2)*RS16e );

      M13p6x22RV +=  + Ab(m34,mu2)*BbV(m11,L) * ( 1./12.*m33*RD146e + 1.
         /12.*m11*RD146e );

      M13p6x22RV +=  + Ab(m34,mu2)*BbV(m33,me2,L) * (  - 4./27.*m33*me2
         *RSe6613 + 2./27.*m33*me2*RSe63 + 4./27.*m33*me2*RSe6633 - 2./
         27.*m33*me2*RSe61 - 4./27.*m33*m44*RSe61 + 4./27.*pow(m33,2)*
         RSe6613 - 2./27.*pow(m33,2)*RSe63 - 4./27.*pow(m33,2)*RSe6633
          - 2./9.*pow(m33,2)*RSe61 - 2./27.*m11*m44*RSe61 - 4./27.*m11*
         m33*RSe61 );

      M13p6x22RV +=  + Ab(m36,mu2)*AbV(m11,L) * (  - 1./180.*RD146e + 
         17./270.*m66*RS14e - 19./1080.*m33*RS1463e + 1./216.*m33*RS14e
          + 1./24.*m33*RC146e + 89./1080.*m11*RS1463e - 83./1080.*m11*
         RS14e + 1./24.*m11*RC146e );

      M13p6x22RV +=  + Ab(m36,mu2)*AbV(m14,L) * (  - 1./4.*m33 - 1./4.*
         m11 );

      M13p6x22RV +=  + Ab(m36,mu2)*AbV(m16,L) * (  - 1./108.*RD346e - 1.
         /108.*RD146e + 2./27.*me2 + 1./27.*me2*RSe6613 - 2./27.*me2*
         RSe63 + 1./54.*me2*RSe6633 - 2./27.*me2*RSe61 + 1./54.*me2*
         RSe6611 - 2./27.*m66 - 1./54.*m66*RS3461e - 1./54.*m66*RS1463e
          + 1./27.*m66*RSe6613 + 1./54.*m66*RSe6633 - 1./108.*m66*
         RC346e + 1./54.*m66*RSe6611 - 1./108.*m66*RC146e - 5./54.*m33
          - 13./216.*m33*RS3461e - 1./24.*m33*RS1463e + 1./12.*m33*
         RSe6613 + 1./36.*m33*RSe63 - 1./54.*m33*RSe6633 + 5./216.*m33*
         RS34e - 5./108.*m33*RSe61 + 1./54.*m33*RSe6611 + 5./216.*m33*
         RS14e - 1./108.*m33*RC146e - 5./54.*m11 - 1./24.*m11*RS3461e
          - 13./216.*m11*RS1463e + 1./12.*m11*RSe6613 - 5./108.*m11*
         RSe63 + 1./54.*m11*RSe6633 + 5./216.*m11*RS34e - 1./108.*m11*
         RC346e + 1./36.*m11*RSe61 - 1./54.*m11*RSe6611 + 5./216.*m11*
         RS14e );

      M13p6x22RV +=  + Ab(m36,mu2)*AbV(m33,L) * (  - 29./540.*RD346e - 
         1./270.*m66*RS34e - 1./27.*m66*RS14e + 109./1080.*m33*RS3461e
          + 1./27.*m33*RS1463e - 2./27.*m33*RSe6613 - 2./27.*m33*RSe63
          + 2./27.*m33*RSe6633 + 19./1080.*m33*RS34e + 2./27.*m33*RSe61
          - 1./9.*m33*RS14e + 41./1080.*m11*RS3461e + 1./27.*m11*
         RS1463e - 1./24.*m11*RS34e - 1./27.*m11*RS14e );

      M13p6x22RV +=  + Ab(m36,mu2)*AbV(me2,L) * (  - 2./45.*me2 - 22./
         135.*me2*RSe6613 - 7./135.*me2*RSe63 + 29./270.*me2*RSe6633 + 
         19./135.*me2*RSe61 + 1./90.*me2*RSe6611 + 2./45.*m66 + 1./135.
         *m66*RSe63 - 7./135.*m66*RSe61 + 19./540.*m33*RSe6613 + 49./
         540.*m33*RSe63 - 49./270.*m33*RSe6633 + 5./36.*m33*RSe61 - 1./
         12.*m33*RSe6611 - 41./540.*m11*RSe6613 + 1./12.*m11*RSe63 + 47.
         /540.*m11*RSe61 - 17./180.*m11*RSe6611 );

      M13p6x22RV +=  + Ab(m36,mu2)*BbV(m11,m33,L) * (  - 1./27.*m33*m66
         *RS14e - 1./27.*pow(m33,2)*RS1463e - 1./27.*pow(m33,2)*RS14e
          - 2./27.*m11*m66*RS14e - 4./27.*m11*m33*RS14e + 1./27.*pow(
         m11,2)*RS1463e - 1./27.*pow(m11,2)*RS14e );

      M13p6x22RV +=  + Ab(m36,mu2)*BbV(m11,L) * ( 1./24.*m33*RD146e + 1.
         /24.*m11*RD146e );

      M13p6x22RV +=  + Ab(m36,mu2)*BbV(m33,me2,L) * (  - 2./27.*m33*me2
         *RSe6613 - 2./27.*m33*me2*RSe63 + 2./27.*m33*me2*RSe6633 + 2./
         27.*m33*me2*RSe61 + 4./27.*m33*m66*RSe61 + 2./27.*pow(m33,2)*
         RSe6613 + 2./27.*pow(m33,2)*RSe63 - 2./27.*pow(m33,2)*RSe6633
          + 2./9.*pow(m33,2)*RSe61 + 2./27.*m11*m66*RSe61 + 4./27.*m11*
         m33*RSe61 );

      M13p6x22RV +=  + Ab(m44,mu2)*AbV(m11,L) * (  - 1./18.*m44*pow(
         RS16e,2) );

      M13p6x22RV +=  + Ab(m44,mu2)*AbV(m33,L) * (  - 1./18.*m44*pow(
         RS36e,2) + 1./9.*m44*RS16e*RS36e );

      M13p6x22RV +=  + Ab(m44,mu2)*AbV(me2,L) * (  - 1./18.*m44*pow(
         RSe63,2) - 1./9.*m44*RS36e*RSe63 + 1./9.*m44*RSe61*RSe63 + 1./
         9.*m44*RSe61*RS36e - 1./18.*m44*pow(RSe61,2) + 1./9.*m44*RS16e
         *RSe63 - 1./9.*m44*RS16e*RSe61 );

      M13p6x22RV +=  + Ab(m44,mu2)*BbV(m11,m33,L) * ( 1./9.*m33*m44*
         RS16e*RS36e + 2./9.*m11*m44*RS16e*RS36e );

      M13p6x22RV +=  + Ab(m44,mu2)*BbV(m11,me2,L) * ( 1./9.*m33*m44*
         RS16e*RSe63 + 2./9.*m11*m44*RS16e*RSe63 );

      M13p6x22RV +=  + Ab(m44,mu2)*BbV(m33,me2,L) * ( 2./9.*m33*m44*
         RSe61*RS36e + 1./9.*m11*m44*RSe61*RS36e );

      M13p6x22RV +=  + Ab(m44,mu2)*BbV(me2,L) * (  - 1./18.*m44*me2*
         pow(RSe63,2) + 1./9.*m44*me2*RSe61*RSe63 - 1./18.*m44*me2*pow(
         RSe61,2) + 1./18.*m33*m44*pow(RSe63,2) + 1./9.*m33*m44*RSe61*
         RSe63 + 1./9.*m11*m44*RSe61*RSe63 + 1./18.*m11*m44*pow(
         RSe61,2) );

      M13p6x22RV +=  + Ab(m46,mu2)*AbV(m11,L) * (  - 2./81.*m66*RS14e*
         RS16e - 1./81.*m66*pow(RS14e,2) - 1./81.*m44*pow(RS16e,2) - 2./
         81.*m44*RS14e*RS16e - 2./27.*m11*RS16e*RSe63 - 2./81.*m11*
         RS16e*RS36e + 2./81.*m11*RS16e*RS34e + 2./27.*m11*RS16e*RSe61
          + 1./81.*m11*pow(RS16e,2) + 2./27.*m11*RS14e*RSe63 + 2./81.*
         m11*RS14e*RS36e - 2./81.*m11*RS14e*RS34e - 2./27.*m11*RS14e*
         RSe61 - 2./81.*m11*RS14e*RS16e + 1./81.*m11*pow(RS14e,2) );

      M13p6x22RV +=  + Ab(m46,mu2)*AbV(m33,L) * (  - 2./81.*m66*RS34e*
         RS36e - 1./81.*m66*pow(RS34e,2) + 2./81.*m66*RS16e*RS34e + 2./
         81.*m66*RS14e*RS36e + 2./81.*m66*RS14e*RS34e - 1./81.*m44*pow(
         RS36e,2) - 2./81.*m44*RS34e*RS36e + 2./81.*m44*RS16e*RS36e + 2.
         /81.*m44*RS16e*RS34e + 2./81.*m44*RS14e*RS36e + 2./27.*m33*
         RS36e*RSe63 + 1./81.*m33*pow(RS36e,2) - 2./27.*m33*RS34e*RSe63
          - 2./81.*m33*RS34e*RS36e + 1./81.*m33*pow(RS34e,2) - 2./27.*
         m33*RSe61*RS36e + 2./27.*m33*RSe61*RS34e - 4./81.*m33*RS16e*
         RS36e + 4./81.*m33*RS16e*RS34e + 4./81.*m33*RS14e*RS36e - 4./
         81.*m33*RS14e*RS34e - 2./81.*m11*RS16e*RS36e + 2./81.*m11*
         RS16e*RS34e + 2./81.*m11*RS14e*RS36e - 2./81.*m11*RS14e*RS34e
          );

      M13p6x22RV +=  + Ab(m46,mu2)*AbV(me2,L) * ( 2./9.*me2*pow(
         RSe63,2) + 2./27.*me2*RS36e*RSe63 - 2./27.*me2*RS34e*RSe63 - 4.
         /9.*me2*RSe61*RSe63 - 2./27.*me2*RSe61*RS36e + 2./27.*me2*
         RSe61*RS34e + 2./9.*me2*pow(RSe61,2) - 2./27.*me2*RS16e*RSe63
          + 2./27.*me2*RS16e*RSe61 + 2./27.*me2*RS14e*RSe63 - 2./27.*
         me2*RS14e*RSe61 + 4./81.*m66*RS36e*RSe63 + 2./81.*m66*RS34e*
         RSe63 - 4./81.*m66*RSe61*RS36e - 2./81.*m66*RSe61*RS34e - 4./
         81.*m66*RS16e*RSe63 + 4./81.*m66*RS16e*RSe61 - 2./81.*m66*
         RS14e*RSe63 + 2./81.*m66*RS14e*RSe61 + 1./27.*m44*pow(RSe63,2)
          + 2./81.*m44*RS36e*RSe63 - 2./81.*m44*RS34e*RSe63 - 2./27.*
         m44*RSe61*RSe63 - 2./81.*m44*RSe61*RS36e + 2./81.*m44*RSe61*
         RS34e + 1./27.*m44*pow(RSe61,2) - 2./81.*m44*RS16e*RSe63 + 2./
         81.*m44*RS16e*RSe61 + 2./81.*m44*RS14e*RSe63 - 2./81.*m44*
         RS14e*RSe61 - 1./9.*m33*pow(RSe63,2) - 2./27.*m33*RS36e*RSe63
          + 2./27.*m33*RS34e*RSe63 - 2./9.*m33*RSe61*RSe63 - 2./27.*m33
         *RSe61*RS36e );

      M13p6x22RV +=  + Ab(m46,mu2)*AbV(me2,L) * ( 2./27.*m33*RSe61*
         RS34e - 2./27.*m33*RS16e*RSe63 + 2./27.*m33*RS14e*RSe63 - 2./9.
         *m11*RSe61*RSe63 - 2./27.*m11*RSe61*RS36e + 2./27.*m11*RSe61*
         RS34e - 1./9.*m11*pow(RSe61,2) - 2./27.*m11*RS16e*RSe63 - 2./
         27.*m11*RS16e*RSe61 + 2./27.*m11*RS14e*RSe63 + 2./27.*m11*
         RS14e*RSe61 );

      M13p6x22RV +=  + Ab(m46,mu2)*BbV(m11,m33,L) * ( 2./81.*m33*m66*
         RS16e*RS34e + 2./81.*m33*m66*RS14e*RS36e + 2./81.*m33*m66*
         RS14e*RS34e + 2./81.*m33*m44*RS16e*RS36e + 2./81.*m33*m44*
         RS16e*RS34e + 2./81.*m33*m44*RS14e*RS36e + 4./81.*m11*m66*
         RS16e*RS34e + 4./81.*m11*m66*RS14e*RS36e + 4./81.*m11*m66*
         RS14e*RS34e + 4./81.*m11*m44*RS16e*RS36e + 4./81.*m11*m44*
         RS16e*RS34e + 4./81.*m11*m44*RS14e*RS36e - 4./81.*m11*m33*
         RS16e*RS36e + 4./81.*m11*m33*RS16e*RS34e + 4./81.*m11*m33*
         RS14e*RS36e - 4./81.*m11*m33*RS14e*RS34e - 2./81.*pow(m11,2)*
         RS16e*RS36e + 2./81.*pow(m11,2)*RS16e*RS34e + 2./81.*pow(
         m11,2)*RS14e*RS36e - 2./81.*pow(m11,2)*RS14e*RS34e );

      M13p6x22RV +=  + Ab(m46,mu2)*BbV(m11,me2,L) * (  - 4./81.*m33*m66
         *RS16e*RSe63 - 2./81.*m33*m66*RS14e*RSe63 - 2./81.*m33*m44*
         RS16e*RSe63 + 2./81.*m33*m44*RS14e*RSe63 - 2./27.*m11*me2*
         RS16e*RSe63 + 2./27.*m11*me2*RS16e*RSe61 + 2./27.*m11*me2*
         RS14e*RSe63 - 2./27.*m11*me2*RS14e*RSe61 - 8./81.*m11*m66*
         RS16e*RSe63 - 4./81.*m11*m66*RS14e*RSe63 - 4./81.*m11*m44*
         RS16e*RSe63 + 4./81.*m11*m44*RS14e*RSe63 - 2./27.*m11*m33*
         RS16e*RSe63 + 2./27.*m11*m33*RS14e*RSe63 - 2./27.*pow(m11,2)*
         RS16e*RSe63 - 2./27.*pow(m11,2)*RS16e*RSe61 + 2./27.*pow(
         m11,2)*RS14e*RSe63 + 2./27.*pow(m11,2)*RS14e*RSe61 );

      M13p6x22RV +=  + Ab(m46,mu2)*BbV(m33,me2,L) * ( 2./27.*m33*me2*
         RS36e*RSe63 - 2./27.*m33*me2*RS34e*RSe63 - 2./27.*m33*me2*
         RSe61*RS36e + 2./27.*m33*me2*RSe61*RS34e - 8./81.*m33*m66*
         RSe61*RS36e - 4./81.*m33*m66*RSe61*RS34e - 4./81.*m33*m44*
         RSe61*RS36e + 4./81.*m33*m44*RSe61*RS34e - 2./27.*pow(m33,2)*
         RS36e*RSe63 + 2./27.*pow(m33,2)*RS34e*RSe63 - 2./27.*pow(
         m33,2)*RSe61*RS36e + 2./27.*pow(m33,2)*RSe61*RS34e - 4./81.*
         m11*m66*RSe61*RS36e - 2./81.*m11*m66*RSe61*RS34e - 2./81.*m11*
         m44*RSe61*RS36e + 2./81.*m11*m44*RSe61*RS34e - 2./27.*m11*m33*
         RSe61*RS36e + 2./27.*m11*m33*RSe61*RS34e );

      M13p6x22RV +=  + Ab(m46,mu2)*BbV(me2,L) * ( 1./9.*pow(me2,2)*pow(
         RSe63,2) - 2./9.*pow(me2,2)*RSe61*RSe63 + 1./9.*pow(me2,2)*
         pow(RSe61,2) + 1./27.*m44*me2*pow(RSe63,2) - 2./27.*m44*me2*
         RSe61*RSe63 + 1./27.*m44*me2*pow(RSe61,2) - 1./9.*m33*me2*pow(
         RSe63,2) - 2./9.*m33*me2*RSe61*RSe63 - 1./27.*m33*m44*pow(
         RSe63,2) - 2./27.*m33*m44*RSe61*RSe63 - 2./9.*m11*me2*RSe61*
         RSe63 - 1./9.*m11*me2*pow(RSe61,2) - 2./27.*m11*m44*RSe61*
         RSe63 - 1./27.*m11*m44*pow(RSe61,2) );

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(m11,L) * ( 1./135.*RD146e*
         RSe6613 - 1./270.*RD146e*RSe6633 - 1./270.*RD146e*RSe6611 - 2./
         135.*me2*RSe6613*RS1463e + 1./135.*me2*RSe6633*RS1463e + 1./
         135.*me2*RSe6611*RS1463e + 1./135.*me2*RC146e*RSe6613 - 1./270.
         *me2*RC146e*RSe6633 - 1./270.*me2*RC146e*RSe6611 - 1./81.*m66*
         pow(RS14e,2) - 1./162.*m44*pow(RS16e,2) - 13./135.*m33*RSe6613
         *RS1463e + 7./270.*m33*RSe6633*RS1463e - 1./270.*m33*RSe6611*
         RS1463e - 1./270.*m33*RC146e*RSe6613 + 17./540.*m33*RC146e*
         RSe6633 - 1./36.*m33*RC146e*RSe6611 - 1./9.*m11*RSe6613*
         RS1463e + 1./270.*m11*RSe6633*RS1463e - 11./270.*m11*RSe6611*
         RS1463e - 11./270.*m11*RC146e*RSe6613 + 1./36.*m11*RC146e*
         RSe6633 + 1./20.*m11*RC146e*RSe6611 + 2./27.*m11*m33*RN1e*
         RC146e*RSe6613 + 4./27.*pow(m11,2)*RN1e*RC146e*RSe6613 );

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(m13,L) * ( 1./81.*RD346e*RSe6613
          - 1./162.*RD346e*RSe6633 - 1./162.*RSe6611*RD346e + 1./81.*
         RD146e*RSe6613 - 1./162.*RD146e*RSe6633 - 1./162.*RD146e*
         RSe6611 - 2./27.*me2*RSe6613*RS3461e - 2./27.*me2*RSe6613*
         RS1463e + 16./81.*me2*pow(RSe6613,2) + 1./27.*me2*RSe6633*
         RS3461e + 1./27.*me2*RSe6633*RS1463e - 16./81.*me2*RSe6633*
         RSe6613 + 4./81.*me2*pow(RSe6633,2) + 1./27.*me2*RC346e*
         RSe6613 - 1./54.*me2*RC346e*RSe6633 + 1./27.*me2*RSe6611*
         RS3461e + 1./27.*me2*RSe6611*RS1463e - 16./81.*me2*RSe6611*
         RSe6613 + 8./81.*me2*RSe6611*RSe6633 - 1./54.*me2*RSe6611*
         RC346e + 4./81.*me2*pow(RSe6611,2) + 1./27.*me2*RC146e*RSe6613
          - 1./54.*me2*RC146e*RSe6633 - 1./54.*me2*RC146e*RSe6611 + 2./
         27.*m33*RSe6613*RS3461e + 8./81.*m33*RSe6613*RS1463e - 16./81.
         *m33*pow(RSe6613,2) - 4./81.*m33*RSe6633*RS3461e - 5./81.*m33*
         RSe6633*RS1463e + 20./81.*m33*RSe6633*RSe6613 - 1./9.*m33*pow(
         RSe6633,2) );

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(m13,L) * (  - 4./81.*m33*RC346e*
         RSe6613 + 4./81.*m33*RC346e*RSe6633 - 2./81.*m33*RSe6611*
         RS3461e - 1./27.*m33*RSe6611*RS1463e + 4./27.*m33*RSe6611*
         RSe6613 - 2./81.*m33*RSe6611*RSe6633 - 5./81.*m33*pow(
         RSe6611,2) - 1./27.*m33*RC146e*RSe6613 + 1./162.*m33*RC146e*
         RSe6633 + 5./162.*m33*RC146e*RSe6611 + 8./81.*m11*RSe6613*
         RS3461e + 2./27.*m11*RSe6613*RS1463e - 16./81.*m11*pow(
         RSe6613,2) - 1./27.*m11*RSe6633*RS3461e - 2./81.*m11*RSe6633*
         RS1463e + 4./27.*m11*RSe6633*RSe6613 - 5./81.*m11*pow(
         RSe6633,2) - 1./27.*m11*RC346e*RSe6613 + 5./162.*m11*RC346e*
         RSe6633 - 5./81.*m11*RSe6611*RS3461e - 4./81.*m11*RSe6611*
         RS1463e + 20./81.*m11*RSe6611*RSe6613 - 2./81.*m11*RSe6611*
         RSe6633 + 1./162.*m11*RSe6611*RC346e - 1./9.*m11*pow(
         RSe6611,2) - 4./81.*m11*RC146e*RSe6613 + 4./81.*m11*RC146e*
         RSe6611 );

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(m14,L) * (  - 1./45.*me2 - 44./
         135.*me2*RSe6613 - 19./135.*me2*RSe63 + 1./45.*me2*RSe6633 + 7.
         /135.*me2*RSe61 + 29./135.*me2*RSe6611 + 1./45.*m44 + 7./135.*
         m44*RSe63 - 1./135.*m44*RSe61 - 41./270.*m33*RSe6613 - 47./540.
         *m33*RSe63 - 17./90.*m33*RSe6633 - 1./12.*m33*RSe61 + 2./27.*
         m33*m44*RN1e*RSe63 + 19./270.*m11*RSe6613 - 5./36.*m11*RSe63
          - 1./6.*m11*RSe6633 - 49./540.*m11*RSe61 - 49./135.*m11*
         RSe6611 + 4./27.*m11*me2*RN1e*RSe6613 + 2./27.*m11*me2*RN1e*
         RSe63 - 2./27.*m11*me2*RN1e*RSe61 - 4./27.*m11*me2*RN1e*
         RSe6611 + 4./27.*m11*m44*RN1e*RSe63 + 4./27.*m11*m33*RN1e*
         RSe63 - 4./27.*pow(m11,2)*RN1e*RSe6613 + 2./9.*pow(m11,2)*RN1e
         *RSe63 + 2./27.*pow(m11,2)*RN1e*RSe61 + 4./27.*pow(m11,2)*RN1e
         *RSe6611 );

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(m16,L) * (  - 2./45.*me2 - 22./
         135.*me2*RSe6613 + 19./135.*me2*RSe63 + 1./90.*me2*RSe6633 - 7.
         /135.*me2*RSe61 + 29./270.*me2*RSe6611 + 2./45.*m66 - 7./135.*
         m66*RSe63 + 1./135.*m66*RSe61 - 41./540.*m33*RSe6613 + 47./540.
         *m33*RSe63 - 17./180.*m33*RSe6633 + 1./12.*m33*RSe61 - 2./27.*
         m33*m66*RN1e*RSe63 + 19./540.*m11*RSe6613 + 5./36.*m11*RSe63
          - 1./12.*m11*RSe6633 + 49./540.*m11*RSe61 - 49./270.*m11*
         RSe6611 + 2./27.*m11*me2*RN1e*RSe6613 - 2./27.*m11*me2*RN1e*
         RSe63 + 2./27.*m11*me2*RN1e*RSe61 - 2./27.*m11*me2*RN1e*
         RSe6611 - 4./27.*m11*m66*RN1e*RSe63 - 4./27.*m11*m33*RN1e*
         RSe63 - 2./27.*pow(m11,2)*RN1e*RSe6613 - 2./9.*pow(m11,2)*RN1e
         *RSe63 - 2./27.*pow(m11,2)*RN1e*RSe61 + 2./27.*pow(m11,2)*RN1e
         *RSe6611 );

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(m33,L) * ( 1./135.*RD346e*
         RSe6613 - 1./270.*RD346e*RSe6633 - 1./270.*RSe6611*RD346e - 2./
         135.*me2*RSe6613*RS3461e + 1./135.*me2*RSe6633*RS3461e + 1./
         135.*me2*RC346e*RSe6613 - 1./270.*me2*RC346e*RSe6633 + 1./135.
         *me2*RSe6611*RS3461e - 1./270.*me2*RSe6611*RC346e - 1./81.*m66
         *pow(RS34e,2) + 2./81.*m66*RS14e*RS34e - 1./162.*m44*pow(
         RS36e,2) + 1./81.*m44*RS16e*RS36e - 1./9.*m33*RSe6613*RS3461e
          - 11./270.*m33*RSe6633*RS3461e - 2./27.*m33*RSe6633*RS1463e
          - 11./270.*m33*RC346e*RSe6613 + 1./20.*m33*RC346e*RSe6633 + 1.
         /270.*m33*RSe6611*RS3461e + 1./36.*m33*RSe6611*RC346e + 4./27.
         *pow(m33,2)*RN3e*RC346e*RSe6613 - 13./135.*m11*RSe6613*RS3461e
          - 1./270.*m11*RSe6633*RS3461e - 1./270.*m11*RC346e*RSe6613 - 
         1./36.*m11*RC346e*RSe6633 - 13./270.*m11*RSe6611*RS3461e + 17./
         540.*m11*RSe6611*RC346e + 2./27.*m11*m33*RN3e*RC346e*RSe6613 )
         ;

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(m34,L) * (  - 1./45.*me2 - 44./
         135.*me2*RSe6613 + 7./135.*me2*RSe63 + 29./135.*me2*RSe6633 - 
         19./135.*me2*RSe61 + 1./45.*me2*RSe6611 + 1./45.*m44 - 1./135.
         *m44*RSe63 + 7./135.*m44*RSe61 + 19./270.*m33*RSe6613 - 49./
         540.*m33*RSe63 - 49./135.*m33*RSe6633 - 5./36.*m33*RSe61 - 1./
         6.*m33*RSe6611 + 4./27.*m33*me2*RN3e*RSe6613 - 2./27.*m33*me2*
         RN3e*RSe63 - 4./27.*m33*me2*RN3e*RSe6633 + 2./27.*m33*me2*RN3e
         *RSe61 + 4./27.*m33*m44*RN3e*RSe61 - 4./27.*pow(m33,2)*RN3e*
         RSe6613 + 2./27.*pow(m33,2)*RN3e*RSe63 + 4./27.*pow(m33,2)*
         RN3e*RSe6633 + 2./9.*pow(m33,2)*RN3e*RSe61 - 41./270.*m11*
         RSe6613 - 1./12.*m11*RSe63 - 47./540.*m11*RSe61 - 17./90.*m11*
         RSe6611 + 2./27.*m11*m44*RN3e*RSe61 + 4./27.*m11*m33*RN3e*
         RSe61 );

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(m36,L) * (  - 2./45.*me2 - 22./
         135.*me2*RSe6613 - 7./135.*me2*RSe63 + 29./270.*me2*RSe6633 + 
         19./135.*me2*RSe61 + 1./90.*me2*RSe6611 + 2./45.*m66 + 1./135.
         *m66*RSe63 - 7./135.*m66*RSe61 + 19./540.*m33*RSe6613 + 49./
         540.*m33*RSe63 - 49./270.*m33*RSe6633 + 5./36.*m33*RSe61 - 1./
         12.*m33*RSe6611 + 2./27.*m33*me2*RN3e*RSe6613 + 2./27.*m33*me2
         *RN3e*RSe63 - 2./27.*m33*me2*RN3e*RSe6633 - 2./27.*m33*me2*
         RN3e*RSe61 - 4./27.*m33*m66*RN3e*RSe61 - 2./27.*pow(m33,2)*
         RN3e*RSe6613 - 2./27.*pow(m33,2)*RN3e*RSe63 + 2./27.*pow(
         m33,2)*RN3e*RSe6633 - 2./9.*pow(m33,2)*RN3e*RSe61 - 41./540.*
         m11*RSe6613 + 1./12.*m11*RSe63 + 47./540.*m11*RSe61 - 17./180.
         *m11*RSe6611 - 2./27.*m11*m66*RN3e*RSe61 - 4./27.*m11*m33*RN3e
         *RSe61 );

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(m44,L) * (  - 1./18.*m44*pow(
         RSe63,2) - 1./9.*m44*RS36e*RSe63 + 1./9.*m44*RSe61*RSe63 + 1./
         9.*m44*RSe61*RS36e - 1./18.*m44*pow(RSe61,2) + 1./9.*m44*RS16e
         *RSe63 - 1./9.*m44*RS16e*RSe61 - 2./9.*m33*m44*RN3e*RSe61*
         RS36e - 1./9.*m33*m44*RN1e*RS16e*RSe63 - 1./9.*m11*m44*RN3e*
         RSe61*RS36e - 2./9.*m11*m44*RN1e*RS16e*RSe63 );

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 2./9.*me2*pow(
         RSe63,2) + 2./27.*me2*RS36e*RSe63 - 2./27.*me2*RS34e*RSe63 - 4.
         /9.*me2*RSe61*RSe63 - 2./27.*me2*RSe61*RS36e + 2./27.*me2*
         RSe61*RS34e + 2./9.*me2*pow(RSe61,2) - 2./27.*me2*RS16e*RSe63
          + 2./27.*me2*RS16e*RSe61 + 2./27.*me2*RS14e*RSe63 - 2./27.*
         me2*RS14e*RSe61 + 4./81.*m66*RS36e*RSe63 + 2./81.*m66*RS34e*
         RSe63 - 4./81.*m66*RSe61*RS36e - 2./81.*m66*RSe61*RS34e - 4./
         81.*m66*RS16e*RSe63 + 4./81.*m66*RS16e*RSe61 - 2./81.*m66*
         RS14e*RSe63 + 2./81.*m66*RS14e*RSe61 + 1./27.*m44*pow(RSe63,2)
          + 2./81.*m44*RS36e*RSe63 - 2./81.*m44*RS34e*RSe63 - 2./27.*
         m44*RSe61*RSe63 - 2./81.*m44*RSe61*RS36e + 2./81.*m44*RSe61*
         RS34e + 1./27.*m44*pow(RSe61,2) - 2./81.*m44*RS16e*RSe63 + 2./
         81.*m44*RS16e*RSe61 + 2./81.*m44*RS14e*RSe63 - 2./81.*m44*
         RS14e*RSe61 - 1./9.*m33*pow(RSe63,2) - 2./27.*m33*RS36e*RSe63
          + 2./27.*m33*RS34e*RSe63 - 2./9.*m33*RSe61*RSe63 - 2./27.*m33
         *RSe61*RS36e );

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 2./27.*m33*RSe61*
         RS34e - 2./27.*m33*RS16e*RSe63 + 2./27.*m33*RS14e*RSe63 - 2./
         27.*m33*me2*RN3e*RS36e*RSe63 + 2./27.*m33*me2*RN3e*RS34e*RSe63
          + 2./27.*m33*me2*RN3e*RSe61*RS36e - 2./27.*m33*me2*RN3e*RSe61
         *RS34e + 8./81.*m33*m66*RN3e*RSe61*RS36e + 4./81.*m33*m66*RN3e
         *RSe61*RS34e + 4./81.*m33*m66*RN1e*RS16e*RSe63 + 2./81.*m33*
         m66*RN1e*RS14e*RSe63 + 4./81.*m33*m44*RN3e*RSe61*RS36e - 4./81.
         *m33*m44*RN3e*RSe61*RS34e + 2./81.*m33*m44*RN1e*RS16e*RSe63 - 
         2./81.*m33*m44*RN1e*RS14e*RSe63 + 2./27.*pow(m33,2)*RN3e*RS36e
         *RSe63 - 2./27.*pow(m33,2)*RN3e*RS34e*RSe63 + 2./27.*pow(
         m33,2)*RN3e*RSe61*RS36e - 2./27.*pow(m33,2)*RN3e*RSe61*RS34e
          - 2./9.*m11*RSe61*RSe63 - 2./27.*m11*RSe61*RS36e + 2./27.*m11
         *RSe61*RS34e - 1./9.*m11*pow(RSe61,2) - 2./27.*m11*RS16e*RSe63
          - 2./27.*m11*RS16e*RSe61 + 2./27.*m11*RS14e*RSe63 + 2./27.*
         m11*RS14e*RSe61 + 2./27.*m11*me2*RN1e*RS16e*RSe63 - 2./27.*m11
         *me2*RN1e*RS16e*RSe61 );

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 2./27.*m11*me2*
         RN1e*RS14e*RSe63 + 2./27.*m11*me2*RN1e*RS14e*RSe61 + 4./81.*
         m11*m66*RN3e*RSe61*RS36e + 2./81.*m11*m66*RN3e*RSe61*RS34e + 8.
         /81.*m11*m66*RN1e*RS16e*RSe63 + 4./81.*m11*m66*RN1e*RS14e*
         RSe63 + 2./81.*m11*m44*RN3e*RSe61*RS36e - 2./81.*m11*m44*RN3e*
         RSe61*RS34e + 4./81.*m11*m44*RN1e*RS16e*RSe63 - 4./81.*m11*m44
         *RN1e*RS14e*RSe63 + 2./27.*m11*m33*RN3e*RSe61*RS36e - 2./27.*
         m11*m33*RN3e*RSe61*RS34e + 2./27.*m11*m33*RN1e*RS16e*RSe63 - 2.
         /27.*m11*m33*RN1e*RS14e*RSe63 + 2./27.*pow(m11,2)*RN1e*RS16e*
         RSe63 + 2./27.*pow(m11,2)*RN1e*RS16e*RSe61 - 2./27.*pow(m11,2)
         *RN1e*RS14e*RSe63 - 2./27.*pow(m11,2)*RN1e*RS14e*RSe61 );

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(me2,L) * ( 8./135.*me2*pow(
         RSe6613,2) - 8./135.*me2*RSe6633*RSe6613 + 2./135.*me2*pow(
         RSe6633,2) - 8./135.*me2*RSe6611*RSe6613 + 4./135.*me2*RSe6611
         *RSe6633 + 2./135.*me2*pow(RSe6611,2) - 8./81.*m66*pow(
         RSe63,2) + 8./81.*m66*RS34e*RSe63 + 16./81.*m66*RSe61*RSe63 - 
         8./81.*m66*RSe61*RS34e - 8./81.*m66*pow(RSe61,2) - 8./81.*m66*
         RS14e*RSe63 + 8./81.*m66*RS14e*RSe61 - 1./81.*m44*pow(RSe63,2)
          - 2./81.*m44*RS36e*RSe63 + 2./81.*m44*RSe61*RSe63 + 2./81.*
         m44*RSe61*RS36e - 1./81.*m44*pow(RSe61,2) + 2./81.*m44*RS16e*
         RSe63 - 2./81.*m44*RS16e*RSe61 + 26./135.*m33*pow(RSe6613,2)
          + 26./135.*m33*RSe6633*RSe6613 - 29./270.*m33*pow(RSe6633,2)
          + 2./135.*m33*RSe6611*RSe6613 - 17./135.*m33*RSe6611*RSe6633
          + 1./18.*m33*pow(RSe6611,2) + 8./81.*m33*m66*RN3e*RSe61*RS34e
          + 4./81.*m33*m66*RN1e*RS14e*RSe63 - 2./81.*m33*m44*RN3e*RSe61
         *RS36e - 1./81.*m33*m44*RN1e*RS16e*RSe63 - 8./27.*pow(m33,2)*
         RN3e*RSe6633*RSe6613 );

      M13p6x22RV +=  + Ab(me2,mu2)*AbV(me2,L) * ( 26./135.*m11*pow(
         RSe6613,2) + 2./135.*m11*RSe6633*RSe6613 + 1./18.*m11*pow(
         RSe6633,2) + 26./135.*m11*RSe6611*RSe6613 - 17./135.*m11*
         RSe6611*RSe6633 - 29./270.*m11*pow(RSe6611,2) + 4./81.*m11*m66
         *RN3e*RSe61*RS34e + 8./81.*m11*m66*RN1e*RS14e*RSe63 - 1./81.*
         m11*m44*RN3e*RSe61*RS36e - 2./81.*m11*m44*RN1e*RS16e*RSe63 - 4.
         /27.*m11*m33*RN3e*RSe6633*RSe6613 - 4./27.*m11*m33*RN1e*
         RSe6611*RSe6613 - 8./27.*pow(m11,2)*RN1e*RSe6611*RSe6613 );

      M13p6x22RV +=  + Ab(me2,mu2)*BbV(m11,m33,L) * ( 2./81.*m33*m66*
         RS14e*RS34e + 1./81.*m33*m44*RS16e*RS36e - 2./27.*pow(m33,2)*
         RSe6633*RS1463e + 4./81.*m11*m66*RS14e*RS34e + 2./81.*m11*m44*
         RS16e*RS36e - 4./27.*m11*m33*RSe6633*RS1463e - 2./27.*m11*m33*
         RSe6611*RS3461e - 4./27.*pow(m11,2)*RSe6611*RS3461e );

      M13p6x22RV +=  + Ab(me2,mu2)*BbV(m11,me2,L) * (  - 4./81.*m33*m66
         *RS14e*RSe63 + 1./81.*m33*m44*RS16e*RSe63 - 8./81.*m11*m66*
         RS14e*RSe63 + 2./81.*m11*m44*RS16e*RSe63 + 4./27.*m11*m33*
         RSe6611*RSe6613 + 8./27.*pow(m11,2)*RSe6611*RSe6613 );

      M13p6x22RV +=  + Ab(me2,mu2)*BbV(m11,L) * ( 1./135.*me2*RD146e*
         RSe6613 - 1./270.*me2*RD146e*RSe6633 - 1./270.*me2*RD146e*
         RSe6611 - 1./270.*m33*RD146e*RSe6613 + 17./540.*m33*RD146e*
         RSe6633 - 1./36.*m33*RD146e*RSe6611 - 11./270.*m11*RD146e*
         RSe6613 + 1./36.*m11*RD146e*RSe6633 + 1./20.*m11*RD146e*
         RSe6611 - 2./27.*m11*m33*RSe6611*RS1463e + 2./27.*m11*m33*RN1e
         *RD146e*RSe6613 - 4./27.*pow(m11,2)*RSe6611*RS1463e + 4./27.*
         pow(m11,2)*RN1e*RD146e*RSe6613 );

      M13p6x22RV +=  + Ab(me2,mu2)*BbV(m33,me2,L) * (  - 8./81.*m33*m66
         *RSe61*RS34e + 2./81.*m33*m44*RSe61*RS36e + 8./27.*pow(m33,2)*
         RSe6633*RSe6613 - 4./81.*m11*m66*RSe61*RS34e + 1./81.*m11*m44*
         RSe61*RS36e + 4./27.*m11*m33*RSe6633*RSe6613 );

      M13p6x22RV +=  + Ab(me2,mu2)*BbV(m33,L) * ( 1./135.*me2*RD346e*
         RSe6613 - 1./270.*me2*RD346e*RSe6633 - 1./270.*me2*RSe6611*
         RD346e - 11./270.*m33*RD346e*RSe6613 + 1./20.*m33*RD346e*
         RSe6633 + 1./36.*m33*RSe6611*RD346e - 4./27.*pow(m33,2)*
         RSe6633*RS3461e + 4./27.*pow(m33,2)*RN3e*RD346e*RSe6613 - 1./
         270.*m11*RD346e*RSe6613 - 1./36.*m11*RD346e*RSe6633 + 17./540.
         *m11*RSe6611*RD346e - 2./27.*m11*m33*RSe6633*RS3461e + 2./27.*
         m11*m33*RN3e*RD346e*RSe6613 );

      M13p6x22RV +=  + Ab(me2,mu2)*BbV(me2,L) * (  - 4./81.*m66*me2*
         pow(RSe63,2) + 8./81.*m66*me2*RSe61*RSe63 - 4./81.*m66*me2*
         pow(RSe61,2) - 1./162.*m44*me2*pow(RSe63,2) + 1./81.*m44*me2*
         RSe61*RSe63 - 1./162.*m44*me2*pow(RSe61,2) + 4./81.*m33*m66*
         pow(RSe63,2) + 8./81.*m33*m66*RSe61*RSe63 + 1./162.*m33*m44*
         pow(RSe63,2) + 1./81.*m33*m44*RSe61*RSe63 + 8./81.*m11*m66*
         RSe61*RSe63 + 4./81.*m11*m66*pow(RSe61,2) + 1./81.*m11*m44*
         RSe61*RSe63 + 1./162.*m11*m44*pow(RSe61,2) );

      M13p6x22RV +=  + Bb(m11,mu2)*AbV(m11,L) * ( 1./540.*pow(RD146e,2)
          + 1./540.*m33*RD146e*RS1463e + 1./72.*m33*RD146e*RC146e + 1./
         60.*m11*RD146e*RS1463e - 5./216.*m11*RD146e*RC146e + 1./27.*
         m11*m33*RC146e*RS1463e + 2./27.*pow(m11,2)*RC146e*RS1463e );

      M13p6x22RV +=  + Bb(m11,mu2)*AbV(m13,L) * ( 1./324.*RD146e*RD346e
          + 1./324.*pow(RD146e,2) + 1./81.*me2*RD146e*RSe6613 - 1./162.
         *me2*RD146e*RSe6633 - 1./162.*me2*RD146e*RSe6611 + 1./81.*m33*
         RD146e*RS3461e + 1./54.*m33*RD146e*RS1463e - 1./27.*m33*RD146e
         *RSe6613 + 1./162.*m33*RD146e*RSe6633 + 5./162.*m33*RD146e*
         RSe6611 - 5./324.*m33*RD146e*RC146e + 1./81.*m11*RD146e*
         RS3461e + 1./162.*m11*RD146e*RS1463e - 2./81.*m11*RD146e*
         RSe6613 - 1./81.*m11*RD146e*RSe6633 + 1./162.*m11*RD146e*
         RC346e + 1./27.*m11*RD146e*RSe6611 - 5./324.*m11*RD146e*RC146e
          );

      M13p6x22RV +=  + Bb(m11,mu2)*AbV(m33,L) * ( 1./540.*RD146e*RD346e
          - 1./540.*m33*RD146e*RS3461e - 1./72.*m33*RD146e*RC346e + 11./
         540.*m11*RD146e*RS3461e - 1./72.*m11*RD146e*RC346e );

      M13p6x22RV +=  + Bb(m11,mu2)*AbV(m34,L) * ( 1./12.*m33*RD146e + 1.
         /12.*m11*RD146e );

      M13p6x22RV +=  + Bb(m11,mu2)*AbV(m36,L) * ( 1./24.*m33*RD146e + 1.
         /24.*m11*RD146e );

      M13p6x22RV +=  + Bb(m11,mu2)*AbV(me2,L) * ( 1./135.*me2*RD146e*
         RSe6613 - 1./270.*me2*RD146e*RSe6633 - 1./270.*me2*RD146e*
         RSe6611 - 1./270.*m33*RD146e*RSe6613 + 17./540.*m33*RD146e*
         RSe6633 - 1./36.*m33*RD146e*RSe6611 - 11./270.*m11*RD146e*
         RSe6613 + 1./36.*m11*RD146e*RSe6633 + 1./20.*m11*RD146e*
         RSe6611 - 2./27.*m11*m33*RSe6611*RS1463e - 4./27.*pow(m11,2)*
         RSe6611*RS1463e );

      M13p6x22RV +=  + Bb(m11,mu2)*BbV(m11,m33,L) * ( 1./27.*m11*m33*
         RD146e*RS3461e + 2./27.*pow(m11,2)*RD146e*RS3461e );

      M13p6x22RV +=  + Bb(m11,mu2)*BbV(m11,me2,L) * (  - 2./27.*m11*m33
         *RD146e*RSe6613 - 4./27.*pow(m11,2)*RD146e*RSe6613 );

      M13p6x22RV +=  + Bb(m11,mu2)*BbV(m11,L) * ( 1./72.*m33*pow(
         RD146e,2) - 5./216.*m11*pow(RD146e,2) + 2./27.*m11*m33*RD146e*
         RS1463e + 4./27.*pow(m11,2)*RD146e*RS1463e );

      M13p6x22RV +=  + Bb(m11,mu2)*BbV(m33,L) * (  - 1./72.*m33*RD146e*
         RD346e - 1./72.*m11*RD146e*RD346e );

      M13p6x22RV +=  + Bb(m33,mu2)*AbV(m11,L) * ( 1./540.*RD146e*RD346e
          - 1./60.*m33*RD346e*RS1463e - 1./72.*m33*RC146e*RD346e - 1./
         540.*m11*RD346e*RS1463e - 1./72.*m11*RC146e*RD346e );

      M13p6x22RV +=  + Bb(m33,mu2)*AbV(m13,L) * ( 1./324.*pow(RD346e,2)
          + 1./324.*RD146e*RD346e + 1./81.*me2*RD346e*RSe6613 - 1./162.
         *me2*RD346e*RSe6633 - 1./162.*me2*RSe6611*RD346e + 1./162.*m33
         *RD346e*RS3461e + 1./81.*m33*RD346e*RS1463e - 2./81.*m33*
         RD346e*RSe6613 + 1./27.*m33*RD346e*RSe6633 - 5./324.*m33*
         RD346e*RC346e - 1./81.*m33*RSe6611*RD346e + 1./162.*m33*RC146e
         *RD346e + 1./54.*m11*RD346e*RS3461e + 1./81.*m11*RD346e*
         RS1463e - 1./27.*m11*RD346e*RSe6613 + 5./162.*m11*RD346e*
         RSe6633 - 5./324.*m11*RD346e*RC346e + 1./162.*m11*RSe6611*
         RD346e );

      M13p6x22RV +=  + Bb(m33,mu2)*AbV(m14,L) * ( 1./12.*m33*RD346e + 1.
         /12.*m11*RD346e );

      M13p6x22RV +=  + Bb(m33,mu2)*AbV(m16,L) * ( 1./24.*m33*RD346e + 1.
         /24.*m11*RD346e );

      M13p6x22RV +=  + Bb(m33,mu2)*AbV(m33,L) * ( 1./540.*pow(RD346e,2)
          + 1./60.*m33*RD346e*RS3461e + 1./27.*m33*RD346e*RS1463e - 5./
         216.*m33*RD346e*RC346e + 2./27.*pow(m33,2)*RC346e*RS3461e + 1./
         540.*m11*RD346e*RS3461e + 1./72.*m11*RD346e*RC346e + 1./27.*
         m11*m33*RC346e*RS3461e );

      M13p6x22RV +=  + Bb(m33,mu2)*AbV(me2,L) * ( 1./135.*me2*RD346e*
         RSe6613 - 1./270.*me2*RD346e*RSe6633 - 1./270.*me2*RSe6611*
         RD346e - 11./270.*m33*RD346e*RSe6613 + 1./20.*m33*RD346e*
         RSe6633 + 1./36.*m33*RSe6611*RD346e - 4./27.*pow(m33,2)*
         RSe6633*RS3461e - 1./270.*m11*RD346e*RSe6613 - 1./36.*m11*
         RD346e*RSe6633 + 17./540.*m11*RSe6611*RD346e - 2./27.*m11*m33*
         RSe6633*RS3461e );

      M13p6x22RV +=  + Bb(m33,mu2)*BbV(m11,m33,L) * ( 1./27.*pow(m33,2)
         *RD346e*RS1463e + 2./27.*m11*m33*RD346e*RS1463e );

      M13p6x22RV +=  + Bb(m33,mu2)*BbV(m11,L) * (  - 1./72.*m33*RD146e*
         RD346e - 1./72.*m11*RD146e*RD346e );

      M13p6x22RV +=  + Bb(m33,mu2)*BbV(m33,me2,L) * (  - 4./27.*pow(
         m33,2)*RD346e*RSe6613 - 2./27.*m11*m33*RD346e*RSe6613 );

      M13p6x22RV +=  + Bb(m33,mu2)*BbV(m33,L) * (  - 5./216.*m33*pow(
         RD346e,2) + 4./27.*pow(m33,2)*RD346e*RS3461e + 1./72.*m11*pow(
         RD346e,2) + 2./27.*m11*m33*RD346e*RS3461e );

      M13p6x22RV +=  + Bb(me2,mu2)*AbV(m44,L) * (  - 1./18.*m44*me2*
         pow(RSe63,2) + 1./9.*m44*me2*RSe61*RSe63 - 1./18.*m44*me2*pow(
         RSe61,2) + 1./18.*m33*m44*pow(RSe63,2) + 1./9.*m33*m44*RSe61*
         RSe63 + 1./9.*m11*m44*RSe61*RSe63 + 1./18.*m11*m44*pow(
         RSe61,2) );

      M13p6x22RV +=  + Bb(me2,mu2)*AbV(m46,L) * ( 1./9.*pow(me2,2)*pow(
         RSe63,2) - 2./9.*pow(me2,2)*RSe61*RSe63 + 1./9.*pow(me2,2)*
         pow(RSe61,2) + 1./27.*m44*me2*pow(RSe63,2) - 2./27.*m44*me2*
         RSe61*RSe63 + 1./27.*m44*me2*pow(RSe61,2) - 1./9.*m33*me2*pow(
         RSe63,2) - 2./9.*m33*me2*RSe61*RSe63 - 1./27.*m33*m44*pow(
         RSe63,2) - 2./27.*m33*m44*RSe61*RSe63 - 2./9.*m11*me2*RSe61*
         RSe63 - 1./9.*m11*me2*pow(RSe61,2) - 2./27.*m11*m44*RSe61*
         RSe63 - 1./27.*m11*m44*pow(RSe61,2) );

      M13p6x22RV +=  + Bb(me2,mu2)*AbV(me2,L) * (  - 4./81.*m66*me2*
         pow(RSe63,2) + 8./81.*m66*me2*RSe61*RSe63 - 4./81.*m66*me2*
         pow(RSe61,2) - 1./162.*m44*me2*pow(RSe63,2) + 1./81.*m44*me2*
         RSe61*RSe63 - 1./162.*m44*me2*pow(RSe61,2) + 4./81.*m33*m66*
         pow(RSe63,2) + 8./81.*m33*m66*RSe61*RSe63 + 1./162.*m33*m44*
         pow(RSe63,2) + 1./81.*m33*m44*RSe61*RSe63 + 8./81.*m11*m66*
         RSe61*RSe63 + 4./81.*m11*m66*pow(RSe61,2) + 1./81.*m11*m44*
         RSe61*RSe63 + 1./162.*m11*m44*pow(RSe61,2) );

      M13p6x22RV +=  + AbV(m11,L)*pi16 * (  - 1./72.*m33*RD346e*RS1463e
          + 1./144.*m33*RC146e*RD346e + 1./48.*m33*RD146e - 1./72.*m33*
         RD146e*RS1463e + 1./144.*m33*RD146e*RC146e - 1./18.*m33*me2*
         RSe6613*RS1463e + 1./36.*m33*me2*RSe6633*RS1463e + 1./36.*m33*
         me2*RSe6611*RS1463e + 1./36.*m33*me2*RC146e*RSe6613 - 1./72.*
         m33*me2*RC146e*RSe6633 - 1./72.*m33*me2*RC146e*RSe6611 + 1./24.
         *m33*m66*RS1463e - 1./48.*m33*m66*RC146e + 1./12.*m33*m44*
         RS1463e - 1./24.*m33*m44*RC146e + 1./24.*pow(m33,2)*RS1463e + 
         1./27.*pow(m33,2)*RS1463e*RS3461e + 1./108.*pow(m33,2)*pow(
         RS1463e,2) - 1./54.*pow(m33,2)*RSe6613*RS1463e + 1./108.*pow(
         m33,2)*RSe6633*RS1463e - 1./54.*pow(m33,2)*RC346e*RS1463e + 1./
         108.*pow(m33,2)*RSe6611*RS1463e - 1./48.*pow(m33,2)*RC146e - 1.
         /54.*pow(m33,2)*RC146e*RS3461e - 1./108.*pow(m33,2)*RC146e*
         RS1463e + 1./108.*pow(m33,2)*RC146e*RSe6613 - 1./216.*pow(
         m33,2)*RC146e*RSe6633 + 1./108.*pow(m33,2)*RC146e*RC346e - 1./
         216.*pow(m33,2)*RC146e*RSe6611 );

      M13p6x22RV +=  + AbV(m11,L)*pi16 * ( 1./432.*pow(m33,2)*pow(
         RC146e,2) - 1./72.*m11*RD346e*RS1463e + 1./144.*m11*RC146e*
         RD346e + 1./48.*m11*RD146e - 1./72.*m11*RD146e*RS1463e + 1./
         144.*m11*RD146e*RC146e - 1./18.*m11*me2*RSe6613*RS1463e + 1./
         36.*m11*me2*RSe6633*RS1463e + 1./36.*m11*me2*RSe6611*RS1463e
          + 1./36.*m11*me2*RC146e*RSe6613 - 1./72.*m11*me2*RC146e*
         RSe6633 - 1./72.*m11*me2*RC146e*RSe6611 + 1./24.*m11*m66*
         RS1463e - 1./48.*m11*m66*RC146e + 1./12.*m11*m44*RS1463e - 1./
         24.*m11*m44*RC146e - 1./16.*m11*m33 + 1./24.*m11*m33*RS1463e
          + 5./108.*m11*m33*RS1463e*RS3461e + 5./108.*m11*m33*pow(
         RS1463e,2) - 1./27.*m11*m33*RSe6613*RS1463e + 1./54.*m11*m33*
         RSe6633*RS1463e - 5./216.*m11*m33*RC346e*RS1463e + 1./54.*m11*
         m33*RSe6611*RS1463e + 1./24.*m11*m33*RS16e + 1./48.*m11*m33*
         RS14e - 1./48.*m11*m33*RC146e - 5./216.*m11*m33*RC146e*RS3461e
          - 5./108.*m11*m33*RC146e*RS1463e + 1./54.*m11*m33*RC146e*
         RSe6613 );

      M13p6x22RV +=  + AbV(m11,L)*pi16 * (  - 1./108.*m11*m33*RC146e*
         RSe6633 + 5./432.*m11*m33*RC146e*RC346e - 1./108.*m11*m33*
         RC146e*RSe6611 + 5./432.*m11*m33*pow(RC146e,2) - 1./16.*pow(
         m11,2) + 1./108.*pow(m11,2)*RS1463e*RS3461e + 1./27.*pow(
         m11,2)*pow(RS1463e,2) - 1./54.*pow(m11,2)*RSe6613*RS1463e + 1./
         108.*pow(m11,2)*RSe6633*RS1463e - 1./216.*pow(m11,2)*RC346e*
         RS1463e + 1./108.*pow(m11,2)*RSe6611*RS1463e + 1./24.*pow(
         m11,2)*RS16e + 1./48.*pow(m11,2)*RS14e - 1./216.*pow(m11,2)*
         RC146e*RS3461e - 1./27.*pow(m11,2)*RC146e*RS1463e + 1./108.*
         pow(m11,2)*RC146e*RSe6613 - 1./216.*pow(m11,2)*RC146e*RSe6633
          + 1./432.*pow(m11,2)*RC146e*RC346e - 1./216.*pow(m11,2)*
         RC146e*RSe6611 + 1./108.*pow(m11,2)*pow(RC146e,2) );

      M13p6x22RV +=  + pow(AbV(m11,L),2) * (  - 1./270.*RD146e*RS1463e
          + 1./540.*RD146e*RC146e + 1./16.*m33 + 13./540.*m33*pow(
         RS1463e,2) + 1./540.*m33*RC146e*RS1463e + 1./144.*m33*pow(
         RC146e,2) + 1./16.*m11 + 17./540.*m11*pow(RS1463e,2) + 1./60.*
         m11*RC146e*RS1463e - 5./432.*m11*pow(RC146e,2) );

      M13p6x22RV +=  + AbV(m11,L)*AbV(m13,L) * (  - 1./162.*RD346e*
         RS1463e + 1./324.*RC146e*RD346e - 1./54.*RD146e*RS3461e - 2./
         81.*RD146e*RS1463e + 1./27.*RD146e*RSe6613 - 1./54.*RD146e*
         RSe6633 + 1./108.*RD146e*RC346e - 1./54.*RD146e*RSe6611 + 1./
         81.*RD146e*RC146e - 2./81.*me2*RSe6613*RS1463e + 1./81.*me2*
         RSe6633*RS1463e + 1./81.*me2*RSe6611*RS1463e + 1./81.*me2*
         RC146e*RSe6613 - 1./162.*me2*RC146e*RSe6633 - 1./162.*me2*
         RC146e*RSe6611 - 1./72.*m33 - 1./27.*m33*RS1463e*RS3461e - 4./
         81.*m33*pow(RS1463e,2) + 8./81.*m33*RSe6613*RS1463e - 5./81.*
         m33*RSe6633*RS1463e + 2./81.*m33*RC346e*RS1463e - 1./27.*m33*
         RSe6611*RS1463e + 1./81.*m33*RC146e*RS3461e + 1./27.*m33*
         RC146e*RS1463e - 1./27.*m33*RC146e*RSe6613 + 1./162.*m33*
         RC146e*RSe6633 + 5./162.*m33*RC146e*RSe6611 - 5./324.*m33*pow(
         RC146e,2) + 1./24.*m11 - 1./81.*m11*RS1463e*RS3461e + 2./81.*
         m11*RSe6613*RS1463e - 2./81.*m11*RSe6611*RS1463e + 1./81.*m11*
         RC146e*RS3461e );

      M13p6x22RV +=  + AbV(m11,L)*AbV(m13,L) * ( 1./81.*m11*RC146e*
         RS1463e - 2./81.*m11*RC146e*RSe6613 - 1./81.*m11*RC146e*
         RSe6633 + 1./162.*m11*RC146e*RC346e + 1./27.*m11*RC146e*
         RSe6611 - 5./324.*m11*pow(RC146e,2) );

      M13p6x22RV +=  + AbV(m11,L)*AbV(m14,L) * (  - 29./270.*RD146e - 1.
         /135.*m44*RS16e + 41./540.*m33*RS1463e - 1./12.*m33*RS16e + 2./
         27.*m11*RS3461e + 109./540.*m11*RS1463e - 4./27.*m11*RSe6613
          - 2./27.*m11*RSe63 - 2./27.*m11*RS36e + 2./27.*m11*RSe61 + 4./
         27.*m11*RSe6611 + 19./540.*m11*RS16e );

      M13p6x22RV +=  + AbV(m11,L)*AbV(m16,L) * (  - 29./540.*RD146e - 1.
         /270.*m66*RS14e + 41./1080.*m33*RS1463e - 1./24.*m33*RS14e + 1.
         /27.*m11*RS3461e + 109./1080.*m11*RS1463e - 2./27.*m11*RSe6613
          + 2./27.*m11*RSe63 - 1./27.*m11*RS34e - 2./27.*m11*RSe61 + 2./
         27.*m11*RSe6611 + 19./1080.*m11*RS14e );

      M13p6x22RV +=  + AbV(m11,L)*AbV(m33,L) * (  - 1./270.*RD346e*
         RS1463e + 1./540.*RC146e*RD346e - 1./270.*RD146e*RS3461e + 1./
         540.*RD146e*RC346e + 1./18.*m33*RS1463e*RS3461e - 1./60.*m33*
         RC346e*RS1463e - 1./540.*m33*RC146e*RS3461e - 1./72.*m33*
         RC146e*RC346e + 1./18.*m11*RS1463e*RS3461e - 1./540.*m11*
         RC346e*RS1463e + 11./540.*m11*RC146e*RS3461e - 1./72.*m11*
         RC146e*RC346e );

      M13p6x22RV +=  + AbV(m11,L)*AbV(m34,L) * (  - 1./90.*RD146e + 17./
         135.*m44*RS16e - 19./540.*m33*RS1463e + 1./108.*m33*RS16e + 1./
         12.*m33*RC146e + 89./540.*m11*RS1463e - 83./540.*m11*RS16e + 1.
         /12.*m11*RC146e );

      M13p6x22RV +=  + AbV(m11,L)*AbV(m36,L) * (  - 1./180.*RD146e + 17.
         /270.*m66*RS14e - 19./1080.*m33*RS1463e + 1./216.*m33*RS14e + 
         1./24.*m33*RC146e + 89./1080.*m11*RS1463e - 83./1080.*m11*
         RS14e + 1./24.*m11*RC146e );

      M13p6x22RV +=  + AbV(m11,L)*AbV(m44,L) * (  - 1./18.*m44*pow(
         RS16e,2) );

      M13p6x22RV +=  + AbV(m11,L)*AbV(m46,L) * (  - 2./81.*m66*RS14e*
         RS16e - 1./81.*m66*pow(RS14e,2) - 1./81.*m44*pow(RS16e,2) - 2./
         81.*m44*RS14e*RS16e - 2./27.*m11*RS16e*RSe63 - 2./81.*m11*
         RS16e*RS36e + 2./81.*m11*RS16e*RS34e + 2./27.*m11*RS16e*RSe61
          + 1./81.*m11*pow(RS16e,2) + 2./27.*m11*RS14e*RSe63 + 2./81.*
         m11*RS14e*RS36e - 2./81.*m11*RS14e*RS34e - 2./27.*m11*RS14e*
         RSe61 - 2./81.*m11*RS14e*RS16e + 1./81.*m11*pow(RS14e,2) );

      M13p6x22RV +=  + AbV(m11,L)*AbV(me2,L) * ( 1./135.*RD146e*RSe6613
          - 1./270.*RD146e*RSe6633 - 1./270.*RD146e*RSe6611 - 2./135.*
         me2*RSe6613*RS1463e + 1./135.*me2*RSe6633*RS1463e + 1./135.*
         me2*RSe6611*RS1463e + 1./135.*me2*RC146e*RSe6613 - 1./270.*me2
         *RC146e*RSe6633 - 1./270.*me2*RC146e*RSe6611 - 1./81.*m66*pow(
         RS14e,2) - 1./162.*m44*pow(RS16e,2) - 13./135.*m33*RSe6613*
         RS1463e + 7./270.*m33*RSe6633*RS1463e - 1./270.*m33*RSe6611*
         RS1463e - 1./270.*m33*RC146e*RSe6613 + 17./540.*m33*RC146e*
         RSe6633 - 1./36.*m33*RC146e*RSe6611 - 1./9.*m11*RSe6613*
         RS1463e + 1./270.*m11*RSe6633*RS1463e - 11./270.*m11*RSe6611*
         RS1463e - 11./270.*m11*RC146e*RSe6613 + 1./36.*m11*RC146e*
         RSe6633 + 1./20.*m11*RC146e*RSe6611 );

      M13p6x22RV +=  + AbV(m11,L)*BbV(m11,m33,L) * ( 1./27.*m11*m33*
         RC146e*RS3461e + 2./27.*pow(m11,2)*RC146e*RS3461e );

      M13p6x22RV +=  + AbV(m11,L)*BbV(m11,me2,L) * (  - 2./27.*m11*m33*
         RC146e*RSe6613 - 4./27.*pow(m11,2)*RC146e*RSe6613 );

      M13p6x22RV +=  + AbV(m11,L)*BbV(m11,L) * ( 1./540.*pow(RD146e,2)
          + 1./540.*m33*RD146e*RS1463e + 1./72.*m33*RD146e*RC146e + 1./
         60.*m11*RD146e*RS1463e - 5./216.*m11*RD146e*RC146e + 1./27.*
         m11*m33*RC146e*RS1463e + 2./27.*pow(m11,2)*RC146e*RS1463e );

      M13p6x22RV +=  + AbV(m11,L)*BbV(m33,L) * ( 1./540.*RD146e*RD346e
          - 1./60.*m33*RD346e*RS1463e - 1./72.*m33*RC146e*RD346e - 1./
         540.*m11*RD346e*RS1463e - 1./72.*m11*RC146e*RD346e );

      M13p6x22RV +=  + AbV(m13,L)*pi16 * ( 1./144.*pow(m33,2)*pow(
         RS3461e,2) + 1./72.*pow(m33,2)*RS1463e*RS3461e + 1./144.*pow(
         m33,2)*pow(RS1463e,2) - 1./36.*pow(m33,2)*RSe6613*RS3461e - 1./
         36.*pow(m33,2)*RSe6613*RS1463e + 1./36.*pow(m33,2)*pow(
         RSe6613,2) + 1./72.*pow(m33,2)*RSe6633*RS3461e + 1./72.*pow(
         m33,2)*RSe6633*RS1463e - 1./36.*pow(m33,2)*RSe6633*RSe6613 + 1.
         /144.*pow(m33,2)*pow(RSe6633,2) - 1./144.*pow(m33,2)*RC346e*
         RS3461e - 1./144.*pow(m33,2)*RC346e*RS1463e + 1./72.*pow(
         m33,2)*RC346e*RSe6613 - 1./144.*pow(m33,2)*RC346e*RSe6633 + 1./
         576.*pow(m33,2)*pow(RC346e,2) + 1./72.*pow(m33,2)*RSe6611*
         RS3461e + 1./72.*pow(m33,2)*RSe6611*RS1463e - 1./36.*pow(
         m33,2)*RSe6611*RSe6613 + 1./72.*pow(m33,2)*RSe6611*RSe6633 - 1.
         /144.*pow(m33,2)*RSe6611*RC346e + 1./144.*pow(m33,2)*pow(
         RSe6611,2) - 1./144.*pow(m33,2)*RC146e*RS3461e - 1./144.*pow(
         m33,2)*RC146e*RS1463e + 1./72.*pow(m33,2)*RC146e*RSe6613 - 1./
         144.*pow(m33,2)*RC146e*RSe6633 );

      M13p6x22RV +=  + AbV(m13,L)*pi16 * ( 1./288.*pow(m33,2)*RC146e*
         RC346e - 1./144.*pow(m33,2)*RC146e*RSe6611 + 1./576.*pow(
         m33,2)*pow(RC146e,2) + 1./72.*m11*m33*pow(RS3461e,2) + 1./36.*
         m11*m33*RS1463e*RS3461e + 1./72.*m11*m33*pow(RS1463e,2) - 1./
         18.*m11*m33*RSe6613*RS3461e - 1./18.*m11*m33*RSe6613*RS1463e
          + 1./18.*m11*m33*pow(RSe6613,2) + 1./36.*m11*m33*RSe6633*
         RS3461e + 1./36.*m11*m33*RSe6633*RS1463e - 1./18.*m11*m33*
         RSe6633*RSe6613 + 1./72.*m11*m33*pow(RSe6633,2) - 1./72.*m11*
         m33*RC346e*RS3461e - 1./72.*m11*m33*RC346e*RS1463e + 1./36.*
         m11*m33*RC346e*RSe6613 - 1./72.*m11*m33*RC346e*RSe6633 + 1./
         288.*m11*m33*pow(RC346e,2) + 1./36.*m11*m33*RSe6611*RS3461e + 
         1./36.*m11*m33*RSe6611*RS1463e - 1./18.*m11*m33*RSe6611*
         RSe6613 + 1./36.*m11*m33*RSe6611*RSe6633 - 1./72.*m11*m33*
         RSe6611*RC346e + 1./72.*m11*m33*pow(RSe6611,2) - 1./72.*m11*
         m33*RC146e*RS3461e - 1./72.*m11*m33*RC146e*RS1463e + 1./36.*
         m11*m33*RC146e*RSe6613 );

      M13p6x22RV +=  + AbV(m13,L)*pi16 * (  - 1./72.*m11*m33*RC146e*
         RSe6633 + 1./144.*m11*m33*RC146e*RC346e - 1./72.*m11*m33*
         RC146e*RSe6611 + 1./288.*m11*m33*pow(RC146e,2) + 1./144.*pow(
         m11,2)*pow(RS3461e,2) + 1./72.*pow(m11,2)*RS1463e*RS3461e + 1./
         144.*pow(m11,2)*pow(RS1463e,2) - 1./36.*pow(m11,2)*RSe6613*
         RS3461e - 1./36.*pow(m11,2)*RSe6613*RS1463e + 1./36.*pow(
         m11,2)*pow(RSe6613,2) + 1./72.*pow(m11,2)*RSe6633*RS3461e + 1./
         72.*pow(m11,2)*RSe6633*RS1463e - 1./36.*pow(m11,2)*RSe6633*
         RSe6613 + 1./144.*pow(m11,2)*pow(RSe6633,2) - 1./144.*pow(
         m11,2)*RC346e*RS3461e - 1./144.*pow(m11,2)*RC346e*RS1463e + 1./
         72.*pow(m11,2)*RC346e*RSe6613 - 1./144.*pow(m11,2)*RC346e*
         RSe6633 + 1./576.*pow(m11,2)*pow(RC346e,2) + 1./72.*pow(m11,2)
         *RSe6611*RS3461e + 1./72.*pow(m11,2)*RSe6611*RS1463e - 1./36.*
         pow(m11,2)*RSe6611*RSe6613 + 1./72.*pow(m11,2)*RSe6611*RSe6633
          - 1./144.*pow(m11,2)*RSe6611*RC346e + 1./144.*pow(m11,2)*pow(
         RSe6611,2) );

      M13p6x22RV +=  + AbV(m13,L)*pi16 * (  - 1./144.*pow(m11,2)*RC146e
         *RS3461e - 1./144.*pow(m11,2)*RC146e*RS1463e + 1./72.*pow(
         m11,2)*RC146e*RSe6613 - 1./144.*pow(m11,2)*RC146e*RSe6633 + 1./
         288.*pow(m11,2)*RC146e*RC346e - 1./144.*pow(m11,2)*RC146e*
         RSe6611 + 1./576.*pow(m11,2)*pow(RC146e,2) );

      M13p6x22RV +=  + pow(AbV(m13,L),2) * (  - 1./8.*m33 - 1./8.*m11 )
         ;

      M13p6x22RV +=  + AbV(m13,L)*AbV(m33,L) * (  - 2./81.*RD346e*
         RS3461e - 1./54.*RD346e*RS1463e + 1./27.*RD346e*RSe6613 - 1./
         54.*RD346e*RSe6633 + 1./81.*RD346e*RC346e - 1./54.*RSe6611*
         RD346e + 1./108.*RC146e*RD346e - 1./162.*RD146e*RS3461e + 1./
         324.*RD146e*RC346e - 2./81.*me2*RSe6613*RS3461e + 1./81.*me2*
         RSe6633*RS3461e + 1./81.*me2*RC346e*RSe6613 - 1./162.*me2*
         RC346e*RSe6633 + 1./81.*me2*RSe6611*RS3461e - 1./162.*me2*
         RSe6611*RC346e + 19./72.*m33 - 1./81.*m33*RS1463e*RS3461e + 2./
         81.*m33*RSe6613*RS3461e - 2./81.*m33*RSe6633*RS3461e + 1./81.*
         m33*RC346e*RS3461e + 1./81.*m33*RC346e*RS1463e - 2./81.*m33*
         RC346e*RSe6613 + 1./27.*m33*RC346e*RSe6633 - 5./324.*m33*pow(
         RC346e,2) - 1./81.*m33*RSe6611*RC346e + 1./162.*m33*RC146e*
         RC346e + 5./24.*m11 - 4./81.*m11*pow(RS3461e,2) - 1./27.*m11*
         RS1463e*RS3461e + 8./81.*m11*RSe6613*RS3461e - 1./27.*m11*
         RSe6633*RS3461e + 1./27.*m11*RC346e*RS3461e + 1./81.*m11*
         RC346e*RS1463e );

      M13p6x22RV +=  + AbV(m13,L)*AbV(m33,L) * (  - 1./27.*m11*RC346e*
         RSe6613 + 5./162.*m11*RC346e*RSe6633 - 5./324.*m11*pow(
         RC346e,2) - 5./81.*m11*RSe6611*RS3461e + 1./162.*m11*RSe6611*
         RC346e + 2./81.*m11*RC146e*RS3461e );

      M13p6x22RV +=  + AbV(m13,L)*AbV(me2,L) * ( 1./81.*RD346e*RSe6613
          - 1./162.*RD346e*RSe6633 - 1./162.*RSe6611*RD346e + 1./81.*
         RD146e*RSe6613 - 1./162.*RD146e*RSe6633 - 1./162.*RD146e*
         RSe6611 - 2./27.*me2*RSe6613*RS3461e - 2./27.*me2*RSe6613*
         RS1463e + 16./81.*me2*pow(RSe6613,2) + 1./27.*me2*RSe6633*
         RS3461e + 1./27.*me2*RSe6633*RS1463e - 16./81.*me2*RSe6633*
         RSe6613 + 4./81.*me2*pow(RSe6633,2) + 1./27.*me2*RC346e*
         RSe6613 - 1./54.*me2*RC346e*RSe6633 + 1./27.*me2*RSe6611*
         RS3461e + 1./27.*me2*RSe6611*RS1463e - 16./81.*me2*RSe6611*
         RSe6613 + 8./81.*me2*RSe6611*RSe6633 - 1./54.*me2*RSe6611*
         RC346e + 4./81.*me2*pow(RSe6611,2) + 1./27.*me2*RC146e*RSe6613
          - 1./54.*me2*RC146e*RSe6633 - 1./54.*me2*RC146e*RSe6611 + 2./
         27.*m33*RSe6613*RS3461e + 8./81.*m33*RSe6613*RS1463e - 16./81.
         *m33*pow(RSe6613,2) - 4./81.*m33*RSe6633*RS3461e - 5./81.*m33*
         RSe6633*RS1463e + 20./81.*m33*RSe6633*RSe6613 - 1./9.*m33*pow(
         RSe6633,2) );

      M13p6x22RV +=  + AbV(m13,L)*AbV(me2,L) * (  - 4./81.*m33*RC346e*
         RSe6613 + 4./81.*m33*RC346e*RSe6633 - 2./81.*m33*RSe6611*
         RS3461e - 1./27.*m33*RSe6611*RS1463e + 4./27.*m33*RSe6611*
         RSe6613 - 2./81.*m33*RSe6611*RSe6633 - 5./81.*m33*pow(
         RSe6611,2) - 1./27.*m33*RC146e*RSe6613 + 1./162.*m33*RC146e*
         RSe6633 + 5./162.*m33*RC146e*RSe6611 + 8./81.*m11*RSe6613*
         RS3461e + 2./27.*m11*RSe6613*RS1463e - 16./81.*m11*pow(
         RSe6613,2) - 1./27.*m11*RSe6633*RS3461e - 2./81.*m11*RSe6633*
         RS1463e + 4./27.*m11*RSe6633*RSe6613 - 5./81.*m11*pow(
         RSe6633,2) - 1./27.*m11*RC346e*RSe6613 + 5./162.*m11*RC346e*
         RSe6633 - 5./81.*m11*RSe6611*RS3461e - 4./81.*m11*RSe6611*
         RS1463e + 20./81.*m11*RSe6611*RSe6613 - 2./81.*m11*RSe6611*
         RSe6633 + 1./162.*m11*RSe6611*RC346e - 1./9.*m11*pow(
         RSe6611,2) - 4./81.*m11*RC146e*RSe6613 + 4./81.*m11*RC146e*
         RSe6611 );

      M13p6x22RV +=  + AbV(m13,L)*BbV(m11,m33,L) * ( 1./9.*pow(m33,2)
          + 5./9.*m11*m33 + 1./3.*pow(m11,2) );

      M13p6x22RV +=  + AbV(m13,L)*BbV(m11,L) * ( 1./324.*RD146e*RD346e
          + 1./324.*pow(RD146e,2) + 1./81.*me2*RD146e*RSe6613 - 1./162.
         *me2*RD146e*RSe6633 - 1./162.*me2*RD146e*RSe6611 + 1./81.*m33*
         RD146e*RS3461e + 1./54.*m33*RD146e*RS1463e - 1./27.*m33*RD146e
         *RSe6613 + 1./162.*m33*RD146e*RSe6633 + 5./162.*m33*RD146e*
         RSe6611 - 5./324.*m33*RD146e*RC146e + 1./81.*m11*RD146e*
         RS3461e + 1./162.*m11*RD146e*RS1463e - 2./81.*m11*RD146e*
         RSe6613 - 1./81.*m11*RD146e*RSe6633 + 1./162.*m11*RD146e*
         RC346e + 1./27.*m11*RD146e*RSe6611 - 5./324.*m11*RD146e*RC146e
          );

      M13p6x22RV +=  + AbV(m13,L)*BbV(m33,L) * ( 1./324.*pow(RD346e,2)
          + 1./324.*RD146e*RD346e + 1./81.*me2*RD346e*RSe6613 - 1./162.
         *me2*RD346e*RSe6633 - 1./162.*me2*RSe6611*RD346e + 1./162.*m33
         *RD346e*RS3461e + 1./81.*m33*RD346e*RS1463e - 2./81.*m33*
         RD346e*RSe6613 + 1./27.*m33*RD346e*RSe6633 - 5./324.*m33*
         RD346e*RC346e - 1./81.*m33*RSe6611*RD346e + 1./162.*m33*RC146e
         *RD346e + 1./54.*m11*RD346e*RS3461e + 1./81.*m11*RD346e*
         RS1463e - 1./27.*m11*RD346e*RSe6613 + 5./162.*m11*RD346e*
         RSe6633 - 5./324.*m11*RD346e*RC346e + 1./162.*m11*RSe6611*
         RD346e );

      M13p6x22RV +=  + AbV(m14,L)*pi16 * (  - 1./24.*m33*RD346e + 1./48.
         *m33*me2 + 1./12.*m33*me2*RSe63 + 1./12.*m33*me2*RSe6633 + 1./
         8.*m33*m66 + 7./24.*m33*m44 + 1./48.*m33*m44*RS3461e + 1./48.*
         m33*m44*RS1463e - 1./24.*m33*m44*RSe6613 + 1./48.*m33*m44*
         RSe63 + 1./24.*m33*m44*RSe6633 + 1./48.*m33*m44*RS36e - 1./48.
         *m33*m44*RC346e + 1./48.*m33*m44*RSe61 + 1./24.*m33*m44*
         RSe6611 + 1./48.*m33*m44*RS16e - 1./48.*m33*m44*RC146e + 1./9.
         *pow(m33,2) + 1./36.*pow(m33,2)*RSe63 + 1./36.*pow(m33,2)*
         RSe6633 + 1./9.*pow(m33,2)*RS36e - 1./18.*pow(m33,2)*RC346e - 
         1./24.*m11*RD346e + 1./48.*m11*me2 + 1./12.*m11*me2*RSe63 + 1./
         12.*m11*me2*RSe6633 + 1./8.*m11*m66 + 7./24.*m11*m44 + 1./48.*
         m11*m44*RS3461e + 1./48.*m11*m44*RS1463e - 1./24.*m11*m44*
         RSe6613 + 1./48.*m11*m44*RSe63 + 1./24.*m11*m44*RSe6633 + 1./
         48.*m11*m44*RS36e - 1./48.*m11*m44*RC346e + 1./48.*m11*m44*
         RSe61 + 1./24.*m11*m44*RSe6611 + 1./48.*m11*m44*RS16e - 1./48.
         *m11*m44*RC146e );

      M13p6x22RV +=  + AbV(m14,L)*pi16 * (  - 1./144.*m11*m33 + 1./48.*
         m11*m33*RS3461e + 1./48.*m11*m33*RS1463e - 1./24.*m11*m33*
         RSe6613 - 1./144.*m11*m33*RSe63 + 1./72.*m11*m33*RSe6633 + 11./
         144.*m11*m33*RS36e - 7./144.*m11*m33*RC346e + 1./48.*m11*m33*
         RSe61 + 1./24.*m11*m33*RSe6611 + 1./48.*m11*m33*RS16e - 1./48.
         *m11*m33*RC146e - 17./144.*pow(m11,2) + 1./48.*pow(m11,2)*
         RS3461e + 1./48.*pow(m11,2)*RS1463e - 1./24.*pow(m11,2)*
         RSe6613 - 5./144.*pow(m11,2)*RSe63 - 1./72.*pow(m11,2)*RSe6633
          - 5./144.*pow(m11,2)*RS36e + 1./144.*pow(m11,2)*RC346e + 1./
         48.*pow(m11,2)*RSe61 + 1./24.*pow(m11,2)*RSe6611 + 1./48.*pow(
         m11,2)*RS16e - 1./48.*pow(m11,2)*RC146e );

      M13p6x22RV +=  + AbV(m14,L)*AbV(m33,L) * (  - 1./90.*RD346e + 7./
         135.*m44*RS36e + 43./180.*m33*RS3461e - 41./180.*m33*RS36e + 1.
         /12.*m33*RC346e - 19./540.*m11*RS3461e - 5./36.*m11*RS36e + 1./
         12.*m11*RC346e );

      M13p6x22RV +=  + AbV(m14,L)*AbV(m34,L) * (  - 1./54.*RD346e - 1./
         54.*RD146e + 1./27.*me2 + 2./27.*me2*RSe6613 + 2./27.*me2*
         RSe63 + 1./27.*me2*RSe6633 + 2./27.*me2*RSe61 + 1./27.*me2*
         RSe6611 - 1./27.*m44 - 1./27.*m44*RS3461e - 1./27.*m44*RS1463e
          + 2./27.*m44*RSe6613 + 1./27.*m44*RSe6633 - 1./54.*m44*RC346e
          + 1./27.*m44*RSe6611 - 1./54.*m44*RC146e - 47./108.*m33 - 13./
         108.*m33*RS3461e - 1./12.*m33*RS1463e + 1./6.*m33*RSe6613 - 1./
         36.*m33*RSe63 - 1./27.*m33*RSe6633 + 5./108.*m33*RS36e + 5./
         108.*m33*RSe61 + 1./27.*m33*RSe6611 + 5./108.*m33*RS16e - 1./
         54.*m33*RC146e - 47./108.*m11 - 1./12.*m11*RS3461e - 13./108.*
         m11*RS1463e + 1./6.*m11*RSe6613 + 5./108.*m11*RSe63 + 1./27.*
         m11*RSe6633 + 5./108.*m11*RS36e - 1./54.*m11*RC346e - 1./36.*
         m11*RSe61 - 1./27.*m11*RSe6611 + 5./108.*m11*RS16e );

      M13p6x22RV +=  + AbV(m14,L)*AbV(m36,L) * (  - 1./4.*m33 - 1./4.*
         m11 );

      M13p6x22RV +=  + AbV(m14,L)*AbV(me2,L) * (  - 1./45.*me2 - 44./
         135.*me2*RSe6613 - 19./135.*me2*RSe63 + 1./45.*me2*RSe6633 + 7.
         /135.*me2*RSe61 + 29./135.*me2*RSe6611 + 1./45.*m44 + 7./135.*
         m44*RSe63 - 1./135.*m44*RSe61 - 41./270.*m33*RSe6613 - 47./540.
         *m33*RSe63 - 17./90.*m33*RSe6633 - 1./12.*m33*RSe61 + 19./270.
         *m11*RSe6613 - 5./36.*m11*RSe63 - 1./6.*m11*RSe6633 - 49./540.
         *m11*RSe61 - 49./135.*m11*RSe6611 );

      M13p6x22RV +=  + AbV(m14,L)*BbV(m11,m33,L) * (  - 2./27.*m33*m44*
         RS36e - 4./27.*m11*m44*RS36e + 2./27.*m11*m33*RS3461e - 2./9.*
         m11*m33*RS36e - 2./27.*pow(m11,2)*RS3461e - 2./9.*pow(m11,2)*
         RS36e );

      M13p6x22RV +=  + AbV(m14,L)*BbV(m11,me2,L) * (  - 2./27.*m33*m44*
         RSe63 - 4./27.*m11*me2*RSe6613 - 2./27.*m11*me2*RSe63 + 2./27.
         *m11*me2*RSe61 + 4./27.*m11*me2*RSe6611 - 4./27.*m11*m44*RSe63
          - 4./27.*m11*m33*RSe63 + 4./27.*pow(m11,2)*RSe6613 - 2./9.*
         pow(m11,2)*RSe63 - 2./27.*pow(m11,2)*RSe61 - 4./27.*pow(m11,2)
         *RSe6611 );

      M13p6x22RV +=  + AbV(m14,L)*BbV(m33,L) * ( 1./12.*m33*RD346e + 1./
         12.*m11*RD346e );

      M13p6x22RV +=  + AbV(m16,L)*pi16 * (  - 1./48.*m33*RD346e + 1./24.
         *m33*me2 - 1./12.*m33*me2*RSe63 + 1./24.*m33*me2*RSe6633 + 5./
         96.*m33*m66 + 1./96.*m33*m66*RS3461e + 1./96.*m33*m66*RS1463e
          - 1./48.*m33*m66*RSe6613 - 1./48.*m33*m66*RSe63 + 1./48.*m33*
         m66*RSe6633 + 1./96.*m33*m66*RS34e - 1./96.*m33*m66*RC346e - 1.
         /48.*m33*m66*RSe61 + 1./48.*m33*m66*RSe6611 + 1./96.*m33*m66*
         RS14e - 1./96.*m33*m66*RC146e + 1./8.*m33*m44 + 1./18.*pow(
         m33,2) - 1./36.*pow(m33,2)*RSe63 + 1./72.*pow(m33,2)*RSe6633
          + 1./18.*pow(m33,2)*RS34e - 1./36.*pow(m33,2)*RC346e - 1./48.
         *m11*RD346e + 1./24.*m11*me2 - 1./12.*m11*me2*RSe63 + 1./24.*
         m11*me2*RSe6633 + 5./96.*m11*m66 + 1./96.*m11*m66*RS3461e + 1./
         96.*m11*m66*RS1463e - 1./48.*m11*m66*RSe6613 - 1./48.*m11*m66*
         RSe63 + 1./48.*m11*m66*RSe6633 + 1./96.*m11*m66*RS34e - 1./96.
         *m11*m66*RC346e - 1./48.*m11*m66*RSe61 + 1./48.*m11*m66*
         RSe6611 + 1./96.*m11*m66*RS14e - 1./96.*m11*m66*RC146e + 1./8.
         *m11*m44 );

      M13p6x22RV +=  + AbV(m16,L)*pi16 * (  - 1./288.*m11*m33 + 1./96.*
         m11*m33*RS3461e + 1./96.*m11*m33*RS1463e - 1./48.*m11*m33*
         RSe6613 + 1./144.*m11*m33*RSe63 + 1./144.*m11*m33*RSe6633 + 11.
         /288.*m11*m33*RS34e - 7./288.*m11*m33*RC346e - 1./48.*m11*m33*
         RSe61 + 1./48.*m11*m33*RSe6611 + 1./96.*m11*m33*RS14e - 1./96.
         *m11*m33*RC146e - 17./288.*pow(m11,2) + 1./96.*pow(m11,2)*
         RS3461e + 1./96.*pow(m11,2)*RS1463e - 1./48.*pow(m11,2)*
         RSe6613 + 5./144.*pow(m11,2)*RSe63 - 1./144.*pow(m11,2)*
         RSe6633 - 5./288.*pow(m11,2)*RS34e + 1./288.*pow(m11,2)*RC346e
          - 1./48.*pow(m11,2)*RSe61 + 1./48.*pow(m11,2)*RSe6611 + 1./96.
         *pow(m11,2)*RS14e - 1./96.*pow(m11,2)*RC146e );

      M13p6x22RV +=  + AbV(m16,L)*AbV(m33,L) * (  - 1./180.*RD346e + 7./
         270.*m66*RS34e + 43./360.*m33*RS3461e - 41./360.*m33*RS34e + 1.
         /24.*m33*RC346e - 19./1080.*m11*RS3461e - 5./72.*m11*RS34e + 1.
         /24.*m11*RC346e );

      M13p6x22RV +=  + AbV(m16,L)*AbV(m34,L) * (  - 1./4.*m33 - 1./4.*
         m11 );

      M13p6x22RV +=  + AbV(m16,L)*AbV(m36,L) * (  - 1./108.*RD346e - 1./
         108.*RD146e + 2./27.*me2 + 1./27.*me2*RSe6613 - 2./27.*me2*
         RSe63 + 1./54.*me2*RSe6633 - 2./27.*me2*RSe61 + 1./54.*me2*
         RSe6611 - 2./27.*m66 - 1./54.*m66*RS3461e - 1./54.*m66*RS1463e
          + 1./27.*m66*RSe6613 + 1./54.*m66*RSe6633 - 1./108.*m66*
         RC346e + 1./54.*m66*RSe6611 - 1./108.*m66*RC146e - 5./54.*m33
          - 13./216.*m33*RS3461e - 1./24.*m33*RS1463e + 1./12.*m33*
         RSe6613 + 1./36.*m33*RSe63 - 1./54.*m33*RSe6633 + 5./216.*m33*
         RS34e - 5./108.*m33*RSe61 + 1./54.*m33*RSe6611 + 5./216.*m33*
         RS14e - 1./108.*m33*RC146e - 5./54.*m11 - 1./24.*m11*RS3461e
          - 13./216.*m11*RS1463e + 1./12.*m11*RSe6613 - 5./108.*m11*
         RSe63 + 1./54.*m11*RSe6633 + 5./216.*m11*RS34e - 1./108.*m11*
         RC346e + 1./36.*m11*RSe61 - 1./54.*m11*RSe6611 + 5./216.*m11*
         RS14e );

      M13p6x22RV +=  + AbV(m16,L)*AbV(me2,L) * (  - 2./45.*me2 - 22./
         135.*me2*RSe6613 + 19./135.*me2*RSe63 + 1./90.*me2*RSe6633 - 7.
         /135.*me2*RSe61 + 29./270.*me2*RSe6611 + 2./45.*m66 - 7./135.*
         m66*RSe63 + 1./135.*m66*RSe61 - 41./540.*m33*RSe6613 + 47./540.
         *m33*RSe63 - 17./180.*m33*RSe6633 + 1./12.*m33*RSe61 + 19./540.
         *m11*RSe6613 + 5./36.*m11*RSe63 - 1./12.*m11*RSe6633 + 49./540.
         *m11*RSe61 - 49./270.*m11*RSe6611 );

      M13p6x22RV +=  + AbV(m16,L)*BbV(m11,m33,L) * (  - 1./27.*m33*m66*
         RS34e - 2./27.*m11*m66*RS34e + 1./27.*m11*m33*RS3461e - 1./9.*
         m11*m33*RS34e - 1./27.*pow(m11,2)*RS3461e - 1./9.*pow(m11,2)*
         RS34e );

      M13p6x22RV +=  + AbV(m16,L)*BbV(m11,me2,L) * ( 2./27.*m33*m66*
         RSe63 - 2./27.*m11*me2*RSe6613 + 2./27.*m11*me2*RSe63 - 2./27.
         *m11*me2*RSe61 + 2./27.*m11*me2*RSe6611 + 4./27.*m11*m66*RSe63
          + 4./27.*m11*m33*RSe63 + 2./27.*pow(m11,2)*RSe6613 + 2./9.*
         pow(m11,2)*RSe63 + 2./27.*pow(m11,2)*RSe61 - 2./27.*pow(m11,2)
         *RSe6611 );

      M13p6x22RV +=  + AbV(m16,L)*BbV(m33,L) * ( 1./24.*m33*RD346e + 1./
         24.*m11*RD346e );

      M13p6x22RV +=  + AbV(m33,L)*pi16 * ( 1./48.*m33*RD346e - 1./72.*
         m33*RD346e*RS3461e + 1./144.*m33*RD346e*RC346e - 1./72.*m33*
         RD146e*RS3461e + 1./144.*m33*RD146e*RC346e - 1./18.*m33*me2*
         RSe6613*RS3461e + 1./36.*m33*me2*RSe6633*RS3461e + 1./36.*m33*
         me2*RC346e*RSe6613 - 1./72.*m33*me2*RC346e*RSe6633 + 1./36.*
         m33*me2*RSe6611*RS3461e - 1./72.*m33*me2*RSe6611*RC346e + 1./
         24.*m33*m66*RS3461e - 1./48.*m33*m66*RC346e + 1./12.*m33*m44*
         RS3461e - 1./24.*m33*m44*RC346e - 1./16.*pow(m33,2) + 1./27.*
         pow(m33,2)*pow(RS3461e,2) + 1./108.*pow(m33,2)*RS1463e*RS3461e
          - 1./54.*pow(m33,2)*RSe6613*RS3461e + 1./108.*pow(m33,2)*
         RSe6633*RS3461e + 1./24.*pow(m33,2)*RS36e + 1./48.*pow(m33,2)*
         RS34e - 1./27.*pow(m33,2)*RC346e*RS3461e - 1./216.*pow(m33,2)*
         RC346e*RS1463e + 1./108.*pow(m33,2)*RC346e*RSe6613 - 1./216.*
         pow(m33,2)*RC346e*RSe6633 + 1./108.*pow(m33,2)*pow(RC346e,2)
          + 1./108.*pow(m33,2)*RSe6611*RS3461e - 1./216.*pow(m33,2)*
         RSe6611*RC346e );

      M13p6x22RV +=  + AbV(m33,L)*pi16 * (  - 1./216.*pow(m33,2)*RC146e
         *RS3461e + 1./432.*pow(m33,2)*RC146e*RC346e + 1./48.*m11*
         RD346e - 1./72.*m11*RD346e*RS3461e + 1./144.*m11*RD346e*RC346e
          - 1./72.*m11*RD146e*RS3461e + 1./144.*m11*RD146e*RC346e - 1./
         18.*m11*me2*RSe6613*RS3461e + 1./36.*m11*me2*RSe6633*RS3461e
          + 1./36.*m11*me2*RC346e*RSe6613 - 1./72.*m11*me2*RC346e*
         RSe6633 + 1./36.*m11*me2*RSe6611*RS3461e - 1./72.*m11*me2*
         RSe6611*RC346e + 1./24.*m11*m66*RS3461e - 1./48.*m11*m66*
         RC346e + 1./12.*m11*m44*RS3461e - 1./24.*m11*m44*RC346e - 1./
         16.*m11*m33 + 1./24.*m11*m33*RS3461e + 5./108.*m11*m33*pow(
         RS3461e,2) + 5./108.*m11*m33*RS1463e*RS3461e - 1./27.*m11*m33*
         RSe6613*RS3461e + 1./54.*m11*m33*RSe6633*RS3461e + 1./24.*m11*
         m33*RS36e + 1./48.*m11*m33*RS34e - 1./48.*m11*m33*RC346e - 5./
         108.*m11*m33*RC346e*RS3461e - 5./216.*m11*m33*RC346e*RS1463e
          + 1./54.*m11*m33*RC346e*RSe6613 - 1./108.*m11*m33*RC346e*
         RSe6633 );

      M13p6x22RV +=  + AbV(m33,L)*pi16 * ( 5./432.*m11*m33*pow(
         RC346e,2) + 1./54.*m11*m33*RSe6611*RS3461e - 1./108.*m11*m33*
         RSe6611*RC346e - 5./216.*m11*m33*RC146e*RS3461e + 5./432.*m11*
         m33*RC146e*RC346e + 1./24.*pow(m11,2)*RS3461e + 1./108.*pow(
         m11,2)*pow(RS3461e,2) + 1./27.*pow(m11,2)*RS1463e*RS3461e - 1./
         54.*pow(m11,2)*RSe6613*RS3461e + 1./108.*pow(m11,2)*RSe6633*
         RS3461e - 1./48.*pow(m11,2)*RC346e - 1./108.*pow(m11,2)*RC346e
         *RS3461e - 1./54.*pow(m11,2)*RC346e*RS1463e + 1./108.*pow(
         m11,2)*RC346e*RSe6613 - 1./216.*pow(m11,2)*RC346e*RSe6633 + 1./
         432.*pow(m11,2)*pow(RC346e,2) + 1./108.*pow(m11,2)*RSe6611*
         RS3461e - 1./216.*pow(m11,2)*RSe6611*RC346e - 1./54.*pow(
         m11,2)*RC146e*RS3461e + 1./108.*pow(m11,2)*RC146e*RC346e );

      M13p6x22RV +=  + pow(AbV(m33,L),2) * (  - 1./270.*RD346e*RS3461e
          + 1./540.*RD346e*RC346e + 1./16.*m33 + 17./540.*m33*pow(
         RS3461e,2) + 1./60.*m33*RC346e*RS3461e + 1./27.*m33*RC346e*
         RS1463e - 5./432.*m33*pow(RC346e,2) + 1./16.*m11 + 13./540.*
         m11*pow(RS3461e,2) + 1./540.*m11*RC346e*RS3461e + 1./144.*m11*
         pow(RC346e,2) );

      M13p6x22RV +=  + AbV(m33,L)*AbV(m34,L) * (  - 29./270.*RD346e - 1.
         /135.*m44*RS36e - 2./27.*m44*RS16e + 109./540.*m33*RS3461e + 2.
         /27.*m33*RS1463e - 4./27.*m33*RSe6613 + 2./27.*m33*RSe63 + 4./
         27.*m33*RSe6633 + 19./540.*m33*RS36e - 2./27.*m33*RSe61 - 2./9.
         *m33*RS16e + 41./540.*m11*RS3461e + 2./27.*m11*RS1463e - 1./12.
         *m11*RS36e - 2./27.*m11*RS16e );

      M13p6x22RV +=  + AbV(m33,L)*AbV(m36,L) * (  - 29./540.*RD346e - 1.
         /270.*m66*RS34e - 1./27.*m66*RS14e + 109./1080.*m33*RS3461e + 
         1./27.*m33*RS1463e - 2./27.*m33*RSe6613 - 2./27.*m33*RSe63 + 2.
         /27.*m33*RSe6633 + 19./1080.*m33*RS34e + 2./27.*m33*RSe61 - 1./
         9.*m33*RS14e + 41./1080.*m11*RS3461e + 1./27.*m11*RS1463e - 1./
         24.*m11*RS34e - 1./27.*m11*RS14e );

      M13p6x22RV +=  + AbV(m33,L)*AbV(m44,L) * (  - 1./18.*m44*pow(
         RS36e,2) + 1./9.*m44*RS16e*RS36e );

      M13p6x22RV +=  + AbV(m33,L)*AbV(m46,L) * (  - 2./81.*m66*RS34e*
         RS36e - 1./81.*m66*pow(RS34e,2) + 2./81.*m66*RS16e*RS34e + 2./
         81.*m66*RS14e*RS36e + 2./81.*m66*RS14e*RS34e - 1./81.*m44*pow(
         RS36e,2) - 2./81.*m44*RS34e*RS36e + 2./81.*m44*RS16e*RS36e + 2.
         /81.*m44*RS16e*RS34e + 2./81.*m44*RS14e*RS36e + 2./27.*m33*
         RS36e*RSe63 + 1./81.*m33*pow(RS36e,2) - 2./27.*m33*RS34e*RSe63
          - 2./81.*m33*RS34e*RS36e + 1./81.*m33*pow(RS34e,2) - 2./27.*
         m33*RSe61*RS36e + 2./27.*m33*RSe61*RS34e - 4./81.*m33*RS16e*
         RS36e + 4./81.*m33*RS16e*RS34e + 4./81.*m33*RS14e*RS36e - 4./
         81.*m33*RS14e*RS34e - 2./81.*m11*RS16e*RS36e + 2./81.*m11*
         RS16e*RS34e + 2./81.*m11*RS14e*RS36e - 2./81.*m11*RS14e*RS34e
          );

      M13p6x22RV +=  + AbV(m33,L)*AbV(me2,L) * ( 1./135.*RD346e*RSe6613
          - 1./270.*RD346e*RSe6633 - 1./270.*RSe6611*RD346e - 2./135.*
         me2*RSe6613*RS3461e + 1./135.*me2*RSe6633*RS3461e + 1./135.*
         me2*RC346e*RSe6613 - 1./270.*me2*RC346e*RSe6633 + 1./135.*me2*
         RSe6611*RS3461e - 1./270.*me2*RSe6611*RC346e - 1./81.*m66*pow(
         RS34e,2) + 2./81.*m66*RS14e*RS34e - 1./162.*m44*pow(RS36e,2)
          + 1./81.*m44*RS16e*RS36e - 1./9.*m33*RSe6613*RS3461e - 11./
         270.*m33*RSe6633*RS3461e - 2./27.*m33*RSe6633*RS1463e - 11./
         270.*m33*RC346e*RSe6613 + 1./20.*m33*RC346e*RSe6633 + 1./270.*
         m33*RSe6611*RS3461e + 1./36.*m33*RSe6611*RC346e - 13./135.*m11
         *RSe6613*RS3461e - 1./270.*m11*RSe6633*RS3461e - 1./270.*m11*
         RC346e*RSe6613 - 1./36.*m11*RC346e*RSe6633 - 13./270.*m11*
         RSe6611*RS3461e + 17./540.*m11*RSe6611*RC346e );

      M13p6x22RV +=  + AbV(m33,L)*BbV(m11,m33,L) * ( 1./27.*pow(m33,2)*
         RC346e*RS1463e + 2./27.*m11*m33*RC346e*RS1463e );

      M13p6x22RV +=  + AbV(m33,L)*BbV(m11,L) * ( 1./540.*RD146e*RD346e
          - 1./540.*m33*RD146e*RS3461e - 1./72.*m33*RD146e*RC346e + 11./
         540.*m11*RD146e*RS3461e - 1./72.*m11*RD146e*RC346e );

      M13p6x22RV +=  + AbV(m33,L)*BbV(m33,me2,L) * (  - 4./27.*pow(
         m33,2)*RC346e*RSe6613 - 2./27.*m11*m33*RC346e*RSe6613 );

      M13p6x22RV +=  + AbV(m33,L)*BbV(m33,L) * ( 1./540.*pow(RD346e,2)
          + 1./60.*m33*RD346e*RS3461e + 1./27.*m33*RD346e*RS1463e - 5./
         216.*m33*RD346e*RC346e + 2./27.*pow(m33,2)*RC346e*RS3461e + 1./
         540.*m11*RD346e*RS3461e + 1./72.*m11*RD346e*RC346e + 1./27.*
         m11*m33*RC346e*RS3461e );

      M13p6x22RV +=  + AbV(m34,L)*pi16 * (  - 1./24.*m33*RD146e + 1./48.
         *m33*me2 + 1./12.*m33*me2*RSe61 + 1./12.*m33*me2*RSe6611 + 1./
         8.*m33*m66 + 7./24.*m33*m44 + 1./48.*m33*m44*RS3461e + 1./48.*
         m33*m44*RS1463e - 1./24.*m33*m44*RSe6613 + 1./48.*m33*m44*
         RSe63 + 1./24.*m33*m44*RSe6633 + 1./48.*m33*m44*RS36e - 1./48.
         *m33*m44*RC346e + 1./48.*m33*m44*RSe61 + 1./24.*m33*m44*
         RSe6611 + 1./48.*m33*m44*RS16e - 1./48.*m33*m44*RC146e - 17./
         144.*pow(m33,2) + 1./48.*pow(m33,2)*RS3461e + 1./48.*pow(
         m33,2)*RS1463e - 1./24.*pow(m33,2)*RSe6613 + 1./48.*pow(m33,2)
         *RSe63 + 1./24.*pow(m33,2)*RSe6633 + 1./48.*pow(m33,2)*RS36e
          - 1./48.*pow(m33,2)*RC346e - 5./144.*pow(m33,2)*RSe61 - 1./72.
         *pow(m33,2)*RSe6611 - 5./144.*pow(m33,2)*RS16e + 1./144.*pow(
         m33,2)*RC146e - 1./24.*m11*RD146e + 1./48.*m11*me2 + 1./12.*
         m11*me2*RSe61 + 1./12.*m11*me2*RSe6611 + 1./8.*m11*m66 + 7./24.
         *m11*m44 + 1./48.*m11*m44*RS3461e + 1./48.*m11*m44*RS1463e - 1.
         /24.*m11*m44*RSe6613 );

      M13p6x22RV +=  + AbV(m34,L)*pi16 * ( 1./48.*m11*m44*RSe63 + 1./24.
         *m11*m44*RSe6633 + 1./48.*m11*m44*RS36e - 1./48.*m11*m44*
         RC346e + 1./48.*m11*m44*RSe61 + 1./24.*m11*m44*RSe6611 + 1./48.
         *m11*m44*RS16e - 1./48.*m11*m44*RC146e - 1./144.*m11*m33 + 1./
         48.*m11*m33*RS3461e + 1./48.*m11*m33*RS1463e - 1./24.*m11*m33*
         RSe6613 + 1./48.*m11*m33*RSe63 + 1./24.*m11*m33*RSe6633 + 1./
         48.*m11*m33*RS36e - 1./48.*m11*m33*RC346e - 1./144.*m11*m33*
         RSe61 + 1./72.*m11*m33*RSe6611 + 11./144.*m11*m33*RS16e - 7./
         144.*m11*m33*RC146e + 1./9.*pow(m11,2) + 1./36.*pow(m11,2)*
         RSe61 + 1./36.*pow(m11,2)*RSe6611 + 1./9.*pow(m11,2)*RS16e - 1.
         /18.*pow(m11,2)*RC146e );

      M13p6x22RV +=  + AbV(m34,L)*AbV(me2,L) * (  - 1./45.*me2 - 44./
         135.*me2*RSe6613 + 7./135.*me2*RSe63 + 29./135.*me2*RSe6633 - 
         19./135.*me2*RSe61 + 1./45.*me2*RSe6611 + 1./45.*m44 - 1./135.
         *m44*RSe63 + 7./135.*m44*RSe61 + 19./270.*m33*RSe6613 - 49./
         540.*m33*RSe63 - 49./135.*m33*RSe6633 - 5./36.*m33*RSe61 - 1./
         6.*m33*RSe6611 - 41./270.*m11*RSe6613 - 1./12.*m11*RSe63 - 47./
         540.*m11*RSe61 - 17./90.*m11*RSe6611 );

      M13p6x22RV +=  + AbV(m34,L)*BbV(m11,m33,L) * (  - 2./27.*m33*m44*
         RS16e - 2./27.*pow(m33,2)*RS1463e - 2./27.*pow(m33,2)*RS16e - 
         4./27.*m11*m44*RS16e - 8./27.*m11*m33*RS16e + 2./27.*pow(
         m11,2)*RS1463e - 2./27.*pow(m11,2)*RS16e );

      M13p6x22RV +=  + AbV(m34,L)*BbV(m11,L) * ( 1./12.*m33*RD146e + 1./
         12.*m11*RD146e );

      M13p6x22RV +=  + AbV(m34,L)*BbV(m33,me2,L) * (  - 4./27.*m33*me2*
         RSe6613 + 2./27.*m33*me2*RSe63 + 4./27.*m33*me2*RSe6633 - 2./
         27.*m33*me2*RSe61 - 4./27.*m33*m44*RSe61 + 4./27.*pow(m33,2)*
         RSe6613 - 2./27.*pow(m33,2)*RSe63 - 4./27.*pow(m33,2)*RSe6633
          - 2./9.*pow(m33,2)*RSe61 - 2./27.*m11*m44*RSe61 - 4./27.*m11*
         m33*RSe61 );

      M13p6x22RV +=  + AbV(m36,L)*pi16 * (  - 1./48.*m33*RD146e + 1./24.
         *m33*me2 - 1./12.*m33*me2*RSe61 + 1./24.*m33*me2*RSe6611 + 5./
         96.*m33*m66 + 1./96.*m33*m66*RS3461e + 1./96.*m33*m66*RS1463e
          - 1./48.*m33*m66*RSe6613 - 1./48.*m33*m66*RSe63 + 1./48.*m33*
         m66*RSe6633 + 1./96.*m33*m66*RS34e - 1./96.*m33*m66*RC346e - 1.
         /48.*m33*m66*RSe61 + 1./48.*m33*m66*RSe6611 + 1./96.*m33*m66*
         RS14e - 1./96.*m33*m66*RC146e + 1./8.*m33*m44 - 17./288.*pow(
         m33,2) + 1./96.*pow(m33,2)*RS3461e + 1./96.*pow(m33,2)*RS1463e
          - 1./48.*pow(m33,2)*RSe6613 - 1./48.*pow(m33,2)*RSe63 + 1./48.
         *pow(m33,2)*RSe6633 + 1./96.*pow(m33,2)*RS34e - 1./96.*pow(
         m33,2)*RC346e + 5./144.*pow(m33,2)*RSe61 - 1./144.*pow(m33,2)*
         RSe6611 - 5./288.*pow(m33,2)*RS14e + 1./288.*pow(m33,2)*RC146e
          - 1./48.*m11*RD146e + 1./24.*m11*me2 - 1./12.*m11*me2*RSe61
          + 1./24.*m11*me2*RSe6611 + 5./96.*m11*m66 + 1./96.*m11*m66*
         RS3461e + 1./96.*m11*m66*RS1463e - 1./48.*m11*m66*RSe6613 - 1./
         48.*m11*m66*RSe63 );

      M13p6x22RV +=  + AbV(m36,L)*pi16 * ( 1./48.*m11*m66*RSe6633 + 1./
         96.*m11*m66*RS34e - 1./96.*m11*m66*RC346e - 1./48.*m11*m66*
         RSe61 + 1./48.*m11*m66*RSe6611 + 1./96.*m11*m66*RS14e - 1./96.
         *m11*m66*RC146e + 1./8.*m11*m44 - 1./288.*m11*m33 + 1./96.*m11
         *m33*RS3461e + 1./96.*m11*m33*RS1463e - 1./48.*m11*m33*RSe6613
          - 1./48.*m11*m33*RSe63 + 1./48.*m11*m33*RSe6633 + 1./96.*m11*
         m33*RS34e - 1./96.*m11*m33*RC346e + 1./144.*m11*m33*RSe61 + 1./
         144.*m11*m33*RSe6611 + 11./288.*m11*m33*RS14e - 7./288.*m11*
         m33*RC146e + 1./18.*pow(m11,2) - 1./36.*pow(m11,2)*RSe61 + 1./
         72.*pow(m11,2)*RSe6611 + 1./18.*pow(m11,2)*RS14e - 1./36.*pow(
         m11,2)*RC146e );

      M13p6x22RV +=  + AbV(m36,L)*AbV(me2,L) * (  - 2./45.*me2 - 22./
         135.*me2*RSe6613 - 7./135.*me2*RSe63 + 29./270.*me2*RSe6633 + 
         19./135.*me2*RSe61 + 1./90.*me2*RSe6611 + 2./45.*m66 + 1./135.
         *m66*RSe63 - 7./135.*m66*RSe61 + 19./540.*m33*RSe6613 + 49./
         540.*m33*RSe63 - 49./270.*m33*RSe6633 + 5./36.*m33*RSe61 - 1./
         12.*m33*RSe6611 - 41./540.*m11*RSe6613 + 1./12.*m11*RSe63 + 47.
         /540.*m11*RSe61 - 17./180.*m11*RSe6611 );

      M13p6x22RV +=  + AbV(m36,L)*BbV(m11,m33,L) * (  - 1./27.*m33*m66*
         RS14e - 1./27.*pow(m33,2)*RS1463e - 1./27.*pow(m33,2)*RS14e - 
         2./27.*m11*m66*RS14e - 4./27.*m11*m33*RS14e + 1./27.*pow(
         m11,2)*RS1463e - 1./27.*pow(m11,2)*RS14e );

      M13p6x22RV +=  + AbV(m36,L)*BbV(m11,L) * ( 1./24.*m33*RD146e + 1./
         24.*m11*RD146e );

      M13p6x22RV +=  + AbV(m36,L)*BbV(m33,me2,L) * (  - 2./27.*m33*me2*
         RSe6613 - 2./27.*m33*me2*RSe63 + 2./27.*m33*me2*RSe6633 + 2./
         27.*m33*me2*RSe61 + 4./27.*m33*m66*RSe61 + 2./27.*pow(m33,2)*
         RSe6613 + 2./27.*pow(m33,2)*RSe63 - 2./27.*pow(m33,2)*RSe6633
          + 2./9.*pow(m33,2)*RSe61 + 2./27.*m11*m66*RSe61 + 4./27.*m11*
         m33*RSe61 );

      M13p6x22RV +=  + AbV(m44,L)*AbV(me2,L) * (  - 1./18.*m44*pow(
         RSe63,2) - 1./9.*m44*RS36e*RSe63 + 1./9.*m44*RSe61*RSe63 + 1./
         9.*m44*RSe61*RS36e - 1./18.*m44*pow(RSe61,2) + 1./9.*m44*RS16e
         *RSe63 - 1./9.*m44*RS16e*RSe61 );

      M13p6x22RV +=  + AbV(m44,L)*BbV(m11,m33,L) * ( 1./9.*m33*m44*
         RS16e*RS36e + 2./9.*m11*m44*RS16e*RS36e );

      M13p6x22RV +=  + AbV(m44,L)*BbV(m11,me2,L) * ( 1./9.*m33*m44*
         RS16e*RSe63 + 2./9.*m11*m44*RS16e*RSe63 );

      M13p6x22RV +=  + AbV(m44,L)*BbV(m33,me2,L) * ( 2./9.*m33*m44*
         RSe61*RS36e + 1./9.*m11*m44*RSe61*RS36e );

      M13p6x22RV +=  + AbV(m44,L)*BbV(me2,L) * (  - 1./18.*m44*me2*pow(
         RSe63,2) + 1./9.*m44*me2*RSe61*RSe63 - 1./18.*m44*me2*pow(
         RSe61,2) + 1./18.*m33*m44*pow(RSe63,2) + 1./9.*m33*m44*RSe61*
         RSe63 + 1./9.*m11*m44*RSe61*RSe63 + 1./18.*m11*m44*pow(
         RSe61,2) );

      M13p6x22RV +=  + AbV(m46,L)*AbV(me2,L) * ( 2./9.*me2*pow(RSe63,2)
          + 2./27.*me2*RS36e*RSe63 - 2./27.*me2*RS34e*RSe63 - 4./9.*me2
         *RSe61*RSe63 - 2./27.*me2*RSe61*RS36e + 2./27.*me2*RSe61*RS34e
          + 2./9.*me2*pow(RSe61,2) - 2./27.*me2*RS16e*RSe63 + 2./27.*
         me2*RS16e*RSe61 + 2./27.*me2*RS14e*RSe63 - 2./27.*me2*RS14e*
         RSe61 + 4./81.*m66*RS36e*RSe63 + 2./81.*m66*RS34e*RSe63 - 4./
         81.*m66*RSe61*RS36e - 2./81.*m66*RSe61*RS34e - 4./81.*m66*
         RS16e*RSe63 + 4./81.*m66*RS16e*RSe61 - 2./81.*m66*RS14e*RSe63
          + 2./81.*m66*RS14e*RSe61 + 1./27.*m44*pow(RSe63,2) + 2./81.*
         m44*RS36e*RSe63 - 2./81.*m44*RS34e*RSe63 - 2./27.*m44*RSe61*
         RSe63 - 2./81.*m44*RSe61*RS36e + 2./81.*m44*RSe61*RS34e + 1./
         27.*m44*pow(RSe61,2) - 2./81.*m44*RS16e*RSe63 + 2./81.*m44*
         RS16e*RSe61 + 2./81.*m44*RS14e*RSe63 - 2./81.*m44*RS14e*RSe61
          - 1./9.*m33*pow(RSe63,2) - 2./27.*m33*RS36e*RSe63 + 2./27.*
         m33*RS34e*RSe63 - 2./9.*m33*RSe61*RSe63 - 2./27.*m33*RSe61*
         RS36e );

      M13p6x22RV +=  + AbV(m46,L)*AbV(me2,L) * ( 2./27.*m33*RSe61*RS34e
          - 2./27.*m33*RS16e*RSe63 + 2./27.*m33*RS14e*RSe63 - 2./9.*m11
         *RSe61*RSe63 - 2./27.*m11*RSe61*RS36e + 2./27.*m11*RSe61*RS34e
          - 1./9.*m11*pow(RSe61,2) - 2./27.*m11*RS16e*RSe63 - 2./27.*
         m11*RS16e*RSe61 + 2./27.*m11*RS14e*RSe63 + 2./27.*m11*RS14e*
         RSe61 );

      M13p6x22RV +=  + AbV(m46,L)*BbV(m11,m33,L) * ( 2./81.*m33*m66*
         RS16e*RS34e + 2./81.*m33*m66*RS14e*RS36e + 2./81.*m33*m66*
         RS14e*RS34e + 2./81.*m33*m44*RS16e*RS36e + 2./81.*m33*m44*
         RS16e*RS34e + 2./81.*m33*m44*RS14e*RS36e + 4./81.*m11*m66*
         RS16e*RS34e + 4./81.*m11*m66*RS14e*RS36e + 4./81.*m11*m66*
         RS14e*RS34e + 4./81.*m11*m44*RS16e*RS36e + 4./81.*m11*m44*
         RS16e*RS34e + 4./81.*m11*m44*RS14e*RS36e - 4./81.*m11*m33*
         RS16e*RS36e + 4./81.*m11*m33*RS16e*RS34e + 4./81.*m11*m33*
         RS14e*RS36e - 4./81.*m11*m33*RS14e*RS34e - 2./81.*pow(m11,2)*
         RS16e*RS36e + 2./81.*pow(m11,2)*RS16e*RS34e + 2./81.*pow(
         m11,2)*RS14e*RS36e - 2./81.*pow(m11,2)*RS14e*RS34e );

      M13p6x22RV +=  + AbV(m46,L)*BbV(m11,me2,L) * (  - 4./81.*m33*m66*
         RS16e*RSe63 - 2./81.*m33*m66*RS14e*RSe63 - 2./81.*m33*m44*
         RS16e*RSe63 + 2./81.*m33*m44*RS14e*RSe63 - 2./27.*m11*me2*
         RS16e*RSe63 + 2./27.*m11*me2*RS16e*RSe61 + 2./27.*m11*me2*
         RS14e*RSe63 - 2./27.*m11*me2*RS14e*RSe61 - 8./81.*m11*m66*
         RS16e*RSe63 - 4./81.*m11*m66*RS14e*RSe63 - 4./81.*m11*m44*
         RS16e*RSe63 + 4./81.*m11*m44*RS14e*RSe63 - 2./27.*m11*m33*
         RS16e*RSe63 + 2./27.*m11*m33*RS14e*RSe63 - 2./27.*pow(m11,2)*
         RS16e*RSe63 - 2./27.*pow(m11,2)*RS16e*RSe61 + 2./27.*pow(
         m11,2)*RS14e*RSe63 + 2./27.*pow(m11,2)*RS14e*RSe61 );

      M13p6x22RV +=  + AbV(m46,L)*BbV(m33,me2,L) * ( 2./27.*m33*me2*
         RS36e*RSe63 - 2./27.*m33*me2*RS34e*RSe63 - 2./27.*m33*me2*
         RSe61*RS36e + 2./27.*m33*me2*RSe61*RS34e - 8./81.*m33*m66*
         RSe61*RS36e - 4./81.*m33*m66*RSe61*RS34e - 4./81.*m33*m44*
         RSe61*RS36e + 4./81.*m33*m44*RSe61*RS34e - 2./27.*pow(m33,2)*
         RS36e*RSe63 + 2./27.*pow(m33,2)*RS34e*RSe63 - 2./27.*pow(
         m33,2)*RSe61*RS36e + 2./27.*pow(m33,2)*RSe61*RS34e - 4./81.*
         m11*m66*RSe61*RS36e - 2./81.*m11*m66*RSe61*RS34e - 2./81.*m11*
         m44*RSe61*RS36e + 2./81.*m11*m44*RSe61*RS34e - 2./27.*m11*m33*
         RSe61*RS36e + 2./27.*m11*m33*RSe61*RS34e );

      M13p6x22RV +=  + AbV(m46,L)*BbV(me2,L) * ( 1./9.*pow(me2,2)*pow(
         RSe63,2) - 2./9.*pow(me2,2)*RSe61*RSe63 + 1./9.*pow(me2,2)*
         pow(RSe61,2) + 1./27.*m44*me2*pow(RSe63,2) - 2./27.*m44*me2*
         RSe61*RSe63 + 1./27.*m44*me2*pow(RSe61,2) - 1./9.*m33*me2*pow(
         RSe63,2) - 2./9.*m33*me2*RSe61*RSe63 - 1./27.*m33*m44*pow(
         RSe63,2) - 2./27.*m33*m44*RSe61*RSe63 - 2./9.*m11*me2*RSe61*
         RSe63 - 1./9.*m11*me2*pow(RSe61,2) - 2./27.*m11*m44*RSe61*
         RSe63 - 1./27.*m11*m44*pow(RSe61,2) );

      M13p6x22RV +=  + AbV(me2,L)*pi16 * ( 1./36.*m33*RD346e*RSe6613 - 
         1./72.*m33*RD346e*RSe6633 - 1./72.*m33*RSe6611*RD346e + 1./36.
         *m33*RD146e*RSe6613 - 1./72.*m33*RD146e*RSe6633 - 1./72.*m33*
         RD146e*RSe6611 + 1./12.*m33*me2*RSe6613 + 1./9.*m33*me2*pow(
         RSe6613,2) - 1./24.*m33*me2*RSe6633 - 1./9.*m33*me2*RSe6633*
         RSe6613 + 1./36.*m33*me2*pow(RSe6633,2) - 1./24.*m33*me2*
         RSe6611 - 1./9.*m33*me2*RSe6611*RSe6613 + 1./18.*m33*me2*
         RSe6611*RSe6633 + 1./36.*m33*me2*pow(RSe6611,2) - 1./12.*m33*
         m66*RSe6613 + 1./24.*m33*m66*RSe6633 + 1./24.*m33*m66*RSe6611
          - 1./6.*m33*m44*RSe6613 + 1./12.*m33*m44*RSe6633 + 1./12.*m33
         *m44*RSe6611 - 1./12.*pow(m33,2)*RSe6613 - 2./27.*pow(m33,2)*
         RSe6613*RS3461e - 1./54.*pow(m33,2)*RSe6613*RS1463e + 1./27.*
         pow(m33,2)*pow(RSe6613,2) + 1./24.*pow(m33,2)*RSe6633 + 1./27.
         *pow(m33,2)*RSe6633*RS3461e + 1./108.*pow(m33,2)*RSe6633*
         RS1463e - 1./27.*pow(m33,2)*RSe6633*RSe6613 + 1./108.*pow(
         m33,2)*pow(RSe6633,2) );

      M13p6x22RV +=  + AbV(me2,L)*pi16 * ( 1./27.*pow(m33,2)*RC346e*
         RSe6613 - 1./54.*pow(m33,2)*RC346e*RSe6633 + 1./24.*pow(m33,2)
         *RSe6611 + 1./27.*pow(m33,2)*RSe6611*RS3461e + 1./108.*pow(
         m33,2)*RSe6611*RS1463e - 1./27.*pow(m33,2)*RSe6611*RSe6613 + 1.
         /54.*pow(m33,2)*RSe6611*RSe6633 - 1./54.*pow(m33,2)*RSe6611*
         RC346e + 1./108.*pow(m33,2)*pow(RSe6611,2) + 1./108.*pow(
         m33,2)*RC146e*RSe6613 - 1./216.*pow(m33,2)*RC146e*RSe6633 - 1./
         216.*pow(m33,2)*RC146e*RSe6611 + 1./36.*m11*RD346e*RSe6613 - 1.
         /72.*m11*RD346e*RSe6633 - 1./72.*m11*RSe6611*RD346e + 1./36.*
         m11*RD146e*RSe6613 - 1./72.*m11*RD146e*RSe6633 - 1./72.*m11*
         RD146e*RSe6611 + 1./12.*m11*me2*RSe6613 + 1./9.*m11*me2*pow(
         RSe6613,2) - 1./24.*m11*me2*RSe6633 - 1./9.*m11*me2*RSe6633*
         RSe6613 + 1./36.*m11*me2*pow(RSe6633,2) - 1./24.*m11*me2*
         RSe6611 - 1./9.*m11*me2*RSe6611*RSe6613 + 1./18.*m11*me2*
         RSe6611*RSe6633 + 1./36.*m11*me2*pow(RSe6611,2) - 1./12.*m11*
         m66*RSe6613 );

      M13p6x22RV +=  + AbV(me2,L)*pi16 * ( 1./24.*m11*m66*RSe6633 + 1./
         24.*m11*m66*RSe6611 - 1./6.*m11*m44*RSe6613 + 1./12.*m11*m44*
         RSe6633 + 1./12.*m11*m44*RSe6611 - 1./6.*m11*m33*RSe6613 - 5./
         54.*m11*m33*RSe6613*RS3461e - 5./54.*m11*m33*RSe6613*RS1463e
          + 2./27.*m11*m33*pow(RSe6613,2) + 1./12.*m11*m33*RSe6633 + 5./
         108.*m11*m33*RSe6633*RS3461e + 5./108.*m11*m33*RSe6633*RS1463e
          - 2./27.*m11*m33*RSe6633*RSe6613 + 1./54.*m11*m33*pow(
         RSe6633,2) + 5./108.*m11*m33*RC346e*RSe6613 - 5./216.*m11*m33*
         RC346e*RSe6633 + 1./12.*m11*m33*RSe6611 + 5./108.*m11*m33*
         RSe6611*RS3461e + 5./108.*m11*m33*RSe6611*RS1463e - 2./27.*m11
         *m33*RSe6611*RSe6613 + 1./27.*m11*m33*RSe6611*RSe6633 - 5./216.
         *m11*m33*RSe6611*RC346e + 1./54.*m11*m33*pow(RSe6611,2) + 5./
         108.*m11*m33*RC146e*RSe6613 - 5./216.*m11*m33*RC146e*RSe6633
          - 5./216.*m11*m33*RC146e*RSe6611 - 1./12.*pow(m11,2)*RSe6613
          - 1./54.*pow(m11,2)*RSe6613*RS3461e - 2./27.*pow(m11,2)*
         RSe6613*RS1463e );

      M13p6x22RV +=  + AbV(me2,L)*pi16 * ( 1./27.*pow(m11,2)*pow(
         RSe6613,2) + 1./24.*pow(m11,2)*RSe6633 + 1./108.*pow(m11,2)*
         RSe6633*RS3461e + 1./27.*pow(m11,2)*RSe6633*RS1463e - 1./27.*
         pow(m11,2)*RSe6633*RSe6613 + 1./108.*pow(m11,2)*pow(RSe6633,2)
          + 1./108.*pow(m11,2)*RC346e*RSe6613 - 1./216.*pow(m11,2)*
         RC346e*RSe6633 + 1./24.*pow(m11,2)*RSe6611 + 1./108.*pow(
         m11,2)*RSe6611*RS3461e + 1./27.*pow(m11,2)*RSe6611*RS1463e - 1.
         /27.*pow(m11,2)*RSe6611*RSe6613 + 1./54.*pow(m11,2)*RSe6611*
         RSe6633 - 1./216.*pow(m11,2)*RSe6611*RC346e + 1./108.*pow(
         m11,2)*pow(RSe6611,2) + 1./27.*pow(m11,2)*RC146e*RSe6613 - 1./
         54.*pow(m11,2)*RC146e*RSe6633 - 1./54.*pow(m11,2)*RC146e*
         RSe6611 );

      M13p6x22RV +=  + pow(AbV(me2,L),2) * ( 4./135.*me2*pow(RSe6613,2)
          - 4./135.*me2*RSe6633*RSe6613 + 1./135.*me2*pow(RSe6633,2) - 
         4./135.*me2*RSe6611*RSe6613 + 2./135.*me2*RSe6611*RSe6633 + 1./
         135.*me2*pow(RSe6611,2) - 4./81.*m66*pow(RSe63,2) + 4./81.*m66
         *RS34e*RSe63 + 8./81.*m66*RSe61*RSe63 - 4./81.*m66*RSe61*RS34e
          - 4./81.*m66*pow(RSe61,2) - 4./81.*m66*RS14e*RSe63 + 4./81.*
         m66*RS14e*RSe61 - 1./162.*m44*pow(RSe63,2) - 1./81.*m44*RS36e*
         RSe63 + 1./81.*m44*RSe61*RSe63 + 1./81.*m44*RSe61*RS36e - 1./
         162.*m44*pow(RSe61,2) + 1./81.*m44*RS16e*RSe63 - 1./81.*m44*
         RS16e*RSe61 + 13./135.*m33*pow(RSe6613,2) + 13./135.*m33*
         RSe6633*RSe6613 - 29./540.*m33*pow(RSe6633,2) + 1./135.*m33*
         RSe6611*RSe6613 - 17./270.*m33*RSe6611*RSe6633 + 1./36.*m33*
         pow(RSe6611,2) + 13./135.*m11*pow(RSe6613,2) + 1./135.*m11*
         RSe6633*RSe6613 + 1./36.*m11*pow(RSe6633,2) + 13./135.*m11*
         RSe6611*RSe6613 - 17./270.*m11*RSe6611*RSe6633 - 29./540.*m11*
         pow(RSe6611,2) );

      M13p6x22RV +=  + AbV(me2,L)*BbV(m11,m33,L) * ( 2./81.*m33*m66*
         RS14e*RS34e + 1./81.*m33*m44*RS16e*RS36e - 2./27.*pow(m33,2)*
         RSe6633*RS1463e + 4./81.*m11*m66*RS14e*RS34e + 2./81.*m11*m44*
         RS16e*RS36e - 4./27.*m11*m33*RSe6633*RS1463e - 2./27.*m11*m33*
         RSe6611*RS3461e - 4./27.*pow(m11,2)*RSe6611*RS3461e );

      M13p6x22RV +=  + AbV(me2,L)*BbV(m11,me2,L) * (  - 4./81.*m33*m66*
         RS14e*RSe63 + 1./81.*m33*m44*RS16e*RSe63 - 8./81.*m11*m66*
         RS14e*RSe63 + 2./81.*m11*m44*RS16e*RSe63 + 4./27.*m11*m33*
         RSe6611*RSe6613 + 8./27.*pow(m11,2)*RSe6611*RSe6613 );

      M13p6x22RV +=  + AbV(me2,L)*BbV(m11,L) * ( 1./135.*me2*RD146e*
         RSe6613 - 1./270.*me2*RD146e*RSe6633 - 1./270.*me2*RD146e*
         RSe6611 - 1./270.*m33*RD146e*RSe6613 + 17./540.*m33*RD146e*
         RSe6633 - 1./36.*m33*RD146e*RSe6611 - 11./270.*m11*RD146e*
         RSe6613 + 1./36.*m11*RD146e*RSe6633 + 1./20.*m11*RD146e*
         RSe6611 - 2./27.*m11*m33*RSe6611*RS1463e - 4./27.*pow(m11,2)*
         RSe6611*RS1463e );

      M13p6x22RV +=  + AbV(me2,L)*BbV(m33,me2,L) * (  - 8./81.*m33*m66*
         RSe61*RS34e + 2./81.*m33*m44*RSe61*RS36e + 8./27.*pow(m33,2)*
         RSe6633*RSe6613 - 4./81.*m11*m66*RSe61*RS34e + 1./81.*m11*m44*
         RSe61*RS36e + 4./27.*m11*m33*RSe6633*RSe6613 );

      M13p6x22RV +=  + AbV(me2,L)*BbV(m33,L) * ( 1./135.*me2*RD346e*
         RSe6613 - 1./270.*me2*RD346e*RSe6633 - 1./270.*me2*RSe6611*
         RD346e - 11./270.*m33*RD346e*RSe6613 + 1./20.*m33*RD346e*
         RSe6633 + 1./36.*m33*RSe6611*RD346e - 4./27.*pow(m33,2)*
         RSe6633*RS3461e - 1./270.*m11*RD346e*RSe6613 - 1./36.*m11*
         RD346e*RSe6633 + 17./540.*m11*RSe6611*RD346e - 2./27.*m11*m33*
         RSe6633*RS3461e );

      M13p6x22RV +=  + AbV(me2,L)*BbV(me2,L) * (  - 4./81.*m66*me2*pow(
         RSe63,2) + 8./81.*m66*me2*RSe61*RSe63 - 4./81.*m66*me2*pow(
         RSe61,2) - 1./162.*m44*me2*pow(RSe63,2) + 1./81.*m44*me2*RSe61
         *RSe63 - 1./162.*m44*me2*pow(RSe61,2) + 4./81.*m33*m66*pow(
         RSe63,2) + 8./81.*m33*m66*RSe61*RSe63 + 1./162.*m33*m44*pow(
         RSe63,2) + 1./81.*m33*m44*RSe61*RSe63 + 8./81.*m11*m66*RSe61*
         RSe63 + 4./81.*m11*m66*pow(RSe61,2) + 1./81.*m11*m44*RSe61*
         RSe63 + 1./162.*m11*m44*pow(RSe61,2) );

      M13p6x22RV +=  + BbV(m11,m33,L)*BbV(m11,L) * ( 1./27.*m11*m33*
         RD146e*RS3461e + 2./27.*pow(m11,2)*RD146e*RS3461e );

      M13p6x22RV +=  + BbV(m11,m33,L)*BbV(m33,L) * ( 1./27.*pow(m33,2)*
         RD346e*RS1463e + 2./27.*m11*m33*RD346e*RS1463e );

      M13p6x22RV +=  + BbV(m11,me2,L)*BbV(m11,L) * (  - 2./27.*m11*m33*
         RD146e*RSe6613 - 4./27.*pow(m11,2)*RD146e*RSe6613 );

      M13p6x22RV +=  + BbV(m11,L)*pi16 * ( 1./144.*m33*RD146e*RD346e + 
         1./144.*m33*pow(RD146e,2) + 1./36.*m33*me2*RD146e*RSe6613 - 1./
         72.*m33*me2*RD146e*RSe6633 - 1./72.*m33*me2*RD146e*RSe6611 - 1.
         /48.*m33*m66*RD146e - 1./24.*m33*m44*RD146e - 1./48.*pow(
         m33,2)*RD146e - 1./54.*pow(m33,2)*RD146e*RS3461e - 1./216.*
         pow(m33,2)*RD146e*RS1463e + 1./108.*pow(m33,2)*RD146e*RSe6613
          - 1./216.*pow(m33,2)*RD146e*RSe6633 + 1./108.*pow(m33,2)*
         RD146e*RC346e - 1./216.*pow(m33,2)*RD146e*RSe6611 + 1./432.*
         pow(m33,2)*RD146e*RC146e + 1./144.*m11*RD146e*RD346e + 1./144.
         *m11*pow(RD146e,2) + 1./36.*m11*me2*RD146e*RSe6613 - 1./72.*
         m11*me2*RD146e*RSe6633 - 1./72.*m11*me2*RD146e*RSe6611 - 1./48.
         *m11*m66*RD146e - 1./24.*m11*m44*RD146e - 1./48.*m11*m33*
         RD146e - 5./216.*m11*m33*RD146e*RS3461e - 5./216.*m11*m33*
         RD146e*RS1463e + 1./54.*m11*m33*RD146e*RSe6613 - 1./108.*m11*
         m33*RD146e*RSe6633 + 5./432.*m11*m33*RD146e*RC346e - 1./108.*
         m11*m33*RD146e*RSe6611 );

      M13p6x22RV +=  + BbV(m11,L)*pi16 * ( 5./432.*m11*m33*RD146e*
         RC146e - 1./216.*pow(m11,2)*RD146e*RS3461e - 1./54.*pow(m11,2)
         *RD146e*RS1463e + 1./108.*pow(m11,2)*RD146e*RSe6613 - 1./216.*
         pow(m11,2)*RD146e*RSe6633 + 1./432.*pow(m11,2)*RD146e*RC346e
          - 1./216.*pow(m11,2)*RD146e*RSe6611 + 1./108.*pow(m11,2)*
         RD146e*RC146e );

      M13p6x22RV +=  + pow(BbV(m11,L),2) * ( 1./144.*m33*pow(RD146e,2)
          - 5./432.*m11*pow(RD146e,2) + 1./27.*m11*m33*RD146e*RS1463e
          + 2./27.*pow(m11,2)*RD146e*RS1463e );

      M13p6x22RV +=  + BbV(m11,L)*BbV(m33,L) * (  - 1./72.*m33*RD146e*
         RD346e - 1./72.*m11*RD146e*RD346e );

      M13p6x22RV +=  + BbV(m33,me2,L)*BbV(m33,L) * (  - 4./27.*pow(
         m33,2)*RD346e*RSe6613 - 2./27.*m11*m33*RD346e*RSe6613 );

      M13p6x22RV +=  + BbV(m33,L)*pi16 * ( 1./144.*m33*pow(RD346e,2) + 
         1./144.*m33*RD146e*RD346e + 1./36.*m33*me2*RD346e*RSe6613 - 1./
         72.*m33*me2*RD346e*RSe6633 - 1./72.*m33*me2*RSe6611*RD346e - 1.
         /48.*m33*m66*RD346e - 1./24.*m33*m44*RD346e - 1./54.*pow(
         m33,2)*RD346e*RS3461e - 1./216.*pow(m33,2)*RD346e*RS1463e + 1./
         108.*pow(m33,2)*RD346e*RSe6613 - 1./216.*pow(m33,2)*RD346e*
         RSe6633 + 1./108.*pow(m33,2)*RD346e*RC346e - 1./216.*pow(
         m33,2)*RSe6611*RD346e + 1./432.*pow(m33,2)*RC146e*RD346e + 1./
         144.*m11*pow(RD346e,2) + 1./144.*m11*RD146e*RD346e + 1./36.*
         m11*me2*RD346e*RSe6613 - 1./72.*m11*me2*RD346e*RSe6633 - 1./72.
         *m11*me2*RSe6611*RD346e - 1./48.*m11*m66*RD346e - 1./24.*m11*
         m44*RD346e - 1./48.*m11*m33*RD346e - 5./216.*m11*m33*RD346e*
         RS3461e - 5./216.*m11*m33*RD346e*RS1463e + 1./54.*m11*m33*
         RD346e*RSe6613 - 1./108.*m11*m33*RD346e*RSe6633 + 5./432.*m11*
         m33*RD346e*RC346e - 1./108.*m11*m33*RSe6611*RD346e + 5./432.*
         m11*m33*RC146e*RD346e );

      M13p6x22RV +=  + BbV(m33,L)*pi16 * (  - 1./48.*pow(m11,2)*RD346e
          - 1./216.*pow(m11,2)*RD346e*RS3461e - 1./54.*pow(m11,2)*
         RD346e*RS1463e + 1./108.*pow(m11,2)*RD346e*RSe6613 - 1./216.*
         pow(m11,2)*RD346e*RSe6633 + 1./432.*pow(m11,2)*RD346e*RC346e
          - 1./216.*pow(m11,2)*RSe6611*RD346e + 1./108.*pow(m11,2)*
         RC146e*RD346e );

      M13p6x22RV +=  + pow(BbV(m33,L),2) * (  - 5./432.*m33*pow(
         RD346e,2) + 2./27.*pow(m33,2)*RD346e*RS3461e + 1./144.*m11*
         pow(RD346e,2) + 1./27.*m11*m33*RD346e*RS3461e );

      M13p6x22RV +=  + A23bV(m11,L)*pi16 * ( 3./16.*m33 - 1./36.*m33*
         RS1463e*RS3461e - 1./36.*m33*pow(RS1463e,2) + 1./18.*m33*
         RSe6613*RS1463e - 1./36.*m33*RSe6633*RS1463e + 1./72.*m33*
         RC346e*RS1463e - 1./36.*m33*RSe6611*RS1463e - 1./8.*m33*RS16e
          - 1./16.*m33*RS14e + 1./72.*m33*RC146e*RS3461e + 1./36.*m33*
         RC146e*RS1463e - 1./36.*m33*RC146e*RSe6613 + 1./72.*m33*RC146e
         *RSe6633 - 1./144.*m33*RC146e*RC346e + 1./72.*m33*RC146e*
         RSe6611 - 1./144.*m33*pow(RC146e,2) + 3./16.*m11 - 1./36.*m11*
         RS1463e*RS3461e - 1./36.*m11*pow(RS1463e,2) + 1./18.*m11*
         RSe6613*RS1463e - 1./36.*m11*RSe6633*RS1463e + 1./72.*m11*
         RC346e*RS1463e - 1./36.*m11*RSe6611*RS1463e - 1./8.*m11*RS16e
          - 1./16.*m11*RS14e + 1./72.*m11*RC146e*RS3461e + 1./36.*m11*
         RC146e*RS1463e - 1./36.*m11*RC146e*RSe6613 + 1./72.*m11*RC146e
         *RSe6633 - 1./144.*m11*RC146e*RC346e + 1./72.*m11*RC146e*
         RSe6611 - 1./144.*m11*pow(RC146e,2) );

      M13p6x22RV +=  + A23bV(m13,L)*pi16 * (  - 1./24.*m33*pow(
         RS3461e,2) - 1./12.*m33*RS1463e*RS3461e - 1./24.*m33*pow(
         RS1463e,2) + 1./6.*m33*RSe6613*RS3461e + 1./6.*m33*RSe6613*
         RS1463e - 1./6.*m33*pow(RSe6613,2) - 1./12.*m33*RSe6633*
         RS3461e - 1./12.*m33*RSe6633*RS1463e + 1./6.*m33*RSe6633*
         RSe6613 - 1./24.*m33*pow(RSe6633,2) + 1./24.*m33*RC346e*
         RS3461e + 1./24.*m33*RC346e*RS1463e - 1./12.*m33*RC346e*
         RSe6613 + 1./24.*m33*RC346e*RSe6633 - 1./96.*m33*pow(RC346e,2)
          - 1./12.*m33*RSe6611*RS3461e - 1./12.*m33*RSe6611*RS1463e + 1.
         /6.*m33*RSe6611*RSe6613 - 1./12.*m33*RSe6611*RSe6633 + 1./24.*
         m33*RSe6611*RC346e - 1./24.*m33*pow(RSe6611,2) + 1./24.*m33*
         RC146e*RS3461e + 1./24.*m33*RC146e*RS1463e - 1./12.*m33*RC146e
         *RSe6613 + 1./24.*m33*RC146e*RSe6633 - 1./48.*m33*RC146e*
         RC346e + 1./24.*m33*RC146e*RSe6611 - 1./96.*m33*pow(RC146e,2)
          - 1./24.*m11*pow(RS3461e,2) - 1./12.*m11*RS1463e*RS3461e - 1./
         24.*m11*pow(RS1463e,2) );

      M13p6x22RV +=  + A23bV(m13,L)*pi16 * ( 1./6.*m11*RSe6613*RS3461e
          + 1./6.*m11*RSe6613*RS1463e - 1./6.*m11*pow(RSe6613,2) - 1./
         12.*m11*RSe6633*RS3461e - 1./12.*m11*RSe6633*RS1463e + 1./6.*
         m11*RSe6633*RSe6613 - 1./24.*m11*pow(RSe6633,2) + 1./24.*m11*
         RC346e*RS3461e + 1./24.*m11*RC346e*RS1463e - 1./12.*m11*RC346e
         *RSe6613 + 1./24.*m11*RC346e*RSe6633 - 1./96.*m11*pow(
         RC346e,2) - 1./12.*m11*RSe6611*RS3461e - 1./12.*m11*RSe6611*
         RS1463e + 1./6.*m11*RSe6611*RSe6613 - 1./12.*m11*RSe6611*
         RSe6633 + 1./24.*m11*RSe6611*RC346e - 1./24.*m11*pow(
         RSe6611,2) + 1./24.*m11*RC146e*RS3461e + 1./24.*m11*RC146e*
         RS1463e - 1./12.*m11*RC146e*RSe6613 + 1./24.*m11*RC146e*
         RSe6633 - 1./48.*m11*RC146e*RC346e + 1./24.*m11*RC146e*RSe6611
          - 1./96.*m11*pow(RC146e,2) );

      M13p6x22RV +=  + A23bV(m14,L)*pi16 * ( 1./24.*m33 - 1./8.*m33*
         RS3461e - 1./8.*m33*RS1463e + 1./4.*m33*RSe6613 + 1./24.*m33*
         RSe63 - 1./12.*m33*RSe6633 + 1./24.*m33*RS36e + 1./24.*m33*
         RC346e - 1./8.*m33*RSe61 - 1./4.*m33*RSe6611 - 1./8.*m33*RS16e
          + 1./8.*m33*RC146e + 1./24.*m11 - 1./8.*m11*RS3461e - 1./8.*
         m11*RS1463e + 1./4.*m11*RSe6613 + 1./24.*m11*RSe63 - 1./12.*
         m11*RSe6633 + 1./24.*m11*RS36e + 1./24.*m11*RC346e - 1./8.*m11
         *RSe61 - 1./4.*m11*RSe6611 - 1./8.*m11*RS16e + 1./8.*m11*
         RC146e );

      M13p6x22RV +=  + A23bV(m16,L)*pi16 * ( 1./48.*m33 - 1./16.*m33*
         RS3461e - 1./16.*m33*RS1463e + 1./8.*m33*RSe6613 - 1./24.*m33*
         RSe63 - 1./24.*m33*RSe6633 + 1./48.*m33*RS34e + 1./48.*m33*
         RC346e + 1./8.*m33*RSe61 - 1./8.*m33*RSe6611 - 1./16.*m33*
         RS14e + 1./16.*m33*RC146e + 1./48.*m11 - 1./16.*m11*RS3461e - 
         1./16.*m11*RS1463e + 1./8.*m11*RSe6613 - 1./24.*m11*RSe63 - 1./
         24.*m11*RSe6633 + 1./48.*m11*RS34e + 1./48.*m11*RC346e + 1./8.
         *m11*RSe61 - 1./8.*m11*RSe6611 - 1./16.*m11*RS14e + 1./16.*m11
         *RC146e );

      M13p6x22RV +=  + A23bV(m33,L)*pi16 * ( 3./16.*m33 - 1./36.*m33*
         pow(RS3461e,2) - 1./36.*m33*RS1463e*RS3461e + 1./18.*m33*
         RSe6613*RS3461e - 1./36.*m33*RSe6633*RS3461e - 1./8.*m33*RS36e
          - 1./16.*m33*RS34e + 1./36.*m33*RC346e*RS3461e + 1./72.*m33*
         RC346e*RS1463e - 1./36.*m33*RC346e*RSe6613 + 1./72.*m33*RC346e
         *RSe6633 - 1./144.*m33*pow(RC346e,2) - 1./36.*m33*RSe6611*
         RS3461e + 1./72.*m33*RSe6611*RC346e + 1./72.*m33*RC146e*
         RS3461e - 1./144.*m33*RC146e*RC346e + 3./16.*m11 - 1./36.*m11*
         pow(RS3461e,2) - 1./36.*m11*RS1463e*RS3461e + 1./18.*m11*
         RSe6613*RS3461e - 1./36.*m11*RSe6633*RS3461e - 1./8.*m11*RS36e
          - 1./16.*m11*RS34e + 1./36.*m11*RC346e*RS3461e + 1./72.*m11*
         RC346e*RS1463e - 1./36.*m11*RC346e*RSe6613 + 1./72.*m11*RC346e
         *RSe6633 - 1./144.*m11*pow(RC346e,2) - 1./36.*m11*RSe6611*
         RS3461e + 1./72.*m11*RSe6611*RC346e + 1./72.*m11*RC146e*
         RS3461e - 1./144.*m11*RC146e*RC346e );

      M13p6x22RV +=  + A23bV(m34,L)*pi16 * ( 1./24.*m33 - 1./8.*m33*
         RS3461e - 1./8.*m33*RS1463e + 1./4.*m33*RSe6613 - 1./8.*m33*
         RSe63 - 1./4.*m33*RSe6633 - 1./8.*m33*RS36e + 1./8.*m33*RC346e
          + 1./24.*m33*RSe61 - 1./12.*m33*RSe6611 + 1./24.*m33*RS16e + 
         1./24.*m33*RC146e + 1./24.*m11 - 1./8.*m11*RS3461e - 1./8.*m11
         *RS1463e + 1./4.*m11*RSe6613 - 1./8.*m11*RSe63 - 1./4.*m11*
         RSe6633 - 1./8.*m11*RS36e + 1./8.*m11*RC346e + 1./24.*m11*
         RSe61 - 1./12.*m11*RSe6611 + 1./24.*m11*RS16e + 1./24.*m11*
         RC146e );

      M13p6x22RV +=  + A23bV(m36,L)*pi16 * ( 1./48.*m33 - 1./16.*m33*
         RS3461e - 1./16.*m33*RS1463e + 1./8.*m33*RSe6613 + 1./8.*m33*
         RSe63 - 1./8.*m33*RSe6633 - 1./16.*m33*RS34e + 1./16.*m33*
         RC346e - 1./24.*m33*RSe61 - 1./24.*m33*RSe6611 + 1./48.*m33*
         RS14e + 1./48.*m33*RC146e + 1./48.*m11 - 1./16.*m11*RS3461e - 
         1./16.*m11*RS1463e + 1./8.*m11*RSe6613 + 1./8.*m11*RSe63 - 1./
         8.*m11*RSe6633 - 1./16.*m11*RS34e + 1./16.*m11*RC346e - 1./24.
         *m11*RSe61 - 1./24.*m11*RSe6611 + 1./48.*m11*RS14e + 1./48.*
         m11*RC146e );

      M13p6x22RV +=  + A23bV(me2,L)*pi16 * ( 1./18.*m33*RSe6613*RS3461e
          + 1./18.*m33*RSe6613*RS1463e - 1./9.*m33*pow(RSe6613,2) - 1./
         36.*m33*RSe6633*RS3461e - 1./36.*m33*RSe6633*RS1463e + 1./9.*
         m33*RSe6633*RSe6613 - 1./36.*m33*pow(RSe6633,2) - 1./36.*m33*
         RC346e*RSe6613 + 1./72.*m33*RC346e*RSe6633 - 1./36.*m33*
         RSe6611*RS3461e - 1./36.*m33*RSe6611*RS1463e + 1./9.*m33*
         RSe6611*RSe6613 - 1./18.*m33*RSe6611*RSe6633 + 1./72.*m33*
         RSe6611*RC346e - 1./36.*m33*pow(RSe6611,2) - 1./36.*m33*RC146e
         *RSe6613 + 1./72.*m33*RC146e*RSe6633 + 1./72.*m33*RC146e*
         RSe6611 + 1./18.*m11*RSe6613*RS3461e + 1./18.*m11*RSe6613*
         RS1463e - 1./9.*m11*pow(RSe6613,2) - 1./36.*m11*RSe6633*
         RS3461e - 1./36.*m11*RSe6633*RS1463e + 1./9.*m11*RSe6633*
         RSe6613 - 1./36.*m11*pow(RSe6633,2) - 1./36.*m11*RC346e*
         RSe6613 + 1./72.*m11*RC346e*RSe6633 - 1./36.*m11*RSe6611*
         RS3461e - 1./36.*m11*RSe6611*RS1463e + 1./9.*m11*RSe6611*
         RSe6613 );

      M13p6x22RV +=  + A23bV(me2,L)*pi16 * (  - 1./18.*m11*RSe6611*
         RSe6633 + 1./72.*m11*RSe6611*RC346e - 1./36.*m11*pow(
         RSe6611,2) - 1./36.*m11*RC146e*RSe6613 + 1./72.*m11*RC146e*
         RSe6633 + 1./72.*m11*RC146e*RSe6611 );

      M13p6x22RV +=  + B23bV(m11,L)*pi16 * ( 1./72.*m33*RD146e*RS3461e
          + 1./72.*m33*RD146e*RS1463e - 1./36.*m33*RD146e*RSe6613 + 1./
         72.*m33*RD146e*RSe6633 - 1./144.*m33*RD146e*RC346e + 1./72.*
         m33*RD146e*RSe6611 - 1./144.*m33*RD146e*RC146e + 1./72.*m11*
         RD146e*RS3461e + 1./72.*m11*RD146e*RS1463e - 1./36.*m11*RD146e
         *RSe6613 + 1./72.*m11*RD146e*RSe6633 - 1./144.*m11*RD146e*
         RC346e + 1./72.*m11*RD146e*RSe6611 - 1./144.*m11*RD146e*RC146e
          );

      M13p6x22RV +=  + B23bV(m33,L)*pi16 * ( 1./72.*m33*RD346e*RS3461e
          + 1./72.*m33*RD346e*RS1463e - 1./36.*m33*RD346e*RSe6613 + 1./
         72.*m33*RD346e*RSe6633 - 1./144.*m33*RD346e*RC346e + 1./72.*
         m33*RSe6611*RD346e - 1./144.*m33*RC146e*RD346e + 1./72.*m11*
         RD346e*RS3461e + 1./72.*m11*RD346e*RS1463e - 1./36.*m11*RD346e
         *RSe6613 + 1./72.*m11*RD346e*RSe6633 - 1./144.*m11*RD346e*
         RC346e + 1./72.*m11*RSe6611*RD346e - 1./144.*m11*RC146e*RD346e
          );

      M13p6x22RV +=  + hhV(1,m11,m11,m13,m13,L,mu2) * ( 1./324.*pow(
         RD146e,2) - 1./81.*m33*RD146e*RS1463e - 1./8.*pow(m33,2) - 7./
         162.*pow(m33,2)*pow(RS1463e,2) + 1./18.*pow(m33,2)*RC146e*
         RS1463e - 1./72.*pow(m33,2)*pow(RC146e,2) - 2./81.*m11*RD146e*
         RS1463e - 3./8.*m11*m33 - 5./81.*m11*m33*pow(RS1463e,2) + 1./9.
         *m11*m33*RC146e*RS1463e - 1./36.*m11*m33*pow(RC146e,2) - 1./4.
         *pow(m11,2) - 1./162.*pow(m11,2)*pow(RS1463e,2) + 1./18.*pow(
         m11,2)*RC146e*RS1463e - 1./72.*pow(m11,2)*pow(RC146e,2) );

      M13p6x22RV +=  + hhV(1,m11,m13,m33,m13,L,mu2) * ( 1./162.*RD146e*
         RD346e + 2./81.*m33*RD346e*RS1463e + 1./81.*m33*RD146e*RS3461e
          + 4./81.*pow(m33,2)*RS1463e*RS3461e + 1./81.*m11*RD346e*
         RS1463e + 2./81.*m11*RD146e*RS3461e + 10./81.*m11*m33*RS1463e*
         RS3461e + 4./81.*pow(m11,2)*RS1463e*RS3461e );

      M13p6x22RV +=  + hhV(1,m11,m13,me2,m13,L,mu2) * ( 2./81.*me2*
         RD146e*RSe6613 - 1./81.*me2*RD146e*RSe6633 - 1./81.*me2*RD146e
         *RSe6611 - 2./81.*pow(me2,2)*RSe6613*RS1463e + 1./81.*pow(
         me2,2)*RSe6633*RS1463e + 1./81.*pow(me2,2)*RSe6611*RS1463e + 1.
         /81.*pow(me2,2)*RC146e*RSe6613 - 1./162.*pow(me2,2)*RC146e*
         RSe6633 - 1./162.*pow(me2,2)*RC146e*RSe6611 - 4./81.*m33*
         RD146e*RSe6613 + 1./81.*m33*RD146e*RSe6633 + 1./27.*m33*RD146e
         *RSe6611 + 10./81.*m33*me2*RSe6613*RS1463e - 2./27.*m33*me2*
         RSe6633*RS1463e - 4./81.*m33*me2*RSe6611*RS1463e - 4./81.*m33*
         me2*RC146e*RSe6613 + 1./81.*m33*me2*RC146e*RSe6633 + 1./27.*
         m33*me2*RC146e*RSe6611 - 16./81.*pow(m33,2)*RSe6613*RS1463e + 
         5./81.*pow(m33,2)*RSe6633*RS1463e + 1./27.*pow(m33,2)*RSe6611*
         RS1463e + 1./27.*pow(m33,2)*RC146e*RSe6613 - 1./162.*pow(
         m33,2)*RC146e*RSe6633 - 1./18.*pow(m33,2)*RC146e*RSe6611 - 4./
         81.*m11*RD146e*RSe6613 + 4./81.*m11*RD146e*RSe6611 + 2./27.*
         m11*me2*RSe6613*RS1463e );

      M13p6x22RV +=  + hhV(1,m11,m13,me2,m13,L,mu2) * (  - 2./81.*m11*
         me2*RSe6633*RS1463e - 4./81.*m11*me2*RSe6611*RS1463e - 4./81.*
         m11*me2*RC146e*RSe6613 + 4./81.*m11*me2*RC146e*RSe6611 - 26./
         81.*m11*m33*RSe6613*RS1463e + 2./81.*m11*m33*RSe6633*RS1463e
          + 4./81.*m11*m33*RSe6611*RS1463e + 4./81.*m11*m33*RC146e*
         RSe6613 - 4./27.*m11*m33*RC146e*RSe6611 - 8./81.*pow(m11,2)*
         RSe6613*RS1463e - 8./81.*pow(m11,2)*RC146e*RSe6611 );

      M13p6x22RV +=  + hhV(1,m11,m14,m34,m13,L,mu2) * (  - 1./6.*pow(
         m33,2)*RS1463e + 1./6.*pow(m33,2)*RS16e - 5./12.*m11*m33*
         RS1463e + 5./12.*m11*m33*RS16e - 1./4.*pow(m11,2)*RS1463e + 1./
         4.*pow(m11,2)*RS16e );

      M13p6x22RV +=  + hhV(1,m11,m16,m36,m13,L,mu2) * (  - 1./12.*pow(
         m33,2)*RS1463e + 1./12.*pow(m33,2)*RS14e - 5./24.*m11*m33*
         RS1463e + 5./24.*m11*m33*RS14e - 1./8.*pow(m11,2)*RS1463e + 1./
         8.*pow(m11,2)*RS14e );

      M13p6x22RV +=  + hhV(1,m13,m11,m11,m13,L,mu2) * ( 1./32.*pow(
         m33,2) + 1./72.*pow(m33,2)*pow(RS1463e,2) - 1./72.*pow(m33,2)*
         RC146e*RS1463e + 1./288.*pow(m33,2)*pow(RC146e,2) + 1./16.*m11
         *m33 + 1./36.*m11*m33*pow(RS1463e,2) - 1./36.*m11*m33*RC146e*
         RS1463e + 1./144.*m11*m33*pow(RC146e,2) + 1./32.*pow(m11,2) + 
         1./72.*pow(m11,2)*pow(RS1463e,2) - 1./72.*pow(m11,2)*RC146e*
         RS1463e + 1./288.*pow(m11,2)*pow(RC146e,2) );

      M13p6x22RV +=  + hhV(1,m13,m11,m33,m13,L,mu2) * ( 1./36.*pow(
         m33,2)*RS1463e*RS3461e - 1./72.*pow(m33,2)*RC346e*RS1463e - 1./
         72.*pow(m33,2)*RC146e*RS3461e + 1./144.*pow(m33,2)*RC146e*
         RC346e + 1./18.*m11*m33*RS1463e*RS3461e - 1./36.*m11*m33*
         RC346e*RS1463e - 1./36.*m11*m33*RC146e*RS3461e + 1./72.*m11*
         m33*RC146e*RC346e + 1./36.*pow(m11,2)*RS1463e*RS3461e - 1./72.
         *pow(m11,2)*RC346e*RS1463e - 1./72.*pow(m11,2)*RC146e*RS3461e
          + 1./144.*pow(m11,2)*RC146e*RC346e );

      M13p6x22RV +=  + hhV(1,m13,m11,me2,m13,L,mu2) * (  - 1./18.*pow(
         m33,2)*RSe6613*RS1463e + 1./36.*pow(m33,2)*RSe6633*RS1463e + 1.
         /36.*pow(m33,2)*RSe6611*RS1463e + 1./36.*pow(m33,2)*RC146e*
         RSe6613 - 1./72.*pow(m33,2)*RC146e*RSe6633 - 1./72.*pow(m33,2)
         *RC146e*RSe6611 - 1./9.*m11*m33*RSe6613*RS1463e + 1./18.*m11*
         m33*RSe6633*RS1463e + 1./18.*m11*m33*RSe6611*RS1463e + 1./18.*
         m11*m33*RC146e*RSe6613 - 1./36.*m11*m33*RC146e*RSe6633 - 1./36.
         *m11*m33*RC146e*RSe6611 - 1./18.*pow(m11,2)*RSe6613*RS1463e + 
         1./36.*pow(m11,2)*RSe6633*RS1463e + 1./36.*pow(m11,2)*RSe6611*
         RS1463e + 1./36.*pow(m11,2)*RC146e*RSe6613 - 1./72.*pow(m11,2)
         *RC146e*RSe6633 - 1./72.*pow(m11,2)*RC146e*RSe6611 );

      M13p6x22RV +=  + hhV(1,m13,m13,m13,m13,L,mu2) * ( 1./16.*pow(
         m33,2) + 1./8.*m11*m33 + 1./16.*pow(m11,2) );

      M13p6x22RV +=  + hhV(1,m13,m33,m33,m13,L,mu2) * ( 1./324.*pow(
         RD346e,2) + 1./81.*m33*RD346e*RS3461e - 1./54.*m33*RD346e*
         RC346e + 5./32.*pow(m33,2) + 17./648.*pow(m33,2)*pow(
         RS3461e,2) + 1./216.*pow(m33,2)*RC346e*RS3461e + 5./288.*pow(
         m33,2)*pow(RC346e,2) + 2./81.*m11*RD346e*RS3461e - 1./54.*m11*
         RD346e*RC346e + 5./16.*m11*m33 + 25./324.*m11*m33*pow(
         RS3461e,2) - 1./36.*m11*m33*RC346e*RS3461e + 5./144.*m11*m33*
         pow(RC346e,2) + 5./32.*pow(m11,2) + 41./648.*pow(m11,2)*pow(
         RS3461e,2) - 7./216.*pow(m11,2)*RC346e*RS3461e + 5./288.*pow(
         m11,2)*pow(RC346e,2) );

      M13p6x22RV +=  + hhV(1,m13,m33,me2,m13,L,mu2) * ( 2./81.*me2*
         RD346e*RSe6613 - 1./81.*me2*RD346e*RSe6633 - 1./81.*me2*
         RSe6611*RD346e - 2./81.*pow(me2,2)*RSe6613*RS3461e + 1./81.*
         pow(me2,2)*RSe6633*RS3461e + 1./81.*pow(me2,2)*RC346e*RSe6613
          - 1./162.*pow(me2,2)*RC346e*RSe6633 + 1./81.*pow(me2,2)*
         RSe6611*RS3461e - 1./162.*pow(me2,2)*RSe6611*RC346e - 4./81.*
         m33*RD346e*RSe6613 + 4./81.*m33*RD346e*RSe6633 + 2./27.*m33*
         me2*RSe6613*RS3461e - 4./81.*m33*me2*RSe6633*RS3461e - 4./81.*
         m33*me2*RC346e*RSe6613 + 4./81.*m33*me2*RC346e*RSe6633 - 2./81.
         *m33*me2*RSe6611*RS3461e - 25./162.*pow(m33,2)*RSe6613*RS3461e
          + 1./36.*pow(m33,2)*RSe6633*RS3461e + 1./36.*pow(m33,2)*
         RC346e*RSe6613 - 73./648.*pow(m33,2)*RC346e*RSe6633 + 1./36.*
         pow(m33,2)*RSe6611*RS3461e - 1./72.*pow(m33,2)*RSe6611*RC346e
          - 4./81.*m11*RD346e*RSe6613 + 1./27.*m11*RD346e*RSe6633 + 1./
         81.*m11*RSe6611*RD346e + 10./81.*m11*me2*RSe6613*RS3461e - 4./
         81.*m11*me2*RSe6633*RS3461e );

      M13p6x22RV +=  + hhV(1,m13,m33,me2,m13,L,mu2) * (  - 4./81.*m11*
         me2*RC346e*RSe6613 + 1./27.*m11*me2*RC346e*RSe6633 - 2./27.*
         m11*me2*RSe6611*RS3461e + 1./81.*m11*me2*RSe6611*RC346e - 35./
         81.*m11*m33*RSe6613*RS3461e + 17./162.*m11*m33*RSe6633*RS3461e
          + 17./162.*m11*m33*RC346e*RSe6613 - 19./108.*m11*m33*RC346e*
         RSe6633 + 13./162.*m11*m33*RSe6611*RS3461e - 1./36.*m11*m33*
         RSe6611*RC346e - 41./162.*pow(m11,2)*RSe6613*RS3461e + 7./108.
         *pow(m11,2)*RSe6633*RS3461e + 7./108.*pow(m11,2)*RC346e*
         RSe6613 - 5./72.*pow(m11,2)*RC346e*RSe6633 + 29./324.*pow(
         m11,2)*RSe6611*RS3461e - 13./648.*pow(m11,2)*RSe6611*RC346e );

      M13p6x22RV +=  + hhV(1,m13,me2,me2,m13,L,mu2) * ( 8./81.*pow(
         me2,2)*pow(RSe6613,2) - 8./81.*pow(me2,2)*RSe6633*RSe6613 + 2./
         81.*pow(me2,2)*pow(RSe6633,2) - 8./81.*pow(me2,2)*RSe6611*
         RSe6613 + 4./81.*pow(me2,2)*RSe6611*RSe6633 + 2./81.*pow(
         me2,2)*pow(RSe6611,2) - 20./81.*m33*me2*pow(RSe6613,2) + 8./27.
         *m33*me2*RSe6633*RSe6613 - 10./81.*m33*me2*pow(RSe6633,2) + 16.
         /81.*m33*me2*RSe6611*RSe6613 - 4./81.*m33*me2*RSe6611*RSe6633
          - 2./27.*m33*me2*pow(RSe6611,2) + 41./162.*pow(m33,2)*pow(
         RSe6613,2) - 29./162.*pow(m33,2)*RSe6633*RSe6613 + 109./648.*
         pow(m33,2)*pow(RSe6633,2) - 7./54.*pow(m33,2)*RSe6611*RSe6613
          + 13./324.*pow(m33,2)*RSe6611*RSe6633 + 5./72.*pow(m33,2)*
         pow(RSe6611,2) - 20./81.*m11*me2*pow(RSe6613,2) + 16./81.*m11*
         me2*RSe6633*RSe6613 - 2./27.*m11*me2*pow(RSe6633,2) + 8./27.*
         m11*me2*RSe6611*RSe6613 - 4./81.*m11*me2*RSe6611*RSe6633 - 10./
         81.*m11*me2*pow(RSe6611,2) + 5./9.*m11*m33*pow(RSe6613,2) - 25.
         /81.*m11*m33*RSe6633*RSe6613 );

      M13p6x22RV +=  + hhV(1,m13,me2,me2,m13,L,mu2) * ( 23./108.*m11*
         m33*pow(RSe6633,2) - 25./81.*m11*m33*RSe6611*RSe6613 + 13./162.
         *m11*m33*RSe6611*RSe6633 + 23./108.*m11*m33*pow(RSe6611,2) + 
         41./162.*pow(m11,2)*pow(RSe6613,2) - 7./54.*pow(m11,2)*RSe6633
         *RSe6613 + 5./72.*pow(m11,2)*pow(RSe6633,2) - 29./162.*pow(
         m11,2)*RSe6611*RSe6613 + 13./324.*pow(m11,2)*RSe6611*RSe6633
          + 109./648.*pow(m11,2)*pow(RSe6611,2) );

      M13p6x22RV +=  + hhV(1,m14,m11,m34,m13,L,mu2) * ( 1./24.*m33*m44*
         RS1463e + 1./24.*m33*m44*RS16e - 1./24.*m33*m44*RC146e + 1./24.
         *m11*m44*RS1463e + 1./24.*m11*m44*RS16e - 1./24.*m11*m44*
         RC146e + 1./24.*m11*m33*RS1463e + 1./24.*m11*m33*RS16e - 1./24.
         *m11*m33*RC146e + 1./24.*pow(m11,2)*RS1463e + 1./24.*pow(
         m11,2)*RS16e - 1./24.*pow(m11,2)*RC146e );

      M13p6x22RV +=  + hhV(1,m14,m33,m34,m13,L,mu2) * (  - 1./9.*m33*
         RD346e + 1./24.*m33*m44*RS3461e - 19./72.*m33*m44*RS36e - 5./
         18.*pow(m33,2)*RS3461e + 1./18.*pow(m33,2)*RS36e - 1./9.*m11*
         RD346e + 1./24.*m11*m44*RS3461e - 19./72.*m11*m44*RS36e - 7./
         24.*m11*m33*RS3461e - 11./72.*m11*m33*RS36e - 1./72.*pow(
         m11,2)*RS3461e - 5./24.*pow(m11,2)*RS36e );

      M13p6x22RV +=  + hhV(1,m14,m34,me2,m13,L,mu2) * ( 1./27.*pow(
         me2,2) + 2./27.*pow(me2,2)*RSe6613 + 2./27.*pow(me2,2)*RSe63
          + 1./27.*pow(me2,2)*RSe6633 + 2./27.*pow(me2,2)*RSe61 + 1./27.
         *pow(me2,2)*RSe6611 - 2./27.*m44*me2 - 2./27.*m44*me2*RSe63 - 
         2./27.*m44*me2*RSe61 + 1./27.*pow(m44,2) - 1./9.*m33*me2 - 2./
         27.*m33*me2*RSe6613 - 5./27.*m33*me2*RSe63 - 2./27.*m33*me2*
         RSe6633 - 1./9.*m33*me2*RSe61 + 1./9.*m33*m44 - 1./12.*m33*m44
         *RSe6613 + 7./216.*m33*m44*RSe63 + 1./24.*m33*m44*RSe61 + 1./
         12.*m33*m44*RSe6611 - 1./3.*pow(m33,2)*RSe6613 - 1./18.*pow(
         m33,2)*RSe63 + 1./27.*pow(m33,2)*RSe6633 - 1./6.*pow(m33,2)*
         RSe61 - 1./9.*m11*me2 - 2./27.*m11*me2*RSe6613 - 1./9.*m11*me2
         *RSe63 - 5./27.*m11*me2*RSe61 - 2./27.*m11*me2*RSe6611 + 1./9.
         *m11*m44 - 1./12.*m11*m44*RSe6613 - 1./24.*m11*m44*RSe63 + 25./
         216.*m11*m44*RSe61 + 1./12.*m11*m44*RSe6611 - 73./108.*m11*m33
         *RSe6613 - 19./72.*m11*m33*RSe63 - 13./72.*m11*m33*RSe61 + 1./
         12.*m11*m33*RSe6611 );

      M13p6x22RV +=  + hhV(1,m14,m34,me2,m13,L,mu2) * (  - 5./12.*pow(
         m11,2)*RSe6613 - 5./24.*pow(m11,2)*RSe63 - 1./72.*pow(m11,2)*
         RSe61 + 13./108.*pow(m11,2)*RSe6611 );

      M13p6x22RV +=  + hhV(1,m16,m11,m36,m13,L,mu2) * ( 1./48.*m33*m66*
         RS1463e + 1./48.*m33*m66*RS14e - 1./48.*m33*m66*RC146e + 1./48.
         *m11*m66*RS1463e + 1./48.*m11*m66*RS14e - 1./48.*m11*m66*
         RC146e + 1./48.*m11*m33*RS1463e + 1./48.*m11*m33*RS14e - 1./48.
         *m11*m33*RC146e + 1./48.*pow(m11,2)*RS1463e + 1./48.*pow(
         m11,2)*RS14e - 1./48.*pow(m11,2)*RC146e );

      M13p6x22RV +=  + hhV(1,m16,m33,m36,m13,L,mu2) * (  - 1./18.*m33*
         RD346e + 1./48.*m33*m66*RS3461e - 19./144.*m33*m66*RS34e - 5./
         36.*pow(m33,2)*RS3461e + 1./36.*pow(m33,2)*RS34e - 1./18.*m11*
         RD346e + 1./48.*m11*m66*RS3461e - 19./144.*m11*m66*RS34e - 7./
         48.*m11*m33*RS3461e - 11./144.*m11*m33*RS34e - 1./144.*pow(
         m11,2)*RS3461e - 5./48.*pow(m11,2)*RS34e );

      M13p6x22RV +=  + hhV(1,m16,m36,me2,m13,L,mu2) * ( 2./27.*pow(
         me2,2) + 1./27.*pow(me2,2)*RSe6613 - 2./27.*pow(me2,2)*RSe63
          + 1./54.*pow(me2,2)*RSe6633 - 2./27.*pow(me2,2)*RSe61 + 1./54.
         *pow(me2,2)*RSe6611 - 4./27.*m66*me2 + 2./27.*m66*me2*RSe63 + 
         2./27.*m66*me2*RSe61 + 2./27.*pow(m66,2) - 2./9.*m33*me2 - 1./
         27.*m33*me2*RSe6613 + 5./27.*m33*me2*RSe63 - 1./27.*m33*me2*
         RSe6633 + 1./9.*m33*me2*RSe61 + 2./9.*m33*m66 - 1./24.*m33*m66
         *RSe6613 - 7./216.*m33*m66*RSe63 - 1./24.*m33*m66*RSe61 + 1./
         24.*m33*m66*RSe6611 - 1./6.*pow(m33,2)*RSe6613 + 1./18.*pow(
         m33,2)*RSe63 + 1./54.*pow(m33,2)*RSe6633 + 1./6.*pow(m33,2)*
         RSe61 - 2./9.*m11*me2 - 1./27.*m11*me2*RSe6613 + 1./9.*m11*me2
         *RSe63 + 5./27.*m11*me2*RSe61 - 1./27.*m11*me2*RSe6611 + 2./9.
         *m11*m66 - 1./24.*m11*m66*RSe6613 + 1./24.*m11*m66*RSe63 - 25./
         216.*m11*m66*RSe61 + 1./24.*m11*m66*RSe6611 - 73./216.*m11*m33
         *RSe6613 + 19./72.*m11*m33*RSe63 + 13./72.*m11*m33*RSe61 + 1./
         24.*m11*m33*RSe6611 );

      M13p6x22RV +=  + hhV(1,m16,m36,me2,m13,L,mu2) * (  - 5./24.*pow(
         m11,2)*RSe6613 + 5./24.*pow(m11,2)*RSe63 + 1./72.*pow(m11,2)*
         RSe61 + 13./216.*pow(m11,2)*RSe6611 );

      M13p6x22RV +=  + hhV(1,m33,m13,m33,m13,L,mu2) * (  - 1./8.*pow(
         m33,2) - 1./8.*m11*m33 );

      M13p6x22RV +=  + hhV(1,m33,m14,m34,m13,L,mu2) * (  - 1./12.*pow(
         m33,2)*RS3461e + 1./12.*pow(m33,2)*RS36e - 1./12.*m11*m33*
         RS3461e + 1./12.*m11*m33*RS36e );

      M13p6x22RV +=  + hhV(1,m33,m16,m36,m13,L,mu2) * (  - 1./24.*pow(
         m33,2)*RS3461e + 1./24.*pow(m33,2)*RS34e - 1./24.*m11*m33*
         RS3461e + 1./24.*m11*m33*RS34e );

      M13p6x22RV +=  + hhV(1,m34,m11,m14,m13,L,mu2) * ( 1./24.*m33*m44*
         RS1463e - 1./24.*m33*m44*RS16e + 1./24.*pow(m33,2)*RS1463e - 1.
         /24.*pow(m33,2)*RS16e + 1./24.*m11*m44*RS1463e - 1./24.*m11*
         m44*RS16e + 1./24.*m11*m33*RS1463e - 1./24.*m11*m33*RS16e );

      M13p6x22RV +=  + hhV(1,m34,m14,m33,m13,L,mu2) * ( 1./24.*m33*m44*
         RS3461e + 1./24.*m33*m44*RS36e - 1./24.*m33*m44*RC346e + 1./24.
         *pow(m33,2)*RS3461e + 1./24.*pow(m33,2)*RS36e - 1./24.*pow(
         m33,2)*RC346e + 1./24.*m11*m44*RS3461e + 1./24.*m11*m44*RS36e
          - 1./24.*m11*m44*RC346e + 1./24.*m11*m33*RS3461e + 1./24.*m11
         *m33*RS36e - 1./24.*m11*m33*RC346e );

      M13p6x22RV +=  + hhV(1,m34,m14,me2,m13,L,mu2) * (  - 1./12.*m33*
         m44*RSe6613 + 1./24.*m33*m44*RSe63 + 1./12.*m33*m44*RSe6633 - 
         1./24.*m33*m44*RSe61 - 1./12.*pow(m33,2)*RSe6613 + 1./24.*pow(
         m33,2)*RSe63 + 1./12.*pow(m33,2)*RSe6633 - 1./24.*pow(m33,2)*
         RSe61 - 1./12.*m11*m44*RSe6613 + 1./24.*m11*m44*RSe63 + 1./12.
         *m11*m44*RSe6633 - 1./24.*m11*m44*RSe61 - 1./12.*m11*m33*
         RSe6613 + 1./24.*m11*m33*RSe63 + 1./12.*m11*m33*RSe6633 - 1./
         24.*m11*m33*RSe61 );

      M13p6x22RV +=  + hhV(1,m36,m11,m16,m13,L,mu2) * ( 1./48.*m33*m66*
         RS1463e - 1./48.*m33*m66*RS14e + 1./48.*pow(m33,2)*RS1463e - 1.
         /48.*pow(m33,2)*RS14e + 1./48.*m11*m66*RS1463e - 1./48.*m11*
         m66*RS14e + 1./48.*m11*m33*RS1463e - 1./48.*m11*m33*RS14e );

      M13p6x22RV +=  + hhV(1,m36,m16,m33,m13,L,mu2) * ( 1./48.*m33*m66*
         RS3461e + 1./48.*m33*m66*RS34e - 1./48.*m33*m66*RC346e + 1./48.
         *pow(m33,2)*RS3461e + 1./48.*pow(m33,2)*RS34e - 1./48.*pow(
         m33,2)*RC346e + 1./48.*m11*m66*RS3461e + 1./48.*m11*m66*RS34e
          - 1./48.*m11*m66*RC346e + 1./48.*m11*m33*RS3461e + 1./48.*m11
         *m33*RS34e - 1./48.*m11*m33*RC346e );

      M13p6x22RV +=  + hhV(1,m36,m16,me2,m13,L,mu2) * (  - 1./24.*m33*
         m66*RSe6613 - 1./24.*m33*m66*RSe63 + 1./24.*m33*m66*RSe6633 + 
         1./24.*m33*m66*RSe61 - 1./24.*pow(m33,2)*RSe6613 - 1./24.*pow(
         m33,2)*RSe63 + 1./24.*pow(m33,2)*RSe6633 + 1./24.*pow(m33,2)*
         RSe61 - 1./24.*m11*m66*RSe6613 - 1./24.*m11*m66*RSe63 + 1./24.
         *m11*m66*RSe6633 + 1./24.*m11*m66*RSe61 - 1./24.*m11*m33*
         RSe6613 - 1./24.*m11*m33*RSe63 + 1./24.*m11*m33*RSe6633 + 1./
         24.*m11*m33*RSe61 );

      M13p6x22RV +=  + hhV(1,m44,m14,m34,m13,L,mu2) * ( 3./8.*m33*m44
          + 3./8.*m11*m44 );

      M13p6x22RV +=  + hhV(1,m46,m14,m36,m13,L,mu2) * ( 1./8.*m33*m66
          + 1./8.*m33*m44 + 1./8.*m11*m66 + 1./8.*m11*m44 );

      M13p6x22RV +=  + hhV(1,m46,m16,m34,m13,L,mu2) * ( 1./8.*m33*m66
          + 1./8.*m33*m44 + 1./8.*m11*m66 + 1./8.*m11*m44 );

      M13p6x22RV +=  + hhV(1,me2,m14,m34,m13,L,mu2) * ( 1./24.*m33*me2
          + 1./6.*m33*me2*RSe6613 + 1./12.*m33*me2*RSe63 + 1./12.*m33*
         me2*RSe61 + 1./24.*m11*me2 + 1./6.*m11*me2*RSe6613 + 1./12.*
         m11*me2*RSe63 + 1./12.*m11*me2*RSe61 );

      M13p6x22RV +=  + hhV(1,me2,m16,m36,m13,L,mu2) * ( 1./12.*m33*me2
          + 1./12.*m33*me2*RSe6613 - 1./12.*m33*me2*RSe63 - 1./12.*m33*
         me2*RSe61 + 1./12.*m11*me2 + 1./12.*m11*me2*RSe6613 - 1./12.*
         m11*me2*RSe63 - 1./12.*m11*me2*RSe61 );

      M13p6x22RV +=  + hhV(2,m11,m11,m13,m13,L,mu2) * ( 1./18.*pow(
         m33,2)*RD146e*RS1463e - 1./36.*pow(m33,2)*RD146e*RC146e + 1./9.
         *m11*m33*RD146e*RS1463e - 1./18.*m11*m33*RD146e*RC146e + 1./18.
         *pow(m11,2)*RD146e*RS1463e - 1./36.*pow(m11,2)*RD146e*RC146e )
         ;

      M13p6x22RV +=  + hhV(2,m11,m13,me2,m13,L,mu2) * ( 1./81.*pow(
         me2,2)*RD146e*RSe6613 - 1./162.*pow(me2,2)*RD146e*RSe6633 - 1./
         162.*pow(me2,2)*RD146e*RSe6611 - 4./81.*m33*me2*RD146e*RSe6613
          + 1./81.*m33*me2*RD146e*RSe6633 + 1./27.*m33*me2*RD146e*
         RSe6611 + 1./27.*pow(m33,2)*RD146e*RSe6613 - 1./162.*pow(
         m33,2)*RD146e*RSe6633 - 1./18.*pow(m33,2)*RD146e*RSe6611 - 4./
         81.*m11*me2*RD146e*RSe6613 + 4./81.*m11*me2*RD146e*RSe6611 + 4.
         /81.*m11*m33*RD146e*RSe6613 - 4./27.*m11*m33*RD146e*RSe6611 - 
         8./81.*pow(m11,2)*RD146e*RSe6611 );

      M13p6x22RV +=  + hhV(3,m13,m11,m11,m13,L,mu2) * (  - 1./72.*pow(
         m33,2)*RD146e*RS1463e + 1./144.*pow(m33,2)*RD146e*RC146e - 1./
         36.*m11*m33*RD146e*RS1463e + 1./72.*m11*m33*RD146e*RC146e - 1./
         72.*pow(m11,2)*RD146e*RS1463e + 1./144.*pow(m11,2)*RD146e*
         RC146e );

      M13p6x22RV +=  + hhV(3,m13,m11,m33,m13,L,mu2) * (  - 1./72.*pow(
         m33,2)*RD146e*RS3461e + 1./144.*pow(m33,2)*RD146e*RC346e - 1./
         36.*m11*m33*RD146e*RS3461e + 1./72.*m11*m33*RD146e*RC346e - 1./
         72.*pow(m11,2)*RD146e*RS3461e + 1./144.*pow(m11,2)*RD146e*
         RC346e );

      M13p6x22RV +=  + hhV(3,m13,m11,me2,m13,L,mu2) * ( 1./36.*pow(
         m33,2)*RD146e*RSe6613 - 1./72.*pow(m33,2)*RD146e*RSe6633 - 1./
         72.*pow(m33,2)*RD146e*RSe6611 + 1./18.*m11*m33*RD146e*RSe6613
          - 1./36.*m11*m33*RD146e*RSe6633 - 1./36.*m11*m33*RD146e*
         RSe6611 + 1./36.*pow(m11,2)*RD146e*RSe6613 - 1./72.*pow(m11,2)
         *RD146e*RSe6633 - 1./72.*pow(m11,2)*RD146e*RSe6611 );

      M13p6x22RV +=  + hhV(3,m13,m33,m33,m13,L,mu2) * (  - 23./1296.*
         m33*pow(RD346e,2) + 1./216.*pow(m33,2)*RD346e*RS3461e + 5./144.
         *pow(m33,2)*RD346e*RC346e - 23./1296.*m11*pow(RD346e,2) - 1./
         36.*m11*m33*RD346e*RS3461e + 5./72.*m11*m33*RD346e*RC346e - 7./
         216.*pow(m11,2)*RD346e*RS3461e + 5./144.*pow(m11,2)*RD346e*
         RC346e );

      M13p6x22RV +=  + hhV(3,m13,m33,me2,m13,L,mu2) * ( 1./81.*pow(
         me2,2)*RD346e*RSe6613 - 1./162.*pow(me2,2)*RD346e*RSe6633 - 1./
         162.*pow(me2,2)*RSe6611*RD346e - 4./81.*m33*me2*RD346e*RSe6613
          + 4./81.*m33*me2*RD346e*RSe6633 + 1./36.*pow(m33,2)*RD346e*
         RSe6613 - 73./648.*pow(m33,2)*RD346e*RSe6633 - 1./72.*pow(
         m33,2)*RSe6611*RD346e - 4./81.*m11*me2*RD346e*RSe6613 + 1./27.
         *m11*me2*RD346e*RSe6633 + 1./81.*m11*me2*RSe6611*RD346e + 17./
         162.*m11*m33*RD346e*RSe6613 - 19./108.*m11*m33*RD346e*RSe6633
          - 1./36.*m11*m33*RSe6611*RD346e + 7./108.*pow(m11,2)*RD346e*
         RSe6613 - 5./72.*pow(m11,2)*RD346e*RSe6633 - 13./648.*pow(
         m11,2)*RSe6611*RD346e );

      M13p6x22RV +=  + hhV(3,m14,m11,m34,m13,L,mu2) * (  - 1./24.*m33*
         m44*RD146e - 1./24.*m11*m44*RD146e - 1./24.*m11*m33*RD146e - 1.
         /24.*pow(m11,2)*RD146e );

      M13p6x22RV +=  + hhV(3,m16,m11,m36,m13,L,mu2) * (  - 1./48.*m33*
         m66*RD146e - 1./48.*m11*m66*RD146e - 1./48.*m11*m33*RD146e - 1.
         /48.*pow(m11,2)*RD146e );

      M13p6x22RV +=  + hhV(4,m13,m11,m33,m13,L,mu2) * (  - 1./72.*pow(
         m33,2)*RD346e*RS1463e + 1./144.*pow(m33,2)*RC146e*RD346e - 1./
         36.*m11*m33*RD346e*RS1463e + 1./72.*m11*m33*RC146e*RD346e - 1./
         72.*pow(m11,2)*RD346e*RS1463e + 1./144.*pow(m11,2)*RC146e*
         RD346e );

      M13p6x22RV +=  + hhV(4,m13,m33,m33,m13,L,mu2) * (  - 1./1296.*m33
         *pow(RD346e,2) - 1./1296.*m11*pow(RD346e,2) );

      M13p6x22RV +=  + hhV(4,m34,m14,m33,m13,L,mu2) * (  - 1./24.*m33*
         m44*RD346e - 1./24.*pow(m33,2)*RD346e - 1./24.*m11*m44*RD346e
          - 1./24.*m11*m33*RD346e );

      M13p6x22RV +=  + hhV(4,m36,m16,m33,m13,L,mu2) * (  - 1./48.*m33*
         m66*RD346e - 1./48.*pow(m33,2)*RD346e - 1./48.*m11*m66*RD346e
          - 1./48.*m11*m33*RD346e );

      M13p6x22RV +=  + hhV(5,m11,m11,m13,m13,L,mu2) * (  - 1./72.*pow(
         m33,2)*pow(RD146e,2) - 1./36.*m11*m33*pow(RD146e,2) - 1./72.*
         pow(m11,2)*pow(RD146e,2) );

      M13p6x22RV +=  + hhV(7,m13,m11,m11,m13,L,mu2) * ( 1./288.*pow(
         m33,2)*pow(RD146e,2) + 1./144.*m11*m33*pow(RD146e,2) + 1./288.
         *pow(m11,2)*pow(RD146e,2) );

      M13p6x22RV +=  + hhV(7,m13,m11,m33,m13,L,mu2) * ( 1./144.*pow(
         m33,2)*RD146e*RD346e + 1./72.*m11*m33*RD146e*RD346e + 1./144.*
         pow(m11,2)*RD146e*RD346e );

      M13p6x22RV +=  + hhV(7,m13,m33,m33,m13,L,mu2) * ( 5./288.*pow(
         m33,2)*pow(RD346e,2) + 5./144.*m11*m33*pow(RD346e,2) + 5./288.
         *pow(m11,2)*pow(RD346e,2) );

      M13p6x22RV +=  + hh1V(1,m11,m11,m13,m13,L,mu2) * ( 2./27.*m33*
         RD146e*RS1463e - 1./27.*m33*RD146e*RC146e + 1./2.*pow(m33,2)
          + 5./27.*pow(m33,2)*pow(RS1463e,2) - 4./27.*pow(m33,2)*RC146e
         *RS1463e + 1./18.*pow(m33,2)*pow(RC146e,2) + 2./27.*m11*RD146e
         *RS1463e - 1./27.*m11*RD146e*RC146e + m11*m33 + 2./9.*m11*m33*
         pow(RS1463e,2) - 2./9.*m11*m33*RC146e*RS1463e + 1./9.*m11*m33*
         pow(RC146e,2) + 1./2.*pow(m11,2) + 1./27.*pow(m11,2)*pow(
         RS1463e,2) - 2./27.*pow(m11,2)*RC146e*RS1463e + 1./18.*pow(
         m11,2)*pow(RC146e,2) );

      M13p6x22RV +=  + hh1V(1,m11,m14,m34,m13,L,mu2) * ( 1./3.*pow(
         m33,2)*RS1463e - 1./3.*pow(m33,2)*RS16e + 2./3.*m11*m33*
         RS1463e - 2./3.*m11*m33*RS16e + 1./3.*pow(m11,2)*RS1463e - 1./
         3.*pow(m11,2)*RS16e );

      M13p6x22RV +=  + hh1V(1,m11,m16,m36,m13,L,mu2) * ( 1./6.*pow(
         m33,2)*RS1463e - 1./6.*pow(m33,2)*RS14e + 1./3.*m11*m33*
         RS1463e - 1./3.*m11*m33*RS14e + 1./6.*pow(m11,2)*RS1463e - 1./
         6.*pow(m11,2)*RS14e );

      M13p6x22RV +=  + hh1V(1,m13,m11,m33,m13,L,mu2) * (  - 1./27.*m33*
         RD346e*RS1463e + 1./54.*m33*RC146e*RD346e - 1./27.*m33*RD146e*
         RS3461e + 1./54.*m33*RD146e*RC346e - 1./9.*pow(m33,2)*RS1463e*
         RS3461e + 2./27.*pow(m33,2)*RC346e*RS1463e + 1./27.*pow(m33,2)
         *RC146e*RS3461e - 1./27.*m11*RD346e*RS1463e + 1./54.*m11*
         RC146e*RD346e - 1./27.*m11*RD146e*RS3461e + 1./54.*m11*RD146e*
         RC346e - 2./9.*m11*m33*RS1463e*RS3461e + 1./9.*m11*m33*RC346e*
         RS1463e + 1./9.*m11*m33*RC146e*RS3461e - 1./9.*pow(m11,2)*
         RS1463e*RS3461e + 1./27.*pow(m11,2)*RC346e*RS1463e + 2./27.*
         pow(m11,2)*RC146e*RS3461e );

      M13p6x22RV +=  + hh1V(1,m13,m11,me2,m13,L,mu2) * ( 2./27.*m33*
         RD146e*RSe6613 - 1./27.*m33*RD146e*RSe6633 - 1./27.*m33*RD146e
         *RSe6611 - 4./27.*m33*me2*RSe6613*RS1463e + 2./27.*m33*me2*
         RSe6633*RS1463e + 2./27.*m33*me2*RSe6611*RS1463e + 2./27.*m33*
         me2*RC146e*RSe6613 - 1./27.*m33*me2*RC146e*RSe6633 - 1./27.*
         m33*me2*RC146e*RSe6611 + 10./27.*pow(m33,2)*RSe6613*RS1463e - 
         2./9.*pow(m33,2)*RSe6633*RS1463e - 4./27.*pow(m33,2)*RSe6611*
         RS1463e - 4./27.*pow(m33,2)*RC146e*RSe6613 + 1./27.*pow(m33,2)
         *RC146e*RSe6633 + 1./9.*pow(m33,2)*RC146e*RSe6611 + 2./27.*m11
         *RD146e*RSe6613 - 1./27.*m11*RD146e*RSe6633 - 1./27.*m11*
         RD146e*RSe6611 - 4./27.*m11*me2*RSe6613*RS1463e + 2./27.*m11*
         me2*RSe6633*RS1463e + 2./27.*m11*me2*RSe6611*RS1463e + 2./27.*
         m11*me2*RC146e*RSe6613 - 1./27.*m11*me2*RC146e*RSe6633 - 1./27.
         *m11*me2*RC146e*RSe6611 + 16./27.*m11*m33*RSe6613*RS1463e - 8./
         27.*m11*m33*RSe6633*RS1463e - 8./27.*m11*m33*RSe6611*RS1463e
          - 8./27.*m11*m33*RC146e*RSe6613 );

      M13p6x22RV +=  + hh1V(1,m13,m11,me2,m13,L,mu2) * ( 1./27.*m11*m33
         *RC146e*RSe6633 + 7./27.*m11*m33*RC146e*RSe6611 + 2./9.*pow(
         m11,2)*RSe6613*RS1463e - 2./27.*pow(m11,2)*RSe6633*RS1463e - 4.
         /27.*pow(m11,2)*RSe6611*RS1463e - 4./27.*pow(m11,2)*RC146e*
         RSe6613 + 4./27.*pow(m11,2)*RC146e*RSe6611 );

      M13p6x22RV +=  + hh1V(1,m13,m33,m33,m13,L,mu2) * (  - 1./27.*m33*
         RD346e*RS3461e + 1./54.*m33*RD346e*RC346e - 1./4.*pow(m33,2)
          - 1./54.*pow(m33,2)*pow(RS3461e,2) + 1./27.*pow(m33,2)*RC346e
         *RS3461e - 1./36.*pow(m33,2)*pow(RC346e,2) - 1./27.*m11*RD346e
         *RS3461e + 1./54.*m11*RD346e*RC346e - 1./2.*m11*m33 - 1./9.*
         m11*m33*pow(RS3461e,2) + 1./9.*m11*m33*RC346e*RS3461e - 1./18.
         *m11*m33*pow(RC346e,2) - 1./4.*pow(m11,2) - 5./54.*pow(m11,2)*
         pow(RS3461e,2) + 2./27.*pow(m11,2)*RC346e*RS3461e - 1./36.*
         pow(m11,2)*pow(RC346e,2) );

      M13p6x22RV +=  + hh1V(1,m13,m33,me2,m13,L,mu2) * ( 2./27.*m33*
         RD346e*RSe6613 - 1./27.*m33*RD346e*RSe6633 - 1./27.*m33*
         RSe6611*RD346e - 4./27.*m33*me2*RSe6613*RS3461e + 2./27.*m33*
         me2*RSe6633*RS3461e + 2./27.*m33*me2*RC346e*RSe6613 - 1./27.*
         m33*me2*RC346e*RSe6633 + 2./27.*m33*me2*RSe6611*RS3461e - 1./
         27.*m33*me2*RSe6611*RC346e + 2./9.*pow(m33,2)*RSe6613*RS3461e
          - 4./27.*pow(m33,2)*RSe6633*RS3461e - 4./27.*pow(m33,2)*
         RC346e*RSe6613 + 4./27.*pow(m33,2)*RC346e*RSe6633 - 2./27.*
         pow(m33,2)*RSe6611*RS3461e + 2./27.*m11*RD346e*RSe6613 - 1./27.
         *m11*RD346e*RSe6633 - 1./27.*m11*RSe6611*RD346e - 4./27.*m11*
         me2*RSe6613*RS3461e + 2./27.*m11*me2*RSe6633*RS3461e + 2./27.*
         m11*me2*RC346e*RSe6613 - 1./27.*m11*me2*RC346e*RSe6633 + 2./27.
         *m11*me2*RSe6611*RS3461e - 1./27.*m11*me2*RSe6611*RC346e + 16./
         27.*m11*m33*RSe6613*RS3461e - 8./27.*m11*m33*RSe6633*RS3461e
          - 8./27.*m11*m33*RC346e*RSe6613 + 7./27.*m11*m33*RC346e*
         RSe6633 );

      M13p6x22RV +=  + hh1V(1,m13,m33,me2,m13,L,mu2) * (  - 8./27.*m11*
         m33*RSe6611*RS3461e + 1./27.*m11*m33*RSe6611*RC346e + 10./27.*
         pow(m11,2)*RSe6613*RS3461e - 4./27.*pow(m11,2)*RSe6633*RS3461e
          - 4./27.*pow(m11,2)*RC346e*RSe6613 + 1./9.*pow(m11,2)*RC346e*
         RSe6633 - 2./9.*pow(m11,2)*RSe6611*RS3461e + 1./27.*pow(m11,2)
         *RSe6611*RC346e );

      M13p6x22RV +=  + hh1V(1,m13,me2,me2,m13,L,mu2) * ( 8./27.*m33*me2
         *pow(RSe6613,2) - 8./27.*m33*me2*RSe6633*RSe6613 + 2./27.*m33*
         me2*pow(RSe6633,2) - 8./27.*m33*me2*RSe6611*RSe6613 + 4./27.*
         m33*me2*RSe6611*RSe6633 + 2./27.*m33*me2*pow(RSe6611,2) - 10./
         27.*pow(m33,2)*pow(RSe6613,2) + 4./9.*pow(m33,2)*RSe6633*
         RSe6613 - 5./27.*pow(m33,2)*pow(RSe6633,2) + 8./27.*pow(m33,2)
         *RSe6611*RSe6613 - 2./27.*pow(m33,2)*RSe6611*RSe6633 - 1./9.*
         pow(m33,2)*pow(RSe6611,2) + 8./27.*m11*me2*pow(RSe6613,2) - 8./
         27.*m11*me2*RSe6633*RSe6613 + 2./27.*m11*me2*pow(RSe6633,2) - 
         8./27.*m11*me2*RSe6611*RSe6613 + 4./27.*m11*me2*RSe6611*
         RSe6633 + 2./27.*m11*me2*pow(RSe6611,2) - 20./27.*m11*m33*pow(
         RSe6613,2) + 20./27.*m11*m33*RSe6633*RSe6613 - 8./27.*m11*m33*
         pow(RSe6633,2) + 20./27.*m11*m33*RSe6611*RSe6613 - 4./27.*m11*
         m33*RSe6611*RSe6633 - 8./27.*m11*m33*pow(RSe6611,2) - 10./27.*
         pow(m11,2)*pow(RSe6613,2) + 8./27.*pow(m11,2)*RSe6633*RSe6613
          - 1./9.*pow(m11,2)*pow(RSe6633,2) );

      M13p6x22RV +=  + hh1V(1,m13,me2,me2,m13,L,mu2) * ( 4./9.*pow(
         m11,2)*RSe6611*RSe6613 - 2./27.*pow(m11,2)*RSe6611*RSe6633 - 5.
         /27.*pow(m11,2)*pow(RSe6611,2) );

      M13p6x22RV +=  + hh1V(1,m14,m11,m34,m13,L,mu2) * (  - 1./9.*m33*
         RD146e - 2./9.*m33*m44*RS16e + 1./9.*pow(m33,2)*RS1463e - 1./3.
         *pow(m33,2)*RS16e - 1./9.*m11*RD146e - 2./9.*m11*m44*RS16e - 4.
         /9.*m11*m33*RS16e - 1./9.*pow(m11,2)*RS1463e - 1./9.*pow(
         m11,2)*RS16e );

      M13p6x22RV +=  + hh1V(1,m14,m33,m34,m13,L,mu2) * ( 1./9.*m33*
         RD346e + 2./9.*m33*m44*RS36e + 1./9.*pow(m33,2)*RS3461e + 1./9.
         *pow(m33,2)*RS36e + 1./9.*m11*RD346e + 2./9.*m11*m44*RS36e + 4.
         /9.*m11*m33*RS36e - 1./9.*pow(m11,2)*RS3461e + 1./3.*pow(
         m11,2)*RS36e );

      M13p6x22RV +=  + hh1V(1,m14,m34,me2,m13,L,mu2) * ( 1./9.*m33*me2
          + 2./9.*m33*me2*RSe6613 + 1./9.*m33*me2*RSe63 + 1./3.*m33*me2
         *RSe61 + 2./9.*m33*me2*RSe6611 - 1./9.*m33*m44 - 2./9.*m33*m44
         *RSe61 + 4./9.*pow(m33,2)*RSe6613 + 2./9.*pow(m33,2)*RSe63 + 1.
         /9.*m11*me2 + 2./9.*m11*me2*RSe6613 + 1./9.*m11*me2*RSe63 + 1./
         3.*m11*me2*RSe61 + 2./9.*m11*me2*RSe6611 - 1./9.*m11*m44 - 2./
         9.*m11*m44*RSe61 + 10./9.*m11*m33*RSe6613 + 5./9.*m11*m33*
         RSe63 - 1./9.*m11*m33*RSe61 - 2./9.*m11*m33*RSe6611 + 2./3.*
         pow(m11,2)*RSe6613 + 1./3.*pow(m11,2)*RSe63 - 1./9.*pow(m11,2)
         *RSe61 - 2./9.*pow(m11,2)*RSe6611 );

      M13p6x22RV +=  + hh1V(1,m16,m11,m36,m13,L,mu2) * (  - 1./18.*m33*
         RD146e - 1./9.*m33*m66*RS14e + 1./18.*pow(m33,2)*RS1463e - 1./
         6.*pow(m33,2)*RS14e - 1./18.*m11*RD146e - 1./9.*m11*m66*RS14e
          - 2./9.*m11*m33*RS14e - 1./18.*pow(m11,2)*RS1463e - 1./18.*
         pow(m11,2)*RS14e );

      M13p6x22RV +=  + hh1V(1,m16,m33,m36,m13,L,mu2) * ( 1./18.*m33*
         RD346e + 1./9.*m33*m66*RS34e + 1./18.*pow(m33,2)*RS3461e + 1./
         18.*pow(m33,2)*RS34e + 1./18.*m11*RD346e + 1./9.*m11*m66*RS34e
          + 2./9.*m11*m33*RS34e - 1./18.*pow(m11,2)*RS3461e + 1./6.*
         pow(m11,2)*RS34e );

      M13p6x22RV +=  + hh1V(1,m16,m36,me2,m13,L,mu2) * ( 2./9.*m33*me2
          + 1./9.*m33*me2*RSe6613 - 1./9.*m33*me2*RSe63 - 1./3.*m33*me2
         *RSe61 + 1./9.*m33*me2*RSe6611 - 2./9.*m33*m66 + 2./9.*m33*m66
         *RSe61 + 2./9.*pow(m33,2)*RSe6613 - 2./9.*pow(m33,2)*RSe63 + 2.
         /9.*m11*me2 + 1./9.*m11*me2*RSe6613 - 1./9.*m11*me2*RSe63 - 1./
         3.*m11*me2*RSe61 + 1./9.*m11*me2*RSe6611 - 2./9.*m11*m66 + 2./
         9.*m11*m66*RSe61 + 5./9.*m11*m33*RSe6613 - 5./9.*m11*m33*RSe63
          + 1./9.*m11*m33*RSe61 - 1./9.*m11*m33*RSe6611 + 1./3.*pow(
         m11,2)*RSe6613 - 1./3.*pow(m11,2)*RSe63 + 1./9.*pow(m11,2)*
         RSe61 - 1./9.*pow(m11,2)*RSe6611 );

      M13p6x22RV +=  + hh1V(1,m33,m14,m34,m13,L,mu2) * ( 1./9.*m33*
         RD346e + 2./9.*m33*m44*RS36e + 4./9.*pow(m33,2)*RS3461e - 2./9.
         *pow(m33,2)*RS36e + 1./9.*m11*RD346e + 2./9.*m11*m44*RS36e + 2.
         /3.*m11*m33*RS3461e - 2./9.*m11*m33*RS36e + 2./9.*pow(m11,2)*
         RS3461e );

      M13p6x22RV +=  + hh1V(1,m33,m16,m36,m13,L,mu2) * ( 1./18.*m33*
         RD346e + 1./9.*m33*m66*RS34e + 2./9.*pow(m33,2)*RS3461e - 1./9.
         *pow(m33,2)*RS34e + 1./18.*m11*RD346e + 1./9.*m11*m66*RS34e + 
         1./3.*m11*m33*RS3461e - 1./9.*m11*m33*RS34e + 1./9.*pow(m11,2)
         *RS3461e );

      M13p6x22RV +=  + hh1V(1,m34,m14,me2,m13,L,mu2) * ( 1./9.*m33*me2
          + 2./9.*m33*me2*RSe6613 + 1./3.*m33*me2*RSe63 + 2./9.*m33*me2
         *RSe6633 + 1./9.*m33*me2*RSe61 - 1./9.*m33*m44 - 2./9.*m33*m44
         *RSe63 + 2./3.*pow(m33,2)*RSe6613 - 1./9.*pow(m33,2)*RSe63 - 2.
         /9.*pow(m33,2)*RSe6633 + 1./3.*pow(m33,2)*RSe61 + 1./9.*m11*
         me2 + 2./9.*m11*me2*RSe6613 + 1./3.*m11*me2*RSe63 + 2./9.*m11*
         me2*RSe6633 + 1./9.*m11*me2*RSe61 - 1./9.*m11*m44 - 2./9.*m11*
         m44*RSe63 + 10./9.*m11*m33*RSe6613 - 1./9.*m11*m33*RSe63 - 2./
         9.*m11*m33*RSe6633 + 5./9.*m11*m33*RSe61 + 4./9.*pow(m11,2)*
         RSe6613 + 2./9.*pow(m11,2)*RSe61 );

      M13p6x22RV +=  + hh1V(1,m36,m16,me2,m13,L,mu2) * ( 2./9.*m33*me2
          + 1./9.*m33*me2*RSe6613 - 1./3.*m33*me2*RSe63 + 1./9.*m33*me2
         *RSe6633 - 1./9.*m33*me2*RSe61 - 2./9.*m33*m66 + 2./9.*m33*m66
         *RSe63 + 1./3.*pow(m33,2)*RSe6613 + 1./9.*pow(m33,2)*RSe63 - 1.
         /9.*pow(m33,2)*RSe6633 - 1./3.*pow(m33,2)*RSe61 + 2./9.*m11*
         me2 + 1./9.*m11*me2*RSe6613 - 1./3.*m11*me2*RSe63 + 1./9.*m11*
         me2*RSe6633 - 1./9.*m11*me2*RSe61 - 2./9.*m11*m66 + 2./9.*m11*
         m66*RSe63 + 5./9.*m11*m33*RSe6613 + 1./9.*m11*m33*RSe63 - 1./9.
         *m11*m33*RSe6633 - 5./9.*m11*m33*RSe61 + 2./9.*pow(m11,2)*
         RSe6613 - 2./9.*pow(m11,2)*RSe61 );

      M13p6x22RV +=  + hh1V(2,m11,m11,m13,m13,L,mu2) * (  - 1./54.*m33*
         pow(RD146e,2) - 2./27.*pow(m33,2)*RD146e*RS1463e + 1./18.*pow(
         m33,2)*RD146e*RC146e - 1./54.*m11*pow(RD146e,2) - 1./9.*m11*
         m33*RD146e*RS1463e + 1./9.*m11*m33*RD146e*RC146e - 1./27.*pow(
         m11,2)*RD146e*RS1463e + 1./18.*pow(m11,2)*RD146e*RC146e );

      M13p6x22RV +=  + hh1V(3,m11,m11,m13,m13,L,mu2) * (  - 1./54.*m33*
         pow(RD146e,2) - 2./27.*pow(m33,2)*RD146e*RS1463e + 1./18.*pow(
         m33,2)*RD146e*RC146e - 1./54.*m11*pow(RD146e,2) - 1./9.*m11*
         m33*RD146e*RS1463e + 1./9.*m11*m33*RD146e*RC146e - 1./27.*pow(
         m11,2)*RD146e*RS1463e + 1./18.*pow(m11,2)*RD146e*RC146e );

      M13p6x22RV +=  + hh1V(3,m13,m11,m33,m13,L,mu2) * ( 1./54.*m33*
         RD146e*RD346e + 1./27.*pow(m33,2)*RD146e*RS3461e + 1./54.*m11*
         RD146e*RD346e + 1./9.*m11*m33*RD146e*RS3461e + 2./27.*pow(
         m11,2)*RD146e*RS3461e );

      M13p6x22RV +=  + hh1V(3,m13,m11,me2,m13,L,mu2) * ( 2./27.*m33*me2
         *RD146e*RSe6613 - 1./27.*m33*me2*RD146e*RSe6633 - 1./27.*m33*
         me2*RD146e*RSe6611 - 4./27.*pow(m33,2)*RD146e*RSe6613 + 1./27.
         *pow(m33,2)*RD146e*RSe6633 + 1./9.*pow(m33,2)*RD146e*RSe6611
          + 2./27.*m11*me2*RD146e*RSe6613 - 1./27.*m11*me2*RD146e*
         RSe6633 - 1./27.*m11*me2*RD146e*RSe6611 - 8./27.*m11*m33*
         RD146e*RSe6613 + 1./27.*m11*m33*RD146e*RSe6633 + 7./27.*m11*
         m33*RD146e*RSe6611 - 4./27.*pow(m11,2)*RD146e*RSe6613 + 4./27.
         *pow(m11,2)*RD146e*RSe6611 );

      M13p6x22RV +=  + hh1V(3,m13,m33,m33,m13,L,mu2) * ( 23./1296.*m33*
         pow(RD346e,2) + 1./27.*pow(m33,2)*RD346e*RS3461e - 1./18.*pow(
         m33,2)*RD346e*RC346e + 23./1296.*m11*pow(RD346e,2) + 1./9.*m11
         *m33*RD346e*RS3461e - 1./9.*m11*m33*RD346e*RC346e + 2./27.*
         pow(m11,2)*RD346e*RS3461e - 1./18.*pow(m11,2)*RD346e*RC346e );

      M13p6x22RV +=  + hh1V(3,m13,m33,me2,m13,L,mu2) * ( 2./27.*m33*me2
         *RD346e*RSe6613 - 1./27.*m33*me2*RD346e*RSe6633 - 1./27.*m33*
         me2*RSe6611*RD346e - 4./27.*pow(m33,2)*RD346e*RSe6613 + 4./27.
         *pow(m33,2)*RD346e*RSe6633 + 2./27.*m11*me2*RD346e*RSe6613 - 1.
         /27.*m11*me2*RD346e*RSe6633 - 1./27.*m11*me2*RSe6611*RD346e - 
         8./27.*m11*m33*RD346e*RSe6613 + 7./27.*m11*m33*RD346e*RSe6633
          + 1./27.*m11*m33*RSe6611*RD346e - 4./27.*pow(m11,2)*RD346e*
         RSe6613 + 1./9.*pow(m11,2)*RD346e*RSe6633 + 1./27.*pow(m11,2)*
         RSe6611*RD346e );

      M13p6x22RV +=  + hh1V(4,m13,m11,m33,m13,L,mu2) * ( 1./54.*m33*
         RD146e*RD346e + 2./27.*pow(m33,2)*RD346e*RS1463e + 1./54.*m11*
         RD146e*RD346e + 1./9.*m11*m33*RD346e*RS1463e + 1./27.*pow(
         m11,2)*RD346e*RS1463e );

      M13p6x22RV +=  + hh1V(4,m13,m33,m33,m13,L,mu2) * ( 1./1296.*m33*
         pow(RD346e,2) + 1./1296.*m11*pow(RD346e,2) );

      M13p6x22RV +=  + hh1V(5,m11,m11,m13,m13,L,mu2) * ( 1./18.*pow(
         m33,2)*pow(RD146e,2) + 1./9.*m11*m33*pow(RD146e,2) + 1./18.*
         pow(m11,2)*pow(RD146e,2) );

      M13p6x22RV +=  + hh1V(7,m13,m33,m33,m13,L,mu2) * (  - 1./36.*pow(
         m33,2)*pow(RD346e,2) - 1./18.*m11*m33*pow(RD346e,2) - 1./36.*
         pow(m11,2)*pow(RD346e,2) );

      M13p6x22RV +=  + hh21V(1,m11,m11,m13,m13,L,mu2) * (  - 3./16.*
         pow(m33,2) - 3./8.*m11*m33 - 3./16.*pow(m11,2) );

      M13p6x22RV +=  + hh21V(1,m11,m14,m34,m13,L,mu2) * (  - 1./8.*pow(
         m33,2)*RS1463e + 1./8.*pow(m33,2)*RS16e - 1./4.*m11*m33*
         RS1463e + 1./4.*m11*m33*RS16e - 1./8.*pow(m11,2)*RS1463e + 1./
         8.*pow(m11,2)*RS16e );

      M13p6x22RV +=  + hh21V(1,m11,m16,m36,m13,L,mu2) * (  - 1./16.*
         pow(m33,2)*RS1463e + 1./16.*pow(m33,2)*RS14e - 1./8.*m11*m33*
         RS1463e + 1./8.*m11*m33*RS14e - 1./16.*pow(m11,2)*RS1463e + 1./
         16.*pow(m11,2)*RS14e );

      M13p6x22RV +=  + hh21V(1,m13,m11,m11,m13,L,mu2) * ( 3./32.*pow(
         m33,2) + 1./24.*pow(m33,2)*pow(RS1463e,2) - 1./24.*pow(m33,2)*
         RC146e*RS1463e + 1./96.*pow(m33,2)*pow(RC146e,2) + 3./16.*m11*
         m33 + 1./12.*m11*m33*pow(RS1463e,2) - 1./12.*m11*m33*RC146e*
         RS1463e + 1./48.*m11*m33*pow(RC146e,2) + 3./32.*pow(m11,2) + 1.
         /24.*pow(m11,2)*pow(RS1463e,2) - 1./24.*pow(m11,2)*RC146e*
         RS1463e + 1./96.*pow(m11,2)*pow(RC146e,2) );

      M13p6x22RV +=  + hh21V(1,m13,m11,m33,m13,L,mu2) * ( 1./12.*pow(
         m33,2)*RS1463e*RS3461e - 1./24.*pow(m33,2)*RC346e*RS1463e - 1./
         24.*pow(m33,2)*RC146e*RS3461e + 1./48.*pow(m33,2)*RC146e*
         RC346e + 1./6.*m11*m33*RS1463e*RS3461e - 1./12.*m11*m33*RC346e
         *RS1463e - 1./12.*m11*m33*RC146e*RS3461e + 1./24.*m11*m33*
         RC146e*RC346e + 1./12.*pow(m11,2)*RS1463e*RS3461e - 1./24.*
         pow(m11,2)*RC346e*RS1463e - 1./24.*pow(m11,2)*RC146e*RS3461e
          + 1./48.*pow(m11,2)*RC146e*RC346e );

      M13p6x22RV +=  + hh21V(1,m13,m11,me2,m13,L,mu2) * (  - 1./6.*pow(
         m33,2)*RSe6613*RS1463e + 1./12.*pow(m33,2)*RSe6633*RS1463e + 1.
         /12.*pow(m33,2)*RSe6611*RS1463e + 1./12.*pow(m33,2)*RC146e*
         RSe6613 - 1./24.*pow(m33,2)*RC146e*RSe6633 - 1./24.*pow(m33,2)
         *RC146e*RSe6611 - 1./3.*m11*m33*RSe6613*RS1463e + 1./6.*m11*
         m33*RSe6633*RS1463e + 1./6.*m11*m33*RSe6611*RS1463e + 1./6.*
         m11*m33*RC146e*RSe6613 - 1./12.*m11*m33*RC146e*RSe6633 - 1./12.
         *m11*m33*RC146e*RSe6611 - 1./6.*pow(m11,2)*RSe6613*RS1463e + 1.
         /12.*pow(m11,2)*RSe6633*RS1463e + 1./12.*pow(m11,2)*RSe6611*
         RS1463e + 1./12.*pow(m11,2)*RC146e*RSe6613 - 1./24.*pow(m11,2)
         *RC146e*RSe6633 - 1./24.*pow(m11,2)*RC146e*RSe6611 );

      M13p6x22RV +=  + hh21V(1,m13,m13,m13,m13,L,mu2) * ( 3./16.*pow(
         m33,2) + 3./8.*m11*m33 + 3./16.*pow(m11,2) );

      M13p6x22RV +=  + hh21V(1,m13,m33,m33,m13,L,mu2) * ( 3./32.*pow(
         m33,2) + 1./24.*pow(m33,2)*pow(RS3461e,2) - 1./24.*pow(m33,2)*
         RC346e*RS3461e + 1./96.*pow(m33,2)*pow(RC346e,2) + 3./16.*m11*
         m33 + 1./12.*m11*m33*pow(RS3461e,2) - 1./12.*m11*m33*RC346e*
         RS3461e + 1./48.*m11*m33*pow(RC346e,2) + 3./32.*pow(m11,2) + 1.
         /24.*pow(m11,2)*pow(RS3461e,2) - 1./24.*pow(m11,2)*RC346e*
         RS3461e + 1./96.*pow(m11,2)*pow(RC346e,2) );

      M13p6x22RV +=  + hh21V(1,m13,m33,me2,m13,L,mu2) * (  - 1./6.*pow(
         m33,2)*RSe6613*RS3461e + 1./12.*pow(m33,2)*RSe6633*RS3461e + 1.
         /12.*pow(m33,2)*RC346e*RSe6613 - 1./24.*pow(m33,2)*RC346e*
         RSe6633 + 1./12.*pow(m33,2)*RSe6611*RS3461e - 1./24.*pow(
         m33,2)*RSe6611*RC346e - 1./3.*m11*m33*RSe6613*RS3461e + 1./6.*
         m11*m33*RSe6633*RS3461e + 1./6.*m11*m33*RC346e*RSe6613 - 1./12.
         *m11*m33*RC346e*RSe6633 + 1./6.*m11*m33*RSe6611*RS3461e - 1./
         12.*m11*m33*RSe6611*RC346e - 1./6.*pow(m11,2)*RSe6613*RS3461e
          + 1./12.*pow(m11,2)*RSe6633*RS3461e + 1./12.*pow(m11,2)*
         RC346e*RSe6613 - 1./24.*pow(m11,2)*RC346e*RSe6633 + 1./12.*
         pow(m11,2)*RSe6611*RS3461e - 1./24.*pow(m11,2)*RSe6611*RC346e
          );

      M13p6x22RV +=  + hh21V(1,m13,me2,me2,m13,L,mu2) * ( 1./6.*pow(
         m33,2)*pow(RSe6613,2) - 1./6.*pow(m33,2)*RSe6633*RSe6613 + 1./
         24.*pow(m33,2)*pow(RSe6633,2) - 1./6.*pow(m33,2)*RSe6611*
         RSe6613 + 1./12.*pow(m33,2)*RSe6611*RSe6633 + 1./24.*pow(
         m33,2)*pow(RSe6611,2) + 1./3.*m11*m33*pow(RSe6613,2) - 1./3.*
         m11*m33*RSe6633*RSe6613 + 1./12.*m11*m33*pow(RSe6633,2) - 1./3.
         *m11*m33*RSe6611*RSe6613 + 1./6.*m11*m33*RSe6611*RSe6633 + 1./
         12.*m11*m33*pow(RSe6611,2) + 1./6.*pow(m11,2)*pow(RSe6613,2)
          - 1./6.*pow(m11,2)*RSe6633*RSe6613 + 1./24.*pow(m11,2)*pow(
         RSe6633,2) - 1./6.*pow(m11,2)*RSe6611*RSe6613 + 1./12.*pow(
         m11,2)*RSe6611*RSe6633 + 1./24.*pow(m11,2)*pow(RSe6611,2) );

      M13p6x22RV +=  + hh21V(1,m14,m11,m34,m13,L,mu2) * ( 1./8.*pow(
         m33,2)*RS1463e + 1./8.*pow(m33,2)*RS16e - 1./8.*pow(m33,2)*
         RC146e + 1./4.*m11*m33*RS1463e + 1./4.*m11*m33*RS16e - 1./4.*
         m11*m33*RC146e + 1./8.*pow(m11,2)*RS1463e + 1./8.*pow(m11,2)*
         RS16e - 1./8.*pow(m11,2)*RC146e );

      M13p6x22RV +=  + hh21V(1,m14,m33,m34,m13,L,mu2) * ( 1./8.*pow(
         m33,2)*RS3461e - 1./8.*pow(m33,2)*RS36e + 1./4.*m11*m33*
         RS3461e - 1./4.*m11*m33*RS36e + 1./8.*pow(m11,2)*RS3461e - 1./
         8.*pow(m11,2)*RS36e );

      M13p6x22RV +=  + hh21V(1,m14,m34,me2,m13,L,mu2) * (  - 1./4.*pow(
         m33,2)*RSe6613 - 1./8.*pow(m33,2)*RSe63 + 1./8.*pow(m33,2)*
         RSe61 + 1./4.*pow(m33,2)*RSe6611 - 1./2.*m11*m33*RSe6613 - 1./
         4.*m11*m33*RSe63 + 1./4.*m11*m33*RSe61 + 1./2.*m11*m33*RSe6611
          - 1./4.*pow(m11,2)*RSe6613 - 1./8.*pow(m11,2)*RSe63 + 1./8.*
         pow(m11,2)*RSe61 + 1./4.*pow(m11,2)*RSe6611 );

      M13p6x22RV +=  + hh21V(1,m16,m11,m36,m13,L,mu2) * ( 1./16.*pow(
         m33,2)*RS1463e + 1./16.*pow(m33,2)*RS14e - 1./16.*pow(m33,2)*
         RC146e + 1./8.*m11*m33*RS1463e + 1./8.*m11*m33*RS14e - 1./8.*
         m11*m33*RC146e + 1./16.*pow(m11,2)*RS1463e + 1./16.*pow(m11,2)
         *RS14e - 1./16.*pow(m11,2)*RC146e );

      M13p6x22RV +=  + hh21V(1,m16,m33,m36,m13,L,mu2) * ( 1./16.*pow(
         m33,2)*RS3461e - 1./16.*pow(m33,2)*RS34e + 1./8.*m11*m33*
         RS3461e - 1./8.*m11*m33*RS34e + 1./16.*pow(m11,2)*RS3461e - 1./
         16.*pow(m11,2)*RS34e );

      M13p6x22RV +=  + hh21V(1,m16,m36,me2,m13,L,mu2) * (  - 1./8.*pow(
         m33,2)*RSe6613 + 1./8.*pow(m33,2)*RSe63 - 1./8.*pow(m33,2)*
         RSe61 + 1./8.*pow(m33,2)*RSe6611 - 1./4.*m11*m33*RSe6613 + 1./
         4.*m11*m33*RSe63 - 1./4.*m11*m33*RSe61 + 1./4.*m11*m33*RSe6611
          - 1./8.*pow(m11,2)*RSe6613 + 1./8.*pow(m11,2)*RSe63 - 1./8.*
         pow(m11,2)*RSe61 + 1./8.*pow(m11,2)*RSe6611 );

      M13p6x22RV +=  + hh21V(1,m33,m13,m33,m13,L,mu2) * (  - 3./16.*
         pow(m33,2) - 3./8.*m11*m33 - 3./16.*pow(m11,2) );

      M13p6x22RV +=  + hh21V(1,m33,m14,m34,m13,L,mu2) * (  - 1./8.*pow(
         m33,2)*RS3461e + 1./8.*pow(m33,2)*RS36e - 1./4.*m11*m33*
         RS3461e + 1./4.*m11*m33*RS36e - 1./8.*pow(m11,2)*RS3461e + 1./
         8.*pow(m11,2)*RS36e );

      M13p6x22RV +=  + hh21V(1,m33,m16,m36,m13,L,mu2) * (  - 1./16.*
         pow(m33,2)*RS3461e + 1./16.*pow(m33,2)*RS34e - 1./8.*m11*m33*
         RS3461e + 1./8.*m11*m33*RS34e - 1./16.*pow(m11,2)*RS3461e + 1./
         16.*pow(m11,2)*RS34e );

      M13p6x22RV +=  + hh21V(1,m34,m11,m14,m13,L,mu2) * ( 1./8.*pow(
         m33,2)*RS1463e - 1./8.*pow(m33,2)*RS16e + 1./4.*m11*m33*
         RS1463e - 1./4.*m11*m33*RS16e + 1./8.*pow(m11,2)*RS1463e - 1./
         8.*pow(m11,2)*RS16e );

      M13p6x22RV +=  + hh21V(1,m34,m14,m33,m13,L,mu2) * ( 1./8.*pow(
         m33,2)*RS3461e + 1./8.*pow(m33,2)*RS36e - 1./8.*pow(m33,2)*
         RC346e + 1./4.*m11*m33*RS3461e + 1./4.*m11*m33*RS36e - 1./4.*
         m11*m33*RC346e + 1./8.*pow(m11,2)*RS3461e + 1./8.*pow(m11,2)*
         RS36e - 1./8.*pow(m11,2)*RC346e );

      M13p6x22RV +=  + hh21V(1,m34,m14,me2,m13,L,mu2) * (  - 1./4.*pow(
         m33,2)*RSe6613 + 1./8.*pow(m33,2)*RSe63 + 1./4.*pow(m33,2)*
         RSe6633 - 1./8.*pow(m33,2)*RSe61 - 1./2.*m11*m33*RSe6613 + 1./
         4.*m11*m33*RSe63 + 1./2.*m11*m33*RSe6633 - 1./4.*m11*m33*RSe61
          - 1./4.*pow(m11,2)*RSe6613 + 1./8.*pow(m11,2)*RSe63 + 1./4.*
         pow(m11,2)*RSe6633 - 1./8.*pow(m11,2)*RSe61 );

      M13p6x22RV +=  + hh21V(1,m36,m11,m16,m13,L,mu2) * ( 1./16.*pow(
         m33,2)*RS1463e - 1./16.*pow(m33,2)*RS14e + 1./8.*m11*m33*
         RS1463e - 1./8.*m11*m33*RS14e + 1./16.*pow(m11,2)*RS1463e - 1./
         16.*pow(m11,2)*RS14e );

      M13p6x22RV +=  + hh21V(1,m36,m16,m33,m13,L,mu2) * ( 1./16.*pow(
         m33,2)*RS3461e + 1./16.*pow(m33,2)*RS34e - 1./16.*pow(m33,2)*
         RC346e + 1./8.*m11*m33*RS3461e + 1./8.*m11*m33*RS34e - 1./8.*
         m11*m33*RC346e + 1./16.*pow(m11,2)*RS3461e + 1./16.*pow(m11,2)
         *RS34e - 1./16.*pow(m11,2)*RC346e );

      M13p6x22RV +=  + hh21V(1,m36,m16,me2,m13,L,mu2) * (  - 1./8.*pow(
         m33,2)*RSe6613 - 1./8.*pow(m33,2)*RSe63 + 1./8.*pow(m33,2)*
         RSe6633 + 1./8.*pow(m33,2)*RSe61 - 1./4.*m11*m33*RSe6613 - 1./
         4.*m11*m33*RSe63 + 1./4.*m11*m33*RSe6633 + 1./4.*m11*m33*RSe61
          - 1./8.*pow(m11,2)*RSe6613 - 1./8.*pow(m11,2)*RSe63 + 1./8.*
         pow(m11,2)*RSe6633 + 1./8.*pow(m11,2)*RSe61 );

      M13p6x22RV +=  + hh21V(1,m44,m14,m34,m13,L,mu2) * ( 9./16.*pow(
         m33,2) + 9./8.*m11*m33 + 9./16.*pow(m11,2) );

      M13p6x22RV +=  + hh21V(1,m46,m14,m36,m13,L,mu2) * ( 3./8.*pow(
         m33,2) + 3./4.*m11*m33 + 3./8.*pow(m11,2) );

      M13p6x22RV +=  + hh21V(1,m46,m16,m34,m13,L,mu2) * ( 3./8.*pow(
         m33,2) + 3./4.*m11*m33 + 3./8.*pow(m11,2) );

      M13p6x22RV +=  + hh21V(1,me2,m14,m34,m13,L,mu2) * ( 1./16.*pow(
         m33,2) + 1./4.*pow(m33,2)*RSe6613 + 1./8.*pow(m33,2)*RSe63 + 1.
         /8.*pow(m33,2)*RSe61 + 1./8.*m11*m33 + 1./2.*m11*m33*RSe6613
          + 1./4.*m11*m33*RSe63 + 1./4.*m11*m33*RSe61 + 1./16.*pow(
         m11,2) + 1./4.*pow(m11,2)*RSe6613 + 1./8.*pow(m11,2)*RSe63 + 1.
         /8.*pow(m11,2)*RSe61 );

      M13p6x22RV +=  + hh21V(1,me2,m16,m36,m13,L,mu2) * ( 1./8.*pow(
         m33,2) + 1./8.*pow(m33,2)*RSe6613 - 1./8.*pow(m33,2)*RSe63 - 1.
         /8.*pow(m33,2)*RSe61 + 1./4.*m11*m33 + 1./4.*m11*m33*RSe6613
          - 1./4.*m11*m33*RSe63 - 1./4.*m11*m33*RSe61 + 1./8.*pow(
         m11,2) + 1./8.*pow(m11,2)*RSe6613 - 1./8.*pow(m11,2)*RSe63 - 1.
         /8.*pow(m11,2)*RSe61 );

      M13p6x22RV +=  + hh21V(3,m13,m11,m11,m13,L,mu2) * (  - 1./24.*
         pow(m33,2)*RD146e*RS1463e + 1./48.*pow(m33,2)*RD146e*RC146e - 
         1./12.*m11*m33*RD146e*RS1463e + 1./24.*m11*m33*RD146e*RC146e
          - 1./24.*pow(m11,2)*RD146e*RS1463e + 1./48.*pow(m11,2)*RD146e
         *RC146e );

      M13p6x22RV +=  + hh21V(3,m13,m11,m33,m13,L,mu2) * (  - 1./24.*
         pow(m33,2)*RD146e*RS3461e + 1./48.*pow(m33,2)*RD146e*RC346e - 
         1./12.*m11*m33*RD146e*RS3461e + 1./24.*m11*m33*RD146e*RC346e
          - 1./24.*pow(m11,2)*RD146e*RS3461e + 1./48.*pow(m11,2)*RD146e
         *RC346e );

      M13p6x22RV +=  + hh21V(3,m13,m11,me2,m13,L,mu2) * ( 1./12.*pow(
         m33,2)*RD146e*RSe6613 - 1./24.*pow(m33,2)*RD146e*RSe6633 - 1./
         24.*pow(m33,2)*RD146e*RSe6611 + 1./6.*m11*m33*RD146e*RSe6613
          - 1./12.*m11*m33*RD146e*RSe6633 - 1./12.*m11*m33*RD146e*
         RSe6611 + 1./12.*pow(m11,2)*RD146e*RSe6613 - 1./24.*pow(m11,2)
         *RD146e*RSe6633 - 1./24.*pow(m11,2)*RD146e*RSe6611 );

      M13p6x22RV +=  + hh21V(3,m13,m33,m33,m13,L,mu2) * (  - 1./24.*
         pow(m33,2)*RD346e*RS3461e + 1./48.*pow(m33,2)*RD346e*RC346e - 
         1./12.*m11*m33*RD346e*RS3461e + 1./24.*m11*m33*RD346e*RC346e
          - 1./24.*pow(m11,2)*RD346e*RS3461e + 1./48.*pow(m11,2)*RD346e
         *RC346e );

      M13p6x22RV +=  + hh21V(3,m13,m33,me2,m13,L,mu2) * ( 1./12.*pow(
         m33,2)*RD346e*RSe6613 - 1./24.*pow(m33,2)*RD346e*RSe6633 - 1./
         24.*pow(m33,2)*RSe6611*RD346e + 1./6.*m11*m33*RD346e*RSe6613
          - 1./12.*m11*m33*RD346e*RSe6633 - 1./12.*m11*m33*RSe6611*
         RD346e + 1./12.*pow(m11,2)*RD346e*RSe6613 - 1./24.*pow(m11,2)*
         RD346e*RSe6633 - 1./24.*pow(m11,2)*RSe6611*RD346e );

      M13p6x22RV +=  + hh21V(3,m14,m11,m34,m13,L,mu2) * (  - 1./8.*pow(
         m33,2)*RD146e - 1./4.*m11*m33*RD146e - 1./8.*pow(m11,2)*RD146e
          );

      M13p6x22RV +=  + hh21V(3,m16,m11,m36,m13,L,mu2) * (  - 1./16.*
         pow(m33,2)*RD146e - 1./8.*m11*m33*RD146e - 1./16.*pow(m11,2)*
         RD146e );

      M13p6x22RV +=  + hh21V(4,m13,m11,m33,m13,L,mu2) * (  - 1./24.*
         pow(m33,2)*RD346e*RS1463e + 1./48.*pow(m33,2)*RC146e*RD346e - 
         1./12.*m11*m33*RD346e*RS1463e + 1./24.*m11*m33*RC146e*RD346e
          - 1./24.*pow(m11,2)*RD346e*RS1463e + 1./48.*pow(m11,2)*RC146e
         *RD346e );

      M13p6x22RV +=  + hh21V(4,m34,m14,m33,m13,L,mu2) * (  - 1./8.*pow(
         m33,2)*RD346e - 1./4.*m11*m33*RD346e - 1./8.*pow(m11,2)*RD346e
          );

      M13p6x22RV +=  + hh21V(4,m36,m16,m33,m13,L,mu2) * (  - 1./16.*
         pow(m33,2)*RD346e - 1./8.*m11*m33*RD346e - 1./16.*pow(m11,2)*
         RD346e );

      M13p6x22RV +=  + hh21V(7,m13,m11,m11,m13,L,mu2) * ( 1./96.*pow(
         m33,2)*pow(RD146e,2) + 1./48.*m11*m33*pow(RD146e,2) + 1./96.*
         pow(m11,2)*pow(RD146e,2) );

      M13p6x22RV +=  + hh21V(7,m13,m11,m33,m13,L,mu2) * ( 1./48.*pow(
         m33,2)*RD146e*RD346e + 1./24.*m11*m33*RD146e*RD346e + 1./48.*
         pow(m11,2)*RD146e*RD346e );

      M13p6x22RV +=  + hh21V(7,m13,m33,m33,m13,L,mu2) * ( 1./96.*pow(
         m33,2)*pow(RD346e,2) + 1./48.*m11*m33*pow(RD346e,2) + 1./96.*
         pow(m11,2)*pow(RD346e,2) );

      M13p6x22RV +=  + hh27V(1,m11,m11,m13,m13,L,mu2) * ( 3./8.*m33 + 3.
         /8.*m11 );

      M13p6x22RV +=  + hh27V(1,m11,m14,m34,m13,L,mu2) * ( 1./4.*m33*
         RS1463e - 1./4.*m33*RS16e + 1./4.*m11*RS1463e - 1./4.*m11*
         RS16e );

      M13p6x22RV +=  + hh27V(1,m11,m16,m36,m13,L,mu2) * ( 1./8.*m33*
         RS1463e - 1./8.*m33*RS14e + 1./8.*m11*RS1463e - 1./8.*m11*
         RS14e );

      M13p6x22RV +=  + hh27V(1,m13,m11,m11,m13,L,mu2) * (  - 3./16.*m33
          - 1./12.*m33*pow(RS1463e,2) + 1./12.*m33*RC146e*RS1463e - 1./
         48.*m33*pow(RC146e,2) - 3./16.*m11 - 1./12.*m11*pow(RS1463e,2)
          + 1./12.*m11*RC146e*RS1463e - 1./48.*m11*pow(RC146e,2) );

      M13p6x22RV +=  + hh27V(1,m13,m11,m33,m13,L,mu2) * (  - 1./6.*m33*
         RS1463e*RS3461e + 1./12.*m33*RC346e*RS1463e + 1./12.*m33*
         RC146e*RS3461e - 1./24.*m33*RC146e*RC346e - 1./6.*m11*RS1463e*
         RS3461e + 1./12.*m11*RC346e*RS1463e + 1./12.*m11*RC146e*
         RS3461e - 1./24.*m11*RC146e*RC346e );

      M13p6x22RV +=  + hh27V(1,m13,m11,me2,m13,L,mu2) * ( 1./3.*m33*
         RSe6613*RS1463e - 1./6.*m33*RSe6633*RS1463e - 1./6.*m33*
         RSe6611*RS1463e - 1./6.*m33*RC146e*RSe6613 + 1./12.*m33*RC146e
         *RSe6633 + 1./12.*m33*RC146e*RSe6611 + 1./3.*m11*RSe6613*
         RS1463e - 1./6.*m11*RSe6633*RS1463e - 1./6.*m11*RSe6611*
         RS1463e - 1./6.*m11*RC146e*RSe6613 + 1./12.*m11*RC146e*RSe6633
          + 1./12.*m11*RC146e*RSe6611 );

      M13p6x22RV +=  + hh27V(1,m13,m13,m13,m13,L,mu2) * (  - 3./8.*m33
          - 3./8.*m11 );

      M13p6x22RV +=  + hh27V(1,m13,m33,m33,m13,L,mu2) * (  - 3./16.*m33
          - 1./12.*m33*pow(RS3461e,2) + 1./12.*m33*RC346e*RS3461e - 1./
         48.*m33*pow(RC346e,2) - 3./16.*m11 - 1./12.*m11*pow(RS3461e,2)
          + 1./12.*m11*RC346e*RS3461e - 1./48.*m11*pow(RC346e,2) );

      M13p6x22RV +=  + hh27V(1,m13,m33,me2,m13,L,mu2) * ( 1./3.*m33*
         RSe6613*RS3461e - 1./6.*m33*RSe6633*RS3461e - 1./6.*m33*RC346e
         *RSe6613 + 1./12.*m33*RC346e*RSe6633 - 1./6.*m33*RSe6611*
         RS3461e + 1./12.*m33*RSe6611*RC346e + 1./3.*m11*RSe6613*
         RS3461e - 1./6.*m11*RSe6633*RS3461e - 1./6.*m11*RC346e*RSe6613
          + 1./12.*m11*RC346e*RSe6633 - 1./6.*m11*RSe6611*RS3461e + 1./
         12.*m11*RSe6611*RC346e );

      M13p6x22RV +=  + hh27V(1,m13,me2,me2,m13,L,mu2) * (  - 1./3.*m33*
         pow(RSe6613,2) + 1./3.*m33*RSe6633*RSe6613 - 1./12.*m33*pow(
         RSe6633,2) + 1./3.*m33*RSe6611*RSe6613 - 1./6.*m33*RSe6611*
         RSe6633 - 1./12.*m33*pow(RSe6611,2) - 1./3.*m11*pow(RSe6613,2)
          + 1./3.*m11*RSe6633*RSe6613 - 1./12.*m11*pow(RSe6633,2) + 1./
         3.*m11*RSe6611*RSe6613 - 1./6.*m11*RSe6611*RSe6633 - 1./12.*
         m11*pow(RSe6611,2) );

      M13p6x22RV +=  + hh27V(1,m14,m11,m34,m13,L,mu2) * (  - 1./4.*m33*
         RS1463e - 1./4.*m33*RS16e + 1./4.*m33*RC146e - 1./4.*m11*
         RS1463e - 1./4.*m11*RS16e + 1./4.*m11*RC146e );

      M13p6x22RV +=  + hh27V(1,m14,m33,m34,m13,L,mu2) * (  - 1./4.*m33*
         RS3461e + 1./4.*m33*RS36e - 1./4.*m11*RS3461e + 1./4.*m11*
         RS36e );

      M13p6x22RV +=  + hh27V(1,m14,m34,me2,m13,L,mu2) * ( 1./2.*m33*
         RSe6613 + 1./4.*m33*RSe63 - 1./4.*m33*RSe61 - 1./2.*m33*
         RSe6611 + 1./2.*m11*RSe6613 + 1./4.*m11*RSe63 - 1./4.*m11*
         RSe61 - 1./2.*m11*RSe6611 );

      M13p6x22RV +=  + hh27V(1,m16,m11,m36,m13,L,mu2) * (  - 1./8.*m33*
         RS1463e - 1./8.*m33*RS14e + 1./8.*m33*RC146e - 1./8.*m11*
         RS1463e - 1./8.*m11*RS14e + 1./8.*m11*RC146e );

      M13p6x22RV +=  + hh27V(1,m16,m33,m36,m13,L,mu2) * (  - 1./8.*m33*
         RS3461e + 1./8.*m33*RS34e - 1./8.*m11*RS3461e + 1./8.*m11*
         RS34e );

      M13p6x22RV +=  + hh27V(1,m16,m36,me2,m13,L,mu2) * ( 1./4.*m33*
         RSe6613 - 1./4.*m33*RSe63 + 1./4.*m33*RSe61 - 1./4.*m33*
         RSe6611 + 1./4.*m11*RSe6613 - 1./4.*m11*RSe63 + 1./4.*m11*
         RSe61 - 1./4.*m11*RSe6611 );

      M13p6x22RV +=  + hh27V(1,m33,m13,m33,m13,L,mu2) * ( 3./8.*m33 + 3.
         /8.*m11 );

      M13p6x22RV +=  + hh27V(1,m33,m14,m34,m13,L,mu2) * ( 1./4.*m33*
         RS3461e - 1./4.*m33*RS36e + 1./4.*m11*RS3461e - 1./4.*m11*
         RS36e );

      M13p6x22RV +=  + hh27V(1,m33,m16,m36,m13,L,mu2) * ( 1./8.*m33*
         RS3461e - 1./8.*m33*RS34e + 1./8.*m11*RS3461e - 1./8.*m11*
         RS34e );

      M13p6x22RV +=  + hh27V(1,m34,m11,m14,m13,L,mu2) * (  - 1./4.*m33*
         RS1463e + 1./4.*m33*RS16e - 1./4.*m11*RS1463e + 1./4.*m11*
         RS16e );

      M13p6x22RV +=  + hh27V(1,m34,m14,m33,m13,L,mu2) * (  - 1./4.*m33*
         RS3461e - 1./4.*m33*RS36e + 1./4.*m33*RC346e - 1./4.*m11*
         RS3461e - 1./4.*m11*RS36e + 1./4.*m11*RC346e );

      M13p6x22RV +=  + hh27V(1,m34,m14,me2,m13,L,mu2) * ( 1./2.*m33*
         RSe6613 - 1./4.*m33*RSe63 - 1./2.*m33*RSe6633 + 1./4.*m33*
         RSe61 + 1./2.*m11*RSe6613 - 1./4.*m11*RSe63 - 1./2.*m11*
         RSe6633 + 1./4.*m11*RSe61 );

      M13p6x22RV +=  + hh27V(1,m36,m11,m16,m13,L,mu2) * (  - 1./8.*m33*
         RS1463e + 1./8.*m33*RS14e - 1./8.*m11*RS1463e + 1./8.*m11*
         RS14e );

      M13p6x22RV +=  + hh27V(1,m36,m16,m33,m13,L,mu2) * (  - 1./8.*m33*
         RS3461e - 1./8.*m33*RS34e + 1./8.*m33*RC346e - 1./8.*m11*
         RS3461e - 1./8.*m11*RS34e + 1./8.*m11*RC346e );

      M13p6x22RV +=  + hh27V(1,m36,m16,me2,m13,L,mu2) * ( 1./4.*m33*
         RSe6613 + 1./4.*m33*RSe63 - 1./4.*m33*RSe6633 - 1./4.*m33*
         RSe61 + 1./4.*m11*RSe6613 + 1./4.*m11*RSe63 - 1./4.*m11*
         RSe6633 - 1./4.*m11*RSe61 );

      M13p6x22RV +=  + hh27V(1,m44,m14,m34,m13,L,mu2) * (  - 9./8.*m33
          - 9./8.*m11 );

      M13p6x22RV +=  + hh27V(1,m46,m14,m36,m13,L,mu2) * (  - 3./4.*m33
          - 3./4.*m11 );

      M13p6x22RV +=  + hh27V(1,m46,m16,m34,m13,L,mu2) * (  - 3./4.*m33
          - 3./4.*m11 );

      M13p6x22RV +=  + hh27V(1,me2,m14,m34,m13,L,mu2) * (  - 1./8.*m33
          - 1./2.*m33*RSe6613 - 1./4.*m33*RSe63 - 1./4.*m33*RSe61 - 1./
         8.*m11 - 1./2.*m11*RSe6613 - 1./4.*m11*RSe63 - 1./4.*m11*RSe61
          );

      M13p6x22RV +=  + hh27V(1,me2,m16,m36,m13,L,mu2) * (  - 1./4.*m33
          - 1./4.*m33*RSe6613 + 1./4.*m33*RSe63 + 1./4.*m33*RSe61 - 1./
         4.*m11 - 1./4.*m11*RSe6613 + 1./4.*m11*RSe63 + 1./4.*m11*RSe61
          );

      M13p6x22RV +=  + hh27V(3,m13,m11,m11,m13,L,mu2) * ( 1./12.*m33*
         RD146e*RS1463e - 1./24.*m33*RD146e*RC146e + 1./12.*m11*RD146e*
         RS1463e - 1./24.*m11*RD146e*RC146e );

      M13p6x22RV +=  + hh27V(3,m13,m11,m33,m13,L,mu2) * ( 1./12.*m33*
         RD146e*RS3461e - 1./24.*m33*RD146e*RC346e + 1./12.*m11*RD146e*
         RS3461e - 1./24.*m11*RD146e*RC346e );

      M13p6x22RV +=  + hh27V(3,m13,m11,me2,m13,L,mu2) * (  - 1./6.*m33*
         RD146e*RSe6613 + 1./12.*m33*RD146e*RSe6633 + 1./12.*m33*RD146e
         *RSe6611 - 1./6.*m11*RD146e*RSe6613 + 1./12.*m11*RD146e*
         RSe6633 + 1./12.*m11*RD146e*RSe6611 );

      M13p6x22RV +=  + hh27V(3,m13,m33,m33,m13,L,mu2) * ( 1./12.*m33*
         RD346e*RS3461e - 1./24.*m33*RD346e*RC346e + 1./12.*m11*RD346e*
         RS3461e - 1./24.*m11*RD346e*RC346e );

      M13p6x22RV +=  + hh27V(3,m13,m33,me2,m13,L,mu2) * (  - 1./6.*m33*
         RD346e*RSe6613 + 1./12.*m33*RD346e*RSe6633 + 1./12.*m33*
         RSe6611*RD346e - 1./6.*m11*RD346e*RSe6613 + 1./12.*m11*RD346e*
         RSe6633 + 1./12.*m11*RSe6611*RD346e );

      M13p6x22RV +=  + hh27V(3,m14,m11,m34,m13,L,mu2) * ( 1./4.*m33*
         RD146e + 1./4.*m11*RD146e );

      M13p6x22RV +=  + hh27V(3,m16,m11,m36,m13,L,mu2) * ( 1./8.*m33*
         RD146e + 1./8.*m11*RD146e );

      M13p6x22RV +=  + hh27V(4,m13,m11,m33,m13,L,mu2) * ( 1./12.*m33*
         RD346e*RS1463e - 1./24.*m33*RC146e*RD346e + 1./12.*m11*RD346e*
         RS1463e - 1./24.*m11*RC146e*RD346e );

      M13p6x22RV +=  + hh27V(4,m34,m14,m33,m13,L,mu2) * ( 1./4.*m33*
         RD346e + 1./4.*m11*RD346e );

      M13p6x22RV +=  + hh27V(4,m36,m16,m33,m13,L,mu2) * ( 1./8.*m33*
         RD346e + 1./8.*m11*RD346e );

      M13p6x22RV +=  + hh27V(7,m13,m11,m11,m13,L,mu2) * (  - 1./48.*m33
         *pow(RD146e,2) - 1./48.*m11*pow(RD146e,2) );

      M13p6x22RV +=  + hh27V(7,m13,m11,m33,m13,L,mu2) * (  - 1./24.*m33
         *RD146e*RD346e - 1./24.*m11*RD146e*RD346e );

      M13p6x22RV +=  + hh27V(7,m13,m33,m33,m13,L,mu2) * (  - 1./48.*m33
         *pow(RD346e,2) - 1./48.*m11*pow(RD346e,2) );

  return M13p6x22RV/pow(f0,4);
}

// 1 valence mass 3 sea mass *****************************************

double mv1s3nf3p4V(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 4) || (B0mq.size() != 4)){
    cout << "ERROR: mv2s3nf3p4V probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m55 = 2.*B0mq[2];
  double m66 = 2.*B0mq[3];
  double xb1 = (m44+m55+m66)/3.;
  double msqr = sqrt(xb1*xb1-(m44*m55+m44*m66+m55*m66)/3.);
  double mp2 = xb1-msqr; 
  double me2 = xb1+msqr;

  //double mu2 = mu0*mu0;
  double RD1456ep = RD(m11,m44,m55,m66,me2,mp2);
  double RC1456pe = RC(m11,m44,m55,m66,mp2,me2);  
  double RSe45611p = RS(me2,m44,m55,m66,m11,m11,mp2);
  double RSp45611e = RS(mp2,m44,m55,m66,m11,m11,me2);

  double M12p4x13V =
       + AbV(m11,L) * (  - 1./3.*m11*RC1456pe );

      M12p4x13V +=  + AbV(mp2,L) * (  - 1./3.*m11*RSp45611e );

      M12p4x13V +=  + AbV(me2,L) * (  - 1./3.*m11*RSe45611p );

      M12p4x13V +=  + BbV(m11,L) * (  - 1./3.*m11*RD1456ep );

  return M12p4x13V/pow(f0,2);
}

double mv1s3nf3p6V(const quarkmassnf mass, const Linf Liin, const double L){
  return mv1s3nf3p6LV(mass,Liin,L)+mv1s3nf3p6RV(mass,L);
}

double mv1s3nf3p6LV(const quarkmassnf mass, const Linf Liin, const double L){
  double L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu;
  int nfLi;
  Liin.out(L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu,nfLi);
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if (mu0 != mu){
    cout << "ERROR: differing mu's in mv1s3nf3p6LV\n";}
  if ( (nq != 4) || (B0mq.size() != 4)){
    cout << "ERROR: mv1s3nf3p6LV probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m55 = 2.*B0mq[2];
  double m66 = 2.*B0mq[3];
  double m14 = (m11+m44)/2;
  double m15 = (m11+m55)/2;
  double m16 = (m11+m66)/2;
  double m45 = (m44+m55)/2;
  double m46 = (m44+m66)/2;
  double m56 = (m55+m66)/2;

  double xb1 = (m44+m55+m66)/3.;
  double msqr = sqrt(xb1*xb1-(m44*m55+m44*m66+m55*m66)/3.);
  double mp2 = xb1-msqr; 
  double me2 = xb1+msqr;
  //double mu2 = mu0*mu0;

  //double RN1p = RN(m11,mp2);
  //double RN1e = RN(m11,me2);
  //double RNep = RN(me2,mp2);

  double RSp456e = RS(mp2,m44,m55,m66,me2);
  double RSp465e = RS(mp2,m44,m66,m55,me2);
  double RSp564e = RS(mp2,m55,m66,m44,me2);
  double RSe456p = RS(me2,m44,m55,m66,mp2);
  double RSe465p = RS(me2,m44,m66,m55,mp2);
  double RSe564p = RS(me2,m55,m66,m44,mp2);
  double RSp451e = RS(mp2,m44,m55,m11,me2);
  double RSp461e = RS(mp2,m44,m66,m11,me2);
  double RSp561e = RS(mp2,m55,m66,m11,me2);
  double RSe451p = RS(me2,m44,m55,m11,mp2);
  double RSe461p = RS(me2,m44,m66,m11,mp2);
  double RSe561p = RS(me2,m55,m66,m11,mp2);
  double RS456pe = RS(m44,m55,m66,mp2,me2);
  double RS546pe = RS(m55,m44,m66,mp2,me2);
  double RS645pe = RS(m66,m44,m55,mp2,me2);
  double RS156pe = RS(m11,m55,m66,mp2,me2);
  double RS146pe = RS(m11,m44,m66,mp2,me2);
  double RS145pe = RS(m11,m44,m55,mp2,me2);

  double RD1456ep = RD(m11,m44,m55,m66,me2,mp2);
  double RC1456pe = RC(m11,m44,m55,m66,mp2,me2);  
  double RSe45611p = RS(me2,m44,m55,m66,m11,m11,mp2);
  double RSp45611e = RS(mp2,m44,m55,m66,m11,m11,me2);

  double M12p6x13LV =
       + AbV(m11,L) * ( 8./3.*m11*L5r*RD1456ep - 8*m11*L3r*RD1456ep - 8
         *m11*L0r*RD1456ep - 32./3.*m11*m66*L7r*RS145pe - 16./3.*m11*
         m66*L6r*RC1456pe - 8./9.*m11*m66*L5r*pow(RS145pe,2) + 8*m11*
         m66*L4r*RC1456pe - 8./9.*m11*m66*L4r*pow(RS145pe,2) - 8./9.*
         m11*m66*L4r*pow(RS146pe,2) - 8./9.*m11*m66*L4r*pow(RS156pe,2)
          - 32./3.*m11*m55*L7r*RS146pe - 16./3.*m11*m55*L6r*RC1456pe - 
         8./9.*m11*m55*L5r*pow(RS146pe,2) + 8*m11*m55*L4r*RC1456pe - 8./
         9.*m11*m55*L4r*pow(RS145pe,2) - 8./9.*m11*m55*L4r*pow(
         RS146pe,2) - 8./9.*m11*m55*L4r*pow(RS156pe,2) - 32./3.*m11*m44
         *L7r*RS156pe - 16./3.*m11*m44*L6r*RC1456pe - 8./9.*m11*m44*L5r
         *pow(RS156pe,2) + 8*m11*m44*L4r*RC1456pe - 8./9.*m11*m44*L4r*
         pow(RS145pe,2) - 8./9.*m11*m44*L4r*pow(RS146pe,2) - 8./9.*m11*
         m44*L4r*pow(RS156pe,2) - 64./3.*pow(m11,2)*L8r*RC1456pe + 32*
         pow(m11,2)*L7r + 32*pow(m11,2)*L6r + 40./3.*pow(m11,2)*L5r*
         RC1456pe - 8*pow(m11,2)*L3r*RC1456pe + 20*pow(m11,2)*L2r + 8*
         pow(m11,2)*L1r );

      M12p6x13LV +=  + AbV(m11,L) * (  - 8*pow(m11,2)*L0r*RC1456pe );

      M12p6x13LV +=  + AbV(m14,L) * (  - 4./3.*m44*m66*L4r - 4./3.*m44*
         m55*L4r - 2./3.*pow(m44,2)*L5r - 4./3.*pow(m44,2)*L4r + 8./3.*
         m14*m66*L4r + 8./3.*m14*m55*L4r + 4./3.*m14*m44*L5r + 8./3.*
         m14*m44*L4r - 4./3.*m11*m66*L4r - 4./3.*m11*m55*L4r + 16*m11*
         m44*L8r - 16./3.*m11*m44*L5r - 4./3.*m11*m44*L4r - 20./3.*m11*
         m14*L5r + 20*m11*m14*L3r + 8*m11*m14*L0r + 16*pow(m11,2)*L8r
          - 14./3.*pow(m11,2)*L5r );

      M12p6x13LV +=  + AbV(m15,L) * (  - 4./3.*m55*m66*L4r - 2./3.*pow(
         m55,2)*L5r - 4./3.*pow(m55,2)*L4r - 4./3.*m44*m55*L4r + 8./3.*
         m15*m66*L4r + 4./3.*m15*m55*L5r + 8./3.*m15*m55*L4r + 8./3.*
         m15*m44*L4r - 4./3.*m11*m66*L4r + 16*m11*m55*L8r - 16./3.*m11*
         m55*L5r - 4./3.*m11*m55*L4r - 4./3.*m11*m44*L4r - 20./3.*m11*
         m15*L5r + 20*m11*m15*L3r + 8*m11*m15*L0r + 16*pow(m11,2)*L8r
          - 14./3.*pow(m11,2)*L5r );

      M12p6x13LV +=  + AbV(m16,L) * (  - 2./3.*pow(m66,2)*L5r - 4./3.*
         pow(m66,2)*L4r - 4./3.*m55*m66*L4r - 4./3.*m44*m66*L4r + 4./3.
         *m16*m66*L5r + 8./3.*m16*m66*L4r + 8./3.*m16*m55*L4r + 8./3.*
         m16*m44*L4r + 16*m11*m66*L8r - 16./3.*m11*m66*L5r - 4./3.*m11*
         m66*L4r - 4./3.*m11*m55*L4r - 4./3.*m11*m44*L4r - 20./3.*m11*
         m16*L5r + 20*m11*m16*L3r + 8*m11*m16*L0r + 16*pow(m11,2)*L8r
          - 14./3.*pow(m11,2)*L5r );

      M12p6x13LV +=  + AbV(m44,L) * ( 16*m11*m44*L6r - 16./3.*m11*m44*
         L6r*RS456pe - 16*m11*m44*L4r + 16./3.*m11*m44*L4r*RS456pe + 4*
         m11*m44*L2r - 4./3.*m11*m44*L2r*RS456pe + 16*m11*m44*L1r - 16./
         3.*m11*m44*L1r*RS456pe );

      M12p6x13LV +=  + AbV(m45,L) * ( 16*m11*m55*L6r - 8*m11*m55*L4r - 
         16*m11*m45*L4r + 8*m11*m45*L2r + 32*m11*m45*L1r + 16*m11*m44*
         L6r - 8*m11*m44*L4r );

      M12p6x13LV +=  + AbV(m46,L) * ( 16*m11*m66*L6r - 8*m11*m66*L4r - 
         16*m11*m46*L4r + 8*m11*m46*L2r + 32*m11*m46*L1r + 16*m11*m44*
         L6r - 8*m11*m44*L4r );

      M12p6x13LV +=  + AbV(m55,L) * ( 16*m11*m55*L6r - 16./3.*m11*m55*
         L6r*RS546pe - 16*m11*m55*L4r + 16./3.*m11*m55*L4r*RS546pe + 4*
         m11*m55*L2r - 4./3.*m11*m55*L2r*RS546pe + 16*m11*m55*L1r - 16./
         3.*m11*m55*L1r*RS546pe );

      M12p6x13LV +=  + AbV(m56,L) * ( 16*m11*m66*L6r - 8*m11*m66*L4r - 
         16*m11*m56*L4r + 8*m11*m56*L2r + 32*m11*m56*L1r + 16*m11*m55*
         L6r - 8*m11*m55*L4r );

      M12p6x13LV +=  + AbV(m66,L) * ( 16*m11*m66*L6r - 16./3.*m11*m66*
         L6r*RS645pe - 16*m11*m66*L4r + 16./3.*m11*m66*L4r*RS645pe + 4*
         m11*m66*L2r - 4./3.*m11*m66*L2r*RS645pe + 16*m11*m66*L1r - 16./
         3.*m11*m66*L1r*RS645pe );

      M12p6x13LV +=  + AbV(mp2,L) * ( 8./3.*m11*mp2*L5r*RSp45611e + 8./
         3.*m11*mp2*L4r*RSp564e + 8./3.*m11*mp2*L4r*RSp465e + 8./3.*m11
         *mp2*L4r*RSp456e - 8*m11*mp2*L3r*RSp45611e - 4./3.*m11*mp2*L2r
         *RSp564e - 4./3.*m11*mp2*L2r*RSp465e - 4./3.*m11*mp2*L2r*
         RSp456e - 16./3.*m11*mp2*L1r*RSp564e - 16./3.*m11*mp2*L1r*
         RSp465e - 16./3.*m11*mp2*L1r*RSp456e - 8*m11*mp2*L0r*RSp45611e
          - 32./3.*m11*m66*L7r*RSp451e - 16./3.*m11*m66*L6r*RSp45611e
          - 16./3.*m11*m66*L6r*RSp456e - 16./9.*m11*m66*L5r*RSp451e*
         RS145pe - 16./9.*m11*m66*L5r*RSp451e*RSe451p - 8./9.*m11*m66*
         L5r*pow(RSp451e,2) + 8*m11*m66*L4r*RSp45611e - 16./9.*m11*m66*
         L4r*RSp561e*RS156pe - 16./9.*m11*m66*L4r*RSp561e*RSe561p - 8./
         9.*m11*m66*L4r*pow(RSp561e,2) - 16./9.*m11*m66*L4r*RSp461e*
         RS146pe - 16./9.*m11*m66*L4r*RSp461e*RSe461p - 8./9.*m11*m66*
         L4r*pow(RSp461e,2) - 16./9.*m11*m66*L4r*RSp451e*RS145pe - 16./
         9.*m11*m66*L4r*RSp451e*RSe451p - 8./9.*m11*m66*L4r*pow(
         RSp451e,2) );

      M12p6x13LV +=  + AbV(mp2,L) * ( 8./3.*m11*m66*L4r*RSp456e - 32./3.
         *m11*m55*L7r*RSp461e - 16./3.*m11*m55*L6r*RSp45611e - 16./3.*
         m11*m55*L6r*RSp465e - 16./9.*m11*m55*L5r*RSp461e*RS146pe - 16./
         9.*m11*m55*L5r*RSp461e*RSe461p - 8./9.*m11*m55*L5r*pow(
         RSp461e,2) + 8*m11*m55*L4r*RSp45611e - 16./9.*m11*m55*L4r*
         RSp561e*RS156pe - 16./9.*m11*m55*L4r*RSp561e*RSe561p - 8./9.*
         m11*m55*L4r*pow(RSp561e,2) - 16./9.*m11*m55*L4r*RSp461e*
         RS146pe - 16./9.*m11*m55*L4r*RSp461e*RSe461p - 8./9.*m11*m55*
         L4r*pow(RSp461e,2) - 16./9.*m11*m55*L4r*RSp451e*RS145pe - 16./
         9.*m11*m55*L4r*RSp451e*RSe451p - 8./9.*m11*m55*L4r*pow(
         RSp451e,2) + 8./3.*m11*m55*L4r*RSp465e - 32./3.*m11*m44*L7r*
         RSp561e - 16./3.*m11*m44*L6r*RSp45611e - 16./3.*m11*m44*L6r*
         RSp564e - 16./9.*m11*m44*L5r*RSp561e*RS156pe - 16./9.*m11*m44*
         L5r*RSp561e*RSe561p - 8./9.*m11*m44*L5r*pow(RSp561e,2) + 8*m11
         *m44*L4r*RSp45611e - 16./9.*m11*m44*L4r*RSp561e*RS156pe - 16./
         9.*m11*m44*L4r*RSp561e*RSe561p );

      M12p6x13LV +=  + AbV(mp2,L) * (  - 8./9.*m11*m44*L4r*pow(
         RSp561e,2) - 16./9.*m11*m44*L4r*RSp461e*RS146pe - 16./9.*m11*
         m44*L4r*RSp461e*RSe461p - 8./9.*m11*m44*L4r*pow(RSp461e,2) - 
         16./9.*m11*m44*L4r*RSp451e*RS145pe - 16./9.*m11*m44*L4r*
         RSp451e*RSe451p - 8./9.*m11*m44*L4r*pow(RSp451e,2) + 8./3.*m11
         *m44*L4r*RSp564e - 64./3.*pow(m11,2)*L8r*RSp45611e + 32./3.*
         pow(m11,2)*L5r*RSp45611e );

      M12p6x13LV +=  + AbV(me2,L) * ( 8./3.*m11*me2*L5r*RSe45611p + 8./
         3.*m11*me2*L4r*RSe564p + 8./3.*m11*me2*L4r*RSe465p + 8./3.*m11
         *me2*L4r*RSe456p - 8*m11*me2*L3r*RSe45611p - 4./3.*m11*me2*L2r
         *RSe564p - 4./3.*m11*me2*L2r*RSe465p - 4./3.*m11*me2*L2r*
         RSe456p - 16./3.*m11*me2*L1r*RSe564p - 16./3.*m11*me2*L1r*
         RSe465p - 16./3.*m11*me2*L1r*RSe456p - 8*m11*me2*L0r*RSe45611p
          - 32./3.*m11*m66*L7r*RSe451p - 16./3.*m11*m66*L6r*RSe45611p
          - 16./3.*m11*m66*L6r*RSe456p - 16./9.*m11*m66*L5r*RSe451p*
         RS145pe - 8./9.*m11*m66*L5r*pow(RSe451p,2) + 8*m11*m66*L4r*
         RSe45611p - 16./9.*m11*m66*L4r*RSe561p*RS156pe - 8./9.*m11*m66
         *L4r*pow(RSe561p,2) - 16./9.*m11*m66*L4r*RSe461p*RS146pe - 8./
         9.*m11*m66*L4r*pow(RSe461p,2) - 16./9.*m11*m66*L4r*RSe451p*
         RS145pe - 8./9.*m11*m66*L4r*pow(RSe451p,2) + 8./3.*m11*m66*L4r
         *RSe456p - 32./3.*m11*m55*L7r*RSe461p - 16./3.*m11*m55*L6r*
         RSe45611p - 16./3.*m11*m55*L6r*RSe465p - 16./9.*m11*m55*L5r*
         RSe461p*RS146pe );

      M12p6x13LV +=  + AbV(me2,L) * (  - 8./9.*m11*m55*L5r*pow(
         RSe461p,2) + 8*m11*m55*L4r*RSe45611p - 16./9.*m11*m55*L4r*
         RSe561p*RS156pe - 8./9.*m11*m55*L4r*pow(RSe561p,2) - 16./9.*
         m11*m55*L4r*RSe461p*RS146pe - 8./9.*m11*m55*L4r*pow(RSe461p,2)
          - 16./9.*m11*m55*L4r*RSe451p*RS145pe - 8./9.*m11*m55*L4r*pow(
         RSe451p,2) + 8./3.*m11*m55*L4r*RSe465p - 32./3.*m11*m44*L7r*
         RSe561p - 16./3.*m11*m44*L6r*RSe45611p - 16./3.*m11*m44*L6r*
         RSe564p - 16./9.*m11*m44*L5r*RSe561p*RS156pe - 8./9.*m11*m44*
         L5r*pow(RSe561p,2) + 8*m11*m44*L4r*RSe45611p - 16./9.*m11*m44*
         L4r*RSe561p*RS156pe - 8./9.*m11*m44*L4r*pow(RSe561p,2) - 16./9.
         *m11*m44*L4r*RSe461p*RS146pe - 8./9.*m11*m44*L4r*pow(
         RSe461p,2) - 16./9.*m11*m44*L4r*RSe451p*RS145pe - 8./9.*m11*
         m44*L4r*pow(RSe451p,2) + 8./3.*m11*m44*L4r*RSe564p - 64./3.*
         pow(m11,2)*L8r*RSe45611p + 32./3.*pow(m11,2)*L5r*RSe45611p );

      M12p6x13LV +=  + BbV(m11,mp2,L) * ( 32./9.*m11*pow(m66,2)*L8r*
         RSp451e*RS145pe + 32./9.*m11*pow(m66,2)*L7r*RSp451e*RS145pe + 
         32./9.*m11*pow(m66,2)*L6r*RSp451e*RS145pe + 32./9.*m11*m55*m66
         *L7r*RSp461e*RS145pe + 32./9.*m11*m55*m66*L7r*RSp451e*RS146pe
          + 32./9.*m11*m55*m66*L6r*RSp461e*RS146pe + 32./9.*m11*m55*m66
         *L6r*RSp451e*RS145pe + 32./9.*m11*pow(m55,2)*L8r*RSp461e*
         RS146pe + 32./9.*m11*pow(m55,2)*L7r*RSp461e*RS146pe + 32./9.*
         m11*pow(m55,2)*L6r*RSp461e*RS146pe + 32./9.*m11*m44*m66*L7r*
         RSp561e*RS145pe + 32./9.*m11*m44*m66*L7r*RSp451e*RS156pe + 32./
         9.*m11*m44*m66*L6r*RSp561e*RS156pe + 32./9.*m11*m44*m66*L6r*
         RSp451e*RS145pe + 32./9.*m11*m44*m55*L7r*RSp561e*RS146pe + 32./
         9.*m11*m44*m55*L7r*RSp461e*RS156pe + 32./9.*m11*m44*m55*L6r*
         RSp561e*RS156pe + 32./9.*m11*m44*m55*L6r*RSp461e*RS146pe + 32./
         9.*m11*pow(m44,2)*L8r*RSp561e*RS156pe + 32./9.*m11*pow(m44,2)*
         L7r*RSp561e*RS156pe + 32./9.*m11*pow(m44,2)*L6r*RSp561e*
         RS156pe );

      M12p6x13LV +=  + BbV(m11,mp2,L) * (  - 32./3.*pow(m11,2)*m66*L7r*
         RSp451e - 32./3.*pow(m11,2)*m66*L6r*RSp45611e - 16./9.*pow(
         m11,2)*m66*L5r*RSp451e*RS145pe + 16./3.*pow(m11,2)*m66*L4r*
         RSp45611e - 16./9.*pow(m11,2)*m66*L4r*RSp561e*RS156pe - 16./9.
         *pow(m11,2)*m66*L4r*RSp461e*RS146pe - 16./9.*pow(m11,2)*m66*
         L4r*RSp451e*RS145pe - 32./3.*pow(m11,2)*m55*L7r*RSp461e - 32./
         3.*pow(m11,2)*m55*L6r*RSp45611e - 16./9.*pow(m11,2)*m55*L5r*
         RSp461e*RS146pe + 16./3.*pow(m11,2)*m55*L4r*RSp45611e - 16./9.
         *pow(m11,2)*m55*L4r*RSp561e*RS156pe - 16./9.*pow(m11,2)*m55*
         L4r*RSp461e*RS146pe - 16./9.*pow(m11,2)*m55*L4r*RSp451e*
         RS145pe - 32./3.*pow(m11,2)*m44*L7r*RSp561e - 32./3.*pow(
         m11,2)*m44*L6r*RSp45611e - 16./9.*pow(m11,2)*m44*L5r*RSp561e*
         RS156pe + 16./3.*pow(m11,2)*m44*L4r*RSp45611e - 16./9.*pow(
         m11,2)*m44*L4r*RSp561e*RS156pe - 16./9.*pow(m11,2)*m44*L4r*
         RSp461e*RS146pe - 16./9.*pow(m11,2)*m44*L4r*RSp451e*RS145pe - 
         32./3.*pow(m11,3)*L8r*RSp45611e );

      M12p6x13LV +=  + BbV(m11,mp2,L) * ( 16./3.*pow(m11,3)*L5r*
         RSp45611e );

      M12p6x13LV +=  + BbV(m11,me2,L) * ( 32./9.*m11*pow(m66,2)*L8r*
         RSe451p*RS145pe + 32./9.*m11*pow(m66,2)*L7r*RSe451p*RS145pe + 
         32./9.*m11*pow(m66,2)*L6r*RSe451p*RS145pe + 32./9.*m11*m55*m66
         *L7r*RSe461p*RS145pe + 32./9.*m11*m55*m66*L7r*RSe451p*RS146pe
          + 32./9.*m11*m55*m66*L6r*RSe461p*RS146pe + 32./9.*m11*m55*m66
         *L6r*RSe451p*RS145pe + 32./9.*m11*pow(m55,2)*L8r*RSe461p*
         RS146pe + 32./9.*m11*pow(m55,2)*L7r*RSe461p*RS146pe + 32./9.*
         m11*pow(m55,2)*L6r*RSe461p*RS146pe + 32./9.*m11*m44*m66*L7r*
         RSe561p*RS145pe + 32./9.*m11*m44*m66*L7r*RSe451p*RS156pe + 32./
         9.*m11*m44*m66*L6r*RSe561p*RS156pe + 32./9.*m11*m44*m66*L6r*
         RSe451p*RS145pe + 32./9.*m11*m44*m55*L7r*RSe561p*RS146pe + 32./
         9.*m11*m44*m55*L7r*RSe461p*RS156pe + 32./9.*m11*m44*m55*L6r*
         RSe561p*RS156pe + 32./9.*m11*m44*m55*L6r*RSe461p*RS146pe + 32./
         9.*m11*pow(m44,2)*L8r*RSe561p*RS156pe + 32./9.*m11*pow(m44,2)*
         L7r*RSe561p*RS156pe + 32./9.*m11*pow(m44,2)*L6r*RSe561p*
         RS156pe );

      M12p6x13LV +=  + BbV(m11,me2,L) * (  - 32./3.*pow(m11,2)*m66*L7r*
         RSe451p - 32./3.*pow(m11,2)*m66*L6r*RSe45611p - 16./9.*pow(
         m11,2)*m66*L5r*RSe451p*RS145pe + 16./3.*pow(m11,2)*m66*L4r*
         RSe45611p - 16./9.*pow(m11,2)*m66*L4r*RSe561p*RS156pe - 16./9.
         *pow(m11,2)*m66*L4r*RSe461p*RS146pe - 16./9.*pow(m11,2)*m66*
         L4r*RSe451p*RS145pe - 32./3.*pow(m11,2)*m55*L7r*RSe461p - 32./
         3.*pow(m11,2)*m55*L6r*RSe45611p - 16./9.*pow(m11,2)*m55*L5r*
         RSe461p*RS146pe + 16./3.*pow(m11,2)*m55*L4r*RSe45611p - 16./9.
         *pow(m11,2)*m55*L4r*RSe561p*RS156pe - 16./9.*pow(m11,2)*m55*
         L4r*RSe461p*RS146pe - 16./9.*pow(m11,2)*m55*L4r*RSe451p*
         RS145pe - 32./3.*pow(m11,2)*m44*L7r*RSe561p - 32./3.*pow(
         m11,2)*m44*L6r*RSe45611p - 16./9.*pow(m11,2)*m44*L5r*RSe561p*
         RS156pe + 16./3.*pow(m11,2)*m44*L4r*RSe45611p - 16./9.*pow(
         m11,2)*m44*L4r*RSe561p*RS156pe - 16./9.*pow(m11,2)*m44*L4r*
         RSe461p*RS146pe - 16./9.*pow(m11,2)*m44*L4r*RSe451p*RS145pe - 
         32./3.*pow(m11,3)*L8r*RSe45611p );

      M12p6x13LV +=  + BbV(m11,me2,L) * ( 16./3.*pow(m11,3)*L5r*
         RSe45611p );

      M12p6x13LV +=  + BbV(m11,L) * (  - 16./3.*m11*m66*L6r*RD1456ep + 
         8*m11*m66*L4r*RD1456ep + 16./9.*m11*pow(m66,2)*L8r*pow(
         RS145pe,2) + 16./9.*m11*pow(m66,2)*L7r*pow(RS145pe,2) + 16./9.
         *m11*pow(m66,2)*L6r*pow(RS145pe,2) - 16./3.*m11*m55*L6r*
         RD1456ep + 8*m11*m55*L4r*RD1456ep + 32./9.*m11*m55*m66*L7r*
         RS146pe*RS145pe + 16./9.*m11*m55*m66*L6r*pow(RS145pe,2) + 16./
         9.*m11*m55*m66*L6r*pow(RS146pe,2) + 16./9.*m11*pow(m55,2)*L8r*
         pow(RS146pe,2) + 16./9.*m11*pow(m55,2)*L7r*pow(RS146pe,2) + 16.
         /9.*m11*pow(m55,2)*L6r*pow(RS146pe,2) - 16./3.*m11*m44*L6r*
         RD1456ep + 8*m11*m44*L4r*RD1456ep + 32./9.*m11*m44*m66*L7r*
         RS156pe*RS145pe + 16./9.*m11*m44*m66*L6r*pow(RS145pe,2) + 16./
         9.*m11*m44*m66*L6r*pow(RS156pe,2) + 32./9.*m11*m44*m55*L7r*
         RS156pe*RS146pe + 16./9.*m11*m44*m55*L6r*pow(RS146pe,2) + 16./
         9.*m11*m44*m55*L6r*pow(RS156pe,2) + 16./9.*m11*pow(m44,2)*L8r*
         pow(RS156pe,2) + 16./9.*m11*pow(m44,2)*L7r*pow(RS156pe,2) + 16.
         /9.*m11*pow(m44,2)*L6r*pow(RS156pe,2) );

      M12p6x13LV +=  + BbV(m11,L) * (  - 64./3.*pow(m11,2)*L8r*RD1456ep
          + 40./3.*pow(m11,2)*L5r*RD1456ep - 8*pow(m11,2)*L3r*RD1456ep
          - 8*pow(m11,2)*L0r*RD1456ep - 32./3.*pow(m11,2)*m66*L7r*
         RS145pe - 32./3.*pow(m11,2)*m66*L6r*RC1456pe - 8./9.*pow(
         m11,2)*m66*L5r*pow(RS145pe,2) + 16./3.*pow(m11,2)*m66*L4r*
         RC1456pe - 8./9.*pow(m11,2)*m66*L4r*pow(RS145pe,2) - 8./9.*
         pow(m11,2)*m66*L4r*pow(RS146pe,2) - 8./9.*pow(m11,2)*m66*L4r*
         pow(RS156pe,2) - 32./3.*pow(m11,2)*m55*L7r*RS146pe - 32./3.*
         pow(m11,2)*m55*L6r*RC1456pe - 8./9.*pow(m11,2)*m55*L5r*pow(
         RS146pe,2) + 16./3.*pow(m11,2)*m55*L4r*RC1456pe - 8./9.*pow(
         m11,2)*m55*L4r*pow(RS145pe,2) - 8./9.*pow(m11,2)*m55*L4r*pow(
         RS146pe,2) - 8./9.*pow(m11,2)*m55*L4r*pow(RS156pe,2) - 32./3.*
         pow(m11,2)*m44*L7r*RS156pe - 32./3.*pow(m11,2)*m44*L6r*
         RC1456pe - 8./9.*pow(m11,2)*m44*L5r*pow(RS156pe,2) + 16./3.*
         pow(m11,2)*m44*L4r*RC1456pe - 8./9.*pow(m11,2)*m44*L4r*pow(
         RS145pe,2) );

      M12p6x13LV +=  + BbV(m11,L) * (  - 8./9.*pow(m11,2)*m44*L4r*pow(
         RS146pe,2) - 8./9.*pow(m11,2)*m44*L4r*pow(RS156pe,2) - 32./3.*
         pow(m11,3)*L8r*RC1456pe + 16*pow(m11,3)*L7r + 16./3.*pow(
         m11,3)*L5r*RC1456pe );

      M12p6x13LV +=  + BbV(m14,L) * ( 4./3.*pow(m44,2)*m66*L6r + 4./3.*
         pow(m44,2)*m55*L6r + 2./3.*pow(m44,3)*L8r + 4./3.*pow(m44,3)*
         L6r - 8./3.*m14*m44*m66*L6r - 4./3.*m14*m44*m66*L4r - 8./3.*
         m14*m44*m55*L6r - 4./3.*m14*m44*m55*L4r - 4./3.*m14*pow(m44,2)
         *L8r - 8./3.*m14*pow(m44,2)*L6r - 2./3.*m14*pow(m44,2)*L5r - 4.
         /3.*m14*pow(m44,2)*L4r + 8./3.*pow(m14,2)*m66*L4r + 8./3.*pow(
         m14,2)*m55*L4r + 4./3.*pow(m14,2)*m44*L5r + 8./3.*pow(m14,2)*
         m44*L4r + 8./3.*m11*m44*m66*L6r + 8./3.*m11*m44*m55*L6r + 2*
         m11*pow(m44,2)*L8r + 8./3.*m11*pow(m44,2)*L6r - 8./3.*m11*m14*
         m66*L6r - 4./3.*m11*m14*m66*L4r - 8./3.*m11*m14*m55*L6r - 4./3.
         *m11*m14*m55*L4r - 8./3.*m11*m14*m44*L8r - 8./3.*m11*m14*m44*
         L6r - 4./3.*m11*m14*m44*L5r - 4./3.*m11*m14*m44*L4r + 4./3.*
         m11*pow(m14,2)*L5r + 4./3.*pow(m11,2)*m66*L6r + 4./3.*pow(
         m11,2)*m55*L6r + 2*pow(m11,2)*m44*L8r + 4./3.*pow(m11,2)*m44*
         L6r - 4./3.*pow(m11,2)*m14*L8r - 2./3.*pow(m11,2)*m14*L5r + 2./
         3.*pow(m11,3)*L8r );

      M12p6x13LV +=  + BbV(m15,L) * ( 4./3.*pow(m55,2)*m66*L6r + 2./3.*
         pow(m55,3)*L8r + 4./3.*pow(m55,3)*L6r + 4./3.*m44*pow(m55,2)*
         L6r - 8./3.*m15*m55*m66*L6r - 4./3.*m15*m55*m66*L4r - 4./3.*
         m15*pow(m55,2)*L8r - 8./3.*m15*pow(m55,2)*L6r - 2./3.*m15*pow(
         m55,2)*L5r - 4./3.*m15*pow(m55,2)*L4r - 8./3.*m15*m44*m55*L6r
          - 4./3.*m15*m44*m55*L4r + 8./3.*pow(m15,2)*m66*L4r + 4./3.*
         pow(m15,2)*m55*L5r + 8./3.*pow(m15,2)*m55*L4r + 8./3.*pow(
         m15,2)*m44*L4r + 8./3.*m11*m55*m66*L6r + 2*m11*pow(m55,2)*L8r
          + 8./3.*m11*pow(m55,2)*L6r + 8./3.*m11*m44*m55*L6r - 8./3.*
         m11*m15*m66*L6r - 4./3.*m11*m15*m66*L4r - 8./3.*m11*m15*m55*
         L8r - 8./3.*m11*m15*m55*L6r - 4./3.*m11*m15*m55*L5r - 4./3.*
         m11*m15*m55*L4r - 8./3.*m11*m15*m44*L6r - 4./3.*m11*m15*m44*
         L4r + 4./3.*m11*pow(m15,2)*L5r + 4./3.*pow(m11,2)*m66*L6r + 2*
         pow(m11,2)*m55*L8r + 4./3.*pow(m11,2)*m55*L6r + 4./3.*pow(
         m11,2)*m44*L6r - 4./3.*pow(m11,2)*m15*L8r - 2./3.*pow(m11,2)*
         m15*L5r );

      M12p6x13LV +=  + BbV(m15,L) * ( 2./3.*pow(m11,3)*L8r );

      M12p6x13LV +=  + BbV(m16,L) * ( 2./3.*pow(m66,3)*L8r + 4./3.*pow(
         m66,3)*L6r + 4./3.*m55*pow(m66,2)*L6r + 4./3.*m44*pow(m66,2)*
         L6r - 4./3.*m16*pow(m66,2)*L8r - 8./3.*m16*pow(m66,2)*L6r - 2./
         3.*m16*pow(m66,2)*L5r - 4./3.*m16*pow(m66,2)*L4r - 8./3.*m16*
         m55*m66*L6r - 4./3.*m16*m55*m66*L4r - 8./3.*m16*m44*m66*L6r - 
         4./3.*m16*m44*m66*L4r + 4./3.*pow(m16,2)*m66*L5r + 8./3.*pow(
         m16,2)*m66*L4r + 8./3.*pow(m16,2)*m55*L4r + 8./3.*pow(m16,2)*
         m44*L4r + 2*m11*pow(m66,2)*L8r + 8./3.*m11*pow(m66,2)*L6r + 8./
         3.*m11*m55*m66*L6r + 8./3.*m11*m44*m66*L6r - 8./3.*m11*m16*m66
         *L8r - 8./3.*m11*m16*m66*L6r - 4./3.*m11*m16*m66*L5r - 4./3.*
         m11*m16*m66*L4r - 8./3.*m11*m16*m55*L6r - 4./3.*m11*m16*m55*
         L4r - 8./3.*m11*m16*m44*L6r - 4./3.*m11*m16*m44*L4r + 4./3.*
         m11*pow(m16,2)*L5r + 2*pow(m11,2)*m66*L8r + 4./3.*pow(m11,2)*
         m66*L6r + 4./3.*pow(m11,2)*m55*L6r + 4./3.*pow(m11,2)*m44*L6r
          - 4./3.*pow(m11,2)*m16*L8r - 2./3.*pow(m11,2)*m16*L5r + 2./3.
         *pow(m11,3)*L8r );

      M12p6x13LV +=  + BbV(mp2,L) * (  - 8./9.*m11*m66*mp2*L5r*pow(
         RSp451e,2) - 8./9.*m11*m66*mp2*L4r*pow(RSp561e,2) - 8./9.*m11*
         m66*mp2*L4r*pow(RSp461e,2) - 8./9.*m11*m66*mp2*L4r*pow(
         RSp451e,2) + 16./9.*m11*pow(m66,2)*L8r*pow(RSp451e,2) + 16./9.
         *m11*pow(m66,2)*L7r*pow(RSp451e,2) + 16./9.*m11*pow(m66,2)*L6r
         *pow(RSp451e,2) - 8./9.*m11*m55*mp2*L5r*pow(RSp461e,2) - 8./9.
         *m11*m55*mp2*L4r*pow(RSp561e,2) - 8./9.*m11*m55*mp2*L4r*pow(
         RSp461e,2) - 8./9.*m11*m55*mp2*L4r*pow(RSp451e,2) + 32./9.*m11
         *m55*m66*L7r*RSp451e*RSp461e + 16./9.*m11*m55*m66*L6r*pow(
         RSp461e,2) + 16./9.*m11*m55*m66*L6r*pow(RSp451e,2) + 16./9.*
         m11*pow(m55,2)*L8r*pow(RSp461e,2) + 16./9.*m11*pow(m55,2)*L7r*
         pow(RSp461e,2) + 16./9.*m11*pow(m55,2)*L6r*pow(RSp461e,2) - 8./
         9.*m11*m44*mp2*L5r*pow(RSp561e,2) - 8./9.*m11*m44*mp2*L4r*pow(
         RSp561e,2) - 8./9.*m11*m44*mp2*L4r*pow(RSp461e,2) - 8./9.*m11*
         m44*mp2*L4r*pow(RSp451e,2) + 32./9.*m11*m44*m66*L7r*RSp451e*
         RSp561e );

      M12p6x13LV +=  + BbV(mp2,L) * ( 16./9.*m11*m44*m66*L6r*pow(
         RSp561e,2) + 16./9.*m11*m44*m66*L6r*pow(RSp451e,2) + 32./9.*
         m11*m44*m55*L7r*RSp461e*RSp561e + 16./9.*m11*m44*m55*L6r*pow(
         RSp561e,2) + 16./9.*m11*m44*m55*L6r*pow(RSp461e,2) + 16./9.*
         m11*pow(m44,2)*L8r*pow(RSp561e,2) + 16./9.*m11*pow(m44,2)*L7r*
         pow(RSp561e,2) + 16./9.*m11*pow(m44,2)*L6r*pow(RSp561e,2) );

      M12p6x13LV +=  + BbV(me2,mp2,L) * (  - 16./9.*m11*m66*me2*L5r*
         RSp451e*RSe451p - 16./9.*m11*m66*me2*L4r*RSp561e*RSe561p - 16./
         9.*m11*m66*me2*L4r*RSp461e*RSe461p - 16./9.*m11*m66*me2*L4r*
         RSp451e*RSe451p + 32./9.*m11*pow(m66,2)*L8r*RSp451e*RSe451p + 
         32./9.*m11*pow(m66,2)*L7r*RSp451e*RSe451p + 32./9.*m11*pow(
         m66,2)*L6r*RSp451e*RSe451p - 16./9.*m11*m55*me2*L5r*RSp461e*
         RSe461p - 16./9.*m11*m55*me2*L4r*RSp561e*RSe561p - 16./9.*m11*
         m55*me2*L4r*RSp461e*RSe461p - 16./9.*m11*m55*me2*L4r*RSp451e*
         RSe451p + 32./9.*m11*m55*m66*L7r*RSp461e*RSe451p + 32./9.*m11*
         m55*m66*L7r*RSp451e*RSe461p + 32./9.*m11*m55*m66*L6r*RSp461e*
         RSe461p + 32./9.*m11*m55*m66*L6r*RSp451e*RSe451p + 32./9.*m11*
         pow(m55,2)*L8r*RSp461e*RSe461p + 32./9.*m11*pow(m55,2)*L7r*
         RSp461e*RSe461p + 32./9.*m11*pow(m55,2)*L6r*RSp461e*RSe461p - 
         16./9.*m11*m44*me2*L5r*RSp561e*RSe561p - 16./9.*m11*m44*me2*
         L4r*RSp561e*RSe561p - 16./9.*m11*m44*me2*L4r*RSp461e*RSe461p
          - 16./9.*m11*m44*me2*L4r*RSp451e*RSe451p );

      M12p6x13LV +=  + BbV(me2,mp2,L) * ( 32./9.*m11*m44*m66*L7r*
         RSp561e*RSe451p + 32./9.*m11*m44*m66*L7r*RSp451e*RSe561p + 32./
         9.*m11*m44*m66*L6r*RSp561e*RSe561p + 32./9.*m11*m44*m66*L6r*
         RSp451e*RSe451p + 32./9.*m11*m44*m55*L7r*RSp561e*RSe461p + 32./
         9.*m11*m44*m55*L7r*RSp461e*RSe561p + 32./9.*m11*m44*m55*L6r*
         RSp561e*RSe561p + 32./9.*m11*m44*m55*L6r*RSp461e*RSe461p + 32./
         9.*m11*pow(m44,2)*L8r*RSp561e*RSe561p + 32./9.*m11*pow(m44,2)*
         L7r*RSp561e*RSe561p + 32./9.*m11*pow(m44,2)*L6r*RSp561e*
         RSe561p );

      M12p6x13LV +=  + BbV(me2,L) * (  - 8./9.*m11*m66*me2*L5r*pow(
         RSe451p,2) - 8./9.*m11*m66*me2*L4r*pow(RSe561p,2) - 8./9.*m11*
         m66*me2*L4r*pow(RSe461p,2) - 8./9.*m11*m66*me2*L4r*pow(
         RSe451p,2) + 16./9.*m11*pow(m66,2)*L8r*pow(RSe451p,2) + 16./9.
         *m11*pow(m66,2)*L7r*pow(RSe451p,2) + 16./9.*m11*pow(m66,2)*L6r
         *pow(RSe451p,2) - 8./9.*m11*m55*me2*L5r*pow(RSe461p,2) - 8./9.
         *m11*m55*me2*L4r*pow(RSe561p,2) - 8./9.*m11*m55*me2*L4r*pow(
         RSe461p,2) - 8./9.*m11*m55*me2*L4r*pow(RSe451p,2) + 32./9.*m11
         *m55*m66*L7r*RSe451p*RSe461p + 16./9.*m11*m55*m66*L6r*pow(
         RSe461p,2) + 16./9.*m11*m55*m66*L6r*pow(RSe451p,2) + 16./9.*
         m11*pow(m55,2)*L8r*pow(RSe461p,2) + 16./9.*m11*pow(m55,2)*L7r*
         pow(RSe461p,2) + 16./9.*m11*pow(m55,2)*L6r*pow(RSe461p,2) - 8./
         9.*m11*m44*me2*L5r*pow(RSe561p,2) - 8./9.*m11*m44*me2*L4r*pow(
         RSe561p,2) - 8./9.*m11*m44*me2*L4r*pow(RSe461p,2) - 8./9.*m11*
         m44*me2*L4r*pow(RSe451p,2) + 32./9.*m11*m44*m66*L7r*RSe451p*
         RSe561p );

      M12p6x13LV +=  + BbV(me2,L) * ( 16./9.*m11*m44*m66*L6r*pow(
         RSe561p,2) + 16./9.*m11*m44*m66*L6r*pow(RSe451p,2) + 32./9.*
         m11*m44*m55*L7r*RSe461p*RSe561p + 16./9.*m11*m44*m55*L6r*pow(
         RSe561p,2) + 16./9.*m11*m44*m55*L6r*pow(RSe461p,2) + 16./9.*
         m11*pow(m44,2)*L8r*pow(RSe561p,2) + 16./9.*m11*pow(m44,2)*L7r*
         pow(RSe561p,2) + 16./9.*m11*pow(m44,2)*L6r*pow(RSe561p,2) );

      M12p6x13LV +=  + CbV(m11,L) * (  - 32./3.*pow(m11,2)*m66*L6r*
         RD1456ep + 16./3.*pow(m11,2)*m66*L4r*RD1456ep - 32./3.*pow(
         m11,2)*m55*L6r*RD1456ep + 16./3.*pow(m11,2)*m55*L4r*RD1456ep
          - 32./3.*pow(m11,2)*m44*L6r*RD1456ep + 16./3.*pow(m11,2)*m44*
         L4r*RD1456ep - 32./3.*pow(m11,3)*L8r*RD1456ep + 16./3.*pow(
         m11,3)*L5r*RD1456ep );

      M12p6x13LV +=  + A23bV(m11,L) * ( 8*m11*L3r*RC1456pe - 12*m11*L2r
          - 24*m11*L1r + 8*m11*L0r*RC1456pe );

      M12p6x13LV +=  + A23bV(m14,L) * (  - 12*m11*L3r - 24*m11*L0r );

      M12p6x13LV +=  + A23bV(m15,L) * (  - 12*m11*L3r - 24*m11*L0r );

      M12p6x13LV +=  + A23bV(m16,L) * (  - 12*m11*L3r - 24*m11*L0r );

      M12p6x13LV +=  + A23bV(m44,L) * (  - 12*m11*L2r + 4*m11*L2r*
         RS456pe );

      M12p6x13LV +=  + A23bV(m45,L) * (  - 24*m11*L2r );

      M12p6x13LV +=  + A23bV(m46,L) * (  - 24*m11*L2r );

      M12p6x13LV +=  + A23bV(m55,L) * (  - 12*m11*L2r + 4*m11*L2r*
         RS546pe );

      M12p6x13LV +=  + A23bV(m56,L) * (  - 24*m11*L2r );

      M12p6x13LV +=  + A23bV(m66,L) * (  - 12*m11*L2r + 4*m11*L2r*
         RS645pe );

      M12p6x13LV +=  + A23bV(mp2,L) * ( 8*m11*L3r*RSp45611e + 4*m11*L2r
         *RSp564e + 4*m11*L2r*RSp465e + 4*m11*L2r*RSp456e + 8*m11*L0r*
         RSp45611e );

      M12p6x13LV +=  + A23bV(me2,L) * ( 8*m11*L3r*RSe45611p + 4*m11*L2r
         *RSe564p + 4*m11*L2r*RSe465p + 4*m11*L2r*RSe456p + 8*m11*L0r*
         RSe45611p );

      M12p6x13LV +=  + B23bV(m11,L) * ( 8*m11*L3r*RD1456ep + 8*m11*L0r*
         RD1456ep );

  return M12p6x13LV/pow(f0,4);
}

double mv1s3nf3p6RV(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 4) || (B0mq.size() != 4)){
    cout << "ERROR: mv1s3nf3p6RV probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m55 = 2.*B0mq[2];
  double m66 = 2.*B0mq[3];
  double m14 = (m11+m44)/2;
  double m15 = (m11+m55)/2;
  double m16 = (m11+m66)/2;
  double m45 = (m44+m55)/2;
  double m46 = (m44+m66)/2;
  double m56 = (m55+m66)/2;

  double xb1 = (m44+m55+m66)/3.;
  double msqr = sqrt(xb1*xb1-(m44*m55+m44*m66+m55*m66)/3.);
  double mp2 = xb1-msqr; 
  double me2 = xb1+msqr;
  double mu2 = mu0*mu0;

  double RN1p = RN(m11,mp2);
  double RN1e = RN(m11,me2);
  //double RNep = RN(me2,mp2);

  double RSp456e = RS(mp2,m44,m55,m66,me2);
  double RSp465e = RS(mp2,m44,m66,m55,me2);
  double RSp564e = RS(mp2,m55,m66,m44,me2);
  double RSe456p = RS(me2,m44,m55,m66,mp2);
  double RSe465p = RS(me2,m44,m66,m55,mp2);
  double RSe564p = RS(me2,m55,m66,m44,mp2);
  double RSp451e = RS(mp2,m44,m55,m11,me2);
  double RSp461e = RS(mp2,m44,m66,m11,me2);
  double RSp561e = RS(mp2,m55,m66,m11,me2);
  double RSe451p = RS(me2,m44,m55,m11,mp2);
  double RSe461p = RS(me2,m44,m66,m11,mp2);
  double RSe561p = RS(me2,m55,m66,m11,mp2);
  double RS456pe = RS(m44,m55,m66,mp2,me2);
  double RS546pe = RS(m55,m44,m66,mp2,me2);
  double RS645pe = RS(m66,m44,m55,mp2,me2);
  double RS156pe = RS(m11,m55,m66,mp2,me2);
  double RS146pe = RS(m11,m44,m66,mp2,me2);
  double RS145pe = RS(m11,m44,m55,mp2,me2);

  double RD1456ep = RD(m11,m44,m55,m66,me2,mp2);
  double RC1456pe = RC(m11,m44,m55,m66,mp2,me2);  
  double RSe45611p = RS(me2,m44,m55,m66,m11,m11,mp2);
  double RSp45611e = RS(mp2,m44,m55,m66,m11,m11,me2);
  //double RS1456ppe = RS(m11,m44,m55,m66,mp2,mp2,me2);
  //double RS1456pee = RS(m11,m44,m55,m66,mp2,me2,me2);

  double M12p6x13RV =
       + Ab(m11,mu2)*AbV(m11,L) * ( m11 + 1./9.*m11*pow(RC1456pe,2) + 2.
         /9.*pow(m11,2)*RN1p*RSp45611e*RC1456pe + 2./9.*pow(m11,2)*RN1e
         *RSe45611p*RC1456pe );

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(m14,L) * ( 2./45.*RD1456ep + 1./
         30.*m44*RC1456pe - 1./45.*m44*RS156pe - 1./15.*m14*RC1456pe + 
         2./15.*m14*RS156pe + 43./90.*m11*RC1456pe - 5./9.*m11*RS156pe
          - 1./9.*m11*m44*RN1p*RSp45611e - 1./9.*m11*m44*RN1p*RSp561e
          - 1./9.*m11*m44*RN1e*RSe45611p - 1./9.*m11*m44*RN1e*RSe561p
          + 2./9.*m11*m14*RN1p*RSp45611e - 2./9.*m11*m14*RN1p*RSp561e
          + 2./9.*m11*m14*RN1e*RSe45611p - 2./9.*m11*m14*RN1e*RSe561p
          - 1./9.*pow(m11,2)*RN1p*RSp45611e - 1./3.*pow(m11,2)*RN1p*
         RSp561e - 1./9.*pow(m11,2)*RN1e*RSe45611p - 1./3.*pow(m11,2)*
         RN1e*RSe561p );

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(m15,L) * ( 2./45.*RD1456ep + 1./
         30.*m55*RC1456pe - 1./45.*m55*RS146pe - 1./15.*m15*RC1456pe + 
         2./15.*m15*RS146pe + 43./90.*m11*RC1456pe - 5./9.*m11*RS146pe
          - 1./9.*m11*m55*RN1p*RSp45611e - 1./9.*m11*m55*RN1p*RSp461e
          - 1./9.*m11*m55*RN1e*RSe45611p - 1./9.*m11*m55*RN1e*RSe461p
          + 2./9.*m11*m15*RN1p*RSp45611e - 2./9.*m11*m15*RN1p*RSp461e
          + 2./9.*m11*m15*RN1e*RSe45611p - 2./9.*m11*m15*RN1e*RSe461p
          - 1./9.*pow(m11,2)*RN1p*RSp45611e - 1./3.*pow(m11,2)*RN1p*
         RSp461e - 1./9.*pow(m11,2)*RN1e*RSe45611p - 1./3.*pow(m11,2)*
         RN1e*RSe461p );

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(m16,L) * ( 2./45.*RD1456ep + 1./
         30.*m66*RC1456pe - 1./45.*m66*RS145pe - 1./15.*m16*RC1456pe + 
         2./15.*m16*RS145pe + 43./90.*m11*RC1456pe - 5./9.*m11*RS145pe
          - 1./9.*m11*m66*RN1p*RSp45611e - 1./9.*m11*m66*RN1p*RSp451e
          - 1./9.*m11*m66*RN1e*RSe45611p - 1./9.*m11*m66*RN1e*RSe451p
          + 2./9.*m11*m16*RN1p*RSp45611e - 2./9.*m11*m16*RN1p*RSp451e
          + 2./9.*m11*m16*RN1e*RSe45611p - 2./9.*m11*m16*RN1e*RSe451p
          - 1./9.*pow(m11,2)*RN1p*RSp45611e - 1./3.*pow(m11,2)*RN1p*
         RSp451e - 1./9.*pow(m11,2)*RN1e*RSe45611p - 1./3.*pow(m11,2)*
         RN1e*RSe451p );

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(m44,L) * ( 2./9.*m11*m44*RN1p*
         RSp561e*RS156pe - 2./27.*m11*m44*RN1p*RSp561e*RS456pe*RS156pe
          + 2./9.*m11*m44*RN1e*RSe561p*RS156pe - 2./27.*m11*m44*RN1e*
         RSe561p*RS456pe*RS156pe );

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(m45,L) * (  - 1./27.*m11*pow(
         RS146pe,2) + 2./27.*m11*RS156pe*RS146pe - 1./27.*m11*pow(
         RS156pe,2) + 1./27.*m11*m55*RN1p*RSp561e*RS146pe + 1./27.*m11*
         m55*RN1p*RSp561e*RS156pe + 1./9.*m11*m55*RN1p*RSp461e*RS146pe
          + 1./27.*m11*m55*RN1p*RSp461e*RS156pe + 1./27.*m11*m55*RN1e*
         RSe561p*RS146pe + 1./27.*m11*m55*RN1e*RSe561p*RS156pe + 1./9.*
         m11*m55*RN1e*RSe461p*RS146pe + 1./27.*m11*m55*RN1e*RSe461p*
         RS156pe + 2./27.*m11*m45*RN1p*RSp561e*RS146pe - 2./27.*m11*m45
         *RN1p*RSp561e*RS156pe - 2./27.*m11*m45*RN1p*RSp461e*RS146pe + 
         2./27.*m11*m45*RN1p*RSp461e*RS156pe + 2./27.*m11*m45*RN1e*
         RSe561p*RS146pe - 2./27.*m11*m45*RN1e*RSe561p*RS156pe - 2./27.
         *m11*m45*RN1e*RSe461p*RS146pe + 2./27.*m11*m45*RN1e*RSe461p*
         RS156pe + 1./27.*m11*m44*RN1p*RSp561e*RS146pe + 1./9.*m11*m44*
         RN1p*RSp561e*RS156pe + 1./27.*m11*m44*RN1p*RSp461e*RS146pe + 1.
         /27.*m11*m44*RN1p*RSp461e*RS156pe + 1./27.*m11*m44*RN1e*
         RSe561p*RS146pe );

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(m45,L) * ( 1./9.*m11*m44*RN1e*
         RSe561p*RS156pe + 1./27.*m11*m44*RN1e*RSe461p*RS146pe + 1./27.
         *m11*m44*RN1e*RSe461p*RS156pe + 2./27.*pow(m11,2)*RN1p*RSp561e
         *RS146pe - 2./27.*pow(m11,2)*RN1p*RSp561e*RS156pe - 2./27.*
         pow(m11,2)*RN1p*RSp461e*RS146pe + 2./27.*pow(m11,2)*RN1p*
         RSp461e*RS156pe + 2./27.*pow(m11,2)*RN1e*RSe561p*RS146pe - 2./
         27.*pow(m11,2)*RN1e*RSe561p*RS156pe - 2./27.*pow(m11,2)*RN1e*
         RSe461p*RS146pe + 2./27.*pow(m11,2)*RN1e*RSe461p*RS156pe );

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(m46,L) * (  - 1./27.*m11*pow(
         RS145pe,2) + 2./27.*m11*RS156pe*RS145pe - 1./27.*m11*pow(
         RS156pe,2) + 1./27.*m11*m66*RN1p*RSp561e*RS145pe + 1./27.*m11*
         m66*RN1p*RSp561e*RS156pe + 1./9.*m11*m66*RN1p*RSp451e*RS145pe
          + 1./27.*m11*m66*RN1p*RSp451e*RS156pe + 1./27.*m11*m66*RN1e*
         RSe561p*RS145pe + 1./27.*m11*m66*RN1e*RSe561p*RS156pe + 1./9.*
         m11*m66*RN1e*RSe451p*RS145pe + 1./27.*m11*m66*RN1e*RSe451p*
         RS156pe + 2./27.*m11*m46*RN1p*RSp561e*RS145pe - 2./27.*m11*m46
         *RN1p*RSp561e*RS156pe - 2./27.*m11*m46*RN1p*RSp451e*RS145pe + 
         2./27.*m11*m46*RN1p*RSp451e*RS156pe + 2./27.*m11*m46*RN1e*
         RSe561p*RS145pe - 2./27.*m11*m46*RN1e*RSe561p*RS156pe - 2./27.
         *m11*m46*RN1e*RSe451p*RS145pe + 2./27.*m11*m46*RN1e*RSe451p*
         RS156pe + 1./27.*m11*m44*RN1p*RSp561e*RS145pe + 1./9.*m11*m44*
         RN1p*RSp561e*RS156pe + 1./27.*m11*m44*RN1p*RSp451e*RS145pe + 1.
         /27.*m11*m44*RN1p*RSp451e*RS156pe + 1./27.*m11*m44*RN1e*
         RSe561p*RS145pe );

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(m46,L) * ( 1./9.*m11*m44*RN1e*
         RSe561p*RS156pe + 1./27.*m11*m44*RN1e*RSe451p*RS145pe + 1./27.
         *m11*m44*RN1e*RSe451p*RS156pe + 2./27.*pow(m11,2)*RN1p*RSp561e
         *RS145pe - 2./27.*pow(m11,2)*RN1p*RSp561e*RS156pe - 2./27.*
         pow(m11,2)*RN1p*RSp451e*RS145pe + 2./27.*pow(m11,2)*RN1p*
         RSp451e*RS156pe + 2./27.*pow(m11,2)*RN1e*RSe561p*RS145pe - 2./
         27.*pow(m11,2)*RN1e*RSe561p*RS156pe - 2./27.*pow(m11,2)*RN1e*
         RSe451p*RS145pe + 2./27.*pow(m11,2)*RN1e*RSe451p*RS156pe );

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(m55,L) * ( 2./9.*m11*m55*RN1p*
         RSp461e*RS146pe - 2./27.*m11*m55*RN1p*RSp461e*RS546pe*RS146pe
          + 2./9.*m11*m55*RN1e*RSe461p*RS146pe - 2./27.*m11*m55*RN1e*
         RSe461p*RS546pe*RS146pe );

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(m56,L) * (  - 1./27.*m11*pow(
         RS145pe,2) + 2./27.*m11*RS146pe*RS145pe - 1./27.*m11*pow(
         RS146pe,2) + 1./27.*m11*m66*RN1p*RSp461e*RS145pe + 1./27.*m11*
         m66*RN1p*RSp461e*RS146pe + 1./9.*m11*m66*RN1p*RSp451e*RS145pe
          + 1./27.*m11*m66*RN1p*RSp451e*RS146pe + 1./27.*m11*m66*RN1e*
         RSe461p*RS145pe + 1./27.*m11*m66*RN1e*RSe461p*RS146pe + 1./9.*
         m11*m66*RN1e*RSe451p*RS145pe + 1./27.*m11*m66*RN1e*RSe451p*
         RS146pe + 2./27.*m11*m56*RN1p*RSp461e*RS145pe - 2./27.*m11*m56
         *RN1p*RSp461e*RS146pe - 2./27.*m11*m56*RN1p*RSp451e*RS145pe + 
         2./27.*m11*m56*RN1p*RSp451e*RS146pe + 2./27.*m11*m56*RN1e*
         RSe461p*RS145pe - 2./27.*m11*m56*RN1e*RSe461p*RS146pe - 2./27.
         *m11*m56*RN1e*RSe451p*RS145pe + 2./27.*m11*m56*RN1e*RSe451p*
         RS146pe + 1./27.*m11*m55*RN1p*RSp461e*RS145pe + 1./9.*m11*m55*
         RN1p*RSp461e*RS146pe + 1./27.*m11*m55*RN1p*RSp451e*RS145pe + 1.
         /27.*m11*m55*RN1p*RSp451e*RS146pe + 1./27.*m11*m55*RN1e*
         RSe461p*RS145pe );

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(m56,L) * ( 1./9.*m11*m55*RN1e*
         RSe461p*RS146pe + 1./27.*m11*m55*RN1e*RSe451p*RS145pe + 1./27.
         *m11*m55*RN1e*RSe451p*RS146pe + 2./27.*pow(m11,2)*RN1p*RSp461e
         *RS145pe - 2./27.*pow(m11,2)*RN1p*RSp461e*RS146pe - 2./27.*
         pow(m11,2)*RN1p*RSp451e*RS145pe + 2./27.*pow(m11,2)*RN1p*
         RSp451e*RS146pe + 2./27.*pow(m11,2)*RN1e*RSe461p*RS145pe - 2./
         27.*pow(m11,2)*RN1e*RSe461p*RS146pe - 2./27.*pow(m11,2)*RN1e*
         RSe451p*RS145pe + 2./27.*pow(m11,2)*RN1e*RSe451p*RS146pe );

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(m66,L) * ( 2./9.*m11*m66*RN1p*
         RSp451e*RS145pe - 2./27.*m11*m66*RN1p*RSp451e*RS645pe*RS145pe
          + 2./9.*m11*m66*RN1e*RSe451p*RS145pe - 2./27.*m11*m66*RN1e*
         RSe451p*RS645pe*RS145pe );

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(mp2,L) * ( 1./9.*m11*RSp45611e*
         RC1456pe - 2./27.*m11*m66*RN1p*RSp456e*RSp451e*RS145pe - 2./27.
         *m11*m66*RN1e*RSp456e*RSe451p*RS145pe - 2./27.*m11*m55*RN1p*
         RSp465e*RSp461e*RS146pe - 2./27.*m11*m55*RN1e*RSp465e*RSe461p*
         RS146pe - 2./27.*m11*m44*RN1p*RSp564e*RSp561e*RS156pe - 2./27.
         *m11*m44*RN1e*RSp564e*RSe561p*RS156pe + 2./9.*pow(m11,2)*RN1p*
         pow(RSp45611e,2) + 2./9.*pow(m11,2)*RN1e*RSp45611e*RSe45611p )
         ;

      M12p6x13RV +=  + Ab(m11,mu2)*AbV(me2,L) * ( 1./9.*m11*RSe45611p*
         RC1456pe - 2./27.*m11*m66*RN1p*RSe456p*RSp451e*RS145pe - 2./27.
         *m11*m66*RN1e*RSe456p*RSe451p*RS145pe - 2./27.*m11*m55*RN1p*
         RSe465p*RSp461e*RS146pe - 2./27.*m11*m55*RN1e*RSe465p*RSe461p*
         RS146pe - 2./27.*m11*m44*RN1p*RSe564p*RSp561e*RS156pe - 2./27.
         *m11*m44*RN1e*RSe564p*RSe561p*RS156pe + 2./9.*pow(m11,2)*RN1p*
         RSp45611e*RSe45611p + 2./9.*pow(m11,2)*RN1e*pow(RSe45611p,2) )
         ;

      M12p6x13RV +=  + Ab(m11,mu2)*BbV(m11,mp2,L) * ( 2./9.*pow(m11,2)*
         RSp45611e*RC1456pe );

      M12p6x13RV +=  + Ab(m11,mu2)*BbV(m11,me2,L) * ( 2./9.*pow(m11,2)*
         RSe45611p*RC1456pe );

      M12p6x13RV +=  + Ab(m11,mu2)*BbV(m11,L) * ( 1./9.*m11*RD1456ep*
         RC1456pe + pow(m11,2) + 2./9.*pow(m11,2)*pow(RC1456pe,2) + 2./
         9.*pow(m11,2)*RN1p*RSp45611e*RD1456ep + 2./9.*pow(m11,2)*RN1e*
         RSe45611p*RD1456ep );

      M12p6x13RV +=  + Ab(m11,mu2)*BbV(m14,L) * ( 1./108.*m44*RD1456ep
          - 1./216.*pow(m44,2)*RC1456pe - 1./108.*pow(m44,2)*RS156pe - 
         1./54.*m14*RD1456ep + 1./54.*m14*m44*RC1456pe - 1./54.*pow(
         m14,2)*RC1456pe + 1./27.*pow(m14,2)*RS156pe + 1./108.*m11*
         RD1456ep - 1./108.*m11*m44*RC1456pe - 1./27.*m11*m44*RS156pe
          + 1./54.*m11*m14*RC1456pe + 1./27.*m11*m14*RS156pe - 1./216.*
         pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS156pe );

      M12p6x13RV +=  + Ab(m11,mu2)*BbV(m15,L) * ( 1./108.*m55*RD1456ep
          - 1./216.*pow(m55,2)*RC1456pe - 1./108.*pow(m55,2)*RS146pe - 
         1./54.*m15*RD1456ep + 1./54.*m15*m55*RC1456pe - 1./54.*pow(
         m15,2)*RC1456pe + 1./27.*pow(m15,2)*RS146pe + 1./108.*m11*
         RD1456ep - 1./108.*m11*m55*RC1456pe - 1./27.*m11*m55*RS146pe
          + 1./54.*m11*m15*RC1456pe + 1./27.*m11*m15*RS146pe - 1./216.*
         pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS146pe );

      M12p6x13RV +=  + Ab(m11,mu2)*BbV(m16,L) * ( 1./108.*m66*RD1456ep
          - 1./216.*pow(m66,2)*RC1456pe - 1./108.*pow(m66,2)*RS145pe - 
         1./54.*m16*RD1456ep + 1./54.*m16*m66*RC1456pe - 1./54.*pow(
         m16,2)*RC1456pe + 1./27.*pow(m16,2)*RS145pe + 1./108.*m11*
         RD1456ep - 1./108.*m11*m66*RC1456pe - 1./27.*m11*m66*RS145pe
          + 1./54.*m11*m16*RC1456pe + 1./27.*m11*m16*RS145pe - 1./216.*
         pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS145pe );

      M12p6x13RV +=  + Ab(m11,mu2)*CbV(m11,L) * ( 2./9.*pow(m11,2)*
         RD1456ep*RC1456pe );

      M12p6x13RV +=  + Ab(m14,mu2)*AbV(m11,L) * ( 2./45.*RD1456ep + 1./
         30.*m44*RC1456pe - 1./45.*m44*RS156pe - 1./15.*m14*RC1456pe + 
         2./15.*m14*RS156pe + 43./90.*m11*RC1456pe - 5./9.*m11*RS156pe
          );

      M12p6x13RV +=  + Ab(m14,mu2)*AbV(m14,L) * (  - 2./27.*RD1456ep - 
         2./27.*me2*RSe45611p + 4./27.*me2*RSe561p - 2./27.*me2*RSe564p
          - 2./27.*mp2*RSp45611e + 4./27.*mp2*RSp561e - 2./27.*mp2*
         RSp564e - 1./10.*m44 - 2./27.*m44*RC1456pe - 2./27.*m44*
         RSe45611p - 2./27.*m44*RSp45611e + 2./27.*m44*RSe564p + 2./27.
         *m44*RSp564e + 1./5.*m14 - 3./5.*m11 - 13./54.*m11*RC1456pe - 
         1./6.*m11*RSe45611p - 1./6.*m11*RSp45611e + 5./27.*m11*RS156pe
          + 7./54.*m11*RS456pe + 1./27.*m11*RSe561p + 1./27.*m11*
         RSp561e + 7./54.*m11*RSe564p + 7./54.*m11*RSp564e );

      M12p6x13RV +=  + Ab(m14,mu2)*AbV(m15,L) * (  - 29./180.*m55 + 2./
         9.*m45 - 29./180.*m44 + 1./10.*m15 + 1./10.*m14 - 3./5.*m11 );

      M12p6x13RV +=  + Ab(m14,mu2)*AbV(m16,L) * (  - 29./180.*m66 + 2./
         9.*m46 - 29./180.*m44 + 1./10.*m16 + 1./10.*m14 - 3./5.*m11 );

      M12p6x13RV +=  + Ab(m14,mu2)*AbV(m44,L) * (  - 1./10.*m44 + 1./30.
         *m44*RS456pe + 1./5.*m14 - 1./15.*m14*RS456pe - 1./10.*m11 + 1.
         /30.*m11*RS456pe );

      M12p6x13RV +=  + Ab(m14,mu2)*AbV(m45,L) * ( 7./180.*m55 - 2./15.*
         m45 - 1./180.*m44 + 1./18.*m15 + 13./90.*m14 - 1./10.*m11 );

      M12p6x13RV +=  + Ab(m14,mu2)*AbV(m46,L) * ( 7./180.*m66 - 2./15.*
         m46 - 1./180.*m44 + 1./18.*m16 + 13./90.*m14 - 1./10.*m11 );

      M12p6x13RV +=  + Ab(m14,mu2)*AbV(mp2,L) * ( 2./45.*mp2*RSp45611e
          - 4./45.*mp2*RSp561e + 2./45.*mp2*RSp564e + 1./30.*m44*
         RSp45611e - 1./45.*m44*RSp561e - 1./90.*m44*RSp564e - 1./15.*
         m14*RSp45611e + 2./15.*m14*RSp561e - 1./15.*m14*RSp564e + 13./
         30.*m11*RSp45611e - 7./15.*m11*RSp561e + 1./30.*m11*RSp564e );

      M12p6x13RV +=  + Ab(m14,mu2)*AbV(me2,L) * ( 2./45.*me2*RSe45611p
          - 4./45.*me2*RSe561p + 2./45.*me2*RSe564p + 1./30.*m44*
         RSe45611p - 1./45.*m44*RSe561p - 1./90.*m44*RSe564p - 1./15.*
         m14*RSe45611p + 2./15.*m14*RSe561p - 1./15.*m14*RSe564p + 13./
         30.*m11*RSe45611p - 7./15.*m11*RSe561p + 1./30.*m11*RSe564p );

      M12p6x13RV +=  + Ab(m14,mu2)*BbV(m11,mp2,L) * (  - 1./9.*m11*m44*
         RSp45611e - 1./9.*m11*m44*RSp561e + 2./9.*m11*m14*RSp45611e - 
         2./9.*m11*m14*RSp561e - 1./9.*pow(m11,2)*RSp45611e - 1./3.*
         pow(m11,2)*RSp561e );

      M12p6x13RV +=  + Ab(m14,mu2)*BbV(m11,me2,L) * (  - 1./9.*m11*m44*
         RSe45611p - 1./9.*m11*m44*RSe561p + 2./9.*m11*m14*RSe45611p - 
         2./9.*m11*m14*RSe561p - 1./9.*pow(m11,2)*RSe45611p - 1./3.*
         pow(m11,2)*RSe561p );

      M12p6x13RV +=  + Ab(m14,mu2)*BbV(m11,L) * ( 1./30.*m44*RD1456ep
          - 1./15.*m14*RD1456ep + 43./90.*m11*RD1456ep - 1./9.*m11*m44*
         RC1456pe - 1./9.*m11*m44*RS156pe + 2./9.*m11*m14*RC1456pe - 2./
         9.*m11*m14*RS156pe - 1./9.*pow(m11,2)*RC1456pe - 1./3.*pow(
         m11,2)*RS156pe );

      M12p6x13RV +=  + Ab(m14,mu2)*BbV(m14,L) * ( 1./36.*pow(m44,2) - 1.
         /9.*m14*m44 + 1./9.*pow(m14,2) + 1./18.*m11*m44 - 1./9.*m11*
         m14 + 1./36.*pow(m11,2) );

      M12p6x13RV +=  + Ab(m14,mu2)*BbV(m15,L) * ( 1./72.*pow(m55,2) + 1.
         /72.*m44*m55 - 1./18.*m15*m55 - 1./36.*m15*m44 + 1./18.*pow(
         m15,2) - 1./36.*m14*m55 + 1./18.*m14*m15 + 1./24.*m11*m55 + 1./
         72.*m11*m44 - 1./12.*m11*m15 - 1./36.*m11*m14 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + Ab(m14,mu2)*BbV(m16,L) * ( 1./72.*pow(m66,2) + 1.
         /72.*m44*m66 - 1./18.*m16*m66 - 1./36.*m16*m44 + 1./18.*pow(
         m16,2) - 1./36.*m14*m66 + 1./18.*m14*m16 + 1./24.*m11*m66 + 1./
         72.*m11*m44 - 1./12.*m11*m16 - 1./36.*m11*m14 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + Ab(m14,mu2)*CbV(m11,L) * (  - 1./9.*m11*m44*
         RD1456ep + 2./9.*m11*m14*RD1456ep - 1./9.*pow(m11,2)*RD1456ep
          );

      M12p6x13RV +=  + Ab(m15,mu2)*AbV(m11,L) * ( 2./45.*RD1456ep + 1./
         30.*m55*RC1456pe - 1./45.*m55*RS146pe - 1./15.*m15*RC1456pe + 
         2./15.*m15*RS146pe + 43./90.*m11*RC1456pe - 5./9.*m11*RS146pe
          );

      M12p6x13RV +=  + Ab(m15,mu2)*AbV(m14,L) * (  - 29./180.*m55 + 2./
         9.*m45 - 29./180.*m44 + 1./10.*m15 + 1./10.*m14 - 3./5.*m11 );

      M12p6x13RV +=  + Ab(m15,mu2)*AbV(m15,L) * (  - 2./27.*RD1456ep - 
         2./27.*me2*RSe45611p + 4./27.*me2*RSe461p - 2./27.*me2*RSe465p
          - 2./27.*mp2*RSp45611e + 4./27.*mp2*RSp461e - 2./27.*mp2*
         RSp465e - 1./10.*m55 - 2./27.*m55*RC1456pe - 2./27.*m55*
         RSe45611p - 2./27.*m55*RSp45611e + 2./27.*m55*RSe465p + 2./27.
         *m55*RSp465e + 1./5.*m15 - 3./5.*m11 - 13./54.*m11*RC1456pe - 
         1./6.*m11*RSe45611p - 1./6.*m11*RSp45611e + 5./27.*m11*RS146pe
          + 7./54.*m11*RS546pe + 1./27.*m11*RSe461p + 1./27.*m11*
         RSp461e + 7./54.*m11*RSe465p + 7./54.*m11*RSp465e );

      M12p6x13RV +=  + Ab(m15,mu2)*AbV(m16,L) * (  - 29./180.*m66 + 2./
         9.*m56 - 29./180.*m55 + 1./10.*m16 + 1./10.*m15 - 3./5.*m11 );

      M12p6x13RV +=  + Ab(m15,mu2)*AbV(m45,L) * (  - 1./180.*m55 - 2./
         15.*m45 + 7./180.*m44 + 13./90.*m15 + 1./18.*m14 - 1./10.*m11
          );

      M12p6x13RV +=  + Ab(m15,mu2)*AbV(m55,L) * (  - 1./10.*m55 + 1./30.
         *m55*RS546pe + 1./5.*m15 - 1./15.*m15*RS546pe - 1./10.*m11 + 1.
         /30.*m11*RS546pe );

      M12p6x13RV +=  + Ab(m15,mu2)*AbV(m56,L) * ( 7./180.*m66 - 2./15.*
         m56 - 1./180.*m55 + 1./18.*m16 + 13./90.*m15 - 1./10.*m11 );

      M12p6x13RV +=  + Ab(m15,mu2)*AbV(mp2,L) * ( 2./45.*mp2*RSp45611e
          - 4./45.*mp2*RSp461e + 2./45.*mp2*RSp465e + 1./30.*m55*
         RSp45611e - 1./45.*m55*RSp461e - 1./90.*m55*RSp465e - 1./15.*
         m15*RSp45611e + 2./15.*m15*RSp461e - 1./15.*m15*RSp465e + 13./
         30.*m11*RSp45611e - 7./15.*m11*RSp461e + 1./30.*m11*RSp465e );

      M12p6x13RV +=  + Ab(m15,mu2)*AbV(me2,L) * ( 2./45.*me2*RSe45611p
          - 4./45.*me2*RSe461p + 2./45.*me2*RSe465p + 1./30.*m55*
         RSe45611p - 1./45.*m55*RSe461p - 1./90.*m55*RSe465p - 1./15.*
         m15*RSe45611p + 2./15.*m15*RSe461p - 1./15.*m15*RSe465p + 13./
         30.*m11*RSe45611p - 7./15.*m11*RSe461p + 1./30.*m11*RSe465p );

      M12p6x13RV +=  + Ab(m15,mu2)*BbV(m11,mp2,L) * (  - 1./9.*m11*m55*
         RSp45611e - 1./9.*m11*m55*RSp461e + 2./9.*m11*m15*RSp45611e - 
         2./9.*m11*m15*RSp461e - 1./9.*pow(m11,2)*RSp45611e - 1./3.*
         pow(m11,2)*RSp461e );

      M12p6x13RV +=  + Ab(m15,mu2)*BbV(m11,me2,L) * (  - 1./9.*m11*m55*
         RSe45611p - 1./9.*m11*m55*RSe461p + 2./9.*m11*m15*RSe45611p - 
         2./9.*m11*m15*RSe461p - 1./9.*pow(m11,2)*RSe45611p - 1./3.*
         pow(m11,2)*RSe461p );

      M12p6x13RV +=  + Ab(m15,mu2)*BbV(m11,L) * ( 1./30.*m55*RD1456ep
          - 1./15.*m15*RD1456ep + 43./90.*m11*RD1456ep - 1./9.*m11*m55*
         RC1456pe - 1./9.*m11*m55*RS146pe + 2./9.*m11*m15*RC1456pe - 2./
         9.*m11*m15*RS146pe - 1./9.*pow(m11,2)*RC1456pe - 1./3.*pow(
         m11,2)*RS146pe );

      M12p6x13RV +=  + Ab(m15,mu2)*BbV(m14,L) * ( 1./72.*m44*m55 + 1./
         72.*pow(m44,2) - 1./36.*m15*m44 - 1./36.*m14*m55 - 1./18.*m14*
         m44 + 1./18.*m14*m15 + 1./18.*pow(m14,2) + 1./72.*m11*m55 + 1./
         24.*m11*m44 - 1./36.*m11*m15 - 1./12.*m11*m14 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + Ab(m15,mu2)*BbV(m15,L) * ( 1./36.*pow(m55,2) - 1.
         /9.*m15*m55 + 1./9.*pow(m15,2) + 1./18.*m11*m55 - 1./9.*m11*
         m15 + 1./36.*pow(m11,2) );

      M12p6x13RV +=  + Ab(m15,mu2)*BbV(m16,L) * ( 1./72.*pow(m66,2) + 1.
         /72.*m55*m66 - 1./18.*m16*m66 - 1./36.*m16*m55 + 1./18.*pow(
         m16,2) - 1./36.*m15*m66 + 1./18.*m15*m16 + 1./24.*m11*m66 + 1./
         72.*m11*m55 - 1./12.*m11*m16 - 1./36.*m11*m15 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + Ab(m15,mu2)*CbV(m11,L) * (  - 1./9.*m11*m55*
         RD1456ep + 2./9.*m11*m15*RD1456ep - 1./9.*pow(m11,2)*RD1456ep
          );

      M12p6x13RV +=  + Ab(m16,mu2)*AbV(m11,L) * ( 2./45.*RD1456ep + 1./
         30.*m66*RC1456pe - 1./45.*m66*RS145pe - 1./15.*m16*RC1456pe + 
         2./15.*m16*RS145pe + 43./90.*m11*RC1456pe - 5./9.*m11*RS145pe
          );

      M12p6x13RV +=  + Ab(m16,mu2)*AbV(m14,L) * (  - 29./180.*m66 + 2./
         9.*m46 - 29./180.*m44 + 1./10.*m16 + 1./10.*m14 - 3./5.*m11 );

      M12p6x13RV +=  + Ab(m16,mu2)*AbV(m15,L) * (  - 29./180.*m66 + 2./
         9.*m56 - 29./180.*m55 + 1./10.*m16 + 1./10.*m15 - 3./5.*m11 );

      M12p6x13RV +=  + Ab(m16,mu2)*AbV(m16,L) * (  - 2./27.*RD1456ep - 
         2./27.*me2*RSe45611p + 4./27.*me2*RSe451p - 2./27.*me2*RSe456p
          - 2./27.*mp2*RSp45611e + 4./27.*mp2*RSp451e - 2./27.*mp2*
         RSp456e - 1./10.*m66 - 2./27.*m66*RC1456pe - 2./27.*m66*
         RSe45611p - 2./27.*m66*RSp45611e + 2./27.*m66*RSe456p + 2./27.
         *m66*RSp456e + 1./5.*m16 - 3./5.*m11 - 13./54.*m11*RC1456pe - 
         1./6.*m11*RSe45611p - 1./6.*m11*RSp45611e + 5./27.*m11*RS145pe
          + 7./54.*m11*RS645pe + 1./27.*m11*RSe451p + 1./27.*m11*
         RSp451e + 7./54.*m11*RSe456p + 7./54.*m11*RSp456e );

      M12p6x13RV +=  + Ab(m16,mu2)*AbV(m46,L) * (  - 1./180.*m66 - 2./
         15.*m46 + 7./180.*m44 + 13./90.*m16 + 1./18.*m14 - 1./10.*m11
          );

      M12p6x13RV +=  + Ab(m16,mu2)*AbV(m56,L) * (  - 1./180.*m66 - 2./
         15.*m56 + 7./180.*m55 + 13./90.*m16 + 1./18.*m15 - 1./10.*m11
          );

      M12p6x13RV +=  + Ab(m16,mu2)*AbV(m66,L) * (  - 1./10.*m66 + 1./30.
         *m66*RS645pe + 1./5.*m16 - 1./15.*m16*RS645pe - 1./10.*m11 + 1.
         /30.*m11*RS645pe );

      M12p6x13RV +=  + Ab(m16,mu2)*AbV(mp2,L) * ( 2./45.*mp2*RSp45611e
          - 4./45.*mp2*RSp451e + 2./45.*mp2*RSp456e + 1./30.*m66*
         RSp45611e - 1./45.*m66*RSp451e - 1./90.*m66*RSp456e - 1./15.*
         m16*RSp45611e + 2./15.*m16*RSp451e - 1./15.*m16*RSp456e + 13./
         30.*m11*RSp45611e - 7./15.*m11*RSp451e + 1./30.*m11*RSp456e );

      M12p6x13RV +=  + Ab(m16,mu2)*AbV(me2,L) * ( 2./45.*me2*RSe45611p
          - 4./45.*me2*RSe451p + 2./45.*me2*RSe456p + 1./30.*m66*
         RSe45611p - 1./45.*m66*RSe451p - 1./90.*m66*RSe456p - 1./15.*
         m16*RSe45611p + 2./15.*m16*RSe451p - 1./15.*m16*RSe456p + 13./
         30.*m11*RSe45611p - 7./15.*m11*RSe451p + 1./30.*m11*RSe456p );

      M12p6x13RV +=  + Ab(m16,mu2)*BbV(m11,mp2,L) * (  - 1./9.*m11*m66*
         RSp45611e - 1./9.*m11*m66*RSp451e + 2./9.*m11*m16*RSp45611e - 
         2./9.*m11*m16*RSp451e - 1./9.*pow(m11,2)*RSp45611e - 1./3.*
         pow(m11,2)*RSp451e );

      M12p6x13RV +=  + Ab(m16,mu2)*BbV(m11,me2,L) * (  - 1./9.*m11*m66*
         RSe45611p - 1./9.*m11*m66*RSe451p + 2./9.*m11*m16*RSe45611p - 
         2./9.*m11*m16*RSe451p - 1./9.*pow(m11,2)*RSe45611p - 1./3.*
         pow(m11,2)*RSe451p );

      M12p6x13RV +=  + Ab(m16,mu2)*BbV(m11,L) * ( 1./30.*m66*RD1456ep
          - 1./15.*m16*RD1456ep + 43./90.*m11*RD1456ep - 1./9.*m11*m66*
         RC1456pe - 1./9.*m11*m66*RS145pe + 2./9.*m11*m16*RC1456pe - 2./
         9.*m11*m16*RS145pe - 1./9.*pow(m11,2)*RC1456pe - 1./3.*pow(
         m11,2)*RS145pe );

      M12p6x13RV +=  + Ab(m16,mu2)*BbV(m14,L) * ( 1./72.*m44*m66 + 1./
         72.*pow(m44,2) - 1./36.*m16*m44 - 1./36.*m14*m66 - 1./18.*m14*
         m44 + 1./18.*m14*m16 + 1./18.*pow(m14,2) + 1./72.*m11*m66 + 1./
         24.*m11*m44 - 1./36.*m11*m16 - 1./12.*m11*m14 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + Ab(m16,mu2)*BbV(m15,L) * ( 1./72.*m55*m66 + 1./
         72.*pow(m55,2) - 1./36.*m16*m55 - 1./36.*m15*m66 - 1./18.*m15*
         m55 + 1./18.*m15*m16 + 1./18.*pow(m15,2) + 1./72.*m11*m66 + 1./
         24.*m11*m55 - 1./36.*m11*m16 - 1./12.*m11*m15 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + Ab(m16,mu2)*BbV(m16,L) * ( 1./36.*pow(m66,2) - 1.
         /9.*m16*m66 + 1./9.*pow(m16,2) + 1./18.*m11*m66 - 1./9.*m11*
         m16 + 1./36.*pow(m11,2) );

      M12p6x13RV +=  + Ab(m16,mu2)*CbV(m11,L) * (  - 1./9.*m11*m66*
         RD1456ep + 2./9.*m11*m16*RD1456ep - 1./9.*pow(m11,2)*RD1456ep
          );

      M12p6x13RV +=  + Ab(m44,mu2)*AbV(m14,L) * (  - 1./10.*m44 + 1./30.
         *m44*RS456pe + 1./5.*m14 - 1./15.*m14*RS456pe - 1./10.*m11 + 1.
         /30.*m11*RS456pe );

      M12p6x13RV +=  + Ab(m44,mu2)*BbV(m11,mp2,L) * ( 2./9.*m11*m44*
         RSp561e*RS156pe - 2./27.*m11*m44*RSp561e*RS456pe*RS156pe );

      M12p6x13RV +=  + Ab(m44,mu2)*BbV(m11,me2,L) * ( 2./9.*m11*m44*
         RSe561p*RS156pe - 2./27.*m11*m44*RSe561p*RS456pe*RS156pe );

      M12p6x13RV +=  + Ab(m44,mu2)*BbV(m11,L) * ( 1./9.*m11*m44*pow(
         RS156pe,2) - 1./27.*m11*m44*RS456pe*pow(RS156pe,2) );

      M12p6x13RV +=  + Ab(m44,mu2)*BbV(m14,L) * ( 1./72.*pow(m44,2) - 1.
         /216.*pow(m44,2)*RS456pe - 1./18.*m14*m44 + 1./54.*m14*m44*
         RS456pe + 1./18.*pow(m14,2) - 1./54.*pow(m14,2)*RS456pe + 1./
         36.*m11*m44 - 1./108.*m11*m44*RS456pe - 1./18.*m11*m14 + 1./54.
         *m11*m14*RS456pe + 1./72.*pow(m11,2) - 1./216.*pow(m11,2)*
         RS456pe );

      M12p6x13RV +=  + Ab(m44,mu2)*BbV(mp2,L) * ( 1./9.*m11*m44*pow(
         RSp561e,2) - 1./27.*m11*m44*pow(RSp561e,2)*RS456pe );

      M12p6x13RV +=  + Ab(m44,mu2)*BbV(me2,mp2,L) * ( 2./9.*m11*m44*
         RSp561e*RSe561p - 2./27.*m11*m44*RSp561e*RSe561p*RS456pe );

      M12p6x13RV +=  + Ab(m44,mu2)*BbV(me2,L) * ( 1./9.*m11*m44*pow(
         RSe561p,2) - 1./27.*m11*m44*pow(RSe561p,2)*RS456pe );

      M12p6x13RV +=  + Ab(m45,mu2)*AbV(m11,L) * (  - 1./27.*m11*pow(
         RS146pe,2) + 2./27.*m11*RS156pe*RS146pe - 1./27.*m11*pow(
         RS156pe,2) );

      M12p6x13RV +=  + Ab(m45,mu2)*AbV(m14,L) * ( 7./180.*m55 - 2./15.*
         m45 - 1./180.*m44 + 1./18.*m15 + 13./90.*m14 - 1./10.*m11 );

      M12p6x13RV +=  + Ab(m45,mu2)*AbV(m15,L) * (  - 1./180.*m55 - 2./
         15.*m45 + 7./180.*m44 + 13./90.*m15 + 1./18.*m14 - 1./10.*m11
          );

      M12p6x13RV +=  + Ab(m45,mu2)*AbV(mp2,L) * ( 2./27.*m11*RSp561e*
         RS146pe - 2./27.*m11*RSp561e*RS156pe - 2./27.*m11*RSp561e*
         RSe561p + 2./27.*m11*RSp561e*RSe461p - 1./27.*m11*pow(
         RSp561e,2) - 2./27.*m11*RSp461e*RS146pe + 2./27.*m11*RSp461e*
         RS156pe + 2./27.*m11*RSp461e*RSe561p - 2./27.*m11*RSp461e*
         RSe461p + 2./27.*m11*RSp461e*RSp561e - 1./27.*m11*pow(
         RSp461e,2) );

      M12p6x13RV +=  + Ab(m45,mu2)*AbV(me2,L) * ( 2./27.*m11*RSe561p*
         RS146pe - 2./27.*m11*RSe561p*RS156pe - 1./27.*m11*pow(
         RSe561p,2) - 2./27.*m11*RSe461p*RS146pe + 2./27.*m11*RSe461p*
         RS156pe + 2./27.*m11*RSe461p*RSe561p - 1./27.*m11*pow(
         RSe461p,2) );

      M12p6x13RV +=  + Ab(m45,mu2)*BbV(m11,mp2,L) * ( 1./27.*m11*m55*
         RSp561e*RS146pe + 1./27.*m11*m55*RSp561e*RS156pe + 1./9.*m11*
         m55*RSp461e*RS146pe + 1./27.*m11*m55*RSp461e*RS156pe + 2./27.*
         m11*m45*RSp561e*RS146pe - 2./27.*m11*m45*RSp561e*RS156pe - 2./
         27.*m11*m45*RSp461e*RS146pe + 2./27.*m11*m45*RSp461e*RS156pe
          + 1./27.*m11*m44*RSp561e*RS146pe + 1./9.*m11*m44*RSp561e*
         RS156pe + 1./27.*m11*m44*RSp461e*RS146pe + 1./27.*m11*m44*
         RSp461e*RS156pe + 2./27.*pow(m11,2)*RSp561e*RS146pe - 2./27.*
         pow(m11,2)*RSp561e*RS156pe - 2./27.*pow(m11,2)*RSp461e*RS146pe
          + 2./27.*pow(m11,2)*RSp461e*RS156pe );

      M12p6x13RV +=  + Ab(m45,mu2)*BbV(m11,me2,L) * ( 1./27.*m11*m55*
         RSe561p*RS146pe + 1./27.*m11*m55*RSe561p*RS156pe + 1./9.*m11*
         m55*RSe461p*RS146pe + 1./27.*m11*m55*RSe461p*RS156pe + 2./27.*
         m11*m45*RSe561p*RS146pe - 2./27.*m11*m45*RSe561p*RS156pe - 2./
         27.*m11*m45*RSe461p*RS146pe + 2./27.*m11*m45*RSe461p*RS156pe
          + 1./27.*m11*m44*RSe561p*RS146pe + 1./9.*m11*m44*RSe561p*
         RS156pe + 1./27.*m11*m44*RSe461p*RS146pe + 1./27.*m11*m44*
         RSe461p*RS156pe + 2./27.*pow(m11,2)*RSe561p*RS146pe - 2./27.*
         pow(m11,2)*RSe561p*RS156pe - 2./27.*pow(m11,2)*RSe461p*RS146pe
          + 2./27.*pow(m11,2)*RSe461p*RS156pe );

      M12p6x13RV +=  + Ab(m45,mu2)*BbV(m11,L) * ( 1./18.*m11*m55*pow(
         RS146pe,2) + 1./27.*m11*m55*RS156pe*RS146pe + 1./54.*m11*m55*
         pow(RS156pe,2) - 1./27.*m11*m45*pow(RS146pe,2) + 2./27.*m11*
         m45*RS156pe*RS146pe - 1./27.*m11*m45*pow(RS156pe,2) + 1./54.*
         m11*m44*pow(RS146pe,2) + 1./27.*m11*m44*RS156pe*RS146pe + 1./
         18.*m11*m44*pow(RS156pe,2) - 1./27.*pow(m11,2)*pow(RS146pe,2)
          + 2./27.*pow(m11,2)*RS156pe*RS146pe - 1./27.*pow(m11,2)*pow(
         RS156pe,2) );

      M12p6x13RV +=  + Ab(m45,mu2)*BbV(m14,L) * ( 1./72.*m44*m55 - 1./
         36.*m44*m45 + 1./36.*pow(m44,2) - 1./36.*m14*m55 + 1./18.*m14*
         m45 - 1./12.*m14*m44 + 1./18.*pow(m14,2) + 1./72.*m11*m55 - 1./
         36.*m11*m45 + 1./24.*m11*m44 - 1./18.*m11*m14 + 1./72.*pow(
         m11,2) );

      M12p6x13RV +=  + Ab(m45,mu2)*BbV(m15,L) * ( 1./36.*pow(m55,2) - 1.
         /36.*m45*m55 + 1./72.*m44*m55 - 1./12.*m15*m55 + 1./18.*m15*
         m45 - 1./36.*m15*m44 + 1./18.*pow(m15,2) + 1./24.*m11*m55 - 1./
         36.*m11*m45 + 1./72.*m11*m44 - 1./18.*m11*m15 + 1./72.*pow(
         m11,2) );

      M12p6x13RV +=  + Ab(m45,mu2)*BbV(mp2,L) * (  - 1./27.*m11*mp2*
         pow(RSp561e,2) + 2./27.*m11*mp2*RSp461e*RSp561e - 1./27.*m11*
         mp2*pow(RSp461e,2) + 1./54.*m11*m55*pow(RSp561e,2) + 1./27.*
         m11*m55*RSp461e*RSp561e + 1./18.*m11*m55*pow(RSp461e,2) - 1./
         27.*m11*m45*pow(RSp561e,2) + 2./27.*m11*m45*RSp461e*RSp561e - 
         1./27.*m11*m45*pow(RSp461e,2) + 1./18.*m11*m44*pow(RSp561e,2)
          + 1./27.*m11*m44*RSp461e*RSp561e + 1./54.*m11*m44*pow(
         RSp461e,2) );

      M12p6x13RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * (  - 2./27.*m11*me2
         *RSp561e*RSe561p + 2./27.*m11*me2*RSp561e*RSe461p + 2./27.*m11
         *me2*RSp461e*RSe561p - 2./27.*m11*me2*RSp461e*RSe461p + 1./27.
         *m11*m55*RSp561e*RSe561p + 1./27.*m11*m55*RSp561e*RSe461p + 1./
         27.*m11*m55*RSp461e*RSe561p + 1./9.*m11*m55*RSp461e*RSe461p - 
         2./27.*m11*m45*RSp561e*RSe561p + 2./27.*m11*m45*RSp561e*
         RSe461p + 2./27.*m11*m45*RSp461e*RSe561p - 2./27.*m11*m45*
         RSp461e*RSe461p + 1./9.*m11*m44*RSp561e*RSe561p + 1./27.*m11*
         m44*RSp561e*RSe461p + 1./27.*m11*m44*RSp461e*RSe561p + 1./27.*
         m11*m44*RSp461e*RSe461p );

      M12p6x13RV +=  + Ab(m45,mu2)*BbV(me2,L) * (  - 1./27.*m11*me2*
         pow(RSe561p,2) + 2./27.*m11*me2*RSe461p*RSe561p - 1./27.*m11*
         me2*pow(RSe461p,2) + 1./54.*m11*m55*pow(RSe561p,2) + 1./27.*
         m11*m55*RSe461p*RSe561p + 1./18.*m11*m55*pow(RSe461p,2) - 1./
         27.*m11*m45*pow(RSe561p,2) + 2./27.*m11*m45*RSe461p*RSe561p - 
         1./27.*m11*m45*pow(RSe461p,2) + 1./18.*m11*m44*pow(RSe561p,2)
          + 1./27.*m11*m44*RSe461p*RSe561p + 1./54.*m11*m44*pow(
         RSe461p,2) );

      M12p6x13RV +=  + Ab(m46,mu2)*AbV(m11,L) * (  - 1./27.*m11*pow(
         RS145pe,2) + 2./27.*m11*RS156pe*RS145pe - 1./27.*m11*pow(
         RS156pe,2) );

      M12p6x13RV +=  + Ab(m46,mu2)*AbV(m14,L) * ( 7./180.*m66 - 2./15.*
         m46 - 1./180.*m44 + 1./18.*m16 + 13./90.*m14 - 1./10.*m11 );

      M12p6x13RV +=  + Ab(m46,mu2)*AbV(m16,L) * (  - 1./180.*m66 - 2./
         15.*m46 + 7./180.*m44 + 13./90.*m16 + 1./18.*m14 - 1./10.*m11
          );

      M12p6x13RV +=  + Ab(m46,mu2)*AbV(mp2,L) * ( 2./27.*m11*RSp561e*
         RS145pe - 2./27.*m11*RSp561e*RS156pe - 2./27.*m11*RSp561e*
         RSe561p + 2./27.*m11*RSp561e*RSe451p - 1./27.*m11*pow(
         RSp561e,2) - 2./27.*m11*RSp451e*RS145pe + 2./27.*m11*RSp451e*
         RS156pe + 2./27.*m11*RSp451e*RSe561p - 2./27.*m11*RSp451e*
         RSe451p + 2./27.*m11*RSp451e*RSp561e - 1./27.*m11*pow(
         RSp451e,2) );

      M12p6x13RV +=  + Ab(m46,mu2)*AbV(me2,L) * ( 2./27.*m11*RSe561p*
         RS145pe - 2./27.*m11*RSe561p*RS156pe - 1./27.*m11*pow(
         RSe561p,2) - 2./27.*m11*RSe451p*RS145pe + 2./27.*m11*RSe451p*
         RS156pe + 2./27.*m11*RSe451p*RSe561p - 1./27.*m11*pow(
         RSe451p,2) );

      M12p6x13RV +=  + Ab(m46,mu2)*BbV(m11,mp2,L) * ( 1./27.*m11*m66*
         RSp561e*RS145pe + 1./27.*m11*m66*RSp561e*RS156pe + 1./9.*m11*
         m66*RSp451e*RS145pe + 1./27.*m11*m66*RSp451e*RS156pe + 2./27.*
         m11*m46*RSp561e*RS145pe - 2./27.*m11*m46*RSp561e*RS156pe - 2./
         27.*m11*m46*RSp451e*RS145pe + 2./27.*m11*m46*RSp451e*RS156pe
          + 1./27.*m11*m44*RSp561e*RS145pe + 1./9.*m11*m44*RSp561e*
         RS156pe + 1./27.*m11*m44*RSp451e*RS145pe + 1./27.*m11*m44*
         RSp451e*RS156pe + 2./27.*pow(m11,2)*RSp561e*RS145pe - 2./27.*
         pow(m11,2)*RSp561e*RS156pe - 2./27.*pow(m11,2)*RSp451e*RS145pe
          + 2./27.*pow(m11,2)*RSp451e*RS156pe );

      M12p6x13RV +=  + Ab(m46,mu2)*BbV(m11,me2,L) * ( 1./27.*m11*m66*
         RSe561p*RS145pe + 1./27.*m11*m66*RSe561p*RS156pe + 1./9.*m11*
         m66*RSe451p*RS145pe + 1./27.*m11*m66*RSe451p*RS156pe + 2./27.*
         m11*m46*RSe561p*RS145pe - 2./27.*m11*m46*RSe561p*RS156pe - 2./
         27.*m11*m46*RSe451p*RS145pe + 2./27.*m11*m46*RSe451p*RS156pe
          + 1./27.*m11*m44*RSe561p*RS145pe + 1./9.*m11*m44*RSe561p*
         RS156pe + 1./27.*m11*m44*RSe451p*RS145pe + 1./27.*m11*m44*
         RSe451p*RS156pe + 2./27.*pow(m11,2)*RSe561p*RS145pe - 2./27.*
         pow(m11,2)*RSe561p*RS156pe - 2./27.*pow(m11,2)*RSe451p*RS145pe
          + 2./27.*pow(m11,2)*RSe451p*RS156pe );

      M12p6x13RV +=  + Ab(m46,mu2)*BbV(m11,L) * ( 1./18.*m11*m66*pow(
         RS145pe,2) + 1./27.*m11*m66*RS156pe*RS145pe + 1./54.*m11*m66*
         pow(RS156pe,2) - 1./27.*m11*m46*pow(RS145pe,2) + 2./27.*m11*
         m46*RS156pe*RS145pe - 1./27.*m11*m46*pow(RS156pe,2) + 1./54.*
         m11*m44*pow(RS145pe,2) + 1./27.*m11*m44*RS156pe*RS145pe + 1./
         18.*m11*m44*pow(RS156pe,2) - 1./27.*pow(m11,2)*pow(RS145pe,2)
          + 2./27.*pow(m11,2)*RS156pe*RS145pe - 1./27.*pow(m11,2)*pow(
         RS156pe,2) );

      M12p6x13RV +=  + Ab(m46,mu2)*BbV(m14,L) * ( 1./72.*m44*m66 - 1./
         36.*m44*m46 + 1./36.*pow(m44,2) - 1./36.*m14*m66 + 1./18.*m14*
         m46 - 1./12.*m14*m44 + 1./18.*pow(m14,2) + 1./72.*m11*m66 - 1./
         36.*m11*m46 + 1./24.*m11*m44 - 1./18.*m11*m14 + 1./72.*pow(
         m11,2) );

      M12p6x13RV +=  + Ab(m46,mu2)*BbV(m16,L) * ( 1./36.*pow(m66,2) - 1.
         /36.*m46*m66 + 1./72.*m44*m66 - 1./12.*m16*m66 + 1./18.*m16*
         m46 - 1./36.*m16*m44 + 1./18.*pow(m16,2) + 1./24.*m11*m66 - 1./
         36.*m11*m46 + 1./72.*m11*m44 - 1./18.*m11*m16 + 1./72.*pow(
         m11,2) );

      M12p6x13RV +=  + Ab(m46,mu2)*BbV(mp2,L) * (  - 1./27.*m11*mp2*
         pow(RSp561e,2) + 2./27.*m11*mp2*RSp451e*RSp561e - 1./27.*m11*
         mp2*pow(RSp451e,2) + 1./54.*m11*m66*pow(RSp561e,2) + 1./27.*
         m11*m66*RSp451e*RSp561e + 1./18.*m11*m66*pow(RSp451e,2) - 1./
         27.*m11*m46*pow(RSp561e,2) + 2./27.*m11*m46*RSp451e*RSp561e - 
         1./27.*m11*m46*pow(RSp451e,2) + 1./18.*m11*m44*pow(RSp561e,2)
          + 1./27.*m11*m44*RSp451e*RSp561e + 1./54.*m11*m44*pow(
         RSp451e,2) );

      M12p6x13RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * (  - 2./27.*m11*me2
         *RSp561e*RSe561p + 2./27.*m11*me2*RSp561e*RSe451p + 2./27.*m11
         *me2*RSp451e*RSe561p - 2./27.*m11*me2*RSp451e*RSe451p + 1./27.
         *m11*m66*RSp561e*RSe561p + 1./27.*m11*m66*RSp561e*RSe451p + 1./
         27.*m11*m66*RSp451e*RSe561p + 1./9.*m11*m66*RSp451e*RSe451p - 
         2./27.*m11*m46*RSp561e*RSe561p + 2./27.*m11*m46*RSp561e*
         RSe451p + 2./27.*m11*m46*RSp451e*RSe561p - 2./27.*m11*m46*
         RSp451e*RSe451p + 1./9.*m11*m44*RSp561e*RSe561p + 1./27.*m11*
         m44*RSp561e*RSe451p + 1./27.*m11*m44*RSp451e*RSe561p + 1./27.*
         m11*m44*RSp451e*RSe451p );

      M12p6x13RV +=  + Ab(m46,mu2)*BbV(me2,L) * (  - 1./27.*m11*me2*
         pow(RSe561p,2) + 2./27.*m11*me2*RSe451p*RSe561p - 1./27.*m11*
         me2*pow(RSe451p,2) + 1./54.*m11*m66*pow(RSe561p,2) + 1./27.*
         m11*m66*RSe451p*RSe561p + 1./18.*m11*m66*pow(RSe451p,2) - 1./
         27.*m11*m46*pow(RSe561p,2) + 2./27.*m11*m46*RSe451p*RSe561p - 
         1./27.*m11*m46*pow(RSe451p,2) + 1./18.*m11*m44*pow(RSe561p,2)
          + 1./27.*m11*m44*RSe451p*RSe561p + 1./54.*m11*m44*pow(
         RSe451p,2) );

      M12p6x13RV +=  + Ab(m55,mu2)*AbV(m15,L) * (  - 1./10.*m55 + 1./30.
         *m55*RS546pe + 1./5.*m15 - 1./15.*m15*RS546pe - 1./10.*m11 + 1.
         /30.*m11*RS546pe );

      M12p6x13RV +=  + Ab(m55,mu2)*BbV(m11,mp2,L) * ( 2./9.*m11*m55*
         RSp461e*RS146pe - 2./27.*m11*m55*RSp461e*RS546pe*RS146pe );

      M12p6x13RV +=  + Ab(m55,mu2)*BbV(m11,me2,L) * ( 2./9.*m11*m55*
         RSe461p*RS146pe - 2./27.*m11*m55*RSe461p*RS546pe*RS146pe );

      M12p6x13RV +=  + Ab(m55,mu2)*BbV(m11,L) * ( 1./9.*m11*m55*pow(
         RS146pe,2) - 1./27.*m11*m55*RS546pe*pow(RS146pe,2) );

      M12p6x13RV +=  + Ab(m55,mu2)*BbV(m15,L) * ( 1./72.*pow(m55,2) - 1.
         /216.*pow(m55,2)*RS546pe - 1./18.*m15*m55 + 1./54.*m15*m55*
         RS546pe + 1./18.*pow(m15,2) - 1./54.*pow(m15,2)*RS546pe + 1./
         36.*m11*m55 - 1./108.*m11*m55*RS546pe - 1./18.*m11*m15 + 1./54.
         *m11*m15*RS546pe + 1./72.*pow(m11,2) - 1./216.*pow(m11,2)*
         RS546pe );

      M12p6x13RV +=  + Ab(m55,mu2)*BbV(mp2,L) * ( 1./9.*m11*m55*pow(
         RSp461e,2) - 1./27.*m11*m55*pow(RSp461e,2)*RS546pe );

      M12p6x13RV +=  + Ab(m55,mu2)*BbV(me2,mp2,L) * ( 2./9.*m11*m55*
         RSp461e*RSe461p - 2./27.*m11*m55*RSp461e*RSe461p*RS546pe );

      M12p6x13RV +=  + Ab(m55,mu2)*BbV(me2,L) * ( 1./9.*m11*m55*pow(
         RSe461p,2) - 1./27.*m11*m55*pow(RSe461p,2)*RS546pe );

      M12p6x13RV +=  + Ab(m56,mu2)*AbV(m11,L) * (  - 1./27.*m11*pow(
         RS145pe,2) + 2./27.*m11*RS146pe*RS145pe - 1./27.*m11*pow(
         RS146pe,2) );

      M12p6x13RV +=  + Ab(m56,mu2)*AbV(m15,L) * ( 7./180.*m66 - 2./15.*
         m56 - 1./180.*m55 + 1./18.*m16 + 13./90.*m15 - 1./10.*m11 );

      M12p6x13RV +=  + Ab(m56,mu2)*AbV(m16,L) * (  - 1./180.*m66 - 2./
         15.*m56 + 7./180.*m55 + 13./90.*m16 + 1./18.*m15 - 1./10.*m11
          );

      M12p6x13RV +=  + Ab(m56,mu2)*AbV(mp2,L) * ( 2./27.*m11*RSp461e*
         RS145pe - 2./27.*m11*RSp461e*RS146pe - 2./27.*m11*RSp461e*
         RSe461p + 2./27.*m11*RSp461e*RSe451p - 1./27.*m11*pow(
         RSp461e,2) - 2./27.*m11*RSp451e*RS145pe + 2./27.*m11*RSp451e*
         RS146pe + 2./27.*m11*RSp451e*RSe461p - 2./27.*m11*RSp451e*
         RSe451p + 2./27.*m11*RSp451e*RSp461e - 1./27.*m11*pow(
         RSp451e,2) );

      M12p6x13RV +=  + Ab(m56,mu2)*AbV(me2,L) * ( 2./27.*m11*RSe461p*
         RS145pe - 2./27.*m11*RSe461p*RS146pe - 1./27.*m11*pow(
         RSe461p,2) - 2./27.*m11*RSe451p*RS145pe + 2./27.*m11*RSe451p*
         RS146pe + 2./27.*m11*RSe451p*RSe461p - 1./27.*m11*pow(
         RSe451p,2) );

      M12p6x13RV +=  + Ab(m56,mu2)*BbV(m11,mp2,L) * ( 1./27.*m11*m66*
         RSp461e*RS145pe + 1./27.*m11*m66*RSp461e*RS146pe + 1./9.*m11*
         m66*RSp451e*RS145pe + 1./27.*m11*m66*RSp451e*RS146pe + 2./27.*
         m11*m56*RSp461e*RS145pe - 2./27.*m11*m56*RSp461e*RS146pe - 2./
         27.*m11*m56*RSp451e*RS145pe + 2./27.*m11*m56*RSp451e*RS146pe
          + 1./27.*m11*m55*RSp461e*RS145pe + 1./9.*m11*m55*RSp461e*
         RS146pe + 1./27.*m11*m55*RSp451e*RS145pe + 1./27.*m11*m55*
         RSp451e*RS146pe + 2./27.*pow(m11,2)*RSp461e*RS145pe - 2./27.*
         pow(m11,2)*RSp461e*RS146pe - 2./27.*pow(m11,2)*RSp451e*RS145pe
          + 2./27.*pow(m11,2)*RSp451e*RS146pe );

      M12p6x13RV +=  + Ab(m56,mu2)*BbV(m11,me2,L) * ( 1./27.*m11*m66*
         RSe461p*RS145pe + 1./27.*m11*m66*RSe461p*RS146pe + 1./9.*m11*
         m66*RSe451p*RS145pe + 1./27.*m11*m66*RSe451p*RS146pe + 2./27.*
         m11*m56*RSe461p*RS145pe - 2./27.*m11*m56*RSe461p*RS146pe - 2./
         27.*m11*m56*RSe451p*RS145pe + 2./27.*m11*m56*RSe451p*RS146pe
          + 1./27.*m11*m55*RSe461p*RS145pe + 1./9.*m11*m55*RSe461p*
         RS146pe + 1./27.*m11*m55*RSe451p*RS145pe + 1./27.*m11*m55*
         RSe451p*RS146pe + 2./27.*pow(m11,2)*RSe461p*RS145pe - 2./27.*
         pow(m11,2)*RSe461p*RS146pe - 2./27.*pow(m11,2)*RSe451p*RS145pe
          + 2./27.*pow(m11,2)*RSe451p*RS146pe );

      M12p6x13RV +=  + Ab(m56,mu2)*BbV(m11,L) * ( 1./18.*m11*m66*pow(
         RS145pe,2) + 1./27.*m11*m66*RS146pe*RS145pe + 1./54.*m11*m66*
         pow(RS146pe,2) - 1./27.*m11*m56*pow(RS145pe,2) + 2./27.*m11*
         m56*RS146pe*RS145pe - 1./27.*m11*m56*pow(RS146pe,2) + 1./54.*
         m11*m55*pow(RS145pe,2) + 1./27.*m11*m55*RS146pe*RS145pe + 1./
         18.*m11*m55*pow(RS146pe,2) - 1./27.*pow(m11,2)*pow(RS145pe,2)
          + 2./27.*pow(m11,2)*RS146pe*RS145pe - 1./27.*pow(m11,2)*pow(
         RS146pe,2) );

      M12p6x13RV +=  + Ab(m56,mu2)*BbV(m15,L) * ( 1./72.*m55*m66 - 1./
         36.*m55*m56 + 1./36.*pow(m55,2) - 1./36.*m15*m66 + 1./18.*m15*
         m56 - 1./12.*m15*m55 + 1./18.*pow(m15,2) + 1./72.*m11*m66 - 1./
         36.*m11*m56 + 1./24.*m11*m55 - 1./18.*m11*m15 + 1./72.*pow(
         m11,2) );

      M12p6x13RV +=  + Ab(m56,mu2)*BbV(m16,L) * ( 1./36.*pow(m66,2) - 1.
         /36.*m56*m66 + 1./72.*m55*m66 - 1./12.*m16*m66 + 1./18.*m16*
         m56 - 1./36.*m16*m55 + 1./18.*pow(m16,2) + 1./24.*m11*m66 - 1./
         36.*m11*m56 + 1./72.*m11*m55 - 1./18.*m11*m16 + 1./72.*pow(
         m11,2) );

      M12p6x13RV +=  + Ab(m56,mu2)*BbV(mp2,L) * (  - 1./27.*m11*mp2*
         pow(RSp461e,2) + 2./27.*m11*mp2*RSp451e*RSp461e - 1./27.*m11*
         mp2*pow(RSp451e,2) + 1./54.*m11*m66*pow(RSp461e,2) + 1./27.*
         m11*m66*RSp451e*RSp461e + 1./18.*m11*m66*pow(RSp451e,2) - 1./
         27.*m11*m56*pow(RSp461e,2) + 2./27.*m11*m56*RSp451e*RSp461e - 
         1./27.*m11*m56*pow(RSp451e,2) + 1./18.*m11*m55*pow(RSp461e,2)
          + 1./27.*m11*m55*RSp451e*RSp461e + 1./54.*m11*m55*pow(
         RSp451e,2) );

      M12p6x13RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * (  - 2./27.*m11*me2
         *RSp461e*RSe461p + 2./27.*m11*me2*RSp461e*RSe451p + 2./27.*m11
         *me2*RSp451e*RSe461p - 2./27.*m11*me2*RSp451e*RSe451p + 1./27.
         *m11*m66*RSp461e*RSe461p + 1./27.*m11*m66*RSp461e*RSe451p + 1./
         27.*m11*m66*RSp451e*RSe461p + 1./9.*m11*m66*RSp451e*RSe451p - 
         2./27.*m11*m56*RSp461e*RSe461p + 2./27.*m11*m56*RSp461e*
         RSe451p + 2./27.*m11*m56*RSp451e*RSe461p - 2./27.*m11*m56*
         RSp451e*RSe451p + 1./9.*m11*m55*RSp461e*RSe461p + 1./27.*m11*
         m55*RSp461e*RSe451p + 1./27.*m11*m55*RSp451e*RSe461p + 1./27.*
         m11*m55*RSp451e*RSe451p );

      M12p6x13RV +=  + Ab(m56,mu2)*BbV(me2,L) * (  - 1./27.*m11*me2*
         pow(RSe461p,2) + 2./27.*m11*me2*RSe451p*RSe461p - 1./27.*m11*
         me2*pow(RSe451p,2) + 1./54.*m11*m66*pow(RSe461p,2) + 1./27.*
         m11*m66*RSe451p*RSe461p + 1./18.*m11*m66*pow(RSe451p,2) - 1./
         27.*m11*m56*pow(RSe461p,2) + 2./27.*m11*m56*RSe451p*RSe461p - 
         1./27.*m11*m56*pow(RSe451p,2) + 1./18.*m11*m55*pow(RSe461p,2)
          + 1./27.*m11*m55*RSe451p*RSe461p + 1./54.*m11*m55*pow(
         RSe451p,2) );

      M12p6x13RV +=  + Ab(m66,mu2)*AbV(m16,L) * (  - 1./10.*m66 + 1./30.
         *m66*RS645pe + 1./5.*m16 - 1./15.*m16*RS645pe - 1./10.*m11 + 1.
         /30.*m11*RS645pe );

      M12p6x13RV +=  + Ab(m66,mu2)*BbV(m11,mp2,L) * ( 2./9.*m11*m66*
         RSp451e*RS145pe - 2./27.*m11*m66*RSp451e*RS645pe*RS145pe );

      M12p6x13RV +=  + Ab(m66,mu2)*BbV(m11,me2,L) * ( 2./9.*m11*m66*
         RSe451p*RS145pe - 2./27.*m11*m66*RSe451p*RS645pe*RS145pe );

      M12p6x13RV +=  + Ab(m66,mu2)*BbV(m11,L) * ( 1./9.*m11*m66*pow(
         RS145pe,2) - 1./27.*m11*m66*RS645pe*pow(RS145pe,2) );

      M12p6x13RV +=  + Ab(m66,mu2)*BbV(m16,L) * ( 1./72.*pow(m66,2) - 1.
         /216.*pow(m66,2)*RS645pe - 1./18.*m16*m66 + 1./54.*m16*m66*
         RS645pe + 1./18.*pow(m16,2) - 1./54.*pow(m16,2)*RS645pe + 1./
         36.*m11*m66 - 1./108.*m11*m66*RS645pe - 1./18.*m11*m16 + 1./54.
         *m11*m16*RS645pe + 1./72.*pow(m11,2) - 1./216.*pow(m11,2)*
         RS645pe );

      M12p6x13RV +=  + Ab(m66,mu2)*BbV(mp2,L) * ( 1./9.*m11*m66*pow(
         RSp451e,2) - 1./27.*m11*m66*pow(RSp451e,2)*RS645pe );

      M12p6x13RV +=  + Ab(m66,mu2)*BbV(me2,mp2,L) * ( 2./9.*m11*m66*
         RSp451e*RSe451p - 2./27.*m11*m66*RSp451e*RSe451p*RS645pe );

      M12p6x13RV +=  + Ab(m66,mu2)*BbV(me2,L) * ( 1./9.*m11*m66*pow(
         RSe451p,2) - 1./27.*m11*m66*pow(RSe451p,2)*RS645pe );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(m11,L) * ( 1./9.*m11*RSp45611e*
         RC1456pe - 2./9.*pow(m11,2)*RN1p*RSp45611e*RC1456pe );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(m14,L) * ( 2./45.*mp2*RSp45611e
          - 4./45.*mp2*RSp561e + 2./45.*mp2*RSp564e + 1./30.*m44*
         RSp45611e - 1./45.*m44*RSp561e - 1./90.*m44*RSp564e - 1./15.*
         m14*RSp45611e + 2./15.*m14*RSp561e - 1./15.*m14*RSp564e + 13./
         30.*m11*RSp45611e - 7./15.*m11*RSp561e + 1./30.*m11*RSp564e + 
         1./9.*m11*m44*RN1p*RSp45611e + 1./9.*m11*m44*RN1p*RSp561e - 2./
         9.*m11*m14*RN1p*RSp45611e + 2./9.*m11*m14*RN1p*RSp561e + 1./9.
         *pow(m11,2)*RN1p*RSp45611e + 1./3.*pow(m11,2)*RN1p*RSp561e );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(m15,L) * ( 2./45.*mp2*RSp45611e
          - 4./45.*mp2*RSp461e + 2./45.*mp2*RSp465e + 1./30.*m55*
         RSp45611e - 1./45.*m55*RSp461e - 1./90.*m55*RSp465e - 1./15.*
         m15*RSp45611e + 2./15.*m15*RSp461e - 1./15.*m15*RSp465e + 13./
         30.*m11*RSp45611e - 7./15.*m11*RSp461e + 1./30.*m11*RSp465e + 
         1./9.*m11*m55*RN1p*RSp45611e + 1./9.*m11*m55*RN1p*RSp461e - 2./
         9.*m11*m15*RN1p*RSp45611e + 2./9.*m11*m15*RN1p*RSp461e + 1./9.
         *pow(m11,2)*RN1p*RSp45611e + 1./3.*pow(m11,2)*RN1p*RSp461e );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(m16,L) * ( 2./45.*mp2*RSp45611e
          - 4./45.*mp2*RSp451e + 2./45.*mp2*RSp456e + 1./30.*m66*
         RSp45611e - 1./45.*m66*RSp451e - 1./90.*m66*RSp456e - 1./15.*
         m16*RSp45611e + 2./15.*m16*RSp451e - 1./15.*m16*RSp456e + 13./
         30.*m11*RSp45611e - 7./15.*m11*RSp451e + 1./30.*m11*RSp456e + 
         1./9.*m11*m66*RN1p*RSp45611e + 1./9.*m11*m66*RN1p*RSp451e - 2./
         9.*m11*m16*RN1p*RSp45611e + 2./9.*m11*m16*RN1p*RSp451e + 1./9.
         *pow(m11,2)*RN1p*RSp45611e + 1./3.*pow(m11,2)*RN1p*RSp451e );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(m44,L) * (  - 2./9.*m11*m44*RN1p
         *RSp561e*RS156pe + 2./27.*m11*m44*RN1p*RSp561e*RS456pe*RS156pe
          + 2./9.*RN(mp2,me2)*m11*m44*RSp561e*RSe561p - 2./27.*RN(mp2,
         me2)*m11*m44*RSp561e*RSe561p*RS456pe );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 2./27.*m11*RSp561e*
         RS146pe - 2./27.*m11*RSp561e*RS156pe - 2./27.*m11*RSp561e*
         RSe561p + 2./27.*m11*RSp561e*RSe461p - 1./27.*m11*pow(
         RSp561e,2) - 2./27.*m11*RSp461e*RS146pe + 2./27.*m11*RSp461e*
         RS156pe + 2./27.*m11*RSp461e*RSe561p - 2./27.*m11*RSp461e*
         RSe461p + 2./27.*m11*RSp461e*RSp561e - 1./27.*m11*pow(
         RSp461e,2) - 1./27.*m11*m55*RN1p*RSp561e*RS146pe - 1./27.*m11*
         m55*RN1p*RSp561e*RS156pe - 1./9.*m11*m55*RN1p*RSp461e*RS146pe
          - 1./27.*m11*m55*RN1p*RSp461e*RS156pe - 2./27.*m11*m45*RN1p*
         RSp561e*RS146pe + 2./27.*m11*m45*RN1p*RSp561e*RS156pe + 2./27.
         *m11*m45*RN1p*RSp461e*RS146pe - 2./27.*m11*m45*RN1p*RSp461e*
         RS156pe - 1./27.*m11*m44*RN1p*RSp561e*RS146pe - 1./9.*m11*m44*
         RN1p*RSp561e*RS156pe - 1./27.*m11*m44*RN1p*RSp461e*RS146pe - 1.
         /27.*m11*m44*RN1p*RSp461e*RS156pe - 2./27.*pow(m11,2)*RN1p*
         RSp561e*RS146pe + 2./27.*pow(m11,2)*RN1p*RSp561e*RS156pe + 2./
         27.*pow(m11,2)*RN1p*RSp461e*RS146pe );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 2./27.*pow(m11,2)*
         RN1p*RSp461e*RS156pe - 2./27.*RN(mp2,me2)*m11*me2*RSp561e*
         RSe561p + 2./27.*RN(mp2,me2)*m11*me2*RSp561e*RSe461p + 2./27.*
         RN(mp2,me2)*m11*me2*RSp461e*RSe561p - 2./27.*RN(mp2,me2)*m11*
         me2*RSp461e*RSe461p + 1./27.*RN(mp2,me2)*m11*m55*RSp561e*
         RSe561p + 1./27.*RN(mp2,me2)*m11*m55*RSp561e*RSe461p + 1./27.*
         RN(mp2,me2)*m11*m55*RSp461e*RSe561p + 1./9.*RN(mp2,me2)*m11*
         m55*RSp461e*RSe461p - 2./27.*RN(mp2,me2)*m11*m45*RSp561e*
         RSe561p + 2./27.*RN(mp2,me2)*m11*m45*RSp561e*RSe461p + 2./27.*
         RN(mp2,me2)*m11*m45*RSp461e*RSe561p - 2./27.*RN(mp2,me2)*m11*
         m45*RSp461e*RSe461p + 1./9.*RN(mp2,me2)*m11*m44*RSp561e*
         RSe561p + 1./27.*RN(mp2,me2)*m11*m44*RSp561e*RSe461p + 1./27.*
         RN(mp2,me2)*m11*m44*RSp461e*RSe561p + 1./27.*RN(mp2,me2)*m11*
         m44*RSp461e*RSe461p );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 2./27.*m11*RSp561e*
         RS145pe - 2./27.*m11*RSp561e*RS156pe - 2./27.*m11*RSp561e*
         RSe561p + 2./27.*m11*RSp561e*RSe451p - 1./27.*m11*pow(
         RSp561e,2) - 2./27.*m11*RSp451e*RS145pe + 2./27.*m11*RSp451e*
         RS156pe + 2./27.*m11*RSp451e*RSe561p - 2./27.*m11*RSp451e*
         RSe451p + 2./27.*m11*RSp451e*RSp561e - 1./27.*m11*pow(
         RSp451e,2) - 1./27.*m11*m66*RN1p*RSp561e*RS145pe - 1./27.*m11*
         m66*RN1p*RSp561e*RS156pe - 1./9.*m11*m66*RN1p*RSp451e*RS145pe
          - 1./27.*m11*m66*RN1p*RSp451e*RS156pe - 2./27.*m11*m46*RN1p*
         RSp561e*RS145pe + 2./27.*m11*m46*RN1p*RSp561e*RS156pe + 2./27.
         *m11*m46*RN1p*RSp451e*RS145pe - 2./27.*m11*m46*RN1p*RSp451e*
         RS156pe - 1./27.*m11*m44*RN1p*RSp561e*RS145pe - 1./9.*m11*m44*
         RN1p*RSp561e*RS156pe - 1./27.*m11*m44*RN1p*RSp451e*RS145pe - 1.
         /27.*m11*m44*RN1p*RSp451e*RS156pe - 2./27.*pow(m11,2)*RN1p*
         RSp561e*RS145pe + 2./27.*pow(m11,2)*RN1p*RSp561e*RS156pe + 2./
         27.*pow(m11,2)*RN1p*RSp451e*RS145pe );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 2./27.*pow(m11,2)*
         RN1p*RSp451e*RS156pe - 2./27.*RN(mp2,me2)*m11*me2*RSp561e*
         RSe561p + 2./27.*RN(mp2,me2)*m11*me2*RSp561e*RSe451p + 2./27.*
         RN(mp2,me2)*m11*me2*RSp451e*RSe561p - 2./27.*RN(mp2,me2)*m11*
         me2*RSp451e*RSe451p + 1./27.*RN(mp2,me2)*m11*m66*RSp561e*
         RSe561p + 1./27.*RN(mp2,me2)*m11*m66*RSp561e*RSe451p + 1./27.*
         RN(mp2,me2)*m11*m66*RSp451e*RSe561p + 1./9.*RN(mp2,me2)*m11*
         m66*RSp451e*RSe451p - 2./27.*RN(mp2,me2)*m11*m46*RSp561e*
         RSe561p + 2./27.*RN(mp2,me2)*m11*m46*RSp561e*RSe451p + 2./27.*
         RN(mp2,me2)*m11*m46*RSp451e*RSe561p - 2./27.*RN(mp2,me2)*m11*
         m46*RSp451e*RSe451p + 1./9.*RN(mp2,me2)*m11*m44*RSp561e*
         RSe561p + 1./27.*RN(mp2,me2)*m11*m44*RSp561e*RSe451p + 1./27.*
         RN(mp2,me2)*m11*m44*RSp451e*RSe561p + 1./27.*RN(mp2,me2)*m11*
         m44*RSp451e*RSe451p );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(m55,L) * (  - 2./9.*m11*m55*RN1p
         *RSp461e*RS146pe + 2./27.*m11*m55*RN1p*RSp461e*RS546pe*RS146pe
          + 2./9.*RN(mp2,me2)*m11*m55*RSp461e*RSe461p - 2./27.*RN(mp2,
         me2)*m11*m55*RSp461e*RSe461p*RS546pe );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 2./27.*m11*RSp461e*
         RS145pe - 2./27.*m11*RSp461e*RS146pe - 2./27.*m11*RSp461e*
         RSe461p + 2./27.*m11*RSp461e*RSe451p - 1./27.*m11*pow(
         RSp461e,2) - 2./27.*m11*RSp451e*RS145pe + 2./27.*m11*RSp451e*
         RS146pe + 2./27.*m11*RSp451e*RSe461p - 2./27.*m11*RSp451e*
         RSe451p + 2./27.*m11*RSp451e*RSp461e - 1./27.*m11*pow(
         RSp451e,2) - 1./27.*m11*m66*RN1p*RSp461e*RS145pe - 1./27.*m11*
         m66*RN1p*RSp461e*RS146pe - 1./9.*m11*m66*RN1p*RSp451e*RS145pe
          - 1./27.*m11*m66*RN1p*RSp451e*RS146pe - 2./27.*m11*m56*RN1p*
         RSp461e*RS145pe + 2./27.*m11*m56*RN1p*RSp461e*RS146pe + 2./27.
         *m11*m56*RN1p*RSp451e*RS145pe - 2./27.*m11*m56*RN1p*RSp451e*
         RS146pe - 1./27.*m11*m55*RN1p*RSp461e*RS145pe - 1./9.*m11*m55*
         RN1p*RSp461e*RS146pe - 1./27.*m11*m55*RN1p*RSp451e*RS145pe - 1.
         /27.*m11*m55*RN1p*RSp451e*RS146pe - 2./27.*pow(m11,2)*RN1p*
         RSp461e*RS145pe + 2./27.*pow(m11,2)*RN1p*RSp461e*RS146pe + 2./
         27.*pow(m11,2)*RN1p*RSp451e*RS145pe );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 2./27.*pow(m11,2)*
         RN1p*RSp451e*RS146pe - 2./27.*RN(mp2,me2)*m11*me2*RSp461e*
         RSe461p + 2./27.*RN(mp2,me2)*m11*me2*RSp461e*RSe451p + 2./27.*
         RN(mp2,me2)*m11*me2*RSp451e*RSe461p - 2./27.*RN(mp2,me2)*m11*
         me2*RSp451e*RSe451p + 1./27.*RN(mp2,me2)*m11*m66*RSp461e*
         RSe461p + 1./27.*RN(mp2,me2)*m11*m66*RSp461e*RSe451p + 1./27.*
         RN(mp2,me2)*m11*m66*RSp451e*RSe461p + 1./9.*RN(mp2,me2)*m11*
         m66*RSp451e*RSe451p - 2./27.*RN(mp2,me2)*m11*m56*RSp461e*
         RSe461p + 2./27.*RN(mp2,me2)*m11*m56*RSp461e*RSe451p + 2./27.*
         RN(mp2,me2)*m11*m56*RSp451e*RSe461p - 2./27.*RN(mp2,me2)*m11*
         m56*RSp451e*RSe451p + 1./9.*RN(mp2,me2)*m11*m55*RSp461e*
         RSe461p + 1./27.*RN(mp2,me2)*m11*m55*RSp461e*RSe451p + 1./27.*
         RN(mp2,me2)*m11*m55*RSp451e*RSe461p + 1./27.*RN(mp2,me2)*m11*
         m55*RSp451e*RSe451p );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(m66,L) * (  - 2./9.*m11*m66*RN1p
         *RSp451e*RS145pe + 2./27.*m11*m66*RN1p*RSp451e*RS645pe*RS145pe
          + 2./9.*RN(mp2,me2)*m11*m66*RSp451e*RSe451p - 2./27.*RN(mp2,
         me2)*m11*m66*RSp451e*RSe451p*RS645pe );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(mp2,L) * ( 1./9.*m11*pow(
         RSp45611e,2) + 2./27.*m11*m66*RN1p*RSp456e*RSp451e*RS145pe + 2.
         /27.*m11*m55*RN1p*RSp465e*RSp461e*RS146pe + 2./27.*m11*m44*
         RN1p*RSp564e*RSp561e*RS156pe - 2./9.*pow(m11,2)*RN1p*pow(
         RSp45611e,2) - 2./27.*RN(mp2,me2)*m11*m66*RSp456e*RSp451e*
         RSe451p - 2./27.*RN(mp2,me2)*m11*m55*RSp465e*RSp461e*RSe461p
          - 2./27.*RN(mp2,me2)*m11*m44*RSp564e*RSp561e*RSe561p );

      M12p6x13RV +=  + Ab(mp2,mu2)*AbV(me2,L) * ( 1./9.*m11*RSp45611e*
         RSe45611p + 2./27.*m11*m66*RN1p*RSe456p*RSp451e*RS145pe + 2./
         27.*m11*m55*RN1p*RSe465p*RSp461e*RS146pe + 2./27.*m11*m44*RN1p
         *RSe564p*RSp561e*RS156pe - 2./9.*pow(m11,2)*RN1p*RSp45611e*
         RSe45611p - 2./27.*RN(mp2,me2)*m11*m66*RSe456p*RSp451e*RSe451p
          - 2./27.*RN(mp2,me2)*m11*m55*RSe465p*RSp461e*RSe461p - 2./27.
         *RN(mp2,me2)*m11*m44*RSe564p*RSp561e*RSe561p );

      M12p6x13RV +=  + Ab(mp2,mu2)*BbV(m11,mp2,L) * (  - 2./27.*m11*m66
         *RSp456e*RSp451e*RS145pe - 2./27.*m11*m55*RSp465e*RSp461e*
         RS146pe - 2./27.*m11*m44*RSp564e*RSp561e*RS156pe + 2./9.*pow(
         m11,2)*pow(RSp45611e,2) );

      M12p6x13RV +=  + Ab(mp2,mu2)*BbV(m11,me2,L) * (  - 2./27.*m11*m66
         *RSp456e*RSe451p*RS145pe - 2./27.*m11*m55*RSp465e*RSe461p*
         RS146pe - 2./27.*m11*m44*RSp564e*RSe561p*RS156pe + 2./9.*pow(
         m11,2)*RSp45611e*RSe45611p );

      M12p6x13RV +=  + Ab(mp2,mu2)*BbV(m11,L) * ( 1./9.*m11*RSp45611e*
         RD1456ep - 1./27.*m11*m66*RSp456e*pow(RS145pe,2) - 1./27.*m11*
         m55*RSp465e*pow(RS146pe,2) - 1./27.*m11*m44*RSp564e*pow(
         RS156pe,2) + 2./9.*pow(m11,2)*RSp45611e*RC1456pe - 2./9.*pow(
         m11,2)*RN1p*RSp45611e*RD1456ep );

      M12p6x13RV +=  + Ab(mp2,mu2)*BbV(m14,L) * ( 1./108.*m44*mp2*
         RSp45611e - 1./54.*m44*mp2*RSp561e + 1./108.*m44*mp2*RSp564e
          - 1./216.*pow(m44,2)*RSp45611e - 1./108.*pow(m44,2)*RSp561e
          - 1./72.*pow(m44,2)*RSp564e - 1./54.*m14*mp2*RSp45611e + 1./
         27.*m14*mp2*RSp561e - 1./54.*m14*mp2*RSp564e + 1./54.*m14*m44*
         RSp45611e + 1./27.*m14*m44*RSp564e - 1./54.*pow(m14,2)*
         RSp45611e + 1./27.*pow(m14,2)*RSp561e - 1./54.*pow(m14,2)*
         RSp564e + 1./108.*m11*mp2*RSp45611e - 1./54.*m11*mp2*RSp561e
          + 1./108.*m11*mp2*RSp564e - 1./54.*m11*m44*RSp45611e - 1./54.
         *m11*m44*RSp561e - 1./54.*m11*m44*RSp564e + 1./27.*m11*m14*
         RSp45611e + 1./54.*m11*m14*RSp564e - 1./72.*pow(m11,2)*
         RSp45611e - 1./108.*pow(m11,2)*RSp561e - 1./216.*pow(m11,2)*
         RSp564e );

      M12p6x13RV +=  + Ab(mp2,mu2)*BbV(m15,L) * ( 1./108.*m55*mp2*
         RSp45611e - 1./54.*m55*mp2*RSp461e + 1./108.*m55*mp2*RSp465e
          - 1./216.*pow(m55,2)*RSp45611e - 1./108.*pow(m55,2)*RSp461e
          - 1./72.*pow(m55,2)*RSp465e - 1./54.*m15*mp2*RSp45611e + 1./
         27.*m15*mp2*RSp461e - 1./54.*m15*mp2*RSp465e + 1./54.*m15*m55*
         RSp45611e + 1./27.*m15*m55*RSp465e - 1./54.*pow(m15,2)*
         RSp45611e + 1./27.*pow(m15,2)*RSp461e - 1./54.*pow(m15,2)*
         RSp465e + 1./108.*m11*mp2*RSp45611e - 1./54.*m11*mp2*RSp461e
          + 1./108.*m11*mp2*RSp465e - 1./54.*m11*m55*RSp45611e - 1./54.
         *m11*m55*RSp461e - 1./54.*m11*m55*RSp465e + 1./27.*m11*m15*
         RSp45611e + 1./54.*m11*m15*RSp465e - 1./72.*pow(m11,2)*
         RSp45611e - 1./108.*pow(m11,2)*RSp461e - 1./216.*pow(m11,2)*
         RSp465e );

      M12p6x13RV +=  + Ab(mp2,mu2)*BbV(m16,L) * ( 1./108.*m66*mp2*
         RSp45611e - 1./54.*m66*mp2*RSp451e + 1./108.*m66*mp2*RSp456e
          - 1./216.*pow(m66,2)*RSp45611e - 1./108.*pow(m66,2)*RSp451e
          - 1./72.*pow(m66,2)*RSp456e - 1./54.*m16*mp2*RSp45611e + 1./
         27.*m16*mp2*RSp451e - 1./54.*m16*mp2*RSp456e + 1./54.*m16*m66*
         RSp45611e + 1./27.*m16*m66*RSp456e - 1./54.*pow(m16,2)*
         RSp45611e + 1./27.*pow(m16,2)*RSp451e - 1./54.*pow(m16,2)*
         RSp456e + 1./108.*m11*mp2*RSp45611e - 1./54.*m11*mp2*RSp451e
          + 1./108.*m11*mp2*RSp456e - 1./54.*m11*m66*RSp45611e - 1./54.
         *m11*m66*RSp451e - 1./54.*m11*m66*RSp456e + 1./27.*m11*m16*
         RSp45611e + 1./54.*m11*m16*RSp456e - 1./72.*pow(m11,2)*
         RSp45611e - 1./108.*pow(m11,2)*RSp451e - 1./216.*pow(m11,2)*
         RSp456e );

      M12p6x13RV +=  + Ab(mp2,mu2)*BbV(mp2,L) * (  - 1./27.*m11*m66*
         RSp456e*pow(RSp451e,2) - 1./27.*m11*m55*RSp465e*pow(RSp461e,2)
          - 1./27.*m11*m44*RSp564e*pow(RSp561e,2) );

      M12p6x13RV +=  + Ab(mp2,mu2)*BbV(me2,mp2,L) * (  - 2./27.*m11*m66
         *RSp456e*RSp451e*RSe451p - 2./27.*m11*m55*RSp465e*RSp461e*
         RSe461p - 2./27.*m11*m44*RSp564e*RSp561e*RSe561p );

      M12p6x13RV +=  + Ab(mp2,mu2)*BbV(me2,L) * (  - 1./27.*m11*m66*
         RSp456e*pow(RSe451p,2) - 1./27.*m11*m55*RSp465e*pow(RSe461p,2)
          - 1./27.*m11*m44*RSp564e*pow(RSe561p,2) );

      M12p6x13RV +=  + Ab(mp2,mu2)*CbV(m11,L) * ( 2./9.*pow(m11,2)*
         RSp45611e*RD1456ep );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(m11,L) * ( 1./9.*m11*RSe45611p*
         RC1456pe - 2./9.*pow(m11,2)*RN1e*RSe45611p*RC1456pe );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(m14,L) * ( 2./45.*me2*RSe45611p
          - 4./45.*me2*RSe561p + 2./45.*me2*RSe564p + 1./30.*m44*
         RSe45611p - 1./45.*m44*RSe561p - 1./90.*m44*RSe564p - 1./15.*
         m14*RSe45611p + 2./15.*m14*RSe561p - 1./15.*m14*RSe564p + 13./
         30.*m11*RSe45611p - 7./15.*m11*RSe561p + 1./30.*m11*RSe564p + 
         1./9.*m11*m44*RN1e*RSe45611p + 1./9.*m11*m44*RN1e*RSe561p - 2./
         9.*m11*m14*RN1e*RSe45611p + 2./9.*m11*m14*RN1e*RSe561p + 1./9.
         *pow(m11,2)*RN1e*RSe45611p + 1./3.*pow(m11,2)*RN1e*RSe561p );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(m15,L) * ( 2./45.*me2*RSe45611p
          - 4./45.*me2*RSe461p + 2./45.*me2*RSe465p + 1./30.*m55*
         RSe45611p - 1./45.*m55*RSe461p - 1./90.*m55*RSe465p - 1./15.*
         m15*RSe45611p + 2./15.*m15*RSe461p - 1./15.*m15*RSe465p + 13./
         30.*m11*RSe45611p - 7./15.*m11*RSe461p + 1./30.*m11*RSe465p + 
         1./9.*m11*m55*RN1e*RSe45611p + 1./9.*m11*m55*RN1e*RSe461p - 2./
         9.*m11*m15*RN1e*RSe45611p + 2./9.*m11*m15*RN1e*RSe461p + 1./9.
         *pow(m11,2)*RN1e*RSe45611p + 1./3.*pow(m11,2)*RN1e*RSe461p );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(m16,L) * ( 2./45.*me2*RSe45611p
          - 4./45.*me2*RSe451p + 2./45.*me2*RSe456p + 1./30.*m66*
         RSe45611p - 1./45.*m66*RSe451p - 1./90.*m66*RSe456p - 1./15.*
         m16*RSe45611p + 2./15.*m16*RSe451p - 1./15.*m16*RSe456p + 13./
         30.*m11*RSe45611p - 7./15.*m11*RSe451p + 1./30.*m11*RSe456p + 
         1./9.*m11*m66*RN1e*RSe45611p + 1./9.*m11*m66*RN1e*RSe451p - 2./
         9.*m11*m16*RN1e*RSe45611p + 2./9.*m11*m16*RN1e*RSe451p + 1./9.
         *pow(m11,2)*RN1e*RSe45611p + 1./3.*pow(m11,2)*RN1e*RSe451p );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(m44,L) * (  - 2./9.*m11*m44*RN1e
         *RSe561p*RS156pe + 2./27.*m11*m44*RN1e*RSe561p*RS456pe*RS156pe
          - 2./9.*RN(mp2,me2)*m11*m44*RSp561e*RSe561p + 2./27.*RN(mp2,
         me2)*m11*m44*RSp561e*RSe561p*RS456pe );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 2./27.*m11*RSe561p*
         RS146pe - 2./27.*m11*RSe561p*RS156pe - 1./27.*m11*pow(
         RSe561p,2) - 2./27.*m11*RSe461p*RS146pe + 2./27.*m11*RSe461p*
         RS156pe + 2./27.*m11*RSe461p*RSe561p - 1./27.*m11*pow(
         RSe461p,2) - 1./27.*m11*m55*RN1e*RSe561p*RS146pe - 1./27.*m11*
         m55*RN1e*RSe561p*RS156pe - 1./9.*m11*m55*RN1e*RSe461p*RS146pe
          - 1./27.*m11*m55*RN1e*RSe461p*RS156pe - 2./27.*m11*m45*RN1e*
         RSe561p*RS146pe + 2./27.*m11*m45*RN1e*RSe561p*RS156pe + 2./27.
         *m11*m45*RN1e*RSe461p*RS146pe - 2./27.*m11*m45*RN1e*RSe461p*
         RS156pe - 1./27.*m11*m44*RN1e*RSe561p*RS146pe - 1./9.*m11*m44*
         RN1e*RSe561p*RS156pe - 1./27.*m11*m44*RN1e*RSe461p*RS146pe - 1.
         /27.*m11*m44*RN1e*RSe461p*RS156pe - 2./27.*pow(m11,2)*RN1e*
         RSe561p*RS146pe + 2./27.*pow(m11,2)*RN1e*RSe561p*RS156pe + 2./
         27.*pow(m11,2)*RN1e*RSe461p*RS146pe - 2./27.*pow(m11,2)*RN1e*
         RSe461p*RS156pe + 2./27.*RN(mp2,me2)*m11*me2*RSp561e*RSe561p
          - 2./27.*RN(mp2,me2)*m11*me2*RSp561e*RSe461p );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 2./27.*RN(mp2,me2)
         *m11*me2*RSp461e*RSe561p + 2./27.*RN(mp2,me2)*m11*me2*RSp461e*
         RSe461p - 1./27.*RN(mp2,me2)*m11*m55*RSp561e*RSe561p - 1./27.*
         RN(mp2,me2)*m11*m55*RSp561e*RSe461p - 1./27.*RN(mp2,me2)*m11*
         m55*RSp461e*RSe561p - 1./9.*RN(mp2,me2)*m11*m55*RSp461e*
         RSe461p + 2./27.*RN(mp2,me2)*m11*m45*RSp561e*RSe561p - 2./27.*
         RN(mp2,me2)*m11*m45*RSp561e*RSe461p - 2./27.*RN(mp2,me2)*m11*
         m45*RSp461e*RSe561p + 2./27.*RN(mp2,me2)*m11*m45*RSp461e*
         RSe461p - 1./9.*RN(mp2,me2)*m11*m44*RSp561e*RSe561p - 1./27.*
         RN(mp2,me2)*m11*m44*RSp561e*RSe461p - 1./27.*RN(mp2,me2)*m11*
         m44*RSp461e*RSe561p - 1./27.*RN(mp2,me2)*m11*m44*RSp461e*
         RSe461p );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 2./27.*m11*RSe561p*
         RS145pe - 2./27.*m11*RSe561p*RS156pe - 1./27.*m11*pow(
         RSe561p,2) - 2./27.*m11*RSe451p*RS145pe + 2./27.*m11*RSe451p*
         RS156pe + 2./27.*m11*RSe451p*RSe561p - 1./27.*m11*pow(
         RSe451p,2) - 1./27.*m11*m66*RN1e*RSe561p*RS145pe - 1./27.*m11*
         m66*RN1e*RSe561p*RS156pe - 1./9.*m11*m66*RN1e*RSe451p*RS145pe
          - 1./27.*m11*m66*RN1e*RSe451p*RS156pe - 2./27.*m11*m46*RN1e*
         RSe561p*RS145pe + 2./27.*m11*m46*RN1e*RSe561p*RS156pe + 2./27.
         *m11*m46*RN1e*RSe451p*RS145pe - 2./27.*m11*m46*RN1e*RSe451p*
         RS156pe - 1./27.*m11*m44*RN1e*RSe561p*RS145pe - 1./9.*m11*m44*
         RN1e*RSe561p*RS156pe - 1./27.*m11*m44*RN1e*RSe451p*RS145pe - 1.
         /27.*m11*m44*RN1e*RSe451p*RS156pe - 2./27.*pow(m11,2)*RN1e*
         RSe561p*RS145pe + 2./27.*pow(m11,2)*RN1e*RSe561p*RS156pe + 2./
         27.*pow(m11,2)*RN1e*RSe451p*RS145pe - 2./27.*pow(m11,2)*RN1e*
         RSe451p*RS156pe + 2./27.*RN(mp2,me2)*m11*me2*RSp561e*RSe561p
          - 2./27.*RN(mp2,me2)*m11*me2*RSp561e*RSe451p );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 2./27.*RN(mp2,me2)
         *m11*me2*RSp451e*RSe561p + 2./27.*RN(mp2,me2)*m11*me2*RSp451e*
         RSe451p - 1./27.*RN(mp2,me2)*m11*m66*RSp561e*RSe561p - 1./27.*
         RN(mp2,me2)*m11*m66*RSp561e*RSe451p - 1./27.*RN(mp2,me2)*m11*
         m66*RSp451e*RSe561p - 1./9.*RN(mp2,me2)*m11*m66*RSp451e*
         RSe451p + 2./27.*RN(mp2,me2)*m11*m46*RSp561e*RSe561p - 2./27.*
         RN(mp2,me2)*m11*m46*RSp561e*RSe451p - 2./27.*RN(mp2,me2)*m11*
         m46*RSp451e*RSe561p + 2./27.*RN(mp2,me2)*m11*m46*RSp451e*
         RSe451p - 1./9.*RN(mp2,me2)*m11*m44*RSp561e*RSe561p - 1./27.*
         RN(mp2,me2)*m11*m44*RSp561e*RSe451p - 1./27.*RN(mp2,me2)*m11*
         m44*RSp451e*RSe561p - 1./27.*RN(mp2,me2)*m11*m44*RSp451e*
         RSe451p );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(m55,L) * (  - 2./9.*m11*m55*RN1e
         *RSe461p*RS146pe + 2./27.*m11*m55*RN1e*RSe461p*RS546pe*RS146pe
          - 2./9.*RN(mp2,me2)*m11*m55*RSp461e*RSe461p + 2./27.*RN(mp2,
         me2)*m11*m55*RSp461e*RSe461p*RS546pe );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 2./27.*m11*RSe461p*
         RS145pe - 2./27.*m11*RSe461p*RS146pe - 1./27.*m11*pow(
         RSe461p,2) - 2./27.*m11*RSe451p*RS145pe + 2./27.*m11*RSe451p*
         RS146pe + 2./27.*m11*RSe451p*RSe461p - 1./27.*m11*pow(
         RSe451p,2) - 1./27.*m11*m66*RN1e*RSe461p*RS145pe - 1./27.*m11*
         m66*RN1e*RSe461p*RS146pe - 1./9.*m11*m66*RN1e*RSe451p*RS145pe
          - 1./27.*m11*m66*RN1e*RSe451p*RS146pe - 2./27.*m11*m56*RN1e*
         RSe461p*RS145pe + 2./27.*m11*m56*RN1e*RSe461p*RS146pe + 2./27.
         *m11*m56*RN1e*RSe451p*RS145pe - 2./27.*m11*m56*RN1e*RSe451p*
         RS146pe - 1./27.*m11*m55*RN1e*RSe461p*RS145pe - 1./9.*m11*m55*
         RN1e*RSe461p*RS146pe - 1./27.*m11*m55*RN1e*RSe451p*RS145pe - 1.
         /27.*m11*m55*RN1e*RSe451p*RS146pe - 2./27.*pow(m11,2)*RN1e*
         RSe461p*RS145pe + 2./27.*pow(m11,2)*RN1e*RSe461p*RS146pe + 2./
         27.*pow(m11,2)*RN1e*RSe451p*RS145pe - 2./27.*pow(m11,2)*RN1e*
         RSe451p*RS146pe + 2./27.*RN(mp2,me2)*m11*me2*RSp461e*RSe461p
          - 2./27.*RN(mp2,me2)*m11*me2*RSp461e*RSe451p );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 2./27.*RN(mp2,me2)
         *m11*me2*RSp451e*RSe461p + 2./27.*RN(mp2,me2)*m11*me2*RSp451e*
         RSe451p - 1./27.*RN(mp2,me2)*m11*m66*RSp461e*RSe461p - 1./27.*
         RN(mp2,me2)*m11*m66*RSp461e*RSe451p - 1./27.*RN(mp2,me2)*m11*
         m66*RSp451e*RSe461p - 1./9.*RN(mp2,me2)*m11*m66*RSp451e*
         RSe451p + 2./27.*RN(mp2,me2)*m11*m56*RSp461e*RSe461p - 2./27.*
         RN(mp2,me2)*m11*m56*RSp461e*RSe451p - 2./27.*RN(mp2,me2)*m11*
         m56*RSp451e*RSe461p + 2./27.*RN(mp2,me2)*m11*m56*RSp451e*
         RSe451p - 1./9.*RN(mp2,me2)*m11*m55*RSp461e*RSe461p - 1./27.*
         RN(mp2,me2)*m11*m55*RSp461e*RSe451p - 1./27.*RN(mp2,me2)*m11*
         m55*RSp451e*RSe461p - 1./27.*RN(mp2,me2)*m11*m55*RSp451e*
         RSe451p );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(m66,L) * (  - 2./9.*m11*m66*RN1e
         *RSe451p*RS145pe + 2./27.*m11*m66*RN1e*RSe451p*RS645pe*RS145pe
          - 2./9.*RN(mp2,me2)*m11*m66*RSp451e*RSe451p + 2./27.*RN(mp2,
         me2)*m11*m66*RSp451e*RSe451p*RS645pe );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(mp2,L) * ( 1./9.*m11*RSp45611e*
         RSe45611p + 2./27.*m11*m66*RN1e*RSp456e*RSe451p*RS145pe + 2./
         27.*m11*m55*RN1e*RSp465e*RSe461p*RS146pe + 2./27.*m11*m44*RN1e
         *RSp564e*RSe561p*RS156pe - 2./9.*pow(m11,2)*RN1e*RSp45611e*
         RSe45611p + 2./27.*RN(mp2,me2)*m11*m66*RSp456e*RSp451e*RSe451p
          + 2./27.*RN(mp2,me2)*m11*m55*RSp465e*RSp461e*RSe461p + 2./27.
         *RN(mp2,me2)*m11*m44*RSp564e*RSp561e*RSe561p );

      M12p6x13RV +=  + Ab(me2,mu2)*AbV(me2,L) * ( 1./9.*m11*pow(
         RSe45611p,2) + 2./27.*m11*m66*RN1e*RSe456p*RSe451p*RS145pe + 2.
         /27.*m11*m55*RN1e*RSe465p*RSe461p*RS146pe + 2./27.*m11*m44*
         RN1e*RSe564p*RSe561p*RS156pe - 2./9.*pow(m11,2)*RN1e*pow(
         RSe45611p,2) + 2./27.*RN(mp2,me2)*m11*m66*RSe456p*RSp451e*
         RSe451p + 2./27.*RN(mp2,me2)*m11*m55*RSe465p*RSp461e*RSe461p
          + 2./27.*RN(mp2,me2)*m11*m44*RSe564p*RSp561e*RSe561p );

      M12p6x13RV +=  + Ab(me2,mu2)*BbV(m11,mp2,L) * (  - 2./27.*m11*m66
         *RSe456p*RSp451e*RS145pe - 2./27.*m11*m55*RSe465p*RSp461e*
         RS146pe - 2./27.*m11*m44*RSe564p*RSp561e*RS156pe + 2./9.*pow(
         m11,2)*RSp45611e*RSe45611p );

      M12p6x13RV +=  + Ab(me2,mu2)*BbV(m11,me2,L) * (  - 2./27.*m11*m66
         *RSe456p*RSe451p*RS145pe - 2./27.*m11*m55*RSe465p*RSe461p*
         RS146pe - 2./27.*m11*m44*RSe564p*RSe561p*RS156pe + 2./9.*pow(
         m11,2)*pow(RSe45611p,2) );

      M12p6x13RV +=  + Ab(me2,mu2)*BbV(m11,L) * ( 1./9.*m11*RSe45611p*
         RD1456ep - 1./27.*m11*m66*RSe456p*pow(RS145pe,2) - 1./27.*m11*
         m55*RSe465p*pow(RS146pe,2) - 1./27.*m11*m44*RSe564p*pow(
         RS156pe,2) + 2./9.*pow(m11,2)*RSe45611p*RC1456pe - 2./9.*pow(
         m11,2)*RN1e*RSe45611p*RD1456ep );

      M12p6x13RV +=  + Ab(me2,mu2)*BbV(m14,L) * ( 1./108.*m44*me2*
         RSe45611p - 1./54.*m44*me2*RSe561p + 1./108.*m44*me2*RSe564p
          - 1./216.*pow(m44,2)*RSe45611p - 1./108.*pow(m44,2)*RSe561p
          - 1./72.*pow(m44,2)*RSe564p - 1./54.*m14*me2*RSe45611p + 1./
         27.*m14*me2*RSe561p - 1./54.*m14*me2*RSe564p + 1./54.*m14*m44*
         RSe45611p + 1./27.*m14*m44*RSe564p - 1./54.*pow(m14,2)*
         RSe45611p + 1./27.*pow(m14,2)*RSe561p - 1./54.*pow(m14,2)*
         RSe564p + 1./108.*m11*me2*RSe45611p - 1./54.*m11*me2*RSe561p
          + 1./108.*m11*me2*RSe564p - 1./54.*m11*m44*RSe45611p - 1./54.
         *m11*m44*RSe561p - 1./54.*m11*m44*RSe564p + 1./27.*m11*m14*
         RSe45611p + 1./54.*m11*m14*RSe564p - 1./72.*pow(m11,2)*
         RSe45611p - 1./108.*pow(m11,2)*RSe561p - 1./216.*pow(m11,2)*
         RSe564p );

      M12p6x13RV +=  + Ab(me2,mu2)*BbV(m15,L) * ( 1./108.*m55*me2*
         RSe45611p - 1./54.*m55*me2*RSe461p + 1./108.*m55*me2*RSe465p
          - 1./216.*pow(m55,2)*RSe45611p - 1./108.*pow(m55,2)*RSe461p
          - 1./72.*pow(m55,2)*RSe465p - 1./54.*m15*me2*RSe45611p + 1./
         27.*m15*me2*RSe461p - 1./54.*m15*me2*RSe465p + 1./54.*m15*m55*
         RSe45611p + 1./27.*m15*m55*RSe465p - 1./54.*pow(m15,2)*
         RSe45611p + 1./27.*pow(m15,2)*RSe461p - 1./54.*pow(m15,2)*
         RSe465p + 1./108.*m11*me2*RSe45611p - 1./54.*m11*me2*RSe461p
          + 1./108.*m11*me2*RSe465p - 1./54.*m11*m55*RSe45611p - 1./54.
         *m11*m55*RSe461p - 1./54.*m11*m55*RSe465p + 1./27.*m11*m15*
         RSe45611p + 1./54.*m11*m15*RSe465p - 1./72.*pow(m11,2)*
         RSe45611p - 1./108.*pow(m11,2)*RSe461p - 1./216.*pow(m11,2)*
         RSe465p );

      M12p6x13RV +=  + Ab(me2,mu2)*BbV(m16,L) * ( 1./108.*m66*me2*
         RSe45611p - 1./54.*m66*me2*RSe451p + 1./108.*m66*me2*RSe456p
          - 1./216.*pow(m66,2)*RSe45611p - 1./108.*pow(m66,2)*RSe451p
          - 1./72.*pow(m66,2)*RSe456p - 1./54.*m16*me2*RSe45611p + 1./
         27.*m16*me2*RSe451p - 1./54.*m16*me2*RSe456p + 1./54.*m16*m66*
         RSe45611p + 1./27.*m16*m66*RSe456p - 1./54.*pow(m16,2)*
         RSe45611p + 1./27.*pow(m16,2)*RSe451p - 1./54.*pow(m16,2)*
         RSe456p + 1./108.*m11*me2*RSe45611p - 1./54.*m11*me2*RSe451p
          + 1./108.*m11*me2*RSe456p - 1./54.*m11*m66*RSe45611p - 1./54.
         *m11*m66*RSe451p - 1./54.*m11*m66*RSe456p + 1./27.*m11*m16*
         RSe45611p + 1./54.*m11*m16*RSe456p - 1./72.*pow(m11,2)*
         RSe45611p - 1./108.*pow(m11,2)*RSe451p - 1./216.*pow(m11,2)*
         RSe456p );

      M12p6x13RV +=  + Ab(me2,mu2)*BbV(mp2,L) * (  - 1./27.*m11*m66*
         RSe456p*pow(RSp451e,2) - 1./27.*m11*m55*RSe465p*pow(RSp461e,2)
          - 1./27.*m11*m44*RSe564p*pow(RSp561e,2) );

      M12p6x13RV +=  + Ab(me2,mu2)*BbV(me2,mp2,L) * (  - 2./27.*m11*m66
         *RSe456p*RSp451e*RSe451p - 2./27.*m11*m55*RSe465p*RSp461e*
         RSe461p - 2./27.*m11*m44*RSe564p*RSp561e*RSe561p );

      M12p6x13RV +=  + Ab(me2,mu2)*BbV(me2,L) * (  - 1./27.*m11*m66*
         RSe456p*pow(RSe451p,2) - 1./27.*m11*m55*RSe465p*pow(RSe461p,2)
          - 1./27.*m11*m44*RSe564p*pow(RSe561p,2) );

      M12p6x13RV +=  + Ab(me2,mu2)*CbV(m11,L) * ( 2./9.*pow(m11,2)*
         RSe45611p*RD1456ep );

      M12p6x13RV +=  + Bb(m11,mu2)*AbV(m11,L) * ( 1./9.*m11*RD1456ep*
         RC1456pe + pow(m11,2) + 2./9.*pow(m11,2)*pow(RC1456pe,2) );

      M12p6x13RV +=  + Bb(m11,mu2)*AbV(m14,L) * ( 1./30.*m44*RD1456ep
          - 1./15.*m14*RD1456ep + 43./90.*m11*RD1456ep - 1./9.*m11*m44*
         RC1456pe - 1./9.*m11*m44*RS156pe + 2./9.*m11*m14*RC1456pe - 2./
         9.*m11*m14*RS156pe - 1./9.*pow(m11,2)*RC1456pe - 1./3.*pow(
         m11,2)*RS156pe );

      M12p6x13RV +=  + Bb(m11,mu2)*AbV(m15,L) * ( 1./30.*m55*RD1456ep
          - 1./15.*m15*RD1456ep + 43./90.*m11*RD1456ep - 1./9.*m11*m55*
         RC1456pe - 1./9.*m11*m55*RS146pe + 2./9.*m11*m15*RC1456pe - 2./
         9.*m11*m15*RS146pe - 1./9.*pow(m11,2)*RC1456pe - 1./3.*pow(
         m11,2)*RS146pe );

      M12p6x13RV +=  + Bb(m11,mu2)*AbV(m16,L) * ( 1./30.*m66*RD1456ep
          - 1./15.*m16*RD1456ep + 43./90.*m11*RD1456ep - 1./9.*m11*m66*
         RC1456pe - 1./9.*m11*m66*RS145pe + 2./9.*m11*m16*RC1456pe - 2./
         9.*m11*m16*RS145pe - 1./9.*pow(m11,2)*RC1456pe - 1./3.*pow(
         m11,2)*RS145pe );

      M12p6x13RV +=  + Bb(m11,mu2)*AbV(m44,L) * ( 1./9.*m11*m44*pow(
         RS156pe,2) - 1./27.*m11*m44*RS456pe*pow(RS156pe,2) );

      M12p6x13RV +=  + Bb(m11,mu2)*AbV(m45,L) * ( 1./18.*m11*m55*pow(
         RS146pe,2) + 1./27.*m11*m55*RS156pe*RS146pe + 1./54.*m11*m55*
         pow(RS156pe,2) - 1./27.*m11*m45*pow(RS146pe,2) + 2./27.*m11*
         m45*RS156pe*RS146pe - 1./27.*m11*m45*pow(RS156pe,2) + 1./54.*
         m11*m44*pow(RS146pe,2) + 1./27.*m11*m44*RS156pe*RS146pe + 1./
         18.*m11*m44*pow(RS156pe,2) - 1./27.*pow(m11,2)*pow(RS146pe,2)
          + 2./27.*pow(m11,2)*RS156pe*RS146pe - 1./27.*pow(m11,2)*pow(
         RS156pe,2) );

      M12p6x13RV +=  + Bb(m11,mu2)*AbV(m46,L) * ( 1./18.*m11*m66*pow(
         RS145pe,2) + 1./27.*m11*m66*RS156pe*RS145pe + 1./54.*m11*m66*
         pow(RS156pe,2) - 1./27.*m11*m46*pow(RS145pe,2) + 2./27.*m11*
         m46*RS156pe*RS145pe - 1./27.*m11*m46*pow(RS156pe,2) + 1./54.*
         m11*m44*pow(RS145pe,2) + 1./27.*m11*m44*RS156pe*RS145pe + 1./
         18.*m11*m44*pow(RS156pe,2) - 1./27.*pow(m11,2)*pow(RS145pe,2)
          + 2./27.*pow(m11,2)*RS156pe*RS145pe - 1./27.*pow(m11,2)*pow(
         RS156pe,2) );

      M12p6x13RV +=  + Bb(m11,mu2)*AbV(m55,L) * ( 1./9.*m11*m55*pow(
         RS146pe,2) - 1./27.*m11*m55*RS546pe*pow(RS146pe,2) );

      M12p6x13RV +=  + Bb(m11,mu2)*AbV(m56,L) * ( 1./18.*m11*m66*pow(
         RS145pe,2) + 1./27.*m11*m66*RS146pe*RS145pe + 1./54.*m11*m66*
         pow(RS146pe,2) - 1./27.*m11*m56*pow(RS145pe,2) + 2./27.*m11*
         m56*RS146pe*RS145pe - 1./27.*m11*m56*pow(RS146pe,2) + 1./54.*
         m11*m55*pow(RS145pe,2) + 1./27.*m11*m55*RS146pe*RS145pe + 1./
         18.*m11*m55*pow(RS146pe,2) - 1./27.*pow(m11,2)*pow(RS145pe,2)
          + 2./27.*pow(m11,2)*RS146pe*RS145pe - 1./27.*pow(m11,2)*pow(
         RS146pe,2) );

      M12p6x13RV +=  + Bb(m11,mu2)*AbV(m66,L) * ( 1./9.*m11*m66*pow(
         RS145pe,2) - 1./27.*m11*m66*RS645pe*pow(RS145pe,2) );

      M12p6x13RV +=  + Bb(m11,mu2)*AbV(mp2,L) * ( 1./9.*m11*RSp45611e*
         RD1456ep - 1./27.*m11*m66*RSp456e*pow(RS145pe,2) - 1./27.*m11*
         m55*RSp465e*pow(RS146pe,2) - 1./27.*m11*m44*RSp564e*pow(
         RS156pe,2) + 2./9.*pow(m11,2)*RSp45611e*RC1456pe );

      M12p6x13RV +=  + Bb(m11,mu2)*AbV(me2,L) * ( 1./9.*m11*RSe45611p*
         RD1456ep - 1./27.*m11*m66*RSe456p*pow(RS145pe,2) - 1./27.*m11*
         m55*RSe465p*pow(RS146pe,2) - 1./27.*m11*m44*RSe564p*pow(
         RS156pe,2) + 2./9.*pow(m11,2)*RSe45611p*RC1456pe );

      M12p6x13RV +=  + Bb(m11,mu2)*BbV(m11,mp2,L) * ( 2./9.*pow(m11,2)*
         RSp45611e*RD1456ep );

      M12p6x13RV +=  + Bb(m11,mu2)*BbV(m11,me2,L) * ( 2./9.*pow(m11,2)*
         RSe45611p*RD1456ep );

      M12p6x13RV +=  + Bb(m11,mu2)*BbV(m11,L) * ( 1./9.*m11*pow(
         RD1456ep,2) + 4./9.*pow(m11,2)*RD1456ep*RC1456pe );

      M12p6x13RV +=  + Bb(m11,mu2)*BbV(m14,L) * (  - 1./216.*pow(m44,2)
         *RD1456ep + 1./54.*m14*m44*RD1456ep - 1./54.*pow(m14,2)*
         RD1456ep - 1./108.*m11*m44*RD1456ep + 1./54.*m11*m14*RD1456ep
          - 1./216.*pow(m11,2)*RD1456ep );

      M12p6x13RV +=  + Bb(m11,mu2)*BbV(m15,L) * (  - 1./216.*pow(m55,2)
         *RD1456ep + 1./54.*m15*m55*RD1456ep - 1./54.*pow(m15,2)*
         RD1456ep - 1./108.*m11*m55*RD1456ep + 1./54.*m11*m15*RD1456ep
          - 1./216.*pow(m11,2)*RD1456ep );

      M12p6x13RV +=  + Bb(m11,mu2)*BbV(m16,L) * (  - 1./216.*pow(m66,2)
         *RD1456ep + 1./54.*m16*m66*RD1456ep - 1./54.*pow(m16,2)*
         RD1456ep - 1./108.*m11*m66*RD1456ep + 1./54.*m11*m16*RD1456ep
          - 1./216.*pow(m11,2)*RD1456ep );

      M12p6x13RV +=  + Bb(m11,mu2)*CbV(m11,L) * ( 2./9.*pow(m11,2)*pow(
         RD1456ep,2) );

      M12p6x13RV +=  + Bb(m14,mu2)*AbV(m11,L) * ( 1./108.*m44*RD1456ep
          - 1./216.*pow(m44,2)*RC1456pe - 1./108.*pow(m44,2)*RS156pe - 
         1./54.*m14*RD1456ep + 1./54.*m14*m44*RC1456pe - 1./54.*pow(
         m14,2)*RC1456pe + 1./27.*pow(m14,2)*RS156pe + 1./108.*m11*
         RD1456ep - 1./108.*m11*m44*RC1456pe - 1./27.*m11*m44*RS156pe
          + 1./54.*m11*m14*RC1456pe + 1./27.*m11*m14*RS156pe - 1./216.*
         pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS156pe );

      M12p6x13RV +=  + Bb(m14,mu2)*AbV(m14,L) * ( 1./36.*pow(m44,2) - 1.
         /9.*m14*m44 + 1./9.*pow(m14,2) + 1./18.*m11*m44 - 1./9.*m11*
         m14 + 1./36.*pow(m11,2) );

      M12p6x13RV +=  + Bb(m14,mu2)*AbV(m15,L) * ( 1./72.*m44*m55 + 1./
         72.*pow(m44,2) - 1./36.*m15*m44 - 1./36.*m14*m55 - 1./18.*m14*
         m44 + 1./18.*m14*m15 + 1./18.*pow(m14,2) + 1./72.*m11*m55 + 1./
         24.*m11*m44 - 1./36.*m11*m15 - 1./12.*m11*m14 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + Bb(m14,mu2)*AbV(m16,L) * ( 1./72.*m44*m66 + 1./
         72.*pow(m44,2) - 1./36.*m16*m44 - 1./36.*m14*m66 - 1./18.*m14*
         m44 + 1./18.*m14*m16 + 1./18.*pow(m14,2) + 1./72.*m11*m66 + 1./
         24.*m11*m44 - 1./36.*m11*m16 - 1./12.*m11*m14 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + Bb(m14,mu2)*AbV(m44,L) * ( 1./72.*pow(m44,2) - 1.
         /216.*pow(m44,2)*RS456pe - 1./18.*m14*m44 + 1./54.*m14*m44*
         RS456pe + 1./18.*pow(m14,2) - 1./54.*pow(m14,2)*RS456pe + 1./
         36.*m11*m44 - 1./108.*m11*m44*RS456pe - 1./18.*m11*m14 + 1./54.
         *m11*m14*RS456pe + 1./72.*pow(m11,2) - 1./216.*pow(m11,2)*
         RS456pe );

      M12p6x13RV +=  + Bb(m14,mu2)*AbV(m45,L) * ( 1./72.*m44*m55 - 1./
         36.*m44*m45 + 1./36.*pow(m44,2) - 1./36.*m14*m55 + 1./18.*m14*
         m45 - 1./12.*m14*m44 + 1./18.*pow(m14,2) + 1./72.*m11*m55 - 1./
         36.*m11*m45 + 1./24.*m11*m44 - 1./18.*m11*m14 + 1./72.*pow(
         m11,2) );

      M12p6x13RV +=  + Bb(m14,mu2)*AbV(m46,L) * ( 1./72.*m44*m66 - 1./
         36.*m44*m46 + 1./36.*pow(m44,2) - 1./36.*m14*m66 + 1./18.*m14*
         m46 - 1./12.*m14*m44 + 1./18.*pow(m14,2) + 1./72.*m11*m66 - 1./
         36.*m11*m46 + 1./24.*m11*m44 - 1./18.*m11*m14 + 1./72.*pow(
         m11,2) );

      M12p6x13RV +=  + Bb(m14,mu2)*AbV(mp2,L) * ( 1./108.*m44*mp2*
         RSp45611e - 1./54.*m44*mp2*RSp561e + 1./108.*m44*mp2*RSp564e
          - 1./216.*pow(m44,2)*RSp45611e - 1./108.*pow(m44,2)*RSp561e
          - 1./72.*pow(m44,2)*RSp564e - 1./54.*m14*mp2*RSp45611e + 1./
         27.*m14*mp2*RSp561e - 1./54.*m14*mp2*RSp564e + 1./54.*m14*m44*
         RSp45611e + 1./27.*m14*m44*RSp564e - 1./54.*pow(m14,2)*
         RSp45611e + 1./27.*pow(m14,2)*RSp561e - 1./54.*pow(m14,2)*
         RSp564e + 1./108.*m11*mp2*RSp45611e - 1./54.*m11*mp2*RSp561e
          + 1./108.*m11*mp2*RSp564e - 1./54.*m11*m44*RSp45611e - 1./54.
         *m11*m44*RSp561e - 1./54.*m11*m44*RSp564e + 1./27.*m11*m14*
         RSp45611e + 1./54.*m11*m14*RSp564e - 1./72.*pow(m11,2)*
         RSp45611e - 1./108.*pow(m11,2)*RSp561e - 1./216.*pow(m11,2)*
         RSp564e );

      M12p6x13RV +=  + Bb(m14,mu2)*AbV(me2,L) * ( 1./108.*m44*me2*
         RSe45611p - 1./54.*m44*me2*RSe561p + 1./108.*m44*me2*RSe564p
          - 1./216.*pow(m44,2)*RSe45611p - 1./108.*pow(m44,2)*RSe561p
          - 1./72.*pow(m44,2)*RSe564p - 1./54.*m14*me2*RSe45611p + 1./
         27.*m14*me2*RSe561p - 1./54.*m14*me2*RSe564p + 1./54.*m14*m44*
         RSe45611p + 1./27.*m14*m44*RSe564p - 1./54.*pow(m14,2)*
         RSe45611p + 1./27.*pow(m14,2)*RSe561p - 1./54.*pow(m14,2)*
         RSe564p + 1./108.*m11*me2*RSe45611p - 1./54.*m11*me2*RSe561p
          + 1./108.*m11*me2*RSe564p - 1./54.*m11*m44*RSe45611p - 1./54.
         *m11*m44*RSe561p - 1./54.*m11*m44*RSe564p + 1./27.*m11*m14*
         RSe45611p + 1./54.*m11*m14*RSe564p - 1./72.*pow(m11,2)*
         RSe45611p - 1./108.*pow(m11,2)*RSe561p - 1./216.*pow(m11,2)*
         RSe564p );

      M12p6x13RV +=  + Bb(m14,mu2)*BbV(m11,L) * (  - 1./216.*pow(m44,2)
         *RD1456ep + 1./54.*m14*m44*RD1456ep - 1./54.*pow(m14,2)*
         RD1456ep - 1./108.*m11*m44*RD1456ep + 1./54.*m11*m14*RD1456ep
          - 1./216.*pow(m11,2)*RD1456ep );

      M12p6x13RV +=  + Bb(m15,mu2)*AbV(m11,L) * ( 1./108.*m55*RD1456ep
          - 1./216.*pow(m55,2)*RC1456pe - 1./108.*pow(m55,2)*RS146pe - 
         1./54.*m15*RD1456ep + 1./54.*m15*m55*RC1456pe - 1./54.*pow(
         m15,2)*RC1456pe + 1./27.*pow(m15,2)*RS146pe + 1./108.*m11*
         RD1456ep - 1./108.*m11*m55*RC1456pe - 1./27.*m11*m55*RS146pe
          + 1./54.*m11*m15*RC1456pe + 1./27.*m11*m15*RS146pe - 1./216.*
         pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS146pe );

      M12p6x13RV +=  + Bb(m15,mu2)*AbV(m14,L) * ( 1./72.*pow(m55,2) + 1.
         /72.*m44*m55 - 1./18.*m15*m55 - 1./36.*m15*m44 + 1./18.*pow(
         m15,2) - 1./36.*m14*m55 + 1./18.*m14*m15 + 1./24.*m11*m55 + 1./
         72.*m11*m44 - 1./12.*m11*m15 - 1./36.*m11*m14 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + Bb(m15,mu2)*AbV(m15,L) * ( 1./36.*pow(m55,2) - 1.
         /9.*m15*m55 + 1./9.*pow(m15,2) + 1./18.*m11*m55 - 1./9.*m11*
         m15 + 1./36.*pow(m11,2) );

      M12p6x13RV +=  + Bb(m15,mu2)*AbV(m16,L) * ( 1./72.*m55*m66 + 1./
         72.*pow(m55,2) - 1./36.*m16*m55 - 1./36.*m15*m66 - 1./18.*m15*
         m55 + 1./18.*m15*m16 + 1./18.*pow(m15,2) + 1./72.*m11*m66 + 1./
         24.*m11*m55 - 1./36.*m11*m16 - 1./12.*m11*m15 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + Bb(m15,mu2)*AbV(m45,L) * ( 1./36.*pow(m55,2) - 1.
         /36.*m45*m55 + 1./72.*m44*m55 - 1./12.*m15*m55 + 1./18.*m15*
         m45 - 1./36.*m15*m44 + 1./18.*pow(m15,2) + 1./24.*m11*m55 - 1./
         36.*m11*m45 + 1./72.*m11*m44 - 1./18.*m11*m15 + 1./72.*pow(
         m11,2) );

      M12p6x13RV +=  + Bb(m15,mu2)*AbV(m55,L) * ( 1./72.*pow(m55,2) - 1.
         /216.*pow(m55,2)*RS546pe - 1./18.*m15*m55 + 1./54.*m15*m55*
         RS546pe + 1./18.*pow(m15,2) - 1./54.*pow(m15,2)*RS546pe + 1./
         36.*m11*m55 - 1./108.*m11*m55*RS546pe - 1./18.*m11*m15 + 1./54.
         *m11*m15*RS546pe + 1./72.*pow(m11,2) - 1./216.*pow(m11,2)*
         RS546pe );

      M12p6x13RV +=  + Bb(m15,mu2)*AbV(m56,L) * ( 1./72.*m55*m66 - 1./
         36.*m55*m56 + 1./36.*pow(m55,2) - 1./36.*m15*m66 + 1./18.*m15*
         m56 - 1./12.*m15*m55 + 1./18.*pow(m15,2) + 1./72.*m11*m66 - 1./
         36.*m11*m56 + 1./24.*m11*m55 - 1./18.*m11*m15 + 1./72.*pow(
         m11,2) );

      M12p6x13RV +=  + Bb(m15,mu2)*AbV(mp2,L) * ( 1./108.*m55*mp2*
         RSp45611e - 1./54.*m55*mp2*RSp461e + 1./108.*m55*mp2*RSp465e
          - 1./216.*pow(m55,2)*RSp45611e - 1./108.*pow(m55,2)*RSp461e
          - 1./72.*pow(m55,2)*RSp465e - 1./54.*m15*mp2*RSp45611e + 1./
         27.*m15*mp2*RSp461e - 1./54.*m15*mp2*RSp465e + 1./54.*m15*m55*
         RSp45611e + 1./27.*m15*m55*RSp465e - 1./54.*pow(m15,2)*
         RSp45611e + 1./27.*pow(m15,2)*RSp461e - 1./54.*pow(m15,2)*
         RSp465e + 1./108.*m11*mp2*RSp45611e - 1./54.*m11*mp2*RSp461e
          + 1./108.*m11*mp2*RSp465e - 1./54.*m11*m55*RSp45611e - 1./54.
         *m11*m55*RSp461e - 1./54.*m11*m55*RSp465e + 1./27.*m11*m15*
         RSp45611e + 1./54.*m11*m15*RSp465e - 1./72.*pow(m11,2)*
         RSp45611e - 1./108.*pow(m11,2)*RSp461e - 1./216.*pow(m11,2)*
         RSp465e );

      M12p6x13RV +=  + Bb(m15,mu2)*AbV(me2,L) * ( 1./108.*m55*me2*
         RSe45611p - 1./54.*m55*me2*RSe461p + 1./108.*m55*me2*RSe465p
          - 1./216.*pow(m55,2)*RSe45611p - 1./108.*pow(m55,2)*RSe461p
          - 1./72.*pow(m55,2)*RSe465p - 1./54.*m15*me2*RSe45611p + 1./
         27.*m15*me2*RSe461p - 1./54.*m15*me2*RSe465p + 1./54.*m15*m55*
         RSe45611p + 1./27.*m15*m55*RSe465p - 1./54.*pow(m15,2)*
         RSe45611p + 1./27.*pow(m15,2)*RSe461p - 1./54.*pow(m15,2)*
         RSe465p + 1./108.*m11*me2*RSe45611p - 1./54.*m11*me2*RSe461p
          + 1./108.*m11*me2*RSe465p - 1./54.*m11*m55*RSe45611p - 1./54.
         *m11*m55*RSe461p - 1./54.*m11*m55*RSe465p + 1./27.*m11*m15*
         RSe45611p + 1./54.*m11*m15*RSe465p - 1./72.*pow(m11,2)*
         RSe45611p - 1./108.*pow(m11,2)*RSe461p - 1./216.*pow(m11,2)*
         RSe465p );

      M12p6x13RV +=  + Bb(m15,mu2)*BbV(m11,L) * (  - 1./216.*pow(m55,2)
         *RD1456ep + 1./54.*m15*m55*RD1456ep - 1./54.*pow(m15,2)*
         RD1456ep - 1./108.*m11*m55*RD1456ep + 1./54.*m11*m15*RD1456ep
          - 1./216.*pow(m11,2)*RD1456ep );

      M12p6x13RV +=  + Bb(m16,mu2)*AbV(m11,L) * ( 1./108.*m66*RD1456ep
          - 1./216.*pow(m66,2)*RC1456pe - 1./108.*pow(m66,2)*RS145pe - 
         1./54.*m16*RD1456ep + 1./54.*m16*m66*RC1456pe - 1./54.*pow(
         m16,2)*RC1456pe + 1./27.*pow(m16,2)*RS145pe + 1./108.*m11*
         RD1456ep - 1./108.*m11*m66*RC1456pe - 1./27.*m11*m66*RS145pe
          + 1./54.*m11*m16*RC1456pe + 1./27.*m11*m16*RS145pe - 1./216.*
         pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS145pe );

      M12p6x13RV +=  + Bb(m16,mu2)*AbV(m14,L) * ( 1./72.*pow(m66,2) + 1.
         /72.*m44*m66 - 1./18.*m16*m66 - 1./36.*m16*m44 + 1./18.*pow(
         m16,2) - 1./36.*m14*m66 + 1./18.*m14*m16 + 1./24.*m11*m66 + 1./
         72.*m11*m44 - 1./12.*m11*m16 - 1./36.*m11*m14 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + Bb(m16,mu2)*AbV(m15,L) * ( 1./72.*pow(m66,2) + 1.
         /72.*m55*m66 - 1./18.*m16*m66 - 1./36.*m16*m55 + 1./18.*pow(
         m16,2) - 1./36.*m15*m66 + 1./18.*m15*m16 + 1./24.*m11*m66 + 1./
         72.*m11*m55 - 1./12.*m11*m16 - 1./36.*m11*m15 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + Bb(m16,mu2)*AbV(m16,L) * ( 1./36.*pow(m66,2) - 1.
         /9.*m16*m66 + 1./9.*pow(m16,2) + 1./18.*m11*m66 - 1./9.*m11*
         m16 + 1./36.*pow(m11,2) );

      M12p6x13RV +=  + Bb(m16,mu2)*AbV(m46,L) * ( 1./36.*pow(m66,2) - 1.
         /36.*m46*m66 + 1./72.*m44*m66 - 1./12.*m16*m66 + 1./18.*m16*
         m46 - 1./36.*m16*m44 + 1./18.*pow(m16,2) + 1./24.*m11*m66 - 1./
         36.*m11*m46 + 1./72.*m11*m44 - 1./18.*m11*m16 + 1./72.*pow(
         m11,2) );

      M12p6x13RV +=  + Bb(m16,mu2)*AbV(m56,L) * ( 1./36.*pow(m66,2) - 1.
         /36.*m56*m66 + 1./72.*m55*m66 - 1./12.*m16*m66 + 1./18.*m16*
         m56 - 1./36.*m16*m55 + 1./18.*pow(m16,2) + 1./24.*m11*m66 - 1./
         36.*m11*m56 + 1./72.*m11*m55 - 1./18.*m11*m16 + 1./72.*pow(
         m11,2) );

      M12p6x13RV +=  + Bb(m16,mu2)*AbV(m66,L) * ( 1./72.*pow(m66,2) - 1.
         /216.*pow(m66,2)*RS645pe - 1./18.*m16*m66 + 1./54.*m16*m66*
         RS645pe + 1./18.*pow(m16,2) - 1./54.*pow(m16,2)*RS645pe + 1./
         36.*m11*m66 - 1./108.*m11*m66*RS645pe - 1./18.*m11*m16 + 1./54.
         *m11*m16*RS645pe + 1./72.*pow(m11,2) - 1./216.*pow(m11,2)*
         RS645pe );

      M12p6x13RV +=  + Bb(m16,mu2)*AbV(mp2,L) * ( 1./108.*m66*mp2*
         RSp45611e - 1./54.*m66*mp2*RSp451e + 1./108.*m66*mp2*RSp456e
          - 1./216.*pow(m66,2)*RSp45611e - 1./108.*pow(m66,2)*RSp451e
          - 1./72.*pow(m66,2)*RSp456e - 1./54.*m16*mp2*RSp45611e + 1./
         27.*m16*mp2*RSp451e - 1./54.*m16*mp2*RSp456e + 1./54.*m16*m66*
         RSp45611e + 1./27.*m16*m66*RSp456e - 1./54.*pow(m16,2)*
         RSp45611e + 1./27.*pow(m16,2)*RSp451e - 1./54.*pow(m16,2)*
         RSp456e + 1./108.*m11*mp2*RSp45611e - 1./54.*m11*mp2*RSp451e
          + 1./108.*m11*mp2*RSp456e - 1./54.*m11*m66*RSp45611e - 1./54.
         *m11*m66*RSp451e - 1./54.*m11*m66*RSp456e + 1./27.*m11*m16*
         RSp45611e + 1./54.*m11*m16*RSp456e - 1./72.*pow(m11,2)*
         RSp45611e - 1./108.*pow(m11,2)*RSp451e - 1./216.*pow(m11,2)*
         RSp456e );

      M12p6x13RV +=  + Bb(m16,mu2)*AbV(me2,L) * ( 1./108.*m66*me2*
         RSe45611p - 1./54.*m66*me2*RSe451p + 1./108.*m66*me2*RSe456p
          - 1./216.*pow(m66,2)*RSe45611p - 1./108.*pow(m66,2)*RSe451p
          - 1./72.*pow(m66,2)*RSe456p - 1./54.*m16*me2*RSe45611p + 1./
         27.*m16*me2*RSe451p - 1./54.*m16*me2*RSe456p + 1./54.*m16*m66*
         RSe45611p + 1./27.*m16*m66*RSe456p - 1./54.*pow(m16,2)*
         RSe45611p + 1./27.*pow(m16,2)*RSe451p - 1./54.*pow(m16,2)*
         RSe456p + 1./108.*m11*me2*RSe45611p - 1./54.*m11*me2*RSe451p
          + 1./108.*m11*me2*RSe456p - 1./54.*m11*m66*RSe45611p - 1./54.
         *m11*m66*RSe451p - 1./54.*m11*m66*RSe456p + 1./27.*m11*m16*
         RSe45611p + 1./54.*m11*m16*RSe456p - 1./72.*pow(m11,2)*
         RSe45611p - 1./108.*pow(m11,2)*RSe451p - 1./216.*pow(m11,2)*
         RSe456p );

      M12p6x13RV +=  + Bb(m16,mu2)*BbV(m11,L) * (  - 1./216.*pow(m66,2)
         *RD1456ep + 1./54.*m16*m66*RD1456ep - 1./54.*pow(m16,2)*
         RD1456ep - 1./108.*m11*m66*RD1456ep + 1./54.*m11*m16*RD1456ep
          - 1./216.*pow(m11,2)*RD1456ep );

      M12p6x13RV +=  + Bb(mp2,mu2)*AbV(m44,L) * ( 1./9.*m11*m44*pow(
         RSp561e,2) - 1./27.*m11*m44*pow(RSp561e,2)*RS456pe );

      M12p6x13RV +=  + Bb(mp2,mu2)*AbV(m45,L) * (  - 1./27.*m11*mp2*
         pow(RSp561e,2) + 2./27.*m11*mp2*RSp461e*RSp561e - 1./27.*m11*
         mp2*pow(RSp461e,2) + 1./54.*m11*m55*pow(RSp561e,2) + 1./27.*
         m11*m55*RSp461e*RSp561e + 1./18.*m11*m55*pow(RSp461e,2) - 1./
         27.*m11*m45*pow(RSp561e,2) + 2./27.*m11*m45*RSp461e*RSp561e - 
         1./27.*m11*m45*pow(RSp461e,2) + 1./18.*m11*m44*pow(RSp561e,2)
          + 1./27.*m11*m44*RSp461e*RSp561e + 1./54.*m11*m44*pow(
         RSp461e,2) );

      M12p6x13RV +=  + Bb(mp2,mu2)*AbV(m46,L) * (  - 1./27.*m11*mp2*
         pow(RSp561e,2) + 2./27.*m11*mp2*RSp451e*RSp561e - 1./27.*m11*
         mp2*pow(RSp451e,2) + 1./54.*m11*m66*pow(RSp561e,2) + 1./27.*
         m11*m66*RSp451e*RSp561e + 1./18.*m11*m66*pow(RSp451e,2) - 1./
         27.*m11*m46*pow(RSp561e,2) + 2./27.*m11*m46*RSp451e*RSp561e - 
         1./27.*m11*m46*pow(RSp451e,2) + 1./18.*m11*m44*pow(RSp561e,2)
          + 1./27.*m11*m44*RSp451e*RSp561e + 1./54.*m11*m44*pow(
         RSp451e,2) );

      M12p6x13RV +=  + Bb(mp2,mu2)*AbV(m55,L) * ( 1./9.*m11*m55*pow(
         RSp461e,2) - 1./27.*m11*m55*pow(RSp461e,2)*RS546pe );

      M12p6x13RV +=  + Bb(mp2,mu2)*AbV(m56,L) * (  - 1./27.*m11*mp2*
         pow(RSp461e,2) + 2./27.*m11*mp2*RSp451e*RSp461e - 1./27.*m11*
         mp2*pow(RSp451e,2) + 1./54.*m11*m66*pow(RSp461e,2) + 1./27.*
         m11*m66*RSp451e*RSp461e + 1./18.*m11*m66*pow(RSp451e,2) - 1./
         27.*m11*m56*pow(RSp461e,2) + 2./27.*m11*m56*RSp451e*RSp461e - 
         1./27.*m11*m56*pow(RSp451e,2) + 1./18.*m11*m55*pow(RSp461e,2)
          + 1./27.*m11*m55*RSp451e*RSp461e + 1./54.*m11*m55*pow(
         RSp451e,2) );

      M12p6x13RV +=  + Bb(mp2,mu2)*AbV(m66,L) * ( 1./9.*m11*m66*pow(
         RSp451e,2) - 1./27.*m11*m66*pow(RSp451e,2)*RS645pe );

      M12p6x13RV +=  + Bb(mp2,mu2)*AbV(mp2,L) * (  - 1./27.*m11*m66*
         RSp456e*pow(RSp451e,2) - 1./27.*m11*m55*RSp465e*pow(RSp461e,2)
          - 1./27.*m11*m44*RSp564e*pow(RSp561e,2) );

      M12p6x13RV +=  + Bb(mp2,mu2)*AbV(me2,L) * (  - 1./27.*m11*m66*
         RSe456p*pow(RSp451e,2) - 1./27.*m11*m55*RSe465p*pow(RSp461e,2)
          - 1./27.*m11*m44*RSe564p*pow(RSp561e,2) );

      M12p6x13RV +=  + Bb(me2,mu2)*AbV(m44,L) * ( 1./9.*m11*m44*pow(
         RSe561p,2) - 1./27.*m11*m44*pow(RSe561p,2)*RS456pe );

      M12p6x13RV +=  + Bb(me2,mu2)*AbV(m45,L) * (  - 1./27.*m11*me2*
         pow(RSe561p,2) + 2./27.*m11*me2*RSe461p*RSe561p - 1./27.*m11*
         me2*pow(RSe461p,2) + 1./54.*m11*m55*pow(RSe561p,2) + 1./27.*
         m11*m55*RSe461p*RSe561p + 1./18.*m11*m55*pow(RSe461p,2) - 1./
         27.*m11*m45*pow(RSe561p,2) + 2./27.*m11*m45*RSe461p*RSe561p - 
         1./27.*m11*m45*pow(RSe461p,2) + 1./18.*m11*m44*pow(RSe561p,2)
          + 1./27.*m11*m44*RSe461p*RSe561p + 1./54.*m11*m44*pow(
         RSe461p,2) );

      M12p6x13RV +=  + Bb(me2,mu2)*AbV(m46,L) * (  - 1./27.*m11*me2*
         pow(RSe561p,2) + 2./27.*m11*me2*RSe451p*RSe561p - 1./27.*m11*
         me2*pow(RSe451p,2) + 1./54.*m11*m66*pow(RSe561p,2) + 1./27.*
         m11*m66*RSe451p*RSe561p + 1./18.*m11*m66*pow(RSe451p,2) - 1./
         27.*m11*m46*pow(RSe561p,2) + 2./27.*m11*m46*RSe451p*RSe561p - 
         1./27.*m11*m46*pow(RSe451p,2) + 1./18.*m11*m44*pow(RSe561p,2)
          + 1./27.*m11*m44*RSe451p*RSe561p + 1./54.*m11*m44*pow(
         RSe451p,2) );

      M12p6x13RV +=  + Bb(me2,mu2)*AbV(m55,L) * ( 1./9.*m11*m55*pow(
         RSe461p,2) - 1./27.*m11*m55*pow(RSe461p,2)*RS546pe );

      M12p6x13RV +=  + Bb(me2,mu2)*AbV(m56,L) * (  - 1./27.*m11*me2*
         pow(RSe461p,2) + 2./27.*m11*me2*RSe451p*RSe461p - 1./27.*m11*
         me2*pow(RSe451p,2) + 1./54.*m11*m66*pow(RSe461p,2) + 1./27.*
         m11*m66*RSe451p*RSe461p + 1./18.*m11*m66*pow(RSe451p,2) - 1./
         27.*m11*m56*pow(RSe461p,2) + 2./27.*m11*m56*RSe451p*RSe461p - 
         1./27.*m11*m56*pow(RSe451p,2) + 1./18.*m11*m55*pow(RSe461p,2)
          + 1./27.*m11*m55*RSe451p*RSe461p + 1./54.*m11*m55*pow(
         RSe451p,2) );

      M12p6x13RV +=  + Bb(me2,mu2)*AbV(m66,L) * ( 1./9.*m11*m66*pow(
         RSe451p,2) - 1./27.*m11*m66*pow(RSe451p,2)*RS645pe );

      M12p6x13RV +=  + Bb(me2,mu2)*AbV(mp2,L) * (  - 1./27.*m11*m66*
         RSp456e*pow(RSe451p,2) - 1./27.*m11*m55*RSp465e*pow(RSe461p,2)
          - 1./27.*m11*m44*RSp564e*pow(RSe561p,2) );

      M12p6x13RV +=  + Bb(me2,mu2)*AbV(me2,L) * (  - 1./27.*m11*m66*
         RSe456p*pow(RSe451p,2) - 1./27.*m11*m55*RSe465p*pow(RSe461p,2)
          - 1./27.*m11*m44*RSe564p*pow(RSe561p,2) );

      M12p6x13RV +=  + Cb(m11,mu2)*AbV(m11,L) * ( 2./9.*pow(m11,2)*
         RD1456ep*RC1456pe );

      M12p6x13RV +=  + Cb(m11,mu2)*AbV(m14,L) * (  - 1./9.*m11*m44*
         RD1456ep + 2./9.*m11*m14*RD1456ep - 1./9.*pow(m11,2)*RD1456ep
          );

      M12p6x13RV +=  + Cb(m11,mu2)*AbV(m15,L) * (  - 1./9.*m11*m55*
         RD1456ep + 2./9.*m11*m15*RD1456ep - 1./9.*pow(m11,2)*RD1456ep
          );

      M12p6x13RV +=  + Cb(m11,mu2)*AbV(m16,L) * (  - 1./9.*m11*m66*
         RD1456ep + 2./9.*m11*m16*RD1456ep - 1./9.*pow(m11,2)*RD1456ep
          );

      M12p6x13RV +=  + Cb(m11,mu2)*AbV(mp2,L) * ( 2./9.*pow(m11,2)*
         RSp45611e*RD1456ep );

      M12p6x13RV +=  + Cb(m11,mu2)*AbV(me2,L) * ( 2./9.*pow(m11,2)*
         RSe45611p*RD1456ep );

      M12p6x13RV +=  + Cb(m11,mu2)*BbV(m11,L) * ( 2./9.*pow(m11,2)*pow(
         RD1456ep,2) );

      M12p6x13RV +=  + AbV(m11,L)*pi16 * (  - 1./4.*pow(m11,2) + 1./12.
         *pow(m11,2)*RS145pe + 1./12.*pow(m11,2)*RS146pe + 1./12.*pow(
         m11,2)*RS156pe );

      M12p6x13RV +=  + pow(AbV(m11,L),2) * ( 1./2.*m11 + 1./18.*m11*
         pow(RC1456pe,2) );

      M12p6x13RV +=  + AbV(m11,L)*AbV(m14,L) * ( 2./45.*RD1456ep + 1./
         30.*m44*RC1456pe - 1./45.*m44*RS156pe - 1./15.*m14*RC1456pe + 
         2./15.*m14*RS156pe + 43./90.*m11*RC1456pe - 5./9.*m11*RS156pe
          );

      M12p6x13RV +=  + AbV(m11,L)*AbV(m15,L) * ( 2./45.*RD1456ep + 1./
         30.*m55*RC1456pe - 1./45.*m55*RS146pe - 1./15.*m15*RC1456pe + 
         2./15.*m15*RS146pe + 43./90.*m11*RC1456pe - 5./9.*m11*RS146pe
          );

      M12p6x13RV +=  + AbV(m11,L)*AbV(m16,L) * ( 2./45.*RD1456ep + 1./
         30.*m66*RC1456pe - 1./45.*m66*RS145pe - 1./15.*m16*RC1456pe + 
         2./15.*m16*RS145pe + 43./90.*m11*RC1456pe - 5./9.*m11*RS145pe
          );

      M12p6x13RV +=  + AbV(m11,L)*AbV(m45,L) * (  - 1./27.*m11*pow(
         RS146pe,2) + 2./27.*m11*RS156pe*RS146pe - 1./27.*m11*pow(
         RS156pe,2) );

      M12p6x13RV +=  + AbV(m11,L)*AbV(m46,L) * (  - 1./27.*m11*pow(
         RS145pe,2) + 2./27.*m11*RS156pe*RS145pe - 1./27.*m11*pow(
         RS156pe,2) );

      M12p6x13RV +=  + AbV(m11,L)*AbV(m56,L) * (  - 1./27.*m11*pow(
         RS145pe,2) + 2./27.*m11*RS146pe*RS145pe - 1./27.*m11*pow(
         RS146pe,2) );

      M12p6x13RV +=  + AbV(m11,L)*AbV(mp2,L) * ( 1./9.*m11*RSp45611e*
         RC1456pe );

      M12p6x13RV +=  + AbV(m11,L)*AbV(me2,L) * ( 1./9.*m11*RSe45611p*
         RC1456pe );

      M12p6x13RV +=  + AbV(m11,L)*BbV(m11,mp2,L) * ( 2./9.*pow(m11,2)*
         RSp45611e*RC1456pe );

      M12p6x13RV +=  + AbV(m11,L)*BbV(m11,me2,L) * ( 2./9.*pow(m11,2)*
         RSe45611p*RC1456pe );

      M12p6x13RV +=  + AbV(m11,L)*BbV(m11,L) * ( 1./9.*m11*RD1456ep*
         RC1456pe + pow(m11,2) + 2./9.*pow(m11,2)*pow(RC1456pe,2) );

      M12p6x13RV +=  + AbV(m11,L)*BbV(m14,L) * ( 1./108.*m44*RD1456ep
          - 1./216.*pow(m44,2)*RC1456pe - 1./108.*pow(m44,2)*RS156pe - 
         1./54.*m14*RD1456ep + 1./54.*m14*m44*RC1456pe - 1./54.*pow(
         m14,2)*RC1456pe + 1./27.*pow(m14,2)*RS156pe + 1./108.*m11*
         RD1456ep - 1./108.*m11*m44*RC1456pe - 1./27.*m11*m44*RS156pe
          + 1./54.*m11*m14*RC1456pe + 1./27.*m11*m14*RS156pe - 1./216.*
         pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS156pe );

      M12p6x13RV +=  + AbV(m11,L)*BbV(m15,L) * ( 1./108.*m55*RD1456ep
          - 1./216.*pow(m55,2)*RC1456pe - 1./108.*pow(m55,2)*RS146pe - 
         1./54.*m15*RD1456ep + 1./54.*m15*m55*RC1456pe - 1./54.*pow(
         m15,2)*RC1456pe + 1./27.*pow(m15,2)*RS146pe + 1./108.*m11*
         RD1456ep - 1./108.*m11*m55*RC1456pe - 1./27.*m11*m55*RS146pe
          + 1./54.*m11*m15*RC1456pe + 1./27.*m11*m15*RS146pe - 1./216.*
         pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS146pe );

      M12p6x13RV +=  + AbV(m11,L)*BbV(m16,L) * ( 1./108.*m66*RD1456ep
          - 1./216.*pow(m66,2)*RC1456pe - 1./108.*pow(m66,2)*RS145pe - 
         1./54.*m16*RD1456ep + 1./54.*m16*m66*RC1456pe - 1./54.*pow(
         m16,2)*RC1456pe + 1./27.*pow(m16,2)*RS145pe + 1./108.*m11*
         RD1456ep - 1./108.*m11*m66*RC1456pe - 1./27.*m11*m66*RS145pe
          + 1./54.*m11*m16*RC1456pe + 1./27.*m11*m16*RS145pe - 1./216.*
         pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS145pe );

      M12p6x13RV +=  + AbV(m11,L)*CbV(m11,L) * ( 2./9.*pow(m11,2)*
         RD1456ep*RC1456pe );

      M12p6x13RV +=  + AbV(m14,L)*pi16 * (  - 1./12.*m11*RD1456ep - 1./
         12.*m11*me2*RSe45611p + 1./6.*m11*me2*RSe561p - 1./12.*m11*me2
         *RSe564p - 1./12.*m11*mp2*RSp45611e + 1./6.*m11*mp2*RSp561e - 
         1./12.*m11*mp2*RSp564e + 1./4.*m11*m46 + 1./4.*m11*m45 + 1./4.
         *m11*m44 - 1./12.*m11*m44*RS456pe + 1./4.*m11*m16 + 1./4.*m11*
         m15 - 1./12.*m11*m14*RC1456pe - 1./12.*m11*m14*RSe45611p - 1./
         12.*m11*m14*RSp45611e + 1./6.*m11*m14*RS156pe - 1./12.*m11*m14
         *RS456pe + 1./6.*m11*m14*RSe561p + 1./6.*m11*m14*RSp561e - 1./
         12.*m11*m14*RSe564p - 1./12.*m11*m14*RSp564e - 1./4.*pow(
         m11,2) - 1./18.*pow(m11,2)*RC1456pe + 1./36.*pow(m11,2)*
         RSe45611p + 1./36.*pow(m11,2)*RSp45611e + 1./9.*pow(m11,2)*
         RS156pe + 1./36.*pow(m11,2)*RS456pe - 1./18.*pow(m11,2)*
         RSe561p - 1./18.*pow(m11,2)*RSp561e + 1./36.*pow(m11,2)*
         RSe564p + 1./36.*pow(m11,2)*RSp564e );

      M12p6x13RV +=  + pow(AbV(m14,L),2) * (  - 1./27.*RD1456ep - 1./27.
         *me2*RSe45611p + 2./27.*me2*RSe561p - 1./27.*me2*RSe564p - 1./
         27.*mp2*RSp45611e + 2./27.*mp2*RSp561e - 1./27.*mp2*RSp564e - 
         1./20.*m44 - 1./27.*m44*RC1456pe - 1./27.*m44*RSe45611p - 1./
         27.*m44*RSp45611e + 1./27.*m44*RSe564p + 1./27.*m44*RSp564e + 
         1./10.*m14 - 3./10.*m11 - 13./108.*m11*RC1456pe - 1./12.*m11*
         RSe45611p - 1./12.*m11*RSp45611e + 5./54.*m11*RS156pe + 7./108.
         *m11*RS456pe + 1./54.*m11*RSe561p + 1./54.*m11*RSp561e + 7./
         108.*m11*RSe564p + 7./108.*m11*RSp564e );

      M12p6x13RV +=  + AbV(m14,L)*AbV(m15,L) * (  - 29./180.*m55 + 2./9.
         *m45 - 29./180.*m44 + 1./10.*m15 + 1./10.*m14 - 3./5.*m11 );

      M12p6x13RV +=  + AbV(m14,L)*AbV(m16,L) * (  - 29./180.*m66 + 2./9.
         *m46 - 29./180.*m44 + 1./10.*m16 + 1./10.*m14 - 3./5.*m11 );

      M12p6x13RV +=  + AbV(m14,L)*AbV(m44,L) * (  - 1./10.*m44 + 1./30.
         *m44*RS456pe + 1./5.*m14 - 1./15.*m14*RS456pe - 1./10.*m11 + 1.
         /30.*m11*RS456pe );

      M12p6x13RV +=  + AbV(m14,L)*AbV(m45,L) * ( 7./180.*m55 - 2./15.*
         m45 - 1./180.*m44 + 1./18.*m15 + 13./90.*m14 - 1./10.*m11 );

      M12p6x13RV +=  + AbV(m14,L)*AbV(m46,L) * ( 7./180.*m66 - 2./15.*
         m46 - 1./180.*m44 + 1./18.*m16 + 13./90.*m14 - 1./10.*m11 );

      M12p6x13RV +=  + AbV(m14,L)*AbV(mp2,L) * ( 2./45.*mp2*RSp45611e
          - 4./45.*mp2*RSp561e + 2./45.*mp2*RSp564e + 1./30.*m44*
         RSp45611e - 1./45.*m44*RSp561e - 1./90.*m44*RSp564e - 1./15.*
         m14*RSp45611e + 2./15.*m14*RSp561e - 1./15.*m14*RSp564e + 13./
         30.*m11*RSp45611e - 7./15.*m11*RSp561e + 1./30.*m11*RSp564e );

      M12p6x13RV +=  + AbV(m14,L)*AbV(me2,L) * ( 2./45.*me2*RSe45611p
          - 4./45.*me2*RSe561p + 2./45.*me2*RSe564p + 1./30.*m44*
         RSe45611p - 1./45.*m44*RSe561p - 1./90.*m44*RSe564p - 1./15.*
         m14*RSe45611p + 2./15.*m14*RSe561p - 1./15.*m14*RSe564p + 13./
         30.*m11*RSe45611p - 7./15.*m11*RSe561p + 1./30.*m11*RSe564p );

      M12p6x13RV +=  + AbV(m14,L)*BbV(m11,mp2,L) * (  - 1./9.*m11*m44*
         RSp45611e - 1./9.*m11*m44*RSp561e + 2./9.*m11*m14*RSp45611e - 
         2./9.*m11*m14*RSp561e - 1./9.*pow(m11,2)*RSp45611e - 1./3.*
         pow(m11,2)*RSp561e );

      M12p6x13RV +=  + AbV(m14,L)*BbV(m11,me2,L) * (  - 1./9.*m11*m44*
         RSe45611p - 1./9.*m11*m44*RSe561p + 2./9.*m11*m14*RSe45611p - 
         2./9.*m11*m14*RSe561p - 1./9.*pow(m11,2)*RSe45611p - 1./3.*
         pow(m11,2)*RSe561p );

      M12p6x13RV +=  + AbV(m14,L)*BbV(m11,L) * ( 1./30.*m44*RD1456ep - 
         1./15.*m14*RD1456ep + 43./90.*m11*RD1456ep - 1./9.*m11*m44*
         RC1456pe - 1./9.*m11*m44*RS156pe + 2./9.*m11*m14*RC1456pe - 2./
         9.*m11*m14*RS156pe - 1./9.*pow(m11,2)*RC1456pe - 1./3.*pow(
         m11,2)*RS156pe );

      M12p6x13RV +=  + AbV(m14,L)*BbV(m14,L) * ( 1./36.*pow(m44,2) - 1./
         9.*m14*m44 + 1./9.*pow(m14,2) + 1./18.*m11*m44 - 1./9.*m11*m14
          + 1./36.*pow(m11,2) );

      M12p6x13RV +=  + AbV(m14,L)*BbV(m15,L) * ( 1./72.*pow(m55,2) + 1./
         72.*m44*m55 - 1./18.*m15*m55 - 1./36.*m15*m44 + 1./18.*pow(
         m15,2) - 1./36.*m14*m55 + 1./18.*m14*m15 + 1./24.*m11*m55 + 1./
         72.*m11*m44 - 1./12.*m11*m15 - 1./36.*m11*m14 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + AbV(m14,L)*BbV(m16,L) * ( 1./72.*pow(m66,2) + 1./
         72.*m44*m66 - 1./18.*m16*m66 - 1./36.*m16*m44 + 1./18.*pow(
         m16,2) - 1./36.*m14*m66 + 1./18.*m14*m16 + 1./24.*m11*m66 + 1./
         72.*m11*m44 - 1./12.*m11*m16 - 1./36.*m11*m14 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + AbV(m14,L)*CbV(m11,L) * (  - 1./9.*m11*m44*
         RD1456ep + 2./9.*m11*m14*RD1456ep - 1./9.*pow(m11,2)*RD1456ep
          );

      M12p6x13RV +=  + AbV(m15,L)*pi16 * (  - 1./12.*m11*RD1456ep - 1./
         12.*m11*me2*RSe45611p + 1./6.*m11*me2*RSe461p - 1./12.*m11*me2
         *RSe465p - 1./12.*m11*mp2*RSp45611e + 1./6.*m11*mp2*RSp461e - 
         1./12.*m11*mp2*RSp465e + 1./4.*m11*m56 + 1./4.*m11*m55 - 1./12.
         *m11*m55*RS546pe + 1./4.*m11*m45 + 1./4.*m11*m16 - 1./12.*m11*
         m15*RC1456pe - 1./12.*m11*m15*RSe45611p - 1./12.*m11*m15*
         RSp45611e + 1./6.*m11*m15*RS146pe - 1./12.*m11*m15*RS546pe + 1.
         /6.*m11*m15*RSe461p + 1./6.*m11*m15*RSp461e - 1./12.*m11*m15*
         RSe465p - 1./12.*m11*m15*RSp465e + 1./4.*m11*m14 - 1./4.*pow(
         m11,2) - 1./18.*pow(m11,2)*RC1456pe + 1./36.*pow(m11,2)*
         RSe45611p + 1./36.*pow(m11,2)*RSp45611e + 1./9.*pow(m11,2)*
         RS146pe + 1./36.*pow(m11,2)*RS546pe - 1./18.*pow(m11,2)*
         RSe461p - 1./18.*pow(m11,2)*RSp461e + 1./36.*pow(m11,2)*
         RSe465p + 1./36.*pow(m11,2)*RSp465e );

      M12p6x13RV +=  + pow(AbV(m15,L),2) * (  - 1./27.*RD1456ep - 1./27.
         *me2*RSe45611p + 2./27.*me2*RSe461p - 1./27.*me2*RSe465p - 1./
         27.*mp2*RSp45611e + 2./27.*mp2*RSp461e - 1./27.*mp2*RSp465e - 
         1./20.*m55 - 1./27.*m55*RC1456pe - 1./27.*m55*RSe45611p - 1./
         27.*m55*RSp45611e + 1./27.*m55*RSe465p + 1./27.*m55*RSp465e + 
         1./10.*m15 - 3./10.*m11 - 13./108.*m11*RC1456pe - 1./12.*m11*
         RSe45611p - 1./12.*m11*RSp45611e + 5./54.*m11*RS146pe + 7./108.
         *m11*RS546pe + 1./54.*m11*RSe461p + 1./54.*m11*RSp461e + 7./
         108.*m11*RSe465p + 7./108.*m11*RSp465e );

      M12p6x13RV +=  + AbV(m15,L)*AbV(m16,L) * (  - 29./180.*m66 + 2./9.
         *m56 - 29./180.*m55 + 1./10.*m16 + 1./10.*m15 - 3./5.*m11 );

      M12p6x13RV +=  + AbV(m15,L)*AbV(m45,L) * (  - 1./180.*m55 - 2./15.
         *m45 + 7./180.*m44 + 13./90.*m15 + 1./18.*m14 - 1./10.*m11 );

      M12p6x13RV +=  + AbV(m15,L)*AbV(m55,L) * (  - 1./10.*m55 + 1./30.
         *m55*RS546pe + 1./5.*m15 - 1./15.*m15*RS546pe - 1./10.*m11 + 1.
         /30.*m11*RS546pe );

      M12p6x13RV +=  + AbV(m15,L)*AbV(m56,L) * ( 7./180.*m66 - 2./15.*
         m56 - 1./180.*m55 + 1./18.*m16 + 13./90.*m15 - 1./10.*m11 );

      M12p6x13RV +=  + AbV(m15,L)*AbV(mp2,L) * ( 2./45.*mp2*RSp45611e
          - 4./45.*mp2*RSp461e + 2./45.*mp2*RSp465e + 1./30.*m55*
         RSp45611e - 1./45.*m55*RSp461e - 1./90.*m55*RSp465e - 1./15.*
         m15*RSp45611e + 2./15.*m15*RSp461e - 1./15.*m15*RSp465e + 13./
         30.*m11*RSp45611e - 7./15.*m11*RSp461e + 1./30.*m11*RSp465e );

      M12p6x13RV +=  + AbV(m15,L)*AbV(me2,L) * ( 2./45.*me2*RSe45611p
          - 4./45.*me2*RSe461p + 2./45.*me2*RSe465p + 1./30.*m55*
         RSe45611p - 1./45.*m55*RSe461p - 1./90.*m55*RSe465p - 1./15.*
         m15*RSe45611p + 2./15.*m15*RSe461p - 1./15.*m15*RSe465p + 13./
         30.*m11*RSe45611p - 7./15.*m11*RSe461p + 1./30.*m11*RSe465p );

      M12p6x13RV +=  + AbV(m15,L)*BbV(m11,mp2,L) * (  - 1./9.*m11*m55*
         RSp45611e - 1./9.*m11*m55*RSp461e + 2./9.*m11*m15*RSp45611e - 
         2./9.*m11*m15*RSp461e - 1./9.*pow(m11,2)*RSp45611e - 1./3.*
         pow(m11,2)*RSp461e );

      M12p6x13RV +=  + AbV(m15,L)*BbV(m11,me2,L) * (  - 1./9.*m11*m55*
         RSe45611p - 1./9.*m11*m55*RSe461p + 2./9.*m11*m15*RSe45611p - 
         2./9.*m11*m15*RSe461p - 1./9.*pow(m11,2)*RSe45611p - 1./3.*
         pow(m11,2)*RSe461p );

      M12p6x13RV +=  + AbV(m15,L)*BbV(m11,L) * ( 1./30.*m55*RD1456ep - 
         1./15.*m15*RD1456ep + 43./90.*m11*RD1456ep - 1./9.*m11*m55*
         RC1456pe - 1./9.*m11*m55*RS146pe + 2./9.*m11*m15*RC1456pe - 2./
         9.*m11*m15*RS146pe - 1./9.*pow(m11,2)*RC1456pe - 1./3.*pow(
         m11,2)*RS146pe );

      M12p6x13RV +=  + AbV(m15,L)*BbV(m14,L) * ( 1./72.*m44*m55 + 1./72.
         *pow(m44,2) - 1./36.*m15*m44 - 1./36.*m14*m55 - 1./18.*m14*m44
          + 1./18.*m14*m15 + 1./18.*pow(m14,2) + 1./72.*m11*m55 + 1./24.
         *m11*m44 - 1./36.*m11*m15 - 1./12.*m11*m14 + 1./36.*pow(m11,2)
          );

      M12p6x13RV +=  + AbV(m15,L)*BbV(m15,L) * ( 1./36.*pow(m55,2) - 1./
         9.*m15*m55 + 1./9.*pow(m15,2) + 1./18.*m11*m55 - 1./9.*m11*m15
          + 1./36.*pow(m11,2) );

      M12p6x13RV +=  + AbV(m15,L)*BbV(m16,L) * ( 1./72.*pow(m66,2) + 1./
         72.*m55*m66 - 1./18.*m16*m66 - 1./36.*m16*m55 + 1./18.*pow(
         m16,2) - 1./36.*m15*m66 + 1./18.*m15*m16 + 1./24.*m11*m66 + 1./
         72.*m11*m55 - 1./12.*m11*m16 - 1./36.*m11*m15 + 1./36.*pow(
         m11,2) );

      M12p6x13RV +=  + AbV(m15,L)*CbV(m11,L) * (  - 1./9.*m11*m55*
         RD1456ep + 2./9.*m11*m15*RD1456ep - 1./9.*pow(m11,2)*RD1456ep
          );

      M12p6x13RV +=  + AbV(m16,L)*pi16 * (  - 1./12.*m11*RD1456ep - 1./
         12.*m11*me2*RSe45611p + 1./6.*m11*me2*RSe451p - 1./12.*m11*me2
         *RSe456p - 1./12.*m11*mp2*RSp45611e + 1./6.*m11*mp2*RSp451e - 
         1./12.*m11*mp2*RSp456e + 1./4.*m11*m66 - 1./12.*m11*m66*
         RS645pe + 1./4.*m11*m56 + 1./4.*m11*m46 - 1./12.*m11*m16*
         RC1456pe - 1./12.*m11*m16*RSe45611p - 1./12.*m11*m16*RSp45611e
          + 1./6.*m11*m16*RS145pe - 1./12.*m11*m16*RS645pe + 1./6.*m11*
         m16*RSe451p + 1./6.*m11*m16*RSp451e - 1./12.*m11*m16*RSe456p
          - 1./12.*m11*m16*RSp456e + 1./4.*m11*m15 + 1./4.*m11*m14 - 1./
         4.*pow(m11,2) - 1./18.*pow(m11,2)*RC1456pe + 1./36.*pow(m11,2)
         *RSe45611p + 1./36.*pow(m11,2)*RSp45611e + 1./9.*pow(m11,2)*
         RS145pe + 1./36.*pow(m11,2)*RS645pe - 1./18.*pow(m11,2)*
         RSe451p - 1./18.*pow(m11,2)*RSp451e + 1./36.*pow(m11,2)*
         RSe456p + 1./36.*pow(m11,2)*RSp456e );

      M12p6x13RV +=  + pow(AbV(m16,L),2) * (  - 1./27.*RD1456ep - 1./27.
         *me2*RSe45611p + 2./27.*me2*RSe451p - 1./27.*me2*RSe456p - 1./
         27.*mp2*RSp45611e + 2./27.*mp2*RSp451e - 1./27.*mp2*RSp456e - 
         1./20.*m66 - 1./27.*m66*RC1456pe - 1./27.*m66*RSe45611p - 1./
         27.*m66*RSp45611e + 1./27.*m66*RSe456p + 1./27.*m66*RSp456e + 
         1./10.*m16 - 3./10.*m11 - 13./108.*m11*RC1456pe - 1./12.*m11*
         RSe45611p - 1./12.*m11*RSp45611e + 5./54.*m11*RS145pe + 7./108.
         *m11*RS645pe + 1./54.*m11*RSe451p + 1./54.*m11*RSp451e + 7./
         108.*m11*RSe456p + 7./108.*m11*RSp456e );

      M12p6x13RV +=  + AbV(m16,L)*AbV(m46,L) * (  - 1./180.*m66 - 2./15.
         *m46 + 7./180.*m44 + 13./90.*m16 + 1./18.*m14 - 1./10.*m11 );

      M12p6x13RV +=  + AbV(m16,L)*AbV(m56,L) * (  - 1./180.*m66 - 2./15.
         *m56 + 7./180.*m55 + 13./90.*m16 + 1./18.*m15 - 1./10.*m11 );

      M12p6x13RV +=  + AbV(m16,L)*AbV(m66,L) * (  - 1./10.*m66 + 1./30.
         *m66*RS645pe + 1./5.*m16 - 1./15.*m16*RS645pe - 1./10.*m11 + 1.
         /30.*m11*RS645pe );

      M12p6x13RV +=  + AbV(m16,L)*AbV(mp2,L) * ( 2./45.*mp2*RSp45611e
          - 4./45.*mp2*RSp451e + 2./45.*mp2*RSp456e + 1./30.*m66*
         RSp45611e - 1./45.*m66*RSp451e - 1./90.*m66*RSp456e - 1./15.*
         m16*RSp45611e + 2./15.*m16*RSp451e - 1./15.*m16*RSp456e + 13./
         30.*m11*RSp45611e - 7./15.*m11*RSp451e + 1./30.*m11*RSp456e );

      M12p6x13RV +=  + AbV(m16,L)*AbV(me2,L) * ( 2./45.*me2*RSe45611p
          - 4./45.*me2*RSe451p + 2./45.*me2*RSe456p + 1./30.*m66*
         RSe45611p - 1./45.*m66*RSe451p - 1./90.*m66*RSe456p - 1./15.*
         m16*RSe45611p + 2./15.*m16*RSe451p - 1./15.*m16*RSe456p + 13./
         30.*m11*RSe45611p - 7./15.*m11*RSe451p + 1./30.*m11*RSe456p );

      M12p6x13RV +=  + AbV(m16,L)*BbV(m11,mp2,L) * (  - 1./9.*m11*m66*
         RSp45611e - 1./9.*m11*m66*RSp451e + 2./9.*m11*m16*RSp45611e - 
         2./9.*m11*m16*RSp451e - 1./9.*pow(m11,2)*RSp45611e - 1./3.*
         pow(m11,2)*RSp451e );

      M12p6x13RV +=  + AbV(m16,L)*BbV(m11,me2,L) * (  - 1./9.*m11*m66*
         RSe45611p - 1./9.*m11*m66*RSe451p + 2./9.*m11*m16*RSe45611p - 
         2./9.*m11*m16*RSe451p - 1./9.*pow(m11,2)*RSe45611p - 1./3.*
         pow(m11,2)*RSe451p );

      M12p6x13RV +=  + AbV(m16,L)*BbV(m11,L) * ( 1./30.*m66*RD1456ep - 
         1./15.*m16*RD1456ep + 43./90.*m11*RD1456ep - 1./9.*m11*m66*
         RC1456pe - 1./9.*m11*m66*RS145pe + 2./9.*m11*m16*RC1456pe - 2./
         9.*m11*m16*RS145pe - 1./9.*pow(m11,2)*RC1456pe - 1./3.*pow(
         m11,2)*RS145pe );

      M12p6x13RV +=  + AbV(m16,L)*BbV(m14,L) * ( 1./72.*m44*m66 + 1./72.
         *pow(m44,2) - 1./36.*m16*m44 - 1./36.*m14*m66 - 1./18.*m14*m44
          + 1./18.*m14*m16 + 1./18.*pow(m14,2) + 1./72.*m11*m66 + 1./24.
         *m11*m44 - 1./36.*m11*m16 - 1./12.*m11*m14 + 1./36.*pow(m11,2)
          );

      M12p6x13RV +=  + AbV(m16,L)*BbV(m15,L) * ( 1./72.*m55*m66 + 1./72.
         *pow(m55,2) - 1./36.*m16*m55 - 1./36.*m15*m66 - 1./18.*m15*m55
          + 1./18.*m15*m16 + 1./18.*pow(m15,2) + 1./72.*m11*m66 + 1./24.
         *m11*m55 - 1./36.*m11*m16 - 1./12.*m11*m15 + 1./36.*pow(m11,2)
          );

      M12p6x13RV +=  + AbV(m16,L)*BbV(m16,L) * ( 1./36.*pow(m66,2) - 1./
         9.*m16*m66 + 1./9.*pow(m16,2) + 1./18.*m11*m66 - 1./9.*m11*m16
          + 1./36.*pow(m11,2) );

      M12p6x13RV +=  + AbV(m16,L)*CbV(m11,L) * (  - 1./9.*m11*m66*
         RD1456ep + 2./9.*m11*m16*RD1456ep - 1./9.*pow(m11,2)*RD1456ep
          );

      M12p6x13RV +=  + AbV(m44,L)*BbV(m11,mp2,L) * ( 2./9.*m11*m44*
         RSp561e*RS156pe - 2./27.*m11*m44*RSp561e*RS456pe*RS156pe );

      M12p6x13RV +=  + AbV(m44,L)*BbV(m11,me2,L) * ( 2./9.*m11*m44*
         RSe561p*RS156pe - 2./27.*m11*m44*RSe561p*RS456pe*RS156pe );

      M12p6x13RV +=  + AbV(m44,L)*BbV(m11,L) * ( 1./9.*m11*m44*pow(
         RS156pe,2) - 1./27.*m11*m44*RS456pe*pow(RS156pe,2) );

      M12p6x13RV +=  + AbV(m44,L)*BbV(m14,L) * ( 1./72.*pow(m44,2) - 1./
         216.*pow(m44,2)*RS456pe - 1./18.*m14*m44 + 1./54.*m14*m44*
         RS456pe + 1./18.*pow(m14,2) - 1./54.*pow(m14,2)*RS456pe + 1./
         36.*m11*m44 - 1./108.*m11*m44*RS456pe - 1./18.*m11*m14 + 1./54.
         *m11*m14*RS456pe + 1./72.*pow(m11,2) - 1./216.*pow(m11,2)*
         RS456pe );

      M12p6x13RV +=  + AbV(m44,L)*BbV(mp2,L) * ( 1./9.*m11*m44*pow(
         RSp561e,2) - 1./27.*m11*m44*pow(RSp561e,2)*RS456pe );

      M12p6x13RV +=  + AbV(m44,L)*BbV(me2,mp2,L) * ( 2./9.*m11*m44*
         RSp561e*RSe561p - 2./27.*m11*m44*RSp561e*RSe561p*RS456pe );

      M12p6x13RV +=  + AbV(m44,L)*BbV(me2,L) * ( 1./9.*m11*m44*pow(
         RSe561p,2) - 1./27.*m11*m44*pow(RSe561p,2)*RS456pe );

      M12p6x13RV +=  + AbV(m45,L)*AbV(mp2,L) * ( 2./27.*m11*RSp561e*
         RS146pe - 2./27.*m11*RSp561e*RS156pe - 2./27.*m11*RSp561e*
         RSe561p + 2./27.*m11*RSp561e*RSe461p - 1./27.*m11*pow(
         RSp561e,2) - 2./27.*m11*RSp461e*RS146pe + 2./27.*m11*RSp461e*
         RS156pe + 2./27.*m11*RSp461e*RSe561p - 2./27.*m11*RSp461e*
         RSe461p + 2./27.*m11*RSp461e*RSp561e - 1./27.*m11*pow(
         RSp461e,2) );

      M12p6x13RV +=  + AbV(m45,L)*AbV(me2,L) * ( 2./27.*m11*RSe561p*
         RS146pe - 2./27.*m11*RSe561p*RS156pe - 1./27.*m11*pow(
         RSe561p,2) - 2./27.*m11*RSe461p*RS146pe + 2./27.*m11*RSe461p*
         RS156pe + 2./27.*m11*RSe461p*RSe561p - 1./27.*m11*pow(
         RSe461p,2) );

      M12p6x13RV +=  + AbV(m45,L)*BbV(m11,mp2,L) * ( 1./27.*m11*m55*
         RSp561e*RS146pe + 1./27.*m11*m55*RSp561e*RS156pe + 1./9.*m11*
         m55*RSp461e*RS146pe + 1./27.*m11*m55*RSp461e*RS156pe + 2./27.*
         m11*m45*RSp561e*RS146pe - 2./27.*m11*m45*RSp561e*RS156pe - 2./
         27.*m11*m45*RSp461e*RS146pe + 2./27.*m11*m45*RSp461e*RS156pe
          + 1./27.*m11*m44*RSp561e*RS146pe + 1./9.*m11*m44*RSp561e*
         RS156pe + 1./27.*m11*m44*RSp461e*RS146pe + 1./27.*m11*m44*
         RSp461e*RS156pe + 2./27.*pow(m11,2)*RSp561e*RS146pe - 2./27.*
         pow(m11,2)*RSp561e*RS156pe - 2./27.*pow(m11,2)*RSp461e*RS146pe
          + 2./27.*pow(m11,2)*RSp461e*RS156pe );

      M12p6x13RV +=  + AbV(m45,L)*BbV(m11,me2,L) * ( 1./27.*m11*m55*
         RSe561p*RS146pe + 1./27.*m11*m55*RSe561p*RS156pe + 1./9.*m11*
         m55*RSe461p*RS146pe + 1./27.*m11*m55*RSe461p*RS156pe + 2./27.*
         m11*m45*RSe561p*RS146pe - 2./27.*m11*m45*RSe561p*RS156pe - 2./
         27.*m11*m45*RSe461p*RS146pe + 2./27.*m11*m45*RSe461p*RS156pe
          + 1./27.*m11*m44*RSe561p*RS146pe + 1./9.*m11*m44*RSe561p*
         RS156pe + 1./27.*m11*m44*RSe461p*RS146pe + 1./27.*m11*m44*
         RSe461p*RS156pe + 2./27.*pow(m11,2)*RSe561p*RS146pe - 2./27.*
         pow(m11,2)*RSe561p*RS156pe - 2./27.*pow(m11,2)*RSe461p*RS146pe
          + 2./27.*pow(m11,2)*RSe461p*RS156pe );

      M12p6x13RV +=  + AbV(m45,L)*BbV(m11,L) * ( 1./18.*m11*m55*pow(
         RS146pe,2) + 1./27.*m11*m55*RS156pe*RS146pe + 1./54.*m11*m55*
         pow(RS156pe,2) - 1./27.*m11*m45*pow(RS146pe,2) + 2./27.*m11*
         m45*RS156pe*RS146pe - 1./27.*m11*m45*pow(RS156pe,2) + 1./54.*
         m11*m44*pow(RS146pe,2) + 1./27.*m11*m44*RS156pe*RS146pe + 1./
         18.*m11*m44*pow(RS156pe,2) - 1./27.*pow(m11,2)*pow(RS146pe,2)
          + 2./27.*pow(m11,2)*RS156pe*RS146pe - 1./27.*pow(m11,2)*pow(
         RS156pe,2) );

      M12p6x13RV +=  + AbV(m45,L)*BbV(m14,L) * ( 1./72.*m44*m55 - 1./36.
         *m44*m45 + 1./36.*pow(m44,2) - 1./36.*m14*m55 + 1./18.*m14*m45
          - 1./12.*m14*m44 + 1./18.*pow(m14,2) + 1./72.*m11*m55 - 1./36.
         *m11*m45 + 1./24.*m11*m44 - 1./18.*m11*m14 + 1./72.*pow(m11,2)
          );

      M12p6x13RV +=  + AbV(m45,L)*BbV(m15,L) * ( 1./36.*pow(m55,2) - 1./
         36.*m45*m55 + 1./72.*m44*m55 - 1./12.*m15*m55 + 1./18.*m15*m45
          - 1./36.*m15*m44 + 1./18.*pow(m15,2) + 1./24.*m11*m55 - 1./36.
         *m11*m45 + 1./72.*m11*m44 - 1./18.*m11*m15 + 1./72.*pow(m11,2)
          );

      M12p6x13RV +=  + AbV(m45,L)*BbV(mp2,L) * (  - 1./27.*m11*mp2*pow(
         RSp561e,2) + 2./27.*m11*mp2*RSp461e*RSp561e - 1./27.*m11*mp2*
         pow(RSp461e,2) + 1./54.*m11*m55*pow(RSp561e,2) + 1./27.*m11*
         m55*RSp461e*RSp561e + 1./18.*m11*m55*pow(RSp461e,2) - 1./27.*
         m11*m45*pow(RSp561e,2) + 2./27.*m11*m45*RSp461e*RSp561e - 1./
         27.*m11*m45*pow(RSp461e,2) + 1./18.*m11*m44*pow(RSp561e,2) + 1.
         /27.*m11*m44*RSp461e*RSp561e + 1./54.*m11*m44*pow(RSp461e,2) )
         ;

      M12p6x13RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * (  - 2./27.*m11*me2*
         RSp561e*RSe561p + 2./27.*m11*me2*RSp561e*RSe461p + 2./27.*m11*
         me2*RSp461e*RSe561p - 2./27.*m11*me2*RSp461e*RSe461p + 1./27.*
         m11*m55*RSp561e*RSe561p + 1./27.*m11*m55*RSp561e*RSe461p + 1./
         27.*m11*m55*RSp461e*RSe561p + 1./9.*m11*m55*RSp461e*RSe461p - 
         2./27.*m11*m45*RSp561e*RSe561p + 2./27.*m11*m45*RSp561e*
         RSe461p + 2./27.*m11*m45*RSp461e*RSe561p - 2./27.*m11*m45*
         RSp461e*RSe461p + 1./9.*m11*m44*RSp561e*RSe561p + 1./27.*m11*
         m44*RSp561e*RSe461p + 1./27.*m11*m44*RSp461e*RSe561p + 1./27.*
         m11*m44*RSp461e*RSe461p );

      M12p6x13RV +=  + AbV(m45,L)*BbV(me2,L) * (  - 1./27.*m11*me2*pow(
         RSe561p,2) + 2./27.*m11*me2*RSe461p*RSe561p - 1./27.*m11*me2*
         pow(RSe461p,2) + 1./54.*m11*m55*pow(RSe561p,2) + 1./27.*m11*
         m55*RSe461p*RSe561p + 1./18.*m11*m55*pow(RSe461p,2) - 1./27.*
         m11*m45*pow(RSe561p,2) + 2./27.*m11*m45*RSe461p*RSe561p - 1./
         27.*m11*m45*pow(RSe461p,2) + 1./18.*m11*m44*pow(RSe561p,2) + 1.
         /27.*m11*m44*RSe461p*RSe561p + 1./54.*m11*m44*pow(RSe461p,2) )
         ;

      M12p6x13RV +=  + AbV(m46,L)*AbV(mp2,L) * ( 2./27.*m11*RSp561e*
         RS145pe - 2./27.*m11*RSp561e*RS156pe - 2./27.*m11*RSp561e*
         RSe561p + 2./27.*m11*RSp561e*RSe451p - 1./27.*m11*pow(
         RSp561e,2) - 2./27.*m11*RSp451e*RS145pe + 2./27.*m11*RSp451e*
         RS156pe + 2./27.*m11*RSp451e*RSe561p - 2./27.*m11*RSp451e*
         RSe451p + 2./27.*m11*RSp451e*RSp561e - 1./27.*m11*pow(
         RSp451e,2) );

      M12p6x13RV +=  + AbV(m46,L)*AbV(me2,L) * ( 2./27.*m11*RSe561p*
         RS145pe - 2./27.*m11*RSe561p*RS156pe - 1./27.*m11*pow(
         RSe561p,2) - 2./27.*m11*RSe451p*RS145pe + 2./27.*m11*RSe451p*
         RS156pe + 2./27.*m11*RSe451p*RSe561p - 1./27.*m11*pow(
         RSe451p,2) );

      M12p6x13RV +=  + AbV(m46,L)*BbV(m11,mp2,L) * ( 1./27.*m11*m66*
         RSp561e*RS145pe + 1./27.*m11*m66*RSp561e*RS156pe + 1./9.*m11*
         m66*RSp451e*RS145pe + 1./27.*m11*m66*RSp451e*RS156pe + 2./27.*
         m11*m46*RSp561e*RS145pe - 2./27.*m11*m46*RSp561e*RS156pe - 2./
         27.*m11*m46*RSp451e*RS145pe + 2./27.*m11*m46*RSp451e*RS156pe
          + 1./27.*m11*m44*RSp561e*RS145pe + 1./9.*m11*m44*RSp561e*
         RS156pe + 1./27.*m11*m44*RSp451e*RS145pe + 1./27.*m11*m44*
         RSp451e*RS156pe + 2./27.*pow(m11,2)*RSp561e*RS145pe - 2./27.*
         pow(m11,2)*RSp561e*RS156pe - 2./27.*pow(m11,2)*RSp451e*RS145pe
          + 2./27.*pow(m11,2)*RSp451e*RS156pe );

      M12p6x13RV +=  + AbV(m46,L)*BbV(m11,me2,L) * ( 1./27.*m11*m66*
         RSe561p*RS145pe + 1./27.*m11*m66*RSe561p*RS156pe + 1./9.*m11*
         m66*RSe451p*RS145pe + 1./27.*m11*m66*RSe451p*RS156pe + 2./27.*
         m11*m46*RSe561p*RS145pe - 2./27.*m11*m46*RSe561p*RS156pe - 2./
         27.*m11*m46*RSe451p*RS145pe + 2./27.*m11*m46*RSe451p*RS156pe
          + 1./27.*m11*m44*RSe561p*RS145pe + 1./9.*m11*m44*RSe561p*
         RS156pe + 1./27.*m11*m44*RSe451p*RS145pe + 1./27.*m11*m44*
         RSe451p*RS156pe + 2./27.*pow(m11,2)*RSe561p*RS145pe - 2./27.*
         pow(m11,2)*RSe561p*RS156pe - 2./27.*pow(m11,2)*RSe451p*RS145pe
          + 2./27.*pow(m11,2)*RSe451p*RS156pe );

      M12p6x13RV +=  + AbV(m46,L)*BbV(m11,L) * ( 1./18.*m11*m66*pow(
         RS145pe,2) + 1./27.*m11*m66*RS156pe*RS145pe + 1./54.*m11*m66*
         pow(RS156pe,2) - 1./27.*m11*m46*pow(RS145pe,2) + 2./27.*m11*
         m46*RS156pe*RS145pe - 1./27.*m11*m46*pow(RS156pe,2) + 1./54.*
         m11*m44*pow(RS145pe,2) + 1./27.*m11*m44*RS156pe*RS145pe + 1./
         18.*m11*m44*pow(RS156pe,2) - 1./27.*pow(m11,2)*pow(RS145pe,2)
          + 2./27.*pow(m11,2)*RS156pe*RS145pe - 1./27.*pow(m11,2)*pow(
         RS156pe,2) );

      M12p6x13RV +=  + AbV(m46,L)*BbV(m14,L) * ( 1./72.*m44*m66 - 1./36.
         *m44*m46 + 1./36.*pow(m44,2) - 1./36.*m14*m66 + 1./18.*m14*m46
          - 1./12.*m14*m44 + 1./18.*pow(m14,2) + 1./72.*m11*m66 - 1./36.
         *m11*m46 + 1./24.*m11*m44 - 1./18.*m11*m14 + 1./72.*pow(m11,2)
          );

      M12p6x13RV +=  + AbV(m46,L)*BbV(m16,L) * ( 1./36.*pow(m66,2) - 1./
         36.*m46*m66 + 1./72.*m44*m66 - 1./12.*m16*m66 + 1./18.*m16*m46
          - 1./36.*m16*m44 + 1./18.*pow(m16,2) + 1./24.*m11*m66 - 1./36.
         *m11*m46 + 1./72.*m11*m44 - 1./18.*m11*m16 + 1./72.*pow(m11,2)
          );

      M12p6x13RV +=  + AbV(m46,L)*BbV(mp2,L) * (  - 1./27.*m11*mp2*pow(
         RSp561e,2) + 2./27.*m11*mp2*RSp451e*RSp561e - 1./27.*m11*mp2*
         pow(RSp451e,2) + 1./54.*m11*m66*pow(RSp561e,2) + 1./27.*m11*
         m66*RSp451e*RSp561e + 1./18.*m11*m66*pow(RSp451e,2) - 1./27.*
         m11*m46*pow(RSp561e,2) + 2./27.*m11*m46*RSp451e*RSp561e - 1./
         27.*m11*m46*pow(RSp451e,2) + 1./18.*m11*m44*pow(RSp561e,2) + 1.
         /27.*m11*m44*RSp451e*RSp561e + 1./54.*m11*m44*pow(RSp451e,2) )
         ;

      M12p6x13RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * (  - 2./27.*m11*me2*
         RSp561e*RSe561p + 2./27.*m11*me2*RSp561e*RSe451p + 2./27.*m11*
         me2*RSp451e*RSe561p - 2./27.*m11*me2*RSp451e*RSe451p + 1./27.*
         m11*m66*RSp561e*RSe561p + 1./27.*m11*m66*RSp561e*RSe451p + 1./
         27.*m11*m66*RSp451e*RSe561p + 1./9.*m11*m66*RSp451e*RSe451p - 
         2./27.*m11*m46*RSp561e*RSe561p + 2./27.*m11*m46*RSp561e*
         RSe451p + 2./27.*m11*m46*RSp451e*RSe561p - 2./27.*m11*m46*
         RSp451e*RSe451p + 1./9.*m11*m44*RSp561e*RSe561p + 1./27.*m11*
         m44*RSp561e*RSe451p + 1./27.*m11*m44*RSp451e*RSe561p + 1./27.*
         m11*m44*RSp451e*RSe451p );

      M12p6x13RV +=  + AbV(m46,L)*BbV(me2,L) * (  - 1./27.*m11*me2*pow(
         RSe561p,2) + 2./27.*m11*me2*RSe451p*RSe561p - 1./27.*m11*me2*
         pow(RSe451p,2) + 1./54.*m11*m66*pow(RSe561p,2) + 1./27.*m11*
         m66*RSe451p*RSe561p + 1./18.*m11*m66*pow(RSe451p,2) - 1./27.*
         m11*m46*pow(RSe561p,2) + 2./27.*m11*m46*RSe451p*RSe561p - 1./
         27.*m11*m46*pow(RSe451p,2) + 1./18.*m11*m44*pow(RSe561p,2) + 1.
         /27.*m11*m44*RSe451p*RSe561p + 1./54.*m11*m44*pow(RSe451p,2) )
         ;

      M12p6x13RV +=  + AbV(m55,L)*BbV(m11,mp2,L) * ( 2./9.*m11*m55*
         RSp461e*RS146pe - 2./27.*m11*m55*RSp461e*RS546pe*RS146pe );

      M12p6x13RV +=  + AbV(m55,L)*BbV(m11,me2,L) * ( 2./9.*m11*m55*
         RSe461p*RS146pe - 2./27.*m11*m55*RSe461p*RS546pe*RS146pe );

      M12p6x13RV +=  + AbV(m55,L)*BbV(m11,L) * ( 1./9.*m11*m55*pow(
         RS146pe,2) - 1./27.*m11*m55*RS546pe*pow(RS146pe,2) );

      M12p6x13RV +=  + AbV(m55,L)*BbV(m15,L) * ( 1./72.*pow(m55,2) - 1./
         216.*pow(m55,2)*RS546pe - 1./18.*m15*m55 + 1./54.*m15*m55*
         RS546pe + 1./18.*pow(m15,2) - 1./54.*pow(m15,2)*RS546pe + 1./
         36.*m11*m55 - 1./108.*m11*m55*RS546pe - 1./18.*m11*m15 + 1./54.
         *m11*m15*RS546pe + 1./72.*pow(m11,2) - 1./216.*pow(m11,2)*
         RS546pe );

      M12p6x13RV +=  + AbV(m55,L)*BbV(mp2,L) * ( 1./9.*m11*m55*pow(
         RSp461e,2) - 1./27.*m11*m55*pow(RSp461e,2)*RS546pe );

      M12p6x13RV +=  + AbV(m55,L)*BbV(me2,mp2,L) * ( 2./9.*m11*m55*
         RSp461e*RSe461p - 2./27.*m11*m55*RSp461e*RSe461p*RS546pe );

      M12p6x13RV +=  + AbV(m55,L)*BbV(me2,L) * ( 1./9.*m11*m55*pow(
         RSe461p,2) - 1./27.*m11*m55*pow(RSe461p,2)*RS546pe );

      M12p6x13RV +=  + AbV(m56,L)*AbV(mp2,L) * ( 2./27.*m11*RSp461e*
         RS145pe - 2./27.*m11*RSp461e*RS146pe - 2./27.*m11*RSp461e*
         RSe461p + 2./27.*m11*RSp461e*RSe451p - 1./27.*m11*pow(
         RSp461e,2) - 2./27.*m11*RSp451e*RS145pe + 2./27.*m11*RSp451e*
         RS146pe + 2./27.*m11*RSp451e*RSe461p - 2./27.*m11*RSp451e*
         RSe451p + 2./27.*m11*RSp451e*RSp461e - 1./27.*m11*pow(
         RSp451e,2) );

      M12p6x13RV +=  + AbV(m56,L)*AbV(me2,L) * ( 2./27.*m11*RSe461p*
         RS145pe - 2./27.*m11*RSe461p*RS146pe - 1./27.*m11*pow(
         RSe461p,2) - 2./27.*m11*RSe451p*RS145pe + 2./27.*m11*RSe451p*
         RS146pe + 2./27.*m11*RSe451p*RSe461p - 1./27.*m11*pow(
         RSe451p,2) );

      M12p6x13RV +=  + AbV(m56,L)*BbV(m11,mp2,L) * ( 1./27.*m11*m66*
         RSp461e*RS145pe + 1./27.*m11*m66*RSp461e*RS146pe + 1./9.*m11*
         m66*RSp451e*RS145pe + 1./27.*m11*m66*RSp451e*RS146pe + 2./27.*
         m11*m56*RSp461e*RS145pe - 2./27.*m11*m56*RSp461e*RS146pe - 2./
         27.*m11*m56*RSp451e*RS145pe + 2./27.*m11*m56*RSp451e*RS146pe
          + 1./27.*m11*m55*RSp461e*RS145pe + 1./9.*m11*m55*RSp461e*
         RS146pe + 1./27.*m11*m55*RSp451e*RS145pe + 1./27.*m11*m55*
         RSp451e*RS146pe + 2./27.*pow(m11,2)*RSp461e*RS145pe - 2./27.*
         pow(m11,2)*RSp461e*RS146pe - 2./27.*pow(m11,2)*RSp451e*RS145pe
          + 2./27.*pow(m11,2)*RSp451e*RS146pe );

      M12p6x13RV +=  + AbV(m56,L)*BbV(m11,me2,L) * ( 1./27.*m11*m66*
         RSe461p*RS145pe + 1./27.*m11*m66*RSe461p*RS146pe + 1./9.*m11*
         m66*RSe451p*RS145pe + 1./27.*m11*m66*RSe451p*RS146pe + 2./27.*
         m11*m56*RSe461p*RS145pe - 2./27.*m11*m56*RSe461p*RS146pe - 2./
         27.*m11*m56*RSe451p*RS145pe + 2./27.*m11*m56*RSe451p*RS146pe
          + 1./27.*m11*m55*RSe461p*RS145pe + 1./9.*m11*m55*RSe461p*
         RS146pe + 1./27.*m11*m55*RSe451p*RS145pe + 1./27.*m11*m55*
         RSe451p*RS146pe + 2./27.*pow(m11,2)*RSe461p*RS145pe - 2./27.*
         pow(m11,2)*RSe461p*RS146pe - 2./27.*pow(m11,2)*RSe451p*RS145pe
          + 2./27.*pow(m11,2)*RSe451p*RS146pe );

      M12p6x13RV +=  + AbV(m56,L)*BbV(m11,L) * ( 1./18.*m11*m66*pow(
         RS145pe,2) + 1./27.*m11*m66*RS146pe*RS145pe + 1./54.*m11*m66*
         pow(RS146pe,2) - 1./27.*m11*m56*pow(RS145pe,2) + 2./27.*m11*
         m56*RS146pe*RS145pe - 1./27.*m11*m56*pow(RS146pe,2) + 1./54.*
         m11*m55*pow(RS145pe,2) + 1./27.*m11*m55*RS146pe*RS145pe + 1./
         18.*m11*m55*pow(RS146pe,2) - 1./27.*pow(m11,2)*pow(RS145pe,2)
          + 2./27.*pow(m11,2)*RS146pe*RS145pe - 1./27.*pow(m11,2)*pow(
         RS146pe,2) );

      M12p6x13RV +=  + AbV(m56,L)*BbV(m15,L) * ( 1./72.*m55*m66 - 1./36.
         *m55*m56 + 1./36.*pow(m55,2) - 1./36.*m15*m66 + 1./18.*m15*m56
          - 1./12.*m15*m55 + 1./18.*pow(m15,2) + 1./72.*m11*m66 - 1./36.
         *m11*m56 + 1./24.*m11*m55 - 1./18.*m11*m15 + 1./72.*pow(m11,2)
          );

      M12p6x13RV +=  + AbV(m56,L)*BbV(m16,L) * ( 1./36.*pow(m66,2) - 1./
         36.*m56*m66 + 1./72.*m55*m66 - 1./12.*m16*m66 + 1./18.*m16*m56
          - 1./36.*m16*m55 + 1./18.*pow(m16,2) + 1./24.*m11*m66 - 1./36.
         *m11*m56 + 1./72.*m11*m55 - 1./18.*m11*m16 + 1./72.*pow(m11,2)
          );

      M12p6x13RV +=  + AbV(m56,L)*BbV(mp2,L) * (  - 1./27.*m11*mp2*pow(
         RSp461e,2) + 2./27.*m11*mp2*RSp451e*RSp461e - 1./27.*m11*mp2*
         pow(RSp451e,2) + 1./54.*m11*m66*pow(RSp461e,2) + 1./27.*m11*
         m66*RSp451e*RSp461e + 1./18.*m11*m66*pow(RSp451e,2) - 1./27.*
         m11*m56*pow(RSp461e,2) + 2./27.*m11*m56*RSp451e*RSp461e - 1./
         27.*m11*m56*pow(RSp451e,2) + 1./18.*m11*m55*pow(RSp461e,2) + 1.
         /27.*m11*m55*RSp451e*RSp461e + 1./54.*m11*m55*pow(RSp451e,2) )
         ;

      M12p6x13RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * (  - 2./27.*m11*me2*
         RSp461e*RSe461p + 2./27.*m11*me2*RSp461e*RSe451p + 2./27.*m11*
         me2*RSp451e*RSe461p - 2./27.*m11*me2*RSp451e*RSe451p + 1./27.*
         m11*m66*RSp461e*RSe461p + 1./27.*m11*m66*RSp461e*RSe451p + 1./
         27.*m11*m66*RSp451e*RSe461p + 1./9.*m11*m66*RSp451e*RSe451p - 
         2./27.*m11*m56*RSp461e*RSe461p + 2./27.*m11*m56*RSp461e*
         RSe451p + 2./27.*m11*m56*RSp451e*RSe461p - 2./27.*m11*m56*
         RSp451e*RSe451p + 1./9.*m11*m55*RSp461e*RSe461p + 1./27.*m11*
         m55*RSp461e*RSe451p + 1./27.*m11*m55*RSp451e*RSe461p + 1./27.*
         m11*m55*RSp451e*RSe451p );

      M12p6x13RV +=  + AbV(m56,L)*BbV(me2,L) * (  - 1./27.*m11*me2*pow(
         RSe461p,2) + 2./27.*m11*me2*RSe451p*RSe461p - 1./27.*m11*me2*
         pow(RSe451p,2) + 1./54.*m11*m66*pow(RSe461p,2) + 1./27.*m11*
         m66*RSe451p*RSe461p + 1./18.*m11*m66*pow(RSe451p,2) - 1./27.*
         m11*m56*pow(RSe461p,2) + 2./27.*m11*m56*RSe451p*RSe461p - 1./
         27.*m11*m56*pow(RSe451p,2) + 1./18.*m11*m55*pow(RSe461p,2) + 1.
         /27.*m11*m55*RSe451p*RSe461p + 1./54.*m11*m55*pow(RSe451p,2) )
         ;

      M12p6x13RV +=  + AbV(m66,L)*BbV(m11,mp2,L) * ( 2./9.*m11*m66*
         RSp451e*RS145pe - 2./27.*m11*m66*RSp451e*RS645pe*RS145pe );

      M12p6x13RV +=  + AbV(m66,L)*BbV(m11,me2,L) * ( 2./9.*m11*m66*
         RSe451p*RS145pe - 2./27.*m11*m66*RSe451p*RS645pe*RS145pe );

      M12p6x13RV +=  + AbV(m66,L)*BbV(m11,L) * ( 1./9.*m11*m66*pow(
         RS145pe,2) - 1./27.*m11*m66*RS645pe*pow(RS145pe,2) );

      M12p6x13RV +=  + AbV(m66,L)*BbV(m16,L) * ( 1./72.*pow(m66,2) - 1./
         216.*pow(m66,2)*RS645pe - 1./18.*m16*m66 + 1./54.*m16*m66*
         RS645pe + 1./18.*pow(m16,2) - 1./54.*pow(m16,2)*RS645pe + 1./
         36.*m11*m66 - 1./108.*m11*m66*RS645pe - 1./18.*m11*m16 + 1./54.
         *m11*m16*RS645pe + 1./72.*pow(m11,2) - 1./216.*pow(m11,2)*
         RS645pe );

      M12p6x13RV +=  + AbV(m66,L)*BbV(mp2,L) * ( 1./9.*m11*m66*pow(
         RSp451e,2) - 1./27.*m11*m66*pow(RSp451e,2)*RS645pe );

      M12p6x13RV +=  + AbV(m66,L)*BbV(me2,mp2,L) * ( 2./9.*m11*m66*
         RSp451e*RSe451p - 2./27.*m11*m66*RSp451e*RSe451p*RS645pe );

      M12p6x13RV +=  + AbV(m66,L)*BbV(me2,L) * ( 1./9.*m11*m66*pow(
         RSe451p,2) - 1./27.*m11*m66*pow(RSe451p,2)*RS645pe );

      M12p6x13RV +=  + AbV(mp2,L)*pi16 * ( 1./12.*m11*mp2*RSp561e + 1./
         12.*m11*mp2*RSp461e + 1./12.*m11*mp2*RSp451e );

      M12p6x13RV +=  + pow(AbV(mp2,L),2) * ( 1./18.*m11*pow(
         RSp45611e,2) );

      M12p6x13RV +=  + AbV(mp2,L)*AbV(me2,L) * ( 1./9.*m11*RSp45611e*
         RSe45611p );

      M12p6x13RV +=  + AbV(mp2,L)*BbV(m11,mp2,L) * (  - 2./27.*m11*m66*
         RSp456e*RSp451e*RS145pe - 2./27.*m11*m55*RSp465e*RSp461e*
         RS146pe - 2./27.*m11*m44*RSp564e*RSp561e*RS156pe + 2./9.*pow(
         m11,2)*pow(RSp45611e,2) );

      M12p6x13RV +=  + AbV(mp2,L)*BbV(m11,me2,L) * (  - 2./27.*m11*m66*
         RSp456e*RSe451p*RS145pe - 2./27.*m11*m55*RSp465e*RSe461p*
         RS146pe - 2./27.*m11*m44*RSp564e*RSe561p*RS156pe + 2./9.*pow(
         m11,2)*RSp45611e*RSe45611p );

      M12p6x13RV +=  + AbV(mp2,L)*BbV(m11,L) * ( 1./9.*m11*RSp45611e*
         RD1456ep - 1./27.*m11*m66*RSp456e*pow(RS145pe,2) - 1./27.*m11*
         m55*RSp465e*pow(RS146pe,2) - 1./27.*m11*m44*RSp564e*pow(
         RS156pe,2) + 2./9.*pow(m11,2)*RSp45611e*RC1456pe );

      M12p6x13RV +=  + AbV(mp2,L)*BbV(m14,L) * ( 1./108.*m44*mp2*
         RSp45611e - 1./54.*m44*mp2*RSp561e + 1./108.*m44*mp2*RSp564e
          - 1./216.*pow(m44,2)*RSp45611e - 1./108.*pow(m44,2)*RSp561e
          - 1./72.*pow(m44,2)*RSp564e - 1./54.*m14*mp2*RSp45611e + 1./
         27.*m14*mp2*RSp561e - 1./54.*m14*mp2*RSp564e + 1./54.*m14*m44*
         RSp45611e + 1./27.*m14*m44*RSp564e - 1./54.*pow(m14,2)*
         RSp45611e + 1./27.*pow(m14,2)*RSp561e - 1./54.*pow(m14,2)*
         RSp564e + 1./108.*m11*mp2*RSp45611e - 1./54.*m11*mp2*RSp561e
          + 1./108.*m11*mp2*RSp564e - 1./54.*m11*m44*RSp45611e - 1./54.
         *m11*m44*RSp561e - 1./54.*m11*m44*RSp564e + 1./27.*m11*m14*
         RSp45611e + 1./54.*m11*m14*RSp564e - 1./72.*pow(m11,2)*
         RSp45611e - 1./108.*pow(m11,2)*RSp561e - 1./216.*pow(m11,2)*
         RSp564e );

      M12p6x13RV +=  + AbV(mp2,L)*BbV(m15,L) * ( 1./108.*m55*mp2*
         RSp45611e - 1./54.*m55*mp2*RSp461e + 1./108.*m55*mp2*RSp465e
          - 1./216.*pow(m55,2)*RSp45611e - 1./108.*pow(m55,2)*RSp461e
          - 1./72.*pow(m55,2)*RSp465e - 1./54.*m15*mp2*RSp45611e + 1./
         27.*m15*mp2*RSp461e - 1./54.*m15*mp2*RSp465e + 1./54.*m15*m55*
         RSp45611e + 1./27.*m15*m55*RSp465e - 1./54.*pow(m15,2)*
         RSp45611e + 1./27.*pow(m15,2)*RSp461e - 1./54.*pow(m15,2)*
         RSp465e + 1./108.*m11*mp2*RSp45611e - 1./54.*m11*mp2*RSp461e
          + 1./108.*m11*mp2*RSp465e - 1./54.*m11*m55*RSp45611e - 1./54.
         *m11*m55*RSp461e - 1./54.*m11*m55*RSp465e + 1./27.*m11*m15*
         RSp45611e + 1./54.*m11*m15*RSp465e - 1./72.*pow(m11,2)*
         RSp45611e - 1./108.*pow(m11,2)*RSp461e - 1./216.*pow(m11,2)*
         RSp465e );

      M12p6x13RV +=  + AbV(mp2,L)*BbV(m16,L) * ( 1./108.*m66*mp2*
         RSp45611e - 1./54.*m66*mp2*RSp451e + 1./108.*m66*mp2*RSp456e
          - 1./216.*pow(m66,2)*RSp45611e - 1./108.*pow(m66,2)*RSp451e
          - 1./72.*pow(m66,2)*RSp456e - 1./54.*m16*mp2*RSp45611e + 1./
         27.*m16*mp2*RSp451e - 1./54.*m16*mp2*RSp456e + 1./54.*m16*m66*
         RSp45611e + 1./27.*m16*m66*RSp456e - 1./54.*pow(m16,2)*
         RSp45611e + 1./27.*pow(m16,2)*RSp451e - 1./54.*pow(m16,2)*
         RSp456e + 1./108.*m11*mp2*RSp45611e - 1./54.*m11*mp2*RSp451e
          + 1./108.*m11*mp2*RSp456e - 1./54.*m11*m66*RSp45611e - 1./54.
         *m11*m66*RSp451e - 1./54.*m11*m66*RSp456e + 1./27.*m11*m16*
         RSp45611e + 1./54.*m11*m16*RSp456e - 1./72.*pow(m11,2)*
         RSp45611e - 1./108.*pow(m11,2)*RSp451e - 1./216.*pow(m11,2)*
         RSp456e );

      M12p6x13RV +=  + AbV(mp2,L)*BbV(mp2,L) * (  - 1./27.*m11*m66*
         RSp456e*pow(RSp451e,2) - 1./27.*m11*m55*RSp465e*pow(RSp461e,2)
          - 1./27.*m11*m44*RSp564e*pow(RSp561e,2) );

      M12p6x13RV +=  + AbV(mp2,L)*BbV(me2,mp2,L) * (  - 2./27.*m11*m66*
         RSp456e*RSp451e*RSe451p - 2./27.*m11*m55*RSp465e*RSp461e*
         RSe461p - 2./27.*m11*m44*RSp564e*RSp561e*RSe561p );

      M12p6x13RV +=  + AbV(mp2,L)*BbV(me2,L) * (  - 1./27.*m11*m66*
         RSp456e*pow(RSe451p,2) - 1./27.*m11*m55*RSp465e*pow(RSe461p,2)
          - 1./27.*m11*m44*RSp564e*pow(RSe561p,2) );

      M12p6x13RV +=  + AbV(mp2,L)*CbV(m11,L) * ( 2./9.*pow(m11,2)*
         RSp45611e*RD1456ep );

      M12p6x13RV +=  + AbV(me2,L)*pi16 * ( 1./12.*m11*me2*RSe561p + 1./
         12.*m11*me2*RSe461p + 1./12.*m11*me2*RSe451p );

      M12p6x13RV +=  + pow(AbV(me2,L),2) * ( 1./18.*m11*pow(
         RSe45611p,2) );

      M12p6x13RV +=  + AbV(me2,L)*BbV(m11,mp2,L) * (  - 2./27.*m11*m66*
         RSe456p*RSp451e*RS145pe - 2./27.*m11*m55*RSe465p*RSp461e*
         RS146pe - 2./27.*m11*m44*RSe564p*RSp561e*RS156pe + 2./9.*pow(
         m11,2)*RSp45611e*RSe45611p );

      M12p6x13RV +=  + AbV(me2,L)*BbV(m11,me2,L) * (  - 2./27.*m11*m66*
         RSe456p*RSe451p*RS145pe - 2./27.*m11*m55*RSe465p*RSe461p*
         RS146pe - 2./27.*m11*m44*RSe564p*RSe561p*RS156pe + 2./9.*pow(
         m11,2)*pow(RSe45611p,2) );

      M12p6x13RV +=  + AbV(me2,L)*BbV(m11,L) * ( 1./9.*m11*RSe45611p*
         RD1456ep - 1./27.*m11*m66*RSe456p*pow(RS145pe,2) - 1./27.*m11*
         m55*RSe465p*pow(RS146pe,2) - 1./27.*m11*m44*RSe564p*pow(
         RS156pe,2) + 2./9.*pow(m11,2)*RSe45611p*RC1456pe );

      M12p6x13RV +=  + AbV(me2,L)*BbV(m14,L) * ( 1./108.*m44*me2*
         RSe45611p - 1./54.*m44*me2*RSe561p + 1./108.*m44*me2*RSe564p
          - 1./216.*pow(m44,2)*RSe45611p - 1./108.*pow(m44,2)*RSe561p
          - 1./72.*pow(m44,2)*RSe564p - 1./54.*m14*me2*RSe45611p + 1./
         27.*m14*me2*RSe561p - 1./54.*m14*me2*RSe564p + 1./54.*m14*m44*
         RSe45611p + 1./27.*m14*m44*RSe564p - 1./54.*pow(m14,2)*
         RSe45611p + 1./27.*pow(m14,2)*RSe561p - 1./54.*pow(m14,2)*
         RSe564p + 1./108.*m11*me2*RSe45611p - 1./54.*m11*me2*RSe561p
          + 1./108.*m11*me2*RSe564p - 1./54.*m11*m44*RSe45611p - 1./54.
         *m11*m44*RSe561p - 1./54.*m11*m44*RSe564p + 1./27.*m11*m14*
         RSe45611p + 1./54.*m11*m14*RSe564p - 1./72.*pow(m11,2)*
         RSe45611p - 1./108.*pow(m11,2)*RSe561p - 1./216.*pow(m11,2)*
         RSe564p );

      M12p6x13RV +=  + AbV(me2,L)*BbV(m15,L) * ( 1./108.*m55*me2*
         RSe45611p - 1./54.*m55*me2*RSe461p + 1./108.*m55*me2*RSe465p
          - 1./216.*pow(m55,2)*RSe45611p - 1./108.*pow(m55,2)*RSe461p
          - 1./72.*pow(m55,2)*RSe465p - 1./54.*m15*me2*RSe45611p + 1./
         27.*m15*me2*RSe461p - 1./54.*m15*me2*RSe465p + 1./54.*m15*m55*
         RSe45611p + 1./27.*m15*m55*RSe465p - 1./54.*pow(m15,2)*
         RSe45611p + 1./27.*pow(m15,2)*RSe461p - 1./54.*pow(m15,2)*
         RSe465p + 1./108.*m11*me2*RSe45611p - 1./54.*m11*me2*RSe461p
          + 1./108.*m11*me2*RSe465p - 1./54.*m11*m55*RSe45611p - 1./54.
         *m11*m55*RSe461p - 1./54.*m11*m55*RSe465p + 1./27.*m11*m15*
         RSe45611p + 1./54.*m11*m15*RSe465p - 1./72.*pow(m11,2)*
         RSe45611p - 1./108.*pow(m11,2)*RSe461p - 1./216.*pow(m11,2)*
         RSe465p );

      M12p6x13RV +=  + AbV(me2,L)*BbV(m16,L) * ( 1./108.*m66*me2*
         RSe45611p - 1./54.*m66*me2*RSe451p + 1./108.*m66*me2*RSe456p
          - 1./216.*pow(m66,2)*RSe45611p - 1./108.*pow(m66,2)*RSe451p
          - 1./72.*pow(m66,2)*RSe456p - 1./54.*m16*me2*RSe45611p + 1./
         27.*m16*me2*RSe451p - 1./54.*m16*me2*RSe456p + 1./54.*m16*m66*
         RSe45611p + 1./27.*m16*m66*RSe456p - 1./54.*pow(m16,2)*
         RSe45611p + 1./27.*pow(m16,2)*RSe451p - 1./54.*pow(m16,2)*
         RSe456p + 1./108.*m11*me2*RSe45611p - 1./54.*m11*me2*RSe451p
          + 1./108.*m11*me2*RSe456p - 1./54.*m11*m66*RSe45611p - 1./54.
         *m11*m66*RSe451p - 1./54.*m11*m66*RSe456p + 1./27.*m11*m16*
         RSe45611p + 1./54.*m11*m16*RSe456p - 1./72.*pow(m11,2)*
         RSe45611p - 1./108.*pow(m11,2)*RSe451p - 1./216.*pow(m11,2)*
         RSe456p );

      M12p6x13RV +=  + AbV(me2,L)*BbV(mp2,L) * (  - 1./27.*m11*m66*
         RSe456p*pow(RSp451e,2) - 1./27.*m11*m55*RSe465p*pow(RSp461e,2)
          - 1./27.*m11*m44*RSe564p*pow(RSp561e,2) );

      M12p6x13RV +=  + AbV(me2,L)*BbV(me2,mp2,L) * (  - 2./27.*m11*m66*
         RSe456p*RSp451e*RSe451p - 2./27.*m11*m55*RSe465p*RSp461e*
         RSe461p - 2./27.*m11*m44*RSe564p*RSp561e*RSe561p );

      M12p6x13RV +=  + AbV(me2,L)*BbV(me2,L) * (  - 1./27.*m11*m66*
         RSe456p*pow(RSe451p,2) - 1./27.*m11*m55*RSe465p*pow(RSe461p,2)
          - 1./27.*m11*m44*RSe564p*pow(RSe561p,2) );

      M12p6x13RV +=  + AbV(me2,L)*CbV(m11,L) * ( 2./9.*pow(m11,2)*
         RSe45611p*RD1456ep );

      M12p6x13RV +=  + BbV(m11,mp2,L)*BbV(m11,L) * ( 2./9.*pow(m11,2)*
         RSp45611e*RD1456ep );

      M12p6x13RV +=  + BbV(m11,me2,L)*BbV(m11,L) * ( 2./9.*pow(m11,2)*
         RSe45611p*RD1456ep );

      M12p6x13RV +=  + pow(BbV(m11,L),2) * ( 1./18.*m11*pow(RD1456ep,2)
          + 2./9.*pow(m11,2)*RD1456ep*RC1456pe );

      M12p6x13RV +=  + BbV(m11,L)*BbV(m14,L) * (  - 1./216.*pow(m44,2)*
         RD1456ep + 1./54.*m14*m44*RD1456ep - 1./54.*pow(m14,2)*
         RD1456ep - 1./108.*m11*m44*RD1456ep + 1./54.*m11*m14*RD1456ep
          - 1./216.*pow(m11,2)*RD1456ep );

      M12p6x13RV +=  + BbV(m11,L)*BbV(m15,L) * (  - 1./216.*pow(m55,2)*
         RD1456ep + 1./54.*m15*m55*RD1456ep - 1./54.*pow(m15,2)*
         RD1456ep - 1./108.*m11*m55*RD1456ep + 1./54.*m11*m15*RD1456ep
          - 1./216.*pow(m11,2)*RD1456ep );

      M12p6x13RV +=  + BbV(m11,L)*BbV(m16,L) * (  - 1./216.*pow(m66,2)*
         RD1456ep + 1./54.*m16*m66*RD1456ep - 1./54.*pow(m16,2)*
         RD1456ep - 1./108.*m11*m66*RD1456ep + 1./54.*m11*m16*RD1456ep
          - 1./216.*pow(m11,2)*RD1456ep );

      M12p6x13RV +=  + BbV(m11,L)*CbV(m11,L) * ( 2./9.*pow(m11,2)*pow(
         RD1456ep,2) );

      M12p6x13RV +=  + A23bV(m11,L)*pi16 * ( 3./4.*m11 - 1./4.*m11*
         RS145pe - 1./4.*m11*RS146pe - 1./4.*m11*RS156pe );

      M12p6x13RV +=  + A23bV(m14,L)*pi16 * ( 1./12.*m11*RC1456pe + 1./
         12.*m11*RSe45611p + 1./12.*m11*RSp45611e - 1./6.*m11*RS156pe
          + 1./12.*m11*RS456pe - 1./6.*m11*RSe561p - 1./6.*m11*RSp561e
          + 1./12.*m11*RSe564p + 1./12.*m11*RSp564e );

      M12p6x13RV +=  + A23bV(m15,L)*pi16 * ( 1./12.*m11*RC1456pe + 1./
         12.*m11*RSe45611p + 1./12.*m11*RSp45611e - 1./6.*m11*RS146pe
          + 1./12.*m11*RS546pe - 1./6.*m11*RSe461p - 1./6.*m11*RSp461e
          + 1./12.*m11*RSe465p + 1./12.*m11*RSp465e );

      M12p6x13RV +=  + A23bV(m16,L)*pi16 * ( 1./12.*m11*RC1456pe + 1./
         12.*m11*RSe45611p + 1./12.*m11*RSp45611e - 1./6.*m11*RS145pe
          + 1./12.*m11*RS645pe - 1./6.*m11*RSe451p - 1./6.*m11*RSp451e
          + 1./12.*m11*RSe456p + 1./12.*m11*RSp456e );

      M12p6x13RV +=  + A23bV(mp2,L)*pi16 * (  - 1./4.*m11*RSp561e - 1./
         4.*m11*RSp461e - 1./4.*m11*RSp451e );

      M12p6x13RV +=  + A23bV(me2,L)*pi16 * (  - 1./4.*m11*RSe561p - 1./
         4.*m11*RSe461p - 1./4.*m11*RSe451p );

      M12p6x13RV +=  + hhV(1,m11,m11,m11,m11,L,mu2) * ( 1./3.*pow(
         m11,2) + 2./9.*pow(m11,2)*pow(RC1456pe,2) );

      M12p6x13RV +=  + hhV(1,m11,m11,mp2,m11,L,mu2) * ( 4./9.*pow(
         m11,2)*RSp45611e*RC1456pe );

      M12p6x13RV +=  + hhV(1,m11,m11,me2,m11,L,mu2) * ( 4./9.*pow(
         m11,2)*RSe45611p*RC1456pe );

      M12p6x13RV +=  + hhV(1,m11,m14,m14,m11,L,mu2) * (  - 1./27.*m44*
         RD1456ep - 1./108.*pow(m44,2)*RC1456pe - 1./54.*pow(m44,2)*
         RS156pe + 2./27.*m14*RD1456ep + 1./27.*m14*m44*RC1456pe - 1./
         27.*pow(m14,2)*RC1456pe + 2./27.*pow(m14,2)*RS156pe - 37./108.
         *m11*RD1456ep - 7./54.*m11*m44*RC1456pe - 2./27.*m11*m44*
         RS156pe + 7./27.*m11*m14*RC1456pe - 10./27.*m11*m14*RS156pe - 
         29./54.*pow(m11,2)*RC1456pe + 5./9.*pow(m11,2)*RS156pe );

      M12p6x13RV +=  + hhV(1,m11,m15,m15,m11,L,mu2) * (  - 1./27.*m55*
         RD1456ep - 1./108.*pow(m55,2)*RC1456pe - 1./54.*pow(m55,2)*
         RS146pe + 2./27.*m15*RD1456ep + 1./27.*m15*m55*RC1456pe - 1./
         27.*pow(m15,2)*RC1456pe + 2./27.*pow(m15,2)*RS146pe - 37./108.
         *m11*RD1456ep - 7./54.*m11*m55*RC1456pe - 2./27.*m11*m55*
         RS146pe + 7./27.*m11*m15*RC1456pe - 10./27.*m11*m15*RS146pe - 
         29./54.*pow(m11,2)*RC1456pe + 5./9.*pow(m11,2)*RS146pe );

      M12p6x13RV +=  + hhV(1,m11,m16,m16,m11,L,mu2) * (  - 1./27.*m66*
         RD1456ep - 1./108.*pow(m66,2)*RC1456pe - 1./54.*pow(m66,2)*
         RS145pe + 2./27.*m16*RD1456ep + 1./27.*m16*m66*RC1456pe - 1./
         27.*pow(m16,2)*RC1456pe + 2./27.*pow(m16,2)*RS145pe - 37./108.
         *m11*RD1456ep - 7./54.*m11*m66*RC1456pe - 2./27.*m11*m66*
         RS145pe + 7./27.*m11*m16*RC1456pe - 10./27.*m11*m16*RS145pe - 
         29./54.*pow(m11,2)*RC1456pe + 5./9.*pow(m11,2)*RS145pe );

      M12p6x13RV +=  + hhV(1,m11,mp2,mp2,m11,L,mu2) * ( 2./9.*pow(
         m11,2)*pow(RSp45611e,2) );

      M12p6x13RV +=  + hhV(1,m11,me2,mp2,m11,L,mu2) * ( 4./9.*pow(
         m11,2)*RSp45611e*RSe45611p );

      M12p6x13RV +=  + hhV(1,m11,me2,me2,m11,L,mu2) * ( 2./9.*pow(
         m11,2)*pow(RSe45611p,2) );

      M12p6x13RV +=  + hhV(1,m14,m14,m44,m11,L,mu2) * ( 1./36.*pow(
         m44,2) - 1./108.*pow(m44,2)*RS456pe - 1./9.*m14*m44 + 1./27.*
         m14*m44*RS456pe + 1./9.*pow(m14,2) - 1./27.*pow(m14,2)*RS456pe
          - 5./18.*m11*m44 + 5./54.*m11*m44*RS456pe + 5./9.*m11*m14 - 5.
         /27.*m11*m14*RS456pe - 11./36.*pow(m11,2) + 11./108.*pow(
         m11,2)*RS456pe );

      M12p6x13RV +=  + hhV(1,m14,m14,mp2,m11,L,mu2) * (  - 1./27.*pow(
         mp2,2)*RSp45611e + 2./27.*pow(mp2,2)*RSp561e - 1./27.*pow(
         mp2,2)*RSp564e - 1./27.*m44*mp2*RSp45611e + 1./27.*m44*mp2*
         RSp564e - 1./108.*pow(m44,2)*RSp45611e - 1./54.*pow(m44,2)*
         RSp561e - 1./108.*pow(m44,2)*RSp564e + 2./27.*m14*mp2*
         RSp45611e - 4./27.*m14*mp2*RSp561e + 2./27.*m14*mp2*RSp564e + 
         1./27.*m14*m44*RSp45611e - 1./27.*m14*m44*RSp564e - 1./27.*
         pow(m14,2)*RSp45611e + 2./27.*pow(m14,2)*RSp561e - 1./27.*pow(
         m14,2)*RSp564e + 1./27.*m11*mp2*RSp45611e - 2./9.*m11*mp2*
         RSp561e + 5./27.*m11*mp2*RSp564e + 1./54.*m11*m44*RSp45611e - 
         2./27.*m11*m44*RSp561e - 5./54.*m11*m44*RSp564e - 1./27.*m11*
         m14*RSp45611e + 2./9.*m11*m14*RSp561e - 5./27.*m11*m14*RSp564e
          + 35./108.*pow(m11,2)*RSp45611e - 31./54.*pow(m11,2)*RSp561e
          + 11./108.*pow(m11,2)*RSp564e );

      M12p6x13RV +=  + hhV(1,m14,m14,me2,m11,L,mu2) * (  - 1./27.*pow(
         me2,2)*RSe45611p + 2./27.*pow(me2,2)*RSe561p - 1./27.*pow(
         me2,2)*RSe564p - 1./27.*m44*me2*RSe45611p + 1./27.*m44*me2*
         RSe564p - 1./108.*pow(m44,2)*RSe45611p - 1./54.*pow(m44,2)*
         RSe561p - 1./108.*pow(m44,2)*RSe564p + 2./27.*m14*me2*
         RSe45611p - 4./27.*m14*me2*RSe561p + 2./27.*m14*me2*RSe564p + 
         1./27.*m14*m44*RSe45611p - 1./27.*m14*m44*RSe564p - 1./27.*
         pow(m14,2)*RSe45611p + 2./27.*pow(m14,2)*RSe561p - 1./27.*pow(
         m14,2)*RSe564p + 1./27.*m11*me2*RSe45611p - 2./9.*m11*me2*
         RSe561p + 5./27.*m11*me2*RSe564p + 1./54.*m11*m44*RSe45611p - 
         2./27.*m11*m44*RSe561p - 5./54.*m11*m44*RSe564p - 1./27.*m11*
         m14*RSe45611p + 2./9.*m11*m14*RSe561p - 5./27.*m11*m14*RSe564p
          + 35./108.*pow(m11,2)*RSe45611p - 31./54.*pow(m11,2)*RSe561p
          + 11./108.*pow(m11,2)*RSe564p );

      M12p6x13RV +=  + hhV(1,m14,m15,m45,m11,L,mu2) * ( 1./72.*pow(
         m55,2) - 1./9.*m45*m55 + 2./9.*pow(m45,2) + 1./36.*m44*m55 - 1.
         /9.*m44*m45 + 1./72.*pow(m44,2) + 1./18.*m15*m55 - 2./9.*m15*
         m45 + 1./18.*m15*m44 + 1./18.*pow(m15,2) + 1./18.*m14*m55 - 2./
         9.*m14*m45 + 1./18.*m14*m44 + 1./9.*m14*m15 + 1./18.*pow(
         m14,2) + 5./18.*m11*m55 - 10./9.*m11*m45 + 5./18.*m11*m44 + 5./
         9.*m11*m15 + 5./9.*m11*m14 - 11./18.*pow(m11,2) );

      M12p6x13RV +=  + hhV(1,m14,m16,m46,m11,L,mu2) * ( 1./72.*pow(
         m66,2) - 1./9.*m46*m66 + 2./9.*pow(m46,2) + 1./36.*m44*m66 - 1.
         /9.*m44*m46 + 1./72.*pow(m44,2) + 1./18.*m16*m66 - 2./9.*m16*
         m46 + 1./18.*m16*m44 + 1./18.*pow(m16,2) + 1./18.*m14*m66 - 2./
         9.*m14*m46 + 1./18.*m14*m44 + 1./9.*m14*m16 + 1./18.*pow(
         m14,2) + 5./18.*m11*m66 - 10./9.*m11*m46 + 5./18.*m11*m44 + 5./
         9.*m11*m16 + 5./9.*m11*m14 - 11./18.*pow(m11,2) );

      M12p6x13RV +=  + hhV(1,m15,m15,m55,m11,L,mu2) * ( 1./36.*pow(
         m55,2) - 1./108.*pow(m55,2)*RS546pe - 1./9.*m15*m55 + 1./27.*
         m15*m55*RS546pe + 1./9.*pow(m15,2) - 1./27.*pow(m15,2)*RS546pe
          - 5./18.*m11*m55 + 5./54.*m11*m55*RS546pe + 5./9.*m11*m15 - 5.
         /27.*m11*m15*RS546pe - 11./36.*pow(m11,2) + 11./108.*pow(
         m11,2)*RS546pe );

      M12p6x13RV +=  + hhV(1,m15,m15,mp2,m11,L,mu2) * (  - 1./27.*pow(
         mp2,2)*RSp45611e + 2./27.*pow(mp2,2)*RSp461e - 1./27.*pow(
         mp2,2)*RSp465e - 1./27.*m55*mp2*RSp45611e + 1./27.*m55*mp2*
         RSp465e - 1./108.*pow(m55,2)*RSp45611e - 1./54.*pow(m55,2)*
         RSp461e - 1./108.*pow(m55,2)*RSp465e + 2./27.*m15*mp2*
         RSp45611e - 4./27.*m15*mp2*RSp461e + 2./27.*m15*mp2*RSp465e + 
         1./27.*m15*m55*RSp45611e - 1./27.*m15*m55*RSp465e - 1./27.*
         pow(m15,2)*RSp45611e + 2./27.*pow(m15,2)*RSp461e - 1./27.*pow(
         m15,2)*RSp465e + 1./27.*m11*mp2*RSp45611e - 2./9.*m11*mp2*
         RSp461e + 5./27.*m11*mp2*RSp465e + 1./54.*m11*m55*RSp45611e - 
         2./27.*m11*m55*RSp461e - 5./54.*m11*m55*RSp465e - 1./27.*m11*
         m15*RSp45611e + 2./9.*m11*m15*RSp461e - 5./27.*m11*m15*RSp465e
          + 35./108.*pow(m11,2)*RSp45611e - 31./54.*pow(m11,2)*RSp461e
          + 11./108.*pow(m11,2)*RSp465e );

      M12p6x13RV +=  + hhV(1,m15,m15,me2,m11,L,mu2) * (  - 1./27.*pow(
         me2,2)*RSe45611p + 2./27.*pow(me2,2)*RSe461p - 1./27.*pow(
         me2,2)*RSe465p - 1./27.*m55*me2*RSe45611p + 1./27.*m55*me2*
         RSe465p - 1./108.*pow(m55,2)*RSe45611p - 1./54.*pow(m55,2)*
         RSe461p - 1./108.*pow(m55,2)*RSe465p + 2./27.*m15*me2*
         RSe45611p - 4./27.*m15*me2*RSe461p + 2./27.*m15*me2*RSe465p + 
         1./27.*m15*m55*RSe45611p - 1./27.*m15*m55*RSe465p - 1./27.*
         pow(m15,2)*RSe45611p + 2./27.*pow(m15,2)*RSe461p - 1./27.*pow(
         m15,2)*RSe465p + 1./27.*m11*me2*RSe45611p - 2./9.*m11*me2*
         RSe461p + 5./27.*m11*me2*RSe465p + 1./54.*m11*m55*RSe45611p - 
         2./27.*m11*m55*RSe461p - 5./54.*m11*m55*RSe465p - 1./27.*m11*
         m15*RSe45611p + 2./9.*m11*m15*RSe461p - 5./27.*m11*m15*RSe465p
          + 35./108.*pow(m11,2)*RSe45611p - 31./54.*pow(m11,2)*RSe461p
          + 11./108.*pow(m11,2)*RSe465p );

      M12p6x13RV +=  + hhV(1,m15,m16,m56,m11,L,mu2) * ( 1./72.*pow(
         m66,2) - 1./9.*m56*m66 + 2./9.*pow(m56,2) + 1./36.*m55*m66 - 1.
         /9.*m55*m56 + 1./72.*pow(m55,2) + 1./18.*m16*m66 - 2./9.*m16*
         m56 + 1./18.*m16*m55 + 1./18.*pow(m16,2) + 1./18.*m15*m66 - 2./
         9.*m15*m56 + 1./18.*m15*m55 + 1./9.*m15*m16 + 1./18.*pow(
         m15,2) + 5./18.*m11*m66 - 10./9.*m11*m56 + 5./18.*m11*m55 + 5./
         9.*m11*m16 + 5./9.*m11*m15 - 11./18.*pow(m11,2) );

      M12p6x13RV +=  + hhV(1,m16,m16,m66,m11,L,mu2) * ( 1./36.*pow(
         m66,2) - 1./108.*pow(m66,2)*RS645pe - 1./9.*m16*m66 + 1./27.*
         m16*m66*RS645pe + 1./9.*pow(m16,2) - 1./27.*pow(m16,2)*RS645pe
          - 5./18.*m11*m66 + 5./54.*m11*m66*RS645pe + 5./9.*m11*m16 - 5.
         /27.*m11*m16*RS645pe - 11./36.*pow(m11,2) + 11./108.*pow(
         m11,2)*RS645pe );

      M12p6x13RV +=  + hhV(1,m16,m16,mp2,m11,L,mu2) * (  - 1./27.*pow(
         mp2,2)*RSp45611e + 2./27.*pow(mp2,2)*RSp451e - 1./27.*pow(
         mp2,2)*RSp456e - 1./27.*m66*mp2*RSp45611e + 1./27.*m66*mp2*
         RSp456e - 1./108.*pow(m66,2)*RSp45611e - 1./54.*pow(m66,2)*
         RSp451e - 1./108.*pow(m66,2)*RSp456e + 2./27.*m16*mp2*
         RSp45611e - 4./27.*m16*mp2*RSp451e + 2./27.*m16*mp2*RSp456e + 
         1./27.*m16*m66*RSp45611e - 1./27.*m16*m66*RSp456e - 1./27.*
         pow(m16,2)*RSp45611e + 2./27.*pow(m16,2)*RSp451e - 1./27.*pow(
         m16,2)*RSp456e + 1./27.*m11*mp2*RSp45611e - 2./9.*m11*mp2*
         RSp451e + 5./27.*m11*mp2*RSp456e + 1./54.*m11*m66*RSp45611e - 
         2./27.*m11*m66*RSp451e - 5./54.*m11*m66*RSp456e - 1./27.*m11*
         m16*RSp45611e + 2./9.*m11*m16*RSp451e - 5./27.*m11*m16*RSp456e
          + 35./108.*pow(m11,2)*RSp45611e - 31./54.*pow(m11,2)*RSp451e
          + 11./108.*pow(m11,2)*RSp456e );

      M12p6x13RV +=  + hhV(1,m16,m16,me2,m11,L,mu2) * (  - 1./27.*pow(
         me2,2)*RSe45611p + 2./27.*pow(me2,2)*RSe451p - 1./27.*pow(
         me2,2)*RSe456p - 1./27.*m66*me2*RSe45611p + 1./27.*m66*me2*
         RSe456p - 1./108.*pow(m66,2)*RSe45611p - 1./54.*pow(m66,2)*
         RSe451p - 1./108.*pow(m66,2)*RSe456p + 2./27.*m16*me2*
         RSe45611p - 4./27.*m16*me2*RSe451p + 2./27.*m16*me2*RSe456p + 
         1./27.*m16*m66*RSe45611p - 1./27.*m16*m66*RSe456p - 1./27.*
         pow(m16,2)*RSe45611p + 2./27.*pow(m16,2)*RSe451p - 1./27.*pow(
         m16,2)*RSe456p + 1./27.*m11*me2*RSe45611p - 2./9.*m11*me2*
         RSe451p + 5./27.*m11*me2*RSe456p + 1./54.*m11*m66*RSe45611p - 
         2./27.*m11*m66*RSe451p - 5./54.*m11*m66*RSe456p - 1./27.*m11*
         m16*RSe45611p + 2./9.*m11*m16*RSe451p - 5./27.*m11*m16*RSe456p
          + 35./108.*pow(m11,2)*RSe45611p - 31./54.*pow(m11,2)*RSe451p
          + 11./108.*pow(m11,2)*RSe456p );

      M12p6x13RV +=  + hhV(1,m44,m14,m14,m11,L,mu2) * ( 1./4.*m11*m44
          - 1./12.*m11*m44*RS456pe );

      M12p6x13RV +=  + hhV(1,m45,m14,m15,m11,L,mu2) * ( 1./2.*m11*m45 )
         ;

      M12p6x13RV +=  + hhV(1,m46,m14,m16,m11,L,mu2) * ( 1./2.*m11*m46 )
         ;

      M12p6x13RV +=  + hhV(1,m55,m15,m15,m11,L,mu2) * ( 1./4.*m11*m55
          - 1./12.*m11*m55*RS546pe );

      M12p6x13RV +=  + hhV(1,m56,m15,m16,m11,L,mu2) * ( 1./2.*m11*m56 )
         ;

      M12p6x13RV +=  + hhV(1,m66,m16,m16,m11,L,mu2) * ( 1./4.*m11*m66
          - 1./12.*m11*m66*RS645pe );

      M12p6x13RV +=  + hhV(1,mp2,m14,m14,m11,L,mu2) * (  - 1./12.*m11*
         mp2*RSp45611e + 1./6.*m11*mp2*RSp561e - 1./12.*m11*mp2*RSp564e
          );

      M12p6x13RV +=  + hhV(1,mp2,m15,m15,m11,L,mu2) * (  - 1./12.*m11*
         mp2*RSp45611e + 1./6.*m11*mp2*RSp461e - 1./12.*m11*mp2*RSp465e
          );

      M12p6x13RV +=  + hhV(1,mp2,m16,m16,m11,L,mu2) * (  - 1./12.*m11*
         mp2*RSp45611e + 1./6.*m11*mp2*RSp451e - 1./12.*m11*mp2*RSp456e
          );

      M12p6x13RV +=  + hhV(1,me2,m14,m14,m11,L,mu2) * (  - 1./12.*m11*
         me2*RSe45611p + 1./6.*m11*me2*RSe561p - 1./12.*m11*me2*RSe564p
          );

      M12p6x13RV +=  + hhV(1,me2,m15,m15,m11,L,mu2) * (  - 1./12.*m11*
         me2*RSe45611p + 1./6.*m11*me2*RSe461p - 1./12.*m11*me2*RSe465p
          );

      M12p6x13RV +=  + hhV(1,me2,m16,m16,m11,L,mu2) * (  - 1./12.*m11*
         me2*RSe45611p + 1./6.*m11*me2*RSe451p - 1./12.*m11*me2*RSe456p
          );

      M12p6x13RV +=  + hhV(2,m11,m11,m11,m11,L,mu2) * ( 4./9.*pow(
         m11,2)*RD1456ep*RC1456pe );

      M12p6x13RV +=  + hhV(2,m11,m11,mp2,m11,L,mu2) * ( 4./9.*pow(
         m11,2)*RSp45611e*RD1456ep );

      M12p6x13RV +=  + hhV(2,m11,m11,me2,m11,L,mu2) * ( 4./9.*pow(
         m11,2)*RSe45611p*RD1456ep );

      M12p6x13RV +=  + hhV(2,m11,m14,m14,m11,L,mu2) * (  - 1./108.*pow(
         m44,2)*RD1456ep + 1./27.*m14*m44*RD1456ep - 1./27.*pow(m14,2)*
         RD1456ep - 7./54.*m11*m44*RD1456ep + 7./27.*m11*m14*RD1456ep
          - 29./54.*pow(m11,2)*RD1456ep );

      M12p6x13RV +=  + hhV(2,m11,m15,m15,m11,L,mu2) * (  - 1./108.*pow(
         m55,2)*RD1456ep + 1./27.*m15*m55*RD1456ep - 1./27.*pow(m15,2)*
         RD1456ep - 7./54.*m11*m55*RD1456ep + 7./27.*m11*m15*RD1456ep
          - 29./54.*pow(m11,2)*RD1456ep );

      M12p6x13RV +=  + hhV(2,m11,m16,m16,m11,L,mu2) * (  - 1./108.*pow(
         m66,2)*RD1456ep + 1./27.*m16*m66*RD1456ep - 1./27.*pow(m16,2)*
         RD1456ep - 7./54.*m11*m66*RD1456ep + 7./27.*m11*m16*RD1456ep
          - 29./54.*pow(m11,2)*RD1456ep );

      M12p6x13RV +=  + hhV(5,m11,m11,m11,m11,L,mu2) * ( 2./9.*pow(
         m11,2)*pow(RD1456ep,2) );

      M12p6x13RV +=  + hh1V(1,m11,m14,m14,m11,L,mu2) * ( 2./9.*m11*
         RD1456ep + 1./9.*m11*m44*RC1456pe - 2./9.*m11*m14*RC1456pe + 4.
         /9.*m11*m14*RS156pe + 7./9.*pow(m11,2)*RC1456pe - 10./9.*pow(
         m11,2)*RS156pe );

      M12p6x13RV +=  + hh1V(1,m11,m15,m15,m11,L,mu2) * ( 2./9.*m11*
         RD1456ep + 1./9.*m11*m55*RC1456pe - 2./9.*m11*m15*RC1456pe + 4.
         /9.*m11*m15*RS146pe + 7./9.*pow(m11,2)*RC1456pe - 10./9.*pow(
         m11,2)*RS146pe );

      M12p6x13RV +=  + hh1V(1,m11,m16,m16,m11,L,mu2) * ( 2./9.*m11*
         RD1456ep + 1./9.*m11*m66*RC1456pe - 2./9.*m11*m16*RC1456pe + 4.
         /9.*m11*m16*RS145pe + 7./9.*pow(m11,2)*RC1456pe - 10./9.*pow(
         m11,2)*RS145pe );

      M12p6x13RV +=  + hh1V(1,m14,m14,m44,m11,L,mu2) * ( 2./3.*m11*m44
          - 2./9.*m11*m44*RS456pe - 4./3.*m11*m14 + 4./9.*m11*m14*
         RS456pe + 2./3.*pow(m11,2) - 2./9.*pow(m11,2)*RS456pe );

      M12p6x13RV +=  + hh1V(1,m14,m14,mp2,m11,L,mu2) * (  - 4./9.*m11*
         mp2*RSp45611e + 8./9.*m11*mp2*RSp561e - 4./9.*m11*mp2*RSp564e
          - 2./9.*m11*m44*RSp45611e + 2./9.*m11*m44*RSp564e + 4./9.*m11
         *m14*RSp45611e - 8./9.*m11*m14*RSp561e + 4./9.*m11*m14*RSp564e
          - 10./9.*pow(m11,2)*RSp45611e + 4./3.*pow(m11,2)*RSp561e - 2./
         9.*pow(m11,2)*RSp564e );

      M12p6x13RV +=  + hh1V(1,m14,m14,me2,m11,L,mu2) * (  - 4./9.*m11*
         me2*RSe45611p + 8./9.*m11*me2*RSe561p - 4./9.*m11*me2*RSe564p
          - 2./9.*m11*m44*RSe45611p + 2./9.*m11*m44*RSe564p + 4./9.*m11
         *m14*RSe45611p - 8./9.*m11*m14*RSe561p + 4./9.*m11*m14*RSe564p
          - 10./9.*pow(m11,2)*RSe45611p + 4./3.*pow(m11,2)*RSe561p - 2./
         9.*pow(m11,2)*RSe564p );

      M12p6x13RV +=  + hh1V(1,m14,m15,m45,m11,L,mu2) * (  - 1./3.*m11*
         m55 + 4./3.*m11*m45 - 1./3.*m11*m44 - 2./3.*m11*m15 - 2./3.*
         m11*m14 + 2./3.*pow(m11,2) );

      M12p6x13RV +=  + hh1V(1,m14,m16,m46,m11,L,mu2) * (  - 1./3.*m11*
         m66 + 4./3.*m11*m46 - 1./3.*m11*m44 - 2./3.*m11*m16 - 2./3.*
         m11*m14 + 2./3.*pow(m11,2) );

      M12p6x13RV +=  + hh1V(1,m15,m14,m45,m11,L,mu2) * (  - 1./3.*m11*
         m55 + 4./3.*m11*m45 - 1./3.*m11*m44 - 2./3.*m11*m15 - 2./3.*
         m11*m14 + 2./3.*pow(m11,2) );

      M12p6x13RV +=  + hh1V(1,m15,m15,m55,m11,L,mu2) * ( 2./3.*m11*m55
          - 2./9.*m11*m55*RS546pe - 4./3.*m11*m15 + 4./9.*m11*m15*
         RS546pe + 2./3.*pow(m11,2) - 2./9.*pow(m11,2)*RS546pe );

      M12p6x13RV +=  + hh1V(1,m15,m15,mp2,m11,L,mu2) * (  - 4./9.*m11*
         mp2*RSp45611e + 8./9.*m11*mp2*RSp461e - 4./9.*m11*mp2*RSp465e
          - 2./9.*m11*m55*RSp45611e + 2./9.*m11*m55*RSp465e + 4./9.*m11
         *m15*RSp45611e - 8./9.*m11*m15*RSp461e + 4./9.*m11*m15*RSp465e
          - 10./9.*pow(m11,2)*RSp45611e + 4./3.*pow(m11,2)*RSp461e - 2./
         9.*pow(m11,2)*RSp465e );

      M12p6x13RV +=  + hh1V(1,m15,m15,me2,m11,L,mu2) * (  - 4./9.*m11*
         me2*RSe45611p + 8./9.*m11*me2*RSe461p - 4./9.*m11*me2*RSe465p
          - 2./9.*m11*m55*RSe45611p + 2./9.*m11*m55*RSe465p + 4./9.*m11
         *m15*RSe45611p - 8./9.*m11*m15*RSe461p + 4./9.*m11*m15*RSe465p
          - 10./9.*pow(m11,2)*RSe45611p + 4./3.*pow(m11,2)*RSe461p - 2./
         9.*pow(m11,2)*RSe465p );

      M12p6x13RV +=  + hh1V(1,m15,m16,m56,m11,L,mu2) * (  - 1./3.*m11*
         m66 + 4./3.*m11*m56 - 1./3.*m11*m55 - 2./3.*m11*m16 - 2./3.*
         m11*m15 + 2./3.*pow(m11,2) );

      M12p6x13RV +=  + hh1V(1,m16,m14,m46,m11,L,mu2) * (  - 1./3.*m11*
         m66 + 4./3.*m11*m46 - 1./3.*m11*m44 - 2./3.*m11*m16 - 2./3.*
         m11*m14 + 2./3.*pow(m11,2) );

      M12p6x13RV +=  + hh1V(1,m16,m15,m56,m11,L,mu2) * (  - 1./3.*m11*
         m66 + 4./3.*m11*m56 - 1./3.*m11*m55 - 2./3.*m11*m16 - 2./3.*
         m11*m15 + 2./3.*pow(m11,2) );

      M12p6x13RV +=  + hh1V(1,m16,m16,m66,m11,L,mu2) * ( 2./3.*m11*m66
          - 2./9.*m11*m66*RS645pe - 4./3.*m11*m16 + 4./9.*m11*m16*
         RS645pe + 2./3.*pow(m11,2) - 2./9.*pow(m11,2)*RS645pe );

      M12p6x13RV +=  + hh1V(1,m16,m16,mp2,m11,L,mu2) * (  - 4./9.*m11*
         mp2*RSp45611e + 8./9.*m11*mp2*RSp451e - 4./9.*m11*mp2*RSp456e
          - 2./9.*m11*m66*RSp45611e + 2./9.*m11*m66*RSp456e + 4./9.*m11
         *m16*RSp45611e - 8./9.*m11*m16*RSp451e + 4./9.*m11*m16*RSp456e
          - 10./9.*pow(m11,2)*RSp45611e + 4./3.*pow(m11,2)*RSp451e - 2./
         9.*pow(m11,2)*RSp456e );

      M12p6x13RV +=  + hh1V(1,m16,m16,me2,m11,L,mu2) * (  - 4./9.*m11*
         me2*RSe45611p + 8./9.*m11*me2*RSe451p - 4./9.*m11*me2*RSe456p
          - 2./9.*m11*m66*RSe45611p + 2./9.*m11*m66*RSe456p + 4./9.*m11
         *m16*RSe45611p - 8./9.*m11*m16*RSe451p + 4./9.*m11*m16*RSe456p
          - 10./9.*pow(m11,2)*RSe45611p + 4./3.*pow(m11,2)*RSe451p - 2./
         9.*pow(m11,2)*RSe456p );

      M12p6x13RV +=  + hh1V(2,m11,m14,m14,m11,L,mu2) * ( 1./9.*m11*m44*
         RD1456ep - 2./9.*m11*m14*RD1456ep + 7./9.*pow(m11,2)*RD1456ep
          );

      M12p6x13RV +=  + hh1V(2,m11,m15,m15,m11,L,mu2) * ( 1./9.*m11*m55*
         RD1456ep - 2./9.*m11*m15*RD1456ep + 7./9.*pow(m11,2)*RD1456ep
          );

      M12p6x13RV +=  + hh1V(2,m11,m16,m16,m11,L,mu2) * ( 1./9.*m11*m66*
         RD1456ep - 2./9.*m11*m16*RD1456ep + 7./9.*pow(m11,2)*RD1456ep
          );

      M12p6x13RV +=  + hh21V(1,m11,m14,m14,m11,L,mu2) * (  - 1./4.*pow(
         m11,2)*RC1456pe + 1./2.*pow(m11,2)*RS156pe );

      M12p6x13RV +=  + hh21V(1,m11,m15,m15,m11,L,mu2) * (  - 1./4.*pow(
         m11,2)*RC1456pe + 1./2.*pow(m11,2)*RS146pe );

      M12p6x13RV +=  + hh21V(1,m11,m16,m16,m11,L,mu2) * (  - 1./4.*pow(
         m11,2)*RC1456pe + 1./2.*pow(m11,2)*RS145pe );

      M12p6x13RV +=  + hh21V(1,m44,m14,m14,m11,L,mu2) * ( 3./4.*pow(
         m11,2) - 1./4.*pow(m11,2)*RS456pe );

      M12p6x13RV +=  + hh21V(1,m45,m14,m15,m11,L,mu2) * ( 3./2.*pow(
         m11,2) );

      M12p6x13RV +=  + hh21V(1,m46,m14,m16,m11,L,mu2) * ( 3./2.*pow(
         m11,2) );

      M12p6x13RV +=  + hh21V(1,m55,m15,m15,m11,L,mu2) * ( 3./4.*pow(
         m11,2) - 1./4.*pow(m11,2)*RS546pe );

      M12p6x13RV +=  + hh21V(1,m56,m15,m16,m11,L,mu2) * ( 3./2.*pow(
         m11,2) );

      M12p6x13RV +=  + hh21V(1,m66,m16,m16,m11,L,mu2) * ( 3./4.*pow(
         m11,2) - 1./4.*pow(m11,2)*RS645pe );

      M12p6x13RV +=  + hh21V(1,mp2,m14,m14,m11,L,mu2) * (  - 1./4.*pow(
         m11,2)*RSp45611e + 1./2.*pow(m11,2)*RSp561e - 1./4.*pow(m11,2)
         *RSp564e );

      M12p6x13RV +=  + hh21V(1,mp2,m15,m15,m11,L,mu2) * (  - 1./4.*pow(
         m11,2)*RSp45611e + 1./2.*pow(m11,2)*RSp461e - 1./4.*pow(m11,2)
         *RSp465e );

      M12p6x13RV +=  + hh21V(1,mp2,m16,m16,m11,L,mu2) * (  - 1./4.*pow(
         m11,2)*RSp45611e + 1./2.*pow(m11,2)*RSp451e - 1./4.*pow(m11,2)
         *RSp456e );

      M12p6x13RV +=  + hh21V(1,me2,m14,m14,m11,L,mu2) * (  - 1./4.*pow(
         m11,2)*RSe45611p + 1./2.*pow(m11,2)*RSe561p - 1./4.*pow(m11,2)
         *RSe564p );

      M12p6x13RV +=  + hh21V(1,me2,m15,m15,m11,L,mu2) * (  - 1./4.*pow(
         m11,2)*RSe45611p + 1./2.*pow(m11,2)*RSe461p - 1./4.*pow(m11,2)
         *RSe465p );

      M12p6x13RV +=  + hh21V(1,me2,m16,m16,m11,L,mu2) * (  - 1./4.*pow(
         m11,2)*RSe45611p + 1./2.*pow(m11,2)*RSe451p - 1./4.*pow(m11,2)
         *RSe456p );

      M12p6x13RV +=  + hh21V(2,m11,m14,m14,m11,L,mu2) * (  - 1./4.*pow(
         m11,2)*RD1456ep );

      M12p6x13RV +=  + hh21V(2,m11,m15,m15,m11,L,mu2) * (  - 1./4.*pow(
         m11,2)*RD1456ep );

      M12p6x13RV +=  + hh21V(2,m11,m16,m16,m11,L,mu2) * (  - 1./4.*pow(
         m11,2)*RD1456ep );

      M12p6x13RV +=  + hh27V(1,m11,m14,m14,m11,L,mu2) * ( 1./4.*m11*
         RC1456pe - 1./2.*m11*RS156pe );

      M12p6x13RV +=  + hh27V(1,m11,m15,m15,m11,L,mu2) * ( 1./4.*m11*
         RC1456pe - 1./2.*m11*RS146pe );

      M12p6x13RV +=  + hh27V(1,m11,m16,m16,m11,L,mu2) * ( 1./4.*m11*
         RC1456pe - 1./2.*m11*RS145pe );

      M12p6x13RV +=  + hh27V(1,m44,m14,m14,m11,L,mu2) * (  - 3./4.*m11
          + 1./4.*m11*RS456pe );

      M12p6x13RV +=  + hh27V(1,m45,m14,m15,m11,L,mu2) * (  - 3./2.*m11
          );

      M12p6x13RV +=  + hh27V(1,m46,m14,m16,m11,L,mu2) * (  - 3./2.*m11
          );

      M12p6x13RV +=  + hh27V(1,m55,m15,m15,m11,L,mu2) * (  - 3./4.*m11
          + 1./4.*m11*RS546pe );

      M12p6x13RV +=  + hh27V(1,m56,m15,m16,m11,L,mu2) * (  - 3./2.*m11
          );

      M12p6x13RV +=  + hh27V(1,m66,m16,m16,m11,L,mu2) * (  - 3./4.*m11
          + 1./4.*m11*RS645pe );

      M12p6x13RV +=  + hh27V(1,mp2,m14,m14,m11,L,mu2) * ( 1./4.*m11*
         RSp45611e - 1./2.*m11*RSp561e + 1./4.*m11*RSp564e );

      M12p6x13RV +=  + hh27V(1,mp2,m15,m15,m11,L,mu2) * ( 1./4.*m11*
         RSp45611e - 1./2.*m11*RSp461e + 1./4.*m11*RSp465e );

      M12p6x13RV +=  + hh27V(1,mp2,m16,m16,m11,L,mu2) * ( 1./4.*m11*
         RSp45611e - 1./2.*m11*RSp451e + 1./4.*m11*RSp456e );

      M12p6x13RV +=  + hh27V(1,me2,m14,m14,m11,L,mu2) * ( 1./4.*m11*
         RSe45611p - 1./2.*m11*RSe561p + 1./4.*m11*RSe564p );

      M12p6x13RV +=  + hh27V(1,me2,m15,m15,m11,L,mu2) * ( 1./4.*m11*
         RSe45611p - 1./2.*m11*RSe461p + 1./4.*m11*RSe465p );

      M12p6x13RV +=  + hh27V(1,me2,m16,m16,m11,L,mu2) * ( 1./4.*m11*
         RSe45611p - 1./2.*m11*RSe451p + 1./4.*m11*RSe456p );

      M12p6x13RV +=  + hh27V(2,m11,m14,m14,m11,L,mu2) * ( 1./4.*m11*
         RD1456ep );

      M12p6x13RV +=  + hh27V(2,m11,m15,m15,m11,L,mu2) * ( 1./4.*m11*
         RD1456ep );

      M12p6x13RV +=  + hh27V(2,m11,m16,m16,m11,L,mu2) * ( 1./4.*m11*
         RD1456ep );

  return M12p6x13RV/pow(f0,4);
}

// 2 valence mass 3 sea mass *****************************************

double mv2s3nf3p4V(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 5) || (B0mq.size() != 5)){
    cout << "ERROR: mv2s3nf3p4R probably called with wrong number"
	 << "of quark masses\n";}
  // five masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m55 = 2.*B0mq[3];
  double m66 = 2.*B0mq[4];
  double xb1 = (m44+m55+m66)/3.;
  double msqr = sqrt(xb1*xb1-(m44*m55+m44*m66+m55*m66)/3.);
  double mp2 = xb1-msqr; 
  double me2 = xb1+msqr;
  //double mu2 = mu0*mu0;
  double RSe45611p = RS(me2,m44,m55,m66,m11,m11,mp2);
  double RSp45611e = RS(mp2,m44,m55,m66,m11,m11,me2);
  double RSe45613p = RS(me2,m44,m55,m66,m11,m33,mp2);
  double RSp45613e = RS(mp2,m44,m55,m66,m11,m33,me2);
  double RSe45633p = RS(me2,m44,m55,m66,m33,m33,mp2);
  double RSp45633e = RS(mp2,m44,m55,m66,m33,m33,me2);
  double RS14563pe = RS(m11,m44,m55,m66,m33,mp2,me2);
  double RS34561pe = RS(m33,m44,m55,m66,m11,mp2,me2);

  double RD1456ep = RD(m11,m44,m55,m66,me2,mp2);
  double RD3456ep = RD(m33,m44,m55,m66,me2,mp2);

  double M13p4x23V =
       + AbV(m11,L) * ( 1./18.*RD1456ep - 1./9.*m33*RS14563pe - 2./9.*
         m11*RS14563pe );

      M13p4x23V +=  + AbV(m33,L) * ( 1./18.*RD3456ep - 2./9.*m33*
         RS34561pe - 1./9.*m11*RS34561pe );

      M13p4x23V +=  + AbV(mp2,L) * (  - 1./9.*mp2*RSp45613e + 1./18.*
         mp2*RSp45633e + 1./18.*mp2*RSp45611e - 1./9.*m33*RSp45613e - 1.
         /18.*m33*RSp45633e - 1./9.*m11*RSp45613e - 1./18.*m11*
         RSp45611e );

      M13p4x23V +=  + AbV(me2,L) * (  - 1./9.*me2*RSe45613p + 1./18.*
         me2*RSe45633p + 1./18.*me2*RSe45611p - 1./9.*m33*RSe45613p - 1.
         /18.*m33*RSe45633p - 1./9.*m11*RSe45613p - 1./18.*m11*
         RSe45611p );

  return M13p4x23V/pow(f0,2);
}

double mv2s3nf3p6V(const quarkmassnf mass, const Linf Liin, const double L){
  return mv2s3nf3p6LV(mass,Liin,L)+mv2s3nf3p6RV(mass,L);
}

double mv2s3nf3p6LV(const quarkmassnf mass, const Linf Liin, const double L){
  double L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu;
  int nfLi;
  Liin.out(L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu,nfLi);
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if (mu0 != mu){
    cout << "ERROR: differing mu's in mv2s3nf3p6LV\n";}
  if ( (nq != 5) || (B0mq.size() != 5)){
    cout << "ERROR: mv2s3nf3p6LV probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m55 = 2.*B0mq[3];
  double m66 = 2.*B0mq[4];
  double m13 = (m11+m33)/2;
  double m14 = (m11+m44)/2;
  double m15 = (m11+m55)/2;
  double m16 = (m11+m66)/2;
  double m34 = (m33+m44)/2;
  double m35 = (m33+m55)/2;
  double m36 = (m33+m66)/2;
  double m45 = (m44+m55)/2;
  double m46 = (m44+m66)/2;
  double m56 = (m55+m66)/2;

  double xb1 = (m44+m55+m66)/3.;
  double msqr = sqrt(xb1*xb1-(m44*m55+m44*m66+m55*m66)/3.);
  double mp2 = xb1-msqr; 
  double me2 = xb1+msqr;
  //double mu2 = mu0*mu0;

  //double RN13 = RN(m11,m33);
  //double RN1p = RN(m11,mp2);
  //double RN1e = RN(m11,me2);
  //double RN3p = RN(m33,mp2);
  //double RN3e = RN(m33,me2);
  //double RNep = RN(me2,mp2);

  double RSp456e = RS(mp2,m44,m55,m66,me2);
  double RSp465e = RS(mp2,m44,m66,m55,me2);
  double RSp564e = RS(mp2,m55,m66,m44,me2);
  double RSe456p = RS(me2,m44,m55,m66,mp2);
  double RSe465p = RS(me2,m44,m66,m55,mp2);
  double RSe564p = RS(me2,m55,m66,m44,mp2);
  double RSp451e = RS(mp2,m44,m55,m11,me2);
  double RSp461e = RS(mp2,m44,m66,m11,me2);
  double RSp561e = RS(mp2,m55,m66,m11,me2);
  double RSe451p = RS(me2,m44,m55,m11,mp2);
  double RSe461p = RS(me2,m44,m66,m11,mp2);
  double RSe561p = RS(me2,m55,m66,m11,mp2);
  double RSp453e = RS(mp2,m44,m55,m33,me2);
  double RSp463e = RS(mp2,m44,m66,m33,me2);
  double RSp563e = RS(mp2,m55,m66,m33,me2);
  double RSe453p = RS(me2,m44,m55,m33,mp2);
  double RSe463p = RS(me2,m44,m66,m33,mp2);
  double RSe563p = RS(me2,m55,m66,m33,mp2);
  double RS456pe = RS(m44,m55,m66,mp2,me2);
  double RS546pe = RS(m55,m44,m66,mp2,me2);
  double RS645pe = RS(m66,m44,m55,mp2,me2);

  double RS156pe = RS(m11,m55,m66,mp2,me2);
  double RS146pe = RS(m11,m44,m66,mp2,me2);
  double RS145pe = RS(m11,m44,m55,mp2,me2);
  double RS356pe = RS(m33,m55,m66,mp2,me2);
  double RS346pe = RS(m33,m44,m66,mp2,me2);
  double RS345pe = RS(m33,m44,m55,mp2,me2);

  double RD1456ep = RD(m11,m44,m55,m66,me2,mp2);
  double RD3456ep = RD(m33,m44,m55,m66,me2,mp2);

  //double RS1456ppe = RS(m11,m44,m55,m66,mp2,mp2,me2);
  //double RS1456pee = RS(m11,m44,m55,m66,mp2,me2,me2);
  double RSe45611p = RS(me2,m44,m55,m66,m11,m11,mp2);
  double RSp45611e = RS(mp2,m44,m55,m66,m11,m11,me2);
  double RSe45633p = RS(me2,m44,m55,m66,m33,m33,mp2);
  double RSp45633e = RS(mp2,m44,m55,m66,m33,m33,me2);
  //double RS3456ppe = RS(m33,m44,m55,m66,mp2,mp2,me2);
  //double RS3456pee = RS(m33,m44,m55,m66,mp2,me2,me2);
  double RSp45613e = RS(mp2,m44,m55,m66,m11,m33,me2);
  double RSe45613p = RS(me2,m44,m55,m66,m11,m33,mp2);
  double RS14563pe = RS(m11,m44,m55,m66,m33,mp2,me2);
  double RS34561pe = RS(m33,m44,m55,m66,m11,mp2,me2);

  double RC1456pe = RC(m11,m44,m55,m66,mp2,me2);
  double RC3456pe = RC(m33,m44,m55,m66,mp2,me2);

  double M13p6x23LV =
       + AbV(m11,L) * (  - 8./9.*m66*L4r*RD1456ep - 8./27.*pow(m66,2)*
         L8r*pow(RS145pe,2) - 8./27.*pow(m66,2)*L7r*pow(RS145pe,2) - 8./
         27.*pow(m66,2)*L6r*pow(RS145pe,2) - 8./9.*m55*L4r*RD1456ep - 
         16./27.*m55*m66*L7r*RS146pe*RS145pe - 8./27.*m55*m66*L6r*pow(
         RS145pe,2) - 8./27.*m55*m66*L6r*pow(RS146pe,2) - 8./27.*pow(
         m55,2)*L8r*pow(RS146pe,2) - 8./27.*pow(m55,2)*L7r*pow(
         RS146pe,2) - 8./27.*pow(m55,2)*L6r*pow(RS146pe,2) - 8./9.*m44*
         L4r*RD1456ep - 16./27.*m44*m66*L7r*RS156pe*RS145pe - 8./27.*
         m44*m66*L6r*pow(RS145pe,2) - 8./27.*m44*m66*L6r*pow(RS156pe,2)
          - 16./27.*m44*m55*L7r*RS156pe*RS146pe - 8./27.*m44*m55*L6r*
         pow(RS146pe,2) - 8./27.*m44*m55*L6r*pow(RS156pe,2) - 8./27.*
         pow(m44,2)*L8r*pow(RS156pe,2) - 8./27.*pow(m44,2)*L7r*pow(
         RS156pe,2) - 8./27.*pow(m44,2)*L6r*pow(RS156pe,2) + 4./9.*m33*
         L5r*RD1456ep - 16./9.*m33*m66*L7r*RS145pe - 16./9.*m33*m66*L6r
         *RS14563pe - 2./27.*m33*m66*L5r*pow(RS145pe,2) + 8./9.*m33*m66
         *L4r*RS14563pe );

      M13p6x23LV +=  + AbV(m11,L) * ( 2./9.*m33*m66*L4r*RC1456pe - 2./
         27.*m33*m66*L4r*pow(RS145pe,2) - 2./27.*m33*m66*L4r*pow(
         RS146pe,2) - 2./27.*m33*m66*L4r*pow(RS156pe,2) - 16./9.*m33*
         m55*L7r*RS146pe - 16./9.*m33*m55*L6r*RS14563pe - 2./27.*m33*
         m55*L5r*pow(RS146pe,2) + 8./9.*m33*m55*L4r*RS14563pe + 2./9.*
         m33*m55*L4r*RC1456pe - 2./27.*m33*m55*L4r*pow(RS145pe,2) - 2./
         27.*m33*m55*L4r*pow(RS146pe,2) - 2./27.*m33*m55*L4r*pow(
         RS156pe,2) - 16./9.*m33*m44*L7r*RS156pe - 16./9.*m33*m44*L6r*
         RS14563pe - 2./27.*m33*m44*L5r*pow(RS156pe,2) + 8./9.*m33*m44*
         L4r*RS14563pe + 2./9.*m33*m44*L4r*RC1456pe - 2./27.*m33*m44*
         L4r*pow(RS145pe,2) - 2./27.*m33*m44*L4r*pow(RS146pe,2) - 2./27.
         *m33*m44*L4r*pow(RS156pe,2) - 16./9.*pow(m33,2)*L8r*RS14563pe
          + 2./9.*pow(m33,2)*L5r*RS14563pe - 1./9.*pow(m33,2)*L5r*
         RC1456pe - 10./3.*m13*L3r*RD1456ep - 4./3.*m13*L0r*RD1456ep + 
         4./27.*m13*m66*L5r*pow(RS145pe,2) - 4./9.*m13*m66*L4r*RC1456pe
          + 4./27.*m13*m66*L4r*pow(RS145pe,2) );

      M13p6x23LV +=  + AbV(m11,L) * ( 4./27.*m13*m66*L4r*pow(RS146pe,2)
          + 4./27.*m13*m66*L4r*pow(RS156pe,2) + 4./27.*m13*m55*L5r*pow(
         RS146pe,2) - 4./9.*m13*m55*L4r*RC1456pe + 4./27.*m13*m55*L4r*
         pow(RS145pe,2) + 4./27.*m13*m55*L4r*pow(RS146pe,2) + 4./27.*
         m13*m55*L4r*pow(RS156pe,2) + 4./27.*m13*m44*L5r*pow(RS156pe,2)
          - 4./9.*m13*m44*L4r*RC1456pe + 4./27.*m13*m44*L4r*pow(
         RS145pe,2) + 4./27.*m13*m44*L4r*pow(RS146pe,2) + 4./27.*m13*
         m44*L4r*pow(RS156pe,2) - 4./9.*m13*m33*L5r*RS14563pe + 2./9.*
         m13*m33*L5r*RC1456pe - 16./9.*m11*m66*L7r*RS145pe - 32./9.*m11
         *m66*L6r*RS14563pe + 8./9.*m11*m66*L6r*RC1456pe - 8./27.*m11*
         m66*L5r*RS145pe*RS345pe - 8./27.*m11*m66*L5r*RS145pe*RSe453p
          - 8./27.*m11*m66*L5r*RS145pe*RSp453e + 2./27.*m11*m66*L5r*
         pow(RS145pe,2) + 8./27.*m11*m66*L5r*RSe451p*RS145pe + 8./27.*
         m11*m66*L5r*RSp451e*RS145pe + 8./9.*m11*m66*L4r*RS34561pe + 8./
         3.*m11*m66*L4r*RS14563pe + 8./9.*m11*m66*L4r*RSe45613p + 8./9.
         *m11*m66*L4r*RSp45613e );

      M13p6x23LV +=  + AbV(m11,L) * (  - 2./9.*m11*m66*L4r*RC1456pe - 8.
         /9.*m11*m66*L4r*RSe45611p - 8./9.*m11*m66*L4r*RSp45611e - 8./
         27.*m11*m66*L4r*RS145pe*RS345pe - 8./27.*m11*m66*L4r*RS145pe*
         RSe453p - 8./27.*m11*m66*L4r*RS145pe*RSp453e + 2./27.*m11*m66*
         L4r*pow(RS145pe,2) - 8./27.*m11*m66*L4r*RS146pe*RS346pe - 8./
         27.*m11*m66*L4r*RS146pe*RSe463p - 8./27.*m11*m66*L4r*RS146pe*
         RSp463e + 2./27.*m11*m66*L4r*pow(RS146pe,2) - 8./27.*m11*m66*
         L4r*RS156pe*RS356pe - 8./27.*m11*m66*L4r*RS156pe*RSe563p - 8./
         27.*m11*m66*L4r*RS156pe*RSp563e + 2./27.*m11*m66*L4r*pow(
         RS156pe,2) + 8./27.*m11*m66*L4r*RSe561p*RS156pe + 8./27.*m11*
         m66*L4r*RSe461p*RS146pe + 8./27.*m11*m66*L4r*RSe451p*RS145pe
          + 8./27.*m11*m66*L4r*RSp561e*RS156pe + 8./27.*m11*m66*L4r*
         RSp461e*RS146pe + 8./27.*m11*m66*L4r*RSp451e*RS145pe - 16./9.*
         m11*m55*L7r*RS146pe - 32./9.*m11*m55*L6r*RS14563pe + 8./9.*m11
         *m55*L6r*RC1456pe - 8./27.*m11*m55*L5r*RS146pe*RS346pe - 8./27.
         *m11*m55*L5r*RS146pe*RSe463p );

      M13p6x23LV +=  + AbV(m11,L) * (  - 8./27.*m11*m55*L5r*RS146pe*
         RSp463e + 2./27.*m11*m55*L5r*pow(RS146pe,2) + 8./27.*m11*m55*
         L5r*RSe461p*RS146pe + 8./27.*m11*m55*L5r*RSp461e*RS146pe + 8./
         9.*m11*m55*L4r*RS34561pe + 8./3.*m11*m55*L4r*RS14563pe + 8./9.
         *m11*m55*L4r*RSe45613p + 8./9.*m11*m55*L4r*RSp45613e - 2./9.*
         m11*m55*L4r*RC1456pe - 8./9.*m11*m55*L4r*RSe45611p - 8./9.*m11
         *m55*L4r*RSp45611e - 8./27.*m11*m55*L4r*RS145pe*RS345pe - 8./
         27.*m11*m55*L4r*RS145pe*RSe453p - 8./27.*m11*m55*L4r*RS145pe*
         RSp453e + 2./27.*m11*m55*L4r*pow(RS145pe,2) - 8./27.*m11*m55*
         L4r*RS146pe*RS346pe - 8./27.*m11*m55*L4r*RS146pe*RSe463p - 8./
         27.*m11*m55*L4r*RS146pe*RSp463e + 2./27.*m11*m55*L4r*pow(
         RS146pe,2) - 8./27.*m11*m55*L4r*RS156pe*RS356pe - 8./27.*m11*
         m55*L4r*RS156pe*RSe563p - 8./27.*m11*m55*L4r*RS156pe*RSp563e
          + 2./27.*m11*m55*L4r*pow(RS156pe,2) + 8./27.*m11*m55*L4r*
         RSe561p*RS156pe + 8./27.*m11*m55*L4r*RSe461p*RS146pe + 8./27.*
         m11*m55*L4r*RSe451p*RS145pe );

      M13p6x23LV +=  + AbV(m11,L) * ( 8./27.*m11*m55*L4r*RSp561e*
         RS156pe + 8./27.*m11*m55*L4r*RSp461e*RS146pe + 8./27.*m11*m55*
         L4r*RSp451e*RS145pe - 16./9.*m11*m44*L7r*RS156pe - 32./9.*m11*
         m44*L6r*RS14563pe + 8./9.*m11*m44*L6r*RC1456pe - 8./27.*m11*
         m44*L5r*RS156pe*RS356pe - 8./27.*m11*m44*L5r*RS156pe*RSe563p
          - 8./27.*m11*m44*L5r*RS156pe*RSp563e + 2./27.*m11*m44*L5r*
         pow(RS156pe,2) + 8./27.*m11*m44*L5r*RSe561p*RS156pe + 8./27.*
         m11*m44*L5r*RSp561e*RS156pe + 8./9.*m11*m44*L4r*RS34561pe + 8./
         3.*m11*m44*L4r*RS14563pe + 8./9.*m11*m44*L4r*RSe45613p + 8./9.
         *m11*m44*L4r*RSp45613e - 2./9.*m11*m44*L4r*RC1456pe - 8./9.*
         m11*m44*L4r*RSe45611p - 8./9.*m11*m44*L4r*RSp45611e - 8./27.*
         m11*m44*L4r*RS145pe*RS345pe - 8./27.*m11*m44*L4r*RS145pe*
         RSe453p - 8./27.*m11*m44*L4r*RS145pe*RSp453e + 2./27.*m11*m44*
         L4r*pow(RS145pe,2) - 8./27.*m11*m44*L4r*RS146pe*RS346pe - 8./
         27.*m11*m44*L4r*RS146pe*RSe463p - 8./27.*m11*m44*L4r*RS146pe*
         RSp463e );

      M13p6x23LV +=  + AbV(m11,L) * ( 2./27.*m11*m44*L4r*pow(RS146pe,2)
          - 8./27.*m11*m44*L4r*RS156pe*RS356pe - 8./27.*m11*m44*L4r*
         RS156pe*RSe563p - 8./27.*m11*m44*L4r*RS156pe*RSp563e + 2./27.*
         m11*m44*L4r*pow(RS156pe,2) + 8./27.*m11*m44*L4r*RSe561p*
         RS156pe + 8./27.*m11*m44*L4r*RSe461p*RS146pe + 8./27.*m11*m44*
         L4r*RSe451p*RS145pe + 8./27.*m11*m44*L4r*RSp561e*RS156pe + 8./
         27.*m11*m44*L4r*RSp461e*RS146pe + 8./27.*m11*m44*L4r*RSp451e*
         RS145pe - 16./3.*m11*m33*L8r*RS14563pe - 8./3.*m11*m33*L8r*
         RC1456pe + 16./3.*m11*m33*L7r + 4./3.*m11*m33*L5r*RS14563pe + 
         8./9.*m11*m33*L5r*RC1456pe + 4./9.*m11*m13*L5r*RS14563pe + 2./
         3.*m11*m13*L5r*RC1456pe - 4./3.*m11*m13*L3r*RS14563pe - 10./3.
         *m11*m13*L3r*RC1456pe - 16./3.*m11*m13*L0r*RS14563pe - 4./3.*
         m11*m13*L0r*RC1456pe - 32./9.*pow(m11,2)*L8r*RS14563pe - 16./9.
         *pow(m11,2)*L8r*RC1456pe + 8*pow(m11,2)*L7r + 8./9.*pow(m11,2)
         *L5r*RS34561pe + 2*pow(m11,2)*L5r*RS14563pe + 8./9.*pow(m11,2)
         *L5r*RSe45613p );

      M13p6x23LV +=  + AbV(m11,L) * ( 8./9.*pow(m11,2)*L5r*RSp45613e + 
         5./9.*pow(m11,2)*L5r*RC1456pe - 8./9.*pow(m11,2)*L5r*RSe45611p
          - 8./9.*pow(m11,2)*L5r*RSp45611e + 8./9.*RS(m11,m44,m55,m66,
         m33,me2,mp2)*m33*m66*L4r + 8./9.*RS(m11,m44,m55,m66,m33,me2,
         mp2)*m33*m55*L4r + 8./9.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33*
         m44*L4r + 4./9.*RS(m11,m44,m55,m66,m33,me2,mp2)*pow(m33,2)*L5r
          + 16./9.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*m66*L4r + 16./9.
         *RS(m11,m44,m55,m66,m33,me2,mp2)*m11*m55*L4r + 16./9.*RS(m11,
         m44,m55,m66,m33,me2,mp2)*m11*m44*L4r + 4./3.*RS(m11,m44,m55,
         m66,m33,me2,mp2)*m11*m33*L5r + 8./9.*RS(m11,m44,m55,m66,m33,
         me2,mp2)*pow(m11,2)*L5r );

      M13p6x23LV +=  + AbV(m13,L) * ( 8*pow(m33,2)*L6r + 20*pow(m13,2)*
         L2r + 8*pow(m13,2)*L1r + 16*m11*m33*L6r + 8*pow(m11,2)*L6r );

      M13p6x23LV +=  + AbV(m14,L) * (  - 2./3.*m44*m66*L4r - 2./3.*m44*
         m55*L4r - 1./3.*pow(m44,2)*L5r - 2./3.*pow(m44,2)*L4r + 4*m33*
         m44*L8r - 1./3.*m33*m44*L5r + 1./3.*pow(m33,2)*L5r + 4./3.*m14
         *m66*L4r + 4./3.*m14*m55*L4r + 2./3.*m14*m44*L5r + 4./3.*m14*
         m44*L4r - 2*m14*m33*L5r - 4./3.*m13*m44*L5r - 2./3.*m13*m33*
         L5r + 10*m13*m14*L3r + 4*m13*m14*L0r - 2./3.*m11*m66*L4r - 2./
         3.*m11*m55*L4r + 4*m11*m44*L8r - m11*m44*L5r - 2./3.*m11*m44*
         L4r + 4*m11*m33*L8r + 1./3.*m11*m33*L5r - 4./3.*m11*m14*L5r - 
         2*m11*m13*L5r + 4*pow(m11,2)*L8r - 1./3.*pow(m11,2)*L5r );

      M13p6x23LV +=  + AbV(m15,L) * (  - 2./3.*m55*m66*L4r - 1./3.*pow(
         m55,2)*L5r - 2./3.*pow(m55,2)*L4r - 2./3.*m44*m55*L4r + 4*m33*
         m55*L8r - 1./3.*m33*m55*L5r + 1./3.*pow(m33,2)*L5r + 4./3.*m15
         *m66*L4r + 2./3.*m15*m55*L5r + 4./3.*m15*m55*L4r + 4./3.*m15*
         m44*L4r - 2*m15*m33*L5r - 4./3.*m13*m55*L5r - 2./3.*m13*m33*
         L5r + 10*m13*m15*L3r + 4*m13*m15*L0r - 2./3.*m11*m66*L4r + 4*
         m11*m55*L8r - m11*m55*L5r - 2./3.*m11*m55*L4r - 2./3.*m11*m44*
         L4r + 4*m11*m33*L8r + 1./3.*m11*m33*L5r - 4./3.*m11*m15*L5r - 
         2*m11*m13*L5r + 4*pow(m11,2)*L8r - 1./3.*pow(m11,2)*L5r );

      M13p6x23LV +=  + AbV(m16,L) * (  - 1./3.*pow(m66,2)*L5r - 2./3.*
         pow(m66,2)*L4r - 2./3.*m55*m66*L4r - 2./3.*m44*m66*L4r + 4*m33
         *m66*L8r - 1./3.*m33*m66*L5r + 1./3.*pow(m33,2)*L5r + 2./3.*
         m16*m66*L5r + 4./3.*m16*m66*L4r + 4./3.*m16*m55*L4r + 4./3.*
         m16*m44*L4r - 2*m16*m33*L5r - 4./3.*m13*m66*L5r - 2./3.*m13*
         m33*L5r + 10*m13*m16*L3r + 4*m13*m16*L0r + 4*m11*m66*L8r - m11
         *m66*L5r - 2./3.*m11*m66*L4r - 2./3.*m11*m55*L4r - 2./3.*m11*
         m44*L4r + 4*m11*m33*L8r + 1./3.*m11*m33*L5r - 4./3.*m11*m16*
         L5r - 2*m11*m13*L5r + 4*pow(m11,2)*L8r - 1./3.*pow(m11,2)*L5r
          );

      M13p6x23LV +=  + AbV(m33,L) * (  - 8./9.*m66*L4r*RD3456ep - 8./27.
         *pow(m66,2)*L8r*pow(RS345pe,2) + 16./27.*pow(m66,2)*L8r*
         RS145pe*RS345pe - 8./27.*pow(m66,2)*L7r*pow(RS345pe,2) + 16./
         27.*pow(m66,2)*L7r*RS145pe*RS345pe - 8./27.*pow(m66,2)*L6r*
         pow(RS345pe,2) + 16./27.*pow(m66,2)*L6r*RS145pe*RS345pe - 8./9.
         *m55*L4r*RD3456ep - 16./27.*m55*m66*L7r*RS346pe*RS345pe + 16./
         27.*m55*m66*L7r*RS145pe*RS346pe + 16./27.*m55*m66*L7r*RS146pe*
         RS345pe - 8./27.*m55*m66*L6r*pow(RS345pe,2) - 8./27.*m55*m66*
         L6r*pow(RS346pe,2) + 16./27.*m55*m66*L6r*RS145pe*RS345pe + 16./
         27.*m55*m66*L6r*RS146pe*RS346pe - 8./27.*pow(m55,2)*L8r*pow(
         RS346pe,2) + 16./27.*pow(m55,2)*L8r*RS146pe*RS346pe - 8./27.*
         pow(m55,2)*L7r*pow(RS346pe,2) + 16./27.*pow(m55,2)*L7r*RS146pe
         *RS346pe - 8./27.*pow(m55,2)*L6r*pow(RS346pe,2) + 16./27.*pow(
         m55,2)*L6r*RS146pe*RS346pe - 8./9.*m44*L4r*RD3456ep - 16./27.*
         m44*m66*L7r*RS356pe*RS345pe + 16./27.*m44*m66*L7r*RS145pe*
         RS356pe );

      M13p6x23LV +=  + AbV(m33,L) * ( 16./27.*m44*m66*L7r*RS156pe*
         RS345pe - 8./27.*m44*m66*L6r*pow(RS345pe,2) - 8./27.*m44*m66*
         L6r*pow(RS356pe,2) + 16./27.*m44*m66*L6r*RS145pe*RS345pe + 16./
         27.*m44*m66*L6r*RS156pe*RS356pe - 16./27.*m44*m55*L7r*RS356pe*
         RS346pe + 16./27.*m44*m55*L7r*RS146pe*RS356pe + 16./27.*m44*
         m55*L7r*RS156pe*RS346pe - 8./27.*m44*m55*L6r*pow(RS346pe,2) - 
         8./27.*m44*m55*L6r*pow(RS356pe,2) + 16./27.*m44*m55*L6r*
         RS146pe*RS346pe + 16./27.*m44*m55*L6r*RS156pe*RS356pe - 8./27.
         *pow(m44,2)*L8r*pow(RS356pe,2) + 16./27.*pow(m44,2)*L8r*
         RS156pe*RS356pe - 8./27.*pow(m44,2)*L7r*pow(RS356pe,2) + 16./
         27.*pow(m44,2)*L7r*RS156pe*RS356pe - 8./27.*pow(m44,2)*L6r*
         pow(RS356pe,2) + 16./27.*pow(m44,2)*L6r*RS156pe*RS356pe - 16./
         9.*m33*m66*L7r*RS345pe - 16./9.*m33*m66*L7r*RS145pe - 32./9.*
         m33*m66*L6r*RS34561pe - 16./9.*m33*m66*L6r*RS14563pe + 8./9.*
         m33*m66*L6r*RC3456pe + 2./27.*m33*m66*L5r*pow(RS345pe,2) + 8./
         27.*m33*m66*L5r*RSe453p*RS345pe );

      M13p6x23LV +=  + AbV(m33,L) * ( 8./27.*m33*m66*L5r*RSp453e*
         RS345pe - 4./9.*m33*m66*L5r*RS145pe*RS345pe - 8./27.*m33*m66*
         L5r*RSe451p*RS345pe - 8./27.*m33*m66*L5r*RSp451e*RS345pe + 28./
         9.*m33*m66*L4r*RS34561pe + 4./3.*m33*m66*L4r*RS14563pe + 8./9.
         *m33*m66*L4r*RSe45613p + 8./9.*m33*m66*L4r*RSp45613e - 2./9.*
         m33*m66*L4r*RC3456pe - 8./9.*m33*m66*L4r*RSe45633p - 8./9.*m33
         *m66*L4r*RSp45633e + 2./27.*m33*m66*L4r*pow(RS345pe,2) + 2./27.
         *m33*m66*L4r*pow(RS346pe,2) + 2./27.*m33*m66*L4r*pow(
         RS356pe,2) + 8./27.*m33*m66*L4r*RSe563p*RS356pe + 8./27.*m33*
         m66*L4r*RSe463p*RS346pe + 8./27.*m33*m66*L4r*RSe453p*RS345pe
          + 8./27.*m33*m66*L4r*RSp563e*RS356pe + 8./27.*m33*m66*L4r*
         RSp463e*RS346pe + 8./27.*m33*m66*L4r*RSp453e*RS345pe - 4./9.*
         m33*m66*L4r*RS145pe*RS345pe - 4./9.*m33*m66*L4r*RS146pe*
         RS346pe - 4./9.*m33*m66*L4r*RS156pe*RS356pe - 8./27.*m33*m66*
         L4r*RSe561p*RS356pe - 8./27.*m33*m66*L4r*RSe461p*RS346pe - 8./
         27.*m33*m66*L4r*RSe451p*RS345pe );

      M13p6x23LV +=  + AbV(m33,L) * (  - 8./27.*m33*m66*L4r*RSp561e*
         RS356pe - 8./27.*m33*m66*L4r*RSp461e*RS346pe - 8./27.*m33*m66*
         L4r*RSp451e*RS345pe - 16./9.*m33*m55*L7r*RS346pe - 16./9.*m33*
         m55*L7r*RS146pe - 32./9.*m33*m55*L6r*RS34561pe - 16./9.*m33*
         m55*L6r*RS14563pe + 8./9.*m33*m55*L6r*RC3456pe + 2./27.*m33*
         m55*L5r*pow(RS346pe,2) + 8./27.*m33*m55*L5r*RSe463p*RS346pe + 
         8./27.*m33*m55*L5r*RSp463e*RS346pe - 4./9.*m33*m55*L5r*RS146pe
         *RS346pe - 8./27.*m33*m55*L5r*RSe461p*RS346pe - 8./27.*m33*m55
         *L5r*RSp461e*RS346pe + 28./9.*m33*m55*L4r*RS34561pe + 4./3.*
         m33*m55*L4r*RS14563pe + 8./9.*m33*m55*L4r*RSe45613p + 8./9.*
         m33*m55*L4r*RSp45613e - 2./9.*m33*m55*L4r*RC3456pe - 8./9.*m33
         *m55*L4r*RSe45633p - 8./9.*m33*m55*L4r*RSp45633e + 2./27.*m33*
         m55*L4r*pow(RS345pe,2) + 2./27.*m33*m55*L4r*pow(RS346pe,2) + 2.
         /27.*m33*m55*L4r*pow(RS356pe,2) + 8./27.*m33*m55*L4r*RSe563p*
         RS356pe + 8./27.*m33*m55*L4r*RSe463p*RS346pe + 8./27.*m33*m55*
         L4r*RSe453p*RS345pe );

      M13p6x23LV +=  + AbV(m33,L) * ( 8./27.*m33*m55*L4r*RSp563e*
         RS356pe + 8./27.*m33*m55*L4r*RSp463e*RS346pe + 8./27.*m33*m55*
         L4r*RSp453e*RS345pe - 4./9.*m33*m55*L4r*RS145pe*RS345pe - 4./9.
         *m33*m55*L4r*RS146pe*RS346pe - 4./9.*m33*m55*L4r*RS156pe*
         RS356pe - 8./27.*m33*m55*L4r*RSe561p*RS356pe - 8./27.*m33*m55*
         L4r*RSe461p*RS346pe - 8./27.*m33*m55*L4r*RSe451p*RS345pe - 8./
         27.*m33*m55*L4r*RSp561e*RS356pe - 8./27.*m33*m55*L4r*RSp461e*
         RS346pe - 8./27.*m33*m55*L4r*RSp451e*RS345pe - 16./9.*m33*m44*
         L7r*RS356pe - 16./9.*m33*m44*L7r*RS156pe - 32./9.*m33*m44*L6r*
         RS34561pe - 16./9.*m33*m44*L6r*RS14563pe + 8./9.*m33*m44*L6r*
         RC3456pe + 2./27.*m33*m44*L5r*pow(RS356pe,2) + 8./27.*m33*m44*
         L5r*RSe563p*RS356pe + 8./27.*m33*m44*L5r*RSp563e*RS356pe - 4./
         9.*m33*m44*L5r*RS156pe*RS356pe - 8./27.*m33*m44*L5r*RSe561p*
         RS356pe - 8./27.*m33*m44*L5r*RSp561e*RS356pe + 28./9.*m33*m44*
         L4r*RS34561pe + 4./3.*m33*m44*L4r*RS14563pe + 8./9.*m33*m44*
         L4r*RSe45613p );

      M13p6x23LV +=  + AbV(m33,L) * ( 8./9.*m33*m44*L4r*RSp45613e - 2./
         9.*m33*m44*L4r*RC3456pe - 8./9.*m33*m44*L4r*RSe45633p - 8./9.*
         m33*m44*L4r*RSp45633e + 2./27.*m33*m44*L4r*pow(RS345pe,2) + 2./
         27.*m33*m44*L4r*pow(RS346pe,2) + 2./27.*m33*m44*L4r*pow(
         RS356pe,2) + 8./27.*m33*m44*L4r*RSe563p*RS356pe + 8./27.*m33*
         m44*L4r*RSe463p*RS346pe + 8./27.*m33*m44*L4r*RSe453p*RS345pe
          + 8./27.*m33*m44*L4r*RSp563e*RS356pe + 8./27.*m33*m44*L4r*
         RSp463e*RS346pe + 8./27.*m33*m44*L4r*RSp453e*RS345pe - 4./9.*
         m33*m44*L4r*RS145pe*RS345pe - 4./9.*m33*m44*L4r*RS146pe*
         RS346pe - 4./9.*m33*m44*L4r*RS156pe*RS356pe - 8./27.*m33*m44*
         L4r*RSe561p*RS356pe - 8./27.*m33*m44*L4r*RSe461p*RS346pe - 8./
         27.*m33*m44*L4r*RSe451p*RS345pe - 8./27.*m33*m44*L4r*RSp561e*
         RS356pe - 8./27.*m33*m44*L4r*RSp461e*RS346pe - 8./27.*m33*m44*
         L4r*RSp451e*RS345pe - 32./9.*pow(m33,2)*L8r*RS34561pe - 16./9.
         *pow(m33,2)*L8r*RS14563pe - 16./9.*pow(m33,2)*L8r*RC3456pe + 8
         *pow(m33,2)*L7r );

      M13p6x23LV +=  + AbV(m33,L) * ( 2*pow(m33,2)*L5r*RS34561pe + 4./3.
         *pow(m33,2)*L5r*RS14563pe + 8./9.*pow(m33,2)*L5r*RSe45613p + 8.
         /9.*pow(m33,2)*L5r*RSp45613e + 5./9.*pow(m33,2)*L5r*RC3456pe
          - 8./9.*pow(m33,2)*L5r*RSe45633p - 8./9.*pow(m33,2)*L5r*
         RSp45633e - 10./3.*m13*L3r*RD3456ep - 4./3.*m13*L0r*RD3456ep
          + 4./27.*m13*m66*L5r*pow(RS345pe,2) - 8./27.*m13*m66*L5r*
         RS145pe*RS345pe + 8./9.*m13*m66*L4r*RS34561pe + 8./9.*m13*m66*
         L4r*RS14563pe - 4./9.*m13*m66*L4r*RC3456pe + 4./27.*m13*m66*
         L4r*pow(RS345pe,2) + 4./27.*m13*m66*L4r*pow(RS346pe,2) + 4./27.
         *m13*m66*L4r*pow(RS356pe,2) - 8./27.*m13*m66*L4r*RS145pe*
         RS345pe - 8./27.*m13*m66*L4r*RS146pe*RS346pe - 8./27.*m13*m66*
         L4r*RS156pe*RS356pe + 4./27.*m13*m55*L5r*pow(RS346pe,2) - 8./
         27.*m13*m55*L5r*RS146pe*RS346pe + 8./9.*m13*m55*L4r*RS34561pe
          + 8./9.*m13*m55*L4r*RS14563pe - 4./9.*m13*m55*L4r*RC3456pe + 
         4./27.*m13*m55*L4r*pow(RS345pe,2) + 4./27.*m13*m55*L4r*pow(
         RS346pe,2) );

      M13p6x23LV +=  + AbV(m33,L) * ( 4./27.*m13*m55*L4r*pow(RS356pe,2)
          - 8./27.*m13*m55*L4r*RS145pe*RS345pe - 8./27.*m13*m55*L4r*
         RS146pe*RS346pe - 8./27.*m13*m55*L4r*RS156pe*RS356pe + 4./27.*
         m13*m44*L5r*pow(RS356pe,2) - 8./27.*m13*m44*L5r*RS156pe*
         RS356pe + 8./9.*m13*m44*L4r*RS34561pe + 8./9.*m13*m44*L4r*
         RS14563pe - 4./9.*m13*m44*L4r*RC3456pe + 4./27.*m13*m44*L4r*
         pow(RS345pe,2) + 4./27.*m13*m44*L4r*pow(RS346pe,2) + 4./27.*
         m13*m44*L4r*pow(RS356pe,2) - 8./27.*m13*m44*L4r*RS145pe*
         RS345pe - 8./27.*m13*m44*L4r*RS146pe*RS346pe - 8./27.*m13*m44*
         L4r*RS156pe*RS356pe + 4./9.*m13*m33*L5r*RS34561pe + 8./9.*m13*
         m33*L5r*RS14563pe + 2./3.*m13*m33*L5r*RC3456pe - 4./3.*m13*m33
         *L3r*RS34561pe - 10./3.*m13*m33*L3r*RC3456pe - 16./3.*m13*m33*
         L0r*RS34561pe - 4./3.*m13*m33*L0r*RC3456pe + 4./9.*m11*L5r*
         RD3456ep - 32./9.*m11*m66*L7r*RS345pe - 32./9.*m11*m66*L6r*
         RS34561pe - 2./27.*m11*m66*L5r*pow(RS345pe,2) - 4./27.*m11*m66
         *L5r*RS145pe*RS345pe );

      M13p6x23LV +=  + AbV(m33,L) * ( 4./3.*m11*m66*L4r*RS34561pe + 4./
         9.*m11*m66*L4r*RS14563pe + 2./9.*m11*m66*L4r*RC3456pe - 2./27.
         *m11*m66*L4r*pow(RS345pe,2) - 2./27.*m11*m66*L4r*pow(
         RS346pe,2) - 2./27.*m11*m66*L4r*pow(RS356pe,2) - 4./27.*m11*
         m66*L4r*RS145pe*RS345pe - 4./27.*m11*m66*L4r*RS146pe*RS346pe
          - 4./27.*m11*m66*L4r*RS156pe*RS356pe - 32./9.*m11*m55*L7r*
         RS346pe - 32./9.*m11*m55*L6r*RS34561pe - 2./27.*m11*m55*L5r*
         pow(RS346pe,2) - 4./27.*m11*m55*L5r*RS146pe*RS346pe + 4./3.*
         m11*m55*L4r*RS34561pe + 4./9.*m11*m55*L4r*RS14563pe + 2./9.*
         m11*m55*L4r*RC3456pe - 2./27.*m11*m55*L4r*pow(RS345pe,2) - 2./
         27.*m11*m55*L4r*pow(RS346pe,2) - 2./27.*m11*m55*L4r*pow(
         RS356pe,2) - 4./27.*m11*m55*L4r*RS145pe*RS345pe - 4./27.*m11*
         m55*L4r*RS146pe*RS346pe - 4./27.*m11*m55*L4r*RS156pe*RS356pe
          - 32./9.*m11*m44*L7r*RS356pe - 32./9.*m11*m44*L6r*RS34561pe
          - 2./27.*m11*m44*L5r*pow(RS356pe,2) - 4./27.*m11*m44*L5r*
         RS156pe*RS356pe );

      M13p6x23LV +=  + AbV(m33,L) * ( 4./3.*m11*m44*L4r*RS34561pe + 4./
         9.*m11*m44*L4r*RS14563pe + 2./9.*m11*m44*L4r*RC3456pe - 2./27.
         *m11*m44*L4r*pow(RS345pe,2) - 2./27.*m11*m44*L4r*pow(
         RS346pe,2) - 2./27.*m11*m44*L4r*pow(RS356pe,2) - 4./27.*m11*
         m44*L4r*RS145pe*RS345pe - 4./27.*m11*m44*L4r*RS146pe*RS346pe
          - 4./27.*m11*m44*L4r*RS156pe*RS356pe - 16./3.*m11*m33*L8r*
         RS34561pe - 8./3.*m11*m33*L8r*RC3456pe + 32./3.*m11*m33*L7r + 
         16./9.*m11*m33*L5r*RS34561pe + 4./9.*m11*m33*L5r*RS14563pe + 8.
         /9.*m11*m33*L5r*RC3456pe + 4./9.*m11*m13*L5r*RS34561pe + 2./9.
         *m11*m13*L5r*RC3456pe - 32./9.*pow(m11,2)*L8r*RS34561pe + 2./3.
         *pow(m11,2)*L5r*RS34561pe - 1./9.*pow(m11,2)*L5r*RC3456pe + 16.
         /9.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33*m66*L4r + 16./9.*RS(
         m33,m44,m55,m66,m11,me2,mp2)*m33*m55*L4r + 16./9.*RS(m33,m44,
         m55,m66,m11,me2,mp2)*m33*m44*L4r + 8./9.*RS(m33,m44,m55,m66,
         m11,me2,mp2)*pow(m33,2)*L5r + 8./9.*RS(m33,m44,m55,m66,m11,me2
         ,mp2)*m11*m66*L4r );

      M13p6x23LV +=  + AbV(m33,L) * ( 8./9.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11*m55*L4r + 8./9.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         m44*L4r + 4./3.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*m33*L5r + 
         4./9.*RS(m33,m44,m55,m66,m11,me2,mp2)*pow(m11,2)*L5r );

      M13p6x23LV +=  + AbV(m34,L) * (  - 2./3.*m44*m66*L4r - 2./3.*m44*
         m55*L4r - 1./3.*pow(m44,2)*L5r - 2./3.*pow(m44,2)*L4r + 4./3.*
         m34*m66*L4r + 4./3.*m34*m55*L4r + 2./3.*m34*m44*L5r + 4./3.*
         m34*m44*L4r - 2./3.*m33*m66*L4r - 2./3.*m33*m55*L4r + 4*m33*
         m44*L8r - m33*m44*L5r - 2./3.*m33*m44*L4r - 4./3.*m33*m34*L5r
          + 4*pow(m33,2)*L8r - 1./3.*pow(m33,2)*L5r - 4./3.*m13*m44*L5r
          + 10*m13*m34*L3r + 4*m13*m34*L0r - 2*m13*m33*L5r + 4*m11*m44*
         L8r - 1./3.*m11*m44*L5r - 2*m11*m34*L5r + 4*m11*m33*L8r + 1./3.
         *m11*m33*L5r - 2./3.*m11*m13*L5r + 1./3.*pow(m11,2)*L5r );

      M13p6x23LV +=  + AbV(m35,L) * (  - 2./3.*m55*m66*L4r - 1./3.*pow(
         m55,2)*L5r - 2./3.*pow(m55,2)*L4r - 2./3.*m44*m55*L4r + 4./3.*
         m35*m66*L4r + 2./3.*m35*m55*L5r + 4./3.*m35*m55*L4r + 4./3.*
         m35*m44*L4r - 2./3.*m33*m66*L4r + 4*m33*m55*L8r - m33*m55*L5r
          - 2./3.*m33*m55*L4r - 2./3.*m33*m44*L4r - 4./3.*m33*m35*L5r
          + 4*pow(m33,2)*L8r - 1./3.*pow(m33,2)*L5r - 4./3.*m13*m55*L5r
          + 10*m13*m35*L3r + 4*m13*m35*L0r - 2*m13*m33*L5r + 4*m11*m55*
         L8r - 1./3.*m11*m55*L5r - 2*m11*m35*L5r + 4*m11*m33*L8r + 1./3.
         *m11*m33*L5r - 2./3.*m11*m13*L5r + 1./3.*pow(m11,2)*L5r );

      M13p6x23LV +=  + AbV(m36,L) * (  - 1./3.*pow(m66,2)*L5r - 2./3.*
         pow(m66,2)*L4r - 2./3.*m55*m66*L4r - 2./3.*m44*m66*L4r + 2./3.
         *m36*m66*L5r + 4./3.*m36*m66*L4r + 4./3.*m36*m55*L4r + 4./3.*
         m36*m44*L4r + 4*m33*m66*L8r - m33*m66*L5r - 2./3.*m33*m66*L4r
          - 2./3.*m33*m55*L4r - 2./3.*m33*m44*L4r - 4./3.*m33*m36*L5r
          + 4*pow(m33,2)*L8r - 1./3.*pow(m33,2)*L5r - 4./3.*m13*m66*L5r
          + 10*m13*m36*L3r + 4*m13*m36*L0r - 2*m13*m33*L5r + 4*m11*m66*
         L8r - 1./3.*m11*m66*L5r - 2*m11*m36*L5r + 4*m11*m33*L8r + 1./3.
         *m11*m33*L5r - 2./3.*m11*m13*L5r + 1./3.*pow(m11,2)*L5r );

      M13p6x23LV +=  + AbV(m44,L) * ( 8*m33*m44*L6r - 8./3.*m33*m44*L6r
         *RS456pe - 4*m33*m44*L4r + 4./3.*m33*m44*L4r*RS456pe - 8*m13*
         m44*L4r + 8./3.*m13*m44*L4r*RS456pe + 4*m13*m44*L2r - 4./3.*
         m13*m44*L2r*RS456pe + 16*m13*m44*L1r - 16./3.*m13*m44*L1r*
         RS456pe + 8*m11*m44*L6r - 8./3.*m11*m44*L6r*RS456pe - 4*m11*
         m44*L4r + 4./3.*m11*m44*L4r*RS456pe );

      M13p6x23LV +=  + AbV(m45,L) * ( 8*m33*m55*L6r - 8*m33*m45*L4r + 8
         *m33*m44*L6r - 8*m13*m55*L4r + 8*m13*m45*L2r + 32*m13*m45*L1r
          - 8*m13*m44*L4r + 8*m11*m55*L6r - 8*m11*m45*L4r + 8*m11*m44*
         L6r );

      M13p6x23LV +=  + AbV(m46,L) * ( 8*m33*m66*L6r - 8*m33*m46*L4r + 8
         *m33*m44*L6r - 8*m13*m66*L4r + 8*m13*m46*L2r + 32*m13*m46*L1r
          - 8*m13*m44*L4r + 8*m11*m66*L6r - 8*m11*m46*L4r + 8*m11*m44*
         L6r );

      M13p6x23LV +=  + AbV(m55,L) * ( 8*m33*m55*L6r - 8./3.*m33*m55*L6r
         *RS546pe - 4*m33*m55*L4r + 4./3.*m33*m55*L4r*RS546pe - 8*m13*
         m55*L4r + 8./3.*m13*m55*L4r*RS546pe + 4*m13*m55*L2r - 4./3.*
         m13*m55*L2r*RS546pe + 16*m13*m55*L1r - 16./3.*m13*m55*L1r*
         RS546pe + 8*m11*m55*L6r - 8./3.*m11*m55*L6r*RS546pe - 4*m11*
         m55*L4r + 4./3.*m11*m55*L4r*RS546pe );

      M13p6x23LV +=  + AbV(m56,L) * ( 8*m33*m66*L6r - 8*m33*m56*L4r + 8
         *m33*m55*L6r - 8*m13*m66*L4r + 8*m13*m56*L2r + 32*m13*m56*L1r
          - 8*m13*m55*L4r + 8*m11*m66*L6r - 8*m11*m56*L4r + 8*m11*m55*
         L6r );

      M13p6x23LV +=  + AbV(m66,L) * ( 8*m33*m66*L6r - 8./3.*m33*m66*L6r
         *RS645pe - 4*m33*m66*L4r + 4./3.*m33*m66*L4r*RS645pe - 8*m13*
         m66*L4r + 8./3.*m13*m66*L4r*RS645pe + 4*m13*m66*L2r - 4./3.*
         m13*m66*L2r*RS645pe + 16*m13*m66*L1r - 16./3.*m13*m66*L1r*
         RS645pe + 8*m11*m66*L6r - 8./3.*m11*m66*L6r*RS645pe - 4*m11*
         m66*L4r + 4./3.*m11*m66*L4r*RS645pe );

      M13p6x23LV +=  + AbV(mp2,L) * ( 8./27.*m66*mp2*L5r*RSp453e*
         RS345pe + 8./27.*m66*mp2*L5r*RSp453e*RSe453p + 8./27.*m66*mp2*
         L5r*pow(RSp453e,2) - 8./27.*m66*mp2*L5r*RS145pe*RSp453e - 8./
         27.*m66*mp2*L5r*RSe451p*RSp453e - 8./27.*m66*mp2*L5r*RSp451e*
         RS345pe - 8./27.*m66*mp2*L5r*RSp451e*RSe453p - 16./27.*m66*mp2
         *L5r*RSp451e*RSp453e + 8./27.*m66*mp2*L5r*RSp451e*RS145pe + 8./
         27.*m66*mp2*L5r*RSp451e*RSe451p + 8./27.*m66*mp2*L5r*pow(
         RSp451e,2) + 16./9.*m66*mp2*L4r*RSp45613e - 8./9.*m66*mp2*L4r*
         RSp45633e + 8./27.*m66*mp2*L4r*RSp563e*RS356pe + 8./27.*m66*
         mp2*L4r*RSp563e*RSe563p + 8./27.*m66*mp2*L4r*pow(RSp563e,2) + 
         8./27.*m66*mp2*L4r*RSp463e*RS346pe + 8./27.*m66*mp2*L4r*
         RSp463e*RSe463p + 8./27.*m66*mp2*L4r*pow(RSp463e,2) + 8./27.*
         m66*mp2*L4r*RSp453e*RS345pe + 8./27.*m66*mp2*L4r*RSp453e*
         RSe453p + 8./27.*m66*mp2*L4r*pow(RSp453e,2) - 8./9.*m66*mp2*
         L4r*RSp45611e - 8./27.*m66*mp2*L4r*RS145pe*RSp453e - 8./27.*
         m66*mp2*L4r*RS146pe*RSp463e );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 8./27.*m66*mp2*L4r*RS156pe*
         RSp563e - 8./27.*m66*mp2*L4r*RSe561p*RSp563e - 8./27.*m66*mp2*
         L4r*RSe461p*RSp463e - 8./27.*m66*mp2*L4r*RSe451p*RSp453e - 8./
         27.*m66*mp2*L4r*RSp561e*RS356pe - 8./27.*m66*mp2*L4r*RSp561e*
         RSe563p - 16./27.*m66*mp2*L4r*RSp561e*RSp563e + 8./27.*m66*mp2
         *L4r*RSp561e*RS156pe + 8./27.*m66*mp2*L4r*RSp561e*RSe561p + 8./
         27.*m66*mp2*L4r*pow(RSp561e,2) - 8./27.*m66*mp2*L4r*RSp461e*
         RS346pe - 8./27.*m66*mp2*L4r*RSp461e*RSe463p - 16./27.*m66*mp2
         *L4r*RSp461e*RSp463e + 8./27.*m66*mp2*L4r*RSp461e*RS146pe + 8./
         27.*m66*mp2*L4r*RSp461e*RSe461p + 8./27.*m66*mp2*L4r*pow(
         RSp461e,2) - 8./27.*m66*mp2*L4r*RSp451e*RS345pe - 8./27.*m66*
         mp2*L4r*RSp451e*RSe453p - 16./27.*m66*mp2*L4r*RSp451e*RSp453e
          + 8./27.*m66*mp2*L4r*RSp451e*RS145pe + 8./27.*m66*mp2*L4r*
         RSp451e*RSe451p + 8./27.*m66*mp2*L4r*pow(RSp451e,2) - 16./27.*
         pow(m66,2)*L8r*RSp453e*RS345pe - 16./27.*pow(m66,2)*L8r*
         RSp453e*RSe453p );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 8./27.*pow(m66,2)*L8r*pow(
         RSp453e,2) + 16./27.*pow(m66,2)*L8r*RS145pe*RSp453e + 16./27.*
         pow(m66,2)*L8r*RSe451p*RSp453e + 16./27.*pow(m66,2)*L8r*
         RSp451e*RS345pe + 16./27.*pow(m66,2)*L8r*RSp451e*RSe453p + 16./
         27.*pow(m66,2)*L8r*RSp451e*RSp453e - 16./27.*pow(m66,2)*L8r*
         RSp451e*RS145pe - 16./27.*pow(m66,2)*L8r*RSp451e*RSe451p - 8./
         27.*pow(m66,2)*L8r*pow(RSp451e,2) - 16./27.*pow(m66,2)*L7r*
         RSp453e*RS345pe - 16./27.*pow(m66,2)*L7r*RSp453e*RSe453p - 8./
         27.*pow(m66,2)*L7r*pow(RSp453e,2) + 16./27.*pow(m66,2)*L7r*
         RS145pe*RSp453e + 16./27.*pow(m66,2)*L7r*RSe451p*RSp453e + 16./
         27.*pow(m66,2)*L7r*RSp451e*RS345pe + 16./27.*pow(m66,2)*L7r*
         RSp451e*RSe453p + 16./27.*pow(m66,2)*L7r*RSp451e*RSp453e - 16./
         27.*pow(m66,2)*L7r*RSp451e*RS145pe - 16./27.*pow(m66,2)*L7r*
         RSp451e*RSe451p - 8./27.*pow(m66,2)*L7r*pow(RSp451e,2) - 16./
         27.*pow(m66,2)*L6r*RSp453e*RS345pe - 16./27.*pow(m66,2)*L6r*
         RSp453e*RSe453p );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 8./27.*pow(m66,2)*L6r*pow(
         RSp453e,2) + 16./27.*pow(m66,2)*L6r*RS145pe*RSp453e + 16./27.*
         pow(m66,2)*L6r*RSe451p*RSp453e + 16./27.*pow(m66,2)*L6r*
         RSp451e*RS345pe + 16./27.*pow(m66,2)*L6r*RSp451e*RSe453p + 16./
         27.*pow(m66,2)*L6r*RSp451e*RSp453e - 16./27.*pow(m66,2)*L6r*
         RSp451e*RS145pe - 16./27.*pow(m66,2)*L6r*RSp451e*RSe451p - 8./
         27.*pow(m66,2)*L6r*pow(RSp451e,2) + 8./27.*m55*mp2*L5r*RSp463e
         *RS346pe + 8./27.*m55*mp2*L5r*RSp463e*RSe463p + 8./27.*m55*mp2
         *L5r*pow(RSp463e,2) - 8./27.*m55*mp2*L5r*RS146pe*RSp463e - 8./
         27.*m55*mp2*L5r*RSe461p*RSp463e - 8./27.*m55*mp2*L5r*RSp461e*
         RS346pe - 8./27.*m55*mp2*L5r*RSp461e*RSe463p - 16./27.*m55*mp2
         *L5r*RSp461e*RSp463e + 8./27.*m55*mp2*L5r*RSp461e*RS146pe + 8./
         27.*m55*mp2*L5r*RSp461e*RSe461p + 8./27.*m55*mp2*L5r*pow(
         RSp461e,2) + 16./9.*m55*mp2*L4r*RSp45613e - 8./9.*m55*mp2*L4r*
         RSp45633e + 8./27.*m55*mp2*L4r*RSp563e*RS356pe + 8./27.*m55*
         mp2*L4r*RSp563e*RSe563p );

      M13p6x23LV +=  + AbV(mp2,L) * ( 8./27.*m55*mp2*L4r*pow(RSp563e,2)
          + 8./27.*m55*mp2*L4r*RSp463e*RS346pe + 8./27.*m55*mp2*L4r*
         RSp463e*RSe463p + 8./27.*m55*mp2*L4r*pow(RSp463e,2) + 8./27.*
         m55*mp2*L4r*RSp453e*RS345pe + 8./27.*m55*mp2*L4r*RSp453e*
         RSe453p + 8./27.*m55*mp2*L4r*pow(RSp453e,2) - 8./9.*m55*mp2*
         L4r*RSp45611e - 8./27.*m55*mp2*L4r*RS145pe*RSp453e - 8./27.*
         m55*mp2*L4r*RS146pe*RSp463e - 8./27.*m55*mp2*L4r*RS156pe*
         RSp563e - 8./27.*m55*mp2*L4r*RSe561p*RSp563e - 8./27.*m55*mp2*
         L4r*RSe461p*RSp463e - 8./27.*m55*mp2*L4r*RSe451p*RSp453e - 8./
         27.*m55*mp2*L4r*RSp561e*RS356pe - 8./27.*m55*mp2*L4r*RSp561e*
         RSe563p - 16./27.*m55*mp2*L4r*RSp561e*RSp563e + 8./27.*m55*mp2
         *L4r*RSp561e*RS156pe + 8./27.*m55*mp2*L4r*RSp561e*RSe561p + 8./
         27.*m55*mp2*L4r*pow(RSp561e,2) - 8./27.*m55*mp2*L4r*RSp461e*
         RS346pe - 8./27.*m55*mp2*L4r*RSp461e*RSe463p - 16./27.*m55*mp2
         *L4r*RSp461e*RSp463e + 8./27.*m55*mp2*L4r*RSp461e*RS146pe + 8./
         27.*m55*mp2*L4r*RSp461e*RSe461p );

      M13p6x23LV +=  + AbV(mp2,L) * ( 8./27.*m55*mp2*L4r*pow(RSp461e,2)
          - 8./27.*m55*mp2*L4r*RSp451e*RS345pe - 8./27.*m55*mp2*L4r*
         RSp451e*RSe453p - 16./27.*m55*mp2*L4r*RSp451e*RSp453e + 8./27.
         *m55*mp2*L4r*RSp451e*RS145pe + 8./27.*m55*mp2*L4r*RSp451e*
         RSe451p + 8./27.*m55*mp2*L4r*pow(RSp451e,2) - 16./27.*m55*m66*
         L7r*RSp463e*RS345pe - 16./27.*m55*m66*L7r*RSp463e*RSe453p - 16.
         /27.*m55*m66*L7r*RSp453e*RS346pe - 16./27.*m55*m66*L7r*RSp453e
         *RSe463p - 16./27.*m55*m66*L7r*RSp453e*RSp463e + 16./27.*m55*
         m66*L7r*RS145pe*RSp463e + 16./27.*m55*m66*L7r*RS146pe*RSp453e
          + 16./27.*m55*m66*L7r*RSe461p*RSp453e + 16./27.*m55*m66*L7r*
         RSe451p*RSp463e + 16./27.*m55*m66*L7r*RSp461e*RS345pe + 16./27.
         *m55*m66*L7r*RSp461e*RSe453p + 16./27.*m55*m66*L7r*RSp461e*
         RSp453e - 16./27.*m55*m66*L7r*RSp461e*RS145pe - 16./27.*m55*
         m66*L7r*RSp461e*RSe451p + 16./27.*m55*m66*L7r*RSp451e*RS346pe
          + 16./27.*m55*m66*L7r*RSp451e*RSe463p + 16./27.*m55*m66*L7r*
         RSp451e*RSp463e );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 16./27.*m55*m66*L7r*RSp451e*
         RS146pe - 16./27.*m55*m66*L7r*RSp451e*RSe461p - 16./27.*m55*
         m66*L7r*RSp451e*RSp461e - 16./27.*m55*m66*L6r*RSp463e*RS346pe
          - 16./27.*m55*m66*L6r*RSp463e*RSe463p - 8./27.*m55*m66*L6r*
         pow(RSp463e,2) - 16./27.*m55*m66*L6r*RSp453e*RS345pe - 16./27.
         *m55*m66*L6r*RSp453e*RSe453p - 8./27.*m55*m66*L6r*pow(
         RSp453e,2) + 16./27.*m55*m66*L6r*RS145pe*RSp453e + 16./27.*m55
         *m66*L6r*RS146pe*RSp463e + 16./27.*m55*m66*L6r*RSe461p*RSp463e
          + 16./27.*m55*m66*L6r*RSe451p*RSp453e + 16./27.*m55*m66*L6r*
         RSp461e*RS346pe + 16./27.*m55*m66*L6r*RSp461e*RSe463p + 16./27.
         *m55*m66*L6r*RSp461e*RSp463e - 16./27.*m55*m66*L6r*RSp461e*
         RS146pe - 16./27.*m55*m66*L6r*RSp461e*RSe461p - 8./27.*m55*m66
         *L6r*pow(RSp461e,2) + 16./27.*m55*m66*L6r*RSp451e*RS345pe + 16.
         /27.*m55*m66*L6r*RSp451e*RSe453p + 16./27.*m55*m66*L6r*RSp451e
         *RSp453e - 16./27.*m55*m66*L6r*RSp451e*RS145pe - 16./27.*m55*
         m66*L6r*RSp451e*RSe451p );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 8./27.*m55*m66*L6r*pow(
         RSp451e,2) - 16./27.*pow(m55,2)*L8r*RSp463e*RS346pe - 16./27.*
         pow(m55,2)*L8r*RSp463e*RSe463p - 8./27.*pow(m55,2)*L8r*pow(
         RSp463e,2) + 16./27.*pow(m55,2)*L8r*RS146pe*RSp463e + 16./27.*
         pow(m55,2)*L8r*RSe461p*RSp463e + 16./27.*pow(m55,2)*L8r*
         RSp461e*RS346pe + 16./27.*pow(m55,2)*L8r*RSp461e*RSe463p + 16./
         27.*pow(m55,2)*L8r*RSp461e*RSp463e - 16./27.*pow(m55,2)*L8r*
         RSp461e*RS146pe - 16./27.*pow(m55,2)*L8r*RSp461e*RSe461p - 8./
         27.*pow(m55,2)*L8r*pow(RSp461e,2) - 16./27.*pow(m55,2)*L7r*
         RSp463e*RS346pe - 16./27.*pow(m55,2)*L7r*RSp463e*RSe463p - 8./
         27.*pow(m55,2)*L7r*pow(RSp463e,2) + 16./27.*pow(m55,2)*L7r*
         RS146pe*RSp463e + 16./27.*pow(m55,2)*L7r*RSe461p*RSp463e + 16./
         27.*pow(m55,2)*L7r*RSp461e*RS346pe + 16./27.*pow(m55,2)*L7r*
         RSp461e*RSe463p + 16./27.*pow(m55,2)*L7r*RSp461e*RSp463e - 16./
         27.*pow(m55,2)*L7r*RSp461e*RS146pe - 16./27.*pow(m55,2)*L7r*
         RSp461e*RSe461p );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 8./27.*pow(m55,2)*L7r*pow(
         RSp461e,2) - 16./27.*pow(m55,2)*L6r*RSp463e*RS346pe - 16./27.*
         pow(m55,2)*L6r*RSp463e*RSe463p - 8./27.*pow(m55,2)*L6r*pow(
         RSp463e,2) + 16./27.*pow(m55,2)*L6r*RS146pe*RSp463e + 16./27.*
         pow(m55,2)*L6r*RSe461p*RSp463e + 16./27.*pow(m55,2)*L6r*
         RSp461e*RS346pe + 16./27.*pow(m55,2)*L6r*RSp461e*RSe463p + 16./
         27.*pow(m55,2)*L6r*RSp461e*RSp463e - 16./27.*pow(m55,2)*L6r*
         RSp461e*RS146pe - 16./27.*pow(m55,2)*L6r*RSp461e*RSe461p - 8./
         27.*pow(m55,2)*L6r*pow(RSp461e,2) + 8./27.*m44*mp2*L5r*RSp563e
         *RS356pe + 8./27.*m44*mp2*L5r*RSp563e*RSe563p + 8./27.*m44*mp2
         *L5r*pow(RSp563e,2) - 8./27.*m44*mp2*L5r*RS156pe*RSp563e - 8./
         27.*m44*mp2*L5r*RSe561p*RSp563e - 8./27.*m44*mp2*L5r*RSp561e*
         RS356pe - 8./27.*m44*mp2*L5r*RSp561e*RSe563p - 16./27.*m44*mp2
         *L5r*RSp561e*RSp563e + 8./27.*m44*mp2*L5r*RSp561e*RS156pe + 8./
         27.*m44*mp2*L5r*RSp561e*RSe561p + 8./27.*m44*mp2*L5r*pow(
         RSp561e,2) );

      M13p6x23LV +=  + AbV(mp2,L) * ( 16./9.*m44*mp2*L4r*RSp45613e - 8./
         9.*m44*mp2*L4r*RSp45633e + 8./27.*m44*mp2*L4r*RSp563e*RS356pe
          + 8./27.*m44*mp2*L4r*RSp563e*RSe563p + 8./27.*m44*mp2*L4r*
         pow(RSp563e,2) + 8./27.*m44*mp2*L4r*RSp463e*RS346pe + 8./27.*
         m44*mp2*L4r*RSp463e*RSe463p + 8./27.*m44*mp2*L4r*pow(
         RSp463e,2) + 8./27.*m44*mp2*L4r*RSp453e*RS345pe + 8./27.*m44*
         mp2*L4r*RSp453e*RSe453p + 8./27.*m44*mp2*L4r*pow(RSp453e,2) - 
         8./9.*m44*mp2*L4r*RSp45611e - 8./27.*m44*mp2*L4r*RS145pe*
         RSp453e - 8./27.*m44*mp2*L4r*RS146pe*RSp463e - 8./27.*m44*mp2*
         L4r*RS156pe*RSp563e - 8./27.*m44*mp2*L4r*RSe561p*RSp563e - 8./
         27.*m44*mp2*L4r*RSe461p*RSp463e - 8./27.*m44*mp2*L4r*RSe451p*
         RSp453e - 8./27.*m44*mp2*L4r*RSp561e*RS356pe - 8./27.*m44*mp2*
         L4r*RSp561e*RSe563p - 16./27.*m44*mp2*L4r*RSp561e*RSp563e + 8./
         27.*m44*mp2*L4r*RSp561e*RS156pe + 8./27.*m44*mp2*L4r*RSp561e*
         RSe561p + 8./27.*m44*mp2*L4r*pow(RSp561e,2) - 8./27.*m44*mp2*
         L4r*RSp461e*RS346pe );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 8./27.*m44*mp2*L4r*RSp461e*
         RSe463p - 16./27.*m44*mp2*L4r*RSp461e*RSp463e + 8./27.*m44*mp2
         *L4r*RSp461e*RS146pe + 8./27.*m44*mp2*L4r*RSp461e*RSe461p + 8./
         27.*m44*mp2*L4r*pow(RSp461e,2) - 8./27.*m44*mp2*L4r*RSp451e*
         RS345pe - 8./27.*m44*mp2*L4r*RSp451e*RSe453p - 16./27.*m44*mp2
         *L4r*RSp451e*RSp453e + 8./27.*m44*mp2*L4r*RSp451e*RS145pe + 8./
         27.*m44*mp2*L4r*RSp451e*RSe451p + 8./27.*m44*mp2*L4r*pow(
         RSp451e,2) - 16./27.*m44*m66*L7r*RSp563e*RS345pe - 16./27.*m44
         *m66*L7r*RSp563e*RSe453p - 16./27.*m44*m66*L7r*RSp453e*RS356pe
          - 16./27.*m44*m66*L7r*RSp453e*RSe563p - 16./27.*m44*m66*L7r*
         RSp453e*RSp563e + 16./27.*m44*m66*L7r*RS145pe*RSp563e + 16./27.
         *m44*m66*L7r*RS156pe*RSp453e + 16./27.*m44*m66*L7r*RSe561p*
         RSp453e + 16./27.*m44*m66*L7r*RSe451p*RSp563e + 16./27.*m44*
         m66*L7r*RSp561e*RS345pe + 16./27.*m44*m66*L7r*RSp561e*RSe453p
          + 16./27.*m44*m66*L7r*RSp561e*RSp453e - 16./27.*m44*m66*L7r*
         RSp561e*RS145pe );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 16./27.*m44*m66*L7r*RSp561e*
         RSe451p + 16./27.*m44*m66*L7r*RSp451e*RS356pe + 16./27.*m44*
         m66*L7r*RSp451e*RSe563p + 16./27.*m44*m66*L7r*RSp451e*RSp563e
          - 16./27.*m44*m66*L7r*RSp451e*RS156pe - 16./27.*m44*m66*L7r*
         RSp451e*RSe561p - 16./27.*m44*m66*L7r*RSp451e*RSp561e - 16./27.
         *m44*m66*L6r*RSp563e*RS356pe - 16./27.*m44*m66*L6r*RSp563e*
         RSe563p - 8./27.*m44*m66*L6r*pow(RSp563e,2) - 16./27.*m44*m66*
         L6r*RSp453e*RS345pe - 16./27.*m44*m66*L6r*RSp453e*RSe453p - 8./
         27.*m44*m66*L6r*pow(RSp453e,2) + 16./27.*m44*m66*L6r*RS145pe*
         RSp453e + 16./27.*m44*m66*L6r*RS156pe*RSp563e + 16./27.*m44*
         m66*L6r*RSe561p*RSp563e + 16./27.*m44*m66*L6r*RSe451p*RSp453e
          + 16./27.*m44*m66*L6r*RSp561e*RS356pe + 16./27.*m44*m66*L6r*
         RSp561e*RSe563p + 16./27.*m44*m66*L6r*RSp561e*RSp563e - 16./27.
         *m44*m66*L6r*RSp561e*RS156pe - 16./27.*m44*m66*L6r*RSp561e*
         RSe561p - 8./27.*m44*m66*L6r*pow(RSp561e,2) + 16./27.*m44*m66*
         L6r*RSp451e*RS345pe );

      M13p6x23LV +=  + AbV(mp2,L) * ( 16./27.*m44*m66*L6r*RSp451e*
         RSe453p + 16./27.*m44*m66*L6r*RSp451e*RSp453e - 16./27.*m44*
         m66*L6r*RSp451e*RS145pe - 16./27.*m44*m66*L6r*RSp451e*RSe451p
          - 8./27.*m44*m66*L6r*pow(RSp451e,2) - 16./27.*m44*m55*L7r*
         RSp563e*RS346pe - 16./27.*m44*m55*L7r*RSp563e*RSe463p - 16./27.
         *m44*m55*L7r*RSp463e*RS356pe - 16./27.*m44*m55*L7r*RSp463e*
         RSe563p - 16./27.*m44*m55*L7r*RSp463e*RSp563e + 16./27.*m44*
         m55*L7r*RS146pe*RSp563e + 16./27.*m44*m55*L7r*RS156pe*RSp463e
          + 16./27.*m44*m55*L7r*RSe561p*RSp463e + 16./27.*m44*m55*L7r*
         RSe461p*RSp563e + 16./27.*m44*m55*L7r*RSp561e*RS346pe + 16./27.
         *m44*m55*L7r*RSp561e*RSe463p + 16./27.*m44*m55*L7r*RSp561e*
         RSp463e - 16./27.*m44*m55*L7r*RSp561e*RS146pe - 16./27.*m44*
         m55*L7r*RSp561e*RSe461p + 16./27.*m44*m55*L7r*RSp461e*RS356pe
          + 16./27.*m44*m55*L7r*RSp461e*RSe563p + 16./27.*m44*m55*L7r*
         RSp461e*RSp563e - 16./27.*m44*m55*L7r*RSp461e*RS156pe - 16./27.
         *m44*m55*L7r*RSp461e*RSe561p );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 16./27.*m44*m55*L7r*RSp461e*
         RSp561e - 16./27.*m44*m55*L6r*RSp563e*RS356pe - 16./27.*m44*
         m55*L6r*RSp563e*RSe563p - 8./27.*m44*m55*L6r*pow(RSp563e,2) - 
         16./27.*m44*m55*L6r*RSp463e*RS346pe - 16./27.*m44*m55*L6r*
         RSp463e*RSe463p - 8./27.*m44*m55*L6r*pow(RSp463e,2) + 16./27.*
         m44*m55*L6r*RS146pe*RSp463e + 16./27.*m44*m55*L6r*RS156pe*
         RSp563e + 16./27.*m44*m55*L6r*RSe561p*RSp563e + 16./27.*m44*
         m55*L6r*RSe461p*RSp463e + 16./27.*m44*m55*L6r*RSp561e*RS356pe
          + 16./27.*m44*m55*L6r*RSp561e*RSe563p + 16./27.*m44*m55*L6r*
         RSp561e*RSp563e - 16./27.*m44*m55*L6r*RSp561e*RS156pe - 16./27.
         *m44*m55*L6r*RSp561e*RSe561p - 8./27.*m44*m55*L6r*pow(
         RSp561e,2) + 16./27.*m44*m55*L6r*RSp461e*RS346pe + 16./27.*m44
         *m55*L6r*RSp461e*RSe463p + 16./27.*m44*m55*L6r*RSp461e*RSp463e
          - 16./27.*m44*m55*L6r*RSp461e*RS146pe - 16./27.*m44*m55*L6r*
         RSp461e*RSe461p - 8./27.*m44*m55*L6r*pow(RSp461e,2) - 16./27.*
         pow(m44,2)*L8r*RSp563e*RS356pe );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 16./27.*pow(m44,2)*L8r*RSp563e
         *RSe563p - 8./27.*pow(m44,2)*L8r*pow(RSp563e,2) + 16./27.*pow(
         m44,2)*L8r*RS156pe*RSp563e + 16./27.*pow(m44,2)*L8r*RSe561p*
         RSp563e + 16./27.*pow(m44,2)*L8r*RSp561e*RS356pe + 16./27.*
         pow(m44,2)*L8r*RSp561e*RSe563p + 16./27.*pow(m44,2)*L8r*
         RSp561e*RSp563e - 16./27.*pow(m44,2)*L8r*RSp561e*RS156pe - 16./
         27.*pow(m44,2)*L8r*RSp561e*RSe561p - 8./27.*pow(m44,2)*L8r*
         pow(RSp561e,2) - 16./27.*pow(m44,2)*L7r*RSp563e*RS356pe - 16./
         27.*pow(m44,2)*L7r*RSp563e*RSe563p - 8./27.*pow(m44,2)*L7r*
         pow(RSp563e,2) + 16./27.*pow(m44,2)*L7r*RS156pe*RSp563e + 16./
         27.*pow(m44,2)*L7r*RSe561p*RSp563e + 16./27.*pow(m44,2)*L7r*
         RSp561e*RS356pe + 16./27.*pow(m44,2)*L7r*RSp561e*RSe563p + 16./
         27.*pow(m44,2)*L7r*RSp561e*RSp563e - 16./27.*pow(m44,2)*L7r*
         RSp561e*RS156pe - 16./27.*pow(m44,2)*L7r*RSp561e*RSe561p - 8./
         27.*pow(m44,2)*L7r*pow(RSp561e,2) - 16./27.*pow(m44,2)*L6r*
         RSp563e*RS356pe );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 16./27.*pow(m44,2)*L6r*RSp563e
         *RSe563p - 8./27.*pow(m44,2)*L6r*pow(RSp563e,2) + 16./27.*pow(
         m44,2)*L6r*RS156pe*RSp563e + 16./27.*pow(m44,2)*L6r*RSe561p*
         RSp563e + 16./27.*pow(m44,2)*L6r*RSp561e*RS356pe + 16./27.*
         pow(m44,2)*L6r*RSp561e*RSe563p + 16./27.*pow(m44,2)*L6r*
         RSp561e*RSp563e - 16./27.*pow(m44,2)*L6r*RSp561e*RS156pe - 16./
         27.*pow(m44,2)*L6r*RSp561e*RSe561p - 8./27.*pow(m44,2)*L6r*
         pow(RSp561e,2) + 8./9.*m33*mp2*L5r*RSp45613e + 4./9.*m33*mp2*
         L5r*RSp45611e + 4./3.*m33*mp2*L4r*RSp564e + 4./3.*m33*mp2*L4r*
         RSp465e + 4./3.*m33*mp2*L4r*RSp456e - 16./9.*m33*m66*L7r*
         RSp453e - 32./9.*m33*m66*L7r*RSp451e - 32./9.*m33*m66*L6r*
         RSp45613e + 8./9.*m33*m66*L6r*RSp45633e - 8./3.*m33*m66*L6r*
         RSp456e - 4./9.*m33*m66*L5r*RSp453e*RS345pe - 4./9.*m33*m66*
         L5r*RSp453e*RSe453p - 2./9.*m33*m66*L5r*pow(RSp453e,2) - 4./27.
         *m33*m66*L5r*RS145pe*RSp453e - 4./27.*m33*m66*L5r*RSe451p*
         RSp453e );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 4./27.*m33*m66*L5r*RSp451e*
         RS345pe - 4./27.*m33*m66*L5r*RSp451e*RSe453p - 4./27.*m33*m66*
         L5r*RSp451e*RSp453e - 4./27.*m33*m66*L5r*RSp451e*RS145pe - 4./
         27.*m33*m66*L5r*RSp451e*RSe451p - 2./27.*m33*m66*L5r*pow(
         RSp451e,2) + 20./9.*m33*m66*L4r*RSp45613e + 14./9.*m33*m66*L4r
         *RSp45633e - 4./9.*m33*m66*L4r*RSp563e*RS356pe - 4./9.*m33*m66
         *L4r*RSp563e*RSe563p - 2./9.*m33*m66*L4r*pow(RSp563e,2) - 4./9.
         *m33*m66*L4r*RSp463e*RS346pe - 4./9.*m33*m66*L4r*RSp463e*
         RSe463p - 2./9.*m33*m66*L4r*pow(RSp463e,2) - 4./9.*m33*m66*L4r
         *RSp453e*RS345pe - 4./9.*m33*m66*L4r*RSp453e*RSe453p - 2./9.*
         m33*m66*L4r*pow(RSp453e,2) + 2./9.*m33*m66*L4r*RSp45611e - 4./
         27.*m33*m66*L4r*RS145pe*RSp453e - 4./27.*m33*m66*L4r*RS146pe*
         RSp463e - 4./27.*m33*m66*L4r*RS156pe*RSp563e - 4./27.*m33*m66*
         L4r*RSe561p*RSp563e - 4./27.*m33*m66*L4r*RSe461p*RSp463e - 4./
         27.*m33*m66*L4r*RSe451p*RSp453e - 4./27.*m33*m66*L4r*RSp561e*
         RS356pe );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 4./27.*m33*m66*L4r*RSp561e*
         RSe563p - 4./27.*m33*m66*L4r*RSp561e*RSp563e - 4./27.*m33*m66*
         L4r*RSp561e*RS156pe - 4./27.*m33*m66*L4r*RSp561e*RSe561p - 2./
         27.*m33*m66*L4r*pow(RSp561e,2) - 4./27.*m33*m66*L4r*RSp461e*
         RS346pe - 4./27.*m33*m66*L4r*RSp461e*RSe463p - 4./27.*m33*m66*
         L4r*RSp461e*RSp463e - 4./27.*m33*m66*L4r*RSp461e*RS146pe - 4./
         27.*m33*m66*L4r*RSp461e*RSe461p - 2./27.*m33*m66*L4r*pow(
         RSp461e,2) - 4./27.*m33*m66*L4r*RSp451e*RS345pe - 4./27.*m33*
         m66*L4r*RSp451e*RSe453p - 4./27.*m33*m66*L4r*RSp451e*RSp453e
          - 4./27.*m33*m66*L4r*RSp451e*RS145pe - 4./27.*m33*m66*L4r*
         RSp451e*RSe451p - 2./27.*m33*m66*L4r*pow(RSp451e,2) - 16./9.*
         m33*m55*L7r*RSp463e - 32./9.*m33*m55*L7r*RSp461e - 32./9.*m33*
         m55*L6r*RSp45613e + 8./9.*m33*m55*L6r*RSp45633e - 8./3.*m33*
         m55*L6r*RSp465e - 4./9.*m33*m55*L5r*RSp463e*RS346pe - 4./9.*
         m33*m55*L5r*RSp463e*RSe463p - 2./9.*m33*m55*L5r*pow(RSp463e,2)
          - 4./27.*m33*m55*L5r*RS146pe*RSp463e );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 4./27.*m33*m55*L5r*RSe461p*
         RSp463e - 4./27.*m33*m55*L5r*RSp461e*RS346pe - 4./27.*m33*m55*
         L5r*RSp461e*RSe463p - 4./27.*m33*m55*L5r*RSp461e*RSp463e - 4./
         27.*m33*m55*L5r*RSp461e*RS146pe - 4./27.*m33*m55*L5r*RSp461e*
         RSe461p - 2./27.*m33*m55*L5r*pow(RSp461e,2) + 20./9.*m33*m55*
         L4r*RSp45613e + 14./9.*m33*m55*L4r*RSp45633e - 4./9.*m33*m55*
         L4r*RSp563e*RS356pe - 4./9.*m33*m55*L4r*RSp563e*RSe563p - 2./9.
         *m33*m55*L4r*pow(RSp563e,2) - 4./9.*m33*m55*L4r*RSp463e*
         RS346pe - 4./9.*m33*m55*L4r*RSp463e*RSe463p - 2./9.*m33*m55*
         L4r*pow(RSp463e,2) - 4./9.*m33*m55*L4r*RSp453e*RS345pe - 4./9.
         *m33*m55*L4r*RSp453e*RSe453p - 2./9.*m33*m55*L4r*pow(
         RSp453e,2) + 2./9.*m33*m55*L4r*RSp45611e - 4./27.*m33*m55*L4r*
         RS145pe*RSp453e - 4./27.*m33*m55*L4r*RS146pe*RSp463e - 4./27.*
         m33*m55*L4r*RS156pe*RSp563e - 4./27.*m33*m55*L4r*RSe561p*
         RSp563e - 4./27.*m33*m55*L4r*RSe461p*RSp463e - 4./27.*m33*m55*
         L4r*RSe451p*RSp453e );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 4./27.*m33*m55*L4r*RSp561e*
         RS356pe - 4./27.*m33*m55*L4r*RSp561e*RSe563p - 4./27.*m33*m55*
         L4r*RSp561e*RSp563e - 4./27.*m33*m55*L4r*RSp561e*RS156pe - 4./
         27.*m33*m55*L4r*RSp561e*RSe561p - 2./27.*m33*m55*L4r*pow(
         RSp561e,2) - 4./27.*m33*m55*L4r*RSp461e*RS346pe - 4./27.*m33*
         m55*L4r*RSp461e*RSe463p - 4./27.*m33*m55*L4r*RSp461e*RSp463e
          - 4./27.*m33*m55*L4r*RSp461e*RS146pe - 4./27.*m33*m55*L4r*
         RSp461e*RSe461p - 2./27.*m33*m55*L4r*pow(RSp461e,2) - 4./27.*
         m33*m55*L4r*RSp451e*RS345pe - 4./27.*m33*m55*L4r*RSp451e*
         RSe453p - 4./27.*m33*m55*L4r*RSp451e*RSp453e - 4./27.*m33*m55*
         L4r*RSp451e*RS145pe - 4./27.*m33*m55*L4r*RSp451e*RSe451p - 2./
         27.*m33*m55*L4r*pow(RSp451e,2) - 16./9.*m33*m44*L7r*RSp563e - 
         32./9.*m33*m44*L7r*RSp561e - 32./9.*m33*m44*L6r*RSp45613e + 8./
         9.*m33*m44*L6r*RSp45633e - 8./3.*m33*m44*L6r*RSp564e - 4./9.*
         m33*m44*L5r*RSp563e*RS356pe - 4./9.*m33*m44*L5r*RSp563e*
         RSe563p );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 2./9.*m33*m44*L5r*pow(
         RSp563e,2) - 4./27.*m33*m44*L5r*RS156pe*RSp563e - 4./27.*m33*
         m44*L5r*RSe561p*RSp563e - 4./27.*m33*m44*L5r*RSp561e*RS356pe
          - 4./27.*m33*m44*L5r*RSp561e*RSe563p - 4./27.*m33*m44*L5r*
         RSp561e*RSp563e - 4./27.*m33*m44*L5r*RSp561e*RS156pe - 4./27.*
         m33*m44*L5r*RSp561e*RSe561p - 2./27.*m33*m44*L5r*pow(
         RSp561e,2) + 20./9.*m33*m44*L4r*RSp45613e + 14./9.*m33*m44*L4r
         *RSp45633e - 4./9.*m33*m44*L4r*RSp563e*RS356pe - 4./9.*m33*m44
         *L4r*RSp563e*RSe563p - 2./9.*m33*m44*L4r*pow(RSp563e,2) - 4./9.
         *m33*m44*L4r*RSp463e*RS346pe - 4./9.*m33*m44*L4r*RSp463e*
         RSe463p - 2./9.*m33*m44*L4r*pow(RSp463e,2) - 4./9.*m33*m44*L4r
         *RSp453e*RS345pe - 4./9.*m33*m44*L4r*RSp453e*RSe453p - 2./9.*
         m33*m44*L4r*pow(RSp453e,2) + 2./9.*m33*m44*L4r*RSp45611e - 4./
         27.*m33*m44*L4r*RS145pe*RSp453e - 4./27.*m33*m44*L4r*RS146pe*
         RSp463e - 4./27.*m33*m44*L4r*RS156pe*RSp563e - 4./27.*m33*m44*
         L4r*RSe561p*RSp563e );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 4./27.*m33*m44*L4r*RSe461p*
         RSp463e - 4./27.*m33*m44*L4r*RSe451p*RSp453e - 4./27.*m33*m44*
         L4r*RSp561e*RS356pe - 4./27.*m33*m44*L4r*RSp561e*RSe563p - 4./
         27.*m33*m44*L4r*RSp561e*RSp563e - 4./27.*m33*m44*L4r*RSp561e*
         RS156pe - 4./27.*m33*m44*L4r*RSp561e*RSe561p - 2./27.*m33*m44*
         L4r*pow(RSp561e,2) - 4./27.*m33*m44*L4r*RSp461e*RS346pe - 4./
         27.*m33*m44*L4r*RSp461e*RSe463p - 4./27.*m33*m44*L4r*RSp461e*
         RSp463e - 4./27.*m33*m44*L4r*RSp461e*RS146pe - 4./27.*m33*m44*
         L4r*RSp461e*RSe461p - 2./27.*m33*m44*L4r*pow(RSp461e,2) - 4./
         27.*m33*m44*L4r*RSp451e*RS345pe - 4./27.*m33*m44*L4r*RSp451e*
         RSe453p - 4./27.*m33*m44*L4r*RSp451e*RSp453e - 4./27.*m33*m44*
         L4r*RSp451e*RS145pe - 4./27.*m33*m44*L4r*RSp451e*RSe451p - 2./
         27.*m33*m44*L4r*pow(RSp451e,2) - 32./9.*pow(m33,2)*L8r*
         RSp45613e - 16./9.*pow(m33,2)*L8r*RSp45633e + 10./9.*pow(
         m33,2)*L5r*RSp45613e + 13./9.*pow(m33,2)*L5r*RSp45633e - 1./9.
         *pow(m33,2)*L5r*RSp45611e );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 4./3.*m13*mp2*L3r*RSp45613e - 
         10./3.*m13*mp2*L3r*RSp45633e - 10./3.*m13*mp2*L3r*RSp45611e - 
         4./3.*m13*mp2*L2r*RSp564e - 4./3.*m13*mp2*L2r*RSp465e - 4./3.*
         m13*mp2*L2r*RSp456e - 16./3.*m13*mp2*L1r*RSp564e - 16./3.*m13*
         mp2*L1r*RSp465e - 16./3.*m13*mp2*L1r*RSp456e - 16./3.*m13*mp2*
         L0r*RSp45613e - 4./3.*m13*mp2*L0r*RSp45633e - 4./3.*m13*mp2*
         L0r*RSp45611e + 8./27.*m13*m66*L5r*RSp453e*RS345pe + 8./27.*
         m13*m66*L5r*RSp453e*RSe453p + 4./27.*m13*m66*L5r*pow(
         RSp453e,2) - 8./27.*m13*m66*L5r*RS145pe*RSp453e - 8./27.*m13*
         m66*L5r*RSe451p*RSp453e - 8./27.*m13*m66*L5r*RSp451e*RS345pe
          - 8./27.*m13*m66*L5r*RSp451e*RSe453p - 8./27.*m13*m66*L5r*
         RSp451e*RSp453e + 8./27.*m13*m66*L5r*RSp451e*RS145pe + 8./27.*
         m13*m66*L5r*RSp451e*RSe451p + 4./27.*m13*m66*L5r*pow(
         RSp451e,2) + 8./9.*m13*m66*L4r*RSp45613e - 4./9.*m13*m66*L4r*
         RSp45633e + 8./27.*m13*m66*L4r*RSp563e*RS356pe + 8./27.*m13*
         m66*L4r*RSp563e*RSe563p );

      M13p6x23LV +=  + AbV(mp2,L) * ( 4./27.*m13*m66*L4r*pow(RSp563e,2)
          + 8./27.*m13*m66*L4r*RSp463e*RS346pe + 8./27.*m13*m66*L4r*
         RSp463e*RSe463p + 4./27.*m13*m66*L4r*pow(RSp463e,2) + 8./27.*
         m13*m66*L4r*RSp453e*RS345pe + 8./27.*m13*m66*L4r*RSp453e*
         RSe453p + 4./27.*m13*m66*L4r*pow(RSp453e,2) - 4./9.*m13*m66*
         L4r*RSp45611e - 8./27.*m13*m66*L4r*RS145pe*RSp453e - 8./27.*
         m13*m66*L4r*RS146pe*RSp463e - 8./27.*m13*m66*L4r*RS156pe*
         RSp563e - 8./27.*m13*m66*L4r*RSe561p*RSp563e - 8./27.*m13*m66*
         L4r*RSe461p*RSp463e - 8./27.*m13*m66*L4r*RSe451p*RSp453e - 8./
         27.*m13*m66*L4r*RSp561e*RS356pe - 8./27.*m13*m66*L4r*RSp561e*
         RSe563p - 8./27.*m13*m66*L4r*RSp561e*RSp563e + 8./27.*m13*m66*
         L4r*RSp561e*RS156pe + 8./27.*m13*m66*L4r*RSp561e*RSe561p + 4./
         27.*m13*m66*L4r*pow(RSp561e,2) - 8./27.*m13*m66*L4r*RSp461e*
         RS346pe - 8./27.*m13*m66*L4r*RSp461e*RSe463p - 8./27.*m13*m66*
         L4r*RSp461e*RSp463e + 8./27.*m13*m66*L4r*RSp461e*RS146pe + 8./
         27.*m13*m66*L4r*RSp461e*RSe461p );

      M13p6x23LV +=  + AbV(mp2,L) * ( 4./27.*m13*m66*L4r*pow(RSp461e,2)
          - 8./27.*m13*m66*L4r*RSp451e*RS345pe - 8./27.*m13*m66*L4r*
         RSp451e*RSe453p - 8./27.*m13*m66*L4r*RSp451e*RSp453e + 8./27.*
         m13*m66*L4r*RSp451e*RS145pe + 8./27.*m13*m66*L4r*RSp451e*
         RSe451p + 4./27.*m13*m66*L4r*pow(RSp451e,2) + 8./3.*m13*m66*
         L4r*RSp456e + 8./27.*m13*m55*L5r*RSp463e*RS346pe + 8./27.*m13*
         m55*L5r*RSp463e*RSe463p + 4./27.*m13*m55*L5r*pow(RSp463e,2) - 
         8./27.*m13*m55*L5r*RS146pe*RSp463e - 8./27.*m13*m55*L5r*
         RSe461p*RSp463e - 8./27.*m13*m55*L5r*RSp461e*RS346pe - 8./27.*
         m13*m55*L5r*RSp461e*RSe463p - 8./27.*m13*m55*L5r*RSp461e*
         RSp463e + 8./27.*m13*m55*L5r*RSp461e*RS146pe + 8./27.*m13*m55*
         L5r*RSp461e*RSe461p + 4./27.*m13*m55*L5r*pow(RSp461e,2) + 8./9.
         *m13*m55*L4r*RSp45613e - 4./9.*m13*m55*L4r*RSp45633e + 8./27.*
         m13*m55*L4r*RSp563e*RS356pe + 8./27.*m13*m55*L4r*RSp563e*
         RSe563p + 4./27.*m13*m55*L4r*pow(RSp563e,2) + 8./27.*m13*m55*
         L4r*RSp463e*RS346pe );

      M13p6x23LV +=  + AbV(mp2,L) * ( 8./27.*m13*m55*L4r*RSp463e*
         RSe463p + 4./27.*m13*m55*L4r*pow(RSp463e,2) + 8./27.*m13*m55*
         L4r*RSp453e*RS345pe + 8./27.*m13*m55*L4r*RSp453e*RSe453p + 4./
         27.*m13*m55*L4r*pow(RSp453e,2) - 4./9.*m13*m55*L4r*RSp45611e
          - 8./27.*m13*m55*L4r*RS145pe*RSp453e - 8./27.*m13*m55*L4r*
         RS146pe*RSp463e - 8./27.*m13*m55*L4r*RS156pe*RSp563e - 8./27.*
         m13*m55*L4r*RSe561p*RSp563e - 8./27.*m13*m55*L4r*RSe461p*
         RSp463e - 8./27.*m13*m55*L4r*RSe451p*RSp453e - 8./27.*m13*m55*
         L4r*RSp561e*RS356pe - 8./27.*m13*m55*L4r*RSp561e*RSe563p - 8./
         27.*m13*m55*L4r*RSp561e*RSp563e + 8./27.*m13*m55*L4r*RSp561e*
         RS156pe + 8./27.*m13*m55*L4r*RSp561e*RSe561p + 4./27.*m13*m55*
         L4r*pow(RSp561e,2) - 8./27.*m13*m55*L4r*RSp461e*RS346pe - 8./
         27.*m13*m55*L4r*RSp461e*RSe463p - 8./27.*m13*m55*L4r*RSp461e*
         RSp463e + 8./27.*m13*m55*L4r*RSp461e*RS146pe + 8./27.*m13*m55*
         L4r*RSp461e*RSe461p + 4./27.*m13*m55*L4r*pow(RSp461e,2) - 8./
         27.*m13*m55*L4r*RSp451e*RS345pe );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 8./27.*m13*m55*L4r*RSp451e*
         RSe453p - 8./27.*m13*m55*L4r*RSp451e*RSp453e + 8./27.*m13*m55*
         L4r*RSp451e*RS145pe + 8./27.*m13*m55*L4r*RSp451e*RSe451p + 4./
         27.*m13*m55*L4r*pow(RSp451e,2) + 8./3.*m13*m55*L4r*RSp465e + 8.
         /27.*m13*m44*L5r*RSp563e*RS356pe + 8./27.*m13*m44*L5r*RSp563e*
         RSe563p + 4./27.*m13*m44*L5r*pow(RSp563e,2) - 8./27.*m13*m44*
         L5r*RS156pe*RSp563e - 8./27.*m13*m44*L5r*RSe561p*RSp563e - 8./
         27.*m13*m44*L5r*RSp561e*RS356pe - 8./27.*m13*m44*L5r*RSp561e*
         RSe563p - 8./27.*m13*m44*L5r*RSp561e*RSp563e + 8./27.*m13*m44*
         L5r*RSp561e*RS156pe + 8./27.*m13*m44*L5r*RSp561e*RSe561p + 4./
         27.*m13*m44*L5r*pow(RSp561e,2) + 8./9.*m13*m44*L4r*RSp45613e
          - 4./9.*m13*m44*L4r*RSp45633e + 8./27.*m13*m44*L4r*RSp563e*
         RS356pe + 8./27.*m13*m44*L4r*RSp563e*RSe563p + 4./27.*m13*m44*
         L4r*pow(RSp563e,2) + 8./27.*m13*m44*L4r*RSp463e*RS346pe + 8./
         27.*m13*m44*L4r*RSp463e*RSe463p + 4./27.*m13*m44*L4r*pow(
         RSp463e,2) );

      M13p6x23LV +=  + AbV(mp2,L) * ( 8./27.*m13*m44*L4r*RSp453e*
         RS345pe + 8./27.*m13*m44*L4r*RSp453e*RSe453p + 4./27.*m13*m44*
         L4r*pow(RSp453e,2) - 4./9.*m13*m44*L4r*RSp45611e - 8./27.*m13*
         m44*L4r*RS145pe*RSp453e - 8./27.*m13*m44*L4r*RS146pe*RSp463e
          - 8./27.*m13*m44*L4r*RS156pe*RSp563e - 8./27.*m13*m44*L4r*
         RSe561p*RSp563e - 8./27.*m13*m44*L4r*RSe461p*RSp463e - 8./27.*
         m13*m44*L4r*RSe451p*RSp453e - 8./27.*m13*m44*L4r*RSp561e*
         RS356pe - 8./27.*m13*m44*L4r*RSp561e*RSe563p - 8./27.*m13*m44*
         L4r*RSp561e*RSp563e + 8./27.*m13*m44*L4r*RSp561e*RS156pe + 8./
         27.*m13*m44*L4r*RSp561e*RSe561p + 4./27.*m13*m44*L4r*pow(
         RSp561e,2) - 8./27.*m13*m44*L4r*RSp461e*RS346pe - 8./27.*m13*
         m44*L4r*RSp461e*RSe463p - 8./27.*m13*m44*L4r*RSp461e*RSp463e
          + 8./27.*m13*m44*L4r*RSp461e*RS146pe + 8./27.*m13*m44*L4r*
         RSp461e*RSe461p + 4./27.*m13*m44*L4r*pow(RSp461e,2) - 8./27.*
         m13*m44*L4r*RSp451e*RS345pe - 8./27.*m13*m44*L4r*RSp451e*
         RSe453p );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 8./27.*m13*m44*L4r*RSp451e*
         RSp453e + 8./27.*m13*m44*L4r*RSp451e*RS145pe + 8./27.*m13*m44*
         L4r*RSp451e*RSe451p + 4./27.*m13*m44*L4r*pow(RSp451e,2) + 8./3.
         *m13*m44*L4r*RSp564e + 4./9.*m13*m33*L5r*RSp45613e + 2./3.*m13
         *m33*L5r*RSp45633e + 2./9.*m13*m33*L5r*RSp45611e + 8./9.*m11*
         mp2*L5r*RSp45613e + 4./9.*m11*mp2*L5r*RSp45633e + 4./3.*m11*
         mp2*L4r*RSp564e + 4./3.*m11*mp2*L4r*RSp465e + 4./3.*m11*mp2*
         L4r*RSp456e - 32./9.*m11*m66*L7r*RSp453e - 16./9.*m11*m66*L7r*
         RSp451e - 32./9.*m11*m66*L6r*RSp45613e + 8./9.*m11*m66*L6r*
         RSp45611e - 8./3.*m11*m66*L6r*RSp456e - 4./27.*m11*m66*L5r*
         RSp453e*RS345pe - 4./27.*m11*m66*L5r*RSp453e*RSe453p - 2./27.*
         m11*m66*L5r*pow(RSp453e,2) - 4./27.*m11*m66*L5r*RS145pe*
         RSp453e - 4./27.*m11*m66*L5r*RSe451p*RSp453e - 4./27.*m11*m66*
         L5r*RSp451e*RS345pe - 4./27.*m11*m66*L5r*RSp451e*RSe453p - 4./
         27.*m11*m66*L5r*RSp451e*RSp453e - 4./9.*m11*m66*L5r*RSp451e*
         RS145pe );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 4./9.*m11*m66*L5r*RSp451e*
         RSe451p - 2./9.*m11*m66*L5r*pow(RSp451e,2) + 20./9.*m11*m66*
         L4r*RSp45613e + 2./9.*m11*m66*L4r*RSp45633e - 4./27.*m11*m66*
         L4r*RSp563e*RS356pe - 4./27.*m11*m66*L4r*RSp563e*RSe563p - 2./
         27.*m11*m66*L4r*pow(RSp563e,2) - 4./27.*m11*m66*L4r*RSp463e*
         RS346pe - 4./27.*m11*m66*L4r*RSp463e*RSe463p - 2./27.*m11*m66*
         L4r*pow(RSp463e,2) - 4./27.*m11*m66*L4r*RSp453e*RS345pe - 4./
         27.*m11*m66*L4r*RSp453e*RSe453p - 2./27.*m11*m66*L4r*pow(
         RSp453e,2) + 14./9.*m11*m66*L4r*RSp45611e - 4./27.*m11*m66*L4r
         *RS145pe*RSp453e - 4./27.*m11*m66*L4r*RS146pe*RSp463e - 4./27.
         *m11*m66*L4r*RS156pe*RSp563e - 4./27.*m11*m66*L4r*RSe561p*
         RSp563e - 4./27.*m11*m66*L4r*RSe461p*RSp463e - 4./27.*m11*m66*
         L4r*RSe451p*RSp453e - 4./27.*m11*m66*L4r*RSp561e*RS356pe - 4./
         27.*m11*m66*L4r*RSp561e*RSe563p - 4./27.*m11*m66*L4r*RSp561e*
         RSp563e - 4./9.*m11*m66*L4r*RSp561e*RS156pe - 4./9.*m11*m66*
         L4r*RSp561e*RSe561p );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 2./9.*m11*m66*L4r*pow(
         RSp561e,2) - 4./27.*m11*m66*L4r*RSp461e*RS346pe - 4./27.*m11*
         m66*L4r*RSp461e*RSe463p - 4./27.*m11*m66*L4r*RSp461e*RSp463e
          - 4./9.*m11*m66*L4r*RSp461e*RS146pe - 4./9.*m11*m66*L4r*
         RSp461e*RSe461p - 2./9.*m11*m66*L4r*pow(RSp461e,2) - 4./27.*
         m11*m66*L4r*RSp451e*RS345pe - 4./27.*m11*m66*L4r*RSp451e*
         RSe453p - 4./27.*m11*m66*L4r*RSp451e*RSp453e - 4./9.*m11*m66*
         L4r*RSp451e*RS145pe - 4./9.*m11*m66*L4r*RSp451e*RSe451p - 2./9.
         *m11*m66*L4r*pow(RSp451e,2) - 32./9.*m11*m55*L7r*RSp463e - 16./
         9.*m11*m55*L7r*RSp461e - 32./9.*m11*m55*L6r*RSp45613e + 8./9.*
         m11*m55*L6r*RSp45611e - 8./3.*m11*m55*L6r*RSp465e - 4./27.*m11
         *m55*L5r*RSp463e*RS346pe - 4./27.*m11*m55*L5r*RSp463e*RSe463p
          - 2./27.*m11*m55*L5r*pow(RSp463e,2) - 4./27.*m11*m55*L5r*
         RS146pe*RSp463e - 4./27.*m11*m55*L5r*RSe461p*RSp463e - 4./27.*
         m11*m55*L5r*RSp461e*RS346pe - 4./27.*m11*m55*L5r*RSp461e*
         RSe463p );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 4./27.*m11*m55*L5r*RSp461e*
         RSp463e - 4./9.*m11*m55*L5r*RSp461e*RS146pe - 4./9.*m11*m55*
         L5r*RSp461e*RSe461p - 2./9.*m11*m55*L5r*pow(RSp461e,2) + 20./9.
         *m11*m55*L4r*RSp45613e + 2./9.*m11*m55*L4r*RSp45633e - 4./27.*
         m11*m55*L4r*RSp563e*RS356pe - 4./27.*m11*m55*L4r*RSp563e*
         RSe563p - 2./27.*m11*m55*L4r*pow(RSp563e,2) - 4./27.*m11*m55*
         L4r*RSp463e*RS346pe - 4./27.*m11*m55*L4r*RSp463e*RSe463p - 2./
         27.*m11*m55*L4r*pow(RSp463e,2) - 4./27.*m11*m55*L4r*RSp453e*
         RS345pe - 4./27.*m11*m55*L4r*RSp453e*RSe453p - 2./27.*m11*m55*
         L4r*pow(RSp453e,2) + 14./9.*m11*m55*L4r*RSp45611e - 4./27.*m11
         *m55*L4r*RS145pe*RSp453e - 4./27.*m11*m55*L4r*RS146pe*RSp463e
          - 4./27.*m11*m55*L4r*RS156pe*RSp563e - 4./27.*m11*m55*L4r*
         RSe561p*RSp563e - 4./27.*m11*m55*L4r*RSe461p*RSp463e - 4./27.*
         m11*m55*L4r*RSe451p*RSp453e - 4./27.*m11*m55*L4r*RSp561e*
         RS356pe - 4./27.*m11*m55*L4r*RSp561e*RSe563p - 4./27.*m11*m55*
         L4r*RSp561e*RSp563e );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 4./9.*m11*m55*L4r*RSp561e*
         RS156pe - 4./9.*m11*m55*L4r*RSp561e*RSe561p - 2./9.*m11*m55*
         L4r*pow(RSp561e,2) - 4./27.*m11*m55*L4r*RSp461e*RS346pe - 4./
         27.*m11*m55*L4r*RSp461e*RSe463p - 4./27.*m11*m55*L4r*RSp461e*
         RSp463e - 4./9.*m11*m55*L4r*RSp461e*RS146pe - 4./9.*m11*m55*
         L4r*RSp461e*RSe461p - 2./9.*m11*m55*L4r*pow(RSp461e,2) - 4./27.
         *m11*m55*L4r*RSp451e*RS345pe - 4./27.*m11*m55*L4r*RSp451e*
         RSe453p - 4./27.*m11*m55*L4r*RSp451e*RSp453e - 4./9.*m11*m55*
         L4r*RSp451e*RS145pe - 4./9.*m11*m55*L4r*RSp451e*RSe451p - 2./9.
         *m11*m55*L4r*pow(RSp451e,2) - 32./9.*m11*m44*L7r*RSp563e - 16./
         9.*m11*m44*L7r*RSp561e - 32./9.*m11*m44*L6r*RSp45613e + 8./9.*
         m11*m44*L6r*RSp45611e - 8./3.*m11*m44*L6r*RSp564e - 4./27.*m11
         *m44*L5r*RSp563e*RS356pe - 4./27.*m11*m44*L5r*RSp563e*RSe563p
          - 2./27.*m11*m44*L5r*pow(RSp563e,2) - 4./27.*m11*m44*L5r*
         RS156pe*RSp563e - 4./27.*m11*m44*L5r*RSe561p*RSp563e - 4./27.*
         m11*m44*L5r*RSp561e*RS356pe );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 4./27.*m11*m44*L5r*RSp561e*
         RSe563p - 4./27.*m11*m44*L5r*RSp561e*RSp563e - 4./9.*m11*m44*
         L5r*RSp561e*RS156pe - 4./9.*m11*m44*L5r*RSp561e*RSe561p - 2./9.
         *m11*m44*L5r*pow(RSp561e,2) + 20./9.*m11*m44*L4r*RSp45613e + 2.
         /9.*m11*m44*L4r*RSp45633e - 4./27.*m11*m44*L4r*RSp563e*RS356pe
          - 4./27.*m11*m44*L4r*RSp563e*RSe563p - 2./27.*m11*m44*L4r*
         pow(RSp563e,2) - 4./27.*m11*m44*L4r*RSp463e*RS346pe - 4./27.*
         m11*m44*L4r*RSp463e*RSe463p - 2./27.*m11*m44*L4r*pow(
         RSp463e,2) - 4./27.*m11*m44*L4r*RSp453e*RS345pe - 4./27.*m11*
         m44*L4r*RSp453e*RSe453p - 2./27.*m11*m44*L4r*pow(RSp453e,2) + 
         14./9.*m11*m44*L4r*RSp45611e - 4./27.*m11*m44*L4r*RS145pe*
         RSp453e - 4./27.*m11*m44*L4r*RS146pe*RSp463e - 4./27.*m11*m44*
         L4r*RS156pe*RSp563e - 4./27.*m11*m44*L4r*RSe561p*RSp563e - 4./
         27.*m11*m44*L4r*RSe461p*RSp463e - 4./27.*m11*m44*L4r*RSe451p*
         RSp453e - 4./27.*m11*m44*L4r*RSp561e*RS356pe - 4./27.*m11*m44*
         L4r*RSp561e*RSe563p );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 4./27.*m11*m44*L4r*RSp561e*
         RSp563e - 4./9.*m11*m44*L4r*RSp561e*RS156pe - 4./9.*m11*m44*
         L4r*RSp561e*RSe561p - 2./9.*m11*m44*L4r*pow(RSp561e,2) - 4./27.
         *m11*m44*L4r*RSp461e*RS346pe - 4./27.*m11*m44*L4r*RSp461e*
         RSe463p - 4./27.*m11*m44*L4r*RSp461e*RSp463e - 4./9.*m11*m44*
         L4r*RSp461e*RS146pe - 4./9.*m11*m44*L4r*RSp461e*RSe461p - 2./9.
         *m11*m44*L4r*pow(RSp461e,2) - 4./27.*m11*m44*L4r*RSp451e*
         RS345pe - 4./27.*m11*m44*L4r*RSp451e*RSe453p - 4./27.*m11*m44*
         L4r*RSp451e*RSp453e - 4./9.*m11*m44*L4r*RSp451e*RS145pe - 4./9.
         *m11*m44*L4r*RSp451e*RSe451p - 2./9.*m11*m44*L4r*pow(
         RSp451e,2) - 16./3.*m11*m33*L8r*RSp45613e - 8./3.*m11*m33*L8r*
         RSp45633e - 8./3.*m11*m33*L8r*RSp45611e + 20./9.*m11*m33*L5r*
         RSp45613e + 4./9.*m11*m33*L5r*RSp45633e + 4./9.*m11*m33*L5r*
         RSp45611e + 4./9.*m11*m13*L5r*RSp45613e + 2./9.*m11*m13*L5r*
         RSp45633e + 2./3.*m11*m13*L5r*RSp45611e - 32./9.*pow(m11,2)*
         L8r*RSp45613e );

      M13p6x23LV +=  + AbV(mp2,L) * (  - 16./9.*pow(m11,2)*L8r*
         RSp45611e + 10./9.*pow(m11,2)*L5r*RSp45613e - 1./9.*pow(m11,2)
         *L5r*RSp45633e + 13./9.*pow(m11,2)*L5r*RSp45611e );

      M13p6x23LV +=  + AbV(me2,L) * ( 8./27.*m66*me2*L5r*RSe453p*
         RS345pe + 8./27.*m66*me2*L5r*pow(RSe453p,2) + 8./27.*m66*me2*
         L5r*RSp453e*RSe453p - 8./27.*m66*me2*L5r*RS145pe*RSe453p - 8./
         27.*m66*me2*L5r*RSe451p*RS345pe - 16./27.*m66*me2*L5r*RSe451p*
         RSe453p - 8./27.*m66*me2*L5r*RSe451p*RSp453e + 8./27.*m66*me2*
         L5r*RSe451p*RS145pe + 8./27.*m66*me2*L5r*pow(RSe451p,2) - 8./
         27.*m66*me2*L5r*RSp451e*RSe453p + 8./27.*m66*me2*L5r*RSp451e*
         RSe451p + 16./9.*m66*me2*L4r*RSe45613p - 8./9.*m66*me2*L4r*
         RSe45633p + 8./27.*m66*me2*L4r*RSe563p*RS356pe + 8./27.*m66*
         me2*L4r*pow(RSe563p,2) + 8./27.*m66*me2*L4r*RSe463p*RS346pe + 
         8./27.*m66*me2*L4r*pow(RSe463p,2) + 8./27.*m66*me2*L4r*RSe453p
         *RS345pe + 8./27.*m66*me2*L4r*pow(RSe453p,2) + 8./27.*m66*me2*
         L4r*RSp563e*RSe563p + 8./27.*m66*me2*L4r*RSp463e*RSe463p + 8./
         27.*m66*me2*L4r*RSp453e*RSe453p - 8./9.*m66*me2*L4r*RSe45611p
          - 8./27.*m66*me2*L4r*RS145pe*RSe453p - 8./27.*m66*me2*L4r*
         RS146pe*RSe463p );

      M13p6x23LV +=  + AbV(me2,L) * (  - 8./27.*m66*me2*L4r*RS156pe*
         RSe563p - 8./27.*m66*me2*L4r*RSe561p*RS356pe - 16./27.*m66*me2
         *L4r*RSe561p*RSe563p - 8./27.*m66*me2*L4r*RSe561p*RSp563e + 8./
         27.*m66*me2*L4r*RSe561p*RS156pe + 8./27.*m66*me2*L4r*pow(
         RSe561p,2) - 8./27.*m66*me2*L4r*RSe461p*RS346pe - 16./27.*m66*
         me2*L4r*RSe461p*RSe463p - 8./27.*m66*me2*L4r*RSe461p*RSp463e
          + 8./27.*m66*me2*L4r*RSe461p*RS146pe + 8./27.*m66*me2*L4r*
         pow(RSe461p,2) - 8./27.*m66*me2*L4r*RSe451p*RS345pe - 16./27.*
         m66*me2*L4r*RSe451p*RSe453p - 8./27.*m66*me2*L4r*RSe451p*
         RSp453e + 8./27.*m66*me2*L4r*RSe451p*RS145pe + 8./27.*m66*me2*
         L4r*pow(RSe451p,2) - 8./27.*m66*me2*L4r*RSp561e*RSe563p + 8./
         27.*m66*me2*L4r*RSp561e*RSe561p - 8./27.*m66*me2*L4r*RSp461e*
         RSe463p + 8./27.*m66*me2*L4r*RSp461e*RSe461p - 8./27.*m66*me2*
         L4r*RSp451e*RSe453p + 8./27.*m66*me2*L4r*RSp451e*RSe451p - 16./
         27.*pow(m66,2)*L8r*RSe453p*RS345pe - 8./27.*pow(m66,2)*L8r*
         pow(RSe453p,2) );

      M13p6x23LV +=  + AbV(me2,L) * ( 16./27.*pow(m66,2)*L8r*RS145pe*
         RSe453p + 16./27.*pow(m66,2)*L8r*RSe451p*RS345pe + 16./27.*
         pow(m66,2)*L8r*RSe451p*RSe453p - 16./27.*pow(m66,2)*L8r*
         RSe451p*RS145pe - 8./27.*pow(m66,2)*L8r*pow(RSe451p,2) - 16./
         27.*pow(m66,2)*L7r*RSe453p*RS345pe - 8./27.*pow(m66,2)*L7r*
         pow(RSe453p,2) + 16./27.*pow(m66,2)*L7r*RS145pe*RSe453p + 16./
         27.*pow(m66,2)*L7r*RSe451p*RS345pe + 16./27.*pow(m66,2)*L7r*
         RSe451p*RSe453p - 16./27.*pow(m66,2)*L7r*RSe451p*RS145pe - 8./
         27.*pow(m66,2)*L7r*pow(RSe451p,2) - 16./27.*pow(m66,2)*L6r*
         RSe453p*RS345pe - 8./27.*pow(m66,2)*L6r*pow(RSe453p,2) + 16./
         27.*pow(m66,2)*L6r*RS145pe*RSe453p + 16./27.*pow(m66,2)*L6r*
         RSe451p*RS345pe + 16./27.*pow(m66,2)*L6r*RSe451p*RSe453p - 16./
         27.*pow(m66,2)*L6r*RSe451p*RS145pe - 8./27.*pow(m66,2)*L6r*
         pow(RSe451p,2) + 8./27.*m55*me2*L5r*RSe463p*RS346pe + 8./27.*
         m55*me2*L5r*pow(RSe463p,2) + 8./27.*m55*me2*L5r*RSp463e*
         RSe463p );

      M13p6x23LV +=  + AbV(me2,L) * (  - 8./27.*m55*me2*L5r*RS146pe*
         RSe463p - 8./27.*m55*me2*L5r*RSe461p*RS346pe - 16./27.*m55*me2
         *L5r*RSe461p*RSe463p - 8./27.*m55*me2*L5r*RSe461p*RSp463e + 8./
         27.*m55*me2*L5r*RSe461p*RS146pe + 8./27.*m55*me2*L5r*pow(
         RSe461p,2) - 8./27.*m55*me2*L5r*RSp461e*RSe463p + 8./27.*m55*
         me2*L5r*RSp461e*RSe461p + 16./9.*m55*me2*L4r*RSe45613p - 8./9.
         *m55*me2*L4r*RSe45633p + 8./27.*m55*me2*L4r*RSe563p*RS356pe + 
         8./27.*m55*me2*L4r*pow(RSe563p,2) + 8./27.*m55*me2*L4r*RSe463p
         *RS346pe + 8./27.*m55*me2*L4r*pow(RSe463p,2) + 8./27.*m55*me2*
         L4r*RSe453p*RS345pe + 8./27.*m55*me2*L4r*pow(RSe453p,2) + 8./
         27.*m55*me2*L4r*RSp563e*RSe563p + 8./27.*m55*me2*L4r*RSp463e*
         RSe463p + 8./27.*m55*me2*L4r*RSp453e*RSe453p - 8./9.*m55*me2*
         L4r*RSe45611p - 8./27.*m55*me2*L4r*RS145pe*RSe453p - 8./27.*
         m55*me2*L4r*RS146pe*RSe463p - 8./27.*m55*me2*L4r*RS156pe*
         RSe563p - 8./27.*m55*me2*L4r*RSe561p*RS356pe - 16./27.*m55*me2
         *L4r*RSe561p*RSe563p );

      M13p6x23LV +=  + AbV(me2,L) * (  - 8./27.*m55*me2*L4r*RSe561p*
         RSp563e + 8./27.*m55*me2*L4r*RSe561p*RS156pe + 8./27.*m55*me2*
         L4r*pow(RSe561p,2) - 8./27.*m55*me2*L4r*RSe461p*RS346pe - 16./
         27.*m55*me2*L4r*RSe461p*RSe463p - 8./27.*m55*me2*L4r*RSe461p*
         RSp463e + 8./27.*m55*me2*L4r*RSe461p*RS146pe + 8./27.*m55*me2*
         L4r*pow(RSe461p,2) - 8./27.*m55*me2*L4r*RSe451p*RS345pe - 16./
         27.*m55*me2*L4r*RSe451p*RSe453p - 8./27.*m55*me2*L4r*RSe451p*
         RSp453e + 8./27.*m55*me2*L4r*RSe451p*RS145pe + 8./27.*m55*me2*
         L4r*pow(RSe451p,2) - 8./27.*m55*me2*L4r*RSp561e*RSe563p + 8./
         27.*m55*me2*L4r*RSp561e*RSe561p - 8./27.*m55*me2*L4r*RSp461e*
         RSe463p + 8./27.*m55*me2*L4r*RSp461e*RSe461p - 8./27.*m55*me2*
         L4r*RSp451e*RSe453p + 8./27.*m55*me2*L4r*RSp451e*RSe451p - 16./
         27.*m55*m66*L7r*RSe463p*RS345pe - 16./27.*m55*m66*L7r*RSe453p*
         RS346pe - 16./27.*m55*m66*L7r*RSe453p*RSe463p + 16./27.*m55*
         m66*L7r*RS145pe*RSe463p + 16./27.*m55*m66*L7r*RS146pe*RSe453p
          + 16./27.*m55*m66*L7r*RSe461p*RS345pe );

      M13p6x23LV +=  + AbV(me2,L) * ( 16./27.*m55*m66*L7r*RSe461p*
         RSe453p - 16./27.*m55*m66*L7r*RSe461p*RS145pe + 16./27.*m55*
         m66*L7r*RSe451p*RS346pe + 16./27.*m55*m66*L7r*RSe451p*RSe463p
          - 16./27.*m55*m66*L7r*RSe451p*RS146pe - 16./27.*m55*m66*L7r*
         RSe451p*RSe461p - 16./27.*m55*m66*L6r*RSe463p*RS346pe - 8./27.
         *m55*m66*L6r*pow(RSe463p,2) - 16./27.*m55*m66*L6r*RSe453p*
         RS345pe - 8./27.*m55*m66*L6r*pow(RSe453p,2) + 16./27.*m55*m66*
         L6r*RS145pe*RSe453p + 16./27.*m55*m66*L6r*RS146pe*RSe463p + 16.
         /27.*m55*m66*L6r*RSe461p*RS346pe + 16./27.*m55*m66*L6r*RSe461p
         *RSe463p - 16./27.*m55*m66*L6r*RSe461p*RS146pe - 8./27.*m55*
         m66*L6r*pow(RSe461p,2) + 16./27.*m55*m66*L6r*RSe451p*RS345pe
          + 16./27.*m55*m66*L6r*RSe451p*RSe453p - 16./27.*m55*m66*L6r*
         RSe451p*RS145pe - 8./27.*m55*m66*L6r*pow(RSe451p,2) - 16./27.*
         pow(m55,2)*L8r*RSe463p*RS346pe - 8./27.*pow(m55,2)*L8r*pow(
         RSe463p,2) + 16./27.*pow(m55,2)*L8r*RS146pe*RSe463p + 16./27.*
         pow(m55,2)*L8r*RSe461p*RS346pe );

      M13p6x23LV +=  + AbV(me2,L) * ( 16./27.*pow(m55,2)*L8r*RSe461p*
         RSe463p - 16./27.*pow(m55,2)*L8r*RSe461p*RS146pe - 8./27.*pow(
         m55,2)*L8r*pow(RSe461p,2) - 16./27.*pow(m55,2)*L7r*RSe463p*
         RS346pe - 8./27.*pow(m55,2)*L7r*pow(RSe463p,2) + 16./27.*pow(
         m55,2)*L7r*RS146pe*RSe463p + 16./27.*pow(m55,2)*L7r*RSe461p*
         RS346pe + 16./27.*pow(m55,2)*L7r*RSe461p*RSe463p - 16./27.*
         pow(m55,2)*L7r*RSe461p*RS146pe - 8./27.*pow(m55,2)*L7r*pow(
         RSe461p,2) - 16./27.*pow(m55,2)*L6r*RSe463p*RS346pe - 8./27.*
         pow(m55,2)*L6r*pow(RSe463p,2) + 16./27.*pow(m55,2)*L6r*RS146pe
         *RSe463p + 16./27.*pow(m55,2)*L6r*RSe461p*RS346pe + 16./27.*
         pow(m55,2)*L6r*RSe461p*RSe463p - 16./27.*pow(m55,2)*L6r*
         RSe461p*RS146pe - 8./27.*pow(m55,2)*L6r*pow(RSe461p,2) + 8./27.
         *m44*me2*L5r*RSe563p*RS356pe + 8./27.*m44*me2*L5r*pow(
         RSe563p,2) + 8./27.*m44*me2*L5r*RSp563e*RSe563p - 8./27.*m44*
         me2*L5r*RS156pe*RSe563p - 8./27.*m44*me2*L5r*RSe561p*RS356pe
          - 16./27.*m44*me2*L5r*RSe561p*RSe563p );

      M13p6x23LV +=  + AbV(me2,L) * (  - 8./27.*m44*me2*L5r*RSe561p*
         RSp563e + 8./27.*m44*me2*L5r*RSe561p*RS156pe + 8./27.*m44*me2*
         L5r*pow(RSe561p,2) - 8./27.*m44*me2*L5r*RSp561e*RSe563p + 8./
         27.*m44*me2*L5r*RSp561e*RSe561p + 16./9.*m44*me2*L4r*RSe45613p
          - 8./9.*m44*me2*L4r*RSe45633p + 8./27.*m44*me2*L4r*RSe563p*
         RS356pe + 8./27.*m44*me2*L4r*pow(RSe563p,2) + 8./27.*m44*me2*
         L4r*RSe463p*RS346pe + 8./27.*m44*me2*L4r*pow(RSe463p,2) + 8./
         27.*m44*me2*L4r*RSe453p*RS345pe + 8./27.*m44*me2*L4r*pow(
         RSe453p,2) + 8./27.*m44*me2*L4r*RSp563e*RSe563p + 8./27.*m44*
         me2*L4r*RSp463e*RSe463p + 8./27.*m44*me2*L4r*RSp453e*RSe453p
          - 8./9.*m44*me2*L4r*RSe45611p - 8./27.*m44*me2*L4r*RS145pe*
         RSe453p - 8./27.*m44*me2*L4r*RS146pe*RSe463p - 8./27.*m44*me2*
         L4r*RS156pe*RSe563p - 8./27.*m44*me2*L4r*RSe561p*RS356pe - 16./
         27.*m44*me2*L4r*RSe561p*RSe563p - 8./27.*m44*me2*L4r*RSe561p*
         RSp563e + 8./27.*m44*me2*L4r*RSe561p*RS156pe + 8./27.*m44*me2*
         L4r*pow(RSe561p,2) );

      M13p6x23LV +=  + AbV(me2,L) * (  - 8./27.*m44*me2*L4r*RSe461p*
         RS346pe - 16./27.*m44*me2*L4r*RSe461p*RSe463p - 8./27.*m44*me2
         *L4r*RSe461p*RSp463e + 8./27.*m44*me2*L4r*RSe461p*RS146pe + 8./
         27.*m44*me2*L4r*pow(RSe461p,2) - 8./27.*m44*me2*L4r*RSe451p*
         RS345pe - 16./27.*m44*me2*L4r*RSe451p*RSe453p - 8./27.*m44*me2
         *L4r*RSe451p*RSp453e + 8./27.*m44*me2*L4r*RSe451p*RS145pe + 8./
         27.*m44*me2*L4r*pow(RSe451p,2) - 8./27.*m44*me2*L4r*RSp561e*
         RSe563p + 8./27.*m44*me2*L4r*RSp561e*RSe561p - 8./27.*m44*me2*
         L4r*RSp461e*RSe463p + 8./27.*m44*me2*L4r*RSp461e*RSe461p - 8./
         27.*m44*me2*L4r*RSp451e*RSe453p + 8./27.*m44*me2*L4r*RSp451e*
         RSe451p - 16./27.*m44*m66*L7r*RSe563p*RS345pe - 16./27.*m44*
         m66*L7r*RSe453p*RS356pe - 16./27.*m44*m66*L7r*RSe453p*RSe563p
          + 16./27.*m44*m66*L7r*RS145pe*RSe563p + 16./27.*m44*m66*L7r*
         RS156pe*RSe453p + 16./27.*m44*m66*L7r*RSe561p*RS345pe + 16./27.
         *m44*m66*L7r*RSe561p*RSe453p - 16./27.*m44*m66*L7r*RSe561p*
         RS145pe );

      M13p6x23LV +=  + AbV(me2,L) * ( 16./27.*m44*m66*L7r*RSe451p*
         RS356pe + 16./27.*m44*m66*L7r*RSe451p*RSe563p - 16./27.*m44*
         m66*L7r*RSe451p*RS156pe - 16./27.*m44*m66*L7r*RSe451p*RSe561p
          - 16./27.*m44*m66*L6r*RSe563p*RS356pe - 8./27.*m44*m66*L6r*
         pow(RSe563p,2) - 16./27.*m44*m66*L6r*RSe453p*RS345pe - 8./27.*
         m44*m66*L6r*pow(RSe453p,2) + 16./27.*m44*m66*L6r*RS145pe*
         RSe453p + 16./27.*m44*m66*L6r*RS156pe*RSe563p + 16./27.*m44*
         m66*L6r*RSe561p*RS356pe + 16./27.*m44*m66*L6r*RSe561p*RSe563p
          - 16./27.*m44*m66*L6r*RSe561p*RS156pe - 8./27.*m44*m66*L6r*
         pow(RSe561p,2) + 16./27.*m44*m66*L6r*RSe451p*RS345pe + 16./27.
         *m44*m66*L6r*RSe451p*RSe453p - 16./27.*m44*m66*L6r*RSe451p*
         RS145pe - 8./27.*m44*m66*L6r*pow(RSe451p,2) - 16./27.*m44*m55*
         L7r*RSe563p*RS346pe - 16./27.*m44*m55*L7r*RSe463p*RS356pe - 16.
         /27.*m44*m55*L7r*RSe463p*RSe563p + 16./27.*m44*m55*L7r*RS146pe
         *RSe563p + 16./27.*m44*m55*L7r*RS156pe*RSe463p + 16./27.*m44*
         m55*L7r*RSe561p*RS346pe );

      M13p6x23LV +=  + AbV(me2,L) * ( 16./27.*m44*m55*L7r*RSe561p*
         RSe463p - 16./27.*m44*m55*L7r*RSe561p*RS146pe + 16./27.*m44*
         m55*L7r*RSe461p*RS356pe + 16./27.*m44*m55*L7r*RSe461p*RSe563p
          - 16./27.*m44*m55*L7r*RSe461p*RS156pe - 16./27.*m44*m55*L7r*
         RSe461p*RSe561p - 16./27.*m44*m55*L6r*RSe563p*RS356pe - 8./27.
         *m44*m55*L6r*pow(RSe563p,2) - 16./27.*m44*m55*L6r*RSe463p*
         RS346pe - 8./27.*m44*m55*L6r*pow(RSe463p,2) + 16./27.*m44*m55*
         L6r*RS146pe*RSe463p + 16./27.*m44*m55*L6r*RS156pe*RSe563p + 16.
         /27.*m44*m55*L6r*RSe561p*RS356pe + 16./27.*m44*m55*L6r*RSe561p
         *RSe563p - 16./27.*m44*m55*L6r*RSe561p*RS156pe - 8./27.*m44*
         m55*L6r*pow(RSe561p,2) + 16./27.*m44*m55*L6r*RSe461p*RS346pe
          + 16./27.*m44*m55*L6r*RSe461p*RSe463p - 16./27.*m44*m55*L6r*
         RSe461p*RS146pe - 8./27.*m44*m55*L6r*pow(RSe461p,2) - 16./27.*
         pow(m44,2)*L8r*RSe563p*RS356pe - 8./27.*pow(m44,2)*L8r*pow(
         RSe563p,2) + 16./27.*pow(m44,2)*L8r*RS156pe*RSe563p + 16./27.*
         pow(m44,2)*L8r*RSe561p*RS356pe );

      M13p6x23LV +=  + AbV(me2,L) * ( 16./27.*pow(m44,2)*L8r*RSe561p*
         RSe563p - 16./27.*pow(m44,2)*L8r*RSe561p*RS156pe - 8./27.*pow(
         m44,2)*L8r*pow(RSe561p,2) - 16./27.*pow(m44,2)*L7r*RSe563p*
         RS356pe - 8./27.*pow(m44,2)*L7r*pow(RSe563p,2) + 16./27.*pow(
         m44,2)*L7r*RS156pe*RSe563p + 16./27.*pow(m44,2)*L7r*RSe561p*
         RS356pe + 16./27.*pow(m44,2)*L7r*RSe561p*RSe563p - 16./27.*
         pow(m44,2)*L7r*RSe561p*RS156pe - 8./27.*pow(m44,2)*L7r*pow(
         RSe561p,2) - 16./27.*pow(m44,2)*L6r*RSe563p*RS356pe - 8./27.*
         pow(m44,2)*L6r*pow(RSe563p,2) + 16./27.*pow(m44,2)*L6r*RS156pe
         *RSe563p + 16./27.*pow(m44,2)*L6r*RSe561p*RS356pe + 16./27.*
         pow(m44,2)*L6r*RSe561p*RSe563p - 16./27.*pow(m44,2)*L6r*
         RSe561p*RS156pe - 8./27.*pow(m44,2)*L6r*pow(RSe561p,2) + 8./9.
         *m33*me2*L5r*RSe45613p + 4./9.*m33*me2*L5r*RSe45611p + 4./3.*
         m33*me2*L4r*RSe564p + 4./3.*m33*me2*L4r*RSe465p + 4./3.*m33*
         me2*L4r*RSe456p - 16./9.*m33*m66*L7r*RSe453p - 32./9.*m33*m66*
         L7r*RSe451p );

      M13p6x23LV +=  + AbV(me2,L) * (  - 32./9.*m33*m66*L6r*RSe45613p
          + 8./9.*m33*m66*L6r*RSe45633p - 8./3.*m33*m66*L6r*RSe456p - 4.
         /9.*m33*m66*L5r*RSe453p*RS345pe - 2./9.*m33*m66*L5r*pow(
         RSe453p,2) - 4./27.*m33*m66*L5r*RS145pe*RSe453p - 4./27.*m33*
         m66*L5r*RSe451p*RS345pe - 4./27.*m33*m66*L5r*RSe451p*RSe453p
          - 4./27.*m33*m66*L5r*RSe451p*RS145pe - 2./27.*m33*m66*L5r*
         pow(RSe451p,2) + 20./9.*m33*m66*L4r*RSe45613p + 14./9.*m33*m66
         *L4r*RSe45633p - 4./9.*m33*m66*L4r*RSe563p*RS356pe - 2./9.*m33
         *m66*L4r*pow(RSe563p,2) - 4./9.*m33*m66*L4r*RSe463p*RS346pe - 
         2./9.*m33*m66*L4r*pow(RSe463p,2) - 4./9.*m33*m66*L4r*RSe453p*
         RS345pe - 2./9.*m33*m66*L4r*pow(RSe453p,2) + 2./9.*m33*m66*L4r
         *RSe45611p - 4./27.*m33*m66*L4r*RS145pe*RSe453p - 4./27.*m33*
         m66*L4r*RS146pe*RSe463p - 4./27.*m33*m66*L4r*RS156pe*RSe563p
          - 4./27.*m33*m66*L4r*RSe561p*RS356pe - 4./27.*m33*m66*L4r*
         RSe561p*RSe563p - 4./27.*m33*m66*L4r*RSe561p*RS156pe - 2./27.*
         m33*m66*L4r*pow(RSe561p,2) );

      M13p6x23LV +=  + AbV(me2,L) * (  - 4./27.*m33*m66*L4r*RSe461p*
         RS346pe - 4./27.*m33*m66*L4r*RSe461p*RSe463p - 4./27.*m33*m66*
         L4r*RSe461p*RS146pe - 2./27.*m33*m66*L4r*pow(RSe461p,2) - 4./
         27.*m33*m66*L4r*RSe451p*RS345pe - 4./27.*m33*m66*L4r*RSe451p*
         RSe453p - 4./27.*m33*m66*L4r*RSe451p*RS145pe - 2./27.*m33*m66*
         L4r*pow(RSe451p,2) - 16./9.*m33*m55*L7r*RSe463p - 32./9.*m33*
         m55*L7r*RSe461p - 32./9.*m33*m55*L6r*RSe45613p + 8./9.*m33*m55
         *L6r*RSe45633p - 8./3.*m33*m55*L6r*RSe465p - 4./9.*m33*m55*L5r
         *RSe463p*RS346pe - 2./9.*m33*m55*L5r*pow(RSe463p,2) - 4./27.*
         m33*m55*L5r*RS146pe*RSe463p - 4./27.*m33*m55*L5r*RSe461p*
         RS346pe - 4./27.*m33*m55*L5r*RSe461p*RSe463p - 4./27.*m33*m55*
         L5r*RSe461p*RS146pe - 2./27.*m33*m55*L5r*pow(RSe461p,2) + 20./
         9.*m33*m55*L4r*RSe45613p + 14./9.*m33*m55*L4r*RSe45633p - 4./9.
         *m33*m55*L4r*RSe563p*RS356pe - 2./9.*m33*m55*L4r*pow(
         RSe563p,2) - 4./9.*m33*m55*L4r*RSe463p*RS346pe - 2./9.*m33*m55
         *L4r*pow(RSe463p,2) );

      M13p6x23LV +=  + AbV(me2,L) * (  - 4./9.*m33*m55*L4r*RSe453p*
         RS345pe - 2./9.*m33*m55*L4r*pow(RSe453p,2) + 2./9.*m33*m55*L4r
         *RSe45611p - 4./27.*m33*m55*L4r*RS145pe*RSe453p - 4./27.*m33*
         m55*L4r*RS146pe*RSe463p - 4./27.*m33*m55*L4r*RS156pe*RSe563p
          - 4./27.*m33*m55*L4r*RSe561p*RS356pe - 4./27.*m33*m55*L4r*
         RSe561p*RSe563p - 4./27.*m33*m55*L4r*RSe561p*RS156pe - 2./27.*
         m33*m55*L4r*pow(RSe561p,2) - 4./27.*m33*m55*L4r*RSe461p*
         RS346pe - 4./27.*m33*m55*L4r*RSe461p*RSe463p - 4./27.*m33*m55*
         L4r*RSe461p*RS146pe - 2./27.*m33*m55*L4r*pow(RSe461p,2) - 4./
         27.*m33*m55*L4r*RSe451p*RS345pe - 4./27.*m33*m55*L4r*RSe451p*
         RSe453p - 4./27.*m33*m55*L4r*RSe451p*RS145pe - 2./27.*m33*m55*
         L4r*pow(RSe451p,2) - 16./9.*m33*m44*L7r*RSe563p - 32./9.*m33*
         m44*L7r*RSe561p - 32./9.*m33*m44*L6r*RSe45613p + 8./9.*m33*m44
         *L6r*RSe45633p - 8./3.*m33*m44*L6r*RSe564p - 4./9.*m33*m44*L5r
         *RSe563p*RS356pe - 2./9.*m33*m44*L5r*pow(RSe563p,2) - 4./27.*
         m33*m44*L5r*RS156pe*RSe563p );

      M13p6x23LV +=  + AbV(me2,L) * (  - 4./27.*m33*m44*L5r*RSe561p*
         RS356pe - 4./27.*m33*m44*L5r*RSe561p*RSe563p - 4./27.*m33*m44*
         L5r*RSe561p*RS156pe - 2./27.*m33*m44*L5r*pow(RSe561p,2) + 20./
         9.*m33*m44*L4r*RSe45613p + 14./9.*m33*m44*L4r*RSe45633p - 4./9.
         *m33*m44*L4r*RSe563p*RS356pe - 2./9.*m33*m44*L4r*pow(
         RSe563p,2) - 4./9.*m33*m44*L4r*RSe463p*RS346pe - 2./9.*m33*m44
         *L4r*pow(RSe463p,2) - 4./9.*m33*m44*L4r*RSe453p*RS345pe - 2./9.
         *m33*m44*L4r*pow(RSe453p,2) + 2./9.*m33*m44*L4r*RSe45611p - 4./
         27.*m33*m44*L4r*RS145pe*RSe453p - 4./27.*m33*m44*L4r*RS146pe*
         RSe463p - 4./27.*m33*m44*L4r*RS156pe*RSe563p - 4./27.*m33*m44*
         L4r*RSe561p*RS356pe - 4./27.*m33*m44*L4r*RSe561p*RSe563p - 4./
         27.*m33*m44*L4r*RSe561p*RS156pe - 2./27.*m33*m44*L4r*pow(
         RSe561p,2) - 4./27.*m33*m44*L4r*RSe461p*RS346pe - 4./27.*m33*
         m44*L4r*RSe461p*RSe463p - 4./27.*m33*m44*L4r*RSe461p*RS146pe
          - 2./27.*m33*m44*L4r*pow(RSe461p,2) - 4./27.*m33*m44*L4r*
         RSe451p*RS345pe );

      M13p6x23LV +=  + AbV(me2,L) * (  - 4./27.*m33*m44*L4r*RSe451p*
         RSe453p - 4./27.*m33*m44*L4r*RSe451p*RS145pe - 2./27.*m33*m44*
         L4r*pow(RSe451p,2) - 32./9.*pow(m33,2)*L8r*RSe45613p - 16./9.*
         pow(m33,2)*L8r*RSe45633p + 10./9.*pow(m33,2)*L5r*RSe45613p + 
         13./9.*pow(m33,2)*L5r*RSe45633p - 1./9.*pow(m33,2)*L5r*
         RSe45611p - 4./3.*m13*me2*L3r*RSe45613p - 10./3.*m13*me2*L3r*
         RSe45633p - 10./3.*m13*me2*L3r*RSe45611p - 4./3.*m13*me2*L2r*
         RSe564p - 4./3.*m13*me2*L2r*RSe465p - 4./3.*m13*me2*L2r*
         RSe456p - 16./3.*m13*me2*L1r*RSe564p - 16./3.*m13*me2*L1r*
         RSe465p - 16./3.*m13*me2*L1r*RSe456p - 16./3.*m13*me2*L0r*
         RSe45613p - 4./3.*m13*me2*L0r*RSe45633p - 4./3.*m13*me2*L0r*
         RSe45611p + 8./27.*m13*m66*L5r*RSe453p*RS345pe + 4./27.*m13*
         m66*L5r*pow(RSe453p,2) - 8./27.*m13*m66*L5r*RS145pe*RSe453p - 
         8./27.*m13*m66*L5r*RSe451p*RS345pe - 8./27.*m13*m66*L5r*
         RSe451p*RSe453p + 8./27.*m13*m66*L5r*RSe451p*RS145pe + 4./27.*
         m13*m66*L5r*pow(RSe451p,2) );

      M13p6x23LV +=  + AbV(me2,L) * ( 8./9.*m13*m66*L4r*RSe45613p - 4./
         9.*m13*m66*L4r*RSe45633p + 8./27.*m13*m66*L4r*RSe563p*RS356pe
          + 4./27.*m13*m66*L4r*pow(RSe563p,2) + 8./27.*m13*m66*L4r*
         RSe463p*RS346pe + 4./27.*m13*m66*L4r*pow(RSe463p,2) + 8./27.*
         m13*m66*L4r*RSe453p*RS345pe + 4./27.*m13*m66*L4r*pow(
         RSe453p,2) - 4./9.*m13*m66*L4r*RSe45611p - 8./27.*m13*m66*L4r*
         RS145pe*RSe453p - 8./27.*m13*m66*L4r*RS146pe*RSe463p - 8./27.*
         m13*m66*L4r*RS156pe*RSe563p - 8./27.*m13*m66*L4r*RSe561p*
         RS356pe - 8./27.*m13*m66*L4r*RSe561p*RSe563p + 8./27.*m13*m66*
         L4r*RSe561p*RS156pe + 4./27.*m13*m66*L4r*pow(RSe561p,2) - 8./
         27.*m13*m66*L4r*RSe461p*RS346pe - 8./27.*m13*m66*L4r*RSe461p*
         RSe463p + 8./27.*m13*m66*L4r*RSe461p*RS146pe + 4./27.*m13*m66*
         L4r*pow(RSe461p,2) - 8./27.*m13*m66*L4r*RSe451p*RS345pe - 8./
         27.*m13*m66*L4r*RSe451p*RSe453p + 8./27.*m13*m66*L4r*RSe451p*
         RS145pe + 4./27.*m13*m66*L4r*pow(RSe451p,2) + 8./3.*m13*m66*
         L4r*RSe456p );

      M13p6x23LV +=  + AbV(me2,L) * ( 8./27.*m13*m55*L5r*RSe463p*
         RS346pe + 4./27.*m13*m55*L5r*pow(RSe463p,2) - 8./27.*m13*m55*
         L5r*RS146pe*RSe463p - 8./27.*m13*m55*L5r*RSe461p*RS346pe - 8./
         27.*m13*m55*L5r*RSe461p*RSe463p + 8./27.*m13*m55*L5r*RSe461p*
         RS146pe + 4./27.*m13*m55*L5r*pow(RSe461p,2) + 8./9.*m13*m55*
         L4r*RSe45613p - 4./9.*m13*m55*L4r*RSe45633p + 8./27.*m13*m55*
         L4r*RSe563p*RS356pe + 4./27.*m13*m55*L4r*pow(RSe563p,2) + 8./
         27.*m13*m55*L4r*RSe463p*RS346pe + 4./27.*m13*m55*L4r*pow(
         RSe463p,2) + 8./27.*m13*m55*L4r*RSe453p*RS345pe + 4./27.*m13*
         m55*L4r*pow(RSe453p,2) - 4./9.*m13*m55*L4r*RSe45611p - 8./27.*
         m13*m55*L4r*RS145pe*RSe453p - 8./27.*m13*m55*L4r*RS146pe*
         RSe463p - 8./27.*m13*m55*L4r*RS156pe*RSe563p - 8./27.*m13*m55*
         L4r*RSe561p*RS356pe - 8./27.*m13*m55*L4r*RSe561p*RSe563p + 8./
         27.*m13*m55*L4r*RSe561p*RS156pe + 4./27.*m13*m55*L4r*pow(
         RSe561p,2) - 8./27.*m13*m55*L4r*RSe461p*RS346pe - 8./27.*m13*
         m55*L4r*RSe461p*RSe463p );

      M13p6x23LV +=  + AbV(me2,L) * ( 8./27.*m13*m55*L4r*RSe461p*
         RS146pe + 4./27.*m13*m55*L4r*pow(RSe461p,2) - 8./27.*m13*m55*
         L4r*RSe451p*RS345pe - 8./27.*m13*m55*L4r*RSe451p*RSe453p + 8./
         27.*m13*m55*L4r*RSe451p*RS145pe + 4./27.*m13*m55*L4r*pow(
         RSe451p,2) + 8./3.*m13*m55*L4r*RSe465p + 8./27.*m13*m44*L5r*
         RSe563p*RS356pe + 4./27.*m13*m44*L5r*pow(RSe563p,2) - 8./27.*
         m13*m44*L5r*RS156pe*RSe563p - 8./27.*m13*m44*L5r*RSe561p*
         RS356pe - 8./27.*m13*m44*L5r*RSe561p*RSe563p + 8./27.*m13*m44*
         L5r*RSe561p*RS156pe + 4./27.*m13*m44*L5r*pow(RSe561p,2) + 8./9.
         *m13*m44*L4r*RSe45613p - 4./9.*m13*m44*L4r*RSe45633p + 8./27.*
         m13*m44*L4r*RSe563p*RS356pe + 4./27.*m13*m44*L4r*pow(
         RSe563p,2) + 8./27.*m13*m44*L4r*RSe463p*RS346pe + 4./27.*m13*
         m44*L4r*pow(RSe463p,2) + 8./27.*m13*m44*L4r*RSe453p*RS345pe + 
         4./27.*m13*m44*L4r*pow(RSe453p,2) - 4./9.*m13*m44*L4r*
         RSe45611p - 8./27.*m13*m44*L4r*RS145pe*RSe453p - 8./27.*m13*
         m44*L4r*RS146pe*RSe463p );

      M13p6x23LV +=  + AbV(me2,L) * (  - 8./27.*m13*m44*L4r*RS156pe*
         RSe563p - 8./27.*m13*m44*L4r*RSe561p*RS356pe - 8./27.*m13*m44*
         L4r*RSe561p*RSe563p + 8./27.*m13*m44*L4r*RSe561p*RS156pe + 4./
         27.*m13*m44*L4r*pow(RSe561p,2) - 8./27.*m13*m44*L4r*RSe461p*
         RS346pe - 8./27.*m13*m44*L4r*RSe461p*RSe463p + 8./27.*m13*m44*
         L4r*RSe461p*RS146pe + 4./27.*m13*m44*L4r*pow(RSe461p,2) - 8./
         27.*m13*m44*L4r*RSe451p*RS345pe - 8./27.*m13*m44*L4r*RSe451p*
         RSe453p + 8./27.*m13*m44*L4r*RSe451p*RS145pe + 4./27.*m13*m44*
         L4r*pow(RSe451p,2) + 8./3.*m13*m44*L4r*RSe564p + 4./9.*m13*m33
         *L5r*RSe45613p + 2./3.*m13*m33*L5r*RSe45633p + 2./9.*m13*m33*
         L5r*RSe45611p + 8./9.*m11*me2*L5r*RSe45613p + 4./9.*m11*me2*
         L5r*RSe45633p + 4./3.*m11*me2*L4r*RSe564p + 4./3.*m11*me2*L4r*
         RSe465p + 4./3.*m11*me2*L4r*RSe456p - 32./9.*m11*m66*L7r*
         RSe453p - 16./9.*m11*m66*L7r*RSe451p - 32./9.*m11*m66*L6r*
         RSe45613p + 8./9.*m11*m66*L6r*RSe45611p - 8./3.*m11*m66*L6r*
         RSe456p );

      M13p6x23LV +=  + AbV(me2,L) * (  - 4./27.*m11*m66*L5r*RSe453p*
         RS345pe - 2./27.*m11*m66*L5r*pow(RSe453p,2) - 4./27.*m11*m66*
         L5r*RS145pe*RSe453p - 4./27.*m11*m66*L5r*RSe451p*RS345pe - 4./
         27.*m11*m66*L5r*RSe451p*RSe453p - 4./9.*m11*m66*L5r*RSe451p*
         RS145pe - 2./9.*m11*m66*L5r*pow(RSe451p,2) + 20./9.*m11*m66*
         L4r*RSe45613p + 2./9.*m11*m66*L4r*RSe45633p - 4./27.*m11*m66*
         L4r*RSe563p*RS356pe - 2./27.*m11*m66*L4r*pow(RSe563p,2) - 4./
         27.*m11*m66*L4r*RSe463p*RS346pe - 2./27.*m11*m66*L4r*pow(
         RSe463p,2) - 4./27.*m11*m66*L4r*RSe453p*RS345pe - 2./27.*m11*
         m66*L4r*pow(RSe453p,2) + 14./9.*m11*m66*L4r*RSe45611p - 4./27.
         *m11*m66*L4r*RS145pe*RSe453p - 4./27.*m11*m66*L4r*RS146pe*
         RSe463p - 4./27.*m11*m66*L4r*RS156pe*RSe563p - 4./27.*m11*m66*
         L4r*RSe561p*RS356pe - 4./27.*m11*m66*L4r*RSe561p*RSe563p - 4./
         9.*m11*m66*L4r*RSe561p*RS156pe - 2./9.*m11*m66*L4r*pow(
         RSe561p,2) - 4./27.*m11*m66*L4r*RSe461p*RS346pe - 4./27.*m11*
         m66*L4r*RSe461p*RSe463p );

      M13p6x23LV +=  + AbV(me2,L) * (  - 4./9.*m11*m66*L4r*RSe461p*
         RS146pe - 2./9.*m11*m66*L4r*pow(RSe461p,2) - 4./27.*m11*m66*
         L4r*RSe451p*RS345pe - 4./27.*m11*m66*L4r*RSe451p*RSe453p - 4./
         9.*m11*m66*L4r*RSe451p*RS145pe - 2./9.*m11*m66*L4r*pow(
         RSe451p,2) - 32./9.*m11*m55*L7r*RSe463p - 16./9.*m11*m55*L7r*
         RSe461p - 32./9.*m11*m55*L6r*RSe45613p + 8./9.*m11*m55*L6r*
         RSe45611p - 8./3.*m11*m55*L6r*RSe465p - 4./27.*m11*m55*L5r*
         RSe463p*RS346pe - 2./27.*m11*m55*L5r*pow(RSe463p,2) - 4./27.*
         m11*m55*L5r*RS146pe*RSe463p - 4./27.*m11*m55*L5r*RSe461p*
         RS346pe - 4./27.*m11*m55*L5r*RSe461p*RSe463p - 4./9.*m11*m55*
         L5r*RSe461p*RS146pe - 2./9.*m11*m55*L5r*pow(RSe461p,2) + 20./9.
         *m11*m55*L4r*RSe45613p + 2./9.*m11*m55*L4r*RSe45633p - 4./27.*
         m11*m55*L4r*RSe563p*RS356pe - 2./27.*m11*m55*L4r*pow(
         RSe563p,2) - 4./27.*m11*m55*L4r*RSe463p*RS346pe - 2./27.*m11*
         m55*L4r*pow(RSe463p,2) - 4./27.*m11*m55*L4r*RSe453p*RS345pe - 
         2./27.*m11*m55*L4r*pow(RSe453p,2) );

      M13p6x23LV +=  + AbV(me2,L) * ( 14./9.*m11*m55*L4r*RSe45611p - 4./
         27.*m11*m55*L4r*RS145pe*RSe453p - 4./27.*m11*m55*L4r*RS146pe*
         RSe463p - 4./27.*m11*m55*L4r*RS156pe*RSe563p - 4./27.*m11*m55*
         L4r*RSe561p*RS356pe - 4./27.*m11*m55*L4r*RSe561p*RSe563p - 4./
         9.*m11*m55*L4r*RSe561p*RS156pe - 2./9.*m11*m55*L4r*pow(
         RSe561p,2) - 4./27.*m11*m55*L4r*RSe461p*RS346pe - 4./27.*m11*
         m55*L4r*RSe461p*RSe463p - 4./9.*m11*m55*L4r*RSe461p*RS146pe - 
         2./9.*m11*m55*L4r*pow(RSe461p,2) - 4./27.*m11*m55*L4r*RSe451p*
         RS345pe - 4./27.*m11*m55*L4r*RSe451p*RSe453p - 4./9.*m11*m55*
         L4r*RSe451p*RS145pe - 2./9.*m11*m55*L4r*pow(RSe451p,2) - 32./9.
         *m11*m44*L7r*RSe563p - 16./9.*m11*m44*L7r*RSe561p - 32./9.*m11
         *m44*L6r*RSe45613p + 8./9.*m11*m44*L6r*RSe45611p - 8./3.*m11*
         m44*L6r*RSe564p - 4./27.*m11*m44*L5r*RSe563p*RS356pe - 2./27.*
         m11*m44*L5r*pow(RSe563p,2) - 4./27.*m11*m44*L5r*RS156pe*
         RSe563p - 4./27.*m11*m44*L5r*RSe561p*RS356pe - 4./27.*m11*m44*
         L5r*RSe561p*RSe563p );

      M13p6x23LV +=  + AbV(me2,L) * (  - 4./9.*m11*m44*L5r*RSe561p*
         RS156pe - 2./9.*m11*m44*L5r*pow(RSe561p,2) + 20./9.*m11*m44*
         L4r*RSe45613p + 2./9.*m11*m44*L4r*RSe45633p - 4./27.*m11*m44*
         L4r*RSe563p*RS356pe - 2./27.*m11*m44*L4r*pow(RSe563p,2) - 4./
         27.*m11*m44*L4r*RSe463p*RS346pe - 2./27.*m11*m44*L4r*pow(
         RSe463p,2) - 4./27.*m11*m44*L4r*RSe453p*RS345pe - 2./27.*m11*
         m44*L4r*pow(RSe453p,2) + 14./9.*m11*m44*L4r*RSe45611p - 4./27.
         *m11*m44*L4r*RS145pe*RSe453p - 4./27.*m11*m44*L4r*RS146pe*
         RSe463p - 4./27.*m11*m44*L4r*RS156pe*RSe563p - 4./27.*m11*m44*
         L4r*RSe561p*RS356pe - 4./27.*m11*m44*L4r*RSe561p*RSe563p - 4./
         9.*m11*m44*L4r*RSe561p*RS156pe - 2./9.*m11*m44*L4r*pow(
         RSe561p,2) - 4./27.*m11*m44*L4r*RSe461p*RS346pe - 4./27.*m11*
         m44*L4r*RSe461p*RSe463p - 4./9.*m11*m44*L4r*RSe461p*RS146pe - 
         2./9.*m11*m44*L4r*pow(RSe461p,2) - 4./27.*m11*m44*L4r*RSe451p*
         RS345pe - 4./27.*m11*m44*L4r*RSe451p*RSe453p - 4./9.*m11*m44*
         L4r*RSe451p*RS145pe );

      M13p6x23LV +=  + AbV(me2,L) * (  - 2./9.*m11*m44*L4r*pow(
         RSe451p,2) - 16./3.*m11*m33*L8r*RSe45613p - 8./3.*m11*m33*L8r*
         RSe45633p - 8./3.*m11*m33*L8r*RSe45611p + 20./9.*m11*m33*L5r*
         RSe45613p + 4./9.*m11*m33*L5r*RSe45633p + 4./9.*m11*m33*L5r*
         RSe45611p + 4./9.*m11*m13*L5r*RSe45613p + 2./9.*m11*m13*L5r*
         RSe45633p + 2./3.*m11*m13*L5r*RSe45611p - 32./9.*pow(m11,2)*
         L8r*RSe45613p - 16./9.*pow(m11,2)*L8r*RSe45611p + 10./9.*pow(
         m11,2)*L5r*RSe45613p - 1./9.*pow(m11,2)*L5r*RSe45633p + 13./9.
         *pow(m11,2)*L5r*RSe45611p );

      M13p6x23LV +=  + BbV(m11,m33,L) * ( 8./27.*m33*pow(m66,2)*L8r*
         RS145pe*RS345pe + 8./27.*m33*pow(m66,2)*L7r*RS145pe*RS345pe + 
         8./27.*m33*pow(m66,2)*L6r*RS145pe*RS345pe + 8./27.*m33*m55*m66
         *L7r*RS145pe*RS346pe + 8./27.*m33*m55*m66*L7r*RS146pe*RS345pe
          + 8./27.*m33*m55*m66*L6r*RS145pe*RS345pe + 8./27.*m33*m55*m66
         *L6r*RS146pe*RS346pe + 8./27.*m33*pow(m55,2)*L8r*RS146pe*
         RS346pe + 8./27.*m33*pow(m55,2)*L7r*RS146pe*RS346pe + 8./27.*
         m33*pow(m55,2)*L6r*RS146pe*RS346pe + 8./27.*m33*m44*m66*L7r*
         RS145pe*RS356pe + 8./27.*m33*m44*m66*L7r*RS156pe*RS345pe + 8./
         27.*m33*m44*m66*L6r*RS145pe*RS345pe + 8./27.*m33*m44*m66*L6r*
         RS156pe*RS356pe + 8./27.*m33*m44*m55*L7r*RS146pe*RS356pe + 8./
         27.*m33*m44*m55*L7r*RS156pe*RS346pe + 8./27.*m33*m44*m55*L6r*
         RS146pe*RS346pe + 8./27.*m33*m44*m55*L6r*RS156pe*RS356pe + 8./
         27.*m33*pow(m44,2)*L8r*RS156pe*RS356pe + 8./27.*m33*pow(m44,2)
         *L7r*RS156pe*RS356pe + 8./27.*m33*pow(m44,2)*L6r*RS156pe*
         RS356pe );

      M13p6x23LV +=  + BbV(m11,m33,L) * (  - 8./9.*pow(m33,2)*m66*L7r*
         RS145pe - 8./9.*pow(m33,2)*m66*L6r*RS14563pe - 8./9.*pow(
         m33,2)*m55*L7r*RS146pe - 8./9.*pow(m33,2)*m55*L6r*RS14563pe - 
         8./9.*pow(m33,2)*m44*L7r*RS156pe - 8./9.*pow(m33,2)*m44*L6r*
         RS14563pe - 8./9.*pow(m33,3)*L8r*RS14563pe + 16./27.*m13*pow(
         m66,2)*L8r*RS145pe*RS345pe + 16./27.*m13*pow(m66,2)*L7r*
         RS145pe*RS345pe + 16./27.*m13*pow(m66,2)*L6r*RS145pe*RS345pe
          + 16./27.*m13*m55*m66*L7r*RS145pe*RS346pe + 16./27.*m13*m55*
         m66*L7r*RS146pe*RS345pe + 16./27.*m13*m55*m66*L6r*RS145pe*
         RS345pe + 16./27.*m13*m55*m66*L6r*RS146pe*RS346pe + 16./27.*
         m13*pow(m55,2)*L8r*RS146pe*RS346pe + 16./27.*m13*pow(m55,2)*
         L7r*RS146pe*RS346pe + 16./27.*m13*pow(m55,2)*L6r*RS146pe*
         RS346pe + 16./27.*m13*m44*m66*L7r*RS145pe*RS356pe + 16./27.*
         m13*m44*m66*L7r*RS156pe*RS345pe + 16./27.*m13*m44*m66*L6r*
         RS145pe*RS345pe + 16./27.*m13*m44*m66*L6r*RS156pe*RS356pe + 16.
         /27.*m13*m44*m55*L7r*RS146pe*RS356pe );

      M13p6x23LV +=  + BbV(m11,m33,L) * ( 16./27.*m13*m44*m55*L7r*
         RS156pe*RS346pe + 16./27.*m13*m44*m55*L6r*RS146pe*RS346pe + 16.
         /27.*m13*m44*m55*L6r*RS156pe*RS356pe + 16./27.*m13*pow(m44,2)*
         L8r*RS156pe*RS356pe + 16./27.*m13*pow(m44,2)*L7r*RS156pe*
         RS356pe + 16./27.*m13*pow(m44,2)*L6r*RS156pe*RS356pe - 16./9.*
         m13*m33*m66*L7r*RS145pe - 16./9.*m13*m33*m66*L6r*RS14563pe - 
         16./9.*m13*m33*m55*L7r*RS146pe - 16./9.*m13*m33*m55*L6r*
         RS14563pe - 16./9.*m13*m33*m44*L7r*RS156pe - 16./9.*m13*m33*
         m44*L6r*RS14563pe - 16./9.*m13*pow(m33,2)*L8r*RS14563pe + 8./9.
         *m11*pow(m66,2)*L8r*RS145pe*RS345pe + 8./9.*m11*pow(m66,2)*L7r
         *RS145pe*RS345pe + 8./9.*m11*pow(m66,2)*L6r*RS145pe*RS345pe + 
         8./9.*m11*m55*m66*L7r*RS145pe*RS346pe + 8./9.*m11*m55*m66*L7r*
         RS146pe*RS345pe + 8./9.*m11*m55*m66*L6r*RS145pe*RS345pe + 8./9.
         *m11*m55*m66*L6r*RS146pe*RS346pe + 8./9.*m11*pow(m55,2)*L8r*
         RS146pe*RS346pe + 8./9.*m11*pow(m55,2)*L7r*RS146pe*RS346pe + 8.
         /9.*m11*pow(m55,2)*L6r*RS146pe*RS346pe );

      M13p6x23LV +=  + BbV(m11,m33,L) * ( 8./9.*m11*m44*m66*L7r*RS145pe
         *RS356pe + 8./9.*m11*m44*m66*L7r*RS156pe*RS345pe + 8./9.*m11*
         m44*m66*L6r*RS145pe*RS345pe + 8./9.*m11*m44*m66*L6r*RS156pe*
         RS356pe + 8./9.*m11*m44*m55*L7r*RS146pe*RS356pe + 8./9.*m11*
         m44*m55*L7r*RS156pe*RS346pe + 8./9.*m11*m44*m55*L6r*RS146pe*
         RS346pe + 8./9.*m11*m44*m55*L6r*RS156pe*RS356pe + 8./9.*m11*
         pow(m44,2)*L8r*RS156pe*RS356pe + 8./9.*m11*pow(m44,2)*L7r*
         RS156pe*RS356pe + 8./9.*m11*pow(m44,2)*L6r*RS156pe*RS356pe - 8.
         /9.*m11*m33*m66*L7r*RS345pe - 8./3.*m11*m33*m66*L7r*RS145pe - 
         8./9.*m11*m33*m66*L6r*RS34561pe - 8./3.*m11*m33*m66*L6r*
         RS14563pe - 4./9.*m11*m33*m66*L5r*RS145pe*RS345pe + 4./3.*m11*
         m33*m66*L4r*RS34561pe + 4./3.*m11*m33*m66*L4r*RS14563pe - 4./9.
         *m11*m33*m66*L4r*RS145pe*RS345pe - 4./9.*m11*m33*m66*L4r*
         RS146pe*RS346pe - 4./9.*m11*m33*m66*L4r*RS156pe*RS356pe - 8./9.
         *m11*m33*m55*L7r*RS346pe - 8./3.*m11*m33*m55*L7r*RS146pe - 8./
         9.*m11*m33*m55*L6r*RS34561pe );

      M13p6x23LV +=  + BbV(m11,m33,L) * (  - 8./3.*m11*m33*m55*L6r*
         RS14563pe - 4./9.*m11*m33*m55*L5r*RS146pe*RS346pe + 4./3.*m11*
         m33*m55*L4r*RS34561pe + 4./3.*m11*m33*m55*L4r*RS14563pe - 4./9.
         *m11*m33*m55*L4r*RS145pe*RS345pe - 4./9.*m11*m33*m55*L4r*
         RS146pe*RS346pe - 4./9.*m11*m33*m55*L4r*RS156pe*RS356pe - 8./9.
         *m11*m33*m44*L7r*RS356pe - 8./3.*m11*m33*m44*L7r*RS156pe - 8./
         9.*m11*m33*m44*L6r*RS34561pe - 8./3.*m11*m33*m44*L6r*RS14563pe
          - 4./9.*m11*m33*m44*L5r*RS156pe*RS356pe + 4./3.*m11*m33*m44*
         L4r*RS34561pe + 4./3.*m11*m33*m44*L4r*RS14563pe - 4./9.*m11*
         m33*m44*L4r*RS145pe*RS345pe - 4./9.*m11*m33*m44*L4r*RS146pe*
         RS346pe - 4./9.*m11*m33*m44*L4r*RS156pe*RS356pe - 8./3.*m11*
         pow(m33,2)*L8r*RS14563pe + 8./3.*m11*pow(m33,2)*L7r + 4./3.*
         m11*pow(m33,2)*L5r*RS14563pe - 16./9.*m11*m13*m66*L7r*RS345pe
          - 16./9.*m11*m13*m66*L6r*RS34561pe - 8./27.*m11*m13*m66*L5r*
         RS145pe*RS345pe + 8./9.*m11*m13*m66*L4r*RS34561pe + 8./9.*m11*
         m13*m66*L4r*RS14563pe );

      M13p6x23LV +=  + BbV(m11,m33,L) * (  - 8./27.*m11*m13*m66*L4r*
         RS145pe*RS345pe - 8./27.*m11*m13*m66*L4r*RS146pe*RS346pe - 8./
         27.*m11*m13*m66*L4r*RS156pe*RS356pe - 16./9.*m11*m13*m55*L7r*
         RS346pe - 16./9.*m11*m13*m55*L6r*RS34561pe - 8./27.*m11*m13*
         m55*L5r*RS146pe*RS346pe + 8./9.*m11*m13*m55*L4r*RS34561pe + 8./
         9.*m11*m13*m55*L4r*RS14563pe - 8./27.*m11*m13*m55*L4r*RS145pe*
         RS345pe - 8./27.*m11*m13*m55*L4r*RS146pe*RS346pe - 8./27.*m11*
         m13*m55*L4r*RS156pe*RS356pe - 16./9.*m11*m13*m44*L7r*RS356pe
          - 16./9.*m11*m13*m44*L6r*RS34561pe - 8./27.*m11*m13*m44*L5r*
         RS156pe*RS356pe + 8./9.*m11*m13*m44*L4r*RS34561pe + 8./9.*m11*
         m13*m44*L4r*RS14563pe - 8./27.*m11*m13*m44*L4r*RS145pe*RS345pe
          - 8./27.*m11*m13*m44*L4r*RS146pe*RS346pe - 8./27.*m11*m13*m44
         *L4r*RS156pe*RS356pe + 16./3.*m11*m13*m33*L7r + 8./9.*m11*m13*
         m33*L5r*RS14563pe - 8./3.*pow(m11,2)*m66*L7r*RS345pe - 8./3.*
         pow(m11,2)*m66*L6r*RS34561pe - 4./27.*pow(m11,2)*m66*L5r*
         RS145pe*RS345pe );

      M13p6x23LV +=  + BbV(m11,m33,L) * ( 4./9.*pow(m11,2)*m66*L4r*
         RS34561pe + 4./9.*pow(m11,2)*m66*L4r*RS14563pe - 4./27.*pow(
         m11,2)*m66*L4r*RS145pe*RS345pe - 4./27.*pow(m11,2)*m66*L4r*
         RS146pe*RS346pe - 4./27.*pow(m11,2)*m66*L4r*RS156pe*RS356pe - 
         8./3.*pow(m11,2)*m55*L7r*RS346pe - 8./3.*pow(m11,2)*m55*L6r*
         RS34561pe - 4./27.*pow(m11,2)*m55*L5r*RS146pe*RS346pe + 4./9.*
         pow(m11,2)*m55*L4r*RS34561pe + 4./9.*pow(m11,2)*m55*L4r*
         RS14563pe - 4./27.*pow(m11,2)*m55*L4r*RS145pe*RS345pe - 4./27.
         *pow(m11,2)*m55*L4r*RS146pe*RS346pe - 4./27.*pow(m11,2)*m55*
         L4r*RS156pe*RS356pe - 8./3.*pow(m11,2)*m44*L7r*RS356pe - 8./3.
         *pow(m11,2)*m44*L6r*RS34561pe - 4./27.*pow(m11,2)*m44*L5r*
         RS156pe*RS356pe + 4./9.*pow(m11,2)*m44*L4r*RS34561pe + 4./9.*
         pow(m11,2)*m44*L4r*RS14563pe - 4./27.*pow(m11,2)*m44*L4r*
         RS145pe*RS345pe - 4./27.*pow(m11,2)*m44*L4r*RS146pe*RS346pe - 
         4./27.*pow(m11,2)*m44*L4r*RS156pe*RS356pe - 8./9.*pow(m11,2)*
         m33*L8r*RS34561pe );

      M13p6x23LV +=  + BbV(m11,m33,L) * ( 8*pow(m11,2)*m33*L7r + 4./3.*
         pow(m11,2)*m33*L5r*RS34561pe + 4./9.*pow(m11,2)*m33*L5r*
         RS14563pe - 16./9.*pow(m11,2)*m13*L8r*RS34561pe + 8./9.*pow(
         m11,2)*m13*L5r*RS34561pe - 8./3.*pow(m11,3)*L8r*RS34561pe + 4./
         9.*pow(m11,3)*L5r*RS34561pe );

      M13p6x23LV +=  + BbV(m11,mp2,L) * ( 8./27.*m33*pow(m66,2)*L8r*
         RS145pe*RSp453e + 8./27.*m33*pow(m66,2)*L8r*RSp451e*RS145pe + 
         8./27.*m33*pow(m66,2)*L7r*RS145pe*RSp453e + 8./27.*m33*pow(
         m66,2)*L7r*RSp451e*RS145pe + 8./27.*m33*pow(m66,2)*L6r*RS145pe
         *RSp453e + 8./27.*m33*pow(m66,2)*L6r*RSp451e*RS145pe + 8./27.*
         m33*m55*m66*L7r*RS145pe*RSp463e + 8./27.*m33*m55*m66*L7r*
         RS146pe*RSp453e + 8./27.*m33*m55*m66*L7r*RSp461e*RS145pe + 8./
         27.*m33*m55*m66*L7r*RSp451e*RS146pe + 8./27.*m33*m55*m66*L6r*
         RS145pe*RSp453e + 8./27.*m33*m55*m66*L6r*RS146pe*RSp463e + 8./
         27.*m33*m55*m66*L6r*RSp461e*RS146pe + 8./27.*m33*m55*m66*L6r*
         RSp451e*RS145pe + 8./27.*m33*pow(m55,2)*L8r*RS146pe*RSp463e + 
         8./27.*m33*pow(m55,2)*L8r*RSp461e*RS146pe + 8./27.*m33*pow(
         m55,2)*L7r*RS146pe*RSp463e + 8./27.*m33*pow(m55,2)*L7r*RSp461e
         *RS146pe + 8./27.*m33*pow(m55,2)*L6r*RS146pe*RSp463e + 8./27.*
         m33*pow(m55,2)*L6r*RSp461e*RS146pe + 8./27.*m33*m44*m66*L7r*
         RS145pe*RSp563e );

      M13p6x23LV +=  + BbV(m11,mp2,L) * ( 8./27.*m33*m44*m66*L7r*
         RS156pe*RSp453e + 8./27.*m33*m44*m66*L7r*RSp561e*RS145pe + 8./
         27.*m33*m44*m66*L7r*RSp451e*RS156pe + 8./27.*m33*m44*m66*L6r*
         RS145pe*RSp453e + 8./27.*m33*m44*m66*L6r*RS156pe*RSp563e + 8./
         27.*m33*m44*m66*L6r*RSp561e*RS156pe + 8./27.*m33*m44*m66*L6r*
         RSp451e*RS145pe + 8./27.*m33*m44*m55*L7r*RS146pe*RSp563e + 8./
         27.*m33*m44*m55*L7r*RS156pe*RSp463e + 8./27.*m33*m44*m55*L7r*
         RSp561e*RS146pe + 8./27.*m33*m44*m55*L7r*RSp461e*RS156pe + 8./
         27.*m33*m44*m55*L6r*RS146pe*RSp463e + 8./27.*m33*m44*m55*L6r*
         RS156pe*RSp563e + 8./27.*m33*m44*m55*L6r*RSp561e*RS156pe + 8./
         27.*m33*m44*m55*L6r*RSp461e*RS146pe + 8./27.*m33*pow(m44,2)*
         L8r*RS156pe*RSp563e + 8./27.*m33*pow(m44,2)*L8r*RSp561e*
         RS156pe + 8./27.*m33*pow(m44,2)*L7r*RS156pe*RSp563e + 8./27.*
         m33*pow(m44,2)*L7r*RSp561e*RS156pe + 8./27.*m33*pow(m44,2)*L6r
         *RS156pe*RSp563e + 8./27.*m33*pow(m44,2)*L6r*RSp561e*RS156pe
          + 16./27.*m13*pow(m66,2)*L8r*RS145pe*RSp453e );

      M13p6x23LV +=  + BbV(m11,mp2,L) * (  - 16./27.*m13*pow(m66,2)*L8r
         *RSp451e*RS145pe + 16./27.*m13*pow(m66,2)*L7r*RS145pe*RSp453e
          - 16./27.*m13*pow(m66,2)*L7r*RSp451e*RS145pe + 16./27.*m13*
         pow(m66,2)*L6r*RS145pe*RSp453e - 16./27.*m13*pow(m66,2)*L6r*
         RSp451e*RS145pe + 16./27.*m13*m55*m66*L7r*RS145pe*RSp463e + 16.
         /27.*m13*m55*m66*L7r*RS146pe*RSp453e - 16./27.*m13*m55*m66*L7r
         *RSp461e*RS145pe - 16./27.*m13*m55*m66*L7r*RSp451e*RS146pe + 
         16./27.*m13*m55*m66*L6r*RS145pe*RSp453e + 16./27.*m13*m55*m66*
         L6r*RS146pe*RSp463e - 16./27.*m13*m55*m66*L6r*RSp461e*RS146pe
          - 16./27.*m13*m55*m66*L6r*RSp451e*RS145pe + 16./27.*m13*pow(
         m55,2)*L8r*RS146pe*RSp463e - 16./27.*m13*pow(m55,2)*L8r*
         RSp461e*RS146pe + 16./27.*m13*pow(m55,2)*L7r*RS146pe*RSp463e
          - 16./27.*m13*pow(m55,2)*L7r*RSp461e*RS146pe + 16./27.*m13*
         pow(m55,2)*L6r*RS146pe*RSp463e - 16./27.*m13*pow(m55,2)*L6r*
         RSp461e*RS146pe + 16./27.*m13*m44*m66*L7r*RS145pe*RSp563e + 16.
         /27.*m13*m44*m66*L7r*RS156pe*RSp453e );

      M13p6x23LV +=  + BbV(m11,mp2,L) * (  - 16./27.*m13*m44*m66*L7r*
         RSp561e*RS145pe - 16./27.*m13*m44*m66*L7r*RSp451e*RS156pe + 16.
         /27.*m13*m44*m66*L6r*RS145pe*RSp453e + 16./27.*m13*m44*m66*L6r
         *RS156pe*RSp563e - 16./27.*m13*m44*m66*L6r*RSp561e*RS156pe - 
         16./27.*m13*m44*m66*L6r*RSp451e*RS145pe + 16./27.*m13*m44*m55*
         L7r*RS146pe*RSp563e + 16./27.*m13*m44*m55*L7r*RS156pe*RSp463e
          - 16./27.*m13*m44*m55*L7r*RSp561e*RS146pe - 16./27.*m13*m44*
         m55*L7r*RSp461e*RS156pe + 16./27.*m13*m44*m55*L6r*RS146pe*
         RSp463e + 16./27.*m13*m44*m55*L6r*RS156pe*RSp563e - 16./27.*
         m13*m44*m55*L6r*RSp561e*RS156pe - 16./27.*m13*m44*m55*L6r*
         RSp461e*RS146pe + 16./27.*m13*pow(m44,2)*L8r*RS156pe*RSp563e
          - 16./27.*m13*pow(m44,2)*L8r*RSp561e*RS156pe + 16./27.*m13*
         pow(m44,2)*L7r*RS156pe*RSp563e - 16./27.*m13*pow(m44,2)*L7r*
         RSp561e*RS156pe + 16./27.*m13*pow(m44,2)*L6r*RS156pe*RSp563e
          - 16./27.*m13*pow(m44,2)*L6r*RSp561e*RS156pe - 8./27.*m11*m66
         *mp2*L5r*RS145pe*RSp453e );

      M13p6x23LV +=  + BbV(m11,mp2,L) * ( 8./27.*m11*m66*mp2*L5r*
         RSp451e*RS145pe + 8./9.*m11*m66*mp2*L4r*RSp45613e - 8./9.*m11*
         m66*mp2*L4r*RSp45611e - 8./27.*m11*m66*mp2*L4r*RS145pe*RSp453e
          - 8./27.*m11*m66*mp2*L4r*RS146pe*RSp463e - 8./27.*m11*m66*mp2
         *L4r*RS156pe*RSp563e + 8./27.*m11*m66*mp2*L4r*RSp561e*RS156pe
          + 8./27.*m11*m66*mp2*L4r*RSp461e*RS146pe + 8./27.*m11*m66*mp2
         *L4r*RSp451e*RS145pe + 8./9.*m11*pow(m66,2)*L8r*RS145pe*
         RSp453e + 8./27.*m11*pow(m66,2)*L8r*RSp451e*RS145pe + 8./9.*
         m11*pow(m66,2)*L7r*RS145pe*RSp453e + 8./27.*m11*pow(m66,2)*L7r
         *RSp451e*RS145pe + 8./9.*m11*pow(m66,2)*L6r*RS145pe*RSp453e + 
         8./27.*m11*pow(m66,2)*L6r*RSp451e*RS145pe - 8./27.*m11*m55*mp2
         *L5r*RS146pe*RSp463e + 8./27.*m11*m55*mp2*L5r*RSp461e*RS146pe
          + 8./9.*m11*m55*mp2*L4r*RSp45613e - 8./9.*m11*m55*mp2*L4r*
         RSp45611e - 8./27.*m11*m55*mp2*L4r*RS145pe*RSp453e - 8./27.*
         m11*m55*mp2*L4r*RS146pe*RSp463e - 8./27.*m11*m55*mp2*L4r*
         RS156pe*RSp563e );

      M13p6x23LV +=  + BbV(m11,mp2,L) * ( 8./27.*m11*m55*mp2*L4r*
         RSp561e*RS156pe + 8./27.*m11*m55*mp2*L4r*RSp461e*RS146pe + 8./
         27.*m11*m55*mp2*L4r*RSp451e*RS145pe + 8./9.*m11*m55*m66*L7r*
         RS145pe*RSp463e + 8./9.*m11*m55*m66*L7r*RS146pe*RSp453e + 8./
         27.*m11*m55*m66*L7r*RSp461e*RS145pe + 8./27.*m11*m55*m66*L7r*
         RSp451e*RS146pe + 8./9.*m11*m55*m66*L6r*RS145pe*RSp453e + 8./9.
         *m11*m55*m66*L6r*RS146pe*RSp463e + 8./27.*m11*m55*m66*L6r*
         RSp461e*RS146pe + 8./27.*m11*m55*m66*L6r*RSp451e*RS145pe + 8./
         9.*m11*pow(m55,2)*L8r*RS146pe*RSp463e + 8./27.*m11*pow(m55,2)*
         L8r*RSp461e*RS146pe + 8./9.*m11*pow(m55,2)*L7r*RS146pe*RSp463e
          + 8./27.*m11*pow(m55,2)*L7r*RSp461e*RS146pe + 8./9.*m11*pow(
         m55,2)*L6r*RS146pe*RSp463e + 8./27.*m11*pow(m55,2)*L6r*RSp461e
         *RS146pe - 8./27.*m11*m44*mp2*L5r*RS156pe*RSp563e + 8./27.*m11
         *m44*mp2*L5r*RSp561e*RS156pe + 8./9.*m11*m44*mp2*L4r*RSp45613e
          - 8./9.*m11*m44*mp2*L4r*RSp45611e - 8./27.*m11*m44*mp2*L4r*
         RS145pe*RSp453e );

      M13p6x23LV +=  + BbV(m11,mp2,L) * (  - 8./27.*m11*m44*mp2*L4r*
         RS146pe*RSp463e - 8./27.*m11*m44*mp2*L4r*RS156pe*RSp563e + 8./
         27.*m11*m44*mp2*L4r*RSp561e*RS156pe + 8./27.*m11*m44*mp2*L4r*
         RSp461e*RS146pe + 8./27.*m11*m44*mp2*L4r*RSp451e*RS145pe + 8./
         9.*m11*m44*m66*L7r*RS145pe*RSp563e + 8./9.*m11*m44*m66*L7r*
         RS156pe*RSp453e + 8./27.*m11*m44*m66*L7r*RSp561e*RS145pe + 8./
         27.*m11*m44*m66*L7r*RSp451e*RS156pe + 8./9.*m11*m44*m66*L6r*
         RS145pe*RSp453e + 8./9.*m11*m44*m66*L6r*RS156pe*RSp563e + 8./
         27.*m11*m44*m66*L6r*RSp561e*RS156pe + 8./27.*m11*m44*m66*L6r*
         RSp451e*RS145pe + 8./9.*m11*m44*m55*L7r*RS146pe*RSp563e + 8./9.
         *m11*m44*m55*L7r*RS156pe*RSp463e + 8./27.*m11*m44*m55*L7r*
         RSp561e*RS146pe + 8./27.*m11*m44*m55*L7r*RSp461e*RS156pe + 8./
         9.*m11*m44*m55*L6r*RS146pe*RSp463e + 8./9.*m11*m44*m55*L6r*
         RS156pe*RSp563e + 8./27.*m11*m44*m55*L6r*RSp561e*RS156pe + 8./
         27.*m11*m44*m55*L6r*RSp461e*RS146pe + 8./9.*m11*pow(m44,2)*L8r
         *RS156pe*RSp563e );

      M13p6x23LV +=  + BbV(m11,mp2,L) * ( 8./27.*m11*pow(m44,2)*L8r*
         RSp561e*RS156pe + 8./9.*m11*pow(m44,2)*L7r*RS156pe*RSp563e + 8.
         /27.*m11*pow(m44,2)*L7r*RSp561e*RS156pe + 8./9.*m11*pow(m44,2)
         *L6r*RS156pe*RSp563e + 8./27.*m11*pow(m44,2)*L6r*RSp561e*
         RS156pe - 8./9.*m11*m33*m66*L7r*RSp453e - 8./9.*m11*m33*m66*
         L7r*RSp451e - 8./9.*m11*m33*m66*L6r*RSp45613e - 8./9.*m11*m33*
         m66*L6r*RSp45611e - 4./27.*m11*m33*m66*L5r*RS145pe*RSp453e - 4.
         /27.*m11*m33*m66*L5r*RSp451e*RS145pe + 4./9.*m11*m33*m66*L4r*
         RSp45613e + 4./9.*m11*m33*m66*L4r*RSp45611e - 4./27.*m11*m33*
         m66*L4r*RS145pe*RSp453e - 4./27.*m11*m33*m66*L4r*RS146pe*
         RSp463e - 4./27.*m11*m33*m66*L4r*RS156pe*RSp563e - 4./27.*m11*
         m33*m66*L4r*RSp561e*RS156pe - 4./27.*m11*m33*m66*L4r*RSp461e*
         RS146pe - 4./27.*m11*m33*m66*L4r*RSp451e*RS145pe - 8./9.*m11*
         m33*m55*L7r*RSp463e - 8./9.*m11*m33*m55*L7r*RSp461e - 8./9.*
         m11*m33*m55*L6r*RSp45613e - 8./9.*m11*m33*m55*L6r*RSp45611e - 
         4./27.*m11*m33*m55*L5r*RS146pe*RSp463e );

      M13p6x23LV +=  + BbV(m11,mp2,L) * (  - 4./27.*m11*m33*m55*L5r*
         RSp461e*RS146pe + 4./9.*m11*m33*m55*L4r*RSp45613e + 4./9.*m11*
         m33*m55*L4r*RSp45611e - 4./27.*m11*m33*m55*L4r*RS145pe*RSp453e
          - 4./27.*m11*m33*m55*L4r*RS146pe*RSp463e - 4./27.*m11*m33*m55
         *L4r*RS156pe*RSp563e - 4./27.*m11*m33*m55*L4r*RSp561e*RS156pe
          - 4./27.*m11*m33*m55*L4r*RSp461e*RS146pe - 4./27.*m11*m33*m55
         *L4r*RSp451e*RS145pe - 8./9.*m11*m33*m44*L7r*RSp563e - 8./9.*
         m11*m33*m44*L7r*RSp561e - 8./9.*m11*m33*m44*L6r*RSp45613e - 8./
         9.*m11*m33*m44*L6r*RSp45611e - 4./27.*m11*m33*m44*L5r*RS156pe*
         RSp563e - 4./27.*m11*m33*m44*L5r*RSp561e*RS156pe + 4./9.*m11*
         m33*m44*L4r*RSp45613e + 4./9.*m11*m33*m44*L4r*RSp45611e - 4./
         27.*m11*m33*m44*L4r*RS145pe*RSp453e - 4./27.*m11*m33*m44*L4r*
         RS146pe*RSp463e - 4./27.*m11*m33*m44*L4r*RS156pe*RSp563e - 4./
         27.*m11*m33*m44*L4r*RSp561e*RS156pe - 4./27.*m11*m33*m44*L4r*
         RSp461e*RS146pe - 4./27.*m11*m33*m44*L4r*RSp451e*RS145pe - 16./
         9.*m11*m13*m66*L7r*RSp453e );

      M13p6x23LV +=  + BbV(m11,mp2,L) * ( 16./9.*m11*m13*m66*L7r*
         RSp451e - 16./9.*m11*m13*m66*L6r*RSp45613e + 16./9.*m11*m13*
         m66*L6r*RSp45611e - 8./27.*m11*m13*m66*L5r*RS145pe*RSp453e + 8.
         /27.*m11*m13*m66*L5r*RSp451e*RS145pe + 8./9.*m11*m13*m66*L4r*
         RSp45613e - 8./9.*m11*m13*m66*L4r*RSp45611e - 8./27.*m11*m13*
         m66*L4r*RS145pe*RSp453e - 8./27.*m11*m13*m66*L4r*RS146pe*
         RSp463e - 8./27.*m11*m13*m66*L4r*RS156pe*RSp563e + 8./27.*m11*
         m13*m66*L4r*RSp561e*RS156pe + 8./27.*m11*m13*m66*L4r*RSp461e*
         RS146pe + 8./27.*m11*m13*m66*L4r*RSp451e*RS145pe - 16./9.*m11*
         m13*m55*L7r*RSp463e + 16./9.*m11*m13*m55*L7r*RSp461e - 16./9.*
         m11*m13*m55*L6r*RSp45613e + 16./9.*m11*m13*m55*L6r*RSp45611e
          - 8./27.*m11*m13*m55*L5r*RS146pe*RSp463e + 8./27.*m11*m13*m55
         *L5r*RSp461e*RS146pe + 8./9.*m11*m13*m55*L4r*RSp45613e - 8./9.
         *m11*m13*m55*L4r*RSp45611e - 8./27.*m11*m13*m55*L4r*RS145pe*
         RSp453e - 8./27.*m11*m13*m55*L4r*RS146pe*RSp463e - 8./27.*m11*
         m13*m55*L4r*RS156pe*RSp563e );

      M13p6x23LV +=  + BbV(m11,mp2,L) * ( 8./27.*m11*m13*m55*L4r*
         RSp561e*RS156pe + 8./27.*m11*m13*m55*L4r*RSp461e*RS146pe + 8./
         27.*m11*m13*m55*L4r*RSp451e*RS145pe - 16./9.*m11*m13*m44*L7r*
         RSp563e + 16./9.*m11*m13*m44*L7r*RSp561e - 16./9.*m11*m13*m44*
         L6r*RSp45613e + 16./9.*m11*m13*m44*L6r*RSp45611e - 8./27.*m11*
         m13*m44*L5r*RS156pe*RSp563e + 8./27.*m11*m13*m44*L5r*RSp561e*
         RS156pe + 8./9.*m11*m13*m44*L4r*RSp45613e - 8./9.*m11*m13*m44*
         L4r*RSp45611e - 8./27.*m11*m13*m44*L4r*RS145pe*RSp453e - 8./27.
         *m11*m13*m44*L4r*RS146pe*RSp463e - 8./27.*m11*m13*m44*L4r*
         RS156pe*RSp563e + 8./27.*m11*m13*m44*L4r*RSp561e*RS156pe + 8./
         27.*m11*m13*m44*L4r*RSp461e*RS146pe + 8./27.*m11*m13*m44*L4r*
         RSp451e*RS145pe + 8./9.*pow(m11,2)*mp2*L5r*RSp45613e - 8./9.*
         pow(m11,2)*mp2*L5r*RSp45611e - 8./3.*pow(m11,2)*m66*L7r*
         RSp453e - 8./9.*pow(m11,2)*m66*L7r*RSp451e - 8./3.*pow(m11,2)*
         m66*L6r*RSp45613e - 8./9.*pow(m11,2)*m66*L6r*RSp45611e - 4./27.
         *pow(m11,2)*m66*L5r*RS145pe*RSp453e );

      M13p6x23LV +=  + BbV(m11,mp2,L) * (  - 4./9.*pow(m11,2)*m66*L5r*
         RSp451e*RS145pe + 4./9.*pow(m11,2)*m66*L4r*RSp45613e + 4./3.*
         pow(m11,2)*m66*L4r*RSp45611e - 4./27.*pow(m11,2)*m66*L4r*
         RS145pe*RSp453e - 4./27.*pow(m11,2)*m66*L4r*RS146pe*RSp463e - 
         4./27.*pow(m11,2)*m66*L4r*RS156pe*RSp563e - 4./9.*pow(m11,2)*
         m66*L4r*RSp561e*RS156pe - 4./9.*pow(m11,2)*m66*L4r*RSp461e*
         RS146pe - 4./9.*pow(m11,2)*m66*L4r*RSp451e*RS145pe - 8./3.*
         pow(m11,2)*m55*L7r*RSp463e - 8./9.*pow(m11,2)*m55*L7r*RSp461e
          - 8./3.*pow(m11,2)*m55*L6r*RSp45613e - 8./9.*pow(m11,2)*m55*
         L6r*RSp45611e - 4./27.*pow(m11,2)*m55*L5r*RS146pe*RSp463e - 4./
         9.*pow(m11,2)*m55*L5r*RSp461e*RS146pe + 4./9.*pow(m11,2)*m55*
         L4r*RSp45613e + 4./3.*pow(m11,2)*m55*L4r*RSp45611e - 4./27.*
         pow(m11,2)*m55*L4r*RS145pe*RSp453e - 4./27.*pow(m11,2)*m55*L4r
         *RS146pe*RSp463e - 4./27.*pow(m11,2)*m55*L4r*RS156pe*RSp563e
          - 4./9.*pow(m11,2)*m55*L4r*RSp561e*RS156pe - 4./9.*pow(m11,2)
         *m55*L4r*RSp461e*RS146pe );

      M13p6x23LV +=  + BbV(m11,mp2,L) * (  - 4./9.*pow(m11,2)*m55*L4r*
         RSp451e*RS145pe - 8./3.*pow(m11,2)*m44*L7r*RSp563e - 8./9.*
         pow(m11,2)*m44*L7r*RSp561e - 8./3.*pow(m11,2)*m44*L6r*
         RSp45613e - 8./9.*pow(m11,2)*m44*L6r*RSp45611e - 4./27.*pow(
         m11,2)*m44*L5r*RS156pe*RSp563e - 4./9.*pow(m11,2)*m44*L5r*
         RSp561e*RS156pe + 4./9.*pow(m11,2)*m44*L4r*RSp45613e + 4./3.*
         pow(m11,2)*m44*L4r*RSp45611e - 4./27.*pow(m11,2)*m44*L4r*
         RS145pe*RSp453e - 4./27.*pow(m11,2)*m44*L4r*RS146pe*RSp463e - 
         4./27.*pow(m11,2)*m44*L4r*RS156pe*RSp563e - 4./9.*pow(m11,2)*
         m44*L4r*RSp561e*RS156pe - 4./9.*pow(m11,2)*m44*L4r*RSp461e*
         RS146pe - 4./9.*pow(m11,2)*m44*L4r*RSp451e*RS145pe - 8./9.*
         pow(m11,2)*m33*L8r*RSp45613e - 8./9.*pow(m11,2)*m33*L8r*
         RSp45611e + 4./9.*pow(m11,2)*m33*L5r*RSp45613e + 4./9.*pow(
         m11,2)*m33*L5r*RSp45611e - 16./9.*pow(m11,2)*m13*L8r*RSp45613e
          + 16./9.*pow(m11,2)*m13*L8r*RSp45611e + 8./9.*pow(m11,2)*m13*
         L5r*RSp45613e );

      M13p6x23LV +=  + BbV(m11,mp2,L) * (  - 8./9.*pow(m11,2)*m13*L5r*
         RSp45611e - 8./3.*pow(m11,3)*L8r*RSp45613e - 8./9.*pow(m11,3)*
         L8r*RSp45611e + 4./9.*pow(m11,3)*L5r*RSp45613e + 4./3.*pow(
         m11,3)*L5r*RSp45611e );

      M13p6x23LV +=  + BbV(m11,me2,L) * ( 8./27.*m33*pow(m66,2)*L8r*
         RS145pe*RSe453p + 8./27.*m33*pow(m66,2)*L8r*RSe451p*RS145pe + 
         8./27.*m33*pow(m66,2)*L7r*RS145pe*RSe453p + 8./27.*m33*pow(
         m66,2)*L7r*RSe451p*RS145pe + 8./27.*m33*pow(m66,2)*L6r*RS145pe
         *RSe453p + 8./27.*m33*pow(m66,2)*L6r*RSe451p*RS145pe + 8./27.*
         m33*m55*m66*L7r*RS145pe*RSe463p + 8./27.*m33*m55*m66*L7r*
         RS146pe*RSe453p + 8./27.*m33*m55*m66*L7r*RSe461p*RS145pe + 8./
         27.*m33*m55*m66*L7r*RSe451p*RS146pe + 8./27.*m33*m55*m66*L6r*
         RS145pe*RSe453p + 8./27.*m33*m55*m66*L6r*RS146pe*RSe463p + 8./
         27.*m33*m55*m66*L6r*RSe461p*RS146pe + 8./27.*m33*m55*m66*L6r*
         RSe451p*RS145pe + 8./27.*m33*pow(m55,2)*L8r*RS146pe*RSe463p + 
         8./27.*m33*pow(m55,2)*L8r*RSe461p*RS146pe + 8./27.*m33*pow(
         m55,2)*L7r*RS146pe*RSe463p + 8./27.*m33*pow(m55,2)*L7r*RSe461p
         *RS146pe + 8./27.*m33*pow(m55,2)*L6r*RS146pe*RSe463p + 8./27.*
         m33*pow(m55,2)*L6r*RSe461p*RS146pe + 8./27.*m33*m44*m66*L7r*
         RS145pe*RSe563p );

      M13p6x23LV +=  + BbV(m11,me2,L) * ( 8./27.*m33*m44*m66*L7r*
         RS156pe*RSe453p + 8./27.*m33*m44*m66*L7r*RSe561p*RS145pe + 8./
         27.*m33*m44*m66*L7r*RSe451p*RS156pe + 8./27.*m33*m44*m66*L6r*
         RS145pe*RSe453p + 8./27.*m33*m44*m66*L6r*RS156pe*RSe563p + 8./
         27.*m33*m44*m66*L6r*RSe561p*RS156pe + 8./27.*m33*m44*m66*L6r*
         RSe451p*RS145pe + 8./27.*m33*m44*m55*L7r*RS146pe*RSe563p + 8./
         27.*m33*m44*m55*L7r*RS156pe*RSe463p + 8./27.*m33*m44*m55*L7r*
         RSe561p*RS146pe + 8./27.*m33*m44*m55*L7r*RSe461p*RS156pe + 8./
         27.*m33*m44*m55*L6r*RS146pe*RSe463p + 8./27.*m33*m44*m55*L6r*
         RS156pe*RSe563p + 8./27.*m33*m44*m55*L6r*RSe561p*RS156pe + 8./
         27.*m33*m44*m55*L6r*RSe461p*RS146pe + 8./27.*m33*pow(m44,2)*
         L8r*RS156pe*RSe563p + 8./27.*m33*pow(m44,2)*L8r*RSe561p*
         RS156pe + 8./27.*m33*pow(m44,2)*L7r*RS156pe*RSe563p + 8./27.*
         m33*pow(m44,2)*L7r*RSe561p*RS156pe + 8./27.*m33*pow(m44,2)*L6r
         *RS156pe*RSe563p + 8./27.*m33*pow(m44,2)*L6r*RSe561p*RS156pe
          + 16./27.*m13*pow(m66,2)*L8r*RS145pe*RSe453p );

      M13p6x23LV +=  + BbV(m11,me2,L) * (  - 16./27.*m13*pow(m66,2)*L8r
         *RSe451p*RS145pe + 16./27.*m13*pow(m66,2)*L7r*RS145pe*RSe453p
          - 16./27.*m13*pow(m66,2)*L7r*RSe451p*RS145pe + 16./27.*m13*
         pow(m66,2)*L6r*RS145pe*RSe453p - 16./27.*m13*pow(m66,2)*L6r*
         RSe451p*RS145pe + 16./27.*m13*m55*m66*L7r*RS145pe*RSe463p + 16.
         /27.*m13*m55*m66*L7r*RS146pe*RSe453p - 16./27.*m13*m55*m66*L7r
         *RSe461p*RS145pe - 16./27.*m13*m55*m66*L7r*RSe451p*RS146pe + 
         16./27.*m13*m55*m66*L6r*RS145pe*RSe453p + 16./27.*m13*m55*m66*
         L6r*RS146pe*RSe463p - 16./27.*m13*m55*m66*L6r*RSe461p*RS146pe
          - 16./27.*m13*m55*m66*L6r*RSe451p*RS145pe + 16./27.*m13*pow(
         m55,2)*L8r*RS146pe*RSe463p - 16./27.*m13*pow(m55,2)*L8r*
         RSe461p*RS146pe + 16./27.*m13*pow(m55,2)*L7r*RS146pe*RSe463p
          - 16./27.*m13*pow(m55,2)*L7r*RSe461p*RS146pe + 16./27.*m13*
         pow(m55,2)*L6r*RS146pe*RSe463p - 16./27.*m13*pow(m55,2)*L6r*
         RSe461p*RS146pe + 16./27.*m13*m44*m66*L7r*RS145pe*RSe563p + 16.
         /27.*m13*m44*m66*L7r*RS156pe*RSe453p );

      M13p6x23LV +=  + BbV(m11,me2,L) * (  - 16./27.*m13*m44*m66*L7r*
         RSe561p*RS145pe - 16./27.*m13*m44*m66*L7r*RSe451p*RS156pe + 16.
         /27.*m13*m44*m66*L6r*RS145pe*RSe453p + 16./27.*m13*m44*m66*L6r
         *RS156pe*RSe563p - 16./27.*m13*m44*m66*L6r*RSe561p*RS156pe - 
         16./27.*m13*m44*m66*L6r*RSe451p*RS145pe + 16./27.*m13*m44*m55*
         L7r*RS146pe*RSe563p + 16./27.*m13*m44*m55*L7r*RS156pe*RSe463p
          - 16./27.*m13*m44*m55*L7r*RSe561p*RS146pe - 16./27.*m13*m44*
         m55*L7r*RSe461p*RS156pe + 16./27.*m13*m44*m55*L6r*RS146pe*
         RSe463p + 16./27.*m13*m44*m55*L6r*RS156pe*RSe563p - 16./27.*
         m13*m44*m55*L6r*RSe561p*RS156pe - 16./27.*m13*m44*m55*L6r*
         RSe461p*RS146pe + 16./27.*m13*pow(m44,2)*L8r*RS156pe*RSe563p
          - 16./27.*m13*pow(m44,2)*L8r*RSe561p*RS156pe + 16./27.*m13*
         pow(m44,2)*L7r*RS156pe*RSe563p - 16./27.*m13*pow(m44,2)*L7r*
         RSe561p*RS156pe + 16./27.*m13*pow(m44,2)*L6r*RS156pe*RSe563p
          - 16./27.*m13*pow(m44,2)*L6r*RSe561p*RS156pe - 8./27.*m11*m66
         *me2*L5r*RS145pe*RSe453p );

      M13p6x23LV +=  + BbV(m11,me2,L) * ( 8./27.*m11*m66*me2*L5r*
         RSe451p*RS145pe + 8./9.*m11*m66*me2*L4r*RSe45613p - 8./9.*m11*
         m66*me2*L4r*RSe45611p - 8./27.*m11*m66*me2*L4r*RS145pe*RSe453p
          - 8./27.*m11*m66*me2*L4r*RS146pe*RSe463p - 8./27.*m11*m66*me2
         *L4r*RS156pe*RSe563p + 8./27.*m11*m66*me2*L4r*RSe561p*RS156pe
          + 8./27.*m11*m66*me2*L4r*RSe461p*RS146pe + 8./27.*m11*m66*me2
         *L4r*RSe451p*RS145pe + 8./9.*m11*pow(m66,2)*L8r*RS145pe*
         RSe453p + 8./27.*m11*pow(m66,2)*L8r*RSe451p*RS145pe + 8./9.*
         m11*pow(m66,2)*L7r*RS145pe*RSe453p + 8./27.*m11*pow(m66,2)*L7r
         *RSe451p*RS145pe + 8./9.*m11*pow(m66,2)*L6r*RS145pe*RSe453p + 
         8./27.*m11*pow(m66,2)*L6r*RSe451p*RS145pe - 8./27.*m11*m55*me2
         *L5r*RS146pe*RSe463p + 8./27.*m11*m55*me2*L5r*RSe461p*RS146pe
          + 8./9.*m11*m55*me2*L4r*RSe45613p - 8./9.*m11*m55*me2*L4r*
         RSe45611p - 8./27.*m11*m55*me2*L4r*RS145pe*RSe453p - 8./27.*
         m11*m55*me2*L4r*RS146pe*RSe463p - 8./27.*m11*m55*me2*L4r*
         RS156pe*RSe563p );

      M13p6x23LV +=  + BbV(m11,me2,L) * ( 8./27.*m11*m55*me2*L4r*
         RSe561p*RS156pe + 8./27.*m11*m55*me2*L4r*RSe461p*RS146pe + 8./
         27.*m11*m55*me2*L4r*RSe451p*RS145pe + 8./9.*m11*m55*m66*L7r*
         RS145pe*RSe463p + 8./9.*m11*m55*m66*L7r*RS146pe*RSe453p + 8./
         27.*m11*m55*m66*L7r*RSe461p*RS145pe + 8./27.*m11*m55*m66*L7r*
         RSe451p*RS146pe + 8./9.*m11*m55*m66*L6r*RS145pe*RSe453p + 8./9.
         *m11*m55*m66*L6r*RS146pe*RSe463p + 8./27.*m11*m55*m66*L6r*
         RSe461p*RS146pe + 8./27.*m11*m55*m66*L6r*RSe451p*RS145pe + 8./
         9.*m11*pow(m55,2)*L8r*RS146pe*RSe463p + 8./27.*m11*pow(m55,2)*
         L8r*RSe461p*RS146pe + 8./9.*m11*pow(m55,2)*L7r*RS146pe*RSe463p
          + 8./27.*m11*pow(m55,2)*L7r*RSe461p*RS146pe + 8./9.*m11*pow(
         m55,2)*L6r*RS146pe*RSe463p + 8./27.*m11*pow(m55,2)*L6r*RSe461p
         *RS146pe - 8./27.*m11*m44*me2*L5r*RS156pe*RSe563p + 8./27.*m11
         *m44*me2*L5r*RSe561p*RS156pe + 8./9.*m11*m44*me2*L4r*RSe45613p
          - 8./9.*m11*m44*me2*L4r*RSe45611p - 8./27.*m11*m44*me2*L4r*
         RS145pe*RSe453p );

      M13p6x23LV +=  + BbV(m11,me2,L) * (  - 8./27.*m11*m44*me2*L4r*
         RS146pe*RSe463p - 8./27.*m11*m44*me2*L4r*RS156pe*RSe563p + 8./
         27.*m11*m44*me2*L4r*RSe561p*RS156pe + 8./27.*m11*m44*me2*L4r*
         RSe461p*RS146pe + 8./27.*m11*m44*me2*L4r*RSe451p*RS145pe + 8./
         9.*m11*m44*m66*L7r*RS145pe*RSe563p + 8./9.*m11*m44*m66*L7r*
         RS156pe*RSe453p + 8./27.*m11*m44*m66*L7r*RSe561p*RS145pe + 8./
         27.*m11*m44*m66*L7r*RSe451p*RS156pe + 8./9.*m11*m44*m66*L6r*
         RS145pe*RSe453p + 8./9.*m11*m44*m66*L6r*RS156pe*RSe563p + 8./
         27.*m11*m44*m66*L6r*RSe561p*RS156pe + 8./27.*m11*m44*m66*L6r*
         RSe451p*RS145pe + 8./9.*m11*m44*m55*L7r*RS146pe*RSe563p + 8./9.
         *m11*m44*m55*L7r*RS156pe*RSe463p + 8./27.*m11*m44*m55*L7r*
         RSe561p*RS146pe + 8./27.*m11*m44*m55*L7r*RSe461p*RS156pe + 8./
         9.*m11*m44*m55*L6r*RS146pe*RSe463p + 8./9.*m11*m44*m55*L6r*
         RS156pe*RSe563p + 8./27.*m11*m44*m55*L6r*RSe561p*RS156pe + 8./
         27.*m11*m44*m55*L6r*RSe461p*RS146pe + 8./9.*m11*pow(m44,2)*L8r
         *RS156pe*RSe563p );

      M13p6x23LV +=  + BbV(m11,me2,L) * ( 8./27.*m11*pow(m44,2)*L8r*
         RSe561p*RS156pe + 8./9.*m11*pow(m44,2)*L7r*RS156pe*RSe563p + 8.
         /27.*m11*pow(m44,2)*L7r*RSe561p*RS156pe + 8./9.*m11*pow(m44,2)
         *L6r*RS156pe*RSe563p + 8./27.*m11*pow(m44,2)*L6r*RSe561p*
         RS156pe - 8./9.*m11*m33*m66*L7r*RSe453p - 8./9.*m11*m33*m66*
         L7r*RSe451p - 8./9.*m11*m33*m66*L6r*RSe45613p - 8./9.*m11*m33*
         m66*L6r*RSe45611p - 4./27.*m11*m33*m66*L5r*RS145pe*RSe453p - 4.
         /27.*m11*m33*m66*L5r*RSe451p*RS145pe + 4./9.*m11*m33*m66*L4r*
         RSe45613p + 4./9.*m11*m33*m66*L4r*RSe45611p - 4./27.*m11*m33*
         m66*L4r*RS145pe*RSe453p - 4./27.*m11*m33*m66*L4r*RS146pe*
         RSe463p - 4./27.*m11*m33*m66*L4r*RS156pe*RSe563p - 4./27.*m11*
         m33*m66*L4r*RSe561p*RS156pe - 4./27.*m11*m33*m66*L4r*RSe461p*
         RS146pe - 4./27.*m11*m33*m66*L4r*RSe451p*RS145pe - 8./9.*m11*
         m33*m55*L7r*RSe463p - 8./9.*m11*m33*m55*L7r*RSe461p - 8./9.*
         m11*m33*m55*L6r*RSe45613p - 8./9.*m11*m33*m55*L6r*RSe45611p - 
         4./27.*m11*m33*m55*L5r*RS146pe*RSe463p );

      M13p6x23LV +=  + BbV(m11,me2,L) * (  - 4./27.*m11*m33*m55*L5r*
         RSe461p*RS146pe + 4./9.*m11*m33*m55*L4r*RSe45613p + 4./9.*m11*
         m33*m55*L4r*RSe45611p - 4./27.*m11*m33*m55*L4r*RS145pe*RSe453p
          - 4./27.*m11*m33*m55*L4r*RS146pe*RSe463p - 4./27.*m11*m33*m55
         *L4r*RS156pe*RSe563p - 4./27.*m11*m33*m55*L4r*RSe561p*RS156pe
          - 4./27.*m11*m33*m55*L4r*RSe461p*RS146pe - 4./27.*m11*m33*m55
         *L4r*RSe451p*RS145pe - 8./9.*m11*m33*m44*L7r*RSe563p - 8./9.*
         m11*m33*m44*L7r*RSe561p - 8./9.*m11*m33*m44*L6r*RSe45613p - 8./
         9.*m11*m33*m44*L6r*RSe45611p - 4./27.*m11*m33*m44*L5r*RS156pe*
         RSe563p - 4./27.*m11*m33*m44*L5r*RSe561p*RS156pe + 4./9.*m11*
         m33*m44*L4r*RSe45613p + 4./9.*m11*m33*m44*L4r*RSe45611p - 4./
         27.*m11*m33*m44*L4r*RS145pe*RSe453p - 4./27.*m11*m33*m44*L4r*
         RS146pe*RSe463p - 4./27.*m11*m33*m44*L4r*RS156pe*RSe563p - 4./
         27.*m11*m33*m44*L4r*RSe561p*RS156pe - 4./27.*m11*m33*m44*L4r*
         RSe461p*RS146pe - 4./27.*m11*m33*m44*L4r*RSe451p*RS145pe - 16./
         9.*m11*m13*m66*L7r*RSe453p );

      M13p6x23LV +=  + BbV(m11,me2,L) * ( 16./9.*m11*m13*m66*L7r*
         RSe451p - 16./9.*m11*m13*m66*L6r*RSe45613p + 16./9.*m11*m13*
         m66*L6r*RSe45611p - 8./27.*m11*m13*m66*L5r*RS145pe*RSe453p + 8.
         /27.*m11*m13*m66*L5r*RSe451p*RS145pe + 8./9.*m11*m13*m66*L4r*
         RSe45613p - 8./9.*m11*m13*m66*L4r*RSe45611p - 8./27.*m11*m13*
         m66*L4r*RS145pe*RSe453p - 8./27.*m11*m13*m66*L4r*RS146pe*
         RSe463p - 8./27.*m11*m13*m66*L4r*RS156pe*RSe563p + 8./27.*m11*
         m13*m66*L4r*RSe561p*RS156pe + 8./27.*m11*m13*m66*L4r*RSe461p*
         RS146pe + 8./27.*m11*m13*m66*L4r*RSe451p*RS145pe - 16./9.*m11*
         m13*m55*L7r*RSe463p + 16./9.*m11*m13*m55*L7r*RSe461p - 16./9.*
         m11*m13*m55*L6r*RSe45613p + 16./9.*m11*m13*m55*L6r*RSe45611p
          - 8./27.*m11*m13*m55*L5r*RS146pe*RSe463p + 8./27.*m11*m13*m55
         *L5r*RSe461p*RS146pe + 8./9.*m11*m13*m55*L4r*RSe45613p - 8./9.
         *m11*m13*m55*L4r*RSe45611p - 8./27.*m11*m13*m55*L4r*RS145pe*
         RSe453p - 8./27.*m11*m13*m55*L4r*RS146pe*RSe463p - 8./27.*m11*
         m13*m55*L4r*RS156pe*RSe563p );

      M13p6x23LV +=  + BbV(m11,me2,L) * ( 8./27.*m11*m13*m55*L4r*
         RSe561p*RS156pe + 8./27.*m11*m13*m55*L4r*RSe461p*RS146pe + 8./
         27.*m11*m13*m55*L4r*RSe451p*RS145pe - 16./9.*m11*m13*m44*L7r*
         RSe563p + 16./9.*m11*m13*m44*L7r*RSe561p - 16./9.*m11*m13*m44*
         L6r*RSe45613p + 16./9.*m11*m13*m44*L6r*RSe45611p - 8./27.*m11*
         m13*m44*L5r*RS156pe*RSe563p + 8./27.*m11*m13*m44*L5r*RSe561p*
         RS156pe + 8./9.*m11*m13*m44*L4r*RSe45613p - 8./9.*m11*m13*m44*
         L4r*RSe45611p - 8./27.*m11*m13*m44*L4r*RS145pe*RSe453p - 8./27.
         *m11*m13*m44*L4r*RS146pe*RSe463p - 8./27.*m11*m13*m44*L4r*
         RS156pe*RSe563p + 8./27.*m11*m13*m44*L4r*RSe561p*RS156pe + 8./
         27.*m11*m13*m44*L4r*RSe461p*RS146pe + 8./27.*m11*m13*m44*L4r*
         RSe451p*RS145pe + 8./9.*pow(m11,2)*me2*L5r*RSe45613p - 8./9.*
         pow(m11,2)*me2*L5r*RSe45611p - 8./3.*pow(m11,2)*m66*L7r*
         RSe453p - 8./9.*pow(m11,2)*m66*L7r*RSe451p - 8./3.*pow(m11,2)*
         m66*L6r*RSe45613p - 8./9.*pow(m11,2)*m66*L6r*RSe45611p - 4./27.
         *pow(m11,2)*m66*L5r*RS145pe*RSe453p );

      M13p6x23LV +=  + BbV(m11,me2,L) * (  - 4./9.*pow(m11,2)*m66*L5r*
         RSe451p*RS145pe + 4./9.*pow(m11,2)*m66*L4r*RSe45613p + 4./3.*
         pow(m11,2)*m66*L4r*RSe45611p - 4./27.*pow(m11,2)*m66*L4r*
         RS145pe*RSe453p - 4./27.*pow(m11,2)*m66*L4r*RS146pe*RSe463p - 
         4./27.*pow(m11,2)*m66*L4r*RS156pe*RSe563p - 4./9.*pow(m11,2)*
         m66*L4r*RSe561p*RS156pe - 4./9.*pow(m11,2)*m66*L4r*RSe461p*
         RS146pe - 4./9.*pow(m11,2)*m66*L4r*RSe451p*RS145pe - 8./3.*
         pow(m11,2)*m55*L7r*RSe463p - 8./9.*pow(m11,2)*m55*L7r*RSe461p
          - 8./3.*pow(m11,2)*m55*L6r*RSe45613p - 8./9.*pow(m11,2)*m55*
         L6r*RSe45611p - 4./27.*pow(m11,2)*m55*L5r*RS146pe*RSe463p - 4./
         9.*pow(m11,2)*m55*L5r*RSe461p*RS146pe + 4./9.*pow(m11,2)*m55*
         L4r*RSe45613p + 4./3.*pow(m11,2)*m55*L4r*RSe45611p - 4./27.*
         pow(m11,2)*m55*L4r*RS145pe*RSe453p - 4./27.*pow(m11,2)*m55*L4r
         *RS146pe*RSe463p - 4./27.*pow(m11,2)*m55*L4r*RS156pe*RSe563p
          - 4./9.*pow(m11,2)*m55*L4r*RSe561p*RS156pe - 4./9.*pow(m11,2)
         *m55*L4r*RSe461p*RS146pe );

      M13p6x23LV +=  + BbV(m11,me2,L) * (  - 4./9.*pow(m11,2)*m55*L4r*
         RSe451p*RS145pe - 8./3.*pow(m11,2)*m44*L7r*RSe563p - 8./9.*
         pow(m11,2)*m44*L7r*RSe561p - 8./3.*pow(m11,2)*m44*L6r*
         RSe45613p - 8./9.*pow(m11,2)*m44*L6r*RSe45611p - 4./27.*pow(
         m11,2)*m44*L5r*RS156pe*RSe563p - 4./9.*pow(m11,2)*m44*L5r*
         RSe561p*RS156pe + 4./9.*pow(m11,2)*m44*L4r*RSe45613p + 4./3.*
         pow(m11,2)*m44*L4r*RSe45611p - 4./27.*pow(m11,2)*m44*L4r*
         RS145pe*RSe453p - 4./27.*pow(m11,2)*m44*L4r*RS146pe*RSe463p - 
         4./27.*pow(m11,2)*m44*L4r*RS156pe*RSe563p - 4./9.*pow(m11,2)*
         m44*L4r*RSe561p*RS156pe - 4./9.*pow(m11,2)*m44*L4r*RSe461p*
         RS146pe - 4./9.*pow(m11,2)*m44*L4r*RSe451p*RS145pe - 8./9.*
         pow(m11,2)*m33*L8r*RSe45613p - 8./9.*pow(m11,2)*m33*L8r*
         RSe45611p + 4./9.*pow(m11,2)*m33*L5r*RSe45613p + 4./9.*pow(
         m11,2)*m33*L5r*RSe45611p - 16./9.*pow(m11,2)*m13*L8r*RSe45613p
          + 16./9.*pow(m11,2)*m13*L8r*RSe45611p + 8./9.*pow(m11,2)*m13*
         L5r*RSe45613p );

      M13p6x23LV +=  + BbV(m11,me2,L) * (  - 8./9.*pow(m11,2)*m13*L5r*
         RSe45611p - 8./3.*pow(m11,3)*L8r*RSe45613p - 8./9.*pow(m11,3)*
         L8r*RSe45611p + 4./9.*pow(m11,3)*L5r*RSe45613p + 4./3.*pow(
         m11,3)*L5r*RSe45611p );

      M13p6x23LV +=  + BbV(m11,L) * ( 2./9.*m33*m66*L4r*RD1456ep + 4./
         27.*m33*pow(m66,2)*L8r*pow(RS145pe,2) + 4./27.*m33*pow(m66,2)*
         L7r*pow(RS145pe,2) + 4./27.*m33*pow(m66,2)*L6r*pow(RS145pe,2)
          + 2./9.*m33*m55*L4r*RD1456ep + 8./27.*m33*m55*m66*L7r*RS146pe
         *RS145pe + 4./27.*m33*m55*m66*L6r*pow(RS145pe,2) + 4./27.*m33*
         m55*m66*L6r*pow(RS146pe,2) + 4./27.*m33*pow(m55,2)*L8r*pow(
         RS146pe,2) + 4./27.*m33*pow(m55,2)*L7r*pow(RS146pe,2) + 4./27.
         *m33*pow(m55,2)*L6r*pow(RS146pe,2) + 2./9.*m33*m44*L4r*
         RD1456ep + 8./27.*m33*m44*m66*L7r*RS156pe*RS145pe + 4./27.*m33
         *m44*m66*L6r*pow(RS145pe,2) + 4./27.*m33*m44*m66*L6r*pow(
         RS156pe,2) + 8./27.*m33*m44*m55*L7r*RS156pe*RS146pe + 4./27.*
         m33*m44*m55*L6r*pow(RS146pe,2) + 4./27.*m33*m44*m55*L6r*pow(
         RS156pe,2) + 4./27.*m33*pow(m44,2)*L8r*pow(RS156pe,2) + 4./27.
         *m33*pow(m44,2)*L7r*pow(RS156pe,2) + 4./27.*m33*pow(m44,2)*L6r
         *pow(RS156pe,2) - 1./9.*pow(m33,2)*L5r*RD1456ep - 4./9.*m13*
         m66*L4r*RD1456ep );

      M13p6x23LV +=  + BbV(m11,L) * (  - 8./27.*m13*pow(m66,2)*L8r*pow(
         RS145pe,2) - 8./27.*m13*pow(m66,2)*L7r*pow(RS145pe,2) - 8./27.
         *m13*pow(m66,2)*L6r*pow(RS145pe,2) - 4./9.*m13*m55*L4r*
         RD1456ep - 16./27.*m13*m55*m66*L7r*RS146pe*RS145pe - 8./27.*
         m13*m55*m66*L6r*pow(RS145pe,2) - 8./27.*m13*m55*m66*L6r*pow(
         RS146pe,2) - 8./27.*m13*pow(m55,2)*L8r*pow(RS146pe,2) - 8./27.
         *m13*pow(m55,2)*L7r*pow(RS146pe,2) - 8./27.*m13*pow(m55,2)*L6r
         *pow(RS146pe,2) - 4./9.*m13*m44*L4r*RD1456ep - 16./27.*m13*m44
         *m66*L7r*RS156pe*RS145pe - 8./27.*m13*m44*m66*L6r*pow(
         RS145pe,2) - 8./27.*m13*m44*m66*L6r*pow(RS156pe,2) - 16./27.*
         m13*m44*m55*L7r*RS156pe*RS146pe - 8./27.*m13*m44*m55*L6r*pow(
         RS146pe,2) - 8./27.*m13*m44*m55*L6r*pow(RS156pe,2) - 8./27.*
         m13*pow(m44,2)*L8r*pow(RS156pe,2) - 8./27.*m13*pow(m44,2)*L7r*
         pow(RS156pe,2) - 8./27.*m13*pow(m44,2)*L6r*pow(RS156pe,2) + 2./
         9.*m13*m33*L5r*RD1456ep + 8./9.*m11*m66*L6r*RD1456ep - 2./9.*
         m11*m66*L4r*RD1456ep );

      M13p6x23LV +=  + BbV(m11,L) * ( 4./27.*m11*pow(m66,2)*L8r*pow(
         RS145pe,2) + 4./27.*m11*pow(m66,2)*L7r*pow(RS145pe,2) + 4./27.
         *m11*pow(m66,2)*L6r*pow(RS145pe,2) + 8./9.*m11*m55*L6r*
         RD1456ep - 2./9.*m11*m55*L4r*RD1456ep + 8./27.*m11*m55*m66*L7r
         *RS146pe*RS145pe + 4./27.*m11*m55*m66*L6r*pow(RS145pe,2) + 4./
         27.*m11*m55*m66*L6r*pow(RS146pe,2) + 4./27.*m11*pow(m55,2)*L8r
         *pow(RS146pe,2) + 4./27.*m11*pow(m55,2)*L7r*pow(RS146pe,2) + 4.
         /27.*m11*pow(m55,2)*L6r*pow(RS146pe,2) + 8./9.*m11*m44*L6r*
         RD1456ep - 2./9.*m11*m44*L4r*RD1456ep + 8./27.*m11*m44*m66*L7r
         *RS156pe*RS145pe + 4./27.*m11*m44*m66*L6r*pow(RS145pe,2) + 4./
         27.*m11*m44*m66*L6r*pow(RS156pe,2) + 8./27.*m11*m44*m55*L7r*
         RS156pe*RS146pe + 4./27.*m11*m44*m55*L6r*pow(RS146pe,2) + 4./
         27.*m11*m44*m55*L6r*pow(RS156pe,2) + 4./27.*m11*pow(m44,2)*L8r
         *pow(RS156pe,2) + 4./27.*m11*pow(m44,2)*L7r*pow(RS156pe,2) + 4.
         /27.*m11*pow(m44,2)*L6r*pow(RS156pe,2) - 8./3.*m11*m33*L8r*
         RD1456ep );

      M13p6x23LV +=  + BbV(m11,L) * ( 8./9.*m11*m33*L5r*RD1456ep - 8./9.
         *m11*m33*m66*L7r*RS145pe - 8./9.*m11*m33*m66*L6r*RS14563pe - 8.
         /9.*m11*m33*m66*L6r*RC1456pe - 2./27.*m11*m33*m66*L5r*pow(
         RS145pe,2) + 4./9.*m11*m33*m66*L4r*RS14563pe + 4./9.*m11*m33*
         m66*L4r*RC1456pe - 2./27.*m11*m33*m66*L4r*pow(RS145pe,2) - 2./
         27.*m11*m33*m66*L4r*pow(RS146pe,2) - 2./27.*m11*m33*m66*L4r*
         pow(RS156pe,2) - 8./9.*m11*m33*m55*L7r*RS146pe - 8./9.*m11*m33
         *m55*L6r*RS14563pe - 8./9.*m11*m33*m55*L6r*RC1456pe - 2./27.*
         m11*m33*m55*L5r*pow(RS146pe,2) + 4./9.*m11*m33*m55*L4r*
         RS14563pe + 4./9.*m11*m33*m55*L4r*RC1456pe - 2./27.*m11*m33*
         m55*L4r*pow(RS145pe,2) - 2./27.*m11*m33*m55*L4r*pow(RS146pe,2)
          - 2./27.*m11*m33*m55*L4r*pow(RS156pe,2) - 8./9.*m11*m33*m44*
         L7r*RS156pe - 8./9.*m11*m33*m44*L6r*RS14563pe - 8./9.*m11*m33*
         m44*L6r*RC1456pe - 2./27.*m11*m33*m44*L5r*pow(RS156pe,2) + 4./
         9.*m11*m33*m44*L4r*RS14563pe + 4./9.*m11*m33*m44*L4r*RC1456pe
          - 2./27.*m11*m33*m44*L4r*pow(RS145pe,2) );

      M13p6x23LV +=  + BbV(m11,L) * (  - 2./27.*m11*m33*m44*L4r*pow(
         RS146pe,2) - 2./27.*m11*m33*m44*L4r*pow(RS156pe,2) + 2./3.*m11
         *m13*L5r*RD1456ep - 10./3.*m11*m13*L3r*RD1456ep - 4./3.*m11*
         m13*L0r*RD1456ep + 16./9.*m11*m13*m66*L7r*RS145pe - 16./9.*m11
         *m13*m66*L6r*RS14563pe + 16./9.*m11*m13*m66*L6r*RC1456pe + 4./
         27.*m11*m13*m66*L5r*pow(RS145pe,2) + 8./9.*m11*m13*m66*L4r*
         RS14563pe - 8./9.*m11*m13*m66*L4r*RC1456pe + 4./27.*m11*m13*
         m66*L4r*pow(RS145pe,2) + 4./27.*m11*m13*m66*L4r*pow(RS146pe,2)
          + 4./27.*m11*m13*m66*L4r*pow(RS156pe,2) + 16./9.*m11*m13*m55*
         L7r*RS146pe - 16./9.*m11*m13*m55*L6r*RS14563pe + 16./9.*m11*
         m13*m55*L6r*RC1456pe + 4./27.*m11*m13*m55*L5r*pow(RS146pe,2)
          + 8./9.*m11*m13*m55*L4r*RS14563pe - 8./9.*m11*m13*m55*L4r*
         RC1456pe + 4./27.*m11*m13*m55*L4r*pow(RS145pe,2) + 4./27.*m11*
         m13*m55*L4r*pow(RS146pe,2) + 4./27.*m11*m13*m55*L4r*pow(
         RS156pe,2) + 16./9.*m11*m13*m44*L7r*RS156pe - 16./9.*m11*m13*
         m44*L6r*RS14563pe );

      M13p6x23LV +=  + BbV(m11,L) * ( 16./9.*m11*m13*m44*L6r*RC1456pe
          + 4./27.*m11*m13*m44*L5r*pow(RS156pe,2) + 8./9.*m11*m13*m44*
         L4r*RS14563pe - 8./9.*m11*m13*m44*L4r*RC1456pe + 4./27.*m11*
         m13*m44*L4r*pow(RS145pe,2) + 4./27.*m11*m13*m44*L4r*pow(
         RS146pe,2) + 4./27.*m11*m13*m44*L4r*pow(RS156pe,2) - 16./9.*
         pow(m11,2)*L8r*RD1456ep + 5./9.*pow(m11,2)*L5r*RD1456ep - 8./9.
         *pow(m11,2)*m66*L7r*RS145pe - 8./3.*pow(m11,2)*m66*L6r*
         RS14563pe - 8./9.*pow(m11,2)*m66*L6r*RC1456pe - 2./27.*pow(
         m11,2)*m66*L5r*pow(RS145pe,2) + 4./3.*pow(m11,2)*m66*L4r*
         RS14563pe + 4./9.*pow(m11,2)*m66*L4r*RC1456pe - 2./27.*pow(
         m11,2)*m66*L4r*pow(RS145pe,2) - 2./27.*pow(m11,2)*m66*L4r*pow(
         RS146pe,2) - 2./27.*pow(m11,2)*m66*L4r*pow(RS156pe,2) - 8./9.*
         pow(m11,2)*m55*L7r*RS146pe - 8./3.*pow(m11,2)*m55*L6r*
         RS14563pe - 8./9.*pow(m11,2)*m55*L6r*RC1456pe - 2./27.*pow(
         m11,2)*m55*L5r*pow(RS146pe,2) + 4./3.*pow(m11,2)*m55*L4r*
         RS14563pe );

      M13p6x23LV +=  + BbV(m11,L) * ( 4./9.*pow(m11,2)*m55*L4r*RC1456pe
          - 2./27.*pow(m11,2)*m55*L4r*pow(RS145pe,2) - 2./27.*pow(
         m11,2)*m55*L4r*pow(RS146pe,2) - 2./27.*pow(m11,2)*m55*L4r*pow(
         RS156pe,2) - 8./9.*pow(m11,2)*m44*L7r*RS156pe - 8./3.*pow(
         m11,2)*m44*L6r*RS14563pe - 8./9.*pow(m11,2)*m44*L6r*RC1456pe
          - 2./27.*pow(m11,2)*m44*L5r*pow(RS156pe,2) + 4./3.*pow(m11,2)
         *m44*L4r*RS14563pe + 4./9.*pow(m11,2)*m44*L4r*RC1456pe - 2./27.
         *pow(m11,2)*m44*L4r*pow(RS145pe,2) - 2./27.*pow(m11,2)*m44*L4r
         *pow(RS146pe,2) - 2./27.*pow(m11,2)*m44*L4r*pow(RS156pe,2) - 8.
         /9.*pow(m11,2)*m33*L8r*RS14563pe - 8./9.*pow(m11,2)*m33*L8r*
         RC1456pe + 4./3.*pow(m11,2)*m33*L7r + 4./9.*pow(m11,2)*m33*L5r
         *RS14563pe + 4./9.*pow(m11,2)*m33*L5r*RC1456pe - 16./9.*pow(
         m11,2)*m13*L8r*RS14563pe + 16./9.*pow(m11,2)*m13*L8r*RC1456pe
          - 8./3.*pow(m11,2)*m13*L7r + 8./9.*pow(m11,2)*m13*L5r*
         RS14563pe - 8./9.*pow(m11,2)*m13*L5r*RC1456pe - 8./3.*pow(
         m11,3)*L8r*RS14563pe );

      M13p6x23LV +=  + BbV(m11,L) * (  - 8./9.*pow(m11,3)*L8r*RC1456pe
          + 4./3.*pow(m11,3)*L7r + 4./3.*pow(m11,3)*L5r*RS14563pe + 4./
         9.*pow(m11,3)*L5r*RC1456pe );

      M13p6x23LV +=  + BbV(m14,L) * ( 2./3.*pow(m44,2)*m66*L6r + 2./3.*
         pow(m44,2)*m55*L6r + 1./3.*pow(m44,3)*L8r + 2./3.*pow(m44,3)*
         L6r + 2./3.*m33*m44*m66*L6r + 2./3.*m33*m44*m55*L6r + 1./3.*
         m33*pow(m44,2)*L8r + 2./3.*m33*pow(m44,2)*L6r - 4./3.*m14*m44*
         m66*L6r - 2./3.*m14*m44*m66*L4r - 4./3.*m14*m44*m55*L6r - 2./3.
         *m14*m44*m55*L4r - 2./3.*m14*pow(m44,2)*L8r - 4./3.*m14*pow(
         m44,2)*L6r - 1./3.*m14*pow(m44,2)*L5r - 2./3.*m14*pow(m44,2)*
         L4r - 2./3.*m14*m33*m66*L4r - 2./3.*m14*m33*m55*L4r - 1./3.*
         m14*m33*m44*L5r - 2./3.*m14*m33*m44*L4r + 4./3.*pow(m14,2)*m66
         *L4r + 4./3.*pow(m14,2)*m55*L4r + 2./3.*pow(m14,2)*m44*L5r + 4.
         /3.*pow(m14,2)*m44*L4r - 4./3.*m13*m44*m66*L6r - 4./3.*m13*m44
         *m55*L6r - 2./3.*m13*pow(m44,2)*L8r - 4./3.*m13*pow(m44,2)*L6r
          + 4./3.*m13*m14*m66*L4r + 4./3.*m13*m14*m55*L4r + 2./3.*m13*
         m14*m44*L5r + 4./3.*m13*m14*m44*L4r + 2*m11*m44*m66*L6r + 2*
         m11*m44*m55*L6r + 4./3.*m11*pow(m44,2)*L8r + 2*m11*pow(m44,2)*
         L6r );

      M13p6x23LV +=  + BbV(m14,L) * ( 2./3.*m11*m33*m66*L6r + 2./3.*m11
         *m33*m55*L6r + 2./3.*m11*m33*m44*L8r + 2./3.*m11*m33*m44*L6r
          - 4./3.*m11*m14*m66*L6r - 4./3.*m11*m14*m66*L4r - 4./3.*m11*
         m14*m55*L6r - 4./3.*m11*m14*m55*L4r - 4./3.*m11*m14*m44*L8r - 
         4./3.*m11*m14*m44*L6r - m11*m14*m44*L5r - 4./3.*m11*m14*m44*
         L4r - 1./3.*m11*m14*m33*L5r + 2./3.*m11*pow(m14,2)*L5r - 4./3.
         *m11*m13*m66*L6r - 4./3.*m11*m13*m55*L6r - 4./3.*m11*m13*m44*
         L8r - 4./3.*m11*m13*m44*L6r + 2./3.*m11*m13*m14*L5r + 4./3.*
         pow(m11,2)*m66*L6r + 4./3.*pow(m11,2)*m55*L6r + 5./3.*pow(
         m11,2)*m44*L8r + 4./3.*pow(m11,2)*m44*L6r + 1./3.*pow(m11,2)*
         m33*L8r - 2./3.*pow(m11,2)*m14*L8r - 2./3.*pow(m11,2)*m14*L5r
          - 2./3.*pow(m11,2)*m13*L8r + 2./3.*pow(m11,3)*L8r );

      M13p6x23LV +=  + BbV(m15,L) * ( 2./3.*pow(m55,2)*m66*L6r + 1./3.*
         pow(m55,3)*L8r + 2./3.*pow(m55,3)*L6r + 2./3.*m44*pow(m55,2)*
         L6r + 2./3.*m33*m55*m66*L6r + 1./3.*m33*pow(m55,2)*L8r + 2./3.
         *m33*pow(m55,2)*L6r + 2./3.*m33*m44*m55*L6r - 4./3.*m15*m55*
         m66*L6r - 2./3.*m15*m55*m66*L4r - 2./3.*m15*pow(m55,2)*L8r - 4.
         /3.*m15*pow(m55,2)*L6r - 1./3.*m15*pow(m55,2)*L5r - 2./3.*m15*
         pow(m55,2)*L4r - 4./3.*m15*m44*m55*L6r - 2./3.*m15*m44*m55*L4r
          - 2./3.*m15*m33*m66*L4r - 1./3.*m15*m33*m55*L5r - 2./3.*m15*
         m33*m55*L4r - 2./3.*m15*m33*m44*L4r + 4./3.*pow(m15,2)*m66*L4r
          + 2./3.*pow(m15,2)*m55*L5r + 4./3.*pow(m15,2)*m55*L4r + 4./3.
         *pow(m15,2)*m44*L4r - 4./3.*m13*m55*m66*L6r - 2./3.*m13*pow(
         m55,2)*L8r - 4./3.*m13*pow(m55,2)*L6r - 4./3.*m13*m44*m55*L6r
          + 4./3.*m13*m15*m66*L4r + 2./3.*m13*m15*m55*L5r + 4./3.*m13*
         m15*m55*L4r + 4./3.*m13*m15*m44*L4r + 2*m11*m55*m66*L6r + 4./3.
         *m11*pow(m55,2)*L8r + 2*m11*pow(m55,2)*L6r + 2*m11*m44*m55*L6r
          + 2./3.*m11*m33*m66*L6r );

      M13p6x23LV +=  + BbV(m15,L) * ( 2./3.*m11*m33*m55*L8r + 2./3.*m11
         *m33*m55*L6r + 2./3.*m11*m33*m44*L6r - 4./3.*m11*m15*m66*L6r
          - 4./3.*m11*m15*m66*L4r - 4./3.*m11*m15*m55*L8r - 4./3.*m11*
         m15*m55*L6r - m11*m15*m55*L5r - 4./3.*m11*m15*m55*L4r - 4./3.*
         m11*m15*m44*L6r - 4./3.*m11*m15*m44*L4r - 1./3.*m11*m15*m33*
         L5r + 2./3.*m11*pow(m15,2)*L5r - 4./3.*m11*m13*m66*L6r - 4./3.
         *m11*m13*m55*L8r - 4./3.*m11*m13*m55*L6r - 4./3.*m11*m13*m44*
         L6r + 2./3.*m11*m13*m15*L5r + 4./3.*pow(m11,2)*m66*L6r + 5./3.
         *pow(m11,2)*m55*L8r + 4./3.*pow(m11,2)*m55*L6r + 4./3.*pow(
         m11,2)*m44*L6r + 1./3.*pow(m11,2)*m33*L8r - 2./3.*pow(m11,2)*
         m15*L8r - 2./3.*pow(m11,2)*m15*L5r - 2./3.*pow(m11,2)*m13*L8r
          + 2./3.*pow(m11,3)*L8r );

      M13p6x23LV +=  + BbV(m16,L) * ( 1./3.*pow(m66,3)*L8r + 2./3.*pow(
         m66,3)*L6r + 2./3.*m55*pow(m66,2)*L6r + 2./3.*m44*pow(m66,2)*
         L6r + 1./3.*m33*pow(m66,2)*L8r + 2./3.*m33*pow(m66,2)*L6r + 2./
         3.*m33*m55*m66*L6r + 2./3.*m33*m44*m66*L6r - 2./3.*m16*pow(
         m66,2)*L8r - 4./3.*m16*pow(m66,2)*L6r - 1./3.*m16*pow(m66,2)*
         L5r - 2./3.*m16*pow(m66,2)*L4r - 4./3.*m16*m55*m66*L6r - 2./3.
         *m16*m55*m66*L4r - 4./3.*m16*m44*m66*L6r - 2./3.*m16*m44*m66*
         L4r - 1./3.*m16*m33*m66*L5r - 2./3.*m16*m33*m66*L4r - 2./3.*
         m16*m33*m55*L4r - 2./3.*m16*m33*m44*L4r + 2./3.*pow(m16,2)*m66
         *L5r + 4./3.*pow(m16,2)*m66*L4r + 4./3.*pow(m16,2)*m55*L4r + 4.
         /3.*pow(m16,2)*m44*L4r - 2./3.*m13*pow(m66,2)*L8r - 4./3.*m13*
         pow(m66,2)*L6r - 4./3.*m13*m55*m66*L6r - 4./3.*m13*m44*m66*L6r
          + 2./3.*m13*m16*m66*L5r + 4./3.*m13*m16*m66*L4r + 4./3.*m13*
         m16*m55*L4r + 4./3.*m13*m16*m44*L4r + 4./3.*m11*pow(m66,2)*L8r
          + 2*m11*pow(m66,2)*L6r + 2*m11*m55*m66*L6r + 2*m11*m44*m66*
         L6r );

      M13p6x23LV +=  + BbV(m16,L) * ( 2./3.*m11*m33*m66*L8r + 2./3.*m11
         *m33*m66*L6r + 2./3.*m11*m33*m55*L6r + 2./3.*m11*m33*m44*L6r
          - 4./3.*m11*m16*m66*L8r - 4./3.*m11*m16*m66*L6r - m11*m16*m66
         *L5r - 4./3.*m11*m16*m66*L4r - 4./3.*m11*m16*m55*L6r - 4./3.*
         m11*m16*m55*L4r - 4./3.*m11*m16*m44*L6r - 4./3.*m11*m16*m44*
         L4r - 1./3.*m11*m16*m33*L5r + 2./3.*m11*pow(m16,2)*L5r - 4./3.
         *m11*m13*m66*L8r - 4./3.*m11*m13*m66*L6r - 4./3.*m11*m13*m55*
         L6r - 4./3.*m11*m13*m44*L6r + 2./3.*m11*m13*m16*L5r + 5./3.*
         pow(m11,2)*m66*L8r + 4./3.*pow(m11,2)*m66*L6r + 4./3.*pow(
         m11,2)*m55*L6r + 4./3.*pow(m11,2)*m44*L6r + 1./3.*pow(m11,2)*
         m33*L8r - 2./3.*pow(m11,2)*m16*L8r - 2./3.*pow(m11,2)*m16*L5r
          - 2./3.*pow(m11,2)*m13*L8r + 2./3.*pow(m11,3)*L8r );

      M13p6x23LV +=  + BbV(m33,mp2,L) * ( 8./27.*m33*m66*mp2*L5r*
         RSp453e*RS345pe - 8./27.*m33*m66*mp2*L5r*RSp451e*RS345pe + 8./
         9.*m33*m66*mp2*L4r*RSp45613e - 8./9.*m33*m66*mp2*L4r*RSp45633e
          + 8./27.*m33*m66*mp2*L4r*RSp563e*RS356pe + 8./27.*m33*m66*mp2
         *L4r*RSp463e*RS346pe + 8./27.*m33*m66*mp2*L4r*RSp453e*RS345pe
          - 8./27.*m33*m66*mp2*L4r*RSp561e*RS356pe - 8./27.*m33*m66*mp2
         *L4r*RSp461e*RS346pe - 8./27.*m33*m66*mp2*L4r*RSp451e*RS345pe
          + 8./27.*m33*pow(m66,2)*L8r*RSp453e*RS345pe + 8./9.*m33*pow(
         m66,2)*L8r*RSp451e*RS345pe + 8./27.*m33*pow(m66,2)*L7r*RSp453e
         *RS345pe + 8./9.*m33*pow(m66,2)*L7r*RSp451e*RS345pe + 8./27.*
         m33*pow(m66,2)*L6r*RSp453e*RS345pe + 8./9.*m33*pow(m66,2)*L6r*
         RSp451e*RS345pe + 8./27.*m33*m55*mp2*L5r*RSp463e*RS346pe - 8./
         27.*m33*m55*mp2*L5r*RSp461e*RS346pe + 8./9.*m33*m55*mp2*L4r*
         RSp45613e - 8./9.*m33*m55*mp2*L4r*RSp45633e + 8./27.*m33*m55*
         mp2*L4r*RSp563e*RS356pe + 8./27.*m33*m55*mp2*L4r*RSp463e*
         RS346pe );

      M13p6x23LV +=  + BbV(m33,mp2,L) * ( 8./27.*m33*m55*mp2*L4r*
         RSp453e*RS345pe - 8./27.*m33*m55*mp2*L4r*RSp561e*RS356pe - 8./
         27.*m33*m55*mp2*L4r*RSp461e*RS346pe - 8./27.*m33*m55*mp2*L4r*
         RSp451e*RS345pe + 8./27.*m33*m55*m66*L7r*RSp463e*RS345pe + 8./
         27.*m33*m55*m66*L7r*RSp453e*RS346pe + 8./9.*m33*m55*m66*L7r*
         RSp461e*RS345pe + 8./9.*m33*m55*m66*L7r*RSp451e*RS346pe + 8./
         27.*m33*m55*m66*L6r*RSp463e*RS346pe + 8./27.*m33*m55*m66*L6r*
         RSp453e*RS345pe + 8./9.*m33*m55*m66*L6r*RSp461e*RS346pe + 8./9.
         *m33*m55*m66*L6r*RSp451e*RS345pe + 8./27.*m33*pow(m55,2)*L8r*
         RSp463e*RS346pe + 8./9.*m33*pow(m55,2)*L8r*RSp461e*RS346pe + 8.
         /27.*m33*pow(m55,2)*L7r*RSp463e*RS346pe + 8./9.*m33*pow(m55,2)
         *L7r*RSp461e*RS346pe + 8./27.*m33*pow(m55,2)*L6r*RSp463e*
         RS346pe + 8./9.*m33*pow(m55,2)*L6r*RSp461e*RS346pe + 8./27.*
         m33*m44*mp2*L5r*RSp563e*RS356pe - 8./27.*m33*m44*mp2*L5r*
         RSp561e*RS356pe + 8./9.*m33*m44*mp2*L4r*RSp45613e - 8./9.*m33*
         m44*mp2*L4r*RSp45633e );

      M13p6x23LV +=  + BbV(m33,mp2,L) * ( 8./27.*m33*m44*mp2*L4r*
         RSp563e*RS356pe + 8./27.*m33*m44*mp2*L4r*RSp463e*RS346pe + 8./
         27.*m33*m44*mp2*L4r*RSp453e*RS345pe - 8./27.*m33*m44*mp2*L4r*
         RSp561e*RS356pe - 8./27.*m33*m44*mp2*L4r*RSp461e*RS346pe - 8./
         27.*m33*m44*mp2*L4r*RSp451e*RS345pe + 8./27.*m33*m44*m66*L7r*
         RSp563e*RS345pe + 8./27.*m33*m44*m66*L7r*RSp453e*RS356pe + 8./
         9.*m33*m44*m66*L7r*RSp561e*RS345pe + 8./9.*m33*m44*m66*L7r*
         RSp451e*RS356pe + 8./27.*m33*m44*m66*L6r*RSp563e*RS356pe + 8./
         27.*m33*m44*m66*L6r*RSp453e*RS345pe + 8./9.*m33*m44*m66*L6r*
         RSp561e*RS356pe + 8./9.*m33*m44*m66*L6r*RSp451e*RS345pe + 8./
         27.*m33*m44*m55*L7r*RSp563e*RS346pe + 8./27.*m33*m44*m55*L7r*
         RSp463e*RS356pe + 8./9.*m33*m44*m55*L7r*RSp561e*RS346pe + 8./9.
         *m33*m44*m55*L7r*RSp461e*RS356pe + 8./27.*m33*m44*m55*L6r*
         RSp563e*RS356pe + 8./27.*m33*m44*m55*L6r*RSp463e*RS346pe + 8./
         9.*m33*m44*m55*L6r*RSp561e*RS356pe + 8./9.*m33*m44*m55*L6r*
         RSp461e*RS346pe );

      M13p6x23LV +=  + BbV(m33,mp2,L) * ( 8./27.*m33*pow(m44,2)*L8r*
         RSp563e*RS356pe + 8./9.*m33*pow(m44,2)*L8r*RSp561e*RS356pe + 8.
         /27.*m33*pow(m44,2)*L7r*RSp563e*RS356pe + 8./9.*m33*pow(m44,2)
         *L7r*RSp561e*RS356pe + 8./27.*m33*pow(m44,2)*L6r*RSp563e*
         RS356pe + 8./9.*m33*pow(m44,2)*L6r*RSp561e*RS356pe + 8./9.*
         pow(m33,2)*mp2*L5r*RSp45613e - 8./9.*pow(m33,2)*mp2*L5r*
         RSp45633e - 8./9.*pow(m33,2)*m66*L7r*RSp453e - 8./3.*pow(
         m33,2)*m66*L7r*RSp451e - 8./3.*pow(m33,2)*m66*L6r*RSp45613e - 
         8./9.*pow(m33,2)*m66*L6r*RSp45633e - 4./9.*pow(m33,2)*m66*L5r*
         RSp453e*RS345pe - 4./27.*pow(m33,2)*m66*L5r*RSp451e*RS345pe + 
         4./9.*pow(m33,2)*m66*L4r*RSp45613e + 4./3.*pow(m33,2)*m66*L4r*
         RSp45633e - 4./9.*pow(m33,2)*m66*L4r*RSp563e*RS356pe - 4./9.*
         pow(m33,2)*m66*L4r*RSp463e*RS346pe - 4./9.*pow(m33,2)*m66*L4r*
         RSp453e*RS345pe - 4./27.*pow(m33,2)*m66*L4r*RSp561e*RS356pe - 
         4./27.*pow(m33,2)*m66*L4r*RSp461e*RS346pe - 4./27.*pow(m33,2)*
         m66*L4r*RSp451e*RS345pe );

      M13p6x23LV +=  + BbV(m33,mp2,L) * (  - 8./9.*pow(m33,2)*m55*L7r*
         RSp463e - 8./3.*pow(m33,2)*m55*L7r*RSp461e - 8./3.*pow(m33,2)*
         m55*L6r*RSp45613e - 8./9.*pow(m33,2)*m55*L6r*RSp45633e - 4./9.
         *pow(m33,2)*m55*L5r*RSp463e*RS346pe - 4./27.*pow(m33,2)*m55*
         L5r*RSp461e*RS346pe + 4./9.*pow(m33,2)*m55*L4r*RSp45613e + 4./
         3.*pow(m33,2)*m55*L4r*RSp45633e - 4./9.*pow(m33,2)*m55*L4r*
         RSp563e*RS356pe - 4./9.*pow(m33,2)*m55*L4r*RSp463e*RS346pe - 4.
         /9.*pow(m33,2)*m55*L4r*RSp453e*RS345pe - 4./27.*pow(m33,2)*m55
         *L4r*RSp561e*RS356pe - 4./27.*pow(m33,2)*m55*L4r*RSp461e*
         RS346pe - 4./27.*pow(m33,2)*m55*L4r*RSp451e*RS345pe - 8./9.*
         pow(m33,2)*m44*L7r*RSp563e - 8./3.*pow(m33,2)*m44*L7r*RSp561e
          - 8./3.*pow(m33,2)*m44*L6r*RSp45613e - 8./9.*pow(m33,2)*m44*
         L6r*RSp45633e - 4./9.*pow(m33,2)*m44*L5r*RSp563e*RS356pe - 4./
         27.*pow(m33,2)*m44*L5r*RSp561e*RS356pe + 4./9.*pow(m33,2)*m44*
         L4r*RSp45613e + 4./3.*pow(m33,2)*m44*L4r*RSp45633e - 4./9.*
         pow(m33,2)*m44*L4r*RSp563e*RS356pe );

      M13p6x23LV +=  + BbV(m33,mp2,L) * (  - 4./9.*pow(m33,2)*m44*L4r*
         RSp463e*RS346pe - 4./9.*pow(m33,2)*m44*L4r*RSp453e*RS345pe - 4.
         /27.*pow(m33,2)*m44*L4r*RSp561e*RS356pe - 4./27.*pow(m33,2)*
         m44*L4r*RSp461e*RS346pe - 4./27.*pow(m33,2)*m44*L4r*RSp451e*
         RS345pe - 8./3.*pow(m33,3)*L8r*RSp45613e - 8./9.*pow(m33,3)*
         L8r*RSp45633e + 4./9.*pow(m33,3)*L5r*RSp45613e + 4./3.*pow(
         m33,3)*L5r*RSp45633e - 16./27.*m13*pow(m66,2)*L8r*RSp453e*
         RS345pe + 16./27.*m13*pow(m66,2)*L8r*RSp451e*RS345pe - 16./27.
         *m13*pow(m66,2)*L7r*RSp453e*RS345pe + 16./27.*m13*pow(m66,2)*
         L7r*RSp451e*RS345pe - 16./27.*m13*pow(m66,2)*L6r*RSp453e*
         RS345pe + 16./27.*m13*pow(m66,2)*L6r*RSp451e*RS345pe - 16./27.
         *m13*m55*m66*L7r*RSp463e*RS345pe - 16./27.*m13*m55*m66*L7r*
         RSp453e*RS346pe + 16./27.*m13*m55*m66*L7r*RSp461e*RS345pe + 16.
         /27.*m13*m55*m66*L7r*RSp451e*RS346pe - 16./27.*m13*m55*m66*L6r
         *RSp463e*RS346pe - 16./27.*m13*m55*m66*L6r*RSp453e*RS345pe + 
         16./27.*m13*m55*m66*L6r*RSp461e*RS346pe );

      M13p6x23LV +=  + BbV(m33,mp2,L) * ( 16./27.*m13*m55*m66*L6r*
         RSp451e*RS345pe - 16./27.*m13*pow(m55,2)*L8r*RSp463e*RS346pe
          + 16./27.*m13*pow(m55,2)*L8r*RSp461e*RS346pe - 16./27.*m13*
         pow(m55,2)*L7r*RSp463e*RS346pe + 16./27.*m13*pow(m55,2)*L7r*
         RSp461e*RS346pe - 16./27.*m13*pow(m55,2)*L6r*RSp463e*RS346pe
          + 16./27.*m13*pow(m55,2)*L6r*RSp461e*RS346pe - 16./27.*m13*
         m44*m66*L7r*RSp563e*RS345pe - 16./27.*m13*m44*m66*L7r*RSp453e*
         RS356pe + 16./27.*m13*m44*m66*L7r*RSp561e*RS345pe + 16./27.*
         m13*m44*m66*L7r*RSp451e*RS356pe - 16./27.*m13*m44*m66*L6r*
         RSp563e*RS356pe - 16./27.*m13*m44*m66*L6r*RSp453e*RS345pe + 16.
         /27.*m13*m44*m66*L6r*RSp561e*RS356pe + 16./27.*m13*m44*m66*L6r
         *RSp451e*RS345pe - 16./27.*m13*m44*m55*L7r*RSp563e*RS346pe - 
         16./27.*m13*m44*m55*L7r*RSp463e*RS356pe + 16./27.*m13*m44*m55*
         L7r*RSp561e*RS346pe + 16./27.*m13*m44*m55*L7r*RSp461e*RS356pe
          - 16./27.*m13*m44*m55*L6r*RSp563e*RS356pe - 16./27.*m13*m44*
         m55*L6r*RSp463e*RS346pe );

      M13p6x23LV +=  + BbV(m33,mp2,L) * ( 16./27.*m13*m44*m55*L6r*
         RSp561e*RS356pe + 16./27.*m13*m44*m55*L6r*RSp461e*RS346pe - 16.
         /27.*m13*pow(m44,2)*L8r*RSp563e*RS356pe + 16./27.*m13*pow(
         m44,2)*L8r*RSp561e*RS356pe - 16./27.*m13*pow(m44,2)*L7r*
         RSp563e*RS356pe + 16./27.*m13*pow(m44,2)*L7r*RSp561e*RS356pe
          - 16./27.*m13*pow(m44,2)*L6r*RSp563e*RS356pe + 16./27.*m13*
         pow(m44,2)*L6r*RSp561e*RS356pe + 16./9.*m13*m33*m66*L7r*
         RSp453e - 16./9.*m13*m33*m66*L7r*RSp451e - 16./9.*m13*m33*m66*
         L6r*RSp45613e + 16./9.*m13*m33*m66*L6r*RSp45633e + 8./27.*m13*
         m33*m66*L5r*RSp453e*RS345pe - 8./27.*m13*m33*m66*L5r*RSp451e*
         RS345pe + 8./9.*m13*m33*m66*L4r*RSp45613e - 8./9.*m13*m33*m66*
         L4r*RSp45633e + 8./27.*m13*m33*m66*L4r*RSp563e*RS356pe + 8./27.
         *m13*m33*m66*L4r*RSp463e*RS346pe + 8./27.*m13*m33*m66*L4r*
         RSp453e*RS345pe - 8./27.*m13*m33*m66*L4r*RSp561e*RS356pe - 8./
         27.*m13*m33*m66*L4r*RSp461e*RS346pe - 8./27.*m13*m33*m66*L4r*
         RSp451e*RS345pe );

      M13p6x23LV +=  + BbV(m33,mp2,L) * ( 16./9.*m13*m33*m55*L7r*
         RSp463e - 16./9.*m13*m33*m55*L7r*RSp461e - 16./9.*m13*m33*m55*
         L6r*RSp45613e + 16./9.*m13*m33*m55*L6r*RSp45633e + 8./27.*m13*
         m33*m55*L5r*RSp463e*RS346pe - 8./27.*m13*m33*m55*L5r*RSp461e*
         RS346pe + 8./9.*m13*m33*m55*L4r*RSp45613e - 8./9.*m13*m33*m55*
         L4r*RSp45633e + 8./27.*m13*m33*m55*L4r*RSp563e*RS356pe + 8./27.
         *m13*m33*m55*L4r*RSp463e*RS346pe + 8./27.*m13*m33*m55*L4r*
         RSp453e*RS345pe - 8./27.*m13*m33*m55*L4r*RSp561e*RS356pe - 8./
         27.*m13*m33*m55*L4r*RSp461e*RS346pe - 8./27.*m13*m33*m55*L4r*
         RSp451e*RS345pe + 16./9.*m13*m33*m44*L7r*RSp563e - 16./9.*m13*
         m33*m44*L7r*RSp561e - 16./9.*m13*m33*m44*L6r*RSp45613e + 16./9.
         *m13*m33*m44*L6r*RSp45633e + 8./27.*m13*m33*m44*L5r*RSp563e*
         RS356pe - 8./27.*m13*m33*m44*L5r*RSp561e*RS356pe + 8./9.*m13*
         m33*m44*L4r*RSp45613e - 8./9.*m13*m33*m44*L4r*RSp45633e + 8./
         27.*m13*m33*m44*L4r*RSp563e*RS356pe + 8./27.*m13*m33*m44*L4r*
         RSp463e*RS346pe );

      M13p6x23LV +=  + BbV(m33,mp2,L) * ( 8./27.*m13*m33*m44*L4r*
         RSp453e*RS345pe - 8./27.*m13*m33*m44*L4r*RSp561e*RS356pe - 8./
         27.*m13*m33*m44*L4r*RSp461e*RS346pe - 8./27.*m13*m33*m44*L4r*
         RSp451e*RS345pe - 16./9.*m13*pow(m33,2)*L8r*RSp45613e + 16./9.
         *m13*pow(m33,2)*L8r*RSp45633e + 8./9.*m13*pow(m33,2)*L5r*
         RSp45613e - 8./9.*m13*pow(m33,2)*L5r*RSp45633e + 8./27.*m11*
         pow(m66,2)*L8r*RSp453e*RS345pe + 8./27.*m11*pow(m66,2)*L8r*
         RSp451e*RS345pe + 8./27.*m11*pow(m66,2)*L7r*RSp453e*RS345pe + 
         8./27.*m11*pow(m66,2)*L7r*RSp451e*RS345pe + 8./27.*m11*pow(
         m66,2)*L6r*RSp453e*RS345pe + 8./27.*m11*pow(m66,2)*L6r*RSp451e
         *RS345pe + 8./27.*m11*m55*m66*L7r*RSp463e*RS345pe + 8./27.*m11
         *m55*m66*L7r*RSp453e*RS346pe + 8./27.*m11*m55*m66*L7r*RSp461e*
         RS345pe + 8./27.*m11*m55*m66*L7r*RSp451e*RS346pe + 8./27.*m11*
         m55*m66*L6r*RSp463e*RS346pe + 8./27.*m11*m55*m66*L6r*RSp453e*
         RS345pe + 8./27.*m11*m55*m66*L6r*RSp461e*RS346pe + 8./27.*m11*
         m55*m66*L6r*RSp451e*RS345pe );

      M13p6x23LV +=  + BbV(m33,mp2,L) * ( 8./27.*m11*pow(m55,2)*L8r*
         RSp463e*RS346pe + 8./27.*m11*pow(m55,2)*L8r*RSp461e*RS346pe + 
         8./27.*m11*pow(m55,2)*L7r*RSp463e*RS346pe + 8./27.*m11*pow(
         m55,2)*L7r*RSp461e*RS346pe + 8./27.*m11*pow(m55,2)*L6r*RSp463e
         *RS346pe + 8./27.*m11*pow(m55,2)*L6r*RSp461e*RS346pe + 8./27.*
         m11*m44*m66*L7r*RSp563e*RS345pe + 8./27.*m11*m44*m66*L7r*
         RSp453e*RS356pe + 8./27.*m11*m44*m66*L7r*RSp561e*RS345pe + 8./
         27.*m11*m44*m66*L7r*RSp451e*RS356pe + 8./27.*m11*m44*m66*L6r*
         RSp563e*RS356pe + 8./27.*m11*m44*m66*L6r*RSp453e*RS345pe + 8./
         27.*m11*m44*m66*L6r*RSp561e*RS356pe + 8./27.*m11*m44*m66*L6r*
         RSp451e*RS345pe + 8./27.*m11*m44*m55*L7r*RSp563e*RS346pe + 8./
         27.*m11*m44*m55*L7r*RSp463e*RS356pe + 8./27.*m11*m44*m55*L7r*
         RSp561e*RS346pe + 8./27.*m11*m44*m55*L7r*RSp461e*RS356pe + 8./
         27.*m11*m44*m55*L6r*RSp563e*RS356pe + 8./27.*m11*m44*m55*L6r*
         RSp463e*RS346pe + 8./27.*m11*m44*m55*L6r*RSp561e*RS356pe + 8./
         27.*m11*m44*m55*L6r*RSp461e*RS346pe );

      M13p6x23LV +=  + BbV(m33,mp2,L) * ( 8./27.*m11*pow(m44,2)*L8r*
         RSp563e*RS356pe + 8./27.*m11*pow(m44,2)*L8r*RSp561e*RS356pe + 
         8./27.*m11*pow(m44,2)*L7r*RSp563e*RS356pe + 8./27.*m11*pow(
         m44,2)*L7r*RSp561e*RS356pe + 8./27.*m11*pow(m44,2)*L6r*RSp563e
         *RS356pe + 8./27.*m11*pow(m44,2)*L6r*RSp561e*RS356pe - 8./9.*
         m11*m33*m66*L7r*RSp453e - 8./9.*m11*m33*m66*L7r*RSp451e - 8./9.
         *m11*m33*m66*L6r*RSp45613e - 8./9.*m11*m33*m66*L6r*RSp45633e
          - 4./27.*m11*m33*m66*L5r*RSp453e*RS345pe - 4./27.*m11*m33*m66
         *L5r*RSp451e*RS345pe + 4./9.*m11*m33*m66*L4r*RSp45613e + 4./9.
         *m11*m33*m66*L4r*RSp45633e - 4./27.*m11*m33*m66*L4r*RSp563e*
         RS356pe - 4./27.*m11*m33*m66*L4r*RSp463e*RS346pe - 4./27.*m11*
         m33*m66*L4r*RSp453e*RS345pe - 4./27.*m11*m33*m66*L4r*RSp561e*
         RS356pe - 4./27.*m11*m33*m66*L4r*RSp461e*RS346pe - 4./27.*m11*
         m33*m66*L4r*RSp451e*RS345pe - 8./9.*m11*m33*m55*L7r*RSp463e - 
         8./9.*m11*m33*m55*L7r*RSp461e - 8./9.*m11*m33*m55*L6r*
         RSp45613e );

      M13p6x23LV +=  + BbV(m33,mp2,L) * (  - 8./9.*m11*m33*m55*L6r*
         RSp45633e - 4./27.*m11*m33*m55*L5r*RSp463e*RS346pe - 4./27.*
         m11*m33*m55*L5r*RSp461e*RS346pe + 4./9.*m11*m33*m55*L4r*
         RSp45613e + 4./9.*m11*m33*m55*L4r*RSp45633e - 4./27.*m11*m33*
         m55*L4r*RSp563e*RS356pe - 4./27.*m11*m33*m55*L4r*RSp463e*
         RS346pe - 4./27.*m11*m33*m55*L4r*RSp453e*RS345pe - 4./27.*m11*
         m33*m55*L4r*RSp561e*RS356pe - 4./27.*m11*m33*m55*L4r*RSp461e*
         RS346pe - 4./27.*m11*m33*m55*L4r*RSp451e*RS345pe - 8./9.*m11*
         m33*m44*L7r*RSp563e - 8./9.*m11*m33*m44*L7r*RSp561e - 8./9.*
         m11*m33*m44*L6r*RSp45613e - 8./9.*m11*m33*m44*L6r*RSp45633e - 
         4./27.*m11*m33*m44*L5r*RSp563e*RS356pe - 4./27.*m11*m33*m44*
         L5r*RSp561e*RS356pe + 4./9.*m11*m33*m44*L4r*RSp45613e + 4./9.*
         m11*m33*m44*L4r*RSp45633e - 4./27.*m11*m33*m44*L4r*RSp563e*
         RS356pe - 4./27.*m11*m33*m44*L4r*RSp463e*RS346pe - 4./27.*m11*
         m33*m44*L4r*RSp453e*RS345pe - 4./27.*m11*m33*m44*L4r*RSp561e*
         RS356pe );

      M13p6x23LV +=  + BbV(m33,mp2,L) * (  - 4./27.*m11*m33*m44*L4r*
         RSp461e*RS346pe - 4./27.*m11*m33*m44*L4r*RSp451e*RS345pe - 8./
         9.*m11*pow(m33,2)*L8r*RSp45613e - 8./9.*m11*pow(m33,2)*L8r*
         RSp45633e + 4./9.*m11*pow(m33,2)*L5r*RSp45613e + 4./9.*m11*
         pow(m33,2)*L5r*RSp45633e );

      M13p6x23LV +=  + BbV(m33,me2,L) * ( 8./27.*m33*m66*me2*L5r*
         RSe453p*RS345pe - 8./27.*m33*m66*me2*L5r*RSe451p*RS345pe + 8./
         9.*m33*m66*me2*L4r*RSe45613p - 8./9.*m33*m66*me2*L4r*RSe45633p
          + 8./27.*m33*m66*me2*L4r*RSe563p*RS356pe + 8./27.*m33*m66*me2
         *L4r*RSe463p*RS346pe + 8./27.*m33*m66*me2*L4r*RSe453p*RS345pe
          - 8./27.*m33*m66*me2*L4r*RSe561p*RS356pe - 8./27.*m33*m66*me2
         *L4r*RSe461p*RS346pe - 8./27.*m33*m66*me2*L4r*RSe451p*RS345pe
          + 8./27.*m33*pow(m66,2)*L8r*RSe453p*RS345pe + 8./9.*m33*pow(
         m66,2)*L8r*RSe451p*RS345pe + 8./27.*m33*pow(m66,2)*L7r*RSe453p
         *RS345pe + 8./9.*m33*pow(m66,2)*L7r*RSe451p*RS345pe + 8./27.*
         m33*pow(m66,2)*L6r*RSe453p*RS345pe + 8./9.*m33*pow(m66,2)*L6r*
         RSe451p*RS345pe + 8./27.*m33*m55*me2*L5r*RSe463p*RS346pe - 8./
         27.*m33*m55*me2*L5r*RSe461p*RS346pe + 8./9.*m33*m55*me2*L4r*
         RSe45613p - 8./9.*m33*m55*me2*L4r*RSe45633p + 8./27.*m33*m55*
         me2*L4r*RSe563p*RS356pe + 8./27.*m33*m55*me2*L4r*RSe463p*
         RS346pe );

      M13p6x23LV +=  + BbV(m33,me2,L) * ( 8./27.*m33*m55*me2*L4r*
         RSe453p*RS345pe - 8./27.*m33*m55*me2*L4r*RSe561p*RS356pe - 8./
         27.*m33*m55*me2*L4r*RSe461p*RS346pe - 8./27.*m33*m55*me2*L4r*
         RSe451p*RS345pe + 8./27.*m33*m55*m66*L7r*RSe463p*RS345pe + 8./
         27.*m33*m55*m66*L7r*RSe453p*RS346pe + 8./9.*m33*m55*m66*L7r*
         RSe461p*RS345pe + 8./9.*m33*m55*m66*L7r*RSe451p*RS346pe + 8./
         27.*m33*m55*m66*L6r*RSe463p*RS346pe + 8./27.*m33*m55*m66*L6r*
         RSe453p*RS345pe + 8./9.*m33*m55*m66*L6r*RSe461p*RS346pe + 8./9.
         *m33*m55*m66*L6r*RSe451p*RS345pe + 8./27.*m33*pow(m55,2)*L8r*
         RSe463p*RS346pe + 8./9.*m33*pow(m55,2)*L8r*RSe461p*RS346pe + 8.
         /27.*m33*pow(m55,2)*L7r*RSe463p*RS346pe + 8./9.*m33*pow(m55,2)
         *L7r*RSe461p*RS346pe + 8./27.*m33*pow(m55,2)*L6r*RSe463p*
         RS346pe + 8./9.*m33*pow(m55,2)*L6r*RSe461p*RS346pe + 8./27.*
         m33*m44*me2*L5r*RSe563p*RS356pe - 8./27.*m33*m44*me2*L5r*
         RSe561p*RS356pe + 8./9.*m33*m44*me2*L4r*RSe45613p - 8./9.*m33*
         m44*me2*L4r*RSe45633p );

      M13p6x23LV +=  + BbV(m33,me2,L) * ( 8./27.*m33*m44*me2*L4r*
         RSe563p*RS356pe + 8./27.*m33*m44*me2*L4r*RSe463p*RS346pe + 8./
         27.*m33*m44*me2*L4r*RSe453p*RS345pe - 8./27.*m33*m44*me2*L4r*
         RSe561p*RS356pe - 8./27.*m33*m44*me2*L4r*RSe461p*RS346pe - 8./
         27.*m33*m44*me2*L4r*RSe451p*RS345pe + 8./27.*m33*m44*m66*L7r*
         RSe563p*RS345pe + 8./27.*m33*m44*m66*L7r*RSe453p*RS356pe + 8./
         9.*m33*m44*m66*L7r*RSe561p*RS345pe + 8./9.*m33*m44*m66*L7r*
         RSe451p*RS356pe + 8./27.*m33*m44*m66*L6r*RSe563p*RS356pe + 8./
         27.*m33*m44*m66*L6r*RSe453p*RS345pe + 8./9.*m33*m44*m66*L6r*
         RSe561p*RS356pe + 8./9.*m33*m44*m66*L6r*RSe451p*RS345pe + 8./
         27.*m33*m44*m55*L7r*RSe563p*RS346pe + 8./27.*m33*m44*m55*L7r*
         RSe463p*RS356pe + 8./9.*m33*m44*m55*L7r*RSe561p*RS346pe + 8./9.
         *m33*m44*m55*L7r*RSe461p*RS356pe + 8./27.*m33*m44*m55*L6r*
         RSe563p*RS356pe + 8./27.*m33*m44*m55*L6r*RSe463p*RS346pe + 8./
         9.*m33*m44*m55*L6r*RSe561p*RS356pe + 8./9.*m33*m44*m55*L6r*
         RSe461p*RS346pe );

      M13p6x23LV +=  + BbV(m33,me2,L) * ( 8./27.*m33*pow(m44,2)*L8r*
         RSe563p*RS356pe + 8./9.*m33*pow(m44,2)*L8r*RSe561p*RS356pe + 8.
         /27.*m33*pow(m44,2)*L7r*RSe563p*RS356pe + 8./9.*m33*pow(m44,2)
         *L7r*RSe561p*RS356pe + 8./27.*m33*pow(m44,2)*L6r*RSe563p*
         RS356pe + 8./9.*m33*pow(m44,2)*L6r*RSe561p*RS356pe + 8./9.*
         pow(m33,2)*me2*L5r*RSe45613p - 8./9.*pow(m33,2)*me2*L5r*
         RSe45633p - 8./9.*pow(m33,2)*m66*L7r*RSe453p - 8./3.*pow(
         m33,2)*m66*L7r*RSe451p - 8./3.*pow(m33,2)*m66*L6r*RSe45613p - 
         8./9.*pow(m33,2)*m66*L6r*RSe45633p - 4./9.*pow(m33,2)*m66*L5r*
         RSe453p*RS345pe - 4./27.*pow(m33,2)*m66*L5r*RSe451p*RS345pe + 
         4./9.*pow(m33,2)*m66*L4r*RSe45613p + 4./3.*pow(m33,2)*m66*L4r*
         RSe45633p - 4./9.*pow(m33,2)*m66*L4r*RSe563p*RS356pe - 4./9.*
         pow(m33,2)*m66*L4r*RSe463p*RS346pe - 4./9.*pow(m33,2)*m66*L4r*
         RSe453p*RS345pe - 4./27.*pow(m33,2)*m66*L4r*RSe561p*RS356pe - 
         4./27.*pow(m33,2)*m66*L4r*RSe461p*RS346pe - 4./27.*pow(m33,2)*
         m66*L4r*RSe451p*RS345pe );

      M13p6x23LV +=  + BbV(m33,me2,L) * (  - 8./9.*pow(m33,2)*m55*L7r*
         RSe463p - 8./3.*pow(m33,2)*m55*L7r*RSe461p - 8./3.*pow(m33,2)*
         m55*L6r*RSe45613p - 8./9.*pow(m33,2)*m55*L6r*RSe45633p - 4./9.
         *pow(m33,2)*m55*L5r*RSe463p*RS346pe - 4./27.*pow(m33,2)*m55*
         L5r*RSe461p*RS346pe + 4./9.*pow(m33,2)*m55*L4r*RSe45613p + 4./
         3.*pow(m33,2)*m55*L4r*RSe45633p - 4./9.*pow(m33,2)*m55*L4r*
         RSe563p*RS356pe - 4./9.*pow(m33,2)*m55*L4r*RSe463p*RS346pe - 4.
         /9.*pow(m33,2)*m55*L4r*RSe453p*RS345pe - 4./27.*pow(m33,2)*m55
         *L4r*RSe561p*RS356pe - 4./27.*pow(m33,2)*m55*L4r*RSe461p*
         RS346pe - 4./27.*pow(m33,2)*m55*L4r*RSe451p*RS345pe - 8./9.*
         pow(m33,2)*m44*L7r*RSe563p - 8./3.*pow(m33,2)*m44*L7r*RSe561p
          - 8./3.*pow(m33,2)*m44*L6r*RSe45613p - 8./9.*pow(m33,2)*m44*
         L6r*RSe45633p - 4./9.*pow(m33,2)*m44*L5r*RSe563p*RS356pe - 4./
         27.*pow(m33,2)*m44*L5r*RSe561p*RS356pe + 4./9.*pow(m33,2)*m44*
         L4r*RSe45613p + 4./3.*pow(m33,2)*m44*L4r*RSe45633p - 4./9.*
         pow(m33,2)*m44*L4r*RSe563p*RS356pe );

      M13p6x23LV +=  + BbV(m33,me2,L) * (  - 4./9.*pow(m33,2)*m44*L4r*
         RSe463p*RS346pe - 4./9.*pow(m33,2)*m44*L4r*RSe453p*RS345pe - 4.
         /27.*pow(m33,2)*m44*L4r*RSe561p*RS356pe - 4./27.*pow(m33,2)*
         m44*L4r*RSe461p*RS346pe - 4./27.*pow(m33,2)*m44*L4r*RSe451p*
         RS345pe - 8./3.*pow(m33,3)*L8r*RSe45613p - 8./9.*pow(m33,3)*
         L8r*RSe45633p + 4./9.*pow(m33,3)*L5r*RSe45613p + 4./3.*pow(
         m33,3)*L5r*RSe45633p - 16./27.*m13*pow(m66,2)*L8r*RSe453p*
         RS345pe + 16./27.*m13*pow(m66,2)*L8r*RSe451p*RS345pe - 16./27.
         *m13*pow(m66,2)*L7r*RSe453p*RS345pe + 16./27.*m13*pow(m66,2)*
         L7r*RSe451p*RS345pe - 16./27.*m13*pow(m66,2)*L6r*RSe453p*
         RS345pe + 16./27.*m13*pow(m66,2)*L6r*RSe451p*RS345pe - 16./27.
         *m13*m55*m66*L7r*RSe463p*RS345pe - 16./27.*m13*m55*m66*L7r*
         RSe453p*RS346pe + 16./27.*m13*m55*m66*L7r*RSe461p*RS345pe + 16.
         /27.*m13*m55*m66*L7r*RSe451p*RS346pe - 16./27.*m13*m55*m66*L6r
         *RSe463p*RS346pe - 16./27.*m13*m55*m66*L6r*RSe453p*RS345pe + 
         16./27.*m13*m55*m66*L6r*RSe461p*RS346pe );

      M13p6x23LV +=  + BbV(m33,me2,L) * ( 16./27.*m13*m55*m66*L6r*
         RSe451p*RS345pe - 16./27.*m13*pow(m55,2)*L8r*RSe463p*RS346pe
          + 16./27.*m13*pow(m55,2)*L8r*RSe461p*RS346pe - 16./27.*m13*
         pow(m55,2)*L7r*RSe463p*RS346pe + 16./27.*m13*pow(m55,2)*L7r*
         RSe461p*RS346pe - 16./27.*m13*pow(m55,2)*L6r*RSe463p*RS346pe
          + 16./27.*m13*pow(m55,2)*L6r*RSe461p*RS346pe - 16./27.*m13*
         m44*m66*L7r*RSe563p*RS345pe - 16./27.*m13*m44*m66*L7r*RSe453p*
         RS356pe + 16./27.*m13*m44*m66*L7r*RSe561p*RS345pe + 16./27.*
         m13*m44*m66*L7r*RSe451p*RS356pe - 16./27.*m13*m44*m66*L6r*
         RSe563p*RS356pe - 16./27.*m13*m44*m66*L6r*RSe453p*RS345pe + 16.
         /27.*m13*m44*m66*L6r*RSe561p*RS356pe + 16./27.*m13*m44*m66*L6r
         *RSe451p*RS345pe - 16./27.*m13*m44*m55*L7r*RSe563p*RS346pe - 
         16./27.*m13*m44*m55*L7r*RSe463p*RS356pe + 16./27.*m13*m44*m55*
         L7r*RSe561p*RS346pe + 16./27.*m13*m44*m55*L7r*RSe461p*RS356pe
          - 16./27.*m13*m44*m55*L6r*RSe563p*RS356pe - 16./27.*m13*m44*
         m55*L6r*RSe463p*RS346pe );

      M13p6x23LV +=  + BbV(m33,me2,L) * ( 16./27.*m13*m44*m55*L6r*
         RSe561p*RS356pe + 16./27.*m13*m44*m55*L6r*RSe461p*RS346pe - 16.
         /27.*m13*pow(m44,2)*L8r*RSe563p*RS356pe + 16./27.*m13*pow(
         m44,2)*L8r*RSe561p*RS356pe - 16./27.*m13*pow(m44,2)*L7r*
         RSe563p*RS356pe + 16./27.*m13*pow(m44,2)*L7r*RSe561p*RS356pe
          - 16./27.*m13*pow(m44,2)*L6r*RSe563p*RS356pe + 16./27.*m13*
         pow(m44,2)*L6r*RSe561p*RS356pe + 16./9.*m13*m33*m66*L7r*
         RSe453p - 16./9.*m13*m33*m66*L7r*RSe451p - 16./9.*m13*m33*m66*
         L6r*RSe45613p + 16./9.*m13*m33*m66*L6r*RSe45633p + 8./27.*m13*
         m33*m66*L5r*RSe453p*RS345pe - 8./27.*m13*m33*m66*L5r*RSe451p*
         RS345pe + 8./9.*m13*m33*m66*L4r*RSe45613p - 8./9.*m13*m33*m66*
         L4r*RSe45633p + 8./27.*m13*m33*m66*L4r*RSe563p*RS356pe + 8./27.
         *m13*m33*m66*L4r*RSe463p*RS346pe + 8./27.*m13*m33*m66*L4r*
         RSe453p*RS345pe - 8./27.*m13*m33*m66*L4r*RSe561p*RS356pe - 8./
         27.*m13*m33*m66*L4r*RSe461p*RS346pe - 8./27.*m13*m33*m66*L4r*
         RSe451p*RS345pe );

      M13p6x23LV +=  + BbV(m33,me2,L) * ( 16./9.*m13*m33*m55*L7r*
         RSe463p - 16./9.*m13*m33*m55*L7r*RSe461p - 16./9.*m13*m33*m55*
         L6r*RSe45613p + 16./9.*m13*m33*m55*L6r*RSe45633p + 8./27.*m13*
         m33*m55*L5r*RSe463p*RS346pe - 8./27.*m13*m33*m55*L5r*RSe461p*
         RS346pe + 8./9.*m13*m33*m55*L4r*RSe45613p - 8./9.*m13*m33*m55*
         L4r*RSe45633p + 8./27.*m13*m33*m55*L4r*RSe563p*RS356pe + 8./27.
         *m13*m33*m55*L4r*RSe463p*RS346pe + 8./27.*m13*m33*m55*L4r*
         RSe453p*RS345pe - 8./27.*m13*m33*m55*L4r*RSe561p*RS356pe - 8./
         27.*m13*m33*m55*L4r*RSe461p*RS346pe - 8./27.*m13*m33*m55*L4r*
         RSe451p*RS345pe + 16./9.*m13*m33*m44*L7r*RSe563p - 16./9.*m13*
         m33*m44*L7r*RSe561p - 16./9.*m13*m33*m44*L6r*RSe45613p + 16./9.
         *m13*m33*m44*L6r*RSe45633p + 8./27.*m13*m33*m44*L5r*RSe563p*
         RS356pe - 8./27.*m13*m33*m44*L5r*RSe561p*RS356pe + 8./9.*m13*
         m33*m44*L4r*RSe45613p - 8./9.*m13*m33*m44*L4r*RSe45633p + 8./
         27.*m13*m33*m44*L4r*RSe563p*RS356pe + 8./27.*m13*m33*m44*L4r*
         RSe463p*RS346pe );

      M13p6x23LV +=  + BbV(m33,me2,L) * ( 8./27.*m13*m33*m44*L4r*
         RSe453p*RS345pe - 8./27.*m13*m33*m44*L4r*RSe561p*RS356pe - 8./
         27.*m13*m33*m44*L4r*RSe461p*RS346pe - 8./27.*m13*m33*m44*L4r*
         RSe451p*RS345pe - 16./9.*m13*pow(m33,2)*L8r*RSe45613p + 16./9.
         *m13*pow(m33,2)*L8r*RSe45633p + 8./9.*m13*pow(m33,2)*L5r*
         RSe45613p - 8./9.*m13*pow(m33,2)*L5r*RSe45633p + 8./27.*m11*
         pow(m66,2)*L8r*RSe453p*RS345pe + 8./27.*m11*pow(m66,2)*L8r*
         RSe451p*RS345pe + 8./27.*m11*pow(m66,2)*L7r*RSe453p*RS345pe + 
         8./27.*m11*pow(m66,2)*L7r*RSe451p*RS345pe + 8./27.*m11*pow(
         m66,2)*L6r*RSe453p*RS345pe + 8./27.*m11*pow(m66,2)*L6r*RSe451p
         *RS345pe + 8./27.*m11*m55*m66*L7r*RSe463p*RS345pe + 8./27.*m11
         *m55*m66*L7r*RSe453p*RS346pe + 8./27.*m11*m55*m66*L7r*RSe461p*
         RS345pe + 8./27.*m11*m55*m66*L7r*RSe451p*RS346pe + 8./27.*m11*
         m55*m66*L6r*RSe463p*RS346pe + 8./27.*m11*m55*m66*L6r*RSe453p*
         RS345pe + 8./27.*m11*m55*m66*L6r*RSe461p*RS346pe + 8./27.*m11*
         m55*m66*L6r*RSe451p*RS345pe );

      M13p6x23LV +=  + BbV(m33,me2,L) * ( 8./27.*m11*pow(m55,2)*L8r*
         RSe463p*RS346pe + 8./27.*m11*pow(m55,2)*L8r*RSe461p*RS346pe + 
         8./27.*m11*pow(m55,2)*L7r*RSe463p*RS346pe + 8./27.*m11*pow(
         m55,2)*L7r*RSe461p*RS346pe + 8./27.*m11*pow(m55,2)*L6r*RSe463p
         *RS346pe + 8./27.*m11*pow(m55,2)*L6r*RSe461p*RS346pe + 8./27.*
         m11*m44*m66*L7r*RSe563p*RS345pe + 8./27.*m11*m44*m66*L7r*
         RSe453p*RS356pe + 8./27.*m11*m44*m66*L7r*RSe561p*RS345pe + 8./
         27.*m11*m44*m66*L7r*RSe451p*RS356pe + 8./27.*m11*m44*m66*L6r*
         RSe563p*RS356pe + 8./27.*m11*m44*m66*L6r*RSe453p*RS345pe + 8./
         27.*m11*m44*m66*L6r*RSe561p*RS356pe + 8./27.*m11*m44*m66*L6r*
         RSe451p*RS345pe + 8./27.*m11*m44*m55*L7r*RSe563p*RS346pe + 8./
         27.*m11*m44*m55*L7r*RSe463p*RS356pe + 8./27.*m11*m44*m55*L7r*
         RSe561p*RS346pe + 8./27.*m11*m44*m55*L7r*RSe461p*RS356pe + 8./
         27.*m11*m44*m55*L6r*RSe563p*RS356pe + 8./27.*m11*m44*m55*L6r*
         RSe463p*RS346pe + 8./27.*m11*m44*m55*L6r*RSe561p*RS356pe + 8./
         27.*m11*m44*m55*L6r*RSe461p*RS346pe );

      M13p6x23LV +=  + BbV(m33,me2,L) * ( 8./27.*m11*pow(m44,2)*L8r*
         RSe563p*RS356pe + 8./27.*m11*pow(m44,2)*L8r*RSe561p*RS356pe + 
         8./27.*m11*pow(m44,2)*L7r*RSe563p*RS356pe + 8./27.*m11*pow(
         m44,2)*L7r*RSe561p*RS356pe + 8./27.*m11*pow(m44,2)*L6r*RSe563p
         *RS356pe + 8./27.*m11*pow(m44,2)*L6r*RSe561p*RS356pe - 8./9.*
         m11*m33*m66*L7r*RSe453p - 8./9.*m11*m33*m66*L7r*RSe451p - 8./9.
         *m11*m33*m66*L6r*RSe45613p - 8./9.*m11*m33*m66*L6r*RSe45633p
          - 4./27.*m11*m33*m66*L5r*RSe453p*RS345pe - 4./27.*m11*m33*m66
         *L5r*RSe451p*RS345pe + 4./9.*m11*m33*m66*L4r*RSe45613p + 4./9.
         *m11*m33*m66*L4r*RSe45633p - 4./27.*m11*m33*m66*L4r*RSe563p*
         RS356pe - 4./27.*m11*m33*m66*L4r*RSe463p*RS346pe - 4./27.*m11*
         m33*m66*L4r*RSe453p*RS345pe - 4./27.*m11*m33*m66*L4r*RSe561p*
         RS356pe - 4./27.*m11*m33*m66*L4r*RSe461p*RS346pe - 4./27.*m11*
         m33*m66*L4r*RSe451p*RS345pe - 8./9.*m11*m33*m55*L7r*RSe463p - 
         8./9.*m11*m33*m55*L7r*RSe461p - 8./9.*m11*m33*m55*L6r*
         RSe45613p );

      M13p6x23LV +=  + BbV(m33,me2,L) * (  - 8./9.*m11*m33*m55*L6r*
         RSe45633p - 4./27.*m11*m33*m55*L5r*RSe463p*RS346pe - 4./27.*
         m11*m33*m55*L5r*RSe461p*RS346pe + 4./9.*m11*m33*m55*L4r*
         RSe45613p + 4./9.*m11*m33*m55*L4r*RSe45633p - 4./27.*m11*m33*
         m55*L4r*RSe563p*RS356pe - 4./27.*m11*m33*m55*L4r*RSe463p*
         RS346pe - 4./27.*m11*m33*m55*L4r*RSe453p*RS345pe - 4./27.*m11*
         m33*m55*L4r*RSe561p*RS356pe - 4./27.*m11*m33*m55*L4r*RSe461p*
         RS346pe - 4./27.*m11*m33*m55*L4r*RSe451p*RS345pe - 8./9.*m11*
         m33*m44*L7r*RSe563p - 8./9.*m11*m33*m44*L7r*RSe561p - 8./9.*
         m11*m33*m44*L6r*RSe45613p - 8./9.*m11*m33*m44*L6r*RSe45633p - 
         4./27.*m11*m33*m44*L5r*RSe563p*RS356pe - 4./27.*m11*m33*m44*
         L5r*RSe561p*RS356pe + 4./9.*m11*m33*m44*L4r*RSe45613p + 4./9.*
         m11*m33*m44*L4r*RSe45633p - 4./27.*m11*m33*m44*L4r*RSe563p*
         RS356pe - 4./27.*m11*m33*m44*L4r*RSe463p*RS346pe - 4./27.*m11*
         m33*m44*L4r*RSe453p*RS345pe - 4./27.*m11*m33*m44*L4r*RSe561p*
         RS356pe );

      M13p6x23LV +=  + BbV(m33,me2,L) * (  - 4./27.*m11*m33*m44*L4r*
         RSe461p*RS346pe - 4./27.*m11*m33*m44*L4r*RSe451p*RS345pe - 8./
         9.*m11*pow(m33,2)*L8r*RSe45613p - 8./9.*m11*pow(m33,2)*L8r*
         RSe45633p + 4./9.*m11*pow(m33,2)*L5r*RSe45613p + 4./9.*m11*
         pow(m33,2)*L5r*RSe45633p );

      M13p6x23LV +=  + BbV(m33,L) * ( 8./9.*m33*m66*L6r*RD3456ep - 2./9.
         *m33*m66*L4r*RD3456ep + 4./27.*m33*pow(m66,2)*L8r*pow(
         RS345pe,2) + 4./27.*m33*pow(m66,2)*L7r*pow(RS345pe,2) + 4./27.
         *m33*pow(m66,2)*L6r*pow(RS345pe,2) + 8./9.*m33*m55*L6r*
         RD3456ep - 2./9.*m33*m55*L4r*RD3456ep + 8./27.*m33*m55*m66*L7r
         *RS346pe*RS345pe + 4./27.*m33*m55*m66*L6r*pow(RS345pe,2) + 4./
         27.*m33*m55*m66*L6r*pow(RS346pe,2) + 4./27.*m33*pow(m55,2)*L8r
         *pow(RS346pe,2) + 4./27.*m33*pow(m55,2)*L7r*pow(RS346pe,2) + 4.
         /27.*m33*pow(m55,2)*L6r*pow(RS346pe,2) + 8./9.*m33*m44*L6r*
         RD3456ep - 2./9.*m33*m44*L4r*RD3456ep + 8./27.*m33*m44*m66*L7r
         *RS356pe*RS345pe + 4./27.*m33*m44*m66*L6r*pow(RS345pe,2) + 4./
         27.*m33*m44*m66*L6r*pow(RS356pe,2) + 8./27.*m33*m44*m55*L7r*
         RS356pe*RS346pe + 4./27.*m33*m44*m55*L6r*pow(RS346pe,2) + 4./
         27.*m33*m44*m55*L6r*pow(RS356pe,2) + 4./27.*m33*pow(m44,2)*L8r
         *pow(RS356pe,2) + 4./27.*m33*pow(m44,2)*L7r*pow(RS356pe,2) + 4.
         /27.*m33*pow(m44,2)*L6r*pow(RS356pe,2) );

      M13p6x23LV +=  + BbV(m33,L) * (  - 16./9.*pow(m33,2)*L8r*RD3456ep
          + 5./9.*pow(m33,2)*L5r*RD3456ep - 8./9.*pow(m33,2)*m66*L7r*
         RS345pe - 8./3.*pow(m33,2)*m66*L6r*RS34561pe - 8./9.*pow(
         m33,2)*m66*L6r*RC3456pe - 2./27.*pow(m33,2)*m66*L5r*pow(
         RS345pe,2) + 4./3.*pow(m33,2)*m66*L4r*RS34561pe + 4./9.*pow(
         m33,2)*m66*L4r*RC3456pe - 2./27.*pow(m33,2)*m66*L4r*pow(
         RS345pe,2) - 2./27.*pow(m33,2)*m66*L4r*pow(RS346pe,2) - 2./27.
         *pow(m33,2)*m66*L4r*pow(RS356pe,2) - 8./9.*pow(m33,2)*m55*L7r*
         RS346pe - 8./3.*pow(m33,2)*m55*L6r*RS34561pe - 8./9.*pow(
         m33,2)*m55*L6r*RC3456pe - 2./27.*pow(m33,2)*m55*L5r*pow(
         RS346pe,2) + 4./3.*pow(m33,2)*m55*L4r*RS34561pe + 4./9.*pow(
         m33,2)*m55*L4r*RC3456pe - 2./27.*pow(m33,2)*m55*L4r*pow(
         RS345pe,2) - 2./27.*pow(m33,2)*m55*L4r*pow(RS346pe,2) - 2./27.
         *pow(m33,2)*m55*L4r*pow(RS356pe,2) - 8./9.*pow(m33,2)*m44*L7r*
         RS356pe - 8./3.*pow(m33,2)*m44*L6r*RS34561pe - 8./9.*pow(
         m33,2)*m44*L6r*RC3456pe );

      M13p6x23LV +=  + BbV(m33,L) * (  - 2./27.*pow(m33,2)*m44*L5r*pow(
         RS356pe,2) + 4./3.*pow(m33,2)*m44*L4r*RS34561pe + 4./9.*pow(
         m33,2)*m44*L4r*RC3456pe - 2./27.*pow(m33,2)*m44*L4r*pow(
         RS345pe,2) - 2./27.*pow(m33,2)*m44*L4r*pow(RS346pe,2) - 2./27.
         *pow(m33,2)*m44*L4r*pow(RS356pe,2) - 8./3.*pow(m33,3)*L8r*
         RS34561pe - 8./9.*pow(m33,3)*L8r*RC3456pe + 4./3.*pow(m33,3)*
         L7r + 4./3.*pow(m33,3)*L5r*RS34561pe + 4./9.*pow(m33,3)*L5r*
         RC3456pe - 4./9.*m13*m66*L4r*RD3456ep - 8./27.*m13*pow(m66,2)*
         L8r*pow(RS345pe,2) - 8./27.*m13*pow(m66,2)*L7r*pow(RS345pe,2)
          - 8./27.*m13*pow(m66,2)*L6r*pow(RS345pe,2) - 4./9.*m13*m55*
         L4r*RD3456ep - 16./27.*m13*m55*m66*L7r*RS346pe*RS345pe - 8./27.
         *m13*m55*m66*L6r*pow(RS345pe,2) - 8./27.*m13*m55*m66*L6r*pow(
         RS346pe,2) - 8./27.*m13*pow(m55,2)*L8r*pow(RS346pe,2) - 8./27.
         *m13*pow(m55,2)*L7r*pow(RS346pe,2) - 8./27.*m13*pow(m55,2)*L6r
         *pow(RS346pe,2) - 4./9.*m13*m44*L4r*RD3456ep - 16./27.*m13*m44
         *m66*L7r*RS356pe*RS345pe );

      M13p6x23LV +=  + BbV(m33,L) * (  - 8./27.*m13*m44*m66*L6r*pow(
         RS345pe,2) - 8./27.*m13*m44*m66*L6r*pow(RS356pe,2) - 16./27.*
         m13*m44*m55*L7r*RS356pe*RS346pe - 8./27.*m13*m44*m55*L6r*pow(
         RS346pe,2) - 8./27.*m13*m44*m55*L6r*pow(RS356pe,2) - 8./27.*
         m13*pow(m44,2)*L8r*pow(RS356pe,2) - 8./27.*m13*pow(m44,2)*L7r*
         pow(RS356pe,2) - 8./27.*m13*pow(m44,2)*L6r*pow(RS356pe,2) + 2./
         3.*m13*m33*L5r*RD3456ep - 10./3.*m13*m33*L3r*RD3456ep - 4./3.*
         m13*m33*L0r*RD3456ep + 16./9.*m13*m33*m66*L7r*RS345pe - 16./9.
         *m13*m33*m66*L6r*RS34561pe + 16./9.*m13*m33*m66*L6r*RC3456pe
          + 4./27.*m13*m33*m66*L5r*pow(RS345pe,2) + 8./9.*m13*m33*m66*
         L4r*RS34561pe - 8./9.*m13*m33*m66*L4r*RC3456pe + 4./27.*m13*
         m33*m66*L4r*pow(RS345pe,2) + 4./27.*m13*m33*m66*L4r*pow(
         RS346pe,2) + 4./27.*m13*m33*m66*L4r*pow(RS356pe,2) + 16./9.*
         m13*m33*m55*L7r*RS346pe - 16./9.*m13*m33*m55*L6r*RS34561pe + 
         16./9.*m13*m33*m55*L6r*RC3456pe + 4./27.*m13*m33*m55*L5r*pow(
         RS346pe,2) );

      M13p6x23LV +=  + BbV(m33,L) * ( 8./9.*m13*m33*m55*L4r*RS34561pe
          - 8./9.*m13*m33*m55*L4r*RC3456pe + 4./27.*m13*m33*m55*L4r*
         pow(RS345pe,2) + 4./27.*m13*m33*m55*L4r*pow(RS346pe,2) + 4./27.
         *m13*m33*m55*L4r*pow(RS356pe,2) + 16./9.*m13*m33*m44*L7r*
         RS356pe - 16./9.*m13*m33*m44*L6r*RS34561pe + 16./9.*m13*m33*
         m44*L6r*RC3456pe + 4./27.*m13*m33*m44*L5r*pow(RS356pe,2) + 8./
         9.*m13*m33*m44*L4r*RS34561pe - 8./9.*m13*m33*m44*L4r*RC3456pe
          + 4./27.*m13*m33*m44*L4r*pow(RS345pe,2) + 4./27.*m13*m33*m44*
         L4r*pow(RS346pe,2) + 4./27.*m13*m33*m44*L4r*pow(RS356pe,2) - 
         16./9.*m13*pow(m33,2)*L8r*RS34561pe + 16./9.*m13*pow(m33,2)*
         L8r*RC3456pe - 8./3.*m13*pow(m33,2)*L7r + 8./9.*m13*pow(m33,2)
         *L5r*RS34561pe - 8./9.*m13*pow(m33,2)*L5r*RC3456pe + 2./9.*m11
         *m66*L4r*RD3456ep + 4./27.*m11*pow(m66,2)*L8r*pow(RS345pe,2)
          + 4./27.*m11*pow(m66,2)*L7r*pow(RS345pe,2) + 4./27.*m11*pow(
         m66,2)*L6r*pow(RS345pe,2) + 2./9.*m11*m55*L4r*RD3456ep + 8./27.
         *m11*m55*m66*L7r*RS346pe*RS345pe );

      M13p6x23LV +=  + BbV(m33,L) * ( 4./27.*m11*m55*m66*L6r*pow(
         RS345pe,2) + 4./27.*m11*m55*m66*L6r*pow(RS346pe,2) + 4./27.*
         m11*pow(m55,2)*L8r*pow(RS346pe,2) + 4./27.*m11*pow(m55,2)*L7r*
         pow(RS346pe,2) + 4./27.*m11*pow(m55,2)*L6r*pow(RS346pe,2) + 2./
         9.*m11*m44*L4r*RD3456ep + 8./27.*m11*m44*m66*L7r*RS356pe*
         RS345pe + 4./27.*m11*m44*m66*L6r*pow(RS345pe,2) + 4./27.*m11*
         m44*m66*L6r*pow(RS356pe,2) + 8./27.*m11*m44*m55*L7r*RS356pe*
         RS346pe + 4./27.*m11*m44*m55*L6r*pow(RS346pe,2) + 4./27.*m11*
         m44*m55*L6r*pow(RS356pe,2) + 4./27.*m11*pow(m44,2)*L8r*pow(
         RS356pe,2) + 4./27.*m11*pow(m44,2)*L7r*pow(RS356pe,2) + 4./27.
         *m11*pow(m44,2)*L6r*pow(RS356pe,2) - 8./3.*m11*m33*L8r*
         RD3456ep + 8./9.*m11*m33*L5r*RD3456ep - 8./9.*m11*m33*m66*L7r*
         RS345pe - 8./9.*m11*m33*m66*L6r*RS34561pe - 8./9.*m11*m33*m66*
         L6r*RC3456pe - 2./27.*m11*m33*m66*L5r*pow(RS345pe,2) + 4./9.*
         m11*m33*m66*L4r*RS34561pe + 4./9.*m11*m33*m66*L4r*RC3456pe - 2.
         /27.*m11*m33*m66*L4r*pow(RS345pe,2) );

      M13p6x23LV +=  + BbV(m33,L) * (  - 2./27.*m11*m33*m66*L4r*pow(
         RS346pe,2) - 2./27.*m11*m33*m66*L4r*pow(RS356pe,2) - 8./9.*m11
         *m33*m55*L7r*RS346pe - 8./9.*m11*m33*m55*L6r*RS34561pe - 8./9.
         *m11*m33*m55*L6r*RC3456pe - 2./27.*m11*m33*m55*L5r*pow(
         RS346pe,2) + 4./9.*m11*m33*m55*L4r*RS34561pe + 4./9.*m11*m33*
         m55*L4r*RC3456pe - 2./27.*m11*m33*m55*L4r*pow(RS345pe,2) - 2./
         27.*m11*m33*m55*L4r*pow(RS346pe,2) - 2./27.*m11*m33*m55*L4r*
         pow(RS356pe,2) - 8./9.*m11*m33*m44*L7r*RS356pe - 8./9.*m11*m33
         *m44*L6r*RS34561pe - 8./9.*m11*m33*m44*L6r*RC3456pe - 2./27.*
         m11*m33*m44*L5r*pow(RS356pe,2) + 4./9.*m11*m33*m44*L4r*
         RS34561pe + 4./9.*m11*m33*m44*L4r*RC3456pe - 2./27.*m11*m33*
         m44*L4r*pow(RS345pe,2) - 2./27.*m11*m33*m44*L4r*pow(RS346pe,2)
          - 2./27.*m11*m33*m44*L4r*pow(RS356pe,2) - 8./9.*m11*pow(
         m33,2)*L8r*RS34561pe - 8./9.*m11*pow(m33,2)*L8r*RC3456pe + 4./
         3.*m11*pow(m33,2)*L7r + 4./9.*m11*pow(m33,2)*L5r*RS34561pe + 4.
         /9.*m11*pow(m33,2)*L5r*RC3456pe );

      M13p6x23LV +=  + BbV(m33,L) * ( 2./9.*m11*m13*L5r*RD3456ep - 1./9.
         *pow(m11,2)*L5r*RD3456ep );

      M13p6x23LV +=  + BbV(m34,L) * ( 2./3.*pow(m44,2)*m66*L6r + 2./3.*
         pow(m44,2)*m55*L6r + 1./3.*pow(m44,3)*L8r + 2./3.*pow(m44,3)*
         L6r - 4./3.*m34*m44*m66*L6r - 2./3.*m34*m44*m66*L4r - 4./3.*
         m34*m44*m55*L6r - 2./3.*m34*m44*m55*L4r - 2./3.*m34*pow(m44,2)
         *L8r - 4./3.*m34*pow(m44,2)*L6r - 1./3.*m34*pow(m44,2)*L5r - 2.
         /3.*m34*pow(m44,2)*L4r + 4./3.*pow(m34,2)*m66*L4r + 4./3.*pow(
         m34,2)*m55*L4r + 2./3.*pow(m34,2)*m44*L5r + 4./3.*pow(m34,2)*
         m44*L4r + 2*m33*m44*m66*L6r + 2*m33*m44*m55*L6r + 4./3.*m33*
         pow(m44,2)*L8r + 2*m33*pow(m44,2)*L6r - 4./3.*m33*m34*m66*L6r
          - 4./3.*m33*m34*m66*L4r - 4./3.*m33*m34*m55*L6r - 4./3.*m33*
         m34*m55*L4r - 4./3.*m33*m34*m44*L8r - 4./3.*m33*m34*m44*L6r - 
         m33*m34*m44*L5r - 4./3.*m33*m34*m44*L4r + 2./3.*m33*pow(m34,2)
         *L5r + 4./3.*pow(m33,2)*m66*L6r + 4./3.*pow(m33,2)*m55*L6r + 5.
         /3.*pow(m33,2)*m44*L8r + 4./3.*pow(m33,2)*m44*L6r - 2./3.*pow(
         m33,2)*m34*L8r - 2./3.*pow(m33,2)*m34*L5r + 2./3.*pow(m33,3)*
         L8r );

      M13p6x23LV +=  + BbV(m34,L) * (  - 4./3.*m13*m44*m66*L6r - 4./3.*
         m13*m44*m55*L6r - 2./3.*m13*pow(m44,2)*L8r - 4./3.*m13*pow(
         m44,2)*L6r + 4./3.*m13*m34*m66*L4r + 4./3.*m13*m34*m55*L4r + 2.
         /3.*m13*m34*m44*L5r + 4./3.*m13*m34*m44*L4r - 4./3.*m13*m33*
         m66*L6r - 4./3.*m13*m33*m55*L6r - 4./3.*m13*m33*m44*L8r - 4./3.
         *m13*m33*m44*L6r + 2./3.*m13*m33*m34*L5r - 2./3.*m13*pow(
         m33,2)*L8r + 2./3.*m11*m44*m66*L6r + 2./3.*m11*m44*m55*L6r + 1.
         /3.*m11*pow(m44,2)*L8r + 2./3.*m11*pow(m44,2)*L6r - 2./3.*m11*
         m34*m66*L4r - 2./3.*m11*m34*m55*L4r - 1./3.*m11*m34*m44*L5r - 
         2./3.*m11*m34*m44*L4r + 2./3.*m11*m33*m66*L6r + 2./3.*m11*m33*
         m55*L6r + 2./3.*m11*m33*m44*L8r + 2./3.*m11*m33*m44*L6r - 1./3.
         *m11*m33*m34*L5r + 1./3.*m11*pow(m33,2)*L8r );

      M13p6x23LV +=  + BbV(m35,L) * ( 2./3.*pow(m55,2)*m66*L6r + 1./3.*
         pow(m55,3)*L8r + 2./3.*pow(m55,3)*L6r + 2./3.*m44*pow(m55,2)*
         L6r - 4./3.*m35*m55*m66*L6r - 2./3.*m35*m55*m66*L4r - 2./3.*
         m35*pow(m55,2)*L8r - 4./3.*m35*pow(m55,2)*L6r - 1./3.*m35*pow(
         m55,2)*L5r - 2./3.*m35*pow(m55,2)*L4r - 4./3.*m35*m44*m55*L6r
          - 2./3.*m35*m44*m55*L4r + 4./3.*pow(m35,2)*m66*L4r + 2./3.*
         pow(m35,2)*m55*L5r + 4./3.*pow(m35,2)*m55*L4r + 4./3.*pow(
         m35,2)*m44*L4r + 2*m33*m55*m66*L6r + 4./3.*m33*pow(m55,2)*L8r
          + 2*m33*pow(m55,2)*L6r + 2*m33*m44*m55*L6r - 4./3.*m33*m35*
         m66*L6r - 4./3.*m33*m35*m66*L4r - 4./3.*m33*m35*m55*L8r - 4./3.
         *m33*m35*m55*L6r - m33*m35*m55*L5r - 4./3.*m33*m35*m55*L4r - 4.
         /3.*m33*m35*m44*L6r - 4./3.*m33*m35*m44*L4r + 2./3.*m33*pow(
         m35,2)*L5r + 4./3.*pow(m33,2)*m66*L6r + 5./3.*pow(m33,2)*m55*
         L8r + 4./3.*pow(m33,2)*m55*L6r + 4./3.*pow(m33,2)*m44*L6r - 2./
         3.*pow(m33,2)*m35*L8r - 2./3.*pow(m33,2)*m35*L5r + 2./3.*pow(
         m33,3)*L8r );

      M13p6x23LV +=  + BbV(m35,L) * (  - 4./3.*m13*m55*m66*L6r - 2./3.*
         m13*pow(m55,2)*L8r - 4./3.*m13*pow(m55,2)*L6r - 4./3.*m13*m44*
         m55*L6r + 4./3.*m13*m35*m66*L4r + 2./3.*m13*m35*m55*L5r + 4./3.
         *m13*m35*m55*L4r + 4./3.*m13*m35*m44*L4r - 4./3.*m13*m33*m66*
         L6r - 4./3.*m13*m33*m55*L8r - 4./3.*m13*m33*m55*L6r - 4./3.*
         m13*m33*m44*L6r + 2./3.*m13*m33*m35*L5r - 2./3.*m13*pow(m33,2)
         *L8r + 2./3.*m11*m55*m66*L6r + 1./3.*m11*pow(m55,2)*L8r + 2./3.
         *m11*pow(m55,2)*L6r + 2./3.*m11*m44*m55*L6r - 2./3.*m11*m35*
         m66*L4r - 1./3.*m11*m35*m55*L5r - 2./3.*m11*m35*m55*L4r - 2./3.
         *m11*m35*m44*L4r + 2./3.*m11*m33*m66*L6r + 2./3.*m11*m33*m55*
         L8r + 2./3.*m11*m33*m55*L6r + 2./3.*m11*m33*m44*L6r - 1./3.*
         m11*m33*m35*L5r + 1./3.*m11*pow(m33,2)*L8r );

      M13p6x23LV +=  + BbV(m36,L) * ( 1./3.*pow(m66,3)*L8r + 2./3.*pow(
         m66,3)*L6r + 2./3.*m55*pow(m66,2)*L6r + 2./3.*m44*pow(m66,2)*
         L6r - 2./3.*m36*pow(m66,2)*L8r - 4./3.*m36*pow(m66,2)*L6r - 1./
         3.*m36*pow(m66,2)*L5r - 2./3.*m36*pow(m66,2)*L4r - 4./3.*m36*
         m55*m66*L6r - 2./3.*m36*m55*m66*L4r - 4./3.*m36*m44*m66*L6r - 
         2./3.*m36*m44*m66*L4r + 2./3.*pow(m36,2)*m66*L5r + 4./3.*pow(
         m36,2)*m66*L4r + 4./3.*pow(m36,2)*m55*L4r + 4./3.*pow(m36,2)*
         m44*L4r + 4./3.*m33*pow(m66,2)*L8r + 2*m33*pow(m66,2)*L6r + 2*
         m33*m55*m66*L6r + 2*m33*m44*m66*L6r - 4./3.*m33*m36*m66*L8r - 
         4./3.*m33*m36*m66*L6r - m33*m36*m66*L5r - 4./3.*m33*m36*m66*
         L4r - 4./3.*m33*m36*m55*L6r - 4./3.*m33*m36*m55*L4r - 4./3.*
         m33*m36*m44*L6r - 4./3.*m33*m36*m44*L4r + 2./3.*m33*pow(m36,2)
         *L5r + 5./3.*pow(m33,2)*m66*L8r + 4./3.*pow(m33,2)*m66*L6r + 4.
         /3.*pow(m33,2)*m55*L6r + 4./3.*pow(m33,2)*m44*L6r - 2./3.*pow(
         m33,2)*m36*L8r - 2./3.*pow(m33,2)*m36*L5r + 2./3.*pow(m33,3)*
         L8r );

      M13p6x23LV +=  + BbV(m36,L) * (  - 2./3.*m13*pow(m66,2)*L8r - 4./
         3.*m13*pow(m66,2)*L6r - 4./3.*m13*m55*m66*L6r - 4./3.*m13*m44*
         m66*L6r + 2./3.*m13*m36*m66*L5r + 4./3.*m13*m36*m66*L4r + 4./3.
         *m13*m36*m55*L4r + 4./3.*m13*m36*m44*L4r - 4./3.*m13*m33*m66*
         L8r - 4./3.*m13*m33*m66*L6r - 4./3.*m13*m33*m55*L6r - 4./3.*
         m13*m33*m44*L6r + 2./3.*m13*m33*m36*L5r - 2./3.*m13*pow(m33,2)
         *L8r + 1./3.*m11*pow(m66,2)*L8r + 2./3.*m11*pow(m66,2)*L6r + 2.
         /3.*m11*m55*m66*L6r + 2./3.*m11*m44*m66*L6r - 1./3.*m11*m36*
         m66*L5r - 2./3.*m11*m36*m66*L4r - 2./3.*m11*m36*m55*L4r - 2./3.
         *m11*m36*m44*L4r + 2./3.*m11*m33*m66*L8r + 2./3.*m11*m33*m66*
         L6r + 2./3.*m11*m33*m55*L6r + 2./3.*m11*m33*m44*L6r - 1./3.*
         m11*m33*m36*L5r + 1./3.*m11*pow(m33,2)*L8r );

      M13p6x23LV +=  + BbV(mp2,L) * ( 4./27.*m66*pow(mp2,2)*L5r*pow(
         RSp453e,2) - 8./27.*m66*pow(mp2,2)*L5r*RSp451e*RSp453e + 4./27.
         *m66*pow(mp2,2)*L5r*pow(RSp451e,2) + 4./27.*m66*pow(mp2,2)*L4r
         *pow(RSp563e,2) + 4./27.*m66*pow(mp2,2)*L4r*pow(RSp463e,2) + 4.
         /27.*m66*pow(mp2,2)*L4r*pow(RSp453e,2) - 8./27.*m66*pow(mp2,2)
         *L4r*RSp561e*RSp563e + 4./27.*m66*pow(mp2,2)*L4r*pow(
         RSp561e,2) - 8./27.*m66*pow(mp2,2)*L4r*RSp461e*RSp463e + 4./27.
         *m66*pow(mp2,2)*L4r*pow(RSp461e,2) - 8./27.*m66*pow(mp2,2)*L4r
         *RSp451e*RSp453e + 4./27.*m66*pow(mp2,2)*L4r*pow(RSp451e,2) - 
         8./27.*pow(m66,2)*mp2*L8r*pow(RSp453e,2) + 16./27.*pow(m66,2)*
         mp2*L8r*RSp451e*RSp453e - 8./27.*pow(m66,2)*mp2*L8r*pow(
         RSp451e,2) - 8./27.*pow(m66,2)*mp2*L7r*pow(RSp453e,2) + 16./27.
         *pow(m66,2)*mp2*L7r*RSp451e*RSp453e - 8./27.*pow(m66,2)*mp2*
         L7r*pow(RSp451e,2) - 8./27.*pow(m66,2)*mp2*L6r*pow(RSp453e,2)
          + 16./27.*pow(m66,2)*mp2*L6r*RSp451e*RSp453e - 8./27.*pow(
         m66,2)*mp2*L6r*pow(RSp451e,2) );

      M13p6x23LV +=  + BbV(mp2,L) * ( 4./27.*m55*pow(mp2,2)*L5r*pow(
         RSp463e,2) - 8./27.*m55*pow(mp2,2)*L5r*RSp461e*RSp463e + 4./27.
         *m55*pow(mp2,2)*L5r*pow(RSp461e,2) + 4./27.*m55*pow(mp2,2)*L4r
         *pow(RSp563e,2) + 4./27.*m55*pow(mp2,2)*L4r*pow(RSp463e,2) + 4.
         /27.*m55*pow(mp2,2)*L4r*pow(RSp453e,2) - 8./27.*m55*pow(mp2,2)
         *L4r*RSp561e*RSp563e + 4./27.*m55*pow(mp2,2)*L4r*pow(
         RSp561e,2) - 8./27.*m55*pow(mp2,2)*L4r*RSp461e*RSp463e + 4./27.
         *m55*pow(mp2,2)*L4r*pow(RSp461e,2) - 8./27.*m55*pow(mp2,2)*L4r
         *RSp451e*RSp453e + 4./27.*m55*pow(mp2,2)*L4r*pow(RSp451e,2) - 
         16./27.*m55*m66*mp2*L7r*RSp453e*RSp463e + 16./27.*m55*m66*mp2*
         L7r*RSp461e*RSp453e + 16./27.*m55*m66*mp2*L7r*RSp451e*RSp463e
          - 16./27.*m55*m66*mp2*L7r*RSp451e*RSp461e - 8./27.*m55*m66*
         mp2*L6r*pow(RSp463e,2) - 8./27.*m55*m66*mp2*L6r*pow(RSp453e,2)
          + 16./27.*m55*m66*mp2*L6r*RSp461e*RSp463e - 8./27.*m55*m66*
         mp2*L6r*pow(RSp461e,2) + 16./27.*m55*m66*mp2*L6r*RSp451e*
         RSp453e );

      M13p6x23LV +=  + BbV(mp2,L) * (  - 8./27.*m55*m66*mp2*L6r*pow(
         RSp451e,2) - 8./27.*pow(m55,2)*mp2*L8r*pow(RSp463e,2) + 16./27.
         *pow(m55,2)*mp2*L8r*RSp461e*RSp463e - 8./27.*pow(m55,2)*mp2*
         L8r*pow(RSp461e,2) - 8./27.*pow(m55,2)*mp2*L7r*pow(RSp463e,2)
          + 16./27.*pow(m55,2)*mp2*L7r*RSp461e*RSp463e - 8./27.*pow(
         m55,2)*mp2*L7r*pow(RSp461e,2) - 8./27.*pow(m55,2)*mp2*L6r*pow(
         RSp463e,2) + 16./27.*pow(m55,2)*mp2*L6r*RSp461e*RSp463e - 8./
         27.*pow(m55,2)*mp2*L6r*pow(RSp461e,2) + 4./27.*m44*pow(mp2,2)*
         L5r*pow(RSp563e,2) - 8./27.*m44*pow(mp2,2)*L5r*RSp561e*RSp563e
          + 4./27.*m44*pow(mp2,2)*L5r*pow(RSp561e,2) + 4./27.*m44*pow(
         mp2,2)*L4r*pow(RSp563e,2) + 4./27.*m44*pow(mp2,2)*L4r*pow(
         RSp463e,2) + 4./27.*m44*pow(mp2,2)*L4r*pow(RSp453e,2) - 8./27.
         *m44*pow(mp2,2)*L4r*RSp561e*RSp563e + 4./27.*m44*pow(mp2,2)*
         L4r*pow(RSp561e,2) - 8./27.*m44*pow(mp2,2)*L4r*RSp461e*RSp463e
          + 4./27.*m44*pow(mp2,2)*L4r*pow(RSp461e,2) - 8./27.*m44*pow(
         mp2,2)*L4r*RSp451e*RSp453e );

      M13p6x23LV +=  + BbV(mp2,L) * ( 4./27.*m44*pow(mp2,2)*L4r*pow(
         RSp451e,2) - 16./27.*m44*m66*mp2*L7r*RSp453e*RSp563e + 16./27.
         *m44*m66*mp2*L7r*RSp561e*RSp453e + 16./27.*m44*m66*mp2*L7r*
         RSp451e*RSp563e - 16./27.*m44*m66*mp2*L7r*RSp451e*RSp561e - 8./
         27.*m44*m66*mp2*L6r*pow(RSp563e,2) - 8./27.*m44*m66*mp2*L6r*
         pow(RSp453e,2) + 16./27.*m44*m66*mp2*L6r*RSp561e*RSp563e - 8./
         27.*m44*m66*mp2*L6r*pow(RSp561e,2) + 16./27.*m44*m66*mp2*L6r*
         RSp451e*RSp453e - 8./27.*m44*m66*mp2*L6r*pow(RSp451e,2) - 16./
         27.*m44*m55*mp2*L7r*RSp463e*RSp563e + 16./27.*m44*m55*mp2*L7r*
         RSp561e*RSp463e + 16./27.*m44*m55*mp2*L7r*RSp461e*RSp563e - 16.
         /27.*m44*m55*mp2*L7r*RSp461e*RSp561e - 8./27.*m44*m55*mp2*L6r*
         pow(RSp563e,2) - 8./27.*m44*m55*mp2*L6r*pow(RSp463e,2) + 16./
         27.*m44*m55*mp2*L6r*RSp561e*RSp563e - 8./27.*m44*m55*mp2*L6r*
         pow(RSp561e,2) + 16./27.*m44*m55*mp2*L6r*RSp461e*RSp463e - 8./
         27.*m44*m55*mp2*L6r*pow(RSp461e,2) - 8./27.*pow(m44,2)*mp2*L8r
         *pow(RSp563e,2) );

      M13p6x23LV +=  + BbV(mp2,L) * ( 16./27.*pow(m44,2)*mp2*L8r*
         RSp561e*RSp563e - 8./27.*pow(m44,2)*mp2*L8r*pow(RSp561e,2) - 8.
         /27.*pow(m44,2)*mp2*L7r*pow(RSp563e,2) + 16./27.*pow(m44,2)*
         mp2*L7r*RSp561e*RSp563e - 8./27.*pow(m44,2)*mp2*L7r*pow(
         RSp561e,2) - 8./27.*pow(m44,2)*mp2*L6r*pow(RSp563e,2) + 16./27.
         *pow(m44,2)*mp2*L6r*RSp561e*RSp563e - 8./27.*pow(m44,2)*mp2*
         L6r*pow(RSp561e,2) - 2./9.*m33*m66*mp2*L5r*pow(RSp453e,2) - 4./
         27.*m33*m66*mp2*L5r*RSp451e*RSp453e - 2./27.*m33*m66*mp2*L5r*
         pow(RSp451e,2) - 2./9.*m33*m66*mp2*L4r*pow(RSp563e,2) - 2./9.*
         m33*m66*mp2*L4r*pow(RSp463e,2) - 2./9.*m33*m66*mp2*L4r*pow(
         RSp453e,2) - 4./27.*m33*m66*mp2*L4r*RSp561e*RSp563e - 2./27.*
         m33*m66*mp2*L4r*pow(RSp561e,2) - 4./27.*m33*m66*mp2*L4r*
         RSp461e*RSp463e - 2./27.*m33*m66*mp2*L4r*pow(RSp461e,2) - 4./
         27.*m33*m66*mp2*L4r*RSp451e*RSp453e - 2./27.*m33*m66*mp2*L4r*
         pow(RSp451e,2) + 4./9.*m33*pow(m66,2)*L8r*pow(RSp453e,2) + 8./
         27.*m33*pow(m66,2)*L8r*RSp451e*RSp453e );

      M13p6x23LV +=  + BbV(mp2,L) * ( 4./27.*m33*pow(m66,2)*L8r*pow(
         RSp451e,2) + 4./9.*m33*pow(m66,2)*L7r*pow(RSp453e,2) + 8./27.*
         m33*pow(m66,2)*L7r*RSp451e*RSp453e + 4./27.*m33*pow(m66,2)*L7r
         *pow(RSp451e,2) + 4./9.*m33*pow(m66,2)*L6r*pow(RSp453e,2) + 8./
         27.*m33*pow(m66,2)*L6r*RSp451e*RSp453e + 4./27.*m33*pow(m66,2)
         *L6r*pow(RSp451e,2) - 2./9.*m33*m55*mp2*L5r*pow(RSp463e,2) - 4.
         /27.*m33*m55*mp2*L5r*RSp461e*RSp463e - 2./27.*m33*m55*mp2*L5r*
         pow(RSp461e,2) - 2./9.*m33*m55*mp2*L4r*pow(RSp563e,2) - 2./9.*
         m33*m55*mp2*L4r*pow(RSp463e,2) - 2./9.*m33*m55*mp2*L4r*pow(
         RSp453e,2) - 4./27.*m33*m55*mp2*L4r*RSp561e*RSp563e - 2./27.*
         m33*m55*mp2*L4r*pow(RSp561e,2) - 4./27.*m33*m55*mp2*L4r*
         RSp461e*RSp463e - 2./27.*m33*m55*mp2*L4r*pow(RSp461e,2) - 4./
         27.*m33*m55*mp2*L4r*RSp451e*RSp453e - 2./27.*m33*m55*mp2*L4r*
         pow(RSp451e,2) + 8./9.*m33*m55*m66*L7r*RSp453e*RSp463e + 8./27.
         *m33*m55*m66*L7r*RSp461e*RSp453e + 8./27.*m33*m55*m66*L7r*
         RSp451e*RSp463e );

      M13p6x23LV +=  + BbV(mp2,L) * ( 8./27.*m33*m55*m66*L7r*RSp451e*
         RSp461e + 4./9.*m33*m55*m66*L6r*pow(RSp463e,2) + 4./9.*m33*m55
         *m66*L6r*pow(RSp453e,2) + 8./27.*m33*m55*m66*L6r*RSp461e*
         RSp463e + 4./27.*m33*m55*m66*L6r*pow(RSp461e,2) + 8./27.*m33*
         m55*m66*L6r*RSp451e*RSp453e + 4./27.*m33*m55*m66*L6r*pow(
         RSp451e,2) + 4./9.*m33*pow(m55,2)*L8r*pow(RSp463e,2) + 8./27.*
         m33*pow(m55,2)*L8r*RSp461e*RSp463e + 4./27.*m33*pow(m55,2)*L8r
         *pow(RSp461e,2) + 4./9.*m33*pow(m55,2)*L7r*pow(RSp463e,2) + 8./
         27.*m33*pow(m55,2)*L7r*RSp461e*RSp463e + 4./27.*m33*pow(m55,2)
         *L7r*pow(RSp461e,2) + 4./9.*m33*pow(m55,2)*L6r*pow(RSp463e,2)
          + 8./27.*m33*pow(m55,2)*L6r*RSp461e*RSp463e + 4./27.*m33*pow(
         m55,2)*L6r*pow(RSp461e,2) - 2./9.*m33*m44*mp2*L5r*pow(
         RSp563e,2) - 4./27.*m33*m44*mp2*L5r*RSp561e*RSp563e - 2./27.*
         m33*m44*mp2*L5r*pow(RSp561e,2) - 2./9.*m33*m44*mp2*L4r*pow(
         RSp563e,2) - 2./9.*m33*m44*mp2*L4r*pow(RSp463e,2) - 2./9.*m33*
         m44*mp2*L4r*pow(RSp453e,2) );

      M13p6x23LV +=  + BbV(mp2,L) * (  - 4./27.*m33*m44*mp2*L4r*RSp561e
         *RSp563e - 2./27.*m33*m44*mp2*L4r*pow(RSp561e,2) - 4./27.*m33*
         m44*mp2*L4r*RSp461e*RSp463e - 2./27.*m33*m44*mp2*L4r*pow(
         RSp461e,2) - 4./27.*m33*m44*mp2*L4r*RSp451e*RSp453e - 2./27.*
         m33*m44*mp2*L4r*pow(RSp451e,2) + 8./9.*m33*m44*m66*L7r*RSp453e
         *RSp563e + 8./27.*m33*m44*m66*L7r*RSp561e*RSp453e + 8./27.*m33
         *m44*m66*L7r*RSp451e*RSp563e + 8./27.*m33*m44*m66*L7r*RSp451e*
         RSp561e + 4./9.*m33*m44*m66*L6r*pow(RSp563e,2) + 4./9.*m33*m44
         *m66*L6r*pow(RSp453e,2) + 8./27.*m33*m44*m66*L6r*RSp561e*
         RSp563e + 4./27.*m33*m44*m66*L6r*pow(RSp561e,2) + 8./27.*m33*
         m44*m66*L6r*RSp451e*RSp453e + 4./27.*m33*m44*m66*L6r*pow(
         RSp451e,2) + 8./9.*m33*m44*m55*L7r*RSp463e*RSp563e + 8./27.*
         m33*m44*m55*L7r*RSp561e*RSp463e + 8./27.*m33*m44*m55*L7r*
         RSp461e*RSp563e + 8./27.*m33*m44*m55*L7r*RSp461e*RSp561e + 4./
         9.*m33*m44*m55*L6r*pow(RSp563e,2) + 4./9.*m33*m44*m55*L6r*pow(
         RSp463e,2) );

      M13p6x23LV +=  + BbV(mp2,L) * ( 8./27.*m33*m44*m55*L6r*RSp561e*
         RSp563e + 4./27.*m33*m44*m55*L6r*pow(RSp561e,2) + 8./27.*m33*
         m44*m55*L6r*RSp461e*RSp463e + 4./27.*m33*m44*m55*L6r*pow(
         RSp461e,2) + 4./9.*m33*pow(m44,2)*L8r*pow(RSp563e,2) + 8./27.*
         m33*pow(m44,2)*L8r*RSp561e*RSp563e + 4./27.*m33*pow(m44,2)*L8r
         *pow(RSp561e,2) + 4./9.*m33*pow(m44,2)*L7r*pow(RSp563e,2) + 8./
         27.*m33*pow(m44,2)*L7r*RSp561e*RSp563e + 4./27.*m33*pow(m44,2)
         *L7r*pow(RSp561e,2) + 4./9.*m33*pow(m44,2)*L6r*pow(RSp563e,2)
          + 8./27.*m33*pow(m44,2)*L6r*RSp561e*RSp563e + 4./27.*m33*pow(
         m44,2)*L6r*pow(RSp561e,2) + 4./27.*m13*m66*mp2*L5r*pow(
         RSp453e,2) - 8./27.*m13*m66*mp2*L5r*RSp451e*RSp453e + 4./27.*
         m13*m66*mp2*L5r*pow(RSp451e,2) + 4./27.*m13*m66*mp2*L4r*pow(
         RSp563e,2) + 4./27.*m13*m66*mp2*L4r*pow(RSp463e,2) + 4./27.*
         m13*m66*mp2*L4r*pow(RSp453e,2) - 8./27.*m13*m66*mp2*L4r*
         RSp561e*RSp563e + 4./27.*m13*m66*mp2*L4r*pow(RSp561e,2) - 8./
         27.*m13*m66*mp2*L4r*RSp461e*RSp463e );

      M13p6x23LV +=  + BbV(mp2,L) * ( 4./27.*m13*m66*mp2*L4r*pow(
         RSp461e,2) - 8./27.*m13*m66*mp2*L4r*RSp451e*RSp453e + 4./27.*
         m13*m66*mp2*L4r*pow(RSp451e,2) - 8./27.*m13*pow(m66,2)*L8r*
         pow(RSp453e,2) + 16./27.*m13*pow(m66,2)*L8r*RSp451e*RSp453e - 
         8./27.*m13*pow(m66,2)*L8r*pow(RSp451e,2) - 8./27.*m13*pow(
         m66,2)*L7r*pow(RSp453e,2) + 16./27.*m13*pow(m66,2)*L7r*RSp451e
         *RSp453e - 8./27.*m13*pow(m66,2)*L7r*pow(RSp451e,2) - 8./27.*
         m13*pow(m66,2)*L6r*pow(RSp453e,2) + 16./27.*m13*pow(m66,2)*L6r
         *RSp451e*RSp453e - 8./27.*m13*pow(m66,2)*L6r*pow(RSp451e,2) + 
         4./27.*m13*m55*mp2*L5r*pow(RSp463e,2) - 8./27.*m13*m55*mp2*L5r
         *RSp461e*RSp463e + 4./27.*m13*m55*mp2*L5r*pow(RSp461e,2) + 4./
         27.*m13*m55*mp2*L4r*pow(RSp563e,2) + 4./27.*m13*m55*mp2*L4r*
         pow(RSp463e,2) + 4./27.*m13*m55*mp2*L4r*pow(RSp453e,2) - 8./27.
         *m13*m55*mp2*L4r*RSp561e*RSp563e + 4./27.*m13*m55*mp2*L4r*pow(
         RSp561e,2) - 8./27.*m13*m55*mp2*L4r*RSp461e*RSp463e + 4./27.*
         m13*m55*mp2*L4r*pow(RSp461e,2) );

      M13p6x23LV +=  + BbV(mp2,L) * (  - 8./27.*m13*m55*mp2*L4r*RSp451e
         *RSp453e + 4./27.*m13*m55*mp2*L4r*pow(RSp451e,2) - 16./27.*m13
         *m55*m66*L7r*RSp453e*RSp463e + 16./27.*m13*m55*m66*L7r*RSp461e
         *RSp453e + 16./27.*m13*m55*m66*L7r*RSp451e*RSp463e - 16./27.*
         m13*m55*m66*L7r*RSp451e*RSp461e - 8./27.*m13*m55*m66*L6r*pow(
         RSp463e,2) - 8./27.*m13*m55*m66*L6r*pow(RSp453e,2) + 16./27.*
         m13*m55*m66*L6r*RSp461e*RSp463e - 8./27.*m13*m55*m66*L6r*pow(
         RSp461e,2) + 16./27.*m13*m55*m66*L6r*RSp451e*RSp453e - 8./27.*
         m13*m55*m66*L6r*pow(RSp451e,2) - 8./27.*m13*pow(m55,2)*L8r*
         pow(RSp463e,2) + 16./27.*m13*pow(m55,2)*L8r*RSp461e*RSp463e - 
         8./27.*m13*pow(m55,2)*L8r*pow(RSp461e,2) - 8./27.*m13*pow(
         m55,2)*L7r*pow(RSp463e,2) + 16./27.*m13*pow(m55,2)*L7r*RSp461e
         *RSp463e - 8./27.*m13*pow(m55,2)*L7r*pow(RSp461e,2) - 8./27.*
         m13*pow(m55,2)*L6r*pow(RSp463e,2) + 16./27.*m13*pow(m55,2)*L6r
         *RSp461e*RSp463e - 8./27.*m13*pow(m55,2)*L6r*pow(RSp461e,2) + 
         4./27.*m13*m44*mp2*L5r*pow(RSp563e,2) );

      M13p6x23LV +=  + BbV(mp2,L) * (  - 8./27.*m13*m44*mp2*L5r*RSp561e
         *RSp563e + 4./27.*m13*m44*mp2*L5r*pow(RSp561e,2) + 4./27.*m13*
         m44*mp2*L4r*pow(RSp563e,2) + 4./27.*m13*m44*mp2*L4r*pow(
         RSp463e,2) + 4./27.*m13*m44*mp2*L4r*pow(RSp453e,2) - 8./27.*
         m13*m44*mp2*L4r*RSp561e*RSp563e + 4./27.*m13*m44*mp2*L4r*pow(
         RSp561e,2) - 8./27.*m13*m44*mp2*L4r*RSp461e*RSp463e + 4./27.*
         m13*m44*mp2*L4r*pow(RSp461e,2) - 8./27.*m13*m44*mp2*L4r*
         RSp451e*RSp453e + 4./27.*m13*m44*mp2*L4r*pow(RSp451e,2) - 16./
         27.*m13*m44*m66*L7r*RSp453e*RSp563e + 16./27.*m13*m44*m66*L7r*
         RSp561e*RSp453e + 16./27.*m13*m44*m66*L7r*RSp451e*RSp563e - 16.
         /27.*m13*m44*m66*L7r*RSp451e*RSp561e - 8./27.*m13*m44*m66*L6r*
         pow(RSp563e,2) - 8./27.*m13*m44*m66*L6r*pow(RSp453e,2) + 16./
         27.*m13*m44*m66*L6r*RSp561e*RSp563e - 8./27.*m13*m44*m66*L6r*
         pow(RSp561e,2) + 16./27.*m13*m44*m66*L6r*RSp451e*RSp453e - 8./
         27.*m13*m44*m66*L6r*pow(RSp451e,2) - 16./27.*m13*m44*m55*L7r*
         RSp463e*RSp563e );

      M13p6x23LV +=  + BbV(mp2,L) * ( 16./27.*m13*m44*m55*L7r*RSp561e*
         RSp463e + 16./27.*m13*m44*m55*L7r*RSp461e*RSp563e - 16./27.*
         m13*m44*m55*L7r*RSp461e*RSp561e - 8./27.*m13*m44*m55*L6r*pow(
         RSp563e,2) - 8./27.*m13*m44*m55*L6r*pow(RSp463e,2) + 16./27.*
         m13*m44*m55*L6r*RSp561e*RSp563e - 8./27.*m13*m44*m55*L6r*pow(
         RSp561e,2) + 16./27.*m13*m44*m55*L6r*RSp461e*RSp463e - 8./27.*
         m13*m44*m55*L6r*pow(RSp461e,2) - 8./27.*m13*pow(m44,2)*L8r*
         pow(RSp563e,2) + 16./27.*m13*pow(m44,2)*L8r*RSp561e*RSp563e - 
         8./27.*m13*pow(m44,2)*L8r*pow(RSp561e,2) - 8./27.*m13*pow(
         m44,2)*L7r*pow(RSp563e,2) + 16./27.*m13*pow(m44,2)*L7r*RSp561e
         *RSp563e - 8./27.*m13*pow(m44,2)*L7r*pow(RSp561e,2) - 8./27.*
         m13*pow(m44,2)*L6r*pow(RSp563e,2) + 16./27.*m13*pow(m44,2)*L6r
         *RSp561e*RSp563e - 8./27.*m13*pow(m44,2)*L6r*pow(RSp561e,2) - 
         2./27.*m11*m66*mp2*L5r*pow(RSp453e,2) - 4./27.*m11*m66*mp2*L5r
         *RSp451e*RSp453e - 2./9.*m11*m66*mp2*L5r*pow(RSp451e,2) - 2./
         27.*m11*m66*mp2*L4r*pow(RSp563e,2) );

      M13p6x23LV +=  + BbV(mp2,L) * (  - 2./27.*m11*m66*mp2*L4r*pow(
         RSp463e,2) - 2./27.*m11*m66*mp2*L4r*pow(RSp453e,2) - 4./27.*
         m11*m66*mp2*L4r*RSp561e*RSp563e - 2./9.*m11*m66*mp2*L4r*pow(
         RSp561e,2) - 4./27.*m11*m66*mp2*L4r*RSp461e*RSp463e - 2./9.*
         m11*m66*mp2*L4r*pow(RSp461e,2) - 4./27.*m11*m66*mp2*L4r*
         RSp451e*RSp453e - 2./9.*m11*m66*mp2*L4r*pow(RSp451e,2) + 4./27.
         *m11*pow(m66,2)*L8r*pow(RSp453e,2) + 8./27.*m11*pow(m66,2)*L8r
         *RSp451e*RSp453e + 4./9.*m11*pow(m66,2)*L8r*pow(RSp451e,2) + 4.
         /27.*m11*pow(m66,2)*L7r*pow(RSp453e,2) + 8./27.*m11*pow(m66,2)
         *L7r*RSp451e*RSp453e + 4./9.*m11*pow(m66,2)*L7r*pow(RSp451e,2)
          + 4./27.*m11*pow(m66,2)*L6r*pow(RSp453e,2) + 8./27.*m11*pow(
         m66,2)*L6r*RSp451e*RSp453e + 4./9.*m11*pow(m66,2)*L6r*pow(
         RSp451e,2) - 2./27.*m11*m55*mp2*L5r*pow(RSp463e,2) - 4./27.*
         m11*m55*mp2*L5r*RSp461e*RSp463e - 2./9.*m11*m55*mp2*L5r*pow(
         RSp461e,2) - 2./27.*m11*m55*mp2*L4r*pow(RSp563e,2) - 2./27.*
         m11*m55*mp2*L4r*pow(RSp463e,2) );

      M13p6x23LV +=  + BbV(mp2,L) * (  - 2./27.*m11*m55*mp2*L4r*pow(
         RSp453e,2) - 4./27.*m11*m55*mp2*L4r*RSp561e*RSp563e - 2./9.*
         m11*m55*mp2*L4r*pow(RSp561e,2) - 4./27.*m11*m55*mp2*L4r*
         RSp461e*RSp463e - 2./9.*m11*m55*mp2*L4r*pow(RSp461e,2) - 4./27.
         *m11*m55*mp2*L4r*RSp451e*RSp453e - 2./9.*m11*m55*mp2*L4r*pow(
         RSp451e,2) + 8./27.*m11*m55*m66*L7r*RSp453e*RSp463e + 8./27.*
         m11*m55*m66*L7r*RSp461e*RSp453e + 8./27.*m11*m55*m66*L7r*
         RSp451e*RSp463e + 8./9.*m11*m55*m66*L7r*RSp451e*RSp461e + 4./
         27.*m11*m55*m66*L6r*pow(RSp463e,2) + 4./27.*m11*m55*m66*L6r*
         pow(RSp453e,2) + 8./27.*m11*m55*m66*L6r*RSp461e*RSp463e + 4./9.
         *m11*m55*m66*L6r*pow(RSp461e,2) + 8./27.*m11*m55*m66*L6r*
         RSp451e*RSp453e + 4./9.*m11*m55*m66*L6r*pow(RSp451e,2) + 4./27.
         *m11*pow(m55,2)*L8r*pow(RSp463e,2) + 8./27.*m11*pow(m55,2)*L8r
         *RSp461e*RSp463e + 4./9.*m11*pow(m55,2)*L8r*pow(RSp461e,2) + 4.
         /27.*m11*pow(m55,2)*L7r*pow(RSp463e,2) + 8./27.*m11*pow(m55,2)
         *L7r*RSp461e*RSp463e );

      M13p6x23LV +=  + BbV(mp2,L) * ( 4./9.*m11*pow(m55,2)*L7r*pow(
         RSp461e,2) + 4./27.*m11*pow(m55,2)*L6r*pow(RSp463e,2) + 8./27.
         *m11*pow(m55,2)*L6r*RSp461e*RSp463e + 4./9.*m11*pow(m55,2)*L6r
         *pow(RSp461e,2) - 2./27.*m11*m44*mp2*L5r*pow(RSp563e,2) - 4./
         27.*m11*m44*mp2*L5r*RSp561e*RSp563e - 2./9.*m11*m44*mp2*L5r*
         pow(RSp561e,2) - 2./27.*m11*m44*mp2*L4r*pow(RSp563e,2) - 2./27.
         *m11*m44*mp2*L4r*pow(RSp463e,2) - 2./27.*m11*m44*mp2*L4r*pow(
         RSp453e,2) - 4./27.*m11*m44*mp2*L4r*RSp561e*RSp563e - 2./9.*
         m11*m44*mp2*L4r*pow(RSp561e,2) - 4./27.*m11*m44*mp2*L4r*
         RSp461e*RSp463e - 2./9.*m11*m44*mp2*L4r*pow(RSp461e,2) - 4./27.
         *m11*m44*mp2*L4r*RSp451e*RSp453e - 2./9.*m11*m44*mp2*L4r*pow(
         RSp451e,2) + 8./27.*m11*m44*m66*L7r*RSp453e*RSp563e + 8./27.*
         m11*m44*m66*L7r*RSp561e*RSp453e + 8./27.*m11*m44*m66*L7r*
         RSp451e*RSp563e + 8./9.*m11*m44*m66*L7r*RSp451e*RSp561e + 4./
         27.*m11*m44*m66*L6r*pow(RSp563e,2) + 4./27.*m11*m44*m66*L6r*
         pow(RSp453e,2) );

      M13p6x23LV +=  + BbV(mp2,L) * ( 8./27.*m11*m44*m66*L6r*RSp561e*
         RSp563e + 4./9.*m11*m44*m66*L6r*pow(RSp561e,2) + 8./27.*m11*
         m44*m66*L6r*RSp451e*RSp453e + 4./9.*m11*m44*m66*L6r*pow(
         RSp451e,2) + 8./27.*m11*m44*m55*L7r*RSp463e*RSp563e + 8./27.*
         m11*m44*m55*L7r*RSp561e*RSp463e + 8./27.*m11*m44*m55*L7r*
         RSp461e*RSp563e + 8./9.*m11*m44*m55*L7r*RSp461e*RSp561e + 4./
         27.*m11*m44*m55*L6r*pow(RSp563e,2) + 4./27.*m11*m44*m55*L6r*
         pow(RSp463e,2) + 8./27.*m11*m44*m55*L6r*RSp561e*RSp563e + 4./9.
         *m11*m44*m55*L6r*pow(RSp561e,2) + 8./27.*m11*m44*m55*L6r*
         RSp461e*RSp463e + 4./9.*m11*m44*m55*L6r*pow(RSp461e,2) + 4./27.
         *m11*pow(m44,2)*L8r*pow(RSp563e,2) + 8./27.*m11*pow(m44,2)*L8r
         *RSp561e*RSp563e + 4./9.*m11*pow(m44,2)*L8r*pow(RSp561e,2) + 4.
         /27.*m11*pow(m44,2)*L7r*pow(RSp563e,2) + 8./27.*m11*pow(m44,2)
         *L7r*RSp561e*RSp563e + 4./9.*m11*pow(m44,2)*L7r*pow(RSp561e,2)
          + 4./27.*m11*pow(m44,2)*L6r*pow(RSp563e,2) + 8./27.*m11*pow(
         m44,2)*L6r*RSp561e*RSp563e );

      M13p6x23LV +=  + BbV(mp2,L) * ( 4./9.*m11*pow(m44,2)*L6r*pow(
         RSp561e,2) );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 8./27.*m66*mp2*me2*L5r*
         RSp453e*RSe453p - 8./27.*m66*mp2*me2*L5r*RSe451p*RSp453e - 8./
         27.*m66*mp2*me2*L5r*RSp451e*RSe453p + 8./27.*m66*mp2*me2*L5r*
         RSp451e*RSe451p + 8./27.*m66*mp2*me2*L4r*RSp563e*RSe563p + 8./
         27.*m66*mp2*me2*L4r*RSp463e*RSe463p + 8./27.*m66*mp2*me2*L4r*
         RSp453e*RSe453p - 8./27.*m66*mp2*me2*L4r*RSe561p*RSp563e - 8./
         27.*m66*mp2*me2*L4r*RSe461p*RSp463e - 8./27.*m66*mp2*me2*L4r*
         RSe451p*RSp453e - 8./27.*m66*mp2*me2*L4r*RSp561e*RSe563p + 8./
         27.*m66*mp2*me2*L4r*RSp561e*RSe561p - 8./27.*m66*mp2*me2*L4r*
         RSp461e*RSe463p + 8./27.*m66*mp2*me2*L4r*RSp461e*RSe461p - 8./
         27.*m66*mp2*me2*L4r*RSp451e*RSe453p + 8./27.*m66*mp2*me2*L4r*
         RSp451e*RSe451p - 16./27.*pow(m66,2)*me2*L8r*RSp453e*RSe453p
          + 16./27.*pow(m66,2)*me2*L8r*RSe451p*RSp453e + 16./27.*pow(
         m66,2)*me2*L8r*RSp451e*RSe453p - 16./27.*pow(m66,2)*me2*L8r*
         RSp451e*RSe451p - 16./27.*pow(m66,2)*me2*L7r*RSp453e*RSe453p
          + 16./27.*pow(m66,2)*me2*L7r*RSe451p*RSp453e );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 16./27.*pow(m66,2)*me2*L7r*
         RSp451e*RSe453p - 16./27.*pow(m66,2)*me2*L7r*RSp451e*RSe451p
          - 16./27.*pow(m66,2)*me2*L6r*RSp453e*RSe453p + 16./27.*pow(
         m66,2)*me2*L6r*RSe451p*RSp453e + 16./27.*pow(m66,2)*me2*L6r*
         RSp451e*RSe453p - 16./27.*pow(m66,2)*me2*L6r*RSp451e*RSe451p
          + 8./27.*m55*mp2*me2*L5r*RSp463e*RSe463p - 8./27.*m55*mp2*me2
         *L5r*RSe461p*RSp463e - 8./27.*m55*mp2*me2*L5r*RSp461e*RSe463p
          + 8./27.*m55*mp2*me2*L5r*RSp461e*RSe461p + 8./27.*m55*mp2*me2
         *L4r*RSp563e*RSe563p + 8./27.*m55*mp2*me2*L4r*RSp463e*RSe463p
          + 8./27.*m55*mp2*me2*L4r*RSp453e*RSe453p - 8./27.*m55*mp2*me2
         *L4r*RSe561p*RSp563e - 8./27.*m55*mp2*me2*L4r*RSe461p*RSp463e
          - 8./27.*m55*mp2*me2*L4r*RSe451p*RSp453e - 8./27.*m55*mp2*me2
         *L4r*RSp561e*RSe563p + 8./27.*m55*mp2*me2*L4r*RSp561e*RSe561p
          - 8./27.*m55*mp2*me2*L4r*RSp461e*RSe463p + 8./27.*m55*mp2*me2
         *L4r*RSp461e*RSe461p - 8./27.*m55*mp2*me2*L4r*RSp451e*RSe453p
          + 8./27.*m55*mp2*me2*L4r*RSp451e*RSe451p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * (  - 16./27.*m55*m66*me2*L7r*
         RSp463e*RSe453p - 16./27.*m55*m66*me2*L7r*RSp453e*RSe463p + 16.
         /27.*m55*m66*me2*L7r*RSe461p*RSp453e + 16./27.*m55*m66*me2*L7r
         *RSe451p*RSp463e + 16./27.*m55*m66*me2*L7r*RSp461e*RSe453p - 
         16./27.*m55*m66*me2*L7r*RSp461e*RSe451p + 16./27.*m55*m66*me2*
         L7r*RSp451e*RSe463p - 16./27.*m55*m66*me2*L7r*RSp451e*RSe461p
          - 16./27.*m55*m66*me2*L6r*RSp463e*RSe463p - 16./27.*m55*m66*
         me2*L6r*RSp453e*RSe453p + 16./27.*m55*m66*me2*L6r*RSe461p*
         RSp463e + 16./27.*m55*m66*me2*L6r*RSe451p*RSp453e + 16./27.*
         m55*m66*me2*L6r*RSp461e*RSe463p - 16./27.*m55*m66*me2*L6r*
         RSp461e*RSe461p + 16./27.*m55*m66*me2*L6r*RSp451e*RSe453p - 16.
         /27.*m55*m66*me2*L6r*RSp451e*RSe451p - 16./27.*pow(m55,2)*me2*
         L8r*RSp463e*RSe463p + 16./27.*pow(m55,2)*me2*L8r*RSe461p*
         RSp463e + 16./27.*pow(m55,2)*me2*L8r*RSp461e*RSe463p - 16./27.
         *pow(m55,2)*me2*L8r*RSp461e*RSe461p - 16./27.*pow(m55,2)*me2*
         L7r*RSp463e*RSe463p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 16./27.*pow(m55,2)*me2*L7r*
         RSe461p*RSp463e + 16./27.*pow(m55,2)*me2*L7r*RSp461e*RSe463p
          - 16./27.*pow(m55,2)*me2*L7r*RSp461e*RSe461p - 16./27.*pow(
         m55,2)*me2*L6r*RSp463e*RSe463p + 16./27.*pow(m55,2)*me2*L6r*
         RSe461p*RSp463e + 16./27.*pow(m55,2)*me2*L6r*RSp461e*RSe463p
          - 16./27.*pow(m55,2)*me2*L6r*RSp461e*RSe461p + 8./27.*m44*mp2
         *me2*L5r*RSp563e*RSe563p - 8./27.*m44*mp2*me2*L5r*RSe561p*
         RSp563e - 8./27.*m44*mp2*me2*L5r*RSp561e*RSe563p + 8./27.*m44*
         mp2*me2*L5r*RSp561e*RSe561p + 8./27.*m44*mp2*me2*L4r*RSp563e*
         RSe563p + 8./27.*m44*mp2*me2*L4r*RSp463e*RSe463p + 8./27.*m44*
         mp2*me2*L4r*RSp453e*RSe453p - 8./27.*m44*mp2*me2*L4r*RSe561p*
         RSp563e - 8./27.*m44*mp2*me2*L4r*RSe461p*RSp463e - 8./27.*m44*
         mp2*me2*L4r*RSe451p*RSp453e - 8./27.*m44*mp2*me2*L4r*RSp561e*
         RSe563p + 8./27.*m44*mp2*me2*L4r*RSp561e*RSe561p - 8./27.*m44*
         mp2*me2*L4r*RSp461e*RSe463p + 8./27.*m44*mp2*me2*L4r*RSp461e*
         RSe461p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * (  - 8./27.*m44*mp2*me2*L4r*
         RSp451e*RSe453p + 8./27.*m44*mp2*me2*L4r*RSp451e*RSe451p - 16./
         27.*m44*m66*me2*L7r*RSp563e*RSe453p - 16./27.*m44*m66*me2*L7r*
         RSp453e*RSe563p + 16./27.*m44*m66*me2*L7r*RSe561p*RSp453e + 16.
         /27.*m44*m66*me2*L7r*RSe451p*RSp563e + 16./27.*m44*m66*me2*L7r
         *RSp561e*RSe453p - 16./27.*m44*m66*me2*L7r*RSp561e*RSe451p + 
         16./27.*m44*m66*me2*L7r*RSp451e*RSe563p - 16./27.*m44*m66*me2*
         L7r*RSp451e*RSe561p - 16./27.*m44*m66*me2*L6r*RSp563e*RSe563p
          - 16./27.*m44*m66*me2*L6r*RSp453e*RSe453p + 16./27.*m44*m66*
         me2*L6r*RSe561p*RSp563e + 16./27.*m44*m66*me2*L6r*RSe451p*
         RSp453e + 16./27.*m44*m66*me2*L6r*RSp561e*RSe563p - 16./27.*
         m44*m66*me2*L6r*RSp561e*RSe561p + 16./27.*m44*m66*me2*L6r*
         RSp451e*RSe453p - 16./27.*m44*m66*me2*L6r*RSp451e*RSe451p - 16.
         /27.*m44*m55*me2*L7r*RSp563e*RSe463p - 16./27.*m44*m55*me2*L7r
         *RSp463e*RSe563p + 16./27.*m44*m55*me2*L7r*RSe561p*RSp463e + 
         16./27.*m44*m55*me2*L7r*RSe461p*RSp563e );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 16./27.*m44*m55*me2*L7r*
         RSp561e*RSe463p - 16./27.*m44*m55*me2*L7r*RSp561e*RSe461p + 16.
         /27.*m44*m55*me2*L7r*RSp461e*RSe563p - 16./27.*m44*m55*me2*L7r
         *RSp461e*RSe561p - 16./27.*m44*m55*me2*L6r*RSp563e*RSe563p - 
         16./27.*m44*m55*me2*L6r*RSp463e*RSe463p + 16./27.*m44*m55*me2*
         L6r*RSe561p*RSp563e + 16./27.*m44*m55*me2*L6r*RSe461p*RSp463e
          + 16./27.*m44*m55*me2*L6r*RSp561e*RSe563p - 16./27.*m44*m55*
         me2*L6r*RSp561e*RSe561p + 16./27.*m44*m55*me2*L6r*RSp461e*
         RSe463p - 16./27.*m44*m55*me2*L6r*RSp461e*RSe461p - 16./27.*
         pow(m44,2)*me2*L8r*RSp563e*RSe563p + 16./27.*pow(m44,2)*me2*
         L8r*RSe561p*RSp563e + 16./27.*pow(m44,2)*me2*L8r*RSp561e*
         RSe563p - 16./27.*pow(m44,2)*me2*L8r*RSp561e*RSe561p - 16./27.
         *pow(m44,2)*me2*L7r*RSp563e*RSe563p + 16./27.*pow(m44,2)*me2*
         L7r*RSe561p*RSp563e + 16./27.*pow(m44,2)*me2*L7r*RSp561e*
         RSe563p - 16./27.*pow(m44,2)*me2*L7r*RSp561e*RSe561p - 16./27.
         *pow(m44,2)*me2*L6r*RSp563e*RSe563p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 16./27.*pow(m44,2)*me2*L6r*
         RSe561p*RSp563e + 16./27.*pow(m44,2)*me2*L6r*RSp561e*RSe563p
          - 16./27.*pow(m44,2)*me2*L6r*RSp561e*RSe561p - 4./9.*m33*m66*
         me2*L5r*RSp453e*RSe453p - 4./27.*m33*m66*me2*L5r*RSe451p*
         RSp453e - 4./27.*m33*m66*me2*L5r*RSp451e*RSe453p - 4./27.*m33*
         m66*me2*L5r*RSp451e*RSe451p - 4./9.*m33*m66*me2*L4r*RSp563e*
         RSe563p - 4./9.*m33*m66*me2*L4r*RSp463e*RSe463p - 4./9.*m33*
         m66*me2*L4r*RSp453e*RSe453p - 4./27.*m33*m66*me2*L4r*RSe561p*
         RSp563e - 4./27.*m33*m66*me2*L4r*RSe461p*RSp463e - 4./27.*m33*
         m66*me2*L4r*RSe451p*RSp453e - 4./27.*m33*m66*me2*L4r*RSp561e*
         RSe563p - 4./27.*m33*m66*me2*L4r*RSp561e*RSe561p - 4./27.*m33*
         m66*me2*L4r*RSp461e*RSe463p - 4./27.*m33*m66*me2*L4r*RSp461e*
         RSe461p - 4./27.*m33*m66*me2*L4r*RSp451e*RSe453p - 4./27.*m33*
         m66*me2*L4r*RSp451e*RSe451p + 8./9.*m33*pow(m66,2)*L8r*RSp453e
         *RSe453p + 8./27.*m33*pow(m66,2)*L8r*RSe451p*RSp453e + 8./27.*
         m33*pow(m66,2)*L8r*RSp451e*RSe453p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 8./27.*m33*pow(m66,2)*L8r*
         RSp451e*RSe451p + 8./9.*m33*pow(m66,2)*L7r*RSp453e*RSe453p + 8.
         /27.*m33*pow(m66,2)*L7r*RSe451p*RSp453e + 8./27.*m33*pow(
         m66,2)*L7r*RSp451e*RSe453p + 8./27.*m33*pow(m66,2)*L7r*RSp451e
         *RSe451p + 8./9.*m33*pow(m66,2)*L6r*RSp453e*RSe453p + 8./27.*
         m33*pow(m66,2)*L6r*RSe451p*RSp453e + 8./27.*m33*pow(m66,2)*L6r
         *RSp451e*RSe453p + 8./27.*m33*pow(m66,2)*L6r*RSp451e*RSe451p
          - 4./9.*m33*m55*me2*L5r*RSp463e*RSe463p - 4./27.*m33*m55*me2*
         L5r*RSe461p*RSp463e - 4./27.*m33*m55*me2*L5r*RSp461e*RSe463p
          - 4./27.*m33*m55*me2*L5r*RSp461e*RSe461p - 4./9.*m33*m55*me2*
         L4r*RSp563e*RSe563p - 4./9.*m33*m55*me2*L4r*RSp463e*RSe463p - 
         4./9.*m33*m55*me2*L4r*RSp453e*RSe453p - 4./27.*m33*m55*me2*L4r
         *RSe561p*RSp563e - 4./27.*m33*m55*me2*L4r*RSe461p*RSp463e - 4./
         27.*m33*m55*me2*L4r*RSe451p*RSp453e - 4./27.*m33*m55*me2*L4r*
         RSp561e*RSe563p - 4./27.*m33*m55*me2*L4r*RSp561e*RSe561p - 4./
         27.*m33*m55*me2*L4r*RSp461e*RSe463p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * (  - 4./27.*m33*m55*me2*L4r*
         RSp461e*RSe461p - 4./27.*m33*m55*me2*L4r*RSp451e*RSe453p - 4./
         27.*m33*m55*me2*L4r*RSp451e*RSe451p + 8./9.*m33*m55*m66*L7r*
         RSp463e*RSe453p + 8./9.*m33*m55*m66*L7r*RSp453e*RSe463p + 8./
         27.*m33*m55*m66*L7r*RSe461p*RSp453e + 8./27.*m33*m55*m66*L7r*
         RSe451p*RSp463e + 8./27.*m33*m55*m66*L7r*RSp461e*RSe453p + 8./
         27.*m33*m55*m66*L7r*RSp461e*RSe451p + 8./27.*m33*m55*m66*L7r*
         RSp451e*RSe463p + 8./27.*m33*m55*m66*L7r*RSp451e*RSe461p + 8./
         9.*m33*m55*m66*L6r*RSp463e*RSe463p + 8./9.*m33*m55*m66*L6r*
         RSp453e*RSe453p + 8./27.*m33*m55*m66*L6r*RSe461p*RSp463e + 8./
         27.*m33*m55*m66*L6r*RSe451p*RSp453e + 8./27.*m33*m55*m66*L6r*
         RSp461e*RSe463p + 8./27.*m33*m55*m66*L6r*RSp461e*RSe461p + 8./
         27.*m33*m55*m66*L6r*RSp451e*RSe453p + 8./27.*m33*m55*m66*L6r*
         RSp451e*RSe451p + 8./9.*m33*pow(m55,2)*L8r*RSp463e*RSe463p + 8.
         /27.*m33*pow(m55,2)*L8r*RSe461p*RSp463e + 8./27.*m33*pow(
         m55,2)*L8r*RSp461e*RSe463p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 8./27.*m33*pow(m55,2)*L8r*
         RSp461e*RSe461p + 8./9.*m33*pow(m55,2)*L7r*RSp463e*RSe463p + 8.
         /27.*m33*pow(m55,2)*L7r*RSe461p*RSp463e + 8./27.*m33*pow(
         m55,2)*L7r*RSp461e*RSe463p + 8./27.*m33*pow(m55,2)*L7r*RSp461e
         *RSe461p + 8./9.*m33*pow(m55,2)*L6r*RSp463e*RSe463p + 8./27.*
         m33*pow(m55,2)*L6r*RSe461p*RSp463e + 8./27.*m33*pow(m55,2)*L6r
         *RSp461e*RSe463p + 8./27.*m33*pow(m55,2)*L6r*RSp461e*RSe461p
          - 4./9.*m33*m44*me2*L5r*RSp563e*RSe563p - 4./27.*m33*m44*me2*
         L5r*RSe561p*RSp563e - 4./27.*m33*m44*me2*L5r*RSp561e*RSe563p
          - 4./27.*m33*m44*me2*L5r*RSp561e*RSe561p - 4./9.*m33*m44*me2*
         L4r*RSp563e*RSe563p - 4./9.*m33*m44*me2*L4r*RSp463e*RSe463p - 
         4./9.*m33*m44*me2*L4r*RSp453e*RSe453p - 4./27.*m33*m44*me2*L4r
         *RSe561p*RSp563e - 4./27.*m33*m44*me2*L4r*RSe461p*RSp463e - 4./
         27.*m33*m44*me2*L4r*RSe451p*RSp453e - 4./27.*m33*m44*me2*L4r*
         RSp561e*RSe563p - 4./27.*m33*m44*me2*L4r*RSp561e*RSe561p - 4./
         27.*m33*m44*me2*L4r*RSp461e*RSe463p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * (  - 4./27.*m33*m44*me2*L4r*
         RSp461e*RSe461p - 4./27.*m33*m44*me2*L4r*RSp451e*RSe453p - 4./
         27.*m33*m44*me2*L4r*RSp451e*RSe451p + 8./9.*m33*m44*m66*L7r*
         RSp563e*RSe453p + 8./9.*m33*m44*m66*L7r*RSp453e*RSe563p + 8./
         27.*m33*m44*m66*L7r*RSe561p*RSp453e + 8./27.*m33*m44*m66*L7r*
         RSe451p*RSp563e + 8./27.*m33*m44*m66*L7r*RSp561e*RSe453p + 8./
         27.*m33*m44*m66*L7r*RSp561e*RSe451p + 8./27.*m33*m44*m66*L7r*
         RSp451e*RSe563p + 8./27.*m33*m44*m66*L7r*RSp451e*RSe561p + 8./
         9.*m33*m44*m66*L6r*RSp563e*RSe563p + 8./9.*m33*m44*m66*L6r*
         RSp453e*RSe453p + 8./27.*m33*m44*m66*L6r*RSe561p*RSp563e + 8./
         27.*m33*m44*m66*L6r*RSe451p*RSp453e + 8./27.*m33*m44*m66*L6r*
         RSp561e*RSe563p + 8./27.*m33*m44*m66*L6r*RSp561e*RSe561p + 8./
         27.*m33*m44*m66*L6r*RSp451e*RSe453p + 8./27.*m33*m44*m66*L6r*
         RSp451e*RSe451p + 8./9.*m33*m44*m55*L7r*RSp563e*RSe463p + 8./9.
         *m33*m44*m55*L7r*RSp463e*RSe563p + 8./27.*m33*m44*m55*L7r*
         RSe561p*RSp463e );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 8./27.*m33*m44*m55*L7r*
         RSe461p*RSp563e + 8./27.*m33*m44*m55*L7r*RSp561e*RSe463p + 8./
         27.*m33*m44*m55*L7r*RSp561e*RSe461p + 8./27.*m33*m44*m55*L7r*
         RSp461e*RSe563p + 8./27.*m33*m44*m55*L7r*RSp461e*RSe561p + 8./
         9.*m33*m44*m55*L6r*RSp563e*RSe563p + 8./9.*m33*m44*m55*L6r*
         RSp463e*RSe463p + 8./27.*m33*m44*m55*L6r*RSe561p*RSp563e + 8./
         27.*m33*m44*m55*L6r*RSe461p*RSp463e + 8./27.*m33*m44*m55*L6r*
         RSp561e*RSe563p + 8./27.*m33*m44*m55*L6r*RSp561e*RSe561p + 8./
         27.*m33*m44*m55*L6r*RSp461e*RSe463p + 8./27.*m33*m44*m55*L6r*
         RSp461e*RSe461p + 8./9.*m33*pow(m44,2)*L8r*RSp563e*RSe563p + 8.
         /27.*m33*pow(m44,2)*L8r*RSe561p*RSp563e + 8./27.*m33*pow(
         m44,2)*L8r*RSp561e*RSe563p + 8./27.*m33*pow(m44,2)*L8r*RSp561e
         *RSe561p + 8./9.*m33*pow(m44,2)*L7r*RSp563e*RSe563p + 8./27.*
         m33*pow(m44,2)*L7r*RSe561p*RSp563e + 8./27.*m33*pow(m44,2)*L7r
         *RSp561e*RSe563p + 8./27.*m33*pow(m44,2)*L7r*RSp561e*RSe561p
          + 8./9.*m33*pow(m44,2)*L6r*RSp563e*RSe563p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 8./27.*m33*pow(m44,2)*L6r*
         RSe561p*RSp563e + 8./27.*m33*pow(m44,2)*L6r*RSp561e*RSe563p + 
         8./27.*m33*pow(m44,2)*L6r*RSp561e*RSe561p + 8./27.*m13*m66*me2
         *L5r*RSp453e*RSe453p - 8./27.*m13*m66*me2*L5r*RSe451p*RSp453e
          - 8./27.*m13*m66*me2*L5r*RSp451e*RSe453p + 8./27.*m13*m66*me2
         *L5r*RSp451e*RSe451p + 8./27.*m13*m66*me2*L4r*RSp563e*RSe563p
          + 8./27.*m13*m66*me2*L4r*RSp463e*RSe463p + 8./27.*m13*m66*me2
         *L4r*RSp453e*RSe453p - 8./27.*m13*m66*me2*L4r*RSe561p*RSp563e
          - 8./27.*m13*m66*me2*L4r*RSe461p*RSp463e - 8./27.*m13*m66*me2
         *L4r*RSe451p*RSp453e - 8./27.*m13*m66*me2*L4r*RSp561e*RSe563p
          + 8./27.*m13*m66*me2*L4r*RSp561e*RSe561p - 8./27.*m13*m66*me2
         *L4r*RSp461e*RSe463p + 8./27.*m13*m66*me2*L4r*RSp461e*RSe461p
          - 8./27.*m13*m66*me2*L4r*RSp451e*RSe453p + 8./27.*m13*m66*me2
         *L4r*RSp451e*RSe451p - 16./27.*m13*pow(m66,2)*L8r*RSp453e*
         RSe453p + 16./27.*m13*pow(m66,2)*L8r*RSe451p*RSp453e + 16./27.
         *m13*pow(m66,2)*L8r*RSp451e*RSe453p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * (  - 16./27.*m13*pow(m66,2)*L8r
         *RSp451e*RSe451p - 16./27.*m13*pow(m66,2)*L7r*RSp453e*RSe453p
          + 16./27.*m13*pow(m66,2)*L7r*RSe451p*RSp453e + 16./27.*m13*
         pow(m66,2)*L7r*RSp451e*RSe453p - 16./27.*m13*pow(m66,2)*L7r*
         RSp451e*RSe451p - 16./27.*m13*pow(m66,2)*L6r*RSp453e*RSe453p
          + 16./27.*m13*pow(m66,2)*L6r*RSe451p*RSp453e + 16./27.*m13*
         pow(m66,2)*L6r*RSp451e*RSe453p - 16./27.*m13*pow(m66,2)*L6r*
         RSp451e*RSe451p + 8./27.*m13*m55*me2*L5r*RSp463e*RSe463p - 8./
         27.*m13*m55*me2*L5r*RSe461p*RSp463e - 8./27.*m13*m55*me2*L5r*
         RSp461e*RSe463p + 8./27.*m13*m55*me2*L5r*RSp461e*RSe461p + 8./
         27.*m13*m55*me2*L4r*RSp563e*RSe563p + 8./27.*m13*m55*me2*L4r*
         RSp463e*RSe463p + 8./27.*m13*m55*me2*L4r*RSp453e*RSe453p - 8./
         27.*m13*m55*me2*L4r*RSe561p*RSp563e - 8./27.*m13*m55*me2*L4r*
         RSe461p*RSp463e - 8./27.*m13*m55*me2*L4r*RSe451p*RSp453e - 8./
         27.*m13*m55*me2*L4r*RSp561e*RSe563p + 8./27.*m13*m55*me2*L4r*
         RSp561e*RSe561p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * (  - 8./27.*m13*m55*me2*L4r*
         RSp461e*RSe463p + 8./27.*m13*m55*me2*L4r*RSp461e*RSe461p - 8./
         27.*m13*m55*me2*L4r*RSp451e*RSe453p + 8./27.*m13*m55*me2*L4r*
         RSp451e*RSe451p - 16./27.*m13*m55*m66*L7r*RSp463e*RSe453p - 16.
         /27.*m13*m55*m66*L7r*RSp453e*RSe463p + 16./27.*m13*m55*m66*L7r
         *RSe461p*RSp453e + 16./27.*m13*m55*m66*L7r*RSe451p*RSp463e + 
         16./27.*m13*m55*m66*L7r*RSp461e*RSe453p - 16./27.*m13*m55*m66*
         L7r*RSp461e*RSe451p + 16./27.*m13*m55*m66*L7r*RSp451e*RSe463p
          - 16./27.*m13*m55*m66*L7r*RSp451e*RSe461p - 16./27.*m13*m55*
         m66*L6r*RSp463e*RSe463p - 16./27.*m13*m55*m66*L6r*RSp453e*
         RSe453p + 16./27.*m13*m55*m66*L6r*RSe461p*RSp463e + 16./27.*
         m13*m55*m66*L6r*RSe451p*RSp453e + 16./27.*m13*m55*m66*L6r*
         RSp461e*RSe463p - 16./27.*m13*m55*m66*L6r*RSp461e*RSe461p + 16.
         /27.*m13*m55*m66*L6r*RSp451e*RSe453p - 16./27.*m13*m55*m66*L6r
         *RSp451e*RSe451p - 16./27.*m13*pow(m55,2)*L8r*RSp463e*RSe463p
          + 16./27.*m13*pow(m55,2)*L8r*RSe461p*RSp463e );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 16./27.*m13*pow(m55,2)*L8r*
         RSp461e*RSe463p - 16./27.*m13*pow(m55,2)*L8r*RSp461e*RSe461p
          - 16./27.*m13*pow(m55,2)*L7r*RSp463e*RSe463p + 16./27.*m13*
         pow(m55,2)*L7r*RSe461p*RSp463e + 16./27.*m13*pow(m55,2)*L7r*
         RSp461e*RSe463p - 16./27.*m13*pow(m55,2)*L7r*RSp461e*RSe461p
          - 16./27.*m13*pow(m55,2)*L6r*RSp463e*RSe463p + 16./27.*m13*
         pow(m55,2)*L6r*RSe461p*RSp463e + 16./27.*m13*pow(m55,2)*L6r*
         RSp461e*RSe463p - 16./27.*m13*pow(m55,2)*L6r*RSp461e*RSe461p
          + 8./27.*m13*m44*me2*L5r*RSp563e*RSe563p - 8./27.*m13*m44*me2
         *L5r*RSe561p*RSp563e - 8./27.*m13*m44*me2*L5r*RSp561e*RSe563p
          + 8./27.*m13*m44*me2*L5r*RSp561e*RSe561p + 8./27.*m13*m44*me2
         *L4r*RSp563e*RSe563p + 8./27.*m13*m44*me2*L4r*RSp463e*RSe463p
          + 8./27.*m13*m44*me2*L4r*RSp453e*RSe453p - 8./27.*m13*m44*me2
         *L4r*RSe561p*RSp563e - 8./27.*m13*m44*me2*L4r*RSe461p*RSp463e
          - 8./27.*m13*m44*me2*L4r*RSe451p*RSp453e - 8./27.*m13*m44*me2
         *L4r*RSp561e*RSe563p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 8./27.*m13*m44*me2*L4r*
         RSp561e*RSe561p - 8./27.*m13*m44*me2*L4r*RSp461e*RSe463p + 8./
         27.*m13*m44*me2*L4r*RSp461e*RSe461p - 8./27.*m13*m44*me2*L4r*
         RSp451e*RSe453p + 8./27.*m13*m44*me2*L4r*RSp451e*RSe451p - 16./
         27.*m13*m44*m66*L7r*RSp563e*RSe453p - 16./27.*m13*m44*m66*L7r*
         RSp453e*RSe563p + 16./27.*m13*m44*m66*L7r*RSe561p*RSp453e + 16.
         /27.*m13*m44*m66*L7r*RSe451p*RSp563e + 16./27.*m13*m44*m66*L7r
         *RSp561e*RSe453p - 16./27.*m13*m44*m66*L7r*RSp561e*RSe451p + 
         16./27.*m13*m44*m66*L7r*RSp451e*RSe563p - 16./27.*m13*m44*m66*
         L7r*RSp451e*RSe561p - 16./27.*m13*m44*m66*L6r*RSp563e*RSe563p
          - 16./27.*m13*m44*m66*L6r*RSp453e*RSe453p + 16./27.*m13*m44*
         m66*L6r*RSe561p*RSp563e + 16./27.*m13*m44*m66*L6r*RSe451p*
         RSp453e + 16./27.*m13*m44*m66*L6r*RSp561e*RSe563p - 16./27.*
         m13*m44*m66*L6r*RSp561e*RSe561p + 16./27.*m13*m44*m66*L6r*
         RSp451e*RSe453p - 16./27.*m13*m44*m66*L6r*RSp451e*RSe451p - 16.
         /27.*m13*m44*m55*L7r*RSp563e*RSe463p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * (  - 16./27.*m13*m44*m55*L7r*
         RSp463e*RSe563p + 16./27.*m13*m44*m55*L7r*RSe561p*RSp463e + 16.
         /27.*m13*m44*m55*L7r*RSe461p*RSp563e + 16./27.*m13*m44*m55*L7r
         *RSp561e*RSe463p - 16./27.*m13*m44*m55*L7r*RSp561e*RSe461p + 
         16./27.*m13*m44*m55*L7r*RSp461e*RSe563p - 16./27.*m13*m44*m55*
         L7r*RSp461e*RSe561p - 16./27.*m13*m44*m55*L6r*RSp563e*RSe563p
          - 16./27.*m13*m44*m55*L6r*RSp463e*RSe463p + 16./27.*m13*m44*
         m55*L6r*RSe561p*RSp563e + 16./27.*m13*m44*m55*L6r*RSe461p*
         RSp463e + 16./27.*m13*m44*m55*L6r*RSp561e*RSe563p - 16./27.*
         m13*m44*m55*L6r*RSp561e*RSe561p + 16./27.*m13*m44*m55*L6r*
         RSp461e*RSe463p - 16./27.*m13*m44*m55*L6r*RSp461e*RSe461p - 16.
         /27.*m13*pow(m44,2)*L8r*RSp563e*RSe563p + 16./27.*m13*pow(
         m44,2)*L8r*RSe561p*RSp563e + 16./27.*m13*pow(m44,2)*L8r*
         RSp561e*RSe563p - 16./27.*m13*pow(m44,2)*L8r*RSp561e*RSe561p
          - 16./27.*m13*pow(m44,2)*L7r*RSp563e*RSe563p + 16./27.*m13*
         pow(m44,2)*L7r*RSe561p*RSp563e );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 16./27.*m13*pow(m44,2)*L7r*
         RSp561e*RSe563p - 16./27.*m13*pow(m44,2)*L7r*RSp561e*RSe561p
          - 16./27.*m13*pow(m44,2)*L6r*RSp563e*RSe563p + 16./27.*m13*
         pow(m44,2)*L6r*RSe561p*RSp563e + 16./27.*m13*pow(m44,2)*L6r*
         RSp561e*RSe563p - 16./27.*m13*pow(m44,2)*L6r*RSp561e*RSe561p
          - 4./27.*m11*m66*me2*L5r*RSp453e*RSe453p - 4./27.*m11*m66*me2
         *L5r*RSe451p*RSp453e - 4./27.*m11*m66*me2*L5r*RSp451e*RSe453p
          - 4./9.*m11*m66*me2*L5r*RSp451e*RSe451p - 4./27.*m11*m66*me2*
         L4r*RSp563e*RSe563p - 4./27.*m11*m66*me2*L4r*RSp463e*RSe463p
          - 4./27.*m11*m66*me2*L4r*RSp453e*RSe453p - 4./27.*m11*m66*me2
         *L4r*RSe561p*RSp563e - 4./27.*m11*m66*me2*L4r*RSe461p*RSp463e
          - 4./27.*m11*m66*me2*L4r*RSe451p*RSp453e - 4./27.*m11*m66*me2
         *L4r*RSp561e*RSe563p - 4./9.*m11*m66*me2*L4r*RSp561e*RSe561p
          - 4./27.*m11*m66*me2*L4r*RSp461e*RSe463p - 4./9.*m11*m66*me2*
         L4r*RSp461e*RSe461p - 4./27.*m11*m66*me2*L4r*RSp451e*RSe453p
          - 4./9.*m11*m66*me2*L4r*RSp451e*RSe451p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 8./27.*m11*pow(m66,2)*L8r*
         RSp453e*RSe453p + 8./27.*m11*pow(m66,2)*L8r*RSe451p*RSp453e + 
         8./27.*m11*pow(m66,2)*L8r*RSp451e*RSe453p + 8./9.*m11*pow(
         m66,2)*L8r*RSp451e*RSe451p + 8./27.*m11*pow(m66,2)*L7r*RSp453e
         *RSe453p + 8./27.*m11*pow(m66,2)*L7r*RSe451p*RSp453e + 8./27.*
         m11*pow(m66,2)*L7r*RSp451e*RSe453p + 8./9.*m11*pow(m66,2)*L7r*
         RSp451e*RSe451p + 8./27.*m11*pow(m66,2)*L6r*RSp453e*RSe453p + 
         8./27.*m11*pow(m66,2)*L6r*RSe451p*RSp453e + 8./27.*m11*pow(
         m66,2)*L6r*RSp451e*RSe453p + 8./9.*m11*pow(m66,2)*L6r*RSp451e*
         RSe451p - 4./27.*m11*m55*me2*L5r*RSp463e*RSe463p - 4./27.*m11*
         m55*me2*L5r*RSe461p*RSp463e - 4./27.*m11*m55*me2*L5r*RSp461e*
         RSe463p - 4./9.*m11*m55*me2*L5r*RSp461e*RSe461p - 4./27.*m11*
         m55*me2*L4r*RSp563e*RSe563p - 4./27.*m11*m55*me2*L4r*RSp463e*
         RSe463p - 4./27.*m11*m55*me2*L4r*RSp453e*RSe453p - 4./27.*m11*
         m55*me2*L4r*RSe561p*RSp563e - 4./27.*m11*m55*me2*L4r*RSe461p*
         RSp463e );

      M13p6x23LV +=  + BbV(me2,mp2,L) * (  - 4./27.*m11*m55*me2*L4r*
         RSe451p*RSp453e - 4./27.*m11*m55*me2*L4r*RSp561e*RSe563p - 4./
         9.*m11*m55*me2*L4r*RSp561e*RSe561p - 4./27.*m11*m55*me2*L4r*
         RSp461e*RSe463p - 4./9.*m11*m55*me2*L4r*RSp461e*RSe461p - 4./
         27.*m11*m55*me2*L4r*RSp451e*RSe453p - 4./9.*m11*m55*me2*L4r*
         RSp451e*RSe451p + 8./27.*m11*m55*m66*L7r*RSp463e*RSe453p + 8./
         27.*m11*m55*m66*L7r*RSp453e*RSe463p + 8./27.*m11*m55*m66*L7r*
         RSe461p*RSp453e + 8./27.*m11*m55*m66*L7r*RSe451p*RSp463e + 8./
         27.*m11*m55*m66*L7r*RSp461e*RSe453p + 8./9.*m11*m55*m66*L7r*
         RSp461e*RSe451p + 8./27.*m11*m55*m66*L7r*RSp451e*RSe463p + 8./
         9.*m11*m55*m66*L7r*RSp451e*RSe461p + 8./27.*m11*m55*m66*L6r*
         RSp463e*RSe463p + 8./27.*m11*m55*m66*L6r*RSp453e*RSe453p + 8./
         27.*m11*m55*m66*L6r*RSe461p*RSp463e + 8./27.*m11*m55*m66*L6r*
         RSe451p*RSp453e + 8./27.*m11*m55*m66*L6r*RSp461e*RSe463p + 8./
         9.*m11*m55*m66*L6r*RSp461e*RSe461p + 8./27.*m11*m55*m66*L6r*
         RSp451e*RSe453p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 8./9.*m11*m55*m66*L6r*RSp451e
         *RSe451p + 8./27.*m11*pow(m55,2)*L8r*RSp463e*RSe463p + 8./27.*
         m11*pow(m55,2)*L8r*RSe461p*RSp463e + 8./27.*m11*pow(m55,2)*L8r
         *RSp461e*RSe463p + 8./9.*m11*pow(m55,2)*L8r*RSp461e*RSe461p + 
         8./27.*m11*pow(m55,2)*L7r*RSp463e*RSe463p + 8./27.*m11*pow(
         m55,2)*L7r*RSe461p*RSp463e + 8./27.*m11*pow(m55,2)*L7r*RSp461e
         *RSe463p + 8./9.*m11*pow(m55,2)*L7r*RSp461e*RSe461p + 8./27.*
         m11*pow(m55,2)*L6r*RSp463e*RSe463p + 8./27.*m11*pow(m55,2)*L6r
         *RSe461p*RSp463e + 8./27.*m11*pow(m55,2)*L6r*RSp461e*RSe463p
          + 8./9.*m11*pow(m55,2)*L6r*RSp461e*RSe461p - 4./27.*m11*m44*
         me2*L5r*RSp563e*RSe563p - 4./27.*m11*m44*me2*L5r*RSe561p*
         RSp563e - 4./27.*m11*m44*me2*L5r*RSp561e*RSe563p - 4./9.*m11*
         m44*me2*L5r*RSp561e*RSe561p - 4./27.*m11*m44*me2*L4r*RSp563e*
         RSe563p - 4./27.*m11*m44*me2*L4r*RSp463e*RSe463p - 4./27.*m11*
         m44*me2*L4r*RSp453e*RSe453p - 4./27.*m11*m44*me2*L4r*RSe561p*
         RSp563e );

      M13p6x23LV +=  + BbV(me2,mp2,L) * (  - 4./27.*m11*m44*me2*L4r*
         RSe461p*RSp463e - 4./27.*m11*m44*me2*L4r*RSe451p*RSp453e - 4./
         27.*m11*m44*me2*L4r*RSp561e*RSe563p - 4./9.*m11*m44*me2*L4r*
         RSp561e*RSe561p - 4./27.*m11*m44*me2*L4r*RSp461e*RSe463p - 4./
         9.*m11*m44*me2*L4r*RSp461e*RSe461p - 4./27.*m11*m44*me2*L4r*
         RSp451e*RSe453p - 4./9.*m11*m44*me2*L4r*RSp451e*RSe451p + 8./
         27.*m11*m44*m66*L7r*RSp563e*RSe453p + 8./27.*m11*m44*m66*L7r*
         RSp453e*RSe563p + 8./27.*m11*m44*m66*L7r*RSe561p*RSp453e + 8./
         27.*m11*m44*m66*L7r*RSe451p*RSp563e + 8./27.*m11*m44*m66*L7r*
         RSp561e*RSe453p + 8./9.*m11*m44*m66*L7r*RSp561e*RSe451p + 8./
         27.*m11*m44*m66*L7r*RSp451e*RSe563p + 8./9.*m11*m44*m66*L7r*
         RSp451e*RSe561p + 8./27.*m11*m44*m66*L6r*RSp563e*RSe563p + 8./
         27.*m11*m44*m66*L6r*RSp453e*RSe453p + 8./27.*m11*m44*m66*L6r*
         RSe561p*RSp563e + 8./27.*m11*m44*m66*L6r*RSe451p*RSp453e + 8./
         27.*m11*m44*m66*L6r*RSp561e*RSe563p + 8./9.*m11*m44*m66*L6r*
         RSp561e*RSe561p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 8./27.*m11*m44*m66*L6r*
         RSp451e*RSe453p + 8./9.*m11*m44*m66*L6r*RSp451e*RSe451p + 8./
         27.*m11*m44*m55*L7r*RSp563e*RSe463p + 8./27.*m11*m44*m55*L7r*
         RSp463e*RSe563p + 8./27.*m11*m44*m55*L7r*RSe561p*RSp463e + 8./
         27.*m11*m44*m55*L7r*RSe461p*RSp563e + 8./27.*m11*m44*m55*L7r*
         RSp561e*RSe463p + 8./9.*m11*m44*m55*L7r*RSp561e*RSe461p + 8./
         27.*m11*m44*m55*L7r*RSp461e*RSe563p + 8./9.*m11*m44*m55*L7r*
         RSp461e*RSe561p + 8./27.*m11*m44*m55*L6r*RSp563e*RSe563p + 8./
         27.*m11*m44*m55*L6r*RSp463e*RSe463p + 8./27.*m11*m44*m55*L6r*
         RSe561p*RSp563e + 8./27.*m11*m44*m55*L6r*RSe461p*RSp463e + 8./
         27.*m11*m44*m55*L6r*RSp561e*RSe563p + 8./9.*m11*m44*m55*L6r*
         RSp561e*RSe561p + 8./27.*m11*m44*m55*L6r*RSp461e*RSe463p + 8./
         9.*m11*m44*m55*L6r*RSp461e*RSe461p + 8./27.*m11*pow(m44,2)*L8r
         *RSp563e*RSe563p + 8./27.*m11*pow(m44,2)*L8r*RSe561p*RSp563e
          + 8./27.*m11*pow(m44,2)*L8r*RSp561e*RSe563p + 8./9.*m11*pow(
         m44,2)*L8r*RSp561e*RSe561p );

      M13p6x23LV +=  + BbV(me2,mp2,L) * ( 8./27.*m11*pow(m44,2)*L7r*
         RSp563e*RSe563p + 8./27.*m11*pow(m44,2)*L7r*RSe561p*RSp563e + 
         8./27.*m11*pow(m44,2)*L7r*RSp561e*RSe563p + 8./9.*m11*pow(
         m44,2)*L7r*RSp561e*RSe561p + 8./27.*m11*pow(m44,2)*L6r*RSp563e
         *RSe563p + 8./27.*m11*pow(m44,2)*L6r*RSe561p*RSp563e + 8./27.*
         m11*pow(m44,2)*L6r*RSp561e*RSe563p + 8./9.*m11*pow(m44,2)*L6r*
         RSp561e*RSe561p );

      M13p6x23LV +=  + BbV(me2,L) * ( 4./27.*m66*pow(me2,2)*L5r*pow(
         RSe453p,2) - 8./27.*m66*pow(me2,2)*L5r*RSe451p*RSe453p + 4./27.
         *m66*pow(me2,2)*L5r*pow(RSe451p,2) + 4./27.*m66*pow(me2,2)*L4r
         *pow(RSe563p,2) + 4./27.*m66*pow(me2,2)*L4r*pow(RSe463p,2) + 4.
         /27.*m66*pow(me2,2)*L4r*pow(RSe453p,2) - 8./27.*m66*pow(me2,2)
         *L4r*RSe561p*RSe563p + 4./27.*m66*pow(me2,2)*L4r*pow(
         RSe561p,2) - 8./27.*m66*pow(me2,2)*L4r*RSe461p*RSe463p + 4./27.
         *m66*pow(me2,2)*L4r*pow(RSe461p,2) - 8./27.*m66*pow(me2,2)*L4r
         *RSe451p*RSe453p + 4./27.*m66*pow(me2,2)*L4r*pow(RSe451p,2) - 
         8./27.*pow(m66,2)*me2*L8r*pow(RSe453p,2) + 16./27.*pow(m66,2)*
         me2*L8r*RSe451p*RSe453p - 8./27.*pow(m66,2)*me2*L8r*pow(
         RSe451p,2) - 8./27.*pow(m66,2)*me2*L7r*pow(RSe453p,2) + 16./27.
         *pow(m66,2)*me2*L7r*RSe451p*RSe453p - 8./27.*pow(m66,2)*me2*
         L7r*pow(RSe451p,2) - 8./27.*pow(m66,2)*me2*L6r*pow(RSe453p,2)
          + 16./27.*pow(m66,2)*me2*L6r*RSe451p*RSe453p - 8./27.*pow(
         m66,2)*me2*L6r*pow(RSe451p,2) );

      M13p6x23LV +=  + BbV(me2,L) * ( 4./27.*m55*pow(me2,2)*L5r*pow(
         RSe463p,2) - 8./27.*m55*pow(me2,2)*L5r*RSe461p*RSe463p + 4./27.
         *m55*pow(me2,2)*L5r*pow(RSe461p,2) + 4./27.*m55*pow(me2,2)*L4r
         *pow(RSe563p,2) + 4./27.*m55*pow(me2,2)*L4r*pow(RSe463p,2) + 4.
         /27.*m55*pow(me2,2)*L4r*pow(RSe453p,2) - 8./27.*m55*pow(me2,2)
         *L4r*RSe561p*RSe563p + 4./27.*m55*pow(me2,2)*L4r*pow(
         RSe561p,2) - 8./27.*m55*pow(me2,2)*L4r*RSe461p*RSe463p + 4./27.
         *m55*pow(me2,2)*L4r*pow(RSe461p,2) - 8./27.*m55*pow(me2,2)*L4r
         *RSe451p*RSe453p + 4./27.*m55*pow(me2,2)*L4r*pow(RSe451p,2) - 
         16./27.*m55*m66*me2*L7r*RSe453p*RSe463p + 16./27.*m55*m66*me2*
         L7r*RSe461p*RSe453p + 16./27.*m55*m66*me2*L7r*RSe451p*RSe463p
          - 16./27.*m55*m66*me2*L7r*RSe451p*RSe461p - 8./27.*m55*m66*
         me2*L6r*pow(RSe463p,2) - 8./27.*m55*m66*me2*L6r*pow(RSe453p,2)
          + 16./27.*m55*m66*me2*L6r*RSe461p*RSe463p - 8./27.*m55*m66*
         me2*L6r*pow(RSe461p,2) + 16./27.*m55*m66*me2*L6r*RSe451p*
         RSe453p );

      M13p6x23LV +=  + BbV(me2,L) * (  - 8./27.*m55*m66*me2*L6r*pow(
         RSe451p,2) - 8./27.*pow(m55,2)*me2*L8r*pow(RSe463p,2) + 16./27.
         *pow(m55,2)*me2*L8r*RSe461p*RSe463p - 8./27.*pow(m55,2)*me2*
         L8r*pow(RSe461p,2) - 8./27.*pow(m55,2)*me2*L7r*pow(RSe463p,2)
          + 16./27.*pow(m55,2)*me2*L7r*RSe461p*RSe463p - 8./27.*pow(
         m55,2)*me2*L7r*pow(RSe461p,2) - 8./27.*pow(m55,2)*me2*L6r*pow(
         RSe463p,2) + 16./27.*pow(m55,2)*me2*L6r*RSe461p*RSe463p - 8./
         27.*pow(m55,2)*me2*L6r*pow(RSe461p,2) + 4./27.*m44*pow(me2,2)*
         L5r*pow(RSe563p,2) - 8./27.*m44*pow(me2,2)*L5r*RSe561p*RSe563p
          + 4./27.*m44*pow(me2,2)*L5r*pow(RSe561p,2) + 4./27.*m44*pow(
         me2,2)*L4r*pow(RSe563p,2) + 4./27.*m44*pow(me2,2)*L4r*pow(
         RSe463p,2) + 4./27.*m44*pow(me2,2)*L4r*pow(RSe453p,2) - 8./27.
         *m44*pow(me2,2)*L4r*RSe561p*RSe563p + 4./27.*m44*pow(me2,2)*
         L4r*pow(RSe561p,2) - 8./27.*m44*pow(me2,2)*L4r*RSe461p*RSe463p
          + 4./27.*m44*pow(me2,2)*L4r*pow(RSe461p,2) - 8./27.*m44*pow(
         me2,2)*L4r*RSe451p*RSe453p );

      M13p6x23LV +=  + BbV(me2,L) * ( 4./27.*m44*pow(me2,2)*L4r*pow(
         RSe451p,2) - 16./27.*m44*m66*me2*L7r*RSe453p*RSe563p + 16./27.
         *m44*m66*me2*L7r*RSe561p*RSe453p + 16./27.*m44*m66*me2*L7r*
         RSe451p*RSe563p - 16./27.*m44*m66*me2*L7r*RSe451p*RSe561p - 8./
         27.*m44*m66*me2*L6r*pow(RSe563p,2) - 8./27.*m44*m66*me2*L6r*
         pow(RSe453p,2) + 16./27.*m44*m66*me2*L6r*RSe561p*RSe563p - 8./
         27.*m44*m66*me2*L6r*pow(RSe561p,2) + 16./27.*m44*m66*me2*L6r*
         RSe451p*RSe453p - 8./27.*m44*m66*me2*L6r*pow(RSe451p,2) - 16./
         27.*m44*m55*me2*L7r*RSe463p*RSe563p + 16./27.*m44*m55*me2*L7r*
         RSe561p*RSe463p + 16./27.*m44*m55*me2*L7r*RSe461p*RSe563p - 16.
         /27.*m44*m55*me2*L7r*RSe461p*RSe561p - 8./27.*m44*m55*me2*L6r*
         pow(RSe563p,2) - 8./27.*m44*m55*me2*L6r*pow(RSe463p,2) + 16./
         27.*m44*m55*me2*L6r*RSe561p*RSe563p - 8./27.*m44*m55*me2*L6r*
         pow(RSe561p,2) + 16./27.*m44*m55*me2*L6r*RSe461p*RSe463p - 8./
         27.*m44*m55*me2*L6r*pow(RSe461p,2) - 8./27.*pow(m44,2)*me2*L8r
         *pow(RSe563p,2) );

      M13p6x23LV +=  + BbV(me2,L) * ( 16./27.*pow(m44,2)*me2*L8r*
         RSe561p*RSe563p - 8./27.*pow(m44,2)*me2*L8r*pow(RSe561p,2) - 8.
         /27.*pow(m44,2)*me2*L7r*pow(RSe563p,2) + 16./27.*pow(m44,2)*
         me2*L7r*RSe561p*RSe563p - 8./27.*pow(m44,2)*me2*L7r*pow(
         RSe561p,2) - 8./27.*pow(m44,2)*me2*L6r*pow(RSe563p,2) + 16./27.
         *pow(m44,2)*me2*L6r*RSe561p*RSe563p - 8./27.*pow(m44,2)*me2*
         L6r*pow(RSe561p,2) - 2./9.*m33*m66*me2*L5r*pow(RSe453p,2) - 4./
         27.*m33*m66*me2*L5r*RSe451p*RSe453p - 2./27.*m33*m66*me2*L5r*
         pow(RSe451p,2) - 2./9.*m33*m66*me2*L4r*pow(RSe563p,2) - 2./9.*
         m33*m66*me2*L4r*pow(RSe463p,2) - 2./9.*m33*m66*me2*L4r*pow(
         RSe453p,2) - 4./27.*m33*m66*me2*L4r*RSe561p*RSe563p - 2./27.*
         m33*m66*me2*L4r*pow(RSe561p,2) - 4./27.*m33*m66*me2*L4r*
         RSe461p*RSe463p - 2./27.*m33*m66*me2*L4r*pow(RSe461p,2) - 4./
         27.*m33*m66*me2*L4r*RSe451p*RSe453p - 2./27.*m33*m66*me2*L4r*
         pow(RSe451p,2) + 4./9.*m33*pow(m66,2)*L8r*pow(RSe453p,2) + 8./
         27.*m33*pow(m66,2)*L8r*RSe451p*RSe453p );

      M13p6x23LV +=  + BbV(me2,L) * ( 4./27.*m33*pow(m66,2)*L8r*pow(
         RSe451p,2) + 4./9.*m33*pow(m66,2)*L7r*pow(RSe453p,2) + 8./27.*
         m33*pow(m66,2)*L7r*RSe451p*RSe453p + 4./27.*m33*pow(m66,2)*L7r
         *pow(RSe451p,2) + 4./9.*m33*pow(m66,2)*L6r*pow(RSe453p,2) + 8./
         27.*m33*pow(m66,2)*L6r*RSe451p*RSe453p + 4./27.*m33*pow(m66,2)
         *L6r*pow(RSe451p,2) - 2./9.*m33*m55*me2*L5r*pow(RSe463p,2) - 4.
         /27.*m33*m55*me2*L5r*RSe461p*RSe463p - 2./27.*m33*m55*me2*L5r*
         pow(RSe461p,2) - 2./9.*m33*m55*me2*L4r*pow(RSe563p,2) - 2./9.*
         m33*m55*me2*L4r*pow(RSe463p,2) - 2./9.*m33*m55*me2*L4r*pow(
         RSe453p,2) - 4./27.*m33*m55*me2*L4r*RSe561p*RSe563p - 2./27.*
         m33*m55*me2*L4r*pow(RSe561p,2) - 4./27.*m33*m55*me2*L4r*
         RSe461p*RSe463p - 2./27.*m33*m55*me2*L4r*pow(RSe461p,2) - 4./
         27.*m33*m55*me2*L4r*RSe451p*RSe453p - 2./27.*m33*m55*me2*L4r*
         pow(RSe451p,2) + 8./9.*m33*m55*m66*L7r*RSe453p*RSe463p + 8./27.
         *m33*m55*m66*L7r*RSe461p*RSe453p + 8./27.*m33*m55*m66*L7r*
         RSe451p*RSe463p );

      M13p6x23LV +=  + BbV(me2,L) * ( 8./27.*m33*m55*m66*L7r*RSe451p*
         RSe461p + 4./9.*m33*m55*m66*L6r*pow(RSe463p,2) + 4./9.*m33*m55
         *m66*L6r*pow(RSe453p,2) + 8./27.*m33*m55*m66*L6r*RSe461p*
         RSe463p + 4./27.*m33*m55*m66*L6r*pow(RSe461p,2) + 8./27.*m33*
         m55*m66*L6r*RSe451p*RSe453p + 4./27.*m33*m55*m66*L6r*pow(
         RSe451p,2) + 4./9.*m33*pow(m55,2)*L8r*pow(RSe463p,2) + 8./27.*
         m33*pow(m55,2)*L8r*RSe461p*RSe463p + 4./27.*m33*pow(m55,2)*L8r
         *pow(RSe461p,2) + 4./9.*m33*pow(m55,2)*L7r*pow(RSe463p,2) + 8./
         27.*m33*pow(m55,2)*L7r*RSe461p*RSe463p + 4./27.*m33*pow(m55,2)
         *L7r*pow(RSe461p,2) + 4./9.*m33*pow(m55,2)*L6r*pow(RSe463p,2)
          + 8./27.*m33*pow(m55,2)*L6r*RSe461p*RSe463p + 4./27.*m33*pow(
         m55,2)*L6r*pow(RSe461p,2) - 2./9.*m33*m44*me2*L5r*pow(
         RSe563p,2) - 4./27.*m33*m44*me2*L5r*RSe561p*RSe563p - 2./27.*
         m33*m44*me2*L5r*pow(RSe561p,2) - 2./9.*m33*m44*me2*L4r*pow(
         RSe563p,2) - 2./9.*m33*m44*me2*L4r*pow(RSe463p,2) - 2./9.*m33*
         m44*me2*L4r*pow(RSe453p,2) );

      M13p6x23LV +=  + BbV(me2,L) * (  - 4./27.*m33*m44*me2*L4r*RSe561p
         *RSe563p - 2./27.*m33*m44*me2*L4r*pow(RSe561p,2) - 4./27.*m33*
         m44*me2*L4r*RSe461p*RSe463p - 2./27.*m33*m44*me2*L4r*pow(
         RSe461p,2) - 4./27.*m33*m44*me2*L4r*RSe451p*RSe453p - 2./27.*
         m33*m44*me2*L4r*pow(RSe451p,2) + 8./9.*m33*m44*m66*L7r*RSe453p
         *RSe563p + 8./27.*m33*m44*m66*L7r*RSe561p*RSe453p + 8./27.*m33
         *m44*m66*L7r*RSe451p*RSe563p + 8./27.*m33*m44*m66*L7r*RSe451p*
         RSe561p + 4./9.*m33*m44*m66*L6r*pow(RSe563p,2) + 4./9.*m33*m44
         *m66*L6r*pow(RSe453p,2) + 8./27.*m33*m44*m66*L6r*RSe561p*
         RSe563p + 4./27.*m33*m44*m66*L6r*pow(RSe561p,2) + 8./27.*m33*
         m44*m66*L6r*RSe451p*RSe453p + 4./27.*m33*m44*m66*L6r*pow(
         RSe451p,2) + 8./9.*m33*m44*m55*L7r*RSe463p*RSe563p + 8./27.*
         m33*m44*m55*L7r*RSe561p*RSe463p + 8./27.*m33*m44*m55*L7r*
         RSe461p*RSe563p + 8./27.*m33*m44*m55*L7r*RSe461p*RSe561p + 4./
         9.*m33*m44*m55*L6r*pow(RSe563p,2) + 4./9.*m33*m44*m55*L6r*pow(
         RSe463p,2) );

      M13p6x23LV +=  + BbV(me2,L) * ( 8./27.*m33*m44*m55*L6r*RSe561p*
         RSe563p + 4./27.*m33*m44*m55*L6r*pow(RSe561p,2) + 8./27.*m33*
         m44*m55*L6r*RSe461p*RSe463p + 4./27.*m33*m44*m55*L6r*pow(
         RSe461p,2) + 4./9.*m33*pow(m44,2)*L8r*pow(RSe563p,2) + 8./27.*
         m33*pow(m44,2)*L8r*RSe561p*RSe563p + 4./27.*m33*pow(m44,2)*L8r
         *pow(RSe561p,2) + 4./9.*m33*pow(m44,2)*L7r*pow(RSe563p,2) + 8./
         27.*m33*pow(m44,2)*L7r*RSe561p*RSe563p + 4./27.*m33*pow(m44,2)
         *L7r*pow(RSe561p,2) + 4./9.*m33*pow(m44,2)*L6r*pow(RSe563p,2)
          + 8./27.*m33*pow(m44,2)*L6r*RSe561p*RSe563p + 4./27.*m33*pow(
         m44,2)*L6r*pow(RSe561p,2) + 4./27.*m13*m66*me2*L5r*pow(
         RSe453p,2) - 8./27.*m13*m66*me2*L5r*RSe451p*RSe453p + 4./27.*
         m13*m66*me2*L5r*pow(RSe451p,2) + 4./27.*m13*m66*me2*L4r*pow(
         RSe563p,2) + 4./27.*m13*m66*me2*L4r*pow(RSe463p,2) + 4./27.*
         m13*m66*me2*L4r*pow(RSe453p,2) - 8./27.*m13*m66*me2*L4r*
         RSe561p*RSe563p + 4./27.*m13*m66*me2*L4r*pow(RSe561p,2) - 8./
         27.*m13*m66*me2*L4r*RSe461p*RSe463p );

      M13p6x23LV +=  + BbV(me2,L) * ( 4./27.*m13*m66*me2*L4r*pow(
         RSe461p,2) - 8./27.*m13*m66*me2*L4r*RSe451p*RSe453p + 4./27.*
         m13*m66*me2*L4r*pow(RSe451p,2) - 8./27.*m13*pow(m66,2)*L8r*
         pow(RSe453p,2) + 16./27.*m13*pow(m66,2)*L8r*RSe451p*RSe453p - 
         8./27.*m13*pow(m66,2)*L8r*pow(RSe451p,2) - 8./27.*m13*pow(
         m66,2)*L7r*pow(RSe453p,2) + 16./27.*m13*pow(m66,2)*L7r*RSe451p
         *RSe453p - 8./27.*m13*pow(m66,2)*L7r*pow(RSe451p,2) - 8./27.*
         m13*pow(m66,2)*L6r*pow(RSe453p,2) + 16./27.*m13*pow(m66,2)*L6r
         *RSe451p*RSe453p - 8./27.*m13*pow(m66,2)*L6r*pow(RSe451p,2) + 
         4./27.*m13*m55*me2*L5r*pow(RSe463p,2) - 8./27.*m13*m55*me2*L5r
         *RSe461p*RSe463p + 4./27.*m13*m55*me2*L5r*pow(RSe461p,2) + 4./
         27.*m13*m55*me2*L4r*pow(RSe563p,2) + 4./27.*m13*m55*me2*L4r*
         pow(RSe463p,2) + 4./27.*m13*m55*me2*L4r*pow(RSe453p,2) - 8./27.
         *m13*m55*me2*L4r*RSe561p*RSe563p + 4./27.*m13*m55*me2*L4r*pow(
         RSe561p,2) - 8./27.*m13*m55*me2*L4r*RSe461p*RSe463p + 4./27.*
         m13*m55*me2*L4r*pow(RSe461p,2) );

      M13p6x23LV +=  + BbV(me2,L) * (  - 8./27.*m13*m55*me2*L4r*RSe451p
         *RSe453p + 4./27.*m13*m55*me2*L4r*pow(RSe451p,2) - 16./27.*m13
         *m55*m66*L7r*RSe453p*RSe463p + 16./27.*m13*m55*m66*L7r*RSe461p
         *RSe453p + 16./27.*m13*m55*m66*L7r*RSe451p*RSe463p - 16./27.*
         m13*m55*m66*L7r*RSe451p*RSe461p - 8./27.*m13*m55*m66*L6r*pow(
         RSe463p,2) - 8./27.*m13*m55*m66*L6r*pow(RSe453p,2) + 16./27.*
         m13*m55*m66*L6r*RSe461p*RSe463p - 8./27.*m13*m55*m66*L6r*pow(
         RSe461p,2) + 16./27.*m13*m55*m66*L6r*RSe451p*RSe453p - 8./27.*
         m13*m55*m66*L6r*pow(RSe451p,2) - 8./27.*m13*pow(m55,2)*L8r*
         pow(RSe463p,2) + 16./27.*m13*pow(m55,2)*L8r*RSe461p*RSe463p - 
         8./27.*m13*pow(m55,2)*L8r*pow(RSe461p,2) - 8./27.*m13*pow(
         m55,2)*L7r*pow(RSe463p,2) + 16./27.*m13*pow(m55,2)*L7r*RSe461p
         *RSe463p - 8./27.*m13*pow(m55,2)*L7r*pow(RSe461p,2) - 8./27.*
         m13*pow(m55,2)*L6r*pow(RSe463p,2) + 16./27.*m13*pow(m55,2)*L6r
         *RSe461p*RSe463p - 8./27.*m13*pow(m55,2)*L6r*pow(RSe461p,2) + 
         4./27.*m13*m44*me2*L5r*pow(RSe563p,2) );

      M13p6x23LV +=  + BbV(me2,L) * (  - 8./27.*m13*m44*me2*L5r*RSe561p
         *RSe563p + 4./27.*m13*m44*me2*L5r*pow(RSe561p,2) + 4./27.*m13*
         m44*me2*L4r*pow(RSe563p,2) + 4./27.*m13*m44*me2*L4r*pow(
         RSe463p,2) + 4./27.*m13*m44*me2*L4r*pow(RSe453p,2) - 8./27.*
         m13*m44*me2*L4r*RSe561p*RSe563p + 4./27.*m13*m44*me2*L4r*pow(
         RSe561p,2) - 8./27.*m13*m44*me2*L4r*RSe461p*RSe463p + 4./27.*
         m13*m44*me2*L4r*pow(RSe461p,2) - 8./27.*m13*m44*me2*L4r*
         RSe451p*RSe453p + 4./27.*m13*m44*me2*L4r*pow(RSe451p,2) - 16./
         27.*m13*m44*m66*L7r*RSe453p*RSe563p + 16./27.*m13*m44*m66*L7r*
         RSe561p*RSe453p + 16./27.*m13*m44*m66*L7r*RSe451p*RSe563p - 16.
         /27.*m13*m44*m66*L7r*RSe451p*RSe561p - 8./27.*m13*m44*m66*L6r*
         pow(RSe563p,2) - 8./27.*m13*m44*m66*L6r*pow(RSe453p,2) + 16./
         27.*m13*m44*m66*L6r*RSe561p*RSe563p - 8./27.*m13*m44*m66*L6r*
         pow(RSe561p,2) + 16./27.*m13*m44*m66*L6r*RSe451p*RSe453p - 8./
         27.*m13*m44*m66*L6r*pow(RSe451p,2) - 16./27.*m13*m44*m55*L7r*
         RSe463p*RSe563p );

      M13p6x23LV +=  + BbV(me2,L) * ( 16./27.*m13*m44*m55*L7r*RSe561p*
         RSe463p + 16./27.*m13*m44*m55*L7r*RSe461p*RSe563p - 16./27.*
         m13*m44*m55*L7r*RSe461p*RSe561p - 8./27.*m13*m44*m55*L6r*pow(
         RSe563p,2) - 8./27.*m13*m44*m55*L6r*pow(RSe463p,2) + 16./27.*
         m13*m44*m55*L6r*RSe561p*RSe563p - 8./27.*m13*m44*m55*L6r*pow(
         RSe561p,2) + 16./27.*m13*m44*m55*L6r*RSe461p*RSe463p - 8./27.*
         m13*m44*m55*L6r*pow(RSe461p,2) - 8./27.*m13*pow(m44,2)*L8r*
         pow(RSe563p,2) + 16./27.*m13*pow(m44,2)*L8r*RSe561p*RSe563p - 
         8./27.*m13*pow(m44,2)*L8r*pow(RSe561p,2) - 8./27.*m13*pow(
         m44,2)*L7r*pow(RSe563p,2) + 16./27.*m13*pow(m44,2)*L7r*RSe561p
         *RSe563p - 8./27.*m13*pow(m44,2)*L7r*pow(RSe561p,2) - 8./27.*
         m13*pow(m44,2)*L6r*pow(RSe563p,2) + 16./27.*m13*pow(m44,2)*L6r
         *RSe561p*RSe563p - 8./27.*m13*pow(m44,2)*L6r*pow(RSe561p,2) - 
         2./27.*m11*m66*me2*L5r*pow(RSe453p,2) - 4./27.*m11*m66*me2*L5r
         *RSe451p*RSe453p - 2./9.*m11*m66*me2*L5r*pow(RSe451p,2) - 2./
         27.*m11*m66*me2*L4r*pow(RSe563p,2) );

      M13p6x23LV +=  + BbV(me2,L) * (  - 2./27.*m11*m66*me2*L4r*pow(
         RSe463p,2) - 2./27.*m11*m66*me2*L4r*pow(RSe453p,2) - 4./27.*
         m11*m66*me2*L4r*RSe561p*RSe563p - 2./9.*m11*m66*me2*L4r*pow(
         RSe561p,2) - 4./27.*m11*m66*me2*L4r*RSe461p*RSe463p - 2./9.*
         m11*m66*me2*L4r*pow(RSe461p,2) - 4./27.*m11*m66*me2*L4r*
         RSe451p*RSe453p - 2./9.*m11*m66*me2*L4r*pow(RSe451p,2) + 4./27.
         *m11*pow(m66,2)*L8r*pow(RSe453p,2) + 8./27.*m11*pow(m66,2)*L8r
         *RSe451p*RSe453p + 4./9.*m11*pow(m66,2)*L8r*pow(RSe451p,2) + 4.
         /27.*m11*pow(m66,2)*L7r*pow(RSe453p,2) + 8./27.*m11*pow(m66,2)
         *L7r*RSe451p*RSe453p + 4./9.*m11*pow(m66,2)*L7r*pow(RSe451p,2)
          + 4./27.*m11*pow(m66,2)*L6r*pow(RSe453p,2) + 8./27.*m11*pow(
         m66,2)*L6r*RSe451p*RSe453p + 4./9.*m11*pow(m66,2)*L6r*pow(
         RSe451p,2) - 2./27.*m11*m55*me2*L5r*pow(RSe463p,2) - 4./27.*
         m11*m55*me2*L5r*RSe461p*RSe463p - 2./9.*m11*m55*me2*L5r*pow(
         RSe461p,2) - 2./27.*m11*m55*me2*L4r*pow(RSe563p,2) - 2./27.*
         m11*m55*me2*L4r*pow(RSe463p,2) );

      M13p6x23LV +=  + BbV(me2,L) * (  - 2./27.*m11*m55*me2*L4r*pow(
         RSe453p,2) - 4./27.*m11*m55*me2*L4r*RSe561p*RSe563p - 2./9.*
         m11*m55*me2*L4r*pow(RSe561p,2) - 4./27.*m11*m55*me2*L4r*
         RSe461p*RSe463p - 2./9.*m11*m55*me2*L4r*pow(RSe461p,2) - 4./27.
         *m11*m55*me2*L4r*RSe451p*RSe453p - 2./9.*m11*m55*me2*L4r*pow(
         RSe451p,2) + 8./27.*m11*m55*m66*L7r*RSe453p*RSe463p + 8./27.*
         m11*m55*m66*L7r*RSe461p*RSe453p + 8./27.*m11*m55*m66*L7r*
         RSe451p*RSe463p + 8./9.*m11*m55*m66*L7r*RSe451p*RSe461p + 4./
         27.*m11*m55*m66*L6r*pow(RSe463p,2) + 4./27.*m11*m55*m66*L6r*
         pow(RSe453p,2) + 8./27.*m11*m55*m66*L6r*RSe461p*RSe463p + 4./9.
         *m11*m55*m66*L6r*pow(RSe461p,2) + 8./27.*m11*m55*m66*L6r*
         RSe451p*RSe453p + 4./9.*m11*m55*m66*L6r*pow(RSe451p,2) + 4./27.
         *m11*pow(m55,2)*L8r*pow(RSe463p,2) + 8./27.*m11*pow(m55,2)*L8r
         *RSe461p*RSe463p + 4./9.*m11*pow(m55,2)*L8r*pow(RSe461p,2) + 4.
         /27.*m11*pow(m55,2)*L7r*pow(RSe463p,2) + 8./27.*m11*pow(m55,2)
         *L7r*RSe461p*RSe463p );

      M13p6x23LV +=  + BbV(me2,L) * ( 4./9.*m11*pow(m55,2)*L7r*pow(
         RSe461p,2) + 4./27.*m11*pow(m55,2)*L6r*pow(RSe463p,2) + 8./27.
         *m11*pow(m55,2)*L6r*RSe461p*RSe463p + 4./9.*m11*pow(m55,2)*L6r
         *pow(RSe461p,2) - 2./27.*m11*m44*me2*L5r*pow(RSe563p,2) - 4./
         27.*m11*m44*me2*L5r*RSe561p*RSe563p - 2./9.*m11*m44*me2*L5r*
         pow(RSe561p,2) - 2./27.*m11*m44*me2*L4r*pow(RSe563p,2) - 2./27.
         *m11*m44*me2*L4r*pow(RSe463p,2) - 2./27.*m11*m44*me2*L4r*pow(
         RSe453p,2) - 4./27.*m11*m44*me2*L4r*RSe561p*RSe563p - 2./9.*
         m11*m44*me2*L4r*pow(RSe561p,2) - 4./27.*m11*m44*me2*L4r*
         RSe461p*RSe463p - 2./9.*m11*m44*me2*L4r*pow(RSe461p,2) - 4./27.
         *m11*m44*me2*L4r*RSe451p*RSe453p - 2./9.*m11*m44*me2*L4r*pow(
         RSe451p,2) + 8./27.*m11*m44*m66*L7r*RSe453p*RSe563p + 8./27.*
         m11*m44*m66*L7r*RSe561p*RSe453p + 8./27.*m11*m44*m66*L7r*
         RSe451p*RSe563p + 8./9.*m11*m44*m66*L7r*RSe451p*RSe561p + 4./
         27.*m11*m44*m66*L6r*pow(RSe563p,2) + 4./27.*m11*m44*m66*L6r*
         pow(RSe453p,2) );

      M13p6x23LV +=  + BbV(me2,L) * ( 8./27.*m11*m44*m66*L6r*RSe561p*
         RSe563p + 4./9.*m11*m44*m66*L6r*pow(RSe561p,2) + 8./27.*m11*
         m44*m66*L6r*RSe451p*RSe453p + 4./9.*m11*m44*m66*L6r*pow(
         RSe451p,2) + 8./27.*m11*m44*m55*L7r*RSe463p*RSe563p + 8./27.*
         m11*m44*m55*L7r*RSe561p*RSe463p + 8./27.*m11*m44*m55*L7r*
         RSe461p*RSe563p + 8./9.*m11*m44*m55*L7r*RSe461p*RSe561p + 4./
         27.*m11*m44*m55*L6r*pow(RSe563p,2) + 4./27.*m11*m44*m55*L6r*
         pow(RSe463p,2) + 8./27.*m11*m44*m55*L6r*RSe561p*RSe563p + 4./9.
         *m11*m44*m55*L6r*pow(RSe561p,2) + 8./27.*m11*m44*m55*L6r*
         RSe461p*RSe463p + 4./9.*m11*m44*m55*L6r*pow(RSe461p,2) + 4./27.
         *m11*pow(m44,2)*L8r*pow(RSe563p,2) + 8./27.*m11*pow(m44,2)*L8r
         *RSe561p*RSe563p + 4./9.*m11*pow(m44,2)*L8r*pow(RSe561p,2) + 4.
         /27.*m11*pow(m44,2)*L7r*pow(RSe563p,2) + 8./27.*m11*pow(m44,2)
         *L7r*RSe561p*RSe563p + 4./9.*m11*pow(m44,2)*L7r*pow(RSe561p,2)
          + 4./27.*m11*pow(m44,2)*L6r*pow(RSe563p,2) + 8./27.*m11*pow(
         m44,2)*L6r*RSe561p*RSe563p );

      M13p6x23LV +=  + BbV(me2,L) * ( 4./9.*m11*pow(m44,2)*L6r*pow(
         RSe561p,2) );

      M13p6x23LV +=  + CbV(m11,L) * (  - 8./9.*m11*m33*m66*L6r*RD1456ep
          + 4./9.*m11*m33*m66*L4r*RD1456ep - 8./9.*m11*m33*m55*L6r*
         RD1456ep + 4./9.*m11*m33*m55*L4r*RD1456ep - 8./9.*m11*m33*m44*
         L6r*RD1456ep + 4./9.*m11*m33*m44*L4r*RD1456ep + 16./9.*m11*m13
         *m66*L6r*RD1456ep - 8./9.*m11*m13*m66*L4r*RD1456ep + 16./9.*
         m11*m13*m55*L6r*RD1456ep - 8./9.*m11*m13*m55*L4r*RD1456ep + 16.
         /9.*m11*m13*m44*L6r*RD1456ep - 8./9.*m11*m13*m44*L4r*RD1456ep
          - 8./9.*pow(m11,2)*m66*L6r*RD1456ep + 4./9.*pow(m11,2)*m66*
         L4r*RD1456ep - 8./9.*pow(m11,2)*m55*L6r*RD1456ep + 4./9.*pow(
         m11,2)*m55*L4r*RD1456ep - 8./9.*pow(m11,2)*m44*L6r*RD1456ep + 
         4./9.*pow(m11,2)*m44*L4r*RD1456ep - 8./9.*pow(m11,2)*m33*L8r*
         RD1456ep + 4./9.*pow(m11,2)*m33*L5r*RD1456ep + 16./9.*pow(
         m11,2)*m13*L8r*RD1456ep - 8./9.*pow(m11,2)*m13*L5r*RD1456ep - 
         8./9.*pow(m11,3)*L8r*RD1456ep + 4./9.*pow(m11,3)*L5r*RD1456ep
          );

      M13p6x23LV +=  + CbV(m33,L) * (  - 8./9.*pow(m33,2)*m66*L6r*
         RD3456ep + 4./9.*pow(m33,2)*m66*L4r*RD3456ep - 8./9.*pow(
         m33,2)*m55*L6r*RD3456ep + 4./9.*pow(m33,2)*m55*L4r*RD3456ep - 
         8./9.*pow(m33,2)*m44*L6r*RD3456ep + 4./9.*pow(m33,2)*m44*L4r*
         RD3456ep - 8./9.*pow(m33,3)*L8r*RD3456ep + 4./9.*pow(m33,3)*
         L5r*RD3456ep + 16./9.*m13*m33*m66*L6r*RD3456ep - 8./9.*m13*m33
         *m66*L4r*RD3456ep + 16./9.*m13*m33*m55*L6r*RD3456ep - 8./9.*
         m13*m33*m55*L4r*RD3456ep + 16./9.*m13*m33*m44*L6r*RD3456ep - 8.
         /9.*m13*m33*m44*L4r*RD3456ep + 16./9.*m13*pow(m33,2)*L8r*
         RD3456ep - 8./9.*m13*pow(m33,2)*L5r*RD3456ep - 8./9.*m11*m33*
         m66*L6r*RD3456ep + 4./9.*m11*m33*m66*L4r*RD3456ep - 8./9.*m11*
         m33*m55*L6r*RD3456ep + 4./9.*m11*m33*m55*L4r*RD3456ep - 8./9.*
         m11*m33*m44*L6r*RD3456ep + 4./9.*m11*m33*m44*L4r*RD3456ep - 8./
         9.*m11*pow(m33,2)*L8r*RD3456ep + 4./9.*m11*pow(m33,2)*L5r*
         RD3456ep );

      M13p6x23LV +=  + A23bV(m11,L) * ( 4*m13*L3r*RS14563pe + 2*m13*L3r
         *RC1456pe + 4*m13*L0r*RC1456pe );

      M13p6x23LV +=  + A23bV(m13,L) * (  - 12*m13*L2r - 24*m13*L1r );

      M13p6x23LV +=  + A23bV(m14,L) * (  - 6*m13*L3r - 12*m13*L0r );

      M13p6x23LV +=  + A23bV(m15,L) * (  - 6*m13*L3r - 12*m13*L0r );

      M13p6x23LV +=  + A23bV(m16,L) * (  - 6*m13*L3r - 12*m13*L0r );

      M13p6x23LV +=  + A23bV(m33,L) * ( 4*m13*L3r*RS34561pe + 2*m13*L3r
         *RC3456pe + 4*m13*L0r*RC3456pe );

      M13p6x23LV +=  + A23bV(m34,L) * (  - 6*m13*L3r - 12*m13*L0r );

      M13p6x23LV +=  + A23bV(m35,L) * (  - 6*m13*L3r - 12*m13*L0r );

      M13p6x23LV +=  + A23bV(m36,L) * (  - 6*m13*L3r - 12*m13*L0r );

      M13p6x23LV +=  + A23bV(m44,L) * (  - 12*m13*L2r + 4*m13*L2r*
         RS456pe );

      M13p6x23LV +=  + A23bV(m45,L) * (  - 24*m13*L2r );

      M13p6x23LV +=  + A23bV(m46,L) * (  - 24*m13*L2r );

      M13p6x23LV +=  + A23bV(m55,L) * (  - 12*m13*L2r + 4*m13*L2r*
         RS546pe );

      M13p6x23LV +=  + A23bV(m56,L) * (  - 24*m13*L2r );

      M13p6x23LV +=  + A23bV(m66,L) * (  - 12*m13*L2r + 4*m13*L2r*
         RS645pe );

      M13p6x23LV +=  + A23bV(mp2,L) * ( 4*m13*L3r*RSp45613e + 2*m13*L3r
         *RSp45633e + 2*m13*L3r*RSp45611e + 4*m13*L2r*RSp564e + 4*m13*
         L2r*RSp465e + 4*m13*L2r*RSp456e + 4*m13*L0r*RSp45633e + 4*m13*
         L0r*RSp45611e );

      M13p6x23LV +=  + A23bV(me2,L) * ( 4*m13*L3r*RSe45613p + 2*m13*L3r
         *RSe45633p + 2*m13*L3r*RSe45611p + 4*m13*L2r*RSe564p + 4*m13*
         L2r*RSe465p + 4*m13*L2r*RSe456p + 4*m13*L0r*RSe45633p + 4*m13*
         L0r*RSe45611p );

      M13p6x23LV +=  + B23bV(m11,L) * ( 2*m13*L3r*RD1456ep + 4*m13*L0r*
         RD1456ep );

      M13p6x23LV +=  + B23bV(m33,L) * ( 2*m13*L3r*RD3456ep + 4*m13*L0r*
         RD3456ep );

  return M13p6x23LV/pow(f0,4);
}

double mv2s3nf3p6RV(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 5) || (B0mq.size() != 5)){
    cout << "ERROR: mv2s3nf3p6RV probably called with wrong number"
	 << "of quark masses\n";}
  // two masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m55 = 2.*B0mq[3];
  double m66 = 2.*B0mq[4];
  double m13 = (m11+m33)/2;
  double m14 = (m11+m44)/2;
  double m15 = (m11+m55)/2;
  double m16 = (m11+m66)/2;
  double m34 = (m33+m44)/2;
  double m35 = (m33+m55)/2;
  double m36 = (m33+m66)/2;
  double m45 = (m44+m55)/2;
  double m46 = (m44+m66)/2;
  double m56 = (m55+m66)/2;

  double xb1 = (m44+m55+m66)/3.;
  double msqr = sqrt(xb1*xb1-(m44*m55+m44*m66+m55*m66)/3.);
  double mp2 = xb1-msqr; 
  double me2 = xb1+msqr;
  double mu2 = mu0*mu0;

  double RN13 = RN(m11,m33);
  double RN1p = RN(m11,mp2);
  double RN1e = RN(m11,me2);
  double RN3p = RN(m33,mp2);
  double RN3e = RN(m33,me2);
  //double RNep = RN(me2,mp2);

  double RSp456e = RS(mp2,m44,m55,m66,me2);
  double RSp465e = RS(mp2,m44,m66,m55,me2);
  double RSp564e = RS(mp2,m55,m66,m44,me2);
  double RSe456p = RS(me2,m44,m55,m66,mp2);
  double RSe465p = RS(me2,m44,m66,m55,mp2);
  double RSe564p = RS(me2,m55,m66,m44,mp2);
  double RSp451e = RS(mp2,m44,m55,m11,me2);
  double RSp461e = RS(mp2,m44,m66,m11,me2);
  double RSp561e = RS(mp2,m55,m66,m11,me2);
  double RSe451p = RS(me2,m44,m55,m11,mp2);
  double RSe461p = RS(me2,m44,m66,m11,mp2);
  double RSe561p = RS(me2,m55,m66,m11,mp2);
  double RSp453e = RS(mp2,m44,m55,m33,me2);
  double RSp463e = RS(mp2,m44,m66,m33,me2);
  double RSp563e = RS(mp2,m55,m66,m33,me2);
  double RSe453p = RS(me2,m44,m55,m33,mp2);
  double RSe463p = RS(me2,m44,m66,m33,mp2);
  double RSe563p = RS(me2,m55,m66,m33,mp2);
  double RS456pe = RS(m44,m55,m66,mp2,me2);
  double RS546pe = RS(m55,m44,m66,mp2,me2);
  double RS645pe = RS(m66,m44,m55,mp2,me2);
  double RS156pe = RS(m11,m55,m66,mp2,me2);
  double RS146pe = RS(m11,m44,m66,mp2,me2);
  double RS145pe = RS(m11,m44,m55,mp2,me2);
  double RS356pe = RS(m33,m55,m66,mp2,me2);
  double RS346pe = RS(m33,m44,m66,mp2,me2);
  double RS345pe = RS(m33,m44,m55,mp2,me2);

  double RD1456ep = RD(m11,m44,m55,m66,me2,mp2);
  double RD3456ep = RD(m33,m44,m55,m66,me2,mp2);
  double RSe45611p = RS(me2,m44,m55,m66,m11,m11,mp2);
  double RSp45611e = RS(mp2,m44,m55,m66,m11,m11,me2);
  //double RS1456ppe = RS(m11,m44,m55,m66,mp2,mp2,me2);
  //double RS1456pee = RS(m11,m44,m55,m66,mp2,me2,me2);
  double RSe45633p = RS(me2,m44,m55,m66,m33,m33,mp2);
  double RSp45633e = RS(mp2,m44,m55,m66,m33,m33,me2);
  //double RS3456ppe = RS(m33,m44,m55,m66,mp2,mp2,me2);
  //double RS3456pee = RS(m33,m44,m55,m66,mp2,me2,me2);
  double RSe45613p = RS(me2,m44,m55,m66,m11,m33,mp2);
  double RSp45613e = RS(mp2,m44,m55,m66,m11,m33,me2);
  double RS14563pe = RS(m11,m44,m55,m66,m33,mp2,me2);
  double RS34561pe = RS(m33,m44,m55,m66,m11,mp2,me2);

  double RC1456pe = RC(m11,m44,m55,m66,mp2,me2);
  double RC3456pe = RC(m33,m44,m55,m66,mp2,me2);

  double M13p6x23RV =
       + Ab(m11,mu2)*AbV(m11,L) * (  - 1./135.*RD1456ep*RS14563pe + 1./
         270.*RD1456ep*RC1456pe + 11./90.*m33 + 13./405.*m33*pow(
         RS14563pe,2) + 1./135.*m33*RC1456pe*RS14563pe + 13./810.*m33*
         pow(RC1456pe,2) + 1./180.*m13 - 7./405.*m13*pow(RS14563pe,2)
          + 7./405.*m13*RC1456pe*RS14563pe - 7./1620.*m13*pow(
         RC1456pe,2) + 11./90.*m11 + 1./45.*m11*pow(RS14563pe,2) + 4./
         81.*m11*RC1456pe*RS14563pe - 17./810.*m11*pow(RC1456pe,2) + 1./
         54.*m11*m33*RN1p*RC1456pe*RSp45613e + 1./54.*m11*m33*RN1p*
         RSp45611e*RC1456pe + 1./54.*m11*m33*RN1e*RC1456pe*RSe45613p + 
         1./54.*m11*m33*RN1e*RSe45611p*RC1456pe + 1./54.*m11*m33*RN13*
         RC1456pe*RS34561pe + 1./27.*m11*m13*RN1p*RC1456pe*RSp45613e - 
         1./27.*m11*m13*RN1p*RSp45611e*RC1456pe + 1./27.*m11*m13*RN1e*
         RC1456pe*RSe45613p - 1./27.*m11*m13*RN1e*RSe45611p*RC1456pe + 
         1./27.*m11*m13*RN13*RC1456pe*RS34561pe + 1./18.*pow(m11,2)*
         RN1p*RC1456pe*RSp45613e + 1./54.*pow(m11,2)*RN1p*RSp45611e*
         RC1456pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m11,L) * ( 1./18.*pow(m11,2)*
         RN1e*RC1456pe*RSe45613p + 1./54.*pow(m11,2)*RN1e*RSe45611p*
         RC1456pe + 1./18.*pow(m11,2)*RN13*RC1456pe*RS34561pe + 2./81.*
         RS(m11,m44,m55,m66,m33,me2,mp2)*m33*RS14563pe - 1./81.*RS(m11,
         m44,m55,m66,m33,me2,mp2)*m33*RC1456pe + 4./81.*RS(m11,m44,m55,
         m66,m33,me2,mp2)*m11*RS14563pe - 2./81.*RS(m11,m44,m55,m66,m33
         ,me2,mp2)*m11*RC1456pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m13,L) * (  - 1./162.*RD3456ep*
         RS14563pe + 1./324.*RC1456pe*RD3456ep - 1./54.*RD1456ep*
         RS34561pe - 2./81.*RD1456ep*RS14563pe - 1./54.*RD1456ep*
         RSe45613p - 1./54.*RD1456ep*RSp45613e + 1./108.*RD1456ep*
         RC3456pe + 1./108.*RD1456ep*RSe45633p + 1./108.*RD1456ep*
         RSp45633e + 1./81.*RD1456ep*RC1456pe + 1./108.*RSe45611p*
         RD1456ep + 1./108.*RSp45611e*RD1456ep + 1./81.*me2*RSe45613p*
         RS14563pe - 1./162.*me2*RSe45633p*RS14563pe - 1./162.*me2*
         RC1456pe*RSe45613p + 1./324.*me2*RC1456pe*RSe45633p - 1./162.*
         me2*RSe45611p*RS14563pe + 1./324.*me2*RSe45611p*RC1456pe + 1./
         81.*mp2*RSp45613e*RS14563pe - 1./162.*mp2*RSp45633e*RS14563pe
          - 1./162.*mp2*RC1456pe*RSp45613e + 1./324.*mp2*RC1456pe*
         RSp45633e - 1./162.*mp2*RSp45611e*RS14563pe + 1./324.*mp2*
         RSp45611e*RC1456pe - 1./60.*m33 - 1./81.*m33*pow(RS14563pe,2)
          - 1./81.*m33*RSe45613p*RS14563pe - 1./81.*m33*RSp45613e*
         RS14563pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m13,L) * ( 1./162.*m33*RC3456pe*
         RS14563pe + 1./81.*m33*RSe45633p*RS14563pe + 1./81.*m33*
         RSp45633e*RS14563pe - 1./162.*m33*RC1456pe*RS34561pe + 1./108.
         *m33*RC1456pe*RC3456pe + 1./162.*m33*RC1456pe*RSe45633p + 1./
         162.*m33*RC1456pe*RSp45633e - 1./162.*m33*pow(RC1456pe,2) - 1./
         162.*m33*RSe45611p*RC1456pe - 1./162.*m33*RSp45611e*RC1456pe
          + 1./36.*pow(m33,2)*RN13 + 1./180.*m13 - 2./27.*m13*RS14563pe
         *RS34561pe - 2./27.*m13*pow(RS14563pe,2) - 2./27.*m13*
         RSe45613p*RS14563pe - 2./27.*m13*RSp45613e*RS14563pe + 1./27.*
         m13*RC3456pe*RS14563pe + 1./27.*m13*RSe45633p*RS14563pe + 1./
         27.*m13*RSp45633e*RS14563pe + 1./27.*m13*RC1456pe*RS34561pe + 
         2./27.*m13*RC1456pe*RS14563pe + 1./27.*m13*RC1456pe*RSe45613p
          + 1./27.*m13*RC1456pe*RSp45613e - 1./54.*m13*RC1456pe*
         RC3456pe - 1./54.*m13*RC1456pe*RSe45633p - 1./54.*m13*RC1456pe
         *RSp45633e - 1./54.*m13*pow(RC1456pe,2) + 1./27.*m13*RSe45611p
         *RS14563pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m13,L) * (  - 1./54.*m13*
         RSe45611p*RC1456pe + 1./27.*m13*RSp45611e*RS14563pe - 1./54.*
         m13*RSp45611e*RC1456pe + 1./9.*m13*m33*RN13 + 1./9.*pow(m13,2)
         *RN13 + 7./180.*m11 + 2./81.*m11*RS14563pe*RS34561pe + 1./27.*
         m11*pow(RS14563pe,2) + 2./81.*m11*RSe45613p*RS14563pe + 2./81.
         *m11*RSp45613e*RS14563pe - 1./54.*m11*RC3456pe*RS14563pe - 1./
         54.*m11*RSe45633p*RS14563pe - 1./54.*m11*RSp45633e*RS14563pe
          - 1./162.*m11*RC1456pe*RS34561pe - 2./81.*m11*RC1456pe*
         RS14563pe - 1./162.*m11*RC1456pe*RSe45613p - 1./162.*m11*
         RC1456pe*RSp45613e + 5./324.*m11*RC1456pe*RC3456pe + 5./324.*
         m11*RC1456pe*RSe45633p + 5./324.*m11*RC1456pe*RSp45633e - 1./
         162.*m11*pow(RC1456pe,2) - 1./162.*m11*RSe45611p*RS14563pe - 1.
         /108.*m11*RSe45611p*RC1456pe - 1./162.*m11*RSp45611e*RS14563pe
          - 1./108.*m11*RSp45611e*RC1456pe + 5./18.*m11*m33*RN13 + 1./3.
         *m11*m13*RN13 + 5./36.*pow(m11,2)*RN13 );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m14,L) * (  - 29./540.*RD1456ep
          - 7./540.*m44*RS14563pe + 7./540.*m44*RC1456pe - 1./60.*m44*
         RS156pe + 1./27.*m34*RS14563pe - 1./108.*m34*RC1456pe - 1./54.
         *m34*RS156pe + 1./90.*m33*RS14563pe + 1./120.*m33*RC1456pe - 1.
         /20.*m33*RS156pe - 1./108.*m33*m44*RN1p*RSp45613e - 1./108.*
         m33*m44*RN1p*RSp563e - 1./108.*m33*m44*RN1p*RSp45611e - 1./108.
         *m33*m44*RN1p*RSp561e - 1./108.*m33*m44*RN1e*RSe45613p - 1./
         108.*m33*m44*RN1e*RSe563p - 1./108.*m33*m44*RN1e*RSe45611p - 1.
         /108.*m33*m44*RN1e*RSe561p - 1./108.*m33*m44*RN13*RS34561pe - 
         1./108.*m33*m44*RN13*RS356pe - 1./90.*m14*RS14563pe - 1./60.*
         m14*RC1456pe + 2./45.*m14*RS156pe + 1./54.*m14*m33*RN1p*
         RSp45613e - 1./54.*m14*m33*RN1p*RSp563e + 1./54.*m14*m33*RN1p*
         RSp45611e - 1./54.*m14*m33*RN1p*RSp561e + 1./54.*m14*m33*RN1e*
         RSe45613p - 1./54.*m14*m33*RN1e*RSe563p + 1./54.*m14*m33*RN1e*
         RSe45611p - 1./54.*m14*m33*RN1e*RSe561p + 1./54.*m14*m33*RN13*
         RS34561pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m14,L) * (  - 1./54.*m14*m33*
         RN13*RS356pe - 11./540.*m13*RS14563pe - 1./135.*m13*RC1456pe
          + 19./540.*m13*RS156pe - 1./54.*m13*m44*RN1p*RSp45613e - 1./
         54.*m13*m44*RN1p*RSp563e + 1./54.*m13*m44*RN1p*RSp45611e + 1./
         54.*m13*m44*RN1p*RSp561e - 1./54.*m13*m44*RN1e*RSe45613p - 1./
         54.*m13*m44*RN1e*RSe563p + 1./54.*m13*m44*RN1e*RSe45611p + 1./
         54.*m13*m44*RN1e*RSe561p - 1./54.*m13*m44*RN13*RS34561pe - 1./
         54.*m13*m44*RN13*RS356pe + 1./27.*m13*m14*RN1p*RSp45613e - 1./
         27.*m13*m14*RN1p*RSp563e - 1./27.*m13*m14*RN1p*RSp45611e + 1./
         27.*m13*m14*RN1p*RSp561e + 1./27.*m13*m14*RN1e*RSe45613p - 1./
         27.*m13*m14*RN1e*RSe563p - 1./27.*m13*m14*RN1e*RSe45611p + 1./
         27.*m13*m14*RN1e*RSe561p + 1./27.*m13*m14*RN13*RS34561pe - 1./
         27.*m13*m14*RN13*RS356pe + 1./27.*m11*RS34561pe + 43./540.*m11
         *RS14563pe + 1./27.*m11*RSe45613p + 1./27.*m11*RSp45613e - 1./
         27.*m11*RS356pe - 1./27.*m11*RSe563p - 1./27.*m11*RSp563e + 13.
         /1080.*m11*RC1456pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m14,L) * (  - 1./27.*m11*
         RSe45611p - 1./27.*m11*RSp45611e - 1./45.*m11*RS156pe + 1./27.
         *m11*RSe561p + 1./27.*m11*RSp561e + 1./27.*m11*me2*RN1e*
         RSe45613p - 1./27.*m11*me2*RN1e*RSe563p - 1./27.*m11*me2*RN1e*
         RSe45611p + 1./27.*m11*me2*RN1e*RSe561p + 1./27.*m11*mp2*RN1p*
         RSp45613e - 1./27.*m11*mp2*RN1p*RSp563e - 1./27.*m11*mp2*RN1p*
         RSp45611e + 1./27.*m11*mp2*RN1p*RSp561e - 1./36.*m11*m44*RN1p*
         RSp45613e - 1./36.*m11*m44*RN1p*RSp563e - 1./108.*m11*m44*RN1p
         *RSp45611e - 1./108.*m11*m44*RN1p*RSp561e - 1./36.*m11*m44*
         RN1e*RSe45613p - 1./36.*m11*m44*RN1e*RSe563p - 1./108.*m11*m44
         *RN1e*RSe45611p - 1./108.*m11*m44*RN1e*RSe561p - 1./36.*m11*
         m44*RN13*RS34561pe - 1./36.*m11*m44*RN13*RS356pe - 1./108.*m11
         *m33*RN1p*RSp45613e - 1./36.*m11*m33*RN1p*RSp563e - 1./108.*
         m11*m33*RN1p*RSp45611e - 1./36.*m11*m33*RN1p*RSp561e - 1./108.
         *m11*m33*RN1e*RSe45613p - 1./36.*m11*m33*RN1e*RSe563p - 1./108.
         *m11*m33*RN1e*RSe45611p );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m14,L) * (  - 1./36.*m11*m33*
         RN1e*RSe561p + 1./36.*m11*m33*RN13*RS34561pe - 7./108.*m11*m33
         *RN13*RS356pe + 1./18.*m11*m14*RN1p*RSp45613e - 1./18.*m11*m14
         *RN1p*RSp563e + 1./54.*m11*m14*RN1p*RSp45611e - 1./54.*m11*m14
         *RN1p*RSp561e + 1./18.*m11*m14*RN1e*RSe45613p - 1./18.*m11*m14
         *RN1e*RSe563p + 1./54.*m11*m14*RN1e*RSe45611p - 1./54.*m11*m14
         *RN1e*RSe561p + 1./18.*m11*m14*RN13*RS34561pe - 1./18.*m11*m14
         *RN13*RS356pe - 1./54.*m11*m13*RN1p*RSp45613e - 1./18.*m11*m13
         *RN1p*RSp563e + 1./54.*m11*m13*RN1p*RSp45611e + 1./18.*m11*m13
         *RN1p*RSp561e - 1./54.*m11*m13*RN1e*RSe45613p - 1./18.*m11*m13
         *RN1e*RSe563p + 1./54.*m11*m13*RN1e*RSe45611p + 1./18.*m11*m13
         *RN1e*RSe561p - 1./54.*m11*m13*RN13*RS34561pe - 1./18.*m11*m13
         *RN13*RS356pe - 7./108.*pow(m11,2)*RN1p*RSp45613e - 5./108.*
         pow(m11,2)*RN1p*RSp563e + 1./36.*pow(m11,2)*RN1p*RSp45611e - 7.
         /108.*pow(m11,2)*RN1p*RSp561e - 7./108.*pow(m11,2)*RN1e*
         RSe45613p );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m14,L) * (  - 5./108.*pow(m11,2)
         *RN1e*RSe563p + 1./36.*pow(m11,2)*RN1e*RSe45611p - 7./108.*
         pow(m11,2)*RN1e*RSe561p - 7./108.*pow(m11,2)*RN13*RS34561pe - 
         5./108.*pow(m11,2)*RN13*RS356pe + 1./54.*RS(m11,m44,m55,m66,
         m33,me2,mp2)*m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11
          );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m15,L) * (  - 29./540.*RD1456ep
          - 7./540.*m55*RS14563pe + 7./540.*m55*RC1456pe - 1./60.*m55*
         RS146pe + 1./27.*m35*RS14563pe - 1./108.*m35*RC1456pe - 1./54.
         *m35*RS146pe + 1./90.*m33*RS14563pe + 1./120.*m33*RC1456pe - 1.
         /20.*m33*RS146pe - 1./108.*m33*m55*RN1p*RSp45613e - 1./108.*
         m33*m55*RN1p*RSp463e - 1./108.*m33*m55*RN1p*RSp45611e - 1./108.
         *m33*m55*RN1p*RSp461e - 1./108.*m33*m55*RN1e*RSe45613p - 1./
         108.*m33*m55*RN1e*RSe463p - 1./108.*m33*m55*RN1e*RSe45611p - 1.
         /108.*m33*m55*RN1e*RSe461p - 1./108.*m33*m55*RN13*RS34561pe - 
         1./108.*m33*m55*RN13*RS346pe - 1./90.*m15*RS14563pe - 1./60.*
         m15*RC1456pe + 2./45.*m15*RS146pe + 1./54.*m15*m33*RN1p*
         RSp45613e - 1./54.*m15*m33*RN1p*RSp463e + 1./54.*m15*m33*RN1p*
         RSp45611e - 1./54.*m15*m33*RN1p*RSp461e + 1./54.*m15*m33*RN1e*
         RSe45613p - 1./54.*m15*m33*RN1e*RSe463p + 1./54.*m15*m33*RN1e*
         RSe45611p - 1./54.*m15*m33*RN1e*RSe461p + 1./54.*m15*m33*RN13*
         RS34561pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m15,L) * (  - 1./54.*m15*m33*
         RN13*RS346pe - 11./540.*m13*RS14563pe - 1./135.*m13*RC1456pe
          + 19./540.*m13*RS146pe - 1./54.*m13*m55*RN1p*RSp45613e - 1./
         54.*m13*m55*RN1p*RSp463e + 1./54.*m13*m55*RN1p*RSp45611e + 1./
         54.*m13*m55*RN1p*RSp461e - 1./54.*m13*m55*RN1e*RSe45613p - 1./
         54.*m13*m55*RN1e*RSe463p + 1./54.*m13*m55*RN1e*RSe45611p + 1./
         54.*m13*m55*RN1e*RSe461p - 1./54.*m13*m55*RN13*RS34561pe - 1./
         54.*m13*m55*RN13*RS346pe + 1./27.*m13*m15*RN1p*RSp45613e - 1./
         27.*m13*m15*RN1p*RSp463e - 1./27.*m13*m15*RN1p*RSp45611e + 1./
         27.*m13*m15*RN1p*RSp461e + 1./27.*m13*m15*RN1e*RSe45613p - 1./
         27.*m13*m15*RN1e*RSe463p - 1./27.*m13*m15*RN1e*RSe45611p + 1./
         27.*m13*m15*RN1e*RSe461p + 1./27.*m13*m15*RN13*RS34561pe - 1./
         27.*m13*m15*RN13*RS346pe + 1./27.*m11*RS34561pe + 43./540.*m11
         *RS14563pe + 1./27.*m11*RSe45613p + 1./27.*m11*RSp45613e - 1./
         27.*m11*RS346pe - 1./27.*m11*RSe463p - 1./27.*m11*RSp463e + 13.
         /1080.*m11*RC1456pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m15,L) * (  - 1./27.*m11*
         RSe45611p - 1./27.*m11*RSp45611e - 1./45.*m11*RS146pe + 1./27.
         *m11*RSe461p + 1./27.*m11*RSp461e + 1./27.*m11*me2*RN1e*
         RSe45613p - 1./27.*m11*me2*RN1e*RSe463p - 1./27.*m11*me2*RN1e*
         RSe45611p + 1./27.*m11*me2*RN1e*RSe461p + 1./27.*m11*mp2*RN1p*
         RSp45613e - 1./27.*m11*mp2*RN1p*RSp463e - 1./27.*m11*mp2*RN1p*
         RSp45611e + 1./27.*m11*mp2*RN1p*RSp461e - 1./36.*m11*m55*RN1p*
         RSp45613e - 1./36.*m11*m55*RN1p*RSp463e - 1./108.*m11*m55*RN1p
         *RSp45611e - 1./108.*m11*m55*RN1p*RSp461e - 1./36.*m11*m55*
         RN1e*RSe45613p - 1./36.*m11*m55*RN1e*RSe463p - 1./108.*m11*m55
         *RN1e*RSe45611p - 1./108.*m11*m55*RN1e*RSe461p - 1./36.*m11*
         m55*RN13*RS34561pe - 1./36.*m11*m55*RN13*RS346pe - 1./108.*m11
         *m33*RN1p*RSp45613e - 1./36.*m11*m33*RN1p*RSp463e - 1./108.*
         m11*m33*RN1p*RSp45611e - 1./36.*m11*m33*RN1p*RSp461e - 1./108.
         *m11*m33*RN1e*RSe45613p - 1./36.*m11*m33*RN1e*RSe463p - 1./108.
         *m11*m33*RN1e*RSe45611p );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m15,L) * (  - 1./36.*m11*m33*
         RN1e*RSe461p + 1./36.*m11*m33*RN13*RS34561pe - 7./108.*m11*m33
         *RN13*RS346pe + 1./18.*m11*m15*RN1p*RSp45613e - 1./18.*m11*m15
         *RN1p*RSp463e + 1./54.*m11*m15*RN1p*RSp45611e - 1./54.*m11*m15
         *RN1p*RSp461e + 1./18.*m11*m15*RN1e*RSe45613p - 1./18.*m11*m15
         *RN1e*RSe463p + 1./54.*m11*m15*RN1e*RSe45611p - 1./54.*m11*m15
         *RN1e*RSe461p + 1./18.*m11*m15*RN13*RS34561pe - 1./18.*m11*m15
         *RN13*RS346pe - 1./54.*m11*m13*RN1p*RSp45613e - 1./18.*m11*m13
         *RN1p*RSp463e + 1./54.*m11*m13*RN1p*RSp45611e + 1./18.*m11*m13
         *RN1p*RSp461e - 1./54.*m11*m13*RN1e*RSe45613p - 1./18.*m11*m13
         *RN1e*RSe463p + 1./54.*m11*m13*RN1e*RSe45611p + 1./18.*m11*m13
         *RN1e*RSe461p - 1./54.*m11*m13*RN13*RS34561pe - 1./18.*m11*m13
         *RN13*RS346pe - 7./108.*pow(m11,2)*RN1p*RSp45613e - 5./108.*
         pow(m11,2)*RN1p*RSp463e + 1./36.*pow(m11,2)*RN1p*RSp45611e - 7.
         /108.*pow(m11,2)*RN1p*RSp461e - 7./108.*pow(m11,2)*RN1e*
         RSe45613p );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m15,L) * (  - 5./108.*pow(m11,2)
         *RN1e*RSe463p + 1./36.*pow(m11,2)*RN1e*RSe45611p - 7./108.*
         pow(m11,2)*RN1e*RSe461p - 7./108.*pow(m11,2)*RN13*RS34561pe - 
         5./108.*pow(m11,2)*RN13*RS346pe + 1./54.*RS(m11,m44,m55,m66,
         m33,me2,mp2)*m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11
          );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m16,L) * (  - 29./540.*RD1456ep
          - 7./540.*m66*RS14563pe + 7./540.*m66*RC1456pe - 1./60.*m66*
         RS145pe + 1./27.*m36*RS14563pe - 1./108.*m36*RC1456pe - 1./54.
         *m36*RS145pe + 1./90.*m33*RS14563pe + 1./120.*m33*RC1456pe - 1.
         /20.*m33*RS145pe - 1./108.*m33*m66*RN1p*RSp45613e - 1./108.*
         m33*m66*RN1p*RSp453e - 1./108.*m33*m66*RN1p*RSp45611e - 1./108.
         *m33*m66*RN1p*RSp451e - 1./108.*m33*m66*RN1e*RSe45613p - 1./
         108.*m33*m66*RN1e*RSe453p - 1./108.*m33*m66*RN1e*RSe45611p - 1.
         /108.*m33*m66*RN1e*RSe451p - 1./108.*m33*m66*RN13*RS34561pe - 
         1./108.*m33*m66*RN13*RS345pe - 1./90.*m16*RS14563pe - 1./60.*
         m16*RC1456pe + 2./45.*m16*RS145pe + 1./54.*m16*m33*RN1p*
         RSp45613e - 1./54.*m16*m33*RN1p*RSp453e + 1./54.*m16*m33*RN1p*
         RSp45611e - 1./54.*m16*m33*RN1p*RSp451e + 1./54.*m16*m33*RN1e*
         RSe45613p - 1./54.*m16*m33*RN1e*RSe453p + 1./54.*m16*m33*RN1e*
         RSe45611p - 1./54.*m16*m33*RN1e*RSe451p + 1./54.*m16*m33*RN13*
         RS34561pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m16,L) * (  - 1./54.*m16*m33*
         RN13*RS345pe - 11./540.*m13*RS14563pe - 1./135.*m13*RC1456pe
          + 19./540.*m13*RS145pe - 1./54.*m13*m66*RN1p*RSp45613e - 1./
         54.*m13*m66*RN1p*RSp453e + 1./54.*m13*m66*RN1p*RSp45611e + 1./
         54.*m13*m66*RN1p*RSp451e - 1./54.*m13*m66*RN1e*RSe45613p - 1./
         54.*m13*m66*RN1e*RSe453p + 1./54.*m13*m66*RN1e*RSe45611p + 1./
         54.*m13*m66*RN1e*RSe451p - 1./54.*m13*m66*RN13*RS34561pe - 1./
         54.*m13*m66*RN13*RS345pe + 1./27.*m13*m16*RN1p*RSp45613e - 1./
         27.*m13*m16*RN1p*RSp453e - 1./27.*m13*m16*RN1p*RSp45611e + 1./
         27.*m13*m16*RN1p*RSp451e + 1./27.*m13*m16*RN1e*RSe45613p - 1./
         27.*m13*m16*RN1e*RSe453p - 1./27.*m13*m16*RN1e*RSe45611p + 1./
         27.*m13*m16*RN1e*RSe451p + 1./27.*m13*m16*RN13*RS34561pe - 1./
         27.*m13*m16*RN13*RS345pe + 1./27.*m11*RS34561pe + 43./540.*m11
         *RS14563pe + 1./27.*m11*RSe45613p + 1./27.*m11*RSp45613e - 1./
         27.*m11*RS345pe - 1./27.*m11*RSe453p - 1./27.*m11*RSp453e + 13.
         /1080.*m11*RC1456pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m16,L) * (  - 1./27.*m11*
         RSe45611p - 1./27.*m11*RSp45611e - 1./45.*m11*RS145pe + 1./27.
         *m11*RSe451p + 1./27.*m11*RSp451e + 1./27.*m11*me2*RN1e*
         RSe45613p - 1./27.*m11*me2*RN1e*RSe453p - 1./27.*m11*me2*RN1e*
         RSe45611p + 1./27.*m11*me2*RN1e*RSe451p + 1./27.*m11*mp2*RN1p*
         RSp45613e - 1./27.*m11*mp2*RN1p*RSp453e - 1./27.*m11*mp2*RN1p*
         RSp45611e + 1./27.*m11*mp2*RN1p*RSp451e - 1./36.*m11*m66*RN1p*
         RSp45613e - 1./36.*m11*m66*RN1p*RSp453e - 1./108.*m11*m66*RN1p
         *RSp45611e - 1./108.*m11*m66*RN1p*RSp451e - 1./36.*m11*m66*
         RN1e*RSe45613p - 1./36.*m11*m66*RN1e*RSe453p - 1./108.*m11*m66
         *RN1e*RSe45611p - 1./108.*m11*m66*RN1e*RSe451p - 1./36.*m11*
         m66*RN13*RS34561pe - 1./36.*m11*m66*RN13*RS345pe - 1./108.*m11
         *m33*RN1p*RSp45613e - 1./36.*m11*m33*RN1p*RSp453e - 1./108.*
         m11*m33*RN1p*RSp45611e - 1./36.*m11*m33*RN1p*RSp451e - 1./108.
         *m11*m33*RN1e*RSe45613p - 1./36.*m11*m33*RN1e*RSe453p - 1./108.
         *m11*m33*RN1e*RSe45611p );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m16,L) * (  - 1./36.*m11*m33*
         RN1e*RSe451p + 1./36.*m11*m33*RN13*RS34561pe - 7./108.*m11*m33
         *RN13*RS345pe + 1./18.*m11*m16*RN1p*RSp45613e - 1./18.*m11*m16
         *RN1p*RSp453e + 1./54.*m11*m16*RN1p*RSp45611e - 1./54.*m11*m16
         *RN1p*RSp451e + 1./18.*m11*m16*RN1e*RSe45613p - 1./18.*m11*m16
         *RN1e*RSe453p + 1./54.*m11*m16*RN1e*RSe45611p - 1./54.*m11*m16
         *RN1e*RSe451p + 1./18.*m11*m16*RN13*RS34561pe - 1./18.*m11*m16
         *RN13*RS345pe - 1./54.*m11*m13*RN1p*RSp45613e - 1./18.*m11*m13
         *RN1p*RSp453e + 1./54.*m11*m13*RN1p*RSp45611e + 1./18.*m11*m13
         *RN1p*RSp451e - 1./54.*m11*m13*RN1e*RSe45613p - 1./18.*m11*m13
         *RN1e*RSe453p + 1./54.*m11*m13*RN1e*RSe45611p + 1./18.*m11*m13
         *RN1e*RSe451p - 1./54.*m11*m13*RN13*RS34561pe - 1./18.*m11*m13
         *RN13*RS345pe - 7./108.*pow(m11,2)*RN1p*RSp45613e - 5./108.*
         pow(m11,2)*RN1p*RSp453e + 1./36.*pow(m11,2)*RN1p*RSp45611e - 7.
         /108.*pow(m11,2)*RN1p*RSp451e - 7./108.*pow(m11,2)*RN1e*
         RSe45613p );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m16,L) * (  - 5./108.*pow(m11,2)
         *RN1e*RSe453p + 1./36.*pow(m11,2)*RN1e*RSe45611p - 7./108.*
         pow(m11,2)*RN1e*RSe451p - 7./108.*pow(m11,2)*RN13*RS34561pe - 
         5./108.*pow(m11,2)*RN13*RS345pe + 1./54.*RS(m11,m44,m55,m66,
         m33,me2,mp2)*m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11
          );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m33,L) * (  - 1./270.*RD3456ep*
         RS14563pe + 1./540.*RC1456pe*RD3456ep - 1./270.*RD1456ep*
         RS34561pe + 1./540.*RD1456ep*RC3456pe + 11./405.*m33*RS14563pe
         *RS34561pe - 2./135.*m33*RC3456pe*RS14563pe + 1./162.*m33*
         RC1456pe*RS34561pe - 19./1620.*m33*RC1456pe*RC3456pe + 1./54.*
         pow(m33,2)*RN13*RC3456pe*RS14563pe - 7./405.*m13*RS14563pe*
         RS34561pe + 7./810.*m13*RC3456pe*RS14563pe + 7./810.*m13*
         RC1456pe*RS34561pe - 7./1620.*m13*RC1456pe*RC3456pe + 1./27.*
         m13*m33*RN13*RC3456pe*RS14563pe + 11./405.*m11*RS14563pe*
         RS34561pe + 1./162.*m11*RC3456pe*RS14563pe + 1./45.*m11*
         RC1456pe*RS34561pe - 19./1620.*m11*RC1456pe*RC3456pe + 1./18.*
         m11*m33*RN13*RC3456pe*RS14563pe + 1./81.*RS(m11,m44,m55,m66,
         m33,me2,mp2)*m33*RS34561pe - 1./162.*RS(m11,m44,m55,m66,m33,
         me2,mp2)*m33*RC3456pe + 2./81.*RS(m11,m44,m55,m66,m33,me2,mp2)
         *m11*RS34561pe - 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*
         RC3456pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m33,L) * ( 2./81.*RS(m33,m44,m55
         ,m66,m11,me2,mp2)*m33*RS14563pe - 1./81.*RS(m33,m44,m55,m66,
         m11,me2,mp2)*m33*RC1456pe + 1./81.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11*RS14563pe - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*
         m11*RC1456pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m34,L) * (  - 1./180.*RD1456ep
          + 1./180.*m44*RS14563pe + 1./60.*m44*RC1456pe + 13./540.*m44*
         RS156pe - 13./270.*m34*RS14563pe + 1./270.*m34*RC1456pe + 11./
         270.*m34*RS156pe + 1./60.*m33*RS14563pe + 2./135.*m33*RC1456pe
          + 1./180.*m33*RS156pe - 1./108.*m33*m44*RN13*RS14563pe - 1./
         108.*m33*m44*RN13*RS156pe + 1./54.*m33*m34*RN13*RS14563pe - 1./
         54.*m33*m34*RN13*RS156pe - 1./36.*pow(m33,2)*RN13*RS14563pe - 
         1./108.*pow(m33,2)*RN13*RS156pe + 1./27.*m14*RS14563pe - 1./27.
         *m14*RC1456pe + 1./27.*m14*RS156pe - 31./540.*m13*RS14563pe + 
         1./20.*m13*RC1456pe - 23./540.*m13*RS156pe - 1./54.*m13*m44*
         RN13*RS14563pe - 1./54.*m13*m44*RN13*RS156pe + 1./27.*m13*m34*
         RN13*RS14563pe - 1./27.*m13*m34*RN13*RS156pe - 1./18.*m13*m33*
         RN13*RS14563pe - 1./54.*m13*m33*RN13*RS156pe + 1./18.*m11*
         RS14563pe + 19./540.*m11*RC1456pe - 2./27.*m11*RS156pe - 1./36.
         *m11*m44*RN13*RS14563pe - 1./36.*m11*m44*RN13*RS156pe + 1./18.
         *m11*m34*RN13*RS14563pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m34,L) * (  - 1./18.*m11*m34*
         RN13*RS156pe - 1./36.*m11*m33*RN13*RS14563pe - 1./12.*m11*m33*
         RN13*RS156pe + 1./27.*m11*m13*RN13*RS14563pe - 1./27.*m11*m13*
         RN13*RS156pe + 1./54.*pow(m11,2)*RN13*RS14563pe - 1./54.*pow(
         m11,2)*RN13*RS156pe + 1./54.*RS(m11,m44,m55,m66,m33,me2,mp2)*
         m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11 );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m35,L) * (  - 1./180.*RD1456ep
          + 1./180.*m55*RS14563pe + 1./60.*m55*RC1456pe + 13./540.*m55*
         RS146pe - 13./270.*m35*RS14563pe + 1./270.*m35*RC1456pe + 11./
         270.*m35*RS146pe + 1./60.*m33*RS14563pe + 2./135.*m33*RC1456pe
          + 1./180.*m33*RS146pe - 1./108.*m33*m55*RN13*RS14563pe - 1./
         108.*m33*m55*RN13*RS146pe + 1./54.*m33*m35*RN13*RS14563pe - 1./
         54.*m33*m35*RN13*RS146pe - 1./36.*pow(m33,2)*RN13*RS14563pe - 
         1./108.*pow(m33,2)*RN13*RS146pe + 1./27.*m15*RS14563pe - 1./27.
         *m15*RC1456pe + 1./27.*m15*RS146pe - 31./540.*m13*RS14563pe + 
         1./20.*m13*RC1456pe - 23./540.*m13*RS146pe - 1./54.*m13*m55*
         RN13*RS14563pe - 1./54.*m13*m55*RN13*RS146pe + 1./27.*m13*m35*
         RN13*RS14563pe - 1./27.*m13*m35*RN13*RS146pe - 1./18.*m13*m33*
         RN13*RS14563pe - 1./54.*m13*m33*RN13*RS146pe + 1./18.*m11*
         RS14563pe + 19./540.*m11*RC1456pe - 2./27.*m11*RS146pe - 1./36.
         *m11*m55*RN13*RS14563pe - 1./36.*m11*m55*RN13*RS146pe + 1./18.
         *m11*m35*RN13*RS14563pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m35,L) * (  - 1./18.*m11*m35*
         RN13*RS146pe - 1./36.*m11*m33*RN13*RS14563pe - 1./12.*m11*m33*
         RN13*RS146pe + 1./27.*m11*m13*RN13*RS14563pe - 1./27.*m11*m13*
         RN13*RS146pe + 1./54.*pow(m11,2)*RN13*RS14563pe - 1./54.*pow(
         m11,2)*RN13*RS146pe + 1./54.*RS(m11,m44,m55,m66,m33,me2,mp2)*
         m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11 );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m36,L) * (  - 1./180.*RD1456ep
          + 1./180.*m66*RS14563pe + 1./60.*m66*RC1456pe + 13./540.*m66*
         RS145pe - 13./270.*m36*RS14563pe + 1./270.*m36*RC1456pe + 11./
         270.*m36*RS145pe + 1./60.*m33*RS14563pe + 2./135.*m33*RC1456pe
          + 1./180.*m33*RS145pe - 1./108.*m33*m66*RN13*RS14563pe - 1./
         108.*m33*m66*RN13*RS145pe + 1./54.*m33*m36*RN13*RS14563pe - 1./
         54.*m33*m36*RN13*RS145pe - 1./36.*pow(m33,2)*RN13*RS14563pe - 
         1./108.*pow(m33,2)*RN13*RS145pe + 1./27.*m16*RS14563pe - 1./27.
         *m16*RC1456pe + 1./27.*m16*RS145pe - 31./540.*m13*RS14563pe + 
         1./20.*m13*RC1456pe - 23./540.*m13*RS145pe - 1./54.*m13*m66*
         RN13*RS14563pe - 1./54.*m13*m66*RN13*RS145pe + 1./27.*m13*m36*
         RN13*RS14563pe - 1./27.*m13*m36*RN13*RS145pe - 1./18.*m13*m33*
         RN13*RS14563pe - 1./54.*m13*m33*RN13*RS145pe + 1./18.*m11*
         RS14563pe + 19./540.*m11*RC1456pe - 2./27.*m11*RS145pe - 1./36.
         *m11*m66*RN13*RS14563pe - 1./36.*m11*m66*RN13*RS145pe + 1./18.
         *m11*m36*RN13*RS14563pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m36,L) * (  - 1./18.*m11*m36*
         RN13*RS145pe - 1./36.*m11*m33*RN13*RS14563pe - 1./12.*m11*m33*
         RN13*RS145pe + 1./27.*m11*m13*RN13*RS14563pe - 1./27.*m11*m13*
         RN13*RS145pe + 1./54.*pow(m11,2)*RN13*RS14563pe - 1./54.*pow(
         m11,2)*RN13*RS145pe + 1./54.*RS(m11,m44,m55,m66,m33,me2,mp2)*
         m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11 );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m44,L) * (  - 1./54.*m44*pow(
         RS156pe,2) + 1./162.*m44*RS456pe*pow(RS156pe,2) + 1./54.*m33*
         m44*RN1p*RS156pe*RSp563e - 1./162.*m33*m44*RN1p*RS456pe*
         RS156pe*RSp563e + 1./54.*m33*m44*RN1p*RSp561e*RS156pe - 1./162.
         *m33*m44*RN1p*RSp561e*RS456pe*RS156pe + 1./54.*m33*m44*RN1e*
         RS156pe*RSe563p - 1./162.*m33*m44*RN1e*RS456pe*RS156pe*RSe563p
          + 1./54.*m33*m44*RN1e*RSe561p*RS156pe - 1./162.*m33*m44*RN1e*
         RSe561p*RS456pe*RS156pe + 1./54.*m33*m44*RN13*RS156pe*RS356pe
          - 1./162.*m33*m44*RN13*RS456pe*RS156pe*RS356pe + 1./27.*m13*
         m44*RN1p*RS156pe*RSp563e - 1./81.*m13*m44*RN1p*RS456pe*RS156pe
         *RSp563e - 1./27.*m13*m44*RN1p*RSp561e*RS156pe + 1./81.*m13*
         m44*RN1p*RSp561e*RS456pe*RS156pe + 1./27.*m13*m44*RN1e*RS156pe
         *RSe563p - 1./81.*m13*m44*RN1e*RS456pe*RS156pe*RSe563p - 1./27.
         *m13*m44*RN1e*RSe561p*RS156pe + 1./81.*m13*m44*RN1e*RSe561p*
         RS456pe*RS156pe + 1./27.*m13*m44*RN13*RS156pe*RS356pe - 1./81.
         *m13*m44*RN13*RS456pe*RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m44,L) * ( 1./18.*m11*m44*RN1p*
         RS156pe*RSp563e - 1./54.*m11*m44*RN1p*RS456pe*RS156pe*RSp563e
          + 1./54.*m11*m44*RN1p*RSp561e*RS156pe - 1./162.*m11*m44*RN1p*
         RSp561e*RS456pe*RS156pe + 1./18.*m11*m44*RN1e*RS156pe*RSe563p
          - 1./54.*m11*m44*RN1e*RS456pe*RS156pe*RSe563p + 1./54.*m11*
         m44*RN1e*RSe561p*RS156pe - 1./162.*m11*m44*RN1e*RSe561p*
         RS456pe*RS156pe + 1./18.*m11*m44*RN13*RS156pe*RS356pe - 1./54.
         *m11*m44*RN13*RS456pe*RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * (  - 1./108.*m55*pow(
         RS146pe,2) - 1./162.*m55*RS156pe*RS146pe - 1./324.*m55*pow(
         RS156pe,2) + 1./162.*m45*pow(RS146pe,2) - 1./81.*m45*RS156pe*
         RS146pe + 1./162.*m45*pow(RS156pe,2) - 1./324.*m44*pow(
         RS146pe,2) - 1./162.*m44*RS156pe*RS146pe - 1./108.*m44*pow(
         RS156pe,2) - 1./324.*m33*pow(RS146pe,2) + 1./162.*m33*RS156pe*
         RS146pe - 1./324.*m33*pow(RS156pe,2) + 1./324.*m33*m55*RN1p*
         RS146pe*RSp563e + 1./108.*m33*m55*RN1p*RS146pe*RSp463e + 1./
         324.*m33*m55*RN1p*RS156pe*RSp563e + 1./324.*m33*m55*RN1p*
         RS156pe*RSp463e + 1./324.*m33*m55*RN1p*RSp561e*RS146pe + 1./
         324.*m33*m55*RN1p*RSp561e*RS156pe + 1./108.*m33*m55*RN1p*
         RSp461e*RS146pe + 1./324.*m33*m55*RN1p*RSp461e*RS156pe + 1./
         324.*m33*m55*RN1e*RS146pe*RSe563p + 1./108.*m33*m55*RN1e*
         RS146pe*RSe463p + 1./324.*m33*m55*RN1e*RS156pe*RSe563p + 1./
         324.*m33*m55*RN1e*RS156pe*RSe463p + 1./324.*m33*m55*RN1e*
         RSe561p*RS146pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * ( 1./324.*m33*m55*RN1e*
         RSe561p*RS156pe + 1./108.*m33*m55*RN1e*RSe461p*RS146pe + 1./
         324.*m33*m55*RN1e*RSe461p*RS156pe + 1./108.*m33*m55*RN13*
         RS146pe*RS346pe + 1./324.*m33*m55*RN13*RS146pe*RS356pe + 1./
         324.*m33*m55*RN13*RS156pe*RS346pe + 1./324.*m33*m55*RN13*
         RS156pe*RS356pe + 1./162.*m33*m45*RN1p*RS146pe*RSp563e - 1./
         162.*m33*m45*RN1p*RS146pe*RSp463e - 1./162.*m33*m45*RN1p*
         RS156pe*RSp563e + 1./162.*m33*m45*RN1p*RS156pe*RSp463e + 1./
         162.*m33*m45*RN1p*RSp561e*RS146pe - 1./162.*m33*m45*RN1p*
         RSp561e*RS156pe - 1./162.*m33*m45*RN1p*RSp461e*RS146pe + 1./
         162.*m33*m45*RN1p*RSp461e*RS156pe + 1./162.*m33*m45*RN1e*
         RS146pe*RSe563p - 1./162.*m33*m45*RN1e*RS146pe*RSe463p - 1./
         162.*m33*m45*RN1e*RS156pe*RSe563p + 1./162.*m33*m45*RN1e*
         RS156pe*RSe463p + 1./162.*m33*m45*RN1e*RSe561p*RS146pe - 1./
         162.*m33*m45*RN1e*RSe561p*RS156pe - 1./162.*m33*m45*RN1e*
         RSe461p*RS146pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * ( 1./162.*m33*m45*RN1e*
         RSe461p*RS156pe - 1./162.*m33*m45*RN13*RS146pe*RS346pe + 1./
         162.*m33*m45*RN13*RS146pe*RS356pe + 1./162.*m33*m45*RN13*
         RS156pe*RS346pe - 1./162.*m33*m45*RN13*RS156pe*RS356pe + 1./
         324.*m33*m44*RN1p*RS146pe*RSp563e + 1./324.*m33*m44*RN1p*
         RS146pe*RSp463e + 1./108.*m33*m44*RN1p*RS156pe*RSp563e + 1./
         324.*m33*m44*RN1p*RS156pe*RSp463e + 1./324.*m33*m44*RN1p*
         RSp561e*RS146pe + 1./108.*m33*m44*RN1p*RSp561e*RS156pe + 1./
         324.*m33*m44*RN1p*RSp461e*RS146pe + 1./324.*m33*m44*RN1p*
         RSp461e*RS156pe + 1./324.*m33*m44*RN1e*RS146pe*RSe563p + 1./
         324.*m33*m44*RN1e*RS146pe*RSe463p + 1./108.*m33*m44*RN1e*
         RS156pe*RSe563p + 1./324.*m33*m44*RN1e*RS156pe*RSe463p + 1./
         324.*m33*m44*RN1e*RSe561p*RS146pe + 1./108.*m33*m44*RN1e*
         RSe561p*RS156pe + 1./324.*m33*m44*RN1e*RSe461p*RS146pe + 1./
         324.*m33*m44*RN1e*RSe461p*RS156pe + 1./324.*m33*m44*RN13*
         RS146pe*RS346pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * ( 1./324.*m33*m44*RN13*
         RS146pe*RS356pe + 1./324.*m33*m44*RN13*RS156pe*RS346pe + 1./
         108.*m33*m44*RN13*RS156pe*RS356pe + 1./162.*m13*pow(RS146pe,2)
          - 1./81.*m13*RS156pe*RS146pe + 1./162.*m13*pow(RS156pe,2) + 1.
         /162.*m13*m55*RN1p*RS146pe*RSp563e + 1./54.*m13*m55*RN1p*
         RS146pe*RSp463e + 1./162.*m13*m55*RN1p*RS156pe*RSp563e + 1./
         162.*m13*m55*RN1p*RS156pe*RSp463e - 1./162.*m13*m55*RN1p*
         RSp561e*RS146pe - 1./162.*m13*m55*RN1p*RSp561e*RS156pe - 1./54.
         *m13*m55*RN1p*RSp461e*RS146pe - 1./162.*m13*m55*RN1p*RSp461e*
         RS156pe + 1./162.*m13*m55*RN1e*RS146pe*RSe563p + 1./54.*m13*
         m55*RN1e*RS146pe*RSe463p + 1./162.*m13*m55*RN1e*RS156pe*
         RSe563p + 1./162.*m13*m55*RN1e*RS156pe*RSe463p - 1./162.*m13*
         m55*RN1e*RSe561p*RS146pe - 1./162.*m13*m55*RN1e*RSe561p*
         RS156pe - 1./54.*m13*m55*RN1e*RSe461p*RS146pe - 1./162.*m13*
         m55*RN1e*RSe461p*RS156pe + 1./54.*m13*m55*RN13*RS146pe*RS346pe
          + 1./162.*m13*m55*RN13*RS146pe*RS356pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * ( 1./162.*m13*m55*RN13*
         RS156pe*RS346pe + 1./162.*m13*m55*RN13*RS156pe*RS356pe + 1./81.
         *m13*m45*RN1p*RS146pe*RSp563e - 1./81.*m13*m45*RN1p*RS146pe*
         RSp463e - 1./81.*m13*m45*RN1p*RS156pe*RSp563e + 1./81.*m13*m45
         *RN1p*RS156pe*RSp463e - 1./81.*m13*m45*RN1p*RSp561e*RS146pe + 
         1./81.*m13*m45*RN1p*RSp561e*RS156pe + 1./81.*m13*m45*RN1p*
         RSp461e*RS146pe - 1./81.*m13*m45*RN1p*RSp461e*RS156pe + 1./81.
         *m13*m45*RN1e*RS146pe*RSe563p - 1./81.*m13*m45*RN1e*RS146pe*
         RSe463p - 1./81.*m13*m45*RN1e*RS156pe*RSe563p + 1./81.*m13*m45
         *RN1e*RS156pe*RSe463p - 1./81.*m13*m45*RN1e*RSe561p*RS146pe + 
         1./81.*m13*m45*RN1e*RSe561p*RS156pe + 1./81.*m13*m45*RN1e*
         RSe461p*RS146pe - 1./81.*m13*m45*RN1e*RSe461p*RS156pe - 1./81.
         *m13*m45*RN13*RS146pe*RS346pe + 1./81.*m13*m45*RN13*RS146pe*
         RS356pe + 1./81.*m13*m45*RN13*RS156pe*RS346pe - 1./81.*m13*m45
         *RN13*RS156pe*RS356pe + 1./162.*m13*m44*RN1p*RS146pe*RSp563e
          + 1./162.*m13*m44*RN1p*RS146pe*RSp463e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * ( 1./54.*m13*m44*RN1p*
         RS156pe*RSp563e + 1./162.*m13*m44*RN1p*RS156pe*RSp463e - 1./
         162.*m13*m44*RN1p*RSp561e*RS146pe - 1./54.*m13*m44*RN1p*
         RSp561e*RS156pe - 1./162.*m13*m44*RN1p*RSp461e*RS146pe - 1./
         162.*m13*m44*RN1p*RSp461e*RS156pe + 1./162.*m13*m44*RN1e*
         RS146pe*RSe563p + 1./162.*m13*m44*RN1e*RS146pe*RSe463p + 1./54.
         *m13*m44*RN1e*RS156pe*RSe563p + 1./162.*m13*m44*RN1e*RS156pe*
         RSe463p - 1./162.*m13*m44*RN1e*RSe561p*RS146pe - 1./54.*m13*
         m44*RN1e*RSe561p*RS156pe - 1./162.*m13*m44*RN1e*RSe461p*
         RS146pe - 1./162.*m13*m44*RN1e*RSe461p*RS156pe + 1./162.*m13*
         m44*RN13*RS146pe*RS346pe + 1./162.*m13*m44*RN13*RS146pe*
         RS356pe + 1./162.*m13*m44*RN13*RS156pe*RS346pe + 1./54.*m13*
         m44*RN13*RS156pe*RS356pe - 1./81.*m11*RS146pe*RS346pe + 1./81.
         *m11*RS146pe*RS356pe + 1./81.*m11*RS146pe*RSe563p - 1./81.*m11
         *RS146pe*RSe463p + 1./81.*m11*RS146pe*RSp563e - 1./81.*m11*
         RS146pe*RSp463e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * ( 1./324.*m11*pow(
         RS146pe,2) + 1./81.*m11*RS156pe*RS346pe - 1./81.*m11*RS156pe*
         RS356pe - 1./81.*m11*RS156pe*RSe563p + 1./81.*m11*RS156pe*
         RSe463p - 1./81.*m11*RS156pe*RSp563e + 1./81.*m11*RS156pe*
         RSp463e - 1./162.*m11*RS156pe*RS146pe + 1./324.*m11*pow(
         RS156pe,2) - 1./81.*m11*RSe561p*RS146pe + 1./81.*m11*RSe561p*
         RS156pe + 1./81.*m11*RSe461p*RS146pe - 1./81.*m11*RSe461p*
         RS156pe - 1./81.*m11*RSp561e*RS146pe + 1./81.*m11*RSp561e*
         RS156pe + 1./81.*m11*RSp461e*RS146pe - 1./81.*m11*RSp461e*
         RS156pe + 1./81.*m11*me2*RN1e*RS146pe*RSe563p - 1./81.*m11*me2
         *RN1e*RS146pe*RSe463p - 1./81.*m11*me2*RN1e*RS156pe*RSe563p + 
         1./81.*m11*me2*RN1e*RS156pe*RSe463p - 1./81.*m11*me2*RN1e*
         RSe561p*RS146pe + 1./81.*m11*me2*RN1e*RSe561p*RS156pe + 1./81.
         *m11*me2*RN1e*RSe461p*RS146pe - 1./81.*m11*me2*RN1e*RSe461p*
         RS156pe + 1./81.*m11*mp2*RN1p*RS146pe*RSp563e - 1./81.*m11*mp2
         *RN1p*RS146pe*RSp463e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * (  - 1./81.*m11*mp2*
         RN1p*RS156pe*RSp563e + 1./81.*m11*mp2*RN1p*RS156pe*RSp463e - 1.
         /81.*m11*mp2*RN1p*RSp561e*RS146pe + 1./81.*m11*mp2*RN1p*
         RSp561e*RS156pe + 1./81.*m11*mp2*RN1p*RSp461e*RS146pe - 1./81.
         *m11*mp2*RN1p*RSp461e*RS156pe + 1./108.*m11*m55*RN1p*RS146pe*
         RSp563e + 1./36.*m11*m55*RN1p*RS146pe*RSp463e + 1./108.*m11*
         m55*RN1p*RS156pe*RSp563e + 1./108.*m11*m55*RN1p*RS156pe*
         RSp463e + 1./324.*m11*m55*RN1p*RSp561e*RS146pe + 1./324.*m11*
         m55*RN1p*RSp561e*RS156pe + 1./108.*m11*m55*RN1p*RSp461e*
         RS146pe + 1./324.*m11*m55*RN1p*RSp461e*RS156pe + 1./108.*m11*
         m55*RN1e*RS146pe*RSe563p + 1./36.*m11*m55*RN1e*RS146pe*RSe463p
          + 1./108.*m11*m55*RN1e*RS156pe*RSe563p + 1./108.*m11*m55*RN1e
         *RS156pe*RSe463p + 1./324.*m11*m55*RN1e*RSe561p*RS146pe + 1./
         324.*m11*m55*RN1e*RSe561p*RS156pe + 1./108.*m11*m55*RN1e*
         RSe461p*RS146pe + 1./324.*m11*m55*RN1e*RSe461p*RS156pe + 1./36.
         *m11*m55*RN13*RS146pe*RS346pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * ( 1./108.*m11*m55*RN13*
         RS146pe*RS356pe + 1./108.*m11*m55*RN13*RS156pe*RS346pe + 1./
         108.*m11*m55*RN13*RS156pe*RS356pe + 1./54.*m11*m45*RN1p*
         RS146pe*RSp563e - 1./54.*m11*m45*RN1p*RS146pe*RSp463e - 1./54.
         *m11*m45*RN1p*RS156pe*RSp563e + 1./54.*m11*m45*RN1p*RS156pe*
         RSp463e + 1./162.*m11*m45*RN1p*RSp561e*RS146pe - 1./162.*m11*
         m45*RN1p*RSp561e*RS156pe - 1./162.*m11*m45*RN1p*RSp461e*
         RS146pe + 1./162.*m11*m45*RN1p*RSp461e*RS156pe + 1./54.*m11*
         m45*RN1e*RS146pe*RSe563p - 1./54.*m11*m45*RN1e*RS146pe*RSe463p
          - 1./54.*m11*m45*RN1e*RS156pe*RSe563p + 1./54.*m11*m45*RN1e*
         RS156pe*RSe463p + 1./162.*m11*m45*RN1e*RSe561p*RS146pe - 1./
         162.*m11*m45*RN1e*RSe561p*RS156pe - 1./162.*m11*m45*RN1e*
         RSe461p*RS146pe + 1./162.*m11*m45*RN1e*RSe461p*RS156pe - 1./54.
         *m11*m45*RN13*RS146pe*RS346pe + 1./54.*m11*m45*RN13*RS146pe*
         RS356pe + 1./54.*m11*m45*RN13*RS156pe*RS346pe - 1./54.*m11*m45
         *RN13*RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * ( 1./108.*m11*m44*RN1p*
         RS146pe*RSp563e + 1./108.*m11*m44*RN1p*RS146pe*RSp463e + 1./36.
         *m11*m44*RN1p*RS156pe*RSp563e + 1./108.*m11*m44*RN1p*RS156pe*
         RSp463e + 1./324.*m11*m44*RN1p*RSp561e*RS146pe + 1./108.*m11*
         m44*RN1p*RSp561e*RS156pe + 1./324.*m11*m44*RN1p*RSp461e*
         RS146pe + 1./324.*m11*m44*RN1p*RSp461e*RS156pe + 1./108.*m11*
         m44*RN1e*RS146pe*RSe563p + 1./108.*m11*m44*RN1e*RS146pe*
         RSe463p + 1./36.*m11*m44*RN1e*RS156pe*RSe563p + 1./108.*m11*
         m44*RN1e*RS156pe*RSe463p + 1./324.*m11*m44*RN1e*RSe561p*
         RS146pe + 1./108.*m11*m44*RN1e*RSe561p*RS156pe + 1./324.*m11*
         m44*RN1e*RSe461p*RS146pe + 1./324.*m11*m44*RN1e*RSe461p*
         RS156pe + 1./108.*m11*m44*RN13*RS146pe*RS346pe + 1./108.*m11*
         m44*RN13*RS146pe*RS356pe + 1./108.*m11*m44*RN13*RS156pe*
         RS346pe + 1./36.*m11*m44*RN13*RS156pe*RS356pe + 1./162.*m11*
         m33*RN1p*RS146pe*RSp563e - 1./162.*m11*m33*RN1p*RS146pe*
         RSp463e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * (  - 1./162.*m11*m33*
         RN1p*RS156pe*RSp563e + 1./162.*m11*m33*RN1p*RS156pe*RSp463e + 
         1./162.*m11*m33*RN1p*RSp561e*RS146pe - 1./162.*m11*m33*RN1p*
         RSp561e*RS156pe - 1./162.*m11*m33*RN1p*RSp461e*RS146pe + 1./
         162.*m11*m33*RN1p*RSp461e*RS156pe + 1./162.*m11*m33*RN1e*
         RS146pe*RSe563p - 1./162.*m11*m33*RN1e*RS146pe*RSe463p - 1./
         162.*m11*m33*RN1e*RS156pe*RSe563p + 1./162.*m11*m33*RN1e*
         RS156pe*RSe463p + 1./162.*m11*m33*RN1e*RSe561p*RS146pe - 1./
         162.*m11*m33*RN1e*RSe561p*RS156pe - 1./162.*m11*m33*RN1e*
         RSe461p*RS146pe + 1./162.*m11*m33*RN1e*RSe461p*RS156pe - 1./54.
         *m11*m33*RN13*RS146pe*RS346pe + 1./54.*m11*m33*RN13*RS146pe*
         RS356pe + 1./54.*m11*m33*RN13*RS156pe*RS346pe - 1./54.*m11*m33
         *RN13*RS156pe*RS356pe + 1./81.*m11*m13*RN1p*RS146pe*RSp563e - 
         1./81.*m11*m13*RN1p*RS146pe*RSp463e - 1./81.*m11*m13*RN1p*
         RS156pe*RSp563e + 1./81.*m11*m13*RN1p*RS156pe*RSp463e - 1./81.
         *m11*m13*RN1p*RSp561e*RS146pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * ( 1./81.*m11*m13*RN1p*
         RSp561e*RS156pe + 1./81.*m11*m13*RN1p*RSp461e*RS146pe - 1./81.
         *m11*m13*RN1p*RSp461e*RS156pe + 1./81.*m11*m13*RN1e*RS146pe*
         RSe563p - 1./81.*m11*m13*RN1e*RS146pe*RSe463p - 1./81.*m11*m13
         *RN1e*RS156pe*RSe563p + 1./81.*m11*m13*RN1e*RS156pe*RSe463p - 
         1./81.*m11*m13*RN1e*RSe561p*RS146pe + 1./81.*m11*m13*RN1e*
         RSe561p*RS156pe + 1./81.*m11*m13*RN1e*RSe461p*RS146pe - 1./81.
         *m11*m13*RN1e*RSe461p*RS156pe - 1./81.*m11*m13*RN13*RS146pe*
         RS346pe + 1./81.*m11*m13*RN13*RS146pe*RS356pe + 1./81.*m11*m13
         *RN13*RS156pe*RS346pe - 1./81.*m11*m13*RN13*RS156pe*RS356pe + 
         1./162.*pow(m11,2)*RN1p*RS146pe*RSp563e - 1./162.*pow(m11,2)*
         RN1p*RS146pe*RSp463e - 1./162.*pow(m11,2)*RN1p*RS156pe*RSp563e
          + 1./162.*pow(m11,2)*RN1p*RS156pe*RSp463e + 1./54.*pow(m11,2)
         *RN1p*RSp561e*RS146pe - 1./54.*pow(m11,2)*RN1p*RSp561e*RS156pe
          - 1./54.*pow(m11,2)*RN1p*RSp461e*RS146pe + 1./54.*pow(m11,2)*
         RN1p*RSp461e*RS156pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * ( 1./162.*pow(m11,2)*
         RN1e*RS146pe*RSe563p - 1./162.*pow(m11,2)*RN1e*RS146pe*RSe463p
          - 1./162.*pow(m11,2)*RN1e*RS156pe*RSe563p + 1./162.*pow(
         m11,2)*RN1e*RS156pe*RSe463p + 1./54.*pow(m11,2)*RN1e*RSe561p*
         RS146pe - 1./54.*pow(m11,2)*RN1e*RSe561p*RS156pe - 1./54.*pow(
         m11,2)*RN1e*RSe461p*RS146pe + 1./54.*pow(m11,2)*RN1e*RSe461p*
         RS156pe - 1./162.*pow(m11,2)*RN13*RS146pe*RS346pe + 1./162.*
         pow(m11,2)*RN13*RS146pe*RS356pe + 1./162.*pow(m11,2)*RN13*
         RS156pe*RS346pe - 1./162.*pow(m11,2)*RN13*RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * (  - 1./108.*m66*pow(
         RS145pe,2) - 1./162.*m66*RS156pe*RS145pe - 1./324.*m66*pow(
         RS156pe,2) + 1./162.*m46*pow(RS145pe,2) - 1./81.*m46*RS156pe*
         RS145pe + 1./162.*m46*pow(RS156pe,2) - 1./324.*m44*pow(
         RS145pe,2) - 1./162.*m44*RS156pe*RS145pe - 1./108.*m44*pow(
         RS156pe,2) - 1./324.*m33*pow(RS145pe,2) + 1./162.*m33*RS156pe*
         RS145pe - 1./324.*m33*pow(RS156pe,2) + 1./324.*m33*m66*RN1p*
         RS145pe*RSp563e + 1./108.*m33*m66*RN1p*RS145pe*RSp453e + 1./
         324.*m33*m66*RN1p*RS156pe*RSp563e + 1./324.*m33*m66*RN1p*
         RS156pe*RSp453e + 1./324.*m33*m66*RN1p*RSp561e*RS145pe + 1./
         324.*m33*m66*RN1p*RSp561e*RS156pe + 1./108.*m33*m66*RN1p*
         RSp451e*RS145pe + 1./324.*m33*m66*RN1p*RSp451e*RS156pe + 1./
         324.*m33*m66*RN1e*RS145pe*RSe563p + 1./108.*m33*m66*RN1e*
         RS145pe*RSe453p + 1./324.*m33*m66*RN1e*RS156pe*RSe563p + 1./
         324.*m33*m66*RN1e*RS156pe*RSe453p + 1./324.*m33*m66*RN1e*
         RSe561p*RS145pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * ( 1./324.*m33*m66*RN1e*
         RSe561p*RS156pe + 1./108.*m33*m66*RN1e*RSe451p*RS145pe + 1./
         324.*m33*m66*RN1e*RSe451p*RS156pe + 1./108.*m33*m66*RN13*
         RS145pe*RS345pe + 1./324.*m33*m66*RN13*RS145pe*RS356pe + 1./
         324.*m33*m66*RN13*RS156pe*RS345pe + 1./324.*m33*m66*RN13*
         RS156pe*RS356pe + 1./162.*m33*m46*RN1p*RS145pe*RSp563e - 1./
         162.*m33*m46*RN1p*RS145pe*RSp453e - 1./162.*m33*m46*RN1p*
         RS156pe*RSp563e + 1./162.*m33*m46*RN1p*RS156pe*RSp453e + 1./
         162.*m33*m46*RN1p*RSp561e*RS145pe - 1./162.*m33*m46*RN1p*
         RSp561e*RS156pe - 1./162.*m33*m46*RN1p*RSp451e*RS145pe + 1./
         162.*m33*m46*RN1p*RSp451e*RS156pe + 1./162.*m33*m46*RN1e*
         RS145pe*RSe563p - 1./162.*m33*m46*RN1e*RS145pe*RSe453p - 1./
         162.*m33*m46*RN1e*RS156pe*RSe563p + 1./162.*m33*m46*RN1e*
         RS156pe*RSe453p + 1./162.*m33*m46*RN1e*RSe561p*RS145pe - 1./
         162.*m33*m46*RN1e*RSe561p*RS156pe - 1./162.*m33*m46*RN1e*
         RSe451p*RS145pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * ( 1./162.*m33*m46*RN1e*
         RSe451p*RS156pe - 1./162.*m33*m46*RN13*RS145pe*RS345pe + 1./
         162.*m33*m46*RN13*RS145pe*RS356pe + 1./162.*m33*m46*RN13*
         RS156pe*RS345pe - 1./162.*m33*m46*RN13*RS156pe*RS356pe + 1./
         324.*m33*m44*RN1p*RS145pe*RSp563e + 1./324.*m33*m44*RN1p*
         RS145pe*RSp453e + 1./108.*m33*m44*RN1p*RS156pe*RSp563e + 1./
         324.*m33*m44*RN1p*RS156pe*RSp453e + 1./324.*m33*m44*RN1p*
         RSp561e*RS145pe + 1./108.*m33*m44*RN1p*RSp561e*RS156pe + 1./
         324.*m33*m44*RN1p*RSp451e*RS145pe + 1./324.*m33*m44*RN1p*
         RSp451e*RS156pe + 1./324.*m33*m44*RN1e*RS145pe*RSe563p + 1./
         324.*m33*m44*RN1e*RS145pe*RSe453p + 1./108.*m33*m44*RN1e*
         RS156pe*RSe563p + 1./324.*m33*m44*RN1e*RS156pe*RSe453p + 1./
         324.*m33*m44*RN1e*RSe561p*RS145pe + 1./108.*m33*m44*RN1e*
         RSe561p*RS156pe + 1./324.*m33*m44*RN1e*RSe451p*RS145pe + 1./
         324.*m33*m44*RN1e*RSe451p*RS156pe + 1./324.*m33*m44*RN13*
         RS145pe*RS345pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * ( 1./324.*m33*m44*RN13*
         RS145pe*RS356pe + 1./324.*m33*m44*RN13*RS156pe*RS345pe + 1./
         108.*m33*m44*RN13*RS156pe*RS356pe + 1./162.*m13*pow(RS145pe,2)
          - 1./81.*m13*RS156pe*RS145pe + 1./162.*m13*pow(RS156pe,2) + 1.
         /162.*m13*m66*RN1p*RS145pe*RSp563e + 1./54.*m13*m66*RN1p*
         RS145pe*RSp453e + 1./162.*m13*m66*RN1p*RS156pe*RSp563e + 1./
         162.*m13*m66*RN1p*RS156pe*RSp453e - 1./162.*m13*m66*RN1p*
         RSp561e*RS145pe - 1./162.*m13*m66*RN1p*RSp561e*RS156pe - 1./54.
         *m13*m66*RN1p*RSp451e*RS145pe - 1./162.*m13*m66*RN1p*RSp451e*
         RS156pe + 1./162.*m13*m66*RN1e*RS145pe*RSe563p + 1./54.*m13*
         m66*RN1e*RS145pe*RSe453p + 1./162.*m13*m66*RN1e*RS156pe*
         RSe563p + 1./162.*m13*m66*RN1e*RS156pe*RSe453p - 1./162.*m13*
         m66*RN1e*RSe561p*RS145pe - 1./162.*m13*m66*RN1e*RSe561p*
         RS156pe - 1./54.*m13*m66*RN1e*RSe451p*RS145pe - 1./162.*m13*
         m66*RN1e*RSe451p*RS156pe + 1./54.*m13*m66*RN13*RS145pe*RS345pe
          + 1./162.*m13*m66*RN13*RS145pe*RS356pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * ( 1./162.*m13*m66*RN13*
         RS156pe*RS345pe + 1./162.*m13*m66*RN13*RS156pe*RS356pe + 1./81.
         *m13*m46*RN1p*RS145pe*RSp563e - 1./81.*m13*m46*RN1p*RS145pe*
         RSp453e - 1./81.*m13*m46*RN1p*RS156pe*RSp563e + 1./81.*m13*m46
         *RN1p*RS156pe*RSp453e - 1./81.*m13*m46*RN1p*RSp561e*RS145pe + 
         1./81.*m13*m46*RN1p*RSp561e*RS156pe + 1./81.*m13*m46*RN1p*
         RSp451e*RS145pe - 1./81.*m13*m46*RN1p*RSp451e*RS156pe + 1./81.
         *m13*m46*RN1e*RS145pe*RSe563p - 1./81.*m13*m46*RN1e*RS145pe*
         RSe453p - 1./81.*m13*m46*RN1e*RS156pe*RSe563p + 1./81.*m13*m46
         *RN1e*RS156pe*RSe453p - 1./81.*m13*m46*RN1e*RSe561p*RS145pe + 
         1./81.*m13*m46*RN1e*RSe561p*RS156pe + 1./81.*m13*m46*RN1e*
         RSe451p*RS145pe - 1./81.*m13*m46*RN1e*RSe451p*RS156pe - 1./81.
         *m13*m46*RN13*RS145pe*RS345pe + 1./81.*m13*m46*RN13*RS145pe*
         RS356pe + 1./81.*m13*m46*RN13*RS156pe*RS345pe - 1./81.*m13*m46
         *RN13*RS156pe*RS356pe + 1./162.*m13*m44*RN1p*RS145pe*RSp563e
          + 1./162.*m13*m44*RN1p*RS145pe*RSp453e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * ( 1./54.*m13*m44*RN1p*
         RS156pe*RSp563e + 1./162.*m13*m44*RN1p*RS156pe*RSp453e - 1./
         162.*m13*m44*RN1p*RSp561e*RS145pe - 1./54.*m13*m44*RN1p*
         RSp561e*RS156pe - 1./162.*m13*m44*RN1p*RSp451e*RS145pe - 1./
         162.*m13*m44*RN1p*RSp451e*RS156pe + 1./162.*m13*m44*RN1e*
         RS145pe*RSe563p + 1./162.*m13*m44*RN1e*RS145pe*RSe453p + 1./54.
         *m13*m44*RN1e*RS156pe*RSe563p + 1./162.*m13*m44*RN1e*RS156pe*
         RSe453p - 1./162.*m13*m44*RN1e*RSe561p*RS145pe - 1./54.*m13*
         m44*RN1e*RSe561p*RS156pe - 1./162.*m13*m44*RN1e*RSe451p*
         RS145pe - 1./162.*m13*m44*RN1e*RSe451p*RS156pe + 1./162.*m13*
         m44*RN13*RS145pe*RS345pe + 1./162.*m13*m44*RN13*RS145pe*
         RS356pe + 1./162.*m13*m44*RN13*RS156pe*RS345pe + 1./54.*m13*
         m44*RN13*RS156pe*RS356pe - 1./81.*m11*RS145pe*RS345pe + 1./81.
         *m11*RS145pe*RS356pe + 1./81.*m11*RS145pe*RSe563p - 1./81.*m11
         *RS145pe*RSe453p + 1./81.*m11*RS145pe*RSp563e - 1./81.*m11*
         RS145pe*RSp453e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * ( 1./324.*m11*pow(
         RS145pe,2) + 1./81.*m11*RS156pe*RS345pe - 1./81.*m11*RS156pe*
         RS356pe - 1./81.*m11*RS156pe*RSe563p + 1./81.*m11*RS156pe*
         RSe453p - 1./81.*m11*RS156pe*RSp563e + 1./81.*m11*RS156pe*
         RSp453e - 1./162.*m11*RS156pe*RS145pe + 1./324.*m11*pow(
         RS156pe,2) - 1./81.*m11*RSe561p*RS145pe + 1./81.*m11*RSe561p*
         RS156pe + 1./81.*m11*RSe451p*RS145pe - 1./81.*m11*RSe451p*
         RS156pe - 1./81.*m11*RSp561e*RS145pe + 1./81.*m11*RSp561e*
         RS156pe + 1./81.*m11*RSp451e*RS145pe - 1./81.*m11*RSp451e*
         RS156pe + 1./81.*m11*me2*RN1e*RS145pe*RSe563p - 1./81.*m11*me2
         *RN1e*RS145pe*RSe453p - 1./81.*m11*me2*RN1e*RS156pe*RSe563p + 
         1./81.*m11*me2*RN1e*RS156pe*RSe453p - 1./81.*m11*me2*RN1e*
         RSe561p*RS145pe + 1./81.*m11*me2*RN1e*RSe561p*RS156pe + 1./81.
         *m11*me2*RN1e*RSe451p*RS145pe - 1./81.*m11*me2*RN1e*RSe451p*
         RS156pe + 1./81.*m11*mp2*RN1p*RS145pe*RSp563e - 1./81.*m11*mp2
         *RN1p*RS145pe*RSp453e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * (  - 1./81.*m11*mp2*
         RN1p*RS156pe*RSp563e + 1./81.*m11*mp2*RN1p*RS156pe*RSp453e - 1.
         /81.*m11*mp2*RN1p*RSp561e*RS145pe + 1./81.*m11*mp2*RN1p*
         RSp561e*RS156pe + 1./81.*m11*mp2*RN1p*RSp451e*RS145pe - 1./81.
         *m11*mp2*RN1p*RSp451e*RS156pe + 1./108.*m11*m66*RN1p*RS145pe*
         RSp563e + 1./36.*m11*m66*RN1p*RS145pe*RSp453e + 1./108.*m11*
         m66*RN1p*RS156pe*RSp563e + 1./108.*m11*m66*RN1p*RS156pe*
         RSp453e + 1./324.*m11*m66*RN1p*RSp561e*RS145pe + 1./324.*m11*
         m66*RN1p*RSp561e*RS156pe + 1./108.*m11*m66*RN1p*RSp451e*
         RS145pe + 1./324.*m11*m66*RN1p*RSp451e*RS156pe + 1./108.*m11*
         m66*RN1e*RS145pe*RSe563p + 1./36.*m11*m66*RN1e*RS145pe*RSe453p
          + 1./108.*m11*m66*RN1e*RS156pe*RSe563p + 1./108.*m11*m66*RN1e
         *RS156pe*RSe453p + 1./324.*m11*m66*RN1e*RSe561p*RS145pe + 1./
         324.*m11*m66*RN1e*RSe561p*RS156pe + 1./108.*m11*m66*RN1e*
         RSe451p*RS145pe + 1./324.*m11*m66*RN1e*RSe451p*RS156pe + 1./36.
         *m11*m66*RN13*RS145pe*RS345pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * ( 1./108.*m11*m66*RN13*
         RS145pe*RS356pe + 1./108.*m11*m66*RN13*RS156pe*RS345pe + 1./
         108.*m11*m66*RN13*RS156pe*RS356pe + 1./54.*m11*m46*RN1p*
         RS145pe*RSp563e - 1./54.*m11*m46*RN1p*RS145pe*RSp453e - 1./54.
         *m11*m46*RN1p*RS156pe*RSp563e + 1./54.*m11*m46*RN1p*RS156pe*
         RSp453e + 1./162.*m11*m46*RN1p*RSp561e*RS145pe - 1./162.*m11*
         m46*RN1p*RSp561e*RS156pe - 1./162.*m11*m46*RN1p*RSp451e*
         RS145pe + 1./162.*m11*m46*RN1p*RSp451e*RS156pe + 1./54.*m11*
         m46*RN1e*RS145pe*RSe563p - 1./54.*m11*m46*RN1e*RS145pe*RSe453p
          - 1./54.*m11*m46*RN1e*RS156pe*RSe563p + 1./54.*m11*m46*RN1e*
         RS156pe*RSe453p + 1./162.*m11*m46*RN1e*RSe561p*RS145pe - 1./
         162.*m11*m46*RN1e*RSe561p*RS156pe - 1./162.*m11*m46*RN1e*
         RSe451p*RS145pe + 1./162.*m11*m46*RN1e*RSe451p*RS156pe - 1./54.
         *m11*m46*RN13*RS145pe*RS345pe + 1./54.*m11*m46*RN13*RS145pe*
         RS356pe + 1./54.*m11*m46*RN13*RS156pe*RS345pe - 1./54.*m11*m46
         *RN13*RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * ( 1./108.*m11*m44*RN1p*
         RS145pe*RSp563e + 1./108.*m11*m44*RN1p*RS145pe*RSp453e + 1./36.
         *m11*m44*RN1p*RS156pe*RSp563e + 1./108.*m11*m44*RN1p*RS156pe*
         RSp453e + 1./324.*m11*m44*RN1p*RSp561e*RS145pe + 1./108.*m11*
         m44*RN1p*RSp561e*RS156pe + 1./324.*m11*m44*RN1p*RSp451e*
         RS145pe + 1./324.*m11*m44*RN1p*RSp451e*RS156pe + 1./108.*m11*
         m44*RN1e*RS145pe*RSe563p + 1./108.*m11*m44*RN1e*RS145pe*
         RSe453p + 1./36.*m11*m44*RN1e*RS156pe*RSe563p + 1./108.*m11*
         m44*RN1e*RS156pe*RSe453p + 1./324.*m11*m44*RN1e*RSe561p*
         RS145pe + 1./108.*m11*m44*RN1e*RSe561p*RS156pe + 1./324.*m11*
         m44*RN1e*RSe451p*RS145pe + 1./324.*m11*m44*RN1e*RSe451p*
         RS156pe + 1./108.*m11*m44*RN13*RS145pe*RS345pe + 1./108.*m11*
         m44*RN13*RS145pe*RS356pe + 1./108.*m11*m44*RN13*RS156pe*
         RS345pe + 1./36.*m11*m44*RN13*RS156pe*RS356pe + 1./162.*m11*
         m33*RN1p*RS145pe*RSp563e - 1./162.*m11*m33*RN1p*RS145pe*
         RSp453e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * (  - 1./162.*m11*m33*
         RN1p*RS156pe*RSp563e + 1./162.*m11*m33*RN1p*RS156pe*RSp453e + 
         1./162.*m11*m33*RN1p*RSp561e*RS145pe - 1./162.*m11*m33*RN1p*
         RSp561e*RS156pe - 1./162.*m11*m33*RN1p*RSp451e*RS145pe + 1./
         162.*m11*m33*RN1p*RSp451e*RS156pe + 1./162.*m11*m33*RN1e*
         RS145pe*RSe563p - 1./162.*m11*m33*RN1e*RS145pe*RSe453p - 1./
         162.*m11*m33*RN1e*RS156pe*RSe563p + 1./162.*m11*m33*RN1e*
         RS156pe*RSe453p + 1./162.*m11*m33*RN1e*RSe561p*RS145pe - 1./
         162.*m11*m33*RN1e*RSe561p*RS156pe - 1./162.*m11*m33*RN1e*
         RSe451p*RS145pe + 1./162.*m11*m33*RN1e*RSe451p*RS156pe - 1./54.
         *m11*m33*RN13*RS145pe*RS345pe + 1./54.*m11*m33*RN13*RS145pe*
         RS356pe + 1./54.*m11*m33*RN13*RS156pe*RS345pe - 1./54.*m11*m33
         *RN13*RS156pe*RS356pe + 1./81.*m11*m13*RN1p*RS145pe*RSp563e - 
         1./81.*m11*m13*RN1p*RS145pe*RSp453e - 1./81.*m11*m13*RN1p*
         RS156pe*RSp563e + 1./81.*m11*m13*RN1p*RS156pe*RSp453e - 1./81.
         *m11*m13*RN1p*RSp561e*RS145pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * ( 1./81.*m11*m13*RN1p*
         RSp561e*RS156pe + 1./81.*m11*m13*RN1p*RSp451e*RS145pe - 1./81.
         *m11*m13*RN1p*RSp451e*RS156pe + 1./81.*m11*m13*RN1e*RS145pe*
         RSe563p - 1./81.*m11*m13*RN1e*RS145pe*RSe453p - 1./81.*m11*m13
         *RN1e*RS156pe*RSe563p + 1./81.*m11*m13*RN1e*RS156pe*RSe453p - 
         1./81.*m11*m13*RN1e*RSe561p*RS145pe + 1./81.*m11*m13*RN1e*
         RSe561p*RS156pe + 1./81.*m11*m13*RN1e*RSe451p*RS145pe - 1./81.
         *m11*m13*RN1e*RSe451p*RS156pe - 1./81.*m11*m13*RN13*RS145pe*
         RS345pe + 1./81.*m11*m13*RN13*RS145pe*RS356pe + 1./81.*m11*m13
         *RN13*RS156pe*RS345pe - 1./81.*m11*m13*RN13*RS156pe*RS356pe + 
         1./162.*pow(m11,2)*RN1p*RS145pe*RSp563e - 1./162.*pow(m11,2)*
         RN1p*RS145pe*RSp453e - 1./162.*pow(m11,2)*RN1p*RS156pe*RSp563e
          + 1./162.*pow(m11,2)*RN1p*RS156pe*RSp453e + 1./54.*pow(m11,2)
         *RN1p*RSp561e*RS145pe - 1./54.*pow(m11,2)*RN1p*RSp561e*RS156pe
          - 1./54.*pow(m11,2)*RN1p*RSp451e*RS145pe + 1./54.*pow(m11,2)*
         RN1p*RSp451e*RS156pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * ( 1./162.*pow(m11,2)*
         RN1e*RS145pe*RSe563p - 1./162.*pow(m11,2)*RN1e*RS145pe*RSe453p
          - 1./162.*pow(m11,2)*RN1e*RS156pe*RSe563p + 1./162.*pow(
         m11,2)*RN1e*RS156pe*RSe453p + 1./54.*pow(m11,2)*RN1e*RSe561p*
         RS145pe - 1./54.*pow(m11,2)*RN1e*RSe561p*RS156pe - 1./54.*pow(
         m11,2)*RN1e*RSe451p*RS145pe + 1./54.*pow(m11,2)*RN1e*RSe451p*
         RS156pe - 1./162.*pow(m11,2)*RN13*RS145pe*RS345pe + 1./162.*
         pow(m11,2)*RN13*RS145pe*RS356pe + 1./162.*pow(m11,2)*RN13*
         RS156pe*RS345pe - 1./162.*pow(m11,2)*RN13*RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m55,L) * (  - 1./54.*m55*pow(
         RS146pe,2) + 1./162.*m55*RS546pe*pow(RS146pe,2) + 1./54.*m33*
         m55*RN1p*RS146pe*RSp463e - 1./162.*m33*m55*RN1p*RS546pe*
         RS146pe*RSp463e + 1./54.*m33*m55*RN1p*RSp461e*RS146pe - 1./162.
         *m33*m55*RN1p*RSp461e*RS546pe*RS146pe + 1./54.*m33*m55*RN1e*
         RS146pe*RSe463p - 1./162.*m33*m55*RN1e*RS546pe*RS146pe*RSe463p
          + 1./54.*m33*m55*RN1e*RSe461p*RS146pe - 1./162.*m33*m55*RN1e*
         RSe461p*RS546pe*RS146pe + 1./54.*m33*m55*RN13*RS146pe*RS346pe
          - 1./162.*m33*m55*RN13*RS546pe*RS146pe*RS346pe + 1./27.*m13*
         m55*RN1p*RS146pe*RSp463e - 1./81.*m13*m55*RN1p*RS546pe*RS146pe
         *RSp463e - 1./27.*m13*m55*RN1p*RSp461e*RS146pe + 1./81.*m13*
         m55*RN1p*RSp461e*RS546pe*RS146pe + 1./27.*m13*m55*RN1e*RS146pe
         *RSe463p - 1./81.*m13*m55*RN1e*RS546pe*RS146pe*RSe463p - 1./27.
         *m13*m55*RN1e*RSe461p*RS146pe + 1./81.*m13*m55*RN1e*RSe461p*
         RS546pe*RS146pe + 1./27.*m13*m55*RN13*RS146pe*RS346pe - 1./81.
         *m13*m55*RN13*RS546pe*RS146pe*RS346pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m55,L) * ( 1./18.*m11*m55*RN1p*
         RS146pe*RSp463e - 1./54.*m11*m55*RN1p*RS546pe*RS146pe*RSp463e
          + 1./54.*m11*m55*RN1p*RSp461e*RS146pe - 1./162.*m11*m55*RN1p*
         RSp461e*RS546pe*RS146pe + 1./18.*m11*m55*RN1e*RS146pe*RSe463p
          - 1./54.*m11*m55*RN1e*RS546pe*RS146pe*RSe463p + 1./54.*m11*
         m55*RN1e*RSe461p*RS146pe - 1./162.*m11*m55*RN1e*RSe461p*
         RS546pe*RS146pe + 1./18.*m11*m55*RN13*RS146pe*RS346pe - 1./54.
         *m11*m55*RN13*RS546pe*RS146pe*RS346pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * (  - 1./108.*m66*pow(
         RS145pe,2) - 1./162.*m66*RS146pe*RS145pe - 1./324.*m66*pow(
         RS146pe,2) + 1./162.*m56*pow(RS145pe,2) - 1./81.*m56*RS146pe*
         RS145pe + 1./162.*m56*pow(RS146pe,2) - 1./324.*m55*pow(
         RS145pe,2) - 1./162.*m55*RS146pe*RS145pe - 1./108.*m55*pow(
         RS146pe,2) - 1./324.*m33*pow(RS145pe,2) + 1./162.*m33*RS146pe*
         RS145pe - 1./324.*m33*pow(RS146pe,2) + 1./324.*m33*m66*RN1p*
         RS145pe*RSp463e + 1./108.*m33*m66*RN1p*RS145pe*RSp453e + 1./
         324.*m33*m66*RN1p*RS146pe*RSp463e + 1./324.*m33*m66*RN1p*
         RS146pe*RSp453e + 1./324.*m33*m66*RN1p*RSp461e*RS145pe + 1./
         324.*m33*m66*RN1p*RSp461e*RS146pe + 1./108.*m33*m66*RN1p*
         RSp451e*RS145pe + 1./324.*m33*m66*RN1p*RSp451e*RS146pe + 1./
         324.*m33*m66*RN1e*RS145pe*RSe463p + 1./108.*m33*m66*RN1e*
         RS145pe*RSe453p + 1./324.*m33*m66*RN1e*RS146pe*RSe463p + 1./
         324.*m33*m66*RN1e*RS146pe*RSe453p + 1./324.*m33*m66*RN1e*
         RSe461p*RS145pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * ( 1./324.*m33*m66*RN1e*
         RSe461p*RS146pe + 1./108.*m33*m66*RN1e*RSe451p*RS145pe + 1./
         324.*m33*m66*RN1e*RSe451p*RS146pe + 1./108.*m33*m66*RN13*
         RS145pe*RS345pe + 1./324.*m33*m66*RN13*RS145pe*RS346pe + 1./
         324.*m33*m66*RN13*RS146pe*RS345pe + 1./324.*m33*m66*RN13*
         RS146pe*RS346pe + 1./162.*m33*m56*RN1p*RS145pe*RSp463e - 1./
         162.*m33*m56*RN1p*RS145pe*RSp453e - 1./162.*m33*m56*RN1p*
         RS146pe*RSp463e + 1./162.*m33*m56*RN1p*RS146pe*RSp453e + 1./
         162.*m33*m56*RN1p*RSp461e*RS145pe - 1./162.*m33*m56*RN1p*
         RSp461e*RS146pe - 1./162.*m33*m56*RN1p*RSp451e*RS145pe + 1./
         162.*m33*m56*RN1p*RSp451e*RS146pe + 1./162.*m33*m56*RN1e*
         RS145pe*RSe463p - 1./162.*m33*m56*RN1e*RS145pe*RSe453p - 1./
         162.*m33*m56*RN1e*RS146pe*RSe463p + 1./162.*m33*m56*RN1e*
         RS146pe*RSe453p + 1./162.*m33*m56*RN1e*RSe461p*RS145pe - 1./
         162.*m33*m56*RN1e*RSe461p*RS146pe - 1./162.*m33*m56*RN1e*
         RSe451p*RS145pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * ( 1./162.*m33*m56*RN1e*
         RSe451p*RS146pe - 1./162.*m33*m56*RN13*RS145pe*RS345pe + 1./
         162.*m33*m56*RN13*RS145pe*RS346pe + 1./162.*m33*m56*RN13*
         RS146pe*RS345pe - 1./162.*m33*m56*RN13*RS146pe*RS346pe + 1./
         324.*m33*m55*RN1p*RS145pe*RSp463e + 1./324.*m33*m55*RN1p*
         RS145pe*RSp453e + 1./108.*m33*m55*RN1p*RS146pe*RSp463e + 1./
         324.*m33*m55*RN1p*RS146pe*RSp453e + 1./324.*m33*m55*RN1p*
         RSp461e*RS145pe + 1./108.*m33*m55*RN1p*RSp461e*RS146pe + 1./
         324.*m33*m55*RN1p*RSp451e*RS145pe + 1./324.*m33*m55*RN1p*
         RSp451e*RS146pe + 1./324.*m33*m55*RN1e*RS145pe*RSe463p + 1./
         324.*m33*m55*RN1e*RS145pe*RSe453p + 1./108.*m33*m55*RN1e*
         RS146pe*RSe463p + 1./324.*m33*m55*RN1e*RS146pe*RSe453p + 1./
         324.*m33*m55*RN1e*RSe461p*RS145pe + 1./108.*m33*m55*RN1e*
         RSe461p*RS146pe + 1./324.*m33*m55*RN1e*RSe451p*RS145pe + 1./
         324.*m33*m55*RN1e*RSe451p*RS146pe + 1./324.*m33*m55*RN13*
         RS145pe*RS345pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * ( 1./324.*m33*m55*RN13*
         RS145pe*RS346pe + 1./324.*m33*m55*RN13*RS146pe*RS345pe + 1./
         108.*m33*m55*RN13*RS146pe*RS346pe + 1./162.*m13*pow(RS145pe,2)
          - 1./81.*m13*RS146pe*RS145pe + 1./162.*m13*pow(RS146pe,2) + 1.
         /162.*m13*m66*RN1p*RS145pe*RSp463e + 1./54.*m13*m66*RN1p*
         RS145pe*RSp453e + 1./162.*m13*m66*RN1p*RS146pe*RSp463e + 1./
         162.*m13*m66*RN1p*RS146pe*RSp453e - 1./162.*m13*m66*RN1p*
         RSp461e*RS145pe - 1./162.*m13*m66*RN1p*RSp461e*RS146pe - 1./54.
         *m13*m66*RN1p*RSp451e*RS145pe - 1./162.*m13*m66*RN1p*RSp451e*
         RS146pe + 1./162.*m13*m66*RN1e*RS145pe*RSe463p + 1./54.*m13*
         m66*RN1e*RS145pe*RSe453p + 1./162.*m13*m66*RN1e*RS146pe*
         RSe463p + 1./162.*m13*m66*RN1e*RS146pe*RSe453p - 1./162.*m13*
         m66*RN1e*RSe461p*RS145pe - 1./162.*m13*m66*RN1e*RSe461p*
         RS146pe - 1./54.*m13*m66*RN1e*RSe451p*RS145pe - 1./162.*m13*
         m66*RN1e*RSe451p*RS146pe + 1./54.*m13*m66*RN13*RS145pe*RS345pe
          + 1./162.*m13*m66*RN13*RS145pe*RS346pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * ( 1./162.*m13*m66*RN13*
         RS146pe*RS345pe + 1./162.*m13*m66*RN13*RS146pe*RS346pe + 1./81.
         *m13*m56*RN1p*RS145pe*RSp463e - 1./81.*m13*m56*RN1p*RS145pe*
         RSp453e - 1./81.*m13*m56*RN1p*RS146pe*RSp463e + 1./81.*m13*m56
         *RN1p*RS146pe*RSp453e - 1./81.*m13*m56*RN1p*RSp461e*RS145pe + 
         1./81.*m13*m56*RN1p*RSp461e*RS146pe + 1./81.*m13*m56*RN1p*
         RSp451e*RS145pe - 1./81.*m13*m56*RN1p*RSp451e*RS146pe + 1./81.
         *m13*m56*RN1e*RS145pe*RSe463p - 1./81.*m13*m56*RN1e*RS145pe*
         RSe453p - 1./81.*m13*m56*RN1e*RS146pe*RSe463p + 1./81.*m13*m56
         *RN1e*RS146pe*RSe453p - 1./81.*m13*m56*RN1e*RSe461p*RS145pe + 
         1./81.*m13*m56*RN1e*RSe461p*RS146pe + 1./81.*m13*m56*RN1e*
         RSe451p*RS145pe - 1./81.*m13*m56*RN1e*RSe451p*RS146pe - 1./81.
         *m13*m56*RN13*RS145pe*RS345pe + 1./81.*m13*m56*RN13*RS145pe*
         RS346pe + 1./81.*m13*m56*RN13*RS146pe*RS345pe - 1./81.*m13*m56
         *RN13*RS146pe*RS346pe + 1./162.*m13*m55*RN1p*RS145pe*RSp463e
          + 1./162.*m13*m55*RN1p*RS145pe*RSp453e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * ( 1./54.*m13*m55*RN1p*
         RS146pe*RSp463e + 1./162.*m13*m55*RN1p*RS146pe*RSp453e - 1./
         162.*m13*m55*RN1p*RSp461e*RS145pe - 1./54.*m13*m55*RN1p*
         RSp461e*RS146pe - 1./162.*m13*m55*RN1p*RSp451e*RS145pe - 1./
         162.*m13*m55*RN1p*RSp451e*RS146pe + 1./162.*m13*m55*RN1e*
         RS145pe*RSe463p + 1./162.*m13*m55*RN1e*RS145pe*RSe453p + 1./54.
         *m13*m55*RN1e*RS146pe*RSe463p + 1./162.*m13*m55*RN1e*RS146pe*
         RSe453p - 1./162.*m13*m55*RN1e*RSe461p*RS145pe - 1./54.*m13*
         m55*RN1e*RSe461p*RS146pe - 1./162.*m13*m55*RN1e*RSe451p*
         RS145pe - 1./162.*m13*m55*RN1e*RSe451p*RS146pe + 1./162.*m13*
         m55*RN13*RS145pe*RS345pe + 1./162.*m13*m55*RN13*RS145pe*
         RS346pe + 1./162.*m13*m55*RN13*RS146pe*RS345pe + 1./54.*m13*
         m55*RN13*RS146pe*RS346pe - 1./81.*m11*RS145pe*RS345pe + 1./81.
         *m11*RS145pe*RS346pe + 1./81.*m11*RS145pe*RSe463p - 1./81.*m11
         *RS145pe*RSe453p + 1./81.*m11*RS145pe*RSp463e - 1./81.*m11*
         RS145pe*RSp453e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * ( 1./324.*m11*pow(
         RS145pe,2) + 1./81.*m11*RS146pe*RS345pe - 1./81.*m11*RS146pe*
         RS346pe - 1./81.*m11*RS146pe*RSe463p + 1./81.*m11*RS146pe*
         RSe453p - 1./81.*m11*RS146pe*RSp463e + 1./81.*m11*RS146pe*
         RSp453e - 1./162.*m11*RS146pe*RS145pe + 1./324.*m11*pow(
         RS146pe,2) - 1./81.*m11*RSe461p*RS145pe + 1./81.*m11*RSe461p*
         RS146pe + 1./81.*m11*RSe451p*RS145pe - 1./81.*m11*RSe451p*
         RS146pe - 1./81.*m11*RSp461e*RS145pe + 1./81.*m11*RSp461e*
         RS146pe + 1./81.*m11*RSp451e*RS145pe - 1./81.*m11*RSp451e*
         RS146pe + 1./81.*m11*me2*RN1e*RS145pe*RSe463p - 1./81.*m11*me2
         *RN1e*RS145pe*RSe453p - 1./81.*m11*me2*RN1e*RS146pe*RSe463p + 
         1./81.*m11*me2*RN1e*RS146pe*RSe453p - 1./81.*m11*me2*RN1e*
         RSe461p*RS145pe + 1./81.*m11*me2*RN1e*RSe461p*RS146pe + 1./81.
         *m11*me2*RN1e*RSe451p*RS145pe - 1./81.*m11*me2*RN1e*RSe451p*
         RS146pe + 1./81.*m11*mp2*RN1p*RS145pe*RSp463e - 1./81.*m11*mp2
         *RN1p*RS145pe*RSp453e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * (  - 1./81.*m11*mp2*
         RN1p*RS146pe*RSp463e + 1./81.*m11*mp2*RN1p*RS146pe*RSp453e - 1.
         /81.*m11*mp2*RN1p*RSp461e*RS145pe + 1./81.*m11*mp2*RN1p*
         RSp461e*RS146pe + 1./81.*m11*mp2*RN1p*RSp451e*RS145pe - 1./81.
         *m11*mp2*RN1p*RSp451e*RS146pe + 1./108.*m11*m66*RN1p*RS145pe*
         RSp463e + 1./36.*m11*m66*RN1p*RS145pe*RSp453e + 1./108.*m11*
         m66*RN1p*RS146pe*RSp463e + 1./108.*m11*m66*RN1p*RS146pe*
         RSp453e + 1./324.*m11*m66*RN1p*RSp461e*RS145pe + 1./324.*m11*
         m66*RN1p*RSp461e*RS146pe + 1./108.*m11*m66*RN1p*RSp451e*
         RS145pe + 1./324.*m11*m66*RN1p*RSp451e*RS146pe + 1./108.*m11*
         m66*RN1e*RS145pe*RSe463p + 1./36.*m11*m66*RN1e*RS145pe*RSe453p
          + 1./108.*m11*m66*RN1e*RS146pe*RSe463p + 1./108.*m11*m66*RN1e
         *RS146pe*RSe453p + 1./324.*m11*m66*RN1e*RSe461p*RS145pe + 1./
         324.*m11*m66*RN1e*RSe461p*RS146pe + 1./108.*m11*m66*RN1e*
         RSe451p*RS145pe + 1./324.*m11*m66*RN1e*RSe451p*RS146pe + 1./36.
         *m11*m66*RN13*RS145pe*RS345pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * ( 1./108.*m11*m66*RN13*
         RS145pe*RS346pe + 1./108.*m11*m66*RN13*RS146pe*RS345pe + 1./
         108.*m11*m66*RN13*RS146pe*RS346pe + 1./54.*m11*m56*RN1p*
         RS145pe*RSp463e - 1./54.*m11*m56*RN1p*RS145pe*RSp453e - 1./54.
         *m11*m56*RN1p*RS146pe*RSp463e + 1./54.*m11*m56*RN1p*RS146pe*
         RSp453e + 1./162.*m11*m56*RN1p*RSp461e*RS145pe - 1./162.*m11*
         m56*RN1p*RSp461e*RS146pe - 1./162.*m11*m56*RN1p*RSp451e*
         RS145pe + 1./162.*m11*m56*RN1p*RSp451e*RS146pe + 1./54.*m11*
         m56*RN1e*RS145pe*RSe463p - 1./54.*m11*m56*RN1e*RS145pe*RSe453p
          - 1./54.*m11*m56*RN1e*RS146pe*RSe463p + 1./54.*m11*m56*RN1e*
         RS146pe*RSe453p + 1./162.*m11*m56*RN1e*RSe461p*RS145pe - 1./
         162.*m11*m56*RN1e*RSe461p*RS146pe - 1./162.*m11*m56*RN1e*
         RSe451p*RS145pe + 1./162.*m11*m56*RN1e*RSe451p*RS146pe - 1./54.
         *m11*m56*RN13*RS145pe*RS345pe + 1./54.*m11*m56*RN13*RS145pe*
         RS346pe + 1./54.*m11*m56*RN13*RS146pe*RS345pe - 1./54.*m11*m56
         *RN13*RS146pe*RS346pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * ( 1./108.*m11*m55*RN1p*
         RS145pe*RSp463e + 1./108.*m11*m55*RN1p*RS145pe*RSp453e + 1./36.
         *m11*m55*RN1p*RS146pe*RSp463e + 1./108.*m11*m55*RN1p*RS146pe*
         RSp453e + 1./324.*m11*m55*RN1p*RSp461e*RS145pe + 1./108.*m11*
         m55*RN1p*RSp461e*RS146pe + 1./324.*m11*m55*RN1p*RSp451e*
         RS145pe + 1./324.*m11*m55*RN1p*RSp451e*RS146pe + 1./108.*m11*
         m55*RN1e*RS145pe*RSe463p + 1./108.*m11*m55*RN1e*RS145pe*
         RSe453p + 1./36.*m11*m55*RN1e*RS146pe*RSe463p + 1./108.*m11*
         m55*RN1e*RS146pe*RSe453p + 1./324.*m11*m55*RN1e*RSe461p*
         RS145pe + 1./108.*m11*m55*RN1e*RSe461p*RS146pe + 1./324.*m11*
         m55*RN1e*RSe451p*RS145pe + 1./324.*m11*m55*RN1e*RSe451p*
         RS146pe + 1./108.*m11*m55*RN13*RS145pe*RS345pe + 1./108.*m11*
         m55*RN13*RS145pe*RS346pe + 1./108.*m11*m55*RN13*RS146pe*
         RS345pe + 1./36.*m11*m55*RN13*RS146pe*RS346pe + 1./162.*m11*
         m33*RN1p*RS145pe*RSp463e - 1./162.*m11*m33*RN1p*RS145pe*
         RSp453e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * (  - 1./162.*m11*m33*
         RN1p*RS146pe*RSp463e + 1./162.*m11*m33*RN1p*RS146pe*RSp453e + 
         1./162.*m11*m33*RN1p*RSp461e*RS145pe - 1./162.*m11*m33*RN1p*
         RSp461e*RS146pe - 1./162.*m11*m33*RN1p*RSp451e*RS145pe + 1./
         162.*m11*m33*RN1p*RSp451e*RS146pe + 1./162.*m11*m33*RN1e*
         RS145pe*RSe463p - 1./162.*m11*m33*RN1e*RS145pe*RSe453p - 1./
         162.*m11*m33*RN1e*RS146pe*RSe463p + 1./162.*m11*m33*RN1e*
         RS146pe*RSe453p + 1./162.*m11*m33*RN1e*RSe461p*RS145pe - 1./
         162.*m11*m33*RN1e*RSe461p*RS146pe - 1./162.*m11*m33*RN1e*
         RSe451p*RS145pe + 1./162.*m11*m33*RN1e*RSe451p*RS146pe - 1./54.
         *m11*m33*RN13*RS145pe*RS345pe + 1./54.*m11*m33*RN13*RS145pe*
         RS346pe + 1./54.*m11*m33*RN13*RS146pe*RS345pe - 1./54.*m11*m33
         *RN13*RS146pe*RS346pe + 1./81.*m11*m13*RN1p*RS145pe*RSp463e - 
         1./81.*m11*m13*RN1p*RS145pe*RSp453e - 1./81.*m11*m13*RN1p*
         RS146pe*RSp463e + 1./81.*m11*m13*RN1p*RS146pe*RSp453e - 1./81.
         *m11*m13*RN1p*RSp461e*RS145pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * ( 1./81.*m11*m13*RN1p*
         RSp461e*RS146pe + 1./81.*m11*m13*RN1p*RSp451e*RS145pe - 1./81.
         *m11*m13*RN1p*RSp451e*RS146pe + 1./81.*m11*m13*RN1e*RS145pe*
         RSe463p - 1./81.*m11*m13*RN1e*RS145pe*RSe453p - 1./81.*m11*m13
         *RN1e*RS146pe*RSe463p + 1./81.*m11*m13*RN1e*RS146pe*RSe453p - 
         1./81.*m11*m13*RN1e*RSe461p*RS145pe + 1./81.*m11*m13*RN1e*
         RSe461p*RS146pe + 1./81.*m11*m13*RN1e*RSe451p*RS145pe - 1./81.
         *m11*m13*RN1e*RSe451p*RS146pe - 1./81.*m11*m13*RN13*RS145pe*
         RS345pe + 1./81.*m11*m13*RN13*RS145pe*RS346pe + 1./81.*m11*m13
         *RN13*RS146pe*RS345pe - 1./81.*m11*m13*RN13*RS146pe*RS346pe + 
         1./162.*pow(m11,2)*RN1p*RS145pe*RSp463e - 1./162.*pow(m11,2)*
         RN1p*RS145pe*RSp453e - 1./162.*pow(m11,2)*RN1p*RS146pe*RSp463e
          + 1./162.*pow(m11,2)*RN1p*RS146pe*RSp453e + 1./54.*pow(m11,2)
         *RN1p*RSp461e*RS145pe - 1./54.*pow(m11,2)*RN1p*RSp461e*RS146pe
          - 1./54.*pow(m11,2)*RN1p*RSp451e*RS145pe + 1./54.*pow(m11,2)*
         RN1p*RSp451e*RS146pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * ( 1./162.*pow(m11,2)*
         RN1e*RS145pe*RSe463p - 1./162.*pow(m11,2)*RN1e*RS145pe*RSe453p
          - 1./162.*pow(m11,2)*RN1e*RS146pe*RSe463p + 1./162.*pow(
         m11,2)*RN1e*RS146pe*RSe453p + 1./54.*pow(m11,2)*RN1e*RSe461p*
         RS145pe - 1./54.*pow(m11,2)*RN1e*RSe461p*RS146pe - 1./54.*pow(
         m11,2)*RN1e*RSe451p*RS145pe + 1./54.*pow(m11,2)*RN1e*RSe451p*
         RS146pe - 1./162.*pow(m11,2)*RN13*RS145pe*RS345pe + 1./162.*
         pow(m11,2)*RN13*RS145pe*RS346pe + 1./162.*pow(m11,2)*RN13*
         RS146pe*RS345pe - 1./162.*pow(m11,2)*RN13*RS146pe*RS346pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m66,L) * (  - 1./54.*m66*pow(
         RS145pe,2) + 1./162.*m66*RS645pe*pow(RS145pe,2) + 1./54.*m33*
         m66*RN1p*RS145pe*RSp453e - 1./162.*m33*m66*RN1p*RS645pe*
         RS145pe*RSp453e + 1./54.*m33*m66*RN1p*RSp451e*RS145pe - 1./162.
         *m33*m66*RN1p*RSp451e*RS645pe*RS145pe + 1./54.*m33*m66*RN1e*
         RS145pe*RSe453p - 1./162.*m33*m66*RN1e*RS645pe*RS145pe*RSe453p
          + 1./54.*m33*m66*RN1e*RSe451p*RS145pe - 1./162.*m33*m66*RN1e*
         RSe451p*RS645pe*RS145pe + 1./54.*m33*m66*RN13*RS145pe*RS345pe
          - 1./162.*m33*m66*RN13*RS645pe*RS145pe*RS345pe + 1./27.*m13*
         m66*RN1p*RS145pe*RSp453e - 1./81.*m13*m66*RN1p*RS645pe*RS145pe
         *RSp453e - 1./27.*m13*m66*RN1p*RSp451e*RS145pe + 1./81.*m13*
         m66*RN1p*RSp451e*RS645pe*RS145pe + 1./27.*m13*m66*RN1e*RS145pe
         *RSe453p - 1./81.*m13*m66*RN1e*RS645pe*RS145pe*RSe453p - 1./27.
         *m13*m66*RN1e*RSe451p*RS145pe + 1./81.*m13*m66*RN1e*RSe451p*
         RS645pe*RS145pe + 1./27.*m13*m66*RN13*RS145pe*RS345pe - 1./81.
         *m13*m66*RN13*RS645pe*RS145pe*RS345pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(m66,L) * ( 1./18.*m11*m66*RN1p*
         RS145pe*RSp453e - 1./54.*m11*m66*RN1p*RS645pe*RS145pe*RSp453e
          + 1./54.*m11*m66*RN1p*RSp451e*RS145pe - 1./162.*m11*m66*RN1p*
         RSp451e*RS645pe*RS145pe + 1./18.*m11*m66*RN1e*RS145pe*RSe453p
          - 1./54.*m11*m66*RN1e*RS645pe*RS145pe*RSe453p + 1./54.*m11*
         m66*RN1e*RSe451p*RS145pe - 1./162.*m11*m66*RN1e*RSe451p*
         RS645pe*RS145pe + 1./18.*m11*m66*RN13*RS145pe*RS345pe - 1./54.
         *m11*m66*RN13*RS645pe*RS145pe*RS345pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(mp2,L) * (  - 1./270.*RD1456ep*
         RSp45613e + 1./540.*RD1456ep*RSp45633e + 1./540.*RSp45611e*
         RD1456ep + 1./135.*mp2*RSp45613e*RS14563pe - 1./270.*mp2*
         RSp45633e*RS14563pe - 1./270.*mp2*RC1456pe*RSp45613e + 1./540.
         *mp2*RC1456pe*RSp45633e - 1./270.*mp2*RSp45611e*RS14563pe + 1./
         540.*mp2*RSp45611e*RC1456pe + 1./162.*m66*RSp456e*pow(
         RS145pe,2) + 1./162.*m55*RSp465e*pow(RS146pe,2) + 1./162.*m44*
         RSp564e*pow(RS156pe,2) + 2./45.*m33*RSp45613e*RS14563pe - 1./
         90.*m33*RSp45633e*RS14563pe - 1./405.*m33*RC1456pe*RSp45613e
          - 11./810.*m33*RC1456pe*RSp45633e + 1./270.*m33*RSp45611e*
         RS14563pe + 13./810.*m33*RSp45611e*RC1456pe - 1./162.*m33*m66*
         RN1p*RSp456e*RS145pe*RSp453e - 1./162.*m33*m66*RN1p*RSp456e*
         RSp451e*RS145pe - 1./162.*m33*m66*RN1e*RSp456e*RS145pe*RSe453p
          - 1./162.*m33*m66*RN1e*RSp456e*RSe451p*RS145pe - 1./162.*m33*
         m66*RN13*RSp456e*RS145pe*RS345pe - 1./162.*m33*m55*RN1p*
         RSp465e*RS146pe*RSp463e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(mp2,L) * (  - 1./162.*m33*m55*
         RN1p*RSp465e*RSp461e*RS146pe - 1./162.*m33*m55*RN1e*RSp465e*
         RS146pe*RSe463p - 1./162.*m33*m55*RN1e*RSp465e*RSe461p*RS146pe
          - 1./162.*m33*m55*RN13*RSp465e*RS146pe*RS346pe - 1./162.*m33*
         m44*RN1p*RSp564e*RS156pe*RSp563e - 1./162.*m33*m44*RN1p*
         RSp564e*RSp561e*RS156pe - 1./162.*m33*m44*RN1e*RSp564e*RS156pe
         *RSe563p - 1./162.*m33*m44*RN1e*RSp564e*RSe561p*RS156pe - 1./
         162.*m33*m44*RN13*RSp564e*RS156pe*RS356pe + 1./54.*pow(m33,2)*
         RN13*RSp45633e*RS14563pe - 7./405.*m13*RSp45613e*RS14563pe + 7.
         /810.*m13*RSp45633e*RS14563pe + 7./810.*m13*RC1456pe*RSp45613e
          - 7./1620.*m13*RC1456pe*RSp45633e + 7./810.*m13*RSp45611e*
         RS14563pe - 7./1620.*m13*RSp45611e*RC1456pe - 1./81.*m13*m66*
         RN1p*RSp456e*RS145pe*RSp453e + 1./81.*m13*m66*RN1p*RSp456e*
         RSp451e*RS145pe - 1./81.*m13*m66*RN1e*RSp456e*RS145pe*RSe453p
          + 1./81.*m13*m66*RN1e*RSp456e*RSe451p*RS145pe - 1./81.*m13*
         m66*RN13*RSp456e*RS145pe*RS345pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(mp2,L) * (  - 1./81.*m13*m55*
         RN1p*RSp465e*RS146pe*RSp463e + 1./81.*m13*m55*RN1p*RSp465e*
         RSp461e*RS146pe - 1./81.*m13*m55*RN1e*RSp465e*RS146pe*RSe463p
          + 1./81.*m13*m55*RN1e*RSp465e*RSe461p*RS146pe - 1./81.*m13*
         m55*RN13*RSp465e*RS146pe*RS346pe - 1./81.*m13*m44*RN1p*RSp564e
         *RS156pe*RSp563e + 1./81.*m13*m44*RN1p*RSp564e*RSp561e*RS156pe
          - 1./81.*m13*m44*RN1e*RSp564e*RS156pe*RSe563p + 1./81.*m13*
         m44*RN1e*RSp564e*RSe561p*RS156pe - 1./81.*m13*m44*RN13*RSp564e
         *RS156pe*RS356pe + 1./27.*m13*m33*RN13*RSp45633e*RS14563pe + 
         16./405.*m11*RSp45613e*RS14563pe + 1./162.*m11*RSp45633e*
         RS14563pe + 13./810.*m11*RC1456pe*RSp45613e - 19./1620.*m11*
         RC1456pe*RSp45633e + 23./810.*m11*RSp45611e*RS14563pe - 37./
         1620.*m11*RSp45611e*RC1456pe - 1./54.*m11*m66*RN1p*RSp456e*
         RS145pe*RSp453e - 1./162.*m11*m66*RN1p*RSp456e*RSp451e*RS145pe
          - 1./54.*m11*m66*RN1e*RSp456e*RS145pe*RSe453p - 1./162.*m11*
         m66*RN1e*RSp456e*RSe451p*RS145pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(mp2,L) * (  - 1./54.*m11*m66*
         RN13*RSp456e*RS145pe*RS345pe - 1./54.*m11*m55*RN1p*RSp465e*
         RS146pe*RSp463e - 1./162.*m11*m55*RN1p*RSp465e*RSp461e*RS146pe
          - 1./54.*m11*m55*RN1e*RSp465e*RS146pe*RSe463p - 1./162.*m11*
         m55*RN1e*RSp465e*RSe461p*RS146pe - 1./54.*m11*m55*RN13*RSp465e
         *RS146pe*RS346pe - 1./54.*m11*m44*RN1p*RSp564e*RS156pe*RSp563e
          - 1./162.*m11*m44*RN1p*RSp564e*RSp561e*RS156pe - 1./54.*m11*
         m44*RN1e*RSp564e*RS156pe*RSe563p - 1./162.*m11*m44*RN1e*
         RSp564e*RSe561p*RS156pe - 1./54.*m11*m44*RN13*RSp564e*RS156pe*
         RS356pe + 1./54.*m11*m33*RN1p*RSp45611e*RSp45613e + 1./54.*m11
         *m33*RN1p*pow(RSp45611e,2) + 1./54.*m11*m33*RN1e*RSp45611e*
         RSe45613p + 1./54.*m11*m33*RN1e*RSp45611e*RSe45611p + 1./18.*
         m11*m33*RN13*RSp45633e*RS14563pe + 1./54.*m11*m33*RN13*
         RSp45611e*RS34561pe + 1./27.*m11*m13*RN1p*RSp45611e*RSp45613e
          - 1./27.*m11*m13*RN1p*pow(RSp45611e,2) + 1./27.*m11*m13*RN1e*
         RSp45611e*RSe45613p );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(mp2,L) * (  - 1./27.*m11*m13*
         RN1e*RSp45611e*RSe45611p + 1./27.*m11*m13*RN13*RSp45611e*
         RS34561pe + 1./18.*pow(m11,2)*RN1p*RSp45611e*RSp45613e + 1./54.
         *pow(m11,2)*RN1p*pow(RSp45611e,2) + 1./18.*pow(m11,2)*RN1e*
         RSp45611e*RSe45613p + 1./54.*pow(m11,2)*RN1e*RSp45611e*
         RSe45611p + 1./18.*pow(m11,2)*RN13*RSp45611e*RS34561pe + 1./81.
         *RS(m11,m44,m55,m66,m33,me2,mp2)*m33*RSp45613e - 1./162.*RS(
         m11,m44,m55,m66,m33,me2,mp2)*m33*RSp45633e - 1./162.*RS(m11,
         m44,m55,m66,m33,me2,mp2)*m33*RSp45611e + 2./81.*RS(m11,m44,m55
         ,m66,m33,me2,mp2)*m11*RSp45613e - 1./81.*RS(m11,m44,m55,m66,
         m33,me2,mp2)*m11*RSp45633e - 1./81.*RS(m11,m44,m55,m66,m33,me2
         ,mp2)*m11*RSp45611e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(me2,L) * (  - 1./270.*RD1456ep*
         RSe45613p + 1./540.*RD1456ep*RSe45633p + 1./540.*RSe45611p*
         RD1456ep + 1./135.*me2*RSe45613p*RS14563pe - 1./270.*me2*
         RSe45633p*RS14563pe - 1./270.*me2*RC1456pe*RSe45613p + 1./540.
         *me2*RC1456pe*RSe45633p - 1./270.*me2*RSe45611p*RS14563pe + 1./
         540.*me2*RSe45611p*RC1456pe + 1./162.*m66*RSe456p*pow(
         RS145pe,2) + 1./162.*m55*RSe465p*pow(RS146pe,2) + 1./162.*m44*
         RSe564p*pow(RS156pe,2) + 2./45.*m33*RSe45613p*RS14563pe - 1./
         90.*m33*RSe45633p*RS14563pe - 1./405.*m33*RC1456pe*RSe45613p
          - 11./810.*m33*RC1456pe*RSe45633p + 1./270.*m33*RSe45611p*
         RS14563pe + 13./810.*m33*RSe45611p*RC1456pe - 1./162.*m33*m66*
         RN1p*RSe456p*RS145pe*RSp453e - 1./162.*m33*m66*RN1p*RSe456p*
         RSp451e*RS145pe - 1./162.*m33*m66*RN1e*RSe456p*RS145pe*RSe453p
          - 1./162.*m33*m66*RN1e*RSe456p*RSe451p*RS145pe - 1./162.*m33*
         m66*RN13*RSe456p*RS145pe*RS345pe - 1./162.*m33*m55*RN1p*
         RSe465p*RS146pe*RSp463e );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(me2,L) * (  - 1./162.*m33*m55*
         RN1p*RSe465p*RSp461e*RS146pe - 1./162.*m33*m55*RN1e*RSe465p*
         RS146pe*RSe463p - 1./162.*m33*m55*RN1e*RSe465p*RSe461p*RS146pe
          - 1./162.*m33*m55*RN13*RSe465p*RS146pe*RS346pe - 1./162.*m33*
         m44*RN1p*RSe564p*RS156pe*RSp563e - 1./162.*m33*m44*RN1p*
         RSe564p*RSp561e*RS156pe - 1./162.*m33*m44*RN1e*RSe564p*RS156pe
         *RSe563p - 1./162.*m33*m44*RN1e*RSe564p*RSe561p*RS156pe - 1./
         162.*m33*m44*RN13*RSe564p*RS156pe*RS356pe + 1./54.*pow(m33,2)*
         RN13*RSe45633p*RS14563pe - 7./405.*m13*RSe45613p*RS14563pe + 7.
         /810.*m13*RSe45633p*RS14563pe + 7./810.*m13*RC1456pe*RSe45613p
          - 7./1620.*m13*RC1456pe*RSe45633p + 7./810.*m13*RSe45611p*
         RS14563pe - 7./1620.*m13*RSe45611p*RC1456pe - 1./81.*m13*m66*
         RN1p*RSe456p*RS145pe*RSp453e + 1./81.*m13*m66*RN1p*RSe456p*
         RSp451e*RS145pe - 1./81.*m13*m66*RN1e*RSe456p*RS145pe*RSe453p
          + 1./81.*m13*m66*RN1e*RSe456p*RSe451p*RS145pe - 1./81.*m13*
         m66*RN13*RSe456p*RS145pe*RS345pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(me2,L) * (  - 1./81.*m13*m55*
         RN1p*RSe465p*RS146pe*RSp463e + 1./81.*m13*m55*RN1p*RSe465p*
         RSp461e*RS146pe - 1./81.*m13*m55*RN1e*RSe465p*RS146pe*RSe463p
          + 1./81.*m13*m55*RN1e*RSe465p*RSe461p*RS146pe - 1./81.*m13*
         m55*RN13*RSe465p*RS146pe*RS346pe - 1./81.*m13*m44*RN1p*RSe564p
         *RS156pe*RSp563e + 1./81.*m13*m44*RN1p*RSe564p*RSp561e*RS156pe
          - 1./81.*m13*m44*RN1e*RSe564p*RS156pe*RSe563p + 1./81.*m13*
         m44*RN1e*RSe564p*RSe561p*RS156pe - 1./81.*m13*m44*RN13*RSe564p
         *RS156pe*RS356pe + 1./27.*m13*m33*RN13*RSe45633p*RS14563pe + 
         16./405.*m11*RSe45613p*RS14563pe + 1./162.*m11*RSe45633p*
         RS14563pe + 13./810.*m11*RC1456pe*RSe45613p - 19./1620.*m11*
         RC1456pe*RSe45633p + 23./810.*m11*RSe45611p*RS14563pe - 37./
         1620.*m11*RSe45611p*RC1456pe - 1./54.*m11*m66*RN1p*RSe456p*
         RS145pe*RSp453e - 1./162.*m11*m66*RN1p*RSe456p*RSp451e*RS145pe
          - 1./54.*m11*m66*RN1e*RSe456p*RS145pe*RSe453p - 1./162.*m11*
         m66*RN1e*RSe456p*RSe451p*RS145pe );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(me2,L) * (  - 1./54.*m11*m66*
         RN13*RSe456p*RS145pe*RS345pe - 1./54.*m11*m55*RN1p*RSe465p*
         RS146pe*RSp463e - 1./162.*m11*m55*RN1p*RSe465p*RSp461e*RS146pe
          - 1./54.*m11*m55*RN1e*RSe465p*RS146pe*RSe463p - 1./162.*m11*
         m55*RN1e*RSe465p*RSe461p*RS146pe - 1./54.*m11*m55*RN13*RSe465p
         *RS146pe*RS346pe - 1./54.*m11*m44*RN1p*RSe564p*RS156pe*RSp563e
          - 1./162.*m11*m44*RN1p*RSe564p*RSp561e*RS156pe - 1./54.*m11*
         m44*RN1e*RSe564p*RS156pe*RSe563p - 1./162.*m11*m44*RN1e*
         RSe564p*RSe561p*RS156pe - 1./54.*m11*m44*RN13*RSe564p*RS156pe*
         RS356pe + 1./54.*m11*m33*RN1p*RSe45611p*RSp45613e + 1./54.*m11
         *m33*RN1p*RSp45611e*RSe45611p + 1./54.*m11*m33*RN1e*RSe45611p*
         RSe45613p + 1./54.*m11*m33*RN1e*pow(RSe45611p,2) + 1./18.*m11*
         m33*RN13*RSe45633p*RS14563pe + 1./54.*m11*m33*RN13*RSe45611p*
         RS34561pe + 1./27.*m11*m13*RN1p*RSe45611p*RSp45613e - 1./27.*
         m11*m13*RN1p*RSp45611e*RSe45611p + 1./27.*m11*m13*RN1e*
         RSe45611p*RSe45613p );

      M13p6x23RV +=  + Ab(m11,mu2)*AbV(me2,L) * (  - 1./27.*m11*m13*
         RN1e*pow(RSe45611p,2) + 1./27.*m11*m13*RN13*RSe45611p*
         RS34561pe + 1./18.*pow(m11,2)*RN1p*RSe45611p*RSp45613e + 1./54.
         *pow(m11,2)*RN1p*RSp45611e*RSe45611p + 1./18.*pow(m11,2)*RN1e*
         RSe45611p*RSe45613p + 1./54.*pow(m11,2)*RN1e*pow(RSe45611p,2)
          + 1./18.*pow(m11,2)*RN13*RSe45611p*RS34561pe + 1./81.*RS(m11,
         m44,m55,m66,m33,me2,mp2)*m33*RSe45613p - 1./162.*RS(m11,m44,
         m55,m66,m33,me2,mp2)*m33*RSe45633p - 1./162.*RS(m11,m44,m55,
         m66,m33,me2,mp2)*m33*RSe45611p + 2./81.*RS(m11,m44,m55,m66,m33
         ,me2,mp2)*m11*RSe45613p - 1./81.*RS(m11,m44,m55,m66,m33,me2,
         mp2)*m11*RSe45633p - 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*
         m11*RSe45611p );

      M13p6x23RV +=  + Ab(m11,mu2)*BbV(m11,m33,L) * ( 1./54.*m11*m33*
         RC1456pe*RS34561pe + 1./27.*m11*m13*RC1456pe*RS34561pe + 1./18.
         *pow(m11,2)*RC1456pe*RS34561pe );

      M13p6x23RV +=  + Ab(m11,mu2)*BbV(m11,mp2,L) * ( 1./54.*m11*m33*
         RC1456pe*RSp45613e + 1./54.*m11*m33*RSp45611e*RC1456pe + 1./27.
         *m11*m13*RC1456pe*RSp45613e - 1./27.*m11*m13*RSp45611e*
         RC1456pe + 1./18.*pow(m11,2)*RC1456pe*RSp45613e + 1./54.*pow(
         m11,2)*RSp45611e*RC1456pe );

      M13p6x23RV +=  + Ab(m11,mu2)*BbV(m11,me2,L) * ( 1./54.*m11*m33*
         RC1456pe*RSe45613p + 1./54.*m11*m33*RSe45611p*RC1456pe + 1./27.
         *m11*m13*RC1456pe*RSe45613p - 1./27.*m11*m13*RSe45611p*
         RC1456pe + 1./18.*pow(m11,2)*RC1456pe*RSe45613p + 1./54.*pow(
         m11,2)*RSe45611p*RC1456pe );

      M13p6x23RV +=  + Ab(m11,mu2)*BbV(m11,L) * ( 1./540.*pow(
         RD1456ep,2) + 1./270.*m33*RD1456ep*RS14563pe + 13./810.*m33*
         RD1456ep*RC1456pe + 7./810.*m13*RD1456ep*RS14563pe - 7./1620.*
         m13*RD1456ep*RC1456pe + 2./81.*m11*RD1456ep*RS14563pe - 17./
         810.*m11*RD1456ep*RC1456pe + 1./12.*m11*m33 + 1./54.*m11*m33*
         RC1456pe*RS14563pe + 1./54.*m11*m33*pow(RC1456pe,2) + 1./54.*
         m11*m33*RN1p*RD1456ep*RSp45613e + 1./54.*m11*m33*RN1p*
         RSp45611e*RD1456ep + 1./54.*m11*m33*RN1e*RD1456ep*RSe45613p + 
         1./54.*m11*m33*RN1e*RSe45611p*RD1456ep + 1./54.*m11*m33*RN13*
         RD1456ep*RS34561pe - 1./6.*m11*m13 + 1./27.*m11*m13*RC1456pe*
         RS14563pe - 1./27.*m11*m13*pow(RC1456pe,2) + 1./27.*m11*m13*
         RN1p*RD1456ep*RSp45613e - 1./27.*m11*m13*RN1p*RSp45611e*
         RD1456ep + 1./27.*m11*m13*RN1e*RD1456ep*RSe45613p - 1./27.*m11
         *m13*RN1e*RSe45611p*RD1456ep + 1./27.*m11*m13*RN13*RD1456ep*
         RS34561pe + 1./12.*pow(m11,2) + 1./18.*pow(m11,2)*RC1456pe*
         RS14563pe );

      M13p6x23RV +=  + Ab(m11,mu2)*BbV(m11,L) * ( 1./54.*pow(m11,2)*
         pow(RC1456pe,2) + 1./18.*pow(m11,2)*RN1p*RD1456ep*RSp45613e + 
         1./54.*pow(m11,2)*RN1p*RSp45611e*RD1456ep + 1./18.*pow(m11,2)*
         RN1e*RD1456ep*RSe45613p + 1./54.*pow(m11,2)*RN1e*RSe45611p*
         RD1456ep + 1./18.*pow(m11,2)*RN13*RD1456ep*RS34561pe - 1./162.
         *RS(m11,m44,m55,m66,m33,me2,mp2)*m33*RD1456ep - 1./81.*RS(m11,
         m44,m55,m66,m33,me2,mp2)*m11*RD1456ep );

      M13p6x23RV +=  + Ab(m11,mu2)*BbV(m14,L) * ( 1./216.*m44*RD1456ep
          - 1./432.*pow(m44,2)*RC1456pe - 1./216.*pow(m44,2)*RS156pe + 
         1./216.*m33*RD1456ep - 1./432.*m33*m44*RC1456pe - 1./216.*m33*
         m44*RS156pe - 1./108.*m14*RD1456ep + 1./108.*m14*m44*RC1456pe
          + 1./216.*m14*m33*RC1456pe - 1./108.*m14*m33*RS156pe - 1./108.
         *pow(m14,2)*RC1456pe + 1./54.*pow(m14,2)*RS156pe - 1./108.*m13
         *RD1456ep + 1./216.*m13*m44*RC1456pe + 1./108.*m13*m44*RS156pe
          - 1./108.*m13*m14*RC1456pe + 1./54.*m13*m14*RS156pe + 1./108.
         *m11*RD1456ep - 1./144.*m11*m44*RC1456pe - 5./216.*m11*m44*
         RS156pe - 1./432.*m11*m33*RC1456pe - 1./72.*m11*m33*RS156pe + 
         1./72.*m11*m14*RC1456pe + 1./108.*m11*m14*RS156pe + 1./216.*
         m11*m13*RC1456pe + 1./36.*m11*m13*RS156pe - 1./216.*pow(m11,2)
         *RC1456pe - 1./36.*pow(m11,2)*RS156pe );

      M13p6x23RV +=  + Ab(m11,mu2)*BbV(m15,L) * ( 1./216.*m55*RD1456ep
          - 1./432.*pow(m55,2)*RC1456pe - 1./216.*pow(m55,2)*RS146pe + 
         1./216.*m33*RD1456ep - 1./432.*m33*m55*RC1456pe - 1./216.*m33*
         m55*RS146pe - 1./108.*m15*RD1456ep + 1./108.*m15*m55*RC1456pe
          + 1./216.*m15*m33*RC1456pe - 1./108.*m15*m33*RS146pe - 1./108.
         *pow(m15,2)*RC1456pe + 1./54.*pow(m15,2)*RS146pe - 1./108.*m13
         *RD1456ep + 1./216.*m13*m55*RC1456pe + 1./108.*m13*m55*RS146pe
          - 1./108.*m13*m15*RC1456pe + 1./54.*m13*m15*RS146pe + 1./108.
         *m11*RD1456ep - 1./144.*m11*m55*RC1456pe - 5./216.*m11*m55*
         RS146pe - 1./432.*m11*m33*RC1456pe - 1./72.*m11*m33*RS146pe + 
         1./72.*m11*m15*RC1456pe + 1./108.*m11*m15*RS146pe + 1./216.*
         m11*m13*RC1456pe + 1./36.*m11*m13*RS146pe - 1./216.*pow(m11,2)
         *RC1456pe - 1./36.*pow(m11,2)*RS146pe );

      M13p6x23RV +=  + Ab(m11,mu2)*BbV(m16,L) * ( 1./216.*m66*RD1456ep
          - 1./432.*pow(m66,2)*RC1456pe - 1./216.*pow(m66,2)*RS145pe + 
         1./216.*m33*RD1456ep - 1./432.*m33*m66*RC1456pe - 1./216.*m33*
         m66*RS145pe - 1./108.*m16*RD1456ep + 1./108.*m16*m66*RC1456pe
          + 1./216.*m16*m33*RC1456pe - 1./108.*m16*m33*RS145pe - 1./108.
         *pow(m16,2)*RC1456pe + 1./54.*pow(m16,2)*RS145pe - 1./108.*m13
         *RD1456ep + 1./216.*m13*m66*RC1456pe + 1./108.*m13*m66*RS145pe
          - 1./108.*m13*m16*RC1456pe + 1./54.*m13*m16*RS145pe + 1./108.
         *m11*RD1456ep - 1./144.*m11*m66*RC1456pe - 5./216.*m11*m66*
         RS145pe - 1./432.*m11*m33*RC1456pe - 1./72.*m11*m33*RS145pe + 
         1./72.*m11*m16*RC1456pe + 1./108.*m11*m16*RS145pe + 1./216.*
         m11*m13*RC1456pe + 1./36.*m11*m13*RS145pe - 1./216.*pow(m11,2)
         *RC1456pe - 1./36.*pow(m11,2)*RS145pe );

      M13p6x23RV +=  + Ab(m11,mu2)*BbV(m33,L) * ( 1./540.*RD1456ep*
         RD3456ep - 2./135.*m33*RD3456ep*RS14563pe - 19./1620.*m33*
         RC1456pe*RD3456ep + 1./54.*pow(m33,2)*RN13*RD3456ep*RS14563pe
          + 7./810.*m13*RD3456ep*RS14563pe - 7./1620.*m13*RC1456pe*
         RD3456ep + 1./27.*m13*m33*RN13*RD3456ep*RS14563pe + 1./162.*
         m11*RD3456ep*RS14563pe - 19./1620.*m11*RC1456pe*RD3456ep + 1./
         18.*m11*m33*RN13*RD3456ep*RS14563pe - 1./162.*RS(m11,m44,m55,
         m66,m33,me2,mp2)*m33*RD3456ep - 1./81.*RS(m11,m44,m55,m66,m33,
         me2,mp2)*m11*RD3456ep );

      M13p6x23RV +=  + Ab(m11,mu2)*CbV(m11,L) * ( 1./54.*m11*m33*
         RD1456ep*RC1456pe - 1./27.*m11*m13*RD1456ep*RC1456pe + 1./54.*
         pow(m11,2)*RD1456ep*RC1456pe );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(m11,L) * (  - 1./162.*RD3456ep*
         RS14563pe + 1./324.*RC1456pe*RD3456ep - 1./54.*RD1456ep*
         RS34561pe - 2./81.*RD1456ep*RS14563pe - 1./54.*RD1456ep*
         RSe45613p - 1./54.*RD1456ep*RSp45613e + 1./108.*RD1456ep*
         RC3456pe + 1./108.*RD1456ep*RSe45633p + 1./108.*RD1456ep*
         RSp45633e + 1./81.*RD1456ep*RC1456pe + 1./108.*RSe45611p*
         RD1456ep + 1./108.*RSp45611e*RD1456ep + 1./81.*me2*RSe45613p*
         RS14563pe - 1./162.*me2*RSe45633p*RS14563pe - 1./162.*me2*
         RC1456pe*RSe45613p + 1./324.*me2*RC1456pe*RSe45633p - 1./162.*
         me2*RSe45611p*RS14563pe + 1./324.*me2*RSe45611p*RC1456pe + 1./
         81.*mp2*RSp45613e*RS14563pe - 1./162.*mp2*RSp45633e*RS14563pe
          - 1./162.*mp2*RC1456pe*RSp45613e + 1./324.*mp2*RC1456pe*
         RSp45633e - 1./162.*mp2*RSp45611e*RS14563pe + 1./324.*mp2*
         RSp45611e*RC1456pe - 1./60.*m33 - 1./81.*m33*pow(RS14563pe,2)
          - 1./81.*m33*RSe45613p*RS14563pe - 1./81.*m33*RSp45613e*
         RS14563pe );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(m11,L) * ( 1./162.*m33*RC3456pe*
         RS14563pe + 1./81.*m33*RSe45633p*RS14563pe + 1./81.*m33*
         RSp45633e*RS14563pe - 1./162.*m33*RC1456pe*RS34561pe + 1./108.
         *m33*RC1456pe*RC3456pe + 1./162.*m33*RC1456pe*RSe45633p + 1./
         162.*m33*RC1456pe*RSp45633e - 1./162.*m33*pow(RC1456pe,2) - 1./
         162.*m33*RSe45611p*RC1456pe - 1./162.*m33*RSp45611e*RC1456pe
          + 1./180.*m13 - 2./27.*m13*RS14563pe*RS34561pe - 2./27.*m13*
         pow(RS14563pe,2) - 2./27.*m13*RSe45613p*RS14563pe - 2./27.*m13
         *RSp45613e*RS14563pe + 1./27.*m13*RC3456pe*RS14563pe + 1./27.*
         m13*RSe45633p*RS14563pe + 1./27.*m13*RSp45633e*RS14563pe + 1./
         27.*m13*RC1456pe*RS34561pe + 2./27.*m13*RC1456pe*RS14563pe + 1.
         /27.*m13*RC1456pe*RSe45613p + 1./27.*m13*RC1456pe*RSp45613e - 
         1./54.*m13*RC1456pe*RC3456pe - 1./54.*m13*RC1456pe*RSe45633p
          - 1./54.*m13*RC1456pe*RSp45633e - 1./54.*m13*pow(RC1456pe,2)
          + 1./27.*m13*RSe45611p*RS14563pe - 1./54.*m13*RSe45611p*
         RC1456pe );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(m11,L) * ( 1./27.*m13*RSp45611e*
         RS14563pe - 1./54.*m13*RSp45611e*RC1456pe + 7./180.*m11 + 2./
         81.*m11*RS14563pe*RS34561pe + 1./27.*m11*pow(RS14563pe,2) + 2./
         81.*m11*RSe45613p*RS14563pe + 2./81.*m11*RSp45613e*RS14563pe
          - 1./54.*m11*RC3456pe*RS14563pe - 1./54.*m11*RSe45633p*
         RS14563pe - 1./54.*m11*RSp45633e*RS14563pe - 1./162.*m11*
         RC1456pe*RS34561pe - 2./81.*m11*RC1456pe*RS14563pe - 1./162.*
         m11*RC1456pe*RSe45613p - 1./162.*m11*RC1456pe*RSp45613e + 5./
         324.*m11*RC1456pe*RC3456pe + 5./324.*m11*RC1456pe*RSe45633p + 
         5./324.*m11*RC1456pe*RSp45633e - 1./162.*m11*pow(RC1456pe,2)
          - 1./162.*m11*RSe45611p*RS14563pe - 1./108.*m11*RSe45611p*
         RC1456pe - 1./162.*m11*RSp45611e*RS14563pe - 1./108.*m11*
         RSp45611e*RC1456pe );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(m13,L) * ( 1./30.*m33 - 17./30.*
         m13 + 1./30.*m11 );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(m33,L) * (  - 2./81.*RD3456ep*
         RS34561pe - 1./54.*RD3456ep*RS14563pe - 1./54.*RD3456ep*
         RSe45613p - 1./54.*RD3456ep*RSp45613e + 1./81.*RD3456ep*
         RC3456pe + 1./108.*RSe45633p*RD3456ep + 1./108.*RSp45633e*
         RD3456ep + 1./108.*RC1456pe*RD3456ep - 1./162.*RD1456ep*
         RS34561pe + 1./324.*RD1456ep*RC3456pe + 1./108.*RSe45611p*
         RD3456ep + 1./108.*RSp45611e*RD3456ep + 1./81.*me2*RSe45613p*
         RS34561pe - 1./162.*me2*RC3456pe*RSe45613p - 1./162.*me2*
         RSe45633p*RS34561pe + 1./324.*me2*RSe45633p*RC3456pe - 1./162.
         *me2*RSe45611p*RS34561pe + 1./324.*me2*RSe45611p*RC3456pe + 1./
         81.*mp2*RSp45613e*RS34561pe - 1./162.*mp2*RC3456pe*RSp45613e
          - 1./162.*mp2*RSp45633e*RS34561pe + 1./324.*mp2*RSp45633e*
         RC3456pe - 1./162.*mp2*RSp45611e*RS34561pe + 1./324.*mp2*
         RSp45611e*RC3456pe + 3./20.*m33 + 1./27.*m33*pow(RS34561pe,2)
          + 2./81.*m33*RS14563pe*RS34561pe + 2./81.*m33*RSe45613p*
         RS34561pe );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(m33,L) * ( 2./81.*m33*RSp45613e*
         RS34561pe - 2./81.*m33*RC3456pe*RS34561pe - 1./162.*m33*
         RC3456pe*RS14563pe - 1./162.*m33*RC3456pe*RSe45613p - 1./162.*
         m33*RC3456pe*RSp45613e - 1./162.*m33*pow(RC3456pe,2) - 1./162.
         *m33*RSe45633p*RS34561pe - 1./108.*m33*RSe45633p*RC3456pe - 1./
         162.*m33*RSp45633e*RS34561pe - 1./108.*m33*RSp45633e*RC3456pe
          - 1./54.*m33*RC1456pe*RS34561pe + 5./324.*m33*RC1456pe*
         RC3456pe - 1./54.*m33*RSe45611p*RS34561pe + 5./324.*m33*
         RSe45611p*RC3456pe - 1./54.*m33*RSp45611e*RS34561pe + 5./324.*
         m33*RSp45611e*RC3456pe + 41./180.*m13 - 2./27.*m13*pow(
         RS34561pe,2) - 2./27.*m13*RS14563pe*RS34561pe - 2./27.*m13*
         RSe45613p*RS34561pe - 2./27.*m13*RSp45613e*RS34561pe + 2./27.*
         m13*RC3456pe*RS34561pe + 1./27.*m13*RC3456pe*RS14563pe + 1./27.
         *m13*RC3456pe*RSe45613p + 1./27.*m13*RC3456pe*RSp45613e - 1./
         54.*m13*pow(RC3456pe,2) + 1./27.*m13*RSe45633p*RS34561pe - 1./
         54.*m13*RSe45633p*RC3456pe );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(m33,L) * ( 1./27.*m13*RSp45633e*
         RS34561pe - 1./54.*m13*RSp45633e*RC3456pe + 1./27.*m13*
         RC1456pe*RS34561pe - 1./54.*m13*RC1456pe*RC3456pe + 1./27.*m13
         *RSe45611p*RS34561pe - 1./54.*m13*RSe45611p*RC3456pe + 1./27.*
         m13*RSp45611e*RS34561pe - 1./54.*m13*RSp45611e*RC3456pe + 17./
         180.*m11 - 1./81.*m11*pow(RS34561pe,2) - 1./81.*m11*RSe45613p*
         RS34561pe - 1./81.*m11*RSp45613e*RS34561pe - 1./162.*m11*
         RC3456pe*RS14563pe - 1./162.*m11*pow(RC3456pe,2) - 1./162.*m11
         *RSe45633p*RC3456pe - 1./162.*m11*RSp45633e*RC3456pe + 1./162.
         *m11*RC1456pe*RS34561pe + 1./108.*m11*RC1456pe*RC3456pe + 1./
         81.*m11*RSe45611p*RS34561pe + 1./162.*m11*RSe45611p*RC3456pe
          + 1./81.*m11*RSp45611e*RS34561pe + 1./162.*m11*RSp45611e*
         RC3456pe );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(mp2,L) * (  - 1./162.*RD3456ep*
         RSp45613e + 1./324.*RSp45633e*RD3456ep - 1./162.*RD1456ep*
         RSp45613e + 1./324.*RD1456ep*RSp45633e + 1./324.*RSp45611e*
         RD3456ep + 1./324.*RSp45611e*RD1456ep + 1./81.*me2*RSp45613e*
         RSe45613p - 1./162.*me2*RSe45633p*RSp45613e - 1./162.*me2*
         RSp45633e*RSe45613p + 1./324.*me2*RSp45633e*RSe45633p - 1./162.
         *me2*RSe45611p*RSp45613e + 1./324.*me2*RSe45611p*RSp45633e - 1.
         /162.*me2*RSp45611e*RSe45613p + 1./324.*me2*RSp45611e*
         RSe45633p + 1./324.*me2*RSp45611e*RSe45611p + 1./27.*mp2*
         RSp45613e*RS34561pe + 1./27.*mp2*RSp45613e*RS14563pe + 1./27.*
         mp2*RSp45613e*RSe45613p + 4./81.*mp2*pow(RSp45613e,2) - 1./54.
         *mp2*RC3456pe*RSp45613e - 1./54.*mp2*RSe45633p*RSp45613e - 1./
         54.*mp2*RSp45633e*RS34561pe - 1./54.*mp2*RSp45633e*RS14563pe
          - 1./54.*mp2*RSp45633e*RSe45613p - 4./81.*mp2*RSp45633e*
         RSp45613e + 1./108.*mp2*RSp45633e*RC3456pe + 1./108.*mp2*
         RSp45633e*RSe45633p );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(mp2,L) * ( 1./81.*mp2*pow(
         RSp45633e,2) - 1./54.*mp2*RC1456pe*RSp45613e + 1./108.*mp2*
         RC1456pe*RSp45633e - 1./54.*mp2*RSe45611p*RSp45613e + 1./108.*
         mp2*RSe45611p*RSp45633e - 1./54.*mp2*RSp45611e*RS34561pe - 1./
         54.*mp2*RSp45611e*RS14563pe - 1./54.*mp2*RSp45611e*RSe45613p
          - 4./81.*mp2*RSp45611e*RSp45613e + 1./108.*mp2*RSp45611e*
         RC3456pe + 1./108.*mp2*RSp45611e*RSe45633p + 2./81.*mp2*
         RSp45611e*RSp45633e + 1./108.*mp2*RSp45611e*RC1456pe + 1./108.
         *mp2*RSp45611e*RSe45611p + 1./81.*mp2*pow(RSp45611e,2) - 1./81.
         *m33*RSp45613e*RS14563pe - 1./81.*m33*RSp45613e*RSe45613p - 1./
         81.*m33*pow(RSp45613e,2) + 1./162.*m33*RC3456pe*RSp45613e + 1./
         81.*m33*RSe45633p*RSp45613e + 1./162.*m33*RSp45633e*RS34561pe
          + 1./81.*m33*RSp45633e*RS14563pe + 1./81.*m33*RSp45633e*
         RSe45613p + 2./81.*m33*RSp45633e*RSp45613e - 5./324.*m33*
         RSp45633e*RC3456pe - 1./54.*m33*RSp45633e*RSe45633p - 1./54.*
         m33*pow(RSp45633e,2) );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(mp2,L) * ( 1./162.*m33*RC1456pe*
         RSp45633e + 1./162.*m33*RSe45611p*RSp45633e - 1./162.*m33*
         RSp45611e*RS34561pe + 1./108.*m33*RSp45611e*RC3456pe + 1./162.
         *m33*RSp45611e*RSe45633p + 1./81.*m33*RSp45611e*RSp45633e - 1./
         162.*m33*RSp45611e*RC1456pe - 1./162.*m33*RSp45611e*RSe45611p
          - 1./162.*m33*pow(RSp45611e,2) - 2./27.*m13*RSp45613e*
         RS34561pe - 2./27.*m13*RSp45613e*RS14563pe - 2./27.*m13*
         RSp45613e*RSe45613p - 2./27.*m13*pow(RSp45613e,2) + 1./27.*m13
         *RC3456pe*RSp45613e + 1./27.*m13*RSe45633p*RSp45613e + 1./27.*
         m13*RSp45633e*RS34561pe + 1./27.*m13*RSp45633e*RS14563pe + 1./
         27.*m13*RSp45633e*RSe45613p + 2./27.*m13*RSp45633e*RSp45613e
          - 1./54.*m13*RSp45633e*RC3456pe - 1./54.*m13*RSp45633e*
         RSe45633p - 1./54.*m13*pow(RSp45633e,2) + 1./27.*m13*RC1456pe*
         RSp45613e - 1./54.*m13*RC1456pe*RSp45633e + 1./27.*m13*
         RSe45611p*RSp45613e - 1./54.*m13*RSe45611p*RSp45633e + 1./27.*
         m13*RSp45611e*RS34561pe );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(mp2,L) * ( 1./27.*m13*RSp45611e*
         RS14563pe + 1./27.*m13*RSp45611e*RSe45613p + 2./27.*m13*
         RSp45611e*RSp45613e - 1./54.*m13*RSp45611e*RC3456pe - 1./54.*
         m13*RSp45611e*RSe45633p - 1./27.*m13*RSp45611e*RSp45633e - 1./
         54.*m13*RSp45611e*RC1456pe - 1./54.*m13*RSp45611e*RSe45611p - 
         1./54.*m13*pow(RSp45611e,2) - 1./81.*m11*RSp45613e*RS34561pe
          - 1./81.*m11*RSp45613e*RSe45613p - 1./81.*m11*pow(
         RSp45613e,2) - 1./162.*m11*RSp45633e*RS14563pe - 1./162.*m11*
         RSp45633e*RC3456pe - 1./162.*m11*RSp45633e*RSe45633p - 1./162.
         *m11*pow(RSp45633e,2) + 1./162.*m11*RC1456pe*RSp45613e + 1./
         108.*m11*RC1456pe*RSp45633e + 1./81.*m11*RSe45611p*RSp45613e
          + 1./162.*m11*RSe45611p*RSp45633e + 1./81.*m11*RSp45611e*
         RS34561pe + 1./162.*m11*RSp45611e*RS14563pe + 1./81.*m11*
         RSp45611e*RSe45613p + 2./81.*m11*RSp45611e*RSp45613e + 1./162.
         *m11*RSp45611e*RC3456pe + 1./162.*m11*RSp45611e*RSe45633p + 1./
         81.*m11*RSp45611e*RSp45633e );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(mp2,L) * (  - 5./324.*m11*
         RSp45611e*RC1456pe - 1./54.*m11*RSp45611e*RSe45611p - 1./54.*
         m11*pow(RSp45611e,2) );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(me2,L) * (  - 1./162.*RD3456ep*
         RSe45613p + 1./324.*RSe45633p*RD3456ep - 1./162.*RD1456ep*
         RSe45613p + 1./324.*RD1456ep*RSe45633p + 1./324.*RSe45611p*
         RD3456ep + 1./324.*RSe45611p*RD1456ep + 1./27.*me2*RSe45613p*
         RS34561pe + 1./27.*me2*RSe45613p*RS14563pe + 4./81.*me2*pow(
         RSe45613p,2) + 1./27.*me2*RSp45613e*RSe45613p - 1./54.*me2*
         RC3456pe*RSe45613p - 1./54.*me2*RSe45633p*RS34561pe - 1./54.*
         me2*RSe45633p*RS14563pe - 4./81.*me2*RSe45633p*RSe45613p - 1./
         54.*me2*RSe45633p*RSp45613e + 1./108.*me2*RSe45633p*RC3456pe
          + 1./81.*me2*pow(RSe45633p,2) - 1./54.*me2*RSp45633e*
         RSe45613p + 1./108.*me2*RSp45633e*RSe45633p - 1./54.*me2*
         RC1456pe*RSe45613p + 1./108.*me2*RC1456pe*RSe45633p - 1./54.*
         me2*RSe45611p*RS34561pe - 1./54.*me2*RSe45611p*RS14563pe - 4./
         81.*me2*RSe45611p*RSe45613p - 1./54.*me2*RSe45611p*RSp45613e
          + 1./108.*me2*RSe45611p*RC3456pe + 2./81.*me2*RSe45611p*
         RSe45633p );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(me2,L) * ( 1./108.*me2*RSe45611p
         *RSp45633e + 1./108.*me2*RSe45611p*RC1456pe + 1./81.*me2*pow(
         RSe45611p,2) - 1./54.*me2*RSp45611e*RSe45613p + 1./108.*me2*
         RSp45611e*RSe45633p + 1./108.*me2*RSp45611e*RSe45611p + 1./81.
         *mp2*RSp45613e*RSe45613p - 1./162.*mp2*RSe45633p*RSp45613e - 1.
         /162.*mp2*RSp45633e*RSe45613p + 1./324.*mp2*RSp45633e*
         RSe45633p - 1./162.*mp2*RSe45611p*RSp45613e + 1./324.*mp2*
         RSe45611p*RSp45633e - 1./162.*mp2*RSp45611e*RSe45613p + 1./324.
         *mp2*RSp45611e*RSe45633p + 1./324.*mp2*RSp45611e*RSe45611p - 1.
         /81.*m33*RSe45613p*RS14563pe - 1./81.*m33*pow(RSe45613p,2) - 1.
         /81.*m33*RSp45613e*RSe45613p + 1./162.*m33*RC3456pe*RSe45613p
          + 1./162.*m33*RSe45633p*RS34561pe + 1./81.*m33*RSe45633p*
         RS14563pe + 2./81.*m33*RSe45633p*RSe45613p + 1./81.*m33*
         RSe45633p*RSp45613e - 5./324.*m33*RSe45633p*RC3456pe - 1./54.*
         m33*pow(RSe45633p,2) + 1./81.*m33*RSp45633e*RSe45613p - 1./54.
         *m33*RSp45633e*RSe45633p );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(me2,L) * ( 1./162.*m33*RC1456pe*
         RSe45633p - 1./162.*m33*RSe45611p*RS34561pe + 1./108.*m33*
         RSe45611p*RC3456pe + 1./81.*m33*RSe45611p*RSe45633p + 1./162.*
         m33*RSe45611p*RSp45633e - 1./162.*m33*RSe45611p*RC1456pe - 1./
         162.*m33*pow(RSe45611p,2) + 1./162.*m33*RSp45611e*RSe45633p - 
         1./162.*m33*RSp45611e*RSe45611p - 2./27.*m13*RSe45613p*
         RS34561pe - 2./27.*m13*RSe45613p*RS14563pe - 2./27.*m13*pow(
         RSe45613p,2) - 2./27.*m13*RSp45613e*RSe45613p + 1./27.*m13*
         RC3456pe*RSe45613p + 1./27.*m13*RSe45633p*RS34561pe + 1./27.*
         m13*RSe45633p*RS14563pe + 2./27.*m13*RSe45633p*RSe45613p + 1./
         27.*m13*RSe45633p*RSp45613e - 1./54.*m13*RSe45633p*RC3456pe - 
         1./54.*m13*pow(RSe45633p,2) + 1./27.*m13*RSp45633e*RSe45613p
          - 1./54.*m13*RSp45633e*RSe45633p + 1./27.*m13*RC1456pe*
         RSe45613p - 1./54.*m13*RC1456pe*RSe45633p + 1./27.*m13*
         RSe45611p*RS34561pe + 1./27.*m13*RSe45611p*RS14563pe + 2./27.*
         m13*RSe45611p*RSe45613p );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(me2,L) * ( 1./27.*m13*RSe45611p*
         RSp45613e - 1./54.*m13*RSe45611p*RC3456pe - 1./27.*m13*
         RSe45611p*RSe45633p - 1./54.*m13*RSe45611p*RSp45633e - 1./54.*
         m13*RSe45611p*RC1456pe - 1./54.*m13*pow(RSe45611p,2) + 1./27.*
         m13*RSp45611e*RSe45613p - 1./54.*m13*RSp45611e*RSe45633p - 1./
         54.*m13*RSp45611e*RSe45611p - 1./81.*m11*RSe45613p*RS34561pe
          - 1./81.*m11*pow(RSe45613p,2) - 1./81.*m11*RSp45613e*
         RSe45613p - 1./162.*m11*RSe45633p*RS14563pe - 1./162.*m11*
         RSe45633p*RC3456pe - 1./162.*m11*pow(RSe45633p,2) - 1./162.*
         m11*RSp45633e*RSe45633p + 1./162.*m11*RC1456pe*RSe45613p + 1./
         108.*m11*RC1456pe*RSe45633p + 1./81.*m11*RSe45611p*RS34561pe
          + 1./162.*m11*RSe45611p*RS14563pe + 2./81.*m11*RSe45611p*
         RSe45613p + 1./81.*m11*RSe45611p*RSp45613e + 1./162.*m11*
         RSe45611p*RC3456pe + 1./81.*m11*RSe45611p*RSe45633p + 1./162.*
         m11*RSe45611p*RSp45633e - 5./324.*m11*RSe45611p*RC1456pe - 1./
         54.*m11*pow(RSe45611p,2) );

      M13p6x23RV +=  + Ab(m13,mu2)*AbV(me2,L) * ( 1./81.*m11*RSp45611e*
         RSe45613p + 1./162.*m11*RSp45611e*RSe45633p - 1./54.*m11*
         RSp45611e*RSe45611p );

      M13p6x23RV +=  + Ab(m13,mu2)*BbV(m11,m33,L) * ( 1./36.*pow(m33,2)
          + 1./9.*m13*m33 + 1./9.*pow(m13,2) + 5./18.*m11*m33 + 1./3.*
         m11*m13 + 5./36.*pow(m11,2) );

      M13p6x23RV +=  + Ab(m13,mu2)*BbV(m11,L) * ( 1./324.*RD1456ep*
         RD3456ep + 1./324.*pow(RD1456ep,2) - 1./162.*me2*RD1456ep*
         RSe45613p + 1./324.*me2*RD1456ep*RSe45633p + 1./324.*me2*
         RSe45611p*RD1456ep - 1./162.*mp2*RD1456ep*RSp45613e + 1./324.*
         mp2*RD1456ep*RSp45633e + 1./324.*mp2*RSp45611e*RD1456ep - 1./
         162.*m33*RD1456ep*RS34561pe + 1./108.*m33*RD1456ep*RC3456pe + 
         1./162.*m33*RD1456ep*RSe45633p + 1./162.*m33*RD1456ep*
         RSp45633e - 1./162.*m33*RD1456ep*RC1456pe - 1./162.*m33*
         RSe45611p*RD1456ep - 1./162.*m33*RSp45611e*RD1456ep + 1./27.*
         m13*RD1456ep*RS34561pe + 1./27.*m13*RD1456ep*RS14563pe + 1./27.
         *m13*RD1456ep*RSe45613p + 1./27.*m13*RD1456ep*RSp45613e - 1./
         54.*m13*RD1456ep*RC3456pe - 1./54.*m13*RD1456ep*RSe45633p - 1./
         54.*m13*RD1456ep*RSp45633e - 1./54.*m13*RD1456ep*RC1456pe - 1./
         54.*m13*RSe45611p*RD1456ep - 1./54.*m13*RSp45611e*RD1456ep - 1.
         /162.*m11*RD1456ep*RS34561pe - 1./81.*m11*RD1456ep*RS14563pe
          - 1./162.*m11*RD1456ep*RSe45613p );

      M13p6x23RV +=  + Ab(m13,mu2)*BbV(m11,L) * (  - 1./162.*m11*
         RD1456ep*RSp45613e + 5./324.*m11*RD1456ep*RC3456pe + 5./324.*
         m11*RD1456ep*RSe45633p + 5./324.*m11*RD1456ep*RSp45633e - 1./
         162.*m11*RD1456ep*RC1456pe - 1./108.*m11*RSe45611p*RD1456ep - 
         1./108.*m11*RSp45611e*RD1456ep );

      M13p6x23RV +=  + Ab(m13,mu2)*BbV(m33,L) * ( 1./324.*pow(
         RD3456ep,2) + 1./324.*RD1456ep*RD3456ep - 1./162.*me2*RD3456ep
         *RSe45613p + 1./324.*me2*RSe45633p*RD3456ep + 1./324.*me2*
         RSe45611p*RD3456ep - 1./162.*mp2*RD3456ep*RSp45613e + 1./324.*
         mp2*RSp45633e*RD3456ep + 1./324.*mp2*RSp45611e*RD3456ep - 1./
         81.*m33*RD3456ep*RS34561pe - 1./162.*m33*RD3456ep*RS14563pe - 
         1./162.*m33*RD3456ep*RSe45613p - 1./162.*m33*RD3456ep*
         RSp45613e - 1./162.*m33*RD3456ep*RC3456pe - 1./108.*m33*
         RSe45633p*RD3456ep - 1./108.*m33*RSp45633e*RD3456ep + 5./324.*
         m33*RC1456pe*RD3456ep + 5./324.*m33*RSe45611p*RD3456ep + 5./
         324.*m33*RSp45611e*RD3456ep + 1./27.*m13*RD3456ep*RS34561pe + 
         1./27.*m13*RD3456ep*RS14563pe + 1./27.*m13*RD3456ep*RSe45613p
          + 1./27.*m13*RD3456ep*RSp45613e - 1./54.*m13*RD3456ep*
         RC3456pe - 1./54.*m13*RSe45633p*RD3456ep - 1./54.*m13*
         RSp45633e*RD3456ep - 1./54.*m13*RC1456pe*RD3456ep - 1./54.*m13
         *RSe45611p*RD3456ep );

      M13p6x23RV +=  + Ab(m13,mu2)*BbV(m33,L) * (  - 1./54.*m13*
         RSp45611e*RD3456ep - 1./162.*m11*RD3456ep*RS14563pe - 1./162.*
         m11*RD3456ep*RC3456pe - 1./162.*m11*RSe45633p*RD3456ep - 1./
         162.*m11*RSp45633e*RD3456ep + 1./108.*m11*RC1456pe*RD3456ep + 
         1./162.*m11*RSe45611p*RD3456ep + 1./162.*m11*RSp45611e*
         RD3456ep );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(m11,L) * (  - 29./540.*RD1456ep
          - 7./540.*m44*RS14563pe + 7./540.*m44*RC1456pe - 1./60.*m44*
         RS156pe + 1./27.*m34*RS14563pe - 1./108.*m34*RC1456pe - 1./54.
         *m34*RS156pe + 1./90.*m33*RS14563pe + 1./120.*m33*RC1456pe - 1.
         /20.*m33*RS156pe - 1./90.*m14*RS14563pe - 1./60.*m14*RC1456pe
          + 2./45.*m14*RS156pe - 11./540.*m13*RS14563pe - 1./135.*m13*
         RC1456pe + 19./540.*m13*RS156pe + 1./27.*m11*RS34561pe + 43./
         540.*m11*RS14563pe + 1./27.*m11*RSe45613p + 1./27.*m11*
         RSp45613e - 1./27.*m11*RS356pe - 1./27.*m11*RSe563p - 1./27.*
         m11*RSp563e + 13./1080.*m11*RC1456pe - 1./27.*m11*RSe45611p - 
         1./27.*m11*RSp45611e - 1./45.*m11*RS156pe + 1./27.*m11*RSe561p
          + 1./27.*m11*RSp561e + 1./54.*RS(m11,m44,m55,m66,m33,me2,mp2)
         *m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11 );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(m14,L) * (  - 11./180.*m44 - 1./
         60.*m33 + 11./90.*m14 + 1./30.*m13 - 7./90.*m11 );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(m15,L) * (  - 11./360.*m55 - 11./
         360.*m44 - 1./60.*m33 + 11./180.*m15 + 11./180.*m14 + 1./30.*
         m13 - 7./90.*m11 );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(m16,L) * (  - 11./360.*m66 - 11./
         360.*m44 - 1./60.*m33 + 11./180.*m16 + 11./180.*m14 + 1./30.*
         m13 - 7./90.*m11 );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(m33,L) * (  - 1./180.*RD3456ep
          - 7./540.*m44*RS34561pe + 1./60.*m44*RC3456pe + 1./180.*m44*
         RS356pe + 1./27.*m34*RS34561pe - 1./27.*m34*RC3456pe + 1./27.*
         m34*RS356pe + 2./27.*m33*RS34561pe + 19./540.*m33*RC3456pe - 5.
         /54.*m33*RS356pe - 1./90.*m14*RS34561pe + 1./270.*m14*RC3456pe
          + 1./270.*m14*RS356pe - 11./540.*m13*RS34561pe + 1./20.*m13*
         RC3456pe - 43./540.*m13*RS356pe - 11./540.*m11*RS34561pe + 2./
         135.*m11*RC3456pe - 17./540.*m11*RS356pe + 1./27.*RS(m33,m44,
         m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11 );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(m34,L) * (  - 1./108.*RD3456ep
          - 1./108.*RD1456ep - 1./54.*me2*RSe45613p - 1./108.*me2*
         RSe45633p + 1./27.*me2*RSe563p - 1./108.*me2*RSe45611p + 1./27.
         *me2*RSe561p - 1./27.*me2*RSe564p - 1./54.*mp2*RSp45613e - 1./
         108.*mp2*RSp45633e + 1./27.*mp2*RSp563e - 1./108.*mp2*
         RSp45611e + 1./27.*mp2*RSp561e - 1./27.*mp2*RSp564e + 1./90.*
         m44 - 1./54.*m44*RS34561pe - 1./54.*m44*RS14563pe - 1./54.*m44
         *RSe45613p - 1./54.*m44*RSp45613e - 1./108.*m44*RC3456pe - 1./
         108.*m44*RSe45633p - 1./108.*m44*RSp45633e - 1./108.*m44*
         RC1456pe - 1./108.*m44*RSe45611p - 1./108.*m44*RSp45611e + 1./
         27.*m44*RSe564p + 1./27.*m44*RSp564e - 1./90.*m34 + 1./36.*m34
         *RC3456pe + 1./36.*m34*RSe45633p + 1./36.*m34*RSp45633e - 1./
         18.*m34*RS356pe - 1./18.*m34*RSe563p - 1./18.*m34*RSp563e - 1./
         36.*m34*RC1456pe - 1./36.*m34*RSe45611p - 1./36.*m34*RSp45611e
          + 1./18.*m34*RS156pe + 1./18.*m34*RSe561p + 1./18.*m34*
         RSp561e );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(m34,L) * (  - 2./45.*m33 - 5./
         108.*m33*RS34561pe - 1./36.*m33*RS14563pe - 1./36.*m33*
         RSe45613p - 1./36.*m33*RSp45613e - 1./36.*m33*RC3456pe - 1./54.
         *m33*RSe45633p - 1./54.*m33*RSp45633e + 7./108.*m33*RS356pe + 
         1./36.*m33*RSe563p + 1./36.*m33*RSp563e - 1./108.*m33*RC1456pe
          - 1./108.*m33*RSe45611p - 1./108.*m33*RSp45611e + 1./108.*m33
         *RS156pe + 1./54.*m33*RS456pe + 1./108.*m33*RSe561p + 1./108.*
         m33*RSp561e + 1./54.*m33*RSe564p + 1./54.*m33*RSp564e - 1./90.
         *m14 - 1./36.*m14*RC3456pe - 1./36.*m14*RSe45633p - 1./36.*m14
         *RSp45633e + 1./18.*m14*RS356pe + 1./18.*m14*RSe563p + 1./18.*
         m14*RSp563e + 1./36.*m14*RC1456pe + 1./36.*m14*RSe45611p + 1./
         36.*m14*RSp45611e - 1./18.*m14*RS156pe - 1./18.*m14*RSe561p - 
         1./18.*m14*RSp561e - 3./20.*m13 - 1./36.*m13*RS34561pe - 1./36.
         *m13*RS14563pe - 1./36.*m13*RSe45613p - 1./36.*m13*RSp45613e
          + 1./36.*m13*RC3456pe + 1./36.*m13*RSe45633p + 1./36.*m13*
         RSp45633e );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(m34,L) * (  - 1./36.*m13*RS356pe
          - 1./36.*m13*RSe563p - 1./36.*m13*RSp563e + 1./36.*m13*
         RC1456pe + 1./36.*m13*RSe45611p + 1./36.*m13*RSp45611e - 1./36.
         *m13*RS156pe + 1./36.*m13*RS456pe - 1./36.*m13*RSe561p - 1./36.
         *m13*RSp561e + 1./36.*m13*RSe564p + 1./36.*m13*RSp564e - 2./45.
         *m11 - 1./36.*m11*RS34561pe - 5./108.*m11*RS14563pe - 1./36.*
         m11*RSe45613p - 1./36.*m11*RSp45613e - 1./108.*m11*RC3456pe - 
         1./108.*m11*RSe45633p - 1./108.*m11*RSp45633e + 1./108.*m11*
         RS356pe + 1./108.*m11*RSe563p + 1./108.*m11*RSp563e - 1./36.*
         m11*RC1456pe - 1./54.*m11*RSe45611p - 1./54.*m11*RSp45611e + 7.
         /108.*m11*RS156pe + 1./54.*m11*RS456pe + 1./36.*m11*RSe561p + 
         1./36.*m11*RSp561e + 1./54.*m11*RSe564p + 1./54.*m11*RSp564e )
         ;

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(m35,L) * (  - 1./20.*m55 + 1./9.
         *m45 - 1./20.*m44 - 1./90.*m35 - 2./45.*m33 - 1./90.*m14 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(m36,L) * (  - 1./20.*m66 + 1./9.
         *m46 - 1./20.*m44 - 1./90.*m36 - 2./45.*m33 - 1./90.*m14 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(m44,L) * (  - 1./20.*m44 + 1./60.
         *m44*RS456pe + 1./36.*m34 - 1./108.*m34*RS456pe + 7./360.*m33
          - 7./1080.*m33*RS456pe + 13./180.*m14 - 13./540.*m14*RS456pe
          - 1./15.*m13 + 1./45.*m13*RS456pe - 1./360.*m11 + 1./1080.*
         m11*RS456pe );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(m45,L) * ( 7./360.*m55 - 1./15.*
         m45 - 1./360.*m44 + 1./36.*m35 + 7./360.*m33 + 13./180.*m14 - 
         1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(m46,L) * ( 7./360.*m66 - 1./15.*
         m46 - 1./360.*m44 + 1./36.*m36 + 7./360.*m33 + 13./180.*m14 - 
         1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(mp2,L) * ( 11./135.*mp2*
         RSp45613e - 1./180.*mp2*RSp45633e - 19./270.*mp2*RSp563e - 29./
         540.*mp2*RSp45611e + 7./270.*mp2*RSp561e + 1./45.*mp2*RSp564e
          - 7./540.*m44*RSp45613e + 1./60.*m44*RSp45633e + 1./180.*m44*
         RSp563e + 7./540.*m44*RSp45611e - 1./60.*m44*RSp561e - 1./180.
         *m44*RSp564e + 1./27.*m34*RSp45613e - 1./27.*m34*RSp45633e + 1.
         /27.*m34*RSp563e - 1./108.*m34*RSp45611e - 1./54.*m34*RSp561e
          - 1./108.*m34*RSp564e + 4./135.*m33*RSp45613e + 11./270.*m33*
         RSp45633e - 1./45.*m33*RSp563e + 1./120.*m33*RSp45611e - 1./20.
         *m33*RSp561e - 7./1080.*m33*RSp564e - 1./90.*m14*RSp45613e + 1.
         /270.*m14*RSp45633e + 1./270.*m14*RSp563e - 1./60.*m14*
         RSp45611e + 2./45.*m14*RSp561e - 13./540.*m14*RSp564e - 11./
         540.*m13*RSp45613e + 1./20.*m13*RSp45633e - 43./540.*m13*
         RSp563e - 1./135.*m13*RSp45611e + 19./540.*m13*RSp561e + 1./45.
         *m13*RSp564e - 1./540.*m11*RSp45613e + 2./135.*m11*RSp45633e
          - 17./540.*m11*RSp563e );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(mp2,L) * ( 37./360.*m11*
         RSp45611e - 23./270.*m11*RSp561e + 1./1080.*m11*RSp564e );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(me2,L) * ( 11./135.*me2*
         RSe45613p - 1./180.*me2*RSe45633p - 19./270.*me2*RSe563p - 29./
         540.*me2*RSe45611p + 7./270.*me2*RSe561p + 1./45.*me2*RSe564p
          - 7./540.*m44*RSe45613p + 1./60.*m44*RSe45633p + 1./180.*m44*
         RSe563p + 7./540.*m44*RSe45611p - 1./60.*m44*RSe561p - 1./180.
         *m44*RSe564p + 1./27.*m34*RSe45613p - 1./27.*m34*RSe45633p + 1.
         /27.*m34*RSe563p - 1./108.*m34*RSe45611p - 1./54.*m34*RSe561p
          - 1./108.*m34*RSe564p + 4./135.*m33*RSe45613p + 11./270.*m33*
         RSe45633p - 1./45.*m33*RSe563p + 1./120.*m33*RSe45611p - 1./20.
         *m33*RSe561p - 7./1080.*m33*RSe564p - 1./90.*m14*RSe45613p + 1.
         /270.*m14*RSe45633p + 1./270.*m14*RSe563p - 1./60.*m14*
         RSe45611p + 2./45.*m14*RSe561p - 13./540.*m14*RSe564p - 11./
         540.*m13*RSe45613p + 1./20.*m13*RSe45633p - 43./540.*m13*
         RSe563p - 1./135.*m13*RSe45611p + 19./540.*m13*RSe561p + 1./45.
         *m13*RSe564p - 1./540.*m11*RSe45613p + 2./135.*m11*RSe45633p
          - 17./540.*m11*RSe563p );

      M13p6x23RV +=  + Ab(m14,mu2)*AbV(me2,L) * ( 37./360.*m11*
         RSe45611p - 23./270.*m11*RSe561p + 1./1080.*m11*RSe564p );

      M13p6x23RV +=  + Ab(m14,mu2)*BbV(m11,m33,L) * (  - 1./108.*m33*
         m44*RS34561pe - 1./108.*m33*m44*RS356pe + 1./54.*m14*m33*
         RS34561pe - 1./54.*m14*m33*RS356pe - 1./54.*m13*m44*RS34561pe
          - 1./54.*m13*m44*RS356pe + 1./27.*m13*m14*RS34561pe - 1./27.*
         m13*m14*RS356pe - 1./36.*m11*m44*RS34561pe - 1./36.*m11*m44*
         RS356pe + 1./36.*m11*m33*RS34561pe - 7./108.*m11*m33*RS356pe
          + 1./18.*m11*m14*RS34561pe - 1./18.*m11*m14*RS356pe - 1./54.*
         m11*m13*RS34561pe - 1./18.*m11*m13*RS356pe - 7./108.*pow(
         m11,2)*RS34561pe - 5./108.*pow(m11,2)*RS356pe );

      M13p6x23RV +=  + Ab(m14,mu2)*BbV(m11,mp2,L) * (  - 1./108.*m33*
         m44*RSp45613e - 1./108.*m33*m44*RSp563e - 1./108.*m33*m44*
         RSp45611e - 1./108.*m33*m44*RSp561e + 1./54.*m14*m33*RSp45613e
          - 1./54.*m14*m33*RSp563e + 1./54.*m14*m33*RSp45611e - 1./54.*
         m14*m33*RSp561e - 1./54.*m13*m44*RSp45613e - 1./54.*m13*m44*
         RSp563e + 1./54.*m13*m44*RSp45611e + 1./54.*m13*m44*RSp561e + 
         1./27.*m13*m14*RSp45613e - 1./27.*m13*m14*RSp563e - 1./27.*m13
         *m14*RSp45611e + 1./27.*m13*m14*RSp561e + 1./27.*m11*mp2*
         RSp45613e - 1./27.*m11*mp2*RSp563e - 1./27.*m11*mp2*RSp45611e
          + 1./27.*m11*mp2*RSp561e - 1./36.*m11*m44*RSp45613e - 1./36.*
         m11*m44*RSp563e - 1./108.*m11*m44*RSp45611e - 1./108.*m11*m44*
         RSp561e - 1./108.*m11*m33*RSp45613e - 1./36.*m11*m33*RSp563e
          - 1./108.*m11*m33*RSp45611e - 1./36.*m11*m33*RSp561e + 1./18.
         *m11*m14*RSp45613e - 1./18.*m11*m14*RSp563e + 1./54.*m11*m14*
         RSp45611e - 1./54.*m11*m14*RSp561e - 1./54.*m11*m13*RSp45613e
          - 1./18.*m11*m13*RSp563e );

      M13p6x23RV +=  + Ab(m14,mu2)*BbV(m11,mp2,L) * ( 1./54.*m11*m13*
         RSp45611e + 1./18.*m11*m13*RSp561e - 7./108.*pow(m11,2)*
         RSp45613e - 5./108.*pow(m11,2)*RSp563e + 1./36.*pow(m11,2)*
         RSp45611e - 7./108.*pow(m11,2)*RSp561e );

      M13p6x23RV +=  + Ab(m14,mu2)*BbV(m11,me2,L) * (  - 1./108.*m33*
         m44*RSe45613p - 1./108.*m33*m44*RSe563p - 1./108.*m33*m44*
         RSe45611p - 1./108.*m33*m44*RSe561p + 1./54.*m14*m33*RSe45613p
          - 1./54.*m14*m33*RSe563p + 1./54.*m14*m33*RSe45611p - 1./54.*
         m14*m33*RSe561p - 1./54.*m13*m44*RSe45613p - 1./54.*m13*m44*
         RSe563p + 1./54.*m13*m44*RSe45611p + 1./54.*m13*m44*RSe561p + 
         1./27.*m13*m14*RSe45613p - 1./27.*m13*m14*RSe563p - 1./27.*m13
         *m14*RSe45611p + 1./27.*m13*m14*RSe561p + 1./27.*m11*me2*
         RSe45613p - 1./27.*m11*me2*RSe563p - 1./27.*m11*me2*RSe45611p
          + 1./27.*m11*me2*RSe561p - 1./36.*m11*m44*RSe45613p - 1./36.*
         m11*m44*RSe563p - 1./108.*m11*m44*RSe45611p - 1./108.*m11*m44*
         RSe561p - 1./108.*m11*m33*RSe45613p - 1./36.*m11*m33*RSe563p
          - 1./108.*m11*m33*RSe45611p - 1./36.*m11*m33*RSe561p + 1./18.
         *m11*m14*RSe45613p - 1./18.*m11*m14*RSe563p + 1./54.*m11*m14*
         RSe45611p - 1./54.*m11*m14*RSe561p - 1./54.*m11*m13*RSe45613p
          - 1./18.*m11*m13*RSe563p );

      M13p6x23RV +=  + Ab(m14,mu2)*BbV(m11,me2,L) * ( 1./54.*m11*m13*
         RSe45611p + 1./18.*m11*m13*RSe561p - 7./108.*pow(m11,2)*
         RSe45613p - 5./108.*pow(m11,2)*RSe563p + 1./36.*pow(m11,2)*
         RSe45611p - 7./108.*pow(m11,2)*RSe561p );

      M13p6x23RV +=  + Ab(m14,mu2)*BbV(m11,L) * ( 7./540.*m44*RD1456ep
          - 1./108.*m34*RD1456ep + 1./120.*m33*RD1456ep - 1./108.*m33*
         m44*RS14563pe - 1./108.*m33*m44*RC1456pe - 1./108.*m33*m44*
         RS156pe - 1./60.*m14*RD1456ep + 1./54.*m14*m33*RS14563pe + 1./
         54.*m14*m33*RC1456pe - 1./54.*m14*m33*RS156pe - 1./135.*m13*
         RD1456ep - 1./54.*m13*m44*RS14563pe + 1./54.*m13*m44*RC1456pe
          + 1./54.*m13*m44*RS156pe + 1./27.*m13*m14*RS14563pe - 1./27.*
         m13*m14*RC1456pe + 1./27.*m13*m14*RS156pe + 13./1080.*m11*
         RD1456ep - 1./36.*m11*m44*RS14563pe - 1./108.*m11*m44*RC1456pe
          - 1./108.*m11*m44*RS156pe - 1./108.*m11*m33*RS14563pe - 1./
         108.*m11*m33*RC1456pe - 1./36.*m11*m33*RS156pe + 1./18.*m11*
         m14*RS14563pe + 1./54.*m11*m14*RC1456pe - 1./54.*m11*m14*
         RS156pe - 1./54.*m11*m13*RS14563pe + 1./54.*m11*m13*RC1456pe
          + 1./18.*m11*m13*RS156pe - 1./36.*pow(m11,2)*RS14563pe - 1./
         108.*pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS156pe );

      M13p6x23RV +=  + Ab(m14,mu2)*BbV(m14,L) * ( 1./72.*pow(m44,2) + 1.
         /72.*m33*m44 - 1./18.*m14*m44 - 1./36.*m14*m33 + 1./18.*pow(
         m14,2) - 1./36.*m13*m44 + 1./18.*m13*m14 + 1./24.*m11*m44 + 1./
         72.*m11*m33 - 1./12.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(
         m11,2) );

      M13p6x23RV +=  + Ab(m14,mu2)*BbV(m15,L) * ( 1./144.*pow(m55,2) + 
         1./144.*m44*m55 + 1./144.*m33*m55 + 1./144.*m33*m44 - 1./36.*
         m15*m55 - 1./72.*m15*m44 - 1./72.*m15*m33 + 1./36.*pow(m15,2)
          - 1./72.*m14*m55 - 1./72.*m14*m33 + 1./36.*m14*m15 - 1./72.*
         m13*m55 - 1./72.*m13*m44 + 1./36.*m13*m15 + 1./36.*m13*m14 + 1.
         /36.*m11*m55 + 1./72.*m11*m44 + 1./72.*m11*m33 - 1./18.*m11*
         m15 - 1./36.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + Ab(m14,mu2)*BbV(m16,L) * ( 1./144.*pow(m66,2) + 
         1./144.*m44*m66 + 1./144.*m33*m66 + 1./144.*m33*m44 - 1./36.*
         m16*m66 - 1./72.*m16*m44 - 1./72.*m16*m33 + 1./36.*pow(m16,2)
          - 1./72.*m14*m66 - 1./72.*m14*m33 + 1./36.*m14*m16 - 1./72.*
         m13*m66 - 1./72.*m13*m44 + 1./36.*m13*m16 + 1./36.*m13*m14 + 1.
         /36.*m11*m66 + 1./72.*m11*m44 + 1./72.*m11*m33 - 1./18.*m11*
         m16 - 1./36.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + Ab(m14,mu2)*BbV(m33,L) * ( 1./60.*m44*RD3456ep
          - 1./27.*m34*RD3456ep + 19./540.*m33*RD3456ep + 1./270.*m14*
         RD3456ep + 1./20.*m13*RD3456ep + 2./135.*m11*RD3456ep );

      M13p6x23RV +=  + Ab(m14,mu2)*CbV(m11,L) * (  - 1./108.*m33*m44*
         RD1456ep + 1./54.*m14*m33*RD1456ep + 1./54.*m13*m44*RD1456ep
          - 1./27.*m13*m14*RD1456ep - 1./108.*m11*m44*RD1456ep - 1./108.
         *m11*m33*RD1456ep + 1./54.*m11*m14*RD1456ep + 1./54.*m11*m13*
         RD1456ep - 1./108.*pow(m11,2)*RD1456ep );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(m11,L) * (  - 29./540.*RD1456ep
          - 7./540.*m55*RS14563pe + 7./540.*m55*RC1456pe - 1./60.*m55*
         RS146pe + 1./27.*m35*RS14563pe - 1./108.*m35*RC1456pe - 1./54.
         *m35*RS146pe + 1./90.*m33*RS14563pe + 1./120.*m33*RC1456pe - 1.
         /20.*m33*RS146pe - 1./90.*m15*RS14563pe - 1./60.*m15*RC1456pe
          + 2./45.*m15*RS146pe - 11./540.*m13*RS14563pe - 1./135.*m13*
         RC1456pe + 19./540.*m13*RS146pe + 1./27.*m11*RS34561pe + 43./
         540.*m11*RS14563pe + 1./27.*m11*RSe45613p + 1./27.*m11*
         RSp45613e - 1./27.*m11*RS346pe - 1./27.*m11*RSe463p - 1./27.*
         m11*RSp463e + 13./1080.*m11*RC1456pe - 1./27.*m11*RSe45611p - 
         1./27.*m11*RSp45611e - 1./45.*m11*RS146pe + 1./27.*m11*RSe461p
          + 1./27.*m11*RSp461e + 1./54.*RS(m11,m44,m55,m66,m33,me2,mp2)
         *m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11 );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(m14,L) * (  - 11./360.*m55 - 11./
         360.*m44 - 1./60.*m33 + 11./180.*m15 + 11./180.*m14 + 1./30.*
         m13 - 7./90.*m11 );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(m15,L) * (  - 11./180.*m55 - 1./
         60.*m33 + 11./90.*m15 + 1./30.*m13 - 7./90.*m11 );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(m16,L) * (  - 11./360.*m66 - 11./
         360.*m55 - 1./60.*m33 + 11./180.*m16 + 11./180.*m15 + 1./30.*
         m13 - 7./90.*m11 );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(m33,L) * (  - 1./180.*RD3456ep
          - 7./540.*m55*RS34561pe + 1./60.*m55*RC3456pe + 1./180.*m55*
         RS346pe + 1./27.*m35*RS34561pe - 1./27.*m35*RC3456pe + 1./27.*
         m35*RS346pe + 2./27.*m33*RS34561pe + 19./540.*m33*RC3456pe - 5.
         /54.*m33*RS346pe - 1./90.*m15*RS34561pe + 1./270.*m15*RC3456pe
          + 1./270.*m15*RS346pe - 11./540.*m13*RS34561pe + 1./20.*m13*
         RC3456pe - 43./540.*m13*RS346pe - 11./540.*m11*RS34561pe + 2./
         135.*m11*RC3456pe - 17./540.*m11*RS346pe + 1./27.*RS(m33,m44,
         m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11 );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(m34,L) * (  - 1./20.*m55 + 1./9.
         *m45 - 1./20.*m44 - 1./90.*m34 - 2./45.*m33 - 1./90.*m15 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(m35,L) * (  - 1./108.*RD3456ep
          - 1./108.*RD1456ep - 1./54.*me2*RSe45613p - 1./108.*me2*
         RSe45633p + 1./27.*me2*RSe463p - 1./108.*me2*RSe45611p + 1./27.
         *me2*RSe461p - 1./27.*me2*RSe465p - 1./54.*mp2*RSp45613e - 1./
         108.*mp2*RSp45633e + 1./27.*mp2*RSp463e - 1./108.*mp2*
         RSp45611e + 1./27.*mp2*RSp461e - 1./27.*mp2*RSp465e + 1./90.*
         m55 - 1./54.*m55*RS34561pe - 1./54.*m55*RS14563pe - 1./54.*m55
         *RSe45613p - 1./54.*m55*RSp45613e - 1./108.*m55*RC3456pe - 1./
         108.*m55*RSe45633p - 1./108.*m55*RSp45633e - 1./108.*m55*
         RC1456pe - 1./108.*m55*RSe45611p - 1./108.*m55*RSp45611e + 1./
         27.*m55*RSe465p + 1./27.*m55*RSp465e - 1./90.*m35 + 1./36.*m35
         *RC3456pe + 1./36.*m35*RSe45633p + 1./36.*m35*RSp45633e - 1./
         18.*m35*RS346pe - 1./18.*m35*RSe463p - 1./18.*m35*RSp463e - 1./
         36.*m35*RC1456pe - 1./36.*m35*RSe45611p - 1./36.*m35*RSp45611e
          + 1./18.*m35*RS146pe + 1./18.*m35*RSe461p + 1./18.*m35*
         RSp461e );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(m35,L) * (  - 2./45.*m33 - 5./
         108.*m33*RS34561pe - 1./36.*m33*RS14563pe - 1./36.*m33*
         RSe45613p - 1./36.*m33*RSp45613e - 1./36.*m33*RC3456pe - 1./54.
         *m33*RSe45633p - 1./54.*m33*RSp45633e + 7./108.*m33*RS346pe + 
         1./36.*m33*RSe463p + 1./36.*m33*RSp463e - 1./108.*m33*RC1456pe
          - 1./108.*m33*RSe45611p - 1./108.*m33*RSp45611e + 1./108.*m33
         *RS146pe + 1./54.*m33*RS546pe + 1./108.*m33*RSe461p + 1./108.*
         m33*RSp461e + 1./54.*m33*RSe465p + 1./54.*m33*RSp465e - 1./90.
         *m15 - 1./36.*m15*RC3456pe - 1./36.*m15*RSe45633p - 1./36.*m15
         *RSp45633e + 1./18.*m15*RS346pe + 1./18.*m15*RSe463p + 1./18.*
         m15*RSp463e + 1./36.*m15*RC1456pe + 1./36.*m15*RSe45611p + 1./
         36.*m15*RSp45611e - 1./18.*m15*RS146pe - 1./18.*m15*RSe461p - 
         1./18.*m15*RSp461e - 3./20.*m13 - 1./36.*m13*RS34561pe - 1./36.
         *m13*RS14563pe - 1./36.*m13*RSe45613p - 1./36.*m13*RSp45613e
          + 1./36.*m13*RC3456pe + 1./36.*m13*RSe45633p + 1./36.*m13*
         RSp45633e );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(m35,L) * (  - 1./36.*m13*RS346pe
          - 1./36.*m13*RSe463p - 1./36.*m13*RSp463e + 1./36.*m13*
         RC1456pe + 1./36.*m13*RSe45611p + 1./36.*m13*RSp45611e - 1./36.
         *m13*RS146pe + 1./36.*m13*RS546pe - 1./36.*m13*RSe461p - 1./36.
         *m13*RSp461e + 1./36.*m13*RSe465p + 1./36.*m13*RSp465e - 2./45.
         *m11 - 1./36.*m11*RS34561pe - 5./108.*m11*RS14563pe - 1./36.*
         m11*RSe45613p - 1./36.*m11*RSp45613e - 1./108.*m11*RC3456pe - 
         1./108.*m11*RSe45633p - 1./108.*m11*RSp45633e + 1./108.*m11*
         RS346pe + 1./108.*m11*RSe463p + 1./108.*m11*RSp463e - 1./36.*
         m11*RC1456pe - 1./54.*m11*RSe45611p - 1./54.*m11*RSp45611e + 7.
         /108.*m11*RS146pe + 1./54.*m11*RS546pe + 1./36.*m11*RSe461p + 
         1./36.*m11*RSp461e + 1./54.*m11*RSe465p + 1./54.*m11*RSp465e )
         ;

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(m36,L) * (  - 1./20.*m66 + 1./9.
         *m56 - 1./20.*m55 - 1./90.*m36 - 2./45.*m33 - 1./90.*m15 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(m45,L) * (  - 1./360.*m55 - 1./
         15.*m45 + 7./360.*m44 + 1./36.*m34 + 7./360.*m33 + 13./180.*
         m15 - 1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(m55,L) * (  - 1./20.*m55 + 1./60.
         *m55*RS546pe + 1./36.*m35 - 1./108.*m35*RS546pe + 7./360.*m33
          - 7./1080.*m33*RS546pe + 13./180.*m15 - 13./540.*m15*RS546pe
          - 1./15.*m13 + 1./45.*m13*RS546pe - 1./360.*m11 + 1./1080.*
         m11*RS546pe );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(m56,L) * ( 7./360.*m66 - 1./15.*
         m56 - 1./360.*m55 + 1./36.*m36 + 7./360.*m33 + 13./180.*m15 - 
         1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(mp2,L) * ( 11./135.*mp2*
         RSp45613e - 1./180.*mp2*RSp45633e - 19./270.*mp2*RSp463e - 29./
         540.*mp2*RSp45611e + 7./270.*mp2*RSp461e + 1./45.*mp2*RSp465e
          - 7./540.*m55*RSp45613e + 1./60.*m55*RSp45633e + 1./180.*m55*
         RSp463e + 7./540.*m55*RSp45611e - 1./60.*m55*RSp461e - 1./180.
         *m55*RSp465e + 1./27.*m35*RSp45613e - 1./27.*m35*RSp45633e + 1.
         /27.*m35*RSp463e - 1./108.*m35*RSp45611e - 1./54.*m35*RSp461e
          - 1./108.*m35*RSp465e + 4./135.*m33*RSp45613e + 11./270.*m33*
         RSp45633e - 1./45.*m33*RSp463e + 1./120.*m33*RSp45611e - 1./20.
         *m33*RSp461e - 7./1080.*m33*RSp465e - 1./90.*m15*RSp45613e + 1.
         /270.*m15*RSp45633e + 1./270.*m15*RSp463e - 1./60.*m15*
         RSp45611e + 2./45.*m15*RSp461e - 13./540.*m15*RSp465e - 11./
         540.*m13*RSp45613e + 1./20.*m13*RSp45633e - 43./540.*m13*
         RSp463e - 1./135.*m13*RSp45611e + 19./540.*m13*RSp461e + 1./45.
         *m13*RSp465e - 1./540.*m11*RSp45613e + 2./135.*m11*RSp45633e
          - 17./540.*m11*RSp463e );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(mp2,L) * ( 37./360.*m11*
         RSp45611e - 23./270.*m11*RSp461e + 1./1080.*m11*RSp465e );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(me2,L) * ( 11./135.*me2*
         RSe45613p - 1./180.*me2*RSe45633p - 19./270.*me2*RSe463p - 29./
         540.*me2*RSe45611p + 7./270.*me2*RSe461p + 1./45.*me2*RSe465p
          - 7./540.*m55*RSe45613p + 1./60.*m55*RSe45633p + 1./180.*m55*
         RSe463p + 7./540.*m55*RSe45611p - 1./60.*m55*RSe461p - 1./180.
         *m55*RSe465p + 1./27.*m35*RSe45613p - 1./27.*m35*RSe45633p + 1.
         /27.*m35*RSe463p - 1./108.*m35*RSe45611p - 1./54.*m35*RSe461p
          - 1./108.*m35*RSe465p + 4./135.*m33*RSe45613p + 11./270.*m33*
         RSe45633p - 1./45.*m33*RSe463p + 1./120.*m33*RSe45611p - 1./20.
         *m33*RSe461p - 7./1080.*m33*RSe465p - 1./90.*m15*RSe45613p + 1.
         /270.*m15*RSe45633p + 1./270.*m15*RSe463p - 1./60.*m15*
         RSe45611p + 2./45.*m15*RSe461p - 13./540.*m15*RSe465p - 11./
         540.*m13*RSe45613p + 1./20.*m13*RSe45633p - 43./540.*m13*
         RSe463p - 1./135.*m13*RSe45611p + 19./540.*m13*RSe461p + 1./45.
         *m13*RSe465p - 1./540.*m11*RSe45613p + 2./135.*m11*RSe45633p
          - 17./540.*m11*RSe463p );

      M13p6x23RV +=  + Ab(m15,mu2)*AbV(me2,L) * ( 37./360.*m11*
         RSe45611p - 23./270.*m11*RSe461p + 1./1080.*m11*RSe465p );

      M13p6x23RV +=  + Ab(m15,mu2)*BbV(m11,m33,L) * (  - 1./108.*m33*
         m55*RS34561pe - 1./108.*m33*m55*RS346pe + 1./54.*m15*m33*
         RS34561pe - 1./54.*m15*m33*RS346pe - 1./54.*m13*m55*RS34561pe
          - 1./54.*m13*m55*RS346pe + 1./27.*m13*m15*RS34561pe - 1./27.*
         m13*m15*RS346pe - 1./36.*m11*m55*RS34561pe - 1./36.*m11*m55*
         RS346pe + 1./36.*m11*m33*RS34561pe - 7./108.*m11*m33*RS346pe
          + 1./18.*m11*m15*RS34561pe - 1./18.*m11*m15*RS346pe - 1./54.*
         m11*m13*RS34561pe - 1./18.*m11*m13*RS346pe - 7./108.*pow(
         m11,2)*RS34561pe - 5./108.*pow(m11,2)*RS346pe );

      M13p6x23RV +=  + Ab(m15,mu2)*BbV(m11,mp2,L) * (  - 1./108.*m33*
         m55*RSp45613e - 1./108.*m33*m55*RSp463e - 1./108.*m33*m55*
         RSp45611e - 1./108.*m33*m55*RSp461e + 1./54.*m15*m33*RSp45613e
          - 1./54.*m15*m33*RSp463e + 1./54.*m15*m33*RSp45611e - 1./54.*
         m15*m33*RSp461e - 1./54.*m13*m55*RSp45613e - 1./54.*m13*m55*
         RSp463e + 1./54.*m13*m55*RSp45611e + 1./54.*m13*m55*RSp461e + 
         1./27.*m13*m15*RSp45613e - 1./27.*m13*m15*RSp463e - 1./27.*m13
         *m15*RSp45611e + 1./27.*m13*m15*RSp461e + 1./27.*m11*mp2*
         RSp45613e - 1./27.*m11*mp2*RSp463e - 1./27.*m11*mp2*RSp45611e
          + 1./27.*m11*mp2*RSp461e - 1./36.*m11*m55*RSp45613e - 1./36.*
         m11*m55*RSp463e - 1./108.*m11*m55*RSp45611e - 1./108.*m11*m55*
         RSp461e - 1./108.*m11*m33*RSp45613e - 1./36.*m11*m33*RSp463e
          - 1./108.*m11*m33*RSp45611e - 1./36.*m11*m33*RSp461e + 1./18.
         *m11*m15*RSp45613e - 1./18.*m11*m15*RSp463e + 1./54.*m11*m15*
         RSp45611e - 1./54.*m11*m15*RSp461e - 1./54.*m11*m13*RSp45613e
          - 1./18.*m11*m13*RSp463e );

      M13p6x23RV +=  + Ab(m15,mu2)*BbV(m11,mp2,L) * ( 1./54.*m11*m13*
         RSp45611e + 1./18.*m11*m13*RSp461e - 7./108.*pow(m11,2)*
         RSp45613e - 5./108.*pow(m11,2)*RSp463e + 1./36.*pow(m11,2)*
         RSp45611e - 7./108.*pow(m11,2)*RSp461e );

      M13p6x23RV +=  + Ab(m15,mu2)*BbV(m11,me2,L) * (  - 1./108.*m33*
         m55*RSe45613p - 1./108.*m33*m55*RSe463p - 1./108.*m33*m55*
         RSe45611p - 1./108.*m33*m55*RSe461p + 1./54.*m15*m33*RSe45613p
          - 1./54.*m15*m33*RSe463p + 1./54.*m15*m33*RSe45611p - 1./54.*
         m15*m33*RSe461p - 1./54.*m13*m55*RSe45613p - 1./54.*m13*m55*
         RSe463p + 1./54.*m13*m55*RSe45611p + 1./54.*m13*m55*RSe461p + 
         1./27.*m13*m15*RSe45613p - 1./27.*m13*m15*RSe463p - 1./27.*m13
         *m15*RSe45611p + 1./27.*m13*m15*RSe461p + 1./27.*m11*me2*
         RSe45613p - 1./27.*m11*me2*RSe463p - 1./27.*m11*me2*RSe45611p
          + 1./27.*m11*me2*RSe461p - 1./36.*m11*m55*RSe45613p - 1./36.*
         m11*m55*RSe463p - 1./108.*m11*m55*RSe45611p - 1./108.*m11*m55*
         RSe461p - 1./108.*m11*m33*RSe45613p - 1./36.*m11*m33*RSe463p
          - 1./108.*m11*m33*RSe45611p - 1./36.*m11*m33*RSe461p + 1./18.
         *m11*m15*RSe45613p - 1./18.*m11*m15*RSe463p + 1./54.*m11*m15*
         RSe45611p - 1./54.*m11*m15*RSe461p - 1./54.*m11*m13*RSe45613p
          - 1./18.*m11*m13*RSe463p );

      M13p6x23RV +=  + Ab(m15,mu2)*BbV(m11,me2,L) * ( 1./54.*m11*m13*
         RSe45611p + 1./18.*m11*m13*RSe461p - 7./108.*pow(m11,2)*
         RSe45613p - 5./108.*pow(m11,2)*RSe463p + 1./36.*pow(m11,2)*
         RSe45611p - 7./108.*pow(m11,2)*RSe461p );

      M13p6x23RV +=  + Ab(m15,mu2)*BbV(m11,L) * ( 7./540.*m55*RD1456ep
          - 1./108.*m35*RD1456ep + 1./120.*m33*RD1456ep - 1./108.*m33*
         m55*RS14563pe - 1./108.*m33*m55*RC1456pe - 1./108.*m33*m55*
         RS146pe - 1./60.*m15*RD1456ep + 1./54.*m15*m33*RS14563pe + 1./
         54.*m15*m33*RC1456pe - 1./54.*m15*m33*RS146pe - 1./135.*m13*
         RD1456ep - 1./54.*m13*m55*RS14563pe + 1./54.*m13*m55*RC1456pe
          + 1./54.*m13*m55*RS146pe + 1./27.*m13*m15*RS14563pe - 1./27.*
         m13*m15*RC1456pe + 1./27.*m13*m15*RS146pe + 13./1080.*m11*
         RD1456ep - 1./36.*m11*m55*RS14563pe - 1./108.*m11*m55*RC1456pe
          - 1./108.*m11*m55*RS146pe - 1./108.*m11*m33*RS14563pe - 1./
         108.*m11*m33*RC1456pe - 1./36.*m11*m33*RS146pe + 1./18.*m11*
         m15*RS14563pe + 1./54.*m11*m15*RC1456pe - 1./54.*m11*m15*
         RS146pe - 1./54.*m11*m13*RS14563pe + 1./54.*m11*m13*RC1456pe
          + 1./18.*m11*m13*RS146pe - 1./36.*pow(m11,2)*RS14563pe - 1./
         108.*pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS146pe );

      M13p6x23RV +=  + Ab(m15,mu2)*BbV(m14,L) * ( 1./144.*m44*m55 + 1./
         144.*pow(m44,2) + 1./144.*m33*m55 + 1./144.*m33*m44 - 1./72.*
         m15*m44 - 1./72.*m15*m33 - 1./72.*m14*m55 - 1./36.*m14*m44 - 1.
         /72.*m14*m33 + 1./36.*m14*m15 + 1./36.*pow(m14,2) - 1./72.*m13
         *m55 - 1./72.*m13*m44 + 1./36.*m13*m15 + 1./36.*m13*m14 + 1./
         72.*m11*m55 + 1./36.*m11*m44 + 1./72.*m11*m33 - 1./36.*m11*m15
          - 1./18.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + Ab(m15,mu2)*BbV(m15,L) * ( 1./72.*pow(m55,2) + 1.
         /72.*m33*m55 - 1./18.*m15*m55 - 1./36.*m15*m33 + 1./18.*pow(
         m15,2) - 1./36.*m13*m55 + 1./18.*m13*m15 + 1./24.*m11*m55 + 1./
         72.*m11*m33 - 1./12.*m11*m15 - 1./36.*m11*m13 + 1./36.*pow(
         m11,2) );

      M13p6x23RV +=  + Ab(m15,mu2)*BbV(m16,L) * ( 1./144.*pow(m66,2) + 
         1./144.*m55*m66 + 1./144.*m33*m66 + 1./144.*m33*m55 - 1./36.*
         m16*m66 - 1./72.*m16*m55 - 1./72.*m16*m33 + 1./36.*pow(m16,2)
          - 1./72.*m15*m66 - 1./72.*m15*m33 + 1./36.*m15*m16 - 1./72.*
         m13*m66 - 1./72.*m13*m55 + 1./36.*m13*m16 + 1./36.*m13*m15 + 1.
         /36.*m11*m66 + 1./72.*m11*m55 + 1./72.*m11*m33 - 1./18.*m11*
         m16 - 1./36.*m11*m15 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + Ab(m15,mu2)*BbV(m33,L) * ( 1./60.*m55*RD3456ep
          - 1./27.*m35*RD3456ep + 19./540.*m33*RD3456ep + 1./270.*m15*
         RD3456ep + 1./20.*m13*RD3456ep + 2./135.*m11*RD3456ep );

      M13p6x23RV +=  + Ab(m15,mu2)*CbV(m11,L) * (  - 1./108.*m33*m55*
         RD1456ep + 1./54.*m15*m33*RD1456ep + 1./54.*m13*m55*RD1456ep
          - 1./27.*m13*m15*RD1456ep - 1./108.*m11*m55*RD1456ep - 1./108.
         *m11*m33*RD1456ep + 1./54.*m11*m15*RD1456ep + 1./54.*m11*m13*
         RD1456ep - 1./108.*pow(m11,2)*RD1456ep );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(m11,L) * (  - 29./540.*RD1456ep
          - 7./540.*m66*RS14563pe + 7./540.*m66*RC1456pe - 1./60.*m66*
         RS145pe + 1./27.*m36*RS14563pe - 1./108.*m36*RC1456pe - 1./54.
         *m36*RS145pe + 1./90.*m33*RS14563pe + 1./120.*m33*RC1456pe - 1.
         /20.*m33*RS145pe - 1./90.*m16*RS14563pe - 1./60.*m16*RC1456pe
          + 2./45.*m16*RS145pe - 11./540.*m13*RS14563pe - 1./135.*m13*
         RC1456pe + 19./540.*m13*RS145pe + 1./27.*m11*RS34561pe + 43./
         540.*m11*RS14563pe + 1./27.*m11*RSe45613p + 1./27.*m11*
         RSp45613e - 1./27.*m11*RS345pe - 1./27.*m11*RSe453p - 1./27.*
         m11*RSp453e + 13./1080.*m11*RC1456pe - 1./27.*m11*RSe45611p - 
         1./27.*m11*RSp45611e - 1./45.*m11*RS145pe + 1./27.*m11*RSe451p
          + 1./27.*m11*RSp451e + 1./54.*RS(m11,m44,m55,m66,m33,me2,mp2)
         *m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11 );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(m14,L) * (  - 11./360.*m66 - 11./
         360.*m44 - 1./60.*m33 + 11./180.*m16 + 11./180.*m14 + 1./30.*
         m13 - 7./90.*m11 );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(m15,L) * (  - 11./360.*m66 - 11./
         360.*m55 - 1./60.*m33 + 11./180.*m16 + 11./180.*m15 + 1./30.*
         m13 - 7./90.*m11 );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(m16,L) * (  - 11./180.*m66 - 1./
         60.*m33 + 11./90.*m16 + 1./30.*m13 - 7./90.*m11 );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(m33,L) * (  - 1./180.*RD3456ep
          - 7./540.*m66*RS34561pe + 1./60.*m66*RC3456pe + 1./180.*m66*
         RS345pe + 1./27.*m36*RS34561pe - 1./27.*m36*RC3456pe + 1./27.*
         m36*RS345pe + 2./27.*m33*RS34561pe + 19./540.*m33*RC3456pe - 5.
         /54.*m33*RS345pe - 1./90.*m16*RS34561pe + 1./270.*m16*RC3456pe
          + 1./270.*m16*RS345pe - 11./540.*m13*RS34561pe + 1./20.*m13*
         RC3456pe - 43./540.*m13*RS345pe - 11./540.*m11*RS34561pe + 2./
         135.*m11*RC3456pe - 17./540.*m11*RS345pe + 1./27.*RS(m33,m44,
         m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11 );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(m34,L) * (  - 1./20.*m66 + 1./9.
         *m46 - 1./20.*m44 - 1./90.*m34 - 2./45.*m33 - 1./90.*m16 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(m35,L) * (  - 1./20.*m66 + 1./9.
         *m56 - 1./20.*m55 - 1./90.*m35 - 2./45.*m33 - 1./90.*m16 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(m36,L) * (  - 1./108.*RD3456ep
          - 1./108.*RD1456ep - 1./54.*me2*RSe45613p - 1./108.*me2*
         RSe45633p + 1./27.*me2*RSe453p - 1./108.*me2*RSe45611p + 1./27.
         *me2*RSe451p - 1./27.*me2*RSe456p - 1./54.*mp2*RSp45613e - 1./
         108.*mp2*RSp45633e + 1./27.*mp2*RSp453e - 1./108.*mp2*
         RSp45611e + 1./27.*mp2*RSp451e - 1./27.*mp2*RSp456e + 1./90.*
         m66 - 1./54.*m66*RS34561pe - 1./54.*m66*RS14563pe - 1./54.*m66
         *RSe45613p - 1./54.*m66*RSp45613e - 1./108.*m66*RC3456pe - 1./
         108.*m66*RSe45633p - 1./108.*m66*RSp45633e - 1./108.*m66*
         RC1456pe - 1./108.*m66*RSe45611p - 1./108.*m66*RSp45611e + 1./
         27.*m66*RSe456p + 1./27.*m66*RSp456e - 1./90.*m36 + 1./36.*m36
         *RC3456pe + 1./36.*m36*RSe45633p + 1./36.*m36*RSp45633e - 1./
         18.*m36*RS345pe - 1./18.*m36*RSe453p - 1./18.*m36*RSp453e - 1./
         36.*m36*RC1456pe - 1./36.*m36*RSe45611p - 1./36.*m36*RSp45611e
          + 1./18.*m36*RS145pe + 1./18.*m36*RSe451p + 1./18.*m36*
         RSp451e );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(m36,L) * (  - 2./45.*m33 - 5./
         108.*m33*RS34561pe - 1./36.*m33*RS14563pe - 1./36.*m33*
         RSe45613p - 1./36.*m33*RSp45613e - 1./36.*m33*RC3456pe - 1./54.
         *m33*RSe45633p - 1./54.*m33*RSp45633e + 7./108.*m33*RS345pe + 
         1./36.*m33*RSe453p + 1./36.*m33*RSp453e - 1./108.*m33*RC1456pe
          - 1./108.*m33*RSe45611p - 1./108.*m33*RSp45611e + 1./108.*m33
         *RS145pe + 1./54.*m33*RS645pe + 1./108.*m33*RSe451p + 1./108.*
         m33*RSp451e + 1./54.*m33*RSe456p + 1./54.*m33*RSp456e - 1./90.
         *m16 - 1./36.*m16*RC3456pe - 1./36.*m16*RSe45633p - 1./36.*m16
         *RSp45633e + 1./18.*m16*RS345pe + 1./18.*m16*RSe453p + 1./18.*
         m16*RSp453e + 1./36.*m16*RC1456pe + 1./36.*m16*RSe45611p + 1./
         36.*m16*RSp45611e - 1./18.*m16*RS145pe - 1./18.*m16*RSe451p - 
         1./18.*m16*RSp451e - 3./20.*m13 - 1./36.*m13*RS34561pe - 1./36.
         *m13*RS14563pe - 1./36.*m13*RSe45613p - 1./36.*m13*RSp45613e
          + 1./36.*m13*RC3456pe + 1./36.*m13*RSe45633p + 1./36.*m13*
         RSp45633e );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(m36,L) * (  - 1./36.*m13*RS345pe
          - 1./36.*m13*RSe453p - 1./36.*m13*RSp453e + 1./36.*m13*
         RC1456pe + 1./36.*m13*RSe45611p + 1./36.*m13*RSp45611e - 1./36.
         *m13*RS145pe + 1./36.*m13*RS645pe - 1./36.*m13*RSe451p - 1./36.
         *m13*RSp451e + 1./36.*m13*RSe456p + 1./36.*m13*RSp456e - 2./45.
         *m11 - 1./36.*m11*RS34561pe - 5./108.*m11*RS14563pe - 1./36.*
         m11*RSe45613p - 1./36.*m11*RSp45613e - 1./108.*m11*RC3456pe - 
         1./108.*m11*RSe45633p - 1./108.*m11*RSp45633e + 1./108.*m11*
         RS345pe + 1./108.*m11*RSe453p + 1./108.*m11*RSp453e - 1./36.*
         m11*RC1456pe - 1./54.*m11*RSe45611p - 1./54.*m11*RSp45611e + 7.
         /108.*m11*RS145pe + 1./54.*m11*RS645pe + 1./36.*m11*RSe451p + 
         1./36.*m11*RSp451e + 1./54.*m11*RSe456p + 1./54.*m11*RSp456e )
         ;

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(m46,L) * (  - 1./360.*m66 - 1./
         15.*m46 + 7./360.*m44 + 1./36.*m34 + 7./360.*m33 + 13./180.*
         m16 - 1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(m56,L) * (  - 1./360.*m66 - 1./
         15.*m56 + 7./360.*m55 + 1./36.*m35 + 7./360.*m33 + 13./180.*
         m16 - 1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(m66,L) * (  - 1./20.*m66 + 1./60.
         *m66*RS645pe + 1./36.*m36 - 1./108.*m36*RS645pe + 7./360.*m33
          - 7./1080.*m33*RS645pe + 13./180.*m16 - 13./540.*m16*RS645pe
          - 1./15.*m13 + 1./45.*m13*RS645pe - 1./360.*m11 + 1./1080.*
         m11*RS645pe );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(mp2,L) * ( 11./135.*mp2*
         RSp45613e - 1./180.*mp2*RSp45633e - 19./270.*mp2*RSp453e - 29./
         540.*mp2*RSp45611e + 7./270.*mp2*RSp451e + 1./45.*mp2*RSp456e
          - 7./540.*m66*RSp45613e + 1./60.*m66*RSp45633e + 1./180.*m66*
         RSp453e + 7./540.*m66*RSp45611e - 1./60.*m66*RSp451e - 1./180.
         *m66*RSp456e + 1./27.*m36*RSp45613e - 1./27.*m36*RSp45633e + 1.
         /27.*m36*RSp453e - 1./108.*m36*RSp45611e - 1./54.*m36*RSp451e
          - 1./108.*m36*RSp456e + 4./135.*m33*RSp45613e + 11./270.*m33*
         RSp45633e - 1./45.*m33*RSp453e + 1./120.*m33*RSp45611e - 1./20.
         *m33*RSp451e - 7./1080.*m33*RSp456e - 1./90.*m16*RSp45613e + 1.
         /270.*m16*RSp45633e + 1./270.*m16*RSp453e - 1./60.*m16*
         RSp45611e + 2./45.*m16*RSp451e - 13./540.*m16*RSp456e - 11./
         540.*m13*RSp45613e + 1./20.*m13*RSp45633e - 43./540.*m13*
         RSp453e - 1./135.*m13*RSp45611e + 19./540.*m13*RSp451e + 1./45.
         *m13*RSp456e - 1./540.*m11*RSp45613e + 2./135.*m11*RSp45633e
          - 17./540.*m11*RSp453e );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(mp2,L) * ( 37./360.*m11*
         RSp45611e - 23./270.*m11*RSp451e + 1./1080.*m11*RSp456e );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(me2,L) * ( 11./135.*me2*
         RSe45613p - 1./180.*me2*RSe45633p - 19./270.*me2*RSe453p - 29./
         540.*me2*RSe45611p + 7./270.*me2*RSe451p + 1./45.*me2*RSe456p
          - 7./540.*m66*RSe45613p + 1./60.*m66*RSe45633p + 1./180.*m66*
         RSe453p + 7./540.*m66*RSe45611p - 1./60.*m66*RSe451p - 1./180.
         *m66*RSe456p + 1./27.*m36*RSe45613p - 1./27.*m36*RSe45633p + 1.
         /27.*m36*RSe453p - 1./108.*m36*RSe45611p - 1./54.*m36*RSe451p
          - 1./108.*m36*RSe456p + 4./135.*m33*RSe45613p + 11./270.*m33*
         RSe45633p - 1./45.*m33*RSe453p + 1./120.*m33*RSe45611p - 1./20.
         *m33*RSe451p - 7./1080.*m33*RSe456p - 1./90.*m16*RSe45613p + 1.
         /270.*m16*RSe45633p + 1./270.*m16*RSe453p - 1./60.*m16*
         RSe45611p + 2./45.*m16*RSe451p - 13./540.*m16*RSe456p - 11./
         540.*m13*RSe45613p + 1./20.*m13*RSe45633p - 43./540.*m13*
         RSe453p - 1./135.*m13*RSe45611p + 19./540.*m13*RSe451p + 1./45.
         *m13*RSe456p - 1./540.*m11*RSe45613p + 2./135.*m11*RSe45633p
          - 17./540.*m11*RSe453p );

      M13p6x23RV +=  + Ab(m16,mu2)*AbV(me2,L) * ( 37./360.*m11*
         RSe45611p - 23./270.*m11*RSe451p + 1./1080.*m11*RSe456p );

      M13p6x23RV +=  + Ab(m16,mu2)*BbV(m11,m33,L) * (  - 1./108.*m33*
         m66*RS34561pe - 1./108.*m33*m66*RS345pe + 1./54.*m16*m33*
         RS34561pe - 1./54.*m16*m33*RS345pe - 1./54.*m13*m66*RS34561pe
          - 1./54.*m13*m66*RS345pe + 1./27.*m13*m16*RS34561pe - 1./27.*
         m13*m16*RS345pe - 1./36.*m11*m66*RS34561pe - 1./36.*m11*m66*
         RS345pe + 1./36.*m11*m33*RS34561pe - 7./108.*m11*m33*RS345pe
          + 1./18.*m11*m16*RS34561pe - 1./18.*m11*m16*RS345pe - 1./54.*
         m11*m13*RS34561pe - 1./18.*m11*m13*RS345pe - 7./108.*pow(
         m11,2)*RS34561pe - 5./108.*pow(m11,2)*RS345pe );

      M13p6x23RV +=  + Ab(m16,mu2)*BbV(m11,mp2,L) * (  - 1./108.*m33*
         m66*RSp45613e - 1./108.*m33*m66*RSp453e - 1./108.*m33*m66*
         RSp45611e - 1./108.*m33*m66*RSp451e + 1./54.*m16*m33*RSp45613e
          - 1./54.*m16*m33*RSp453e + 1./54.*m16*m33*RSp45611e - 1./54.*
         m16*m33*RSp451e - 1./54.*m13*m66*RSp45613e - 1./54.*m13*m66*
         RSp453e + 1./54.*m13*m66*RSp45611e + 1./54.*m13*m66*RSp451e + 
         1./27.*m13*m16*RSp45613e - 1./27.*m13*m16*RSp453e - 1./27.*m13
         *m16*RSp45611e + 1./27.*m13*m16*RSp451e + 1./27.*m11*mp2*
         RSp45613e - 1./27.*m11*mp2*RSp453e - 1./27.*m11*mp2*RSp45611e
          + 1./27.*m11*mp2*RSp451e - 1./36.*m11*m66*RSp45613e - 1./36.*
         m11*m66*RSp453e - 1./108.*m11*m66*RSp45611e - 1./108.*m11*m66*
         RSp451e - 1./108.*m11*m33*RSp45613e - 1./36.*m11*m33*RSp453e
          - 1./108.*m11*m33*RSp45611e - 1./36.*m11*m33*RSp451e + 1./18.
         *m11*m16*RSp45613e - 1./18.*m11*m16*RSp453e + 1./54.*m11*m16*
         RSp45611e - 1./54.*m11*m16*RSp451e - 1./54.*m11*m13*RSp45613e
          - 1./18.*m11*m13*RSp453e );

      M13p6x23RV +=  + Ab(m16,mu2)*BbV(m11,mp2,L) * ( 1./54.*m11*m13*
         RSp45611e + 1./18.*m11*m13*RSp451e - 7./108.*pow(m11,2)*
         RSp45613e - 5./108.*pow(m11,2)*RSp453e + 1./36.*pow(m11,2)*
         RSp45611e - 7./108.*pow(m11,2)*RSp451e );

      M13p6x23RV +=  + Ab(m16,mu2)*BbV(m11,me2,L) * (  - 1./108.*m33*
         m66*RSe45613p - 1./108.*m33*m66*RSe453p - 1./108.*m33*m66*
         RSe45611p - 1./108.*m33*m66*RSe451p + 1./54.*m16*m33*RSe45613p
          - 1./54.*m16*m33*RSe453p + 1./54.*m16*m33*RSe45611p - 1./54.*
         m16*m33*RSe451p - 1./54.*m13*m66*RSe45613p - 1./54.*m13*m66*
         RSe453p + 1./54.*m13*m66*RSe45611p + 1./54.*m13*m66*RSe451p + 
         1./27.*m13*m16*RSe45613p - 1./27.*m13*m16*RSe453p - 1./27.*m13
         *m16*RSe45611p + 1./27.*m13*m16*RSe451p + 1./27.*m11*me2*
         RSe45613p - 1./27.*m11*me2*RSe453p - 1./27.*m11*me2*RSe45611p
          + 1./27.*m11*me2*RSe451p - 1./36.*m11*m66*RSe45613p - 1./36.*
         m11*m66*RSe453p - 1./108.*m11*m66*RSe45611p - 1./108.*m11*m66*
         RSe451p - 1./108.*m11*m33*RSe45613p - 1./36.*m11*m33*RSe453p
          - 1./108.*m11*m33*RSe45611p - 1./36.*m11*m33*RSe451p + 1./18.
         *m11*m16*RSe45613p - 1./18.*m11*m16*RSe453p + 1./54.*m11*m16*
         RSe45611p - 1./54.*m11*m16*RSe451p - 1./54.*m11*m13*RSe45613p
          - 1./18.*m11*m13*RSe453p );

      M13p6x23RV +=  + Ab(m16,mu2)*BbV(m11,me2,L) * ( 1./54.*m11*m13*
         RSe45611p + 1./18.*m11*m13*RSe451p - 7./108.*pow(m11,2)*
         RSe45613p - 5./108.*pow(m11,2)*RSe453p + 1./36.*pow(m11,2)*
         RSe45611p - 7./108.*pow(m11,2)*RSe451p );

      M13p6x23RV +=  + Ab(m16,mu2)*BbV(m11,L) * ( 7./540.*m66*RD1456ep
          - 1./108.*m36*RD1456ep + 1./120.*m33*RD1456ep - 1./108.*m33*
         m66*RS14563pe - 1./108.*m33*m66*RC1456pe - 1./108.*m33*m66*
         RS145pe - 1./60.*m16*RD1456ep + 1./54.*m16*m33*RS14563pe + 1./
         54.*m16*m33*RC1456pe - 1./54.*m16*m33*RS145pe - 1./135.*m13*
         RD1456ep - 1./54.*m13*m66*RS14563pe + 1./54.*m13*m66*RC1456pe
          + 1./54.*m13*m66*RS145pe + 1./27.*m13*m16*RS14563pe - 1./27.*
         m13*m16*RC1456pe + 1./27.*m13*m16*RS145pe + 13./1080.*m11*
         RD1456ep - 1./36.*m11*m66*RS14563pe - 1./108.*m11*m66*RC1456pe
          - 1./108.*m11*m66*RS145pe - 1./108.*m11*m33*RS14563pe - 1./
         108.*m11*m33*RC1456pe - 1./36.*m11*m33*RS145pe + 1./18.*m11*
         m16*RS14563pe + 1./54.*m11*m16*RC1456pe - 1./54.*m11*m16*
         RS145pe - 1./54.*m11*m13*RS14563pe + 1./54.*m11*m13*RC1456pe
          + 1./18.*m11*m13*RS145pe - 1./36.*pow(m11,2)*RS14563pe - 1./
         108.*pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS145pe );

      M13p6x23RV +=  + Ab(m16,mu2)*BbV(m14,L) * ( 1./144.*m44*m66 + 1./
         144.*pow(m44,2) + 1./144.*m33*m66 + 1./144.*m33*m44 - 1./72.*
         m16*m44 - 1./72.*m16*m33 - 1./72.*m14*m66 - 1./36.*m14*m44 - 1.
         /72.*m14*m33 + 1./36.*m14*m16 + 1./36.*pow(m14,2) - 1./72.*m13
         *m66 - 1./72.*m13*m44 + 1./36.*m13*m16 + 1./36.*m13*m14 + 1./
         72.*m11*m66 + 1./36.*m11*m44 + 1./72.*m11*m33 - 1./36.*m11*m16
          - 1./18.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + Ab(m16,mu2)*BbV(m15,L) * ( 1./144.*m55*m66 + 1./
         144.*pow(m55,2) + 1./144.*m33*m66 + 1./144.*m33*m55 - 1./72.*
         m16*m55 - 1./72.*m16*m33 - 1./72.*m15*m66 - 1./36.*m15*m55 - 1.
         /72.*m15*m33 + 1./36.*m15*m16 + 1./36.*pow(m15,2) - 1./72.*m13
         *m66 - 1./72.*m13*m55 + 1./36.*m13*m16 + 1./36.*m13*m15 + 1./
         72.*m11*m66 + 1./36.*m11*m55 + 1./72.*m11*m33 - 1./36.*m11*m16
          - 1./18.*m11*m15 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + Ab(m16,mu2)*BbV(m16,L) * ( 1./72.*pow(m66,2) + 1.
         /72.*m33*m66 - 1./18.*m16*m66 - 1./36.*m16*m33 + 1./18.*pow(
         m16,2) - 1./36.*m13*m66 + 1./18.*m13*m16 + 1./24.*m11*m66 + 1./
         72.*m11*m33 - 1./12.*m11*m16 - 1./36.*m11*m13 + 1./36.*pow(
         m11,2) );

      M13p6x23RV +=  + Ab(m16,mu2)*BbV(m33,L) * ( 1./60.*m66*RD3456ep
          - 1./27.*m36*RD3456ep + 19./540.*m33*RD3456ep + 1./270.*m16*
         RD3456ep + 1./20.*m13*RD3456ep + 2./135.*m11*RD3456ep );

      M13p6x23RV +=  + Ab(m16,mu2)*CbV(m11,L) * (  - 1./108.*m33*m66*
         RD1456ep + 1./54.*m16*m33*RD1456ep + 1./54.*m13*m66*RD1456ep
          - 1./27.*m13*m16*RD1456ep - 1./108.*m11*m66*RD1456ep - 1./108.
         *m11*m33*RD1456ep + 1./54.*m11*m16*RD1456ep + 1./54.*m11*m13*
         RD1456ep - 1./108.*pow(m11,2)*RD1456ep );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m11,L) * (  - 1./270.*RD3456ep*
         RS14563pe + 1./540.*RC1456pe*RD3456ep - 1./270.*RD1456ep*
         RS34561pe + 1./540.*RD1456ep*RC3456pe + 11./405.*m33*RS14563pe
         *RS34561pe - 2./135.*m33*RC3456pe*RS14563pe + 1./162.*m33*
         RC1456pe*RS34561pe - 19./1620.*m33*RC1456pe*RC3456pe - 7./405.
         *m13*RS14563pe*RS34561pe + 7./810.*m13*RC3456pe*RS14563pe + 7./
         810.*m13*RC1456pe*RS34561pe - 7./1620.*m13*RC1456pe*RC3456pe
          + 11./405.*m11*RS14563pe*RS34561pe + 1./162.*m11*RC3456pe*
         RS14563pe + 1./45.*m11*RC1456pe*RS34561pe - 19./1620.*m11*
         RC1456pe*RC3456pe - 1./54.*m11*m33*RN13*RC1456pe*RS34561pe - 1.
         /27.*m11*m13*RN13*RC1456pe*RS34561pe - 1./18.*pow(m11,2)*RN13*
         RC1456pe*RS34561pe + 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*
         m33*RS34561pe - 1./162.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33*
         RC3456pe + 2./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*
         RS34561pe - 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*
         RC3456pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m11,L) * ( 2./81.*RS(m33,m44,m55
         ,m66,m11,me2,mp2)*m33*RS14563pe - 1./81.*RS(m33,m44,m55,m66,
         m11,me2,mp2)*m33*RC1456pe + 1./81.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11*RS14563pe - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*
         m11*RC1456pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m13,L) * (  - 2./81.*RD3456ep*
         RS34561pe - 1./54.*RD3456ep*RS14563pe - 1./54.*RD3456ep*
         RSe45613p - 1./54.*RD3456ep*RSp45613e + 1./81.*RD3456ep*
         RC3456pe + 1./108.*RSe45633p*RD3456ep + 1./108.*RSp45633e*
         RD3456ep + 1./108.*RC1456pe*RD3456ep - 1./162.*RD1456ep*
         RS34561pe + 1./324.*RD1456ep*RC3456pe + 1./108.*RSe45611p*
         RD3456ep + 1./108.*RSp45611e*RD3456ep + 1./81.*me2*RSe45613p*
         RS34561pe - 1./162.*me2*RC3456pe*RSe45613p - 1./162.*me2*
         RSe45633p*RS34561pe + 1./324.*me2*RSe45633p*RC3456pe - 1./162.
         *me2*RSe45611p*RS34561pe + 1./324.*me2*RSe45611p*RC3456pe + 1./
         81.*mp2*RSp45613e*RS34561pe - 1./162.*mp2*RC3456pe*RSp45613e
          - 1./162.*mp2*RSp45633e*RS34561pe + 1./324.*mp2*RSp45633e*
         RC3456pe - 1./162.*mp2*RSp45611e*RS34561pe + 1./324.*mp2*
         RSp45611e*RC3456pe + 3./20.*m33 + 1./27.*m33*pow(RS34561pe,2)
          + 2./81.*m33*RS14563pe*RS34561pe + 2./81.*m33*RSe45613p*
         RS34561pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m13,L) * ( 2./81.*m33*RSp45613e*
         RS34561pe - 2./81.*m33*RC3456pe*RS34561pe - 1./162.*m33*
         RC3456pe*RS14563pe - 1./162.*m33*RC3456pe*RSe45613p - 1./162.*
         m33*RC3456pe*RSp45613e - 1./162.*m33*pow(RC3456pe,2) - 1./162.
         *m33*RSe45633p*RS34561pe - 1./108.*m33*RSe45633p*RC3456pe - 1./
         162.*m33*RSp45633e*RS34561pe - 1./108.*m33*RSp45633e*RC3456pe
          - 1./54.*m33*RC1456pe*RS34561pe + 5./324.*m33*RC1456pe*
         RC3456pe - 1./54.*m33*RSe45611p*RS34561pe + 5./324.*m33*
         RSe45611p*RC3456pe - 1./54.*m33*RSp45611e*RS34561pe + 5./324.*
         m33*RSp45611e*RC3456pe - 1./36.*pow(m33,2)*RN13 + 41./180.*m13
          - 2./27.*m13*pow(RS34561pe,2) - 2./27.*m13*RS14563pe*
         RS34561pe - 2./27.*m13*RSe45613p*RS34561pe - 2./27.*m13*
         RSp45613e*RS34561pe + 2./27.*m13*RC3456pe*RS34561pe + 1./27.*
         m13*RC3456pe*RS14563pe + 1./27.*m13*RC3456pe*RSe45613p + 1./27.
         *m13*RC3456pe*RSp45613e - 1./54.*m13*pow(RC3456pe,2) + 1./27.*
         m13*RSe45633p*RS34561pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m13,L) * (  - 1./54.*m13*
         RSe45633p*RC3456pe + 1./27.*m13*RSp45633e*RS34561pe - 1./54.*
         m13*RSp45633e*RC3456pe + 1./27.*m13*RC1456pe*RS34561pe - 1./54.
         *m13*RC1456pe*RC3456pe + 1./27.*m13*RSe45611p*RS34561pe - 1./
         54.*m13*RSe45611p*RC3456pe + 1./27.*m13*RSp45611e*RS34561pe - 
         1./54.*m13*RSp45611e*RC3456pe - 1./9.*m13*m33*RN13 - 1./9.*
         pow(m13,2)*RN13 + 17./180.*m11 - 1./81.*m11*pow(RS34561pe,2)
          - 1./81.*m11*RSe45613p*RS34561pe - 1./81.*m11*RSp45613e*
         RS34561pe - 1./162.*m11*RC3456pe*RS14563pe - 1./162.*m11*pow(
         RC3456pe,2) - 1./162.*m11*RSe45633p*RC3456pe - 1./162.*m11*
         RSp45633e*RC3456pe + 1./162.*m11*RC1456pe*RS34561pe + 1./108.*
         m11*RC1456pe*RC3456pe + 1./81.*m11*RSe45611p*RS34561pe + 1./
         162.*m11*RSe45611p*RC3456pe + 1./81.*m11*RSp45611e*RS34561pe
          + 1./162.*m11*RSp45611e*RC3456pe - 5./18.*m11*m33*RN13 - 1./3.
         *m11*m13*RN13 - 5./36.*pow(m11,2)*RN13 );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m14,L) * (  - 1./180.*RD3456ep
          - 7./540.*m44*RS34561pe + 1./60.*m44*RC3456pe + 1./180.*m44*
         RS356pe + 1./27.*m34*RS34561pe - 1./27.*m34*RC3456pe + 1./27.*
         m34*RS356pe + 2./27.*m33*RS34561pe + 19./540.*m33*RC3456pe - 5.
         /54.*m33*RS356pe + 1./108.*m33*m44*RN13*RS34561pe + 1./108.*
         m33*m44*RN13*RS356pe - 1./90.*m14*RS34561pe + 1./270.*m14*
         RC3456pe + 1./270.*m14*RS356pe - 1./54.*m14*m33*RN13*RS34561pe
          + 1./54.*m14*m33*RN13*RS356pe - 11./540.*m13*RS34561pe + 1./
         20.*m13*RC3456pe - 43./540.*m13*RS356pe + 1./54.*m13*m44*RN13*
         RS34561pe + 1./54.*m13*m44*RN13*RS356pe - 1./27.*m13*m14*RN13*
         RS34561pe + 1./27.*m13*m14*RN13*RS356pe - 11./540.*m11*
         RS34561pe + 2./135.*m11*RC3456pe - 17./540.*m11*RS356pe + 1./
         36.*m11*m44*RN13*RS34561pe + 1./36.*m11*m44*RN13*RS356pe - 1./
         36.*m11*m33*RN13*RS34561pe + 7./108.*m11*m33*RN13*RS356pe - 1./
         18.*m11*m14*RN13*RS34561pe + 1./18.*m11*m14*RN13*RS356pe + 1./
         54.*m11*m13*RN13*RS34561pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m14,L) * ( 1./18.*m11*m13*RN13*
         RS356pe + 7./108.*pow(m11,2)*RN13*RS34561pe + 5./108.*pow(
         m11,2)*RN13*RS356pe + 1./27.*RS(m33,m44,m55,m66,m11,me2,mp2)*
         m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11 );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m15,L) * (  - 1./180.*RD3456ep
          - 7./540.*m55*RS34561pe + 1./60.*m55*RC3456pe + 1./180.*m55*
         RS346pe + 1./27.*m35*RS34561pe - 1./27.*m35*RC3456pe + 1./27.*
         m35*RS346pe + 2./27.*m33*RS34561pe + 19./540.*m33*RC3456pe - 5.
         /54.*m33*RS346pe + 1./108.*m33*m55*RN13*RS34561pe + 1./108.*
         m33*m55*RN13*RS346pe - 1./90.*m15*RS34561pe + 1./270.*m15*
         RC3456pe + 1./270.*m15*RS346pe - 1./54.*m15*m33*RN13*RS34561pe
          + 1./54.*m15*m33*RN13*RS346pe - 11./540.*m13*RS34561pe + 1./
         20.*m13*RC3456pe - 43./540.*m13*RS346pe + 1./54.*m13*m55*RN13*
         RS34561pe + 1./54.*m13*m55*RN13*RS346pe - 1./27.*m13*m15*RN13*
         RS34561pe + 1./27.*m13*m15*RN13*RS346pe - 11./540.*m11*
         RS34561pe + 2./135.*m11*RC3456pe - 17./540.*m11*RS346pe + 1./
         36.*m11*m55*RN13*RS34561pe + 1./36.*m11*m55*RN13*RS346pe - 1./
         36.*m11*m33*RN13*RS34561pe + 7./108.*m11*m33*RN13*RS346pe - 1./
         18.*m11*m15*RN13*RS34561pe + 1./18.*m11*m15*RN13*RS346pe + 1./
         54.*m11*m13*RN13*RS34561pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m15,L) * ( 1./18.*m11*m13*RN13*
         RS346pe + 7./108.*pow(m11,2)*RN13*RS34561pe + 5./108.*pow(
         m11,2)*RN13*RS346pe + 1./27.*RS(m33,m44,m55,m66,m11,me2,mp2)*
         m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11 );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m16,L) * (  - 1./180.*RD3456ep
          - 7./540.*m66*RS34561pe + 1./60.*m66*RC3456pe + 1./180.*m66*
         RS345pe + 1./27.*m36*RS34561pe - 1./27.*m36*RC3456pe + 1./27.*
         m36*RS345pe + 2./27.*m33*RS34561pe + 19./540.*m33*RC3456pe - 5.
         /54.*m33*RS345pe + 1./108.*m33*m66*RN13*RS34561pe + 1./108.*
         m33*m66*RN13*RS345pe - 1./90.*m16*RS34561pe + 1./270.*m16*
         RC3456pe + 1./270.*m16*RS345pe - 1./54.*m16*m33*RN13*RS34561pe
          + 1./54.*m16*m33*RN13*RS345pe - 11./540.*m13*RS34561pe + 1./
         20.*m13*RC3456pe - 43./540.*m13*RS345pe + 1./54.*m13*m66*RN13*
         RS34561pe + 1./54.*m13*m66*RN13*RS345pe - 1./27.*m13*m16*RN13*
         RS34561pe + 1./27.*m13*m16*RN13*RS345pe - 11./540.*m11*
         RS34561pe + 2./135.*m11*RC3456pe - 17./540.*m11*RS345pe + 1./
         36.*m11*m66*RN13*RS34561pe + 1./36.*m11*m66*RN13*RS345pe - 1./
         36.*m11*m33*RN13*RS34561pe + 7./108.*m11*m33*RN13*RS345pe - 1./
         18.*m11*m16*RN13*RS34561pe + 1./18.*m11*m16*RN13*RS345pe + 1./
         54.*m11*m13*RN13*RS34561pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m16,L) * ( 1./18.*m11*m13*RN13*
         RS345pe + 7./108.*pow(m11,2)*RN13*RS34561pe + 5./108.*pow(
         m11,2)*RN13*RS345pe + 1./27.*RS(m33,m44,m55,m66,m11,me2,mp2)*
         m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11 );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m33,L) * (  - 1./135.*RD3456ep*
         RS34561pe + 1./270.*RD3456ep*RC3456pe + 11./90.*m33 + 1./45.*
         m33*pow(RS34561pe,2) + 4./81.*m33*RC3456pe*RS34561pe + 2./27.*
         m33*RC3456pe*RS14563pe - 17./810.*m33*pow(RC3456pe,2) + 1./18.
         *pow(m33,2)*RN3p*RC3456pe*RSp45613e + 1./54.*pow(m33,2)*RN3p*
         RSp45633e*RC3456pe + 1./18.*pow(m33,2)*RN3e*RC3456pe*RSe45613p
          + 1./54.*pow(m33,2)*RN3e*RSe45633p*RC3456pe - 1./54.*pow(
         m33,2)*RN13*RC3456pe*RS14563pe + 1./180.*m13 - 7./405.*m13*
         pow(RS34561pe,2) + 7./405.*m13*RC3456pe*RS34561pe - 7./1620.*
         m13*pow(RC3456pe,2) + 1./27.*m13*m33*RN3p*RC3456pe*RSp45613e
          - 1./27.*m13*m33*RN3p*RSp45633e*RC3456pe + 1./27.*m13*m33*
         RN3e*RC3456pe*RSe45613p - 1./27.*m13*m33*RN3e*RSe45633p*
         RC3456pe - 1./27.*m13*m33*RN13*RC3456pe*RS14563pe + 11./90.*
         m11 + 13./405.*m11*pow(RS34561pe,2) + 1./135.*m11*RC3456pe*
         RS34561pe + 13./810.*m11*pow(RC3456pe,2) + 1./54.*m11*m33*RN3p
         *RC3456pe*RSp45613e );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m33,L) * ( 1./54.*m11*m33*RN3p*
         RSp45633e*RC3456pe + 1./54.*m11*m33*RN3e*RC3456pe*RSe45613p + 
         1./54.*m11*m33*RN3e*RSe45633p*RC3456pe - 1./18.*m11*m33*RN13*
         RC3456pe*RS14563pe + 4./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*
         m33*RS34561pe - 2./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33*
         RC3456pe + 2./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RS34561pe - 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RC3456pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m34,L) * (  - 29./540.*RD3456ep
          - 7./540.*m44*RS34561pe - 1./54.*m44*RS14563pe + 7./540.*m44*
         RC3456pe - 1./60.*m44*RS356pe - 1./54.*m44*RS156pe - 1./90.*
         m34*RS34561pe + 1./27.*m34*RS14563pe - 1./60.*m34*RC3456pe + 2.
         /45.*m34*RS356pe - 1./27.*m34*RS156pe + 43./540.*m33*RS34561pe
          + 1./27.*m33*RSe45613p + 1./27.*m33*RSp45613e + 13./1080.*m33
         *RC3456pe - 1./27.*m33*RSe45633p - 1./27.*m33*RSp45633e - 1./
         45.*m33*RS356pe + 1./27.*m33*RSe563p + 1./27.*m33*RSp563e - 2./
         27.*m33*RS156pe - 1./27.*m33*RSe561p - 1./27.*m33*RSp561e + 1./
         27.*m33*me2*RN3e*RSe45613p - 1./27.*m33*me2*RN3e*RSe45633p + 1.
         /27.*m33*me2*RN3e*RSe563p - 1./27.*m33*me2*RN3e*RSe561p + 1./
         27.*m33*mp2*RN3p*RSp45613e - 1./27.*m33*mp2*RN3p*RSp45633e + 1.
         /27.*m33*mp2*RN3p*RSp563e - 1./27.*m33*mp2*RN3p*RSp561e - 1./
         36.*m33*m44*RN3p*RSp45613e - 1./108.*m33*m44*RN3p*RSp45633e - 
         1./108.*m33*m44*RN3p*RSp563e - 1./36.*m33*m44*RN3p*RSp561e - 1.
         /36.*m33*m44*RN3e*RSe45613p );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m34,L) * (  - 1./108.*m33*m44*
         RN3e*RSe45633p - 1./108.*m33*m44*RN3e*RSe563p - 1./36.*m33*m44
         *RN3e*RSe561p + 1./108.*m33*m44*RN13*RS14563pe + 1./108.*m33*
         m44*RN13*RS156pe + 1./18.*m33*m34*RN3p*RSp45613e + 1./54.*m33*
         m34*RN3p*RSp45633e - 1./54.*m33*m34*RN3p*RSp563e - 1./18.*m33*
         m34*RN3p*RSp561e + 1./18.*m33*m34*RN3e*RSe45613p + 1./54.*m33*
         m34*RN3e*RSe45633p - 1./54.*m33*m34*RN3e*RSe563p - 1./18.*m33*
         m34*RN3e*RSe561p - 1./54.*m33*m34*RN13*RS14563pe + 1./54.*m33*
         m34*RN13*RS156pe - 7./108.*pow(m33,2)*RN3p*RSp45613e + 1./36.*
         pow(m33,2)*RN3p*RSp45633e - 7./108.*pow(m33,2)*RN3p*RSp563e - 
         5./108.*pow(m33,2)*RN3p*RSp561e - 7./108.*pow(m33,2)*RN3e*
         RSe45613p + 1./36.*pow(m33,2)*RN3e*RSe45633p - 7./108.*pow(
         m33,2)*RN3e*RSe563p - 5./108.*pow(m33,2)*RN3e*RSe561p + 1./36.
         *pow(m33,2)*RN13*RS14563pe + 1./108.*pow(m33,2)*RN13*RS156pe
          + 1./27.*m14*RS34561pe - 1./108.*m14*RC3456pe - 1./54.*m14*
         RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m34,L) * (  - 11./540.*m13*
         RS34561pe + 1./27.*m13*RS14563pe - 1./135.*m13*RC3456pe + 19./
         540.*m13*RS356pe - 1./27.*m13*RS156pe - 1./54.*m13*m44*RN3p*
         RSp45613e + 1./54.*m13*m44*RN3p*RSp45633e + 1./54.*m13*m44*
         RN3p*RSp563e - 1./54.*m13*m44*RN3p*RSp561e - 1./54.*m13*m44*
         RN3e*RSe45613p + 1./54.*m13*m44*RN3e*RSe45633p + 1./54.*m13*
         m44*RN3e*RSe563p - 1./54.*m13*m44*RN3e*RSe561p + 1./54.*m13*
         m44*RN13*RS14563pe + 1./54.*m13*m44*RN13*RS156pe + 1./27.*m13*
         m34*RN3p*RSp45613e - 1./27.*m13*m34*RN3p*RSp45633e + 1./27.*
         m13*m34*RN3p*RSp563e - 1./27.*m13*m34*RN3p*RSp561e + 1./27.*
         m13*m34*RN3e*RSe45613p - 1./27.*m13*m34*RN3e*RSe45633p + 1./27.
         *m13*m34*RN3e*RSe563p - 1./27.*m13*m34*RN3e*RSe561p - 1./27.*
         m13*m34*RN13*RS14563pe + 1./27.*m13*m34*RN13*RS156pe - 1./54.*
         m13*m33*RN3p*RSp45613e + 1./54.*m13*m33*RN3p*RSp45633e + 1./18.
         *m13*m33*RN3p*RSp563e - 1./18.*m13*m33*RN3p*RSp561e - 1./54.*
         m13*m33*RN3e*RSe45613p );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m34,L) * ( 1./54.*m13*m33*RN3e*
         RSe45633p + 1./18.*m13*m33*RN3e*RSe563p - 1./18.*m13*m33*RN3e*
         RSe561p + 1./18.*m13*m33*RN13*RS14563pe + 1./54.*m13*m33*RN13*
         RS156pe + 1./90.*m11*RS34561pe + 1./54.*m11*RS14563pe + 1./120.
         *m11*RC3456pe - 1./20.*m11*RS356pe - 1./54.*m11*RS156pe - 1./
         108.*m11*m44*RN3p*RSp45613e - 1./108.*m11*m44*RN3p*RSp45633e
          - 1./108.*m11*m44*RN3p*RSp563e - 1./108.*m11*m44*RN3p*RSp561e
          - 1./108.*m11*m44*RN3e*RSe45613p - 1./108.*m11*m44*RN3e*
         RSe45633p - 1./108.*m11*m44*RN3e*RSe563p - 1./108.*m11*m44*
         RN3e*RSe561p + 1./36.*m11*m44*RN13*RS14563pe + 1./36.*m11*m44*
         RN13*RS156pe + 1./54.*m11*m34*RN3p*RSp45613e + 1./54.*m11*m34*
         RN3p*RSp45633e - 1./54.*m11*m34*RN3p*RSp563e - 1./54.*m11*m34*
         RN3p*RSp561e + 1./54.*m11*m34*RN3e*RSe45613p + 1./54.*m11*m34*
         RN3e*RSe45633p - 1./54.*m11*m34*RN3e*RSe563p - 1./54.*m11*m34*
         RN3e*RSe561p - 1./18.*m11*m34*RN13*RS14563pe + 1./18.*m11*m34*
         RN13*RS156pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m34,L) * (  - 1./108.*m11*m33*
         RN3p*RSp45613e - 1./108.*m11*m33*RN3p*RSp45633e - 1./36.*m11*
         m33*RN3p*RSp563e - 1./36.*m11*m33*RN3p*RSp561e - 1./108.*m11*
         m33*RN3e*RSe45613p - 1./108.*m11*m33*RN3e*RSe45633p - 1./36.*
         m11*m33*RN3e*RSe563p - 1./36.*m11*m33*RN3e*RSe561p + 1./36.*
         m11*m33*RN13*RS14563pe + 1./12.*m11*m33*RN13*RS156pe - 1./27.*
         m11*m13*RN13*RS14563pe + 1./27.*m11*m13*RN13*RS156pe - 1./54.*
         pow(m11,2)*RN13*RS14563pe + 1./54.*pow(m11,2)*RN13*RS156pe + 1.
         /27.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,
         m55,m66,m11,me2,mp2)*m11 );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m35,L) * (  - 29./540.*RD3456ep
          - 7./540.*m55*RS34561pe - 1./54.*m55*RS14563pe + 7./540.*m55*
         RC3456pe - 1./60.*m55*RS346pe - 1./54.*m55*RS146pe - 1./90.*
         m35*RS34561pe + 1./27.*m35*RS14563pe - 1./60.*m35*RC3456pe + 2.
         /45.*m35*RS346pe - 1./27.*m35*RS146pe + 43./540.*m33*RS34561pe
          + 1./27.*m33*RSe45613p + 1./27.*m33*RSp45613e + 13./1080.*m33
         *RC3456pe - 1./27.*m33*RSe45633p - 1./27.*m33*RSp45633e - 1./
         45.*m33*RS346pe + 1./27.*m33*RSe463p + 1./27.*m33*RSp463e - 2./
         27.*m33*RS146pe - 1./27.*m33*RSe461p - 1./27.*m33*RSp461e + 1./
         27.*m33*me2*RN3e*RSe45613p - 1./27.*m33*me2*RN3e*RSe45633p + 1.
         /27.*m33*me2*RN3e*RSe463p - 1./27.*m33*me2*RN3e*RSe461p + 1./
         27.*m33*mp2*RN3p*RSp45613e - 1./27.*m33*mp2*RN3p*RSp45633e + 1.
         /27.*m33*mp2*RN3p*RSp463e - 1./27.*m33*mp2*RN3p*RSp461e - 1./
         36.*m33*m55*RN3p*RSp45613e - 1./108.*m33*m55*RN3p*RSp45633e - 
         1./108.*m33*m55*RN3p*RSp463e - 1./36.*m33*m55*RN3p*RSp461e - 1.
         /36.*m33*m55*RN3e*RSe45613p );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m35,L) * (  - 1./108.*m33*m55*
         RN3e*RSe45633p - 1./108.*m33*m55*RN3e*RSe463p - 1./36.*m33*m55
         *RN3e*RSe461p + 1./108.*m33*m55*RN13*RS14563pe + 1./108.*m33*
         m55*RN13*RS146pe + 1./18.*m33*m35*RN3p*RSp45613e + 1./54.*m33*
         m35*RN3p*RSp45633e - 1./54.*m33*m35*RN3p*RSp463e - 1./18.*m33*
         m35*RN3p*RSp461e + 1./18.*m33*m35*RN3e*RSe45613p + 1./54.*m33*
         m35*RN3e*RSe45633p - 1./54.*m33*m35*RN3e*RSe463p - 1./18.*m33*
         m35*RN3e*RSe461p - 1./54.*m33*m35*RN13*RS14563pe + 1./54.*m33*
         m35*RN13*RS146pe - 7./108.*pow(m33,2)*RN3p*RSp45613e + 1./36.*
         pow(m33,2)*RN3p*RSp45633e - 7./108.*pow(m33,2)*RN3p*RSp463e - 
         5./108.*pow(m33,2)*RN3p*RSp461e - 7./108.*pow(m33,2)*RN3e*
         RSe45613p + 1./36.*pow(m33,2)*RN3e*RSe45633p - 7./108.*pow(
         m33,2)*RN3e*RSe463p - 5./108.*pow(m33,2)*RN3e*RSe461p + 1./36.
         *pow(m33,2)*RN13*RS14563pe + 1./108.*pow(m33,2)*RN13*RS146pe
          + 1./27.*m15*RS34561pe - 1./108.*m15*RC3456pe - 1./54.*m15*
         RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m35,L) * (  - 11./540.*m13*
         RS34561pe + 1./27.*m13*RS14563pe - 1./135.*m13*RC3456pe + 19./
         540.*m13*RS346pe - 1./27.*m13*RS146pe - 1./54.*m13*m55*RN3p*
         RSp45613e + 1./54.*m13*m55*RN3p*RSp45633e + 1./54.*m13*m55*
         RN3p*RSp463e - 1./54.*m13*m55*RN3p*RSp461e - 1./54.*m13*m55*
         RN3e*RSe45613p + 1./54.*m13*m55*RN3e*RSe45633p + 1./54.*m13*
         m55*RN3e*RSe463p - 1./54.*m13*m55*RN3e*RSe461p + 1./54.*m13*
         m55*RN13*RS14563pe + 1./54.*m13*m55*RN13*RS146pe + 1./27.*m13*
         m35*RN3p*RSp45613e - 1./27.*m13*m35*RN3p*RSp45633e + 1./27.*
         m13*m35*RN3p*RSp463e - 1./27.*m13*m35*RN3p*RSp461e + 1./27.*
         m13*m35*RN3e*RSe45613p - 1./27.*m13*m35*RN3e*RSe45633p + 1./27.
         *m13*m35*RN3e*RSe463p - 1./27.*m13*m35*RN3e*RSe461p - 1./27.*
         m13*m35*RN13*RS14563pe + 1./27.*m13*m35*RN13*RS146pe - 1./54.*
         m13*m33*RN3p*RSp45613e + 1./54.*m13*m33*RN3p*RSp45633e + 1./18.
         *m13*m33*RN3p*RSp463e - 1./18.*m13*m33*RN3p*RSp461e - 1./54.*
         m13*m33*RN3e*RSe45613p );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m35,L) * ( 1./54.*m13*m33*RN3e*
         RSe45633p + 1./18.*m13*m33*RN3e*RSe463p - 1./18.*m13*m33*RN3e*
         RSe461p + 1./18.*m13*m33*RN13*RS14563pe + 1./54.*m13*m33*RN13*
         RS146pe + 1./90.*m11*RS34561pe + 1./54.*m11*RS14563pe + 1./120.
         *m11*RC3456pe - 1./20.*m11*RS346pe - 1./54.*m11*RS146pe - 1./
         108.*m11*m55*RN3p*RSp45613e - 1./108.*m11*m55*RN3p*RSp45633e
          - 1./108.*m11*m55*RN3p*RSp463e - 1./108.*m11*m55*RN3p*RSp461e
          - 1./108.*m11*m55*RN3e*RSe45613p - 1./108.*m11*m55*RN3e*
         RSe45633p - 1./108.*m11*m55*RN3e*RSe463p - 1./108.*m11*m55*
         RN3e*RSe461p + 1./36.*m11*m55*RN13*RS14563pe + 1./36.*m11*m55*
         RN13*RS146pe + 1./54.*m11*m35*RN3p*RSp45613e + 1./54.*m11*m35*
         RN3p*RSp45633e - 1./54.*m11*m35*RN3p*RSp463e - 1./54.*m11*m35*
         RN3p*RSp461e + 1./54.*m11*m35*RN3e*RSe45613p + 1./54.*m11*m35*
         RN3e*RSe45633p - 1./54.*m11*m35*RN3e*RSe463p - 1./54.*m11*m35*
         RN3e*RSe461p - 1./18.*m11*m35*RN13*RS14563pe + 1./18.*m11*m35*
         RN13*RS146pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m35,L) * (  - 1./108.*m11*m33*
         RN3p*RSp45613e - 1./108.*m11*m33*RN3p*RSp45633e - 1./36.*m11*
         m33*RN3p*RSp463e - 1./36.*m11*m33*RN3p*RSp461e - 1./108.*m11*
         m33*RN3e*RSe45613p - 1./108.*m11*m33*RN3e*RSe45633p - 1./36.*
         m11*m33*RN3e*RSe463p - 1./36.*m11*m33*RN3e*RSe461p + 1./36.*
         m11*m33*RN13*RS14563pe + 1./12.*m11*m33*RN13*RS146pe - 1./27.*
         m11*m13*RN13*RS14563pe + 1./27.*m11*m13*RN13*RS146pe - 1./54.*
         pow(m11,2)*RN13*RS14563pe + 1./54.*pow(m11,2)*RN13*RS146pe + 1.
         /27.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,
         m55,m66,m11,me2,mp2)*m11 );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m36,L) * (  - 29./540.*RD3456ep
          - 7./540.*m66*RS34561pe - 1./54.*m66*RS14563pe + 7./540.*m66*
         RC3456pe - 1./60.*m66*RS345pe - 1./54.*m66*RS145pe - 1./90.*
         m36*RS34561pe + 1./27.*m36*RS14563pe - 1./60.*m36*RC3456pe + 2.
         /45.*m36*RS345pe - 1./27.*m36*RS145pe + 43./540.*m33*RS34561pe
          + 1./27.*m33*RSe45613p + 1./27.*m33*RSp45613e + 13./1080.*m33
         *RC3456pe - 1./27.*m33*RSe45633p - 1./27.*m33*RSp45633e - 1./
         45.*m33*RS345pe + 1./27.*m33*RSe453p + 1./27.*m33*RSp453e - 2./
         27.*m33*RS145pe - 1./27.*m33*RSe451p - 1./27.*m33*RSp451e + 1./
         27.*m33*me2*RN3e*RSe45613p - 1./27.*m33*me2*RN3e*RSe45633p + 1.
         /27.*m33*me2*RN3e*RSe453p - 1./27.*m33*me2*RN3e*RSe451p + 1./
         27.*m33*mp2*RN3p*RSp45613e - 1./27.*m33*mp2*RN3p*RSp45633e + 1.
         /27.*m33*mp2*RN3p*RSp453e - 1./27.*m33*mp2*RN3p*RSp451e - 1./
         36.*m33*m66*RN3p*RSp45613e - 1./108.*m33*m66*RN3p*RSp45633e - 
         1./108.*m33*m66*RN3p*RSp453e - 1./36.*m33*m66*RN3p*RSp451e - 1.
         /36.*m33*m66*RN3e*RSe45613p );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m36,L) * (  - 1./108.*m33*m66*
         RN3e*RSe45633p - 1./108.*m33*m66*RN3e*RSe453p - 1./36.*m33*m66
         *RN3e*RSe451p + 1./108.*m33*m66*RN13*RS14563pe + 1./108.*m33*
         m66*RN13*RS145pe + 1./18.*m33*m36*RN3p*RSp45613e + 1./54.*m33*
         m36*RN3p*RSp45633e - 1./54.*m33*m36*RN3p*RSp453e - 1./18.*m33*
         m36*RN3p*RSp451e + 1./18.*m33*m36*RN3e*RSe45613p + 1./54.*m33*
         m36*RN3e*RSe45633p - 1./54.*m33*m36*RN3e*RSe453p - 1./18.*m33*
         m36*RN3e*RSe451p - 1./54.*m33*m36*RN13*RS14563pe + 1./54.*m33*
         m36*RN13*RS145pe - 7./108.*pow(m33,2)*RN3p*RSp45613e + 1./36.*
         pow(m33,2)*RN3p*RSp45633e - 7./108.*pow(m33,2)*RN3p*RSp453e - 
         5./108.*pow(m33,2)*RN3p*RSp451e - 7./108.*pow(m33,2)*RN3e*
         RSe45613p + 1./36.*pow(m33,2)*RN3e*RSe45633p - 7./108.*pow(
         m33,2)*RN3e*RSe453p - 5./108.*pow(m33,2)*RN3e*RSe451p + 1./36.
         *pow(m33,2)*RN13*RS14563pe + 1./108.*pow(m33,2)*RN13*RS145pe
          + 1./27.*m16*RS34561pe - 1./108.*m16*RC3456pe - 1./54.*m16*
         RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m36,L) * (  - 11./540.*m13*
         RS34561pe + 1./27.*m13*RS14563pe - 1./135.*m13*RC3456pe + 19./
         540.*m13*RS345pe - 1./27.*m13*RS145pe - 1./54.*m13*m66*RN3p*
         RSp45613e + 1./54.*m13*m66*RN3p*RSp45633e + 1./54.*m13*m66*
         RN3p*RSp453e - 1./54.*m13*m66*RN3p*RSp451e - 1./54.*m13*m66*
         RN3e*RSe45613p + 1./54.*m13*m66*RN3e*RSe45633p + 1./54.*m13*
         m66*RN3e*RSe453p - 1./54.*m13*m66*RN3e*RSe451p + 1./54.*m13*
         m66*RN13*RS14563pe + 1./54.*m13*m66*RN13*RS145pe + 1./27.*m13*
         m36*RN3p*RSp45613e - 1./27.*m13*m36*RN3p*RSp45633e + 1./27.*
         m13*m36*RN3p*RSp453e - 1./27.*m13*m36*RN3p*RSp451e + 1./27.*
         m13*m36*RN3e*RSe45613p - 1./27.*m13*m36*RN3e*RSe45633p + 1./27.
         *m13*m36*RN3e*RSe453p - 1./27.*m13*m36*RN3e*RSe451p - 1./27.*
         m13*m36*RN13*RS14563pe + 1./27.*m13*m36*RN13*RS145pe - 1./54.*
         m13*m33*RN3p*RSp45613e + 1./54.*m13*m33*RN3p*RSp45633e + 1./18.
         *m13*m33*RN3p*RSp453e - 1./18.*m13*m33*RN3p*RSp451e - 1./54.*
         m13*m33*RN3e*RSe45613p );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m36,L) * ( 1./54.*m13*m33*RN3e*
         RSe45633p + 1./18.*m13*m33*RN3e*RSe453p - 1./18.*m13*m33*RN3e*
         RSe451p + 1./18.*m13*m33*RN13*RS14563pe + 1./54.*m13*m33*RN13*
         RS145pe + 1./90.*m11*RS34561pe + 1./54.*m11*RS14563pe + 1./120.
         *m11*RC3456pe - 1./20.*m11*RS345pe - 1./54.*m11*RS145pe - 1./
         108.*m11*m66*RN3p*RSp45613e - 1./108.*m11*m66*RN3p*RSp45633e
          - 1./108.*m11*m66*RN3p*RSp453e - 1./108.*m11*m66*RN3p*RSp451e
          - 1./108.*m11*m66*RN3e*RSe45613p - 1./108.*m11*m66*RN3e*
         RSe45633p - 1./108.*m11*m66*RN3e*RSe453p - 1./108.*m11*m66*
         RN3e*RSe451p + 1./36.*m11*m66*RN13*RS14563pe + 1./36.*m11*m66*
         RN13*RS145pe + 1./54.*m11*m36*RN3p*RSp45613e + 1./54.*m11*m36*
         RN3p*RSp45633e - 1./54.*m11*m36*RN3p*RSp453e - 1./54.*m11*m36*
         RN3p*RSp451e + 1./54.*m11*m36*RN3e*RSe45613p + 1./54.*m11*m36*
         RN3e*RSe45633p - 1./54.*m11*m36*RN3e*RSe453p - 1./54.*m11*m36*
         RN3e*RSe451p - 1./18.*m11*m36*RN13*RS14563pe + 1./18.*m11*m36*
         RN13*RS145pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m36,L) * (  - 1./108.*m11*m33*
         RN3p*RSp45613e - 1./108.*m11*m33*RN3p*RSp45633e - 1./36.*m11*
         m33*RN3p*RSp453e - 1./36.*m11*m33*RN3p*RSp451e - 1./108.*m11*
         m33*RN3e*RSe45613p - 1./108.*m11*m33*RN3e*RSe45633p - 1./36.*
         m11*m33*RN3e*RSe453p - 1./36.*m11*m33*RN3e*RSe451p + 1./36.*
         m11*m33*RN13*RS14563pe + 1./12.*m11*m33*RN13*RS145pe - 1./27.*
         m11*m13*RN13*RS14563pe + 1./27.*m11*m13*RN13*RS145pe - 1./54.*
         pow(m11,2)*RN13*RS14563pe + 1./54.*pow(m11,2)*RN13*RS145pe + 1.
         /27.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,
         m55,m66,m11,me2,mp2)*m11 );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m44,L) * (  - 1./54.*m44*pow(
         RS356pe,2) + 1./27.*m44*RS156pe*RS356pe + 1./162.*m44*RS456pe*
         pow(RS356pe,2) - 1./81.*m44*RS456pe*RS156pe*RS356pe + 1./54.*
         m33*m44*RN3p*RSp563e*RS356pe - 1./162.*m33*m44*RN3p*RS456pe*
         RSp563e*RS356pe + 1./18.*m33*m44*RN3p*RSp561e*RS356pe - 1./54.
         *m33*m44*RN3p*RSp561e*RS456pe*RS356pe + 1./54.*m33*m44*RN3e*
         RSe563p*RS356pe - 1./162.*m33*m44*RN3e*RS456pe*RSe563p*RS356pe
          + 1./18.*m33*m44*RN3e*RSe561p*RS356pe - 1./54.*m33*m44*RN3e*
         RSe561p*RS456pe*RS356pe - 1./54.*m33*m44*RN13*RS156pe*RS356pe
          + 1./162.*m33*m44*RN13*RS456pe*RS156pe*RS356pe - 1./27.*m13*
         m44*RN3p*RSp563e*RS356pe + 1./81.*m13*m44*RN3p*RS456pe*RSp563e
         *RS356pe + 1./27.*m13*m44*RN3p*RSp561e*RS356pe - 1./81.*m13*
         m44*RN3p*RSp561e*RS456pe*RS356pe - 1./27.*m13*m44*RN3e*RSe563p
         *RS356pe + 1./81.*m13*m44*RN3e*RS456pe*RSe563p*RS356pe + 1./27.
         *m13*m44*RN3e*RSe561p*RS356pe - 1./81.*m13*m44*RN3e*RSe561p*
         RS456pe*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m44,L) * (  - 1./27.*m13*m44*
         RN13*RS156pe*RS356pe + 1./81.*m13*m44*RN13*RS456pe*RS156pe*
         RS356pe + 1./54.*m11*m44*RN3p*RSp563e*RS356pe - 1./162.*m11*
         m44*RN3p*RS456pe*RSp563e*RS356pe + 1./54.*m11*m44*RN3p*RSp561e
         *RS356pe - 1./162.*m11*m44*RN3p*RSp561e*RS456pe*RS356pe + 1./
         54.*m11*m44*RN3e*RSe563p*RS356pe - 1./162.*m11*m44*RN3e*
         RS456pe*RSe563p*RS356pe + 1./54.*m11*m44*RN3e*RSe561p*RS356pe
          - 1./162.*m11*m44*RN3e*RSe561p*RS456pe*RS356pe - 1./18.*m11*
         m44*RN13*RS156pe*RS356pe + 1./54.*m11*m44*RN13*RS456pe*RS156pe
         *RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * (  - 1./108.*m55*pow(
         RS346pe,2) - 1./162.*m55*RS356pe*RS346pe - 1./324.*m55*pow(
         RS356pe,2) + 1./54.*m55*RS146pe*RS346pe + 1./162.*m55*RS146pe*
         RS356pe + 1./162.*m55*RS156pe*RS346pe + 1./162.*m55*RS156pe*
         RS356pe + 1./162.*m45*pow(RS346pe,2) - 1./81.*m45*RS356pe*
         RS346pe + 1./162.*m45*pow(RS356pe,2) - 1./81.*m45*RS146pe*
         RS346pe + 1./81.*m45*RS146pe*RS356pe + 1./81.*m45*RS156pe*
         RS346pe - 1./81.*m45*RS156pe*RS356pe - 1./324.*m44*pow(
         RS346pe,2) - 1./162.*m44*RS356pe*RS346pe - 1./108.*m44*pow(
         RS356pe,2) + 1./162.*m44*RS146pe*RS346pe + 1./162.*m44*RS146pe
         *RS356pe + 1./162.*m44*RS156pe*RS346pe + 1./54.*m44*RS156pe*
         RS356pe + 1./324.*m33*pow(RS346pe,2) - 1./162.*m33*RS356pe*
         RS346pe + 1./324.*m33*pow(RS356pe,2) - 1./81.*m33*RSe563p*
         RS346pe + 1./81.*m33*RSe563p*RS356pe + 1./81.*m33*RSe463p*
         RS346pe - 1./81.*m33*RSe463p*RS356pe - 1./81.*m33*RSp563e*
         RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * ( 1./81.*m33*RSp563e*
         RS356pe + 1./81.*m33*RSp463e*RS346pe - 1./81.*m33*RSp463e*
         RS356pe - 1./54.*m33*RS146pe*RS346pe + 1./54.*m33*RS146pe*
         RS356pe + 1./54.*m33*RS156pe*RS346pe - 1./54.*m33*RS156pe*
         RS356pe + 1./81.*m33*RSe561p*RS346pe - 1./81.*m33*RSe561p*
         RS356pe - 1./81.*m33*RSe461p*RS346pe + 1./81.*m33*RSe461p*
         RS356pe + 1./81.*m33*RSp561e*RS346pe - 1./81.*m33*RSp561e*
         RS356pe - 1./81.*m33*RSp461e*RS346pe + 1./81.*m33*RSp461e*
         RS356pe - 1./81.*m33*me2*RN3e*RSe563p*RS346pe + 1./81.*m33*me2
         *RN3e*RSe563p*RS356pe + 1./81.*m33*me2*RN3e*RSe463p*RS346pe - 
         1./81.*m33*me2*RN3e*RSe463p*RS356pe + 1./81.*m33*me2*RN3e*
         RSe561p*RS346pe - 1./81.*m33*me2*RN3e*RSe561p*RS356pe - 1./81.
         *m33*me2*RN3e*RSe461p*RS346pe + 1./81.*m33*me2*RN3e*RSe461p*
         RS356pe - 1./81.*m33*mp2*RN3p*RSp563e*RS346pe + 1./81.*m33*mp2
         *RN3p*RSp563e*RS356pe + 1./81.*m33*mp2*RN3p*RSp463e*RS346pe - 
         1./81.*m33*mp2*RN3p*RSp463e*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * ( 1./81.*m33*mp2*RN3p*
         RSp561e*RS346pe - 1./81.*m33*mp2*RN3p*RSp561e*RS356pe - 1./81.
         *m33*mp2*RN3p*RSp461e*RS346pe + 1./81.*m33*mp2*RN3p*RSp461e*
         RS356pe + 1./324.*m33*m55*RN3p*RSp563e*RS346pe + 1./324.*m33*
         m55*RN3p*RSp563e*RS356pe + 1./108.*m33*m55*RN3p*RSp463e*
         RS346pe + 1./324.*m33*m55*RN3p*RSp463e*RS356pe + 1./108.*m33*
         m55*RN3p*RSp561e*RS346pe + 1./108.*m33*m55*RN3p*RSp561e*
         RS356pe + 1./36.*m33*m55*RN3p*RSp461e*RS346pe + 1./108.*m33*
         m55*RN3p*RSp461e*RS356pe + 1./324.*m33*m55*RN3e*RSe563p*
         RS346pe + 1./324.*m33*m55*RN3e*RSe563p*RS356pe + 1./108.*m33*
         m55*RN3e*RSe463p*RS346pe + 1./324.*m33*m55*RN3e*RSe463p*
         RS356pe + 1./108.*m33*m55*RN3e*RSe561p*RS346pe + 1./108.*m33*
         m55*RN3e*RSe561p*RS356pe + 1./36.*m33*m55*RN3e*RSe461p*RS346pe
          + 1./108.*m33*m55*RN3e*RSe461p*RS356pe - 1./108.*m33*m55*RN13
         *RS146pe*RS346pe - 1./324.*m33*m55*RN13*RS146pe*RS356pe - 1./
         324.*m33*m55*RN13*RS156pe*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * (  - 1./324.*m33*m55*
         RN13*RS156pe*RS356pe + 1./162.*m33*m45*RN3p*RSp563e*RS346pe - 
         1./162.*m33*m45*RN3p*RSp563e*RS356pe - 1./162.*m33*m45*RN3p*
         RSp463e*RS346pe + 1./162.*m33*m45*RN3p*RSp463e*RS356pe + 1./54.
         *m33*m45*RN3p*RSp561e*RS346pe - 1./54.*m33*m45*RN3p*RSp561e*
         RS356pe - 1./54.*m33*m45*RN3p*RSp461e*RS346pe + 1./54.*m33*m45
         *RN3p*RSp461e*RS356pe + 1./162.*m33*m45*RN3e*RSe563p*RS346pe
          - 1./162.*m33*m45*RN3e*RSe563p*RS356pe - 1./162.*m33*m45*RN3e
         *RSe463p*RS346pe + 1./162.*m33*m45*RN3e*RSe463p*RS356pe + 1./
         54.*m33*m45*RN3e*RSe561p*RS346pe - 1./54.*m33*m45*RN3e*RSe561p
         *RS356pe - 1./54.*m33*m45*RN3e*RSe461p*RS346pe + 1./54.*m33*
         m45*RN3e*RSe461p*RS356pe + 1./162.*m33*m45*RN13*RS146pe*
         RS346pe - 1./162.*m33*m45*RN13*RS146pe*RS356pe - 1./162.*m33*
         m45*RN13*RS156pe*RS346pe + 1./162.*m33*m45*RN13*RS156pe*
         RS356pe + 1./324.*m33*m44*RN3p*RSp563e*RS346pe + 1./108.*m33*
         m44*RN3p*RSp563e*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * ( 1./324.*m33*m44*RN3p*
         RSp463e*RS346pe + 1./324.*m33*m44*RN3p*RSp463e*RS356pe + 1./
         108.*m33*m44*RN3p*RSp561e*RS346pe + 1./36.*m33*m44*RN3p*
         RSp561e*RS356pe + 1./108.*m33*m44*RN3p*RSp461e*RS346pe + 1./
         108.*m33*m44*RN3p*RSp461e*RS356pe + 1./324.*m33*m44*RN3e*
         RSe563p*RS346pe + 1./108.*m33*m44*RN3e*RSe563p*RS356pe + 1./
         324.*m33*m44*RN3e*RSe463p*RS346pe + 1./324.*m33*m44*RN3e*
         RSe463p*RS356pe + 1./108.*m33*m44*RN3e*RSe561p*RS346pe + 1./36.
         *m33*m44*RN3e*RSe561p*RS356pe + 1./108.*m33*m44*RN3e*RSe461p*
         RS346pe + 1./108.*m33*m44*RN3e*RSe461p*RS356pe - 1./324.*m33*
         m44*RN13*RS146pe*RS346pe - 1./324.*m33*m44*RN13*RS146pe*
         RS356pe - 1./324.*m33*m44*RN13*RS156pe*RS346pe - 1./108.*m33*
         m44*RN13*RS156pe*RS356pe + 1./54.*pow(m33,2)*RN3p*RSp563e*
         RS346pe - 1./54.*pow(m33,2)*RN3p*RSp563e*RS356pe - 1./54.*pow(
         m33,2)*RN3p*RSp463e*RS346pe + 1./54.*pow(m33,2)*RN3p*RSp463e*
         RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * ( 1./162.*pow(m33,2)*
         RN3p*RSp561e*RS346pe - 1./162.*pow(m33,2)*RN3p*RSp561e*RS356pe
          - 1./162.*pow(m33,2)*RN3p*RSp461e*RS346pe + 1./162.*pow(
         m33,2)*RN3p*RSp461e*RS356pe + 1./54.*pow(m33,2)*RN3e*RSe563p*
         RS346pe - 1./54.*pow(m33,2)*RN3e*RSe563p*RS356pe - 1./54.*pow(
         m33,2)*RN3e*RSe463p*RS346pe + 1./54.*pow(m33,2)*RN3e*RSe463p*
         RS356pe + 1./162.*pow(m33,2)*RN3e*RSe561p*RS346pe - 1./162.*
         pow(m33,2)*RN3e*RSe561p*RS356pe - 1./162.*pow(m33,2)*RN3e*
         RSe461p*RS346pe + 1./162.*pow(m33,2)*RN3e*RSe461p*RS356pe + 1./
         162.*m13*pow(RS346pe,2) - 1./81.*m13*RS356pe*RS346pe + 1./162.
         *m13*pow(RS356pe,2) - 1./81.*m13*RS146pe*RS346pe + 1./81.*m13*
         RS146pe*RS356pe + 1./81.*m13*RS156pe*RS346pe - 1./81.*m13*
         RS156pe*RS356pe - 1./162.*m13*m55*RN3p*RSp563e*RS346pe - 1./
         162.*m13*m55*RN3p*RSp563e*RS356pe - 1./54.*m13*m55*RN3p*
         RSp463e*RS346pe - 1./162.*m13*m55*RN3p*RSp463e*RS356pe + 1./
         162.*m13*m55*RN3p*RSp561e*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * ( 1./162.*m13*m55*RN3p*
         RSp561e*RS356pe + 1./54.*m13*m55*RN3p*RSp461e*RS346pe + 1./162.
         *m13*m55*RN3p*RSp461e*RS356pe - 1./162.*m13*m55*RN3e*RSe563p*
         RS346pe - 1./162.*m13*m55*RN3e*RSe563p*RS356pe - 1./54.*m13*
         m55*RN3e*RSe463p*RS346pe - 1./162.*m13*m55*RN3e*RSe463p*
         RS356pe + 1./162.*m13*m55*RN3e*RSe561p*RS346pe + 1./162.*m13*
         m55*RN3e*RSe561p*RS356pe + 1./54.*m13*m55*RN3e*RSe461p*RS346pe
          + 1./162.*m13*m55*RN3e*RSe461p*RS356pe - 1./54.*m13*m55*RN13*
         RS146pe*RS346pe - 1./162.*m13*m55*RN13*RS146pe*RS356pe - 1./
         162.*m13*m55*RN13*RS156pe*RS346pe - 1./162.*m13*m55*RN13*
         RS156pe*RS356pe - 1./81.*m13*m45*RN3p*RSp563e*RS346pe + 1./81.
         *m13*m45*RN3p*RSp563e*RS356pe + 1./81.*m13*m45*RN3p*RSp463e*
         RS346pe - 1./81.*m13*m45*RN3p*RSp463e*RS356pe + 1./81.*m13*m45
         *RN3p*RSp561e*RS346pe - 1./81.*m13*m45*RN3p*RSp561e*RS356pe - 
         1./81.*m13*m45*RN3p*RSp461e*RS346pe + 1./81.*m13*m45*RN3p*
         RSp461e*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * (  - 1./81.*m13*m45*
         RN3e*RSe563p*RS346pe + 1./81.*m13*m45*RN3e*RSe563p*RS356pe + 1.
         /81.*m13*m45*RN3e*RSe463p*RS346pe - 1./81.*m13*m45*RN3e*
         RSe463p*RS356pe + 1./81.*m13*m45*RN3e*RSe561p*RS346pe - 1./81.
         *m13*m45*RN3e*RSe561p*RS356pe - 1./81.*m13*m45*RN3e*RSe461p*
         RS346pe + 1./81.*m13*m45*RN3e*RSe461p*RS356pe + 1./81.*m13*m45
         *RN13*RS146pe*RS346pe - 1./81.*m13*m45*RN13*RS146pe*RS356pe - 
         1./81.*m13*m45*RN13*RS156pe*RS346pe + 1./81.*m13*m45*RN13*
         RS156pe*RS356pe - 1./162.*m13*m44*RN3p*RSp563e*RS346pe - 1./54.
         *m13*m44*RN3p*RSp563e*RS356pe - 1./162.*m13*m44*RN3p*RSp463e*
         RS346pe - 1./162.*m13*m44*RN3p*RSp463e*RS356pe + 1./162.*m13*
         m44*RN3p*RSp561e*RS346pe + 1./54.*m13*m44*RN3p*RSp561e*RS356pe
          + 1./162.*m13*m44*RN3p*RSp461e*RS346pe + 1./162.*m13*m44*RN3p
         *RSp461e*RS356pe - 1./162.*m13*m44*RN3e*RSe563p*RS346pe - 1./
         54.*m13*m44*RN3e*RSe563p*RS356pe - 1./162.*m13*m44*RN3e*
         RSe463p*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * (  - 1./162.*m13*m44*
         RN3e*RSe463p*RS356pe + 1./162.*m13*m44*RN3e*RSe561p*RS346pe + 
         1./54.*m13*m44*RN3e*RSe561p*RS356pe + 1./162.*m13*m44*RN3e*
         RSe461p*RS346pe + 1./162.*m13*m44*RN3e*RSe461p*RS356pe - 1./
         162.*m13*m44*RN13*RS146pe*RS346pe - 1./162.*m13*m44*RN13*
         RS146pe*RS356pe - 1./162.*m13*m44*RN13*RS156pe*RS346pe - 1./54.
         *m13*m44*RN13*RS156pe*RS356pe - 1./81.*m13*m33*RN3p*RSp563e*
         RS346pe + 1./81.*m13*m33*RN3p*RSp563e*RS356pe + 1./81.*m13*m33
         *RN3p*RSp463e*RS346pe - 1./81.*m13*m33*RN3p*RSp463e*RS356pe + 
         1./81.*m13*m33*RN3p*RSp561e*RS346pe - 1./81.*m13*m33*RN3p*
         RSp561e*RS356pe - 1./81.*m13*m33*RN3p*RSp461e*RS346pe + 1./81.
         *m13*m33*RN3p*RSp461e*RS356pe - 1./81.*m13*m33*RN3e*RSe563p*
         RS346pe + 1./81.*m13*m33*RN3e*RSe563p*RS356pe + 1./81.*m13*m33
         *RN3e*RSe463p*RS346pe - 1./81.*m13*m33*RN3e*RSe463p*RS356pe + 
         1./81.*m13*m33*RN3e*RSe561p*RS346pe - 1./81.*m13*m33*RN3e*
         RSe561p*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * (  - 1./81.*m13*m33*
         RN3e*RSe461p*RS346pe + 1./81.*m13*m33*RN3e*RSe461p*RS356pe - 1.
         /324.*m11*pow(RS346pe,2) + 1./162.*m11*RS356pe*RS346pe - 1./
         324.*m11*pow(RS356pe,2) - 1./162.*m11*RS146pe*RS346pe + 1./162.
         *m11*RS146pe*RS356pe + 1./162.*m11*RS156pe*RS346pe - 1./162.*
         m11*RS156pe*RS356pe + 1./324.*m11*m55*RN3p*RSp563e*RS346pe + 1.
         /324.*m11*m55*RN3p*RSp563e*RS356pe + 1./108.*m11*m55*RN3p*
         RSp463e*RS346pe + 1./324.*m11*m55*RN3p*RSp463e*RS356pe + 1./
         324.*m11*m55*RN3p*RSp561e*RS346pe + 1./324.*m11*m55*RN3p*
         RSp561e*RS356pe + 1./108.*m11*m55*RN3p*RSp461e*RS346pe + 1./
         324.*m11*m55*RN3p*RSp461e*RS356pe + 1./324.*m11*m55*RN3e*
         RSe563p*RS346pe + 1./324.*m11*m55*RN3e*RSe563p*RS356pe + 1./
         108.*m11*m55*RN3e*RSe463p*RS346pe + 1./324.*m11*m55*RN3e*
         RSe463p*RS356pe + 1./324.*m11*m55*RN3e*RSe561p*RS346pe + 1./
         324.*m11*m55*RN3e*RSe561p*RS356pe + 1./108.*m11*m55*RN3e*
         RSe461p*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * ( 1./324.*m11*m55*RN3e*
         RSe461p*RS356pe - 1./36.*m11*m55*RN13*RS146pe*RS346pe - 1./108.
         *m11*m55*RN13*RS146pe*RS356pe - 1./108.*m11*m55*RN13*RS156pe*
         RS346pe - 1./108.*m11*m55*RN13*RS156pe*RS356pe + 1./162.*m11*
         m45*RN3p*RSp563e*RS346pe - 1./162.*m11*m45*RN3p*RSp563e*
         RS356pe - 1./162.*m11*m45*RN3p*RSp463e*RS346pe + 1./162.*m11*
         m45*RN3p*RSp463e*RS356pe + 1./162.*m11*m45*RN3p*RSp561e*
         RS346pe - 1./162.*m11*m45*RN3p*RSp561e*RS356pe - 1./162.*m11*
         m45*RN3p*RSp461e*RS346pe + 1./162.*m11*m45*RN3p*RSp461e*
         RS356pe + 1./162.*m11*m45*RN3e*RSe563p*RS346pe - 1./162.*m11*
         m45*RN3e*RSe563p*RS356pe - 1./162.*m11*m45*RN3e*RSe463p*
         RS346pe + 1./162.*m11*m45*RN3e*RSe463p*RS356pe + 1./162.*m11*
         m45*RN3e*RSe561p*RS346pe - 1./162.*m11*m45*RN3e*RSe561p*
         RS356pe - 1./162.*m11*m45*RN3e*RSe461p*RS346pe + 1./162.*m11*
         m45*RN3e*RSe461p*RS356pe + 1./54.*m11*m45*RN13*RS146pe*RS346pe
          - 1./54.*m11*m45*RN13*RS146pe*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * (  - 1./54.*m11*m45*
         RN13*RS156pe*RS346pe + 1./54.*m11*m45*RN13*RS156pe*RS356pe + 1.
         /324.*m11*m44*RN3p*RSp563e*RS346pe + 1./108.*m11*m44*RN3p*
         RSp563e*RS356pe + 1./324.*m11*m44*RN3p*RSp463e*RS346pe + 1./
         324.*m11*m44*RN3p*RSp463e*RS356pe + 1./324.*m11*m44*RN3p*
         RSp561e*RS346pe + 1./108.*m11*m44*RN3p*RSp561e*RS356pe + 1./
         324.*m11*m44*RN3p*RSp461e*RS346pe + 1./324.*m11*m44*RN3p*
         RSp461e*RS356pe + 1./324.*m11*m44*RN3e*RSe563p*RS346pe + 1./
         108.*m11*m44*RN3e*RSe563p*RS356pe + 1./324.*m11*m44*RN3e*
         RSe463p*RS346pe + 1./324.*m11*m44*RN3e*RSe463p*RS356pe + 1./
         324.*m11*m44*RN3e*RSe561p*RS346pe + 1./108.*m11*m44*RN3e*
         RSe561p*RS356pe + 1./324.*m11*m44*RN3e*RSe461p*RS346pe + 1./
         324.*m11*m44*RN3e*RSe461p*RS356pe - 1./108.*m11*m44*RN13*
         RS146pe*RS346pe - 1./108.*m11*m44*RN13*RS146pe*RS356pe - 1./
         108.*m11*m44*RN13*RS156pe*RS346pe - 1./36.*m11*m44*RN13*
         RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * ( 1./162.*m11*m33*RN3p*
         RSp563e*RS346pe - 1./162.*m11*m33*RN3p*RSp563e*RS356pe - 1./
         162.*m11*m33*RN3p*RSp463e*RS346pe + 1./162.*m11*m33*RN3p*
         RSp463e*RS356pe + 1./162.*m11*m33*RN3p*RSp561e*RS346pe - 1./
         162.*m11*m33*RN3p*RSp561e*RS356pe - 1./162.*m11*m33*RN3p*
         RSp461e*RS346pe + 1./162.*m11*m33*RN3p*RSp461e*RS356pe + 1./
         162.*m11*m33*RN3e*RSe563p*RS346pe - 1./162.*m11*m33*RN3e*
         RSe563p*RS356pe - 1./162.*m11*m33*RN3e*RSe463p*RS346pe + 1./
         162.*m11*m33*RN3e*RSe463p*RS356pe + 1./162.*m11*m33*RN3e*
         RSe561p*RS346pe - 1./162.*m11*m33*RN3e*RSe561p*RS356pe - 1./
         162.*m11*m33*RN3e*RSe461p*RS346pe + 1./162.*m11*m33*RN3e*
         RSe461p*RS356pe + 1./54.*m11*m33*RN13*RS146pe*RS346pe - 1./54.
         *m11*m33*RN13*RS146pe*RS356pe - 1./54.*m11*m33*RN13*RS156pe*
         RS346pe + 1./54.*m11*m33*RN13*RS156pe*RS356pe + 1./81.*m11*m13
         *RN13*RS146pe*RS346pe - 1./81.*m11*m13*RN13*RS146pe*RS356pe - 
         1./81.*m11*m13*RN13*RS156pe*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * ( 1./81.*m11*m13*RN13*
         RS156pe*RS356pe + 1./162.*pow(m11,2)*RN13*RS146pe*RS346pe - 1./
         162.*pow(m11,2)*RN13*RS146pe*RS356pe - 1./162.*pow(m11,2)*RN13
         *RS156pe*RS346pe + 1./162.*pow(m11,2)*RN13*RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * (  - 1./108.*m66*pow(
         RS345pe,2) - 1./162.*m66*RS356pe*RS345pe - 1./324.*m66*pow(
         RS356pe,2) + 1./54.*m66*RS145pe*RS345pe + 1./162.*m66*RS145pe*
         RS356pe + 1./162.*m66*RS156pe*RS345pe + 1./162.*m66*RS156pe*
         RS356pe + 1./162.*m46*pow(RS345pe,2) - 1./81.*m46*RS356pe*
         RS345pe + 1./162.*m46*pow(RS356pe,2) - 1./81.*m46*RS145pe*
         RS345pe + 1./81.*m46*RS145pe*RS356pe + 1./81.*m46*RS156pe*
         RS345pe - 1./81.*m46*RS156pe*RS356pe - 1./324.*m44*pow(
         RS345pe,2) - 1./162.*m44*RS356pe*RS345pe - 1./108.*m44*pow(
         RS356pe,2) + 1./162.*m44*RS145pe*RS345pe + 1./162.*m44*RS145pe
         *RS356pe + 1./162.*m44*RS156pe*RS345pe + 1./54.*m44*RS156pe*
         RS356pe + 1./324.*m33*pow(RS345pe,2) - 1./162.*m33*RS356pe*
         RS345pe + 1./324.*m33*pow(RS356pe,2) - 1./81.*m33*RSe563p*
         RS345pe + 1./81.*m33*RSe563p*RS356pe + 1./81.*m33*RSe453p*
         RS345pe - 1./81.*m33*RSe453p*RS356pe - 1./81.*m33*RSp563e*
         RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * ( 1./81.*m33*RSp563e*
         RS356pe + 1./81.*m33*RSp453e*RS345pe - 1./81.*m33*RSp453e*
         RS356pe - 1./54.*m33*RS145pe*RS345pe + 1./54.*m33*RS145pe*
         RS356pe + 1./54.*m33*RS156pe*RS345pe - 1./54.*m33*RS156pe*
         RS356pe + 1./81.*m33*RSe561p*RS345pe - 1./81.*m33*RSe561p*
         RS356pe - 1./81.*m33*RSe451p*RS345pe + 1./81.*m33*RSe451p*
         RS356pe + 1./81.*m33*RSp561e*RS345pe - 1./81.*m33*RSp561e*
         RS356pe - 1./81.*m33*RSp451e*RS345pe + 1./81.*m33*RSp451e*
         RS356pe - 1./81.*m33*me2*RN3e*RSe563p*RS345pe + 1./81.*m33*me2
         *RN3e*RSe563p*RS356pe + 1./81.*m33*me2*RN3e*RSe453p*RS345pe - 
         1./81.*m33*me2*RN3e*RSe453p*RS356pe + 1./81.*m33*me2*RN3e*
         RSe561p*RS345pe - 1./81.*m33*me2*RN3e*RSe561p*RS356pe - 1./81.
         *m33*me2*RN3e*RSe451p*RS345pe + 1./81.*m33*me2*RN3e*RSe451p*
         RS356pe - 1./81.*m33*mp2*RN3p*RSp563e*RS345pe + 1./81.*m33*mp2
         *RN3p*RSp563e*RS356pe + 1./81.*m33*mp2*RN3p*RSp453e*RS345pe - 
         1./81.*m33*mp2*RN3p*RSp453e*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * ( 1./81.*m33*mp2*RN3p*
         RSp561e*RS345pe - 1./81.*m33*mp2*RN3p*RSp561e*RS356pe - 1./81.
         *m33*mp2*RN3p*RSp451e*RS345pe + 1./81.*m33*mp2*RN3p*RSp451e*
         RS356pe + 1./324.*m33*m66*RN3p*RSp563e*RS345pe + 1./324.*m33*
         m66*RN3p*RSp563e*RS356pe + 1./108.*m33*m66*RN3p*RSp453e*
         RS345pe + 1./324.*m33*m66*RN3p*RSp453e*RS356pe + 1./108.*m33*
         m66*RN3p*RSp561e*RS345pe + 1./108.*m33*m66*RN3p*RSp561e*
         RS356pe + 1./36.*m33*m66*RN3p*RSp451e*RS345pe + 1./108.*m33*
         m66*RN3p*RSp451e*RS356pe + 1./324.*m33*m66*RN3e*RSe563p*
         RS345pe + 1./324.*m33*m66*RN3e*RSe563p*RS356pe + 1./108.*m33*
         m66*RN3e*RSe453p*RS345pe + 1./324.*m33*m66*RN3e*RSe453p*
         RS356pe + 1./108.*m33*m66*RN3e*RSe561p*RS345pe + 1./108.*m33*
         m66*RN3e*RSe561p*RS356pe + 1./36.*m33*m66*RN3e*RSe451p*RS345pe
          + 1./108.*m33*m66*RN3e*RSe451p*RS356pe - 1./108.*m33*m66*RN13
         *RS145pe*RS345pe - 1./324.*m33*m66*RN13*RS145pe*RS356pe - 1./
         324.*m33*m66*RN13*RS156pe*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * (  - 1./324.*m33*m66*
         RN13*RS156pe*RS356pe + 1./162.*m33*m46*RN3p*RSp563e*RS345pe - 
         1./162.*m33*m46*RN3p*RSp563e*RS356pe - 1./162.*m33*m46*RN3p*
         RSp453e*RS345pe + 1./162.*m33*m46*RN3p*RSp453e*RS356pe + 1./54.
         *m33*m46*RN3p*RSp561e*RS345pe - 1./54.*m33*m46*RN3p*RSp561e*
         RS356pe - 1./54.*m33*m46*RN3p*RSp451e*RS345pe + 1./54.*m33*m46
         *RN3p*RSp451e*RS356pe + 1./162.*m33*m46*RN3e*RSe563p*RS345pe
          - 1./162.*m33*m46*RN3e*RSe563p*RS356pe - 1./162.*m33*m46*RN3e
         *RSe453p*RS345pe + 1./162.*m33*m46*RN3e*RSe453p*RS356pe + 1./
         54.*m33*m46*RN3e*RSe561p*RS345pe - 1./54.*m33*m46*RN3e*RSe561p
         *RS356pe - 1./54.*m33*m46*RN3e*RSe451p*RS345pe + 1./54.*m33*
         m46*RN3e*RSe451p*RS356pe + 1./162.*m33*m46*RN13*RS145pe*
         RS345pe - 1./162.*m33*m46*RN13*RS145pe*RS356pe - 1./162.*m33*
         m46*RN13*RS156pe*RS345pe + 1./162.*m33*m46*RN13*RS156pe*
         RS356pe + 1./324.*m33*m44*RN3p*RSp563e*RS345pe + 1./108.*m33*
         m44*RN3p*RSp563e*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * ( 1./324.*m33*m44*RN3p*
         RSp453e*RS345pe + 1./324.*m33*m44*RN3p*RSp453e*RS356pe + 1./
         108.*m33*m44*RN3p*RSp561e*RS345pe + 1./36.*m33*m44*RN3p*
         RSp561e*RS356pe + 1./108.*m33*m44*RN3p*RSp451e*RS345pe + 1./
         108.*m33*m44*RN3p*RSp451e*RS356pe + 1./324.*m33*m44*RN3e*
         RSe563p*RS345pe + 1./108.*m33*m44*RN3e*RSe563p*RS356pe + 1./
         324.*m33*m44*RN3e*RSe453p*RS345pe + 1./324.*m33*m44*RN3e*
         RSe453p*RS356pe + 1./108.*m33*m44*RN3e*RSe561p*RS345pe + 1./36.
         *m33*m44*RN3e*RSe561p*RS356pe + 1./108.*m33*m44*RN3e*RSe451p*
         RS345pe + 1./108.*m33*m44*RN3e*RSe451p*RS356pe - 1./324.*m33*
         m44*RN13*RS145pe*RS345pe - 1./324.*m33*m44*RN13*RS145pe*
         RS356pe - 1./324.*m33*m44*RN13*RS156pe*RS345pe - 1./108.*m33*
         m44*RN13*RS156pe*RS356pe + 1./54.*pow(m33,2)*RN3p*RSp563e*
         RS345pe - 1./54.*pow(m33,2)*RN3p*RSp563e*RS356pe - 1./54.*pow(
         m33,2)*RN3p*RSp453e*RS345pe + 1./54.*pow(m33,2)*RN3p*RSp453e*
         RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * ( 1./162.*pow(m33,2)*
         RN3p*RSp561e*RS345pe - 1./162.*pow(m33,2)*RN3p*RSp561e*RS356pe
          - 1./162.*pow(m33,2)*RN3p*RSp451e*RS345pe + 1./162.*pow(
         m33,2)*RN3p*RSp451e*RS356pe + 1./54.*pow(m33,2)*RN3e*RSe563p*
         RS345pe - 1./54.*pow(m33,2)*RN3e*RSe563p*RS356pe - 1./54.*pow(
         m33,2)*RN3e*RSe453p*RS345pe + 1./54.*pow(m33,2)*RN3e*RSe453p*
         RS356pe + 1./162.*pow(m33,2)*RN3e*RSe561p*RS345pe - 1./162.*
         pow(m33,2)*RN3e*RSe561p*RS356pe - 1./162.*pow(m33,2)*RN3e*
         RSe451p*RS345pe + 1./162.*pow(m33,2)*RN3e*RSe451p*RS356pe + 1./
         162.*m13*pow(RS345pe,2) - 1./81.*m13*RS356pe*RS345pe + 1./162.
         *m13*pow(RS356pe,2) - 1./81.*m13*RS145pe*RS345pe + 1./81.*m13*
         RS145pe*RS356pe + 1./81.*m13*RS156pe*RS345pe - 1./81.*m13*
         RS156pe*RS356pe - 1./162.*m13*m66*RN3p*RSp563e*RS345pe - 1./
         162.*m13*m66*RN3p*RSp563e*RS356pe - 1./54.*m13*m66*RN3p*
         RSp453e*RS345pe - 1./162.*m13*m66*RN3p*RSp453e*RS356pe + 1./
         162.*m13*m66*RN3p*RSp561e*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * ( 1./162.*m13*m66*RN3p*
         RSp561e*RS356pe + 1./54.*m13*m66*RN3p*RSp451e*RS345pe + 1./162.
         *m13*m66*RN3p*RSp451e*RS356pe - 1./162.*m13*m66*RN3e*RSe563p*
         RS345pe - 1./162.*m13*m66*RN3e*RSe563p*RS356pe - 1./54.*m13*
         m66*RN3e*RSe453p*RS345pe - 1./162.*m13*m66*RN3e*RSe453p*
         RS356pe + 1./162.*m13*m66*RN3e*RSe561p*RS345pe + 1./162.*m13*
         m66*RN3e*RSe561p*RS356pe + 1./54.*m13*m66*RN3e*RSe451p*RS345pe
          + 1./162.*m13*m66*RN3e*RSe451p*RS356pe - 1./54.*m13*m66*RN13*
         RS145pe*RS345pe - 1./162.*m13*m66*RN13*RS145pe*RS356pe - 1./
         162.*m13*m66*RN13*RS156pe*RS345pe - 1./162.*m13*m66*RN13*
         RS156pe*RS356pe - 1./81.*m13*m46*RN3p*RSp563e*RS345pe + 1./81.
         *m13*m46*RN3p*RSp563e*RS356pe + 1./81.*m13*m46*RN3p*RSp453e*
         RS345pe - 1./81.*m13*m46*RN3p*RSp453e*RS356pe + 1./81.*m13*m46
         *RN3p*RSp561e*RS345pe - 1./81.*m13*m46*RN3p*RSp561e*RS356pe - 
         1./81.*m13*m46*RN3p*RSp451e*RS345pe + 1./81.*m13*m46*RN3p*
         RSp451e*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * (  - 1./81.*m13*m46*
         RN3e*RSe563p*RS345pe + 1./81.*m13*m46*RN3e*RSe563p*RS356pe + 1.
         /81.*m13*m46*RN3e*RSe453p*RS345pe - 1./81.*m13*m46*RN3e*
         RSe453p*RS356pe + 1./81.*m13*m46*RN3e*RSe561p*RS345pe - 1./81.
         *m13*m46*RN3e*RSe561p*RS356pe - 1./81.*m13*m46*RN3e*RSe451p*
         RS345pe + 1./81.*m13*m46*RN3e*RSe451p*RS356pe + 1./81.*m13*m46
         *RN13*RS145pe*RS345pe - 1./81.*m13*m46*RN13*RS145pe*RS356pe - 
         1./81.*m13*m46*RN13*RS156pe*RS345pe + 1./81.*m13*m46*RN13*
         RS156pe*RS356pe - 1./162.*m13*m44*RN3p*RSp563e*RS345pe - 1./54.
         *m13*m44*RN3p*RSp563e*RS356pe - 1./162.*m13*m44*RN3p*RSp453e*
         RS345pe - 1./162.*m13*m44*RN3p*RSp453e*RS356pe + 1./162.*m13*
         m44*RN3p*RSp561e*RS345pe + 1./54.*m13*m44*RN3p*RSp561e*RS356pe
          + 1./162.*m13*m44*RN3p*RSp451e*RS345pe + 1./162.*m13*m44*RN3p
         *RSp451e*RS356pe - 1./162.*m13*m44*RN3e*RSe563p*RS345pe - 1./
         54.*m13*m44*RN3e*RSe563p*RS356pe - 1./162.*m13*m44*RN3e*
         RSe453p*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * (  - 1./162.*m13*m44*
         RN3e*RSe453p*RS356pe + 1./162.*m13*m44*RN3e*RSe561p*RS345pe + 
         1./54.*m13*m44*RN3e*RSe561p*RS356pe + 1./162.*m13*m44*RN3e*
         RSe451p*RS345pe + 1./162.*m13*m44*RN3e*RSe451p*RS356pe - 1./
         162.*m13*m44*RN13*RS145pe*RS345pe - 1./162.*m13*m44*RN13*
         RS145pe*RS356pe - 1./162.*m13*m44*RN13*RS156pe*RS345pe - 1./54.
         *m13*m44*RN13*RS156pe*RS356pe - 1./81.*m13*m33*RN3p*RSp563e*
         RS345pe + 1./81.*m13*m33*RN3p*RSp563e*RS356pe + 1./81.*m13*m33
         *RN3p*RSp453e*RS345pe - 1./81.*m13*m33*RN3p*RSp453e*RS356pe + 
         1./81.*m13*m33*RN3p*RSp561e*RS345pe - 1./81.*m13*m33*RN3p*
         RSp561e*RS356pe - 1./81.*m13*m33*RN3p*RSp451e*RS345pe + 1./81.
         *m13*m33*RN3p*RSp451e*RS356pe - 1./81.*m13*m33*RN3e*RSe563p*
         RS345pe + 1./81.*m13*m33*RN3e*RSe563p*RS356pe + 1./81.*m13*m33
         *RN3e*RSe453p*RS345pe - 1./81.*m13*m33*RN3e*RSe453p*RS356pe + 
         1./81.*m13*m33*RN3e*RSe561p*RS345pe - 1./81.*m13*m33*RN3e*
         RSe561p*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * (  - 1./81.*m13*m33*
         RN3e*RSe451p*RS345pe + 1./81.*m13*m33*RN3e*RSe451p*RS356pe - 1.
         /324.*m11*pow(RS345pe,2) + 1./162.*m11*RS356pe*RS345pe - 1./
         324.*m11*pow(RS356pe,2) - 1./162.*m11*RS145pe*RS345pe + 1./162.
         *m11*RS145pe*RS356pe + 1./162.*m11*RS156pe*RS345pe - 1./162.*
         m11*RS156pe*RS356pe + 1./324.*m11*m66*RN3p*RSp563e*RS345pe + 1.
         /324.*m11*m66*RN3p*RSp563e*RS356pe + 1./108.*m11*m66*RN3p*
         RSp453e*RS345pe + 1./324.*m11*m66*RN3p*RSp453e*RS356pe + 1./
         324.*m11*m66*RN3p*RSp561e*RS345pe + 1./324.*m11*m66*RN3p*
         RSp561e*RS356pe + 1./108.*m11*m66*RN3p*RSp451e*RS345pe + 1./
         324.*m11*m66*RN3p*RSp451e*RS356pe + 1./324.*m11*m66*RN3e*
         RSe563p*RS345pe + 1./324.*m11*m66*RN3e*RSe563p*RS356pe + 1./
         108.*m11*m66*RN3e*RSe453p*RS345pe + 1./324.*m11*m66*RN3e*
         RSe453p*RS356pe + 1./324.*m11*m66*RN3e*RSe561p*RS345pe + 1./
         324.*m11*m66*RN3e*RSe561p*RS356pe + 1./108.*m11*m66*RN3e*
         RSe451p*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * ( 1./324.*m11*m66*RN3e*
         RSe451p*RS356pe - 1./36.*m11*m66*RN13*RS145pe*RS345pe - 1./108.
         *m11*m66*RN13*RS145pe*RS356pe - 1./108.*m11*m66*RN13*RS156pe*
         RS345pe - 1./108.*m11*m66*RN13*RS156pe*RS356pe + 1./162.*m11*
         m46*RN3p*RSp563e*RS345pe - 1./162.*m11*m46*RN3p*RSp563e*
         RS356pe - 1./162.*m11*m46*RN3p*RSp453e*RS345pe + 1./162.*m11*
         m46*RN3p*RSp453e*RS356pe + 1./162.*m11*m46*RN3p*RSp561e*
         RS345pe - 1./162.*m11*m46*RN3p*RSp561e*RS356pe - 1./162.*m11*
         m46*RN3p*RSp451e*RS345pe + 1./162.*m11*m46*RN3p*RSp451e*
         RS356pe + 1./162.*m11*m46*RN3e*RSe563p*RS345pe - 1./162.*m11*
         m46*RN3e*RSe563p*RS356pe - 1./162.*m11*m46*RN3e*RSe453p*
         RS345pe + 1./162.*m11*m46*RN3e*RSe453p*RS356pe + 1./162.*m11*
         m46*RN3e*RSe561p*RS345pe - 1./162.*m11*m46*RN3e*RSe561p*
         RS356pe - 1./162.*m11*m46*RN3e*RSe451p*RS345pe + 1./162.*m11*
         m46*RN3e*RSe451p*RS356pe + 1./54.*m11*m46*RN13*RS145pe*RS345pe
          - 1./54.*m11*m46*RN13*RS145pe*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * (  - 1./54.*m11*m46*
         RN13*RS156pe*RS345pe + 1./54.*m11*m46*RN13*RS156pe*RS356pe + 1.
         /324.*m11*m44*RN3p*RSp563e*RS345pe + 1./108.*m11*m44*RN3p*
         RSp563e*RS356pe + 1./324.*m11*m44*RN3p*RSp453e*RS345pe + 1./
         324.*m11*m44*RN3p*RSp453e*RS356pe + 1./324.*m11*m44*RN3p*
         RSp561e*RS345pe + 1./108.*m11*m44*RN3p*RSp561e*RS356pe + 1./
         324.*m11*m44*RN3p*RSp451e*RS345pe + 1./324.*m11*m44*RN3p*
         RSp451e*RS356pe + 1./324.*m11*m44*RN3e*RSe563p*RS345pe + 1./
         108.*m11*m44*RN3e*RSe563p*RS356pe + 1./324.*m11*m44*RN3e*
         RSe453p*RS345pe + 1./324.*m11*m44*RN3e*RSe453p*RS356pe + 1./
         324.*m11*m44*RN3e*RSe561p*RS345pe + 1./108.*m11*m44*RN3e*
         RSe561p*RS356pe + 1./324.*m11*m44*RN3e*RSe451p*RS345pe + 1./
         324.*m11*m44*RN3e*RSe451p*RS356pe - 1./108.*m11*m44*RN13*
         RS145pe*RS345pe - 1./108.*m11*m44*RN13*RS145pe*RS356pe - 1./
         108.*m11*m44*RN13*RS156pe*RS345pe - 1./36.*m11*m44*RN13*
         RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * ( 1./162.*m11*m33*RN3p*
         RSp563e*RS345pe - 1./162.*m11*m33*RN3p*RSp563e*RS356pe - 1./
         162.*m11*m33*RN3p*RSp453e*RS345pe + 1./162.*m11*m33*RN3p*
         RSp453e*RS356pe + 1./162.*m11*m33*RN3p*RSp561e*RS345pe - 1./
         162.*m11*m33*RN3p*RSp561e*RS356pe - 1./162.*m11*m33*RN3p*
         RSp451e*RS345pe + 1./162.*m11*m33*RN3p*RSp451e*RS356pe + 1./
         162.*m11*m33*RN3e*RSe563p*RS345pe - 1./162.*m11*m33*RN3e*
         RSe563p*RS356pe - 1./162.*m11*m33*RN3e*RSe453p*RS345pe + 1./
         162.*m11*m33*RN3e*RSe453p*RS356pe + 1./162.*m11*m33*RN3e*
         RSe561p*RS345pe - 1./162.*m11*m33*RN3e*RSe561p*RS356pe - 1./
         162.*m11*m33*RN3e*RSe451p*RS345pe + 1./162.*m11*m33*RN3e*
         RSe451p*RS356pe + 1./54.*m11*m33*RN13*RS145pe*RS345pe - 1./54.
         *m11*m33*RN13*RS145pe*RS356pe - 1./54.*m11*m33*RN13*RS156pe*
         RS345pe + 1./54.*m11*m33*RN13*RS156pe*RS356pe + 1./81.*m11*m13
         *RN13*RS145pe*RS345pe - 1./81.*m11*m13*RN13*RS145pe*RS356pe - 
         1./81.*m11*m13*RN13*RS156pe*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * ( 1./81.*m11*m13*RN13*
         RS156pe*RS356pe + 1./162.*pow(m11,2)*RN13*RS145pe*RS345pe - 1./
         162.*pow(m11,2)*RN13*RS145pe*RS356pe - 1./162.*pow(m11,2)*RN13
         *RS156pe*RS345pe + 1./162.*pow(m11,2)*RN13*RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m55,L) * (  - 1./54.*m55*pow(
         RS346pe,2) + 1./27.*m55*RS146pe*RS346pe + 1./162.*m55*RS546pe*
         pow(RS346pe,2) - 1./81.*m55*RS546pe*RS146pe*RS346pe + 1./54.*
         m33*m55*RN3p*RSp463e*RS346pe - 1./162.*m33*m55*RN3p*RS546pe*
         RSp463e*RS346pe + 1./18.*m33*m55*RN3p*RSp461e*RS346pe - 1./54.
         *m33*m55*RN3p*RSp461e*RS546pe*RS346pe + 1./54.*m33*m55*RN3e*
         RSe463p*RS346pe - 1./162.*m33*m55*RN3e*RS546pe*RSe463p*RS346pe
          + 1./18.*m33*m55*RN3e*RSe461p*RS346pe - 1./54.*m33*m55*RN3e*
         RSe461p*RS546pe*RS346pe - 1./54.*m33*m55*RN13*RS146pe*RS346pe
          + 1./162.*m33*m55*RN13*RS546pe*RS146pe*RS346pe - 1./27.*m13*
         m55*RN3p*RSp463e*RS346pe + 1./81.*m13*m55*RN3p*RS546pe*RSp463e
         *RS346pe + 1./27.*m13*m55*RN3p*RSp461e*RS346pe - 1./81.*m13*
         m55*RN3p*RSp461e*RS546pe*RS346pe - 1./27.*m13*m55*RN3e*RSe463p
         *RS346pe + 1./81.*m13*m55*RN3e*RS546pe*RSe463p*RS346pe + 1./27.
         *m13*m55*RN3e*RSe461p*RS346pe - 1./81.*m13*m55*RN3e*RSe461p*
         RS546pe*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m55,L) * (  - 1./27.*m13*m55*
         RN13*RS146pe*RS346pe + 1./81.*m13*m55*RN13*RS546pe*RS146pe*
         RS346pe + 1./54.*m11*m55*RN3p*RSp463e*RS346pe - 1./162.*m11*
         m55*RN3p*RS546pe*RSp463e*RS346pe + 1./54.*m11*m55*RN3p*RSp461e
         *RS346pe - 1./162.*m11*m55*RN3p*RSp461e*RS546pe*RS346pe + 1./
         54.*m11*m55*RN3e*RSe463p*RS346pe - 1./162.*m11*m55*RN3e*
         RS546pe*RSe463p*RS346pe + 1./54.*m11*m55*RN3e*RSe461p*RS346pe
          - 1./162.*m11*m55*RN3e*RSe461p*RS546pe*RS346pe - 1./18.*m11*
         m55*RN13*RS146pe*RS346pe + 1./54.*m11*m55*RN13*RS546pe*RS146pe
         *RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * (  - 1./108.*m66*pow(
         RS345pe,2) - 1./162.*m66*RS346pe*RS345pe - 1./324.*m66*pow(
         RS346pe,2) + 1./54.*m66*RS145pe*RS345pe + 1./162.*m66*RS145pe*
         RS346pe + 1./162.*m66*RS146pe*RS345pe + 1./162.*m66*RS146pe*
         RS346pe + 1./162.*m56*pow(RS345pe,2) - 1./81.*m56*RS346pe*
         RS345pe + 1./162.*m56*pow(RS346pe,2) - 1./81.*m56*RS145pe*
         RS345pe + 1./81.*m56*RS145pe*RS346pe + 1./81.*m56*RS146pe*
         RS345pe - 1./81.*m56*RS146pe*RS346pe - 1./324.*m55*pow(
         RS345pe,2) - 1./162.*m55*RS346pe*RS345pe - 1./108.*m55*pow(
         RS346pe,2) + 1./162.*m55*RS145pe*RS345pe + 1./162.*m55*RS145pe
         *RS346pe + 1./162.*m55*RS146pe*RS345pe + 1./54.*m55*RS146pe*
         RS346pe + 1./324.*m33*pow(RS345pe,2) - 1./162.*m33*RS346pe*
         RS345pe + 1./324.*m33*pow(RS346pe,2) - 1./81.*m33*RSe463p*
         RS345pe + 1./81.*m33*RSe463p*RS346pe + 1./81.*m33*RSe453p*
         RS345pe - 1./81.*m33*RSe453p*RS346pe - 1./81.*m33*RSp463e*
         RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * ( 1./81.*m33*RSp463e*
         RS346pe + 1./81.*m33*RSp453e*RS345pe - 1./81.*m33*RSp453e*
         RS346pe - 1./54.*m33*RS145pe*RS345pe + 1./54.*m33*RS145pe*
         RS346pe + 1./54.*m33*RS146pe*RS345pe - 1./54.*m33*RS146pe*
         RS346pe + 1./81.*m33*RSe461p*RS345pe - 1./81.*m33*RSe461p*
         RS346pe - 1./81.*m33*RSe451p*RS345pe + 1./81.*m33*RSe451p*
         RS346pe + 1./81.*m33*RSp461e*RS345pe - 1./81.*m33*RSp461e*
         RS346pe - 1./81.*m33*RSp451e*RS345pe + 1./81.*m33*RSp451e*
         RS346pe - 1./81.*m33*me2*RN3e*RSe463p*RS345pe + 1./81.*m33*me2
         *RN3e*RSe463p*RS346pe + 1./81.*m33*me2*RN3e*RSe453p*RS345pe - 
         1./81.*m33*me2*RN3e*RSe453p*RS346pe + 1./81.*m33*me2*RN3e*
         RSe461p*RS345pe - 1./81.*m33*me2*RN3e*RSe461p*RS346pe - 1./81.
         *m33*me2*RN3e*RSe451p*RS345pe + 1./81.*m33*me2*RN3e*RSe451p*
         RS346pe - 1./81.*m33*mp2*RN3p*RSp463e*RS345pe + 1./81.*m33*mp2
         *RN3p*RSp463e*RS346pe + 1./81.*m33*mp2*RN3p*RSp453e*RS345pe - 
         1./81.*m33*mp2*RN3p*RSp453e*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * ( 1./81.*m33*mp2*RN3p*
         RSp461e*RS345pe - 1./81.*m33*mp2*RN3p*RSp461e*RS346pe - 1./81.
         *m33*mp2*RN3p*RSp451e*RS345pe + 1./81.*m33*mp2*RN3p*RSp451e*
         RS346pe + 1./324.*m33*m66*RN3p*RSp463e*RS345pe + 1./324.*m33*
         m66*RN3p*RSp463e*RS346pe + 1./108.*m33*m66*RN3p*RSp453e*
         RS345pe + 1./324.*m33*m66*RN3p*RSp453e*RS346pe + 1./108.*m33*
         m66*RN3p*RSp461e*RS345pe + 1./108.*m33*m66*RN3p*RSp461e*
         RS346pe + 1./36.*m33*m66*RN3p*RSp451e*RS345pe + 1./108.*m33*
         m66*RN3p*RSp451e*RS346pe + 1./324.*m33*m66*RN3e*RSe463p*
         RS345pe + 1./324.*m33*m66*RN3e*RSe463p*RS346pe + 1./108.*m33*
         m66*RN3e*RSe453p*RS345pe + 1./324.*m33*m66*RN3e*RSe453p*
         RS346pe + 1./108.*m33*m66*RN3e*RSe461p*RS345pe + 1./108.*m33*
         m66*RN3e*RSe461p*RS346pe + 1./36.*m33*m66*RN3e*RSe451p*RS345pe
          + 1./108.*m33*m66*RN3e*RSe451p*RS346pe - 1./108.*m33*m66*RN13
         *RS145pe*RS345pe - 1./324.*m33*m66*RN13*RS145pe*RS346pe - 1./
         324.*m33*m66*RN13*RS146pe*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * (  - 1./324.*m33*m66*
         RN13*RS146pe*RS346pe + 1./162.*m33*m56*RN3p*RSp463e*RS345pe - 
         1./162.*m33*m56*RN3p*RSp463e*RS346pe - 1./162.*m33*m56*RN3p*
         RSp453e*RS345pe + 1./162.*m33*m56*RN3p*RSp453e*RS346pe + 1./54.
         *m33*m56*RN3p*RSp461e*RS345pe - 1./54.*m33*m56*RN3p*RSp461e*
         RS346pe - 1./54.*m33*m56*RN3p*RSp451e*RS345pe + 1./54.*m33*m56
         *RN3p*RSp451e*RS346pe + 1./162.*m33*m56*RN3e*RSe463p*RS345pe
          - 1./162.*m33*m56*RN3e*RSe463p*RS346pe - 1./162.*m33*m56*RN3e
         *RSe453p*RS345pe + 1./162.*m33*m56*RN3e*RSe453p*RS346pe + 1./
         54.*m33*m56*RN3e*RSe461p*RS345pe - 1./54.*m33*m56*RN3e*RSe461p
         *RS346pe - 1./54.*m33*m56*RN3e*RSe451p*RS345pe + 1./54.*m33*
         m56*RN3e*RSe451p*RS346pe + 1./162.*m33*m56*RN13*RS145pe*
         RS345pe - 1./162.*m33*m56*RN13*RS145pe*RS346pe - 1./162.*m33*
         m56*RN13*RS146pe*RS345pe + 1./162.*m33*m56*RN13*RS146pe*
         RS346pe + 1./324.*m33*m55*RN3p*RSp463e*RS345pe + 1./108.*m33*
         m55*RN3p*RSp463e*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * ( 1./324.*m33*m55*RN3p*
         RSp453e*RS345pe + 1./324.*m33*m55*RN3p*RSp453e*RS346pe + 1./
         108.*m33*m55*RN3p*RSp461e*RS345pe + 1./36.*m33*m55*RN3p*
         RSp461e*RS346pe + 1./108.*m33*m55*RN3p*RSp451e*RS345pe + 1./
         108.*m33*m55*RN3p*RSp451e*RS346pe + 1./324.*m33*m55*RN3e*
         RSe463p*RS345pe + 1./108.*m33*m55*RN3e*RSe463p*RS346pe + 1./
         324.*m33*m55*RN3e*RSe453p*RS345pe + 1./324.*m33*m55*RN3e*
         RSe453p*RS346pe + 1./108.*m33*m55*RN3e*RSe461p*RS345pe + 1./36.
         *m33*m55*RN3e*RSe461p*RS346pe + 1./108.*m33*m55*RN3e*RSe451p*
         RS345pe + 1./108.*m33*m55*RN3e*RSe451p*RS346pe - 1./324.*m33*
         m55*RN13*RS145pe*RS345pe - 1./324.*m33*m55*RN13*RS145pe*
         RS346pe - 1./324.*m33*m55*RN13*RS146pe*RS345pe - 1./108.*m33*
         m55*RN13*RS146pe*RS346pe + 1./54.*pow(m33,2)*RN3p*RSp463e*
         RS345pe - 1./54.*pow(m33,2)*RN3p*RSp463e*RS346pe - 1./54.*pow(
         m33,2)*RN3p*RSp453e*RS345pe + 1./54.*pow(m33,2)*RN3p*RSp453e*
         RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * ( 1./162.*pow(m33,2)*
         RN3p*RSp461e*RS345pe - 1./162.*pow(m33,2)*RN3p*RSp461e*RS346pe
          - 1./162.*pow(m33,2)*RN3p*RSp451e*RS345pe + 1./162.*pow(
         m33,2)*RN3p*RSp451e*RS346pe + 1./54.*pow(m33,2)*RN3e*RSe463p*
         RS345pe - 1./54.*pow(m33,2)*RN3e*RSe463p*RS346pe - 1./54.*pow(
         m33,2)*RN3e*RSe453p*RS345pe + 1./54.*pow(m33,2)*RN3e*RSe453p*
         RS346pe + 1./162.*pow(m33,2)*RN3e*RSe461p*RS345pe - 1./162.*
         pow(m33,2)*RN3e*RSe461p*RS346pe - 1./162.*pow(m33,2)*RN3e*
         RSe451p*RS345pe + 1./162.*pow(m33,2)*RN3e*RSe451p*RS346pe + 1./
         162.*m13*pow(RS345pe,2) - 1./81.*m13*RS346pe*RS345pe + 1./162.
         *m13*pow(RS346pe,2) - 1./81.*m13*RS145pe*RS345pe + 1./81.*m13*
         RS145pe*RS346pe + 1./81.*m13*RS146pe*RS345pe - 1./81.*m13*
         RS146pe*RS346pe - 1./162.*m13*m66*RN3p*RSp463e*RS345pe - 1./
         162.*m13*m66*RN3p*RSp463e*RS346pe - 1./54.*m13*m66*RN3p*
         RSp453e*RS345pe - 1./162.*m13*m66*RN3p*RSp453e*RS346pe + 1./
         162.*m13*m66*RN3p*RSp461e*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * ( 1./162.*m13*m66*RN3p*
         RSp461e*RS346pe + 1./54.*m13*m66*RN3p*RSp451e*RS345pe + 1./162.
         *m13*m66*RN3p*RSp451e*RS346pe - 1./162.*m13*m66*RN3e*RSe463p*
         RS345pe - 1./162.*m13*m66*RN3e*RSe463p*RS346pe - 1./54.*m13*
         m66*RN3e*RSe453p*RS345pe - 1./162.*m13*m66*RN3e*RSe453p*
         RS346pe + 1./162.*m13*m66*RN3e*RSe461p*RS345pe + 1./162.*m13*
         m66*RN3e*RSe461p*RS346pe + 1./54.*m13*m66*RN3e*RSe451p*RS345pe
          + 1./162.*m13*m66*RN3e*RSe451p*RS346pe - 1./54.*m13*m66*RN13*
         RS145pe*RS345pe - 1./162.*m13*m66*RN13*RS145pe*RS346pe - 1./
         162.*m13*m66*RN13*RS146pe*RS345pe - 1./162.*m13*m66*RN13*
         RS146pe*RS346pe - 1./81.*m13*m56*RN3p*RSp463e*RS345pe + 1./81.
         *m13*m56*RN3p*RSp463e*RS346pe + 1./81.*m13*m56*RN3p*RSp453e*
         RS345pe - 1./81.*m13*m56*RN3p*RSp453e*RS346pe + 1./81.*m13*m56
         *RN3p*RSp461e*RS345pe - 1./81.*m13*m56*RN3p*RSp461e*RS346pe - 
         1./81.*m13*m56*RN3p*RSp451e*RS345pe + 1./81.*m13*m56*RN3p*
         RSp451e*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * (  - 1./81.*m13*m56*
         RN3e*RSe463p*RS345pe + 1./81.*m13*m56*RN3e*RSe463p*RS346pe + 1.
         /81.*m13*m56*RN3e*RSe453p*RS345pe - 1./81.*m13*m56*RN3e*
         RSe453p*RS346pe + 1./81.*m13*m56*RN3e*RSe461p*RS345pe - 1./81.
         *m13*m56*RN3e*RSe461p*RS346pe - 1./81.*m13*m56*RN3e*RSe451p*
         RS345pe + 1./81.*m13*m56*RN3e*RSe451p*RS346pe + 1./81.*m13*m56
         *RN13*RS145pe*RS345pe - 1./81.*m13*m56*RN13*RS145pe*RS346pe - 
         1./81.*m13*m56*RN13*RS146pe*RS345pe + 1./81.*m13*m56*RN13*
         RS146pe*RS346pe - 1./162.*m13*m55*RN3p*RSp463e*RS345pe - 1./54.
         *m13*m55*RN3p*RSp463e*RS346pe - 1./162.*m13*m55*RN3p*RSp453e*
         RS345pe - 1./162.*m13*m55*RN3p*RSp453e*RS346pe + 1./162.*m13*
         m55*RN3p*RSp461e*RS345pe + 1./54.*m13*m55*RN3p*RSp461e*RS346pe
          + 1./162.*m13*m55*RN3p*RSp451e*RS345pe + 1./162.*m13*m55*RN3p
         *RSp451e*RS346pe - 1./162.*m13*m55*RN3e*RSe463p*RS345pe - 1./
         54.*m13*m55*RN3e*RSe463p*RS346pe - 1./162.*m13*m55*RN3e*
         RSe453p*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * (  - 1./162.*m13*m55*
         RN3e*RSe453p*RS346pe + 1./162.*m13*m55*RN3e*RSe461p*RS345pe + 
         1./54.*m13*m55*RN3e*RSe461p*RS346pe + 1./162.*m13*m55*RN3e*
         RSe451p*RS345pe + 1./162.*m13*m55*RN3e*RSe451p*RS346pe - 1./
         162.*m13*m55*RN13*RS145pe*RS345pe - 1./162.*m13*m55*RN13*
         RS145pe*RS346pe - 1./162.*m13*m55*RN13*RS146pe*RS345pe - 1./54.
         *m13*m55*RN13*RS146pe*RS346pe - 1./81.*m13*m33*RN3p*RSp463e*
         RS345pe + 1./81.*m13*m33*RN3p*RSp463e*RS346pe + 1./81.*m13*m33
         *RN3p*RSp453e*RS345pe - 1./81.*m13*m33*RN3p*RSp453e*RS346pe + 
         1./81.*m13*m33*RN3p*RSp461e*RS345pe - 1./81.*m13*m33*RN3p*
         RSp461e*RS346pe - 1./81.*m13*m33*RN3p*RSp451e*RS345pe + 1./81.
         *m13*m33*RN3p*RSp451e*RS346pe - 1./81.*m13*m33*RN3e*RSe463p*
         RS345pe + 1./81.*m13*m33*RN3e*RSe463p*RS346pe + 1./81.*m13*m33
         *RN3e*RSe453p*RS345pe - 1./81.*m13*m33*RN3e*RSe453p*RS346pe + 
         1./81.*m13*m33*RN3e*RSe461p*RS345pe - 1./81.*m13*m33*RN3e*
         RSe461p*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * (  - 1./81.*m13*m33*
         RN3e*RSe451p*RS345pe + 1./81.*m13*m33*RN3e*RSe451p*RS346pe - 1.
         /324.*m11*pow(RS345pe,2) + 1./162.*m11*RS346pe*RS345pe - 1./
         324.*m11*pow(RS346pe,2) - 1./162.*m11*RS145pe*RS345pe + 1./162.
         *m11*RS145pe*RS346pe + 1./162.*m11*RS146pe*RS345pe - 1./162.*
         m11*RS146pe*RS346pe + 1./324.*m11*m66*RN3p*RSp463e*RS345pe + 1.
         /324.*m11*m66*RN3p*RSp463e*RS346pe + 1./108.*m11*m66*RN3p*
         RSp453e*RS345pe + 1./324.*m11*m66*RN3p*RSp453e*RS346pe + 1./
         324.*m11*m66*RN3p*RSp461e*RS345pe + 1./324.*m11*m66*RN3p*
         RSp461e*RS346pe + 1./108.*m11*m66*RN3p*RSp451e*RS345pe + 1./
         324.*m11*m66*RN3p*RSp451e*RS346pe + 1./324.*m11*m66*RN3e*
         RSe463p*RS345pe + 1./324.*m11*m66*RN3e*RSe463p*RS346pe + 1./
         108.*m11*m66*RN3e*RSe453p*RS345pe + 1./324.*m11*m66*RN3e*
         RSe453p*RS346pe + 1./324.*m11*m66*RN3e*RSe461p*RS345pe + 1./
         324.*m11*m66*RN3e*RSe461p*RS346pe + 1./108.*m11*m66*RN3e*
         RSe451p*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * ( 1./324.*m11*m66*RN3e*
         RSe451p*RS346pe - 1./36.*m11*m66*RN13*RS145pe*RS345pe - 1./108.
         *m11*m66*RN13*RS145pe*RS346pe - 1./108.*m11*m66*RN13*RS146pe*
         RS345pe - 1./108.*m11*m66*RN13*RS146pe*RS346pe + 1./162.*m11*
         m56*RN3p*RSp463e*RS345pe - 1./162.*m11*m56*RN3p*RSp463e*
         RS346pe - 1./162.*m11*m56*RN3p*RSp453e*RS345pe + 1./162.*m11*
         m56*RN3p*RSp453e*RS346pe + 1./162.*m11*m56*RN3p*RSp461e*
         RS345pe - 1./162.*m11*m56*RN3p*RSp461e*RS346pe - 1./162.*m11*
         m56*RN3p*RSp451e*RS345pe + 1./162.*m11*m56*RN3p*RSp451e*
         RS346pe + 1./162.*m11*m56*RN3e*RSe463p*RS345pe - 1./162.*m11*
         m56*RN3e*RSe463p*RS346pe - 1./162.*m11*m56*RN3e*RSe453p*
         RS345pe + 1./162.*m11*m56*RN3e*RSe453p*RS346pe + 1./162.*m11*
         m56*RN3e*RSe461p*RS345pe - 1./162.*m11*m56*RN3e*RSe461p*
         RS346pe - 1./162.*m11*m56*RN3e*RSe451p*RS345pe + 1./162.*m11*
         m56*RN3e*RSe451p*RS346pe + 1./54.*m11*m56*RN13*RS145pe*RS345pe
          - 1./54.*m11*m56*RN13*RS145pe*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * (  - 1./54.*m11*m56*
         RN13*RS146pe*RS345pe + 1./54.*m11*m56*RN13*RS146pe*RS346pe + 1.
         /324.*m11*m55*RN3p*RSp463e*RS345pe + 1./108.*m11*m55*RN3p*
         RSp463e*RS346pe + 1./324.*m11*m55*RN3p*RSp453e*RS345pe + 1./
         324.*m11*m55*RN3p*RSp453e*RS346pe + 1./324.*m11*m55*RN3p*
         RSp461e*RS345pe + 1./108.*m11*m55*RN3p*RSp461e*RS346pe + 1./
         324.*m11*m55*RN3p*RSp451e*RS345pe + 1./324.*m11*m55*RN3p*
         RSp451e*RS346pe + 1./324.*m11*m55*RN3e*RSe463p*RS345pe + 1./
         108.*m11*m55*RN3e*RSe463p*RS346pe + 1./324.*m11*m55*RN3e*
         RSe453p*RS345pe + 1./324.*m11*m55*RN3e*RSe453p*RS346pe + 1./
         324.*m11*m55*RN3e*RSe461p*RS345pe + 1./108.*m11*m55*RN3e*
         RSe461p*RS346pe + 1./324.*m11*m55*RN3e*RSe451p*RS345pe + 1./
         324.*m11*m55*RN3e*RSe451p*RS346pe - 1./108.*m11*m55*RN13*
         RS145pe*RS345pe - 1./108.*m11*m55*RN13*RS145pe*RS346pe - 1./
         108.*m11*m55*RN13*RS146pe*RS345pe - 1./36.*m11*m55*RN13*
         RS146pe*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * ( 1./162.*m11*m33*RN3p*
         RSp463e*RS345pe - 1./162.*m11*m33*RN3p*RSp463e*RS346pe - 1./
         162.*m11*m33*RN3p*RSp453e*RS345pe + 1./162.*m11*m33*RN3p*
         RSp453e*RS346pe + 1./162.*m11*m33*RN3p*RSp461e*RS345pe - 1./
         162.*m11*m33*RN3p*RSp461e*RS346pe - 1./162.*m11*m33*RN3p*
         RSp451e*RS345pe + 1./162.*m11*m33*RN3p*RSp451e*RS346pe + 1./
         162.*m11*m33*RN3e*RSe463p*RS345pe - 1./162.*m11*m33*RN3e*
         RSe463p*RS346pe - 1./162.*m11*m33*RN3e*RSe453p*RS345pe + 1./
         162.*m11*m33*RN3e*RSe453p*RS346pe + 1./162.*m11*m33*RN3e*
         RSe461p*RS345pe - 1./162.*m11*m33*RN3e*RSe461p*RS346pe - 1./
         162.*m11*m33*RN3e*RSe451p*RS345pe + 1./162.*m11*m33*RN3e*
         RSe451p*RS346pe + 1./54.*m11*m33*RN13*RS145pe*RS345pe - 1./54.
         *m11*m33*RN13*RS145pe*RS346pe - 1./54.*m11*m33*RN13*RS146pe*
         RS345pe + 1./54.*m11*m33*RN13*RS146pe*RS346pe + 1./81.*m11*m13
         *RN13*RS145pe*RS345pe - 1./81.*m11*m13*RN13*RS145pe*RS346pe - 
         1./81.*m11*m13*RN13*RS146pe*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * ( 1./81.*m11*m13*RN13*
         RS146pe*RS346pe + 1./162.*pow(m11,2)*RN13*RS145pe*RS345pe - 1./
         162.*pow(m11,2)*RN13*RS145pe*RS346pe - 1./162.*pow(m11,2)*RN13
         *RS146pe*RS345pe + 1./162.*pow(m11,2)*RN13*RS146pe*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m66,L) * (  - 1./54.*m66*pow(
         RS345pe,2) + 1./27.*m66*RS145pe*RS345pe + 1./162.*m66*RS645pe*
         pow(RS345pe,2) - 1./81.*m66*RS645pe*RS145pe*RS345pe + 1./54.*
         m33*m66*RN3p*RSp453e*RS345pe - 1./162.*m33*m66*RN3p*RS645pe*
         RSp453e*RS345pe + 1./18.*m33*m66*RN3p*RSp451e*RS345pe - 1./54.
         *m33*m66*RN3p*RSp451e*RS645pe*RS345pe + 1./54.*m33*m66*RN3e*
         RSe453p*RS345pe - 1./162.*m33*m66*RN3e*RS645pe*RSe453p*RS345pe
          + 1./18.*m33*m66*RN3e*RSe451p*RS345pe - 1./54.*m33*m66*RN3e*
         RSe451p*RS645pe*RS345pe - 1./54.*m33*m66*RN13*RS145pe*RS345pe
          + 1./162.*m33*m66*RN13*RS645pe*RS145pe*RS345pe - 1./27.*m13*
         m66*RN3p*RSp453e*RS345pe + 1./81.*m13*m66*RN3p*RS645pe*RSp453e
         *RS345pe + 1./27.*m13*m66*RN3p*RSp451e*RS345pe - 1./81.*m13*
         m66*RN3p*RSp451e*RS645pe*RS345pe - 1./27.*m13*m66*RN3e*RSe453p
         *RS345pe + 1./81.*m13*m66*RN3e*RS645pe*RSe453p*RS345pe + 1./27.
         *m13*m66*RN3e*RSe451p*RS345pe - 1./81.*m13*m66*RN3e*RSe451p*
         RS645pe*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(m66,L) * (  - 1./27.*m13*m66*
         RN13*RS145pe*RS345pe + 1./81.*m13*m66*RN13*RS645pe*RS145pe*
         RS345pe + 1./54.*m11*m66*RN3p*RSp453e*RS345pe - 1./162.*m11*
         m66*RN3p*RS645pe*RSp453e*RS345pe + 1./54.*m11*m66*RN3p*RSp451e
         *RS345pe - 1./162.*m11*m66*RN3p*RSp451e*RS645pe*RS345pe + 1./
         54.*m11*m66*RN3e*RSe453p*RS345pe - 1./162.*m11*m66*RN3e*
         RS645pe*RSe453p*RS345pe + 1./54.*m11*m66*RN3e*RSe451p*RS345pe
          - 1./162.*m11*m66*RN3e*RSe451p*RS645pe*RS345pe - 1./18.*m11*
         m66*RN13*RS145pe*RS345pe + 1./54.*m11*m66*RN13*RS645pe*RS145pe
         *RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(mp2,L) * (  - 1./270.*RD3456ep*
         RSp45613e + 1./540.*RSp45633e*RD3456ep + 1./540.*RSp45611e*
         RD3456ep + 1./135.*mp2*RSp45613e*RS34561pe - 1./270.*mp2*
         RC3456pe*RSp45613e - 1./270.*mp2*RSp45633e*RS34561pe + 1./540.
         *mp2*RSp45633e*RC3456pe - 1./270.*mp2*RSp45611e*RS34561pe + 1./
         540.*mp2*RSp45611e*RC3456pe + 1./162.*m66*RSp456e*pow(
         RS345pe,2) - 1./81.*m66*RSp456e*RS145pe*RS345pe + 1./162.*m55*
         RSp465e*pow(RS346pe,2) - 1./81.*m55*RSp465e*RS146pe*RS346pe + 
         1./162.*m44*RSp564e*pow(RS356pe,2) - 1./81.*m44*RSp564e*
         RS156pe*RS356pe + 16./405.*m33*RSp45613e*RS34561pe + 13./810.*
         m33*RC3456pe*RSp45613e + 23./810.*m33*RSp45633e*RS34561pe + 1./
         27.*m33*RSp45633e*RS14563pe - 37./1620.*m33*RSp45633e*RC3456pe
          + 1./162.*m33*RSp45611e*RS34561pe - 19./1620.*m33*RSp45611e*
         RC3456pe - 1./162.*m33*m66*RN3p*RSp456e*RSp453e*RS345pe - 1./
         54.*m33*m66*RN3p*RSp456e*RSp451e*RS345pe - 1./162.*m33*m66*
         RN3e*RSp456e*RSe453p*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(mp2,L) * (  - 1./54.*m33*m66*
         RN3e*RSp456e*RSe451p*RS345pe + 1./162.*m33*m66*RN13*RSp456e*
         RS145pe*RS345pe - 1./162.*m33*m55*RN3p*RSp465e*RSp463e*RS346pe
          - 1./54.*m33*m55*RN3p*RSp465e*RSp461e*RS346pe - 1./162.*m33*
         m55*RN3e*RSp465e*RSe463p*RS346pe - 1./54.*m33*m55*RN3e*RSp465e
         *RSe461p*RS346pe + 1./162.*m33*m55*RN13*RSp465e*RS146pe*
         RS346pe - 1./162.*m33*m44*RN3p*RSp564e*RSp563e*RS356pe - 1./54.
         *m33*m44*RN3p*RSp564e*RSp561e*RS356pe - 1./162.*m33*m44*RN3e*
         RSp564e*RSe563p*RS356pe - 1./54.*m33*m44*RN3e*RSp564e*RSe561p*
         RS356pe + 1./162.*m33*m44*RN13*RSp564e*RS156pe*RS356pe + 1./18.
         *pow(m33,2)*RN3p*RSp45633e*RSp45613e + 1./54.*pow(m33,2)*RN3p*
         pow(RSp45633e,2) + 1./18.*pow(m33,2)*RN3e*RSp45633e*RSe45613p
          + 1./54.*pow(m33,2)*RN3e*RSp45633e*RSe45633p - 1./54.*pow(
         m33,2)*RN13*RSp45633e*RS14563pe - 7./405.*m13*RSp45613e*
         RS34561pe + 7./810.*m13*RC3456pe*RSp45613e + 7./810.*m13*
         RSp45633e*RS34561pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(mp2,L) * (  - 7./1620.*m13*
         RSp45633e*RC3456pe + 7./810.*m13*RSp45611e*RS34561pe - 7./1620.
         *m13*RSp45611e*RC3456pe + 1./81.*m13*m66*RN3p*RSp456e*RSp453e*
         RS345pe - 1./81.*m13*m66*RN3p*RSp456e*RSp451e*RS345pe + 1./81.
         *m13*m66*RN3e*RSp456e*RSe453p*RS345pe - 1./81.*m13*m66*RN3e*
         RSp456e*RSe451p*RS345pe + 1./81.*m13*m66*RN13*RSp456e*RS145pe*
         RS345pe + 1./81.*m13*m55*RN3p*RSp465e*RSp463e*RS346pe - 1./81.
         *m13*m55*RN3p*RSp465e*RSp461e*RS346pe + 1./81.*m13*m55*RN3e*
         RSp465e*RSe463p*RS346pe - 1./81.*m13*m55*RN3e*RSp465e*RSe461p*
         RS346pe + 1./81.*m13*m55*RN13*RSp465e*RS146pe*RS346pe + 1./81.
         *m13*m44*RN3p*RSp564e*RSp563e*RS356pe - 1./81.*m13*m44*RN3p*
         RSp564e*RSp561e*RS356pe + 1./81.*m13*m44*RN3e*RSp564e*RSe563p*
         RS356pe - 1./81.*m13*m44*RN3e*RSp564e*RSe561p*RS356pe + 1./81.
         *m13*m44*RN13*RSp564e*RS156pe*RS356pe + 1./27.*m13*m33*RN3p*
         RSp45633e*RSp45613e - 1./27.*m13*m33*RN3p*pow(RSp45633e,2) + 1.
         /27.*m13*m33*RN3e*RSp45633e*RSe45613p );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(mp2,L) * (  - 1./27.*m13*m33*
         RN3e*RSp45633e*RSe45633p - 1./27.*m13*m33*RN13*RSp45633e*
         RS14563pe + 2./45.*m11*RSp45613e*RS34561pe - 1./405.*m11*
         RC3456pe*RSp45613e + 1./270.*m11*RSp45633e*RS34561pe + 13./810.
         *m11*RSp45633e*RC3456pe + 7./270.*m11*RSp45611e*RS34561pe - 11.
         /810.*m11*RSp45611e*RC3456pe - 1./162.*m11*m66*RN3p*RSp456e*
         RSp453e*RS345pe - 1./162.*m11*m66*RN3p*RSp456e*RSp451e*RS345pe
          - 1./162.*m11*m66*RN3e*RSp456e*RSe453p*RS345pe - 1./162.*m11*
         m66*RN3e*RSp456e*RSe451p*RS345pe + 1./54.*m11*m66*RN13*RSp456e
         *RS145pe*RS345pe - 1./162.*m11*m55*RN3p*RSp465e*RSp463e*
         RS346pe - 1./162.*m11*m55*RN3p*RSp465e*RSp461e*RS346pe - 1./
         162.*m11*m55*RN3e*RSp465e*RSe463p*RS346pe - 1./162.*m11*m55*
         RN3e*RSp465e*RSe461p*RS346pe + 1./54.*m11*m55*RN13*RSp465e*
         RS146pe*RS346pe - 1./162.*m11*m44*RN3p*RSp564e*RSp563e*RS356pe
          - 1./162.*m11*m44*RN3p*RSp564e*RSp561e*RS356pe - 1./162.*m11*
         m44*RN3e*RSp564e*RSe563p*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(mp2,L) * (  - 1./162.*m11*m44*
         RN3e*RSp564e*RSe561p*RS356pe + 1./54.*m11*m44*RN13*RSp564e*
         RS156pe*RS356pe + 1./54.*m11*m33*RN3p*RSp45633e*RSp45613e + 1./
         54.*m11*m33*RN3p*pow(RSp45633e,2) + 1./54.*m11*m33*RN3e*
         RSp45633e*RSe45613p + 1./54.*m11*m33*RN3e*RSp45633e*RSe45633p
          - 1./18.*m11*m33*RN13*RSp45633e*RS14563pe - 1./54.*m11*m33*
         RN13*RSp45611e*RS34561pe - 1./27.*m11*m13*RN13*RSp45611e*
         RS34561pe - 1./18.*pow(m11,2)*RN13*RSp45611e*RS34561pe + 2./81.
         *RS(m33,m44,m55,m66,m11,me2,mp2)*m33*RSp45613e - 1./81.*RS(m33
         ,m44,m55,m66,m11,me2,mp2)*m33*RSp45633e - 1./81.*RS(m33,m44,
         m55,m66,m11,me2,mp2)*m33*RSp45611e + 1./81.*RS(m33,m44,m55,m66
         ,m11,me2,mp2)*m11*RSp45613e - 1./162.*RS(m33,m44,m55,m66,m11,
         me2,mp2)*m11*RSp45633e - 1./162.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11*RSp45611e );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(me2,L) * (  - 1./270.*RD3456ep*
         RSe45613p + 1./540.*RSe45633p*RD3456ep + 1./540.*RSe45611p*
         RD3456ep + 1./135.*me2*RSe45613p*RS34561pe - 1./270.*me2*
         RC3456pe*RSe45613p - 1./270.*me2*RSe45633p*RS34561pe + 1./540.
         *me2*RSe45633p*RC3456pe - 1./270.*me2*RSe45611p*RS34561pe + 1./
         540.*me2*RSe45611p*RC3456pe + 1./162.*m66*RSe456p*pow(
         RS345pe,2) - 1./81.*m66*RSe456p*RS145pe*RS345pe + 1./162.*m55*
         RSe465p*pow(RS346pe,2) - 1./81.*m55*RSe465p*RS146pe*RS346pe + 
         1./162.*m44*RSe564p*pow(RS356pe,2) - 1./81.*m44*RSe564p*
         RS156pe*RS356pe + 16./405.*m33*RSe45613p*RS34561pe + 13./810.*
         m33*RC3456pe*RSe45613p + 23./810.*m33*RSe45633p*RS34561pe + 1./
         27.*m33*RSe45633p*RS14563pe - 37./1620.*m33*RSe45633p*RC3456pe
          + 1./162.*m33*RSe45611p*RS34561pe - 19./1620.*m33*RSe45611p*
         RC3456pe - 1./162.*m33*m66*RN3p*RSe456p*RSp453e*RS345pe - 1./
         54.*m33*m66*RN3p*RSe456p*RSp451e*RS345pe - 1./162.*m33*m66*
         RN3e*RSe456p*RSe453p*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(me2,L) * (  - 1./54.*m33*m66*
         RN3e*RSe456p*RSe451p*RS345pe + 1./162.*m33*m66*RN13*RSe456p*
         RS145pe*RS345pe - 1./162.*m33*m55*RN3p*RSe465p*RSp463e*RS346pe
          - 1./54.*m33*m55*RN3p*RSe465p*RSp461e*RS346pe - 1./162.*m33*
         m55*RN3e*RSe465p*RSe463p*RS346pe - 1./54.*m33*m55*RN3e*RSe465p
         *RSe461p*RS346pe + 1./162.*m33*m55*RN13*RSe465p*RS146pe*
         RS346pe - 1./162.*m33*m44*RN3p*RSe564p*RSp563e*RS356pe - 1./54.
         *m33*m44*RN3p*RSe564p*RSp561e*RS356pe - 1./162.*m33*m44*RN3e*
         RSe564p*RSe563p*RS356pe - 1./54.*m33*m44*RN3e*RSe564p*RSe561p*
         RS356pe + 1./162.*m33*m44*RN13*RSe564p*RS156pe*RS356pe + 1./18.
         *pow(m33,2)*RN3p*RSe45633p*RSp45613e + 1./54.*pow(m33,2)*RN3p*
         RSp45633e*RSe45633p + 1./18.*pow(m33,2)*RN3e*RSe45633p*
         RSe45613p + 1./54.*pow(m33,2)*RN3e*pow(RSe45633p,2) - 1./54.*
         pow(m33,2)*RN13*RSe45633p*RS14563pe - 7./405.*m13*RSe45613p*
         RS34561pe + 7./810.*m13*RC3456pe*RSe45613p + 7./810.*m13*
         RSe45633p*RS34561pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(me2,L) * (  - 7./1620.*m13*
         RSe45633p*RC3456pe + 7./810.*m13*RSe45611p*RS34561pe - 7./1620.
         *m13*RSe45611p*RC3456pe + 1./81.*m13*m66*RN3p*RSe456p*RSp453e*
         RS345pe - 1./81.*m13*m66*RN3p*RSe456p*RSp451e*RS345pe + 1./81.
         *m13*m66*RN3e*RSe456p*RSe453p*RS345pe - 1./81.*m13*m66*RN3e*
         RSe456p*RSe451p*RS345pe + 1./81.*m13*m66*RN13*RSe456p*RS145pe*
         RS345pe + 1./81.*m13*m55*RN3p*RSe465p*RSp463e*RS346pe - 1./81.
         *m13*m55*RN3p*RSe465p*RSp461e*RS346pe + 1./81.*m13*m55*RN3e*
         RSe465p*RSe463p*RS346pe - 1./81.*m13*m55*RN3e*RSe465p*RSe461p*
         RS346pe + 1./81.*m13*m55*RN13*RSe465p*RS146pe*RS346pe + 1./81.
         *m13*m44*RN3p*RSe564p*RSp563e*RS356pe - 1./81.*m13*m44*RN3p*
         RSe564p*RSp561e*RS356pe + 1./81.*m13*m44*RN3e*RSe564p*RSe563p*
         RS356pe - 1./81.*m13*m44*RN3e*RSe564p*RSe561p*RS356pe + 1./81.
         *m13*m44*RN13*RSe564p*RS156pe*RS356pe + 1./27.*m13*m33*RN3p*
         RSe45633p*RSp45613e - 1./27.*m13*m33*RN3p*RSp45633e*RSe45633p
          + 1./27.*m13*m33*RN3e*RSe45633p*RSe45613p );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(me2,L) * (  - 1./27.*m13*m33*
         RN3e*pow(RSe45633p,2) - 1./27.*m13*m33*RN13*RSe45633p*
         RS14563pe + 2./45.*m11*RSe45613p*RS34561pe - 1./405.*m11*
         RC3456pe*RSe45613p + 1./270.*m11*RSe45633p*RS34561pe + 13./810.
         *m11*RSe45633p*RC3456pe + 7./270.*m11*RSe45611p*RS34561pe - 11.
         /810.*m11*RSe45611p*RC3456pe - 1./162.*m11*m66*RN3p*RSe456p*
         RSp453e*RS345pe - 1./162.*m11*m66*RN3p*RSe456p*RSp451e*RS345pe
          - 1./162.*m11*m66*RN3e*RSe456p*RSe453p*RS345pe - 1./162.*m11*
         m66*RN3e*RSe456p*RSe451p*RS345pe + 1./54.*m11*m66*RN13*RSe456p
         *RS145pe*RS345pe - 1./162.*m11*m55*RN3p*RSe465p*RSp463e*
         RS346pe - 1./162.*m11*m55*RN3p*RSe465p*RSp461e*RS346pe - 1./
         162.*m11*m55*RN3e*RSe465p*RSe463p*RS346pe - 1./162.*m11*m55*
         RN3e*RSe465p*RSe461p*RS346pe + 1./54.*m11*m55*RN13*RSe465p*
         RS146pe*RS346pe - 1./162.*m11*m44*RN3p*RSe564p*RSp563e*RS356pe
          - 1./162.*m11*m44*RN3p*RSe564p*RSp561e*RS356pe - 1./162.*m11*
         m44*RN3e*RSe564p*RSe563p*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*AbV(me2,L) * (  - 1./162.*m11*m44*
         RN3e*RSe564p*RSe561p*RS356pe + 1./54.*m11*m44*RN13*RSe564p*
         RS156pe*RS356pe + 1./54.*m11*m33*RN3p*RSe45633p*RSp45613e + 1./
         54.*m11*m33*RN3p*RSp45633e*RSe45633p + 1./54.*m11*m33*RN3e*
         RSe45633p*RSe45613p + 1./54.*m11*m33*RN3e*pow(RSe45633p,2) - 1.
         /18.*m11*m33*RN13*RSe45633p*RS14563pe - 1./54.*m11*m33*RN13*
         RSe45611p*RS34561pe - 1./27.*m11*m13*RN13*RSe45611p*RS34561pe
          - 1./18.*pow(m11,2)*RN13*RSe45611p*RS34561pe + 2./81.*RS(m33,
         m44,m55,m66,m11,me2,mp2)*m33*RSe45613p - 1./81.*RS(m33,m44,m55
         ,m66,m11,me2,mp2)*m33*RSe45633p - 1./81.*RS(m33,m44,m55,m66,
         m11,me2,mp2)*m33*RSe45611p + 1./81.*RS(m33,m44,m55,m66,m11,me2
         ,mp2)*m11*RSe45613p - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*
         m11*RSe45633p - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RSe45611p );

      M13p6x23RV +=  + Ab(m33,mu2)*BbV(m11,m33,L) * ( 1./54.*pow(m33,2)
         *RC3456pe*RS14563pe + 1./27.*m13*m33*RC3456pe*RS14563pe + 1./
         18.*m11*m33*RC3456pe*RS14563pe );

      M13p6x23RV +=  + Ab(m33,mu2)*BbV(m11,L) * ( 1./540.*RD1456ep*
         RD3456ep + 1./162.*m33*RD1456ep*RS34561pe - 19./1620.*m33*
         RD1456ep*RC3456pe + 7./810.*m13*RD1456ep*RS34561pe - 7./1620.*
         m13*RD1456ep*RC3456pe + 1./45.*m11*RD1456ep*RS34561pe - 19./
         1620.*m11*RD1456ep*RC3456pe - 1./54.*m11*m33*RN13*RD1456ep*
         RS34561pe - 1./27.*m11*m13*RN13*RD1456ep*RS34561pe - 1./18.*
         pow(m11,2)*RN13*RD1456ep*RS34561pe - 1./81.*RS(m33,m44,m55,m66
         ,m11,me2,mp2)*m33*RD1456ep - 1./162.*RS(m33,m44,m55,m66,m11,
         me2,mp2)*m11*RD1456ep );

      M13p6x23RV +=  + Ab(m33,mu2)*BbV(m33,mp2,L) * ( 1./18.*pow(m33,2)
         *RC3456pe*RSp45613e + 1./54.*pow(m33,2)*RSp45633e*RC3456pe + 1.
         /27.*m13*m33*RC3456pe*RSp45613e - 1./27.*m13*m33*RSp45633e*
         RC3456pe + 1./54.*m11*m33*RC3456pe*RSp45613e + 1./54.*m11*m33*
         RSp45633e*RC3456pe );

      M13p6x23RV +=  + Ab(m33,mu2)*BbV(m33,me2,L) * ( 1./18.*pow(m33,2)
         *RC3456pe*RSe45613p + 1./54.*pow(m33,2)*RSe45633p*RC3456pe + 1.
         /27.*m13*m33*RC3456pe*RSe45613p - 1./27.*m13*m33*RSe45633p*
         RC3456pe + 1./54.*m11*m33*RC3456pe*RSe45613p + 1./54.*m11*m33*
         RSe45633p*RC3456pe );

      M13p6x23RV +=  + Ab(m33,mu2)*BbV(m33,L) * ( 1./540.*pow(
         RD3456ep,2) + 2./81.*m33*RD3456ep*RS34561pe + 1./27.*m33*
         RD3456ep*RS14563pe - 17./810.*m33*RD3456ep*RC3456pe + 1./12.*
         pow(m33,2) + 1./18.*pow(m33,2)*RC3456pe*RS34561pe + 1./54.*
         pow(m33,2)*pow(RC3456pe,2) + 1./18.*pow(m33,2)*RN3p*RD3456ep*
         RSp45613e + 1./54.*pow(m33,2)*RN3p*RSp45633e*RD3456ep + 1./18.
         *pow(m33,2)*RN3e*RD3456ep*RSe45613p + 1./54.*pow(m33,2)*RN3e*
         RSe45633p*RD3456ep - 1./54.*pow(m33,2)*RN13*RD3456ep*RS14563pe
          + 7./810.*m13*RD3456ep*RS34561pe - 7./1620.*m13*RD3456ep*
         RC3456pe - 1./6.*m13*m33 + 1./27.*m13*m33*RC3456pe*RS34561pe
          - 1./27.*m13*m33*pow(RC3456pe,2) + 1./27.*m13*m33*RN3p*
         RD3456ep*RSp45613e - 1./27.*m13*m33*RN3p*RSp45633e*RD3456ep + 
         1./27.*m13*m33*RN3e*RD3456ep*RSe45613p - 1./27.*m13*m33*RN3e*
         RSe45633p*RD3456ep - 1./27.*m13*m33*RN13*RD3456ep*RS14563pe + 
         1./270.*m11*RD3456ep*RS34561pe + 13./810.*m11*RD3456ep*
         RC3456pe );

      M13p6x23RV +=  + Ab(m33,mu2)*BbV(m33,L) * ( 1./12.*m11*m33 + 1./
         54.*m11*m33*RC3456pe*RS34561pe + 1./54.*m11*m33*pow(
         RC3456pe,2) + 1./54.*m11*m33*RN3p*RD3456ep*RSp45613e + 1./54.*
         m11*m33*RN3p*RSp45633e*RD3456ep + 1./54.*m11*m33*RN3e*RD3456ep
         *RSe45613p + 1./54.*m11*m33*RN3e*RSe45633p*RD3456ep - 1./18.*
         m11*m33*RN13*RD3456ep*RS14563pe - 1./81.*RS(m33,m44,m55,m66,
         m11,me2,mp2)*m33*RD3456ep - 1./162.*RS(m33,m44,m55,m66,m11,me2
         ,mp2)*m11*RD3456ep );

      M13p6x23RV +=  + Ab(m33,mu2)*BbV(m34,L) * ( 1./216.*m44*RD3456ep
          - 1./432.*pow(m44,2)*RC3456pe - 1./216.*pow(m44,2)*RS356pe - 
         1./108.*m34*RD3456ep + 1./108.*m34*m44*RC3456pe - 1./108.*pow(
         m34,2)*RC3456pe + 1./54.*pow(m34,2)*RS356pe + 1./108.*m33*
         RD3456ep - 1./144.*m33*m44*RC3456pe - 5./216.*m33*m44*RS356pe
          + 1./72.*m33*m34*RC3456pe + 1./108.*m33*m34*RS356pe - 1./216.
         *pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS356pe - 1./108.*m13
         *RD3456ep + 1./216.*m13*m44*RC3456pe + 1./108.*m13*m44*RS356pe
          - 1./108.*m13*m34*RC3456pe + 1./54.*m13*m34*RS356pe + 1./216.
         *m13*m33*RC3456pe + 1./36.*m13*m33*RS356pe + 1./216.*m11*
         RD3456ep - 1./432.*m11*m44*RC3456pe - 1./216.*m11*m44*RS356pe
          + 1./216.*m11*m34*RC3456pe - 1./108.*m11*m34*RS356pe - 1./432.
         *m11*m33*RC3456pe - 1./72.*m11*m33*RS356pe );

      M13p6x23RV +=  + Ab(m33,mu2)*BbV(m35,L) * ( 1./216.*m55*RD3456ep
          - 1./432.*pow(m55,2)*RC3456pe - 1./216.*pow(m55,2)*RS346pe - 
         1./108.*m35*RD3456ep + 1./108.*m35*m55*RC3456pe - 1./108.*pow(
         m35,2)*RC3456pe + 1./54.*pow(m35,2)*RS346pe + 1./108.*m33*
         RD3456ep - 1./144.*m33*m55*RC3456pe - 5./216.*m33*m55*RS346pe
          + 1./72.*m33*m35*RC3456pe + 1./108.*m33*m35*RS346pe - 1./216.
         *pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS346pe - 1./108.*m13
         *RD3456ep + 1./216.*m13*m55*RC3456pe + 1./108.*m13*m55*RS346pe
          - 1./108.*m13*m35*RC3456pe + 1./54.*m13*m35*RS346pe + 1./216.
         *m13*m33*RC3456pe + 1./36.*m13*m33*RS346pe + 1./216.*m11*
         RD3456ep - 1./432.*m11*m55*RC3456pe - 1./216.*m11*m55*RS346pe
          + 1./216.*m11*m35*RC3456pe - 1./108.*m11*m35*RS346pe - 1./432.
         *m11*m33*RC3456pe - 1./72.*m11*m33*RS346pe );

      M13p6x23RV +=  + Ab(m33,mu2)*BbV(m36,L) * ( 1./216.*m66*RD3456ep
          - 1./432.*pow(m66,2)*RC3456pe - 1./216.*pow(m66,2)*RS345pe - 
         1./108.*m36*RD3456ep + 1./108.*m36*m66*RC3456pe - 1./108.*pow(
         m36,2)*RC3456pe + 1./54.*pow(m36,2)*RS345pe + 1./108.*m33*
         RD3456ep - 1./144.*m33*m66*RC3456pe - 5./216.*m33*m66*RS345pe
          + 1./72.*m33*m36*RC3456pe + 1./108.*m33*m36*RS345pe - 1./216.
         *pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS345pe - 1./108.*m13
         *RD3456ep + 1./216.*m13*m66*RC3456pe + 1./108.*m13*m66*RS345pe
          - 1./108.*m13*m36*RC3456pe + 1./54.*m13*m36*RS345pe + 1./216.
         *m13*m33*RC3456pe + 1./36.*m13*m33*RS345pe + 1./216.*m11*
         RD3456ep - 1./432.*m11*m66*RC3456pe - 1./216.*m11*m66*RS345pe
          + 1./216.*m11*m36*RC3456pe - 1./108.*m11*m36*RS345pe - 1./432.
         *m11*m33*RC3456pe - 1./72.*m11*m33*RS345pe );

      M13p6x23RV +=  + Ab(m33,mu2)*CbV(m33,L) * ( 1./54.*pow(m33,2)*
         RD3456ep*RC3456pe - 1./27.*m13*m33*RD3456ep*RC3456pe + 1./54.*
         m11*m33*RD3456ep*RC3456pe );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(m11,L) * (  - 1./180.*RD1456ep
          + 1./180.*m44*RS14563pe + 1./60.*m44*RC1456pe + 13./540.*m44*
         RS156pe - 13./270.*m34*RS14563pe + 1./270.*m34*RC1456pe + 11./
         270.*m34*RS156pe + 1./60.*m33*RS14563pe + 2./135.*m33*RC1456pe
          + 1./180.*m33*RS156pe + 1./27.*m14*RS14563pe - 1./27.*m14*
         RC1456pe + 1./27.*m14*RS156pe - 31./540.*m13*RS14563pe + 1./20.
         *m13*RC1456pe - 23./540.*m13*RS156pe + 1./18.*m11*RS14563pe + 
         19./540.*m11*RC1456pe - 2./27.*m11*RS156pe + 1./54.*RS(m11,m44
         ,m55,m66,m33,me2,mp2)*m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,
         mp2)*m11 );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(m14,L) * (  - 1./108.*RD3456ep
          - 1./108.*RD1456ep - 1./54.*me2*RSe45613p - 1./108.*me2*
         RSe45633p + 1./27.*me2*RSe563p - 1./108.*me2*RSe45611p + 1./27.
         *me2*RSe561p - 1./27.*me2*RSe564p - 1./54.*mp2*RSp45613e - 1./
         108.*mp2*RSp45633e + 1./27.*mp2*RSp563e - 1./108.*mp2*
         RSp45611e + 1./27.*mp2*RSp561e - 1./27.*mp2*RSp564e + 1./90.*
         m44 - 1./54.*m44*RS34561pe - 1./54.*m44*RS14563pe - 1./54.*m44
         *RSe45613p - 1./54.*m44*RSp45613e - 1./108.*m44*RC3456pe - 1./
         108.*m44*RSe45633p - 1./108.*m44*RSp45633e - 1./108.*m44*
         RC1456pe - 1./108.*m44*RSe45611p - 1./108.*m44*RSp45611e + 1./
         27.*m44*RSe564p + 1./27.*m44*RSp564e - 1./90.*m34 + 1./36.*m34
         *RC3456pe + 1./36.*m34*RSe45633p + 1./36.*m34*RSp45633e - 1./
         18.*m34*RS356pe - 1./18.*m34*RSe563p - 1./18.*m34*RSp563e - 1./
         36.*m34*RC1456pe - 1./36.*m34*RSe45611p - 1./36.*m34*RSp45611e
          + 1./18.*m34*RS156pe + 1./18.*m34*RSe561p + 1./18.*m34*
         RSp561e );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(m14,L) * (  - 2./45.*m33 - 5./
         108.*m33*RS34561pe - 1./36.*m33*RS14563pe - 1./36.*m33*
         RSe45613p - 1./36.*m33*RSp45613e - 1./36.*m33*RC3456pe - 1./54.
         *m33*RSe45633p - 1./54.*m33*RSp45633e + 7./108.*m33*RS356pe + 
         1./36.*m33*RSe563p + 1./36.*m33*RSp563e - 1./108.*m33*RC1456pe
          - 1./108.*m33*RSe45611p - 1./108.*m33*RSp45611e + 1./108.*m33
         *RS156pe + 1./54.*m33*RS456pe + 1./108.*m33*RSe561p + 1./108.*
         m33*RSp561e + 1./54.*m33*RSe564p + 1./54.*m33*RSp564e - 1./90.
         *m14 - 1./36.*m14*RC3456pe - 1./36.*m14*RSe45633p - 1./36.*m14
         *RSp45633e + 1./18.*m14*RS356pe + 1./18.*m14*RSe563p + 1./18.*
         m14*RSp563e + 1./36.*m14*RC1456pe + 1./36.*m14*RSe45611p + 1./
         36.*m14*RSp45611e - 1./18.*m14*RS156pe - 1./18.*m14*RSe561p - 
         1./18.*m14*RSp561e - 3./20.*m13 - 1./36.*m13*RS34561pe - 1./36.
         *m13*RS14563pe - 1./36.*m13*RSe45613p - 1./36.*m13*RSp45613e
          + 1./36.*m13*RC3456pe + 1./36.*m13*RSe45633p + 1./36.*m13*
         RSp45633e );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(m14,L) * (  - 1./36.*m13*RS356pe
          - 1./36.*m13*RSe563p - 1./36.*m13*RSp563e + 1./36.*m13*
         RC1456pe + 1./36.*m13*RSe45611p + 1./36.*m13*RSp45611e - 1./36.
         *m13*RS156pe + 1./36.*m13*RS456pe - 1./36.*m13*RSe561p - 1./36.
         *m13*RSp561e + 1./36.*m13*RSe564p + 1./36.*m13*RSp564e - 2./45.
         *m11 - 1./36.*m11*RS34561pe - 5./108.*m11*RS14563pe - 1./36.*
         m11*RSe45613p - 1./36.*m11*RSp45613e - 1./108.*m11*RC3456pe - 
         1./108.*m11*RSe45633p - 1./108.*m11*RSp45633e + 1./108.*m11*
         RS356pe + 1./108.*m11*RSe563p + 1./108.*m11*RSp563e - 1./36.*
         m11*RC1456pe - 1./54.*m11*RSe45611p - 1./54.*m11*RSp45611e + 7.
         /108.*m11*RS156pe + 1./54.*m11*RS456pe + 1./36.*m11*RSe561p + 
         1./36.*m11*RSp561e + 1./54.*m11*RSe564p + 1./54.*m11*RSp564e )
         ;

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(m15,L) * (  - 1./20.*m55 + 1./9.
         *m45 - 1./20.*m44 - 1./90.*m34 - 2./45.*m33 - 1./90.*m15 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(m16,L) * (  - 1./20.*m66 + 1./9.
         *m46 - 1./20.*m44 - 1./90.*m34 - 2./45.*m33 - 1./90.*m16 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(m33,L) * (  - 29./540.*RD3456ep
          - 7./540.*m44*RS34561pe - 1./54.*m44*RS14563pe + 7./540.*m44*
         RC3456pe - 1./60.*m44*RS356pe - 1./54.*m44*RS156pe - 1./90.*
         m34*RS34561pe + 1./27.*m34*RS14563pe - 1./60.*m34*RC3456pe + 2.
         /45.*m34*RS356pe - 1./27.*m34*RS156pe + 43./540.*m33*RS34561pe
          + 1./27.*m33*RSe45613p + 1./27.*m33*RSp45613e + 13./1080.*m33
         *RC3456pe - 1./27.*m33*RSe45633p - 1./27.*m33*RSp45633e - 1./
         45.*m33*RS356pe + 1./27.*m33*RSe563p + 1./27.*m33*RSp563e - 2./
         27.*m33*RS156pe - 1./27.*m33*RSe561p - 1./27.*m33*RSp561e + 1./
         27.*m14*RS34561pe - 1./108.*m14*RC3456pe - 1./54.*m14*RS356pe
          - 11./540.*m13*RS34561pe + 1./27.*m13*RS14563pe - 1./135.*m13
         *RC3456pe + 19./540.*m13*RS356pe - 1./27.*m13*RS156pe + 1./90.
         *m11*RS34561pe + 1./54.*m11*RS14563pe + 1./120.*m11*RC3456pe
          - 1./20.*m11*RS356pe - 1./54.*m11*RS156pe + 1./27.*RS(m33,m44
         ,m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11 );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(m34,L) * (  - 11./180.*m44 + 11./
         90.*m34 - 7./90.*m33 + 1./30.*m13 - 1./60.*m11 );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(m35,L) * (  - 11./360.*m55 - 11./
         360.*m44 + 11./180.*m35 + 11./180.*m34 - 7./90.*m33 + 1./30.*
         m13 - 1./60.*m11 );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(m36,L) * (  - 11./360.*m66 - 11./
         360.*m44 + 11./180.*m36 + 11./180.*m34 - 7./90.*m33 + 1./30.*
         m13 - 1./60.*m11 );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(m44,L) * (  - 1./20.*m44 + 1./60.
         *m44*RS456pe + 13./180.*m34 - 13./540.*m34*RS456pe - 1./360.*
         m33 + 1./1080.*m33*RS456pe + 1./36.*m14 - 1./108.*m14*RS456pe
          - 1./15.*m13 + 1./45.*m13*RS456pe + 7./360.*m11 - 7./1080.*
         m11*RS456pe );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(m45,L) * ( 7./360.*m55 - 1./15.*
         m45 - 1./360.*m44 + 13./180.*m34 - 1./360.*m33 + 1./36.*m15 - 
         1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(m46,L) * ( 7./360.*m66 - 1./15.*
         m46 - 1./360.*m44 + 13./180.*m34 - 1./360.*m33 + 1./36.*m16 - 
         1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(mp2,L) * ( 11./135.*mp2*
         RSp45613e - 29./540.*mp2*RSp45633e + 7./270.*mp2*RSp563e - 1./
         180.*mp2*RSp45611e - 19./270.*mp2*RSp561e + 1./45.*mp2*RSp564e
          - 7./540.*m44*RSp45613e + 7./540.*m44*RSp45633e - 1./60.*m44*
         RSp563e + 1./60.*m44*RSp45611e + 1./180.*m44*RSp561e - 1./180.
         *m44*RSp564e - 1./90.*m34*RSp45613e - 1./60.*m34*RSp45633e + 2.
         /45.*m34*RSp563e + 1./270.*m34*RSp45611e + 1./270.*m34*RSp561e
          - 13./540.*m34*RSp564e - 1./540.*m33*RSp45613e + 37./360.*m33
         *RSp45633e - 23./270.*m33*RSp563e + 2./135.*m33*RSp45611e - 17.
         /540.*m33*RSp561e + 1./1080.*m33*RSp564e + 1./27.*m14*
         RSp45613e - 1./108.*m14*RSp45633e - 1./54.*m14*RSp563e - 1./27.
         *m14*RSp45611e + 1./27.*m14*RSp561e - 1./108.*m14*RSp564e - 11.
         /540.*m13*RSp45613e - 1./135.*m13*RSp45633e + 19./540.*m13*
         RSp563e + 1./20.*m13*RSp45611e - 43./540.*m13*RSp561e + 1./45.
         *m13*RSp564e + 4./135.*m11*RSp45613e + 1./120.*m11*RSp45633e
          - 1./20.*m11*RSp563e );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(mp2,L) * ( 11./270.*m11*
         RSp45611e - 1./45.*m11*RSp561e - 7./1080.*m11*RSp564e );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(me2,L) * ( 11./135.*me2*
         RSe45613p - 29./540.*me2*RSe45633p + 7./270.*me2*RSe563p - 1./
         180.*me2*RSe45611p - 19./270.*me2*RSe561p + 1./45.*me2*RSe564p
          - 7./540.*m44*RSe45613p + 7./540.*m44*RSe45633p - 1./60.*m44*
         RSe563p + 1./60.*m44*RSe45611p + 1./180.*m44*RSe561p - 1./180.
         *m44*RSe564p - 1./90.*m34*RSe45613p - 1./60.*m34*RSe45633p + 2.
         /45.*m34*RSe563p + 1./270.*m34*RSe45611p + 1./270.*m34*RSe561p
          - 13./540.*m34*RSe564p - 1./540.*m33*RSe45613p + 37./360.*m33
         *RSe45633p - 23./270.*m33*RSe563p + 2./135.*m33*RSe45611p - 17.
         /540.*m33*RSe561p + 1./1080.*m33*RSe564p + 1./27.*m14*
         RSe45613p - 1./108.*m14*RSe45633p - 1./54.*m14*RSe563p - 1./27.
         *m14*RSe45611p + 1./27.*m14*RSe561p - 1./108.*m14*RSe564p - 11.
         /540.*m13*RSe45613p - 1./135.*m13*RSe45633p + 19./540.*m13*
         RSe563p + 1./20.*m13*RSe45611p - 43./540.*m13*RSe561p + 1./45.
         *m13*RSe564p + 4./135.*m11*RSe45613p + 1./120.*m11*RSe45633p
          - 1./20.*m11*RSe563p );

      M13p6x23RV +=  + Ab(m34,mu2)*AbV(me2,L) * ( 11./270.*m11*
         RSe45611p - 1./45.*m11*RSe561p - 7./1080.*m11*RSe564p );

      M13p6x23RV +=  + Ab(m34,mu2)*BbV(m11,m33,L) * (  - 1./108.*m33*
         m44*RS14563pe - 1./108.*m33*m44*RS156pe + 1./54.*m33*m34*
         RS14563pe - 1./54.*m33*m34*RS156pe - 1./36.*pow(m33,2)*
         RS14563pe - 1./108.*pow(m33,2)*RS156pe - 1./54.*m13*m44*
         RS14563pe - 1./54.*m13*m44*RS156pe + 1./27.*m13*m34*RS14563pe
          - 1./27.*m13*m34*RS156pe - 1./18.*m13*m33*RS14563pe - 1./54.*
         m13*m33*RS156pe - 1./36.*m11*m44*RS14563pe - 1./36.*m11*m44*
         RS156pe + 1./18.*m11*m34*RS14563pe - 1./18.*m11*m34*RS156pe - 
         1./36.*m11*m33*RS14563pe - 1./12.*m11*m33*RS156pe + 1./27.*m11
         *m13*RS14563pe - 1./27.*m11*m13*RS156pe + 1./54.*pow(m11,2)*
         RS14563pe - 1./54.*pow(m11,2)*RS156pe );

      M13p6x23RV +=  + Ab(m34,mu2)*BbV(m11,L) * ( 1./60.*m44*RD1456ep
          + 1./270.*m34*RD1456ep + 2./135.*m33*RD1456ep - 1./27.*m14*
         RD1456ep + 1./20.*m13*RD1456ep + 19./540.*m11*RD1456ep );

      M13p6x23RV +=  + Ab(m34,mu2)*BbV(m33,mp2,L) * ( 1./27.*m33*mp2*
         RSp45613e - 1./27.*m33*mp2*RSp45633e + 1./27.*m33*mp2*RSp563e
          - 1./27.*m33*mp2*RSp561e - 1./36.*m33*m44*RSp45613e - 1./108.
         *m33*m44*RSp45633e - 1./108.*m33*m44*RSp563e - 1./36.*m33*m44*
         RSp561e + 1./18.*m33*m34*RSp45613e + 1./54.*m33*m34*RSp45633e
          - 1./54.*m33*m34*RSp563e - 1./18.*m33*m34*RSp561e - 7./108.*
         pow(m33,2)*RSp45613e + 1./36.*pow(m33,2)*RSp45633e - 7./108.*
         pow(m33,2)*RSp563e - 5./108.*pow(m33,2)*RSp561e - 1./54.*m13*
         m44*RSp45613e + 1./54.*m13*m44*RSp45633e + 1./54.*m13*m44*
         RSp563e - 1./54.*m13*m44*RSp561e + 1./27.*m13*m34*RSp45613e - 
         1./27.*m13*m34*RSp45633e + 1./27.*m13*m34*RSp563e - 1./27.*m13
         *m34*RSp561e - 1./54.*m13*m33*RSp45613e + 1./54.*m13*m33*
         RSp45633e + 1./18.*m13*m33*RSp563e - 1./18.*m13*m33*RSp561e - 
         1./108.*m11*m44*RSp45613e - 1./108.*m11*m44*RSp45633e - 1./108.
         *m11*m44*RSp563e - 1./108.*m11*m44*RSp561e + 1./54.*m11*m34*
         RSp45613e );

      M13p6x23RV +=  + Ab(m34,mu2)*BbV(m33,mp2,L) * ( 1./54.*m11*m34*
         RSp45633e - 1./54.*m11*m34*RSp563e - 1./54.*m11*m34*RSp561e - 
         1./108.*m11*m33*RSp45613e - 1./108.*m11*m33*RSp45633e - 1./36.
         *m11*m33*RSp563e - 1./36.*m11*m33*RSp561e );

      M13p6x23RV +=  + Ab(m34,mu2)*BbV(m33,me2,L) * ( 1./27.*m33*me2*
         RSe45613p - 1./27.*m33*me2*RSe45633p + 1./27.*m33*me2*RSe563p
          - 1./27.*m33*me2*RSe561p - 1./36.*m33*m44*RSe45613p - 1./108.
         *m33*m44*RSe45633p - 1./108.*m33*m44*RSe563p - 1./36.*m33*m44*
         RSe561p + 1./18.*m33*m34*RSe45613p + 1./54.*m33*m34*RSe45633p
          - 1./54.*m33*m34*RSe563p - 1./18.*m33*m34*RSe561p - 7./108.*
         pow(m33,2)*RSe45613p + 1./36.*pow(m33,2)*RSe45633p - 7./108.*
         pow(m33,2)*RSe563p - 5./108.*pow(m33,2)*RSe561p - 1./54.*m13*
         m44*RSe45613p + 1./54.*m13*m44*RSe45633p + 1./54.*m13*m44*
         RSe563p - 1./54.*m13*m44*RSe561p + 1./27.*m13*m34*RSe45613p - 
         1./27.*m13*m34*RSe45633p + 1./27.*m13*m34*RSe563p - 1./27.*m13
         *m34*RSe561p - 1./54.*m13*m33*RSe45613p + 1./54.*m13*m33*
         RSe45633p + 1./18.*m13*m33*RSe563p - 1./18.*m13*m33*RSe561p - 
         1./108.*m11*m44*RSe45613p - 1./108.*m11*m44*RSe45633p - 1./108.
         *m11*m44*RSe563p - 1./108.*m11*m44*RSe561p + 1./54.*m11*m34*
         RSe45613p );

      M13p6x23RV +=  + Ab(m34,mu2)*BbV(m33,me2,L) * ( 1./54.*m11*m34*
         RSe45633p - 1./54.*m11*m34*RSe563p - 1./54.*m11*m34*RSe561p - 
         1./108.*m11*m33*RSe45613p - 1./108.*m11*m33*RSe45633p - 1./36.
         *m11*m33*RSe563p - 1./36.*m11*m33*RSe561p );

      M13p6x23RV +=  + Ab(m34,mu2)*BbV(m33,L) * ( 7./540.*m44*RD3456ep
          - 1./60.*m34*RD3456ep + 13./1080.*m33*RD3456ep - 1./36.*m33*
         m44*RS34561pe - 1./108.*m33*m44*RC3456pe - 1./108.*m33*m44*
         RS356pe + 1./18.*m33*m34*RS34561pe + 1./54.*m33*m34*RC3456pe
          - 1./54.*m33*m34*RS356pe - 1./36.*pow(m33,2)*RS34561pe - 1./
         108.*pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS356pe - 1./108.
         *m14*RD3456ep - 1./135.*m13*RD3456ep - 1./54.*m13*m44*
         RS34561pe + 1./54.*m13*m44*RC3456pe + 1./54.*m13*m44*RS356pe
          + 1./27.*m13*m34*RS34561pe - 1./27.*m13*m34*RC3456pe + 1./27.
         *m13*m34*RS356pe - 1./54.*m13*m33*RS34561pe + 1./54.*m13*m33*
         RC3456pe + 1./18.*m13*m33*RS356pe + 1./120.*m11*RD3456ep - 1./
         108.*m11*m44*RS34561pe - 1./108.*m11*m44*RC3456pe - 1./108.*
         m11*m44*RS356pe + 1./54.*m11*m34*RS34561pe + 1./54.*m11*m34*
         RC3456pe - 1./54.*m11*m34*RS356pe - 1./108.*m11*m33*RS34561pe
          - 1./108.*m11*m33*RC3456pe - 1./36.*m11*m33*RS356pe );

      M13p6x23RV +=  + Ab(m34,mu2)*BbV(m34,L) * ( 1./72.*pow(m44,2) - 1.
         /18.*m34*m44 + 1./18.*pow(m34,2) + 1./24.*m33*m44 - 1./12.*m33
         *m34 + 1./36.*pow(m33,2) - 1./36.*m13*m44 + 1./18.*m13*m34 - 1.
         /36.*m13*m33 + 1./72.*m11*m44 - 1./36.*m11*m34 + 1./72.*m11*
         m33 );

      M13p6x23RV +=  + Ab(m34,mu2)*BbV(m35,L) * ( 1./144.*pow(m55,2) + 
         1./144.*m44*m55 - 1./36.*m35*m55 - 1./72.*m35*m44 + 1./36.*
         pow(m35,2) - 1./72.*m34*m55 + 1./36.*m34*m35 + 1./36.*m33*m55
          + 1./72.*m33*m44 - 1./18.*m33*m35 - 1./36.*m33*m34 + 1./36.*
         pow(m33,2) - 1./72.*m13*m55 - 1./72.*m13*m44 + 1./36.*m13*m35
          + 1./36.*m13*m34 - 1./36.*m13*m33 + 1./144.*m11*m55 + 1./144.
         *m11*m44 - 1./72.*m11*m35 - 1./72.*m11*m34 + 1./72.*m11*m33 );

      M13p6x23RV +=  + Ab(m34,mu2)*BbV(m36,L) * ( 1./144.*pow(m66,2) + 
         1./144.*m44*m66 - 1./36.*m36*m66 - 1./72.*m36*m44 + 1./36.*
         pow(m36,2) - 1./72.*m34*m66 + 1./36.*m34*m36 + 1./36.*m33*m66
          + 1./72.*m33*m44 - 1./18.*m33*m36 - 1./36.*m33*m34 + 1./36.*
         pow(m33,2) - 1./72.*m13*m66 - 1./72.*m13*m44 + 1./36.*m13*m36
          + 1./36.*m13*m34 - 1./36.*m13*m33 + 1./144.*m11*m66 + 1./144.
         *m11*m44 - 1./72.*m11*m36 - 1./72.*m11*m34 + 1./72.*m11*m33 );

      M13p6x23RV +=  + Ab(m34,mu2)*CbV(m33,L) * (  - 1./108.*m33*m44*
         RD3456ep + 1./54.*m33*m34*RD3456ep - 1./108.*pow(m33,2)*
         RD3456ep + 1./54.*m13*m44*RD3456ep - 1./27.*m13*m34*RD3456ep
          + 1./54.*m13*m33*RD3456ep - 1./108.*m11*m44*RD3456ep + 1./54.
         *m11*m34*RD3456ep - 1./108.*m11*m33*RD3456ep );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(m11,L) * (  - 1./180.*RD1456ep
          + 1./180.*m55*RS14563pe + 1./60.*m55*RC1456pe + 13./540.*m55*
         RS146pe - 13./270.*m35*RS14563pe + 1./270.*m35*RC1456pe + 11./
         270.*m35*RS146pe + 1./60.*m33*RS14563pe + 2./135.*m33*RC1456pe
          + 1./180.*m33*RS146pe + 1./27.*m15*RS14563pe - 1./27.*m15*
         RC1456pe + 1./27.*m15*RS146pe - 31./540.*m13*RS14563pe + 1./20.
         *m13*RC1456pe - 23./540.*m13*RS146pe + 1./18.*m11*RS14563pe + 
         19./540.*m11*RC1456pe - 2./27.*m11*RS146pe + 1./54.*RS(m11,m44
         ,m55,m66,m33,me2,mp2)*m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,
         mp2)*m11 );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(m14,L) * (  - 1./20.*m55 + 1./9.
         *m45 - 1./20.*m44 - 1./90.*m35 - 2./45.*m33 - 1./90.*m14 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(m15,L) * (  - 1./108.*RD3456ep
          - 1./108.*RD1456ep - 1./54.*me2*RSe45613p - 1./108.*me2*
         RSe45633p + 1./27.*me2*RSe463p - 1./108.*me2*RSe45611p + 1./27.
         *me2*RSe461p - 1./27.*me2*RSe465p - 1./54.*mp2*RSp45613e - 1./
         108.*mp2*RSp45633e + 1./27.*mp2*RSp463e - 1./108.*mp2*
         RSp45611e + 1./27.*mp2*RSp461e - 1./27.*mp2*RSp465e + 1./90.*
         m55 - 1./54.*m55*RS34561pe - 1./54.*m55*RS14563pe - 1./54.*m55
         *RSe45613p - 1./54.*m55*RSp45613e - 1./108.*m55*RC3456pe - 1./
         108.*m55*RSe45633p - 1./108.*m55*RSp45633e - 1./108.*m55*
         RC1456pe - 1./108.*m55*RSe45611p - 1./108.*m55*RSp45611e + 1./
         27.*m55*RSe465p + 1./27.*m55*RSp465e - 1./90.*m35 + 1./36.*m35
         *RC3456pe + 1./36.*m35*RSe45633p + 1./36.*m35*RSp45633e - 1./
         18.*m35*RS346pe - 1./18.*m35*RSe463p - 1./18.*m35*RSp463e - 1./
         36.*m35*RC1456pe - 1./36.*m35*RSe45611p - 1./36.*m35*RSp45611e
          + 1./18.*m35*RS146pe + 1./18.*m35*RSe461p + 1./18.*m35*
         RSp461e );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(m15,L) * (  - 2./45.*m33 - 5./
         108.*m33*RS34561pe - 1./36.*m33*RS14563pe - 1./36.*m33*
         RSe45613p - 1./36.*m33*RSp45613e - 1./36.*m33*RC3456pe - 1./54.
         *m33*RSe45633p - 1./54.*m33*RSp45633e + 7./108.*m33*RS346pe + 
         1./36.*m33*RSe463p + 1./36.*m33*RSp463e - 1./108.*m33*RC1456pe
          - 1./108.*m33*RSe45611p - 1./108.*m33*RSp45611e + 1./108.*m33
         *RS146pe + 1./54.*m33*RS546pe + 1./108.*m33*RSe461p + 1./108.*
         m33*RSp461e + 1./54.*m33*RSe465p + 1./54.*m33*RSp465e - 1./90.
         *m15 - 1./36.*m15*RC3456pe - 1./36.*m15*RSe45633p - 1./36.*m15
         *RSp45633e + 1./18.*m15*RS346pe + 1./18.*m15*RSe463p + 1./18.*
         m15*RSp463e + 1./36.*m15*RC1456pe + 1./36.*m15*RSe45611p + 1./
         36.*m15*RSp45611e - 1./18.*m15*RS146pe - 1./18.*m15*RSe461p - 
         1./18.*m15*RSp461e - 3./20.*m13 - 1./36.*m13*RS34561pe - 1./36.
         *m13*RS14563pe - 1./36.*m13*RSe45613p - 1./36.*m13*RSp45613e
          + 1./36.*m13*RC3456pe + 1./36.*m13*RSe45633p + 1./36.*m13*
         RSp45633e );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(m15,L) * (  - 1./36.*m13*RS346pe
          - 1./36.*m13*RSe463p - 1./36.*m13*RSp463e + 1./36.*m13*
         RC1456pe + 1./36.*m13*RSe45611p + 1./36.*m13*RSp45611e - 1./36.
         *m13*RS146pe + 1./36.*m13*RS546pe - 1./36.*m13*RSe461p - 1./36.
         *m13*RSp461e + 1./36.*m13*RSe465p + 1./36.*m13*RSp465e - 2./45.
         *m11 - 1./36.*m11*RS34561pe - 5./108.*m11*RS14563pe - 1./36.*
         m11*RSe45613p - 1./36.*m11*RSp45613e - 1./108.*m11*RC3456pe - 
         1./108.*m11*RSe45633p - 1./108.*m11*RSp45633e + 1./108.*m11*
         RS346pe + 1./108.*m11*RSe463p + 1./108.*m11*RSp463e - 1./36.*
         m11*RC1456pe - 1./54.*m11*RSe45611p - 1./54.*m11*RSp45611e + 7.
         /108.*m11*RS146pe + 1./54.*m11*RS546pe + 1./36.*m11*RSe461p + 
         1./36.*m11*RSp461e + 1./54.*m11*RSe465p + 1./54.*m11*RSp465e )
         ;

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(m16,L) * (  - 1./20.*m66 + 1./9.
         *m56 - 1./20.*m55 - 1./90.*m35 - 2./45.*m33 - 1./90.*m16 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(m33,L) * (  - 29./540.*RD3456ep
          - 7./540.*m55*RS34561pe - 1./54.*m55*RS14563pe + 7./540.*m55*
         RC3456pe - 1./60.*m55*RS346pe - 1./54.*m55*RS146pe - 1./90.*
         m35*RS34561pe + 1./27.*m35*RS14563pe - 1./60.*m35*RC3456pe + 2.
         /45.*m35*RS346pe - 1./27.*m35*RS146pe + 43./540.*m33*RS34561pe
          + 1./27.*m33*RSe45613p + 1./27.*m33*RSp45613e + 13./1080.*m33
         *RC3456pe - 1./27.*m33*RSe45633p - 1./27.*m33*RSp45633e - 1./
         45.*m33*RS346pe + 1./27.*m33*RSe463p + 1./27.*m33*RSp463e - 2./
         27.*m33*RS146pe - 1./27.*m33*RSe461p - 1./27.*m33*RSp461e + 1./
         27.*m15*RS34561pe - 1./108.*m15*RC3456pe - 1./54.*m15*RS346pe
          - 11./540.*m13*RS34561pe + 1./27.*m13*RS14563pe - 1./135.*m13
         *RC3456pe + 19./540.*m13*RS346pe - 1./27.*m13*RS146pe + 1./90.
         *m11*RS34561pe + 1./54.*m11*RS14563pe + 1./120.*m11*RC3456pe
          - 1./20.*m11*RS346pe - 1./54.*m11*RS146pe + 1./27.*RS(m33,m44
         ,m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11 );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(m34,L) * (  - 11./360.*m55 - 11./
         360.*m44 + 11./180.*m35 + 11./180.*m34 - 7./90.*m33 + 1./30.*
         m13 - 1./60.*m11 );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(m35,L) * (  - 11./180.*m55 + 11./
         90.*m35 - 7./90.*m33 + 1./30.*m13 - 1./60.*m11 );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(m36,L) * (  - 11./360.*m66 - 11./
         360.*m55 + 11./180.*m36 + 11./180.*m35 - 7./90.*m33 + 1./30.*
         m13 - 1./60.*m11 );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(m45,L) * (  - 1./360.*m55 - 1./
         15.*m45 + 7./360.*m44 + 13./180.*m35 - 1./360.*m33 + 1./36.*
         m14 - 1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(m55,L) * (  - 1./20.*m55 + 1./60.
         *m55*RS546pe + 13./180.*m35 - 13./540.*m35*RS546pe - 1./360.*
         m33 + 1./1080.*m33*RS546pe + 1./36.*m15 - 1./108.*m15*RS546pe
          - 1./15.*m13 + 1./45.*m13*RS546pe + 7./360.*m11 - 7./1080.*
         m11*RS546pe );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(m56,L) * ( 7./360.*m66 - 1./15.*
         m56 - 1./360.*m55 + 13./180.*m35 - 1./360.*m33 + 1./36.*m16 - 
         1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(mp2,L) * ( 11./135.*mp2*
         RSp45613e - 29./540.*mp2*RSp45633e + 7./270.*mp2*RSp463e - 1./
         180.*mp2*RSp45611e - 19./270.*mp2*RSp461e + 1./45.*mp2*RSp465e
          - 7./540.*m55*RSp45613e + 7./540.*m55*RSp45633e - 1./60.*m55*
         RSp463e + 1./60.*m55*RSp45611e + 1./180.*m55*RSp461e - 1./180.
         *m55*RSp465e - 1./90.*m35*RSp45613e - 1./60.*m35*RSp45633e + 2.
         /45.*m35*RSp463e + 1./270.*m35*RSp45611e + 1./270.*m35*RSp461e
          - 13./540.*m35*RSp465e - 1./540.*m33*RSp45613e + 37./360.*m33
         *RSp45633e - 23./270.*m33*RSp463e + 2./135.*m33*RSp45611e - 17.
         /540.*m33*RSp461e + 1./1080.*m33*RSp465e + 1./27.*m15*
         RSp45613e - 1./108.*m15*RSp45633e - 1./54.*m15*RSp463e - 1./27.
         *m15*RSp45611e + 1./27.*m15*RSp461e - 1./108.*m15*RSp465e - 11.
         /540.*m13*RSp45613e - 1./135.*m13*RSp45633e + 19./540.*m13*
         RSp463e + 1./20.*m13*RSp45611e - 43./540.*m13*RSp461e + 1./45.
         *m13*RSp465e + 4./135.*m11*RSp45613e + 1./120.*m11*RSp45633e
          - 1./20.*m11*RSp463e );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(mp2,L) * ( 11./270.*m11*
         RSp45611e - 1./45.*m11*RSp461e - 7./1080.*m11*RSp465e );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(me2,L) * ( 11./135.*me2*
         RSe45613p - 29./540.*me2*RSe45633p + 7./270.*me2*RSe463p - 1./
         180.*me2*RSe45611p - 19./270.*me2*RSe461p + 1./45.*me2*RSe465p
          - 7./540.*m55*RSe45613p + 7./540.*m55*RSe45633p - 1./60.*m55*
         RSe463p + 1./60.*m55*RSe45611p + 1./180.*m55*RSe461p - 1./180.
         *m55*RSe465p - 1./90.*m35*RSe45613p - 1./60.*m35*RSe45633p + 2.
         /45.*m35*RSe463p + 1./270.*m35*RSe45611p + 1./270.*m35*RSe461p
          - 13./540.*m35*RSe465p - 1./540.*m33*RSe45613p + 37./360.*m33
         *RSe45633p - 23./270.*m33*RSe463p + 2./135.*m33*RSe45611p - 17.
         /540.*m33*RSe461p + 1./1080.*m33*RSe465p + 1./27.*m15*
         RSe45613p - 1./108.*m15*RSe45633p - 1./54.*m15*RSe463p - 1./27.
         *m15*RSe45611p + 1./27.*m15*RSe461p - 1./108.*m15*RSe465p - 11.
         /540.*m13*RSe45613p - 1./135.*m13*RSe45633p + 19./540.*m13*
         RSe463p + 1./20.*m13*RSe45611p - 43./540.*m13*RSe461p + 1./45.
         *m13*RSe465p + 4./135.*m11*RSe45613p + 1./120.*m11*RSe45633p
          - 1./20.*m11*RSe463p );

      M13p6x23RV +=  + Ab(m35,mu2)*AbV(me2,L) * ( 11./270.*m11*
         RSe45611p - 1./45.*m11*RSe461p - 7./1080.*m11*RSe465p );

      M13p6x23RV +=  + Ab(m35,mu2)*BbV(m11,m33,L) * (  - 1./108.*m33*
         m55*RS14563pe - 1./108.*m33*m55*RS146pe + 1./54.*m33*m35*
         RS14563pe - 1./54.*m33*m35*RS146pe - 1./36.*pow(m33,2)*
         RS14563pe - 1./108.*pow(m33,2)*RS146pe - 1./54.*m13*m55*
         RS14563pe - 1./54.*m13*m55*RS146pe + 1./27.*m13*m35*RS14563pe
          - 1./27.*m13*m35*RS146pe - 1./18.*m13*m33*RS14563pe - 1./54.*
         m13*m33*RS146pe - 1./36.*m11*m55*RS14563pe - 1./36.*m11*m55*
         RS146pe + 1./18.*m11*m35*RS14563pe - 1./18.*m11*m35*RS146pe - 
         1./36.*m11*m33*RS14563pe - 1./12.*m11*m33*RS146pe + 1./27.*m11
         *m13*RS14563pe - 1./27.*m11*m13*RS146pe + 1./54.*pow(m11,2)*
         RS14563pe - 1./54.*pow(m11,2)*RS146pe );

      M13p6x23RV +=  + Ab(m35,mu2)*BbV(m11,L) * ( 1./60.*m55*RD1456ep
          + 1./270.*m35*RD1456ep + 2./135.*m33*RD1456ep - 1./27.*m15*
         RD1456ep + 1./20.*m13*RD1456ep + 19./540.*m11*RD1456ep );

      M13p6x23RV +=  + Ab(m35,mu2)*BbV(m33,mp2,L) * ( 1./27.*m33*mp2*
         RSp45613e - 1./27.*m33*mp2*RSp45633e + 1./27.*m33*mp2*RSp463e
          - 1./27.*m33*mp2*RSp461e - 1./36.*m33*m55*RSp45613e - 1./108.
         *m33*m55*RSp45633e - 1./108.*m33*m55*RSp463e - 1./36.*m33*m55*
         RSp461e + 1./18.*m33*m35*RSp45613e + 1./54.*m33*m35*RSp45633e
          - 1./54.*m33*m35*RSp463e - 1./18.*m33*m35*RSp461e - 7./108.*
         pow(m33,2)*RSp45613e + 1./36.*pow(m33,2)*RSp45633e - 7./108.*
         pow(m33,2)*RSp463e - 5./108.*pow(m33,2)*RSp461e - 1./54.*m13*
         m55*RSp45613e + 1./54.*m13*m55*RSp45633e + 1./54.*m13*m55*
         RSp463e - 1./54.*m13*m55*RSp461e + 1./27.*m13*m35*RSp45613e - 
         1./27.*m13*m35*RSp45633e + 1./27.*m13*m35*RSp463e - 1./27.*m13
         *m35*RSp461e - 1./54.*m13*m33*RSp45613e + 1./54.*m13*m33*
         RSp45633e + 1./18.*m13*m33*RSp463e - 1./18.*m13*m33*RSp461e - 
         1./108.*m11*m55*RSp45613e - 1./108.*m11*m55*RSp45633e - 1./108.
         *m11*m55*RSp463e - 1./108.*m11*m55*RSp461e + 1./54.*m11*m35*
         RSp45613e );

      M13p6x23RV +=  + Ab(m35,mu2)*BbV(m33,mp2,L) * ( 1./54.*m11*m35*
         RSp45633e - 1./54.*m11*m35*RSp463e - 1./54.*m11*m35*RSp461e - 
         1./108.*m11*m33*RSp45613e - 1./108.*m11*m33*RSp45633e - 1./36.
         *m11*m33*RSp463e - 1./36.*m11*m33*RSp461e );

      M13p6x23RV +=  + Ab(m35,mu2)*BbV(m33,me2,L) * ( 1./27.*m33*me2*
         RSe45613p - 1./27.*m33*me2*RSe45633p + 1./27.*m33*me2*RSe463p
          - 1./27.*m33*me2*RSe461p - 1./36.*m33*m55*RSe45613p - 1./108.
         *m33*m55*RSe45633p - 1./108.*m33*m55*RSe463p - 1./36.*m33*m55*
         RSe461p + 1./18.*m33*m35*RSe45613p + 1./54.*m33*m35*RSe45633p
          - 1./54.*m33*m35*RSe463p - 1./18.*m33*m35*RSe461p - 7./108.*
         pow(m33,2)*RSe45613p + 1./36.*pow(m33,2)*RSe45633p - 7./108.*
         pow(m33,2)*RSe463p - 5./108.*pow(m33,2)*RSe461p - 1./54.*m13*
         m55*RSe45613p + 1./54.*m13*m55*RSe45633p + 1./54.*m13*m55*
         RSe463p - 1./54.*m13*m55*RSe461p + 1./27.*m13*m35*RSe45613p - 
         1./27.*m13*m35*RSe45633p + 1./27.*m13*m35*RSe463p - 1./27.*m13
         *m35*RSe461p - 1./54.*m13*m33*RSe45613p + 1./54.*m13*m33*
         RSe45633p + 1./18.*m13*m33*RSe463p - 1./18.*m13*m33*RSe461p - 
         1./108.*m11*m55*RSe45613p - 1./108.*m11*m55*RSe45633p - 1./108.
         *m11*m55*RSe463p - 1./108.*m11*m55*RSe461p + 1./54.*m11*m35*
         RSe45613p );

      M13p6x23RV +=  + Ab(m35,mu2)*BbV(m33,me2,L) * ( 1./54.*m11*m35*
         RSe45633p - 1./54.*m11*m35*RSe463p - 1./54.*m11*m35*RSe461p - 
         1./108.*m11*m33*RSe45613p - 1./108.*m11*m33*RSe45633p - 1./36.
         *m11*m33*RSe463p - 1./36.*m11*m33*RSe461p );

      M13p6x23RV +=  + Ab(m35,mu2)*BbV(m33,L) * ( 7./540.*m55*RD3456ep
          - 1./60.*m35*RD3456ep + 13./1080.*m33*RD3456ep - 1./36.*m33*
         m55*RS34561pe - 1./108.*m33*m55*RC3456pe - 1./108.*m33*m55*
         RS346pe + 1./18.*m33*m35*RS34561pe + 1./54.*m33*m35*RC3456pe
          - 1./54.*m33*m35*RS346pe - 1./36.*pow(m33,2)*RS34561pe - 1./
         108.*pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS346pe - 1./108.
         *m15*RD3456ep - 1./135.*m13*RD3456ep - 1./54.*m13*m55*
         RS34561pe + 1./54.*m13*m55*RC3456pe + 1./54.*m13*m55*RS346pe
          + 1./27.*m13*m35*RS34561pe - 1./27.*m13*m35*RC3456pe + 1./27.
         *m13*m35*RS346pe - 1./54.*m13*m33*RS34561pe + 1./54.*m13*m33*
         RC3456pe + 1./18.*m13*m33*RS346pe + 1./120.*m11*RD3456ep - 1./
         108.*m11*m55*RS34561pe - 1./108.*m11*m55*RC3456pe - 1./108.*
         m11*m55*RS346pe + 1./54.*m11*m35*RS34561pe + 1./54.*m11*m35*
         RC3456pe - 1./54.*m11*m35*RS346pe - 1./108.*m11*m33*RS34561pe
          - 1./108.*m11*m33*RC3456pe - 1./36.*m11*m33*RS346pe );

      M13p6x23RV +=  + Ab(m35,mu2)*BbV(m34,L) * ( 1./144.*m44*m55 + 1./
         144.*pow(m44,2) - 1./72.*m35*m44 - 1./72.*m34*m55 - 1./36.*m34
         *m44 + 1./36.*m34*m35 + 1./36.*pow(m34,2) + 1./72.*m33*m55 + 1.
         /36.*m33*m44 - 1./36.*m33*m35 - 1./18.*m33*m34 + 1./36.*pow(
         m33,2) - 1./72.*m13*m55 - 1./72.*m13*m44 + 1./36.*m13*m35 + 1./
         36.*m13*m34 - 1./36.*m13*m33 + 1./144.*m11*m55 + 1./144.*m11*
         m44 - 1./72.*m11*m35 - 1./72.*m11*m34 + 1./72.*m11*m33 );

      M13p6x23RV +=  + Ab(m35,mu2)*BbV(m35,L) * ( 1./72.*pow(m55,2) - 1.
         /18.*m35*m55 + 1./18.*pow(m35,2) + 1./24.*m33*m55 - 1./12.*m33
         *m35 + 1./36.*pow(m33,2) - 1./36.*m13*m55 + 1./18.*m13*m35 - 1.
         /36.*m13*m33 + 1./72.*m11*m55 - 1./36.*m11*m35 + 1./72.*m11*
         m33 );

      M13p6x23RV +=  + Ab(m35,mu2)*BbV(m36,L) * ( 1./144.*pow(m66,2) + 
         1./144.*m55*m66 - 1./36.*m36*m66 - 1./72.*m36*m55 + 1./36.*
         pow(m36,2) - 1./72.*m35*m66 + 1./36.*m35*m36 + 1./36.*m33*m66
          + 1./72.*m33*m55 - 1./18.*m33*m36 - 1./36.*m33*m35 + 1./36.*
         pow(m33,2) - 1./72.*m13*m66 - 1./72.*m13*m55 + 1./36.*m13*m36
          + 1./36.*m13*m35 - 1./36.*m13*m33 + 1./144.*m11*m66 + 1./144.
         *m11*m55 - 1./72.*m11*m36 - 1./72.*m11*m35 + 1./72.*m11*m33 );

      M13p6x23RV +=  + Ab(m35,mu2)*CbV(m33,L) * (  - 1./108.*m33*m55*
         RD3456ep + 1./54.*m33*m35*RD3456ep - 1./108.*pow(m33,2)*
         RD3456ep + 1./54.*m13*m55*RD3456ep - 1./27.*m13*m35*RD3456ep
          + 1./54.*m13*m33*RD3456ep - 1./108.*m11*m55*RD3456ep + 1./54.
         *m11*m35*RD3456ep - 1./108.*m11*m33*RD3456ep );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(m11,L) * (  - 1./180.*RD1456ep
          + 1./180.*m66*RS14563pe + 1./60.*m66*RC1456pe + 13./540.*m66*
         RS145pe - 13./270.*m36*RS14563pe + 1./270.*m36*RC1456pe + 11./
         270.*m36*RS145pe + 1./60.*m33*RS14563pe + 2./135.*m33*RC1456pe
          + 1./180.*m33*RS145pe + 1./27.*m16*RS14563pe - 1./27.*m16*
         RC1456pe + 1./27.*m16*RS145pe - 31./540.*m13*RS14563pe + 1./20.
         *m13*RC1456pe - 23./540.*m13*RS145pe + 1./18.*m11*RS14563pe + 
         19./540.*m11*RC1456pe - 2./27.*m11*RS145pe + 1./54.*RS(m11,m44
         ,m55,m66,m33,me2,mp2)*m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,
         mp2)*m11 );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(m14,L) * (  - 1./20.*m66 + 1./9.
         *m46 - 1./20.*m44 - 1./90.*m36 - 2./45.*m33 - 1./90.*m14 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(m15,L) * (  - 1./20.*m66 + 1./9.
         *m56 - 1./20.*m55 - 1./90.*m36 - 2./45.*m33 - 1./90.*m15 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(m16,L) * (  - 1./108.*RD3456ep
          - 1./108.*RD1456ep - 1./54.*me2*RSe45613p - 1./108.*me2*
         RSe45633p + 1./27.*me2*RSe453p - 1./108.*me2*RSe45611p + 1./27.
         *me2*RSe451p - 1./27.*me2*RSe456p - 1./54.*mp2*RSp45613e - 1./
         108.*mp2*RSp45633e + 1./27.*mp2*RSp453e - 1./108.*mp2*
         RSp45611e + 1./27.*mp2*RSp451e - 1./27.*mp2*RSp456e + 1./90.*
         m66 - 1./54.*m66*RS34561pe - 1./54.*m66*RS14563pe - 1./54.*m66
         *RSe45613p - 1./54.*m66*RSp45613e - 1./108.*m66*RC3456pe - 1./
         108.*m66*RSe45633p - 1./108.*m66*RSp45633e - 1./108.*m66*
         RC1456pe - 1./108.*m66*RSe45611p - 1./108.*m66*RSp45611e + 1./
         27.*m66*RSe456p + 1./27.*m66*RSp456e - 1./90.*m36 + 1./36.*m36
         *RC3456pe + 1./36.*m36*RSe45633p + 1./36.*m36*RSp45633e - 1./
         18.*m36*RS345pe - 1./18.*m36*RSe453p - 1./18.*m36*RSp453e - 1./
         36.*m36*RC1456pe - 1./36.*m36*RSe45611p - 1./36.*m36*RSp45611e
          + 1./18.*m36*RS145pe + 1./18.*m36*RSe451p + 1./18.*m36*
         RSp451e );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(m16,L) * (  - 2./45.*m33 - 5./
         108.*m33*RS34561pe - 1./36.*m33*RS14563pe - 1./36.*m33*
         RSe45613p - 1./36.*m33*RSp45613e - 1./36.*m33*RC3456pe - 1./54.
         *m33*RSe45633p - 1./54.*m33*RSp45633e + 7./108.*m33*RS345pe + 
         1./36.*m33*RSe453p + 1./36.*m33*RSp453e - 1./108.*m33*RC1456pe
          - 1./108.*m33*RSe45611p - 1./108.*m33*RSp45611e + 1./108.*m33
         *RS145pe + 1./54.*m33*RS645pe + 1./108.*m33*RSe451p + 1./108.*
         m33*RSp451e + 1./54.*m33*RSe456p + 1./54.*m33*RSp456e - 1./90.
         *m16 - 1./36.*m16*RC3456pe - 1./36.*m16*RSe45633p - 1./36.*m16
         *RSp45633e + 1./18.*m16*RS345pe + 1./18.*m16*RSe453p + 1./18.*
         m16*RSp453e + 1./36.*m16*RC1456pe + 1./36.*m16*RSe45611p + 1./
         36.*m16*RSp45611e - 1./18.*m16*RS145pe - 1./18.*m16*RSe451p - 
         1./18.*m16*RSp451e - 3./20.*m13 - 1./36.*m13*RS34561pe - 1./36.
         *m13*RS14563pe - 1./36.*m13*RSe45613p - 1./36.*m13*RSp45613e
          + 1./36.*m13*RC3456pe + 1./36.*m13*RSe45633p + 1./36.*m13*
         RSp45633e );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(m16,L) * (  - 1./36.*m13*RS345pe
          - 1./36.*m13*RSe453p - 1./36.*m13*RSp453e + 1./36.*m13*
         RC1456pe + 1./36.*m13*RSe45611p + 1./36.*m13*RSp45611e - 1./36.
         *m13*RS145pe + 1./36.*m13*RS645pe - 1./36.*m13*RSe451p - 1./36.
         *m13*RSp451e + 1./36.*m13*RSe456p + 1./36.*m13*RSp456e - 2./45.
         *m11 - 1./36.*m11*RS34561pe - 5./108.*m11*RS14563pe - 1./36.*
         m11*RSe45613p - 1./36.*m11*RSp45613e - 1./108.*m11*RC3456pe - 
         1./108.*m11*RSe45633p - 1./108.*m11*RSp45633e + 1./108.*m11*
         RS345pe + 1./108.*m11*RSe453p + 1./108.*m11*RSp453e - 1./36.*
         m11*RC1456pe - 1./54.*m11*RSe45611p - 1./54.*m11*RSp45611e + 7.
         /108.*m11*RS145pe + 1./54.*m11*RS645pe + 1./36.*m11*RSe451p + 
         1./36.*m11*RSp451e + 1./54.*m11*RSe456p + 1./54.*m11*RSp456e )
         ;

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(m33,L) * (  - 29./540.*RD3456ep
          - 7./540.*m66*RS34561pe - 1./54.*m66*RS14563pe + 7./540.*m66*
         RC3456pe - 1./60.*m66*RS345pe - 1./54.*m66*RS145pe - 1./90.*
         m36*RS34561pe + 1./27.*m36*RS14563pe - 1./60.*m36*RC3456pe + 2.
         /45.*m36*RS345pe - 1./27.*m36*RS145pe + 43./540.*m33*RS34561pe
          + 1./27.*m33*RSe45613p + 1./27.*m33*RSp45613e + 13./1080.*m33
         *RC3456pe - 1./27.*m33*RSe45633p - 1./27.*m33*RSp45633e - 1./
         45.*m33*RS345pe + 1./27.*m33*RSe453p + 1./27.*m33*RSp453e - 2./
         27.*m33*RS145pe - 1./27.*m33*RSe451p - 1./27.*m33*RSp451e + 1./
         27.*m16*RS34561pe - 1./108.*m16*RC3456pe - 1./54.*m16*RS345pe
          - 11./540.*m13*RS34561pe + 1./27.*m13*RS14563pe - 1./135.*m13
         *RC3456pe + 19./540.*m13*RS345pe - 1./27.*m13*RS145pe + 1./90.
         *m11*RS34561pe + 1./54.*m11*RS14563pe + 1./120.*m11*RC3456pe
          - 1./20.*m11*RS345pe - 1./54.*m11*RS145pe + 1./27.*RS(m33,m44
         ,m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11 );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(m34,L) * (  - 11./360.*m66 - 11./
         360.*m44 + 11./180.*m36 + 11./180.*m34 - 7./90.*m33 + 1./30.*
         m13 - 1./60.*m11 );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(m35,L) * (  - 11./360.*m66 - 11./
         360.*m55 + 11./180.*m36 + 11./180.*m35 - 7./90.*m33 + 1./30.*
         m13 - 1./60.*m11 );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(m36,L) * (  - 11./180.*m66 + 11./
         90.*m36 - 7./90.*m33 + 1./30.*m13 - 1./60.*m11 );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(m46,L) * (  - 1./360.*m66 - 1./
         15.*m46 + 7./360.*m44 + 13./180.*m36 - 1./360.*m33 + 1./36.*
         m14 - 1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(m56,L) * (  - 1./360.*m66 - 1./
         15.*m56 + 7./360.*m55 + 13./180.*m36 - 1./360.*m33 + 1./36.*
         m15 - 1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(m66,L) * (  - 1./20.*m66 + 1./60.
         *m66*RS645pe + 13./180.*m36 - 13./540.*m36*RS645pe - 1./360.*
         m33 + 1./1080.*m33*RS645pe + 1./36.*m16 - 1./108.*m16*RS645pe
          - 1./15.*m13 + 1./45.*m13*RS645pe + 7./360.*m11 - 7./1080.*
         m11*RS645pe );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(mp2,L) * ( 11./135.*mp2*
         RSp45613e - 29./540.*mp2*RSp45633e + 7./270.*mp2*RSp453e - 1./
         180.*mp2*RSp45611e - 19./270.*mp2*RSp451e + 1./45.*mp2*RSp456e
          - 7./540.*m66*RSp45613e + 7./540.*m66*RSp45633e - 1./60.*m66*
         RSp453e + 1./60.*m66*RSp45611e + 1./180.*m66*RSp451e - 1./180.
         *m66*RSp456e - 1./90.*m36*RSp45613e - 1./60.*m36*RSp45633e + 2.
         /45.*m36*RSp453e + 1./270.*m36*RSp45611e + 1./270.*m36*RSp451e
          - 13./540.*m36*RSp456e - 1./540.*m33*RSp45613e + 37./360.*m33
         *RSp45633e - 23./270.*m33*RSp453e + 2./135.*m33*RSp45611e - 17.
         /540.*m33*RSp451e + 1./1080.*m33*RSp456e + 1./27.*m16*
         RSp45613e - 1./108.*m16*RSp45633e - 1./54.*m16*RSp453e - 1./27.
         *m16*RSp45611e + 1./27.*m16*RSp451e - 1./108.*m16*RSp456e - 11.
         /540.*m13*RSp45613e - 1./135.*m13*RSp45633e + 19./540.*m13*
         RSp453e + 1./20.*m13*RSp45611e - 43./540.*m13*RSp451e + 1./45.
         *m13*RSp456e + 4./135.*m11*RSp45613e + 1./120.*m11*RSp45633e
          - 1./20.*m11*RSp453e );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(mp2,L) * ( 11./270.*m11*
         RSp45611e - 1./45.*m11*RSp451e - 7./1080.*m11*RSp456e );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(me2,L) * ( 11./135.*me2*
         RSe45613p - 29./540.*me2*RSe45633p + 7./270.*me2*RSe453p - 1./
         180.*me2*RSe45611p - 19./270.*me2*RSe451p + 1./45.*me2*RSe456p
          - 7./540.*m66*RSe45613p + 7./540.*m66*RSe45633p - 1./60.*m66*
         RSe453p + 1./60.*m66*RSe45611p + 1./180.*m66*RSe451p - 1./180.
         *m66*RSe456p - 1./90.*m36*RSe45613p - 1./60.*m36*RSe45633p + 2.
         /45.*m36*RSe453p + 1./270.*m36*RSe45611p + 1./270.*m36*RSe451p
          - 13./540.*m36*RSe456p - 1./540.*m33*RSe45613p + 37./360.*m33
         *RSe45633p - 23./270.*m33*RSe453p + 2./135.*m33*RSe45611p - 17.
         /540.*m33*RSe451p + 1./1080.*m33*RSe456p + 1./27.*m16*
         RSe45613p - 1./108.*m16*RSe45633p - 1./54.*m16*RSe453p - 1./27.
         *m16*RSe45611p + 1./27.*m16*RSe451p - 1./108.*m16*RSe456p - 11.
         /540.*m13*RSe45613p - 1./135.*m13*RSe45633p + 19./540.*m13*
         RSe453p + 1./20.*m13*RSe45611p - 43./540.*m13*RSe451p + 1./45.
         *m13*RSe456p + 4./135.*m11*RSe45613p + 1./120.*m11*RSe45633p
          - 1./20.*m11*RSe453p );

      M13p6x23RV +=  + Ab(m36,mu2)*AbV(me2,L) * ( 11./270.*m11*
         RSe45611p - 1./45.*m11*RSe451p - 7./1080.*m11*RSe456p );

      M13p6x23RV +=  + Ab(m36,mu2)*BbV(m11,m33,L) * (  - 1./108.*m33*
         m66*RS14563pe - 1./108.*m33*m66*RS145pe + 1./54.*m33*m36*
         RS14563pe - 1./54.*m33*m36*RS145pe - 1./36.*pow(m33,2)*
         RS14563pe - 1./108.*pow(m33,2)*RS145pe - 1./54.*m13*m66*
         RS14563pe - 1./54.*m13*m66*RS145pe + 1./27.*m13*m36*RS14563pe
          - 1./27.*m13*m36*RS145pe - 1./18.*m13*m33*RS14563pe - 1./54.*
         m13*m33*RS145pe - 1./36.*m11*m66*RS14563pe - 1./36.*m11*m66*
         RS145pe + 1./18.*m11*m36*RS14563pe - 1./18.*m11*m36*RS145pe - 
         1./36.*m11*m33*RS14563pe - 1./12.*m11*m33*RS145pe + 1./27.*m11
         *m13*RS14563pe - 1./27.*m11*m13*RS145pe + 1./54.*pow(m11,2)*
         RS14563pe - 1./54.*pow(m11,2)*RS145pe );

      M13p6x23RV +=  + Ab(m36,mu2)*BbV(m11,L) * ( 1./60.*m66*RD1456ep
          + 1./270.*m36*RD1456ep + 2./135.*m33*RD1456ep - 1./27.*m16*
         RD1456ep + 1./20.*m13*RD1456ep + 19./540.*m11*RD1456ep );

      M13p6x23RV +=  + Ab(m36,mu2)*BbV(m33,mp2,L) * ( 1./27.*m33*mp2*
         RSp45613e - 1./27.*m33*mp2*RSp45633e + 1./27.*m33*mp2*RSp453e
          - 1./27.*m33*mp2*RSp451e - 1./36.*m33*m66*RSp45613e - 1./108.
         *m33*m66*RSp45633e - 1./108.*m33*m66*RSp453e - 1./36.*m33*m66*
         RSp451e + 1./18.*m33*m36*RSp45613e + 1./54.*m33*m36*RSp45633e
          - 1./54.*m33*m36*RSp453e - 1./18.*m33*m36*RSp451e - 7./108.*
         pow(m33,2)*RSp45613e + 1./36.*pow(m33,2)*RSp45633e - 7./108.*
         pow(m33,2)*RSp453e - 5./108.*pow(m33,2)*RSp451e - 1./54.*m13*
         m66*RSp45613e + 1./54.*m13*m66*RSp45633e + 1./54.*m13*m66*
         RSp453e - 1./54.*m13*m66*RSp451e + 1./27.*m13*m36*RSp45613e - 
         1./27.*m13*m36*RSp45633e + 1./27.*m13*m36*RSp453e - 1./27.*m13
         *m36*RSp451e - 1./54.*m13*m33*RSp45613e + 1./54.*m13*m33*
         RSp45633e + 1./18.*m13*m33*RSp453e - 1./18.*m13*m33*RSp451e - 
         1./108.*m11*m66*RSp45613e - 1./108.*m11*m66*RSp45633e - 1./108.
         *m11*m66*RSp453e - 1./108.*m11*m66*RSp451e + 1./54.*m11*m36*
         RSp45613e );

      M13p6x23RV +=  + Ab(m36,mu2)*BbV(m33,mp2,L) * ( 1./54.*m11*m36*
         RSp45633e - 1./54.*m11*m36*RSp453e - 1./54.*m11*m36*RSp451e - 
         1./108.*m11*m33*RSp45613e - 1./108.*m11*m33*RSp45633e - 1./36.
         *m11*m33*RSp453e - 1./36.*m11*m33*RSp451e );

      M13p6x23RV +=  + Ab(m36,mu2)*BbV(m33,me2,L) * ( 1./27.*m33*me2*
         RSe45613p - 1./27.*m33*me2*RSe45633p + 1./27.*m33*me2*RSe453p
          - 1./27.*m33*me2*RSe451p - 1./36.*m33*m66*RSe45613p - 1./108.
         *m33*m66*RSe45633p - 1./108.*m33*m66*RSe453p - 1./36.*m33*m66*
         RSe451p + 1./18.*m33*m36*RSe45613p + 1./54.*m33*m36*RSe45633p
          - 1./54.*m33*m36*RSe453p - 1./18.*m33*m36*RSe451p - 7./108.*
         pow(m33,2)*RSe45613p + 1./36.*pow(m33,2)*RSe45633p - 7./108.*
         pow(m33,2)*RSe453p - 5./108.*pow(m33,2)*RSe451p - 1./54.*m13*
         m66*RSe45613p + 1./54.*m13*m66*RSe45633p + 1./54.*m13*m66*
         RSe453p - 1./54.*m13*m66*RSe451p + 1./27.*m13*m36*RSe45613p - 
         1./27.*m13*m36*RSe45633p + 1./27.*m13*m36*RSe453p - 1./27.*m13
         *m36*RSe451p - 1./54.*m13*m33*RSe45613p + 1./54.*m13*m33*
         RSe45633p + 1./18.*m13*m33*RSe453p - 1./18.*m13*m33*RSe451p - 
         1./108.*m11*m66*RSe45613p - 1./108.*m11*m66*RSe45633p - 1./108.
         *m11*m66*RSe453p - 1./108.*m11*m66*RSe451p + 1./54.*m11*m36*
         RSe45613p );

      M13p6x23RV +=  + Ab(m36,mu2)*BbV(m33,me2,L) * ( 1./54.*m11*m36*
         RSe45633p - 1./54.*m11*m36*RSe453p - 1./54.*m11*m36*RSe451p - 
         1./108.*m11*m33*RSe45613p - 1./108.*m11*m33*RSe45633p - 1./36.
         *m11*m33*RSe453p - 1./36.*m11*m33*RSe451p );

      M13p6x23RV +=  + Ab(m36,mu2)*BbV(m33,L) * ( 7./540.*m66*RD3456ep
          - 1./60.*m36*RD3456ep + 13./1080.*m33*RD3456ep - 1./36.*m33*
         m66*RS34561pe - 1./108.*m33*m66*RC3456pe - 1./108.*m33*m66*
         RS345pe + 1./18.*m33*m36*RS34561pe + 1./54.*m33*m36*RC3456pe
          - 1./54.*m33*m36*RS345pe - 1./36.*pow(m33,2)*RS34561pe - 1./
         108.*pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS345pe - 1./108.
         *m16*RD3456ep - 1./135.*m13*RD3456ep - 1./54.*m13*m66*
         RS34561pe + 1./54.*m13*m66*RC3456pe + 1./54.*m13*m66*RS345pe
          + 1./27.*m13*m36*RS34561pe - 1./27.*m13*m36*RC3456pe + 1./27.
         *m13*m36*RS345pe - 1./54.*m13*m33*RS34561pe + 1./54.*m13*m33*
         RC3456pe + 1./18.*m13*m33*RS345pe + 1./120.*m11*RD3456ep - 1./
         108.*m11*m66*RS34561pe - 1./108.*m11*m66*RC3456pe - 1./108.*
         m11*m66*RS345pe + 1./54.*m11*m36*RS34561pe + 1./54.*m11*m36*
         RC3456pe - 1./54.*m11*m36*RS345pe - 1./108.*m11*m33*RS34561pe
          - 1./108.*m11*m33*RC3456pe - 1./36.*m11*m33*RS345pe );

      M13p6x23RV +=  + Ab(m36,mu2)*BbV(m34,L) * ( 1./144.*m44*m66 + 1./
         144.*pow(m44,2) - 1./72.*m36*m44 - 1./72.*m34*m66 - 1./36.*m34
         *m44 + 1./36.*m34*m36 + 1./36.*pow(m34,2) + 1./72.*m33*m66 + 1.
         /36.*m33*m44 - 1./36.*m33*m36 - 1./18.*m33*m34 + 1./36.*pow(
         m33,2) - 1./72.*m13*m66 - 1./72.*m13*m44 + 1./36.*m13*m36 + 1./
         36.*m13*m34 - 1./36.*m13*m33 + 1./144.*m11*m66 + 1./144.*m11*
         m44 - 1./72.*m11*m36 - 1./72.*m11*m34 + 1./72.*m11*m33 );

      M13p6x23RV +=  + Ab(m36,mu2)*BbV(m35,L) * ( 1./144.*m55*m66 + 1./
         144.*pow(m55,2) - 1./72.*m36*m55 - 1./72.*m35*m66 - 1./36.*m35
         *m55 + 1./36.*m35*m36 + 1./36.*pow(m35,2) + 1./72.*m33*m66 + 1.
         /36.*m33*m55 - 1./36.*m33*m36 - 1./18.*m33*m35 + 1./36.*pow(
         m33,2) - 1./72.*m13*m66 - 1./72.*m13*m55 + 1./36.*m13*m36 + 1./
         36.*m13*m35 - 1./36.*m13*m33 + 1./144.*m11*m66 + 1./144.*m11*
         m55 - 1./72.*m11*m36 - 1./72.*m11*m35 + 1./72.*m11*m33 );

      M13p6x23RV +=  + Ab(m36,mu2)*BbV(m36,L) * ( 1./72.*pow(m66,2) - 1.
         /18.*m36*m66 + 1./18.*pow(m36,2) + 1./24.*m33*m66 - 1./12.*m33
         *m36 + 1./36.*pow(m33,2) - 1./36.*m13*m66 + 1./18.*m13*m36 - 1.
         /36.*m13*m33 + 1./72.*m11*m66 - 1./36.*m11*m36 + 1./72.*m11*
         m33 );

      M13p6x23RV +=  + Ab(m36,mu2)*CbV(m33,L) * (  - 1./108.*m33*m66*
         RD3456ep + 1./54.*m33*m36*RD3456ep - 1./108.*pow(m33,2)*
         RD3456ep + 1./54.*m13*m66*RD3456ep - 1./27.*m13*m36*RD3456ep
          + 1./54.*m13*m33*RD3456ep - 1./108.*m11*m66*RD3456ep + 1./54.
         *m11*m36*RD3456ep - 1./108.*m11*m33*RD3456ep );

      M13p6x23RV +=  + Ab(m44,mu2)*AbV(m11,L) * (  - 1./54.*m44*pow(
         RS156pe,2) + 1./162.*m44*RS456pe*pow(RS156pe,2) );

      M13p6x23RV +=  + Ab(m44,mu2)*AbV(m14,L) * (  - 1./20.*m44 + 1./60.
         *m44*RS456pe + 1./36.*m34 - 1./108.*m34*RS456pe + 7./360.*m33
          - 7./1080.*m33*RS456pe + 13./180.*m14 - 13./540.*m14*RS456pe
          - 1./15.*m13 + 1./45.*m13*RS456pe - 1./360.*m11 + 1./1080.*
         m11*RS456pe );

      M13p6x23RV +=  + Ab(m44,mu2)*AbV(m33,L) * (  - 1./54.*m44*pow(
         RS356pe,2) + 1./27.*m44*RS156pe*RS356pe + 1./162.*m44*RS456pe*
         pow(RS356pe,2) - 1./81.*m44*RS456pe*RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m44,mu2)*AbV(m34,L) * (  - 1./20.*m44 + 1./60.
         *m44*RS456pe + 13./180.*m34 - 13./540.*m34*RS456pe - 1./360.*
         m33 + 1./1080.*m33*RS456pe + 1./36.*m14 - 1./108.*m14*RS456pe
          - 1./15.*m13 + 1./45.*m13*RS456pe + 7./360.*m11 - 7./1080.*
         m11*RS456pe );

      M13p6x23RV +=  + Ab(m44,mu2)*AbV(mp2,L) * (  - 1./27.*m44*RSp563e
         *RS356pe - 1./27.*m44*RSp563e*RSe563p - 1./54.*m44*pow(
         RSp563e,2) + 1./27.*m44*RS156pe*RSp563e + 1./81.*m44*RS456pe*
         RSp563e*RS356pe + 1./81.*m44*RS456pe*RSp563e*RSe563p + 1./162.
         *m44*RS456pe*pow(RSp563e,2) - 1./81.*m44*RS456pe*RS156pe*
         RSp563e + 1./27.*m44*RSe561p*RSp563e - 1./81.*m44*RSe561p*
         RS456pe*RSp563e + 1./27.*m44*RSp561e*RS356pe + 1./27.*m44*
         RSp561e*RSe563p + 1./27.*m44*RSp561e*RSp563e - 1./27.*m44*
         RSp561e*RS156pe - 1./81.*m44*RSp561e*RS456pe*RS356pe - 1./81.*
         m44*RSp561e*RS456pe*RSe563p - 1./81.*m44*RSp561e*RS456pe*
         RSp563e + 1./81.*m44*RSp561e*RS456pe*RS156pe - 1./27.*m44*
         RSp561e*RSe561p + 1./81.*m44*RSp561e*RSe561p*RS456pe - 1./54.*
         m44*pow(RSp561e,2) + 1./162.*m44*pow(RSp561e,2)*RS456pe );

      M13p6x23RV +=  + Ab(m44,mu2)*AbV(me2,L) * (  - 1./27.*m44*RSe563p
         *RS356pe - 1./54.*m44*pow(RSe563p,2) + 1./27.*m44*RS156pe*
         RSe563p + 1./81.*m44*RS456pe*RSe563p*RS356pe + 1./162.*m44*
         RS456pe*pow(RSe563p,2) - 1./81.*m44*RS456pe*RS156pe*RSe563p + 
         1./27.*m44*RSe561p*RS356pe + 1./27.*m44*RSe561p*RSe563p - 1./
         27.*m44*RSe561p*RS156pe - 1./81.*m44*RSe561p*RS456pe*RS356pe
          - 1./81.*m44*RSe561p*RS456pe*RSe563p + 1./81.*m44*RSe561p*
         RS456pe*RS156pe - 1./54.*m44*pow(RSe561p,2) + 1./162.*m44*pow(
         RSe561p,2)*RS456pe );

      M13p6x23RV +=  + Ab(m44,mu2)*BbV(m11,m33,L) * ( 1./54.*m33*m44*
         RS156pe*RS356pe - 1./162.*m33*m44*RS456pe*RS156pe*RS356pe + 1./
         27.*m13*m44*RS156pe*RS356pe - 1./81.*m13*m44*RS456pe*RS156pe*
         RS356pe + 1./18.*m11*m44*RS156pe*RS356pe - 1./54.*m11*m44*
         RS456pe*RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m44,mu2)*BbV(m11,mp2,L) * ( 1./54.*m33*m44*
         RS156pe*RSp563e - 1./162.*m33*m44*RS456pe*RS156pe*RSp563e + 1./
         54.*m33*m44*RSp561e*RS156pe - 1./162.*m33*m44*RSp561e*RS456pe*
         RS156pe + 1./27.*m13*m44*RS156pe*RSp563e - 1./81.*m13*m44*
         RS456pe*RS156pe*RSp563e - 1./27.*m13*m44*RSp561e*RS156pe + 1./
         81.*m13*m44*RSp561e*RS456pe*RS156pe + 1./18.*m11*m44*RS156pe*
         RSp563e - 1./54.*m11*m44*RS456pe*RS156pe*RSp563e + 1./54.*m11*
         m44*RSp561e*RS156pe - 1./162.*m11*m44*RSp561e*RS456pe*RS156pe
          );

      M13p6x23RV +=  + Ab(m44,mu2)*BbV(m11,me2,L) * ( 1./54.*m33*m44*
         RS156pe*RSe563p - 1./162.*m33*m44*RS456pe*RS156pe*RSe563p + 1./
         54.*m33*m44*RSe561p*RS156pe - 1./162.*m33*m44*RSe561p*RS456pe*
         RS156pe + 1./27.*m13*m44*RS156pe*RSe563p - 1./81.*m13*m44*
         RS456pe*RS156pe*RSe563p - 1./27.*m13*m44*RSe561p*RS156pe + 1./
         81.*m13*m44*RSe561p*RS456pe*RS156pe + 1./18.*m11*m44*RS156pe*
         RSe563p - 1./54.*m11*m44*RS456pe*RS156pe*RSe563p + 1./54.*m11*
         m44*RSe561p*RS156pe - 1./162.*m11*m44*RSe561p*RS456pe*RS156pe
          );

      M13p6x23RV +=  + Ab(m44,mu2)*BbV(m11,L) * ( 1./108.*m33*m44*pow(
         RS156pe,2) - 1./324.*m33*m44*RS456pe*pow(RS156pe,2) - 1./54.*
         m13*m44*pow(RS156pe,2) + 1./162.*m13*m44*RS456pe*pow(
         RS156pe,2) + 1./108.*m11*m44*pow(RS156pe,2) - 1./324.*m11*m44*
         RS456pe*pow(RS156pe,2) );

      M13p6x23RV +=  + Ab(m44,mu2)*BbV(m14,L) * ( 1./144.*pow(m44,2) - 
         1./432.*pow(m44,2)*RS456pe + 1./144.*m33*m44 - 1./432.*m33*m44
         *RS456pe - 1./36.*m14*m44 + 1./108.*m14*m44*RS456pe - 1./72.*
         m14*m33 + 1./216.*m14*m33*RS456pe + 1./36.*pow(m14,2) - 1./108.
         *pow(m14,2)*RS456pe - 1./72.*m13*m44 + 1./216.*m13*m44*RS456pe
          + 1./36.*m13*m14 - 1./108.*m13*m14*RS456pe + 1./48.*m11*m44
          - 1./144.*m11*m44*RS456pe + 1./144.*m11*m33 - 1./432.*m11*m33
         *RS456pe - 1./24.*m11*m14 + 1./72.*m11*m14*RS456pe - 1./72.*
         m11*m13 + 1./216.*m11*m13*RS456pe + 1./72.*pow(m11,2) - 1./216.
         *pow(m11,2)*RS456pe );

      M13p6x23RV +=  + Ab(m44,mu2)*BbV(m33,mp2,L) * ( 1./54.*m33*m44*
         RSp563e*RS356pe - 1./162.*m33*m44*RS456pe*RSp563e*RS356pe + 1./
         18.*m33*m44*RSp561e*RS356pe - 1./54.*m33*m44*RSp561e*RS456pe*
         RS356pe - 1./27.*m13*m44*RSp563e*RS356pe + 1./81.*m13*m44*
         RS456pe*RSp563e*RS356pe + 1./27.*m13*m44*RSp561e*RS356pe - 1./
         81.*m13*m44*RSp561e*RS456pe*RS356pe + 1./54.*m11*m44*RSp563e*
         RS356pe - 1./162.*m11*m44*RS456pe*RSp563e*RS356pe + 1./54.*m11
         *m44*RSp561e*RS356pe - 1./162.*m11*m44*RSp561e*RS456pe*RS356pe
          );

      M13p6x23RV +=  + Ab(m44,mu2)*BbV(m33,me2,L) * ( 1./54.*m33*m44*
         RSe563p*RS356pe - 1./162.*m33*m44*RS456pe*RSe563p*RS356pe + 1./
         18.*m33*m44*RSe561p*RS356pe - 1./54.*m33*m44*RSe561p*RS456pe*
         RS356pe - 1./27.*m13*m44*RSe563p*RS356pe + 1./81.*m13*m44*
         RS456pe*RSe563p*RS356pe + 1./27.*m13*m44*RSe561p*RS356pe - 1./
         81.*m13*m44*RSe561p*RS456pe*RS356pe + 1./54.*m11*m44*RSe563p*
         RS356pe - 1./162.*m11*m44*RS456pe*RSe563p*RS356pe + 1./54.*m11
         *m44*RSe561p*RS356pe - 1./162.*m11*m44*RSe561p*RS456pe*RS356pe
          );

      M13p6x23RV +=  + Ab(m44,mu2)*BbV(m33,L) * ( 1./108.*m33*m44*pow(
         RS356pe,2) - 1./324.*m33*m44*RS456pe*pow(RS356pe,2) - 1./54.*
         m13*m44*pow(RS356pe,2) + 1./162.*m13*m44*RS456pe*pow(
         RS356pe,2) + 1./108.*m11*m44*pow(RS356pe,2) - 1./324.*m11*m44*
         RS456pe*pow(RS356pe,2) );

      M13p6x23RV +=  + Ab(m44,mu2)*BbV(m34,L) * ( 1./144.*pow(m44,2) - 
         1./432.*pow(m44,2)*RS456pe - 1./36.*m34*m44 + 1./108.*m34*m44*
         RS456pe + 1./36.*pow(m34,2) - 1./108.*pow(m34,2)*RS456pe + 1./
         48.*m33*m44 - 1./144.*m33*m44*RS456pe - 1./24.*m33*m34 + 1./72.
         *m33*m34*RS456pe + 1./72.*pow(m33,2) - 1./216.*pow(m33,2)*
         RS456pe - 1./72.*m13*m44 + 1./216.*m13*m44*RS456pe + 1./36.*
         m13*m34 - 1./108.*m13*m34*RS456pe - 1./72.*m13*m33 + 1./216.*
         m13*m33*RS456pe + 1./144.*m11*m44 - 1./432.*m11*m44*RS456pe - 
         1./72.*m11*m34 + 1./216.*m11*m34*RS456pe + 1./144.*m11*m33 - 1.
         /432.*m11*m33*RS456pe );

      M13p6x23RV +=  + Ab(m44,mu2)*BbV(mp2,L) * (  - 1./54.*m44*mp2*
         pow(RSp563e,2) + 1./162.*m44*mp2*RS456pe*pow(RSp563e,2) + 1./
         27.*m44*mp2*RSp561e*RSp563e - 1./81.*m44*mp2*RSp561e*RS456pe*
         RSp563e - 1./54.*m44*mp2*pow(RSp561e,2) + 1./162.*m44*mp2*pow(
         RSp561e,2)*RS456pe + 1./36.*m33*m44*pow(RSp563e,2) - 1./108.*
         m33*m44*RS456pe*pow(RSp563e,2) + 1./54.*m33*m44*RSp561e*
         RSp563e - 1./162.*m33*m44*RSp561e*RS456pe*RSp563e + 1./108.*
         m33*m44*pow(RSp561e,2) - 1./324.*m33*m44*pow(RSp561e,2)*
         RS456pe - 1./54.*m13*m44*pow(RSp563e,2) + 1./162.*m13*m44*
         RS456pe*pow(RSp563e,2) + 1./27.*m13*m44*RSp561e*RSp563e - 1./
         81.*m13*m44*RSp561e*RS456pe*RSp563e - 1./54.*m13*m44*pow(
         RSp561e,2) + 1./162.*m13*m44*pow(RSp561e,2)*RS456pe + 1./108.*
         m11*m44*pow(RSp563e,2) - 1./324.*m11*m44*RS456pe*pow(
         RSp563e,2) + 1./54.*m11*m44*RSp561e*RSp563e - 1./162.*m11*m44*
         RSp561e*RS456pe*RSp563e + 1./36.*m11*m44*pow(RSp561e,2) - 1./
         108.*m11*m44*pow(RSp561e,2)*RS456pe );

      M13p6x23RV +=  + Ab(m44,mu2)*BbV(me2,mp2,L) * (  - 1./27.*m44*me2
         *RSp563e*RSe563p + 1./81.*m44*me2*RS456pe*RSp563e*RSe563p + 1./
         27.*m44*me2*RSe561p*RSp563e - 1./81.*m44*me2*RSe561p*RS456pe*
         RSp563e + 1./27.*m44*me2*RSp561e*RSe563p - 1./81.*m44*me2*
         RSp561e*RS456pe*RSe563p - 1./27.*m44*me2*RSp561e*RSe561p + 1./
         81.*m44*me2*RSp561e*RSe561p*RS456pe + 1./18.*m33*m44*RSp563e*
         RSe563p - 1./54.*m33*m44*RS456pe*RSp563e*RSe563p + 1./54.*m33*
         m44*RSe561p*RSp563e - 1./162.*m33*m44*RSe561p*RS456pe*RSp563e
          + 1./54.*m33*m44*RSp561e*RSe563p - 1./162.*m33*m44*RSp561e*
         RS456pe*RSe563p + 1./54.*m33*m44*RSp561e*RSe561p - 1./162.*m33
         *m44*RSp561e*RSe561p*RS456pe - 1./27.*m13*m44*RSp563e*RSe563p
          + 1./81.*m13*m44*RS456pe*RSp563e*RSe563p + 1./27.*m13*m44*
         RSe561p*RSp563e - 1./81.*m13*m44*RSe561p*RS456pe*RSp563e + 1./
         27.*m13*m44*RSp561e*RSe563p - 1./81.*m13*m44*RSp561e*RS456pe*
         RSe563p - 1./27.*m13*m44*RSp561e*RSe561p + 1./81.*m13*m44*
         RSp561e*RSe561p*RS456pe );

      M13p6x23RV +=  + Ab(m44,mu2)*BbV(me2,mp2,L) * ( 1./54.*m11*m44*
         RSp563e*RSe563p - 1./162.*m11*m44*RS456pe*RSp563e*RSe563p + 1./
         54.*m11*m44*RSe561p*RSp563e - 1./162.*m11*m44*RSe561p*RS456pe*
         RSp563e + 1./54.*m11*m44*RSp561e*RSe563p - 1./162.*m11*m44*
         RSp561e*RS456pe*RSe563p + 1./18.*m11*m44*RSp561e*RSe561p - 1./
         54.*m11*m44*RSp561e*RSe561p*RS456pe );

      M13p6x23RV +=  + Ab(m44,mu2)*BbV(me2,L) * (  - 1./54.*m44*me2*
         pow(RSe563p,2) + 1./162.*m44*me2*RS456pe*pow(RSe563p,2) + 1./
         27.*m44*me2*RSe561p*RSe563p - 1./81.*m44*me2*RSe561p*RS456pe*
         RSe563p - 1./54.*m44*me2*pow(RSe561p,2) + 1./162.*m44*me2*pow(
         RSe561p,2)*RS456pe + 1./36.*m33*m44*pow(RSe563p,2) - 1./108.*
         m33*m44*RS456pe*pow(RSe563p,2) + 1./54.*m33*m44*RSe561p*
         RSe563p - 1./162.*m33*m44*RSe561p*RS456pe*RSe563p + 1./108.*
         m33*m44*pow(RSe561p,2) - 1./324.*m33*m44*pow(RSe561p,2)*
         RS456pe - 1./54.*m13*m44*pow(RSe563p,2) + 1./162.*m13*m44*
         RS456pe*pow(RSe563p,2) + 1./27.*m13*m44*RSe561p*RSe563p - 1./
         81.*m13*m44*RSe561p*RS456pe*RSe563p - 1./54.*m13*m44*pow(
         RSe561p,2) + 1./162.*m13*m44*pow(RSe561p,2)*RS456pe + 1./108.*
         m11*m44*pow(RSe563p,2) - 1./324.*m11*m44*RS456pe*pow(
         RSe563p,2) + 1./54.*m11*m44*RSe561p*RSe563p - 1./162.*m11*m44*
         RSe561p*RS456pe*RSe563p + 1./36.*m11*m44*pow(RSe561p,2) - 1./
         108.*m11*m44*pow(RSe561p,2)*RS456pe );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(m11,L) * (  - 1./108.*m55*pow(
         RS146pe,2) - 1./162.*m55*RS156pe*RS146pe - 1./324.*m55*pow(
         RS156pe,2) + 1./162.*m45*pow(RS146pe,2) - 1./81.*m45*RS156pe*
         RS146pe + 1./162.*m45*pow(RS156pe,2) - 1./324.*m44*pow(
         RS146pe,2) - 1./162.*m44*RS156pe*RS146pe - 1./108.*m44*pow(
         RS156pe,2) - 1./324.*m33*pow(RS146pe,2) + 1./162.*m33*RS156pe*
         RS146pe - 1./324.*m33*pow(RS156pe,2) + 1./162.*m13*pow(
         RS146pe,2) - 1./81.*m13*RS156pe*RS146pe + 1./162.*m13*pow(
         RS156pe,2) - 1./81.*m11*RS146pe*RS346pe + 1./81.*m11*RS146pe*
         RS356pe + 1./81.*m11*RS146pe*RSe563p - 1./81.*m11*RS146pe*
         RSe463p + 1./81.*m11*RS146pe*RSp563e - 1./81.*m11*RS146pe*
         RSp463e + 1./324.*m11*pow(RS146pe,2) + 1./81.*m11*RS156pe*
         RS346pe - 1./81.*m11*RS156pe*RS356pe - 1./81.*m11*RS156pe*
         RSe563p + 1./81.*m11*RS156pe*RSe463p - 1./81.*m11*RS156pe*
         RSp563e + 1./81.*m11*RS156pe*RSp463e - 1./162.*m11*RS156pe*
         RS146pe );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(m11,L) * ( 1./324.*m11*pow(
         RS156pe,2) - 1./81.*m11*RSe561p*RS146pe + 1./81.*m11*RSe561p*
         RS156pe + 1./81.*m11*RSe461p*RS146pe - 1./81.*m11*RSe461p*
         RS156pe - 1./81.*m11*RSp561e*RS146pe + 1./81.*m11*RSp561e*
         RS156pe + 1./81.*m11*RSp461e*RS146pe - 1./81.*m11*RSp461e*
         RS156pe );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(m14,L) * ( 7./360.*m55 - 1./15.*
         m45 - 1./360.*m44 + 1./36.*m35 + 7./360.*m33 + 13./180.*m14 - 
         1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(m15,L) * (  - 1./360.*m55 - 1./
         15.*m45 + 7./360.*m44 + 1./36.*m34 + 7./360.*m33 + 13./180.*
         m15 - 1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(m33,L) * (  - 1./108.*m55*pow(
         RS346pe,2) - 1./162.*m55*RS356pe*RS346pe - 1./324.*m55*pow(
         RS356pe,2) + 1./54.*m55*RS146pe*RS346pe + 1./162.*m55*RS146pe*
         RS356pe + 1./162.*m55*RS156pe*RS346pe + 1./162.*m55*RS156pe*
         RS356pe + 1./162.*m45*pow(RS346pe,2) - 1./81.*m45*RS356pe*
         RS346pe + 1./162.*m45*pow(RS356pe,2) - 1./81.*m45*RS146pe*
         RS346pe + 1./81.*m45*RS146pe*RS356pe + 1./81.*m45*RS156pe*
         RS346pe - 1./81.*m45*RS156pe*RS356pe - 1./324.*m44*pow(
         RS346pe,2) - 1./162.*m44*RS356pe*RS346pe - 1./108.*m44*pow(
         RS356pe,2) + 1./162.*m44*RS146pe*RS346pe + 1./162.*m44*RS146pe
         *RS356pe + 1./162.*m44*RS156pe*RS346pe + 1./54.*m44*RS156pe*
         RS356pe + 1./324.*m33*pow(RS346pe,2) - 1./162.*m33*RS356pe*
         RS346pe + 1./324.*m33*pow(RS356pe,2) - 1./81.*m33*RSe563p*
         RS346pe + 1./81.*m33*RSe563p*RS356pe + 1./81.*m33*RSe463p*
         RS346pe - 1./81.*m33*RSe463p*RS356pe - 1./81.*m33*RSp563e*
         RS346pe );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(m33,L) * ( 1./81.*m33*RSp563e*
         RS356pe + 1./81.*m33*RSp463e*RS346pe - 1./81.*m33*RSp463e*
         RS356pe - 1./54.*m33*RS146pe*RS346pe + 1./54.*m33*RS146pe*
         RS356pe + 1./54.*m33*RS156pe*RS346pe - 1./54.*m33*RS156pe*
         RS356pe + 1./81.*m33*RSe561p*RS346pe - 1./81.*m33*RSe561p*
         RS356pe - 1./81.*m33*RSe461p*RS346pe + 1./81.*m33*RSe461p*
         RS356pe + 1./81.*m33*RSp561e*RS346pe - 1./81.*m33*RSp561e*
         RS356pe - 1./81.*m33*RSp461e*RS346pe + 1./81.*m33*RSp461e*
         RS356pe + 1./162.*m13*pow(RS346pe,2) - 1./81.*m13*RS356pe*
         RS346pe + 1./162.*m13*pow(RS356pe,2) - 1./81.*m13*RS146pe*
         RS346pe + 1./81.*m13*RS146pe*RS356pe + 1./81.*m13*RS156pe*
         RS346pe - 1./81.*m13*RS156pe*RS356pe - 1./324.*m11*pow(
         RS346pe,2) + 1./162.*m11*RS356pe*RS346pe - 1./324.*m11*pow(
         RS356pe,2) - 1./162.*m11*RS146pe*RS346pe + 1./162.*m11*RS146pe
         *RS356pe + 1./162.*m11*RS156pe*RS346pe - 1./162.*m11*RS156pe*
         RS356pe );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(m34,L) * ( 7./360.*m55 - 1./15.*
         m45 - 1./360.*m44 + 13./180.*m34 - 1./360.*m33 + 1./36.*m15 - 
         1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(m35,L) * (  - 1./360.*m55 - 1./
         15.*m45 + 7./360.*m44 + 13./180.*m35 - 1./360.*m33 + 1./36.*
         m14 - 1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(mp2,L) * (  - 1./81.*mp2*RSp563e
         *RS346pe + 1./81.*mp2*RSp563e*RS356pe + 1./81.*mp2*RSp563e*
         RSe563p - 1./81.*mp2*RSp563e*RSe463p + 1./81.*mp2*pow(
         RSp563e,2) + 1./81.*mp2*RSp463e*RS346pe - 1./81.*mp2*RSp463e*
         RS356pe - 1./81.*mp2*RSp463e*RSe563p + 1./81.*mp2*RSp463e*
         RSe463p - 2./81.*mp2*RSp463e*RSp563e + 1./81.*mp2*pow(
         RSp463e,2) + 1./81.*mp2*RS146pe*RSp563e - 1./81.*mp2*RS146pe*
         RSp463e - 1./81.*mp2*RS156pe*RSp563e + 1./81.*mp2*RS156pe*
         RSp463e - 1./81.*mp2*RSe561p*RSp563e + 1./81.*mp2*RSe561p*
         RSp463e + 1./81.*mp2*RSe461p*RSp563e - 1./81.*mp2*RSe461p*
         RSp463e + 1./81.*mp2*RSp561e*RS346pe - 1./81.*mp2*RSp561e*
         RS356pe - 1./81.*mp2*RSp561e*RSe563p + 1./81.*mp2*RSp561e*
         RSe463p - 2./81.*mp2*RSp561e*RSp563e + 2./81.*mp2*RSp561e*
         RSp463e - 1./81.*mp2*RSp561e*RS146pe + 1./81.*mp2*RSp561e*
         RS156pe + 1./81.*mp2*RSp561e*RSe561p - 1./81.*mp2*RSp561e*
         RSe461p );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(mp2,L) * ( 1./81.*mp2*pow(
         RSp561e,2) - 1./81.*mp2*RSp461e*RS346pe + 1./81.*mp2*RSp461e*
         RS356pe + 1./81.*mp2*RSp461e*RSe563p - 1./81.*mp2*RSp461e*
         RSe463p + 2./81.*mp2*RSp461e*RSp563e - 2./81.*mp2*RSp461e*
         RSp463e + 1./81.*mp2*RSp461e*RS146pe - 1./81.*mp2*RSp461e*
         RS156pe - 1./81.*mp2*RSp461e*RSe561p + 1./81.*mp2*RSp461e*
         RSe461p - 2./81.*mp2*RSp461e*RSp561e + 1./81.*mp2*pow(
         RSp461e,2) - 1./162.*m55*RSp563e*RS346pe - 1./162.*m55*RSp563e
         *RS356pe - 1./162.*m55*RSp563e*RSe563p - 1./162.*m55*RSp563e*
         RSe463p - 1./324.*m55*pow(RSp563e,2) - 1./54.*m55*RSp463e*
         RS346pe - 1./162.*m55*RSp463e*RS356pe - 1./162.*m55*RSp463e*
         RSe563p - 1./54.*m55*RSp463e*RSe463p - 1./162.*m55*RSp463e*
         RSp563e - 1./108.*m55*pow(RSp463e,2) + 1./162.*m55*RS146pe*
         RSp563e + 1./54.*m55*RS146pe*RSp463e + 1./162.*m55*RS156pe*
         RSp563e + 1./162.*m55*RS156pe*RSp463e + 1./162.*m55*RSe561p*
         RSp563e );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(mp2,L) * ( 1./162.*m55*RSe561p*
         RSp463e + 1./162.*m55*RSe461p*RSp563e + 1./54.*m55*RSe461p*
         RSp463e + 1./162.*m55*RSp561e*RS346pe + 1./162.*m55*RSp561e*
         RS356pe + 1./162.*m55*RSp561e*RSe563p + 1./162.*m55*RSp561e*
         RSe463p + 1./162.*m55*RSp561e*RSp563e + 1./162.*m55*RSp561e*
         RSp463e - 1./162.*m55*RSp561e*RS146pe - 1./162.*m55*RSp561e*
         RS156pe - 1./162.*m55*RSp561e*RSe561p - 1./162.*m55*RSp561e*
         RSe461p - 1./324.*m55*pow(RSp561e,2) + 1./54.*m55*RSp461e*
         RS346pe + 1./162.*m55*RSp461e*RS356pe + 1./162.*m55*RSp461e*
         RSe563p + 1./54.*m55*RSp461e*RSe463p + 1./162.*m55*RSp461e*
         RSp563e + 1./54.*m55*RSp461e*RSp463e - 1./54.*m55*RSp461e*
         RS146pe - 1./162.*m55*RSp461e*RS156pe - 1./162.*m55*RSp461e*
         RSe561p - 1./54.*m55*RSp461e*RSe461p - 1./162.*m55*RSp461e*
         RSp561e - 1./108.*m55*pow(RSp461e,2) - 1./81.*m45*RSp563e*
         RS346pe + 1./81.*m45*RSp563e*RS356pe + 1./81.*m45*RSp563e*
         RSe563p );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(mp2,L) * (  - 1./81.*m45*RSp563e
         *RSe463p + 1./162.*m45*pow(RSp563e,2) + 1./81.*m45*RSp463e*
         RS346pe - 1./81.*m45*RSp463e*RS356pe - 1./81.*m45*RSp463e*
         RSe563p + 1./81.*m45*RSp463e*RSe463p - 1./81.*m45*RSp463e*
         RSp563e + 1./162.*m45*pow(RSp463e,2) + 1./81.*m45*RS146pe*
         RSp563e - 1./81.*m45*RS146pe*RSp463e - 1./81.*m45*RS156pe*
         RSp563e + 1./81.*m45*RS156pe*RSp463e - 1./81.*m45*RSe561p*
         RSp563e + 1./81.*m45*RSe561p*RSp463e + 1./81.*m45*RSe461p*
         RSp563e - 1./81.*m45*RSe461p*RSp463e + 1./81.*m45*RSp561e*
         RS346pe - 1./81.*m45*RSp561e*RS356pe - 1./81.*m45*RSp561e*
         RSe563p + 1./81.*m45*RSp561e*RSe463p - 1./81.*m45*RSp561e*
         RSp563e + 1./81.*m45*RSp561e*RSp463e - 1./81.*m45*RSp561e*
         RS146pe + 1./81.*m45*RSp561e*RS156pe + 1./81.*m45*RSp561e*
         RSe561p - 1./81.*m45*RSp561e*RSe461p + 1./162.*m45*pow(
         RSp561e,2) - 1./81.*m45*RSp461e*RS346pe + 1./81.*m45*RSp461e*
         RS356pe );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(mp2,L) * ( 1./81.*m45*RSp461e*
         RSe563p - 1./81.*m45*RSp461e*RSe463p + 1./81.*m45*RSp461e*
         RSp563e - 1./81.*m45*RSp461e*RSp463e + 1./81.*m45*RSp461e*
         RS146pe - 1./81.*m45*RSp461e*RS156pe - 1./81.*m45*RSp461e*
         RSe561p + 1./81.*m45*RSp461e*RSe461p - 1./81.*m45*RSp461e*
         RSp561e + 1./162.*m45*pow(RSp461e,2) - 1./162.*m44*RSp563e*
         RS346pe - 1./54.*m44*RSp563e*RS356pe - 1./54.*m44*RSp563e*
         RSe563p - 1./162.*m44*RSp563e*RSe463p - 1./108.*m44*pow(
         RSp563e,2) - 1./162.*m44*RSp463e*RS346pe - 1./162.*m44*RSp463e
         *RS356pe - 1./162.*m44*RSp463e*RSe563p - 1./162.*m44*RSp463e*
         RSe463p - 1./162.*m44*RSp463e*RSp563e - 1./324.*m44*pow(
         RSp463e,2) + 1./162.*m44*RS146pe*RSp563e + 1./162.*m44*RS146pe
         *RSp463e + 1./54.*m44*RS156pe*RSp563e + 1./162.*m44*RS156pe*
         RSp463e + 1./54.*m44*RSe561p*RSp563e + 1./162.*m44*RSe561p*
         RSp463e + 1./162.*m44*RSe461p*RSp563e + 1./162.*m44*RSe461p*
         RSp463e );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(mp2,L) * ( 1./162.*m44*RSp561e*
         RS346pe + 1./54.*m44*RSp561e*RS356pe + 1./54.*m44*RSp561e*
         RSe563p + 1./162.*m44*RSp561e*RSe463p + 1./54.*m44*RSp561e*
         RSp563e + 1./162.*m44*RSp561e*RSp463e - 1./162.*m44*RSp561e*
         RS146pe - 1./54.*m44*RSp561e*RS156pe - 1./54.*m44*RSp561e*
         RSe561p - 1./162.*m44*RSp561e*RSe461p - 1./108.*m44*pow(
         RSp561e,2) + 1./162.*m44*RSp461e*RS346pe + 1./162.*m44*RSp461e
         *RS356pe + 1./162.*m44*RSp461e*RSe563p + 1./162.*m44*RSp461e*
         RSe463p + 1./162.*m44*RSp461e*RSp563e + 1./162.*m44*RSp461e*
         RSp463e - 1./162.*m44*RSp461e*RS146pe - 1./162.*m44*RSp461e*
         RS156pe - 1./162.*m44*RSp461e*RSe561p - 1./162.*m44*RSp461e*
         RSe461p - 1./162.*m44*RSp461e*RSp561e - 1./324.*m44*pow(
         RSp461e,2) + 1./54.*m33*RSp563e*RS346pe - 1./54.*m33*RSp563e*
         RS356pe - 1./54.*m33*RSp563e*RSe563p + 1./54.*m33*RSp563e*
         RSe463p - 1./108.*m33*pow(RSp563e,2) - 1./54.*m33*RSp463e*
         RS346pe );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(mp2,L) * ( 1./54.*m33*RSp463e*
         RS356pe + 1./54.*m33*RSp463e*RSe563p - 1./54.*m33*RSp463e*
         RSe463p + 1./54.*m33*RSp463e*RSp563e - 1./108.*m33*pow(
         RSp463e,2) + 1./162.*m33*RS146pe*RSp563e - 1./162.*m33*RS146pe
         *RSp463e - 1./162.*m33*RS156pe*RSp563e + 1./162.*m33*RS156pe*
         RSp463e - 1./162.*m33*RSe561p*RSp563e + 1./162.*m33*RSe561p*
         RSp463e + 1./162.*m33*RSe461p*RSp563e - 1./162.*m33*RSe461p*
         RSp463e + 1./162.*m33*RSp561e*RS346pe - 1./162.*m33*RSp561e*
         RS356pe - 1./162.*m33*RSp561e*RSe563p + 1./162.*m33*RSp561e*
         RSe463p - 1./162.*m33*RSp561e*RSp563e + 1./162.*m33*RSp561e*
         RSp463e + 1./162.*m33*RSp561e*RS146pe - 1./162.*m33*RSp561e*
         RS156pe - 1./162.*m33*RSp561e*RSe561p + 1./162.*m33*RSp561e*
         RSe461p - 1./324.*m33*pow(RSp561e,2) - 1./162.*m33*RSp461e*
         RS346pe + 1./162.*m33*RSp461e*RS356pe + 1./162.*m33*RSp461e*
         RSe563p - 1./162.*m33*RSp461e*RSe463p + 1./162.*m33*RSp461e*
         RSp563e );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(mp2,L) * (  - 1./162.*m33*
         RSp461e*RSp463e - 1./162.*m33*RSp461e*RS146pe + 1./162.*m33*
         RSp461e*RS156pe + 1./162.*m33*RSp461e*RSe561p - 1./162.*m33*
         RSp461e*RSe461p + 1./162.*m33*RSp461e*RSp561e - 1./324.*m33*
         pow(RSp461e,2) - 1./81.*m13*RSp563e*RS346pe + 1./81.*m13*
         RSp563e*RS356pe + 1./81.*m13*RSp563e*RSe563p - 1./81.*m13*
         RSp563e*RSe463p + 1./162.*m13*pow(RSp563e,2) + 1./81.*m13*
         RSp463e*RS346pe - 1./81.*m13*RSp463e*RS356pe - 1./81.*m13*
         RSp463e*RSe563p + 1./81.*m13*RSp463e*RSe463p - 1./81.*m13*
         RSp463e*RSp563e + 1./162.*m13*pow(RSp463e,2) + 1./81.*m13*
         RS146pe*RSp563e - 1./81.*m13*RS146pe*RSp463e - 1./81.*m13*
         RS156pe*RSp563e + 1./81.*m13*RS156pe*RSp463e - 1./81.*m13*
         RSe561p*RSp563e + 1./81.*m13*RSe561p*RSp463e + 1./81.*m13*
         RSe461p*RSp563e - 1./81.*m13*RSe461p*RSp463e + 1./81.*m13*
         RSp561e*RS346pe - 1./81.*m13*RSp561e*RS356pe - 1./81.*m13*
         RSp561e*RSe563p );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(mp2,L) * ( 1./81.*m13*RSp561e*
         RSe463p - 1./81.*m13*RSp561e*RSp563e + 1./81.*m13*RSp561e*
         RSp463e - 1./81.*m13*RSp561e*RS146pe + 1./81.*m13*RSp561e*
         RS156pe + 1./81.*m13*RSp561e*RSe561p - 1./81.*m13*RSp561e*
         RSe461p + 1./162.*m13*pow(RSp561e,2) - 1./81.*m13*RSp461e*
         RS346pe + 1./81.*m13*RSp461e*RS356pe + 1./81.*m13*RSp461e*
         RSe563p - 1./81.*m13*RSp461e*RSe463p + 1./81.*m13*RSp461e*
         RSp563e - 1./81.*m13*RSp461e*RSp463e + 1./81.*m13*RSp461e*
         RS146pe - 1./81.*m13*RSp461e*RS156pe - 1./81.*m13*RSp461e*
         RSe561p + 1./81.*m13*RSp461e*RSe461p - 1./81.*m13*RSp461e*
         RSp561e + 1./162.*m13*pow(RSp461e,2) + 1./162.*m11*RSp563e*
         RS346pe - 1./162.*m11*RSp563e*RS356pe - 1./162.*m11*RSp563e*
         RSe563p + 1./162.*m11*RSp563e*RSe463p - 1./324.*m11*pow(
         RSp563e,2) - 1./162.*m11*RSp463e*RS346pe + 1./162.*m11*RSp463e
         *RS356pe + 1./162.*m11*RSp463e*RSe563p - 1./162.*m11*RSp463e*
         RSe463p );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(mp2,L) * ( 1./162.*m11*RSp463e*
         RSp563e - 1./324.*m11*pow(RSp463e,2) + 1./162.*m11*RS146pe*
         RSp563e - 1./162.*m11*RS146pe*RSp463e - 1./162.*m11*RS156pe*
         RSp563e + 1./162.*m11*RS156pe*RSp463e - 1./162.*m11*RSe561p*
         RSp563e + 1./162.*m11*RSe561p*RSp463e + 1./162.*m11*RSe461p*
         RSp563e - 1./162.*m11*RSe461p*RSp463e + 1./162.*m11*RSp561e*
         RS346pe - 1./162.*m11*RSp561e*RS356pe - 1./162.*m11*RSp561e*
         RSe563p + 1./162.*m11*RSp561e*RSe463p - 1./162.*m11*RSp561e*
         RSp563e + 1./162.*m11*RSp561e*RSp463e + 1./54.*m11*RSp561e*
         RS146pe - 1./54.*m11*RSp561e*RS156pe - 1./54.*m11*RSp561e*
         RSe561p + 1./54.*m11*RSp561e*RSe461p - 1./108.*m11*pow(
         RSp561e,2) - 1./162.*m11*RSp461e*RS346pe + 1./162.*m11*RSp461e
         *RS356pe + 1./162.*m11*RSp461e*RSe563p - 1./162.*m11*RSp461e*
         RSe463p + 1./162.*m11*RSp461e*RSp563e - 1./162.*m11*RSp461e*
         RSp463e - 1./54.*m11*RSp461e*RS146pe + 1./54.*m11*RSp461e*
         RS156pe );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(mp2,L) * ( 1./54.*m11*RSp461e*
         RSe561p - 1./54.*m11*RSp461e*RSe461p + 1./54.*m11*RSp461e*
         RSp561e - 1./108.*m11*pow(RSp461e,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(me2,L) * (  - 1./81.*me2*RSe563p
         *RS346pe + 1./81.*me2*RSe563p*RS356pe + 1./81.*me2*pow(
         RSe563p,2) + 1./81.*me2*RSe463p*RS346pe - 1./81.*me2*RSe463p*
         RS356pe - 2./81.*me2*RSe463p*RSe563p + 1./81.*me2*pow(
         RSe463p,2) + 1./81.*me2*RSp563e*RSe563p - 1./81.*me2*RSp563e*
         RSe463p - 1./81.*me2*RSp463e*RSe563p + 1./81.*me2*RSp463e*
         RSe463p + 1./81.*me2*RS146pe*RSe563p - 1./81.*me2*RS146pe*
         RSe463p - 1./81.*me2*RS156pe*RSe563p + 1./81.*me2*RS156pe*
         RSe463p + 1./81.*me2*RSe561p*RS346pe - 1./81.*me2*RSe561p*
         RS356pe - 2./81.*me2*RSe561p*RSe563p + 2./81.*me2*RSe561p*
         RSe463p - 1./81.*me2*RSe561p*RSp563e + 1./81.*me2*RSe561p*
         RSp463e - 1./81.*me2*RSe561p*RS146pe + 1./81.*me2*RSe561p*
         RS156pe + 1./81.*me2*pow(RSe561p,2) - 1./81.*me2*RSe461p*
         RS346pe + 1./81.*me2*RSe461p*RS356pe + 2./81.*me2*RSe461p*
         RSe563p - 2./81.*me2*RSe461p*RSe463p + 1./81.*me2*RSe461p*
         RSp563e );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(me2,L) * (  - 1./81.*me2*RSe461p
         *RSp463e + 1./81.*me2*RSe461p*RS146pe - 1./81.*me2*RSe461p*
         RS156pe - 2./81.*me2*RSe461p*RSe561p + 1./81.*me2*pow(
         RSe461p,2) - 1./81.*me2*RSp561e*RSe563p + 1./81.*me2*RSp561e*
         RSe463p + 1./81.*me2*RSp561e*RSe561p - 1./81.*me2*RSp561e*
         RSe461p + 1./81.*me2*RSp461e*RSe563p - 1./81.*me2*RSp461e*
         RSe463p - 1./81.*me2*RSp461e*RSe561p + 1./81.*me2*RSp461e*
         RSe461p - 1./162.*m55*RSe563p*RS346pe - 1./162.*m55*RSe563p*
         RS356pe - 1./324.*m55*pow(RSe563p,2) - 1./54.*m55*RSe463p*
         RS346pe - 1./162.*m55*RSe463p*RS356pe - 1./162.*m55*RSe463p*
         RSe563p - 1./108.*m55*pow(RSe463p,2) + 1./162.*m55*RS146pe*
         RSe563p + 1./54.*m55*RS146pe*RSe463p + 1./162.*m55*RS156pe*
         RSe563p + 1./162.*m55*RS156pe*RSe463p + 1./162.*m55*RSe561p*
         RS346pe + 1./162.*m55*RSe561p*RS356pe + 1./162.*m55*RSe561p*
         RSe563p + 1./162.*m55*RSe561p*RSe463p - 1./162.*m55*RSe561p*
         RS146pe );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(me2,L) * (  - 1./162.*m55*
         RSe561p*RS156pe - 1./324.*m55*pow(RSe561p,2) + 1./54.*m55*
         RSe461p*RS346pe + 1./162.*m55*RSe461p*RS356pe + 1./162.*m55*
         RSe461p*RSe563p + 1./54.*m55*RSe461p*RSe463p - 1./54.*m55*
         RSe461p*RS146pe - 1./162.*m55*RSe461p*RS156pe - 1./162.*m55*
         RSe461p*RSe561p - 1./108.*m55*pow(RSe461p,2) - 1./81.*m45*
         RSe563p*RS346pe + 1./81.*m45*RSe563p*RS356pe + 1./162.*m45*
         pow(RSe563p,2) + 1./81.*m45*RSe463p*RS346pe - 1./81.*m45*
         RSe463p*RS356pe - 1./81.*m45*RSe463p*RSe563p + 1./162.*m45*
         pow(RSe463p,2) + 1./81.*m45*RS146pe*RSe563p - 1./81.*m45*
         RS146pe*RSe463p - 1./81.*m45*RS156pe*RSe563p + 1./81.*m45*
         RS156pe*RSe463p + 1./81.*m45*RSe561p*RS346pe - 1./81.*m45*
         RSe561p*RS356pe - 1./81.*m45*RSe561p*RSe563p + 1./81.*m45*
         RSe561p*RSe463p - 1./81.*m45*RSe561p*RS146pe + 1./81.*m45*
         RSe561p*RS156pe + 1./162.*m45*pow(RSe561p,2) - 1./81.*m45*
         RSe461p*RS346pe );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(me2,L) * ( 1./81.*m45*RSe461p*
         RS356pe + 1./81.*m45*RSe461p*RSe563p - 1./81.*m45*RSe461p*
         RSe463p + 1./81.*m45*RSe461p*RS146pe - 1./81.*m45*RSe461p*
         RS156pe - 1./81.*m45*RSe461p*RSe561p + 1./162.*m45*pow(
         RSe461p,2) - 1./162.*m44*RSe563p*RS346pe - 1./54.*m44*RSe563p*
         RS356pe - 1./108.*m44*pow(RSe563p,2) - 1./162.*m44*RSe463p*
         RS346pe - 1./162.*m44*RSe463p*RS356pe - 1./162.*m44*RSe463p*
         RSe563p - 1./324.*m44*pow(RSe463p,2) + 1./162.*m44*RS146pe*
         RSe563p + 1./162.*m44*RS146pe*RSe463p + 1./54.*m44*RS156pe*
         RSe563p + 1./162.*m44*RS156pe*RSe463p + 1./162.*m44*RSe561p*
         RS346pe + 1./54.*m44*RSe561p*RS356pe + 1./54.*m44*RSe561p*
         RSe563p + 1./162.*m44*RSe561p*RSe463p - 1./162.*m44*RSe561p*
         RS146pe - 1./54.*m44*RSe561p*RS156pe - 1./108.*m44*pow(
         RSe561p,2) + 1./162.*m44*RSe461p*RS346pe + 1./162.*m44*RSe461p
         *RS356pe + 1./162.*m44*RSe461p*RSe563p + 1./162.*m44*RSe461p*
         RSe463p );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(me2,L) * (  - 1./162.*m44*
         RSe461p*RS146pe - 1./162.*m44*RSe461p*RS156pe - 1./162.*m44*
         RSe461p*RSe561p - 1./324.*m44*pow(RSe461p,2) + 1./54.*m33*
         RSe563p*RS346pe - 1./54.*m33*RSe563p*RS356pe - 1./108.*m33*
         pow(RSe563p,2) - 1./54.*m33*RSe463p*RS346pe + 1./54.*m33*
         RSe463p*RS356pe + 1./54.*m33*RSe463p*RSe563p - 1./108.*m33*
         pow(RSe463p,2) + 1./162.*m33*RS146pe*RSe563p - 1./162.*m33*
         RS146pe*RSe463p - 1./162.*m33*RS156pe*RSe563p + 1./162.*m33*
         RS156pe*RSe463p + 1./162.*m33*RSe561p*RS346pe - 1./162.*m33*
         RSe561p*RS356pe - 1./162.*m33*RSe561p*RSe563p + 1./162.*m33*
         RSe561p*RSe463p + 1./162.*m33*RSe561p*RS146pe - 1./162.*m33*
         RSe561p*RS156pe - 1./324.*m33*pow(RSe561p,2) - 1./162.*m33*
         RSe461p*RS346pe + 1./162.*m33*RSe461p*RS356pe + 1./162.*m33*
         RSe461p*RSe563p - 1./162.*m33*RSe461p*RSe463p - 1./162.*m33*
         RSe461p*RS146pe + 1./162.*m33*RSe461p*RS156pe + 1./162.*m33*
         RSe461p*RSe561p );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(me2,L) * (  - 1./324.*m33*pow(
         RSe461p,2) - 1./81.*m13*RSe563p*RS346pe + 1./81.*m13*RSe563p*
         RS356pe + 1./162.*m13*pow(RSe563p,2) + 1./81.*m13*RSe463p*
         RS346pe - 1./81.*m13*RSe463p*RS356pe - 1./81.*m13*RSe463p*
         RSe563p + 1./162.*m13*pow(RSe463p,2) + 1./81.*m13*RS146pe*
         RSe563p - 1./81.*m13*RS146pe*RSe463p - 1./81.*m13*RS156pe*
         RSe563p + 1./81.*m13*RS156pe*RSe463p + 1./81.*m13*RSe561p*
         RS346pe - 1./81.*m13*RSe561p*RS356pe - 1./81.*m13*RSe561p*
         RSe563p + 1./81.*m13*RSe561p*RSe463p - 1./81.*m13*RSe561p*
         RS146pe + 1./81.*m13*RSe561p*RS156pe + 1./162.*m13*pow(
         RSe561p,2) - 1./81.*m13*RSe461p*RS346pe + 1./81.*m13*RSe461p*
         RS356pe + 1./81.*m13*RSe461p*RSe563p - 1./81.*m13*RSe461p*
         RSe463p + 1./81.*m13*RSe461p*RS146pe - 1./81.*m13*RSe461p*
         RS156pe - 1./81.*m13*RSe461p*RSe561p + 1./162.*m13*pow(
         RSe461p,2) + 1./162.*m11*RSe563p*RS346pe - 1./162.*m11*RSe563p
         *RS356pe );

      M13p6x23RV +=  + Ab(m45,mu2)*AbV(me2,L) * (  - 1./324.*m11*pow(
         RSe563p,2) - 1./162.*m11*RSe463p*RS346pe + 1./162.*m11*RSe463p
         *RS356pe + 1./162.*m11*RSe463p*RSe563p - 1./324.*m11*pow(
         RSe463p,2) + 1./162.*m11*RS146pe*RSe563p - 1./162.*m11*RS146pe
         *RSe463p - 1./162.*m11*RS156pe*RSe563p + 1./162.*m11*RS156pe*
         RSe463p + 1./162.*m11*RSe561p*RS346pe - 1./162.*m11*RSe561p*
         RS356pe - 1./162.*m11*RSe561p*RSe563p + 1./162.*m11*RSe561p*
         RSe463p + 1./54.*m11*RSe561p*RS146pe - 1./54.*m11*RSe561p*
         RS156pe - 1./108.*m11*pow(RSe561p,2) - 1./162.*m11*RSe461p*
         RS346pe + 1./162.*m11*RSe461p*RS356pe + 1./162.*m11*RSe461p*
         RSe563p - 1./162.*m11*RSe461p*RSe463p - 1./54.*m11*RSe461p*
         RS146pe + 1./54.*m11*RSe461p*RS156pe + 1./54.*m11*RSe461p*
         RSe561p - 1./108.*m11*pow(RSe461p,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,m33,L) * ( 1./108.*m33*m55*
         RS146pe*RS346pe + 1./324.*m33*m55*RS146pe*RS356pe + 1./324.*
         m33*m55*RS156pe*RS346pe + 1./324.*m33*m55*RS156pe*RS356pe - 1./
         162.*m33*m45*RS146pe*RS346pe + 1./162.*m33*m45*RS146pe*RS356pe
          + 1./162.*m33*m45*RS156pe*RS346pe - 1./162.*m33*m45*RS156pe*
         RS356pe + 1./324.*m33*m44*RS146pe*RS346pe + 1./324.*m33*m44*
         RS146pe*RS356pe + 1./324.*m33*m44*RS156pe*RS346pe + 1./108.*
         m33*m44*RS156pe*RS356pe + 1./54.*m13*m55*RS146pe*RS346pe + 1./
         162.*m13*m55*RS146pe*RS356pe + 1./162.*m13*m55*RS156pe*RS346pe
          + 1./162.*m13*m55*RS156pe*RS356pe - 1./81.*m13*m45*RS146pe*
         RS346pe + 1./81.*m13*m45*RS146pe*RS356pe + 1./81.*m13*m45*
         RS156pe*RS346pe - 1./81.*m13*m45*RS156pe*RS356pe + 1./162.*m13
         *m44*RS146pe*RS346pe + 1./162.*m13*m44*RS146pe*RS356pe + 1./
         162.*m13*m44*RS156pe*RS346pe + 1./54.*m13*m44*RS156pe*RS356pe
          + 1./36.*m11*m55*RS146pe*RS346pe + 1./108.*m11*m55*RS146pe*
         RS356pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,m33,L) * ( 1./108.*m11*m55*
         RS156pe*RS346pe + 1./108.*m11*m55*RS156pe*RS356pe - 1./54.*m11
         *m45*RS146pe*RS346pe + 1./54.*m11*m45*RS146pe*RS356pe + 1./54.
         *m11*m45*RS156pe*RS346pe - 1./54.*m11*m45*RS156pe*RS356pe + 1./
         108.*m11*m44*RS146pe*RS346pe + 1./108.*m11*m44*RS146pe*RS356pe
          + 1./108.*m11*m44*RS156pe*RS346pe + 1./36.*m11*m44*RS156pe*
         RS356pe - 1./54.*m11*m33*RS146pe*RS346pe + 1./54.*m11*m33*
         RS146pe*RS356pe + 1./54.*m11*m33*RS156pe*RS346pe - 1./54.*m11*
         m33*RS156pe*RS356pe - 1./81.*m11*m13*RS146pe*RS346pe + 1./81.*
         m11*m13*RS146pe*RS356pe + 1./81.*m11*m13*RS156pe*RS346pe - 1./
         81.*m11*m13*RS156pe*RS356pe - 1./162.*pow(m11,2)*RS146pe*
         RS346pe + 1./162.*pow(m11,2)*RS146pe*RS356pe + 1./162.*pow(
         m11,2)*RS156pe*RS346pe - 1./162.*pow(m11,2)*RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,mp2,L) * ( 1./324.*m33*m55*
         RS146pe*RSp563e + 1./108.*m33*m55*RS146pe*RSp463e + 1./324.*
         m33*m55*RS156pe*RSp563e + 1./324.*m33*m55*RS156pe*RSp463e + 1./
         324.*m33*m55*RSp561e*RS146pe + 1./324.*m33*m55*RSp561e*RS156pe
          + 1./108.*m33*m55*RSp461e*RS146pe + 1./324.*m33*m55*RSp461e*
         RS156pe + 1./162.*m33*m45*RS146pe*RSp563e - 1./162.*m33*m45*
         RS146pe*RSp463e - 1./162.*m33*m45*RS156pe*RSp563e + 1./162.*
         m33*m45*RS156pe*RSp463e + 1./162.*m33*m45*RSp561e*RS146pe - 1./
         162.*m33*m45*RSp561e*RS156pe - 1./162.*m33*m45*RSp461e*RS146pe
          + 1./162.*m33*m45*RSp461e*RS156pe + 1./324.*m33*m44*RS146pe*
         RSp563e + 1./324.*m33*m44*RS146pe*RSp463e + 1./108.*m33*m44*
         RS156pe*RSp563e + 1./324.*m33*m44*RS156pe*RSp463e + 1./324.*
         m33*m44*RSp561e*RS146pe + 1./108.*m33*m44*RSp561e*RS156pe + 1./
         324.*m33*m44*RSp461e*RS146pe + 1./324.*m33*m44*RSp461e*RS156pe
          + 1./162.*m13*m55*RS146pe*RSp563e + 1./54.*m13*m55*RS146pe*
         RSp463e );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,mp2,L) * ( 1./162.*m13*m55*
         RS156pe*RSp563e + 1./162.*m13*m55*RS156pe*RSp463e - 1./162.*
         m13*m55*RSp561e*RS146pe - 1./162.*m13*m55*RSp561e*RS156pe - 1./
         54.*m13*m55*RSp461e*RS146pe - 1./162.*m13*m55*RSp461e*RS156pe
          + 1./81.*m13*m45*RS146pe*RSp563e - 1./81.*m13*m45*RS146pe*
         RSp463e - 1./81.*m13*m45*RS156pe*RSp563e + 1./81.*m13*m45*
         RS156pe*RSp463e - 1./81.*m13*m45*RSp561e*RS146pe + 1./81.*m13*
         m45*RSp561e*RS156pe + 1./81.*m13*m45*RSp461e*RS146pe - 1./81.*
         m13*m45*RSp461e*RS156pe + 1./162.*m13*m44*RS146pe*RSp563e + 1./
         162.*m13*m44*RS146pe*RSp463e + 1./54.*m13*m44*RS156pe*RSp563e
          + 1./162.*m13*m44*RS156pe*RSp463e - 1./162.*m13*m44*RSp561e*
         RS146pe - 1./54.*m13*m44*RSp561e*RS156pe - 1./162.*m13*m44*
         RSp461e*RS146pe - 1./162.*m13*m44*RSp461e*RS156pe + 1./81.*m11
         *mp2*RS146pe*RSp563e - 1./81.*m11*mp2*RS146pe*RSp463e - 1./81.
         *m11*mp2*RS156pe*RSp563e + 1./81.*m11*mp2*RS156pe*RSp463e - 1./
         81.*m11*mp2*RSp561e*RS146pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,mp2,L) * ( 1./81.*m11*mp2*
         RSp561e*RS156pe + 1./81.*m11*mp2*RSp461e*RS146pe - 1./81.*m11*
         mp2*RSp461e*RS156pe + 1./108.*m11*m55*RS146pe*RSp563e + 1./36.
         *m11*m55*RS146pe*RSp463e + 1./108.*m11*m55*RS156pe*RSp563e + 1.
         /108.*m11*m55*RS156pe*RSp463e + 1./324.*m11*m55*RSp561e*
         RS146pe + 1./324.*m11*m55*RSp561e*RS156pe + 1./108.*m11*m55*
         RSp461e*RS146pe + 1./324.*m11*m55*RSp461e*RS156pe + 1./54.*m11
         *m45*RS146pe*RSp563e - 1./54.*m11*m45*RS146pe*RSp463e - 1./54.
         *m11*m45*RS156pe*RSp563e + 1./54.*m11*m45*RS156pe*RSp463e + 1./
         162.*m11*m45*RSp561e*RS146pe - 1./162.*m11*m45*RSp561e*RS156pe
          - 1./162.*m11*m45*RSp461e*RS146pe + 1./162.*m11*m45*RSp461e*
         RS156pe + 1./108.*m11*m44*RS146pe*RSp563e + 1./108.*m11*m44*
         RS146pe*RSp463e + 1./36.*m11*m44*RS156pe*RSp563e + 1./108.*m11
         *m44*RS156pe*RSp463e + 1./324.*m11*m44*RSp561e*RS146pe + 1./
         108.*m11*m44*RSp561e*RS156pe + 1./324.*m11*m44*RSp461e*RS146pe
          + 1./324.*m11*m44*RSp461e*RS156pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,mp2,L) * ( 1./162.*m11*m33*
         RS146pe*RSp563e - 1./162.*m11*m33*RS146pe*RSp463e - 1./162.*
         m11*m33*RS156pe*RSp563e + 1./162.*m11*m33*RS156pe*RSp463e + 1./
         162.*m11*m33*RSp561e*RS146pe - 1./162.*m11*m33*RSp561e*RS156pe
          - 1./162.*m11*m33*RSp461e*RS146pe + 1./162.*m11*m33*RSp461e*
         RS156pe + 1./81.*m11*m13*RS146pe*RSp563e - 1./81.*m11*m13*
         RS146pe*RSp463e - 1./81.*m11*m13*RS156pe*RSp563e + 1./81.*m11*
         m13*RS156pe*RSp463e - 1./81.*m11*m13*RSp561e*RS146pe + 1./81.*
         m11*m13*RSp561e*RS156pe + 1./81.*m11*m13*RSp461e*RS146pe - 1./
         81.*m11*m13*RSp461e*RS156pe + 1./162.*pow(m11,2)*RS146pe*
         RSp563e - 1./162.*pow(m11,2)*RS146pe*RSp463e - 1./162.*pow(
         m11,2)*RS156pe*RSp563e + 1./162.*pow(m11,2)*RS156pe*RSp463e + 
         1./54.*pow(m11,2)*RSp561e*RS146pe - 1./54.*pow(m11,2)*RSp561e*
         RS156pe - 1./54.*pow(m11,2)*RSp461e*RS146pe + 1./54.*pow(
         m11,2)*RSp461e*RS156pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,me2,L) * ( 1./324.*m33*m55*
         RS146pe*RSe563p + 1./108.*m33*m55*RS146pe*RSe463p + 1./324.*
         m33*m55*RS156pe*RSe563p + 1./324.*m33*m55*RS156pe*RSe463p + 1./
         324.*m33*m55*RSe561p*RS146pe + 1./324.*m33*m55*RSe561p*RS156pe
          + 1./108.*m33*m55*RSe461p*RS146pe + 1./324.*m33*m55*RSe461p*
         RS156pe + 1./162.*m33*m45*RS146pe*RSe563p - 1./162.*m33*m45*
         RS146pe*RSe463p - 1./162.*m33*m45*RS156pe*RSe563p + 1./162.*
         m33*m45*RS156pe*RSe463p + 1./162.*m33*m45*RSe561p*RS146pe - 1./
         162.*m33*m45*RSe561p*RS156pe - 1./162.*m33*m45*RSe461p*RS146pe
          + 1./162.*m33*m45*RSe461p*RS156pe + 1./324.*m33*m44*RS146pe*
         RSe563p + 1./324.*m33*m44*RS146pe*RSe463p + 1./108.*m33*m44*
         RS156pe*RSe563p + 1./324.*m33*m44*RS156pe*RSe463p + 1./324.*
         m33*m44*RSe561p*RS146pe + 1./108.*m33*m44*RSe561p*RS156pe + 1./
         324.*m33*m44*RSe461p*RS146pe + 1./324.*m33*m44*RSe461p*RS156pe
          + 1./162.*m13*m55*RS146pe*RSe563p + 1./54.*m13*m55*RS146pe*
         RSe463p );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,me2,L) * ( 1./162.*m13*m55*
         RS156pe*RSe563p + 1./162.*m13*m55*RS156pe*RSe463p - 1./162.*
         m13*m55*RSe561p*RS146pe - 1./162.*m13*m55*RSe561p*RS156pe - 1./
         54.*m13*m55*RSe461p*RS146pe - 1./162.*m13*m55*RSe461p*RS156pe
          + 1./81.*m13*m45*RS146pe*RSe563p - 1./81.*m13*m45*RS146pe*
         RSe463p - 1./81.*m13*m45*RS156pe*RSe563p + 1./81.*m13*m45*
         RS156pe*RSe463p - 1./81.*m13*m45*RSe561p*RS146pe + 1./81.*m13*
         m45*RSe561p*RS156pe + 1./81.*m13*m45*RSe461p*RS146pe - 1./81.*
         m13*m45*RSe461p*RS156pe + 1./162.*m13*m44*RS146pe*RSe563p + 1./
         162.*m13*m44*RS146pe*RSe463p + 1./54.*m13*m44*RS156pe*RSe563p
          + 1./162.*m13*m44*RS156pe*RSe463p - 1./162.*m13*m44*RSe561p*
         RS146pe - 1./54.*m13*m44*RSe561p*RS156pe - 1./162.*m13*m44*
         RSe461p*RS146pe - 1./162.*m13*m44*RSe461p*RS156pe + 1./81.*m11
         *me2*RS146pe*RSe563p - 1./81.*m11*me2*RS146pe*RSe463p - 1./81.
         *m11*me2*RS156pe*RSe563p + 1./81.*m11*me2*RS156pe*RSe463p - 1./
         81.*m11*me2*RSe561p*RS146pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,me2,L) * ( 1./81.*m11*me2*
         RSe561p*RS156pe + 1./81.*m11*me2*RSe461p*RS146pe - 1./81.*m11*
         me2*RSe461p*RS156pe + 1./108.*m11*m55*RS146pe*RSe563p + 1./36.
         *m11*m55*RS146pe*RSe463p + 1./108.*m11*m55*RS156pe*RSe563p + 1.
         /108.*m11*m55*RS156pe*RSe463p + 1./324.*m11*m55*RSe561p*
         RS146pe + 1./324.*m11*m55*RSe561p*RS156pe + 1./108.*m11*m55*
         RSe461p*RS146pe + 1./324.*m11*m55*RSe461p*RS156pe + 1./54.*m11
         *m45*RS146pe*RSe563p - 1./54.*m11*m45*RS146pe*RSe463p - 1./54.
         *m11*m45*RS156pe*RSe563p + 1./54.*m11*m45*RS156pe*RSe463p + 1./
         162.*m11*m45*RSe561p*RS146pe - 1./162.*m11*m45*RSe561p*RS156pe
          - 1./162.*m11*m45*RSe461p*RS146pe + 1./162.*m11*m45*RSe461p*
         RS156pe + 1./108.*m11*m44*RS146pe*RSe563p + 1./108.*m11*m44*
         RS146pe*RSe463p + 1./36.*m11*m44*RS156pe*RSe563p + 1./108.*m11
         *m44*RS156pe*RSe463p + 1./324.*m11*m44*RSe561p*RS146pe + 1./
         108.*m11*m44*RSe561p*RS156pe + 1./324.*m11*m44*RSe461p*RS146pe
          + 1./324.*m11*m44*RSe461p*RS156pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,me2,L) * ( 1./162.*m11*m33*
         RS146pe*RSe563p - 1./162.*m11*m33*RS146pe*RSe463p - 1./162.*
         m11*m33*RS156pe*RSe563p + 1./162.*m11*m33*RS156pe*RSe463p + 1./
         162.*m11*m33*RSe561p*RS146pe - 1./162.*m11*m33*RSe561p*RS156pe
          - 1./162.*m11*m33*RSe461p*RS146pe + 1./162.*m11*m33*RSe461p*
         RS156pe + 1./81.*m11*m13*RS146pe*RSe563p - 1./81.*m11*m13*
         RS146pe*RSe463p - 1./81.*m11*m13*RS156pe*RSe563p + 1./81.*m11*
         m13*RS156pe*RSe463p - 1./81.*m11*m13*RSe561p*RS146pe + 1./81.*
         m11*m13*RSe561p*RS156pe + 1./81.*m11*m13*RSe461p*RS146pe - 1./
         81.*m11*m13*RSe461p*RS156pe + 1./162.*pow(m11,2)*RS146pe*
         RSe563p - 1./162.*pow(m11,2)*RS146pe*RSe463p - 1./162.*pow(
         m11,2)*RS156pe*RSe563p + 1./162.*pow(m11,2)*RS156pe*RSe463p + 
         1./54.*pow(m11,2)*RSe561p*RS146pe - 1./54.*pow(m11,2)*RSe561p*
         RS156pe - 1./54.*pow(m11,2)*RSe461p*RS146pe + 1./54.*pow(
         m11,2)*RSe461p*RS156pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,L) * ( 1./216.*m33*m55*pow(
         RS146pe,2) + 1./324.*m33*m55*RS156pe*RS146pe + 1./648.*m33*m55
         *pow(RS156pe,2) - 1./324.*m33*m45*pow(RS146pe,2) + 1./162.*m33
         *m45*RS156pe*RS146pe - 1./324.*m33*m45*pow(RS156pe,2) + 1./648.
         *m33*m44*pow(RS146pe,2) + 1./324.*m33*m44*RS156pe*RS146pe + 1./
         216.*m33*m44*pow(RS156pe,2) - 1./108.*m13*m55*pow(RS146pe,2)
          - 1./162.*m13*m55*RS156pe*RS146pe - 1./324.*m13*m55*pow(
         RS156pe,2) + 1./162.*m13*m45*pow(RS146pe,2) - 1./81.*m13*m45*
         RS156pe*RS146pe + 1./162.*m13*m45*pow(RS156pe,2) - 1./324.*m13
         *m44*pow(RS146pe,2) - 1./162.*m13*m44*RS156pe*RS146pe - 1./108.
         *m13*m44*pow(RS156pe,2) + 1./216.*m11*m55*pow(RS146pe,2) + 1./
         324.*m11*m55*RS156pe*RS146pe + 1./648.*m11*m55*pow(RS156pe,2)
          - 1./324.*m11*m45*pow(RS146pe,2) + 1./162.*m11*m45*RS156pe*
         RS146pe - 1./324.*m11*m45*pow(RS156pe,2) + 1./648.*m11*m44*
         pow(RS146pe,2) + 1./324.*m11*m44*RS156pe*RS146pe + 1./216.*m11
         *m44*pow(RS156pe,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,L) * (  - 1./324.*m11*m33*
         pow(RS146pe,2) + 1./162.*m11*m33*RS156pe*RS146pe - 1./324.*m11
         *m33*pow(RS156pe,2) + 1./162.*m11*m13*pow(RS146pe,2) - 1./81.*
         m11*m13*RS156pe*RS146pe + 1./162.*m11*m13*pow(RS156pe,2) - 1./
         324.*pow(m11,2)*pow(RS146pe,2) + 1./162.*pow(m11,2)*RS156pe*
         RS146pe - 1./324.*pow(m11,2)*pow(RS156pe,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m14,L) * ( 1./144.*m44*m55 - 1./
         72.*m44*m45 + 1./72.*pow(m44,2) + 1./144.*m33*m55 - 1./72.*m33
         *m45 + 1./72.*m33*m44 - 1./72.*m14*m55 + 1./36.*m14*m45 - 1./
         24.*m14*m44 - 1./72.*m14*m33 + 1./36.*pow(m14,2) - 1./72.*m13*
         m55 + 1./36.*m13*m45 - 1./36.*m13*m44 + 1./36.*m13*m14 + 1./72.
         *m11*m55 - 1./36.*m11*m45 + 5./144.*m11*m44 + 1./144.*m11*m33
          - 1./24.*m11*m14 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m15,L) * ( 1./72.*pow(m55,2) - 1.
         /72.*m45*m55 + 1./144.*m44*m55 + 1./72.*m33*m55 - 1./72.*m33*
         m45 + 1./144.*m33*m44 - 1./24.*m15*m55 + 1./36.*m15*m45 - 1./
         72.*m15*m44 - 1./72.*m15*m33 + 1./36.*pow(m15,2) - 1./36.*m13*
         m55 + 1./36.*m13*m45 - 1./72.*m13*m44 + 1./36.*m13*m15 + 5./
         144.*m11*m55 - 1./36.*m11*m45 + 1./72.*m11*m44 + 1./144.*m11*
         m33 - 1./24.*m11*m15 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,mp2,L) * (  - 1./81.*m33*mp2
         *RSp563e*RS346pe + 1./81.*m33*mp2*RSp563e*RS356pe + 1./81.*m33
         *mp2*RSp463e*RS346pe - 1./81.*m33*mp2*RSp463e*RS356pe + 1./81.
         *m33*mp2*RSp561e*RS346pe - 1./81.*m33*mp2*RSp561e*RS356pe - 1./
         81.*m33*mp2*RSp461e*RS346pe + 1./81.*m33*mp2*RSp461e*RS356pe
          + 1./324.*m33*m55*RSp563e*RS346pe + 1./324.*m33*m55*RSp563e*
         RS356pe + 1./108.*m33*m55*RSp463e*RS346pe + 1./324.*m33*m55*
         RSp463e*RS356pe + 1./108.*m33*m55*RSp561e*RS346pe + 1./108.*
         m33*m55*RSp561e*RS356pe + 1./36.*m33*m55*RSp461e*RS346pe + 1./
         108.*m33*m55*RSp461e*RS356pe + 1./162.*m33*m45*RSp563e*RS346pe
          - 1./162.*m33*m45*RSp563e*RS356pe - 1./162.*m33*m45*RSp463e*
         RS346pe + 1./162.*m33*m45*RSp463e*RS356pe + 1./54.*m33*m45*
         RSp561e*RS346pe - 1./54.*m33*m45*RSp561e*RS356pe - 1./54.*m33*
         m45*RSp461e*RS346pe + 1./54.*m33*m45*RSp461e*RS356pe + 1./324.
         *m33*m44*RSp563e*RS346pe + 1./108.*m33*m44*RSp563e*RS356pe + 1.
         /324.*m33*m44*RSp463e*RS346pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,mp2,L) * ( 1./324.*m33*m44*
         RSp463e*RS356pe + 1./108.*m33*m44*RSp561e*RS346pe + 1./36.*m33
         *m44*RSp561e*RS356pe + 1./108.*m33*m44*RSp461e*RS346pe + 1./
         108.*m33*m44*RSp461e*RS356pe + 1./54.*pow(m33,2)*RSp563e*
         RS346pe - 1./54.*pow(m33,2)*RSp563e*RS356pe - 1./54.*pow(
         m33,2)*RSp463e*RS346pe + 1./54.*pow(m33,2)*RSp463e*RS356pe + 1.
         /162.*pow(m33,2)*RSp561e*RS346pe - 1./162.*pow(m33,2)*RSp561e*
         RS356pe - 1./162.*pow(m33,2)*RSp461e*RS346pe + 1./162.*pow(
         m33,2)*RSp461e*RS356pe - 1./162.*m13*m55*RSp563e*RS346pe - 1./
         162.*m13*m55*RSp563e*RS356pe - 1./54.*m13*m55*RSp463e*RS346pe
          - 1./162.*m13*m55*RSp463e*RS356pe + 1./162.*m13*m55*RSp561e*
         RS346pe + 1./162.*m13*m55*RSp561e*RS356pe + 1./54.*m13*m55*
         RSp461e*RS346pe + 1./162.*m13*m55*RSp461e*RS356pe - 1./81.*m13
         *m45*RSp563e*RS346pe + 1./81.*m13*m45*RSp563e*RS356pe + 1./81.
         *m13*m45*RSp463e*RS346pe - 1./81.*m13*m45*RSp463e*RS356pe + 1./
         81.*m13*m45*RSp561e*RS346pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,mp2,L) * (  - 1./81.*m13*m45
         *RSp561e*RS356pe - 1./81.*m13*m45*RSp461e*RS346pe + 1./81.*m13
         *m45*RSp461e*RS356pe - 1./162.*m13*m44*RSp563e*RS346pe - 1./54.
         *m13*m44*RSp563e*RS356pe - 1./162.*m13*m44*RSp463e*RS346pe - 1.
         /162.*m13*m44*RSp463e*RS356pe + 1./162.*m13*m44*RSp561e*
         RS346pe + 1./54.*m13*m44*RSp561e*RS356pe + 1./162.*m13*m44*
         RSp461e*RS346pe + 1./162.*m13*m44*RSp461e*RS356pe - 1./81.*m13
         *m33*RSp563e*RS346pe + 1./81.*m13*m33*RSp563e*RS356pe + 1./81.
         *m13*m33*RSp463e*RS346pe - 1./81.*m13*m33*RSp463e*RS356pe + 1./
         81.*m13*m33*RSp561e*RS346pe - 1./81.*m13*m33*RSp561e*RS356pe
          - 1./81.*m13*m33*RSp461e*RS346pe + 1./81.*m13*m33*RSp461e*
         RS356pe + 1./324.*m11*m55*RSp563e*RS346pe + 1./324.*m11*m55*
         RSp563e*RS356pe + 1./108.*m11*m55*RSp463e*RS346pe + 1./324.*
         m11*m55*RSp463e*RS356pe + 1./324.*m11*m55*RSp561e*RS346pe + 1./
         324.*m11*m55*RSp561e*RS356pe + 1./108.*m11*m55*RSp461e*RS346pe
          + 1./324.*m11*m55*RSp461e*RS356pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,mp2,L) * ( 1./162.*m11*m45*
         RSp563e*RS346pe - 1./162.*m11*m45*RSp563e*RS356pe - 1./162.*
         m11*m45*RSp463e*RS346pe + 1./162.*m11*m45*RSp463e*RS356pe + 1./
         162.*m11*m45*RSp561e*RS346pe - 1./162.*m11*m45*RSp561e*RS356pe
          - 1./162.*m11*m45*RSp461e*RS346pe + 1./162.*m11*m45*RSp461e*
         RS356pe + 1./324.*m11*m44*RSp563e*RS346pe + 1./108.*m11*m44*
         RSp563e*RS356pe + 1./324.*m11*m44*RSp463e*RS346pe + 1./324.*
         m11*m44*RSp463e*RS356pe + 1./324.*m11*m44*RSp561e*RS346pe + 1./
         108.*m11*m44*RSp561e*RS356pe + 1./324.*m11*m44*RSp461e*RS346pe
          + 1./324.*m11*m44*RSp461e*RS356pe + 1./162.*m11*m33*RSp563e*
         RS346pe - 1./162.*m11*m33*RSp563e*RS356pe - 1./162.*m11*m33*
         RSp463e*RS346pe + 1./162.*m11*m33*RSp463e*RS356pe + 1./162.*
         m11*m33*RSp561e*RS346pe - 1./162.*m11*m33*RSp561e*RS356pe - 1./
         162.*m11*m33*RSp461e*RS346pe + 1./162.*m11*m33*RSp461e*RS356pe
          );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,me2,L) * (  - 1./81.*m33*me2
         *RSe563p*RS346pe + 1./81.*m33*me2*RSe563p*RS356pe + 1./81.*m33
         *me2*RSe463p*RS346pe - 1./81.*m33*me2*RSe463p*RS356pe + 1./81.
         *m33*me2*RSe561p*RS346pe - 1./81.*m33*me2*RSe561p*RS356pe - 1./
         81.*m33*me2*RSe461p*RS346pe + 1./81.*m33*me2*RSe461p*RS356pe
          + 1./324.*m33*m55*RSe563p*RS346pe + 1./324.*m33*m55*RSe563p*
         RS356pe + 1./108.*m33*m55*RSe463p*RS346pe + 1./324.*m33*m55*
         RSe463p*RS356pe + 1./108.*m33*m55*RSe561p*RS346pe + 1./108.*
         m33*m55*RSe561p*RS356pe + 1./36.*m33*m55*RSe461p*RS346pe + 1./
         108.*m33*m55*RSe461p*RS356pe + 1./162.*m33*m45*RSe563p*RS346pe
          - 1./162.*m33*m45*RSe563p*RS356pe - 1./162.*m33*m45*RSe463p*
         RS346pe + 1./162.*m33*m45*RSe463p*RS356pe + 1./54.*m33*m45*
         RSe561p*RS346pe - 1./54.*m33*m45*RSe561p*RS356pe - 1./54.*m33*
         m45*RSe461p*RS346pe + 1./54.*m33*m45*RSe461p*RS356pe + 1./324.
         *m33*m44*RSe563p*RS346pe + 1./108.*m33*m44*RSe563p*RS356pe + 1.
         /324.*m33*m44*RSe463p*RS346pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,me2,L) * ( 1./324.*m33*m44*
         RSe463p*RS356pe + 1./108.*m33*m44*RSe561p*RS346pe + 1./36.*m33
         *m44*RSe561p*RS356pe + 1./108.*m33*m44*RSe461p*RS346pe + 1./
         108.*m33*m44*RSe461p*RS356pe + 1./54.*pow(m33,2)*RSe563p*
         RS346pe - 1./54.*pow(m33,2)*RSe563p*RS356pe - 1./54.*pow(
         m33,2)*RSe463p*RS346pe + 1./54.*pow(m33,2)*RSe463p*RS356pe + 1.
         /162.*pow(m33,2)*RSe561p*RS346pe - 1./162.*pow(m33,2)*RSe561p*
         RS356pe - 1./162.*pow(m33,2)*RSe461p*RS346pe + 1./162.*pow(
         m33,2)*RSe461p*RS356pe - 1./162.*m13*m55*RSe563p*RS346pe - 1./
         162.*m13*m55*RSe563p*RS356pe - 1./54.*m13*m55*RSe463p*RS346pe
          - 1./162.*m13*m55*RSe463p*RS356pe + 1./162.*m13*m55*RSe561p*
         RS346pe + 1./162.*m13*m55*RSe561p*RS356pe + 1./54.*m13*m55*
         RSe461p*RS346pe + 1./162.*m13*m55*RSe461p*RS356pe - 1./81.*m13
         *m45*RSe563p*RS346pe + 1./81.*m13*m45*RSe563p*RS356pe + 1./81.
         *m13*m45*RSe463p*RS346pe - 1./81.*m13*m45*RSe463p*RS356pe + 1./
         81.*m13*m45*RSe561p*RS346pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,me2,L) * (  - 1./81.*m13*m45
         *RSe561p*RS356pe - 1./81.*m13*m45*RSe461p*RS346pe + 1./81.*m13
         *m45*RSe461p*RS356pe - 1./162.*m13*m44*RSe563p*RS346pe - 1./54.
         *m13*m44*RSe563p*RS356pe - 1./162.*m13*m44*RSe463p*RS346pe - 1.
         /162.*m13*m44*RSe463p*RS356pe + 1./162.*m13*m44*RSe561p*
         RS346pe + 1./54.*m13*m44*RSe561p*RS356pe + 1./162.*m13*m44*
         RSe461p*RS346pe + 1./162.*m13*m44*RSe461p*RS356pe - 1./81.*m13
         *m33*RSe563p*RS346pe + 1./81.*m13*m33*RSe563p*RS356pe + 1./81.
         *m13*m33*RSe463p*RS346pe - 1./81.*m13*m33*RSe463p*RS356pe + 1./
         81.*m13*m33*RSe561p*RS346pe - 1./81.*m13*m33*RSe561p*RS356pe
          - 1./81.*m13*m33*RSe461p*RS346pe + 1./81.*m13*m33*RSe461p*
         RS356pe + 1./324.*m11*m55*RSe563p*RS346pe + 1./324.*m11*m55*
         RSe563p*RS356pe + 1./108.*m11*m55*RSe463p*RS346pe + 1./324.*
         m11*m55*RSe463p*RS356pe + 1./324.*m11*m55*RSe561p*RS346pe + 1./
         324.*m11*m55*RSe561p*RS356pe + 1./108.*m11*m55*RSe461p*RS346pe
          + 1./324.*m11*m55*RSe461p*RS356pe );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,me2,L) * ( 1./162.*m11*m45*
         RSe563p*RS346pe - 1./162.*m11*m45*RSe563p*RS356pe - 1./162.*
         m11*m45*RSe463p*RS346pe + 1./162.*m11*m45*RSe463p*RS356pe + 1./
         162.*m11*m45*RSe561p*RS346pe - 1./162.*m11*m45*RSe561p*RS356pe
          - 1./162.*m11*m45*RSe461p*RS346pe + 1./162.*m11*m45*RSe461p*
         RS356pe + 1./324.*m11*m44*RSe563p*RS346pe + 1./108.*m11*m44*
         RSe563p*RS356pe + 1./324.*m11*m44*RSe463p*RS346pe + 1./324.*
         m11*m44*RSe463p*RS356pe + 1./324.*m11*m44*RSe561p*RS346pe + 1./
         108.*m11*m44*RSe561p*RS356pe + 1./324.*m11*m44*RSe461p*RS346pe
          + 1./324.*m11*m44*RSe461p*RS356pe + 1./162.*m11*m33*RSe563p*
         RS346pe - 1./162.*m11*m33*RSe563p*RS356pe - 1./162.*m11*m33*
         RSe463p*RS346pe + 1./162.*m11*m33*RSe463p*RS356pe + 1./162.*
         m11*m33*RSe561p*RS346pe - 1./162.*m11*m33*RSe561p*RS356pe - 1./
         162.*m11*m33*RSe461p*RS346pe + 1./162.*m11*m33*RSe461p*RS356pe
          );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,L) * ( 1./216.*m33*m55*pow(
         RS346pe,2) + 1./324.*m33*m55*RS356pe*RS346pe + 1./648.*m33*m55
         *pow(RS356pe,2) - 1./324.*m33*m45*pow(RS346pe,2) + 1./162.*m33
         *m45*RS356pe*RS346pe - 1./324.*m33*m45*pow(RS356pe,2) + 1./648.
         *m33*m44*pow(RS346pe,2) + 1./324.*m33*m44*RS356pe*RS346pe + 1./
         216.*m33*m44*pow(RS356pe,2) - 1./324.*pow(m33,2)*pow(
         RS346pe,2) + 1./162.*pow(m33,2)*RS356pe*RS346pe - 1./324.*pow(
         m33,2)*pow(RS356pe,2) - 1./108.*m13*m55*pow(RS346pe,2) - 1./
         162.*m13*m55*RS356pe*RS346pe - 1./324.*m13*m55*pow(RS356pe,2)
          + 1./162.*m13*m45*pow(RS346pe,2) - 1./81.*m13*m45*RS356pe*
         RS346pe + 1./162.*m13*m45*pow(RS356pe,2) - 1./324.*m13*m44*
         pow(RS346pe,2) - 1./162.*m13*m44*RS356pe*RS346pe - 1./108.*m13
         *m44*pow(RS356pe,2) + 1./162.*m13*m33*pow(RS346pe,2) - 1./81.*
         m13*m33*RS356pe*RS346pe + 1./162.*m13*m33*pow(RS356pe,2) + 1./
         216.*m11*m55*pow(RS346pe,2) + 1./324.*m11*m55*RS356pe*RS346pe
          + 1./648.*m11*m55*pow(RS356pe,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,L) * (  - 1./324.*m11*m45*
         pow(RS346pe,2) + 1./162.*m11*m45*RS356pe*RS346pe - 1./324.*m11
         *m45*pow(RS356pe,2) + 1./648.*m11*m44*pow(RS346pe,2) + 1./324.
         *m11*m44*RS356pe*RS346pe + 1./216.*m11*m44*pow(RS356pe,2) - 1./
         324.*m11*m33*pow(RS346pe,2) + 1./162.*m11*m33*RS356pe*RS346pe
          - 1./324.*m11*m33*pow(RS356pe,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m34,L) * ( 1./144.*m44*m55 - 1./
         72.*m44*m45 + 1./72.*pow(m44,2) - 1./72.*m34*m55 + 1./36.*m34*
         m45 - 1./24.*m34*m44 + 1./36.*pow(m34,2) + 1./72.*m33*m55 - 1./
         36.*m33*m45 + 5./144.*m33*m44 - 1./24.*m33*m34 + 1./72.*pow(
         m33,2) - 1./72.*m13*m55 + 1./36.*m13*m45 - 1./36.*m13*m44 + 1./
         36.*m13*m34 - 1./72.*m13*m33 + 1./144.*m11*m55 - 1./72.*m11*
         m45 + 1./72.*m11*m44 - 1./72.*m11*m34 + 1./144.*m11*m33 );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(m35,L) * ( 1./72.*pow(m55,2) - 1.
         /72.*m45*m55 + 1./144.*m44*m55 - 1./24.*m35*m55 + 1./36.*m35*
         m45 - 1./72.*m35*m44 + 1./36.*pow(m35,2) + 5./144.*m33*m55 - 1.
         /36.*m33*m45 + 1./72.*m33*m44 - 1./24.*m33*m35 + 1./72.*pow(
         m33,2) - 1./36.*m13*m55 + 1./36.*m13*m45 - 1./72.*m13*m44 + 1./
         36.*m13*m35 - 1./72.*m13*m33 + 1./72.*m11*m55 - 1./72.*m11*m45
          + 1./144.*m11*m44 - 1./72.*m11*m35 + 1./144.*m11*m33 );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(mp2,L) * ( 1./162.*pow(mp2,2)*
         pow(RSp563e,2) - 1./81.*pow(mp2,2)*RSp463e*RSp563e + 1./162.*
         pow(mp2,2)*pow(RSp463e,2) - 1./81.*pow(mp2,2)*RSp561e*RSp563e
          + 1./81.*pow(mp2,2)*RSp561e*RSp463e + 1./162.*pow(mp2,2)*pow(
         RSp561e,2) + 1./81.*pow(mp2,2)*RSp461e*RSp563e - 1./81.*pow(
         mp2,2)*RSp461e*RSp463e - 1./81.*pow(mp2,2)*RSp461e*RSp561e + 1.
         /162.*pow(mp2,2)*pow(RSp461e,2) - 1./324.*m55*mp2*pow(
         RSp563e,2) - 1./162.*m55*mp2*RSp463e*RSp563e - 1./108.*m55*mp2
         *pow(RSp463e,2) + 1./162.*m55*mp2*RSp561e*RSp563e + 1./162.*
         m55*mp2*RSp561e*RSp463e - 1./324.*m55*mp2*pow(RSp561e,2) + 1./
         162.*m55*mp2*RSp461e*RSp563e + 1./54.*m55*mp2*RSp461e*RSp463e
          - 1./162.*m55*mp2*RSp461e*RSp561e - 1./108.*m55*mp2*pow(
         RSp461e,2) + 1./162.*m45*mp2*pow(RSp563e,2) - 1./81.*m45*mp2*
         RSp463e*RSp563e + 1./162.*m45*mp2*pow(RSp463e,2) - 1./81.*m45*
         mp2*RSp561e*RSp563e + 1./81.*m45*mp2*RSp561e*RSp463e + 1./162.
         *m45*mp2*pow(RSp561e,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(mp2,L) * ( 1./81.*m45*mp2*
         RSp461e*RSp563e - 1./81.*m45*mp2*RSp461e*RSp463e - 1./81.*m45*
         mp2*RSp461e*RSp561e + 1./162.*m45*mp2*pow(RSp461e,2) - 1./108.
         *m44*mp2*pow(RSp563e,2) - 1./162.*m44*mp2*RSp463e*RSp563e - 1./
         324.*m44*mp2*pow(RSp463e,2) + 1./54.*m44*mp2*RSp561e*RSp563e
          + 1./162.*m44*mp2*RSp561e*RSp463e - 1./108.*m44*mp2*pow(
         RSp561e,2) + 1./162.*m44*mp2*RSp461e*RSp563e + 1./162.*m44*mp2
         *RSp461e*RSp463e - 1./162.*m44*mp2*RSp461e*RSp561e - 1./324.*
         m44*mp2*pow(RSp461e,2) - 1./108.*m33*mp2*pow(RSp563e,2) + 1./
         54.*m33*mp2*RSp463e*RSp563e - 1./108.*m33*mp2*pow(RSp463e,2)
          - 1./162.*m33*mp2*RSp561e*RSp563e + 1./162.*m33*mp2*RSp561e*
         RSp463e - 1./324.*m33*mp2*pow(RSp561e,2) + 1./162.*m33*mp2*
         RSp461e*RSp563e - 1./162.*m33*mp2*RSp461e*RSp463e + 1./162.*
         m33*mp2*RSp461e*RSp561e - 1./324.*m33*mp2*pow(RSp461e,2) + 1./
         216.*m33*m55*pow(RSp563e,2) + 1./108.*m33*m55*RSp463e*RSp563e
          + 1./72.*m33*m55*pow(RSp463e,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(mp2,L) * ( 1./324.*m33*m55*
         RSp561e*RSp563e + 1./324.*m33*m55*RSp561e*RSp463e + 1./648.*
         m33*m55*pow(RSp561e,2) + 1./324.*m33*m55*RSp461e*RSp563e + 1./
         108.*m33*m55*RSp461e*RSp463e + 1./324.*m33*m55*RSp461e*RSp561e
          + 1./216.*m33*m55*pow(RSp461e,2) - 1./108.*m33*m45*pow(
         RSp563e,2) + 1./54.*m33*m45*RSp463e*RSp563e - 1./108.*m33*m45*
         pow(RSp463e,2) - 1./162.*m33*m45*RSp561e*RSp563e + 1./162.*m33
         *m45*RSp561e*RSp463e - 1./324.*m33*m45*pow(RSp561e,2) + 1./162.
         *m33*m45*RSp461e*RSp563e - 1./162.*m33*m45*RSp461e*RSp463e + 1.
         /162.*m33*m45*RSp461e*RSp561e - 1./324.*m33*m45*pow(RSp461e,2)
          + 1./72.*m33*m44*pow(RSp563e,2) + 1./108.*m33*m44*RSp463e*
         RSp563e + 1./216.*m33*m44*pow(RSp463e,2) + 1./108.*m33*m44*
         RSp561e*RSp563e + 1./324.*m33*m44*RSp561e*RSp463e + 1./216.*
         m33*m44*pow(RSp561e,2) + 1./324.*m33*m44*RSp461e*RSp563e + 1./
         324.*m33*m44*RSp461e*RSp463e + 1./324.*m33*m44*RSp461e*RSp561e
          + 1./648.*m33*m44*pow(RSp461e,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(mp2,L) * ( 1./162.*m13*mp2*pow(
         RSp563e,2) - 1./81.*m13*mp2*RSp463e*RSp563e + 1./162.*m13*mp2*
         pow(RSp463e,2) - 1./81.*m13*mp2*RSp561e*RSp563e + 1./81.*m13*
         mp2*RSp561e*RSp463e + 1./162.*m13*mp2*pow(RSp561e,2) + 1./81.*
         m13*mp2*RSp461e*RSp563e - 1./81.*m13*mp2*RSp461e*RSp463e - 1./
         81.*m13*mp2*RSp461e*RSp561e + 1./162.*m13*mp2*pow(RSp461e,2)
          - 1./324.*m13*m55*pow(RSp563e,2) - 1./162.*m13*m55*RSp463e*
         RSp563e - 1./108.*m13*m55*pow(RSp463e,2) + 1./162.*m13*m55*
         RSp561e*RSp563e + 1./162.*m13*m55*RSp561e*RSp463e - 1./324.*
         m13*m55*pow(RSp561e,2) + 1./162.*m13*m55*RSp461e*RSp563e + 1./
         54.*m13*m55*RSp461e*RSp463e - 1./162.*m13*m55*RSp461e*RSp561e
          - 1./108.*m13*m55*pow(RSp461e,2) + 1./162.*m13*m45*pow(
         RSp563e,2) - 1./81.*m13*m45*RSp463e*RSp563e + 1./162.*m13*m45*
         pow(RSp463e,2) - 1./81.*m13*m45*RSp561e*RSp563e + 1./81.*m13*
         m45*RSp561e*RSp463e + 1./162.*m13*m45*pow(RSp561e,2) + 1./81.*
         m13*m45*RSp461e*RSp563e );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(mp2,L) * (  - 1./81.*m13*m45*
         RSp461e*RSp463e - 1./81.*m13*m45*RSp461e*RSp561e + 1./162.*m13
         *m45*pow(RSp461e,2) - 1./108.*m13*m44*pow(RSp563e,2) - 1./162.
         *m13*m44*RSp463e*RSp563e - 1./324.*m13*m44*pow(RSp463e,2) + 1./
         54.*m13*m44*RSp561e*RSp563e + 1./162.*m13*m44*RSp561e*RSp463e
          - 1./108.*m13*m44*pow(RSp561e,2) + 1./162.*m13*m44*RSp461e*
         RSp563e + 1./162.*m13*m44*RSp461e*RSp463e - 1./162.*m13*m44*
         RSp461e*RSp561e - 1./324.*m13*m44*pow(RSp461e,2) - 1./324.*m11
         *mp2*pow(RSp563e,2) + 1./162.*m11*mp2*RSp463e*RSp563e - 1./324.
         *m11*mp2*pow(RSp463e,2) - 1./162.*m11*mp2*RSp561e*RSp563e + 1./
         162.*m11*mp2*RSp561e*RSp463e - 1./108.*m11*mp2*pow(RSp561e,2)
          + 1./162.*m11*mp2*RSp461e*RSp563e - 1./162.*m11*mp2*RSp461e*
         RSp463e + 1./54.*m11*mp2*RSp461e*RSp561e - 1./108.*m11*mp2*
         pow(RSp461e,2) + 1./648.*m11*m55*pow(RSp563e,2) + 1./324.*m11*
         m55*RSp463e*RSp563e + 1./216.*m11*m55*pow(RSp463e,2) + 1./324.
         *m11*m55*RSp561e*RSp563e );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(mp2,L) * ( 1./324.*m11*m55*
         RSp561e*RSp463e + 1./216.*m11*m55*pow(RSp561e,2) + 1./324.*m11
         *m55*RSp461e*RSp563e + 1./108.*m11*m55*RSp461e*RSp463e + 1./
         108.*m11*m55*RSp461e*RSp561e + 1./72.*m11*m55*pow(RSp461e,2)
          - 1./324.*m11*m45*pow(RSp563e,2) + 1./162.*m11*m45*RSp463e*
         RSp563e - 1./324.*m11*m45*pow(RSp463e,2) - 1./162.*m11*m45*
         RSp561e*RSp563e + 1./162.*m11*m45*RSp561e*RSp463e - 1./108.*
         m11*m45*pow(RSp561e,2) + 1./162.*m11*m45*RSp461e*RSp563e - 1./
         162.*m11*m45*RSp461e*RSp463e + 1./54.*m11*m45*RSp461e*RSp561e
          - 1./108.*m11*m45*pow(RSp461e,2) + 1./216.*m11*m44*pow(
         RSp563e,2) + 1./324.*m11*m44*RSp463e*RSp563e + 1./648.*m11*m44
         *pow(RSp463e,2) + 1./108.*m11*m44*RSp561e*RSp563e + 1./324.*
         m11*m44*RSp561e*RSp463e + 1./72.*m11*m44*pow(RSp561e,2) + 1./
         324.*m11*m44*RSp461e*RSp563e + 1./324.*m11*m44*RSp461e*RSp463e
          + 1./108.*m11*m44*RSp461e*RSp561e + 1./216.*m11*m44*pow(
         RSp461e,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * ( 1./81.*mp2*me2*
         RSp563e*RSe563p - 1./81.*mp2*me2*RSp563e*RSe463p - 1./81.*mp2*
         me2*RSp463e*RSe563p + 1./81.*mp2*me2*RSp463e*RSe463p - 1./81.*
         mp2*me2*RSe561p*RSp563e + 1./81.*mp2*me2*RSe561p*RSp463e + 1./
         81.*mp2*me2*RSe461p*RSp563e - 1./81.*mp2*me2*RSe461p*RSp463e
          - 1./81.*mp2*me2*RSp561e*RSe563p + 1./81.*mp2*me2*RSp561e*
         RSe463p + 1./81.*mp2*me2*RSp561e*RSe561p - 1./81.*mp2*me2*
         RSp561e*RSe461p + 1./81.*mp2*me2*RSp461e*RSe563p - 1./81.*mp2*
         me2*RSp461e*RSe463p - 1./81.*mp2*me2*RSp461e*RSe561p + 1./81.*
         mp2*me2*RSp461e*RSe461p - 1./162.*m55*me2*RSp563e*RSe563p - 1./
         162.*m55*me2*RSp563e*RSe463p - 1./162.*m55*me2*RSp463e*RSe563p
          - 1./54.*m55*me2*RSp463e*RSe463p + 1./162.*m55*me2*RSe561p*
         RSp563e + 1./162.*m55*me2*RSe561p*RSp463e + 1./162.*m55*me2*
         RSe461p*RSp563e + 1./54.*m55*me2*RSe461p*RSp463e + 1./162.*m55
         *me2*RSp561e*RSe563p + 1./162.*m55*me2*RSp561e*RSe463p - 1./
         162.*m55*me2*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * (  - 1./162.*m55*
         me2*RSp561e*RSe461p + 1./162.*m55*me2*RSp461e*RSe563p + 1./54.
         *m55*me2*RSp461e*RSe463p - 1./162.*m55*me2*RSp461e*RSe561p - 1.
         /54.*m55*me2*RSp461e*RSe461p + 1./81.*m45*me2*RSp563e*RSe563p
          - 1./81.*m45*me2*RSp563e*RSe463p - 1./81.*m45*me2*RSp463e*
         RSe563p + 1./81.*m45*me2*RSp463e*RSe463p - 1./81.*m45*me2*
         RSe561p*RSp563e + 1./81.*m45*me2*RSe561p*RSp463e + 1./81.*m45*
         me2*RSe461p*RSp563e - 1./81.*m45*me2*RSe461p*RSp463e - 1./81.*
         m45*me2*RSp561e*RSe563p + 1./81.*m45*me2*RSp561e*RSe463p + 1./
         81.*m45*me2*RSp561e*RSe561p - 1./81.*m45*me2*RSp561e*RSe461p
          + 1./81.*m45*me2*RSp461e*RSe563p - 1./81.*m45*me2*RSp461e*
         RSe463p - 1./81.*m45*me2*RSp461e*RSe561p + 1./81.*m45*me2*
         RSp461e*RSe461p - 1./54.*m44*me2*RSp563e*RSe563p - 1./162.*m44
         *me2*RSp563e*RSe463p - 1./162.*m44*me2*RSp463e*RSe563p - 1./
         162.*m44*me2*RSp463e*RSe463p + 1./54.*m44*me2*RSe561p*RSp563e
          + 1./162.*m44*me2*RSe561p*RSp463e );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * ( 1./162.*m44*me2*
         RSe461p*RSp563e + 1./162.*m44*me2*RSe461p*RSp463e + 1./54.*m44
         *me2*RSp561e*RSe563p + 1./162.*m44*me2*RSp561e*RSe463p - 1./54.
         *m44*me2*RSp561e*RSe561p - 1./162.*m44*me2*RSp561e*RSe461p + 1.
         /162.*m44*me2*RSp461e*RSe563p + 1./162.*m44*me2*RSp461e*
         RSe463p - 1./162.*m44*me2*RSp461e*RSe561p - 1./162.*m44*me2*
         RSp461e*RSe461p - 1./54.*m33*me2*RSp563e*RSe563p + 1./54.*m33*
         me2*RSp563e*RSe463p + 1./54.*m33*me2*RSp463e*RSe563p - 1./54.*
         m33*me2*RSp463e*RSe463p - 1./162.*m33*me2*RSe561p*RSp563e + 1./
         162.*m33*me2*RSe561p*RSp463e + 1./162.*m33*me2*RSe461p*RSp563e
          - 1./162.*m33*me2*RSe461p*RSp463e - 1./162.*m33*me2*RSp561e*
         RSe563p + 1./162.*m33*me2*RSp561e*RSe463p - 1./162.*m33*me2*
         RSp561e*RSe561p + 1./162.*m33*me2*RSp561e*RSe461p + 1./162.*
         m33*me2*RSp461e*RSe563p - 1./162.*m33*me2*RSp461e*RSe463p + 1./
         162.*m33*me2*RSp461e*RSe561p - 1./162.*m33*me2*RSp461e*RSe461p
          + 1./108.*m33*m55*RSp563e*RSe563p );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * ( 1./108.*m33*m55*
         RSp563e*RSe463p + 1./108.*m33*m55*RSp463e*RSe563p + 1./36.*m33
         *m55*RSp463e*RSe463p + 1./324.*m33*m55*RSe561p*RSp563e + 1./
         324.*m33*m55*RSe561p*RSp463e + 1./324.*m33*m55*RSe461p*RSp563e
          + 1./108.*m33*m55*RSe461p*RSp463e + 1./324.*m33*m55*RSp561e*
         RSe563p + 1./324.*m33*m55*RSp561e*RSe463p + 1./324.*m33*m55*
         RSp561e*RSe561p + 1./324.*m33*m55*RSp561e*RSe461p + 1./324.*
         m33*m55*RSp461e*RSe563p + 1./108.*m33*m55*RSp461e*RSe463p + 1./
         324.*m33*m55*RSp461e*RSe561p + 1./108.*m33*m55*RSp461e*RSe461p
          - 1./54.*m33*m45*RSp563e*RSe563p + 1./54.*m33*m45*RSp563e*
         RSe463p + 1./54.*m33*m45*RSp463e*RSe563p - 1./54.*m33*m45*
         RSp463e*RSe463p - 1./162.*m33*m45*RSe561p*RSp563e + 1./162.*
         m33*m45*RSe561p*RSp463e + 1./162.*m33*m45*RSe461p*RSp563e - 1./
         162.*m33*m45*RSe461p*RSp463e - 1./162.*m33*m45*RSp561e*RSe563p
          + 1./162.*m33*m45*RSp561e*RSe463p - 1./162.*m33*m45*RSp561e*
         RSe561p );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * ( 1./162.*m33*m45*
         RSp561e*RSe461p + 1./162.*m33*m45*RSp461e*RSe563p - 1./162.*
         m33*m45*RSp461e*RSe463p + 1./162.*m33*m45*RSp461e*RSe561p - 1./
         162.*m33*m45*RSp461e*RSe461p + 1./36.*m33*m44*RSp563e*RSe563p
          + 1./108.*m33*m44*RSp563e*RSe463p + 1./108.*m33*m44*RSp463e*
         RSe563p + 1./108.*m33*m44*RSp463e*RSe463p + 1./108.*m33*m44*
         RSe561p*RSp563e + 1./324.*m33*m44*RSe561p*RSp463e + 1./324.*
         m33*m44*RSe461p*RSp563e + 1./324.*m33*m44*RSe461p*RSp463e + 1./
         108.*m33*m44*RSp561e*RSe563p + 1./324.*m33*m44*RSp561e*RSe463p
          + 1./108.*m33*m44*RSp561e*RSe561p + 1./324.*m33*m44*RSp561e*
         RSe461p + 1./324.*m33*m44*RSp461e*RSe563p + 1./324.*m33*m44*
         RSp461e*RSe463p + 1./324.*m33*m44*RSp461e*RSe561p + 1./324.*
         m33*m44*RSp461e*RSe461p + 1./81.*m13*me2*RSp563e*RSe563p - 1./
         81.*m13*me2*RSp563e*RSe463p - 1./81.*m13*me2*RSp463e*RSe563p
          + 1./81.*m13*me2*RSp463e*RSe463p - 1./81.*m13*me2*RSe561p*
         RSp563e );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * ( 1./81.*m13*me2*
         RSe561p*RSp463e + 1./81.*m13*me2*RSe461p*RSp563e - 1./81.*m13*
         me2*RSe461p*RSp463e - 1./81.*m13*me2*RSp561e*RSe563p + 1./81.*
         m13*me2*RSp561e*RSe463p + 1./81.*m13*me2*RSp561e*RSe561p - 1./
         81.*m13*me2*RSp561e*RSe461p + 1./81.*m13*me2*RSp461e*RSe563p
          - 1./81.*m13*me2*RSp461e*RSe463p - 1./81.*m13*me2*RSp461e*
         RSe561p + 1./81.*m13*me2*RSp461e*RSe461p - 1./162.*m13*m55*
         RSp563e*RSe563p - 1./162.*m13*m55*RSp563e*RSe463p - 1./162.*
         m13*m55*RSp463e*RSe563p - 1./54.*m13*m55*RSp463e*RSe463p + 1./
         162.*m13*m55*RSe561p*RSp563e + 1./162.*m13*m55*RSe561p*RSp463e
          + 1./162.*m13*m55*RSe461p*RSp563e + 1./54.*m13*m55*RSe461p*
         RSp463e + 1./162.*m13*m55*RSp561e*RSe563p + 1./162.*m13*m55*
         RSp561e*RSe463p - 1./162.*m13*m55*RSp561e*RSe561p - 1./162.*
         m13*m55*RSp561e*RSe461p + 1./162.*m13*m55*RSp461e*RSe563p + 1./
         54.*m13*m55*RSp461e*RSe463p - 1./162.*m13*m55*RSp461e*RSe561p
          - 1./54.*m13*m55*RSp461e*RSe461p );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * ( 1./81.*m13*m45*
         RSp563e*RSe563p - 1./81.*m13*m45*RSp563e*RSe463p - 1./81.*m13*
         m45*RSp463e*RSe563p + 1./81.*m13*m45*RSp463e*RSe463p - 1./81.*
         m13*m45*RSe561p*RSp563e + 1./81.*m13*m45*RSe561p*RSp463e + 1./
         81.*m13*m45*RSe461p*RSp563e - 1./81.*m13*m45*RSe461p*RSp463e
          - 1./81.*m13*m45*RSp561e*RSe563p + 1./81.*m13*m45*RSp561e*
         RSe463p + 1./81.*m13*m45*RSp561e*RSe561p - 1./81.*m13*m45*
         RSp561e*RSe461p + 1./81.*m13*m45*RSp461e*RSe563p - 1./81.*m13*
         m45*RSp461e*RSe463p - 1./81.*m13*m45*RSp461e*RSe561p + 1./81.*
         m13*m45*RSp461e*RSe461p - 1./54.*m13*m44*RSp563e*RSe563p - 1./
         162.*m13*m44*RSp563e*RSe463p - 1./162.*m13*m44*RSp463e*RSe563p
          - 1./162.*m13*m44*RSp463e*RSe463p + 1./54.*m13*m44*RSe561p*
         RSp563e + 1./162.*m13*m44*RSe561p*RSp463e + 1./162.*m13*m44*
         RSe461p*RSp563e + 1./162.*m13*m44*RSe461p*RSp463e + 1./54.*m13
         *m44*RSp561e*RSe563p + 1./162.*m13*m44*RSp561e*RSe463p - 1./54.
         *m13*m44*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * (  - 1./162.*m13*
         m44*RSp561e*RSe461p + 1./162.*m13*m44*RSp461e*RSe563p + 1./162.
         *m13*m44*RSp461e*RSe463p - 1./162.*m13*m44*RSp461e*RSe561p - 1.
         /162.*m13*m44*RSp461e*RSe461p - 1./162.*m11*me2*RSp563e*
         RSe563p + 1./162.*m11*me2*RSp563e*RSe463p + 1./162.*m11*me2*
         RSp463e*RSe563p - 1./162.*m11*me2*RSp463e*RSe463p - 1./162.*
         m11*me2*RSe561p*RSp563e + 1./162.*m11*me2*RSe561p*RSp463e + 1./
         162.*m11*me2*RSe461p*RSp563e - 1./162.*m11*me2*RSe461p*RSp463e
          - 1./162.*m11*me2*RSp561e*RSe563p + 1./162.*m11*me2*RSp561e*
         RSe463p - 1./54.*m11*me2*RSp561e*RSe561p + 1./54.*m11*me2*
         RSp561e*RSe461p + 1./162.*m11*me2*RSp461e*RSe563p - 1./162.*
         m11*me2*RSp461e*RSe463p + 1./54.*m11*me2*RSp461e*RSe561p - 1./
         54.*m11*me2*RSp461e*RSe461p + 1./324.*m11*m55*RSp563e*RSe563p
          + 1./324.*m11*m55*RSp563e*RSe463p + 1./324.*m11*m55*RSp463e*
         RSe563p + 1./108.*m11*m55*RSp463e*RSe463p + 1./324.*m11*m55*
         RSe561p*RSp563e );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * ( 1./324.*m11*m55*
         RSe561p*RSp463e + 1./324.*m11*m55*RSe461p*RSp563e + 1./108.*
         m11*m55*RSe461p*RSp463e + 1./324.*m11*m55*RSp561e*RSe563p + 1./
         324.*m11*m55*RSp561e*RSe463p + 1./108.*m11*m55*RSp561e*RSe561p
          + 1./108.*m11*m55*RSp561e*RSe461p + 1./324.*m11*m55*RSp461e*
         RSe563p + 1./108.*m11*m55*RSp461e*RSe463p + 1./108.*m11*m55*
         RSp461e*RSe561p + 1./36.*m11*m55*RSp461e*RSe461p - 1./162.*m11
         *m45*RSp563e*RSe563p + 1./162.*m11*m45*RSp563e*RSe463p + 1./
         162.*m11*m45*RSp463e*RSe563p - 1./162.*m11*m45*RSp463e*RSe463p
          - 1./162.*m11*m45*RSe561p*RSp563e + 1./162.*m11*m45*RSe561p*
         RSp463e + 1./162.*m11*m45*RSe461p*RSp563e - 1./162.*m11*m45*
         RSe461p*RSp463e - 1./162.*m11*m45*RSp561e*RSe563p + 1./162.*
         m11*m45*RSp561e*RSe463p - 1./54.*m11*m45*RSp561e*RSe561p + 1./
         54.*m11*m45*RSp561e*RSe461p + 1./162.*m11*m45*RSp461e*RSe563p
          - 1./162.*m11*m45*RSp461e*RSe463p + 1./54.*m11*m45*RSp461e*
         RSe561p );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * (  - 1./54.*m11*m45
         *RSp461e*RSe461p + 1./108.*m11*m44*RSp563e*RSe563p + 1./324.*
         m11*m44*RSp563e*RSe463p + 1./324.*m11*m44*RSp463e*RSe563p + 1./
         324.*m11*m44*RSp463e*RSe463p + 1./108.*m11*m44*RSe561p*RSp563e
          + 1./324.*m11*m44*RSe561p*RSp463e + 1./324.*m11*m44*RSe461p*
         RSp563e + 1./324.*m11*m44*RSe461p*RSp463e + 1./108.*m11*m44*
         RSp561e*RSe563p + 1./324.*m11*m44*RSp561e*RSe463p + 1./36.*m11
         *m44*RSp561e*RSe561p + 1./108.*m11*m44*RSp561e*RSe461p + 1./
         324.*m11*m44*RSp461e*RSe563p + 1./324.*m11*m44*RSp461e*RSe463p
          + 1./108.*m11*m44*RSp461e*RSe561p + 1./108.*m11*m44*RSp461e*
         RSe461p );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,L) * ( 1./162.*pow(me2,2)*
         pow(RSe563p,2) - 1./81.*pow(me2,2)*RSe463p*RSe563p + 1./162.*
         pow(me2,2)*pow(RSe463p,2) - 1./81.*pow(me2,2)*RSe561p*RSe563p
          + 1./81.*pow(me2,2)*RSe561p*RSe463p + 1./162.*pow(me2,2)*pow(
         RSe561p,2) + 1./81.*pow(me2,2)*RSe461p*RSe563p - 1./81.*pow(
         me2,2)*RSe461p*RSe463p - 1./81.*pow(me2,2)*RSe461p*RSe561p + 1.
         /162.*pow(me2,2)*pow(RSe461p,2) - 1./324.*m55*me2*pow(
         RSe563p,2) - 1./162.*m55*me2*RSe463p*RSe563p - 1./108.*m55*me2
         *pow(RSe463p,2) + 1./162.*m55*me2*RSe561p*RSe563p + 1./162.*
         m55*me2*RSe561p*RSe463p - 1./324.*m55*me2*pow(RSe561p,2) + 1./
         162.*m55*me2*RSe461p*RSe563p + 1./54.*m55*me2*RSe461p*RSe463p
          - 1./162.*m55*me2*RSe461p*RSe561p - 1./108.*m55*me2*pow(
         RSe461p,2) + 1./162.*m45*me2*pow(RSe563p,2) - 1./81.*m45*me2*
         RSe463p*RSe563p + 1./162.*m45*me2*pow(RSe463p,2) - 1./81.*m45*
         me2*RSe561p*RSe563p + 1./81.*m45*me2*RSe561p*RSe463p + 1./162.
         *m45*me2*pow(RSe561p,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,L) * ( 1./81.*m45*me2*
         RSe461p*RSe563p - 1./81.*m45*me2*RSe461p*RSe463p - 1./81.*m45*
         me2*RSe461p*RSe561p + 1./162.*m45*me2*pow(RSe461p,2) - 1./108.
         *m44*me2*pow(RSe563p,2) - 1./162.*m44*me2*RSe463p*RSe563p - 1./
         324.*m44*me2*pow(RSe463p,2) + 1./54.*m44*me2*RSe561p*RSe563p
          + 1./162.*m44*me2*RSe561p*RSe463p - 1./108.*m44*me2*pow(
         RSe561p,2) + 1./162.*m44*me2*RSe461p*RSe563p + 1./162.*m44*me2
         *RSe461p*RSe463p - 1./162.*m44*me2*RSe461p*RSe561p - 1./324.*
         m44*me2*pow(RSe461p,2) - 1./108.*m33*me2*pow(RSe563p,2) + 1./
         54.*m33*me2*RSe463p*RSe563p - 1./108.*m33*me2*pow(RSe463p,2)
          - 1./162.*m33*me2*RSe561p*RSe563p + 1./162.*m33*me2*RSe561p*
         RSe463p - 1./324.*m33*me2*pow(RSe561p,2) + 1./162.*m33*me2*
         RSe461p*RSe563p - 1./162.*m33*me2*RSe461p*RSe463p + 1./162.*
         m33*me2*RSe461p*RSe561p - 1./324.*m33*me2*pow(RSe461p,2) + 1./
         216.*m33*m55*pow(RSe563p,2) + 1./108.*m33*m55*RSe463p*RSe563p
          + 1./72.*m33*m55*pow(RSe463p,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,L) * ( 1./324.*m33*m55*
         RSe561p*RSe563p + 1./324.*m33*m55*RSe561p*RSe463p + 1./648.*
         m33*m55*pow(RSe561p,2) + 1./324.*m33*m55*RSe461p*RSe563p + 1./
         108.*m33*m55*RSe461p*RSe463p + 1./324.*m33*m55*RSe461p*RSe561p
          + 1./216.*m33*m55*pow(RSe461p,2) - 1./108.*m33*m45*pow(
         RSe563p,2) + 1./54.*m33*m45*RSe463p*RSe563p - 1./108.*m33*m45*
         pow(RSe463p,2) - 1./162.*m33*m45*RSe561p*RSe563p + 1./162.*m33
         *m45*RSe561p*RSe463p - 1./324.*m33*m45*pow(RSe561p,2) + 1./162.
         *m33*m45*RSe461p*RSe563p - 1./162.*m33*m45*RSe461p*RSe463p + 1.
         /162.*m33*m45*RSe461p*RSe561p - 1./324.*m33*m45*pow(RSe461p,2)
          + 1./72.*m33*m44*pow(RSe563p,2) + 1./108.*m33*m44*RSe463p*
         RSe563p + 1./216.*m33*m44*pow(RSe463p,2) + 1./108.*m33*m44*
         RSe561p*RSe563p + 1./324.*m33*m44*RSe561p*RSe463p + 1./216.*
         m33*m44*pow(RSe561p,2) + 1./324.*m33*m44*RSe461p*RSe563p + 1./
         324.*m33*m44*RSe461p*RSe463p + 1./324.*m33*m44*RSe461p*RSe561p
          + 1./648.*m33*m44*pow(RSe461p,2) );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,L) * ( 1./162.*m13*me2*pow(
         RSe563p,2) - 1./81.*m13*me2*RSe463p*RSe563p + 1./162.*m13*me2*
         pow(RSe463p,2) - 1./81.*m13*me2*RSe561p*RSe563p + 1./81.*m13*
         me2*RSe561p*RSe463p + 1./162.*m13*me2*pow(RSe561p,2) + 1./81.*
         m13*me2*RSe461p*RSe563p - 1./81.*m13*me2*RSe461p*RSe463p - 1./
         81.*m13*me2*RSe461p*RSe561p + 1./162.*m13*me2*pow(RSe461p,2)
          - 1./324.*m13*m55*pow(RSe563p,2) - 1./162.*m13*m55*RSe463p*
         RSe563p - 1./108.*m13*m55*pow(RSe463p,2) + 1./162.*m13*m55*
         RSe561p*RSe563p + 1./162.*m13*m55*RSe561p*RSe463p - 1./324.*
         m13*m55*pow(RSe561p,2) + 1./162.*m13*m55*RSe461p*RSe563p + 1./
         54.*m13*m55*RSe461p*RSe463p - 1./162.*m13*m55*RSe461p*RSe561p
          - 1./108.*m13*m55*pow(RSe461p,2) + 1./162.*m13*m45*pow(
         RSe563p,2) - 1./81.*m13*m45*RSe463p*RSe563p + 1./162.*m13*m45*
         pow(RSe463p,2) - 1./81.*m13*m45*RSe561p*RSe563p + 1./81.*m13*
         m45*RSe561p*RSe463p + 1./162.*m13*m45*pow(RSe561p,2) + 1./81.*
         m13*m45*RSe461p*RSe563p );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,L) * (  - 1./81.*m13*m45*
         RSe461p*RSe463p - 1./81.*m13*m45*RSe461p*RSe561p + 1./162.*m13
         *m45*pow(RSe461p,2) - 1./108.*m13*m44*pow(RSe563p,2) - 1./162.
         *m13*m44*RSe463p*RSe563p - 1./324.*m13*m44*pow(RSe463p,2) + 1./
         54.*m13*m44*RSe561p*RSe563p + 1./162.*m13*m44*RSe561p*RSe463p
          - 1./108.*m13*m44*pow(RSe561p,2) + 1./162.*m13*m44*RSe461p*
         RSe563p + 1./162.*m13*m44*RSe461p*RSe463p - 1./162.*m13*m44*
         RSe461p*RSe561p - 1./324.*m13*m44*pow(RSe461p,2) - 1./324.*m11
         *me2*pow(RSe563p,2) + 1./162.*m11*me2*RSe463p*RSe563p - 1./324.
         *m11*me2*pow(RSe463p,2) - 1./162.*m11*me2*RSe561p*RSe563p + 1./
         162.*m11*me2*RSe561p*RSe463p - 1./108.*m11*me2*pow(RSe561p,2)
          + 1./162.*m11*me2*RSe461p*RSe563p - 1./162.*m11*me2*RSe461p*
         RSe463p + 1./54.*m11*me2*RSe461p*RSe561p - 1./108.*m11*me2*
         pow(RSe461p,2) + 1./648.*m11*m55*pow(RSe563p,2) + 1./324.*m11*
         m55*RSe463p*RSe563p + 1./216.*m11*m55*pow(RSe463p,2) + 1./324.
         *m11*m55*RSe561p*RSe563p );

      M13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,L) * ( 1./324.*m11*m55*
         RSe561p*RSe463p + 1./216.*m11*m55*pow(RSe561p,2) + 1./324.*m11
         *m55*RSe461p*RSe563p + 1./108.*m11*m55*RSe461p*RSe463p + 1./
         108.*m11*m55*RSe461p*RSe561p + 1./72.*m11*m55*pow(RSe461p,2)
          - 1./324.*m11*m45*pow(RSe563p,2) + 1./162.*m11*m45*RSe463p*
         RSe563p - 1./324.*m11*m45*pow(RSe463p,2) - 1./162.*m11*m45*
         RSe561p*RSe563p + 1./162.*m11*m45*RSe561p*RSe463p - 1./108.*
         m11*m45*pow(RSe561p,2) + 1./162.*m11*m45*RSe461p*RSe563p - 1./
         162.*m11*m45*RSe461p*RSe463p + 1./54.*m11*m45*RSe461p*RSe561p
          - 1./108.*m11*m45*pow(RSe461p,2) + 1./216.*m11*m44*pow(
         RSe563p,2) + 1./324.*m11*m44*RSe463p*RSe563p + 1./648.*m11*m44
         *pow(RSe463p,2) + 1./108.*m11*m44*RSe561p*RSe563p + 1./324.*
         m11*m44*RSe561p*RSe463p + 1./72.*m11*m44*pow(RSe561p,2) + 1./
         324.*m11*m44*RSe461p*RSe563p + 1./324.*m11*m44*RSe461p*RSe463p
          + 1./108.*m11*m44*RSe461p*RSe561p + 1./216.*m11*m44*pow(
         RSe461p,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(m11,L) * (  - 1./108.*m66*pow(
         RS145pe,2) - 1./162.*m66*RS156pe*RS145pe - 1./324.*m66*pow(
         RS156pe,2) + 1./162.*m46*pow(RS145pe,2) - 1./81.*m46*RS156pe*
         RS145pe + 1./162.*m46*pow(RS156pe,2) - 1./324.*m44*pow(
         RS145pe,2) - 1./162.*m44*RS156pe*RS145pe - 1./108.*m44*pow(
         RS156pe,2) - 1./324.*m33*pow(RS145pe,2) + 1./162.*m33*RS156pe*
         RS145pe - 1./324.*m33*pow(RS156pe,2) + 1./162.*m13*pow(
         RS145pe,2) - 1./81.*m13*RS156pe*RS145pe + 1./162.*m13*pow(
         RS156pe,2) - 1./81.*m11*RS145pe*RS345pe + 1./81.*m11*RS145pe*
         RS356pe + 1./81.*m11*RS145pe*RSe563p - 1./81.*m11*RS145pe*
         RSe453p + 1./81.*m11*RS145pe*RSp563e - 1./81.*m11*RS145pe*
         RSp453e + 1./324.*m11*pow(RS145pe,2) + 1./81.*m11*RS156pe*
         RS345pe - 1./81.*m11*RS156pe*RS356pe - 1./81.*m11*RS156pe*
         RSe563p + 1./81.*m11*RS156pe*RSe453p - 1./81.*m11*RS156pe*
         RSp563e + 1./81.*m11*RS156pe*RSp453e - 1./162.*m11*RS156pe*
         RS145pe );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(m11,L) * ( 1./324.*m11*pow(
         RS156pe,2) - 1./81.*m11*RSe561p*RS145pe + 1./81.*m11*RSe561p*
         RS156pe + 1./81.*m11*RSe451p*RS145pe - 1./81.*m11*RSe451p*
         RS156pe - 1./81.*m11*RSp561e*RS145pe + 1./81.*m11*RSp561e*
         RS156pe + 1./81.*m11*RSp451e*RS145pe - 1./81.*m11*RSp451e*
         RS156pe );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(m14,L) * ( 7./360.*m66 - 1./15.*
         m46 - 1./360.*m44 + 1./36.*m36 + 7./360.*m33 + 13./180.*m14 - 
         1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(m16,L) * (  - 1./360.*m66 - 1./
         15.*m46 + 7./360.*m44 + 1./36.*m34 + 7./360.*m33 + 13./180.*
         m16 - 1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(m33,L) * (  - 1./108.*m66*pow(
         RS345pe,2) - 1./162.*m66*RS356pe*RS345pe - 1./324.*m66*pow(
         RS356pe,2) + 1./54.*m66*RS145pe*RS345pe + 1./162.*m66*RS145pe*
         RS356pe + 1./162.*m66*RS156pe*RS345pe + 1./162.*m66*RS156pe*
         RS356pe + 1./162.*m46*pow(RS345pe,2) - 1./81.*m46*RS356pe*
         RS345pe + 1./162.*m46*pow(RS356pe,2) - 1./81.*m46*RS145pe*
         RS345pe + 1./81.*m46*RS145pe*RS356pe + 1./81.*m46*RS156pe*
         RS345pe - 1./81.*m46*RS156pe*RS356pe - 1./324.*m44*pow(
         RS345pe,2) - 1./162.*m44*RS356pe*RS345pe - 1./108.*m44*pow(
         RS356pe,2) + 1./162.*m44*RS145pe*RS345pe + 1./162.*m44*RS145pe
         *RS356pe + 1./162.*m44*RS156pe*RS345pe + 1./54.*m44*RS156pe*
         RS356pe + 1./324.*m33*pow(RS345pe,2) - 1./162.*m33*RS356pe*
         RS345pe + 1./324.*m33*pow(RS356pe,2) - 1./81.*m33*RSe563p*
         RS345pe + 1./81.*m33*RSe563p*RS356pe + 1./81.*m33*RSe453p*
         RS345pe - 1./81.*m33*RSe453p*RS356pe - 1./81.*m33*RSp563e*
         RS345pe );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(m33,L) * ( 1./81.*m33*RSp563e*
         RS356pe + 1./81.*m33*RSp453e*RS345pe - 1./81.*m33*RSp453e*
         RS356pe - 1./54.*m33*RS145pe*RS345pe + 1./54.*m33*RS145pe*
         RS356pe + 1./54.*m33*RS156pe*RS345pe - 1./54.*m33*RS156pe*
         RS356pe + 1./81.*m33*RSe561p*RS345pe - 1./81.*m33*RSe561p*
         RS356pe - 1./81.*m33*RSe451p*RS345pe + 1./81.*m33*RSe451p*
         RS356pe + 1./81.*m33*RSp561e*RS345pe - 1./81.*m33*RSp561e*
         RS356pe - 1./81.*m33*RSp451e*RS345pe + 1./81.*m33*RSp451e*
         RS356pe + 1./162.*m13*pow(RS345pe,2) - 1./81.*m13*RS356pe*
         RS345pe + 1./162.*m13*pow(RS356pe,2) - 1./81.*m13*RS145pe*
         RS345pe + 1./81.*m13*RS145pe*RS356pe + 1./81.*m13*RS156pe*
         RS345pe - 1./81.*m13*RS156pe*RS356pe - 1./324.*m11*pow(
         RS345pe,2) + 1./162.*m11*RS356pe*RS345pe - 1./324.*m11*pow(
         RS356pe,2) - 1./162.*m11*RS145pe*RS345pe + 1./162.*m11*RS145pe
         *RS356pe + 1./162.*m11*RS156pe*RS345pe - 1./162.*m11*RS156pe*
         RS356pe );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(m34,L) * ( 7./360.*m66 - 1./15.*
         m46 - 1./360.*m44 + 13./180.*m34 - 1./360.*m33 + 1./36.*m16 - 
         1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(m36,L) * (  - 1./360.*m66 - 1./
         15.*m46 + 7./360.*m44 + 13./180.*m36 - 1./360.*m33 + 1./36.*
         m14 - 1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(mp2,L) * (  - 1./81.*mp2*RSp563e
         *RS345pe + 1./81.*mp2*RSp563e*RS356pe + 1./81.*mp2*RSp563e*
         RSe563p - 1./81.*mp2*RSp563e*RSe453p + 1./81.*mp2*pow(
         RSp563e,2) + 1./81.*mp2*RSp453e*RS345pe - 1./81.*mp2*RSp453e*
         RS356pe - 1./81.*mp2*RSp453e*RSe563p + 1./81.*mp2*RSp453e*
         RSe453p - 2./81.*mp2*RSp453e*RSp563e + 1./81.*mp2*pow(
         RSp453e,2) + 1./81.*mp2*RS145pe*RSp563e - 1./81.*mp2*RS145pe*
         RSp453e - 1./81.*mp2*RS156pe*RSp563e + 1./81.*mp2*RS156pe*
         RSp453e - 1./81.*mp2*RSe561p*RSp563e + 1./81.*mp2*RSe561p*
         RSp453e + 1./81.*mp2*RSe451p*RSp563e - 1./81.*mp2*RSe451p*
         RSp453e + 1./81.*mp2*RSp561e*RS345pe - 1./81.*mp2*RSp561e*
         RS356pe - 1./81.*mp2*RSp561e*RSe563p + 1./81.*mp2*RSp561e*
         RSe453p - 2./81.*mp2*RSp561e*RSp563e + 2./81.*mp2*RSp561e*
         RSp453e - 1./81.*mp2*RSp561e*RS145pe + 1./81.*mp2*RSp561e*
         RS156pe + 1./81.*mp2*RSp561e*RSe561p - 1./81.*mp2*RSp561e*
         RSe451p );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(mp2,L) * ( 1./81.*mp2*pow(
         RSp561e,2) - 1./81.*mp2*RSp451e*RS345pe + 1./81.*mp2*RSp451e*
         RS356pe + 1./81.*mp2*RSp451e*RSe563p - 1./81.*mp2*RSp451e*
         RSe453p + 2./81.*mp2*RSp451e*RSp563e - 2./81.*mp2*RSp451e*
         RSp453e + 1./81.*mp2*RSp451e*RS145pe - 1./81.*mp2*RSp451e*
         RS156pe - 1./81.*mp2*RSp451e*RSe561p + 1./81.*mp2*RSp451e*
         RSe451p - 2./81.*mp2*RSp451e*RSp561e + 1./81.*mp2*pow(
         RSp451e,2) - 1./162.*m66*RSp563e*RS345pe - 1./162.*m66*RSp563e
         *RS356pe - 1./162.*m66*RSp563e*RSe563p - 1./162.*m66*RSp563e*
         RSe453p - 1./324.*m66*pow(RSp563e,2) - 1./54.*m66*RSp453e*
         RS345pe - 1./162.*m66*RSp453e*RS356pe - 1./162.*m66*RSp453e*
         RSe563p - 1./54.*m66*RSp453e*RSe453p - 1./162.*m66*RSp453e*
         RSp563e - 1./108.*m66*pow(RSp453e,2) + 1./162.*m66*RS145pe*
         RSp563e + 1./54.*m66*RS145pe*RSp453e + 1./162.*m66*RS156pe*
         RSp563e + 1./162.*m66*RS156pe*RSp453e + 1./162.*m66*RSe561p*
         RSp563e );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(mp2,L) * ( 1./162.*m66*RSe561p*
         RSp453e + 1./162.*m66*RSe451p*RSp563e + 1./54.*m66*RSe451p*
         RSp453e + 1./162.*m66*RSp561e*RS345pe + 1./162.*m66*RSp561e*
         RS356pe + 1./162.*m66*RSp561e*RSe563p + 1./162.*m66*RSp561e*
         RSe453p + 1./162.*m66*RSp561e*RSp563e + 1./162.*m66*RSp561e*
         RSp453e - 1./162.*m66*RSp561e*RS145pe - 1./162.*m66*RSp561e*
         RS156pe - 1./162.*m66*RSp561e*RSe561p - 1./162.*m66*RSp561e*
         RSe451p - 1./324.*m66*pow(RSp561e,2) + 1./54.*m66*RSp451e*
         RS345pe + 1./162.*m66*RSp451e*RS356pe + 1./162.*m66*RSp451e*
         RSe563p + 1./54.*m66*RSp451e*RSe453p + 1./162.*m66*RSp451e*
         RSp563e + 1./54.*m66*RSp451e*RSp453e - 1./54.*m66*RSp451e*
         RS145pe - 1./162.*m66*RSp451e*RS156pe - 1./162.*m66*RSp451e*
         RSe561p - 1./54.*m66*RSp451e*RSe451p - 1./162.*m66*RSp451e*
         RSp561e - 1./108.*m66*pow(RSp451e,2) - 1./81.*m46*RSp563e*
         RS345pe + 1./81.*m46*RSp563e*RS356pe + 1./81.*m46*RSp563e*
         RSe563p );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(mp2,L) * (  - 1./81.*m46*RSp563e
         *RSe453p + 1./162.*m46*pow(RSp563e,2) + 1./81.*m46*RSp453e*
         RS345pe - 1./81.*m46*RSp453e*RS356pe - 1./81.*m46*RSp453e*
         RSe563p + 1./81.*m46*RSp453e*RSe453p - 1./81.*m46*RSp453e*
         RSp563e + 1./162.*m46*pow(RSp453e,2) + 1./81.*m46*RS145pe*
         RSp563e - 1./81.*m46*RS145pe*RSp453e - 1./81.*m46*RS156pe*
         RSp563e + 1./81.*m46*RS156pe*RSp453e - 1./81.*m46*RSe561p*
         RSp563e + 1./81.*m46*RSe561p*RSp453e + 1./81.*m46*RSe451p*
         RSp563e - 1./81.*m46*RSe451p*RSp453e + 1./81.*m46*RSp561e*
         RS345pe - 1./81.*m46*RSp561e*RS356pe - 1./81.*m46*RSp561e*
         RSe563p + 1./81.*m46*RSp561e*RSe453p - 1./81.*m46*RSp561e*
         RSp563e + 1./81.*m46*RSp561e*RSp453e - 1./81.*m46*RSp561e*
         RS145pe + 1./81.*m46*RSp561e*RS156pe + 1./81.*m46*RSp561e*
         RSe561p - 1./81.*m46*RSp561e*RSe451p + 1./162.*m46*pow(
         RSp561e,2) - 1./81.*m46*RSp451e*RS345pe + 1./81.*m46*RSp451e*
         RS356pe );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(mp2,L) * ( 1./81.*m46*RSp451e*
         RSe563p - 1./81.*m46*RSp451e*RSe453p + 1./81.*m46*RSp451e*
         RSp563e - 1./81.*m46*RSp451e*RSp453e + 1./81.*m46*RSp451e*
         RS145pe - 1./81.*m46*RSp451e*RS156pe - 1./81.*m46*RSp451e*
         RSe561p + 1./81.*m46*RSp451e*RSe451p - 1./81.*m46*RSp451e*
         RSp561e + 1./162.*m46*pow(RSp451e,2) - 1./162.*m44*RSp563e*
         RS345pe - 1./54.*m44*RSp563e*RS356pe - 1./54.*m44*RSp563e*
         RSe563p - 1./162.*m44*RSp563e*RSe453p - 1./108.*m44*pow(
         RSp563e,2) - 1./162.*m44*RSp453e*RS345pe - 1./162.*m44*RSp453e
         *RS356pe - 1./162.*m44*RSp453e*RSe563p - 1./162.*m44*RSp453e*
         RSe453p - 1./162.*m44*RSp453e*RSp563e - 1./324.*m44*pow(
         RSp453e,2) + 1./162.*m44*RS145pe*RSp563e + 1./162.*m44*RS145pe
         *RSp453e + 1./54.*m44*RS156pe*RSp563e + 1./162.*m44*RS156pe*
         RSp453e + 1./54.*m44*RSe561p*RSp563e + 1./162.*m44*RSe561p*
         RSp453e + 1./162.*m44*RSe451p*RSp563e + 1./162.*m44*RSe451p*
         RSp453e );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(mp2,L) * ( 1./162.*m44*RSp561e*
         RS345pe + 1./54.*m44*RSp561e*RS356pe + 1./54.*m44*RSp561e*
         RSe563p + 1./162.*m44*RSp561e*RSe453p + 1./54.*m44*RSp561e*
         RSp563e + 1./162.*m44*RSp561e*RSp453e - 1./162.*m44*RSp561e*
         RS145pe - 1./54.*m44*RSp561e*RS156pe - 1./54.*m44*RSp561e*
         RSe561p - 1./162.*m44*RSp561e*RSe451p - 1./108.*m44*pow(
         RSp561e,2) + 1./162.*m44*RSp451e*RS345pe + 1./162.*m44*RSp451e
         *RS356pe + 1./162.*m44*RSp451e*RSe563p + 1./162.*m44*RSp451e*
         RSe453p + 1./162.*m44*RSp451e*RSp563e + 1./162.*m44*RSp451e*
         RSp453e - 1./162.*m44*RSp451e*RS145pe - 1./162.*m44*RSp451e*
         RS156pe - 1./162.*m44*RSp451e*RSe561p - 1./162.*m44*RSp451e*
         RSe451p - 1./162.*m44*RSp451e*RSp561e - 1./324.*m44*pow(
         RSp451e,2) + 1./54.*m33*RSp563e*RS345pe - 1./54.*m33*RSp563e*
         RS356pe - 1./54.*m33*RSp563e*RSe563p + 1./54.*m33*RSp563e*
         RSe453p - 1./108.*m33*pow(RSp563e,2) - 1./54.*m33*RSp453e*
         RS345pe );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(mp2,L) * ( 1./54.*m33*RSp453e*
         RS356pe + 1./54.*m33*RSp453e*RSe563p - 1./54.*m33*RSp453e*
         RSe453p + 1./54.*m33*RSp453e*RSp563e - 1./108.*m33*pow(
         RSp453e,2) + 1./162.*m33*RS145pe*RSp563e - 1./162.*m33*RS145pe
         *RSp453e - 1./162.*m33*RS156pe*RSp563e + 1./162.*m33*RS156pe*
         RSp453e - 1./162.*m33*RSe561p*RSp563e + 1./162.*m33*RSe561p*
         RSp453e + 1./162.*m33*RSe451p*RSp563e - 1./162.*m33*RSe451p*
         RSp453e + 1./162.*m33*RSp561e*RS345pe - 1./162.*m33*RSp561e*
         RS356pe - 1./162.*m33*RSp561e*RSe563p + 1./162.*m33*RSp561e*
         RSe453p - 1./162.*m33*RSp561e*RSp563e + 1./162.*m33*RSp561e*
         RSp453e + 1./162.*m33*RSp561e*RS145pe - 1./162.*m33*RSp561e*
         RS156pe - 1./162.*m33*RSp561e*RSe561p + 1./162.*m33*RSp561e*
         RSe451p - 1./324.*m33*pow(RSp561e,2) - 1./162.*m33*RSp451e*
         RS345pe + 1./162.*m33*RSp451e*RS356pe + 1./162.*m33*RSp451e*
         RSe563p - 1./162.*m33*RSp451e*RSe453p + 1./162.*m33*RSp451e*
         RSp563e );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(mp2,L) * (  - 1./162.*m33*
         RSp451e*RSp453e - 1./162.*m33*RSp451e*RS145pe + 1./162.*m33*
         RSp451e*RS156pe + 1./162.*m33*RSp451e*RSe561p - 1./162.*m33*
         RSp451e*RSe451p + 1./162.*m33*RSp451e*RSp561e - 1./324.*m33*
         pow(RSp451e,2) - 1./81.*m13*RSp563e*RS345pe + 1./81.*m13*
         RSp563e*RS356pe + 1./81.*m13*RSp563e*RSe563p - 1./81.*m13*
         RSp563e*RSe453p + 1./162.*m13*pow(RSp563e,2) + 1./81.*m13*
         RSp453e*RS345pe - 1./81.*m13*RSp453e*RS356pe - 1./81.*m13*
         RSp453e*RSe563p + 1./81.*m13*RSp453e*RSe453p - 1./81.*m13*
         RSp453e*RSp563e + 1./162.*m13*pow(RSp453e,2) + 1./81.*m13*
         RS145pe*RSp563e - 1./81.*m13*RS145pe*RSp453e - 1./81.*m13*
         RS156pe*RSp563e + 1./81.*m13*RS156pe*RSp453e - 1./81.*m13*
         RSe561p*RSp563e + 1./81.*m13*RSe561p*RSp453e + 1./81.*m13*
         RSe451p*RSp563e - 1./81.*m13*RSe451p*RSp453e + 1./81.*m13*
         RSp561e*RS345pe - 1./81.*m13*RSp561e*RS356pe - 1./81.*m13*
         RSp561e*RSe563p );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(mp2,L) * ( 1./81.*m13*RSp561e*
         RSe453p - 1./81.*m13*RSp561e*RSp563e + 1./81.*m13*RSp561e*
         RSp453e - 1./81.*m13*RSp561e*RS145pe + 1./81.*m13*RSp561e*
         RS156pe + 1./81.*m13*RSp561e*RSe561p - 1./81.*m13*RSp561e*
         RSe451p + 1./162.*m13*pow(RSp561e,2) - 1./81.*m13*RSp451e*
         RS345pe + 1./81.*m13*RSp451e*RS356pe + 1./81.*m13*RSp451e*
         RSe563p - 1./81.*m13*RSp451e*RSe453p + 1./81.*m13*RSp451e*
         RSp563e - 1./81.*m13*RSp451e*RSp453e + 1./81.*m13*RSp451e*
         RS145pe - 1./81.*m13*RSp451e*RS156pe - 1./81.*m13*RSp451e*
         RSe561p + 1./81.*m13*RSp451e*RSe451p - 1./81.*m13*RSp451e*
         RSp561e + 1./162.*m13*pow(RSp451e,2) + 1./162.*m11*RSp563e*
         RS345pe - 1./162.*m11*RSp563e*RS356pe - 1./162.*m11*RSp563e*
         RSe563p + 1./162.*m11*RSp563e*RSe453p - 1./324.*m11*pow(
         RSp563e,2) - 1./162.*m11*RSp453e*RS345pe + 1./162.*m11*RSp453e
         *RS356pe + 1./162.*m11*RSp453e*RSe563p - 1./162.*m11*RSp453e*
         RSe453p );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(mp2,L) * ( 1./162.*m11*RSp453e*
         RSp563e - 1./324.*m11*pow(RSp453e,2) + 1./162.*m11*RS145pe*
         RSp563e - 1./162.*m11*RS145pe*RSp453e - 1./162.*m11*RS156pe*
         RSp563e + 1./162.*m11*RS156pe*RSp453e - 1./162.*m11*RSe561p*
         RSp563e + 1./162.*m11*RSe561p*RSp453e + 1./162.*m11*RSe451p*
         RSp563e - 1./162.*m11*RSe451p*RSp453e + 1./162.*m11*RSp561e*
         RS345pe - 1./162.*m11*RSp561e*RS356pe - 1./162.*m11*RSp561e*
         RSe563p + 1./162.*m11*RSp561e*RSe453p - 1./162.*m11*RSp561e*
         RSp563e + 1./162.*m11*RSp561e*RSp453e + 1./54.*m11*RSp561e*
         RS145pe - 1./54.*m11*RSp561e*RS156pe - 1./54.*m11*RSp561e*
         RSe561p + 1./54.*m11*RSp561e*RSe451p - 1./108.*m11*pow(
         RSp561e,2) - 1./162.*m11*RSp451e*RS345pe + 1./162.*m11*RSp451e
         *RS356pe + 1./162.*m11*RSp451e*RSe563p - 1./162.*m11*RSp451e*
         RSe453p + 1./162.*m11*RSp451e*RSp563e - 1./162.*m11*RSp451e*
         RSp453e - 1./54.*m11*RSp451e*RS145pe + 1./54.*m11*RSp451e*
         RS156pe );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(mp2,L) * ( 1./54.*m11*RSp451e*
         RSe561p - 1./54.*m11*RSp451e*RSe451p + 1./54.*m11*RSp451e*
         RSp561e - 1./108.*m11*pow(RSp451e,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(me2,L) * (  - 1./81.*me2*RSe563p
         *RS345pe + 1./81.*me2*RSe563p*RS356pe + 1./81.*me2*pow(
         RSe563p,2) + 1./81.*me2*RSe453p*RS345pe - 1./81.*me2*RSe453p*
         RS356pe - 2./81.*me2*RSe453p*RSe563p + 1./81.*me2*pow(
         RSe453p,2) + 1./81.*me2*RSp563e*RSe563p - 1./81.*me2*RSp563e*
         RSe453p - 1./81.*me2*RSp453e*RSe563p + 1./81.*me2*RSp453e*
         RSe453p + 1./81.*me2*RS145pe*RSe563p - 1./81.*me2*RS145pe*
         RSe453p - 1./81.*me2*RS156pe*RSe563p + 1./81.*me2*RS156pe*
         RSe453p + 1./81.*me2*RSe561p*RS345pe - 1./81.*me2*RSe561p*
         RS356pe - 2./81.*me2*RSe561p*RSe563p + 2./81.*me2*RSe561p*
         RSe453p - 1./81.*me2*RSe561p*RSp563e + 1./81.*me2*RSe561p*
         RSp453e - 1./81.*me2*RSe561p*RS145pe + 1./81.*me2*RSe561p*
         RS156pe + 1./81.*me2*pow(RSe561p,2) - 1./81.*me2*RSe451p*
         RS345pe + 1./81.*me2*RSe451p*RS356pe + 2./81.*me2*RSe451p*
         RSe563p - 2./81.*me2*RSe451p*RSe453p + 1./81.*me2*RSe451p*
         RSp563e );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(me2,L) * (  - 1./81.*me2*RSe451p
         *RSp453e + 1./81.*me2*RSe451p*RS145pe - 1./81.*me2*RSe451p*
         RS156pe - 2./81.*me2*RSe451p*RSe561p + 1./81.*me2*pow(
         RSe451p,2) - 1./81.*me2*RSp561e*RSe563p + 1./81.*me2*RSp561e*
         RSe453p + 1./81.*me2*RSp561e*RSe561p - 1./81.*me2*RSp561e*
         RSe451p + 1./81.*me2*RSp451e*RSe563p - 1./81.*me2*RSp451e*
         RSe453p - 1./81.*me2*RSp451e*RSe561p + 1./81.*me2*RSp451e*
         RSe451p - 1./162.*m66*RSe563p*RS345pe - 1./162.*m66*RSe563p*
         RS356pe - 1./324.*m66*pow(RSe563p,2) - 1./54.*m66*RSe453p*
         RS345pe - 1./162.*m66*RSe453p*RS356pe - 1./162.*m66*RSe453p*
         RSe563p - 1./108.*m66*pow(RSe453p,2) + 1./162.*m66*RS145pe*
         RSe563p + 1./54.*m66*RS145pe*RSe453p + 1./162.*m66*RS156pe*
         RSe563p + 1./162.*m66*RS156pe*RSe453p + 1./162.*m66*RSe561p*
         RS345pe + 1./162.*m66*RSe561p*RS356pe + 1./162.*m66*RSe561p*
         RSe563p + 1./162.*m66*RSe561p*RSe453p - 1./162.*m66*RSe561p*
         RS145pe );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(me2,L) * (  - 1./162.*m66*
         RSe561p*RS156pe - 1./324.*m66*pow(RSe561p,2) + 1./54.*m66*
         RSe451p*RS345pe + 1./162.*m66*RSe451p*RS356pe + 1./162.*m66*
         RSe451p*RSe563p + 1./54.*m66*RSe451p*RSe453p - 1./54.*m66*
         RSe451p*RS145pe - 1./162.*m66*RSe451p*RS156pe - 1./162.*m66*
         RSe451p*RSe561p - 1./108.*m66*pow(RSe451p,2) - 1./81.*m46*
         RSe563p*RS345pe + 1./81.*m46*RSe563p*RS356pe + 1./162.*m46*
         pow(RSe563p,2) + 1./81.*m46*RSe453p*RS345pe - 1./81.*m46*
         RSe453p*RS356pe - 1./81.*m46*RSe453p*RSe563p + 1./162.*m46*
         pow(RSe453p,2) + 1./81.*m46*RS145pe*RSe563p - 1./81.*m46*
         RS145pe*RSe453p - 1./81.*m46*RS156pe*RSe563p + 1./81.*m46*
         RS156pe*RSe453p + 1./81.*m46*RSe561p*RS345pe - 1./81.*m46*
         RSe561p*RS356pe - 1./81.*m46*RSe561p*RSe563p + 1./81.*m46*
         RSe561p*RSe453p - 1./81.*m46*RSe561p*RS145pe + 1./81.*m46*
         RSe561p*RS156pe + 1./162.*m46*pow(RSe561p,2) - 1./81.*m46*
         RSe451p*RS345pe );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(me2,L) * ( 1./81.*m46*RSe451p*
         RS356pe + 1./81.*m46*RSe451p*RSe563p - 1./81.*m46*RSe451p*
         RSe453p + 1./81.*m46*RSe451p*RS145pe - 1./81.*m46*RSe451p*
         RS156pe - 1./81.*m46*RSe451p*RSe561p + 1./162.*m46*pow(
         RSe451p,2) - 1./162.*m44*RSe563p*RS345pe - 1./54.*m44*RSe563p*
         RS356pe - 1./108.*m44*pow(RSe563p,2) - 1./162.*m44*RSe453p*
         RS345pe - 1./162.*m44*RSe453p*RS356pe - 1./162.*m44*RSe453p*
         RSe563p - 1./324.*m44*pow(RSe453p,2) + 1./162.*m44*RS145pe*
         RSe563p + 1./162.*m44*RS145pe*RSe453p + 1./54.*m44*RS156pe*
         RSe563p + 1./162.*m44*RS156pe*RSe453p + 1./162.*m44*RSe561p*
         RS345pe + 1./54.*m44*RSe561p*RS356pe + 1./54.*m44*RSe561p*
         RSe563p + 1./162.*m44*RSe561p*RSe453p - 1./162.*m44*RSe561p*
         RS145pe - 1./54.*m44*RSe561p*RS156pe - 1./108.*m44*pow(
         RSe561p,2) + 1./162.*m44*RSe451p*RS345pe + 1./162.*m44*RSe451p
         *RS356pe + 1./162.*m44*RSe451p*RSe563p + 1./162.*m44*RSe451p*
         RSe453p );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(me2,L) * (  - 1./162.*m44*
         RSe451p*RS145pe - 1./162.*m44*RSe451p*RS156pe - 1./162.*m44*
         RSe451p*RSe561p - 1./324.*m44*pow(RSe451p,2) + 1./54.*m33*
         RSe563p*RS345pe - 1./54.*m33*RSe563p*RS356pe - 1./108.*m33*
         pow(RSe563p,2) - 1./54.*m33*RSe453p*RS345pe + 1./54.*m33*
         RSe453p*RS356pe + 1./54.*m33*RSe453p*RSe563p - 1./108.*m33*
         pow(RSe453p,2) + 1./162.*m33*RS145pe*RSe563p - 1./162.*m33*
         RS145pe*RSe453p - 1./162.*m33*RS156pe*RSe563p + 1./162.*m33*
         RS156pe*RSe453p + 1./162.*m33*RSe561p*RS345pe - 1./162.*m33*
         RSe561p*RS356pe - 1./162.*m33*RSe561p*RSe563p + 1./162.*m33*
         RSe561p*RSe453p + 1./162.*m33*RSe561p*RS145pe - 1./162.*m33*
         RSe561p*RS156pe - 1./324.*m33*pow(RSe561p,2) - 1./162.*m33*
         RSe451p*RS345pe + 1./162.*m33*RSe451p*RS356pe + 1./162.*m33*
         RSe451p*RSe563p - 1./162.*m33*RSe451p*RSe453p - 1./162.*m33*
         RSe451p*RS145pe + 1./162.*m33*RSe451p*RS156pe + 1./162.*m33*
         RSe451p*RSe561p );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(me2,L) * (  - 1./324.*m33*pow(
         RSe451p,2) - 1./81.*m13*RSe563p*RS345pe + 1./81.*m13*RSe563p*
         RS356pe + 1./162.*m13*pow(RSe563p,2) + 1./81.*m13*RSe453p*
         RS345pe - 1./81.*m13*RSe453p*RS356pe - 1./81.*m13*RSe453p*
         RSe563p + 1./162.*m13*pow(RSe453p,2) + 1./81.*m13*RS145pe*
         RSe563p - 1./81.*m13*RS145pe*RSe453p - 1./81.*m13*RS156pe*
         RSe563p + 1./81.*m13*RS156pe*RSe453p + 1./81.*m13*RSe561p*
         RS345pe - 1./81.*m13*RSe561p*RS356pe - 1./81.*m13*RSe561p*
         RSe563p + 1./81.*m13*RSe561p*RSe453p - 1./81.*m13*RSe561p*
         RS145pe + 1./81.*m13*RSe561p*RS156pe + 1./162.*m13*pow(
         RSe561p,2) - 1./81.*m13*RSe451p*RS345pe + 1./81.*m13*RSe451p*
         RS356pe + 1./81.*m13*RSe451p*RSe563p - 1./81.*m13*RSe451p*
         RSe453p + 1./81.*m13*RSe451p*RS145pe - 1./81.*m13*RSe451p*
         RS156pe - 1./81.*m13*RSe451p*RSe561p + 1./162.*m13*pow(
         RSe451p,2) + 1./162.*m11*RSe563p*RS345pe - 1./162.*m11*RSe563p
         *RS356pe );

      M13p6x23RV +=  + Ab(m46,mu2)*AbV(me2,L) * (  - 1./324.*m11*pow(
         RSe563p,2) - 1./162.*m11*RSe453p*RS345pe + 1./162.*m11*RSe453p
         *RS356pe + 1./162.*m11*RSe453p*RSe563p - 1./324.*m11*pow(
         RSe453p,2) + 1./162.*m11*RS145pe*RSe563p - 1./162.*m11*RS145pe
         *RSe453p - 1./162.*m11*RS156pe*RSe563p + 1./162.*m11*RS156pe*
         RSe453p + 1./162.*m11*RSe561p*RS345pe - 1./162.*m11*RSe561p*
         RS356pe - 1./162.*m11*RSe561p*RSe563p + 1./162.*m11*RSe561p*
         RSe453p + 1./54.*m11*RSe561p*RS145pe - 1./54.*m11*RSe561p*
         RS156pe - 1./108.*m11*pow(RSe561p,2) - 1./162.*m11*RSe451p*
         RS345pe + 1./162.*m11*RSe451p*RS356pe + 1./162.*m11*RSe451p*
         RSe563p - 1./162.*m11*RSe451p*RSe453p - 1./54.*m11*RSe451p*
         RS145pe + 1./54.*m11*RSe451p*RS156pe + 1./54.*m11*RSe451p*
         RSe561p - 1./108.*m11*pow(RSe451p,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,m33,L) * ( 1./108.*m33*m66*
         RS145pe*RS345pe + 1./324.*m33*m66*RS145pe*RS356pe + 1./324.*
         m33*m66*RS156pe*RS345pe + 1./324.*m33*m66*RS156pe*RS356pe - 1./
         162.*m33*m46*RS145pe*RS345pe + 1./162.*m33*m46*RS145pe*RS356pe
          + 1./162.*m33*m46*RS156pe*RS345pe - 1./162.*m33*m46*RS156pe*
         RS356pe + 1./324.*m33*m44*RS145pe*RS345pe + 1./324.*m33*m44*
         RS145pe*RS356pe + 1./324.*m33*m44*RS156pe*RS345pe + 1./108.*
         m33*m44*RS156pe*RS356pe + 1./54.*m13*m66*RS145pe*RS345pe + 1./
         162.*m13*m66*RS145pe*RS356pe + 1./162.*m13*m66*RS156pe*RS345pe
          + 1./162.*m13*m66*RS156pe*RS356pe - 1./81.*m13*m46*RS145pe*
         RS345pe + 1./81.*m13*m46*RS145pe*RS356pe + 1./81.*m13*m46*
         RS156pe*RS345pe - 1./81.*m13*m46*RS156pe*RS356pe + 1./162.*m13
         *m44*RS145pe*RS345pe + 1./162.*m13*m44*RS145pe*RS356pe + 1./
         162.*m13*m44*RS156pe*RS345pe + 1./54.*m13*m44*RS156pe*RS356pe
          + 1./36.*m11*m66*RS145pe*RS345pe + 1./108.*m11*m66*RS145pe*
         RS356pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,m33,L) * ( 1./108.*m11*m66*
         RS156pe*RS345pe + 1./108.*m11*m66*RS156pe*RS356pe - 1./54.*m11
         *m46*RS145pe*RS345pe + 1./54.*m11*m46*RS145pe*RS356pe + 1./54.
         *m11*m46*RS156pe*RS345pe - 1./54.*m11*m46*RS156pe*RS356pe + 1./
         108.*m11*m44*RS145pe*RS345pe + 1./108.*m11*m44*RS145pe*RS356pe
          + 1./108.*m11*m44*RS156pe*RS345pe + 1./36.*m11*m44*RS156pe*
         RS356pe - 1./54.*m11*m33*RS145pe*RS345pe + 1./54.*m11*m33*
         RS145pe*RS356pe + 1./54.*m11*m33*RS156pe*RS345pe - 1./54.*m11*
         m33*RS156pe*RS356pe - 1./81.*m11*m13*RS145pe*RS345pe + 1./81.*
         m11*m13*RS145pe*RS356pe + 1./81.*m11*m13*RS156pe*RS345pe - 1./
         81.*m11*m13*RS156pe*RS356pe - 1./162.*pow(m11,2)*RS145pe*
         RS345pe + 1./162.*pow(m11,2)*RS145pe*RS356pe + 1./162.*pow(
         m11,2)*RS156pe*RS345pe - 1./162.*pow(m11,2)*RS156pe*RS356pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,mp2,L) * ( 1./324.*m33*m66*
         RS145pe*RSp563e + 1./108.*m33*m66*RS145pe*RSp453e + 1./324.*
         m33*m66*RS156pe*RSp563e + 1./324.*m33*m66*RS156pe*RSp453e + 1./
         324.*m33*m66*RSp561e*RS145pe + 1./324.*m33*m66*RSp561e*RS156pe
          + 1./108.*m33*m66*RSp451e*RS145pe + 1./324.*m33*m66*RSp451e*
         RS156pe + 1./162.*m33*m46*RS145pe*RSp563e - 1./162.*m33*m46*
         RS145pe*RSp453e - 1./162.*m33*m46*RS156pe*RSp563e + 1./162.*
         m33*m46*RS156pe*RSp453e + 1./162.*m33*m46*RSp561e*RS145pe - 1./
         162.*m33*m46*RSp561e*RS156pe - 1./162.*m33*m46*RSp451e*RS145pe
          + 1./162.*m33*m46*RSp451e*RS156pe + 1./324.*m33*m44*RS145pe*
         RSp563e + 1./324.*m33*m44*RS145pe*RSp453e + 1./108.*m33*m44*
         RS156pe*RSp563e + 1./324.*m33*m44*RS156pe*RSp453e + 1./324.*
         m33*m44*RSp561e*RS145pe + 1./108.*m33*m44*RSp561e*RS156pe + 1./
         324.*m33*m44*RSp451e*RS145pe + 1./324.*m33*m44*RSp451e*RS156pe
          + 1./162.*m13*m66*RS145pe*RSp563e + 1./54.*m13*m66*RS145pe*
         RSp453e );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,mp2,L) * ( 1./162.*m13*m66*
         RS156pe*RSp563e + 1./162.*m13*m66*RS156pe*RSp453e - 1./162.*
         m13*m66*RSp561e*RS145pe - 1./162.*m13*m66*RSp561e*RS156pe - 1./
         54.*m13*m66*RSp451e*RS145pe - 1./162.*m13*m66*RSp451e*RS156pe
          + 1./81.*m13*m46*RS145pe*RSp563e - 1./81.*m13*m46*RS145pe*
         RSp453e - 1./81.*m13*m46*RS156pe*RSp563e + 1./81.*m13*m46*
         RS156pe*RSp453e - 1./81.*m13*m46*RSp561e*RS145pe + 1./81.*m13*
         m46*RSp561e*RS156pe + 1./81.*m13*m46*RSp451e*RS145pe - 1./81.*
         m13*m46*RSp451e*RS156pe + 1./162.*m13*m44*RS145pe*RSp563e + 1./
         162.*m13*m44*RS145pe*RSp453e + 1./54.*m13*m44*RS156pe*RSp563e
          + 1./162.*m13*m44*RS156pe*RSp453e - 1./162.*m13*m44*RSp561e*
         RS145pe - 1./54.*m13*m44*RSp561e*RS156pe - 1./162.*m13*m44*
         RSp451e*RS145pe - 1./162.*m13*m44*RSp451e*RS156pe + 1./81.*m11
         *mp2*RS145pe*RSp563e - 1./81.*m11*mp2*RS145pe*RSp453e - 1./81.
         *m11*mp2*RS156pe*RSp563e + 1./81.*m11*mp2*RS156pe*RSp453e - 1./
         81.*m11*mp2*RSp561e*RS145pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,mp2,L) * ( 1./81.*m11*mp2*
         RSp561e*RS156pe + 1./81.*m11*mp2*RSp451e*RS145pe - 1./81.*m11*
         mp2*RSp451e*RS156pe + 1./108.*m11*m66*RS145pe*RSp563e + 1./36.
         *m11*m66*RS145pe*RSp453e + 1./108.*m11*m66*RS156pe*RSp563e + 1.
         /108.*m11*m66*RS156pe*RSp453e + 1./324.*m11*m66*RSp561e*
         RS145pe + 1./324.*m11*m66*RSp561e*RS156pe + 1./108.*m11*m66*
         RSp451e*RS145pe + 1./324.*m11*m66*RSp451e*RS156pe + 1./54.*m11
         *m46*RS145pe*RSp563e - 1./54.*m11*m46*RS145pe*RSp453e - 1./54.
         *m11*m46*RS156pe*RSp563e + 1./54.*m11*m46*RS156pe*RSp453e + 1./
         162.*m11*m46*RSp561e*RS145pe - 1./162.*m11*m46*RSp561e*RS156pe
          - 1./162.*m11*m46*RSp451e*RS145pe + 1./162.*m11*m46*RSp451e*
         RS156pe + 1./108.*m11*m44*RS145pe*RSp563e + 1./108.*m11*m44*
         RS145pe*RSp453e + 1./36.*m11*m44*RS156pe*RSp563e + 1./108.*m11
         *m44*RS156pe*RSp453e + 1./324.*m11*m44*RSp561e*RS145pe + 1./
         108.*m11*m44*RSp561e*RS156pe + 1./324.*m11*m44*RSp451e*RS145pe
          + 1./324.*m11*m44*RSp451e*RS156pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,mp2,L) * ( 1./162.*m11*m33*
         RS145pe*RSp563e - 1./162.*m11*m33*RS145pe*RSp453e - 1./162.*
         m11*m33*RS156pe*RSp563e + 1./162.*m11*m33*RS156pe*RSp453e + 1./
         162.*m11*m33*RSp561e*RS145pe - 1./162.*m11*m33*RSp561e*RS156pe
          - 1./162.*m11*m33*RSp451e*RS145pe + 1./162.*m11*m33*RSp451e*
         RS156pe + 1./81.*m11*m13*RS145pe*RSp563e - 1./81.*m11*m13*
         RS145pe*RSp453e - 1./81.*m11*m13*RS156pe*RSp563e + 1./81.*m11*
         m13*RS156pe*RSp453e - 1./81.*m11*m13*RSp561e*RS145pe + 1./81.*
         m11*m13*RSp561e*RS156pe + 1./81.*m11*m13*RSp451e*RS145pe - 1./
         81.*m11*m13*RSp451e*RS156pe + 1./162.*pow(m11,2)*RS145pe*
         RSp563e - 1./162.*pow(m11,2)*RS145pe*RSp453e - 1./162.*pow(
         m11,2)*RS156pe*RSp563e + 1./162.*pow(m11,2)*RS156pe*RSp453e + 
         1./54.*pow(m11,2)*RSp561e*RS145pe - 1./54.*pow(m11,2)*RSp561e*
         RS156pe - 1./54.*pow(m11,2)*RSp451e*RS145pe + 1./54.*pow(
         m11,2)*RSp451e*RS156pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,me2,L) * ( 1./324.*m33*m66*
         RS145pe*RSe563p + 1./108.*m33*m66*RS145pe*RSe453p + 1./324.*
         m33*m66*RS156pe*RSe563p + 1./324.*m33*m66*RS156pe*RSe453p + 1./
         324.*m33*m66*RSe561p*RS145pe + 1./324.*m33*m66*RSe561p*RS156pe
          + 1./108.*m33*m66*RSe451p*RS145pe + 1./324.*m33*m66*RSe451p*
         RS156pe + 1./162.*m33*m46*RS145pe*RSe563p - 1./162.*m33*m46*
         RS145pe*RSe453p - 1./162.*m33*m46*RS156pe*RSe563p + 1./162.*
         m33*m46*RS156pe*RSe453p + 1./162.*m33*m46*RSe561p*RS145pe - 1./
         162.*m33*m46*RSe561p*RS156pe - 1./162.*m33*m46*RSe451p*RS145pe
          + 1./162.*m33*m46*RSe451p*RS156pe + 1./324.*m33*m44*RS145pe*
         RSe563p + 1./324.*m33*m44*RS145pe*RSe453p + 1./108.*m33*m44*
         RS156pe*RSe563p + 1./324.*m33*m44*RS156pe*RSe453p + 1./324.*
         m33*m44*RSe561p*RS145pe + 1./108.*m33*m44*RSe561p*RS156pe + 1./
         324.*m33*m44*RSe451p*RS145pe + 1./324.*m33*m44*RSe451p*RS156pe
          + 1./162.*m13*m66*RS145pe*RSe563p + 1./54.*m13*m66*RS145pe*
         RSe453p );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,me2,L) * ( 1./162.*m13*m66*
         RS156pe*RSe563p + 1./162.*m13*m66*RS156pe*RSe453p - 1./162.*
         m13*m66*RSe561p*RS145pe - 1./162.*m13*m66*RSe561p*RS156pe - 1./
         54.*m13*m66*RSe451p*RS145pe - 1./162.*m13*m66*RSe451p*RS156pe
          + 1./81.*m13*m46*RS145pe*RSe563p - 1./81.*m13*m46*RS145pe*
         RSe453p - 1./81.*m13*m46*RS156pe*RSe563p + 1./81.*m13*m46*
         RS156pe*RSe453p - 1./81.*m13*m46*RSe561p*RS145pe + 1./81.*m13*
         m46*RSe561p*RS156pe + 1./81.*m13*m46*RSe451p*RS145pe - 1./81.*
         m13*m46*RSe451p*RS156pe + 1./162.*m13*m44*RS145pe*RSe563p + 1./
         162.*m13*m44*RS145pe*RSe453p + 1./54.*m13*m44*RS156pe*RSe563p
          + 1./162.*m13*m44*RS156pe*RSe453p - 1./162.*m13*m44*RSe561p*
         RS145pe - 1./54.*m13*m44*RSe561p*RS156pe - 1./162.*m13*m44*
         RSe451p*RS145pe - 1./162.*m13*m44*RSe451p*RS156pe + 1./81.*m11
         *me2*RS145pe*RSe563p - 1./81.*m11*me2*RS145pe*RSe453p - 1./81.
         *m11*me2*RS156pe*RSe563p + 1./81.*m11*me2*RS156pe*RSe453p - 1./
         81.*m11*me2*RSe561p*RS145pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,me2,L) * ( 1./81.*m11*me2*
         RSe561p*RS156pe + 1./81.*m11*me2*RSe451p*RS145pe - 1./81.*m11*
         me2*RSe451p*RS156pe + 1./108.*m11*m66*RS145pe*RSe563p + 1./36.
         *m11*m66*RS145pe*RSe453p + 1./108.*m11*m66*RS156pe*RSe563p + 1.
         /108.*m11*m66*RS156pe*RSe453p + 1./324.*m11*m66*RSe561p*
         RS145pe + 1./324.*m11*m66*RSe561p*RS156pe + 1./108.*m11*m66*
         RSe451p*RS145pe + 1./324.*m11*m66*RSe451p*RS156pe + 1./54.*m11
         *m46*RS145pe*RSe563p - 1./54.*m11*m46*RS145pe*RSe453p - 1./54.
         *m11*m46*RS156pe*RSe563p + 1./54.*m11*m46*RS156pe*RSe453p + 1./
         162.*m11*m46*RSe561p*RS145pe - 1./162.*m11*m46*RSe561p*RS156pe
          - 1./162.*m11*m46*RSe451p*RS145pe + 1./162.*m11*m46*RSe451p*
         RS156pe + 1./108.*m11*m44*RS145pe*RSe563p + 1./108.*m11*m44*
         RS145pe*RSe453p + 1./36.*m11*m44*RS156pe*RSe563p + 1./108.*m11
         *m44*RS156pe*RSe453p + 1./324.*m11*m44*RSe561p*RS145pe + 1./
         108.*m11*m44*RSe561p*RS156pe + 1./324.*m11*m44*RSe451p*RS145pe
          + 1./324.*m11*m44*RSe451p*RS156pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,me2,L) * ( 1./162.*m11*m33*
         RS145pe*RSe563p - 1./162.*m11*m33*RS145pe*RSe453p - 1./162.*
         m11*m33*RS156pe*RSe563p + 1./162.*m11*m33*RS156pe*RSe453p + 1./
         162.*m11*m33*RSe561p*RS145pe - 1./162.*m11*m33*RSe561p*RS156pe
          - 1./162.*m11*m33*RSe451p*RS145pe + 1./162.*m11*m33*RSe451p*
         RS156pe + 1./81.*m11*m13*RS145pe*RSe563p - 1./81.*m11*m13*
         RS145pe*RSe453p - 1./81.*m11*m13*RS156pe*RSe563p + 1./81.*m11*
         m13*RS156pe*RSe453p - 1./81.*m11*m13*RSe561p*RS145pe + 1./81.*
         m11*m13*RSe561p*RS156pe + 1./81.*m11*m13*RSe451p*RS145pe - 1./
         81.*m11*m13*RSe451p*RS156pe + 1./162.*pow(m11,2)*RS145pe*
         RSe563p - 1./162.*pow(m11,2)*RS145pe*RSe453p - 1./162.*pow(
         m11,2)*RS156pe*RSe563p + 1./162.*pow(m11,2)*RS156pe*RSe453p + 
         1./54.*pow(m11,2)*RSe561p*RS145pe - 1./54.*pow(m11,2)*RSe561p*
         RS156pe - 1./54.*pow(m11,2)*RSe451p*RS145pe + 1./54.*pow(
         m11,2)*RSe451p*RS156pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,L) * ( 1./216.*m33*m66*pow(
         RS145pe,2) + 1./324.*m33*m66*RS156pe*RS145pe + 1./648.*m33*m66
         *pow(RS156pe,2) - 1./324.*m33*m46*pow(RS145pe,2) + 1./162.*m33
         *m46*RS156pe*RS145pe - 1./324.*m33*m46*pow(RS156pe,2) + 1./648.
         *m33*m44*pow(RS145pe,2) + 1./324.*m33*m44*RS156pe*RS145pe + 1./
         216.*m33*m44*pow(RS156pe,2) - 1./108.*m13*m66*pow(RS145pe,2)
          - 1./162.*m13*m66*RS156pe*RS145pe - 1./324.*m13*m66*pow(
         RS156pe,2) + 1./162.*m13*m46*pow(RS145pe,2) - 1./81.*m13*m46*
         RS156pe*RS145pe + 1./162.*m13*m46*pow(RS156pe,2) - 1./324.*m13
         *m44*pow(RS145pe,2) - 1./162.*m13*m44*RS156pe*RS145pe - 1./108.
         *m13*m44*pow(RS156pe,2) + 1./216.*m11*m66*pow(RS145pe,2) + 1./
         324.*m11*m66*RS156pe*RS145pe + 1./648.*m11*m66*pow(RS156pe,2)
          - 1./324.*m11*m46*pow(RS145pe,2) + 1./162.*m11*m46*RS156pe*
         RS145pe - 1./324.*m11*m46*pow(RS156pe,2) + 1./648.*m11*m44*
         pow(RS145pe,2) + 1./324.*m11*m44*RS156pe*RS145pe + 1./216.*m11
         *m44*pow(RS156pe,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,L) * (  - 1./324.*m11*m33*
         pow(RS145pe,2) + 1./162.*m11*m33*RS156pe*RS145pe - 1./324.*m11
         *m33*pow(RS156pe,2) + 1./162.*m11*m13*pow(RS145pe,2) - 1./81.*
         m11*m13*RS156pe*RS145pe + 1./162.*m11*m13*pow(RS156pe,2) - 1./
         324.*pow(m11,2)*pow(RS145pe,2) + 1./162.*pow(m11,2)*RS156pe*
         RS145pe - 1./324.*pow(m11,2)*pow(RS156pe,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m14,L) * ( 1./144.*m44*m66 - 1./
         72.*m44*m46 + 1./72.*pow(m44,2) + 1./144.*m33*m66 - 1./72.*m33
         *m46 + 1./72.*m33*m44 - 1./72.*m14*m66 + 1./36.*m14*m46 - 1./
         24.*m14*m44 - 1./72.*m14*m33 + 1./36.*pow(m14,2) - 1./72.*m13*
         m66 + 1./36.*m13*m46 - 1./36.*m13*m44 + 1./36.*m13*m14 + 1./72.
         *m11*m66 - 1./36.*m11*m46 + 5./144.*m11*m44 + 1./144.*m11*m33
          - 1./24.*m11*m14 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m16,L) * ( 1./72.*pow(m66,2) - 1.
         /72.*m46*m66 + 1./144.*m44*m66 + 1./72.*m33*m66 - 1./72.*m33*
         m46 + 1./144.*m33*m44 - 1./24.*m16*m66 + 1./36.*m16*m46 - 1./
         72.*m16*m44 - 1./72.*m16*m33 + 1./36.*pow(m16,2) - 1./36.*m13*
         m66 + 1./36.*m13*m46 - 1./72.*m13*m44 + 1./36.*m13*m16 + 5./
         144.*m11*m66 - 1./36.*m11*m46 + 1./72.*m11*m44 + 1./144.*m11*
         m33 - 1./24.*m11*m16 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,mp2,L) * (  - 1./81.*m33*mp2
         *RSp563e*RS345pe + 1./81.*m33*mp2*RSp563e*RS356pe + 1./81.*m33
         *mp2*RSp453e*RS345pe - 1./81.*m33*mp2*RSp453e*RS356pe + 1./81.
         *m33*mp2*RSp561e*RS345pe - 1./81.*m33*mp2*RSp561e*RS356pe - 1./
         81.*m33*mp2*RSp451e*RS345pe + 1./81.*m33*mp2*RSp451e*RS356pe
          + 1./324.*m33*m66*RSp563e*RS345pe + 1./324.*m33*m66*RSp563e*
         RS356pe + 1./108.*m33*m66*RSp453e*RS345pe + 1./324.*m33*m66*
         RSp453e*RS356pe + 1./108.*m33*m66*RSp561e*RS345pe + 1./108.*
         m33*m66*RSp561e*RS356pe + 1./36.*m33*m66*RSp451e*RS345pe + 1./
         108.*m33*m66*RSp451e*RS356pe + 1./162.*m33*m46*RSp563e*RS345pe
          - 1./162.*m33*m46*RSp563e*RS356pe - 1./162.*m33*m46*RSp453e*
         RS345pe + 1./162.*m33*m46*RSp453e*RS356pe + 1./54.*m33*m46*
         RSp561e*RS345pe - 1./54.*m33*m46*RSp561e*RS356pe - 1./54.*m33*
         m46*RSp451e*RS345pe + 1./54.*m33*m46*RSp451e*RS356pe + 1./324.
         *m33*m44*RSp563e*RS345pe + 1./108.*m33*m44*RSp563e*RS356pe + 1.
         /324.*m33*m44*RSp453e*RS345pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,mp2,L) * ( 1./324.*m33*m44*
         RSp453e*RS356pe + 1./108.*m33*m44*RSp561e*RS345pe + 1./36.*m33
         *m44*RSp561e*RS356pe + 1./108.*m33*m44*RSp451e*RS345pe + 1./
         108.*m33*m44*RSp451e*RS356pe + 1./54.*pow(m33,2)*RSp563e*
         RS345pe - 1./54.*pow(m33,2)*RSp563e*RS356pe - 1./54.*pow(
         m33,2)*RSp453e*RS345pe + 1./54.*pow(m33,2)*RSp453e*RS356pe + 1.
         /162.*pow(m33,2)*RSp561e*RS345pe - 1./162.*pow(m33,2)*RSp561e*
         RS356pe - 1./162.*pow(m33,2)*RSp451e*RS345pe + 1./162.*pow(
         m33,2)*RSp451e*RS356pe - 1./162.*m13*m66*RSp563e*RS345pe - 1./
         162.*m13*m66*RSp563e*RS356pe - 1./54.*m13*m66*RSp453e*RS345pe
          - 1./162.*m13*m66*RSp453e*RS356pe + 1./162.*m13*m66*RSp561e*
         RS345pe + 1./162.*m13*m66*RSp561e*RS356pe + 1./54.*m13*m66*
         RSp451e*RS345pe + 1./162.*m13*m66*RSp451e*RS356pe - 1./81.*m13
         *m46*RSp563e*RS345pe + 1./81.*m13*m46*RSp563e*RS356pe + 1./81.
         *m13*m46*RSp453e*RS345pe - 1./81.*m13*m46*RSp453e*RS356pe + 1./
         81.*m13*m46*RSp561e*RS345pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,mp2,L) * (  - 1./81.*m13*m46
         *RSp561e*RS356pe - 1./81.*m13*m46*RSp451e*RS345pe + 1./81.*m13
         *m46*RSp451e*RS356pe - 1./162.*m13*m44*RSp563e*RS345pe - 1./54.
         *m13*m44*RSp563e*RS356pe - 1./162.*m13*m44*RSp453e*RS345pe - 1.
         /162.*m13*m44*RSp453e*RS356pe + 1./162.*m13*m44*RSp561e*
         RS345pe + 1./54.*m13*m44*RSp561e*RS356pe + 1./162.*m13*m44*
         RSp451e*RS345pe + 1./162.*m13*m44*RSp451e*RS356pe - 1./81.*m13
         *m33*RSp563e*RS345pe + 1./81.*m13*m33*RSp563e*RS356pe + 1./81.
         *m13*m33*RSp453e*RS345pe - 1./81.*m13*m33*RSp453e*RS356pe + 1./
         81.*m13*m33*RSp561e*RS345pe - 1./81.*m13*m33*RSp561e*RS356pe
          - 1./81.*m13*m33*RSp451e*RS345pe + 1./81.*m13*m33*RSp451e*
         RS356pe + 1./324.*m11*m66*RSp563e*RS345pe + 1./324.*m11*m66*
         RSp563e*RS356pe + 1./108.*m11*m66*RSp453e*RS345pe + 1./324.*
         m11*m66*RSp453e*RS356pe + 1./324.*m11*m66*RSp561e*RS345pe + 1./
         324.*m11*m66*RSp561e*RS356pe + 1./108.*m11*m66*RSp451e*RS345pe
          + 1./324.*m11*m66*RSp451e*RS356pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,mp2,L) * ( 1./162.*m11*m46*
         RSp563e*RS345pe - 1./162.*m11*m46*RSp563e*RS356pe - 1./162.*
         m11*m46*RSp453e*RS345pe + 1./162.*m11*m46*RSp453e*RS356pe + 1./
         162.*m11*m46*RSp561e*RS345pe - 1./162.*m11*m46*RSp561e*RS356pe
          - 1./162.*m11*m46*RSp451e*RS345pe + 1./162.*m11*m46*RSp451e*
         RS356pe + 1./324.*m11*m44*RSp563e*RS345pe + 1./108.*m11*m44*
         RSp563e*RS356pe + 1./324.*m11*m44*RSp453e*RS345pe + 1./324.*
         m11*m44*RSp453e*RS356pe + 1./324.*m11*m44*RSp561e*RS345pe + 1./
         108.*m11*m44*RSp561e*RS356pe + 1./324.*m11*m44*RSp451e*RS345pe
          + 1./324.*m11*m44*RSp451e*RS356pe + 1./162.*m11*m33*RSp563e*
         RS345pe - 1./162.*m11*m33*RSp563e*RS356pe - 1./162.*m11*m33*
         RSp453e*RS345pe + 1./162.*m11*m33*RSp453e*RS356pe + 1./162.*
         m11*m33*RSp561e*RS345pe - 1./162.*m11*m33*RSp561e*RS356pe - 1./
         162.*m11*m33*RSp451e*RS345pe + 1./162.*m11*m33*RSp451e*RS356pe
          );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,me2,L) * (  - 1./81.*m33*me2
         *RSe563p*RS345pe + 1./81.*m33*me2*RSe563p*RS356pe + 1./81.*m33
         *me2*RSe453p*RS345pe - 1./81.*m33*me2*RSe453p*RS356pe + 1./81.
         *m33*me2*RSe561p*RS345pe - 1./81.*m33*me2*RSe561p*RS356pe - 1./
         81.*m33*me2*RSe451p*RS345pe + 1./81.*m33*me2*RSe451p*RS356pe
          + 1./324.*m33*m66*RSe563p*RS345pe + 1./324.*m33*m66*RSe563p*
         RS356pe + 1./108.*m33*m66*RSe453p*RS345pe + 1./324.*m33*m66*
         RSe453p*RS356pe + 1./108.*m33*m66*RSe561p*RS345pe + 1./108.*
         m33*m66*RSe561p*RS356pe + 1./36.*m33*m66*RSe451p*RS345pe + 1./
         108.*m33*m66*RSe451p*RS356pe + 1./162.*m33*m46*RSe563p*RS345pe
          - 1./162.*m33*m46*RSe563p*RS356pe - 1./162.*m33*m46*RSe453p*
         RS345pe + 1./162.*m33*m46*RSe453p*RS356pe + 1./54.*m33*m46*
         RSe561p*RS345pe - 1./54.*m33*m46*RSe561p*RS356pe - 1./54.*m33*
         m46*RSe451p*RS345pe + 1./54.*m33*m46*RSe451p*RS356pe + 1./324.
         *m33*m44*RSe563p*RS345pe + 1./108.*m33*m44*RSe563p*RS356pe + 1.
         /324.*m33*m44*RSe453p*RS345pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,me2,L) * ( 1./324.*m33*m44*
         RSe453p*RS356pe + 1./108.*m33*m44*RSe561p*RS345pe + 1./36.*m33
         *m44*RSe561p*RS356pe + 1./108.*m33*m44*RSe451p*RS345pe + 1./
         108.*m33*m44*RSe451p*RS356pe + 1./54.*pow(m33,2)*RSe563p*
         RS345pe - 1./54.*pow(m33,2)*RSe563p*RS356pe - 1./54.*pow(
         m33,2)*RSe453p*RS345pe + 1./54.*pow(m33,2)*RSe453p*RS356pe + 1.
         /162.*pow(m33,2)*RSe561p*RS345pe - 1./162.*pow(m33,2)*RSe561p*
         RS356pe - 1./162.*pow(m33,2)*RSe451p*RS345pe + 1./162.*pow(
         m33,2)*RSe451p*RS356pe - 1./162.*m13*m66*RSe563p*RS345pe - 1./
         162.*m13*m66*RSe563p*RS356pe - 1./54.*m13*m66*RSe453p*RS345pe
          - 1./162.*m13*m66*RSe453p*RS356pe + 1./162.*m13*m66*RSe561p*
         RS345pe + 1./162.*m13*m66*RSe561p*RS356pe + 1./54.*m13*m66*
         RSe451p*RS345pe + 1./162.*m13*m66*RSe451p*RS356pe - 1./81.*m13
         *m46*RSe563p*RS345pe + 1./81.*m13*m46*RSe563p*RS356pe + 1./81.
         *m13*m46*RSe453p*RS345pe - 1./81.*m13*m46*RSe453p*RS356pe + 1./
         81.*m13*m46*RSe561p*RS345pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,me2,L) * (  - 1./81.*m13*m46
         *RSe561p*RS356pe - 1./81.*m13*m46*RSe451p*RS345pe + 1./81.*m13
         *m46*RSe451p*RS356pe - 1./162.*m13*m44*RSe563p*RS345pe - 1./54.
         *m13*m44*RSe563p*RS356pe - 1./162.*m13*m44*RSe453p*RS345pe - 1.
         /162.*m13*m44*RSe453p*RS356pe + 1./162.*m13*m44*RSe561p*
         RS345pe + 1./54.*m13*m44*RSe561p*RS356pe + 1./162.*m13*m44*
         RSe451p*RS345pe + 1./162.*m13*m44*RSe451p*RS356pe - 1./81.*m13
         *m33*RSe563p*RS345pe + 1./81.*m13*m33*RSe563p*RS356pe + 1./81.
         *m13*m33*RSe453p*RS345pe - 1./81.*m13*m33*RSe453p*RS356pe + 1./
         81.*m13*m33*RSe561p*RS345pe - 1./81.*m13*m33*RSe561p*RS356pe
          - 1./81.*m13*m33*RSe451p*RS345pe + 1./81.*m13*m33*RSe451p*
         RS356pe + 1./324.*m11*m66*RSe563p*RS345pe + 1./324.*m11*m66*
         RSe563p*RS356pe + 1./108.*m11*m66*RSe453p*RS345pe + 1./324.*
         m11*m66*RSe453p*RS356pe + 1./324.*m11*m66*RSe561p*RS345pe + 1./
         324.*m11*m66*RSe561p*RS356pe + 1./108.*m11*m66*RSe451p*RS345pe
          + 1./324.*m11*m66*RSe451p*RS356pe );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,me2,L) * ( 1./162.*m11*m46*
         RSe563p*RS345pe - 1./162.*m11*m46*RSe563p*RS356pe - 1./162.*
         m11*m46*RSe453p*RS345pe + 1./162.*m11*m46*RSe453p*RS356pe + 1./
         162.*m11*m46*RSe561p*RS345pe - 1./162.*m11*m46*RSe561p*RS356pe
          - 1./162.*m11*m46*RSe451p*RS345pe + 1./162.*m11*m46*RSe451p*
         RS356pe + 1./324.*m11*m44*RSe563p*RS345pe + 1./108.*m11*m44*
         RSe563p*RS356pe + 1./324.*m11*m44*RSe453p*RS345pe + 1./324.*
         m11*m44*RSe453p*RS356pe + 1./324.*m11*m44*RSe561p*RS345pe + 1./
         108.*m11*m44*RSe561p*RS356pe + 1./324.*m11*m44*RSe451p*RS345pe
          + 1./324.*m11*m44*RSe451p*RS356pe + 1./162.*m11*m33*RSe563p*
         RS345pe - 1./162.*m11*m33*RSe563p*RS356pe - 1./162.*m11*m33*
         RSe453p*RS345pe + 1./162.*m11*m33*RSe453p*RS356pe + 1./162.*
         m11*m33*RSe561p*RS345pe - 1./162.*m11*m33*RSe561p*RS356pe - 1./
         162.*m11*m33*RSe451p*RS345pe + 1./162.*m11*m33*RSe451p*RS356pe
          );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,L) * ( 1./216.*m33*m66*pow(
         RS345pe,2) + 1./324.*m33*m66*RS356pe*RS345pe + 1./648.*m33*m66
         *pow(RS356pe,2) - 1./324.*m33*m46*pow(RS345pe,2) + 1./162.*m33
         *m46*RS356pe*RS345pe - 1./324.*m33*m46*pow(RS356pe,2) + 1./648.
         *m33*m44*pow(RS345pe,2) + 1./324.*m33*m44*RS356pe*RS345pe + 1./
         216.*m33*m44*pow(RS356pe,2) - 1./324.*pow(m33,2)*pow(
         RS345pe,2) + 1./162.*pow(m33,2)*RS356pe*RS345pe - 1./324.*pow(
         m33,2)*pow(RS356pe,2) - 1./108.*m13*m66*pow(RS345pe,2) - 1./
         162.*m13*m66*RS356pe*RS345pe - 1./324.*m13*m66*pow(RS356pe,2)
          + 1./162.*m13*m46*pow(RS345pe,2) - 1./81.*m13*m46*RS356pe*
         RS345pe + 1./162.*m13*m46*pow(RS356pe,2) - 1./324.*m13*m44*
         pow(RS345pe,2) - 1./162.*m13*m44*RS356pe*RS345pe - 1./108.*m13
         *m44*pow(RS356pe,2) + 1./162.*m13*m33*pow(RS345pe,2) - 1./81.*
         m13*m33*RS356pe*RS345pe + 1./162.*m13*m33*pow(RS356pe,2) + 1./
         216.*m11*m66*pow(RS345pe,2) + 1./324.*m11*m66*RS356pe*RS345pe
          + 1./648.*m11*m66*pow(RS356pe,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,L) * (  - 1./324.*m11*m46*
         pow(RS345pe,2) + 1./162.*m11*m46*RS356pe*RS345pe - 1./324.*m11
         *m46*pow(RS356pe,2) + 1./648.*m11*m44*pow(RS345pe,2) + 1./324.
         *m11*m44*RS356pe*RS345pe + 1./216.*m11*m44*pow(RS356pe,2) - 1./
         324.*m11*m33*pow(RS345pe,2) + 1./162.*m11*m33*RS356pe*RS345pe
          - 1./324.*m11*m33*pow(RS356pe,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m34,L) * ( 1./144.*m44*m66 - 1./
         72.*m44*m46 + 1./72.*pow(m44,2) - 1./72.*m34*m66 + 1./36.*m34*
         m46 - 1./24.*m34*m44 + 1./36.*pow(m34,2) + 1./72.*m33*m66 - 1./
         36.*m33*m46 + 5./144.*m33*m44 - 1./24.*m33*m34 + 1./72.*pow(
         m33,2) - 1./72.*m13*m66 + 1./36.*m13*m46 - 1./36.*m13*m44 + 1./
         36.*m13*m34 - 1./72.*m13*m33 + 1./144.*m11*m66 - 1./72.*m11*
         m46 + 1./72.*m11*m44 - 1./72.*m11*m34 + 1./144.*m11*m33 );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(m36,L) * ( 1./72.*pow(m66,2) - 1.
         /72.*m46*m66 + 1./144.*m44*m66 - 1./24.*m36*m66 + 1./36.*m36*
         m46 - 1./72.*m36*m44 + 1./36.*pow(m36,2) + 5./144.*m33*m66 - 1.
         /36.*m33*m46 + 1./72.*m33*m44 - 1./24.*m33*m36 + 1./72.*pow(
         m33,2) - 1./36.*m13*m66 + 1./36.*m13*m46 - 1./72.*m13*m44 + 1./
         36.*m13*m36 - 1./72.*m13*m33 + 1./72.*m11*m66 - 1./72.*m11*m46
          + 1./144.*m11*m44 - 1./72.*m11*m36 + 1./144.*m11*m33 );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(mp2,L) * ( 1./162.*pow(mp2,2)*
         pow(RSp563e,2) - 1./81.*pow(mp2,2)*RSp453e*RSp563e + 1./162.*
         pow(mp2,2)*pow(RSp453e,2) - 1./81.*pow(mp2,2)*RSp561e*RSp563e
          + 1./81.*pow(mp2,2)*RSp561e*RSp453e + 1./162.*pow(mp2,2)*pow(
         RSp561e,2) + 1./81.*pow(mp2,2)*RSp451e*RSp563e - 1./81.*pow(
         mp2,2)*RSp451e*RSp453e - 1./81.*pow(mp2,2)*RSp451e*RSp561e + 1.
         /162.*pow(mp2,2)*pow(RSp451e,2) - 1./324.*m66*mp2*pow(
         RSp563e,2) - 1./162.*m66*mp2*RSp453e*RSp563e - 1./108.*m66*mp2
         *pow(RSp453e,2) + 1./162.*m66*mp2*RSp561e*RSp563e + 1./162.*
         m66*mp2*RSp561e*RSp453e - 1./324.*m66*mp2*pow(RSp561e,2) + 1./
         162.*m66*mp2*RSp451e*RSp563e + 1./54.*m66*mp2*RSp451e*RSp453e
          - 1./162.*m66*mp2*RSp451e*RSp561e - 1./108.*m66*mp2*pow(
         RSp451e,2) + 1./162.*m46*mp2*pow(RSp563e,2) - 1./81.*m46*mp2*
         RSp453e*RSp563e + 1./162.*m46*mp2*pow(RSp453e,2) - 1./81.*m46*
         mp2*RSp561e*RSp563e + 1./81.*m46*mp2*RSp561e*RSp453e + 1./162.
         *m46*mp2*pow(RSp561e,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(mp2,L) * ( 1./81.*m46*mp2*
         RSp451e*RSp563e - 1./81.*m46*mp2*RSp451e*RSp453e - 1./81.*m46*
         mp2*RSp451e*RSp561e + 1./162.*m46*mp2*pow(RSp451e,2) - 1./108.
         *m44*mp2*pow(RSp563e,2) - 1./162.*m44*mp2*RSp453e*RSp563e - 1./
         324.*m44*mp2*pow(RSp453e,2) + 1./54.*m44*mp2*RSp561e*RSp563e
          + 1./162.*m44*mp2*RSp561e*RSp453e - 1./108.*m44*mp2*pow(
         RSp561e,2) + 1./162.*m44*mp2*RSp451e*RSp563e + 1./162.*m44*mp2
         *RSp451e*RSp453e - 1./162.*m44*mp2*RSp451e*RSp561e - 1./324.*
         m44*mp2*pow(RSp451e,2) - 1./108.*m33*mp2*pow(RSp563e,2) + 1./
         54.*m33*mp2*RSp453e*RSp563e - 1./108.*m33*mp2*pow(RSp453e,2)
          - 1./162.*m33*mp2*RSp561e*RSp563e + 1./162.*m33*mp2*RSp561e*
         RSp453e - 1./324.*m33*mp2*pow(RSp561e,2) + 1./162.*m33*mp2*
         RSp451e*RSp563e - 1./162.*m33*mp2*RSp451e*RSp453e + 1./162.*
         m33*mp2*RSp451e*RSp561e - 1./324.*m33*mp2*pow(RSp451e,2) + 1./
         216.*m33*m66*pow(RSp563e,2) + 1./108.*m33*m66*RSp453e*RSp563e
          + 1./72.*m33*m66*pow(RSp453e,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(mp2,L) * ( 1./324.*m33*m66*
         RSp561e*RSp563e + 1./324.*m33*m66*RSp561e*RSp453e + 1./648.*
         m33*m66*pow(RSp561e,2) + 1./324.*m33*m66*RSp451e*RSp563e + 1./
         108.*m33*m66*RSp451e*RSp453e + 1./324.*m33*m66*RSp451e*RSp561e
          + 1./216.*m33*m66*pow(RSp451e,2) - 1./108.*m33*m46*pow(
         RSp563e,2) + 1./54.*m33*m46*RSp453e*RSp563e - 1./108.*m33*m46*
         pow(RSp453e,2) - 1./162.*m33*m46*RSp561e*RSp563e + 1./162.*m33
         *m46*RSp561e*RSp453e - 1./324.*m33*m46*pow(RSp561e,2) + 1./162.
         *m33*m46*RSp451e*RSp563e - 1./162.*m33*m46*RSp451e*RSp453e + 1.
         /162.*m33*m46*RSp451e*RSp561e - 1./324.*m33*m46*pow(RSp451e,2)
          + 1./72.*m33*m44*pow(RSp563e,2) + 1./108.*m33*m44*RSp453e*
         RSp563e + 1./216.*m33*m44*pow(RSp453e,2) + 1./108.*m33*m44*
         RSp561e*RSp563e + 1./324.*m33*m44*RSp561e*RSp453e + 1./216.*
         m33*m44*pow(RSp561e,2) + 1./324.*m33*m44*RSp451e*RSp563e + 1./
         324.*m33*m44*RSp451e*RSp453e + 1./324.*m33*m44*RSp451e*RSp561e
          + 1./648.*m33*m44*pow(RSp451e,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(mp2,L) * ( 1./162.*m13*mp2*pow(
         RSp563e,2) - 1./81.*m13*mp2*RSp453e*RSp563e + 1./162.*m13*mp2*
         pow(RSp453e,2) - 1./81.*m13*mp2*RSp561e*RSp563e + 1./81.*m13*
         mp2*RSp561e*RSp453e + 1./162.*m13*mp2*pow(RSp561e,2) + 1./81.*
         m13*mp2*RSp451e*RSp563e - 1./81.*m13*mp2*RSp451e*RSp453e - 1./
         81.*m13*mp2*RSp451e*RSp561e + 1./162.*m13*mp2*pow(RSp451e,2)
          - 1./324.*m13*m66*pow(RSp563e,2) - 1./162.*m13*m66*RSp453e*
         RSp563e - 1./108.*m13*m66*pow(RSp453e,2) + 1./162.*m13*m66*
         RSp561e*RSp563e + 1./162.*m13*m66*RSp561e*RSp453e - 1./324.*
         m13*m66*pow(RSp561e,2) + 1./162.*m13*m66*RSp451e*RSp563e + 1./
         54.*m13*m66*RSp451e*RSp453e - 1./162.*m13*m66*RSp451e*RSp561e
          - 1./108.*m13*m66*pow(RSp451e,2) + 1./162.*m13*m46*pow(
         RSp563e,2) - 1./81.*m13*m46*RSp453e*RSp563e + 1./162.*m13*m46*
         pow(RSp453e,2) - 1./81.*m13*m46*RSp561e*RSp563e + 1./81.*m13*
         m46*RSp561e*RSp453e + 1./162.*m13*m46*pow(RSp561e,2) + 1./81.*
         m13*m46*RSp451e*RSp563e );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(mp2,L) * (  - 1./81.*m13*m46*
         RSp451e*RSp453e - 1./81.*m13*m46*RSp451e*RSp561e + 1./162.*m13
         *m46*pow(RSp451e,2) - 1./108.*m13*m44*pow(RSp563e,2) - 1./162.
         *m13*m44*RSp453e*RSp563e - 1./324.*m13*m44*pow(RSp453e,2) + 1./
         54.*m13*m44*RSp561e*RSp563e + 1./162.*m13*m44*RSp561e*RSp453e
          - 1./108.*m13*m44*pow(RSp561e,2) + 1./162.*m13*m44*RSp451e*
         RSp563e + 1./162.*m13*m44*RSp451e*RSp453e - 1./162.*m13*m44*
         RSp451e*RSp561e - 1./324.*m13*m44*pow(RSp451e,2) - 1./324.*m11
         *mp2*pow(RSp563e,2) + 1./162.*m11*mp2*RSp453e*RSp563e - 1./324.
         *m11*mp2*pow(RSp453e,2) - 1./162.*m11*mp2*RSp561e*RSp563e + 1./
         162.*m11*mp2*RSp561e*RSp453e - 1./108.*m11*mp2*pow(RSp561e,2)
          + 1./162.*m11*mp2*RSp451e*RSp563e - 1./162.*m11*mp2*RSp451e*
         RSp453e + 1./54.*m11*mp2*RSp451e*RSp561e - 1./108.*m11*mp2*
         pow(RSp451e,2) + 1./648.*m11*m66*pow(RSp563e,2) + 1./324.*m11*
         m66*RSp453e*RSp563e + 1./216.*m11*m66*pow(RSp453e,2) + 1./324.
         *m11*m66*RSp561e*RSp563e );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(mp2,L) * ( 1./324.*m11*m66*
         RSp561e*RSp453e + 1./216.*m11*m66*pow(RSp561e,2) + 1./324.*m11
         *m66*RSp451e*RSp563e + 1./108.*m11*m66*RSp451e*RSp453e + 1./
         108.*m11*m66*RSp451e*RSp561e + 1./72.*m11*m66*pow(RSp451e,2)
          - 1./324.*m11*m46*pow(RSp563e,2) + 1./162.*m11*m46*RSp453e*
         RSp563e - 1./324.*m11*m46*pow(RSp453e,2) - 1./162.*m11*m46*
         RSp561e*RSp563e + 1./162.*m11*m46*RSp561e*RSp453e - 1./108.*
         m11*m46*pow(RSp561e,2) + 1./162.*m11*m46*RSp451e*RSp563e - 1./
         162.*m11*m46*RSp451e*RSp453e + 1./54.*m11*m46*RSp451e*RSp561e
          - 1./108.*m11*m46*pow(RSp451e,2) + 1./216.*m11*m44*pow(
         RSp563e,2) + 1./324.*m11*m44*RSp453e*RSp563e + 1./648.*m11*m44
         *pow(RSp453e,2) + 1./108.*m11*m44*RSp561e*RSp563e + 1./324.*
         m11*m44*RSp561e*RSp453e + 1./72.*m11*m44*pow(RSp561e,2) + 1./
         324.*m11*m44*RSp451e*RSp563e + 1./324.*m11*m44*RSp451e*RSp453e
          + 1./108.*m11*m44*RSp451e*RSp561e + 1./216.*m11*m44*pow(
         RSp451e,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * ( 1./81.*mp2*me2*
         RSp563e*RSe563p - 1./81.*mp2*me2*RSp563e*RSe453p - 1./81.*mp2*
         me2*RSp453e*RSe563p + 1./81.*mp2*me2*RSp453e*RSe453p - 1./81.*
         mp2*me2*RSe561p*RSp563e + 1./81.*mp2*me2*RSe561p*RSp453e + 1./
         81.*mp2*me2*RSe451p*RSp563e - 1./81.*mp2*me2*RSe451p*RSp453e
          - 1./81.*mp2*me2*RSp561e*RSe563p + 1./81.*mp2*me2*RSp561e*
         RSe453p + 1./81.*mp2*me2*RSp561e*RSe561p - 1./81.*mp2*me2*
         RSp561e*RSe451p + 1./81.*mp2*me2*RSp451e*RSe563p - 1./81.*mp2*
         me2*RSp451e*RSe453p - 1./81.*mp2*me2*RSp451e*RSe561p + 1./81.*
         mp2*me2*RSp451e*RSe451p - 1./162.*m66*me2*RSp563e*RSe563p - 1./
         162.*m66*me2*RSp563e*RSe453p - 1./162.*m66*me2*RSp453e*RSe563p
          - 1./54.*m66*me2*RSp453e*RSe453p + 1./162.*m66*me2*RSe561p*
         RSp563e + 1./162.*m66*me2*RSe561p*RSp453e + 1./162.*m66*me2*
         RSe451p*RSp563e + 1./54.*m66*me2*RSe451p*RSp453e + 1./162.*m66
         *me2*RSp561e*RSe563p + 1./162.*m66*me2*RSp561e*RSe453p - 1./
         162.*m66*me2*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * (  - 1./162.*m66*
         me2*RSp561e*RSe451p + 1./162.*m66*me2*RSp451e*RSe563p + 1./54.
         *m66*me2*RSp451e*RSe453p - 1./162.*m66*me2*RSp451e*RSe561p - 1.
         /54.*m66*me2*RSp451e*RSe451p + 1./81.*m46*me2*RSp563e*RSe563p
          - 1./81.*m46*me2*RSp563e*RSe453p - 1./81.*m46*me2*RSp453e*
         RSe563p + 1./81.*m46*me2*RSp453e*RSe453p - 1./81.*m46*me2*
         RSe561p*RSp563e + 1./81.*m46*me2*RSe561p*RSp453e + 1./81.*m46*
         me2*RSe451p*RSp563e - 1./81.*m46*me2*RSe451p*RSp453e - 1./81.*
         m46*me2*RSp561e*RSe563p + 1./81.*m46*me2*RSp561e*RSe453p + 1./
         81.*m46*me2*RSp561e*RSe561p - 1./81.*m46*me2*RSp561e*RSe451p
          + 1./81.*m46*me2*RSp451e*RSe563p - 1./81.*m46*me2*RSp451e*
         RSe453p - 1./81.*m46*me2*RSp451e*RSe561p + 1./81.*m46*me2*
         RSp451e*RSe451p - 1./54.*m44*me2*RSp563e*RSe563p - 1./162.*m44
         *me2*RSp563e*RSe453p - 1./162.*m44*me2*RSp453e*RSe563p - 1./
         162.*m44*me2*RSp453e*RSe453p + 1./54.*m44*me2*RSe561p*RSp563e
          + 1./162.*m44*me2*RSe561p*RSp453e );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * ( 1./162.*m44*me2*
         RSe451p*RSp563e + 1./162.*m44*me2*RSe451p*RSp453e + 1./54.*m44
         *me2*RSp561e*RSe563p + 1./162.*m44*me2*RSp561e*RSe453p - 1./54.
         *m44*me2*RSp561e*RSe561p - 1./162.*m44*me2*RSp561e*RSe451p + 1.
         /162.*m44*me2*RSp451e*RSe563p + 1./162.*m44*me2*RSp451e*
         RSe453p - 1./162.*m44*me2*RSp451e*RSe561p - 1./162.*m44*me2*
         RSp451e*RSe451p - 1./54.*m33*me2*RSp563e*RSe563p + 1./54.*m33*
         me2*RSp563e*RSe453p + 1./54.*m33*me2*RSp453e*RSe563p - 1./54.*
         m33*me2*RSp453e*RSe453p - 1./162.*m33*me2*RSe561p*RSp563e + 1./
         162.*m33*me2*RSe561p*RSp453e + 1./162.*m33*me2*RSe451p*RSp563e
          - 1./162.*m33*me2*RSe451p*RSp453e - 1./162.*m33*me2*RSp561e*
         RSe563p + 1./162.*m33*me2*RSp561e*RSe453p - 1./162.*m33*me2*
         RSp561e*RSe561p + 1./162.*m33*me2*RSp561e*RSe451p + 1./162.*
         m33*me2*RSp451e*RSe563p - 1./162.*m33*me2*RSp451e*RSe453p + 1./
         162.*m33*me2*RSp451e*RSe561p - 1./162.*m33*me2*RSp451e*RSe451p
          + 1./108.*m33*m66*RSp563e*RSe563p );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * ( 1./108.*m33*m66*
         RSp563e*RSe453p + 1./108.*m33*m66*RSp453e*RSe563p + 1./36.*m33
         *m66*RSp453e*RSe453p + 1./324.*m33*m66*RSe561p*RSp563e + 1./
         324.*m33*m66*RSe561p*RSp453e + 1./324.*m33*m66*RSe451p*RSp563e
          + 1./108.*m33*m66*RSe451p*RSp453e + 1./324.*m33*m66*RSp561e*
         RSe563p + 1./324.*m33*m66*RSp561e*RSe453p + 1./324.*m33*m66*
         RSp561e*RSe561p + 1./324.*m33*m66*RSp561e*RSe451p + 1./324.*
         m33*m66*RSp451e*RSe563p + 1./108.*m33*m66*RSp451e*RSe453p + 1./
         324.*m33*m66*RSp451e*RSe561p + 1./108.*m33*m66*RSp451e*RSe451p
          - 1./54.*m33*m46*RSp563e*RSe563p + 1./54.*m33*m46*RSp563e*
         RSe453p + 1./54.*m33*m46*RSp453e*RSe563p - 1./54.*m33*m46*
         RSp453e*RSe453p - 1./162.*m33*m46*RSe561p*RSp563e + 1./162.*
         m33*m46*RSe561p*RSp453e + 1./162.*m33*m46*RSe451p*RSp563e - 1./
         162.*m33*m46*RSe451p*RSp453e - 1./162.*m33*m46*RSp561e*RSe563p
          + 1./162.*m33*m46*RSp561e*RSe453p - 1./162.*m33*m46*RSp561e*
         RSe561p );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * ( 1./162.*m33*m46*
         RSp561e*RSe451p + 1./162.*m33*m46*RSp451e*RSe563p - 1./162.*
         m33*m46*RSp451e*RSe453p + 1./162.*m33*m46*RSp451e*RSe561p - 1./
         162.*m33*m46*RSp451e*RSe451p + 1./36.*m33*m44*RSp563e*RSe563p
          + 1./108.*m33*m44*RSp563e*RSe453p + 1./108.*m33*m44*RSp453e*
         RSe563p + 1./108.*m33*m44*RSp453e*RSe453p + 1./108.*m33*m44*
         RSe561p*RSp563e + 1./324.*m33*m44*RSe561p*RSp453e + 1./324.*
         m33*m44*RSe451p*RSp563e + 1./324.*m33*m44*RSe451p*RSp453e + 1./
         108.*m33*m44*RSp561e*RSe563p + 1./324.*m33*m44*RSp561e*RSe453p
          + 1./108.*m33*m44*RSp561e*RSe561p + 1./324.*m33*m44*RSp561e*
         RSe451p + 1./324.*m33*m44*RSp451e*RSe563p + 1./324.*m33*m44*
         RSp451e*RSe453p + 1./324.*m33*m44*RSp451e*RSe561p + 1./324.*
         m33*m44*RSp451e*RSe451p + 1./81.*m13*me2*RSp563e*RSe563p - 1./
         81.*m13*me2*RSp563e*RSe453p - 1./81.*m13*me2*RSp453e*RSe563p
          + 1./81.*m13*me2*RSp453e*RSe453p - 1./81.*m13*me2*RSe561p*
         RSp563e );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * ( 1./81.*m13*me2*
         RSe561p*RSp453e + 1./81.*m13*me2*RSe451p*RSp563e - 1./81.*m13*
         me2*RSe451p*RSp453e - 1./81.*m13*me2*RSp561e*RSe563p + 1./81.*
         m13*me2*RSp561e*RSe453p + 1./81.*m13*me2*RSp561e*RSe561p - 1./
         81.*m13*me2*RSp561e*RSe451p + 1./81.*m13*me2*RSp451e*RSe563p
          - 1./81.*m13*me2*RSp451e*RSe453p - 1./81.*m13*me2*RSp451e*
         RSe561p + 1./81.*m13*me2*RSp451e*RSe451p - 1./162.*m13*m66*
         RSp563e*RSe563p - 1./162.*m13*m66*RSp563e*RSe453p - 1./162.*
         m13*m66*RSp453e*RSe563p - 1./54.*m13*m66*RSp453e*RSe453p + 1./
         162.*m13*m66*RSe561p*RSp563e + 1./162.*m13*m66*RSe561p*RSp453e
          + 1./162.*m13*m66*RSe451p*RSp563e + 1./54.*m13*m66*RSe451p*
         RSp453e + 1./162.*m13*m66*RSp561e*RSe563p + 1./162.*m13*m66*
         RSp561e*RSe453p - 1./162.*m13*m66*RSp561e*RSe561p - 1./162.*
         m13*m66*RSp561e*RSe451p + 1./162.*m13*m66*RSp451e*RSe563p + 1./
         54.*m13*m66*RSp451e*RSe453p - 1./162.*m13*m66*RSp451e*RSe561p
          - 1./54.*m13*m66*RSp451e*RSe451p );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * ( 1./81.*m13*m46*
         RSp563e*RSe563p - 1./81.*m13*m46*RSp563e*RSe453p - 1./81.*m13*
         m46*RSp453e*RSe563p + 1./81.*m13*m46*RSp453e*RSe453p - 1./81.*
         m13*m46*RSe561p*RSp563e + 1./81.*m13*m46*RSe561p*RSp453e + 1./
         81.*m13*m46*RSe451p*RSp563e - 1./81.*m13*m46*RSe451p*RSp453e
          - 1./81.*m13*m46*RSp561e*RSe563p + 1./81.*m13*m46*RSp561e*
         RSe453p + 1./81.*m13*m46*RSp561e*RSe561p - 1./81.*m13*m46*
         RSp561e*RSe451p + 1./81.*m13*m46*RSp451e*RSe563p - 1./81.*m13*
         m46*RSp451e*RSe453p - 1./81.*m13*m46*RSp451e*RSe561p + 1./81.*
         m13*m46*RSp451e*RSe451p - 1./54.*m13*m44*RSp563e*RSe563p - 1./
         162.*m13*m44*RSp563e*RSe453p - 1./162.*m13*m44*RSp453e*RSe563p
          - 1./162.*m13*m44*RSp453e*RSe453p + 1./54.*m13*m44*RSe561p*
         RSp563e + 1./162.*m13*m44*RSe561p*RSp453e + 1./162.*m13*m44*
         RSe451p*RSp563e + 1./162.*m13*m44*RSe451p*RSp453e + 1./54.*m13
         *m44*RSp561e*RSe563p + 1./162.*m13*m44*RSp561e*RSe453p - 1./54.
         *m13*m44*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * (  - 1./162.*m13*
         m44*RSp561e*RSe451p + 1./162.*m13*m44*RSp451e*RSe563p + 1./162.
         *m13*m44*RSp451e*RSe453p - 1./162.*m13*m44*RSp451e*RSe561p - 1.
         /162.*m13*m44*RSp451e*RSe451p - 1./162.*m11*me2*RSp563e*
         RSe563p + 1./162.*m11*me2*RSp563e*RSe453p + 1./162.*m11*me2*
         RSp453e*RSe563p - 1./162.*m11*me2*RSp453e*RSe453p - 1./162.*
         m11*me2*RSe561p*RSp563e + 1./162.*m11*me2*RSe561p*RSp453e + 1./
         162.*m11*me2*RSe451p*RSp563e - 1./162.*m11*me2*RSe451p*RSp453e
          - 1./162.*m11*me2*RSp561e*RSe563p + 1./162.*m11*me2*RSp561e*
         RSe453p - 1./54.*m11*me2*RSp561e*RSe561p + 1./54.*m11*me2*
         RSp561e*RSe451p + 1./162.*m11*me2*RSp451e*RSe563p - 1./162.*
         m11*me2*RSp451e*RSe453p + 1./54.*m11*me2*RSp451e*RSe561p - 1./
         54.*m11*me2*RSp451e*RSe451p + 1./324.*m11*m66*RSp563e*RSe563p
          + 1./324.*m11*m66*RSp563e*RSe453p + 1./324.*m11*m66*RSp453e*
         RSe563p + 1./108.*m11*m66*RSp453e*RSe453p + 1./324.*m11*m66*
         RSe561p*RSp563e );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * ( 1./324.*m11*m66*
         RSe561p*RSp453e + 1./324.*m11*m66*RSe451p*RSp563e + 1./108.*
         m11*m66*RSe451p*RSp453e + 1./324.*m11*m66*RSp561e*RSe563p + 1./
         324.*m11*m66*RSp561e*RSe453p + 1./108.*m11*m66*RSp561e*RSe561p
          + 1./108.*m11*m66*RSp561e*RSe451p + 1./324.*m11*m66*RSp451e*
         RSe563p + 1./108.*m11*m66*RSp451e*RSe453p + 1./108.*m11*m66*
         RSp451e*RSe561p + 1./36.*m11*m66*RSp451e*RSe451p - 1./162.*m11
         *m46*RSp563e*RSe563p + 1./162.*m11*m46*RSp563e*RSe453p + 1./
         162.*m11*m46*RSp453e*RSe563p - 1./162.*m11*m46*RSp453e*RSe453p
          - 1./162.*m11*m46*RSe561p*RSp563e + 1./162.*m11*m46*RSe561p*
         RSp453e + 1./162.*m11*m46*RSe451p*RSp563e - 1./162.*m11*m46*
         RSe451p*RSp453e - 1./162.*m11*m46*RSp561e*RSe563p + 1./162.*
         m11*m46*RSp561e*RSe453p - 1./54.*m11*m46*RSp561e*RSe561p + 1./
         54.*m11*m46*RSp561e*RSe451p + 1./162.*m11*m46*RSp451e*RSe563p
          - 1./162.*m11*m46*RSp451e*RSe453p + 1./54.*m11*m46*RSp451e*
         RSe561p );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * (  - 1./54.*m11*m46
         *RSp451e*RSe451p + 1./108.*m11*m44*RSp563e*RSe563p + 1./324.*
         m11*m44*RSp563e*RSe453p + 1./324.*m11*m44*RSp453e*RSe563p + 1./
         324.*m11*m44*RSp453e*RSe453p + 1./108.*m11*m44*RSe561p*RSp563e
          + 1./324.*m11*m44*RSe561p*RSp453e + 1./324.*m11*m44*RSe451p*
         RSp563e + 1./324.*m11*m44*RSe451p*RSp453e + 1./108.*m11*m44*
         RSp561e*RSe563p + 1./324.*m11*m44*RSp561e*RSe453p + 1./36.*m11
         *m44*RSp561e*RSe561p + 1./108.*m11*m44*RSp561e*RSe451p + 1./
         324.*m11*m44*RSp451e*RSe563p + 1./324.*m11*m44*RSp451e*RSe453p
          + 1./108.*m11*m44*RSp451e*RSe561p + 1./108.*m11*m44*RSp451e*
         RSe451p );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,L) * ( 1./162.*pow(me2,2)*
         pow(RSe563p,2) - 1./81.*pow(me2,2)*RSe453p*RSe563p + 1./162.*
         pow(me2,2)*pow(RSe453p,2) - 1./81.*pow(me2,2)*RSe561p*RSe563p
          + 1./81.*pow(me2,2)*RSe561p*RSe453p + 1./162.*pow(me2,2)*pow(
         RSe561p,2) + 1./81.*pow(me2,2)*RSe451p*RSe563p - 1./81.*pow(
         me2,2)*RSe451p*RSe453p - 1./81.*pow(me2,2)*RSe451p*RSe561p + 1.
         /162.*pow(me2,2)*pow(RSe451p,2) - 1./324.*m66*me2*pow(
         RSe563p,2) - 1./162.*m66*me2*RSe453p*RSe563p - 1./108.*m66*me2
         *pow(RSe453p,2) + 1./162.*m66*me2*RSe561p*RSe563p + 1./162.*
         m66*me2*RSe561p*RSe453p - 1./324.*m66*me2*pow(RSe561p,2) + 1./
         162.*m66*me2*RSe451p*RSe563p + 1./54.*m66*me2*RSe451p*RSe453p
          - 1./162.*m66*me2*RSe451p*RSe561p - 1./108.*m66*me2*pow(
         RSe451p,2) + 1./162.*m46*me2*pow(RSe563p,2) - 1./81.*m46*me2*
         RSe453p*RSe563p + 1./162.*m46*me2*pow(RSe453p,2) - 1./81.*m46*
         me2*RSe561p*RSe563p + 1./81.*m46*me2*RSe561p*RSe453p + 1./162.
         *m46*me2*pow(RSe561p,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,L) * ( 1./81.*m46*me2*
         RSe451p*RSe563p - 1./81.*m46*me2*RSe451p*RSe453p - 1./81.*m46*
         me2*RSe451p*RSe561p + 1./162.*m46*me2*pow(RSe451p,2) - 1./108.
         *m44*me2*pow(RSe563p,2) - 1./162.*m44*me2*RSe453p*RSe563p - 1./
         324.*m44*me2*pow(RSe453p,2) + 1./54.*m44*me2*RSe561p*RSe563p
          + 1./162.*m44*me2*RSe561p*RSe453p - 1./108.*m44*me2*pow(
         RSe561p,2) + 1./162.*m44*me2*RSe451p*RSe563p + 1./162.*m44*me2
         *RSe451p*RSe453p - 1./162.*m44*me2*RSe451p*RSe561p - 1./324.*
         m44*me2*pow(RSe451p,2) - 1./108.*m33*me2*pow(RSe563p,2) + 1./
         54.*m33*me2*RSe453p*RSe563p - 1./108.*m33*me2*pow(RSe453p,2)
          - 1./162.*m33*me2*RSe561p*RSe563p + 1./162.*m33*me2*RSe561p*
         RSe453p - 1./324.*m33*me2*pow(RSe561p,2) + 1./162.*m33*me2*
         RSe451p*RSe563p - 1./162.*m33*me2*RSe451p*RSe453p + 1./162.*
         m33*me2*RSe451p*RSe561p - 1./324.*m33*me2*pow(RSe451p,2) + 1./
         216.*m33*m66*pow(RSe563p,2) + 1./108.*m33*m66*RSe453p*RSe563p
          + 1./72.*m33*m66*pow(RSe453p,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,L) * ( 1./324.*m33*m66*
         RSe561p*RSe563p + 1./324.*m33*m66*RSe561p*RSe453p + 1./648.*
         m33*m66*pow(RSe561p,2) + 1./324.*m33*m66*RSe451p*RSe563p + 1./
         108.*m33*m66*RSe451p*RSe453p + 1./324.*m33*m66*RSe451p*RSe561p
          + 1./216.*m33*m66*pow(RSe451p,2) - 1./108.*m33*m46*pow(
         RSe563p,2) + 1./54.*m33*m46*RSe453p*RSe563p - 1./108.*m33*m46*
         pow(RSe453p,2) - 1./162.*m33*m46*RSe561p*RSe563p + 1./162.*m33
         *m46*RSe561p*RSe453p - 1./324.*m33*m46*pow(RSe561p,2) + 1./162.
         *m33*m46*RSe451p*RSe563p - 1./162.*m33*m46*RSe451p*RSe453p + 1.
         /162.*m33*m46*RSe451p*RSe561p - 1./324.*m33*m46*pow(RSe451p,2)
          + 1./72.*m33*m44*pow(RSe563p,2) + 1./108.*m33*m44*RSe453p*
         RSe563p + 1./216.*m33*m44*pow(RSe453p,2) + 1./108.*m33*m44*
         RSe561p*RSe563p + 1./324.*m33*m44*RSe561p*RSe453p + 1./216.*
         m33*m44*pow(RSe561p,2) + 1./324.*m33*m44*RSe451p*RSe563p + 1./
         324.*m33*m44*RSe451p*RSe453p + 1./324.*m33*m44*RSe451p*RSe561p
          + 1./648.*m33*m44*pow(RSe451p,2) );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,L) * ( 1./162.*m13*me2*pow(
         RSe563p,2) - 1./81.*m13*me2*RSe453p*RSe563p + 1./162.*m13*me2*
         pow(RSe453p,2) - 1./81.*m13*me2*RSe561p*RSe563p + 1./81.*m13*
         me2*RSe561p*RSe453p + 1./162.*m13*me2*pow(RSe561p,2) + 1./81.*
         m13*me2*RSe451p*RSe563p - 1./81.*m13*me2*RSe451p*RSe453p - 1./
         81.*m13*me2*RSe451p*RSe561p + 1./162.*m13*me2*pow(RSe451p,2)
          - 1./324.*m13*m66*pow(RSe563p,2) - 1./162.*m13*m66*RSe453p*
         RSe563p - 1./108.*m13*m66*pow(RSe453p,2) + 1./162.*m13*m66*
         RSe561p*RSe563p + 1./162.*m13*m66*RSe561p*RSe453p - 1./324.*
         m13*m66*pow(RSe561p,2) + 1./162.*m13*m66*RSe451p*RSe563p + 1./
         54.*m13*m66*RSe451p*RSe453p - 1./162.*m13*m66*RSe451p*RSe561p
          - 1./108.*m13*m66*pow(RSe451p,2) + 1./162.*m13*m46*pow(
         RSe563p,2) - 1./81.*m13*m46*RSe453p*RSe563p + 1./162.*m13*m46*
         pow(RSe453p,2) - 1./81.*m13*m46*RSe561p*RSe563p + 1./81.*m13*
         m46*RSe561p*RSe453p + 1./162.*m13*m46*pow(RSe561p,2) + 1./81.*
         m13*m46*RSe451p*RSe563p );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,L) * (  - 1./81.*m13*m46*
         RSe451p*RSe453p - 1./81.*m13*m46*RSe451p*RSe561p + 1./162.*m13
         *m46*pow(RSe451p,2) - 1./108.*m13*m44*pow(RSe563p,2) - 1./162.
         *m13*m44*RSe453p*RSe563p - 1./324.*m13*m44*pow(RSe453p,2) + 1./
         54.*m13*m44*RSe561p*RSe563p + 1./162.*m13*m44*RSe561p*RSe453p
          - 1./108.*m13*m44*pow(RSe561p,2) + 1./162.*m13*m44*RSe451p*
         RSe563p + 1./162.*m13*m44*RSe451p*RSe453p - 1./162.*m13*m44*
         RSe451p*RSe561p - 1./324.*m13*m44*pow(RSe451p,2) - 1./324.*m11
         *me2*pow(RSe563p,2) + 1./162.*m11*me2*RSe453p*RSe563p - 1./324.
         *m11*me2*pow(RSe453p,2) - 1./162.*m11*me2*RSe561p*RSe563p + 1./
         162.*m11*me2*RSe561p*RSe453p - 1./108.*m11*me2*pow(RSe561p,2)
          + 1./162.*m11*me2*RSe451p*RSe563p - 1./162.*m11*me2*RSe451p*
         RSe453p + 1./54.*m11*me2*RSe451p*RSe561p - 1./108.*m11*me2*
         pow(RSe451p,2) + 1./648.*m11*m66*pow(RSe563p,2) + 1./324.*m11*
         m66*RSe453p*RSe563p + 1./216.*m11*m66*pow(RSe453p,2) + 1./324.
         *m11*m66*RSe561p*RSe563p );

      M13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,L) * ( 1./324.*m11*m66*
         RSe561p*RSe453p + 1./216.*m11*m66*pow(RSe561p,2) + 1./324.*m11
         *m66*RSe451p*RSe563p + 1./108.*m11*m66*RSe451p*RSe453p + 1./
         108.*m11*m66*RSe451p*RSe561p + 1./72.*m11*m66*pow(RSe451p,2)
          - 1./324.*m11*m46*pow(RSe563p,2) + 1./162.*m11*m46*RSe453p*
         RSe563p - 1./324.*m11*m46*pow(RSe453p,2) - 1./162.*m11*m46*
         RSe561p*RSe563p + 1./162.*m11*m46*RSe561p*RSe453p - 1./108.*
         m11*m46*pow(RSe561p,2) + 1./162.*m11*m46*RSe451p*RSe563p - 1./
         162.*m11*m46*RSe451p*RSe453p + 1./54.*m11*m46*RSe451p*RSe561p
          - 1./108.*m11*m46*pow(RSe451p,2) + 1./216.*m11*m44*pow(
         RSe563p,2) + 1./324.*m11*m44*RSe453p*RSe563p + 1./648.*m11*m44
         *pow(RSe453p,2) + 1./108.*m11*m44*RSe561p*RSe563p + 1./324.*
         m11*m44*RSe561p*RSe453p + 1./72.*m11*m44*pow(RSe561p,2) + 1./
         324.*m11*m44*RSe451p*RSe563p + 1./324.*m11*m44*RSe451p*RSe453p
          + 1./108.*m11*m44*RSe451p*RSe561p + 1./216.*m11*m44*pow(
         RSe451p,2) );

      M13p6x23RV +=  + Ab(m55,mu2)*AbV(m11,L) * (  - 1./54.*m55*pow(
         RS146pe,2) + 1./162.*m55*RS546pe*pow(RS146pe,2) );

      M13p6x23RV +=  + Ab(m55,mu2)*AbV(m15,L) * (  - 1./20.*m55 + 1./60.
         *m55*RS546pe + 1./36.*m35 - 1./108.*m35*RS546pe + 7./360.*m33
          - 7./1080.*m33*RS546pe + 13./180.*m15 - 13./540.*m15*RS546pe
          - 1./15.*m13 + 1./45.*m13*RS546pe - 1./360.*m11 + 1./1080.*
         m11*RS546pe );

      M13p6x23RV +=  + Ab(m55,mu2)*AbV(m33,L) * (  - 1./54.*m55*pow(
         RS346pe,2) + 1./27.*m55*RS146pe*RS346pe + 1./162.*m55*RS546pe*
         pow(RS346pe,2) - 1./81.*m55*RS546pe*RS146pe*RS346pe );

      M13p6x23RV +=  + Ab(m55,mu2)*AbV(m35,L) * (  - 1./20.*m55 + 1./60.
         *m55*RS546pe + 13./180.*m35 - 13./540.*m35*RS546pe - 1./360.*
         m33 + 1./1080.*m33*RS546pe + 1./36.*m15 - 1./108.*m15*RS546pe
          - 1./15.*m13 + 1./45.*m13*RS546pe + 7./360.*m11 - 7./1080.*
         m11*RS546pe );

      M13p6x23RV +=  + Ab(m55,mu2)*AbV(mp2,L) * (  - 1./27.*m55*RSp463e
         *RS346pe - 1./27.*m55*RSp463e*RSe463p - 1./54.*m55*pow(
         RSp463e,2) + 1./27.*m55*RS146pe*RSp463e + 1./81.*m55*RS546pe*
         RSp463e*RS346pe + 1./81.*m55*RS546pe*RSp463e*RSe463p + 1./162.
         *m55*RS546pe*pow(RSp463e,2) - 1./81.*m55*RS546pe*RS146pe*
         RSp463e + 1./27.*m55*RSe461p*RSp463e - 1./81.*m55*RSe461p*
         RS546pe*RSp463e + 1./27.*m55*RSp461e*RS346pe + 1./27.*m55*
         RSp461e*RSe463p + 1./27.*m55*RSp461e*RSp463e - 1./27.*m55*
         RSp461e*RS146pe - 1./81.*m55*RSp461e*RS546pe*RS346pe - 1./81.*
         m55*RSp461e*RS546pe*RSe463p - 1./81.*m55*RSp461e*RS546pe*
         RSp463e + 1./81.*m55*RSp461e*RS546pe*RS146pe - 1./27.*m55*
         RSp461e*RSe461p + 1./81.*m55*RSp461e*RSe461p*RS546pe - 1./54.*
         m55*pow(RSp461e,2) + 1./162.*m55*pow(RSp461e,2)*RS546pe );

      M13p6x23RV +=  + Ab(m55,mu2)*AbV(me2,L) * (  - 1./27.*m55*RSe463p
         *RS346pe - 1./54.*m55*pow(RSe463p,2) + 1./27.*m55*RS146pe*
         RSe463p + 1./81.*m55*RS546pe*RSe463p*RS346pe + 1./162.*m55*
         RS546pe*pow(RSe463p,2) - 1./81.*m55*RS546pe*RS146pe*RSe463p + 
         1./27.*m55*RSe461p*RS346pe + 1./27.*m55*RSe461p*RSe463p - 1./
         27.*m55*RSe461p*RS146pe - 1./81.*m55*RSe461p*RS546pe*RS346pe
          - 1./81.*m55*RSe461p*RS546pe*RSe463p + 1./81.*m55*RSe461p*
         RS546pe*RS146pe - 1./54.*m55*pow(RSe461p,2) + 1./162.*m55*pow(
         RSe461p,2)*RS546pe );

      M13p6x23RV +=  + Ab(m55,mu2)*BbV(m11,m33,L) * ( 1./54.*m33*m55*
         RS146pe*RS346pe - 1./162.*m33*m55*RS546pe*RS146pe*RS346pe + 1./
         27.*m13*m55*RS146pe*RS346pe - 1./81.*m13*m55*RS546pe*RS146pe*
         RS346pe + 1./18.*m11*m55*RS146pe*RS346pe - 1./54.*m11*m55*
         RS546pe*RS146pe*RS346pe );

      M13p6x23RV +=  + Ab(m55,mu2)*BbV(m11,mp2,L) * ( 1./54.*m33*m55*
         RS146pe*RSp463e - 1./162.*m33*m55*RS546pe*RS146pe*RSp463e + 1./
         54.*m33*m55*RSp461e*RS146pe - 1./162.*m33*m55*RSp461e*RS546pe*
         RS146pe + 1./27.*m13*m55*RS146pe*RSp463e - 1./81.*m13*m55*
         RS546pe*RS146pe*RSp463e - 1./27.*m13*m55*RSp461e*RS146pe + 1./
         81.*m13*m55*RSp461e*RS546pe*RS146pe + 1./18.*m11*m55*RS146pe*
         RSp463e - 1./54.*m11*m55*RS546pe*RS146pe*RSp463e + 1./54.*m11*
         m55*RSp461e*RS146pe - 1./162.*m11*m55*RSp461e*RS546pe*RS146pe
          );

      M13p6x23RV +=  + Ab(m55,mu2)*BbV(m11,me2,L) * ( 1./54.*m33*m55*
         RS146pe*RSe463p - 1./162.*m33*m55*RS546pe*RS146pe*RSe463p + 1./
         54.*m33*m55*RSe461p*RS146pe - 1./162.*m33*m55*RSe461p*RS546pe*
         RS146pe + 1./27.*m13*m55*RS146pe*RSe463p - 1./81.*m13*m55*
         RS546pe*RS146pe*RSe463p - 1./27.*m13*m55*RSe461p*RS146pe + 1./
         81.*m13*m55*RSe461p*RS546pe*RS146pe + 1./18.*m11*m55*RS146pe*
         RSe463p - 1./54.*m11*m55*RS546pe*RS146pe*RSe463p + 1./54.*m11*
         m55*RSe461p*RS146pe - 1./162.*m11*m55*RSe461p*RS546pe*RS146pe
          );

      M13p6x23RV +=  + Ab(m55,mu2)*BbV(m11,L) * ( 1./108.*m33*m55*pow(
         RS146pe,2) - 1./324.*m33*m55*RS546pe*pow(RS146pe,2) - 1./54.*
         m13*m55*pow(RS146pe,2) + 1./162.*m13*m55*RS546pe*pow(
         RS146pe,2) + 1./108.*m11*m55*pow(RS146pe,2) - 1./324.*m11*m55*
         RS546pe*pow(RS146pe,2) );

      M13p6x23RV +=  + Ab(m55,mu2)*BbV(m15,L) * ( 1./144.*pow(m55,2) - 
         1./432.*pow(m55,2)*RS546pe + 1./144.*m33*m55 - 1./432.*m33*m55
         *RS546pe - 1./36.*m15*m55 + 1./108.*m15*m55*RS546pe - 1./72.*
         m15*m33 + 1./216.*m15*m33*RS546pe + 1./36.*pow(m15,2) - 1./108.
         *pow(m15,2)*RS546pe - 1./72.*m13*m55 + 1./216.*m13*m55*RS546pe
          + 1./36.*m13*m15 - 1./108.*m13*m15*RS546pe + 1./48.*m11*m55
          - 1./144.*m11*m55*RS546pe + 1./144.*m11*m33 - 1./432.*m11*m33
         *RS546pe - 1./24.*m11*m15 + 1./72.*m11*m15*RS546pe - 1./72.*
         m11*m13 + 1./216.*m11*m13*RS546pe + 1./72.*pow(m11,2) - 1./216.
         *pow(m11,2)*RS546pe );

      M13p6x23RV +=  + Ab(m55,mu2)*BbV(m33,mp2,L) * ( 1./54.*m33*m55*
         RSp463e*RS346pe - 1./162.*m33*m55*RS546pe*RSp463e*RS346pe + 1./
         18.*m33*m55*RSp461e*RS346pe - 1./54.*m33*m55*RSp461e*RS546pe*
         RS346pe - 1./27.*m13*m55*RSp463e*RS346pe + 1./81.*m13*m55*
         RS546pe*RSp463e*RS346pe + 1./27.*m13*m55*RSp461e*RS346pe - 1./
         81.*m13*m55*RSp461e*RS546pe*RS346pe + 1./54.*m11*m55*RSp463e*
         RS346pe - 1./162.*m11*m55*RS546pe*RSp463e*RS346pe + 1./54.*m11
         *m55*RSp461e*RS346pe - 1./162.*m11*m55*RSp461e*RS546pe*RS346pe
          );

      M13p6x23RV +=  + Ab(m55,mu2)*BbV(m33,me2,L) * ( 1./54.*m33*m55*
         RSe463p*RS346pe - 1./162.*m33*m55*RS546pe*RSe463p*RS346pe + 1./
         18.*m33*m55*RSe461p*RS346pe - 1./54.*m33*m55*RSe461p*RS546pe*
         RS346pe - 1./27.*m13*m55*RSe463p*RS346pe + 1./81.*m13*m55*
         RS546pe*RSe463p*RS346pe + 1./27.*m13*m55*RSe461p*RS346pe - 1./
         81.*m13*m55*RSe461p*RS546pe*RS346pe + 1./54.*m11*m55*RSe463p*
         RS346pe - 1./162.*m11*m55*RS546pe*RSe463p*RS346pe + 1./54.*m11
         *m55*RSe461p*RS346pe - 1./162.*m11*m55*RSe461p*RS546pe*RS346pe
          );

      M13p6x23RV +=  + Ab(m55,mu2)*BbV(m33,L) * ( 1./108.*m33*m55*pow(
         RS346pe,2) - 1./324.*m33*m55*RS546pe*pow(RS346pe,2) - 1./54.*
         m13*m55*pow(RS346pe,2) + 1./162.*m13*m55*RS546pe*pow(
         RS346pe,2) + 1./108.*m11*m55*pow(RS346pe,2) - 1./324.*m11*m55*
         RS546pe*pow(RS346pe,2) );

      M13p6x23RV +=  + Ab(m55,mu2)*BbV(m35,L) * ( 1./144.*pow(m55,2) - 
         1./432.*pow(m55,2)*RS546pe - 1./36.*m35*m55 + 1./108.*m35*m55*
         RS546pe + 1./36.*pow(m35,2) - 1./108.*pow(m35,2)*RS546pe + 1./
         48.*m33*m55 - 1./144.*m33*m55*RS546pe - 1./24.*m33*m35 + 1./72.
         *m33*m35*RS546pe + 1./72.*pow(m33,2) - 1./216.*pow(m33,2)*
         RS546pe - 1./72.*m13*m55 + 1./216.*m13*m55*RS546pe + 1./36.*
         m13*m35 - 1./108.*m13*m35*RS546pe - 1./72.*m13*m33 + 1./216.*
         m13*m33*RS546pe + 1./144.*m11*m55 - 1./432.*m11*m55*RS546pe - 
         1./72.*m11*m35 + 1./216.*m11*m35*RS546pe + 1./144.*m11*m33 - 1.
         /432.*m11*m33*RS546pe );

      M13p6x23RV +=  + Ab(m55,mu2)*BbV(mp2,L) * (  - 1./54.*m55*mp2*
         pow(RSp463e,2) + 1./162.*m55*mp2*RS546pe*pow(RSp463e,2) + 1./
         27.*m55*mp2*RSp461e*RSp463e - 1./81.*m55*mp2*RSp461e*RS546pe*
         RSp463e - 1./54.*m55*mp2*pow(RSp461e,2) + 1./162.*m55*mp2*pow(
         RSp461e,2)*RS546pe + 1./36.*m33*m55*pow(RSp463e,2) - 1./108.*
         m33*m55*RS546pe*pow(RSp463e,2) + 1./54.*m33*m55*RSp461e*
         RSp463e - 1./162.*m33*m55*RSp461e*RS546pe*RSp463e + 1./108.*
         m33*m55*pow(RSp461e,2) - 1./324.*m33*m55*pow(RSp461e,2)*
         RS546pe - 1./54.*m13*m55*pow(RSp463e,2) + 1./162.*m13*m55*
         RS546pe*pow(RSp463e,2) + 1./27.*m13*m55*RSp461e*RSp463e - 1./
         81.*m13*m55*RSp461e*RS546pe*RSp463e - 1./54.*m13*m55*pow(
         RSp461e,2) + 1./162.*m13*m55*pow(RSp461e,2)*RS546pe + 1./108.*
         m11*m55*pow(RSp463e,2) - 1./324.*m11*m55*RS546pe*pow(
         RSp463e,2) + 1./54.*m11*m55*RSp461e*RSp463e - 1./162.*m11*m55*
         RSp461e*RS546pe*RSp463e + 1./36.*m11*m55*pow(RSp461e,2) - 1./
         108.*m11*m55*pow(RSp461e,2)*RS546pe );

      M13p6x23RV +=  + Ab(m55,mu2)*BbV(me2,mp2,L) * (  - 1./27.*m55*me2
         *RSp463e*RSe463p + 1./81.*m55*me2*RS546pe*RSp463e*RSe463p + 1./
         27.*m55*me2*RSe461p*RSp463e - 1./81.*m55*me2*RSe461p*RS546pe*
         RSp463e + 1./27.*m55*me2*RSp461e*RSe463p - 1./81.*m55*me2*
         RSp461e*RS546pe*RSe463p - 1./27.*m55*me2*RSp461e*RSe461p + 1./
         81.*m55*me2*RSp461e*RSe461p*RS546pe + 1./18.*m33*m55*RSp463e*
         RSe463p - 1./54.*m33*m55*RS546pe*RSp463e*RSe463p + 1./54.*m33*
         m55*RSe461p*RSp463e - 1./162.*m33*m55*RSe461p*RS546pe*RSp463e
          + 1./54.*m33*m55*RSp461e*RSe463p - 1./162.*m33*m55*RSp461e*
         RS546pe*RSe463p + 1./54.*m33*m55*RSp461e*RSe461p - 1./162.*m33
         *m55*RSp461e*RSe461p*RS546pe - 1./27.*m13*m55*RSp463e*RSe463p
          + 1./81.*m13*m55*RS546pe*RSp463e*RSe463p + 1./27.*m13*m55*
         RSe461p*RSp463e - 1./81.*m13*m55*RSe461p*RS546pe*RSp463e + 1./
         27.*m13*m55*RSp461e*RSe463p - 1./81.*m13*m55*RSp461e*RS546pe*
         RSe463p - 1./27.*m13*m55*RSp461e*RSe461p + 1./81.*m13*m55*
         RSp461e*RSe461p*RS546pe );

      M13p6x23RV +=  + Ab(m55,mu2)*BbV(me2,mp2,L) * ( 1./54.*m11*m55*
         RSp463e*RSe463p - 1./162.*m11*m55*RS546pe*RSp463e*RSe463p + 1./
         54.*m11*m55*RSe461p*RSp463e - 1./162.*m11*m55*RSe461p*RS546pe*
         RSp463e + 1./54.*m11*m55*RSp461e*RSe463p - 1./162.*m11*m55*
         RSp461e*RS546pe*RSe463p + 1./18.*m11*m55*RSp461e*RSe461p - 1./
         54.*m11*m55*RSp461e*RSe461p*RS546pe );

      M13p6x23RV +=  + Ab(m55,mu2)*BbV(me2,L) * (  - 1./54.*m55*me2*
         pow(RSe463p,2) + 1./162.*m55*me2*RS546pe*pow(RSe463p,2) + 1./
         27.*m55*me2*RSe461p*RSe463p - 1./81.*m55*me2*RSe461p*RS546pe*
         RSe463p - 1./54.*m55*me2*pow(RSe461p,2) + 1./162.*m55*me2*pow(
         RSe461p,2)*RS546pe + 1./36.*m33*m55*pow(RSe463p,2) - 1./108.*
         m33*m55*RS546pe*pow(RSe463p,2) + 1./54.*m33*m55*RSe461p*
         RSe463p - 1./162.*m33*m55*RSe461p*RS546pe*RSe463p + 1./108.*
         m33*m55*pow(RSe461p,2) - 1./324.*m33*m55*pow(RSe461p,2)*
         RS546pe - 1./54.*m13*m55*pow(RSe463p,2) + 1./162.*m13*m55*
         RS546pe*pow(RSe463p,2) + 1./27.*m13*m55*RSe461p*RSe463p - 1./
         81.*m13*m55*RSe461p*RS546pe*RSe463p - 1./54.*m13*m55*pow(
         RSe461p,2) + 1./162.*m13*m55*pow(RSe461p,2)*RS546pe + 1./108.*
         m11*m55*pow(RSe463p,2) - 1./324.*m11*m55*RS546pe*pow(
         RSe463p,2) + 1./54.*m11*m55*RSe461p*RSe463p - 1./162.*m11*m55*
         RSe461p*RS546pe*RSe463p + 1./36.*m11*m55*pow(RSe461p,2) - 1./
         108.*m11*m55*pow(RSe461p,2)*RS546pe );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(m11,L) * (  - 1./108.*m66*pow(
         RS145pe,2) - 1./162.*m66*RS146pe*RS145pe - 1./324.*m66*pow(
         RS146pe,2) + 1./162.*m56*pow(RS145pe,2) - 1./81.*m56*RS146pe*
         RS145pe + 1./162.*m56*pow(RS146pe,2) - 1./324.*m55*pow(
         RS145pe,2) - 1./162.*m55*RS146pe*RS145pe - 1./108.*m55*pow(
         RS146pe,2) - 1./324.*m33*pow(RS145pe,2) + 1./162.*m33*RS146pe*
         RS145pe - 1./324.*m33*pow(RS146pe,2) + 1./162.*m13*pow(
         RS145pe,2) - 1./81.*m13*RS146pe*RS145pe + 1./162.*m13*pow(
         RS146pe,2) - 1./81.*m11*RS145pe*RS345pe + 1./81.*m11*RS145pe*
         RS346pe + 1./81.*m11*RS145pe*RSe463p - 1./81.*m11*RS145pe*
         RSe453p + 1./81.*m11*RS145pe*RSp463e - 1./81.*m11*RS145pe*
         RSp453e + 1./324.*m11*pow(RS145pe,2) + 1./81.*m11*RS146pe*
         RS345pe - 1./81.*m11*RS146pe*RS346pe - 1./81.*m11*RS146pe*
         RSe463p + 1./81.*m11*RS146pe*RSe453p - 1./81.*m11*RS146pe*
         RSp463e + 1./81.*m11*RS146pe*RSp453e - 1./162.*m11*RS146pe*
         RS145pe );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(m11,L) * ( 1./324.*m11*pow(
         RS146pe,2) - 1./81.*m11*RSe461p*RS145pe + 1./81.*m11*RSe461p*
         RS146pe + 1./81.*m11*RSe451p*RS145pe - 1./81.*m11*RSe451p*
         RS146pe - 1./81.*m11*RSp461e*RS145pe + 1./81.*m11*RSp461e*
         RS146pe + 1./81.*m11*RSp451e*RS145pe - 1./81.*m11*RSp451e*
         RS146pe );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(m15,L) * ( 7./360.*m66 - 1./15.*
         m56 - 1./360.*m55 + 1./36.*m36 + 7./360.*m33 + 13./180.*m15 - 
         1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(m16,L) * (  - 1./360.*m66 - 1./
         15.*m56 + 7./360.*m55 + 1./36.*m35 + 7./360.*m33 + 13./180.*
         m16 - 1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(m33,L) * (  - 1./108.*m66*pow(
         RS345pe,2) - 1./162.*m66*RS346pe*RS345pe - 1./324.*m66*pow(
         RS346pe,2) + 1./54.*m66*RS145pe*RS345pe + 1./162.*m66*RS145pe*
         RS346pe + 1./162.*m66*RS146pe*RS345pe + 1./162.*m66*RS146pe*
         RS346pe + 1./162.*m56*pow(RS345pe,2) - 1./81.*m56*RS346pe*
         RS345pe + 1./162.*m56*pow(RS346pe,2) - 1./81.*m56*RS145pe*
         RS345pe + 1./81.*m56*RS145pe*RS346pe + 1./81.*m56*RS146pe*
         RS345pe - 1./81.*m56*RS146pe*RS346pe - 1./324.*m55*pow(
         RS345pe,2) - 1./162.*m55*RS346pe*RS345pe - 1./108.*m55*pow(
         RS346pe,2) + 1./162.*m55*RS145pe*RS345pe + 1./162.*m55*RS145pe
         *RS346pe + 1./162.*m55*RS146pe*RS345pe + 1./54.*m55*RS146pe*
         RS346pe + 1./324.*m33*pow(RS345pe,2) - 1./162.*m33*RS346pe*
         RS345pe + 1./324.*m33*pow(RS346pe,2) - 1./81.*m33*RSe463p*
         RS345pe + 1./81.*m33*RSe463p*RS346pe + 1./81.*m33*RSe453p*
         RS345pe - 1./81.*m33*RSe453p*RS346pe - 1./81.*m33*RSp463e*
         RS345pe );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(m33,L) * ( 1./81.*m33*RSp463e*
         RS346pe + 1./81.*m33*RSp453e*RS345pe - 1./81.*m33*RSp453e*
         RS346pe - 1./54.*m33*RS145pe*RS345pe + 1./54.*m33*RS145pe*
         RS346pe + 1./54.*m33*RS146pe*RS345pe - 1./54.*m33*RS146pe*
         RS346pe + 1./81.*m33*RSe461p*RS345pe - 1./81.*m33*RSe461p*
         RS346pe - 1./81.*m33*RSe451p*RS345pe + 1./81.*m33*RSe451p*
         RS346pe + 1./81.*m33*RSp461e*RS345pe - 1./81.*m33*RSp461e*
         RS346pe - 1./81.*m33*RSp451e*RS345pe + 1./81.*m33*RSp451e*
         RS346pe + 1./162.*m13*pow(RS345pe,2) - 1./81.*m13*RS346pe*
         RS345pe + 1./162.*m13*pow(RS346pe,2) - 1./81.*m13*RS145pe*
         RS345pe + 1./81.*m13*RS145pe*RS346pe + 1./81.*m13*RS146pe*
         RS345pe - 1./81.*m13*RS146pe*RS346pe - 1./324.*m11*pow(
         RS345pe,2) + 1./162.*m11*RS346pe*RS345pe - 1./324.*m11*pow(
         RS346pe,2) - 1./162.*m11*RS145pe*RS345pe + 1./162.*m11*RS145pe
         *RS346pe + 1./162.*m11*RS146pe*RS345pe - 1./162.*m11*RS146pe*
         RS346pe );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(m35,L) * ( 7./360.*m66 - 1./15.*
         m56 - 1./360.*m55 + 13./180.*m35 - 1./360.*m33 + 1./36.*m16 - 
         1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(m36,L) * (  - 1./360.*m66 - 1./
         15.*m56 + 7./360.*m55 + 13./180.*m36 - 1./360.*m33 + 1./36.*
         m15 - 1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(mp2,L) * (  - 1./81.*mp2*RSp463e
         *RS345pe + 1./81.*mp2*RSp463e*RS346pe + 1./81.*mp2*RSp463e*
         RSe463p - 1./81.*mp2*RSp463e*RSe453p + 1./81.*mp2*pow(
         RSp463e,2) + 1./81.*mp2*RSp453e*RS345pe - 1./81.*mp2*RSp453e*
         RS346pe - 1./81.*mp2*RSp453e*RSe463p + 1./81.*mp2*RSp453e*
         RSe453p - 2./81.*mp2*RSp453e*RSp463e + 1./81.*mp2*pow(
         RSp453e,2) + 1./81.*mp2*RS145pe*RSp463e - 1./81.*mp2*RS145pe*
         RSp453e - 1./81.*mp2*RS146pe*RSp463e + 1./81.*mp2*RS146pe*
         RSp453e - 1./81.*mp2*RSe461p*RSp463e + 1./81.*mp2*RSe461p*
         RSp453e + 1./81.*mp2*RSe451p*RSp463e - 1./81.*mp2*RSe451p*
         RSp453e + 1./81.*mp2*RSp461e*RS345pe - 1./81.*mp2*RSp461e*
         RS346pe - 1./81.*mp2*RSp461e*RSe463p + 1./81.*mp2*RSp461e*
         RSe453p - 2./81.*mp2*RSp461e*RSp463e + 2./81.*mp2*RSp461e*
         RSp453e - 1./81.*mp2*RSp461e*RS145pe + 1./81.*mp2*RSp461e*
         RS146pe + 1./81.*mp2*RSp461e*RSe461p - 1./81.*mp2*RSp461e*
         RSe451p );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(mp2,L) * ( 1./81.*mp2*pow(
         RSp461e,2) - 1./81.*mp2*RSp451e*RS345pe + 1./81.*mp2*RSp451e*
         RS346pe + 1./81.*mp2*RSp451e*RSe463p - 1./81.*mp2*RSp451e*
         RSe453p + 2./81.*mp2*RSp451e*RSp463e - 2./81.*mp2*RSp451e*
         RSp453e + 1./81.*mp2*RSp451e*RS145pe - 1./81.*mp2*RSp451e*
         RS146pe - 1./81.*mp2*RSp451e*RSe461p + 1./81.*mp2*RSp451e*
         RSe451p - 2./81.*mp2*RSp451e*RSp461e + 1./81.*mp2*pow(
         RSp451e,2) - 1./162.*m66*RSp463e*RS345pe - 1./162.*m66*RSp463e
         *RS346pe - 1./162.*m66*RSp463e*RSe463p - 1./162.*m66*RSp463e*
         RSe453p - 1./324.*m66*pow(RSp463e,2) - 1./54.*m66*RSp453e*
         RS345pe - 1./162.*m66*RSp453e*RS346pe - 1./162.*m66*RSp453e*
         RSe463p - 1./54.*m66*RSp453e*RSe453p - 1./162.*m66*RSp453e*
         RSp463e - 1./108.*m66*pow(RSp453e,2) + 1./162.*m66*RS145pe*
         RSp463e + 1./54.*m66*RS145pe*RSp453e + 1./162.*m66*RS146pe*
         RSp463e + 1./162.*m66*RS146pe*RSp453e + 1./162.*m66*RSe461p*
         RSp463e );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(mp2,L) * ( 1./162.*m66*RSe461p*
         RSp453e + 1./162.*m66*RSe451p*RSp463e + 1./54.*m66*RSe451p*
         RSp453e + 1./162.*m66*RSp461e*RS345pe + 1./162.*m66*RSp461e*
         RS346pe + 1./162.*m66*RSp461e*RSe463p + 1./162.*m66*RSp461e*
         RSe453p + 1./162.*m66*RSp461e*RSp463e + 1./162.*m66*RSp461e*
         RSp453e - 1./162.*m66*RSp461e*RS145pe - 1./162.*m66*RSp461e*
         RS146pe - 1./162.*m66*RSp461e*RSe461p - 1./162.*m66*RSp461e*
         RSe451p - 1./324.*m66*pow(RSp461e,2) + 1./54.*m66*RSp451e*
         RS345pe + 1./162.*m66*RSp451e*RS346pe + 1./162.*m66*RSp451e*
         RSe463p + 1./54.*m66*RSp451e*RSe453p + 1./162.*m66*RSp451e*
         RSp463e + 1./54.*m66*RSp451e*RSp453e - 1./54.*m66*RSp451e*
         RS145pe - 1./162.*m66*RSp451e*RS146pe - 1./162.*m66*RSp451e*
         RSe461p - 1./54.*m66*RSp451e*RSe451p - 1./162.*m66*RSp451e*
         RSp461e - 1./108.*m66*pow(RSp451e,2) - 1./81.*m56*RSp463e*
         RS345pe + 1./81.*m56*RSp463e*RS346pe + 1./81.*m56*RSp463e*
         RSe463p );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(mp2,L) * (  - 1./81.*m56*RSp463e
         *RSe453p + 1./162.*m56*pow(RSp463e,2) + 1./81.*m56*RSp453e*
         RS345pe - 1./81.*m56*RSp453e*RS346pe - 1./81.*m56*RSp453e*
         RSe463p + 1./81.*m56*RSp453e*RSe453p - 1./81.*m56*RSp453e*
         RSp463e + 1./162.*m56*pow(RSp453e,2) + 1./81.*m56*RS145pe*
         RSp463e - 1./81.*m56*RS145pe*RSp453e - 1./81.*m56*RS146pe*
         RSp463e + 1./81.*m56*RS146pe*RSp453e - 1./81.*m56*RSe461p*
         RSp463e + 1./81.*m56*RSe461p*RSp453e + 1./81.*m56*RSe451p*
         RSp463e - 1./81.*m56*RSe451p*RSp453e + 1./81.*m56*RSp461e*
         RS345pe - 1./81.*m56*RSp461e*RS346pe - 1./81.*m56*RSp461e*
         RSe463p + 1./81.*m56*RSp461e*RSe453p - 1./81.*m56*RSp461e*
         RSp463e + 1./81.*m56*RSp461e*RSp453e - 1./81.*m56*RSp461e*
         RS145pe + 1./81.*m56*RSp461e*RS146pe + 1./81.*m56*RSp461e*
         RSe461p - 1./81.*m56*RSp461e*RSe451p + 1./162.*m56*pow(
         RSp461e,2) - 1./81.*m56*RSp451e*RS345pe + 1./81.*m56*RSp451e*
         RS346pe );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(mp2,L) * ( 1./81.*m56*RSp451e*
         RSe463p - 1./81.*m56*RSp451e*RSe453p + 1./81.*m56*RSp451e*
         RSp463e - 1./81.*m56*RSp451e*RSp453e + 1./81.*m56*RSp451e*
         RS145pe - 1./81.*m56*RSp451e*RS146pe - 1./81.*m56*RSp451e*
         RSe461p + 1./81.*m56*RSp451e*RSe451p - 1./81.*m56*RSp451e*
         RSp461e + 1./162.*m56*pow(RSp451e,2) - 1./162.*m55*RSp463e*
         RS345pe - 1./54.*m55*RSp463e*RS346pe - 1./54.*m55*RSp463e*
         RSe463p - 1./162.*m55*RSp463e*RSe453p - 1./108.*m55*pow(
         RSp463e,2) - 1./162.*m55*RSp453e*RS345pe - 1./162.*m55*RSp453e
         *RS346pe - 1./162.*m55*RSp453e*RSe463p - 1./162.*m55*RSp453e*
         RSe453p - 1./162.*m55*RSp453e*RSp463e - 1./324.*m55*pow(
         RSp453e,2) + 1./162.*m55*RS145pe*RSp463e + 1./162.*m55*RS145pe
         *RSp453e + 1./54.*m55*RS146pe*RSp463e + 1./162.*m55*RS146pe*
         RSp453e + 1./54.*m55*RSe461p*RSp463e + 1./162.*m55*RSe461p*
         RSp453e + 1./162.*m55*RSe451p*RSp463e + 1./162.*m55*RSe451p*
         RSp453e );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(mp2,L) * ( 1./162.*m55*RSp461e*
         RS345pe + 1./54.*m55*RSp461e*RS346pe + 1./54.*m55*RSp461e*
         RSe463p + 1./162.*m55*RSp461e*RSe453p + 1./54.*m55*RSp461e*
         RSp463e + 1./162.*m55*RSp461e*RSp453e - 1./162.*m55*RSp461e*
         RS145pe - 1./54.*m55*RSp461e*RS146pe - 1./54.*m55*RSp461e*
         RSe461p - 1./162.*m55*RSp461e*RSe451p - 1./108.*m55*pow(
         RSp461e,2) + 1./162.*m55*RSp451e*RS345pe + 1./162.*m55*RSp451e
         *RS346pe + 1./162.*m55*RSp451e*RSe463p + 1./162.*m55*RSp451e*
         RSe453p + 1./162.*m55*RSp451e*RSp463e + 1./162.*m55*RSp451e*
         RSp453e - 1./162.*m55*RSp451e*RS145pe - 1./162.*m55*RSp451e*
         RS146pe - 1./162.*m55*RSp451e*RSe461p - 1./162.*m55*RSp451e*
         RSe451p - 1./162.*m55*RSp451e*RSp461e - 1./324.*m55*pow(
         RSp451e,2) + 1./54.*m33*RSp463e*RS345pe - 1./54.*m33*RSp463e*
         RS346pe - 1./54.*m33*RSp463e*RSe463p + 1./54.*m33*RSp463e*
         RSe453p - 1./108.*m33*pow(RSp463e,2) - 1./54.*m33*RSp453e*
         RS345pe );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(mp2,L) * ( 1./54.*m33*RSp453e*
         RS346pe + 1./54.*m33*RSp453e*RSe463p - 1./54.*m33*RSp453e*
         RSe453p + 1./54.*m33*RSp453e*RSp463e - 1./108.*m33*pow(
         RSp453e,2) + 1./162.*m33*RS145pe*RSp463e - 1./162.*m33*RS145pe
         *RSp453e - 1./162.*m33*RS146pe*RSp463e + 1./162.*m33*RS146pe*
         RSp453e - 1./162.*m33*RSe461p*RSp463e + 1./162.*m33*RSe461p*
         RSp453e + 1./162.*m33*RSe451p*RSp463e - 1./162.*m33*RSe451p*
         RSp453e + 1./162.*m33*RSp461e*RS345pe - 1./162.*m33*RSp461e*
         RS346pe - 1./162.*m33*RSp461e*RSe463p + 1./162.*m33*RSp461e*
         RSe453p - 1./162.*m33*RSp461e*RSp463e + 1./162.*m33*RSp461e*
         RSp453e + 1./162.*m33*RSp461e*RS145pe - 1./162.*m33*RSp461e*
         RS146pe - 1./162.*m33*RSp461e*RSe461p + 1./162.*m33*RSp461e*
         RSe451p - 1./324.*m33*pow(RSp461e,2) - 1./162.*m33*RSp451e*
         RS345pe + 1./162.*m33*RSp451e*RS346pe + 1./162.*m33*RSp451e*
         RSe463p - 1./162.*m33*RSp451e*RSe453p + 1./162.*m33*RSp451e*
         RSp463e );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(mp2,L) * (  - 1./162.*m33*
         RSp451e*RSp453e - 1./162.*m33*RSp451e*RS145pe + 1./162.*m33*
         RSp451e*RS146pe + 1./162.*m33*RSp451e*RSe461p - 1./162.*m33*
         RSp451e*RSe451p + 1./162.*m33*RSp451e*RSp461e - 1./324.*m33*
         pow(RSp451e,2) - 1./81.*m13*RSp463e*RS345pe + 1./81.*m13*
         RSp463e*RS346pe + 1./81.*m13*RSp463e*RSe463p - 1./81.*m13*
         RSp463e*RSe453p + 1./162.*m13*pow(RSp463e,2) + 1./81.*m13*
         RSp453e*RS345pe - 1./81.*m13*RSp453e*RS346pe - 1./81.*m13*
         RSp453e*RSe463p + 1./81.*m13*RSp453e*RSe453p - 1./81.*m13*
         RSp453e*RSp463e + 1./162.*m13*pow(RSp453e,2) + 1./81.*m13*
         RS145pe*RSp463e - 1./81.*m13*RS145pe*RSp453e - 1./81.*m13*
         RS146pe*RSp463e + 1./81.*m13*RS146pe*RSp453e - 1./81.*m13*
         RSe461p*RSp463e + 1./81.*m13*RSe461p*RSp453e + 1./81.*m13*
         RSe451p*RSp463e - 1./81.*m13*RSe451p*RSp453e + 1./81.*m13*
         RSp461e*RS345pe - 1./81.*m13*RSp461e*RS346pe - 1./81.*m13*
         RSp461e*RSe463p );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(mp2,L) * ( 1./81.*m13*RSp461e*
         RSe453p - 1./81.*m13*RSp461e*RSp463e + 1./81.*m13*RSp461e*
         RSp453e - 1./81.*m13*RSp461e*RS145pe + 1./81.*m13*RSp461e*
         RS146pe + 1./81.*m13*RSp461e*RSe461p - 1./81.*m13*RSp461e*
         RSe451p + 1./162.*m13*pow(RSp461e,2) - 1./81.*m13*RSp451e*
         RS345pe + 1./81.*m13*RSp451e*RS346pe + 1./81.*m13*RSp451e*
         RSe463p - 1./81.*m13*RSp451e*RSe453p + 1./81.*m13*RSp451e*
         RSp463e - 1./81.*m13*RSp451e*RSp453e + 1./81.*m13*RSp451e*
         RS145pe - 1./81.*m13*RSp451e*RS146pe - 1./81.*m13*RSp451e*
         RSe461p + 1./81.*m13*RSp451e*RSe451p - 1./81.*m13*RSp451e*
         RSp461e + 1./162.*m13*pow(RSp451e,2) + 1./162.*m11*RSp463e*
         RS345pe - 1./162.*m11*RSp463e*RS346pe - 1./162.*m11*RSp463e*
         RSe463p + 1./162.*m11*RSp463e*RSe453p - 1./324.*m11*pow(
         RSp463e,2) - 1./162.*m11*RSp453e*RS345pe + 1./162.*m11*RSp453e
         *RS346pe + 1./162.*m11*RSp453e*RSe463p - 1./162.*m11*RSp453e*
         RSe453p );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(mp2,L) * ( 1./162.*m11*RSp453e*
         RSp463e - 1./324.*m11*pow(RSp453e,2) + 1./162.*m11*RS145pe*
         RSp463e - 1./162.*m11*RS145pe*RSp453e - 1./162.*m11*RS146pe*
         RSp463e + 1./162.*m11*RS146pe*RSp453e - 1./162.*m11*RSe461p*
         RSp463e + 1./162.*m11*RSe461p*RSp453e + 1./162.*m11*RSe451p*
         RSp463e - 1./162.*m11*RSe451p*RSp453e + 1./162.*m11*RSp461e*
         RS345pe - 1./162.*m11*RSp461e*RS346pe - 1./162.*m11*RSp461e*
         RSe463p + 1./162.*m11*RSp461e*RSe453p - 1./162.*m11*RSp461e*
         RSp463e + 1./162.*m11*RSp461e*RSp453e + 1./54.*m11*RSp461e*
         RS145pe - 1./54.*m11*RSp461e*RS146pe - 1./54.*m11*RSp461e*
         RSe461p + 1./54.*m11*RSp461e*RSe451p - 1./108.*m11*pow(
         RSp461e,2) - 1./162.*m11*RSp451e*RS345pe + 1./162.*m11*RSp451e
         *RS346pe + 1./162.*m11*RSp451e*RSe463p - 1./162.*m11*RSp451e*
         RSe453p + 1./162.*m11*RSp451e*RSp463e - 1./162.*m11*RSp451e*
         RSp453e - 1./54.*m11*RSp451e*RS145pe + 1./54.*m11*RSp451e*
         RS146pe );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(mp2,L) * ( 1./54.*m11*RSp451e*
         RSe461p - 1./54.*m11*RSp451e*RSe451p + 1./54.*m11*RSp451e*
         RSp461e - 1./108.*m11*pow(RSp451e,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(me2,L) * (  - 1./81.*me2*RSe463p
         *RS345pe + 1./81.*me2*RSe463p*RS346pe + 1./81.*me2*pow(
         RSe463p,2) + 1./81.*me2*RSe453p*RS345pe - 1./81.*me2*RSe453p*
         RS346pe - 2./81.*me2*RSe453p*RSe463p + 1./81.*me2*pow(
         RSe453p,2) + 1./81.*me2*RSp463e*RSe463p - 1./81.*me2*RSp463e*
         RSe453p - 1./81.*me2*RSp453e*RSe463p + 1./81.*me2*RSp453e*
         RSe453p + 1./81.*me2*RS145pe*RSe463p - 1./81.*me2*RS145pe*
         RSe453p - 1./81.*me2*RS146pe*RSe463p + 1./81.*me2*RS146pe*
         RSe453p + 1./81.*me2*RSe461p*RS345pe - 1./81.*me2*RSe461p*
         RS346pe - 2./81.*me2*RSe461p*RSe463p + 2./81.*me2*RSe461p*
         RSe453p - 1./81.*me2*RSe461p*RSp463e + 1./81.*me2*RSe461p*
         RSp453e - 1./81.*me2*RSe461p*RS145pe + 1./81.*me2*RSe461p*
         RS146pe + 1./81.*me2*pow(RSe461p,2) - 1./81.*me2*RSe451p*
         RS345pe + 1./81.*me2*RSe451p*RS346pe + 2./81.*me2*RSe451p*
         RSe463p - 2./81.*me2*RSe451p*RSe453p + 1./81.*me2*RSe451p*
         RSp463e );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(me2,L) * (  - 1./81.*me2*RSe451p
         *RSp453e + 1./81.*me2*RSe451p*RS145pe - 1./81.*me2*RSe451p*
         RS146pe - 2./81.*me2*RSe451p*RSe461p + 1./81.*me2*pow(
         RSe451p,2) - 1./81.*me2*RSp461e*RSe463p + 1./81.*me2*RSp461e*
         RSe453p + 1./81.*me2*RSp461e*RSe461p - 1./81.*me2*RSp461e*
         RSe451p + 1./81.*me2*RSp451e*RSe463p - 1./81.*me2*RSp451e*
         RSe453p - 1./81.*me2*RSp451e*RSe461p + 1./81.*me2*RSp451e*
         RSe451p - 1./162.*m66*RSe463p*RS345pe - 1./162.*m66*RSe463p*
         RS346pe - 1./324.*m66*pow(RSe463p,2) - 1./54.*m66*RSe453p*
         RS345pe - 1./162.*m66*RSe453p*RS346pe - 1./162.*m66*RSe453p*
         RSe463p - 1./108.*m66*pow(RSe453p,2) + 1./162.*m66*RS145pe*
         RSe463p + 1./54.*m66*RS145pe*RSe453p + 1./162.*m66*RS146pe*
         RSe463p + 1./162.*m66*RS146pe*RSe453p + 1./162.*m66*RSe461p*
         RS345pe + 1./162.*m66*RSe461p*RS346pe + 1./162.*m66*RSe461p*
         RSe463p + 1./162.*m66*RSe461p*RSe453p - 1./162.*m66*RSe461p*
         RS145pe );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(me2,L) * (  - 1./162.*m66*
         RSe461p*RS146pe - 1./324.*m66*pow(RSe461p,2) + 1./54.*m66*
         RSe451p*RS345pe + 1./162.*m66*RSe451p*RS346pe + 1./162.*m66*
         RSe451p*RSe463p + 1./54.*m66*RSe451p*RSe453p - 1./54.*m66*
         RSe451p*RS145pe - 1./162.*m66*RSe451p*RS146pe - 1./162.*m66*
         RSe451p*RSe461p - 1./108.*m66*pow(RSe451p,2) - 1./81.*m56*
         RSe463p*RS345pe + 1./81.*m56*RSe463p*RS346pe + 1./162.*m56*
         pow(RSe463p,2) + 1./81.*m56*RSe453p*RS345pe - 1./81.*m56*
         RSe453p*RS346pe - 1./81.*m56*RSe453p*RSe463p + 1./162.*m56*
         pow(RSe453p,2) + 1./81.*m56*RS145pe*RSe463p - 1./81.*m56*
         RS145pe*RSe453p - 1./81.*m56*RS146pe*RSe463p + 1./81.*m56*
         RS146pe*RSe453p + 1./81.*m56*RSe461p*RS345pe - 1./81.*m56*
         RSe461p*RS346pe - 1./81.*m56*RSe461p*RSe463p + 1./81.*m56*
         RSe461p*RSe453p - 1./81.*m56*RSe461p*RS145pe + 1./81.*m56*
         RSe461p*RS146pe + 1./162.*m56*pow(RSe461p,2) - 1./81.*m56*
         RSe451p*RS345pe );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(me2,L) * ( 1./81.*m56*RSe451p*
         RS346pe + 1./81.*m56*RSe451p*RSe463p - 1./81.*m56*RSe451p*
         RSe453p + 1./81.*m56*RSe451p*RS145pe - 1./81.*m56*RSe451p*
         RS146pe - 1./81.*m56*RSe451p*RSe461p + 1./162.*m56*pow(
         RSe451p,2) - 1./162.*m55*RSe463p*RS345pe - 1./54.*m55*RSe463p*
         RS346pe - 1./108.*m55*pow(RSe463p,2) - 1./162.*m55*RSe453p*
         RS345pe - 1./162.*m55*RSe453p*RS346pe - 1./162.*m55*RSe453p*
         RSe463p - 1./324.*m55*pow(RSe453p,2) + 1./162.*m55*RS145pe*
         RSe463p + 1./162.*m55*RS145pe*RSe453p + 1./54.*m55*RS146pe*
         RSe463p + 1./162.*m55*RS146pe*RSe453p + 1./162.*m55*RSe461p*
         RS345pe + 1./54.*m55*RSe461p*RS346pe + 1./54.*m55*RSe461p*
         RSe463p + 1./162.*m55*RSe461p*RSe453p - 1./162.*m55*RSe461p*
         RS145pe - 1./54.*m55*RSe461p*RS146pe - 1./108.*m55*pow(
         RSe461p,2) + 1./162.*m55*RSe451p*RS345pe + 1./162.*m55*RSe451p
         *RS346pe + 1./162.*m55*RSe451p*RSe463p + 1./162.*m55*RSe451p*
         RSe453p );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(me2,L) * (  - 1./162.*m55*
         RSe451p*RS145pe - 1./162.*m55*RSe451p*RS146pe - 1./162.*m55*
         RSe451p*RSe461p - 1./324.*m55*pow(RSe451p,2) + 1./54.*m33*
         RSe463p*RS345pe - 1./54.*m33*RSe463p*RS346pe - 1./108.*m33*
         pow(RSe463p,2) - 1./54.*m33*RSe453p*RS345pe + 1./54.*m33*
         RSe453p*RS346pe + 1./54.*m33*RSe453p*RSe463p - 1./108.*m33*
         pow(RSe453p,2) + 1./162.*m33*RS145pe*RSe463p - 1./162.*m33*
         RS145pe*RSe453p - 1./162.*m33*RS146pe*RSe463p + 1./162.*m33*
         RS146pe*RSe453p + 1./162.*m33*RSe461p*RS345pe - 1./162.*m33*
         RSe461p*RS346pe - 1./162.*m33*RSe461p*RSe463p + 1./162.*m33*
         RSe461p*RSe453p + 1./162.*m33*RSe461p*RS145pe - 1./162.*m33*
         RSe461p*RS146pe - 1./324.*m33*pow(RSe461p,2) - 1./162.*m33*
         RSe451p*RS345pe + 1./162.*m33*RSe451p*RS346pe + 1./162.*m33*
         RSe451p*RSe463p - 1./162.*m33*RSe451p*RSe453p - 1./162.*m33*
         RSe451p*RS145pe + 1./162.*m33*RSe451p*RS146pe + 1./162.*m33*
         RSe451p*RSe461p );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(me2,L) * (  - 1./324.*m33*pow(
         RSe451p,2) - 1./81.*m13*RSe463p*RS345pe + 1./81.*m13*RSe463p*
         RS346pe + 1./162.*m13*pow(RSe463p,2) + 1./81.*m13*RSe453p*
         RS345pe - 1./81.*m13*RSe453p*RS346pe - 1./81.*m13*RSe453p*
         RSe463p + 1./162.*m13*pow(RSe453p,2) + 1./81.*m13*RS145pe*
         RSe463p - 1./81.*m13*RS145pe*RSe453p - 1./81.*m13*RS146pe*
         RSe463p + 1./81.*m13*RS146pe*RSe453p + 1./81.*m13*RSe461p*
         RS345pe - 1./81.*m13*RSe461p*RS346pe - 1./81.*m13*RSe461p*
         RSe463p + 1./81.*m13*RSe461p*RSe453p - 1./81.*m13*RSe461p*
         RS145pe + 1./81.*m13*RSe461p*RS146pe + 1./162.*m13*pow(
         RSe461p,2) - 1./81.*m13*RSe451p*RS345pe + 1./81.*m13*RSe451p*
         RS346pe + 1./81.*m13*RSe451p*RSe463p - 1./81.*m13*RSe451p*
         RSe453p + 1./81.*m13*RSe451p*RS145pe - 1./81.*m13*RSe451p*
         RS146pe - 1./81.*m13*RSe451p*RSe461p + 1./162.*m13*pow(
         RSe451p,2) + 1./162.*m11*RSe463p*RS345pe - 1./162.*m11*RSe463p
         *RS346pe );

      M13p6x23RV +=  + Ab(m56,mu2)*AbV(me2,L) * (  - 1./324.*m11*pow(
         RSe463p,2) - 1./162.*m11*RSe453p*RS345pe + 1./162.*m11*RSe453p
         *RS346pe + 1./162.*m11*RSe453p*RSe463p - 1./324.*m11*pow(
         RSe453p,2) + 1./162.*m11*RS145pe*RSe463p - 1./162.*m11*RS145pe
         *RSe453p - 1./162.*m11*RS146pe*RSe463p + 1./162.*m11*RS146pe*
         RSe453p + 1./162.*m11*RSe461p*RS345pe - 1./162.*m11*RSe461p*
         RS346pe - 1./162.*m11*RSe461p*RSe463p + 1./162.*m11*RSe461p*
         RSe453p + 1./54.*m11*RSe461p*RS145pe - 1./54.*m11*RSe461p*
         RS146pe - 1./108.*m11*pow(RSe461p,2) - 1./162.*m11*RSe451p*
         RS345pe + 1./162.*m11*RSe451p*RS346pe + 1./162.*m11*RSe451p*
         RSe463p - 1./162.*m11*RSe451p*RSe453p - 1./54.*m11*RSe451p*
         RS145pe + 1./54.*m11*RSe451p*RS146pe + 1./54.*m11*RSe451p*
         RSe461p - 1./108.*m11*pow(RSe451p,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,m33,L) * ( 1./108.*m33*m66*
         RS145pe*RS345pe + 1./324.*m33*m66*RS145pe*RS346pe + 1./324.*
         m33*m66*RS146pe*RS345pe + 1./324.*m33*m66*RS146pe*RS346pe - 1./
         162.*m33*m56*RS145pe*RS345pe + 1./162.*m33*m56*RS145pe*RS346pe
          + 1./162.*m33*m56*RS146pe*RS345pe - 1./162.*m33*m56*RS146pe*
         RS346pe + 1./324.*m33*m55*RS145pe*RS345pe + 1./324.*m33*m55*
         RS145pe*RS346pe + 1./324.*m33*m55*RS146pe*RS345pe + 1./108.*
         m33*m55*RS146pe*RS346pe + 1./54.*m13*m66*RS145pe*RS345pe + 1./
         162.*m13*m66*RS145pe*RS346pe + 1./162.*m13*m66*RS146pe*RS345pe
          + 1./162.*m13*m66*RS146pe*RS346pe - 1./81.*m13*m56*RS145pe*
         RS345pe + 1./81.*m13*m56*RS145pe*RS346pe + 1./81.*m13*m56*
         RS146pe*RS345pe - 1./81.*m13*m56*RS146pe*RS346pe + 1./162.*m13
         *m55*RS145pe*RS345pe + 1./162.*m13*m55*RS145pe*RS346pe + 1./
         162.*m13*m55*RS146pe*RS345pe + 1./54.*m13*m55*RS146pe*RS346pe
          + 1./36.*m11*m66*RS145pe*RS345pe + 1./108.*m11*m66*RS145pe*
         RS346pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,m33,L) * ( 1./108.*m11*m66*
         RS146pe*RS345pe + 1./108.*m11*m66*RS146pe*RS346pe - 1./54.*m11
         *m56*RS145pe*RS345pe + 1./54.*m11*m56*RS145pe*RS346pe + 1./54.
         *m11*m56*RS146pe*RS345pe - 1./54.*m11*m56*RS146pe*RS346pe + 1./
         108.*m11*m55*RS145pe*RS345pe + 1./108.*m11*m55*RS145pe*RS346pe
          + 1./108.*m11*m55*RS146pe*RS345pe + 1./36.*m11*m55*RS146pe*
         RS346pe - 1./54.*m11*m33*RS145pe*RS345pe + 1./54.*m11*m33*
         RS145pe*RS346pe + 1./54.*m11*m33*RS146pe*RS345pe - 1./54.*m11*
         m33*RS146pe*RS346pe - 1./81.*m11*m13*RS145pe*RS345pe + 1./81.*
         m11*m13*RS145pe*RS346pe + 1./81.*m11*m13*RS146pe*RS345pe - 1./
         81.*m11*m13*RS146pe*RS346pe - 1./162.*pow(m11,2)*RS145pe*
         RS345pe + 1./162.*pow(m11,2)*RS145pe*RS346pe + 1./162.*pow(
         m11,2)*RS146pe*RS345pe - 1./162.*pow(m11,2)*RS146pe*RS346pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,mp2,L) * ( 1./324.*m33*m66*
         RS145pe*RSp463e + 1./108.*m33*m66*RS145pe*RSp453e + 1./324.*
         m33*m66*RS146pe*RSp463e + 1./324.*m33*m66*RS146pe*RSp453e + 1./
         324.*m33*m66*RSp461e*RS145pe + 1./324.*m33*m66*RSp461e*RS146pe
          + 1./108.*m33*m66*RSp451e*RS145pe + 1./324.*m33*m66*RSp451e*
         RS146pe + 1./162.*m33*m56*RS145pe*RSp463e - 1./162.*m33*m56*
         RS145pe*RSp453e - 1./162.*m33*m56*RS146pe*RSp463e + 1./162.*
         m33*m56*RS146pe*RSp453e + 1./162.*m33*m56*RSp461e*RS145pe - 1./
         162.*m33*m56*RSp461e*RS146pe - 1./162.*m33*m56*RSp451e*RS145pe
          + 1./162.*m33*m56*RSp451e*RS146pe + 1./324.*m33*m55*RS145pe*
         RSp463e + 1./324.*m33*m55*RS145pe*RSp453e + 1./108.*m33*m55*
         RS146pe*RSp463e + 1./324.*m33*m55*RS146pe*RSp453e + 1./324.*
         m33*m55*RSp461e*RS145pe + 1./108.*m33*m55*RSp461e*RS146pe + 1./
         324.*m33*m55*RSp451e*RS145pe + 1./324.*m33*m55*RSp451e*RS146pe
          + 1./162.*m13*m66*RS145pe*RSp463e + 1./54.*m13*m66*RS145pe*
         RSp453e );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,mp2,L) * ( 1./162.*m13*m66*
         RS146pe*RSp463e + 1./162.*m13*m66*RS146pe*RSp453e - 1./162.*
         m13*m66*RSp461e*RS145pe - 1./162.*m13*m66*RSp461e*RS146pe - 1./
         54.*m13*m66*RSp451e*RS145pe - 1./162.*m13*m66*RSp451e*RS146pe
          + 1./81.*m13*m56*RS145pe*RSp463e - 1./81.*m13*m56*RS145pe*
         RSp453e - 1./81.*m13*m56*RS146pe*RSp463e + 1./81.*m13*m56*
         RS146pe*RSp453e - 1./81.*m13*m56*RSp461e*RS145pe + 1./81.*m13*
         m56*RSp461e*RS146pe + 1./81.*m13*m56*RSp451e*RS145pe - 1./81.*
         m13*m56*RSp451e*RS146pe + 1./162.*m13*m55*RS145pe*RSp463e + 1./
         162.*m13*m55*RS145pe*RSp453e + 1./54.*m13*m55*RS146pe*RSp463e
          + 1./162.*m13*m55*RS146pe*RSp453e - 1./162.*m13*m55*RSp461e*
         RS145pe - 1./54.*m13*m55*RSp461e*RS146pe - 1./162.*m13*m55*
         RSp451e*RS145pe - 1./162.*m13*m55*RSp451e*RS146pe + 1./81.*m11
         *mp2*RS145pe*RSp463e - 1./81.*m11*mp2*RS145pe*RSp453e - 1./81.
         *m11*mp2*RS146pe*RSp463e + 1./81.*m11*mp2*RS146pe*RSp453e - 1./
         81.*m11*mp2*RSp461e*RS145pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,mp2,L) * ( 1./81.*m11*mp2*
         RSp461e*RS146pe + 1./81.*m11*mp2*RSp451e*RS145pe - 1./81.*m11*
         mp2*RSp451e*RS146pe + 1./108.*m11*m66*RS145pe*RSp463e + 1./36.
         *m11*m66*RS145pe*RSp453e + 1./108.*m11*m66*RS146pe*RSp463e + 1.
         /108.*m11*m66*RS146pe*RSp453e + 1./324.*m11*m66*RSp461e*
         RS145pe + 1./324.*m11*m66*RSp461e*RS146pe + 1./108.*m11*m66*
         RSp451e*RS145pe + 1./324.*m11*m66*RSp451e*RS146pe + 1./54.*m11
         *m56*RS145pe*RSp463e - 1./54.*m11*m56*RS145pe*RSp453e - 1./54.
         *m11*m56*RS146pe*RSp463e + 1./54.*m11*m56*RS146pe*RSp453e + 1./
         162.*m11*m56*RSp461e*RS145pe - 1./162.*m11*m56*RSp461e*RS146pe
          - 1./162.*m11*m56*RSp451e*RS145pe + 1./162.*m11*m56*RSp451e*
         RS146pe + 1./108.*m11*m55*RS145pe*RSp463e + 1./108.*m11*m55*
         RS145pe*RSp453e + 1./36.*m11*m55*RS146pe*RSp463e + 1./108.*m11
         *m55*RS146pe*RSp453e + 1./324.*m11*m55*RSp461e*RS145pe + 1./
         108.*m11*m55*RSp461e*RS146pe + 1./324.*m11*m55*RSp451e*RS145pe
          + 1./324.*m11*m55*RSp451e*RS146pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,mp2,L) * ( 1./162.*m11*m33*
         RS145pe*RSp463e - 1./162.*m11*m33*RS145pe*RSp453e - 1./162.*
         m11*m33*RS146pe*RSp463e + 1./162.*m11*m33*RS146pe*RSp453e + 1./
         162.*m11*m33*RSp461e*RS145pe - 1./162.*m11*m33*RSp461e*RS146pe
          - 1./162.*m11*m33*RSp451e*RS145pe + 1./162.*m11*m33*RSp451e*
         RS146pe + 1./81.*m11*m13*RS145pe*RSp463e - 1./81.*m11*m13*
         RS145pe*RSp453e - 1./81.*m11*m13*RS146pe*RSp463e + 1./81.*m11*
         m13*RS146pe*RSp453e - 1./81.*m11*m13*RSp461e*RS145pe + 1./81.*
         m11*m13*RSp461e*RS146pe + 1./81.*m11*m13*RSp451e*RS145pe - 1./
         81.*m11*m13*RSp451e*RS146pe + 1./162.*pow(m11,2)*RS145pe*
         RSp463e - 1./162.*pow(m11,2)*RS145pe*RSp453e - 1./162.*pow(
         m11,2)*RS146pe*RSp463e + 1./162.*pow(m11,2)*RS146pe*RSp453e + 
         1./54.*pow(m11,2)*RSp461e*RS145pe - 1./54.*pow(m11,2)*RSp461e*
         RS146pe - 1./54.*pow(m11,2)*RSp451e*RS145pe + 1./54.*pow(
         m11,2)*RSp451e*RS146pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,me2,L) * ( 1./324.*m33*m66*
         RS145pe*RSe463p + 1./108.*m33*m66*RS145pe*RSe453p + 1./324.*
         m33*m66*RS146pe*RSe463p + 1./324.*m33*m66*RS146pe*RSe453p + 1./
         324.*m33*m66*RSe461p*RS145pe + 1./324.*m33*m66*RSe461p*RS146pe
          + 1./108.*m33*m66*RSe451p*RS145pe + 1./324.*m33*m66*RSe451p*
         RS146pe + 1./162.*m33*m56*RS145pe*RSe463p - 1./162.*m33*m56*
         RS145pe*RSe453p - 1./162.*m33*m56*RS146pe*RSe463p + 1./162.*
         m33*m56*RS146pe*RSe453p + 1./162.*m33*m56*RSe461p*RS145pe - 1./
         162.*m33*m56*RSe461p*RS146pe - 1./162.*m33*m56*RSe451p*RS145pe
          + 1./162.*m33*m56*RSe451p*RS146pe + 1./324.*m33*m55*RS145pe*
         RSe463p + 1./324.*m33*m55*RS145pe*RSe453p + 1./108.*m33*m55*
         RS146pe*RSe463p + 1./324.*m33*m55*RS146pe*RSe453p + 1./324.*
         m33*m55*RSe461p*RS145pe + 1./108.*m33*m55*RSe461p*RS146pe + 1./
         324.*m33*m55*RSe451p*RS145pe + 1./324.*m33*m55*RSe451p*RS146pe
          + 1./162.*m13*m66*RS145pe*RSe463p + 1./54.*m13*m66*RS145pe*
         RSe453p );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,me2,L) * ( 1./162.*m13*m66*
         RS146pe*RSe463p + 1./162.*m13*m66*RS146pe*RSe453p - 1./162.*
         m13*m66*RSe461p*RS145pe - 1./162.*m13*m66*RSe461p*RS146pe - 1./
         54.*m13*m66*RSe451p*RS145pe - 1./162.*m13*m66*RSe451p*RS146pe
          + 1./81.*m13*m56*RS145pe*RSe463p - 1./81.*m13*m56*RS145pe*
         RSe453p - 1./81.*m13*m56*RS146pe*RSe463p + 1./81.*m13*m56*
         RS146pe*RSe453p - 1./81.*m13*m56*RSe461p*RS145pe + 1./81.*m13*
         m56*RSe461p*RS146pe + 1./81.*m13*m56*RSe451p*RS145pe - 1./81.*
         m13*m56*RSe451p*RS146pe + 1./162.*m13*m55*RS145pe*RSe463p + 1./
         162.*m13*m55*RS145pe*RSe453p + 1./54.*m13*m55*RS146pe*RSe463p
          + 1./162.*m13*m55*RS146pe*RSe453p - 1./162.*m13*m55*RSe461p*
         RS145pe - 1./54.*m13*m55*RSe461p*RS146pe - 1./162.*m13*m55*
         RSe451p*RS145pe - 1./162.*m13*m55*RSe451p*RS146pe + 1./81.*m11
         *me2*RS145pe*RSe463p - 1./81.*m11*me2*RS145pe*RSe453p - 1./81.
         *m11*me2*RS146pe*RSe463p + 1./81.*m11*me2*RS146pe*RSe453p - 1./
         81.*m11*me2*RSe461p*RS145pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,me2,L) * ( 1./81.*m11*me2*
         RSe461p*RS146pe + 1./81.*m11*me2*RSe451p*RS145pe - 1./81.*m11*
         me2*RSe451p*RS146pe + 1./108.*m11*m66*RS145pe*RSe463p + 1./36.
         *m11*m66*RS145pe*RSe453p + 1./108.*m11*m66*RS146pe*RSe463p + 1.
         /108.*m11*m66*RS146pe*RSe453p + 1./324.*m11*m66*RSe461p*
         RS145pe + 1./324.*m11*m66*RSe461p*RS146pe + 1./108.*m11*m66*
         RSe451p*RS145pe + 1./324.*m11*m66*RSe451p*RS146pe + 1./54.*m11
         *m56*RS145pe*RSe463p - 1./54.*m11*m56*RS145pe*RSe453p - 1./54.
         *m11*m56*RS146pe*RSe463p + 1./54.*m11*m56*RS146pe*RSe453p + 1./
         162.*m11*m56*RSe461p*RS145pe - 1./162.*m11*m56*RSe461p*RS146pe
          - 1./162.*m11*m56*RSe451p*RS145pe + 1./162.*m11*m56*RSe451p*
         RS146pe + 1./108.*m11*m55*RS145pe*RSe463p + 1./108.*m11*m55*
         RS145pe*RSe453p + 1./36.*m11*m55*RS146pe*RSe463p + 1./108.*m11
         *m55*RS146pe*RSe453p + 1./324.*m11*m55*RSe461p*RS145pe + 1./
         108.*m11*m55*RSe461p*RS146pe + 1./324.*m11*m55*RSe451p*RS145pe
          + 1./324.*m11*m55*RSe451p*RS146pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,me2,L) * ( 1./162.*m11*m33*
         RS145pe*RSe463p - 1./162.*m11*m33*RS145pe*RSe453p - 1./162.*
         m11*m33*RS146pe*RSe463p + 1./162.*m11*m33*RS146pe*RSe453p + 1./
         162.*m11*m33*RSe461p*RS145pe - 1./162.*m11*m33*RSe461p*RS146pe
          - 1./162.*m11*m33*RSe451p*RS145pe + 1./162.*m11*m33*RSe451p*
         RS146pe + 1./81.*m11*m13*RS145pe*RSe463p - 1./81.*m11*m13*
         RS145pe*RSe453p - 1./81.*m11*m13*RS146pe*RSe463p + 1./81.*m11*
         m13*RS146pe*RSe453p - 1./81.*m11*m13*RSe461p*RS145pe + 1./81.*
         m11*m13*RSe461p*RS146pe + 1./81.*m11*m13*RSe451p*RS145pe - 1./
         81.*m11*m13*RSe451p*RS146pe + 1./162.*pow(m11,2)*RS145pe*
         RSe463p - 1./162.*pow(m11,2)*RS145pe*RSe453p - 1./162.*pow(
         m11,2)*RS146pe*RSe463p + 1./162.*pow(m11,2)*RS146pe*RSe453p + 
         1./54.*pow(m11,2)*RSe461p*RS145pe - 1./54.*pow(m11,2)*RSe461p*
         RS146pe - 1./54.*pow(m11,2)*RSe451p*RS145pe + 1./54.*pow(
         m11,2)*RSe451p*RS146pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,L) * ( 1./216.*m33*m66*pow(
         RS145pe,2) + 1./324.*m33*m66*RS146pe*RS145pe + 1./648.*m33*m66
         *pow(RS146pe,2) - 1./324.*m33*m56*pow(RS145pe,2) + 1./162.*m33
         *m56*RS146pe*RS145pe - 1./324.*m33*m56*pow(RS146pe,2) + 1./648.
         *m33*m55*pow(RS145pe,2) + 1./324.*m33*m55*RS146pe*RS145pe + 1./
         216.*m33*m55*pow(RS146pe,2) - 1./108.*m13*m66*pow(RS145pe,2)
          - 1./162.*m13*m66*RS146pe*RS145pe - 1./324.*m13*m66*pow(
         RS146pe,2) + 1./162.*m13*m56*pow(RS145pe,2) - 1./81.*m13*m56*
         RS146pe*RS145pe + 1./162.*m13*m56*pow(RS146pe,2) - 1./324.*m13
         *m55*pow(RS145pe,2) - 1./162.*m13*m55*RS146pe*RS145pe - 1./108.
         *m13*m55*pow(RS146pe,2) + 1./216.*m11*m66*pow(RS145pe,2) + 1./
         324.*m11*m66*RS146pe*RS145pe + 1./648.*m11*m66*pow(RS146pe,2)
          - 1./324.*m11*m56*pow(RS145pe,2) + 1./162.*m11*m56*RS146pe*
         RS145pe - 1./324.*m11*m56*pow(RS146pe,2) + 1./648.*m11*m55*
         pow(RS145pe,2) + 1./324.*m11*m55*RS146pe*RS145pe + 1./216.*m11
         *m55*pow(RS146pe,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,L) * (  - 1./324.*m11*m33*
         pow(RS145pe,2) + 1./162.*m11*m33*RS146pe*RS145pe - 1./324.*m11
         *m33*pow(RS146pe,2) + 1./162.*m11*m13*pow(RS145pe,2) - 1./81.*
         m11*m13*RS146pe*RS145pe + 1./162.*m11*m13*pow(RS146pe,2) - 1./
         324.*pow(m11,2)*pow(RS145pe,2) + 1./162.*pow(m11,2)*RS146pe*
         RS145pe - 1./324.*pow(m11,2)*pow(RS146pe,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m15,L) * ( 1./144.*m55*m66 - 1./
         72.*m55*m56 + 1./72.*pow(m55,2) + 1./144.*m33*m66 - 1./72.*m33
         *m56 + 1./72.*m33*m55 - 1./72.*m15*m66 + 1./36.*m15*m56 - 1./
         24.*m15*m55 - 1./72.*m15*m33 + 1./36.*pow(m15,2) - 1./72.*m13*
         m66 + 1./36.*m13*m56 - 1./36.*m13*m55 + 1./36.*m13*m15 + 1./72.
         *m11*m66 - 1./36.*m11*m56 + 5./144.*m11*m55 + 1./144.*m11*m33
          - 1./24.*m11*m15 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m16,L) * ( 1./72.*pow(m66,2) - 1.
         /72.*m56*m66 + 1./144.*m55*m66 + 1./72.*m33*m66 - 1./72.*m33*
         m56 + 1./144.*m33*m55 - 1./24.*m16*m66 + 1./36.*m16*m56 - 1./
         72.*m16*m55 - 1./72.*m16*m33 + 1./36.*pow(m16,2) - 1./36.*m13*
         m66 + 1./36.*m13*m56 - 1./72.*m13*m55 + 1./36.*m13*m16 + 5./
         144.*m11*m66 - 1./36.*m11*m56 + 1./72.*m11*m55 + 1./144.*m11*
         m33 - 1./24.*m11*m16 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,mp2,L) * (  - 1./81.*m33*mp2
         *RSp463e*RS345pe + 1./81.*m33*mp2*RSp463e*RS346pe + 1./81.*m33
         *mp2*RSp453e*RS345pe - 1./81.*m33*mp2*RSp453e*RS346pe + 1./81.
         *m33*mp2*RSp461e*RS345pe - 1./81.*m33*mp2*RSp461e*RS346pe - 1./
         81.*m33*mp2*RSp451e*RS345pe + 1./81.*m33*mp2*RSp451e*RS346pe
          + 1./324.*m33*m66*RSp463e*RS345pe + 1./324.*m33*m66*RSp463e*
         RS346pe + 1./108.*m33*m66*RSp453e*RS345pe + 1./324.*m33*m66*
         RSp453e*RS346pe + 1./108.*m33*m66*RSp461e*RS345pe + 1./108.*
         m33*m66*RSp461e*RS346pe + 1./36.*m33*m66*RSp451e*RS345pe + 1./
         108.*m33*m66*RSp451e*RS346pe + 1./162.*m33*m56*RSp463e*RS345pe
          - 1./162.*m33*m56*RSp463e*RS346pe - 1./162.*m33*m56*RSp453e*
         RS345pe + 1./162.*m33*m56*RSp453e*RS346pe + 1./54.*m33*m56*
         RSp461e*RS345pe - 1./54.*m33*m56*RSp461e*RS346pe - 1./54.*m33*
         m56*RSp451e*RS345pe + 1./54.*m33*m56*RSp451e*RS346pe + 1./324.
         *m33*m55*RSp463e*RS345pe + 1./108.*m33*m55*RSp463e*RS346pe + 1.
         /324.*m33*m55*RSp453e*RS345pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,mp2,L) * ( 1./324.*m33*m55*
         RSp453e*RS346pe + 1./108.*m33*m55*RSp461e*RS345pe + 1./36.*m33
         *m55*RSp461e*RS346pe + 1./108.*m33*m55*RSp451e*RS345pe + 1./
         108.*m33*m55*RSp451e*RS346pe + 1./54.*pow(m33,2)*RSp463e*
         RS345pe - 1./54.*pow(m33,2)*RSp463e*RS346pe - 1./54.*pow(
         m33,2)*RSp453e*RS345pe + 1./54.*pow(m33,2)*RSp453e*RS346pe + 1.
         /162.*pow(m33,2)*RSp461e*RS345pe - 1./162.*pow(m33,2)*RSp461e*
         RS346pe - 1./162.*pow(m33,2)*RSp451e*RS345pe + 1./162.*pow(
         m33,2)*RSp451e*RS346pe - 1./162.*m13*m66*RSp463e*RS345pe - 1./
         162.*m13*m66*RSp463e*RS346pe - 1./54.*m13*m66*RSp453e*RS345pe
          - 1./162.*m13*m66*RSp453e*RS346pe + 1./162.*m13*m66*RSp461e*
         RS345pe + 1./162.*m13*m66*RSp461e*RS346pe + 1./54.*m13*m66*
         RSp451e*RS345pe + 1./162.*m13*m66*RSp451e*RS346pe - 1./81.*m13
         *m56*RSp463e*RS345pe + 1./81.*m13*m56*RSp463e*RS346pe + 1./81.
         *m13*m56*RSp453e*RS345pe - 1./81.*m13*m56*RSp453e*RS346pe + 1./
         81.*m13*m56*RSp461e*RS345pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,mp2,L) * (  - 1./81.*m13*m56
         *RSp461e*RS346pe - 1./81.*m13*m56*RSp451e*RS345pe + 1./81.*m13
         *m56*RSp451e*RS346pe - 1./162.*m13*m55*RSp463e*RS345pe - 1./54.
         *m13*m55*RSp463e*RS346pe - 1./162.*m13*m55*RSp453e*RS345pe - 1.
         /162.*m13*m55*RSp453e*RS346pe + 1./162.*m13*m55*RSp461e*
         RS345pe + 1./54.*m13*m55*RSp461e*RS346pe + 1./162.*m13*m55*
         RSp451e*RS345pe + 1./162.*m13*m55*RSp451e*RS346pe - 1./81.*m13
         *m33*RSp463e*RS345pe + 1./81.*m13*m33*RSp463e*RS346pe + 1./81.
         *m13*m33*RSp453e*RS345pe - 1./81.*m13*m33*RSp453e*RS346pe + 1./
         81.*m13*m33*RSp461e*RS345pe - 1./81.*m13*m33*RSp461e*RS346pe
          - 1./81.*m13*m33*RSp451e*RS345pe + 1./81.*m13*m33*RSp451e*
         RS346pe + 1./324.*m11*m66*RSp463e*RS345pe + 1./324.*m11*m66*
         RSp463e*RS346pe + 1./108.*m11*m66*RSp453e*RS345pe + 1./324.*
         m11*m66*RSp453e*RS346pe + 1./324.*m11*m66*RSp461e*RS345pe + 1./
         324.*m11*m66*RSp461e*RS346pe + 1./108.*m11*m66*RSp451e*RS345pe
          + 1./324.*m11*m66*RSp451e*RS346pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,mp2,L) * ( 1./162.*m11*m56*
         RSp463e*RS345pe - 1./162.*m11*m56*RSp463e*RS346pe - 1./162.*
         m11*m56*RSp453e*RS345pe + 1./162.*m11*m56*RSp453e*RS346pe + 1./
         162.*m11*m56*RSp461e*RS345pe - 1./162.*m11*m56*RSp461e*RS346pe
          - 1./162.*m11*m56*RSp451e*RS345pe + 1./162.*m11*m56*RSp451e*
         RS346pe + 1./324.*m11*m55*RSp463e*RS345pe + 1./108.*m11*m55*
         RSp463e*RS346pe + 1./324.*m11*m55*RSp453e*RS345pe + 1./324.*
         m11*m55*RSp453e*RS346pe + 1./324.*m11*m55*RSp461e*RS345pe + 1./
         108.*m11*m55*RSp461e*RS346pe + 1./324.*m11*m55*RSp451e*RS345pe
          + 1./324.*m11*m55*RSp451e*RS346pe + 1./162.*m11*m33*RSp463e*
         RS345pe - 1./162.*m11*m33*RSp463e*RS346pe - 1./162.*m11*m33*
         RSp453e*RS345pe + 1./162.*m11*m33*RSp453e*RS346pe + 1./162.*
         m11*m33*RSp461e*RS345pe - 1./162.*m11*m33*RSp461e*RS346pe - 1./
         162.*m11*m33*RSp451e*RS345pe + 1./162.*m11*m33*RSp451e*RS346pe
          );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,me2,L) * (  - 1./81.*m33*me2
         *RSe463p*RS345pe + 1./81.*m33*me2*RSe463p*RS346pe + 1./81.*m33
         *me2*RSe453p*RS345pe - 1./81.*m33*me2*RSe453p*RS346pe + 1./81.
         *m33*me2*RSe461p*RS345pe - 1./81.*m33*me2*RSe461p*RS346pe - 1./
         81.*m33*me2*RSe451p*RS345pe + 1./81.*m33*me2*RSe451p*RS346pe
          + 1./324.*m33*m66*RSe463p*RS345pe + 1./324.*m33*m66*RSe463p*
         RS346pe + 1./108.*m33*m66*RSe453p*RS345pe + 1./324.*m33*m66*
         RSe453p*RS346pe + 1./108.*m33*m66*RSe461p*RS345pe + 1./108.*
         m33*m66*RSe461p*RS346pe + 1./36.*m33*m66*RSe451p*RS345pe + 1./
         108.*m33*m66*RSe451p*RS346pe + 1./162.*m33*m56*RSe463p*RS345pe
          - 1./162.*m33*m56*RSe463p*RS346pe - 1./162.*m33*m56*RSe453p*
         RS345pe + 1./162.*m33*m56*RSe453p*RS346pe + 1./54.*m33*m56*
         RSe461p*RS345pe - 1./54.*m33*m56*RSe461p*RS346pe - 1./54.*m33*
         m56*RSe451p*RS345pe + 1./54.*m33*m56*RSe451p*RS346pe + 1./324.
         *m33*m55*RSe463p*RS345pe + 1./108.*m33*m55*RSe463p*RS346pe + 1.
         /324.*m33*m55*RSe453p*RS345pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,me2,L) * ( 1./324.*m33*m55*
         RSe453p*RS346pe + 1./108.*m33*m55*RSe461p*RS345pe + 1./36.*m33
         *m55*RSe461p*RS346pe + 1./108.*m33*m55*RSe451p*RS345pe + 1./
         108.*m33*m55*RSe451p*RS346pe + 1./54.*pow(m33,2)*RSe463p*
         RS345pe - 1./54.*pow(m33,2)*RSe463p*RS346pe - 1./54.*pow(
         m33,2)*RSe453p*RS345pe + 1./54.*pow(m33,2)*RSe453p*RS346pe + 1.
         /162.*pow(m33,2)*RSe461p*RS345pe - 1./162.*pow(m33,2)*RSe461p*
         RS346pe - 1./162.*pow(m33,2)*RSe451p*RS345pe + 1./162.*pow(
         m33,2)*RSe451p*RS346pe - 1./162.*m13*m66*RSe463p*RS345pe - 1./
         162.*m13*m66*RSe463p*RS346pe - 1./54.*m13*m66*RSe453p*RS345pe
          - 1./162.*m13*m66*RSe453p*RS346pe + 1./162.*m13*m66*RSe461p*
         RS345pe + 1./162.*m13*m66*RSe461p*RS346pe + 1./54.*m13*m66*
         RSe451p*RS345pe + 1./162.*m13*m66*RSe451p*RS346pe - 1./81.*m13
         *m56*RSe463p*RS345pe + 1./81.*m13*m56*RSe463p*RS346pe + 1./81.
         *m13*m56*RSe453p*RS345pe - 1./81.*m13*m56*RSe453p*RS346pe + 1./
         81.*m13*m56*RSe461p*RS345pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,me2,L) * (  - 1./81.*m13*m56
         *RSe461p*RS346pe - 1./81.*m13*m56*RSe451p*RS345pe + 1./81.*m13
         *m56*RSe451p*RS346pe - 1./162.*m13*m55*RSe463p*RS345pe - 1./54.
         *m13*m55*RSe463p*RS346pe - 1./162.*m13*m55*RSe453p*RS345pe - 1.
         /162.*m13*m55*RSe453p*RS346pe + 1./162.*m13*m55*RSe461p*
         RS345pe + 1./54.*m13*m55*RSe461p*RS346pe + 1./162.*m13*m55*
         RSe451p*RS345pe + 1./162.*m13*m55*RSe451p*RS346pe - 1./81.*m13
         *m33*RSe463p*RS345pe + 1./81.*m13*m33*RSe463p*RS346pe + 1./81.
         *m13*m33*RSe453p*RS345pe - 1./81.*m13*m33*RSe453p*RS346pe + 1./
         81.*m13*m33*RSe461p*RS345pe - 1./81.*m13*m33*RSe461p*RS346pe
          - 1./81.*m13*m33*RSe451p*RS345pe + 1./81.*m13*m33*RSe451p*
         RS346pe + 1./324.*m11*m66*RSe463p*RS345pe + 1./324.*m11*m66*
         RSe463p*RS346pe + 1./108.*m11*m66*RSe453p*RS345pe + 1./324.*
         m11*m66*RSe453p*RS346pe + 1./324.*m11*m66*RSe461p*RS345pe + 1./
         324.*m11*m66*RSe461p*RS346pe + 1./108.*m11*m66*RSe451p*RS345pe
          + 1./324.*m11*m66*RSe451p*RS346pe );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,me2,L) * ( 1./162.*m11*m56*
         RSe463p*RS345pe - 1./162.*m11*m56*RSe463p*RS346pe - 1./162.*
         m11*m56*RSe453p*RS345pe + 1./162.*m11*m56*RSe453p*RS346pe + 1./
         162.*m11*m56*RSe461p*RS345pe - 1./162.*m11*m56*RSe461p*RS346pe
          - 1./162.*m11*m56*RSe451p*RS345pe + 1./162.*m11*m56*RSe451p*
         RS346pe + 1./324.*m11*m55*RSe463p*RS345pe + 1./108.*m11*m55*
         RSe463p*RS346pe + 1./324.*m11*m55*RSe453p*RS345pe + 1./324.*
         m11*m55*RSe453p*RS346pe + 1./324.*m11*m55*RSe461p*RS345pe + 1./
         108.*m11*m55*RSe461p*RS346pe + 1./324.*m11*m55*RSe451p*RS345pe
          + 1./324.*m11*m55*RSe451p*RS346pe + 1./162.*m11*m33*RSe463p*
         RS345pe - 1./162.*m11*m33*RSe463p*RS346pe - 1./162.*m11*m33*
         RSe453p*RS345pe + 1./162.*m11*m33*RSe453p*RS346pe + 1./162.*
         m11*m33*RSe461p*RS345pe - 1./162.*m11*m33*RSe461p*RS346pe - 1./
         162.*m11*m33*RSe451p*RS345pe + 1./162.*m11*m33*RSe451p*RS346pe
          );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,L) * ( 1./216.*m33*m66*pow(
         RS345pe,2) + 1./324.*m33*m66*RS346pe*RS345pe + 1./648.*m33*m66
         *pow(RS346pe,2) - 1./324.*m33*m56*pow(RS345pe,2) + 1./162.*m33
         *m56*RS346pe*RS345pe - 1./324.*m33*m56*pow(RS346pe,2) + 1./648.
         *m33*m55*pow(RS345pe,2) + 1./324.*m33*m55*RS346pe*RS345pe + 1./
         216.*m33*m55*pow(RS346pe,2) - 1./324.*pow(m33,2)*pow(
         RS345pe,2) + 1./162.*pow(m33,2)*RS346pe*RS345pe - 1./324.*pow(
         m33,2)*pow(RS346pe,2) - 1./108.*m13*m66*pow(RS345pe,2) - 1./
         162.*m13*m66*RS346pe*RS345pe - 1./324.*m13*m66*pow(RS346pe,2)
          + 1./162.*m13*m56*pow(RS345pe,2) - 1./81.*m13*m56*RS346pe*
         RS345pe + 1./162.*m13*m56*pow(RS346pe,2) - 1./324.*m13*m55*
         pow(RS345pe,2) - 1./162.*m13*m55*RS346pe*RS345pe - 1./108.*m13
         *m55*pow(RS346pe,2) + 1./162.*m13*m33*pow(RS345pe,2) - 1./81.*
         m13*m33*RS346pe*RS345pe + 1./162.*m13*m33*pow(RS346pe,2) + 1./
         216.*m11*m66*pow(RS345pe,2) + 1./324.*m11*m66*RS346pe*RS345pe
          + 1./648.*m11*m66*pow(RS346pe,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,L) * (  - 1./324.*m11*m56*
         pow(RS345pe,2) + 1./162.*m11*m56*RS346pe*RS345pe - 1./324.*m11
         *m56*pow(RS346pe,2) + 1./648.*m11*m55*pow(RS345pe,2) + 1./324.
         *m11*m55*RS346pe*RS345pe + 1./216.*m11*m55*pow(RS346pe,2) - 1./
         324.*m11*m33*pow(RS345pe,2) + 1./162.*m11*m33*RS346pe*RS345pe
          - 1./324.*m11*m33*pow(RS346pe,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m35,L) * ( 1./144.*m55*m66 - 1./
         72.*m55*m56 + 1./72.*pow(m55,2) - 1./72.*m35*m66 + 1./36.*m35*
         m56 - 1./24.*m35*m55 + 1./36.*pow(m35,2) + 1./72.*m33*m66 - 1./
         36.*m33*m56 + 5./144.*m33*m55 - 1./24.*m33*m35 + 1./72.*pow(
         m33,2) - 1./72.*m13*m66 + 1./36.*m13*m56 - 1./36.*m13*m55 + 1./
         36.*m13*m35 - 1./72.*m13*m33 + 1./144.*m11*m66 - 1./72.*m11*
         m56 + 1./72.*m11*m55 - 1./72.*m11*m35 + 1./144.*m11*m33 );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(m36,L) * ( 1./72.*pow(m66,2) - 1.
         /72.*m56*m66 + 1./144.*m55*m66 - 1./24.*m36*m66 + 1./36.*m36*
         m56 - 1./72.*m36*m55 + 1./36.*pow(m36,2) + 5./144.*m33*m66 - 1.
         /36.*m33*m56 + 1./72.*m33*m55 - 1./24.*m33*m36 + 1./72.*pow(
         m33,2) - 1./36.*m13*m66 + 1./36.*m13*m56 - 1./72.*m13*m55 + 1./
         36.*m13*m36 - 1./72.*m13*m33 + 1./72.*m11*m66 - 1./72.*m11*m56
          + 1./144.*m11*m55 - 1./72.*m11*m36 + 1./144.*m11*m33 );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(mp2,L) * ( 1./162.*pow(mp2,2)*
         pow(RSp463e,2) - 1./81.*pow(mp2,2)*RSp453e*RSp463e + 1./162.*
         pow(mp2,2)*pow(RSp453e,2) - 1./81.*pow(mp2,2)*RSp461e*RSp463e
          + 1./81.*pow(mp2,2)*RSp461e*RSp453e + 1./162.*pow(mp2,2)*pow(
         RSp461e,2) + 1./81.*pow(mp2,2)*RSp451e*RSp463e - 1./81.*pow(
         mp2,2)*RSp451e*RSp453e - 1./81.*pow(mp2,2)*RSp451e*RSp461e + 1.
         /162.*pow(mp2,2)*pow(RSp451e,2) - 1./324.*m66*mp2*pow(
         RSp463e,2) - 1./162.*m66*mp2*RSp453e*RSp463e - 1./108.*m66*mp2
         *pow(RSp453e,2) + 1./162.*m66*mp2*RSp461e*RSp463e + 1./162.*
         m66*mp2*RSp461e*RSp453e - 1./324.*m66*mp2*pow(RSp461e,2) + 1./
         162.*m66*mp2*RSp451e*RSp463e + 1./54.*m66*mp2*RSp451e*RSp453e
          - 1./162.*m66*mp2*RSp451e*RSp461e - 1./108.*m66*mp2*pow(
         RSp451e,2) + 1./162.*m56*mp2*pow(RSp463e,2) - 1./81.*m56*mp2*
         RSp453e*RSp463e + 1./162.*m56*mp2*pow(RSp453e,2) - 1./81.*m56*
         mp2*RSp461e*RSp463e + 1./81.*m56*mp2*RSp461e*RSp453e + 1./162.
         *m56*mp2*pow(RSp461e,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(mp2,L) * ( 1./81.*m56*mp2*
         RSp451e*RSp463e - 1./81.*m56*mp2*RSp451e*RSp453e - 1./81.*m56*
         mp2*RSp451e*RSp461e + 1./162.*m56*mp2*pow(RSp451e,2) - 1./108.
         *m55*mp2*pow(RSp463e,2) - 1./162.*m55*mp2*RSp453e*RSp463e - 1./
         324.*m55*mp2*pow(RSp453e,2) + 1./54.*m55*mp2*RSp461e*RSp463e
          + 1./162.*m55*mp2*RSp461e*RSp453e - 1./108.*m55*mp2*pow(
         RSp461e,2) + 1./162.*m55*mp2*RSp451e*RSp463e + 1./162.*m55*mp2
         *RSp451e*RSp453e - 1./162.*m55*mp2*RSp451e*RSp461e - 1./324.*
         m55*mp2*pow(RSp451e,2) - 1./108.*m33*mp2*pow(RSp463e,2) + 1./
         54.*m33*mp2*RSp453e*RSp463e - 1./108.*m33*mp2*pow(RSp453e,2)
          - 1./162.*m33*mp2*RSp461e*RSp463e + 1./162.*m33*mp2*RSp461e*
         RSp453e - 1./324.*m33*mp2*pow(RSp461e,2) + 1./162.*m33*mp2*
         RSp451e*RSp463e - 1./162.*m33*mp2*RSp451e*RSp453e + 1./162.*
         m33*mp2*RSp451e*RSp461e - 1./324.*m33*mp2*pow(RSp451e,2) + 1./
         216.*m33*m66*pow(RSp463e,2) + 1./108.*m33*m66*RSp453e*RSp463e
          + 1./72.*m33*m66*pow(RSp453e,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(mp2,L) * ( 1./324.*m33*m66*
         RSp461e*RSp463e + 1./324.*m33*m66*RSp461e*RSp453e + 1./648.*
         m33*m66*pow(RSp461e,2) + 1./324.*m33*m66*RSp451e*RSp463e + 1./
         108.*m33*m66*RSp451e*RSp453e + 1./324.*m33*m66*RSp451e*RSp461e
          + 1./216.*m33*m66*pow(RSp451e,2) - 1./108.*m33*m56*pow(
         RSp463e,2) + 1./54.*m33*m56*RSp453e*RSp463e - 1./108.*m33*m56*
         pow(RSp453e,2) - 1./162.*m33*m56*RSp461e*RSp463e + 1./162.*m33
         *m56*RSp461e*RSp453e - 1./324.*m33*m56*pow(RSp461e,2) + 1./162.
         *m33*m56*RSp451e*RSp463e - 1./162.*m33*m56*RSp451e*RSp453e + 1.
         /162.*m33*m56*RSp451e*RSp461e - 1./324.*m33*m56*pow(RSp451e,2)
          + 1./72.*m33*m55*pow(RSp463e,2) + 1./108.*m33*m55*RSp453e*
         RSp463e + 1./216.*m33*m55*pow(RSp453e,2) + 1./108.*m33*m55*
         RSp461e*RSp463e + 1./324.*m33*m55*RSp461e*RSp453e + 1./216.*
         m33*m55*pow(RSp461e,2) + 1./324.*m33*m55*RSp451e*RSp463e + 1./
         324.*m33*m55*RSp451e*RSp453e + 1./324.*m33*m55*RSp451e*RSp461e
          + 1./648.*m33*m55*pow(RSp451e,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(mp2,L) * ( 1./162.*m13*mp2*pow(
         RSp463e,2) - 1./81.*m13*mp2*RSp453e*RSp463e + 1./162.*m13*mp2*
         pow(RSp453e,2) - 1./81.*m13*mp2*RSp461e*RSp463e + 1./81.*m13*
         mp2*RSp461e*RSp453e + 1./162.*m13*mp2*pow(RSp461e,2) + 1./81.*
         m13*mp2*RSp451e*RSp463e - 1./81.*m13*mp2*RSp451e*RSp453e - 1./
         81.*m13*mp2*RSp451e*RSp461e + 1./162.*m13*mp2*pow(RSp451e,2)
          - 1./324.*m13*m66*pow(RSp463e,2) - 1./162.*m13*m66*RSp453e*
         RSp463e - 1./108.*m13*m66*pow(RSp453e,2) + 1./162.*m13*m66*
         RSp461e*RSp463e + 1./162.*m13*m66*RSp461e*RSp453e - 1./324.*
         m13*m66*pow(RSp461e,2) + 1./162.*m13*m66*RSp451e*RSp463e + 1./
         54.*m13*m66*RSp451e*RSp453e - 1./162.*m13*m66*RSp451e*RSp461e
          - 1./108.*m13*m66*pow(RSp451e,2) + 1./162.*m13*m56*pow(
         RSp463e,2) - 1./81.*m13*m56*RSp453e*RSp463e + 1./162.*m13*m56*
         pow(RSp453e,2) - 1./81.*m13*m56*RSp461e*RSp463e + 1./81.*m13*
         m56*RSp461e*RSp453e + 1./162.*m13*m56*pow(RSp461e,2) + 1./81.*
         m13*m56*RSp451e*RSp463e );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(mp2,L) * (  - 1./81.*m13*m56*
         RSp451e*RSp453e - 1./81.*m13*m56*RSp451e*RSp461e + 1./162.*m13
         *m56*pow(RSp451e,2) - 1./108.*m13*m55*pow(RSp463e,2) - 1./162.
         *m13*m55*RSp453e*RSp463e - 1./324.*m13*m55*pow(RSp453e,2) + 1./
         54.*m13*m55*RSp461e*RSp463e + 1./162.*m13*m55*RSp461e*RSp453e
          - 1./108.*m13*m55*pow(RSp461e,2) + 1./162.*m13*m55*RSp451e*
         RSp463e + 1./162.*m13*m55*RSp451e*RSp453e - 1./162.*m13*m55*
         RSp451e*RSp461e - 1./324.*m13*m55*pow(RSp451e,2) - 1./324.*m11
         *mp2*pow(RSp463e,2) + 1./162.*m11*mp2*RSp453e*RSp463e - 1./324.
         *m11*mp2*pow(RSp453e,2) - 1./162.*m11*mp2*RSp461e*RSp463e + 1./
         162.*m11*mp2*RSp461e*RSp453e - 1./108.*m11*mp2*pow(RSp461e,2)
          + 1./162.*m11*mp2*RSp451e*RSp463e - 1./162.*m11*mp2*RSp451e*
         RSp453e + 1./54.*m11*mp2*RSp451e*RSp461e - 1./108.*m11*mp2*
         pow(RSp451e,2) + 1./648.*m11*m66*pow(RSp463e,2) + 1./324.*m11*
         m66*RSp453e*RSp463e + 1./216.*m11*m66*pow(RSp453e,2) + 1./324.
         *m11*m66*RSp461e*RSp463e );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(mp2,L) * ( 1./324.*m11*m66*
         RSp461e*RSp453e + 1./216.*m11*m66*pow(RSp461e,2) + 1./324.*m11
         *m66*RSp451e*RSp463e + 1./108.*m11*m66*RSp451e*RSp453e + 1./
         108.*m11*m66*RSp451e*RSp461e + 1./72.*m11*m66*pow(RSp451e,2)
          - 1./324.*m11*m56*pow(RSp463e,2) + 1./162.*m11*m56*RSp453e*
         RSp463e - 1./324.*m11*m56*pow(RSp453e,2) - 1./162.*m11*m56*
         RSp461e*RSp463e + 1./162.*m11*m56*RSp461e*RSp453e - 1./108.*
         m11*m56*pow(RSp461e,2) + 1./162.*m11*m56*RSp451e*RSp463e - 1./
         162.*m11*m56*RSp451e*RSp453e + 1./54.*m11*m56*RSp451e*RSp461e
          - 1./108.*m11*m56*pow(RSp451e,2) + 1./216.*m11*m55*pow(
         RSp463e,2) + 1./324.*m11*m55*RSp453e*RSp463e + 1./648.*m11*m55
         *pow(RSp453e,2) + 1./108.*m11*m55*RSp461e*RSp463e + 1./324.*
         m11*m55*RSp461e*RSp453e + 1./72.*m11*m55*pow(RSp461e,2) + 1./
         324.*m11*m55*RSp451e*RSp463e + 1./324.*m11*m55*RSp451e*RSp453e
          + 1./108.*m11*m55*RSp451e*RSp461e + 1./216.*m11*m55*pow(
         RSp451e,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * ( 1./81.*mp2*me2*
         RSp463e*RSe463p - 1./81.*mp2*me2*RSp463e*RSe453p - 1./81.*mp2*
         me2*RSp453e*RSe463p + 1./81.*mp2*me2*RSp453e*RSe453p - 1./81.*
         mp2*me2*RSe461p*RSp463e + 1./81.*mp2*me2*RSe461p*RSp453e + 1./
         81.*mp2*me2*RSe451p*RSp463e - 1./81.*mp2*me2*RSe451p*RSp453e
          - 1./81.*mp2*me2*RSp461e*RSe463p + 1./81.*mp2*me2*RSp461e*
         RSe453p + 1./81.*mp2*me2*RSp461e*RSe461p - 1./81.*mp2*me2*
         RSp461e*RSe451p + 1./81.*mp2*me2*RSp451e*RSe463p - 1./81.*mp2*
         me2*RSp451e*RSe453p - 1./81.*mp2*me2*RSp451e*RSe461p + 1./81.*
         mp2*me2*RSp451e*RSe451p - 1./162.*m66*me2*RSp463e*RSe463p - 1./
         162.*m66*me2*RSp463e*RSe453p - 1./162.*m66*me2*RSp453e*RSe463p
          - 1./54.*m66*me2*RSp453e*RSe453p + 1./162.*m66*me2*RSe461p*
         RSp463e + 1./162.*m66*me2*RSe461p*RSp453e + 1./162.*m66*me2*
         RSe451p*RSp463e + 1./54.*m66*me2*RSe451p*RSp453e + 1./162.*m66
         *me2*RSp461e*RSe463p + 1./162.*m66*me2*RSp461e*RSe453p - 1./
         162.*m66*me2*RSp461e*RSe461p );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * (  - 1./162.*m66*
         me2*RSp461e*RSe451p + 1./162.*m66*me2*RSp451e*RSe463p + 1./54.
         *m66*me2*RSp451e*RSe453p - 1./162.*m66*me2*RSp451e*RSe461p - 1.
         /54.*m66*me2*RSp451e*RSe451p + 1./81.*m56*me2*RSp463e*RSe463p
          - 1./81.*m56*me2*RSp463e*RSe453p - 1./81.*m56*me2*RSp453e*
         RSe463p + 1./81.*m56*me2*RSp453e*RSe453p - 1./81.*m56*me2*
         RSe461p*RSp463e + 1./81.*m56*me2*RSe461p*RSp453e + 1./81.*m56*
         me2*RSe451p*RSp463e - 1./81.*m56*me2*RSe451p*RSp453e - 1./81.*
         m56*me2*RSp461e*RSe463p + 1./81.*m56*me2*RSp461e*RSe453p + 1./
         81.*m56*me2*RSp461e*RSe461p - 1./81.*m56*me2*RSp461e*RSe451p
          + 1./81.*m56*me2*RSp451e*RSe463p - 1./81.*m56*me2*RSp451e*
         RSe453p - 1./81.*m56*me2*RSp451e*RSe461p + 1./81.*m56*me2*
         RSp451e*RSe451p - 1./54.*m55*me2*RSp463e*RSe463p - 1./162.*m55
         *me2*RSp463e*RSe453p - 1./162.*m55*me2*RSp453e*RSe463p - 1./
         162.*m55*me2*RSp453e*RSe453p + 1./54.*m55*me2*RSe461p*RSp463e
          + 1./162.*m55*me2*RSe461p*RSp453e );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * ( 1./162.*m55*me2*
         RSe451p*RSp463e + 1./162.*m55*me2*RSe451p*RSp453e + 1./54.*m55
         *me2*RSp461e*RSe463p + 1./162.*m55*me2*RSp461e*RSe453p - 1./54.
         *m55*me2*RSp461e*RSe461p - 1./162.*m55*me2*RSp461e*RSe451p + 1.
         /162.*m55*me2*RSp451e*RSe463p + 1./162.*m55*me2*RSp451e*
         RSe453p - 1./162.*m55*me2*RSp451e*RSe461p - 1./162.*m55*me2*
         RSp451e*RSe451p - 1./54.*m33*me2*RSp463e*RSe463p + 1./54.*m33*
         me2*RSp463e*RSe453p + 1./54.*m33*me2*RSp453e*RSe463p - 1./54.*
         m33*me2*RSp453e*RSe453p - 1./162.*m33*me2*RSe461p*RSp463e + 1./
         162.*m33*me2*RSe461p*RSp453e + 1./162.*m33*me2*RSe451p*RSp463e
          - 1./162.*m33*me2*RSe451p*RSp453e - 1./162.*m33*me2*RSp461e*
         RSe463p + 1./162.*m33*me2*RSp461e*RSe453p - 1./162.*m33*me2*
         RSp461e*RSe461p + 1./162.*m33*me2*RSp461e*RSe451p + 1./162.*
         m33*me2*RSp451e*RSe463p - 1./162.*m33*me2*RSp451e*RSe453p + 1./
         162.*m33*me2*RSp451e*RSe461p - 1./162.*m33*me2*RSp451e*RSe451p
          + 1./108.*m33*m66*RSp463e*RSe463p );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * ( 1./108.*m33*m66*
         RSp463e*RSe453p + 1./108.*m33*m66*RSp453e*RSe463p + 1./36.*m33
         *m66*RSp453e*RSe453p + 1./324.*m33*m66*RSe461p*RSp463e + 1./
         324.*m33*m66*RSe461p*RSp453e + 1./324.*m33*m66*RSe451p*RSp463e
          + 1./108.*m33*m66*RSe451p*RSp453e + 1./324.*m33*m66*RSp461e*
         RSe463p + 1./324.*m33*m66*RSp461e*RSe453p + 1./324.*m33*m66*
         RSp461e*RSe461p + 1./324.*m33*m66*RSp461e*RSe451p + 1./324.*
         m33*m66*RSp451e*RSe463p + 1./108.*m33*m66*RSp451e*RSe453p + 1./
         324.*m33*m66*RSp451e*RSe461p + 1./108.*m33*m66*RSp451e*RSe451p
          - 1./54.*m33*m56*RSp463e*RSe463p + 1./54.*m33*m56*RSp463e*
         RSe453p + 1./54.*m33*m56*RSp453e*RSe463p - 1./54.*m33*m56*
         RSp453e*RSe453p - 1./162.*m33*m56*RSe461p*RSp463e + 1./162.*
         m33*m56*RSe461p*RSp453e + 1./162.*m33*m56*RSe451p*RSp463e - 1./
         162.*m33*m56*RSe451p*RSp453e - 1./162.*m33*m56*RSp461e*RSe463p
          + 1./162.*m33*m56*RSp461e*RSe453p - 1./162.*m33*m56*RSp461e*
         RSe461p );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * ( 1./162.*m33*m56*
         RSp461e*RSe451p + 1./162.*m33*m56*RSp451e*RSe463p - 1./162.*
         m33*m56*RSp451e*RSe453p + 1./162.*m33*m56*RSp451e*RSe461p - 1./
         162.*m33*m56*RSp451e*RSe451p + 1./36.*m33*m55*RSp463e*RSe463p
          + 1./108.*m33*m55*RSp463e*RSe453p + 1./108.*m33*m55*RSp453e*
         RSe463p + 1./108.*m33*m55*RSp453e*RSe453p + 1./108.*m33*m55*
         RSe461p*RSp463e + 1./324.*m33*m55*RSe461p*RSp453e + 1./324.*
         m33*m55*RSe451p*RSp463e + 1./324.*m33*m55*RSe451p*RSp453e + 1./
         108.*m33*m55*RSp461e*RSe463p + 1./324.*m33*m55*RSp461e*RSe453p
          + 1./108.*m33*m55*RSp461e*RSe461p + 1./324.*m33*m55*RSp461e*
         RSe451p + 1./324.*m33*m55*RSp451e*RSe463p + 1./324.*m33*m55*
         RSp451e*RSe453p + 1./324.*m33*m55*RSp451e*RSe461p + 1./324.*
         m33*m55*RSp451e*RSe451p + 1./81.*m13*me2*RSp463e*RSe463p - 1./
         81.*m13*me2*RSp463e*RSe453p - 1./81.*m13*me2*RSp453e*RSe463p
          + 1./81.*m13*me2*RSp453e*RSe453p - 1./81.*m13*me2*RSe461p*
         RSp463e );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * ( 1./81.*m13*me2*
         RSe461p*RSp453e + 1./81.*m13*me2*RSe451p*RSp463e - 1./81.*m13*
         me2*RSe451p*RSp453e - 1./81.*m13*me2*RSp461e*RSe463p + 1./81.*
         m13*me2*RSp461e*RSe453p + 1./81.*m13*me2*RSp461e*RSe461p - 1./
         81.*m13*me2*RSp461e*RSe451p + 1./81.*m13*me2*RSp451e*RSe463p
          - 1./81.*m13*me2*RSp451e*RSe453p - 1./81.*m13*me2*RSp451e*
         RSe461p + 1./81.*m13*me2*RSp451e*RSe451p - 1./162.*m13*m66*
         RSp463e*RSe463p - 1./162.*m13*m66*RSp463e*RSe453p - 1./162.*
         m13*m66*RSp453e*RSe463p - 1./54.*m13*m66*RSp453e*RSe453p + 1./
         162.*m13*m66*RSe461p*RSp463e + 1./162.*m13*m66*RSe461p*RSp453e
          + 1./162.*m13*m66*RSe451p*RSp463e + 1./54.*m13*m66*RSe451p*
         RSp453e + 1./162.*m13*m66*RSp461e*RSe463p + 1./162.*m13*m66*
         RSp461e*RSe453p - 1./162.*m13*m66*RSp461e*RSe461p - 1./162.*
         m13*m66*RSp461e*RSe451p + 1./162.*m13*m66*RSp451e*RSe463p + 1./
         54.*m13*m66*RSp451e*RSe453p - 1./162.*m13*m66*RSp451e*RSe461p
          - 1./54.*m13*m66*RSp451e*RSe451p );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * ( 1./81.*m13*m56*
         RSp463e*RSe463p - 1./81.*m13*m56*RSp463e*RSe453p - 1./81.*m13*
         m56*RSp453e*RSe463p + 1./81.*m13*m56*RSp453e*RSe453p - 1./81.*
         m13*m56*RSe461p*RSp463e + 1./81.*m13*m56*RSe461p*RSp453e + 1./
         81.*m13*m56*RSe451p*RSp463e - 1./81.*m13*m56*RSe451p*RSp453e
          - 1./81.*m13*m56*RSp461e*RSe463p + 1./81.*m13*m56*RSp461e*
         RSe453p + 1./81.*m13*m56*RSp461e*RSe461p - 1./81.*m13*m56*
         RSp461e*RSe451p + 1./81.*m13*m56*RSp451e*RSe463p - 1./81.*m13*
         m56*RSp451e*RSe453p - 1./81.*m13*m56*RSp451e*RSe461p + 1./81.*
         m13*m56*RSp451e*RSe451p - 1./54.*m13*m55*RSp463e*RSe463p - 1./
         162.*m13*m55*RSp463e*RSe453p - 1./162.*m13*m55*RSp453e*RSe463p
          - 1./162.*m13*m55*RSp453e*RSe453p + 1./54.*m13*m55*RSe461p*
         RSp463e + 1./162.*m13*m55*RSe461p*RSp453e + 1./162.*m13*m55*
         RSe451p*RSp463e + 1./162.*m13*m55*RSe451p*RSp453e + 1./54.*m13
         *m55*RSp461e*RSe463p + 1./162.*m13*m55*RSp461e*RSe453p - 1./54.
         *m13*m55*RSp461e*RSe461p );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * (  - 1./162.*m13*
         m55*RSp461e*RSe451p + 1./162.*m13*m55*RSp451e*RSe463p + 1./162.
         *m13*m55*RSp451e*RSe453p - 1./162.*m13*m55*RSp451e*RSe461p - 1.
         /162.*m13*m55*RSp451e*RSe451p - 1./162.*m11*me2*RSp463e*
         RSe463p + 1./162.*m11*me2*RSp463e*RSe453p + 1./162.*m11*me2*
         RSp453e*RSe463p - 1./162.*m11*me2*RSp453e*RSe453p - 1./162.*
         m11*me2*RSe461p*RSp463e + 1./162.*m11*me2*RSe461p*RSp453e + 1./
         162.*m11*me2*RSe451p*RSp463e - 1./162.*m11*me2*RSe451p*RSp453e
          - 1./162.*m11*me2*RSp461e*RSe463p + 1./162.*m11*me2*RSp461e*
         RSe453p - 1./54.*m11*me2*RSp461e*RSe461p + 1./54.*m11*me2*
         RSp461e*RSe451p + 1./162.*m11*me2*RSp451e*RSe463p - 1./162.*
         m11*me2*RSp451e*RSe453p + 1./54.*m11*me2*RSp451e*RSe461p - 1./
         54.*m11*me2*RSp451e*RSe451p + 1./324.*m11*m66*RSp463e*RSe463p
          + 1./324.*m11*m66*RSp463e*RSe453p + 1./324.*m11*m66*RSp453e*
         RSe463p + 1./108.*m11*m66*RSp453e*RSe453p + 1./324.*m11*m66*
         RSe461p*RSp463e );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * ( 1./324.*m11*m66*
         RSe461p*RSp453e + 1./324.*m11*m66*RSe451p*RSp463e + 1./108.*
         m11*m66*RSe451p*RSp453e + 1./324.*m11*m66*RSp461e*RSe463p + 1./
         324.*m11*m66*RSp461e*RSe453p + 1./108.*m11*m66*RSp461e*RSe461p
          + 1./108.*m11*m66*RSp461e*RSe451p + 1./324.*m11*m66*RSp451e*
         RSe463p + 1./108.*m11*m66*RSp451e*RSe453p + 1./108.*m11*m66*
         RSp451e*RSe461p + 1./36.*m11*m66*RSp451e*RSe451p - 1./162.*m11
         *m56*RSp463e*RSe463p + 1./162.*m11*m56*RSp463e*RSe453p + 1./
         162.*m11*m56*RSp453e*RSe463p - 1./162.*m11*m56*RSp453e*RSe453p
          - 1./162.*m11*m56*RSe461p*RSp463e + 1./162.*m11*m56*RSe461p*
         RSp453e + 1./162.*m11*m56*RSe451p*RSp463e - 1./162.*m11*m56*
         RSe451p*RSp453e - 1./162.*m11*m56*RSp461e*RSe463p + 1./162.*
         m11*m56*RSp461e*RSe453p - 1./54.*m11*m56*RSp461e*RSe461p + 1./
         54.*m11*m56*RSp461e*RSe451p + 1./162.*m11*m56*RSp451e*RSe463p
          - 1./162.*m11*m56*RSp451e*RSe453p + 1./54.*m11*m56*RSp451e*
         RSe461p );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * (  - 1./54.*m11*m56
         *RSp451e*RSe451p + 1./108.*m11*m55*RSp463e*RSe463p + 1./324.*
         m11*m55*RSp463e*RSe453p + 1./324.*m11*m55*RSp453e*RSe463p + 1./
         324.*m11*m55*RSp453e*RSe453p + 1./108.*m11*m55*RSe461p*RSp463e
          + 1./324.*m11*m55*RSe461p*RSp453e + 1./324.*m11*m55*RSe451p*
         RSp463e + 1./324.*m11*m55*RSe451p*RSp453e + 1./108.*m11*m55*
         RSp461e*RSe463p + 1./324.*m11*m55*RSp461e*RSe453p + 1./36.*m11
         *m55*RSp461e*RSe461p + 1./108.*m11*m55*RSp461e*RSe451p + 1./
         324.*m11*m55*RSp451e*RSe463p + 1./324.*m11*m55*RSp451e*RSe453p
          + 1./108.*m11*m55*RSp451e*RSe461p + 1./108.*m11*m55*RSp451e*
         RSe451p );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,L) * ( 1./162.*pow(me2,2)*
         pow(RSe463p,2) - 1./81.*pow(me2,2)*RSe453p*RSe463p + 1./162.*
         pow(me2,2)*pow(RSe453p,2) - 1./81.*pow(me2,2)*RSe461p*RSe463p
          + 1./81.*pow(me2,2)*RSe461p*RSe453p + 1./162.*pow(me2,2)*pow(
         RSe461p,2) + 1./81.*pow(me2,2)*RSe451p*RSe463p - 1./81.*pow(
         me2,2)*RSe451p*RSe453p - 1./81.*pow(me2,2)*RSe451p*RSe461p + 1.
         /162.*pow(me2,2)*pow(RSe451p,2) - 1./324.*m66*me2*pow(
         RSe463p,2) - 1./162.*m66*me2*RSe453p*RSe463p - 1./108.*m66*me2
         *pow(RSe453p,2) + 1./162.*m66*me2*RSe461p*RSe463p + 1./162.*
         m66*me2*RSe461p*RSe453p - 1./324.*m66*me2*pow(RSe461p,2) + 1./
         162.*m66*me2*RSe451p*RSe463p + 1./54.*m66*me2*RSe451p*RSe453p
          - 1./162.*m66*me2*RSe451p*RSe461p - 1./108.*m66*me2*pow(
         RSe451p,2) + 1./162.*m56*me2*pow(RSe463p,2) - 1./81.*m56*me2*
         RSe453p*RSe463p + 1./162.*m56*me2*pow(RSe453p,2) - 1./81.*m56*
         me2*RSe461p*RSe463p + 1./81.*m56*me2*RSe461p*RSe453p + 1./162.
         *m56*me2*pow(RSe461p,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,L) * ( 1./81.*m56*me2*
         RSe451p*RSe463p - 1./81.*m56*me2*RSe451p*RSe453p - 1./81.*m56*
         me2*RSe451p*RSe461p + 1./162.*m56*me2*pow(RSe451p,2) - 1./108.
         *m55*me2*pow(RSe463p,2) - 1./162.*m55*me2*RSe453p*RSe463p - 1./
         324.*m55*me2*pow(RSe453p,2) + 1./54.*m55*me2*RSe461p*RSe463p
          + 1./162.*m55*me2*RSe461p*RSe453p - 1./108.*m55*me2*pow(
         RSe461p,2) + 1./162.*m55*me2*RSe451p*RSe463p + 1./162.*m55*me2
         *RSe451p*RSe453p - 1./162.*m55*me2*RSe451p*RSe461p - 1./324.*
         m55*me2*pow(RSe451p,2) - 1./108.*m33*me2*pow(RSe463p,2) + 1./
         54.*m33*me2*RSe453p*RSe463p - 1./108.*m33*me2*pow(RSe453p,2)
          - 1./162.*m33*me2*RSe461p*RSe463p + 1./162.*m33*me2*RSe461p*
         RSe453p - 1./324.*m33*me2*pow(RSe461p,2) + 1./162.*m33*me2*
         RSe451p*RSe463p - 1./162.*m33*me2*RSe451p*RSe453p + 1./162.*
         m33*me2*RSe451p*RSe461p - 1./324.*m33*me2*pow(RSe451p,2) + 1./
         216.*m33*m66*pow(RSe463p,2) + 1./108.*m33*m66*RSe453p*RSe463p
          + 1./72.*m33*m66*pow(RSe453p,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,L) * ( 1./324.*m33*m66*
         RSe461p*RSe463p + 1./324.*m33*m66*RSe461p*RSe453p + 1./648.*
         m33*m66*pow(RSe461p,2) + 1./324.*m33*m66*RSe451p*RSe463p + 1./
         108.*m33*m66*RSe451p*RSe453p + 1./324.*m33*m66*RSe451p*RSe461p
          + 1./216.*m33*m66*pow(RSe451p,2) - 1./108.*m33*m56*pow(
         RSe463p,2) + 1./54.*m33*m56*RSe453p*RSe463p - 1./108.*m33*m56*
         pow(RSe453p,2) - 1./162.*m33*m56*RSe461p*RSe463p + 1./162.*m33
         *m56*RSe461p*RSe453p - 1./324.*m33*m56*pow(RSe461p,2) + 1./162.
         *m33*m56*RSe451p*RSe463p - 1./162.*m33*m56*RSe451p*RSe453p + 1.
         /162.*m33*m56*RSe451p*RSe461p - 1./324.*m33*m56*pow(RSe451p,2)
          + 1./72.*m33*m55*pow(RSe463p,2) + 1./108.*m33*m55*RSe453p*
         RSe463p + 1./216.*m33*m55*pow(RSe453p,2) + 1./108.*m33*m55*
         RSe461p*RSe463p + 1./324.*m33*m55*RSe461p*RSe453p + 1./216.*
         m33*m55*pow(RSe461p,2) + 1./324.*m33*m55*RSe451p*RSe463p + 1./
         324.*m33*m55*RSe451p*RSe453p + 1./324.*m33*m55*RSe451p*RSe461p
          + 1./648.*m33*m55*pow(RSe451p,2) );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,L) * ( 1./162.*m13*me2*pow(
         RSe463p,2) - 1./81.*m13*me2*RSe453p*RSe463p + 1./162.*m13*me2*
         pow(RSe453p,2) - 1./81.*m13*me2*RSe461p*RSe463p + 1./81.*m13*
         me2*RSe461p*RSe453p + 1./162.*m13*me2*pow(RSe461p,2) + 1./81.*
         m13*me2*RSe451p*RSe463p - 1./81.*m13*me2*RSe451p*RSe453p - 1./
         81.*m13*me2*RSe451p*RSe461p + 1./162.*m13*me2*pow(RSe451p,2)
          - 1./324.*m13*m66*pow(RSe463p,2) - 1./162.*m13*m66*RSe453p*
         RSe463p - 1./108.*m13*m66*pow(RSe453p,2) + 1./162.*m13*m66*
         RSe461p*RSe463p + 1./162.*m13*m66*RSe461p*RSe453p - 1./324.*
         m13*m66*pow(RSe461p,2) + 1./162.*m13*m66*RSe451p*RSe463p + 1./
         54.*m13*m66*RSe451p*RSe453p - 1./162.*m13*m66*RSe451p*RSe461p
          - 1./108.*m13*m66*pow(RSe451p,2) + 1./162.*m13*m56*pow(
         RSe463p,2) - 1./81.*m13*m56*RSe453p*RSe463p + 1./162.*m13*m56*
         pow(RSe453p,2) - 1./81.*m13*m56*RSe461p*RSe463p + 1./81.*m13*
         m56*RSe461p*RSe453p + 1./162.*m13*m56*pow(RSe461p,2) + 1./81.*
         m13*m56*RSe451p*RSe463p );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,L) * (  - 1./81.*m13*m56*
         RSe451p*RSe453p - 1./81.*m13*m56*RSe451p*RSe461p + 1./162.*m13
         *m56*pow(RSe451p,2) - 1./108.*m13*m55*pow(RSe463p,2) - 1./162.
         *m13*m55*RSe453p*RSe463p - 1./324.*m13*m55*pow(RSe453p,2) + 1./
         54.*m13*m55*RSe461p*RSe463p + 1./162.*m13*m55*RSe461p*RSe453p
          - 1./108.*m13*m55*pow(RSe461p,2) + 1./162.*m13*m55*RSe451p*
         RSe463p + 1./162.*m13*m55*RSe451p*RSe453p - 1./162.*m13*m55*
         RSe451p*RSe461p - 1./324.*m13*m55*pow(RSe451p,2) - 1./324.*m11
         *me2*pow(RSe463p,2) + 1./162.*m11*me2*RSe453p*RSe463p - 1./324.
         *m11*me2*pow(RSe453p,2) - 1./162.*m11*me2*RSe461p*RSe463p + 1./
         162.*m11*me2*RSe461p*RSe453p - 1./108.*m11*me2*pow(RSe461p,2)
          + 1./162.*m11*me2*RSe451p*RSe463p - 1./162.*m11*me2*RSe451p*
         RSe453p + 1./54.*m11*me2*RSe451p*RSe461p - 1./108.*m11*me2*
         pow(RSe451p,2) + 1./648.*m11*m66*pow(RSe463p,2) + 1./324.*m11*
         m66*RSe453p*RSe463p + 1./216.*m11*m66*pow(RSe453p,2) + 1./324.
         *m11*m66*RSe461p*RSe463p );

      M13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,L) * ( 1./324.*m11*m66*
         RSe461p*RSe453p + 1./216.*m11*m66*pow(RSe461p,2) + 1./324.*m11
         *m66*RSe451p*RSe463p + 1./108.*m11*m66*RSe451p*RSe453p + 1./
         108.*m11*m66*RSe451p*RSe461p + 1./72.*m11*m66*pow(RSe451p,2)
          - 1./324.*m11*m56*pow(RSe463p,2) + 1./162.*m11*m56*RSe453p*
         RSe463p - 1./324.*m11*m56*pow(RSe453p,2) - 1./162.*m11*m56*
         RSe461p*RSe463p + 1./162.*m11*m56*RSe461p*RSe453p - 1./108.*
         m11*m56*pow(RSe461p,2) + 1./162.*m11*m56*RSe451p*RSe463p - 1./
         162.*m11*m56*RSe451p*RSe453p + 1./54.*m11*m56*RSe451p*RSe461p
          - 1./108.*m11*m56*pow(RSe451p,2) + 1./216.*m11*m55*pow(
         RSe463p,2) + 1./324.*m11*m55*RSe453p*RSe463p + 1./648.*m11*m55
         *pow(RSe453p,2) + 1./108.*m11*m55*RSe461p*RSe463p + 1./324.*
         m11*m55*RSe461p*RSe453p + 1./72.*m11*m55*pow(RSe461p,2) + 1./
         324.*m11*m55*RSe451p*RSe463p + 1./324.*m11*m55*RSe451p*RSe453p
          + 1./108.*m11*m55*RSe451p*RSe461p + 1./216.*m11*m55*pow(
         RSe451p,2) );

      M13p6x23RV +=  + Ab(m66,mu2)*AbV(m11,L) * (  - 1./54.*m66*pow(
         RS145pe,2) + 1./162.*m66*RS645pe*pow(RS145pe,2) );

      M13p6x23RV +=  + Ab(m66,mu2)*AbV(m16,L) * (  - 1./20.*m66 + 1./60.
         *m66*RS645pe + 1./36.*m36 - 1./108.*m36*RS645pe + 7./360.*m33
          - 7./1080.*m33*RS645pe + 13./180.*m16 - 13./540.*m16*RS645pe
          - 1./15.*m13 + 1./45.*m13*RS645pe - 1./360.*m11 + 1./1080.*
         m11*RS645pe );

      M13p6x23RV +=  + Ab(m66,mu2)*AbV(m33,L) * (  - 1./54.*m66*pow(
         RS345pe,2) + 1./27.*m66*RS145pe*RS345pe + 1./162.*m66*RS645pe*
         pow(RS345pe,2) - 1./81.*m66*RS645pe*RS145pe*RS345pe );

      M13p6x23RV +=  + Ab(m66,mu2)*AbV(m36,L) * (  - 1./20.*m66 + 1./60.
         *m66*RS645pe + 13./180.*m36 - 13./540.*m36*RS645pe - 1./360.*
         m33 + 1./1080.*m33*RS645pe + 1./36.*m16 - 1./108.*m16*RS645pe
          - 1./15.*m13 + 1./45.*m13*RS645pe + 7./360.*m11 - 7./1080.*
         m11*RS645pe );

      M13p6x23RV +=  + Ab(m66,mu2)*AbV(mp2,L) * (  - 1./27.*m66*RSp453e
         *RS345pe - 1./27.*m66*RSp453e*RSe453p - 1./54.*m66*pow(
         RSp453e,2) + 1./27.*m66*RS145pe*RSp453e + 1./81.*m66*RS645pe*
         RSp453e*RS345pe + 1./81.*m66*RS645pe*RSp453e*RSe453p + 1./162.
         *m66*RS645pe*pow(RSp453e,2) - 1./81.*m66*RS645pe*RS145pe*
         RSp453e + 1./27.*m66*RSe451p*RSp453e - 1./81.*m66*RSe451p*
         RS645pe*RSp453e + 1./27.*m66*RSp451e*RS345pe + 1./27.*m66*
         RSp451e*RSe453p + 1./27.*m66*RSp451e*RSp453e - 1./27.*m66*
         RSp451e*RS145pe - 1./81.*m66*RSp451e*RS645pe*RS345pe - 1./81.*
         m66*RSp451e*RS645pe*RSe453p - 1./81.*m66*RSp451e*RS645pe*
         RSp453e + 1./81.*m66*RSp451e*RS645pe*RS145pe - 1./27.*m66*
         RSp451e*RSe451p + 1./81.*m66*RSp451e*RSe451p*RS645pe - 1./54.*
         m66*pow(RSp451e,2) + 1./162.*m66*pow(RSp451e,2)*RS645pe );

      M13p6x23RV +=  + Ab(m66,mu2)*AbV(me2,L) * (  - 1./27.*m66*RSe453p
         *RS345pe - 1./54.*m66*pow(RSe453p,2) + 1./27.*m66*RS145pe*
         RSe453p + 1./81.*m66*RS645pe*RSe453p*RS345pe + 1./162.*m66*
         RS645pe*pow(RSe453p,2) - 1./81.*m66*RS645pe*RS145pe*RSe453p + 
         1./27.*m66*RSe451p*RS345pe + 1./27.*m66*RSe451p*RSe453p - 1./
         27.*m66*RSe451p*RS145pe - 1./81.*m66*RSe451p*RS645pe*RS345pe
          - 1./81.*m66*RSe451p*RS645pe*RSe453p + 1./81.*m66*RSe451p*
         RS645pe*RS145pe - 1./54.*m66*pow(RSe451p,2) + 1./162.*m66*pow(
         RSe451p,2)*RS645pe );

      M13p6x23RV +=  + Ab(m66,mu2)*BbV(m11,m33,L) * ( 1./54.*m33*m66*
         RS145pe*RS345pe - 1./162.*m33*m66*RS645pe*RS145pe*RS345pe + 1./
         27.*m13*m66*RS145pe*RS345pe - 1./81.*m13*m66*RS645pe*RS145pe*
         RS345pe + 1./18.*m11*m66*RS145pe*RS345pe - 1./54.*m11*m66*
         RS645pe*RS145pe*RS345pe );

      M13p6x23RV +=  + Ab(m66,mu2)*BbV(m11,mp2,L) * ( 1./54.*m33*m66*
         RS145pe*RSp453e - 1./162.*m33*m66*RS645pe*RS145pe*RSp453e + 1./
         54.*m33*m66*RSp451e*RS145pe - 1./162.*m33*m66*RSp451e*RS645pe*
         RS145pe + 1./27.*m13*m66*RS145pe*RSp453e - 1./81.*m13*m66*
         RS645pe*RS145pe*RSp453e - 1./27.*m13*m66*RSp451e*RS145pe + 1./
         81.*m13*m66*RSp451e*RS645pe*RS145pe + 1./18.*m11*m66*RS145pe*
         RSp453e - 1./54.*m11*m66*RS645pe*RS145pe*RSp453e + 1./54.*m11*
         m66*RSp451e*RS145pe - 1./162.*m11*m66*RSp451e*RS645pe*RS145pe
          );

      M13p6x23RV +=  + Ab(m66,mu2)*BbV(m11,me2,L) * ( 1./54.*m33*m66*
         RS145pe*RSe453p - 1./162.*m33*m66*RS645pe*RS145pe*RSe453p + 1./
         54.*m33*m66*RSe451p*RS145pe - 1./162.*m33*m66*RSe451p*RS645pe*
         RS145pe + 1./27.*m13*m66*RS145pe*RSe453p - 1./81.*m13*m66*
         RS645pe*RS145pe*RSe453p - 1./27.*m13*m66*RSe451p*RS145pe + 1./
         81.*m13*m66*RSe451p*RS645pe*RS145pe + 1./18.*m11*m66*RS145pe*
         RSe453p - 1./54.*m11*m66*RS645pe*RS145pe*RSe453p + 1./54.*m11*
         m66*RSe451p*RS145pe - 1./162.*m11*m66*RSe451p*RS645pe*RS145pe
          );

      M13p6x23RV +=  + Ab(m66,mu2)*BbV(m11,L) * ( 1./108.*m33*m66*pow(
         RS145pe,2) - 1./324.*m33*m66*RS645pe*pow(RS145pe,2) - 1./54.*
         m13*m66*pow(RS145pe,2) + 1./162.*m13*m66*RS645pe*pow(
         RS145pe,2) + 1./108.*m11*m66*pow(RS145pe,2) - 1./324.*m11*m66*
         RS645pe*pow(RS145pe,2) );

      M13p6x23RV +=  + Ab(m66,mu2)*BbV(m16,L) * ( 1./144.*pow(m66,2) - 
         1./432.*pow(m66,2)*RS645pe + 1./144.*m33*m66 - 1./432.*m33*m66
         *RS645pe - 1./36.*m16*m66 + 1./108.*m16*m66*RS645pe - 1./72.*
         m16*m33 + 1./216.*m16*m33*RS645pe + 1./36.*pow(m16,2) - 1./108.
         *pow(m16,2)*RS645pe - 1./72.*m13*m66 + 1./216.*m13*m66*RS645pe
          + 1./36.*m13*m16 - 1./108.*m13*m16*RS645pe + 1./48.*m11*m66
          - 1./144.*m11*m66*RS645pe + 1./144.*m11*m33 - 1./432.*m11*m33
         *RS645pe - 1./24.*m11*m16 + 1./72.*m11*m16*RS645pe - 1./72.*
         m11*m13 + 1./216.*m11*m13*RS645pe + 1./72.*pow(m11,2) - 1./216.
         *pow(m11,2)*RS645pe );

      M13p6x23RV +=  + Ab(m66,mu2)*BbV(m33,mp2,L) * ( 1./54.*m33*m66*
         RSp453e*RS345pe - 1./162.*m33*m66*RS645pe*RSp453e*RS345pe + 1./
         18.*m33*m66*RSp451e*RS345pe - 1./54.*m33*m66*RSp451e*RS645pe*
         RS345pe - 1./27.*m13*m66*RSp453e*RS345pe + 1./81.*m13*m66*
         RS645pe*RSp453e*RS345pe + 1./27.*m13*m66*RSp451e*RS345pe - 1./
         81.*m13*m66*RSp451e*RS645pe*RS345pe + 1./54.*m11*m66*RSp453e*
         RS345pe - 1./162.*m11*m66*RS645pe*RSp453e*RS345pe + 1./54.*m11
         *m66*RSp451e*RS345pe - 1./162.*m11*m66*RSp451e*RS645pe*RS345pe
          );

      M13p6x23RV +=  + Ab(m66,mu2)*BbV(m33,me2,L) * ( 1./54.*m33*m66*
         RSe453p*RS345pe - 1./162.*m33*m66*RS645pe*RSe453p*RS345pe + 1./
         18.*m33*m66*RSe451p*RS345pe - 1./54.*m33*m66*RSe451p*RS645pe*
         RS345pe - 1./27.*m13*m66*RSe453p*RS345pe + 1./81.*m13*m66*
         RS645pe*RSe453p*RS345pe + 1./27.*m13*m66*RSe451p*RS345pe - 1./
         81.*m13*m66*RSe451p*RS645pe*RS345pe + 1./54.*m11*m66*RSe453p*
         RS345pe - 1./162.*m11*m66*RS645pe*RSe453p*RS345pe + 1./54.*m11
         *m66*RSe451p*RS345pe - 1./162.*m11*m66*RSe451p*RS645pe*RS345pe
          );

      M13p6x23RV +=  + Ab(m66,mu2)*BbV(m33,L) * ( 1./108.*m33*m66*pow(
         RS345pe,2) - 1./324.*m33*m66*RS645pe*pow(RS345pe,2) - 1./54.*
         m13*m66*pow(RS345pe,2) + 1./162.*m13*m66*RS645pe*pow(
         RS345pe,2) + 1./108.*m11*m66*pow(RS345pe,2) - 1./324.*m11*m66*
         RS645pe*pow(RS345pe,2) );

      M13p6x23RV +=  + Ab(m66,mu2)*BbV(m36,L) * ( 1./144.*pow(m66,2) - 
         1./432.*pow(m66,2)*RS645pe - 1./36.*m36*m66 + 1./108.*m36*m66*
         RS645pe + 1./36.*pow(m36,2) - 1./108.*pow(m36,2)*RS645pe + 1./
         48.*m33*m66 - 1./144.*m33*m66*RS645pe - 1./24.*m33*m36 + 1./72.
         *m33*m36*RS645pe + 1./72.*pow(m33,2) - 1./216.*pow(m33,2)*
         RS645pe - 1./72.*m13*m66 + 1./216.*m13*m66*RS645pe + 1./36.*
         m13*m36 - 1./108.*m13*m36*RS645pe - 1./72.*m13*m33 + 1./216.*
         m13*m33*RS645pe + 1./144.*m11*m66 - 1./432.*m11*m66*RS645pe - 
         1./72.*m11*m36 + 1./216.*m11*m36*RS645pe + 1./144.*m11*m33 - 1.
         /432.*m11*m33*RS645pe );

      M13p6x23RV +=  + Ab(m66,mu2)*BbV(mp2,L) * (  - 1./54.*m66*mp2*
         pow(RSp453e,2) + 1./162.*m66*mp2*RS645pe*pow(RSp453e,2) + 1./
         27.*m66*mp2*RSp451e*RSp453e - 1./81.*m66*mp2*RSp451e*RS645pe*
         RSp453e - 1./54.*m66*mp2*pow(RSp451e,2) + 1./162.*m66*mp2*pow(
         RSp451e,2)*RS645pe + 1./36.*m33*m66*pow(RSp453e,2) - 1./108.*
         m33*m66*RS645pe*pow(RSp453e,2) + 1./54.*m33*m66*RSp451e*
         RSp453e - 1./162.*m33*m66*RSp451e*RS645pe*RSp453e + 1./108.*
         m33*m66*pow(RSp451e,2) - 1./324.*m33*m66*pow(RSp451e,2)*
         RS645pe - 1./54.*m13*m66*pow(RSp453e,2) + 1./162.*m13*m66*
         RS645pe*pow(RSp453e,2) + 1./27.*m13*m66*RSp451e*RSp453e - 1./
         81.*m13*m66*RSp451e*RS645pe*RSp453e - 1./54.*m13*m66*pow(
         RSp451e,2) + 1./162.*m13*m66*pow(RSp451e,2)*RS645pe + 1./108.*
         m11*m66*pow(RSp453e,2) - 1./324.*m11*m66*RS645pe*pow(
         RSp453e,2) + 1./54.*m11*m66*RSp451e*RSp453e - 1./162.*m11*m66*
         RSp451e*RS645pe*RSp453e + 1./36.*m11*m66*pow(RSp451e,2) - 1./
         108.*m11*m66*pow(RSp451e,2)*RS645pe );

      M13p6x23RV +=  + Ab(m66,mu2)*BbV(me2,mp2,L) * (  - 1./27.*m66*me2
         *RSp453e*RSe453p + 1./81.*m66*me2*RS645pe*RSp453e*RSe453p + 1./
         27.*m66*me2*RSe451p*RSp453e - 1./81.*m66*me2*RSe451p*RS645pe*
         RSp453e + 1./27.*m66*me2*RSp451e*RSe453p - 1./81.*m66*me2*
         RSp451e*RS645pe*RSe453p - 1./27.*m66*me2*RSp451e*RSe451p + 1./
         81.*m66*me2*RSp451e*RSe451p*RS645pe + 1./18.*m33*m66*RSp453e*
         RSe453p - 1./54.*m33*m66*RS645pe*RSp453e*RSe453p + 1./54.*m33*
         m66*RSe451p*RSp453e - 1./162.*m33*m66*RSe451p*RS645pe*RSp453e
          + 1./54.*m33*m66*RSp451e*RSe453p - 1./162.*m33*m66*RSp451e*
         RS645pe*RSe453p + 1./54.*m33*m66*RSp451e*RSe451p - 1./162.*m33
         *m66*RSp451e*RSe451p*RS645pe - 1./27.*m13*m66*RSp453e*RSe453p
          + 1./81.*m13*m66*RS645pe*RSp453e*RSe453p + 1./27.*m13*m66*
         RSe451p*RSp453e - 1./81.*m13*m66*RSe451p*RS645pe*RSp453e + 1./
         27.*m13*m66*RSp451e*RSe453p - 1./81.*m13*m66*RSp451e*RS645pe*
         RSe453p - 1./27.*m13*m66*RSp451e*RSe451p + 1./81.*m13*m66*
         RSp451e*RSe451p*RS645pe );

      M13p6x23RV +=  + Ab(m66,mu2)*BbV(me2,mp2,L) * ( 1./54.*m11*m66*
         RSp453e*RSe453p - 1./162.*m11*m66*RS645pe*RSp453e*RSe453p + 1./
         54.*m11*m66*RSe451p*RSp453e - 1./162.*m11*m66*RSe451p*RS645pe*
         RSp453e + 1./54.*m11*m66*RSp451e*RSe453p - 1./162.*m11*m66*
         RSp451e*RS645pe*RSe453p + 1./18.*m11*m66*RSp451e*RSe451p - 1./
         54.*m11*m66*RSp451e*RSe451p*RS645pe );

      M13p6x23RV +=  + Ab(m66,mu2)*BbV(me2,L) * (  - 1./54.*m66*me2*
         pow(RSe453p,2) + 1./162.*m66*me2*RS645pe*pow(RSe453p,2) + 1./
         27.*m66*me2*RSe451p*RSe453p - 1./81.*m66*me2*RSe451p*RS645pe*
         RSe453p - 1./54.*m66*me2*pow(RSe451p,2) + 1./162.*m66*me2*pow(
         RSe451p,2)*RS645pe + 1./36.*m33*m66*pow(RSe453p,2) - 1./108.*
         m33*m66*RS645pe*pow(RSe453p,2) + 1./54.*m33*m66*RSe451p*
         RSe453p - 1./162.*m33*m66*RSe451p*RS645pe*RSe453p + 1./108.*
         m33*m66*pow(RSe451p,2) - 1./324.*m33*m66*pow(RSe451p,2)*
         RS645pe - 1./54.*m13*m66*pow(RSe453p,2) + 1./162.*m13*m66*
         RS645pe*pow(RSe453p,2) + 1./27.*m13*m66*RSe451p*RSe453p - 1./
         81.*m13*m66*RSe451p*RS645pe*RSe453p - 1./54.*m13*m66*pow(
         RSe451p,2) + 1./162.*m13*m66*pow(RSe451p,2)*RS645pe + 1./108.*
         m11*m66*pow(RSe453p,2) - 1./324.*m11*m66*RS645pe*pow(
         RSe453p,2) + 1./54.*m11*m66*RSe451p*RSe453p - 1./162.*m11*m66*
         RSe451p*RS645pe*RSe453p + 1./36.*m11*m66*pow(RSe451p,2) - 1./
         108.*m11*m66*pow(RSe451p,2)*RS645pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m11,L) * (  - 1./270.*RD1456ep*
         RSp45613e + 1./540.*RD1456ep*RSp45633e + 1./540.*RSp45611e*
         RD1456ep + 1./135.*mp2*RSp45613e*RS14563pe - 1./270.*mp2*
         RSp45633e*RS14563pe - 1./270.*mp2*RC1456pe*RSp45613e + 1./540.
         *mp2*RC1456pe*RSp45633e - 1./270.*mp2*RSp45611e*RS14563pe + 1./
         540.*mp2*RSp45611e*RC1456pe + 1./162.*m66*RSp456e*pow(
         RS145pe,2) + 1./162.*m55*RSp465e*pow(RS146pe,2) + 1./162.*m44*
         RSp564e*pow(RS156pe,2) + 2./45.*m33*RSp45613e*RS14563pe - 1./
         90.*m33*RSp45633e*RS14563pe - 1./405.*m33*RC1456pe*RSp45613e
          - 11./810.*m33*RC1456pe*RSp45633e + 1./270.*m33*RSp45611e*
         RS14563pe + 13./810.*m33*RSp45611e*RC1456pe - 7./405.*m13*
         RSp45613e*RS14563pe + 7./810.*m13*RSp45633e*RS14563pe + 7./810.
         *m13*RC1456pe*RSp45613e - 7./1620.*m13*RC1456pe*RSp45633e + 7./
         810.*m13*RSp45611e*RS14563pe - 7./1620.*m13*RSp45611e*RC1456pe
          + 16./405.*m11*RSp45613e*RS14563pe + 1./162.*m11*RSp45633e*
         RS14563pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m11,L) * ( 13./810.*m11*RC1456pe
         *RSp45613e - 19./1620.*m11*RC1456pe*RSp45633e + 23./810.*m11*
         RSp45611e*RS14563pe - 37./1620.*m11*RSp45611e*RC1456pe - 1./54.
         *m11*m33*RN1p*RC1456pe*RSp45613e - 1./54.*m11*m33*RN1p*
         RSp45611e*RC1456pe - 1./27.*m11*m13*RN1p*RC1456pe*RSp45613e + 
         1./27.*m11*m13*RN1p*RSp45611e*RC1456pe - 1./18.*pow(m11,2)*
         RN1p*RC1456pe*RSp45613e - 1./54.*pow(m11,2)*RN1p*RSp45611e*
         RC1456pe + 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33*
         RSp45613e - 1./162.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33*
         RSp45633e - 1./162.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33*
         RSp45611e + 2./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*
         RSp45613e - 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*
         RSp45633e - 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*
         RSp45611e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m13,L) * (  - 1./162.*RD3456ep*
         RSp45613e + 1./324.*RSp45633e*RD3456ep - 1./162.*RD1456ep*
         RSp45613e + 1./324.*RD1456ep*RSp45633e + 1./324.*RSp45611e*
         RD3456ep + 1./324.*RSp45611e*RD1456ep + 1./81.*me2*RSp45613e*
         RSe45613p - 1./162.*me2*RSe45633p*RSp45613e - 1./162.*me2*
         RSp45633e*RSe45613p + 1./324.*me2*RSp45633e*RSe45633p - 1./162.
         *me2*RSe45611p*RSp45613e + 1./324.*me2*RSe45611p*RSp45633e - 1.
         /162.*me2*RSp45611e*RSe45613p + 1./324.*me2*RSp45611e*
         RSe45633p + 1./324.*me2*RSp45611e*RSe45611p + 1./27.*mp2*
         RSp45613e*RS34561pe + 1./27.*mp2*RSp45613e*RS14563pe + 1./27.*
         mp2*RSp45613e*RSe45613p + 4./81.*mp2*pow(RSp45613e,2) - 1./54.
         *mp2*RC3456pe*RSp45613e - 1./54.*mp2*RSe45633p*RSp45613e - 1./
         54.*mp2*RSp45633e*RS34561pe - 1./54.*mp2*RSp45633e*RS14563pe
          - 1./54.*mp2*RSp45633e*RSe45613p - 4./81.*mp2*RSp45633e*
         RSp45613e + 1./108.*mp2*RSp45633e*RC3456pe + 1./108.*mp2*
         RSp45633e*RSe45633p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m13,L) * ( 1./81.*mp2*pow(
         RSp45633e,2) - 1./54.*mp2*RC1456pe*RSp45613e + 1./108.*mp2*
         RC1456pe*RSp45633e - 1./54.*mp2*RSe45611p*RSp45613e + 1./108.*
         mp2*RSe45611p*RSp45633e - 1./54.*mp2*RSp45611e*RS34561pe - 1./
         54.*mp2*RSp45611e*RS14563pe - 1./54.*mp2*RSp45611e*RSe45613p
          - 4./81.*mp2*RSp45611e*RSp45613e + 1./108.*mp2*RSp45611e*
         RC3456pe + 1./108.*mp2*RSp45611e*RSe45633p + 2./81.*mp2*
         RSp45611e*RSp45633e + 1./108.*mp2*RSp45611e*RC1456pe + 1./108.
         *mp2*RSp45611e*RSe45611p + 1./81.*mp2*pow(RSp45611e,2) - 1./81.
         *m33*RSp45613e*RS14563pe - 1./81.*m33*RSp45613e*RSe45613p - 1./
         81.*m33*pow(RSp45613e,2) + 1./162.*m33*RC3456pe*RSp45613e + 1./
         81.*m33*RSe45633p*RSp45613e + 1./162.*m33*RSp45633e*RS34561pe
          + 1./81.*m33*RSp45633e*RS14563pe + 1./81.*m33*RSp45633e*
         RSe45613p + 2./81.*m33*RSp45633e*RSp45613e - 5./324.*m33*
         RSp45633e*RC3456pe - 1./54.*m33*RSp45633e*RSe45633p - 1./54.*
         m33*pow(RSp45633e,2) );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m13,L) * ( 1./162.*m33*RC1456pe*
         RSp45633e + 1./162.*m33*RSe45611p*RSp45633e - 1./162.*m33*
         RSp45611e*RS34561pe + 1./108.*m33*RSp45611e*RC3456pe + 1./162.
         *m33*RSp45611e*RSe45633p + 1./81.*m33*RSp45611e*RSp45633e - 1./
         162.*m33*RSp45611e*RC1456pe - 1./162.*m33*RSp45611e*RSe45611p
          - 1./162.*m33*pow(RSp45611e,2) - 2./27.*m13*RSp45613e*
         RS34561pe - 2./27.*m13*RSp45613e*RS14563pe - 2./27.*m13*
         RSp45613e*RSe45613p - 2./27.*m13*pow(RSp45613e,2) + 1./27.*m13
         *RC3456pe*RSp45613e + 1./27.*m13*RSe45633p*RSp45613e + 1./27.*
         m13*RSp45633e*RS34561pe + 1./27.*m13*RSp45633e*RS14563pe + 1./
         27.*m13*RSp45633e*RSe45613p + 2./27.*m13*RSp45633e*RSp45613e
          - 1./54.*m13*RSp45633e*RC3456pe - 1./54.*m13*RSp45633e*
         RSe45633p - 1./54.*m13*pow(RSp45633e,2) + 1./27.*m13*RC1456pe*
         RSp45613e - 1./54.*m13*RC1456pe*RSp45633e + 1./27.*m13*
         RSe45611p*RSp45613e - 1./54.*m13*RSe45611p*RSp45633e + 1./27.*
         m13*RSp45611e*RS34561pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m13,L) * ( 1./27.*m13*RSp45611e*
         RS14563pe + 1./27.*m13*RSp45611e*RSe45613p + 2./27.*m13*
         RSp45611e*RSp45613e - 1./54.*m13*RSp45611e*RC3456pe - 1./54.*
         m13*RSp45611e*RSe45633p - 1./27.*m13*RSp45611e*RSp45633e - 1./
         54.*m13*RSp45611e*RC1456pe - 1./54.*m13*RSp45611e*RSe45611p - 
         1./54.*m13*pow(RSp45611e,2) - 1./81.*m11*RSp45613e*RS34561pe
          - 1./81.*m11*RSp45613e*RSe45613p - 1./81.*m11*pow(
         RSp45613e,2) - 1./162.*m11*RSp45633e*RS14563pe - 1./162.*m11*
         RSp45633e*RC3456pe - 1./162.*m11*RSp45633e*RSe45633p - 1./162.
         *m11*pow(RSp45633e,2) + 1./162.*m11*RC1456pe*RSp45613e + 1./
         108.*m11*RC1456pe*RSp45633e + 1./81.*m11*RSe45611p*RSp45613e
          + 1./162.*m11*RSe45611p*RSp45633e + 1./81.*m11*RSp45611e*
         RS34561pe + 1./162.*m11*RSp45611e*RS14563pe + 1./81.*m11*
         RSp45611e*RSe45613p + 2./81.*m11*RSp45611e*RSp45613e + 1./162.
         *m11*RSp45611e*RC3456pe + 1./162.*m11*RSp45611e*RSe45633p + 1./
         81.*m11*RSp45611e*RSp45633e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m13,L) * (  - 5./324.*m11*
         RSp45611e*RC1456pe - 1./54.*m11*RSp45611e*RSe45611p - 1./54.*
         m11*pow(RSp45611e,2) );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m14,L) * ( 11./135.*mp2*
         RSp45613e - 1./180.*mp2*RSp45633e - 19./270.*mp2*RSp563e - 29./
         540.*mp2*RSp45611e + 7./270.*mp2*RSp561e + 1./45.*mp2*RSp564e
          - 7./540.*m44*RSp45613e + 1./60.*m44*RSp45633e + 1./180.*m44*
         RSp563e + 7./540.*m44*RSp45611e - 1./60.*m44*RSp561e - 1./180.
         *m44*RSp564e + 1./27.*m34*RSp45613e - 1./27.*m34*RSp45633e + 1.
         /27.*m34*RSp563e - 1./108.*m34*RSp45611e - 1./54.*m34*RSp561e
          - 1./108.*m34*RSp564e + 4./135.*m33*RSp45613e + 11./270.*m33*
         RSp45633e - 1./45.*m33*RSp563e + 1./120.*m33*RSp45611e - 1./20.
         *m33*RSp561e - 7./1080.*m33*RSp564e + 1./108.*m33*m44*RN1p*
         RSp45613e + 1./108.*m33*m44*RN1p*RSp563e + 1./108.*m33*m44*
         RN1p*RSp45611e + 1./108.*m33*m44*RN1p*RSp561e - 1./90.*m14*
         RSp45613e + 1./270.*m14*RSp45633e + 1./270.*m14*RSp563e - 1./
         60.*m14*RSp45611e + 2./45.*m14*RSp561e - 13./540.*m14*RSp564e
          - 1./54.*m14*m33*RN1p*RSp45613e + 1./54.*m14*m33*RN1p*RSp563e
          - 1./54.*m14*m33*RN1p*RSp45611e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m14,L) * ( 1./54.*m14*m33*RN1p*
         RSp561e - 11./540.*m13*RSp45613e + 1./20.*m13*RSp45633e - 43./
         540.*m13*RSp563e - 1./135.*m13*RSp45611e + 19./540.*m13*
         RSp561e + 1./45.*m13*RSp564e + 1./54.*m13*m44*RN1p*RSp45613e
          + 1./54.*m13*m44*RN1p*RSp563e - 1./54.*m13*m44*RN1p*RSp45611e
          - 1./54.*m13*m44*RN1p*RSp561e - 1./27.*m13*m14*RN1p*RSp45613e
          + 1./27.*m13*m14*RN1p*RSp563e + 1./27.*m13*m14*RN1p*RSp45611e
          - 1./27.*m13*m14*RN1p*RSp561e - 1./540.*m11*RSp45613e + 2./
         135.*m11*RSp45633e - 17./540.*m11*RSp563e + 37./360.*m11*
         RSp45611e - 23./270.*m11*RSp561e + 1./1080.*m11*RSp564e - 1./
         27.*m11*mp2*RN1p*RSp45613e + 1./27.*m11*mp2*RN1p*RSp563e + 1./
         27.*m11*mp2*RN1p*RSp45611e - 1./27.*m11*mp2*RN1p*RSp561e + 1./
         36.*m11*m44*RN1p*RSp45613e + 1./36.*m11*m44*RN1p*RSp563e + 1./
         108.*m11*m44*RN1p*RSp45611e + 1./108.*m11*m44*RN1p*RSp561e + 1.
         /108.*m11*m33*RN1p*RSp45613e + 1./36.*m11*m33*RN1p*RSp563e + 1.
         /108.*m11*m33*RN1p*RSp45611e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m14,L) * ( 1./36.*m11*m33*RN1p*
         RSp561e - 1./18.*m11*m14*RN1p*RSp45613e + 1./18.*m11*m14*RN1p*
         RSp563e - 1./54.*m11*m14*RN1p*RSp45611e + 1./54.*m11*m14*RN1p*
         RSp561e + 1./54.*m11*m13*RN1p*RSp45613e + 1./18.*m11*m13*RN1p*
         RSp563e - 1./54.*m11*m13*RN1p*RSp45611e - 1./18.*m11*m13*RN1p*
         RSp561e + 7./108.*pow(m11,2)*RN1p*RSp45613e + 5./108.*pow(
         m11,2)*RN1p*RSp563e - 1./36.*pow(m11,2)*RN1p*RSp45611e + 7./
         108.*pow(m11,2)*RN1p*RSp561e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m15,L) * ( 11./135.*mp2*
         RSp45613e - 1./180.*mp2*RSp45633e - 19./270.*mp2*RSp463e - 29./
         540.*mp2*RSp45611e + 7./270.*mp2*RSp461e + 1./45.*mp2*RSp465e
          - 7./540.*m55*RSp45613e + 1./60.*m55*RSp45633e + 1./180.*m55*
         RSp463e + 7./540.*m55*RSp45611e - 1./60.*m55*RSp461e - 1./180.
         *m55*RSp465e + 1./27.*m35*RSp45613e - 1./27.*m35*RSp45633e + 1.
         /27.*m35*RSp463e - 1./108.*m35*RSp45611e - 1./54.*m35*RSp461e
          - 1./108.*m35*RSp465e + 4./135.*m33*RSp45613e + 11./270.*m33*
         RSp45633e - 1./45.*m33*RSp463e + 1./120.*m33*RSp45611e - 1./20.
         *m33*RSp461e - 7./1080.*m33*RSp465e + 1./108.*m33*m55*RN1p*
         RSp45613e + 1./108.*m33*m55*RN1p*RSp463e + 1./108.*m33*m55*
         RN1p*RSp45611e + 1./108.*m33*m55*RN1p*RSp461e - 1./90.*m15*
         RSp45613e + 1./270.*m15*RSp45633e + 1./270.*m15*RSp463e - 1./
         60.*m15*RSp45611e + 2./45.*m15*RSp461e - 13./540.*m15*RSp465e
          - 1./54.*m15*m33*RN1p*RSp45613e + 1./54.*m15*m33*RN1p*RSp463e
          - 1./54.*m15*m33*RN1p*RSp45611e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m15,L) * ( 1./54.*m15*m33*RN1p*
         RSp461e - 11./540.*m13*RSp45613e + 1./20.*m13*RSp45633e - 43./
         540.*m13*RSp463e - 1./135.*m13*RSp45611e + 19./540.*m13*
         RSp461e + 1./45.*m13*RSp465e + 1./54.*m13*m55*RN1p*RSp45613e
          + 1./54.*m13*m55*RN1p*RSp463e - 1./54.*m13*m55*RN1p*RSp45611e
          - 1./54.*m13*m55*RN1p*RSp461e - 1./27.*m13*m15*RN1p*RSp45613e
          + 1./27.*m13*m15*RN1p*RSp463e + 1./27.*m13*m15*RN1p*RSp45611e
          - 1./27.*m13*m15*RN1p*RSp461e - 1./540.*m11*RSp45613e + 2./
         135.*m11*RSp45633e - 17./540.*m11*RSp463e + 37./360.*m11*
         RSp45611e - 23./270.*m11*RSp461e + 1./1080.*m11*RSp465e - 1./
         27.*m11*mp2*RN1p*RSp45613e + 1./27.*m11*mp2*RN1p*RSp463e + 1./
         27.*m11*mp2*RN1p*RSp45611e - 1./27.*m11*mp2*RN1p*RSp461e + 1./
         36.*m11*m55*RN1p*RSp45613e + 1./36.*m11*m55*RN1p*RSp463e + 1./
         108.*m11*m55*RN1p*RSp45611e + 1./108.*m11*m55*RN1p*RSp461e + 1.
         /108.*m11*m33*RN1p*RSp45613e + 1./36.*m11*m33*RN1p*RSp463e + 1.
         /108.*m11*m33*RN1p*RSp45611e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m15,L) * ( 1./36.*m11*m33*RN1p*
         RSp461e - 1./18.*m11*m15*RN1p*RSp45613e + 1./18.*m11*m15*RN1p*
         RSp463e - 1./54.*m11*m15*RN1p*RSp45611e + 1./54.*m11*m15*RN1p*
         RSp461e + 1./54.*m11*m13*RN1p*RSp45613e + 1./18.*m11*m13*RN1p*
         RSp463e - 1./54.*m11*m13*RN1p*RSp45611e - 1./18.*m11*m13*RN1p*
         RSp461e + 7./108.*pow(m11,2)*RN1p*RSp45613e + 5./108.*pow(
         m11,2)*RN1p*RSp463e - 1./36.*pow(m11,2)*RN1p*RSp45611e + 7./
         108.*pow(m11,2)*RN1p*RSp461e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m16,L) * ( 11./135.*mp2*
         RSp45613e - 1./180.*mp2*RSp45633e - 19./270.*mp2*RSp453e - 29./
         540.*mp2*RSp45611e + 7./270.*mp2*RSp451e + 1./45.*mp2*RSp456e
          - 7./540.*m66*RSp45613e + 1./60.*m66*RSp45633e + 1./180.*m66*
         RSp453e + 7./540.*m66*RSp45611e - 1./60.*m66*RSp451e - 1./180.
         *m66*RSp456e + 1./27.*m36*RSp45613e - 1./27.*m36*RSp45633e + 1.
         /27.*m36*RSp453e - 1./108.*m36*RSp45611e - 1./54.*m36*RSp451e
          - 1./108.*m36*RSp456e + 4./135.*m33*RSp45613e + 11./270.*m33*
         RSp45633e - 1./45.*m33*RSp453e + 1./120.*m33*RSp45611e - 1./20.
         *m33*RSp451e - 7./1080.*m33*RSp456e + 1./108.*m33*m66*RN1p*
         RSp45613e + 1./108.*m33*m66*RN1p*RSp453e + 1./108.*m33*m66*
         RN1p*RSp45611e + 1./108.*m33*m66*RN1p*RSp451e - 1./90.*m16*
         RSp45613e + 1./270.*m16*RSp45633e + 1./270.*m16*RSp453e - 1./
         60.*m16*RSp45611e + 2./45.*m16*RSp451e - 13./540.*m16*RSp456e
          - 1./54.*m16*m33*RN1p*RSp45613e + 1./54.*m16*m33*RN1p*RSp453e
          - 1./54.*m16*m33*RN1p*RSp45611e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m16,L) * ( 1./54.*m16*m33*RN1p*
         RSp451e - 11./540.*m13*RSp45613e + 1./20.*m13*RSp45633e - 43./
         540.*m13*RSp453e - 1./135.*m13*RSp45611e + 19./540.*m13*
         RSp451e + 1./45.*m13*RSp456e + 1./54.*m13*m66*RN1p*RSp45613e
          + 1./54.*m13*m66*RN1p*RSp453e - 1./54.*m13*m66*RN1p*RSp45611e
          - 1./54.*m13*m66*RN1p*RSp451e - 1./27.*m13*m16*RN1p*RSp45613e
          + 1./27.*m13*m16*RN1p*RSp453e + 1./27.*m13*m16*RN1p*RSp45611e
          - 1./27.*m13*m16*RN1p*RSp451e - 1./540.*m11*RSp45613e + 2./
         135.*m11*RSp45633e - 17./540.*m11*RSp453e + 37./360.*m11*
         RSp45611e - 23./270.*m11*RSp451e + 1./1080.*m11*RSp456e - 1./
         27.*m11*mp2*RN1p*RSp45613e + 1./27.*m11*mp2*RN1p*RSp453e + 1./
         27.*m11*mp2*RN1p*RSp45611e - 1./27.*m11*mp2*RN1p*RSp451e + 1./
         36.*m11*m66*RN1p*RSp45613e + 1./36.*m11*m66*RN1p*RSp453e + 1./
         108.*m11*m66*RN1p*RSp45611e + 1./108.*m11*m66*RN1p*RSp451e + 1.
         /108.*m11*m33*RN1p*RSp45613e + 1./36.*m11*m33*RN1p*RSp453e + 1.
         /108.*m11*m33*RN1p*RSp45611e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m16,L) * ( 1./36.*m11*m33*RN1p*
         RSp451e - 1./18.*m11*m16*RN1p*RSp45613e + 1./18.*m11*m16*RN1p*
         RSp453e - 1./54.*m11*m16*RN1p*RSp45611e + 1./54.*m11*m16*RN1p*
         RSp451e + 1./54.*m11*m13*RN1p*RSp45613e + 1./18.*m11*m13*RN1p*
         RSp453e - 1./54.*m11*m13*RN1p*RSp45611e - 1./18.*m11*m13*RN1p*
         RSp451e + 7./108.*pow(m11,2)*RN1p*RSp45613e + 5./108.*pow(
         m11,2)*RN1p*RSp453e - 1./36.*pow(m11,2)*RN1p*RSp45611e + 7./
         108.*pow(m11,2)*RN1p*RSp451e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m33,L) * (  - 1./270.*RD3456ep*
         RSp45613e + 1./540.*RSp45633e*RD3456ep + 1./540.*RSp45611e*
         RD3456ep + 1./135.*mp2*RSp45613e*RS34561pe - 1./270.*mp2*
         RC3456pe*RSp45613e - 1./270.*mp2*RSp45633e*RS34561pe + 1./540.
         *mp2*RSp45633e*RC3456pe - 1./270.*mp2*RSp45611e*RS34561pe + 1./
         540.*mp2*RSp45611e*RC3456pe + 1./162.*m66*RSp456e*pow(
         RS345pe,2) - 1./81.*m66*RSp456e*RS145pe*RS345pe + 1./162.*m55*
         RSp465e*pow(RS346pe,2) - 1./81.*m55*RSp465e*RS146pe*RS346pe + 
         1./162.*m44*RSp564e*pow(RS356pe,2) - 1./81.*m44*RSp564e*
         RS156pe*RS356pe + 16./405.*m33*RSp45613e*RS34561pe + 13./810.*
         m33*RC3456pe*RSp45613e + 23./810.*m33*RSp45633e*RS34561pe + 1./
         27.*m33*RSp45633e*RS14563pe - 37./1620.*m33*RSp45633e*RC3456pe
          + 1./162.*m33*RSp45611e*RS34561pe - 19./1620.*m33*RSp45611e*
         RC3456pe - 1./18.*pow(m33,2)*RN3p*RC3456pe*RSp45613e - 1./54.*
         pow(m33,2)*RN3p*RSp45633e*RC3456pe - 7./405.*m13*RSp45613e*
         RS34561pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m33,L) * ( 7./810.*m13*RC3456pe*
         RSp45613e + 7./810.*m13*RSp45633e*RS34561pe - 7./1620.*m13*
         RSp45633e*RC3456pe + 7./810.*m13*RSp45611e*RS34561pe - 7./1620.
         *m13*RSp45611e*RC3456pe - 1./27.*m13*m33*RN3p*RC3456pe*
         RSp45613e + 1./27.*m13*m33*RN3p*RSp45633e*RC3456pe + 2./45.*
         m11*RSp45613e*RS34561pe - 1./405.*m11*RC3456pe*RSp45613e + 1./
         270.*m11*RSp45633e*RS34561pe + 13./810.*m11*RSp45633e*RC3456pe
          + 7./270.*m11*RSp45611e*RS34561pe - 11./810.*m11*RSp45611e*
         RC3456pe - 1./54.*m11*m33*RN3p*RC3456pe*RSp45613e - 1./54.*m11
         *m33*RN3p*RSp45633e*RC3456pe + 2./81.*RS(m33,m44,m55,m66,m11,
         me2,mp2)*m33*RSp45613e - 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2
         )*m33*RSp45633e - 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33*
         RSp45611e + 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RSp45613e - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RSp45633e - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RSp45611e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m34,L) * ( 11./135.*mp2*
         RSp45613e - 29./540.*mp2*RSp45633e + 7./270.*mp2*RSp563e - 1./
         180.*mp2*RSp45611e - 19./270.*mp2*RSp561e + 1./45.*mp2*RSp564e
          - 7./540.*m44*RSp45613e + 7./540.*m44*RSp45633e - 1./60.*m44*
         RSp563e + 1./60.*m44*RSp45611e + 1./180.*m44*RSp561e - 1./180.
         *m44*RSp564e - 1./90.*m34*RSp45613e - 1./60.*m34*RSp45633e + 2.
         /45.*m34*RSp563e + 1./270.*m34*RSp45611e + 1./270.*m34*RSp561e
          - 13./540.*m34*RSp564e - 1./540.*m33*RSp45613e + 37./360.*m33
         *RSp45633e - 23./270.*m33*RSp563e + 2./135.*m33*RSp45611e - 17.
         /540.*m33*RSp561e + 1./1080.*m33*RSp564e - 1./27.*m33*mp2*RN3p
         *RSp45613e + 1./27.*m33*mp2*RN3p*RSp45633e - 1./27.*m33*mp2*
         RN3p*RSp563e + 1./27.*m33*mp2*RN3p*RSp561e + 1./36.*m33*m44*
         RN3p*RSp45613e + 1./108.*m33*m44*RN3p*RSp45633e + 1./108.*m33*
         m44*RN3p*RSp563e + 1./36.*m33*m44*RN3p*RSp561e - 1./18.*m33*
         m34*RN3p*RSp45613e - 1./54.*m33*m34*RN3p*RSp45633e + 1./54.*
         m33*m34*RN3p*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m34,L) * ( 1./18.*m33*m34*RN3p*
         RSp561e + 7./108.*pow(m33,2)*RN3p*RSp45613e - 1./36.*pow(
         m33,2)*RN3p*RSp45633e + 7./108.*pow(m33,2)*RN3p*RSp563e + 5./
         108.*pow(m33,2)*RN3p*RSp561e + 1./27.*m14*RSp45613e - 1./108.*
         m14*RSp45633e - 1./54.*m14*RSp563e - 1./27.*m14*RSp45611e + 1./
         27.*m14*RSp561e - 1./108.*m14*RSp564e - 11./540.*m13*RSp45613e
          - 1./135.*m13*RSp45633e + 19./540.*m13*RSp563e + 1./20.*m13*
         RSp45611e - 43./540.*m13*RSp561e + 1./45.*m13*RSp564e + 1./54.
         *m13*m44*RN3p*RSp45613e - 1./54.*m13*m44*RN3p*RSp45633e - 1./
         54.*m13*m44*RN3p*RSp563e + 1./54.*m13*m44*RN3p*RSp561e - 1./27.
         *m13*m34*RN3p*RSp45613e + 1./27.*m13*m34*RN3p*RSp45633e - 1./
         27.*m13*m34*RN3p*RSp563e + 1./27.*m13*m34*RN3p*RSp561e + 1./54.
         *m13*m33*RN3p*RSp45613e - 1./54.*m13*m33*RN3p*RSp45633e - 1./
         18.*m13*m33*RN3p*RSp563e + 1./18.*m13*m33*RN3p*RSp561e + 4./
         135.*m11*RSp45613e + 1./120.*m11*RSp45633e - 1./20.*m11*
         RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m34,L) * ( 11./270.*m11*
         RSp45611e - 1./45.*m11*RSp561e - 7./1080.*m11*RSp564e + 1./108.
         *m11*m44*RN3p*RSp45613e + 1./108.*m11*m44*RN3p*RSp45633e + 1./
         108.*m11*m44*RN3p*RSp563e + 1./108.*m11*m44*RN3p*RSp561e - 1./
         54.*m11*m34*RN3p*RSp45613e - 1./54.*m11*m34*RN3p*RSp45633e + 1.
         /54.*m11*m34*RN3p*RSp563e + 1./54.*m11*m34*RN3p*RSp561e + 1./
         108.*m11*m33*RN3p*RSp45613e + 1./108.*m11*m33*RN3p*RSp45633e
          + 1./36.*m11*m33*RN3p*RSp563e + 1./36.*m11*m33*RN3p*RSp561e )
         ;

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m35,L) * ( 11./135.*mp2*
         RSp45613e - 29./540.*mp2*RSp45633e + 7./270.*mp2*RSp463e - 1./
         180.*mp2*RSp45611e - 19./270.*mp2*RSp461e + 1./45.*mp2*RSp465e
          - 7./540.*m55*RSp45613e + 7./540.*m55*RSp45633e - 1./60.*m55*
         RSp463e + 1./60.*m55*RSp45611e + 1./180.*m55*RSp461e - 1./180.
         *m55*RSp465e - 1./90.*m35*RSp45613e - 1./60.*m35*RSp45633e + 2.
         /45.*m35*RSp463e + 1./270.*m35*RSp45611e + 1./270.*m35*RSp461e
          - 13./540.*m35*RSp465e - 1./540.*m33*RSp45613e + 37./360.*m33
         *RSp45633e - 23./270.*m33*RSp463e + 2./135.*m33*RSp45611e - 17.
         /540.*m33*RSp461e + 1./1080.*m33*RSp465e - 1./27.*m33*mp2*RN3p
         *RSp45613e + 1./27.*m33*mp2*RN3p*RSp45633e - 1./27.*m33*mp2*
         RN3p*RSp463e + 1./27.*m33*mp2*RN3p*RSp461e + 1./36.*m33*m55*
         RN3p*RSp45613e + 1./108.*m33*m55*RN3p*RSp45633e + 1./108.*m33*
         m55*RN3p*RSp463e + 1./36.*m33*m55*RN3p*RSp461e - 1./18.*m33*
         m35*RN3p*RSp45613e - 1./54.*m33*m35*RN3p*RSp45633e + 1./54.*
         m33*m35*RN3p*RSp463e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m35,L) * ( 1./18.*m33*m35*RN3p*
         RSp461e + 7./108.*pow(m33,2)*RN3p*RSp45613e - 1./36.*pow(
         m33,2)*RN3p*RSp45633e + 7./108.*pow(m33,2)*RN3p*RSp463e + 5./
         108.*pow(m33,2)*RN3p*RSp461e + 1./27.*m15*RSp45613e - 1./108.*
         m15*RSp45633e - 1./54.*m15*RSp463e - 1./27.*m15*RSp45611e + 1./
         27.*m15*RSp461e - 1./108.*m15*RSp465e - 11./540.*m13*RSp45613e
          - 1./135.*m13*RSp45633e + 19./540.*m13*RSp463e + 1./20.*m13*
         RSp45611e - 43./540.*m13*RSp461e + 1./45.*m13*RSp465e + 1./54.
         *m13*m55*RN3p*RSp45613e - 1./54.*m13*m55*RN3p*RSp45633e - 1./
         54.*m13*m55*RN3p*RSp463e + 1./54.*m13*m55*RN3p*RSp461e - 1./27.
         *m13*m35*RN3p*RSp45613e + 1./27.*m13*m35*RN3p*RSp45633e - 1./
         27.*m13*m35*RN3p*RSp463e + 1./27.*m13*m35*RN3p*RSp461e + 1./54.
         *m13*m33*RN3p*RSp45613e - 1./54.*m13*m33*RN3p*RSp45633e - 1./
         18.*m13*m33*RN3p*RSp463e + 1./18.*m13*m33*RN3p*RSp461e + 4./
         135.*m11*RSp45613e + 1./120.*m11*RSp45633e - 1./20.*m11*
         RSp463e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m35,L) * ( 11./270.*m11*
         RSp45611e - 1./45.*m11*RSp461e - 7./1080.*m11*RSp465e + 1./108.
         *m11*m55*RN3p*RSp45613e + 1./108.*m11*m55*RN3p*RSp45633e + 1./
         108.*m11*m55*RN3p*RSp463e + 1./108.*m11*m55*RN3p*RSp461e - 1./
         54.*m11*m35*RN3p*RSp45613e - 1./54.*m11*m35*RN3p*RSp45633e + 1.
         /54.*m11*m35*RN3p*RSp463e + 1./54.*m11*m35*RN3p*RSp461e + 1./
         108.*m11*m33*RN3p*RSp45613e + 1./108.*m11*m33*RN3p*RSp45633e
          + 1./36.*m11*m33*RN3p*RSp463e + 1./36.*m11*m33*RN3p*RSp461e )
         ;

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m36,L) * ( 11./135.*mp2*
         RSp45613e - 29./540.*mp2*RSp45633e + 7./270.*mp2*RSp453e - 1./
         180.*mp2*RSp45611e - 19./270.*mp2*RSp451e + 1./45.*mp2*RSp456e
          - 7./540.*m66*RSp45613e + 7./540.*m66*RSp45633e - 1./60.*m66*
         RSp453e + 1./60.*m66*RSp45611e + 1./180.*m66*RSp451e - 1./180.
         *m66*RSp456e - 1./90.*m36*RSp45613e - 1./60.*m36*RSp45633e + 2.
         /45.*m36*RSp453e + 1./270.*m36*RSp45611e + 1./270.*m36*RSp451e
          - 13./540.*m36*RSp456e - 1./540.*m33*RSp45613e + 37./360.*m33
         *RSp45633e - 23./270.*m33*RSp453e + 2./135.*m33*RSp45611e - 17.
         /540.*m33*RSp451e + 1./1080.*m33*RSp456e - 1./27.*m33*mp2*RN3p
         *RSp45613e + 1./27.*m33*mp2*RN3p*RSp45633e - 1./27.*m33*mp2*
         RN3p*RSp453e + 1./27.*m33*mp2*RN3p*RSp451e + 1./36.*m33*m66*
         RN3p*RSp45613e + 1./108.*m33*m66*RN3p*RSp45633e + 1./108.*m33*
         m66*RN3p*RSp453e + 1./36.*m33*m66*RN3p*RSp451e - 1./18.*m33*
         m36*RN3p*RSp45613e - 1./54.*m33*m36*RN3p*RSp45633e + 1./54.*
         m33*m36*RN3p*RSp453e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m36,L) * ( 1./18.*m33*m36*RN3p*
         RSp451e + 7./108.*pow(m33,2)*RN3p*RSp45613e - 1./36.*pow(
         m33,2)*RN3p*RSp45633e + 7./108.*pow(m33,2)*RN3p*RSp453e + 5./
         108.*pow(m33,2)*RN3p*RSp451e + 1./27.*m16*RSp45613e - 1./108.*
         m16*RSp45633e - 1./54.*m16*RSp453e - 1./27.*m16*RSp45611e + 1./
         27.*m16*RSp451e - 1./108.*m16*RSp456e - 11./540.*m13*RSp45613e
          - 1./135.*m13*RSp45633e + 19./540.*m13*RSp453e + 1./20.*m13*
         RSp45611e - 43./540.*m13*RSp451e + 1./45.*m13*RSp456e + 1./54.
         *m13*m66*RN3p*RSp45613e - 1./54.*m13*m66*RN3p*RSp45633e - 1./
         54.*m13*m66*RN3p*RSp453e + 1./54.*m13*m66*RN3p*RSp451e - 1./27.
         *m13*m36*RN3p*RSp45613e + 1./27.*m13*m36*RN3p*RSp45633e - 1./
         27.*m13*m36*RN3p*RSp453e + 1./27.*m13*m36*RN3p*RSp451e + 1./54.
         *m13*m33*RN3p*RSp45613e - 1./54.*m13*m33*RN3p*RSp45633e - 1./
         18.*m13*m33*RN3p*RSp453e + 1./18.*m13*m33*RN3p*RSp451e + 4./
         135.*m11*RSp45613e + 1./120.*m11*RSp45633e - 1./20.*m11*
         RSp453e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m36,L) * ( 11./270.*m11*
         RSp45611e - 1./45.*m11*RSp451e - 7./1080.*m11*RSp456e + 1./108.
         *m11*m66*RN3p*RSp45613e + 1./108.*m11*m66*RN3p*RSp45633e + 1./
         108.*m11*m66*RN3p*RSp453e + 1./108.*m11*m66*RN3p*RSp451e - 1./
         54.*m11*m36*RN3p*RSp45613e - 1./54.*m11*m36*RN3p*RSp45633e + 1.
         /54.*m11*m36*RN3p*RSp453e + 1./54.*m11*m36*RN3p*RSp451e + 1./
         108.*m11*m33*RN3p*RSp45613e + 1./108.*m11*m33*RN3p*RSp45633e
          + 1./36.*m11*m33*RN3p*RSp453e + 1./36.*m11*m33*RN3p*RSp451e )
         ;

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m44,L) * (  - 1./27.*m44*RSp563e
         *RS356pe - 1./27.*m44*RSp563e*RSe563p - 1./54.*m44*pow(
         RSp563e,2) + 1./27.*m44*RS156pe*RSp563e + 1./81.*m44*RS456pe*
         RSp563e*RS356pe + 1./81.*m44*RS456pe*RSp563e*RSe563p + 1./162.
         *m44*RS456pe*pow(RSp563e,2) - 1./81.*m44*RS456pe*RS156pe*
         RSp563e + 1./27.*m44*RSe561p*RSp563e - 1./81.*m44*RSe561p*
         RS456pe*RSp563e + 1./27.*m44*RSp561e*RS356pe + 1./27.*m44*
         RSp561e*RSe563p + 1./27.*m44*RSp561e*RSp563e - 1./27.*m44*
         RSp561e*RS156pe - 1./81.*m44*RSp561e*RS456pe*RS356pe - 1./81.*
         m44*RSp561e*RS456pe*RSe563p - 1./81.*m44*RSp561e*RS456pe*
         RSp563e + 1./81.*m44*RSp561e*RS456pe*RS156pe - 1./27.*m44*
         RSp561e*RSe561p + 1./81.*m44*RSp561e*RSe561p*RS456pe - 1./54.*
         m44*pow(RSp561e,2) + 1./162.*m44*pow(RSp561e,2)*RS456pe - 1./
         54.*m33*m44*RN3p*RSp563e*RS356pe + 1./162.*m33*m44*RN3p*
         RS456pe*RSp563e*RS356pe - 1./18.*m33*m44*RN3p*RSp561e*RS356pe
          + 1./54.*m33*m44*RN3p*RSp561e*RS456pe*RS356pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m44,L) * (  - 1./54.*m33*m44*
         RN1p*RS156pe*RSp563e + 1./162.*m33*m44*RN1p*RS456pe*RS156pe*
         RSp563e - 1./54.*m33*m44*RN1p*RSp561e*RS156pe + 1./162.*m33*
         m44*RN1p*RSp561e*RS456pe*RS156pe + 1./27.*m13*m44*RN3p*RSp563e
         *RS356pe - 1./81.*m13*m44*RN3p*RS456pe*RSp563e*RS356pe - 1./27.
         *m13*m44*RN3p*RSp561e*RS356pe + 1./81.*m13*m44*RN3p*RSp561e*
         RS456pe*RS356pe - 1./27.*m13*m44*RN1p*RS156pe*RSp563e + 1./81.
         *m13*m44*RN1p*RS456pe*RS156pe*RSp563e + 1./27.*m13*m44*RN1p*
         RSp561e*RS156pe - 1./81.*m13*m44*RN1p*RSp561e*RS456pe*RS156pe
          - 1./54.*m11*m44*RN3p*RSp563e*RS356pe + 1./162.*m11*m44*RN3p*
         RS456pe*RSp563e*RS356pe - 1./54.*m11*m44*RN3p*RSp561e*RS356pe
          + 1./162.*m11*m44*RN3p*RSp561e*RS456pe*RS356pe - 1./18.*m11*
         m44*RN1p*RS156pe*RSp563e + 1./54.*m11*m44*RN1p*RS456pe*RS156pe
         *RSp563e - 1./54.*m11*m44*RN1p*RSp561e*RS156pe + 1./162.*m11*
         m44*RN1p*RSp561e*RS456pe*RS156pe - 1./27.*RN(mp2,me2)*m44*me2*
         RSp563e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m44,L) * ( 1./81.*RN(mp2,me2)*
         m44*me2*RS456pe*RSp563e*RSe563p + 1./27.*RN(mp2,me2)*m44*me2*
         RSe561p*RSp563e - 1./81.*RN(mp2,me2)*m44*me2*RSe561p*RS456pe*
         RSp563e + 1./27.*RN(mp2,me2)*m44*me2*RSp561e*RSe563p - 1./81.*
         RN(mp2,me2)*m44*me2*RSp561e*RS456pe*RSe563p - 1./27.*RN(mp2,
         me2)*m44*me2*RSp561e*RSe561p + 1./81.*RN(mp2,me2)*m44*me2*
         RSp561e*RSe561p*RS456pe + 1./18.*RN(mp2,me2)*m33*m44*RSp563e*
         RSe563p - 1./54.*RN(mp2,me2)*m33*m44*RS456pe*RSp563e*RSe563p
          + 1./54.*RN(mp2,me2)*m33*m44*RSe561p*RSp563e - 1./162.*RN(mp2
         ,me2)*m33*m44*RSe561p*RS456pe*RSp563e + 1./54.*RN(mp2,me2)*m33
         *m44*RSp561e*RSe563p - 1./162.*RN(mp2,me2)*m33*m44*RSp561e*
         RS456pe*RSe563p + 1./54.*RN(mp2,me2)*m33*m44*RSp561e*RSe561p
          - 1./162.*RN(mp2,me2)*m33*m44*RSp561e*RSe561p*RS456pe - 1./27.
         *RN(mp2,me2)*m13*m44*RSp563e*RSe563p + 1./81.*RN(mp2,me2)*m13*
         m44*RS456pe*RSp563e*RSe563p + 1./27.*RN(mp2,me2)*m13*m44*
         RSe561p*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m44,L) * (  - 1./81.*RN(mp2,me2)
         *m13*m44*RSe561p*RS456pe*RSp563e + 1./27.*RN(mp2,me2)*m13*m44*
         RSp561e*RSe563p - 1./81.*RN(mp2,me2)*m13*m44*RSp561e*RS456pe*
         RSe563p - 1./27.*RN(mp2,me2)*m13*m44*RSp561e*RSe561p + 1./81.*
         RN(mp2,me2)*m13*m44*RSp561e*RSe561p*RS456pe + 1./54.*RN(mp2,
         me2)*m11*m44*RSp563e*RSe563p - 1./162.*RN(mp2,me2)*m11*m44*
         RS456pe*RSp563e*RSe563p + 1./54.*RN(mp2,me2)*m11*m44*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m11*m44*RSe561p*RS456pe*RSp563e
          + 1./54.*RN(mp2,me2)*m11*m44*RSp561e*RSe563p - 1./162.*RN(mp2
         ,me2)*m11*m44*RSp561e*RS456pe*RSe563p + 1./18.*RN(mp2,me2)*m11
         *m44*RSp561e*RSe561p - 1./54.*RN(mp2,me2)*m11*m44*RSp561e*
         RSe561p*RS456pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./81.*mp2*RSp563e
         *RS346pe + 1./81.*mp2*RSp563e*RS356pe + 1./81.*mp2*RSp563e*
         RSe563p - 1./81.*mp2*RSp563e*RSe463p + 1./81.*mp2*pow(
         RSp563e,2) + 1./81.*mp2*RSp463e*RS346pe - 1./81.*mp2*RSp463e*
         RS356pe - 1./81.*mp2*RSp463e*RSe563p + 1./81.*mp2*RSp463e*
         RSe463p - 2./81.*mp2*RSp463e*RSp563e + 1./81.*mp2*pow(
         RSp463e,2) + 1./81.*mp2*RS146pe*RSp563e - 1./81.*mp2*RS146pe*
         RSp463e - 1./81.*mp2*RS156pe*RSp563e + 1./81.*mp2*RS156pe*
         RSp463e - 1./81.*mp2*RSe561p*RSp563e + 1./81.*mp2*RSe561p*
         RSp463e + 1./81.*mp2*RSe461p*RSp563e - 1./81.*mp2*RSe461p*
         RSp463e + 1./81.*mp2*RSp561e*RS346pe - 1./81.*mp2*RSp561e*
         RS356pe - 1./81.*mp2*RSp561e*RSe563p + 1./81.*mp2*RSp561e*
         RSe463p - 2./81.*mp2*RSp561e*RSp563e + 2./81.*mp2*RSp561e*
         RSp463e - 1./81.*mp2*RSp561e*RS146pe + 1./81.*mp2*RSp561e*
         RS156pe + 1./81.*mp2*RSp561e*RSe561p - 1./81.*mp2*RSp561e*
         RSe461p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./81.*mp2*pow(
         RSp561e,2) - 1./81.*mp2*RSp461e*RS346pe + 1./81.*mp2*RSp461e*
         RS356pe + 1./81.*mp2*RSp461e*RSe563p - 1./81.*mp2*RSp461e*
         RSe463p + 2./81.*mp2*RSp461e*RSp563e - 2./81.*mp2*RSp461e*
         RSp463e + 1./81.*mp2*RSp461e*RS146pe - 1./81.*mp2*RSp461e*
         RS156pe - 1./81.*mp2*RSp461e*RSe561p + 1./81.*mp2*RSp461e*
         RSe461p - 2./81.*mp2*RSp461e*RSp561e + 1./81.*mp2*pow(
         RSp461e,2) - 1./162.*m55*RSp563e*RS346pe - 1./162.*m55*RSp563e
         *RS356pe - 1./162.*m55*RSp563e*RSe563p - 1./162.*m55*RSp563e*
         RSe463p - 1./324.*m55*pow(RSp563e,2) - 1./54.*m55*RSp463e*
         RS346pe - 1./162.*m55*RSp463e*RS356pe - 1./162.*m55*RSp463e*
         RSe563p - 1./54.*m55*RSp463e*RSe463p - 1./162.*m55*RSp463e*
         RSp563e - 1./108.*m55*pow(RSp463e,2) + 1./162.*m55*RS146pe*
         RSp563e + 1./54.*m55*RS146pe*RSp463e + 1./162.*m55*RS156pe*
         RSp563e + 1./162.*m55*RS156pe*RSp463e + 1./162.*m55*RSe561p*
         RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./162.*m55*RSe561p*
         RSp463e + 1./162.*m55*RSe461p*RSp563e + 1./54.*m55*RSe461p*
         RSp463e + 1./162.*m55*RSp561e*RS346pe + 1./162.*m55*RSp561e*
         RS356pe + 1./162.*m55*RSp561e*RSe563p + 1./162.*m55*RSp561e*
         RSe463p + 1./162.*m55*RSp561e*RSp563e + 1./162.*m55*RSp561e*
         RSp463e - 1./162.*m55*RSp561e*RS146pe - 1./162.*m55*RSp561e*
         RS156pe - 1./162.*m55*RSp561e*RSe561p - 1./162.*m55*RSp561e*
         RSe461p - 1./324.*m55*pow(RSp561e,2) + 1./54.*m55*RSp461e*
         RS346pe + 1./162.*m55*RSp461e*RS356pe + 1./162.*m55*RSp461e*
         RSe563p + 1./54.*m55*RSp461e*RSe463p + 1./162.*m55*RSp461e*
         RSp563e + 1./54.*m55*RSp461e*RSp463e - 1./54.*m55*RSp461e*
         RS146pe - 1./162.*m55*RSp461e*RS156pe - 1./162.*m55*RSp461e*
         RSe561p - 1./54.*m55*RSp461e*RSe461p - 1./162.*m55*RSp461e*
         RSp561e - 1./108.*m55*pow(RSp461e,2) - 1./81.*m45*RSp563e*
         RS346pe + 1./81.*m45*RSp563e*RS356pe + 1./81.*m45*RSp563e*
         RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./81.*m45*RSp563e
         *RSe463p + 1./162.*m45*pow(RSp563e,2) + 1./81.*m45*RSp463e*
         RS346pe - 1./81.*m45*RSp463e*RS356pe - 1./81.*m45*RSp463e*
         RSe563p + 1./81.*m45*RSp463e*RSe463p - 1./81.*m45*RSp463e*
         RSp563e + 1./162.*m45*pow(RSp463e,2) + 1./81.*m45*RS146pe*
         RSp563e - 1./81.*m45*RS146pe*RSp463e - 1./81.*m45*RS156pe*
         RSp563e + 1./81.*m45*RS156pe*RSp463e - 1./81.*m45*RSe561p*
         RSp563e + 1./81.*m45*RSe561p*RSp463e + 1./81.*m45*RSe461p*
         RSp563e - 1./81.*m45*RSe461p*RSp463e + 1./81.*m45*RSp561e*
         RS346pe - 1./81.*m45*RSp561e*RS356pe - 1./81.*m45*RSp561e*
         RSe563p + 1./81.*m45*RSp561e*RSe463p - 1./81.*m45*RSp561e*
         RSp563e + 1./81.*m45*RSp561e*RSp463e - 1./81.*m45*RSp561e*
         RS146pe + 1./81.*m45*RSp561e*RS156pe + 1./81.*m45*RSp561e*
         RSe561p - 1./81.*m45*RSp561e*RSe461p + 1./162.*m45*pow(
         RSp561e,2) - 1./81.*m45*RSp461e*RS346pe + 1./81.*m45*RSp461e*
         RS356pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./81.*m45*RSp461e*
         RSe563p - 1./81.*m45*RSp461e*RSe463p + 1./81.*m45*RSp461e*
         RSp563e - 1./81.*m45*RSp461e*RSp463e + 1./81.*m45*RSp461e*
         RS146pe - 1./81.*m45*RSp461e*RS156pe - 1./81.*m45*RSp461e*
         RSe561p + 1./81.*m45*RSp461e*RSe461p - 1./81.*m45*RSp461e*
         RSp561e + 1./162.*m45*pow(RSp461e,2) - 1./162.*m44*RSp563e*
         RS346pe - 1./54.*m44*RSp563e*RS356pe - 1./54.*m44*RSp563e*
         RSe563p - 1./162.*m44*RSp563e*RSe463p - 1./108.*m44*pow(
         RSp563e,2) - 1./162.*m44*RSp463e*RS346pe - 1./162.*m44*RSp463e
         *RS356pe - 1./162.*m44*RSp463e*RSe563p - 1./162.*m44*RSp463e*
         RSe463p - 1./162.*m44*RSp463e*RSp563e - 1./324.*m44*pow(
         RSp463e,2) + 1./162.*m44*RS146pe*RSp563e + 1./162.*m44*RS146pe
         *RSp463e + 1./54.*m44*RS156pe*RSp563e + 1./162.*m44*RS156pe*
         RSp463e + 1./54.*m44*RSe561p*RSp563e + 1./162.*m44*RSe561p*
         RSp463e + 1./162.*m44*RSe461p*RSp563e + 1./162.*m44*RSe461p*
         RSp463e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./162.*m44*RSp561e*
         RS346pe + 1./54.*m44*RSp561e*RS356pe + 1./54.*m44*RSp561e*
         RSe563p + 1./162.*m44*RSp561e*RSe463p + 1./54.*m44*RSp561e*
         RSp563e + 1./162.*m44*RSp561e*RSp463e - 1./162.*m44*RSp561e*
         RS146pe - 1./54.*m44*RSp561e*RS156pe - 1./54.*m44*RSp561e*
         RSe561p - 1./162.*m44*RSp561e*RSe461p - 1./108.*m44*pow(
         RSp561e,2) + 1./162.*m44*RSp461e*RS346pe + 1./162.*m44*RSp461e
         *RS356pe + 1./162.*m44*RSp461e*RSe563p + 1./162.*m44*RSp461e*
         RSe463p + 1./162.*m44*RSp461e*RSp563e + 1./162.*m44*RSp461e*
         RSp463e - 1./162.*m44*RSp461e*RS146pe - 1./162.*m44*RSp461e*
         RS156pe - 1./162.*m44*RSp461e*RSe561p - 1./162.*m44*RSp461e*
         RSe461p - 1./162.*m44*RSp461e*RSp561e - 1./324.*m44*pow(
         RSp461e,2) + 1./54.*m33*RSp563e*RS346pe - 1./54.*m33*RSp563e*
         RS356pe - 1./54.*m33*RSp563e*RSe563p + 1./54.*m33*RSp563e*
         RSe463p - 1./108.*m33*pow(RSp563e,2) - 1./54.*m33*RSp463e*
         RS346pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./54.*m33*RSp463e*
         RS356pe + 1./54.*m33*RSp463e*RSe563p - 1./54.*m33*RSp463e*
         RSe463p + 1./54.*m33*RSp463e*RSp563e - 1./108.*m33*pow(
         RSp463e,2) + 1./162.*m33*RS146pe*RSp563e - 1./162.*m33*RS146pe
         *RSp463e - 1./162.*m33*RS156pe*RSp563e + 1./162.*m33*RS156pe*
         RSp463e - 1./162.*m33*RSe561p*RSp563e + 1./162.*m33*RSe561p*
         RSp463e + 1./162.*m33*RSe461p*RSp563e - 1./162.*m33*RSe461p*
         RSp463e + 1./162.*m33*RSp561e*RS346pe - 1./162.*m33*RSp561e*
         RS356pe - 1./162.*m33*RSp561e*RSe563p + 1./162.*m33*RSp561e*
         RSe463p - 1./162.*m33*RSp561e*RSp563e + 1./162.*m33*RSp561e*
         RSp463e + 1./162.*m33*RSp561e*RS146pe - 1./162.*m33*RSp561e*
         RS156pe - 1./162.*m33*RSp561e*RSe561p + 1./162.*m33*RSp561e*
         RSe461p - 1./324.*m33*pow(RSp561e,2) - 1./162.*m33*RSp461e*
         RS346pe + 1./162.*m33*RSp461e*RS356pe + 1./162.*m33*RSp461e*
         RSe563p - 1./162.*m33*RSp461e*RSe463p + 1./162.*m33*RSp461e*
         RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./162.*m33*
         RSp461e*RSp463e - 1./162.*m33*RSp461e*RS146pe + 1./162.*m33*
         RSp461e*RS156pe + 1./162.*m33*RSp461e*RSe561p - 1./162.*m33*
         RSp461e*RSe461p + 1./162.*m33*RSp461e*RSp561e - 1./324.*m33*
         pow(RSp461e,2) + 1./81.*m33*mp2*RN3p*RSp563e*RS346pe - 1./81.*
         m33*mp2*RN3p*RSp563e*RS356pe - 1./81.*m33*mp2*RN3p*RSp463e*
         RS346pe + 1./81.*m33*mp2*RN3p*RSp463e*RS356pe - 1./81.*m33*mp2
         *RN3p*RSp561e*RS346pe + 1./81.*m33*mp2*RN3p*RSp561e*RS356pe + 
         1./81.*m33*mp2*RN3p*RSp461e*RS346pe - 1./81.*m33*mp2*RN3p*
         RSp461e*RS356pe - 1./324.*m33*m55*RN3p*RSp563e*RS346pe - 1./
         324.*m33*m55*RN3p*RSp563e*RS356pe - 1./108.*m33*m55*RN3p*
         RSp463e*RS346pe - 1./324.*m33*m55*RN3p*RSp463e*RS356pe - 1./
         108.*m33*m55*RN3p*RSp561e*RS346pe - 1./108.*m33*m55*RN3p*
         RSp561e*RS356pe - 1./36.*m33*m55*RN3p*RSp461e*RS346pe - 1./108.
         *m33*m55*RN3p*RSp461e*RS356pe - 1./324.*m33*m55*RN1p*RS146pe*
         RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./108.*m33*m55*
         RN1p*RS146pe*RSp463e - 1./324.*m33*m55*RN1p*RS156pe*RSp563e - 
         1./324.*m33*m55*RN1p*RS156pe*RSp463e - 1./324.*m33*m55*RN1p*
         RSp561e*RS146pe - 1./324.*m33*m55*RN1p*RSp561e*RS156pe - 1./
         108.*m33*m55*RN1p*RSp461e*RS146pe - 1./324.*m33*m55*RN1p*
         RSp461e*RS156pe - 1./162.*m33*m45*RN3p*RSp563e*RS346pe + 1./
         162.*m33*m45*RN3p*RSp563e*RS356pe + 1./162.*m33*m45*RN3p*
         RSp463e*RS346pe - 1./162.*m33*m45*RN3p*RSp463e*RS356pe - 1./54.
         *m33*m45*RN3p*RSp561e*RS346pe + 1./54.*m33*m45*RN3p*RSp561e*
         RS356pe + 1./54.*m33*m45*RN3p*RSp461e*RS346pe - 1./54.*m33*m45
         *RN3p*RSp461e*RS356pe - 1./162.*m33*m45*RN1p*RS146pe*RSp563e
          + 1./162.*m33*m45*RN1p*RS146pe*RSp463e + 1./162.*m33*m45*RN1p
         *RS156pe*RSp563e - 1./162.*m33*m45*RN1p*RS156pe*RSp463e - 1./
         162.*m33*m45*RN1p*RSp561e*RS146pe + 1./162.*m33*m45*RN1p*
         RSp561e*RS156pe + 1./162.*m33*m45*RN1p*RSp461e*RS146pe - 1./
         162.*m33*m45*RN1p*RSp461e*RS156pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./324.*m33*m44*
         RN3p*RSp563e*RS346pe - 1./108.*m33*m44*RN3p*RSp563e*RS356pe - 
         1./324.*m33*m44*RN3p*RSp463e*RS346pe - 1./324.*m33*m44*RN3p*
         RSp463e*RS356pe - 1./108.*m33*m44*RN3p*RSp561e*RS346pe - 1./36.
         *m33*m44*RN3p*RSp561e*RS356pe - 1./108.*m33*m44*RN3p*RSp461e*
         RS346pe - 1./108.*m33*m44*RN3p*RSp461e*RS356pe - 1./324.*m33*
         m44*RN1p*RS146pe*RSp563e - 1./324.*m33*m44*RN1p*RS146pe*
         RSp463e - 1./108.*m33*m44*RN1p*RS156pe*RSp563e - 1./324.*m33*
         m44*RN1p*RS156pe*RSp463e - 1./324.*m33*m44*RN1p*RSp561e*
         RS146pe - 1./108.*m33*m44*RN1p*RSp561e*RS156pe - 1./324.*m33*
         m44*RN1p*RSp461e*RS146pe - 1./324.*m33*m44*RN1p*RSp461e*
         RS156pe - 1./54.*pow(m33,2)*RN3p*RSp563e*RS346pe + 1./54.*pow(
         m33,2)*RN3p*RSp563e*RS356pe + 1./54.*pow(m33,2)*RN3p*RSp463e*
         RS346pe - 1./54.*pow(m33,2)*RN3p*RSp463e*RS356pe - 1./162.*
         pow(m33,2)*RN3p*RSp561e*RS346pe + 1./162.*pow(m33,2)*RN3p*
         RSp561e*RS356pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./162.*pow(m33,2)*
         RN3p*RSp461e*RS346pe - 1./162.*pow(m33,2)*RN3p*RSp461e*RS356pe
          - 1./81.*m13*RSp563e*RS346pe + 1./81.*m13*RSp563e*RS356pe + 1.
         /81.*m13*RSp563e*RSe563p - 1./81.*m13*RSp563e*RSe463p + 1./162.
         *m13*pow(RSp563e,2) + 1./81.*m13*RSp463e*RS346pe - 1./81.*m13*
         RSp463e*RS356pe - 1./81.*m13*RSp463e*RSe563p + 1./81.*m13*
         RSp463e*RSe463p - 1./81.*m13*RSp463e*RSp563e + 1./162.*m13*
         pow(RSp463e,2) + 1./81.*m13*RS146pe*RSp563e - 1./81.*m13*
         RS146pe*RSp463e - 1./81.*m13*RS156pe*RSp563e + 1./81.*m13*
         RS156pe*RSp463e - 1./81.*m13*RSe561p*RSp563e + 1./81.*m13*
         RSe561p*RSp463e + 1./81.*m13*RSe461p*RSp563e - 1./81.*m13*
         RSe461p*RSp463e + 1./81.*m13*RSp561e*RS346pe - 1./81.*m13*
         RSp561e*RS356pe - 1./81.*m13*RSp561e*RSe563p + 1./81.*m13*
         RSp561e*RSe463p - 1./81.*m13*RSp561e*RSp563e + 1./81.*m13*
         RSp561e*RSp463e - 1./81.*m13*RSp561e*RS146pe + 1./81.*m13*
         RSp561e*RS156pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./81.*m13*RSp561e*
         RSe561p - 1./81.*m13*RSp561e*RSe461p + 1./162.*m13*pow(
         RSp561e,2) - 1./81.*m13*RSp461e*RS346pe + 1./81.*m13*RSp461e*
         RS356pe + 1./81.*m13*RSp461e*RSe563p - 1./81.*m13*RSp461e*
         RSe463p + 1./81.*m13*RSp461e*RSp563e - 1./81.*m13*RSp461e*
         RSp463e + 1./81.*m13*RSp461e*RS146pe - 1./81.*m13*RSp461e*
         RS156pe - 1./81.*m13*RSp461e*RSe561p + 1./81.*m13*RSp461e*
         RSe461p - 1./81.*m13*RSp461e*RSp561e + 1./162.*m13*pow(
         RSp461e,2) + 1./162.*m13*m55*RN3p*RSp563e*RS346pe + 1./162.*
         m13*m55*RN3p*RSp563e*RS356pe + 1./54.*m13*m55*RN3p*RSp463e*
         RS346pe + 1./162.*m13*m55*RN3p*RSp463e*RS356pe - 1./162.*m13*
         m55*RN3p*RSp561e*RS346pe - 1./162.*m13*m55*RN3p*RSp561e*
         RS356pe - 1./54.*m13*m55*RN3p*RSp461e*RS346pe - 1./162.*m13*
         m55*RN3p*RSp461e*RS356pe - 1./162.*m13*m55*RN1p*RS146pe*
         RSp563e - 1./54.*m13*m55*RN1p*RS146pe*RSp463e - 1./162.*m13*
         m55*RN1p*RS156pe*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./162.*m13*m55*
         RN1p*RS156pe*RSp463e + 1./162.*m13*m55*RN1p*RSp561e*RS146pe + 
         1./162.*m13*m55*RN1p*RSp561e*RS156pe + 1./54.*m13*m55*RN1p*
         RSp461e*RS146pe + 1./162.*m13*m55*RN1p*RSp461e*RS156pe + 1./81.
         *m13*m45*RN3p*RSp563e*RS346pe - 1./81.*m13*m45*RN3p*RSp563e*
         RS356pe - 1./81.*m13*m45*RN3p*RSp463e*RS346pe + 1./81.*m13*m45
         *RN3p*RSp463e*RS356pe - 1./81.*m13*m45*RN3p*RSp561e*RS346pe + 
         1./81.*m13*m45*RN3p*RSp561e*RS356pe + 1./81.*m13*m45*RN3p*
         RSp461e*RS346pe - 1./81.*m13*m45*RN3p*RSp461e*RS356pe - 1./81.
         *m13*m45*RN1p*RS146pe*RSp563e + 1./81.*m13*m45*RN1p*RS146pe*
         RSp463e + 1./81.*m13*m45*RN1p*RS156pe*RSp563e - 1./81.*m13*m45
         *RN1p*RS156pe*RSp463e + 1./81.*m13*m45*RN1p*RSp561e*RS146pe - 
         1./81.*m13*m45*RN1p*RSp561e*RS156pe - 1./81.*m13*m45*RN1p*
         RSp461e*RS146pe + 1./81.*m13*m45*RN1p*RSp461e*RS156pe + 1./162.
         *m13*m44*RN3p*RSp563e*RS346pe + 1./54.*m13*m44*RN3p*RSp563e*
         RS356pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./162.*m13*m44*RN3p*
         RSp463e*RS346pe + 1./162.*m13*m44*RN3p*RSp463e*RS356pe - 1./
         162.*m13*m44*RN3p*RSp561e*RS346pe - 1./54.*m13*m44*RN3p*
         RSp561e*RS356pe - 1./162.*m13*m44*RN3p*RSp461e*RS346pe - 1./
         162.*m13*m44*RN3p*RSp461e*RS356pe - 1./162.*m13*m44*RN1p*
         RS146pe*RSp563e - 1./162.*m13*m44*RN1p*RS146pe*RSp463e - 1./54.
         *m13*m44*RN1p*RS156pe*RSp563e - 1./162.*m13*m44*RN1p*RS156pe*
         RSp463e + 1./162.*m13*m44*RN1p*RSp561e*RS146pe + 1./54.*m13*
         m44*RN1p*RSp561e*RS156pe + 1./162.*m13*m44*RN1p*RSp461e*
         RS146pe + 1./162.*m13*m44*RN1p*RSp461e*RS156pe + 1./81.*m13*
         m33*RN3p*RSp563e*RS346pe - 1./81.*m13*m33*RN3p*RSp563e*RS356pe
          - 1./81.*m13*m33*RN3p*RSp463e*RS346pe + 1./81.*m13*m33*RN3p*
         RSp463e*RS356pe - 1./81.*m13*m33*RN3p*RSp561e*RS346pe + 1./81.
         *m13*m33*RN3p*RSp561e*RS356pe + 1./81.*m13*m33*RN3p*RSp461e*
         RS346pe - 1./81.*m13*m33*RN3p*RSp461e*RS356pe + 1./162.*m11*
         RSp563e*RS346pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./162.*m11*
         RSp563e*RS356pe - 1./162.*m11*RSp563e*RSe563p + 1./162.*m11*
         RSp563e*RSe463p - 1./324.*m11*pow(RSp563e,2) - 1./162.*m11*
         RSp463e*RS346pe + 1./162.*m11*RSp463e*RS356pe + 1./162.*m11*
         RSp463e*RSe563p - 1./162.*m11*RSp463e*RSe463p + 1./162.*m11*
         RSp463e*RSp563e - 1./324.*m11*pow(RSp463e,2) + 1./162.*m11*
         RS146pe*RSp563e - 1./162.*m11*RS146pe*RSp463e - 1./162.*m11*
         RS156pe*RSp563e + 1./162.*m11*RS156pe*RSp463e - 1./162.*m11*
         RSe561p*RSp563e + 1./162.*m11*RSe561p*RSp463e + 1./162.*m11*
         RSe461p*RSp563e - 1./162.*m11*RSe461p*RSp463e + 1./162.*m11*
         RSp561e*RS346pe - 1./162.*m11*RSp561e*RS356pe - 1./162.*m11*
         RSp561e*RSe563p + 1./162.*m11*RSp561e*RSe463p - 1./162.*m11*
         RSp561e*RSp563e + 1./162.*m11*RSp561e*RSp463e + 1./54.*m11*
         RSp561e*RS146pe - 1./54.*m11*RSp561e*RS156pe - 1./54.*m11*
         RSp561e*RSe561p + 1./54.*m11*RSp561e*RSe461p - 1./108.*m11*
         pow(RSp561e,2) );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./162.*m11*
         RSp461e*RS346pe + 1./162.*m11*RSp461e*RS356pe + 1./162.*m11*
         RSp461e*RSe563p - 1./162.*m11*RSp461e*RSe463p + 1./162.*m11*
         RSp461e*RSp563e - 1./162.*m11*RSp461e*RSp463e - 1./54.*m11*
         RSp461e*RS146pe + 1./54.*m11*RSp461e*RS156pe + 1./54.*m11*
         RSp461e*RSe561p - 1./54.*m11*RSp461e*RSe461p + 1./54.*m11*
         RSp461e*RSp561e - 1./108.*m11*pow(RSp461e,2) - 1./81.*m11*mp2*
         RN1p*RS146pe*RSp563e + 1./81.*m11*mp2*RN1p*RS146pe*RSp463e + 1.
         /81.*m11*mp2*RN1p*RS156pe*RSp563e - 1./81.*m11*mp2*RN1p*
         RS156pe*RSp463e + 1./81.*m11*mp2*RN1p*RSp561e*RS146pe - 1./81.
         *m11*mp2*RN1p*RSp561e*RS156pe - 1./81.*m11*mp2*RN1p*RSp461e*
         RS146pe + 1./81.*m11*mp2*RN1p*RSp461e*RS156pe - 1./324.*m11*
         m55*RN3p*RSp563e*RS346pe - 1./324.*m11*m55*RN3p*RSp563e*
         RS356pe - 1./108.*m11*m55*RN3p*RSp463e*RS346pe - 1./324.*m11*
         m55*RN3p*RSp463e*RS356pe - 1./324.*m11*m55*RN3p*RSp561e*
         RS346pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./324.*m11*m55*
         RN3p*RSp561e*RS356pe - 1./108.*m11*m55*RN3p*RSp461e*RS346pe - 
         1./324.*m11*m55*RN3p*RSp461e*RS356pe - 1./108.*m11*m55*RN1p*
         RS146pe*RSp563e - 1./36.*m11*m55*RN1p*RS146pe*RSp463e - 1./108.
         *m11*m55*RN1p*RS156pe*RSp563e - 1./108.*m11*m55*RN1p*RS156pe*
         RSp463e - 1./324.*m11*m55*RN1p*RSp561e*RS146pe - 1./324.*m11*
         m55*RN1p*RSp561e*RS156pe - 1./108.*m11*m55*RN1p*RSp461e*
         RS146pe - 1./324.*m11*m55*RN1p*RSp461e*RS156pe - 1./162.*m11*
         m45*RN3p*RSp563e*RS346pe + 1./162.*m11*m45*RN3p*RSp563e*
         RS356pe + 1./162.*m11*m45*RN3p*RSp463e*RS346pe - 1./162.*m11*
         m45*RN3p*RSp463e*RS356pe - 1./162.*m11*m45*RN3p*RSp561e*
         RS346pe + 1./162.*m11*m45*RN3p*RSp561e*RS356pe + 1./162.*m11*
         m45*RN3p*RSp461e*RS346pe - 1./162.*m11*m45*RN3p*RSp461e*
         RS356pe - 1./54.*m11*m45*RN1p*RS146pe*RSp563e + 1./54.*m11*m45
         *RN1p*RS146pe*RSp463e + 1./54.*m11*m45*RN1p*RS156pe*RSp563e - 
         1./54.*m11*m45*RN1p*RS156pe*RSp463e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./162.*m11*m45*
         RN1p*RSp561e*RS146pe + 1./162.*m11*m45*RN1p*RSp561e*RS156pe + 
         1./162.*m11*m45*RN1p*RSp461e*RS146pe - 1./162.*m11*m45*RN1p*
         RSp461e*RS156pe - 1./324.*m11*m44*RN3p*RSp563e*RS346pe - 1./
         108.*m11*m44*RN3p*RSp563e*RS356pe - 1./324.*m11*m44*RN3p*
         RSp463e*RS346pe - 1./324.*m11*m44*RN3p*RSp463e*RS356pe - 1./
         324.*m11*m44*RN3p*RSp561e*RS346pe - 1./108.*m11*m44*RN3p*
         RSp561e*RS356pe - 1./324.*m11*m44*RN3p*RSp461e*RS346pe - 1./
         324.*m11*m44*RN3p*RSp461e*RS356pe - 1./108.*m11*m44*RN1p*
         RS146pe*RSp563e - 1./108.*m11*m44*RN1p*RS146pe*RSp463e - 1./36.
         *m11*m44*RN1p*RS156pe*RSp563e - 1./108.*m11*m44*RN1p*RS156pe*
         RSp463e - 1./324.*m11*m44*RN1p*RSp561e*RS146pe - 1./108.*m11*
         m44*RN1p*RSp561e*RS156pe - 1./324.*m11*m44*RN1p*RSp461e*
         RS146pe - 1./324.*m11*m44*RN1p*RSp461e*RS156pe - 1./162.*m11*
         m33*RN3p*RSp563e*RS346pe + 1./162.*m11*m33*RN3p*RSp563e*
         RS356pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./162.*m11*m33*RN3p*
         RSp463e*RS346pe - 1./162.*m11*m33*RN3p*RSp463e*RS356pe - 1./
         162.*m11*m33*RN3p*RSp561e*RS346pe + 1./162.*m11*m33*RN3p*
         RSp561e*RS356pe + 1./162.*m11*m33*RN3p*RSp461e*RS346pe - 1./
         162.*m11*m33*RN3p*RSp461e*RS356pe - 1./162.*m11*m33*RN1p*
         RS146pe*RSp563e + 1./162.*m11*m33*RN1p*RS146pe*RSp463e + 1./
         162.*m11*m33*RN1p*RS156pe*RSp563e - 1./162.*m11*m33*RN1p*
         RS156pe*RSp463e - 1./162.*m11*m33*RN1p*RSp561e*RS146pe + 1./
         162.*m11*m33*RN1p*RSp561e*RS156pe + 1./162.*m11*m33*RN1p*
         RSp461e*RS146pe - 1./162.*m11*m33*RN1p*RSp461e*RS156pe - 1./81.
         *m11*m13*RN1p*RS146pe*RSp563e + 1./81.*m11*m13*RN1p*RS146pe*
         RSp463e + 1./81.*m11*m13*RN1p*RS156pe*RSp563e - 1./81.*m11*m13
         *RN1p*RS156pe*RSp463e + 1./81.*m11*m13*RN1p*RSp561e*RS146pe - 
         1./81.*m11*m13*RN1p*RSp561e*RS156pe - 1./81.*m11*m13*RN1p*
         RSp461e*RS146pe + 1./81.*m11*m13*RN1p*RSp461e*RS156pe - 1./162.
         *pow(m11,2)*RN1p*RS146pe*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./162.*pow(m11,2)*
         RN1p*RS146pe*RSp463e + 1./162.*pow(m11,2)*RN1p*RS156pe*RSp563e
          - 1./162.*pow(m11,2)*RN1p*RS156pe*RSp463e - 1./54.*pow(m11,2)
         *RN1p*RSp561e*RS146pe + 1./54.*pow(m11,2)*RN1p*RSp561e*RS156pe
          + 1./54.*pow(m11,2)*RN1p*RSp461e*RS146pe - 1./54.*pow(m11,2)*
         RN1p*RSp461e*RS156pe + 1./81.*RN(mp2,me2)*mp2*me2*RSp563e*
         RSe563p - 1./81.*RN(mp2,me2)*mp2*me2*RSp563e*RSe463p - 1./81.*
         RN(mp2,me2)*mp2*me2*RSp463e*RSe563p + 1./81.*RN(mp2,me2)*mp2*
         me2*RSp463e*RSe463p - 1./81.*RN(mp2,me2)*mp2*me2*RSe561p*
         RSp563e + 1./81.*RN(mp2,me2)*mp2*me2*RSe561p*RSp463e + 1./81.*
         RN(mp2,me2)*mp2*me2*RSe461p*RSp563e - 1./81.*RN(mp2,me2)*mp2*
         me2*RSe461p*RSp463e - 1./81.*RN(mp2,me2)*mp2*me2*RSp561e*
         RSe563p + 1./81.*RN(mp2,me2)*mp2*me2*RSp561e*RSe463p + 1./81.*
         RN(mp2,me2)*mp2*me2*RSp561e*RSe561p - 1./81.*RN(mp2,me2)*mp2*
         me2*RSp561e*RSe461p + 1./81.*RN(mp2,me2)*mp2*me2*RSp461e*
         RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./81.*RN(mp2,me2)
         *mp2*me2*RSp461e*RSe463p - 1./81.*RN(mp2,me2)*mp2*me2*RSp461e*
         RSe561p + 1./81.*RN(mp2,me2)*mp2*me2*RSp461e*RSe461p - 1./162.
         *RN(mp2,me2)*m55*me2*RSp563e*RSe563p - 1./162.*RN(mp2,me2)*m55
         *me2*RSp563e*RSe463p - 1./162.*RN(mp2,me2)*m55*me2*RSp463e*
         RSe563p - 1./54.*RN(mp2,me2)*m55*me2*RSp463e*RSe463p + 1./162.
         *RN(mp2,me2)*m55*me2*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m55
         *me2*RSe561p*RSp463e + 1./162.*RN(mp2,me2)*m55*me2*RSe461p*
         RSp563e + 1./54.*RN(mp2,me2)*m55*me2*RSe461p*RSp463e + 1./162.
         *RN(mp2,me2)*m55*me2*RSp561e*RSe563p + 1./162.*RN(mp2,me2)*m55
         *me2*RSp561e*RSe463p - 1./162.*RN(mp2,me2)*m55*me2*RSp561e*
         RSe561p - 1./162.*RN(mp2,me2)*m55*me2*RSp561e*RSe461p + 1./162.
         *RN(mp2,me2)*m55*me2*RSp461e*RSe563p + 1./54.*RN(mp2,me2)*m55*
         me2*RSp461e*RSe463p - 1./162.*RN(mp2,me2)*m55*me2*RSp461e*
         RSe561p - 1./54.*RN(mp2,me2)*m55*me2*RSp461e*RSe461p + 1./81.*
         RN(mp2,me2)*m45*me2*RSp563e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./81.*RN(mp2,me2)
         *m45*me2*RSp563e*RSe463p - 1./81.*RN(mp2,me2)*m45*me2*RSp463e*
         RSe563p + 1./81.*RN(mp2,me2)*m45*me2*RSp463e*RSe463p - 1./81.*
         RN(mp2,me2)*m45*me2*RSe561p*RSp563e + 1./81.*RN(mp2,me2)*m45*
         me2*RSe561p*RSp463e + 1./81.*RN(mp2,me2)*m45*me2*RSe461p*
         RSp563e - 1./81.*RN(mp2,me2)*m45*me2*RSe461p*RSp463e - 1./81.*
         RN(mp2,me2)*m45*me2*RSp561e*RSe563p + 1./81.*RN(mp2,me2)*m45*
         me2*RSp561e*RSe463p + 1./81.*RN(mp2,me2)*m45*me2*RSp561e*
         RSe561p - 1./81.*RN(mp2,me2)*m45*me2*RSp561e*RSe461p + 1./81.*
         RN(mp2,me2)*m45*me2*RSp461e*RSe563p - 1./81.*RN(mp2,me2)*m45*
         me2*RSp461e*RSe463p - 1./81.*RN(mp2,me2)*m45*me2*RSp461e*
         RSe561p + 1./81.*RN(mp2,me2)*m45*me2*RSp461e*RSe461p - 1./54.*
         RN(mp2,me2)*m44*me2*RSp563e*RSe563p - 1./162.*RN(mp2,me2)*m44*
         me2*RSp563e*RSe463p - 1./162.*RN(mp2,me2)*m44*me2*RSp463e*
         RSe563p - 1./162.*RN(mp2,me2)*m44*me2*RSp463e*RSe463p + 1./54.
         *RN(mp2,me2)*m44*me2*RSe561p*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./162.*RN(mp2,me2)*
         m44*me2*RSe561p*RSp463e + 1./162.*RN(mp2,me2)*m44*me2*RSe461p*
         RSp563e + 1./162.*RN(mp2,me2)*m44*me2*RSe461p*RSp463e + 1./54.
         *RN(mp2,me2)*m44*me2*RSp561e*RSe563p + 1./162.*RN(mp2,me2)*m44
         *me2*RSp561e*RSe463p - 1./54.*RN(mp2,me2)*m44*me2*RSp561e*
         RSe561p - 1./162.*RN(mp2,me2)*m44*me2*RSp561e*RSe461p + 1./162.
         *RN(mp2,me2)*m44*me2*RSp461e*RSe563p + 1./162.*RN(mp2,me2)*m44
         *me2*RSp461e*RSe463p - 1./162.*RN(mp2,me2)*m44*me2*RSp461e*
         RSe561p - 1./162.*RN(mp2,me2)*m44*me2*RSp461e*RSe461p - 1./54.
         *RN(mp2,me2)*m33*me2*RSp563e*RSe563p + 1./54.*RN(mp2,me2)*m33*
         me2*RSp563e*RSe463p + 1./54.*RN(mp2,me2)*m33*me2*RSp463e*
         RSe563p - 1./54.*RN(mp2,me2)*m33*me2*RSp463e*RSe463p - 1./162.
         *RN(mp2,me2)*m33*me2*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m33
         *me2*RSe561p*RSp463e + 1./162.*RN(mp2,me2)*m33*me2*RSe461p*
         RSp563e - 1./162.*RN(mp2,me2)*m33*me2*RSe461p*RSp463e - 1./162.
         *RN(mp2,me2)*m33*me2*RSp561e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./162.*RN(mp2,me2)*
         m33*me2*RSp561e*RSe463p - 1./162.*RN(mp2,me2)*m33*me2*RSp561e*
         RSe561p + 1./162.*RN(mp2,me2)*m33*me2*RSp561e*RSe461p + 1./162.
         *RN(mp2,me2)*m33*me2*RSp461e*RSe563p - 1./162.*RN(mp2,me2)*m33
         *me2*RSp461e*RSe463p + 1./162.*RN(mp2,me2)*m33*me2*RSp461e*
         RSe561p - 1./162.*RN(mp2,me2)*m33*me2*RSp461e*RSe461p + 1./108.
         *RN(mp2,me2)*m33*m55*RSp563e*RSe563p + 1./108.*RN(mp2,me2)*m33
         *m55*RSp563e*RSe463p + 1./108.*RN(mp2,me2)*m33*m55*RSp463e*
         RSe563p + 1./36.*RN(mp2,me2)*m33*m55*RSp463e*RSe463p + 1./324.
         *RN(mp2,me2)*m33*m55*RSe561p*RSp563e + 1./324.*RN(mp2,me2)*m33
         *m55*RSe561p*RSp463e + 1./324.*RN(mp2,me2)*m33*m55*RSe461p*
         RSp563e + 1./108.*RN(mp2,me2)*m33*m55*RSe461p*RSp463e + 1./324.
         *RN(mp2,me2)*m33*m55*RSp561e*RSe563p + 1./324.*RN(mp2,me2)*m33
         *m55*RSp561e*RSe463p + 1./324.*RN(mp2,me2)*m33*m55*RSp561e*
         RSe561p + 1./324.*RN(mp2,me2)*m33*m55*RSp561e*RSe461p + 1./324.
         *RN(mp2,me2)*m33*m55*RSp461e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./108.*RN(mp2,me2)*
         m33*m55*RSp461e*RSe463p + 1./324.*RN(mp2,me2)*m33*m55*RSp461e*
         RSe561p + 1./108.*RN(mp2,me2)*m33*m55*RSp461e*RSe461p - 1./54.
         *RN(mp2,me2)*m33*m45*RSp563e*RSe563p + 1./54.*RN(mp2,me2)*m33*
         m45*RSp563e*RSe463p + 1./54.*RN(mp2,me2)*m33*m45*RSp463e*
         RSe563p - 1./54.*RN(mp2,me2)*m33*m45*RSp463e*RSe463p - 1./162.
         *RN(mp2,me2)*m33*m45*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m33
         *m45*RSe561p*RSp463e + 1./162.*RN(mp2,me2)*m33*m45*RSe461p*
         RSp563e - 1./162.*RN(mp2,me2)*m33*m45*RSe461p*RSp463e - 1./162.
         *RN(mp2,me2)*m33*m45*RSp561e*RSe563p + 1./162.*RN(mp2,me2)*m33
         *m45*RSp561e*RSe463p - 1./162.*RN(mp2,me2)*m33*m45*RSp561e*
         RSe561p + 1./162.*RN(mp2,me2)*m33*m45*RSp561e*RSe461p + 1./162.
         *RN(mp2,me2)*m33*m45*RSp461e*RSe563p - 1./162.*RN(mp2,me2)*m33
         *m45*RSp461e*RSe463p + 1./162.*RN(mp2,me2)*m33*m45*RSp461e*
         RSe561p - 1./162.*RN(mp2,me2)*m33*m45*RSp461e*RSe461p + 1./36.
         *RN(mp2,me2)*m33*m44*RSp563e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./108.*RN(mp2,me2)*
         m33*m44*RSp563e*RSe463p + 1./108.*RN(mp2,me2)*m33*m44*RSp463e*
         RSe563p + 1./108.*RN(mp2,me2)*m33*m44*RSp463e*RSe463p + 1./108.
         *RN(mp2,me2)*m33*m44*RSe561p*RSp563e + 1./324.*RN(mp2,me2)*m33
         *m44*RSe561p*RSp463e + 1./324.*RN(mp2,me2)*m33*m44*RSe461p*
         RSp563e + 1./324.*RN(mp2,me2)*m33*m44*RSe461p*RSp463e + 1./108.
         *RN(mp2,me2)*m33*m44*RSp561e*RSe563p + 1./324.*RN(mp2,me2)*m33
         *m44*RSp561e*RSe463p + 1./108.*RN(mp2,me2)*m33*m44*RSp561e*
         RSe561p + 1./324.*RN(mp2,me2)*m33*m44*RSp561e*RSe461p + 1./324.
         *RN(mp2,me2)*m33*m44*RSp461e*RSe563p + 1./324.*RN(mp2,me2)*m33
         *m44*RSp461e*RSe463p + 1./324.*RN(mp2,me2)*m33*m44*RSp461e*
         RSe561p + 1./324.*RN(mp2,me2)*m33*m44*RSp461e*RSe461p + 1./81.
         *RN(mp2,me2)*m13*me2*RSp563e*RSe563p - 1./81.*RN(mp2,me2)*m13*
         me2*RSp563e*RSe463p - 1./81.*RN(mp2,me2)*m13*me2*RSp463e*
         RSe563p + 1./81.*RN(mp2,me2)*m13*me2*RSp463e*RSe463p - 1./81.*
         RN(mp2,me2)*m13*me2*RSe561p*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./81.*RN(mp2,me2)*
         m13*me2*RSe561p*RSp463e + 1./81.*RN(mp2,me2)*m13*me2*RSe461p*
         RSp563e - 1./81.*RN(mp2,me2)*m13*me2*RSe461p*RSp463e - 1./81.*
         RN(mp2,me2)*m13*me2*RSp561e*RSe563p + 1./81.*RN(mp2,me2)*m13*
         me2*RSp561e*RSe463p + 1./81.*RN(mp2,me2)*m13*me2*RSp561e*
         RSe561p - 1./81.*RN(mp2,me2)*m13*me2*RSp561e*RSe461p + 1./81.*
         RN(mp2,me2)*m13*me2*RSp461e*RSe563p - 1./81.*RN(mp2,me2)*m13*
         me2*RSp461e*RSe463p - 1./81.*RN(mp2,me2)*m13*me2*RSp461e*
         RSe561p + 1./81.*RN(mp2,me2)*m13*me2*RSp461e*RSe461p - 1./162.
         *RN(mp2,me2)*m13*m55*RSp563e*RSe563p - 1./162.*RN(mp2,me2)*m13
         *m55*RSp563e*RSe463p - 1./162.*RN(mp2,me2)*m13*m55*RSp463e*
         RSe563p - 1./54.*RN(mp2,me2)*m13*m55*RSp463e*RSe463p + 1./162.
         *RN(mp2,me2)*m13*m55*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m13
         *m55*RSe561p*RSp463e + 1./162.*RN(mp2,me2)*m13*m55*RSe461p*
         RSp563e + 1./54.*RN(mp2,me2)*m13*m55*RSe461p*RSp463e + 1./162.
         *RN(mp2,me2)*m13*m55*RSp561e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./162.*RN(mp2,me2)*
         m13*m55*RSp561e*RSe463p - 1./162.*RN(mp2,me2)*m13*m55*RSp561e*
         RSe561p - 1./162.*RN(mp2,me2)*m13*m55*RSp561e*RSe461p + 1./162.
         *RN(mp2,me2)*m13*m55*RSp461e*RSe563p + 1./54.*RN(mp2,me2)*m13*
         m55*RSp461e*RSe463p - 1./162.*RN(mp2,me2)*m13*m55*RSp461e*
         RSe561p - 1./54.*RN(mp2,me2)*m13*m55*RSp461e*RSe461p + 1./81.*
         RN(mp2,me2)*m13*m45*RSp563e*RSe563p - 1./81.*RN(mp2,me2)*m13*
         m45*RSp563e*RSe463p - 1./81.*RN(mp2,me2)*m13*m45*RSp463e*
         RSe563p + 1./81.*RN(mp2,me2)*m13*m45*RSp463e*RSe463p - 1./81.*
         RN(mp2,me2)*m13*m45*RSe561p*RSp563e + 1./81.*RN(mp2,me2)*m13*
         m45*RSe561p*RSp463e + 1./81.*RN(mp2,me2)*m13*m45*RSe461p*
         RSp563e - 1./81.*RN(mp2,me2)*m13*m45*RSe461p*RSp463e - 1./81.*
         RN(mp2,me2)*m13*m45*RSp561e*RSe563p + 1./81.*RN(mp2,me2)*m13*
         m45*RSp561e*RSe463p + 1./81.*RN(mp2,me2)*m13*m45*RSp561e*
         RSe561p - 1./81.*RN(mp2,me2)*m13*m45*RSp561e*RSe461p + 1./81.*
         RN(mp2,me2)*m13*m45*RSp461e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./81.*RN(mp2,me2)
         *m13*m45*RSp461e*RSe463p - 1./81.*RN(mp2,me2)*m13*m45*RSp461e*
         RSe561p + 1./81.*RN(mp2,me2)*m13*m45*RSp461e*RSe461p - 1./54.*
         RN(mp2,me2)*m13*m44*RSp563e*RSe563p - 1./162.*RN(mp2,me2)*m13*
         m44*RSp563e*RSe463p - 1./162.*RN(mp2,me2)*m13*m44*RSp463e*
         RSe563p - 1./162.*RN(mp2,me2)*m13*m44*RSp463e*RSe463p + 1./54.
         *RN(mp2,me2)*m13*m44*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m13
         *m44*RSe561p*RSp463e + 1./162.*RN(mp2,me2)*m13*m44*RSe461p*
         RSp563e + 1./162.*RN(mp2,me2)*m13*m44*RSe461p*RSp463e + 1./54.
         *RN(mp2,me2)*m13*m44*RSp561e*RSe563p + 1./162.*RN(mp2,me2)*m13
         *m44*RSp561e*RSe463p - 1./54.*RN(mp2,me2)*m13*m44*RSp561e*
         RSe561p - 1./162.*RN(mp2,me2)*m13*m44*RSp561e*RSe461p + 1./162.
         *RN(mp2,me2)*m13*m44*RSp461e*RSe563p + 1./162.*RN(mp2,me2)*m13
         *m44*RSp461e*RSe463p - 1./162.*RN(mp2,me2)*m13*m44*RSp461e*
         RSe561p - 1./162.*RN(mp2,me2)*m13*m44*RSp461e*RSe461p - 1./162.
         *RN(mp2,me2)*m11*me2*RSp563e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./162.*RN(mp2,me2)*
         m11*me2*RSp563e*RSe463p + 1./162.*RN(mp2,me2)*m11*me2*RSp463e*
         RSe563p - 1./162.*RN(mp2,me2)*m11*me2*RSp463e*RSe463p - 1./162.
         *RN(mp2,me2)*m11*me2*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m11
         *me2*RSe561p*RSp463e + 1./162.*RN(mp2,me2)*m11*me2*RSe461p*
         RSp563e - 1./162.*RN(mp2,me2)*m11*me2*RSe461p*RSp463e - 1./162.
         *RN(mp2,me2)*m11*me2*RSp561e*RSe563p + 1./162.*RN(mp2,me2)*m11
         *me2*RSp561e*RSe463p - 1./54.*RN(mp2,me2)*m11*me2*RSp561e*
         RSe561p + 1./54.*RN(mp2,me2)*m11*me2*RSp561e*RSe461p + 1./162.
         *RN(mp2,me2)*m11*me2*RSp461e*RSe563p - 1./162.*RN(mp2,me2)*m11
         *me2*RSp461e*RSe463p + 1./54.*RN(mp2,me2)*m11*me2*RSp461e*
         RSe561p - 1./54.*RN(mp2,me2)*m11*me2*RSp461e*RSe461p + 1./324.
         *RN(mp2,me2)*m11*m55*RSp563e*RSe563p + 1./324.*RN(mp2,me2)*m11
         *m55*RSp563e*RSe463p + 1./324.*RN(mp2,me2)*m11*m55*RSp463e*
         RSe563p + 1./108.*RN(mp2,me2)*m11*m55*RSp463e*RSe463p + 1./324.
         *RN(mp2,me2)*m11*m55*RSe561p*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./324.*RN(mp2,me2)*
         m11*m55*RSe561p*RSp463e + 1./324.*RN(mp2,me2)*m11*m55*RSe461p*
         RSp563e + 1./108.*RN(mp2,me2)*m11*m55*RSe461p*RSp463e + 1./324.
         *RN(mp2,me2)*m11*m55*RSp561e*RSe563p + 1./324.*RN(mp2,me2)*m11
         *m55*RSp561e*RSe463p + 1./108.*RN(mp2,me2)*m11*m55*RSp561e*
         RSe561p + 1./108.*RN(mp2,me2)*m11*m55*RSp561e*RSe461p + 1./324.
         *RN(mp2,me2)*m11*m55*RSp461e*RSe563p + 1./108.*RN(mp2,me2)*m11
         *m55*RSp461e*RSe463p + 1./108.*RN(mp2,me2)*m11*m55*RSp461e*
         RSe561p + 1./36.*RN(mp2,me2)*m11*m55*RSp461e*RSe461p - 1./162.
         *RN(mp2,me2)*m11*m45*RSp563e*RSe563p + 1./162.*RN(mp2,me2)*m11
         *m45*RSp563e*RSe463p + 1./162.*RN(mp2,me2)*m11*m45*RSp463e*
         RSe563p - 1./162.*RN(mp2,me2)*m11*m45*RSp463e*RSe463p - 1./162.
         *RN(mp2,me2)*m11*m45*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m11
         *m45*RSe561p*RSp463e + 1./162.*RN(mp2,me2)*m11*m45*RSe461p*
         RSp563e - 1./162.*RN(mp2,me2)*m11*m45*RSe461p*RSp463e - 1./162.
         *RN(mp2,me2)*m11*m45*RSp561e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./162.*RN(mp2,me2)*
         m11*m45*RSp561e*RSe463p - 1./54.*RN(mp2,me2)*m11*m45*RSp561e*
         RSe561p + 1./54.*RN(mp2,me2)*m11*m45*RSp561e*RSe461p + 1./162.
         *RN(mp2,me2)*m11*m45*RSp461e*RSe563p - 1./162.*RN(mp2,me2)*m11
         *m45*RSp461e*RSe463p + 1./54.*RN(mp2,me2)*m11*m45*RSp461e*
         RSe561p - 1./54.*RN(mp2,me2)*m11*m45*RSp461e*RSe461p + 1./108.
         *RN(mp2,me2)*m11*m44*RSp563e*RSe563p + 1./324.*RN(mp2,me2)*m11
         *m44*RSp563e*RSe463p + 1./324.*RN(mp2,me2)*m11*m44*RSp463e*
         RSe563p + 1./324.*RN(mp2,me2)*m11*m44*RSp463e*RSe463p + 1./108.
         *RN(mp2,me2)*m11*m44*RSe561p*RSp563e + 1./324.*RN(mp2,me2)*m11
         *m44*RSe561p*RSp463e + 1./324.*RN(mp2,me2)*m11*m44*RSe461p*
         RSp563e + 1./324.*RN(mp2,me2)*m11*m44*RSe461p*RSp463e + 1./108.
         *RN(mp2,me2)*m11*m44*RSp561e*RSe563p + 1./324.*RN(mp2,me2)*m11
         *m44*RSp561e*RSe463p + 1./36.*RN(mp2,me2)*m11*m44*RSp561e*
         RSe561p + 1./108.*RN(mp2,me2)*m11*m44*RSp561e*RSe461p + 1./324.
         *RN(mp2,me2)*m11*m44*RSp461e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./324.*RN(mp2,me2)*
         m11*m44*RSp461e*RSe463p + 1./108.*RN(mp2,me2)*m11*m44*RSp461e*
         RSe561p + 1./108.*RN(mp2,me2)*m11*m44*RSp461e*RSe461p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./81.*mp2*RSp563e
         *RS345pe + 1./81.*mp2*RSp563e*RS356pe + 1./81.*mp2*RSp563e*
         RSe563p - 1./81.*mp2*RSp563e*RSe453p + 1./81.*mp2*pow(
         RSp563e,2) + 1./81.*mp2*RSp453e*RS345pe - 1./81.*mp2*RSp453e*
         RS356pe - 1./81.*mp2*RSp453e*RSe563p + 1./81.*mp2*RSp453e*
         RSe453p - 2./81.*mp2*RSp453e*RSp563e + 1./81.*mp2*pow(
         RSp453e,2) + 1./81.*mp2*RS145pe*RSp563e - 1./81.*mp2*RS145pe*
         RSp453e - 1./81.*mp2*RS156pe*RSp563e + 1./81.*mp2*RS156pe*
         RSp453e - 1./81.*mp2*RSe561p*RSp563e + 1./81.*mp2*RSe561p*
         RSp453e + 1./81.*mp2*RSe451p*RSp563e - 1./81.*mp2*RSe451p*
         RSp453e + 1./81.*mp2*RSp561e*RS345pe - 1./81.*mp2*RSp561e*
         RS356pe - 1./81.*mp2*RSp561e*RSe563p + 1./81.*mp2*RSp561e*
         RSe453p - 2./81.*mp2*RSp561e*RSp563e + 2./81.*mp2*RSp561e*
         RSp453e - 1./81.*mp2*RSp561e*RS145pe + 1./81.*mp2*RSp561e*
         RS156pe + 1./81.*mp2*RSp561e*RSe561p - 1./81.*mp2*RSp561e*
         RSe451p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./81.*mp2*pow(
         RSp561e,2) - 1./81.*mp2*RSp451e*RS345pe + 1./81.*mp2*RSp451e*
         RS356pe + 1./81.*mp2*RSp451e*RSe563p - 1./81.*mp2*RSp451e*
         RSe453p + 2./81.*mp2*RSp451e*RSp563e - 2./81.*mp2*RSp451e*
         RSp453e + 1./81.*mp2*RSp451e*RS145pe - 1./81.*mp2*RSp451e*
         RS156pe - 1./81.*mp2*RSp451e*RSe561p + 1./81.*mp2*RSp451e*
         RSe451p - 2./81.*mp2*RSp451e*RSp561e + 1./81.*mp2*pow(
         RSp451e,2) - 1./162.*m66*RSp563e*RS345pe - 1./162.*m66*RSp563e
         *RS356pe - 1./162.*m66*RSp563e*RSe563p - 1./162.*m66*RSp563e*
         RSe453p - 1./324.*m66*pow(RSp563e,2) - 1./54.*m66*RSp453e*
         RS345pe - 1./162.*m66*RSp453e*RS356pe - 1./162.*m66*RSp453e*
         RSe563p - 1./54.*m66*RSp453e*RSe453p - 1./162.*m66*RSp453e*
         RSp563e - 1./108.*m66*pow(RSp453e,2) + 1./162.*m66*RS145pe*
         RSp563e + 1./54.*m66*RS145pe*RSp453e + 1./162.*m66*RS156pe*
         RSp563e + 1./162.*m66*RS156pe*RSp453e + 1./162.*m66*RSe561p*
         RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./162.*m66*RSe561p*
         RSp453e + 1./162.*m66*RSe451p*RSp563e + 1./54.*m66*RSe451p*
         RSp453e + 1./162.*m66*RSp561e*RS345pe + 1./162.*m66*RSp561e*
         RS356pe + 1./162.*m66*RSp561e*RSe563p + 1./162.*m66*RSp561e*
         RSe453p + 1./162.*m66*RSp561e*RSp563e + 1./162.*m66*RSp561e*
         RSp453e - 1./162.*m66*RSp561e*RS145pe - 1./162.*m66*RSp561e*
         RS156pe - 1./162.*m66*RSp561e*RSe561p - 1./162.*m66*RSp561e*
         RSe451p - 1./324.*m66*pow(RSp561e,2) + 1./54.*m66*RSp451e*
         RS345pe + 1./162.*m66*RSp451e*RS356pe + 1./162.*m66*RSp451e*
         RSe563p + 1./54.*m66*RSp451e*RSe453p + 1./162.*m66*RSp451e*
         RSp563e + 1./54.*m66*RSp451e*RSp453e - 1./54.*m66*RSp451e*
         RS145pe - 1./162.*m66*RSp451e*RS156pe - 1./162.*m66*RSp451e*
         RSe561p - 1./54.*m66*RSp451e*RSe451p - 1./162.*m66*RSp451e*
         RSp561e - 1./108.*m66*pow(RSp451e,2) - 1./81.*m46*RSp563e*
         RS345pe + 1./81.*m46*RSp563e*RS356pe + 1./81.*m46*RSp563e*
         RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./81.*m46*RSp563e
         *RSe453p + 1./162.*m46*pow(RSp563e,2) + 1./81.*m46*RSp453e*
         RS345pe - 1./81.*m46*RSp453e*RS356pe - 1./81.*m46*RSp453e*
         RSe563p + 1./81.*m46*RSp453e*RSe453p - 1./81.*m46*RSp453e*
         RSp563e + 1./162.*m46*pow(RSp453e,2) + 1./81.*m46*RS145pe*
         RSp563e - 1./81.*m46*RS145pe*RSp453e - 1./81.*m46*RS156pe*
         RSp563e + 1./81.*m46*RS156pe*RSp453e - 1./81.*m46*RSe561p*
         RSp563e + 1./81.*m46*RSe561p*RSp453e + 1./81.*m46*RSe451p*
         RSp563e - 1./81.*m46*RSe451p*RSp453e + 1./81.*m46*RSp561e*
         RS345pe - 1./81.*m46*RSp561e*RS356pe - 1./81.*m46*RSp561e*
         RSe563p + 1./81.*m46*RSp561e*RSe453p - 1./81.*m46*RSp561e*
         RSp563e + 1./81.*m46*RSp561e*RSp453e - 1./81.*m46*RSp561e*
         RS145pe + 1./81.*m46*RSp561e*RS156pe + 1./81.*m46*RSp561e*
         RSe561p - 1./81.*m46*RSp561e*RSe451p + 1./162.*m46*pow(
         RSp561e,2) - 1./81.*m46*RSp451e*RS345pe + 1./81.*m46*RSp451e*
         RS356pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./81.*m46*RSp451e*
         RSe563p - 1./81.*m46*RSp451e*RSe453p + 1./81.*m46*RSp451e*
         RSp563e - 1./81.*m46*RSp451e*RSp453e + 1./81.*m46*RSp451e*
         RS145pe - 1./81.*m46*RSp451e*RS156pe - 1./81.*m46*RSp451e*
         RSe561p + 1./81.*m46*RSp451e*RSe451p - 1./81.*m46*RSp451e*
         RSp561e + 1./162.*m46*pow(RSp451e,2) - 1./162.*m44*RSp563e*
         RS345pe - 1./54.*m44*RSp563e*RS356pe - 1./54.*m44*RSp563e*
         RSe563p - 1./162.*m44*RSp563e*RSe453p - 1./108.*m44*pow(
         RSp563e,2) - 1./162.*m44*RSp453e*RS345pe - 1./162.*m44*RSp453e
         *RS356pe - 1./162.*m44*RSp453e*RSe563p - 1./162.*m44*RSp453e*
         RSe453p - 1./162.*m44*RSp453e*RSp563e - 1./324.*m44*pow(
         RSp453e,2) + 1./162.*m44*RS145pe*RSp563e + 1./162.*m44*RS145pe
         *RSp453e + 1./54.*m44*RS156pe*RSp563e + 1./162.*m44*RS156pe*
         RSp453e + 1./54.*m44*RSe561p*RSp563e + 1./162.*m44*RSe561p*
         RSp453e + 1./162.*m44*RSe451p*RSp563e + 1./162.*m44*RSe451p*
         RSp453e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./162.*m44*RSp561e*
         RS345pe + 1./54.*m44*RSp561e*RS356pe + 1./54.*m44*RSp561e*
         RSe563p + 1./162.*m44*RSp561e*RSe453p + 1./54.*m44*RSp561e*
         RSp563e + 1./162.*m44*RSp561e*RSp453e - 1./162.*m44*RSp561e*
         RS145pe - 1./54.*m44*RSp561e*RS156pe - 1./54.*m44*RSp561e*
         RSe561p - 1./162.*m44*RSp561e*RSe451p - 1./108.*m44*pow(
         RSp561e,2) + 1./162.*m44*RSp451e*RS345pe + 1./162.*m44*RSp451e
         *RS356pe + 1./162.*m44*RSp451e*RSe563p + 1./162.*m44*RSp451e*
         RSe453p + 1./162.*m44*RSp451e*RSp563e + 1./162.*m44*RSp451e*
         RSp453e - 1./162.*m44*RSp451e*RS145pe - 1./162.*m44*RSp451e*
         RS156pe - 1./162.*m44*RSp451e*RSe561p - 1./162.*m44*RSp451e*
         RSe451p - 1./162.*m44*RSp451e*RSp561e - 1./324.*m44*pow(
         RSp451e,2) + 1./54.*m33*RSp563e*RS345pe - 1./54.*m33*RSp563e*
         RS356pe - 1./54.*m33*RSp563e*RSe563p + 1./54.*m33*RSp563e*
         RSe453p - 1./108.*m33*pow(RSp563e,2) - 1./54.*m33*RSp453e*
         RS345pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./54.*m33*RSp453e*
         RS356pe + 1./54.*m33*RSp453e*RSe563p - 1./54.*m33*RSp453e*
         RSe453p + 1./54.*m33*RSp453e*RSp563e - 1./108.*m33*pow(
         RSp453e,2) + 1./162.*m33*RS145pe*RSp563e - 1./162.*m33*RS145pe
         *RSp453e - 1./162.*m33*RS156pe*RSp563e + 1./162.*m33*RS156pe*
         RSp453e - 1./162.*m33*RSe561p*RSp563e + 1./162.*m33*RSe561p*
         RSp453e + 1./162.*m33*RSe451p*RSp563e - 1./162.*m33*RSe451p*
         RSp453e + 1./162.*m33*RSp561e*RS345pe - 1./162.*m33*RSp561e*
         RS356pe - 1./162.*m33*RSp561e*RSe563p + 1./162.*m33*RSp561e*
         RSe453p - 1./162.*m33*RSp561e*RSp563e + 1./162.*m33*RSp561e*
         RSp453e + 1./162.*m33*RSp561e*RS145pe - 1./162.*m33*RSp561e*
         RS156pe - 1./162.*m33*RSp561e*RSe561p + 1./162.*m33*RSp561e*
         RSe451p - 1./324.*m33*pow(RSp561e,2) - 1./162.*m33*RSp451e*
         RS345pe + 1./162.*m33*RSp451e*RS356pe + 1./162.*m33*RSp451e*
         RSe563p - 1./162.*m33*RSp451e*RSe453p + 1./162.*m33*RSp451e*
         RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./162.*m33*
         RSp451e*RSp453e - 1./162.*m33*RSp451e*RS145pe + 1./162.*m33*
         RSp451e*RS156pe + 1./162.*m33*RSp451e*RSe561p - 1./162.*m33*
         RSp451e*RSe451p + 1./162.*m33*RSp451e*RSp561e - 1./324.*m33*
         pow(RSp451e,2) + 1./81.*m33*mp2*RN3p*RSp563e*RS345pe - 1./81.*
         m33*mp2*RN3p*RSp563e*RS356pe - 1./81.*m33*mp2*RN3p*RSp453e*
         RS345pe + 1./81.*m33*mp2*RN3p*RSp453e*RS356pe - 1./81.*m33*mp2
         *RN3p*RSp561e*RS345pe + 1./81.*m33*mp2*RN3p*RSp561e*RS356pe + 
         1./81.*m33*mp2*RN3p*RSp451e*RS345pe - 1./81.*m33*mp2*RN3p*
         RSp451e*RS356pe - 1./324.*m33*m66*RN3p*RSp563e*RS345pe - 1./
         324.*m33*m66*RN3p*RSp563e*RS356pe - 1./108.*m33*m66*RN3p*
         RSp453e*RS345pe - 1./324.*m33*m66*RN3p*RSp453e*RS356pe - 1./
         108.*m33*m66*RN3p*RSp561e*RS345pe - 1./108.*m33*m66*RN3p*
         RSp561e*RS356pe - 1./36.*m33*m66*RN3p*RSp451e*RS345pe - 1./108.
         *m33*m66*RN3p*RSp451e*RS356pe - 1./324.*m33*m66*RN1p*RS145pe*
         RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./108.*m33*m66*
         RN1p*RS145pe*RSp453e - 1./324.*m33*m66*RN1p*RS156pe*RSp563e - 
         1./324.*m33*m66*RN1p*RS156pe*RSp453e - 1./324.*m33*m66*RN1p*
         RSp561e*RS145pe - 1./324.*m33*m66*RN1p*RSp561e*RS156pe - 1./
         108.*m33*m66*RN1p*RSp451e*RS145pe - 1./324.*m33*m66*RN1p*
         RSp451e*RS156pe - 1./162.*m33*m46*RN3p*RSp563e*RS345pe + 1./
         162.*m33*m46*RN3p*RSp563e*RS356pe + 1./162.*m33*m46*RN3p*
         RSp453e*RS345pe - 1./162.*m33*m46*RN3p*RSp453e*RS356pe - 1./54.
         *m33*m46*RN3p*RSp561e*RS345pe + 1./54.*m33*m46*RN3p*RSp561e*
         RS356pe + 1./54.*m33*m46*RN3p*RSp451e*RS345pe - 1./54.*m33*m46
         *RN3p*RSp451e*RS356pe - 1./162.*m33*m46*RN1p*RS145pe*RSp563e
          + 1./162.*m33*m46*RN1p*RS145pe*RSp453e + 1./162.*m33*m46*RN1p
         *RS156pe*RSp563e - 1./162.*m33*m46*RN1p*RS156pe*RSp453e - 1./
         162.*m33*m46*RN1p*RSp561e*RS145pe + 1./162.*m33*m46*RN1p*
         RSp561e*RS156pe + 1./162.*m33*m46*RN1p*RSp451e*RS145pe - 1./
         162.*m33*m46*RN1p*RSp451e*RS156pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./324.*m33*m44*
         RN3p*RSp563e*RS345pe - 1./108.*m33*m44*RN3p*RSp563e*RS356pe - 
         1./324.*m33*m44*RN3p*RSp453e*RS345pe - 1./324.*m33*m44*RN3p*
         RSp453e*RS356pe - 1./108.*m33*m44*RN3p*RSp561e*RS345pe - 1./36.
         *m33*m44*RN3p*RSp561e*RS356pe - 1./108.*m33*m44*RN3p*RSp451e*
         RS345pe - 1./108.*m33*m44*RN3p*RSp451e*RS356pe - 1./324.*m33*
         m44*RN1p*RS145pe*RSp563e - 1./324.*m33*m44*RN1p*RS145pe*
         RSp453e - 1./108.*m33*m44*RN1p*RS156pe*RSp563e - 1./324.*m33*
         m44*RN1p*RS156pe*RSp453e - 1./324.*m33*m44*RN1p*RSp561e*
         RS145pe - 1./108.*m33*m44*RN1p*RSp561e*RS156pe - 1./324.*m33*
         m44*RN1p*RSp451e*RS145pe - 1./324.*m33*m44*RN1p*RSp451e*
         RS156pe - 1./54.*pow(m33,2)*RN3p*RSp563e*RS345pe + 1./54.*pow(
         m33,2)*RN3p*RSp563e*RS356pe + 1./54.*pow(m33,2)*RN3p*RSp453e*
         RS345pe - 1./54.*pow(m33,2)*RN3p*RSp453e*RS356pe - 1./162.*
         pow(m33,2)*RN3p*RSp561e*RS345pe + 1./162.*pow(m33,2)*RN3p*
         RSp561e*RS356pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./162.*pow(m33,2)*
         RN3p*RSp451e*RS345pe - 1./162.*pow(m33,2)*RN3p*RSp451e*RS356pe
          - 1./81.*m13*RSp563e*RS345pe + 1./81.*m13*RSp563e*RS356pe + 1.
         /81.*m13*RSp563e*RSe563p - 1./81.*m13*RSp563e*RSe453p + 1./162.
         *m13*pow(RSp563e,2) + 1./81.*m13*RSp453e*RS345pe - 1./81.*m13*
         RSp453e*RS356pe - 1./81.*m13*RSp453e*RSe563p + 1./81.*m13*
         RSp453e*RSe453p - 1./81.*m13*RSp453e*RSp563e + 1./162.*m13*
         pow(RSp453e,2) + 1./81.*m13*RS145pe*RSp563e - 1./81.*m13*
         RS145pe*RSp453e - 1./81.*m13*RS156pe*RSp563e + 1./81.*m13*
         RS156pe*RSp453e - 1./81.*m13*RSe561p*RSp563e + 1./81.*m13*
         RSe561p*RSp453e + 1./81.*m13*RSe451p*RSp563e - 1./81.*m13*
         RSe451p*RSp453e + 1./81.*m13*RSp561e*RS345pe - 1./81.*m13*
         RSp561e*RS356pe - 1./81.*m13*RSp561e*RSe563p + 1./81.*m13*
         RSp561e*RSe453p - 1./81.*m13*RSp561e*RSp563e + 1./81.*m13*
         RSp561e*RSp453e - 1./81.*m13*RSp561e*RS145pe + 1./81.*m13*
         RSp561e*RS156pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./81.*m13*RSp561e*
         RSe561p - 1./81.*m13*RSp561e*RSe451p + 1./162.*m13*pow(
         RSp561e,2) - 1./81.*m13*RSp451e*RS345pe + 1./81.*m13*RSp451e*
         RS356pe + 1./81.*m13*RSp451e*RSe563p - 1./81.*m13*RSp451e*
         RSe453p + 1./81.*m13*RSp451e*RSp563e - 1./81.*m13*RSp451e*
         RSp453e + 1./81.*m13*RSp451e*RS145pe - 1./81.*m13*RSp451e*
         RS156pe - 1./81.*m13*RSp451e*RSe561p + 1./81.*m13*RSp451e*
         RSe451p - 1./81.*m13*RSp451e*RSp561e + 1./162.*m13*pow(
         RSp451e,2) + 1./162.*m13*m66*RN3p*RSp563e*RS345pe + 1./162.*
         m13*m66*RN3p*RSp563e*RS356pe + 1./54.*m13*m66*RN3p*RSp453e*
         RS345pe + 1./162.*m13*m66*RN3p*RSp453e*RS356pe - 1./162.*m13*
         m66*RN3p*RSp561e*RS345pe - 1./162.*m13*m66*RN3p*RSp561e*
         RS356pe - 1./54.*m13*m66*RN3p*RSp451e*RS345pe - 1./162.*m13*
         m66*RN3p*RSp451e*RS356pe - 1./162.*m13*m66*RN1p*RS145pe*
         RSp563e - 1./54.*m13*m66*RN1p*RS145pe*RSp453e - 1./162.*m13*
         m66*RN1p*RS156pe*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./162.*m13*m66*
         RN1p*RS156pe*RSp453e + 1./162.*m13*m66*RN1p*RSp561e*RS145pe + 
         1./162.*m13*m66*RN1p*RSp561e*RS156pe + 1./54.*m13*m66*RN1p*
         RSp451e*RS145pe + 1./162.*m13*m66*RN1p*RSp451e*RS156pe + 1./81.
         *m13*m46*RN3p*RSp563e*RS345pe - 1./81.*m13*m46*RN3p*RSp563e*
         RS356pe - 1./81.*m13*m46*RN3p*RSp453e*RS345pe + 1./81.*m13*m46
         *RN3p*RSp453e*RS356pe - 1./81.*m13*m46*RN3p*RSp561e*RS345pe + 
         1./81.*m13*m46*RN3p*RSp561e*RS356pe + 1./81.*m13*m46*RN3p*
         RSp451e*RS345pe - 1./81.*m13*m46*RN3p*RSp451e*RS356pe - 1./81.
         *m13*m46*RN1p*RS145pe*RSp563e + 1./81.*m13*m46*RN1p*RS145pe*
         RSp453e + 1./81.*m13*m46*RN1p*RS156pe*RSp563e - 1./81.*m13*m46
         *RN1p*RS156pe*RSp453e + 1./81.*m13*m46*RN1p*RSp561e*RS145pe - 
         1./81.*m13*m46*RN1p*RSp561e*RS156pe - 1./81.*m13*m46*RN1p*
         RSp451e*RS145pe + 1./81.*m13*m46*RN1p*RSp451e*RS156pe + 1./162.
         *m13*m44*RN3p*RSp563e*RS345pe + 1./54.*m13*m44*RN3p*RSp563e*
         RS356pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./162.*m13*m44*RN3p*
         RSp453e*RS345pe + 1./162.*m13*m44*RN3p*RSp453e*RS356pe - 1./
         162.*m13*m44*RN3p*RSp561e*RS345pe - 1./54.*m13*m44*RN3p*
         RSp561e*RS356pe - 1./162.*m13*m44*RN3p*RSp451e*RS345pe - 1./
         162.*m13*m44*RN3p*RSp451e*RS356pe - 1./162.*m13*m44*RN1p*
         RS145pe*RSp563e - 1./162.*m13*m44*RN1p*RS145pe*RSp453e - 1./54.
         *m13*m44*RN1p*RS156pe*RSp563e - 1./162.*m13*m44*RN1p*RS156pe*
         RSp453e + 1./162.*m13*m44*RN1p*RSp561e*RS145pe + 1./54.*m13*
         m44*RN1p*RSp561e*RS156pe + 1./162.*m13*m44*RN1p*RSp451e*
         RS145pe + 1./162.*m13*m44*RN1p*RSp451e*RS156pe + 1./81.*m13*
         m33*RN3p*RSp563e*RS345pe - 1./81.*m13*m33*RN3p*RSp563e*RS356pe
          - 1./81.*m13*m33*RN3p*RSp453e*RS345pe + 1./81.*m13*m33*RN3p*
         RSp453e*RS356pe - 1./81.*m13*m33*RN3p*RSp561e*RS345pe + 1./81.
         *m13*m33*RN3p*RSp561e*RS356pe + 1./81.*m13*m33*RN3p*RSp451e*
         RS345pe - 1./81.*m13*m33*RN3p*RSp451e*RS356pe + 1./162.*m11*
         RSp563e*RS345pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./162.*m11*
         RSp563e*RS356pe - 1./162.*m11*RSp563e*RSe563p + 1./162.*m11*
         RSp563e*RSe453p - 1./324.*m11*pow(RSp563e,2) - 1./162.*m11*
         RSp453e*RS345pe + 1./162.*m11*RSp453e*RS356pe + 1./162.*m11*
         RSp453e*RSe563p - 1./162.*m11*RSp453e*RSe453p + 1./162.*m11*
         RSp453e*RSp563e - 1./324.*m11*pow(RSp453e,2) + 1./162.*m11*
         RS145pe*RSp563e - 1./162.*m11*RS145pe*RSp453e - 1./162.*m11*
         RS156pe*RSp563e + 1./162.*m11*RS156pe*RSp453e - 1./162.*m11*
         RSe561p*RSp563e + 1./162.*m11*RSe561p*RSp453e + 1./162.*m11*
         RSe451p*RSp563e - 1./162.*m11*RSe451p*RSp453e + 1./162.*m11*
         RSp561e*RS345pe - 1./162.*m11*RSp561e*RS356pe - 1./162.*m11*
         RSp561e*RSe563p + 1./162.*m11*RSp561e*RSe453p - 1./162.*m11*
         RSp561e*RSp563e + 1./162.*m11*RSp561e*RSp453e + 1./54.*m11*
         RSp561e*RS145pe - 1./54.*m11*RSp561e*RS156pe - 1./54.*m11*
         RSp561e*RSe561p + 1./54.*m11*RSp561e*RSe451p - 1./108.*m11*
         pow(RSp561e,2) );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./162.*m11*
         RSp451e*RS345pe + 1./162.*m11*RSp451e*RS356pe + 1./162.*m11*
         RSp451e*RSe563p - 1./162.*m11*RSp451e*RSe453p + 1./162.*m11*
         RSp451e*RSp563e - 1./162.*m11*RSp451e*RSp453e - 1./54.*m11*
         RSp451e*RS145pe + 1./54.*m11*RSp451e*RS156pe + 1./54.*m11*
         RSp451e*RSe561p - 1./54.*m11*RSp451e*RSe451p + 1./54.*m11*
         RSp451e*RSp561e - 1./108.*m11*pow(RSp451e,2) - 1./81.*m11*mp2*
         RN1p*RS145pe*RSp563e + 1./81.*m11*mp2*RN1p*RS145pe*RSp453e + 1.
         /81.*m11*mp2*RN1p*RS156pe*RSp563e - 1./81.*m11*mp2*RN1p*
         RS156pe*RSp453e + 1./81.*m11*mp2*RN1p*RSp561e*RS145pe - 1./81.
         *m11*mp2*RN1p*RSp561e*RS156pe - 1./81.*m11*mp2*RN1p*RSp451e*
         RS145pe + 1./81.*m11*mp2*RN1p*RSp451e*RS156pe - 1./324.*m11*
         m66*RN3p*RSp563e*RS345pe - 1./324.*m11*m66*RN3p*RSp563e*
         RS356pe - 1./108.*m11*m66*RN3p*RSp453e*RS345pe - 1./324.*m11*
         m66*RN3p*RSp453e*RS356pe - 1./324.*m11*m66*RN3p*RSp561e*
         RS345pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./324.*m11*m66*
         RN3p*RSp561e*RS356pe - 1./108.*m11*m66*RN3p*RSp451e*RS345pe - 
         1./324.*m11*m66*RN3p*RSp451e*RS356pe - 1./108.*m11*m66*RN1p*
         RS145pe*RSp563e - 1./36.*m11*m66*RN1p*RS145pe*RSp453e - 1./108.
         *m11*m66*RN1p*RS156pe*RSp563e - 1./108.*m11*m66*RN1p*RS156pe*
         RSp453e - 1./324.*m11*m66*RN1p*RSp561e*RS145pe - 1./324.*m11*
         m66*RN1p*RSp561e*RS156pe - 1./108.*m11*m66*RN1p*RSp451e*
         RS145pe - 1./324.*m11*m66*RN1p*RSp451e*RS156pe - 1./162.*m11*
         m46*RN3p*RSp563e*RS345pe + 1./162.*m11*m46*RN3p*RSp563e*
         RS356pe + 1./162.*m11*m46*RN3p*RSp453e*RS345pe - 1./162.*m11*
         m46*RN3p*RSp453e*RS356pe - 1./162.*m11*m46*RN3p*RSp561e*
         RS345pe + 1./162.*m11*m46*RN3p*RSp561e*RS356pe + 1./162.*m11*
         m46*RN3p*RSp451e*RS345pe - 1./162.*m11*m46*RN3p*RSp451e*
         RS356pe - 1./54.*m11*m46*RN1p*RS145pe*RSp563e + 1./54.*m11*m46
         *RN1p*RS145pe*RSp453e + 1./54.*m11*m46*RN1p*RS156pe*RSp563e - 
         1./54.*m11*m46*RN1p*RS156pe*RSp453e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./162.*m11*m46*
         RN1p*RSp561e*RS145pe + 1./162.*m11*m46*RN1p*RSp561e*RS156pe + 
         1./162.*m11*m46*RN1p*RSp451e*RS145pe - 1./162.*m11*m46*RN1p*
         RSp451e*RS156pe - 1./324.*m11*m44*RN3p*RSp563e*RS345pe - 1./
         108.*m11*m44*RN3p*RSp563e*RS356pe - 1./324.*m11*m44*RN3p*
         RSp453e*RS345pe - 1./324.*m11*m44*RN3p*RSp453e*RS356pe - 1./
         324.*m11*m44*RN3p*RSp561e*RS345pe - 1./108.*m11*m44*RN3p*
         RSp561e*RS356pe - 1./324.*m11*m44*RN3p*RSp451e*RS345pe - 1./
         324.*m11*m44*RN3p*RSp451e*RS356pe - 1./108.*m11*m44*RN1p*
         RS145pe*RSp563e - 1./108.*m11*m44*RN1p*RS145pe*RSp453e - 1./36.
         *m11*m44*RN1p*RS156pe*RSp563e - 1./108.*m11*m44*RN1p*RS156pe*
         RSp453e - 1./324.*m11*m44*RN1p*RSp561e*RS145pe - 1./108.*m11*
         m44*RN1p*RSp561e*RS156pe - 1./324.*m11*m44*RN1p*RSp451e*
         RS145pe - 1./324.*m11*m44*RN1p*RSp451e*RS156pe - 1./162.*m11*
         m33*RN3p*RSp563e*RS345pe + 1./162.*m11*m33*RN3p*RSp563e*
         RS356pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./162.*m11*m33*RN3p*
         RSp453e*RS345pe - 1./162.*m11*m33*RN3p*RSp453e*RS356pe - 1./
         162.*m11*m33*RN3p*RSp561e*RS345pe + 1./162.*m11*m33*RN3p*
         RSp561e*RS356pe + 1./162.*m11*m33*RN3p*RSp451e*RS345pe - 1./
         162.*m11*m33*RN3p*RSp451e*RS356pe - 1./162.*m11*m33*RN1p*
         RS145pe*RSp563e + 1./162.*m11*m33*RN1p*RS145pe*RSp453e + 1./
         162.*m11*m33*RN1p*RS156pe*RSp563e - 1./162.*m11*m33*RN1p*
         RS156pe*RSp453e - 1./162.*m11*m33*RN1p*RSp561e*RS145pe + 1./
         162.*m11*m33*RN1p*RSp561e*RS156pe + 1./162.*m11*m33*RN1p*
         RSp451e*RS145pe - 1./162.*m11*m33*RN1p*RSp451e*RS156pe - 1./81.
         *m11*m13*RN1p*RS145pe*RSp563e + 1./81.*m11*m13*RN1p*RS145pe*
         RSp453e + 1./81.*m11*m13*RN1p*RS156pe*RSp563e - 1./81.*m11*m13
         *RN1p*RS156pe*RSp453e + 1./81.*m11*m13*RN1p*RSp561e*RS145pe - 
         1./81.*m11*m13*RN1p*RSp561e*RS156pe - 1./81.*m11*m13*RN1p*
         RSp451e*RS145pe + 1./81.*m11*m13*RN1p*RSp451e*RS156pe - 1./162.
         *pow(m11,2)*RN1p*RS145pe*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./162.*pow(m11,2)*
         RN1p*RS145pe*RSp453e + 1./162.*pow(m11,2)*RN1p*RS156pe*RSp563e
          - 1./162.*pow(m11,2)*RN1p*RS156pe*RSp453e - 1./54.*pow(m11,2)
         *RN1p*RSp561e*RS145pe + 1./54.*pow(m11,2)*RN1p*RSp561e*RS156pe
          + 1./54.*pow(m11,2)*RN1p*RSp451e*RS145pe - 1./54.*pow(m11,2)*
         RN1p*RSp451e*RS156pe + 1./81.*RN(mp2,me2)*mp2*me2*RSp563e*
         RSe563p - 1./81.*RN(mp2,me2)*mp2*me2*RSp563e*RSe453p - 1./81.*
         RN(mp2,me2)*mp2*me2*RSp453e*RSe563p + 1./81.*RN(mp2,me2)*mp2*
         me2*RSp453e*RSe453p - 1./81.*RN(mp2,me2)*mp2*me2*RSe561p*
         RSp563e + 1./81.*RN(mp2,me2)*mp2*me2*RSe561p*RSp453e + 1./81.*
         RN(mp2,me2)*mp2*me2*RSe451p*RSp563e - 1./81.*RN(mp2,me2)*mp2*
         me2*RSe451p*RSp453e - 1./81.*RN(mp2,me2)*mp2*me2*RSp561e*
         RSe563p + 1./81.*RN(mp2,me2)*mp2*me2*RSp561e*RSe453p + 1./81.*
         RN(mp2,me2)*mp2*me2*RSp561e*RSe561p - 1./81.*RN(mp2,me2)*mp2*
         me2*RSp561e*RSe451p + 1./81.*RN(mp2,me2)*mp2*me2*RSp451e*
         RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./81.*RN(mp2,me2)
         *mp2*me2*RSp451e*RSe453p - 1./81.*RN(mp2,me2)*mp2*me2*RSp451e*
         RSe561p + 1./81.*RN(mp2,me2)*mp2*me2*RSp451e*RSe451p - 1./162.
         *RN(mp2,me2)*m66*me2*RSp563e*RSe563p - 1./162.*RN(mp2,me2)*m66
         *me2*RSp563e*RSe453p - 1./162.*RN(mp2,me2)*m66*me2*RSp453e*
         RSe563p - 1./54.*RN(mp2,me2)*m66*me2*RSp453e*RSe453p + 1./162.
         *RN(mp2,me2)*m66*me2*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m66
         *me2*RSe561p*RSp453e + 1./162.*RN(mp2,me2)*m66*me2*RSe451p*
         RSp563e + 1./54.*RN(mp2,me2)*m66*me2*RSe451p*RSp453e + 1./162.
         *RN(mp2,me2)*m66*me2*RSp561e*RSe563p + 1./162.*RN(mp2,me2)*m66
         *me2*RSp561e*RSe453p - 1./162.*RN(mp2,me2)*m66*me2*RSp561e*
         RSe561p - 1./162.*RN(mp2,me2)*m66*me2*RSp561e*RSe451p + 1./162.
         *RN(mp2,me2)*m66*me2*RSp451e*RSe563p + 1./54.*RN(mp2,me2)*m66*
         me2*RSp451e*RSe453p - 1./162.*RN(mp2,me2)*m66*me2*RSp451e*
         RSe561p - 1./54.*RN(mp2,me2)*m66*me2*RSp451e*RSe451p + 1./81.*
         RN(mp2,me2)*m46*me2*RSp563e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./81.*RN(mp2,me2)
         *m46*me2*RSp563e*RSe453p - 1./81.*RN(mp2,me2)*m46*me2*RSp453e*
         RSe563p + 1./81.*RN(mp2,me2)*m46*me2*RSp453e*RSe453p - 1./81.*
         RN(mp2,me2)*m46*me2*RSe561p*RSp563e + 1./81.*RN(mp2,me2)*m46*
         me2*RSe561p*RSp453e + 1./81.*RN(mp2,me2)*m46*me2*RSe451p*
         RSp563e - 1./81.*RN(mp2,me2)*m46*me2*RSe451p*RSp453e - 1./81.*
         RN(mp2,me2)*m46*me2*RSp561e*RSe563p + 1./81.*RN(mp2,me2)*m46*
         me2*RSp561e*RSe453p + 1./81.*RN(mp2,me2)*m46*me2*RSp561e*
         RSe561p - 1./81.*RN(mp2,me2)*m46*me2*RSp561e*RSe451p + 1./81.*
         RN(mp2,me2)*m46*me2*RSp451e*RSe563p - 1./81.*RN(mp2,me2)*m46*
         me2*RSp451e*RSe453p - 1./81.*RN(mp2,me2)*m46*me2*RSp451e*
         RSe561p + 1./81.*RN(mp2,me2)*m46*me2*RSp451e*RSe451p - 1./54.*
         RN(mp2,me2)*m44*me2*RSp563e*RSe563p - 1./162.*RN(mp2,me2)*m44*
         me2*RSp563e*RSe453p - 1./162.*RN(mp2,me2)*m44*me2*RSp453e*
         RSe563p - 1./162.*RN(mp2,me2)*m44*me2*RSp453e*RSe453p + 1./54.
         *RN(mp2,me2)*m44*me2*RSe561p*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./162.*RN(mp2,me2)*
         m44*me2*RSe561p*RSp453e + 1./162.*RN(mp2,me2)*m44*me2*RSe451p*
         RSp563e + 1./162.*RN(mp2,me2)*m44*me2*RSe451p*RSp453e + 1./54.
         *RN(mp2,me2)*m44*me2*RSp561e*RSe563p + 1./162.*RN(mp2,me2)*m44
         *me2*RSp561e*RSe453p - 1./54.*RN(mp2,me2)*m44*me2*RSp561e*
         RSe561p - 1./162.*RN(mp2,me2)*m44*me2*RSp561e*RSe451p + 1./162.
         *RN(mp2,me2)*m44*me2*RSp451e*RSe563p + 1./162.*RN(mp2,me2)*m44
         *me2*RSp451e*RSe453p - 1./162.*RN(mp2,me2)*m44*me2*RSp451e*
         RSe561p - 1./162.*RN(mp2,me2)*m44*me2*RSp451e*RSe451p - 1./54.
         *RN(mp2,me2)*m33*me2*RSp563e*RSe563p + 1./54.*RN(mp2,me2)*m33*
         me2*RSp563e*RSe453p + 1./54.*RN(mp2,me2)*m33*me2*RSp453e*
         RSe563p - 1./54.*RN(mp2,me2)*m33*me2*RSp453e*RSe453p - 1./162.
         *RN(mp2,me2)*m33*me2*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m33
         *me2*RSe561p*RSp453e + 1./162.*RN(mp2,me2)*m33*me2*RSe451p*
         RSp563e - 1./162.*RN(mp2,me2)*m33*me2*RSe451p*RSp453e - 1./162.
         *RN(mp2,me2)*m33*me2*RSp561e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./162.*RN(mp2,me2)*
         m33*me2*RSp561e*RSe453p - 1./162.*RN(mp2,me2)*m33*me2*RSp561e*
         RSe561p + 1./162.*RN(mp2,me2)*m33*me2*RSp561e*RSe451p + 1./162.
         *RN(mp2,me2)*m33*me2*RSp451e*RSe563p - 1./162.*RN(mp2,me2)*m33
         *me2*RSp451e*RSe453p + 1./162.*RN(mp2,me2)*m33*me2*RSp451e*
         RSe561p - 1./162.*RN(mp2,me2)*m33*me2*RSp451e*RSe451p + 1./108.
         *RN(mp2,me2)*m33*m66*RSp563e*RSe563p + 1./108.*RN(mp2,me2)*m33
         *m66*RSp563e*RSe453p + 1./108.*RN(mp2,me2)*m33*m66*RSp453e*
         RSe563p + 1./36.*RN(mp2,me2)*m33*m66*RSp453e*RSe453p + 1./324.
         *RN(mp2,me2)*m33*m66*RSe561p*RSp563e + 1./324.*RN(mp2,me2)*m33
         *m66*RSe561p*RSp453e + 1./324.*RN(mp2,me2)*m33*m66*RSe451p*
         RSp563e + 1./108.*RN(mp2,me2)*m33*m66*RSe451p*RSp453e + 1./324.
         *RN(mp2,me2)*m33*m66*RSp561e*RSe563p + 1./324.*RN(mp2,me2)*m33
         *m66*RSp561e*RSe453p + 1./324.*RN(mp2,me2)*m33*m66*RSp561e*
         RSe561p + 1./324.*RN(mp2,me2)*m33*m66*RSp561e*RSe451p + 1./324.
         *RN(mp2,me2)*m33*m66*RSp451e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./108.*RN(mp2,me2)*
         m33*m66*RSp451e*RSe453p + 1./324.*RN(mp2,me2)*m33*m66*RSp451e*
         RSe561p + 1./108.*RN(mp2,me2)*m33*m66*RSp451e*RSe451p - 1./54.
         *RN(mp2,me2)*m33*m46*RSp563e*RSe563p + 1./54.*RN(mp2,me2)*m33*
         m46*RSp563e*RSe453p + 1./54.*RN(mp2,me2)*m33*m46*RSp453e*
         RSe563p - 1./54.*RN(mp2,me2)*m33*m46*RSp453e*RSe453p - 1./162.
         *RN(mp2,me2)*m33*m46*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m33
         *m46*RSe561p*RSp453e + 1./162.*RN(mp2,me2)*m33*m46*RSe451p*
         RSp563e - 1./162.*RN(mp2,me2)*m33*m46*RSe451p*RSp453e - 1./162.
         *RN(mp2,me2)*m33*m46*RSp561e*RSe563p + 1./162.*RN(mp2,me2)*m33
         *m46*RSp561e*RSe453p - 1./162.*RN(mp2,me2)*m33*m46*RSp561e*
         RSe561p + 1./162.*RN(mp2,me2)*m33*m46*RSp561e*RSe451p + 1./162.
         *RN(mp2,me2)*m33*m46*RSp451e*RSe563p - 1./162.*RN(mp2,me2)*m33
         *m46*RSp451e*RSe453p + 1./162.*RN(mp2,me2)*m33*m46*RSp451e*
         RSe561p - 1./162.*RN(mp2,me2)*m33*m46*RSp451e*RSe451p + 1./36.
         *RN(mp2,me2)*m33*m44*RSp563e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./108.*RN(mp2,me2)*
         m33*m44*RSp563e*RSe453p + 1./108.*RN(mp2,me2)*m33*m44*RSp453e*
         RSe563p + 1./108.*RN(mp2,me2)*m33*m44*RSp453e*RSe453p + 1./108.
         *RN(mp2,me2)*m33*m44*RSe561p*RSp563e + 1./324.*RN(mp2,me2)*m33
         *m44*RSe561p*RSp453e + 1./324.*RN(mp2,me2)*m33*m44*RSe451p*
         RSp563e + 1./324.*RN(mp2,me2)*m33*m44*RSe451p*RSp453e + 1./108.
         *RN(mp2,me2)*m33*m44*RSp561e*RSe563p + 1./324.*RN(mp2,me2)*m33
         *m44*RSp561e*RSe453p + 1./108.*RN(mp2,me2)*m33*m44*RSp561e*
         RSe561p + 1./324.*RN(mp2,me2)*m33*m44*RSp561e*RSe451p + 1./324.
         *RN(mp2,me2)*m33*m44*RSp451e*RSe563p + 1./324.*RN(mp2,me2)*m33
         *m44*RSp451e*RSe453p + 1./324.*RN(mp2,me2)*m33*m44*RSp451e*
         RSe561p + 1./324.*RN(mp2,me2)*m33*m44*RSp451e*RSe451p + 1./81.
         *RN(mp2,me2)*m13*me2*RSp563e*RSe563p - 1./81.*RN(mp2,me2)*m13*
         me2*RSp563e*RSe453p - 1./81.*RN(mp2,me2)*m13*me2*RSp453e*
         RSe563p + 1./81.*RN(mp2,me2)*m13*me2*RSp453e*RSe453p - 1./81.*
         RN(mp2,me2)*m13*me2*RSe561p*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./81.*RN(mp2,me2)*
         m13*me2*RSe561p*RSp453e + 1./81.*RN(mp2,me2)*m13*me2*RSe451p*
         RSp563e - 1./81.*RN(mp2,me2)*m13*me2*RSe451p*RSp453e - 1./81.*
         RN(mp2,me2)*m13*me2*RSp561e*RSe563p + 1./81.*RN(mp2,me2)*m13*
         me2*RSp561e*RSe453p + 1./81.*RN(mp2,me2)*m13*me2*RSp561e*
         RSe561p - 1./81.*RN(mp2,me2)*m13*me2*RSp561e*RSe451p + 1./81.*
         RN(mp2,me2)*m13*me2*RSp451e*RSe563p - 1./81.*RN(mp2,me2)*m13*
         me2*RSp451e*RSe453p - 1./81.*RN(mp2,me2)*m13*me2*RSp451e*
         RSe561p + 1./81.*RN(mp2,me2)*m13*me2*RSp451e*RSe451p - 1./162.
         *RN(mp2,me2)*m13*m66*RSp563e*RSe563p - 1./162.*RN(mp2,me2)*m13
         *m66*RSp563e*RSe453p - 1./162.*RN(mp2,me2)*m13*m66*RSp453e*
         RSe563p - 1./54.*RN(mp2,me2)*m13*m66*RSp453e*RSe453p + 1./162.
         *RN(mp2,me2)*m13*m66*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m13
         *m66*RSe561p*RSp453e + 1./162.*RN(mp2,me2)*m13*m66*RSe451p*
         RSp563e + 1./54.*RN(mp2,me2)*m13*m66*RSe451p*RSp453e + 1./162.
         *RN(mp2,me2)*m13*m66*RSp561e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./162.*RN(mp2,me2)*
         m13*m66*RSp561e*RSe453p - 1./162.*RN(mp2,me2)*m13*m66*RSp561e*
         RSe561p - 1./162.*RN(mp2,me2)*m13*m66*RSp561e*RSe451p + 1./162.
         *RN(mp2,me2)*m13*m66*RSp451e*RSe563p + 1./54.*RN(mp2,me2)*m13*
         m66*RSp451e*RSe453p - 1./162.*RN(mp2,me2)*m13*m66*RSp451e*
         RSe561p - 1./54.*RN(mp2,me2)*m13*m66*RSp451e*RSe451p + 1./81.*
         RN(mp2,me2)*m13*m46*RSp563e*RSe563p - 1./81.*RN(mp2,me2)*m13*
         m46*RSp563e*RSe453p - 1./81.*RN(mp2,me2)*m13*m46*RSp453e*
         RSe563p + 1./81.*RN(mp2,me2)*m13*m46*RSp453e*RSe453p - 1./81.*
         RN(mp2,me2)*m13*m46*RSe561p*RSp563e + 1./81.*RN(mp2,me2)*m13*
         m46*RSe561p*RSp453e + 1./81.*RN(mp2,me2)*m13*m46*RSe451p*
         RSp563e - 1./81.*RN(mp2,me2)*m13*m46*RSe451p*RSp453e - 1./81.*
         RN(mp2,me2)*m13*m46*RSp561e*RSe563p + 1./81.*RN(mp2,me2)*m13*
         m46*RSp561e*RSe453p + 1./81.*RN(mp2,me2)*m13*m46*RSp561e*
         RSe561p - 1./81.*RN(mp2,me2)*m13*m46*RSp561e*RSe451p + 1./81.*
         RN(mp2,me2)*m13*m46*RSp451e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./81.*RN(mp2,me2)
         *m13*m46*RSp451e*RSe453p - 1./81.*RN(mp2,me2)*m13*m46*RSp451e*
         RSe561p + 1./81.*RN(mp2,me2)*m13*m46*RSp451e*RSe451p - 1./54.*
         RN(mp2,me2)*m13*m44*RSp563e*RSe563p - 1./162.*RN(mp2,me2)*m13*
         m44*RSp563e*RSe453p - 1./162.*RN(mp2,me2)*m13*m44*RSp453e*
         RSe563p - 1./162.*RN(mp2,me2)*m13*m44*RSp453e*RSe453p + 1./54.
         *RN(mp2,me2)*m13*m44*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m13
         *m44*RSe561p*RSp453e + 1./162.*RN(mp2,me2)*m13*m44*RSe451p*
         RSp563e + 1./162.*RN(mp2,me2)*m13*m44*RSe451p*RSp453e + 1./54.
         *RN(mp2,me2)*m13*m44*RSp561e*RSe563p + 1./162.*RN(mp2,me2)*m13
         *m44*RSp561e*RSe453p - 1./54.*RN(mp2,me2)*m13*m44*RSp561e*
         RSe561p - 1./162.*RN(mp2,me2)*m13*m44*RSp561e*RSe451p + 1./162.
         *RN(mp2,me2)*m13*m44*RSp451e*RSe563p + 1./162.*RN(mp2,me2)*m13
         *m44*RSp451e*RSe453p - 1./162.*RN(mp2,me2)*m13*m44*RSp451e*
         RSe561p - 1./162.*RN(mp2,me2)*m13*m44*RSp451e*RSe451p - 1./162.
         *RN(mp2,me2)*m11*me2*RSp563e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./162.*RN(mp2,me2)*
         m11*me2*RSp563e*RSe453p + 1./162.*RN(mp2,me2)*m11*me2*RSp453e*
         RSe563p - 1./162.*RN(mp2,me2)*m11*me2*RSp453e*RSe453p - 1./162.
         *RN(mp2,me2)*m11*me2*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m11
         *me2*RSe561p*RSp453e + 1./162.*RN(mp2,me2)*m11*me2*RSe451p*
         RSp563e - 1./162.*RN(mp2,me2)*m11*me2*RSe451p*RSp453e - 1./162.
         *RN(mp2,me2)*m11*me2*RSp561e*RSe563p + 1./162.*RN(mp2,me2)*m11
         *me2*RSp561e*RSe453p - 1./54.*RN(mp2,me2)*m11*me2*RSp561e*
         RSe561p + 1./54.*RN(mp2,me2)*m11*me2*RSp561e*RSe451p + 1./162.
         *RN(mp2,me2)*m11*me2*RSp451e*RSe563p - 1./162.*RN(mp2,me2)*m11
         *me2*RSp451e*RSe453p + 1./54.*RN(mp2,me2)*m11*me2*RSp451e*
         RSe561p - 1./54.*RN(mp2,me2)*m11*me2*RSp451e*RSe451p + 1./324.
         *RN(mp2,me2)*m11*m66*RSp563e*RSe563p + 1./324.*RN(mp2,me2)*m11
         *m66*RSp563e*RSe453p + 1./324.*RN(mp2,me2)*m11*m66*RSp453e*
         RSe563p + 1./108.*RN(mp2,me2)*m11*m66*RSp453e*RSe453p + 1./324.
         *RN(mp2,me2)*m11*m66*RSe561p*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./324.*RN(mp2,me2)*
         m11*m66*RSe561p*RSp453e + 1./324.*RN(mp2,me2)*m11*m66*RSe451p*
         RSp563e + 1./108.*RN(mp2,me2)*m11*m66*RSe451p*RSp453e + 1./324.
         *RN(mp2,me2)*m11*m66*RSp561e*RSe563p + 1./324.*RN(mp2,me2)*m11
         *m66*RSp561e*RSe453p + 1./108.*RN(mp2,me2)*m11*m66*RSp561e*
         RSe561p + 1./108.*RN(mp2,me2)*m11*m66*RSp561e*RSe451p + 1./324.
         *RN(mp2,me2)*m11*m66*RSp451e*RSe563p + 1./108.*RN(mp2,me2)*m11
         *m66*RSp451e*RSe453p + 1./108.*RN(mp2,me2)*m11*m66*RSp451e*
         RSe561p + 1./36.*RN(mp2,me2)*m11*m66*RSp451e*RSe451p - 1./162.
         *RN(mp2,me2)*m11*m46*RSp563e*RSe563p + 1./162.*RN(mp2,me2)*m11
         *m46*RSp563e*RSe453p + 1./162.*RN(mp2,me2)*m11*m46*RSp453e*
         RSe563p - 1./162.*RN(mp2,me2)*m11*m46*RSp453e*RSe453p - 1./162.
         *RN(mp2,me2)*m11*m46*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m11
         *m46*RSe561p*RSp453e + 1./162.*RN(mp2,me2)*m11*m46*RSe451p*
         RSp563e - 1./162.*RN(mp2,me2)*m11*m46*RSe451p*RSp453e - 1./162.
         *RN(mp2,me2)*m11*m46*RSp561e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./162.*RN(mp2,me2)*
         m11*m46*RSp561e*RSe453p - 1./54.*RN(mp2,me2)*m11*m46*RSp561e*
         RSe561p + 1./54.*RN(mp2,me2)*m11*m46*RSp561e*RSe451p + 1./162.
         *RN(mp2,me2)*m11*m46*RSp451e*RSe563p - 1./162.*RN(mp2,me2)*m11
         *m46*RSp451e*RSe453p + 1./54.*RN(mp2,me2)*m11*m46*RSp451e*
         RSe561p - 1./54.*RN(mp2,me2)*m11*m46*RSp451e*RSe451p + 1./108.
         *RN(mp2,me2)*m11*m44*RSp563e*RSe563p + 1./324.*RN(mp2,me2)*m11
         *m44*RSp563e*RSe453p + 1./324.*RN(mp2,me2)*m11*m44*RSp453e*
         RSe563p + 1./324.*RN(mp2,me2)*m11*m44*RSp453e*RSe453p + 1./108.
         *RN(mp2,me2)*m11*m44*RSe561p*RSp563e + 1./324.*RN(mp2,me2)*m11
         *m44*RSe561p*RSp453e + 1./324.*RN(mp2,me2)*m11*m44*RSe451p*
         RSp563e + 1./324.*RN(mp2,me2)*m11*m44*RSe451p*RSp453e + 1./108.
         *RN(mp2,me2)*m11*m44*RSp561e*RSe563p + 1./324.*RN(mp2,me2)*m11
         *m44*RSp561e*RSe453p + 1./36.*RN(mp2,me2)*m11*m44*RSp561e*
         RSe561p + 1./108.*RN(mp2,me2)*m11*m44*RSp561e*RSe451p + 1./324.
         *RN(mp2,me2)*m11*m44*RSp451e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./324.*RN(mp2,me2)*
         m11*m44*RSp451e*RSe453p + 1./108.*RN(mp2,me2)*m11*m44*RSp451e*
         RSe561p + 1./108.*RN(mp2,me2)*m11*m44*RSp451e*RSe451p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m55,L) * (  - 1./27.*m55*RSp463e
         *RS346pe - 1./27.*m55*RSp463e*RSe463p - 1./54.*m55*pow(
         RSp463e,2) + 1./27.*m55*RS146pe*RSp463e + 1./81.*m55*RS546pe*
         RSp463e*RS346pe + 1./81.*m55*RS546pe*RSp463e*RSe463p + 1./162.
         *m55*RS546pe*pow(RSp463e,2) - 1./81.*m55*RS546pe*RS146pe*
         RSp463e + 1./27.*m55*RSe461p*RSp463e - 1./81.*m55*RSe461p*
         RS546pe*RSp463e + 1./27.*m55*RSp461e*RS346pe + 1./27.*m55*
         RSp461e*RSe463p + 1./27.*m55*RSp461e*RSp463e - 1./27.*m55*
         RSp461e*RS146pe - 1./81.*m55*RSp461e*RS546pe*RS346pe - 1./81.*
         m55*RSp461e*RS546pe*RSe463p - 1./81.*m55*RSp461e*RS546pe*
         RSp463e + 1./81.*m55*RSp461e*RS546pe*RS146pe - 1./27.*m55*
         RSp461e*RSe461p + 1./81.*m55*RSp461e*RSe461p*RS546pe - 1./54.*
         m55*pow(RSp461e,2) + 1./162.*m55*pow(RSp461e,2)*RS546pe - 1./
         54.*m33*m55*RN3p*RSp463e*RS346pe + 1./162.*m33*m55*RN3p*
         RS546pe*RSp463e*RS346pe - 1./18.*m33*m55*RN3p*RSp461e*RS346pe
          + 1./54.*m33*m55*RN3p*RSp461e*RS546pe*RS346pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m55,L) * (  - 1./54.*m33*m55*
         RN1p*RS146pe*RSp463e + 1./162.*m33*m55*RN1p*RS546pe*RS146pe*
         RSp463e - 1./54.*m33*m55*RN1p*RSp461e*RS146pe + 1./162.*m33*
         m55*RN1p*RSp461e*RS546pe*RS146pe + 1./27.*m13*m55*RN3p*RSp463e
         *RS346pe - 1./81.*m13*m55*RN3p*RS546pe*RSp463e*RS346pe - 1./27.
         *m13*m55*RN3p*RSp461e*RS346pe + 1./81.*m13*m55*RN3p*RSp461e*
         RS546pe*RS346pe - 1./27.*m13*m55*RN1p*RS146pe*RSp463e + 1./81.
         *m13*m55*RN1p*RS546pe*RS146pe*RSp463e + 1./27.*m13*m55*RN1p*
         RSp461e*RS146pe - 1./81.*m13*m55*RN1p*RSp461e*RS546pe*RS146pe
          - 1./54.*m11*m55*RN3p*RSp463e*RS346pe + 1./162.*m11*m55*RN3p*
         RS546pe*RSp463e*RS346pe - 1./54.*m11*m55*RN3p*RSp461e*RS346pe
          + 1./162.*m11*m55*RN3p*RSp461e*RS546pe*RS346pe - 1./18.*m11*
         m55*RN1p*RS146pe*RSp463e + 1./54.*m11*m55*RN1p*RS546pe*RS146pe
         *RSp463e - 1./54.*m11*m55*RN1p*RSp461e*RS146pe + 1./162.*m11*
         m55*RN1p*RSp461e*RS546pe*RS146pe - 1./27.*RN(mp2,me2)*m55*me2*
         RSp463e*RSe463p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m55,L) * ( 1./81.*RN(mp2,me2)*
         m55*me2*RS546pe*RSp463e*RSe463p + 1./27.*RN(mp2,me2)*m55*me2*
         RSe461p*RSp463e - 1./81.*RN(mp2,me2)*m55*me2*RSe461p*RS546pe*
         RSp463e + 1./27.*RN(mp2,me2)*m55*me2*RSp461e*RSe463p - 1./81.*
         RN(mp2,me2)*m55*me2*RSp461e*RS546pe*RSe463p - 1./27.*RN(mp2,
         me2)*m55*me2*RSp461e*RSe461p + 1./81.*RN(mp2,me2)*m55*me2*
         RSp461e*RSe461p*RS546pe + 1./18.*RN(mp2,me2)*m33*m55*RSp463e*
         RSe463p - 1./54.*RN(mp2,me2)*m33*m55*RS546pe*RSp463e*RSe463p
          + 1./54.*RN(mp2,me2)*m33*m55*RSe461p*RSp463e - 1./162.*RN(mp2
         ,me2)*m33*m55*RSe461p*RS546pe*RSp463e + 1./54.*RN(mp2,me2)*m33
         *m55*RSp461e*RSe463p - 1./162.*RN(mp2,me2)*m33*m55*RSp461e*
         RS546pe*RSe463p + 1./54.*RN(mp2,me2)*m33*m55*RSp461e*RSe461p
          - 1./162.*RN(mp2,me2)*m33*m55*RSp461e*RSe461p*RS546pe - 1./27.
         *RN(mp2,me2)*m13*m55*RSp463e*RSe463p + 1./81.*RN(mp2,me2)*m13*
         m55*RS546pe*RSp463e*RSe463p + 1./27.*RN(mp2,me2)*m13*m55*
         RSe461p*RSp463e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m55,L) * (  - 1./81.*RN(mp2,me2)
         *m13*m55*RSe461p*RS546pe*RSp463e + 1./27.*RN(mp2,me2)*m13*m55*
         RSp461e*RSe463p - 1./81.*RN(mp2,me2)*m13*m55*RSp461e*RS546pe*
         RSe463p - 1./27.*RN(mp2,me2)*m13*m55*RSp461e*RSe461p + 1./81.*
         RN(mp2,me2)*m13*m55*RSp461e*RSe461p*RS546pe + 1./54.*RN(mp2,
         me2)*m11*m55*RSp463e*RSe463p - 1./162.*RN(mp2,me2)*m11*m55*
         RS546pe*RSp463e*RSe463p + 1./54.*RN(mp2,me2)*m11*m55*RSe461p*
         RSp463e - 1./162.*RN(mp2,me2)*m11*m55*RSe461p*RS546pe*RSp463e
          + 1./54.*RN(mp2,me2)*m11*m55*RSp461e*RSe463p - 1./162.*RN(mp2
         ,me2)*m11*m55*RSp461e*RS546pe*RSe463p + 1./18.*RN(mp2,me2)*m11
         *m55*RSp461e*RSe461p - 1./54.*RN(mp2,me2)*m11*m55*RSp461e*
         RSe461p*RS546pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./81.*mp2*RSp463e
         *RS345pe + 1./81.*mp2*RSp463e*RS346pe + 1./81.*mp2*RSp463e*
         RSe463p - 1./81.*mp2*RSp463e*RSe453p + 1./81.*mp2*pow(
         RSp463e,2) + 1./81.*mp2*RSp453e*RS345pe - 1./81.*mp2*RSp453e*
         RS346pe - 1./81.*mp2*RSp453e*RSe463p + 1./81.*mp2*RSp453e*
         RSe453p - 2./81.*mp2*RSp453e*RSp463e + 1./81.*mp2*pow(
         RSp453e,2) + 1./81.*mp2*RS145pe*RSp463e - 1./81.*mp2*RS145pe*
         RSp453e - 1./81.*mp2*RS146pe*RSp463e + 1./81.*mp2*RS146pe*
         RSp453e - 1./81.*mp2*RSe461p*RSp463e + 1./81.*mp2*RSe461p*
         RSp453e + 1./81.*mp2*RSe451p*RSp463e - 1./81.*mp2*RSe451p*
         RSp453e + 1./81.*mp2*RSp461e*RS345pe - 1./81.*mp2*RSp461e*
         RS346pe - 1./81.*mp2*RSp461e*RSe463p + 1./81.*mp2*RSp461e*
         RSe453p - 2./81.*mp2*RSp461e*RSp463e + 2./81.*mp2*RSp461e*
         RSp453e - 1./81.*mp2*RSp461e*RS145pe + 1./81.*mp2*RSp461e*
         RS146pe + 1./81.*mp2*RSp461e*RSe461p - 1./81.*mp2*RSp461e*
         RSe451p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./81.*mp2*pow(
         RSp461e,2) - 1./81.*mp2*RSp451e*RS345pe + 1./81.*mp2*RSp451e*
         RS346pe + 1./81.*mp2*RSp451e*RSe463p - 1./81.*mp2*RSp451e*
         RSe453p + 2./81.*mp2*RSp451e*RSp463e - 2./81.*mp2*RSp451e*
         RSp453e + 1./81.*mp2*RSp451e*RS145pe - 1./81.*mp2*RSp451e*
         RS146pe - 1./81.*mp2*RSp451e*RSe461p + 1./81.*mp2*RSp451e*
         RSe451p - 2./81.*mp2*RSp451e*RSp461e + 1./81.*mp2*pow(
         RSp451e,2) - 1./162.*m66*RSp463e*RS345pe - 1./162.*m66*RSp463e
         *RS346pe - 1./162.*m66*RSp463e*RSe463p - 1./162.*m66*RSp463e*
         RSe453p - 1./324.*m66*pow(RSp463e,2) - 1./54.*m66*RSp453e*
         RS345pe - 1./162.*m66*RSp453e*RS346pe - 1./162.*m66*RSp453e*
         RSe463p - 1./54.*m66*RSp453e*RSe453p - 1./162.*m66*RSp453e*
         RSp463e - 1./108.*m66*pow(RSp453e,2) + 1./162.*m66*RS145pe*
         RSp463e + 1./54.*m66*RS145pe*RSp453e + 1./162.*m66*RS146pe*
         RSp463e + 1./162.*m66*RS146pe*RSp453e + 1./162.*m66*RSe461p*
         RSp463e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./162.*m66*RSe461p*
         RSp453e + 1./162.*m66*RSe451p*RSp463e + 1./54.*m66*RSe451p*
         RSp453e + 1./162.*m66*RSp461e*RS345pe + 1./162.*m66*RSp461e*
         RS346pe + 1./162.*m66*RSp461e*RSe463p + 1./162.*m66*RSp461e*
         RSe453p + 1./162.*m66*RSp461e*RSp463e + 1./162.*m66*RSp461e*
         RSp453e - 1./162.*m66*RSp461e*RS145pe - 1./162.*m66*RSp461e*
         RS146pe - 1./162.*m66*RSp461e*RSe461p - 1./162.*m66*RSp461e*
         RSe451p - 1./324.*m66*pow(RSp461e,2) + 1./54.*m66*RSp451e*
         RS345pe + 1./162.*m66*RSp451e*RS346pe + 1./162.*m66*RSp451e*
         RSe463p + 1./54.*m66*RSp451e*RSe453p + 1./162.*m66*RSp451e*
         RSp463e + 1./54.*m66*RSp451e*RSp453e - 1./54.*m66*RSp451e*
         RS145pe - 1./162.*m66*RSp451e*RS146pe - 1./162.*m66*RSp451e*
         RSe461p - 1./54.*m66*RSp451e*RSe451p - 1./162.*m66*RSp451e*
         RSp461e - 1./108.*m66*pow(RSp451e,2) - 1./81.*m56*RSp463e*
         RS345pe + 1./81.*m56*RSp463e*RS346pe + 1./81.*m56*RSp463e*
         RSe463p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./81.*m56*RSp463e
         *RSe453p + 1./162.*m56*pow(RSp463e,2) + 1./81.*m56*RSp453e*
         RS345pe - 1./81.*m56*RSp453e*RS346pe - 1./81.*m56*RSp453e*
         RSe463p + 1./81.*m56*RSp453e*RSe453p - 1./81.*m56*RSp453e*
         RSp463e + 1./162.*m56*pow(RSp453e,2) + 1./81.*m56*RS145pe*
         RSp463e - 1./81.*m56*RS145pe*RSp453e - 1./81.*m56*RS146pe*
         RSp463e + 1./81.*m56*RS146pe*RSp453e - 1./81.*m56*RSe461p*
         RSp463e + 1./81.*m56*RSe461p*RSp453e + 1./81.*m56*RSe451p*
         RSp463e - 1./81.*m56*RSe451p*RSp453e + 1./81.*m56*RSp461e*
         RS345pe - 1./81.*m56*RSp461e*RS346pe - 1./81.*m56*RSp461e*
         RSe463p + 1./81.*m56*RSp461e*RSe453p - 1./81.*m56*RSp461e*
         RSp463e + 1./81.*m56*RSp461e*RSp453e - 1./81.*m56*RSp461e*
         RS145pe + 1./81.*m56*RSp461e*RS146pe + 1./81.*m56*RSp461e*
         RSe461p - 1./81.*m56*RSp461e*RSe451p + 1./162.*m56*pow(
         RSp461e,2) - 1./81.*m56*RSp451e*RS345pe + 1./81.*m56*RSp451e*
         RS346pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./81.*m56*RSp451e*
         RSe463p - 1./81.*m56*RSp451e*RSe453p + 1./81.*m56*RSp451e*
         RSp463e - 1./81.*m56*RSp451e*RSp453e + 1./81.*m56*RSp451e*
         RS145pe - 1./81.*m56*RSp451e*RS146pe - 1./81.*m56*RSp451e*
         RSe461p + 1./81.*m56*RSp451e*RSe451p - 1./81.*m56*RSp451e*
         RSp461e + 1./162.*m56*pow(RSp451e,2) - 1./162.*m55*RSp463e*
         RS345pe - 1./54.*m55*RSp463e*RS346pe - 1./54.*m55*RSp463e*
         RSe463p - 1./162.*m55*RSp463e*RSe453p - 1./108.*m55*pow(
         RSp463e,2) - 1./162.*m55*RSp453e*RS345pe - 1./162.*m55*RSp453e
         *RS346pe - 1./162.*m55*RSp453e*RSe463p - 1./162.*m55*RSp453e*
         RSe453p - 1./162.*m55*RSp453e*RSp463e - 1./324.*m55*pow(
         RSp453e,2) + 1./162.*m55*RS145pe*RSp463e + 1./162.*m55*RS145pe
         *RSp453e + 1./54.*m55*RS146pe*RSp463e + 1./162.*m55*RS146pe*
         RSp453e + 1./54.*m55*RSe461p*RSp463e + 1./162.*m55*RSe461p*
         RSp453e + 1./162.*m55*RSe451p*RSp463e + 1./162.*m55*RSe451p*
         RSp453e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./162.*m55*RSp461e*
         RS345pe + 1./54.*m55*RSp461e*RS346pe + 1./54.*m55*RSp461e*
         RSe463p + 1./162.*m55*RSp461e*RSe453p + 1./54.*m55*RSp461e*
         RSp463e + 1./162.*m55*RSp461e*RSp453e - 1./162.*m55*RSp461e*
         RS145pe - 1./54.*m55*RSp461e*RS146pe - 1./54.*m55*RSp461e*
         RSe461p - 1./162.*m55*RSp461e*RSe451p - 1./108.*m55*pow(
         RSp461e,2) + 1./162.*m55*RSp451e*RS345pe + 1./162.*m55*RSp451e
         *RS346pe + 1./162.*m55*RSp451e*RSe463p + 1./162.*m55*RSp451e*
         RSe453p + 1./162.*m55*RSp451e*RSp463e + 1./162.*m55*RSp451e*
         RSp453e - 1./162.*m55*RSp451e*RS145pe - 1./162.*m55*RSp451e*
         RS146pe - 1./162.*m55*RSp451e*RSe461p - 1./162.*m55*RSp451e*
         RSe451p - 1./162.*m55*RSp451e*RSp461e - 1./324.*m55*pow(
         RSp451e,2) + 1./54.*m33*RSp463e*RS345pe - 1./54.*m33*RSp463e*
         RS346pe - 1./54.*m33*RSp463e*RSe463p + 1./54.*m33*RSp463e*
         RSe453p - 1./108.*m33*pow(RSp463e,2) - 1./54.*m33*RSp453e*
         RS345pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./54.*m33*RSp453e*
         RS346pe + 1./54.*m33*RSp453e*RSe463p - 1./54.*m33*RSp453e*
         RSe453p + 1./54.*m33*RSp453e*RSp463e - 1./108.*m33*pow(
         RSp453e,2) + 1./162.*m33*RS145pe*RSp463e - 1./162.*m33*RS145pe
         *RSp453e - 1./162.*m33*RS146pe*RSp463e + 1./162.*m33*RS146pe*
         RSp453e - 1./162.*m33*RSe461p*RSp463e + 1./162.*m33*RSe461p*
         RSp453e + 1./162.*m33*RSe451p*RSp463e - 1./162.*m33*RSe451p*
         RSp453e + 1./162.*m33*RSp461e*RS345pe - 1./162.*m33*RSp461e*
         RS346pe - 1./162.*m33*RSp461e*RSe463p + 1./162.*m33*RSp461e*
         RSe453p - 1./162.*m33*RSp461e*RSp463e + 1./162.*m33*RSp461e*
         RSp453e + 1./162.*m33*RSp461e*RS145pe - 1./162.*m33*RSp461e*
         RS146pe - 1./162.*m33*RSp461e*RSe461p + 1./162.*m33*RSp461e*
         RSe451p - 1./324.*m33*pow(RSp461e,2) - 1./162.*m33*RSp451e*
         RS345pe + 1./162.*m33*RSp451e*RS346pe + 1./162.*m33*RSp451e*
         RSe463p - 1./162.*m33*RSp451e*RSe453p + 1./162.*m33*RSp451e*
         RSp463e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./162.*m33*
         RSp451e*RSp453e - 1./162.*m33*RSp451e*RS145pe + 1./162.*m33*
         RSp451e*RS146pe + 1./162.*m33*RSp451e*RSe461p - 1./162.*m33*
         RSp451e*RSe451p + 1./162.*m33*RSp451e*RSp461e - 1./324.*m33*
         pow(RSp451e,2) + 1./81.*m33*mp2*RN3p*RSp463e*RS345pe - 1./81.*
         m33*mp2*RN3p*RSp463e*RS346pe - 1./81.*m33*mp2*RN3p*RSp453e*
         RS345pe + 1./81.*m33*mp2*RN3p*RSp453e*RS346pe - 1./81.*m33*mp2
         *RN3p*RSp461e*RS345pe + 1./81.*m33*mp2*RN3p*RSp461e*RS346pe + 
         1./81.*m33*mp2*RN3p*RSp451e*RS345pe - 1./81.*m33*mp2*RN3p*
         RSp451e*RS346pe - 1./324.*m33*m66*RN3p*RSp463e*RS345pe - 1./
         324.*m33*m66*RN3p*RSp463e*RS346pe - 1./108.*m33*m66*RN3p*
         RSp453e*RS345pe - 1./324.*m33*m66*RN3p*RSp453e*RS346pe - 1./
         108.*m33*m66*RN3p*RSp461e*RS345pe - 1./108.*m33*m66*RN3p*
         RSp461e*RS346pe - 1./36.*m33*m66*RN3p*RSp451e*RS345pe - 1./108.
         *m33*m66*RN3p*RSp451e*RS346pe - 1./324.*m33*m66*RN1p*RS145pe*
         RSp463e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./108.*m33*m66*
         RN1p*RS145pe*RSp453e - 1./324.*m33*m66*RN1p*RS146pe*RSp463e - 
         1./324.*m33*m66*RN1p*RS146pe*RSp453e - 1./324.*m33*m66*RN1p*
         RSp461e*RS145pe - 1./324.*m33*m66*RN1p*RSp461e*RS146pe - 1./
         108.*m33*m66*RN1p*RSp451e*RS145pe - 1./324.*m33*m66*RN1p*
         RSp451e*RS146pe - 1./162.*m33*m56*RN3p*RSp463e*RS345pe + 1./
         162.*m33*m56*RN3p*RSp463e*RS346pe + 1./162.*m33*m56*RN3p*
         RSp453e*RS345pe - 1./162.*m33*m56*RN3p*RSp453e*RS346pe - 1./54.
         *m33*m56*RN3p*RSp461e*RS345pe + 1./54.*m33*m56*RN3p*RSp461e*
         RS346pe + 1./54.*m33*m56*RN3p*RSp451e*RS345pe - 1./54.*m33*m56
         *RN3p*RSp451e*RS346pe - 1./162.*m33*m56*RN1p*RS145pe*RSp463e
          + 1./162.*m33*m56*RN1p*RS145pe*RSp453e + 1./162.*m33*m56*RN1p
         *RS146pe*RSp463e - 1./162.*m33*m56*RN1p*RS146pe*RSp453e - 1./
         162.*m33*m56*RN1p*RSp461e*RS145pe + 1./162.*m33*m56*RN1p*
         RSp461e*RS146pe + 1./162.*m33*m56*RN1p*RSp451e*RS145pe - 1./
         162.*m33*m56*RN1p*RSp451e*RS146pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./324.*m33*m55*
         RN3p*RSp463e*RS345pe - 1./108.*m33*m55*RN3p*RSp463e*RS346pe - 
         1./324.*m33*m55*RN3p*RSp453e*RS345pe - 1./324.*m33*m55*RN3p*
         RSp453e*RS346pe - 1./108.*m33*m55*RN3p*RSp461e*RS345pe - 1./36.
         *m33*m55*RN3p*RSp461e*RS346pe - 1./108.*m33*m55*RN3p*RSp451e*
         RS345pe - 1./108.*m33*m55*RN3p*RSp451e*RS346pe - 1./324.*m33*
         m55*RN1p*RS145pe*RSp463e - 1./324.*m33*m55*RN1p*RS145pe*
         RSp453e - 1./108.*m33*m55*RN1p*RS146pe*RSp463e - 1./324.*m33*
         m55*RN1p*RS146pe*RSp453e - 1./324.*m33*m55*RN1p*RSp461e*
         RS145pe - 1./108.*m33*m55*RN1p*RSp461e*RS146pe - 1./324.*m33*
         m55*RN1p*RSp451e*RS145pe - 1./324.*m33*m55*RN1p*RSp451e*
         RS146pe - 1./54.*pow(m33,2)*RN3p*RSp463e*RS345pe + 1./54.*pow(
         m33,2)*RN3p*RSp463e*RS346pe + 1./54.*pow(m33,2)*RN3p*RSp453e*
         RS345pe - 1./54.*pow(m33,2)*RN3p*RSp453e*RS346pe - 1./162.*
         pow(m33,2)*RN3p*RSp461e*RS345pe + 1./162.*pow(m33,2)*RN3p*
         RSp461e*RS346pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./162.*pow(m33,2)*
         RN3p*RSp451e*RS345pe - 1./162.*pow(m33,2)*RN3p*RSp451e*RS346pe
          - 1./81.*m13*RSp463e*RS345pe + 1./81.*m13*RSp463e*RS346pe + 1.
         /81.*m13*RSp463e*RSe463p - 1./81.*m13*RSp463e*RSe453p + 1./162.
         *m13*pow(RSp463e,2) + 1./81.*m13*RSp453e*RS345pe - 1./81.*m13*
         RSp453e*RS346pe - 1./81.*m13*RSp453e*RSe463p + 1./81.*m13*
         RSp453e*RSe453p - 1./81.*m13*RSp453e*RSp463e + 1./162.*m13*
         pow(RSp453e,2) + 1./81.*m13*RS145pe*RSp463e - 1./81.*m13*
         RS145pe*RSp453e - 1./81.*m13*RS146pe*RSp463e + 1./81.*m13*
         RS146pe*RSp453e - 1./81.*m13*RSe461p*RSp463e + 1./81.*m13*
         RSe461p*RSp453e + 1./81.*m13*RSe451p*RSp463e - 1./81.*m13*
         RSe451p*RSp453e + 1./81.*m13*RSp461e*RS345pe - 1./81.*m13*
         RSp461e*RS346pe - 1./81.*m13*RSp461e*RSe463p + 1./81.*m13*
         RSp461e*RSe453p - 1./81.*m13*RSp461e*RSp463e + 1./81.*m13*
         RSp461e*RSp453e - 1./81.*m13*RSp461e*RS145pe + 1./81.*m13*
         RSp461e*RS146pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./81.*m13*RSp461e*
         RSe461p - 1./81.*m13*RSp461e*RSe451p + 1./162.*m13*pow(
         RSp461e,2) - 1./81.*m13*RSp451e*RS345pe + 1./81.*m13*RSp451e*
         RS346pe + 1./81.*m13*RSp451e*RSe463p - 1./81.*m13*RSp451e*
         RSe453p + 1./81.*m13*RSp451e*RSp463e - 1./81.*m13*RSp451e*
         RSp453e + 1./81.*m13*RSp451e*RS145pe - 1./81.*m13*RSp451e*
         RS146pe - 1./81.*m13*RSp451e*RSe461p + 1./81.*m13*RSp451e*
         RSe451p - 1./81.*m13*RSp451e*RSp461e + 1./162.*m13*pow(
         RSp451e,2) + 1./162.*m13*m66*RN3p*RSp463e*RS345pe + 1./162.*
         m13*m66*RN3p*RSp463e*RS346pe + 1./54.*m13*m66*RN3p*RSp453e*
         RS345pe + 1./162.*m13*m66*RN3p*RSp453e*RS346pe - 1./162.*m13*
         m66*RN3p*RSp461e*RS345pe - 1./162.*m13*m66*RN3p*RSp461e*
         RS346pe - 1./54.*m13*m66*RN3p*RSp451e*RS345pe - 1./162.*m13*
         m66*RN3p*RSp451e*RS346pe - 1./162.*m13*m66*RN1p*RS145pe*
         RSp463e - 1./54.*m13*m66*RN1p*RS145pe*RSp453e - 1./162.*m13*
         m66*RN1p*RS146pe*RSp463e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./162.*m13*m66*
         RN1p*RS146pe*RSp453e + 1./162.*m13*m66*RN1p*RSp461e*RS145pe + 
         1./162.*m13*m66*RN1p*RSp461e*RS146pe + 1./54.*m13*m66*RN1p*
         RSp451e*RS145pe + 1./162.*m13*m66*RN1p*RSp451e*RS146pe + 1./81.
         *m13*m56*RN3p*RSp463e*RS345pe - 1./81.*m13*m56*RN3p*RSp463e*
         RS346pe - 1./81.*m13*m56*RN3p*RSp453e*RS345pe + 1./81.*m13*m56
         *RN3p*RSp453e*RS346pe - 1./81.*m13*m56*RN3p*RSp461e*RS345pe + 
         1./81.*m13*m56*RN3p*RSp461e*RS346pe + 1./81.*m13*m56*RN3p*
         RSp451e*RS345pe - 1./81.*m13*m56*RN3p*RSp451e*RS346pe - 1./81.
         *m13*m56*RN1p*RS145pe*RSp463e + 1./81.*m13*m56*RN1p*RS145pe*
         RSp453e + 1./81.*m13*m56*RN1p*RS146pe*RSp463e - 1./81.*m13*m56
         *RN1p*RS146pe*RSp453e + 1./81.*m13*m56*RN1p*RSp461e*RS145pe - 
         1./81.*m13*m56*RN1p*RSp461e*RS146pe - 1./81.*m13*m56*RN1p*
         RSp451e*RS145pe + 1./81.*m13*m56*RN1p*RSp451e*RS146pe + 1./162.
         *m13*m55*RN3p*RSp463e*RS345pe + 1./54.*m13*m55*RN3p*RSp463e*
         RS346pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./162.*m13*m55*RN3p*
         RSp453e*RS345pe + 1./162.*m13*m55*RN3p*RSp453e*RS346pe - 1./
         162.*m13*m55*RN3p*RSp461e*RS345pe - 1./54.*m13*m55*RN3p*
         RSp461e*RS346pe - 1./162.*m13*m55*RN3p*RSp451e*RS345pe - 1./
         162.*m13*m55*RN3p*RSp451e*RS346pe - 1./162.*m13*m55*RN1p*
         RS145pe*RSp463e - 1./162.*m13*m55*RN1p*RS145pe*RSp453e - 1./54.
         *m13*m55*RN1p*RS146pe*RSp463e - 1./162.*m13*m55*RN1p*RS146pe*
         RSp453e + 1./162.*m13*m55*RN1p*RSp461e*RS145pe + 1./54.*m13*
         m55*RN1p*RSp461e*RS146pe + 1./162.*m13*m55*RN1p*RSp451e*
         RS145pe + 1./162.*m13*m55*RN1p*RSp451e*RS146pe + 1./81.*m13*
         m33*RN3p*RSp463e*RS345pe - 1./81.*m13*m33*RN3p*RSp463e*RS346pe
          - 1./81.*m13*m33*RN3p*RSp453e*RS345pe + 1./81.*m13*m33*RN3p*
         RSp453e*RS346pe - 1./81.*m13*m33*RN3p*RSp461e*RS345pe + 1./81.
         *m13*m33*RN3p*RSp461e*RS346pe + 1./81.*m13*m33*RN3p*RSp451e*
         RS345pe - 1./81.*m13*m33*RN3p*RSp451e*RS346pe + 1./162.*m11*
         RSp463e*RS345pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./162.*m11*
         RSp463e*RS346pe - 1./162.*m11*RSp463e*RSe463p + 1./162.*m11*
         RSp463e*RSe453p - 1./324.*m11*pow(RSp463e,2) - 1./162.*m11*
         RSp453e*RS345pe + 1./162.*m11*RSp453e*RS346pe + 1./162.*m11*
         RSp453e*RSe463p - 1./162.*m11*RSp453e*RSe453p + 1./162.*m11*
         RSp453e*RSp463e - 1./324.*m11*pow(RSp453e,2) + 1./162.*m11*
         RS145pe*RSp463e - 1./162.*m11*RS145pe*RSp453e - 1./162.*m11*
         RS146pe*RSp463e + 1./162.*m11*RS146pe*RSp453e - 1./162.*m11*
         RSe461p*RSp463e + 1./162.*m11*RSe461p*RSp453e + 1./162.*m11*
         RSe451p*RSp463e - 1./162.*m11*RSe451p*RSp453e + 1./162.*m11*
         RSp461e*RS345pe - 1./162.*m11*RSp461e*RS346pe - 1./162.*m11*
         RSp461e*RSe463p + 1./162.*m11*RSp461e*RSe453p - 1./162.*m11*
         RSp461e*RSp463e + 1./162.*m11*RSp461e*RSp453e + 1./54.*m11*
         RSp461e*RS145pe - 1./54.*m11*RSp461e*RS146pe - 1./54.*m11*
         RSp461e*RSe461p + 1./54.*m11*RSp461e*RSe451p - 1./108.*m11*
         pow(RSp461e,2) );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./162.*m11*
         RSp451e*RS345pe + 1./162.*m11*RSp451e*RS346pe + 1./162.*m11*
         RSp451e*RSe463p - 1./162.*m11*RSp451e*RSe453p + 1./162.*m11*
         RSp451e*RSp463e - 1./162.*m11*RSp451e*RSp453e - 1./54.*m11*
         RSp451e*RS145pe + 1./54.*m11*RSp451e*RS146pe + 1./54.*m11*
         RSp451e*RSe461p - 1./54.*m11*RSp451e*RSe451p + 1./54.*m11*
         RSp451e*RSp461e - 1./108.*m11*pow(RSp451e,2) - 1./81.*m11*mp2*
         RN1p*RS145pe*RSp463e + 1./81.*m11*mp2*RN1p*RS145pe*RSp453e + 1.
         /81.*m11*mp2*RN1p*RS146pe*RSp463e - 1./81.*m11*mp2*RN1p*
         RS146pe*RSp453e + 1./81.*m11*mp2*RN1p*RSp461e*RS145pe - 1./81.
         *m11*mp2*RN1p*RSp461e*RS146pe - 1./81.*m11*mp2*RN1p*RSp451e*
         RS145pe + 1./81.*m11*mp2*RN1p*RSp451e*RS146pe - 1./324.*m11*
         m66*RN3p*RSp463e*RS345pe - 1./324.*m11*m66*RN3p*RSp463e*
         RS346pe - 1./108.*m11*m66*RN3p*RSp453e*RS345pe - 1./324.*m11*
         m66*RN3p*RSp453e*RS346pe - 1./324.*m11*m66*RN3p*RSp461e*
         RS345pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./324.*m11*m66*
         RN3p*RSp461e*RS346pe - 1./108.*m11*m66*RN3p*RSp451e*RS345pe - 
         1./324.*m11*m66*RN3p*RSp451e*RS346pe - 1./108.*m11*m66*RN1p*
         RS145pe*RSp463e - 1./36.*m11*m66*RN1p*RS145pe*RSp453e - 1./108.
         *m11*m66*RN1p*RS146pe*RSp463e - 1./108.*m11*m66*RN1p*RS146pe*
         RSp453e - 1./324.*m11*m66*RN1p*RSp461e*RS145pe - 1./324.*m11*
         m66*RN1p*RSp461e*RS146pe - 1./108.*m11*m66*RN1p*RSp451e*
         RS145pe - 1./324.*m11*m66*RN1p*RSp451e*RS146pe - 1./162.*m11*
         m56*RN3p*RSp463e*RS345pe + 1./162.*m11*m56*RN3p*RSp463e*
         RS346pe + 1./162.*m11*m56*RN3p*RSp453e*RS345pe - 1./162.*m11*
         m56*RN3p*RSp453e*RS346pe - 1./162.*m11*m56*RN3p*RSp461e*
         RS345pe + 1./162.*m11*m56*RN3p*RSp461e*RS346pe + 1./162.*m11*
         m56*RN3p*RSp451e*RS345pe - 1./162.*m11*m56*RN3p*RSp451e*
         RS346pe - 1./54.*m11*m56*RN1p*RS145pe*RSp463e + 1./54.*m11*m56
         *RN1p*RS145pe*RSp453e + 1./54.*m11*m56*RN1p*RS146pe*RSp463e - 
         1./54.*m11*m56*RN1p*RS146pe*RSp453e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./162.*m11*m56*
         RN1p*RSp461e*RS145pe + 1./162.*m11*m56*RN1p*RSp461e*RS146pe + 
         1./162.*m11*m56*RN1p*RSp451e*RS145pe - 1./162.*m11*m56*RN1p*
         RSp451e*RS146pe - 1./324.*m11*m55*RN3p*RSp463e*RS345pe - 1./
         108.*m11*m55*RN3p*RSp463e*RS346pe - 1./324.*m11*m55*RN3p*
         RSp453e*RS345pe - 1./324.*m11*m55*RN3p*RSp453e*RS346pe - 1./
         324.*m11*m55*RN3p*RSp461e*RS345pe - 1./108.*m11*m55*RN3p*
         RSp461e*RS346pe - 1./324.*m11*m55*RN3p*RSp451e*RS345pe - 1./
         324.*m11*m55*RN3p*RSp451e*RS346pe - 1./108.*m11*m55*RN1p*
         RS145pe*RSp463e - 1./108.*m11*m55*RN1p*RS145pe*RSp453e - 1./36.
         *m11*m55*RN1p*RS146pe*RSp463e - 1./108.*m11*m55*RN1p*RS146pe*
         RSp453e - 1./324.*m11*m55*RN1p*RSp461e*RS145pe - 1./108.*m11*
         m55*RN1p*RSp461e*RS146pe - 1./324.*m11*m55*RN1p*RSp451e*
         RS145pe - 1./324.*m11*m55*RN1p*RSp451e*RS146pe - 1./162.*m11*
         m33*RN3p*RSp463e*RS345pe + 1./162.*m11*m33*RN3p*RSp463e*
         RS346pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./162.*m11*m33*RN3p*
         RSp453e*RS345pe - 1./162.*m11*m33*RN3p*RSp453e*RS346pe - 1./
         162.*m11*m33*RN3p*RSp461e*RS345pe + 1./162.*m11*m33*RN3p*
         RSp461e*RS346pe + 1./162.*m11*m33*RN3p*RSp451e*RS345pe - 1./
         162.*m11*m33*RN3p*RSp451e*RS346pe - 1./162.*m11*m33*RN1p*
         RS145pe*RSp463e + 1./162.*m11*m33*RN1p*RS145pe*RSp453e + 1./
         162.*m11*m33*RN1p*RS146pe*RSp463e - 1./162.*m11*m33*RN1p*
         RS146pe*RSp453e - 1./162.*m11*m33*RN1p*RSp461e*RS145pe + 1./
         162.*m11*m33*RN1p*RSp461e*RS146pe + 1./162.*m11*m33*RN1p*
         RSp451e*RS145pe - 1./162.*m11*m33*RN1p*RSp451e*RS146pe - 1./81.
         *m11*m13*RN1p*RS145pe*RSp463e + 1./81.*m11*m13*RN1p*RS145pe*
         RSp453e + 1./81.*m11*m13*RN1p*RS146pe*RSp463e - 1./81.*m11*m13
         *RN1p*RS146pe*RSp453e + 1./81.*m11*m13*RN1p*RSp461e*RS145pe - 
         1./81.*m11*m13*RN1p*RSp461e*RS146pe - 1./81.*m11*m13*RN1p*
         RSp451e*RS145pe + 1./81.*m11*m13*RN1p*RSp451e*RS146pe - 1./162.
         *pow(m11,2)*RN1p*RS145pe*RSp463e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./162.*pow(m11,2)*
         RN1p*RS145pe*RSp453e + 1./162.*pow(m11,2)*RN1p*RS146pe*RSp463e
          - 1./162.*pow(m11,2)*RN1p*RS146pe*RSp453e - 1./54.*pow(m11,2)
         *RN1p*RSp461e*RS145pe + 1./54.*pow(m11,2)*RN1p*RSp461e*RS146pe
          + 1./54.*pow(m11,2)*RN1p*RSp451e*RS145pe - 1./54.*pow(m11,2)*
         RN1p*RSp451e*RS146pe + 1./81.*RN(mp2,me2)*mp2*me2*RSp463e*
         RSe463p - 1./81.*RN(mp2,me2)*mp2*me2*RSp463e*RSe453p - 1./81.*
         RN(mp2,me2)*mp2*me2*RSp453e*RSe463p + 1./81.*RN(mp2,me2)*mp2*
         me2*RSp453e*RSe453p - 1./81.*RN(mp2,me2)*mp2*me2*RSe461p*
         RSp463e + 1./81.*RN(mp2,me2)*mp2*me2*RSe461p*RSp453e + 1./81.*
         RN(mp2,me2)*mp2*me2*RSe451p*RSp463e - 1./81.*RN(mp2,me2)*mp2*
         me2*RSe451p*RSp453e - 1./81.*RN(mp2,me2)*mp2*me2*RSp461e*
         RSe463p + 1./81.*RN(mp2,me2)*mp2*me2*RSp461e*RSe453p + 1./81.*
         RN(mp2,me2)*mp2*me2*RSp461e*RSe461p - 1./81.*RN(mp2,me2)*mp2*
         me2*RSp461e*RSe451p + 1./81.*RN(mp2,me2)*mp2*me2*RSp451e*
         RSe463p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./81.*RN(mp2,me2)
         *mp2*me2*RSp451e*RSe453p - 1./81.*RN(mp2,me2)*mp2*me2*RSp451e*
         RSe461p + 1./81.*RN(mp2,me2)*mp2*me2*RSp451e*RSe451p - 1./162.
         *RN(mp2,me2)*m66*me2*RSp463e*RSe463p - 1./162.*RN(mp2,me2)*m66
         *me2*RSp463e*RSe453p - 1./162.*RN(mp2,me2)*m66*me2*RSp453e*
         RSe463p - 1./54.*RN(mp2,me2)*m66*me2*RSp453e*RSe453p + 1./162.
         *RN(mp2,me2)*m66*me2*RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m66
         *me2*RSe461p*RSp453e + 1./162.*RN(mp2,me2)*m66*me2*RSe451p*
         RSp463e + 1./54.*RN(mp2,me2)*m66*me2*RSe451p*RSp453e + 1./162.
         *RN(mp2,me2)*m66*me2*RSp461e*RSe463p + 1./162.*RN(mp2,me2)*m66
         *me2*RSp461e*RSe453p - 1./162.*RN(mp2,me2)*m66*me2*RSp461e*
         RSe461p - 1./162.*RN(mp2,me2)*m66*me2*RSp461e*RSe451p + 1./162.
         *RN(mp2,me2)*m66*me2*RSp451e*RSe463p + 1./54.*RN(mp2,me2)*m66*
         me2*RSp451e*RSe453p - 1./162.*RN(mp2,me2)*m66*me2*RSp451e*
         RSe461p - 1./54.*RN(mp2,me2)*m66*me2*RSp451e*RSe451p + 1./81.*
         RN(mp2,me2)*m56*me2*RSp463e*RSe463p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./81.*RN(mp2,me2)
         *m56*me2*RSp463e*RSe453p - 1./81.*RN(mp2,me2)*m56*me2*RSp453e*
         RSe463p + 1./81.*RN(mp2,me2)*m56*me2*RSp453e*RSe453p - 1./81.*
         RN(mp2,me2)*m56*me2*RSe461p*RSp463e + 1./81.*RN(mp2,me2)*m56*
         me2*RSe461p*RSp453e + 1./81.*RN(mp2,me2)*m56*me2*RSe451p*
         RSp463e - 1./81.*RN(mp2,me2)*m56*me2*RSe451p*RSp453e - 1./81.*
         RN(mp2,me2)*m56*me2*RSp461e*RSe463p + 1./81.*RN(mp2,me2)*m56*
         me2*RSp461e*RSe453p + 1./81.*RN(mp2,me2)*m56*me2*RSp461e*
         RSe461p - 1./81.*RN(mp2,me2)*m56*me2*RSp461e*RSe451p + 1./81.*
         RN(mp2,me2)*m56*me2*RSp451e*RSe463p - 1./81.*RN(mp2,me2)*m56*
         me2*RSp451e*RSe453p - 1./81.*RN(mp2,me2)*m56*me2*RSp451e*
         RSe461p + 1./81.*RN(mp2,me2)*m56*me2*RSp451e*RSe451p - 1./54.*
         RN(mp2,me2)*m55*me2*RSp463e*RSe463p - 1./162.*RN(mp2,me2)*m55*
         me2*RSp463e*RSe453p - 1./162.*RN(mp2,me2)*m55*me2*RSp453e*
         RSe463p - 1./162.*RN(mp2,me2)*m55*me2*RSp453e*RSe453p + 1./54.
         *RN(mp2,me2)*m55*me2*RSe461p*RSp463e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./162.*RN(mp2,me2)*
         m55*me2*RSe461p*RSp453e + 1./162.*RN(mp2,me2)*m55*me2*RSe451p*
         RSp463e + 1./162.*RN(mp2,me2)*m55*me2*RSe451p*RSp453e + 1./54.
         *RN(mp2,me2)*m55*me2*RSp461e*RSe463p + 1./162.*RN(mp2,me2)*m55
         *me2*RSp461e*RSe453p - 1./54.*RN(mp2,me2)*m55*me2*RSp461e*
         RSe461p - 1./162.*RN(mp2,me2)*m55*me2*RSp461e*RSe451p + 1./162.
         *RN(mp2,me2)*m55*me2*RSp451e*RSe463p + 1./162.*RN(mp2,me2)*m55
         *me2*RSp451e*RSe453p - 1./162.*RN(mp2,me2)*m55*me2*RSp451e*
         RSe461p - 1./162.*RN(mp2,me2)*m55*me2*RSp451e*RSe451p - 1./54.
         *RN(mp2,me2)*m33*me2*RSp463e*RSe463p + 1./54.*RN(mp2,me2)*m33*
         me2*RSp463e*RSe453p + 1./54.*RN(mp2,me2)*m33*me2*RSp453e*
         RSe463p - 1./54.*RN(mp2,me2)*m33*me2*RSp453e*RSe453p - 1./162.
         *RN(mp2,me2)*m33*me2*RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m33
         *me2*RSe461p*RSp453e + 1./162.*RN(mp2,me2)*m33*me2*RSe451p*
         RSp463e - 1./162.*RN(mp2,me2)*m33*me2*RSe451p*RSp453e - 1./162.
         *RN(mp2,me2)*m33*me2*RSp461e*RSe463p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./162.*RN(mp2,me2)*
         m33*me2*RSp461e*RSe453p - 1./162.*RN(mp2,me2)*m33*me2*RSp461e*
         RSe461p + 1./162.*RN(mp2,me2)*m33*me2*RSp461e*RSe451p + 1./162.
         *RN(mp2,me2)*m33*me2*RSp451e*RSe463p - 1./162.*RN(mp2,me2)*m33
         *me2*RSp451e*RSe453p + 1./162.*RN(mp2,me2)*m33*me2*RSp451e*
         RSe461p - 1./162.*RN(mp2,me2)*m33*me2*RSp451e*RSe451p + 1./108.
         *RN(mp2,me2)*m33*m66*RSp463e*RSe463p + 1./108.*RN(mp2,me2)*m33
         *m66*RSp463e*RSe453p + 1./108.*RN(mp2,me2)*m33*m66*RSp453e*
         RSe463p + 1./36.*RN(mp2,me2)*m33*m66*RSp453e*RSe453p + 1./324.
         *RN(mp2,me2)*m33*m66*RSe461p*RSp463e + 1./324.*RN(mp2,me2)*m33
         *m66*RSe461p*RSp453e + 1./324.*RN(mp2,me2)*m33*m66*RSe451p*
         RSp463e + 1./108.*RN(mp2,me2)*m33*m66*RSe451p*RSp453e + 1./324.
         *RN(mp2,me2)*m33*m66*RSp461e*RSe463p + 1./324.*RN(mp2,me2)*m33
         *m66*RSp461e*RSe453p + 1./324.*RN(mp2,me2)*m33*m66*RSp461e*
         RSe461p + 1./324.*RN(mp2,me2)*m33*m66*RSp461e*RSe451p + 1./324.
         *RN(mp2,me2)*m33*m66*RSp451e*RSe463p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./108.*RN(mp2,me2)*
         m33*m66*RSp451e*RSe453p + 1./324.*RN(mp2,me2)*m33*m66*RSp451e*
         RSe461p + 1./108.*RN(mp2,me2)*m33*m66*RSp451e*RSe451p - 1./54.
         *RN(mp2,me2)*m33*m56*RSp463e*RSe463p + 1./54.*RN(mp2,me2)*m33*
         m56*RSp463e*RSe453p + 1./54.*RN(mp2,me2)*m33*m56*RSp453e*
         RSe463p - 1./54.*RN(mp2,me2)*m33*m56*RSp453e*RSe453p - 1./162.
         *RN(mp2,me2)*m33*m56*RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m33
         *m56*RSe461p*RSp453e + 1./162.*RN(mp2,me2)*m33*m56*RSe451p*
         RSp463e - 1./162.*RN(mp2,me2)*m33*m56*RSe451p*RSp453e - 1./162.
         *RN(mp2,me2)*m33*m56*RSp461e*RSe463p + 1./162.*RN(mp2,me2)*m33
         *m56*RSp461e*RSe453p - 1./162.*RN(mp2,me2)*m33*m56*RSp461e*
         RSe461p + 1./162.*RN(mp2,me2)*m33*m56*RSp461e*RSe451p + 1./162.
         *RN(mp2,me2)*m33*m56*RSp451e*RSe463p - 1./162.*RN(mp2,me2)*m33
         *m56*RSp451e*RSe453p + 1./162.*RN(mp2,me2)*m33*m56*RSp451e*
         RSe461p - 1./162.*RN(mp2,me2)*m33*m56*RSp451e*RSe451p + 1./36.
         *RN(mp2,me2)*m33*m55*RSp463e*RSe463p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./108.*RN(mp2,me2)*
         m33*m55*RSp463e*RSe453p + 1./108.*RN(mp2,me2)*m33*m55*RSp453e*
         RSe463p + 1./108.*RN(mp2,me2)*m33*m55*RSp453e*RSe453p + 1./108.
         *RN(mp2,me2)*m33*m55*RSe461p*RSp463e + 1./324.*RN(mp2,me2)*m33
         *m55*RSe461p*RSp453e + 1./324.*RN(mp2,me2)*m33*m55*RSe451p*
         RSp463e + 1./324.*RN(mp2,me2)*m33*m55*RSe451p*RSp453e + 1./108.
         *RN(mp2,me2)*m33*m55*RSp461e*RSe463p + 1./324.*RN(mp2,me2)*m33
         *m55*RSp461e*RSe453p + 1./108.*RN(mp2,me2)*m33*m55*RSp461e*
         RSe461p + 1./324.*RN(mp2,me2)*m33*m55*RSp461e*RSe451p + 1./324.
         *RN(mp2,me2)*m33*m55*RSp451e*RSe463p + 1./324.*RN(mp2,me2)*m33
         *m55*RSp451e*RSe453p + 1./324.*RN(mp2,me2)*m33*m55*RSp451e*
         RSe461p + 1./324.*RN(mp2,me2)*m33*m55*RSp451e*RSe451p + 1./81.
         *RN(mp2,me2)*m13*me2*RSp463e*RSe463p - 1./81.*RN(mp2,me2)*m13*
         me2*RSp463e*RSe453p - 1./81.*RN(mp2,me2)*m13*me2*RSp453e*
         RSe463p + 1./81.*RN(mp2,me2)*m13*me2*RSp453e*RSe453p - 1./81.*
         RN(mp2,me2)*m13*me2*RSe461p*RSp463e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./81.*RN(mp2,me2)*
         m13*me2*RSe461p*RSp453e + 1./81.*RN(mp2,me2)*m13*me2*RSe451p*
         RSp463e - 1./81.*RN(mp2,me2)*m13*me2*RSe451p*RSp453e - 1./81.*
         RN(mp2,me2)*m13*me2*RSp461e*RSe463p + 1./81.*RN(mp2,me2)*m13*
         me2*RSp461e*RSe453p + 1./81.*RN(mp2,me2)*m13*me2*RSp461e*
         RSe461p - 1./81.*RN(mp2,me2)*m13*me2*RSp461e*RSe451p + 1./81.*
         RN(mp2,me2)*m13*me2*RSp451e*RSe463p - 1./81.*RN(mp2,me2)*m13*
         me2*RSp451e*RSe453p - 1./81.*RN(mp2,me2)*m13*me2*RSp451e*
         RSe461p + 1./81.*RN(mp2,me2)*m13*me2*RSp451e*RSe451p - 1./162.
         *RN(mp2,me2)*m13*m66*RSp463e*RSe463p - 1./162.*RN(mp2,me2)*m13
         *m66*RSp463e*RSe453p - 1./162.*RN(mp2,me2)*m13*m66*RSp453e*
         RSe463p - 1./54.*RN(mp2,me2)*m13*m66*RSp453e*RSe453p + 1./162.
         *RN(mp2,me2)*m13*m66*RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m13
         *m66*RSe461p*RSp453e + 1./162.*RN(mp2,me2)*m13*m66*RSe451p*
         RSp463e + 1./54.*RN(mp2,me2)*m13*m66*RSe451p*RSp453e + 1./162.
         *RN(mp2,me2)*m13*m66*RSp461e*RSe463p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./162.*RN(mp2,me2)*
         m13*m66*RSp461e*RSe453p - 1./162.*RN(mp2,me2)*m13*m66*RSp461e*
         RSe461p - 1./162.*RN(mp2,me2)*m13*m66*RSp461e*RSe451p + 1./162.
         *RN(mp2,me2)*m13*m66*RSp451e*RSe463p + 1./54.*RN(mp2,me2)*m13*
         m66*RSp451e*RSe453p - 1./162.*RN(mp2,me2)*m13*m66*RSp451e*
         RSe461p - 1./54.*RN(mp2,me2)*m13*m66*RSp451e*RSe451p + 1./81.*
         RN(mp2,me2)*m13*m56*RSp463e*RSe463p - 1./81.*RN(mp2,me2)*m13*
         m56*RSp463e*RSe453p - 1./81.*RN(mp2,me2)*m13*m56*RSp453e*
         RSe463p + 1./81.*RN(mp2,me2)*m13*m56*RSp453e*RSe453p - 1./81.*
         RN(mp2,me2)*m13*m56*RSe461p*RSp463e + 1./81.*RN(mp2,me2)*m13*
         m56*RSe461p*RSp453e + 1./81.*RN(mp2,me2)*m13*m56*RSe451p*
         RSp463e - 1./81.*RN(mp2,me2)*m13*m56*RSe451p*RSp453e - 1./81.*
         RN(mp2,me2)*m13*m56*RSp461e*RSe463p + 1./81.*RN(mp2,me2)*m13*
         m56*RSp461e*RSe453p + 1./81.*RN(mp2,me2)*m13*m56*RSp461e*
         RSe461p - 1./81.*RN(mp2,me2)*m13*m56*RSp461e*RSe451p + 1./81.*
         RN(mp2,me2)*m13*m56*RSp451e*RSe463p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./81.*RN(mp2,me2)
         *m13*m56*RSp451e*RSe453p - 1./81.*RN(mp2,me2)*m13*m56*RSp451e*
         RSe461p + 1./81.*RN(mp2,me2)*m13*m56*RSp451e*RSe451p - 1./54.*
         RN(mp2,me2)*m13*m55*RSp463e*RSe463p - 1./162.*RN(mp2,me2)*m13*
         m55*RSp463e*RSe453p - 1./162.*RN(mp2,me2)*m13*m55*RSp453e*
         RSe463p - 1./162.*RN(mp2,me2)*m13*m55*RSp453e*RSe453p + 1./54.
         *RN(mp2,me2)*m13*m55*RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m13
         *m55*RSe461p*RSp453e + 1./162.*RN(mp2,me2)*m13*m55*RSe451p*
         RSp463e + 1./162.*RN(mp2,me2)*m13*m55*RSe451p*RSp453e + 1./54.
         *RN(mp2,me2)*m13*m55*RSp461e*RSe463p + 1./162.*RN(mp2,me2)*m13
         *m55*RSp461e*RSe453p - 1./54.*RN(mp2,me2)*m13*m55*RSp461e*
         RSe461p - 1./162.*RN(mp2,me2)*m13*m55*RSp461e*RSe451p + 1./162.
         *RN(mp2,me2)*m13*m55*RSp451e*RSe463p + 1./162.*RN(mp2,me2)*m13
         *m55*RSp451e*RSe453p - 1./162.*RN(mp2,me2)*m13*m55*RSp451e*
         RSe461p - 1./162.*RN(mp2,me2)*m13*m55*RSp451e*RSe451p - 1./162.
         *RN(mp2,me2)*m11*me2*RSp463e*RSe463p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./162.*RN(mp2,me2)*
         m11*me2*RSp463e*RSe453p + 1./162.*RN(mp2,me2)*m11*me2*RSp453e*
         RSe463p - 1./162.*RN(mp2,me2)*m11*me2*RSp453e*RSe453p - 1./162.
         *RN(mp2,me2)*m11*me2*RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m11
         *me2*RSe461p*RSp453e + 1./162.*RN(mp2,me2)*m11*me2*RSe451p*
         RSp463e - 1./162.*RN(mp2,me2)*m11*me2*RSe451p*RSp453e - 1./162.
         *RN(mp2,me2)*m11*me2*RSp461e*RSe463p + 1./162.*RN(mp2,me2)*m11
         *me2*RSp461e*RSe453p - 1./54.*RN(mp2,me2)*m11*me2*RSp461e*
         RSe461p + 1./54.*RN(mp2,me2)*m11*me2*RSp461e*RSe451p + 1./162.
         *RN(mp2,me2)*m11*me2*RSp451e*RSe463p - 1./162.*RN(mp2,me2)*m11
         *me2*RSp451e*RSe453p + 1./54.*RN(mp2,me2)*m11*me2*RSp451e*
         RSe461p - 1./54.*RN(mp2,me2)*m11*me2*RSp451e*RSe451p + 1./324.
         *RN(mp2,me2)*m11*m66*RSp463e*RSe463p + 1./324.*RN(mp2,me2)*m11
         *m66*RSp463e*RSe453p + 1./324.*RN(mp2,me2)*m11*m66*RSp453e*
         RSe463p + 1./108.*RN(mp2,me2)*m11*m66*RSp453e*RSe453p + 1./324.
         *RN(mp2,me2)*m11*m66*RSe461p*RSp463e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./324.*RN(mp2,me2)*
         m11*m66*RSe461p*RSp453e + 1./324.*RN(mp2,me2)*m11*m66*RSe451p*
         RSp463e + 1./108.*RN(mp2,me2)*m11*m66*RSe451p*RSp453e + 1./324.
         *RN(mp2,me2)*m11*m66*RSp461e*RSe463p + 1./324.*RN(mp2,me2)*m11
         *m66*RSp461e*RSe453p + 1./108.*RN(mp2,me2)*m11*m66*RSp461e*
         RSe461p + 1./108.*RN(mp2,me2)*m11*m66*RSp461e*RSe451p + 1./324.
         *RN(mp2,me2)*m11*m66*RSp451e*RSe463p + 1./108.*RN(mp2,me2)*m11
         *m66*RSp451e*RSe453p + 1./108.*RN(mp2,me2)*m11*m66*RSp451e*
         RSe461p + 1./36.*RN(mp2,me2)*m11*m66*RSp451e*RSe451p - 1./162.
         *RN(mp2,me2)*m11*m56*RSp463e*RSe463p + 1./162.*RN(mp2,me2)*m11
         *m56*RSp463e*RSe453p + 1./162.*RN(mp2,me2)*m11*m56*RSp453e*
         RSe463p - 1./162.*RN(mp2,me2)*m11*m56*RSp453e*RSe453p - 1./162.
         *RN(mp2,me2)*m11*m56*RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m11
         *m56*RSe461p*RSp453e + 1./162.*RN(mp2,me2)*m11*m56*RSe451p*
         RSp463e - 1./162.*RN(mp2,me2)*m11*m56*RSe451p*RSp453e - 1./162.
         *RN(mp2,me2)*m11*m56*RSp461e*RSe463p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./162.*RN(mp2,me2)*
         m11*m56*RSp461e*RSe453p - 1./54.*RN(mp2,me2)*m11*m56*RSp461e*
         RSe461p + 1./54.*RN(mp2,me2)*m11*m56*RSp461e*RSe451p + 1./162.
         *RN(mp2,me2)*m11*m56*RSp451e*RSe463p - 1./162.*RN(mp2,me2)*m11
         *m56*RSp451e*RSe453p + 1./54.*RN(mp2,me2)*m11*m56*RSp451e*
         RSe461p - 1./54.*RN(mp2,me2)*m11*m56*RSp451e*RSe451p + 1./108.
         *RN(mp2,me2)*m11*m55*RSp463e*RSe463p + 1./324.*RN(mp2,me2)*m11
         *m55*RSp463e*RSe453p + 1./324.*RN(mp2,me2)*m11*m55*RSp453e*
         RSe463p + 1./324.*RN(mp2,me2)*m11*m55*RSp453e*RSe453p + 1./108.
         *RN(mp2,me2)*m11*m55*RSe461p*RSp463e + 1./324.*RN(mp2,me2)*m11
         *m55*RSe461p*RSp453e + 1./324.*RN(mp2,me2)*m11*m55*RSe451p*
         RSp463e + 1./324.*RN(mp2,me2)*m11*m55*RSe451p*RSp453e + 1./108.
         *RN(mp2,me2)*m11*m55*RSp461e*RSe463p + 1./324.*RN(mp2,me2)*m11
         *m55*RSp461e*RSe453p + 1./36.*RN(mp2,me2)*m11*m55*RSp461e*
         RSe461p + 1./108.*RN(mp2,me2)*m11*m55*RSp461e*RSe451p + 1./324.
         *RN(mp2,me2)*m11*m55*RSp451e*RSe463p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./324.*RN(mp2,me2)*
         m11*m55*RSp451e*RSe453p + 1./108.*RN(mp2,me2)*m11*m55*RSp451e*
         RSe461p + 1./108.*RN(mp2,me2)*m11*m55*RSp451e*RSe451p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m66,L) * (  - 1./27.*m66*RSp453e
         *RS345pe - 1./27.*m66*RSp453e*RSe453p - 1./54.*m66*pow(
         RSp453e,2) + 1./27.*m66*RS145pe*RSp453e + 1./81.*m66*RS645pe*
         RSp453e*RS345pe + 1./81.*m66*RS645pe*RSp453e*RSe453p + 1./162.
         *m66*RS645pe*pow(RSp453e,2) - 1./81.*m66*RS645pe*RS145pe*
         RSp453e + 1./27.*m66*RSe451p*RSp453e - 1./81.*m66*RSe451p*
         RS645pe*RSp453e + 1./27.*m66*RSp451e*RS345pe + 1./27.*m66*
         RSp451e*RSe453p + 1./27.*m66*RSp451e*RSp453e - 1./27.*m66*
         RSp451e*RS145pe - 1./81.*m66*RSp451e*RS645pe*RS345pe - 1./81.*
         m66*RSp451e*RS645pe*RSe453p - 1./81.*m66*RSp451e*RS645pe*
         RSp453e + 1./81.*m66*RSp451e*RS645pe*RS145pe - 1./27.*m66*
         RSp451e*RSe451p + 1./81.*m66*RSp451e*RSe451p*RS645pe - 1./54.*
         m66*pow(RSp451e,2) + 1./162.*m66*pow(RSp451e,2)*RS645pe - 1./
         54.*m33*m66*RN3p*RSp453e*RS345pe + 1./162.*m33*m66*RN3p*
         RS645pe*RSp453e*RS345pe - 1./18.*m33*m66*RN3p*RSp451e*RS345pe
          + 1./54.*m33*m66*RN3p*RSp451e*RS645pe*RS345pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m66,L) * (  - 1./54.*m33*m66*
         RN1p*RS145pe*RSp453e + 1./162.*m33*m66*RN1p*RS645pe*RS145pe*
         RSp453e - 1./54.*m33*m66*RN1p*RSp451e*RS145pe + 1./162.*m33*
         m66*RN1p*RSp451e*RS645pe*RS145pe + 1./27.*m13*m66*RN3p*RSp453e
         *RS345pe - 1./81.*m13*m66*RN3p*RS645pe*RSp453e*RS345pe - 1./27.
         *m13*m66*RN3p*RSp451e*RS345pe + 1./81.*m13*m66*RN3p*RSp451e*
         RS645pe*RS345pe - 1./27.*m13*m66*RN1p*RS145pe*RSp453e + 1./81.
         *m13*m66*RN1p*RS645pe*RS145pe*RSp453e + 1./27.*m13*m66*RN1p*
         RSp451e*RS145pe - 1./81.*m13*m66*RN1p*RSp451e*RS645pe*RS145pe
          - 1./54.*m11*m66*RN3p*RSp453e*RS345pe + 1./162.*m11*m66*RN3p*
         RS645pe*RSp453e*RS345pe - 1./54.*m11*m66*RN3p*RSp451e*RS345pe
          + 1./162.*m11*m66*RN3p*RSp451e*RS645pe*RS345pe - 1./18.*m11*
         m66*RN1p*RS145pe*RSp453e + 1./54.*m11*m66*RN1p*RS645pe*RS145pe
         *RSp453e - 1./54.*m11*m66*RN1p*RSp451e*RS145pe + 1./162.*m11*
         m66*RN1p*RSp451e*RS645pe*RS145pe - 1./27.*RN(mp2,me2)*m66*me2*
         RSp453e*RSe453p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m66,L) * ( 1./81.*RN(mp2,me2)*
         m66*me2*RS645pe*RSp453e*RSe453p + 1./27.*RN(mp2,me2)*m66*me2*
         RSe451p*RSp453e - 1./81.*RN(mp2,me2)*m66*me2*RSe451p*RS645pe*
         RSp453e + 1./27.*RN(mp2,me2)*m66*me2*RSp451e*RSe453p - 1./81.*
         RN(mp2,me2)*m66*me2*RSp451e*RS645pe*RSe453p - 1./27.*RN(mp2,
         me2)*m66*me2*RSp451e*RSe451p + 1./81.*RN(mp2,me2)*m66*me2*
         RSp451e*RSe451p*RS645pe + 1./18.*RN(mp2,me2)*m33*m66*RSp453e*
         RSe453p - 1./54.*RN(mp2,me2)*m33*m66*RS645pe*RSp453e*RSe453p
          + 1./54.*RN(mp2,me2)*m33*m66*RSe451p*RSp453e - 1./162.*RN(mp2
         ,me2)*m33*m66*RSe451p*RS645pe*RSp453e + 1./54.*RN(mp2,me2)*m33
         *m66*RSp451e*RSe453p - 1./162.*RN(mp2,me2)*m33*m66*RSp451e*
         RS645pe*RSe453p + 1./54.*RN(mp2,me2)*m33*m66*RSp451e*RSe451p
          - 1./162.*RN(mp2,me2)*m33*m66*RSp451e*RSe451p*RS645pe - 1./27.
         *RN(mp2,me2)*m13*m66*RSp453e*RSe453p + 1./81.*RN(mp2,me2)*m13*
         m66*RS645pe*RSp453e*RSe453p + 1./27.*RN(mp2,me2)*m13*m66*
         RSe451p*RSp453e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(m66,L) * (  - 1./81.*RN(mp2,me2)
         *m13*m66*RSe451p*RS645pe*RSp453e + 1./27.*RN(mp2,me2)*m13*m66*
         RSp451e*RSe453p - 1./81.*RN(mp2,me2)*m13*m66*RSp451e*RS645pe*
         RSe453p - 1./27.*RN(mp2,me2)*m13*m66*RSp451e*RSe451p + 1./81.*
         RN(mp2,me2)*m13*m66*RSp451e*RSe451p*RS645pe + 1./54.*RN(mp2,
         me2)*m11*m66*RSp453e*RSe453p - 1./162.*RN(mp2,me2)*m11*m66*
         RS645pe*RSp453e*RSe453p + 1./54.*RN(mp2,me2)*m11*m66*RSe451p*
         RSp453e - 1./162.*RN(mp2,me2)*m11*m66*RSe451p*RS645pe*RSp453e
          + 1./54.*RN(mp2,me2)*m11*m66*RSp451e*RSe453p - 1./162.*RN(mp2
         ,me2)*m11*m66*RSp451e*RS645pe*RSe453p + 1./18.*RN(mp2,me2)*m11
         *m66*RSp451e*RSe451p - 1./54.*RN(mp2,me2)*m11*m66*RSp451e*
         RSe451p*RS645pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(mp2,L) * ( 2./135.*mp2*pow(
         RSp45613e,2) - 2./135.*mp2*RSp45633e*RSp45613e + 1./270.*mp2*
         pow(RSp45633e,2) - 2./135.*mp2*RSp45611e*RSp45613e + 1./135.*
         mp2*RSp45611e*RSp45633e + 1./270.*mp2*pow(RSp45611e,2) + 2./81.
         *m66*RSp456e*RSp453e*RS345pe + 2./81.*m66*RSp456e*RSp453e*
         RSe453p + 1./81.*m66*RSp456e*pow(RSp453e,2) - 2./81.*m66*
         RSp456e*RS145pe*RSp453e - 2./81.*m66*RSp456e*RSe451p*RSp453e
          - 2./81.*m66*RSp456e*RSp451e*RS345pe - 2./81.*m66*RSp456e*
         RSp451e*RSe453p - 2./81.*m66*RSp456e*RSp451e*RSp453e + 2./81.*
         m66*RSp456e*RSp451e*RS145pe + 2./81.*m66*RSp456e*RSp451e*
         RSe451p + 1./81.*m66*RSp456e*pow(RSp451e,2) + 2./81.*m55*
         RSp465e*RSp463e*RS346pe + 2./81.*m55*RSp465e*RSp463e*RSe463p
          + 1./81.*m55*RSp465e*pow(RSp463e,2) - 2./81.*m55*RSp465e*
         RS146pe*RSp463e - 2./81.*m55*RSp465e*RSe461p*RSp463e - 2./81.*
         m55*RSp465e*RSp461e*RS346pe - 2./81.*m55*RSp465e*RSp461e*
         RSe463p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(mp2,L) * (  - 2./81.*m55*RSp465e
         *RSp461e*RSp463e + 2./81.*m55*RSp465e*RSp461e*RS146pe + 2./81.
         *m55*RSp465e*RSp461e*RSe461p + 1./81.*m55*RSp465e*pow(
         RSp461e,2) + 2./81.*m44*RSp564e*RSp563e*RS356pe + 2./81.*m44*
         RSp564e*RSp563e*RSe563p + 1./81.*m44*RSp564e*pow(RSp563e,2) - 
         2./81.*m44*RSp564e*RS156pe*RSp563e - 2./81.*m44*RSp564e*
         RSe561p*RSp563e - 2./81.*m44*RSp564e*RSp561e*RS356pe - 2./81.*
         m44*RSp564e*RSp561e*RSe563p - 2./81.*m44*RSp564e*RSp561e*
         RSp563e + 2./81.*m44*RSp564e*RSp561e*RS156pe + 2./81.*m44*
         RSp564e*RSp561e*RSe561p + 1./81.*m44*RSp564e*pow(RSp561e,2) + 
         23./405.*m33*pow(RSp45613e,2) + 16./405.*m33*RSp45633e*
         RSp45613e - 2./81.*m33*pow(RSp45633e,2) - 2./405.*m33*
         RSp45611e*RSp45613e - 11./405.*m33*RSp45611e*RSp45633e + 13./
         810.*m33*pow(RSp45611e,2) + 1./162.*m33*m66*RN3p*RSp456e*
         RSp453e*RS345pe + 1./54.*m33*m66*RN3p*RSp456e*RSp451e*RS345pe
          + 1./162.*m33*m66*RN1p*RSp456e*RS145pe*RSp453e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(mp2,L) * ( 1./162.*m33*m66*RN1p*
         RSp456e*RSp451e*RS145pe + 1./162.*m33*m55*RN3p*RSp465e*RSp463e
         *RS346pe + 1./54.*m33*m55*RN3p*RSp465e*RSp461e*RS346pe + 1./
         162.*m33*m55*RN1p*RSp465e*RS146pe*RSp463e + 1./162.*m33*m55*
         RN1p*RSp465e*RSp461e*RS146pe + 1./162.*m33*m44*RN3p*RSp564e*
         RSp563e*RS356pe + 1./54.*m33*m44*RN3p*RSp564e*RSp561e*RS356pe
          + 1./162.*m33*m44*RN1p*RSp564e*RS156pe*RSp563e + 1./162.*m33*
         m44*RN1p*RSp564e*RSp561e*RS156pe - 1./18.*pow(m33,2)*RN3p*
         RSp45633e*RSp45613e - 1./54.*pow(m33,2)*RN3p*pow(RSp45633e,2)
          - 7./405.*m13*pow(RSp45613e,2) + 7./405.*m13*RSp45633e*
         RSp45613e - 7./1620.*m13*pow(RSp45633e,2) + 7./405.*m13*
         RSp45611e*RSp45613e - 7./810.*m13*RSp45611e*RSp45633e - 7./
         1620.*m13*pow(RSp45611e,2) - 1./81.*m13*m66*RN3p*RSp456e*
         RSp453e*RS345pe + 1./81.*m13*m66*RN3p*RSp456e*RSp451e*RS345pe
          + 1./81.*m13*m66*RN1p*RSp456e*RS145pe*RSp453e - 1./81.*m13*
         m66*RN1p*RSp456e*RSp451e*RS145pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(mp2,L) * (  - 1./81.*m13*m55*
         RN3p*RSp465e*RSp463e*RS346pe + 1./81.*m13*m55*RN3p*RSp465e*
         RSp461e*RS346pe + 1./81.*m13*m55*RN1p*RSp465e*RS146pe*RSp463e
          - 1./81.*m13*m55*RN1p*RSp465e*RSp461e*RS146pe - 1./81.*m13*
         m44*RN3p*RSp564e*RSp563e*RS356pe + 1./81.*m13*m44*RN3p*RSp564e
         *RSp561e*RS356pe + 1./81.*m13*m44*RN1p*RSp564e*RS156pe*RSp563e
          - 1./81.*m13*m44*RN1p*RSp564e*RSp561e*RS156pe - 1./27.*m13*
         m33*RN3p*RSp45633e*RSp45613e + 1./27.*m13*m33*RN3p*pow(
         RSp45633e,2) + 23./405.*m11*pow(RSp45613e,2) - 2./405.*m11*
         RSp45633e*RSp45613e + 13./810.*m11*pow(RSp45633e,2) + 16./405.
         *m11*RSp45611e*RSp45613e - 11./405.*m11*RSp45611e*RSp45633e - 
         2./81.*m11*pow(RSp45611e,2) + 1./162.*m11*m66*RN3p*RSp456e*
         RSp453e*RS345pe + 1./162.*m11*m66*RN3p*RSp456e*RSp451e*RS345pe
          + 1./54.*m11*m66*RN1p*RSp456e*RS145pe*RSp453e + 1./162.*m11*
         m66*RN1p*RSp456e*RSp451e*RS145pe + 1./162.*m11*m55*RN3p*
         RSp465e*RSp463e*RS346pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(mp2,L) * ( 1./162.*m11*m55*RN3p*
         RSp465e*RSp461e*RS346pe + 1./54.*m11*m55*RN1p*RSp465e*RS146pe*
         RSp463e + 1./162.*m11*m55*RN1p*RSp465e*RSp461e*RS146pe + 1./
         162.*m11*m44*RN3p*RSp564e*RSp563e*RS356pe + 1./162.*m11*m44*
         RN3p*RSp564e*RSp561e*RS356pe + 1./54.*m11*m44*RN1p*RSp564e*
         RS156pe*RSp563e + 1./162.*m11*m44*RN1p*RSp564e*RSp561e*RS156pe
          - 1./54.*m11*m33*RN3p*RSp45633e*RSp45613e - 1./54.*m11*m33*
         RN3p*pow(RSp45633e,2) - 1./54.*m11*m33*RN1p*RSp45611e*
         RSp45613e - 1./54.*m11*m33*RN1p*pow(RSp45611e,2) - 1./27.*m11*
         m13*RN1p*RSp45611e*RSp45613e + 1./27.*m11*m13*RN1p*pow(
         RSp45611e,2) - 1./18.*pow(m11,2)*RN1p*RSp45611e*RSp45613e - 1./
         54.*pow(m11,2)*RN1p*pow(RSp45611e,2) + 1./81.*RN(mp2,me2)*m66*
         me2*RSp456e*RSp453e*RSe453p - 1./81.*RN(mp2,me2)*m66*me2*
         RSp456e*RSe451p*RSp453e - 1./81.*RN(mp2,me2)*m66*me2*RSp456e*
         RSp451e*RSe453p + 1./81.*RN(mp2,me2)*m66*me2*RSp456e*RSp451e*
         RSe451p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(mp2,L) * ( 1./81.*RN(mp2,me2)*
         m55*me2*RSp465e*RSp463e*RSe463p - 1./81.*RN(mp2,me2)*m55*me2*
         RSp465e*RSe461p*RSp463e - 1./81.*RN(mp2,me2)*m55*me2*RSp465e*
         RSp461e*RSe463p + 1./81.*RN(mp2,me2)*m55*me2*RSp465e*RSp461e*
         RSe461p + 1./81.*RN(mp2,me2)*m44*me2*RSp564e*RSp563e*RSe563p
          - 1./81.*RN(mp2,me2)*m44*me2*RSp564e*RSe561p*RSp563e - 1./81.
         *RN(mp2,me2)*m44*me2*RSp564e*RSp561e*RSe563p + 1./81.*RN(mp2,
         me2)*m44*me2*RSp564e*RSp561e*RSe561p - 1./54.*RN(mp2,me2)*m33*
         m66*RSp456e*RSp453e*RSe453p - 1./162.*RN(mp2,me2)*m33*m66*
         RSp456e*RSe451p*RSp453e - 1./162.*RN(mp2,me2)*m33*m66*RSp456e*
         RSp451e*RSe453p - 1./162.*RN(mp2,me2)*m33*m66*RSp456e*RSp451e*
         RSe451p - 1./54.*RN(mp2,me2)*m33*m55*RSp465e*RSp463e*RSe463p
          - 1./162.*RN(mp2,me2)*m33*m55*RSp465e*RSe461p*RSp463e - 1./
         162.*RN(mp2,me2)*m33*m55*RSp465e*RSp461e*RSe463p - 1./162.*RN(
         mp2,me2)*m33*m55*RSp465e*RSp461e*RSe461p - 1./54.*RN(mp2,me2)*
         m33*m44*RSp564e*RSp563e*RSe563p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(mp2,L) * (  - 1./162.*RN(mp2,me2
         )*m33*m44*RSp564e*RSe561p*RSp563e - 1./162.*RN(mp2,me2)*m33*
         m44*RSp564e*RSp561e*RSe563p - 1./162.*RN(mp2,me2)*m33*m44*
         RSp564e*RSp561e*RSe561p + 1./81.*RN(mp2,me2)*m13*m66*RSp456e*
         RSp453e*RSe453p - 1./81.*RN(mp2,me2)*m13*m66*RSp456e*RSe451p*
         RSp453e - 1./81.*RN(mp2,me2)*m13*m66*RSp456e*RSp451e*RSe453p
          + 1./81.*RN(mp2,me2)*m13*m66*RSp456e*RSp451e*RSe451p + 1./81.
         *RN(mp2,me2)*m13*m55*RSp465e*RSp463e*RSe463p - 1./81.*RN(mp2,
         me2)*m13*m55*RSp465e*RSe461p*RSp463e - 1./81.*RN(mp2,me2)*m13*
         m55*RSp465e*RSp461e*RSe463p + 1./81.*RN(mp2,me2)*m13*m55*
         RSp465e*RSp461e*RSe461p + 1./81.*RN(mp2,me2)*m13*m44*RSp564e*
         RSp563e*RSe563p - 1./81.*RN(mp2,me2)*m13*m44*RSp564e*RSe561p*
         RSp563e - 1./81.*RN(mp2,me2)*m13*m44*RSp564e*RSp561e*RSe563p
          + 1./81.*RN(mp2,me2)*m13*m44*RSp564e*RSp561e*RSe561p - 1./162.
         *RN(mp2,me2)*m11*m66*RSp456e*RSp453e*RSe453p - 1./162.*RN(mp2,
         me2)*m11*m66*RSp456e*RSe451p*RSp453e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(mp2,L) * (  - 1./162.*RN(mp2,me2
         )*m11*m66*RSp456e*RSp451e*RSe453p - 1./54.*RN(mp2,me2)*m11*m66
         *RSp456e*RSp451e*RSe451p - 1./162.*RN(mp2,me2)*m11*m55*RSp465e
         *RSp463e*RSe463p - 1./162.*RN(mp2,me2)*m11*m55*RSp465e*RSe461p
         *RSp463e - 1./162.*RN(mp2,me2)*m11*m55*RSp465e*RSp461e*RSe463p
          - 1./54.*RN(mp2,me2)*m11*m55*RSp465e*RSp461e*RSe461p - 1./162.
         *RN(mp2,me2)*m11*m44*RSp564e*RSp563e*RSe563p - 1./162.*RN(mp2,
         me2)*m11*m44*RSp564e*RSe561p*RSp563e - 1./162.*RN(mp2,me2)*m11
         *m44*RSp564e*RSp561e*RSe563p - 1./54.*RN(mp2,me2)*m11*m44*
         RSp564e*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(me2,L) * ( 1./135.*me2*RSp45613e
         *RSe45613p - 1./270.*me2*RSe45633p*RSp45613e - 1./270.*me2*
         RSp45633e*RSe45613p + 1./540.*me2*RSp45633e*RSe45633p - 1./270.
         *me2*RSe45611p*RSp45613e + 1./540.*me2*RSe45611p*RSp45633e - 1.
         /270.*me2*RSp45611e*RSe45613p + 1./540.*me2*RSp45611e*
         RSe45633p + 1./540.*me2*RSp45611e*RSe45611p + 1./135.*mp2*
         RSp45613e*RSe45613p - 1./270.*mp2*RSe45633p*RSp45613e - 1./270.
         *mp2*RSp45633e*RSe45613p + 1./540.*mp2*RSp45633e*RSe45633p - 1.
         /270.*mp2*RSe45611p*RSp45613e + 1./540.*mp2*RSe45611p*
         RSp45633e - 1./270.*mp2*RSp45611e*RSe45613p + 1./540.*mp2*
         RSp45611e*RSe45633p + 1./540.*mp2*RSp45611e*RSe45611p + 1./81.
         *m66*RSe456p*RSp453e*RS345pe + 1./81.*m66*RSe456p*RSp453e*
         RSe453p + 1./162.*m66*RSe456p*pow(RSp453e,2) - 1./81.*m66*
         RSe456p*RS145pe*RSp453e - 1./81.*m66*RSe456p*RSe451p*RSp453e
          - 1./81.*m66*RSe456p*RSp451e*RS345pe - 1./81.*m66*RSe456p*
         RSp451e*RSe453p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(me2,L) * (  - 1./81.*m66*RSe456p
         *RSp451e*RSp453e + 1./81.*m66*RSe456p*RSp451e*RS145pe + 1./81.
         *m66*RSe456p*RSp451e*RSe451p + 1./162.*m66*RSe456p*pow(
         RSp451e,2) + 1./81.*m66*RSp456e*RSe453p*RS345pe + 1./162.*m66*
         RSp456e*pow(RSe453p,2) - 1./81.*m66*RSp456e*RS145pe*RSe453p - 
         1./81.*m66*RSp456e*RSe451p*RS345pe - 1./81.*m66*RSp456e*
         RSe451p*RSe453p + 1./81.*m66*RSp456e*RSe451p*RS145pe + 1./162.
         *m66*RSp456e*pow(RSe451p,2) + 1./81.*m55*RSe465p*RSp463e*
         RS346pe + 1./81.*m55*RSe465p*RSp463e*RSe463p + 1./162.*m55*
         RSe465p*pow(RSp463e,2) - 1./81.*m55*RSe465p*RS146pe*RSp463e - 
         1./81.*m55*RSe465p*RSe461p*RSp463e - 1./81.*m55*RSe465p*
         RSp461e*RS346pe - 1./81.*m55*RSe465p*RSp461e*RSe463p - 1./81.*
         m55*RSe465p*RSp461e*RSp463e + 1./81.*m55*RSe465p*RSp461e*
         RS146pe + 1./81.*m55*RSe465p*RSp461e*RSe461p + 1./162.*m55*
         RSe465p*pow(RSp461e,2) + 1./81.*m55*RSp465e*RSe463p*RS346pe + 
         1./162.*m55*RSp465e*pow(RSe463p,2) );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(me2,L) * (  - 1./81.*m55*RSp465e
         *RS146pe*RSe463p - 1./81.*m55*RSp465e*RSe461p*RS346pe - 1./81.
         *m55*RSp465e*RSe461p*RSe463p + 1./81.*m55*RSp465e*RSe461p*
         RS146pe + 1./162.*m55*RSp465e*pow(RSe461p,2) + 1./81.*m44*
         RSe564p*RSp563e*RS356pe + 1./81.*m44*RSe564p*RSp563e*RSe563p
          + 1./162.*m44*RSe564p*pow(RSp563e,2) - 1./81.*m44*RSe564p*
         RS156pe*RSp563e - 1./81.*m44*RSe564p*RSe561p*RSp563e - 1./81.*
         m44*RSe564p*RSp561e*RS356pe - 1./81.*m44*RSe564p*RSp561e*
         RSe563p - 1./81.*m44*RSe564p*RSp561e*RSp563e + 1./81.*m44*
         RSe564p*RSp561e*RS156pe + 1./81.*m44*RSe564p*RSp561e*RSe561p
          + 1./162.*m44*RSe564p*pow(RSp561e,2) + 1./81.*m44*RSp564e*
         RSe563p*RS356pe + 1./162.*m44*RSp564e*pow(RSe563p,2) - 1./81.*
         m44*RSp564e*RS156pe*RSe563p - 1./81.*m44*RSp564e*RSe561p*
         RS356pe - 1./81.*m44*RSp564e*RSe561p*RSe563p + 1./81.*m44*
         RSp564e*RSe561p*RS156pe + 1./162.*m44*RSp564e*pow(RSe561p,2)
          + 23./405.*m33*RSp45613e*RSe45613p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(me2,L) * ( 8./405.*m33*RSe45633p
         *RSp45613e + 8./405.*m33*RSp45633e*RSe45613p - 2./81.*m33*
         RSp45633e*RSe45633p - 1./405.*m33*RSe45611p*RSp45613e - 11./
         810.*m33*RSe45611p*RSp45633e - 1./405.*m33*RSp45611e*RSe45613p
          - 11./810.*m33*RSp45611e*RSe45633p + 13./810.*m33*RSp45611e*
         RSe45611p + 1./162.*m33*m66*RN3p*RSe456p*RSp453e*RS345pe + 1./
         54.*m33*m66*RN3p*RSe456p*RSp451e*RS345pe + 1./162.*m33*m66*
         RN1p*RSe456p*RS145pe*RSp453e + 1./162.*m33*m66*RN1p*RSe456p*
         RSp451e*RS145pe + 1./162.*m33*m55*RN3p*RSe465p*RSp463e*RS346pe
          + 1./54.*m33*m55*RN3p*RSe465p*RSp461e*RS346pe + 1./162.*m33*
         m55*RN1p*RSe465p*RS146pe*RSp463e + 1./162.*m33*m55*RN1p*
         RSe465p*RSp461e*RS146pe + 1./162.*m33*m44*RN3p*RSe564p*RSp563e
         *RS356pe + 1./54.*m33*m44*RN3p*RSe564p*RSp561e*RS356pe + 1./
         162.*m33*m44*RN1p*RSe564p*RS156pe*RSp563e + 1./162.*m33*m44*
         RN1p*RSe564p*RSp561e*RS156pe - 1./18.*pow(m33,2)*RN3p*
         RSe45633p*RSp45613e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(me2,L) * (  - 1./54.*pow(m33,2)*
         RN3p*RSp45633e*RSe45633p - 7./405.*m13*RSp45613e*RSe45613p + 7.
         /810.*m13*RSe45633p*RSp45613e + 7./810.*m13*RSp45633e*
         RSe45613p - 7./1620.*m13*RSp45633e*RSe45633p + 7./810.*m13*
         RSe45611p*RSp45613e - 7./1620.*m13*RSe45611p*RSp45633e + 7./
         810.*m13*RSp45611e*RSe45613p - 7./1620.*m13*RSp45611e*
         RSe45633p - 7./1620.*m13*RSp45611e*RSe45611p - 1./81.*m13*m66*
         RN3p*RSe456p*RSp453e*RS345pe + 1./81.*m13*m66*RN3p*RSe456p*
         RSp451e*RS345pe + 1./81.*m13*m66*RN1p*RSe456p*RS145pe*RSp453e
          - 1./81.*m13*m66*RN1p*RSe456p*RSp451e*RS145pe - 1./81.*m13*
         m55*RN3p*RSe465p*RSp463e*RS346pe + 1./81.*m13*m55*RN3p*RSe465p
         *RSp461e*RS346pe + 1./81.*m13*m55*RN1p*RSe465p*RS146pe*RSp463e
          - 1./81.*m13*m55*RN1p*RSe465p*RSp461e*RS146pe - 1./81.*m13*
         m44*RN3p*RSe564p*RSp563e*RS356pe + 1./81.*m13*m44*RN3p*RSe564p
         *RSp561e*RS356pe + 1./81.*m13*m44*RN1p*RSe564p*RS156pe*RSp563e
          - 1./81.*m13*m44*RN1p*RSe564p*RSp561e*RS156pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(me2,L) * (  - 1./27.*m13*m33*
         RN3p*RSe45633p*RSp45613e + 1./27.*m13*m33*RN3p*RSp45633e*
         RSe45633p + 23./405.*m11*RSp45613e*RSe45613p - 1./405.*m11*
         RSe45633p*RSp45613e - 1./405.*m11*RSp45633e*RSe45613p + 13./
         810.*m11*RSp45633e*RSe45633p + 8./405.*m11*RSe45611p*RSp45613e
          - 11./810.*m11*RSe45611p*RSp45633e + 8./405.*m11*RSp45611e*
         RSe45613p - 11./810.*m11*RSp45611e*RSe45633p - 2./81.*m11*
         RSp45611e*RSe45611p + 1./162.*m11*m66*RN3p*RSe456p*RSp453e*
         RS345pe + 1./162.*m11*m66*RN3p*RSe456p*RSp451e*RS345pe + 1./54.
         *m11*m66*RN1p*RSe456p*RS145pe*RSp453e + 1./162.*m11*m66*RN1p*
         RSe456p*RSp451e*RS145pe + 1./162.*m11*m55*RN3p*RSe465p*RSp463e
         *RS346pe + 1./162.*m11*m55*RN3p*RSe465p*RSp461e*RS346pe + 1./
         54.*m11*m55*RN1p*RSe465p*RS146pe*RSp463e + 1./162.*m11*m55*
         RN1p*RSe465p*RSp461e*RS146pe + 1./162.*m11*m44*RN3p*RSe564p*
         RSp563e*RS356pe + 1./162.*m11*m44*RN3p*RSe564p*RSp561e*RS356pe
          + 1./54.*m11*m44*RN1p*RSe564p*RS156pe*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(me2,L) * ( 1./162.*m11*m44*RN1p*
         RSe564p*RSp561e*RS156pe - 1./54.*m11*m33*RN3p*RSe45633p*
         RSp45613e - 1./54.*m11*m33*RN3p*RSp45633e*RSe45633p - 1./54.*
         m11*m33*RN1p*RSe45611p*RSp45613e - 1./54.*m11*m33*RN1p*
         RSp45611e*RSe45611p - 1./27.*m11*m13*RN1p*RSe45611p*RSp45613e
          + 1./27.*m11*m13*RN1p*RSp45611e*RSe45611p - 1./18.*pow(m11,2)
         *RN1p*RSe45611p*RSp45613e - 1./54.*pow(m11,2)*RN1p*RSp45611e*
         RSe45611p + 1./81.*RN(mp2,me2)*m66*me2*RSe456p*RSp453e*RSe453p
          - 1./81.*RN(mp2,me2)*m66*me2*RSe456p*RSe451p*RSp453e - 1./81.
         *RN(mp2,me2)*m66*me2*RSe456p*RSp451e*RSe453p + 1./81.*RN(mp2,
         me2)*m66*me2*RSe456p*RSp451e*RSe451p + 1./81.*RN(mp2,me2)*m55*
         me2*RSe465p*RSp463e*RSe463p - 1./81.*RN(mp2,me2)*m55*me2*
         RSe465p*RSe461p*RSp463e - 1./81.*RN(mp2,me2)*m55*me2*RSe465p*
         RSp461e*RSe463p + 1./81.*RN(mp2,me2)*m55*me2*RSe465p*RSp461e*
         RSe461p + 1./81.*RN(mp2,me2)*m44*me2*RSe564p*RSp563e*RSe563p
          - 1./81.*RN(mp2,me2)*m44*me2*RSe564p*RSe561p*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(me2,L) * (  - 1./81.*RN(mp2,me2)
         *m44*me2*RSe564p*RSp561e*RSe563p + 1./81.*RN(mp2,me2)*m44*me2*
         RSe564p*RSp561e*RSe561p - 1./54.*RN(mp2,me2)*m33*m66*RSe456p*
         RSp453e*RSe453p - 1./162.*RN(mp2,me2)*m33*m66*RSe456p*RSe451p*
         RSp453e - 1./162.*RN(mp2,me2)*m33*m66*RSe456p*RSp451e*RSe453p
          - 1./162.*RN(mp2,me2)*m33*m66*RSe456p*RSp451e*RSe451p - 1./54.
         *RN(mp2,me2)*m33*m55*RSe465p*RSp463e*RSe463p - 1./162.*RN(mp2,
         me2)*m33*m55*RSe465p*RSe461p*RSp463e - 1./162.*RN(mp2,me2)*m33
         *m55*RSe465p*RSp461e*RSe463p - 1./162.*RN(mp2,me2)*m33*m55*
         RSe465p*RSp461e*RSe461p - 1./54.*RN(mp2,me2)*m33*m44*RSe564p*
         RSp563e*RSe563p - 1./162.*RN(mp2,me2)*m33*m44*RSe564p*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m33*m44*RSe564p*RSp561e*RSe563p
          - 1./162.*RN(mp2,me2)*m33*m44*RSe564p*RSp561e*RSe561p + 1./81.
         *RN(mp2,me2)*m13*m66*RSe456p*RSp453e*RSe453p - 1./81.*RN(mp2,
         me2)*m13*m66*RSe456p*RSe451p*RSp453e - 1./81.*RN(mp2,me2)*m13*
         m66*RSe456p*RSp451e*RSe453p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(me2,L) * ( 1./81.*RN(mp2,me2)*
         m13*m66*RSe456p*RSp451e*RSe451p + 1./81.*RN(mp2,me2)*m13*m55*
         RSe465p*RSp463e*RSe463p - 1./81.*RN(mp2,me2)*m13*m55*RSe465p*
         RSe461p*RSp463e - 1./81.*RN(mp2,me2)*m13*m55*RSe465p*RSp461e*
         RSe463p + 1./81.*RN(mp2,me2)*m13*m55*RSe465p*RSp461e*RSe461p
          + 1./81.*RN(mp2,me2)*m13*m44*RSe564p*RSp563e*RSe563p - 1./81.
         *RN(mp2,me2)*m13*m44*RSe564p*RSe561p*RSp563e - 1./81.*RN(mp2,
         me2)*m13*m44*RSe564p*RSp561e*RSe563p + 1./81.*RN(mp2,me2)*m13*
         m44*RSe564p*RSp561e*RSe561p - 1./162.*RN(mp2,me2)*m11*m66*
         RSe456p*RSp453e*RSe453p - 1./162.*RN(mp2,me2)*m11*m66*RSe456p*
         RSe451p*RSp453e - 1./162.*RN(mp2,me2)*m11*m66*RSe456p*RSp451e*
         RSe453p - 1./54.*RN(mp2,me2)*m11*m66*RSe456p*RSp451e*RSe451p
          - 1./162.*RN(mp2,me2)*m11*m55*RSe465p*RSp463e*RSe463p - 1./
         162.*RN(mp2,me2)*m11*m55*RSe465p*RSe461p*RSp463e - 1./162.*RN(
         mp2,me2)*m11*m55*RSe465p*RSp461e*RSe463p - 1./54.*RN(mp2,me2)*
         m11*m55*RSe465p*RSp461e*RSe461p );

      M13p6x23RV +=  + Ab(mp2,mu2)*AbV(me2,L) * (  - 1./162.*RN(mp2,me2
         )*m11*m44*RSe564p*RSp563e*RSe563p - 1./162.*RN(mp2,me2)*m11*
         m44*RSe564p*RSe561p*RSp563e - 1./162.*RN(mp2,me2)*m11*m44*
         RSe564p*RSp561e*RSe563p - 1./54.*RN(mp2,me2)*m11*m44*RSe564p*
         RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m11,m33,L) * (  - 1./162.*m33*
         m66*RSp456e*RS145pe*RS345pe - 1./162.*m33*m55*RSp465e*RS146pe*
         RS346pe - 1./162.*m33*m44*RSp564e*RS156pe*RS356pe + 1./54.*
         pow(m33,2)*RSp45633e*RS14563pe - 1./81.*m13*m66*RSp456e*
         RS145pe*RS345pe - 1./81.*m13*m55*RSp465e*RS146pe*RS346pe - 1./
         81.*m13*m44*RSp564e*RS156pe*RS356pe + 1./27.*m13*m33*RSp45633e
         *RS14563pe - 1./54.*m11*m66*RSp456e*RS145pe*RS345pe - 1./54.*
         m11*m55*RSp465e*RS146pe*RS346pe - 1./54.*m11*m44*RSp564e*
         RS156pe*RS356pe + 1./18.*m11*m33*RSp45633e*RS14563pe + 1./54.*
         m11*m33*RSp45611e*RS34561pe + 1./27.*m11*m13*RSp45611e*
         RS34561pe + 1./18.*pow(m11,2)*RSp45611e*RS34561pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m11,mp2,L) * (  - 1./162.*m33*
         m66*RSp456e*RS145pe*RSp453e - 1./162.*m33*m66*RSp456e*RSp451e*
         RS145pe - 1./162.*m33*m55*RSp465e*RS146pe*RSp463e - 1./162.*
         m33*m55*RSp465e*RSp461e*RS146pe - 1./162.*m33*m44*RSp564e*
         RS156pe*RSp563e - 1./162.*m33*m44*RSp564e*RSp561e*RS156pe - 1./
         81.*m13*m66*RSp456e*RS145pe*RSp453e + 1./81.*m13*m66*RSp456e*
         RSp451e*RS145pe - 1./81.*m13*m55*RSp465e*RS146pe*RSp463e + 1./
         81.*m13*m55*RSp465e*RSp461e*RS146pe - 1./81.*m13*m44*RSp564e*
         RS156pe*RSp563e + 1./81.*m13*m44*RSp564e*RSp561e*RS156pe - 1./
         54.*m11*m66*RSp456e*RS145pe*RSp453e - 1./162.*m11*m66*RSp456e*
         RSp451e*RS145pe - 1./54.*m11*m55*RSp465e*RS146pe*RSp463e - 1./
         162.*m11*m55*RSp465e*RSp461e*RS146pe - 1./54.*m11*m44*RSp564e*
         RS156pe*RSp563e - 1./162.*m11*m44*RSp564e*RSp561e*RS156pe + 1./
         54.*m11*m33*RSp45611e*RSp45613e + 1./54.*m11*m33*pow(
         RSp45611e,2) + 1./27.*m11*m13*RSp45611e*RSp45613e - 1./27.*m11
         *m13*pow(RSp45611e,2) );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m11,mp2,L) * ( 1./18.*pow(m11,2)
         *RSp45611e*RSp45613e + 1./54.*pow(m11,2)*pow(RSp45611e,2) );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m11,me2,L) * (  - 1./162.*m33*
         m66*RSp456e*RS145pe*RSe453p - 1./162.*m33*m66*RSp456e*RSe451p*
         RS145pe - 1./162.*m33*m55*RSp465e*RS146pe*RSe463p - 1./162.*
         m33*m55*RSp465e*RSe461p*RS146pe - 1./162.*m33*m44*RSp564e*
         RS156pe*RSe563p - 1./162.*m33*m44*RSp564e*RSe561p*RS156pe - 1./
         81.*m13*m66*RSp456e*RS145pe*RSe453p + 1./81.*m13*m66*RSp456e*
         RSe451p*RS145pe - 1./81.*m13*m55*RSp465e*RS146pe*RSe463p + 1./
         81.*m13*m55*RSp465e*RSe461p*RS146pe - 1./81.*m13*m44*RSp564e*
         RS156pe*RSe563p + 1./81.*m13*m44*RSp564e*RSe561p*RS156pe - 1./
         54.*m11*m66*RSp456e*RS145pe*RSe453p - 1./162.*m11*m66*RSp456e*
         RSe451p*RS145pe - 1./54.*m11*m55*RSp465e*RS146pe*RSe463p - 1./
         162.*m11*m55*RSp465e*RSe461p*RS146pe - 1./54.*m11*m44*RSp564e*
         RS156pe*RSe563p - 1./162.*m11*m44*RSp564e*RSe561p*RS156pe + 1./
         54.*m11*m33*RSp45611e*RSe45613p + 1./54.*m11*m33*RSp45611e*
         RSe45611p + 1./27.*m11*m13*RSp45611e*RSe45613p - 1./27.*m11*
         m13*RSp45611e*RSe45611p );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m11,me2,L) * ( 1./18.*pow(m11,2)
         *RSp45611e*RSe45613p + 1./54.*pow(m11,2)*RSp45611e*RSe45611p )
         ;

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m11,L) * (  - 1./270.*mp2*
         RD1456ep*RSp45613e + 1./540.*mp2*RD1456ep*RSp45633e + 1./540.*
         mp2*RSp45611e*RD1456ep - 1./405.*m33*RD1456ep*RSp45613e - 11./
         810.*m33*RD1456ep*RSp45633e + 13./810.*m33*RSp45611e*RD1456ep
          - 1./324.*m33*m66*RSp456e*pow(RS145pe,2) - 1./324.*m33*m55*
         RSp465e*pow(RS146pe,2) - 1./324.*m33*m44*RSp564e*pow(
         RS156pe,2) + 7./810.*m13*RD1456ep*RSp45613e - 7./1620.*m13*
         RD1456ep*RSp45633e - 7./1620.*m13*RSp45611e*RD1456ep + 1./162.
         *m13*m66*RSp456e*pow(RS145pe,2) + 1./162.*m13*m55*RSp465e*pow(
         RS146pe,2) + 1./162.*m13*m44*RSp564e*pow(RS156pe,2) + 13./810.
         *m11*RD1456ep*RSp45613e - 19./1620.*m11*RD1456ep*RSp45633e - 
         37./1620.*m11*RSp45611e*RD1456ep - 1./324.*m11*m66*RSp456e*
         pow(RS145pe,2) - 1./324.*m11*m55*RSp465e*pow(RS146pe,2) - 1./
         324.*m11*m44*RSp564e*pow(RS156pe,2) + 1./54.*m11*m33*RSp45611e
         *RS14563pe + 1./54.*m11*m33*RSp45611e*RC1456pe - 1./54.*m11*
         m33*RN1p*RD1456ep*RSp45613e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m11,L) * (  - 1./54.*m11*m33*
         RN1p*RSp45611e*RD1456ep + 1./27.*m11*m13*RSp45611e*RS14563pe
          - 1./27.*m11*m13*RSp45611e*RC1456pe - 1./27.*m11*m13*RN1p*
         RD1456ep*RSp45613e + 1./27.*m11*m13*RN1p*RSp45611e*RD1456ep + 
         1./18.*pow(m11,2)*RSp45611e*RS14563pe + 1./54.*pow(m11,2)*
         RSp45611e*RC1456pe - 1./18.*pow(m11,2)*RN1p*RD1456ep*RSp45613e
          - 1./54.*pow(m11,2)*RN1p*RSp45611e*RD1456ep );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m14,L) * ( 1./216.*m44*mp2*
         RSp45611e - 1./108.*m44*mp2*RSp561e + 1./216.*m44*mp2*RSp564e
          - 1./432.*pow(m44,2)*RSp45611e - 1./216.*pow(m44,2)*RSp561e
          - 1./144.*pow(m44,2)*RSp564e + 1./216.*m33*mp2*RSp45611e - 1./
         108.*m33*mp2*RSp561e + 1./216.*m33*mp2*RSp564e - 1./432.*m33*
         m44*RSp45611e - 1./216.*m33*m44*RSp561e - 1./144.*m33*m44*
         RSp564e - 1./108.*m14*mp2*RSp45611e + 1./54.*m14*mp2*RSp561e
          - 1./108.*m14*mp2*RSp564e + 1./108.*m14*m44*RSp45611e + 1./54.
         *m14*m44*RSp564e + 1./216.*m14*m33*RSp45611e - 1./108.*m14*m33
         *RSp561e + 1./216.*m14*m33*RSp564e - 1./108.*pow(m14,2)*
         RSp45611e + 1./54.*pow(m14,2)*RSp561e - 1./108.*pow(m14,2)*
         RSp564e - 1./108.*m13*mp2*RSp45611e + 1./54.*m13*mp2*RSp561e
          - 1./108.*m13*mp2*RSp564e + 1./216.*m13*m44*RSp45611e + 1./
         108.*m13*m44*RSp561e + 1./72.*m13*m44*RSp564e - 1./108.*m13*
         m14*RSp45611e + 1./54.*m13*m14*RSp561e - 1./108.*m13*m14*
         RSp564e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m14,L) * ( 1./108.*m11*mp2*
         RSp45611e - 1./54.*m11*mp2*RSp561e + 1./108.*m11*mp2*RSp564e
          - 5./432.*m11*m44*RSp45611e - 1./72.*m11*m44*RSp561e - 7./432.
         *m11*m44*RSp564e - 1./144.*m11*m33*RSp45611e - 1./216.*m11*m33
         *RSp561e - 1./432.*m11*m33*RSp564e + 5./216.*m11*m14*RSp45611e
          - 1./108.*m11*m14*RSp561e + 1./72.*m11*m14*RSp564e + 1./72.*
         m11*m13*RSp45611e + 1./108.*m11*m13*RSp561e + 1./216.*m11*m13*
         RSp564e - 1./72.*pow(m11,2)*RSp45611e - 1./108.*pow(m11,2)*
         RSp561e - 1./216.*pow(m11,2)*RSp564e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m15,L) * ( 1./216.*m55*mp2*
         RSp45611e - 1./108.*m55*mp2*RSp461e + 1./216.*m55*mp2*RSp465e
          - 1./432.*pow(m55,2)*RSp45611e - 1./216.*pow(m55,2)*RSp461e
          - 1./144.*pow(m55,2)*RSp465e + 1./216.*m33*mp2*RSp45611e - 1./
         108.*m33*mp2*RSp461e + 1./216.*m33*mp2*RSp465e - 1./432.*m33*
         m55*RSp45611e - 1./216.*m33*m55*RSp461e - 1./144.*m33*m55*
         RSp465e - 1./108.*m15*mp2*RSp45611e + 1./54.*m15*mp2*RSp461e
          - 1./108.*m15*mp2*RSp465e + 1./108.*m15*m55*RSp45611e + 1./54.
         *m15*m55*RSp465e + 1./216.*m15*m33*RSp45611e - 1./108.*m15*m33
         *RSp461e + 1./216.*m15*m33*RSp465e - 1./108.*pow(m15,2)*
         RSp45611e + 1./54.*pow(m15,2)*RSp461e - 1./108.*pow(m15,2)*
         RSp465e - 1./108.*m13*mp2*RSp45611e + 1./54.*m13*mp2*RSp461e
          - 1./108.*m13*mp2*RSp465e + 1./216.*m13*m55*RSp45611e + 1./
         108.*m13*m55*RSp461e + 1./72.*m13*m55*RSp465e - 1./108.*m13*
         m15*RSp45611e + 1./54.*m13*m15*RSp461e - 1./108.*m13*m15*
         RSp465e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m15,L) * ( 1./108.*m11*mp2*
         RSp45611e - 1./54.*m11*mp2*RSp461e + 1./108.*m11*mp2*RSp465e
          - 5./432.*m11*m55*RSp45611e - 1./72.*m11*m55*RSp461e - 7./432.
         *m11*m55*RSp465e - 1./144.*m11*m33*RSp45611e - 1./216.*m11*m33
         *RSp461e - 1./432.*m11*m33*RSp465e + 5./216.*m11*m15*RSp45611e
          - 1./108.*m11*m15*RSp461e + 1./72.*m11*m15*RSp465e + 1./72.*
         m11*m13*RSp45611e + 1./108.*m11*m13*RSp461e + 1./216.*m11*m13*
         RSp465e - 1./72.*pow(m11,2)*RSp45611e - 1./108.*pow(m11,2)*
         RSp461e - 1./216.*pow(m11,2)*RSp465e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m16,L) * ( 1./216.*m66*mp2*
         RSp45611e - 1./108.*m66*mp2*RSp451e + 1./216.*m66*mp2*RSp456e
          - 1./432.*pow(m66,2)*RSp45611e - 1./216.*pow(m66,2)*RSp451e
          - 1./144.*pow(m66,2)*RSp456e + 1./216.*m33*mp2*RSp45611e - 1./
         108.*m33*mp2*RSp451e + 1./216.*m33*mp2*RSp456e - 1./432.*m33*
         m66*RSp45611e - 1./216.*m33*m66*RSp451e - 1./144.*m33*m66*
         RSp456e - 1./108.*m16*mp2*RSp45611e + 1./54.*m16*mp2*RSp451e
          - 1./108.*m16*mp2*RSp456e + 1./108.*m16*m66*RSp45611e + 1./54.
         *m16*m66*RSp456e + 1./216.*m16*m33*RSp45611e - 1./108.*m16*m33
         *RSp451e + 1./216.*m16*m33*RSp456e - 1./108.*pow(m16,2)*
         RSp45611e + 1./54.*pow(m16,2)*RSp451e - 1./108.*pow(m16,2)*
         RSp456e - 1./108.*m13*mp2*RSp45611e + 1./54.*m13*mp2*RSp451e
          - 1./108.*m13*mp2*RSp456e + 1./216.*m13*m66*RSp45611e + 1./
         108.*m13*m66*RSp451e + 1./72.*m13*m66*RSp456e - 1./108.*m13*
         m16*RSp45611e + 1./54.*m13*m16*RSp451e - 1./108.*m13*m16*
         RSp456e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m16,L) * ( 1./108.*m11*mp2*
         RSp45611e - 1./54.*m11*mp2*RSp451e + 1./108.*m11*mp2*RSp456e
          - 5./432.*m11*m66*RSp45611e - 1./72.*m11*m66*RSp451e - 7./432.
         *m11*m66*RSp456e - 1./144.*m11*m33*RSp45611e - 1./216.*m11*m33
         *RSp451e - 1./432.*m11*m33*RSp456e + 5./216.*m11*m16*RSp45611e
          - 1./108.*m11*m16*RSp451e + 1./72.*m11*m16*RSp456e + 1./72.*
         m11*m13*RSp45611e + 1./108.*m11*m13*RSp451e + 1./216.*m11*m13*
         RSp456e - 1./72.*pow(m11,2)*RSp45611e - 1./108.*pow(m11,2)*
         RSp451e - 1./216.*pow(m11,2)*RSp456e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m33,mp2,L) * (  - 1./162.*m33*
         m66*RSp456e*RSp453e*RS345pe - 1./54.*m33*m66*RSp456e*RSp451e*
         RS345pe - 1./162.*m33*m55*RSp465e*RSp463e*RS346pe - 1./54.*m33
         *m55*RSp465e*RSp461e*RS346pe - 1./162.*m33*m44*RSp564e*RSp563e
         *RS356pe - 1./54.*m33*m44*RSp564e*RSp561e*RS356pe + 1./18.*
         pow(m33,2)*RSp45633e*RSp45613e + 1./54.*pow(m33,2)*pow(
         RSp45633e,2) + 1./81.*m13*m66*RSp456e*RSp453e*RS345pe - 1./81.
         *m13*m66*RSp456e*RSp451e*RS345pe + 1./81.*m13*m55*RSp465e*
         RSp463e*RS346pe - 1./81.*m13*m55*RSp465e*RSp461e*RS346pe + 1./
         81.*m13*m44*RSp564e*RSp563e*RS356pe - 1./81.*m13*m44*RSp564e*
         RSp561e*RS356pe + 1./27.*m13*m33*RSp45633e*RSp45613e - 1./27.*
         m13*m33*pow(RSp45633e,2) - 1./162.*m11*m66*RSp456e*RSp453e*
         RS345pe - 1./162.*m11*m66*RSp456e*RSp451e*RS345pe - 1./162.*
         m11*m55*RSp465e*RSp463e*RS346pe - 1./162.*m11*m55*RSp465e*
         RSp461e*RS346pe - 1./162.*m11*m44*RSp564e*RSp563e*RS356pe - 1./
         162.*m11*m44*RSp564e*RSp561e*RS356pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m33,mp2,L) * ( 1./54.*m11*m33*
         RSp45633e*RSp45613e + 1./54.*m11*m33*pow(RSp45633e,2) );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m33,me2,L) * (  - 1./162.*m33*
         m66*RSp456e*RSe453p*RS345pe - 1./54.*m33*m66*RSp456e*RSe451p*
         RS345pe - 1./162.*m33*m55*RSp465e*RSe463p*RS346pe - 1./54.*m33
         *m55*RSp465e*RSe461p*RS346pe - 1./162.*m33*m44*RSp564e*RSe563p
         *RS356pe - 1./54.*m33*m44*RSp564e*RSe561p*RS356pe + 1./18.*
         pow(m33,2)*RSp45633e*RSe45613p + 1./54.*pow(m33,2)*RSp45633e*
         RSe45633p + 1./81.*m13*m66*RSp456e*RSe453p*RS345pe - 1./81.*
         m13*m66*RSp456e*RSe451p*RS345pe + 1./81.*m13*m55*RSp465e*
         RSe463p*RS346pe - 1./81.*m13*m55*RSp465e*RSe461p*RS346pe + 1./
         81.*m13*m44*RSp564e*RSe563p*RS356pe - 1./81.*m13*m44*RSp564e*
         RSe561p*RS356pe + 1./27.*m13*m33*RSp45633e*RSe45613p - 1./27.*
         m13*m33*RSp45633e*RSe45633p - 1./162.*m11*m66*RSp456e*RSe453p*
         RS345pe - 1./162.*m11*m66*RSp456e*RSe451p*RS345pe - 1./162.*
         m11*m55*RSp465e*RSe463p*RS346pe - 1./162.*m11*m55*RSp465e*
         RSe461p*RS346pe - 1./162.*m11*m44*RSp564e*RSe563p*RS356pe - 1./
         162.*m11*m44*RSp564e*RSe561p*RS356pe );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m33,me2,L) * ( 1./54.*m11*m33*
         RSp45633e*RSe45613p + 1./54.*m11*m33*RSp45633e*RSe45633p );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m33,L) * (  - 1./270.*mp2*
         RD3456ep*RSp45613e + 1./540.*mp2*RSp45633e*RD3456ep + 1./540.*
         mp2*RSp45611e*RD3456ep + 13./810.*m33*RD3456ep*RSp45613e - 37./
         1620.*m33*RSp45633e*RD3456ep - 19./1620.*m33*RSp45611e*
         RD3456ep - 1./324.*m33*m66*RSp456e*pow(RS345pe,2) - 1./324.*
         m33*m55*RSp465e*pow(RS346pe,2) - 1./324.*m33*m44*RSp564e*pow(
         RS356pe,2) + 1./18.*pow(m33,2)*RSp45633e*RS34561pe + 1./54.*
         pow(m33,2)*RSp45633e*RC3456pe - 1./18.*pow(m33,2)*RN3p*
         RD3456ep*RSp45613e - 1./54.*pow(m33,2)*RN3p*RSp45633e*RD3456ep
          + 7./810.*m13*RD3456ep*RSp45613e - 7./1620.*m13*RSp45633e*
         RD3456ep - 7./1620.*m13*RSp45611e*RD3456ep + 1./162.*m13*m66*
         RSp456e*pow(RS345pe,2) + 1./162.*m13*m55*RSp465e*pow(
         RS346pe,2) + 1./162.*m13*m44*RSp564e*pow(RS356pe,2) + 1./27.*
         m13*m33*RSp45633e*RS34561pe - 1./27.*m13*m33*RSp45633e*
         RC3456pe - 1./27.*m13*m33*RN3p*RD3456ep*RSp45613e + 1./27.*m13
         *m33*RN3p*RSp45633e*RD3456ep );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m33,L) * (  - 1./405.*m11*
         RD3456ep*RSp45613e + 13./810.*m11*RSp45633e*RD3456ep - 11./810.
         *m11*RSp45611e*RD3456ep - 1./324.*m11*m66*RSp456e*pow(
         RS345pe,2) - 1./324.*m11*m55*RSp465e*pow(RS346pe,2) - 1./324.*
         m11*m44*RSp564e*pow(RS356pe,2) + 1./54.*m11*m33*RSp45633e*
         RS34561pe + 1./54.*m11*m33*RSp45633e*RC3456pe - 1./54.*m11*m33
         *RN3p*RD3456ep*RSp45613e - 1./54.*m11*m33*RN3p*RSp45633e*
         RD3456ep );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m34,L) * ( 1./216.*m44*mp2*
         RSp45633e - 1./108.*m44*mp2*RSp563e + 1./216.*m44*mp2*RSp564e
          - 1./432.*pow(m44,2)*RSp45633e - 1./216.*pow(m44,2)*RSp563e
          - 1./144.*pow(m44,2)*RSp564e - 1./108.*m34*mp2*RSp45633e + 1./
         54.*m34*mp2*RSp563e - 1./108.*m34*mp2*RSp564e + 1./108.*m34*
         m44*RSp45633e + 1./54.*m34*m44*RSp564e - 1./108.*pow(m34,2)*
         RSp45633e + 1./54.*pow(m34,2)*RSp563e - 1./108.*pow(m34,2)*
         RSp564e + 1./108.*m33*mp2*RSp45633e - 1./54.*m33*mp2*RSp563e
          + 1./108.*m33*mp2*RSp564e - 5./432.*m33*m44*RSp45633e - 1./72.
         *m33*m44*RSp563e - 7./432.*m33*m44*RSp564e + 5./216.*m33*m34*
         RSp45633e - 1./108.*m33*m34*RSp563e + 1./72.*m33*m34*RSp564e
          - 1./72.*pow(m33,2)*RSp45633e - 1./108.*pow(m33,2)*RSp563e - 
         1./216.*pow(m33,2)*RSp564e - 1./108.*m13*mp2*RSp45633e + 1./54.
         *m13*mp2*RSp563e - 1./108.*m13*mp2*RSp564e + 1./216.*m13*m44*
         RSp45633e + 1./108.*m13*m44*RSp563e + 1./72.*m13*m44*RSp564e
          - 1./108.*m13*m34*RSp45633e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m34,L) * ( 1./54.*m13*m34*
         RSp563e - 1./108.*m13*m34*RSp564e + 1./72.*m13*m33*RSp45633e
          + 1./108.*m13*m33*RSp563e + 1./216.*m13*m33*RSp564e + 1./216.
         *m11*mp2*RSp45633e - 1./108.*m11*mp2*RSp563e + 1./216.*m11*mp2
         *RSp564e - 1./432.*m11*m44*RSp45633e - 1./216.*m11*m44*RSp563e
          - 1./144.*m11*m44*RSp564e + 1./216.*m11*m34*RSp45633e - 1./
         108.*m11*m34*RSp563e + 1./216.*m11*m34*RSp564e - 1./144.*m11*
         m33*RSp45633e - 1./216.*m11*m33*RSp563e - 1./432.*m11*m33*
         RSp564e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m35,L) * ( 1./216.*m55*mp2*
         RSp45633e - 1./108.*m55*mp2*RSp463e + 1./216.*m55*mp2*RSp465e
          - 1./432.*pow(m55,2)*RSp45633e - 1./216.*pow(m55,2)*RSp463e
          - 1./144.*pow(m55,2)*RSp465e - 1./108.*m35*mp2*RSp45633e + 1./
         54.*m35*mp2*RSp463e - 1./108.*m35*mp2*RSp465e + 1./108.*m35*
         m55*RSp45633e + 1./54.*m35*m55*RSp465e - 1./108.*pow(m35,2)*
         RSp45633e + 1./54.*pow(m35,2)*RSp463e - 1./108.*pow(m35,2)*
         RSp465e + 1./108.*m33*mp2*RSp45633e - 1./54.*m33*mp2*RSp463e
          + 1./108.*m33*mp2*RSp465e - 5./432.*m33*m55*RSp45633e - 1./72.
         *m33*m55*RSp463e - 7./432.*m33*m55*RSp465e + 5./216.*m33*m35*
         RSp45633e - 1./108.*m33*m35*RSp463e + 1./72.*m33*m35*RSp465e
          - 1./72.*pow(m33,2)*RSp45633e - 1./108.*pow(m33,2)*RSp463e - 
         1./216.*pow(m33,2)*RSp465e - 1./108.*m13*mp2*RSp45633e + 1./54.
         *m13*mp2*RSp463e - 1./108.*m13*mp2*RSp465e + 1./216.*m13*m55*
         RSp45633e + 1./108.*m13*m55*RSp463e + 1./72.*m13*m55*RSp465e
          - 1./108.*m13*m35*RSp45633e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m35,L) * ( 1./54.*m13*m35*
         RSp463e - 1./108.*m13*m35*RSp465e + 1./72.*m13*m33*RSp45633e
          + 1./108.*m13*m33*RSp463e + 1./216.*m13*m33*RSp465e + 1./216.
         *m11*mp2*RSp45633e - 1./108.*m11*mp2*RSp463e + 1./216.*m11*mp2
         *RSp465e - 1./432.*m11*m55*RSp45633e - 1./216.*m11*m55*RSp463e
          - 1./144.*m11*m55*RSp465e + 1./216.*m11*m35*RSp45633e - 1./
         108.*m11*m35*RSp463e + 1./216.*m11*m35*RSp465e - 1./144.*m11*
         m33*RSp45633e - 1./216.*m11*m33*RSp463e - 1./432.*m11*m33*
         RSp465e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m36,L) * ( 1./216.*m66*mp2*
         RSp45633e - 1./108.*m66*mp2*RSp453e + 1./216.*m66*mp2*RSp456e
          - 1./432.*pow(m66,2)*RSp45633e - 1./216.*pow(m66,2)*RSp453e
          - 1./144.*pow(m66,2)*RSp456e - 1./108.*m36*mp2*RSp45633e + 1./
         54.*m36*mp2*RSp453e - 1./108.*m36*mp2*RSp456e + 1./108.*m36*
         m66*RSp45633e + 1./54.*m36*m66*RSp456e - 1./108.*pow(m36,2)*
         RSp45633e + 1./54.*pow(m36,2)*RSp453e - 1./108.*pow(m36,2)*
         RSp456e + 1./108.*m33*mp2*RSp45633e - 1./54.*m33*mp2*RSp453e
          + 1./108.*m33*mp2*RSp456e - 5./432.*m33*m66*RSp45633e - 1./72.
         *m33*m66*RSp453e - 7./432.*m33*m66*RSp456e + 5./216.*m33*m36*
         RSp45633e - 1./108.*m33*m36*RSp453e + 1./72.*m33*m36*RSp456e
          - 1./72.*pow(m33,2)*RSp45633e - 1./108.*pow(m33,2)*RSp453e - 
         1./216.*pow(m33,2)*RSp456e - 1./108.*m13*mp2*RSp45633e + 1./54.
         *m13*mp2*RSp453e - 1./108.*m13*mp2*RSp456e + 1./216.*m13*m66*
         RSp45633e + 1./108.*m13*m66*RSp453e + 1./72.*m13*m66*RSp456e
          - 1./108.*m13*m36*RSp45633e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(m36,L) * ( 1./54.*m13*m36*
         RSp453e - 1./108.*m13*m36*RSp456e + 1./72.*m13*m33*RSp45633e
          + 1./108.*m13*m33*RSp453e + 1./216.*m13*m33*RSp456e + 1./216.
         *m11*mp2*RSp45633e - 1./108.*m11*mp2*RSp453e + 1./216.*m11*mp2
         *RSp456e - 1./432.*m11*m66*RSp45633e - 1./216.*m11*m66*RSp453e
          - 1./144.*m11*m66*RSp456e + 1./216.*m11*m36*RSp45633e - 1./
         108.*m11*m36*RSp453e + 1./216.*m11*m36*RSp456e - 1./144.*m11*
         m33*RSp45633e - 1./216.*m11*m33*RSp453e - 1./432.*m11*m33*
         RSp456e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(mp2,L) * ( 1./162.*m66*mp2*
         RSp456e*pow(RSp453e,2) - 1./81.*m66*mp2*RSp456e*RSp451e*
         RSp453e + 1./162.*m66*mp2*RSp456e*pow(RSp451e,2) + 1./162.*m55
         *mp2*RSp465e*pow(RSp463e,2) - 1./81.*m55*mp2*RSp465e*RSp461e*
         RSp463e + 1./162.*m55*mp2*RSp465e*pow(RSp461e,2) + 1./162.*m44
         *mp2*RSp564e*pow(RSp563e,2) - 1./81.*m44*mp2*RSp564e*RSp561e*
         RSp563e + 1./162.*m44*mp2*RSp564e*pow(RSp561e,2) - 1./108.*m33
         *m66*RSp456e*pow(RSp453e,2) - 1./162.*m33*m66*RSp456e*RSp451e*
         RSp453e - 1./324.*m33*m66*RSp456e*pow(RSp451e,2) - 1./108.*m33
         *m55*RSp465e*pow(RSp463e,2) - 1./162.*m33*m55*RSp465e*RSp461e*
         RSp463e - 1./324.*m33*m55*RSp465e*pow(RSp461e,2) - 1./108.*m33
         *m44*RSp564e*pow(RSp563e,2) - 1./162.*m33*m44*RSp564e*RSp561e*
         RSp563e - 1./324.*m33*m44*RSp564e*pow(RSp561e,2) + 1./162.*m13
         *m66*RSp456e*pow(RSp453e,2) - 1./81.*m13*m66*RSp456e*RSp451e*
         RSp453e + 1./162.*m13*m66*RSp456e*pow(RSp451e,2) + 1./162.*m13
         *m55*RSp465e*pow(RSp463e,2) );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(mp2,L) * (  - 1./81.*m13*m55*
         RSp465e*RSp461e*RSp463e + 1./162.*m13*m55*RSp465e*pow(
         RSp461e,2) + 1./162.*m13*m44*RSp564e*pow(RSp563e,2) - 1./81.*
         m13*m44*RSp564e*RSp561e*RSp563e + 1./162.*m13*m44*RSp564e*pow(
         RSp561e,2) - 1./324.*m11*m66*RSp456e*pow(RSp453e,2) - 1./162.*
         m11*m66*RSp456e*RSp451e*RSp453e - 1./108.*m11*m66*RSp456e*pow(
         RSp451e,2) - 1./324.*m11*m55*RSp465e*pow(RSp463e,2) - 1./162.*
         m11*m55*RSp465e*RSp461e*RSp463e - 1./108.*m11*m55*RSp465e*pow(
         RSp461e,2) - 1./324.*m11*m44*RSp564e*pow(RSp563e,2) - 1./162.*
         m11*m44*RSp564e*RSp561e*RSp563e - 1./108.*m11*m44*RSp564e*pow(
         RSp561e,2) );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(me2,mp2,L) * ( 1./81.*m66*me2*
         RSp456e*RSp453e*RSe453p - 1./81.*m66*me2*RSp456e*RSe451p*
         RSp453e - 1./81.*m66*me2*RSp456e*RSp451e*RSe453p + 1./81.*m66*
         me2*RSp456e*RSp451e*RSe451p + 1./81.*m55*me2*RSp465e*RSp463e*
         RSe463p - 1./81.*m55*me2*RSp465e*RSe461p*RSp463e - 1./81.*m55*
         me2*RSp465e*RSp461e*RSe463p + 1./81.*m55*me2*RSp465e*RSp461e*
         RSe461p + 1./81.*m44*me2*RSp564e*RSp563e*RSe563p - 1./81.*m44*
         me2*RSp564e*RSe561p*RSp563e - 1./81.*m44*me2*RSp564e*RSp561e*
         RSe563p + 1./81.*m44*me2*RSp564e*RSp561e*RSe561p - 1./54.*m33*
         m66*RSp456e*RSp453e*RSe453p - 1./162.*m33*m66*RSp456e*RSe451p*
         RSp453e - 1./162.*m33*m66*RSp456e*RSp451e*RSe453p - 1./162.*
         m33*m66*RSp456e*RSp451e*RSe451p - 1./54.*m33*m55*RSp465e*
         RSp463e*RSe463p - 1./162.*m33*m55*RSp465e*RSe461p*RSp463e - 1./
         162.*m33*m55*RSp465e*RSp461e*RSe463p - 1./162.*m33*m55*RSp465e
         *RSp461e*RSe461p - 1./54.*m33*m44*RSp564e*RSp563e*RSe563p - 1./
         162.*m33*m44*RSp564e*RSe561p*RSp563e );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(me2,mp2,L) * (  - 1./162.*m33*
         m44*RSp564e*RSp561e*RSe563p - 1./162.*m33*m44*RSp564e*RSp561e*
         RSe561p + 1./81.*m13*m66*RSp456e*RSp453e*RSe453p - 1./81.*m13*
         m66*RSp456e*RSe451p*RSp453e - 1./81.*m13*m66*RSp456e*RSp451e*
         RSe453p + 1./81.*m13*m66*RSp456e*RSp451e*RSe451p + 1./81.*m13*
         m55*RSp465e*RSp463e*RSe463p - 1./81.*m13*m55*RSp465e*RSe461p*
         RSp463e - 1./81.*m13*m55*RSp465e*RSp461e*RSe463p + 1./81.*m13*
         m55*RSp465e*RSp461e*RSe461p + 1./81.*m13*m44*RSp564e*RSp563e*
         RSe563p - 1./81.*m13*m44*RSp564e*RSe561p*RSp563e - 1./81.*m13*
         m44*RSp564e*RSp561e*RSe563p + 1./81.*m13*m44*RSp564e*RSp561e*
         RSe561p - 1./162.*m11*m66*RSp456e*RSp453e*RSe453p - 1./162.*
         m11*m66*RSp456e*RSe451p*RSp453e - 1./162.*m11*m66*RSp456e*
         RSp451e*RSe453p - 1./54.*m11*m66*RSp456e*RSp451e*RSe451p - 1./
         162.*m11*m55*RSp465e*RSp463e*RSe463p - 1./162.*m11*m55*RSp465e
         *RSe461p*RSp463e - 1./162.*m11*m55*RSp465e*RSp461e*RSe463p - 1.
         /54.*m11*m55*RSp465e*RSp461e*RSe461p );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(me2,mp2,L) * (  - 1./162.*m11*
         m44*RSp564e*RSp563e*RSe563p - 1./162.*m11*m44*RSp564e*RSe561p*
         RSp563e - 1./162.*m11*m44*RSp564e*RSp561e*RSe563p - 1./54.*m11
         *m44*RSp564e*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(me2,L) * ( 1./162.*m66*me2*
         RSp456e*pow(RSe453p,2) - 1./81.*m66*me2*RSp456e*RSe451p*
         RSe453p + 1./162.*m66*me2*RSp456e*pow(RSe451p,2) + 1./162.*m55
         *me2*RSp465e*pow(RSe463p,2) - 1./81.*m55*me2*RSp465e*RSe461p*
         RSe463p + 1./162.*m55*me2*RSp465e*pow(RSe461p,2) + 1./162.*m44
         *me2*RSp564e*pow(RSe563p,2) - 1./81.*m44*me2*RSp564e*RSe561p*
         RSe563p + 1./162.*m44*me2*RSp564e*pow(RSe561p,2) - 1./108.*m33
         *m66*RSp456e*pow(RSe453p,2) - 1./162.*m33*m66*RSp456e*RSe451p*
         RSe453p - 1./324.*m33*m66*RSp456e*pow(RSe451p,2) - 1./108.*m33
         *m55*RSp465e*pow(RSe463p,2) - 1./162.*m33*m55*RSp465e*RSe461p*
         RSe463p - 1./324.*m33*m55*RSp465e*pow(RSe461p,2) - 1./108.*m33
         *m44*RSp564e*pow(RSe563p,2) - 1./162.*m33*m44*RSp564e*RSe561p*
         RSe563p - 1./324.*m33*m44*RSp564e*pow(RSe561p,2) + 1./162.*m13
         *m66*RSp456e*pow(RSe453p,2) - 1./81.*m13*m66*RSp456e*RSe451p*
         RSe453p + 1./162.*m13*m66*RSp456e*pow(RSe451p,2) + 1./162.*m13
         *m55*RSp465e*pow(RSe463p,2) );

      M13p6x23RV +=  + Ab(mp2,mu2)*BbV(me2,L) * (  - 1./81.*m13*m55*
         RSp465e*RSe461p*RSe463p + 1./162.*m13*m55*RSp465e*pow(
         RSe461p,2) + 1./162.*m13*m44*RSp564e*pow(RSe563p,2) - 1./81.*
         m13*m44*RSp564e*RSe561p*RSe563p + 1./162.*m13*m44*RSp564e*pow(
         RSe561p,2) - 1./324.*m11*m66*RSp456e*pow(RSe453p,2) - 1./162.*
         m11*m66*RSp456e*RSe451p*RSe453p - 1./108.*m11*m66*RSp456e*pow(
         RSe451p,2) - 1./324.*m11*m55*RSp465e*pow(RSe463p,2) - 1./162.*
         m11*m55*RSp465e*RSe461p*RSe463p - 1./108.*m11*m55*RSp465e*pow(
         RSe461p,2) - 1./324.*m11*m44*RSp564e*pow(RSe563p,2) - 1./162.*
         m11*m44*RSp564e*RSe561p*RSe563p - 1./108.*m11*m44*RSp564e*pow(
         RSe561p,2) );

      M13p6x23RV +=  + Ab(mp2,mu2)*CbV(m11,L) * ( 1./54.*m11*m33*
         RSp45611e*RD1456ep - 1./27.*m11*m13*RSp45611e*RD1456ep + 1./54.
         *pow(m11,2)*RSp45611e*RD1456ep );

      M13p6x23RV +=  + Ab(mp2,mu2)*CbV(m33,L) * ( 1./54.*pow(m33,2)*
         RSp45633e*RD3456ep - 1./27.*m13*m33*RSp45633e*RD3456ep + 1./54.
         *m11*m33*RSp45633e*RD3456ep );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m11,L) * (  - 1./270.*RD1456ep*
         RSe45613p + 1./540.*RD1456ep*RSe45633p + 1./540.*RSe45611p*
         RD1456ep + 1./135.*me2*RSe45613p*RS14563pe - 1./270.*me2*
         RSe45633p*RS14563pe - 1./270.*me2*RC1456pe*RSe45613p + 1./540.
         *me2*RC1456pe*RSe45633p - 1./270.*me2*RSe45611p*RS14563pe + 1./
         540.*me2*RSe45611p*RC1456pe + 1./162.*m66*RSe456p*pow(
         RS145pe,2) + 1./162.*m55*RSe465p*pow(RS146pe,2) + 1./162.*m44*
         RSe564p*pow(RS156pe,2) + 2./45.*m33*RSe45613p*RS14563pe - 1./
         90.*m33*RSe45633p*RS14563pe - 1./405.*m33*RC1456pe*RSe45613p
          - 11./810.*m33*RC1456pe*RSe45633p + 1./270.*m33*RSe45611p*
         RS14563pe + 13./810.*m33*RSe45611p*RC1456pe - 7./405.*m13*
         RSe45613p*RS14563pe + 7./810.*m13*RSe45633p*RS14563pe + 7./810.
         *m13*RC1456pe*RSe45613p - 7./1620.*m13*RC1456pe*RSe45633p + 7./
         810.*m13*RSe45611p*RS14563pe - 7./1620.*m13*RSe45611p*RC1456pe
          + 16./405.*m11*RSe45613p*RS14563pe + 1./162.*m11*RSe45633p*
         RS14563pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m11,L) * ( 13./810.*m11*RC1456pe
         *RSe45613p - 19./1620.*m11*RC1456pe*RSe45633p + 23./810.*m11*
         RSe45611p*RS14563pe - 37./1620.*m11*RSe45611p*RC1456pe - 1./54.
         *m11*m33*RN1e*RC1456pe*RSe45613p - 1./54.*m11*m33*RN1e*
         RSe45611p*RC1456pe - 1./27.*m11*m13*RN1e*RC1456pe*RSe45613p + 
         1./27.*m11*m13*RN1e*RSe45611p*RC1456pe - 1./18.*pow(m11,2)*
         RN1e*RC1456pe*RSe45613p - 1./54.*pow(m11,2)*RN1e*RSe45611p*
         RC1456pe + 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33*
         RSe45613p - 1./162.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33*
         RSe45633p - 1./162.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33*
         RSe45611p + 2./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*
         RSe45613p - 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*
         RSe45633p - 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*
         RSe45611p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m13,L) * (  - 1./162.*RD3456ep*
         RSe45613p + 1./324.*RSe45633p*RD3456ep - 1./162.*RD1456ep*
         RSe45613p + 1./324.*RD1456ep*RSe45633p + 1./324.*RSe45611p*
         RD3456ep + 1./324.*RSe45611p*RD1456ep + 1./27.*me2*RSe45613p*
         RS34561pe + 1./27.*me2*RSe45613p*RS14563pe + 4./81.*me2*pow(
         RSe45613p,2) + 1./27.*me2*RSp45613e*RSe45613p - 1./54.*me2*
         RC3456pe*RSe45613p - 1./54.*me2*RSe45633p*RS34561pe - 1./54.*
         me2*RSe45633p*RS14563pe - 4./81.*me2*RSe45633p*RSe45613p - 1./
         54.*me2*RSe45633p*RSp45613e + 1./108.*me2*RSe45633p*RC3456pe
          + 1./81.*me2*pow(RSe45633p,2) - 1./54.*me2*RSp45633e*
         RSe45613p + 1./108.*me2*RSp45633e*RSe45633p - 1./54.*me2*
         RC1456pe*RSe45613p + 1./108.*me2*RC1456pe*RSe45633p - 1./54.*
         me2*RSe45611p*RS34561pe - 1./54.*me2*RSe45611p*RS14563pe - 4./
         81.*me2*RSe45611p*RSe45613p - 1./54.*me2*RSe45611p*RSp45613e
          + 1./108.*me2*RSe45611p*RC3456pe + 2./81.*me2*RSe45611p*
         RSe45633p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m13,L) * ( 1./108.*me2*RSe45611p
         *RSp45633e + 1./108.*me2*RSe45611p*RC1456pe + 1./81.*me2*pow(
         RSe45611p,2) - 1./54.*me2*RSp45611e*RSe45613p + 1./108.*me2*
         RSp45611e*RSe45633p + 1./108.*me2*RSp45611e*RSe45611p + 1./81.
         *mp2*RSp45613e*RSe45613p - 1./162.*mp2*RSe45633p*RSp45613e - 1.
         /162.*mp2*RSp45633e*RSe45613p + 1./324.*mp2*RSp45633e*
         RSe45633p - 1./162.*mp2*RSe45611p*RSp45613e + 1./324.*mp2*
         RSe45611p*RSp45633e - 1./162.*mp2*RSp45611e*RSe45613p + 1./324.
         *mp2*RSp45611e*RSe45633p + 1./324.*mp2*RSp45611e*RSe45611p - 1.
         /81.*m33*RSe45613p*RS14563pe - 1./81.*m33*pow(RSe45613p,2) - 1.
         /81.*m33*RSp45613e*RSe45613p + 1./162.*m33*RC3456pe*RSe45613p
          + 1./162.*m33*RSe45633p*RS34561pe + 1./81.*m33*RSe45633p*
         RS14563pe + 2./81.*m33*RSe45633p*RSe45613p + 1./81.*m33*
         RSe45633p*RSp45613e - 5./324.*m33*RSe45633p*RC3456pe - 1./54.*
         m33*pow(RSe45633p,2) + 1./81.*m33*RSp45633e*RSe45613p - 1./54.
         *m33*RSp45633e*RSe45633p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m13,L) * ( 1./162.*m33*RC1456pe*
         RSe45633p - 1./162.*m33*RSe45611p*RS34561pe + 1./108.*m33*
         RSe45611p*RC3456pe + 1./81.*m33*RSe45611p*RSe45633p + 1./162.*
         m33*RSe45611p*RSp45633e - 1./162.*m33*RSe45611p*RC1456pe - 1./
         162.*m33*pow(RSe45611p,2) + 1./162.*m33*RSp45611e*RSe45633p - 
         1./162.*m33*RSp45611e*RSe45611p - 2./27.*m13*RSe45613p*
         RS34561pe - 2./27.*m13*RSe45613p*RS14563pe - 2./27.*m13*pow(
         RSe45613p,2) - 2./27.*m13*RSp45613e*RSe45613p + 1./27.*m13*
         RC3456pe*RSe45613p + 1./27.*m13*RSe45633p*RS34561pe + 1./27.*
         m13*RSe45633p*RS14563pe + 2./27.*m13*RSe45633p*RSe45613p + 1./
         27.*m13*RSe45633p*RSp45613e - 1./54.*m13*RSe45633p*RC3456pe - 
         1./54.*m13*pow(RSe45633p,2) + 1./27.*m13*RSp45633e*RSe45613p
          - 1./54.*m13*RSp45633e*RSe45633p + 1./27.*m13*RC1456pe*
         RSe45613p - 1./54.*m13*RC1456pe*RSe45633p + 1./27.*m13*
         RSe45611p*RS34561pe + 1./27.*m13*RSe45611p*RS14563pe + 2./27.*
         m13*RSe45611p*RSe45613p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m13,L) * ( 1./27.*m13*RSe45611p*
         RSp45613e - 1./54.*m13*RSe45611p*RC3456pe - 1./27.*m13*
         RSe45611p*RSe45633p - 1./54.*m13*RSe45611p*RSp45633e - 1./54.*
         m13*RSe45611p*RC1456pe - 1./54.*m13*pow(RSe45611p,2) + 1./27.*
         m13*RSp45611e*RSe45613p - 1./54.*m13*RSp45611e*RSe45633p - 1./
         54.*m13*RSp45611e*RSe45611p - 1./81.*m11*RSe45613p*RS34561pe
          - 1./81.*m11*pow(RSe45613p,2) - 1./81.*m11*RSp45613e*
         RSe45613p - 1./162.*m11*RSe45633p*RS14563pe - 1./162.*m11*
         RSe45633p*RC3456pe - 1./162.*m11*pow(RSe45633p,2) - 1./162.*
         m11*RSp45633e*RSe45633p + 1./162.*m11*RC1456pe*RSe45613p + 1./
         108.*m11*RC1456pe*RSe45633p + 1./81.*m11*RSe45611p*RS34561pe
          + 1./162.*m11*RSe45611p*RS14563pe + 2./81.*m11*RSe45611p*
         RSe45613p + 1./81.*m11*RSe45611p*RSp45613e + 1./162.*m11*
         RSe45611p*RC3456pe + 1./81.*m11*RSe45611p*RSe45633p + 1./162.*
         m11*RSe45611p*RSp45633e - 5./324.*m11*RSe45611p*RC1456pe - 1./
         54.*m11*pow(RSe45611p,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m13,L) * ( 1./81.*m11*RSp45611e*
         RSe45613p + 1./162.*m11*RSp45611e*RSe45633p - 1./54.*m11*
         RSp45611e*RSe45611p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m14,L) * ( 11./135.*me2*
         RSe45613p - 1./180.*me2*RSe45633p - 19./270.*me2*RSe563p - 29./
         540.*me2*RSe45611p + 7./270.*me2*RSe561p + 1./45.*me2*RSe564p
          - 7./540.*m44*RSe45613p + 1./60.*m44*RSe45633p + 1./180.*m44*
         RSe563p + 7./540.*m44*RSe45611p - 1./60.*m44*RSe561p - 1./180.
         *m44*RSe564p + 1./27.*m34*RSe45613p - 1./27.*m34*RSe45633p + 1.
         /27.*m34*RSe563p - 1./108.*m34*RSe45611p - 1./54.*m34*RSe561p
          - 1./108.*m34*RSe564p + 4./135.*m33*RSe45613p + 11./270.*m33*
         RSe45633p - 1./45.*m33*RSe563p + 1./120.*m33*RSe45611p - 1./20.
         *m33*RSe561p - 7./1080.*m33*RSe564p + 1./108.*m33*m44*RN1e*
         RSe45613p + 1./108.*m33*m44*RN1e*RSe563p + 1./108.*m33*m44*
         RN1e*RSe45611p + 1./108.*m33*m44*RN1e*RSe561p - 1./90.*m14*
         RSe45613p + 1./270.*m14*RSe45633p + 1./270.*m14*RSe563p - 1./
         60.*m14*RSe45611p + 2./45.*m14*RSe561p - 13./540.*m14*RSe564p
          - 1./54.*m14*m33*RN1e*RSe45613p + 1./54.*m14*m33*RN1e*RSe563p
          - 1./54.*m14*m33*RN1e*RSe45611p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m14,L) * ( 1./54.*m14*m33*RN1e*
         RSe561p - 11./540.*m13*RSe45613p + 1./20.*m13*RSe45633p - 43./
         540.*m13*RSe563p - 1./135.*m13*RSe45611p + 19./540.*m13*
         RSe561p + 1./45.*m13*RSe564p + 1./54.*m13*m44*RN1e*RSe45613p
          + 1./54.*m13*m44*RN1e*RSe563p - 1./54.*m13*m44*RN1e*RSe45611p
          - 1./54.*m13*m44*RN1e*RSe561p - 1./27.*m13*m14*RN1e*RSe45613p
          + 1./27.*m13*m14*RN1e*RSe563p + 1./27.*m13*m14*RN1e*RSe45611p
          - 1./27.*m13*m14*RN1e*RSe561p - 1./540.*m11*RSe45613p + 2./
         135.*m11*RSe45633p - 17./540.*m11*RSe563p + 37./360.*m11*
         RSe45611p - 23./270.*m11*RSe561p + 1./1080.*m11*RSe564p - 1./
         27.*m11*me2*RN1e*RSe45613p + 1./27.*m11*me2*RN1e*RSe563p + 1./
         27.*m11*me2*RN1e*RSe45611p - 1./27.*m11*me2*RN1e*RSe561p + 1./
         36.*m11*m44*RN1e*RSe45613p + 1./36.*m11*m44*RN1e*RSe563p + 1./
         108.*m11*m44*RN1e*RSe45611p + 1./108.*m11*m44*RN1e*RSe561p + 1.
         /108.*m11*m33*RN1e*RSe45613p + 1./36.*m11*m33*RN1e*RSe563p + 1.
         /108.*m11*m33*RN1e*RSe45611p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m14,L) * ( 1./36.*m11*m33*RN1e*
         RSe561p - 1./18.*m11*m14*RN1e*RSe45613p + 1./18.*m11*m14*RN1e*
         RSe563p - 1./54.*m11*m14*RN1e*RSe45611p + 1./54.*m11*m14*RN1e*
         RSe561p + 1./54.*m11*m13*RN1e*RSe45613p + 1./18.*m11*m13*RN1e*
         RSe563p - 1./54.*m11*m13*RN1e*RSe45611p - 1./18.*m11*m13*RN1e*
         RSe561p + 7./108.*pow(m11,2)*RN1e*RSe45613p + 5./108.*pow(
         m11,2)*RN1e*RSe563p - 1./36.*pow(m11,2)*RN1e*RSe45611p + 7./
         108.*pow(m11,2)*RN1e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m15,L) * ( 11./135.*me2*
         RSe45613p - 1./180.*me2*RSe45633p - 19./270.*me2*RSe463p - 29./
         540.*me2*RSe45611p + 7./270.*me2*RSe461p + 1./45.*me2*RSe465p
          - 7./540.*m55*RSe45613p + 1./60.*m55*RSe45633p + 1./180.*m55*
         RSe463p + 7./540.*m55*RSe45611p - 1./60.*m55*RSe461p - 1./180.
         *m55*RSe465p + 1./27.*m35*RSe45613p - 1./27.*m35*RSe45633p + 1.
         /27.*m35*RSe463p - 1./108.*m35*RSe45611p - 1./54.*m35*RSe461p
          - 1./108.*m35*RSe465p + 4./135.*m33*RSe45613p + 11./270.*m33*
         RSe45633p - 1./45.*m33*RSe463p + 1./120.*m33*RSe45611p - 1./20.
         *m33*RSe461p - 7./1080.*m33*RSe465p + 1./108.*m33*m55*RN1e*
         RSe45613p + 1./108.*m33*m55*RN1e*RSe463p + 1./108.*m33*m55*
         RN1e*RSe45611p + 1./108.*m33*m55*RN1e*RSe461p - 1./90.*m15*
         RSe45613p + 1./270.*m15*RSe45633p + 1./270.*m15*RSe463p - 1./
         60.*m15*RSe45611p + 2./45.*m15*RSe461p - 13./540.*m15*RSe465p
          - 1./54.*m15*m33*RN1e*RSe45613p + 1./54.*m15*m33*RN1e*RSe463p
          - 1./54.*m15*m33*RN1e*RSe45611p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m15,L) * ( 1./54.*m15*m33*RN1e*
         RSe461p - 11./540.*m13*RSe45613p + 1./20.*m13*RSe45633p - 43./
         540.*m13*RSe463p - 1./135.*m13*RSe45611p + 19./540.*m13*
         RSe461p + 1./45.*m13*RSe465p + 1./54.*m13*m55*RN1e*RSe45613p
          + 1./54.*m13*m55*RN1e*RSe463p - 1./54.*m13*m55*RN1e*RSe45611p
          - 1./54.*m13*m55*RN1e*RSe461p - 1./27.*m13*m15*RN1e*RSe45613p
          + 1./27.*m13*m15*RN1e*RSe463p + 1./27.*m13*m15*RN1e*RSe45611p
          - 1./27.*m13*m15*RN1e*RSe461p - 1./540.*m11*RSe45613p + 2./
         135.*m11*RSe45633p - 17./540.*m11*RSe463p + 37./360.*m11*
         RSe45611p - 23./270.*m11*RSe461p + 1./1080.*m11*RSe465p - 1./
         27.*m11*me2*RN1e*RSe45613p + 1./27.*m11*me2*RN1e*RSe463p + 1./
         27.*m11*me2*RN1e*RSe45611p - 1./27.*m11*me2*RN1e*RSe461p + 1./
         36.*m11*m55*RN1e*RSe45613p + 1./36.*m11*m55*RN1e*RSe463p + 1./
         108.*m11*m55*RN1e*RSe45611p + 1./108.*m11*m55*RN1e*RSe461p + 1.
         /108.*m11*m33*RN1e*RSe45613p + 1./36.*m11*m33*RN1e*RSe463p + 1.
         /108.*m11*m33*RN1e*RSe45611p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m15,L) * ( 1./36.*m11*m33*RN1e*
         RSe461p - 1./18.*m11*m15*RN1e*RSe45613p + 1./18.*m11*m15*RN1e*
         RSe463p - 1./54.*m11*m15*RN1e*RSe45611p + 1./54.*m11*m15*RN1e*
         RSe461p + 1./54.*m11*m13*RN1e*RSe45613p + 1./18.*m11*m13*RN1e*
         RSe463p - 1./54.*m11*m13*RN1e*RSe45611p - 1./18.*m11*m13*RN1e*
         RSe461p + 7./108.*pow(m11,2)*RN1e*RSe45613p + 5./108.*pow(
         m11,2)*RN1e*RSe463p - 1./36.*pow(m11,2)*RN1e*RSe45611p + 7./
         108.*pow(m11,2)*RN1e*RSe461p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m16,L) * ( 11./135.*me2*
         RSe45613p - 1./180.*me2*RSe45633p - 19./270.*me2*RSe453p - 29./
         540.*me2*RSe45611p + 7./270.*me2*RSe451p + 1./45.*me2*RSe456p
          - 7./540.*m66*RSe45613p + 1./60.*m66*RSe45633p + 1./180.*m66*
         RSe453p + 7./540.*m66*RSe45611p - 1./60.*m66*RSe451p - 1./180.
         *m66*RSe456p + 1./27.*m36*RSe45613p - 1./27.*m36*RSe45633p + 1.
         /27.*m36*RSe453p - 1./108.*m36*RSe45611p - 1./54.*m36*RSe451p
          - 1./108.*m36*RSe456p + 4./135.*m33*RSe45613p + 11./270.*m33*
         RSe45633p - 1./45.*m33*RSe453p + 1./120.*m33*RSe45611p - 1./20.
         *m33*RSe451p - 7./1080.*m33*RSe456p + 1./108.*m33*m66*RN1e*
         RSe45613p + 1./108.*m33*m66*RN1e*RSe453p + 1./108.*m33*m66*
         RN1e*RSe45611p + 1./108.*m33*m66*RN1e*RSe451p - 1./90.*m16*
         RSe45613p + 1./270.*m16*RSe45633p + 1./270.*m16*RSe453p - 1./
         60.*m16*RSe45611p + 2./45.*m16*RSe451p - 13./540.*m16*RSe456p
          - 1./54.*m16*m33*RN1e*RSe45613p + 1./54.*m16*m33*RN1e*RSe453p
          - 1./54.*m16*m33*RN1e*RSe45611p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m16,L) * ( 1./54.*m16*m33*RN1e*
         RSe451p - 11./540.*m13*RSe45613p + 1./20.*m13*RSe45633p - 43./
         540.*m13*RSe453p - 1./135.*m13*RSe45611p + 19./540.*m13*
         RSe451p + 1./45.*m13*RSe456p + 1./54.*m13*m66*RN1e*RSe45613p
          + 1./54.*m13*m66*RN1e*RSe453p - 1./54.*m13*m66*RN1e*RSe45611p
          - 1./54.*m13*m66*RN1e*RSe451p - 1./27.*m13*m16*RN1e*RSe45613p
          + 1./27.*m13*m16*RN1e*RSe453p + 1./27.*m13*m16*RN1e*RSe45611p
          - 1./27.*m13*m16*RN1e*RSe451p - 1./540.*m11*RSe45613p + 2./
         135.*m11*RSe45633p - 17./540.*m11*RSe453p + 37./360.*m11*
         RSe45611p - 23./270.*m11*RSe451p + 1./1080.*m11*RSe456p - 1./
         27.*m11*me2*RN1e*RSe45613p + 1./27.*m11*me2*RN1e*RSe453p + 1./
         27.*m11*me2*RN1e*RSe45611p - 1./27.*m11*me2*RN1e*RSe451p + 1./
         36.*m11*m66*RN1e*RSe45613p + 1./36.*m11*m66*RN1e*RSe453p + 1./
         108.*m11*m66*RN1e*RSe45611p + 1./108.*m11*m66*RN1e*RSe451p + 1.
         /108.*m11*m33*RN1e*RSe45613p + 1./36.*m11*m33*RN1e*RSe453p + 1.
         /108.*m11*m33*RN1e*RSe45611p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m16,L) * ( 1./36.*m11*m33*RN1e*
         RSe451p - 1./18.*m11*m16*RN1e*RSe45613p + 1./18.*m11*m16*RN1e*
         RSe453p - 1./54.*m11*m16*RN1e*RSe45611p + 1./54.*m11*m16*RN1e*
         RSe451p + 1./54.*m11*m13*RN1e*RSe45613p + 1./18.*m11*m13*RN1e*
         RSe453p - 1./54.*m11*m13*RN1e*RSe45611p - 1./18.*m11*m13*RN1e*
         RSe451p + 7./108.*pow(m11,2)*RN1e*RSe45613p + 5./108.*pow(
         m11,2)*RN1e*RSe453p - 1./36.*pow(m11,2)*RN1e*RSe45611p + 7./
         108.*pow(m11,2)*RN1e*RSe451p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m33,L) * (  - 1./270.*RD3456ep*
         RSe45613p + 1./540.*RSe45633p*RD3456ep + 1./540.*RSe45611p*
         RD3456ep + 1./135.*me2*RSe45613p*RS34561pe - 1./270.*me2*
         RC3456pe*RSe45613p - 1./270.*me2*RSe45633p*RS34561pe + 1./540.
         *me2*RSe45633p*RC3456pe - 1./270.*me2*RSe45611p*RS34561pe + 1./
         540.*me2*RSe45611p*RC3456pe + 1./162.*m66*RSe456p*pow(
         RS345pe,2) - 1./81.*m66*RSe456p*RS145pe*RS345pe + 1./162.*m55*
         RSe465p*pow(RS346pe,2) - 1./81.*m55*RSe465p*RS146pe*RS346pe + 
         1./162.*m44*RSe564p*pow(RS356pe,2) - 1./81.*m44*RSe564p*
         RS156pe*RS356pe + 16./405.*m33*RSe45613p*RS34561pe + 13./810.*
         m33*RC3456pe*RSe45613p + 23./810.*m33*RSe45633p*RS34561pe + 1./
         27.*m33*RSe45633p*RS14563pe - 37./1620.*m33*RSe45633p*RC3456pe
          + 1./162.*m33*RSe45611p*RS34561pe - 19./1620.*m33*RSe45611p*
         RC3456pe - 1./18.*pow(m33,2)*RN3e*RC3456pe*RSe45613p - 1./54.*
         pow(m33,2)*RN3e*RSe45633p*RC3456pe - 7./405.*m13*RSe45613p*
         RS34561pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m33,L) * ( 7./810.*m13*RC3456pe*
         RSe45613p + 7./810.*m13*RSe45633p*RS34561pe - 7./1620.*m13*
         RSe45633p*RC3456pe + 7./810.*m13*RSe45611p*RS34561pe - 7./1620.
         *m13*RSe45611p*RC3456pe - 1./27.*m13*m33*RN3e*RC3456pe*
         RSe45613p + 1./27.*m13*m33*RN3e*RSe45633p*RC3456pe + 2./45.*
         m11*RSe45613p*RS34561pe - 1./405.*m11*RC3456pe*RSe45613p + 1./
         270.*m11*RSe45633p*RS34561pe + 13./810.*m11*RSe45633p*RC3456pe
          + 7./270.*m11*RSe45611p*RS34561pe - 11./810.*m11*RSe45611p*
         RC3456pe - 1./54.*m11*m33*RN3e*RC3456pe*RSe45613p - 1./54.*m11
         *m33*RN3e*RSe45633p*RC3456pe + 2./81.*RS(m33,m44,m55,m66,m11,
         me2,mp2)*m33*RSe45613p - 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2
         )*m33*RSe45633p - 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33*
         RSe45611p + 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RSe45613p - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RSe45633p - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RSe45611p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m34,L) * ( 11./135.*me2*
         RSe45613p - 29./540.*me2*RSe45633p + 7./270.*me2*RSe563p - 1./
         180.*me2*RSe45611p - 19./270.*me2*RSe561p + 1./45.*me2*RSe564p
          - 7./540.*m44*RSe45613p + 7./540.*m44*RSe45633p - 1./60.*m44*
         RSe563p + 1./60.*m44*RSe45611p + 1./180.*m44*RSe561p - 1./180.
         *m44*RSe564p - 1./90.*m34*RSe45613p - 1./60.*m34*RSe45633p + 2.
         /45.*m34*RSe563p + 1./270.*m34*RSe45611p + 1./270.*m34*RSe561p
          - 13./540.*m34*RSe564p - 1./540.*m33*RSe45613p + 37./360.*m33
         *RSe45633p - 23./270.*m33*RSe563p + 2./135.*m33*RSe45611p - 17.
         /540.*m33*RSe561p + 1./1080.*m33*RSe564p - 1./27.*m33*me2*RN3e
         *RSe45613p + 1./27.*m33*me2*RN3e*RSe45633p - 1./27.*m33*me2*
         RN3e*RSe563p + 1./27.*m33*me2*RN3e*RSe561p + 1./36.*m33*m44*
         RN3e*RSe45613p + 1./108.*m33*m44*RN3e*RSe45633p + 1./108.*m33*
         m44*RN3e*RSe563p + 1./36.*m33*m44*RN3e*RSe561p - 1./18.*m33*
         m34*RN3e*RSe45613p - 1./54.*m33*m34*RN3e*RSe45633p + 1./54.*
         m33*m34*RN3e*RSe563p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m34,L) * ( 1./18.*m33*m34*RN3e*
         RSe561p + 7./108.*pow(m33,2)*RN3e*RSe45613p - 1./36.*pow(
         m33,2)*RN3e*RSe45633p + 7./108.*pow(m33,2)*RN3e*RSe563p + 5./
         108.*pow(m33,2)*RN3e*RSe561p + 1./27.*m14*RSe45613p - 1./108.*
         m14*RSe45633p - 1./54.*m14*RSe563p - 1./27.*m14*RSe45611p + 1./
         27.*m14*RSe561p - 1./108.*m14*RSe564p - 11./540.*m13*RSe45613p
          - 1./135.*m13*RSe45633p + 19./540.*m13*RSe563p + 1./20.*m13*
         RSe45611p - 43./540.*m13*RSe561p + 1./45.*m13*RSe564p + 1./54.
         *m13*m44*RN3e*RSe45613p - 1./54.*m13*m44*RN3e*RSe45633p - 1./
         54.*m13*m44*RN3e*RSe563p + 1./54.*m13*m44*RN3e*RSe561p - 1./27.
         *m13*m34*RN3e*RSe45613p + 1./27.*m13*m34*RN3e*RSe45633p - 1./
         27.*m13*m34*RN3e*RSe563p + 1./27.*m13*m34*RN3e*RSe561p + 1./54.
         *m13*m33*RN3e*RSe45613p - 1./54.*m13*m33*RN3e*RSe45633p - 1./
         18.*m13*m33*RN3e*RSe563p + 1./18.*m13*m33*RN3e*RSe561p + 4./
         135.*m11*RSe45613p + 1./120.*m11*RSe45633p - 1./20.*m11*
         RSe563p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m34,L) * ( 11./270.*m11*
         RSe45611p - 1./45.*m11*RSe561p - 7./1080.*m11*RSe564p + 1./108.
         *m11*m44*RN3e*RSe45613p + 1./108.*m11*m44*RN3e*RSe45633p + 1./
         108.*m11*m44*RN3e*RSe563p + 1./108.*m11*m44*RN3e*RSe561p - 1./
         54.*m11*m34*RN3e*RSe45613p - 1./54.*m11*m34*RN3e*RSe45633p + 1.
         /54.*m11*m34*RN3e*RSe563p + 1./54.*m11*m34*RN3e*RSe561p + 1./
         108.*m11*m33*RN3e*RSe45613p + 1./108.*m11*m33*RN3e*RSe45633p
          + 1./36.*m11*m33*RN3e*RSe563p + 1./36.*m11*m33*RN3e*RSe561p )
         ;

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m35,L) * ( 11./135.*me2*
         RSe45613p - 29./540.*me2*RSe45633p + 7./270.*me2*RSe463p - 1./
         180.*me2*RSe45611p - 19./270.*me2*RSe461p + 1./45.*me2*RSe465p
          - 7./540.*m55*RSe45613p + 7./540.*m55*RSe45633p - 1./60.*m55*
         RSe463p + 1./60.*m55*RSe45611p + 1./180.*m55*RSe461p - 1./180.
         *m55*RSe465p - 1./90.*m35*RSe45613p - 1./60.*m35*RSe45633p + 2.
         /45.*m35*RSe463p + 1./270.*m35*RSe45611p + 1./270.*m35*RSe461p
          - 13./540.*m35*RSe465p - 1./540.*m33*RSe45613p + 37./360.*m33
         *RSe45633p - 23./270.*m33*RSe463p + 2./135.*m33*RSe45611p - 17.
         /540.*m33*RSe461p + 1./1080.*m33*RSe465p - 1./27.*m33*me2*RN3e
         *RSe45613p + 1./27.*m33*me2*RN3e*RSe45633p - 1./27.*m33*me2*
         RN3e*RSe463p + 1./27.*m33*me2*RN3e*RSe461p + 1./36.*m33*m55*
         RN3e*RSe45613p + 1./108.*m33*m55*RN3e*RSe45633p + 1./108.*m33*
         m55*RN3e*RSe463p + 1./36.*m33*m55*RN3e*RSe461p - 1./18.*m33*
         m35*RN3e*RSe45613p - 1./54.*m33*m35*RN3e*RSe45633p + 1./54.*
         m33*m35*RN3e*RSe463p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m35,L) * ( 1./18.*m33*m35*RN3e*
         RSe461p + 7./108.*pow(m33,2)*RN3e*RSe45613p - 1./36.*pow(
         m33,2)*RN3e*RSe45633p + 7./108.*pow(m33,2)*RN3e*RSe463p + 5./
         108.*pow(m33,2)*RN3e*RSe461p + 1./27.*m15*RSe45613p - 1./108.*
         m15*RSe45633p - 1./54.*m15*RSe463p - 1./27.*m15*RSe45611p + 1./
         27.*m15*RSe461p - 1./108.*m15*RSe465p - 11./540.*m13*RSe45613p
          - 1./135.*m13*RSe45633p + 19./540.*m13*RSe463p + 1./20.*m13*
         RSe45611p - 43./540.*m13*RSe461p + 1./45.*m13*RSe465p + 1./54.
         *m13*m55*RN3e*RSe45613p - 1./54.*m13*m55*RN3e*RSe45633p - 1./
         54.*m13*m55*RN3e*RSe463p + 1./54.*m13*m55*RN3e*RSe461p - 1./27.
         *m13*m35*RN3e*RSe45613p + 1./27.*m13*m35*RN3e*RSe45633p - 1./
         27.*m13*m35*RN3e*RSe463p + 1./27.*m13*m35*RN3e*RSe461p + 1./54.
         *m13*m33*RN3e*RSe45613p - 1./54.*m13*m33*RN3e*RSe45633p - 1./
         18.*m13*m33*RN3e*RSe463p + 1./18.*m13*m33*RN3e*RSe461p + 4./
         135.*m11*RSe45613p + 1./120.*m11*RSe45633p - 1./20.*m11*
         RSe463p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m35,L) * ( 11./270.*m11*
         RSe45611p - 1./45.*m11*RSe461p - 7./1080.*m11*RSe465p + 1./108.
         *m11*m55*RN3e*RSe45613p + 1./108.*m11*m55*RN3e*RSe45633p + 1./
         108.*m11*m55*RN3e*RSe463p + 1./108.*m11*m55*RN3e*RSe461p - 1./
         54.*m11*m35*RN3e*RSe45613p - 1./54.*m11*m35*RN3e*RSe45633p + 1.
         /54.*m11*m35*RN3e*RSe463p + 1./54.*m11*m35*RN3e*RSe461p + 1./
         108.*m11*m33*RN3e*RSe45613p + 1./108.*m11*m33*RN3e*RSe45633p
          + 1./36.*m11*m33*RN3e*RSe463p + 1./36.*m11*m33*RN3e*RSe461p )
         ;

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m36,L) * ( 11./135.*me2*
         RSe45613p - 29./540.*me2*RSe45633p + 7./270.*me2*RSe453p - 1./
         180.*me2*RSe45611p - 19./270.*me2*RSe451p + 1./45.*me2*RSe456p
          - 7./540.*m66*RSe45613p + 7./540.*m66*RSe45633p - 1./60.*m66*
         RSe453p + 1./60.*m66*RSe45611p + 1./180.*m66*RSe451p - 1./180.
         *m66*RSe456p - 1./90.*m36*RSe45613p - 1./60.*m36*RSe45633p + 2.
         /45.*m36*RSe453p + 1./270.*m36*RSe45611p + 1./270.*m36*RSe451p
          - 13./540.*m36*RSe456p - 1./540.*m33*RSe45613p + 37./360.*m33
         *RSe45633p - 23./270.*m33*RSe453p + 2./135.*m33*RSe45611p - 17.
         /540.*m33*RSe451p + 1./1080.*m33*RSe456p - 1./27.*m33*me2*RN3e
         *RSe45613p + 1./27.*m33*me2*RN3e*RSe45633p - 1./27.*m33*me2*
         RN3e*RSe453p + 1./27.*m33*me2*RN3e*RSe451p + 1./36.*m33*m66*
         RN3e*RSe45613p + 1./108.*m33*m66*RN3e*RSe45633p + 1./108.*m33*
         m66*RN3e*RSe453p + 1./36.*m33*m66*RN3e*RSe451p - 1./18.*m33*
         m36*RN3e*RSe45613p - 1./54.*m33*m36*RN3e*RSe45633p + 1./54.*
         m33*m36*RN3e*RSe453p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m36,L) * ( 1./18.*m33*m36*RN3e*
         RSe451p + 7./108.*pow(m33,2)*RN3e*RSe45613p - 1./36.*pow(
         m33,2)*RN3e*RSe45633p + 7./108.*pow(m33,2)*RN3e*RSe453p + 5./
         108.*pow(m33,2)*RN3e*RSe451p + 1./27.*m16*RSe45613p - 1./108.*
         m16*RSe45633p - 1./54.*m16*RSe453p - 1./27.*m16*RSe45611p + 1./
         27.*m16*RSe451p - 1./108.*m16*RSe456p - 11./540.*m13*RSe45613p
          - 1./135.*m13*RSe45633p + 19./540.*m13*RSe453p + 1./20.*m13*
         RSe45611p - 43./540.*m13*RSe451p + 1./45.*m13*RSe456p + 1./54.
         *m13*m66*RN3e*RSe45613p - 1./54.*m13*m66*RN3e*RSe45633p - 1./
         54.*m13*m66*RN3e*RSe453p + 1./54.*m13*m66*RN3e*RSe451p - 1./27.
         *m13*m36*RN3e*RSe45613p + 1./27.*m13*m36*RN3e*RSe45633p - 1./
         27.*m13*m36*RN3e*RSe453p + 1./27.*m13*m36*RN3e*RSe451p + 1./54.
         *m13*m33*RN3e*RSe45613p - 1./54.*m13*m33*RN3e*RSe45633p - 1./
         18.*m13*m33*RN3e*RSe453p + 1./18.*m13*m33*RN3e*RSe451p + 4./
         135.*m11*RSe45613p + 1./120.*m11*RSe45633p - 1./20.*m11*
         RSe453p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m36,L) * ( 11./270.*m11*
         RSe45611p - 1./45.*m11*RSe451p - 7./1080.*m11*RSe456p + 1./108.
         *m11*m66*RN3e*RSe45613p + 1./108.*m11*m66*RN3e*RSe45633p + 1./
         108.*m11*m66*RN3e*RSe453p + 1./108.*m11*m66*RN3e*RSe451p - 1./
         54.*m11*m36*RN3e*RSe45613p - 1./54.*m11*m36*RN3e*RSe45633p + 1.
         /54.*m11*m36*RN3e*RSe453p + 1./54.*m11*m36*RN3e*RSe451p + 1./
         108.*m11*m33*RN3e*RSe45613p + 1./108.*m11*m33*RN3e*RSe45633p
          + 1./36.*m11*m33*RN3e*RSe453p + 1./36.*m11*m33*RN3e*RSe451p )
         ;

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m44,L) * (  - 1./27.*m44*RSe563p
         *RS356pe - 1./54.*m44*pow(RSe563p,2) + 1./27.*m44*RS156pe*
         RSe563p + 1./81.*m44*RS456pe*RSe563p*RS356pe + 1./162.*m44*
         RS456pe*pow(RSe563p,2) - 1./81.*m44*RS456pe*RS156pe*RSe563p + 
         1./27.*m44*RSe561p*RS356pe + 1./27.*m44*RSe561p*RSe563p - 1./
         27.*m44*RSe561p*RS156pe - 1./81.*m44*RSe561p*RS456pe*RS356pe
          - 1./81.*m44*RSe561p*RS456pe*RSe563p + 1./81.*m44*RSe561p*
         RS456pe*RS156pe - 1./54.*m44*pow(RSe561p,2) + 1./162.*m44*pow(
         RSe561p,2)*RS456pe - 1./54.*m33*m44*RN3e*RSe563p*RS356pe + 1./
         162.*m33*m44*RN3e*RS456pe*RSe563p*RS356pe - 1./18.*m33*m44*
         RN3e*RSe561p*RS356pe + 1./54.*m33*m44*RN3e*RSe561p*RS456pe*
         RS356pe - 1./54.*m33*m44*RN1e*RS156pe*RSe563p + 1./162.*m33*
         m44*RN1e*RS456pe*RS156pe*RSe563p - 1./54.*m33*m44*RN1e*RSe561p
         *RS156pe + 1./162.*m33*m44*RN1e*RSe561p*RS456pe*RS156pe + 1./
         27.*m13*m44*RN3e*RSe563p*RS356pe - 1./81.*m13*m44*RN3e*RS456pe
         *RSe563p*RS356pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m44,L) * (  - 1./27.*m13*m44*
         RN3e*RSe561p*RS356pe + 1./81.*m13*m44*RN3e*RSe561p*RS456pe*
         RS356pe - 1./27.*m13*m44*RN1e*RS156pe*RSe563p + 1./81.*m13*m44
         *RN1e*RS456pe*RS156pe*RSe563p + 1./27.*m13*m44*RN1e*RSe561p*
         RS156pe - 1./81.*m13*m44*RN1e*RSe561p*RS456pe*RS156pe - 1./54.
         *m11*m44*RN3e*RSe563p*RS356pe + 1./162.*m11*m44*RN3e*RS456pe*
         RSe563p*RS356pe - 1./54.*m11*m44*RN3e*RSe561p*RS356pe + 1./162.
         *m11*m44*RN3e*RSe561p*RS456pe*RS356pe - 1./18.*m11*m44*RN1e*
         RS156pe*RSe563p + 1./54.*m11*m44*RN1e*RS456pe*RS156pe*RSe563p
          - 1./54.*m11*m44*RN1e*RSe561p*RS156pe + 1./162.*m11*m44*RN1e*
         RSe561p*RS456pe*RS156pe + 1./27.*RN(mp2,me2)*m44*me2*RSp563e*
         RSe563p - 1./81.*RN(mp2,me2)*m44*me2*RS456pe*RSp563e*RSe563p
          - 1./27.*RN(mp2,me2)*m44*me2*RSe561p*RSp563e + 1./81.*RN(mp2,
         me2)*m44*me2*RSe561p*RS456pe*RSp563e - 1./27.*RN(mp2,me2)*m44*
         me2*RSp561e*RSe563p + 1./81.*RN(mp2,me2)*m44*me2*RSp561e*
         RS456pe*RSe563p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m44,L) * ( 1./27.*RN(mp2,me2)*
         m44*me2*RSp561e*RSe561p - 1./81.*RN(mp2,me2)*m44*me2*RSp561e*
         RSe561p*RS456pe - 1./18.*RN(mp2,me2)*m33*m44*RSp563e*RSe563p
          + 1./54.*RN(mp2,me2)*m33*m44*RS456pe*RSp563e*RSe563p - 1./54.
         *RN(mp2,me2)*m33*m44*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m33
         *m44*RSe561p*RS456pe*RSp563e - 1./54.*RN(mp2,me2)*m33*m44*
         RSp561e*RSe563p + 1./162.*RN(mp2,me2)*m33*m44*RSp561e*RS456pe*
         RSe563p - 1./54.*RN(mp2,me2)*m33*m44*RSp561e*RSe561p + 1./162.
         *RN(mp2,me2)*m33*m44*RSp561e*RSe561p*RS456pe + 1./27.*RN(mp2,
         me2)*m13*m44*RSp563e*RSe563p - 1./81.*RN(mp2,me2)*m13*m44*
         RS456pe*RSp563e*RSe563p - 1./27.*RN(mp2,me2)*m13*m44*RSe561p*
         RSp563e + 1./81.*RN(mp2,me2)*m13*m44*RSe561p*RS456pe*RSp563e
          - 1./27.*RN(mp2,me2)*m13*m44*RSp561e*RSe563p + 1./81.*RN(mp2,
         me2)*m13*m44*RSp561e*RS456pe*RSe563p + 1./27.*RN(mp2,me2)*m13*
         m44*RSp561e*RSe561p - 1./81.*RN(mp2,me2)*m13*m44*RSp561e*
         RSe561p*RS456pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m44,L) * (  - 1./54.*RN(mp2,me2)
         *m11*m44*RSp563e*RSe563p + 1./162.*RN(mp2,me2)*m11*m44*RS456pe
         *RSp563e*RSe563p - 1./54.*RN(mp2,me2)*m11*m44*RSe561p*RSp563e
          + 1./162.*RN(mp2,me2)*m11*m44*RSe561p*RS456pe*RSp563e - 1./54.
         *RN(mp2,me2)*m11*m44*RSp561e*RSe563p + 1./162.*RN(mp2,me2)*m11
         *m44*RSp561e*RS456pe*RSe563p - 1./18.*RN(mp2,me2)*m11*m44*
         RSp561e*RSe561p + 1./54.*RN(mp2,me2)*m11*m44*RSp561e*RSe561p*
         RS456pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./81.*me2*RSe563p
         *RS346pe + 1./81.*me2*RSe563p*RS356pe + 1./81.*me2*pow(
         RSe563p,2) + 1./81.*me2*RSe463p*RS346pe - 1./81.*me2*RSe463p*
         RS356pe - 2./81.*me2*RSe463p*RSe563p + 1./81.*me2*pow(
         RSe463p,2) + 1./81.*me2*RSp563e*RSe563p - 1./81.*me2*RSp563e*
         RSe463p - 1./81.*me2*RSp463e*RSe563p + 1./81.*me2*RSp463e*
         RSe463p + 1./81.*me2*RS146pe*RSe563p - 1./81.*me2*RS146pe*
         RSe463p - 1./81.*me2*RS156pe*RSe563p + 1./81.*me2*RS156pe*
         RSe463p + 1./81.*me2*RSe561p*RS346pe - 1./81.*me2*RSe561p*
         RS356pe - 2./81.*me2*RSe561p*RSe563p + 2./81.*me2*RSe561p*
         RSe463p - 1./81.*me2*RSe561p*RSp563e + 1./81.*me2*RSe561p*
         RSp463e - 1./81.*me2*RSe561p*RS146pe + 1./81.*me2*RSe561p*
         RS156pe + 1./81.*me2*pow(RSe561p,2) - 1./81.*me2*RSe461p*
         RS346pe + 1./81.*me2*RSe461p*RS356pe + 2./81.*me2*RSe461p*
         RSe563p - 2./81.*me2*RSe461p*RSe463p + 1./81.*me2*RSe461p*
         RSp563e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./81.*me2*RSe461p
         *RSp463e + 1./81.*me2*RSe461p*RS146pe - 1./81.*me2*RSe461p*
         RS156pe - 2./81.*me2*RSe461p*RSe561p + 1./81.*me2*pow(
         RSe461p,2) - 1./81.*me2*RSp561e*RSe563p + 1./81.*me2*RSp561e*
         RSe463p + 1./81.*me2*RSp561e*RSe561p - 1./81.*me2*RSp561e*
         RSe461p + 1./81.*me2*RSp461e*RSe563p - 1./81.*me2*RSp461e*
         RSe463p - 1./81.*me2*RSp461e*RSe561p + 1./81.*me2*RSp461e*
         RSe461p - 1./162.*m55*RSe563p*RS346pe - 1./162.*m55*RSe563p*
         RS356pe - 1./324.*m55*pow(RSe563p,2) - 1./54.*m55*RSe463p*
         RS346pe - 1./162.*m55*RSe463p*RS356pe - 1./162.*m55*RSe463p*
         RSe563p - 1./108.*m55*pow(RSe463p,2) + 1./162.*m55*RS146pe*
         RSe563p + 1./54.*m55*RS146pe*RSe463p + 1./162.*m55*RS156pe*
         RSe563p + 1./162.*m55*RS156pe*RSe463p + 1./162.*m55*RSe561p*
         RS346pe + 1./162.*m55*RSe561p*RS356pe + 1./162.*m55*RSe561p*
         RSe563p + 1./162.*m55*RSe561p*RSe463p - 1./162.*m55*RSe561p*
         RS146pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./162.*m55*
         RSe561p*RS156pe - 1./324.*m55*pow(RSe561p,2) + 1./54.*m55*
         RSe461p*RS346pe + 1./162.*m55*RSe461p*RS356pe + 1./162.*m55*
         RSe461p*RSe563p + 1./54.*m55*RSe461p*RSe463p - 1./54.*m55*
         RSe461p*RS146pe - 1./162.*m55*RSe461p*RS156pe - 1./162.*m55*
         RSe461p*RSe561p - 1./108.*m55*pow(RSe461p,2) - 1./81.*m45*
         RSe563p*RS346pe + 1./81.*m45*RSe563p*RS356pe + 1./162.*m45*
         pow(RSe563p,2) + 1./81.*m45*RSe463p*RS346pe - 1./81.*m45*
         RSe463p*RS356pe - 1./81.*m45*RSe463p*RSe563p + 1./162.*m45*
         pow(RSe463p,2) + 1./81.*m45*RS146pe*RSe563p - 1./81.*m45*
         RS146pe*RSe463p - 1./81.*m45*RS156pe*RSe563p + 1./81.*m45*
         RS156pe*RSe463p + 1./81.*m45*RSe561p*RS346pe - 1./81.*m45*
         RSe561p*RS356pe - 1./81.*m45*RSe561p*RSe563p + 1./81.*m45*
         RSe561p*RSe463p - 1./81.*m45*RSe561p*RS146pe + 1./81.*m45*
         RSe561p*RS156pe + 1./162.*m45*pow(RSe561p,2) - 1./81.*m45*
         RSe461p*RS346pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./81.*m45*RSe461p*
         RS356pe + 1./81.*m45*RSe461p*RSe563p - 1./81.*m45*RSe461p*
         RSe463p + 1./81.*m45*RSe461p*RS146pe - 1./81.*m45*RSe461p*
         RS156pe - 1./81.*m45*RSe461p*RSe561p + 1./162.*m45*pow(
         RSe461p,2) - 1./162.*m44*RSe563p*RS346pe - 1./54.*m44*RSe563p*
         RS356pe - 1./108.*m44*pow(RSe563p,2) - 1./162.*m44*RSe463p*
         RS346pe - 1./162.*m44*RSe463p*RS356pe - 1./162.*m44*RSe463p*
         RSe563p - 1./324.*m44*pow(RSe463p,2) + 1./162.*m44*RS146pe*
         RSe563p + 1./162.*m44*RS146pe*RSe463p + 1./54.*m44*RS156pe*
         RSe563p + 1./162.*m44*RS156pe*RSe463p + 1./162.*m44*RSe561p*
         RS346pe + 1./54.*m44*RSe561p*RS356pe + 1./54.*m44*RSe561p*
         RSe563p + 1./162.*m44*RSe561p*RSe463p - 1./162.*m44*RSe561p*
         RS146pe - 1./54.*m44*RSe561p*RS156pe - 1./108.*m44*pow(
         RSe561p,2) + 1./162.*m44*RSe461p*RS346pe + 1./162.*m44*RSe461p
         *RS356pe + 1./162.*m44*RSe461p*RSe563p + 1./162.*m44*RSe461p*
         RSe463p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./162.*m44*
         RSe461p*RS146pe - 1./162.*m44*RSe461p*RS156pe - 1./162.*m44*
         RSe461p*RSe561p - 1./324.*m44*pow(RSe461p,2) + 1./54.*m33*
         RSe563p*RS346pe - 1./54.*m33*RSe563p*RS356pe - 1./108.*m33*
         pow(RSe563p,2) - 1./54.*m33*RSe463p*RS346pe + 1./54.*m33*
         RSe463p*RS356pe + 1./54.*m33*RSe463p*RSe563p - 1./108.*m33*
         pow(RSe463p,2) + 1./162.*m33*RS146pe*RSe563p - 1./162.*m33*
         RS146pe*RSe463p - 1./162.*m33*RS156pe*RSe563p + 1./162.*m33*
         RS156pe*RSe463p + 1./162.*m33*RSe561p*RS346pe - 1./162.*m33*
         RSe561p*RS356pe - 1./162.*m33*RSe561p*RSe563p + 1./162.*m33*
         RSe561p*RSe463p + 1./162.*m33*RSe561p*RS146pe - 1./162.*m33*
         RSe561p*RS156pe - 1./324.*m33*pow(RSe561p,2) - 1./162.*m33*
         RSe461p*RS346pe + 1./162.*m33*RSe461p*RS356pe + 1./162.*m33*
         RSe461p*RSe563p - 1./162.*m33*RSe461p*RSe463p - 1./162.*m33*
         RSe461p*RS146pe + 1./162.*m33*RSe461p*RS156pe + 1./162.*m33*
         RSe461p*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./324.*m33*pow(
         RSe461p,2) + 1./81.*m33*me2*RN3e*RSe563p*RS346pe - 1./81.*m33*
         me2*RN3e*RSe563p*RS356pe - 1./81.*m33*me2*RN3e*RSe463p*RS346pe
          + 1./81.*m33*me2*RN3e*RSe463p*RS356pe - 1./81.*m33*me2*RN3e*
         RSe561p*RS346pe + 1./81.*m33*me2*RN3e*RSe561p*RS356pe + 1./81.
         *m33*me2*RN3e*RSe461p*RS346pe - 1./81.*m33*me2*RN3e*RSe461p*
         RS356pe - 1./324.*m33*m55*RN3e*RSe563p*RS346pe - 1./324.*m33*
         m55*RN3e*RSe563p*RS356pe - 1./108.*m33*m55*RN3e*RSe463p*
         RS346pe - 1./324.*m33*m55*RN3e*RSe463p*RS356pe - 1./108.*m33*
         m55*RN3e*RSe561p*RS346pe - 1./108.*m33*m55*RN3e*RSe561p*
         RS356pe - 1./36.*m33*m55*RN3e*RSe461p*RS346pe - 1./108.*m33*
         m55*RN3e*RSe461p*RS356pe - 1./324.*m33*m55*RN1e*RS146pe*
         RSe563p - 1./108.*m33*m55*RN1e*RS146pe*RSe463p - 1./324.*m33*
         m55*RN1e*RS156pe*RSe563p - 1./324.*m33*m55*RN1e*RS156pe*
         RSe463p - 1./324.*m33*m55*RN1e*RSe561p*RS146pe - 1./324.*m33*
         m55*RN1e*RSe561p*RS156pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./108.*m33*m55*
         RN1e*RSe461p*RS146pe - 1./324.*m33*m55*RN1e*RSe461p*RS156pe - 
         1./162.*m33*m45*RN3e*RSe563p*RS346pe + 1./162.*m33*m45*RN3e*
         RSe563p*RS356pe + 1./162.*m33*m45*RN3e*RSe463p*RS346pe - 1./
         162.*m33*m45*RN3e*RSe463p*RS356pe - 1./54.*m33*m45*RN3e*
         RSe561p*RS346pe + 1./54.*m33*m45*RN3e*RSe561p*RS356pe + 1./54.
         *m33*m45*RN3e*RSe461p*RS346pe - 1./54.*m33*m45*RN3e*RSe461p*
         RS356pe - 1./162.*m33*m45*RN1e*RS146pe*RSe563p + 1./162.*m33*
         m45*RN1e*RS146pe*RSe463p + 1./162.*m33*m45*RN1e*RS156pe*
         RSe563p - 1./162.*m33*m45*RN1e*RS156pe*RSe463p - 1./162.*m33*
         m45*RN1e*RSe561p*RS146pe + 1./162.*m33*m45*RN1e*RSe561p*
         RS156pe + 1./162.*m33*m45*RN1e*RSe461p*RS146pe - 1./162.*m33*
         m45*RN1e*RSe461p*RS156pe - 1./324.*m33*m44*RN3e*RSe563p*
         RS346pe - 1./108.*m33*m44*RN3e*RSe563p*RS356pe - 1./324.*m33*
         m44*RN3e*RSe463p*RS346pe - 1./324.*m33*m44*RN3e*RSe463p*
         RS356pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./108.*m33*m44*
         RN3e*RSe561p*RS346pe - 1./36.*m33*m44*RN3e*RSe561p*RS356pe - 1.
         /108.*m33*m44*RN3e*RSe461p*RS346pe - 1./108.*m33*m44*RN3e*
         RSe461p*RS356pe - 1./324.*m33*m44*RN1e*RS146pe*RSe563p - 1./
         324.*m33*m44*RN1e*RS146pe*RSe463p - 1./108.*m33*m44*RN1e*
         RS156pe*RSe563p - 1./324.*m33*m44*RN1e*RS156pe*RSe463p - 1./
         324.*m33*m44*RN1e*RSe561p*RS146pe - 1./108.*m33*m44*RN1e*
         RSe561p*RS156pe - 1./324.*m33*m44*RN1e*RSe461p*RS146pe - 1./
         324.*m33*m44*RN1e*RSe461p*RS156pe - 1./54.*pow(m33,2)*RN3e*
         RSe563p*RS346pe + 1./54.*pow(m33,2)*RN3e*RSe563p*RS356pe + 1./
         54.*pow(m33,2)*RN3e*RSe463p*RS346pe - 1./54.*pow(m33,2)*RN3e*
         RSe463p*RS356pe - 1./162.*pow(m33,2)*RN3e*RSe561p*RS346pe + 1./
         162.*pow(m33,2)*RN3e*RSe561p*RS356pe + 1./162.*pow(m33,2)*RN3e
         *RSe461p*RS346pe - 1./162.*pow(m33,2)*RN3e*RSe461p*RS356pe - 1.
         /81.*m13*RSe563p*RS346pe + 1./81.*m13*RSe563p*RS356pe + 1./162.
         *m13*pow(RSe563p,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./81.*m13*RSe463p*
         RS346pe - 1./81.*m13*RSe463p*RS356pe - 1./81.*m13*RSe463p*
         RSe563p + 1./162.*m13*pow(RSe463p,2) + 1./81.*m13*RS146pe*
         RSe563p - 1./81.*m13*RS146pe*RSe463p - 1./81.*m13*RS156pe*
         RSe563p + 1./81.*m13*RS156pe*RSe463p + 1./81.*m13*RSe561p*
         RS346pe - 1./81.*m13*RSe561p*RS356pe - 1./81.*m13*RSe561p*
         RSe563p + 1./81.*m13*RSe561p*RSe463p - 1./81.*m13*RSe561p*
         RS146pe + 1./81.*m13*RSe561p*RS156pe + 1./162.*m13*pow(
         RSe561p,2) - 1./81.*m13*RSe461p*RS346pe + 1./81.*m13*RSe461p*
         RS356pe + 1./81.*m13*RSe461p*RSe563p - 1./81.*m13*RSe461p*
         RSe463p + 1./81.*m13*RSe461p*RS146pe - 1./81.*m13*RSe461p*
         RS156pe - 1./81.*m13*RSe461p*RSe561p + 1./162.*m13*pow(
         RSe461p,2) + 1./162.*m13*m55*RN3e*RSe563p*RS346pe + 1./162.*
         m13*m55*RN3e*RSe563p*RS356pe + 1./54.*m13*m55*RN3e*RSe463p*
         RS346pe + 1./162.*m13*m55*RN3e*RSe463p*RS356pe - 1./162.*m13*
         m55*RN3e*RSe561p*RS346pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./162.*m13*m55*
         RN3e*RSe561p*RS356pe - 1./54.*m13*m55*RN3e*RSe461p*RS346pe - 1.
         /162.*m13*m55*RN3e*RSe461p*RS356pe - 1./162.*m13*m55*RN1e*
         RS146pe*RSe563p - 1./54.*m13*m55*RN1e*RS146pe*RSe463p - 1./162.
         *m13*m55*RN1e*RS156pe*RSe563p - 1./162.*m13*m55*RN1e*RS156pe*
         RSe463p + 1./162.*m13*m55*RN1e*RSe561p*RS146pe + 1./162.*m13*
         m55*RN1e*RSe561p*RS156pe + 1./54.*m13*m55*RN1e*RSe461p*RS146pe
          + 1./162.*m13*m55*RN1e*RSe461p*RS156pe + 1./81.*m13*m45*RN3e*
         RSe563p*RS346pe - 1./81.*m13*m45*RN3e*RSe563p*RS356pe - 1./81.
         *m13*m45*RN3e*RSe463p*RS346pe + 1./81.*m13*m45*RN3e*RSe463p*
         RS356pe - 1./81.*m13*m45*RN3e*RSe561p*RS346pe + 1./81.*m13*m45
         *RN3e*RSe561p*RS356pe + 1./81.*m13*m45*RN3e*RSe461p*RS346pe - 
         1./81.*m13*m45*RN3e*RSe461p*RS356pe - 1./81.*m13*m45*RN1e*
         RS146pe*RSe563p + 1./81.*m13*m45*RN1e*RS146pe*RSe463p + 1./81.
         *m13*m45*RN1e*RS156pe*RSe563p - 1./81.*m13*m45*RN1e*RS156pe*
         RSe463p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./81.*m13*m45*RN1e*
         RSe561p*RS146pe - 1./81.*m13*m45*RN1e*RSe561p*RS156pe - 1./81.
         *m13*m45*RN1e*RSe461p*RS146pe + 1./81.*m13*m45*RN1e*RSe461p*
         RS156pe + 1./162.*m13*m44*RN3e*RSe563p*RS346pe + 1./54.*m13*
         m44*RN3e*RSe563p*RS356pe + 1./162.*m13*m44*RN3e*RSe463p*
         RS346pe + 1./162.*m13*m44*RN3e*RSe463p*RS356pe - 1./162.*m13*
         m44*RN3e*RSe561p*RS346pe - 1./54.*m13*m44*RN3e*RSe561p*RS356pe
          - 1./162.*m13*m44*RN3e*RSe461p*RS346pe - 1./162.*m13*m44*RN3e
         *RSe461p*RS356pe - 1./162.*m13*m44*RN1e*RS146pe*RSe563p - 1./
         162.*m13*m44*RN1e*RS146pe*RSe463p - 1./54.*m13*m44*RN1e*
         RS156pe*RSe563p - 1./162.*m13*m44*RN1e*RS156pe*RSe463p + 1./
         162.*m13*m44*RN1e*RSe561p*RS146pe + 1./54.*m13*m44*RN1e*
         RSe561p*RS156pe + 1./162.*m13*m44*RN1e*RSe461p*RS146pe + 1./
         162.*m13*m44*RN1e*RSe461p*RS156pe + 1./81.*m13*m33*RN3e*
         RSe563p*RS346pe - 1./81.*m13*m33*RN3e*RSe563p*RS356pe - 1./81.
         *m13*m33*RN3e*RSe463p*RS346pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./81.*m13*m33*RN3e*
         RSe463p*RS356pe - 1./81.*m13*m33*RN3e*RSe561p*RS346pe + 1./81.
         *m13*m33*RN3e*RSe561p*RS356pe + 1./81.*m13*m33*RN3e*RSe461p*
         RS346pe - 1./81.*m13*m33*RN3e*RSe461p*RS356pe + 1./162.*m11*
         RSe563p*RS346pe - 1./162.*m11*RSe563p*RS356pe - 1./324.*m11*
         pow(RSe563p,2) - 1./162.*m11*RSe463p*RS346pe + 1./162.*m11*
         RSe463p*RS356pe + 1./162.*m11*RSe463p*RSe563p - 1./324.*m11*
         pow(RSe463p,2) + 1./162.*m11*RS146pe*RSe563p - 1./162.*m11*
         RS146pe*RSe463p - 1./162.*m11*RS156pe*RSe563p + 1./162.*m11*
         RS156pe*RSe463p + 1./162.*m11*RSe561p*RS346pe - 1./162.*m11*
         RSe561p*RS356pe - 1./162.*m11*RSe561p*RSe563p + 1./162.*m11*
         RSe561p*RSe463p + 1./54.*m11*RSe561p*RS146pe - 1./54.*m11*
         RSe561p*RS156pe - 1./108.*m11*pow(RSe561p,2) - 1./162.*m11*
         RSe461p*RS346pe + 1./162.*m11*RSe461p*RS356pe + 1./162.*m11*
         RSe461p*RSe563p - 1./162.*m11*RSe461p*RSe463p - 1./54.*m11*
         RSe461p*RS146pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./54.*m11*RSe461p*
         RS156pe + 1./54.*m11*RSe461p*RSe561p - 1./108.*m11*pow(
         RSe461p,2) - 1./81.*m11*me2*RN1e*RS146pe*RSe563p + 1./81.*m11*
         me2*RN1e*RS146pe*RSe463p + 1./81.*m11*me2*RN1e*RS156pe*RSe563p
          - 1./81.*m11*me2*RN1e*RS156pe*RSe463p + 1./81.*m11*me2*RN1e*
         RSe561p*RS146pe - 1./81.*m11*me2*RN1e*RSe561p*RS156pe - 1./81.
         *m11*me2*RN1e*RSe461p*RS146pe + 1./81.*m11*me2*RN1e*RSe461p*
         RS156pe - 1./324.*m11*m55*RN3e*RSe563p*RS346pe - 1./324.*m11*
         m55*RN3e*RSe563p*RS356pe - 1./108.*m11*m55*RN3e*RSe463p*
         RS346pe - 1./324.*m11*m55*RN3e*RSe463p*RS356pe - 1./324.*m11*
         m55*RN3e*RSe561p*RS346pe - 1./324.*m11*m55*RN3e*RSe561p*
         RS356pe - 1./108.*m11*m55*RN3e*RSe461p*RS346pe - 1./324.*m11*
         m55*RN3e*RSe461p*RS356pe - 1./108.*m11*m55*RN1e*RS146pe*
         RSe563p - 1./36.*m11*m55*RN1e*RS146pe*RSe463p - 1./108.*m11*
         m55*RN1e*RS156pe*RSe563p - 1./108.*m11*m55*RN1e*RS156pe*
         RSe463p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./324.*m11*m55*
         RN1e*RSe561p*RS146pe - 1./324.*m11*m55*RN1e*RSe561p*RS156pe - 
         1./108.*m11*m55*RN1e*RSe461p*RS146pe - 1./324.*m11*m55*RN1e*
         RSe461p*RS156pe - 1./162.*m11*m45*RN3e*RSe563p*RS346pe + 1./
         162.*m11*m45*RN3e*RSe563p*RS356pe + 1./162.*m11*m45*RN3e*
         RSe463p*RS346pe - 1./162.*m11*m45*RN3e*RSe463p*RS356pe - 1./
         162.*m11*m45*RN3e*RSe561p*RS346pe + 1./162.*m11*m45*RN3e*
         RSe561p*RS356pe + 1./162.*m11*m45*RN3e*RSe461p*RS346pe - 1./
         162.*m11*m45*RN3e*RSe461p*RS356pe - 1./54.*m11*m45*RN1e*
         RS146pe*RSe563p + 1./54.*m11*m45*RN1e*RS146pe*RSe463p + 1./54.
         *m11*m45*RN1e*RS156pe*RSe563p - 1./54.*m11*m45*RN1e*RS156pe*
         RSe463p - 1./162.*m11*m45*RN1e*RSe561p*RS146pe + 1./162.*m11*
         m45*RN1e*RSe561p*RS156pe + 1./162.*m11*m45*RN1e*RSe461p*
         RS146pe - 1./162.*m11*m45*RN1e*RSe461p*RS156pe - 1./324.*m11*
         m44*RN3e*RSe563p*RS346pe - 1./108.*m11*m44*RN3e*RSe563p*
         RS356pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./324.*m11*m44*
         RN3e*RSe463p*RS346pe - 1./324.*m11*m44*RN3e*RSe463p*RS356pe - 
         1./324.*m11*m44*RN3e*RSe561p*RS346pe - 1./108.*m11*m44*RN3e*
         RSe561p*RS356pe - 1./324.*m11*m44*RN3e*RSe461p*RS346pe - 1./
         324.*m11*m44*RN3e*RSe461p*RS356pe - 1./108.*m11*m44*RN1e*
         RS146pe*RSe563p - 1./108.*m11*m44*RN1e*RS146pe*RSe463p - 1./36.
         *m11*m44*RN1e*RS156pe*RSe563p - 1./108.*m11*m44*RN1e*RS156pe*
         RSe463p - 1./324.*m11*m44*RN1e*RSe561p*RS146pe - 1./108.*m11*
         m44*RN1e*RSe561p*RS156pe - 1./324.*m11*m44*RN1e*RSe461p*
         RS146pe - 1./324.*m11*m44*RN1e*RSe461p*RS156pe - 1./162.*m11*
         m33*RN3e*RSe563p*RS346pe + 1./162.*m11*m33*RN3e*RSe563p*
         RS356pe + 1./162.*m11*m33*RN3e*RSe463p*RS346pe - 1./162.*m11*
         m33*RN3e*RSe463p*RS356pe - 1./162.*m11*m33*RN3e*RSe561p*
         RS346pe + 1./162.*m11*m33*RN3e*RSe561p*RS356pe + 1./162.*m11*
         m33*RN3e*RSe461p*RS346pe - 1./162.*m11*m33*RN3e*RSe461p*
         RS356pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./162.*m11*m33*
         RN1e*RS146pe*RSe563p + 1./162.*m11*m33*RN1e*RS146pe*RSe463p + 
         1./162.*m11*m33*RN1e*RS156pe*RSe563p - 1./162.*m11*m33*RN1e*
         RS156pe*RSe463p - 1./162.*m11*m33*RN1e*RSe561p*RS146pe + 1./
         162.*m11*m33*RN1e*RSe561p*RS156pe + 1./162.*m11*m33*RN1e*
         RSe461p*RS146pe - 1./162.*m11*m33*RN1e*RSe461p*RS156pe - 1./81.
         *m11*m13*RN1e*RS146pe*RSe563p + 1./81.*m11*m13*RN1e*RS146pe*
         RSe463p + 1./81.*m11*m13*RN1e*RS156pe*RSe563p - 1./81.*m11*m13
         *RN1e*RS156pe*RSe463p + 1./81.*m11*m13*RN1e*RSe561p*RS146pe - 
         1./81.*m11*m13*RN1e*RSe561p*RS156pe - 1./81.*m11*m13*RN1e*
         RSe461p*RS146pe + 1./81.*m11*m13*RN1e*RSe461p*RS156pe - 1./162.
         *pow(m11,2)*RN1e*RS146pe*RSe563p + 1./162.*pow(m11,2)*RN1e*
         RS146pe*RSe463p + 1./162.*pow(m11,2)*RN1e*RS156pe*RSe563p - 1./
         162.*pow(m11,2)*RN1e*RS156pe*RSe463p - 1./54.*pow(m11,2)*RN1e*
         RSe561p*RS146pe + 1./54.*pow(m11,2)*RN1e*RSe561p*RS156pe + 1./
         54.*pow(m11,2)*RN1e*RSe461p*RS146pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./54.*pow(m11,2)*
         RN1e*RSe461p*RS156pe - 1./81.*RN(mp2,me2)*mp2*me2*RSp563e*
         RSe563p + 1./81.*RN(mp2,me2)*mp2*me2*RSp563e*RSe463p + 1./81.*
         RN(mp2,me2)*mp2*me2*RSp463e*RSe563p - 1./81.*RN(mp2,me2)*mp2*
         me2*RSp463e*RSe463p + 1./81.*RN(mp2,me2)*mp2*me2*RSe561p*
         RSp563e - 1./81.*RN(mp2,me2)*mp2*me2*RSe561p*RSp463e - 1./81.*
         RN(mp2,me2)*mp2*me2*RSe461p*RSp563e + 1./81.*RN(mp2,me2)*mp2*
         me2*RSe461p*RSp463e + 1./81.*RN(mp2,me2)*mp2*me2*RSp561e*
         RSe563p - 1./81.*RN(mp2,me2)*mp2*me2*RSp561e*RSe463p - 1./81.*
         RN(mp2,me2)*mp2*me2*RSp561e*RSe561p + 1./81.*RN(mp2,me2)*mp2*
         me2*RSp561e*RSe461p - 1./81.*RN(mp2,me2)*mp2*me2*RSp461e*
         RSe563p + 1./81.*RN(mp2,me2)*mp2*me2*RSp461e*RSe463p + 1./81.*
         RN(mp2,me2)*mp2*me2*RSp461e*RSe561p - 1./81.*RN(mp2,me2)*mp2*
         me2*RSp461e*RSe461p + 1./162.*RN(mp2,me2)*m55*me2*RSp563e*
         RSe563p + 1./162.*RN(mp2,me2)*m55*me2*RSp563e*RSe463p + 1./162.
         *RN(mp2,me2)*m55*me2*RSp463e*RSe563p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./54.*RN(mp2,me2)*
         m55*me2*RSp463e*RSe463p - 1./162.*RN(mp2,me2)*m55*me2*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m55*me2*RSe561p*RSp463e - 1./162.
         *RN(mp2,me2)*m55*me2*RSe461p*RSp563e - 1./54.*RN(mp2,me2)*m55*
         me2*RSe461p*RSp463e - 1./162.*RN(mp2,me2)*m55*me2*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m55*me2*RSp561e*RSe463p + 1./162.
         *RN(mp2,me2)*m55*me2*RSp561e*RSe561p + 1./162.*RN(mp2,me2)*m55
         *me2*RSp561e*RSe461p - 1./162.*RN(mp2,me2)*m55*me2*RSp461e*
         RSe563p - 1./54.*RN(mp2,me2)*m55*me2*RSp461e*RSe463p + 1./162.
         *RN(mp2,me2)*m55*me2*RSp461e*RSe561p + 1./54.*RN(mp2,me2)*m55*
         me2*RSp461e*RSe461p - 1./81.*RN(mp2,me2)*m45*me2*RSp563e*
         RSe563p + 1./81.*RN(mp2,me2)*m45*me2*RSp563e*RSe463p + 1./81.*
         RN(mp2,me2)*m45*me2*RSp463e*RSe563p - 1./81.*RN(mp2,me2)*m45*
         me2*RSp463e*RSe463p + 1./81.*RN(mp2,me2)*m45*me2*RSe561p*
         RSp563e - 1./81.*RN(mp2,me2)*m45*me2*RSe561p*RSp463e - 1./81.*
         RN(mp2,me2)*m45*me2*RSe461p*RSp563e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./81.*RN(mp2,me2)*
         m45*me2*RSe461p*RSp463e + 1./81.*RN(mp2,me2)*m45*me2*RSp561e*
         RSe563p - 1./81.*RN(mp2,me2)*m45*me2*RSp561e*RSe463p - 1./81.*
         RN(mp2,me2)*m45*me2*RSp561e*RSe561p + 1./81.*RN(mp2,me2)*m45*
         me2*RSp561e*RSe461p - 1./81.*RN(mp2,me2)*m45*me2*RSp461e*
         RSe563p + 1./81.*RN(mp2,me2)*m45*me2*RSp461e*RSe463p + 1./81.*
         RN(mp2,me2)*m45*me2*RSp461e*RSe561p - 1./81.*RN(mp2,me2)*m45*
         me2*RSp461e*RSe461p + 1./54.*RN(mp2,me2)*m44*me2*RSp563e*
         RSe563p + 1./162.*RN(mp2,me2)*m44*me2*RSp563e*RSe463p + 1./162.
         *RN(mp2,me2)*m44*me2*RSp463e*RSe563p + 1./162.*RN(mp2,me2)*m44
         *me2*RSp463e*RSe463p - 1./54.*RN(mp2,me2)*m44*me2*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m44*me2*RSe561p*RSp463e - 1./162.
         *RN(mp2,me2)*m44*me2*RSe461p*RSp563e - 1./162.*RN(mp2,me2)*m44
         *me2*RSe461p*RSp463e - 1./54.*RN(mp2,me2)*m44*me2*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m44*me2*RSp561e*RSe463p + 1./54.
         *RN(mp2,me2)*m44*me2*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./162.*RN(mp2,me2)*
         m44*me2*RSp561e*RSe461p - 1./162.*RN(mp2,me2)*m44*me2*RSp461e*
         RSe563p - 1./162.*RN(mp2,me2)*m44*me2*RSp461e*RSe463p + 1./162.
         *RN(mp2,me2)*m44*me2*RSp461e*RSe561p + 1./162.*RN(mp2,me2)*m44
         *me2*RSp461e*RSe461p + 1./54.*RN(mp2,me2)*m33*me2*RSp563e*
         RSe563p - 1./54.*RN(mp2,me2)*m33*me2*RSp563e*RSe463p - 1./54.*
         RN(mp2,me2)*m33*me2*RSp463e*RSe563p + 1./54.*RN(mp2,me2)*m33*
         me2*RSp463e*RSe463p + 1./162.*RN(mp2,me2)*m33*me2*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m33*me2*RSe561p*RSp463e - 1./162.
         *RN(mp2,me2)*m33*me2*RSe461p*RSp563e + 1./162.*RN(mp2,me2)*m33
         *me2*RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m33*me2*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m33*me2*RSp561e*RSe463p + 1./162.
         *RN(mp2,me2)*m33*me2*RSp561e*RSe561p - 1./162.*RN(mp2,me2)*m33
         *me2*RSp561e*RSe461p - 1./162.*RN(mp2,me2)*m33*me2*RSp461e*
         RSe563p + 1./162.*RN(mp2,me2)*m33*me2*RSp461e*RSe463p - 1./162.
         *RN(mp2,me2)*m33*me2*RSp461e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./162.*RN(mp2,me2)*
         m33*me2*RSp461e*RSe461p - 1./108.*RN(mp2,me2)*m33*m55*RSp563e*
         RSe563p - 1./108.*RN(mp2,me2)*m33*m55*RSp563e*RSe463p - 1./108.
         *RN(mp2,me2)*m33*m55*RSp463e*RSe563p - 1./36.*RN(mp2,me2)*m33*
         m55*RSp463e*RSe463p - 1./324.*RN(mp2,me2)*m33*m55*RSe561p*
         RSp563e - 1./324.*RN(mp2,me2)*m33*m55*RSe561p*RSp463e - 1./324.
         *RN(mp2,me2)*m33*m55*RSe461p*RSp563e - 1./108.*RN(mp2,me2)*m33
         *m55*RSe461p*RSp463e - 1./324.*RN(mp2,me2)*m33*m55*RSp561e*
         RSe563p - 1./324.*RN(mp2,me2)*m33*m55*RSp561e*RSe463p - 1./324.
         *RN(mp2,me2)*m33*m55*RSp561e*RSe561p - 1./324.*RN(mp2,me2)*m33
         *m55*RSp561e*RSe461p - 1./324.*RN(mp2,me2)*m33*m55*RSp461e*
         RSe563p - 1./108.*RN(mp2,me2)*m33*m55*RSp461e*RSe463p - 1./324.
         *RN(mp2,me2)*m33*m55*RSp461e*RSe561p - 1./108.*RN(mp2,me2)*m33
         *m55*RSp461e*RSe461p + 1./54.*RN(mp2,me2)*m33*m45*RSp563e*
         RSe563p - 1./54.*RN(mp2,me2)*m33*m45*RSp563e*RSe463p - 1./54.*
         RN(mp2,me2)*m33*m45*RSp463e*RSe563p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./54.*RN(mp2,me2)*
         m33*m45*RSp463e*RSe463p + 1./162.*RN(mp2,me2)*m33*m45*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m33*m45*RSe561p*RSp463e - 1./162.
         *RN(mp2,me2)*m33*m45*RSe461p*RSp563e + 1./162.*RN(mp2,me2)*m33
         *m45*RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m33*m45*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m33*m45*RSp561e*RSe463p + 1./162.
         *RN(mp2,me2)*m33*m45*RSp561e*RSe561p - 1./162.*RN(mp2,me2)*m33
         *m45*RSp561e*RSe461p - 1./162.*RN(mp2,me2)*m33*m45*RSp461e*
         RSe563p + 1./162.*RN(mp2,me2)*m33*m45*RSp461e*RSe463p - 1./162.
         *RN(mp2,me2)*m33*m45*RSp461e*RSe561p + 1./162.*RN(mp2,me2)*m33
         *m45*RSp461e*RSe461p - 1./36.*RN(mp2,me2)*m33*m44*RSp563e*
         RSe563p - 1./108.*RN(mp2,me2)*m33*m44*RSp563e*RSe463p - 1./108.
         *RN(mp2,me2)*m33*m44*RSp463e*RSe563p - 1./108.*RN(mp2,me2)*m33
         *m44*RSp463e*RSe463p - 1./108.*RN(mp2,me2)*m33*m44*RSe561p*
         RSp563e - 1./324.*RN(mp2,me2)*m33*m44*RSe561p*RSp463e - 1./324.
         *RN(mp2,me2)*m33*m44*RSe461p*RSp563e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./324.*RN(mp2,me2
         )*m33*m44*RSe461p*RSp463e - 1./108.*RN(mp2,me2)*m33*m44*
         RSp561e*RSe563p - 1./324.*RN(mp2,me2)*m33*m44*RSp561e*RSe463p
          - 1./108.*RN(mp2,me2)*m33*m44*RSp561e*RSe561p - 1./324.*RN(
         mp2,me2)*m33*m44*RSp561e*RSe461p - 1./324.*RN(mp2,me2)*m33*m44
         *RSp461e*RSe563p - 1./324.*RN(mp2,me2)*m33*m44*RSp461e*RSe463p
          - 1./324.*RN(mp2,me2)*m33*m44*RSp461e*RSe561p - 1./324.*RN(
         mp2,me2)*m33*m44*RSp461e*RSe461p - 1./81.*RN(mp2,me2)*m13*me2*
         RSp563e*RSe563p + 1./81.*RN(mp2,me2)*m13*me2*RSp563e*RSe463p
          + 1./81.*RN(mp2,me2)*m13*me2*RSp463e*RSe563p - 1./81.*RN(mp2,
         me2)*m13*me2*RSp463e*RSe463p + 1./81.*RN(mp2,me2)*m13*me2*
         RSe561p*RSp563e - 1./81.*RN(mp2,me2)*m13*me2*RSe561p*RSp463e
          - 1./81.*RN(mp2,me2)*m13*me2*RSe461p*RSp563e + 1./81.*RN(mp2,
         me2)*m13*me2*RSe461p*RSp463e + 1./81.*RN(mp2,me2)*m13*me2*
         RSp561e*RSe563p - 1./81.*RN(mp2,me2)*m13*me2*RSp561e*RSe463p
          - 1./81.*RN(mp2,me2)*m13*me2*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./81.*RN(mp2,me2)*
         m13*me2*RSp561e*RSe461p - 1./81.*RN(mp2,me2)*m13*me2*RSp461e*
         RSe563p + 1./81.*RN(mp2,me2)*m13*me2*RSp461e*RSe463p + 1./81.*
         RN(mp2,me2)*m13*me2*RSp461e*RSe561p - 1./81.*RN(mp2,me2)*m13*
         me2*RSp461e*RSe461p + 1./162.*RN(mp2,me2)*m13*m55*RSp563e*
         RSe563p + 1./162.*RN(mp2,me2)*m13*m55*RSp563e*RSe463p + 1./162.
         *RN(mp2,me2)*m13*m55*RSp463e*RSe563p + 1./54.*RN(mp2,me2)*m13*
         m55*RSp463e*RSe463p - 1./162.*RN(mp2,me2)*m13*m55*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m13*m55*RSe561p*RSp463e - 1./162.
         *RN(mp2,me2)*m13*m55*RSe461p*RSp563e - 1./54.*RN(mp2,me2)*m13*
         m55*RSe461p*RSp463e - 1./162.*RN(mp2,me2)*m13*m55*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m13*m55*RSp561e*RSe463p + 1./162.
         *RN(mp2,me2)*m13*m55*RSp561e*RSe561p + 1./162.*RN(mp2,me2)*m13
         *m55*RSp561e*RSe461p - 1./162.*RN(mp2,me2)*m13*m55*RSp461e*
         RSe563p - 1./54.*RN(mp2,me2)*m13*m55*RSp461e*RSe463p + 1./162.
         *RN(mp2,me2)*m13*m55*RSp461e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./54.*RN(mp2,me2)*
         m13*m55*RSp461e*RSe461p - 1./81.*RN(mp2,me2)*m13*m45*RSp563e*
         RSe563p + 1./81.*RN(mp2,me2)*m13*m45*RSp563e*RSe463p + 1./81.*
         RN(mp2,me2)*m13*m45*RSp463e*RSe563p - 1./81.*RN(mp2,me2)*m13*
         m45*RSp463e*RSe463p + 1./81.*RN(mp2,me2)*m13*m45*RSe561p*
         RSp563e - 1./81.*RN(mp2,me2)*m13*m45*RSe561p*RSp463e - 1./81.*
         RN(mp2,me2)*m13*m45*RSe461p*RSp563e + 1./81.*RN(mp2,me2)*m13*
         m45*RSe461p*RSp463e + 1./81.*RN(mp2,me2)*m13*m45*RSp561e*
         RSe563p - 1./81.*RN(mp2,me2)*m13*m45*RSp561e*RSe463p - 1./81.*
         RN(mp2,me2)*m13*m45*RSp561e*RSe561p + 1./81.*RN(mp2,me2)*m13*
         m45*RSp561e*RSe461p - 1./81.*RN(mp2,me2)*m13*m45*RSp461e*
         RSe563p + 1./81.*RN(mp2,me2)*m13*m45*RSp461e*RSe463p + 1./81.*
         RN(mp2,me2)*m13*m45*RSp461e*RSe561p - 1./81.*RN(mp2,me2)*m13*
         m45*RSp461e*RSe461p + 1./54.*RN(mp2,me2)*m13*m44*RSp563e*
         RSe563p + 1./162.*RN(mp2,me2)*m13*m44*RSp563e*RSe463p + 1./162.
         *RN(mp2,me2)*m13*m44*RSp463e*RSe563p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./162.*RN(mp2,me2)*
         m13*m44*RSp463e*RSe463p - 1./54.*RN(mp2,me2)*m13*m44*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m13*m44*RSe561p*RSp463e - 1./162.
         *RN(mp2,me2)*m13*m44*RSe461p*RSp563e - 1./162.*RN(mp2,me2)*m13
         *m44*RSe461p*RSp463e - 1./54.*RN(mp2,me2)*m13*m44*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m13*m44*RSp561e*RSe463p + 1./54.
         *RN(mp2,me2)*m13*m44*RSp561e*RSe561p + 1./162.*RN(mp2,me2)*m13
         *m44*RSp561e*RSe461p - 1./162.*RN(mp2,me2)*m13*m44*RSp461e*
         RSe563p - 1./162.*RN(mp2,me2)*m13*m44*RSp461e*RSe463p + 1./162.
         *RN(mp2,me2)*m13*m44*RSp461e*RSe561p + 1./162.*RN(mp2,me2)*m13
         *m44*RSp461e*RSe461p + 1./162.*RN(mp2,me2)*m11*me2*RSp563e*
         RSe563p - 1./162.*RN(mp2,me2)*m11*me2*RSp563e*RSe463p - 1./162.
         *RN(mp2,me2)*m11*me2*RSp463e*RSe563p + 1./162.*RN(mp2,me2)*m11
         *me2*RSp463e*RSe463p + 1./162.*RN(mp2,me2)*m11*me2*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m11*me2*RSe561p*RSp463e - 1./162.
         *RN(mp2,me2)*m11*me2*RSe461p*RSp563e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./162.*RN(mp2,me2)*
         m11*me2*RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m11*me2*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m11*me2*RSp561e*RSe463p + 1./54.
         *RN(mp2,me2)*m11*me2*RSp561e*RSe561p - 1./54.*RN(mp2,me2)*m11*
         me2*RSp561e*RSe461p - 1./162.*RN(mp2,me2)*m11*me2*RSp461e*
         RSe563p + 1./162.*RN(mp2,me2)*m11*me2*RSp461e*RSe463p - 1./54.
         *RN(mp2,me2)*m11*me2*RSp461e*RSe561p + 1./54.*RN(mp2,me2)*m11*
         me2*RSp461e*RSe461p - 1./324.*RN(mp2,me2)*m11*m55*RSp563e*
         RSe563p - 1./324.*RN(mp2,me2)*m11*m55*RSp563e*RSe463p - 1./324.
         *RN(mp2,me2)*m11*m55*RSp463e*RSe563p - 1./108.*RN(mp2,me2)*m11
         *m55*RSp463e*RSe463p - 1./324.*RN(mp2,me2)*m11*m55*RSe561p*
         RSp563e - 1./324.*RN(mp2,me2)*m11*m55*RSe561p*RSp463e - 1./324.
         *RN(mp2,me2)*m11*m55*RSe461p*RSp563e - 1./108.*RN(mp2,me2)*m11
         *m55*RSe461p*RSp463e - 1./324.*RN(mp2,me2)*m11*m55*RSp561e*
         RSe563p - 1./324.*RN(mp2,me2)*m11*m55*RSp561e*RSe463p - 1./108.
         *RN(mp2,me2)*m11*m55*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./108.*RN(mp2,me2
         )*m11*m55*RSp561e*RSe461p - 1./324.*RN(mp2,me2)*m11*m55*
         RSp461e*RSe563p - 1./108.*RN(mp2,me2)*m11*m55*RSp461e*RSe463p
          - 1./108.*RN(mp2,me2)*m11*m55*RSp461e*RSe561p - 1./36.*RN(mp2
         ,me2)*m11*m55*RSp461e*RSe461p + 1./162.*RN(mp2,me2)*m11*m45*
         RSp563e*RSe563p - 1./162.*RN(mp2,me2)*m11*m45*RSp563e*RSe463p
          - 1./162.*RN(mp2,me2)*m11*m45*RSp463e*RSe563p + 1./162.*RN(
         mp2,me2)*m11*m45*RSp463e*RSe463p + 1./162.*RN(mp2,me2)*m11*m45
         *RSe561p*RSp563e - 1./162.*RN(mp2,me2)*m11*m45*RSe561p*RSp463e
          - 1./162.*RN(mp2,me2)*m11*m45*RSe461p*RSp563e + 1./162.*RN(
         mp2,me2)*m11*m45*RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m11*m45
         *RSp561e*RSe563p - 1./162.*RN(mp2,me2)*m11*m45*RSp561e*RSe463p
          + 1./54.*RN(mp2,me2)*m11*m45*RSp561e*RSe561p - 1./54.*RN(mp2,
         me2)*m11*m45*RSp561e*RSe461p - 1./162.*RN(mp2,me2)*m11*m45*
         RSp461e*RSe563p + 1./162.*RN(mp2,me2)*m11*m45*RSp461e*RSe463p
          - 1./54.*RN(mp2,me2)*m11*m45*RSp461e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./54.*RN(mp2,me2)*
         m11*m45*RSp461e*RSe461p - 1./108.*RN(mp2,me2)*m11*m44*RSp563e*
         RSe563p - 1./324.*RN(mp2,me2)*m11*m44*RSp563e*RSe463p - 1./324.
         *RN(mp2,me2)*m11*m44*RSp463e*RSe563p - 1./324.*RN(mp2,me2)*m11
         *m44*RSp463e*RSe463p - 1./108.*RN(mp2,me2)*m11*m44*RSe561p*
         RSp563e - 1./324.*RN(mp2,me2)*m11*m44*RSe561p*RSp463e - 1./324.
         *RN(mp2,me2)*m11*m44*RSe461p*RSp563e - 1./324.*RN(mp2,me2)*m11
         *m44*RSe461p*RSp463e - 1./108.*RN(mp2,me2)*m11*m44*RSp561e*
         RSe563p - 1./324.*RN(mp2,me2)*m11*m44*RSp561e*RSe463p - 1./36.
         *RN(mp2,me2)*m11*m44*RSp561e*RSe561p - 1./108.*RN(mp2,me2)*m11
         *m44*RSp561e*RSe461p - 1./324.*RN(mp2,me2)*m11*m44*RSp461e*
         RSe563p - 1./324.*RN(mp2,me2)*m11*m44*RSp461e*RSe463p - 1./108.
         *RN(mp2,me2)*m11*m44*RSp461e*RSe561p - 1./108.*RN(mp2,me2)*m11
         *m44*RSp461e*RSe461p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./81.*me2*RSe563p
         *RS345pe + 1./81.*me2*RSe563p*RS356pe + 1./81.*me2*pow(
         RSe563p,2) + 1./81.*me2*RSe453p*RS345pe - 1./81.*me2*RSe453p*
         RS356pe - 2./81.*me2*RSe453p*RSe563p + 1./81.*me2*pow(
         RSe453p,2) + 1./81.*me2*RSp563e*RSe563p - 1./81.*me2*RSp563e*
         RSe453p - 1./81.*me2*RSp453e*RSe563p + 1./81.*me2*RSp453e*
         RSe453p + 1./81.*me2*RS145pe*RSe563p - 1./81.*me2*RS145pe*
         RSe453p - 1./81.*me2*RS156pe*RSe563p + 1./81.*me2*RS156pe*
         RSe453p + 1./81.*me2*RSe561p*RS345pe - 1./81.*me2*RSe561p*
         RS356pe - 2./81.*me2*RSe561p*RSe563p + 2./81.*me2*RSe561p*
         RSe453p - 1./81.*me2*RSe561p*RSp563e + 1./81.*me2*RSe561p*
         RSp453e - 1./81.*me2*RSe561p*RS145pe + 1./81.*me2*RSe561p*
         RS156pe + 1./81.*me2*pow(RSe561p,2) - 1./81.*me2*RSe451p*
         RS345pe + 1./81.*me2*RSe451p*RS356pe + 2./81.*me2*RSe451p*
         RSe563p - 2./81.*me2*RSe451p*RSe453p + 1./81.*me2*RSe451p*
         RSp563e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./81.*me2*RSe451p
         *RSp453e + 1./81.*me2*RSe451p*RS145pe - 1./81.*me2*RSe451p*
         RS156pe - 2./81.*me2*RSe451p*RSe561p + 1./81.*me2*pow(
         RSe451p,2) - 1./81.*me2*RSp561e*RSe563p + 1./81.*me2*RSp561e*
         RSe453p + 1./81.*me2*RSp561e*RSe561p - 1./81.*me2*RSp561e*
         RSe451p + 1./81.*me2*RSp451e*RSe563p - 1./81.*me2*RSp451e*
         RSe453p - 1./81.*me2*RSp451e*RSe561p + 1./81.*me2*RSp451e*
         RSe451p - 1./162.*m66*RSe563p*RS345pe - 1./162.*m66*RSe563p*
         RS356pe - 1./324.*m66*pow(RSe563p,2) - 1./54.*m66*RSe453p*
         RS345pe - 1./162.*m66*RSe453p*RS356pe - 1./162.*m66*RSe453p*
         RSe563p - 1./108.*m66*pow(RSe453p,2) + 1./162.*m66*RS145pe*
         RSe563p + 1./54.*m66*RS145pe*RSe453p + 1./162.*m66*RS156pe*
         RSe563p + 1./162.*m66*RS156pe*RSe453p + 1./162.*m66*RSe561p*
         RS345pe + 1./162.*m66*RSe561p*RS356pe + 1./162.*m66*RSe561p*
         RSe563p + 1./162.*m66*RSe561p*RSe453p - 1./162.*m66*RSe561p*
         RS145pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./162.*m66*
         RSe561p*RS156pe - 1./324.*m66*pow(RSe561p,2) + 1./54.*m66*
         RSe451p*RS345pe + 1./162.*m66*RSe451p*RS356pe + 1./162.*m66*
         RSe451p*RSe563p + 1./54.*m66*RSe451p*RSe453p - 1./54.*m66*
         RSe451p*RS145pe - 1./162.*m66*RSe451p*RS156pe - 1./162.*m66*
         RSe451p*RSe561p - 1./108.*m66*pow(RSe451p,2) - 1./81.*m46*
         RSe563p*RS345pe + 1./81.*m46*RSe563p*RS356pe + 1./162.*m46*
         pow(RSe563p,2) + 1./81.*m46*RSe453p*RS345pe - 1./81.*m46*
         RSe453p*RS356pe - 1./81.*m46*RSe453p*RSe563p + 1./162.*m46*
         pow(RSe453p,2) + 1./81.*m46*RS145pe*RSe563p - 1./81.*m46*
         RS145pe*RSe453p - 1./81.*m46*RS156pe*RSe563p + 1./81.*m46*
         RS156pe*RSe453p + 1./81.*m46*RSe561p*RS345pe - 1./81.*m46*
         RSe561p*RS356pe - 1./81.*m46*RSe561p*RSe563p + 1./81.*m46*
         RSe561p*RSe453p - 1./81.*m46*RSe561p*RS145pe + 1./81.*m46*
         RSe561p*RS156pe + 1./162.*m46*pow(RSe561p,2) - 1./81.*m46*
         RSe451p*RS345pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./81.*m46*RSe451p*
         RS356pe + 1./81.*m46*RSe451p*RSe563p - 1./81.*m46*RSe451p*
         RSe453p + 1./81.*m46*RSe451p*RS145pe - 1./81.*m46*RSe451p*
         RS156pe - 1./81.*m46*RSe451p*RSe561p + 1./162.*m46*pow(
         RSe451p,2) - 1./162.*m44*RSe563p*RS345pe - 1./54.*m44*RSe563p*
         RS356pe - 1./108.*m44*pow(RSe563p,2) - 1./162.*m44*RSe453p*
         RS345pe - 1./162.*m44*RSe453p*RS356pe - 1./162.*m44*RSe453p*
         RSe563p - 1./324.*m44*pow(RSe453p,2) + 1./162.*m44*RS145pe*
         RSe563p + 1./162.*m44*RS145pe*RSe453p + 1./54.*m44*RS156pe*
         RSe563p + 1./162.*m44*RS156pe*RSe453p + 1./162.*m44*RSe561p*
         RS345pe + 1./54.*m44*RSe561p*RS356pe + 1./54.*m44*RSe561p*
         RSe563p + 1./162.*m44*RSe561p*RSe453p - 1./162.*m44*RSe561p*
         RS145pe - 1./54.*m44*RSe561p*RS156pe - 1./108.*m44*pow(
         RSe561p,2) + 1./162.*m44*RSe451p*RS345pe + 1./162.*m44*RSe451p
         *RS356pe + 1./162.*m44*RSe451p*RSe563p + 1./162.*m44*RSe451p*
         RSe453p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./162.*m44*
         RSe451p*RS145pe - 1./162.*m44*RSe451p*RS156pe - 1./162.*m44*
         RSe451p*RSe561p - 1./324.*m44*pow(RSe451p,2) + 1./54.*m33*
         RSe563p*RS345pe - 1./54.*m33*RSe563p*RS356pe - 1./108.*m33*
         pow(RSe563p,2) - 1./54.*m33*RSe453p*RS345pe + 1./54.*m33*
         RSe453p*RS356pe + 1./54.*m33*RSe453p*RSe563p - 1./108.*m33*
         pow(RSe453p,2) + 1./162.*m33*RS145pe*RSe563p - 1./162.*m33*
         RS145pe*RSe453p - 1./162.*m33*RS156pe*RSe563p + 1./162.*m33*
         RS156pe*RSe453p + 1./162.*m33*RSe561p*RS345pe - 1./162.*m33*
         RSe561p*RS356pe - 1./162.*m33*RSe561p*RSe563p + 1./162.*m33*
         RSe561p*RSe453p + 1./162.*m33*RSe561p*RS145pe - 1./162.*m33*
         RSe561p*RS156pe - 1./324.*m33*pow(RSe561p,2) - 1./162.*m33*
         RSe451p*RS345pe + 1./162.*m33*RSe451p*RS356pe + 1./162.*m33*
         RSe451p*RSe563p - 1./162.*m33*RSe451p*RSe453p - 1./162.*m33*
         RSe451p*RS145pe + 1./162.*m33*RSe451p*RS156pe + 1./162.*m33*
         RSe451p*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./324.*m33*pow(
         RSe451p,2) + 1./81.*m33*me2*RN3e*RSe563p*RS345pe - 1./81.*m33*
         me2*RN3e*RSe563p*RS356pe - 1./81.*m33*me2*RN3e*RSe453p*RS345pe
          + 1./81.*m33*me2*RN3e*RSe453p*RS356pe - 1./81.*m33*me2*RN3e*
         RSe561p*RS345pe + 1./81.*m33*me2*RN3e*RSe561p*RS356pe + 1./81.
         *m33*me2*RN3e*RSe451p*RS345pe - 1./81.*m33*me2*RN3e*RSe451p*
         RS356pe - 1./324.*m33*m66*RN3e*RSe563p*RS345pe - 1./324.*m33*
         m66*RN3e*RSe563p*RS356pe - 1./108.*m33*m66*RN3e*RSe453p*
         RS345pe - 1./324.*m33*m66*RN3e*RSe453p*RS356pe - 1./108.*m33*
         m66*RN3e*RSe561p*RS345pe - 1./108.*m33*m66*RN3e*RSe561p*
         RS356pe - 1./36.*m33*m66*RN3e*RSe451p*RS345pe - 1./108.*m33*
         m66*RN3e*RSe451p*RS356pe - 1./324.*m33*m66*RN1e*RS145pe*
         RSe563p - 1./108.*m33*m66*RN1e*RS145pe*RSe453p - 1./324.*m33*
         m66*RN1e*RS156pe*RSe563p - 1./324.*m33*m66*RN1e*RS156pe*
         RSe453p - 1./324.*m33*m66*RN1e*RSe561p*RS145pe - 1./324.*m33*
         m66*RN1e*RSe561p*RS156pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./108.*m33*m66*
         RN1e*RSe451p*RS145pe - 1./324.*m33*m66*RN1e*RSe451p*RS156pe - 
         1./162.*m33*m46*RN3e*RSe563p*RS345pe + 1./162.*m33*m46*RN3e*
         RSe563p*RS356pe + 1./162.*m33*m46*RN3e*RSe453p*RS345pe - 1./
         162.*m33*m46*RN3e*RSe453p*RS356pe - 1./54.*m33*m46*RN3e*
         RSe561p*RS345pe + 1./54.*m33*m46*RN3e*RSe561p*RS356pe + 1./54.
         *m33*m46*RN3e*RSe451p*RS345pe - 1./54.*m33*m46*RN3e*RSe451p*
         RS356pe - 1./162.*m33*m46*RN1e*RS145pe*RSe563p + 1./162.*m33*
         m46*RN1e*RS145pe*RSe453p + 1./162.*m33*m46*RN1e*RS156pe*
         RSe563p - 1./162.*m33*m46*RN1e*RS156pe*RSe453p - 1./162.*m33*
         m46*RN1e*RSe561p*RS145pe + 1./162.*m33*m46*RN1e*RSe561p*
         RS156pe + 1./162.*m33*m46*RN1e*RSe451p*RS145pe - 1./162.*m33*
         m46*RN1e*RSe451p*RS156pe - 1./324.*m33*m44*RN3e*RSe563p*
         RS345pe - 1./108.*m33*m44*RN3e*RSe563p*RS356pe - 1./324.*m33*
         m44*RN3e*RSe453p*RS345pe - 1./324.*m33*m44*RN3e*RSe453p*
         RS356pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./108.*m33*m44*
         RN3e*RSe561p*RS345pe - 1./36.*m33*m44*RN3e*RSe561p*RS356pe - 1.
         /108.*m33*m44*RN3e*RSe451p*RS345pe - 1./108.*m33*m44*RN3e*
         RSe451p*RS356pe - 1./324.*m33*m44*RN1e*RS145pe*RSe563p - 1./
         324.*m33*m44*RN1e*RS145pe*RSe453p - 1./108.*m33*m44*RN1e*
         RS156pe*RSe563p - 1./324.*m33*m44*RN1e*RS156pe*RSe453p - 1./
         324.*m33*m44*RN1e*RSe561p*RS145pe - 1./108.*m33*m44*RN1e*
         RSe561p*RS156pe - 1./324.*m33*m44*RN1e*RSe451p*RS145pe - 1./
         324.*m33*m44*RN1e*RSe451p*RS156pe - 1./54.*pow(m33,2)*RN3e*
         RSe563p*RS345pe + 1./54.*pow(m33,2)*RN3e*RSe563p*RS356pe + 1./
         54.*pow(m33,2)*RN3e*RSe453p*RS345pe - 1./54.*pow(m33,2)*RN3e*
         RSe453p*RS356pe - 1./162.*pow(m33,2)*RN3e*RSe561p*RS345pe + 1./
         162.*pow(m33,2)*RN3e*RSe561p*RS356pe + 1./162.*pow(m33,2)*RN3e
         *RSe451p*RS345pe - 1./162.*pow(m33,2)*RN3e*RSe451p*RS356pe - 1.
         /81.*m13*RSe563p*RS345pe + 1./81.*m13*RSe563p*RS356pe + 1./162.
         *m13*pow(RSe563p,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./81.*m13*RSe453p*
         RS345pe - 1./81.*m13*RSe453p*RS356pe - 1./81.*m13*RSe453p*
         RSe563p + 1./162.*m13*pow(RSe453p,2) + 1./81.*m13*RS145pe*
         RSe563p - 1./81.*m13*RS145pe*RSe453p - 1./81.*m13*RS156pe*
         RSe563p + 1./81.*m13*RS156pe*RSe453p + 1./81.*m13*RSe561p*
         RS345pe - 1./81.*m13*RSe561p*RS356pe - 1./81.*m13*RSe561p*
         RSe563p + 1./81.*m13*RSe561p*RSe453p - 1./81.*m13*RSe561p*
         RS145pe + 1./81.*m13*RSe561p*RS156pe + 1./162.*m13*pow(
         RSe561p,2) - 1./81.*m13*RSe451p*RS345pe + 1./81.*m13*RSe451p*
         RS356pe + 1./81.*m13*RSe451p*RSe563p - 1./81.*m13*RSe451p*
         RSe453p + 1./81.*m13*RSe451p*RS145pe - 1./81.*m13*RSe451p*
         RS156pe - 1./81.*m13*RSe451p*RSe561p + 1./162.*m13*pow(
         RSe451p,2) + 1./162.*m13*m66*RN3e*RSe563p*RS345pe + 1./162.*
         m13*m66*RN3e*RSe563p*RS356pe + 1./54.*m13*m66*RN3e*RSe453p*
         RS345pe + 1./162.*m13*m66*RN3e*RSe453p*RS356pe - 1./162.*m13*
         m66*RN3e*RSe561p*RS345pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./162.*m13*m66*
         RN3e*RSe561p*RS356pe - 1./54.*m13*m66*RN3e*RSe451p*RS345pe - 1.
         /162.*m13*m66*RN3e*RSe451p*RS356pe - 1./162.*m13*m66*RN1e*
         RS145pe*RSe563p - 1./54.*m13*m66*RN1e*RS145pe*RSe453p - 1./162.
         *m13*m66*RN1e*RS156pe*RSe563p - 1./162.*m13*m66*RN1e*RS156pe*
         RSe453p + 1./162.*m13*m66*RN1e*RSe561p*RS145pe + 1./162.*m13*
         m66*RN1e*RSe561p*RS156pe + 1./54.*m13*m66*RN1e*RSe451p*RS145pe
          + 1./162.*m13*m66*RN1e*RSe451p*RS156pe + 1./81.*m13*m46*RN3e*
         RSe563p*RS345pe - 1./81.*m13*m46*RN3e*RSe563p*RS356pe - 1./81.
         *m13*m46*RN3e*RSe453p*RS345pe + 1./81.*m13*m46*RN3e*RSe453p*
         RS356pe - 1./81.*m13*m46*RN3e*RSe561p*RS345pe + 1./81.*m13*m46
         *RN3e*RSe561p*RS356pe + 1./81.*m13*m46*RN3e*RSe451p*RS345pe - 
         1./81.*m13*m46*RN3e*RSe451p*RS356pe - 1./81.*m13*m46*RN1e*
         RS145pe*RSe563p + 1./81.*m13*m46*RN1e*RS145pe*RSe453p + 1./81.
         *m13*m46*RN1e*RS156pe*RSe563p - 1./81.*m13*m46*RN1e*RS156pe*
         RSe453p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./81.*m13*m46*RN1e*
         RSe561p*RS145pe - 1./81.*m13*m46*RN1e*RSe561p*RS156pe - 1./81.
         *m13*m46*RN1e*RSe451p*RS145pe + 1./81.*m13*m46*RN1e*RSe451p*
         RS156pe + 1./162.*m13*m44*RN3e*RSe563p*RS345pe + 1./54.*m13*
         m44*RN3e*RSe563p*RS356pe + 1./162.*m13*m44*RN3e*RSe453p*
         RS345pe + 1./162.*m13*m44*RN3e*RSe453p*RS356pe - 1./162.*m13*
         m44*RN3e*RSe561p*RS345pe - 1./54.*m13*m44*RN3e*RSe561p*RS356pe
          - 1./162.*m13*m44*RN3e*RSe451p*RS345pe - 1./162.*m13*m44*RN3e
         *RSe451p*RS356pe - 1./162.*m13*m44*RN1e*RS145pe*RSe563p - 1./
         162.*m13*m44*RN1e*RS145pe*RSe453p - 1./54.*m13*m44*RN1e*
         RS156pe*RSe563p - 1./162.*m13*m44*RN1e*RS156pe*RSe453p + 1./
         162.*m13*m44*RN1e*RSe561p*RS145pe + 1./54.*m13*m44*RN1e*
         RSe561p*RS156pe + 1./162.*m13*m44*RN1e*RSe451p*RS145pe + 1./
         162.*m13*m44*RN1e*RSe451p*RS156pe + 1./81.*m13*m33*RN3e*
         RSe563p*RS345pe - 1./81.*m13*m33*RN3e*RSe563p*RS356pe - 1./81.
         *m13*m33*RN3e*RSe453p*RS345pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./81.*m13*m33*RN3e*
         RSe453p*RS356pe - 1./81.*m13*m33*RN3e*RSe561p*RS345pe + 1./81.
         *m13*m33*RN3e*RSe561p*RS356pe + 1./81.*m13*m33*RN3e*RSe451p*
         RS345pe - 1./81.*m13*m33*RN3e*RSe451p*RS356pe + 1./162.*m11*
         RSe563p*RS345pe - 1./162.*m11*RSe563p*RS356pe - 1./324.*m11*
         pow(RSe563p,2) - 1./162.*m11*RSe453p*RS345pe + 1./162.*m11*
         RSe453p*RS356pe + 1./162.*m11*RSe453p*RSe563p - 1./324.*m11*
         pow(RSe453p,2) + 1./162.*m11*RS145pe*RSe563p - 1./162.*m11*
         RS145pe*RSe453p - 1./162.*m11*RS156pe*RSe563p + 1./162.*m11*
         RS156pe*RSe453p + 1./162.*m11*RSe561p*RS345pe - 1./162.*m11*
         RSe561p*RS356pe - 1./162.*m11*RSe561p*RSe563p + 1./162.*m11*
         RSe561p*RSe453p + 1./54.*m11*RSe561p*RS145pe - 1./54.*m11*
         RSe561p*RS156pe - 1./108.*m11*pow(RSe561p,2) - 1./162.*m11*
         RSe451p*RS345pe + 1./162.*m11*RSe451p*RS356pe + 1./162.*m11*
         RSe451p*RSe563p - 1./162.*m11*RSe451p*RSe453p - 1./54.*m11*
         RSe451p*RS145pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./54.*m11*RSe451p*
         RS156pe + 1./54.*m11*RSe451p*RSe561p - 1./108.*m11*pow(
         RSe451p,2) - 1./81.*m11*me2*RN1e*RS145pe*RSe563p + 1./81.*m11*
         me2*RN1e*RS145pe*RSe453p + 1./81.*m11*me2*RN1e*RS156pe*RSe563p
          - 1./81.*m11*me2*RN1e*RS156pe*RSe453p + 1./81.*m11*me2*RN1e*
         RSe561p*RS145pe - 1./81.*m11*me2*RN1e*RSe561p*RS156pe - 1./81.
         *m11*me2*RN1e*RSe451p*RS145pe + 1./81.*m11*me2*RN1e*RSe451p*
         RS156pe - 1./324.*m11*m66*RN3e*RSe563p*RS345pe - 1./324.*m11*
         m66*RN3e*RSe563p*RS356pe - 1./108.*m11*m66*RN3e*RSe453p*
         RS345pe - 1./324.*m11*m66*RN3e*RSe453p*RS356pe - 1./324.*m11*
         m66*RN3e*RSe561p*RS345pe - 1./324.*m11*m66*RN3e*RSe561p*
         RS356pe - 1./108.*m11*m66*RN3e*RSe451p*RS345pe - 1./324.*m11*
         m66*RN3e*RSe451p*RS356pe - 1./108.*m11*m66*RN1e*RS145pe*
         RSe563p - 1./36.*m11*m66*RN1e*RS145pe*RSe453p - 1./108.*m11*
         m66*RN1e*RS156pe*RSe563p - 1./108.*m11*m66*RN1e*RS156pe*
         RSe453p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./324.*m11*m66*
         RN1e*RSe561p*RS145pe - 1./324.*m11*m66*RN1e*RSe561p*RS156pe - 
         1./108.*m11*m66*RN1e*RSe451p*RS145pe - 1./324.*m11*m66*RN1e*
         RSe451p*RS156pe - 1./162.*m11*m46*RN3e*RSe563p*RS345pe + 1./
         162.*m11*m46*RN3e*RSe563p*RS356pe + 1./162.*m11*m46*RN3e*
         RSe453p*RS345pe - 1./162.*m11*m46*RN3e*RSe453p*RS356pe - 1./
         162.*m11*m46*RN3e*RSe561p*RS345pe + 1./162.*m11*m46*RN3e*
         RSe561p*RS356pe + 1./162.*m11*m46*RN3e*RSe451p*RS345pe - 1./
         162.*m11*m46*RN3e*RSe451p*RS356pe - 1./54.*m11*m46*RN1e*
         RS145pe*RSe563p + 1./54.*m11*m46*RN1e*RS145pe*RSe453p + 1./54.
         *m11*m46*RN1e*RS156pe*RSe563p - 1./54.*m11*m46*RN1e*RS156pe*
         RSe453p - 1./162.*m11*m46*RN1e*RSe561p*RS145pe + 1./162.*m11*
         m46*RN1e*RSe561p*RS156pe + 1./162.*m11*m46*RN1e*RSe451p*
         RS145pe - 1./162.*m11*m46*RN1e*RSe451p*RS156pe - 1./324.*m11*
         m44*RN3e*RSe563p*RS345pe - 1./108.*m11*m44*RN3e*RSe563p*
         RS356pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./324.*m11*m44*
         RN3e*RSe453p*RS345pe - 1./324.*m11*m44*RN3e*RSe453p*RS356pe - 
         1./324.*m11*m44*RN3e*RSe561p*RS345pe - 1./108.*m11*m44*RN3e*
         RSe561p*RS356pe - 1./324.*m11*m44*RN3e*RSe451p*RS345pe - 1./
         324.*m11*m44*RN3e*RSe451p*RS356pe - 1./108.*m11*m44*RN1e*
         RS145pe*RSe563p - 1./108.*m11*m44*RN1e*RS145pe*RSe453p - 1./36.
         *m11*m44*RN1e*RS156pe*RSe563p - 1./108.*m11*m44*RN1e*RS156pe*
         RSe453p - 1./324.*m11*m44*RN1e*RSe561p*RS145pe - 1./108.*m11*
         m44*RN1e*RSe561p*RS156pe - 1./324.*m11*m44*RN1e*RSe451p*
         RS145pe - 1./324.*m11*m44*RN1e*RSe451p*RS156pe - 1./162.*m11*
         m33*RN3e*RSe563p*RS345pe + 1./162.*m11*m33*RN3e*RSe563p*
         RS356pe + 1./162.*m11*m33*RN3e*RSe453p*RS345pe - 1./162.*m11*
         m33*RN3e*RSe453p*RS356pe - 1./162.*m11*m33*RN3e*RSe561p*
         RS345pe + 1./162.*m11*m33*RN3e*RSe561p*RS356pe + 1./162.*m11*
         m33*RN3e*RSe451p*RS345pe - 1./162.*m11*m33*RN3e*RSe451p*
         RS356pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./162.*m11*m33*
         RN1e*RS145pe*RSe563p + 1./162.*m11*m33*RN1e*RS145pe*RSe453p + 
         1./162.*m11*m33*RN1e*RS156pe*RSe563p - 1./162.*m11*m33*RN1e*
         RS156pe*RSe453p - 1./162.*m11*m33*RN1e*RSe561p*RS145pe + 1./
         162.*m11*m33*RN1e*RSe561p*RS156pe + 1./162.*m11*m33*RN1e*
         RSe451p*RS145pe - 1./162.*m11*m33*RN1e*RSe451p*RS156pe - 1./81.
         *m11*m13*RN1e*RS145pe*RSe563p + 1./81.*m11*m13*RN1e*RS145pe*
         RSe453p + 1./81.*m11*m13*RN1e*RS156pe*RSe563p - 1./81.*m11*m13
         *RN1e*RS156pe*RSe453p + 1./81.*m11*m13*RN1e*RSe561p*RS145pe - 
         1./81.*m11*m13*RN1e*RSe561p*RS156pe - 1./81.*m11*m13*RN1e*
         RSe451p*RS145pe + 1./81.*m11*m13*RN1e*RSe451p*RS156pe - 1./162.
         *pow(m11,2)*RN1e*RS145pe*RSe563p + 1./162.*pow(m11,2)*RN1e*
         RS145pe*RSe453p + 1./162.*pow(m11,2)*RN1e*RS156pe*RSe563p - 1./
         162.*pow(m11,2)*RN1e*RS156pe*RSe453p - 1./54.*pow(m11,2)*RN1e*
         RSe561p*RS145pe + 1./54.*pow(m11,2)*RN1e*RSe561p*RS156pe + 1./
         54.*pow(m11,2)*RN1e*RSe451p*RS145pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./54.*pow(m11,2)*
         RN1e*RSe451p*RS156pe - 1./81.*RN(mp2,me2)*mp2*me2*RSp563e*
         RSe563p + 1./81.*RN(mp2,me2)*mp2*me2*RSp563e*RSe453p + 1./81.*
         RN(mp2,me2)*mp2*me2*RSp453e*RSe563p - 1./81.*RN(mp2,me2)*mp2*
         me2*RSp453e*RSe453p + 1./81.*RN(mp2,me2)*mp2*me2*RSe561p*
         RSp563e - 1./81.*RN(mp2,me2)*mp2*me2*RSe561p*RSp453e - 1./81.*
         RN(mp2,me2)*mp2*me2*RSe451p*RSp563e + 1./81.*RN(mp2,me2)*mp2*
         me2*RSe451p*RSp453e + 1./81.*RN(mp2,me2)*mp2*me2*RSp561e*
         RSe563p - 1./81.*RN(mp2,me2)*mp2*me2*RSp561e*RSe453p - 1./81.*
         RN(mp2,me2)*mp2*me2*RSp561e*RSe561p + 1./81.*RN(mp2,me2)*mp2*
         me2*RSp561e*RSe451p - 1./81.*RN(mp2,me2)*mp2*me2*RSp451e*
         RSe563p + 1./81.*RN(mp2,me2)*mp2*me2*RSp451e*RSe453p + 1./81.*
         RN(mp2,me2)*mp2*me2*RSp451e*RSe561p - 1./81.*RN(mp2,me2)*mp2*
         me2*RSp451e*RSe451p + 1./162.*RN(mp2,me2)*m66*me2*RSp563e*
         RSe563p + 1./162.*RN(mp2,me2)*m66*me2*RSp563e*RSe453p + 1./162.
         *RN(mp2,me2)*m66*me2*RSp453e*RSe563p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./54.*RN(mp2,me2)*
         m66*me2*RSp453e*RSe453p - 1./162.*RN(mp2,me2)*m66*me2*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m66*me2*RSe561p*RSp453e - 1./162.
         *RN(mp2,me2)*m66*me2*RSe451p*RSp563e - 1./54.*RN(mp2,me2)*m66*
         me2*RSe451p*RSp453e - 1./162.*RN(mp2,me2)*m66*me2*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m66*me2*RSp561e*RSe453p + 1./162.
         *RN(mp2,me2)*m66*me2*RSp561e*RSe561p + 1./162.*RN(mp2,me2)*m66
         *me2*RSp561e*RSe451p - 1./162.*RN(mp2,me2)*m66*me2*RSp451e*
         RSe563p - 1./54.*RN(mp2,me2)*m66*me2*RSp451e*RSe453p + 1./162.
         *RN(mp2,me2)*m66*me2*RSp451e*RSe561p + 1./54.*RN(mp2,me2)*m66*
         me2*RSp451e*RSe451p - 1./81.*RN(mp2,me2)*m46*me2*RSp563e*
         RSe563p + 1./81.*RN(mp2,me2)*m46*me2*RSp563e*RSe453p + 1./81.*
         RN(mp2,me2)*m46*me2*RSp453e*RSe563p - 1./81.*RN(mp2,me2)*m46*
         me2*RSp453e*RSe453p + 1./81.*RN(mp2,me2)*m46*me2*RSe561p*
         RSp563e - 1./81.*RN(mp2,me2)*m46*me2*RSe561p*RSp453e - 1./81.*
         RN(mp2,me2)*m46*me2*RSe451p*RSp563e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./81.*RN(mp2,me2)*
         m46*me2*RSe451p*RSp453e + 1./81.*RN(mp2,me2)*m46*me2*RSp561e*
         RSe563p - 1./81.*RN(mp2,me2)*m46*me2*RSp561e*RSe453p - 1./81.*
         RN(mp2,me2)*m46*me2*RSp561e*RSe561p + 1./81.*RN(mp2,me2)*m46*
         me2*RSp561e*RSe451p - 1./81.*RN(mp2,me2)*m46*me2*RSp451e*
         RSe563p + 1./81.*RN(mp2,me2)*m46*me2*RSp451e*RSe453p + 1./81.*
         RN(mp2,me2)*m46*me2*RSp451e*RSe561p - 1./81.*RN(mp2,me2)*m46*
         me2*RSp451e*RSe451p + 1./54.*RN(mp2,me2)*m44*me2*RSp563e*
         RSe563p + 1./162.*RN(mp2,me2)*m44*me2*RSp563e*RSe453p + 1./162.
         *RN(mp2,me2)*m44*me2*RSp453e*RSe563p + 1./162.*RN(mp2,me2)*m44
         *me2*RSp453e*RSe453p - 1./54.*RN(mp2,me2)*m44*me2*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m44*me2*RSe561p*RSp453e - 1./162.
         *RN(mp2,me2)*m44*me2*RSe451p*RSp563e - 1./162.*RN(mp2,me2)*m44
         *me2*RSe451p*RSp453e - 1./54.*RN(mp2,me2)*m44*me2*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m44*me2*RSp561e*RSe453p + 1./54.
         *RN(mp2,me2)*m44*me2*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./162.*RN(mp2,me2)*
         m44*me2*RSp561e*RSe451p - 1./162.*RN(mp2,me2)*m44*me2*RSp451e*
         RSe563p - 1./162.*RN(mp2,me2)*m44*me2*RSp451e*RSe453p + 1./162.
         *RN(mp2,me2)*m44*me2*RSp451e*RSe561p + 1./162.*RN(mp2,me2)*m44
         *me2*RSp451e*RSe451p + 1./54.*RN(mp2,me2)*m33*me2*RSp563e*
         RSe563p - 1./54.*RN(mp2,me2)*m33*me2*RSp563e*RSe453p - 1./54.*
         RN(mp2,me2)*m33*me2*RSp453e*RSe563p + 1./54.*RN(mp2,me2)*m33*
         me2*RSp453e*RSe453p + 1./162.*RN(mp2,me2)*m33*me2*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m33*me2*RSe561p*RSp453e - 1./162.
         *RN(mp2,me2)*m33*me2*RSe451p*RSp563e + 1./162.*RN(mp2,me2)*m33
         *me2*RSe451p*RSp453e + 1./162.*RN(mp2,me2)*m33*me2*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m33*me2*RSp561e*RSe453p + 1./162.
         *RN(mp2,me2)*m33*me2*RSp561e*RSe561p - 1./162.*RN(mp2,me2)*m33
         *me2*RSp561e*RSe451p - 1./162.*RN(mp2,me2)*m33*me2*RSp451e*
         RSe563p + 1./162.*RN(mp2,me2)*m33*me2*RSp451e*RSe453p - 1./162.
         *RN(mp2,me2)*m33*me2*RSp451e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./162.*RN(mp2,me2)*
         m33*me2*RSp451e*RSe451p - 1./108.*RN(mp2,me2)*m33*m66*RSp563e*
         RSe563p - 1./108.*RN(mp2,me2)*m33*m66*RSp563e*RSe453p - 1./108.
         *RN(mp2,me2)*m33*m66*RSp453e*RSe563p - 1./36.*RN(mp2,me2)*m33*
         m66*RSp453e*RSe453p - 1./324.*RN(mp2,me2)*m33*m66*RSe561p*
         RSp563e - 1./324.*RN(mp2,me2)*m33*m66*RSe561p*RSp453e - 1./324.
         *RN(mp2,me2)*m33*m66*RSe451p*RSp563e - 1./108.*RN(mp2,me2)*m33
         *m66*RSe451p*RSp453e - 1./324.*RN(mp2,me2)*m33*m66*RSp561e*
         RSe563p - 1./324.*RN(mp2,me2)*m33*m66*RSp561e*RSe453p - 1./324.
         *RN(mp2,me2)*m33*m66*RSp561e*RSe561p - 1./324.*RN(mp2,me2)*m33
         *m66*RSp561e*RSe451p - 1./324.*RN(mp2,me2)*m33*m66*RSp451e*
         RSe563p - 1./108.*RN(mp2,me2)*m33*m66*RSp451e*RSe453p - 1./324.
         *RN(mp2,me2)*m33*m66*RSp451e*RSe561p - 1./108.*RN(mp2,me2)*m33
         *m66*RSp451e*RSe451p + 1./54.*RN(mp2,me2)*m33*m46*RSp563e*
         RSe563p - 1./54.*RN(mp2,me2)*m33*m46*RSp563e*RSe453p - 1./54.*
         RN(mp2,me2)*m33*m46*RSp453e*RSe563p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./54.*RN(mp2,me2)*
         m33*m46*RSp453e*RSe453p + 1./162.*RN(mp2,me2)*m33*m46*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m33*m46*RSe561p*RSp453e - 1./162.
         *RN(mp2,me2)*m33*m46*RSe451p*RSp563e + 1./162.*RN(mp2,me2)*m33
         *m46*RSe451p*RSp453e + 1./162.*RN(mp2,me2)*m33*m46*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m33*m46*RSp561e*RSe453p + 1./162.
         *RN(mp2,me2)*m33*m46*RSp561e*RSe561p - 1./162.*RN(mp2,me2)*m33
         *m46*RSp561e*RSe451p - 1./162.*RN(mp2,me2)*m33*m46*RSp451e*
         RSe563p + 1./162.*RN(mp2,me2)*m33*m46*RSp451e*RSe453p - 1./162.
         *RN(mp2,me2)*m33*m46*RSp451e*RSe561p + 1./162.*RN(mp2,me2)*m33
         *m46*RSp451e*RSe451p - 1./36.*RN(mp2,me2)*m33*m44*RSp563e*
         RSe563p - 1./108.*RN(mp2,me2)*m33*m44*RSp563e*RSe453p - 1./108.
         *RN(mp2,me2)*m33*m44*RSp453e*RSe563p - 1./108.*RN(mp2,me2)*m33
         *m44*RSp453e*RSe453p - 1./108.*RN(mp2,me2)*m33*m44*RSe561p*
         RSp563e - 1./324.*RN(mp2,me2)*m33*m44*RSe561p*RSp453e - 1./324.
         *RN(mp2,me2)*m33*m44*RSe451p*RSp563e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./324.*RN(mp2,me2
         )*m33*m44*RSe451p*RSp453e - 1./108.*RN(mp2,me2)*m33*m44*
         RSp561e*RSe563p - 1./324.*RN(mp2,me2)*m33*m44*RSp561e*RSe453p
          - 1./108.*RN(mp2,me2)*m33*m44*RSp561e*RSe561p - 1./324.*RN(
         mp2,me2)*m33*m44*RSp561e*RSe451p - 1./324.*RN(mp2,me2)*m33*m44
         *RSp451e*RSe563p - 1./324.*RN(mp2,me2)*m33*m44*RSp451e*RSe453p
          - 1./324.*RN(mp2,me2)*m33*m44*RSp451e*RSe561p - 1./324.*RN(
         mp2,me2)*m33*m44*RSp451e*RSe451p - 1./81.*RN(mp2,me2)*m13*me2*
         RSp563e*RSe563p + 1./81.*RN(mp2,me2)*m13*me2*RSp563e*RSe453p
          + 1./81.*RN(mp2,me2)*m13*me2*RSp453e*RSe563p - 1./81.*RN(mp2,
         me2)*m13*me2*RSp453e*RSe453p + 1./81.*RN(mp2,me2)*m13*me2*
         RSe561p*RSp563e - 1./81.*RN(mp2,me2)*m13*me2*RSe561p*RSp453e
          - 1./81.*RN(mp2,me2)*m13*me2*RSe451p*RSp563e + 1./81.*RN(mp2,
         me2)*m13*me2*RSe451p*RSp453e + 1./81.*RN(mp2,me2)*m13*me2*
         RSp561e*RSe563p - 1./81.*RN(mp2,me2)*m13*me2*RSp561e*RSe453p
          - 1./81.*RN(mp2,me2)*m13*me2*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./81.*RN(mp2,me2)*
         m13*me2*RSp561e*RSe451p - 1./81.*RN(mp2,me2)*m13*me2*RSp451e*
         RSe563p + 1./81.*RN(mp2,me2)*m13*me2*RSp451e*RSe453p + 1./81.*
         RN(mp2,me2)*m13*me2*RSp451e*RSe561p - 1./81.*RN(mp2,me2)*m13*
         me2*RSp451e*RSe451p + 1./162.*RN(mp2,me2)*m13*m66*RSp563e*
         RSe563p + 1./162.*RN(mp2,me2)*m13*m66*RSp563e*RSe453p + 1./162.
         *RN(mp2,me2)*m13*m66*RSp453e*RSe563p + 1./54.*RN(mp2,me2)*m13*
         m66*RSp453e*RSe453p - 1./162.*RN(mp2,me2)*m13*m66*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m13*m66*RSe561p*RSp453e - 1./162.
         *RN(mp2,me2)*m13*m66*RSe451p*RSp563e - 1./54.*RN(mp2,me2)*m13*
         m66*RSe451p*RSp453e - 1./162.*RN(mp2,me2)*m13*m66*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m13*m66*RSp561e*RSe453p + 1./162.
         *RN(mp2,me2)*m13*m66*RSp561e*RSe561p + 1./162.*RN(mp2,me2)*m13
         *m66*RSp561e*RSe451p - 1./162.*RN(mp2,me2)*m13*m66*RSp451e*
         RSe563p - 1./54.*RN(mp2,me2)*m13*m66*RSp451e*RSe453p + 1./162.
         *RN(mp2,me2)*m13*m66*RSp451e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./54.*RN(mp2,me2)*
         m13*m66*RSp451e*RSe451p - 1./81.*RN(mp2,me2)*m13*m46*RSp563e*
         RSe563p + 1./81.*RN(mp2,me2)*m13*m46*RSp563e*RSe453p + 1./81.*
         RN(mp2,me2)*m13*m46*RSp453e*RSe563p - 1./81.*RN(mp2,me2)*m13*
         m46*RSp453e*RSe453p + 1./81.*RN(mp2,me2)*m13*m46*RSe561p*
         RSp563e - 1./81.*RN(mp2,me2)*m13*m46*RSe561p*RSp453e - 1./81.*
         RN(mp2,me2)*m13*m46*RSe451p*RSp563e + 1./81.*RN(mp2,me2)*m13*
         m46*RSe451p*RSp453e + 1./81.*RN(mp2,me2)*m13*m46*RSp561e*
         RSe563p - 1./81.*RN(mp2,me2)*m13*m46*RSp561e*RSe453p - 1./81.*
         RN(mp2,me2)*m13*m46*RSp561e*RSe561p + 1./81.*RN(mp2,me2)*m13*
         m46*RSp561e*RSe451p - 1./81.*RN(mp2,me2)*m13*m46*RSp451e*
         RSe563p + 1./81.*RN(mp2,me2)*m13*m46*RSp451e*RSe453p + 1./81.*
         RN(mp2,me2)*m13*m46*RSp451e*RSe561p - 1./81.*RN(mp2,me2)*m13*
         m46*RSp451e*RSe451p + 1./54.*RN(mp2,me2)*m13*m44*RSp563e*
         RSe563p + 1./162.*RN(mp2,me2)*m13*m44*RSp563e*RSe453p + 1./162.
         *RN(mp2,me2)*m13*m44*RSp453e*RSe563p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./162.*RN(mp2,me2)*
         m13*m44*RSp453e*RSe453p - 1./54.*RN(mp2,me2)*m13*m44*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m13*m44*RSe561p*RSp453e - 1./162.
         *RN(mp2,me2)*m13*m44*RSe451p*RSp563e - 1./162.*RN(mp2,me2)*m13
         *m44*RSe451p*RSp453e - 1./54.*RN(mp2,me2)*m13*m44*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m13*m44*RSp561e*RSe453p + 1./54.
         *RN(mp2,me2)*m13*m44*RSp561e*RSe561p + 1./162.*RN(mp2,me2)*m13
         *m44*RSp561e*RSe451p - 1./162.*RN(mp2,me2)*m13*m44*RSp451e*
         RSe563p - 1./162.*RN(mp2,me2)*m13*m44*RSp451e*RSe453p + 1./162.
         *RN(mp2,me2)*m13*m44*RSp451e*RSe561p + 1./162.*RN(mp2,me2)*m13
         *m44*RSp451e*RSe451p + 1./162.*RN(mp2,me2)*m11*me2*RSp563e*
         RSe563p - 1./162.*RN(mp2,me2)*m11*me2*RSp563e*RSe453p - 1./162.
         *RN(mp2,me2)*m11*me2*RSp453e*RSe563p + 1./162.*RN(mp2,me2)*m11
         *me2*RSp453e*RSe453p + 1./162.*RN(mp2,me2)*m11*me2*RSe561p*
         RSp563e - 1./162.*RN(mp2,me2)*m11*me2*RSe561p*RSp453e - 1./162.
         *RN(mp2,me2)*m11*me2*RSe451p*RSp563e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./162.*RN(mp2,me2)*
         m11*me2*RSe451p*RSp453e + 1./162.*RN(mp2,me2)*m11*me2*RSp561e*
         RSe563p - 1./162.*RN(mp2,me2)*m11*me2*RSp561e*RSe453p + 1./54.
         *RN(mp2,me2)*m11*me2*RSp561e*RSe561p - 1./54.*RN(mp2,me2)*m11*
         me2*RSp561e*RSe451p - 1./162.*RN(mp2,me2)*m11*me2*RSp451e*
         RSe563p + 1./162.*RN(mp2,me2)*m11*me2*RSp451e*RSe453p - 1./54.
         *RN(mp2,me2)*m11*me2*RSp451e*RSe561p + 1./54.*RN(mp2,me2)*m11*
         me2*RSp451e*RSe451p - 1./324.*RN(mp2,me2)*m11*m66*RSp563e*
         RSe563p - 1./324.*RN(mp2,me2)*m11*m66*RSp563e*RSe453p - 1./324.
         *RN(mp2,me2)*m11*m66*RSp453e*RSe563p - 1./108.*RN(mp2,me2)*m11
         *m66*RSp453e*RSe453p - 1./324.*RN(mp2,me2)*m11*m66*RSe561p*
         RSp563e - 1./324.*RN(mp2,me2)*m11*m66*RSe561p*RSp453e - 1./324.
         *RN(mp2,me2)*m11*m66*RSe451p*RSp563e - 1./108.*RN(mp2,me2)*m11
         *m66*RSe451p*RSp453e - 1./324.*RN(mp2,me2)*m11*m66*RSp561e*
         RSe563p - 1./324.*RN(mp2,me2)*m11*m66*RSp561e*RSe453p - 1./108.
         *RN(mp2,me2)*m11*m66*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./108.*RN(mp2,me2
         )*m11*m66*RSp561e*RSe451p - 1./324.*RN(mp2,me2)*m11*m66*
         RSp451e*RSe563p - 1./108.*RN(mp2,me2)*m11*m66*RSp451e*RSe453p
          - 1./108.*RN(mp2,me2)*m11*m66*RSp451e*RSe561p - 1./36.*RN(mp2
         ,me2)*m11*m66*RSp451e*RSe451p + 1./162.*RN(mp2,me2)*m11*m46*
         RSp563e*RSe563p - 1./162.*RN(mp2,me2)*m11*m46*RSp563e*RSe453p
          - 1./162.*RN(mp2,me2)*m11*m46*RSp453e*RSe563p + 1./162.*RN(
         mp2,me2)*m11*m46*RSp453e*RSe453p + 1./162.*RN(mp2,me2)*m11*m46
         *RSe561p*RSp563e - 1./162.*RN(mp2,me2)*m11*m46*RSe561p*RSp453e
          - 1./162.*RN(mp2,me2)*m11*m46*RSe451p*RSp563e + 1./162.*RN(
         mp2,me2)*m11*m46*RSe451p*RSp453e + 1./162.*RN(mp2,me2)*m11*m46
         *RSp561e*RSe563p - 1./162.*RN(mp2,me2)*m11*m46*RSp561e*RSe453p
          + 1./54.*RN(mp2,me2)*m11*m46*RSp561e*RSe561p - 1./54.*RN(mp2,
         me2)*m11*m46*RSp561e*RSe451p - 1./162.*RN(mp2,me2)*m11*m46*
         RSp451e*RSe563p + 1./162.*RN(mp2,me2)*m11*m46*RSp451e*RSe453p
          - 1./54.*RN(mp2,me2)*m11*m46*RSp451e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./54.*RN(mp2,me2)*
         m11*m46*RSp451e*RSe451p - 1./108.*RN(mp2,me2)*m11*m44*RSp563e*
         RSe563p - 1./324.*RN(mp2,me2)*m11*m44*RSp563e*RSe453p - 1./324.
         *RN(mp2,me2)*m11*m44*RSp453e*RSe563p - 1./324.*RN(mp2,me2)*m11
         *m44*RSp453e*RSe453p - 1./108.*RN(mp2,me2)*m11*m44*RSe561p*
         RSp563e - 1./324.*RN(mp2,me2)*m11*m44*RSe561p*RSp453e - 1./324.
         *RN(mp2,me2)*m11*m44*RSe451p*RSp563e - 1./324.*RN(mp2,me2)*m11
         *m44*RSe451p*RSp453e - 1./108.*RN(mp2,me2)*m11*m44*RSp561e*
         RSe563p - 1./324.*RN(mp2,me2)*m11*m44*RSp561e*RSe453p - 1./36.
         *RN(mp2,me2)*m11*m44*RSp561e*RSe561p - 1./108.*RN(mp2,me2)*m11
         *m44*RSp561e*RSe451p - 1./324.*RN(mp2,me2)*m11*m44*RSp451e*
         RSe563p - 1./324.*RN(mp2,me2)*m11*m44*RSp451e*RSe453p - 1./108.
         *RN(mp2,me2)*m11*m44*RSp451e*RSe561p - 1./108.*RN(mp2,me2)*m11
         *m44*RSp451e*RSe451p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m55,L) * (  - 1./27.*m55*RSe463p
         *RS346pe - 1./54.*m55*pow(RSe463p,2) + 1./27.*m55*RS146pe*
         RSe463p + 1./81.*m55*RS546pe*RSe463p*RS346pe + 1./162.*m55*
         RS546pe*pow(RSe463p,2) - 1./81.*m55*RS546pe*RS146pe*RSe463p + 
         1./27.*m55*RSe461p*RS346pe + 1./27.*m55*RSe461p*RSe463p - 1./
         27.*m55*RSe461p*RS146pe - 1./81.*m55*RSe461p*RS546pe*RS346pe
          - 1./81.*m55*RSe461p*RS546pe*RSe463p + 1./81.*m55*RSe461p*
         RS546pe*RS146pe - 1./54.*m55*pow(RSe461p,2) + 1./162.*m55*pow(
         RSe461p,2)*RS546pe - 1./54.*m33*m55*RN3e*RSe463p*RS346pe + 1./
         162.*m33*m55*RN3e*RS546pe*RSe463p*RS346pe - 1./18.*m33*m55*
         RN3e*RSe461p*RS346pe + 1./54.*m33*m55*RN3e*RSe461p*RS546pe*
         RS346pe - 1./54.*m33*m55*RN1e*RS146pe*RSe463p + 1./162.*m33*
         m55*RN1e*RS546pe*RS146pe*RSe463p - 1./54.*m33*m55*RN1e*RSe461p
         *RS146pe + 1./162.*m33*m55*RN1e*RSe461p*RS546pe*RS146pe + 1./
         27.*m13*m55*RN3e*RSe463p*RS346pe - 1./81.*m13*m55*RN3e*RS546pe
         *RSe463p*RS346pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m55,L) * (  - 1./27.*m13*m55*
         RN3e*RSe461p*RS346pe + 1./81.*m13*m55*RN3e*RSe461p*RS546pe*
         RS346pe - 1./27.*m13*m55*RN1e*RS146pe*RSe463p + 1./81.*m13*m55
         *RN1e*RS546pe*RS146pe*RSe463p + 1./27.*m13*m55*RN1e*RSe461p*
         RS146pe - 1./81.*m13*m55*RN1e*RSe461p*RS546pe*RS146pe - 1./54.
         *m11*m55*RN3e*RSe463p*RS346pe + 1./162.*m11*m55*RN3e*RS546pe*
         RSe463p*RS346pe - 1./54.*m11*m55*RN3e*RSe461p*RS346pe + 1./162.
         *m11*m55*RN3e*RSe461p*RS546pe*RS346pe - 1./18.*m11*m55*RN1e*
         RS146pe*RSe463p + 1./54.*m11*m55*RN1e*RS546pe*RS146pe*RSe463p
          - 1./54.*m11*m55*RN1e*RSe461p*RS146pe + 1./162.*m11*m55*RN1e*
         RSe461p*RS546pe*RS146pe + 1./27.*RN(mp2,me2)*m55*me2*RSp463e*
         RSe463p - 1./81.*RN(mp2,me2)*m55*me2*RS546pe*RSp463e*RSe463p
          - 1./27.*RN(mp2,me2)*m55*me2*RSe461p*RSp463e + 1./81.*RN(mp2,
         me2)*m55*me2*RSe461p*RS546pe*RSp463e - 1./27.*RN(mp2,me2)*m55*
         me2*RSp461e*RSe463p + 1./81.*RN(mp2,me2)*m55*me2*RSp461e*
         RS546pe*RSe463p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m55,L) * ( 1./27.*RN(mp2,me2)*
         m55*me2*RSp461e*RSe461p - 1./81.*RN(mp2,me2)*m55*me2*RSp461e*
         RSe461p*RS546pe - 1./18.*RN(mp2,me2)*m33*m55*RSp463e*RSe463p
          + 1./54.*RN(mp2,me2)*m33*m55*RS546pe*RSp463e*RSe463p - 1./54.
         *RN(mp2,me2)*m33*m55*RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m33
         *m55*RSe461p*RS546pe*RSp463e - 1./54.*RN(mp2,me2)*m33*m55*
         RSp461e*RSe463p + 1./162.*RN(mp2,me2)*m33*m55*RSp461e*RS546pe*
         RSe463p - 1./54.*RN(mp2,me2)*m33*m55*RSp461e*RSe461p + 1./162.
         *RN(mp2,me2)*m33*m55*RSp461e*RSe461p*RS546pe + 1./27.*RN(mp2,
         me2)*m13*m55*RSp463e*RSe463p - 1./81.*RN(mp2,me2)*m13*m55*
         RS546pe*RSp463e*RSe463p - 1./27.*RN(mp2,me2)*m13*m55*RSe461p*
         RSp463e + 1./81.*RN(mp2,me2)*m13*m55*RSe461p*RS546pe*RSp463e
          - 1./27.*RN(mp2,me2)*m13*m55*RSp461e*RSe463p + 1./81.*RN(mp2,
         me2)*m13*m55*RSp461e*RS546pe*RSe463p + 1./27.*RN(mp2,me2)*m13*
         m55*RSp461e*RSe461p - 1./81.*RN(mp2,me2)*m13*m55*RSp461e*
         RSe461p*RS546pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m55,L) * (  - 1./54.*RN(mp2,me2)
         *m11*m55*RSp463e*RSe463p + 1./162.*RN(mp2,me2)*m11*m55*RS546pe
         *RSp463e*RSe463p - 1./54.*RN(mp2,me2)*m11*m55*RSe461p*RSp463e
          + 1./162.*RN(mp2,me2)*m11*m55*RSe461p*RS546pe*RSp463e - 1./54.
         *RN(mp2,me2)*m11*m55*RSp461e*RSe463p + 1./162.*RN(mp2,me2)*m11
         *m55*RSp461e*RS546pe*RSe463p - 1./18.*RN(mp2,me2)*m11*m55*
         RSp461e*RSe461p + 1./54.*RN(mp2,me2)*m11*m55*RSp461e*RSe461p*
         RS546pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./81.*me2*RSe463p
         *RS345pe + 1./81.*me2*RSe463p*RS346pe + 1./81.*me2*pow(
         RSe463p,2) + 1./81.*me2*RSe453p*RS345pe - 1./81.*me2*RSe453p*
         RS346pe - 2./81.*me2*RSe453p*RSe463p + 1./81.*me2*pow(
         RSe453p,2) + 1./81.*me2*RSp463e*RSe463p - 1./81.*me2*RSp463e*
         RSe453p - 1./81.*me2*RSp453e*RSe463p + 1./81.*me2*RSp453e*
         RSe453p + 1./81.*me2*RS145pe*RSe463p - 1./81.*me2*RS145pe*
         RSe453p - 1./81.*me2*RS146pe*RSe463p + 1./81.*me2*RS146pe*
         RSe453p + 1./81.*me2*RSe461p*RS345pe - 1./81.*me2*RSe461p*
         RS346pe - 2./81.*me2*RSe461p*RSe463p + 2./81.*me2*RSe461p*
         RSe453p - 1./81.*me2*RSe461p*RSp463e + 1./81.*me2*RSe461p*
         RSp453e - 1./81.*me2*RSe461p*RS145pe + 1./81.*me2*RSe461p*
         RS146pe + 1./81.*me2*pow(RSe461p,2) - 1./81.*me2*RSe451p*
         RS345pe + 1./81.*me2*RSe451p*RS346pe + 2./81.*me2*RSe451p*
         RSe463p - 2./81.*me2*RSe451p*RSe453p + 1./81.*me2*RSe451p*
         RSp463e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./81.*me2*RSe451p
         *RSp453e + 1./81.*me2*RSe451p*RS145pe - 1./81.*me2*RSe451p*
         RS146pe - 2./81.*me2*RSe451p*RSe461p + 1./81.*me2*pow(
         RSe451p,2) - 1./81.*me2*RSp461e*RSe463p + 1./81.*me2*RSp461e*
         RSe453p + 1./81.*me2*RSp461e*RSe461p - 1./81.*me2*RSp461e*
         RSe451p + 1./81.*me2*RSp451e*RSe463p - 1./81.*me2*RSp451e*
         RSe453p - 1./81.*me2*RSp451e*RSe461p + 1./81.*me2*RSp451e*
         RSe451p - 1./162.*m66*RSe463p*RS345pe - 1./162.*m66*RSe463p*
         RS346pe - 1./324.*m66*pow(RSe463p,2) - 1./54.*m66*RSe453p*
         RS345pe - 1./162.*m66*RSe453p*RS346pe - 1./162.*m66*RSe453p*
         RSe463p - 1./108.*m66*pow(RSe453p,2) + 1./162.*m66*RS145pe*
         RSe463p + 1./54.*m66*RS145pe*RSe453p + 1./162.*m66*RS146pe*
         RSe463p + 1./162.*m66*RS146pe*RSe453p + 1./162.*m66*RSe461p*
         RS345pe + 1./162.*m66*RSe461p*RS346pe + 1./162.*m66*RSe461p*
         RSe463p + 1./162.*m66*RSe461p*RSe453p - 1./162.*m66*RSe461p*
         RS145pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./162.*m66*
         RSe461p*RS146pe - 1./324.*m66*pow(RSe461p,2) + 1./54.*m66*
         RSe451p*RS345pe + 1./162.*m66*RSe451p*RS346pe + 1./162.*m66*
         RSe451p*RSe463p + 1./54.*m66*RSe451p*RSe453p - 1./54.*m66*
         RSe451p*RS145pe - 1./162.*m66*RSe451p*RS146pe - 1./162.*m66*
         RSe451p*RSe461p - 1./108.*m66*pow(RSe451p,2) - 1./81.*m56*
         RSe463p*RS345pe + 1./81.*m56*RSe463p*RS346pe + 1./162.*m56*
         pow(RSe463p,2) + 1./81.*m56*RSe453p*RS345pe - 1./81.*m56*
         RSe453p*RS346pe - 1./81.*m56*RSe453p*RSe463p + 1./162.*m56*
         pow(RSe453p,2) + 1./81.*m56*RS145pe*RSe463p - 1./81.*m56*
         RS145pe*RSe453p - 1./81.*m56*RS146pe*RSe463p + 1./81.*m56*
         RS146pe*RSe453p + 1./81.*m56*RSe461p*RS345pe - 1./81.*m56*
         RSe461p*RS346pe - 1./81.*m56*RSe461p*RSe463p + 1./81.*m56*
         RSe461p*RSe453p - 1./81.*m56*RSe461p*RS145pe + 1./81.*m56*
         RSe461p*RS146pe + 1./162.*m56*pow(RSe461p,2) - 1./81.*m56*
         RSe451p*RS345pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./81.*m56*RSe451p*
         RS346pe + 1./81.*m56*RSe451p*RSe463p - 1./81.*m56*RSe451p*
         RSe453p + 1./81.*m56*RSe451p*RS145pe - 1./81.*m56*RSe451p*
         RS146pe - 1./81.*m56*RSe451p*RSe461p + 1./162.*m56*pow(
         RSe451p,2) - 1./162.*m55*RSe463p*RS345pe - 1./54.*m55*RSe463p*
         RS346pe - 1./108.*m55*pow(RSe463p,2) - 1./162.*m55*RSe453p*
         RS345pe - 1./162.*m55*RSe453p*RS346pe - 1./162.*m55*RSe453p*
         RSe463p - 1./324.*m55*pow(RSe453p,2) + 1./162.*m55*RS145pe*
         RSe463p + 1./162.*m55*RS145pe*RSe453p + 1./54.*m55*RS146pe*
         RSe463p + 1./162.*m55*RS146pe*RSe453p + 1./162.*m55*RSe461p*
         RS345pe + 1./54.*m55*RSe461p*RS346pe + 1./54.*m55*RSe461p*
         RSe463p + 1./162.*m55*RSe461p*RSe453p - 1./162.*m55*RSe461p*
         RS145pe - 1./54.*m55*RSe461p*RS146pe - 1./108.*m55*pow(
         RSe461p,2) + 1./162.*m55*RSe451p*RS345pe + 1./162.*m55*RSe451p
         *RS346pe + 1./162.*m55*RSe451p*RSe463p + 1./162.*m55*RSe451p*
         RSe453p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./162.*m55*
         RSe451p*RS145pe - 1./162.*m55*RSe451p*RS146pe - 1./162.*m55*
         RSe451p*RSe461p - 1./324.*m55*pow(RSe451p,2) + 1./54.*m33*
         RSe463p*RS345pe - 1./54.*m33*RSe463p*RS346pe - 1./108.*m33*
         pow(RSe463p,2) - 1./54.*m33*RSe453p*RS345pe + 1./54.*m33*
         RSe453p*RS346pe + 1./54.*m33*RSe453p*RSe463p - 1./108.*m33*
         pow(RSe453p,2) + 1./162.*m33*RS145pe*RSe463p - 1./162.*m33*
         RS145pe*RSe453p - 1./162.*m33*RS146pe*RSe463p + 1./162.*m33*
         RS146pe*RSe453p + 1./162.*m33*RSe461p*RS345pe - 1./162.*m33*
         RSe461p*RS346pe - 1./162.*m33*RSe461p*RSe463p + 1./162.*m33*
         RSe461p*RSe453p + 1./162.*m33*RSe461p*RS145pe - 1./162.*m33*
         RSe461p*RS146pe - 1./324.*m33*pow(RSe461p,2) - 1./162.*m33*
         RSe451p*RS345pe + 1./162.*m33*RSe451p*RS346pe + 1./162.*m33*
         RSe451p*RSe463p - 1./162.*m33*RSe451p*RSe453p - 1./162.*m33*
         RSe451p*RS145pe + 1./162.*m33*RSe451p*RS146pe + 1./162.*m33*
         RSe451p*RSe461p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./324.*m33*pow(
         RSe451p,2) + 1./81.*m33*me2*RN3e*RSe463p*RS345pe - 1./81.*m33*
         me2*RN3e*RSe463p*RS346pe - 1./81.*m33*me2*RN3e*RSe453p*RS345pe
          + 1./81.*m33*me2*RN3e*RSe453p*RS346pe - 1./81.*m33*me2*RN3e*
         RSe461p*RS345pe + 1./81.*m33*me2*RN3e*RSe461p*RS346pe + 1./81.
         *m33*me2*RN3e*RSe451p*RS345pe - 1./81.*m33*me2*RN3e*RSe451p*
         RS346pe - 1./324.*m33*m66*RN3e*RSe463p*RS345pe - 1./324.*m33*
         m66*RN3e*RSe463p*RS346pe - 1./108.*m33*m66*RN3e*RSe453p*
         RS345pe - 1./324.*m33*m66*RN3e*RSe453p*RS346pe - 1./108.*m33*
         m66*RN3e*RSe461p*RS345pe - 1./108.*m33*m66*RN3e*RSe461p*
         RS346pe - 1./36.*m33*m66*RN3e*RSe451p*RS345pe - 1./108.*m33*
         m66*RN3e*RSe451p*RS346pe - 1./324.*m33*m66*RN1e*RS145pe*
         RSe463p - 1./108.*m33*m66*RN1e*RS145pe*RSe453p - 1./324.*m33*
         m66*RN1e*RS146pe*RSe463p - 1./324.*m33*m66*RN1e*RS146pe*
         RSe453p - 1./324.*m33*m66*RN1e*RSe461p*RS145pe - 1./324.*m33*
         m66*RN1e*RSe461p*RS146pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./108.*m33*m66*
         RN1e*RSe451p*RS145pe - 1./324.*m33*m66*RN1e*RSe451p*RS146pe - 
         1./162.*m33*m56*RN3e*RSe463p*RS345pe + 1./162.*m33*m56*RN3e*
         RSe463p*RS346pe + 1./162.*m33*m56*RN3e*RSe453p*RS345pe - 1./
         162.*m33*m56*RN3e*RSe453p*RS346pe - 1./54.*m33*m56*RN3e*
         RSe461p*RS345pe + 1./54.*m33*m56*RN3e*RSe461p*RS346pe + 1./54.
         *m33*m56*RN3e*RSe451p*RS345pe - 1./54.*m33*m56*RN3e*RSe451p*
         RS346pe - 1./162.*m33*m56*RN1e*RS145pe*RSe463p + 1./162.*m33*
         m56*RN1e*RS145pe*RSe453p + 1./162.*m33*m56*RN1e*RS146pe*
         RSe463p - 1./162.*m33*m56*RN1e*RS146pe*RSe453p - 1./162.*m33*
         m56*RN1e*RSe461p*RS145pe + 1./162.*m33*m56*RN1e*RSe461p*
         RS146pe + 1./162.*m33*m56*RN1e*RSe451p*RS145pe - 1./162.*m33*
         m56*RN1e*RSe451p*RS146pe - 1./324.*m33*m55*RN3e*RSe463p*
         RS345pe - 1./108.*m33*m55*RN3e*RSe463p*RS346pe - 1./324.*m33*
         m55*RN3e*RSe453p*RS345pe - 1./324.*m33*m55*RN3e*RSe453p*
         RS346pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./108.*m33*m55*
         RN3e*RSe461p*RS345pe - 1./36.*m33*m55*RN3e*RSe461p*RS346pe - 1.
         /108.*m33*m55*RN3e*RSe451p*RS345pe - 1./108.*m33*m55*RN3e*
         RSe451p*RS346pe - 1./324.*m33*m55*RN1e*RS145pe*RSe463p - 1./
         324.*m33*m55*RN1e*RS145pe*RSe453p - 1./108.*m33*m55*RN1e*
         RS146pe*RSe463p - 1./324.*m33*m55*RN1e*RS146pe*RSe453p - 1./
         324.*m33*m55*RN1e*RSe461p*RS145pe - 1./108.*m33*m55*RN1e*
         RSe461p*RS146pe - 1./324.*m33*m55*RN1e*RSe451p*RS145pe - 1./
         324.*m33*m55*RN1e*RSe451p*RS146pe - 1./54.*pow(m33,2)*RN3e*
         RSe463p*RS345pe + 1./54.*pow(m33,2)*RN3e*RSe463p*RS346pe + 1./
         54.*pow(m33,2)*RN3e*RSe453p*RS345pe - 1./54.*pow(m33,2)*RN3e*
         RSe453p*RS346pe - 1./162.*pow(m33,2)*RN3e*RSe461p*RS345pe + 1./
         162.*pow(m33,2)*RN3e*RSe461p*RS346pe + 1./162.*pow(m33,2)*RN3e
         *RSe451p*RS345pe - 1./162.*pow(m33,2)*RN3e*RSe451p*RS346pe - 1.
         /81.*m13*RSe463p*RS345pe + 1./81.*m13*RSe463p*RS346pe + 1./162.
         *m13*pow(RSe463p,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./81.*m13*RSe453p*
         RS345pe - 1./81.*m13*RSe453p*RS346pe - 1./81.*m13*RSe453p*
         RSe463p + 1./162.*m13*pow(RSe453p,2) + 1./81.*m13*RS145pe*
         RSe463p - 1./81.*m13*RS145pe*RSe453p - 1./81.*m13*RS146pe*
         RSe463p + 1./81.*m13*RS146pe*RSe453p + 1./81.*m13*RSe461p*
         RS345pe - 1./81.*m13*RSe461p*RS346pe - 1./81.*m13*RSe461p*
         RSe463p + 1./81.*m13*RSe461p*RSe453p - 1./81.*m13*RSe461p*
         RS145pe + 1./81.*m13*RSe461p*RS146pe + 1./162.*m13*pow(
         RSe461p,2) - 1./81.*m13*RSe451p*RS345pe + 1./81.*m13*RSe451p*
         RS346pe + 1./81.*m13*RSe451p*RSe463p - 1./81.*m13*RSe451p*
         RSe453p + 1./81.*m13*RSe451p*RS145pe - 1./81.*m13*RSe451p*
         RS146pe - 1./81.*m13*RSe451p*RSe461p + 1./162.*m13*pow(
         RSe451p,2) + 1./162.*m13*m66*RN3e*RSe463p*RS345pe + 1./162.*
         m13*m66*RN3e*RSe463p*RS346pe + 1./54.*m13*m66*RN3e*RSe453p*
         RS345pe + 1./162.*m13*m66*RN3e*RSe453p*RS346pe - 1./162.*m13*
         m66*RN3e*RSe461p*RS345pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./162.*m13*m66*
         RN3e*RSe461p*RS346pe - 1./54.*m13*m66*RN3e*RSe451p*RS345pe - 1.
         /162.*m13*m66*RN3e*RSe451p*RS346pe - 1./162.*m13*m66*RN1e*
         RS145pe*RSe463p - 1./54.*m13*m66*RN1e*RS145pe*RSe453p - 1./162.
         *m13*m66*RN1e*RS146pe*RSe463p - 1./162.*m13*m66*RN1e*RS146pe*
         RSe453p + 1./162.*m13*m66*RN1e*RSe461p*RS145pe + 1./162.*m13*
         m66*RN1e*RSe461p*RS146pe + 1./54.*m13*m66*RN1e*RSe451p*RS145pe
          + 1./162.*m13*m66*RN1e*RSe451p*RS146pe + 1./81.*m13*m56*RN3e*
         RSe463p*RS345pe - 1./81.*m13*m56*RN3e*RSe463p*RS346pe - 1./81.
         *m13*m56*RN3e*RSe453p*RS345pe + 1./81.*m13*m56*RN3e*RSe453p*
         RS346pe - 1./81.*m13*m56*RN3e*RSe461p*RS345pe + 1./81.*m13*m56
         *RN3e*RSe461p*RS346pe + 1./81.*m13*m56*RN3e*RSe451p*RS345pe - 
         1./81.*m13*m56*RN3e*RSe451p*RS346pe - 1./81.*m13*m56*RN1e*
         RS145pe*RSe463p + 1./81.*m13*m56*RN1e*RS145pe*RSe453p + 1./81.
         *m13*m56*RN1e*RS146pe*RSe463p - 1./81.*m13*m56*RN1e*RS146pe*
         RSe453p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./81.*m13*m56*RN1e*
         RSe461p*RS145pe - 1./81.*m13*m56*RN1e*RSe461p*RS146pe - 1./81.
         *m13*m56*RN1e*RSe451p*RS145pe + 1./81.*m13*m56*RN1e*RSe451p*
         RS146pe + 1./162.*m13*m55*RN3e*RSe463p*RS345pe + 1./54.*m13*
         m55*RN3e*RSe463p*RS346pe + 1./162.*m13*m55*RN3e*RSe453p*
         RS345pe + 1./162.*m13*m55*RN3e*RSe453p*RS346pe - 1./162.*m13*
         m55*RN3e*RSe461p*RS345pe - 1./54.*m13*m55*RN3e*RSe461p*RS346pe
          - 1./162.*m13*m55*RN3e*RSe451p*RS345pe - 1./162.*m13*m55*RN3e
         *RSe451p*RS346pe - 1./162.*m13*m55*RN1e*RS145pe*RSe463p - 1./
         162.*m13*m55*RN1e*RS145pe*RSe453p - 1./54.*m13*m55*RN1e*
         RS146pe*RSe463p - 1./162.*m13*m55*RN1e*RS146pe*RSe453p + 1./
         162.*m13*m55*RN1e*RSe461p*RS145pe + 1./54.*m13*m55*RN1e*
         RSe461p*RS146pe + 1./162.*m13*m55*RN1e*RSe451p*RS145pe + 1./
         162.*m13*m55*RN1e*RSe451p*RS146pe + 1./81.*m13*m33*RN3e*
         RSe463p*RS345pe - 1./81.*m13*m33*RN3e*RSe463p*RS346pe - 1./81.
         *m13*m33*RN3e*RSe453p*RS345pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./81.*m13*m33*RN3e*
         RSe453p*RS346pe - 1./81.*m13*m33*RN3e*RSe461p*RS345pe + 1./81.
         *m13*m33*RN3e*RSe461p*RS346pe + 1./81.*m13*m33*RN3e*RSe451p*
         RS345pe - 1./81.*m13*m33*RN3e*RSe451p*RS346pe + 1./162.*m11*
         RSe463p*RS345pe - 1./162.*m11*RSe463p*RS346pe - 1./324.*m11*
         pow(RSe463p,2) - 1./162.*m11*RSe453p*RS345pe + 1./162.*m11*
         RSe453p*RS346pe + 1./162.*m11*RSe453p*RSe463p - 1./324.*m11*
         pow(RSe453p,2) + 1./162.*m11*RS145pe*RSe463p - 1./162.*m11*
         RS145pe*RSe453p - 1./162.*m11*RS146pe*RSe463p + 1./162.*m11*
         RS146pe*RSe453p + 1./162.*m11*RSe461p*RS345pe - 1./162.*m11*
         RSe461p*RS346pe - 1./162.*m11*RSe461p*RSe463p + 1./162.*m11*
         RSe461p*RSe453p + 1./54.*m11*RSe461p*RS145pe - 1./54.*m11*
         RSe461p*RS146pe - 1./108.*m11*pow(RSe461p,2) - 1./162.*m11*
         RSe451p*RS345pe + 1./162.*m11*RSe451p*RS346pe + 1./162.*m11*
         RSe451p*RSe463p - 1./162.*m11*RSe451p*RSe453p - 1./54.*m11*
         RSe451p*RS145pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./54.*m11*RSe451p*
         RS146pe + 1./54.*m11*RSe451p*RSe461p - 1./108.*m11*pow(
         RSe451p,2) - 1./81.*m11*me2*RN1e*RS145pe*RSe463p + 1./81.*m11*
         me2*RN1e*RS145pe*RSe453p + 1./81.*m11*me2*RN1e*RS146pe*RSe463p
          - 1./81.*m11*me2*RN1e*RS146pe*RSe453p + 1./81.*m11*me2*RN1e*
         RSe461p*RS145pe - 1./81.*m11*me2*RN1e*RSe461p*RS146pe - 1./81.
         *m11*me2*RN1e*RSe451p*RS145pe + 1./81.*m11*me2*RN1e*RSe451p*
         RS146pe - 1./324.*m11*m66*RN3e*RSe463p*RS345pe - 1./324.*m11*
         m66*RN3e*RSe463p*RS346pe - 1./108.*m11*m66*RN3e*RSe453p*
         RS345pe - 1./324.*m11*m66*RN3e*RSe453p*RS346pe - 1./324.*m11*
         m66*RN3e*RSe461p*RS345pe - 1./324.*m11*m66*RN3e*RSe461p*
         RS346pe - 1./108.*m11*m66*RN3e*RSe451p*RS345pe - 1./324.*m11*
         m66*RN3e*RSe451p*RS346pe - 1./108.*m11*m66*RN1e*RS145pe*
         RSe463p - 1./36.*m11*m66*RN1e*RS145pe*RSe453p - 1./108.*m11*
         m66*RN1e*RS146pe*RSe463p - 1./108.*m11*m66*RN1e*RS146pe*
         RSe453p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./324.*m11*m66*
         RN1e*RSe461p*RS145pe - 1./324.*m11*m66*RN1e*RSe461p*RS146pe - 
         1./108.*m11*m66*RN1e*RSe451p*RS145pe - 1./324.*m11*m66*RN1e*
         RSe451p*RS146pe - 1./162.*m11*m56*RN3e*RSe463p*RS345pe + 1./
         162.*m11*m56*RN3e*RSe463p*RS346pe + 1./162.*m11*m56*RN3e*
         RSe453p*RS345pe - 1./162.*m11*m56*RN3e*RSe453p*RS346pe - 1./
         162.*m11*m56*RN3e*RSe461p*RS345pe + 1./162.*m11*m56*RN3e*
         RSe461p*RS346pe + 1./162.*m11*m56*RN3e*RSe451p*RS345pe - 1./
         162.*m11*m56*RN3e*RSe451p*RS346pe - 1./54.*m11*m56*RN1e*
         RS145pe*RSe463p + 1./54.*m11*m56*RN1e*RS145pe*RSe453p + 1./54.
         *m11*m56*RN1e*RS146pe*RSe463p - 1./54.*m11*m56*RN1e*RS146pe*
         RSe453p - 1./162.*m11*m56*RN1e*RSe461p*RS145pe + 1./162.*m11*
         m56*RN1e*RSe461p*RS146pe + 1./162.*m11*m56*RN1e*RSe451p*
         RS145pe - 1./162.*m11*m56*RN1e*RSe451p*RS146pe - 1./324.*m11*
         m55*RN3e*RSe463p*RS345pe - 1./108.*m11*m55*RN3e*RSe463p*
         RS346pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./324.*m11*m55*
         RN3e*RSe453p*RS345pe - 1./324.*m11*m55*RN3e*RSe453p*RS346pe - 
         1./324.*m11*m55*RN3e*RSe461p*RS345pe - 1./108.*m11*m55*RN3e*
         RSe461p*RS346pe - 1./324.*m11*m55*RN3e*RSe451p*RS345pe - 1./
         324.*m11*m55*RN3e*RSe451p*RS346pe - 1./108.*m11*m55*RN1e*
         RS145pe*RSe463p - 1./108.*m11*m55*RN1e*RS145pe*RSe453p - 1./36.
         *m11*m55*RN1e*RS146pe*RSe463p - 1./108.*m11*m55*RN1e*RS146pe*
         RSe453p - 1./324.*m11*m55*RN1e*RSe461p*RS145pe - 1./108.*m11*
         m55*RN1e*RSe461p*RS146pe - 1./324.*m11*m55*RN1e*RSe451p*
         RS145pe - 1./324.*m11*m55*RN1e*RSe451p*RS146pe - 1./162.*m11*
         m33*RN3e*RSe463p*RS345pe + 1./162.*m11*m33*RN3e*RSe463p*
         RS346pe + 1./162.*m11*m33*RN3e*RSe453p*RS345pe - 1./162.*m11*
         m33*RN3e*RSe453p*RS346pe - 1./162.*m11*m33*RN3e*RSe461p*
         RS345pe + 1./162.*m11*m33*RN3e*RSe461p*RS346pe + 1./162.*m11*
         m33*RN3e*RSe451p*RS345pe - 1./162.*m11*m33*RN3e*RSe451p*
         RS346pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./162.*m11*m33*
         RN1e*RS145pe*RSe463p + 1./162.*m11*m33*RN1e*RS145pe*RSe453p + 
         1./162.*m11*m33*RN1e*RS146pe*RSe463p - 1./162.*m11*m33*RN1e*
         RS146pe*RSe453p - 1./162.*m11*m33*RN1e*RSe461p*RS145pe + 1./
         162.*m11*m33*RN1e*RSe461p*RS146pe + 1./162.*m11*m33*RN1e*
         RSe451p*RS145pe - 1./162.*m11*m33*RN1e*RSe451p*RS146pe - 1./81.
         *m11*m13*RN1e*RS145pe*RSe463p + 1./81.*m11*m13*RN1e*RS145pe*
         RSe453p + 1./81.*m11*m13*RN1e*RS146pe*RSe463p - 1./81.*m11*m13
         *RN1e*RS146pe*RSe453p + 1./81.*m11*m13*RN1e*RSe461p*RS145pe - 
         1./81.*m11*m13*RN1e*RSe461p*RS146pe - 1./81.*m11*m13*RN1e*
         RSe451p*RS145pe + 1./81.*m11*m13*RN1e*RSe451p*RS146pe - 1./162.
         *pow(m11,2)*RN1e*RS145pe*RSe463p + 1./162.*pow(m11,2)*RN1e*
         RS145pe*RSe453p + 1./162.*pow(m11,2)*RN1e*RS146pe*RSe463p - 1./
         162.*pow(m11,2)*RN1e*RS146pe*RSe453p - 1./54.*pow(m11,2)*RN1e*
         RSe461p*RS145pe + 1./54.*pow(m11,2)*RN1e*RSe461p*RS146pe + 1./
         54.*pow(m11,2)*RN1e*RSe451p*RS145pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./54.*pow(m11,2)*
         RN1e*RSe451p*RS146pe - 1./81.*RN(mp2,me2)*mp2*me2*RSp463e*
         RSe463p + 1./81.*RN(mp2,me2)*mp2*me2*RSp463e*RSe453p + 1./81.*
         RN(mp2,me2)*mp2*me2*RSp453e*RSe463p - 1./81.*RN(mp2,me2)*mp2*
         me2*RSp453e*RSe453p + 1./81.*RN(mp2,me2)*mp2*me2*RSe461p*
         RSp463e - 1./81.*RN(mp2,me2)*mp2*me2*RSe461p*RSp453e - 1./81.*
         RN(mp2,me2)*mp2*me2*RSe451p*RSp463e + 1./81.*RN(mp2,me2)*mp2*
         me2*RSe451p*RSp453e + 1./81.*RN(mp2,me2)*mp2*me2*RSp461e*
         RSe463p - 1./81.*RN(mp2,me2)*mp2*me2*RSp461e*RSe453p - 1./81.*
         RN(mp2,me2)*mp2*me2*RSp461e*RSe461p + 1./81.*RN(mp2,me2)*mp2*
         me2*RSp461e*RSe451p - 1./81.*RN(mp2,me2)*mp2*me2*RSp451e*
         RSe463p + 1./81.*RN(mp2,me2)*mp2*me2*RSp451e*RSe453p + 1./81.*
         RN(mp2,me2)*mp2*me2*RSp451e*RSe461p - 1./81.*RN(mp2,me2)*mp2*
         me2*RSp451e*RSe451p + 1./162.*RN(mp2,me2)*m66*me2*RSp463e*
         RSe463p + 1./162.*RN(mp2,me2)*m66*me2*RSp463e*RSe453p + 1./162.
         *RN(mp2,me2)*m66*me2*RSp453e*RSe463p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./54.*RN(mp2,me2)*
         m66*me2*RSp453e*RSe453p - 1./162.*RN(mp2,me2)*m66*me2*RSe461p*
         RSp463e - 1./162.*RN(mp2,me2)*m66*me2*RSe461p*RSp453e - 1./162.
         *RN(mp2,me2)*m66*me2*RSe451p*RSp463e - 1./54.*RN(mp2,me2)*m66*
         me2*RSe451p*RSp453e - 1./162.*RN(mp2,me2)*m66*me2*RSp461e*
         RSe463p - 1./162.*RN(mp2,me2)*m66*me2*RSp461e*RSe453p + 1./162.
         *RN(mp2,me2)*m66*me2*RSp461e*RSe461p + 1./162.*RN(mp2,me2)*m66
         *me2*RSp461e*RSe451p - 1./162.*RN(mp2,me2)*m66*me2*RSp451e*
         RSe463p - 1./54.*RN(mp2,me2)*m66*me2*RSp451e*RSe453p + 1./162.
         *RN(mp2,me2)*m66*me2*RSp451e*RSe461p + 1./54.*RN(mp2,me2)*m66*
         me2*RSp451e*RSe451p - 1./81.*RN(mp2,me2)*m56*me2*RSp463e*
         RSe463p + 1./81.*RN(mp2,me2)*m56*me2*RSp463e*RSe453p + 1./81.*
         RN(mp2,me2)*m56*me2*RSp453e*RSe463p - 1./81.*RN(mp2,me2)*m56*
         me2*RSp453e*RSe453p + 1./81.*RN(mp2,me2)*m56*me2*RSe461p*
         RSp463e - 1./81.*RN(mp2,me2)*m56*me2*RSe461p*RSp453e - 1./81.*
         RN(mp2,me2)*m56*me2*RSe451p*RSp463e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./81.*RN(mp2,me2)*
         m56*me2*RSe451p*RSp453e + 1./81.*RN(mp2,me2)*m56*me2*RSp461e*
         RSe463p - 1./81.*RN(mp2,me2)*m56*me2*RSp461e*RSe453p - 1./81.*
         RN(mp2,me2)*m56*me2*RSp461e*RSe461p + 1./81.*RN(mp2,me2)*m56*
         me2*RSp461e*RSe451p - 1./81.*RN(mp2,me2)*m56*me2*RSp451e*
         RSe463p + 1./81.*RN(mp2,me2)*m56*me2*RSp451e*RSe453p + 1./81.*
         RN(mp2,me2)*m56*me2*RSp451e*RSe461p - 1./81.*RN(mp2,me2)*m56*
         me2*RSp451e*RSe451p + 1./54.*RN(mp2,me2)*m55*me2*RSp463e*
         RSe463p + 1./162.*RN(mp2,me2)*m55*me2*RSp463e*RSe453p + 1./162.
         *RN(mp2,me2)*m55*me2*RSp453e*RSe463p + 1./162.*RN(mp2,me2)*m55
         *me2*RSp453e*RSe453p - 1./54.*RN(mp2,me2)*m55*me2*RSe461p*
         RSp463e - 1./162.*RN(mp2,me2)*m55*me2*RSe461p*RSp453e - 1./162.
         *RN(mp2,me2)*m55*me2*RSe451p*RSp463e - 1./162.*RN(mp2,me2)*m55
         *me2*RSe451p*RSp453e - 1./54.*RN(mp2,me2)*m55*me2*RSp461e*
         RSe463p - 1./162.*RN(mp2,me2)*m55*me2*RSp461e*RSe453p + 1./54.
         *RN(mp2,me2)*m55*me2*RSp461e*RSe461p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./162.*RN(mp2,me2)*
         m55*me2*RSp461e*RSe451p - 1./162.*RN(mp2,me2)*m55*me2*RSp451e*
         RSe463p - 1./162.*RN(mp2,me2)*m55*me2*RSp451e*RSe453p + 1./162.
         *RN(mp2,me2)*m55*me2*RSp451e*RSe461p + 1./162.*RN(mp2,me2)*m55
         *me2*RSp451e*RSe451p + 1./54.*RN(mp2,me2)*m33*me2*RSp463e*
         RSe463p - 1./54.*RN(mp2,me2)*m33*me2*RSp463e*RSe453p - 1./54.*
         RN(mp2,me2)*m33*me2*RSp453e*RSe463p + 1./54.*RN(mp2,me2)*m33*
         me2*RSp453e*RSe453p + 1./162.*RN(mp2,me2)*m33*me2*RSe461p*
         RSp463e - 1./162.*RN(mp2,me2)*m33*me2*RSe461p*RSp453e - 1./162.
         *RN(mp2,me2)*m33*me2*RSe451p*RSp463e + 1./162.*RN(mp2,me2)*m33
         *me2*RSe451p*RSp453e + 1./162.*RN(mp2,me2)*m33*me2*RSp461e*
         RSe463p - 1./162.*RN(mp2,me2)*m33*me2*RSp461e*RSe453p + 1./162.
         *RN(mp2,me2)*m33*me2*RSp461e*RSe461p - 1./162.*RN(mp2,me2)*m33
         *me2*RSp461e*RSe451p - 1./162.*RN(mp2,me2)*m33*me2*RSp451e*
         RSe463p + 1./162.*RN(mp2,me2)*m33*me2*RSp451e*RSe453p - 1./162.
         *RN(mp2,me2)*m33*me2*RSp451e*RSe461p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./162.*RN(mp2,me2)*
         m33*me2*RSp451e*RSe451p - 1./108.*RN(mp2,me2)*m33*m66*RSp463e*
         RSe463p - 1./108.*RN(mp2,me2)*m33*m66*RSp463e*RSe453p - 1./108.
         *RN(mp2,me2)*m33*m66*RSp453e*RSe463p - 1./36.*RN(mp2,me2)*m33*
         m66*RSp453e*RSe453p - 1./324.*RN(mp2,me2)*m33*m66*RSe461p*
         RSp463e - 1./324.*RN(mp2,me2)*m33*m66*RSe461p*RSp453e - 1./324.
         *RN(mp2,me2)*m33*m66*RSe451p*RSp463e - 1./108.*RN(mp2,me2)*m33
         *m66*RSe451p*RSp453e - 1./324.*RN(mp2,me2)*m33*m66*RSp461e*
         RSe463p - 1./324.*RN(mp2,me2)*m33*m66*RSp461e*RSe453p - 1./324.
         *RN(mp2,me2)*m33*m66*RSp461e*RSe461p - 1./324.*RN(mp2,me2)*m33
         *m66*RSp461e*RSe451p - 1./324.*RN(mp2,me2)*m33*m66*RSp451e*
         RSe463p - 1./108.*RN(mp2,me2)*m33*m66*RSp451e*RSe453p - 1./324.
         *RN(mp2,me2)*m33*m66*RSp451e*RSe461p - 1./108.*RN(mp2,me2)*m33
         *m66*RSp451e*RSe451p + 1./54.*RN(mp2,me2)*m33*m56*RSp463e*
         RSe463p - 1./54.*RN(mp2,me2)*m33*m56*RSp463e*RSe453p - 1./54.*
         RN(mp2,me2)*m33*m56*RSp453e*RSe463p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./54.*RN(mp2,me2)*
         m33*m56*RSp453e*RSe453p + 1./162.*RN(mp2,me2)*m33*m56*RSe461p*
         RSp463e - 1./162.*RN(mp2,me2)*m33*m56*RSe461p*RSp453e - 1./162.
         *RN(mp2,me2)*m33*m56*RSe451p*RSp463e + 1./162.*RN(mp2,me2)*m33
         *m56*RSe451p*RSp453e + 1./162.*RN(mp2,me2)*m33*m56*RSp461e*
         RSe463p - 1./162.*RN(mp2,me2)*m33*m56*RSp461e*RSe453p + 1./162.
         *RN(mp2,me2)*m33*m56*RSp461e*RSe461p - 1./162.*RN(mp2,me2)*m33
         *m56*RSp461e*RSe451p - 1./162.*RN(mp2,me2)*m33*m56*RSp451e*
         RSe463p + 1./162.*RN(mp2,me2)*m33*m56*RSp451e*RSe453p - 1./162.
         *RN(mp2,me2)*m33*m56*RSp451e*RSe461p + 1./162.*RN(mp2,me2)*m33
         *m56*RSp451e*RSe451p - 1./36.*RN(mp2,me2)*m33*m55*RSp463e*
         RSe463p - 1./108.*RN(mp2,me2)*m33*m55*RSp463e*RSe453p - 1./108.
         *RN(mp2,me2)*m33*m55*RSp453e*RSe463p - 1./108.*RN(mp2,me2)*m33
         *m55*RSp453e*RSe453p - 1./108.*RN(mp2,me2)*m33*m55*RSe461p*
         RSp463e - 1./324.*RN(mp2,me2)*m33*m55*RSe461p*RSp453e - 1./324.
         *RN(mp2,me2)*m33*m55*RSe451p*RSp463e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./324.*RN(mp2,me2
         )*m33*m55*RSe451p*RSp453e - 1./108.*RN(mp2,me2)*m33*m55*
         RSp461e*RSe463p - 1./324.*RN(mp2,me2)*m33*m55*RSp461e*RSe453p
          - 1./108.*RN(mp2,me2)*m33*m55*RSp461e*RSe461p - 1./324.*RN(
         mp2,me2)*m33*m55*RSp461e*RSe451p - 1./324.*RN(mp2,me2)*m33*m55
         *RSp451e*RSe463p - 1./324.*RN(mp2,me2)*m33*m55*RSp451e*RSe453p
          - 1./324.*RN(mp2,me2)*m33*m55*RSp451e*RSe461p - 1./324.*RN(
         mp2,me2)*m33*m55*RSp451e*RSe451p - 1./81.*RN(mp2,me2)*m13*me2*
         RSp463e*RSe463p + 1./81.*RN(mp2,me2)*m13*me2*RSp463e*RSe453p
          + 1./81.*RN(mp2,me2)*m13*me2*RSp453e*RSe463p - 1./81.*RN(mp2,
         me2)*m13*me2*RSp453e*RSe453p + 1./81.*RN(mp2,me2)*m13*me2*
         RSe461p*RSp463e - 1./81.*RN(mp2,me2)*m13*me2*RSe461p*RSp453e
          - 1./81.*RN(mp2,me2)*m13*me2*RSe451p*RSp463e + 1./81.*RN(mp2,
         me2)*m13*me2*RSe451p*RSp453e + 1./81.*RN(mp2,me2)*m13*me2*
         RSp461e*RSe463p - 1./81.*RN(mp2,me2)*m13*me2*RSp461e*RSe453p
          - 1./81.*RN(mp2,me2)*m13*me2*RSp461e*RSe461p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./81.*RN(mp2,me2)*
         m13*me2*RSp461e*RSe451p - 1./81.*RN(mp2,me2)*m13*me2*RSp451e*
         RSe463p + 1./81.*RN(mp2,me2)*m13*me2*RSp451e*RSe453p + 1./81.*
         RN(mp2,me2)*m13*me2*RSp451e*RSe461p - 1./81.*RN(mp2,me2)*m13*
         me2*RSp451e*RSe451p + 1./162.*RN(mp2,me2)*m13*m66*RSp463e*
         RSe463p + 1./162.*RN(mp2,me2)*m13*m66*RSp463e*RSe453p + 1./162.
         *RN(mp2,me2)*m13*m66*RSp453e*RSe463p + 1./54.*RN(mp2,me2)*m13*
         m66*RSp453e*RSe453p - 1./162.*RN(mp2,me2)*m13*m66*RSe461p*
         RSp463e - 1./162.*RN(mp2,me2)*m13*m66*RSe461p*RSp453e - 1./162.
         *RN(mp2,me2)*m13*m66*RSe451p*RSp463e - 1./54.*RN(mp2,me2)*m13*
         m66*RSe451p*RSp453e - 1./162.*RN(mp2,me2)*m13*m66*RSp461e*
         RSe463p - 1./162.*RN(mp2,me2)*m13*m66*RSp461e*RSe453p + 1./162.
         *RN(mp2,me2)*m13*m66*RSp461e*RSe461p + 1./162.*RN(mp2,me2)*m13
         *m66*RSp461e*RSe451p - 1./162.*RN(mp2,me2)*m13*m66*RSp451e*
         RSe463p - 1./54.*RN(mp2,me2)*m13*m66*RSp451e*RSe453p + 1./162.
         *RN(mp2,me2)*m13*m66*RSp451e*RSe461p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./54.*RN(mp2,me2)*
         m13*m66*RSp451e*RSe451p - 1./81.*RN(mp2,me2)*m13*m56*RSp463e*
         RSe463p + 1./81.*RN(mp2,me2)*m13*m56*RSp463e*RSe453p + 1./81.*
         RN(mp2,me2)*m13*m56*RSp453e*RSe463p - 1./81.*RN(mp2,me2)*m13*
         m56*RSp453e*RSe453p + 1./81.*RN(mp2,me2)*m13*m56*RSe461p*
         RSp463e - 1./81.*RN(mp2,me2)*m13*m56*RSe461p*RSp453e - 1./81.*
         RN(mp2,me2)*m13*m56*RSe451p*RSp463e + 1./81.*RN(mp2,me2)*m13*
         m56*RSe451p*RSp453e + 1./81.*RN(mp2,me2)*m13*m56*RSp461e*
         RSe463p - 1./81.*RN(mp2,me2)*m13*m56*RSp461e*RSe453p - 1./81.*
         RN(mp2,me2)*m13*m56*RSp461e*RSe461p + 1./81.*RN(mp2,me2)*m13*
         m56*RSp461e*RSe451p - 1./81.*RN(mp2,me2)*m13*m56*RSp451e*
         RSe463p + 1./81.*RN(mp2,me2)*m13*m56*RSp451e*RSe453p + 1./81.*
         RN(mp2,me2)*m13*m56*RSp451e*RSe461p - 1./81.*RN(mp2,me2)*m13*
         m56*RSp451e*RSe451p + 1./54.*RN(mp2,me2)*m13*m55*RSp463e*
         RSe463p + 1./162.*RN(mp2,me2)*m13*m55*RSp463e*RSe453p + 1./162.
         *RN(mp2,me2)*m13*m55*RSp453e*RSe463p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./162.*RN(mp2,me2)*
         m13*m55*RSp453e*RSe453p - 1./54.*RN(mp2,me2)*m13*m55*RSe461p*
         RSp463e - 1./162.*RN(mp2,me2)*m13*m55*RSe461p*RSp453e - 1./162.
         *RN(mp2,me2)*m13*m55*RSe451p*RSp463e - 1./162.*RN(mp2,me2)*m13
         *m55*RSe451p*RSp453e - 1./54.*RN(mp2,me2)*m13*m55*RSp461e*
         RSe463p - 1./162.*RN(mp2,me2)*m13*m55*RSp461e*RSe453p + 1./54.
         *RN(mp2,me2)*m13*m55*RSp461e*RSe461p + 1./162.*RN(mp2,me2)*m13
         *m55*RSp461e*RSe451p - 1./162.*RN(mp2,me2)*m13*m55*RSp451e*
         RSe463p - 1./162.*RN(mp2,me2)*m13*m55*RSp451e*RSe453p + 1./162.
         *RN(mp2,me2)*m13*m55*RSp451e*RSe461p + 1./162.*RN(mp2,me2)*m13
         *m55*RSp451e*RSe451p + 1./162.*RN(mp2,me2)*m11*me2*RSp463e*
         RSe463p - 1./162.*RN(mp2,me2)*m11*me2*RSp463e*RSe453p - 1./162.
         *RN(mp2,me2)*m11*me2*RSp453e*RSe463p + 1./162.*RN(mp2,me2)*m11
         *me2*RSp453e*RSe453p + 1./162.*RN(mp2,me2)*m11*me2*RSe461p*
         RSp463e - 1./162.*RN(mp2,me2)*m11*me2*RSe461p*RSp453e - 1./162.
         *RN(mp2,me2)*m11*me2*RSe451p*RSp463e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./162.*RN(mp2,me2)*
         m11*me2*RSe451p*RSp453e + 1./162.*RN(mp2,me2)*m11*me2*RSp461e*
         RSe463p - 1./162.*RN(mp2,me2)*m11*me2*RSp461e*RSe453p + 1./54.
         *RN(mp2,me2)*m11*me2*RSp461e*RSe461p - 1./54.*RN(mp2,me2)*m11*
         me2*RSp461e*RSe451p - 1./162.*RN(mp2,me2)*m11*me2*RSp451e*
         RSe463p + 1./162.*RN(mp2,me2)*m11*me2*RSp451e*RSe453p - 1./54.
         *RN(mp2,me2)*m11*me2*RSp451e*RSe461p + 1./54.*RN(mp2,me2)*m11*
         me2*RSp451e*RSe451p - 1./324.*RN(mp2,me2)*m11*m66*RSp463e*
         RSe463p - 1./324.*RN(mp2,me2)*m11*m66*RSp463e*RSe453p - 1./324.
         *RN(mp2,me2)*m11*m66*RSp453e*RSe463p - 1./108.*RN(mp2,me2)*m11
         *m66*RSp453e*RSe453p - 1./324.*RN(mp2,me2)*m11*m66*RSe461p*
         RSp463e - 1./324.*RN(mp2,me2)*m11*m66*RSe461p*RSp453e - 1./324.
         *RN(mp2,me2)*m11*m66*RSe451p*RSp463e - 1./108.*RN(mp2,me2)*m11
         *m66*RSe451p*RSp453e - 1./324.*RN(mp2,me2)*m11*m66*RSp461e*
         RSe463p - 1./324.*RN(mp2,me2)*m11*m66*RSp461e*RSe453p - 1./108.
         *RN(mp2,me2)*m11*m66*RSp461e*RSe461p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./108.*RN(mp2,me2
         )*m11*m66*RSp461e*RSe451p - 1./324.*RN(mp2,me2)*m11*m66*
         RSp451e*RSe463p - 1./108.*RN(mp2,me2)*m11*m66*RSp451e*RSe453p
          - 1./108.*RN(mp2,me2)*m11*m66*RSp451e*RSe461p - 1./36.*RN(mp2
         ,me2)*m11*m66*RSp451e*RSe451p + 1./162.*RN(mp2,me2)*m11*m56*
         RSp463e*RSe463p - 1./162.*RN(mp2,me2)*m11*m56*RSp463e*RSe453p
          - 1./162.*RN(mp2,me2)*m11*m56*RSp453e*RSe463p + 1./162.*RN(
         mp2,me2)*m11*m56*RSp453e*RSe453p + 1./162.*RN(mp2,me2)*m11*m56
         *RSe461p*RSp463e - 1./162.*RN(mp2,me2)*m11*m56*RSe461p*RSp453e
          - 1./162.*RN(mp2,me2)*m11*m56*RSe451p*RSp463e + 1./162.*RN(
         mp2,me2)*m11*m56*RSe451p*RSp453e + 1./162.*RN(mp2,me2)*m11*m56
         *RSp461e*RSe463p - 1./162.*RN(mp2,me2)*m11*m56*RSp461e*RSe453p
          + 1./54.*RN(mp2,me2)*m11*m56*RSp461e*RSe461p - 1./54.*RN(mp2,
         me2)*m11*m56*RSp461e*RSe451p - 1./162.*RN(mp2,me2)*m11*m56*
         RSp451e*RSe463p + 1./162.*RN(mp2,me2)*m11*m56*RSp451e*RSe453p
          - 1./54.*RN(mp2,me2)*m11*m56*RSp451e*RSe461p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./54.*RN(mp2,me2)*
         m11*m56*RSp451e*RSe451p - 1./108.*RN(mp2,me2)*m11*m55*RSp463e*
         RSe463p - 1./324.*RN(mp2,me2)*m11*m55*RSp463e*RSe453p - 1./324.
         *RN(mp2,me2)*m11*m55*RSp453e*RSe463p - 1./324.*RN(mp2,me2)*m11
         *m55*RSp453e*RSe453p - 1./108.*RN(mp2,me2)*m11*m55*RSe461p*
         RSp463e - 1./324.*RN(mp2,me2)*m11*m55*RSe461p*RSp453e - 1./324.
         *RN(mp2,me2)*m11*m55*RSe451p*RSp463e - 1./324.*RN(mp2,me2)*m11
         *m55*RSe451p*RSp453e - 1./108.*RN(mp2,me2)*m11*m55*RSp461e*
         RSe463p - 1./324.*RN(mp2,me2)*m11*m55*RSp461e*RSe453p - 1./36.
         *RN(mp2,me2)*m11*m55*RSp461e*RSe461p - 1./108.*RN(mp2,me2)*m11
         *m55*RSp461e*RSe451p - 1./324.*RN(mp2,me2)*m11*m55*RSp451e*
         RSe463p - 1./324.*RN(mp2,me2)*m11*m55*RSp451e*RSe453p - 1./108.
         *RN(mp2,me2)*m11*m55*RSp451e*RSe461p - 1./108.*RN(mp2,me2)*m11
         *m55*RSp451e*RSe451p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m66,L) * (  - 1./27.*m66*RSe453p
         *RS345pe - 1./54.*m66*pow(RSe453p,2) + 1./27.*m66*RS145pe*
         RSe453p + 1./81.*m66*RS645pe*RSe453p*RS345pe + 1./162.*m66*
         RS645pe*pow(RSe453p,2) - 1./81.*m66*RS645pe*RS145pe*RSe453p + 
         1./27.*m66*RSe451p*RS345pe + 1./27.*m66*RSe451p*RSe453p - 1./
         27.*m66*RSe451p*RS145pe - 1./81.*m66*RSe451p*RS645pe*RS345pe
          - 1./81.*m66*RSe451p*RS645pe*RSe453p + 1./81.*m66*RSe451p*
         RS645pe*RS145pe - 1./54.*m66*pow(RSe451p,2) + 1./162.*m66*pow(
         RSe451p,2)*RS645pe - 1./54.*m33*m66*RN3e*RSe453p*RS345pe + 1./
         162.*m33*m66*RN3e*RS645pe*RSe453p*RS345pe - 1./18.*m33*m66*
         RN3e*RSe451p*RS345pe + 1./54.*m33*m66*RN3e*RSe451p*RS645pe*
         RS345pe - 1./54.*m33*m66*RN1e*RS145pe*RSe453p + 1./162.*m33*
         m66*RN1e*RS645pe*RS145pe*RSe453p - 1./54.*m33*m66*RN1e*RSe451p
         *RS145pe + 1./162.*m33*m66*RN1e*RSe451p*RS645pe*RS145pe + 1./
         27.*m13*m66*RN3e*RSe453p*RS345pe - 1./81.*m13*m66*RN3e*RS645pe
         *RSe453p*RS345pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m66,L) * (  - 1./27.*m13*m66*
         RN3e*RSe451p*RS345pe + 1./81.*m13*m66*RN3e*RSe451p*RS645pe*
         RS345pe - 1./27.*m13*m66*RN1e*RS145pe*RSe453p + 1./81.*m13*m66
         *RN1e*RS645pe*RS145pe*RSe453p + 1./27.*m13*m66*RN1e*RSe451p*
         RS145pe - 1./81.*m13*m66*RN1e*RSe451p*RS645pe*RS145pe - 1./54.
         *m11*m66*RN3e*RSe453p*RS345pe + 1./162.*m11*m66*RN3e*RS645pe*
         RSe453p*RS345pe - 1./54.*m11*m66*RN3e*RSe451p*RS345pe + 1./162.
         *m11*m66*RN3e*RSe451p*RS645pe*RS345pe - 1./18.*m11*m66*RN1e*
         RS145pe*RSe453p + 1./54.*m11*m66*RN1e*RS645pe*RS145pe*RSe453p
          - 1./54.*m11*m66*RN1e*RSe451p*RS145pe + 1./162.*m11*m66*RN1e*
         RSe451p*RS645pe*RS145pe + 1./27.*RN(mp2,me2)*m66*me2*RSp453e*
         RSe453p - 1./81.*RN(mp2,me2)*m66*me2*RS645pe*RSp453e*RSe453p
          - 1./27.*RN(mp2,me2)*m66*me2*RSe451p*RSp453e + 1./81.*RN(mp2,
         me2)*m66*me2*RSe451p*RS645pe*RSp453e - 1./27.*RN(mp2,me2)*m66*
         me2*RSp451e*RSe453p + 1./81.*RN(mp2,me2)*m66*me2*RSp451e*
         RS645pe*RSe453p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m66,L) * ( 1./27.*RN(mp2,me2)*
         m66*me2*RSp451e*RSe451p - 1./81.*RN(mp2,me2)*m66*me2*RSp451e*
         RSe451p*RS645pe - 1./18.*RN(mp2,me2)*m33*m66*RSp453e*RSe453p
          + 1./54.*RN(mp2,me2)*m33*m66*RS645pe*RSp453e*RSe453p - 1./54.
         *RN(mp2,me2)*m33*m66*RSe451p*RSp453e + 1./162.*RN(mp2,me2)*m33
         *m66*RSe451p*RS645pe*RSp453e - 1./54.*RN(mp2,me2)*m33*m66*
         RSp451e*RSe453p + 1./162.*RN(mp2,me2)*m33*m66*RSp451e*RS645pe*
         RSe453p - 1./54.*RN(mp2,me2)*m33*m66*RSp451e*RSe451p + 1./162.
         *RN(mp2,me2)*m33*m66*RSp451e*RSe451p*RS645pe + 1./27.*RN(mp2,
         me2)*m13*m66*RSp453e*RSe453p - 1./81.*RN(mp2,me2)*m13*m66*
         RS645pe*RSp453e*RSe453p - 1./27.*RN(mp2,me2)*m13*m66*RSe451p*
         RSp453e + 1./81.*RN(mp2,me2)*m13*m66*RSe451p*RS645pe*RSp453e
          - 1./27.*RN(mp2,me2)*m13*m66*RSp451e*RSe453p + 1./81.*RN(mp2,
         me2)*m13*m66*RSp451e*RS645pe*RSe453p + 1./27.*RN(mp2,me2)*m13*
         m66*RSp451e*RSe451p - 1./81.*RN(mp2,me2)*m13*m66*RSp451e*
         RSe451p*RS645pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(m66,L) * (  - 1./54.*RN(mp2,me2)
         *m11*m66*RSp453e*RSe453p + 1./162.*RN(mp2,me2)*m11*m66*RS645pe
         *RSp453e*RSe453p - 1./54.*RN(mp2,me2)*m11*m66*RSe451p*RSp453e
          + 1./162.*RN(mp2,me2)*m11*m66*RSe451p*RS645pe*RSp453e - 1./54.
         *RN(mp2,me2)*m11*m66*RSp451e*RSe453p + 1./162.*RN(mp2,me2)*m11
         *m66*RSp451e*RS645pe*RSe453p - 1./18.*RN(mp2,me2)*m11*m66*
         RSp451e*RSe451p + 1./54.*RN(mp2,me2)*m11*m66*RSp451e*RSe451p*
         RS645pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(mp2,L) * ( 1./135.*me2*RSp45613e
         *RSe45613p - 1./270.*me2*RSe45633p*RSp45613e - 1./270.*me2*
         RSp45633e*RSe45613p + 1./540.*me2*RSp45633e*RSe45633p - 1./270.
         *me2*RSe45611p*RSp45613e + 1./540.*me2*RSe45611p*RSp45633e - 1.
         /270.*me2*RSp45611e*RSe45613p + 1./540.*me2*RSp45611e*
         RSe45633p + 1./540.*me2*RSp45611e*RSe45611p + 1./135.*mp2*
         RSp45613e*RSe45613p - 1./270.*mp2*RSe45633p*RSp45613e - 1./270.
         *mp2*RSp45633e*RSe45613p + 1./540.*mp2*RSp45633e*RSe45633p - 1.
         /270.*mp2*RSe45611p*RSp45613e + 1./540.*mp2*RSe45611p*
         RSp45633e - 1./270.*mp2*RSp45611e*RSe45613p + 1./540.*mp2*
         RSp45611e*RSe45633p + 1./540.*mp2*RSp45611e*RSe45611p + 1./81.
         *m66*RSe456p*RSp453e*RS345pe + 1./81.*m66*RSe456p*RSp453e*
         RSe453p + 1./162.*m66*RSe456p*pow(RSp453e,2) - 1./81.*m66*
         RSe456p*RS145pe*RSp453e - 1./81.*m66*RSe456p*RSe451p*RSp453e
          - 1./81.*m66*RSe456p*RSp451e*RS345pe - 1./81.*m66*RSe456p*
         RSp451e*RSe453p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(mp2,L) * (  - 1./81.*m66*RSe456p
         *RSp451e*RSp453e + 1./81.*m66*RSe456p*RSp451e*RS145pe + 1./81.
         *m66*RSe456p*RSp451e*RSe451p + 1./162.*m66*RSe456p*pow(
         RSp451e,2) + 1./81.*m66*RSp456e*RSe453p*RS345pe + 1./162.*m66*
         RSp456e*pow(RSe453p,2) - 1./81.*m66*RSp456e*RS145pe*RSe453p - 
         1./81.*m66*RSp456e*RSe451p*RS345pe - 1./81.*m66*RSp456e*
         RSe451p*RSe453p + 1./81.*m66*RSp456e*RSe451p*RS145pe + 1./162.
         *m66*RSp456e*pow(RSe451p,2) + 1./81.*m55*RSe465p*RSp463e*
         RS346pe + 1./81.*m55*RSe465p*RSp463e*RSe463p + 1./162.*m55*
         RSe465p*pow(RSp463e,2) - 1./81.*m55*RSe465p*RS146pe*RSp463e - 
         1./81.*m55*RSe465p*RSe461p*RSp463e - 1./81.*m55*RSe465p*
         RSp461e*RS346pe - 1./81.*m55*RSe465p*RSp461e*RSe463p - 1./81.*
         m55*RSe465p*RSp461e*RSp463e + 1./81.*m55*RSe465p*RSp461e*
         RS146pe + 1./81.*m55*RSe465p*RSp461e*RSe461p + 1./162.*m55*
         RSe465p*pow(RSp461e,2) + 1./81.*m55*RSp465e*RSe463p*RS346pe + 
         1./162.*m55*RSp465e*pow(RSe463p,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(mp2,L) * (  - 1./81.*m55*RSp465e
         *RS146pe*RSe463p - 1./81.*m55*RSp465e*RSe461p*RS346pe - 1./81.
         *m55*RSp465e*RSe461p*RSe463p + 1./81.*m55*RSp465e*RSe461p*
         RS146pe + 1./162.*m55*RSp465e*pow(RSe461p,2) + 1./81.*m44*
         RSe564p*RSp563e*RS356pe + 1./81.*m44*RSe564p*RSp563e*RSe563p
          + 1./162.*m44*RSe564p*pow(RSp563e,2) - 1./81.*m44*RSe564p*
         RS156pe*RSp563e - 1./81.*m44*RSe564p*RSe561p*RSp563e - 1./81.*
         m44*RSe564p*RSp561e*RS356pe - 1./81.*m44*RSe564p*RSp561e*
         RSe563p - 1./81.*m44*RSe564p*RSp561e*RSp563e + 1./81.*m44*
         RSe564p*RSp561e*RS156pe + 1./81.*m44*RSe564p*RSp561e*RSe561p
          + 1./162.*m44*RSe564p*pow(RSp561e,2) + 1./81.*m44*RSp564e*
         RSe563p*RS356pe + 1./162.*m44*RSp564e*pow(RSe563p,2) - 1./81.*
         m44*RSp564e*RS156pe*RSe563p - 1./81.*m44*RSp564e*RSe561p*
         RS356pe - 1./81.*m44*RSp564e*RSe561p*RSe563p + 1./81.*m44*
         RSp564e*RSe561p*RS156pe + 1./162.*m44*RSp564e*pow(RSe561p,2)
          + 23./405.*m33*RSp45613e*RSe45613p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(mp2,L) * ( 8./405.*m33*RSe45633p
         *RSp45613e + 8./405.*m33*RSp45633e*RSe45613p - 2./81.*m33*
         RSp45633e*RSe45633p - 1./405.*m33*RSe45611p*RSp45613e - 11./
         810.*m33*RSe45611p*RSp45633e - 1./405.*m33*RSp45611e*RSe45613p
          - 11./810.*m33*RSp45611e*RSe45633p + 13./810.*m33*RSp45611e*
         RSe45611p + 1./162.*m33*m66*RN3e*RSp456e*RSe453p*RS345pe + 1./
         54.*m33*m66*RN3e*RSp456e*RSe451p*RS345pe + 1./162.*m33*m66*
         RN1e*RSp456e*RS145pe*RSe453p + 1./162.*m33*m66*RN1e*RSp456e*
         RSe451p*RS145pe + 1./162.*m33*m55*RN3e*RSp465e*RSe463p*RS346pe
          + 1./54.*m33*m55*RN3e*RSp465e*RSe461p*RS346pe + 1./162.*m33*
         m55*RN1e*RSp465e*RS146pe*RSe463p + 1./162.*m33*m55*RN1e*
         RSp465e*RSe461p*RS146pe + 1./162.*m33*m44*RN3e*RSp564e*RSe563p
         *RS356pe + 1./54.*m33*m44*RN3e*RSp564e*RSe561p*RS356pe + 1./
         162.*m33*m44*RN1e*RSp564e*RS156pe*RSe563p + 1./162.*m33*m44*
         RN1e*RSp564e*RSe561p*RS156pe - 1./18.*pow(m33,2)*RN3e*
         RSp45633e*RSe45613p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(mp2,L) * (  - 1./54.*pow(m33,2)*
         RN3e*RSp45633e*RSe45633p - 7./405.*m13*RSp45613e*RSe45613p + 7.
         /810.*m13*RSe45633p*RSp45613e + 7./810.*m13*RSp45633e*
         RSe45613p - 7./1620.*m13*RSp45633e*RSe45633p + 7./810.*m13*
         RSe45611p*RSp45613e - 7./1620.*m13*RSe45611p*RSp45633e + 7./
         810.*m13*RSp45611e*RSe45613p - 7./1620.*m13*RSp45611e*
         RSe45633p - 7./1620.*m13*RSp45611e*RSe45611p - 1./81.*m13*m66*
         RN3e*RSp456e*RSe453p*RS345pe + 1./81.*m13*m66*RN3e*RSp456e*
         RSe451p*RS345pe + 1./81.*m13*m66*RN1e*RSp456e*RS145pe*RSe453p
          - 1./81.*m13*m66*RN1e*RSp456e*RSe451p*RS145pe - 1./81.*m13*
         m55*RN3e*RSp465e*RSe463p*RS346pe + 1./81.*m13*m55*RN3e*RSp465e
         *RSe461p*RS346pe + 1./81.*m13*m55*RN1e*RSp465e*RS146pe*RSe463p
          - 1./81.*m13*m55*RN1e*RSp465e*RSe461p*RS146pe - 1./81.*m13*
         m44*RN3e*RSp564e*RSe563p*RS356pe + 1./81.*m13*m44*RN3e*RSp564e
         *RSe561p*RS356pe + 1./81.*m13*m44*RN1e*RSp564e*RS156pe*RSe563p
          - 1./81.*m13*m44*RN1e*RSp564e*RSe561p*RS156pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(mp2,L) * (  - 1./27.*m13*m33*
         RN3e*RSp45633e*RSe45613p + 1./27.*m13*m33*RN3e*RSp45633e*
         RSe45633p + 23./405.*m11*RSp45613e*RSe45613p - 1./405.*m11*
         RSe45633p*RSp45613e - 1./405.*m11*RSp45633e*RSe45613p + 13./
         810.*m11*RSp45633e*RSe45633p + 8./405.*m11*RSe45611p*RSp45613e
          - 11./810.*m11*RSe45611p*RSp45633e + 8./405.*m11*RSp45611e*
         RSe45613p - 11./810.*m11*RSp45611e*RSe45633p - 2./81.*m11*
         RSp45611e*RSe45611p + 1./162.*m11*m66*RN3e*RSp456e*RSe453p*
         RS345pe + 1./162.*m11*m66*RN3e*RSp456e*RSe451p*RS345pe + 1./54.
         *m11*m66*RN1e*RSp456e*RS145pe*RSe453p + 1./162.*m11*m66*RN1e*
         RSp456e*RSe451p*RS145pe + 1./162.*m11*m55*RN3e*RSp465e*RSe463p
         *RS346pe + 1./162.*m11*m55*RN3e*RSp465e*RSe461p*RS346pe + 1./
         54.*m11*m55*RN1e*RSp465e*RS146pe*RSe463p + 1./162.*m11*m55*
         RN1e*RSp465e*RSe461p*RS146pe + 1./162.*m11*m44*RN3e*RSp564e*
         RSe563p*RS356pe + 1./162.*m11*m44*RN3e*RSp564e*RSe561p*RS356pe
          + 1./54.*m11*m44*RN1e*RSp564e*RS156pe*RSe563p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(mp2,L) * ( 1./162.*m11*m44*RN1e*
         RSp564e*RSe561p*RS156pe - 1./54.*m11*m33*RN3e*RSp45633e*
         RSe45613p - 1./54.*m11*m33*RN3e*RSp45633e*RSe45633p - 1./54.*
         m11*m33*RN1e*RSp45611e*RSe45613p - 1./54.*m11*m33*RN1e*
         RSp45611e*RSe45611p - 1./27.*m11*m13*RN1e*RSp45611e*RSe45613p
          + 1./27.*m11*m13*RN1e*RSp45611e*RSe45611p - 1./18.*pow(m11,2)
         *RN1e*RSp45611e*RSe45613p - 1./54.*pow(m11,2)*RN1e*RSp45611e*
         RSe45611p - 1./81.*RN(mp2,me2)*m66*me2*RSp456e*RSp453e*RSe453p
          + 1./81.*RN(mp2,me2)*m66*me2*RSp456e*RSe451p*RSp453e + 1./81.
         *RN(mp2,me2)*m66*me2*RSp456e*RSp451e*RSe453p - 1./81.*RN(mp2,
         me2)*m66*me2*RSp456e*RSp451e*RSe451p - 1./81.*RN(mp2,me2)*m55*
         me2*RSp465e*RSp463e*RSe463p + 1./81.*RN(mp2,me2)*m55*me2*
         RSp465e*RSe461p*RSp463e + 1./81.*RN(mp2,me2)*m55*me2*RSp465e*
         RSp461e*RSe463p - 1./81.*RN(mp2,me2)*m55*me2*RSp465e*RSp461e*
         RSe461p - 1./81.*RN(mp2,me2)*m44*me2*RSp564e*RSp563e*RSe563p
          + 1./81.*RN(mp2,me2)*m44*me2*RSp564e*RSe561p*RSp563e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(mp2,L) * ( 1./81.*RN(mp2,me2)*
         m44*me2*RSp564e*RSp561e*RSe563p - 1./81.*RN(mp2,me2)*m44*me2*
         RSp564e*RSp561e*RSe561p + 1./54.*RN(mp2,me2)*m33*m66*RSp456e*
         RSp453e*RSe453p + 1./162.*RN(mp2,me2)*m33*m66*RSp456e*RSe451p*
         RSp453e + 1./162.*RN(mp2,me2)*m33*m66*RSp456e*RSp451e*RSe453p
          + 1./162.*RN(mp2,me2)*m33*m66*RSp456e*RSp451e*RSe451p + 1./54.
         *RN(mp2,me2)*m33*m55*RSp465e*RSp463e*RSe463p + 1./162.*RN(mp2,
         me2)*m33*m55*RSp465e*RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m33
         *m55*RSp465e*RSp461e*RSe463p + 1./162.*RN(mp2,me2)*m33*m55*
         RSp465e*RSp461e*RSe461p + 1./54.*RN(mp2,me2)*m33*m44*RSp564e*
         RSp563e*RSe563p + 1./162.*RN(mp2,me2)*m33*m44*RSp564e*RSe561p*
         RSp563e + 1./162.*RN(mp2,me2)*m33*m44*RSp564e*RSp561e*RSe563p
          + 1./162.*RN(mp2,me2)*m33*m44*RSp564e*RSp561e*RSe561p - 1./81.
         *RN(mp2,me2)*m13*m66*RSp456e*RSp453e*RSe453p + 1./81.*RN(mp2,
         me2)*m13*m66*RSp456e*RSe451p*RSp453e + 1./81.*RN(mp2,me2)*m13*
         m66*RSp456e*RSp451e*RSe453p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(mp2,L) * (  - 1./81.*RN(mp2,me2)
         *m13*m66*RSp456e*RSp451e*RSe451p - 1./81.*RN(mp2,me2)*m13*m55*
         RSp465e*RSp463e*RSe463p + 1./81.*RN(mp2,me2)*m13*m55*RSp465e*
         RSe461p*RSp463e + 1./81.*RN(mp2,me2)*m13*m55*RSp465e*RSp461e*
         RSe463p - 1./81.*RN(mp2,me2)*m13*m55*RSp465e*RSp461e*RSe461p
          - 1./81.*RN(mp2,me2)*m13*m44*RSp564e*RSp563e*RSe563p + 1./81.
         *RN(mp2,me2)*m13*m44*RSp564e*RSe561p*RSp563e + 1./81.*RN(mp2,
         me2)*m13*m44*RSp564e*RSp561e*RSe563p - 1./81.*RN(mp2,me2)*m13*
         m44*RSp564e*RSp561e*RSe561p + 1./162.*RN(mp2,me2)*m11*m66*
         RSp456e*RSp453e*RSe453p + 1./162.*RN(mp2,me2)*m11*m66*RSp456e*
         RSe451p*RSp453e + 1./162.*RN(mp2,me2)*m11*m66*RSp456e*RSp451e*
         RSe453p + 1./54.*RN(mp2,me2)*m11*m66*RSp456e*RSp451e*RSe451p
          + 1./162.*RN(mp2,me2)*m11*m55*RSp465e*RSp463e*RSe463p + 1./
         162.*RN(mp2,me2)*m11*m55*RSp465e*RSe461p*RSp463e + 1./162.*RN(
         mp2,me2)*m11*m55*RSp465e*RSp461e*RSe463p + 1./54.*RN(mp2,me2)*
         m11*m55*RSp465e*RSp461e*RSe461p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(mp2,L) * ( 1./162.*RN(mp2,me2)*
         m11*m44*RSp564e*RSp563e*RSe563p + 1./162.*RN(mp2,me2)*m11*m44*
         RSp564e*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m11*m44*RSp564e*
         RSp561e*RSe563p + 1./54.*RN(mp2,me2)*m11*m44*RSp564e*RSp561e*
         RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(me2,L) * ( 2./135.*me2*pow(
         RSe45613p,2) - 2./135.*me2*RSe45633p*RSe45613p + 1./270.*me2*
         pow(RSe45633p,2) - 2./135.*me2*RSe45611p*RSe45613p + 1./135.*
         me2*RSe45611p*RSe45633p + 1./270.*me2*pow(RSe45611p,2) + 2./81.
         *m66*RSe456p*RSe453p*RS345pe + 1./81.*m66*RSe456p*pow(
         RSe453p,2) - 2./81.*m66*RSe456p*RS145pe*RSe453p - 2./81.*m66*
         RSe456p*RSe451p*RS345pe - 2./81.*m66*RSe456p*RSe451p*RSe453p
          + 2./81.*m66*RSe456p*RSe451p*RS145pe + 1./81.*m66*RSe456p*
         pow(RSe451p,2) + 2./81.*m55*RSe465p*RSe463p*RS346pe + 1./81.*
         m55*RSe465p*pow(RSe463p,2) - 2./81.*m55*RSe465p*RS146pe*
         RSe463p - 2./81.*m55*RSe465p*RSe461p*RS346pe - 2./81.*m55*
         RSe465p*RSe461p*RSe463p + 2./81.*m55*RSe465p*RSe461p*RS146pe
          + 1./81.*m55*RSe465p*pow(RSe461p,2) + 2./81.*m44*RSe564p*
         RSe563p*RS356pe + 1./81.*m44*RSe564p*pow(RSe563p,2) - 2./81.*
         m44*RSe564p*RS156pe*RSe563p - 2./81.*m44*RSe564p*RSe561p*
         RS356pe );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(me2,L) * (  - 2./81.*m44*RSe564p
         *RSe561p*RSe563p + 2./81.*m44*RSe564p*RSe561p*RS156pe + 1./81.
         *m44*RSe564p*pow(RSe561p,2) + 23./405.*m33*pow(RSe45613p,2) + 
         16./405.*m33*RSe45633p*RSe45613p - 2./81.*m33*pow(RSe45633p,2)
          - 2./405.*m33*RSe45611p*RSe45613p - 11./405.*m33*RSe45611p*
         RSe45633p + 13./810.*m33*pow(RSe45611p,2) + 1./162.*m33*m66*
         RN3e*RSe456p*RSe453p*RS345pe + 1./54.*m33*m66*RN3e*RSe456p*
         RSe451p*RS345pe + 1./162.*m33*m66*RN1e*RSe456p*RS145pe*RSe453p
          + 1./162.*m33*m66*RN1e*RSe456p*RSe451p*RS145pe + 1./162.*m33*
         m55*RN3e*RSe465p*RSe463p*RS346pe + 1./54.*m33*m55*RN3e*RSe465p
         *RSe461p*RS346pe + 1./162.*m33*m55*RN1e*RSe465p*RS146pe*
         RSe463p + 1./162.*m33*m55*RN1e*RSe465p*RSe461p*RS146pe + 1./
         162.*m33*m44*RN3e*RSe564p*RSe563p*RS356pe + 1./54.*m33*m44*
         RN3e*RSe564p*RSe561p*RS356pe + 1./162.*m33*m44*RN1e*RSe564p*
         RS156pe*RSe563p + 1./162.*m33*m44*RN1e*RSe564p*RSe561p*RS156pe
          - 1./18.*pow(m33,2)*RN3e*RSe45633p*RSe45613p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(me2,L) * (  - 1./54.*pow(m33,2)*
         RN3e*pow(RSe45633p,2) - 7./405.*m13*pow(RSe45613p,2) + 7./405.
         *m13*RSe45633p*RSe45613p - 7./1620.*m13*pow(RSe45633p,2) + 7./
         405.*m13*RSe45611p*RSe45613p - 7./810.*m13*RSe45611p*RSe45633p
          - 7./1620.*m13*pow(RSe45611p,2) - 1./81.*m13*m66*RN3e*RSe456p
         *RSe453p*RS345pe + 1./81.*m13*m66*RN3e*RSe456p*RSe451p*RS345pe
          + 1./81.*m13*m66*RN1e*RSe456p*RS145pe*RSe453p - 1./81.*m13*
         m66*RN1e*RSe456p*RSe451p*RS145pe - 1./81.*m13*m55*RN3e*RSe465p
         *RSe463p*RS346pe + 1./81.*m13*m55*RN3e*RSe465p*RSe461p*RS346pe
          + 1./81.*m13*m55*RN1e*RSe465p*RS146pe*RSe463p - 1./81.*m13*
         m55*RN1e*RSe465p*RSe461p*RS146pe - 1./81.*m13*m44*RN3e*RSe564p
         *RSe563p*RS356pe + 1./81.*m13*m44*RN3e*RSe564p*RSe561p*RS356pe
          + 1./81.*m13*m44*RN1e*RSe564p*RS156pe*RSe563p - 1./81.*m13*
         m44*RN1e*RSe564p*RSe561p*RS156pe - 1./27.*m13*m33*RN3e*
         RSe45633p*RSe45613p + 1./27.*m13*m33*RN3e*pow(RSe45633p,2) + 
         23./405.*m11*pow(RSe45613p,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(me2,L) * (  - 2./405.*m11*
         RSe45633p*RSe45613p + 13./810.*m11*pow(RSe45633p,2) + 16./405.
         *m11*RSe45611p*RSe45613p - 11./405.*m11*RSe45611p*RSe45633p - 
         2./81.*m11*pow(RSe45611p,2) + 1./162.*m11*m66*RN3e*RSe456p*
         RSe453p*RS345pe + 1./162.*m11*m66*RN3e*RSe456p*RSe451p*RS345pe
          + 1./54.*m11*m66*RN1e*RSe456p*RS145pe*RSe453p + 1./162.*m11*
         m66*RN1e*RSe456p*RSe451p*RS145pe + 1./162.*m11*m55*RN3e*
         RSe465p*RSe463p*RS346pe + 1./162.*m11*m55*RN3e*RSe465p*RSe461p
         *RS346pe + 1./54.*m11*m55*RN1e*RSe465p*RS146pe*RSe463p + 1./
         162.*m11*m55*RN1e*RSe465p*RSe461p*RS146pe + 1./162.*m11*m44*
         RN3e*RSe564p*RSe563p*RS356pe + 1./162.*m11*m44*RN3e*RSe564p*
         RSe561p*RS356pe + 1./54.*m11*m44*RN1e*RSe564p*RS156pe*RSe563p
          + 1./162.*m11*m44*RN1e*RSe564p*RSe561p*RS156pe - 1./54.*m11*
         m33*RN3e*RSe45633p*RSe45613p - 1./54.*m11*m33*RN3e*pow(
         RSe45633p,2) - 1./54.*m11*m33*RN1e*RSe45611p*RSe45613p - 1./54.
         *m11*m33*RN1e*pow(RSe45611p,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(me2,L) * (  - 1./27.*m11*m13*
         RN1e*RSe45611p*RSe45613p + 1./27.*m11*m13*RN1e*pow(
         RSe45611p,2) - 1./18.*pow(m11,2)*RN1e*RSe45611p*RSe45613p - 1./
         54.*pow(m11,2)*RN1e*pow(RSe45611p,2) - 1./81.*RN(mp2,me2)*m66*
         me2*RSe456p*RSp453e*RSe453p + 1./81.*RN(mp2,me2)*m66*me2*
         RSe456p*RSe451p*RSp453e + 1./81.*RN(mp2,me2)*m66*me2*RSe456p*
         RSp451e*RSe453p - 1./81.*RN(mp2,me2)*m66*me2*RSe456p*RSp451e*
         RSe451p - 1./81.*RN(mp2,me2)*m55*me2*RSe465p*RSp463e*RSe463p
          + 1./81.*RN(mp2,me2)*m55*me2*RSe465p*RSe461p*RSp463e + 1./81.
         *RN(mp2,me2)*m55*me2*RSe465p*RSp461e*RSe463p - 1./81.*RN(mp2,
         me2)*m55*me2*RSe465p*RSp461e*RSe461p - 1./81.*RN(mp2,me2)*m44*
         me2*RSe564p*RSp563e*RSe563p + 1./81.*RN(mp2,me2)*m44*me2*
         RSe564p*RSe561p*RSp563e + 1./81.*RN(mp2,me2)*m44*me2*RSe564p*
         RSp561e*RSe563p - 1./81.*RN(mp2,me2)*m44*me2*RSe564p*RSp561e*
         RSe561p + 1./54.*RN(mp2,me2)*m33*m66*RSe456p*RSp453e*RSe453p
          + 1./162.*RN(mp2,me2)*m33*m66*RSe456p*RSe451p*RSp453e );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(me2,L) * ( 1./162.*RN(mp2,me2)*
         m33*m66*RSe456p*RSp451e*RSe453p + 1./162.*RN(mp2,me2)*m33*m66*
         RSe456p*RSp451e*RSe451p + 1./54.*RN(mp2,me2)*m33*m55*RSe465p*
         RSp463e*RSe463p + 1./162.*RN(mp2,me2)*m33*m55*RSe465p*RSe461p*
         RSp463e + 1./162.*RN(mp2,me2)*m33*m55*RSe465p*RSp461e*RSe463p
          + 1./162.*RN(mp2,me2)*m33*m55*RSe465p*RSp461e*RSe461p + 1./54.
         *RN(mp2,me2)*m33*m44*RSe564p*RSp563e*RSe563p + 1./162.*RN(mp2,
         me2)*m33*m44*RSe564p*RSe561p*RSp563e + 1./162.*RN(mp2,me2)*m33
         *m44*RSe564p*RSp561e*RSe563p + 1./162.*RN(mp2,me2)*m33*m44*
         RSe564p*RSp561e*RSe561p - 1./81.*RN(mp2,me2)*m13*m66*RSe456p*
         RSp453e*RSe453p + 1./81.*RN(mp2,me2)*m13*m66*RSe456p*RSe451p*
         RSp453e + 1./81.*RN(mp2,me2)*m13*m66*RSe456p*RSp451e*RSe453p
          - 1./81.*RN(mp2,me2)*m13*m66*RSe456p*RSp451e*RSe451p - 1./81.
         *RN(mp2,me2)*m13*m55*RSe465p*RSp463e*RSe463p + 1./81.*RN(mp2,
         me2)*m13*m55*RSe465p*RSe461p*RSp463e + 1./81.*RN(mp2,me2)*m13*
         m55*RSe465p*RSp461e*RSe463p );

      M13p6x23RV +=  + Ab(me2,mu2)*AbV(me2,L) * (  - 1./81.*RN(mp2,me2)
         *m13*m55*RSe465p*RSp461e*RSe461p - 1./81.*RN(mp2,me2)*m13*m44*
         RSe564p*RSp563e*RSe563p + 1./81.*RN(mp2,me2)*m13*m44*RSe564p*
         RSe561p*RSp563e + 1./81.*RN(mp2,me2)*m13*m44*RSe564p*RSp561e*
         RSe563p - 1./81.*RN(mp2,me2)*m13*m44*RSe564p*RSp561e*RSe561p
          + 1./162.*RN(mp2,me2)*m11*m66*RSe456p*RSp453e*RSe453p + 1./
         162.*RN(mp2,me2)*m11*m66*RSe456p*RSe451p*RSp453e + 1./162.*RN(
         mp2,me2)*m11*m66*RSe456p*RSp451e*RSe453p + 1./54.*RN(mp2,me2)*
         m11*m66*RSe456p*RSp451e*RSe451p + 1./162.*RN(mp2,me2)*m11*m55*
         RSe465p*RSp463e*RSe463p + 1./162.*RN(mp2,me2)*m11*m55*RSe465p*
         RSe461p*RSp463e + 1./162.*RN(mp2,me2)*m11*m55*RSe465p*RSp461e*
         RSe463p + 1./54.*RN(mp2,me2)*m11*m55*RSe465p*RSp461e*RSe461p
          + 1./162.*RN(mp2,me2)*m11*m44*RSe564p*RSp563e*RSe563p + 1./
         162.*RN(mp2,me2)*m11*m44*RSe564p*RSe561p*RSp563e + 1./162.*RN(
         mp2,me2)*m11*m44*RSe564p*RSp561e*RSe563p + 1./54.*RN(mp2,me2)*
         m11*m44*RSe564p*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m11,m33,L) * (  - 1./162.*m33*
         m66*RSe456p*RS145pe*RS345pe - 1./162.*m33*m55*RSe465p*RS146pe*
         RS346pe - 1./162.*m33*m44*RSe564p*RS156pe*RS356pe + 1./54.*
         pow(m33,2)*RSe45633p*RS14563pe - 1./81.*m13*m66*RSe456p*
         RS145pe*RS345pe - 1./81.*m13*m55*RSe465p*RS146pe*RS346pe - 1./
         81.*m13*m44*RSe564p*RS156pe*RS356pe + 1./27.*m13*m33*RSe45633p
         *RS14563pe - 1./54.*m11*m66*RSe456p*RS145pe*RS345pe - 1./54.*
         m11*m55*RSe465p*RS146pe*RS346pe - 1./54.*m11*m44*RSe564p*
         RS156pe*RS356pe + 1./18.*m11*m33*RSe45633p*RS14563pe + 1./54.*
         m11*m33*RSe45611p*RS34561pe + 1./27.*m11*m13*RSe45611p*
         RS34561pe + 1./18.*pow(m11,2)*RSe45611p*RS34561pe );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m11,mp2,L) * (  - 1./162.*m33*
         m66*RSe456p*RS145pe*RSp453e - 1./162.*m33*m66*RSe456p*RSp451e*
         RS145pe - 1./162.*m33*m55*RSe465p*RS146pe*RSp463e - 1./162.*
         m33*m55*RSe465p*RSp461e*RS146pe - 1./162.*m33*m44*RSe564p*
         RS156pe*RSp563e - 1./162.*m33*m44*RSe564p*RSp561e*RS156pe - 1./
         81.*m13*m66*RSe456p*RS145pe*RSp453e + 1./81.*m13*m66*RSe456p*
         RSp451e*RS145pe - 1./81.*m13*m55*RSe465p*RS146pe*RSp463e + 1./
         81.*m13*m55*RSe465p*RSp461e*RS146pe - 1./81.*m13*m44*RSe564p*
         RS156pe*RSp563e + 1./81.*m13*m44*RSe564p*RSp561e*RS156pe - 1./
         54.*m11*m66*RSe456p*RS145pe*RSp453e - 1./162.*m11*m66*RSe456p*
         RSp451e*RS145pe - 1./54.*m11*m55*RSe465p*RS146pe*RSp463e - 1./
         162.*m11*m55*RSe465p*RSp461e*RS146pe - 1./54.*m11*m44*RSe564p*
         RS156pe*RSp563e - 1./162.*m11*m44*RSe564p*RSp561e*RS156pe + 1./
         54.*m11*m33*RSe45611p*RSp45613e + 1./54.*m11*m33*RSp45611e*
         RSe45611p + 1./27.*m11*m13*RSe45611p*RSp45613e - 1./27.*m11*
         m13*RSp45611e*RSe45611p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m11,mp2,L) * ( 1./18.*pow(m11,2)
         *RSe45611p*RSp45613e + 1./54.*pow(m11,2)*RSp45611e*RSe45611p )
         ;

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m11,me2,L) * (  - 1./162.*m33*
         m66*RSe456p*RS145pe*RSe453p - 1./162.*m33*m66*RSe456p*RSe451p*
         RS145pe - 1./162.*m33*m55*RSe465p*RS146pe*RSe463p - 1./162.*
         m33*m55*RSe465p*RSe461p*RS146pe - 1./162.*m33*m44*RSe564p*
         RS156pe*RSe563p - 1./162.*m33*m44*RSe564p*RSe561p*RS156pe - 1./
         81.*m13*m66*RSe456p*RS145pe*RSe453p + 1./81.*m13*m66*RSe456p*
         RSe451p*RS145pe - 1./81.*m13*m55*RSe465p*RS146pe*RSe463p + 1./
         81.*m13*m55*RSe465p*RSe461p*RS146pe - 1./81.*m13*m44*RSe564p*
         RS156pe*RSe563p + 1./81.*m13*m44*RSe564p*RSe561p*RS156pe - 1./
         54.*m11*m66*RSe456p*RS145pe*RSe453p - 1./162.*m11*m66*RSe456p*
         RSe451p*RS145pe - 1./54.*m11*m55*RSe465p*RS146pe*RSe463p - 1./
         162.*m11*m55*RSe465p*RSe461p*RS146pe - 1./54.*m11*m44*RSe564p*
         RS156pe*RSe563p - 1./162.*m11*m44*RSe564p*RSe561p*RS156pe + 1./
         54.*m11*m33*RSe45611p*RSe45613p + 1./54.*m11*m33*pow(
         RSe45611p,2) + 1./27.*m11*m13*RSe45611p*RSe45613p - 1./27.*m11
         *m13*pow(RSe45611p,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m11,me2,L) * ( 1./18.*pow(m11,2)
         *RSe45611p*RSe45613p + 1./54.*pow(m11,2)*pow(RSe45611p,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m11,L) * (  - 1./270.*me2*
         RD1456ep*RSe45613p + 1./540.*me2*RD1456ep*RSe45633p + 1./540.*
         me2*RSe45611p*RD1456ep - 1./405.*m33*RD1456ep*RSe45613p - 11./
         810.*m33*RD1456ep*RSe45633p + 13./810.*m33*RSe45611p*RD1456ep
          - 1./324.*m33*m66*RSe456p*pow(RS145pe,2) - 1./324.*m33*m55*
         RSe465p*pow(RS146pe,2) - 1./324.*m33*m44*RSe564p*pow(
         RS156pe,2) + 7./810.*m13*RD1456ep*RSe45613p - 7./1620.*m13*
         RD1456ep*RSe45633p - 7./1620.*m13*RSe45611p*RD1456ep + 1./162.
         *m13*m66*RSe456p*pow(RS145pe,2) + 1./162.*m13*m55*RSe465p*pow(
         RS146pe,2) + 1./162.*m13*m44*RSe564p*pow(RS156pe,2) + 13./810.
         *m11*RD1456ep*RSe45613p - 19./1620.*m11*RD1456ep*RSe45633p - 
         37./1620.*m11*RSe45611p*RD1456ep - 1./324.*m11*m66*RSe456p*
         pow(RS145pe,2) - 1./324.*m11*m55*RSe465p*pow(RS146pe,2) - 1./
         324.*m11*m44*RSe564p*pow(RS156pe,2) + 1./54.*m11*m33*RSe45611p
         *RS14563pe + 1./54.*m11*m33*RSe45611p*RC1456pe - 1./54.*m11*
         m33*RN1e*RD1456ep*RSe45613p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m11,L) * (  - 1./54.*m11*m33*
         RN1e*RSe45611p*RD1456ep + 1./27.*m11*m13*RSe45611p*RS14563pe
          - 1./27.*m11*m13*RSe45611p*RC1456pe - 1./27.*m11*m13*RN1e*
         RD1456ep*RSe45613p + 1./27.*m11*m13*RN1e*RSe45611p*RD1456ep + 
         1./18.*pow(m11,2)*RSe45611p*RS14563pe + 1./54.*pow(m11,2)*
         RSe45611p*RC1456pe - 1./18.*pow(m11,2)*RN1e*RD1456ep*RSe45613p
          - 1./54.*pow(m11,2)*RN1e*RSe45611p*RD1456ep );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m14,L) * ( 1./216.*m44*me2*
         RSe45611p - 1./108.*m44*me2*RSe561p + 1./216.*m44*me2*RSe564p
          - 1./432.*pow(m44,2)*RSe45611p - 1./216.*pow(m44,2)*RSe561p
          - 1./144.*pow(m44,2)*RSe564p + 1./216.*m33*me2*RSe45611p - 1./
         108.*m33*me2*RSe561p + 1./216.*m33*me2*RSe564p - 1./432.*m33*
         m44*RSe45611p - 1./216.*m33*m44*RSe561p - 1./144.*m33*m44*
         RSe564p - 1./108.*m14*me2*RSe45611p + 1./54.*m14*me2*RSe561p
          - 1./108.*m14*me2*RSe564p + 1./108.*m14*m44*RSe45611p + 1./54.
         *m14*m44*RSe564p + 1./216.*m14*m33*RSe45611p - 1./108.*m14*m33
         *RSe561p + 1./216.*m14*m33*RSe564p - 1./108.*pow(m14,2)*
         RSe45611p + 1./54.*pow(m14,2)*RSe561p - 1./108.*pow(m14,2)*
         RSe564p - 1./108.*m13*me2*RSe45611p + 1./54.*m13*me2*RSe561p
          - 1./108.*m13*me2*RSe564p + 1./216.*m13*m44*RSe45611p + 1./
         108.*m13*m44*RSe561p + 1./72.*m13*m44*RSe564p - 1./108.*m13*
         m14*RSe45611p + 1./54.*m13*m14*RSe561p - 1./108.*m13*m14*
         RSe564p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m14,L) * ( 1./108.*m11*me2*
         RSe45611p - 1./54.*m11*me2*RSe561p + 1./108.*m11*me2*RSe564p
          - 5./432.*m11*m44*RSe45611p - 1./72.*m11*m44*RSe561p - 7./432.
         *m11*m44*RSe564p - 1./144.*m11*m33*RSe45611p - 1./216.*m11*m33
         *RSe561p - 1./432.*m11*m33*RSe564p + 5./216.*m11*m14*RSe45611p
          - 1./108.*m11*m14*RSe561p + 1./72.*m11*m14*RSe564p + 1./72.*
         m11*m13*RSe45611p + 1./108.*m11*m13*RSe561p + 1./216.*m11*m13*
         RSe564p - 1./72.*pow(m11,2)*RSe45611p - 1./108.*pow(m11,2)*
         RSe561p - 1./216.*pow(m11,2)*RSe564p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m15,L) * ( 1./216.*m55*me2*
         RSe45611p - 1./108.*m55*me2*RSe461p + 1./216.*m55*me2*RSe465p
          - 1./432.*pow(m55,2)*RSe45611p - 1./216.*pow(m55,2)*RSe461p
          - 1./144.*pow(m55,2)*RSe465p + 1./216.*m33*me2*RSe45611p - 1./
         108.*m33*me2*RSe461p + 1./216.*m33*me2*RSe465p - 1./432.*m33*
         m55*RSe45611p - 1./216.*m33*m55*RSe461p - 1./144.*m33*m55*
         RSe465p - 1./108.*m15*me2*RSe45611p + 1./54.*m15*me2*RSe461p
          - 1./108.*m15*me2*RSe465p + 1./108.*m15*m55*RSe45611p + 1./54.
         *m15*m55*RSe465p + 1./216.*m15*m33*RSe45611p - 1./108.*m15*m33
         *RSe461p + 1./216.*m15*m33*RSe465p - 1./108.*pow(m15,2)*
         RSe45611p + 1./54.*pow(m15,2)*RSe461p - 1./108.*pow(m15,2)*
         RSe465p - 1./108.*m13*me2*RSe45611p + 1./54.*m13*me2*RSe461p
          - 1./108.*m13*me2*RSe465p + 1./216.*m13*m55*RSe45611p + 1./
         108.*m13*m55*RSe461p + 1./72.*m13*m55*RSe465p - 1./108.*m13*
         m15*RSe45611p + 1./54.*m13*m15*RSe461p - 1./108.*m13*m15*
         RSe465p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m15,L) * ( 1./108.*m11*me2*
         RSe45611p - 1./54.*m11*me2*RSe461p + 1./108.*m11*me2*RSe465p
          - 5./432.*m11*m55*RSe45611p - 1./72.*m11*m55*RSe461p - 7./432.
         *m11*m55*RSe465p - 1./144.*m11*m33*RSe45611p - 1./216.*m11*m33
         *RSe461p - 1./432.*m11*m33*RSe465p + 5./216.*m11*m15*RSe45611p
          - 1./108.*m11*m15*RSe461p + 1./72.*m11*m15*RSe465p + 1./72.*
         m11*m13*RSe45611p + 1./108.*m11*m13*RSe461p + 1./216.*m11*m13*
         RSe465p - 1./72.*pow(m11,2)*RSe45611p - 1./108.*pow(m11,2)*
         RSe461p - 1./216.*pow(m11,2)*RSe465p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m16,L) * ( 1./216.*m66*me2*
         RSe45611p - 1./108.*m66*me2*RSe451p + 1./216.*m66*me2*RSe456p
          - 1./432.*pow(m66,2)*RSe45611p - 1./216.*pow(m66,2)*RSe451p
          - 1./144.*pow(m66,2)*RSe456p + 1./216.*m33*me2*RSe45611p - 1./
         108.*m33*me2*RSe451p + 1./216.*m33*me2*RSe456p - 1./432.*m33*
         m66*RSe45611p - 1./216.*m33*m66*RSe451p - 1./144.*m33*m66*
         RSe456p - 1./108.*m16*me2*RSe45611p + 1./54.*m16*me2*RSe451p
          - 1./108.*m16*me2*RSe456p + 1./108.*m16*m66*RSe45611p + 1./54.
         *m16*m66*RSe456p + 1./216.*m16*m33*RSe45611p - 1./108.*m16*m33
         *RSe451p + 1./216.*m16*m33*RSe456p - 1./108.*pow(m16,2)*
         RSe45611p + 1./54.*pow(m16,2)*RSe451p - 1./108.*pow(m16,2)*
         RSe456p - 1./108.*m13*me2*RSe45611p + 1./54.*m13*me2*RSe451p
          - 1./108.*m13*me2*RSe456p + 1./216.*m13*m66*RSe45611p + 1./
         108.*m13*m66*RSe451p + 1./72.*m13*m66*RSe456p - 1./108.*m13*
         m16*RSe45611p + 1./54.*m13*m16*RSe451p - 1./108.*m13*m16*
         RSe456p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m16,L) * ( 1./108.*m11*me2*
         RSe45611p - 1./54.*m11*me2*RSe451p + 1./108.*m11*me2*RSe456p
          - 5./432.*m11*m66*RSe45611p - 1./72.*m11*m66*RSe451p - 7./432.
         *m11*m66*RSe456p - 1./144.*m11*m33*RSe45611p - 1./216.*m11*m33
         *RSe451p - 1./432.*m11*m33*RSe456p + 5./216.*m11*m16*RSe45611p
          - 1./108.*m11*m16*RSe451p + 1./72.*m11*m16*RSe456p + 1./72.*
         m11*m13*RSe45611p + 1./108.*m11*m13*RSe451p + 1./216.*m11*m13*
         RSe456p - 1./72.*pow(m11,2)*RSe45611p - 1./108.*pow(m11,2)*
         RSe451p - 1./216.*pow(m11,2)*RSe456p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m33,mp2,L) * (  - 1./162.*m33*
         m66*RSe456p*RSp453e*RS345pe - 1./54.*m33*m66*RSe456p*RSp451e*
         RS345pe - 1./162.*m33*m55*RSe465p*RSp463e*RS346pe - 1./54.*m33
         *m55*RSe465p*RSp461e*RS346pe - 1./162.*m33*m44*RSe564p*RSp563e
         *RS356pe - 1./54.*m33*m44*RSe564p*RSp561e*RS356pe + 1./18.*
         pow(m33,2)*RSe45633p*RSp45613e + 1./54.*pow(m33,2)*RSp45633e*
         RSe45633p + 1./81.*m13*m66*RSe456p*RSp453e*RS345pe - 1./81.*
         m13*m66*RSe456p*RSp451e*RS345pe + 1./81.*m13*m55*RSe465p*
         RSp463e*RS346pe - 1./81.*m13*m55*RSe465p*RSp461e*RS346pe + 1./
         81.*m13*m44*RSe564p*RSp563e*RS356pe - 1./81.*m13*m44*RSe564p*
         RSp561e*RS356pe + 1./27.*m13*m33*RSe45633p*RSp45613e - 1./27.*
         m13*m33*RSp45633e*RSe45633p - 1./162.*m11*m66*RSe456p*RSp453e*
         RS345pe - 1./162.*m11*m66*RSe456p*RSp451e*RS345pe - 1./162.*
         m11*m55*RSe465p*RSp463e*RS346pe - 1./162.*m11*m55*RSe465p*
         RSp461e*RS346pe - 1./162.*m11*m44*RSe564p*RSp563e*RS356pe - 1./
         162.*m11*m44*RSe564p*RSp561e*RS356pe );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m33,mp2,L) * ( 1./54.*m11*m33*
         RSe45633p*RSp45613e + 1./54.*m11*m33*RSp45633e*RSe45633p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m33,me2,L) * (  - 1./162.*m33*
         m66*RSe456p*RSe453p*RS345pe - 1./54.*m33*m66*RSe456p*RSe451p*
         RS345pe - 1./162.*m33*m55*RSe465p*RSe463p*RS346pe - 1./54.*m33
         *m55*RSe465p*RSe461p*RS346pe - 1./162.*m33*m44*RSe564p*RSe563p
         *RS356pe - 1./54.*m33*m44*RSe564p*RSe561p*RS356pe + 1./18.*
         pow(m33,2)*RSe45633p*RSe45613p + 1./54.*pow(m33,2)*pow(
         RSe45633p,2) + 1./81.*m13*m66*RSe456p*RSe453p*RS345pe - 1./81.
         *m13*m66*RSe456p*RSe451p*RS345pe + 1./81.*m13*m55*RSe465p*
         RSe463p*RS346pe - 1./81.*m13*m55*RSe465p*RSe461p*RS346pe + 1./
         81.*m13*m44*RSe564p*RSe563p*RS356pe - 1./81.*m13*m44*RSe564p*
         RSe561p*RS356pe + 1./27.*m13*m33*RSe45633p*RSe45613p - 1./27.*
         m13*m33*pow(RSe45633p,2) - 1./162.*m11*m66*RSe456p*RSe453p*
         RS345pe - 1./162.*m11*m66*RSe456p*RSe451p*RS345pe - 1./162.*
         m11*m55*RSe465p*RSe463p*RS346pe - 1./162.*m11*m55*RSe465p*
         RSe461p*RS346pe - 1./162.*m11*m44*RSe564p*RSe563p*RS356pe - 1./
         162.*m11*m44*RSe564p*RSe561p*RS356pe );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m33,me2,L) * ( 1./54.*m11*m33*
         RSe45633p*RSe45613p + 1./54.*m11*m33*pow(RSe45633p,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m33,L) * (  - 1./270.*me2*
         RD3456ep*RSe45613p + 1./540.*me2*RSe45633p*RD3456ep + 1./540.*
         me2*RSe45611p*RD3456ep + 13./810.*m33*RD3456ep*RSe45613p - 37./
         1620.*m33*RSe45633p*RD3456ep - 19./1620.*m33*RSe45611p*
         RD3456ep - 1./324.*m33*m66*RSe456p*pow(RS345pe,2) - 1./324.*
         m33*m55*RSe465p*pow(RS346pe,2) - 1./324.*m33*m44*RSe564p*pow(
         RS356pe,2) + 1./18.*pow(m33,2)*RSe45633p*RS34561pe + 1./54.*
         pow(m33,2)*RSe45633p*RC3456pe - 1./18.*pow(m33,2)*RN3e*
         RD3456ep*RSe45613p - 1./54.*pow(m33,2)*RN3e*RSe45633p*RD3456ep
          + 7./810.*m13*RD3456ep*RSe45613p - 7./1620.*m13*RSe45633p*
         RD3456ep - 7./1620.*m13*RSe45611p*RD3456ep + 1./162.*m13*m66*
         RSe456p*pow(RS345pe,2) + 1./162.*m13*m55*RSe465p*pow(
         RS346pe,2) + 1./162.*m13*m44*RSe564p*pow(RS356pe,2) + 1./27.*
         m13*m33*RSe45633p*RS34561pe - 1./27.*m13*m33*RSe45633p*
         RC3456pe - 1./27.*m13*m33*RN3e*RD3456ep*RSe45613p + 1./27.*m13
         *m33*RN3e*RSe45633p*RD3456ep );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m33,L) * (  - 1./405.*m11*
         RD3456ep*RSe45613p + 13./810.*m11*RSe45633p*RD3456ep - 11./810.
         *m11*RSe45611p*RD3456ep - 1./324.*m11*m66*RSe456p*pow(
         RS345pe,2) - 1./324.*m11*m55*RSe465p*pow(RS346pe,2) - 1./324.*
         m11*m44*RSe564p*pow(RS356pe,2) + 1./54.*m11*m33*RSe45633p*
         RS34561pe + 1./54.*m11*m33*RSe45633p*RC3456pe - 1./54.*m11*m33
         *RN3e*RD3456ep*RSe45613p - 1./54.*m11*m33*RN3e*RSe45633p*
         RD3456ep );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m34,L) * ( 1./216.*m44*me2*
         RSe45633p - 1./108.*m44*me2*RSe563p + 1./216.*m44*me2*RSe564p
          - 1./432.*pow(m44,2)*RSe45633p - 1./216.*pow(m44,2)*RSe563p
          - 1./144.*pow(m44,2)*RSe564p - 1./108.*m34*me2*RSe45633p + 1./
         54.*m34*me2*RSe563p - 1./108.*m34*me2*RSe564p + 1./108.*m34*
         m44*RSe45633p + 1./54.*m34*m44*RSe564p - 1./108.*pow(m34,2)*
         RSe45633p + 1./54.*pow(m34,2)*RSe563p - 1./108.*pow(m34,2)*
         RSe564p + 1./108.*m33*me2*RSe45633p - 1./54.*m33*me2*RSe563p
          + 1./108.*m33*me2*RSe564p - 5./432.*m33*m44*RSe45633p - 1./72.
         *m33*m44*RSe563p - 7./432.*m33*m44*RSe564p + 5./216.*m33*m34*
         RSe45633p - 1./108.*m33*m34*RSe563p + 1./72.*m33*m34*RSe564p
          - 1./72.*pow(m33,2)*RSe45633p - 1./108.*pow(m33,2)*RSe563p - 
         1./216.*pow(m33,2)*RSe564p - 1./108.*m13*me2*RSe45633p + 1./54.
         *m13*me2*RSe563p - 1./108.*m13*me2*RSe564p + 1./216.*m13*m44*
         RSe45633p + 1./108.*m13*m44*RSe563p + 1./72.*m13*m44*RSe564p
          - 1./108.*m13*m34*RSe45633p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m34,L) * ( 1./54.*m13*m34*
         RSe563p - 1./108.*m13*m34*RSe564p + 1./72.*m13*m33*RSe45633p
          + 1./108.*m13*m33*RSe563p + 1./216.*m13*m33*RSe564p + 1./216.
         *m11*me2*RSe45633p - 1./108.*m11*me2*RSe563p + 1./216.*m11*me2
         *RSe564p - 1./432.*m11*m44*RSe45633p - 1./216.*m11*m44*RSe563p
          - 1./144.*m11*m44*RSe564p + 1./216.*m11*m34*RSe45633p - 1./
         108.*m11*m34*RSe563p + 1./216.*m11*m34*RSe564p - 1./144.*m11*
         m33*RSe45633p - 1./216.*m11*m33*RSe563p - 1./432.*m11*m33*
         RSe564p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m35,L) * ( 1./216.*m55*me2*
         RSe45633p - 1./108.*m55*me2*RSe463p + 1./216.*m55*me2*RSe465p
          - 1./432.*pow(m55,2)*RSe45633p - 1./216.*pow(m55,2)*RSe463p
          - 1./144.*pow(m55,2)*RSe465p - 1./108.*m35*me2*RSe45633p + 1./
         54.*m35*me2*RSe463p - 1./108.*m35*me2*RSe465p + 1./108.*m35*
         m55*RSe45633p + 1./54.*m35*m55*RSe465p - 1./108.*pow(m35,2)*
         RSe45633p + 1./54.*pow(m35,2)*RSe463p - 1./108.*pow(m35,2)*
         RSe465p + 1./108.*m33*me2*RSe45633p - 1./54.*m33*me2*RSe463p
          + 1./108.*m33*me2*RSe465p - 5./432.*m33*m55*RSe45633p - 1./72.
         *m33*m55*RSe463p - 7./432.*m33*m55*RSe465p + 5./216.*m33*m35*
         RSe45633p - 1./108.*m33*m35*RSe463p + 1./72.*m33*m35*RSe465p
          - 1./72.*pow(m33,2)*RSe45633p - 1./108.*pow(m33,2)*RSe463p - 
         1./216.*pow(m33,2)*RSe465p - 1./108.*m13*me2*RSe45633p + 1./54.
         *m13*me2*RSe463p - 1./108.*m13*me2*RSe465p + 1./216.*m13*m55*
         RSe45633p + 1./108.*m13*m55*RSe463p + 1./72.*m13*m55*RSe465p
          - 1./108.*m13*m35*RSe45633p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m35,L) * ( 1./54.*m13*m35*
         RSe463p - 1./108.*m13*m35*RSe465p + 1./72.*m13*m33*RSe45633p
          + 1./108.*m13*m33*RSe463p + 1./216.*m13*m33*RSe465p + 1./216.
         *m11*me2*RSe45633p - 1./108.*m11*me2*RSe463p + 1./216.*m11*me2
         *RSe465p - 1./432.*m11*m55*RSe45633p - 1./216.*m11*m55*RSe463p
          - 1./144.*m11*m55*RSe465p + 1./216.*m11*m35*RSe45633p - 1./
         108.*m11*m35*RSe463p + 1./216.*m11*m35*RSe465p - 1./144.*m11*
         m33*RSe45633p - 1./216.*m11*m33*RSe463p - 1./432.*m11*m33*
         RSe465p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m36,L) * ( 1./216.*m66*me2*
         RSe45633p - 1./108.*m66*me2*RSe453p + 1./216.*m66*me2*RSe456p
          - 1./432.*pow(m66,2)*RSe45633p - 1./216.*pow(m66,2)*RSe453p
          - 1./144.*pow(m66,2)*RSe456p - 1./108.*m36*me2*RSe45633p + 1./
         54.*m36*me2*RSe453p - 1./108.*m36*me2*RSe456p + 1./108.*m36*
         m66*RSe45633p + 1./54.*m36*m66*RSe456p - 1./108.*pow(m36,2)*
         RSe45633p + 1./54.*pow(m36,2)*RSe453p - 1./108.*pow(m36,2)*
         RSe456p + 1./108.*m33*me2*RSe45633p - 1./54.*m33*me2*RSe453p
          + 1./108.*m33*me2*RSe456p - 5./432.*m33*m66*RSe45633p - 1./72.
         *m33*m66*RSe453p - 7./432.*m33*m66*RSe456p + 5./216.*m33*m36*
         RSe45633p - 1./108.*m33*m36*RSe453p + 1./72.*m33*m36*RSe456p
          - 1./72.*pow(m33,2)*RSe45633p - 1./108.*pow(m33,2)*RSe453p - 
         1./216.*pow(m33,2)*RSe456p - 1./108.*m13*me2*RSe45633p + 1./54.
         *m13*me2*RSe453p - 1./108.*m13*me2*RSe456p + 1./216.*m13*m66*
         RSe45633p + 1./108.*m13*m66*RSe453p + 1./72.*m13*m66*RSe456p
          - 1./108.*m13*m36*RSe45633p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(m36,L) * ( 1./54.*m13*m36*
         RSe453p - 1./108.*m13*m36*RSe456p + 1./72.*m13*m33*RSe45633p
          + 1./108.*m13*m33*RSe453p + 1./216.*m13*m33*RSe456p + 1./216.
         *m11*me2*RSe45633p - 1./108.*m11*me2*RSe453p + 1./216.*m11*me2
         *RSe456p - 1./432.*m11*m66*RSe45633p - 1./216.*m11*m66*RSe453p
          - 1./144.*m11*m66*RSe456p + 1./216.*m11*m36*RSe45633p - 1./
         108.*m11*m36*RSe453p + 1./216.*m11*m36*RSe456p - 1./144.*m11*
         m33*RSe45633p - 1./216.*m11*m33*RSe453p - 1./432.*m11*m33*
         RSe456p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(mp2,L) * ( 1./162.*m66*mp2*
         RSe456p*pow(RSp453e,2) - 1./81.*m66*mp2*RSe456p*RSp451e*
         RSp453e + 1./162.*m66*mp2*RSe456p*pow(RSp451e,2) + 1./162.*m55
         *mp2*RSe465p*pow(RSp463e,2) - 1./81.*m55*mp2*RSe465p*RSp461e*
         RSp463e + 1./162.*m55*mp2*RSe465p*pow(RSp461e,2) + 1./162.*m44
         *mp2*RSe564p*pow(RSp563e,2) - 1./81.*m44*mp2*RSe564p*RSp561e*
         RSp563e + 1./162.*m44*mp2*RSe564p*pow(RSp561e,2) - 1./108.*m33
         *m66*RSe456p*pow(RSp453e,2) - 1./162.*m33*m66*RSe456p*RSp451e*
         RSp453e - 1./324.*m33*m66*RSe456p*pow(RSp451e,2) - 1./108.*m33
         *m55*RSe465p*pow(RSp463e,2) - 1./162.*m33*m55*RSe465p*RSp461e*
         RSp463e - 1./324.*m33*m55*RSe465p*pow(RSp461e,2) - 1./108.*m33
         *m44*RSe564p*pow(RSp563e,2) - 1./162.*m33*m44*RSe564p*RSp561e*
         RSp563e - 1./324.*m33*m44*RSe564p*pow(RSp561e,2) + 1./162.*m13
         *m66*RSe456p*pow(RSp453e,2) - 1./81.*m13*m66*RSe456p*RSp451e*
         RSp453e + 1./162.*m13*m66*RSe456p*pow(RSp451e,2) + 1./162.*m13
         *m55*RSe465p*pow(RSp463e,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(mp2,L) * (  - 1./81.*m13*m55*
         RSe465p*RSp461e*RSp463e + 1./162.*m13*m55*RSe465p*pow(
         RSp461e,2) + 1./162.*m13*m44*RSe564p*pow(RSp563e,2) - 1./81.*
         m13*m44*RSe564p*RSp561e*RSp563e + 1./162.*m13*m44*RSe564p*pow(
         RSp561e,2) - 1./324.*m11*m66*RSe456p*pow(RSp453e,2) - 1./162.*
         m11*m66*RSe456p*RSp451e*RSp453e - 1./108.*m11*m66*RSe456p*pow(
         RSp451e,2) - 1./324.*m11*m55*RSe465p*pow(RSp463e,2) - 1./162.*
         m11*m55*RSe465p*RSp461e*RSp463e - 1./108.*m11*m55*RSe465p*pow(
         RSp461e,2) - 1./324.*m11*m44*RSe564p*pow(RSp563e,2) - 1./162.*
         m11*m44*RSe564p*RSp561e*RSp563e - 1./108.*m11*m44*RSe564p*pow(
         RSp561e,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(me2,mp2,L) * ( 1./81.*m66*me2*
         RSe456p*RSp453e*RSe453p - 1./81.*m66*me2*RSe456p*RSe451p*
         RSp453e - 1./81.*m66*me2*RSe456p*RSp451e*RSe453p + 1./81.*m66*
         me2*RSe456p*RSp451e*RSe451p + 1./81.*m55*me2*RSe465p*RSp463e*
         RSe463p - 1./81.*m55*me2*RSe465p*RSe461p*RSp463e - 1./81.*m55*
         me2*RSe465p*RSp461e*RSe463p + 1./81.*m55*me2*RSe465p*RSp461e*
         RSe461p + 1./81.*m44*me2*RSe564p*RSp563e*RSe563p - 1./81.*m44*
         me2*RSe564p*RSe561p*RSp563e - 1./81.*m44*me2*RSe564p*RSp561e*
         RSe563p + 1./81.*m44*me2*RSe564p*RSp561e*RSe561p - 1./54.*m33*
         m66*RSe456p*RSp453e*RSe453p - 1./162.*m33*m66*RSe456p*RSe451p*
         RSp453e - 1./162.*m33*m66*RSe456p*RSp451e*RSe453p - 1./162.*
         m33*m66*RSe456p*RSp451e*RSe451p - 1./54.*m33*m55*RSe465p*
         RSp463e*RSe463p - 1./162.*m33*m55*RSe465p*RSe461p*RSp463e - 1./
         162.*m33*m55*RSe465p*RSp461e*RSe463p - 1./162.*m33*m55*RSe465p
         *RSp461e*RSe461p - 1./54.*m33*m44*RSe564p*RSp563e*RSe563p - 1./
         162.*m33*m44*RSe564p*RSe561p*RSp563e );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(me2,mp2,L) * (  - 1./162.*m33*
         m44*RSe564p*RSp561e*RSe563p - 1./162.*m33*m44*RSe564p*RSp561e*
         RSe561p + 1./81.*m13*m66*RSe456p*RSp453e*RSe453p - 1./81.*m13*
         m66*RSe456p*RSe451p*RSp453e - 1./81.*m13*m66*RSe456p*RSp451e*
         RSe453p + 1./81.*m13*m66*RSe456p*RSp451e*RSe451p + 1./81.*m13*
         m55*RSe465p*RSp463e*RSe463p - 1./81.*m13*m55*RSe465p*RSe461p*
         RSp463e - 1./81.*m13*m55*RSe465p*RSp461e*RSe463p + 1./81.*m13*
         m55*RSe465p*RSp461e*RSe461p + 1./81.*m13*m44*RSe564p*RSp563e*
         RSe563p - 1./81.*m13*m44*RSe564p*RSe561p*RSp563e - 1./81.*m13*
         m44*RSe564p*RSp561e*RSe563p + 1./81.*m13*m44*RSe564p*RSp561e*
         RSe561p - 1./162.*m11*m66*RSe456p*RSp453e*RSe453p - 1./162.*
         m11*m66*RSe456p*RSe451p*RSp453e - 1./162.*m11*m66*RSe456p*
         RSp451e*RSe453p - 1./54.*m11*m66*RSe456p*RSp451e*RSe451p - 1./
         162.*m11*m55*RSe465p*RSp463e*RSe463p - 1./162.*m11*m55*RSe465p
         *RSe461p*RSp463e - 1./162.*m11*m55*RSe465p*RSp461e*RSe463p - 1.
         /54.*m11*m55*RSe465p*RSp461e*RSe461p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(me2,mp2,L) * (  - 1./162.*m11*
         m44*RSe564p*RSp563e*RSe563p - 1./162.*m11*m44*RSe564p*RSe561p*
         RSp563e - 1./162.*m11*m44*RSe564p*RSp561e*RSe563p - 1./54.*m11
         *m44*RSe564p*RSp561e*RSe561p );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(me2,L) * ( 1./162.*m66*me2*
         RSe456p*pow(RSe453p,2) - 1./81.*m66*me2*RSe456p*RSe451p*
         RSe453p + 1./162.*m66*me2*RSe456p*pow(RSe451p,2) + 1./162.*m55
         *me2*RSe465p*pow(RSe463p,2) - 1./81.*m55*me2*RSe465p*RSe461p*
         RSe463p + 1./162.*m55*me2*RSe465p*pow(RSe461p,2) + 1./162.*m44
         *me2*RSe564p*pow(RSe563p,2) - 1./81.*m44*me2*RSe564p*RSe561p*
         RSe563p + 1./162.*m44*me2*RSe564p*pow(RSe561p,2) - 1./108.*m33
         *m66*RSe456p*pow(RSe453p,2) - 1./162.*m33*m66*RSe456p*RSe451p*
         RSe453p - 1./324.*m33*m66*RSe456p*pow(RSe451p,2) - 1./108.*m33
         *m55*RSe465p*pow(RSe463p,2) - 1./162.*m33*m55*RSe465p*RSe461p*
         RSe463p - 1./324.*m33*m55*RSe465p*pow(RSe461p,2) - 1./108.*m33
         *m44*RSe564p*pow(RSe563p,2) - 1./162.*m33*m44*RSe564p*RSe561p*
         RSe563p - 1./324.*m33*m44*RSe564p*pow(RSe561p,2) + 1./162.*m13
         *m66*RSe456p*pow(RSe453p,2) - 1./81.*m13*m66*RSe456p*RSe451p*
         RSe453p + 1./162.*m13*m66*RSe456p*pow(RSe451p,2) + 1./162.*m13
         *m55*RSe465p*pow(RSe463p,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*BbV(me2,L) * (  - 1./81.*m13*m55*
         RSe465p*RSe461p*RSe463p + 1./162.*m13*m55*RSe465p*pow(
         RSe461p,2) + 1./162.*m13*m44*RSe564p*pow(RSe563p,2) - 1./81.*
         m13*m44*RSe564p*RSe561p*RSe563p + 1./162.*m13*m44*RSe564p*pow(
         RSe561p,2) - 1./324.*m11*m66*RSe456p*pow(RSe453p,2) - 1./162.*
         m11*m66*RSe456p*RSe451p*RSe453p - 1./108.*m11*m66*RSe456p*pow(
         RSe451p,2) - 1./324.*m11*m55*RSe465p*pow(RSe463p,2) - 1./162.*
         m11*m55*RSe465p*RSe461p*RSe463p - 1./108.*m11*m55*RSe465p*pow(
         RSe461p,2) - 1./324.*m11*m44*RSe564p*pow(RSe563p,2) - 1./162.*
         m11*m44*RSe564p*RSe561p*RSe563p - 1./108.*m11*m44*RSe564p*pow(
         RSe561p,2) );

      M13p6x23RV +=  + Ab(me2,mu2)*CbV(m11,L) * ( 1./54.*m11*m33*
         RSe45611p*RD1456ep - 1./27.*m11*m13*RSe45611p*RD1456ep + 1./54.
         *pow(m11,2)*RSe45611p*RD1456ep );

      M13p6x23RV +=  + Ab(me2,mu2)*CbV(m33,L) * ( 1./54.*pow(m33,2)*
         RSe45633p*RD3456ep - 1./27.*m13*m33*RSe45633p*RD3456ep + 1./54.
         *m11*m33*RSe45633p*RD3456ep );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m11,L) * ( 1./540.*pow(
         RD1456ep,2) + 1./270.*m33*RD1456ep*RS14563pe + 13./810.*m33*
         RD1456ep*RC1456pe + 7./810.*m13*RD1456ep*RS14563pe - 7./1620.*
         m13*RD1456ep*RC1456pe + 2./81.*m11*RD1456ep*RS14563pe - 17./
         810.*m11*RD1456ep*RC1456pe + 1./12.*m11*m33 + 1./54.*m11*m33*
         RC1456pe*RS14563pe + 1./54.*m11*m33*pow(RC1456pe,2) - 1./6.*
         m11*m13 + 1./27.*m11*m13*RC1456pe*RS14563pe - 1./27.*m11*m13*
         pow(RC1456pe,2) + 1./12.*pow(m11,2) + 1./18.*pow(m11,2)*
         RC1456pe*RS14563pe + 1./54.*pow(m11,2)*pow(RC1456pe,2) - 1./
         162.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33*RD1456ep - 1./81.*RS(
         m11,m44,m55,m66,m33,me2,mp2)*m11*RD1456ep );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m13,L) * ( 1./324.*RD1456ep*
         RD3456ep + 1./324.*pow(RD1456ep,2) - 1./162.*me2*RD1456ep*
         RSe45613p + 1./324.*me2*RD1456ep*RSe45633p + 1./324.*me2*
         RSe45611p*RD1456ep - 1./162.*mp2*RD1456ep*RSp45613e + 1./324.*
         mp2*RD1456ep*RSp45633e + 1./324.*mp2*RSp45611e*RD1456ep - 1./
         162.*m33*RD1456ep*RS34561pe + 1./108.*m33*RD1456ep*RC3456pe + 
         1./162.*m33*RD1456ep*RSe45633p + 1./162.*m33*RD1456ep*
         RSp45633e - 1./162.*m33*RD1456ep*RC1456pe - 1./162.*m33*
         RSe45611p*RD1456ep - 1./162.*m33*RSp45611e*RD1456ep + 1./27.*
         m13*RD1456ep*RS34561pe + 1./27.*m13*RD1456ep*RS14563pe + 1./27.
         *m13*RD1456ep*RSe45613p + 1./27.*m13*RD1456ep*RSp45613e - 1./
         54.*m13*RD1456ep*RC3456pe - 1./54.*m13*RD1456ep*RSe45633p - 1./
         54.*m13*RD1456ep*RSp45633e - 1./54.*m13*RD1456ep*RC1456pe - 1./
         54.*m13*RSe45611p*RD1456ep - 1./54.*m13*RSp45611e*RD1456ep - 1.
         /162.*m11*RD1456ep*RS34561pe - 1./81.*m11*RD1456ep*RS14563pe
          - 1./162.*m11*RD1456ep*RSe45613p );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m13,L) * (  - 1./162.*m11*
         RD1456ep*RSp45613e + 5./324.*m11*RD1456ep*RC3456pe + 5./324.*
         m11*RD1456ep*RSe45633p + 5./324.*m11*RD1456ep*RSp45633e - 1./
         162.*m11*RD1456ep*RC1456pe - 1./108.*m11*RSe45611p*RD1456ep - 
         1./108.*m11*RSp45611e*RD1456ep );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m14,L) * ( 7./540.*m44*RD1456ep
          - 1./108.*m34*RD1456ep + 1./120.*m33*RD1456ep - 1./108.*m33*
         m44*RS14563pe - 1./108.*m33*m44*RC1456pe - 1./108.*m33*m44*
         RS156pe - 1./60.*m14*RD1456ep + 1./54.*m14*m33*RS14563pe + 1./
         54.*m14*m33*RC1456pe - 1./54.*m14*m33*RS156pe - 1./135.*m13*
         RD1456ep - 1./54.*m13*m44*RS14563pe + 1./54.*m13*m44*RC1456pe
          + 1./54.*m13*m44*RS156pe + 1./27.*m13*m14*RS14563pe - 1./27.*
         m13*m14*RC1456pe + 1./27.*m13*m14*RS156pe + 13./1080.*m11*
         RD1456ep - 1./36.*m11*m44*RS14563pe - 1./108.*m11*m44*RC1456pe
          - 1./108.*m11*m44*RS156pe - 1./108.*m11*m33*RS14563pe - 1./
         108.*m11*m33*RC1456pe - 1./36.*m11*m33*RS156pe + 1./18.*m11*
         m14*RS14563pe + 1./54.*m11*m14*RC1456pe - 1./54.*m11*m14*
         RS156pe - 1./54.*m11*m13*RS14563pe + 1./54.*m11*m13*RC1456pe
          + 1./18.*m11*m13*RS156pe - 1./36.*pow(m11,2)*RS14563pe - 1./
         108.*pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS156pe );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m15,L) * ( 7./540.*m55*RD1456ep
          - 1./108.*m35*RD1456ep + 1./120.*m33*RD1456ep - 1./108.*m33*
         m55*RS14563pe - 1./108.*m33*m55*RC1456pe - 1./108.*m33*m55*
         RS146pe - 1./60.*m15*RD1456ep + 1./54.*m15*m33*RS14563pe + 1./
         54.*m15*m33*RC1456pe - 1./54.*m15*m33*RS146pe - 1./135.*m13*
         RD1456ep - 1./54.*m13*m55*RS14563pe + 1./54.*m13*m55*RC1456pe
          + 1./54.*m13*m55*RS146pe + 1./27.*m13*m15*RS14563pe - 1./27.*
         m13*m15*RC1456pe + 1./27.*m13*m15*RS146pe + 13./1080.*m11*
         RD1456ep - 1./36.*m11*m55*RS14563pe - 1./108.*m11*m55*RC1456pe
          - 1./108.*m11*m55*RS146pe - 1./108.*m11*m33*RS14563pe - 1./
         108.*m11*m33*RC1456pe - 1./36.*m11*m33*RS146pe + 1./18.*m11*
         m15*RS14563pe + 1./54.*m11*m15*RC1456pe - 1./54.*m11*m15*
         RS146pe - 1./54.*m11*m13*RS14563pe + 1./54.*m11*m13*RC1456pe
          + 1./18.*m11*m13*RS146pe - 1./36.*pow(m11,2)*RS14563pe - 1./
         108.*pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS146pe );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m16,L) * ( 7./540.*m66*RD1456ep
          - 1./108.*m36*RD1456ep + 1./120.*m33*RD1456ep - 1./108.*m33*
         m66*RS14563pe - 1./108.*m33*m66*RC1456pe - 1./108.*m33*m66*
         RS145pe - 1./60.*m16*RD1456ep + 1./54.*m16*m33*RS14563pe + 1./
         54.*m16*m33*RC1456pe - 1./54.*m16*m33*RS145pe - 1./135.*m13*
         RD1456ep - 1./54.*m13*m66*RS14563pe + 1./54.*m13*m66*RC1456pe
          + 1./54.*m13*m66*RS145pe + 1./27.*m13*m16*RS14563pe - 1./27.*
         m13*m16*RC1456pe + 1./27.*m13*m16*RS145pe + 13./1080.*m11*
         RD1456ep - 1./36.*m11*m66*RS14563pe - 1./108.*m11*m66*RC1456pe
          - 1./108.*m11*m66*RS145pe - 1./108.*m11*m33*RS14563pe - 1./
         108.*m11*m33*RC1456pe - 1./36.*m11*m33*RS145pe + 1./18.*m11*
         m16*RS14563pe + 1./54.*m11*m16*RC1456pe - 1./54.*m11*m16*
         RS145pe - 1./54.*m11*m13*RS14563pe + 1./54.*m11*m13*RC1456pe
          + 1./18.*m11*m13*RS145pe - 1./36.*pow(m11,2)*RS14563pe - 1./
         108.*pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS145pe );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m33,L) * ( 1./540.*RD1456ep*
         RD3456ep + 1./162.*m33*RD1456ep*RS34561pe - 19./1620.*m33*
         RD1456ep*RC3456pe + 7./810.*m13*RD1456ep*RS34561pe - 7./1620.*
         m13*RD1456ep*RC3456pe + 1./45.*m11*RD1456ep*RS34561pe - 19./
         1620.*m11*RD1456ep*RC3456pe - 1./81.*RS(m33,m44,m55,m66,m11,
         me2,mp2)*m33*RD1456ep - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2
         )*m11*RD1456ep );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m34,L) * ( 1./60.*m44*RD1456ep
          + 1./270.*m34*RD1456ep + 2./135.*m33*RD1456ep - 1./27.*m14*
         RD1456ep + 1./20.*m13*RD1456ep + 19./540.*m11*RD1456ep );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m35,L) * ( 1./60.*m55*RD1456ep
          + 1./270.*m35*RD1456ep + 2./135.*m33*RD1456ep - 1./27.*m15*
         RD1456ep + 1./20.*m13*RD1456ep + 19./540.*m11*RD1456ep );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m36,L) * ( 1./60.*m66*RD1456ep
          + 1./270.*m36*RD1456ep + 2./135.*m33*RD1456ep - 1./27.*m16*
         RD1456ep + 1./20.*m13*RD1456ep + 19./540.*m11*RD1456ep );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m44,L) * ( 1./108.*m33*m44*pow(
         RS156pe,2) - 1./324.*m33*m44*RS456pe*pow(RS156pe,2) - 1./54.*
         m13*m44*pow(RS156pe,2) + 1./162.*m13*m44*RS456pe*pow(
         RS156pe,2) + 1./108.*m11*m44*pow(RS156pe,2) - 1./324.*m11*m44*
         RS456pe*pow(RS156pe,2) );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m45,L) * ( 1./216.*m33*m55*pow(
         RS146pe,2) + 1./324.*m33*m55*RS156pe*RS146pe + 1./648.*m33*m55
         *pow(RS156pe,2) - 1./324.*m33*m45*pow(RS146pe,2) + 1./162.*m33
         *m45*RS156pe*RS146pe - 1./324.*m33*m45*pow(RS156pe,2) + 1./648.
         *m33*m44*pow(RS146pe,2) + 1./324.*m33*m44*RS156pe*RS146pe + 1./
         216.*m33*m44*pow(RS156pe,2) - 1./108.*m13*m55*pow(RS146pe,2)
          - 1./162.*m13*m55*RS156pe*RS146pe - 1./324.*m13*m55*pow(
         RS156pe,2) + 1./162.*m13*m45*pow(RS146pe,2) - 1./81.*m13*m45*
         RS156pe*RS146pe + 1./162.*m13*m45*pow(RS156pe,2) - 1./324.*m13
         *m44*pow(RS146pe,2) - 1./162.*m13*m44*RS156pe*RS146pe - 1./108.
         *m13*m44*pow(RS156pe,2) + 1./216.*m11*m55*pow(RS146pe,2) + 1./
         324.*m11*m55*RS156pe*RS146pe + 1./648.*m11*m55*pow(RS156pe,2)
          - 1./324.*m11*m45*pow(RS146pe,2) + 1./162.*m11*m45*RS156pe*
         RS146pe - 1./324.*m11*m45*pow(RS156pe,2) + 1./648.*m11*m44*
         pow(RS146pe,2) + 1./324.*m11*m44*RS156pe*RS146pe + 1./216.*m11
         *m44*pow(RS156pe,2) );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m45,L) * (  - 1./324.*m11*m33*
         pow(RS146pe,2) + 1./162.*m11*m33*RS156pe*RS146pe - 1./324.*m11
         *m33*pow(RS156pe,2) + 1./162.*m11*m13*pow(RS146pe,2) - 1./81.*
         m11*m13*RS156pe*RS146pe + 1./162.*m11*m13*pow(RS156pe,2) - 1./
         324.*pow(m11,2)*pow(RS146pe,2) + 1./162.*pow(m11,2)*RS156pe*
         RS146pe - 1./324.*pow(m11,2)*pow(RS156pe,2) );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m46,L) * ( 1./216.*m33*m66*pow(
         RS145pe,2) + 1./324.*m33*m66*RS156pe*RS145pe + 1./648.*m33*m66
         *pow(RS156pe,2) - 1./324.*m33*m46*pow(RS145pe,2) + 1./162.*m33
         *m46*RS156pe*RS145pe - 1./324.*m33*m46*pow(RS156pe,2) + 1./648.
         *m33*m44*pow(RS145pe,2) + 1./324.*m33*m44*RS156pe*RS145pe + 1./
         216.*m33*m44*pow(RS156pe,2) - 1./108.*m13*m66*pow(RS145pe,2)
          - 1./162.*m13*m66*RS156pe*RS145pe - 1./324.*m13*m66*pow(
         RS156pe,2) + 1./162.*m13*m46*pow(RS145pe,2) - 1./81.*m13*m46*
         RS156pe*RS145pe + 1./162.*m13*m46*pow(RS156pe,2) - 1./324.*m13
         *m44*pow(RS145pe,2) - 1./162.*m13*m44*RS156pe*RS145pe - 1./108.
         *m13*m44*pow(RS156pe,2) + 1./216.*m11*m66*pow(RS145pe,2) + 1./
         324.*m11*m66*RS156pe*RS145pe + 1./648.*m11*m66*pow(RS156pe,2)
          - 1./324.*m11*m46*pow(RS145pe,2) + 1./162.*m11*m46*RS156pe*
         RS145pe - 1./324.*m11*m46*pow(RS156pe,2) + 1./648.*m11*m44*
         pow(RS145pe,2) + 1./324.*m11*m44*RS156pe*RS145pe + 1./216.*m11
         *m44*pow(RS156pe,2) );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m46,L) * (  - 1./324.*m11*m33*
         pow(RS145pe,2) + 1./162.*m11*m33*RS156pe*RS145pe - 1./324.*m11
         *m33*pow(RS156pe,2) + 1./162.*m11*m13*pow(RS145pe,2) - 1./81.*
         m11*m13*RS156pe*RS145pe + 1./162.*m11*m13*pow(RS156pe,2) - 1./
         324.*pow(m11,2)*pow(RS145pe,2) + 1./162.*pow(m11,2)*RS156pe*
         RS145pe - 1./324.*pow(m11,2)*pow(RS156pe,2) );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m55,L) * ( 1./108.*m33*m55*pow(
         RS146pe,2) - 1./324.*m33*m55*RS546pe*pow(RS146pe,2) - 1./54.*
         m13*m55*pow(RS146pe,2) + 1./162.*m13*m55*RS546pe*pow(
         RS146pe,2) + 1./108.*m11*m55*pow(RS146pe,2) - 1./324.*m11*m55*
         RS546pe*pow(RS146pe,2) );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m56,L) * ( 1./216.*m33*m66*pow(
         RS145pe,2) + 1./324.*m33*m66*RS146pe*RS145pe + 1./648.*m33*m66
         *pow(RS146pe,2) - 1./324.*m33*m56*pow(RS145pe,2) + 1./162.*m33
         *m56*RS146pe*RS145pe - 1./324.*m33*m56*pow(RS146pe,2) + 1./648.
         *m33*m55*pow(RS145pe,2) + 1./324.*m33*m55*RS146pe*RS145pe + 1./
         216.*m33*m55*pow(RS146pe,2) - 1./108.*m13*m66*pow(RS145pe,2)
          - 1./162.*m13*m66*RS146pe*RS145pe - 1./324.*m13*m66*pow(
         RS146pe,2) + 1./162.*m13*m56*pow(RS145pe,2) - 1./81.*m13*m56*
         RS146pe*RS145pe + 1./162.*m13*m56*pow(RS146pe,2) - 1./324.*m13
         *m55*pow(RS145pe,2) - 1./162.*m13*m55*RS146pe*RS145pe - 1./108.
         *m13*m55*pow(RS146pe,2) + 1./216.*m11*m66*pow(RS145pe,2) + 1./
         324.*m11*m66*RS146pe*RS145pe + 1./648.*m11*m66*pow(RS146pe,2)
          - 1./324.*m11*m56*pow(RS145pe,2) + 1./162.*m11*m56*RS146pe*
         RS145pe - 1./324.*m11*m56*pow(RS146pe,2) + 1./648.*m11*m55*
         pow(RS145pe,2) + 1./324.*m11*m55*RS146pe*RS145pe + 1./216.*m11
         *m55*pow(RS146pe,2) );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m56,L) * (  - 1./324.*m11*m33*
         pow(RS145pe,2) + 1./162.*m11*m33*RS146pe*RS145pe - 1./324.*m11
         *m33*pow(RS146pe,2) + 1./162.*m11*m13*pow(RS145pe,2) - 1./81.*
         m11*m13*RS146pe*RS145pe + 1./162.*m11*m13*pow(RS146pe,2) - 1./
         324.*pow(m11,2)*pow(RS145pe,2) + 1./162.*pow(m11,2)*RS146pe*
         RS145pe - 1./324.*pow(m11,2)*pow(RS146pe,2) );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(m66,L) * ( 1./108.*m33*m66*pow(
         RS145pe,2) - 1./324.*m33*m66*RS645pe*pow(RS145pe,2) - 1./54.*
         m13*m66*pow(RS145pe,2) + 1./162.*m13*m66*RS645pe*pow(
         RS145pe,2) + 1./108.*m11*m66*pow(RS145pe,2) - 1./324.*m11*m66*
         RS645pe*pow(RS145pe,2) );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(mp2,L) * (  - 1./270.*mp2*
         RD1456ep*RSp45613e + 1./540.*mp2*RD1456ep*RSp45633e + 1./540.*
         mp2*RSp45611e*RD1456ep - 1./405.*m33*RD1456ep*RSp45613e - 11./
         810.*m33*RD1456ep*RSp45633e + 13./810.*m33*RSp45611e*RD1456ep
          - 1./324.*m33*m66*RSp456e*pow(RS145pe,2) - 1./324.*m33*m55*
         RSp465e*pow(RS146pe,2) - 1./324.*m33*m44*RSp564e*pow(
         RS156pe,2) + 7./810.*m13*RD1456ep*RSp45613e - 7./1620.*m13*
         RD1456ep*RSp45633e - 7./1620.*m13*RSp45611e*RD1456ep + 1./162.
         *m13*m66*RSp456e*pow(RS145pe,2) + 1./162.*m13*m55*RSp465e*pow(
         RS146pe,2) + 1./162.*m13*m44*RSp564e*pow(RS156pe,2) + 13./810.
         *m11*RD1456ep*RSp45613e - 19./1620.*m11*RD1456ep*RSp45633e - 
         37./1620.*m11*RSp45611e*RD1456ep - 1./324.*m11*m66*RSp456e*
         pow(RS145pe,2) - 1./324.*m11*m55*RSp465e*pow(RS146pe,2) - 1./
         324.*m11*m44*RSp564e*pow(RS156pe,2) + 1./54.*m11*m33*RSp45611e
         *RS14563pe + 1./54.*m11*m33*RSp45611e*RC1456pe + 1./27.*m11*
         m13*RSp45611e*RS14563pe );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(mp2,L) * (  - 1./27.*m11*m13*
         RSp45611e*RC1456pe + 1./18.*pow(m11,2)*RSp45611e*RS14563pe + 1.
         /54.*pow(m11,2)*RSp45611e*RC1456pe );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(me2,L) * (  - 1./270.*me2*
         RD1456ep*RSe45613p + 1./540.*me2*RD1456ep*RSe45633p + 1./540.*
         me2*RSe45611p*RD1456ep - 1./405.*m33*RD1456ep*RSe45613p - 11./
         810.*m33*RD1456ep*RSe45633p + 13./810.*m33*RSe45611p*RD1456ep
          - 1./324.*m33*m66*RSe456p*pow(RS145pe,2) - 1./324.*m33*m55*
         RSe465p*pow(RS146pe,2) - 1./324.*m33*m44*RSe564p*pow(
         RS156pe,2) + 7./810.*m13*RD1456ep*RSe45613p - 7./1620.*m13*
         RD1456ep*RSe45633p - 7./1620.*m13*RSe45611p*RD1456ep + 1./162.
         *m13*m66*RSe456p*pow(RS145pe,2) + 1./162.*m13*m55*RSe465p*pow(
         RS146pe,2) + 1./162.*m13*m44*RSe564p*pow(RS156pe,2) + 13./810.
         *m11*RD1456ep*RSe45613p - 19./1620.*m11*RD1456ep*RSe45633p - 
         37./1620.*m11*RSe45611p*RD1456ep - 1./324.*m11*m66*RSe456p*
         pow(RS145pe,2) - 1./324.*m11*m55*RSe465p*pow(RS146pe,2) - 1./
         324.*m11*m44*RSe564p*pow(RS156pe,2) + 1./54.*m11*m33*RSe45611p
         *RS14563pe + 1./54.*m11*m33*RSe45611p*RC1456pe + 1./27.*m11*
         m13*RSe45611p*RS14563pe );

      M13p6x23RV +=  + Bb(m11,mu2)*AbV(me2,L) * (  - 1./27.*m11*m13*
         RSe45611p*RC1456pe + 1./18.*pow(m11,2)*RSe45611p*RS14563pe + 1.
         /54.*pow(m11,2)*RSe45611p*RC1456pe );

      M13p6x23RV +=  + Bb(m11,mu2)*BbV(m11,m33,L) * ( 1./54.*m11*m33*
         RD1456ep*RS34561pe + 1./27.*m11*m13*RD1456ep*RS34561pe + 1./18.
         *pow(m11,2)*RD1456ep*RS34561pe );

      M13p6x23RV +=  + Bb(m11,mu2)*BbV(m11,mp2,L) * ( 1./54.*m11*m33*
         RD1456ep*RSp45613e + 1./54.*m11*m33*RSp45611e*RD1456ep + 1./27.
         *m11*m13*RD1456ep*RSp45613e - 1./27.*m11*m13*RSp45611e*
         RD1456ep + 1./18.*pow(m11,2)*RD1456ep*RSp45613e + 1./54.*pow(
         m11,2)*RSp45611e*RD1456ep );

      M13p6x23RV +=  + Bb(m11,mu2)*BbV(m11,me2,L) * ( 1./54.*m11*m33*
         RD1456ep*RSe45613p + 1./54.*m11*m33*RSe45611p*RD1456ep + 1./27.
         *m11*m13*RD1456ep*RSe45613p - 1./27.*m11*m13*RSe45611p*
         RD1456ep + 1./18.*pow(m11,2)*RD1456ep*RSe45613p + 1./54.*pow(
         m11,2)*RSe45611p*RD1456ep );

      M13p6x23RV +=  + Bb(m11,mu2)*BbV(m11,L) * ( 13./810.*m33*pow(
         RD1456ep,2) - 7./1620.*m13*pow(RD1456ep,2) - 17./810.*m11*pow(
         RD1456ep,2) + 1./27.*m11*m33*RD1456ep*RS14563pe + 1./27.*m11*
         m33*RD1456ep*RC1456pe + 2./27.*m11*m13*RD1456ep*RS14563pe - 2./
         27.*m11*m13*RD1456ep*RC1456pe + 1./9.*pow(m11,2)*RD1456ep*
         RS14563pe + 1./27.*pow(m11,2)*RD1456ep*RC1456pe );

      M13p6x23RV +=  + Bb(m11,mu2)*BbV(m14,L) * (  - 1./432.*pow(m44,2)
         *RD1456ep - 1./432.*m33*m44*RD1456ep + 1./108.*m14*m44*
         RD1456ep + 1./216.*m14*m33*RD1456ep - 1./108.*pow(m14,2)*
         RD1456ep + 1./216.*m13*m44*RD1456ep - 1./108.*m13*m14*RD1456ep
          - 1./144.*m11*m44*RD1456ep - 1./432.*m11*m33*RD1456ep + 1./72.
         *m11*m14*RD1456ep + 1./216.*m11*m13*RD1456ep - 1./216.*pow(
         m11,2)*RD1456ep );

      M13p6x23RV +=  + Bb(m11,mu2)*BbV(m15,L) * (  - 1./432.*pow(m55,2)
         *RD1456ep - 1./432.*m33*m55*RD1456ep + 1./108.*m15*m55*
         RD1456ep + 1./216.*m15*m33*RD1456ep - 1./108.*pow(m15,2)*
         RD1456ep + 1./216.*m13*m55*RD1456ep - 1./108.*m13*m15*RD1456ep
          - 1./144.*m11*m55*RD1456ep - 1./432.*m11*m33*RD1456ep + 1./72.
         *m11*m15*RD1456ep + 1./216.*m11*m13*RD1456ep - 1./216.*pow(
         m11,2)*RD1456ep );

      M13p6x23RV +=  + Bb(m11,mu2)*BbV(m16,L) * (  - 1./432.*pow(m66,2)
         *RD1456ep - 1./432.*m33*m66*RD1456ep + 1./108.*m16*m66*
         RD1456ep + 1./216.*m16*m33*RD1456ep - 1./108.*pow(m16,2)*
         RD1456ep + 1./216.*m13*m66*RD1456ep - 1./108.*m13*m16*RD1456ep
          - 1./144.*m11*m66*RD1456ep - 1./432.*m11*m33*RD1456ep + 1./72.
         *m11*m16*RD1456ep + 1./216.*m11*m13*RD1456ep - 1./216.*pow(
         m11,2)*RD1456ep );

      M13p6x23RV +=  + Bb(m11,mu2)*BbV(m33,L) * (  - 19./1620.*m33*
         RD1456ep*RD3456ep - 7./1620.*m13*RD1456ep*RD3456ep - 19./1620.
         *m11*RD1456ep*RD3456ep );

      M13p6x23RV +=  + Bb(m11,mu2)*CbV(m11,L) * ( 1./54.*m11*m33*pow(
         RD1456ep,2) - 1./27.*m11*m13*pow(RD1456ep,2) + 1./54.*pow(
         m11,2)*pow(RD1456ep,2) );

      M13p6x23RV +=  + Bb(m14,mu2)*AbV(m11,L) * ( 1./216.*m44*RD1456ep
          - 1./432.*pow(m44,2)*RC1456pe - 1./216.*pow(m44,2)*RS156pe + 
         1./216.*m33*RD1456ep - 1./432.*m33*m44*RC1456pe - 1./216.*m33*
         m44*RS156pe - 1./108.*m14*RD1456ep + 1./108.*m14*m44*RC1456pe
          + 1./216.*m14*m33*RC1456pe - 1./108.*m14*m33*RS156pe - 1./108.
         *pow(m14,2)*RC1456pe + 1./54.*pow(m14,2)*RS156pe - 1./108.*m13
         *RD1456ep + 1./216.*m13*m44*RC1456pe + 1./108.*m13*m44*RS156pe
          - 1./108.*m13*m14*RC1456pe + 1./54.*m13*m14*RS156pe + 1./108.
         *m11*RD1456ep - 1./144.*m11*m44*RC1456pe - 5./216.*m11*m44*
         RS156pe - 1./432.*m11*m33*RC1456pe - 1./72.*m11*m33*RS156pe + 
         1./72.*m11*m14*RC1456pe + 1./108.*m11*m14*RS156pe + 1./216.*
         m11*m13*RC1456pe + 1./36.*m11*m13*RS156pe - 1./216.*pow(m11,2)
         *RC1456pe - 1./36.*pow(m11,2)*RS156pe );

      M13p6x23RV +=  + Bb(m14,mu2)*AbV(m14,L) * ( 1./72.*pow(m44,2) + 1.
         /72.*m33*m44 - 1./18.*m14*m44 - 1./36.*m14*m33 + 1./18.*pow(
         m14,2) - 1./36.*m13*m44 + 1./18.*m13*m14 + 1./24.*m11*m44 + 1./
         72.*m11*m33 - 1./12.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(
         m11,2) );

      M13p6x23RV +=  + Bb(m14,mu2)*AbV(m15,L) * ( 1./144.*m44*m55 + 1./
         144.*pow(m44,2) + 1./144.*m33*m55 + 1./144.*m33*m44 - 1./72.*
         m15*m44 - 1./72.*m15*m33 - 1./72.*m14*m55 - 1./36.*m14*m44 - 1.
         /72.*m14*m33 + 1./36.*m14*m15 + 1./36.*pow(m14,2) - 1./72.*m13
         *m55 - 1./72.*m13*m44 + 1./36.*m13*m15 + 1./36.*m13*m14 + 1./
         72.*m11*m55 + 1./36.*m11*m44 + 1./72.*m11*m33 - 1./36.*m11*m15
          - 1./18.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + Bb(m14,mu2)*AbV(m16,L) * ( 1./144.*m44*m66 + 1./
         144.*pow(m44,2) + 1./144.*m33*m66 + 1./144.*m33*m44 - 1./72.*
         m16*m44 - 1./72.*m16*m33 - 1./72.*m14*m66 - 1./36.*m14*m44 - 1.
         /72.*m14*m33 + 1./36.*m14*m16 + 1./36.*pow(m14,2) - 1./72.*m13
         *m66 - 1./72.*m13*m44 + 1./36.*m13*m16 + 1./36.*m13*m14 + 1./
         72.*m11*m66 + 1./36.*m11*m44 + 1./72.*m11*m33 - 1./36.*m11*m16
          - 1./18.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + Bb(m14,mu2)*AbV(m44,L) * ( 1./144.*pow(m44,2) - 
         1./432.*pow(m44,2)*RS456pe + 1./144.*m33*m44 - 1./432.*m33*m44
         *RS456pe - 1./36.*m14*m44 + 1./108.*m14*m44*RS456pe - 1./72.*
         m14*m33 + 1./216.*m14*m33*RS456pe + 1./36.*pow(m14,2) - 1./108.
         *pow(m14,2)*RS456pe - 1./72.*m13*m44 + 1./216.*m13*m44*RS456pe
          + 1./36.*m13*m14 - 1./108.*m13*m14*RS456pe + 1./48.*m11*m44
          - 1./144.*m11*m44*RS456pe + 1./144.*m11*m33 - 1./432.*m11*m33
         *RS456pe - 1./24.*m11*m14 + 1./72.*m11*m14*RS456pe - 1./72.*
         m11*m13 + 1./216.*m11*m13*RS456pe + 1./72.*pow(m11,2) - 1./216.
         *pow(m11,2)*RS456pe );

      M13p6x23RV +=  + Bb(m14,mu2)*AbV(m45,L) * ( 1./144.*m44*m55 - 1./
         72.*m44*m45 + 1./72.*pow(m44,2) + 1./144.*m33*m55 - 1./72.*m33
         *m45 + 1./72.*m33*m44 - 1./72.*m14*m55 + 1./36.*m14*m45 - 1./
         24.*m14*m44 - 1./72.*m14*m33 + 1./36.*pow(m14,2) - 1./72.*m13*
         m55 + 1./36.*m13*m45 - 1./36.*m13*m44 + 1./36.*m13*m14 + 1./72.
         *m11*m55 - 1./36.*m11*m45 + 5./144.*m11*m44 + 1./144.*m11*m33
          - 1./24.*m11*m14 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + Bb(m14,mu2)*AbV(m46,L) * ( 1./144.*m44*m66 - 1./
         72.*m44*m46 + 1./72.*pow(m44,2) + 1./144.*m33*m66 - 1./72.*m33
         *m46 + 1./72.*m33*m44 - 1./72.*m14*m66 + 1./36.*m14*m46 - 1./
         24.*m14*m44 - 1./72.*m14*m33 + 1./36.*pow(m14,2) - 1./72.*m13*
         m66 + 1./36.*m13*m46 - 1./36.*m13*m44 + 1./36.*m13*m14 + 1./72.
         *m11*m66 - 1./36.*m11*m46 + 5./144.*m11*m44 + 1./144.*m11*m33
          - 1./24.*m11*m14 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + Bb(m14,mu2)*AbV(mp2,L) * ( 1./216.*m44*mp2*
         RSp45611e - 1./108.*m44*mp2*RSp561e + 1./216.*m44*mp2*RSp564e
          - 1./432.*pow(m44,2)*RSp45611e - 1./216.*pow(m44,2)*RSp561e
          - 1./144.*pow(m44,2)*RSp564e + 1./216.*m33*mp2*RSp45611e - 1./
         108.*m33*mp2*RSp561e + 1./216.*m33*mp2*RSp564e - 1./432.*m33*
         m44*RSp45611e - 1./216.*m33*m44*RSp561e - 1./144.*m33*m44*
         RSp564e - 1./108.*m14*mp2*RSp45611e + 1./54.*m14*mp2*RSp561e
          - 1./108.*m14*mp2*RSp564e + 1./108.*m14*m44*RSp45611e + 1./54.
         *m14*m44*RSp564e + 1./216.*m14*m33*RSp45611e - 1./108.*m14*m33
         *RSp561e + 1./216.*m14*m33*RSp564e - 1./108.*pow(m14,2)*
         RSp45611e + 1./54.*pow(m14,2)*RSp561e - 1./108.*pow(m14,2)*
         RSp564e - 1./108.*m13*mp2*RSp45611e + 1./54.*m13*mp2*RSp561e
          - 1./108.*m13*mp2*RSp564e + 1./216.*m13*m44*RSp45611e + 1./
         108.*m13*m44*RSp561e + 1./72.*m13*m44*RSp564e - 1./108.*m13*
         m14*RSp45611e + 1./54.*m13*m14*RSp561e - 1./108.*m13*m14*
         RSp564e );

      M13p6x23RV +=  + Bb(m14,mu2)*AbV(mp2,L) * ( 1./108.*m11*mp2*
         RSp45611e - 1./54.*m11*mp2*RSp561e + 1./108.*m11*mp2*RSp564e
          - 5./432.*m11*m44*RSp45611e - 1./72.*m11*m44*RSp561e - 7./432.
         *m11*m44*RSp564e - 1./144.*m11*m33*RSp45611e - 1./216.*m11*m33
         *RSp561e - 1./432.*m11*m33*RSp564e + 5./216.*m11*m14*RSp45611e
          - 1./108.*m11*m14*RSp561e + 1./72.*m11*m14*RSp564e + 1./72.*
         m11*m13*RSp45611e + 1./108.*m11*m13*RSp561e + 1./216.*m11*m13*
         RSp564e - 1./72.*pow(m11,2)*RSp45611e - 1./108.*pow(m11,2)*
         RSp561e - 1./216.*pow(m11,2)*RSp564e );

      M13p6x23RV +=  + Bb(m14,mu2)*AbV(me2,L) * ( 1./216.*m44*me2*
         RSe45611p - 1./108.*m44*me2*RSe561p + 1./216.*m44*me2*RSe564p
          - 1./432.*pow(m44,2)*RSe45611p - 1./216.*pow(m44,2)*RSe561p
          - 1./144.*pow(m44,2)*RSe564p + 1./216.*m33*me2*RSe45611p - 1./
         108.*m33*me2*RSe561p + 1./216.*m33*me2*RSe564p - 1./432.*m33*
         m44*RSe45611p - 1./216.*m33*m44*RSe561p - 1./144.*m33*m44*
         RSe564p - 1./108.*m14*me2*RSe45611p + 1./54.*m14*me2*RSe561p
          - 1./108.*m14*me2*RSe564p + 1./108.*m14*m44*RSe45611p + 1./54.
         *m14*m44*RSe564p + 1./216.*m14*m33*RSe45611p - 1./108.*m14*m33
         *RSe561p + 1./216.*m14*m33*RSe564p - 1./108.*pow(m14,2)*
         RSe45611p + 1./54.*pow(m14,2)*RSe561p - 1./108.*pow(m14,2)*
         RSe564p - 1./108.*m13*me2*RSe45611p + 1./54.*m13*me2*RSe561p
          - 1./108.*m13*me2*RSe564p + 1./216.*m13*m44*RSe45611p + 1./
         108.*m13*m44*RSe561p + 1./72.*m13*m44*RSe564p - 1./108.*m13*
         m14*RSe45611p + 1./54.*m13*m14*RSe561p - 1./108.*m13*m14*
         RSe564p );

      M13p6x23RV +=  + Bb(m14,mu2)*AbV(me2,L) * ( 1./108.*m11*me2*
         RSe45611p - 1./54.*m11*me2*RSe561p + 1./108.*m11*me2*RSe564p
          - 5./432.*m11*m44*RSe45611p - 1./72.*m11*m44*RSe561p - 7./432.
         *m11*m44*RSe564p - 1./144.*m11*m33*RSe45611p - 1./216.*m11*m33
         *RSe561p - 1./432.*m11*m33*RSe564p + 5./216.*m11*m14*RSe45611p
          - 1./108.*m11*m14*RSe561p + 1./72.*m11*m14*RSe564p + 1./72.*
         m11*m13*RSe45611p + 1./108.*m11*m13*RSe561p + 1./216.*m11*m13*
         RSe564p - 1./72.*pow(m11,2)*RSe45611p - 1./108.*pow(m11,2)*
         RSe561p - 1./216.*pow(m11,2)*RSe564p );

      M13p6x23RV +=  + Bb(m14,mu2)*BbV(m11,L) * (  - 1./432.*pow(m44,2)
         *RD1456ep - 1./432.*m33*m44*RD1456ep + 1./108.*m14*m44*
         RD1456ep + 1./216.*m14*m33*RD1456ep - 1./108.*pow(m14,2)*
         RD1456ep + 1./216.*m13*m44*RD1456ep - 1./108.*m13*m14*RD1456ep
          - 1./144.*m11*m44*RD1456ep - 1./432.*m11*m33*RD1456ep + 1./72.
         *m11*m14*RD1456ep + 1./216.*m11*m13*RD1456ep - 1./216.*pow(
         m11,2)*RD1456ep );

      M13p6x23RV +=  + Bb(m15,mu2)*AbV(m11,L) * ( 1./216.*m55*RD1456ep
          - 1./432.*pow(m55,2)*RC1456pe - 1./216.*pow(m55,2)*RS146pe + 
         1./216.*m33*RD1456ep - 1./432.*m33*m55*RC1456pe - 1./216.*m33*
         m55*RS146pe - 1./108.*m15*RD1456ep + 1./108.*m15*m55*RC1456pe
          + 1./216.*m15*m33*RC1456pe - 1./108.*m15*m33*RS146pe - 1./108.
         *pow(m15,2)*RC1456pe + 1./54.*pow(m15,2)*RS146pe - 1./108.*m13
         *RD1456ep + 1./216.*m13*m55*RC1456pe + 1./108.*m13*m55*RS146pe
          - 1./108.*m13*m15*RC1456pe + 1./54.*m13*m15*RS146pe + 1./108.
         *m11*RD1456ep - 1./144.*m11*m55*RC1456pe - 5./216.*m11*m55*
         RS146pe - 1./432.*m11*m33*RC1456pe - 1./72.*m11*m33*RS146pe + 
         1./72.*m11*m15*RC1456pe + 1./108.*m11*m15*RS146pe + 1./216.*
         m11*m13*RC1456pe + 1./36.*m11*m13*RS146pe - 1./216.*pow(m11,2)
         *RC1456pe - 1./36.*pow(m11,2)*RS146pe );

      M13p6x23RV +=  + Bb(m15,mu2)*AbV(m14,L) * ( 1./144.*pow(m55,2) + 
         1./144.*m44*m55 + 1./144.*m33*m55 + 1./144.*m33*m44 - 1./36.*
         m15*m55 - 1./72.*m15*m44 - 1./72.*m15*m33 + 1./36.*pow(m15,2)
          - 1./72.*m14*m55 - 1./72.*m14*m33 + 1./36.*m14*m15 - 1./72.*
         m13*m55 - 1./72.*m13*m44 + 1./36.*m13*m15 + 1./36.*m13*m14 + 1.
         /36.*m11*m55 + 1./72.*m11*m44 + 1./72.*m11*m33 - 1./18.*m11*
         m15 - 1./36.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + Bb(m15,mu2)*AbV(m15,L) * ( 1./72.*pow(m55,2) + 1.
         /72.*m33*m55 - 1./18.*m15*m55 - 1./36.*m15*m33 + 1./18.*pow(
         m15,2) - 1./36.*m13*m55 + 1./18.*m13*m15 + 1./24.*m11*m55 + 1./
         72.*m11*m33 - 1./12.*m11*m15 - 1./36.*m11*m13 + 1./36.*pow(
         m11,2) );

      M13p6x23RV +=  + Bb(m15,mu2)*AbV(m16,L) * ( 1./144.*m55*m66 + 1./
         144.*pow(m55,2) + 1./144.*m33*m66 + 1./144.*m33*m55 - 1./72.*
         m16*m55 - 1./72.*m16*m33 - 1./72.*m15*m66 - 1./36.*m15*m55 - 1.
         /72.*m15*m33 + 1./36.*m15*m16 + 1./36.*pow(m15,2) - 1./72.*m13
         *m66 - 1./72.*m13*m55 + 1./36.*m13*m16 + 1./36.*m13*m15 + 1./
         72.*m11*m66 + 1./36.*m11*m55 + 1./72.*m11*m33 - 1./36.*m11*m16
          - 1./18.*m11*m15 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + Bb(m15,mu2)*AbV(m45,L) * ( 1./72.*pow(m55,2) - 1.
         /72.*m45*m55 + 1./144.*m44*m55 + 1./72.*m33*m55 - 1./72.*m33*
         m45 + 1./144.*m33*m44 - 1./24.*m15*m55 + 1./36.*m15*m45 - 1./
         72.*m15*m44 - 1./72.*m15*m33 + 1./36.*pow(m15,2) - 1./36.*m13*
         m55 + 1./36.*m13*m45 - 1./72.*m13*m44 + 1./36.*m13*m15 + 5./
         144.*m11*m55 - 1./36.*m11*m45 + 1./72.*m11*m44 + 1./144.*m11*
         m33 - 1./24.*m11*m15 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + Bb(m15,mu2)*AbV(m55,L) * ( 1./144.*pow(m55,2) - 
         1./432.*pow(m55,2)*RS546pe + 1./144.*m33*m55 - 1./432.*m33*m55
         *RS546pe - 1./36.*m15*m55 + 1./108.*m15*m55*RS546pe - 1./72.*
         m15*m33 + 1./216.*m15*m33*RS546pe + 1./36.*pow(m15,2) - 1./108.
         *pow(m15,2)*RS546pe - 1./72.*m13*m55 + 1./216.*m13*m55*RS546pe
          + 1./36.*m13*m15 - 1./108.*m13*m15*RS546pe + 1./48.*m11*m55
          - 1./144.*m11*m55*RS546pe + 1./144.*m11*m33 - 1./432.*m11*m33
         *RS546pe - 1./24.*m11*m15 + 1./72.*m11*m15*RS546pe - 1./72.*
         m11*m13 + 1./216.*m11*m13*RS546pe + 1./72.*pow(m11,2) - 1./216.
         *pow(m11,2)*RS546pe );

      M13p6x23RV +=  + Bb(m15,mu2)*AbV(m56,L) * ( 1./144.*m55*m66 - 1./
         72.*m55*m56 + 1./72.*pow(m55,2) + 1./144.*m33*m66 - 1./72.*m33
         *m56 + 1./72.*m33*m55 - 1./72.*m15*m66 + 1./36.*m15*m56 - 1./
         24.*m15*m55 - 1./72.*m15*m33 + 1./36.*pow(m15,2) - 1./72.*m13*
         m66 + 1./36.*m13*m56 - 1./36.*m13*m55 + 1./36.*m13*m15 + 1./72.
         *m11*m66 - 1./36.*m11*m56 + 5./144.*m11*m55 + 1./144.*m11*m33
          - 1./24.*m11*m15 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + Bb(m15,mu2)*AbV(mp2,L) * ( 1./216.*m55*mp2*
         RSp45611e - 1./108.*m55*mp2*RSp461e + 1./216.*m55*mp2*RSp465e
          - 1./432.*pow(m55,2)*RSp45611e - 1./216.*pow(m55,2)*RSp461e
          - 1./144.*pow(m55,2)*RSp465e + 1./216.*m33*mp2*RSp45611e - 1./
         108.*m33*mp2*RSp461e + 1./216.*m33*mp2*RSp465e - 1./432.*m33*
         m55*RSp45611e - 1./216.*m33*m55*RSp461e - 1./144.*m33*m55*
         RSp465e - 1./108.*m15*mp2*RSp45611e + 1./54.*m15*mp2*RSp461e
          - 1./108.*m15*mp2*RSp465e + 1./108.*m15*m55*RSp45611e + 1./54.
         *m15*m55*RSp465e + 1./216.*m15*m33*RSp45611e - 1./108.*m15*m33
         *RSp461e + 1./216.*m15*m33*RSp465e - 1./108.*pow(m15,2)*
         RSp45611e + 1./54.*pow(m15,2)*RSp461e - 1./108.*pow(m15,2)*
         RSp465e - 1./108.*m13*mp2*RSp45611e + 1./54.*m13*mp2*RSp461e
          - 1./108.*m13*mp2*RSp465e + 1./216.*m13*m55*RSp45611e + 1./
         108.*m13*m55*RSp461e + 1./72.*m13*m55*RSp465e - 1./108.*m13*
         m15*RSp45611e + 1./54.*m13*m15*RSp461e - 1./108.*m13*m15*
         RSp465e );

      M13p6x23RV +=  + Bb(m15,mu2)*AbV(mp2,L) * ( 1./108.*m11*mp2*
         RSp45611e - 1./54.*m11*mp2*RSp461e + 1./108.*m11*mp2*RSp465e
          - 5./432.*m11*m55*RSp45611e - 1./72.*m11*m55*RSp461e - 7./432.
         *m11*m55*RSp465e - 1./144.*m11*m33*RSp45611e - 1./216.*m11*m33
         *RSp461e - 1./432.*m11*m33*RSp465e + 5./216.*m11*m15*RSp45611e
          - 1./108.*m11*m15*RSp461e + 1./72.*m11*m15*RSp465e + 1./72.*
         m11*m13*RSp45611e + 1./108.*m11*m13*RSp461e + 1./216.*m11*m13*
         RSp465e - 1./72.*pow(m11,2)*RSp45611e - 1./108.*pow(m11,2)*
         RSp461e - 1./216.*pow(m11,2)*RSp465e );

      M13p6x23RV +=  + Bb(m15,mu2)*AbV(me2,L) * ( 1./216.*m55*me2*
         RSe45611p - 1./108.*m55*me2*RSe461p + 1./216.*m55*me2*RSe465p
          - 1./432.*pow(m55,2)*RSe45611p - 1./216.*pow(m55,2)*RSe461p
          - 1./144.*pow(m55,2)*RSe465p + 1./216.*m33*me2*RSe45611p - 1./
         108.*m33*me2*RSe461p + 1./216.*m33*me2*RSe465p - 1./432.*m33*
         m55*RSe45611p - 1./216.*m33*m55*RSe461p - 1./144.*m33*m55*
         RSe465p - 1./108.*m15*me2*RSe45611p + 1./54.*m15*me2*RSe461p
          - 1./108.*m15*me2*RSe465p + 1./108.*m15*m55*RSe45611p + 1./54.
         *m15*m55*RSe465p + 1./216.*m15*m33*RSe45611p - 1./108.*m15*m33
         *RSe461p + 1./216.*m15*m33*RSe465p - 1./108.*pow(m15,2)*
         RSe45611p + 1./54.*pow(m15,2)*RSe461p - 1./108.*pow(m15,2)*
         RSe465p - 1./108.*m13*me2*RSe45611p + 1./54.*m13*me2*RSe461p
          - 1./108.*m13*me2*RSe465p + 1./216.*m13*m55*RSe45611p + 1./
         108.*m13*m55*RSe461p + 1./72.*m13*m55*RSe465p - 1./108.*m13*
         m15*RSe45611p + 1./54.*m13*m15*RSe461p - 1./108.*m13*m15*
         RSe465p );

      M13p6x23RV +=  + Bb(m15,mu2)*AbV(me2,L) * ( 1./108.*m11*me2*
         RSe45611p - 1./54.*m11*me2*RSe461p + 1./108.*m11*me2*RSe465p
          - 5./432.*m11*m55*RSe45611p - 1./72.*m11*m55*RSe461p - 7./432.
         *m11*m55*RSe465p - 1./144.*m11*m33*RSe45611p - 1./216.*m11*m33
         *RSe461p - 1./432.*m11*m33*RSe465p + 5./216.*m11*m15*RSe45611p
          - 1./108.*m11*m15*RSe461p + 1./72.*m11*m15*RSe465p + 1./72.*
         m11*m13*RSe45611p + 1./108.*m11*m13*RSe461p + 1./216.*m11*m13*
         RSe465p - 1./72.*pow(m11,2)*RSe45611p - 1./108.*pow(m11,2)*
         RSe461p - 1./216.*pow(m11,2)*RSe465p );

      M13p6x23RV +=  + Bb(m15,mu2)*BbV(m11,L) * (  - 1./432.*pow(m55,2)
         *RD1456ep - 1./432.*m33*m55*RD1456ep + 1./108.*m15*m55*
         RD1456ep + 1./216.*m15*m33*RD1456ep - 1./108.*pow(m15,2)*
         RD1456ep + 1./216.*m13*m55*RD1456ep - 1./108.*m13*m15*RD1456ep
          - 1./144.*m11*m55*RD1456ep - 1./432.*m11*m33*RD1456ep + 1./72.
         *m11*m15*RD1456ep + 1./216.*m11*m13*RD1456ep - 1./216.*pow(
         m11,2)*RD1456ep );

      M13p6x23RV +=  + Bb(m16,mu2)*AbV(m11,L) * ( 1./216.*m66*RD1456ep
          - 1./432.*pow(m66,2)*RC1456pe - 1./216.*pow(m66,2)*RS145pe + 
         1./216.*m33*RD1456ep - 1./432.*m33*m66*RC1456pe - 1./216.*m33*
         m66*RS145pe - 1./108.*m16*RD1456ep + 1./108.*m16*m66*RC1456pe
          + 1./216.*m16*m33*RC1456pe - 1./108.*m16*m33*RS145pe - 1./108.
         *pow(m16,2)*RC1456pe + 1./54.*pow(m16,2)*RS145pe - 1./108.*m13
         *RD1456ep + 1./216.*m13*m66*RC1456pe + 1./108.*m13*m66*RS145pe
          - 1./108.*m13*m16*RC1456pe + 1./54.*m13*m16*RS145pe + 1./108.
         *m11*RD1456ep - 1./144.*m11*m66*RC1456pe - 5./216.*m11*m66*
         RS145pe - 1./432.*m11*m33*RC1456pe - 1./72.*m11*m33*RS145pe + 
         1./72.*m11*m16*RC1456pe + 1./108.*m11*m16*RS145pe + 1./216.*
         m11*m13*RC1456pe + 1./36.*m11*m13*RS145pe - 1./216.*pow(m11,2)
         *RC1456pe - 1./36.*pow(m11,2)*RS145pe );

      M13p6x23RV +=  + Bb(m16,mu2)*AbV(m14,L) * ( 1./144.*pow(m66,2) + 
         1./144.*m44*m66 + 1./144.*m33*m66 + 1./144.*m33*m44 - 1./36.*
         m16*m66 - 1./72.*m16*m44 - 1./72.*m16*m33 + 1./36.*pow(m16,2)
          - 1./72.*m14*m66 - 1./72.*m14*m33 + 1./36.*m14*m16 - 1./72.*
         m13*m66 - 1./72.*m13*m44 + 1./36.*m13*m16 + 1./36.*m13*m14 + 1.
         /36.*m11*m66 + 1./72.*m11*m44 + 1./72.*m11*m33 - 1./18.*m11*
         m16 - 1./36.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + Bb(m16,mu2)*AbV(m15,L) * ( 1./144.*pow(m66,2) + 
         1./144.*m55*m66 + 1./144.*m33*m66 + 1./144.*m33*m55 - 1./36.*
         m16*m66 - 1./72.*m16*m55 - 1./72.*m16*m33 + 1./36.*pow(m16,2)
          - 1./72.*m15*m66 - 1./72.*m15*m33 + 1./36.*m15*m16 - 1./72.*
         m13*m66 - 1./72.*m13*m55 + 1./36.*m13*m16 + 1./36.*m13*m15 + 1.
         /36.*m11*m66 + 1./72.*m11*m55 + 1./72.*m11*m33 - 1./18.*m11*
         m16 - 1./36.*m11*m15 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + Bb(m16,mu2)*AbV(m16,L) * ( 1./72.*pow(m66,2) + 1.
         /72.*m33*m66 - 1./18.*m16*m66 - 1./36.*m16*m33 + 1./18.*pow(
         m16,2) - 1./36.*m13*m66 + 1./18.*m13*m16 + 1./24.*m11*m66 + 1./
         72.*m11*m33 - 1./12.*m11*m16 - 1./36.*m11*m13 + 1./36.*pow(
         m11,2) );

      M13p6x23RV +=  + Bb(m16,mu2)*AbV(m46,L) * ( 1./72.*pow(m66,2) - 1.
         /72.*m46*m66 + 1./144.*m44*m66 + 1./72.*m33*m66 - 1./72.*m33*
         m46 + 1./144.*m33*m44 - 1./24.*m16*m66 + 1./36.*m16*m46 - 1./
         72.*m16*m44 - 1./72.*m16*m33 + 1./36.*pow(m16,2) - 1./36.*m13*
         m66 + 1./36.*m13*m46 - 1./72.*m13*m44 + 1./36.*m13*m16 + 5./
         144.*m11*m66 - 1./36.*m11*m46 + 1./72.*m11*m44 + 1./144.*m11*
         m33 - 1./24.*m11*m16 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + Bb(m16,mu2)*AbV(m56,L) * ( 1./72.*pow(m66,2) - 1.
         /72.*m56*m66 + 1./144.*m55*m66 + 1./72.*m33*m66 - 1./72.*m33*
         m56 + 1./144.*m33*m55 - 1./24.*m16*m66 + 1./36.*m16*m56 - 1./
         72.*m16*m55 - 1./72.*m16*m33 + 1./36.*pow(m16,2) - 1./36.*m13*
         m66 + 1./36.*m13*m56 - 1./72.*m13*m55 + 1./36.*m13*m16 + 5./
         144.*m11*m66 - 1./36.*m11*m56 + 1./72.*m11*m55 + 1./144.*m11*
         m33 - 1./24.*m11*m16 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + Bb(m16,mu2)*AbV(m66,L) * ( 1./144.*pow(m66,2) - 
         1./432.*pow(m66,2)*RS645pe + 1./144.*m33*m66 - 1./432.*m33*m66
         *RS645pe - 1./36.*m16*m66 + 1./108.*m16*m66*RS645pe - 1./72.*
         m16*m33 + 1./216.*m16*m33*RS645pe + 1./36.*pow(m16,2) - 1./108.
         *pow(m16,2)*RS645pe - 1./72.*m13*m66 + 1./216.*m13*m66*RS645pe
          + 1./36.*m13*m16 - 1./108.*m13*m16*RS645pe + 1./48.*m11*m66
          - 1./144.*m11*m66*RS645pe + 1./144.*m11*m33 - 1./432.*m11*m33
         *RS645pe - 1./24.*m11*m16 + 1./72.*m11*m16*RS645pe - 1./72.*
         m11*m13 + 1./216.*m11*m13*RS645pe + 1./72.*pow(m11,2) - 1./216.
         *pow(m11,2)*RS645pe );

      M13p6x23RV +=  + Bb(m16,mu2)*AbV(mp2,L) * ( 1./216.*m66*mp2*
         RSp45611e - 1./108.*m66*mp2*RSp451e + 1./216.*m66*mp2*RSp456e
          - 1./432.*pow(m66,2)*RSp45611e - 1./216.*pow(m66,2)*RSp451e
          - 1./144.*pow(m66,2)*RSp456e + 1./216.*m33*mp2*RSp45611e - 1./
         108.*m33*mp2*RSp451e + 1./216.*m33*mp2*RSp456e - 1./432.*m33*
         m66*RSp45611e - 1./216.*m33*m66*RSp451e - 1./144.*m33*m66*
         RSp456e - 1./108.*m16*mp2*RSp45611e + 1./54.*m16*mp2*RSp451e
          - 1./108.*m16*mp2*RSp456e + 1./108.*m16*m66*RSp45611e + 1./54.
         *m16*m66*RSp456e + 1./216.*m16*m33*RSp45611e - 1./108.*m16*m33
         *RSp451e + 1./216.*m16*m33*RSp456e - 1./108.*pow(m16,2)*
         RSp45611e + 1./54.*pow(m16,2)*RSp451e - 1./108.*pow(m16,2)*
         RSp456e - 1./108.*m13*mp2*RSp45611e + 1./54.*m13*mp2*RSp451e
          - 1./108.*m13*mp2*RSp456e + 1./216.*m13*m66*RSp45611e + 1./
         108.*m13*m66*RSp451e + 1./72.*m13*m66*RSp456e - 1./108.*m13*
         m16*RSp45611e + 1./54.*m13*m16*RSp451e - 1./108.*m13*m16*
         RSp456e );

      M13p6x23RV +=  + Bb(m16,mu2)*AbV(mp2,L) * ( 1./108.*m11*mp2*
         RSp45611e - 1./54.*m11*mp2*RSp451e + 1./108.*m11*mp2*RSp456e
          - 5./432.*m11*m66*RSp45611e - 1./72.*m11*m66*RSp451e - 7./432.
         *m11*m66*RSp456e - 1./144.*m11*m33*RSp45611e - 1./216.*m11*m33
         *RSp451e - 1./432.*m11*m33*RSp456e + 5./216.*m11*m16*RSp45611e
          - 1./108.*m11*m16*RSp451e + 1./72.*m11*m16*RSp456e + 1./72.*
         m11*m13*RSp45611e + 1./108.*m11*m13*RSp451e + 1./216.*m11*m13*
         RSp456e - 1./72.*pow(m11,2)*RSp45611e - 1./108.*pow(m11,2)*
         RSp451e - 1./216.*pow(m11,2)*RSp456e );

      M13p6x23RV +=  + Bb(m16,mu2)*AbV(me2,L) * ( 1./216.*m66*me2*
         RSe45611p - 1./108.*m66*me2*RSe451p + 1./216.*m66*me2*RSe456p
          - 1./432.*pow(m66,2)*RSe45611p - 1./216.*pow(m66,2)*RSe451p
          - 1./144.*pow(m66,2)*RSe456p + 1./216.*m33*me2*RSe45611p - 1./
         108.*m33*me2*RSe451p + 1./216.*m33*me2*RSe456p - 1./432.*m33*
         m66*RSe45611p - 1./216.*m33*m66*RSe451p - 1./144.*m33*m66*
         RSe456p - 1./108.*m16*me2*RSe45611p + 1./54.*m16*me2*RSe451p
          - 1./108.*m16*me2*RSe456p + 1./108.*m16*m66*RSe45611p + 1./54.
         *m16*m66*RSe456p + 1./216.*m16*m33*RSe45611p - 1./108.*m16*m33
         *RSe451p + 1./216.*m16*m33*RSe456p - 1./108.*pow(m16,2)*
         RSe45611p + 1./54.*pow(m16,2)*RSe451p - 1./108.*pow(m16,2)*
         RSe456p - 1./108.*m13*me2*RSe45611p + 1./54.*m13*me2*RSe451p
          - 1./108.*m13*me2*RSe456p + 1./216.*m13*m66*RSe45611p + 1./
         108.*m13*m66*RSe451p + 1./72.*m13*m66*RSe456p - 1./108.*m13*
         m16*RSe45611p + 1./54.*m13*m16*RSe451p - 1./108.*m13*m16*
         RSe456p );

      M13p6x23RV +=  + Bb(m16,mu2)*AbV(me2,L) * ( 1./108.*m11*me2*
         RSe45611p - 1./54.*m11*me2*RSe451p + 1./108.*m11*me2*RSe456p
          - 5./432.*m11*m66*RSe45611p - 1./72.*m11*m66*RSe451p - 7./432.
         *m11*m66*RSe456p - 1./144.*m11*m33*RSe45611p - 1./216.*m11*m33
         *RSe451p - 1./432.*m11*m33*RSe456p + 5./216.*m11*m16*RSe45611p
          - 1./108.*m11*m16*RSe451p + 1./72.*m11*m16*RSe456p + 1./72.*
         m11*m13*RSe45611p + 1./108.*m11*m13*RSe451p + 1./216.*m11*m13*
         RSe456p - 1./72.*pow(m11,2)*RSe45611p - 1./108.*pow(m11,2)*
         RSe451p - 1./216.*pow(m11,2)*RSe456p );

      M13p6x23RV +=  + Bb(m16,mu2)*BbV(m11,L) * (  - 1./432.*pow(m66,2)
         *RD1456ep - 1./432.*m33*m66*RD1456ep + 1./108.*m16*m66*
         RD1456ep + 1./216.*m16*m33*RD1456ep - 1./108.*pow(m16,2)*
         RD1456ep + 1./216.*m13*m66*RD1456ep - 1./108.*m13*m16*RD1456ep
          - 1./144.*m11*m66*RD1456ep - 1./432.*m11*m33*RD1456ep + 1./72.
         *m11*m16*RD1456ep + 1./216.*m11*m13*RD1456ep - 1./216.*pow(
         m11,2)*RD1456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m11,L) * ( 1./540.*RD1456ep*
         RD3456ep - 2./135.*m33*RD3456ep*RS14563pe - 19./1620.*m33*
         RC1456pe*RD3456ep + 7./810.*m13*RD3456ep*RS14563pe - 7./1620.*
         m13*RC1456pe*RD3456ep + 1./162.*m11*RD3456ep*RS14563pe - 19./
         1620.*m11*RC1456pe*RD3456ep - 1./162.*RS(m11,m44,m55,m66,m33,
         me2,mp2)*m33*RD3456ep - 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)
         *m11*RD3456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m13,L) * ( 1./324.*pow(
         RD3456ep,2) + 1./324.*RD1456ep*RD3456ep - 1./162.*me2*RD3456ep
         *RSe45613p + 1./324.*me2*RSe45633p*RD3456ep + 1./324.*me2*
         RSe45611p*RD3456ep - 1./162.*mp2*RD3456ep*RSp45613e + 1./324.*
         mp2*RSp45633e*RD3456ep + 1./324.*mp2*RSp45611e*RD3456ep - 1./
         81.*m33*RD3456ep*RS34561pe - 1./162.*m33*RD3456ep*RS14563pe - 
         1./162.*m33*RD3456ep*RSe45613p - 1./162.*m33*RD3456ep*
         RSp45613e - 1./162.*m33*RD3456ep*RC3456pe - 1./108.*m33*
         RSe45633p*RD3456ep - 1./108.*m33*RSp45633e*RD3456ep + 5./324.*
         m33*RC1456pe*RD3456ep + 5./324.*m33*RSe45611p*RD3456ep + 5./
         324.*m33*RSp45611e*RD3456ep + 1./27.*m13*RD3456ep*RS34561pe + 
         1./27.*m13*RD3456ep*RS14563pe + 1./27.*m13*RD3456ep*RSe45613p
          + 1./27.*m13*RD3456ep*RSp45613e - 1./54.*m13*RD3456ep*
         RC3456pe - 1./54.*m13*RSe45633p*RD3456ep - 1./54.*m13*
         RSp45633e*RD3456ep - 1./54.*m13*RC1456pe*RD3456ep - 1./54.*m13
         *RSe45611p*RD3456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m13,L) * (  - 1./54.*m13*
         RSp45611e*RD3456ep - 1./162.*m11*RD3456ep*RS14563pe - 1./162.*
         m11*RD3456ep*RC3456pe - 1./162.*m11*RSe45633p*RD3456ep - 1./
         162.*m11*RSp45633e*RD3456ep + 1./108.*m11*RC1456pe*RD3456ep + 
         1./162.*m11*RSe45611p*RD3456ep + 1./162.*m11*RSp45611e*
         RD3456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m14,L) * ( 1./60.*m44*RD3456ep
          - 1./27.*m34*RD3456ep + 19./540.*m33*RD3456ep + 1./270.*m14*
         RD3456ep + 1./20.*m13*RD3456ep + 2./135.*m11*RD3456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m15,L) * ( 1./60.*m55*RD3456ep
          - 1./27.*m35*RD3456ep + 19./540.*m33*RD3456ep + 1./270.*m15*
         RD3456ep + 1./20.*m13*RD3456ep + 2./135.*m11*RD3456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m16,L) * ( 1./60.*m66*RD3456ep
          - 1./27.*m36*RD3456ep + 19./540.*m33*RD3456ep + 1./270.*m16*
         RD3456ep + 1./20.*m13*RD3456ep + 2./135.*m11*RD3456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m33,L) * ( 1./540.*pow(
         RD3456ep,2) + 2./81.*m33*RD3456ep*RS34561pe + 1./27.*m33*
         RD3456ep*RS14563pe - 17./810.*m33*RD3456ep*RC3456pe + 1./12.*
         pow(m33,2) + 1./18.*pow(m33,2)*RC3456pe*RS34561pe + 1./54.*
         pow(m33,2)*pow(RC3456pe,2) + 7./810.*m13*RD3456ep*RS34561pe - 
         7./1620.*m13*RD3456ep*RC3456pe - 1./6.*m13*m33 + 1./27.*m13*
         m33*RC3456pe*RS34561pe - 1./27.*m13*m33*pow(RC3456pe,2) + 1./
         270.*m11*RD3456ep*RS34561pe + 13./810.*m11*RD3456ep*RC3456pe
          + 1./12.*m11*m33 + 1./54.*m11*m33*RC3456pe*RS34561pe + 1./54.
         *m11*m33*pow(RC3456pe,2) - 1./81.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m33*RD3456ep - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*
         m11*RD3456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m34,L) * ( 7./540.*m44*RD3456ep
          - 1./60.*m34*RD3456ep + 13./1080.*m33*RD3456ep - 1./36.*m33*
         m44*RS34561pe - 1./108.*m33*m44*RC3456pe - 1./108.*m33*m44*
         RS356pe + 1./18.*m33*m34*RS34561pe + 1./54.*m33*m34*RC3456pe
          - 1./54.*m33*m34*RS356pe - 1./36.*pow(m33,2)*RS34561pe - 1./
         108.*pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS356pe - 1./108.
         *m14*RD3456ep - 1./135.*m13*RD3456ep - 1./54.*m13*m44*
         RS34561pe + 1./54.*m13*m44*RC3456pe + 1./54.*m13*m44*RS356pe
          + 1./27.*m13*m34*RS34561pe - 1./27.*m13*m34*RC3456pe + 1./27.
         *m13*m34*RS356pe - 1./54.*m13*m33*RS34561pe + 1./54.*m13*m33*
         RC3456pe + 1./18.*m13*m33*RS356pe + 1./120.*m11*RD3456ep - 1./
         108.*m11*m44*RS34561pe - 1./108.*m11*m44*RC3456pe - 1./108.*
         m11*m44*RS356pe + 1./54.*m11*m34*RS34561pe + 1./54.*m11*m34*
         RC3456pe - 1./54.*m11*m34*RS356pe - 1./108.*m11*m33*RS34561pe
          - 1./108.*m11*m33*RC3456pe - 1./36.*m11*m33*RS356pe );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m35,L) * ( 7./540.*m55*RD3456ep
          - 1./60.*m35*RD3456ep + 13./1080.*m33*RD3456ep - 1./36.*m33*
         m55*RS34561pe - 1./108.*m33*m55*RC3456pe - 1./108.*m33*m55*
         RS346pe + 1./18.*m33*m35*RS34561pe + 1./54.*m33*m35*RC3456pe
          - 1./54.*m33*m35*RS346pe - 1./36.*pow(m33,2)*RS34561pe - 1./
         108.*pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS346pe - 1./108.
         *m15*RD3456ep - 1./135.*m13*RD3456ep - 1./54.*m13*m55*
         RS34561pe + 1./54.*m13*m55*RC3456pe + 1./54.*m13*m55*RS346pe
          + 1./27.*m13*m35*RS34561pe - 1./27.*m13*m35*RC3456pe + 1./27.
         *m13*m35*RS346pe - 1./54.*m13*m33*RS34561pe + 1./54.*m13*m33*
         RC3456pe + 1./18.*m13*m33*RS346pe + 1./120.*m11*RD3456ep - 1./
         108.*m11*m55*RS34561pe - 1./108.*m11*m55*RC3456pe - 1./108.*
         m11*m55*RS346pe + 1./54.*m11*m35*RS34561pe + 1./54.*m11*m35*
         RC3456pe - 1./54.*m11*m35*RS346pe - 1./108.*m11*m33*RS34561pe
          - 1./108.*m11*m33*RC3456pe - 1./36.*m11*m33*RS346pe );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m36,L) * ( 7./540.*m66*RD3456ep
          - 1./60.*m36*RD3456ep + 13./1080.*m33*RD3456ep - 1./36.*m33*
         m66*RS34561pe - 1./108.*m33*m66*RC3456pe - 1./108.*m33*m66*
         RS345pe + 1./18.*m33*m36*RS34561pe + 1./54.*m33*m36*RC3456pe
          - 1./54.*m33*m36*RS345pe - 1./36.*pow(m33,2)*RS34561pe - 1./
         108.*pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS345pe - 1./108.
         *m16*RD3456ep - 1./135.*m13*RD3456ep - 1./54.*m13*m66*
         RS34561pe + 1./54.*m13*m66*RC3456pe + 1./54.*m13*m66*RS345pe
          + 1./27.*m13*m36*RS34561pe - 1./27.*m13*m36*RC3456pe + 1./27.
         *m13*m36*RS345pe - 1./54.*m13*m33*RS34561pe + 1./54.*m13*m33*
         RC3456pe + 1./18.*m13*m33*RS345pe + 1./120.*m11*RD3456ep - 1./
         108.*m11*m66*RS34561pe - 1./108.*m11*m66*RC3456pe - 1./108.*
         m11*m66*RS345pe + 1./54.*m11*m36*RS34561pe + 1./54.*m11*m36*
         RC3456pe - 1./54.*m11*m36*RS345pe - 1./108.*m11*m33*RS34561pe
          - 1./108.*m11*m33*RC3456pe - 1./36.*m11*m33*RS345pe );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m44,L) * ( 1./108.*m33*m44*pow(
         RS356pe,2) - 1./324.*m33*m44*RS456pe*pow(RS356pe,2) - 1./54.*
         m13*m44*pow(RS356pe,2) + 1./162.*m13*m44*RS456pe*pow(
         RS356pe,2) + 1./108.*m11*m44*pow(RS356pe,2) - 1./324.*m11*m44*
         RS456pe*pow(RS356pe,2) );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m45,L) * ( 1./216.*m33*m55*pow(
         RS346pe,2) + 1./324.*m33*m55*RS356pe*RS346pe + 1./648.*m33*m55
         *pow(RS356pe,2) - 1./324.*m33*m45*pow(RS346pe,2) + 1./162.*m33
         *m45*RS356pe*RS346pe - 1./324.*m33*m45*pow(RS356pe,2) + 1./648.
         *m33*m44*pow(RS346pe,2) + 1./324.*m33*m44*RS356pe*RS346pe + 1./
         216.*m33*m44*pow(RS356pe,2) - 1./324.*pow(m33,2)*pow(
         RS346pe,2) + 1./162.*pow(m33,2)*RS356pe*RS346pe - 1./324.*pow(
         m33,2)*pow(RS356pe,2) - 1./108.*m13*m55*pow(RS346pe,2) - 1./
         162.*m13*m55*RS356pe*RS346pe - 1./324.*m13*m55*pow(RS356pe,2)
          + 1./162.*m13*m45*pow(RS346pe,2) - 1./81.*m13*m45*RS356pe*
         RS346pe + 1./162.*m13*m45*pow(RS356pe,2) - 1./324.*m13*m44*
         pow(RS346pe,2) - 1./162.*m13*m44*RS356pe*RS346pe - 1./108.*m13
         *m44*pow(RS356pe,2) + 1./162.*m13*m33*pow(RS346pe,2) - 1./81.*
         m13*m33*RS356pe*RS346pe + 1./162.*m13*m33*pow(RS356pe,2) + 1./
         216.*m11*m55*pow(RS346pe,2) + 1./324.*m11*m55*RS356pe*RS346pe
          + 1./648.*m11*m55*pow(RS356pe,2) );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m45,L) * (  - 1./324.*m11*m45*
         pow(RS346pe,2) + 1./162.*m11*m45*RS356pe*RS346pe - 1./324.*m11
         *m45*pow(RS356pe,2) + 1./648.*m11*m44*pow(RS346pe,2) + 1./324.
         *m11*m44*RS356pe*RS346pe + 1./216.*m11*m44*pow(RS356pe,2) - 1./
         324.*m11*m33*pow(RS346pe,2) + 1./162.*m11*m33*RS356pe*RS346pe
          - 1./324.*m11*m33*pow(RS356pe,2) );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m46,L) * ( 1./216.*m33*m66*pow(
         RS345pe,2) + 1./324.*m33*m66*RS356pe*RS345pe + 1./648.*m33*m66
         *pow(RS356pe,2) - 1./324.*m33*m46*pow(RS345pe,2) + 1./162.*m33
         *m46*RS356pe*RS345pe - 1./324.*m33*m46*pow(RS356pe,2) + 1./648.
         *m33*m44*pow(RS345pe,2) + 1./324.*m33*m44*RS356pe*RS345pe + 1./
         216.*m33*m44*pow(RS356pe,2) - 1./324.*pow(m33,2)*pow(
         RS345pe,2) + 1./162.*pow(m33,2)*RS356pe*RS345pe - 1./324.*pow(
         m33,2)*pow(RS356pe,2) - 1./108.*m13*m66*pow(RS345pe,2) - 1./
         162.*m13*m66*RS356pe*RS345pe - 1./324.*m13*m66*pow(RS356pe,2)
          + 1./162.*m13*m46*pow(RS345pe,2) - 1./81.*m13*m46*RS356pe*
         RS345pe + 1./162.*m13*m46*pow(RS356pe,2) - 1./324.*m13*m44*
         pow(RS345pe,2) - 1./162.*m13*m44*RS356pe*RS345pe - 1./108.*m13
         *m44*pow(RS356pe,2) + 1./162.*m13*m33*pow(RS345pe,2) - 1./81.*
         m13*m33*RS356pe*RS345pe + 1./162.*m13*m33*pow(RS356pe,2) + 1./
         216.*m11*m66*pow(RS345pe,2) + 1./324.*m11*m66*RS356pe*RS345pe
          + 1./648.*m11*m66*pow(RS356pe,2) );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m46,L) * (  - 1./324.*m11*m46*
         pow(RS345pe,2) + 1./162.*m11*m46*RS356pe*RS345pe - 1./324.*m11
         *m46*pow(RS356pe,2) + 1./648.*m11*m44*pow(RS345pe,2) + 1./324.
         *m11*m44*RS356pe*RS345pe + 1./216.*m11*m44*pow(RS356pe,2) - 1./
         324.*m11*m33*pow(RS345pe,2) + 1./162.*m11*m33*RS356pe*RS345pe
          - 1./324.*m11*m33*pow(RS356pe,2) );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m55,L) * ( 1./108.*m33*m55*pow(
         RS346pe,2) - 1./324.*m33*m55*RS546pe*pow(RS346pe,2) - 1./54.*
         m13*m55*pow(RS346pe,2) + 1./162.*m13*m55*RS546pe*pow(
         RS346pe,2) + 1./108.*m11*m55*pow(RS346pe,2) - 1./324.*m11*m55*
         RS546pe*pow(RS346pe,2) );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m56,L) * ( 1./216.*m33*m66*pow(
         RS345pe,2) + 1./324.*m33*m66*RS346pe*RS345pe + 1./648.*m33*m66
         *pow(RS346pe,2) - 1./324.*m33*m56*pow(RS345pe,2) + 1./162.*m33
         *m56*RS346pe*RS345pe - 1./324.*m33*m56*pow(RS346pe,2) + 1./648.
         *m33*m55*pow(RS345pe,2) + 1./324.*m33*m55*RS346pe*RS345pe + 1./
         216.*m33*m55*pow(RS346pe,2) - 1./324.*pow(m33,2)*pow(
         RS345pe,2) + 1./162.*pow(m33,2)*RS346pe*RS345pe - 1./324.*pow(
         m33,2)*pow(RS346pe,2) - 1./108.*m13*m66*pow(RS345pe,2) - 1./
         162.*m13*m66*RS346pe*RS345pe - 1./324.*m13*m66*pow(RS346pe,2)
          + 1./162.*m13*m56*pow(RS345pe,2) - 1./81.*m13*m56*RS346pe*
         RS345pe + 1./162.*m13*m56*pow(RS346pe,2) - 1./324.*m13*m55*
         pow(RS345pe,2) - 1./162.*m13*m55*RS346pe*RS345pe - 1./108.*m13
         *m55*pow(RS346pe,2) + 1./162.*m13*m33*pow(RS345pe,2) - 1./81.*
         m13*m33*RS346pe*RS345pe + 1./162.*m13*m33*pow(RS346pe,2) + 1./
         216.*m11*m66*pow(RS345pe,2) + 1./324.*m11*m66*RS346pe*RS345pe
          + 1./648.*m11*m66*pow(RS346pe,2) );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m56,L) * (  - 1./324.*m11*m56*
         pow(RS345pe,2) + 1./162.*m11*m56*RS346pe*RS345pe - 1./324.*m11
         *m56*pow(RS346pe,2) + 1./648.*m11*m55*pow(RS345pe,2) + 1./324.
         *m11*m55*RS346pe*RS345pe + 1./216.*m11*m55*pow(RS346pe,2) - 1./
         324.*m11*m33*pow(RS345pe,2) + 1./162.*m11*m33*RS346pe*RS345pe
          - 1./324.*m11*m33*pow(RS346pe,2) );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(m66,L) * ( 1./108.*m33*m66*pow(
         RS345pe,2) - 1./324.*m33*m66*RS645pe*pow(RS345pe,2) - 1./54.*
         m13*m66*pow(RS345pe,2) + 1./162.*m13*m66*RS645pe*pow(
         RS345pe,2) + 1./108.*m11*m66*pow(RS345pe,2) - 1./324.*m11*m66*
         RS645pe*pow(RS345pe,2) );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(mp2,L) * (  - 1./270.*mp2*
         RD3456ep*RSp45613e + 1./540.*mp2*RSp45633e*RD3456ep + 1./540.*
         mp2*RSp45611e*RD3456ep + 13./810.*m33*RD3456ep*RSp45613e - 37./
         1620.*m33*RSp45633e*RD3456ep - 19./1620.*m33*RSp45611e*
         RD3456ep - 1./324.*m33*m66*RSp456e*pow(RS345pe,2) - 1./324.*
         m33*m55*RSp465e*pow(RS346pe,2) - 1./324.*m33*m44*RSp564e*pow(
         RS356pe,2) + 1./18.*pow(m33,2)*RSp45633e*RS34561pe + 1./54.*
         pow(m33,2)*RSp45633e*RC3456pe + 7./810.*m13*RD3456ep*RSp45613e
          - 7./1620.*m13*RSp45633e*RD3456ep - 7./1620.*m13*RSp45611e*
         RD3456ep + 1./162.*m13*m66*RSp456e*pow(RS345pe,2) + 1./162.*
         m13*m55*RSp465e*pow(RS346pe,2) + 1./162.*m13*m44*RSp564e*pow(
         RS356pe,2) + 1./27.*m13*m33*RSp45633e*RS34561pe - 1./27.*m13*
         m33*RSp45633e*RC3456pe - 1./405.*m11*RD3456ep*RSp45613e + 13./
         810.*m11*RSp45633e*RD3456ep - 11./810.*m11*RSp45611e*RD3456ep
          - 1./324.*m11*m66*RSp456e*pow(RS345pe,2) - 1./324.*m11*m55*
         RSp465e*pow(RS346pe,2) );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(mp2,L) * (  - 1./324.*m11*m44*
         RSp564e*pow(RS356pe,2) + 1./54.*m11*m33*RSp45633e*RS34561pe + 
         1./54.*m11*m33*RSp45633e*RC3456pe );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(me2,L) * (  - 1./270.*me2*
         RD3456ep*RSe45613p + 1./540.*me2*RSe45633p*RD3456ep + 1./540.*
         me2*RSe45611p*RD3456ep + 13./810.*m33*RD3456ep*RSe45613p - 37./
         1620.*m33*RSe45633p*RD3456ep - 19./1620.*m33*RSe45611p*
         RD3456ep - 1./324.*m33*m66*RSe456p*pow(RS345pe,2) - 1./324.*
         m33*m55*RSe465p*pow(RS346pe,2) - 1./324.*m33*m44*RSe564p*pow(
         RS356pe,2) + 1./18.*pow(m33,2)*RSe45633p*RS34561pe + 1./54.*
         pow(m33,2)*RSe45633p*RC3456pe + 7./810.*m13*RD3456ep*RSe45613p
          - 7./1620.*m13*RSe45633p*RD3456ep - 7./1620.*m13*RSe45611p*
         RD3456ep + 1./162.*m13*m66*RSe456p*pow(RS345pe,2) + 1./162.*
         m13*m55*RSe465p*pow(RS346pe,2) + 1./162.*m13*m44*RSe564p*pow(
         RS356pe,2) + 1./27.*m13*m33*RSe45633p*RS34561pe - 1./27.*m13*
         m33*RSe45633p*RC3456pe - 1./405.*m11*RD3456ep*RSe45613p + 13./
         810.*m11*RSe45633p*RD3456ep - 11./810.*m11*RSe45611p*RD3456ep
          - 1./324.*m11*m66*RSe456p*pow(RS345pe,2) - 1./324.*m11*m55*
         RSe465p*pow(RS346pe,2) );

      M13p6x23RV +=  + Bb(m33,mu2)*AbV(me2,L) * (  - 1./324.*m11*m44*
         RSe564p*pow(RS356pe,2) + 1./54.*m11*m33*RSe45633p*RS34561pe + 
         1./54.*m11*m33*RSe45633p*RC3456pe );

      M13p6x23RV +=  + Bb(m33,mu2)*BbV(m11,m33,L) * ( 1./54.*pow(m33,2)
         *RD3456ep*RS14563pe + 1./27.*m13*m33*RD3456ep*RS14563pe + 1./
         18.*m11*m33*RD3456ep*RS14563pe );

      M13p6x23RV +=  + Bb(m33,mu2)*BbV(m11,L) * (  - 19./1620.*m33*
         RD1456ep*RD3456ep - 7./1620.*m13*RD1456ep*RD3456ep - 19./1620.
         *m11*RD1456ep*RD3456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*BbV(m33,mp2,L) * ( 1./18.*pow(m33,2)
         *RD3456ep*RSp45613e + 1./54.*pow(m33,2)*RSp45633e*RD3456ep + 1.
         /27.*m13*m33*RD3456ep*RSp45613e - 1./27.*m13*m33*RSp45633e*
         RD3456ep + 1./54.*m11*m33*RD3456ep*RSp45613e + 1./54.*m11*m33*
         RSp45633e*RD3456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*BbV(m33,me2,L) * ( 1./18.*pow(m33,2)
         *RD3456ep*RSe45613p + 1./54.*pow(m33,2)*RSe45633p*RD3456ep + 1.
         /27.*m13*m33*RD3456ep*RSe45613p - 1./27.*m13*m33*RSe45633p*
         RD3456ep + 1./54.*m11*m33*RD3456ep*RSe45613p + 1./54.*m11*m33*
         RSe45633p*RD3456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*BbV(m33,L) * (  - 17./810.*m33*pow(
         RD3456ep,2) + 1./9.*pow(m33,2)*RD3456ep*RS34561pe + 1./27.*
         pow(m33,2)*RD3456ep*RC3456pe - 7./1620.*m13*pow(RD3456ep,2) + 
         2./27.*m13*m33*RD3456ep*RS34561pe - 2./27.*m13*m33*RD3456ep*
         RC3456pe + 13./810.*m11*pow(RD3456ep,2) + 1./27.*m11*m33*
         RD3456ep*RS34561pe + 1./27.*m11*m33*RD3456ep*RC3456pe );

      M13p6x23RV +=  + Bb(m33,mu2)*BbV(m34,L) * (  - 1./432.*pow(m44,2)
         *RD3456ep + 1./108.*m34*m44*RD3456ep - 1./108.*pow(m34,2)*
         RD3456ep - 1./144.*m33*m44*RD3456ep + 1./72.*m33*m34*RD3456ep
          - 1./216.*pow(m33,2)*RD3456ep + 1./216.*m13*m44*RD3456ep - 1./
         108.*m13*m34*RD3456ep + 1./216.*m13*m33*RD3456ep - 1./432.*m11
         *m44*RD3456ep + 1./216.*m11*m34*RD3456ep - 1./432.*m11*m33*
         RD3456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*BbV(m35,L) * (  - 1./432.*pow(m55,2)
         *RD3456ep + 1./108.*m35*m55*RD3456ep - 1./108.*pow(m35,2)*
         RD3456ep - 1./144.*m33*m55*RD3456ep + 1./72.*m33*m35*RD3456ep
          - 1./216.*pow(m33,2)*RD3456ep + 1./216.*m13*m55*RD3456ep - 1./
         108.*m13*m35*RD3456ep + 1./216.*m13*m33*RD3456ep - 1./432.*m11
         *m55*RD3456ep + 1./216.*m11*m35*RD3456ep - 1./432.*m11*m33*
         RD3456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*BbV(m36,L) * (  - 1./432.*pow(m66,2)
         *RD3456ep + 1./108.*m36*m66*RD3456ep - 1./108.*pow(m36,2)*
         RD3456ep - 1./144.*m33*m66*RD3456ep + 1./72.*m33*m36*RD3456ep
          - 1./216.*pow(m33,2)*RD3456ep + 1./216.*m13*m66*RD3456ep - 1./
         108.*m13*m36*RD3456ep + 1./216.*m13*m33*RD3456ep - 1./432.*m11
         *m66*RD3456ep + 1./216.*m11*m36*RD3456ep - 1./432.*m11*m33*
         RD3456ep );

      M13p6x23RV +=  + Bb(m33,mu2)*CbV(m33,L) * ( 1./54.*pow(m33,2)*
         pow(RD3456ep,2) - 1./27.*m13*m33*pow(RD3456ep,2) + 1./54.*m11*
         m33*pow(RD3456ep,2) );

      M13p6x23RV +=  + Bb(m34,mu2)*AbV(m33,L) * ( 1./216.*m44*RD3456ep
          - 1./432.*pow(m44,2)*RC3456pe - 1./216.*pow(m44,2)*RS356pe - 
         1./108.*m34*RD3456ep + 1./108.*m34*m44*RC3456pe - 1./108.*pow(
         m34,2)*RC3456pe + 1./54.*pow(m34,2)*RS356pe + 1./108.*m33*
         RD3456ep - 1./144.*m33*m44*RC3456pe - 5./216.*m33*m44*RS356pe
          + 1./72.*m33*m34*RC3456pe + 1./108.*m33*m34*RS356pe - 1./216.
         *pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS356pe - 1./108.*m13
         *RD3456ep + 1./216.*m13*m44*RC3456pe + 1./108.*m13*m44*RS356pe
          - 1./108.*m13*m34*RC3456pe + 1./54.*m13*m34*RS356pe + 1./216.
         *m13*m33*RC3456pe + 1./36.*m13*m33*RS356pe + 1./216.*m11*
         RD3456ep - 1./432.*m11*m44*RC3456pe - 1./216.*m11*m44*RS356pe
          + 1./216.*m11*m34*RC3456pe - 1./108.*m11*m34*RS356pe - 1./432.
         *m11*m33*RC3456pe - 1./72.*m11*m33*RS356pe );

      M13p6x23RV +=  + Bb(m34,mu2)*AbV(m34,L) * ( 1./72.*pow(m44,2) - 1.
         /18.*m34*m44 + 1./18.*pow(m34,2) + 1./24.*m33*m44 - 1./12.*m33
         *m34 + 1./36.*pow(m33,2) - 1./36.*m13*m44 + 1./18.*m13*m34 - 1.
         /36.*m13*m33 + 1./72.*m11*m44 - 1./36.*m11*m34 + 1./72.*m11*
         m33 );

      M13p6x23RV +=  + Bb(m34,mu2)*AbV(m35,L) * ( 1./144.*m44*m55 + 1./
         144.*pow(m44,2) - 1./72.*m35*m44 - 1./72.*m34*m55 - 1./36.*m34
         *m44 + 1./36.*m34*m35 + 1./36.*pow(m34,2) + 1./72.*m33*m55 + 1.
         /36.*m33*m44 - 1./36.*m33*m35 - 1./18.*m33*m34 + 1./36.*pow(
         m33,2) - 1./72.*m13*m55 - 1./72.*m13*m44 + 1./36.*m13*m35 + 1./
         36.*m13*m34 - 1./36.*m13*m33 + 1./144.*m11*m55 + 1./144.*m11*
         m44 - 1./72.*m11*m35 - 1./72.*m11*m34 + 1./72.*m11*m33 );

      M13p6x23RV +=  + Bb(m34,mu2)*AbV(m36,L) * ( 1./144.*m44*m66 + 1./
         144.*pow(m44,2) - 1./72.*m36*m44 - 1./72.*m34*m66 - 1./36.*m34
         *m44 + 1./36.*m34*m36 + 1./36.*pow(m34,2) + 1./72.*m33*m66 + 1.
         /36.*m33*m44 - 1./36.*m33*m36 - 1./18.*m33*m34 + 1./36.*pow(
         m33,2) - 1./72.*m13*m66 - 1./72.*m13*m44 + 1./36.*m13*m36 + 1./
         36.*m13*m34 - 1./36.*m13*m33 + 1./144.*m11*m66 + 1./144.*m11*
         m44 - 1./72.*m11*m36 - 1./72.*m11*m34 + 1./72.*m11*m33 );

      M13p6x23RV +=  + Bb(m34,mu2)*AbV(m44,L) * ( 1./144.*pow(m44,2) - 
         1./432.*pow(m44,2)*RS456pe - 1./36.*m34*m44 + 1./108.*m34*m44*
         RS456pe + 1./36.*pow(m34,2) - 1./108.*pow(m34,2)*RS456pe + 1./
         48.*m33*m44 - 1./144.*m33*m44*RS456pe - 1./24.*m33*m34 + 1./72.
         *m33*m34*RS456pe + 1./72.*pow(m33,2) - 1./216.*pow(m33,2)*
         RS456pe - 1./72.*m13*m44 + 1./216.*m13*m44*RS456pe + 1./36.*
         m13*m34 - 1./108.*m13*m34*RS456pe - 1./72.*m13*m33 + 1./216.*
         m13*m33*RS456pe + 1./144.*m11*m44 - 1./432.*m11*m44*RS456pe - 
         1./72.*m11*m34 + 1./216.*m11*m34*RS456pe + 1./144.*m11*m33 - 1.
         /432.*m11*m33*RS456pe );

      M13p6x23RV +=  + Bb(m34,mu2)*AbV(m45,L) * ( 1./144.*m44*m55 - 1./
         72.*m44*m45 + 1./72.*pow(m44,2) - 1./72.*m34*m55 + 1./36.*m34*
         m45 - 1./24.*m34*m44 + 1./36.*pow(m34,2) + 1./72.*m33*m55 - 1./
         36.*m33*m45 + 5./144.*m33*m44 - 1./24.*m33*m34 + 1./72.*pow(
         m33,2) - 1./72.*m13*m55 + 1./36.*m13*m45 - 1./36.*m13*m44 + 1./
         36.*m13*m34 - 1./72.*m13*m33 + 1./144.*m11*m55 - 1./72.*m11*
         m45 + 1./72.*m11*m44 - 1./72.*m11*m34 + 1./144.*m11*m33 );

      M13p6x23RV +=  + Bb(m34,mu2)*AbV(m46,L) * ( 1./144.*m44*m66 - 1./
         72.*m44*m46 + 1./72.*pow(m44,2) - 1./72.*m34*m66 + 1./36.*m34*
         m46 - 1./24.*m34*m44 + 1./36.*pow(m34,2) + 1./72.*m33*m66 - 1./
         36.*m33*m46 + 5./144.*m33*m44 - 1./24.*m33*m34 + 1./72.*pow(
         m33,2) - 1./72.*m13*m66 + 1./36.*m13*m46 - 1./36.*m13*m44 + 1./
         36.*m13*m34 - 1./72.*m13*m33 + 1./144.*m11*m66 - 1./72.*m11*
         m46 + 1./72.*m11*m44 - 1./72.*m11*m34 + 1./144.*m11*m33 );

      M13p6x23RV +=  + Bb(m34,mu2)*AbV(mp2,L) * ( 1./216.*m44*mp2*
         RSp45633e - 1./108.*m44*mp2*RSp563e + 1./216.*m44*mp2*RSp564e
          - 1./432.*pow(m44,2)*RSp45633e - 1./216.*pow(m44,2)*RSp563e
          - 1./144.*pow(m44,2)*RSp564e - 1./108.*m34*mp2*RSp45633e + 1./
         54.*m34*mp2*RSp563e - 1./108.*m34*mp2*RSp564e + 1./108.*m34*
         m44*RSp45633e + 1./54.*m34*m44*RSp564e - 1./108.*pow(m34,2)*
         RSp45633e + 1./54.*pow(m34,2)*RSp563e - 1./108.*pow(m34,2)*
         RSp564e + 1./108.*m33*mp2*RSp45633e - 1./54.*m33*mp2*RSp563e
          + 1./108.*m33*mp2*RSp564e - 5./432.*m33*m44*RSp45633e - 1./72.
         *m33*m44*RSp563e - 7./432.*m33*m44*RSp564e + 5./216.*m33*m34*
         RSp45633e - 1./108.*m33*m34*RSp563e + 1./72.*m33*m34*RSp564e
          - 1./72.*pow(m33,2)*RSp45633e - 1./108.*pow(m33,2)*RSp563e - 
         1./216.*pow(m33,2)*RSp564e - 1./108.*m13*mp2*RSp45633e + 1./54.
         *m13*mp2*RSp563e - 1./108.*m13*mp2*RSp564e + 1./216.*m13*m44*
         RSp45633e + 1./108.*m13*m44*RSp563e + 1./72.*m13*m44*RSp564e
          - 1./108.*m13*m34*RSp45633e );

      M13p6x23RV +=  + Bb(m34,mu2)*AbV(mp2,L) * ( 1./54.*m13*m34*
         RSp563e - 1./108.*m13*m34*RSp564e + 1./72.*m13*m33*RSp45633e
          + 1./108.*m13*m33*RSp563e + 1./216.*m13*m33*RSp564e + 1./216.
         *m11*mp2*RSp45633e - 1./108.*m11*mp2*RSp563e + 1./216.*m11*mp2
         *RSp564e - 1./432.*m11*m44*RSp45633e - 1./216.*m11*m44*RSp563e
          - 1./144.*m11*m44*RSp564e + 1./216.*m11*m34*RSp45633e - 1./
         108.*m11*m34*RSp563e + 1./216.*m11*m34*RSp564e - 1./144.*m11*
         m33*RSp45633e - 1./216.*m11*m33*RSp563e - 1./432.*m11*m33*
         RSp564e );

      M13p6x23RV +=  + Bb(m34,mu2)*AbV(me2,L) * ( 1./216.*m44*me2*
         RSe45633p - 1./108.*m44*me2*RSe563p + 1./216.*m44*me2*RSe564p
          - 1./432.*pow(m44,2)*RSe45633p - 1./216.*pow(m44,2)*RSe563p
          - 1./144.*pow(m44,2)*RSe564p - 1./108.*m34*me2*RSe45633p + 1./
         54.*m34*me2*RSe563p - 1./108.*m34*me2*RSe564p + 1./108.*m34*
         m44*RSe45633p + 1./54.*m34*m44*RSe564p - 1./108.*pow(m34,2)*
         RSe45633p + 1./54.*pow(m34,2)*RSe563p - 1./108.*pow(m34,2)*
         RSe564p + 1./108.*m33*me2*RSe45633p - 1./54.*m33*me2*RSe563p
          + 1./108.*m33*me2*RSe564p - 5./432.*m33*m44*RSe45633p - 1./72.
         *m33*m44*RSe563p - 7./432.*m33*m44*RSe564p + 5./216.*m33*m34*
         RSe45633p - 1./108.*m33*m34*RSe563p + 1./72.*m33*m34*RSe564p
          - 1./72.*pow(m33,2)*RSe45633p - 1./108.*pow(m33,2)*RSe563p - 
         1./216.*pow(m33,2)*RSe564p - 1./108.*m13*me2*RSe45633p + 1./54.
         *m13*me2*RSe563p - 1./108.*m13*me2*RSe564p + 1./216.*m13*m44*
         RSe45633p + 1./108.*m13*m44*RSe563p + 1./72.*m13*m44*RSe564p
          - 1./108.*m13*m34*RSe45633p );

      M13p6x23RV +=  + Bb(m34,mu2)*AbV(me2,L) * ( 1./54.*m13*m34*
         RSe563p - 1./108.*m13*m34*RSe564p + 1./72.*m13*m33*RSe45633p
          + 1./108.*m13*m33*RSe563p + 1./216.*m13*m33*RSe564p + 1./216.
         *m11*me2*RSe45633p - 1./108.*m11*me2*RSe563p + 1./216.*m11*me2
         *RSe564p - 1./432.*m11*m44*RSe45633p - 1./216.*m11*m44*RSe563p
          - 1./144.*m11*m44*RSe564p + 1./216.*m11*m34*RSe45633p - 1./
         108.*m11*m34*RSe563p + 1./216.*m11*m34*RSe564p - 1./144.*m11*
         m33*RSe45633p - 1./216.*m11*m33*RSe563p - 1./432.*m11*m33*
         RSe564p );

      M13p6x23RV +=  + Bb(m34,mu2)*BbV(m33,L) * (  - 1./432.*pow(m44,2)
         *RD3456ep + 1./108.*m34*m44*RD3456ep - 1./108.*pow(m34,2)*
         RD3456ep - 1./144.*m33*m44*RD3456ep + 1./72.*m33*m34*RD3456ep
          - 1./216.*pow(m33,2)*RD3456ep + 1./216.*m13*m44*RD3456ep - 1./
         108.*m13*m34*RD3456ep + 1./216.*m13*m33*RD3456ep - 1./432.*m11
         *m44*RD3456ep + 1./216.*m11*m34*RD3456ep - 1./432.*m11*m33*
         RD3456ep );

      M13p6x23RV +=  + Bb(m35,mu2)*AbV(m33,L) * ( 1./216.*m55*RD3456ep
          - 1./432.*pow(m55,2)*RC3456pe - 1./216.*pow(m55,2)*RS346pe - 
         1./108.*m35*RD3456ep + 1./108.*m35*m55*RC3456pe - 1./108.*pow(
         m35,2)*RC3456pe + 1./54.*pow(m35,2)*RS346pe + 1./108.*m33*
         RD3456ep - 1./144.*m33*m55*RC3456pe - 5./216.*m33*m55*RS346pe
          + 1./72.*m33*m35*RC3456pe + 1./108.*m33*m35*RS346pe - 1./216.
         *pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS346pe - 1./108.*m13
         *RD3456ep + 1./216.*m13*m55*RC3456pe + 1./108.*m13*m55*RS346pe
          - 1./108.*m13*m35*RC3456pe + 1./54.*m13*m35*RS346pe + 1./216.
         *m13*m33*RC3456pe + 1./36.*m13*m33*RS346pe + 1./216.*m11*
         RD3456ep - 1./432.*m11*m55*RC3456pe - 1./216.*m11*m55*RS346pe
          + 1./216.*m11*m35*RC3456pe - 1./108.*m11*m35*RS346pe - 1./432.
         *m11*m33*RC3456pe - 1./72.*m11*m33*RS346pe );

      M13p6x23RV +=  + Bb(m35,mu2)*AbV(m34,L) * ( 1./144.*pow(m55,2) + 
         1./144.*m44*m55 - 1./36.*m35*m55 - 1./72.*m35*m44 + 1./36.*
         pow(m35,2) - 1./72.*m34*m55 + 1./36.*m34*m35 + 1./36.*m33*m55
          + 1./72.*m33*m44 - 1./18.*m33*m35 - 1./36.*m33*m34 + 1./36.*
         pow(m33,2) - 1./72.*m13*m55 - 1./72.*m13*m44 + 1./36.*m13*m35
          + 1./36.*m13*m34 - 1./36.*m13*m33 + 1./144.*m11*m55 + 1./144.
         *m11*m44 - 1./72.*m11*m35 - 1./72.*m11*m34 + 1./72.*m11*m33 );

      M13p6x23RV +=  + Bb(m35,mu2)*AbV(m35,L) * ( 1./72.*pow(m55,2) - 1.
         /18.*m35*m55 + 1./18.*pow(m35,2) + 1./24.*m33*m55 - 1./12.*m33
         *m35 + 1./36.*pow(m33,2) - 1./36.*m13*m55 + 1./18.*m13*m35 - 1.
         /36.*m13*m33 + 1./72.*m11*m55 - 1./36.*m11*m35 + 1./72.*m11*
         m33 );

      M13p6x23RV +=  + Bb(m35,mu2)*AbV(m36,L) * ( 1./144.*m55*m66 + 1./
         144.*pow(m55,2) - 1./72.*m36*m55 - 1./72.*m35*m66 - 1./36.*m35
         *m55 + 1./36.*m35*m36 + 1./36.*pow(m35,2) + 1./72.*m33*m66 + 1.
         /36.*m33*m55 - 1./36.*m33*m36 - 1./18.*m33*m35 + 1./36.*pow(
         m33,2) - 1./72.*m13*m66 - 1./72.*m13*m55 + 1./36.*m13*m36 + 1./
         36.*m13*m35 - 1./36.*m13*m33 + 1./144.*m11*m66 + 1./144.*m11*
         m55 - 1./72.*m11*m36 - 1./72.*m11*m35 + 1./72.*m11*m33 );

      M13p6x23RV +=  + Bb(m35,mu2)*AbV(m45,L) * ( 1./72.*pow(m55,2) - 1.
         /72.*m45*m55 + 1./144.*m44*m55 - 1./24.*m35*m55 + 1./36.*m35*
         m45 - 1./72.*m35*m44 + 1./36.*pow(m35,2) + 5./144.*m33*m55 - 1.
         /36.*m33*m45 + 1./72.*m33*m44 - 1./24.*m33*m35 + 1./72.*pow(
         m33,2) - 1./36.*m13*m55 + 1./36.*m13*m45 - 1./72.*m13*m44 + 1./
         36.*m13*m35 - 1./72.*m13*m33 + 1./72.*m11*m55 - 1./72.*m11*m45
          + 1./144.*m11*m44 - 1./72.*m11*m35 + 1./144.*m11*m33 );

      M13p6x23RV +=  + Bb(m35,mu2)*AbV(m55,L) * ( 1./144.*pow(m55,2) - 
         1./432.*pow(m55,2)*RS546pe - 1./36.*m35*m55 + 1./108.*m35*m55*
         RS546pe + 1./36.*pow(m35,2) - 1./108.*pow(m35,2)*RS546pe + 1./
         48.*m33*m55 - 1./144.*m33*m55*RS546pe - 1./24.*m33*m35 + 1./72.
         *m33*m35*RS546pe + 1./72.*pow(m33,2) - 1./216.*pow(m33,2)*
         RS546pe - 1./72.*m13*m55 + 1./216.*m13*m55*RS546pe + 1./36.*
         m13*m35 - 1./108.*m13*m35*RS546pe - 1./72.*m13*m33 + 1./216.*
         m13*m33*RS546pe + 1./144.*m11*m55 - 1./432.*m11*m55*RS546pe - 
         1./72.*m11*m35 + 1./216.*m11*m35*RS546pe + 1./144.*m11*m33 - 1.
         /432.*m11*m33*RS546pe );

      M13p6x23RV +=  + Bb(m35,mu2)*AbV(m56,L) * ( 1./144.*m55*m66 - 1./
         72.*m55*m56 + 1./72.*pow(m55,2) - 1./72.*m35*m66 + 1./36.*m35*
         m56 - 1./24.*m35*m55 + 1./36.*pow(m35,2) + 1./72.*m33*m66 - 1./
         36.*m33*m56 + 5./144.*m33*m55 - 1./24.*m33*m35 + 1./72.*pow(
         m33,2) - 1./72.*m13*m66 + 1./36.*m13*m56 - 1./36.*m13*m55 + 1./
         36.*m13*m35 - 1./72.*m13*m33 + 1./144.*m11*m66 - 1./72.*m11*
         m56 + 1./72.*m11*m55 - 1./72.*m11*m35 + 1./144.*m11*m33 );

      M13p6x23RV +=  + Bb(m35,mu2)*AbV(mp2,L) * ( 1./216.*m55*mp2*
         RSp45633e - 1./108.*m55*mp2*RSp463e + 1./216.*m55*mp2*RSp465e
          - 1./432.*pow(m55,2)*RSp45633e - 1./216.*pow(m55,2)*RSp463e
          - 1./144.*pow(m55,2)*RSp465e - 1./108.*m35*mp2*RSp45633e + 1./
         54.*m35*mp2*RSp463e - 1./108.*m35*mp2*RSp465e + 1./108.*m35*
         m55*RSp45633e + 1./54.*m35*m55*RSp465e - 1./108.*pow(m35,2)*
         RSp45633e + 1./54.*pow(m35,2)*RSp463e - 1./108.*pow(m35,2)*
         RSp465e + 1./108.*m33*mp2*RSp45633e - 1./54.*m33*mp2*RSp463e
          + 1./108.*m33*mp2*RSp465e - 5./432.*m33*m55*RSp45633e - 1./72.
         *m33*m55*RSp463e - 7./432.*m33*m55*RSp465e + 5./216.*m33*m35*
         RSp45633e - 1./108.*m33*m35*RSp463e + 1./72.*m33*m35*RSp465e
          - 1./72.*pow(m33,2)*RSp45633e - 1./108.*pow(m33,2)*RSp463e - 
         1./216.*pow(m33,2)*RSp465e - 1./108.*m13*mp2*RSp45633e + 1./54.
         *m13*mp2*RSp463e - 1./108.*m13*mp2*RSp465e + 1./216.*m13*m55*
         RSp45633e + 1./108.*m13*m55*RSp463e + 1./72.*m13*m55*RSp465e
          - 1./108.*m13*m35*RSp45633e );

      M13p6x23RV +=  + Bb(m35,mu2)*AbV(mp2,L) * ( 1./54.*m13*m35*
         RSp463e - 1./108.*m13*m35*RSp465e + 1./72.*m13*m33*RSp45633e
          + 1./108.*m13*m33*RSp463e + 1./216.*m13*m33*RSp465e + 1./216.
         *m11*mp2*RSp45633e - 1./108.*m11*mp2*RSp463e + 1./216.*m11*mp2
         *RSp465e - 1./432.*m11*m55*RSp45633e - 1./216.*m11*m55*RSp463e
          - 1./144.*m11*m55*RSp465e + 1./216.*m11*m35*RSp45633e - 1./
         108.*m11*m35*RSp463e + 1./216.*m11*m35*RSp465e - 1./144.*m11*
         m33*RSp45633e - 1./216.*m11*m33*RSp463e - 1./432.*m11*m33*
         RSp465e );

      M13p6x23RV +=  + Bb(m35,mu2)*AbV(me2,L) * ( 1./216.*m55*me2*
         RSe45633p - 1./108.*m55*me2*RSe463p + 1./216.*m55*me2*RSe465p
          - 1./432.*pow(m55,2)*RSe45633p - 1./216.*pow(m55,2)*RSe463p
          - 1./144.*pow(m55,2)*RSe465p - 1./108.*m35*me2*RSe45633p + 1./
         54.*m35*me2*RSe463p - 1./108.*m35*me2*RSe465p + 1./108.*m35*
         m55*RSe45633p + 1./54.*m35*m55*RSe465p - 1./108.*pow(m35,2)*
         RSe45633p + 1./54.*pow(m35,2)*RSe463p - 1./108.*pow(m35,2)*
         RSe465p + 1./108.*m33*me2*RSe45633p - 1./54.*m33*me2*RSe463p
          + 1./108.*m33*me2*RSe465p - 5./432.*m33*m55*RSe45633p - 1./72.
         *m33*m55*RSe463p - 7./432.*m33*m55*RSe465p + 5./216.*m33*m35*
         RSe45633p - 1./108.*m33*m35*RSe463p + 1./72.*m33*m35*RSe465p
          - 1./72.*pow(m33,2)*RSe45633p - 1./108.*pow(m33,2)*RSe463p - 
         1./216.*pow(m33,2)*RSe465p - 1./108.*m13*me2*RSe45633p + 1./54.
         *m13*me2*RSe463p - 1./108.*m13*me2*RSe465p + 1./216.*m13*m55*
         RSe45633p + 1./108.*m13*m55*RSe463p + 1./72.*m13*m55*RSe465p
          - 1./108.*m13*m35*RSe45633p );

      M13p6x23RV +=  + Bb(m35,mu2)*AbV(me2,L) * ( 1./54.*m13*m35*
         RSe463p - 1./108.*m13*m35*RSe465p + 1./72.*m13*m33*RSe45633p
          + 1./108.*m13*m33*RSe463p + 1./216.*m13*m33*RSe465p + 1./216.
         *m11*me2*RSe45633p - 1./108.*m11*me2*RSe463p + 1./216.*m11*me2
         *RSe465p - 1./432.*m11*m55*RSe45633p - 1./216.*m11*m55*RSe463p
          - 1./144.*m11*m55*RSe465p + 1./216.*m11*m35*RSe45633p - 1./
         108.*m11*m35*RSe463p + 1./216.*m11*m35*RSe465p - 1./144.*m11*
         m33*RSe45633p - 1./216.*m11*m33*RSe463p - 1./432.*m11*m33*
         RSe465p );

      M13p6x23RV +=  + Bb(m35,mu2)*BbV(m33,L) * (  - 1./432.*pow(m55,2)
         *RD3456ep + 1./108.*m35*m55*RD3456ep - 1./108.*pow(m35,2)*
         RD3456ep - 1./144.*m33*m55*RD3456ep + 1./72.*m33*m35*RD3456ep
          - 1./216.*pow(m33,2)*RD3456ep + 1./216.*m13*m55*RD3456ep - 1./
         108.*m13*m35*RD3456ep + 1./216.*m13*m33*RD3456ep - 1./432.*m11
         *m55*RD3456ep + 1./216.*m11*m35*RD3456ep - 1./432.*m11*m33*
         RD3456ep );

      M13p6x23RV +=  + Bb(m36,mu2)*AbV(m33,L) * ( 1./216.*m66*RD3456ep
          - 1./432.*pow(m66,2)*RC3456pe - 1./216.*pow(m66,2)*RS345pe - 
         1./108.*m36*RD3456ep + 1./108.*m36*m66*RC3456pe - 1./108.*pow(
         m36,2)*RC3456pe + 1./54.*pow(m36,2)*RS345pe + 1./108.*m33*
         RD3456ep - 1./144.*m33*m66*RC3456pe - 5./216.*m33*m66*RS345pe
          + 1./72.*m33*m36*RC3456pe + 1./108.*m33*m36*RS345pe - 1./216.
         *pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS345pe - 1./108.*m13
         *RD3456ep + 1./216.*m13*m66*RC3456pe + 1./108.*m13*m66*RS345pe
          - 1./108.*m13*m36*RC3456pe + 1./54.*m13*m36*RS345pe + 1./216.
         *m13*m33*RC3456pe + 1./36.*m13*m33*RS345pe + 1./216.*m11*
         RD3456ep - 1./432.*m11*m66*RC3456pe - 1./216.*m11*m66*RS345pe
          + 1./216.*m11*m36*RC3456pe - 1./108.*m11*m36*RS345pe - 1./432.
         *m11*m33*RC3456pe - 1./72.*m11*m33*RS345pe );

      M13p6x23RV +=  + Bb(m36,mu2)*AbV(m34,L) * ( 1./144.*pow(m66,2) + 
         1./144.*m44*m66 - 1./36.*m36*m66 - 1./72.*m36*m44 + 1./36.*
         pow(m36,2) - 1./72.*m34*m66 + 1./36.*m34*m36 + 1./36.*m33*m66
          + 1./72.*m33*m44 - 1./18.*m33*m36 - 1./36.*m33*m34 + 1./36.*
         pow(m33,2) - 1./72.*m13*m66 - 1./72.*m13*m44 + 1./36.*m13*m36
          + 1./36.*m13*m34 - 1./36.*m13*m33 + 1./144.*m11*m66 + 1./144.
         *m11*m44 - 1./72.*m11*m36 - 1./72.*m11*m34 + 1./72.*m11*m33 );

      M13p6x23RV +=  + Bb(m36,mu2)*AbV(m35,L) * ( 1./144.*pow(m66,2) + 
         1./144.*m55*m66 - 1./36.*m36*m66 - 1./72.*m36*m55 + 1./36.*
         pow(m36,2) - 1./72.*m35*m66 + 1./36.*m35*m36 + 1./36.*m33*m66
          + 1./72.*m33*m55 - 1./18.*m33*m36 - 1./36.*m33*m35 + 1./36.*
         pow(m33,2) - 1./72.*m13*m66 - 1./72.*m13*m55 + 1./36.*m13*m36
          + 1./36.*m13*m35 - 1./36.*m13*m33 + 1./144.*m11*m66 + 1./144.
         *m11*m55 - 1./72.*m11*m36 - 1./72.*m11*m35 + 1./72.*m11*m33 );

      M13p6x23RV +=  + Bb(m36,mu2)*AbV(m36,L) * ( 1./72.*pow(m66,2) - 1.
         /18.*m36*m66 + 1./18.*pow(m36,2) + 1./24.*m33*m66 - 1./12.*m33
         *m36 + 1./36.*pow(m33,2) - 1./36.*m13*m66 + 1./18.*m13*m36 - 1.
         /36.*m13*m33 + 1./72.*m11*m66 - 1./36.*m11*m36 + 1./72.*m11*
         m33 );

      M13p6x23RV +=  + Bb(m36,mu2)*AbV(m46,L) * ( 1./72.*pow(m66,2) - 1.
         /72.*m46*m66 + 1./144.*m44*m66 - 1./24.*m36*m66 + 1./36.*m36*
         m46 - 1./72.*m36*m44 + 1./36.*pow(m36,2) + 5./144.*m33*m66 - 1.
         /36.*m33*m46 + 1./72.*m33*m44 - 1./24.*m33*m36 + 1./72.*pow(
         m33,2) - 1./36.*m13*m66 + 1./36.*m13*m46 - 1./72.*m13*m44 + 1./
         36.*m13*m36 - 1./72.*m13*m33 + 1./72.*m11*m66 - 1./72.*m11*m46
          + 1./144.*m11*m44 - 1./72.*m11*m36 + 1./144.*m11*m33 );

      M13p6x23RV +=  + Bb(m36,mu2)*AbV(m56,L) * ( 1./72.*pow(m66,2) - 1.
         /72.*m56*m66 + 1./144.*m55*m66 - 1./24.*m36*m66 + 1./36.*m36*
         m56 - 1./72.*m36*m55 + 1./36.*pow(m36,2) + 5./144.*m33*m66 - 1.
         /36.*m33*m56 + 1./72.*m33*m55 - 1./24.*m33*m36 + 1./72.*pow(
         m33,2) - 1./36.*m13*m66 + 1./36.*m13*m56 - 1./72.*m13*m55 + 1./
         36.*m13*m36 - 1./72.*m13*m33 + 1./72.*m11*m66 - 1./72.*m11*m56
          + 1./144.*m11*m55 - 1./72.*m11*m36 + 1./144.*m11*m33 );

      M13p6x23RV +=  + Bb(m36,mu2)*AbV(m66,L) * ( 1./144.*pow(m66,2) - 
         1./432.*pow(m66,2)*RS645pe - 1./36.*m36*m66 + 1./108.*m36*m66*
         RS645pe + 1./36.*pow(m36,2) - 1./108.*pow(m36,2)*RS645pe + 1./
         48.*m33*m66 - 1./144.*m33*m66*RS645pe - 1./24.*m33*m36 + 1./72.
         *m33*m36*RS645pe + 1./72.*pow(m33,2) - 1./216.*pow(m33,2)*
         RS645pe - 1./72.*m13*m66 + 1./216.*m13*m66*RS645pe + 1./36.*
         m13*m36 - 1./108.*m13*m36*RS645pe - 1./72.*m13*m33 + 1./216.*
         m13*m33*RS645pe + 1./144.*m11*m66 - 1./432.*m11*m66*RS645pe - 
         1./72.*m11*m36 + 1./216.*m11*m36*RS645pe + 1./144.*m11*m33 - 1.
         /432.*m11*m33*RS645pe );

      M13p6x23RV +=  + Bb(m36,mu2)*AbV(mp2,L) * ( 1./216.*m66*mp2*
         RSp45633e - 1./108.*m66*mp2*RSp453e + 1./216.*m66*mp2*RSp456e
          - 1./432.*pow(m66,2)*RSp45633e - 1./216.*pow(m66,2)*RSp453e
          - 1./144.*pow(m66,2)*RSp456e - 1./108.*m36*mp2*RSp45633e + 1./
         54.*m36*mp2*RSp453e - 1./108.*m36*mp2*RSp456e + 1./108.*m36*
         m66*RSp45633e + 1./54.*m36*m66*RSp456e - 1./108.*pow(m36,2)*
         RSp45633e + 1./54.*pow(m36,2)*RSp453e - 1./108.*pow(m36,2)*
         RSp456e + 1./108.*m33*mp2*RSp45633e - 1./54.*m33*mp2*RSp453e
          + 1./108.*m33*mp2*RSp456e - 5./432.*m33*m66*RSp45633e - 1./72.
         *m33*m66*RSp453e - 7./432.*m33*m66*RSp456e + 5./216.*m33*m36*
         RSp45633e - 1./108.*m33*m36*RSp453e + 1./72.*m33*m36*RSp456e
          - 1./72.*pow(m33,2)*RSp45633e - 1./108.*pow(m33,2)*RSp453e - 
         1./216.*pow(m33,2)*RSp456e - 1./108.*m13*mp2*RSp45633e + 1./54.
         *m13*mp2*RSp453e - 1./108.*m13*mp2*RSp456e + 1./216.*m13*m66*
         RSp45633e + 1./108.*m13*m66*RSp453e + 1./72.*m13*m66*RSp456e
          - 1./108.*m13*m36*RSp45633e );

      M13p6x23RV +=  + Bb(m36,mu2)*AbV(mp2,L) * ( 1./54.*m13*m36*
         RSp453e - 1./108.*m13*m36*RSp456e + 1./72.*m13*m33*RSp45633e
          + 1./108.*m13*m33*RSp453e + 1./216.*m13*m33*RSp456e + 1./216.
         *m11*mp2*RSp45633e - 1./108.*m11*mp2*RSp453e + 1./216.*m11*mp2
         *RSp456e - 1./432.*m11*m66*RSp45633e - 1./216.*m11*m66*RSp453e
          - 1./144.*m11*m66*RSp456e + 1./216.*m11*m36*RSp45633e - 1./
         108.*m11*m36*RSp453e + 1./216.*m11*m36*RSp456e - 1./144.*m11*
         m33*RSp45633e - 1./216.*m11*m33*RSp453e - 1./432.*m11*m33*
         RSp456e );

      M13p6x23RV +=  + Bb(m36,mu2)*AbV(me2,L) * ( 1./216.*m66*me2*
         RSe45633p - 1./108.*m66*me2*RSe453p + 1./216.*m66*me2*RSe456p
          - 1./432.*pow(m66,2)*RSe45633p - 1./216.*pow(m66,2)*RSe453p
          - 1./144.*pow(m66,2)*RSe456p - 1./108.*m36*me2*RSe45633p + 1./
         54.*m36*me2*RSe453p - 1./108.*m36*me2*RSe456p + 1./108.*m36*
         m66*RSe45633p + 1./54.*m36*m66*RSe456p - 1./108.*pow(m36,2)*
         RSe45633p + 1./54.*pow(m36,2)*RSe453p - 1./108.*pow(m36,2)*
         RSe456p + 1./108.*m33*me2*RSe45633p - 1./54.*m33*me2*RSe453p
          + 1./108.*m33*me2*RSe456p - 5./432.*m33*m66*RSe45633p - 1./72.
         *m33*m66*RSe453p - 7./432.*m33*m66*RSe456p + 5./216.*m33*m36*
         RSe45633p - 1./108.*m33*m36*RSe453p + 1./72.*m33*m36*RSe456p
          - 1./72.*pow(m33,2)*RSe45633p - 1./108.*pow(m33,2)*RSe453p - 
         1./216.*pow(m33,2)*RSe456p - 1./108.*m13*me2*RSe45633p + 1./54.
         *m13*me2*RSe453p - 1./108.*m13*me2*RSe456p + 1./216.*m13*m66*
         RSe45633p + 1./108.*m13*m66*RSe453p + 1./72.*m13*m66*RSe456p
          - 1./108.*m13*m36*RSe45633p );

      M13p6x23RV +=  + Bb(m36,mu2)*AbV(me2,L) * ( 1./54.*m13*m36*
         RSe453p - 1./108.*m13*m36*RSe456p + 1./72.*m13*m33*RSe45633p
          + 1./108.*m13*m33*RSe453p + 1./216.*m13*m33*RSe456p + 1./216.
         *m11*me2*RSe45633p - 1./108.*m11*me2*RSe453p + 1./216.*m11*me2
         *RSe456p - 1./432.*m11*m66*RSe45633p - 1./216.*m11*m66*RSe453p
          - 1./144.*m11*m66*RSe456p + 1./216.*m11*m36*RSe45633p - 1./
         108.*m11*m36*RSe453p + 1./216.*m11*m36*RSe456p - 1./144.*m11*
         m33*RSe45633p - 1./216.*m11*m33*RSe453p - 1./432.*m11*m33*
         RSe456p );

      M13p6x23RV +=  + Bb(m36,mu2)*BbV(m33,L) * (  - 1./432.*pow(m66,2)
         *RD3456ep + 1./108.*m36*m66*RD3456ep - 1./108.*pow(m36,2)*
         RD3456ep - 1./144.*m33*m66*RD3456ep + 1./72.*m33*m36*RD3456ep
          - 1./216.*pow(m33,2)*RD3456ep + 1./216.*m13*m66*RD3456ep - 1./
         108.*m13*m36*RD3456ep + 1./216.*m13*m33*RD3456ep - 1./432.*m11
         *m66*RD3456ep + 1./216.*m11*m36*RD3456ep - 1./432.*m11*m33*
         RD3456ep );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m44,L) * (  - 1./54.*m44*mp2*
         pow(RSp563e,2) + 1./162.*m44*mp2*RS456pe*pow(RSp563e,2) + 1./
         27.*m44*mp2*RSp561e*RSp563e - 1./81.*m44*mp2*RSp561e*RS456pe*
         RSp563e - 1./54.*m44*mp2*pow(RSp561e,2) + 1./162.*m44*mp2*pow(
         RSp561e,2)*RS456pe + 1./36.*m33*m44*pow(RSp563e,2) - 1./108.*
         m33*m44*RS456pe*pow(RSp563e,2) + 1./54.*m33*m44*RSp561e*
         RSp563e - 1./162.*m33*m44*RSp561e*RS456pe*RSp563e + 1./108.*
         m33*m44*pow(RSp561e,2) - 1./324.*m33*m44*pow(RSp561e,2)*
         RS456pe - 1./54.*m13*m44*pow(RSp563e,2) + 1./162.*m13*m44*
         RS456pe*pow(RSp563e,2) + 1./27.*m13*m44*RSp561e*RSp563e - 1./
         81.*m13*m44*RSp561e*RS456pe*RSp563e - 1./54.*m13*m44*pow(
         RSp561e,2) + 1./162.*m13*m44*pow(RSp561e,2)*RS456pe + 1./108.*
         m11*m44*pow(RSp563e,2) - 1./324.*m11*m44*RS456pe*pow(
         RSp563e,2) + 1./54.*m11*m44*RSp561e*RSp563e - 1./162.*m11*m44*
         RSp561e*RS456pe*RSp563e + 1./36.*m11*m44*pow(RSp561e,2) - 1./
         108.*m11*m44*pow(RSp561e,2)*RS456pe );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m45,L) * ( 1./162.*pow(mp2,2)*
         pow(RSp563e,2) - 1./81.*pow(mp2,2)*RSp463e*RSp563e + 1./162.*
         pow(mp2,2)*pow(RSp463e,2) - 1./81.*pow(mp2,2)*RSp561e*RSp563e
          + 1./81.*pow(mp2,2)*RSp561e*RSp463e + 1./162.*pow(mp2,2)*pow(
         RSp561e,2) + 1./81.*pow(mp2,2)*RSp461e*RSp563e - 1./81.*pow(
         mp2,2)*RSp461e*RSp463e - 1./81.*pow(mp2,2)*RSp461e*RSp561e + 1.
         /162.*pow(mp2,2)*pow(RSp461e,2) - 1./324.*m55*mp2*pow(
         RSp563e,2) - 1./162.*m55*mp2*RSp463e*RSp563e - 1./108.*m55*mp2
         *pow(RSp463e,2) + 1./162.*m55*mp2*RSp561e*RSp563e + 1./162.*
         m55*mp2*RSp561e*RSp463e - 1./324.*m55*mp2*pow(RSp561e,2) + 1./
         162.*m55*mp2*RSp461e*RSp563e + 1./54.*m55*mp2*RSp461e*RSp463e
          - 1./162.*m55*mp2*RSp461e*RSp561e - 1./108.*m55*mp2*pow(
         RSp461e,2) + 1./162.*m45*mp2*pow(RSp563e,2) - 1./81.*m45*mp2*
         RSp463e*RSp563e + 1./162.*m45*mp2*pow(RSp463e,2) - 1./81.*m45*
         mp2*RSp561e*RSp563e + 1./81.*m45*mp2*RSp561e*RSp463e + 1./162.
         *m45*mp2*pow(RSp561e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m45,L) * ( 1./81.*m45*mp2*
         RSp461e*RSp563e - 1./81.*m45*mp2*RSp461e*RSp463e - 1./81.*m45*
         mp2*RSp461e*RSp561e + 1./162.*m45*mp2*pow(RSp461e,2) - 1./108.
         *m44*mp2*pow(RSp563e,2) - 1./162.*m44*mp2*RSp463e*RSp563e - 1./
         324.*m44*mp2*pow(RSp463e,2) + 1./54.*m44*mp2*RSp561e*RSp563e
          + 1./162.*m44*mp2*RSp561e*RSp463e - 1./108.*m44*mp2*pow(
         RSp561e,2) + 1./162.*m44*mp2*RSp461e*RSp563e + 1./162.*m44*mp2
         *RSp461e*RSp463e - 1./162.*m44*mp2*RSp461e*RSp561e - 1./324.*
         m44*mp2*pow(RSp461e,2) - 1./108.*m33*mp2*pow(RSp563e,2) + 1./
         54.*m33*mp2*RSp463e*RSp563e - 1./108.*m33*mp2*pow(RSp463e,2)
          - 1./162.*m33*mp2*RSp561e*RSp563e + 1./162.*m33*mp2*RSp561e*
         RSp463e - 1./324.*m33*mp2*pow(RSp561e,2) + 1./162.*m33*mp2*
         RSp461e*RSp563e - 1./162.*m33*mp2*RSp461e*RSp463e + 1./162.*
         m33*mp2*RSp461e*RSp561e - 1./324.*m33*mp2*pow(RSp461e,2) + 1./
         216.*m33*m55*pow(RSp563e,2) + 1./108.*m33*m55*RSp463e*RSp563e
          + 1./72.*m33*m55*pow(RSp463e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m45,L) * ( 1./324.*m33*m55*
         RSp561e*RSp563e + 1./324.*m33*m55*RSp561e*RSp463e + 1./648.*
         m33*m55*pow(RSp561e,2) + 1./324.*m33*m55*RSp461e*RSp563e + 1./
         108.*m33*m55*RSp461e*RSp463e + 1./324.*m33*m55*RSp461e*RSp561e
          + 1./216.*m33*m55*pow(RSp461e,2) - 1./108.*m33*m45*pow(
         RSp563e,2) + 1./54.*m33*m45*RSp463e*RSp563e - 1./108.*m33*m45*
         pow(RSp463e,2) - 1./162.*m33*m45*RSp561e*RSp563e + 1./162.*m33
         *m45*RSp561e*RSp463e - 1./324.*m33*m45*pow(RSp561e,2) + 1./162.
         *m33*m45*RSp461e*RSp563e - 1./162.*m33*m45*RSp461e*RSp463e + 1.
         /162.*m33*m45*RSp461e*RSp561e - 1./324.*m33*m45*pow(RSp461e,2)
          + 1./72.*m33*m44*pow(RSp563e,2) + 1./108.*m33*m44*RSp463e*
         RSp563e + 1./216.*m33*m44*pow(RSp463e,2) + 1./108.*m33*m44*
         RSp561e*RSp563e + 1./324.*m33*m44*RSp561e*RSp463e + 1./216.*
         m33*m44*pow(RSp561e,2) + 1./324.*m33*m44*RSp461e*RSp563e + 1./
         324.*m33*m44*RSp461e*RSp463e + 1./324.*m33*m44*RSp461e*RSp561e
          + 1./648.*m33*m44*pow(RSp461e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m45,L) * ( 1./162.*m13*mp2*pow(
         RSp563e,2) - 1./81.*m13*mp2*RSp463e*RSp563e + 1./162.*m13*mp2*
         pow(RSp463e,2) - 1./81.*m13*mp2*RSp561e*RSp563e + 1./81.*m13*
         mp2*RSp561e*RSp463e + 1./162.*m13*mp2*pow(RSp561e,2) + 1./81.*
         m13*mp2*RSp461e*RSp563e - 1./81.*m13*mp2*RSp461e*RSp463e - 1./
         81.*m13*mp2*RSp461e*RSp561e + 1./162.*m13*mp2*pow(RSp461e,2)
          - 1./324.*m13*m55*pow(RSp563e,2) - 1./162.*m13*m55*RSp463e*
         RSp563e - 1./108.*m13*m55*pow(RSp463e,2) + 1./162.*m13*m55*
         RSp561e*RSp563e + 1./162.*m13*m55*RSp561e*RSp463e - 1./324.*
         m13*m55*pow(RSp561e,2) + 1./162.*m13*m55*RSp461e*RSp563e + 1./
         54.*m13*m55*RSp461e*RSp463e - 1./162.*m13*m55*RSp461e*RSp561e
          - 1./108.*m13*m55*pow(RSp461e,2) + 1./162.*m13*m45*pow(
         RSp563e,2) - 1./81.*m13*m45*RSp463e*RSp563e + 1./162.*m13*m45*
         pow(RSp463e,2) - 1./81.*m13*m45*RSp561e*RSp563e + 1./81.*m13*
         m45*RSp561e*RSp463e + 1./162.*m13*m45*pow(RSp561e,2) + 1./81.*
         m13*m45*RSp461e*RSp563e );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m45,L) * (  - 1./81.*m13*m45*
         RSp461e*RSp463e - 1./81.*m13*m45*RSp461e*RSp561e + 1./162.*m13
         *m45*pow(RSp461e,2) - 1./108.*m13*m44*pow(RSp563e,2) - 1./162.
         *m13*m44*RSp463e*RSp563e - 1./324.*m13*m44*pow(RSp463e,2) + 1./
         54.*m13*m44*RSp561e*RSp563e + 1./162.*m13*m44*RSp561e*RSp463e
          - 1./108.*m13*m44*pow(RSp561e,2) + 1./162.*m13*m44*RSp461e*
         RSp563e + 1./162.*m13*m44*RSp461e*RSp463e - 1./162.*m13*m44*
         RSp461e*RSp561e - 1./324.*m13*m44*pow(RSp461e,2) - 1./324.*m11
         *mp2*pow(RSp563e,2) + 1./162.*m11*mp2*RSp463e*RSp563e - 1./324.
         *m11*mp2*pow(RSp463e,2) - 1./162.*m11*mp2*RSp561e*RSp563e + 1./
         162.*m11*mp2*RSp561e*RSp463e - 1./108.*m11*mp2*pow(RSp561e,2)
          + 1./162.*m11*mp2*RSp461e*RSp563e - 1./162.*m11*mp2*RSp461e*
         RSp463e + 1./54.*m11*mp2*RSp461e*RSp561e - 1./108.*m11*mp2*
         pow(RSp461e,2) + 1./648.*m11*m55*pow(RSp563e,2) + 1./324.*m11*
         m55*RSp463e*RSp563e + 1./216.*m11*m55*pow(RSp463e,2) + 1./324.
         *m11*m55*RSp561e*RSp563e );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m45,L) * ( 1./324.*m11*m55*
         RSp561e*RSp463e + 1./216.*m11*m55*pow(RSp561e,2) + 1./324.*m11
         *m55*RSp461e*RSp563e + 1./108.*m11*m55*RSp461e*RSp463e + 1./
         108.*m11*m55*RSp461e*RSp561e + 1./72.*m11*m55*pow(RSp461e,2)
          - 1./324.*m11*m45*pow(RSp563e,2) + 1./162.*m11*m45*RSp463e*
         RSp563e - 1./324.*m11*m45*pow(RSp463e,2) - 1./162.*m11*m45*
         RSp561e*RSp563e + 1./162.*m11*m45*RSp561e*RSp463e - 1./108.*
         m11*m45*pow(RSp561e,2) + 1./162.*m11*m45*RSp461e*RSp563e - 1./
         162.*m11*m45*RSp461e*RSp463e + 1./54.*m11*m45*RSp461e*RSp561e
          - 1./108.*m11*m45*pow(RSp461e,2) + 1./216.*m11*m44*pow(
         RSp563e,2) + 1./324.*m11*m44*RSp463e*RSp563e + 1./648.*m11*m44
         *pow(RSp463e,2) + 1./108.*m11*m44*RSp561e*RSp563e + 1./324.*
         m11*m44*RSp561e*RSp463e + 1./72.*m11*m44*pow(RSp561e,2) + 1./
         324.*m11*m44*RSp461e*RSp563e + 1./324.*m11*m44*RSp461e*RSp463e
          + 1./108.*m11*m44*RSp461e*RSp561e + 1./216.*m11*m44*pow(
         RSp461e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m46,L) * ( 1./162.*pow(mp2,2)*
         pow(RSp563e,2) - 1./81.*pow(mp2,2)*RSp453e*RSp563e + 1./162.*
         pow(mp2,2)*pow(RSp453e,2) - 1./81.*pow(mp2,2)*RSp561e*RSp563e
          + 1./81.*pow(mp2,2)*RSp561e*RSp453e + 1./162.*pow(mp2,2)*pow(
         RSp561e,2) + 1./81.*pow(mp2,2)*RSp451e*RSp563e - 1./81.*pow(
         mp2,2)*RSp451e*RSp453e - 1./81.*pow(mp2,2)*RSp451e*RSp561e + 1.
         /162.*pow(mp2,2)*pow(RSp451e,2) - 1./324.*m66*mp2*pow(
         RSp563e,2) - 1./162.*m66*mp2*RSp453e*RSp563e - 1./108.*m66*mp2
         *pow(RSp453e,2) + 1./162.*m66*mp2*RSp561e*RSp563e + 1./162.*
         m66*mp2*RSp561e*RSp453e - 1./324.*m66*mp2*pow(RSp561e,2) + 1./
         162.*m66*mp2*RSp451e*RSp563e + 1./54.*m66*mp2*RSp451e*RSp453e
          - 1./162.*m66*mp2*RSp451e*RSp561e - 1./108.*m66*mp2*pow(
         RSp451e,2) + 1./162.*m46*mp2*pow(RSp563e,2) - 1./81.*m46*mp2*
         RSp453e*RSp563e + 1./162.*m46*mp2*pow(RSp453e,2) - 1./81.*m46*
         mp2*RSp561e*RSp563e + 1./81.*m46*mp2*RSp561e*RSp453e + 1./162.
         *m46*mp2*pow(RSp561e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m46,L) * ( 1./81.*m46*mp2*
         RSp451e*RSp563e - 1./81.*m46*mp2*RSp451e*RSp453e - 1./81.*m46*
         mp2*RSp451e*RSp561e + 1./162.*m46*mp2*pow(RSp451e,2) - 1./108.
         *m44*mp2*pow(RSp563e,2) - 1./162.*m44*mp2*RSp453e*RSp563e - 1./
         324.*m44*mp2*pow(RSp453e,2) + 1./54.*m44*mp2*RSp561e*RSp563e
          + 1./162.*m44*mp2*RSp561e*RSp453e - 1./108.*m44*mp2*pow(
         RSp561e,2) + 1./162.*m44*mp2*RSp451e*RSp563e + 1./162.*m44*mp2
         *RSp451e*RSp453e - 1./162.*m44*mp2*RSp451e*RSp561e - 1./324.*
         m44*mp2*pow(RSp451e,2) - 1./108.*m33*mp2*pow(RSp563e,2) + 1./
         54.*m33*mp2*RSp453e*RSp563e - 1./108.*m33*mp2*pow(RSp453e,2)
          - 1./162.*m33*mp2*RSp561e*RSp563e + 1./162.*m33*mp2*RSp561e*
         RSp453e - 1./324.*m33*mp2*pow(RSp561e,2) + 1./162.*m33*mp2*
         RSp451e*RSp563e - 1./162.*m33*mp2*RSp451e*RSp453e + 1./162.*
         m33*mp2*RSp451e*RSp561e - 1./324.*m33*mp2*pow(RSp451e,2) + 1./
         216.*m33*m66*pow(RSp563e,2) + 1./108.*m33*m66*RSp453e*RSp563e
          + 1./72.*m33*m66*pow(RSp453e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m46,L) * ( 1./324.*m33*m66*
         RSp561e*RSp563e + 1./324.*m33*m66*RSp561e*RSp453e + 1./648.*
         m33*m66*pow(RSp561e,2) + 1./324.*m33*m66*RSp451e*RSp563e + 1./
         108.*m33*m66*RSp451e*RSp453e + 1./324.*m33*m66*RSp451e*RSp561e
          + 1./216.*m33*m66*pow(RSp451e,2) - 1./108.*m33*m46*pow(
         RSp563e,2) + 1./54.*m33*m46*RSp453e*RSp563e - 1./108.*m33*m46*
         pow(RSp453e,2) - 1./162.*m33*m46*RSp561e*RSp563e + 1./162.*m33
         *m46*RSp561e*RSp453e - 1./324.*m33*m46*pow(RSp561e,2) + 1./162.
         *m33*m46*RSp451e*RSp563e - 1./162.*m33*m46*RSp451e*RSp453e + 1.
         /162.*m33*m46*RSp451e*RSp561e - 1./324.*m33*m46*pow(RSp451e,2)
          + 1./72.*m33*m44*pow(RSp563e,2) + 1./108.*m33*m44*RSp453e*
         RSp563e + 1./216.*m33*m44*pow(RSp453e,2) + 1./108.*m33*m44*
         RSp561e*RSp563e + 1./324.*m33*m44*RSp561e*RSp453e + 1./216.*
         m33*m44*pow(RSp561e,2) + 1./324.*m33*m44*RSp451e*RSp563e + 1./
         324.*m33*m44*RSp451e*RSp453e + 1./324.*m33*m44*RSp451e*RSp561e
          + 1./648.*m33*m44*pow(RSp451e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m46,L) * ( 1./162.*m13*mp2*pow(
         RSp563e,2) - 1./81.*m13*mp2*RSp453e*RSp563e + 1./162.*m13*mp2*
         pow(RSp453e,2) - 1./81.*m13*mp2*RSp561e*RSp563e + 1./81.*m13*
         mp2*RSp561e*RSp453e + 1./162.*m13*mp2*pow(RSp561e,2) + 1./81.*
         m13*mp2*RSp451e*RSp563e - 1./81.*m13*mp2*RSp451e*RSp453e - 1./
         81.*m13*mp2*RSp451e*RSp561e + 1./162.*m13*mp2*pow(RSp451e,2)
          - 1./324.*m13*m66*pow(RSp563e,2) - 1./162.*m13*m66*RSp453e*
         RSp563e - 1./108.*m13*m66*pow(RSp453e,2) + 1./162.*m13*m66*
         RSp561e*RSp563e + 1./162.*m13*m66*RSp561e*RSp453e - 1./324.*
         m13*m66*pow(RSp561e,2) + 1./162.*m13*m66*RSp451e*RSp563e + 1./
         54.*m13*m66*RSp451e*RSp453e - 1./162.*m13*m66*RSp451e*RSp561e
          - 1./108.*m13*m66*pow(RSp451e,2) + 1./162.*m13*m46*pow(
         RSp563e,2) - 1./81.*m13*m46*RSp453e*RSp563e + 1./162.*m13*m46*
         pow(RSp453e,2) - 1./81.*m13*m46*RSp561e*RSp563e + 1./81.*m13*
         m46*RSp561e*RSp453e + 1./162.*m13*m46*pow(RSp561e,2) + 1./81.*
         m13*m46*RSp451e*RSp563e );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m46,L) * (  - 1./81.*m13*m46*
         RSp451e*RSp453e - 1./81.*m13*m46*RSp451e*RSp561e + 1./162.*m13
         *m46*pow(RSp451e,2) - 1./108.*m13*m44*pow(RSp563e,2) - 1./162.
         *m13*m44*RSp453e*RSp563e - 1./324.*m13*m44*pow(RSp453e,2) + 1./
         54.*m13*m44*RSp561e*RSp563e + 1./162.*m13*m44*RSp561e*RSp453e
          - 1./108.*m13*m44*pow(RSp561e,2) + 1./162.*m13*m44*RSp451e*
         RSp563e + 1./162.*m13*m44*RSp451e*RSp453e - 1./162.*m13*m44*
         RSp451e*RSp561e - 1./324.*m13*m44*pow(RSp451e,2) - 1./324.*m11
         *mp2*pow(RSp563e,2) + 1./162.*m11*mp2*RSp453e*RSp563e - 1./324.
         *m11*mp2*pow(RSp453e,2) - 1./162.*m11*mp2*RSp561e*RSp563e + 1./
         162.*m11*mp2*RSp561e*RSp453e - 1./108.*m11*mp2*pow(RSp561e,2)
          + 1./162.*m11*mp2*RSp451e*RSp563e - 1./162.*m11*mp2*RSp451e*
         RSp453e + 1./54.*m11*mp2*RSp451e*RSp561e - 1./108.*m11*mp2*
         pow(RSp451e,2) + 1./648.*m11*m66*pow(RSp563e,2) + 1./324.*m11*
         m66*RSp453e*RSp563e + 1./216.*m11*m66*pow(RSp453e,2) + 1./324.
         *m11*m66*RSp561e*RSp563e );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m46,L) * ( 1./324.*m11*m66*
         RSp561e*RSp453e + 1./216.*m11*m66*pow(RSp561e,2) + 1./324.*m11
         *m66*RSp451e*RSp563e + 1./108.*m11*m66*RSp451e*RSp453e + 1./
         108.*m11*m66*RSp451e*RSp561e + 1./72.*m11*m66*pow(RSp451e,2)
          - 1./324.*m11*m46*pow(RSp563e,2) + 1./162.*m11*m46*RSp453e*
         RSp563e - 1./324.*m11*m46*pow(RSp453e,2) - 1./162.*m11*m46*
         RSp561e*RSp563e + 1./162.*m11*m46*RSp561e*RSp453e - 1./108.*
         m11*m46*pow(RSp561e,2) + 1./162.*m11*m46*RSp451e*RSp563e - 1./
         162.*m11*m46*RSp451e*RSp453e + 1./54.*m11*m46*RSp451e*RSp561e
          - 1./108.*m11*m46*pow(RSp451e,2) + 1./216.*m11*m44*pow(
         RSp563e,2) + 1./324.*m11*m44*RSp453e*RSp563e + 1./648.*m11*m44
         *pow(RSp453e,2) + 1./108.*m11*m44*RSp561e*RSp563e + 1./324.*
         m11*m44*RSp561e*RSp453e + 1./72.*m11*m44*pow(RSp561e,2) + 1./
         324.*m11*m44*RSp451e*RSp563e + 1./324.*m11*m44*RSp451e*RSp453e
          + 1./108.*m11*m44*RSp451e*RSp561e + 1./216.*m11*m44*pow(
         RSp451e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m55,L) * (  - 1./54.*m55*mp2*
         pow(RSp463e,2) + 1./162.*m55*mp2*RS546pe*pow(RSp463e,2) + 1./
         27.*m55*mp2*RSp461e*RSp463e - 1./81.*m55*mp2*RSp461e*RS546pe*
         RSp463e - 1./54.*m55*mp2*pow(RSp461e,2) + 1./162.*m55*mp2*pow(
         RSp461e,2)*RS546pe + 1./36.*m33*m55*pow(RSp463e,2) - 1./108.*
         m33*m55*RS546pe*pow(RSp463e,2) + 1./54.*m33*m55*RSp461e*
         RSp463e - 1./162.*m33*m55*RSp461e*RS546pe*RSp463e + 1./108.*
         m33*m55*pow(RSp461e,2) - 1./324.*m33*m55*pow(RSp461e,2)*
         RS546pe - 1./54.*m13*m55*pow(RSp463e,2) + 1./162.*m13*m55*
         RS546pe*pow(RSp463e,2) + 1./27.*m13*m55*RSp461e*RSp463e - 1./
         81.*m13*m55*RSp461e*RS546pe*RSp463e - 1./54.*m13*m55*pow(
         RSp461e,2) + 1./162.*m13*m55*pow(RSp461e,2)*RS546pe + 1./108.*
         m11*m55*pow(RSp463e,2) - 1./324.*m11*m55*RS546pe*pow(
         RSp463e,2) + 1./54.*m11*m55*RSp461e*RSp463e - 1./162.*m11*m55*
         RSp461e*RS546pe*RSp463e + 1./36.*m11*m55*pow(RSp461e,2) - 1./
         108.*m11*m55*pow(RSp461e,2)*RS546pe );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m56,L) * ( 1./162.*pow(mp2,2)*
         pow(RSp463e,2) - 1./81.*pow(mp2,2)*RSp453e*RSp463e + 1./162.*
         pow(mp2,2)*pow(RSp453e,2) - 1./81.*pow(mp2,2)*RSp461e*RSp463e
          + 1./81.*pow(mp2,2)*RSp461e*RSp453e + 1./162.*pow(mp2,2)*pow(
         RSp461e,2) + 1./81.*pow(mp2,2)*RSp451e*RSp463e - 1./81.*pow(
         mp2,2)*RSp451e*RSp453e - 1./81.*pow(mp2,2)*RSp451e*RSp461e + 1.
         /162.*pow(mp2,2)*pow(RSp451e,2) - 1./324.*m66*mp2*pow(
         RSp463e,2) - 1./162.*m66*mp2*RSp453e*RSp463e - 1./108.*m66*mp2
         *pow(RSp453e,2) + 1./162.*m66*mp2*RSp461e*RSp463e + 1./162.*
         m66*mp2*RSp461e*RSp453e - 1./324.*m66*mp2*pow(RSp461e,2) + 1./
         162.*m66*mp2*RSp451e*RSp463e + 1./54.*m66*mp2*RSp451e*RSp453e
          - 1./162.*m66*mp2*RSp451e*RSp461e - 1./108.*m66*mp2*pow(
         RSp451e,2) + 1./162.*m56*mp2*pow(RSp463e,2) - 1./81.*m56*mp2*
         RSp453e*RSp463e + 1./162.*m56*mp2*pow(RSp453e,2) - 1./81.*m56*
         mp2*RSp461e*RSp463e + 1./81.*m56*mp2*RSp461e*RSp453e + 1./162.
         *m56*mp2*pow(RSp461e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m56,L) * ( 1./81.*m56*mp2*
         RSp451e*RSp463e - 1./81.*m56*mp2*RSp451e*RSp453e - 1./81.*m56*
         mp2*RSp451e*RSp461e + 1./162.*m56*mp2*pow(RSp451e,2) - 1./108.
         *m55*mp2*pow(RSp463e,2) - 1./162.*m55*mp2*RSp453e*RSp463e - 1./
         324.*m55*mp2*pow(RSp453e,2) + 1./54.*m55*mp2*RSp461e*RSp463e
          + 1./162.*m55*mp2*RSp461e*RSp453e - 1./108.*m55*mp2*pow(
         RSp461e,2) + 1./162.*m55*mp2*RSp451e*RSp463e + 1./162.*m55*mp2
         *RSp451e*RSp453e - 1./162.*m55*mp2*RSp451e*RSp461e - 1./324.*
         m55*mp2*pow(RSp451e,2) - 1./108.*m33*mp2*pow(RSp463e,2) + 1./
         54.*m33*mp2*RSp453e*RSp463e - 1./108.*m33*mp2*pow(RSp453e,2)
          - 1./162.*m33*mp2*RSp461e*RSp463e + 1./162.*m33*mp2*RSp461e*
         RSp453e - 1./324.*m33*mp2*pow(RSp461e,2) + 1./162.*m33*mp2*
         RSp451e*RSp463e - 1./162.*m33*mp2*RSp451e*RSp453e + 1./162.*
         m33*mp2*RSp451e*RSp461e - 1./324.*m33*mp2*pow(RSp451e,2) + 1./
         216.*m33*m66*pow(RSp463e,2) + 1./108.*m33*m66*RSp453e*RSp463e
          + 1./72.*m33*m66*pow(RSp453e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m56,L) * ( 1./324.*m33*m66*
         RSp461e*RSp463e + 1./324.*m33*m66*RSp461e*RSp453e + 1./648.*
         m33*m66*pow(RSp461e,2) + 1./324.*m33*m66*RSp451e*RSp463e + 1./
         108.*m33*m66*RSp451e*RSp453e + 1./324.*m33*m66*RSp451e*RSp461e
          + 1./216.*m33*m66*pow(RSp451e,2) - 1./108.*m33*m56*pow(
         RSp463e,2) + 1./54.*m33*m56*RSp453e*RSp463e - 1./108.*m33*m56*
         pow(RSp453e,2) - 1./162.*m33*m56*RSp461e*RSp463e + 1./162.*m33
         *m56*RSp461e*RSp453e - 1./324.*m33*m56*pow(RSp461e,2) + 1./162.
         *m33*m56*RSp451e*RSp463e - 1./162.*m33*m56*RSp451e*RSp453e + 1.
         /162.*m33*m56*RSp451e*RSp461e - 1./324.*m33*m56*pow(RSp451e,2)
          + 1./72.*m33*m55*pow(RSp463e,2) + 1./108.*m33*m55*RSp453e*
         RSp463e + 1./216.*m33*m55*pow(RSp453e,2) + 1./108.*m33*m55*
         RSp461e*RSp463e + 1./324.*m33*m55*RSp461e*RSp453e + 1./216.*
         m33*m55*pow(RSp461e,2) + 1./324.*m33*m55*RSp451e*RSp463e + 1./
         324.*m33*m55*RSp451e*RSp453e + 1./324.*m33*m55*RSp451e*RSp461e
          + 1./648.*m33*m55*pow(RSp451e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m56,L) * ( 1./162.*m13*mp2*pow(
         RSp463e,2) - 1./81.*m13*mp2*RSp453e*RSp463e + 1./162.*m13*mp2*
         pow(RSp453e,2) - 1./81.*m13*mp2*RSp461e*RSp463e + 1./81.*m13*
         mp2*RSp461e*RSp453e + 1./162.*m13*mp2*pow(RSp461e,2) + 1./81.*
         m13*mp2*RSp451e*RSp463e - 1./81.*m13*mp2*RSp451e*RSp453e - 1./
         81.*m13*mp2*RSp451e*RSp461e + 1./162.*m13*mp2*pow(RSp451e,2)
          - 1./324.*m13*m66*pow(RSp463e,2) - 1./162.*m13*m66*RSp453e*
         RSp463e - 1./108.*m13*m66*pow(RSp453e,2) + 1./162.*m13*m66*
         RSp461e*RSp463e + 1./162.*m13*m66*RSp461e*RSp453e - 1./324.*
         m13*m66*pow(RSp461e,2) + 1./162.*m13*m66*RSp451e*RSp463e + 1./
         54.*m13*m66*RSp451e*RSp453e - 1./162.*m13*m66*RSp451e*RSp461e
          - 1./108.*m13*m66*pow(RSp451e,2) + 1./162.*m13*m56*pow(
         RSp463e,2) - 1./81.*m13*m56*RSp453e*RSp463e + 1./162.*m13*m56*
         pow(RSp453e,2) - 1./81.*m13*m56*RSp461e*RSp463e + 1./81.*m13*
         m56*RSp461e*RSp453e + 1./162.*m13*m56*pow(RSp461e,2) + 1./81.*
         m13*m56*RSp451e*RSp463e );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m56,L) * (  - 1./81.*m13*m56*
         RSp451e*RSp453e - 1./81.*m13*m56*RSp451e*RSp461e + 1./162.*m13
         *m56*pow(RSp451e,2) - 1./108.*m13*m55*pow(RSp463e,2) - 1./162.
         *m13*m55*RSp453e*RSp463e - 1./324.*m13*m55*pow(RSp453e,2) + 1./
         54.*m13*m55*RSp461e*RSp463e + 1./162.*m13*m55*RSp461e*RSp453e
          - 1./108.*m13*m55*pow(RSp461e,2) + 1./162.*m13*m55*RSp451e*
         RSp463e + 1./162.*m13*m55*RSp451e*RSp453e - 1./162.*m13*m55*
         RSp451e*RSp461e - 1./324.*m13*m55*pow(RSp451e,2) - 1./324.*m11
         *mp2*pow(RSp463e,2) + 1./162.*m11*mp2*RSp453e*RSp463e - 1./324.
         *m11*mp2*pow(RSp453e,2) - 1./162.*m11*mp2*RSp461e*RSp463e + 1./
         162.*m11*mp2*RSp461e*RSp453e - 1./108.*m11*mp2*pow(RSp461e,2)
          + 1./162.*m11*mp2*RSp451e*RSp463e - 1./162.*m11*mp2*RSp451e*
         RSp453e + 1./54.*m11*mp2*RSp451e*RSp461e - 1./108.*m11*mp2*
         pow(RSp451e,2) + 1./648.*m11*m66*pow(RSp463e,2) + 1./324.*m11*
         m66*RSp453e*RSp463e + 1./216.*m11*m66*pow(RSp453e,2) + 1./324.
         *m11*m66*RSp461e*RSp463e );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m56,L) * ( 1./324.*m11*m66*
         RSp461e*RSp453e + 1./216.*m11*m66*pow(RSp461e,2) + 1./324.*m11
         *m66*RSp451e*RSp463e + 1./108.*m11*m66*RSp451e*RSp453e + 1./
         108.*m11*m66*RSp451e*RSp461e + 1./72.*m11*m66*pow(RSp451e,2)
          - 1./324.*m11*m56*pow(RSp463e,2) + 1./162.*m11*m56*RSp453e*
         RSp463e - 1./324.*m11*m56*pow(RSp453e,2) - 1./162.*m11*m56*
         RSp461e*RSp463e + 1./162.*m11*m56*RSp461e*RSp453e - 1./108.*
         m11*m56*pow(RSp461e,2) + 1./162.*m11*m56*RSp451e*RSp463e - 1./
         162.*m11*m56*RSp451e*RSp453e + 1./54.*m11*m56*RSp451e*RSp461e
          - 1./108.*m11*m56*pow(RSp451e,2) + 1./216.*m11*m55*pow(
         RSp463e,2) + 1./324.*m11*m55*RSp453e*RSp463e + 1./648.*m11*m55
         *pow(RSp453e,2) + 1./108.*m11*m55*RSp461e*RSp463e + 1./324.*
         m11*m55*RSp461e*RSp453e + 1./72.*m11*m55*pow(RSp461e,2) + 1./
         324.*m11*m55*RSp451e*RSp463e + 1./324.*m11*m55*RSp451e*RSp453e
          + 1./108.*m11*m55*RSp451e*RSp461e + 1./216.*m11*m55*pow(
         RSp451e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(m66,L) * (  - 1./54.*m66*mp2*
         pow(RSp453e,2) + 1./162.*m66*mp2*RS645pe*pow(RSp453e,2) + 1./
         27.*m66*mp2*RSp451e*RSp453e - 1./81.*m66*mp2*RSp451e*RS645pe*
         RSp453e - 1./54.*m66*mp2*pow(RSp451e,2) + 1./162.*m66*mp2*pow(
         RSp451e,2)*RS645pe + 1./36.*m33*m66*pow(RSp453e,2) - 1./108.*
         m33*m66*RS645pe*pow(RSp453e,2) + 1./54.*m33*m66*RSp451e*
         RSp453e - 1./162.*m33*m66*RSp451e*RS645pe*RSp453e + 1./108.*
         m33*m66*pow(RSp451e,2) - 1./324.*m33*m66*pow(RSp451e,2)*
         RS645pe - 1./54.*m13*m66*pow(RSp453e,2) + 1./162.*m13*m66*
         RS645pe*pow(RSp453e,2) + 1./27.*m13*m66*RSp451e*RSp453e - 1./
         81.*m13*m66*RSp451e*RS645pe*RSp453e - 1./54.*m13*m66*pow(
         RSp451e,2) + 1./162.*m13*m66*pow(RSp451e,2)*RS645pe + 1./108.*
         m11*m66*pow(RSp453e,2) - 1./324.*m11*m66*RS645pe*pow(
         RSp453e,2) + 1./54.*m11*m66*RSp451e*RSp453e - 1./162.*m11*m66*
         RSp451e*RS645pe*RSp453e + 1./36.*m11*m66*pow(RSp451e,2) - 1./
         108.*m11*m66*pow(RSp451e,2)*RS645pe );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(mp2,L) * ( 1./162.*m66*mp2*
         RSp456e*pow(RSp453e,2) - 1./81.*m66*mp2*RSp456e*RSp451e*
         RSp453e + 1./162.*m66*mp2*RSp456e*pow(RSp451e,2) + 1./162.*m55
         *mp2*RSp465e*pow(RSp463e,2) - 1./81.*m55*mp2*RSp465e*RSp461e*
         RSp463e + 1./162.*m55*mp2*RSp465e*pow(RSp461e,2) + 1./162.*m44
         *mp2*RSp564e*pow(RSp563e,2) - 1./81.*m44*mp2*RSp564e*RSp561e*
         RSp563e + 1./162.*m44*mp2*RSp564e*pow(RSp561e,2) - 1./108.*m33
         *m66*RSp456e*pow(RSp453e,2) - 1./162.*m33*m66*RSp456e*RSp451e*
         RSp453e - 1./324.*m33*m66*RSp456e*pow(RSp451e,2) - 1./108.*m33
         *m55*RSp465e*pow(RSp463e,2) - 1./162.*m33*m55*RSp465e*RSp461e*
         RSp463e - 1./324.*m33*m55*RSp465e*pow(RSp461e,2) - 1./108.*m33
         *m44*RSp564e*pow(RSp563e,2) - 1./162.*m33*m44*RSp564e*RSp561e*
         RSp563e - 1./324.*m33*m44*RSp564e*pow(RSp561e,2) + 1./162.*m13
         *m66*RSp456e*pow(RSp453e,2) - 1./81.*m13*m66*RSp456e*RSp451e*
         RSp453e + 1./162.*m13*m66*RSp456e*pow(RSp451e,2) + 1./162.*m13
         *m55*RSp465e*pow(RSp463e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(mp2,L) * (  - 1./81.*m13*m55*
         RSp465e*RSp461e*RSp463e + 1./162.*m13*m55*RSp465e*pow(
         RSp461e,2) + 1./162.*m13*m44*RSp564e*pow(RSp563e,2) - 1./81.*
         m13*m44*RSp564e*RSp561e*RSp563e + 1./162.*m13*m44*RSp564e*pow(
         RSp561e,2) - 1./324.*m11*m66*RSp456e*pow(RSp453e,2) - 1./162.*
         m11*m66*RSp456e*RSp451e*RSp453e - 1./108.*m11*m66*RSp456e*pow(
         RSp451e,2) - 1./324.*m11*m55*RSp465e*pow(RSp463e,2) - 1./162.*
         m11*m55*RSp465e*RSp461e*RSp463e - 1./108.*m11*m55*RSp465e*pow(
         RSp461e,2) - 1./324.*m11*m44*RSp564e*pow(RSp563e,2) - 1./162.*
         m11*m44*RSp564e*RSp561e*RSp563e - 1./108.*m11*m44*RSp564e*pow(
         RSp561e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(me2,L) * ( 1./162.*m66*mp2*
         RSe456p*pow(RSp453e,2) - 1./81.*m66*mp2*RSe456p*RSp451e*
         RSp453e + 1./162.*m66*mp2*RSe456p*pow(RSp451e,2) + 1./162.*m55
         *mp2*RSe465p*pow(RSp463e,2) - 1./81.*m55*mp2*RSe465p*RSp461e*
         RSp463e + 1./162.*m55*mp2*RSe465p*pow(RSp461e,2) + 1./162.*m44
         *mp2*RSe564p*pow(RSp563e,2) - 1./81.*m44*mp2*RSe564p*RSp561e*
         RSp563e + 1./162.*m44*mp2*RSe564p*pow(RSp561e,2) - 1./108.*m33
         *m66*RSe456p*pow(RSp453e,2) - 1./162.*m33*m66*RSe456p*RSp451e*
         RSp453e - 1./324.*m33*m66*RSe456p*pow(RSp451e,2) - 1./108.*m33
         *m55*RSe465p*pow(RSp463e,2) - 1./162.*m33*m55*RSe465p*RSp461e*
         RSp463e - 1./324.*m33*m55*RSe465p*pow(RSp461e,2) - 1./108.*m33
         *m44*RSe564p*pow(RSp563e,2) - 1./162.*m33*m44*RSe564p*RSp561e*
         RSp563e - 1./324.*m33*m44*RSe564p*pow(RSp561e,2) + 1./162.*m13
         *m66*RSe456p*pow(RSp453e,2) - 1./81.*m13*m66*RSe456p*RSp451e*
         RSp453e + 1./162.*m13*m66*RSe456p*pow(RSp451e,2) + 1./162.*m13
         *m55*RSe465p*pow(RSp463e,2) );

      M13p6x23RV +=  + Bb(mp2,mu2)*AbV(me2,L) * (  - 1./81.*m13*m55*
         RSe465p*RSp461e*RSp463e + 1./162.*m13*m55*RSe465p*pow(
         RSp461e,2) + 1./162.*m13*m44*RSe564p*pow(RSp563e,2) - 1./81.*
         m13*m44*RSe564p*RSp561e*RSp563e + 1./162.*m13*m44*RSe564p*pow(
         RSp561e,2) - 1./324.*m11*m66*RSe456p*pow(RSp453e,2) - 1./162.*
         m11*m66*RSe456p*RSp451e*RSp453e - 1./108.*m11*m66*RSe456p*pow(
         RSp451e,2) - 1./324.*m11*m55*RSe465p*pow(RSp463e,2) - 1./162.*
         m11*m55*RSe465p*RSp461e*RSp463e - 1./108.*m11*m55*RSe465p*pow(
         RSp461e,2) - 1./324.*m11*m44*RSe564p*pow(RSp563e,2) - 1./162.*
         m11*m44*RSe564p*RSp561e*RSp563e - 1./108.*m11*m44*RSe564p*pow(
         RSp561e,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m44,L) * (  - 1./54.*m44*me2*
         pow(RSe563p,2) + 1./162.*m44*me2*RS456pe*pow(RSe563p,2) + 1./
         27.*m44*me2*RSe561p*RSe563p - 1./81.*m44*me2*RSe561p*RS456pe*
         RSe563p - 1./54.*m44*me2*pow(RSe561p,2) + 1./162.*m44*me2*pow(
         RSe561p,2)*RS456pe + 1./36.*m33*m44*pow(RSe563p,2) - 1./108.*
         m33*m44*RS456pe*pow(RSe563p,2) + 1./54.*m33*m44*RSe561p*
         RSe563p - 1./162.*m33*m44*RSe561p*RS456pe*RSe563p + 1./108.*
         m33*m44*pow(RSe561p,2) - 1./324.*m33*m44*pow(RSe561p,2)*
         RS456pe - 1./54.*m13*m44*pow(RSe563p,2) + 1./162.*m13*m44*
         RS456pe*pow(RSe563p,2) + 1./27.*m13*m44*RSe561p*RSe563p - 1./
         81.*m13*m44*RSe561p*RS456pe*RSe563p - 1./54.*m13*m44*pow(
         RSe561p,2) + 1./162.*m13*m44*pow(RSe561p,2)*RS456pe + 1./108.*
         m11*m44*pow(RSe563p,2) - 1./324.*m11*m44*RS456pe*pow(
         RSe563p,2) + 1./54.*m11*m44*RSe561p*RSe563p - 1./162.*m11*m44*
         RSe561p*RS456pe*RSe563p + 1./36.*m11*m44*pow(RSe561p,2) - 1./
         108.*m11*m44*pow(RSe561p,2)*RS456pe );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m45,L) * ( 1./162.*pow(me2,2)*
         pow(RSe563p,2) - 1./81.*pow(me2,2)*RSe463p*RSe563p + 1./162.*
         pow(me2,2)*pow(RSe463p,2) - 1./81.*pow(me2,2)*RSe561p*RSe563p
          + 1./81.*pow(me2,2)*RSe561p*RSe463p + 1./162.*pow(me2,2)*pow(
         RSe561p,2) + 1./81.*pow(me2,2)*RSe461p*RSe563p - 1./81.*pow(
         me2,2)*RSe461p*RSe463p - 1./81.*pow(me2,2)*RSe461p*RSe561p + 1.
         /162.*pow(me2,2)*pow(RSe461p,2) - 1./324.*m55*me2*pow(
         RSe563p,2) - 1./162.*m55*me2*RSe463p*RSe563p - 1./108.*m55*me2
         *pow(RSe463p,2) + 1./162.*m55*me2*RSe561p*RSe563p + 1./162.*
         m55*me2*RSe561p*RSe463p - 1./324.*m55*me2*pow(RSe561p,2) + 1./
         162.*m55*me2*RSe461p*RSe563p + 1./54.*m55*me2*RSe461p*RSe463p
          - 1./162.*m55*me2*RSe461p*RSe561p - 1./108.*m55*me2*pow(
         RSe461p,2) + 1./162.*m45*me2*pow(RSe563p,2) - 1./81.*m45*me2*
         RSe463p*RSe563p + 1./162.*m45*me2*pow(RSe463p,2) - 1./81.*m45*
         me2*RSe561p*RSe563p + 1./81.*m45*me2*RSe561p*RSe463p + 1./162.
         *m45*me2*pow(RSe561p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m45,L) * ( 1./81.*m45*me2*
         RSe461p*RSe563p - 1./81.*m45*me2*RSe461p*RSe463p - 1./81.*m45*
         me2*RSe461p*RSe561p + 1./162.*m45*me2*pow(RSe461p,2) - 1./108.
         *m44*me2*pow(RSe563p,2) - 1./162.*m44*me2*RSe463p*RSe563p - 1./
         324.*m44*me2*pow(RSe463p,2) + 1./54.*m44*me2*RSe561p*RSe563p
          + 1./162.*m44*me2*RSe561p*RSe463p - 1./108.*m44*me2*pow(
         RSe561p,2) + 1./162.*m44*me2*RSe461p*RSe563p + 1./162.*m44*me2
         *RSe461p*RSe463p - 1./162.*m44*me2*RSe461p*RSe561p - 1./324.*
         m44*me2*pow(RSe461p,2) - 1./108.*m33*me2*pow(RSe563p,2) + 1./
         54.*m33*me2*RSe463p*RSe563p - 1./108.*m33*me2*pow(RSe463p,2)
          - 1./162.*m33*me2*RSe561p*RSe563p + 1./162.*m33*me2*RSe561p*
         RSe463p - 1./324.*m33*me2*pow(RSe561p,2) + 1./162.*m33*me2*
         RSe461p*RSe563p - 1./162.*m33*me2*RSe461p*RSe463p + 1./162.*
         m33*me2*RSe461p*RSe561p - 1./324.*m33*me2*pow(RSe461p,2) + 1./
         216.*m33*m55*pow(RSe563p,2) + 1./108.*m33*m55*RSe463p*RSe563p
          + 1./72.*m33*m55*pow(RSe463p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m45,L) * ( 1./324.*m33*m55*
         RSe561p*RSe563p + 1./324.*m33*m55*RSe561p*RSe463p + 1./648.*
         m33*m55*pow(RSe561p,2) + 1./324.*m33*m55*RSe461p*RSe563p + 1./
         108.*m33*m55*RSe461p*RSe463p + 1./324.*m33*m55*RSe461p*RSe561p
          + 1./216.*m33*m55*pow(RSe461p,2) - 1./108.*m33*m45*pow(
         RSe563p,2) + 1./54.*m33*m45*RSe463p*RSe563p - 1./108.*m33*m45*
         pow(RSe463p,2) - 1./162.*m33*m45*RSe561p*RSe563p + 1./162.*m33
         *m45*RSe561p*RSe463p - 1./324.*m33*m45*pow(RSe561p,2) + 1./162.
         *m33*m45*RSe461p*RSe563p - 1./162.*m33*m45*RSe461p*RSe463p + 1.
         /162.*m33*m45*RSe461p*RSe561p - 1./324.*m33*m45*pow(RSe461p,2)
          + 1./72.*m33*m44*pow(RSe563p,2) + 1./108.*m33*m44*RSe463p*
         RSe563p + 1./216.*m33*m44*pow(RSe463p,2) + 1./108.*m33*m44*
         RSe561p*RSe563p + 1./324.*m33*m44*RSe561p*RSe463p + 1./216.*
         m33*m44*pow(RSe561p,2) + 1./324.*m33*m44*RSe461p*RSe563p + 1./
         324.*m33*m44*RSe461p*RSe463p + 1./324.*m33*m44*RSe461p*RSe561p
          + 1./648.*m33*m44*pow(RSe461p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m45,L) * ( 1./162.*m13*me2*pow(
         RSe563p,2) - 1./81.*m13*me2*RSe463p*RSe563p + 1./162.*m13*me2*
         pow(RSe463p,2) - 1./81.*m13*me2*RSe561p*RSe563p + 1./81.*m13*
         me2*RSe561p*RSe463p + 1./162.*m13*me2*pow(RSe561p,2) + 1./81.*
         m13*me2*RSe461p*RSe563p - 1./81.*m13*me2*RSe461p*RSe463p - 1./
         81.*m13*me2*RSe461p*RSe561p + 1./162.*m13*me2*pow(RSe461p,2)
          - 1./324.*m13*m55*pow(RSe563p,2) - 1./162.*m13*m55*RSe463p*
         RSe563p - 1./108.*m13*m55*pow(RSe463p,2) + 1./162.*m13*m55*
         RSe561p*RSe563p + 1./162.*m13*m55*RSe561p*RSe463p - 1./324.*
         m13*m55*pow(RSe561p,2) + 1./162.*m13*m55*RSe461p*RSe563p + 1./
         54.*m13*m55*RSe461p*RSe463p - 1./162.*m13*m55*RSe461p*RSe561p
          - 1./108.*m13*m55*pow(RSe461p,2) + 1./162.*m13*m45*pow(
         RSe563p,2) - 1./81.*m13*m45*RSe463p*RSe563p + 1./162.*m13*m45*
         pow(RSe463p,2) - 1./81.*m13*m45*RSe561p*RSe563p + 1./81.*m13*
         m45*RSe561p*RSe463p + 1./162.*m13*m45*pow(RSe561p,2) + 1./81.*
         m13*m45*RSe461p*RSe563p );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m45,L) * (  - 1./81.*m13*m45*
         RSe461p*RSe463p - 1./81.*m13*m45*RSe461p*RSe561p + 1./162.*m13
         *m45*pow(RSe461p,2) - 1./108.*m13*m44*pow(RSe563p,2) - 1./162.
         *m13*m44*RSe463p*RSe563p - 1./324.*m13*m44*pow(RSe463p,2) + 1./
         54.*m13*m44*RSe561p*RSe563p + 1./162.*m13*m44*RSe561p*RSe463p
          - 1./108.*m13*m44*pow(RSe561p,2) + 1./162.*m13*m44*RSe461p*
         RSe563p + 1./162.*m13*m44*RSe461p*RSe463p - 1./162.*m13*m44*
         RSe461p*RSe561p - 1./324.*m13*m44*pow(RSe461p,2) - 1./324.*m11
         *me2*pow(RSe563p,2) + 1./162.*m11*me2*RSe463p*RSe563p - 1./324.
         *m11*me2*pow(RSe463p,2) - 1./162.*m11*me2*RSe561p*RSe563p + 1./
         162.*m11*me2*RSe561p*RSe463p - 1./108.*m11*me2*pow(RSe561p,2)
          + 1./162.*m11*me2*RSe461p*RSe563p - 1./162.*m11*me2*RSe461p*
         RSe463p + 1./54.*m11*me2*RSe461p*RSe561p - 1./108.*m11*me2*
         pow(RSe461p,2) + 1./648.*m11*m55*pow(RSe563p,2) + 1./324.*m11*
         m55*RSe463p*RSe563p + 1./216.*m11*m55*pow(RSe463p,2) + 1./324.
         *m11*m55*RSe561p*RSe563p );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m45,L) * ( 1./324.*m11*m55*
         RSe561p*RSe463p + 1./216.*m11*m55*pow(RSe561p,2) + 1./324.*m11
         *m55*RSe461p*RSe563p + 1./108.*m11*m55*RSe461p*RSe463p + 1./
         108.*m11*m55*RSe461p*RSe561p + 1./72.*m11*m55*pow(RSe461p,2)
          - 1./324.*m11*m45*pow(RSe563p,2) + 1./162.*m11*m45*RSe463p*
         RSe563p - 1./324.*m11*m45*pow(RSe463p,2) - 1./162.*m11*m45*
         RSe561p*RSe563p + 1./162.*m11*m45*RSe561p*RSe463p - 1./108.*
         m11*m45*pow(RSe561p,2) + 1./162.*m11*m45*RSe461p*RSe563p - 1./
         162.*m11*m45*RSe461p*RSe463p + 1./54.*m11*m45*RSe461p*RSe561p
          - 1./108.*m11*m45*pow(RSe461p,2) + 1./216.*m11*m44*pow(
         RSe563p,2) + 1./324.*m11*m44*RSe463p*RSe563p + 1./648.*m11*m44
         *pow(RSe463p,2) + 1./108.*m11*m44*RSe561p*RSe563p + 1./324.*
         m11*m44*RSe561p*RSe463p + 1./72.*m11*m44*pow(RSe561p,2) + 1./
         324.*m11*m44*RSe461p*RSe563p + 1./324.*m11*m44*RSe461p*RSe463p
          + 1./108.*m11*m44*RSe461p*RSe561p + 1./216.*m11*m44*pow(
         RSe461p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m46,L) * ( 1./162.*pow(me2,2)*
         pow(RSe563p,2) - 1./81.*pow(me2,2)*RSe453p*RSe563p + 1./162.*
         pow(me2,2)*pow(RSe453p,2) - 1./81.*pow(me2,2)*RSe561p*RSe563p
          + 1./81.*pow(me2,2)*RSe561p*RSe453p + 1./162.*pow(me2,2)*pow(
         RSe561p,2) + 1./81.*pow(me2,2)*RSe451p*RSe563p - 1./81.*pow(
         me2,2)*RSe451p*RSe453p - 1./81.*pow(me2,2)*RSe451p*RSe561p + 1.
         /162.*pow(me2,2)*pow(RSe451p,2) - 1./324.*m66*me2*pow(
         RSe563p,2) - 1./162.*m66*me2*RSe453p*RSe563p - 1./108.*m66*me2
         *pow(RSe453p,2) + 1./162.*m66*me2*RSe561p*RSe563p + 1./162.*
         m66*me2*RSe561p*RSe453p - 1./324.*m66*me2*pow(RSe561p,2) + 1./
         162.*m66*me2*RSe451p*RSe563p + 1./54.*m66*me2*RSe451p*RSe453p
          - 1./162.*m66*me2*RSe451p*RSe561p - 1./108.*m66*me2*pow(
         RSe451p,2) + 1./162.*m46*me2*pow(RSe563p,2) - 1./81.*m46*me2*
         RSe453p*RSe563p + 1./162.*m46*me2*pow(RSe453p,2) - 1./81.*m46*
         me2*RSe561p*RSe563p + 1./81.*m46*me2*RSe561p*RSe453p + 1./162.
         *m46*me2*pow(RSe561p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m46,L) * ( 1./81.*m46*me2*
         RSe451p*RSe563p - 1./81.*m46*me2*RSe451p*RSe453p - 1./81.*m46*
         me2*RSe451p*RSe561p + 1./162.*m46*me2*pow(RSe451p,2) - 1./108.
         *m44*me2*pow(RSe563p,2) - 1./162.*m44*me2*RSe453p*RSe563p - 1./
         324.*m44*me2*pow(RSe453p,2) + 1./54.*m44*me2*RSe561p*RSe563p
          + 1./162.*m44*me2*RSe561p*RSe453p - 1./108.*m44*me2*pow(
         RSe561p,2) + 1./162.*m44*me2*RSe451p*RSe563p + 1./162.*m44*me2
         *RSe451p*RSe453p - 1./162.*m44*me2*RSe451p*RSe561p - 1./324.*
         m44*me2*pow(RSe451p,2) - 1./108.*m33*me2*pow(RSe563p,2) + 1./
         54.*m33*me2*RSe453p*RSe563p - 1./108.*m33*me2*pow(RSe453p,2)
          - 1./162.*m33*me2*RSe561p*RSe563p + 1./162.*m33*me2*RSe561p*
         RSe453p - 1./324.*m33*me2*pow(RSe561p,2) + 1./162.*m33*me2*
         RSe451p*RSe563p - 1./162.*m33*me2*RSe451p*RSe453p + 1./162.*
         m33*me2*RSe451p*RSe561p - 1./324.*m33*me2*pow(RSe451p,2) + 1./
         216.*m33*m66*pow(RSe563p,2) + 1./108.*m33*m66*RSe453p*RSe563p
          + 1./72.*m33*m66*pow(RSe453p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m46,L) * ( 1./324.*m33*m66*
         RSe561p*RSe563p + 1./324.*m33*m66*RSe561p*RSe453p + 1./648.*
         m33*m66*pow(RSe561p,2) + 1./324.*m33*m66*RSe451p*RSe563p + 1./
         108.*m33*m66*RSe451p*RSe453p + 1./324.*m33*m66*RSe451p*RSe561p
          + 1./216.*m33*m66*pow(RSe451p,2) - 1./108.*m33*m46*pow(
         RSe563p,2) + 1./54.*m33*m46*RSe453p*RSe563p - 1./108.*m33*m46*
         pow(RSe453p,2) - 1./162.*m33*m46*RSe561p*RSe563p + 1./162.*m33
         *m46*RSe561p*RSe453p - 1./324.*m33*m46*pow(RSe561p,2) + 1./162.
         *m33*m46*RSe451p*RSe563p - 1./162.*m33*m46*RSe451p*RSe453p + 1.
         /162.*m33*m46*RSe451p*RSe561p - 1./324.*m33*m46*pow(RSe451p,2)
          + 1./72.*m33*m44*pow(RSe563p,2) + 1./108.*m33*m44*RSe453p*
         RSe563p + 1./216.*m33*m44*pow(RSe453p,2) + 1./108.*m33*m44*
         RSe561p*RSe563p + 1./324.*m33*m44*RSe561p*RSe453p + 1./216.*
         m33*m44*pow(RSe561p,2) + 1./324.*m33*m44*RSe451p*RSe563p + 1./
         324.*m33*m44*RSe451p*RSe453p + 1./324.*m33*m44*RSe451p*RSe561p
          + 1./648.*m33*m44*pow(RSe451p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m46,L) * ( 1./162.*m13*me2*pow(
         RSe563p,2) - 1./81.*m13*me2*RSe453p*RSe563p + 1./162.*m13*me2*
         pow(RSe453p,2) - 1./81.*m13*me2*RSe561p*RSe563p + 1./81.*m13*
         me2*RSe561p*RSe453p + 1./162.*m13*me2*pow(RSe561p,2) + 1./81.*
         m13*me2*RSe451p*RSe563p - 1./81.*m13*me2*RSe451p*RSe453p - 1./
         81.*m13*me2*RSe451p*RSe561p + 1./162.*m13*me2*pow(RSe451p,2)
          - 1./324.*m13*m66*pow(RSe563p,2) - 1./162.*m13*m66*RSe453p*
         RSe563p - 1./108.*m13*m66*pow(RSe453p,2) + 1./162.*m13*m66*
         RSe561p*RSe563p + 1./162.*m13*m66*RSe561p*RSe453p - 1./324.*
         m13*m66*pow(RSe561p,2) + 1./162.*m13*m66*RSe451p*RSe563p + 1./
         54.*m13*m66*RSe451p*RSe453p - 1./162.*m13*m66*RSe451p*RSe561p
          - 1./108.*m13*m66*pow(RSe451p,2) + 1./162.*m13*m46*pow(
         RSe563p,2) - 1./81.*m13*m46*RSe453p*RSe563p + 1./162.*m13*m46*
         pow(RSe453p,2) - 1./81.*m13*m46*RSe561p*RSe563p + 1./81.*m13*
         m46*RSe561p*RSe453p + 1./162.*m13*m46*pow(RSe561p,2) + 1./81.*
         m13*m46*RSe451p*RSe563p );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m46,L) * (  - 1./81.*m13*m46*
         RSe451p*RSe453p - 1./81.*m13*m46*RSe451p*RSe561p + 1./162.*m13
         *m46*pow(RSe451p,2) - 1./108.*m13*m44*pow(RSe563p,2) - 1./162.
         *m13*m44*RSe453p*RSe563p - 1./324.*m13*m44*pow(RSe453p,2) + 1./
         54.*m13*m44*RSe561p*RSe563p + 1./162.*m13*m44*RSe561p*RSe453p
          - 1./108.*m13*m44*pow(RSe561p,2) + 1./162.*m13*m44*RSe451p*
         RSe563p + 1./162.*m13*m44*RSe451p*RSe453p - 1./162.*m13*m44*
         RSe451p*RSe561p - 1./324.*m13*m44*pow(RSe451p,2) - 1./324.*m11
         *me2*pow(RSe563p,2) + 1./162.*m11*me2*RSe453p*RSe563p - 1./324.
         *m11*me2*pow(RSe453p,2) - 1./162.*m11*me2*RSe561p*RSe563p + 1./
         162.*m11*me2*RSe561p*RSe453p - 1./108.*m11*me2*pow(RSe561p,2)
          + 1./162.*m11*me2*RSe451p*RSe563p - 1./162.*m11*me2*RSe451p*
         RSe453p + 1./54.*m11*me2*RSe451p*RSe561p - 1./108.*m11*me2*
         pow(RSe451p,2) + 1./648.*m11*m66*pow(RSe563p,2) + 1./324.*m11*
         m66*RSe453p*RSe563p + 1./216.*m11*m66*pow(RSe453p,2) + 1./324.
         *m11*m66*RSe561p*RSe563p );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m46,L) * ( 1./324.*m11*m66*
         RSe561p*RSe453p + 1./216.*m11*m66*pow(RSe561p,2) + 1./324.*m11
         *m66*RSe451p*RSe563p + 1./108.*m11*m66*RSe451p*RSe453p + 1./
         108.*m11*m66*RSe451p*RSe561p + 1./72.*m11*m66*pow(RSe451p,2)
          - 1./324.*m11*m46*pow(RSe563p,2) + 1./162.*m11*m46*RSe453p*
         RSe563p - 1./324.*m11*m46*pow(RSe453p,2) - 1./162.*m11*m46*
         RSe561p*RSe563p + 1./162.*m11*m46*RSe561p*RSe453p - 1./108.*
         m11*m46*pow(RSe561p,2) + 1./162.*m11*m46*RSe451p*RSe563p - 1./
         162.*m11*m46*RSe451p*RSe453p + 1./54.*m11*m46*RSe451p*RSe561p
          - 1./108.*m11*m46*pow(RSe451p,2) + 1./216.*m11*m44*pow(
         RSe563p,2) + 1./324.*m11*m44*RSe453p*RSe563p + 1./648.*m11*m44
         *pow(RSe453p,2) + 1./108.*m11*m44*RSe561p*RSe563p + 1./324.*
         m11*m44*RSe561p*RSe453p + 1./72.*m11*m44*pow(RSe561p,2) + 1./
         324.*m11*m44*RSe451p*RSe563p + 1./324.*m11*m44*RSe451p*RSe453p
          + 1./108.*m11*m44*RSe451p*RSe561p + 1./216.*m11*m44*pow(
         RSe451p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m55,L) * (  - 1./54.*m55*me2*
         pow(RSe463p,2) + 1./162.*m55*me2*RS546pe*pow(RSe463p,2) + 1./
         27.*m55*me2*RSe461p*RSe463p - 1./81.*m55*me2*RSe461p*RS546pe*
         RSe463p - 1./54.*m55*me2*pow(RSe461p,2) + 1./162.*m55*me2*pow(
         RSe461p,2)*RS546pe + 1./36.*m33*m55*pow(RSe463p,2) - 1./108.*
         m33*m55*RS546pe*pow(RSe463p,2) + 1./54.*m33*m55*RSe461p*
         RSe463p - 1./162.*m33*m55*RSe461p*RS546pe*RSe463p + 1./108.*
         m33*m55*pow(RSe461p,2) - 1./324.*m33*m55*pow(RSe461p,2)*
         RS546pe - 1./54.*m13*m55*pow(RSe463p,2) + 1./162.*m13*m55*
         RS546pe*pow(RSe463p,2) + 1./27.*m13*m55*RSe461p*RSe463p - 1./
         81.*m13*m55*RSe461p*RS546pe*RSe463p - 1./54.*m13*m55*pow(
         RSe461p,2) + 1./162.*m13*m55*pow(RSe461p,2)*RS546pe + 1./108.*
         m11*m55*pow(RSe463p,2) - 1./324.*m11*m55*RS546pe*pow(
         RSe463p,2) + 1./54.*m11*m55*RSe461p*RSe463p - 1./162.*m11*m55*
         RSe461p*RS546pe*RSe463p + 1./36.*m11*m55*pow(RSe461p,2) - 1./
         108.*m11*m55*pow(RSe461p,2)*RS546pe );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m56,L) * ( 1./162.*pow(me2,2)*
         pow(RSe463p,2) - 1./81.*pow(me2,2)*RSe453p*RSe463p + 1./162.*
         pow(me2,2)*pow(RSe453p,2) - 1./81.*pow(me2,2)*RSe461p*RSe463p
          + 1./81.*pow(me2,2)*RSe461p*RSe453p + 1./162.*pow(me2,2)*pow(
         RSe461p,2) + 1./81.*pow(me2,2)*RSe451p*RSe463p - 1./81.*pow(
         me2,2)*RSe451p*RSe453p - 1./81.*pow(me2,2)*RSe451p*RSe461p + 1.
         /162.*pow(me2,2)*pow(RSe451p,2) - 1./324.*m66*me2*pow(
         RSe463p,2) - 1./162.*m66*me2*RSe453p*RSe463p - 1./108.*m66*me2
         *pow(RSe453p,2) + 1./162.*m66*me2*RSe461p*RSe463p + 1./162.*
         m66*me2*RSe461p*RSe453p - 1./324.*m66*me2*pow(RSe461p,2) + 1./
         162.*m66*me2*RSe451p*RSe463p + 1./54.*m66*me2*RSe451p*RSe453p
          - 1./162.*m66*me2*RSe451p*RSe461p - 1./108.*m66*me2*pow(
         RSe451p,2) + 1./162.*m56*me2*pow(RSe463p,2) - 1./81.*m56*me2*
         RSe453p*RSe463p + 1./162.*m56*me2*pow(RSe453p,2) - 1./81.*m56*
         me2*RSe461p*RSe463p + 1./81.*m56*me2*RSe461p*RSe453p + 1./162.
         *m56*me2*pow(RSe461p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m56,L) * ( 1./81.*m56*me2*
         RSe451p*RSe463p - 1./81.*m56*me2*RSe451p*RSe453p - 1./81.*m56*
         me2*RSe451p*RSe461p + 1./162.*m56*me2*pow(RSe451p,2) - 1./108.
         *m55*me2*pow(RSe463p,2) - 1./162.*m55*me2*RSe453p*RSe463p - 1./
         324.*m55*me2*pow(RSe453p,2) + 1./54.*m55*me2*RSe461p*RSe463p
          + 1./162.*m55*me2*RSe461p*RSe453p - 1./108.*m55*me2*pow(
         RSe461p,2) + 1./162.*m55*me2*RSe451p*RSe463p + 1./162.*m55*me2
         *RSe451p*RSe453p - 1./162.*m55*me2*RSe451p*RSe461p - 1./324.*
         m55*me2*pow(RSe451p,2) - 1./108.*m33*me2*pow(RSe463p,2) + 1./
         54.*m33*me2*RSe453p*RSe463p - 1./108.*m33*me2*pow(RSe453p,2)
          - 1./162.*m33*me2*RSe461p*RSe463p + 1./162.*m33*me2*RSe461p*
         RSe453p - 1./324.*m33*me2*pow(RSe461p,2) + 1./162.*m33*me2*
         RSe451p*RSe463p - 1./162.*m33*me2*RSe451p*RSe453p + 1./162.*
         m33*me2*RSe451p*RSe461p - 1./324.*m33*me2*pow(RSe451p,2) + 1./
         216.*m33*m66*pow(RSe463p,2) + 1./108.*m33*m66*RSe453p*RSe463p
          + 1./72.*m33*m66*pow(RSe453p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m56,L) * ( 1./324.*m33*m66*
         RSe461p*RSe463p + 1./324.*m33*m66*RSe461p*RSe453p + 1./648.*
         m33*m66*pow(RSe461p,2) + 1./324.*m33*m66*RSe451p*RSe463p + 1./
         108.*m33*m66*RSe451p*RSe453p + 1./324.*m33*m66*RSe451p*RSe461p
          + 1./216.*m33*m66*pow(RSe451p,2) - 1./108.*m33*m56*pow(
         RSe463p,2) + 1./54.*m33*m56*RSe453p*RSe463p - 1./108.*m33*m56*
         pow(RSe453p,2) - 1./162.*m33*m56*RSe461p*RSe463p + 1./162.*m33
         *m56*RSe461p*RSe453p - 1./324.*m33*m56*pow(RSe461p,2) + 1./162.
         *m33*m56*RSe451p*RSe463p - 1./162.*m33*m56*RSe451p*RSe453p + 1.
         /162.*m33*m56*RSe451p*RSe461p - 1./324.*m33*m56*pow(RSe451p,2)
          + 1./72.*m33*m55*pow(RSe463p,2) + 1./108.*m33*m55*RSe453p*
         RSe463p + 1./216.*m33*m55*pow(RSe453p,2) + 1./108.*m33*m55*
         RSe461p*RSe463p + 1./324.*m33*m55*RSe461p*RSe453p + 1./216.*
         m33*m55*pow(RSe461p,2) + 1./324.*m33*m55*RSe451p*RSe463p + 1./
         324.*m33*m55*RSe451p*RSe453p + 1./324.*m33*m55*RSe451p*RSe461p
          + 1./648.*m33*m55*pow(RSe451p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m56,L) * ( 1./162.*m13*me2*pow(
         RSe463p,2) - 1./81.*m13*me2*RSe453p*RSe463p + 1./162.*m13*me2*
         pow(RSe453p,2) - 1./81.*m13*me2*RSe461p*RSe463p + 1./81.*m13*
         me2*RSe461p*RSe453p + 1./162.*m13*me2*pow(RSe461p,2) + 1./81.*
         m13*me2*RSe451p*RSe463p - 1./81.*m13*me2*RSe451p*RSe453p - 1./
         81.*m13*me2*RSe451p*RSe461p + 1./162.*m13*me2*pow(RSe451p,2)
          - 1./324.*m13*m66*pow(RSe463p,2) - 1./162.*m13*m66*RSe453p*
         RSe463p - 1./108.*m13*m66*pow(RSe453p,2) + 1./162.*m13*m66*
         RSe461p*RSe463p + 1./162.*m13*m66*RSe461p*RSe453p - 1./324.*
         m13*m66*pow(RSe461p,2) + 1./162.*m13*m66*RSe451p*RSe463p + 1./
         54.*m13*m66*RSe451p*RSe453p - 1./162.*m13*m66*RSe451p*RSe461p
          - 1./108.*m13*m66*pow(RSe451p,2) + 1./162.*m13*m56*pow(
         RSe463p,2) - 1./81.*m13*m56*RSe453p*RSe463p + 1./162.*m13*m56*
         pow(RSe453p,2) - 1./81.*m13*m56*RSe461p*RSe463p + 1./81.*m13*
         m56*RSe461p*RSe453p + 1./162.*m13*m56*pow(RSe461p,2) + 1./81.*
         m13*m56*RSe451p*RSe463p );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m56,L) * (  - 1./81.*m13*m56*
         RSe451p*RSe453p - 1./81.*m13*m56*RSe451p*RSe461p + 1./162.*m13
         *m56*pow(RSe451p,2) - 1./108.*m13*m55*pow(RSe463p,2) - 1./162.
         *m13*m55*RSe453p*RSe463p - 1./324.*m13*m55*pow(RSe453p,2) + 1./
         54.*m13*m55*RSe461p*RSe463p + 1./162.*m13*m55*RSe461p*RSe453p
          - 1./108.*m13*m55*pow(RSe461p,2) + 1./162.*m13*m55*RSe451p*
         RSe463p + 1./162.*m13*m55*RSe451p*RSe453p - 1./162.*m13*m55*
         RSe451p*RSe461p - 1./324.*m13*m55*pow(RSe451p,2) - 1./324.*m11
         *me2*pow(RSe463p,2) + 1./162.*m11*me2*RSe453p*RSe463p - 1./324.
         *m11*me2*pow(RSe453p,2) - 1./162.*m11*me2*RSe461p*RSe463p + 1./
         162.*m11*me2*RSe461p*RSe453p - 1./108.*m11*me2*pow(RSe461p,2)
          + 1./162.*m11*me2*RSe451p*RSe463p - 1./162.*m11*me2*RSe451p*
         RSe453p + 1./54.*m11*me2*RSe451p*RSe461p - 1./108.*m11*me2*
         pow(RSe451p,2) + 1./648.*m11*m66*pow(RSe463p,2) + 1./324.*m11*
         m66*RSe453p*RSe463p + 1./216.*m11*m66*pow(RSe453p,2) + 1./324.
         *m11*m66*RSe461p*RSe463p );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m56,L) * ( 1./324.*m11*m66*
         RSe461p*RSe453p + 1./216.*m11*m66*pow(RSe461p,2) + 1./324.*m11
         *m66*RSe451p*RSe463p + 1./108.*m11*m66*RSe451p*RSe453p + 1./
         108.*m11*m66*RSe451p*RSe461p + 1./72.*m11*m66*pow(RSe451p,2)
          - 1./324.*m11*m56*pow(RSe463p,2) + 1./162.*m11*m56*RSe453p*
         RSe463p - 1./324.*m11*m56*pow(RSe453p,2) - 1./162.*m11*m56*
         RSe461p*RSe463p + 1./162.*m11*m56*RSe461p*RSe453p - 1./108.*
         m11*m56*pow(RSe461p,2) + 1./162.*m11*m56*RSe451p*RSe463p - 1./
         162.*m11*m56*RSe451p*RSe453p + 1./54.*m11*m56*RSe451p*RSe461p
          - 1./108.*m11*m56*pow(RSe451p,2) + 1./216.*m11*m55*pow(
         RSe463p,2) + 1./324.*m11*m55*RSe453p*RSe463p + 1./648.*m11*m55
         *pow(RSe453p,2) + 1./108.*m11*m55*RSe461p*RSe463p + 1./324.*
         m11*m55*RSe461p*RSe453p + 1./72.*m11*m55*pow(RSe461p,2) + 1./
         324.*m11*m55*RSe451p*RSe463p + 1./324.*m11*m55*RSe451p*RSe453p
          + 1./108.*m11*m55*RSe451p*RSe461p + 1./216.*m11*m55*pow(
         RSe451p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(m66,L) * (  - 1./54.*m66*me2*
         pow(RSe453p,2) + 1./162.*m66*me2*RS645pe*pow(RSe453p,2) + 1./
         27.*m66*me2*RSe451p*RSe453p - 1./81.*m66*me2*RSe451p*RS645pe*
         RSe453p - 1./54.*m66*me2*pow(RSe451p,2) + 1./162.*m66*me2*pow(
         RSe451p,2)*RS645pe + 1./36.*m33*m66*pow(RSe453p,2) - 1./108.*
         m33*m66*RS645pe*pow(RSe453p,2) + 1./54.*m33*m66*RSe451p*
         RSe453p - 1./162.*m33*m66*RSe451p*RS645pe*RSe453p + 1./108.*
         m33*m66*pow(RSe451p,2) - 1./324.*m33*m66*pow(RSe451p,2)*
         RS645pe - 1./54.*m13*m66*pow(RSe453p,2) + 1./162.*m13*m66*
         RS645pe*pow(RSe453p,2) + 1./27.*m13*m66*RSe451p*RSe453p - 1./
         81.*m13*m66*RSe451p*RS645pe*RSe453p - 1./54.*m13*m66*pow(
         RSe451p,2) + 1./162.*m13*m66*pow(RSe451p,2)*RS645pe + 1./108.*
         m11*m66*pow(RSe453p,2) - 1./324.*m11*m66*RS645pe*pow(
         RSe453p,2) + 1./54.*m11*m66*RSe451p*RSe453p - 1./162.*m11*m66*
         RSe451p*RS645pe*RSe453p + 1./36.*m11*m66*pow(RSe451p,2) - 1./
         108.*m11*m66*pow(RSe451p,2)*RS645pe );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(mp2,L) * ( 1./162.*m66*me2*
         RSp456e*pow(RSe453p,2) - 1./81.*m66*me2*RSp456e*RSe451p*
         RSe453p + 1./162.*m66*me2*RSp456e*pow(RSe451p,2) + 1./162.*m55
         *me2*RSp465e*pow(RSe463p,2) - 1./81.*m55*me2*RSp465e*RSe461p*
         RSe463p + 1./162.*m55*me2*RSp465e*pow(RSe461p,2) + 1./162.*m44
         *me2*RSp564e*pow(RSe563p,2) - 1./81.*m44*me2*RSp564e*RSe561p*
         RSe563p + 1./162.*m44*me2*RSp564e*pow(RSe561p,2) - 1./108.*m33
         *m66*RSp456e*pow(RSe453p,2) - 1./162.*m33*m66*RSp456e*RSe451p*
         RSe453p - 1./324.*m33*m66*RSp456e*pow(RSe451p,2) - 1./108.*m33
         *m55*RSp465e*pow(RSe463p,2) - 1./162.*m33*m55*RSp465e*RSe461p*
         RSe463p - 1./324.*m33*m55*RSp465e*pow(RSe461p,2) - 1./108.*m33
         *m44*RSp564e*pow(RSe563p,2) - 1./162.*m33*m44*RSp564e*RSe561p*
         RSe563p - 1./324.*m33*m44*RSp564e*pow(RSe561p,2) + 1./162.*m13
         *m66*RSp456e*pow(RSe453p,2) - 1./81.*m13*m66*RSp456e*RSe451p*
         RSe453p + 1./162.*m13*m66*RSp456e*pow(RSe451p,2) + 1./162.*m13
         *m55*RSp465e*pow(RSe463p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(mp2,L) * (  - 1./81.*m13*m55*
         RSp465e*RSe461p*RSe463p + 1./162.*m13*m55*RSp465e*pow(
         RSe461p,2) + 1./162.*m13*m44*RSp564e*pow(RSe563p,2) - 1./81.*
         m13*m44*RSp564e*RSe561p*RSe563p + 1./162.*m13*m44*RSp564e*pow(
         RSe561p,2) - 1./324.*m11*m66*RSp456e*pow(RSe453p,2) - 1./162.*
         m11*m66*RSp456e*RSe451p*RSe453p - 1./108.*m11*m66*RSp456e*pow(
         RSe451p,2) - 1./324.*m11*m55*RSp465e*pow(RSe463p,2) - 1./162.*
         m11*m55*RSp465e*RSe461p*RSe463p - 1./108.*m11*m55*RSp465e*pow(
         RSe461p,2) - 1./324.*m11*m44*RSp564e*pow(RSe563p,2) - 1./162.*
         m11*m44*RSp564e*RSe561p*RSe563p - 1./108.*m11*m44*RSp564e*pow(
         RSe561p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(me2,L) * ( 1./162.*m66*me2*
         RSe456p*pow(RSe453p,2) - 1./81.*m66*me2*RSe456p*RSe451p*
         RSe453p + 1./162.*m66*me2*RSe456p*pow(RSe451p,2) + 1./162.*m55
         *me2*RSe465p*pow(RSe463p,2) - 1./81.*m55*me2*RSe465p*RSe461p*
         RSe463p + 1./162.*m55*me2*RSe465p*pow(RSe461p,2) + 1./162.*m44
         *me2*RSe564p*pow(RSe563p,2) - 1./81.*m44*me2*RSe564p*RSe561p*
         RSe563p + 1./162.*m44*me2*RSe564p*pow(RSe561p,2) - 1./108.*m33
         *m66*RSe456p*pow(RSe453p,2) - 1./162.*m33*m66*RSe456p*RSe451p*
         RSe453p - 1./324.*m33*m66*RSe456p*pow(RSe451p,2) - 1./108.*m33
         *m55*RSe465p*pow(RSe463p,2) - 1./162.*m33*m55*RSe465p*RSe461p*
         RSe463p - 1./324.*m33*m55*RSe465p*pow(RSe461p,2) - 1./108.*m33
         *m44*RSe564p*pow(RSe563p,2) - 1./162.*m33*m44*RSe564p*RSe561p*
         RSe563p - 1./324.*m33*m44*RSe564p*pow(RSe561p,2) + 1./162.*m13
         *m66*RSe456p*pow(RSe453p,2) - 1./81.*m13*m66*RSe456p*RSe451p*
         RSe453p + 1./162.*m13*m66*RSe456p*pow(RSe451p,2) + 1./162.*m13
         *m55*RSe465p*pow(RSe463p,2) );

      M13p6x23RV +=  + Bb(me2,mu2)*AbV(me2,L) * (  - 1./81.*m13*m55*
         RSe465p*RSe461p*RSe463p + 1./162.*m13*m55*RSe465p*pow(
         RSe461p,2) + 1./162.*m13*m44*RSe564p*pow(RSe563p,2) - 1./81.*
         m13*m44*RSe564p*RSe561p*RSe563p + 1./162.*m13*m44*RSe564p*pow(
         RSe561p,2) - 1./324.*m11*m66*RSe456p*pow(RSe453p,2) - 1./162.*
         m11*m66*RSe456p*RSe451p*RSe453p - 1./108.*m11*m66*RSe456p*pow(
         RSe451p,2) - 1./324.*m11*m55*RSe465p*pow(RSe463p,2) - 1./162.*
         m11*m55*RSe465p*RSe461p*RSe463p - 1./108.*m11*m55*RSe465p*pow(
         RSe461p,2) - 1./324.*m11*m44*RSe564p*pow(RSe563p,2) - 1./162.*
         m11*m44*RSe564p*RSe561p*RSe563p - 1./108.*m11*m44*RSe564p*pow(
         RSe561p,2) );

      M13p6x23RV +=  + Cb(m11,mu2)*AbV(m11,L) * ( 1./54.*m11*m33*
         RD1456ep*RC1456pe - 1./27.*m11*m13*RD1456ep*RC1456pe + 1./54.*
         pow(m11,2)*RD1456ep*RC1456pe );

      M13p6x23RV +=  + Cb(m11,mu2)*AbV(m14,L) * (  - 1./108.*m33*m44*
         RD1456ep + 1./54.*m14*m33*RD1456ep + 1./54.*m13*m44*RD1456ep
          - 1./27.*m13*m14*RD1456ep - 1./108.*m11*m44*RD1456ep - 1./108.
         *m11*m33*RD1456ep + 1./54.*m11*m14*RD1456ep + 1./54.*m11*m13*
         RD1456ep - 1./108.*pow(m11,2)*RD1456ep );

      M13p6x23RV +=  + Cb(m11,mu2)*AbV(m15,L) * (  - 1./108.*m33*m55*
         RD1456ep + 1./54.*m15*m33*RD1456ep + 1./54.*m13*m55*RD1456ep
          - 1./27.*m13*m15*RD1456ep - 1./108.*m11*m55*RD1456ep - 1./108.
         *m11*m33*RD1456ep + 1./54.*m11*m15*RD1456ep + 1./54.*m11*m13*
         RD1456ep - 1./108.*pow(m11,2)*RD1456ep );

      M13p6x23RV +=  + Cb(m11,mu2)*AbV(m16,L) * (  - 1./108.*m33*m66*
         RD1456ep + 1./54.*m16*m33*RD1456ep + 1./54.*m13*m66*RD1456ep
          - 1./27.*m13*m16*RD1456ep - 1./108.*m11*m66*RD1456ep - 1./108.
         *m11*m33*RD1456ep + 1./54.*m11*m16*RD1456ep + 1./54.*m11*m13*
         RD1456ep - 1./108.*pow(m11,2)*RD1456ep );

      M13p6x23RV +=  + Cb(m11,mu2)*AbV(mp2,L) * ( 1./54.*m11*m33*
         RSp45611e*RD1456ep - 1./27.*m11*m13*RSp45611e*RD1456ep + 1./54.
         *pow(m11,2)*RSp45611e*RD1456ep );

      M13p6x23RV +=  + Cb(m11,mu2)*AbV(me2,L) * ( 1./54.*m11*m33*
         RSe45611p*RD1456ep - 1./27.*m11*m13*RSe45611p*RD1456ep + 1./54.
         *pow(m11,2)*RSe45611p*RD1456ep );

      M13p6x23RV +=  + Cb(m11,mu2)*BbV(m11,L) * ( 1./54.*m11*m33*pow(
         RD1456ep,2) - 1./27.*m11*m13*pow(RD1456ep,2) + 1./54.*pow(
         m11,2)*pow(RD1456ep,2) );

      M13p6x23RV +=  + Cb(m33,mu2)*AbV(m33,L) * ( 1./54.*pow(m33,2)*
         RD3456ep*RC3456pe - 1./27.*m13*m33*RD3456ep*RC3456pe + 1./54.*
         m11*m33*RD3456ep*RC3456pe );

      M13p6x23RV +=  + Cb(m33,mu2)*AbV(m34,L) * (  - 1./108.*m33*m44*
         RD3456ep + 1./54.*m33*m34*RD3456ep - 1./108.*pow(m33,2)*
         RD3456ep + 1./54.*m13*m44*RD3456ep - 1./27.*m13*m34*RD3456ep
          + 1./54.*m13*m33*RD3456ep - 1./108.*m11*m44*RD3456ep + 1./54.
         *m11*m34*RD3456ep - 1./108.*m11*m33*RD3456ep );

      M13p6x23RV +=  + Cb(m33,mu2)*AbV(m35,L) * (  - 1./108.*m33*m55*
         RD3456ep + 1./54.*m33*m35*RD3456ep - 1./108.*pow(m33,2)*
         RD3456ep + 1./54.*m13*m55*RD3456ep - 1./27.*m13*m35*RD3456ep
          + 1./54.*m13*m33*RD3456ep - 1./108.*m11*m55*RD3456ep + 1./54.
         *m11*m35*RD3456ep - 1./108.*m11*m33*RD3456ep );

      M13p6x23RV +=  + Cb(m33,mu2)*AbV(m36,L) * (  - 1./108.*m33*m66*
         RD3456ep + 1./54.*m33*m36*RD3456ep - 1./108.*pow(m33,2)*
         RD3456ep + 1./54.*m13*m66*RD3456ep - 1./27.*m13*m36*RD3456ep
          + 1./54.*m13*m33*RD3456ep - 1./108.*m11*m66*RD3456ep + 1./54.
         *m11*m36*RD3456ep - 1./108.*m11*m33*RD3456ep );

      M13p6x23RV +=  + Cb(m33,mu2)*AbV(mp2,L) * ( 1./54.*pow(m33,2)*
         RSp45633e*RD3456ep - 1./27.*m13*m33*RSp45633e*RD3456ep + 1./54.
         *m11*m33*RSp45633e*RD3456ep );

      M13p6x23RV +=  + Cb(m33,mu2)*AbV(me2,L) * ( 1./54.*pow(m33,2)*
         RSe45633p*RD3456ep - 1./27.*m13*m33*RSe45633p*RD3456ep + 1./54.
         *m11*m33*RSe45633p*RD3456ep );

      M13p6x23RV +=  + Cb(m33,mu2)*BbV(m33,L) * ( 1./54.*pow(m33,2)*
         pow(RD3456ep,2) - 1./27.*m13*m33*pow(RD3456ep,2) + 1./54.*m11*
         m33*pow(RD3456ep,2) );

      M13p6x23RV +=  + AbV(m11,L)*pi16 * (  - 1./36.*m13*RD3456ep*
         RS14563pe + 1./72.*m13*RC1456pe*RD3456ep + 1./24.*m13*RD1456ep
          - 1./36.*m13*RD1456ep*RS14563pe + 1./72.*m13*RD1456ep*
         RC1456pe + 1./18.*m13*me2*RSe45613p*RS14563pe - 1./36.*m13*me2
         *RSe45633p*RS14563pe - 1./36.*m13*me2*RC1456pe*RSe45613p + 1./
         72.*m13*me2*RC1456pe*RSe45633p - 1./36.*m13*me2*RSe45611p*
         RS14563pe + 1./72.*m13*me2*RSe45611p*RC1456pe + 1./18.*m13*mp2
         *RSp45613e*RS14563pe - 1./36.*m13*mp2*RSp45633e*RS14563pe - 1./
         36.*m13*mp2*RC1456pe*RSp45613e + 1./72.*m13*mp2*RC1456pe*
         RSp45633e - 1./36.*m13*mp2*RSp45611e*RS14563pe + 1./72.*m13*
         mp2*RSp45611e*RC1456pe + 1./12.*m13*m36*RS14563pe - 1./24.*m13
         *m36*RC1456pe + 1./12.*m13*m35*RS14563pe - 1./24.*m13*m35*
         RC1456pe + 1./12.*m13*m34*RS14563pe - 1./24.*m13*m34*RC1456pe
          + 1./18.*m13*m33*RS14563pe*RS34561pe - 1./36.*m13*m33*
         RC3456pe*RS14563pe - 1./36.*m13*m33*RC1456pe*RS34561pe + 1./72.
         *m13*m33*RC1456pe*RC3456pe );

      M13p6x23RV +=  + AbV(m11,L)*pi16 * ( 1./12.*m13*m16*RS14563pe - 1.
         /24.*m13*m16*RC1456pe + 1./12.*m13*m15*RS14563pe - 1./24.*m13*
         m15*RC1456pe + 1./12.*m13*m14*RS14563pe - 1./24.*m13*m14*
         RC1456pe - 1./12.*pow(m13,2)*RS14563pe + 1./27.*pow(m13,2)*
         RS14563pe*RS34561pe + 1./27.*pow(m13,2)*pow(RS14563pe,2) + 1./
         27.*pow(m13,2)*RSe45613p*RS14563pe + 1./27.*pow(m13,2)*
         RSp45613e*RS14563pe - 1./54.*pow(m13,2)*RC3456pe*RS14563pe - 1.
         /54.*pow(m13,2)*RSe45633p*RS14563pe - 1./54.*pow(m13,2)*
         RSp45633e*RS14563pe + 1./24.*pow(m13,2)*RC1456pe - 1./54.*pow(
         m13,2)*RC1456pe*RS34561pe - 1./27.*pow(m13,2)*RC1456pe*
         RS14563pe - 1./54.*pow(m13,2)*RC1456pe*RSe45613p - 1./54.*pow(
         m13,2)*RC1456pe*RSp45613e + 1./108.*pow(m13,2)*RC1456pe*
         RC3456pe + 1./108.*pow(m13,2)*RC1456pe*RSe45633p + 1./108.*
         pow(m13,2)*RC1456pe*RSp45633e + 1./108.*pow(m13,2)*pow(
         RC1456pe,2) - 1./54.*pow(m13,2)*RSe45611p*RS14563pe + 1./108.*
         pow(m13,2)*RSe45611p*RC1456pe );

      M13p6x23RV +=  + AbV(m11,L)*pi16 * (  - 1./54.*pow(m13,2)*
         RSp45611e*RS14563pe + 1./108.*pow(m13,2)*RSp45611e*RC1456pe - 
         1./8.*m11*m13 - 1./12.*m11*m13*RS14563pe + 1./18.*m11*m13*pow(
         RS14563pe,2) + 1./24.*m11*m13*RC1456pe - 1./18.*m11*m13*
         RC1456pe*RS14563pe + 1./72.*m11*m13*pow(RC1456pe,2) + 1./24.*
         m11*m13*RS145pe + 1./24.*m11*m13*RS146pe + 1./24.*m11*m13*
         RS156pe );

      M13p6x23RV +=  + pow(AbV(m11,L),2) * (  - 1./270.*RD1456ep*
         RS14563pe + 1./540.*RD1456ep*RC1456pe + 11./180.*m33 + 13./810.
         *m33*pow(RS14563pe,2) + 1./270.*m33*RC1456pe*RS14563pe + 13./
         1620.*m33*pow(RC1456pe,2) + 1./360.*m13 - 7./810.*m13*pow(
         RS14563pe,2) + 7./810.*m13*RC1456pe*RS14563pe - 7./3240.*m13*
         pow(RC1456pe,2) + 11./180.*m11 + 1./90.*m11*pow(RS14563pe,2)
          + 2./81.*m11*RC1456pe*RS14563pe - 17./1620.*m11*pow(
         RC1456pe,2) + 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33*
         RS14563pe - 1./162.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33*
         RC1456pe + 2./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*
         RS14563pe - 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*
         RC1456pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m13,L) * (  - 1./162.*RD3456ep*
         RS14563pe + 1./324.*RC1456pe*RD3456ep - 1./54.*RD1456ep*
         RS34561pe - 2./81.*RD1456ep*RS14563pe - 1./54.*RD1456ep*
         RSe45613p - 1./54.*RD1456ep*RSp45613e + 1./108.*RD1456ep*
         RC3456pe + 1./108.*RD1456ep*RSe45633p + 1./108.*RD1456ep*
         RSp45633e + 1./81.*RD1456ep*RC1456pe + 1./108.*RSe45611p*
         RD1456ep + 1./108.*RSp45611e*RD1456ep + 1./81.*me2*RSe45613p*
         RS14563pe - 1./162.*me2*RSe45633p*RS14563pe - 1./162.*me2*
         RC1456pe*RSe45613p + 1./324.*me2*RC1456pe*RSe45633p - 1./162.*
         me2*RSe45611p*RS14563pe + 1./324.*me2*RSe45611p*RC1456pe + 1./
         81.*mp2*RSp45613e*RS14563pe - 1./162.*mp2*RSp45633e*RS14563pe
          - 1./162.*mp2*RC1456pe*RSp45613e + 1./324.*mp2*RC1456pe*
         RSp45633e - 1./162.*mp2*RSp45611e*RS14563pe + 1./324.*mp2*
         RSp45611e*RC1456pe - 1./60.*m33 - 1./81.*m33*pow(RS14563pe,2)
          - 1./81.*m33*RSe45613p*RS14563pe - 1./81.*m33*RSp45613e*
         RS14563pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m13,L) * ( 1./162.*m33*RC3456pe*
         RS14563pe + 1./81.*m33*RSe45633p*RS14563pe + 1./81.*m33*
         RSp45633e*RS14563pe - 1./162.*m33*RC1456pe*RS34561pe + 1./108.
         *m33*RC1456pe*RC3456pe + 1./162.*m33*RC1456pe*RSe45633p + 1./
         162.*m33*RC1456pe*RSp45633e - 1./162.*m33*pow(RC1456pe,2) - 1./
         162.*m33*RSe45611p*RC1456pe - 1./162.*m33*RSp45611e*RC1456pe
          + 1./180.*m13 - 2./27.*m13*RS14563pe*RS34561pe - 2./27.*m13*
         pow(RS14563pe,2) - 2./27.*m13*RSe45613p*RS14563pe - 2./27.*m13
         *RSp45613e*RS14563pe + 1./27.*m13*RC3456pe*RS14563pe + 1./27.*
         m13*RSe45633p*RS14563pe + 1./27.*m13*RSp45633e*RS14563pe + 1./
         27.*m13*RC1456pe*RS34561pe + 2./27.*m13*RC1456pe*RS14563pe + 1.
         /27.*m13*RC1456pe*RSe45613p + 1./27.*m13*RC1456pe*RSp45613e - 
         1./54.*m13*RC1456pe*RC3456pe - 1./54.*m13*RC1456pe*RSe45633p
          - 1./54.*m13*RC1456pe*RSp45633e - 1./54.*m13*pow(RC1456pe,2)
          + 1./27.*m13*RSe45611p*RS14563pe - 1./54.*m13*RSe45611p*
         RC1456pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m13,L) * ( 1./27.*m13*RSp45611e*
         RS14563pe - 1./54.*m13*RSp45611e*RC1456pe + 7./180.*m11 + 2./
         81.*m11*RS14563pe*RS34561pe + 1./27.*m11*pow(RS14563pe,2) + 2./
         81.*m11*RSe45613p*RS14563pe + 2./81.*m11*RSp45613e*RS14563pe
          - 1./54.*m11*RC3456pe*RS14563pe - 1./54.*m11*RSe45633p*
         RS14563pe - 1./54.*m11*RSp45633e*RS14563pe - 1./162.*m11*
         RC1456pe*RS34561pe - 2./81.*m11*RC1456pe*RS14563pe - 1./162.*
         m11*RC1456pe*RSe45613p - 1./162.*m11*RC1456pe*RSp45613e + 5./
         324.*m11*RC1456pe*RC3456pe + 5./324.*m11*RC1456pe*RSe45633p + 
         5./324.*m11*RC1456pe*RSp45633e - 1./162.*m11*pow(RC1456pe,2)
          - 1./162.*m11*RSe45611p*RS14563pe - 1./108.*m11*RSe45611p*
         RC1456pe - 1./162.*m11*RSp45611e*RS14563pe - 1./108.*m11*
         RSp45611e*RC1456pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m14,L) * (  - 29./540.*RD1456ep
          - 7./540.*m44*RS14563pe + 7./540.*m44*RC1456pe - 1./60.*m44*
         RS156pe + 1./27.*m34*RS14563pe - 1./108.*m34*RC1456pe - 1./54.
         *m34*RS156pe + 1./90.*m33*RS14563pe + 1./120.*m33*RC1456pe - 1.
         /20.*m33*RS156pe - 1./90.*m14*RS14563pe - 1./60.*m14*RC1456pe
          + 2./45.*m14*RS156pe - 11./540.*m13*RS14563pe - 1./135.*m13*
         RC1456pe + 19./540.*m13*RS156pe + 1./27.*m11*RS34561pe + 43./
         540.*m11*RS14563pe + 1./27.*m11*RSe45613p + 1./27.*m11*
         RSp45613e - 1./27.*m11*RS356pe - 1./27.*m11*RSe563p - 1./27.*
         m11*RSp563e + 13./1080.*m11*RC1456pe - 1./27.*m11*RSe45611p - 
         1./27.*m11*RSp45611e - 1./45.*m11*RS156pe + 1./27.*m11*RSe561p
          + 1./27.*m11*RSp561e + 1./54.*RS(m11,m44,m55,m66,m33,me2,mp2)
         *m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11 );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m15,L) * (  - 29./540.*RD1456ep
          - 7./540.*m55*RS14563pe + 7./540.*m55*RC1456pe - 1./60.*m55*
         RS146pe + 1./27.*m35*RS14563pe - 1./108.*m35*RC1456pe - 1./54.
         *m35*RS146pe + 1./90.*m33*RS14563pe + 1./120.*m33*RC1456pe - 1.
         /20.*m33*RS146pe - 1./90.*m15*RS14563pe - 1./60.*m15*RC1456pe
          + 2./45.*m15*RS146pe - 11./540.*m13*RS14563pe - 1./135.*m13*
         RC1456pe + 19./540.*m13*RS146pe + 1./27.*m11*RS34561pe + 43./
         540.*m11*RS14563pe + 1./27.*m11*RSe45613p + 1./27.*m11*
         RSp45613e - 1./27.*m11*RS346pe - 1./27.*m11*RSe463p - 1./27.*
         m11*RSp463e + 13./1080.*m11*RC1456pe - 1./27.*m11*RSe45611p - 
         1./27.*m11*RSp45611e - 1./45.*m11*RS146pe + 1./27.*m11*RSe461p
          + 1./27.*m11*RSp461e + 1./54.*RS(m11,m44,m55,m66,m33,me2,mp2)
         *m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11 );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m16,L) * (  - 29./540.*RD1456ep
          - 7./540.*m66*RS14563pe + 7./540.*m66*RC1456pe - 1./60.*m66*
         RS145pe + 1./27.*m36*RS14563pe - 1./108.*m36*RC1456pe - 1./54.
         *m36*RS145pe + 1./90.*m33*RS14563pe + 1./120.*m33*RC1456pe - 1.
         /20.*m33*RS145pe - 1./90.*m16*RS14563pe - 1./60.*m16*RC1456pe
          + 2./45.*m16*RS145pe - 11./540.*m13*RS14563pe - 1./135.*m13*
         RC1456pe + 19./540.*m13*RS145pe + 1./27.*m11*RS34561pe + 43./
         540.*m11*RS14563pe + 1./27.*m11*RSe45613p + 1./27.*m11*
         RSp45613e - 1./27.*m11*RS345pe - 1./27.*m11*RSe453p - 1./27.*
         m11*RSp453e + 13./1080.*m11*RC1456pe - 1./27.*m11*RSe45611p - 
         1./27.*m11*RSp45611e - 1./45.*m11*RS145pe + 1./27.*m11*RSe451p
          + 1./27.*m11*RSp451e + 1./54.*RS(m11,m44,m55,m66,m33,me2,mp2)
         *m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11 );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m33,L) * (  - 1./270.*RD3456ep*
         RS14563pe + 1./540.*RC1456pe*RD3456ep - 1./270.*RD1456ep*
         RS34561pe + 1./540.*RD1456ep*RC3456pe + 11./405.*m33*RS14563pe
         *RS34561pe - 2./135.*m33*RC3456pe*RS14563pe + 1./162.*m33*
         RC1456pe*RS34561pe - 19./1620.*m33*RC1456pe*RC3456pe - 7./405.
         *m13*RS14563pe*RS34561pe + 7./810.*m13*RC3456pe*RS14563pe + 7./
         810.*m13*RC1456pe*RS34561pe - 7./1620.*m13*RC1456pe*RC3456pe
          + 11./405.*m11*RS14563pe*RS34561pe + 1./162.*m11*RC3456pe*
         RS14563pe + 1./45.*m11*RC1456pe*RS34561pe - 19./1620.*m11*
         RC1456pe*RC3456pe + 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33
         *RS34561pe - 1./162.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33*
         RC3456pe + 2./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*
         RS34561pe - 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)*m11*
         RC3456pe + 2./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33*
         RS14563pe - 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33*
         RC1456pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m33,L) * ( 1./81.*RS(m33,m44,m55,
         m66,m11,me2,mp2)*m11*RS14563pe - 1./162.*RS(m33,m44,m55,m66,
         m11,me2,mp2)*m11*RC1456pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m34,L) * (  - 1./180.*RD1456ep + 
         1./180.*m44*RS14563pe + 1./60.*m44*RC1456pe + 13./540.*m44*
         RS156pe - 13./270.*m34*RS14563pe + 1./270.*m34*RC1456pe + 11./
         270.*m34*RS156pe + 1./60.*m33*RS14563pe + 2./135.*m33*RC1456pe
          + 1./180.*m33*RS156pe + 1./27.*m14*RS14563pe - 1./27.*m14*
         RC1456pe + 1./27.*m14*RS156pe - 31./540.*m13*RS14563pe + 1./20.
         *m13*RC1456pe - 23./540.*m13*RS156pe + 1./18.*m11*RS14563pe + 
         19./540.*m11*RC1456pe - 2./27.*m11*RS156pe + 1./54.*RS(m11,m44
         ,m55,m66,m33,me2,mp2)*m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,
         mp2)*m11 );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m35,L) * (  - 1./180.*RD1456ep + 
         1./180.*m55*RS14563pe + 1./60.*m55*RC1456pe + 13./540.*m55*
         RS146pe - 13./270.*m35*RS14563pe + 1./270.*m35*RC1456pe + 11./
         270.*m35*RS146pe + 1./60.*m33*RS14563pe + 2./135.*m33*RC1456pe
          + 1./180.*m33*RS146pe + 1./27.*m15*RS14563pe - 1./27.*m15*
         RC1456pe + 1./27.*m15*RS146pe - 31./540.*m13*RS14563pe + 1./20.
         *m13*RC1456pe - 23./540.*m13*RS146pe + 1./18.*m11*RS14563pe + 
         19./540.*m11*RC1456pe - 2./27.*m11*RS146pe + 1./54.*RS(m11,m44
         ,m55,m66,m33,me2,mp2)*m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,
         mp2)*m11 );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m36,L) * (  - 1./180.*RD1456ep + 
         1./180.*m66*RS14563pe + 1./60.*m66*RC1456pe + 13./540.*m66*
         RS145pe - 13./270.*m36*RS14563pe + 1./270.*m36*RC1456pe + 11./
         270.*m36*RS145pe + 1./60.*m33*RS14563pe + 2./135.*m33*RC1456pe
          + 1./180.*m33*RS145pe + 1./27.*m16*RS14563pe - 1./27.*m16*
         RC1456pe + 1./27.*m16*RS145pe - 31./540.*m13*RS14563pe + 1./20.
         *m13*RC1456pe - 23./540.*m13*RS145pe + 1./18.*m11*RS14563pe + 
         19./540.*m11*RC1456pe - 2./27.*m11*RS145pe + 1./54.*RS(m11,m44
         ,m55,m66,m33,me2,mp2)*m33 + 1./27.*RS(m11,m44,m55,m66,m33,me2,
         mp2)*m11 );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m44,L) * (  - 1./54.*m44*pow(
         RS156pe,2) + 1./162.*m44*RS456pe*pow(RS156pe,2) );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m45,L) * (  - 1./108.*m55*pow(
         RS146pe,2) - 1./162.*m55*RS156pe*RS146pe - 1./324.*m55*pow(
         RS156pe,2) + 1./162.*m45*pow(RS146pe,2) - 1./81.*m45*RS156pe*
         RS146pe + 1./162.*m45*pow(RS156pe,2) - 1./324.*m44*pow(
         RS146pe,2) - 1./162.*m44*RS156pe*RS146pe - 1./108.*m44*pow(
         RS156pe,2) - 1./324.*m33*pow(RS146pe,2) + 1./162.*m33*RS156pe*
         RS146pe - 1./324.*m33*pow(RS156pe,2) + 1./162.*m13*pow(
         RS146pe,2) - 1./81.*m13*RS156pe*RS146pe + 1./162.*m13*pow(
         RS156pe,2) - 1./81.*m11*RS146pe*RS346pe + 1./81.*m11*RS146pe*
         RS356pe + 1./81.*m11*RS146pe*RSe563p - 1./81.*m11*RS146pe*
         RSe463p + 1./81.*m11*RS146pe*RSp563e - 1./81.*m11*RS146pe*
         RSp463e + 1./324.*m11*pow(RS146pe,2) + 1./81.*m11*RS156pe*
         RS346pe - 1./81.*m11*RS156pe*RS356pe - 1./81.*m11*RS156pe*
         RSe563p + 1./81.*m11*RS156pe*RSe463p - 1./81.*m11*RS156pe*
         RSp563e + 1./81.*m11*RS156pe*RSp463e - 1./162.*m11*RS156pe*
         RS146pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m45,L) * ( 1./324.*m11*pow(
         RS156pe,2) - 1./81.*m11*RSe561p*RS146pe + 1./81.*m11*RSe561p*
         RS156pe + 1./81.*m11*RSe461p*RS146pe - 1./81.*m11*RSe461p*
         RS156pe - 1./81.*m11*RSp561e*RS146pe + 1./81.*m11*RSp561e*
         RS156pe + 1./81.*m11*RSp461e*RS146pe - 1./81.*m11*RSp461e*
         RS156pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m46,L) * (  - 1./108.*m66*pow(
         RS145pe,2) - 1./162.*m66*RS156pe*RS145pe - 1./324.*m66*pow(
         RS156pe,2) + 1./162.*m46*pow(RS145pe,2) - 1./81.*m46*RS156pe*
         RS145pe + 1./162.*m46*pow(RS156pe,2) - 1./324.*m44*pow(
         RS145pe,2) - 1./162.*m44*RS156pe*RS145pe - 1./108.*m44*pow(
         RS156pe,2) - 1./324.*m33*pow(RS145pe,2) + 1./162.*m33*RS156pe*
         RS145pe - 1./324.*m33*pow(RS156pe,2) + 1./162.*m13*pow(
         RS145pe,2) - 1./81.*m13*RS156pe*RS145pe + 1./162.*m13*pow(
         RS156pe,2) - 1./81.*m11*RS145pe*RS345pe + 1./81.*m11*RS145pe*
         RS356pe + 1./81.*m11*RS145pe*RSe563p - 1./81.*m11*RS145pe*
         RSe453p + 1./81.*m11*RS145pe*RSp563e - 1./81.*m11*RS145pe*
         RSp453e + 1./324.*m11*pow(RS145pe,2) + 1./81.*m11*RS156pe*
         RS345pe - 1./81.*m11*RS156pe*RS356pe - 1./81.*m11*RS156pe*
         RSe563p + 1./81.*m11*RS156pe*RSe453p - 1./81.*m11*RS156pe*
         RSp563e + 1./81.*m11*RS156pe*RSp453e - 1./162.*m11*RS156pe*
         RS145pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m46,L) * ( 1./324.*m11*pow(
         RS156pe,2) - 1./81.*m11*RSe561p*RS145pe + 1./81.*m11*RSe561p*
         RS156pe + 1./81.*m11*RSe451p*RS145pe - 1./81.*m11*RSe451p*
         RS156pe - 1./81.*m11*RSp561e*RS145pe + 1./81.*m11*RSp561e*
         RS156pe + 1./81.*m11*RSp451e*RS145pe - 1./81.*m11*RSp451e*
         RS156pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m55,L) * (  - 1./54.*m55*pow(
         RS146pe,2) + 1./162.*m55*RS546pe*pow(RS146pe,2) );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m56,L) * (  - 1./108.*m66*pow(
         RS145pe,2) - 1./162.*m66*RS146pe*RS145pe - 1./324.*m66*pow(
         RS146pe,2) + 1./162.*m56*pow(RS145pe,2) - 1./81.*m56*RS146pe*
         RS145pe + 1./162.*m56*pow(RS146pe,2) - 1./324.*m55*pow(
         RS145pe,2) - 1./162.*m55*RS146pe*RS145pe - 1./108.*m55*pow(
         RS146pe,2) - 1./324.*m33*pow(RS145pe,2) + 1./162.*m33*RS146pe*
         RS145pe - 1./324.*m33*pow(RS146pe,2) + 1./162.*m13*pow(
         RS145pe,2) - 1./81.*m13*RS146pe*RS145pe + 1./162.*m13*pow(
         RS146pe,2) - 1./81.*m11*RS145pe*RS345pe + 1./81.*m11*RS145pe*
         RS346pe + 1./81.*m11*RS145pe*RSe463p - 1./81.*m11*RS145pe*
         RSe453p + 1./81.*m11*RS145pe*RSp463e - 1./81.*m11*RS145pe*
         RSp453e + 1./324.*m11*pow(RS145pe,2) + 1./81.*m11*RS146pe*
         RS345pe - 1./81.*m11*RS146pe*RS346pe - 1./81.*m11*RS146pe*
         RSe463p + 1./81.*m11*RS146pe*RSe453p - 1./81.*m11*RS146pe*
         RSp463e + 1./81.*m11*RS146pe*RSp453e - 1./162.*m11*RS146pe*
         RS145pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m56,L) * ( 1./324.*m11*pow(
         RS146pe,2) - 1./81.*m11*RSe461p*RS145pe + 1./81.*m11*RSe461p*
         RS146pe + 1./81.*m11*RSe451p*RS145pe - 1./81.*m11*RSe451p*
         RS146pe - 1./81.*m11*RSp461e*RS145pe + 1./81.*m11*RSp461e*
         RS146pe + 1./81.*m11*RSp451e*RS145pe - 1./81.*m11*RSp451e*
         RS146pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(m66,L) * (  - 1./54.*m66*pow(
         RS145pe,2) + 1./162.*m66*RS645pe*pow(RS145pe,2) );

      M13p6x23RV +=  + AbV(m11,L)*AbV(mp2,L) * (  - 1./270.*RD1456ep*
         RSp45613e + 1./540.*RD1456ep*RSp45633e + 1./540.*RSp45611e*
         RD1456ep + 1./135.*mp2*RSp45613e*RS14563pe - 1./270.*mp2*
         RSp45633e*RS14563pe - 1./270.*mp2*RC1456pe*RSp45613e + 1./540.
         *mp2*RC1456pe*RSp45633e - 1./270.*mp2*RSp45611e*RS14563pe + 1./
         540.*mp2*RSp45611e*RC1456pe + 1./162.*m66*RSp456e*pow(
         RS145pe,2) + 1./162.*m55*RSp465e*pow(RS146pe,2) + 1./162.*m44*
         RSp564e*pow(RS156pe,2) + 2./45.*m33*RSp45613e*RS14563pe - 1./
         90.*m33*RSp45633e*RS14563pe - 1./405.*m33*RC1456pe*RSp45613e
          - 11./810.*m33*RC1456pe*RSp45633e + 1./270.*m33*RSp45611e*
         RS14563pe + 13./810.*m33*RSp45611e*RC1456pe - 7./405.*m13*
         RSp45613e*RS14563pe + 7./810.*m13*RSp45633e*RS14563pe + 7./810.
         *m13*RC1456pe*RSp45613e - 7./1620.*m13*RC1456pe*RSp45633e + 7./
         810.*m13*RSp45611e*RS14563pe - 7./1620.*m13*RSp45611e*RC1456pe
          + 16./405.*m11*RSp45613e*RS14563pe + 1./162.*m11*RSp45633e*
         RS14563pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(mp2,L) * ( 13./810.*m11*RC1456pe*
         RSp45613e - 19./1620.*m11*RC1456pe*RSp45633e + 23./810.*m11*
         RSp45611e*RS14563pe - 37./1620.*m11*RSp45611e*RC1456pe + 1./81.
         *RS(m11,m44,m55,m66,m33,me2,mp2)*m33*RSp45613e - 1./162.*RS(
         m11,m44,m55,m66,m33,me2,mp2)*m33*RSp45633e - 1./162.*RS(m11,
         m44,m55,m66,m33,me2,mp2)*m33*RSp45611e + 2./81.*RS(m11,m44,m55
         ,m66,m33,me2,mp2)*m11*RSp45613e - 1./81.*RS(m11,m44,m55,m66,
         m33,me2,mp2)*m11*RSp45633e - 1./81.*RS(m11,m44,m55,m66,m33,me2
         ,mp2)*m11*RSp45611e );

      M13p6x23RV +=  + AbV(m11,L)*AbV(me2,L) * (  - 1./270.*RD1456ep*
         RSe45613p + 1./540.*RD1456ep*RSe45633p + 1./540.*RSe45611p*
         RD1456ep + 1./135.*me2*RSe45613p*RS14563pe - 1./270.*me2*
         RSe45633p*RS14563pe - 1./270.*me2*RC1456pe*RSe45613p + 1./540.
         *me2*RC1456pe*RSe45633p - 1./270.*me2*RSe45611p*RS14563pe + 1./
         540.*me2*RSe45611p*RC1456pe + 1./162.*m66*RSe456p*pow(
         RS145pe,2) + 1./162.*m55*RSe465p*pow(RS146pe,2) + 1./162.*m44*
         RSe564p*pow(RS156pe,2) + 2./45.*m33*RSe45613p*RS14563pe - 1./
         90.*m33*RSe45633p*RS14563pe - 1./405.*m33*RC1456pe*RSe45613p
          - 11./810.*m33*RC1456pe*RSe45633p + 1./270.*m33*RSe45611p*
         RS14563pe + 13./810.*m33*RSe45611p*RC1456pe - 7./405.*m13*
         RSe45613p*RS14563pe + 7./810.*m13*RSe45633p*RS14563pe + 7./810.
         *m13*RC1456pe*RSe45613p - 7./1620.*m13*RC1456pe*RSe45633p + 7./
         810.*m13*RSe45611p*RS14563pe - 7./1620.*m13*RSe45611p*RC1456pe
          + 16./405.*m11*RSe45613p*RS14563pe + 1./162.*m11*RSe45633p*
         RS14563pe );

      M13p6x23RV +=  + AbV(m11,L)*AbV(me2,L) * ( 13./810.*m11*RC1456pe*
         RSe45613p - 19./1620.*m11*RC1456pe*RSe45633p + 23./810.*m11*
         RSe45611p*RS14563pe - 37./1620.*m11*RSe45611p*RC1456pe + 1./81.
         *RS(m11,m44,m55,m66,m33,me2,mp2)*m33*RSe45613p - 1./162.*RS(
         m11,m44,m55,m66,m33,me2,mp2)*m33*RSe45633p - 1./162.*RS(m11,
         m44,m55,m66,m33,me2,mp2)*m33*RSe45611p + 2./81.*RS(m11,m44,m55
         ,m66,m33,me2,mp2)*m11*RSe45613p - 1./81.*RS(m11,m44,m55,m66,
         m33,me2,mp2)*m11*RSe45633p - 1./81.*RS(m11,m44,m55,m66,m33,me2
         ,mp2)*m11*RSe45611p );

      M13p6x23RV +=  + AbV(m11,L)*BbV(m11,m33,L) * ( 1./54.*m11*m33*
         RC1456pe*RS34561pe + 1./27.*m11*m13*RC1456pe*RS34561pe + 1./18.
         *pow(m11,2)*RC1456pe*RS34561pe );

      M13p6x23RV +=  + AbV(m11,L)*BbV(m11,mp2,L) * ( 1./54.*m11*m33*
         RC1456pe*RSp45613e + 1./54.*m11*m33*RSp45611e*RC1456pe + 1./27.
         *m11*m13*RC1456pe*RSp45613e - 1./27.*m11*m13*RSp45611e*
         RC1456pe + 1./18.*pow(m11,2)*RC1456pe*RSp45613e + 1./54.*pow(
         m11,2)*RSp45611e*RC1456pe );

      M13p6x23RV +=  + AbV(m11,L)*BbV(m11,me2,L) * ( 1./54.*m11*m33*
         RC1456pe*RSe45613p + 1./54.*m11*m33*RSe45611p*RC1456pe + 1./27.
         *m11*m13*RC1456pe*RSe45613p - 1./27.*m11*m13*RSe45611p*
         RC1456pe + 1./18.*pow(m11,2)*RC1456pe*RSe45613p + 1./54.*pow(
         m11,2)*RSe45611p*RC1456pe );

      M13p6x23RV +=  + AbV(m11,L)*BbV(m11,L) * ( 1./540.*pow(
         RD1456ep,2) + 1./270.*m33*RD1456ep*RS14563pe + 13./810.*m33*
         RD1456ep*RC1456pe + 7./810.*m13*RD1456ep*RS14563pe - 7./1620.*
         m13*RD1456ep*RC1456pe + 2./81.*m11*RD1456ep*RS14563pe - 17./
         810.*m11*RD1456ep*RC1456pe + 1./12.*m11*m33 + 1./54.*m11*m33*
         RC1456pe*RS14563pe + 1./54.*m11*m33*pow(RC1456pe,2) - 1./6.*
         m11*m13 + 1./27.*m11*m13*RC1456pe*RS14563pe - 1./27.*m11*m13*
         pow(RC1456pe,2) + 1./12.*pow(m11,2) + 1./18.*pow(m11,2)*
         RC1456pe*RS14563pe + 1./54.*pow(m11,2)*pow(RC1456pe,2) - 1./
         162.*RS(m11,m44,m55,m66,m33,me2,mp2)*m33*RD1456ep - 1./81.*RS(
         m11,m44,m55,m66,m33,me2,mp2)*m11*RD1456ep );

      M13p6x23RV +=  + AbV(m11,L)*BbV(m14,L) * ( 1./216.*m44*RD1456ep
          - 1./432.*pow(m44,2)*RC1456pe - 1./216.*pow(m44,2)*RS156pe + 
         1./216.*m33*RD1456ep - 1./432.*m33*m44*RC1456pe - 1./216.*m33*
         m44*RS156pe - 1./108.*m14*RD1456ep + 1./108.*m14*m44*RC1456pe
          + 1./216.*m14*m33*RC1456pe - 1./108.*m14*m33*RS156pe - 1./108.
         *pow(m14,2)*RC1456pe + 1./54.*pow(m14,2)*RS156pe - 1./108.*m13
         *RD1456ep + 1./216.*m13*m44*RC1456pe + 1./108.*m13*m44*RS156pe
          - 1./108.*m13*m14*RC1456pe + 1./54.*m13*m14*RS156pe + 1./108.
         *m11*RD1456ep - 1./144.*m11*m44*RC1456pe - 5./216.*m11*m44*
         RS156pe - 1./432.*m11*m33*RC1456pe - 1./72.*m11*m33*RS156pe + 
         1./72.*m11*m14*RC1456pe + 1./108.*m11*m14*RS156pe + 1./216.*
         m11*m13*RC1456pe + 1./36.*m11*m13*RS156pe - 1./216.*pow(m11,2)
         *RC1456pe - 1./36.*pow(m11,2)*RS156pe );

      M13p6x23RV +=  + AbV(m11,L)*BbV(m15,L) * ( 1./216.*m55*RD1456ep
          - 1./432.*pow(m55,2)*RC1456pe - 1./216.*pow(m55,2)*RS146pe + 
         1./216.*m33*RD1456ep - 1./432.*m33*m55*RC1456pe - 1./216.*m33*
         m55*RS146pe - 1./108.*m15*RD1456ep + 1./108.*m15*m55*RC1456pe
          + 1./216.*m15*m33*RC1456pe - 1./108.*m15*m33*RS146pe - 1./108.
         *pow(m15,2)*RC1456pe + 1./54.*pow(m15,2)*RS146pe - 1./108.*m13
         *RD1456ep + 1./216.*m13*m55*RC1456pe + 1./108.*m13*m55*RS146pe
          - 1./108.*m13*m15*RC1456pe + 1./54.*m13*m15*RS146pe + 1./108.
         *m11*RD1456ep - 1./144.*m11*m55*RC1456pe - 5./216.*m11*m55*
         RS146pe - 1./432.*m11*m33*RC1456pe - 1./72.*m11*m33*RS146pe + 
         1./72.*m11*m15*RC1456pe + 1./108.*m11*m15*RS146pe + 1./216.*
         m11*m13*RC1456pe + 1./36.*m11*m13*RS146pe - 1./216.*pow(m11,2)
         *RC1456pe - 1./36.*pow(m11,2)*RS146pe );

      M13p6x23RV +=  + AbV(m11,L)*BbV(m16,L) * ( 1./216.*m66*RD1456ep
          - 1./432.*pow(m66,2)*RC1456pe - 1./216.*pow(m66,2)*RS145pe + 
         1./216.*m33*RD1456ep - 1./432.*m33*m66*RC1456pe - 1./216.*m33*
         m66*RS145pe - 1./108.*m16*RD1456ep + 1./108.*m16*m66*RC1456pe
          + 1./216.*m16*m33*RC1456pe - 1./108.*m16*m33*RS145pe - 1./108.
         *pow(m16,2)*RC1456pe + 1./54.*pow(m16,2)*RS145pe - 1./108.*m13
         *RD1456ep + 1./216.*m13*m66*RC1456pe + 1./108.*m13*m66*RS145pe
          - 1./108.*m13*m16*RC1456pe + 1./54.*m13*m16*RS145pe + 1./108.
         *m11*RD1456ep - 1./144.*m11*m66*RC1456pe - 5./216.*m11*m66*
         RS145pe - 1./432.*m11*m33*RC1456pe - 1./72.*m11*m33*RS145pe + 
         1./72.*m11*m16*RC1456pe + 1./108.*m11*m16*RS145pe + 1./216.*
         m11*m13*RC1456pe + 1./36.*m11*m13*RS145pe - 1./216.*pow(m11,2)
         *RC1456pe - 1./36.*pow(m11,2)*RS145pe );

      M13p6x23RV +=  + AbV(m11,L)*BbV(m33,L) * ( 1./540.*RD1456ep*
         RD3456ep - 2./135.*m33*RD3456ep*RS14563pe - 19./1620.*m33*
         RC1456pe*RD3456ep + 7./810.*m13*RD3456ep*RS14563pe - 7./1620.*
         m13*RC1456pe*RD3456ep + 1./162.*m11*RD3456ep*RS14563pe - 19./
         1620.*m11*RC1456pe*RD3456ep - 1./162.*RS(m11,m44,m55,m66,m33,
         me2,mp2)*m33*RD3456ep - 1./81.*RS(m11,m44,m55,m66,m33,me2,mp2)
         *m11*RD3456ep );

      M13p6x23RV +=  + AbV(m11,L)*CbV(m11,L) * ( 1./54.*m11*m33*
         RD1456ep*RC1456pe - 1./27.*m11*m13*RD1456ep*RC1456pe + 1./54.*
         pow(m11,2)*RD1456ep*RC1456pe );

      M13p6x23RV +=  + AbV(m13,L)*pi16 * (  - 1./4.*m13*m33 + 1./2.*
         pow(m13,2) + 1./36.*pow(m13,2)*pow(RS34561pe,2) + 1./18.*pow(
         m13,2)*RS14563pe*RS34561pe + 1./36.*pow(m13,2)*pow(
         RS14563pe,2) + 1./18.*pow(m13,2)*RSe45613p*RS34561pe + 1./18.*
         pow(m13,2)*RSe45613p*RS14563pe + 1./36.*pow(m13,2)*pow(
         RSe45613p,2) + 1./18.*pow(m13,2)*RSp45613e*RS34561pe + 1./18.*
         pow(m13,2)*RSp45613e*RS14563pe + 1./18.*pow(m13,2)*RSp45613e*
         RSe45613p + 1./36.*pow(m13,2)*pow(RSp45613e,2) - 1./36.*pow(
         m13,2)*RC3456pe*RS34561pe - 1./36.*pow(m13,2)*RC3456pe*
         RS14563pe - 1./36.*pow(m13,2)*RC3456pe*RSe45613p - 1./36.*pow(
         m13,2)*RC3456pe*RSp45613e + 1./144.*pow(m13,2)*pow(RC3456pe,2)
          - 1./36.*pow(m13,2)*RSe45633p*RS34561pe - 1./36.*pow(m13,2)*
         RSe45633p*RS14563pe - 1./36.*pow(m13,2)*RSe45633p*RSe45613p - 
         1./36.*pow(m13,2)*RSe45633p*RSp45613e + 1./72.*pow(m13,2)*
         RSe45633p*RC3456pe + 1./144.*pow(m13,2)*pow(RSe45633p,2) - 1./
         36.*pow(m13,2)*RSp45633e*RS34561pe );

      M13p6x23RV +=  + AbV(m13,L)*pi16 * (  - 1./36.*pow(m13,2)*
         RSp45633e*RS14563pe - 1./36.*pow(m13,2)*RSp45633e*RSe45613p - 
         1./36.*pow(m13,2)*RSp45633e*RSp45613e + 1./72.*pow(m13,2)*
         RSp45633e*RC3456pe + 1./72.*pow(m13,2)*RSp45633e*RSe45633p + 1.
         /144.*pow(m13,2)*pow(RSp45633e,2) - 1./36.*pow(m13,2)*RC1456pe
         *RS34561pe - 1./36.*pow(m13,2)*RC1456pe*RS14563pe - 1./36.*
         pow(m13,2)*RC1456pe*RSe45613p - 1./36.*pow(m13,2)*RC1456pe*
         RSp45613e + 1./72.*pow(m13,2)*RC1456pe*RC3456pe + 1./72.*pow(
         m13,2)*RC1456pe*RSe45633p + 1./72.*pow(m13,2)*RC1456pe*
         RSp45633e + 1./144.*pow(m13,2)*pow(RC1456pe,2) - 1./36.*pow(
         m13,2)*RSe45611p*RS34561pe - 1./36.*pow(m13,2)*RSe45611p*
         RS14563pe - 1./36.*pow(m13,2)*RSe45611p*RSe45613p - 1./36.*
         pow(m13,2)*RSe45611p*RSp45613e + 1./72.*pow(m13,2)*RSe45611p*
         RC3456pe + 1./72.*pow(m13,2)*RSe45611p*RSe45633p + 1./72.*pow(
         m13,2)*RSe45611p*RSp45633e + 1./72.*pow(m13,2)*RSe45611p*
         RC1456pe );

      M13p6x23RV +=  + AbV(m13,L)*pi16 * ( 1./144.*pow(m13,2)*pow(
         RSe45611p,2) - 1./36.*pow(m13,2)*RSp45611e*RS34561pe - 1./36.*
         pow(m13,2)*RSp45611e*RS14563pe - 1./36.*pow(m13,2)*RSp45611e*
         RSe45613p - 1./36.*pow(m13,2)*RSp45611e*RSp45613e + 1./72.*
         pow(m13,2)*RSp45611e*RC3456pe + 1./72.*pow(m13,2)*RSp45611e*
         RSe45633p + 1./72.*pow(m13,2)*RSp45611e*RSp45633e + 1./72.*
         pow(m13,2)*RSp45611e*RC1456pe + 1./72.*pow(m13,2)*RSp45611e*
         RSe45611p + 1./144.*pow(m13,2)*pow(RSp45611e,2) - 1./4.*m11*
         m13 );

      M13p6x23RV +=  + pow(AbV(m13,L),2) * ( 1./60.*m33 - 17./60.*m13
          + 1./60.*m11 );

      M13p6x23RV +=  + AbV(m13,L)*AbV(m33,L) * (  - 2./81.*RD3456ep*
         RS34561pe - 1./54.*RD3456ep*RS14563pe - 1./54.*RD3456ep*
         RSe45613p - 1./54.*RD3456ep*RSp45613e + 1./81.*RD3456ep*
         RC3456pe + 1./108.*RSe45633p*RD3456ep + 1./108.*RSp45633e*
         RD3456ep + 1./108.*RC1456pe*RD3456ep - 1./162.*RD1456ep*
         RS34561pe + 1./324.*RD1456ep*RC3456pe + 1./108.*RSe45611p*
         RD3456ep + 1./108.*RSp45611e*RD3456ep + 1./81.*me2*RSe45613p*
         RS34561pe - 1./162.*me2*RC3456pe*RSe45613p - 1./162.*me2*
         RSe45633p*RS34561pe + 1./324.*me2*RSe45633p*RC3456pe - 1./162.
         *me2*RSe45611p*RS34561pe + 1./324.*me2*RSe45611p*RC3456pe + 1./
         81.*mp2*RSp45613e*RS34561pe - 1./162.*mp2*RC3456pe*RSp45613e
          - 1./162.*mp2*RSp45633e*RS34561pe + 1./324.*mp2*RSp45633e*
         RC3456pe - 1./162.*mp2*RSp45611e*RS34561pe + 1./324.*mp2*
         RSp45611e*RC3456pe + 3./20.*m33 + 1./27.*m33*pow(RS34561pe,2)
          + 2./81.*m33*RS14563pe*RS34561pe + 2./81.*m33*RSe45613p*
         RS34561pe );

      M13p6x23RV +=  + AbV(m13,L)*AbV(m33,L) * ( 2./81.*m33*RSp45613e*
         RS34561pe - 2./81.*m33*RC3456pe*RS34561pe - 1./162.*m33*
         RC3456pe*RS14563pe - 1./162.*m33*RC3456pe*RSe45613p - 1./162.*
         m33*RC3456pe*RSp45613e - 1./162.*m33*pow(RC3456pe,2) - 1./162.
         *m33*RSe45633p*RS34561pe - 1./108.*m33*RSe45633p*RC3456pe - 1./
         162.*m33*RSp45633e*RS34561pe - 1./108.*m33*RSp45633e*RC3456pe
          - 1./54.*m33*RC1456pe*RS34561pe + 5./324.*m33*RC1456pe*
         RC3456pe - 1./54.*m33*RSe45611p*RS34561pe + 5./324.*m33*
         RSe45611p*RC3456pe - 1./54.*m33*RSp45611e*RS34561pe + 5./324.*
         m33*RSp45611e*RC3456pe + 41./180.*m13 - 2./27.*m13*pow(
         RS34561pe,2) - 2./27.*m13*RS14563pe*RS34561pe - 2./27.*m13*
         RSe45613p*RS34561pe - 2./27.*m13*RSp45613e*RS34561pe + 2./27.*
         m13*RC3456pe*RS34561pe + 1./27.*m13*RC3456pe*RS14563pe + 1./27.
         *m13*RC3456pe*RSe45613p + 1./27.*m13*RC3456pe*RSp45613e - 1./
         54.*m13*pow(RC3456pe,2) + 1./27.*m13*RSe45633p*RS34561pe - 1./
         54.*m13*RSe45633p*RC3456pe );

      M13p6x23RV +=  + AbV(m13,L)*AbV(m33,L) * ( 1./27.*m13*RSp45633e*
         RS34561pe - 1./54.*m13*RSp45633e*RC3456pe + 1./27.*m13*
         RC1456pe*RS34561pe - 1./54.*m13*RC1456pe*RC3456pe + 1./27.*m13
         *RSe45611p*RS34561pe - 1./54.*m13*RSe45611p*RC3456pe + 1./27.*
         m13*RSp45611e*RS34561pe - 1./54.*m13*RSp45611e*RC3456pe + 17./
         180.*m11 - 1./81.*m11*pow(RS34561pe,2) - 1./81.*m11*RSe45613p*
         RS34561pe - 1./81.*m11*RSp45613e*RS34561pe - 1./162.*m11*
         RC3456pe*RS14563pe - 1./162.*m11*pow(RC3456pe,2) - 1./162.*m11
         *RSe45633p*RC3456pe - 1./162.*m11*RSp45633e*RC3456pe + 1./162.
         *m11*RC1456pe*RS34561pe + 1./108.*m11*RC1456pe*RC3456pe + 1./
         81.*m11*RSe45611p*RS34561pe + 1./162.*m11*RSe45611p*RC3456pe
          + 1./81.*m11*RSp45611e*RS34561pe + 1./162.*m11*RSp45611e*
         RC3456pe );

      M13p6x23RV +=  + AbV(m13,L)*AbV(mp2,L) * (  - 1./162.*RD3456ep*
         RSp45613e + 1./324.*RSp45633e*RD3456ep - 1./162.*RD1456ep*
         RSp45613e + 1./324.*RD1456ep*RSp45633e + 1./324.*RSp45611e*
         RD3456ep + 1./324.*RSp45611e*RD1456ep + 1./81.*me2*RSp45613e*
         RSe45613p - 1./162.*me2*RSe45633p*RSp45613e - 1./162.*me2*
         RSp45633e*RSe45613p + 1./324.*me2*RSp45633e*RSe45633p - 1./162.
         *me2*RSe45611p*RSp45613e + 1./324.*me2*RSe45611p*RSp45633e - 1.
         /162.*me2*RSp45611e*RSe45613p + 1./324.*me2*RSp45611e*
         RSe45633p + 1./324.*me2*RSp45611e*RSe45611p + 1./27.*mp2*
         RSp45613e*RS34561pe + 1./27.*mp2*RSp45613e*RS14563pe + 1./27.*
         mp2*RSp45613e*RSe45613p + 4./81.*mp2*pow(RSp45613e,2) - 1./54.
         *mp2*RC3456pe*RSp45613e - 1./54.*mp2*RSe45633p*RSp45613e - 1./
         54.*mp2*RSp45633e*RS34561pe - 1./54.*mp2*RSp45633e*RS14563pe
          - 1./54.*mp2*RSp45633e*RSe45613p - 4./81.*mp2*RSp45633e*
         RSp45613e + 1./108.*mp2*RSp45633e*RC3456pe + 1./108.*mp2*
         RSp45633e*RSe45633p );

      M13p6x23RV +=  + AbV(m13,L)*AbV(mp2,L) * ( 1./81.*mp2*pow(
         RSp45633e,2) - 1./54.*mp2*RC1456pe*RSp45613e + 1./108.*mp2*
         RC1456pe*RSp45633e - 1./54.*mp2*RSe45611p*RSp45613e + 1./108.*
         mp2*RSe45611p*RSp45633e - 1./54.*mp2*RSp45611e*RS34561pe - 1./
         54.*mp2*RSp45611e*RS14563pe - 1./54.*mp2*RSp45611e*RSe45613p
          - 4./81.*mp2*RSp45611e*RSp45613e + 1./108.*mp2*RSp45611e*
         RC3456pe + 1./108.*mp2*RSp45611e*RSe45633p + 2./81.*mp2*
         RSp45611e*RSp45633e + 1./108.*mp2*RSp45611e*RC1456pe + 1./108.
         *mp2*RSp45611e*RSe45611p + 1./81.*mp2*pow(RSp45611e,2) - 1./81.
         *m33*RSp45613e*RS14563pe - 1./81.*m33*RSp45613e*RSe45613p - 1./
         81.*m33*pow(RSp45613e,2) + 1./162.*m33*RC3456pe*RSp45613e + 1./
         81.*m33*RSe45633p*RSp45613e + 1./162.*m33*RSp45633e*RS34561pe
          + 1./81.*m33*RSp45633e*RS14563pe + 1./81.*m33*RSp45633e*
         RSe45613p + 2./81.*m33*RSp45633e*RSp45613e - 5./324.*m33*
         RSp45633e*RC3456pe - 1./54.*m33*RSp45633e*RSe45633p - 1./54.*
         m33*pow(RSp45633e,2) );

      M13p6x23RV +=  + AbV(m13,L)*AbV(mp2,L) * ( 1./162.*m33*RC1456pe*
         RSp45633e + 1./162.*m33*RSe45611p*RSp45633e - 1./162.*m33*
         RSp45611e*RS34561pe + 1./108.*m33*RSp45611e*RC3456pe + 1./162.
         *m33*RSp45611e*RSe45633p + 1./81.*m33*RSp45611e*RSp45633e - 1./
         162.*m33*RSp45611e*RC1456pe - 1./162.*m33*RSp45611e*RSe45611p
          - 1./162.*m33*pow(RSp45611e,2) - 2./27.*m13*RSp45613e*
         RS34561pe - 2./27.*m13*RSp45613e*RS14563pe - 2./27.*m13*
         RSp45613e*RSe45613p - 2./27.*m13*pow(RSp45613e,2) + 1./27.*m13
         *RC3456pe*RSp45613e + 1./27.*m13*RSe45633p*RSp45613e + 1./27.*
         m13*RSp45633e*RS34561pe + 1./27.*m13*RSp45633e*RS14563pe + 1./
         27.*m13*RSp45633e*RSe45613p + 2./27.*m13*RSp45633e*RSp45613e
          - 1./54.*m13*RSp45633e*RC3456pe - 1./54.*m13*RSp45633e*
         RSe45633p - 1./54.*m13*pow(RSp45633e,2) + 1./27.*m13*RC1456pe*
         RSp45613e - 1./54.*m13*RC1456pe*RSp45633e + 1./27.*m13*
         RSe45611p*RSp45613e - 1./54.*m13*RSe45611p*RSp45633e + 1./27.*
         m13*RSp45611e*RS34561pe );

      M13p6x23RV +=  + AbV(m13,L)*AbV(mp2,L) * ( 1./27.*m13*RSp45611e*
         RS14563pe + 1./27.*m13*RSp45611e*RSe45613p + 2./27.*m13*
         RSp45611e*RSp45613e - 1./54.*m13*RSp45611e*RC3456pe - 1./54.*
         m13*RSp45611e*RSe45633p - 1./27.*m13*RSp45611e*RSp45633e - 1./
         54.*m13*RSp45611e*RC1456pe - 1./54.*m13*RSp45611e*RSe45611p - 
         1./54.*m13*pow(RSp45611e,2) - 1./81.*m11*RSp45613e*RS34561pe
          - 1./81.*m11*RSp45613e*RSe45613p - 1./81.*m11*pow(
         RSp45613e,2) - 1./162.*m11*RSp45633e*RS14563pe - 1./162.*m11*
         RSp45633e*RC3456pe - 1./162.*m11*RSp45633e*RSe45633p - 1./162.
         *m11*pow(RSp45633e,2) + 1./162.*m11*RC1456pe*RSp45613e + 1./
         108.*m11*RC1456pe*RSp45633e + 1./81.*m11*RSe45611p*RSp45613e
          + 1./162.*m11*RSe45611p*RSp45633e + 1./81.*m11*RSp45611e*
         RS34561pe + 1./162.*m11*RSp45611e*RS14563pe + 1./81.*m11*
         RSp45611e*RSe45613p + 2./81.*m11*RSp45611e*RSp45613e + 1./162.
         *m11*RSp45611e*RC3456pe + 1./162.*m11*RSp45611e*RSe45633p + 1./
         81.*m11*RSp45611e*RSp45633e );

      M13p6x23RV +=  + AbV(m13,L)*AbV(mp2,L) * (  - 5./324.*m11*
         RSp45611e*RC1456pe - 1./54.*m11*RSp45611e*RSe45611p - 1./54.*
         m11*pow(RSp45611e,2) );

      M13p6x23RV +=  + AbV(m13,L)*AbV(me2,L) * (  - 1./162.*RD3456ep*
         RSe45613p + 1./324.*RSe45633p*RD3456ep - 1./162.*RD1456ep*
         RSe45613p + 1./324.*RD1456ep*RSe45633p + 1./324.*RSe45611p*
         RD3456ep + 1./324.*RSe45611p*RD1456ep + 1./27.*me2*RSe45613p*
         RS34561pe + 1./27.*me2*RSe45613p*RS14563pe + 4./81.*me2*pow(
         RSe45613p,2) + 1./27.*me2*RSp45613e*RSe45613p - 1./54.*me2*
         RC3456pe*RSe45613p - 1./54.*me2*RSe45633p*RS34561pe - 1./54.*
         me2*RSe45633p*RS14563pe - 4./81.*me2*RSe45633p*RSe45613p - 1./
         54.*me2*RSe45633p*RSp45613e + 1./108.*me2*RSe45633p*RC3456pe
          + 1./81.*me2*pow(RSe45633p,2) - 1./54.*me2*RSp45633e*
         RSe45613p + 1./108.*me2*RSp45633e*RSe45633p - 1./54.*me2*
         RC1456pe*RSe45613p + 1./108.*me2*RC1456pe*RSe45633p - 1./54.*
         me2*RSe45611p*RS34561pe - 1./54.*me2*RSe45611p*RS14563pe - 4./
         81.*me2*RSe45611p*RSe45613p - 1./54.*me2*RSe45611p*RSp45613e
          + 1./108.*me2*RSe45611p*RC3456pe + 2./81.*me2*RSe45611p*
         RSe45633p );

      M13p6x23RV +=  + AbV(m13,L)*AbV(me2,L) * ( 1./108.*me2*RSe45611p*
         RSp45633e + 1./108.*me2*RSe45611p*RC1456pe + 1./81.*me2*pow(
         RSe45611p,2) - 1./54.*me2*RSp45611e*RSe45613p + 1./108.*me2*
         RSp45611e*RSe45633p + 1./108.*me2*RSp45611e*RSe45611p + 1./81.
         *mp2*RSp45613e*RSe45613p - 1./162.*mp2*RSe45633p*RSp45613e - 1.
         /162.*mp2*RSp45633e*RSe45613p + 1./324.*mp2*RSp45633e*
         RSe45633p - 1./162.*mp2*RSe45611p*RSp45613e + 1./324.*mp2*
         RSe45611p*RSp45633e - 1./162.*mp2*RSp45611e*RSe45613p + 1./324.
         *mp2*RSp45611e*RSe45633p + 1./324.*mp2*RSp45611e*RSe45611p - 1.
         /81.*m33*RSe45613p*RS14563pe - 1./81.*m33*pow(RSe45613p,2) - 1.
         /81.*m33*RSp45613e*RSe45613p + 1./162.*m33*RC3456pe*RSe45613p
          + 1./162.*m33*RSe45633p*RS34561pe + 1./81.*m33*RSe45633p*
         RS14563pe + 2./81.*m33*RSe45633p*RSe45613p + 1./81.*m33*
         RSe45633p*RSp45613e - 5./324.*m33*RSe45633p*RC3456pe - 1./54.*
         m33*pow(RSe45633p,2) + 1./81.*m33*RSp45633e*RSe45613p - 1./54.
         *m33*RSp45633e*RSe45633p );

      M13p6x23RV +=  + AbV(m13,L)*AbV(me2,L) * ( 1./162.*m33*RC1456pe*
         RSe45633p - 1./162.*m33*RSe45611p*RS34561pe + 1./108.*m33*
         RSe45611p*RC3456pe + 1./81.*m33*RSe45611p*RSe45633p + 1./162.*
         m33*RSe45611p*RSp45633e - 1./162.*m33*RSe45611p*RC1456pe - 1./
         162.*m33*pow(RSe45611p,2) + 1./162.*m33*RSp45611e*RSe45633p - 
         1./162.*m33*RSp45611e*RSe45611p - 2./27.*m13*RSe45613p*
         RS34561pe - 2./27.*m13*RSe45613p*RS14563pe - 2./27.*m13*pow(
         RSe45613p,2) - 2./27.*m13*RSp45613e*RSe45613p + 1./27.*m13*
         RC3456pe*RSe45613p + 1./27.*m13*RSe45633p*RS34561pe + 1./27.*
         m13*RSe45633p*RS14563pe + 2./27.*m13*RSe45633p*RSe45613p + 1./
         27.*m13*RSe45633p*RSp45613e - 1./54.*m13*RSe45633p*RC3456pe - 
         1./54.*m13*pow(RSe45633p,2) + 1./27.*m13*RSp45633e*RSe45613p
          - 1./54.*m13*RSp45633e*RSe45633p + 1./27.*m13*RC1456pe*
         RSe45613p - 1./54.*m13*RC1456pe*RSe45633p + 1./27.*m13*
         RSe45611p*RS34561pe + 1./27.*m13*RSe45611p*RS14563pe + 2./27.*
         m13*RSe45611p*RSe45613p );

      M13p6x23RV +=  + AbV(m13,L)*AbV(me2,L) * ( 1./27.*m13*RSe45611p*
         RSp45613e - 1./54.*m13*RSe45611p*RC3456pe - 1./27.*m13*
         RSe45611p*RSe45633p - 1./54.*m13*RSe45611p*RSp45633e - 1./54.*
         m13*RSe45611p*RC1456pe - 1./54.*m13*pow(RSe45611p,2) + 1./27.*
         m13*RSp45611e*RSe45613p - 1./54.*m13*RSp45611e*RSe45633p - 1./
         54.*m13*RSp45611e*RSe45611p - 1./81.*m11*RSe45613p*RS34561pe
          - 1./81.*m11*pow(RSe45613p,2) - 1./81.*m11*RSp45613e*
         RSe45613p - 1./162.*m11*RSe45633p*RS14563pe - 1./162.*m11*
         RSe45633p*RC3456pe - 1./162.*m11*pow(RSe45633p,2) - 1./162.*
         m11*RSp45633e*RSe45633p + 1./162.*m11*RC1456pe*RSe45613p + 1./
         108.*m11*RC1456pe*RSe45633p + 1./81.*m11*RSe45611p*RS34561pe
          + 1./162.*m11*RSe45611p*RS14563pe + 2./81.*m11*RSe45611p*
         RSe45613p + 1./81.*m11*RSe45611p*RSp45613e + 1./162.*m11*
         RSe45611p*RC3456pe + 1./81.*m11*RSe45611p*RSe45633p + 1./162.*
         m11*RSe45611p*RSp45633e - 5./324.*m11*RSe45611p*RC1456pe - 1./
         54.*m11*pow(RSe45611p,2) );

      M13p6x23RV +=  + AbV(m13,L)*AbV(me2,L) * ( 1./81.*m11*RSp45611e*
         RSe45613p + 1./162.*m11*RSp45611e*RSe45633p - 1./54.*m11*
         RSp45611e*RSe45611p );

      M13p6x23RV +=  + AbV(m13,L)*BbV(m11,m33,L) * ( 1./36.*pow(m33,2)
          + 1./9.*m13*m33 + 1./9.*pow(m13,2) + 5./18.*m11*m33 + 1./3.*
         m11*m13 + 5./36.*pow(m11,2) );

      M13p6x23RV +=  + AbV(m13,L)*BbV(m11,L) * ( 1./324.*RD1456ep*
         RD3456ep + 1./324.*pow(RD1456ep,2) - 1./162.*me2*RD1456ep*
         RSe45613p + 1./324.*me2*RD1456ep*RSe45633p + 1./324.*me2*
         RSe45611p*RD1456ep - 1./162.*mp2*RD1456ep*RSp45613e + 1./324.*
         mp2*RD1456ep*RSp45633e + 1./324.*mp2*RSp45611e*RD1456ep - 1./
         162.*m33*RD1456ep*RS34561pe + 1./108.*m33*RD1456ep*RC3456pe + 
         1./162.*m33*RD1456ep*RSe45633p + 1./162.*m33*RD1456ep*
         RSp45633e - 1./162.*m33*RD1456ep*RC1456pe - 1./162.*m33*
         RSe45611p*RD1456ep - 1./162.*m33*RSp45611e*RD1456ep + 1./27.*
         m13*RD1456ep*RS34561pe + 1./27.*m13*RD1456ep*RS14563pe + 1./27.
         *m13*RD1456ep*RSe45613p + 1./27.*m13*RD1456ep*RSp45613e - 1./
         54.*m13*RD1456ep*RC3456pe - 1./54.*m13*RD1456ep*RSe45633p - 1./
         54.*m13*RD1456ep*RSp45633e - 1./54.*m13*RD1456ep*RC1456pe - 1./
         54.*m13*RSe45611p*RD1456ep - 1./54.*m13*RSp45611e*RD1456ep - 1.
         /162.*m11*RD1456ep*RS34561pe - 1./81.*m11*RD1456ep*RS14563pe
          - 1./162.*m11*RD1456ep*RSe45613p );

      M13p6x23RV +=  + AbV(m13,L)*BbV(m11,L) * (  - 1./162.*m11*
         RD1456ep*RSp45613e + 5./324.*m11*RD1456ep*RC3456pe + 5./324.*
         m11*RD1456ep*RSe45633p + 5./324.*m11*RD1456ep*RSp45633e - 1./
         162.*m11*RD1456ep*RC1456pe - 1./108.*m11*RSe45611p*RD1456ep - 
         1./108.*m11*RSp45611e*RD1456ep );

      M13p6x23RV +=  + AbV(m13,L)*BbV(m33,L) * ( 1./324.*pow(
         RD3456ep,2) + 1./324.*RD1456ep*RD3456ep - 1./162.*me2*RD3456ep
         *RSe45613p + 1./324.*me2*RSe45633p*RD3456ep + 1./324.*me2*
         RSe45611p*RD3456ep - 1./162.*mp2*RD3456ep*RSp45613e + 1./324.*
         mp2*RSp45633e*RD3456ep + 1./324.*mp2*RSp45611e*RD3456ep - 1./
         81.*m33*RD3456ep*RS34561pe - 1./162.*m33*RD3456ep*RS14563pe - 
         1./162.*m33*RD3456ep*RSe45613p - 1./162.*m33*RD3456ep*
         RSp45613e - 1./162.*m33*RD3456ep*RC3456pe - 1./108.*m33*
         RSe45633p*RD3456ep - 1./108.*m33*RSp45633e*RD3456ep + 5./324.*
         m33*RC1456pe*RD3456ep + 5./324.*m33*RSe45611p*RD3456ep + 5./
         324.*m33*RSp45611e*RD3456ep + 1./27.*m13*RD3456ep*RS34561pe + 
         1./27.*m13*RD3456ep*RS14563pe + 1./27.*m13*RD3456ep*RSe45613p
          + 1./27.*m13*RD3456ep*RSp45613e - 1./54.*m13*RD3456ep*
         RC3456pe - 1./54.*m13*RSe45633p*RD3456ep - 1./54.*m13*
         RSp45633e*RD3456ep - 1./54.*m13*RC1456pe*RD3456ep - 1./54.*m13
         *RSe45611p*RD3456ep );

      M13p6x23RV +=  + AbV(m13,L)*BbV(m33,L) * (  - 1./54.*m13*
         RSp45611e*RD3456ep - 1./162.*m11*RD3456ep*RS14563pe - 1./162.*
         m11*RD3456ep*RC3456pe - 1./162.*m11*RSe45633p*RD3456ep - 1./
         162.*m11*RSp45633e*RD3456ep + 1./108.*m11*RC1456pe*RD3456ep + 
         1./162.*m11*RSe45611p*RD3456ep + 1./162.*m11*RSp45611e*
         RD3456ep );

      M13p6x23RV +=  + AbV(m14,L)*pi16 * (  - 1./24.*m13*RD3456ep - 1./
         24.*m13*me2*RSe45633p + 1./12.*m13*me2*RSe563p - 1./24.*m13*
         me2*RSe564p - 1./24.*m13*mp2*RSp45633e + 1./12.*m13*mp2*
         RSp563e - 1./24.*m13*mp2*RSp564e + 1./8.*m13*m46 + 1./8.*m13*
         m45 + 1./8.*m13*m44 - 1./24.*m13*m44*RS456pe + 1./8.*m13*m36
          + 1./8.*m13*m35 + 1./8.*m13*m34 - 1./24.*m13*m34*RC3456pe - 1.
         /24.*m13*m34*RSe45633p - 1./24.*m13*m34*RSp45633e + 1./12.*m13
         *m34*RS356pe + 1./12.*m13*m34*RSe563p + 1./12.*m13*m34*RSp563e
          - 1./24.*m13*m34*RS456pe - 1./24.*m13*m34*RSe564p - 1./24.*
         m13*m34*RSp564e - 1./24.*m13*m33*RC3456pe + 1./12.*m13*m33*
         RS356pe - 1./8.*m13*m14 + 1./24.*m13*m14*RS34561pe + 1./24.*
         m13*m14*RS14563pe + 1./24.*m13*m14*RSe45613p + 1./24.*m13*m14*
         RSp45613e - 1./24.*m13*m14*RS356pe - 1./24.*m13*m14*RSe563p - 
         1./24.*m13*m14*RSp563e - 1./24.*m13*m14*RC1456pe - 1./24.*m13*
         m14*RSe45611p - 1./24.*m13*m14*RSp45611e + 1./24.*m13*m14*
         RS156pe );

      M13p6x23RV +=  + AbV(m14,L)*pi16 * ( 1./24.*m13*m14*RSe561p + 1./
         24.*m13*m14*RSp561e - 1./8.*pow(m13,2) + 1./72.*pow(m13,2)*
         RC3456pe + 1./72.*pow(m13,2)*RSe45633p + 1./72.*pow(m13,2)*
         RSp45633e - 1./36.*pow(m13,2)*RS356pe - 1./36.*pow(m13,2)*
         RSe563p - 1./36.*pow(m13,2)*RSp563e + 1./72.*pow(m13,2)*
         RS456pe + 1./72.*pow(m13,2)*RSe564p + 1./72.*pow(m13,2)*
         RSp564e );

      M13p6x23RV +=  + pow(AbV(m14,L),2) * (  - 11./360.*m44 - 1./120.*
         m33 + 11./180.*m14 + 1./60.*m13 - 7./180.*m11 );

      M13p6x23RV +=  + AbV(m14,L)*AbV(m15,L) * (  - 11./360.*m55 - 11./
         360.*m44 - 1./60.*m33 + 11./180.*m15 + 11./180.*m14 + 1./30.*
         m13 - 7./90.*m11 );

      M13p6x23RV +=  + AbV(m14,L)*AbV(m16,L) * (  - 11./360.*m66 - 11./
         360.*m44 - 1./60.*m33 + 11./180.*m16 + 11./180.*m14 + 1./30.*
         m13 - 7./90.*m11 );

      M13p6x23RV +=  + AbV(m14,L)*AbV(m33,L) * (  - 1./180.*RD3456ep - 
         7./540.*m44*RS34561pe + 1./60.*m44*RC3456pe + 1./180.*m44*
         RS356pe + 1./27.*m34*RS34561pe - 1./27.*m34*RC3456pe + 1./27.*
         m34*RS356pe + 2./27.*m33*RS34561pe + 19./540.*m33*RC3456pe - 5.
         /54.*m33*RS356pe - 1./90.*m14*RS34561pe + 1./270.*m14*RC3456pe
          + 1./270.*m14*RS356pe - 11./540.*m13*RS34561pe + 1./20.*m13*
         RC3456pe - 43./540.*m13*RS356pe - 11./540.*m11*RS34561pe + 2./
         135.*m11*RC3456pe - 17./540.*m11*RS356pe + 1./27.*RS(m33,m44,
         m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11 );

      M13p6x23RV +=  + AbV(m14,L)*AbV(m34,L) * (  - 1./108.*RD3456ep - 
         1./108.*RD1456ep - 1./54.*me2*RSe45613p - 1./108.*me2*
         RSe45633p + 1./27.*me2*RSe563p - 1./108.*me2*RSe45611p + 1./27.
         *me2*RSe561p - 1./27.*me2*RSe564p - 1./54.*mp2*RSp45613e - 1./
         108.*mp2*RSp45633e + 1./27.*mp2*RSp563e - 1./108.*mp2*
         RSp45611e + 1./27.*mp2*RSp561e - 1./27.*mp2*RSp564e + 1./90.*
         m44 - 1./54.*m44*RS34561pe - 1./54.*m44*RS14563pe - 1./54.*m44
         *RSe45613p - 1./54.*m44*RSp45613e - 1./108.*m44*RC3456pe - 1./
         108.*m44*RSe45633p - 1./108.*m44*RSp45633e - 1./108.*m44*
         RC1456pe - 1./108.*m44*RSe45611p - 1./108.*m44*RSp45611e + 1./
         27.*m44*RSe564p + 1./27.*m44*RSp564e - 1./90.*m34 + 1./36.*m34
         *RC3456pe + 1./36.*m34*RSe45633p + 1./36.*m34*RSp45633e - 1./
         18.*m34*RS356pe - 1./18.*m34*RSe563p - 1./18.*m34*RSp563e - 1./
         36.*m34*RC1456pe - 1./36.*m34*RSe45611p - 1./36.*m34*RSp45611e
          + 1./18.*m34*RS156pe + 1./18.*m34*RSe561p + 1./18.*m34*
         RSp561e );

      M13p6x23RV +=  + AbV(m14,L)*AbV(m34,L) * (  - 2./45.*m33 - 5./108.
         *m33*RS34561pe - 1./36.*m33*RS14563pe - 1./36.*m33*RSe45613p
          - 1./36.*m33*RSp45613e - 1./36.*m33*RC3456pe - 1./54.*m33*
         RSe45633p - 1./54.*m33*RSp45633e + 7./108.*m33*RS356pe + 1./36.
         *m33*RSe563p + 1./36.*m33*RSp563e - 1./108.*m33*RC1456pe - 1./
         108.*m33*RSe45611p - 1./108.*m33*RSp45611e + 1./108.*m33*
         RS156pe + 1./54.*m33*RS456pe + 1./108.*m33*RSe561p + 1./108.*
         m33*RSp561e + 1./54.*m33*RSe564p + 1./54.*m33*RSp564e - 1./90.
         *m14 - 1./36.*m14*RC3456pe - 1./36.*m14*RSe45633p - 1./36.*m14
         *RSp45633e + 1./18.*m14*RS356pe + 1./18.*m14*RSe563p + 1./18.*
         m14*RSp563e + 1./36.*m14*RC1456pe + 1./36.*m14*RSe45611p + 1./
         36.*m14*RSp45611e - 1./18.*m14*RS156pe - 1./18.*m14*RSe561p - 
         1./18.*m14*RSp561e - 3./20.*m13 - 1./36.*m13*RS34561pe - 1./36.
         *m13*RS14563pe - 1./36.*m13*RSe45613p - 1./36.*m13*RSp45613e
          + 1./36.*m13*RC3456pe + 1./36.*m13*RSe45633p + 1./36.*m13*
         RSp45633e );

      M13p6x23RV +=  + AbV(m14,L)*AbV(m34,L) * (  - 1./36.*m13*RS356pe
          - 1./36.*m13*RSe563p - 1./36.*m13*RSp563e + 1./36.*m13*
         RC1456pe + 1./36.*m13*RSe45611p + 1./36.*m13*RSp45611e - 1./36.
         *m13*RS156pe + 1./36.*m13*RS456pe - 1./36.*m13*RSe561p - 1./36.
         *m13*RSp561e + 1./36.*m13*RSe564p + 1./36.*m13*RSp564e - 2./45.
         *m11 - 1./36.*m11*RS34561pe - 5./108.*m11*RS14563pe - 1./36.*
         m11*RSe45613p - 1./36.*m11*RSp45613e - 1./108.*m11*RC3456pe - 
         1./108.*m11*RSe45633p - 1./108.*m11*RSp45633e + 1./108.*m11*
         RS356pe + 1./108.*m11*RSe563p + 1./108.*m11*RSp563e - 1./36.*
         m11*RC1456pe - 1./54.*m11*RSe45611p - 1./54.*m11*RSp45611e + 7.
         /108.*m11*RS156pe + 1./54.*m11*RS456pe + 1./36.*m11*RSe561p + 
         1./36.*m11*RSp561e + 1./54.*m11*RSe564p + 1./54.*m11*RSp564e )
         ;

      M13p6x23RV +=  + AbV(m14,L)*AbV(m35,L) * (  - 1./20.*m55 + 1./9.*
         m45 - 1./20.*m44 - 1./90.*m35 - 2./45.*m33 - 1./90.*m14 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + AbV(m14,L)*AbV(m36,L) * (  - 1./20.*m66 + 1./9.*
         m46 - 1./20.*m44 - 1./90.*m36 - 2./45.*m33 - 1./90.*m14 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + AbV(m14,L)*AbV(m44,L) * (  - 1./20.*m44 + 1./60.
         *m44*RS456pe + 1./36.*m34 - 1./108.*m34*RS456pe + 7./360.*m33
          - 7./1080.*m33*RS456pe + 13./180.*m14 - 13./540.*m14*RS456pe
          - 1./15.*m13 + 1./45.*m13*RS456pe - 1./360.*m11 + 1./1080.*
         m11*RS456pe );

      M13p6x23RV +=  + AbV(m14,L)*AbV(m45,L) * ( 7./360.*m55 - 1./15.*
         m45 - 1./360.*m44 + 1./36.*m35 + 7./360.*m33 + 13./180.*m14 - 
         1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + AbV(m14,L)*AbV(m46,L) * ( 7./360.*m66 - 1./15.*
         m46 - 1./360.*m44 + 1./36.*m36 + 7./360.*m33 + 13./180.*m14 - 
         1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + AbV(m14,L)*AbV(mp2,L) * ( 11./135.*mp2*RSp45613e
          - 1./180.*mp2*RSp45633e - 19./270.*mp2*RSp563e - 29./540.*mp2
         *RSp45611e + 7./270.*mp2*RSp561e + 1./45.*mp2*RSp564e - 7./540.
         *m44*RSp45613e + 1./60.*m44*RSp45633e + 1./180.*m44*RSp563e + 
         7./540.*m44*RSp45611e - 1./60.*m44*RSp561e - 1./180.*m44*
         RSp564e + 1./27.*m34*RSp45613e - 1./27.*m34*RSp45633e + 1./27.
         *m34*RSp563e - 1./108.*m34*RSp45611e - 1./54.*m34*RSp561e - 1./
         108.*m34*RSp564e + 4./135.*m33*RSp45613e + 11./270.*m33*
         RSp45633e - 1./45.*m33*RSp563e + 1./120.*m33*RSp45611e - 1./20.
         *m33*RSp561e - 7./1080.*m33*RSp564e - 1./90.*m14*RSp45613e + 1.
         /270.*m14*RSp45633e + 1./270.*m14*RSp563e - 1./60.*m14*
         RSp45611e + 2./45.*m14*RSp561e - 13./540.*m14*RSp564e - 11./
         540.*m13*RSp45613e + 1./20.*m13*RSp45633e - 43./540.*m13*
         RSp563e - 1./135.*m13*RSp45611e + 19./540.*m13*RSp561e + 1./45.
         *m13*RSp564e - 1./540.*m11*RSp45613e + 2./135.*m11*RSp45633e
          - 17./540.*m11*RSp563e );

      M13p6x23RV +=  + AbV(m14,L)*AbV(mp2,L) * ( 37./360.*m11*RSp45611e
          - 23./270.*m11*RSp561e + 1./1080.*m11*RSp564e );

      M13p6x23RV +=  + AbV(m14,L)*AbV(me2,L) * ( 11./135.*me2*RSe45613p
          - 1./180.*me2*RSe45633p - 19./270.*me2*RSe563p - 29./540.*me2
         *RSe45611p + 7./270.*me2*RSe561p + 1./45.*me2*RSe564p - 7./540.
         *m44*RSe45613p + 1./60.*m44*RSe45633p + 1./180.*m44*RSe563p + 
         7./540.*m44*RSe45611p - 1./60.*m44*RSe561p - 1./180.*m44*
         RSe564p + 1./27.*m34*RSe45613p - 1./27.*m34*RSe45633p + 1./27.
         *m34*RSe563p - 1./108.*m34*RSe45611p - 1./54.*m34*RSe561p - 1./
         108.*m34*RSe564p + 4./135.*m33*RSe45613p + 11./270.*m33*
         RSe45633p - 1./45.*m33*RSe563p + 1./120.*m33*RSe45611p - 1./20.
         *m33*RSe561p - 7./1080.*m33*RSe564p - 1./90.*m14*RSe45613p + 1.
         /270.*m14*RSe45633p + 1./270.*m14*RSe563p - 1./60.*m14*
         RSe45611p + 2./45.*m14*RSe561p - 13./540.*m14*RSe564p - 11./
         540.*m13*RSe45613p + 1./20.*m13*RSe45633p - 43./540.*m13*
         RSe563p - 1./135.*m13*RSe45611p + 19./540.*m13*RSe561p + 1./45.
         *m13*RSe564p - 1./540.*m11*RSe45613p + 2./135.*m11*RSe45633p
          - 17./540.*m11*RSe563p );

      M13p6x23RV +=  + AbV(m14,L)*AbV(me2,L) * ( 37./360.*m11*RSe45611p
          - 23./270.*m11*RSe561p + 1./1080.*m11*RSe564p );

      M13p6x23RV +=  + AbV(m14,L)*BbV(m11,m33,L) * (  - 1./108.*m33*m44
         *RS34561pe - 1./108.*m33*m44*RS356pe + 1./54.*m14*m33*
         RS34561pe - 1./54.*m14*m33*RS356pe - 1./54.*m13*m44*RS34561pe
          - 1./54.*m13*m44*RS356pe + 1./27.*m13*m14*RS34561pe - 1./27.*
         m13*m14*RS356pe - 1./36.*m11*m44*RS34561pe - 1./36.*m11*m44*
         RS356pe + 1./36.*m11*m33*RS34561pe - 7./108.*m11*m33*RS356pe
          + 1./18.*m11*m14*RS34561pe - 1./18.*m11*m14*RS356pe - 1./54.*
         m11*m13*RS34561pe - 1./18.*m11*m13*RS356pe - 7./108.*pow(
         m11,2)*RS34561pe - 5./108.*pow(m11,2)*RS356pe );

      M13p6x23RV +=  + AbV(m14,L)*BbV(m11,mp2,L) * (  - 1./108.*m33*m44
         *RSp45613e - 1./108.*m33*m44*RSp563e - 1./108.*m33*m44*
         RSp45611e - 1./108.*m33*m44*RSp561e + 1./54.*m14*m33*RSp45613e
          - 1./54.*m14*m33*RSp563e + 1./54.*m14*m33*RSp45611e - 1./54.*
         m14*m33*RSp561e - 1./54.*m13*m44*RSp45613e - 1./54.*m13*m44*
         RSp563e + 1./54.*m13*m44*RSp45611e + 1./54.*m13*m44*RSp561e + 
         1./27.*m13*m14*RSp45613e - 1./27.*m13*m14*RSp563e - 1./27.*m13
         *m14*RSp45611e + 1./27.*m13*m14*RSp561e + 1./27.*m11*mp2*
         RSp45613e - 1./27.*m11*mp2*RSp563e - 1./27.*m11*mp2*RSp45611e
          + 1./27.*m11*mp2*RSp561e - 1./36.*m11*m44*RSp45613e - 1./36.*
         m11*m44*RSp563e - 1./108.*m11*m44*RSp45611e - 1./108.*m11*m44*
         RSp561e - 1./108.*m11*m33*RSp45613e - 1./36.*m11*m33*RSp563e
          - 1./108.*m11*m33*RSp45611e - 1./36.*m11*m33*RSp561e + 1./18.
         *m11*m14*RSp45613e - 1./18.*m11*m14*RSp563e + 1./54.*m11*m14*
         RSp45611e - 1./54.*m11*m14*RSp561e - 1./54.*m11*m13*RSp45613e
          - 1./18.*m11*m13*RSp563e );

      M13p6x23RV +=  + AbV(m14,L)*BbV(m11,mp2,L) * ( 1./54.*m11*m13*
         RSp45611e + 1./18.*m11*m13*RSp561e - 7./108.*pow(m11,2)*
         RSp45613e - 5./108.*pow(m11,2)*RSp563e + 1./36.*pow(m11,2)*
         RSp45611e - 7./108.*pow(m11,2)*RSp561e );

      M13p6x23RV +=  + AbV(m14,L)*BbV(m11,me2,L) * (  - 1./108.*m33*m44
         *RSe45613p - 1./108.*m33*m44*RSe563p - 1./108.*m33*m44*
         RSe45611p - 1./108.*m33*m44*RSe561p + 1./54.*m14*m33*RSe45613p
          - 1./54.*m14*m33*RSe563p + 1./54.*m14*m33*RSe45611p - 1./54.*
         m14*m33*RSe561p - 1./54.*m13*m44*RSe45613p - 1./54.*m13*m44*
         RSe563p + 1./54.*m13*m44*RSe45611p + 1./54.*m13*m44*RSe561p + 
         1./27.*m13*m14*RSe45613p - 1./27.*m13*m14*RSe563p - 1./27.*m13
         *m14*RSe45611p + 1./27.*m13*m14*RSe561p + 1./27.*m11*me2*
         RSe45613p - 1./27.*m11*me2*RSe563p - 1./27.*m11*me2*RSe45611p
          + 1./27.*m11*me2*RSe561p - 1./36.*m11*m44*RSe45613p - 1./36.*
         m11*m44*RSe563p - 1./108.*m11*m44*RSe45611p - 1./108.*m11*m44*
         RSe561p - 1./108.*m11*m33*RSe45613p - 1./36.*m11*m33*RSe563p
          - 1./108.*m11*m33*RSe45611p - 1./36.*m11*m33*RSe561p + 1./18.
         *m11*m14*RSe45613p - 1./18.*m11*m14*RSe563p + 1./54.*m11*m14*
         RSe45611p - 1./54.*m11*m14*RSe561p - 1./54.*m11*m13*RSe45613p
          - 1./18.*m11*m13*RSe563p );

      M13p6x23RV +=  + AbV(m14,L)*BbV(m11,me2,L) * ( 1./54.*m11*m13*
         RSe45611p + 1./18.*m11*m13*RSe561p - 7./108.*pow(m11,2)*
         RSe45613p - 5./108.*pow(m11,2)*RSe563p + 1./36.*pow(m11,2)*
         RSe45611p - 7./108.*pow(m11,2)*RSe561p );

      M13p6x23RV +=  + AbV(m14,L)*BbV(m11,L) * ( 7./540.*m44*RD1456ep
          - 1./108.*m34*RD1456ep + 1./120.*m33*RD1456ep - 1./108.*m33*
         m44*RS14563pe - 1./108.*m33*m44*RC1456pe - 1./108.*m33*m44*
         RS156pe - 1./60.*m14*RD1456ep + 1./54.*m14*m33*RS14563pe + 1./
         54.*m14*m33*RC1456pe - 1./54.*m14*m33*RS156pe - 1./135.*m13*
         RD1456ep - 1./54.*m13*m44*RS14563pe + 1./54.*m13*m44*RC1456pe
          + 1./54.*m13*m44*RS156pe + 1./27.*m13*m14*RS14563pe - 1./27.*
         m13*m14*RC1456pe + 1./27.*m13*m14*RS156pe + 13./1080.*m11*
         RD1456ep - 1./36.*m11*m44*RS14563pe - 1./108.*m11*m44*RC1456pe
          - 1./108.*m11*m44*RS156pe - 1./108.*m11*m33*RS14563pe - 1./
         108.*m11*m33*RC1456pe - 1./36.*m11*m33*RS156pe + 1./18.*m11*
         m14*RS14563pe + 1./54.*m11*m14*RC1456pe - 1./54.*m11*m14*
         RS156pe - 1./54.*m11*m13*RS14563pe + 1./54.*m11*m13*RC1456pe
          + 1./18.*m11*m13*RS156pe - 1./36.*pow(m11,2)*RS14563pe - 1./
         108.*pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS156pe );

      M13p6x23RV +=  + AbV(m14,L)*BbV(m14,L) * ( 1./72.*pow(m44,2) + 1./
         72.*m33*m44 - 1./18.*m14*m44 - 1./36.*m14*m33 + 1./18.*pow(
         m14,2) - 1./36.*m13*m44 + 1./18.*m13*m14 + 1./24.*m11*m44 + 1./
         72.*m11*m33 - 1./12.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(
         m11,2) );

      M13p6x23RV +=  + AbV(m14,L)*BbV(m15,L) * ( 1./144.*pow(m55,2) + 1.
         /144.*m44*m55 + 1./144.*m33*m55 + 1./144.*m33*m44 - 1./36.*m15
         *m55 - 1./72.*m15*m44 - 1./72.*m15*m33 + 1./36.*pow(m15,2) - 1.
         /72.*m14*m55 - 1./72.*m14*m33 + 1./36.*m14*m15 - 1./72.*m13*
         m55 - 1./72.*m13*m44 + 1./36.*m13*m15 + 1./36.*m13*m14 + 1./36.
         *m11*m55 + 1./72.*m11*m44 + 1./72.*m11*m33 - 1./18.*m11*m15 - 
         1./36.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + AbV(m14,L)*BbV(m16,L) * ( 1./144.*pow(m66,2) + 1.
         /144.*m44*m66 + 1./144.*m33*m66 + 1./144.*m33*m44 - 1./36.*m16
         *m66 - 1./72.*m16*m44 - 1./72.*m16*m33 + 1./36.*pow(m16,2) - 1.
         /72.*m14*m66 - 1./72.*m14*m33 + 1./36.*m14*m16 - 1./72.*m13*
         m66 - 1./72.*m13*m44 + 1./36.*m13*m16 + 1./36.*m13*m14 + 1./36.
         *m11*m66 + 1./72.*m11*m44 + 1./72.*m11*m33 - 1./18.*m11*m16 - 
         1./36.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + AbV(m14,L)*BbV(m33,L) * ( 1./60.*m44*RD3456ep - 
         1./27.*m34*RD3456ep + 19./540.*m33*RD3456ep + 1./270.*m14*
         RD3456ep + 1./20.*m13*RD3456ep + 2./135.*m11*RD3456ep );

      M13p6x23RV +=  + AbV(m14,L)*CbV(m11,L) * (  - 1./108.*m33*m44*
         RD1456ep + 1./54.*m14*m33*RD1456ep + 1./54.*m13*m44*RD1456ep
          - 1./27.*m13*m14*RD1456ep - 1./108.*m11*m44*RD1456ep - 1./108.
         *m11*m33*RD1456ep + 1./54.*m11*m14*RD1456ep + 1./54.*m11*m13*
         RD1456ep - 1./108.*pow(m11,2)*RD1456ep );

      M13p6x23RV +=  + AbV(m15,L)*pi16 * (  - 1./24.*m13*RD3456ep - 1./
         24.*m13*me2*RSe45633p + 1./12.*m13*me2*RSe463p - 1./24.*m13*
         me2*RSe465p - 1./24.*m13*mp2*RSp45633e + 1./12.*m13*mp2*
         RSp463e - 1./24.*m13*mp2*RSp465e + 1./8.*m13*m56 + 1./8.*m13*
         m55 - 1./24.*m13*m55*RS546pe + 1./8.*m13*m45 + 1./8.*m13*m36
          + 1./8.*m13*m35 - 1./24.*m13*m35*RC3456pe - 1./24.*m13*m35*
         RSe45633p - 1./24.*m13*m35*RSp45633e + 1./12.*m13*m35*RS346pe
          + 1./12.*m13*m35*RSe463p + 1./12.*m13*m35*RSp463e - 1./24.*
         m13*m35*RS546pe - 1./24.*m13*m35*RSe465p - 1./24.*m13*m35*
         RSp465e + 1./8.*m13*m34 - 1./24.*m13*m33*RC3456pe + 1./12.*m13
         *m33*RS346pe - 1./8.*m13*m15 + 1./24.*m13*m15*RS34561pe + 1./
         24.*m13*m15*RS14563pe + 1./24.*m13*m15*RSe45613p + 1./24.*m13*
         m15*RSp45613e - 1./24.*m13*m15*RS346pe - 1./24.*m13*m15*
         RSe463p - 1./24.*m13*m15*RSp463e - 1./24.*m13*m15*RC1456pe - 1.
         /24.*m13*m15*RSe45611p - 1./24.*m13*m15*RSp45611e + 1./24.*m13
         *m15*RS146pe );

      M13p6x23RV +=  + AbV(m15,L)*pi16 * ( 1./24.*m13*m15*RSe461p + 1./
         24.*m13*m15*RSp461e - 1./8.*pow(m13,2) + 1./72.*pow(m13,2)*
         RC3456pe + 1./72.*pow(m13,2)*RSe45633p + 1./72.*pow(m13,2)*
         RSp45633e - 1./36.*pow(m13,2)*RS346pe - 1./36.*pow(m13,2)*
         RSe463p - 1./36.*pow(m13,2)*RSp463e + 1./72.*pow(m13,2)*
         RS546pe + 1./72.*pow(m13,2)*RSe465p + 1./72.*pow(m13,2)*
         RSp465e );

      M13p6x23RV +=  + pow(AbV(m15,L),2) * (  - 11./360.*m55 - 1./120.*
         m33 + 11./180.*m15 + 1./60.*m13 - 7./180.*m11 );

      M13p6x23RV +=  + AbV(m15,L)*AbV(m16,L) * (  - 11./360.*m66 - 11./
         360.*m55 - 1./60.*m33 + 11./180.*m16 + 11./180.*m15 + 1./30.*
         m13 - 7./90.*m11 );

      M13p6x23RV +=  + AbV(m15,L)*AbV(m33,L) * (  - 1./180.*RD3456ep - 
         7./540.*m55*RS34561pe + 1./60.*m55*RC3456pe + 1./180.*m55*
         RS346pe + 1./27.*m35*RS34561pe - 1./27.*m35*RC3456pe + 1./27.*
         m35*RS346pe + 2./27.*m33*RS34561pe + 19./540.*m33*RC3456pe - 5.
         /54.*m33*RS346pe - 1./90.*m15*RS34561pe + 1./270.*m15*RC3456pe
          + 1./270.*m15*RS346pe - 11./540.*m13*RS34561pe + 1./20.*m13*
         RC3456pe - 43./540.*m13*RS346pe - 11./540.*m11*RS34561pe + 2./
         135.*m11*RC3456pe - 17./540.*m11*RS346pe + 1./27.*RS(m33,m44,
         m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11 );

      M13p6x23RV +=  + AbV(m15,L)*AbV(m34,L) * (  - 1./20.*m55 + 1./9.*
         m45 - 1./20.*m44 - 1./90.*m34 - 2./45.*m33 - 1./90.*m15 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + AbV(m15,L)*AbV(m35,L) * (  - 1./108.*RD3456ep - 
         1./108.*RD1456ep - 1./54.*me2*RSe45613p - 1./108.*me2*
         RSe45633p + 1./27.*me2*RSe463p - 1./108.*me2*RSe45611p + 1./27.
         *me2*RSe461p - 1./27.*me2*RSe465p - 1./54.*mp2*RSp45613e - 1./
         108.*mp2*RSp45633e + 1./27.*mp2*RSp463e - 1./108.*mp2*
         RSp45611e + 1./27.*mp2*RSp461e - 1./27.*mp2*RSp465e + 1./90.*
         m55 - 1./54.*m55*RS34561pe - 1./54.*m55*RS14563pe - 1./54.*m55
         *RSe45613p - 1./54.*m55*RSp45613e - 1./108.*m55*RC3456pe - 1./
         108.*m55*RSe45633p - 1./108.*m55*RSp45633e - 1./108.*m55*
         RC1456pe - 1./108.*m55*RSe45611p - 1./108.*m55*RSp45611e + 1./
         27.*m55*RSe465p + 1./27.*m55*RSp465e - 1./90.*m35 + 1./36.*m35
         *RC3456pe + 1./36.*m35*RSe45633p + 1./36.*m35*RSp45633e - 1./
         18.*m35*RS346pe - 1./18.*m35*RSe463p - 1./18.*m35*RSp463e - 1./
         36.*m35*RC1456pe - 1./36.*m35*RSe45611p - 1./36.*m35*RSp45611e
          + 1./18.*m35*RS146pe + 1./18.*m35*RSe461p + 1./18.*m35*
         RSp461e );

      M13p6x23RV +=  + AbV(m15,L)*AbV(m35,L) * (  - 2./45.*m33 - 5./108.
         *m33*RS34561pe - 1./36.*m33*RS14563pe - 1./36.*m33*RSe45613p
          - 1./36.*m33*RSp45613e - 1./36.*m33*RC3456pe - 1./54.*m33*
         RSe45633p - 1./54.*m33*RSp45633e + 7./108.*m33*RS346pe + 1./36.
         *m33*RSe463p + 1./36.*m33*RSp463e - 1./108.*m33*RC1456pe - 1./
         108.*m33*RSe45611p - 1./108.*m33*RSp45611e + 1./108.*m33*
         RS146pe + 1./54.*m33*RS546pe + 1./108.*m33*RSe461p + 1./108.*
         m33*RSp461e + 1./54.*m33*RSe465p + 1./54.*m33*RSp465e - 1./90.
         *m15 - 1./36.*m15*RC3456pe - 1./36.*m15*RSe45633p - 1./36.*m15
         *RSp45633e + 1./18.*m15*RS346pe + 1./18.*m15*RSe463p + 1./18.*
         m15*RSp463e + 1./36.*m15*RC1456pe + 1./36.*m15*RSe45611p + 1./
         36.*m15*RSp45611e - 1./18.*m15*RS146pe - 1./18.*m15*RSe461p - 
         1./18.*m15*RSp461e - 3./20.*m13 - 1./36.*m13*RS34561pe - 1./36.
         *m13*RS14563pe - 1./36.*m13*RSe45613p - 1./36.*m13*RSp45613e
          + 1./36.*m13*RC3456pe + 1./36.*m13*RSe45633p + 1./36.*m13*
         RSp45633e );

      M13p6x23RV +=  + AbV(m15,L)*AbV(m35,L) * (  - 1./36.*m13*RS346pe
          - 1./36.*m13*RSe463p - 1./36.*m13*RSp463e + 1./36.*m13*
         RC1456pe + 1./36.*m13*RSe45611p + 1./36.*m13*RSp45611e - 1./36.
         *m13*RS146pe + 1./36.*m13*RS546pe - 1./36.*m13*RSe461p - 1./36.
         *m13*RSp461e + 1./36.*m13*RSe465p + 1./36.*m13*RSp465e - 2./45.
         *m11 - 1./36.*m11*RS34561pe - 5./108.*m11*RS14563pe - 1./36.*
         m11*RSe45613p - 1./36.*m11*RSp45613e - 1./108.*m11*RC3456pe - 
         1./108.*m11*RSe45633p - 1./108.*m11*RSp45633e + 1./108.*m11*
         RS346pe + 1./108.*m11*RSe463p + 1./108.*m11*RSp463e - 1./36.*
         m11*RC1456pe - 1./54.*m11*RSe45611p - 1./54.*m11*RSp45611e + 7.
         /108.*m11*RS146pe + 1./54.*m11*RS546pe + 1./36.*m11*RSe461p + 
         1./36.*m11*RSp461e + 1./54.*m11*RSe465p + 1./54.*m11*RSp465e )
         ;

      M13p6x23RV +=  + AbV(m15,L)*AbV(m36,L) * (  - 1./20.*m66 + 1./9.*
         m56 - 1./20.*m55 - 1./90.*m36 - 2./45.*m33 - 1./90.*m15 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + AbV(m15,L)*AbV(m45,L) * (  - 1./360.*m55 - 1./15.
         *m45 + 7./360.*m44 + 1./36.*m34 + 7./360.*m33 + 13./180.*m15
          - 1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + AbV(m15,L)*AbV(m55,L) * (  - 1./20.*m55 + 1./60.
         *m55*RS546pe + 1./36.*m35 - 1./108.*m35*RS546pe + 7./360.*m33
          - 7./1080.*m33*RS546pe + 13./180.*m15 - 13./540.*m15*RS546pe
          - 1./15.*m13 + 1./45.*m13*RS546pe - 1./360.*m11 + 1./1080.*
         m11*RS546pe );

      M13p6x23RV +=  + AbV(m15,L)*AbV(m56,L) * ( 7./360.*m66 - 1./15.*
         m56 - 1./360.*m55 + 1./36.*m36 + 7./360.*m33 + 13./180.*m15 - 
         1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + AbV(m15,L)*AbV(mp2,L) * ( 11./135.*mp2*RSp45613e
          - 1./180.*mp2*RSp45633e - 19./270.*mp2*RSp463e - 29./540.*mp2
         *RSp45611e + 7./270.*mp2*RSp461e + 1./45.*mp2*RSp465e - 7./540.
         *m55*RSp45613e + 1./60.*m55*RSp45633e + 1./180.*m55*RSp463e + 
         7./540.*m55*RSp45611e - 1./60.*m55*RSp461e - 1./180.*m55*
         RSp465e + 1./27.*m35*RSp45613e - 1./27.*m35*RSp45633e + 1./27.
         *m35*RSp463e - 1./108.*m35*RSp45611e - 1./54.*m35*RSp461e - 1./
         108.*m35*RSp465e + 4./135.*m33*RSp45613e + 11./270.*m33*
         RSp45633e - 1./45.*m33*RSp463e + 1./120.*m33*RSp45611e - 1./20.
         *m33*RSp461e - 7./1080.*m33*RSp465e - 1./90.*m15*RSp45613e + 1.
         /270.*m15*RSp45633e + 1./270.*m15*RSp463e - 1./60.*m15*
         RSp45611e + 2./45.*m15*RSp461e - 13./540.*m15*RSp465e - 11./
         540.*m13*RSp45613e + 1./20.*m13*RSp45633e - 43./540.*m13*
         RSp463e - 1./135.*m13*RSp45611e + 19./540.*m13*RSp461e + 1./45.
         *m13*RSp465e - 1./540.*m11*RSp45613e + 2./135.*m11*RSp45633e
          - 17./540.*m11*RSp463e );

      M13p6x23RV +=  + AbV(m15,L)*AbV(mp2,L) * ( 37./360.*m11*RSp45611e
          - 23./270.*m11*RSp461e + 1./1080.*m11*RSp465e );

      M13p6x23RV +=  + AbV(m15,L)*AbV(me2,L) * ( 11./135.*me2*RSe45613p
          - 1./180.*me2*RSe45633p - 19./270.*me2*RSe463p - 29./540.*me2
         *RSe45611p + 7./270.*me2*RSe461p + 1./45.*me2*RSe465p - 7./540.
         *m55*RSe45613p + 1./60.*m55*RSe45633p + 1./180.*m55*RSe463p + 
         7./540.*m55*RSe45611p - 1./60.*m55*RSe461p - 1./180.*m55*
         RSe465p + 1./27.*m35*RSe45613p - 1./27.*m35*RSe45633p + 1./27.
         *m35*RSe463p - 1./108.*m35*RSe45611p - 1./54.*m35*RSe461p - 1./
         108.*m35*RSe465p + 4./135.*m33*RSe45613p + 11./270.*m33*
         RSe45633p - 1./45.*m33*RSe463p + 1./120.*m33*RSe45611p - 1./20.
         *m33*RSe461p - 7./1080.*m33*RSe465p - 1./90.*m15*RSe45613p + 1.
         /270.*m15*RSe45633p + 1./270.*m15*RSe463p - 1./60.*m15*
         RSe45611p + 2./45.*m15*RSe461p - 13./540.*m15*RSe465p - 11./
         540.*m13*RSe45613p + 1./20.*m13*RSe45633p - 43./540.*m13*
         RSe463p - 1./135.*m13*RSe45611p + 19./540.*m13*RSe461p + 1./45.
         *m13*RSe465p - 1./540.*m11*RSe45613p + 2./135.*m11*RSe45633p
          - 17./540.*m11*RSe463p );

      M13p6x23RV +=  + AbV(m15,L)*AbV(me2,L) * ( 37./360.*m11*RSe45611p
          - 23./270.*m11*RSe461p + 1./1080.*m11*RSe465p );

      M13p6x23RV +=  + AbV(m15,L)*BbV(m11,m33,L) * (  - 1./108.*m33*m55
         *RS34561pe - 1./108.*m33*m55*RS346pe + 1./54.*m15*m33*
         RS34561pe - 1./54.*m15*m33*RS346pe - 1./54.*m13*m55*RS34561pe
          - 1./54.*m13*m55*RS346pe + 1./27.*m13*m15*RS34561pe - 1./27.*
         m13*m15*RS346pe - 1./36.*m11*m55*RS34561pe - 1./36.*m11*m55*
         RS346pe + 1./36.*m11*m33*RS34561pe - 7./108.*m11*m33*RS346pe
          + 1./18.*m11*m15*RS34561pe - 1./18.*m11*m15*RS346pe - 1./54.*
         m11*m13*RS34561pe - 1./18.*m11*m13*RS346pe - 7./108.*pow(
         m11,2)*RS34561pe - 5./108.*pow(m11,2)*RS346pe );

      M13p6x23RV +=  + AbV(m15,L)*BbV(m11,mp2,L) * (  - 1./108.*m33*m55
         *RSp45613e - 1./108.*m33*m55*RSp463e - 1./108.*m33*m55*
         RSp45611e - 1./108.*m33*m55*RSp461e + 1./54.*m15*m33*RSp45613e
          - 1./54.*m15*m33*RSp463e + 1./54.*m15*m33*RSp45611e - 1./54.*
         m15*m33*RSp461e - 1./54.*m13*m55*RSp45613e - 1./54.*m13*m55*
         RSp463e + 1./54.*m13*m55*RSp45611e + 1./54.*m13*m55*RSp461e + 
         1./27.*m13*m15*RSp45613e - 1./27.*m13*m15*RSp463e - 1./27.*m13
         *m15*RSp45611e + 1./27.*m13*m15*RSp461e + 1./27.*m11*mp2*
         RSp45613e - 1./27.*m11*mp2*RSp463e - 1./27.*m11*mp2*RSp45611e
          + 1./27.*m11*mp2*RSp461e - 1./36.*m11*m55*RSp45613e - 1./36.*
         m11*m55*RSp463e - 1./108.*m11*m55*RSp45611e - 1./108.*m11*m55*
         RSp461e - 1./108.*m11*m33*RSp45613e - 1./36.*m11*m33*RSp463e
          - 1./108.*m11*m33*RSp45611e - 1./36.*m11*m33*RSp461e + 1./18.
         *m11*m15*RSp45613e - 1./18.*m11*m15*RSp463e + 1./54.*m11*m15*
         RSp45611e - 1./54.*m11*m15*RSp461e - 1./54.*m11*m13*RSp45613e
          - 1./18.*m11*m13*RSp463e );

      M13p6x23RV +=  + AbV(m15,L)*BbV(m11,mp2,L) * ( 1./54.*m11*m13*
         RSp45611e + 1./18.*m11*m13*RSp461e - 7./108.*pow(m11,2)*
         RSp45613e - 5./108.*pow(m11,2)*RSp463e + 1./36.*pow(m11,2)*
         RSp45611e - 7./108.*pow(m11,2)*RSp461e );

      M13p6x23RV +=  + AbV(m15,L)*BbV(m11,me2,L) * (  - 1./108.*m33*m55
         *RSe45613p - 1./108.*m33*m55*RSe463p - 1./108.*m33*m55*
         RSe45611p - 1./108.*m33*m55*RSe461p + 1./54.*m15*m33*RSe45613p
          - 1./54.*m15*m33*RSe463p + 1./54.*m15*m33*RSe45611p - 1./54.*
         m15*m33*RSe461p - 1./54.*m13*m55*RSe45613p - 1./54.*m13*m55*
         RSe463p + 1./54.*m13*m55*RSe45611p + 1./54.*m13*m55*RSe461p + 
         1./27.*m13*m15*RSe45613p - 1./27.*m13*m15*RSe463p - 1./27.*m13
         *m15*RSe45611p + 1./27.*m13*m15*RSe461p + 1./27.*m11*me2*
         RSe45613p - 1./27.*m11*me2*RSe463p - 1./27.*m11*me2*RSe45611p
          + 1./27.*m11*me2*RSe461p - 1./36.*m11*m55*RSe45613p - 1./36.*
         m11*m55*RSe463p - 1./108.*m11*m55*RSe45611p - 1./108.*m11*m55*
         RSe461p - 1./108.*m11*m33*RSe45613p - 1./36.*m11*m33*RSe463p
          - 1./108.*m11*m33*RSe45611p - 1./36.*m11*m33*RSe461p + 1./18.
         *m11*m15*RSe45613p - 1./18.*m11*m15*RSe463p + 1./54.*m11*m15*
         RSe45611p - 1./54.*m11*m15*RSe461p - 1./54.*m11*m13*RSe45613p
          - 1./18.*m11*m13*RSe463p );

      M13p6x23RV +=  + AbV(m15,L)*BbV(m11,me2,L) * ( 1./54.*m11*m13*
         RSe45611p + 1./18.*m11*m13*RSe461p - 7./108.*pow(m11,2)*
         RSe45613p - 5./108.*pow(m11,2)*RSe463p + 1./36.*pow(m11,2)*
         RSe45611p - 7./108.*pow(m11,2)*RSe461p );

      M13p6x23RV +=  + AbV(m15,L)*BbV(m11,L) * ( 7./540.*m55*RD1456ep
          - 1./108.*m35*RD1456ep + 1./120.*m33*RD1456ep - 1./108.*m33*
         m55*RS14563pe - 1./108.*m33*m55*RC1456pe - 1./108.*m33*m55*
         RS146pe - 1./60.*m15*RD1456ep + 1./54.*m15*m33*RS14563pe + 1./
         54.*m15*m33*RC1456pe - 1./54.*m15*m33*RS146pe - 1./135.*m13*
         RD1456ep - 1./54.*m13*m55*RS14563pe + 1./54.*m13*m55*RC1456pe
          + 1./54.*m13*m55*RS146pe + 1./27.*m13*m15*RS14563pe - 1./27.*
         m13*m15*RC1456pe + 1./27.*m13*m15*RS146pe + 13./1080.*m11*
         RD1456ep - 1./36.*m11*m55*RS14563pe - 1./108.*m11*m55*RC1456pe
          - 1./108.*m11*m55*RS146pe - 1./108.*m11*m33*RS14563pe - 1./
         108.*m11*m33*RC1456pe - 1./36.*m11*m33*RS146pe + 1./18.*m11*
         m15*RS14563pe + 1./54.*m11*m15*RC1456pe - 1./54.*m11*m15*
         RS146pe - 1./54.*m11*m13*RS14563pe + 1./54.*m11*m13*RC1456pe
          + 1./18.*m11*m13*RS146pe - 1./36.*pow(m11,2)*RS14563pe - 1./
         108.*pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS146pe );

      M13p6x23RV +=  + AbV(m15,L)*BbV(m14,L) * ( 1./144.*m44*m55 + 1./
         144.*pow(m44,2) + 1./144.*m33*m55 + 1./144.*m33*m44 - 1./72.*
         m15*m44 - 1./72.*m15*m33 - 1./72.*m14*m55 - 1./36.*m14*m44 - 1.
         /72.*m14*m33 + 1./36.*m14*m15 + 1./36.*pow(m14,2) - 1./72.*m13
         *m55 - 1./72.*m13*m44 + 1./36.*m13*m15 + 1./36.*m13*m14 + 1./
         72.*m11*m55 + 1./36.*m11*m44 + 1./72.*m11*m33 - 1./36.*m11*m15
          - 1./18.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + AbV(m15,L)*BbV(m15,L) * ( 1./72.*pow(m55,2) + 1./
         72.*m33*m55 - 1./18.*m15*m55 - 1./36.*m15*m33 + 1./18.*pow(
         m15,2) - 1./36.*m13*m55 + 1./18.*m13*m15 + 1./24.*m11*m55 + 1./
         72.*m11*m33 - 1./12.*m11*m15 - 1./36.*m11*m13 + 1./36.*pow(
         m11,2) );

      M13p6x23RV +=  + AbV(m15,L)*BbV(m16,L) * ( 1./144.*pow(m66,2) + 1.
         /144.*m55*m66 + 1./144.*m33*m66 + 1./144.*m33*m55 - 1./36.*m16
         *m66 - 1./72.*m16*m55 - 1./72.*m16*m33 + 1./36.*pow(m16,2) - 1.
         /72.*m15*m66 - 1./72.*m15*m33 + 1./36.*m15*m16 - 1./72.*m13*
         m66 - 1./72.*m13*m55 + 1./36.*m13*m16 + 1./36.*m13*m15 + 1./36.
         *m11*m66 + 1./72.*m11*m55 + 1./72.*m11*m33 - 1./18.*m11*m16 - 
         1./36.*m11*m15 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + AbV(m15,L)*BbV(m33,L) * ( 1./60.*m55*RD3456ep - 
         1./27.*m35*RD3456ep + 19./540.*m33*RD3456ep + 1./270.*m15*
         RD3456ep + 1./20.*m13*RD3456ep + 2./135.*m11*RD3456ep );

      M13p6x23RV +=  + AbV(m15,L)*CbV(m11,L) * (  - 1./108.*m33*m55*
         RD1456ep + 1./54.*m15*m33*RD1456ep + 1./54.*m13*m55*RD1456ep
          - 1./27.*m13*m15*RD1456ep - 1./108.*m11*m55*RD1456ep - 1./108.
         *m11*m33*RD1456ep + 1./54.*m11*m15*RD1456ep + 1./54.*m11*m13*
         RD1456ep - 1./108.*pow(m11,2)*RD1456ep );

      M13p6x23RV +=  + AbV(m16,L)*pi16 * (  - 1./24.*m13*RD3456ep - 1./
         24.*m13*me2*RSe45633p + 1./12.*m13*me2*RSe453p - 1./24.*m13*
         me2*RSe456p - 1./24.*m13*mp2*RSp45633e + 1./12.*m13*mp2*
         RSp453e - 1./24.*m13*mp2*RSp456e + 1./8.*m13*m66 - 1./24.*m13*
         m66*RS645pe + 1./8.*m13*m56 + 1./8.*m13*m46 + 1./8.*m13*m36 - 
         1./24.*m13*m36*RC3456pe - 1./24.*m13*m36*RSe45633p - 1./24.*
         m13*m36*RSp45633e + 1./12.*m13*m36*RS345pe + 1./12.*m13*m36*
         RSe453p + 1./12.*m13*m36*RSp453e - 1./24.*m13*m36*RS645pe - 1./
         24.*m13*m36*RSe456p - 1./24.*m13*m36*RSp456e + 1./8.*m13*m35
          + 1./8.*m13*m34 - 1./24.*m13*m33*RC3456pe + 1./12.*m13*m33*
         RS345pe - 1./8.*m13*m16 + 1./24.*m13*m16*RS34561pe + 1./24.*
         m13*m16*RS14563pe + 1./24.*m13*m16*RSe45613p + 1./24.*m13*m16*
         RSp45613e - 1./24.*m13*m16*RS345pe - 1./24.*m13*m16*RSe453p - 
         1./24.*m13*m16*RSp453e - 1./24.*m13*m16*RC1456pe - 1./24.*m13*
         m16*RSe45611p - 1./24.*m13*m16*RSp45611e + 1./24.*m13*m16*
         RS145pe );

      M13p6x23RV +=  + AbV(m16,L)*pi16 * ( 1./24.*m13*m16*RSe451p + 1./
         24.*m13*m16*RSp451e - 1./8.*pow(m13,2) + 1./72.*pow(m13,2)*
         RC3456pe + 1./72.*pow(m13,2)*RSe45633p + 1./72.*pow(m13,2)*
         RSp45633e - 1./36.*pow(m13,2)*RS345pe - 1./36.*pow(m13,2)*
         RSe453p - 1./36.*pow(m13,2)*RSp453e + 1./72.*pow(m13,2)*
         RS645pe + 1./72.*pow(m13,2)*RSe456p + 1./72.*pow(m13,2)*
         RSp456e );

      M13p6x23RV +=  + pow(AbV(m16,L),2) * (  - 11./360.*m66 - 1./120.*
         m33 + 11./180.*m16 + 1./60.*m13 - 7./180.*m11 );

      M13p6x23RV +=  + AbV(m16,L)*AbV(m33,L) * (  - 1./180.*RD3456ep - 
         7./540.*m66*RS34561pe + 1./60.*m66*RC3456pe + 1./180.*m66*
         RS345pe + 1./27.*m36*RS34561pe - 1./27.*m36*RC3456pe + 1./27.*
         m36*RS345pe + 2./27.*m33*RS34561pe + 19./540.*m33*RC3456pe - 5.
         /54.*m33*RS345pe - 1./90.*m16*RS34561pe + 1./270.*m16*RC3456pe
          + 1./270.*m16*RS345pe - 11./540.*m13*RS34561pe + 1./20.*m13*
         RC3456pe - 43./540.*m13*RS345pe - 11./540.*m11*RS34561pe + 2./
         135.*m11*RC3456pe - 17./540.*m11*RS345pe + 1./27.*RS(m33,m44,
         m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11 );

      M13p6x23RV +=  + AbV(m16,L)*AbV(m34,L) * (  - 1./20.*m66 + 1./9.*
         m46 - 1./20.*m44 - 1./90.*m34 - 2./45.*m33 - 1./90.*m16 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + AbV(m16,L)*AbV(m35,L) * (  - 1./20.*m66 + 1./9.*
         m56 - 1./20.*m55 - 1./90.*m35 - 2./45.*m33 - 1./90.*m16 - 3./
         20.*m13 - 2./45.*m11 );

      M13p6x23RV +=  + AbV(m16,L)*AbV(m36,L) * (  - 1./108.*RD3456ep - 
         1./108.*RD1456ep - 1./54.*me2*RSe45613p - 1./108.*me2*
         RSe45633p + 1./27.*me2*RSe453p - 1./108.*me2*RSe45611p + 1./27.
         *me2*RSe451p - 1./27.*me2*RSe456p - 1./54.*mp2*RSp45613e - 1./
         108.*mp2*RSp45633e + 1./27.*mp2*RSp453e - 1./108.*mp2*
         RSp45611e + 1./27.*mp2*RSp451e - 1./27.*mp2*RSp456e + 1./90.*
         m66 - 1./54.*m66*RS34561pe - 1./54.*m66*RS14563pe - 1./54.*m66
         *RSe45613p - 1./54.*m66*RSp45613e - 1./108.*m66*RC3456pe - 1./
         108.*m66*RSe45633p - 1./108.*m66*RSp45633e - 1./108.*m66*
         RC1456pe - 1./108.*m66*RSe45611p - 1./108.*m66*RSp45611e + 1./
         27.*m66*RSe456p + 1./27.*m66*RSp456e - 1./90.*m36 + 1./36.*m36
         *RC3456pe + 1./36.*m36*RSe45633p + 1./36.*m36*RSp45633e - 1./
         18.*m36*RS345pe - 1./18.*m36*RSe453p - 1./18.*m36*RSp453e - 1./
         36.*m36*RC1456pe - 1./36.*m36*RSe45611p - 1./36.*m36*RSp45611e
          + 1./18.*m36*RS145pe + 1./18.*m36*RSe451p + 1./18.*m36*
         RSp451e );

      M13p6x23RV +=  + AbV(m16,L)*AbV(m36,L) * (  - 2./45.*m33 - 5./108.
         *m33*RS34561pe - 1./36.*m33*RS14563pe - 1./36.*m33*RSe45613p
          - 1./36.*m33*RSp45613e - 1./36.*m33*RC3456pe - 1./54.*m33*
         RSe45633p - 1./54.*m33*RSp45633e + 7./108.*m33*RS345pe + 1./36.
         *m33*RSe453p + 1./36.*m33*RSp453e - 1./108.*m33*RC1456pe - 1./
         108.*m33*RSe45611p - 1./108.*m33*RSp45611e + 1./108.*m33*
         RS145pe + 1./54.*m33*RS645pe + 1./108.*m33*RSe451p + 1./108.*
         m33*RSp451e + 1./54.*m33*RSe456p + 1./54.*m33*RSp456e - 1./90.
         *m16 - 1./36.*m16*RC3456pe - 1./36.*m16*RSe45633p - 1./36.*m16
         *RSp45633e + 1./18.*m16*RS345pe + 1./18.*m16*RSe453p + 1./18.*
         m16*RSp453e + 1./36.*m16*RC1456pe + 1./36.*m16*RSe45611p + 1./
         36.*m16*RSp45611e - 1./18.*m16*RS145pe - 1./18.*m16*RSe451p - 
         1./18.*m16*RSp451e - 3./20.*m13 - 1./36.*m13*RS34561pe - 1./36.
         *m13*RS14563pe - 1./36.*m13*RSe45613p - 1./36.*m13*RSp45613e
          + 1./36.*m13*RC3456pe + 1./36.*m13*RSe45633p + 1./36.*m13*
         RSp45633e );

      M13p6x23RV +=  + AbV(m16,L)*AbV(m36,L) * (  - 1./36.*m13*RS345pe
          - 1./36.*m13*RSe453p - 1./36.*m13*RSp453e + 1./36.*m13*
         RC1456pe + 1./36.*m13*RSe45611p + 1./36.*m13*RSp45611e - 1./36.
         *m13*RS145pe + 1./36.*m13*RS645pe - 1./36.*m13*RSe451p - 1./36.
         *m13*RSp451e + 1./36.*m13*RSe456p + 1./36.*m13*RSp456e - 2./45.
         *m11 - 1./36.*m11*RS34561pe - 5./108.*m11*RS14563pe - 1./36.*
         m11*RSe45613p - 1./36.*m11*RSp45613e - 1./108.*m11*RC3456pe - 
         1./108.*m11*RSe45633p - 1./108.*m11*RSp45633e + 1./108.*m11*
         RS345pe + 1./108.*m11*RSe453p + 1./108.*m11*RSp453e - 1./36.*
         m11*RC1456pe - 1./54.*m11*RSe45611p - 1./54.*m11*RSp45611e + 7.
         /108.*m11*RS145pe + 1./54.*m11*RS645pe + 1./36.*m11*RSe451p + 
         1./36.*m11*RSp451e + 1./54.*m11*RSe456p + 1./54.*m11*RSp456e )
         ;

      M13p6x23RV +=  + AbV(m16,L)*AbV(m46,L) * (  - 1./360.*m66 - 1./15.
         *m46 + 7./360.*m44 + 1./36.*m34 + 7./360.*m33 + 13./180.*m16
          - 1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + AbV(m16,L)*AbV(m56,L) * (  - 1./360.*m66 - 1./15.
         *m56 + 7./360.*m55 + 1./36.*m35 + 7./360.*m33 + 13./180.*m16
          - 1./15.*m13 - 1./360.*m11 );

      M13p6x23RV +=  + AbV(m16,L)*AbV(m66,L) * (  - 1./20.*m66 + 1./60.
         *m66*RS645pe + 1./36.*m36 - 1./108.*m36*RS645pe + 7./360.*m33
          - 7./1080.*m33*RS645pe + 13./180.*m16 - 13./540.*m16*RS645pe
          - 1./15.*m13 + 1./45.*m13*RS645pe - 1./360.*m11 + 1./1080.*
         m11*RS645pe );

      M13p6x23RV +=  + AbV(m16,L)*AbV(mp2,L) * ( 11./135.*mp2*RSp45613e
          - 1./180.*mp2*RSp45633e - 19./270.*mp2*RSp453e - 29./540.*mp2
         *RSp45611e + 7./270.*mp2*RSp451e + 1./45.*mp2*RSp456e - 7./540.
         *m66*RSp45613e + 1./60.*m66*RSp45633e + 1./180.*m66*RSp453e + 
         7./540.*m66*RSp45611e - 1./60.*m66*RSp451e - 1./180.*m66*
         RSp456e + 1./27.*m36*RSp45613e - 1./27.*m36*RSp45633e + 1./27.
         *m36*RSp453e - 1./108.*m36*RSp45611e - 1./54.*m36*RSp451e - 1./
         108.*m36*RSp456e + 4./135.*m33*RSp45613e + 11./270.*m33*
         RSp45633e - 1./45.*m33*RSp453e + 1./120.*m33*RSp45611e - 1./20.
         *m33*RSp451e - 7./1080.*m33*RSp456e - 1./90.*m16*RSp45613e + 1.
         /270.*m16*RSp45633e + 1./270.*m16*RSp453e - 1./60.*m16*
         RSp45611e + 2./45.*m16*RSp451e - 13./540.*m16*RSp456e - 11./
         540.*m13*RSp45613e + 1./20.*m13*RSp45633e - 43./540.*m13*
         RSp453e - 1./135.*m13*RSp45611e + 19./540.*m13*RSp451e + 1./45.
         *m13*RSp456e - 1./540.*m11*RSp45613e + 2./135.*m11*RSp45633e
          - 17./540.*m11*RSp453e );

      M13p6x23RV +=  + AbV(m16,L)*AbV(mp2,L) * ( 37./360.*m11*RSp45611e
          - 23./270.*m11*RSp451e + 1./1080.*m11*RSp456e );

      M13p6x23RV +=  + AbV(m16,L)*AbV(me2,L) * ( 11./135.*me2*RSe45613p
          - 1./180.*me2*RSe45633p - 19./270.*me2*RSe453p - 29./540.*me2
         *RSe45611p + 7./270.*me2*RSe451p + 1./45.*me2*RSe456p - 7./540.
         *m66*RSe45613p + 1./60.*m66*RSe45633p + 1./180.*m66*RSe453p + 
         7./540.*m66*RSe45611p - 1./60.*m66*RSe451p - 1./180.*m66*
         RSe456p + 1./27.*m36*RSe45613p - 1./27.*m36*RSe45633p + 1./27.
         *m36*RSe453p - 1./108.*m36*RSe45611p - 1./54.*m36*RSe451p - 1./
         108.*m36*RSe456p + 4./135.*m33*RSe45613p + 11./270.*m33*
         RSe45633p - 1./45.*m33*RSe453p + 1./120.*m33*RSe45611p - 1./20.
         *m33*RSe451p - 7./1080.*m33*RSe456p - 1./90.*m16*RSe45613p + 1.
         /270.*m16*RSe45633p + 1./270.*m16*RSe453p - 1./60.*m16*
         RSe45611p + 2./45.*m16*RSe451p - 13./540.*m16*RSe456p - 11./
         540.*m13*RSe45613p + 1./20.*m13*RSe45633p - 43./540.*m13*
         RSe453p - 1./135.*m13*RSe45611p + 19./540.*m13*RSe451p + 1./45.
         *m13*RSe456p - 1./540.*m11*RSe45613p + 2./135.*m11*RSe45633p
          - 17./540.*m11*RSe453p );

      M13p6x23RV +=  + AbV(m16,L)*AbV(me2,L) * ( 37./360.*m11*RSe45611p
          - 23./270.*m11*RSe451p + 1./1080.*m11*RSe456p );

      M13p6x23RV +=  + AbV(m16,L)*BbV(m11,m33,L) * (  - 1./108.*m33*m66
         *RS34561pe - 1./108.*m33*m66*RS345pe + 1./54.*m16*m33*
         RS34561pe - 1./54.*m16*m33*RS345pe - 1./54.*m13*m66*RS34561pe
          - 1./54.*m13*m66*RS345pe + 1./27.*m13*m16*RS34561pe - 1./27.*
         m13*m16*RS345pe - 1./36.*m11*m66*RS34561pe - 1./36.*m11*m66*
         RS345pe + 1./36.*m11*m33*RS34561pe - 7./108.*m11*m33*RS345pe
          + 1./18.*m11*m16*RS34561pe - 1./18.*m11*m16*RS345pe - 1./54.*
         m11*m13*RS34561pe - 1./18.*m11*m13*RS345pe - 7./108.*pow(
         m11,2)*RS34561pe - 5./108.*pow(m11,2)*RS345pe );

      M13p6x23RV +=  + AbV(m16,L)*BbV(m11,mp2,L) * (  - 1./108.*m33*m66
         *RSp45613e - 1./108.*m33*m66*RSp453e - 1./108.*m33*m66*
         RSp45611e - 1./108.*m33*m66*RSp451e + 1./54.*m16*m33*RSp45613e
          - 1./54.*m16*m33*RSp453e + 1./54.*m16*m33*RSp45611e - 1./54.*
         m16*m33*RSp451e - 1./54.*m13*m66*RSp45613e - 1./54.*m13*m66*
         RSp453e + 1./54.*m13*m66*RSp45611e + 1./54.*m13*m66*RSp451e + 
         1./27.*m13*m16*RSp45613e - 1./27.*m13*m16*RSp453e - 1./27.*m13
         *m16*RSp45611e + 1./27.*m13*m16*RSp451e + 1./27.*m11*mp2*
         RSp45613e - 1./27.*m11*mp2*RSp453e - 1./27.*m11*mp2*RSp45611e
          + 1./27.*m11*mp2*RSp451e - 1./36.*m11*m66*RSp45613e - 1./36.*
         m11*m66*RSp453e - 1./108.*m11*m66*RSp45611e - 1./108.*m11*m66*
         RSp451e - 1./108.*m11*m33*RSp45613e - 1./36.*m11*m33*RSp453e
          - 1./108.*m11*m33*RSp45611e - 1./36.*m11*m33*RSp451e + 1./18.
         *m11*m16*RSp45613e - 1./18.*m11*m16*RSp453e + 1./54.*m11*m16*
         RSp45611e - 1./54.*m11*m16*RSp451e - 1./54.*m11*m13*RSp45613e
          - 1./18.*m11*m13*RSp453e );

      M13p6x23RV +=  + AbV(m16,L)*BbV(m11,mp2,L) * ( 1./54.*m11*m13*
         RSp45611e + 1./18.*m11*m13*RSp451e - 7./108.*pow(m11,2)*
         RSp45613e - 5./108.*pow(m11,2)*RSp453e + 1./36.*pow(m11,2)*
         RSp45611e - 7./108.*pow(m11,2)*RSp451e );

      M13p6x23RV +=  + AbV(m16,L)*BbV(m11,me2,L) * (  - 1./108.*m33*m66
         *RSe45613p - 1./108.*m33*m66*RSe453p - 1./108.*m33*m66*
         RSe45611p - 1./108.*m33*m66*RSe451p + 1./54.*m16*m33*RSe45613p
          - 1./54.*m16*m33*RSe453p + 1./54.*m16*m33*RSe45611p - 1./54.*
         m16*m33*RSe451p - 1./54.*m13*m66*RSe45613p - 1./54.*m13*m66*
         RSe453p + 1./54.*m13*m66*RSe45611p + 1./54.*m13*m66*RSe451p + 
         1./27.*m13*m16*RSe45613p - 1./27.*m13*m16*RSe453p - 1./27.*m13
         *m16*RSe45611p + 1./27.*m13*m16*RSe451p + 1./27.*m11*me2*
         RSe45613p - 1./27.*m11*me2*RSe453p - 1./27.*m11*me2*RSe45611p
          + 1./27.*m11*me2*RSe451p - 1./36.*m11*m66*RSe45613p - 1./36.*
         m11*m66*RSe453p - 1./108.*m11*m66*RSe45611p - 1./108.*m11*m66*
         RSe451p - 1./108.*m11*m33*RSe45613p - 1./36.*m11*m33*RSe453p
          - 1./108.*m11*m33*RSe45611p - 1./36.*m11*m33*RSe451p + 1./18.
         *m11*m16*RSe45613p - 1./18.*m11*m16*RSe453p + 1./54.*m11*m16*
         RSe45611p - 1./54.*m11*m16*RSe451p - 1./54.*m11*m13*RSe45613p
          - 1./18.*m11*m13*RSe453p );

      M13p6x23RV +=  + AbV(m16,L)*BbV(m11,me2,L) * ( 1./54.*m11*m13*
         RSe45611p + 1./18.*m11*m13*RSe451p - 7./108.*pow(m11,2)*
         RSe45613p - 5./108.*pow(m11,2)*RSe453p + 1./36.*pow(m11,2)*
         RSe45611p - 7./108.*pow(m11,2)*RSe451p );

      M13p6x23RV +=  + AbV(m16,L)*BbV(m11,L) * ( 7./540.*m66*RD1456ep
          - 1./108.*m36*RD1456ep + 1./120.*m33*RD1456ep - 1./108.*m33*
         m66*RS14563pe - 1./108.*m33*m66*RC1456pe - 1./108.*m33*m66*
         RS145pe - 1./60.*m16*RD1456ep + 1./54.*m16*m33*RS14563pe + 1./
         54.*m16*m33*RC1456pe - 1./54.*m16*m33*RS145pe - 1./135.*m13*
         RD1456ep - 1./54.*m13*m66*RS14563pe + 1./54.*m13*m66*RC1456pe
          + 1./54.*m13*m66*RS145pe + 1./27.*m13*m16*RS14563pe - 1./27.*
         m13*m16*RC1456pe + 1./27.*m13*m16*RS145pe + 13./1080.*m11*
         RD1456ep - 1./36.*m11*m66*RS14563pe - 1./108.*m11*m66*RC1456pe
          - 1./108.*m11*m66*RS145pe - 1./108.*m11*m33*RS14563pe - 1./
         108.*m11*m33*RC1456pe - 1./36.*m11*m33*RS145pe + 1./18.*m11*
         m16*RS14563pe + 1./54.*m11*m16*RC1456pe - 1./54.*m11*m16*
         RS145pe - 1./54.*m11*m13*RS14563pe + 1./54.*m11*m13*RC1456pe
          + 1./18.*m11*m13*RS145pe - 1./36.*pow(m11,2)*RS14563pe - 1./
         108.*pow(m11,2)*RC1456pe - 1./36.*pow(m11,2)*RS145pe );

      M13p6x23RV +=  + AbV(m16,L)*BbV(m14,L) * ( 1./144.*m44*m66 + 1./
         144.*pow(m44,2) + 1./144.*m33*m66 + 1./144.*m33*m44 - 1./72.*
         m16*m44 - 1./72.*m16*m33 - 1./72.*m14*m66 - 1./36.*m14*m44 - 1.
         /72.*m14*m33 + 1./36.*m14*m16 + 1./36.*pow(m14,2) - 1./72.*m13
         *m66 - 1./72.*m13*m44 + 1./36.*m13*m16 + 1./36.*m13*m14 + 1./
         72.*m11*m66 + 1./36.*m11*m44 + 1./72.*m11*m33 - 1./36.*m11*m16
          - 1./18.*m11*m14 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + AbV(m16,L)*BbV(m15,L) * ( 1./144.*m55*m66 + 1./
         144.*pow(m55,2) + 1./144.*m33*m66 + 1./144.*m33*m55 - 1./72.*
         m16*m55 - 1./72.*m16*m33 - 1./72.*m15*m66 - 1./36.*m15*m55 - 1.
         /72.*m15*m33 + 1./36.*m15*m16 + 1./36.*pow(m15,2) - 1./72.*m13
         *m66 - 1./72.*m13*m55 + 1./36.*m13*m16 + 1./36.*m13*m15 + 1./
         72.*m11*m66 + 1./36.*m11*m55 + 1./72.*m11*m33 - 1./36.*m11*m16
          - 1./18.*m11*m15 - 1./36.*m11*m13 + 1./36.*pow(m11,2) );

      M13p6x23RV +=  + AbV(m16,L)*BbV(m16,L) * ( 1./72.*pow(m66,2) + 1./
         72.*m33*m66 - 1./18.*m16*m66 - 1./36.*m16*m33 + 1./18.*pow(
         m16,2) - 1./36.*m13*m66 + 1./18.*m13*m16 + 1./24.*m11*m66 + 1./
         72.*m11*m33 - 1./12.*m11*m16 - 1./36.*m11*m13 + 1./36.*pow(
         m11,2) );

      M13p6x23RV +=  + AbV(m16,L)*BbV(m33,L) * ( 1./60.*m66*RD3456ep - 
         1./27.*m36*RD3456ep + 19./540.*m33*RD3456ep + 1./270.*m16*
         RD3456ep + 1./20.*m13*RD3456ep + 2./135.*m11*RD3456ep );

      M13p6x23RV +=  + AbV(m16,L)*CbV(m11,L) * (  - 1./108.*m33*m66*
         RD1456ep + 1./54.*m16*m33*RD1456ep + 1./54.*m13*m66*RD1456ep
          - 1./27.*m13*m16*RD1456ep - 1./108.*m11*m66*RD1456ep - 1./108.
         *m11*m33*RD1456ep + 1./54.*m11*m16*RD1456ep + 1./54.*m11*m13*
         RD1456ep - 1./108.*pow(m11,2)*RD1456ep );

      M13p6x23RV +=  + AbV(m33,L)*pi16 * ( 1./24.*m13*RD3456ep - 1./36.
         *m13*RD3456ep*RS34561pe + 1./72.*m13*RD3456ep*RC3456pe - 1./36.
         *m13*RD1456ep*RS34561pe + 1./72.*m13*RD1456ep*RC3456pe + 1./18.
         *m13*me2*RSe45613p*RS34561pe - 1./36.*m13*me2*RC3456pe*
         RSe45613p - 1./36.*m13*me2*RSe45633p*RS34561pe + 1./72.*m13*
         me2*RSe45633p*RC3456pe - 1./36.*m13*me2*RSe45611p*RS34561pe + 
         1./72.*m13*me2*RSe45611p*RC3456pe + 1./18.*m13*mp2*RSp45613e*
         RS34561pe - 1./36.*m13*mp2*RC3456pe*RSp45613e - 1./36.*m13*mp2
         *RSp45633e*RS34561pe + 1./72.*m13*mp2*RSp45633e*RC3456pe - 1./
         36.*m13*mp2*RSp45611e*RS34561pe + 1./72.*m13*mp2*RSp45611e*
         RC3456pe + 1./12.*m13*m36*RS34561pe - 1./24.*m13*m36*RC3456pe
          + 1./12.*m13*m35*RS34561pe - 1./24.*m13*m35*RC3456pe + 1./12.
         *m13*m34*RS34561pe - 1./24.*m13*m34*RC3456pe - 1./8.*m13*m33
          - 1./12.*m13*m33*RS34561pe + 1./18.*m13*m33*pow(RS34561pe,2)
          + 1./24.*m13*m33*RC3456pe - 1./18.*m13*m33*RC3456pe*RS34561pe
          + 1./72.*m13*m33*pow(RC3456pe,2) );

      M13p6x23RV +=  + AbV(m33,L)*pi16 * ( 1./24.*m13*m33*RS345pe + 1./
         24.*m13*m33*RS346pe + 1./24.*m13*m33*RS356pe + 1./12.*m13*m16*
         RS34561pe - 1./24.*m13*m16*RC3456pe + 1./12.*m13*m15*RS34561pe
          - 1./24.*m13*m15*RC3456pe + 1./12.*m13*m14*RS34561pe - 1./24.
         *m13*m14*RC3456pe - 1./12.*pow(m13,2)*RS34561pe + 1./27.*pow(
         m13,2)*pow(RS34561pe,2) + 1./27.*pow(m13,2)*RS14563pe*
         RS34561pe + 1./27.*pow(m13,2)*RSe45613p*RS34561pe + 1./27.*
         pow(m13,2)*RSp45613e*RS34561pe + 1./24.*pow(m13,2)*RC3456pe - 
         1./27.*pow(m13,2)*RC3456pe*RS34561pe - 1./54.*pow(m13,2)*
         RC3456pe*RS14563pe - 1./54.*pow(m13,2)*RC3456pe*RSe45613p - 1./
         54.*pow(m13,2)*RC3456pe*RSp45613e + 1./108.*pow(m13,2)*pow(
         RC3456pe,2) - 1./54.*pow(m13,2)*RSe45633p*RS34561pe + 1./108.*
         pow(m13,2)*RSe45633p*RC3456pe - 1./54.*pow(m13,2)*RSp45633e*
         RS34561pe + 1./108.*pow(m13,2)*RSp45633e*RC3456pe - 1./54.*
         pow(m13,2)*RC1456pe*RS34561pe + 1./108.*pow(m13,2)*RC1456pe*
         RC3456pe );

      M13p6x23RV +=  + AbV(m33,L)*pi16 * (  - 1./54.*pow(m13,2)*
         RSe45611p*RS34561pe + 1./108.*pow(m13,2)*RSe45611p*RC3456pe - 
         1./54.*pow(m13,2)*RSp45611e*RS34561pe + 1./108.*pow(m13,2)*
         RSp45611e*RC3456pe + 1./18.*m11*m13*RS14563pe*RS34561pe - 1./
         36.*m11*m13*RC3456pe*RS14563pe - 1./36.*m11*m13*RC1456pe*
         RS34561pe + 1./72.*m11*m13*RC1456pe*RC3456pe );

      M13p6x23RV +=  + pow(AbV(m33,L),2) * (  - 1./270.*RD3456ep*
         RS34561pe + 1./540.*RD3456ep*RC3456pe + 11./180.*m33 + 1./90.*
         m33*pow(RS34561pe,2) + 2./81.*m33*RC3456pe*RS34561pe + 1./27.*
         m33*RC3456pe*RS14563pe - 17./1620.*m33*pow(RC3456pe,2) + 1./
         360.*m13 - 7./810.*m13*pow(RS34561pe,2) + 7./810.*m13*RC3456pe
         *RS34561pe - 7./3240.*m13*pow(RC3456pe,2) + 11./180.*m11 + 13./
         810.*m11*pow(RS34561pe,2) + 1./270.*m11*RC3456pe*RS34561pe + 
         13./1620.*m11*pow(RC3456pe,2) + 2./81.*RS(m33,m44,m55,m66,m11,
         me2,mp2)*m33*RS34561pe - 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2
         )*m33*RC3456pe + 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RS34561pe - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RC3456pe );

      M13p6x23RV +=  + AbV(m33,L)*AbV(m34,L) * (  - 29./540.*RD3456ep
          - 7./540.*m44*RS34561pe - 1./54.*m44*RS14563pe + 7./540.*m44*
         RC3456pe - 1./60.*m44*RS356pe - 1./54.*m44*RS156pe - 1./90.*
         m34*RS34561pe + 1./27.*m34*RS14563pe - 1./60.*m34*RC3456pe + 2.
         /45.*m34*RS356pe - 1./27.*m34*RS156pe + 43./540.*m33*RS34561pe
          + 1./27.*m33*RSe45613p + 1./27.*m33*RSp45613e + 13./1080.*m33
         *RC3456pe - 1./27.*m33*RSe45633p - 1./27.*m33*RSp45633e - 1./
         45.*m33*RS356pe + 1./27.*m33*RSe563p + 1./27.*m33*RSp563e - 2./
         27.*m33*RS156pe - 1./27.*m33*RSe561p - 1./27.*m33*RSp561e + 1./
         27.*m14*RS34561pe - 1./108.*m14*RC3456pe - 1./54.*m14*RS356pe
          - 11./540.*m13*RS34561pe + 1./27.*m13*RS14563pe - 1./135.*m13
         *RC3456pe + 19./540.*m13*RS356pe - 1./27.*m13*RS156pe + 1./90.
         *m11*RS34561pe + 1./54.*m11*RS14563pe + 1./120.*m11*RC3456pe
          - 1./20.*m11*RS356pe - 1./54.*m11*RS156pe + 1./27.*RS(m33,m44
         ,m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11 );

      M13p6x23RV +=  + AbV(m33,L)*AbV(m35,L) * (  - 29./540.*RD3456ep
          - 7./540.*m55*RS34561pe - 1./54.*m55*RS14563pe + 7./540.*m55*
         RC3456pe - 1./60.*m55*RS346pe - 1./54.*m55*RS146pe - 1./90.*
         m35*RS34561pe + 1./27.*m35*RS14563pe - 1./60.*m35*RC3456pe + 2.
         /45.*m35*RS346pe - 1./27.*m35*RS146pe + 43./540.*m33*RS34561pe
          + 1./27.*m33*RSe45613p + 1./27.*m33*RSp45613e + 13./1080.*m33
         *RC3456pe - 1./27.*m33*RSe45633p - 1./27.*m33*RSp45633e - 1./
         45.*m33*RS346pe + 1./27.*m33*RSe463p + 1./27.*m33*RSp463e - 2./
         27.*m33*RS146pe - 1./27.*m33*RSe461p - 1./27.*m33*RSp461e + 1./
         27.*m15*RS34561pe - 1./108.*m15*RC3456pe - 1./54.*m15*RS346pe
          - 11./540.*m13*RS34561pe + 1./27.*m13*RS14563pe - 1./135.*m13
         *RC3456pe + 19./540.*m13*RS346pe - 1./27.*m13*RS146pe + 1./90.
         *m11*RS34561pe + 1./54.*m11*RS14563pe + 1./120.*m11*RC3456pe
          - 1./20.*m11*RS346pe - 1./54.*m11*RS146pe + 1./27.*RS(m33,m44
         ,m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11 );

      M13p6x23RV +=  + AbV(m33,L)*AbV(m36,L) * (  - 29./540.*RD3456ep
          - 7./540.*m66*RS34561pe - 1./54.*m66*RS14563pe + 7./540.*m66*
         RC3456pe - 1./60.*m66*RS345pe - 1./54.*m66*RS145pe - 1./90.*
         m36*RS34561pe + 1./27.*m36*RS14563pe - 1./60.*m36*RC3456pe + 2.
         /45.*m36*RS345pe - 1./27.*m36*RS145pe + 43./540.*m33*RS34561pe
          + 1./27.*m33*RSe45613p + 1./27.*m33*RSp45613e + 13./1080.*m33
         *RC3456pe - 1./27.*m33*RSe45633p - 1./27.*m33*RSp45633e - 1./
         45.*m33*RS345pe + 1./27.*m33*RSe453p + 1./27.*m33*RSp453e - 2./
         27.*m33*RS145pe - 1./27.*m33*RSe451p - 1./27.*m33*RSp451e + 1./
         27.*m16*RS34561pe - 1./108.*m16*RC3456pe - 1./54.*m16*RS345pe
          - 11./540.*m13*RS34561pe + 1./27.*m13*RS14563pe - 1./135.*m13
         *RC3456pe + 19./540.*m13*RS345pe - 1./27.*m13*RS145pe + 1./90.
         *m11*RS34561pe + 1./54.*m11*RS14563pe + 1./120.*m11*RC3456pe
          - 1./20.*m11*RS345pe - 1./54.*m11*RS145pe + 1./27.*RS(m33,m44
         ,m55,m66,m11,me2,mp2)*m33 + 1./54.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m11 );

      M13p6x23RV +=  + AbV(m33,L)*AbV(m44,L) * (  - 1./54.*m44*pow(
         RS356pe,2) + 1./27.*m44*RS156pe*RS356pe + 1./162.*m44*RS456pe*
         pow(RS356pe,2) - 1./81.*m44*RS456pe*RS156pe*RS356pe );

      M13p6x23RV +=  + AbV(m33,L)*AbV(m45,L) * (  - 1./108.*m55*pow(
         RS346pe,2) - 1./162.*m55*RS356pe*RS346pe - 1./324.*m55*pow(
         RS356pe,2) + 1./54.*m55*RS146pe*RS346pe + 1./162.*m55*RS146pe*
         RS356pe + 1./162.*m55*RS156pe*RS346pe + 1./162.*m55*RS156pe*
         RS356pe + 1./162.*m45*pow(RS346pe,2) - 1./81.*m45*RS356pe*
         RS346pe + 1./162.*m45*pow(RS356pe,2) - 1./81.*m45*RS146pe*
         RS346pe + 1./81.*m45*RS146pe*RS356pe + 1./81.*m45*RS156pe*
         RS346pe - 1./81.*m45*RS156pe*RS356pe - 1./324.*m44*pow(
         RS346pe,2) - 1./162.*m44*RS356pe*RS346pe - 1./108.*m44*pow(
         RS356pe,2) + 1./162.*m44*RS146pe*RS346pe + 1./162.*m44*RS146pe
         *RS356pe + 1./162.*m44*RS156pe*RS346pe + 1./54.*m44*RS156pe*
         RS356pe + 1./324.*m33*pow(RS346pe,2) - 1./162.*m33*RS356pe*
         RS346pe + 1./324.*m33*pow(RS356pe,2) - 1./81.*m33*RSe563p*
         RS346pe + 1./81.*m33*RSe563p*RS356pe + 1./81.*m33*RSe463p*
         RS346pe - 1./81.*m33*RSe463p*RS356pe - 1./81.*m33*RSp563e*
         RS346pe );

      M13p6x23RV +=  + AbV(m33,L)*AbV(m45,L) * ( 1./81.*m33*RSp563e*
         RS356pe + 1./81.*m33*RSp463e*RS346pe - 1./81.*m33*RSp463e*
         RS356pe - 1./54.*m33*RS146pe*RS346pe + 1./54.*m33*RS146pe*
         RS356pe + 1./54.*m33*RS156pe*RS346pe - 1./54.*m33*RS156pe*
         RS356pe + 1./81.*m33*RSe561p*RS346pe - 1./81.*m33*RSe561p*
         RS356pe - 1./81.*m33*RSe461p*RS346pe + 1./81.*m33*RSe461p*
         RS356pe + 1./81.*m33*RSp561e*RS346pe - 1./81.*m33*RSp561e*
         RS356pe - 1./81.*m33*RSp461e*RS346pe + 1./81.*m33*RSp461e*
         RS356pe + 1./162.*m13*pow(RS346pe,2) - 1./81.*m13*RS356pe*
         RS346pe + 1./162.*m13*pow(RS356pe,2) - 1./81.*m13*RS146pe*
         RS346pe + 1./81.*m13*RS146pe*RS356pe + 1./81.*m13*RS156pe*
         RS346pe - 1./81.*m13*RS156pe*RS356pe - 1./324.*m11*pow(
         RS346pe,2) + 1./162.*m11*RS356pe*RS346pe - 1./324.*m11*pow(
         RS356pe,2) - 1./162.*m11*RS146pe*RS346pe + 1./162.*m11*RS146pe
         *RS356pe + 1./162.*m11*RS156pe*RS346pe - 1./162.*m11*RS156pe*
         RS356pe );

      M13p6x23RV +=  + AbV(m33,L)*AbV(m46,L) * (  - 1./108.*m66*pow(
         RS345pe,2) - 1./162.*m66*RS356pe*RS345pe - 1./324.*m66*pow(
         RS356pe,2) + 1./54.*m66*RS145pe*RS345pe + 1./162.*m66*RS145pe*
         RS356pe + 1./162.*m66*RS156pe*RS345pe + 1./162.*m66*RS156pe*
         RS356pe + 1./162.*m46*pow(RS345pe,2) - 1./81.*m46*RS356pe*
         RS345pe + 1./162.*m46*pow(RS356pe,2) - 1./81.*m46*RS145pe*
         RS345pe + 1./81.*m46*RS145pe*RS356pe + 1./81.*m46*RS156pe*
         RS345pe - 1./81.*m46*RS156pe*RS356pe - 1./324.*m44*pow(
         RS345pe,2) - 1./162.*m44*RS356pe*RS345pe - 1./108.*m44*pow(
         RS356pe,2) + 1./162.*m44*RS145pe*RS345pe + 1./162.*m44*RS145pe
         *RS356pe + 1./162.*m44*RS156pe*RS345pe + 1./54.*m44*RS156pe*
         RS356pe + 1./324.*m33*pow(RS345pe,2) - 1./162.*m33*RS356pe*
         RS345pe + 1./324.*m33*pow(RS356pe,2) - 1./81.*m33*RSe563p*
         RS345pe + 1./81.*m33*RSe563p*RS356pe + 1./81.*m33*RSe453p*
         RS345pe - 1./81.*m33*RSe453p*RS356pe - 1./81.*m33*RSp563e*
         RS345pe );

      M13p6x23RV +=  + AbV(m33,L)*AbV(m46,L) * ( 1./81.*m33*RSp563e*
         RS356pe + 1./81.*m33*RSp453e*RS345pe - 1./81.*m33*RSp453e*
         RS356pe - 1./54.*m33*RS145pe*RS345pe + 1./54.*m33*RS145pe*
         RS356pe + 1./54.*m33*RS156pe*RS345pe - 1./54.*m33*RS156pe*
         RS356pe + 1./81.*m33*RSe561p*RS345pe - 1./81.*m33*RSe561p*
         RS356pe - 1./81.*m33*RSe451p*RS345pe + 1./81.*m33*RSe451p*
         RS356pe + 1./81.*m33*RSp561e*RS345pe - 1./81.*m33*RSp561e*
         RS356pe - 1./81.*m33*RSp451e*RS345pe + 1./81.*m33*RSp451e*
         RS356pe + 1./162.*m13*pow(RS345pe,2) - 1./81.*m13*RS356pe*
         RS345pe + 1./162.*m13*pow(RS356pe,2) - 1./81.*m13*RS145pe*
         RS345pe + 1./81.*m13*RS145pe*RS356pe + 1./81.*m13*RS156pe*
         RS345pe - 1./81.*m13*RS156pe*RS356pe - 1./324.*m11*pow(
         RS345pe,2) + 1./162.*m11*RS356pe*RS345pe - 1./324.*m11*pow(
         RS356pe,2) - 1./162.*m11*RS145pe*RS345pe + 1./162.*m11*RS145pe
         *RS356pe + 1./162.*m11*RS156pe*RS345pe - 1./162.*m11*RS156pe*
         RS356pe );

      M13p6x23RV +=  + AbV(m33,L)*AbV(m55,L) * (  - 1./54.*m55*pow(
         RS346pe,2) + 1./27.*m55*RS146pe*RS346pe + 1./162.*m55*RS546pe*
         pow(RS346pe,2) - 1./81.*m55*RS546pe*RS146pe*RS346pe );

      M13p6x23RV +=  + AbV(m33,L)*AbV(m56,L) * (  - 1./108.*m66*pow(
         RS345pe,2) - 1./162.*m66*RS346pe*RS345pe - 1./324.*m66*pow(
         RS346pe,2) + 1./54.*m66*RS145pe*RS345pe + 1./162.*m66*RS145pe*
         RS346pe + 1./162.*m66*RS146pe*RS345pe + 1./162.*m66*RS146pe*
         RS346pe + 1./162.*m56*pow(RS345pe,2) - 1./81.*m56*RS346pe*
         RS345pe + 1./162.*m56*pow(RS346pe,2) - 1./81.*m56*RS145pe*
         RS345pe + 1./81.*m56*RS145pe*RS346pe + 1./81.*m56*RS146pe*
         RS345pe - 1./81.*m56*RS146pe*RS346pe - 1./324.*m55*pow(
         RS345pe,2) - 1./162.*m55*RS346pe*RS345pe - 1./108.*m55*pow(
         RS346pe,2) + 1./162.*m55*RS145pe*RS345pe + 1./162.*m55*RS145pe
         *RS346pe + 1./162.*m55*RS146pe*RS345pe + 1./54.*m55*RS146pe*
         RS346pe + 1./324.*m33*pow(RS345pe,2) - 1./162.*m33*RS346pe*
         RS345pe + 1./324.*m33*pow(RS346pe,2) - 1./81.*m33*RSe463p*
         RS345pe + 1./81.*m33*RSe463p*RS346pe + 1./81.*m33*RSe453p*
         RS345pe - 1./81.*m33*RSe453p*RS346pe - 1./81.*m33*RSp463e*
         RS345pe );

      M13p6x23RV +=  + AbV(m33,L)*AbV(m56,L) * ( 1./81.*m33*RSp463e*
         RS346pe + 1./81.*m33*RSp453e*RS345pe - 1./81.*m33*RSp453e*
         RS346pe - 1./54.*m33*RS145pe*RS345pe + 1./54.*m33*RS145pe*
         RS346pe + 1./54.*m33*RS146pe*RS345pe - 1./54.*m33*RS146pe*
         RS346pe + 1./81.*m33*RSe461p*RS345pe - 1./81.*m33*RSe461p*
         RS346pe - 1./81.*m33*RSe451p*RS345pe + 1./81.*m33*RSe451p*
         RS346pe + 1./81.*m33*RSp461e*RS345pe - 1./81.*m33*RSp461e*
         RS346pe - 1./81.*m33*RSp451e*RS345pe + 1./81.*m33*RSp451e*
         RS346pe + 1./162.*m13*pow(RS345pe,2) - 1./81.*m13*RS346pe*
         RS345pe + 1./162.*m13*pow(RS346pe,2) - 1./81.*m13*RS145pe*
         RS345pe + 1./81.*m13*RS145pe*RS346pe + 1./81.*m13*RS146pe*
         RS345pe - 1./81.*m13*RS146pe*RS346pe - 1./324.*m11*pow(
         RS345pe,2) + 1./162.*m11*RS346pe*RS345pe - 1./324.*m11*pow(
         RS346pe,2) - 1./162.*m11*RS145pe*RS345pe + 1./162.*m11*RS145pe
         *RS346pe + 1./162.*m11*RS146pe*RS345pe - 1./162.*m11*RS146pe*
         RS346pe );

      M13p6x23RV +=  + AbV(m33,L)*AbV(m66,L) * (  - 1./54.*m66*pow(
         RS345pe,2) + 1./27.*m66*RS145pe*RS345pe + 1./162.*m66*RS645pe*
         pow(RS345pe,2) - 1./81.*m66*RS645pe*RS145pe*RS345pe );

      M13p6x23RV +=  + AbV(m33,L)*AbV(mp2,L) * (  - 1./270.*RD3456ep*
         RSp45613e + 1./540.*RSp45633e*RD3456ep + 1./540.*RSp45611e*
         RD3456ep + 1./135.*mp2*RSp45613e*RS34561pe - 1./270.*mp2*
         RC3456pe*RSp45613e - 1./270.*mp2*RSp45633e*RS34561pe + 1./540.
         *mp2*RSp45633e*RC3456pe - 1./270.*mp2*RSp45611e*RS34561pe + 1./
         540.*mp2*RSp45611e*RC3456pe + 1./162.*m66*RSp456e*pow(
         RS345pe,2) - 1./81.*m66*RSp456e*RS145pe*RS345pe + 1./162.*m55*
         RSp465e*pow(RS346pe,2) - 1./81.*m55*RSp465e*RS146pe*RS346pe + 
         1./162.*m44*RSp564e*pow(RS356pe,2) - 1./81.*m44*RSp564e*
         RS156pe*RS356pe + 16./405.*m33*RSp45613e*RS34561pe + 13./810.*
         m33*RC3456pe*RSp45613e + 23./810.*m33*RSp45633e*RS34561pe + 1./
         27.*m33*RSp45633e*RS14563pe - 37./1620.*m33*RSp45633e*RC3456pe
          + 1./162.*m33*RSp45611e*RS34561pe - 19./1620.*m33*RSp45611e*
         RC3456pe - 7./405.*m13*RSp45613e*RS34561pe + 7./810.*m13*
         RC3456pe*RSp45613e + 7./810.*m13*RSp45633e*RS34561pe - 7./1620.
         *m13*RSp45633e*RC3456pe );

      M13p6x23RV +=  + AbV(m33,L)*AbV(mp2,L) * ( 7./810.*m13*RSp45611e*
         RS34561pe - 7./1620.*m13*RSp45611e*RC3456pe + 2./45.*m11*
         RSp45613e*RS34561pe - 1./405.*m11*RC3456pe*RSp45613e + 1./270.
         *m11*RSp45633e*RS34561pe + 13./810.*m11*RSp45633e*RC3456pe + 7.
         /270.*m11*RSp45611e*RS34561pe - 11./810.*m11*RSp45611e*
         RC3456pe + 2./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33*
         RSp45613e - 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33*
         RSp45633e - 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33*
         RSp45611e + 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RSp45613e - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RSp45633e - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RSp45611e );

      M13p6x23RV +=  + AbV(m33,L)*AbV(me2,L) * (  - 1./270.*RD3456ep*
         RSe45613p + 1./540.*RSe45633p*RD3456ep + 1./540.*RSe45611p*
         RD3456ep + 1./135.*me2*RSe45613p*RS34561pe - 1./270.*me2*
         RC3456pe*RSe45613p - 1./270.*me2*RSe45633p*RS34561pe + 1./540.
         *me2*RSe45633p*RC3456pe - 1./270.*me2*RSe45611p*RS34561pe + 1./
         540.*me2*RSe45611p*RC3456pe + 1./162.*m66*RSe456p*pow(
         RS345pe,2) - 1./81.*m66*RSe456p*RS145pe*RS345pe + 1./162.*m55*
         RSe465p*pow(RS346pe,2) - 1./81.*m55*RSe465p*RS146pe*RS346pe + 
         1./162.*m44*RSe564p*pow(RS356pe,2) - 1./81.*m44*RSe564p*
         RS156pe*RS356pe + 16./405.*m33*RSe45613p*RS34561pe + 13./810.*
         m33*RC3456pe*RSe45613p + 23./810.*m33*RSe45633p*RS34561pe + 1./
         27.*m33*RSe45633p*RS14563pe - 37./1620.*m33*RSe45633p*RC3456pe
          + 1./162.*m33*RSe45611p*RS34561pe - 19./1620.*m33*RSe45611p*
         RC3456pe - 7./405.*m13*RSe45613p*RS34561pe + 7./810.*m13*
         RC3456pe*RSe45613p + 7./810.*m13*RSe45633p*RS34561pe - 7./1620.
         *m13*RSe45633p*RC3456pe );

      M13p6x23RV +=  + AbV(m33,L)*AbV(me2,L) * ( 7./810.*m13*RSe45611p*
         RS34561pe - 7./1620.*m13*RSe45611p*RC3456pe + 2./45.*m11*
         RSe45613p*RS34561pe - 1./405.*m11*RC3456pe*RSe45613p + 1./270.
         *m11*RSe45633p*RS34561pe + 13./810.*m11*RSe45633p*RC3456pe + 7.
         /270.*m11*RSe45611p*RS34561pe - 11./810.*m11*RSe45611p*
         RC3456pe + 2./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33*
         RSe45613p - 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33*
         RSe45633p - 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m33*
         RSe45611p + 1./81.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RSe45613p - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RSe45633p - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*m11*
         RSe45611p );

      M13p6x23RV +=  + AbV(m33,L)*BbV(m11,m33,L) * ( 1./54.*pow(m33,2)*
         RC3456pe*RS14563pe + 1./27.*m13*m33*RC3456pe*RS14563pe + 1./18.
         *m11*m33*RC3456pe*RS14563pe );

      M13p6x23RV +=  + AbV(m33,L)*BbV(m11,L) * ( 1./540.*RD1456ep*
         RD3456ep + 1./162.*m33*RD1456ep*RS34561pe - 19./1620.*m33*
         RD1456ep*RC3456pe + 7./810.*m13*RD1456ep*RS34561pe - 7./1620.*
         m13*RD1456ep*RC3456pe + 1./45.*m11*RD1456ep*RS34561pe - 19./
         1620.*m11*RD1456ep*RC3456pe - 1./81.*RS(m33,m44,m55,m66,m11,
         me2,mp2)*m33*RD1456ep - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2
         )*m11*RD1456ep );

      M13p6x23RV +=  + AbV(m33,L)*BbV(m33,mp2,L) * ( 1./18.*pow(m33,2)*
         RC3456pe*RSp45613e + 1./54.*pow(m33,2)*RSp45633e*RC3456pe + 1./
         27.*m13*m33*RC3456pe*RSp45613e - 1./27.*m13*m33*RSp45633e*
         RC3456pe + 1./54.*m11*m33*RC3456pe*RSp45613e + 1./54.*m11*m33*
         RSp45633e*RC3456pe );

      M13p6x23RV +=  + AbV(m33,L)*BbV(m33,me2,L) * ( 1./18.*pow(m33,2)*
         RC3456pe*RSe45613p + 1./54.*pow(m33,2)*RSe45633p*RC3456pe + 1./
         27.*m13*m33*RC3456pe*RSe45613p - 1./27.*m13*m33*RSe45633p*
         RC3456pe + 1./54.*m11*m33*RC3456pe*RSe45613p + 1./54.*m11*m33*
         RSe45633p*RC3456pe );

      M13p6x23RV +=  + AbV(m33,L)*BbV(m33,L) * ( 1./540.*pow(
         RD3456ep,2) + 2./81.*m33*RD3456ep*RS34561pe + 1./27.*m33*
         RD3456ep*RS14563pe - 17./810.*m33*RD3456ep*RC3456pe + 1./12.*
         pow(m33,2) + 1./18.*pow(m33,2)*RC3456pe*RS34561pe + 1./54.*
         pow(m33,2)*pow(RC3456pe,2) + 7./810.*m13*RD3456ep*RS34561pe - 
         7./1620.*m13*RD3456ep*RC3456pe - 1./6.*m13*m33 + 1./27.*m13*
         m33*RC3456pe*RS34561pe - 1./27.*m13*m33*pow(RC3456pe,2) + 1./
         270.*m11*RD3456ep*RS34561pe + 13./810.*m11*RD3456ep*RC3456pe
          + 1./12.*m11*m33 + 1./54.*m11*m33*RC3456pe*RS34561pe + 1./54.
         *m11*m33*pow(RC3456pe,2) - 1./81.*RS(m33,m44,m55,m66,m11,me2,
         mp2)*m33*RD3456ep - 1./162.*RS(m33,m44,m55,m66,m11,me2,mp2)*
         m11*RD3456ep );

      M13p6x23RV +=  + AbV(m33,L)*BbV(m34,L) * ( 1./216.*m44*RD3456ep
          - 1./432.*pow(m44,2)*RC3456pe - 1./216.*pow(m44,2)*RS356pe - 
         1./108.*m34*RD3456ep + 1./108.*m34*m44*RC3456pe - 1./108.*pow(
         m34,2)*RC3456pe + 1./54.*pow(m34,2)*RS356pe + 1./108.*m33*
         RD3456ep - 1./144.*m33*m44*RC3456pe - 5./216.*m33*m44*RS356pe
          + 1./72.*m33*m34*RC3456pe + 1./108.*m33*m34*RS356pe - 1./216.
         *pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS356pe - 1./108.*m13
         *RD3456ep + 1./216.*m13*m44*RC3456pe + 1./108.*m13*m44*RS356pe
          - 1./108.*m13*m34*RC3456pe + 1./54.*m13*m34*RS356pe + 1./216.
         *m13*m33*RC3456pe + 1./36.*m13*m33*RS356pe + 1./216.*m11*
         RD3456ep - 1./432.*m11*m44*RC3456pe - 1./216.*m11*m44*RS356pe
          + 1./216.*m11*m34*RC3456pe - 1./108.*m11*m34*RS356pe - 1./432.
         *m11*m33*RC3456pe - 1./72.*m11*m33*RS356pe );

      M13p6x23RV +=  + AbV(m33,L)*BbV(m35,L) * ( 1./216.*m55*RD3456ep
          - 1./432.*pow(m55,2)*RC3456pe - 1./216.*pow(m55,2)*RS346pe - 
         1./108.*m35*RD3456ep + 1./108.*m35*m55*RC3456pe - 1./108.*pow(
         m35,2)*RC3456pe + 1./54.*pow(m35,2)*RS346pe + 1./108.*m33*
         RD3456ep - 1./144.*m33*m55*RC3456pe - 5./216.*m33*m55*RS346pe
          + 1./72.*m33*m35*RC3456pe + 1./108.*m33*m35*RS346pe - 1./216.
         *pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS346pe - 1./108.*m13
         *RD3456ep + 1./216.*m13*m55*RC3456pe + 1./108.*m13*m55*RS346pe
          - 1./108.*m13*m35*RC3456pe + 1./54.*m13*m35*RS346pe + 1./216.
         *m13*m33*RC3456pe + 1./36.*m13*m33*RS346pe + 1./216.*m11*
         RD3456ep - 1./432.*m11*m55*RC3456pe - 1./216.*m11*m55*RS346pe
          + 1./216.*m11*m35*RC3456pe - 1./108.*m11*m35*RS346pe - 1./432.
         *m11*m33*RC3456pe - 1./72.*m11*m33*RS346pe );

      M13p6x23RV +=  + AbV(m33,L)*BbV(m36,L) * ( 1./216.*m66*RD3456ep
          - 1./432.*pow(m66,2)*RC3456pe - 1./216.*pow(m66,2)*RS345pe - 
         1./108.*m36*RD3456ep + 1./108.*m36*m66*RC3456pe - 1./108.*pow(
         m36,2)*RC3456pe + 1./54.*pow(m36,2)*RS345pe + 1./108.*m33*
         RD3456ep - 1./144.*m33*m66*RC3456pe - 5./216.*m33*m66*RS345pe
          + 1./72.*m33*m36*RC3456pe + 1./108.*m33*m36*RS345pe - 1./216.
         *pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS345pe - 1./108.*m13
         *RD3456ep + 1./216.*m13*m66*RC3456pe + 1./108.*m13*m66*RS345pe
          - 1./108.*m13*m36*RC3456pe + 1./54.*m13*m36*RS345pe + 1./216.
         *m13*m33*RC3456pe + 1./36.*m13*m33*RS345pe + 1./216.*m11*
         RD3456ep - 1./432.*m11*m66*RC3456pe - 1./216.*m11*m66*RS345pe
          + 1./216.*m11*m36*RC3456pe - 1./108.*m11*m36*RS345pe - 1./432.
         *m11*m33*RC3456pe - 1./72.*m11*m33*RS345pe );

      M13p6x23RV +=  + AbV(m33,L)*CbV(m33,L) * ( 1./54.*pow(m33,2)*
         RD3456ep*RC3456pe - 1./27.*m13*m33*RD3456ep*RC3456pe + 1./54.*
         m11*m33*RD3456ep*RC3456pe );

      M13p6x23RV +=  + AbV(m34,L)*pi16 * (  - 1./24.*m13*RD1456ep - 1./
         24.*m13*me2*RSe45611p + 1./12.*m13*me2*RSe561p - 1./24.*m13*
         me2*RSe564p - 1./24.*m13*mp2*RSp45611e + 1./12.*m13*mp2*
         RSp561e - 1./24.*m13*mp2*RSp564e + 1./8.*m13*m46 + 1./8.*m13*
         m45 + 1./8.*m13*m44 - 1./24.*m13*m44*RS456pe - 1./8.*m13*m34
          + 1./24.*m13*m34*RS34561pe + 1./24.*m13*m34*RS14563pe + 1./24.
         *m13*m34*RSe45613p + 1./24.*m13*m34*RSp45613e - 1./24.*m13*m34
         *RC3456pe - 1./24.*m13*m34*RSe45633p - 1./24.*m13*m34*
         RSp45633e + 1./24.*m13*m34*RS356pe + 1./24.*m13*m34*RSe563p + 
         1./24.*m13*m34*RSp563e - 1./24.*m13*m34*RS156pe - 1./24.*m13*
         m34*RSe561p - 1./24.*m13*m34*RSp561e + 1./8.*m13*m16 + 1./8.*
         m13*m15 + 1./8.*m13*m14 - 1./24.*m13*m14*RC1456pe - 1./24.*m13
         *m14*RSe45611p - 1./24.*m13*m14*RSp45611e + 1./12.*m13*m14*
         RS156pe - 1./24.*m13*m14*RS456pe + 1./12.*m13*m14*RSe561p + 1./
         12.*m13*m14*RSp561e - 1./24.*m13*m14*RSe564p - 1./24.*m13*m14*
         RSp564e );

      M13p6x23RV +=  + AbV(m34,L)*pi16 * (  - 1./8.*pow(m13,2) + 1./72.
         *pow(m13,2)*RC1456pe + 1./72.*pow(m13,2)*RSe45611p + 1./72.*
         pow(m13,2)*RSp45611e - 1./36.*pow(m13,2)*RS156pe + 1./72.*pow(
         m13,2)*RS456pe - 1./36.*pow(m13,2)*RSe561p - 1./36.*pow(m13,2)
         *RSp561e + 1./72.*pow(m13,2)*RSe564p + 1./72.*pow(m13,2)*
         RSp564e - 1./24.*m11*m13*RC1456pe + 1./12.*m11*m13*RS156pe );

      M13p6x23RV +=  + pow(AbV(m34,L),2) * (  - 11./360.*m44 + 11./180.
         *m34 - 7./180.*m33 + 1./60.*m13 - 1./120.*m11 );

      M13p6x23RV +=  + AbV(m34,L)*AbV(m35,L) * (  - 11./360.*m55 - 11./
         360.*m44 + 11./180.*m35 + 11./180.*m34 - 7./90.*m33 + 1./30.*
         m13 - 1./60.*m11 );

      M13p6x23RV +=  + AbV(m34,L)*AbV(m36,L) * (  - 11./360.*m66 - 11./
         360.*m44 + 11./180.*m36 + 11./180.*m34 - 7./90.*m33 + 1./30.*
         m13 - 1./60.*m11 );

      M13p6x23RV +=  + AbV(m34,L)*AbV(m44,L) * (  - 1./20.*m44 + 1./60.
         *m44*RS456pe + 13./180.*m34 - 13./540.*m34*RS456pe - 1./360.*
         m33 + 1./1080.*m33*RS456pe + 1./36.*m14 - 1./108.*m14*RS456pe
          - 1./15.*m13 + 1./45.*m13*RS456pe + 7./360.*m11 - 7./1080.*
         m11*RS456pe );

      M13p6x23RV +=  + AbV(m34,L)*AbV(m45,L) * ( 7./360.*m55 - 1./15.*
         m45 - 1./360.*m44 + 13./180.*m34 - 1./360.*m33 + 1./36.*m15 - 
         1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + AbV(m34,L)*AbV(m46,L) * ( 7./360.*m66 - 1./15.*
         m46 - 1./360.*m44 + 13./180.*m34 - 1./360.*m33 + 1./36.*m16 - 
         1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + AbV(m34,L)*AbV(mp2,L) * ( 11./135.*mp2*RSp45613e
          - 29./540.*mp2*RSp45633e + 7./270.*mp2*RSp563e - 1./180.*mp2*
         RSp45611e - 19./270.*mp2*RSp561e + 1./45.*mp2*RSp564e - 7./540.
         *m44*RSp45613e + 7./540.*m44*RSp45633e - 1./60.*m44*RSp563e + 
         1./60.*m44*RSp45611e + 1./180.*m44*RSp561e - 1./180.*m44*
         RSp564e - 1./90.*m34*RSp45613e - 1./60.*m34*RSp45633e + 2./45.
         *m34*RSp563e + 1./270.*m34*RSp45611e + 1./270.*m34*RSp561e - 
         13./540.*m34*RSp564e - 1./540.*m33*RSp45613e + 37./360.*m33*
         RSp45633e - 23./270.*m33*RSp563e + 2./135.*m33*RSp45611e - 17./
         540.*m33*RSp561e + 1./1080.*m33*RSp564e + 1./27.*m14*RSp45613e
          - 1./108.*m14*RSp45633e - 1./54.*m14*RSp563e - 1./27.*m14*
         RSp45611e + 1./27.*m14*RSp561e - 1./108.*m14*RSp564e - 11./540.
         *m13*RSp45613e - 1./135.*m13*RSp45633e + 19./540.*m13*RSp563e
          + 1./20.*m13*RSp45611e - 43./540.*m13*RSp561e + 1./45.*m13*
         RSp564e + 4./135.*m11*RSp45613e + 1./120.*m11*RSp45633e - 1./
         20.*m11*RSp563e );

      M13p6x23RV +=  + AbV(m34,L)*AbV(mp2,L) * ( 11./270.*m11*RSp45611e
          - 1./45.*m11*RSp561e - 7./1080.*m11*RSp564e );

      M13p6x23RV +=  + AbV(m34,L)*AbV(me2,L) * ( 11./135.*me2*RSe45613p
          - 29./540.*me2*RSe45633p + 7./270.*me2*RSe563p - 1./180.*me2*
         RSe45611p - 19./270.*me2*RSe561p + 1./45.*me2*RSe564p - 7./540.
         *m44*RSe45613p + 7./540.*m44*RSe45633p - 1./60.*m44*RSe563p + 
         1./60.*m44*RSe45611p + 1./180.*m44*RSe561p - 1./180.*m44*
         RSe564p - 1./90.*m34*RSe45613p - 1./60.*m34*RSe45633p + 2./45.
         *m34*RSe563p + 1./270.*m34*RSe45611p + 1./270.*m34*RSe561p - 
         13./540.*m34*RSe564p - 1./540.*m33*RSe45613p + 37./360.*m33*
         RSe45633p - 23./270.*m33*RSe563p + 2./135.*m33*RSe45611p - 17./
         540.*m33*RSe561p + 1./1080.*m33*RSe564p + 1./27.*m14*RSe45613p
          - 1./108.*m14*RSe45633p - 1./54.*m14*RSe563p - 1./27.*m14*
         RSe45611p + 1./27.*m14*RSe561p - 1./108.*m14*RSe564p - 11./540.
         *m13*RSe45613p - 1./135.*m13*RSe45633p + 19./540.*m13*RSe563p
          + 1./20.*m13*RSe45611p - 43./540.*m13*RSe561p + 1./45.*m13*
         RSe564p + 4./135.*m11*RSe45613p + 1./120.*m11*RSe45633p - 1./
         20.*m11*RSe563p );

      M13p6x23RV +=  + AbV(m34,L)*AbV(me2,L) * ( 11./270.*m11*RSe45611p
          - 1./45.*m11*RSe561p - 7./1080.*m11*RSe564p );

      M13p6x23RV +=  + AbV(m34,L)*BbV(m11,m33,L) * (  - 1./108.*m33*m44
         *RS14563pe - 1./108.*m33*m44*RS156pe + 1./54.*m33*m34*
         RS14563pe - 1./54.*m33*m34*RS156pe - 1./36.*pow(m33,2)*
         RS14563pe - 1./108.*pow(m33,2)*RS156pe - 1./54.*m13*m44*
         RS14563pe - 1./54.*m13*m44*RS156pe + 1./27.*m13*m34*RS14563pe
          - 1./27.*m13*m34*RS156pe - 1./18.*m13*m33*RS14563pe - 1./54.*
         m13*m33*RS156pe - 1./36.*m11*m44*RS14563pe - 1./36.*m11*m44*
         RS156pe + 1./18.*m11*m34*RS14563pe - 1./18.*m11*m34*RS156pe - 
         1./36.*m11*m33*RS14563pe - 1./12.*m11*m33*RS156pe + 1./27.*m11
         *m13*RS14563pe - 1./27.*m11*m13*RS156pe + 1./54.*pow(m11,2)*
         RS14563pe - 1./54.*pow(m11,2)*RS156pe );

      M13p6x23RV +=  + AbV(m34,L)*BbV(m11,L) * ( 1./60.*m44*RD1456ep + 
         1./270.*m34*RD1456ep + 2./135.*m33*RD1456ep - 1./27.*m14*
         RD1456ep + 1./20.*m13*RD1456ep + 19./540.*m11*RD1456ep );

      M13p6x23RV +=  + AbV(m34,L)*BbV(m33,mp2,L) * ( 1./27.*m33*mp2*
         RSp45613e - 1./27.*m33*mp2*RSp45633e + 1./27.*m33*mp2*RSp563e
          - 1./27.*m33*mp2*RSp561e - 1./36.*m33*m44*RSp45613e - 1./108.
         *m33*m44*RSp45633e - 1./108.*m33*m44*RSp563e - 1./36.*m33*m44*
         RSp561e + 1./18.*m33*m34*RSp45613e + 1./54.*m33*m34*RSp45633e
          - 1./54.*m33*m34*RSp563e - 1./18.*m33*m34*RSp561e - 7./108.*
         pow(m33,2)*RSp45613e + 1./36.*pow(m33,2)*RSp45633e - 7./108.*
         pow(m33,2)*RSp563e - 5./108.*pow(m33,2)*RSp561e - 1./54.*m13*
         m44*RSp45613e + 1./54.*m13*m44*RSp45633e + 1./54.*m13*m44*
         RSp563e - 1./54.*m13*m44*RSp561e + 1./27.*m13*m34*RSp45613e - 
         1./27.*m13*m34*RSp45633e + 1./27.*m13*m34*RSp563e - 1./27.*m13
         *m34*RSp561e - 1./54.*m13*m33*RSp45613e + 1./54.*m13*m33*
         RSp45633e + 1./18.*m13*m33*RSp563e - 1./18.*m13*m33*RSp561e - 
         1./108.*m11*m44*RSp45613e - 1./108.*m11*m44*RSp45633e - 1./108.
         *m11*m44*RSp563e - 1./108.*m11*m44*RSp561e + 1./54.*m11*m34*
         RSp45613e );

      M13p6x23RV +=  + AbV(m34,L)*BbV(m33,mp2,L) * ( 1./54.*m11*m34*
         RSp45633e - 1./54.*m11*m34*RSp563e - 1./54.*m11*m34*RSp561e - 
         1./108.*m11*m33*RSp45613e - 1./108.*m11*m33*RSp45633e - 1./36.
         *m11*m33*RSp563e - 1./36.*m11*m33*RSp561e );

      M13p6x23RV +=  + AbV(m34,L)*BbV(m33,me2,L) * ( 1./27.*m33*me2*
         RSe45613p - 1./27.*m33*me2*RSe45633p + 1./27.*m33*me2*RSe563p
          - 1./27.*m33*me2*RSe561p - 1./36.*m33*m44*RSe45613p - 1./108.
         *m33*m44*RSe45633p - 1./108.*m33*m44*RSe563p - 1./36.*m33*m44*
         RSe561p + 1./18.*m33*m34*RSe45613p + 1./54.*m33*m34*RSe45633p
          - 1./54.*m33*m34*RSe563p - 1./18.*m33*m34*RSe561p - 7./108.*
         pow(m33,2)*RSe45613p + 1./36.*pow(m33,2)*RSe45633p - 7./108.*
         pow(m33,2)*RSe563p - 5./108.*pow(m33,2)*RSe561p - 1./54.*m13*
         m44*RSe45613p + 1./54.*m13*m44*RSe45633p + 1./54.*m13*m44*
         RSe563p - 1./54.*m13*m44*RSe561p + 1./27.*m13*m34*RSe45613p - 
         1./27.*m13*m34*RSe45633p + 1./27.*m13*m34*RSe563p - 1./27.*m13
         *m34*RSe561p - 1./54.*m13*m33*RSe45613p + 1./54.*m13*m33*
         RSe45633p + 1./18.*m13*m33*RSe563p - 1./18.*m13*m33*RSe561p - 
         1./108.*m11*m44*RSe45613p - 1./108.*m11*m44*RSe45633p - 1./108.
         *m11*m44*RSe563p - 1./108.*m11*m44*RSe561p + 1./54.*m11*m34*
         RSe45613p );

      M13p6x23RV +=  + AbV(m34,L)*BbV(m33,me2,L) * ( 1./54.*m11*m34*
         RSe45633p - 1./54.*m11*m34*RSe563p - 1./54.*m11*m34*RSe561p - 
         1./108.*m11*m33*RSe45613p - 1./108.*m11*m33*RSe45633p - 1./36.
         *m11*m33*RSe563p - 1./36.*m11*m33*RSe561p );

      M13p6x23RV +=  + AbV(m34,L)*BbV(m33,L) * ( 7./540.*m44*RD3456ep
          - 1./60.*m34*RD3456ep + 13./1080.*m33*RD3456ep - 1./36.*m33*
         m44*RS34561pe - 1./108.*m33*m44*RC3456pe - 1./108.*m33*m44*
         RS356pe + 1./18.*m33*m34*RS34561pe + 1./54.*m33*m34*RC3456pe
          - 1./54.*m33*m34*RS356pe - 1./36.*pow(m33,2)*RS34561pe - 1./
         108.*pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS356pe - 1./108.
         *m14*RD3456ep - 1./135.*m13*RD3456ep - 1./54.*m13*m44*
         RS34561pe + 1./54.*m13*m44*RC3456pe + 1./54.*m13*m44*RS356pe
          + 1./27.*m13*m34*RS34561pe - 1./27.*m13*m34*RC3456pe + 1./27.
         *m13*m34*RS356pe - 1./54.*m13*m33*RS34561pe + 1./54.*m13*m33*
         RC3456pe + 1./18.*m13*m33*RS356pe + 1./120.*m11*RD3456ep - 1./
         108.*m11*m44*RS34561pe - 1./108.*m11*m44*RC3456pe - 1./108.*
         m11*m44*RS356pe + 1./54.*m11*m34*RS34561pe + 1./54.*m11*m34*
         RC3456pe - 1./54.*m11*m34*RS356pe - 1./108.*m11*m33*RS34561pe
          - 1./108.*m11*m33*RC3456pe - 1./36.*m11*m33*RS356pe );

      M13p6x23RV +=  + AbV(m34,L)*BbV(m34,L) * ( 1./72.*pow(m44,2) - 1./
         18.*m34*m44 + 1./18.*pow(m34,2) + 1./24.*m33*m44 - 1./12.*m33*
         m34 + 1./36.*pow(m33,2) - 1./36.*m13*m44 + 1./18.*m13*m34 - 1./
         36.*m13*m33 + 1./72.*m11*m44 - 1./36.*m11*m34 + 1./72.*m11*m33
          );

      M13p6x23RV +=  + AbV(m34,L)*BbV(m35,L) * ( 1./144.*pow(m55,2) + 1.
         /144.*m44*m55 - 1./36.*m35*m55 - 1./72.*m35*m44 + 1./36.*pow(
         m35,2) - 1./72.*m34*m55 + 1./36.*m34*m35 + 1./36.*m33*m55 + 1./
         72.*m33*m44 - 1./18.*m33*m35 - 1./36.*m33*m34 + 1./36.*pow(
         m33,2) - 1./72.*m13*m55 - 1./72.*m13*m44 + 1./36.*m13*m35 + 1./
         36.*m13*m34 - 1./36.*m13*m33 + 1./144.*m11*m55 + 1./144.*m11*
         m44 - 1./72.*m11*m35 - 1./72.*m11*m34 + 1./72.*m11*m33 );

      M13p6x23RV +=  + AbV(m34,L)*BbV(m36,L) * ( 1./144.*pow(m66,2) + 1.
         /144.*m44*m66 - 1./36.*m36*m66 - 1./72.*m36*m44 + 1./36.*pow(
         m36,2) - 1./72.*m34*m66 + 1./36.*m34*m36 + 1./36.*m33*m66 + 1./
         72.*m33*m44 - 1./18.*m33*m36 - 1./36.*m33*m34 + 1./36.*pow(
         m33,2) - 1./72.*m13*m66 - 1./72.*m13*m44 + 1./36.*m13*m36 + 1./
         36.*m13*m34 - 1./36.*m13*m33 + 1./144.*m11*m66 + 1./144.*m11*
         m44 - 1./72.*m11*m36 - 1./72.*m11*m34 + 1./72.*m11*m33 );

      M13p6x23RV +=  + AbV(m34,L)*CbV(m33,L) * (  - 1./108.*m33*m44*
         RD3456ep + 1./54.*m33*m34*RD3456ep - 1./108.*pow(m33,2)*
         RD3456ep + 1./54.*m13*m44*RD3456ep - 1./27.*m13*m34*RD3456ep
          + 1./54.*m13*m33*RD3456ep - 1./108.*m11*m44*RD3456ep + 1./54.
         *m11*m34*RD3456ep - 1./108.*m11*m33*RD3456ep );

      M13p6x23RV +=  + AbV(m35,L)*pi16 * (  - 1./24.*m13*RD1456ep - 1./
         24.*m13*me2*RSe45611p + 1./12.*m13*me2*RSe461p - 1./24.*m13*
         me2*RSe465p - 1./24.*m13*mp2*RSp45611e + 1./12.*m13*mp2*
         RSp461e - 1./24.*m13*mp2*RSp465e + 1./8.*m13*m56 + 1./8.*m13*
         m55 - 1./24.*m13*m55*RS546pe + 1./8.*m13*m45 - 1./8.*m13*m35
          + 1./24.*m13*m35*RS34561pe + 1./24.*m13*m35*RS14563pe + 1./24.
         *m13*m35*RSe45613p + 1./24.*m13*m35*RSp45613e - 1./24.*m13*m35
         *RC3456pe - 1./24.*m13*m35*RSe45633p - 1./24.*m13*m35*
         RSp45633e + 1./24.*m13*m35*RS346pe + 1./24.*m13*m35*RSe463p + 
         1./24.*m13*m35*RSp463e - 1./24.*m13*m35*RS146pe - 1./24.*m13*
         m35*RSe461p - 1./24.*m13*m35*RSp461e + 1./8.*m13*m16 + 1./8.*
         m13*m15 - 1./24.*m13*m15*RC1456pe - 1./24.*m13*m15*RSe45611p
          - 1./24.*m13*m15*RSp45611e + 1./12.*m13*m15*RS146pe - 1./24.*
         m13*m15*RS546pe + 1./12.*m13*m15*RSe461p + 1./12.*m13*m15*
         RSp461e - 1./24.*m13*m15*RSe465p - 1./24.*m13*m15*RSp465e + 1./
         8.*m13*m14 );

      M13p6x23RV +=  + AbV(m35,L)*pi16 * (  - 1./8.*pow(m13,2) + 1./72.
         *pow(m13,2)*RC1456pe + 1./72.*pow(m13,2)*RSe45611p + 1./72.*
         pow(m13,2)*RSp45611e - 1./36.*pow(m13,2)*RS146pe + 1./72.*pow(
         m13,2)*RS546pe - 1./36.*pow(m13,2)*RSe461p - 1./36.*pow(m13,2)
         *RSp461e + 1./72.*pow(m13,2)*RSe465p + 1./72.*pow(m13,2)*
         RSp465e - 1./24.*m11*m13*RC1456pe + 1./12.*m11*m13*RS146pe );

      M13p6x23RV +=  + pow(AbV(m35,L),2) * (  - 11./360.*m55 + 11./180.
         *m35 - 7./180.*m33 + 1./60.*m13 - 1./120.*m11 );

      M13p6x23RV +=  + AbV(m35,L)*AbV(m36,L) * (  - 11./360.*m66 - 11./
         360.*m55 + 11./180.*m36 + 11./180.*m35 - 7./90.*m33 + 1./30.*
         m13 - 1./60.*m11 );

      M13p6x23RV +=  + AbV(m35,L)*AbV(m45,L) * (  - 1./360.*m55 - 1./15.
         *m45 + 7./360.*m44 + 13./180.*m35 - 1./360.*m33 + 1./36.*m14
          - 1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + AbV(m35,L)*AbV(m55,L) * (  - 1./20.*m55 + 1./60.
         *m55*RS546pe + 13./180.*m35 - 13./540.*m35*RS546pe - 1./360.*
         m33 + 1./1080.*m33*RS546pe + 1./36.*m15 - 1./108.*m15*RS546pe
          - 1./15.*m13 + 1./45.*m13*RS546pe + 7./360.*m11 - 7./1080.*
         m11*RS546pe );

      M13p6x23RV +=  + AbV(m35,L)*AbV(m56,L) * ( 7./360.*m66 - 1./15.*
         m56 - 1./360.*m55 + 13./180.*m35 - 1./360.*m33 + 1./36.*m16 - 
         1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + AbV(m35,L)*AbV(mp2,L) * ( 11./135.*mp2*RSp45613e
          - 29./540.*mp2*RSp45633e + 7./270.*mp2*RSp463e - 1./180.*mp2*
         RSp45611e - 19./270.*mp2*RSp461e + 1./45.*mp2*RSp465e - 7./540.
         *m55*RSp45613e + 7./540.*m55*RSp45633e - 1./60.*m55*RSp463e + 
         1./60.*m55*RSp45611e + 1./180.*m55*RSp461e - 1./180.*m55*
         RSp465e - 1./90.*m35*RSp45613e - 1./60.*m35*RSp45633e + 2./45.
         *m35*RSp463e + 1./270.*m35*RSp45611e + 1./270.*m35*RSp461e - 
         13./540.*m35*RSp465e - 1./540.*m33*RSp45613e + 37./360.*m33*
         RSp45633e - 23./270.*m33*RSp463e + 2./135.*m33*RSp45611e - 17./
         540.*m33*RSp461e + 1./1080.*m33*RSp465e + 1./27.*m15*RSp45613e
          - 1./108.*m15*RSp45633e - 1./54.*m15*RSp463e - 1./27.*m15*
         RSp45611e + 1./27.*m15*RSp461e - 1./108.*m15*RSp465e - 11./540.
         *m13*RSp45613e - 1./135.*m13*RSp45633e + 19./540.*m13*RSp463e
          + 1./20.*m13*RSp45611e - 43./540.*m13*RSp461e + 1./45.*m13*
         RSp465e + 4./135.*m11*RSp45613e + 1./120.*m11*RSp45633e - 1./
         20.*m11*RSp463e );

      M13p6x23RV +=  + AbV(m35,L)*AbV(mp2,L) * ( 11./270.*m11*RSp45611e
          - 1./45.*m11*RSp461e - 7./1080.*m11*RSp465e );

      M13p6x23RV +=  + AbV(m35,L)*AbV(me2,L) * ( 11./135.*me2*RSe45613p
          - 29./540.*me2*RSe45633p + 7./270.*me2*RSe463p - 1./180.*me2*
         RSe45611p - 19./270.*me2*RSe461p + 1./45.*me2*RSe465p - 7./540.
         *m55*RSe45613p + 7./540.*m55*RSe45633p - 1./60.*m55*RSe463p + 
         1./60.*m55*RSe45611p + 1./180.*m55*RSe461p - 1./180.*m55*
         RSe465p - 1./90.*m35*RSe45613p - 1./60.*m35*RSe45633p + 2./45.
         *m35*RSe463p + 1./270.*m35*RSe45611p + 1./270.*m35*RSe461p - 
         13./540.*m35*RSe465p - 1./540.*m33*RSe45613p + 37./360.*m33*
         RSe45633p - 23./270.*m33*RSe463p + 2./135.*m33*RSe45611p - 17./
         540.*m33*RSe461p + 1./1080.*m33*RSe465p + 1./27.*m15*RSe45613p
          - 1./108.*m15*RSe45633p - 1./54.*m15*RSe463p - 1./27.*m15*
         RSe45611p + 1./27.*m15*RSe461p - 1./108.*m15*RSe465p - 11./540.
         *m13*RSe45613p - 1./135.*m13*RSe45633p + 19./540.*m13*RSe463p
          + 1./20.*m13*RSe45611p - 43./540.*m13*RSe461p + 1./45.*m13*
         RSe465p + 4./135.*m11*RSe45613p + 1./120.*m11*RSe45633p - 1./
         20.*m11*RSe463p );

      M13p6x23RV +=  + AbV(m35,L)*AbV(me2,L) * ( 11./270.*m11*RSe45611p
          - 1./45.*m11*RSe461p - 7./1080.*m11*RSe465p );

      M13p6x23RV +=  + AbV(m35,L)*BbV(m11,m33,L) * (  - 1./108.*m33*m55
         *RS14563pe - 1./108.*m33*m55*RS146pe + 1./54.*m33*m35*
         RS14563pe - 1./54.*m33*m35*RS146pe - 1./36.*pow(m33,2)*
         RS14563pe - 1./108.*pow(m33,2)*RS146pe - 1./54.*m13*m55*
         RS14563pe - 1./54.*m13*m55*RS146pe + 1./27.*m13*m35*RS14563pe
          - 1./27.*m13*m35*RS146pe - 1./18.*m13*m33*RS14563pe - 1./54.*
         m13*m33*RS146pe - 1./36.*m11*m55*RS14563pe - 1./36.*m11*m55*
         RS146pe + 1./18.*m11*m35*RS14563pe - 1./18.*m11*m35*RS146pe - 
         1./36.*m11*m33*RS14563pe - 1./12.*m11*m33*RS146pe + 1./27.*m11
         *m13*RS14563pe - 1./27.*m11*m13*RS146pe + 1./54.*pow(m11,2)*
         RS14563pe - 1./54.*pow(m11,2)*RS146pe );

      M13p6x23RV +=  + AbV(m35,L)*BbV(m11,L) * ( 1./60.*m55*RD1456ep + 
         1./270.*m35*RD1456ep + 2./135.*m33*RD1456ep - 1./27.*m15*
         RD1456ep + 1./20.*m13*RD1456ep + 19./540.*m11*RD1456ep );

      M13p6x23RV +=  + AbV(m35,L)*BbV(m33,mp2,L) * ( 1./27.*m33*mp2*
         RSp45613e - 1./27.*m33*mp2*RSp45633e + 1./27.*m33*mp2*RSp463e
          - 1./27.*m33*mp2*RSp461e - 1./36.*m33*m55*RSp45613e - 1./108.
         *m33*m55*RSp45633e - 1./108.*m33*m55*RSp463e - 1./36.*m33*m55*
         RSp461e + 1./18.*m33*m35*RSp45613e + 1./54.*m33*m35*RSp45633e
          - 1./54.*m33*m35*RSp463e - 1./18.*m33*m35*RSp461e - 7./108.*
         pow(m33,2)*RSp45613e + 1./36.*pow(m33,2)*RSp45633e - 7./108.*
         pow(m33,2)*RSp463e - 5./108.*pow(m33,2)*RSp461e - 1./54.*m13*
         m55*RSp45613e + 1./54.*m13*m55*RSp45633e + 1./54.*m13*m55*
         RSp463e - 1./54.*m13*m55*RSp461e + 1./27.*m13*m35*RSp45613e - 
         1./27.*m13*m35*RSp45633e + 1./27.*m13*m35*RSp463e - 1./27.*m13
         *m35*RSp461e - 1./54.*m13*m33*RSp45613e + 1./54.*m13*m33*
         RSp45633e + 1./18.*m13*m33*RSp463e - 1./18.*m13*m33*RSp461e - 
         1./108.*m11*m55*RSp45613e - 1./108.*m11*m55*RSp45633e - 1./108.
         *m11*m55*RSp463e - 1./108.*m11*m55*RSp461e + 1./54.*m11*m35*
         RSp45613e );

      M13p6x23RV +=  + AbV(m35,L)*BbV(m33,mp2,L) * ( 1./54.*m11*m35*
         RSp45633e - 1./54.*m11*m35*RSp463e - 1./54.*m11*m35*RSp461e - 
         1./108.*m11*m33*RSp45613e - 1./108.*m11*m33*RSp45633e - 1./36.
         *m11*m33*RSp463e - 1./36.*m11*m33*RSp461e );

      M13p6x23RV +=  + AbV(m35,L)*BbV(m33,me2,L) * ( 1./27.*m33*me2*
         RSe45613p - 1./27.*m33*me2*RSe45633p + 1./27.*m33*me2*RSe463p
          - 1./27.*m33*me2*RSe461p - 1./36.*m33*m55*RSe45613p - 1./108.
         *m33*m55*RSe45633p - 1./108.*m33*m55*RSe463p - 1./36.*m33*m55*
         RSe461p + 1./18.*m33*m35*RSe45613p + 1./54.*m33*m35*RSe45633p
          - 1./54.*m33*m35*RSe463p - 1./18.*m33*m35*RSe461p - 7./108.*
         pow(m33,2)*RSe45613p + 1./36.*pow(m33,2)*RSe45633p - 7./108.*
         pow(m33,2)*RSe463p - 5./108.*pow(m33,2)*RSe461p - 1./54.*m13*
         m55*RSe45613p + 1./54.*m13*m55*RSe45633p + 1./54.*m13*m55*
         RSe463p - 1./54.*m13*m55*RSe461p + 1./27.*m13*m35*RSe45613p - 
         1./27.*m13*m35*RSe45633p + 1./27.*m13*m35*RSe463p - 1./27.*m13
         *m35*RSe461p - 1./54.*m13*m33*RSe45613p + 1./54.*m13*m33*
         RSe45633p + 1./18.*m13*m33*RSe463p - 1./18.*m13*m33*RSe461p - 
         1./108.*m11*m55*RSe45613p - 1./108.*m11*m55*RSe45633p - 1./108.
         *m11*m55*RSe463p - 1./108.*m11*m55*RSe461p + 1./54.*m11*m35*
         RSe45613p );

      M13p6x23RV +=  + AbV(m35,L)*BbV(m33,me2,L) * ( 1./54.*m11*m35*
         RSe45633p - 1./54.*m11*m35*RSe463p - 1./54.*m11*m35*RSe461p - 
         1./108.*m11*m33*RSe45613p - 1./108.*m11*m33*RSe45633p - 1./36.
         *m11*m33*RSe463p - 1./36.*m11*m33*RSe461p );

      M13p6x23RV +=  + AbV(m35,L)*BbV(m33,L) * ( 7./540.*m55*RD3456ep
          - 1./60.*m35*RD3456ep + 13./1080.*m33*RD3456ep - 1./36.*m33*
         m55*RS34561pe - 1./108.*m33*m55*RC3456pe - 1./108.*m33*m55*
         RS346pe + 1./18.*m33*m35*RS34561pe + 1./54.*m33*m35*RC3456pe
          - 1./54.*m33*m35*RS346pe - 1./36.*pow(m33,2)*RS34561pe - 1./
         108.*pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS346pe - 1./108.
         *m15*RD3456ep - 1./135.*m13*RD3456ep - 1./54.*m13*m55*
         RS34561pe + 1./54.*m13*m55*RC3456pe + 1./54.*m13*m55*RS346pe
          + 1./27.*m13*m35*RS34561pe - 1./27.*m13*m35*RC3456pe + 1./27.
         *m13*m35*RS346pe - 1./54.*m13*m33*RS34561pe + 1./54.*m13*m33*
         RC3456pe + 1./18.*m13*m33*RS346pe + 1./120.*m11*RD3456ep - 1./
         108.*m11*m55*RS34561pe - 1./108.*m11*m55*RC3456pe - 1./108.*
         m11*m55*RS346pe + 1./54.*m11*m35*RS34561pe + 1./54.*m11*m35*
         RC3456pe - 1./54.*m11*m35*RS346pe - 1./108.*m11*m33*RS34561pe
          - 1./108.*m11*m33*RC3456pe - 1./36.*m11*m33*RS346pe );

      M13p6x23RV +=  + AbV(m35,L)*BbV(m34,L) * ( 1./144.*m44*m55 + 1./
         144.*pow(m44,2) - 1./72.*m35*m44 - 1./72.*m34*m55 - 1./36.*m34
         *m44 + 1./36.*m34*m35 + 1./36.*pow(m34,2) + 1./72.*m33*m55 + 1.
         /36.*m33*m44 - 1./36.*m33*m35 - 1./18.*m33*m34 + 1./36.*pow(
         m33,2) - 1./72.*m13*m55 - 1./72.*m13*m44 + 1./36.*m13*m35 + 1./
         36.*m13*m34 - 1./36.*m13*m33 + 1./144.*m11*m55 + 1./144.*m11*
         m44 - 1./72.*m11*m35 - 1./72.*m11*m34 + 1./72.*m11*m33 );

      M13p6x23RV +=  + AbV(m35,L)*BbV(m35,L) * ( 1./72.*pow(m55,2) - 1./
         18.*m35*m55 + 1./18.*pow(m35,2) + 1./24.*m33*m55 - 1./12.*m33*
         m35 + 1./36.*pow(m33,2) - 1./36.*m13*m55 + 1./18.*m13*m35 - 1./
         36.*m13*m33 + 1./72.*m11*m55 - 1./36.*m11*m35 + 1./72.*m11*m33
          );

      M13p6x23RV +=  + AbV(m35,L)*BbV(m36,L) * ( 1./144.*pow(m66,2) + 1.
         /144.*m55*m66 - 1./36.*m36*m66 - 1./72.*m36*m55 + 1./36.*pow(
         m36,2) - 1./72.*m35*m66 + 1./36.*m35*m36 + 1./36.*m33*m66 + 1./
         72.*m33*m55 - 1./18.*m33*m36 - 1./36.*m33*m35 + 1./36.*pow(
         m33,2) - 1./72.*m13*m66 - 1./72.*m13*m55 + 1./36.*m13*m36 + 1./
         36.*m13*m35 - 1./36.*m13*m33 + 1./144.*m11*m66 + 1./144.*m11*
         m55 - 1./72.*m11*m36 - 1./72.*m11*m35 + 1./72.*m11*m33 );

      M13p6x23RV +=  + AbV(m35,L)*CbV(m33,L) * (  - 1./108.*m33*m55*
         RD3456ep + 1./54.*m33*m35*RD3456ep - 1./108.*pow(m33,2)*
         RD3456ep + 1./54.*m13*m55*RD3456ep - 1./27.*m13*m35*RD3456ep
          + 1./54.*m13*m33*RD3456ep - 1./108.*m11*m55*RD3456ep + 1./54.
         *m11*m35*RD3456ep - 1./108.*m11*m33*RD3456ep );

      M13p6x23RV +=  + AbV(m36,L)*pi16 * (  - 1./24.*m13*RD1456ep - 1./
         24.*m13*me2*RSe45611p + 1./12.*m13*me2*RSe451p - 1./24.*m13*
         me2*RSe456p - 1./24.*m13*mp2*RSp45611e + 1./12.*m13*mp2*
         RSp451e - 1./24.*m13*mp2*RSp456e + 1./8.*m13*m66 - 1./24.*m13*
         m66*RS645pe + 1./8.*m13*m56 + 1./8.*m13*m46 - 1./8.*m13*m36 + 
         1./24.*m13*m36*RS34561pe + 1./24.*m13*m36*RS14563pe + 1./24.*
         m13*m36*RSe45613p + 1./24.*m13*m36*RSp45613e - 1./24.*m13*m36*
         RC3456pe - 1./24.*m13*m36*RSe45633p - 1./24.*m13*m36*RSp45633e
          + 1./24.*m13*m36*RS345pe + 1./24.*m13*m36*RSe453p + 1./24.*
         m13*m36*RSp453e - 1./24.*m13*m36*RS145pe - 1./24.*m13*m36*
         RSe451p - 1./24.*m13*m36*RSp451e + 1./8.*m13*m16 - 1./24.*m13*
         m16*RC1456pe - 1./24.*m13*m16*RSe45611p - 1./24.*m13*m16*
         RSp45611e + 1./12.*m13*m16*RS145pe - 1./24.*m13*m16*RS645pe + 
         1./12.*m13*m16*RSe451p + 1./12.*m13*m16*RSp451e - 1./24.*m13*
         m16*RSe456p - 1./24.*m13*m16*RSp456e + 1./8.*m13*m15 + 1./8.*
         m13*m14 );

      M13p6x23RV +=  + AbV(m36,L)*pi16 * (  - 1./8.*pow(m13,2) + 1./72.
         *pow(m13,2)*RC1456pe + 1./72.*pow(m13,2)*RSe45611p + 1./72.*
         pow(m13,2)*RSp45611e - 1./36.*pow(m13,2)*RS145pe + 1./72.*pow(
         m13,2)*RS645pe - 1./36.*pow(m13,2)*RSe451p - 1./36.*pow(m13,2)
         *RSp451e + 1./72.*pow(m13,2)*RSe456p + 1./72.*pow(m13,2)*
         RSp456e - 1./24.*m11*m13*RC1456pe + 1./12.*m11*m13*RS145pe );

      M13p6x23RV +=  + pow(AbV(m36,L),2) * (  - 11./360.*m66 + 11./180.
         *m36 - 7./180.*m33 + 1./60.*m13 - 1./120.*m11 );

      M13p6x23RV +=  + AbV(m36,L)*AbV(m46,L) * (  - 1./360.*m66 - 1./15.
         *m46 + 7./360.*m44 + 13./180.*m36 - 1./360.*m33 + 1./36.*m14
          - 1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + AbV(m36,L)*AbV(m56,L) * (  - 1./360.*m66 - 1./15.
         *m56 + 7./360.*m55 + 13./180.*m36 - 1./360.*m33 + 1./36.*m15
          - 1./15.*m13 + 7./360.*m11 );

      M13p6x23RV +=  + AbV(m36,L)*AbV(m66,L) * (  - 1./20.*m66 + 1./60.
         *m66*RS645pe + 13./180.*m36 - 13./540.*m36*RS645pe - 1./360.*
         m33 + 1./1080.*m33*RS645pe + 1./36.*m16 - 1./108.*m16*RS645pe
          - 1./15.*m13 + 1./45.*m13*RS645pe + 7./360.*m11 - 7./1080.*
         m11*RS645pe );

      M13p6x23RV +=  + AbV(m36,L)*AbV(mp2,L) * ( 11./135.*mp2*RSp45613e
          - 29./540.*mp2*RSp45633e + 7./270.*mp2*RSp453e - 1./180.*mp2*
         RSp45611e - 19./270.*mp2*RSp451e + 1./45.*mp2*RSp456e - 7./540.
         *m66*RSp45613e + 7./540.*m66*RSp45633e - 1./60.*m66*RSp453e + 
         1./60.*m66*RSp45611e + 1./180.*m66*RSp451e - 1./180.*m66*
         RSp456e - 1./90.*m36*RSp45613e - 1./60.*m36*RSp45633e + 2./45.
         *m36*RSp453e + 1./270.*m36*RSp45611e + 1./270.*m36*RSp451e - 
         13./540.*m36*RSp456e - 1./540.*m33*RSp45613e + 37./360.*m33*
         RSp45633e - 23./270.*m33*RSp453e + 2./135.*m33*RSp45611e - 17./
         540.*m33*RSp451e + 1./1080.*m33*RSp456e + 1./27.*m16*RSp45613e
          - 1./108.*m16*RSp45633e - 1./54.*m16*RSp453e - 1./27.*m16*
         RSp45611e + 1./27.*m16*RSp451e - 1./108.*m16*RSp456e - 11./540.
         *m13*RSp45613e - 1./135.*m13*RSp45633e + 19./540.*m13*RSp453e
          + 1./20.*m13*RSp45611e - 43./540.*m13*RSp451e + 1./45.*m13*
         RSp456e + 4./135.*m11*RSp45613e + 1./120.*m11*RSp45633e - 1./
         20.*m11*RSp453e );

      M13p6x23RV +=  + AbV(m36,L)*AbV(mp2,L) * ( 11./270.*m11*RSp45611e
          - 1./45.*m11*RSp451e - 7./1080.*m11*RSp456e );

      M13p6x23RV +=  + AbV(m36,L)*AbV(me2,L) * ( 11./135.*me2*RSe45613p
          - 29./540.*me2*RSe45633p + 7./270.*me2*RSe453p - 1./180.*me2*
         RSe45611p - 19./270.*me2*RSe451p + 1./45.*me2*RSe456p - 7./540.
         *m66*RSe45613p + 7./540.*m66*RSe45633p - 1./60.*m66*RSe453p + 
         1./60.*m66*RSe45611p + 1./180.*m66*RSe451p - 1./180.*m66*
         RSe456p - 1./90.*m36*RSe45613p - 1./60.*m36*RSe45633p + 2./45.
         *m36*RSe453p + 1./270.*m36*RSe45611p + 1./270.*m36*RSe451p - 
         13./540.*m36*RSe456p - 1./540.*m33*RSe45613p + 37./360.*m33*
         RSe45633p - 23./270.*m33*RSe453p + 2./135.*m33*RSe45611p - 17./
         540.*m33*RSe451p + 1./1080.*m33*RSe456p + 1./27.*m16*RSe45613p
          - 1./108.*m16*RSe45633p - 1./54.*m16*RSe453p - 1./27.*m16*
         RSe45611p + 1./27.*m16*RSe451p - 1./108.*m16*RSe456p - 11./540.
         *m13*RSe45613p - 1./135.*m13*RSe45633p + 19./540.*m13*RSe453p
          + 1./20.*m13*RSe45611p - 43./540.*m13*RSe451p + 1./45.*m13*
         RSe456p + 4./135.*m11*RSe45613p + 1./120.*m11*RSe45633p - 1./
         20.*m11*RSe453p );

      M13p6x23RV +=  + AbV(m36,L)*AbV(me2,L) * ( 11./270.*m11*RSe45611p
          - 1./45.*m11*RSe451p - 7./1080.*m11*RSe456p );

      M13p6x23RV +=  + AbV(m36,L)*BbV(m11,m33,L) * (  - 1./108.*m33*m66
         *RS14563pe - 1./108.*m33*m66*RS145pe + 1./54.*m33*m36*
         RS14563pe - 1./54.*m33*m36*RS145pe - 1./36.*pow(m33,2)*
         RS14563pe - 1./108.*pow(m33,2)*RS145pe - 1./54.*m13*m66*
         RS14563pe - 1./54.*m13*m66*RS145pe + 1./27.*m13*m36*RS14563pe
          - 1./27.*m13*m36*RS145pe - 1./18.*m13*m33*RS14563pe - 1./54.*
         m13*m33*RS145pe - 1./36.*m11*m66*RS14563pe - 1./36.*m11*m66*
         RS145pe + 1./18.*m11*m36*RS14563pe - 1./18.*m11*m36*RS145pe - 
         1./36.*m11*m33*RS14563pe - 1./12.*m11*m33*RS145pe + 1./27.*m11
         *m13*RS14563pe - 1./27.*m11*m13*RS145pe + 1./54.*pow(m11,2)*
         RS14563pe - 1./54.*pow(m11,2)*RS145pe );

      M13p6x23RV +=  + AbV(m36,L)*BbV(m11,L) * ( 1./60.*m66*RD1456ep + 
         1./270.*m36*RD1456ep + 2./135.*m33*RD1456ep - 1./27.*m16*
         RD1456ep + 1./20.*m13*RD1456ep + 19./540.*m11*RD1456ep );

      M13p6x23RV +=  + AbV(m36,L)*BbV(m33,mp2,L) * ( 1./27.*m33*mp2*
         RSp45613e - 1./27.*m33*mp2*RSp45633e + 1./27.*m33*mp2*RSp453e
          - 1./27.*m33*mp2*RSp451e - 1./36.*m33*m66*RSp45613e - 1./108.
         *m33*m66*RSp45633e - 1./108.*m33*m66*RSp453e - 1./36.*m33*m66*
         RSp451e + 1./18.*m33*m36*RSp45613e + 1./54.*m33*m36*RSp45633e
          - 1./54.*m33*m36*RSp453e - 1./18.*m33*m36*RSp451e - 7./108.*
         pow(m33,2)*RSp45613e + 1./36.*pow(m33,2)*RSp45633e - 7./108.*
         pow(m33,2)*RSp453e - 5./108.*pow(m33,2)*RSp451e - 1./54.*m13*
         m66*RSp45613e + 1./54.*m13*m66*RSp45633e + 1./54.*m13*m66*
         RSp453e - 1./54.*m13*m66*RSp451e + 1./27.*m13*m36*RSp45613e - 
         1./27.*m13*m36*RSp45633e + 1./27.*m13*m36*RSp453e - 1./27.*m13
         *m36*RSp451e - 1./54.*m13*m33*RSp45613e + 1./54.*m13*m33*
         RSp45633e + 1./18.*m13*m33*RSp453e - 1./18.*m13*m33*RSp451e - 
         1./108.*m11*m66*RSp45613e - 1./108.*m11*m66*RSp45633e - 1./108.
         *m11*m66*RSp453e - 1./108.*m11*m66*RSp451e + 1./54.*m11*m36*
         RSp45613e );

      M13p6x23RV +=  + AbV(m36,L)*BbV(m33,mp2,L) * ( 1./54.*m11*m36*
         RSp45633e - 1./54.*m11*m36*RSp453e - 1./54.*m11*m36*RSp451e - 
         1./108.*m11*m33*RSp45613e - 1./108.*m11*m33*RSp45633e - 1./36.
         *m11*m33*RSp453e - 1./36.*m11*m33*RSp451e );

      M13p6x23RV +=  + AbV(m36,L)*BbV(m33,me2,L) * ( 1./27.*m33*me2*
         RSe45613p - 1./27.*m33*me2*RSe45633p + 1./27.*m33*me2*RSe453p
          - 1./27.*m33*me2*RSe451p - 1./36.*m33*m66*RSe45613p - 1./108.
         *m33*m66*RSe45633p - 1./108.*m33*m66*RSe453p - 1./36.*m33*m66*
         RSe451p + 1./18.*m33*m36*RSe45613p + 1./54.*m33*m36*RSe45633p
          - 1./54.*m33*m36*RSe453p - 1./18.*m33*m36*RSe451p - 7./108.*
         pow(m33,2)*RSe45613p + 1./36.*pow(m33,2)*RSe45633p - 7./108.*
         pow(m33,2)*RSe453p - 5./108.*pow(m33,2)*RSe451p - 1./54.*m13*
         m66*RSe45613p + 1./54.*m13*m66*RSe45633p + 1./54.*m13*m66*
         RSe453p - 1./54.*m13*m66*RSe451p + 1./27.*m13*m36*RSe45613p - 
         1./27.*m13*m36*RSe45633p + 1./27.*m13*m36*RSe453p - 1./27.*m13
         *m36*RSe451p - 1./54.*m13*m33*RSe45613p + 1./54.*m13*m33*
         RSe45633p + 1./18.*m13*m33*RSe453p - 1./18.*m13*m33*RSe451p - 
         1./108.*m11*m66*RSe45613p - 1./108.*m11*m66*RSe45633p - 1./108.
         *m11*m66*RSe453p - 1./108.*m11*m66*RSe451p + 1./54.*m11*m36*
         RSe45613p );

      M13p6x23RV +=  + AbV(m36,L)*BbV(m33,me2,L) * ( 1./54.*m11*m36*
         RSe45633p - 1./54.*m11*m36*RSe453p - 1./54.*m11*m36*RSe451p - 
         1./108.*m11*m33*RSe45613p - 1./108.*m11*m33*RSe45633p - 1./36.
         *m11*m33*RSe453p - 1./36.*m11*m33*RSe451p );

      M13p6x23RV +=  + AbV(m36,L)*BbV(m33,L) * ( 7./540.*m66*RD3456ep
          - 1./60.*m36*RD3456ep + 13./1080.*m33*RD3456ep - 1./36.*m33*
         m66*RS34561pe - 1./108.*m33*m66*RC3456pe - 1./108.*m33*m66*
         RS345pe + 1./18.*m33*m36*RS34561pe + 1./54.*m33*m36*RC3456pe
          - 1./54.*m33*m36*RS345pe - 1./36.*pow(m33,2)*RS34561pe - 1./
         108.*pow(m33,2)*RC3456pe - 1./36.*pow(m33,2)*RS345pe - 1./108.
         *m16*RD3456ep - 1./135.*m13*RD3456ep - 1./54.*m13*m66*
         RS34561pe + 1./54.*m13*m66*RC3456pe + 1./54.*m13*m66*RS345pe
          + 1./27.*m13*m36*RS34561pe - 1./27.*m13*m36*RC3456pe + 1./27.
         *m13*m36*RS345pe - 1./54.*m13*m33*RS34561pe + 1./54.*m13*m33*
         RC3456pe + 1./18.*m13*m33*RS345pe + 1./120.*m11*RD3456ep - 1./
         108.*m11*m66*RS34561pe - 1./108.*m11*m66*RC3456pe - 1./108.*
         m11*m66*RS345pe + 1./54.*m11*m36*RS34561pe + 1./54.*m11*m36*
         RC3456pe - 1./54.*m11*m36*RS345pe - 1./108.*m11*m33*RS34561pe
          - 1./108.*m11*m33*RC3456pe - 1./36.*m11*m33*RS345pe );

      M13p6x23RV +=  + AbV(m36,L)*BbV(m34,L) * ( 1./144.*m44*m66 + 1./
         144.*pow(m44,2) - 1./72.*m36*m44 - 1./72.*m34*m66 - 1./36.*m34
         *m44 + 1./36.*m34*m36 + 1./36.*pow(m34,2) + 1./72.*m33*m66 + 1.
         /36.*m33*m44 - 1./36.*m33*m36 - 1./18.*m33*m34 + 1./36.*pow(
         m33,2) - 1./72.*m13*m66 - 1./72.*m13*m44 + 1./36.*m13*m36 + 1./
         36.*m13*m34 - 1./36.*m13*m33 + 1./144.*m11*m66 + 1./144.*m11*
         m44 - 1./72.*m11*m36 - 1./72.*m11*m34 + 1./72.*m11*m33 );

      M13p6x23RV +=  + AbV(m36,L)*BbV(m35,L) * ( 1./144.*m55*m66 + 1./
         144.*pow(m55,2) - 1./72.*m36*m55 - 1./72.*m35*m66 - 1./36.*m35
         *m55 + 1./36.*m35*m36 + 1./36.*pow(m35,2) + 1./72.*m33*m66 + 1.
         /36.*m33*m55 - 1./36.*m33*m36 - 1./18.*m33*m35 + 1./36.*pow(
         m33,2) - 1./72.*m13*m66 - 1./72.*m13*m55 + 1./36.*m13*m36 + 1./
         36.*m13*m35 - 1./36.*m13*m33 + 1./144.*m11*m66 + 1./144.*m11*
         m55 - 1./72.*m11*m36 - 1./72.*m11*m35 + 1./72.*m11*m33 );

      M13p6x23RV +=  + AbV(m36,L)*BbV(m36,L) * ( 1./72.*pow(m66,2) - 1./
         18.*m36*m66 + 1./18.*pow(m36,2) + 1./24.*m33*m66 - 1./12.*m33*
         m36 + 1./36.*pow(m33,2) - 1./36.*m13*m66 + 1./18.*m13*m36 - 1./
         36.*m13*m33 + 1./72.*m11*m66 - 1./36.*m11*m36 + 1./72.*m11*m33
          );

      M13p6x23RV +=  + AbV(m36,L)*CbV(m33,L) * (  - 1./108.*m33*m66*
         RD3456ep + 1./54.*m33*m36*RD3456ep - 1./108.*pow(m33,2)*
         RD3456ep + 1./54.*m13*m66*RD3456ep - 1./27.*m13*m36*RD3456ep
          + 1./54.*m13*m33*RD3456ep - 1./108.*m11*m66*RD3456ep + 1./54.
         *m11*m36*RD3456ep - 1./108.*m11*m33*RD3456ep );

      M13p6x23RV +=  + AbV(m44,L)*AbV(mp2,L) * (  - 1./27.*m44*RSp563e*
         RS356pe - 1./27.*m44*RSp563e*RSe563p - 1./54.*m44*pow(
         RSp563e,2) + 1./27.*m44*RS156pe*RSp563e + 1./81.*m44*RS456pe*
         RSp563e*RS356pe + 1./81.*m44*RS456pe*RSp563e*RSe563p + 1./162.
         *m44*RS456pe*pow(RSp563e,2) - 1./81.*m44*RS456pe*RS156pe*
         RSp563e + 1./27.*m44*RSe561p*RSp563e - 1./81.*m44*RSe561p*
         RS456pe*RSp563e + 1./27.*m44*RSp561e*RS356pe + 1./27.*m44*
         RSp561e*RSe563p + 1./27.*m44*RSp561e*RSp563e - 1./27.*m44*
         RSp561e*RS156pe - 1./81.*m44*RSp561e*RS456pe*RS356pe - 1./81.*
         m44*RSp561e*RS456pe*RSe563p - 1./81.*m44*RSp561e*RS456pe*
         RSp563e + 1./81.*m44*RSp561e*RS456pe*RS156pe - 1./27.*m44*
         RSp561e*RSe561p + 1./81.*m44*RSp561e*RSe561p*RS456pe - 1./54.*
         m44*pow(RSp561e,2) + 1./162.*m44*pow(RSp561e,2)*RS456pe );

      M13p6x23RV +=  + AbV(m44,L)*AbV(me2,L) * (  - 1./27.*m44*RSe563p*
         RS356pe - 1./54.*m44*pow(RSe563p,2) + 1./27.*m44*RS156pe*
         RSe563p + 1./81.*m44*RS456pe*RSe563p*RS356pe + 1./162.*m44*
         RS456pe*pow(RSe563p,2) - 1./81.*m44*RS456pe*RS156pe*RSe563p + 
         1./27.*m44*RSe561p*RS356pe + 1./27.*m44*RSe561p*RSe563p - 1./
         27.*m44*RSe561p*RS156pe - 1./81.*m44*RSe561p*RS456pe*RS356pe
          - 1./81.*m44*RSe561p*RS456pe*RSe563p + 1./81.*m44*RSe561p*
         RS456pe*RS156pe - 1./54.*m44*pow(RSe561p,2) + 1./162.*m44*pow(
         RSe561p,2)*RS456pe );

      M13p6x23RV +=  + AbV(m44,L)*BbV(m11,m33,L) * ( 1./54.*m33*m44*
         RS156pe*RS356pe - 1./162.*m33*m44*RS456pe*RS156pe*RS356pe + 1./
         27.*m13*m44*RS156pe*RS356pe - 1./81.*m13*m44*RS456pe*RS156pe*
         RS356pe + 1./18.*m11*m44*RS156pe*RS356pe - 1./54.*m11*m44*
         RS456pe*RS156pe*RS356pe );

      M13p6x23RV +=  + AbV(m44,L)*BbV(m11,mp2,L) * ( 1./54.*m33*m44*
         RS156pe*RSp563e - 1./162.*m33*m44*RS456pe*RS156pe*RSp563e + 1./
         54.*m33*m44*RSp561e*RS156pe - 1./162.*m33*m44*RSp561e*RS456pe*
         RS156pe + 1./27.*m13*m44*RS156pe*RSp563e - 1./81.*m13*m44*
         RS456pe*RS156pe*RSp563e - 1./27.*m13*m44*RSp561e*RS156pe + 1./
         81.*m13*m44*RSp561e*RS456pe*RS156pe + 1./18.*m11*m44*RS156pe*
         RSp563e - 1./54.*m11*m44*RS456pe*RS156pe*RSp563e + 1./54.*m11*
         m44*RSp561e*RS156pe - 1./162.*m11*m44*RSp561e*RS456pe*RS156pe
          );

      M13p6x23RV +=  + AbV(m44,L)*BbV(m11,me2,L) * ( 1./54.*m33*m44*
         RS156pe*RSe563p - 1./162.*m33*m44*RS456pe*RS156pe*RSe563p + 1./
         54.*m33*m44*RSe561p*RS156pe - 1./162.*m33*m44*RSe561p*RS456pe*
         RS156pe + 1./27.*m13*m44*RS156pe*RSe563p - 1./81.*m13*m44*
         RS456pe*RS156pe*RSe563p - 1./27.*m13*m44*RSe561p*RS156pe + 1./
         81.*m13*m44*RSe561p*RS456pe*RS156pe + 1./18.*m11*m44*RS156pe*
         RSe563p - 1./54.*m11*m44*RS456pe*RS156pe*RSe563p + 1./54.*m11*
         m44*RSe561p*RS156pe - 1./162.*m11*m44*RSe561p*RS456pe*RS156pe
          );

      M13p6x23RV +=  + AbV(m44,L)*BbV(m11,L) * ( 1./108.*m33*m44*pow(
         RS156pe,2) - 1./324.*m33*m44*RS456pe*pow(RS156pe,2) - 1./54.*
         m13*m44*pow(RS156pe,2) + 1./162.*m13*m44*RS456pe*pow(
         RS156pe,2) + 1./108.*m11*m44*pow(RS156pe,2) - 1./324.*m11*m44*
         RS456pe*pow(RS156pe,2) );

      M13p6x23RV +=  + AbV(m44,L)*BbV(m14,L) * ( 1./144.*pow(m44,2) - 1.
         /432.*pow(m44,2)*RS456pe + 1./144.*m33*m44 - 1./432.*m33*m44*
         RS456pe - 1./36.*m14*m44 + 1./108.*m14*m44*RS456pe - 1./72.*
         m14*m33 + 1./216.*m14*m33*RS456pe + 1./36.*pow(m14,2) - 1./108.
         *pow(m14,2)*RS456pe - 1./72.*m13*m44 + 1./216.*m13*m44*RS456pe
          + 1./36.*m13*m14 - 1./108.*m13*m14*RS456pe + 1./48.*m11*m44
          - 1./144.*m11*m44*RS456pe + 1./144.*m11*m33 - 1./432.*m11*m33
         *RS456pe - 1./24.*m11*m14 + 1./72.*m11*m14*RS456pe - 1./72.*
         m11*m13 + 1./216.*m11*m13*RS456pe + 1./72.*pow(m11,2) - 1./216.
         *pow(m11,2)*RS456pe );

      M13p6x23RV +=  + AbV(m44,L)*BbV(m33,mp2,L) * ( 1./54.*m33*m44*
         RSp563e*RS356pe - 1./162.*m33*m44*RS456pe*RSp563e*RS356pe + 1./
         18.*m33*m44*RSp561e*RS356pe - 1./54.*m33*m44*RSp561e*RS456pe*
         RS356pe - 1./27.*m13*m44*RSp563e*RS356pe + 1./81.*m13*m44*
         RS456pe*RSp563e*RS356pe + 1./27.*m13*m44*RSp561e*RS356pe - 1./
         81.*m13*m44*RSp561e*RS456pe*RS356pe + 1./54.*m11*m44*RSp563e*
         RS356pe - 1./162.*m11*m44*RS456pe*RSp563e*RS356pe + 1./54.*m11
         *m44*RSp561e*RS356pe - 1./162.*m11*m44*RSp561e*RS456pe*RS356pe
          );

      M13p6x23RV +=  + AbV(m44,L)*BbV(m33,me2,L) * ( 1./54.*m33*m44*
         RSe563p*RS356pe - 1./162.*m33*m44*RS456pe*RSe563p*RS356pe + 1./
         18.*m33*m44*RSe561p*RS356pe - 1./54.*m33*m44*RSe561p*RS456pe*
         RS356pe - 1./27.*m13*m44*RSe563p*RS356pe + 1./81.*m13*m44*
         RS456pe*RSe563p*RS356pe + 1./27.*m13*m44*RSe561p*RS356pe - 1./
         81.*m13*m44*RSe561p*RS456pe*RS356pe + 1./54.*m11*m44*RSe563p*
         RS356pe - 1./162.*m11*m44*RS456pe*RSe563p*RS356pe + 1./54.*m11
         *m44*RSe561p*RS356pe - 1./162.*m11*m44*RSe561p*RS456pe*RS356pe
          );

      M13p6x23RV +=  + AbV(m44,L)*BbV(m33,L) * ( 1./108.*m33*m44*pow(
         RS356pe,2) - 1./324.*m33*m44*RS456pe*pow(RS356pe,2) - 1./54.*
         m13*m44*pow(RS356pe,2) + 1./162.*m13*m44*RS456pe*pow(
         RS356pe,2) + 1./108.*m11*m44*pow(RS356pe,2) - 1./324.*m11*m44*
         RS456pe*pow(RS356pe,2) );

      M13p6x23RV +=  + AbV(m44,L)*BbV(m34,L) * ( 1./144.*pow(m44,2) - 1.
         /432.*pow(m44,2)*RS456pe - 1./36.*m34*m44 + 1./108.*m34*m44*
         RS456pe + 1./36.*pow(m34,2) - 1./108.*pow(m34,2)*RS456pe + 1./
         48.*m33*m44 - 1./144.*m33*m44*RS456pe - 1./24.*m33*m34 + 1./72.
         *m33*m34*RS456pe + 1./72.*pow(m33,2) - 1./216.*pow(m33,2)*
         RS456pe - 1./72.*m13*m44 + 1./216.*m13*m44*RS456pe + 1./36.*
         m13*m34 - 1./108.*m13*m34*RS456pe - 1./72.*m13*m33 + 1./216.*
         m13*m33*RS456pe + 1./144.*m11*m44 - 1./432.*m11*m44*RS456pe - 
         1./72.*m11*m34 + 1./216.*m11*m34*RS456pe + 1./144.*m11*m33 - 1.
         /432.*m11*m33*RS456pe );

      M13p6x23RV +=  + AbV(m44,L)*BbV(mp2,L) * (  - 1./54.*m44*mp2*pow(
         RSp563e,2) + 1./162.*m44*mp2*RS456pe*pow(RSp563e,2) + 1./27.*
         m44*mp2*RSp561e*RSp563e - 1./81.*m44*mp2*RSp561e*RS456pe*
         RSp563e - 1./54.*m44*mp2*pow(RSp561e,2) + 1./162.*m44*mp2*pow(
         RSp561e,2)*RS456pe + 1./36.*m33*m44*pow(RSp563e,2) - 1./108.*
         m33*m44*RS456pe*pow(RSp563e,2) + 1./54.*m33*m44*RSp561e*
         RSp563e - 1./162.*m33*m44*RSp561e*RS456pe*RSp563e + 1./108.*
         m33*m44*pow(RSp561e,2) - 1./324.*m33*m44*pow(RSp561e,2)*
         RS456pe - 1./54.*m13*m44*pow(RSp563e,2) + 1./162.*m13*m44*
         RS456pe*pow(RSp563e,2) + 1./27.*m13*m44*RSp561e*RSp563e - 1./
         81.*m13*m44*RSp561e*RS456pe*RSp563e - 1./54.*m13*m44*pow(
         RSp561e,2) + 1./162.*m13*m44*pow(RSp561e,2)*RS456pe + 1./108.*
         m11*m44*pow(RSp563e,2) - 1./324.*m11*m44*RS456pe*pow(
         RSp563e,2) + 1./54.*m11*m44*RSp561e*RSp563e - 1./162.*m11*m44*
         RSp561e*RS456pe*RSp563e + 1./36.*m11*m44*pow(RSp561e,2) - 1./
         108.*m11*m44*pow(RSp561e,2)*RS456pe );

      M13p6x23RV +=  + AbV(m44,L)*BbV(me2,mp2,L) * (  - 1./27.*m44*me2*
         RSp563e*RSe563p + 1./81.*m44*me2*RS456pe*RSp563e*RSe563p + 1./
         27.*m44*me2*RSe561p*RSp563e - 1./81.*m44*me2*RSe561p*RS456pe*
         RSp563e + 1./27.*m44*me2*RSp561e*RSe563p - 1./81.*m44*me2*
         RSp561e*RS456pe*RSe563p - 1./27.*m44*me2*RSp561e*RSe561p + 1./
         81.*m44*me2*RSp561e*RSe561p*RS456pe + 1./18.*m33*m44*RSp563e*
         RSe563p - 1./54.*m33*m44*RS456pe*RSp563e*RSe563p + 1./54.*m33*
         m44*RSe561p*RSp563e - 1./162.*m33*m44*RSe561p*RS456pe*RSp563e
          + 1./54.*m33*m44*RSp561e*RSe563p - 1./162.*m33*m44*RSp561e*
         RS456pe*RSe563p + 1./54.*m33*m44*RSp561e*RSe561p - 1./162.*m33
         *m44*RSp561e*RSe561p*RS456pe - 1./27.*m13*m44*RSp563e*RSe563p
          + 1./81.*m13*m44*RS456pe*RSp563e*RSe563p + 1./27.*m13*m44*
         RSe561p*RSp563e - 1./81.*m13*m44*RSe561p*RS456pe*RSp563e + 1./
         27.*m13*m44*RSp561e*RSe563p - 1./81.*m13*m44*RSp561e*RS456pe*
         RSe563p - 1./27.*m13*m44*RSp561e*RSe561p + 1./81.*m13*m44*
         RSp561e*RSe561p*RS456pe );

      M13p6x23RV +=  + AbV(m44,L)*BbV(me2,mp2,L) * ( 1./54.*m11*m44*
         RSp563e*RSe563p - 1./162.*m11*m44*RS456pe*RSp563e*RSe563p + 1./
         54.*m11*m44*RSe561p*RSp563e - 1./162.*m11*m44*RSe561p*RS456pe*
         RSp563e + 1./54.*m11*m44*RSp561e*RSe563p - 1./162.*m11*m44*
         RSp561e*RS456pe*RSe563p + 1./18.*m11*m44*RSp561e*RSe561p - 1./
         54.*m11*m44*RSp561e*RSe561p*RS456pe );

      M13p6x23RV +=  + AbV(m44,L)*BbV(me2,L) * (  - 1./54.*m44*me2*pow(
         RSe563p,2) + 1./162.*m44*me2*RS456pe*pow(RSe563p,2) + 1./27.*
         m44*me2*RSe561p*RSe563p - 1./81.*m44*me2*RSe561p*RS456pe*
         RSe563p - 1./54.*m44*me2*pow(RSe561p,2) + 1./162.*m44*me2*pow(
         RSe561p,2)*RS456pe + 1./36.*m33*m44*pow(RSe563p,2) - 1./108.*
         m33*m44*RS456pe*pow(RSe563p,2) + 1./54.*m33*m44*RSe561p*
         RSe563p - 1./162.*m33*m44*RSe561p*RS456pe*RSe563p + 1./108.*
         m33*m44*pow(RSe561p,2) - 1./324.*m33*m44*pow(RSe561p,2)*
         RS456pe - 1./54.*m13*m44*pow(RSe563p,2) + 1./162.*m13*m44*
         RS456pe*pow(RSe563p,2) + 1./27.*m13*m44*RSe561p*RSe563p - 1./
         81.*m13*m44*RSe561p*RS456pe*RSe563p - 1./54.*m13*m44*pow(
         RSe561p,2) + 1./162.*m13*m44*pow(RSe561p,2)*RS456pe + 1./108.*
         m11*m44*pow(RSe563p,2) - 1./324.*m11*m44*RS456pe*pow(
         RSe563p,2) + 1./54.*m11*m44*RSe561p*RSe563p - 1./162.*m11*m44*
         RSe561p*RS456pe*RSe563p + 1./36.*m11*m44*pow(RSe561p,2) - 1./
         108.*m11*m44*pow(RSe561p,2)*RS456pe );

      M13p6x23RV +=  + AbV(m45,L)*AbV(mp2,L) * (  - 1./81.*mp2*RSp563e*
         RS346pe + 1./81.*mp2*RSp563e*RS356pe + 1./81.*mp2*RSp563e*
         RSe563p - 1./81.*mp2*RSp563e*RSe463p + 1./81.*mp2*pow(
         RSp563e,2) + 1./81.*mp2*RSp463e*RS346pe - 1./81.*mp2*RSp463e*
         RS356pe - 1./81.*mp2*RSp463e*RSe563p + 1./81.*mp2*RSp463e*
         RSe463p - 2./81.*mp2*RSp463e*RSp563e + 1./81.*mp2*pow(
         RSp463e,2) + 1./81.*mp2*RS146pe*RSp563e - 1./81.*mp2*RS146pe*
         RSp463e - 1./81.*mp2*RS156pe*RSp563e + 1./81.*mp2*RS156pe*
         RSp463e - 1./81.*mp2*RSe561p*RSp563e + 1./81.*mp2*RSe561p*
         RSp463e + 1./81.*mp2*RSe461p*RSp563e - 1./81.*mp2*RSe461p*
         RSp463e + 1./81.*mp2*RSp561e*RS346pe - 1./81.*mp2*RSp561e*
         RS356pe - 1./81.*mp2*RSp561e*RSe563p + 1./81.*mp2*RSp561e*
         RSe463p - 2./81.*mp2*RSp561e*RSp563e + 2./81.*mp2*RSp561e*
         RSp463e - 1./81.*mp2*RSp561e*RS146pe + 1./81.*mp2*RSp561e*
         RS156pe + 1./81.*mp2*RSp561e*RSe561p - 1./81.*mp2*RSp561e*
         RSe461p );

      M13p6x23RV +=  + AbV(m45,L)*AbV(mp2,L) * ( 1./81.*mp2*pow(
         RSp561e,2) - 1./81.*mp2*RSp461e*RS346pe + 1./81.*mp2*RSp461e*
         RS356pe + 1./81.*mp2*RSp461e*RSe563p - 1./81.*mp2*RSp461e*
         RSe463p + 2./81.*mp2*RSp461e*RSp563e - 2./81.*mp2*RSp461e*
         RSp463e + 1./81.*mp2*RSp461e*RS146pe - 1./81.*mp2*RSp461e*
         RS156pe - 1./81.*mp2*RSp461e*RSe561p + 1./81.*mp2*RSp461e*
         RSe461p - 2./81.*mp2*RSp461e*RSp561e + 1./81.*mp2*pow(
         RSp461e,2) - 1./162.*m55*RSp563e*RS346pe - 1./162.*m55*RSp563e
         *RS356pe - 1./162.*m55*RSp563e*RSe563p - 1./162.*m55*RSp563e*
         RSe463p - 1./324.*m55*pow(RSp563e,2) - 1./54.*m55*RSp463e*
         RS346pe - 1./162.*m55*RSp463e*RS356pe - 1./162.*m55*RSp463e*
         RSe563p - 1./54.*m55*RSp463e*RSe463p - 1./162.*m55*RSp463e*
         RSp563e - 1./108.*m55*pow(RSp463e,2) + 1./162.*m55*RS146pe*
         RSp563e + 1./54.*m55*RS146pe*RSp463e + 1./162.*m55*RS156pe*
         RSp563e + 1./162.*m55*RS156pe*RSp463e + 1./162.*m55*RSe561p*
         RSp563e );

      M13p6x23RV +=  + AbV(m45,L)*AbV(mp2,L) * ( 1./162.*m55*RSe561p*
         RSp463e + 1./162.*m55*RSe461p*RSp563e + 1./54.*m55*RSe461p*
         RSp463e + 1./162.*m55*RSp561e*RS346pe + 1./162.*m55*RSp561e*
         RS356pe + 1./162.*m55*RSp561e*RSe563p + 1./162.*m55*RSp561e*
         RSe463p + 1./162.*m55*RSp561e*RSp563e + 1./162.*m55*RSp561e*
         RSp463e - 1./162.*m55*RSp561e*RS146pe - 1./162.*m55*RSp561e*
         RS156pe - 1./162.*m55*RSp561e*RSe561p - 1./162.*m55*RSp561e*
         RSe461p - 1./324.*m55*pow(RSp561e,2) + 1./54.*m55*RSp461e*
         RS346pe + 1./162.*m55*RSp461e*RS356pe + 1./162.*m55*RSp461e*
         RSe563p + 1./54.*m55*RSp461e*RSe463p + 1./162.*m55*RSp461e*
         RSp563e + 1./54.*m55*RSp461e*RSp463e - 1./54.*m55*RSp461e*
         RS146pe - 1./162.*m55*RSp461e*RS156pe - 1./162.*m55*RSp461e*
         RSe561p - 1./54.*m55*RSp461e*RSe461p - 1./162.*m55*RSp461e*
         RSp561e - 1./108.*m55*pow(RSp461e,2) - 1./81.*m45*RSp563e*
         RS346pe + 1./81.*m45*RSp563e*RS356pe + 1./81.*m45*RSp563e*
         RSe563p );

      M13p6x23RV +=  + AbV(m45,L)*AbV(mp2,L) * (  - 1./81.*m45*RSp563e*
         RSe463p + 1./162.*m45*pow(RSp563e,2) + 1./81.*m45*RSp463e*
         RS346pe - 1./81.*m45*RSp463e*RS356pe - 1./81.*m45*RSp463e*
         RSe563p + 1./81.*m45*RSp463e*RSe463p - 1./81.*m45*RSp463e*
         RSp563e + 1./162.*m45*pow(RSp463e,2) + 1./81.*m45*RS146pe*
         RSp563e - 1./81.*m45*RS146pe*RSp463e - 1./81.*m45*RS156pe*
         RSp563e + 1./81.*m45*RS156pe*RSp463e - 1./81.*m45*RSe561p*
         RSp563e + 1./81.*m45*RSe561p*RSp463e + 1./81.*m45*RSe461p*
         RSp563e - 1./81.*m45*RSe461p*RSp463e + 1./81.*m45*RSp561e*
         RS346pe - 1./81.*m45*RSp561e*RS356pe - 1./81.*m45*RSp561e*
         RSe563p + 1./81.*m45*RSp561e*RSe463p - 1./81.*m45*RSp561e*
         RSp563e + 1./81.*m45*RSp561e*RSp463e - 1./81.*m45*RSp561e*
         RS146pe + 1./81.*m45*RSp561e*RS156pe + 1./81.*m45*RSp561e*
         RSe561p - 1./81.*m45*RSp561e*RSe461p + 1./162.*m45*pow(
         RSp561e,2) - 1./81.*m45*RSp461e*RS346pe + 1./81.*m45*RSp461e*
         RS356pe );

      M13p6x23RV +=  + AbV(m45,L)*AbV(mp2,L) * ( 1./81.*m45*RSp461e*
         RSe563p - 1./81.*m45*RSp461e*RSe463p + 1./81.*m45*RSp461e*
         RSp563e - 1./81.*m45*RSp461e*RSp463e + 1./81.*m45*RSp461e*
         RS146pe - 1./81.*m45*RSp461e*RS156pe - 1./81.*m45*RSp461e*
         RSe561p + 1./81.*m45*RSp461e*RSe461p - 1./81.*m45*RSp461e*
         RSp561e + 1./162.*m45*pow(RSp461e,2) - 1./162.*m44*RSp563e*
         RS346pe - 1./54.*m44*RSp563e*RS356pe - 1./54.*m44*RSp563e*
         RSe563p - 1./162.*m44*RSp563e*RSe463p - 1./108.*m44*pow(
         RSp563e,2) - 1./162.*m44*RSp463e*RS346pe - 1./162.*m44*RSp463e
         *RS356pe - 1./162.*m44*RSp463e*RSe563p - 1./162.*m44*RSp463e*
         RSe463p - 1./162.*m44*RSp463e*RSp563e - 1./324.*m44*pow(
         RSp463e,2) + 1./162.*m44*RS146pe*RSp563e + 1./162.*m44*RS146pe
         *RSp463e + 1./54.*m44*RS156pe*RSp563e + 1./162.*m44*RS156pe*
         RSp463e + 1./54.*m44*RSe561p*RSp563e + 1./162.*m44*RSe561p*
         RSp463e + 1./162.*m44*RSe461p*RSp563e + 1./162.*m44*RSe461p*
         RSp463e );

      M13p6x23RV +=  + AbV(m45,L)*AbV(mp2,L) * ( 1./162.*m44*RSp561e*
         RS346pe + 1./54.*m44*RSp561e*RS356pe + 1./54.*m44*RSp561e*
         RSe563p + 1./162.*m44*RSp561e*RSe463p + 1./54.*m44*RSp561e*
         RSp563e + 1./162.*m44*RSp561e*RSp463e - 1./162.*m44*RSp561e*
         RS146pe - 1./54.*m44*RSp561e*RS156pe - 1./54.*m44*RSp561e*
         RSe561p - 1./162.*m44*RSp561e*RSe461p - 1./108.*m44*pow(
         RSp561e,2) + 1./162.*m44*RSp461e*RS346pe + 1./162.*m44*RSp461e
         *RS356pe + 1./162.*m44*RSp461e*RSe563p + 1./162.*m44*RSp461e*
         RSe463p + 1./162.*m44*RSp461e*RSp563e + 1./162.*m44*RSp461e*
         RSp463e - 1./162.*m44*RSp461e*RS146pe - 1./162.*m44*RSp461e*
         RS156pe - 1./162.*m44*RSp461e*RSe561p - 1./162.*m44*RSp461e*
         RSe461p - 1./162.*m44*RSp461e*RSp561e - 1./324.*m44*pow(
         RSp461e,2) + 1./54.*m33*RSp563e*RS346pe - 1./54.*m33*RSp563e*
         RS356pe - 1./54.*m33*RSp563e*RSe563p + 1./54.*m33*RSp563e*
         RSe463p - 1./108.*m33*pow(RSp563e,2) - 1./54.*m33*RSp463e*
         RS346pe );

      M13p6x23RV +=  + AbV(m45,L)*AbV(mp2,L) * ( 1./54.*m33*RSp463e*
         RS356pe + 1./54.*m33*RSp463e*RSe563p - 1./54.*m33*RSp463e*
         RSe463p + 1./54.*m33*RSp463e*RSp563e - 1./108.*m33*pow(
         RSp463e,2) + 1./162.*m33*RS146pe*RSp563e - 1./162.*m33*RS146pe
         *RSp463e - 1./162.*m33*RS156pe*RSp563e + 1./162.*m33*RS156pe*
         RSp463e - 1./162.*m33*RSe561p*RSp563e + 1./162.*m33*RSe561p*
         RSp463e + 1./162.*m33*RSe461p*RSp563e - 1./162.*m33*RSe461p*
         RSp463e + 1./162.*m33*RSp561e*RS346pe - 1./162.*m33*RSp561e*
         RS356pe - 1./162.*m33*RSp561e*RSe563p + 1./162.*m33*RSp561e*
         RSe463p - 1./162.*m33*RSp561e*RSp563e + 1./162.*m33*RSp561e*
         RSp463e + 1./162.*m33*RSp561e*RS146pe - 1./162.*m33*RSp561e*
         RS156pe - 1./162.*m33*RSp561e*RSe561p + 1./162.*m33*RSp561e*
         RSe461p - 1./324.*m33*pow(RSp561e,2) - 1./162.*m33*RSp461e*
         RS346pe + 1./162.*m33*RSp461e*RS356pe + 1./162.*m33*RSp461e*
         RSe563p - 1./162.*m33*RSp461e*RSe463p + 1./162.*m33*RSp461e*
         RSp563e );

      M13p6x23RV +=  + AbV(m45,L)*AbV(mp2,L) * (  - 1./162.*m33*RSp461e
         *RSp463e - 1./162.*m33*RSp461e*RS146pe + 1./162.*m33*RSp461e*
         RS156pe + 1./162.*m33*RSp461e*RSe561p - 1./162.*m33*RSp461e*
         RSe461p + 1./162.*m33*RSp461e*RSp561e - 1./324.*m33*pow(
         RSp461e,2) - 1./81.*m13*RSp563e*RS346pe + 1./81.*m13*RSp563e*
         RS356pe + 1./81.*m13*RSp563e*RSe563p - 1./81.*m13*RSp563e*
         RSe463p + 1./162.*m13*pow(RSp563e,2) + 1./81.*m13*RSp463e*
         RS346pe - 1./81.*m13*RSp463e*RS356pe - 1./81.*m13*RSp463e*
         RSe563p + 1./81.*m13*RSp463e*RSe463p - 1./81.*m13*RSp463e*
         RSp563e + 1./162.*m13*pow(RSp463e,2) + 1./81.*m13*RS146pe*
         RSp563e - 1./81.*m13*RS146pe*RSp463e - 1./81.*m13*RS156pe*
         RSp563e + 1./81.*m13*RS156pe*RSp463e - 1./81.*m13*RSe561p*
         RSp563e + 1./81.*m13*RSe561p*RSp463e + 1./81.*m13*RSe461p*
         RSp563e - 1./81.*m13*RSe461p*RSp463e + 1./81.*m13*RSp561e*
         RS346pe - 1./81.*m13*RSp561e*RS356pe - 1./81.*m13*RSp561e*
         RSe563p );

      M13p6x23RV +=  + AbV(m45,L)*AbV(mp2,L) * ( 1./81.*m13*RSp561e*
         RSe463p - 1./81.*m13*RSp561e*RSp563e + 1./81.*m13*RSp561e*
         RSp463e - 1./81.*m13*RSp561e*RS146pe + 1./81.*m13*RSp561e*
         RS156pe + 1./81.*m13*RSp561e*RSe561p - 1./81.*m13*RSp561e*
         RSe461p + 1./162.*m13*pow(RSp561e,2) - 1./81.*m13*RSp461e*
         RS346pe + 1./81.*m13*RSp461e*RS356pe + 1./81.*m13*RSp461e*
         RSe563p - 1./81.*m13*RSp461e*RSe463p + 1./81.*m13*RSp461e*
         RSp563e - 1./81.*m13*RSp461e*RSp463e + 1./81.*m13*RSp461e*
         RS146pe - 1./81.*m13*RSp461e*RS156pe - 1./81.*m13*RSp461e*
         RSe561p + 1./81.*m13*RSp461e*RSe461p - 1./81.*m13*RSp461e*
         RSp561e + 1./162.*m13*pow(RSp461e,2) + 1./162.*m11*RSp563e*
         RS346pe - 1./162.*m11*RSp563e*RS356pe - 1./162.*m11*RSp563e*
         RSe563p + 1./162.*m11*RSp563e*RSe463p - 1./324.*m11*pow(
         RSp563e,2) - 1./162.*m11*RSp463e*RS346pe + 1./162.*m11*RSp463e
         *RS356pe + 1./162.*m11*RSp463e*RSe563p - 1./162.*m11*RSp463e*
         RSe463p );

      M13p6x23RV +=  + AbV(m45,L)*AbV(mp2,L) * ( 1./162.*m11*RSp463e*
         RSp563e - 1./324.*m11*pow(RSp463e,2) + 1./162.*m11*RS146pe*
         RSp563e - 1./162.*m11*RS146pe*RSp463e - 1./162.*m11*RS156pe*
         RSp563e + 1./162.*m11*RS156pe*RSp463e - 1./162.*m11*RSe561p*
         RSp563e + 1./162.*m11*RSe561p*RSp463e + 1./162.*m11*RSe461p*
         RSp563e - 1./162.*m11*RSe461p*RSp463e + 1./162.*m11*RSp561e*
         RS346pe - 1./162.*m11*RSp561e*RS356pe - 1./162.*m11*RSp561e*
         RSe563p + 1./162.*m11*RSp561e*RSe463p - 1./162.*m11*RSp561e*
         RSp563e + 1./162.*m11*RSp561e*RSp463e + 1./54.*m11*RSp561e*
         RS146pe - 1./54.*m11*RSp561e*RS156pe - 1./54.*m11*RSp561e*
         RSe561p + 1./54.*m11*RSp561e*RSe461p - 1./108.*m11*pow(
         RSp561e,2) - 1./162.*m11*RSp461e*RS346pe + 1./162.*m11*RSp461e
         *RS356pe + 1./162.*m11*RSp461e*RSe563p - 1./162.*m11*RSp461e*
         RSe463p + 1./162.*m11*RSp461e*RSp563e - 1./162.*m11*RSp461e*
         RSp463e - 1./54.*m11*RSp461e*RS146pe + 1./54.*m11*RSp461e*
         RS156pe );

      M13p6x23RV +=  + AbV(m45,L)*AbV(mp2,L) * ( 1./54.*m11*RSp461e*
         RSe561p - 1./54.*m11*RSp461e*RSe461p + 1./54.*m11*RSp461e*
         RSp561e - 1./108.*m11*pow(RSp461e,2) );

      M13p6x23RV +=  + AbV(m45,L)*AbV(me2,L) * (  - 1./81.*me2*RSe563p*
         RS346pe + 1./81.*me2*RSe563p*RS356pe + 1./81.*me2*pow(
         RSe563p,2) + 1./81.*me2*RSe463p*RS346pe - 1./81.*me2*RSe463p*
         RS356pe - 2./81.*me2*RSe463p*RSe563p + 1./81.*me2*pow(
         RSe463p,2) + 1./81.*me2*RSp563e*RSe563p - 1./81.*me2*RSp563e*
         RSe463p - 1./81.*me2*RSp463e*RSe563p + 1./81.*me2*RSp463e*
         RSe463p + 1./81.*me2*RS146pe*RSe563p - 1./81.*me2*RS146pe*
         RSe463p - 1./81.*me2*RS156pe*RSe563p + 1./81.*me2*RS156pe*
         RSe463p + 1./81.*me2*RSe561p*RS346pe - 1./81.*me2*RSe561p*
         RS356pe - 2./81.*me2*RSe561p*RSe563p + 2./81.*me2*RSe561p*
         RSe463p - 1./81.*me2*RSe561p*RSp563e + 1./81.*me2*RSe561p*
         RSp463e - 1./81.*me2*RSe561p*RS146pe + 1./81.*me2*RSe561p*
         RS156pe + 1./81.*me2*pow(RSe561p,2) - 1./81.*me2*RSe461p*
         RS346pe + 1./81.*me2*RSe461p*RS356pe + 2./81.*me2*RSe461p*
         RSe563p - 2./81.*me2*RSe461p*RSe463p + 1./81.*me2*RSe461p*
         RSp563e );

      M13p6x23RV +=  + AbV(m45,L)*AbV(me2,L) * (  - 1./81.*me2*RSe461p*
         RSp463e + 1./81.*me2*RSe461p*RS146pe - 1./81.*me2*RSe461p*
         RS156pe - 2./81.*me2*RSe461p*RSe561p + 1./81.*me2*pow(
         RSe461p,2) - 1./81.*me2*RSp561e*RSe563p + 1./81.*me2*RSp561e*
         RSe463p + 1./81.*me2*RSp561e*RSe561p - 1./81.*me2*RSp561e*
         RSe461p + 1./81.*me2*RSp461e*RSe563p - 1./81.*me2*RSp461e*
         RSe463p - 1./81.*me2*RSp461e*RSe561p + 1./81.*me2*RSp461e*
         RSe461p - 1./162.*m55*RSe563p*RS346pe - 1./162.*m55*RSe563p*
         RS356pe - 1./324.*m55*pow(RSe563p,2) - 1./54.*m55*RSe463p*
         RS346pe - 1./162.*m55*RSe463p*RS356pe - 1./162.*m55*RSe463p*
         RSe563p - 1./108.*m55*pow(RSe463p,2) + 1./162.*m55*RS146pe*
         RSe563p + 1./54.*m55*RS146pe*RSe463p + 1./162.*m55*RS156pe*
         RSe563p + 1./162.*m55*RS156pe*RSe463p + 1./162.*m55*RSe561p*
         RS346pe + 1./162.*m55*RSe561p*RS356pe + 1./162.*m55*RSe561p*
         RSe563p + 1./162.*m55*RSe561p*RSe463p - 1./162.*m55*RSe561p*
         RS146pe );

      M13p6x23RV +=  + AbV(m45,L)*AbV(me2,L) * (  - 1./162.*m55*RSe561p
         *RS156pe - 1./324.*m55*pow(RSe561p,2) + 1./54.*m55*RSe461p*
         RS346pe + 1./162.*m55*RSe461p*RS356pe + 1./162.*m55*RSe461p*
         RSe563p + 1./54.*m55*RSe461p*RSe463p - 1./54.*m55*RSe461p*
         RS146pe - 1./162.*m55*RSe461p*RS156pe - 1./162.*m55*RSe461p*
         RSe561p - 1./108.*m55*pow(RSe461p,2) - 1./81.*m45*RSe563p*
         RS346pe + 1./81.*m45*RSe563p*RS356pe + 1./162.*m45*pow(
         RSe563p,2) + 1./81.*m45*RSe463p*RS346pe - 1./81.*m45*RSe463p*
         RS356pe - 1./81.*m45*RSe463p*RSe563p + 1./162.*m45*pow(
         RSe463p,2) + 1./81.*m45*RS146pe*RSe563p - 1./81.*m45*RS146pe*
         RSe463p - 1./81.*m45*RS156pe*RSe563p + 1./81.*m45*RS156pe*
         RSe463p + 1./81.*m45*RSe561p*RS346pe - 1./81.*m45*RSe561p*
         RS356pe - 1./81.*m45*RSe561p*RSe563p + 1./81.*m45*RSe561p*
         RSe463p - 1./81.*m45*RSe561p*RS146pe + 1./81.*m45*RSe561p*
         RS156pe + 1./162.*m45*pow(RSe561p,2) - 1./81.*m45*RSe461p*
         RS346pe );

      M13p6x23RV +=  + AbV(m45,L)*AbV(me2,L) * ( 1./81.*m45*RSe461p*
         RS356pe + 1./81.*m45*RSe461p*RSe563p - 1./81.*m45*RSe461p*
         RSe463p + 1./81.*m45*RSe461p*RS146pe - 1./81.*m45*RSe461p*
         RS156pe - 1./81.*m45*RSe461p*RSe561p + 1./162.*m45*pow(
         RSe461p,2) - 1./162.*m44*RSe563p*RS346pe - 1./54.*m44*RSe563p*
         RS356pe - 1./108.*m44*pow(RSe563p,2) - 1./162.*m44*RSe463p*
         RS346pe - 1./162.*m44*RSe463p*RS356pe - 1./162.*m44*RSe463p*
         RSe563p - 1./324.*m44*pow(RSe463p,2) + 1./162.*m44*RS146pe*
         RSe563p + 1./162.*m44*RS146pe*RSe463p + 1./54.*m44*RS156pe*
         RSe563p + 1./162.*m44*RS156pe*RSe463p + 1./162.*m44*RSe561p*
         RS346pe + 1./54.*m44*RSe561p*RS356pe + 1./54.*m44*RSe561p*
         RSe563p + 1./162.*m44*RSe561p*RSe463p - 1./162.*m44*RSe561p*
         RS146pe - 1./54.*m44*RSe561p*RS156pe - 1./108.*m44*pow(
         RSe561p,2) + 1./162.*m44*RSe461p*RS346pe + 1./162.*m44*RSe461p
         *RS356pe + 1./162.*m44*RSe461p*RSe563p + 1./162.*m44*RSe461p*
         RSe463p );

      M13p6x23RV +=  + AbV(m45,L)*AbV(me2,L) * (  - 1./162.*m44*RSe461p
         *RS146pe - 1./162.*m44*RSe461p*RS156pe - 1./162.*m44*RSe461p*
         RSe561p - 1./324.*m44*pow(RSe461p,2) + 1./54.*m33*RSe563p*
         RS346pe - 1./54.*m33*RSe563p*RS356pe - 1./108.*m33*pow(
         RSe563p,2) - 1./54.*m33*RSe463p*RS346pe + 1./54.*m33*RSe463p*
         RS356pe + 1./54.*m33*RSe463p*RSe563p - 1./108.*m33*pow(
         RSe463p,2) + 1./162.*m33*RS146pe*RSe563p - 1./162.*m33*RS146pe
         *RSe463p - 1./162.*m33*RS156pe*RSe563p + 1./162.*m33*RS156pe*
         RSe463p + 1./162.*m33*RSe561p*RS346pe - 1./162.*m33*RSe561p*
         RS356pe - 1./162.*m33*RSe561p*RSe563p + 1./162.*m33*RSe561p*
         RSe463p + 1./162.*m33*RSe561p*RS146pe - 1./162.*m33*RSe561p*
         RS156pe - 1./324.*m33*pow(RSe561p,2) - 1./162.*m33*RSe461p*
         RS346pe + 1./162.*m33*RSe461p*RS356pe + 1./162.*m33*RSe461p*
         RSe563p - 1./162.*m33*RSe461p*RSe463p - 1./162.*m33*RSe461p*
         RS146pe + 1./162.*m33*RSe461p*RS156pe + 1./162.*m33*RSe461p*
         RSe561p );

      M13p6x23RV +=  + AbV(m45,L)*AbV(me2,L) * (  - 1./324.*m33*pow(
         RSe461p,2) - 1./81.*m13*RSe563p*RS346pe + 1./81.*m13*RSe563p*
         RS356pe + 1./162.*m13*pow(RSe563p,2) + 1./81.*m13*RSe463p*
         RS346pe - 1./81.*m13*RSe463p*RS356pe - 1./81.*m13*RSe463p*
         RSe563p + 1./162.*m13*pow(RSe463p,2) + 1./81.*m13*RS146pe*
         RSe563p - 1./81.*m13*RS146pe*RSe463p - 1./81.*m13*RS156pe*
         RSe563p + 1./81.*m13*RS156pe*RSe463p + 1./81.*m13*RSe561p*
         RS346pe - 1./81.*m13*RSe561p*RS356pe - 1./81.*m13*RSe561p*
         RSe563p + 1./81.*m13*RSe561p*RSe463p - 1./81.*m13*RSe561p*
         RS146pe + 1./81.*m13*RSe561p*RS156pe + 1./162.*m13*pow(
         RSe561p,2) - 1./81.*m13*RSe461p*RS346pe + 1./81.*m13*RSe461p*
         RS356pe + 1./81.*m13*RSe461p*RSe563p - 1./81.*m13*RSe461p*
         RSe463p + 1./81.*m13*RSe461p*RS146pe - 1./81.*m13*RSe461p*
         RS156pe - 1./81.*m13*RSe461p*RSe561p + 1./162.*m13*pow(
         RSe461p,2) + 1./162.*m11*RSe563p*RS346pe - 1./162.*m11*RSe563p
         *RS356pe );

      M13p6x23RV +=  + AbV(m45,L)*AbV(me2,L) * (  - 1./324.*m11*pow(
         RSe563p,2) - 1./162.*m11*RSe463p*RS346pe + 1./162.*m11*RSe463p
         *RS356pe + 1./162.*m11*RSe463p*RSe563p - 1./324.*m11*pow(
         RSe463p,2) + 1./162.*m11*RS146pe*RSe563p - 1./162.*m11*RS146pe
         *RSe463p - 1./162.*m11*RS156pe*RSe563p + 1./162.*m11*RS156pe*
         RSe463p + 1./162.*m11*RSe561p*RS346pe - 1./162.*m11*RSe561p*
         RS356pe - 1./162.*m11*RSe561p*RSe563p + 1./162.*m11*RSe561p*
         RSe463p + 1./54.*m11*RSe561p*RS146pe - 1./54.*m11*RSe561p*
         RS156pe - 1./108.*m11*pow(RSe561p,2) - 1./162.*m11*RSe461p*
         RS346pe + 1./162.*m11*RSe461p*RS356pe + 1./162.*m11*RSe461p*
         RSe563p - 1./162.*m11*RSe461p*RSe463p - 1./54.*m11*RSe461p*
         RS146pe + 1./54.*m11*RSe461p*RS156pe + 1./54.*m11*RSe461p*
         RSe561p - 1./108.*m11*pow(RSe461p,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m11,m33,L) * ( 1./108.*m33*m55*
         RS146pe*RS346pe + 1./324.*m33*m55*RS146pe*RS356pe + 1./324.*
         m33*m55*RS156pe*RS346pe + 1./324.*m33*m55*RS156pe*RS356pe - 1./
         162.*m33*m45*RS146pe*RS346pe + 1./162.*m33*m45*RS146pe*RS356pe
          + 1./162.*m33*m45*RS156pe*RS346pe - 1./162.*m33*m45*RS156pe*
         RS356pe + 1./324.*m33*m44*RS146pe*RS346pe + 1./324.*m33*m44*
         RS146pe*RS356pe + 1./324.*m33*m44*RS156pe*RS346pe + 1./108.*
         m33*m44*RS156pe*RS356pe + 1./54.*m13*m55*RS146pe*RS346pe + 1./
         162.*m13*m55*RS146pe*RS356pe + 1./162.*m13*m55*RS156pe*RS346pe
          + 1./162.*m13*m55*RS156pe*RS356pe - 1./81.*m13*m45*RS146pe*
         RS346pe + 1./81.*m13*m45*RS146pe*RS356pe + 1./81.*m13*m45*
         RS156pe*RS346pe - 1./81.*m13*m45*RS156pe*RS356pe + 1./162.*m13
         *m44*RS146pe*RS346pe + 1./162.*m13*m44*RS146pe*RS356pe + 1./
         162.*m13*m44*RS156pe*RS346pe + 1./54.*m13*m44*RS156pe*RS356pe
          + 1./36.*m11*m55*RS146pe*RS346pe + 1./108.*m11*m55*RS146pe*
         RS356pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m11,m33,L) * ( 1./108.*m11*m55*
         RS156pe*RS346pe + 1./108.*m11*m55*RS156pe*RS356pe - 1./54.*m11
         *m45*RS146pe*RS346pe + 1./54.*m11*m45*RS146pe*RS356pe + 1./54.
         *m11*m45*RS156pe*RS346pe - 1./54.*m11*m45*RS156pe*RS356pe + 1./
         108.*m11*m44*RS146pe*RS346pe + 1./108.*m11*m44*RS146pe*RS356pe
          + 1./108.*m11*m44*RS156pe*RS346pe + 1./36.*m11*m44*RS156pe*
         RS356pe - 1./54.*m11*m33*RS146pe*RS346pe + 1./54.*m11*m33*
         RS146pe*RS356pe + 1./54.*m11*m33*RS156pe*RS346pe - 1./54.*m11*
         m33*RS156pe*RS356pe - 1./81.*m11*m13*RS146pe*RS346pe + 1./81.*
         m11*m13*RS146pe*RS356pe + 1./81.*m11*m13*RS156pe*RS346pe - 1./
         81.*m11*m13*RS156pe*RS356pe - 1./162.*pow(m11,2)*RS146pe*
         RS346pe + 1./162.*pow(m11,2)*RS146pe*RS356pe + 1./162.*pow(
         m11,2)*RS156pe*RS346pe - 1./162.*pow(m11,2)*RS156pe*RS356pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m11,mp2,L) * ( 1./324.*m33*m55*
         RS146pe*RSp563e + 1./108.*m33*m55*RS146pe*RSp463e + 1./324.*
         m33*m55*RS156pe*RSp563e + 1./324.*m33*m55*RS156pe*RSp463e + 1./
         324.*m33*m55*RSp561e*RS146pe + 1./324.*m33*m55*RSp561e*RS156pe
          + 1./108.*m33*m55*RSp461e*RS146pe + 1./324.*m33*m55*RSp461e*
         RS156pe + 1./162.*m33*m45*RS146pe*RSp563e - 1./162.*m33*m45*
         RS146pe*RSp463e - 1./162.*m33*m45*RS156pe*RSp563e + 1./162.*
         m33*m45*RS156pe*RSp463e + 1./162.*m33*m45*RSp561e*RS146pe - 1./
         162.*m33*m45*RSp561e*RS156pe - 1./162.*m33*m45*RSp461e*RS146pe
          + 1./162.*m33*m45*RSp461e*RS156pe + 1./324.*m33*m44*RS146pe*
         RSp563e + 1./324.*m33*m44*RS146pe*RSp463e + 1./108.*m33*m44*
         RS156pe*RSp563e + 1./324.*m33*m44*RS156pe*RSp463e + 1./324.*
         m33*m44*RSp561e*RS146pe + 1./108.*m33*m44*RSp561e*RS156pe + 1./
         324.*m33*m44*RSp461e*RS146pe + 1./324.*m33*m44*RSp461e*RS156pe
          + 1./162.*m13*m55*RS146pe*RSp563e + 1./54.*m13*m55*RS146pe*
         RSp463e );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m11,mp2,L) * ( 1./162.*m13*m55*
         RS156pe*RSp563e + 1./162.*m13*m55*RS156pe*RSp463e - 1./162.*
         m13*m55*RSp561e*RS146pe - 1./162.*m13*m55*RSp561e*RS156pe - 1./
         54.*m13*m55*RSp461e*RS146pe - 1./162.*m13*m55*RSp461e*RS156pe
          + 1./81.*m13*m45*RS146pe*RSp563e - 1./81.*m13*m45*RS146pe*
         RSp463e - 1./81.*m13*m45*RS156pe*RSp563e + 1./81.*m13*m45*
         RS156pe*RSp463e - 1./81.*m13*m45*RSp561e*RS146pe + 1./81.*m13*
         m45*RSp561e*RS156pe + 1./81.*m13*m45*RSp461e*RS146pe - 1./81.*
         m13*m45*RSp461e*RS156pe + 1./162.*m13*m44*RS146pe*RSp563e + 1./
         162.*m13*m44*RS146pe*RSp463e + 1./54.*m13*m44*RS156pe*RSp563e
          + 1./162.*m13*m44*RS156pe*RSp463e - 1./162.*m13*m44*RSp561e*
         RS146pe - 1./54.*m13*m44*RSp561e*RS156pe - 1./162.*m13*m44*
         RSp461e*RS146pe - 1./162.*m13*m44*RSp461e*RS156pe + 1./81.*m11
         *mp2*RS146pe*RSp563e - 1./81.*m11*mp2*RS146pe*RSp463e - 1./81.
         *m11*mp2*RS156pe*RSp563e + 1./81.*m11*mp2*RS156pe*RSp463e - 1./
         81.*m11*mp2*RSp561e*RS146pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m11,mp2,L) * ( 1./81.*m11*mp2*
         RSp561e*RS156pe + 1./81.*m11*mp2*RSp461e*RS146pe - 1./81.*m11*
         mp2*RSp461e*RS156pe + 1./108.*m11*m55*RS146pe*RSp563e + 1./36.
         *m11*m55*RS146pe*RSp463e + 1./108.*m11*m55*RS156pe*RSp563e + 1.
         /108.*m11*m55*RS156pe*RSp463e + 1./324.*m11*m55*RSp561e*
         RS146pe + 1./324.*m11*m55*RSp561e*RS156pe + 1./108.*m11*m55*
         RSp461e*RS146pe + 1./324.*m11*m55*RSp461e*RS156pe + 1./54.*m11
         *m45*RS146pe*RSp563e - 1./54.*m11*m45*RS146pe*RSp463e - 1./54.
         *m11*m45*RS156pe*RSp563e + 1./54.*m11*m45*RS156pe*RSp463e + 1./
         162.*m11*m45*RSp561e*RS146pe - 1./162.*m11*m45*RSp561e*RS156pe
          - 1./162.*m11*m45*RSp461e*RS146pe + 1./162.*m11*m45*RSp461e*
         RS156pe + 1./108.*m11*m44*RS146pe*RSp563e + 1./108.*m11*m44*
         RS146pe*RSp463e + 1./36.*m11*m44*RS156pe*RSp563e + 1./108.*m11
         *m44*RS156pe*RSp463e + 1./324.*m11*m44*RSp561e*RS146pe + 1./
         108.*m11*m44*RSp561e*RS156pe + 1./324.*m11*m44*RSp461e*RS146pe
          + 1./324.*m11*m44*RSp461e*RS156pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m11,mp2,L) * ( 1./162.*m11*m33*
         RS146pe*RSp563e - 1./162.*m11*m33*RS146pe*RSp463e - 1./162.*
         m11*m33*RS156pe*RSp563e + 1./162.*m11*m33*RS156pe*RSp463e + 1./
         162.*m11*m33*RSp561e*RS146pe - 1./162.*m11*m33*RSp561e*RS156pe
          - 1./162.*m11*m33*RSp461e*RS146pe + 1./162.*m11*m33*RSp461e*
         RS156pe + 1./81.*m11*m13*RS146pe*RSp563e - 1./81.*m11*m13*
         RS146pe*RSp463e - 1./81.*m11*m13*RS156pe*RSp563e + 1./81.*m11*
         m13*RS156pe*RSp463e - 1./81.*m11*m13*RSp561e*RS146pe + 1./81.*
         m11*m13*RSp561e*RS156pe + 1./81.*m11*m13*RSp461e*RS146pe - 1./
         81.*m11*m13*RSp461e*RS156pe + 1./162.*pow(m11,2)*RS146pe*
         RSp563e - 1./162.*pow(m11,2)*RS146pe*RSp463e - 1./162.*pow(
         m11,2)*RS156pe*RSp563e + 1./162.*pow(m11,2)*RS156pe*RSp463e + 
         1./54.*pow(m11,2)*RSp561e*RS146pe - 1./54.*pow(m11,2)*RSp561e*
         RS156pe - 1./54.*pow(m11,2)*RSp461e*RS146pe + 1./54.*pow(
         m11,2)*RSp461e*RS156pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m11,me2,L) * ( 1./324.*m33*m55*
         RS146pe*RSe563p + 1./108.*m33*m55*RS146pe*RSe463p + 1./324.*
         m33*m55*RS156pe*RSe563p + 1./324.*m33*m55*RS156pe*RSe463p + 1./
         324.*m33*m55*RSe561p*RS146pe + 1./324.*m33*m55*RSe561p*RS156pe
          + 1./108.*m33*m55*RSe461p*RS146pe + 1./324.*m33*m55*RSe461p*
         RS156pe + 1./162.*m33*m45*RS146pe*RSe563p - 1./162.*m33*m45*
         RS146pe*RSe463p - 1./162.*m33*m45*RS156pe*RSe563p + 1./162.*
         m33*m45*RS156pe*RSe463p + 1./162.*m33*m45*RSe561p*RS146pe - 1./
         162.*m33*m45*RSe561p*RS156pe - 1./162.*m33*m45*RSe461p*RS146pe
          + 1./162.*m33*m45*RSe461p*RS156pe + 1./324.*m33*m44*RS146pe*
         RSe563p + 1./324.*m33*m44*RS146pe*RSe463p + 1./108.*m33*m44*
         RS156pe*RSe563p + 1./324.*m33*m44*RS156pe*RSe463p + 1./324.*
         m33*m44*RSe561p*RS146pe + 1./108.*m33*m44*RSe561p*RS156pe + 1./
         324.*m33*m44*RSe461p*RS146pe + 1./324.*m33*m44*RSe461p*RS156pe
          + 1./162.*m13*m55*RS146pe*RSe563p + 1./54.*m13*m55*RS146pe*
         RSe463p );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m11,me2,L) * ( 1./162.*m13*m55*
         RS156pe*RSe563p + 1./162.*m13*m55*RS156pe*RSe463p - 1./162.*
         m13*m55*RSe561p*RS146pe - 1./162.*m13*m55*RSe561p*RS156pe - 1./
         54.*m13*m55*RSe461p*RS146pe - 1./162.*m13*m55*RSe461p*RS156pe
          + 1./81.*m13*m45*RS146pe*RSe563p - 1./81.*m13*m45*RS146pe*
         RSe463p - 1./81.*m13*m45*RS156pe*RSe563p + 1./81.*m13*m45*
         RS156pe*RSe463p - 1./81.*m13*m45*RSe561p*RS146pe + 1./81.*m13*
         m45*RSe561p*RS156pe + 1./81.*m13*m45*RSe461p*RS146pe - 1./81.*
         m13*m45*RSe461p*RS156pe + 1./162.*m13*m44*RS146pe*RSe563p + 1./
         162.*m13*m44*RS146pe*RSe463p + 1./54.*m13*m44*RS156pe*RSe563p
          + 1./162.*m13*m44*RS156pe*RSe463p - 1./162.*m13*m44*RSe561p*
         RS146pe - 1./54.*m13*m44*RSe561p*RS156pe - 1./162.*m13*m44*
         RSe461p*RS146pe - 1./162.*m13*m44*RSe461p*RS156pe + 1./81.*m11
         *me2*RS146pe*RSe563p - 1./81.*m11*me2*RS146pe*RSe463p - 1./81.
         *m11*me2*RS156pe*RSe563p + 1./81.*m11*me2*RS156pe*RSe463p - 1./
         81.*m11*me2*RSe561p*RS146pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m11,me2,L) * ( 1./81.*m11*me2*
         RSe561p*RS156pe + 1./81.*m11*me2*RSe461p*RS146pe - 1./81.*m11*
         me2*RSe461p*RS156pe + 1./108.*m11*m55*RS146pe*RSe563p + 1./36.
         *m11*m55*RS146pe*RSe463p + 1./108.*m11*m55*RS156pe*RSe563p + 1.
         /108.*m11*m55*RS156pe*RSe463p + 1./324.*m11*m55*RSe561p*
         RS146pe + 1./324.*m11*m55*RSe561p*RS156pe + 1./108.*m11*m55*
         RSe461p*RS146pe + 1./324.*m11*m55*RSe461p*RS156pe + 1./54.*m11
         *m45*RS146pe*RSe563p - 1./54.*m11*m45*RS146pe*RSe463p - 1./54.
         *m11*m45*RS156pe*RSe563p + 1./54.*m11*m45*RS156pe*RSe463p + 1./
         162.*m11*m45*RSe561p*RS146pe - 1./162.*m11*m45*RSe561p*RS156pe
          - 1./162.*m11*m45*RSe461p*RS146pe + 1./162.*m11*m45*RSe461p*
         RS156pe + 1./108.*m11*m44*RS146pe*RSe563p + 1./108.*m11*m44*
         RS146pe*RSe463p + 1./36.*m11*m44*RS156pe*RSe563p + 1./108.*m11
         *m44*RS156pe*RSe463p + 1./324.*m11*m44*RSe561p*RS146pe + 1./
         108.*m11*m44*RSe561p*RS156pe + 1./324.*m11*m44*RSe461p*RS146pe
          + 1./324.*m11*m44*RSe461p*RS156pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m11,me2,L) * ( 1./162.*m11*m33*
         RS146pe*RSe563p - 1./162.*m11*m33*RS146pe*RSe463p - 1./162.*
         m11*m33*RS156pe*RSe563p + 1./162.*m11*m33*RS156pe*RSe463p + 1./
         162.*m11*m33*RSe561p*RS146pe - 1./162.*m11*m33*RSe561p*RS156pe
          - 1./162.*m11*m33*RSe461p*RS146pe + 1./162.*m11*m33*RSe461p*
         RS156pe + 1./81.*m11*m13*RS146pe*RSe563p - 1./81.*m11*m13*
         RS146pe*RSe463p - 1./81.*m11*m13*RS156pe*RSe563p + 1./81.*m11*
         m13*RS156pe*RSe463p - 1./81.*m11*m13*RSe561p*RS146pe + 1./81.*
         m11*m13*RSe561p*RS156pe + 1./81.*m11*m13*RSe461p*RS146pe - 1./
         81.*m11*m13*RSe461p*RS156pe + 1./162.*pow(m11,2)*RS146pe*
         RSe563p - 1./162.*pow(m11,2)*RS146pe*RSe463p - 1./162.*pow(
         m11,2)*RS156pe*RSe563p + 1./162.*pow(m11,2)*RS156pe*RSe463p + 
         1./54.*pow(m11,2)*RSe561p*RS146pe - 1./54.*pow(m11,2)*RSe561p*
         RS156pe - 1./54.*pow(m11,2)*RSe461p*RS146pe + 1./54.*pow(
         m11,2)*RSe461p*RS156pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m11,L) * ( 1./216.*m33*m55*pow(
         RS146pe,2) + 1./324.*m33*m55*RS156pe*RS146pe + 1./648.*m33*m55
         *pow(RS156pe,2) - 1./324.*m33*m45*pow(RS146pe,2) + 1./162.*m33
         *m45*RS156pe*RS146pe - 1./324.*m33*m45*pow(RS156pe,2) + 1./648.
         *m33*m44*pow(RS146pe,2) + 1./324.*m33*m44*RS156pe*RS146pe + 1./
         216.*m33*m44*pow(RS156pe,2) - 1./108.*m13*m55*pow(RS146pe,2)
          - 1./162.*m13*m55*RS156pe*RS146pe - 1./324.*m13*m55*pow(
         RS156pe,2) + 1./162.*m13*m45*pow(RS146pe,2) - 1./81.*m13*m45*
         RS156pe*RS146pe + 1./162.*m13*m45*pow(RS156pe,2) - 1./324.*m13
         *m44*pow(RS146pe,2) - 1./162.*m13*m44*RS156pe*RS146pe - 1./108.
         *m13*m44*pow(RS156pe,2) + 1./216.*m11*m55*pow(RS146pe,2) + 1./
         324.*m11*m55*RS156pe*RS146pe + 1./648.*m11*m55*pow(RS156pe,2)
          - 1./324.*m11*m45*pow(RS146pe,2) + 1./162.*m11*m45*RS156pe*
         RS146pe - 1./324.*m11*m45*pow(RS156pe,2) + 1./648.*m11*m44*
         pow(RS146pe,2) + 1./324.*m11*m44*RS156pe*RS146pe + 1./216.*m11
         *m44*pow(RS156pe,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m11,L) * (  - 1./324.*m11*m33*
         pow(RS146pe,2) + 1./162.*m11*m33*RS156pe*RS146pe - 1./324.*m11
         *m33*pow(RS156pe,2) + 1./162.*m11*m13*pow(RS146pe,2) - 1./81.*
         m11*m13*RS156pe*RS146pe + 1./162.*m11*m13*pow(RS156pe,2) - 1./
         324.*pow(m11,2)*pow(RS146pe,2) + 1./162.*pow(m11,2)*RS156pe*
         RS146pe - 1./324.*pow(m11,2)*pow(RS156pe,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m14,L) * ( 1./144.*m44*m55 - 1./
         72.*m44*m45 + 1./72.*pow(m44,2) + 1./144.*m33*m55 - 1./72.*m33
         *m45 + 1./72.*m33*m44 - 1./72.*m14*m55 + 1./36.*m14*m45 - 1./
         24.*m14*m44 - 1./72.*m14*m33 + 1./36.*pow(m14,2) - 1./72.*m13*
         m55 + 1./36.*m13*m45 - 1./36.*m13*m44 + 1./36.*m13*m14 + 1./72.
         *m11*m55 - 1./36.*m11*m45 + 5./144.*m11*m44 + 1./144.*m11*m33
          - 1./24.*m11*m14 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m15,L) * ( 1./72.*pow(m55,2) - 1./
         72.*m45*m55 + 1./144.*m44*m55 + 1./72.*m33*m55 - 1./72.*m33*
         m45 + 1./144.*m33*m44 - 1./24.*m15*m55 + 1./36.*m15*m45 - 1./
         72.*m15*m44 - 1./72.*m15*m33 + 1./36.*pow(m15,2) - 1./36.*m13*
         m55 + 1./36.*m13*m45 - 1./72.*m13*m44 + 1./36.*m13*m15 + 5./
         144.*m11*m55 - 1./36.*m11*m45 + 1./72.*m11*m44 + 1./144.*m11*
         m33 - 1./24.*m11*m15 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m33,mp2,L) * (  - 1./81.*m33*mp2*
         RSp563e*RS346pe + 1./81.*m33*mp2*RSp563e*RS356pe + 1./81.*m33*
         mp2*RSp463e*RS346pe - 1./81.*m33*mp2*RSp463e*RS356pe + 1./81.*
         m33*mp2*RSp561e*RS346pe - 1./81.*m33*mp2*RSp561e*RS356pe - 1./
         81.*m33*mp2*RSp461e*RS346pe + 1./81.*m33*mp2*RSp461e*RS356pe
          + 1./324.*m33*m55*RSp563e*RS346pe + 1./324.*m33*m55*RSp563e*
         RS356pe + 1./108.*m33*m55*RSp463e*RS346pe + 1./324.*m33*m55*
         RSp463e*RS356pe + 1./108.*m33*m55*RSp561e*RS346pe + 1./108.*
         m33*m55*RSp561e*RS356pe + 1./36.*m33*m55*RSp461e*RS346pe + 1./
         108.*m33*m55*RSp461e*RS356pe + 1./162.*m33*m45*RSp563e*RS346pe
          - 1./162.*m33*m45*RSp563e*RS356pe - 1./162.*m33*m45*RSp463e*
         RS346pe + 1./162.*m33*m45*RSp463e*RS356pe + 1./54.*m33*m45*
         RSp561e*RS346pe - 1./54.*m33*m45*RSp561e*RS356pe - 1./54.*m33*
         m45*RSp461e*RS346pe + 1./54.*m33*m45*RSp461e*RS356pe + 1./324.
         *m33*m44*RSp563e*RS346pe + 1./108.*m33*m44*RSp563e*RS356pe + 1.
         /324.*m33*m44*RSp463e*RS346pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m33,mp2,L) * ( 1./324.*m33*m44*
         RSp463e*RS356pe + 1./108.*m33*m44*RSp561e*RS346pe + 1./36.*m33
         *m44*RSp561e*RS356pe + 1./108.*m33*m44*RSp461e*RS346pe + 1./
         108.*m33*m44*RSp461e*RS356pe + 1./54.*pow(m33,2)*RSp563e*
         RS346pe - 1./54.*pow(m33,2)*RSp563e*RS356pe - 1./54.*pow(
         m33,2)*RSp463e*RS346pe + 1./54.*pow(m33,2)*RSp463e*RS356pe + 1.
         /162.*pow(m33,2)*RSp561e*RS346pe - 1./162.*pow(m33,2)*RSp561e*
         RS356pe - 1./162.*pow(m33,2)*RSp461e*RS346pe + 1./162.*pow(
         m33,2)*RSp461e*RS356pe - 1./162.*m13*m55*RSp563e*RS346pe - 1./
         162.*m13*m55*RSp563e*RS356pe - 1./54.*m13*m55*RSp463e*RS346pe
          - 1./162.*m13*m55*RSp463e*RS356pe + 1./162.*m13*m55*RSp561e*
         RS346pe + 1./162.*m13*m55*RSp561e*RS356pe + 1./54.*m13*m55*
         RSp461e*RS346pe + 1./162.*m13*m55*RSp461e*RS356pe - 1./81.*m13
         *m45*RSp563e*RS346pe + 1./81.*m13*m45*RSp563e*RS356pe + 1./81.
         *m13*m45*RSp463e*RS346pe - 1./81.*m13*m45*RSp463e*RS356pe + 1./
         81.*m13*m45*RSp561e*RS346pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m33,mp2,L) * (  - 1./81.*m13*m45*
         RSp561e*RS356pe - 1./81.*m13*m45*RSp461e*RS346pe + 1./81.*m13*
         m45*RSp461e*RS356pe - 1./162.*m13*m44*RSp563e*RS346pe - 1./54.
         *m13*m44*RSp563e*RS356pe - 1./162.*m13*m44*RSp463e*RS346pe - 1.
         /162.*m13*m44*RSp463e*RS356pe + 1./162.*m13*m44*RSp561e*
         RS346pe + 1./54.*m13*m44*RSp561e*RS356pe + 1./162.*m13*m44*
         RSp461e*RS346pe + 1./162.*m13*m44*RSp461e*RS356pe - 1./81.*m13
         *m33*RSp563e*RS346pe + 1./81.*m13*m33*RSp563e*RS356pe + 1./81.
         *m13*m33*RSp463e*RS346pe - 1./81.*m13*m33*RSp463e*RS356pe + 1./
         81.*m13*m33*RSp561e*RS346pe - 1./81.*m13*m33*RSp561e*RS356pe
          - 1./81.*m13*m33*RSp461e*RS346pe + 1./81.*m13*m33*RSp461e*
         RS356pe + 1./324.*m11*m55*RSp563e*RS346pe + 1./324.*m11*m55*
         RSp563e*RS356pe + 1./108.*m11*m55*RSp463e*RS346pe + 1./324.*
         m11*m55*RSp463e*RS356pe + 1./324.*m11*m55*RSp561e*RS346pe + 1./
         324.*m11*m55*RSp561e*RS356pe + 1./108.*m11*m55*RSp461e*RS346pe
          + 1./324.*m11*m55*RSp461e*RS356pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m33,mp2,L) * ( 1./162.*m11*m45*
         RSp563e*RS346pe - 1./162.*m11*m45*RSp563e*RS356pe - 1./162.*
         m11*m45*RSp463e*RS346pe + 1./162.*m11*m45*RSp463e*RS356pe + 1./
         162.*m11*m45*RSp561e*RS346pe - 1./162.*m11*m45*RSp561e*RS356pe
          - 1./162.*m11*m45*RSp461e*RS346pe + 1./162.*m11*m45*RSp461e*
         RS356pe + 1./324.*m11*m44*RSp563e*RS346pe + 1./108.*m11*m44*
         RSp563e*RS356pe + 1./324.*m11*m44*RSp463e*RS346pe + 1./324.*
         m11*m44*RSp463e*RS356pe + 1./324.*m11*m44*RSp561e*RS346pe + 1./
         108.*m11*m44*RSp561e*RS356pe + 1./324.*m11*m44*RSp461e*RS346pe
          + 1./324.*m11*m44*RSp461e*RS356pe + 1./162.*m11*m33*RSp563e*
         RS346pe - 1./162.*m11*m33*RSp563e*RS356pe - 1./162.*m11*m33*
         RSp463e*RS346pe + 1./162.*m11*m33*RSp463e*RS356pe + 1./162.*
         m11*m33*RSp561e*RS346pe - 1./162.*m11*m33*RSp561e*RS356pe - 1./
         162.*m11*m33*RSp461e*RS346pe + 1./162.*m11*m33*RSp461e*RS356pe
          );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m33,me2,L) * (  - 1./81.*m33*me2*
         RSe563p*RS346pe + 1./81.*m33*me2*RSe563p*RS356pe + 1./81.*m33*
         me2*RSe463p*RS346pe - 1./81.*m33*me2*RSe463p*RS356pe + 1./81.*
         m33*me2*RSe561p*RS346pe - 1./81.*m33*me2*RSe561p*RS356pe - 1./
         81.*m33*me2*RSe461p*RS346pe + 1./81.*m33*me2*RSe461p*RS356pe
          + 1./324.*m33*m55*RSe563p*RS346pe + 1./324.*m33*m55*RSe563p*
         RS356pe + 1./108.*m33*m55*RSe463p*RS346pe + 1./324.*m33*m55*
         RSe463p*RS356pe + 1./108.*m33*m55*RSe561p*RS346pe + 1./108.*
         m33*m55*RSe561p*RS356pe + 1./36.*m33*m55*RSe461p*RS346pe + 1./
         108.*m33*m55*RSe461p*RS356pe + 1./162.*m33*m45*RSe563p*RS346pe
          - 1./162.*m33*m45*RSe563p*RS356pe - 1./162.*m33*m45*RSe463p*
         RS346pe + 1./162.*m33*m45*RSe463p*RS356pe + 1./54.*m33*m45*
         RSe561p*RS346pe - 1./54.*m33*m45*RSe561p*RS356pe - 1./54.*m33*
         m45*RSe461p*RS346pe + 1./54.*m33*m45*RSe461p*RS356pe + 1./324.
         *m33*m44*RSe563p*RS346pe + 1./108.*m33*m44*RSe563p*RS356pe + 1.
         /324.*m33*m44*RSe463p*RS346pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m33,me2,L) * ( 1./324.*m33*m44*
         RSe463p*RS356pe + 1./108.*m33*m44*RSe561p*RS346pe + 1./36.*m33
         *m44*RSe561p*RS356pe + 1./108.*m33*m44*RSe461p*RS346pe + 1./
         108.*m33*m44*RSe461p*RS356pe + 1./54.*pow(m33,2)*RSe563p*
         RS346pe - 1./54.*pow(m33,2)*RSe563p*RS356pe - 1./54.*pow(
         m33,2)*RSe463p*RS346pe + 1./54.*pow(m33,2)*RSe463p*RS356pe + 1.
         /162.*pow(m33,2)*RSe561p*RS346pe - 1./162.*pow(m33,2)*RSe561p*
         RS356pe - 1./162.*pow(m33,2)*RSe461p*RS346pe + 1./162.*pow(
         m33,2)*RSe461p*RS356pe - 1./162.*m13*m55*RSe563p*RS346pe - 1./
         162.*m13*m55*RSe563p*RS356pe - 1./54.*m13*m55*RSe463p*RS346pe
          - 1./162.*m13*m55*RSe463p*RS356pe + 1./162.*m13*m55*RSe561p*
         RS346pe + 1./162.*m13*m55*RSe561p*RS356pe + 1./54.*m13*m55*
         RSe461p*RS346pe + 1./162.*m13*m55*RSe461p*RS356pe - 1./81.*m13
         *m45*RSe563p*RS346pe + 1./81.*m13*m45*RSe563p*RS356pe + 1./81.
         *m13*m45*RSe463p*RS346pe - 1./81.*m13*m45*RSe463p*RS356pe + 1./
         81.*m13*m45*RSe561p*RS346pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m33,me2,L) * (  - 1./81.*m13*m45*
         RSe561p*RS356pe - 1./81.*m13*m45*RSe461p*RS346pe + 1./81.*m13*
         m45*RSe461p*RS356pe - 1./162.*m13*m44*RSe563p*RS346pe - 1./54.
         *m13*m44*RSe563p*RS356pe - 1./162.*m13*m44*RSe463p*RS346pe - 1.
         /162.*m13*m44*RSe463p*RS356pe + 1./162.*m13*m44*RSe561p*
         RS346pe + 1./54.*m13*m44*RSe561p*RS356pe + 1./162.*m13*m44*
         RSe461p*RS346pe + 1./162.*m13*m44*RSe461p*RS356pe - 1./81.*m13
         *m33*RSe563p*RS346pe + 1./81.*m13*m33*RSe563p*RS356pe + 1./81.
         *m13*m33*RSe463p*RS346pe - 1./81.*m13*m33*RSe463p*RS356pe + 1./
         81.*m13*m33*RSe561p*RS346pe - 1./81.*m13*m33*RSe561p*RS356pe
          - 1./81.*m13*m33*RSe461p*RS346pe + 1./81.*m13*m33*RSe461p*
         RS356pe + 1./324.*m11*m55*RSe563p*RS346pe + 1./324.*m11*m55*
         RSe563p*RS356pe + 1./108.*m11*m55*RSe463p*RS346pe + 1./324.*
         m11*m55*RSe463p*RS356pe + 1./324.*m11*m55*RSe561p*RS346pe + 1./
         324.*m11*m55*RSe561p*RS356pe + 1./108.*m11*m55*RSe461p*RS346pe
          + 1./324.*m11*m55*RSe461p*RS356pe );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m33,me2,L) * ( 1./162.*m11*m45*
         RSe563p*RS346pe - 1./162.*m11*m45*RSe563p*RS356pe - 1./162.*
         m11*m45*RSe463p*RS346pe + 1./162.*m11*m45*RSe463p*RS356pe + 1./
         162.*m11*m45*RSe561p*RS346pe - 1./162.*m11*m45*RSe561p*RS356pe
          - 1./162.*m11*m45*RSe461p*RS346pe + 1./162.*m11*m45*RSe461p*
         RS356pe + 1./324.*m11*m44*RSe563p*RS346pe + 1./108.*m11*m44*
         RSe563p*RS356pe + 1./324.*m11*m44*RSe463p*RS346pe + 1./324.*
         m11*m44*RSe463p*RS356pe + 1./324.*m11*m44*RSe561p*RS346pe + 1./
         108.*m11*m44*RSe561p*RS356pe + 1./324.*m11*m44*RSe461p*RS346pe
          + 1./324.*m11*m44*RSe461p*RS356pe + 1./162.*m11*m33*RSe563p*
         RS346pe - 1./162.*m11*m33*RSe563p*RS356pe - 1./162.*m11*m33*
         RSe463p*RS346pe + 1./162.*m11*m33*RSe463p*RS356pe + 1./162.*
         m11*m33*RSe561p*RS346pe - 1./162.*m11*m33*RSe561p*RS356pe - 1./
         162.*m11*m33*RSe461p*RS346pe + 1./162.*m11*m33*RSe461p*RS356pe
          );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m33,L) * ( 1./216.*m33*m55*pow(
         RS346pe,2) + 1./324.*m33*m55*RS356pe*RS346pe + 1./648.*m33*m55
         *pow(RS356pe,2) - 1./324.*m33*m45*pow(RS346pe,2) + 1./162.*m33
         *m45*RS356pe*RS346pe - 1./324.*m33*m45*pow(RS356pe,2) + 1./648.
         *m33*m44*pow(RS346pe,2) + 1./324.*m33*m44*RS356pe*RS346pe + 1./
         216.*m33*m44*pow(RS356pe,2) - 1./324.*pow(m33,2)*pow(
         RS346pe,2) + 1./162.*pow(m33,2)*RS356pe*RS346pe - 1./324.*pow(
         m33,2)*pow(RS356pe,2) - 1./108.*m13*m55*pow(RS346pe,2) - 1./
         162.*m13*m55*RS356pe*RS346pe - 1./324.*m13*m55*pow(RS356pe,2)
          + 1./162.*m13*m45*pow(RS346pe,2) - 1./81.*m13*m45*RS356pe*
         RS346pe + 1./162.*m13*m45*pow(RS356pe,2) - 1./324.*m13*m44*
         pow(RS346pe,2) - 1./162.*m13*m44*RS356pe*RS346pe - 1./108.*m13
         *m44*pow(RS356pe,2) + 1./162.*m13*m33*pow(RS346pe,2) - 1./81.*
         m13*m33*RS356pe*RS346pe + 1./162.*m13*m33*pow(RS356pe,2) + 1./
         216.*m11*m55*pow(RS346pe,2) + 1./324.*m11*m55*RS356pe*RS346pe
          + 1./648.*m11*m55*pow(RS356pe,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m33,L) * (  - 1./324.*m11*m45*
         pow(RS346pe,2) + 1./162.*m11*m45*RS356pe*RS346pe - 1./324.*m11
         *m45*pow(RS356pe,2) + 1./648.*m11*m44*pow(RS346pe,2) + 1./324.
         *m11*m44*RS356pe*RS346pe + 1./216.*m11*m44*pow(RS356pe,2) - 1./
         324.*m11*m33*pow(RS346pe,2) + 1./162.*m11*m33*RS356pe*RS346pe
          - 1./324.*m11*m33*pow(RS356pe,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m34,L) * ( 1./144.*m44*m55 - 1./
         72.*m44*m45 + 1./72.*pow(m44,2) - 1./72.*m34*m55 + 1./36.*m34*
         m45 - 1./24.*m34*m44 + 1./36.*pow(m34,2) + 1./72.*m33*m55 - 1./
         36.*m33*m45 + 5./144.*m33*m44 - 1./24.*m33*m34 + 1./72.*pow(
         m33,2) - 1./72.*m13*m55 + 1./36.*m13*m45 - 1./36.*m13*m44 + 1./
         36.*m13*m34 - 1./72.*m13*m33 + 1./144.*m11*m55 - 1./72.*m11*
         m45 + 1./72.*m11*m44 - 1./72.*m11*m34 + 1./144.*m11*m33 );

      M13p6x23RV +=  + AbV(m45,L)*BbV(m35,L) * ( 1./72.*pow(m55,2) - 1./
         72.*m45*m55 + 1./144.*m44*m55 - 1./24.*m35*m55 + 1./36.*m35*
         m45 - 1./72.*m35*m44 + 1./36.*pow(m35,2) + 5./144.*m33*m55 - 1.
         /36.*m33*m45 + 1./72.*m33*m44 - 1./24.*m33*m35 + 1./72.*pow(
         m33,2) - 1./36.*m13*m55 + 1./36.*m13*m45 - 1./72.*m13*m44 + 1./
         36.*m13*m35 - 1./72.*m13*m33 + 1./72.*m11*m55 - 1./72.*m11*m45
          + 1./144.*m11*m44 - 1./72.*m11*m35 + 1./144.*m11*m33 );

      M13p6x23RV +=  + AbV(m45,L)*BbV(mp2,L) * ( 1./162.*pow(mp2,2)*
         pow(RSp563e,2) - 1./81.*pow(mp2,2)*RSp463e*RSp563e + 1./162.*
         pow(mp2,2)*pow(RSp463e,2) - 1./81.*pow(mp2,2)*RSp561e*RSp563e
          + 1./81.*pow(mp2,2)*RSp561e*RSp463e + 1./162.*pow(mp2,2)*pow(
         RSp561e,2) + 1./81.*pow(mp2,2)*RSp461e*RSp563e - 1./81.*pow(
         mp2,2)*RSp461e*RSp463e - 1./81.*pow(mp2,2)*RSp461e*RSp561e + 1.
         /162.*pow(mp2,2)*pow(RSp461e,2) - 1./324.*m55*mp2*pow(
         RSp563e,2) - 1./162.*m55*mp2*RSp463e*RSp563e - 1./108.*m55*mp2
         *pow(RSp463e,2) + 1./162.*m55*mp2*RSp561e*RSp563e + 1./162.*
         m55*mp2*RSp561e*RSp463e - 1./324.*m55*mp2*pow(RSp561e,2) + 1./
         162.*m55*mp2*RSp461e*RSp563e + 1./54.*m55*mp2*RSp461e*RSp463e
          - 1./162.*m55*mp2*RSp461e*RSp561e - 1./108.*m55*mp2*pow(
         RSp461e,2) + 1./162.*m45*mp2*pow(RSp563e,2) - 1./81.*m45*mp2*
         RSp463e*RSp563e + 1./162.*m45*mp2*pow(RSp463e,2) - 1./81.*m45*
         mp2*RSp561e*RSp563e + 1./81.*m45*mp2*RSp561e*RSp463e + 1./162.
         *m45*mp2*pow(RSp561e,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(mp2,L) * ( 1./81.*m45*mp2*RSp461e
         *RSp563e - 1./81.*m45*mp2*RSp461e*RSp463e - 1./81.*m45*mp2*
         RSp461e*RSp561e + 1./162.*m45*mp2*pow(RSp461e,2) - 1./108.*m44
         *mp2*pow(RSp563e,2) - 1./162.*m44*mp2*RSp463e*RSp563e - 1./324.
         *m44*mp2*pow(RSp463e,2) + 1./54.*m44*mp2*RSp561e*RSp563e + 1./
         162.*m44*mp2*RSp561e*RSp463e - 1./108.*m44*mp2*pow(RSp561e,2)
          + 1./162.*m44*mp2*RSp461e*RSp563e + 1./162.*m44*mp2*RSp461e*
         RSp463e - 1./162.*m44*mp2*RSp461e*RSp561e - 1./324.*m44*mp2*
         pow(RSp461e,2) - 1./108.*m33*mp2*pow(RSp563e,2) + 1./54.*m33*
         mp2*RSp463e*RSp563e - 1./108.*m33*mp2*pow(RSp463e,2) - 1./162.
         *m33*mp2*RSp561e*RSp563e + 1./162.*m33*mp2*RSp561e*RSp463e - 1.
         /324.*m33*mp2*pow(RSp561e,2) + 1./162.*m33*mp2*RSp461e*RSp563e
          - 1./162.*m33*mp2*RSp461e*RSp463e + 1./162.*m33*mp2*RSp461e*
         RSp561e - 1./324.*m33*mp2*pow(RSp461e,2) + 1./216.*m33*m55*
         pow(RSp563e,2) + 1./108.*m33*m55*RSp463e*RSp563e + 1./72.*m33*
         m55*pow(RSp463e,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(mp2,L) * ( 1./324.*m33*m55*
         RSp561e*RSp563e + 1./324.*m33*m55*RSp561e*RSp463e + 1./648.*
         m33*m55*pow(RSp561e,2) + 1./324.*m33*m55*RSp461e*RSp563e + 1./
         108.*m33*m55*RSp461e*RSp463e + 1./324.*m33*m55*RSp461e*RSp561e
          + 1./216.*m33*m55*pow(RSp461e,2) - 1./108.*m33*m45*pow(
         RSp563e,2) + 1./54.*m33*m45*RSp463e*RSp563e - 1./108.*m33*m45*
         pow(RSp463e,2) - 1./162.*m33*m45*RSp561e*RSp563e + 1./162.*m33
         *m45*RSp561e*RSp463e - 1./324.*m33*m45*pow(RSp561e,2) + 1./162.
         *m33*m45*RSp461e*RSp563e - 1./162.*m33*m45*RSp461e*RSp463e + 1.
         /162.*m33*m45*RSp461e*RSp561e - 1./324.*m33*m45*pow(RSp461e,2)
          + 1./72.*m33*m44*pow(RSp563e,2) + 1./108.*m33*m44*RSp463e*
         RSp563e + 1./216.*m33*m44*pow(RSp463e,2) + 1./108.*m33*m44*
         RSp561e*RSp563e + 1./324.*m33*m44*RSp561e*RSp463e + 1./216.*
         m33*m44*pow(RSp561e,2) + 1./324.*m33*m44*RSp461e*RSp563e + 1./
         324.*m33*m44*RSp461e*RSp463e + 1./324.*m33*m44*RSp461e*RSp561e
          + 1./648.*m33*m44*pow(RSp461e,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(mp2,L) * ( 1./162.*m13*mp2*pow(
         RSp563e,2) - 1./81.*m13*mp2*RSp463e*RSp563e + 1./162.*m13*mp2*
         pow(RSp463e,2) - 1./81.*m13*mp2*RSp561e*RSp563e + 1./81.*m13*
         mp2*RSp561e*RSp463e + 1./162.*m13*mp2*pow(RSp561e,2) + 1./81.*
         m13*mp2*RSp461e*RSp563e - 1./81.*m13*mp2*RSp461e*RSp463e - 1./
         81.*m13*mp2*RSp461e*RSp561e + 1./162.*m13*mp2*pow(RSp461e,2)
          - 1./324.*m13*m55*pow(RSp563e,2) - 1./162.*m13*m55*RSp463e*
         RSp563e - 1./108.*m13*m55*pow(RSp463e,2) + 1./162.*m13*m55*
         RSp561e*RSp563e + 1./162.*m13*m55*RSp561e*RSp463e - 1./324.*
         m13*m55*pow(RSp561e,2) + 1./162.*m13*m55*RSp461e*RSp563e + 1./
         54.*m13*m55*RSp461e*RSp463e - 1./162.*m13*m55*RSp461e*RSp561e
          - 1./108.*m13*m55*pow(RSp461e,2) + 1./162.*m13*m45*pow(
         RSp563e,2) - 1./81.*m13*m45*RSp463e*RSp563e + 1./162.*m13*m45*
         pow(RSp463e,2) - 1./81.*m13*m45*RSp561e*RSp563e + 1./81.*m13*
         m45*RSp561e*RSp463e + 1./162.*m13*m45*pow(RSp561e,2) + 1./81.*
         m13*m45*RSp461e*RSp563e );

      M13p6x23RV +=  + AbV(m45,L)*BbV(mp2,L) * (  - 1./81.*m13*m45*
         RSp461e*RSp463e - 1./81.*m13*m45*RSp461e*RSp561e + 1./162.*m13
         *m45*pow(RSp461e,2) - 1./108.*m13*m44*pow(RSp563e,2) - 1./162.
         *m13*m44*RSp463e*RSp563e - 1./324.*m13*m44*pow(RSp463e,2) + 1./
         54.*m13*m44*RSp561e*RSp563e + 1./162.*m13*m44*RSp561e*RSp463e
          - 1./108.*m13*m44*pow(RSp561e,2) + 1./162.*m13*m44*RSp461e*
         RSp563e + 1./162.*m13*m44*RSp461e*RSp463e - 1./162.*m13*m44*
         RSp461e*RSp561e - 1./324.*m13*m44*pow(RSp461e,2) - 1./324.*m11
         *mp2*pow(RSp563e,2) + 1./162.*m11*mp2*RSp463e*RSp563e - 1./324.
         *m11*mp2*pow(RSp463e,2) - 1./162.*m11*mp2*RSp561e*RSp563e + 1./
         162.*m11*mp2*RSp561e*RSp463e - 1./108.*m11*mp2*pow(RSp561e,2)
          + 1./162.*m11*mp2*RSp461e*RSp563e - 1./162.*m11*mp2*RSp461e*
         RSp463e + 1./54.*m11*mp2*RSp461e*RSp561e - 1./108.*m11*mp2*
         pow(RSp461e,2) + 1./648.*m11*m55*pow(RSp563e,2) + 1./324.*m11*
         m55*RSp463e*RSp563e + 1./216.*m11*m55*pow(RSp463e,2) + 1./324.
         *m11*m55*RSp561e*RSp563e );

      M13p6x23RV +=  + AbV(m45,L)*BbV(mp2,L) * ( 1./324.*m11*m55*
         RSp561e*RSp463e + 1./216.*m11*m55*pow(RSp561e,2) + 1./324.*m11
         *m55*RSp461e*RSp563e + 1./108.*m11*m55*RSp461e*RSp463e + 1./
         108.*m11*m55*RSp461e*RSp561e + 1./72.*m11*m55*pow(RSp461e,2)
          - 1./324.*m11*m45*pow(RSp563e,2) + 1./162.*m11*m45*RSp463e*
         RSp563e - 1./324.*m11*m45*pow(RSp463e,2) - 1./162.*m11*m45*
         RSp561e*RSp563e + 1./162.*m11*m45*RSp561e*RSp463e - 1./108.*
         m11*m45*pow(RSp561e,2) + 1./162.*m11*m45*RSp461e*RSp563e - 1./
         162.*m11*m45*RSp461e*RSp463e + 1./54.*m11*m45*RSp461e*RSp561e
          - 1./108.*m11*m45*pow(RSp461e,2) + 1./216.*m11*m44*pow(
         RSp563e,2) + 1./324.*m11*m44*RSp463e*RSp563e + 1./648.*m11*m44
         *pow(RSp463e,2) + 1./108.*m11*m44*RSp561e*RSp563e + 1./324.*
         m11*m44*RSp561e*RSp463e + 1./72.*m11*m44*pow(RSp561e,2) + 1./
         324.*m11*m44*RSp461e*RSp563e + 1./324.*m11*m44*RSp461e*RSp463e
          + 1./108.*m11*m44*RSp461e*RSp561e + 1./216.*m11*m44*pow(
         RSp461e,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * ( 1./81.*mp2*me2*
         RSp563e*RSe563p - 1./81.*mp2*me2*RSp563e*RSe463p - 1./81.*mp2*
         me2*RSp463e*RSe563p + 1./81.*mp2*me2*RSp463e*RSe463p - 1./81.*
         mp2*me2*RSe561p*RSp563e + 1./81.*mp2*me2*RSe561p*RSp463e + 1./
         81.*mp2*me2*RSe461p*RSp563e - 1./81.*mp2*me2*RSe461p*RSp463e
          - 1./81.*mp2*me2*RSp561e*RSe563p + 1./81.*mp2*me2*RSp561e*
         RSe463p + 1./81.*mp2*me2*RSp561e*RSe561p - 1./81.*mp2*me2*
         RSp561e*RSe461p + 1./81.*mp2*me2*RSp461e*RSe563p - 1./81.*mp2*
         me2*RSp461e*RSe463p - 1./81.*mp2*me2*RSp461e*RSe561p + 1./81.*
         mp2*me2*RSp461e*RSe461p - 1./162.*m55*me2*RSp563e*RSe563p - 1./
         162.*m55*me2*RSp563e*RSe463p - 1./162.*m55*me2*RSp463e*RSe563p
          - 1./54.*m55*me2*RSp463e*RSe463p + 1./162.*m55*me2*RSe561p*
         RSp563e + 1./162.*m55*me2*RSe561p*RSp463e + 1./162.*m55*me2*
         RSe461p*RSp563e + 1./54.*m55*me2*RSe461p*RSp463e + 1./162.*m55
         *me2*RSp561e*RSe563p + 1./162.*m55*me2*RSp561e*RSe463p - 1./
         162.*m55*me2*RSp561e*RSe561p );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * (  - 1./162.*m55*me2
         *RSp561e*RSe461p + 1./162.*m55*me2*RSp461e*RSe563p + 1./54.*
         m55*me2*RSp461e*RSe463p - 1./162.*m55*me2*RSp461e*RSe561p - 1./
         54.*m55*me2*RSp461e*RSe461p + 1./81.*m45*me2*RSp563e*RSe563p
          - 1./81.*m45*me2*RSp563e*RSe463p - 1./81.*m45*me2*RSp463e*
         RSe563p + 1./81.*m45*me2*RSp463e*RSe463p - 1./81.*m45*me2*
         RSe561p*RSp563e + 1./81.*m45*me2*RSe561p*RSp463e + 1./81.*m45*
         me2*RSe461p*RSp563e - 1./81.*m45*me2*RSe461p*RSp463e - 1./81.*
         m45*me2*RSp561e*RSe563p + 1./81.*m45*me2*RSp561e*RSe463p + 1./
         81.*m45*me2*RSp561e*RSe561p - 1./81.*m45*me2*RSp561e*RSe461p
          + 1./81.*m45*me2*RSp461e*RSe563p - 1./81.*m45*me2*RSp461e*
         RSe463p - 1./81.*m45*me2*RSp461e*RSe561p + 1./81.*m45*me2*
         RSp461e*RSe461p - 1./54.*m44*me2*RSp563e*RSe563p - 1./162.*m44
         *me2*RSp563e*RSe463p - 1./162.*m44*me2*RSp463e*RSe563p - 1./
         162.*m44*me2*RSp463e*RSe463p + 1./54.*m44*me2*RSe561p*RSp563e
          + 1./162.*m44*me2*RSe561p*RSp463e );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * ( 1./162.*m44*me2*
         RSe461p*RSp563e + 1./162.*m44*me2*RSe461p*RSp463e + 1./54.*m44
         *me2*RSp561e*RSe563p + 1./162.*m44*me2*RSp561e*RSe463p - 1./54.
         *m44*me2*RSp561e*RSe561p - 1./162.*m44*me2*RSp561e*RSe461p + 1.
         /162.*m44*me2*RSp461e*RSe563p + 1./162.*m44*me2*RSp461e*
         RSe463p - 1./162.*m44*me2*RSp461e*RSe561p - 1./162.*m44*me2*
         RSp461e*RSe461p - 1./54.*m33*me2*RSp563e*RSe563p + 1./54.*m33*
         me2*RSp563e*RSe463p + 1./54.*m33*me2*RSp463e*RSe563p - 1./54.*
         m33*me2*RSp463e*RSe463p - 1./162.*m33*me2*RSe561p*RSp563e + 1./
         162.*m33*me2*RSe561p*RSp463e + 1./162.*m33*me2*RSe461p*RSp563e
          - 1./162.*m33*me2*RSe461p*RSp463e - 1./162.*m33*me2*RSp561e*
         RSe563p + 1./162.*m33*me2*RSp561e*RSe463p - 1./162.*m33*me2*
         RSp561e*RSe561p + 1./162.*m33*me2*RSp561e*RSe461p + 1./162.*
         m33*me2*RSp461e*RSe563p - 1./162.*m33*me2*RSp461e*RSe463p + 1./
         162.*m33*me2*RSp461e*RSe561p - 1./162.*m33*me2*RSp461e*RSe461p
          + 1./108.*m33*m55*RSp563e*RSe563p );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * ( 1./108.*m33*m55*
         RSp563e*RSe463p + 1./108.*m33*m55*RSp463e*RSe563p + 1./36.*m33
         *m55*RSp463e*RSe463p + 1./324.*m33*m55*RSe561p*RSp563e + 1./
         324.*m33*m55*RSe561p*RSp463e + 1./324.*m33*m55*RSe461p*RSp563e
          + 1./108.*m33*m55*RSe461p*RSp463e + 1./324.*m33*m55*RSp561e*
         RSe563p + 1./324.*m33*m55*RSp561e*RSe463p + 1./324.*m33*m55*
         RSp561e*RSe561p + 1./324.*m33*m55*RSp561e*RSe461p + 1./324.*
         m33*m55*RSp461e*RSe563p + 1./108.*m33*m55*RSp461e*RSe463p + 1./
         324.*m33*m55*RSp461e*RSe561p + 1./108.*m33*m55*RSp461e*RSe461p
          - 1./54.*m33*m45*RSp563e*RSe563p + 1./54.*m33*m45*RSp563e*
         RSe463p + 1./54.*m33*m45*RSp463e*RSe563p - 1./54.*m33*m45*
         RSp463e*RSe463p - 1./162.*m33*m45*RSe561p*RSp563e + 1./162.*
         m33*m45*RSe561p*RSp463e + 1./162.*m33*m45*RSe461p*RSp563e - 1./
         162.*m33*m45*RSe461p*RSp463e - 1./162.*m33*m45*RSp561e*RSe563p
          + 1./162.*m33*m45*RSp561e*RSe463p - 1./162.*m33*m45*RSp561e*
         RSe561p );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * ( 1./162.*m33*m45*
         RSp561e*RSe461p + 1./162.*m33*m45*RSp461e*RSe563p - 1./162.*
         m33*m45*RSp461e*RSe463p + 1./162.*m33*m45*RSp461e*RSe561p - 1./
         162.*m33*m45*RSp461e*RSe461p + 1./36.*m33*m44*RSp563e*RSe563p
          + 1./108.*m33*m44*RSp563e*RSe463p + 1./108.*m33*m44*RSp463e*
         RSe563p + 1./108.*m33*m44*RSp463e*RSe463p + 1./108.*m33*m44*
         RSe561p*RSp563e + 1./324.*m33*m44*RSe561p*RSp463e + 1./324.*
         m33*m44*RSe461p*RSp563e + 1./324.*m33*m44*RSe461p*RSp463e + 1./
         108.*m33*m44*RSp561e*RSe563p + 1./324.*m33*m44*RSp561e*RSe463p
          + 1./108.*m33*m44*RSp561e*RSe561p + 1./324.*m33*m44*RSp561e*
         RSe461p + 1./324.*m33*m44*RSp461e*RSe563p + 1./324.*m33*m44*
         RSp461e*RSe463p + 1./324.*m33*m44*RSp461e*RSe561p + 1./324.*
         m33*m44*RSp461e*RSe461p + 1./81.*m13*me2*RSp563e*RSe563p - 1./
         81.*m13*me2*RSp563e*RSe463p - 1./81.*m13*me2*RSp463e*RSe563p
          + 1./81.*m13*me2*RSp463e*RSe463p - 1./81.*m13*me2*RSe561p*
         RSp563e );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * ( 1./81.*m13*me2*
         RSe561p*RSp463e + 1./81.*m13*me2*RSe461p*RSp563e - 1./81.*m13*
         me2*RSe461p*RSp463e - 1./81.*m13*me2*RSp561e*RSe563p + 1./81.*
         m13*me2*RSp561e*RSe463p + 1./81.*m13*me2*RSp561e*RSe561p - 1./
         81.*m13*me2*RSp561e*RSe461p + 1./81.*m13*me2*RSp461e*RSe563p
          - 1./81.*m13*me2*RSp461e*RSe463p - 1./81.*m13*me2*RSp461e*
         RSe561p + 1./81.*m13*me2*RSp461e*RSe461p - 1./162.*m13*m55*
         RSp563e*RSe563p - 1./162.*m13*m55*RSp563e*RSe463p - 1./162.*
         m13*m55*RSp463e*RSe563p - 1./54.*m13*m55*RSp463e*RSe463p + 1./
         162.*m13*m55*RSe561p*RSp563e + 1./162.*m13*m55*RSe561p*RSp463e
          + 1./162.*m13*m55*RSe461p*RSp563e + 1./54.*m13*m55*RSe461p*
         RSp463e + 1./162.*m13*m55*RSp561e*RSe563p + 1./162.*m13*m55*
         RSp561e*RSe463p - 1./162.*m13*m55*RSp561e*RSe561p - 1./162.*
         m13*m55*RSp561e*RSe461p + 1./162.*m13*m55*RSp461e*RSe563p + 1./
         54.*m13*m55*RSp461e*RSe463p - 1./162.*m13*m55*RSp461e*RSe561p
          - 1./54.*m13*m55*RSp461e*RSe461p );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * ( 1./81.*m13*m45*
         RSp563e*RSe563p - 1./81.*m13*m45*RSp563e*RSe463p - 1./81.*m13*
         m45*RSp463e*RSe563p + 1./81.*m13*m45*RSp463e*RSe463p - 1./81.*
         m13*m45*RSe561p*RSp563e + 1./81.*m13*m45*RSe561p*RSp463e + 1./
         81.*m13*m45*RSe461p*RSp563e - 1./81.*m13*m45*RSe461p*RSp463e
          - 1./81.*m13*m45*RSp561e*RSe563p + 1./81.*m13*m45*RSp561e*
         RSe463p + 1./81.*m13*m45*RSp561e*RSe561p - 1./81.*m13*m45*
         RSp561e*RSe461p + 1./81.*m13*m45*RSp461e*RSe563p - 1./81.*m13*
         m45*RSp461e*RSe463p - 1./81.*m13*m45*RSp461e*RSe561p + 1./81.*
         m13*m45*RSp461e*RSe461p - 1./54.*m13*m44*RSp563e*RSe563p - 1./
         162.*m13*m44*RSp563e*RSe463p - 1./162.*m13*m44*RSp463e*RSe563p
          - 1./162.*m13*m44*RSp463e*RSe463p + 1./54.*m13*m44*RSe561p*
         RSp563e + 1./162.*m13*m44*RSe561p*RSp463e + 1./162.*m13*m44*
         RSe461p*RSp563e + 1./162.*m13*m44*RSe461p*RSp463e + 1./54.*m13
         *m44*RSp561e*RSe563p + 1./162.*m13*m44*RSp561e*RSe463p - 1./54.
         *m13*m44*RSp561e*RSe561p );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * (  - 1./162.*m13*m44
         *RSp561e*RSe461p + 1./162.*m13*m44*RSp461e*RSe563p + 1./162.*
         m13*m44*RSp461e*RSe463p - 1./162.*m13*m44*RSp461e*RSe561p - 1./
         162.*m13*m44*RSp461e*RSe461p - 1./162.*m11*me2*RSp563e*RSe563p
          + 1./162.*m11*me2*RSp563e*RSe463p + 1./162.*m11*me2*RSp463e*
         RSe563p - 1./162.*m11*me2*RSp463e*RSe463p - 1./162.*m11*me2*
         RSe561p*RSp563e + 1./162.*m11*me2*RSe561p*RSp463e + 1./162.*
         m11*me2*RSe461p*RSp563e - 1./162.*m11*me2*RSe461p*RSp463e - 1./
         162.*m11*me2*RSp561e*RSe563p + 1./162.*m11*me2*RSp561e*RSe463p
          - 1./54.*m11*me2*RSp561e*RSe561p + 1./54.*m11*me2*RSp561e*
         RSe461p + 1./162.*m11*me2*RSp461e*RSe563p - 1./162.*m11*me2*
         RSp461e*RSe463p + 1./54.*m11*me2*RSp461e*RSe561p - 1./54.*m11*
         me2*RSp461e*RSe461p + 1./324.*m11*m55*RSp563e*RSe563p + 1./324.
         *m11*m55*RSp563e*RSe463p + 1./324.*m11*m55*RSp463e*RSe563p + 1.
         /108.*m11*m55*RSp463e*RSe463p + 1./324.*m11*m55*RSe561p*
         RSp563e );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * ( 1./324.*m11*m55*
         RSe561p*RSp463e + 1./324.*m11*m55*RSe461p*RSp563e + 1./108.*
         m11*m55*RSe461p*RSp463e + 1./324.*m11*m55*RSp561e*RSe563p + 1./
         324.*m11*m55*RSp561e*RSe463p + 1./108.*m11*m55*RSp561e*RSe561p
          + 1./108.*m11*m55*RSp561e*RSe461p + 1./324.*m11*m55*RSp461e*
         RSe563p + 1./108.*m11*m55*RSp461e*RSe463p + 1./108.*m11*m55*
         RSp461e*RSe561p + 1./36.*m11*m55*RSp461e*RSe461p - 1./162.*m11
         *m45*RSp563e*RSe563p + 1./162.*m11*m45*RSp563e*RSe463p + 1./
         162.*m11*m45*RSp463e*RSe563p - 1./162.*m11*m45*RSp463e*RSe463p
          - 1./162.*m11*m45*RSe561p*RSp563e + 1./162.*m11*m45*RSe561p*
         RSp463e + 1./162.*m11*m45*RSe461p*RSp563e - 1./162.*m11*m45*
         RSe461p*RSp463e - 1./162.*m11*m45*RSp561e*RSe563p + 1./162.*
         m11*m45*RSp561e*RSe463p - 1./54.*m11*m45*RSp561e*RSe561p + 1./
         54.*m11*m45*RSp561e*RSe461p + 1./162.*m11*m45*RSp461e*RSe563p
          - 1./162.*m11*m45*RSp461e*RSe463p + 1./54.*m11*m45*RSp461e*
         RSe561p );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * (  - 1./54.*m11*m45*
         RSp461e*RSe461p + 1./108.*m11*m44*RSp563e*RSe563p + 1./324.*
         m11*m44*RSp563e*RSe463p + 1./324.*m11*m44*RSp463e*RSe563p + 1./
         324.*m11*m44*RSp463e*RSe463p + 1./108.*m11*m44*RSe561p*RSp563e
          + 1./324.*m11*m44*RSe561p*RSp463e + 1./324.*m11*m44*RSe461p*
         RSp563e + 1./324.*m11*m44*RSe461p*RSp463e + 1./108.*m11*m44*
         RSp561e*RSe563p + 1./324.*m11*m44*RSp561e*RSe463p + 1./36.*m11
         *m44*RSp561e*RSe561p + 1./108.*m11*m44*RSp561e*RSe461p + 1./
         324.*m11*m44*RSp461e*RSe563p + 1./324.*m11*m44*RSp461e*RSe463p
          + 1./108.*m11*m44*RSp461e*RSe561p + 1./108.*m11*m44*RSp461e*
         RSe461p );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,L) * ( 1./162.*pow(me2,2)*
         pow(RSe563p,2) - 1./81.*pow(me2,2)*RSe463p*RSe563p + 1./162.*
         pow(me2,2)*pow(RSe463p,2) - 1./81.*pow(me2,2)*RSe561p*RSe563p
          + 1./81.*pow(me2,2)*RSe561p*RSe463p + 1./162.*pow(me2,2)*pow(
         RSe561p,2) + 1./81.*pow(me2,2)*RSe461p*RSe563p - 1./81.*pow(
         me2,2)*RSe461p*RSe463p - 1./81.*pow(me2,2)*RSe461p*RSe561p + 1.
         /162.*pow(me2,2)*pow(RSe461p,2) - 1./324.*m55*me2*pow(
         RSe563p,2) - 1./162.*m55*me2*RSe463p*RSe563p - 1./108.*m55*me2
         *pow(RSe463p,2) + 1./162.*m55*me2*RSe561p*RSe563p + 1./162.*
         m55*me2*RSe561p*RSe463p - 1./324.*m55*me2*pow(RSe561p,2) + 1./
         162.*m55*me2*RSe461p*RSe563p + 1./54.*m55*me2*RSe461p*RSe463p
          - 1./162.*m55*me2*RSe461p*RSe561p - 1./108.*m55*me2*pow(
         RSe461p,2) + 1./162.*m45*me2*pow(RSe563p,2) - 1./81.*m45*me2*
         RSe463p*RSe563p + 1./162.*m45*me2*pow(RSe463p,2) - 1./81.*m45*
         me2*RSe561p*RSe563p + 1./81.*m45*me2*RSe561p*RSe463p + 1./162.
         *m45*me2*pow(RSe561p,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,L) * ( 1./81.*m45*me2*RSe461p
         *RSe563p - 1./81.*m45*me2*RSe461p*RSe463p - 1./81.*m45*me2*
         RSe461p*RSe561p + 1./162.*m45*me2*pow(RSe461p,2) - 1./108.*m44
         *me2*pow(RSe563p,2) - 1./162.*m44*me2*RSe463p*RSe563p - 1./324.
         *m44*me2*pow(RSe463p,2) + 1./54.*m44*me2*RSe561p*RSe563p + 1./
         162.*m44*me2*RSe561p*RSe463p - 1./108.*m44*me2*pow(RSe561p,2)
          + 1./162.*m44*me2*RSe461p*RSe563p + 1./162.*m44*me2*RSe461p*
         RSe463p - 1./162.*m44*me2*RSe461p*RSe561p - 1./324.*m44*me2*
         pow(RSe461p,2) - 1./108.*m33*me2*pow(RSe563p,2) + 1./54.*m33*
         me2*RSe463p*RSe563p - 1./108.*m33*me2*pow(RSe463p,2) - 1./162.
         *m33*me2*RSe561p*RSe563p + 1./162.*m33*me2*RSe561p*RSe463p - 1.
         /324.*m33*me2*pow(RSe561p,2) + 1./162.*m33*me2*RSe461p*RSe563p
          - 1./162.*m33*me2*RSe461p*RSe463p + 1./162.*m33*me2*RSe461p*
         RSe561p - 1./324.*m33*me2*pow(RSe461p,2) + 1./216.*m33*m55*
         pow(RSe563p,2) + 1./108.*m33*m55*RSe463p*RSe563p + 1./72.*m33*
         m55*pow(RSe463p,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,L) * ( 1./324.*m33*m55*
         RSe561p*RSe563p + 1./324.*m33*m55*RSe561p*RSe463p + 1./648.*
         m33*m55*pow(RSe561p,2) + 1./324.*m33*m55*RSe461p*RSe563p + 1./
         108.*m33*m55*RSe461p*RSe463p + 1./324.*m33*m55*RSe461p*RSe561p
          + 1./216.*m33*m55*pow(RSe461p,2) - 1./108.*m33*m45*pow(
         RSe563p,2) + 1./54.*m33*m45*RSe463p*RSe563p - 1./108.*m33*m45*
         pow(RSe463p,2) - 1./162.*m33*m45*RSe561p*RSe563p + 1./162.*m33
         *m45*RSe561p*RSe463p - 1./324.*m33*m45*pow(RSe561p,2) + 1./162.
         *m33*m45*RSe461p*RSe563p - 1./162.*m33*m45*RSe461p*RSe463p + 1.
         /162.*m33*m45*RSe461p*RSe561p - 1./324.*m33*m45*pow(RSe461p,2)
          + 1./72.*m33*m44*pow(RSe563p,2) + 1./108.*m33*m44*RSe463p*
         RSe563p + 1./216.*m33*m44*pow(RSe463p,2) + 1./108.*m33*m44*
         RSe561p*RSe563p + 1./324.*m33*m44*RSe561p*RSe463p + 1./216.*
         m33*m44*pow(RSe561p,2) + 1./324.*m33*m44*RSe461p*RSe563p + 1./
         324.*m33*m44*RSe461p*RSe463p + 1./324.*m33*m44*RSe461p*RSe561p
          + 1./648.*m33*m44*pow(RSe461p,2) );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,L) * ( 1./162.*m13*me2*pow(
         RSe563p,2) - 1./81.*m13*me2*RSe463p*RSe563p + 1./162.*m13*me2*
         pow(RSe463p,2) - 1./81.*m13*me2*RSe561p*RSe563p + 1./81.*m13*
         me2*RSe561p*RSe463p + 1./162.*m13*me2*pow(RSe561p,2) + 1./81.*
         m13*me2*RSe461p*RSe563p - 1./81.*m13*me2*RSe461p*RSe463p - 1./
         81.*m13*me2*RSe461p*RSe561p + 1./162.*m13*me2*pow(RSe461p,2)
          - 1./324.*m13*m55*pow(RSe563p,2) - 1./162.*m13*m55*RSe463p*
         RSe563p - 1./108.*m13*m55*pow(RSe463p,2) + 1./162.*m13*m55*
         RSe561p*RSe563p + 1./162.*m13*m55*RSe561p*RSe463p - 1./324.*
         m13*m55*pow(RSe561p,2) + 1./162.*m13*m55*RSe461p*RSe563p + 1./
         54.*m13*m55*RSe461p*RSe463p - 1./162.*m13*m55*RSe461p*RSe561p
          - 1./108.*m13*m55*pow(RSe461p,2) + 1./162.*m13*m45*pow(
         RSe563p,2) - 1./81.*m13*m45*RSe463p*RSe563p + 1./162.*m13*m45*
         pow(RSe463p,2) - 1./81.*m13*m45*RSe561p*RSe563p + 1./81.*m13*
         m45*RSe561p*RSe463p + 1./162.*m13*m45*pow(RSe561p,2) + 1./81.*
         m13*m45*RSe461p*RSe563p );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,L) * (  - 1./81.*m13*m45*
         RSe461p*RSe463p - 1./81.*m13*m45*RSe461p*RSe561p + 1./162.*m13
         *m45*pow(RSe461p,2) - 1./108.*m13*m44*pow(RSe563p,2) - 1./162.
         *m13*m44*RSe463p*RSe563p - 1./324.*m13*m44*pow(RSe463p,2) + 1./
         54.*m13*m44*RSe561p*RSe563p + 1./162.*m13*m44*RSe561p*RSe463p
          - 1./108.*m13*m44*pow(RSe561p,2) + 1./162.*m13*m44*RSe461p*
         RSe563p + 1./162.*m13*m44*RSe461p*RSe463p - 1./162.*m13*m44*
         RSe461p*RSe561p - 1./324.*m13*m44*pow(RSe461p,2) - 1./324.*m11
         *me2*pow(RSe563p,2) + 1./162.*m11*me2*RSe463p*RSe563p - 1./324.
         *m11*me2*pow(RSe463p,2) - 1./162.*m11*me2*RSe561p*RSe563p + 1./
         162.*m11*me2*RSe561p*RSe463p - 1./108.*m11*me2*pow(RSe561p,2)
          + 1./162.*m11*me2*RSe461p*RSe563p - 1./162.*m11*me2*RSe461p*
         RSe463p + 1./54.*m11*me2*RSe461p*RSe561p - 1./108.*m11*me2*
         pow(RSe461p,2) + 1./648.*m11*m55*pow(RSe563p,2) + 1./324.*m11*
         m55*RSe463p*RSe563p + 1./216.*m11*m55*pow(RSe463p,2) + 1./324.
         *m11*m55*RSe561p*RSe563p );

      M13p6x23RV +=  + AbV(m45,L)*BbV(me2,L) * ( 1./324.*m11*m55*
         RSe561p*RSe463p + 1./216.*m11*m55*pow(RSe561p,2) + 1./324.*m11
         *m55*RSe461p*RSe563p + 1./108.*m11*m55*RSe461p*RSe463p + 1./
         108.*m11*m55*RSe461p*RSe561p + 1./72.*m11*m55*pow(RSe461p,2)
          - 1./324.*m11*m45*pow(RSe563p,2) + 1./162.*m11*m45*RSe463p*
         RSe563p - 1./324.*m11*m45*pow(RSe463p,2) - 1./162.*m11*m45*
         RSe561p*RSe563p + 1./162.*m11*m45*RSe561p*RSe463p - 1./108.*
         m11*m45*pow(RSe561p,2) + 1./162.*m11*m45*RSe461p*RSe563p - 1./
         162.*m11*m45*RSe461p*RSe463p + 1./54.*m11*m45*RSe461p*RSe561p
          - 1./108.*m11*m45*pow(RSe461p,2) + 1./216.*m11*m44*pow(
         RSe563p,2) + 1./324.*m11*m44*RSe463p*RSe563p + 1./648.*m11*m44
         *pow(RSe463p,2) + 1./108.*m11*m44*RSe561p*RSe563p + 1./324.*
         m11*m44*RSe561p*RSe463p + 1./72.*m11*m44*pow(RSe561p,2) + 1./
         324.*m11*m44*RSe461p*RSe563p + 1./324.*m11*m44*RSe461p*RSe463p
          + 1./108.*m11*m44*RSe461p*RSe561p + 1./216.*m11*m44*pow(
         RSe461p,2) );

      M13p6x23RV +=  + AbV(m46,L)*AbV(mp2,L) * (  - 1./81.*mp2*RSp563e*
         RS345pe + 1./81.*mp2*RSp563e*RS356pe + 1./81.*mp2*RSp563e*
         RSe563p - 1./81.*mp2*RSp563e*RSe453p + 1./81.*mp2*pow(
         RSp563e,2) + 1./81.*mp2*RSp453e*RS345pe - 1./81.*mp2*RSp453e*
         RS356pe - 1./81.*mp2*RSp453e*RSe563p + 1./81.*mp2*RSp453e*
         RSe453p - 2./81.*mp2*RSp453e*RSp563e + 1./81.*mp2*pow(
         RSp453e,2) + 1./81.*mp2*RS145pe*RSp563e - 1./81.*mp2*RS145pe*
         RSp453e - 1./81.*mp2*RS156pe*RSp563e + 1./81.*mp2*RS156pe*
         RSp453e - 1./81.*mp2*RSe561p*RSp563e + 1./81.*mp2*RSe561p*
         RSp453e + 1./81.*mp2*RSe451p*RSp563e - 1./81.*mp2*RSe451p*
         RSp453e + 1./81.*mp2*RSp561e*RS345pe - 1./81.*mp2*RSp561e*
         RS356pe - 1./81.*mp2*RSp561e*RSe563p + 1./81.*mp2*RSp561e*
         RSe453p - 2./81.*mp2*RSp561e*RSp563e + 2./81.*mp2*RSp561e*
         RSp453e - 1./81.*mp2*RSp561e*RS145pe + 1./81.*mp2*RSp561e*
         RS156pe + 1./81.*mp2*RSp561e*RSe561p - 1./81.*mp2*RSp561e*
         RSe451p );

      M13p6x23RV +=  + AbV(m46,L)*AbV(mp2,L) * ( 1./81.*mp2*pow(
         RSp561e,2) - 1./81.*mp2*RSp451e*RS345pe + 1./81.*mp2*RSp451e*
         RS356pe + 1./81.*mp2*RSp451e*RSe563p - 1./81.*mp2*RSp451e*
         RSe453p + 2./81.*mp2*RSp451e*RSp563e - 2./81.*mp2*RSp451e*
         RSp453e + 1./81.*mp2*RSp451e*RS145pe - 1./81.*mp2*RSp451e*
         RS156pe - 1./81.*mp2*RSp451e*RSe561p + 1./81.*mp2*RSp451e*
         RSe451p - 2./81.*mp2*RSp451e*RSp561e + 1./81.*mp2*pow(
         RSp451e,2) - 1./162.*m66*RSp563e*RS345pe - 1./162.*m66*RSp563e
         *RS356pe - 1./162.*m66*RSp563e*RSe563p - 1./162.*m66*RSp563e*
         RSe453p - 1./324.*m66*pow(RSp563e,2) - 1./54.*m66*RSp453e*
         RS345pe - 1./162.*m66*RSp453e*RS356pe - 1./162.*m66*RSp453e*
         RSe563p - 1./54.*m66*RSp453e*RSe453p - 1./162.*m66*RSp453e*
         RSp563e - 1./108.*m66*pow(RSp453e,2) + 1./162.*m66*RS145pe*
         RSp563e + 1./54.*m66*RS145pe*RSp453e + 1./162.*m66*RS156pe*
         RSp563e + 1./162.*m66*RS156pe*RSp453e + 1./162.*m66*RSe561p*
         RSp563e );

      M13p6x23RV +=  + AbV(m46,L)*AbV(mp2,L) * ( 1./162.*m66*RSe561p*
         RSp453e + 1./162.*m66*RSe451p*RSp563e + 1./54.*m66*RSe451p*
         RSp453e + 1./162.*m66*RSp561e*RS345pe + 1./162.*m66*RSp561e*
         RS356pe + 1./162.*m66*RSp561e*RSe563p + 1./162.*m66*RSp561e*
         RSe453p + 1./162.*m66*RSp561e*RSp563e + 1./162.*m66*RSp561e*
         RSp453e - 1./162.*m66*RSp561e*RS145pe - 1./162.*m66*RSp561e*
         RS156pe - 1./162.*m66*RSp561e*RSe561p - 1./162.*m66*RSp561e*
         RSe451p - 1./324.*m66*pow(RSp561e,2) + 1./54.*m66*RSp451e*
         RS345pe + 1./162.*m66*RSp451e*RS356pe + 1./162.*m66*RSp451e*
         RSe563p + 1./54.*m66*RSp451e*RSe453p + 1./162.*m66*RSp451e*
         RSp563e + 1./54.*m66*RSp451e*RSp453e - 1./54.*m66*RSp451e*
         RS145pe - 1./162.*m66*RSp451e*RS156pe - 1./162.*m66*RSp451e*
         RSe561p - 1./54.*m66*RSp451e*RSe451p - 1./162.*m66*RSp451e*
         RSp561e - 1./108.*m66*pow(RSp451e,2) - 1./81.*m46*RSp563e*
         RS345pe + 1./81.*m46*RSp563e*RS356pe + 1./81.*m46*RSp563e*
         RSe563p );

      M13p6x23RV +=  + AbV(m46,L)*AbV(mp2,L) * (  - 1./81.*m46*RSp563e*
         RSe453p + 1./162.*m46*pow(RSp563e,2) + 1./81.*m46*RSp453e*
         RS345pe - 1./81.*m46*RSp453e*RS356pe - 1./81.*m46*RSp453e*
         RSe563p + 1./81.*m46*RSp453e*RSe453p - 1./81.*m46*RSp453e*
         RSp563e + 1./162.*m46*pow(RSp453e,2) + 1./81.*m46*RS145pe*
         RSp563e - 1./81.*m46*RS145pe*RSp453e - 1./81.*m46*RS156pe*
         RSp563e + 1./81.*m46*RS156pe*RSp453e - 1./81.*m46*RSe561p*
         RSp563e + 1./81.*m46*RSe561p*RSp453e + 1./81.*m46*RSe451p*
         RSp563e - 1./81.*m46*RSe451p*RSp453e + 1./81.*m46*RSp561e*
         RS345pe - 1./81.*m46*RSp561e*RS356pe - 1./81.*m46*RSp561e*
         RSe563p + 1./81.*m46*RSp561e*RSe453p - 1./81.*m46*RSp561e*
         RSp563e + 1./81.*m46*RSp561e*RSp453e - 1./81.*m46*RSp561e*
         RS145pe + 1./81.*m46*RSp561e*RS156pe + 1./81.*m46*RSp561e*
         RSe561p - 1./81.*m46*RSp561e*RSe451p + 1./162.*m46*pow(
         RSp561e,2) - 1./81.*m46*RSp451e*RS345pe + 1./81.*m46*RSp451e*
         RS356pe );

      M13p6x23RV +=  + AbV(m46,L)*AbV(mp2,L) * ( 1./81.*m46*RSp451e*
         RSe563p - 1./81.*m46*RSp451e*RSe453p + 1./81.*m46*RSp451e*
         RSp563e - 1./81.*m46*RSp451e*RSp453e + 1./81.*m46*RSp451e*
         RS145pe - 1./81.*m46*RSp451e*RS156pe - 1./81.*m46*RSp451e*
         RSe561p + 1./81.*m46*RSp451e*RSe451p - 1./81.*m46*RSp451e*
         RSp561e + 1./162.*m46*pow(RSp451e,2) - 1./162.*m44*RSp563e*
         RS345pe - 1./54.*m44*RSp563e*RS356pe - 1./54.*m44*RSp563e*
         RSe563p - 1./162.*m44*RSp563e*RSe453p - 1./108.*m44*pow(
         RSp563e,2) - 1./162.*m44*RSp453e*RS345pe - 1./162.*m44*RSp453e
         *RS356pe - 1./162.*m44*RSp453e*RSe563p - 1./162.*m44*RSp453e*
         RSe453p - 1./162.*m44*RSp453e*RSp563e - 1./324.*m44*pow(
         RSp453e,2) + 1./162.*m44*RS145pe*RSp563e + 1./162.*m44*RS145pe
         *RSp453e + 1./54.*m44*RS156pe*RSp563e + 1./162.*m44*RS156pe*
         RSp453e + 1./54.*m44*RSe561p*RSp563e + 1./162.*m44*RSe561p*
         RSp453e + 1./162.*m44*RSe451p*RSp563e + 1./162.*m44*RSe451p*
         RSp453e );

      M13p6x23RV +=  + AbV(m46,L)*AbV(mp2,L) * ( 1./162.*m44*RSp561e*
         RS345pe + 1./54.*m44*RSp561e*RS356pe + 1./54.*m44*RSp561e*
         RSe563p + 1./162.*m44*RSp561e*RSe453p + 1./54.*m44*RSp561e*
         RSp563e + 1./162.*m44*RSp561e*RSp453e - 1./162.*m44*RSp561e*
         RS145pe - 1./54.*m44*RSp561e*RS156pe - 1./54.*m44*RSp561e*
         RSe561p - 1./162.*m44*RSp561e*RSe451p - 1./108.*m44*pow(
         RSp561e,2) + 1./162.*m44*RSp451e*RS345pe + 1./162.*m44*RSp451e
         *RS356pe + 1./162.*m44*RSp451e*RSe563p + 1./162.*m44*RSp451e*
         RSe453p + 1./162.*m44*RSp451e*RSp563e + 1./162.*m44*RSp451e*
         RSp453e - 1./162.*m44*RSp451e*RS145pe - 1./162.*m44*RSp451e*
         RS156pe - 1./162.*m44*RSp451e*RSe561p - 1./162.*m44*RSp451e*
         RSe451p - 1./162.*m44*RSp451e*RSp561e - 1./324.*m44*pow(
         RSp451e,2) + 1./54.*m33*RSp563e*RS345pe - 1./54.*m33*RSp563e*
         RS356pe - 1./54.*m33*RSp563e*RSe563p + 1./54.*m33*RSp563e*
         RSe453p - 1./108.*m33*pow(RSp563e,2) - 1./54.*m33*RSp453e*
         RS345pe );

      M13p6x23RV +=  + AbV(m46,L)*AbV(mp2,L) * ( 1./54.*m33*RSp453e*
         RS356pe + 1./54.*m33*RSp453e*RSe563p - 1./54.*m33*RSp453e*
         RSe453p + 1./54.*m33*RSp453e*RSp563e - 1./108.*m33*pow(
         RSp453e,2) + 1./162.*m33*RS145pe*RSp563e - 1./162.*m33*RS145pe
         *RSp453e - 1./162.*m33*RS156pe*RSp563e + 1./162.*m33*RS156pe*
         RSp453e - 1./162.*m33*RSe561p*RSp563e + 1./162.*m33*RSe561p*
         RSp453e + 1./162.*m33*RSe451p*RSp563e - 1./162.*m33*RSe451p*
         RSp453e + 1./162.*m33*RSp561e*RS345pe - 1./162.*m33*RSp561e*
         RS356pe - 1./162.*m33*RSp561e*RSe563p + 1./162.*m33*RSp561e*
         RSe453p - 1./162.*m33*RSp561e*RSp563e + 1./162.*m33*RSp561e*
         RSp453e + 1./162.*m33*RSp561e*RS145pe - 1./162.*m33*RSp561e*
         RS156pe - 1./162.*m33*RSp561e*RSe561p + 1./162.*m33*RSp561e*
         RSe451p - 1./324.*m33*pow(RSp561e,2) - 1./162.*m33*RSp451e*
         RS345pe + 1./162.*m33*RSp451e*RS356pe + 1./162.*m33*RSp451e*
         RSe563p - 1./162.*m33*RSp451e*RSe453p + 1./162.*m33*RSp451e*
         RSp563e );

      M13p6x23RV +=  + AbV(m46,L)*AbV(mp2,L) * (  - 1./162.*m33*RSp451e
         *RSp453e - 1./162.*m33*RSp451e*RS145pe + 1./162.*m33*RSp451e*
         RS156pe + 1./162.*m33*RSp451e*RSe561p - 1./162.*m33*RSp451e*
         RSe451p + 1./162.*m33*RSp451e*RSp561e - 1./324.*m33*pow(
         RSp451e,2) - 1./81.*m13*RSp563e*RS345pe + 1./81.*m13*RSp563e*
         RS356pe + 1./81.*m13*RSp563e*RSe563p - 1./81.*m13*RSp563e*
         RSe453p + 1./162.*m13*pow(RSp563e,2) + 1./81.*m13*RSp453e*
         RS345pe - 1./81.*m13*RSp453e*RS356pe - 1./81.*m13*RSp453e*
         RSe563p + 1./81.*m13*RSp453e*RSe453p - 1./81.*m13*RSp453e*
         RSp563e + 1./162.*m13*pow(RSp453e,2) + 1./81.*m13*RS145pe*
         RSp563e - 1./81.*m13*RS145pe*RSp453e - 1./81.*m13*RS156pe*
         RSp563e + 1./81.*m13*RS156pe*RSp453e - 1./81.*m13*RSe561p*
         RSp563e + 1./81.*m13*RSe561p*RSp453e + 1./81.*m13*RSe451p*
         RSp563e - 1./81.*m13*RSe451p*RSp453e + 1./81.*m13*RSp561e*
         RS345pe - 1./81.*m13*RSp561e*RS356pe - 1./81.*m13*RSp561e*
         RSe563p );

      M13p6x23RV +=  + AbV(m46,L)*AbV(mp2,L) * ( 1./81.*m13*RSp561e*
         RSe453p - 1./81.*m13*RSp561e*RSp563e + 1./81.*m13*RSp561e*
         RSp453e - 1./81.*m13*RSp561e*RS145pe + 1./81.*m13*RSp561e*
         RS156pe + 1./81.*m13*RSp561e*RSe561p - 1./81.*m13*RSp561e*
         RSe451p + 1./162.*m13*pow(RSp561e,2) - 1./81.*m13*RSp451e*
         RS345pe + 1./81.*m13*RSp451e*RS356pe + 1./81.*m13*RSp451e*
         RSe563p - 1./81.*m13*RSp451e*RSe453p + 1./81.*m13*RSp451e*
         RSp563e - 1./81.*m13*RSp451e*RSp453e + 1./81.*m13*RSp451e*
         RS145pe - 1./81.*m13*RSp451e*RS156pe - 1./81.*m13*RSp451e*
         RSe561p + 1./81.*m13*RSp451e*RSe451p - 1./81.*m13*RSp451e*
         RSp561e + 1./162.*m13*pow(RSp451e,2) + 1./162.*m11*RSp563e*
         RS345pe - 1./162.*m11*RSp563e*RS356pe - 1./162.*m11*RSp563e*
         RSe563p + 1./162.*m11*RSp563e*RSe453p - 1./324.*m11*pow(
         RSp563e,2) - 1./162.*m11*RSp453e*RS345pe + 1./162.*m11*RSp453e
         *RS356pe + 1./162.*m11*RSp453e*RSe563p - 1./162.*m11*RSp453e*
         RSe453p );

      M13p6x23RV +=  + AbV(m46,L)*AbV(mp2,L) * ( 1./162.*m11*RSp453e*
         RSp563e - 1./324.*m11*pow(RSp453e,2) + 1./162.*m11*RS145pe*
         RSp563e - 1./162.*m11*RS145pe*RSp453e - 1./162.*m11*RS156pe*
         RSp563e + 1./162.*m11*RS156pe*RSp453e - 1./162.*m11*RSe561p*
         RSp563e + 1./162.*m11*RSe561p*RSp453e + 1./162.*m11*RSe451p*
         RSp563e - 1./162.*m11*RSe451p*RSp453e + 1./162.*m11*RSp561e*
         RS345pe - 1./162.*m11*RSp561e*RS356pe - 1./162.*m11*RSp561e*
         RSe563p + 1./162.*m11*RSp561e*RSe453p - 1./162.*m11*RSp561e*
         RSp563e + 1./162.*m11*RSp561e*RSp453e + 1./54.*m11*RSp561e*
         RS145pe - 1./54.*m11*RSp561e*RS156pe - 1./54.*m11*RSp561e*
         RSe561p + 1./54.*m11*RSp561e*RSe451p - 1./108.*m11*pow(
         RSp561e,2) - 1./162.*m11*RSp451e*RS345pe + 1./162.*m11*RSp451e
         *RS356pe + 1./162.*m11*RSp451e*RSe563p - 1./162.*m11*RSp451e*
         RSe453p + 1./162.*m11*RSp451e*RSp563e - 1./162.*m11*RSp451e*
         RSp453e - 1./54.*m11*RSp451e*RS145pe + 1./54.*m11*RSp451e*
         RS156pe );

      M13p6x23RV +=  + AbV(m46,L)*AbV(mp2,L) * ( 1./54.*m11*RSp451e*
         RSe561p - 1./54.*m11*RSp451e*RSe451p + 1./54.*m11*RSp451e*
         RSp561e - 1./108.*m11*pow(RSp451e,2) );

      M13p6x23RV +=  + AbV(m46,L)*AbV(me2,L) * (  - 1./81.*me2*RSe563p*
         RS345pe + 1./81.*me2*RSe563p*RS356pe + 1./81.*me2*pow(
         RSe563p,2) + 1./81.*me2*RSe453p*RS345pe - 1./81.*me2*RSe453p*
         RS356pe - 2./81.*me2*RSe453p*RSe563p + 1./81.*me2*pow(
         RSe453p,2) + 1./81.*me2*RSp563e*RSe563p - 1./81.*me2*RSp563e*
         RSe453p - 1./81.*me2*RSp453e*RSe563p + 1./81.*me2*RSp453e*
         RSe453p + 1./81.*me2*RS145pe*RSe563p - 1./81.*me2*RS145pe*
         RSe453p - 1./81.*me2*RS156pe*RSe563p + 1./81.*me2*RS156pe*
         RSe453p + 1./81.*me2*RSe561p*RS345pe - 1./81.*me2*RSe561p*
         RS356pe - 2./81.*me2*RSe561p*RSe563p + 2./81.*me2*RSe561p*
         RSe453p - 1./81.*me2*RSe561p*RSp563e + 1./81.*me2*RSe561p*
         RSp453e - 1./81.*me2*RSe561p*RS145pe + 1./81.*me2*RSe561p*
         RS156pe + 1./81.*me2*pow(RSe561p,2) - 1./81.*me2*RSe451p*
         RS345pe + 1./81.*me2*RSe451p*RS356pe + 2./81.*me2*RSe451p*
         RSe563p - 2./81.*me2*RSe451p*RSe453p + 1./81.*me2*RSe451p*
         RSp563e );

      M13p6x23RV +=  + AbV(m46,L)*AbV(me2,L) * (  - 1./81.*me2*RSe451p*
         RSp453e + 1./81.*me2*RSe451p*RS145pe - 1./81.*me2*RSe451p*
         RS156pe - 2./81.*me2*RSe451p*RSe561p + 1./81.*me2*pow(
         RSe451p,2) - 1./81.*me2*RSp561e*RSe563p + 1./81.*me2*RSp561e*
         RSe453p + 1./81.*me2*RSp561e*RSe561p - 1./81.*me2*RSp561e*
         RSe451p + 1./81.*me2*RSp451e*RSe563p - 1./81.*me2*RSp451e*
         RSe453p - 1./81.*me2*RSp451e*RSe561p + 1./81.*me2*RSp451e*
         RSe451p - 1./162.*m66*RSe563p*RS345pe - 1./162.*m66*RSe563p*
         RS356pe - 1./324.*m66*pow(RSe563p,2) - 1./54.*m66*RSe453p*
         RS345pe - 1./162.*m66*RSe453p*RS356pe - 1./162.*m66*RSe453p*
         RSe563p - 1./108.*m66*pow(RSe453p,2) + 1./162.*m66*RS145pe*
         RSe563p + 1./54.*m66*RS145pe*RSe453p + 1./162.*m66*RS156pe*
         RSe563p + 1./162.*m66*RS156pe*RSe453p + 1./162.*m66*RSe561p*
         RS345pe + 1./162.*m66*RSe561p*RS356pe + 1./162.*m66*RSe561p*
         RSe563p + 1./162.*m66*RSe561p*RSe453p - 1./162.*m66*RSe561p*
         RS145pe );

      M13p6x23RV +=  + AbV(m46,L)*AbV(me2,L) * (  - 1./162.*m66*RSe561p
         *RS156pe - 1./324.*m66*pow(RSe561p,2) + 1./54.*m66*RSe451p*
         RS345pe + 1./162.*m66*RSe451p*RS356pe + 1./162.*m66*RSe451p*
         RSe563p + 1./54.*m66*RSe451p*RSe453p - 1./54.*m66*RSe451p*
         RS145pe - 1./162.*m66*RSe451p*RS156pe - 1./162.*m66*RSe451p*
         RSe561p - 1./108.*m66*pow(RSe451p,2) - 1./81.*m46*RSe563p*
         RS345pe + 1./81.*m46*RSe563p*RS356pe + 1./162.*m46*pow(
         RSe563p,2) + 1./81.*m46*RSe453p*RS345pe - 1./81.*m46*RSe453p*
         RS356pe - 1./81.*m46*RSe453p*RSe563p + 1./162.*m46*pow(
         RSe453p,2) + 1./81.*m46*RS145pe*RSe563p - 1./81.*m46*RS145pe*
         RSe453p - 1./81.*m46*RS156pe*RSe563p + 1./81.*m46*RS156pe*
         RSe453p + 1./81.*m46*RSe561p*RS345pe - 1./81.*m46*RSe561p*
         RS356pe - 1./81.*m46*RSe561p*RSe563p + 1./81.*m46*RSe561p*
         RSe453p - 1./81.*m46*RSe561p*RS145pe + 1./81.*m46*RSe561p*
         RS156pe + 1./162.*m46*pow(RSe561p,2) - 1./81.*m46*RSe451p*
         RS345pe );

      M13p6x23RV +=  + AbV(m46,L)*AbV(me2,L) * ( 1./81.*m46*RSe451p*
         RS356pe + 1./81.*m46*RSe451p*RSe563p - 1./81.*m46*RSe451p*
         RSe453p + 1./81.*m46*RSe451p*RS145pe - 1./81.*m46*RSe451p*
         RS156pe - 1./81.*m46*RSe451p*RSe561p + 1./162.*m46*pow(
         RSe451p,2) - 1./162.*m44*RSe563p*RS345pe - 1./54.*m44*RSe563p*
         RS356pe - 1./108.*m44*pow(RSe563p,2) - 1./162.*m44*RSe453p*
         RS345pe - 1./162.*m44*RSe453p*RS356pe - 1./162.*m44*RSe453p*
         RSe563p - 1./324.*m44*pow(RSe453p,2) + 1./162.*m44*RS145pe*
         RSe563p + 1./162.*m44*RS145pe*RSe453p + 1./54.*m44*RS156pe*
         RSe563p + 1./162.*m44*RS156pe*RSe453p + 1./162.*m44*RSe561p*
         RS345pe + 1./54.*m44*RSe561p*RS356pe + 1./54.*m44*RSe561p*
         RSe563p + 1./162.*m44*RSe561p*RSe453p - 1./162.*m44*RSe561p*
         RS145pe - 1./54.*m44*RSe561p*RS156pe - 1./108.*m44*pow(
         RSe561p,2) + 1./162.*m44*RSe451p*RS345pe + 1./162.*m44*RSe451p
         *RS356pe + 1./162.*m44*RSe451p*RSe563p + 1./162.*m44*RSe451p*
         RSe453p );

      M13p6x23RV +=  + AbV(m46,L)*AbV(me2,L) * (  - 1./162.*m44*RSe451p
         *RS145pe - 1./162.*m44*RSe451p*RS156pe - 1./162.*m44*RSe451p*
         RSe561p - 1./324.*m44*pow(RSe451p,2) + 1./54.*m33*RSe563p*
         RS345pe - 1./54.*m33*RSe563p*RS356pe - 1./108.*m33*pow(
         RSe563p,2) - 1./54.*m33*RSe453p*RS345pe + 1./54.*m33*RSe453p*
         RS356pe + 1./54.*m33*RSe453p*RSe563p - 1./108.*m33*pow(
         RSe453p,2) + 1./162.*m33*RS145pe*RSe563p - 1./162.*m33*RS145pe
         *RSe453p - 1./162.*m33*RS156pe*RSe563p + 1./162.*m33*RS156pe*
         RSe453p + 1./162.*m33*RSe561p*RS345pe - 1./162.*m33*RSe561p*
         RS356pe - 1./162.*m33*RSe561p*RSe563p + 1./162.*m33*RSe561p*
         RSe453p + 1./162.*m33*RSe561p*RS145pe - 1./162.*m33*RSe561p*
         RS156pe - 1./324.*m33*pow(RSe561p,2) - 1./162.*m33*RSe451p*
         RS345pe + 1./162.*m33*RSe451p*RS356pe + 1./162.*m33*RSe451p*
         RSe563p - 1./162.*m33*RSe451p*RSe453p - 1./162.*m33*RSe451p*
         RS145pe + 1./162.*m33*RSe451p*RS156pe + 1./162.*m33*RSe451p*
         RSe561p );

      M13p6x23RV +=  + AbV(m46,L)*AbV(me2,L) * (  - 1./324.*m33*pow(
         RSe451p,2) - 1./81.*m13*RSe563p*RS345pe + 1./81.*m13*RSe563p*
         RS356pe + 1./162.*m13*pow(RSe563p,2) + 1./81.*m13*RSe453p*
         RS345pe - 1./81.*m13*RSe453p*RS356pe - 1./81.*m13*RSe453p*
         RSe563p + 1./162.*m13*pow(RSe453p,2) + 1./81.*m13*RS145pe*
         RSe563p - 1./81.*m13*RS145pe*RSe453p - 1./81.*m13*RS156pe*
         RSe563p + 1./81.*m13*RS156pe*RSe453p + 1./81.*m13*RSe561p*
         RS345pe - 1./81.*m13*RSe561p*RS356pe - 1./81.*m13*RSe561p*
         RSe563p + 1./81.*m13*RSe561p*RSe453p - 1./81.*m13*RSe561p*
         RS145pe + 1./81.*m13*RSe561p*RS156pe + 1./162.*m13*pow(
         RSe561p,2) - 1./81.*m13*RSe451p*RS345pe + 1./81.*m13*RSe451p*
         RS356pe + 1./81.*m13*RSe451p*RSe563p - 1./81.*m13*RSe451p*
         RSe453p + 1./81.*m13*RSe451p*RS145pe - 1./81.*m13*RSe451p*
         RS156pe - 1./81.*m13*RSe451p*RSe561p + 1./162.*m13*pow(
         RSe451p,2) + 1./162.*m11*RSe563p*RS345pe - 1./162.*m11*RSe563p
         *RS356pe );

      M13p6x23RV +=  + AbV(m46,L)*AbV(me2,L) * (  - 1./324.*m11*pow(
         RSe563p,2) - 1./162.*m11*RSe453p*RS345pe + 1./162.*m11*RSe453p
         *RS356pe + 1./162.*m11*RSe453p*RSe563p - 1./324.*m11*pow(
         RSe453p,2) + 1./162.*m11*RS145pe*RSe563p - 1./162.*m11*RS145pe
         *RSe453p - 1./162.*m11*RS156pe*RSe563p + 1./162.*m11*RS156pe*
         RSe453p + 1./162.*m11*RSe561p*RS345pe - 1./162.*m11*RSe561p*
         RS356pe - 1./162.*m11*RSe561p*RSe563p + 1./162.*m11*RSe561p*
         RSe453p + 1./54.*m11*RSe561p*RS145pe - 1./54.*m11*RSe561p*
         RS156pe - 1./108.*m11*pow(RSe561p,2) - 1./162.*m11*RSe451p*
         RS345pe + 1./162.*m11*RSe451p*RS356pe + 1./162.*m11*RSe451p*
         RSe563p - 1./162.*m11*RSe451p*RSe453p - 1./54.*m11*RSe451p*
         RS145pe + 1./54.*m11*RSe451p*RS156pe + 1./54.*m11*RSe451p*
         RSe561p - 1./108.*m11*pow(RSe451p,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m11,m33,L) * ( 1./108.*m33*m66*
         RS145pe*RS345pe + 1./324.*m33*m66*RS145pe*RS356pe + 1./324.*
         m33*m66*RS156pe*RS345pe + 1./324.*m33*m66*RS156pe*RS356pe - 1./
         162.*m33*m46*RS145pe*RS345pe + 1./162.*m33*m46*RS145pe*RS356pe
          + 1./162.*m33*m46*RS156pe*RS345pe - 1./162.*m33*m46*RS156pe*
         RS356pe + 1./324.*m33*m44*RS145pe*RS345pe + 1./324.*m33*m44*
         RS145pe*RS356pe + 1./324.*m33*m44*RS156pe*RS345pe + 1./108.*
         m33*m44*RS156pe*RS356pe + 1./54.*m13*m66*RS145pe*RS345pe + 1./
         162.*m13*m66*RS145pe*RS356pe + 1./162.*m13*m66*RS156pe*RS345pe
          + 1./162.*m13*m66*RS156pe*RS356pe - 1./81.*m13*m46*RS145pe*
         RS345pe + 1./81.*m13*m46*RS145pe*RS356pe + 1./81.*m13*m46*
         RS156pe*RS345pe - 1./81.*m13*m46*RS156pe*RS356pe + 1./162.*m13
         *m44*RS145pe*RS345pe + 1./162.*m13*m44*RS145pe*RS356pe + 1./
         162.*m13*m44*RS156pe*RS345pe + 1./54.*m13*m44*RS156pe*RS356pe
          + 1./36.*m11*m66*RS145pe*RS345pe + 1./108.*m11*m66*RS145pe*
         RS356pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m11,m33,L) * ( 1./108.*m11*m66*
         RS156pe*RS345pe + 1./108.*m11*m66*RS156pe*RS356pe - 1./54.*m11
         *m46*RS145pe*RS345pe + 1./54.*m11*m46*RS145pe*RS356pe + 1./54.
         *m11*m46*RS156pe*RS345pe - 1./54.*m11*m46*RS156pe*RS356pe + 1./
         108.*m11*m44*RS145pe*RS345pe + 1./108.*m11*m44*RS145pe*RS356pe
          + 1./108.*m11*m44*RS156pe*RS345pe + 1./36.*m11*m44*RS156pe*
         RS356pe - 1./54.*m11*m33*RS145pe*RS345pe + 1./54.*m11*m33*
         RS145pe*RS356pe + 1./54.*m11*m33*RS156pe*RS345pe - 1./54.*m11*
         m33*RS156pe*RS356pe - 1./81.*m11*m13*RS145pe*RS345pe + 1./81.*
         m11*m13*RS145pe*RS356pe + 1./81.*m11*m13*RS156pe*RS345pe - 1./
         81.*m11*m13*RS156pe*RS356pe - 1./162.*pow(m11,2)*RS145pe*
         RS345pe + 1./162.*pow(m11,2)*RS145pe*RS356pe + 1./162.*pow(
         m11,2)*RS156pe*RS345pe - 1./162.*pow(m11,2)*RS156pe*RS356pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m11,mp2,L) * ( 1./324.*m33*m66*
         RS145pe*RSp563e + 1./108.*m33*m66*RS145pe*RSp453e + 1./324.*
         m33*m66*RS156pe*RSp563e + 1./324.*m33*m66*RS156pe*RSp453e + 1./
         324.*m33*m66*RSp561e*RS145pe + 1./324.*m33*m66*RSp561e*RS156pe
          + 1./108.*m33*m66*RSp451e*RS145pe + 1./324.*m33*m66*RSp451e*
         RS156pe + 1./162.*m33*m46*RS145pe*RSp563e - 1./162.*m33*m46*
         RS145pe*RSp453e - 1./162.*m33*m46*RS156pe*RSp563e + 1./162.*
         m33*m46*RS156pe*RSp453e + 1./162.*m33*m46*RSp561e*RS145pe - 1./
         162.*m33*m46*RSp561e*RS156pe - 1./162.*m33*m46*RSp451e*RS145pe
          + 1./162.*m33*m46*RSp451e*RS156pe + 1./324.*m33*m44*RS145pe*
         RSp563e + 1./324.*m33*m44*RS145pe*RSp453e + 1./108.*m33*m44*
         RS156pe*RSp563e + 1./324.*m33*m44*RS156pe*RSp453e + 1./324.*
         m33*m44*RSp561e*RS145pe + 1./108.*m33*m44*RSp561e*RS156pe + 1./
         324.*m33*m44*RSp451e*RS145pe + 1./324.*m33*m44*RSp451e*RS156pe
          + 1./162.*m13*m66*RS145pe*RSp563e + 1./54.*m13*m66*RS145pe*
         RSp453e );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m11,mp2,L) * ( 1./162.*m13*m66*
         RS156pe*RSp563e + 1./162.*m13*m66*RS156pe*RSp453e - 1./162.*
         m13*m66*RSp561e*RS145pe - 1./162.*m13*m66*RSp561e*RS156pe - 1./
         54.*m13*m66*RSp451e*RS145pe - 1./162.*m13*m66*RSp451e*RS156pe
          + 1./81.*m13*m46*RS145pe*RSp563e - 1./81.*m13*m46*RS145pe*
         RSp453e - 1./81.*m13*m46*RS156pe*RSp563e + 1./81.*m13*m46*
         RS156pe*RSp453e - 1./81.*m13*m46*RSp561e*RS145pe + 1./81.*m13*
         m46*RSp561e*RS156pe + 1./81.*m13*m46*RSp451e*RS145pe - 1./81.*
         m13*m46*RSp451e*RS156pe + 1./162.*m13*m44*RS145pe*RSp563e + 1./
         162.*m13*m44*RS145pe*RSp453e + 1./54.*m13*m44*RS156pe*RSp563e
          + 1./162.*m13*m44*RS156pe*RSp453e - 1./162.*m13*m44*RSp561e*
         RS145pe - 1./54.*m13*m44*RSp561e*RS156pe - 1./162.*m13*m44*
         RSp451e*RS145pe - 1./162.*m13*m44*RSp451e*RS156pe + 1./81.*m11
         *mp2*RS145pe*RSp563e - 1./81.*m11*mp2*RS145pe*RSp453e - 1./81.
         *m11*mp2*RS156pe*RSp563e + 1./81.*m11*mp2*RS156pe*RSp453e - 1./
         81.*m11*mp2*RSp561e*RS145pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m11,mp2,L) * ( 1./81.*m11*mp2*
         RSp561e*RS156pe + 1./81.*m11*mp2*RSp451e*RS145pe - 1./81.*m11*
         mp2*RSp451e*RS156pe + 1./108.*m11*m66*RS145pe*RSp563e + 1./36.
         *m11*m66*RS145pe*RSp453e + 1./108.*m11*m66*RS156pe*RSp563e + 1.
         /108.*m11*m66*RS156pe*RSp453e + 1./324.*m11*m66*RSp561e*
         RS145pe + 1./324.*m11*m66*RSp561e*RS156pe + 1./108.*m11*m66*
         RSp451e*RS145pe + 1./324.*m11*m66*RSp451e*RS156pe + 1./54.*m11
         *m46*RS145pe*RSp563e - 1./54.*m11*m46*RS145pe*RSp453e - 1./54.
         *m11*m46*RS156pe*RSp563e + 1./54.*m11*m46*RS156pe*RSp453e + 1./
         162.*m11*m46*RSp561e*RS145pe - 1./162.*m11*m46*RSp561e*RS156pe
          - 1./162.*m11*m46*RSp451e*RS145pe + 1./162.*m11*m46*RSp451e*
         RS156pe + 1./108.*m11*m44*RS145pe*RSp563e + 1./108.*m11*m44*
         RS145pe*RSp453e + 1./36.*m11*m44*RS156pe*RSp563e + 1./108.*m11
         *m44*RS156pe*RSp453e + 1./324.*m11*m44*RSp561e*RS145pe + 1./
         108.*m11*m44*RSp561e*RS156pe + 1./324.*m11*m44*RSp451e*RS145pe
          + 1./324.*m11*m44*RSp451e*RS156pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m11,mp2,L) * ( 1./162.*m11*m33*
         RS145pe*RSp563e - 1./162.*m11*m33*RS145pe*RSp453e - 1./162.*
         m11*m33*RS156pe*RSp563e + 1./162.*m11*m33*RS156pe*RSp453e + 1./
         162.*m11*m33*RSp561e*RS145pe - 1./162.*m11*m33*RSp561e*RS156pe
          - 1./162.*m11*m33*RSp451e*RS145pe + 1./162.*m11*m33*RSp451e*
         RS156pe + 1./81.*m11*m13*RS145pe*RSp563e - 1./81.*m11*m13*
         RS145pe*RSp453e - 1./81.*m11*m13*RS156pe*RSp563e + 1./81.*m11*
         m13*RS156pe*RSp453e - 1./81.*m11*m13*RSp561e*RS145pe + 1./81.*
         m11*m13*RSp561e*RS156pe + 1./81.*m11*m13*RSp451e*RS145pe - 1./
         81.*m11*m13*RSp451e*RS156pe + 1./162.*pow(m11,2)*RS145pe*
         RSp563e - 1./162.*pow(m11,2)*RS145pe*RSp453e - 1./162.*pow(
         m11,2)*RS156pe*RSp563e + 1./162.*pow(m11,2)*RS156pe*RSp453e + 
         1./54.*pow(m11,2)*RSp561e*RS145pe - 1./54.*pow(m11,2)*RSp561e*
         RS156pe - 1./54.*pow(m11,2)*RSp451e*RS145pe + 1./54.*pow(
         m11,2)*RSp451e*RS156pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m11,me2,L) * ( 1./324.*m33*m66*
         RS145pe*RSe563p + 1./108.*m33*m66*RS145pe*RSe453p + 1./324.*
         m33*m66*RS156pe*RSe563p + 1./324.*m33*m66*RS156pe*RSe453p + 1./
         324.*m33*m66*RSe561p*RS145pe + 1./324.*m33*m66*RSe561p*RS156pe
          + 1./108.*m33*m66*RSe451p*RS145pe + 1./324.*m33*m66*RSe451p*
         RS156pe + 1./162.*m33*m46*RS145pe*RSe563p - 1./162.*m33*m46*
         RS145pe*RSe453p - 1./162.*m33*m46*RS156pe*RSe563p + 1./162.*
         m33*m46*RS156pe*RSe453p + 1./162.*m33*m46*RSe561p*RS145pe - 1./
         162.*m33*m46*RSe561p*RS156pe - 1./162.*m33*m46*RSe451p*RS145pe
          + 1./162.*m33*m46*RSe451p*RS156pe + 1./324.*m33*m44*RS145pe*
         RSe563p + 1./324.*m33*m44*RS145pe*RSe453p + 1./108.*m33*m44*
         RS156pe*RSe563p + 1./324.*m33*m44*RS156pe*RSe453p + 1./324.*
         m33*m44*RSe561p*RS145pe + 1./108.*m33*m44*RSe561p*RS156pe + 1./
         324.*m33*m44*RSe451p*RS145pe + 1./324.*m33*m44*RSe451p*RS156pe
          + 1./162.*m13*m66*RS145pe*RSe563p + 1./54.*m13*m66*RS145pe*
         RSe453p );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m11,me2,L) * ( 1./162.*m13*m66*
         RS156pe*RSe563p + 1./162.*m13*m66*RS156pe*RSe453p - 1./162.*
         m13*m66*RSe561p*RS145pe - 1./162.*m13*m66*RSe561p*RS156pe - 1./
         54.*m13*m66*RSe451p*RS145pe - 1./162.*m13*m66*RSe451p*RS156pe
          + 1./81.*m13*m46*RS145pe*RSe563p - 1./81.*m13*m46*RS145pe*
         RSe453p - 1./81.*m13*m46*RS156pe*RSe563p + 1./81.*m13*m46*
         RS156pe*RSe453p - 1./81.*m13*m46*RSe561p*RS145pe + 1./81.*m13*
         m46*RSe561p*RS156pe + 1./81.*m13*m46*RSe451p*RS145pe - 1./81.*
         m13*m46*RSe451p*RS156pe + 1./162.*m13*m44*RS145pe*RSe563p + 1./
         162.*m13*m44*RS145pe*RSe453p + 1./54.*m13*m44*RS156pe*RSe563p
          + 1./162.*m13*m44*RS156pe*RSe453p - 1./162.*m13*m44*RSe561p*
         RS145pe - 1./54.*m13*m44*RSe561p*RS156pe - 1./162.*m13*m44*
         RSe451p*RS145pe - 1./162.*m13*m44*RSe451p*RS156pe + 1./81.*m11
         *me2*RS145pe*RSe563p - 1./81.*m11*me2*RS145pe*RSe453p - 1./81.
         *m11*me2*RS156pe*RSe563p + 1./81.*m11*me2*RS156pe*RSe453p - 1./
         81.*m11*me2*RSe561p*RS145pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m11,me2,L) * ( 1./81.*m11*me2*
         RSe561p*RS156pe + 1./81.*m11*me2*RSe451p*RS145pe - 1./81.*m11*
         me2*RSe451p*RS156pe + 1./108.*m11*m66*RS145pe*RSe563p + 1./36.
         *m11*m66*RS145pe*RSe453p + 1./108.*m11*m66*RS156pe*RSe563p + 1.
         /108.*m11*m66*RS156pe*RSe453p + 1./324.*m11*m66*RSe561p*
         RS145pe + 1./324.*m11*m66*RSe561p*RS156pe + 1./108.*m11*m66*
         RSe451p*RS145pe + 1./324.*m11*m66*RSe451p*RS156pe + 1./54.*m11
         *m46*RS145pe*RSe563p - 1./54.*m11*m46*RS145pe*RSe453p - 1./54.
         *m11*m46*RS156pe*RSe563p + 1./54.*m11*m46*RS156pe*RSe453p + 1./
         162.*m11*m46*RSe561p*RS145pe - 1./162.*m11*m46*RSe561p*RS156pe
          - 1./162.*m11*m46*RSe451p*RS145pe + 1./162.*m11*m46*RSe451p*
         RS156pe + 1./108.*m11*m44*RS145pe*RSe563p + 1./108.*m11*m44*
         RS145pe*RSe453p + 1./36.*m11*m44*RS156pe*RSe563p + 1./108.*m11
         *m44*RS156pe*RSe453p + 1./324.*m11*m44*RSe561p*RS145pe + 1./
         108.*m11*m44*RSe561p*RS156pe + 1./324.*m11*m44*RSe451p*RS145pe
          + 1./324.*m11*m44*RSe451p*RS156pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m11,me2,L) * ( 1./162.*m11*m33*
         RS145pe*RSe563p - 1./162.*m11*m33*RS145pe*RSe453p - 1./162.*
         m11*m33*RS156pe*RSe563p + 1./162.*m11*m33*RS156pe*RSe453p + 1./
         162.*m11*m33*RSe561p*RS145pe - 1./162.*m11*m33*RSe561p*RS156pe
          - 1./162.*m11*m33*RSe451p*RS145pe + 1./162.*m11*m33*RSe451p*
         RS156pe + 1./81.*m11*m13*RS145pe*RSe563p - 1./81.*m11*m13*
         RS145pe*RSe453p - 1./81.*m11*m13*RS156pe*RSe563p + 1./81.*m11*
         m13*RS156pe*RSe453p - 1./81.*m11*m13*RSe561p*RS145pe + 1./81.*
         m11*m13*RSe561p*RS156pe + 1./81.*m11*m13*RSe451p*RS145pe - 1./
         81.*m11*m13*RSe451p*RS156pe + 1./162.*pow(m11,2)*RS145pe*
         RSe563p - 1./162.*pow(m11,2)*RS145pe*RSe453p - 1./162.*pow(
         m11,2)*RS156pe*RSe563p + 1./162.*pow(m11,2)*RS156pe*RSe453p + 
         1./54.*pow(m11,2)*RSe561p*RS145pe - 1./54.*pow(m11,2)*RSe561p*
         RS156pe - 1./54.*pow(m11,2)*RSe451p*RS145pe + 1./54.*pow(
         m11,2)*RSe451p*RS156pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m11,L) * ( 1./216.*m33*m66*pow(
         RS145pe,2) + 1./324.*m33*m66*RS156pe*RS145pe + 1./648.*m33*m66
         *pow(RS156pe,2) - 1./324.*m33*m46*pow(RS145pe,2) + 1./162.*m33
         *m46*RS156pe*RS145pe - 1./324.*m33*m46*pow(RS156pe,2) + 1./648.
         *m33*m44*pow(RS145pe,2) + 1./324.*m33*m44*RS156pe*RS145pe + 1./
         216.*m33*m44*pow(RS156pe,2) - 1./108.*m13*m66*pow(RS145pe,2)
          - 1./162.*m13*m66*RS156pe*RS145pe - 1./324.*m13*m66*pow(
         RS156pe,2) + 1./162.*m13*m46*pow(RS145pe,2) - 1./81.*m13*m46*
         RS156pe*RS145pe + 1./162.*m13*m46*pow(RS156pe,2) - 1./324.*m13
         *m44*pow(RS145pe,2) - 1./162.*m13*m44*RS156pe*RS145pe - 1./108.
         *m13*m44*pow(RS156pe,2) + 1./216.*m11*m66*pow(RS145pe,2) + 1./
         324.*m11*m66*RS156pe*RS145pe + 1./648.*m11*m66*pow(RS156pe,2)
          - 1./324.*m11*m46*pow(RS145pe,2) + 1./162.*m11*m46*RS156pe*
         RS145pe - 1./324.*m11*m46*pow(RS156pe,2) + 1./648.*m11*m44*
         pow(RS145pe,2) + 1./324.*m11*m44*RS156pe*RS145pe + 1./216.*m11
         *m44*pow(RS156pe,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m11,L) * (  - 1./324.*m11*m33*
         pow(RS145pe,2) + 1./162.*m11*m33*RS156pe*RS145pe - 1./324.*m11
         *m33*pow(RS156pe,2) + 1./162.*m11*m13*pow(RS145pe,2) - 1./81.*
         m11*m13*RS156pe*RS145pe + 1./162.*m11*m13*pow(RS156pe,2) - 1./
         324.*pow(m11,2)*pow(RS145pe,2) + 1./162.*pow(m11,2)*RS156pe*
         RS145pe - 1./324.*pow(m11,2)*pow(RS156pe,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m14,L) * ( 1./144.*m44*m66 - 1./
         72.*m44*m46 + 1./72.*pow(m44,2) + 1./144.*m33*m66 - 1./72.*m33
         *m46 + 1./72.*m33*m44 - 1./72.*m14*m66 + 1./36.*m14*m46 - 1./
         24.*m14*m44 - 1./72.*m14*m33 + 1./36.*pow(m14,2) - 1./72.*m13*
         m66 + 1./36.*m13*m46 - 1./36.*m13*m44 + 1./36.*m13*m14 + 1./72.
         *m11*m66 - 1./36.*m11*m46 + 5./144.*m11*m44 + 1./144.*m11*m33
          - 1./24.*m11*m14 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m16,L) * ( 1./72.*pow(m66,2) - 1./
         72.*m46*m66 + 1./144.*m44*m66 + 1./72.*m33*m66 - 1./72.*m33*
         m46 + 1./144.*m33*m44 - 1./24.*m16*m66 + 1./36.*m16*m46 - 1./
         72.*m16*m44 - 1./72.*m16*m33 + 1./36.*pow(m16,2) - 1./36.*m13*
         m66 + 1./36.*m13*m46 - 1./72.*m13*m44 + 1./36.*m13*m16 + 5./
         144.*m11*m66 - 1./36.*m11*m46 + 1./72.*m11*m44 + 1./144.*m11*
         m33 - 1./24.*m11*m16 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m33,mp2,L) * (  - 1./81.*m33*mp2*
         RSp563e*RS345pe + 1./81.*m33*mp2*RSp563e*RS356pe + 1./81.*m33*
         mp2*RSp453e*RS345pe - 1./81.*m33*mp2*RSp453e*RS356pe + 1./81.*
         m33*mp2*RSp561e*RS345pe - 1./81.*m33*mp2*RSp561e*RS356pe - 1./
         81.*m33*mp2*RSp451e*RS345pe + 1./81.*m33*mp2*RSp451e*RS356pe
          + 1./324.*m33*m66*RSp563e*RS345pe + 1./324.*m33*m66*RSp563e*
         RS356pe + 1./108.*m33*m66*RSp453e*RS345pe + 1./324.*m33*m66*
         RSp453e*RS356pe + 1./108.*m33*m66*RSp561e*RS345pe + 1./108.*
         m33*m66*RSp561e*RS356pe + 1./36.*m33*m66*RSp451e*RS345pe + 1./
         108.*m33*m66*RSp451e*RS356pe + 1./162.*m33*m46*RSp563e*RS345pe
          - 1./162.*m33*m46*RSp563e*RS356pe - 1./162.*m33*m46*RSp453e*
         RS345pe + 1./162.*m33*m46*RSp453e*RS356pe + 1./54.*m33*m46*
         RSp561e*RS345pe - 1./54.*m33*m46*RSp561e*RS356pe - 1./54.*m33*
         m46*RSp451e*RS345pe + 1./54.*m33*m46*RSp451e*RS356pe + 1./324.
         *m33*m44*RSp563e*RS345pe + 1./108.*m33*m44*RSp563e*RS356pe + 1.
         /324.*m33*m44*RSp453e*RS345pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m33,mp2,L) * ( 1./324.*m33*m44*
         RSp453e*RS356pe + 1./108.*m33*m44*RSp561e*RS345pe + 1./36.*m33
         *m44*RSp561e*RS356pe + 1./108.*m33*m44*RSp451e*RS345pe + 1./
         108.*m33*m44*RSp451e*RS356pe + 1./54.*pow(m33,2)*RSp563e*
         RS345pe - 1./54.*pow(m33,2)*RSp563e*RS356pe - 1./54.*pow(
         m33,2)*RSp453e*RS345pe + 1./54.*pow(m33,2)*RSp453e*RS356pe + 1.
         /162.*pow(m33,2)*RSp561e*RS345pe - 1./162.*pow(m33,2)*RSp561e*
         RS356pe - 1./162.*pow(m33,2)*RSp451e*RS345pe + 1./162.*pow(
         m33,2)*RSp451e*RS356pe - 1./162.*m13*m66*RSp563e*RS345pe - 1./
         162.*m13*m66*RSp563e*RS356pe - 1./54.*m13*m66*RSp453e*RS345pe
          - 1./162.*m13*m66*RSp453e*RS356pe + 1./162.*m13*m66*RSp561e*
         RS345pe + 1./162.*m13*m66*RSp561e*RS356pe + 1./54.*m13*m66*
         RSp451e*RS345pe + 1./162.*m13*m66*RSp451e*RS356pe - 1./81.*m13
         *m46*RSp563e*RS345pe + 1./81.*m13*m46*RSp563e*RS356pe + 1./81.
         *m13*m46*RSp453e*RS345pe - 1./81.*m13*m46*RSp453e*RS356pe + 1./
         81.*m13*m46*RSp561e*RS345pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m33,mp2,L) * (  - 1./81.*m13*m46*
         RSp561e*RS356pe - 1./81.*m13*m46*RSp451e*RS345pe + 1./81.*m13*
         m46*RSp451e*RS356pe - 1./162.*m13*m44*RSp563e*RS345pe - 1./54.
         *m13*m44*RSp563e*RS356pe - 1./162.*m13*m44*RSp453e*RS345pe - 1.
         /162.*m13*m44*RSp453e*RS356pe + 1./162.*m13*m44*RSp561e*
         RS345pe + 1./54.*m13*m44*RSp561e*RS356pe + 1./162.*m13*m44*
         RSp451e*RS345pe + 1./162.*m13*m44*RSp451e*RS356pe - 1./81.*m13
         *m33*RSp563e*RS345pe + 1./81.*m13*m33*RSp563e*RS356pe + 1./81.
         *m13*m33*RSp453e*RS345pe - 1./81.*m13*m33*RSp453e*RS356pe + 1./
         81.*m13*m33*RSp561e*RS345pe - 1./81.*m13*m33*RSp561e*RS356pe
          - 1./81.*m13*m33*RSp451e*RS345pe + 1./81.*m13*m33*RSp451e*
         RS356pe + 1./324.*m11*m66*RSp563e*RS345pe + 1./324.*m11*m66*
         RSp563e*RS356pe + 1./108.*m11*m66*RSp453e*RS345pe + 1./324.*
         m11*m66*RSp453e*RS356pe + 1./324.*m11*m66*RSp561e*RS345pe + 1./
         324.*m11*m66*RSp561e*RS356pe + 1./108.*m11*m66*RSp451e*RS345pe
          + 1./324.*m11*m66*RSp451e*RS356pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m33,mp2,L) * ( 1./162.*m11*m46*
         RSp563e*RS345pe - 1./162.*m11*m46*RSp563e*RS356pe - 1./162.*
         m11*m46*RSp453e*RS345pe + 1./162.*m11*m46*RSp453e*RS356pe + 1./
         162.*m11*m46*RSp561e*RS345pe - 1./162.*m11*m46*RSp561e*RS356pe
          - 1./162.*m11*m46*RSp451e*RS345pe + 1./162.*m11*m46*RSp451e*
         RS356pe + 1./324.*m11*m44*RSp563e*RS345pe + 1./108.*m11*m44*
         RSp563e*RS356pe + 1./324.*m11*m44*RSp453e*RS345pe + 1./324.*
         m11*m44*RSp453e*RS356pe + 1./324.*m11*m44*RSp561e*RS345pe + 1./
         108.*m11*m44*RSp561e*RS356pe + 1./324.*m11*m44*RSp451e*RS345pe
          + 1./324.*m11*m44*RSp451e*RS356pe + 1./162.*m11*m33*RSp563e*
         RS345pe - 1./162.*m11*m33*RSp563e*RS356pe - 1./162.*m11*m33*
         RSp453e*RS345pe + 1./162.*m11*m33*RSp453e*RS356pe + 1./162.*
         m11*m33*RSp561e*RS345pe - 1./162.*m11*m33*RSp561e*RS356pe - 1./
         162.*m11*m33*RSp451e*RS345pe + 1./162.*m11*m33*RSp451e*RS356pe
          );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m33,me2,L) * (  - 1./81.*m33*me2*
         RSe563p*RS345pe + 1./81.*m33*me2*RSe563p*RS356pe + 1./81.*m33*
         me2*RSe453p*RS345pe - 1./81.*m33*me2*RSe453p*RS356pe + 1./81.*
         m33*me2*RSe561p*RS345pe - 1./81.*m33*me2*RSe561p*RS356pe - 1./
         81.*m33*me2*RSe451p*RS345pe + 1./81.*m33*me2*RSe451p*RS356pe
          + 1./324.*m33*m66*RSe563p*RS345pe + 1./324.*m33*m66*RSe563p*
         RS356pe + 1./108.*m33*m66*RSe453p*RS345pe + 1./324.*m33*m66*
         RSe453p*RS356pe + 1./108.*m33*m66*RSe561p*RS345pe + 1./108.*
         m33*m66*RSe561p*RS356pe + 1./36.*m33*m66*RSe451p*RS345pe + 1./
         108.*m33*m66*RSe451p*RS356pe + 1./162.*m33*m46*RSe563p*RS345pe
          - 1./162.*m33*m46*RSe563p*RS356pe - 1./162.*m33*m46*RSe453p*
         RS345pe + 1./162.*m33*m46*RSe453p*RS356pe + 1./54.*m33*m46*
         RSe561p*RS345pe - 1./54.*m33*m46*RSe561p*RS356pe - 1./54.*m33*
         m46*RSe451p*RS345pe + 1./54.*m33*m46*RSe451p*RS356pe + 1./324.
         *m33*m44*RSe563p*RS345pe + 1./108.*m33*m44*RSe563p*RS356pe + 1.
         /324.*m33*m44*RSe453p*RS345pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m33,me2,L) * ( 1./324.*m33*m44*
         RSe453p*RS356pe + 1./108.*m33*m44*RSe561p*RS345pe + 1./36.*m33
         *m44*RSe561p*RS356pe + 1./108.*m33*m44*RSe451p*RS345pe + 1./
         108.*m33*m44*RSe451p*RS356pe + 1./54.*pow(m33,2)*RSe563p*
         RS345pe - 1./54.*pow(m33,2)*RSe563p*RS356pe - 1./54.*pow(
         m33,2)*RSe453p*RS345pe + 1./54.*pow(m33,2)*RSe453p*RS356pe + 1.
         /162.*pow(m33,2)*RSe561p*RS345pe - 1./162.*pow(m33,2)*RSe561p*
         RS356pe - 1./162.*pow(m33,2)*RSe451p*RS345pe + 1./162.*pow(
         m33,2)*RSe451p*RS356pe - 1./162.*m13*m66*RSe563p*RS345pe - 1./
         162.*m13*m66*RSe563p*RS356pe - 1./54.*m13*m66*RSe453p*RS345pe
          - 1./162.*m13*m66*RSe453p*RS356pe + 1./162.*m13*m66*RSe561p*
         RS345pe + 1./162.*m13*m66*RSe561p*RS356pe + 1./54.*m13*m66*
         RSe451p*RS345pe + 1./162.*m13*m66*RSe451p*RS356pe - 1./81.*m13
         *m46*RSe563p*RS345pe + 1./81.*m13*m46*RSe563p*RS356pe + 1./81.
         *m13*m46*RSe453p*RS345pe - 1./81.*m13*m46*RSe453p*RS356pe + 1./
         81.*m13*m46*RSe561p*RS345pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m33,me2,L) * (  - 1./81.*m13*m46*
         RSe561p*RS356pe - 1./81.*m13*m46*RSe451p*RS345pe + 1./81.*m13*
         m46*RSe451p*RS356pe - 1./162.*m13*m44*RSe563p*RS345pe - 1./54.
         *m13*m44*RSe563p*RS356pe - 1./162.*m13*m44*RSe453p*RS345pe - 1.
         /162.*m13*m44*RSe453p*RS356pe + 1./162.*m13*m44*RSe561p*
         RS345pe + 1./54.*m13*m44*RSe561p*RS356pe + 1./162.*m13*m44*
         RSe451p*RS345pe + 1./162.*m13*m44*RSe451p*RS356pe - 1./81.*m13
         *m33*RSe563p*RS345pe + 1./81.*m13*m33*RSe563p*RS356pe + 1./81.
         *m13*m33*RSe453p*RS345pe - 1./81.*m13*m33*RSe453p*RS356pe + 1./
         81.*m13*m33*RSe561p*RS345pe - 1./81.*m13*m33*RSe561p*RS356pe
          - 1./81.*m13*m33*RSe451p*RS345pe + 1./81.*m13*m33*RSe451p*
         RS356pe + 1./324.*m11*m66*RSe563p*RS345pe + 1./324.*m11*m66*
         RSe563p*RS356pe + 1./108.*m11*m66*RSe453p*RS345pe + 1./324.*
         m11*m66*RSe453p*RS356pe + 1./324.*m11*m66*RSe561p*RS345pe + 1./
         324.*m11*m66*RSe561p*RS356pe + 1./108.*m11*m66*RSe451p*RS345pe
          + 1./324.*m11*m66*RSe451p*RS356pe );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m33,me2,L) * ( 1./162.*m11*m46*
         RSe563p*RS345pe - 1./162.*m11*m46*RSe563p*RS356pe - 1./162.*
         m11*m46*RSe453p*RS345pe + 1./162.*m11*m46*RSe453p*RS356pe + 1./
         162.*m11*m46*RSe561p*RS345pe - 1./162.*m11*m46*RSe561p*RS356pe
          - 1./162.*m11*m46*RSe451p*RS345pe + 1./162.*m11*m46*RSe451p*
         RS356pe + 1./324.*m11*m44*RSe563p*RS345pe + 1./108.*m11*m44*
         RSe563p*RS356pe + 1./324.*m11*m44*RSe453p*RS345pe + 1./324.*
         m11*m44*RSe453p*RS356pe + 1./324.*m11*m44*RSe561p*RS345pe + 1./
         108.*m11*m44*RSe561p*RS356pe + 1./324.*m11*m44*RSe451p*RS345pe
          + 1./324.*m11*m44*RSe451p*RS356pe + 1./162.*m11*m33*RSe563p*
         RS345pe - 1./162.*m11*m33*RSe563p*RS356pe - 1./162.*m11*m33*
         RSe453p*RS345pe + 1./162.*m11*m33*RSe453p*RS356pe + 1./162.*
         m11*m33*RSe561p*RS345pe - 1./162.*m11*m33*RSe561p*RS356pe - 1./
         162.*m11*m33*RSe451p*RS345pe + 1./162.*m11*m33*RSe451p*RS356pe
          );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m33,L) * ( 1./216.*m33*m66*pow(
         RS345pe,2) + 1./324.*m33*m66*RS356pe*RS345pe + 1./648.*m33*m66
         *pow(RS356pe,2) - 1./324.*m33*m46*pow(RS345pe,2) + 1./162.*m33
         *m46*RS356pe*RS345pe - 1./324.*m33*m46*pow(RS356pe,2) + 1./648.
         *m33*m44*pow(RS345pe,2) + 1./324.*m33*m44*RS356pe*RS345pe + 1./
         216.*m33*m44*pow(RS356pe,2) - 1./324.*pow(m33,2)*pow(
         RS345pe,2) + 1./162.*pow(m33,2)*RS356pe*RS345pe - 1./324.*pow(
         m33,2)*pow(RS356pe,2) - 1./108.*m13*m66*pow(RS345pe,2) - 1./
         162.*m13*m66*RS356pe*RS345pe - 1./324.*m13*m66*pow(RS356pe,2)
          + 1./162.*m13*m46*pow(RS345pe,2) - 1./81.*m13*m46*RS356pe*
         RS345pe + 1./162.*m13*m46*pow(RS356pe,2) - 1./324.*m13*m44*
         pow(RS345pe,2) - 1./162.*m13*m44*RS356pe*RS345pe - 1./108.*m13
         *m44*pow(RS356pe,2) + 1./162.*m13*m33*pow(RS345pe,2) - 1./81.*
         m13*m33*RS356pe*RS345pe + 1./162.*m13*m33*pow(RS356pe,2) + 1./
         216.*m11*m66*pow(RS345pe,2) + 1./324.*m11*m66*RS356pe*RS345pe
          + 1./648.*m11*m66*pow(RS356pe,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m33,L) * (  - 1./324.*m11*m46*
         pow(RS345pe,2) + 1./162.*m11*m46*RS356pe*RS345pe - 1./324.*m11
         *m46*pow(RS356pe,2) + 1./648.*m11*m44*pow(RS345pe,2) + 1./324.
         *m11*m44*RS356pe*RS345pe + 1./216.*m11*m44*pow(RS356pe,2) - 1./
         324.*m11*m33*pow(RS345pe,2) + 1./162.*m11*m33*RS356pe*RS345pe
          - 1./324.*m11*m33*pow(RS356pe,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m34,L) * ( 1./144.*m44*m66 - 1./
         72.*m44*m46 + 1./72.*pow(m44,2) - 1./72.*m34*m66 + 1./36.*m34*
         m46 - 1./24.*m34*m44 + 1./36.*pow(m34,2) + 1./72.*m33*m66 - 1./
         36.*m33*m46 + 5./144.*m33*m44 - 1./24.*m33*m34 + 1./72.*pow(
         m33,2) - 1./72.*m13*m66 + 1./36.*m13*m46 - 1./36.*m13*m44 + 1./
         36.*m13*m34 - 1./72.*m13*m33 + 1./144.*m11*m66 - 1./72.*m11*
         m46 + 1./72.*m11*m44 - 1./72.*m11*m34 + 1./144.*m11*m33 );

      M13p6x23RV +=  + AbV(m46,L)*BbV(m36,L) * ( 1./72.*pow(m66,2) - 1./
         72.*m46*m66 + 1./144.*m44*m66 - 1./24.*m36*m66 + 1./36.*m36*
         m46 - 1./72.*m36*m44 + 1./36.*pow(m36,2) + 5./144.*m33*m66 - 1.
         /36.*m33*m46 + 1./72.*m33*m44 - 1./24.*m33*m36 + 1./72.*pow(
         m33,2) - 1./36.*m13*m66 + 1./36.*m13*m46 - 1./72.*m13*m44 + 1./
         36.*m13*m36 - 1./72.*m13*m33 + 1./72.*m11*m66 - 1./72.*m11*m46
          + 1./144.*m11*m44 - 1./72.*m11*m36 + 1./144.*m11*m33 );

      M13p6x23RV +=  + AbV(m46,L)*BbV(mp2,L) * ( 1./162.*pow(mp2,2)*
         pow(RSp563e,2) - 1./81.*pow(mp2,2)*RSp453e*RSp563e + 1./162.*
         pow(mp2,2)*pow(RSp453e,2) - 1./81.*pow(mp2,2)*RSp561e*RSp563e
          + 1./81.*pow(mp2,2)*RSp561e*RSp453e + 1./162.*pow(mp2,2)*pow(
         RSp561e,2) + 1./81.*pow(mp2,2)*RSp451e*RSp563e - 1./81.*pow(
         mp2,2)*RSp451e*RSp453e - 1./81.*pow(mp2,2)*RSp451e*RSp561e + 1.
         /162.*pow(mp2,2)*pow(RSp451e,2) - 1./324.*m66*mp2*pow(
         RSp563e,2) - 1./162.*m66*mp2*RSp453e*RSp563e - 1./108.*m66*mp2
         *pow(RSp453e,2) + 1./162.*m66*mp2*RSp561e*RSp563e + 1./162.*
         m66*mp2*RSp561e*RSp453e - 1./324.*m66*mp2*pow(RSp561e,2) + 1./
         162.*m66*mp2*RSp451e*RSp563e + 1./54.*m66*mp2*RSp451e*RSp453e
          - 1./162.*m66*mp2*RSp451e*RSp561e - 1./108.*m66*mp2*pow(
         RSp451e,2) + 1./162.*m46*mp2*pow(RSp563e,2) - 1./81.*m46*mp2*
         RSp453e*RSp563e + 1./162.*m46*mp2*pow(RSp453e,2) - 1./81.*m46*
         mp2*RSp561e*RSp563e + 1./81.*m46*mp2*RSp561e*RSp453e + 1./162.
         *m46*mp2*pow(RSp561e,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(mp2,L) * ( 1./81.*m46*mp2*RSp451e
         *RSp563e - 1./81.*m46*mp2*RSp451e*RSp453e - 1./81.*m46*mp2*
         RSp451e*RSp561e + 1./162.*m46*mp2*pow(RSp451e,2) - 1./108.*m44
         *mp2*pow(RSp563e,2) - 1./162.*m44*mp2*RSp453e*RSp563e - 1./324.
         *m44*mp2*pow(RSp453e,2) + 1./54.*m44*mp2*RSp561e*RSp563e + 1./
         162.*m44*mp2*RSp561e*RSp453e - 1./108.*m44*mp2*pow(RSp561e,2)
          + 1./162.*m44*mp2*RSp451e*RSp563e + 1./162.*m44*mp2*RSp451e*
         RSp453e - 1./162.*m44*mp2*RSp451e*RSp561e - 1./324.*m44*mp2*
         pow(RSp451e,2) - 1./108.*m33*mp2*pow(RSp563e,2) + 1./54.*m33*
         mp2*RSp453e*RSp563e - 1./108.*m33*mp2*pow(RSp453e,2) - 1./162.
         *m33*mp2*RSp561e*RSp563e + 1./162.*m33*mp2*RSp561e*RSp453e - 1.
         /324.*m33*mp2*pow(RSp561e,2) + 1./162.*m33*mp2*RSp451e*RSp563e
          - 1./162.*m33*mp2*RSp451e*RSp453e + 1./162.*m33*mp2*RSp451e*
         RSp561e - 1./324.*m33*mp2*pow(RSp451e,2) + 1./216.*m33*m66*
         pow(RSp563e,2) + 1./108.*m33*m66*RSp453e*RSp563e + 1./72.*m33*
         m66*pow(RSp453e,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(mp2,L) * ( 1./324.*m33*m66*
         RSp561e*RSp563e + 1./324.*m33*m66*RSp561e*RSp453e + 1./648.*
         m33*m66*pow(RSp561e,2) + 1./324.*m33*m66*RSp451e*RSp563e + 1./
         108.*m33*m66*RSp451e*RSp453e + 1./324.*m33*m66*RSp451e*RSp561e
          + 1./216.*m33*m66*pow(RSp451e,2) - 1./108.*m33*m46*pow(
         RSp563e,2) + 1./54.*m33*m46*RSp453e*RSp563e - 1./108.*m33*m46*
         pow(RSp453e,2) - 1./162.*m33*m46*RSp561e*RSp563e + 1./162.*m33
         *m46*RSp561e*RSp453e - 1./324.*m33*m46*pow(RSp561e,2) + 1./162.
         *m33*m46*RSp451e*RSp563e - 1./162.*m33*m46*RSp451e*RSp453e + 1.
         /162.*m33*m46*RSp451e*RSp561e - 1./324.*m33*m46*pow(RSp451e,2)
          + 1./72.*m33*m44*pow(RSp563e,2) + 1./108.*m33*m44*RSp453e*
         RSp563e + 1./216.*m33*m44*pow(RSp453e,2) + 1./108.*m33*m44*
         RSp561e*RSp563e + 1./324.*m33*m44*RSp561e*RSp453e + 1./216.*
         m33*m44*pow(RSp561e,2) + 1./324.*m33*m44*RSp451e*RSp563e + 1./
         324.*m33*m44*RSp451e*RSp453e + 1./324.*m33*m44*RSp451e*RSp561e
          + 1./648.*m33*m44*pow(RSp451e,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(mp2,L) * ( 1./162.*m13*mp2*pow(
         RSp563e,2) - 1./81.*m13*mp2*RSp453e*RSp563e + 1./162.*m13*mp2*
         pow(RSp453e,2) - 1./81.*m13*mp2*RSp561e*RSp563e + 1./81.*m13*
         mp2*RSp561e*RSp453e + 1./162.*m13*mp2*pow(RSp561e,2) + 1./81.*
         m13*mp2*RSp451e*RSp563e - 1./81.*m13*mp2*RSp451e*RSp453e - 1./
         81.*m13*mp2*RSp451e*RSp561e + 1./162.*m13*mp2*pow(RSp451e,2)
          - 1./324.*m13*m66*pow(RSp563e,2) - 1./162.*m13*m66*RSp453e*
         RSp563e - 1./108.*m13*m66*pow(RSp453e,2) + 1./162.*m13*m66*
         RSp561e*RSp563e + 1./162.*m13*m66*RSp561e*RSp453e - 1./324.*
         m13*m66*pow(RSp561e,2) + 1./162.*m13*m66*RSp451e*RSp563e + 1./
         54.*m13*m66*RSp451e*RSp453e - 1./162.*m13*m66*RSp451e*RSp561e
          - 1./108.*m13*m66*pow(RSp451e,2) + 1./162.*m13*m46*pow(
         RSp563e,2) - 1./81.*m13*m46*RSp453e*RSp563e + 1./162.*m13*m46*
         pow(RSp453e,2) - 1./81.*m13*m46*RSp561e*RSp563e + 1./81.*m13*
         m46*RSp561e*RSp453e + 1./162.*m13*m46*pow(RSp561e,2) + 1./81.*
         m13*m46*RSp451e*RSp563e );

      M13p6x23RV +=  + AbV(m46,L)*BbV(mp2,L) * (  - 1./81.*m13*m46*
         RSp451e*RSp453e - 1./81.*m13*m46*RSp451e*RSp561e + 1./162.*m13
         *m46*pow(RSp451e,2) - 1./108.*m13*m44*pow(RSp563e,2) - 1./162.
         *m13*m44*RSp453e*RSp563e - 1./324.*m13*m44*pow(RSp453e,2) + 1./
         54.*m13*m44*RSp561e*RSp563e + 1./162.*m13*m44*RSp561e*RSp453e
          - 1./108.*m13*m44*pow(RSp561e,2) + 1./162.*m13*m44*RSp451e*
         RSp563e + 1./162.*m13*m44*RSp451e*RSp453e - 1./162.*m13*m44*
         RSp451e*RSp561e - 1./324.*m13*m44*pow(RSp451e,2) - 1./324.*m11
         *mp2*pow(RSp563e,2) + 1./162.*m11*mp2*RSp453e*RSp563e - 1./324.
         *m11*mp2*pow(RSp453e,2) - 1./162.*m11*mp2*RSp561e*RSp563e + 1./
         162.*m11*mp2*RSp561e*RSp453e - 1./108.*m11*mp2*pow(RSp561e,2)
          + 1./162.*m11*mp2*RSp451e*RSp563e - 1./162.*m11*mp2*RSp451e*
         RSp453e + 1./54.*m11*mp2*RSp451e*RSp561e - 1./108.*m11*mp2*
         pow(RSp451e,2) + 1./648.*m11*m66*pow(RSp563e,2) + 1./324.*m11*
         m66*RSp453e*RSp563e + 1./216.*m11*m66*pow(RSp453e,2) + 1./324.
         *m11*m66*RSp561e*RSp563e );

      M13p6x23RV +=  + AbV(m46,L)*BbV(mp2,L) * ( 1./324.*m11*m66*
         RSp561e*RSp453e + 1./216.*m11*m66*pow(RSp561e,2) + 1./324.*m11
         *m66*RSp451e*RSp563e + 1./108.*m11*m66*RSp451e*RSp453e + 1./
         108.*m11*m66*RSp451e*RSp561e + 1./72.*m11*m66*pow(RSp451e,2)
          - 1./324.*m11*m46*pow(RSp563e,2) + 1./162.*m11*m46*RSp453e*
         RSp563e - 1./324.*m11*m46*pow(RSp453e,2) - 1./162.*m11*m46*
         RSp561e*RSp563e + 1./162.*m11*m46*RSp561e*RSp453e - 1./108.*
         m11*m46*pow(RSp561e,2) + 1./162.*m11*m46*RSp451e*RSp563e - 1./
         162.*m11*m46*RSp451e*RSp453e + 1./54.*m11*m46*RSp451e*RSp561e
          - 1./108.*m11*m46*pow(RSp451e,2) + 1./216.*m11*m44*pow(
         RSp563e,2) + 1./324.*m11*m44*RSp453e*RSp563e + 1./648.*m11*m44
         *pow(RSp453e,2) + 1./108.*m11*m44*RSp561e*RSp563e + 1./324.*
         m11*m44*RSp561e*RSp453e + 1./72.*m11*m44*pow(RSp561e,2) + 1./
         324.*m11*m44*RSp451e*RSp563e + 1./324.*m11*m44*RSp451e*RSp453e
          + 1./108.*m11*m44*RSp451e*RSp561e + 1./216.*m11*m44*pow(
         RSp451e,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * ( 1./81.*mp2*me2*
         RSp563e*RSe563p - 1./81.*mp2*me2*RSp563e*RSe453p - 1./81.*mp2*
         me2*RSp453e*RSe563p + 1./81.*mp2*me2*RSp453e*RSe453p - 1./81.*
         mp2*me2*RSe561p*RSp563e + 1./81.*mp2*me2*RSe561p*RSp453e + 1./
         81.*mp2*me2*RSe451p*RSp563e - 1./81.*mp2*me2*RSe451p*RSp453e
          - 1./81.*mp2*me2*RSp561e*RSe563p + 1./81.*mp2*me2*RSp561e*
         RSe453p + 1./81.*mp2*me2*RSp561e*RSe561p - 1./81.*mp2*me2*
         RSp561e*RSe451p + 1./81.*mp2*me2*RSp451e*RSe563p - 1./81.*mp2*
         me2*RSp451e*RSe453p - 1./81.*mp2*me2*RSp451e*RSe561p + 1./81.*
         mp2*me2*RSp451e*RSe451p - 1./162.*m66*me2*RSp563e*RSe563p - 1./
         162.*m66*me2*RSp563e*RSe453p - 1./162.*m66*me2*RSp453e*RSe563p
          - 1./54.*m66*me2*RSp453e*RSe453p + 1./162.*m66*me2*RSe561p*
         RSp563e + 1./162.*m66*me2*RSe561p*RSp453e + 1./162.*m66*me2*
         RSe451p*RSp563e + 1./54.*m66*me2*RSe451p*RSp453e + 1./162.*m66
         *me2*RSp561e*RSe563p + 1./162.*m66*me2*RSp561e*RSe453p - 1./
         162.*m66*me2*RSp561e*RSe561p );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * (  - 1./162.*m66*me2
         *RSp561e*RSe451p + 1./162.*m66*me2*RSp451e*RSe563p + 1./54.*
         m66*me2*RSp451e*RSe453p - 1./162.*m66*me2*RSp451e*RSe561p - 1./
         54.*m66*me2*RSp451e*RSe451p + 1./81.*m46*me2*RSp563e*RSe563p
          - 1./81.*m46*me2*RSp563e*RSe453p - 1./81.*m46*me2*RSp453e*
         RSe563p + 1./81.*m46*me2*RSp453e*RSe453p - 1./81.*m46*me2*
         RSe561p*RSp563e + 1./81.*m46*me2*RSe561p*RSp453e + 1./81.*m46*
         me2*RSe451p*RSp563e - 1./81.*m46*me2*RSe451p*RSp453e - 1./81.*
         m46*me2*RSp561e*RSe563p + 1./81.*m46*me2*RSp561e*RSe453p + 1./
         81.*m46*me2*RSp561e*RSe561p - 1./81.*m46*me2*RSp561e*RSe451p
          + 1./81.*m46*me2*RSp451e*RSe563p - 1./81.*m46*me2*RSp451e*
         RSe453p - 1./81.*m46*me2*RSp451e*RSe561p + 1./81.*m46*me2*
         RSp451e*RSe451p - 1./54.*m44*me2*RSp563e*RSe563p - 1./162.*m44
         *me2*RSp563e*RSe453p - 1./162.*m44*me2*RSp453e*RSe563p - 1./
         162.*m44*me2*RSp453e*RSe453p + 1./54.*m44*me2*RSe561p*RSp563e
          + 1./162.*m44*me2*RSe561p*RSp453e );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * ( 1./162.*m44*me2*
         RSe451p*RSp563e + 1./162.*m44*me2*RSe451p*RSp453e + 1./54.*m44
         *me2*RSp561e*RSe563p + 1./162.*m44*me2*RSp561e*RSe453p - 1./54.
         *m44*me2*RSp561e*RSe561p - 1./162.*m44*me2*RSp561e*RSe451p + 1.
         /162.*m44*me2*RSp451e*RSe563p + 1./162.*m44*me2*RSp451e*
         RSe453p - 1./162.*m44*me2*RSp451e*RSe561p - 1./162.*m44*me2*
         RSp451e*RSe451p - 1./54.*m33*me2*RSp563e*RSe563p + 1./54.*m33*
         me2*RSp563e*RSe453p + 1./54.*m33*me2*RSp453e*RSe563p - 1./54.*
         m33*me2*RSp453e*RSe453p - 1./162.*m33*me2*RSe561p*RSp563e + 1./
         162.*m33*me2*RSe561p*RSp453e + 1./162.*m33*me2*RSe451p*RSp563e
          - 1./162.*m33*me2*RSe451p*RSp453e - 1./162.*m33*me2*RSp561e*
         RSe563p + 1./162.*m33*me2*RSp561e*RSe453p - 1./162.*m33*me2*
         RSp561e*RSe561p + 1./162.*m33*me2*RSp561e*RSe451p + 1./162.*
         m33*me2*RSp451e*RSe563p - 1./162.*m33*me2*RSp451e*RSe453p + 1./
         162.*m33*me2*RSp451e*RSe561p - 1./162.*m33*me2*RSp451e*RSe451p
          + 1./108.*m33*m66*RSp563e*RSe563p );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * ( 1./108.*m33*m66*
         RSp563e*RSe453p + 1./108.*m33*m66*RSp453e*RSe563p + 1./36.*m33
         *m66*RSp453e*RSe453p + 1./324.*m33*m66*RSe561p*RSp563e + 1./
         324.*m33*m66*RSe561p*RSp453e + 1./324.*m33*m66*RSe451p*RSp563e
          + 1./108.*m33*m66*RSe451p*RSp453e + 1./324.*m33*m66*RSp561e*
         RSe563p + 1./324.*m33*m66*RSp561e*RSe453p + 1./324.*m33*m66*
         RSp561e*RSe561p + 1./324.*m33*m66*RSp561e*RSe451p + 1./324.*
         m33*m66*RSp451e*RSe563p + 1./108.*m33*m66*RSp451e*RSe453p + 1./
         324.*m33*m66*RSp451e*RSe561p + 1./108.*m33*m66*RSp451e*RSe451p
          - 1./54.*m33*m46*RSp563e*RSe563p + 1./54.*m33*m46*RSp563e*
         RSe453p + 1./54.*m33*m46*RSp453e*RSe563p - 1./54.*m33*m46*
         RSp453e*RSe453p - 1./162.*m33*m46*RSe561p*RSp563e + 1./162.*
         m33*m46*RSe561p*RSp453e + 1./162.*m33*m46*RSe451p*RSp563e - 1./
         162.*m33*m46*RSe451p*RSp453e - 1./162.*m33*m46*RSp561e*RSe563p
          + 1./162.*m33*m46*RSp561e*RSe453p - 1./162.*m33*m46*RSp561e*
         RSe561p );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * ( 1./162.*m33*m46*
         RSp561e*RSe451p + 1./162.*m33*m46*RSp451e*RSe563p - 1./162.*
         m33*m46*RSp451e*RSe453p + 1./162.*m33*m46*RSp451e*RSe561p - 1./
         162.*m33*m46*RSp451e*RSe451p + 1./36.*m33*m44*RSp563e*RSe563p
          + 1./108.*m33*m44*RSp563e*RSe453p + 1./108.*m33*m44*RSp453e*
         RSe563p + 1./108.*m33*m44*RSp453e*RSe453p + 1./108.*m33*m44*
         RSe561p*RSp563e + 1./324.*m33*m44*RSe561p*RSp453e + 1./324.*
         m33*m44*RSe451p*RSp563e + 1./324.*m33*m44*RSe451p*RSp453e + 1./
         108.*m33*m44*RSp561e*RSe563p + 1./324.*m33*m44*RSp561e*RSe453p
          + 1./108.*m33*m44*RSp561e*RSe561p + 1./324.*m33*m44*RSp561e*
         RSe451p + 1./324.*m33*m44*RSp451e*RSe563p + 1./324.*m33*m44*
         RSp451e*RSe453p + 1./324.*m33*m44*RSp451e*RSe561p + 1./324.*
         m33*m44*RSp451e*RSe451p + 1./81.*m13*me2*RSp563e*RSe563p - 1./
         81.*m13*me2*RSp563e*RSe453p - 1./81.*m13*me2*RSp453e*RSe563p
          + 1./81.*m13*me2*RSp453e*RSe453p - 1./81.*m13*me2*RSe561p*
         RSp563e );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * ( 1./81.*m13*me2*
         RSe561p*RSp453e + 1./81.*m13*me2*RSe451p*RSp563e - 1./81.*m13*
         me2*RSe451p*RSp453e - 1./81.*m13*me2*RSp561e*RSe563p + 1./81.*
         m13*me2*RSp561e*RSe453p + 1./81.*m13*me2*RSp561e*RSe561p - 1./
         81.*m13*me2*RSp561e*RSe451p + 1./81.*m13*me2*RSp451e*RSe563p
          - 1./81.*m13*me2*RSp451e*RSe453p - 1./81.*m13*me2*RSp451e*
         RSe561p + 1./81.*m13*me2*RSp451e*RSe451p - 1./162.*m13*m66*
         RSp563e*RSe563p - 1./162.*m13*m66*RSp563e*RSe453p - 1./162.*
         m13*m66*RSp453e*RSe563p - 1./54.*m13*m66*RSp453e*RSe453p + 1./
         162.*m13*m66*RSe561p*RSp563e + 1./162.*m13*m66*RSe561p*RSp453e
          + 1./162.*m13*m66*RSe451p*RSp563e + 1./54.*m13*m66*RSe451p*
         RSp453e + 1./162.*m13*m66*RSp561e*RSe563p + 1./162.*m13*m66*
         RSp561e*RSe453p - 1./162.*m13*m66*RSp561e*RSe561p - 1./162.*
         m13*m66*RSp561e*RSe451p + 1./162.*m13*m66*RSp451e*RSe563p + 1./
         54.*m13*m66*RSp451e*RSe453p - 1./162.*m13*m66*RSp451e*RSe561p
          - 1./54.*m13*m66*RSp451e*RSe451p );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * ( 1./81.*m13*m46*
         RSp563e*RSe563p - 1./81.*m13*m46*RSp563e*RSe453p - 1./81.*m13*
         m46*RSp453e*RSe563p + 1./81.*m13*m46*RSp453e*RSe453p - 1./81.*
         m13*m46*RSe561p*RSp563e + 1./81.*m13*m46*RSe561p*RSp453e + 1./
         81.*m13*m46*RSe451p*RSp563e - 1./81.*m13*m46*RSe451p*RSp453e
          - 1./81.*m13*m46*RSp561e*RSe563p + 1./81.*m13*m46*RSp561e*
         RSe453p + 1./81.*m13*m46*RSp561e*RSe561p - 1./81.*m13*m46*
         RSp561e*RSe451p + 1./81.*m13*m46*RSp451e*RSe563p - 1./81.*m13*
         m46*RSp451e*RSe453p - 1./81.*m13*m46*RSp451e*RSe561p + 1./81.*
         m13*m46*RSp451e*RSe451p - 1./54.*m13*m44*RSp563e*RSe563p - 1./
         162.*m13*m44*RSp563e*RSe453p - 1./162.*m13*m44*RSp453e*RSe563p
          - 1./162.*m13*m44*RSp453e*RSe453p + 1./54.*m13*m44*RSe561p*
         RSp563e + 1./162.*m13*m44*RSe561p*RSp453e + 1./162.*m13*m44*
         RSe451p*RSp563e + 1./162.*m13*m44*RSe451p*RSp453e + 1./54.*m13
         *m44*RSp561e*RSe563p + 1./162.*m13*m44*RSp561e*RSe453p - 1./54.
         *m13*m44*RSp561e*RSe561p );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * (  - 1./162.*m13*m44
         *RSp561e*RSe451p + 1./162.*m13*m44*RSp451e*RSe563p + 1./162.*
         m13*m44*RSp451e*RSe453p - 1./162.*m13*m44*RSp451e*RSe561p - 1./
         162.*m13*m44*RSp451e*RSe451p - 1./162.*m11*me2*RSp563e*RSe563p
          + 1./162.*m11*me2*RSp563e*RSe453p + 1./162.*m11*me2*RSp453e*
         RSe563p - 1./162.*m11*me2*RSp453e*RSe453p - 1./162.*m11*me2*
         RSe561p*RSp563e + 1./162.*m11*me2*RSe561p*RSp453e + 1./162.*
         m11*me2*RSe451p*RSp563e - 1./162.*m11*me2*RSe451p*RSp453e - 1./
         162.*m11*me2*RSp561e*RSe563p + 1./162.*m11*me2*RSp561e*RSe453p
          - 1./54.*m11*me2*RSp561e*RSe561p + 1./54.*m11*me2*RSp561e*
         RSe451p + 1./162.*m11*me2*RSp451e*RSe563p - 1./162.*m11*me2*
         RSp451e*RSe453p + 1./54.*m11*me2*RSp451e*RSe561p - 1./54.*m11*
         me2*RSp451e*RSe451p + 1./324.*m11*m66*RSp563e*RSe563p + 1./324.
         *m11*m66*RSp563e*RSe453p + 1./324.*m11*m66*RSp453e*RSe563p + 1.
         /108.*m11*m66*RSp453e*RSe453p + 1./324.*m11*m66*RSe561p*
         RSp563e );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * ( 1./324.*m11*m66*
         RSe561p*RSp453e + 1./324.*m11*m66*RSe451p*RSp563e + 1./108.*
         m11*m66*RSe451p*RSp453e + 1./324.*m11*m66*RSp561e*RSe563p + 1./
         324.*m11*m66*RSp561e*RSe453p + 1./108.*m11*m66*RSp561e*RSe561p
          + 1./108.*m11*m66*RSp561e*RSe451p + 1./324.*m11*m66*RSp451e*
         RSe563p + 1./108.*m11*m66*RSp451e*RSe453p + 1./108.*m11*m66*
         RSp451e*RSe561p + 1./36.*m11*m66*RSp451e*RSe451p - 1./162.*m11
         *m46*RSp563e*RSe563p + 1./162.*m11*m46*RSp563e*RSe453p + 1./
         162.*m11*m46*RSp453e*RSe563p - 1./162.*m11*m46*RSp453e*RSe453p
          - 1./162.*m11*m46*RSe561p*RSp563e + 1./162.*m11*m46*RSe561p*
         RSp453e + 1./162.*m11*m46*RSe451p*RSp563e - 1./162.*m11*m46*
         RSe451p*RSp453e - 1./162.*m11*m46*RSp561e*RSe563p + 1./162.*
         m11*m46*RSp561e*RSe453p - 1./54.*m11*m46*RSp561e*RSe561p + 1./
         54.*m11*m46*RSp561e*RSe451p + 1./162.*m11*m46*RSp451e*RSe563p
          - 1./162.*m11*m46*RSp451e*RSe453p + 1./54.*m11*m46*RSp451e*
         RSe561p );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * (  - 1./54.*m11*m46*
         RSp451e*RSe451p + 1./108.*m11*m44*RSp563e*RSe563p + 1./324.*
         m11*m44*RSp563e*RSe453p + 1./324.*m11*m44*RSp453e*RSe563p + 1./
         324.*m11*m44*RSp453e*RSe453p + 1./108.*m11*m44*RSe561p*RSp563e
          + 1./324.*m11*m44*RSe561p*RSp453e + 1./324.*m11*m44*RSe451p*
         RSp563e + 1./324.*m11*m44*RSe451p*RSp453e + 1./108.*m11*m44*
         RSp561e*RSe563p + 1./324.*m11*m44*RSp561e*RSe453p + 1./36.*m11
         *m44*RSp561e*RSe561p + 1./108.*m11*m44*RSp561e*RSe451p + 1./
         324.*m11*m44*RSp451e*RSe563p + 1./324.*m11*m44*RSp451e*RSe453p
          + 1./108.*m11*m44*RSp451e*RSe561p + 1./108.*m11*m44*RSp451e*
         RSe451p );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,L) * ( 1./162.*pow(me2,2)*
         pow(RSe563p,2) - 1./81.*pow(me2,2)*RSe453p*RSe563p + 1./162.*
         pow(me2,2)*pow(RSe453p,2) - 1./81.*pow(me2,2)*RSe561p*RSe563p
          + 1./81.*pow(me2,2)*RSe561p*RSe453p + 1./162.*pow(me2,2)*pow(
         RSe561p,2) + 1./81.*pow(me2,2)*RSe451p*RSe563p - 1./81.*pow(
         me2,2)*RSe451p*RSe453p - 1./81.*pow(me2,2)*RSe451p*RSe561p + 1.
         /162.*pow(me2,2)*pow(RSe451p,2) - 1./324.*m66*me2*pow(
         RSe563p,2) - 1./162.*m66*me2*RSe453p*RSe563p - 1./108.*m66*me2
         *pow(RSe453p,2) + 1./162.*m66*me2*RSe561p*RSe563p + 1./162.*
         m66*me2*RSe561p*RSe453p - 1./324.*m66*me2*pow(RSe561p,2) + 1./
         162.*m66*me2*RSe451p*RSe563p + 1./54.*m66*me2*RSe451p*RSe453p
          - 1./162.*m66*me2*RSe451p*RSe561p - 1./108.*m66*me2*pow(
         RSe451p,2) + 1./162.*m46*me2*pow(RSe563p,2) - 1./81.*m46*me2*
         RSe453p*RSe563p + 1./162.*m46*me2*pow(RSe453p,2) - 1./81.*m46*
         me2*RSe561p*RSe563p + 1./81.*m46*me2*RSe561p*RSe453p + 1./162.
         *m46*me2*pow(RSe561p,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,L) * ( 1./81.*m46*me2*RSe451p
         *RSe563p - 1./81.*m46*me2*RSe451p*RSe453p - 1./81.*m46*me2*
         RSe451p*RSe561p + 1./162.*m46*me2*pow(RSe451p,2) - 1./108.*m44
         *me2*pow(RSe563p,2) - 1./162.*m44*me2*RSe453p*RSe563p - 1./324.
         *m44*me2*pow(RSe453p,2) + 1./54.*m44*me2*RSe561p*RSe563p + 1./
         162.*m44*me2*RSe561p*RSe453p - 1./108.*m44*me2*pow(RSe561p,2)
          + 1./162.*m44*me2*RSe451p*RSe563p + 1./162.*m44*me2*RSe451p*
         RSe453p - 1./162.*m44*me2*RSe451p*RSe561p - 1./324.*m44*me2*
         pow(RSe451p,2) - 1./108.*m33*me2*pow(RSe563p,2) + 1./54.*m33*
         me2*RSe453p*RSe563p - 1./108.*m33*me2*pow(RSe453p,2) - 1./162.
         *m33*me2*RSe561p*RSe563p + 1./162.*m33*me2*RSe561p*RSe453p - 1.
         /324.*m33*me2*pow(RSe561p,2) + 1./162.*m33*me2*RSe451p*RSe563p
          - 1./162.*m33*me2*RSe451p*RSe453p + 1./162.*m33*me2*RSe451p*
         RSe561p - 1./324.*m33*me2*pow(RSe451p,2) + 1./216.*m33*m66*
         pow(RSe563p,2) + 1./108.*m33*m66*RSe453p*RSe563p + 1./72.*m33*
         m66*pow(RSe453p,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,L) * ( 1./324.*m33*m66*
         RSe561p*RSe563p + 1./324.*m33*m66*RSe561p*RSe453p + 1./648.*
         m33*m66*pow(RSe561p,2) + 1./324.*m33*m66*RSe451p*RSe563p + 1./
         108.*m33*m66*RSe451p*RSe453p + 1./324.*m33*m66*RSe451p*RSe561p
          + 1./216.*m33*m66*pow(RSe451p,2) - 1./108.*m33*m46*pow(
         RSe563p,2) + 1./54.*m33*m46*RSe453p*RSe563p - 1./108.*m33*m46*
         pow(RSe453p,2) - 1./162.*m33*m46*RSe561p*RSe563p + 1./162.*m33
         *m46*RSe561p*RSe453p - 1./324.*m33*m46*pow(RSe561p,2) + 1./162.
         *m33*m46*RSe451p*RSe563p - 1./162.*m33*m46*RSe451p*RSe453p + 1.
         /162.*m33*m46*RSe451p*RSe561p - 1./324.*m33*m46*pow(RSe451p,2)
          + 1./72.*m33*m44*pow(RSe563p,2) + 1./108.*m33*m44*RSe453p*
         RSe563p + 1./216.*m33*m44*pow(RSe453p,2) + 1./108.*m33*m44*
         RSe561p*RSe563p + 1./324.*m33*m44*RSe561p*RSe453p + 1./216.*
         m33*m44*pow(RSe561p,2) + 1./324.*m33*m44*RSe451p*RSe563p + 1./
         324.*m33*m44*RSe451p*RSe453p + 1./324.*m33*m44*RSe451p*RSe561p
          + 1./648.*m33*m44*pow(RSe451p,2) );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,L) * ( 1./162.*m13*me2*pow(
         RSe563p,2) - 1./81.*m13*me2*RSe453p*RSe563p + 1./162.*m13*me2*
         pow(RSe453p,2) - 1./81.*m13*me2*RSe561p*RSe563p + 1./81.*m13*
         me2*RSe561p*RSe453p + 1./162.*m13*me2*pow(RSe561p,2) + 1./81.*
         m13*me2*RSe451p*RSe563p - 1./81.*m13*me2*RSe451p*RSe453p - 1./
         81.*m13*me2*RSe451p*RSe561p + 1./162.*m13*me2*pow(RSe451p,2)
          - 1./324.*m13*m66*pow(RSe563p,2) - 1./162.*m13*m66*RSe453p*
         RSe563p - 1./108.*m13*m66*pow(RSe453p,2) + 1./162.*m13*m66*
         RSe561p*RSe563p + 1./162.*m13*m66*RSe561p*RSe453p - 1./324.*
         m13*m66*pow(RSe561p,2) + 1./162.*m13*m66*RSe451p*RSe563p + 1./
         54.*m13*m66*RSe451p*RSe453p - 1./162.*m13*m66*RSe451p*RSe561p
          - 1./108.*m13*m66*pow(RSe451p,2) + 1./162.*m13*m46*pow(
         RSe563p,2) - 1./81.*m13*m46*RSe453p*RSe563p + 1./162.*m13*m46*
         pow(RSe453p,2) - 1./81.*m13*m46*RSe561p*RSe563p + 1./81.*m13*
         m46*RSe561p*RSe453p + 1./162.*m13*m46*pow(RSe561p,2) + 1./81.*
         m13*m46*RSe451p*RSe563p );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,L) * (  - 1./81.*m13*m46*
         RSe451p*RSe453p - 1./81.*m13*m46*RSe451p*RSe561p + 1./162.*m13
         *m46*pow(RSe451p,2) - 1./108.*m13*m44*pow(RSe563p,2) - 1./162.
         *m13*m44*RSe453p*RSe563p - 1./324.*m13*m44*pow(RSe453p,2) + 1./
         54.*m13*m44*RSe561p*RSe563p + 1./162.*m13*m44*RSe561p*RSe453p
          - 1./108.*m13*m44*pow(RSe561p,2) + 1./162.*m13*m44*RSe451p*
         RSe563p + 1./162.*m13*m44*RSe451p*RSe453p - 1./162.*m13*m44*
         RSe451p*RSe561p - 1./324.*m13*m44*pow(RSe451p,2) - 1./324.*m11
         *me2*pow(RSe563p,2) + 1./162.*m11*me2*RSe453p*RSe563p - 1./324.
         *m11*me2*pow(RSe453p,2) - 1./162.*m11*me2*RSe561p*RSe563p + 1./
         162.*m11*me2*RSe561p*RSe453p - 1./108.*m11*me2*pow(RSe561p,2)
          + 1./162.*m11*me2*RSe451p*RSe563p - 1./162.*m11*me2*RSe451p*
         RSe453p + 1./54.*m11*me2*RSe451p*RSe561p - 1./108.*m11*me2*
         pow(RSe451p,2) + 1./648.*m11*m66*pow(RSe563p,2) + 1./324.*m11*
         m66*RSe453p*RSe563p + 1./216.*m11*m66*pow(RSe453p,2) + 1./324.
         *m11*m66*RSe561p*RSe563p );

      M13p6x23RV +=  + AbV(m46,L)*BbV(me2,L) * ( 1./324.*m11*m66*
         RSe561p*RSe453p + 1./216.*m11*m66*pow(RSe561p,2) + 1./324.*m11
         *m66*RSe451p*RSe563p + 1./108.*m11*m66*RSe451p*RSe453p + 1./
         108.*m11*m66*RSe451p*RSe561p + 1./72.*m11*m66*pow(RSe451p,2)
          - 1./324.*m11*m46*pow(RSe563p,2) + 1./162.*m11*m46*RSe453p*
         RSe563p - 1./324.*m11*m46*pow(RSe453p,2) - 1./162.*m11*m46*
         RSe561p*RSe563p + 1./162.*m11*m46*RSe561p*RSe453p - 1./108.*
         m11*m46*pow(RSe561p,2) + 1./162.*m11*m46*RSe451p*RSe563p - 1./
         162.*m11*m46*RSe451p*RSe453p + 1./54.*m11*m46*RSe451p*RSe561p
          - 1./108.*m11*m46*pow(RSe451p,2) + 1./216.*m11*m44*pow(
         RSe563p,2) + 1./324.*m11*m44*RSe453p*RSe563p + 1./648.*m11*m44
         *pow(RSe453p,2) + 1./108.*m11*m44*RSe561p*RSe563p + 1./324.*
         m11*m44*RSe561p*RSe453p + 1./72.*m11*m44*pow(RSe561p,2) + 1./
         324.*m11*m44*RSe451p*RSe563p + 1./324.*m11*m44*RSe451p*RSe453p
          + 1./108.*m11*m44*RSe451p*RSe561p + 1./216.*m11*m44*pow(
         RSe451p,2) );

      M13p6x23RV +=  + AbV(m55,L)*AbV(mp2,L) * (  - 1./27.*m55*RSp463e*
         RS346pe - 1./27.*m55*RSp463e*RSe463p - 1./54.*m55*pow(
         RSp463e,2) + 1./27.*m55*RS146pe*RSp463e + 1./81.*m55*RS546pe*
         RSp463e*RS346pe + 1./81.*m55*RS546pe*RSp463e*RSe463p + 1./162.
         *m55*RS546pe*pow(RSp463e,2) - 1./81.*m55*RS546pe*RS146pe*
         RSp463e + 1./27.*m55*RSe461p*RSp463e - 1./81.*m55*RSe461p*
         RS546pe*RSp463e + 1./27.*m55*RSp461e*RS346pe + 1./27.*m55*
         RSp461e*RSe463p + 1./27.*m55*RSp461e*RSp463e - 1./27.*m55*
         RSp461e*RS146pe - 1./81.*m55*RSp461e*RS546pe*RS346pe - 1./81.*
         m55*RSp461e*RS546pe*RSe463p - 1./81.*m55*RSp461e*RS546pe*
         RSp463e + 1./81.*m55*RSp461e*RS546pe*RS146pe - 1./27.*m55*
         RSp461e*RSe461p + 1./81.*m55*RSp461e*RSe461p*RS546pe - 1./54.*
         m55*pow(RSp461e,2) + 1./162.*m55*pow(RSp461e,2)*RS546pe );

      M13p6x23RV +=  + AbV(m55,L)*AbV(me2,L) * (  - 1./27.*m55*RSe463p*
         RS346pe - 1./54.*m55*pow(RSe463p,2) + 1./27.*m55*RS146pe*
         RSe463p + 1./81.*m55*RS546pe*RSe463p*RS346pe + 1./162.*m55*
         RS546pe*pow(RSe463p,2) - 1./81.*m55*RS546pe*RS146pe*RSe463p + 
         1./27.*m55*RSe461p*RS346pe + 1./27.*m55*RSe461p*RSe463p - 1./
         27.*m55*RSe461p*RS146pe - 1./81.*m55*RSe461p*RS546pe*RS346pe
          - 1./81.*m55*RSe461p*RS546pe*RSe463p + 1./81.*m55*RSe461p*
         RS546pe*RS146pe - 1./54.*m55*pow(RSe461p,2) + 1./162.*m55*pow(
         RSe461p,2)*RS546pe );

      M13p6x23RV +=  + AbV(m55,L)*BbV(m11,m33,L) * ( 1./54.*m33*m55*
         RS146pe*RS346pe - 1./162.*m33*m55*RS546pe*RS146pe*RS346pe + 1./
         27.*m13*m55*RS146pe*RS346pe - 1./81.*m13*m55*RS546pe*RS146pe*
         RS346pe + 1./18.*m11*m55*RS146pe*RS346pe - 1./54.*m11*m55*
         RS546pe*RS146pe*RS346pe );

      M13p6x23RV +=  + AbV(m55,L)*BbV(m11,mp2,L) * ( 1./54.*m33*m55*
         RS146pe*RSp463e - 1./162.*m33*m55*RS546pe*RS146pe*RSp463e + 1./
         54.*m33*m55*RSp461e*RS146pe - 1./162.*m33*m55*RSp461e*RS546pe*
         RS146pe + 1./27.*m13*m55*RS146pe*RSp463e - 1./81.*m13*m55*
         RS546pe*RS146pe*RSp463e - 1./27.*m13*m55*RSp461e*RS146pe + 1./
         81.*m13*m55*RSp461e*RS546pe*RS146pe + 1./18.*m11*m55*RS146pe*
         RSp463e - 1./54.*m11*m55*RS546pe*RS146pe*RSp463e + 1./54.*m11*
         m55*RSp461e*RS146pe - 1./162.*m11*m55*RSp461e*RS546pe*RS146pe
          );

      M13p6x23RV +=  + AbV(m55,L)*BbV(m11,me2,L) * ( 1./54.*m33*m55*
         RS146pe*RSe463p - 1./162.*m33*m55*RS546pe*RS146pe*RSe463p + 1./
         54.*m33*m55*RSe461p*RS146pe - 1./162.*m33*m55*RSe461p*RS546pe*
         RS146pe + 1./27.*m13*m55*RS146pe*RSe463p - 1./81.*m13*m55*
         RS546pe*RS146pe*RSe463p - 1./27.*m13*m55*RSe461p*RS146pe + 1./
         81.*m13*m55*RSe461p*RS546pe*RS146pe + 1./18.*m11*m55*RS146pe*
         RSe463p - 1./54.*m11*m55*RS546pe*RS146pe*RSe463p + 1./54.*m11*
         m55*RSe461p*RS146pe - 1./162.*m11*m55*RSe461p*RS546pe*RS146pe
          );

      M13p6x23RV +=  + AbV(m55,L)*BbV(m11,L) * ( 1./108.*m33*m55*pow(
         RS146pe,2) - 1./324.*m33*m55*RS546pe*pow(RS146pe,2) - 1./54.*
         m13*m55*pow(RS146pe,2) + 1./162.*m13*m55*RS546pe*pow(
         RS146pe,2) + 1./108.*m11*m55*pow(RS146pe,2) - 1./324.*m11*m55*
         RS546pe*pow(RS146pe,2) );

      M13p6x23RV +=  + AbV(m55,L)*BbV(m15,L) * ( 1./144.*pow(m55,2) - 1.
         /432.*pow(m55,2)*RS546pe + 1./144.*m33*m55 - 1./432.*m33*m55*
         RS546pe - 1./36.*m15*m55 + 1./108.*m15*m55*RS546pe - 1./72.*
         m15*m33 + 1./216.*m15*m33*RS546pe + 1./36.*pow(m15,2) - 1./108.
         *pow(m15,2)*RS546pe - 1./72.*m13*m55 + 1./216.*m13*m55*RS546pe
          + 1./36.*m13*m15 - 1./108.*m13*m15*RS546pe + 1./48.*m11*m55
          - 1./144.*m11*m55*RS546pe + 1./144.*m11*m33 - 1./432.*m11*m33
         *RS546pe - 1./24.*m11*m15 + 1./72.*m11*m15*RS546pe - 1./72.*
         m11*m13 + 1./216.*m11*m13*RS546pe + 1./72.*pow(m11,2) - 1./216.
         *pow(m11,2)*RS546pe );

      M13p6x23RV +=  + AbV(m55,L)*BbV(m33,mp2,L) * ( 1./54.*m33*m55*
         RSp463e*RS346pe - 1./162.*m33*m55*RS546pe*RSp463e*RS346pe + 1./
         18.*m33*m55*RSp461e*RS346pe - 1./54.*m33*m55*RSp461e*RS546pe*
         RS346pe - 1./27.*m13*m55*RSp463e*RS346pe + 1./81.*m13*m55*
         RS546pe*RSp463e*RS346pe + 1./27.*m13*m55*RSp461e*RS346pe - 1./
         81.*m13*m55*RSp461e*RS546pe*RS346pe + 1./54.*m11*m55*RSp463e*
         RS346pe - 1./162.*m11*m55*RS546pe*RSp463e*RS346pe + 1./54.*m11
         *m55*RSp461e*RS346pe - 1./162.*m11*m55*RSp461e*RS546pe*RS346pe
          );

      M13p6x23RV +=  + AbV(m55,L)*BbV(m33,me2,L) * ( 1./54.*m33*m55*
         RSe463p*RS346pe - 1./162.*m33*m55*RS546pe*RSe463p*RS346pe + 1./
         18.*m33*m55*RSe461p*RS346pe - 1./54.*m33*m55*RSe461p*RS546pe*
         RS346pe - 1./27.*m13*m55*RSe463p*RS346pe + 1./81.*m13*m55*
         RS546pe*RSe463p*RS346pe + 1./27.*m13*m55*RSe461p*RS346pe - 1./
         81.*m13*m55*RSe461p*RS546pe*RS346pe + 1./54.*m11*m55*RSe463p*
         RS346pe - 1./162.*m11*m55*RS546pe*RSe463p*RS346pe + 1./54.*m11
         *m55*RSe461p*RS346pe - 1./162.*m11*m55*RSe461p*RS546pe*RS346pe
          );

      M13p6x23RV +=  + AbV(m55,L)*BbV(m33,L) * ( 1./108.*m33*m55*pow(
         RS346pe,2) - 1./324.*m33*m55*RS546pe*pow(RS346pe,2) - 1./54.*
         m13*m55*pow(RS346pe,2) + 1./162.*m13*m55*RS546pe*pow(
         RS346pe,2) + 1./108.*m11*m55*pow(RS346pe,2) - 1./324.*m11*m55*
         RS546pe*pow(RS346pe,2) );

      M13p6x23RV +=  + AbV(m55,L)*BbV(m35,L) * ( 1./144.*pow(m55,2) - 1.
         /432.*pow(m55,2)*RS546pe - 1./36.*m35*m55 + 1./108.*m35*m55*
         RS546pe + 1./36.*pow(m35,2) - 1./108.*pow(m35,2)*RS546pe + 1./
         48.*m33*m55 - 1./144.*m33*m55*RS546pe - 1./24.*m33*m35 + 1./72.
         *m33*m35*RS546pe + 1./72.*pow(m33,2) - 1./216.*pow(m33,2)*
         RS546pe - 1./72.*m13*m55 + 1./216.*m13*m55*RS546pe + 1./36.*
         m13*m35 - 1./108.*m13*m35*RS546pe - 1./72.*m13*m33 + 1./216.*
         m13*m33*RS546pe + 1./144.*m11*m55 - 1./432.*m11*m55*RS546pe - 
         1./72.*m11*m35 + 1./216.*m11*m35*RS546pe + 1./144.*m11*m33 - 1.
         /432.*m11*m33*RS546pe );

      M13p6x23RV +=  + AbV(m55,L)*BbV(mp2,L) * (  - 1./54.*m55*mp2*pow(
         RSp463e,2) + 1./162.*m55*mp2*RS546pe*pow(RSp463e,2) + 1./27.*
         m55*mp2*RSp461e*RSp463e - 1./81.*m55*mp2*RSp461e*RS546pe*
         RSp463e - 1./54.*m55*mp2*pow(RSp461e,2) + 1./162.*m55*mp2*pow(
         RSp461e,2)*RS546pe + 1./36.*m33*m55*pow(RSp463e,2) - 1./108.*
         m33*m55*RS546pe*pow(RSp463e,2) + 1./54.*m33*m55*RSp461e*
         RSp463e - 1./162.*m33*m55*RSp461e*RS546pe*RSp463e + 1./108.*
         m33*m55*pow(RSp461e,2) - 1./324.*m33*m55*pow(RSp461e,2)*
         RS546pe - 1./54.*m13*m55*pow(RSp463e,2) + 1./162.*m13*m55*
         RS546pe*pow(RSp463e,2) + 1./27.*m13*m55*RSp461e*RSp463e - 1./
         81.*m13*m55*RSp461e*RS546pe*RSp463e - 1./54.*m13*m55*pow(
         RSp461e,2) + 1./162.*m13*m55*pow(RSp461e,2)*RS546pe + 1./108.*
         m11*m55*pow(RSp463e,2) - 1./324.*m11*m55*RS546pe*pow(
         RSp463e,2) + 1./54.*m11*m55*RSp461e*RSp463e - 1./162.*m11*m55*
         RSp461e*RS546pe*RSp463e + 1./36.*m11*m55*pow(RSp461e,2) - 1./
         108.*m11*m55*pow(RSp461e,2)*RS546pe );

      M13p6x23RV +=  + AbV(m55,L)*BbV(me2,mp2,L) * (  - 1./27.*m55*me2*
         RSp463e*RSe463p + 1./81.*m55*me2*RS546pe*RSp463e*RSe463p + 1./
         27.*m55*me2*RSe461p*RSp463e - 1./81.*m55*me2*RSe461p*RS546pe*
         RSp463e + 1./27.*m55*me2*RSp461e*RSe463p - 1./81.*m55*me2*
         RSp461e*RS546pe*RSe463p - 1./27.*m55*me2*RSp461e*RSe461p + 1./
         81.*m55*me2*RSp461e*RSe461p*RS546pe + 1./18.*m33*m55*RSp463e*
         RSe463p - 1./54.*m33*m55*RS546pe*RSp463e*RSe463p + 1./54.*m33*
         m55*RSe461p*RSp463e - 1./162.*m33*m55*RSe461p*RS546pe*RSp463e
          + 1./54.*m33*m55*RSp461e*RSe463p - 1./162.*m33*m55*RSp461e*
         RS546pe*RSe463p + 1./54.*m33*m55*RSp461e*RSe461p - 1./162.*m33
         *m55*RSp461e*RSe461p*RS546pe - 1./27.*m13*m55*RSp463e*RSe463p
          + 1./81.*m13*m55*RS546pe*RSp463e*RSe463p + 1./27.*m13*m55*
         RSe461p*RSp463e - 1./81.*m13*m55*RSe461p*RS546pe*RSp463e + 1./
         27.*m13*m55*RSp461e*RSe463p - 1./81.*m13*m55*RSp461e*RS546pe*
         RSe463p - 1./27.*m13*m55*RSp461e*RSe461p + 1./81.*m13*m55*
         RSp461e*RSe461p*RS546pe );

      M13p6x23RV +=  + AbV(m55,L)*BbV(me2,mp2,L) * ( 1./54.*m11*m55*
         RSp463e*RSe463p - 1./162.*m11*m55*RS546pe*RSp463e*RSe463p + 1./
         54.*m11*m55*RSe461p*RSp463e - 1./162.*m11*m55*RSe461p*RS546pe*
         RSp463e + 1./54.*m11*m55*RSp461e*RSe463p - 1./162.*m11*m55*
         RSp461e*RS546pe*RSe463p + 1./18.*m11*m55*RSp461e*RSe461p - 1./
         54.*m11*m55*RSp461e*RSe461p*RS546pe );

      M13p6x23RV +=  + AbV(m55,L)*BbV(me2,L) * (  - 1./54.*m55*me2*pow(
         RSe463p,2) + 1./162.*m55*me2*RS546pe*pow(RSe463p,2) + 1./27.*
         m55*me2*RSe461p*RSe463p - 1./81.*m55*me2*RSe461p*RS546pe*
         RSe463p - 1./54.*m55*me2*pow(RSe461p,2) + 1./162.*m55*me2*pow(
         RSe461p,2)*RS546pe + 1./36.*m33*m55*pow(RSe463p,2) - 1./108.*
         m33*m55*RS546pe*pow(RSe463p,2) + 1./54.*m33*m55*RSe461p*
         RSe463p - 1./162.*m33*m55*RSe461p*RS546pe*RSe463p + 1./108.*
         m33*m55*pow(RSe461p,2) - 1./324.*m33*m55*pow(RSe461p,2)*
         RS546pe - 1./54.*m13*m55*pow(RSe463p,2) + 1./162.*m13*m55*
         RS546pe*pow(RSe463p,2) + 1./27.*m13*m55*RSe461p*RSe463p - 1./
         81.*m13*m55*RSe461p*RS546pe*RSe463p - 1./54.*m13*m55*pow(
         RSe461p,2) + 1./162.*m13*m55*pow(RSe461p,2)*RS546pe + 1./108.*
         m11*m55*pow(RSe463p,2) - 1./324.*m11*m55*RS546pe*pow(
         RSe463p,2) + 1./54.*m11*m55*RSe461p*RSe463p - 1./162.*m11*m55*
         RSe461p*RS546pe*RSe463p + 1./36.*m11*m55*pow(RSe461p,2) - 1./
         108.*m11*m55*pow(RSe461p,2)*RS546pe );

      M13p6x23RV +=  + AbV(m56,L)*AbV(mp2,L) * (  - 1./81.*mp2*RSp463e*
         RS345pe + 1./81.*mp2*RSp463e*RS346pe + 1./81.*mp2*RSp463e*
         RSe463p - 1./81.*mp2*RSp463e*RSe453p + 1./81.*mp2*pow(
         RSp463e,2) + 1./81.*mp2*RSp453e*RS345pe - 1./81.*mp2*RSp453e*
         RS346pe - 1./81.*mp2*RSp453e*RSe463p + 1./81.*mp2*RSp453e*
         RSe453p - 2./81.*mp2*RSp453e*RSp463e + 1./81.*mp2*pow(
         RSp453e,2) + 1./81.*mp2*RS145pe*RSp463e - 1./81.*mp2*RS145pe*
         RSp453e - 1./81.*mp2*RS146pe*RSp463e + 1./81.*mp2*RS146pe*
         RSp453e - 1./81.*mp2*RSe461p*RSp463e + 1./81.*mp2*RSe461p*
         RSp453e + 1./81.*mp2*RSe451p*RSp463e - 1./81.*mp2*RSe451p*
         RSp453e + 1./81.*mp2*RSp461e*RS345pe - 1./81.*mp2*RSp461e*
         RS346pe - 1./81.*mp2*RSp461e*RSe463p + 1./81.*mp2*RSp461e*
         RSe453p - 2./81.*mp2*RSp461e*RSp463e + 2./81.*mp2*RSp461e*
         RSp453e - 1./81.*mp2*RSp461e*RS145pe + 1./81.*mp2*RSp461e*
         RS146pe + 1./81.*mp2*RSp461e*RSe461p - 1./81.*mp2*RSp461e*
         RSe451p );

      M13p6x23RV +=  + AbV(m56,L)*AbV(mp2,L) * ( 1./81.*mp2*pow(
         RSp461e,2) - 1./81.*mp2*RSp451e*RS345pe + 1./81.*mp2*RSp451e*
         RS346pe + 1./81.*mp2*RSp451e*RSe463p - 1./81.*mp2*RSp451e*
         RSe453p + 2./81.*mp2*RSp451e*RSp463e - 2./81.*mp2*RSp451e*
         RSp453e + 1./81.*mp2*RSp451e*RS145pe - 1./81.*mp2*RSp451e*
         RS146pe - 1./81.*mp2*RSp451e*RSe461p + 1./81.*mp2*RSp451e*
         RSe451p - 2./81.*mp2*RSp451e*RSp461e + 1./81.*mp2*pow(
         RSp451e,2) - 1./162.*m66*RSp463e*RS345pe - 1./162.*m66*RSp463e
         *RS346pe - 1./162.*m66*RSp463e*RSe463p - 1./162.*m66*RSp463e*
         RSe453p - 1./324.*m66*pow(RSp463e,2) - 1./54.*m66*RSp453e*
         RS345pe - 1./162.*m66*RSp453e*RS346pe - 1./162.*m66*RSp453e*
         RSe463p - 1./54.*m66*RSp453e*RSe453p - 1./162.*m66*RSp453e*
         RSp463e - 1./108.*m66*pow(RSp453e,2) + 1./162.*m66*RS145pe*
         RSp463e + 1./54.*m66*RS145pe*RSp453e + 1./162.*m66*RS146pe*
         RSp463e + 1./162.*m66*RS146pe*RSp453e + 1./162.*m66*RSe461p*
         RSp463e );

      M13p6x23RV +=  + AbV(m56,L)*AbV(mp2,L) * ( 1./162.*m66*RSe461p*
         RSp453e + 1./162.*m66*RSe451p*RSp463e + 1./54.*m66*RSe451p*
         RSp453e + 1./162.*m66*RSp461e*RS345pe + 1./162.*m66*RSp461e*
         RS346pe + 1./162.*m66*RSp461e*RSe463p + 1./162.*m66*RSp461e*
         RSe453p + 1./162.*m66*RSp461e*RSp463e + 1./162.*m66*RSp461e*
         RSp453e - 1./162.*m66*RSp461e*RS145pe - 1./162.*m66*RSp461e*
         RS146pe - 1./162.*m66*RSp461e*RSe461p - 1./162.*m66*RSp461e*
         RSe451p - 1./324.*m66*pow(RSp461e,2) + 1./54.*m66*RSp451e*
         RS345pe + 1./162.*m66*RSp451e*RS346pe + 1./162.*m66*RSp451e*
         RSe463p + 1./54.*m66*RSp451e*RSe453p + 1./162.*m66*RSp451e*
         RSp463e + 1./54.*m66*RSp451e*RSp453e - 1./54.*m66*RSp451e*
         RS145pe - 1./162.*m66*RSp451e*RS146pe - 1./162.*m66*RSp451e*
         RSe461p - 1./54.*m66*RSp451e*RSe451p - 1./162.*m66*RSp451e*
         RSp461e - 1./108.*m66*pow(RSp451e,2) - 1./81.*m56*RSp463e*
         RS345pe + 1./81.*m56*RSp463e*RS346pe + 1./81.*m56*RSp463e*
         RSe463p );

      M13p6x23RV +=  + AbV(m56,L)*AbV(mp2,L) * (  - 1./81.*m56*RSp463e*
         RSe453p + 1./162.*m56*pow(RSp463e,2) + 1./81.*m56*RSp453e*
         RS345pe - 1./81.*m56*RSp453e*RS346pe - 1./81.*m56*RSp453e*
         RSe463p + 1./81.*m56*RSp453e*RSe453p - 1./81.*m56*RSp453e*
         RSp463e + 1./162.*m56*pow(RSp453e,2) + 1./81.*m56*RS145pe*
         RSp463e - 1./81.*m56*RS145pe*RSp453e - 1./81.*m56*RS146pe*
         RSp463e + 1./81.*m56*RS146pe*RSp453e - 1./81.*m56*RSe461p*
         RSp463e + 1./81.*m56*RSe461p*RSp453e + 1./81.*m56*RSe451p*
         RSp463e - 1./81.*m56*RSe451p*RSp453e + 1./81.*m56*RSp461e*
         RS345pe - 1./81.*m56*RSp461e*RS346pe - 1./81.*m56*RSp461e*
         RSe463p + 1./81.*m56*RSp461e*RSe453p - 1./81.*m56*RSp461e*
         RSp463e + 1./81.*m56*RSp461e*RSp453e - 1./81.*m56*RSp461e*
         RS145pe + 1./81.*m56*RSp461e*RS146pe + 1./81.*m56*RSp461e*
         RSe461p - 1./81.*m56*RSp461e*RSe451p + 1./162.*m56*pow(
         RSp461e,2) - 1./81.*m56*RSp451e*RS345pe + 1./81.*m56*RSp451e*
         RS346pe );

      M13p6x23RV +=  + AbV(m56,L)*AbV(mp2,L) * ( 1./81.*m56*RSp451e*
         RSe463p - 1./81.*m56*RSp451e*RSe453p + 1./81.*m56*RSp451e*
         RSp463e - 1./81.*m56*RSp451e*RSp453e + 1./81.*m56*RSp451e*
         RS145pe - 1./81.*m56*RSp451e*RS146pe - 1./81.*m56*RSp451e*
         RSe461p + 1./81.*m56*RSp451e*RSe451p - 1./81.*m56*RSp451e*
         RSp461e + 1./162.*m56*pow(RSp451e,2) - 1./162.*m55*RSp463e*
         RS345pe - 1./54.*m55*RSp463e*RS346pe - 1./54.*m55*RSp463e*
         RSe463p - 1./162.*m55*RSp463e*RSe453p - 1./108.*m55*pow(
         RSp463e,2) - 1./162.*m55*RSp453e*RS345pe - 1./162.*m55*RSp453e
         *RS346pe - 1./162.*m55*RSp453e*RSe463p - 1./162.*m55*RSp453e*
         RSe453p - 1./162.*m55*RSp453e*RSp463e - 1./324.*m55*pow(
         RSp453e,2) + 1./162.*m55*RS145pe*RSp463e + 1./162.*m55*RS145pe
         *RSp453e + 1./54.*m55*RS146pe*RSp463e + 1./162.*m55*RS146pe*
         RSp453e + 1./54.*m55*RSe461p*RSp463e + 1./162.*m55*RSe461p*
         RSp453e + 1./162.*m55*RSe451p*RSp463e + 1./162.*m55*RSe451p*
         RSp453e );

      M13p6x23RV +=  + AbV(m56,L)*AbV(mp2,L) * ( 1./162.*m55*RSp461e*
         RS345pe + 1./54.*m55*RSp461e*RS346pe + 1./54.*m55*RSp461e*
         RSe463p + 1./162.*m55*RSp461e*RSe453p + 1./54.*m55*RSp461e*
         RSp463e + 1./162.*m55*RSp461e*RSp453e - 1./162.*m55*RSp461e*
         RS145pe - 1./54.*m55*RSp461e*RS146pe - 1./54.*m55*RSp461e*
         RSe461p - 1./162.*m55*RSp461e*RSe451p - 1./108.*m55*pow(
         RSp461e,2) + 1./162.*m55*RSp451e*RS345pe + 1./162.*m55*RSp451e
         *RS346pe + 1./162.*m55*RSp451e*RSe463p + 1./162.*m55*RSp451e*
         RSe453p + 1./162.*m55*RSp451e*RSp463e + 1./162.*m55*RSp451e*
         RSp453e - 1./162.*m55*RSp451e*RS145pe - 1./162.*m55*RSp451e*
         RS146pe - 1./162.*m55*RSp451e*RSe461p - 1./162.*m55*RSp451e*
         RSe451p - 1./162.*m55*RSp451e*RSp461e - 1./324.*m55*pow(
         RSp451e,2) + 1./54.*m33*RSp463e*RS345pe - 1./54.*m33*RSp463e*
         RS346pe - 1./54.*m33*RSp463e*RSe463p + 1./54.*m33*RSp463e*
         RSe453p - 1./108.*m33*pow(RSp463e,2) - 1./54.*m33*RSp453e*
         RS345pe );

      M13p6x23RV +=  + AbV(m56,L)*AbV(mp2,L) * ( 1./54.*m33*RSp453e*
         RS346pe + 1./54.*m33*RSp453e*RSe463p - 1./54.*m33*RSp453e*
         RSe453p + 1./54.*m33*RSp453e*RSp463e - 1./108.*m33*pow(
         RSp453e,2) + 1./162.*m33*RS145pe*RSp463e - 1./162.*m33*RS145pe
         *RSp453e - 1./162.*m33*RS146pe*RSp463e + 1./162.*m33*RS146pe*
         RSp453e - 1./162.*m33*RSe461p*RSp463e + 1./162.*m33*RSe461p*
         RSp453e + 1./162.*m33*RSe451p*RSp463e - 1./162.*m33*RSe451p*
         RSp453e + 1./162.*m33*RSp461e*RS345pe - 1./162.*m33*RSp461e*
         RS346pe - 1./162.*m33*RSp461e*RSe463p + 1./162.*m33*RSp461e*
         RSe453p - 1./162.*m33*RSp461e*RSp463e + 1./162.*m33*RSp461e*
         RSp453e + 1./162.*m33*RSp461e*RS145pe - 1./162.*m33*RSp461e*
         RS146pe - 1./162.*m33*RSp461e*RSe461p + 1./162.*m33*RSp461e*
         RSe451p - 1./324.*m33*pow(RSp461e,2) - 1./162.*m33*RSp451e*
         RS345pe + 1./162.*m33*RSp451e*RS346pe + 1./162.*m33*RSp451e*
         RSe463p - 1./162.*m33*RSp451e*RSe453p + 1./162.*m33*RSp451e*
         RSp463e );

      M13p6x23RV +=  + AbV(m56,L)*AbV(mp2,L) * (  - 1./162.*m33*RSp451e
         *RSp453e - 1./162.*m33*RSp451e*RS145pe + 1./162.*m33*RSp451e*
         RS146pe + 1./162.*m33*RSp451e*RSe461p - 1./162.*m33*RSp451e*
         RSe451p + 1./162.*m33*RSp451e*RSp461e - 1./324.*m33*pow(
         RSp451e,2) - 1./81.*m13*RSp463e*RS345pe + 1./81.*m13*RSp463e*
         RS346pe + 1./81.*m13*RSp463e*RSe463p - 1./81.*m13*RSp463e*
         RSe453p + 1./162.*m13*pow(RSp463e,2) + 1./81.*m13*RSp453e*
         RS345pe - 1./81.*m13*RSp453e*RS346pe - 1./81.*m13*RSp453e*
         RSe463p + 1./81.*m13*RSp453e*RSe453p - 1./81.*m13*RSp453e*
         RSp463e + 1./162.*m13*pow(RSp453e,2) + 1./81.*m13*RS145pe*
         RSp463e - 1./81.*m13*RS145pe*RSp453e - 1./81.*m13*RS146pe*
         RSp463e + 1./81.*m13*RS146pe*RSp453e - 1./81.*m13*RSe461p*
         RSp463e + 1./81.*m13*RSe461p*RSp453e + 1./81.*m13*RSe451p*
         RSp463e - 1./81.*m13*RSe451p*RSp453e + 1./81.*m13*RSp461e*
         RS345pe - 1./81.*m13*RSp461e*RS346pe - 1./81.*m13*RSp461e*
         RSe463p );

      M13p6x23RV +=  + AbV(m56,L)*AbV(mp2,L) * ( 1./81.*m13*RSp461e*
         RSe453p - 1./81.*m13*RSp461e*RSp463e + 1./81.*m13*RSp461e*
         RSp453e - 1./81.*m13*RSp461e*RS145pe + 1./81.*m13*RSp461e*
         RS146pe + 1./81.*m13*RSp461e*RSe461p - 1./81.*m13*RSp461e*
         RSe451p + 1./162.*m13*pow(RSp461e,2) - 1./81.*m13*RSp451e*
         RS345pe + 1./81.*m13*RSp451e*RS346pe + 1./81.*m13*RSp451e*
         RSe463p - 1./81.*m13*RSp451e*RSe453p + 1./81.*m13*RSp451e*
         RSp463e - 1./81.*m13*RSp451e*RSp453e + 1./81.*m13*RSp451e*
         RS145pe - 1./81.*m13*RSp451e*RS146pe - 1./81.*m13*RSp451e*
         RSe461p + 1./81.*m13*RSp451e*RSe451p - 1./81.*m13*RSp451e*
         RSp461e + 1./162.*m13*pow(RSp451e,2) + 1./162.*m11*RSp463e*
         RS345pe - 1./162.*m11*RSp463e*RS346pe - 1./162.*m11*RSp463e*
         RSe463p + 1./162.*m11*RSp463e*RSe453p - 1./324.*m11*pow(
         RSp463e,2) - 1./162.*m11*RSp453e*RS345pe + 1./162.*m11*RSp453e
         *RS346pe + 1./162.*m11*RSp453e*RSe463p - 1./162.*m11*RSp453e*
         RSe453p );

      M13p6x23RV +=  + AbV(m56,L)*AbV(mp2,L) * ( 1./162.*m11*RSp453e*
         RSp463e - 1./324.*m11*pow(RSp453e,2) + 1./162.*m11*RS145pe*
         RSp463e - 1./162.*m11*RS145pe*RSp453e - 1./162.*m11*RS146pe*
         RSp463e + 1./162.*m11*RS146pe*RSp453e - 1./162.*m11*RSe461p*
         RSp463e + 1./162.*m11*RSe461p*RSp453e + 1./162.*m11*RSe451p*
         RSp463e - 1./162.*m11*RSe451p*RSp453e + 1./162.*m11*RSp461e*
         RS345pe - 1./162.*m11*RSp461e*RS346pe - 1./162.*m11*RSp461e*
         RSe463p + 1./162.*m11*RSp461e*RSe453p - 1./162.*m11*RSp461e*
         RSp463e + 1./162.*m11*RSp461e*RSp453e + 1./54.*m11*RSp461e*
         RS145pe - 1./54.*m11*RSp461e*RS146pe - 1./54.*m11*RSp461e*
         RSe461p + 1./54.*m11*RSp461e*RSe451p - 1./108.*m11*pow(
         RSp461e,2) - 1./162.*m11*RSp451e*RS345pe + 1./162.*m11*RSp451e
         *RS346pe + 1./162.*m11*RSp451e*RSe463p - 1./162.*m11*RSp451e*
         RSe453p + 1./162.*m11*RSp451e*RSp463e - 1./162.*m11*RSp451e*
         RSp453e - 1./54.*m11*RSp451e*RS145pe + 1./54.*m11*RSp451e*
         RS146pe );

      M13p6x23RV +=  + AbV(m56,L)*AbV(mp2,L) * ( 1./54.*m11*RSp451e*
         RSe461p - 1./54.*m11*RSp451e*RSe451p + 1./54.*m11*RSp451e*
         RSp461e - 1./108.*m11*pow(RSp451e,2) );

      M13p6x23RV +=  + AbV(m56,L)*AbV(me2,L) * (  - 1./81.*me2*RSe463p*
         RS345pe + 1./81.*me2*RSe463p*RS346pe + 1./81.*me2*pow(
         RSe463p,2) + 1./81.*me2*RSe453p*RS345pe - 1./81.*me2*RSe453p*
         RS346pe - 2./81.*me2*RSe453p*RSe463p + 1./81.*me2*pow(
         RSe453p,2) + 1./81.*me2*RSp463e*RSe463p - 1./81.*me2*RSp463e*
         RSe453p - 1./81.*me2*RSp453e*RSe463p + 1./81.*me2*RSp453e*
         RSe453p + 1./81.*me2*RS145pe*RSe463p - 1./81.*me2*RS145pe*
         RSe453p - 1./81.*me2*RS146pe*RSe463p + 1./81.*me2*RS146pe*
         RSe453p + 1./81.*me2*RSe461p*RS345pe - 1./81.*me2*RSe461p*
         RS346pe - 2./81.*me2*RSe461p*RSe463p + 2./81.*me2*RSe461p*
         RSe453p - 1./81.*me2*RSe461p*RSp463e + 1./81.*me2*RSe461p*
         RSp453e - 1./81.*me2*RSe461p*RS145pe + 1./81.*me2*RSe461p*
         RS146pe + 1./81.*me2*pow(RSe461p,2) - 1./81.*me2*RSe451p*
         RS345pe + 1./81.*me2*RSe451p*RS346pe + 2./81.*me2*RSe451p*
         RSe463p - 2./81.*me2*RSe451p*RSe453p + 1./81.*me2*RSe451p*
         RSp463e );

      M13p6x23RV +=  + AbV(m56,L)*AbV(me2,L) * (  - 1./81.*me2*RSe451p*
         RSp453e + 1./81.*me2*RSe451p*RS145pe - 1./81.*me2*RSe451p*
         RS146pe - 2./81.*me2*RSe451p*RSe461p + 1./81.*me2*pow(
         RSe451p,2) - 1./81.*me2*RSp461e*RSe463p + 1./81.*me2*RSp461e*
         RSe453p + 1./81.*me2*RSp461e*RSe461p - 1./81.*me2*RSp461e*
         RSe451p + 1./81.*me2*RSp451e*RSe463p - 1./81.*me2*RSp451e*
         RSe453p - 1./81.*me2*RSp451e*RSe461p + 1./81.*me2*RSp451e*
         RSe451p - 1./162.*m66*RSe463p*RS345pe - 1./162.*m66*RSe463p*
         RS346pe - 1./324.*m66*pow(RSe463p,2) - 1./54.*m66*RSe453p*
         RS345pe - 1./162.*m66*RSe453p*RS346pe - 1./162.*m66*RSe453p*
         RSe463p - 1./108.*m66*pow(RSe453p,2) + 1./162.*m66*RS145pe*
         RSe463p + 1./54.*m66*RS145pe*RSe453p + 1./162.*m66*RS146pe*
         RSe463p + 1./162.*m66*RS146pe*RSe453p + 1./162.*m66*RSe461p*
         RS345pe + 1./162.*m66*RSe461p*RS346pe + 1./162.*m66*RSe461p*
         RSe463p + 1./162.*m66*RSe461p*RSe453p - 1./162.*m66*RSe461p*
         RS145pe );

      M13p6x23RV +=  + AbV(m56,L)*AbV(me2,L) * (  - 1./162.*m66*RSe461p
         *RS146pe - 1./324.*m66*pow(RSe461p,2) + 1./54.*m66*RSe451p*
         RS345pe + 1./162.*m66*RSe451p*RS346pe + 1./162.*m66*RSe451p*
         RSe463p + 1./54.*m66*RSe451p*RSe453p - 1./54.*m66*RSe451p*
         RS145pe - 1./162.*m66*RSe451p*RS146pe - 1./162.*m66*RSe451p*
         RSe461p - 1./108.*m66*pow(RSe451p,2) - 1./81.*m56*RSe463p*
         RS345pe + 1./81.*m56*RSe463p*RS346pe + 1./162.*m56*pow(
         RSe463p,2) + 1./81.*m56*RSe453p*RS345pe - 1./81.*m56*RSe453p*
         RS346pe - 1./81.*m56*RSe453p*RSe463p + 1./162.*m56*pow(
         RSe453p,2) + 1./81.*m56*RS145pe*RSe463p - 1./81.*m56*RS145pe*
         RSe453p - 1./81.*m56*RS146pe*RSe463p + 1./81.*m56*RS146pe*
         RSe453p + 1./81.*m56*RSe461p*RS345pe - 1./81.*m56*RSe461p*
         RS346pe - 1./81.*m56*RSe461p*RSe463p + 1./81.*m56*RSe461p*
         RSe453p - 1./81.*m56*RSe461p*RS145pe + 1./81.*m56*RSe461p*
         RS146pe + 1./162.*m56*pow(RSe461p,2) - 1./81.*m56*RSe451p*
         RS345pe );

      M13p6x23RV +=  + AbV(m56,L)*AbV(me2,L) * ( 1./81.*m56*RSe451p*
         RS346pe + 1./81.*m56*RSe451p*RSe463p - 1./81.*m56*RSe451p*
         RSe453p + 1./81.*m56*RSe451p*RS145pe - 1./81.*m56*RSe451p*
         RS146pe - 1./81.*m56*RSe451p*RSe461p + 1./162.*m56*pow(
         RSe451p,2) - 1./162.*m55*RSe463p*RS345pe - 1./54.*m55*RSe463p*
         RS346pe - 1./108.*m55*pow(RSe463p,2) - 1./162.*m55*RSe453p*
         RS345pe - 1./162.*m55*RSe453p*RS346pe - 1./162.*m55*RSe453p*
         RSe463p - 1./324.*m55*pow(RSe453p,2) + 1./162.*m55*RS145pe*
         RSe463p + 1./162.*m55*RS145pe*RSe453p + 1./54.*m55*RS146pe*
         RSe463p + 1./162.*m55*RS146pe*RSe453p + 1./162.*m55*RSe461p*
         RS345pe + 1./54.*m55*RSe461p*RS346pe + 1./54.*m55*RSe461p*
         RSe463p + 1./162.*m55*RSe461p*RSe453p - 1./162.*m55*RSe461p*
         RS145pe - 1./54.*m55*RSe461p*RS146pe - 1./108.*m55*pow(
         RSe461p,2) + 1./162.*m55*RSe451p*RS345pe + 1./162.*m55*RSe451p
         *RS346pe + 1./162.*m55*RSe451p*RSe463p + 1./162.*m55*RSe451p*
         RSe453p );

      M13p6x23RV +=  + AbV(m56,L)*AbV(me2,L) * (  - 1./162.*m55*RSe451p
         *RS145pe - 1./162.*m55*RSe451p*RS146pe - 1./162.*m55*RSe451p*
         RSe461p - 1./324.*m55*pow(RSe451p,2) + 1./54.*m33*RSe463p*
         RS345pe - 1./54.*m33*RSe463p*RS346pe - 1./108.*m33*pow(
         RSe463p,2) - 1./54.*m33*RSe453p*RS345pe + 1./54.*m33*RSe453p*
         RS346pe + 1./54.*m33*RSe453p*RSe463p - 1./108.*m33*pow(
         RSe453p,2) + 1./162.*m33*RS145pe*RSe463p - 1./162.*m33*RS145pe
         *RSe453p - 1./162.*m33*RS146pe*RSe463p + 1./162.*m33*RS146pe*
         RSe453p + 1./162.*m33*RSe461p*RS345pe - 1./162.*m33*RSe461p*
         RS346pe - 1./162.*m33*RSe461p*RSe463p + 1./162.*m33*RSe461p*
         RSe453p + 1./162.*m33*RSe461p*RS145pe - 1./162.*m33*RSe461p*
         RS146pe - 1./324.*m33*pow(RSe461p,2) - 1./162.*m33*RSe451p*
         RS345pe + 1./162.*m33*RSe451p*RS346pe + 1./162.*m33*RSe451p*
         RSe463p - 1./162.*m33*RSe451p*RSe453p - 1./162.*m33*RSe451p*
         RS145pe + 1./162.*m33*RSe451p*RS146pe + 1./162.*m33*RSe451p*
         RSe461p );

      M13p6x23RV +=  + AbV(m56,L)*AbV(me2,L) * (  - 1./324.*m33*pow(
         RSe451p,2) - 1./81.*m13*RSe463p*RS345pe + 1./81.*m13*RSe463p*
         RS346pe + 1./162.*m13*pow(RSe463p,2) + 1./81.*m13*RSe453p*
         RS345pe - 1./81.*m13*RSe453p*RS346pe - 1./81.*m13*RSe453p*
         RSe463p + 1./162.*m13*pow(RSe453p,2) + 1./81.*m13*RS145pe*
         RSe463p - 1./81.*m13*RS145pe*RSe453p - 1./81.*m13*RS146pe*
         RSe463p + 1./81.*m13*RS146pe*RSe453p + 1./81.*m13*RSe461p*
         RS345pe - 1./81.*m13*RSe461p*RS346pe - 1./81.*m13*RSe461p*
         RSe463p + 1./81.*m13*RSe461p*RSe453p - 1./81.*m13*RSe461p*
         RS145pe + 1./81.*m13*RSe461p*RS146pe + 1./162.*m13*pow(
         RSe461p,2) - 1./81.*m13*RSe451p*RS345pe + 1./81.*m13*RSe451p*
         RS346pe + 1./81.*m13*RSe451p*RSe463p - 1./81.*m13*RSe451p*
         RSe453p + 1./81.*m13*RSe451p*RS145pe - 1./81.*m13*RSe451p*
         RS146pe - 1./81.*m13*RSe451p*RSe461p + 1./162.*m13*pow(
         RSe451p,2) + 1./162.*m11*RSe463p*RS345pe - 1./162.*m11*RSe463p
         *RS346pe );

      M13p6x23RV +=  + AbV(m56,L)*AbV(me2,L) * (  - 1./324.*m11*pow(
         RSe463p,2) - 1./162.*m11*RSe453p*RS345pe + 1./162.*m11*RSe453p
         *RS346pe + 1./162.*m11*RSe453p*RSe463p - 1./324.*m11*pow(
         RSe453p,2) + 1./162.*m11*RS145pe*RSe463p - 1./162.*m11*RS145pe
         *RSe453p - 1./162.*m11*RS146pe*RSe463p + 1./162.*m11*RS146pe*
         RSe453p + 1./162.*m11*RSe461p*RS345pe - 1./162.*m11*RSe461p*
         RS346pe - 1./162.*m11*RSe461p*RSe463p + 1./162.*m11*RSe461p*
         RSe453p + 1./54.*m11*RSe461p*RS145pe - 1./54.*m11*RSe461p*
         RS146pe - 1./108.*m11*pow(RSe461p,2) - 1./162.*m11*RSe451p*
         RS345pe + 1./162.*m11*RSe451p*RS346pe + 1./162.*m11*RSe451p*
         RSe463p - 1./162.*m11*RSe451p*RSe453p - 1./54.*m11*RSe451p*
         RS145pe + 1./54.*m11*RSe451p*RS146pe + 1./54.*m11*RSe451p*
         RSe461p - 1./108.*m11*pow(RSe451p,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m11,m33,L) * ( 1./108.*m33*m66*
         RS145pe*RS345pe + 1./324.*m33*m66*RS145pe*RS346pe + 1./324.*
         m33*m66*RS146pe*RS345pe + 1./324.*m33*m66*RS146pe*RS346pe - 1./
         162.*m33*m56*RS145pe*RS345pe + 1./162.*m33*m56*RS145pe*RS346pe
          + 1./162.*m33*m56*RS146pe*RS345pe - 1./162.*m33*m56*RS146pe*
         RS346pe + 1./324.*m33*m55*RS145pe*RS345pe + 1./324.*m33*m55*
         RS145pe*RS346pe + 1./324.*m33*m55*RS146pe*RS345pe + 1./108.*
         m33*m55*RS146pe*RS346pe + 1./54.*m13*m66*RS145pe*RS345pe + 1./
         162.*m13*m66*RS145pe*RS346pe + 1./162.*m13*m66*RS146pe*RS345pe
          + 1./162.*m13*m66*RS146pe*RS346pe - 1./81.*m13*m56*RS145pe*
         RS345pe + 1./81.*m13*m56*RS145pe*RS346pe + 1./81.*m13*m56*
         RS146pe*RS345pe - 1./81.*m13*m56*RS146pe*RS346pe + 1./162.*m13
         *m55*RS145pe*RS345pe + 1./162.*m13*m55*RS145pe*RS346pe + 1./
         162.*m13*m55*RS146pe*RS345pe + 1./54.*m13*m55*RS146pe*RS346pe
          + 1./36.*m11*m66*RS145pe*RS345pe + 1./108.*m11*m66*RS145pe*
         RS346pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m11,m33,L) * ( 1./108.*m11*m66*
         RS146pe*RS345pe + 1./108.*m11*m66*RS146pe*RS346pe - 1./54.*m11
         *m56*RS145pe*RS345pe + 1./54.*m11*m56*RS145pe*RS346pe + 1./54.
         *m11*m56*RS146pe*RS345pe - 1./54.*m11*m56*RS146pe*RS346pe + 1./
         108.*m11*m55*RS145pe*RS345pe + 1./108.*m11*m55*RS145pe*RS346pe
          + 1./108.*m11*m55*RS146pe*RS345pe + 1./36.*m11*m55*RS146pe*
         RS346pe - 1./54.*m11*m33*RS145pe*RS345pe + 1./54.*m11*m33*
         RS145pe*RS346pe + 1./54.*m11*m33*RS146pe*RS345pe - 1./54.*m11*
         m33*RS146pe*RS346pe - 1./81.*m11*m13*RS145pe*RS345pe + 1./81.*
         m11*m13*RS145pe*RS346pe + 1./81.*m11*m13*RS146pe*RS345pe - 1./
         81.*m11*m13*RS146pe*RS346pe - 1./162.*pow(m11,2)*RS145pe*
         RS345pe + 1./162.*pow(m11,2)*RS145pe*RS346pe + 1./162.*pow(
         m11,2)*RS146pe*RS345pe - 1./162.*pow(m11,2)*RS146pe*RS346pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m11,mp2,L) * ( 1./324.*m33*m66*
         RS145pe*RSp463e + 1./108.*m33*m66*RS145pe*RSp453e + 1./324.*
         m33*m66*RS146pe*RSp463e + 1./324.*m33*m66*RS146pe*RSp453e + 1./
         324.*m33*m66*RSp461e*RS145pe + 1./324.*m33*m66*RSp461e*RS146pe
          + 1./108.*m33*m66*RSp451e*RS145pe + 1./324.*m33*m66*RSp451e*
         RS146pe + 1./162.*m33*m56*RS145pe*RSp463e - 1./162.*m33*m56*
         RS145pe*RSp453e - 1./162.*m33*m56*RS146pe*RSp463e + 1./162.*
         m33*m56*RS146pe*RSp453e + 1./162.*m33*m56*RSp461e*RS145pe - 1./
         162.*m33*m56*RSp461e*RS146pe - 1./162.*m33*m56*RSp451e*RS145pe
          + 1./162.*m33*m56*RSp451e*RS146pe + 1./324.*m33*m55*RS145pe*
         RSp463e + 1./324.*m33*m55*RS145pe*RSp453e + 1./108.*m33*m55*
         RS146pe*RSp463e + 1./324.*m33*m55*RS146pe*RSp453e + 1./324.*
         m33*m55*RSp461e*RS145pe + 1./108.*m33*m55*RSp461e*RS146pe + 1./
         324.*m33*m55*RSp451e*RS145pe + 1./324.*m33*m55*RSp451e*RS146pe
          + 1./162.*m13*m66*RS145pe*RSp463e + 1./54.*m13*m66*RS145pe*
         RSp453e );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m11,mp2,L) * ( 1./162.*m13*m66*
         RS146pe*RSp463e + 1./162.*m13*m66*RS146pe*RSp453e - 1./162.*
         m13*m66*RSp461e*RS145pe - 1./162.*m13*m66*RSp461e*RS146pe - 1./
         54.*m13*m66*RSp451e*RS145pe - 1./162.*m13*m66*RSp451e*RS146pe
          + 1./81.*m13*m56*RS145pe*RSp463e - 1./81.*m13*m56*RS145pe*
         RSp453e - 1./81.*m13*m56*RS146pe*RSp463e + 1./81.*m13*m56*
         RS146pe*RSp453e - 1./81.*m13*m56*RSp461e*RS145pe + 1./81.*m13*
         m56*RSp461e*RS146pe + 1./81.*m13*m56*RSp451e*RS145pe - 1./81.*
         m13*m56*RSp451e*RS146pe + 1./162.*m13*m55*RS145pe*RSp463e + 1./
         162.*m13*m55*RS145pe*RSp453e + 1./54.*m13*m55*RS146pe*RSp463e
          + 1./162.*m13*m55*RS146pe*RSp453e - 1./162.*m13*m55*RSp461e*
         RS145pe - 1./54.*m13*m55*RSp461e*RS146pe - 1./162.*m13*m55*
         RSp451e*RS145pe - 1./162.*m13*m55*RSp451e*RS146pe + 1./81.*m11
         *mp2*RS145pe*RSp463e - 1./81.*m11*mp2*RS145pe*RSp453e - 1./81.
         *m11*mp2*RS146pe*RSp463e + 1./81.*m11*mp2*RS146pe*RSp453e - 1./
         81.*m11*mp2*RSp461e*RS145pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m11,mp2,L) * ( 1./81.*m11*mp2*
         RSp461e*RS146pe + 1./81.*m11*mp2*RSp451e*RS145pe - 1./81.*m11*
         mp2*RSp451e*RS146pe + 1./108.*m11*m66*RS145pe*RSp463e + 1./36.
         *m11*m66*RS145pe*RSp453e + 1./108.*m11*m66*RS146pe*RSp463e + 1.
         /108.*m11*m66*RS146pe*RSp453e + 1./324.*m11*m66*RSp461e*
         RS145pe + 1./324.*m11*m66*RSp461e*RS146pe + 1./108.*m11*m66*
         RSp451e*RS145pe + 1./324.*m11*m66*RSp451e*RS146pe + 1./54.*m11
         *m56*RS145pe*RSp463e - 1./54.*m11*m56*RS145pe*RSp453e - 1./54.
         *m11*m56*RS146pe*RSp463e + 1./54.*m11*m56*RS146pe*RSp453e + 1./
         162.*m11*m56*RSp461e*RS145pe - 1./162.*m11*m56*RSp461e*RS146pe
          - 1./162.*m11*m56*RSp451e*RS145pe + 1./162.*m11*m56*RSp451e*
         RS146pe + 1./108.*m11*m55*RS145pe*RSp463e + 1./108.*m11*m55*
         RS145pe*RSp453e + 1./36.*m11*m55*RS146pe*RSp463e + 1./108.*m11
         *m55*RS146pe*RSp453e + 1./324.*m11*m55*RSp461e*RS145pe + 1./
         108.*m11*m55*RSp461e*RS146pe + 1./324.*m11*m55*RSp451e*RS145pe
          + 1./324.*m11*m55*RSp451e*RS146pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m11,mp2,L) * ( 1./162.*m11*m33*
         RS145pe*RSp463e - 1./162.*m11*m33*RS145pe*RSp453e - 1./162.*
         m11*m33*RS146pe*RSp463e + 1./162.*m11*m33*RS146pe*RSp453e + 1./
         162.*m11*m33*RSp461e*RS145pe - 1./162.*m11*m33*RSp461e*RS146pe
          - 1./162.*m11*m33*RSp451e*RS145pe + 1./162.*m11*m33*RSp451e*
         RS146pe + 1./81.*m11*m13*RS145pe*RSp463e - 1./81.*m11*m13*
         RS145pe*RSp453e - 1./81.*m11*m13*RS146pe*RSp463e + 1./81.*m11*
         m13*RS146pe*RSp453e - 1./81.*m11*m13*RSp461e*RS145pe + 1./81.*
         m11*m13*RSp461e*RS146pe + 1./81.*m11*m13*RSp451e*RS145pe - 1./
         81.*m11*m13*RSp451e*RS146pe + 1./162.*pow(m11,2)*RS145pe*
         RSp463e - 1./162.*pow(m11,2)*RS145pe*RSp453e - 1./162.*pow(
         m11,2)*RS146pe*RSp463e + 1./162.*pow(m11,2)*RS146pe*RSp453e + 
         1./54.*pow(m11,2)*RSp461e*RS145pe - 1./54.*pow(m11,2)*RSp461e*
         RS146pe - 1./54.*pow(m11,2)*RSp451e*RS145pe + 1./54.*pow(
         m11,2)*RSp451e*RS146pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m11,me2,L) * ( 1./324.*m33*m66*
         RS145pe*RSe463p + 1./108.*m33*m66*RS145pe*RSe453p + 1./324.*
         m33*m66*RS146pe*RSe463p + 1./324.*m33*m66*RS146pe*RSe453p + 1./
         324.*m33*m66*RSe461p*RS145pe + 1./324.*m33*m66*RSe461p*RS146pe
          + 1./108.*m33*m66*RSe451p*RS145pe + 1./324.*m33*m66*RSe451p*
         RS146pe + 1./162.*m33*m56*RS145pe*RSe463p - 1./162.*m33*m56*
         RS145pe*RSe453p - 1./162.*m33*m56*RS146pe*RSe463p + 1./162.*
         m33*m56*RS146pe*RSe453p + 1./162.*m33*m56*RSe461p*RS145pe - 1./
         162.*m33*m56*RSe461p*RS146pe - 1./162.*m33*m56*RSe451p*RS145pe
          + 1./162.*m33*m56*RSe451p*RS146pe + 1./324.*m33*m55*RS145pe*
         RSe463p + 1./324.*m33*m55*RS145pe*RSe453p + 1./108.*m33*m55*
         RS146pe*RSe463p + 1./324.*m33*m55*RS146pe*RSe453p + 1./324.*
         m33*m55*RSe461p*RS145pe + 1./108.*m33*m55*RSe461p*RS146pe + 1./
         324.*m33*m55*RSe451p*RS145pe + 1./324.*m33*m55*RSe451p*RS146pe
          + 1./162.*m13*m66*RS145pe*RSe463p + 1./54.*m13*m66*RS145pe*
         RSe453p );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m11,me2,L) * ( 1./162.*m13*m66*
         RS146pe*RSe463p + 1./162.*m13*m66*RS146pe*RSe453p - 1./162.*
         m13*m66*RSe461p*RS145pe - 1./162.*m13*m66*RSe461p*RS146pe - 1./
         54.*m13*m66*RSe451p*RS145pe - 1./162.*m13*m66*RSe451p*RS146pe
          + 1./81.*m13*m56*RS145pe*RSe463p - 1./81.*m13*m56*RS145pe*
         RSe453p - 1./81.*m13*m56*RS146pe*RSe463p + 1./81.*m13*m56*
         RS146pe*RSe453p - 1./81.*m13*m56*RSe461p*RS145pe + 1./81.*m13*
         m56*RSe461p*RS146pe + 1./81.*m13*m56*RSe451p*RS145pe - 1./81.*
         m13*m56*RSe451p*RS146pe + 1./162.*m13*m55*RS145pe*RSe463p + 1./
         162.*m13*m55*RS145pe*RSe453p + 1./54.*m13*m55*RS146pe*RSe463p
          + 1./162.*m13*m55*RS146pe*RSe453p - 1./162.*m13*m55*RSe461p*
         RS145pe - 1./54.*m13*m55*RSe461p*RS146pe - 1./162.*m13*m55*
         RSe451p*RS145pe - 1./162.*m13*m55*RSe451p*RS146pe + 1./81.*m11
         *me2*RS145pe*RSe463p - 1./81.*m11*me2*RS145pe*RSe453p - 1./81.
         *m11*me2*RS146pe*RSe463p + 1./81.*m11*me2*RS146pe*RSe453p - 1./
         81.*m11*me2*RSe461p*RS145pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m11,me2,L) * ( 1./81.*m11*me2*
         RSe461p*RS146pe + 1./81.*m11*me2*RSe451p*RS145pe - 1./81.*m11*
         me2*RSe451p*RS146pe + 1./108.*m11*m66*RS145pe*RSe463p + 1./36.
         *m11*m66*RS145pe*RSe453p + 1./108.*m11*m66*RS146pe*RSe463p + 1.
         /108.*m11*m66*RS146pe*RSe453p + 1./324.*m11*m66*RSe461p*
         RS145pe + 1./324.*m11*m66*RSe461p*RS146pe + 1./108.*m11*m66*
         RSe451p*RS145pe + 1./324.*m11*m66*RSe451p*RS146pe + 1./54.*m11
         *m56*RS145pe*RSe463p - 1./54.*m11*m56*RS145pe*RSe453p - 1./54.
         *m11*m56*RS146pe*RSe463p + 1./54.*m11*m56*RS146pe*RSe453p + 1./
         162.*m11*m56*RSe461p*RS145pe - 1./162.*m11*m56*RSe461p*RS146pe
          - 1./162.*m11*m56*RSe451p*RS145pe + 1./162.*m11*m56*RSe451p*
         RS146pe + 1./108.*m11*m55*RS145pe*RSe463p + 1./108.*m11*m55*
         RS145pe*RSe453p + 1./36.*m11*m55*RS146pe*RSe463p + 1./108.*m11
         *m55*RS146pe*RSe453p + 1./324.*m11*m55*RSe461p*RS145pe + 1./
         108.*m11*m55*RSe461p*RS146pe + 1./324.*m11*m55*RSe451p*RS145pe
          + 1./324.*m11*m55*RSe451p*RS146pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m11,me2,L) * ( 1./162.*m11*m33*
         RS145pe*RSe463p - 1./162.*m11*m33*RS145pe*RSe453p - 1./162.*
         m11*m33*RS146pe*RSe463p + 1./162.*m11*m33*RS146pe*RSe453p + 1./
         162.*m11*m33*RSe461p*RS145pe - 1./162.*m11*m33*RSe461p*RS146pe
          - 1./162.*m11*m33*RSe451p*RS145pe + 1./162.*m11*m33*RSe451p*
         RS146pe + 1./81.*m11*m13*RS145pe*RSe463p - 1./81.*m11*m13*
         RS145pe*RSe453p - 1./81.*m11*m13*RS146pe*RSe463p + 1./81.*m11*
         m13*RS146pe*RSe453p - 1./81.*m11*m13*RSe461p*RS145pe + 1./81.*
         m11*m13*RSe461p*RS146pe + 1./81.*m11*m13*RSe451p*RS145pe - 1./
         81.*m11*m13*RSe451p*RS146pe + 1./162.*pow(m11,2)*RS145pe*
         RSe463p - 1./162.*pow(m11,2)*RS145pe*RSe453p - 1./162.*pow(
         m11,2)*RS146pe*RSe463p + 1./162.*pow(m11,2)*RS146pe*RSe453p + 
         1./54.*pow(m11,2)*RSe461p*RS145pe - 1./54.*pow(m11,2)*RSe461p*
         RS146pe - 1./54.*pow(m11,2)*RSe451p*RS145pe + 1./54.*pow(
         m11,2)*RSe451p*RS146pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m11,L) * ( 1./216.*m33*m66*pow(
         RS145pe,2) + 1./324.*m33*m66*RS146pe*RS145pe + 1./648.*m33*m66
         *pow(RS146pe,2) - 1./324.*m33*m56*pow(RS145pe,2) + 1./162.*m33
         *m56*RS146pe*RS145pe - 1./324.*m33*m56*pow(RS146pe,2) + 1./648.
         *m33*m55*pow(RS145pe,2) + 1./324.*m33*m55*RS146pe*RS145pe + 1./
         216.*m33*m55*pow(RS146pe,2) - 1./108.*m13*m66*pow(RS145pe,2)
          - 1./162.*m13*m66*RS146pe*RS145pe - 1./324.*m13*m66*pow(
         RS146pe,2) + 1./162.*m13*m56*pow(RS145pe,2) - 1./81.*m13*m56*
         RS146pe*RS145pe + 1./162.*m13*m56*pow(RS146pe,2) - 1./324.*m13
         *m55*pow(RS145pe,2) - 1./162.*m13*m55*RS146pe*RS145pe - 1./108.
         *m13*m55*pow(RS146pe,2) + 1./216.*m11*m66*pow(RS145pe,2) + 1./
         324.*m11*m66*RS146pe*RS145pe + 1./648.*m11*m66*pow(RS146pe,2)
          - 1./324.*m11*m56*pow(RS145pe,2) + 1./162.*m11*m56*RS146pe*
         RS145pe - 1./324.*m11*m56*pow(RS146pe,2) + 1./648.*m11*m55*
         pow(RS145pe,2) + 1./324.*m11*m55*RS146pe*RS145pe + 1./216.*m11
         *m55*pow(RS146pe,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m11,L) * (  - 1./324.*m11*m33*
         pow(RS145pe,2) + 1./162.*m11*m33*RS146pe*RS145pe - 1./324.*m11
         *m33*pow(RS146pe,2) + 1./162.*m11*m13*pow(RS145pe,2) - 1./81.*
         m11*m13*RS146pe*RS145pe + 1./162.*m11*m13*pow(RS146pe,2) - 1./
         324.*pow(m11,2)*pow(RS145pe,2) + 1./162.*pow(m11,2)*RS146pe*
         RS145pe - 1./324.*pow(m11,2)*pow(RS146pe,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m15,L) * ( 1./144.*m55*m66 - 1./
         72.*m55*m56 + 1./72.*pow(m55,2) + 1./144.*m33*m66 - 1./72.*m33
         *m56 + 1./72.*m33*m55 - 1./72.*m15*m66 + 1./36.*m15*m56 - 1./
         24.*m15*m55 - 1./72.*m15*m33 + 1./36.*pow(m15,2) - 1./72.*m13*
         m66 + 1./36.*m13*m56 - 1./36.*m13*m55 + 1./36.*m13*m15 + 1./72.
         *m11*m66 - 1./36.*m11*m56 + 5./144.*m11*m55 + 1./144.*m11*m33
          - 1./24.*m11*m15 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m16,L) * ( 1./72.*pow(m66,2) - 1./
         72.*m56*m66 + 1./144.*m55*m66 + 1./72.*m33*m66 - 1./72.*m33*
         m56 + 1./144.*m33*m55 - 1./24.*m16*m66 + 1./36.*m16*m56 - 1./
         72.*m16*m55 - 1./72.*m16*m33 + 1./36.*pow(m16,2) - 1./36.*m13*
         m66 + 1./36.*m13*m56 - 1./72.*m13*m55 + 1./36.*m13*m16 + 5./
         144.*m11*m66 - 1./36.*m11*m56 + 1./72.*m11*m55 + 1./144.*m11*
         m33 - 1./24.*m11*m16 - 1./72.*m11*m13 + 1./72.*pow(m11,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m33,mp2,L) * (  - 1./81.*m33*mp2*
         RSp463e*RS345pe + 1./81.*m33*mp2*RSp463e*RS346pe + 1./81.*m33*
         mp2*RSp453e*RS345pe - 1./81.*m33*mp2*RSp453e*RS346pe + 1./81.*
         m33*mp2*RSp461e*RS345pe - 1./81.*m33*mp2*RSp461e*RS346pe - 1./
         81.*m33*mp2*RSp451e*RS345pe + 1./81.*m33*mp2*RSp451e*RS346pe
          + 1./324.*m33*m66*RSp463e*RS345pe + 1./324.*m33*m66*RSp463e*
         RS346pe + 1./108.*m33*m66*RSp453e*RS345pe + 1./324.*m33*m66*
         RSp453e*RS346pe + 1./108.*m33*m66*RSp461e*RS345pe + 1./108.*
         m33*m66*RSp461e*RS346pe + 1./36.*m33*m66*RSp451e*RS345pe + 1./
         108.*m33*m66*RSp451e*RS346pe + 1./162.*m33*m56*RSp463e*RS345pe
          - 1./162.*m33*m56*RSp463e*RS346pe - 1./162.*m33*m56*RSp453e*
         RS345pe + 1./162.*m33*m56*RSp453e*RS346pe + 1./54.*m33*m56*
         RSp461e*RS345pe - 1./54.*m33*m56*RSp461e*RS346pe - 1./54.*m33*
         m56*RSp451e*RS345pe + 1./54.*m33*m56*RSp451e*RS346pe + 1./324.
         *m33*m55*RSp463e*RS345pe + 1./108.*m33*m55*RSp463e*RS346pe + 1.
         /324.*m33*m55*RSp453e*RS345pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m33,mp2,L) * ( 1./324.*m33*m55*
         RSp453e*RS346pe + 1./108.*m33*m55*RSp461e*RS345pe + 1./36.*m33
         *m55*RSp461e*RS346pe + 1./108.*m33*m55*RSp451e*RS345pe + 1./
         108.*m33*m55*RSp451e*RS346pe + 1./54.*pow(m33,2)*RSp463e*
         RS345pe - 1./54.*pow(m33,2)*RSp463e*RS346pe - 1./54.*pow(
         m33,2)*RSp453e*RS345pe + 1./54.*pow(m33,2)*RSp453e*RS346pe + 1.
         /162.*pow(m33,2)*RSp461e*RS345pe - 1./162.*pow(m33,2)*RSp461e*
         RS346pe - 1./162.*pow(m33,2)*RSp451e*RS345pe + 1./162.*pow(
         m33,2)*RSp451e*RS346pe - 1./162.*m13*m66*RSp463e*RS345pe - 1./
         162.*m13*m66*RSp463e*RS346pe - 1./54.*m13*m66*RSp453e*RS345pe
          - 1./162.*m13*m66*RSp453e*RS346pe + 1./162.*m13*m66*RSp461e*
         RS345pe + 1./162.*m13*m66*RSp461e*RS346pe + 1./54.*m13*m66*
         RSp451e*RS345pe + 1./162.*m13*m66*RSp451e*RS346pe - 1./81.*m13
         *m56*RSp463e*RS345pe + 1./81.*m13*m56*RSp463e*RS346pe + 1./81.
         *m13*m56*RSp453e*RS345pe - 1./81.*m13*m56*RSp453e*RS346pe + 1./
         81.*m13*m56*RSp461e*RS345pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m33,mp2,L) * (  - 1./81.*m13*m56*
         RSp461e*RS346pe - 1./81.*m13*m56*RSp451e*RS345pe + 1./81.*m13*
         m56*RSp451e*RS346pe - 1./162.*m13*m55*RSp463e*RS345pe - 1./54.
         *m13*m55*RSp463e*RS346pe - 1./162.*m13*m55*RSp453e*RS345pe - 1.
         /162.*m13*m55*RSp453e*RS346pe + 1./162.*m13*m55*RSp461e*
         RS345pe + 1./54.*m13*m55*RSp461e*RS346pe + 1./162.*m13*m55*
         RSp451e*RS345pe + 1./162.*m13*m55*RSp451e*RS346pe - 1./81.*m13
         *m33*RSp463e*RS345pe + 1./81.*m13*m33*RSp463e*RS346pe + 1./81.
         *m13*m33*RSp453e*RS345pe - 1./81.*m13*m33*RSp453e*RS346pe + 1./
         81.*m13*m33*RSp461e*RS345pe - 1./81.*m13*m33*RSp461e*RS346pe
          - 1./81.*m13*m33*RSp451e*RS345pe + 1./81.*m13*m33*RSp451e*
         RS346pe + 1./324.*m11*m66*RSp463e*RS345pe + 1./324.*m11*m66*
         RSp463e*RS346pe + 1./108.*m11*m66*RSp453e*RS345pe + 1./324.*
         m11*m66*RSp453e*RS346pe + 1./324.*m11*m66*RSp461e*RS345pe + 1./
         324.*m11*m66*RSp461e*RS346pe + 1./108.*m11*m66*RSp451e*RS345pe
          + 1./324.*m11*m66*RSp451e*RS346pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m33,mp2,L) * ( 1./162.*m11*m56*
         RSp463e*RS345pe - 1./162.*m11*m56*RSp463e*RS346pe - 1./162.*
         m11*m56*RSp453e*RS345pe + 1./162.*m11*m56*RSp453e*RS346pe + 1./
         162.*m11*m56*RSp461e*RS345pe - 1./162.*m11*m56*RSp461e*RS346pe
          - 1./162.*m11*m56*RSp451e*RS345pe + 1./162.*m11*m56*RSp451e*
         RS346pe + 1./324.*m11*m55*RSp463e*RS345pe + 1./108.*m11*m55*
         RSp463e*RS346pe + 1./324.*m11*m55*RSp453e*RS345pe + 1./324.*
         m11*m55*RSp453e*RS346pe + 1./324.*m11*m55*RSp461e*RS345pe + 1./
         108.*m11*m55*RSp461e*RS346pe + 1./324.*m11*m55*RSp451e*RS345pe
          + 1./324.*m11*m55*RSp451e*RS346pe + 1./162.*m11*m33*RSp463e*
         RS345pe - 1./162.*m11*m33*RSp463e*RS346pe - 1./162.*m11*m33*
         RSp453e*RS345pe + 1./162.*m11*m33*RSp453e*RS346pe + 1./162.*
         m11*m33*RSp461e*RS345pe - 1./162.*m11*m33*RSp461e*RS346pe - 1./
         162.*m11*m33*RSp451e*RS345pe + 1./162.*m11*m33*RSp451e*RS346pe
          );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m33,me2,L) * (  - 1./81.*m33*me2*
         RSe463p*RS345pe + 1./81.*m33*me2*RSe463p*RS346pe + 1./81.*m33*
         me2*RSe453p*RS345pe - 1./81.*m33*me2*RSe453p*RS346pe + 1./81.*
         m33*me2*RSe461p*RS345pe - 1./81.*m33*me2*RSe461p*RS346pe - 1./
         81.*m33*me2*RSe451p*RS345pe + 1./81.*m33*me2*RSe451p*RS346pe
          + 1./324.*m33*m66*RSe463p*RS345pe + 1./324.*m33*m66*RSe463p*
         RS346pe + 1./108.*m33*m66*RSe453p*RS345pe + 1./324.*m33*m66*
         RSe453p*RS346pe + 1./108.*m33*m66*RSe461p*RS345pe + 1./108.*
         m33*m66*RSe461p*RS346pe + 1./36.*m33*m66*RSe451p*RS345pe + 1./
         108.*m33*m66*RSe451p*RS346pe + 1./162.*m33*m56*RSe463p*RS345pe
          - 1./162.*m33*m56*RSe463p*RS346pe - 1./162.*m33*m56*RSe453p*
         RS345pe + 1./162.*m33*m56*RSe453p*RS346pe + 1./54.*m33*m56*
         RSe461p*RS345pe - 1./54.*m33*m56*RSe461p*RS346pe - 1./54.*m33*
         m56*RSe451p*RS345pe + 1./54.*m33*m56*RSe451p*RS346pe + 1./324.
         *m33*m55*RSe463p*RS345pe + 1./108.*m33*m55*RSe463p*RS346pe + 1.
         /324.*m33*m55*RSe453p*RS345pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m33,me2,L) * ( 1./324.*m33*m55*
         RSe453p*RS346pe + 1./108.*m33*m55*RSe461p*RS345pe + 1./36.*m33
         *m55*RSe461p*RS346pe + 1./108.*m33*m55*RSe451p*RS345pe + 1./
         108.*m33*m55*RSe451p*RS346pe + 1./54.*pow(m33,2)*RSe463p*
         RS345pe - 1./54.*pow(m33,2)*RSe463p*RS346pe - 1./54.*pow(
         m33,2)*RSe453p*RS345pe + 1./54.*pow(m33,2)*RSe453p*RS346pe + 1.
         /162.*pow(m33,2)*RSe461p*RS345pe - 1./162.*pow(m33,2)*RSe461p*
         RS346pe - 1./162.*pow(m33,2)*RSe451p*RS345pe + 1./162.*pow(
         m33,2)*RSe451p*RS346pe - 1./162.*m13*m66*RSe463p*RS345pe - 1./
         162.*m13*m66*RSe463p*RS346pe - 1./54.*m13*m66*RSe453p*RS345pe
          - 1./162.*m13*m66*RSe453p*RS346pe + 1./162.*m13*m66*RSe461p*
         RS345pe + 1./162.*m13*m66*RSe461p*RS346pe + 1./54.*m13*m66*
         RSe451p*RS345pe + 1./162.*m13*m66*RSe451p*RS346pe - 1./81.*m13
         *m56*RSe463p*RS345pe + 1./81.*m13*m56*RSe463p*RS346pe + 1./81.
         *m13*m56*RSe453p*RS345pe - 1./81.*m13*m56*RSe453p*RS346pe + 1./
         81.*m13*m56*RSe461p*RS345pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m33,me2,L) * (  - 1./81.*m13*m56*
         RSe461p*RS346pe - 1./81.*m13*m56*RSe451p*RS345pe + 1./81.*m13*
         m56*RSe451p*RS346pe - 1./162.*m13*m55*RSe463p*RS345pe - 1./54.
         *m13*m55*RSe463p*RS346pe - 1./162.*m13*m55*RSe453p*RS345pe - 1.
         /162.*m13*m55*RSe453p*RS346pe + 1./162.*m13*m55*RSe461p*
         RS345pe + 1./54.*m13*m55*RSe461p*RS346pe + 1./162.*m13*m55*
         RSe451p*RS345pe + 1./162.*m13*m55*RSe451p*RS346pe - 1./81.*m13
         *m33*RSe463p*RS345pe + 1./81.*m13*m33*RSe463p*RS346pe + 1./81.
         *m13*m33*RSe453p*RS345pe - 1./81.*m13*m33*RSe453p*RS346pe + 1./
         81.*m13*m33*RSe461p*RS345pe - 1./81.*m13*m33*RSe461p*RS346pe
          - 1./81.*m13*m33*RSe451p*RS345pe + 1./81.*m13*m33*RSe451p*
         RS346pe + 1./324.*m11*m66*RSe463p*RS345pe + 1./324.*m11*m66*
         RSe463p*RS346pe + 1./108.*m11*m66*RSe453p*RS345pe + 1./324.*
         m11*m66*RSe453p*RS346pe + 1./324.*m11*m66*RSe461p*RS345pe + 1./
         324.*m11*m66*RSe461p*RS346pe + 1./108.*m11*m66*RSe451p*RS345pe
          + 1./324.*m11*m66*RSe451p*RS346pe );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m33,me2,L) * ( 1./162.*m11*m56*
         RSe463p*RS345pe - 1./162.*m11*m56*RSe463p*RS346pe - 1./162.*
         m11*m56*RSe453p*RS345pe + 1./162.*m11*m56*RSe453p*RS346pe + 1./
         162.*m11*m56*RSe461p*RS345pe - 1./162.*m11*m56*RSe461p*RS346pe
          - 1./162.*m11*m56*RSe451p*RS345pe + 1./162.*m11*m56*RSe451p*
         RS346pe + 1./324.*m11*m55*RSe463p*RS345pe + 1./108.*m11*m55*
         RSe463p*RS346pe + 1./324.*m11*m55*RSe453p*RS345pe + 1./324.*
         m11*m55*RSe453p*RS346pe + 1./324.*m11*m55*RSe461p*RS345pe + 1./
         108.*m11*m55*RSe461p*RS346pe + 1./324.*m11*m55*RSe451p*RS345pe
          + 1./324.*m11*m55*RSe451p*RS346pe + 1./162.*m11*m33*RSe463p*
         RS345pe - 1./162.*m11*m33*RSe463p*RS346pe - 1./162.*m11*m33*
         RSe453p*RS345pe + 1./162.*m11*m33*RSe453p*RS346pe + 1./162.*
         m11*m33*RSe461p*RS345pe - 1./162.*m11*m33*RSe461p*RS346pe - 1./
         162.*m11*m33*RSe451p*RS345pe + 1./162.*m11*m33*RSe451p*RS346pe
          );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m33,L) * ( 1./216.*m33*m66*pow(
         RS345pe,2) + 1./324.*m33*m66*RS346pe*RS345pe + 1./648.*m33*m66
         *pow(RS346pe,2) - 1./324.*m33*m56*pow(RS345pe,2) + 1./162.*m33
         *m56*RS346pe*RS345pe - 1./324.*m33*m56*pow(RS346pe,2) + 1./648.
         *m33*m55*pow(RS345pe,2) + 1./324.*m33*m55*RS346pe*RS345pe + 1./
         216.*m33*m55*pow(RS346pe,2) - 1./324.*pow(m33,2)*pow(
         RS345pe,2) + 1./162.*pow(m33,2)*RS346pe*RS345pe - 1./324.*pow(
         m33,2)*pow(RS346pe,2) - 1./108.*m13*m66*pow(RS345pe,2) - 1./
         162.*m13*m66*RS346pe*RS345pe - 1./324.*m13*m66*pow(RS346pe,2)
          + 1./162.*m13*m56*pow(RS345pe,2) - 1./81.*m13*m56*RS346pe*
         RS345pe + 1./162.*m13*m56*pow(RS346pe,2) - 1./324.*m13*m55*
         pow(RS345pe,2) - 1./162.*m13*m55*RS346pe*RS345pe - 1./108.*m13
         *m55*pow(RS346pe,2) + 1./162.*m13*m33*pow(RS345pe,2) - 1./81.*
         m13*m33*RS346pe*RS345pe + 1./162.*m13*m33*pow(RS346pe,2) + 1./
         216.*m11*m66*pow(RS345pe,2) + 1./324.*m11*m66*RS346pe*RS345pe
          + 1./648.*m11*m66*pow(RS346pe,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m33,L) * (  - 1./324.*m11*m56*
         pow(RS345pe,2) + 1./162.*m11*m56*RS346pe*RS345pe - 1./324.*m11
         *m56*pow(RS346pe,2) + 1./648.*m11*m55*pow(RS345pe,2) + 1./324.
         *m11*m55*RS346pe*RS345pe + 1./216.*m11*m55*pow(RS346pe,2) - 1./
         324.*m11*m33*pow(RS345pe,2) + 1./162.*m11*m33*RS346pe*RS345pe
          - 1./324.*m11*m33*pow(RS346pe,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m35,L) * ( 1./144.*m55*m66 - 1./
         72.*m55*m56 + 1./72.*pow(m55,2) - 1./72.*m35*m66 + 1./36.*m35*
         m56 - 1./24.*m35*m55 + 1./36.*pow(m35,2) + 1./72.*m33*m66 - 1./
         36.*m33*m56 + 5./144.*m33*m55 - 1./24.*m33*m35 + 1./72.*pow(
         m33,2) - 1./72.*m13*m66 + 1./36.*m13*m56 - 1./36.*m13*m55 + 1./
         36.*m13*m35 - 1./72.*m13*m33 + 1./144.*m11*m66 - 1./72.*m11*
         m56 + 1./72.*m11*m55 - 1./72.*m11*m35 + 1./144.*m11*m33 );

      M13p6x23RV +=  + AbV(m56,L)*BbV(m36,L) * ( 1./72.*pow(m66,2) - 1./
         72.*m56*m66 + 1./144.*m55*m66 - 1./24.*m36*m66 + 1./36.*m36*
         m56 - 1./72.*m36*m55 + 1./36.*pow(m36,2) + 5./144.*m33*m66 - 1.
         /36.*m33*m56 + 1./72.*m33*m55 - 1./24.*m33*m36 + 1./72.*pow(
         m33,2) - 1./36.*m13*m66 + 1./36.*m13*m56 - 1./72.*m13*m55 + 1./
         36.*m13*m36 - 1./72.*m13*m33 + 1./72.*m11*m66 - 1./72.*m11*m56
          + 1./144.*m11*m55 - 1./72.*m11*m36 + 1./144.*m11*m33 );

      M13p6x23RV +=  + AbV(m56,L)*BbV(mp2,L) * ( 1./162.*pow(mp2,2)*
         pow(RSp463e,2) - 1./81.*pow(mp2,2)*RSp453e*RSp463e + 1./162.*
         pow(mp2,2)*pow(RSp453e,2) - 1./81.*pow(mp2,2)*RSp461e*RSp463e
          + 1./81.*pow(mp2,2)*RSp461e*RSp453e + 1./162.*pow(mp2,2)*pow(
         RSp461e,2) + 1./81.*pow(mp2,2)*RSp451e*RSp463e - 1./81.*pow(
         mp2,2)*RSp451e*RSp453e - 1./81.*pow(mp2,2)*RSp451e*RSp461e + 1.
         /162.*pow(mp2,2)*pow(RSp451e,2) - 1./324.*m66*mp2*pow(
         RSp463e,2) - 1./162.*m66*mp2*RSp453e*RSp463e - 1./108.*m66*mp2
         *pow(RSp453e,2) + 1./162.*m66*mp2*RSp461e*RSp463e + 1./162.*
         m66*mp2*RSp461e*RSp453e - 1./324.*m66*mp2*pow(RSp461e,2) + 1./
         162.*m66*mp2*RSp451e*RSp463e + 1./54.*m66*mp2*RSp451e*RSp453e
          - 1./162.*m66*mp2*RSp451e*RSp461e - 1./108.*m66*mp2*pow(
         RSp451e,2) + 1./162.*m56*mp2*pow(RSp463e,2) - 1./81.*m56*mp2*
         RSp453e*RSp463e + 1./162.*m56*mp2*pow(RSp453e,2) - 1./81.*m56*
         mp2*RSp461e*RSp463e + 1./81.*m56*mp2*RSp461e*RSp453e + 1./162.
         *m56*mp2*pow(RSp461e,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(mp2,L) * ( 1./81.*m56*mp2*RSp451e
         *RSp463e - 1./81.*m56*mp2*RSp451e*RSp453e - 1./81.*m56*mp2*
         RSp451e*RSp461e + 1./162.*m56*mp2*pow(RSp451e,2) - 1./108.*m55
         *mp2*pow(RSp463e,2) - 1./162.*m55*mp2*RSp453e*RSp463e - 1./324.
         *m55*mp2*pow(RSp453e,2) + 1./54.*m55*mp2*RSp461e*RSp463e + 1./
         162.*m55*mp2*RSp461e*RSp453e - 1./108.*m55*mp2*pow(RSp461e,2)
          + 1./162.*m55*mp2*RSp451e*RSp463e + 1./162.*m55*mp2*RSp451e*
         RSp453e - 1./162.*m55*mp2*RSp451e*RSp461e - 1./324.*m55*mp2*
         pow(RSp451e,2) - 1./108.*m33*mp2*pow(RSp463e,2) + 1./54.*m33*
         mp2*RSp453e*RSp463e - 1./108.*m33*mp2*pow(RSp453e,2) - 1./162.
         *m33*mp2*RSp461e*RSp463e + 1./162.*m33*mp2*RSp461e*RSp453e - 1.
         /324.*m33*mp2*pow(RSp461e,2) + 1./162.*m33*mp2*RSp451e*RSp463e
          - 1./162.*m33*mp2*RSp451e*RSp453e + 1./162.*m33*mp2*RSp451e*
         RSp461e - 1./324.*m33*mp2*pow(RSp451e,2) + 1./216.*m33*m66*
         pow(RSp463e,2) + 1./108.*m33*m66*RSp453e*RSp463e + 1./72.*m33*
         m66*pow(RSp453e,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(mp2,L) * ( 1./324.*m33*m66*
         RSp461e*RSp463e + 1./324.*m33*m66*RSp461e*RSp453e + 1./648.*
         m33*m66*pow(RSp461e,2) + 1./324.*m33*m66*RSp451e*RSp463e + 1./
         108.*m33*m66*RSp451e*RSp453e + 1./324.*m33*m66*RSp451e*RSp461e
          + 1./216.*m33*m66*pow(RSp451e,2) - 1./108.*m33*m56*pow(
         RSp463e,2) + 1./54.*m33*m56*RSp453e*RSp463e - 1./108.*m33*m56*
         pow(RSp453e,2) - 1./162.*m33*m56*RSp461e*RSp463e + 1./162.*m33
         *m56*RSp461e*RSp453e - 1./324.*m33*m56*pow(RSp461e,2) + 1./162.
         *m33*m56*RSp451e*RSp463e - 1./162.*m33*m56*RSp451e*RSp453e + 1.
         /162.*m33*m56*RSp451e*RSp461e - 1./324.*m33*m56*pow(RSp451e,2)
          + 1./72.*m33*m55*pow(RSp463e,2) + 1./108.*m33*m55*RSp453e*
         RSp463e + 1./216.*m33*m55*pow(RSp453e,2) + 1./108.*m33*m55*
         RSp461e*RSp463e + 1./324.*m33*m55*RSp461e*RSp453e + 1./216.*
         m33*m55*pow(RSp461e,2) + 1./324.*m33*m55*RSp451e*RSp463e + 1./
         324.*m33*m55*RSp451e*RSp453e + 1./324.*m33*m55*RSp451e*RSp461e
          + 1./648.*m33*m55*pow(RSp451e,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(mp2,L) * ( 1./162.*m13*mp2*pow(
         RSp463e,2) - 1./81.*m13*mp2*RSp453e*RSp463e + 1./162.*m13*mp2*
         pow(RSp453e,2) - 1./81.*m13*mp2*RSp461e*RSp463e + 1./81.*m13*
         mp2*RSp461e*RSp453e + 1./162.*m13*mp2*pow(RSp461e,2) + 1./81.*
         m13*mp2*RSp451e*RSp463e - 1./81.*m13*mp2*RSp451e*RSp453e - 1./
         81.*m13*mp2*RSp451e*RSp461e + 1./162.*m13*mp2*pow(RSp451e,2)
          - 1./324.*m13*m66*pow(RSp463e,2) - 1./162.*m13*m66*RSp453e*
         RSp463e - 1./108.*m13*m66*pow(RSp453e,2) + 1./162.*m13*m66*
         RSp461e*RSp463e + 1./162.*m13*m66*RSp461e*RSp453e - 1./324.*
         m13*m66*pow(RSp461e,2) + 1./162.*m13*m66*RSp451e*RSp463e + 1./
         54.*m13*m66*RSp451e*RSp453e - 1./162.*m13*m66*RSp451e*RSp461e
          - 1./108.*m13*m66*pow(RSp451e,2) + 1./162.*m13*m56*pow(
         RSp463e,2) - 1./81.*m13*m56*RSp453e*RSp463e + 1./162.*m13*m56*
         pow(RSp453e,2) - 1./81.*m13*m56*RSp461e*RSp463e + 1./81.*m13*
         m56*RSp461e*RSp453e + 1./162.*m13*m56*pow(RSp461e,2) + 1./81.*
         m13*m56*RSp451e*RSp463e );

      M13p6x23RV +=  + AbV(m56,L)*BbV(mp2,L) * (  - 1./81.*m13*m56*
         RSp451e*RSp453e - 1./81.*m13*m56*RSp451e*RSp461e + 1./162.*m13
         *m56*pow(RSp451e,2) - 1./108.*m13*m55*pow(RSp463e,2) - 1./162.
         *m13*m55*RSp453e*RSp463e - 1./324.*m13*m55*pow(RSp453e,2) + 1./
         54.*m13*m55*RSp461e*RSp463e + 1./162.*m13*m55*RSp461e*RSp453e
          - 1./108.*m13*m55*pow(RSp461e,2) + 1./162.*m13*m55*RSp451e*
         RSp463e + 1./162.*m13*m55*RSp451e*RSp453e - 1./162.*m13*m55*
         RSp451e*RSp461e - 1./324.*m13*m55*pow(RSp451e,2) - 1./324.*m11
         *mp2*pow(RSp463e,2) + 1./162.*m11*mp2*RSp453e*RSp463e - 1./324.
         *m11*mp2*pow(RSp453e,2) - 1./162.*m11*mp2*RSp461e*RSp463e + 1./
         162.*m11*mp2*RSp461e*RSp453e - 1./108.*m11*mp2*pow(RSp461e,2)
          + 1./162.*m11*mp2*RSp451e*RSp463e - 1./162.*m11*mp2*RSp451e*
         RSp453e + 1./54.*m11*mp2*RSp451e*RSp461e - 1./108.*m11*mp2*
         pow(RSp451e,2) + 1./648.*m11*m66*pow(RSp463e,2) + 1./324.*m11*
         m66*RSp453e*RSp463e + 1./216.*m11*m66*pow(RSp453e,2) + 1./324.
         *m11*m66*RSp461e*RSp463e );

      M13p6x23RV +=  + AbV(m56,L)*BbV(mp2,L) * ( 1./324.*m11*m66*
         RSp461e*RSp453e + 1./216.*m11*m66*pow(RSp461e,2) + 1./324.*m11
         *m66*RSp451e*RSp463e + 1./108.*m11*m66*RSp451e*RSp453e + 1./
         108.*m11*m66*RSp451e*RSp461e + 1./72.*m11*m66*pow(RSp451e,2)
          - 1./324.*m11*m56*pow(RSp463e,2) + 1./162.*m11*m56*RSp453e*
         RSp463e - 1./324.*m11*m56*pow(RSp453e,2) - 1./162.*m11*m56*
         RSp461e*RSp463e + 1./162.*m11*m56*RSp461e*RSp453e - 1./108.*
         m11*m56*pow(RSp461e,2) + 1./162.*m11*m56*RSp451e*RSp463e - 1./
         162.*m11*m56*RSp451e*RSp453e + 1./54.*m11*m56*RSp451e*RSp461e
          - 1./108.*m11*m56*pow(RSp451e,2) + 1./216.*m11*m55*pow(
         RSp463e,2) + 1./324.*m11*m55*RSp453e*RSp463e + 1./648.*m11*m55
         *pow(RSp453e,2) + 1./108.*m11*m55*RSp461e*RSp463e + 1./324.*
         m11*m55*RSp461e*RSp453e + 1./72.*m11*m55*pow(RSp461e,2) + 1./
         324.*m11*m55*RSp451e*RSp463e + 1./324.*m11*m55*RSp451e*RSp453e
          + 1./108.*m11*m55*RSp451e*RSp461e + 1./216.*m11*m55*pow(
         RSp451e,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * ( 1./81.*mp2*me2*
         RSp463e*RSe463p - 1./81.*mp2*me2*RSp463e*RSe453p - 1./81.*mp2*
         me2*RSp453e*RSe463p + 1./81.*mp2*me2*RSp453e*RSe453p - 1./81.*
         mp2*me2*RSe461p*RSp463e + 1./81.*mp2*me2*RSe461p*RSp453e + 1./
         81.*mp2*me2*RSe451p*RSp463e - 1./81.*mp2*me2*RSe451p*RSp453e
          - 1./81.*mp2*me2*RSp461e*RSe463p + 1./81.*mp2*me2*RSp461e*
         RSe453p + 1./81.*mp2*me2*RSp461e*RSe461p - 1./81.*mp2*me2*
         RSp461e*RSe451p + 1./81.*mp2*me2*RSp451e*RSe463p - 1./81.*mp2*
         me2*RSp451e*RSe453p - 1./81.*mp2*me2*RSp451e*RSe461p + 1./81.*
         mp2*me2*RSp451e*RSe451p - 1./162.*m66*me2*RSp463e*RSe463p - 1./
         162.*m66*me2*RSp463e*RSe453p - 1./162.*m66*me2*RSp453e*RSe463p
          - 1./54.*m66*me2*RSp453e*RSe453p + 1./162.*m66*me2*RSe461p*
         RSp463e + 1./162.*m66*me2*RSe461p*RSp453e + 1./162.*m66*me2*
         RSe451p*RSp463e + 1./54.*m66*me2*RSe451p*RSp453e + 1./162.*m66
         *me2*RSp461e*RSe463p + 1./162.*m66*me2*RSp461e*RSe453p - 1./
         162.*m66*me2*RSp461e*RSe461p );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * (  - 1./162.*m66*me2
         *RSp461e*RSe451p + 1./162.*m66*me2*RSp451e*RSe463p + 1./54.*
         m66*me2*RSp451e*RSe453p - 1./162.*m66*me2*RSp451e*RSe461p - 1./
         54.*m66*me2*RSp451e*RSe451p + 1./81.*m56*me2*RSp463e*RSe463p
          - 1./81.*m56*me2*RSp463e*RSe453p - 1./81.*m56*me2*RSp453e*
         RSe463p + 1./81.*m56*me2*RSp453e*RSe453p - 1./81.*m56*me2*
         RSe461p*RSp463e + 1./81.*m56*me2*RSe461p*RSp453e + 1./81.*m56*
         me2*RSe451p*RSp463e - 1./81.*m56*me2*RSe451p*RSp453e - 1./81.*
         m56*me2*RSp461e*RSe463p + 1./81.*m56*me2*RSp461e*RSe453p + 1./
         81.*m56*me2*RSp461e*RSe461p - 1./81.*m56*me2*RSp461e*RSe451p
          + 1./81.*m56*me2*RSp451e*RSe463p - 1./81.*m56*me2*RSp451e*
         RSe453p - 1./81.*m56*me2*RSp451e*RSe461p + 1./81.*m56*me2*
         RSp451e*RSe451p - 1./54.*m55*me2*RSp463e*RSe463p - 1./162.*m55
         *me2*RSp463e*RSe453p - 1./162.*m55*me2*RSp453e*RSe463p - 1./
         162.*m55*me2*RSp453e*RSe453p + 1./54.*m55*me2*RSe461p*RSp463e
          + 1./162.*m55*me2*RSe461p*RSp453e );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * ( 1./162.*m55*me2*
         RSe451p*RSp463e + 1./162.*m55*me2*RSe451p*RSp453e + 1./54.*m55
         *me2*RSp461e*RSe463p + 1./162.*m55*me2*RSp461e*RSe453p - 1./54.
         *m55*me2*RSp461e*RSe461p - 1./162.*m55*me2*RSp461e*RSe451p + 1.
         /162.*m55*me2*RSp451e*RSe463p + 1./162.*m55*me2*RSp451e*
         RSe453p - 1./162.*m55*me2*RSp451e*RSe461p - 1./162.*m55*me2*
         RSp451e*RSe451p - 1./54.*m33*me2*RSp463e*RSe463p + 1./54.*m33*
         me2*RSp463e*RSe453p + 1./54.*m33*me2*RSp453e*RSe463p - 1./54.*
         m33*me2*RSp453e*RSe453p - 1./162.*m33*me2*RSe461p*RSp463e + 1./
         162.*m33*me2*RSe461p*RSp453e + 1./162.*m33*me2*RSe451p*RSp463e
          - 1./162.*m33*me2*RSe451p*RSp453e - 1./162.*m33*me2*RSp461e*
         RSe463p + 1./162.*m33*me2*RSp461e*RSe453p - 1./162.*m33*me2*
         RSp461e*RSe461p + 1./162.*m33*me2*RSp461e*RSe451p + 1./162.*
         m33*me2*RSp451e*RSe463p - 1./162.*m33*me2*RSp451e*RSe453p + 1./
         162.*m33*me2*RSp451e*RSe461p - 1./162.*m33*me2*RSp451e*RSe451p
          + 1./108.*m33*m66*RSp463e*RSe463p );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * ( 1./108.*m33*m66*
         RSp463e*RSe453p + 1./108.*m33*m66*RSp453e*RSe463p + 1./36.*m33
         *m66*RSp453e*RSe453p + 1./324.*m33*m66*RSe461p*RSp463e + 1./
         324.*m33*m66*RSe461p*RSp453e + 1./324.*m33*m66*RSe451p*RSp463e
          + 1./108.*m33*m66*RSe451p*RSp453e + 1./324.*m33*m66*RSp461e*
         RSe463p + 1./324.*m33*m66*RSp461e*RSe453p + 1./324.*m33*m66*
         RSp461e*RSe461p + 1./324.*m33*m66*RSp461e*RSe451p + 1./324.*
         m33*m66*RSp451e*RSe463p + 1./108.*m33*m66*RSp451e*RSe453p + 1./
         324.*m33*m66*RSp451e*RSe461p + 1./108.*m33*m66*RSp451e*RSe451p
          - 1./54.*m33*m56*RSp463e*RSe463p + 1./54.*m33*m56*RSp463e*
         RSe453p + 1./54.*m33*m56*RSp453e*RSe463p - 1./54.*m33*m56*
         RSp453e*RSe453p - 1./162.*m33*m56*RSe461p*RSp463e + 1./162.*
         m33*m56*RSe461p*RSp453e + 1./162.*m33*m56*RSe451p*RSp463e - 1./
         162.*m33*m56*RSe451p*RSp453e - 1./162.*m33*m56*RSp461e*RSe463p
          + 1./162.*m33*m56*RSp461e*RSe453p - 1./162.*m33*m56*RSp461e*
         RSe461p );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * ( 1./162.*m33*m56*
         RSp461e*RSe451p + 1./162.*m33*m56*RSp451e*RSe463p - 1./162.*
         m33*m56*RSp451e*RSe453p + 1./162.*m33*m56*RSp451e*RSe461p - 1./
         162.*m33*m56*RSp451e*RSe451p + 1./36.*m33*m55*RSp463e*RSe463p
          + 1./108.*m33*m55*RSp463e*RSe453p + 1./108.*m33*m55*RSp453e*
         RSe463p + 1./108.*m33*m55*RSp453e*RSe453p + 1./108.*m33*m55*
         RSe461p*RSp463e + 1./324.*m33*m55*RSe461p*RSp453e + 1./324.*
         m33*m55*RSe451p*RSp463e + 1./324.*m33*m55*RSe451p*RSp453e + 1./
         108.*m33*m55*RSp461e*RSe463p + 1./324.*m33*m55*RSp461e*RSe453p
          + 1./108.*m33*m55*RSp461e*RSe461p + 1./324.*m33*m55*RSp461e*
         RSe451p + 1./324.*m33*m55*RSp451e*RSe463p + 1./324.*m33*m55*
         RSp451e*RSe453p + 1./324.*m33*m55*RSp451e*RSe461p + 1./324.*
         m33*m55*RSp451e*RSe451p + 1./81.*m13*me2*RSp463e*RSe463p - 1./
         81.*m13*me2*RSp463e*RSe453p - 1./81.*m13*me2*RSp453e*RSe463p
          + 1./81.*m13*me2*RSp453e*RSe453p - 1./81.*m13*me2*RSe461p*
         RSp463e );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * ( 1./81.*m13*me2*
         RSe461p*RSp453e + 1./81.*m13*me2*RSe451p*RSp463e - 1./81.*m13*
         me2*RSe451p*RSp453e - 1./81.*m13*me2*RSp461e*RSe463p + 1./81.*
         m13*me2*RSp461e*RSe453p + 1./81.*m13*me2*RSp461e*RSe461p - 1./
         81.*m13*me2*RSp461e*RSe451p + 1./81.*m13*me2*RSp451e*RSe463p
          - 1./81.*m13*me2*RSp451e*RSe453p - 1./81.*m13*me2*RSp451e*
         RSe461p + 1./81.*m13*me2*RSp451e*RSe451p - 1./162.*m13*m66*
         RSp463e*RSe463p - 1./162.*m13*m66*RSp463e*RSe453p - 1./162.*
         m13*m66*RSp453e*RSe463p - 1./54.*m13*m66*RSp453e*RSe453p + 1./
         162.*m13*m66*RSe461p*RSp463e + 1./162.*m13*m66*RSe461p*RSp453e
          + 1./162.*m13*m66*RSe451p*RSp463e + 1./54.*m13*m66*RSe451p*
         RSp453e + 1./162.*m13*m66*RSp461e*RSe463p + 1./162.*m13*m66*
         RSp461e*RSe453p - 1./162.*m13*m66*RSp461e*RSe461p - 1./162.*
         m13*m66*RSp461e*RSe451p + 1./162.*m13*m66*RSp451e*RSe463p + 1./
         54.*m13*m66*RSp451e*RSe453p - 1./162.*m13*m66*RSp451e*RSe461p
          - 1./54.*m13*m66*RSp451e*RSe451p );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * ( 1./81.*m13*m56*
         RSp463e*RSe463p - 1./81.*m13*m56*RSp463e*RSe453p - 1./81.*m13*
         m56*RSp453e*RSe463p + 1./81.*m13*m56*RSp453e*RSe453p - 1./81.*
         m13*m56*RSe461p*RSp463e + 1./81.*m13*m56*RSe461p*RSp453e + 1./
         81.*m13*m56*RSe451p*RSp463e - 1./81.*m13*m56*RSe451p*RSp453e
          - 1./81.*m13*m56*RSp461e*RSe463p + 1./81.*m13*m56*RSp461e*
         RSe453p + 1./81.*m13*m56*RSp461e*RSe461p - 1./81.*m13*m56*
         RSp461e*RSe451p + 1./81.*m13*m56*RSp451e*RSe463p - 1./81.*m13*
         m56*RSp451e*RSe453p - 1./81.*m13*m56*RSp451e*RSe461p + 1./81.*
         m13*m56*RSp451e*RSe451p - 1./54.*m13*m55*RSp463e*RSe463p - 1./
         162.*m13*m55*RSp463e*RSe453p - 1./162.*m13*m55*RSp453e*RSe463p
          - 1./162.*m13*m55*RSp453e*RSe453p + 1./54.*m13*m55*RSe461p*
         RSp463e + 1./162.*m13*m55*RSe461p*RSp453e + 1./162.*m13*m55*
         RSe451p*RSp463e + 1./162.*m13*m55*RSe451p*RSp453e + 1./54.*m13
         *m55*RSp461e*RSe463p + 1./162.*m13*m55*RSp461e*RSe453p - 1./54.
         *m13*m55*RSp461e*RSe461p );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * (  - 1./162.*m13*m55
         *RSp461e*RSe451p + 1./162.*m13*m55*RSp451e*RSe463p + 1./162.*
         m13*m55*RSp451e*RSe453p - 1./162.*m13*m55*RSp451e*RSe461p - 1./
         162.*m13*m55*RSp451e*RSe451p - 1./162.*m11*me2*RSp463e*RSe463p
          + 1./162.*m11*me2*RSp463e*RSe453p + 1./162.*m11*me2*RSp453e*
         RSe463p - 1./162.*m11*me2*RSp453e*RSe453p - 1./162.*m11*me2*
         RSe461p*RSp463e + 1./162.*m11*me2*RSe461p*RSp453e + 1./162.*
         m11*me2*RSe451p*RSp463e - 1./162.*m11*me2*RSe451p*RSp453e - 1./
         162.*m11*me2*RSp461e*RSe463p + 1./162.*m11*me2*RSp461e*RSe453p
          - 1./54.*m11*me2*RSp461e*RSe461p + 1./54.*m11*me2*RSp461e*
         RSe451p + 1./162.*m11*me2*RSp451e*RSe463p - 1./162.*m11*me2*
         RSp451e*RSe453p + 1./54.*m11*me2*RSp451e*RSe461p - 1./54.*m11*
         me2*RSp451e*RSe451p + 1./324.*m11*m66*RSp463e*RSe463p + 1./324.
         *m11*m66*RSp463e*RSe453p + 1./324.*m11*m66*RSp453e*RSe463p + 1.
         /108.*m11*m66*RSp453e*RSe453p + 1./324.*m11*m66*RSe461p*
         RSp463e );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * ( 1./324.*m11*m66*
         RSe461p*RSp453e + 1./324.*m11*m66*RSe451p*RSp463e + 1./108.*
         m11*m66*RSe451p*RSp453e + 1./324.*m11*m66*RSp461e*RSe463p + 1./
         324.*m11*m66*RSp461e*RSe453p + 1./108.*m11*m66*RSp461e*RSe461p
          + 1./108.*m11*m66*RSp461e*RSe451p + 1./324.*m11*m66*RSp451e*
         RSe463p + 1./108.*m11*m66*RSp451e*RSe453p + 1./108.*m11*m66*
         RSp451e*RSe461p + 1./36.*m11*m66*RSp451e*RSe451p - 1./162.*m11
         *m56*RSp463e*RSe463p + 1./162.*m11*m56*RSp463e*RSe453p + 1./
         162.*m11*m56*RSp453e*RSe463p - 1./162.*m11*m56*RSp453e*RSe453p
          - 1./162.*m11*m56*RSe461p*RSp463e + 1./162.*m11*m56*RSe461p*
         RSp453e + 1./162.*m11*m56*RSe451p*RSp463e - 1./162.*m11*m56*
         RSe451p*RSp453e - 1./162.*m11*m56*RSp461e*RSe463p + 1./162.*
         m11*m56*RSp461e*RSe453p - 1./54.*m11*m56*RSp461e*RSe461p + 1./
         54.*m11*m56*RSp461e*RSe451p + 1./162.*m11*m56*RSp451e*RSe463p
          - 1./162.*m11*m56*RSp451e*RSe453p + 1./54.*m11*m56*RSp451e*
         RSe461p );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * (  - 1./54.*m11*m56*
         RSp451e*RSe451p + 1./108.*m11*m55*RSp463e*RSe463p + 1./324.*
         m11*m55*RSp463e*RSe453p + 1./324.*m11*m55*RSp453e*RSe463p + 1./
         324.*m11*m55*RSp453e*RSe453p + 1./108.*m11*m55*RSe461p*RSp463e
          + 1./324.*m11*m55*RSe461p*RSp453e + 1./324.*m11*m55*RSe451p*
         RSp463e + 1./324.*m11*m55*RSe451p*RSp453e + 1./108.*m11*m55*
         RSp461e*RSe463p + 1./324.*m11*m55*RSp461e*RSe453p + 1./36.*m11
         *m55*RSp461e*RSe461p + 1./108.*m11*m55*RSp461e*RSe451p + 1./
         324.*m11*m55*RSp451e*RSe463p + 1./324.*m11*m55*RSp451e*RSe453p
          + 1./108.*m11*m55*RSp451e*RSe461p + 1./108.*m11*m55*RSp451e*
         RSe451p );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,L) * ( 1./162.*pow(me2,2)*
         pow(RSe463p,2) - 1./81.*pow(me2,2)*RSe453p*RSe463p + 1./162.*
         pow(me2,2)*pow(RSe453p,2) - 1./81.*pow(me2,2)*RSe461p*RSe463p
          + 1./81.*pow(me2,2)*RSe461p*RSe453p + 1./162.*pow(me2,2)*pow(
         RSe461p,2) + 1./81.*pow(me2,2)*RSe451p*RSe463p - 1./81.*pow(
         me2,2)*RSe451p*RSe453p - 1./81.*pow(me2,2)*RSe451p*RSe461p + 1.
         /162.*pow(me2,2)*pow(RSe451p,2) - 1./324.*m66*me2*pow(
         RSe463p,2) - 1./162.*m66*me2*RSe453p*RSe463p - 1./108.*m66*me2
         *pow(RSe453p,2) + 1./162.*m66*me2*RSe461p*RSe463p + 1./162.*
         m66*me2*RSe461p*RSe453p - 1./324.*m66*me2*pow(RSe461p,2) + 1./
         162.*m66*me2*RSe451p*RSe463p + 1./54.*m66*me2*RSe451p*RSe453p
          - 1./162.*m66*me2*RSe451p*RSe461p - 1./108.*m66*me2*pow(
         RSe451p,2) + 1./162.*m56*me2*pow(RSe463p,2) - 1./81.*m56*me2*
         RSe453p*RSe463p + 1./162.*m56*me2*pow(RSe453p,2) - 1./81.*m56*
         me2*RSe461p*RSe463p + 1./81.*m56*me2*RSe461p*RSe453p + 1./162.
         *m56*me2*pow(RSe461p,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,L) * ( 1./81.*m56*me2*RSe451p
         *RSe463p - 1./81.*m56*me2*RSe451p*RSe453p - 1./81.*m56*me2*
         RSe451p*RSe461p + 1./162.*m56*me2*pow(RSe451p,2) - 1./108.*m55
         *me2*pow(RSe463p,2) - 1./162.*m55*me2*RSe453p*RSe463p - 1./324.
         *m55*me2*pow(RSe453p,2) + 1./54.*m55*me2*RSe461p*RSe463p + 1./
         162.*m55*me2*RSe461p*RSe453p - 1./108.*m55*me2*pow(RSe461p,2)
          + 1./162.*m55*me2*RSe451p*RSe463p + 1./162.*m55*me2*RSe451p*
         RSe453p - 1./162.*m55*me2*RSe451p*RSe461p - 1./324.*m55*me2*
         pow(RSe451p,2) - 1./108.*m33*me2*pow(RSe463p,2) + 1./54.*m33*
         me2*RSe453p*RSe463p - 1./108.*m33*me2*pow(RSe453p,2) - 1./162.
         *m33*me2*RSe461p*RSe463p + 1./162.*m33*me2*RSe461p*RSe453p - 1.
         /324.*m33*me2*pow(RSe461p,2) + 1./162.*m33*me2*RSe451p*RSe463p
          - 1./162.*m33*me2*RSe451p*RSe453p + 1./162.*m33*me2*RSe451p*
         RSe461p - 1./324.*m33*me2*pow(RSe451p,2) + 1./216.*m33*m66*
         pow(RSe463p,2) + 1./108.*m33*m66*RSe453p*RSe463p + 1./72.*m33*
         m66*pow(RSe453p,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,L) * ( 1./324.*m33*m66*
         RSe461p*RSe463p + 1./324.*m33*m66*RSe461p*RSe453p + 1./648.*
         m33*m66*pow(RSe461p,2) + 1./324.*m33*m66*RSe451p*RSe463p + 1./
         108.*m33*m66*RSe451p*RSe453p + 1./324.*m33*m66*RSe451p*RSe461p
          + 1./216.*m33*m66*pow(RSe451p,2) - 1./108.*m33*m56*pow(
         RSe463p,2) + 1./54.*m33*m56*RSe453p*RSe463p - 1./108.*m33*m56*
         pow(RSe453p,2) - 1./162.*m33*m56*RSe461p*RSe463p + 1./162.*m33
         *m56*RSe461p*RSe453p - 1./324.*m33*m56*pow(RSe461p,2) + 1./162.
         *m33*m56*RSe451p*RSe463p - 1./162.*m33*m56*RSe451p*RSe453p + 1.
         /162.*m33*m56*RSe451p*RSe461p - 1./324.*m33*m56*pow(RSe451p,2)
          + 1./72.*m33*m55*pow(RSe463p,2) + 1./108.*m33*m55*RSe453p*
         RSe463p + 1./216.*m33*m55*pow(RSe453p,2) + 1./108.*m33*m55*
         RSe461p*RSe463p + 1./324.*m33*m55*RSe461p*RSe453p + 1./216.*
         m33*m55*pow(RSe461p,2) + 1./324.*m33*m55*RSe451p*RSe463p + 1./
         324.*m33*m55*RSe451p*RSe453p + 1./324.*m33*m55*RSe451p*RSe461p
          + 1./648.*m33*m55*pow(RSe451p,2) );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,L) * ( 1./162.*m13*me2*pow(
         RSe463p,2) - 1./81.*m13*me2*RSe453p*RSe463p + 1./162.*m13*me2*
         pow(RSe453p,2) - 1./81.*m13*me2*RSe461p*RSe463p + 1./81.*m13*
         me2*RSe461p*RSe453p + 1./162.*m13*me2*pow(RSe461p,2) + 1./81.*
         m13*me2*RSe451p*RSe463p - 1./81.*m13*me2*RSe451p*RSe453p - 1./
         81.*m13*me2*RSe451p*RSe461p + 1./162.*m13*me2*pow(RSe451p,2)
          - 1./324.*m13*m66*pow(RSe463p,2) - 1./162.*m13*m66*RSe453p*
         RSe463p - 1./108.*m13*m66*pow(RSe453p,2) + 1./162.*m13*m66*
         RSe461p*RSe463p + 1./162.*m13*m66*RSe461p*RSe453p - 1./324.*
         m13*m66*pow(RSe461p,2) + 1./162.*m13*m66*RSe451p*RSe463p + 1./
         54.*m13*m66*RSe451p*RSe453p - 1./162.*m13*m66*RSe451p*RSe461p
          - 1./108.*m13*m66*pow(RSe451p,2) + 1./162.*m13*m56*pow(
         RSe463p,2) - 1./81.*m13*m56*RSe453p*RSe463p + 1./162.*m13*m56*
         pow(RSe453p,2) - 1./81.*m13*m56*RSe461p*RSe463p + 1./81.*m13*
         m56*RSe461p*RSe453p + 1./162.*m13*m56*pow(RSe461p,2) + 1./81.*
         m13*m56*RSe451p*RSe463p );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,L) * (  - 1./81.*m13*m56*
         RSe451p*RSe453p - 1./81.*m13*m56*RSe451p*RSe461p + 1./162.*m13
         *m56*pow(RSe451p,2) - 1./108.*m13*m55*pow(RSe463p,2) - 1./162.
         *m13*m55*RSe453p*RSe463p - 1./324.*m13*m55*pow(RSe453p,2) + 1./
         54.*m13*m55*RSe461p*RSe463p + 1./162.*m13*m55*RSe461p*RSe453p
          - 1./108.*m13*m55*pow(RSe461p,2) + 1./162.*m13*m55*RSe451p*
         RSe463p + 1./162.*m13*m55*RSe451p*RSe453p - 1./162.*m13*m55*
         RSe451p*RSe461p - 1./324.*m13*m55*pow(RSe451p,2) - 1./324.*m11
         *me2*pow(RSe463p,2) + 1./162.*m11*me2*RSe453p*RSe463p - 1./324.
         *m11*me2*pow(RSe453p,2) - 1./162.*m11*me2*RSe461p*RSe463p + 1./
         162.*m11*me2*RSe461p*RSe453p - 1./108.*m11*me2*pow(RSe461p,2)
          + 1./162.*m11*me2*RSe451p*RSe463p - 1./162.*m11*me2*RSe451p*
         RSe453p + 1./54.*m11*me2*RSe451p*RSe461p - 1./108.*m11*me2*
         pow(RSe451p,2) + 1./648.*m11*m66*pow(RSe463p,2) + 1./324.*m11*
         m66*RSe453p*RSe463p + 1./216.*m11*m66*pow(RSe453p,2) + 1./324.
         *m11*m66*RSe461p*RSe463p );

      M13p6x23RV +=  + AbV(m56,L)*BbV(me2,L) * ( 1./324.*m11*m66*
         RSe461p*RSe453p + 1./216.*m11*m66*pow(RSe461p,2) + 1./324.*m11
         *m66*RSe451p*RSe463p + 1./108.*m11*m66*RSe451p*RSe453p + 1./
         108.*m11*m66*RSe451p*RSe461p + 1./72.*m11*m66*pow(RSe451p,2)
          - 1./324.*m11*m56*pow(RSe463p,2) + 1./162.*m11*m56*RSe453p*
         RSe463p - 1./324.*m11*m56*pow(RSe453p,2) - 1./162.*m11*m56*
         RSe461p*RSe463p + 1./162.*m11*m56*RSe461p*RSe453p - 1./108.*
         m11*m56*pow(RSe461p,2) + 1./162.*m11*m56*RSe451p*RSe463p - 1./
         162.*m11*m56*RSe451p*RSe453p + 1./54.*m11*m56*RSe451p*RSe461p
          - 1./108.*m11*m56*pow(RSe451p,2) + 1./216.*m11*m55*pow(
         RSe463p,2) + 1./324.*m11*m55*RSe453p*RSe463p + 1./648.*m11*m55
         *pow(RSe453p,2) + 1./108.*m11*m55*RSe461p*RSe463p + 1./324.*
         m11*m55*RSe461p*RSe453p + 1./72.*m11*m55*pow(RSe461p,2) + 1./
         324.*m11*m55*RSe451p*RSe463p + 1./324.*m11*m55*RSe451p*RSe453p
          + 1./108.*m11*m55*RSe451p*RSe461p + 1./216.*m11*m55*pow(
         RSe451p,2) );

      M13p6x23RV +=  + AbV(m66,L)*AbV(mp2,L) * (  - 1./27.*m66*RSp453e*
         RS345pe - 1./27.*m66*RSp453e*RSe453p - 1./54.*m66*pow(
         RSp453e,2) + 1./27.*m66*RS145pe*RSp453e + 1./81.*m66*RS645pe*
         RSp453e*RS345pe + 1./81.*m66*RS645pe*RSp453e*RSe453p + 1./162.
         *m66*RS645pe*pow(RSp453e,2) - 1./81.*m66*RS645pe*RS145pe*
         RSp453e + 1./27.*m66*RSe451p*RSp453e - 1./81.*m66*RSe451p*
         RS645pe*RSp453e + 1./27.*m66*RSp451e*RS345pe + 1./27.*m66*
         RSp451e*RSe453p + 1./27.*m66*RSp451e*RSp453e - 1./27.*m66*
         RSp451e*RS145pe - 1./81.*m66*RSp451e*RS645pe*RS345pe - 1./81.*
         m66*RSp451e*RS645pe*RSe453p - 1./81.*m66*RSp451e*RS645pe*
         RSp453e + 1./81.*m66*RSp451e*RS645pe*RS145pe - 1./27.*m66*
         RSp451e*RSe451p + 1./81.*m66*RSp451e*RSe451p*RS645pe - 1./54.*
         m66*pow(RSp451e,2) + 1./162.*m66*pow(RSp451e,2)*RS645pe );

      M13p6x23RV +=  + AbV(m66,L)*AbV(me2,L) * (  - 1./27.*m66*RSe453p*
         RS345pe - 1./54.*m66*pow(RSe453p,2) + 1./27.*m66*RS145pe*
         RSe453p + 1./81.*m66*RS645pe*RSe453p*RS345pe + 1./162.*m66*
         RS645pe*pow(RSe453p,2) - 1./81.*m66*RS645pe*RS145pe*RSe453p + 
         1./27.*m66*RSe451p*RS345pe + 1./27.*m66*RSe451p*RSe453p - 1./
         27.*m66*RSe451p*RS145pe - 1./81.*m66*RSe451p*RS645pe*RS345pe
          - 1./81.*m66*RSe451p*RS645pe*RSe453p + 1./81.*m66*RSe451p*
         RS645pe*RS145pe - 1./54.*m66*pow(RSe451p,2) + 1./162.*m66*pow(
         RSe451p,2)*RS645pe );

      M13p6x23RV +=  + AbV(m66,L)*BbV(m11,m33,L) * ( 1./54.*m33*m66*
         RS145pe*RS345pe - 1./162.*m33*m66*RS645pe*RS145pe*RS345pe + 1./
         27.*m13*m66*RS145pe*RS345pe - 1./81.*m13*m66*RS645pe*RS145pe*
         RS345pe + 1./18.*m11*m66*RS145pe*RS345pe - 1./54.*m11*m66*
         RS645pe*RS145pe*RS345pe );

      M13p6x23RV +=  + AbV(m66,L)*BbV(m11,mp2,L) * ( 1./54.*m33*m66*
         RS145pe*RSp453e - 1./162.*m33*m66*RS645pe*RS145pe*RSp453e + 1./
         54.*m33*m66*RSp451e*RS145pe - 1./162.*m33*m66*RSp451e*RS645pe*
         RS145pe + 1./27.*m13*m66*RS145pe*RSp453e - 1./81.*m13*m66*
         RS645pe*RS145pe*RSp453e - 1./27.*m13*m66*RSp451e*RS145pe + 1./
         81.*m13*m66*RSp451e*RS645pe*RS145pe + 1./18.*m11*m66*RS145pe*
         RSp453e - 1./54.*m11*m66*RS645pe*RS145pe*RSp453e + 1./54.*m11*
         m66*RSp451e*RS145pe - 1./162.*m11*m66*RSp451e*RS645pe*RS145pe
          );

      M13p6x23RV +=  + AbV(m66,L)*BbV(m11,me2,L) * ( 1./54.*m33*m66*
         RS145pe*RSe453p - 1./162.*m33*m66*RS645pe*RS145pe*RSe453p + 1./
         54.*m33*m66*RSe451p*RS145pe - 1./162.*m33*m66*RSe451p*RS645pe*
         RS145pe + 1./27.*m13*m66*RS145pe*RSe453p - 1./81.*m13*m66*
         RS645pe*RS145pe*RSe453p - 1./27.*m13*m66*RSe451p*RS145pe + 1./
         81.*m13*m66*RSe451p*RS645pe*RS145pe + 1./18.*m11*m66*RS145pe*
         RSe453p - 1./54.*m11*m66*RS645pe*RS145pe*RSe453p + 1./54.*m11*
         m66*RSe451p*RS145pe - 1./162.*m11*m66*RSe451p*RS645pe*RS145pe
          );

      M13p6x23RV +=  + AbV(m66,L)*BbV(m11,L) * ( 1./108.*m33*m66*pow(
         RS145pe,2) - 1./324.*m33*m66*RS645pe*pow(RS145pe,2) - 1./54.*
         m13*m66*pow(RS145pe,2) + 1./162.*m13*m66*RS645pe*pow(
         RS145pe,2) + 1./108.*m11*m66*pow(RS145pe,2) - 1./324.*m11*m66*
         RS645pe*pow(RS145pe,2) );

      M13p6x23RV +=  + AbV(m66,L)*BbV(m16,L) * ( 1./144.*pow(m66,2) - 1.
         /432.*pow(m66,2)*RS645pe + 1./144.*m33*m66 - 1./432.*m33*m66*
         RS645pe - 1./36.*m16*m66 + 1./108.*m16*m66*RS645pe - 1./72.*
         m16*m33 + 1./216.*m16*m33*RS645pe + 1./36.*pow(m16,2) - 1./108.
         *pow(m16,2)*RS645pe - 1./72.*m13*m66 + 1./216.*m13*m66*RS645pe
          + 1./36.*m13*m16 - 1./108.*m13*m16*RS645pe + 1./48.*m11*m66
          - 1./144.*m11*m66*RS645pe + 1./144.*m11*m33 - 1./432.*m11*m33
         *RS645pe - 1./24.*m11*m16 + 1./72.*m11*m16*RS645pe - 1./72.*
         m11*m13 + 1./216.*m11*m13*RS645pe + 1./72.*pow(m11,2) - 1./216.
         *pow(m11,2)*RS645pe );

      M13p6x23RV +=  + AbV(m66,L)*BbV(m33,mp2,L) * ( 1./54.*m33*m66*
         RSp453e*RS345pe - 1./162.*m33*m66*RS645pe*RSp453e*RS345pe + 1./
         18.*m33*m66*RSp451e*RS345pe - 1./54.*m33*m66*RSp451e*RS645pe*
         RS345pe - 1./27.*m13*m66*RSp453e*RS345pe + 1./81.*m13*m66*
         RS645pe*RSp453e*RS345pe + 1./27.*m13*m66*RSp451e*RS345pe - 1./
         81.*m13*m66*RSp451e*RS645pe*RS345pe + 1./54.*m11*m66*RSp453e*
         RS345pe - 1./162.*m11*m66*RS645pe*RSp453e*RS345pe + 1./54.*m11
         *m66*RSp451e*RS345pe - 1./162.*m11*m66*RSp451e*RS645pe*RS345pe
          );

      M13p6x23RV +=  + AbV(m66,L)*BbV(m33,me2,L) * ( 1./54.*m33*m66*
         RSe453p*RS345pe - 1./162.*m33*m66*RS645pe*RSe453p*RS345pe + 1./
         18.*m33*m66*RSe451p*RS345pe - 1./54.*m33*m66*RSe451p*RS645pe*
         RS345pe - 1./27.*m13*m66*RSe453p*RS345pe + 1./81.*m13*m66*
         RS645pe*RSe453p*RS345pe + 1./27.*m13*m66*RSe451p*RS345pe - 1./
         81.*m13*m66*RSe451p*RS645pe*RS345pe + 1./54.*m11*m66*RSe453p*
         RS345pe - 1./162.*m11*m66*RS645pe*RSe453p*RS345pe + 1./54.*m11
         *m66*RSe451p*RS345pe - 1./162.*m11*m66*RSe451p*RS645pe*RS345pe
          );

      M13p6x23RV +=  + AbV(m66,L)*BbV(m33,L) * ( 1./108.*m33*m66*pow(
         RS345pe,2) - 1./324.*m33*m66*RS645pe*pow(RS345pe,2) - 1./54.*
         m13*m66*pow(RS345pe,2) + 1./162.*m13*m66*RS645pe*pow(
         RS345pe,2) + 1./108.*m11*m66*pow(RS345pe,2) - 1./324.*m11*m66*
         RS645pe*pow(RS345pe,2) );

      M13p6x23RV +=  + AbV(m66,L)*BbV(m36,L) * ( 1./144.*pow(m66,2) - 1.
         /432.*pow(m66,2)*RS645pe - 1./36.*m36*m66 + 1./108.*m36*m66*
         RS645pe + 1./36.*pow(m36,2) - 1./108.*pow(m36,2)*RS645pe + 1./
         48.*m33*m66 - 1./144.*m33*m66*RS645pe - 1./24.*m33*m36 + 1./72.
         *m33*m36*RS645pe + 1./72.*pow(m33,2) - 1./216.*pow(m33,2)*
         RS645pe - 1./72.*m13*m66 + 1./216.*m13*m66*RS645pe + 1./36.*
         m13*m36 - 1./108.*m13*m36*RS645pe - 1./72.*m13*m33 + 1./216.*
         m13*m33*RS645pe + 1./144.*m11*m66 - 1./432.*m11*m66*RS645pe - 
         1./72.*m11*m36 + 1./216.*m11*m36*RS645pe + 1./144.*m11*m33 - 1.
         /432.*m11*m33*RS645pe );

      M13p6x23RV +=  + AbV(m66,L)*BbV(mp2,L) * (  - 1./54.*m66*mp2*pow(
         RSp453e,2) + 1./162.*m66*mp2*RS645pe*pow(RSp453e,2) + 1./27.*
         m66*mp2*RSp451e*RSp453e - 1./81.*m66*mp2*RSp451e*RS645pe*
         RSp453e - 1./54.*m66*mp2*pow(RSp451e,2) + 1./162.*m66*mp2*pow(
         RSp451e,2)*RS645pe + 1./36.*m33*m66*pow(RSp453e,2) - 1./108.*
         m33*m66*RS645pe*pow(RSp453e,2) + 1./54.*m33*m66*RSp451e*
         RSp453e - 1./162.*m33*m66*RSp451e*RS645pe*RSp453e + 1./108.*
         m33*m66*pow(RSp451e,2) - 1./324.*m33*m66*pow(RSp451e,2)*
         RS645pe - 1./54.*m13*m66*pow(RSp453e,2) + 1./162.*m13*m66*
         RS645pe*pow(RSp453e,2) + 1./27.*m13*m66*RSp451e*RSp453e - 1./
         81.*m13*m66*RSp451e*RS645pe*RSp453e - 1./54.*m13*m66*pow(
         RSp451e,2) + 1./162.*m13*m66*pow(RSp451e,2)*RS645pe + 1./108.*
         m11*m66*pow(RSp453e,2) - 1./324.*m11*m66*RS645pe*pow(
         RSp453e,2) + 1./54.*m11*m66*RSp451e*RSp453e - 1./162.*m11*m66*
         RSp451e*RS645pe*RSp453e + 1./36.*m11*m66*pow(RSp451e,2) - 1./
         108.*m11*m66*pow(RSp451e,2)*RS645pe );

      M13p6x23RV +=  + AbV(m66,L)*BbV(me2,mp2,L) * (  - 1./27.*m66*me2*
         RSp453e*RSe453p + 1./81.*m66*me2*RS645pe*RSp453e*RSe453p + 1./
         27.*m66*me2*RSe451p*RSp453e - 1./81.*m66*me2*RSe451p*RS645pe*
         RSp453e + 1./27.*m66*me2*RSp451e*RSe453p - 1./81.*m66*me2*
         RSp451e*RS645pe*RSe453p - 1./27.*m66*me2*RSp451e*RSe451p + 1./
         81.*m66*me2*RSp451e*RSe451p*RS645pe + 1./18.*m33*m66*RSp453e*
         RSe453p - 1./54.*m33*m66*RS645pe*RSp453e*RSe453p + 1./54.*m33*
         m66*RSe451p*RSp453e - 1./162.*m33*m66*RSe451p*RS645pe*RSp453e
          + 1./54.*m33*m66*RSp451e*RSe453p - 1./162.*m33*m66*RSp451e*
         RS645pe*RSe453p + 1./54.*m33*m66*RSp451e*RSe451p - 1./162.*m33
         *m66*RSp451e*RSe451p*RS645pe - 1./27.*m13*m66*RSp453e*RSe453p
          + 1./81.*m13*m66*RS645pe*RSp453e*RSe453p + 1./27.*m13*m66*
         RSe451p*RSp453e - 1./81.*m13*m66*RSe451p*RS645pe*RSp453e + 1./
         27.*m13*m66*RSp451e*RSe453p - 1./81.*m13*m66*RSp451e*RS645pe*
         RSe453p - 1./27.*m13*m66*RSp451e*RSe451p + 1./81.*m13*m66*
         RSp451e*RSe451p*RS645pe );

      M13p6x23RV +=  + AbV(m66,L)*BbV(me2,mp2,L) * ( 1./54.*m11*m66*
         RSp453e*RSe453p - 1./162.*m11*m66*RS645pe*RSp453e*RSe453p + 1./
         54.*m11*m66*RSe451p*RSp453e - 1./162.*m11*m66*RSe451p*RS645pe*
         RSp453e + 1./54.*m11*m66*RSp451e*RSe453p - 1./162.*m11*m66*
         RSp451e*RS645pe*RSe453p + 1./18.*m11*m66*RSp451e*RSe451p - 1./
         54.*m11*m66*RSp451e*RSe451p*RS645pe );

      M13p6x23RV +=  + AbV(m66,L)*BbV(me2,L) * (  - 1./54.*m66*me2*pow(
         RSe453p,2) + 1./162.*m66*me2*RS645pe*pow(RSe453p,2) + 1./27.*
         m66*me2*RSe451p*RSe453p - 1./81.*m66*me2*RSe451p*RS645pe*
         RSe453p - 1./54.*m66*me2*pow(RSe451p,2) + 1./162.*m66*me2*pow(
         RSe451p,2)*RS645pe + 1./36.*m33*m66*pow(RSe453p,2) - 1./108.*
         m33*m66*RS645pe*pow(RSe453p,2) + 1./54.*m33*m66*RSe451p*
         RSe453p - 1./162.*m33*m66*RSe451p*RS645pe*RSe453p + 1./108.*
         m33*m66*pow(RSe451p,2) - 1./324.*m33*m66*pow(RSe451p,2)*
         RS645pe - 1./54.*m13*m66*pow(RSe453p,2) + 1./162.*m13*m66*
         RS645pe*pow(RSe453p,2) + 1./27.*m13*m66*RSe451p*RSe453p - 1./
         81.*m13*m66*RSe451p*RS645pe*RSe453p - 1./54.*m13*m66*pow(
         RSe451p,2) + 1./162.*m13*m66*pow(RSe451p,2)*RS645pe + 1./108.*
         m11*m66*pow(RSe453p,2) - 1./324.*m11*m66*RS645pe*pow(
         RSe453p,2) + 1./54.*m11*m66*RSe451p*RSe453p - 1./162.*m11*m66*
         RSe451p*RS645pe*RSe453p + 1./36.*m11*m66*pow(RSe451p,2) - 1./
         108.*m11*m66*pow(RSe451p,2)*RS645pe );

      M13p6x23RV +=  + AbV(mp2,L)*pi16 * (  - 1./36.*m13*RD3456ep*
         RSp45613e + 1./72.*m13*RSp45633e*RD3456ep - 1./36.*m13*
         RD1456ep*RSp45613e + 1./72.*m13*RD1456ep*RSp45633e + 1./72.*
         m13*RSp45611e*RD3456ep + 1./72.*m13*RSp45611e*RD1456ep + 1./18.
         *m13*me2*RSp45613e*RSe45613p - 1./36.*m13*me2*RSe45633p*
         RSp45613e - 1./36.*m13*me2*RSp45633e*RSe45613p + 1./72.*m13*
         me2*RSp45633e*RSe45633p - 1./36.*m13*me2*RSe45611p*RSp45613e
          + 1./72.*m13*me2*RSe45611p*RSp45633e - 1./36.*m13*me2*
         RSp45611e*RSe45613p + 1./72.*m13*me2*RSp45611e*RSe45633p + 1./
         72.*m13*me2*RSp45611e*RSe45611p - 1./12.*m13*mp2*RSp45613e + 1.
         /18.*m13*mp2*pow(RSp45613e,2) + 1./24.*m13*mp2*RSp45633e - 1./
         18.*m13*mp2*RSp45633e*RSp45613e + 1./72.*m13*mp2*pow(
         RSp45633e,2) + 1./24.*m13*mp2*RSp563e + 1./24.*m13*mp2*RSp463e
          + 1./24.*m13*mp2*RSp453e + 1./24.*m13*mp2*RSp45611e - 1./18.*
         m13*mp2*RSp45611e*RSp45613e + 1./36.*m13*mp2*RSp45611e*
         RSp45633e );

      M13p6x23RV +=  + AbV(mp2,L)*pi16 * ( 1./72.*m13*mp2*pow(
         RSp45611e,2) + 1./24.*m13*mp2*RSp561e + 1./24.*m13*mp2*RSp461e
          + 1./24.*m13*mp2*RSp451e + 1./12.*m13*m36*RSp45613e - 1./24.*
         m13*m36*RSp45633e - 1./24.*m13*m36*RSp45611e + 1./12.*m13*m35*
         RSp45613e - 1./24.*m13*m35*RSp45633e - 1./24.*m13*m35*
         RSp45611e + 1./12.*m13*m34*RSp45613e - 1./24.*m13*m34*
         RSp45633e - 1./24.*m13*m34*RSp45611e + 1./18.*m13*m33*
         RSp45613e*RS34561pe - 1./36.*m13*m33*RC3456pe*RSp45613e - 1./
         36.*m13*m33*RSp45633e*RS34561pe + 1./72.*m13*m33*RSp45633e*
         RC3456pe - 1./36.*m13*m33*RSp45611e*RS34561pe + 1./72.*m13*m33
         *RSp45611e*RC3456pe + 1./12.*m13*m16*RSp45613e - 1./24.*m13*
         m16*RSp45633e - 1./24.*m13*m16*RSp45611e + 1./12.*m13*m15*
         RSp45613e - 1./24.*m13*m15*RSp45633e - 1./24.*m13*m15*
         RSp45611e + 1./12.*m13*m14*RSp45613e - 1./24.*m13*m14*
         RSp45633e - 1./24.*m13*m14*RSp45611e - 1./12.*pow(m13,2)*
         RSp45613e );

      M13p6x23RV +=  + AbV(mp2,L)*pi16 * ( 1./27.*pow(m13,2)*RSp45613e*
         RS34561pe + 1./27.*pow(m13,2)*RSp45613e*RS14563pe + 1./27.*
         pow(m13,2)*RSp45613e*RSe45613p + 1./27.*pow(m13,2)*pow(
         RSp45613e,2) - 1./54.*pow(m13,2)*RC3456pe*RSp45613e - 1./54.*
         pow(m13,2)*RSe45633p*RSp45613e + 1./24.*pow(m13,2)*RSp45633e
          - 1./54.*pow(m13,2)*RSp45633e*RS34561pe - 1./54.*pow(m13,2)*
         RSp45633e*RS14563pe - 1./54.*pow(m13,2)*RSp45633e*RSe45613p - 
         1./27.*pow(m13,2)*RSp45633e*RSp45613e + 1./108.*pow(m13,2)*
         RSp45633e*RC3456pe + 1./108.*pow(m13,2)*RSp45633e*RSe45633p + 
         1./108.*pow(m13,2)*pow(RSp45633e,2) - 1./54.*pow(m13,2)*
         RC1456pe*RSp45613e + 1./108.*pow(m13,2)*RC1456pe*RSp45633e - 1.
         /54.*pow(m13,2)*RSe45611p*RSp45613e + 1./108.*pow(m13,2)*
         RSe45611p*RSp45633e + 1./24.*pow(m13,2)*RSp45611e - 1./54.*
         pow(m13,2)*RSp45611e*RS34561pe - 1./54.*pow(m13,2)*RSp45611e*
         RS14563pe - 1./54.*pow(m13,2)*RSp45611e*RSe45613p - 1./27.*
         pow(m13,2)*RSp45611e*RSp45613e );

      M13p6x23RV +=  + AbV(mp2,L)*pi16 * ( 1./108.*pow(m13,2)*RSp45611e
         *RC3456pe + 1./108.*pow(m13,2)*RSp45611e*RSe45633p + 1./54.*
         pow(m13,2)*RSp45611e*RSp45633e + 1./108.*pow(m13,2)*RSp45611e*
         RC1456pe + 1./108.*pow(m13,2)*RSp45611e*RSe45611p + 1./108.*
         pow(m13,2)*pow(RSp45611e,2) + 1./18.*m11*m13*RSp45613e*
         RS14563pe - 1./36.*m11*m13*RSp45633e*RS14563pe - 1./36.*m11*
         m13*RC1456pe*RSp45613e + 1./72.*m11*m13*RC1456pe*RSp45633e - 1.
         /36.*m11*m13*RSp45611e*RS14563pe + 1./72.*m11*m13*RSp45611e*
         RC1456pe );

      M13p6x23RV +=  + pow(AbV(mp2,L),2) * ( 1./135.*mp2*pow(
         RSp45613e,2) - 1./135.*mp2*RSp45633e*RSp45613e + 1./540.*mp2*
         pow(RSp45633e,2) - 1./135.*mp2*RSp45611e*RSp45613e + 1./270.*
         mp2*RSp45611e*RSp45633e + 1./540.*mp2*pow(RSp45611e,2) + 1./81.
         *m66*RSp456e*RSp453e*RS345pe + 1./81.*m66*RSp456e*RSp453e*
         RSe453p + 1./162.*m66*RSp456e*pow(RSp453e,2) - 1./81.*m66*
         RSp456e*RS145pe*RSp453e - 1./81.*m66*RSp456e*RSe451p*RSp453e
          - 1./81.*m66*RSp456e*RSp451e*RS345pe - 1./81.*m66*RSp456e*
         RSp451e*RSe453p - 1./81.*m66*RSp456e*RSp451e*RSp453e + 1./81.*
         m66*RSp456e*RSp451e*RS145pe + 1./81.*m66*RSp456e*RSp451e*
         RSe451p + 1./162.*m66*RSp456e*pow(RSp451e,2) + 1./81.*m55*
         RSp465e*RSp463e*RS346pe + 1./81.*m55*RSp465e*RSp463e*RSe463p
          + 1./162.*m55*RSp465e*pow(RSp463e,2) - 1./81.*m55*RSp465e*
         RS146pe*RSp463e - 1./81.*m55*RSp465e*RSe461p*RSp463e - 1./81.*
         m55*RSp465e*RSp461e*RS346pe - 1./81.*m55*RSp465e*RSp461e*
         RSe463p );

      M13p6x23RV +=  + pow(AbV(mp2,L),2) * (  - 1./81.*m55*RSp465e*
         RSp461e*RSp463e + 1./81.*m55*RSp465e*RSp461e*RS146pe + 1./81.*
         m55*RSp465e*RSp461e*RSe461p + 1./162.*m55*RSp465e*pow(
         RSp461e,2) + 1./81.*m44*RSp564e*RSp563e*RS356pe + 1./81.*m44*
         RSp564e*RSp563e*RSe563p + 1./162.*m44*RSp564e*pow(RSp563e,2)
          - 1./81.*m44*RSp564e*RS156pe*RSp563e - 1./81.*m44*RSp564e*
         RSe561p*RSp563e - 1./81.*m44*RSp564e*RSp561e*RS356pe - 1./81.*
         m44*RSp564e*RSp561e*RSe563p - 1./81.*m44*RSp564e*RSp561e*
         RSp563e + 1./81.*m44*RSp564e*RSp561e*RS156pe + 1./81.*m44*
         RSp564e*RSp561e*RSe561p + 1./162.*m44*RSp564e*pow(RSp561e,2)
          + 23./810.*m33*pow(RSp45613e,2) + 8./405.*m33*RSp45633e*
         RSp45613e - 1./81.*m33*pow(RSp45633e,2) - 1./405.*m33*
         RSp45611e*RSp45613e - 11./810.*m33*RSp45611e*RSp45633e + 13./
         1620.*m33*pow(RSp45611e,2) - 7./810.*m13*pow(RSp45613e,2) + 7./
         810.*m13*RSp45633e*RSp45613e - 7./3240.*m13*pow(RSp45633e,2)
          + 7./810.*m13*RSp45611e*RSp45613e );

      M13p6x23RV +=  + pow(AbV(mp2,L),2) * (  - 7./1620.*m13*RSp45611e*
         RSp45633e - 7./3240.*m13*pow(RSp45611e,2) + 23./810.*m11*pow(
         RSp45613e,2) - 1./405.*m11*RSp45633e*RSp45613e + 13./1620.*m11
         *pow(RSp45633e,2) + 8./405.*m11*RSp45611e*RSp45613e - 11./810.
         *m11*RSp45611e*RSp45633e - 1./81.*m11*pow(RSp45611e,2) );

      M13p6x23RV +=  + AbV(mp2,L)*AbV(me2,L) * ( 1./135.*me2*RSp45613e*
         RSe45613p - 1./270.*me2*RSe45633p*RSp45613e - 1./270.*me2*
         RSp45633e*RSe45613p + 1./540.*me2*RSp45633e*RSe45633p - 1./270.
         *me2*RSe45611p*RSp45613e + 1./540.*me2*RSe45611p*RSp45633e - 1.
         /270.*me2*RSp45611e*RSe45613p + 1./540.*me2*RSp45611e*
         RSe45633p + 1./540.*me2*RSp45611e*RSe45611p + 1./135.*mp2*
         RSp45613e*RSe45613p - 1./270.*mp2*RSe45633p*RSp45613e - 1./270.
         *mp2*RSp45633e*RSe45613p + 1./540.*mp2*RSp45633e*RSe45633p - 1.
         /270.*mp2*RSe45611p*RSp45613e + 1./540.*mp2*RSe45611p*
         RSp45633e - 1./270.*mp2*RSp45611e*RSe45613p + 1./540.*mp2*
         RSp45611e*RSe45633p + 1./540.*mp2*RSp45611e*RSe45611p + 1./81.
         *m66*RSe456p*RSp453e*RS345pe + 1./81.*m66*RSe456p*RSp453e*
         RSe453p + 1./162.*m66*RSe456p*pow(RSp453e,2) - 1./81.*m66*
         RSe456p*RS145pe*RSp453e - 1./81.*m66*RSe456p*RSe451p*RSp453e
          - 1./81.*m66*RSe456p*RSp451e*RS345pe - 1./81.*m66*RSe456p*
         RSp451e*RSe453p );

      M13p6x23RV +=  + AbV(mp2,L)*AbV(me2,L) * (  - 1./81.*m66*RSe456p*
         RSp451e*RSp453e + 1./81.*m66*RSe456p*RSp451e*RS145pe + 1./81.*
         m66*RSe456p*RSp451e*RSe451p + 1./162.*m66*RSe456p*pow(
         RSp451e,2) + 1./81.*m66*RSp456e*RSe453p*RS345pe + 1./162.*m66*
         RSp456e*pow(RSe453p,2) - 1./81.*m66*RSp456e*RS145pe*RSe453p - 
         1./81.*m66*RSp456e*RSe451p*RS345pe - 1./81.*m66*RSp456e*
         RSe451p*RSe453p + 1./81.*m66*RSp456e*RSe451p*RS145pe + 1./162.
         *m66*RSp456e*pow(RSe451p,2) + 1./81.*m55*RSe465p*RSp463e*
         RS346pe + 1./81.*m55*RSe465p*RSp463e*RSe463p + 1./162.*m55*
         RSe465p*pow(RSp463e,2) - 1./81.*m55*RSe465p*RS146pe*RSp463e - 
         1./81.*m55*RSe465p*RSe461p*RSp463e - 1./81.*m55*RSe465p*
         RSp461e*RS346pe - 1./81.*m55*RSe465p*RSp461e*RSe463p - 1./81.*
         m55*RSe465p*RSp461e*RSp463e + 1./81.*m55*RSe465p*RSp461e*
         RS146pe + 1./81.*m55*RSe465p*RSp461e*RSe461p + 1./162.*m55*
         RSe465p*pow(RSp461e,2) + 1./81.*m55*RSp465e*RSe463p*RS346pe + 
         1./162.*m55*RSp465e*pow(RSe463p,2) );

      M13p6x23RV +=  + AbV(mp2,L)*AbV(me2,L) * (  - 1./81.*m55*RSp465e*
         RS146pe*RSe463p - 1./81.*m55*RSp465e*RSe461p*RS346pe - 1./81.*
         m55*RSp465e*RSe461p*RSe463p + 1./81.*m55*RSp465e*RSe461p*
         RS146pe + 1./162.*m55*RSp465e*pow(RSe461p,2) + 1./81.*m44*
         RSe564p*RSp563e*RS356pe + 1./81.*m44*RSe564p*RSp563e*RSe563p
          + 1./162.*m44*RSe564p*pow(RSp563e,2) - 1./81.*m44*RSe564p*
         RS156pe*RSp563e - 1./81.*m44*RSe564p*RSe561p*RSp563e - 1./81.*
         m44*RSe564p*RSp561e*RS356pe - 1./81.*m44*RSe564p*RSp561e*
         RSe563p - 1./81.*m44*RSe564p*RSp561e*RSp563e + 1./81.*m44*
         RSe564p*RSp561e*RS156pe + 1./81.*m44*RSe564p*RSp561e*RSe561p
          + 1./162.*m44*RSe564p*pow(RSp561e,2) + 1./81.*m44*RSp564e*
         RSe563p*RS356pe + 1./162.*m44*RSp564e*pow(RSe563p,2) - 1./81.*
         m44*RSp564e*RS156pe*RSe563p - 1./81.*m44*RSp564e*RSe561p*
         RS356pe - 1./81.*m44*RSp564e*RSe561p*RSe563p + 1./81.*m44*
         RSp564e*RSe561p*RS156pe + 1./162.*m44*RSp564e*pow(RSe561p,2)
          + 23./405.*m33*RSp45613e*RSe45613p );

      M13p6x23RV +=  + AbV(mp2,L)*AbV(me2,L) * ( 8./405.*m33*RSe45633p*
         RSp45613e + 8./405.*m33*RSp45633e*RSe45613p - 2./81.*m33*
         RSp45633e*RSe45633p - 1./405.*m33*RSe45611p*RSp45613e - 11./
         810.*m33*RSe45611p*RSp45633e - 1./405.*m33*RSp45611e*RSe45613p
          - 11./810.*m33*RSp45611e*RSe45633p + 13./810.*m33*RSp45611e*
         RSe45611p - 7./405.*m13*RSp45613e*RSe45613p + 7./810.*m13*
         RSe45633p*RSp45613e + 7./810.*m13*RSp45633e*RSe45613p - 7./
         1620.*m13*RSp45633e*RSe45633p + 7./810.*m13*RSe45611p*
         RSp45613e - 7./1620.*m13*RSe45611p*RSp45633e + 7./810.*m13*
         RSp45611e*RSe45613p - 7./1620.*m13*RSp45611e*RSe45633p - 7./
         1620.*m13*RSp45611e*RSe45611p + 23./405.*m11*RSp45613e*
         RSe45613p - 1./405.*m11*RSe45633p*RSp45613e - 1./405.*m11*
         RSp45633e*RSe45613p + 13./810.*m11*RSp45633e*RSe45633p + 8./
         405.*m11*RSe45611p*RSp45613e - 11./810.*m11*RSe45611p*
         RSp45633e + 8./405.*m11*RSp45611e*RSe45613p - 11./810.*m11*
         RSp45611e*RSe45633p );

      M13p6x23RV +=  + AbV(mp2,L)*AbV(me2,L) * (  - 2./81.*m11*
         RSp45611e*RSe45611p );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m11,m33,L) * (  - 1./162.*m33*m66
         *RSp456e*RS145pe*RS345pe - 1./162.*m33*m55*RSp465e*RS146pe*
         RS346pe - 1./162.*m33*m44*RSp564e*RS156pe*RS356pe + 1./54.*
         pow(m33,2)*RSp45633e*RS14563pe - 1./81.*m13*m66*RSp456e*
         RS145pe*RS345pe - 1./81.*m13*m55*RSp465e*RS146pe*RS346pe - 1./
         81.*m13*m44*RSp564e*RS156pe*RS356pe + 1./27.*m13*m33*RSp45633e
         *RS14563pe - 1./54.*m11*m66*RSp456e*RS145pe*RS345pe - 1./54.*
         m11*m55*RSp465e*RS146pe*RS346pe - 1./54.*m11*m44*RSp564e*
         RS156pe*RS356pe + 1./18.*m11*m33*RSp45633e*RS14563pe + 1./54.*
         m11*m33*RSp45611e*RS34561pe + 1./27.*m11*m13*RSp45611e*
         RS34561pe + 1./18.*pow(m11,2)*RSp45611e*RS34561pe );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m11,mp2,L) * (  - 1./162.*m33*m66
         *RSp456e*RS145pe*RSp453e - 1./162.*m33*m66*RSp456e*RSp451e*
         RS145pe - 1./162.*m33*m55*RSp465e*RS146pe*RSp463e - 1./162.*
         m33*m55*RSp465e*RSp461e*RS146pe - 1./162.*m33*m44*RSp564e*
         RS156pe*RSp563e - 1./162.*m33*m44*RSp564e*RSp561e*RS156pe - 1./
         81.*m13*m66*RSp456e*RS145pe*RSp453e + 1./81.*m13*m66*RSp456e*
         RSp451e*RS145pe - 1./81.*m13*m55*RSp465e*RS146pe*RSp463e + 1./
         81.*m13*m55*RSp465e*RSp461e*RS146pe - 1./81.*m13*m44*RSp564e*
         RS156pe*RSp563e + 1./81.*m13*m44*RSp564e*RSp561e*RS156pe - 1./
         54.*m11*m66*RSp456e*RS145pe*RSp453e - 1./162.*m11*m66*RSp456e*
         RSp451e*RS145pe - 1./54.*m11*m55*RSp465e*RS146pe*RSp463e - 1./
         162.*m11*m55*RSp465e*RSp461e*RS146pe - 1./54.*m11*m44*RSp564e*
         RS156pe*RSp563e - 1./162.*m11*m44*RSp564e*RSp561e*RS156pe + 1./
         54.*m11*m33*RSp45611e*RSp45613e + 1./54.*m11*m33*pow(
         RSp45611e,2) + 1./27.*m11*m13*RSp45611e*RSp45613e - 1./27.*m11
         *m13*pow(RSp45611e,2) );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m11,mp2,L) * ( 1./18.*pow(m11,2)*
         RSp45611e*RSp45613e + 1./54.*pow(m11,2)*pow(RSp45611e,2) );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m11,me2,L) * (  - 1./162.*m33*m66
         *RSp456e*RS145pe*RSe453p - 1./162.*m33*m66*RSp456e*RSe451p*
         RS145pe - 1./162.*m33*m55*RSp465e*RS146pe*RSe463p - 1./162.*
         m33*m55*RSp465e*RSe461p*RS146pe - 1./162.*m33*m44*RSp564e*
         RS156pe*RSe563p - 1./162.*m33*m44*RSp564e*RSe561p*RS156pe - 1./
         81.*m13*m66*RSp456e*RS145pe*RSe453p + 1./81.*m13*m66*RSp456e*
         RSe451p*RS145pe - 1./81.*m13*m55*RSp465e*RS146pe*RSe463p + 1./
         81.*m13*m55*RSp465e*RSe461p*RS146pe - 1./81.*m13*m44*RSp564e*
         RS156pe*RSe563p + 1./81.*m13*m44*RSp564e*RSe561p*RS156pe - 1./
         54.*m11*m66*RSp456e*RS145pe*RSe453p - 1./162.*m11*m66*RSp456e*
         RSe451p*RS145pe - 1./54.*m11*m55*RSp465e*RS146pe*RSe463p - 1./
         162.*m11*m55*RSp465e*RSe461p*RS146pe - 1./54.*m11*m44*RSp564e*
         RS156pe*RSe563p - 1./162.*m11*m44*RSp564e*RSe561p*RS156pe + 1./
         54.*m11*m33*RSp45611e*RSe45613p + 1./54.*m11*m33*RSp45611e*
         RSe45611p + 1./27.*m11*m13*RSp45611e*RSe45613p - 1./27.*m11*
         m13*RSp45611e*RSe45611p );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m11,me2,L) * ( 1./18.*pow(m11,2)*
         RSp45611e*RSe45613p + 1./54.*pow(m11,2)*RSp45611e*RSe45611p );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m11,L) * (  - 1./270.*mp2*
         RD1456ep*RSp45613e + 1./540.*mp2*RD1456ep*RSp45633e + 1./540.*
         mp2*RSp45611e*RD1456ep - 1./405.*m33*RD1456ep*RSp45613e - 11./
         810.*m33*RD1456ep*RSp45633e + 13./810.*m33*RSp45611e*RD1456ep
          - 1./324.*m33*m66*RSp456e*pow(RS145pe,2) - 1./324.*m33*m55*
         RSp465e*pow(RS146pe,2) - 1./324.*m33*m44*RSp564e*pow(
         RS156pe,2) + 7./810.*m13*RD1456ep*RSp45613e - 7./1620.*m13*
         RD1456ep*RSp45633e - 7./1620.*m13*RSp45611e*RD1456ep + 1./162.
         *m13*m66*RSp456e*pow(RS145pe,2) + 1./162.*m13*m55*RSp465e*pow(
         RS146pe,2) + 1./162.*m13*m44*RSp564e*pow(RS156pe,2) + 13./810.
         *m11*RD1456ep*RSp45613e - 19./1620.*m11*RD1456ep*RSp45633e - 
         37./1620.*m11*RSp45611e*RD1456ep - 1./324.*m11*m66*RSp456e*
         pow(RS145pe,2) - 1./324.*m11*m55*RSp465e*pow(RS146pe,2) - 1./
         324.*m11*m44*RSp564e*pow(RS156pe,2) + 1./54.*m11*m33*RSp45611e
         *RS14563pe + 1./54.*m11*m33*RSp45611e*RC1456pe + 1./27.*m11*
         m13*RSp45611e*RS14563pe );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m11,L) * (  - 1./27.*m11*m13*
         RSp45611e*RC1456pe + 1./18.*pow(m11,2)*RSp45611e*RS14563pe + 1.
         /54.*pow(m11,2)*RSp45611e*RC1456pe );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m14,L) * ( 1./216.*m44*mp2*
         RSp45611e - 1./108.*m44*mp2*RSp561e + 1./216.*m44*mp2*RSp564e
          - 1./432.*pow(m44,2)*RSp45611e - 1./216.*pow(m44,2)*RSp561e
          - 1./144.*pow(m44,2)*RSp564e + 1./216.*m33*mp2*RSp45611e - 1./
         108.*m33*mp2*RSp561e + 1./216.*m33*mp2*RSp564e - 1./432.*m33*
         m44*RSp45611e - 1./216.*m33*m44*RSp561e - 1./144.*m33*m44*
         RSp564e - 1./108.*m14*mp2*RSp45611e + 1./54.*m14*mp2*RSp561e
          - 1./108.*m14*mp2*RSp564e + 1./108.*m14*m44*RSp45611e + 1./54.
         *m14*m44*RSp564e + 1./216.*m14*m33*RSp45611e - 1./108.*m14*m33
         *RSp561e + 1./216.*m14*m33*RSp564e - 1./108.*pow(m14,2)*
         RSp45611e + 1./54.*pow(m14,2)*RSp561e - 1./108.*pow(m14,2)*
         RSp564e - 1./108.*m13*mp2*RSp45611e + 1./54.*m13*mp2*RSp561e
          - 1./108.*m13*mp2*RSp564e + 1./216.*m13*m44*RSp45611e + 1./
         108.*m13*m44*RSp561e + 1./72.*m13*m44*RSp564e - 1./108.*m13*
         m14*RSp45611e + 1./54.*m13*m14*RSp561e - 1./108.*m13*m14*
         RSp564e );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m14,L) * ( 1./108.*m11*mp2*
         RSp45611e - 1./54.*m11*mp2*RSp561e + 1./108.*m11*mp2*RSp564e
          - 5./432.*m11*m44*RSp45611e - 1./72.*m11*m44*RSp561e - 7./432.
         *m11*m44*RSp564e - 1./144.*m11*m33*RSp45611e - 1./216.*m11*m33
         *RSp561e - 1./432.*m11*m33*RSp564e + 5./216.*m11*m14*RSp45611e
          - 1./108.*m11*m14*RSp561e + 1./72.*m11*m14*RSp564e + 1./72.*
         m11*m13*RSp45611e + 1./108.*m11*m13*RSp561e + 1./216.*m11*m13*
         RSp564e - 1./72.*pow(m11,2)*RSp45611e - 1./108.*pow(m11,2)*
         RSp561e - 1./216.*pow(m11,2)*RSp564e );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m15,L) * ( 1./216.*m55*mp2*
         RSp45611e - 1./108.*m55*mp2*RSp461e + 1./216.*m55*mp2*RSp465e
          - 1./432.*pow(m55,2)*RSp45611e - 1./216.*pow(m55,2)*RSp461e
          - 1./144.*pow(m55,2)*RSp465e + 1./216.*m33*mp2*RSp45611e - 1./
         108.*m33*mp2*RSp461e + 1./216.*m33*mp2*RSp465e - 1./432.*m33*
         m55*RSp45611e - 1./216.*m33*m55*RSp461e - 1./144.*m33*m55*
         RSp465e - 1./108.*m15*mp2*RSp45611e + 1./54.*m15*mp2*RSp461e
          - 1./108.*m15*mp2*RSp465e + 1./108.*m15*m55*RSp45611e + 1./54.
         *m15*m55*RSp465e + 1./216.*m15*m33*RSp45611e - 1./108.*m15*m33
         *RSp461e + 1./216.*m15*m33*RSp465e - 1./108.*pow(m15,2)*
         RSp45611e + 1./54.*pow(m15,2)*RSp461e - 1./108.*pow(m15,2)*
         RSp465e - 1./108.*m13*mp2*RSp45611e + 1./54.*m13*mp2*RSp461e
          - 1./108.*m13*mp2*RSp465e + 1./216.*m13*m55*RSp45611e + 1./
         108.*m13*m55*RSp461e + 1./72.*m13*m55*RSp465e - 1./108.*m13*
         m15*RSp45611e + 1./54.*m13*m15*RSp461e - 1./108.*m13*m15*
         RSp465e );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m15,L) * ( 1./108.*m11*mp2*
         RSp45611e - 1./54.*m11*mp2*RSp461e + 1./108.*m11*mp2*RSp465e
          - 5./432.*m11*m55*RSp45611e - 1./72.*m11*m55*RSp461e - 7./432.
         *m11*m55*RSp465e - 1./144.*m11*m33*RSp45611e - 1./216.*m11*m33
         *RSp461e - 1./432.*m11*m33*RSp465e + 5./216.*m11*m15*RSp45611e
          - 1./108.*m11*m15*RSp461e + 1./72.*m11*m15*RSp465e + 1./72.*
         m11*m13*RSp45611e + 1./108.*m11*m13*RSp461e + 1./216.*m11*m13*
         RSp465e - 1./72.*pow(m11,2)*RSp45611e - 1./108.*pow(m11,2)*
         RSp461e - 1./216.*pow(m11,2)*RSp465e );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m16,L) * ( 1./216.*m66*mp2*
         RSp45611e - 1./108.*m66*mp2*RSp451e + 1./216.*m66*mp2*RSp456e
          - 1./432.*pow(m66,2)*RSp45611e - 1./216.*pow(m66,2)*RSp451e
          - 1./144.*pow(m66,2)*RSp456e + 1./216.*m33*mp2*RSp45611e - 1./
         108.*m33*mp2*RSp451e + 1./216.*m33*mp2*RSp456e - 1./432.*m33*
         m66*RSp45611e - 1./216.*m33*m66*RSp451e - 1./144.*m33*m66*
         RSp456e - 1./108.*m16*mp2*RSp45611e + 1./54.*m16*mp2*RSp451e
          - 1./108.*m16*mp2*RSp456e + 1./108.*m16*m66*RSp45611e + 1./54.
         *m16*m66*RSp456e + 1./216.*m16*m33*RSp45611e - 1./108.*m16*m33
         *RSp451e + 1./216.*m16*m33*RSp456e - 1./108.*pow(m16,2)*
         RSp45611e + 1./54.*pow(m16,2)*RSp451e - 1./108.*pow(m16,2)*
         RSp456e - 1./108.*m13*mp2*RSp45611e + 1./54.*m13*mp2*RSp451e
          - 1./108.*m13*mp2*RSp456e + 1./216.*m13*m66*RSp45611e + 1./
         108.*m13*m66*RSp451e + 1./72.*m13*m66*RSp456e - 1./108.*m13*
         m16*RSp45611e + 1./54.*m13*m16*RSp451e - 1./108.*m13*m16*
         RSp456e );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m16,L) * ( 1./108.*m11*mp2*
         RSp45611e - 1./54.*m11*mp2*RSp451e + 1./108.*m11*mp2*RSp456e
          - 5./432.*m11*m66*RSp45611e - 1./72.*m11*m66*RSp451e - 7./432.
         *m11*m66*RSp456e - 1./144.*m11*m33*RSp45611e - 1./216.*m11*m33
         *RSp451e - 1./432.*m11*m33*RSp456e + 5./216.*m11*m16*RSp45611e
          - 1./108.*m11*m16*RSp451e + 1./72.*m11*m16*RSp456e + 1./72.*
         m11*m13*RSp45611e + 1./108.*m11*m13*RSp451e + 1./216.*m11*m13*
         RSp456e - 1./72.*pow(m11,2)*RSp45611e - 1./108.*pow(m11,2)*
         RSp451e - 1./216.*pow(m11,2)*RSp456e );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m33,mp2,L) * (  - 1./162.*m33*m66
         *RSp456e*RSp453e*RS345pe - 1./54.*m33*m66*RSp456e*RSp451e*
         RS345pe - 1./162.*m33*m55*RSp465e*RSp463e*RS346pe - 1./54.*m33
         *m55*RSp465e*RSp461e*RS346pe - 1./162.*m33*m44*RSp564e*RSp563e
         *RS356pe - 1./54.*m33*m44*RSp564e*RSp561e*RS356pe + 1./18.*
         pow(m33,2)*RSp45633e*RSp45613e + 1./54.*pow(m33,2)*pow(
         RSp45633e,2) + 1./81.*m13*m66*RSp456e*RSp453e*RS345pe - 1./81.
         *m13*m66*RSp456e*RSp451e*RS345pe + 1./81.*m13*m55*RSp465e*
         RSp463e*RS346pe - 1./81.*m13*m55*RSp465e*RSp461e*RS346pe + 1./
         81.*m13*m44*RSp564e*RSp563e*RS356pe - 1./81.*m13*m44*RSp564e*
         RSp561e*RS356pe + 1./27.*m13*m33*RSp45633e*RSp45613e - 1./27.*
         m13*m33*pow(RSp45633e,2) - 1./162.*m11*m66*RSp456e*RSp453e*
         RS345pe - 1./162.*m11*m66*RSp456e*RSp451e*RS345pe - 1./162.*
         m11*m55*RSp465e*RSp463e*RS346pe - 1./162.*m11*m55*RSp465e*
         RSp461e*RS346pe - 1./162.*m11*m44*RSp564e*RSp563e*RS356pe - 1./
         162.*m11*m44*RSp564e*RSp561e*RS356pe );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m33,mp2,L) * ( 1./54.*m11*m33*
         RSp45633e*RSp45613e + 1./54.*m11*m33*pow(RSp45633e,2) );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m33,me2,L) * (  - 1./162.*m33*m66
         *RSp456e*RSe453p*RS345pe - 1./54.*m33*m66*RSp456e*RSe451p*
         RS345pe - 1./162.*m33*m55*RSp465e*RSe463p*RS346pe - 1./54.*m33
         *m55*RSp465e*RSe461p*RS346pe - 1./162.*m33*m44*RSp564e*RSe563p
         *RS356pe - 1./54.*m33*m44*RSp564e*RSe561p*RS356pe + 1./18.*
         pow(m33,2)*RSp45633e*RSe45613p + 1./54.*pow(m33,2)*RSp45633e*
         RSe45633p + 1./81.*m13*m66*RSp456e*RSe453p*RS345pe - 1./81.*
         m13*m66*RSp456e*RSe451p*RS345pe + 1./81.*m13*m55*RSp465e*
         RSe463p*RS346pe - 1./81.*m13*m55*RSp465e*RSe461p*RS346pe + 1./
         81.*m13*m44*RSp564e*RSe563p*RS356pe - 1./81.*m13*m44*RSp564e*
         RSe561p*RS356pe + 1./27.*m13*m33*RSp45633e*RSe45613p - 1./27.*
         m13*m33*RSp45633e*RSe45633p - 1./162.*m11*m66*RSp456e*RSe453p*
         RS345pe - 1./162.*m11*m66*RSp456e*RSe451p*RS345pe - 1./162.*
         m11*m55*RSp465e*RSe463p*RS346pe - 1./162.*m11*m55*RSp465e*
         RSe461p*RS346pe - 1./162.*m11*m44*RSp564e*RSe563p*RS356pe - 1./
         162.*m11*m44*RSp564e*RSe561p*RS356pe );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m33,me2,L) * ( 1./54.*m11*m33*
         RSp45633e*RSe45613p + 1./54.*m11*m33*RSp45633e*RSe45633p );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m33,L) * (  - 1./270.*mp2*
         RD3456ep*RSp45613e + 1./540.*mp2*RSp45633e*RD3456ep + 1./540.*
         mp2*RSp45611e*RD3456ep + 13./810.*m33*RD3456ep*RSp45613e - 37./
         1620.*m33*RSp45633e*RD3456ep - 19./1620.*m33*RSp45611e*
         RD3456ep - 1./324.*m33*m66*RSp456e*pow(RS345pe,2) - 1./324.*
         m33*m55*RSp465e*pow(RS346pe,2) - 1./324.*m33*m44*RSp564e*pow(
         RS356pe,2) + 1./18.*pow(m33,2)*RSp45633e*RS34561pe + 1./54.*
         pow(m33,2)*RSp45633e*RC3456pe + 7./810.*m13*RD3456ep*RSp45613e
          - 7./1620.*m13*RSp45633e*RD3456ep - 7./1620.*m13*RSp45611e*
         RD3456ep + 1./162.*m13*m66*RSp456e*pow(RS345pe,2) + 1./162.*
         m13*m55*RSp465e*pow(RS346pe,2) + 1./162.*m13*m44*RSp564e*pow(
         RS356pe,2) + 1./27.*m13*m33*RSp45633e*RS34561pe - 1./27.*m13*
         m33*RSp45633e*RC3456pe - 1./405.*m11*RD3456ep*RSp45613e + 13./
         810.*m11*RSp45633e*RD3456ep - 11./810.*m11*RSp45611e*RD3456ep
          - 1./324.*m11*m66*RSp456e*pow(RS345pe,2) - 1./324.*m11*m55*
         RSp465e*pow(RS346pe,2) );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m33,L) * (  - 1./324.*m11*m44*
         RSp564e*pow(RS356pe,2) + 1./54.*m11*m33*RSp45633e*RS34561pe + 
         1./54.*m11*m33*RSp45633e*RC3456pe );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m34,L) * ( 1./216.*m44*mp2*
         RSp45633e - 1./108.*m44*mp2*RSp563e + 1./216.*m44*mp2*RSp564e
          - 1./432.*pow(m44,2)*RSp45633e - 1./216.*pow(m44,2)*RSp563e
          - 1./144.*pow(m44,2)*RSp564e - 1./108.*m34*mp2*RSp45633e + 1./
         54.*m34*mp2*RSp563e - 1./108.*m34*mp2*RSp564e + 1./108.*m34*
         m44*RSp45633e + 1./54.*m34*m44*RSp564e - 1./108.*pow(m34,2)*
         RSp45633e + 1./54.*pow(m34,2)*RSp563e - 1./108.*pow(m34,2)*
         RSp564e + 1./108.*m33*mp2*RSp45633e - 1./54.*m33*mp2*RSp563e
          + 1./108.*m33*mp2*RSp564e - 5./432.*m33*m44*RSp45633e - 1./72.
         *m33*m44*RSp563e - 7./432.*m33*m44*RSp564e + 5./216.*m33*m34*
         RSp45633e - 1./108.*m33*m34*RSp563e + 1./72.*m33*m34*RSp564e
          - 1./72.*pow(m33,2)*RSp45633e - 1./108.*pow(m33,2)*RSp563e - 
         1./216.*pow(m33,2)*RSp564e - 1./108.*m13*mp2*RSp45633e + 1./54.
         *m13*mp2*RSp563e - 1./108.*m13*mp2*RSp564e + 1./216.*m13*m44*
         RSp45633e + 1./108.*m13*m44*RSp563e + 1./72.*m13*m44*RSp564e
          - 1./108.*m13*m34*RSp45633e );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m34,L) * ( 1./54.*m13*m34*RSp563e
          - 1./108.*m13*m34*RSp564e + 1./72.*m13*m33*RSp45633e + 1./108.
         *m13*m33*RSp563e + 1./216.*m13*m33*RSp564e + 1./216.*m11*mp2*
         RSp45633e - 1./108.*m11*mp2*RSp563e + 1./216.*m11*mp2*RSp564e
          - 1./432.*m11*m44*RSp45633e - 1./216.*m11*m44*RSp563e - 1./
         144.*m11*m44*RSp564e + 1./216.*m11*m34*RSp45633e - 1./108.*m11
         *m34*RSp563e + 1./216.*m11*m34*RSp564e - 1./144.*m11*m33*
         RSp45633e - 1./216.*m11*m33*RSp563e - 1./432.*m11*m33*RSp564e
          );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m35,L) * ( 1./216.*m55*mp2*
         RSp45633e - 1./108.*m55*mp2*RSp463e + 1./216.*m55*mp2*RSp465e
          - 1./432.*pow(m55,2)*RSp45633e - 1./216.*pow(m55,2)*RSp463e
          - 1./144.*pow(m55,2)*RSp465e - 1./108.*m35*mp2*RSp45633e + 1./
         54.*m35*mp2*RSp463e - 1./108.*m35*mp2*RSp465e + 1./108.*m35*
         m55*RSp45633e + 1./54.*m35*m55*RSp465e - 1./108.*pow(m35,2)*
         RSp45633e + 1./54.*pow(m35,2)*RSp463e - 1./108.*pow(m35,2)*
         RSp465e + 1./108.*m33*mp2*RSp45633e - 1./54.*m33*mp2*RSp463e
          + 1./108.*m33*mp2*RSp465e - 5./432.*m33*m55*RSp45633e - 1./72.
         *m33*m55*RSp463e - 7./432.*m33*m55*RSp465e + 5./216.*m33*m35*
         RSp45633e - 1./108.*m33*m35*RSp463e + 1./72.*m33*m35*RSp465e
          - 1./72.*pow(m33,2)*RSp45633e - 1./108.*pow(m33,2)*RSp463e - 
         1./216.*pow(m33,2)*RSp465e - 1./108.*m13*mp2*RSp45633e + 1./54.
         *m13*mp2*RSp463e - 1./108.*m13*mp2*RSp465e + 1./216.*m13*m55*
         RSp45633e + 1./108.*m13*m55*RSp463e + 1./72.*m13*m55*RSp465e
          - 1./108.*m13*m35*RSp45633e );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m35,L) * ( 1./54.*m13*m35*RSp463e
          - 1./108.*m13*m35*RSp465e + 1./72.*m13*m33*RSp45633e + 1./108.
         *m13*m33*RSp463e + 1./216.*m13*m33*RSp465e + 1./216.*m11*mp2*
         RSp45633e - 1./108.*m11*mp2*RSp463e + 1./216.*m11*mp2*RSp465e
          - 1./432.*m11*m55*RSp45633e - 1./216.*m11*m55*RSp463e - 1./
         144.*m11*m55*RSp465e + 1./216.*m11*m35*RSp45633e - 1./108.*m11
         *m35*RSp463e + 1./216.*m11*m35*RSp465e - 1./144.*m11*m33*
         RSp45633e - 1./216.*m11*m33*RSp463e - 1./432.*m11*m33*RSp465e
          );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m36,L) * ( 1./216.*m66*mp2*
         RSp45633e - 1./108.*m66*mp2*RSp453e + 1./216.*m66*mp2*RSp456e
          - 1./432.*pow(m66,2)*RSp45633e - 1./216.*pow(m66,2)*RSp453e
          - 1./144.*pow(m66,2)*RSp456e - 1./108.*m36*mp2*RSp45633e + 1./
         54.*m36*mp2*RSp453e - 1./108.*m36*mp2*RSp456e + 1./108.*m36*
         m66*RSp45633e + 1./54.*m36*m66*RSp456e - 1./108.*pow(m36,2)*
         RSp45633e + 1./54.*pow(m36,2)*RSp453e - 1./108.*pow(m36,2)*
         RSp456e + 1./108.*m33*mp2*RSp45633e - 1./54.*m33*mp2*RSp453e
          + 1./108.*m33*mp2*RSp456e - 5./432.*m33*m66*RSp45633e - 1./72.
         *m33*m66*RSp453e - 7./432.*m33*m66*RSp456e + 5./216.*m33*m36*
         RSp45633e - 1./108.*m33*m36*RSp453e + 1./72.*m33*m36*RSp456e
          - 1./72.*pow(m33,2)*RSp45633e - 1./108.*pow(m33,2)*RSp453e - 
         1./216.*pow(m33,2)*RSp456e - 1./108.*m13*mp2*RSp45633e + 1./54.
         *m13*mp2*RSp453e - 1./108.*m13*mp2*RSp456e + 1./216.*m13*m66*
         RSp45633e + 1./108.*m13*m66*RSp453e + 1./72.*m13*m66*RSp456e
          - 1./108.*m13*m36*RSp45633e );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(m36,L) * ( 1./54.*m13*m36*RSp453e
          - 1./108.*m13*m36*RSp456e + 1./72.*m13*m33*RSp45633e + 1./108.
         *m13*m33*RSp453e + 1./216.*m13*m33*RSp456e + 1./216.*m11*mp2*
         RSp45633e - 1./108.*m11*mp2*RSp453e + 1./216.*m11*mp2*RSp456e
          - 1./432.*m11*m66*RSp45633e - 1./216.*m11*m66*RSp453e - 1./
         144.*m11*m66*RSp456e + 1./216.*m11*m36*RSp45633e - 1./108.*m11
         *m36*RSp453e + 1./216.*m11*m36*RSp456e - 1./144.*m11*m33*
         RSp45633e - 1./216.*m11*m33*RSp453e - 1./432.*m11*m33*RSp456e
          );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(mp2,L) * ( 1./162.*m66*mp2*
         RSp456e*pow(RSp453e,2) - 1./81.*m66*mp2*RSp456e*RSp451e*
         RSp453e + 1./162.*m66*mp2*RSp456e*pow(RSp451e,2) + 1./162.*m55
         *mp2*RSp465e*pow(RSp463e,2) - 1./81.*m55*mp2*RSp465e*RSp461e*
         RSp463e + 1./162.*m55*mp2*RSp465e*pow(RSp461e,2) + 1./162.*m44
         *mp2*RSp564e*pow(RSp563e,2) - 1./81.*m44*mp2*RSp564e*RSp561e*
         RSp563e + 1./162.*m44*mp2*RSp564e*pow(RSp561e,2) - 1./108.*m33
         *m66*RSp456e*pow(RSp453e,2) - 1./162.*m33*m66*RSp456e*RSp451e*
         RSp453e - 1./324.*m33*m66*RSp456e*pow(RSp451e,2) - 1./108.*m33
         *m55*RSp465e*pow(RSp463e,2) - 1./162.*m33*m55*RSp465e*RSp461e*
         RSp463e - 1./324.*m33*m55*RSp465e*pow(RSp461e,2) - 1./108.*m33
         *m44*RSp564e*pow(RSp563e,2) - 1./162.*m33*m44*RSp564e*RSp561e*
         RSp563e - 1./324.*m33*m44*RSp564e*pow(RSp561e,2) + 1./162.*m13
         *m66*RSp456e*pow(RSp453e,2) - 1./81.*m13*m66*RSp456e*RSp451e*
         RSp453e + 1./162.*m13*m66*RSp456e*pow(RSp451e,2) + 1./162.*m13
         *m55*RSp465e*pow(RSp463e,2) );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(mp2,L) * (  - 1./81.*m13*m55*
         RSp465e*RSp461e*RSp463e + 1./162.*m13*m55*RSp465e*pow(
         RSp461e,2) + 1./162.*m13*m44*RSp564e*pow(RSp563e,2) - 1./81.*
         m13*m44*RSp564e*RSp561e*RSp563e + 1./162.*m13*m44*RSp564e*pow(
         RSp561e,2) - 1./324.*m11*m66*RSp456e*pow(RSp453e,2) - 1./162.*
         m11*m66*RSp456e*RSp451e*RSp453e - 1./108.*m11*m66*RSp456e*pow(
         RSp451e,2) - 1./324.*m11*m55*RSp465e*pow(RSp463e,2) - 1./162.*
         m11*m55*RSp465e*RSp461e*RSp463e - 1./108.*m11*m55*RSp465e*pow(
         RSp461e,2) - 1./324.*m11*m44*RSp564e*pow(RSp563e,2) - 1./162.*
         m11*m44*RSp564e*RSp561e*RSp563e - 1./108.*m11*m44*RSp564e*pow(
         RSp561e,2) );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(me2,mp2,L) * ( 1./81.*m66*me2*
         RSp456e*RSp453e*RSe453p - 1./81.*m66*me2*RSp456e*RSe451p*
         RSp453e - 1./81.*m66*me2*RSp456e*RSp451e*RSe453p + 1./81.*m66*
         me2*RSp456e*RSp451e*RSe451p + 1./81.*m55*me2*RSp465e*RSp463e*
         RSe463p - 1./81.*m55*me2*RSp465e*RSe461p*RSp463e - 1./81.*m55*
         me2*RSp465e*RSp461e*RSe463p + 1./81.*m55*me2*RSp465e*RSp461e*
         RSe461p + 1./81.*m44*me2*RSp564e*RSp563e*RSe563p - 1./81.*m44*
         me2*RSp564e*RSe561p*RSp563e - 1./81.*m44*me2*RSp564e*RSp561e*
         RSe563p + 1./81.*m44*me2*RSp564e*RSp561e*RSe561p - 1./54.*m33*
         m66*RSp456e*RSp453e*RSe453p - 1./162.*m33*m66*RSp456e*RSe451p*
         RSp453e - 1./162.*m33*m66*RSp456e*RSp451e*RSe453p - 1./162.*
         m33*m66*RSp456e*RSp451e*RSe451p - 1./54.*m33*m55*RSp465e*
         RSp463e*RSe463p - 1./162.*m33*m55*RSp465e*RSe461p*RSp463e - 1./
         162.*m33*m55*RSp465e*RSp461e*RSe463p - 1./162.*m33*m55*RSp465e
         *RSp461e*RSe461p - 1./54.*m33*m44*RSp564e*RSp563e*RSe563p - 1./
         162.*m33*m44*RSp564e*RSe561p*RSp563e );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(me2,mp2,L) * (  - 1./162.*m33*m44
         *RSp564e*RSp561e*RSe563p - 1./162.*m33*m44*RSp564e*RSp561e*
         RSe561p + 1./81.*m13*m66*RSp456e*RSp453e*RSe453p - 1./81.*m13*
         m66*RSp456e*RSe451p*RSp453e - 1./81.*m13*m66*RSp456e*RSp451e*
         RSe453p + 1./81.*m13*m66*RSp456e*RSp451e*RSe451p + 1./81.*m13*
         m55*RSp465e*RSp463e*RSe463p - 1./81.*m13*m55*RSp465e*RSe461p*
         RSp463e - 1./81.*m13*m55*RSp465e*RSp461e*RSe463p + 1./81.*m13*
         m55*RSp465e*RSp461e*RSe461p + 1./81.*m13*m44*RSp564e*RSp563e*
         RSe563p - 1./81.*m13*m44*RSp564e*RSe561p*RSp563e - 1./81.*m13*
         m44*RSp564e*RSp561e*RSe563p + 1./81.*m13*m44*RSp564e*RSp561e*
         RSe561p - 1./162.*m11*m66*RSp456e*RSp453e*RSe453p - 1./162.*
         m11*m66*RSp456e*RSe451p*RSp453e - 1./162.*m11*m66*RSp456e*
         RSp451e*RSe453p - 1./54.*m11*m66*RSp456e*RSp451e*RSe451p - 1./
         162.*m11*m55*RSp465e*RSp463e*RSe463p - 1./162.*m11*m55*RSp465e
         *RSe461p*RSp463e - 1./162.*m11*m55*RSp465e*RSp461e*RSe463p - 1.
         /54.*m11*m55*RSp465e*RSp461e*RSe461p );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(me2,mp2,L) * (  - 1./162.*m11*m44
         *RSp564e*RSp563e*RSe563p - 1./162.*m11*m44*RSp564e*RSe561p*
         RSp563e - 1./162.*m11*m44*RSp564e*RSp561e*RSe563p - 1./54.*m11
         *m44*RSp564e*RSp561e*RSe561p );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(me2,L) * ( 1./162.*m66*me2*
         RSp456e*pow(RSe453p,2) - 1./81.*m66*me2*RSp456e*RSe451p*
         RSe453p + 1./162.*m66*me2*RSp456e*pow(RSe451p,2) + 1./162.*m55
         *me2*RSp465e*pow(RSe463p,2) - 1./81.*m55*me2*RSp465e*RSe461p*
         RSe463p + 1./162.*m55*me2*RSp465e*pow(RSe461p,2) + 1./162.*m44
         *me2*RSp564e*pow(RSe563p,2) - 1./81.*m44*me2*RSp564e*RSe561p*
         RSe563p + 1./162.*m44*me2*RSp564e*pow(RSe561p,2) - 1./108.*m33
         *m66*RSp456e*pow(RSe453p,2) - 1./162.*m33*m66*RSp456e*RSe451p*
         RSe453p - 1./324.*m33*m66*RSp456e*pow(RSe451p,2) - 1./108.*m33
         *m55*RSp465e*pow(RSe463p,2) - 1./162.*m33*m55*RSp465e*RSe461p*
         RSe463p - 1./324.*m33*m55*RSp465e*pow(RSe461p,2) - 1./108.*m33
         *m44*RSp564e*pow(RSe563p,2) - 1./162.*m33*m44*RSp564e*RSe561p*
         RSe563p - 1./324.*m33*m44*RSp564e*pow(RSe561p,2) + 1./162.*m13
         *m66*RSp456e*pow(RSe453p,2) - 1./81.*m13*m66*RSp456e*RSe451p*
         RSe453p + 1./162.*m13*m66*RSp456e*pow(RSe451p,2) + 1./162.*m13
         *m55*RSp465e*pow(RSe463p,2) );

      M13p6x23RV +=  + AbV(mp2,L)*BbV(me2,L) * (  - 1./81.*m13*m55*
         RSp465e*RSe461p*RSe463p + 1./162.*m13*m55*RSp465e*pow(
         RSe461p,2) + 1./162.*m13*m44*RSp564e*pow(RSe563p,2) - 1./81.*
         m13*m44*RSp564e*RSe561p*RSe563p + 1./162.*m13*m44*RSp564e*pow(
         RSe561p,2) - 1./324.*m11*m66*RSp456e*pow(RSe453p,2) - 1./162.*
         m11*m66*RSp456e*RSe451p*RSe453p - 1./108.*m11*m66*RSp456e*pow(
         RSe451p,2) - 1./324.*m11*m55*RSp465e*pow(RSe463p,2) - 1./162.*
         m11*m55*RSp465e*RSe461p*RSe463p - 1./108.*m11*m55*RSp465e*pow(
         RSe461p,2) - 1./324.*m11*m44*RSp564e*pow(RSe563p,2) - 1./162.*
         m11*m44*RSp564e*RSe561p*RSe563p - 1./108.*m11*m44*RSp564e*pow(
         RSe561p,2) );

      M13p6x23RV +=  + AbV(mp2,L)*CbV(m11,L) * ( 1./54.*m11*m33*
         RSp45611e*RD1456ep - 1./27.*m11*m13*RSp45611e*RD1456ep + 1./54.
         *pow(m11,2)*RSp45611e*RD1456ep );

      M13p6x23RV +=  + AbV(mp2,L)*CbV(m33,L) * ( 1./54.*pow(m33,2)*
         RSp45633e*RD3456ep - 1./27.*m13*m33*RSp45633e*RD3456ep + 1./54.
         *m11*m33*RSp45633e*RD3456ep );

      M13p6x23RV +=  + AbV(me2,L)*pi16 * (  - 1./36.*m13*RD3456ep*
         RSe45613p + 1./72.*m13*RSe45633p*RD3456ep - 1./36.*m13*
         RD1456ep*RSe45613p + 1./72.*m13*RD1456ep*RSe45633p + 1./72.*
         m13*RSe45611p*RD3456ep + 1./72.*m13*RSe45611p*RD1456ep - 1./12.
         *m13*me2*RSe45613p + 1./18.*m13*me2*pow(RSe45613p,2) + 1./24.*
         m13*me2*RSe45633p - 1./18.*m13*me2*RSe45633p*RSe45613p + 1./72.
         *m13*me2*pow(RSe45633p,2) + 1./24.*m13*me2*RSe563p + 1./24.*
         m13*me2*RSe463p + 1./24.*m13*me2*RSe453p + 1./24.*m13*me2*
         RSe45611p - 1./18.*m13*me2*RSe45611p*RSe45613p + 1./36.*m13*
         me2*RSe45611p*RSe45633p + 1./72.*m13*me2*pow(RSe45611p,2) + 1./
         24.*m13*me2*RSe561p + 1./24.*m13*me2*RSe461p + 1./24.*m13*me2*
         RSe451p + 1./18.*m13*mp2*RSp45613e*RSe45613p - 1./36.*m13*mp2*
         RSe45633p*RSp45613e - 1./36.*m13*mp2*RSp45633e*RSe45613p + 1./
         72.*m13*mp2*RSp45633e*RSe45633p - 1./36.*m13*mp2*RSe45611p*
         RSp45613e + 1./72.*m13*mp2*RSe45611p*RSp45633e - 1./36.*m13*
         mp2*RSp45611e*RSe45613p );

      M13p6x23RV +=  + AbV(me2,L)*pi16 * ( 1./72.*m13*mp2*RSp45611e*
         RSe45633p + 1./72.*m13*mp2*RSp45611e*RSe45611p + 1./12.*m13*
         m36*RSe45613p - 1./24.*m13*m36*RSe45633p - 1./24.*m13*m36*
         RSe45611p + 1./12.*m13*m35*RSe45613p - 1./24.*m13*m35*
         RSe45633p - 1./24.*m13*m35*RSe45611p + 1./12.*m13*m34*
         RSe45613p - 1./24.*m13*m34*RSe45633p - 1./24.*m13*m34*
         RSe45611p + 1./18.*m13*m33*RSe45613p*RS34561pe - 1./36.*m13*
         m33*RC3456pe*RSe45613p - 1./36.*m13*m33*RSe45633p*RS34561pe + 
         1./72.*m13*m33*RSe45633p*RC3456pe - 1./36.*m13*m33*RSe45611p*
         RS34561pe + 1./72.*m13*m33*RSe45611p*RC3456pe + 1./12.*m13*m16
         *RSe45613p - 1./24.*m13*m16*RSe45633p - 1./24.*m13*m16*
         RSe45611p + 1./12.*m13*m15*RSe45613p - 1./24.*m13*m15*
         RSe45633p - 1./24.*m13*m15*RSe45611p + 1./12.*m13*m14*
         RSe45613p - 1./24.*m13*m14*RSe45633p - 1./24.*m13*m14*
         RSe45611p - 1./12.*pow(m13,2)*RSe45613p + 1./27.*pow(m13,2)*
         RSe45613p*RS34561pe );

      M13p6x23RV +=  + AbV(me2,L)*pi16 * ( 1./27.*pow(m13,2)*RSe45613p*
         RS14563pe + 1./27.*pow(m13,2)*pow(RSe45613p,2) + 1./27.*pow(
         m13,2)*RSp45613e*RSe45613p - 1./54.*pow(m13,2)*RC3456pe*
         RSe45613p + 1./24.*pow(m13,2)*RSe45633p - 1./54.*pow(m13,2)*
         RSe45633p*RS34561pe - 1./54.*pow(m13,2)*RSe45633p*RS14563pe - 
         1./27.*pow(m13,2)*RSe45633p*RSe45613p - 1./54.*pow(m13,2)*
         RSe45633p*RSp45613e + 1./108.*pow(m13,2)*RSe45633p*RC3456pe + 
         1./108.*pow(m13,2)*pow(RSe45633p,2) - 1./54.*pow(m13,2)*
         RSp45633e*RSe45613p + 1./108.*pow(m13,2)*RSp45633e*RSe45633p
          - 1./54.*pow(m13,2)*RC1456pe*RSe45613p + 1./108.*pow(m13,2)*
         RC1456pe*RSe45633p + 1./24.*pow(m13,2)*RSe45611p - 1./54.*pow(
         m13,2)*RSe45611p*RS34561pe - 1./54.*pow(m13,2)*RSe45611p*
         RS14563pe - 1./27.*pow(m13,2)*RSe45611p*RSe45613p - 1./54.*
         pow(m13,2)*RSe45611p*RSp45613e + 1./108.*pow(m13,2)*RSe45611p*
         RC3456pe + 1./54.*pow(m13,2)*RSe45611p*RSe45633p + 1./108.*
         pow(m13,2)*RSe45611p*RSp45633e );

      M13p6x23RV +=  + AbV(me2,L)*pi16 * ( 1./108.*pow(m13,2)*RSe45611p
         *RC1456pe + 1./108.*pow(m13,2)*pow(RSe45611p,2) - 1./54.*pow(
         m13,2)*RSp45611e*RSe45613p + 1./108.*pow(m13,2)*RSp45611e*
         RSe45633p + 1./108.*pow(m13,2)*RSp45611e*RSe45611p + 1./18.*
         m11*m13*RSe45613p*RS14563pe - 1./36.*m11*m13*RSe45633p*
         RS14563pe - 1./36.*m11*m13*RC1456pe*RSe45613p + 1./72.*m11*m13
         *RC1456pe*RSe45633p - 1./36.*m11*m13*RSe45611p*RS14563pe + 1./
         72.*m11*m13*RSe45611p*RC1456pe );

      M13p6x23RV +=  + pow(AbV(me2,L),2) * ( 1./135.*me2*pow(
         RSe45613p,2) - 1./135.*me2*RSe45633p*RSe45613p + 1./540.*me2*
         pow(RSe45633p,2) - 1./135.*me2*RSe45611p*RSe45613p + 1./270.*
         me2*RSe45611p*RSe45633p + 1./540.*me2*pow(RSe45611p,2) + 1./81.
         *m66*RSe456p*RSe453p*RS345pe + 1./162.*m66*RSe456p*pow(
         RSe453p,2) - 1./81.*m66*RSe456p*RS145pe*RSe453p - 1./81.*m66*
         RSe456p*RSe451p*RS345pe - 1./81.*m66*RSe456p*RSe451p*RSe453p
          + 1./81.*m66*RSe456p*RSe451p*RS145pe + 1./162.*m66*RSe456p*
         pow(RSe451p,2) + 1./81.*m55*RSe465p*RSe463p*RS346pe + 1./162.*
         m55*RSe465p*pow(RSe463p,2) - 1./81.*m55*RSe465p*RS146pe*
         RSe463p - 1./81.*m55*RSe465p*RSe461p*RS346pe - 1./81.*m55*
         RSe465p*RSe461p*RSe463p + 1./81.*m55*RSe465p*RSe461p*RS146pe
          + 1./162.*m55*RSe465p*pow(RSe461p,2) + 1./81.*m44*RSe564p*
         RSe563p*RS356pe + 1./162.*m44*RSe564p*pow(RSe563p,2) - 1./81.*
         m44*RSe564p*RS156pe*RSe563p - 1./81.*m44*RSe564p*RSe561p*
         RS356pe );

      M13p6x23RV +=  + pow(AbV(me2,L),2) * (  - 1./81.*m44*RSe564p*
         RSe561p*RSe563p + 1./81.*m44*RSe564p*RSe561p*RS156pe + 1./162.
         *m44*RSe564p*pow(RSe561p,2) + 23./810.*m33*pow(RSe45613p,2) + 
         8./405.*m33*RSe45633p*RSe45613p - 1./81.*m33*pow(RSe45633p,2)
          - 1./405.*m33*RSe45611p*RSe45613p - 11./810.*m33*RSe45611p*
         RSe45633p + 13./1620.*m33*pow(RSe45611p,2) - 7./810.*m13*pow(
         RSe45613p,2) + 7./810.*m13*RSe45633p*RSe45613p - 7./3240.*m13*
         pow(RSe45633p,2) + 7./810.*m13*RSe45611p*RSe45613p - 7./1620.*
         m13*RSe45611p*RSe45633p - 7./3240.*m13*pow(RSe45611p,2) + 23./
         810.*m11*pow(RSe45613p,2) - 1./405.*m11*RSe45633p*RSe45613p + 
         13./1620.*m11*pow(RSe45633p,2) + 8./405.*m11*RSe45611p*
         RSe45613p - 11./810.*m11*RSe45611p*RSe45633p - 1./81.*m11*pow(
         RSe45611p,2) );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m11,m33,L) * (  - 1./162.*m33*m66
         *RSe456p*RS145pe*RS345pe - 1./162.*m33*m55*RSe465p*RS146pe*
         RS346pe - 1./162.*m33*m44*RSe564p*RS156pe*RS356pe + 1./54.*
         pow(m33,2)*RSe45633p*RS14563pe - 1./81.*m13*m66*RSe456p*
         RS145pe*RS345pe - 1./81.*m13*m55*RSe465p*RS146pe*RS346pe - 1./
         81.*m13*m44*RSe564p*RS156pe*RS356pe + 1./27.*m13*m33*RSe45633p
         *RS14563pe - 1./54.*m11*m66*RSe456p*RS145pe*RS345pe - 1./54.*
         m11*m55*RSe465p*RS146pe*RS346pe - 1./54.*m11*m44*RSe564p*
         RS156pe*RS356pe + 1./18.*m11*m33*RSe45633p*RS14563pe + 1./54.*
         m11*m33*RSe45611p*RS34561pe + 1./27.*m11*m13*RSe45611p*
         RS34561pe + 1./18.*pow(m11,2)*RSe45611p*RS34561pe );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m11,mp2,L) * (  - 1./162.*m33*m66
         *RSe456p*RS145pe*RSp453e - 1./162.*m33*m66*RSe456p*RSp451e*
         RS145pe - 1./162.*m33*m55*RSe465p*RS146pe*RSp463e - 1./162.*
         m33*m55*RSe465p*RSp461e*RS146pe - 1./162.*m33*m44*RSe564p*
         RS156pe*RSp563e - 1./162.*m33*m44*RSe564p*RSp561e*RS156pe - 1./
         81.*m13*m66*RSe456p*RS145pe*RSp453e + 1./81.*m13*m66*RSe456p*
         RSp451e*RS145pe - 1./81.*m13*m55*RSe465p*RS146pe*RSp463e + 1./
         81.*m13*m55*RSe465p*RSp461e*RS146pe - 1./81.*m13*m44*RSe564p*
         RS156pe*RSp563e + 1./81.*m13*m44*RSe564p*RSp561e*RS156pe - 1./
         54.*m11*m66*RSe456p*RS145pe*RSp453e - 1./162.*m11*m66*RSe456p*
         RSp451e*RS145pe - 1./54.*m11*m55*RSe465p*RS146pe*RSp463e - 1./
         162.*m11*m55*RSe465p*RSp461e*RS146pe - 1./54.*m11*m44*RSe564p*
         RS156pe*RSp563e - 1./162.*m11*m44*RSe564p*RSp561e*RS156pe + 1./
         54.*m11*m33*RSe45611p*RSp45613e + 1./54.*m11*m33*RSp45611e*
         RSe45611p + 1./27.*m11*m13*RSe45611p*RSp45613e - 1./27.*m11*
         m13*RSp45611e*RSe45611p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m11,mp2,L) * ( 1./18.*pow(m11,2)*
         RSe45611p*RSp45613e + 1./54.*pow(m11,2)*RSp45611e*RSe45611p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m11,me2,L) * (  - 1./162.*m33*m66
         *RSe456p*RS145pe*RSe453p - 1./162.*m33*m66*RSe456p*RSe451p*
         RS145pe - 1./162.*m33*m55*RSe465p*RS146pe*RSe463p - 1./162.*
         m33*m55*RSe465p*RSe461p*RS146pe - 1./162.*m33*m44*RSe564p*
         RS156pe*RSe563p - 1./162.*m33*m44*RSe564p*RSe561p*RS156pe - 1./
         81.*m13*m66*RSe456p*RS145pe*RSe453p + 1./81.*m13*m66*RSe456p*
         RSe451p*RS145pe - 1./81.*m13*m55*RSe465p*RS146pe*RSe463p + 1./
         81.*m13*m55*RSe465p*RSe461p*RS146pe - 1./81.*m13*m44*RSe564p*
         RS156pe*RSe563p + 1./81.*m13*m44*RSe564p*RSe561p*RS156pe - 1./
         54.*m11*m66*RSe456p*RS145pe*RSe453p - 1./162.*m11*m66*RSe456p*
         RSe451p*RS145pe - 1./54.*m11*m55*RSe465p*RS146pe*RSe463p - 1./
         162.*m11*m55*RSe465p*RSe461p*RS146pe - 1./54.*m11*m44*RSe564p*
         RS156pe*RSe563p - 1./162.*m11*m44*RSe564p*RSe561p*RS156pe + 1./
         54.*m11*m33*RSe45611p*RSe45613p + 1./54.*m11*m33*pow(
         RSe45611p,2) + 1./27.*m11*m13*RSe45611p*RSe45613p - 1./27.*m11
         *m13*pow(RSe45611p,2) );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m11,me2,L) * ( 1./18.*pow(m11,2)*
         RSe45611p*RSe45613p + 1./54.*pow(m11,2)*pow(RSe45611p,2) );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m11,L) * (  - 1./270.*me2*
         RD1456ep*RSe45613p + 1./540.*me2*RD1456ep*RSe45633p + 1./540.*
         me2*RSe45611p*RD1456ep - 1./405.*m33*RD1456ep*RSe45613p - 11./
         810.*m33*RD1456ep*RSe45633p + 13./810.*m33*RSe45611p*RD1456ep
          - 1./324.*m33*m66*RSe456p*pow(RS145pe,2) - 1./324.*m33*m55*
         RSe465p*pow(RS146pe,2) - 1./324.*m33*m44*RSe564p*pow(
         RS156pe,2) + 7./810.*m13*RD1456ep*RSe45613p - 7./1620.*m13*
         RD1456ep*RSe45633p - 7./1620.*m13*RSe45611p*RD1456ep + 1./162.
         *m13*m66*RSe456p*pow(RS145pe,2) + 1./162.*m13*m55*RSe465p*pow(
         RS146pe,2) + 1./162.*m13*m44*RSe564p*pow(RS156pe,2) + 13./810.
         *m11*RD1456ep*RSe45613p - 19./1620.*m11*RD1456ep*RSe45633p - 
         37./1620.*m11*RSe45611p*RD1456ep - 1./324.*m11*m66*RSe456p*
         pow(RS145pe,2) - 1./324.*m11*m55*RSe465p*pow(RS146pe,2) - 1./
         324.*m11*m44*RSe564p*pow(RS156pe,2) + 1./54.*m11*m33*RSe45611p
         *RS14563pe + 1./54.*m11*m33*RSe45611p*RC1456pe + 1./27.*m11*
         m13*RSe45611p*RS14563pe );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m11,L) * (  - 1./27.*m11*m13*
         RSe45611p*RC1456pe + 1./18.*pow(m11,2)*RSe45611p*RS14563pe + 1.
         /54.*pow(m11,2)*RSe45611p*RC1456pe );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m14,L) * ( 1./216.*m44*me2*
         RSe45611p - 1./108.*m44*me2*RSe561p + 1./216.*m44*me2*RSe564p
          - 1./432.*pow(m44,2)*RSe45611p - 1./216.*pow(m44,2)*RSe561p
          - 1./144.*pow(m44,2)*RSe564p + 1./216.*m33*me2*RSe45611p - 1./
         108.*m33*me2*RSe561p + 1./216.*m33*me2*RSe564p - 1./432.*m33*
         m44*RSe45611p - 1./216.*m33*m44*RSe561p - 1./144.*m33*m44*
         RSe564p - 1./108.*m14*me2*RSe45611p + 1./54.*m14*me2*RSe561p
          - 1./108.*m14*me2*RSe564p + 1./108.*m14*m44*RSe45611p + 1./54.
         *m14*m44*RSe564p + 1./216.*m14*m33*RSe45611p - 1./108.*m14*m33
         *RSe561p + 1./216.*m14*m33*RSe564p - 1./108.*pow(m14,2)*
         RSe45611p + 1./54.*pow(m14,2)*RSe561p - 1./108.*pow(m14,2)*
         RSe564p - 1./108.*m13*me2*RSe45611p + 1./54.*m13*me2*RSe561p
          - 1./108.*m13*me2*RSe564p + 1./216.*m13*m44*RSe45611p + 1./
         108.*m13*m44*RSe561p + 1./72.*m13*m44*RSe564p - 1./108.*m13*
         m14*RSe45611p + 1./54.*m13*m14*RSe561p - 1./108.*m13*m14*
         RSe564p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m14,L) * ( 1./108.*m11*me2*
         RSe45611p - 1./54.*m11*me2*RSe561p + 1./108.*m11*me2*RSe564p
          - 5./432.*m11*m44*RSe45611p - 1./72.*m11*m44*RSe561p - 7./432.
         *m11*m44*RSe564p - 1./144.*m11*m33*RSe45611p - 1./216.*m11*m33
         *RSe561p - 1./432.*m11*m33*RSe564p + 5./216.*m11*m14*RSe45611p
          - 1./108.*m11*m14*RSe561p + 1./72.*m11*m14*RSe564p + 1./72.*
         m11*m13*RSe45611p + 1./108.*m11*m13*RSe561p + 1./216.*m11*m13*
         RSe564p - 1./72.*pow(m11,2)*RSe45611p - 1./108.*pow(m11,2)*
         RSe561p - 1./216.*pow(m11,2)*RSe564p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m15,L) * ( 1./216.*m55*me2*
         RSe45611p - 1./108.*m55*me2*RSe461p + 1./216.*m55*me2*RSe465p
          - 1./432.*pow(m55,2)*RSe45611p - 1./216.*pow(m55,2)*RSe461p
          - 1./144.*pow(m55,2)*RSe465p + 1./216.*m33*me2*RSe45611p - 1./
         108.*m33*me2*RSe461p + 1./216.*m33*me2*RSe465p - 1./432.*m33*
         m55*RSe45611p - 1./216.*m33*m55*RSe461p - 1./144.*m33*m55*
         RSe465p - 1./108.*m15*me2*RSe45611p + 1./54.*m15*me2*RSe461p
          - 1./108.*m15*me2*RSe465p + 1./108.*m15*m55*RSe45611p + 1./54.
         *m15*m55*RSe465p + 1./216.*m15*m33*RSe45611p - 1./108.*m15*m33
         *RSe461p + 1./216.*m15*m33*RSe465p - 1./108.*pow(m15,2)*
         RSe45611p + 1./54.*pow(m15,2)*RSe461p - 1./108.*pow(m15,2)*
         RSe465p - 1./108.*m13*me2*RSe45611p + 1./54.*m13*me2*RSe461p
          - 1./108.*m13*me2*RSe465p + 1./216.*m13*m55*RSe45611p + 1./
         108.*m13*m55*RSe461p + 1./72.*m13*m55*RSe465p - 1./108.*m13*
         m15*RSe45611p + 1./54.*m13*m15*RSe461p - 1./108.*m13*m15*
         RSe465p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m15,L) * ( 1./108.*m11*me2*
         RSe45611p - 1./54.*m11*me2*RSe461p + 1./108.*m11*me2*RSe465p
          - 5./432.*m11*m55*RSe45611p - 1./72.*m11*m55*RSe461p - 7./432.
         *m11*m55*RSe465p - 1./144.*m11*m33*RSe45611p - 1./216.*m11*m33
         *RSe461p - 1./432.*m11*m33*RSe465p + 5./216.*m11*m15*RSe45611p
          - 1./108.*m11*m15*RSe461p + 1./72.*m11*m15*RSe465p + 1./72.*
         m11*m13*RSe45611p + 1./108.*m11*m13*RSe461p + 1./216.*m11*m13*
         RSe465p - 1./72.*pow(m11,2)*RSe45611p - 1./108.*pow(m11,2)*
         RSe461p - 1./216.*pow(m11,2)*RSe465p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m16,L) * ( 1./216.*m66*me2*
         RSe45611p - 1./108.*m66*me2*RSe451p + 1./216.*m66*me2*RSe456p
          - 1./432.*pow(m66,2)*RSe45611p - 1./216.*pow(m66,2)*RSe451p
          - 1./144.*pow(m66,2)*RSe456p + 1./216.*m33*me2*RSe45611p - 1./
         108.*m33*me2*RSe451p + 1./216.*m33*me2*RSe456p - 1./432.*m33*
         m66*RSe45611p - 1./216.*m33*m66*RSe451p - 1./144.*m33*m66*
         RSe456p - 1./108.*m16*me2*RSe45611p + 1./54.*m16*me2*RSe451p
          - 1./108.*m16*me2*RSe456p + 1./108.*m16*m66*RSe45611p + 1./54.
         *m16*m66*RSe456p + 1./216.*m16*m33*RSe45611p - 1./108.*m16*m33
         *RSe451p + 1./216.*m16*m33*RSe456p - 1./108.*pow(m16,2)*
         RSe45611p + 1./54.*pow(m16,2)*RSe451p - 1./108.*pow(m16,2)*
         RSe456p - 1./108.*m13*me2*RSe45611p + 1./54.*m13*me2*RSe451p
          - 1./108.*m13*me2*RSe456p + 1./216.*m13*m66*RSe45611p + 1./
         108.*m13*m66*RSe451p + 1./72.*m13*m66*RSe456p - 1./108.*m13*
         m16*RSe45611p + 1./54.*m13*m16*RSe451p - 1./108.*m13*m16*
         RSe456p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m16,L) * ( 1./108.*m11*me2*
         RSe45611p - 1./54.*m11*me2*RSe451p + 1./108.*m11*me2*RSe456p
          - 5./432.*m11*m66*RSe45611p - 1./72.*m11*m66*RSe451p - 7./432.
         *m11*m66*RSe456p - 1./144.*m11*m33*RSe45611p - 1./216.*m11*m33
         *RSe451p - 1./432.*m11*m33*RSe456p + 5./216.*m11*m16*RSe45611p
          - 1./108.*m11*m16*RSe451p + 1./72.*m11*m16*RSe456p + 1./72.*
         m11*m13*RSe45611p + 1./108.*m11*m13*RSe451p + 1./216.*m11*m13*
         RSe456p - 1./72.*pow(m11,2)*RSe45611p - 1./108.*pow(m11,2)*
         RSe451p - 1./216.*pow(m11,2)*RSe456p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m33,mp2,L) * (  - 1./162.*m33*m66
         *RSe456p*RSp453e*RS345pe - 1./54.*m33*m66*RSe456p*RSp451e*
         RS345pe - 1./162.*m33*m55*RSe465p*RSp463e*RS346pe - 1./54.*m33
         *m55*RSe465p*RSp461e*RS346pe - 1./162.*m33*m44*RSe564p*RSp563e
         *RS356pe - 1./54.*m33*m44*RSe564p*RSp561e*RS356pe + 1./18.*
         pow(m33,2)*RSe45633p*RSp45613e + 1./54.*pow(m33,2)*RSp45633e*
         RSe45633p + 1./81.*m13*m66*RSe456p*RSp453e*RS345pe - 1./81.*
         m13*m66*RSe456p*RSp451e*RS345pe + 1./81.*m13*m55*RSe465p*
         RSp463e*RS346pe - 1./81.*m13*m55*RSe465p*RSp461e*RS346pe + 1./
         81.*m13*m44*RSe564p*RSp563e*RS356pe - 1./81.*m13*m44*RSe564p*
         RSp561e*RS356pe + 1./27.*m13*m33*RSe45633p*RSp45613e - 1./27.*
         m13*m33*RSp45633e*RSe45633p - 1./162.*m11*m66*RSe456p*RSp453e*
         RS345pe - 1./162.*m11*m66*RSe456p*RSp451e*RS345pe - 1./162.*
         m11*m55*RSe465p*RSp463e*RS346pe - 1./162.*m11*m55*RSe465p*
         RSp461e*RS346pe - 1./162.*m11*m44*RSe564p*RSp563e*RS356pe - 1./
         162.*m11*m44*RSe564p*RSp561e*RS356pe );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m33,mp2,L) * ( 1./54.*m11*m33*
         RSe45633p*RSp45613e + 1./54.*m11*m33*RSp45633e*RSe45633p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m33,me2,L) * (  - 1./162.*m33*m66
         *RSe456p*RSe453p*RS345pe - 1./54.*m33*m66*RSe456p*RSe451p*
         RS345pe - 1./162.*m33*m55*RSe465p*RSe463p*RS346pe - 1./54.*m33
         *m55*RSe465p*RSe461p*RS346pe - 1./162.*m33*m44*RSe564p*RSe563p
         *RS356pe - 1./54.*m33*m44*RSe564p*RSe561p*RS356pe + 1./18.*
         pow(m33,2)*RSe45633p*RSe45613p + 1./54.*pow(m33,2)*pow(
         RSe45633p,2) + 1./81.*m13*m66*RSe456p*RSe453p*RS345pe - 1./81.
         *m13*m66*RSe456p*RSe451p*RS345pe + 1./81.*m13*m55*RSe465p*
         RSe463p*RS346pe - 1./81.*m13*m55*RSe465p*RSe461p*RS346pe + 1./
         81.*m13*m44*RSe564p*RSe563p*RS356pe - 1./81.*m13*m44*RSe564p*
         RSe561p*RS356pe + 1./27.*m13*m33*RSe45633p*RSe45613p - 1./27.*
         m13*m33*pow(RSe45633p,2) - 1./162.*m11*m66*RSe456p*RSe453p*
         RS345pe - 1./162.*m11*m66*RSe456p*RSe451p*RS345pe - 1./162.*
         m11*m55*RSe465p*RSe463p*RS346pe - 1./162.*m11*m55*RSe465p*
         RSe461p*RS346pe - 1./162.*m11*m44*RSe564p*RSe563p*RS356pe - 1./
         162.*m11*m44*RSe564p*RSe561p*RS356pe );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m33,me2,L) * ( 1./54.*m11*m33*
         RSe45633p*RSe45613p + 1./54.*m11*m33*pow(RSe45633p,2) );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m33,L) * (  - 1./270.*me2*
         RD3456ep*RSe45613p + 1./540.*me2*RSe45633p*RD3456ep + 1./540.*
         me2*RSe45611p*RD3456ep + 13./810.*m33*RD3456ep*RSe45613p - 37./
         1620.*m33*RSe45633p*RD3456ep - 19./1620.*m33*RSe45611p*
         RD3456ep - 1./324.*m33*m66*RSe456p*pow(RS345pe,2) - 1./324.*
         m33*m55*RSe465p*pow(RS346pe,2) - 1./324.*m33*m44*RSe564p*pow(
         RS356pe,2) + 1./18.*pow(m33,2)*RSe45633p*RS34561pe + 1./54.*
         pow(m33,2)*RSe45633p*RC3456pe + 7./810.*m13*RD3456ep*RSe45613p
          - 7./1620.*m13*RSe45633p*RD3456ep - 7./1620.*m13*RSe45611p*
         RD3456ep + 1./162.*m13*m66*RSe456p*pow(RS345pe,2) + 1./162.*
         m13*m55*RSe465p*pow(RS346pe,2) + 1./162.*m13*m44*RSe564p*pow(
         RS356pe,2) + 1./27.*m13*m33*RSe45633p*RS34561pe - 1./27.*m13*
         m33*RSe45633p*RC3456pe - 1./405.*m11*RD3456ep*RSe45613p + 13./
         810.*m11*RSe45633p*RD3456ep - 11./810.*m11*RSe45611p*RD3456ep
          - 1./324.*m11*m66*RSe456p*pow(RS345pe,2) - 1./324.*m11*m55*
         RSe465p*pow(RS346pe,2) );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m33,L) * (  - 1./324.*m11*m44*
         RSe564p*pow(RS356pe,2) + 1./54.*m11*m33*RSe45633p*RS34561pe + 
         1./54.*m11*m33*RSe45633p*RC3456pe );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m34,L) * ( 1./216.*m44*me2*
         RSe45633p - 1./108.*m44*me2*RSe563p + 1./216.*m44*me2*RSe564p
          - 1./432.*pow(m44,2)*RSe45633p - 1./216.*pow(m44,2)*RSe563p
          - 1./144.*pow(m44,2)*RSe564p - 1./108.*m34*me2*RSe45633p + 1./
         54.*m34*me2*RSe563p - 1./108.*m34*me2*RSe564p + 1./108.*m34*
         m44*RSe45633p + 1./54.*m34*m44*RSe564p - 1./108.*pow(m34,2)*
         RSe45633p + 1./54.*pow(m34,2)*RSe563p - 1./108.*pow(m34,2)*
         RSe564p + 1./108.*m33*me2*RSe45633p - 1./54.*m33*me2*RSe563p
          + 1./108.*m33*me2*RSe564p - 5./432.*m33*m44*RSe45633p - 1./72.
         *m33*m44*RSe563p - 7./432.*m33*m44*RSe564p + 5./216.*m33*m34*
         RSe45633p - 1./108.*m33*m34*RSe563p + 1./72.*m33*m34*RSe564p
          - 1./72.*pow(m33,2)*RSe45633p - 1./108.*pow(m33,2)*RSe563p - 
         1./216.*pow(m33,2)*RSe564p - 1./108.*m13*me2*RSe45633p + 1./54.
         *m13*me2*RSe563p - 1./108.*m13*me2*RSe564p + 1./216.*m13*m44*
         RSe45633p + 1./108.*m13*m44*RSe563p + 1./72.*m13*m44*RSe564p
          - 1./108.*m13*m34*RSe45633p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m34,L) * ( 1./54.*m13*m34*RSe563p
          - 1./108.*m13*m34*RSe564p + 1./72.*m13*m33*RSe45633p + 1./108.
         *m13*m33*RSe563p + 1./216.*m13*m33*RSe564p + 1./216.*m11*me2*
         RSe45633p - 1./108.*m11*me2*RSe563p + 1./216.*m11*me2*RSe564p
          - 1./432.*m11*m44*RSe45633p - 1./216.*m11*m44*RSe563p - 1./
         144.*m11*m44*RSe564p + 1./216.*m11*m34*RSe45633p - 1./108.*m11
         *m34*RSe563p + 1./216.*m11*m34*RSe564p - 1./144.*m11*m33*
         RSe45633p - 1./216.*m11*m33*RSe563p - 1./432.*m11*m33*RSe564p
          );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m35,L) * ( 1./216.*m55*me2*
         RSe45633p - 1./108.*m55*me2*RSe463p + 1./216.*m55*me2*RSe465p
          - 1./432.*pow(m55,2)*RSe45633p - 1./216.*pow(m55,2)*RSe463p
          - 1./144.*pow(m55,2)*RSe465p - 1./108.*m35*me2*RSe45633p + 1./
         54.*m35*me2*RSe463p - 1./108.*m35*me2*RSe465p + 1./108.*m35*
         m55*RSe45633p + 1./54.*m35*m55*RSe465p - 1./108.*pow(m35,2)*
         RSe45633p + 1./54.*pow(m35,2)*RSe463p - 1./108.*pow(m35,2)*
         RSe465p + 1./108.*m33*me2*RSe45633p - 1./54.*m33*me2*RSe463p
          + 1./108.*m33*me2*RSe465p - 5./432.*m33*m55*RSe45633p - 1./72.
         *m33*m55*RSe463p - 7./432.*m33*m55*RSe465p + 5./216.*m33*m35*
         RSe45633p - 1./108.*m33*m35*RSe463p + 1./72.*m33*m35*RSe465p
          - 1./72.*pow(m33,2)*RSe45633p - 1./108.*pow(m33,2)*RSe463p - 
         1./216.*pow(m33,2)*RSe465p - 1./108.*m13*me2*RSe45633p + 1./54.
         *m13*me2*RSe463p - 1./108.*m13*me2*RSe465p + 1./216.*m13*m55*
         RSe45633p + 1./108.*m13*m55*RSe463p + 1./72.*m13*m55*RSe465p
          - 1./108.*m13*m35*RSe45633p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m35,L) * ( 1./54.*m13*m35*RSe463p
          - 1./108.*m13*m35*RSe465p + 1./72.*m13*m33*RSe45633p + 1./108.
         *m13*m33*RSe463p + 1./216.*m13*m33*RSe465p + 1./216.*m11*me2*
         RSe45633p - 1./108.*m11*me2*RSe463p + 1./216.*m11*me2*RSe465p
          - 1./432.*m11*m55*RSe45633p - 1./216.*m11*m55*RSe463p - 1./
         144.*m11*m55*RSe465p + 1./216.*m11*m35*RSe45633p - 1./108.*m11
         *m35*RSe463p + 1./216.*m11*m35*RSe465p - 1./144.*m11*m33*
         RSe45633p - 1./216.*m11*m33*RSe463p - 1./432.*m11*m33*RSe465p
          );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m36,L) * ( 1./216.*m66*me2*
         RSe45633p - 1./108.*m66*me2*RSe453p + 1./216.*m66*me2*RSe456p
          - 1./432.*pow(m66,2)*RSe45633p - 1./216.*pow(m66,2)*RSe453p
          - 1./144.*pow(m66,2)*RSe456p - 1./108.*m36*me2*RSe45633p + 1./
         54.*m36*me2*RSe453p - 1./108.*m36*me2*RSe456p + 1./108.*m36*
         m66*RSe45633p + 1./54.*m36*m66*RSe456p - 1./108.*pow(m36,2)*
         RSe45633p + 1./54.*pow(m36,2)*RSe453p - 1./108.*pow(m36,2)*
         RSe456p + 1./108.*m33*me2*RSe45633p - 1./54.*m33*me2*RSe453p
          + 1./108.*m33*me2*RSe456p - 5./432.*m33*m66*RSe45633p - 1./72.
         *m33*m66*RSe453p - 7./432.*m33*m66*RSe456p + 5./216.*m33*m36*
         RSe45633p - 1./108.*m33*m36*RSe453p + 1./72.*m33*m36*RSe456p
          - 1./72.*pow(m33,2)*RSe45633p - 1./108.*pow(m33,2)*RSe453p - 
         1./216.*pow(m33,2)*RSe456p - 1./108.*m13*me2*RSe45633p + 1./54.
         *m13*me2*RSe453p - 1./108.*m13*me2*RSe456p + 1./216.*m13*m66*
         RSe45633p + 1./108.*m13*m66*RSe453p + 1./72.*m13*m66*RSe456p
          - 1./108.*m13*m36*RSe45633p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(m36,L) * ( 1./54.*m13*m36*RSe453p
          - 1./108.*m13*m36*RSe456p + 1./72.*m13*m33*RSe45633p + 1./108.
         *m13*m33*RSe453p + 1./216.*m13*m33*RSe456p + 1./216.*m11*me2*
         RSe45633p - 1./108.*m11*me2*RSe453p + 1./216.*m11*me2*RSe456p
          - 1./432.*m11*m66*RSe45633p - 1./216.*m11*m66*RSe453p - 1./
         144.*m11*m66*RSe456p + 1./216.*m11*m36*RSe45633p - 1./108.*m11
         *m36*RSe453p + 1./216.*m11*m36*RSe456p - 1./144.*m11*m33*
         RSe45633p - 1./216.*m11*m33*RSe453p - 1./432.*m11*m33*RSe456p
          );

      M13p6x23RV +=  + AbV(me2,L)*BbV(mp2,L) * ( 1./162.*m66*mp2*
         RSe456p*pow(RSp453e,2) - 1./81.*m66*mp2*RSe456p*RSp451e*
         RSp453e + 1./162.*m66*mp2*RSe456p*pow(RSp451e,2) + 1./162.*m55
         *mp2*RSe465p*pow(RSp463e,2) - 1./81.*m55*mp2*RSe465p*RSp461e*
         RSp463e + 1./162.*m55*mp2*RSe465p*pow(RSp461e,2) + 1./162.*m44
         *mp2*RSe564p*pow(RSp563e,2) - 1./81.*m44*mp2*RSe564p*RSp561e*
         RSp563e + 1./162.*m44*mp2*RSe564p*pow(RSp561e,2) - 1./108.*m33
         *m66*RSe456p*pow(RSp453e,2) - 1./162.*m33*m66*RSe456p*RSp451e*
         RSp453e - 1./324.*m33*m66*RSe456p*pow(RSp451e,2) - 1./108.*m33
         *m55*RSe465p*pow(RSp463e,2) - 1./162.*m33*m55*RSe465p*RSp461e*
         RSp463e - 1./324.*m33*m55*RSe465p*pow(RSp461e,2) - 1./108.*m33
         *m44*RSe564p*pow(RSp563e,2) - 1./162.*m33*m44*RSe564p*RSp561e*
         RSp563e - 1./324.*m33*m44*RSe564p*pow(RSp561e,2) + 1./162.*m13
         *m66*RSe456p*pow(RSp453e,2) - 1./81.*m13*m66*RSe456p*RSp451e*
         RSp453e + 1./162.*m13*m66*RSe456p*pow(RSp451e,2) + 1./162.*m13
         *m55*RSe465p*pow(RSp463e,2) );

      M13p6x23RV +=  + AbV(me2,L)*BbV(mp2,L) * (  - 1./81.*m13*m55*
         RSe465p*RSp461e*RSp463e + 1./162.*m13*m55*RSe465p*pow(
         RSp461e,2) + 1./162.*m13*m44*RSe564p*pow(RSp563e,2) - 1./81.*
         m13*m44*RSe564p*RSp561e*RSp563e + 1./162.*m13*m44*RSe564p*pow(
         RSp561e,2) - 1./324.*m11*m66*RSe456p*pow(RSp453e,2) - 1./162.*
         m11*m66*RSe456p*RSp451e*RSp453e - 1./108.*m11*m66*RSe456p*pow(
         RSp451e,2) - 1./324.*m11*m55*RSe465p*pow(RSp463e,2) - 1./162.*
         m11*m55*RSe465p*RSp461e*RSp463e - 1./108.*m11*m55*RSe465p*pow(
         RSp461e,2) - 1./324.*m11*m44*RSe564p*pow(RSp563e,2) - 1./162.*
         m11*m44*RSe564p*RSp561e*RSp563e - 1./108.*m11*m44*RSe564p*pow(
         RSp561e,2) );

      M13p6x23RV +=  + AbV(me2,L)*BbV(me2,mp2,L) * ( 1./81.*m66*me2*
         RSe456p*RSp453e*RSe453p - 1./81.*m66*me2*RSe456p*RSe451p*
         RSp453e - 1./81.*m66*me2*RSe456p*RSp451e*RSe453p + 1./81.*m66*
         me2*RSe456p*RSp451e*RSe451p + 1./81.*m55*me2*RSe465p*RSp463e*
         RSe463p - 1./81.*m55*me2*RSe465p*RSe461p*RSp463e - 1./81.*m55*
         me2*RSe465p*RSp461e*RSe463p + 1./81.*m55*me2*RSe465p*RSp461e*
         RSe461p + 1./81.*m44*me2*RSe564p*RSp563e*RSe563p - 1./81.*m44*
         me2*RSe564p*RSe561p*RSp563e - 1./81.*m44*me2*RSe564p*RSp561e*
         RSe563p + 1./81.*m44*me2*RSe564p*RSp561e*RSe561p - 1./54.*m33*
         m66*RSe456p*RSp453e*RSe453p - 1./162.*m33*m66*RSe456p*RSe451p*
         RSp453e - 1./162.*m33*m66*RSe456p*RSp451e*RSe453p - 1./162.*
         m33*m66*RSe456p*RSp451e*RSe451p - 1./54.*m33*m55*RSe465p*
         RSp463e*RSe463p - 1./162.*m33*m55*RSe465p*RSe461p*RSp463e - 1./
         162.*m33*m55*RSe465p*RSp461e*RSe463p - 1./162.*m33*m55*RSe465p
         *RSp461e*RSe461p - 1./54.*m33*m44*RSe564p*RSp563e*RSe563p - 1./
         162.*m33*m44*RSe564p*RSe561p*RSp563e );

      M13p6x23RV +=  + AbV(me2,L)*BbV(me2,mp2,L) * (  - 1./162.*m33*m44
         *RSe564p*RSp561e*RSe563p - 1./162.*m33*m44*RSe564p*RSp561e*
         RSe561p + 1./81.*m13*m66*RSe456p*RSp453e*RSe453p - 1./81.*m13*
         m66*RSe456p*RSe451p*RSp453e - 1./81.*m13*m66*RSe456p*RSp451e*
         RSe453p + 1./81.*m13*m66*RSe456p*RSp451e*RSe451p + 1./81.*m13*
         m55*RSe465p*RSp463e*RSe463p - 1./81.*m13*m55*RSe465p*RSe461p*
         RSp463e - 1./81.*m13*m55*RSe465p*RSp461e*RSe463p + 1./81.*m13*
         m55*RSe465p*RSp461e*RSe461p + 1./81.*m13*m44*RSe564p*RSp563e*
         RSe563p - 1./81.*m13*m44*RSe564p*RSe561p*RSp563e - 1./81.*m13*
         m44*RSe564p*RSp561e*RSe563p + 1./81.*m13*m44*RSe564p*RSp561e*
         RSe561p - 1./162.*m11*m66*RSe456p*RSp453e*RSe453p - 1./162.*
         m11*m66*RSe456p*RSe451p*RSp453e - 1./162.*m11*m66*RSe456p*
         RSp451e*RSe453p - 1./54.*m11*m66*RSe456p*RSp451e*RSe451p - 1./
         162.*m11*m55*RSe465p*RSp463e*RSe463p - 1./162.*m11*m55*RSe465p
         *RSe461p*RSp463e - 1./162.*m11*m55*RSe465p*RSp461e*RSe463p - 1.
         /54.*m11*m55*RSe465p*RSp461e*RSe461p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(me2,mp2,L) * (  - 1./162.*m11*m44
         *RSe564p*RSp563e*RSe563p - 1./162.*m11*m44*RSe564p*RSe561p*
         RSp563e - 1./162.*m11*m44*RSe564p*RSp561e*RSe563p - 1./54.*m11
         *m44*RSe564p*RSp561e*RSe561p );

      M13p6x23RV +=  + AbV(me2,L)*BbV(me2,L) * ( 1./162.*m66*me2*
         RSe456p*pow(RSe453p,2) - 1./81.*m66*me2*RSe456p*RSe451p*
         RSe453p + 1./162.*m66*me2*RSe456p*pow(RSe451p,2) + 1./162.*m55
         *me2*RSe465p*pow(RSe463p,2) - 1./81.*m55*me2*RSe465p*RSe461p*
         RSe463p + 1./162.*m55*me2*RSe465p*pow(RSe461p,2) + 1./162.*m44
         *me2*RSe564p*pow(RSe563p,2) - 1./81.*m44*me2*RSe564p*RSe561p*
         RSe563p + 1./162.*m44*me2*RSe564p*pow(RSe561p,2) - 1./108.*m33
         *m66*RSe456p*pow(RSe453p,2) - 1./162.*m33*m66*RSe456p*RSe451p*
         RSe453p - 1./324.*m33*m66*RSe456p*pow(RSe451p,2) - 1./108.*m33
         *m55*RSe465p*pow(RSe463p,2) - 1./162.*m33*m55*RSe465p*RSe461p*
         RSe463p - 1./324.*m33*m55*RSe465p*pow(RSe461p,2) - 1./108.*m33
         *m44*RSe564p*pow(RSe563p,2) - 1./162.*m33*m44*RSe564p*RSe561p*
         RSe563p - 1./324.*m33*m44*RSe564p*pow(RSe561p,2) + 1./162.*m13
         *m66*RSe456p*pow(RSe453p,2) - 1./81.*m13*m66*RSe456p*RSe451p*
         RSe453p + 1./162.*m13*m66*RSe456p*pow(RSe451p,2) + 1./162.*m13
         *m55*RSe465p*pow(RSe463p,2) );

      M13p6x23RV +=  + AbV(me2,L)*BbV(me2,L) * (  - 1./81.*m13*m55*
         RSe465p*RSe461p*RSe463p + 1./162.*m13*m55*RSe465p*pow(
         RSe461p,2) + 1./162.*m13*m44*RSe564p*pow(RSe563p,2) - 1./81.*
         m13*m44*RSe564p*RSe561p*RSe563p + 1./162.*m13*m44*RSe564p*pow(
         RSe561p,2) - 1./324.*m11*m66*RSe456p*pow(RSe453p,2) - 1./162.*
         m11*m66*RSe456p*RSe451p*RSe453p - 1./108.*m11*m66*RSe456p*pow(
         RSe451p,2) - 1./324.*m11*m55*RSe465p*pow(RSe463p,2) - 1./162.*
         m11*m55*RSe465p*RSe461p*RSe463p - 1./108.*m11*m55*RSe465p*pow(
         RSe461p,2) - 1./324.*m11*m44*RSe564p*pow(RSe563p,2) - 1./162.*
         m11*m44*RSe564p*RSe561p*RSe563p - 1./108.*m11*m44*RSe564p*pow(
         RSe561p,2) );

      M13p6x23RV +=  + AbV(me2,L)*CbV(m11,L) * ( 1./54.*m11*m33*
         RSe45611p*RD1456ep - 1./27.*m11*m13*RSe45611p*RD1456ep + 1./54.
         *pow(m11,2)*RSe45611p*RD1456ep );

      M13p6x23RV +=  + AbV(me2,L)*CbV(m33,L) * ( 1./54.*pow(m33,2)*
         RSe45633p*RD3456ep - 1./27.*m13*m33*RSe45633p*RD3456ep + 1./54.
         *m11*m33*RSe45633p*RD3456ep );

      M13p6x23RV +=  + BbV(m11,m33,L)*BbV(m11,L) * ( 1./54.*m11*m33*
         RD1456ep*RS34561pe + 1./27.*m11*m13*RD1456ep*RS34561pe + 1./18.
         *pow(m11,2)*RD1456ep*RS34561pe );

      M13p6x23RV +=  + BbV(m11,m33,L)*BbV(m33,L) * ( 1./54.*pow(m33,2)*
         RD3456ep*RS14563pe + 1./27.*m13*m33*RD3456ep*RS14563pe + 1./18.
         *m11*m33*RD3456ep*RS14563pe );

      M13p6x23RV +=  + BbV(m11,mp2,L)*BbV(m11,L) * ( 1./54.*m11*m33*
         RD1456ep*RSp45613e + 1./54.*m11*m33*RSp45611e*RD1456ep + 1./27.
         *m11*m13*RD1456ep*RSp45613e - 1./27.*m11*m13*RSp45611e*
         RD1456ep + 1./18.*pow(m11,2)*RD1456ep*RSp45613e + 1./54.*pow(
         m11,2)*RSp45611e*RD1456ep );

      M13p6x23RV +=  + BbV(m11,me2,L)*BbV(m11,L) * ( 1./54.*m11*m33*
         RD1456ep*RSe45613p + 1./54.*m11*m33*RSe45611p*RD1456ep + 1./27.
         *m11*m13*RD1456ep*RSe45613p - 1./27.*m11*m13*RSe45611p*
         RD1456ep + 1./18.*pow(m11,2)*RD1456ep*RSe45613p + 1./54.*pow(
         m11,2)*RSe45611p*RD1456ep );

      M13p6x23RV +=  + BbV(m11,L)*pi16 * ( 1./72.*m13*RD1456ep*RD3456ep
          + 1./72.*m13*pow(RD1456ep,2) - 1./36.*m13*me2*RD1456ep*
         RSe45613p + 1./72.*m13*me2*RD1456ep*RSe45633p + 1./72.*m13*me2
         *RSe45611p*RD1456ep - 1./36.*m13*mp2*RD1456ep*RSp45613e + 1./
         72.*m13*mp2*RD1456ep*RSp45633e + 1./72.*m13*mp2*RSp45611e*
         RD1456ep - 1./24.*m13*m36*RD1456ep - 1./24.*m13*m35*RD1456ep
          - 1./24.*m13*m34*RD1456ep - 1./36.*m13*m33*RD1456ep*RS34561pe
          + 1./72.*m13*m33*RD1456ep*RC3456pe - 1./24.*m13*m16*RD1456ep
          - 1./24.*m13*m15*RD1456ep - 1./24.*m13*m14*RD1456ep + 1./24.*
         pow(m13,2)*RD1456ep - 1./54.*pow(m13,2)*RD1456ep*RS34561pe - 1.
         /54.*pow(m13,2)*RD1456ep*RS14563pe - 1./54.*pow(m13,2)*
         RD1456ep*RSe45613p - 1./54.*pow(m13,2)*RD1456ep*RSp45613e + 1./
         108.*pow(m13,2)*RD1456ep*RC3456pe + 1./108.*pow(m13,2)*
         RD1456ep*RSe45633p + 1./108.*pow(m13,2)*RD1456ep*RSp45633e + 1.
         /108.*pow(m13,2)*RD1456ep*RC1456pe + 1./108.*pow(m13,2)*
         RSe45611p*RD1456ep );

      M13p6x23RV +=  + BbV(m11,L)*pi16 * ( 1./108.*pow(m13,2)*RSp45611e
         *RD1456ep + 1./24.*m11*m13*RD1456ep - 1./36.*m11*m13*RD1456ep*
         RS14563pe + 1./72.*m11*m13*RD1456ep*RC1456pe );

      M13p6x23RV +=  + pow(BbV(m11,L),2) * ( 13./1620.*m33*pow(
         RD1456ep,2) - 7./3240.*m13*pow(RD1456ep,2) - 17./1620.*m11*
         pow(RD1456ep,2) + 1./54.*m11*m33*RD1456ep*RS14563pe + 1./54.*
         m11*m33*RD1456ep*RC1456pe + 1./27.*m11*m13*RD1456ep*RS14563pe
          - 1./27.*m11*m13*RD1456ep*RC1456pe + 1./18.*pow(m11,2)*
         RD1456ep*RS14563pe + 1./54.*pow(m11,2)*RD1456ep*RC1456pe );

      M13p6x23RV +=  + BbV(m11,L)*BbV(m14,L) * (  - 1./432.*pow(m44,2)*
         RD1456ep - 1./432.*m33*m44*RD1456ep + 1./108.*m14*m44*RD1456ep
          + 1./216.*m14*m33*RD1456ep - 1./108.*pow(m14,2)*RD1456ep + 1./
         216.*m13*m44*RD1456ep - 1./108.*m13*m14*RD1456ep - 1./144.*m11
         *m44*RD1456ep - 1./432.*m11*m33*RD1456ep + 1./72.*m11*m14*
         RD1456ep + 1./216.*m11*m13*RD1456ep - 1./216.*pow(m11,2)*
         RD1456ep );

      M13p6x23RV +=  + BbV(m11,L)*BbV(m15,L) * (  - 1./432.*pow(m55,2)*
         RD1456ep - 1./432.*m33*m55*RD1456ep + 1./108.*m15*m55*RD1456ep
          + 1./216.*m15*m33*RD1456ep - 1./108.*pow(m15,2)*RD1456ep + 1./
         216.*m13*m55*RD1456ep - 1./108.*m13*m15*RD1456ep - 1./144.*m11
         *m55*RD1456ep - 1./432.*m11*m33*RD1456ep + 1./72.*m11*m15*
         RD1456ep + 1./216.*m11*m13*RD1456ep - 1./216.*pow(m11,2)*
         RD1456ep );

      M13p6x23RV +=  + BbV(m11,L)*BbV(m16,L) * (  - 1./432.*pow(m66,2)*
         RD1456ep - 1./432.*m33*m66*RD1456ep + 1./108.*m16*m66*RD1456ep
          + 1./216.*m16*m33*RD1456ep - 1./108.*pow(m16,2)*RD1456ep + 1./
         216.*m13*m66*RD1456ep - 1./108.*m13*m16*RD1456ep - 1./144.*m11
         *m66*RD1456ep - 1./432.*m11*m33*RD1456ep + 1./72.*m11*m16*
         RD1456ep + 1./216.*m11*m13*RD1456ep - 1./216.*pow(m11,2)*
         RD1456ep );

      M13p6x23RV +=  + BbV(m11,L)*BbV(m33,L) * (  - 19./1620.*m33*
         RD1456ep*RD3456ep - 7./1620.*m13*RD1456ep*RD3456ep - 19./1620.
         *m11*RD1456ep*RD3456ep );

      M13p6x23RV +=  + BbV(m11,L)*CbV(m11,L) * ( 1./54.*m11*m33*pow(
         RD1456ep,2) - 1./27.*m11*m13*pow(RD1456ep,2) + 1./54.*pow(
         m11,2)*pow(RD1456ep,2) );

      M13p6x23RV +=  + BbV(m33,mp2,L)*BbV(m33,L) * ( 1./18.*pow(m33,2)*
         RD3456ep*RSp45613e + 1./54.*pow(m33,2)*RSp45633e*RD3456ep + 1./
         27.*m13*m33*RD3456ep*RSp45613e - 1./27.*m13*m33*RSp45633e*
         RD3456ep + 1./54.*m11*m33*RD3456ep*RSp45613e + 1./54.*m11*m33*
         RSp45633e*RD3456ep );

      M13p6x23RV +=  + BbV(m33,me2,L)*BbV(m33,L) * ( 1./18.*pow(m33,2)*
         RD3456ep*RSe45613p + 1./54.*pow(m33,2)*RSe45633p*RD3456ep + 1./
         27.*m13*m33*RD3456ep*RSe45613p - 1./27.*m13*m33*RSe45633p*
         RD3456ep + 1./54.*m11*m33*RD3456ep*RSe45613p + 1./54.*m11*m33*
         RSe45633p*RD3456ep );

      M13p6x23RV +=  + BbV(m33,L)*pi16 * ( 1./72.*m13*pow(RD3456ep,2)
          + 1./72.*m13*RD1456ep*RD3456ep - 1./36.*m13*me2*RD3456ep*
         RSe45613p + 1./72.*m13*me2*RSe45633p*RD3456ep + 1./72.*m13*me2
         *RSe45611p*RD3456ep - 1./36.*m13*mp2*RD3456ep*RSp45613e + 1./
         72.*m13*mp2*RSp45633e*RD3456ep + 1./72.*m13*mp2*RSp45611e*
         RD3456ep - 1./24.*m13*m36*RD3456ep - 1./24.*m13*m35*RD3456ep
          - 1./24.*m13*m34*RD3456ep + 1./24.*m13*m33*RD3456ep - 1./36.*
         m13*m33*RD3456ep*RS34561pe + 1./72.*m13*m33*RD3456ep*RC3456pe
          - 1./24.*m13*m16*RD3456ep - 1./24.*m13*m15*RD3456ep - 1./24.*
         m13*m14*RD3456ep + 1./24.*pow(m13,2)*RD3456ep - 1./54.*pow(
         m13,2)*RD3456ep*RS34561pe - 1./54.*pow(m13,2)*RD3456ep*
         RS14563pe - 1./54.*pow(m13,2)*RD3456ep*RSe45613p - 1./54.*pow(
         m13,2)*RD3456ep*RSp45613e + 1./108.*pow(m13,2)*RD3456ep*
         RC3456pe + 1./108.*pow(m13,2)*RSe45633p*RD3456ep + 1./108.*
         pow(m13,2)*RSp45633e*RD3456ep + 1./108.*pow(m13,2)*RC1456pe*
         RD3456ep );

      M13p6x23RV +=  + BbV(m33,L)*pi16 * ( 1./108.*pow(m13,2)*RSe45611p
         *RD3456ep + 1./108.*pow(m13,2)*RSp45611e*RD3456ep - 1./36.*m11
         *m13*RD3456ep*RS14563pe + 1./72.*m11*m13*RC1456pe*RD3456ep );

      M13p6x23RV +=  + pow(BbV(m33,L),2) * (  - 17./1620.*m33*pow(
         RD3456ep,2) + 1./18.*pow(m33,2)*RD3456ep*RS34561pe + 1./54.*
         pow(m33,2)*RD3456ep*RC3456pe - 7./3240.*m13*pow(RD3456ep,2) + 
         1./27.*m13*m33*RD3456ep*RS34561pe - 1./27.*m13*m33*RD3456ep*
         RC3456pe + 13./1620.*m11*pow(RD3456ep,2) + 1./54.*m11*m33*
         RD3456ep*RS34561pe + 1./54.*m11*m33*RD3456ep*RC3456pe );

      M13p6x23RV +=  + BbV(m33,L)*BbV(m34,L) * (  - 1./432.*pow(m44,2)*
         RD3456ep + 1./108.*m34*m44*RD3456ep - 1./108.*pow(m34,2)*
         RD3456ep - 1./144.*m33*m44*RD3456ep + 1./72.*m33*m34*RD3456ep
          - 1./216.*pow(m33,2)*RD3456ep + 1./216.*m13*m44*RD3456ep - 1./
         108.*m13*m34*RD3456ep + 1./216.*m13*m33*RD3456ep - 1./432.*m11
         *m44*RD3456ep + 1./216.*m11*m34*RD3456ep - 1./432.*m11*m33*
         RD3456ep );

      M13p6x23RV +=  + BbV(m33,L)*BbV(m35,L) * (  - 1./432.*pow(m55,2)*
         RD3456ep + 1./108.*m35*m55*RD3456ep - 1./108.*pow(m35,2)*
         RD3456ep - 1./144.*m33*m55*RD3456ep + 1./72.*m33*m35*RD3456ep
          - 1./216.*pow(m33,2)*RD3456ep + 1./216.*m13*m55*RD3456ep - 1./
         108.*m13*m35*RD3456ep + 1./216.*m13*m33*RD3456ep - 1./432.*m11
         *m55*RD3456ep + 1./216.*m11*m35*RD3456ep - 1./432.*m11*m33*
         RD3456ep );

      M13p6x23RV +=  + BbV(m33,L)*BbV(m36,L) * (  - 1./432.*pow(m66,2)*
         RD3456ep + 1./108.*m36*m66*RD3456ep - 1./108.*pow(m36,2)*
         RD3456ep - 1./144.*m33*m66*RD3456ep + 1./72.*m33*m36*RD3456ep
          - 1./216.*pow(m33,2)*RD3456ep + 1./216.*m13*m66*RD3456ep - 1./
         108.*m13*m36*RD3456ep + 1./216.*m13*m33*RD3456ep - 1./432.*m11
         *m66*RD3456ep + 1./216.*m11*m36*RD3456ep - 1./432.*m11*m33*
         RD3456ep );

      M13p6x23RV +=  + BbV(m33,L)*CbV(m33,L) * ( 1./54.*pow(m33,2)*pow(
         RD3456ep,2) - 1./27.*m13*m33*pow(RD3456ep,2) + 1./54.*m11*m33*
         pow(RD3456ep,2) );

      M13p6x23RV +=  + A23bV(m11,L)*pi16 * ( 3./8.*m13 - 1./18.*m13*
         RS14563pe*RS34561pe - 1./18.*m13*pow(RS14563pe,2) - 1./18.*m13
         *RSe45613p*RS14563pe - 1./18.*m13*RSp45613e*RS14563pe + 1./36.
         *m13*RC3456pe*RS14563pe + 1./36.*m13*RSe45633p*RS14563pe + 1./
         36.*m13*RSp45633e*RS14563pe + 1./36.*m13*RC1456pe*RS34561pe + 
         1./18.*m13*RC1456pe*RS14563pe + 1./36.*m13*RC1456pe*RSe45613p
          + 1./36.*m13*RC1456pe*RSp45613e - 1./72.*m13*RC1456pe*
         RC3456pe - 1./72.*m13*RC1456pe*RSe45633p - 1./72.*m13*RC1456pe
         *RSp45633e - 1./72.*m13*pow(RC1456pe,2) + 1./36.*m13*RSe45611p
         *RS14563pe - 1./72.*m13*RSe45611p*RC1456pe + 1./36.*m13*
         RSp45611e*RS14563pe - 1./72.*m13*RSp45611e*RC1456pe - 1./8.*
         m13*RS145pe - 1./8.*m13*RS146pe - 1./8.*m13*RS156pe );

      M13p6x23RV +=  + A23bV(m13,L)*pi16 * (  - 1./12.*m13*pow(
         RS34561pe,2) - 1./6.*m13*RS14563pe*RS34561pe - 1./12.*m13*pow(
         RS14563pe,2) - 1./6.*m13*RSe45613p*RS34561pe - 1./6.*m13*
         RSe45613p*RS14563pe - 1./12.*m13*pow(RSe45613p,2) - 1./6.*m13*
         RSp45613e*RS34561pe - 1./6.*m13*RSp45613e*RS14563pe - 1./6.*
         m13*RSp45613e*RSe45613p - 1./12.*m13*pow(RSp45613e,2) + 1./12.
         *m13*RC3456pe*RS34561pe + 1./12.*m13*RC3456pe*RS14563pe + 1./
         12.*m13*RC3456pe*RSe45613p + 1./12.*m13*RC3456pe*RSp45613e - 1.
         /48.*m13*pow(RC3456pe,2) + 1./12.*m13*RSe45633p*RS34561pe + 1./
         12.*m13*RSe45633p*RS14563pe + 1./12.*m13*RSe45633p*RSe45613p
          + 1./12.*m13*RSe45633p*RSp45613e - 1./24.*m13*RSe45633p*
         RC3456pe - 1./48.*m13*pow(RSe45633p,2) + 1./12.*m13*RSp45633e*
         RS34561pe + 1./12.*m13*RSp45633e*RS14563pe + 1./12.*m13*
         RSp45633e*RSe45613p + 1./12.*m13*RSp45633e*RSp45613e - 1./24.*
         m13*RSp45633e*RC3456pe - 1./24.*m13*RSp45633e*RSe45633p - 1./
         48.*m13*pow(RSp45633e,2) );

      M13p6x23RV +=  + A23bV(m13,L)*pi16 * ( 1./12.*m13*RC1456pe*
         RS34561pe + 1./12.*m13*RC1456pe*RS14563pe + 1./12.*m13*
         RC1456pe*RSe45613p + 1./12.*m13*RC1456pe*RSp45613e - 1./24.*
         m13*RC1456pe*RC3456pe - 1./24.*m13*RC1456pe*RSe45633p - 1./24.
         *m13*RC1456pe*RSp45633e - 1./48.*m13*pow(RC1456pe,2) + 1./12.*
         m13*RSe45611p*RS34561pe + 1./12.*m13*RSe45611p*RS14563pe + 1./
         12.*m13*RSe45611p*RSe45613p + 1./12.*m13*RSe45611p*RSp45613e
          - 1./24.*m13*RSe45611p*RC3456pe - 1./24.*m13*RSe45611p*
         RSe45633p - 1./24.*m13*RSe45611p*RSp45633e - 1./24.*m13*
         RSe45611p*RC1456pe - 1./48.*m13*pow(RSe45611p,2) + 1./12.*m13*
         RSp45611e*RS34561pe + 1./12.*m13*RSp45611e*RS14563pe + 1./12.*
         m13*RSp45611e*RSe45613p + 1./12.*m13*RSp45611e*RSp45613e - 1./
         24.*m13*RSp45611e*RC3456pe - 1./24.*m13*RSp45611e*RSe45633p - 
         1./24.*m13*RSp45611e*RSp45633e - 1./24.*m13*RSp45611e*RC1456pe
          - 1./24.*m13*RSp45611e*RSe45611p - 1./48.*m13*pow(
         RSp45611e,2) );

      M13p6x23RV +=  + A23bV(m14,L)*pi16 * (  - 1./8.*m13*RS34561pe - 1.
         /8.*m13*RS14563pe - 1./8.*m13*RSe45613p - 1./8.*m13*RSp45613e
          + 1./24.*m13*RC3456pe + 1./24.*m13*RSe45633p + 1./24.*m13*
         RSp45633e + 1./24.*m13*RS356pe + 1./24.*m13*RSe563p + 1./24.*
         m13*RSp563e + 1./8.*m13*RC1456pe + 1./8.*m13*RSe45611p + 1./8.
         *m13*RSp45611e - 1./8.*m13*RS156pe + 1./24.*m13*RS456pe - 1./8.
         *m13*RSe561p - 1./8.*m13*RSp561e + 1./24.*m13*RSe564p + 1./24.
         *m13*RSp564e );

      M13p6x23RV +=  + A23bV(m15,L)*pi16 * (  - 1./8.*m13*RS34561pe - 1.
         /8.*m13*RS14563pe - 1./8.*m13*RSe45613p - 1./8.*m13*RSp45613e
          + 1./24.*m13*RC3456pe + 1./24.*m13*RSe45633p + 1./24.*m13*
         RSp45633e + 1./24.*m13*RS346pe + 1./24.*m13*RSe463p + 1./24.*
         m13*RSp463e + 1./8.*m13*RC1456pe + 1./8.*m13*RSe45611p + 1./8.
         *m13*RSp45611e - 1./8.*m13*RS146pe + 1./24.*m13*RS546pe - 1./8.
         *m13*RSe461p - 1./8.*m13*RSp461e + 1./24.*m13*RSe465p + 1./24.
         *m13*RSp465e );

      M13p6x23RV +=  + A23bV(m16,L)*pi16 * (  - 1./8.*m13*RS34561pe - 1.
         /8.*m13*RS14563pe - 1./8.*m13*RSe45613p - 1./8.*m13*RSp45613e
          + 1./24.*m13*RC3456pe + 1./24.*m13*RSe45633p + 1./24.*m13*
         RSp45633e + 1./24.*m13*RS345pe + 1./24.*m13*RSe453p + 1./24.*
         m13*RSp453e + 1./8.*m13*RC1456pe + 1./8.*m13*RSe45611p + 1./8.
         *m13*RSp45611e - 1./8.*m13*RS145pe + 1./24.*m13*RS645pe - 1./8.
         *m13*RSe451p - 1./8.*m13*RSp451e + 1./24.*m13*RSe456p + 1./24.
         *m13*RSp456e );

      M13p6x23RV +=  + A23bV(m33,L)*pi16 * ( 3./8.*m13 - 1./18.*m13*
         pow(RS34561pe,2) - 1./18.*m13*RS14563pe*RS34561pe - 1./18.*m13
         *RSe45613p*RS34561pe - 1./18.*m13*RSp45613e*RS34561pe + 1./18.
         *m13*RC3456pe*RS34561pe + 1./36.*m13*RC3456pe*RS14563pe + 1./
         36.*m13*RC3456pe*RSe45613p + 1./36.*m13*RC3456pe*RSp45613e - 1.
         /72.*m13*pow(RC3456pe,2) + 1./36.*m13*RSe45633p*RS34561pe - 1./
         72.*m13*RSe45633p*RC3456pe + 1./36.*m13*RSp45633e*RS34561pe - 
         1./72.*m13*RSp45633e*RC3456pe - 1./8.*m13*RS345pe - 1./8.*m13*
         RS346pe - 1./8.*m13*RS356pe + 1./36.*m13*RC1456pe*RS34561pe - 
         1./72.*m13*RC1456pe*RC3456pe + 1./36.*m13*RSe45611p*RS34561pe
          - 1./72.*m13*RSe45611p*RC3456pe + 1./36.*m13*RSp45611e*
         RS34561pe - 1./72.*m13*RSp45611e*RC3456pe );

      M13p6x23RV +=  + A23bV(m34,L)*pi16 * (  - 1./8.*m13*RS34561pe - 1.
         /8.*m13*RS14563pe - 1./8.*m13*RSe45613p - 1./8.*m13*RSp45613e
          + 1./8.*m13*RC3456pe + 1./8.*m13*RSe45633p + 1./8.*m13*
         RSp45633e - 1./8.*m13*RS356pe - 1./8.*m13*RSe563p - 1./8.*m13*
         RSp563e + 1./24.*m13*RC1456pe + 1./24.*m13*RSe45611p + 1./24.*
         m13*RSp45611e + 1./24.*m13*RS156pe + 1./24.*m13*RS456pe + 1./
         24.*m13*RSe561p + 1./24.*m13*RSp561e + 1./24.*m13*RSe564p + 1./
         24.*m13*RSp564e );

      M13p6x23RV +=  + A23bV(m35,L)*pi16 * (  - 1./8.*m13*RS34561pe - 1.
         /8.*m13*RS14563pe - 1./8.*m13*RSe45613p - 1./8.*m13*RSp45613e
          + 1./8.*m13*RC3456pe + 1./8.*m13*RSe45633p + 1./8.*m13*
         RSp45633e - 1./8.*m13*RS346pe - 1./8.*m13*RSe463p - 1./8.*m13*
         RSp463e + 1./24.*m13*RC1456pe + 1./24.*m13*RSe45611p + 1./24.*
         m13*RSp45611e + 1./24.*m13*RS146pe + 1./24.*m13*RS546pe + 1./
         24.*m13*RSe461p + 1./24.*m13*RSp461e + 1./24.*m13*RSe465p + 1./
         24.*m13*RSp465e );

      M13p6x23RV +=  + A23bV(m36,L)*pi16 * (  - 1./8.*m13*RS34561pe - 1.
         /8.*m13*RS14563pe - 1./8.*m13*RSe45613p - 1./8.*m13*RSp45613e
          + 1./8.*m13*RC3456pe + 1./8.*m13*RSe45633p + 1./8.*m13*
         RSp45633e - 1./8.*m13*RS345pe - 1./8.*m13*RSe453p - 1./8.*m13*
         RSp453e + 1./24.*m13*RC1456pe + 1./24.*m13*RSe45611p + 1./24.*
         m13*RSp45611e + 1./24.*m13*RS145pe + 1./24.*m13*RS645pe + 1./
         24.*m13*RSe451p + 1./24.*m13*RSp451e + 1./24.*m13*RSe456p + 1./
         24.*m13*RSp456e );

      M13p6x23RV +=  + A23bV(mp2,L)*pi16 * (  - 1./18.*m13*RSp45613e*
         RS34561pe - 1./18.*m13*RSp45613e*RS14563pe - 1./18.*m13*
         RSp45613e*RSe45613p - 1./18.*m13*pow(RSp45613e,2) + 1./36.*m13
         *RC3456pe*RSp45613e + 1./36.*m13*RSe45633p*RSp45613e + 1./36.*
         m13*RSp45633e*RS34561pe + 1./36.*m13*RSp45633e*RS14563pe + 1./
         36.*m13*RSp45633e*RSe45613p + 1./18.*m13*RSp45633e*RSp45613e
          - 1./72.*m13*RSp45633e*RC3456pe - 1./72.*m13*RSp45633e*
         RSe45633p - 1./72.*m13*pow(RSp45633e,2) - 1./8.*m13*RSp563e - 
         1./8.*m13*RSp463e - 1./8.*m13*RSp453e + 1./36.*m13*RC1456pe*
         RSp45613e - 1./72.*m13*RC1456pe*RSp45633e + 1./36.*m13*
         RSe45611p*RSp45613e - 1./72.*m13*RSe45611p*RSp45633e + 1./36.*
         m13*RSp45611e*RS34561pe + 1./36.*m13*RSp45611e*RS14563pe + 1./
         36.*m13*RSp45611e*RSe45613p + 1./18.*m13*RSp45611e*RSp45613e
          - 1./72.*m13*RSp45611e*RC3456pe - 1./72.*m13*RSp45611e*
         RSe45633p - 1./36.*m13*RSp45611e*RSp45633e - 1./72.*m13*
         RSp45611e*RC1456pe );

      M13p6x23RV +=  + A23bV(mp2,L)*pi16 * (  - 1./72.*m13*RSp45611e*
         RSe45611p - 1./72.*m13*pow(RSp45611e,2) - 1./8.*m13*RSp561e - 
         1./8.*m13*RSp461e - 1./8.*m13*RSp451e );

      M13p6x23RV +=  + A23bV(me2,L)*pi16 * (  - 1./18.*m13*RSe45613p*
         RS34561pe - 1./18.*m13*RSe45613p*RS14563pe - 1./18.*m13*pow(
         RSe45613p,2) - 1./18.*m13*RSp45613e*RSe45613p + 1./36.*m13*
         RC3456pe*RSe45613p + 1./36.*m13*RSe45633p*RS34561pe + 1./36.*
         m13*RSe45633p*RS14563pe + 1./18.*m13*RSe45633p*RSe45613p + 1./
         36.*m13*RSe45633p*RSp45613e - 1./72.*m13*RSe45633p*RC3456pe - 
         1./72.*m13*pow(RSe45633p,2) + 1./36.*m13*RSp45633e*RSe45613p
          - 1./72.*m13*RSp45633e*RSe45633p - 1./8.*m13*RSe563p - 1./8.*
         m13*RSe463p - 1./8.*m13*RSe453p + 1./36.*m13*RC1456pe*
         RSe45613p - 1./72.*m13*RC1456pe*RSe45633p + 1./36.*m13*
         RSe45611p*RS34561pe + 1./36.*m13*RSe45611p*RS14563pe + 1./18.*
         m13*RSe45611p*RSe45613p + 1./36.*m13*RSe45611p*RSp45613e - 1./
         72.*m13*RSe45611p*RC3456pe - 1./36.*m13*RSe45611p*RSe45633p - 
         1./72.*m13*RSe45611p*RSp45633e - 1./72.*m13*RSe45611p*RC1456pe
          - 1./72.*m13*pow(RSe45611p,2) + 1./36.*m13*RSp45611e*
         RSe45613p );

      M13p6x23RV +=  + A23bV(me2,L)*pi16 * (  - 1./72.*m13*RSp45611e*
         RSe45633p - 1./72.*m13*RSp45611e*RSe45611p - 1./8.*m13*RSe561p
          - 1./8.*m13*RSe461p - 1./8.*m13*RSe451p );

      M13p6x23RV +=  + B23bV(m11,L)*pi16 * ( 1./36.*m13*RD1456ep*
         RS34561pe + 1./36.*m13*RD1456ep*RS14563pe + 1./36.*m13*
         RD1456ep*RSe45613p + 1./36.*m13*RD1456ep*RSp45613e - 1./72.*
         m13*RD1456ep*RC3456pe - 1./72.*m13*RD1456ep*RSe45633p - 1./72.
         *m13*RD1456ep*RSp45633e - 1./72.*m13*RD1456ep*RC1456pe - 1./72.
         *m13*RSe45611p*RD1456ep - 1./72.*m13*RSp45611e*RD1456ep );

      M13p6x23RV +=  + B23bV(m33,L)*pi16 * ( 1./36.*m13*RD3456ep*
         RS34561pe + 1./36.*m13*RD3456ep*RS14563pe + 1./36.*m13*
         RD3456ep*RSe45613p + 1./36.*m13*RD3456ep*RSp45613e - 1./72.*
         m13*RD3456ep*RC3456pe - 1./72.*m13*RSe45633p*RD3456ep - 1./72.
         *m13*RSp45633e*RD3456ep - 1./72.*m13*RC1456pe*RD3456ep - 1./72.
         *m13*RSe45611p*RD3456ep - 1./72.*m13*RSp45611e*RD3456ep );

      M13p6x23RV +=  + hhV(1,m11,m11,m13,m13,L,mu2) * ( 1./324.*pow(
         RD1456ep,2) - 1./162.*m33*RD1456ep*RC1456pe + 1./144.*pow(
         m33,2) + 1./81.*pow(m33,2)*pow(RS14563pe,2) + 1./162.*pow(
         m33,2)*RC1456pe*RS14563pe + 1./648.*pow(m33,2)*pow(RC1456pe,2)
          - 2./81.*m13*RD1456ep*RS14563pe + 1./81.*m13*RD1456ep*
         RC1456pe - 1./36.*m13*m33 - 1./81.*m13*m33*pow(RS14563pe,2) - 
         1./162.*m13*m33*pow(RC1456pe,2) - 17./36.*pow(m13,2) - 16./81.
         *pow(m13,2)*pow(RS14563pe,2) + 16./81.*pow(m13,2)*RC1456pe*
         RS14563pe - 4./81.*pow(m13,2)*pow(RC1456pe,2) - 1./81.*m11*
         RD1456ep*RS14563pe - 1./162.*m11*RD1456ep*RC1456pe + 1./72.*
         m11*m33 + 2./81.*m11*m33*pow(RS14563pe,2) + 2./81.*m11*m33*
         RC1456pe*RS14563pe + 1./324.*m11*m33*pow(RC1456pe,2) - 5./18.*
         m11*m13 + 1./27.*m11*m13*pow(RS14563pe,2) - 2./81.*m11*m13*
         RC1456pe*RS14563pe - 1./162.*m11*m13*pow(RC1456pe,2) + 1./144.
         *pow(m11,2) + 2./81.*pow(m11,2)*pow(RS14563pe,2) + 1./54.*pow(
         m11,2)*RC1456pe*RS14563pe );

      M13p6x23RV +=  + hhV(1,m11,m11,m13,m13,L,mu2) * ( 1./648.*pow(
         m11,2)*pow(RC1456pe,2) );

      M13p6x23RV +=  + hhV(1,m11,m13,m33,m13,L,mu2) * ( 1./162.*
         RD1456ep*RD3456ep + 1./81.*m33*RC1456pe*RD3456ep - 1./81.*m33*
         RD1456ep*RS34561pe + 1./81.*m33*RD1456ep*RC3456pe + 2./81.*
         pow(m33,2)*RS14563pe*RS34561pe + 2./81.*pow(m33,2)*RC3456pe*
         RS14563pe + 1./81.*pow(m33,2)*RC1456pe*RC3456pe + 4./81.*m13*
         RD3456ep*RS14563pe - 2./81.*m13*RC1456pe*RD3456ep + 4./81.*m13
         *RD1456ep*RS34561pe - 2./81.*m13*RD1456ep*RC3456pe - 4./81.*
         m13*m33*RS14563pe*RS34561pe + 4./81.*m13*m33*RC1456pe*
         RS34561pe - 4./81.*m13*m33*RC1456pe*RC3456pe + 16./81.*pow(
         m13,2)*RS14563pe*RS34561pe - 8./81.*pow(m13,2)*RC3456pe*
         RS14563pe - 8./81.*pow(m13,2)*RC1456pe*RS34561pe + 4./81.*pow(
         m13,2)*RC1456pe*RC3456pe - 1./81.*m11*RD3456ep*RS14563pe + 1./
         81.*m11*RC1456pe*RD3456ep + 1./81.*m11*RD1456ep*RC3456pe + 2./
         27.*m11*m33*RS14563pe*RS34561pe + 2./81.*m11*m33*RC3456pe*
         RS14563pe + 2./81.*m11*m33*RC1456pe*RS34561pe + 2./81.*m11*m33
         *RC1456pe*RC3456pe );

      M13p6x23RV +=  + hhV(1,m11,m13,m33,m13,L,mu2) * (  - 4./81.*m11*
         m13*RS14563pe*RS34561pe + 4./81.*m11*m13*RC3456pe*RS14563pe - 
         4./81.*m11*m13*RC1456pe*RC3456pe + 2./81.*pow(m11,2)*RS14563pe
         *RS34561pe + 2./81.*pow(m11,2)*RC1456pe*RS34561pe + 1./81.*
         pow(m11,2)*RC1456pe*RC3456pe );

      M13p6x23RV +=  + hhV(1,m11,m13,mp2,m13,L,mu2) * (  - 1./81.*mp2*
         RD1456ep*RSp45613e + 1./162.*mp2*RD1456ep*RSp45633e + 1./162.*
         mp2*RSp45611e*RD1456ep + 1./81.*pow(mp2,2)*RSp45613e*RS14563pe
          - 1./162.*pow(mp2,2)*RSp45633e*RS14563pe - 1./162.*pow(mp2,2)
         *RC1456pe*RSp45613e + 1./324.*pow(mp2,2)*RC1456pe*RSp45633e - 
         1./162.*pow(mp2,2)*RSp45611e*RS14563pe + 1./324.*pow(mp2,2)*
         RSp45611e*RC1456pe + 1./162.*m33*RD1456ep*RSp45633e - 1./162.*
         m33*RSp45611e*RD1456ep - 1./81.*m33*mp2*RSp45613e*RS14563pe + 
         1./81.*m33*mp2*RSp45633e*RS14563pe + 1./162.*m33*mp2*RC1456pe*
         RSp45633e - 1./162.*m33*mp2*RSp45611e*RC1456pe + 2./81.*pow(
         m33,2)*RSp45613e*RS14563pe + 1./54.*pow(m33,2)*RSp45633e*
         RS14563pe + 1./162.*pow(m33,2)*RC1456pe*RSp45613e + 1./324.*
         pow(m33,2)*RC1456pe*RSp45633e + 1./162.*pow(m33,2)*RSp45611e*
         RS14563pe + 1./324.*pow(m33,2)*RSp45611e*RC1456pe + 4./81.*m13
         *RD1456ep*RSp45613e - 2./81.*m13*RD1456ep*RSp45633e - 2./81.*
         m13*RSp45611e*RD1456ep );

      M13p6x23RV +=  + hhV(1,m11,m13,mp2,m13,L,mu2) * (  - 8./81.*m13*
         mp2*RSp45613e*RS14563pe + 4./81.*m13*mp2*RSp45633e*RS14563pe
          + 4./81.*m13*mp2*RC1456pe*RSp45613e - 2./81.*m13*mp2*RC1456pe
         *RSp45633e + 4./81.*m13*mp2*RSp45611e*RS14563pe - 2./81.*m13*
         mp2*RSp45611e*RC1456pe + 4./81.*m13*m33*RSp45613e*RS14563pe - 
         4./81.*m13*m33*RSp45633e*RS14563pe - 2./81.*m13*m33*RC1456pe*
         RSp45633e + 2./81.*m13*m33*RSp45611e*RC1456pe + 16./81.*pow(
         m13,2)*RSp45613e*RS14563pe - 8./81.*pow(m13,2)*RSp45633e*
         RS14563pe - 8./81.*pow(m13,2)*RC1456pe*RSp45613e + 4./81.*pow(
         m13,2)*RC1456pe*RSp45633e - 8./81.*pow(m13,2)*RSp45611e*
         RS14563pe + 4./81.*pow(m13,2)*RSp45611e*RC1456pe + 1./81.*m11*
         RD1456ep*RSp45633e - 1./81.*m11*RSp45611e*RD1456ep + 1./81.*
         m11*mp2*RSp45613e*RS14563pe - 1./81.*m11*mp2*RSp45633e*
         RS14563pe + 1./81.*m11*mp2*RC1456pe*RSp45633e - 1./81.*m11*mp2
         *RSp45611e*RC1456pe + 5./81.*m11*m33*RSp45613e*RS14563pe + 1./
         27.*m11*m33*RSp45633e*RS14563pe );

      M13p6x23RV +=  + hhV(1,m11,m13,mp2,m13,L,mu2) * ( 2./81.*m11*m33*
         RC1456pe*RSp45613e + 1./81.*m11*m33*RC1456pe*RSp45633e + 2./81.
         *m11*m33*RSp45611e*RS14563pe + 1./81.*m11*m33*RSp45611e*
         RC1456pe - 4./81.*m11*m13*RSp45613e*RS14563pe + 4./81.*m11*m13
         *RSp45633e*RS14563pe - 4./81.*m11*m13*RC1456pe*RSp45633e + 4./
         81.*m11*m13*RSp45611e*RC1456pe + 2./81.*pow(m11,2)*RSp45613e*
         RS14563pe + 2./81.*pow(m11,2)*RC1456pe*RSp45613e + 1./81.*pow(
         m11,2)*RC1456pe*RSp45633e + 2./81.*pow(m11,2)*RSp45611e*
         RS14563pe + 1./81.*pow(m11,2)*RSp45611e*RC1456pe );

      M13p6x23RV +=  + hhV(1,m11,m13,me2,m13,L,mu2) * (  - 1./81.*me2*
         RD1456ep*RSe45613p + 1./162.*me2*RD1456ep*RSe45633p + 1./162.*
         me2*RSe45611p*RD1456ep + 1./81.*pow(me2,2)*RSe45613p*RS14563pe
          - 1./162.*pow(me2,2)*RSe45633p*RS14563pe - 1./162.*pow(me2,2)
         *RC1456pe*RSe45613p + 1./324.*pow(me2,2)*RC1456pe*RSe45633p - 
         1./162.*pow(me2,2)*RSe45611p*RS14563pe + 1./324.*pow(me2,2)*
         RSe45611p*RC1456pe + 1./162.*m33*RD1456ep*RSe45633p - 1./162.*
         m33*RSe45611p*RD1456ep - 1./81.*m33*me2*RSe45613p*RS14563pe + 
         1./81.*m33*me2*RSe45633p*RS14563pe + 1./162.*m33*me2*RC1456pe*
         RSe45633p - 1./162.*m33*me2*RSe45611p*RC1456pe + 2./81.*pow(
         m33,2)*RSe45613p*RS14563pe + 1./54.*pow(m33,2)*RSe45633p*
         RS14563pe + 1./162.*pow(m33,2)*RC1456pe*RSe45613p + 1./324.*
         pow(m33,2)*RC1456pe*RSe45633p + 1./162.*pow(m33,2)*RSe45611p*
         RS14563pe + 1./324.*pow(m33,2)*RSe45611p*RC1456pe + 4./81.*m13
         *RD1456ep*RSe45613p - 2./81.*m13*RD1456ep*RSe45633p - 2./81.*
         m13*RSe45611p*RD1456ep );

      M13p6x23RV +=  + hhV(1,m11,m13,me2,m13,L,mu2) * (  - 8./81.*m13*
         me2*RSe45613p*RS14563pe + 4./81.*m13*me2*RSe45633p*RS14563pe
          + 4./81.*m13*me2*RC1456pe*RSe45613p - 2./81.*m13*me2*RC1456pe
         *RSe45633p + 4./81.*m13*me2*RSe45611p*RS14563pe - 2./81.*m13*
         me2*RSe45611p*RC1456pe + 4./81.*m13*m33*RSe45613p*RS14563pe - 
         4./81.*m13*m33*RSe45633p*RS14563pe - 2./81.*m13*m33*RC1456pe*
         RSe45633p + 2./81.*m13*m33*RSe45611p*RC1456pe + 16./81.*pow(
         m13,2)*RSe45613p*RS14563pe - 8./81.*pow(m13,2)*RSe45633p*
         RS14563pe - 8./81.*pow(m13,2)*RC1456pe*RSe45613p + 4./81.*pow(
         m13,2)*RC1456pe*RSe45633p - 8./81.*pow(m13,2)*RSe45611p*
         RS14563pe + 4./81.*pow(m13,2)*RSe45611p*RC1456pe + 1./81.*m11*
         RD1456ep*RSe45633p - 1./81.*m11*RSe45611p*RD1456ep + 1./81.*
         m11*me2*RSe45613p*RS14563pe - 1./81.*m11*me2*RSe45633p*
         RS14563pe + 1./81.*m11*me2*RC1456pe*RSe45633p - 1./81.*m11*me2
         *RSe45611p*RC1456pe + 5./81.*m11*m33*RSe45613p*RS14563pe + 1./
         27.*m11*m33*RSe45633p*RS14563pe );

      M13p6x23RV +=  + hhV(1,m11,m13,me2,m13,L,mu2) * ( 2./81.*m11*m33*
         RC1456pe*RSe45613p + 1./81.*m11*m33*RC1456pe*RSe45633p + 2./81.
         *m11*m33*RSe45611p*RS14563pe + 1./81.*m11*m33*RSe45611p*
         RC1456pe - 4./81.*m11*m13*RSe45613p*RS14563pe + 4./81.*m11*m13
         *RSe45633p*RS14563pe - 4./81.*m11*m13*RC1456pe*RSe45633p + 4./
         81.*m11*m13*RSe45611p*RC1456pe + 2./81.*pow(m11,2)*RSe45613p*
         RS14563pe + 2./81.*pow(m11,2)*RC1456pe*RSe45613p + 1./81.*pow(
         m11,2)*RC1456pe*RSe45633p + 2./81.*pow(m11,2)*RSe45611p*
         RS14563pe + 1./81.*pow(m11,2)*RSe45611p*RC1456pe );

      M13p6x23RV +=  + hhV(1,m11,m14,m34,m13,L,mu2) * (  - 1./108.*m44*
         RD1456ep - 1./216.*pow(m44,2)*RS14563pe - 1./432.*pow(m44,2)*
         RC1456pe - 1./108.*pow(m44,2)*RS156pe - 1./54.*m34*RD1456ep + 
         1./108.*m34*m44*RS14563pe - 1./108.*m34*m44*RC1456pe - 1./36.*
         m34*m44*RS156pe + 1./27.*pow(m34,2)*RS14563pe - 1./108.*pow(
         m34,2)*RC1456pe - 1./54.*pow(m34,2)*RS156pe - 1./108.*m33*
         RD1456ep - 1./72.*m33*m44*RS14563pe - 1./216.*m33*m44*RC1456pe
          - 1./72.*m33*m44*RS156pe - 1./108.*m33*m34*RC1456pe - 1./54.*
         m33*m34*RS156pe - 1./108.*pow(m33,2)*RS14563pe - 1./432.*pow(
         m33,2)*RC1456pe - 1./216.*pow(m33,2)*RS156pe + 1./27.*m14*
         RD1456ep + 1./108.*m14*m44*RS14563pe + 1./54.*m14*m44*RC1456pe
          + 1./36.*m14*m44*RS156pe - 5./54.*m14*m34*RS14563pe + 1./27.*
         m14*m34*RC1456pe + 1./54.*m14*m34*RS156pe + 1./36.*m14*m33*
         RS14563pe + 1./54.*m14*m33*RC1456pe + 1./108.*m14*m33*RS156pe
          + 1./27.*pow(m14,2)*RS14563pe - 1./27.*pow(m14,2)*RC1456pe + 
         1./27.*pow(m14,2)*RS156pe );

      M13p6x23RV +=  + hhV(1,m11,m14,m34,m13,L,mu2) * ( 1./27.*m13*
         RD1456ep - 1./54.*m13*m44*RS14563pe + 1./54.*m13*m44*RC1456pe
          + 1./18.*m13*m44*RS156pe - 4./27.*m13*m34*RS14563pe + 1./27.*
         m13*m34*RC1456pe + 2./27.*m13*m34*RS156pe + 1./54.*m13*m33*
         RC1456pe + 1./27.*m13*m33*RS156pe + 5./27.*m13*m14*RS14563pe
          - 2./27.*m13*m14*RC1456pe - 1./27.*m13*m14*RS156pe - 5./27.*
         pow(m13,2)*RS14563pe - 1./27.*pow(m13,2)*RC1456pe + 7./27.*
         pow(m13,2)*RS156pe - 1./27.*m11*RD1456ep - 7./216.*m11*m44*
         RS14563pe - 1./54.*m11*m44*RC1456pe - 5./216.*m11*m44*RS156pe
          + 5./108.*m11*m34*RS14563pe - 1./27.*m11*m34*RC1456pe - 1./
         108.*m11*m34*RS156pe - 11./216.*m11*m33*RS14563pe - 1./54.*m11
         *m33*RC1456pe - 1./216.*m11*m33*RS156pe + 1./54.*m11*m14*
         RS14563pe + 2./27.*m11*m14*RC1456pe - 5./54.*m11*m14*RS156pe
          - 19./108.*m11*m13*RS14563pe + 2./27.*m11*m13*RC1456pe + 11./
         108.*m11*m13*RS156pe - 1./18.*pow(m11,2)*RS14563pe - 1./27.*
         pow(m11,2)*RC1456pe );

      M13p6x23RV +=  + hhV(1,m11,m14,m34,m13,L,mu2) * ( 1./18.*pow(
         m11,2)*RS156pe );

      M13p6x23RV +=  + hhV(1,m11,m15,m35,m13,L,mu2) * (  - 1./108.*m55*
         RD1456ep - 1./216.*pow(m55,2)*RS14563pe - 1./432.*pow(m55,2)*
         RC1456pe - 1./108.*pow(m55,2)*RS146pe - 1./54.*m35*RD1456ep + 
         1./108.*m35*m55*RS14563pe - 1./108.*m35*m55*RC1456pe - 1./36.*
         m35*m55*RS146pe + 1./27.*pow(m35,2)*RS14563pe - 1./108.*pow(
         m35,2)*RC1456pe - 1./54.*pow(m35,2)*RS146pe - 1./108.*m33*
         RD1456ep - 1./72.*m33*m55*RS14563pe - 1./216.*m33*m55*RC1456pe
          - 1./72.*m33*m55*RS146pe - 1./108.*m33*m35*RC1456pe - 1./54.*
         m33*m35*RS146pe - 1./108.*pow(m33,2)*RS14563pe - 1./432.*pow(
         m33,2)*RC1456pe - 1./216.*pow(m33,2)*RS146pe + 1./27.*m15*
         RD1456ep + 1./108.*m15*m55*RS14563pe + 1./54.*m15*m55*RC1456pe
          + 1./36.*m15*m55*RS146pe - 5./54.*m15*m35*RS14563pe + 1./27.*
         m15*m35*RC1456pe + 1./54.*m15*m35*RS146pe + 1./36.*m15*m33*
         RS14563pe + 1./54.*m15*m33*RC1456pe + 1./108.*m15*m33*RS146pe
          + 1./27.*pow(m15,2)*RS14563pe - 1./27.*pow(m15,2)*RC1456pe + 
         1./27.*pow(m15,2)*RS146pe );

      M13p6x23RV +=  + hhV(1,m11,m15,m35,m13,L,mu2) * ( 1./27.*m13*
         RD1456ep - 1./54.*m13*m55*RS14563pe + 1./54.*m13*m55*RC1456pe
          + 1./18.*m13*m55*RS146pe - 4./27.*m13*m35*RS14563pe + 1./27.*
         m13*m35*RC1456pe + 2./27.*m13*m35*RS146pe + 1./54.*m13*m33*
         RC1456pe + 1./27.*m13*m33*RS146pe + 5./27.*m13*m15*RS14563pe
          - 2./27.*m13*m15*RC1456pe - 1./27.*m13*m15*RS146pe - 5./27.*
         pow(m13,2)*RS14563pe - 1./27.*pow(m13,2)*RC1456pe + 7./27.*
         pow(m13,2)*RS146pe - 1./27.*m11*RD1456ep - 7./216.*m11*m55*
         RS14563pe - 1./54.*m11*m55*RC1456pe - 5./216.*m11*m55*RS146pe
          + 5./108.*m11*m35*RS14563pe - 1./27.*m11*m35*RC1456pe - 1./
         108.*m11*m35*RS146pe - 11./216.*m11*m33*RS14563pe - 1./54.*m11
         *m33*RC1456pe - 1./216.*m11*m33*RS146pe + 1./54.*m11*m15*
         RS14563pe + 2./27.*m11*m15*RC1456pe - 5./54.*m11*m15*RS146pe
          - 19./108.*m11*m13*RS14563pe + 2./27.*m11*m13*RC1456pe + 11./
         108.*m11*m13*RS146pe - 1./18.*pow(m11,2)*RS14563pe - 1./27.*
         pow(m11,2)*RC1456pe );

      M13p6x23RV +=  + hhV(1,m11,m15,m35,m13,L,mu2) * ( 1./18.*pow(
         m11,2)*RS146pe );

      M13p6x23RV +=  + hhV(1,m11,m16,m36,m13,L,mu2) * (  - 1./108.*m66*
         RD1456ep - 1./216.*pow(m66,2)*RS14563pe - 1./432.*pow(m66,2)*
         RC1456pe - 1./108.*pow(m66,2)*RS145pe - 1./54.*m36*RD1456ep + 
         1./108.*m36*m66*RS14563pe - 1./108.*m36*m66*RC1456pe - 1./36.*
         m36*m66*RS145pe + 1./27.*pow(m36,2)*RS14563pe - 1./108.*pow(
         m36,2)*RC1456pe - 1./54.*pow(m36,2)*RS145pe - 1./108.*m33*
         RD1456ep - 1./72.*m33*m66*RS14563pe - 1./216.*m33*m66*RC1456pe
          - 1./72.*m33*m66*RS145pe - 1./108.*m33*m36*RC1456pe - 1./54.*
         m33*m36*RS145pe - 1./108.*pow(m33,2)*RS14563pe - 1./432.*pow(
         m33,2)*RC1456pe - 1./216.*pow(m33,2)*RS145pe + 1./27.*m16*
         RD1456ep + 1./108.*m16*m66*RS14563pe + 1./54.*m16*m66*RC1456pe
          + 1./36.*m16*m66*RS145pe - 5./54.*m16*m36*RS14563pe + 1./27.*
         m16*m36*RC1456pe + 1./54.*m16*m36*RS145pe + 1./36.*m16*m33*
         RS14563pe + 1./54.*m16*m33*RC1456pe + 1./108.*m16*m33*RS145pe
          + 1./27.*pow(m16,2)*RS14563pe - 1./27.*pow(m16,2)*RC1456pe + 
         1./27.*pow(m16,2)*RS145pe );

      M13p6x23RV +=  + hhV(1,m11,m16,m36,m13,L,mu2) * ( 1./27.*m13*
         RD1456ep - 1./54.*m13*m66*RS14563pe + 1./54.*m13*m66*RC1456pe
          + 1./18.*m13*m66*RS145pe - 4./27.*m13*m36*RS14563pe + 1./27.*
         m13*m36*RC1456pe + 2./27.*m13*m36*RS145pe + 1./54.*m13*m33*
         RC1456pe + 1./27.*m13*m33*RS145pe + 5./27.*m13*m16*RS14563pe
          - 2./27.*m13*m16*RC1456pe - 1./27.*m13*m16*RS145pe - 5./27.*
         pow(m13,2)*RS14563pe - 1./27.*pow(m13,2)*RC1456pe + 7./27.*
         pow(m13,2)*RS145pe - 1./27.*m11*RD1456ep - 7./216.*m11*m66*
         RS14563pe - 1./54.*m11*m66*RC1456pe - 5./216.*m11*m66*RS145pe
          + 5./108.*m11*m36*RS14563pe - 1./27.*m11*m36*RC1456pe - 1./
         108.*m11*m36*RS145pe - 11./216.*m11*m33*RS14563pe - 1./54.*m11
         *m33*RC1456pe - 1./216.*m11*m33*RS145pe + 1./54.*m11*m16*
         RS14563pe + 2./27.*m11*m16*RC1456pe - 5./54.*m11*m16*RS145pe
          - 19./108.*m11*m13*RS14563pe + 2./27.*m11*m13*RC1456pe + 11./
         108.*m11*m13*RS145pe - 1./18.*pow(m11,2)*RS14563pe - 1./27.*
         pow(m11,2)*RC1456pe );

      M13p6x23RV +=  + hhV(1,m11,m16,m36,m13,L,mu2) * ( 1./18.*pow(
         m11,2)*RS145pe );

      M13p6x23RV +=  + hhV(1,m13,m11,m11,m13,L,mu2) * ( 1./8.*pow(
         m13,2) + 1./18.*pow(m13,2)*pow(RS14563pe,2) - 1./18.*pow(
         m13,2)*RC1456pe*RS14563pe + 1./72.*pow(m13,2)*pow(RC1456pe,2)
          );

      M13p6x23RV +=  + hhV(1,m13,m11,m33,m13,L,mu2) * ( 1./9.*pow(
         m13,2)*RS14563pe*RS34561pe - 1./18.*pow(m13,2)*RC3456pe*
         RS14563pe - 1./18.*pow(m13,2)*RC1456pe*RS34561pe + 1./36.*pow(
         m13,2)*RC1456pe*RC3456pe );

      M13p6x23RV +=  + hhV(1,m13,m11,mp2,m13,L,mu2) * ( 1./9.*pow(
         m13,2)*RSp45613e*RS14563pe - 1./18.*pow(m13,2)*RSp45633e*
         RS14563pe - 1./18.*pow(m13,2)*RC1456pe*RSp45613e + 1./36.*pow(
         m13,2)*RC1456pe*RSp45633e - 1./18.*pow(m13,2)*RSp45611e*
         RS14563pe + 1./36.*pow(m13,2)*RSp45611e*RC1456pe );

      M13p6x23RV +=  + hhV(1,m13,m11,me2,m13,L,mu2) * ( 1./9.*pow(
         m13,2)*RSe45613p*RS14563pe - 1./18.*pow(m13,2)*RSe45633p*
         RS14563pe - 1./18.*pow(m13,2)*RC1456pe*RSe45613p + 1./36.*pow(
         m13,2)*RC1456pe*RSe45633p - 1./18.*pow(m13,2)*RSe45611p*
         RS14563pe + 1./36.*pow(m13,2)*RSe45611p*RC1456pe );

      M13p6x23RV +=  + hhV(1,m13,m13,m13,m13,L,mu2) * ( 1./36.*pow(
         m33,2) + 5./36.*pow(m13,2) + 1./18.*m11*m33 + 1./36.*pow(
         m11,2) );

      M13p6x23RV +=  + hhV(1,m13,m33,m33,m13,L,mu2) * ( 1./324.*pow(
         RD3456ep,2) - 1./81.*m33*RD3456ep*RS34561pe - 1./162.*m33*
         RD3456ep*RC3456pe + 1./144.*pow(m33,2) + 2./81.*pow(m33,2)*
         pow(RS34561pe,2) + 1./54.*pow(m33,2)*RC3456pe*RS34561pe + 1./
         648.*pow(m33,2)*pow(RC3456pe,2) + 4./81.*m13*RD3456ep*
         RS34561pe - 2./81.*m13*RD3456ep*RC3456pe + 1./18.*m13*m33 - 2./
         27.*m13*m33*pow(RS34561pe,2) + 4./81.*m13*m33*RC3456pe*
         RS34561pe + 1./81.*m13*m33*pow(RC3456pe,2) + 35./72.*pow(
         m13,2) + 25./162.*pow(m13,2)*pow(RS34561pe,2) - 25./162.*pow(
         m13,2)*RC3456pe*RS34561pe + 25./648.*pow(m13,2)*pow(
         RC3456pe,2) - 1./162.*m11*RD3456ep*RC3456pe + 1./72.*m11*m33
          + 2./81.*m11*m33*pow(RS34561pe,2) + 2./81.*m11*m33*RC3456pe*
         RS34561pe + 1./324.*m11*m33*pow(RC3456pe,2) + 1./18.*m11*m13
          + 2./81.*m11*m13*pow(RS34561pe,2) + 1./81.*m11*m13*pow(
         RC3456pe,2) + 1./144.*pow(m11,2) + 1./81.*pow(m11,2)*pow(
         RS34561pe,2) );

      M13p6x23RV +=  + hhV(1,m13,m33,m33,m13,L,mu2) * ( 1./162.*pow(
         m11,2)*RC3456pe*RS34561pe + 1./648.*pow(m11,2)*pow(RC3456pe,2)
          );

      M13p6x23RV +=  + hhV(1,m13,m33,mp2,m13,L,mu2) * (  - 1./81.*mp2*
         RD3456ep*RSp45613e + 1./162.*mp2*RSp45633e*RD3456ep + 1./162.*
         mp2*RSp45611e*RD3456ep + 1./81.*pow(mp2,2)*RSp45613e*RS34561pe
          - 1./162.*pow(mp2,2)*RC3456pe*RSp45613e - 1./162.*pow(mp2,2)*
         RSp45633e*RS34561pe + 1./324.*pow(mp2,2)*RSp45633e*RC3456pe - 
         1./162.*pow(mp2,2)*RSp45611e*RS34561pe + 1./324.*pow(mp2,2)*
         RSp45611e*RC3456pe - 1./81.*m33*RSp45633e*RD3456ep + 1./81.*
         m33*RSp45611e*RD3456ep + 1./81.*m33*mp2*RSp45613e*RS34561pe - 
         1./81.*m33*mp2*RSp45633e*RC3456pe - 1./81.*m33*mp2*RSp45611e*
         RS34561pe + 1./81.*m33*mp2*RSp45611e*RC3456pe + 2./81.*pow(
         m33,2)*RSp45613e*RS34561pe + 2./81.*pow(m33,2)*RC3456pe*
         RSp45613e + 2./81.*pow(m33,2)*RSp45633e*RS34561pe + 1./81.*
         pow(m33,2)*RSp45633e*RC3456pe + 1./81.*pow(m33,2)*RSp45611e*
         RC3456pe + 4./81.*m13*RD3456ep*RSp45613e - 2./81.*m13*
         RSp45633e*RD3456ep - 2./81.*m13*RSp45611e*RD3456ep - 8./81.*
         m13*mp2*RSp45613e*RS34561pe );

      M13p6x23RV +=  + hhV(1,m13,m33,mp2,m13,L,mu2) * ( 4./81.*m13*mp2*
         RC3456pe*RSp45613e + 4./81.*m13*mp2*RSp45633e*RS34561pe - 2./
         81.*m13*mp2*RSp45633e*RC3456pe + 4./81.*m13*mp2*RSp45611e*
         RS34561pe - 2./81.*m13*mp2*RSp45611e*RC3456pe - 4./81.*m13*m33
         *RSp45613e*RS34561pe + 4./81.*m13*m33*RSp45633e*RC3456pe + 4./
         81.*m13*m33*RSp45611e*RS34561pe - 4./81.*m13*m33*RSp45611e*
         RC3456pe + 25./81.*pow(m13,2)*RSp45613e*RS34561pe - 25./162.*
         pow(m13,2)*RC3456pe*RSp45613e - 25./162.*pow(m13,2)*RSp45633e*
         RS34561pe + 25./324.*pow(m13,2)*RSp45633e*RC3456pe - 25./162.*
         pow(m13,2)*RSp45611e*RS34561pe + 25./324.*pow(m13,2)*RSp45611e
         *RC3456pe - 1./162.*m11*RSp45633e*RD3456ep + 1./162.*m11*
         RSp45611e*RD3456ep - 1./81.*m11*mp2*RSp45613e*RS34561pe - 1./
         162.*m11*mp2*RSp45633e*RC3456pe + 1./81.*m11*mp2*RSp45611e*
         RS34561pe + 1./162.*m11*mp2*RSp45611e*RC3456pe + 5./81.*m11*
         m33*RSp45613e*RS34561pe + 2./81.*m11*m33*RC3456pe*RSp45613e + 
         2./81.*m11*m33*RSp45633e*RS34561pe );

      M13p6x23RV +=  + hhV(1,m13,m33,mp2,m13,L,mu2) * ( 1./81.*m11*m33*
         RSp45633e*RC3456pe + 1./27.*m11*m33*RSp45611e*RS34561pe + 1./
         81.*m11*m33*RSp45611e*RC3456pe + 4./81.*m11*m13*RSp45613e*
         RS34561pe + 2./81.*m11*m13*RSp45633e*RC3456pe - 4./81.*m11*m13
         *RSp45611e*RS34561pe - 2./81.*m11*m13*RSp45611e*RC3456pe + 2./
         81.*pow(m11,2)*RSp45613e*RS34561pe + 1./162.*pow(m11,2)*
         RC3456pe*RSp45613e + 1./162.*pow(m11,2)*RSp45633e*RS34561pe + 
         1./324.*pow(m11,2)*RSp45633e*RC3456pe + 1./54.*pow(m11,2)*
         RSp45611e*RS34561pe + 1./324.*pow(m11,2)*RSp45611e*RC3456pe );

      M13p6x23RV +=  + hhV(1,m13,m33,me2,m13,L,mu2) * (  - 1./81.*me2*
         RD3456ep*RSe45613p + 1./162.*me2*RSe45633p*RD3456ep + 1./162.*
         me2*RSe45611p*RD3456ep + 1./81.*pow(me2,2)*RSe45613p*RS34561pe
          - 1./162.*pow(me2,2)*RC3456pe*RSe45613p - 1./162.*pow(me2,2)*
         RSe45633p*RS34561pe + 1./324.*pow(me2,2)*RSe45633p*RC3456pe - 
         1./162.*pow(me2,2)*RSe45611p*RS34561pe + 1./324.*pow(me2,2)*
         RSe45611p*RC3456pe - 1./81.*m33*RSe45633p*RD3456ep + 1./81.*
         m33*RSe45611p*RD3456ep + 1./81.*m33*me2*RSe45613p*RS34561pe - 
         1./81.*m33*me2*RSe45633p*RC3456pe - 1./81.*m33*me2*RSe45611p*
         RS34561pe + 1./81.*m33*me2*RSe45611p*RC3456pe + 2./81.*pow(
         m33,2)*RSe45613p*RS34561pe + 2./81.*pow(m33,2)*RC3456pe*
         RSe45613p + 2./81.*pow(m33,2)*RSe45633p*RS34561pe + 1./81.*
         pow(m33,2)*RSe45633p*RC3456pe + 1./81.*pow(m33,2)*RSe45611p*
         RC3456pe + 4./81.*m13*RD3456ep*RSe45613p - 2./81.*m13*
         RSe45633p*RD3456ep - 2./81.*m13*RSe45611p*RD3456ep - 8./81.*
         m13*me2*RSe45613p*RS34561pe );

      M13p6x23RV +=  + hhV(1,m13,m33,me2,m13,L,mu2) * ( 4./81.*m13*me2*
         RC3456pe*RSe45613p + 4./81.*m13*me2*RSe45633p*RS34561pe - 2./
         81.*m13*me2*RSe45633p*RC3456pe + 4./81.*m13*me2*RSe45611p*
         RS34561pe - 2./81.*m13*me2*RSe45611p*RC3456pe - 4./81.*m13*m33
         *RSe45613p*RS34561pe + 4./81.*m13*m33*RSe45633p*RC3456pe + 4./
         81.*m13*m33*RSe45611p*RS34561pe - 4./81.*m13*m33*RSe45611p*
         RC3456pe + 25./81.*pow(m13,2)*RSe45613p*RS34561pe - 25./162.*
         pow(m13,2)*RC3456pe*RSe45613p - 25./162.*pow(m13,2)*RSe45633p*
         RS34561pe + 25./324.*pow(m13,2)*RSe45633p*RC3456pe - 25./162.*
         pow(m13,2)*RSe45611p*RS34561pe + 25./324.*pow(m13,2)*RSe45611p
         *RC3456pe - 1./162.*m11*RSe45633p*RD3456ep + 1./162.*m11*
         RSe45611p*RD3456ep - 1./81.*m11*me2*RSe45613p*RS34561pe - 1./
         162.*m11*me2*RSe45633p*RC3456pe + 1./81.*m11*me2*RSe45611p*
         RS34561pe + 1./162.*m11*me2*RSe45611p*RC3456pe + 5./81.*m11*
         m33*RSe45613p*RS34561pe + 2./81.*m11*m33*RC3456pe*RSe45613p + 
         2./81.*m11*m33*RSe45633p*RS34561pe );

      M13p6x23RV +=  + hhV(1,m13,m33,me2,m13,L,mu2) * ( 1./81.*m11*m33*
         RSe45633p*RC3456pe + 1./27.*m11*m33*RSe45611p*RS34561pe + 1./
         81.*m11*m33*RSe45611p*RC3456pe + 4./81.*m11*m13*RSe45613p*
         RS34561pe + 2./81.*m11*m13*RSe45633p*RC3456pe - 4./81.*m11*m13
         *RSe45611p*RS34561pe - 2./81.*m11*m13*RSe45611p*RC3456pe + 2./
         81.*pow(m11,2)*RSe45613p*RS34561pe + 1./162.*pow(m11,2)*
         RC3456pe*RSe45613p + 1./162.*pow(m11,2)*RSe45633p*RS34561pe + 
         1./324.*pow(m11,2)*RSe45633p*RC3456pe + 1./54.*pow(m11,2)*
         RSe45611p*RS34561pe + 1./324.*pow(m11,2)*RSe45611p*RC3456pe );

      M13p6x23RV +=  + hhV(1,m13,mp2,mp2,m13,L,mu2) * ( 2./81.*pow(
         mp2,2)*pow(RSp45613e,2) - 2./81.*pow(mp2,2)*RSp45633e*
         RSp45613e + 1./162.*pow(mp2,2)*pow(RSp45633e,2) - 2./81.*pow(
         mp2,2)*RSp45611e*RSp45613e + 1./81.*pow(mp2,2)*RSp45611e*
         RSp45633e + 1./162.*pow(mp2,2)*pow(RSp45611e,2) - 1./81.*m33*
         mp2*pow(RSp45613e,2) + 2./81.*m33*mp2*RSp45633e*RSp45613e - 1./
         54.*m33*mp2*pow(RSp45633e,2) + 1./81.*m33*mp2*RSp45611e*
         RSp45633e - 1./162.*m33*mp2*pow(RSp45611e,2) + 1./81.*pow(
         m33,2)*pow(RSp45613e,2) + 1./54.*pow(m33,2)*RSp45633e*
         RSp45613e + 1./72.*pow(m33,2)*pow(RSp45633e,2) + 1./162.*pow(
         m33,2)*RSp45611e*RSp45613e + 1./324.*pow(m33,2)*RSp45611e*
         RSp45633e + 1./648.*pow(m33,2)*pow(RSp45611e,2) - 8./81.*m13*
         mp2*pow(RSp45613e,2) + 8./81.*m13*mp2*RSp45633e*RSp45613e - 2./
         81.*m13*mp2*pow(RSp45633e,2) + 8./81.*m13*mp2*RSp45611e*
         RSp45613e - 4./81.*m13*mp2*RSp45611e*RSp45633e - 2./81.*m13*
         mp2*pow(RSp45611e,2) );

      M13p6x23RV +=  + hhV(1,m13,mp2,mp2,m13,L,mu2) * ( 2./81.*m13*m33*
         pow(RSp45613e,2) - 4./81.*m13*m33*RSp45633e*RSp45613e + 1./27.
         *m13*m33*pow(RSp45633e,2) - 2./81.*m13*m33*RSp45611e*RSp45633e
          + 1./81.*m13*m33*pow(RSp45611e,2) + 25./162.*pow(m13,2)*pow(
         RSp45613e,2) - 25./162.*pow(m13,2)*RSp45633e*RSp45613e + 25./
         648.*pow(m13,2)*pow(RSp45633e,2) - 25./162.*pow(m13,2)*
         RSp45611e*RSp45613e + 25./324.*pow(m13,2)*RSp45611e*RSp45633e
          + 25./648.*pow(m13,2)*pow(RSp45611e,2) - 1./81.*m11*mp2*pow(
         RSp45613e,2) - 1./162.*m11*mp2*pow(RSp45633e,2) + 2./81.*m11*
         mp2*RSp45611e*RSp45613e + 1./81.*m11*mp2*RSp45611e*RSp45633e
          - 1./54.*m11*mp2*pow(RSp45611e,2) + 1./27.*m11*m33*pow(
         RSp45613e,2) + 2./81.*m11*m33*RSp45633e*RSp45613e + 1./108.*
         m11*m33*pow(RSp45633e,2) + 2./81.*m11*m33*RSp45611e*RSp45613e
          + 1./162.*m11*m33*RSp45611e*RSp45633e + 1./108.*m11*m33*pow(
         RSp45611e,2) + 2./81.*m11*m13*pow(RSp45613e,2) + 1./81.*m11*
         m13*pow(RSp45633e,2) );

      M13p6x23RV +=  + hhV(1,m13,mp2,mp2,m13,L,mu2) * (  - 4./81.*m11*
         m13*RSp45611e*RSp45613e - 2./81.*m11*m13*RSp45611e*RSp45633e
          + 1./27.*m11*m13*pow(RSp45611e,2) + 1./81.*pow(m11,2)*pow(
         RSp45613e,2) + 1./162.*pow(m11,2)*RSp45633e*RSp45613e + 1./648.
         *pow(m11,2)*pow(RSp45633e,2) + 1./54.*pow(m11,2)*RSp45611e*
         RSp45613e + 1./324.*pow(m11,2)*RSp45611e*RSp45633e + 1./72.*
         pow(m11,2)*pow(RSp45611e,2) );

      M13p6x23RV +=  + hhV(1,m13,me2,mp2,m13,L,mu2) * ( 1./81.*pow(
         me2,2)*RSp45613e*RSe45613p - 1./162.*pow(me2,2)*RSe45633p*
         RSp45613e - 1./162.*pow(me2,2)*RSp45633e*RSe45613p + 1./324.*
         pow(me2,2)*RSp45633e*RSe45633p - 1./162.*pow(me2,2)*RSe45611p*
         RSp45613e + 1./324.*pow(me2,2)*RSe45611p*RSp45633e - 1./162.*
         pow(me2,2)*RSp45611e*RSe45613p + 1./324.*pow(me2,2)*RSp45611e*
         RSe45633p + 1./324.*pow(me2,2)*RSp45611e*RSe45611p + 2./81.*
         mp2*me2*RSp45613e*RSe45613p - 1./81.*mp2*me2*RSe45633p*
         RSp45613e - 1./81.*mp2*me2*RSp45633e*RSe45613p + 1./162.*mp2*
         me2*RSp45633e*RSe45633p - 1./81.*mp2*me2*RSe45611p*RSp45613e
          + 1./162.*mp2*me2*RSe45611p*RSp45633e - 1./81.*mp2*me2*
         RSp45611e*RSe45613p + 1./162.*mp2*me2*RSp45611e*RSe45633p + 1./
         162.*mp2*me2*RSp45611e*RSe45611p + 1./81.*pow(mp2,2)*RSp45613e
         *RSe45613p - 1./162.*pow(mp2,2)*RSe45633p*RSp45613e - 1./162.*
         pow(mp2,2)*RSp45633e*RSe45613p + 1./324.*pow(mp2,2)*RSp45633e*
         RSe45633p );

      M13p6x23RV +=  + hhV(1,m13,me2,mp2,m13,L,mu2) * (  - 1./162.*pow(
         mp2,2)*RSe45611p*RSp45613e + 1./324.*pow(mp2,2)*RSe45611p*
         RSp45633e - 1./162.*pow(mp2,2)*RSp45611e*RSe45613p + 1./324.*
         pow(mp2,2)*RSp45611e*RSe45633p + 1./324.*pow(mp2,2)*RSp45611e*
         RSe45611p - 1./81.*m33*me2*RSp45613e*RSe45613p + 1./81.*m33*
         me2*RSe45633p*RSp45613e + 1./81.*m33*me2*RSp45633e*RSe45613p
          - 1./54.*m33*me2*RSp45633e*RSe45633p + 1./162.*m33*me2*
         RSe45611p*RSp45633e + 1./162.*m33*me2*RSp45611e*RSe45633p - 1./
         162.*m33*me2*RSp45611e*RSe45611p - 1./81.*m33*mp2*RSp45613e*
         RSe45613p + 1./81.*m33*mp2*RSe45633p*RSp45613e + 1./81.*m33*
         mp2*RSp45633e*RSe45613p - 1./54.*m33*mp2*RSp45633e*RSe45633p
          + 1./162.*m33*mp2*RSe45611p*RSp45633e + 1./162.*m33*mp2*
         RSp45611e*RSe45633p - 1./162.*m33*mp2*RSp45611e*RSe45611p + 2./
         81.*pow(m33,2)*RSp45613e*RSe45613p + 1./54.*pow(m33,2)*
         RSe45633p*RSp45613e + 1./54.*pow(m33,2)*RSp45633e*RSe45613p + 
         1./36.*pow(m33,2)*RSp45633e*RSe45633p );

      M13p6x23RV +=  + hhV(1,m13,me2,mp2,m13,L,mu2) * ( 1./162.*pow(
         m33,2)*RSe45611p*RSp45613e + 1./324.*pow(m33,2)*RSe45611p*
         RSp45633e + 1./162.*pow(m33,2)*RSp45611e*RSe45613p + 1./324.*
         pow(m33,2)*RSp45611e*RSe45633p + 1./324.*pow(m33,2)*RSp45611e*
         RSe45611p - 8./81.*m13*me2*RSp45613e*RSe45613p + 4./81.*m13*
         me2*RSe45633p*RSp45613e + 4./81.*m13*me2*RSp45633e*RSe45613p
          - 2./81.*m13*me2*RSp45633e*RSe45633p + 4./81.*m13*me2*
         RSe45611p*RSp45613e - 2./81.*m13*me2*RSe45611p*RSp45633e + 4./
         81.*m13*me2*RSp45611e*RSe45613p - 2./81.*m13*me2*RSp45611e*
         RSe45633p - 2./81.*m13*me2*RSp45611e*RSe45611p - 8./81.*m13*
         mp2*RSp45613e*RSe45613p + 4./81.*m13*mp2*RSe45633p*RSp45613e
          + 4./81.*m13*mp2*RSp45633e*RSe45613p - 2./81.*m13*mp2*
         RSp45633e*RSe45633p + 4./81.*m13*mp2*RSe45611p*RSp45613e - 2./
         81.*m13*mp2*RSe45611p*RSp45633e + 4./81.*m13*mp2*RSp45611e*
         RSe45613p - 2./81.*m13*mp2*RSp45611e*RSe45633p - 2./81.*m13*
         mp2*RSp45611e*RSe45611p );

      M13p6x23RV +=  + hhV(1,m13,me2,mp2,m13,L,mu2) * ( 4./81.*m13*m33*
         RSp45613e*RSe45613p - 4./81.*m13*m33*RSe45633p*RSp45613e - 4./
         81.*m13*m33*RSp45633e*RSe45613p + 2./27.*m13*m33*RSp45633e*
         RSe45633p - 2./81.*m13*m33*RSe45611p*RSp45633e - 2./81.*m13*
         m33*RSp45611e*RSe45633p + 2./81.*m13*m33*RSp45611e*RSe45611p
          + 25./81.*pow(m13,2)*RSp45613e*RSe45613p - 25./162.*pow(
         m13,2)*RSe45633p*RSp45613e - 25./162.*pow(m13,2)*RSp45633e*
         RSe45613p + 25./324.*pow(m13,2)*RSp45633e*RSe45633p - 25./162.
         *pow(m13,2)*RSe45611p*RSp45613e + 25./324.*pow(m13,2)*
         RSe45611p*RSp45633e - 25./162.*pow(m13,2)*RSp45611e*RSe45613p
          + 25./324.*pow(m13,2)*RSp45611e*RSe45633p + 25./324.*pow(
         m13,2)*RSp45611e*RSe45611p - 1./81.*m11*me2*RSp45613e*
         RSe45613p - 1./162.*m11*me2*RSp45633e*RSe45633p + 1./81.*m11*
         me2*RSe45611p*RSp45613e + 1./162.*m11*me2*RSe45611p*RSp45633e
          + 1./81.*m11*me2*RSp45611e*RSe45613p + 1./162.*m11*me2*
         RSp45611e*RSe45633p );

      M13p6x23RV +=  + hhV(1,m13,me2,mp2,m13,L,mu2) * (  - 1./54.*m11*
         me2*RSp45611e*RSe45611p - 1./81.*m11*mp2*RSp45613e*RSe45613p
          - 1./162.*m11*mp2*RSp45633e*RSe45633p + 1./81.*m11*mp2*
         RSe45611p*RSp45613e + 1./162.*m11*mp2*RSe45611p*RSp45633e + 1./
         81.*m11*mp2*RSp45611e*RSe45613p + 1./162.*m11*mp2*RSp45611e*
         RSe45633p - 1./54.*m11*mp2*RSp45611e*RSe45611p + 2./27.*m11*
         m33*RSp45613e*RSe45613p + 2./81.*m11*m33*RSe45633p*RSp45613e
          + 2./81.*m11*m33*RSp45633e*RSe45613p + 1./54.*m11*m33*
         RSp45633e*RSe45633p + 2./81.*m11*m33*RSe45611p*RSp45613e + 1./
         162.*m11*m33*RSe45611p*RSp45633e + 2./81.*m11*m33*RSp45611e*
         RSe45613p + 1./162.*m11*m33*RSp45611e*RSe45633p + 1./54.*m11*
         m33*RSp45611e*RSe45611p + 4./81.*m11*m13*RSp45613e*RSe45613p
          + 2./81.*m11*m13*RSp45633e*RSe45633p - 4./81.*m11*m13*
         RSe45611p*RSp45613e - 2./81.*m11*m13*RSe45611p*RSp45633e - 4./
         81.*m11*m13*RSp45611e*RSe45613p - 2./81.*m11*m13*RSp45611e*
         RSe45633p );

      M13p6x23RV +=  + hhV(1,m13,me2,mp2,m13,L,mu2) * ( 2./27.*m11*m13*
         RSp45611e*RSe45611p + 2./81.*pow(m11,2)*RSp45613e*RSe45613p + 
         1./162.*pow(m11,2)*RSe45633p*RSp45613e + 1./162.*pow(m11,2)*
         RSp45633e*RSe45613p + 1./324.*pow(m11,2)*RSp45633e*RSe45633p
          + 1./54.*pow(m11,2)*RSe45611p*RSp45613e + 1./324.*pow(m11,2)*
         RSe45611p*RSp45633e + 1./54.*pow(m11,2)*RSp45611e*RSe45613p + 
         1./324.*pow(m11,2)*RSp45611e*RSe45633p + 1./36.*pow(m11,2)*
         RSp45611e*RSe45611p );

      M13p6x23RV +=  + hhV(1,m13,me2,me2,m13,L,mu2) * ( 2./81.*pow(
         me2,2)*pow(RSe45613p,2) - 2./81.*pow(me2,2)*RSe45633p*
         RSe45613p + 1./162.*pow(me2,2)*pow(RSe45633p,2) - 2./81.*pow(
         me2,2)*RSe45611p*RSe45613p + 1./81.*pow(me2,2)*RSe45611p*
         RSe45633p + 1./162.*pow(me2,2)*pow(RSe45611p,2) - 1./81.*m33*
         me2*pow(RSe45613p,2) + 2./81.*m33*me2*RSe45633p*RSe45613p - 1./
         54.*m33*me2*pow(RSe45633p,2) + 1./81.*m33*me2*RSe45611p*
         RSe45633p - 1./162.*m33*me2*pow(RSe45611p,2) + 1./81.*pow(
         m33,2)*pow(RSe45613p,2) + 1./54.*pow(m33,2)*RSe45633p*
         RSe45613p + 1./72.*pow(m33,2)*pow(RSe45633p,2) + 1./162.*pow(
         m33,2)*RSe45611p*RSe45613p + 1./324.*pow(m33,2)*RSe45611p*
         RSe45633p + 1./648.*pow(m33,2)*pow(RSe45611p,2) - 8./81.*m13*
         me2*pow(RSe45613p,2) + 8./81.*m13*me2*RSe45633p*RSe45613p - 2./
         81.*m13*me2*pow(RSe45633p,2) + 8./81.*m13*me2*RSe45611p*
         RSe45613p - 4./81.*m13*me2*RSe45611p*RSe45633p - 2./81.*m13*
         me2*pow(RSe45611p,2) );

      M13p6x23RV +=  + hhV(1,m13,me2,me2,m13,L,mu2) * ( 2./81.*m13*m33*
         pow(RSe45613p,2) - 4./81.*m13*m33*RSe45633p*RSe45613p + 1./27.
         *m13*m33*pow(RSe45633p,2) - 2./81.*m13*m33*RSe45611p*RSe45633p
          + 1./81.*m13*m33*pow(RSe45611p,2) + 25./162.*pow(m13,2)*pow(
         RSe45613p,2) - 25./162.*pow(m13,2)*RSe45633p*RSe45613p + 25./
         648.*pow(m13,2)*pow(RSe45633p,2) - 25./162.*pow(m13,2)*
         RSe45611p*RSe45613p + 25./324.*pow(m13,2)*RSe45611p*RSe45633p
          + 25./648.*pow(m13,2)*pow(RSe45611p,2) - 1./81.*m11*me2*pow(
         RSe45613p,2) - 1./162.*m11*me2*pow(RSe45633p,2) + 2./81.*m11*
         me2*RSe45611p*RSe45613p + 1./81.*m11*me2*RSe45611p*RSe45633p
          - 1./54.*m11*me2*pow(RSe45611p,2) + 1./27.*m11*m33*pow(
         RSe45613p,2) + 2./81.*m11*m33*RSe45633p*RSe45613p + 1./108.*
         m11*m33*pow(RSe45633p,2) + 2./81.*m11*m33*RSe45611p*RSe45613p
          + 1./162.*m11*m33*RSe45611p*RSe45633p + 1./108.*m11*m33*pow(
         RSe45611p,2) + 2./81.*m11*m13*pow(RSe45613p,2) + 1./81.*m11*
         m13*pow(RSe45633p,2) );

      M13p6x23RV +=  + hhV(1,m13,me2,me2,m13,L,mu2) * (  - 4./81.*m11*
         m13*RSe45611p*RSe45613p - 2./81.*m11*m13*RSe45611p*RSe45633p
          + 1./27.*m11*m13*pow(RSe45611p,2) + 1./81.*pow(m11,2)*pow(
         RSe45613p,2) + 1./162.*pow(m11,2)*RSe45633p*RSe45613p + 1./648.
         *pow(m11,2)*pow(RSe45633p,2) + 1./54.*pow(m11,2)*RSe45611p*
         RSe45613p + 1./324.*pow(m11,2)*RSe45611p*RSe45633p + 1./72.*
         pow(m11,2)*pow(RSe45611p,2) );

      M13p6x23RV +=  + hhV(1,m14,m11,m34,m13,L,mu2) * ( 1./12.*m13*m14*
         RS14563pe - 1./12.*m13*m14*RC1456pe + 1./12.*m13*m14*RS156pe )
         ;

      M13p6x23RV +=  + hhV(1,m14,m33,m34,m13,L,mu2) * (  - 1./108.*m44*
         RD3456ep - 1./216.*pow(m44,2)*RS34561pe - 1./432.*pow(m44,2)*
         RC3456pe - 1./108.*pow(m44,2)*RS356pe + 1./27.*m34*RD3456ep + 
         1./108.*m34*m44*RS34561pe + 1./54.*m34*m44*RC3456pe + 1./36.*
         m34*m44*RS356pe + 1./27.*pow(m34,2)*RS34561pe - 1./27.*pow(
         m34,2)*RC3456pe + 1./27.*pow(m34,2)*RS356pe - 1./27.*m33*
         RD3456ep - 7./216.*m33*m44*RS34561pe - 1./54.*m33*m44*RC3456pe
          - 5./216.*m33*m44*RS356pe + 1./54.*m33*m34*RS34561pe + 2./27.
         *m33*m34*RC3456pe - 5./54.*m33*m34*RS356pe - 1./18.*pow(m33,2)
         *RS34561pe - 1./27.*pow(m33,2)*RC3456pe + 1./18.*pow(m33,2)*
         RS356pe - 1./54.*m14*RD3456ep + 1./108.*m14*m44*RS34561pe - 1./
         108.*m14*m44*RC3456pe - 1./36.*m14*m44*RS356pe - 5./54.*m14*
         m34*RS34561pe + 1./27.*m14*m34*RC3456pe + 1./54.*m14*m34*
         RS356pe + 5./108.*m14*m33*RS34561pe - 1./27.*m14*m33*RC3456pe
          - 1./108.*m14*m33*RS356pe + 1./27.*pow(m14,2)*RS34561pe - 1./
         108.*pow(m14,2)*RC3456pe );

      M13p6x23RV +=  + hhV(1,m14,m33,m34,m13,L,mu2) * (  - 1./54.*pow(
         m14,2)*RS356pe - 2./27.*m13*RD3456ep - 1./54.*m13*m44*
         RS34561pe - 1./27.*m13*m44*RC3456pe - 1./18.*m13*m44*RS356pe
          - 4./27.*m13*m34*RS34561pe + 4./27.*m13*m34*RC3456pe - 4./27.
         *m13*m34*RS356pe - 1./27.*m13*m33*RS34561pe - 4./27.*m13*m33*
         RC3456pe + 5./27.*m13*m33*RS356pe + 29./108.*m13*m14*RS34561pe
          - 2./27.*m13*m14*RC3456pe - 13./108.*m13*m14*RS356pe - 5./27.
         *pow(m13,2)*RS34561pe + 5./27.*pow(m13,2)*RC3456pe - 5./27.*
         pow(m13,2)*RS356pe - 1./108.*m11*RD3456ep - 1./72.*m11*m44*
         RS34561pe - 1./216.*m11*m44*RC3456pe - 1./72.*m11*m44*RS356pe
          + 1./36.*m11*m34*RS34561pe + 1./54.*m11*m34*RC3456pe + 1./108.
         *m11*m34*RS356pe - 11./216.*m11*m33*RS34561pe - 1./54.*m11*m33
         *RC3456pe - 1./216.*m11*m33*RS356pe - 1./108.*m11*m14*RC3456pe
          - 1./54.*m11*m14*RS356pe - 1./18.*m11*m13*RS34561pe - 1./27.*
         m11*m13*RC3456pe - 1./54.*m11*m13*RS356pe - 1./108.*pow(m11,2)
         *RS34561pe );

      M13p6x23RV +=  + hhV(1,m14,m33,m34,m13,L,mu2) * (  - 1./432.*pow(
         m11,2)*RC3456pe - 1./216.*pow(m11,2)*RS356pe );

      M13p6x23RV +=  + hhV(1,m14,m34,m44,m13,L,mu2) * ( 1./36.*pow(
         m44,2) - 1./108.*pow(m44,2)*RS456pe - 1./18.*m34*m44 + 1./54.*
         m34*m44*RS456pe + 1./36.*pow(m34,2) - 1./108.*pow(m34,2)*
         RS456pe - 1./36.*m33*m44 + 1./108.*m33*m44*RS456pe + 1./36.*
         m33*m34 - 1./108.*m33*m34*RS456pe + 1./144.*pow(m33,2) - 1./
         432.*pow(m33,2)*RS456pe - 1./18.*m14*m44 + 1./54.*m14*m44*
         RS456pe + 1./18.*m14*m34 - 1./54.*m14*m34*RS456pe + 1./36.*m14
         *m33 - 1./108.*m14*m33*RS456pe + 1./36.*pow(m14,2) - 1./108.*
         pow(m14,2)*RS456pe - 2./9.*m13*m44 + 2./27.*m13*m44*RS456pe + 
         2./9.*m13*m34 - 2./27.*m13*m34*RS456pe + 1./9.*m13*m33 - 1./27.
         *m13*m33*RS456pe + 2./9.*m13*m14 - 2./27.*m13*m14*RS456pe - 5./
         9.*pow(m13,2) + 5./27.*pow(m13,2)*RS456pe - 1./36.*m11*m44 + 1.
         /108.*m11*m44*RS456pe + 1./36.*m11*m34 - 1./108.*m11*m34*
         RS456pe + 1./72.*m11*m33 - 1./216.*m11*m33*RS456pe + 1./36.*
         m11*m14 - 1./108.*m11*m14*RS456pe + 1./9.*m11*m13 - 1./27.*m11
         *m13*RS456pe );

      M13p6x23RV +=  + hhV(1,m14,m34,m44,m13,L,mu2) * ( 1./144.*pow(
         m11,2) - 1./432.*pow(m11,2)*RS456pe );

      M13p6x23RV +=  + hhV(1,m14,m34,mp2,m13,L,mu2) * (  - 1./54.*pow(
         mp2,2)*RSp45613e - 1./108.*pow(mp2,2)*RSp45633e + 1./27.*pow(
         mp2,2)*RSp563e - 1./108.*pow(mp2,2)*RSp45611e + 1./27.*pow(
         mp2,2)*RSp561e - 1./27.*pow(mp2,2)*RSp564e - 1./54.*m44*mp2*
         RSp45613e - 1./108.*m44*mp2*RSp45633e - 1./108.*m44*mp2*
         RSp45611e + 1./27.*m44*mp2*RSp564e - 1./216.*pow(m44,2)*
         RSp45613e - 1./432.*pow(m44,2)*RSp45633e - 1./108.*pow(m44,2)*
         RSp563e - 1./432.*pow(m44,2)*RSp45611e - 1./108.*pow(m44,2)*
         RSp561e - 1./108.*pow(m44,2)*RSp564e + 1./54.*m34*mp2*
         RSp45613e + 1./27.*m34*mp2*RSp45633e - 5./54.*m34*mp2*RSp563e
          - 1./54.*m34*mp2*RSp45611e + 1./54.*m34*mp2*RSp561e + 1./27.*
         m34*mp2*RSp564e + 1./108.*m34*m44*RSp45613e + 1./54.*m34*m44*
         RSp45633e + 1./36.*m34*m44*RSp563e - 1./108.*m34*m44*RSp45611e
          - 1./36.*m34*m44*RSp561e - 1./54.*m34*m44*RSp564e + 1./27.*
         pow(m34,2)*RSp45613e - 1./27.*pow(m34,2)*RSp45633e + 1./27.*
         pow(m34,2)*RSp563e );

      M13p6x23RV +=  + hhV(1,m14,m34,mp2,m13,L,mu2) * (  - 1./108.*pow(
         m34,2)*RSp45611e - 1./54.*pow(m34,2)*RSp561e - 1./108.*pow(
         m34,2)*RSp564e - 1./36.*m33*mp2*RSp45613e - 1./54.*m33*mp2*
         RSp45633e + 1./36.*m33*mp2*RSp563e - 1./108.*m33*mp2*RSp45611e
          + 1./108.*m33*mp2*RSp561e + 1./54.*m33*mp2*RSp564e - 1./72.*
         m33*m44*RSp45613e - 1./108.*m33*m44*RSp45633e - 5./216.*m33*
         m44*RSp563e - 1./216.*m33*m44*RSp45611e - 1./72.*m33*m44*
         RSp561e - 1./108.*m33*m44*RSp564e + 1./27.*m33*m34*RSp45633e
          - 1./108.*m33*m34*RSp45611e - 1./54.*m33*m34*RSp561e - 1./108.
         *m33*m34*RSp564e - 1./108.*pow(m33,2)*RSp45613e - 1./108.*pow(
         m33,2)*RSp45633e - 1./108.*pow(m33,2)*RSp563e - 1./432.*pow(
         m33,2)*RSp45611e - 1./216.*pow(m33,2)*RSp561e - 1./432.*pow(
         m33,2)*RSp564e + 1./54.*m14*mp2*RSp45613e - 1./54.*m14*mp2*
         RSp45633e + 1./54.*m14*mp2*RSp563e + 1./27.*m14*mp2*RSp45611e
          - 5./54.*m14*mp2*RSp561e + 1./27.*m14*mp2*RSp564e + 1./108.*
         m14*m44*RSp45613e );

      M13p6x23RV +=  + hhV(1,m14,m34,mp2,m13,L,mu2) * (  - 1./108.*m14*
         m44*RSp45633e - 1./36.*m14*m44*RSp563e + 1./54.*m14*m44*
         RSp45611e + 1./36.*m14*m44*RSp561e - 1./54.*m14*m44*RSp564e - 
         5./54.*m14*m34*RSp45613e + 1./27.*m14*m34*RSp45633e + 1./54.*
         m14*m34*RSp563e + 1./27.*m14*m34*RSp45611e + 1./54.*m14*m34*
         RSp561e - 1./54.*m14*m34*RSp564e + 1./36.*m14*m33*RSp45613e - 
         1./54.*m14*m33*RSp45633e - 1./36.*m14*m33*RSp563e + 1./54.*m14
         *m33*RSp45611e + 1./108.*m14*m33*RSp561e - 1./108.*m14*m33*
         RSp564e + 1./27.*pow(m14,2)*RSp45613e - 1./108.*pow(m14,2)*
         RSp45633e - 1./54.*pow(m14,2)*RSp563e - 1./27.*pow(m14,2)*
         RSp45611e + 1./27.*pow(m14,2)*RSp561e - 1./108.*pow(m14,2)*
         RSp564e + 2./27.*m13*mp2*RSp45613e + 1./27.*m13*mp2*RSp45633e
          - 4./27.*m13*mp2*RSp563e + 1./27.*m13*mp2*RSp45611e - 4./27.*
         m13*mp2*RSp561e + 4./27.*m13*mp2*RSp564e + 1./27.*m13*m44*
         RSp45613e + 1./54.*m13*m44*RSp45633e + 1./54.*m13*m44*
         RSp45611e );

      M13p6x23RV +=  + hhV(1,m14,m34,mp2,m13,L,mu2) * (  - 2./27.*m13*
         m44*RSp564e - 1./27.*m13*m34*RSp45613e - 2./27.*m13*m34*
         RSp45633e + 5./27.*m13*m34*RSp563e + 1./27.*m13*m34*RSp45611e
          - 1./27.*m13*m34*RSp561e - 2./27.*m13*m34*RSp564e + 1./18.*
         m13*m33*RSp45613e + 1./27.*m13*m33*RSp45633e - 1./18.*m13*m33*
         RSp563e + 1./54.*m13*m33*RSp45611e - 1./54.*m13*m33*RSp561e - 
         1./27.*m13*m33*RSp564e + 5./108.*m13*m14*RSp45613e + 1./27.*
         m13*m14*RSp45633e - 13./108.*m13*m14*RSp563e - 17./108.*m13*
         m14*RSp45611e + 29./108.*m13*m14*RSp561e - 2./27.*m13*m14*
         RSp564e + 7./27.*pow(m13,2)*RSp45613e - 1./27.*pow(m13,2)*
         RSp45633e - 5./27.*pow(m13,2)*RSp563e - 1./27.*pow(m13,2)*
         RSp45611e - 5./27.*pow(m13,2)*RSp561e + 5./27.*pow(m13,2)*
         RSp564e - 1./36.*m11*mp2*RSp45613e - 1./108.*m11*mp2*RSp45633e
          + 1./108.*m11*mp2*RSp563e - 1./54.*m11*mp2*RSp45611e + 1./36.
         *m11*mp2*RSp561e + 1./54.*m11*mp2*RSp564e - 1./72.*m11*m44*
         RSp45613e );

      M13p6x23RV +=  + hhV(1,m14,m34,mp2,m13,L,mu2) * (  - 1./216.*m11*
         m44*RSp45633e - 1./72.*m11*m44*RSp563e - 1./108.*m11*m44*
         RSp45611e - 5./216.*m11*m44*RSp561e - 1./108.*m11*m44*RSp564e
          + 1./36.*m11*m34*RSp45613e + 1./54.*m11*m34*RSp45633e + 1./
         108.*m11*m34*RSp563e - 1./54.*m11*m34*RSp45611e - 1./36.*m11*
         m34*RSp561e - 1./108.*m11*m34*RSp564e - 5./216.*m11*m33*
         RSp45613e - 1./108.*m11*m33*RSp45633e - 1./72.*m11*m33*RSp563e
          - 1./108.*m11*m33*RSp45611e - 1./72.*m11*m33*RSp561e - 1./216.
         *m11*m33*RSp564e - 1./108.*m11*m14*RSp45633e - 1./54.*m11*m14*
         RSp563e + 1./27.*m11*m14*RSp45611e - 1./108.*m11*m14*RSp564e
          + 1./18.*m11*m13*RSp45613e + 1./54.*m11*m13*RSp45633e - 1./54.
         *m11*m13*RSp563e + 1./27.*m11*m13*RSp45611e - 1./18.*m11*m13*
         RSp561e - 1./27.*m11*m13*RSp564e - 1./108.*pow(m11,2)*
         RSp45613e - 1./432.*pow(m11,2)*RSp45633e - 1./216.*pow(m11,2)*
         RSp563e - 1./108.*pow(m11,2)*RSp45611e - 1./108.*pow(m11,2)*
         RSp561e );

      M13p6x23RV +=  + hhV(1,m14,m34,mp2,m13,L,mu2) * (  - 1./432.*pow(
         m11,2)*RSp564e );

      M13p6x23RV +=  + hhV(1,m14,m34,me2,m13,L,mu2) * (  - 1./54.*pow(
         me2,2)*RSe45613p - 1./108.*pow(me2,2)*RSe45633p + 1./27.*pow(
         me2,2)*RSe563p - 1./108.*pow(me2,2)*RSe45611p + 1./27.*pow(
         me2,2)*RSe561p - 1./27.*pow(me2,2)*RSe564p - 1./54.*m44*me2*
         RSe45613p - 1./108.*m44*me2*RSe45633p - 1./108.*m44*me2*
         RSe45611p + 1./27.*m44*me2*RSe564p - 1./216.*pow(m44,2)*
         RSe45613p - 1./432.*pow(m44,2)*RSe45633p - 1./108.*pow(m44,2)*
         RSe563p - 1./432.*pow(m44,2)*RSe45611p - 1./108.*pow(m44,2)*
         RSe561p - 1./108.*pow(m44,2)*RSe564p + 1./54.*m34*me2*
         RSe45613p + 1./27.*m34*me2*RSe45633p - 5./54.*m34*me2*RSe563p
          - 1./54.*m34*me2*RSe45611p + 1./54.*m34*me2*RSe561p + 1./27.*
         m34*me2*RSe564p + 1./108.*m34*m44*RSe45613p + 1./54.*m34*m44*
         RSe45633p + 1./36.*m34*m44*RSe563p - 1./108.*m34*m44*RSe45611p
          - 1./36.*m34*m44*RSe561p - 1./54.*m34*m44*RSe564p + 1./27.*
         pow(m34,2)*RSe45613p - 1./27.*pow(m34,2)*RSe45633p + 1./27.*
         pow(m34,2)*RSe563p );

      M13p6x23RV +=  + hhV(1,m14,m34,me2,m13,L,mu2) * (  - 1./108.*pow(
         m34,2)*RSe45611p - 1./54.*pow(m34,2)*RSe561p - 1./108.*pow(
         m34,2)*RSe564p - 1./36.*m33*me2*RSe45613p - 1./54.*m33*me2*
         RSe45633p + 1./36.*m33*me2*RSe563p - 1./108.*m33*me2*RSe45611p
          + 1./108.*m33*me2*RSe561p + 1./54.*m33*me2*RSe564p - 1./72.*
         m33*m44*RSe45613p - 1./108.*m33*m44*RSe45633p - 5./216.*m33*
         m44*RSe563p - 1./216.*m33*m44*RSe45611p - 1./72.*m33*m44*
         RSe561p - 1./108.*m33*m44*RSe564p + 1./27.*m33*m34*RSe45633p
          - 1./108.*m33*m34*RSe45611p - 1./54.*m33*m34*RSe561p - 1./108.
         *m33*m34*RSe564p - 1./108.*pow(m33,2)*RSe45613p - 1./108.*pow(
         m33,2)*RSe45633p - 1./108.*pow(m33,2)*RSe563p - 1./432.*pow(
         m33,2)*RSe45611p - 1./216.*pow(m33,2)*RSe561p - 1./432.*pow(
         m33,2)*RSe564p + 1./54.*m14*me2*RSe45613p - 1./54.*m14*me2*
         RSe45633p + 1./54.*m14*me2*RSe563p + 1./27.*m14*me2*RSe45611p
          - 5./54.*m14*me2*RSe561p + 1./27.*m14*me2*RSe564p + 1./108.*
         m14*m44*RSe45613p );

      M13p6x23RV +=  + hhV(1,m14,m34,me2,m13,L,mu2) * (  - 1./108.*m14*
         m44*RSe45633p - 1./36.*m14*m44*RSe563p + 1./54.*m14*m44*
         RSe45611p + 1./36.*m14*m44*RSe561p - 1./54.*m14*m44*RSe564p - 
         5./54.*m14*m34*RSe45613p + 1./27.*m14*m34*RSe45633p + 1./54.*
         m14*m34*RSe563p + 1./27.*m14*m34*RSe45611p + 1./54.*m14*m34*
         RSe561p - 1./54.*m14*m34*RSe564p + 1./36.*m14*m33*RSe45613p - 
         1./54.*m14*m33*RSe45633p - 1./36.*m14*m33*RSe563p + 1./54.*m14
         *m33*RSe45611p + 1./108.*m14*m33*RSe561p - 1./108.*m14*m33*
         RSe564p + 1./27.*pow(m14,2)*RSe45613p - 1./108.*pow(m14,2)*
         RSe45633p - 1./54.*pow(m14,2)*RSe563p - 1./27.*pow(m14,2)*
         RSe45611p + 1./27.*pow(m14,2)*RSe561p - 1./108.*pow(m14,2)*
         RSe564p + 2./27.*m13*me2*RSe45613p + 1./27.*m13*me2*RSe45633p
          - 4./27.*m13*me2*RSe563p + 1./27.*m13*me2*RSe45611p - 4./27.*
         m13*me2*RSe561p + 4./27.*m13*me2*RSe564p + 1./27.*m13*m44*
         RSe45613p + 1./54.*m13*m44*RSe45633p + 1./54.*m13*m44*
         RSe45611p );

      M13p6x23RV +=  + hhV(1,m14,m34,me2,m13,L,mu2) * (  - 2./27.*m13*
         m44*RSe564p - 1./27.*m13*m34*RSe45613p - 2./27.*m13*m34*
         RSe45633p + 5./27.*m13*m34*RSe563p + 1./27.*m13*m34*RSe45611p
          - 1./27.*m13*m34*RSe561p - 2./27.*m13*m34*RSe564p + 1./18.*
         m13*m33*RSe45613p + 1./27.*m13*m33*RSe45633p - 1./18.*m13*m33*
         RSe563p + 1./54.*m13*m33*RSe45611p - 1./54.*m13*m33*RSe561p - 
         1./27.*m13*m33*RSe564p + 5./108.*m13*m14*RSe45613p + 1./27.*
         m13*m14*RSe45633p - 13./108.*m13*m14*RSe563p - 17./108.*m13*
         m14*RSe45611p + 29./108.*m13*m14*RSe561p - 2./27.*m13*m14*
         RSe564p + 7./27.*pow(m13,2)*RSe45613p - 1./27.*pow(m13,2)*
         RSe45633p - 5./27.*pow(m13,2)*RSe563p - 1./27.*pow(m13,2)*
         RSe45611p - 5./27.*pow(m13,2)*RSe561p + 5./27.*pow(m13,2)*
         RSe564p - 1./36.*m11*me2*RSe45613p - 1./108.*m11*me2*RSe45633p
          + 1./108.*m11*me2*RSe563p - 1./54.*m11*me2*RSe45611p + 1./36.
         *m11*me2*RSe561p + 1./54.*m11*me2*RSe564p - 1./72.*m11*m44*
         RSe45613p );

      M13p6x23RV +=  + hhV(1,m14,m34,me2,m13,L,mu2) * (  - 1./216.*m11*
         m44*RSe45633p - 1./72.*m11*m44*RSe563p - 1./108.*m11*m44*
         RSe45611p - 5./216.*m11*m44*RSe561p - 1./108.*m11*m44*RSe564p
          + 1./36.*m11*m34*RSe45613p + 1./54.*m11*m34*RSe45633p + 1./
         108.*m11*m34*RSe563p - 1./54.*m11*m34*RSe45611p - 1./36.*m11*
         m34*RSe561p - 1./108.*m11*m34*RSe564p - 5./216.*m11*m33*
         RSe45613p - 1./108.*m11*m33*RSe45633p - 1./72.*m11*m33*RSe563p
          - 1./108.*m11*m33*RSe45611p - 1./72.*m11*m33*RSe561p - 1./216.
         *m11*m33*RSe564p - 1./108.*m11*m14*RSe45633p - 1./54.*m11*m14*
         RSe563p + 1./27.*m11*m14*RSe45611p - 1./108.*m11*m14*RSe564p
          + 1./18.*m11*m13*RSe45613p + 1./54.*m11*m13*RSe45633p - 1./54.
         *m11*m13*RSe563p + 1./27.*m11*m13*RSe45611p - 1./18.*m11*m13*
         RSe561p - 1./27.*m11*m13*RSe564p - 1./108.*pow(m11,2)*
         RSe45613p - 1./432.*pow(m11,2)*RSe45633p - 1./216.*pow(m11,2)*
         RSe563p - 1./108.*pow(m11,2)*RSe45611p - 1./108.*pow(m11,2)*
         RSe561p );

      M13p6x23RV +=  + hhV(1,m14,m34,me2,m13,L,mu2) * (  - 1./432.*pow(
         m11,2)*RSe564p );

      M13p6x23RV +=  + hhV(1,m14,m35,m45,m13,L,mu2) * ( 1./144.*pow(
         m55,2) - 1./18.*m45*m55 + 1./9.*pow(m45,2) + 1./72.*m44*m55 - 
         1./18.*m44*m45 + 1./144.*pow(m44,2) + 1./36.*m35*m55 - 1./9.*
         m35*m45 + 1./36.*m35*m44 + 1./36.*pow(m35,2) + 1./72.*m33*m55
          - 1./18.*m33*m45 + 1./72.*m33*m44 + 1./36.*m33*m35 + 1./144.*
         pow(m33,2) + 1./36.*m14*m55 - 1./9.*m14*m45 + 1./36.*m14*m44
          + 1./18.*m14*m35 + 1./36.*m14*m33 + 1./36.*pow(m14,2) + 1./9.
         *m13*m55 - 4./9.*m13*m45 + 1./9.*m13*m44 + 2./9.*m13*m35 + 1./
         9.*m13*m33 + 2./9.*m13*m14 - 5./9.*pow(m13,2) + 1./72.*m11*m55
          - 1./18.*m11*m45 + 1./72.*m11*m44 + 1./36.*m11*m35 + 1./72.*
         m11*m33 + 1./36.*m11*m14 + 1./9.*m11*m13 + 1./144.*pow(m11,2)
          );

      M13p6x23RV +=  + hhV(1,m14,m36,m46,m13,L,mu2) * ( 1./144.*pow(
         m66,2) - 1./18.*m46*m66 + 1./9.*pow(m46,2) + 1./72.*m44*m66 - 
         1./18.*m44*m46 + 1./144.*pow(m44,2) + 1./36.*m36*m66 - 1./9.*
         m36*m46 + 1./36.*m36*m44 + 1./36.*pow(m36,2) + 1./72.*m33*m66
          - 1./18.*m33*m46 + 1./72.*m33*m44 + 1./36.*m33*m36 + 1./144.*
         pow(m33,2) + 1./36.*m14*m66 - 1./9.*m14*m46 + 1./36.*m14*m44
          + 1./18.*m14*m36 + 1./36.*m14*m33 + 1./36.*pow(m14,2) + 1./9.
         *m13*m66 - 4./9.*m13*m46 + 1./9.*m13*m44 + 2./9.*m13*m36 + 1./
         9.*m13*m33 + 2./9.*m13*m14 - 5./9.*pow(m13,2) + 1./72.*m11*m66
          - 1./18.*m11*m46 + 1./72.*m11*m44 + 1./36.*m11*m36 + 1./72.*
         m11*m33 + 1./36.*m11*m14 + 1./9.*m11*m13 + 1./144.*pow(m11,2)
          );

      M13p6x23RV +=  + hhV(1,m15,m11,m35,m13,L,mu2) * ( 1./12.*m13*m15*
         RS14563pe - 1./12.*m13*m15*RC1456pe + 1./12.*m13*m15*RS146pe )
         ;

      M13p6x23RV +=  + hhV(1,m15,m33,m35,m13,L,mu2) * (  - 1./108.*m55*
         RD3456ep - 1./216.*pow(m55,2)*RS34561pe - 1./432.*pow(m55,2)*
         RC3456pe - 1./108.*pow(m55,2)*RS346pe + 1./27.*m35*RD3456ep + 
         1./108.*m35*m55*RS34561pe + 1./54.*m35*m55*RC3456pe + 1./36.*
         m35*m55*RS346pe + 1./27.*pow(m35,2)*RS34561pe - 1./27.*pow(
         m35,2)*RC3456pe + 1./27.*pow(m35,2)*RS346pe - 1./27.*m33*
         RD3456ep - 7./216.*m33*m55*RS34561pe - 1./54.*m33*m55*RC3456pe
          - 5./216.*m33*m55*RS346pe + 1./54.*m33*m35*RS34561pe + 2./27.
         *m33*m35*RC3456pe - 5./54.*m33*m35*RS346pe - 1./18.*pow(m33,2)
         *RS34561pe - 1./27.*pow(m33,2)*RC3456pe + 1./18.*pow(m33,2)*
         RS346pe - 1./54.*m15*RD3456ep + 1./108.*m15*m55*RS34561pe - 1./
         108.*m15*m55*RC3456pe - 1./36.*m15*m55*RS346pe - 5./54.*m15*
         m35*RS34561pe + 1./27.*m15*m35*RC3456pe + 1./54.*m15*m35*
         RS346pe + 5./108.*m15*m33*RS34561pe - 1./27.*m15*m33*RC3456pe
          - 1./108.*m15*m33*RS346pe + 1./27.*pow(m15,2)*RS34561pe - 1./
         108.*pow(m15,2)*RC3456pe );

      M13p6x23RV +=  + hhV(1,m15,m33,m35,m13,L,mu2) * (  - 1./54.*pow(
         m15,2)*RS346pe - 2./27.*m13*RD3456ep - 1./54.*m13*m55*
         RS34561pe - 1./27.*m13*m55*RC3456pe - 1./18.*m13*m55*RS346pe
          - 4./27.*m13*m35*RS34561pe + 4./27.*m13*m35*RC3456pe - 4./27.
         *m13*m35*RS346pe - 1./27.*m13*m33*RS34561pe - 4./27.*m13*m33*
         RC3456pe + 5./27.*m13*m33*RS346pe + 29./108.*m13*m15*RS34561pe
          - 2./27.*m13*m15*RC3456pe - 13./108.*m13*m15*RS346pe - 5./27.
         *pow(m13,2)*RS34561pe + 5./27.*pow(m13,2)*RC3456pe - 5./27.*
         pow(m13,2)*RS346pe - 1./108.*m11*RD3456ep - 1./72.*m11*m55*
         RS34561pe - 1./216.*m11*m55*RC3456pe - 1./72.*m11*m55*RS346pe
          + 1./36.*m11*m35*RS34561pe + 1./54.*m11*m35*RC3456pe + 1./108.
         *m11*m35*RS346pe - 11./216.*m11*m33*RS34561pe - 1./54.*m11*m33
         *RC3456pe - 1./216.*m11*m33*RS346pe - 1./108.*m11*m15*RC3456pe
          - 1./54.*m11*m15*RS346pe - 1./18.*m11*m13*RS34561pe - 1./27.*
         m11*m13*RC3456pe - 1./54.*m11*m13*RS346pe - 1./108.*pow(m11,2)
         *RS34561pe );

      M13p6x23RV +=  + hhV(1,m15,m33,m35,m13,L,mu2) * (  - 1./432.*pow(
         m11,2)*RC3456pe - 1./216.*pow(m11,2)*RS346pe );

      M13p6x23RV +=  + hhV(1,m15,m34,m45,m13,L,mu2) * ( 1./144.*pow(
         m55,2) - 1./18.*m45*m55 + 1./9.*pow(m45,2) + 1./72.*m44*m55 - 
         1./18.*m44*m45 + 1./144.*pow(m44,2) + 1./36.*m34*m55 - 1./9.*
         m34*m45 + 1./36.*m34*m44 + 1./36.*pow(m34,2) + 1./72.*m33*m55
          - 1./18.*m33*m45 + 1./72.*m33*m44 + 1./36.*m33*m34 + 1./144.*
         pow(m33,2) + 1./36.*m15*m55 - 1./9.*m15*m45 + 1./36.*m15*m44
          + 1./18.*m15*m34 + 1./36.*m15*m33 + 1./36.*pow(m15,2) + 1./9.
         *m13*m55 - 4./9.*m13*m45 + 1./9.*m13*m44 + 2./9.*m13*m34 + 1./
         9.*m13*m33 + 2./9.*m13*m15 - 5./9.*pow(m13,2) + 1./72.*m11*m55
          - 1./18.*m11*m45 + 1./72.*m11*m44 + 1./36.*m11*m34 + 1./72.*
         m11*m33 + 1./36.*m11*m15 + 1./9.*m11*m13 + 1./144.*pow(m11,2)
          );

      M13p6x23RV +=  + hhV(1,m15,m35,m55,m13,L,mu2) * ( 1./36.*pow(
         m55,2) - 1./108.*pow(m55,2)*RS546pe - 1./18.*m35*m55 + 1./54.*
         m35*m55*RS546pe + 1./36.*pow(m35,2) - 1./108.*pow(m35,2)*
         RS546pe - 1./36.*m33*m55 + 1./108.*m33*m55*RS546pe + 1./36.*
         m33*m35 - 1./108.*m33*m35*RS546pe + 1./144.*pow(m33,2) - 1./
         432.*pow(m33,2)*RS546pe - 1./18.*m15*m55 + 1./54.*m15*m55*
         RS546pe + 1./18.*m15*m35 - 1./54.*m15*m35*RS546pe + 1./36.*m15
         *m33 - 1./108.*m15*m33*RS546pe + 1./36.*pow(m15,2) - 1./108.*
         pow(m15,2)*RS546pe - 2./9.*m13*m55 + 2./27.*m13*m55*RS546pe + 
         2./9.*m13*m35 - 2./27.*m13*m35*RS546pe + 1./9.*m13*m33 - 1./27.
         *m13*m33*RS546pe + 2./9.*m13*m15 - 2./27.*m13*m15*RS546pe - 5./
         9.*pow(m13,2) + 5./27.*pow(m13,2)*RS546pe - 1./36.*m11*m55 + 1.
         /108.*m11*m55*RS546pe + 1./36.*m11*m35 - 1./108.*m11*m35*
         RS546pe + 1./72.*m11*m33 - 1./216.*m11*m33*RS546pe + 1./36.*
         m11*m15 - 1./108.*m11*m15*RS546pe + 1./9.*m11*m13 - 1./27.*m11
         *m13*RS546pe );

      M13p6x23RV +=  + hhV(1,m15,m35,m55,m13,L,mu2) * ( 1./144.*pow(
         m11,2) - 1./432.*pow(m11,2)*RS546pe );

      M13p6x23RV +=  + hhV(1,m15,m35,mp2,m13,L,mu2) * (  - 1./54.*pow(
         mp2,2)*RSp45613e - 1./108.*pow(mp2,2)*RSp45633e + 1./27.*pow(
         mp2,2)*RSp463e - 1./108.*pow(mp2,2)*RSp45611e + 1./27.*pow(
         mp2,2)*RSp461e - 1./27.*pow(mp2,2)*RSp465e - 1./54.*m55*mp2*
         RSp45613e - 1./108.*m55*mp2*RSp45633e - 1./108.*m55*mp2*
         RSp45611e + 1./27.*m55*mp2*RSp465e - 1./216.*pow(m55,2)*
         RSp45613e - 1./432.*pow(m55,2)*RSp45633e - 1./108.*pow(m55,2)*
         RSp463e - 1./432.*pow(m55,2)*RSp45611e - 1./108.*pow(m55,2)*
         RSp461e - 1./108.*pow(m55,2)*RSp465e + 1./54.*m35*mp2*
         RSp45613e + 1./27.*m35*mp2*RSp45633e - 5./54.*m35*mp2*RSp463e
          - 1./54.*m35*mp2*RSp45611e + 1./54.*m35*mp2*RSp461e + 1./27.*
         m35*mp2*RSp465e + 1./108.*m35*m55*RSp45613e + 1./54.*m35*m55*
         RSp45633e + 1./36.*m35*m55*RSp463e - 1./108.*m35*m55*RSp45611e
          - 1./36.*m35*m55*RSp461e - 1./54.*m35*m55*RSp465e + 1./27.*
         pow(m35,2)*RSp45613e - 1./27.*pow(m35,2)*RSp45633e + 1./27.*
         pow(m35,2)*RSp463e );

      M13p6x23RV +=  + hhV(1,m15,m35,mp2,m13,L,mu2) * (  - 1./108.*pow(
         m35,2)*RSp45611e - 1./54.*pow(m35,2)*RSp461e - 1./108.*pow(
         m35,2)*RSp465e - 1./36.*m33*mp2*RSp45613e - 1./54.*m33*mp2*
         RSp45633e + 1./36.*m33*mp2*RSp463e - 1./108.*m33*mp2*RSp45611e
          + 1./108.*m33*mp2*RSp461e + 1./54.*m33*mp2*RSp465e - 1./72.*
         m33*m55*RSp45613e - 1./108.*m33*m55*RSp45633e - 5./216.*m33*
         m55*RSp463e - 1./216.*m33*m55*RSp45611e - 1./72.*m33*m55*
         RSp461e - 1./108.*m33*m55*RSp465e + 1./27.*m33*m35*RSp45633e
          - 1./108.*m33*m35*RSp45611e - 1./54.*m33*m35*RSp461e - 1./108.
         *m33*m35*RSp465e - 1./108.*pow(m33,2)*RSp45613e - 1./108.*pow(
         m33,2)*RSp45633e - 1./108.*pow(m33,2)*RSp463e - 1./432.*pow(
         m33,2)*RSp45611e - 1./216.*pow(m33,2)*RSp461e - 1./432.*pow(
         m33,2)*RSp465e + 1./54.*m15*mp2*RSp45613e - 1./54.*m15*mp2*
         RSp45633e + 1./54.*m15*mp2*RSp463e + 1./27.*m15*mp2*RSp45611e
          - 5./54.*m15*mp2*RSp461e + 1./27.*m15*mp2*RSp465e + 1./108.*
         m15*m55*RSp45613e );

      M13p6x23RV +=  + hhV(1,m15,m35,mp2,m13,L,mu2) * (  - 1./108.*m15*
         m55*RSp45633e - 1./36.*m15*m55*RSp463e + 1./54.*m15*m55*
         RSp45611e + 1./36.*m15*m55*RSp461e - 1./54.*m15*m55*RSp465e - 
         5./54.*m15*m35*RSp45613e + 1./27.*m15*m35*RSp45633e + 1./54.*
         m15*m35*RSp463e + 1./27.*m15*m35*RSp45611e + 1./54.*m15*m35*
         RSp461e - 1./54.*m15*m35*RSp465e + 1./36.*m15*m33*RSp45613e - 
         1./54.*m15*m33*RSp45633e - 1./36.*m15*m33*RSp463e + 1./54.*m15
         *m33*RSp45611e + 1./108.*m15*m33*RSp461e - 1./108.*m15*m33*
         RSp465e + 1./27.*pow(m15,2)*RSp45613e - 1./108.*pow(m15,2)*
         RSp45633e - 1./54.*pow(m15,2)*RSp463e - 1./27.*pow(m15,2)*
         RSp45611e + 1./27.*pow(m15,2)*RSp461e - 1./108.*pow(m15,2)*
         RSp465e + 2./27.*m13*mp2*RSp45613e + 1./27.*m13*mp2*RSp45633e
          - 4./27.*m13*mp2*RSp463e + 1./27.*m13*mp2*RSp45611e - 4./27.*
         m13*mp2*RSp461e + 4./27.*m13*mp2*RSp465e + 1./27.*m13*m55*
         RSp45613e + 1./54.*m13*m55*RSp45633e + 1./54.*m13*m55*
         RSp45611e );

      M13p6x23RV +=  + hhV(1,m15,m35,mp2,m13,L,mu2) * (  - 2./27.*m13*
         m55*RSp465e - 1./27.*m13*m35*RSp45613e - 2./27.*m13*m35*
         RSp45633e + 5./27.*m13*m35*RSp463e + 1./27.*m13*m35*RSp45611e
          - 1./27.*m13*m35*RSp461e - 2./27.*m13*m35*RSp465e + 1./18.*
         m13*m33*RSp45613e + 1./27.*m13*m33*RSp45633e - 1./18.*m13*m33*
         RSp463e + 1./54.*m13*m33*RSp45611e - 1./54.*m13*m33*RSp461e - 
         1./27.*m13*m33*RSp465e + 5./108.*m13*m15*RSp45613e + 1./27.*
         m13*m15*RSp45633e - 13./108.*m13*m15*RSp463e - 17./108.*m13*
         m15*RSp45611e + 29./108.*m13*m15*RSp461e - 2./27.*m13*m15*
         RSp465e + 7./27.*pow(m13,2)*RSp45613e - 1./27.*pow(m13,2)*
         RSp45633e - 5./27.*pow(m13,2)*RSp463e - 1./27.*pow(m13,2)*
         RSp45611e - 5./27.*pow(m13,2)*RSp461e + 5./27.*pow(m13,2)*
         RSp465e - 1./36.*m11*mp2*RSp45613e - 1./108.*m11*mp2*RSp45633e
          + 1./108.*m11*mp2*RSp463e - 1./54.*m11*mp2*RSp45611e + 1./36.
         *m11*mp2*RSp461e + 1./54.*m11*mp2*RSp465e - 1./72.*m11*m55*
         RSp45613e );

      M13p6x23RV +=  + hhV(1,m15,m35,mp2,m13,L,mu2) * (  - 1./216.*m11*
         m55*RSp45633e - 1./72.*m11*m55*RSp463e - 1./108.*m11*m55*
         RSp45611e - 5./216.*m11*m55*RSp461e - 1./108.*m11*m55*RSp465e
          + 1./36.*m11*m35*RSp45613e + 1./54.*m11*m35*RSp45633e + 1./
         108.*m11*m35*RSp463e - 1./54.*m11*m35*RSp45611e - 1./36.*m11*
         m35*RSp461e - 1./108.*m11*m35*RSp465e - 5./216.*m11*m33*
         RSp45613e - 1./108.*m11*m33*RSp45633e - 1./72.*m11*m33*RSp463e
          - 1./108.*m11*m33*RSp45611e - 1./72.*m11*m33*RSp461e - 1./216.
         *m11*m33*RSp465e - 1./108.*m11*m15*RSp45633e - 1./54.*m11*m15*
         RSp463e + 1./27.*m11*m15*RSp45611e - 1./108.*m11*m15*RSp465e
          + 1./18.*m11*m13*RSp45613e + 1./54.*m11*m13*RSp45633e - 1./54.
         *m11*m13*RSp463e + 1./27.*m11*m13*RSp45611e - 1./18.*m11*m13*
         RSp461e - 1./27.*m11*m13*RSp465e - 1./108.*pow(m11,2)*
         RSp45613e - 1./432.*pow(m11,2)*RSp45633e - 1./216.*pow(m11,2)*
         RSp463e - 1./108.*pow(m11,2)*RSp45611e - 1./108.*pow(m11,2)*
         RSp461e );

      M13p6x23RV +=  + hhV(1,m15,m35,mp2,m13,L,mu2) * (  - 1./432.*pow(
         m11,2)*RSp465e );

      M13p6x23RV +=  + hhV(1,m15,m35,me2,m13,L,mu2) * (  - 1./54.*pow(
         me2,2)*RSe45613p - 1./108.*pow(me2,2)*RSe45633p + 1./27.*pow(
         me2,2)*RSe463p - 1./108.*pow(me2,2)*RSe45611p + 1./27.*pow(
         me2,2)*RSe461p - 1./27.*pow(me2,2)*RSe465p - 1./54.*m55*me2*
         RSe45613p - 1./108.*m55*me2*RSe45633p - 1./108.*m55*me2*
         RSe45611p + 1./27.*m55*me2*RSe465p - 1./216.*pow(m55,2)*
         RSe45613p - 1./432.*pow(m55,2)*RSe45633p - 1./108.*pow(m55,2)*
         RSe463p - 1./432.*pow(m55,2)*RSe45611p - 1./108.*pow(m55,2)*
         RSe461p - 1./108.*pow(m55,2)*RSe465p + 1./54.*m35*me2*
         RSe45613p + 1./27.*m35*me2*RSe45633p - 5./54.*m35*me2*RSe463p
          - 1./54.*m35*me2*RSe45611p + 1./54.*m35*me2*RSe461p + 1./27.*
         m35*me2*RSe465p + 1./108.*m35*m55*RSe45613p + 1./54.*m35*m55*
         RSe45633p + 1./36.*m35*m55*RSe463p - 1./108.*m35*m55*RSe45611p
          - 1./36.*m35*m55*RSe461p - 1./54.*m35*m55*RSe465p + 1./27.*
         pow(m35,2)*RSe45613p - 1./27.*pow(m35,2)*RSe45633p + 1./27.*
         pow(m35,2)*RSe463p );

      M13p6x23RV +=  + hhV(1,m15,m35,me2,m13,L,mu2) * (  - 1./108.*pow(
         m35,2)*RSe45611p - 1./54.*pow(m35,2)*RSe461p - 1./108.*pow(
         m35,2)*RSe465p - 1./36.*m33*me2*RSe45613p - 1./54.*m33*me2*
         RSe45633p + 1./36.*m33*me2*RSe463p - 1./108.*m33*me2*RSe45611p
          + 1./108.*m33*me2*RSe461p + 1./54.*m33*me2*RSe465p - 1./72.*
         m33*m55*RSe45613p - 1./108.*m33*m55*RSe45633p - 5./216.*m33*
         m55*RSe463p - 1./216.*m33*m55*RSe45611p - 1./72.*m33*m55*
         RSe461p - 1./108.*m33*m55*RSe465p + 1./27.*m33*m35*RSe45633p
          - 1./108.*m33*m35*RSe45611p - 1./54.*m33*m35*RSe461p - 1./108.
         *m33*m35*RSe465p - 1./108.*pow(m33,2)*RSe45613p - 1./108.*pow(
         m33,2)*RSe45633p - 1./108.*pow(m33,2)*RSe463p - 1./432.*pow(
         m33,2)*RSe45611p - 1./216.*pow(m33,2)*RSe461p - 1./432.*pow(
         m33,2)*RSe465p + 1./54.*m15*me2*RSe45613p - 1./54.*m15*me2*
         RSe45633p + 1./54.*m15*me2*RSe463p + 1./27.*m15*me2*RSe45611p
          - 5./54.*m15*me2*RSe461p + 1./27.*m15*me2*RSe465p + 1./108.*
         m15*m55*RSe45613p );

      M13p6x23RV +=  + hhV(1,m15,m35,me2,m13,L,mu2) * (  - 1./108.*m15*
         m55*RSe45633p - 1./36.*m15*m55*RSe463p + 1./54.*m15*m55*
         RSe45611p + 1./36.*m15*m55*RSe461p - 1./54.*m15*m55*RSe465p - 
         5./54.*m15*m35*RSe45613p + 1./27.*m15*m35*RSe45633p + 1./54.*
         m15*m35*RSe463p + 1./27.*m15*m35*RSe45611p + 1./54.*m15*m35*
         RSe461p - 1./54.*m15*m35*RSe465p + 1./36.*m15*m33*RSe45613p - 
         1./54.*m15*m33*RSe45633p - 1./36.*m15*m33*RSe463p + 1./54.*m15
         *m33*RSe45611p + 1./108.*m15*m33*RSe461p - 1./108.*m15*m33*
         RSe465p + 1./27.*pow(m15,2)*RSe45613p - 1./108.*pow(m15,2)*
         RSe45633p - 1./54.*pow(m15,2)*RSe463p - 1./27.*pow(m15,2)*
         RSe45611p + 1./27.*pow(m15,2)*RSe461p - 1./108.*pow(m15,2)*
         RSe465p + 2./27.*m13*me2*RSe45613p + 1./27.*m13*me2*RSe45633p
          - 4./27.*m13*me2*RSe463p + 1./27.*m13*me2*RSe45611p - 4./27.*
         m13*me2*RSe461p + 4./27.*m13*me2*RSe465p + 1./27.*m13*m55*
         RSe45613p + 1./54.*m13*m55*RSe45633p + 1./54.*m13*m55*
         RSe45611p );

      M13p6x23RV +=  + hhV(1,m15,m35,me2,m13,L,mu2) * (  - 2./27.*m13*
         m55*RSe465p - 1./27.*m13*m35*RSe45613p - 2./27.*m13*m35*
         RSe45633p + 5./27.*m13*m35*RSe463p + 1./27.*m13*m35*RSe45611p
          - 1./27.*m13*m35*RSe461p - 2./27.*m13*m35*RSe465p + 1./18.*
         m13*m33*RSe45613p + 1./27.*m13*m33*RSe45633p - 1./18.*m13*m33*
         RSe463p + 1./54.*m13*m33*RSe45611p - 1./54.*m13*m33*RSe461p - 
         1./27.*m13*m33*RSe465p + 5./108.*m13*m15*RSe45613p + 1./27.*
         m13*m15*RSe45633p - 13./108.*m13*m15*RSe463p - 17./108.*m13*
         m15*RSe45611p + 29./108.*m13*m15*RSe461p - 2./27.*m13*m15*
         RSe465p + 7./27.*pow(m13,2)*RSe45613p - 1./27.*pow(m13,2)*
         RSe45633p - 5./27.*pow(m13,2)*RSe463p - 1./27.*pow(m13,2)*
         RSe45611p - 5./27.*pow(m13,2)*RSe461p + 5./27.*pow(m13,2)*
         RSe465p - 1./36.*m11*me2*RSe45613p - 1./108.*m11*me2*RSe45633p
          + 1./108.*m11*me2*RSe463p - 1./54.*m11*me2*RSe45611p + 1./36.
         *m11*me2*RSe461p + 1./54.*m11*me2*RSe465p - 1./72.*m11*m55*
         RSe45613p );

      M13p6x23RV +=  + hhV(1,m15,m35,me2,m13,L,mu2) * (  - 1./216.*m11*
         m55*RSe45633p - 1./72.*m11*m55*RSe463p - 1./108.*m11*m55*
         RSe45611p - 5./216.*m11*m55*RSe461p - 1./108.*m11*m55*RSe465p
          + 1./36.*m11*m35*RSe45613p + 1./54.*m11*m35*RSe45633p + 1./
         108.*m11*m35*RSe463p - 1./54.*m11*m35*RSe45611p - 1./36.*m11*
         m35*RSe461p - 1./108.*m11*m35*RSe465p - 5./216.*m11*m33*
         RSe45613p - 1./108.*m11*m33*RSe45633p - 1./72.*m11*m33*RSe463p
          - 1./108.*m11*m33*RSe45611p - 1./72.*m11*m33*RSe461p - 1./216.
         *m11*m33*RSe465p - 1./108.*m11*m15*RSe45633p - 1./54.*m11*m15*
         RSe463p + 1./27.*m11*m15*RSe45611p - 1./108.*m11*m15*RSe465p
          + 1./18.*m11*m13*RSe45613p + 1./54.*m11*m13*RSe45633p - 1./54.
         *m11*m13*RSe463p + 1./27.*m11*m13*RSe45611p - 1./18.*m11*m13*
         RSe461p - 1./27.*m11*m13*RSe465p - 1./108.*pow(m11,2)*
         RSe45613p - 1./432.*pow(m11,2)*RSe45633p - 1./216.*pow(m11,2)*
         RSe463p - 1./108.*pow(m11,2)*RSe45611p - 1./108.*pow(m11,2)*
         RSe461p );

      M13p6x23RV +=  + hhV(1,m15,m35,me2,m13,L,mu2) * (  - 1./432.*pow(
         m11,2)*RSe465p );

      M13p6x23RV +=  + hhV(1,m15,m36,m56,m13,L,mu2) * ( 1./144.*pow(
         m66,2) - 1./18.*m56*m66 + 1./9.*pow(m56,2) + 1./72.*m55*m66 - 
         1./18.*m55*m56 + 1./144.*pow(m55,2) + 1./36.*m36*m66 - 1./9.*
         m36*m56 + 1./36.*m36*m55 + 1./36.*pow(m36,2) + 1./72.*m33*m66
          - 1./18.*m33*m56 + 1./72.*m33*m55 + 1./36.*m33*m36 + 1./144.*
         pow(m33,2) + 1./36.*m15*m66 - 1./9.*m15*m56 + 1./36.*m15*m55
          + 1./18.*m15*m36 + 1./36.*m15*m33 + 1./36.*pow(m15,2) + 1./9.
         *m13*m66 - 4./9.*m13*m56 + 1./9.*m13*m55 + 2./9.*m13*m36 + 1./
         9.*m13*m33 + 2./9.*m13*m15 - 5./9.*pow(m13,2) + 1./72.*m11*m66
          - 1./18.*m11*m56 + 1./72.*m11*m55 + 1./36.*m11*m36 + 1./72.*
         m11*m33 + 1./36.*m11*m15 + 1./9.*m11*m13 + 1./144.*pow(m11,2)
          );

      M13p6x23RV +=  + hhV(1,m16,m11,m36,m13,L,mu2) * ( 1./12.*m13*m16*
         RS14563pe - 1./12.*m13*m16*RC1456pe + 1./12.*m13*m16*RS145pe )
         ;

      M13p6x23RV +=  + hhV(1,m16,m33,m36,m13,L,mu2) * (  - 1./108.*m66*
         RD3456ep - 1./216.*pow(m66,2)*RS34561pe - 1./432.*pow(m66,2)*
         RC3456pe - 1./108.*pow(m66,2)*RS345pe + 1./27.*m36*RD3456ep + 
         1./108.*m36*m66*RS34561pe + 1./54.*m36*m66*RC3456pe + 1./36.*
         m36*m66*RS345pe + 1./27.*pow(m36,2)*RS34561pe - 1./27.*pow(
         m36,2)*RC3456pe + 1./27.*pow(m36,2)*RS345pe - 1./27.*m33*
         RD3456ep - 7./216.*m33*m66*RS34561pe - 1./54.*m33*m66*RC3456pe
          - 5./216.*m33*m66*RS345pe + 1./54.*m33*m36*RS34561pe + 2./27.
         *m33*m36*RC3456pe - 5./54.*m33*m36*RS345pe - 1./18.*pow(m33,2)
         *RS34561pe - 1./27.*pow(m33,2)*RC3456pe + 1./18.*pow(m33,2)*
         RS345pe - 1./54.*m16*RD3456ep + 1./108.*m16*m66*RS34561pe - 1./
         108.*m16*m66*RC3456pe - 1./36.*m16*m66*RS345pe - 5./54.*m16*
         m36*RS34561pe + 1./27.*m16*m36*RC3456pe + 1./54.*m16*m36*
         RS345pe + 5./108.*m16*m33*RS34561pe - 1./27.*m16*m33*RC3456pe
          - 1./108.*m16*m33*RS345pe + 1./27.*pow(m16,2)*RS34561pe - 1./
         108.*pow(m16,2)*RC3456pe );

      M13p6x23RV +=  + hhV(1,m16,m33,m36,m13,L,mu2) * (  - 1./54.*pow(
         m16,2)*RS345pe - 2./27.*m13*RD3456ep - 1./54.*m13*m66*
         RS34561pe - 1./27.*m13*m66*RC3456pe - 1./18.*m13*m66*RS345pe
          - 4./27.*m13*m36*RS34561pe + 4./27.*m13*m36*RC3456pe - 4./27.
         *m13*m36*RS345pe - 1./27.*m13*m33*RS34561pe - 4./27.*m13*m33*
         RC3456pe + 5./27.*m13*m33*RS345pe + 29./108.*m13*m16*RS34561pe
          - 2./27.*m13*m16*RC3456pe - 13./108.*m13*m16*RS345pe - 5./27.
         *pow(m13,2)*RS34561pe + 5./27.*pow(m13,2)*RC3456pe - 5./27.*
         pow(m13,2)*RS345pe - 1./108.*m11*RD3456ep - 1./72.*m11*m66*
         RS34561pe - 1./216.*m11*m66*RC3456pe - 1./72.*m11*m66*RS345pe
          + 1./36.*m11*m36*RS34561pe + 1./54.*m11*m36*RC3456pe + 1./108.
         *m11*m36*RS345pe - 11./216.*m11*m33*RS34561pe - 1./54.*m11*m33
         *RC3456pe - 1./216.*m11*m33*RS345pe - 1./108.*m11*m16*RC3456pe
          - 1./54.*m11*m16*RS345pe - 1./18.*m11*m13*RS34561pe - 1./27.*
         m11*m13*RC3456pe - 1./54.*m11*m13*RS345pe - 1./108.*pow(m11,2)
         *RS34561pe );

      M13p6x23RV +=  + hhV(1,m16,m33,m36,m13,L,mu2) * (  - 1./432.*pow(
         m11,2)*RC3456pe - 1./216.*pow(m11,2)*RS345pe );

      M13p6x23RV +=  + hhV(1,m16,m34,m46,m13,L,mu2) * ( 1./144.*pow(
         m66,2) - 1./18.*m46*m66 + 1./9.*pow(m46,2) + 1./72.*m44*m66 - 
         1./18.*m44*m46 + 1./144.*pow(m44,2) + 1./36.*m34*m66 - 1./9.*
         m34*m46 + 1./36.*m34*m44 + 1./36.*pow(m34,2) + 1./72.*m33*m66
          - 1./18.*m33*m46 + 1./72.*m33*m44 + 1./36.*m33*m34 + 1./144.*
         pow(m33,2) + 1./36.*m16*m66 - 1./9.*m16*m46 + 1./36.*m16*m44
          + 1./18.*m16*m34 + 1./36.*m16*m33 + 1./36.*pow(m16,2) + 1./9.
         *m13*m66 - 4./9.*m13*m46 + 1./9.*m13*m44 + 2./9.*m13*m34 + 1./
         9.*m13*m33 + 2./9.*m13*m16 - 5./9.*pow(m13,2) + 1./72.*m11*m66
          - 1./18.*m11*m46 + 1./72.*m11*m44 + 1./36.*m11*m34 + 1./72.*
         m11*m33 + 1./36.*m11*m16 + 1./9.*m11*m13 + 1./144.*pow(m11,2)
          );

      M13p6x23RV +=  + hhV(1,m16,m35,m56,m13,L,mu2) * ( 1./144.*pow(
         m66,2) - 1./18.*m56*m66 + 1./9.*pow(m56,2) + 1./72.*m55*m66 - 
         1./18.*m55*m56 + 1./144.*pow(m55,2) + 1./36.*m35*m66 - 1./9.*
         m35*m56 + 1./36.*m35*m55 + 1./36.*pow(m35,2) + 1./72.*m33*m66
          - 1./18.*m33*m56 + 1./72.*m33*m55 + 1./36.*m33*m35 + 1./144.*
         pow(m33,2) + 1./36.*m16*m66 - 1./9.*m16*m56 + 1./36.*m16*m55
          + 1./18.*m16*m35 + 1./36.*m16*m33 + 1./36.*pow(m16,2) + 1./9.
         *m13*m66 - 4./9.*m13*m56 + 1./9.*m13*m55 + 2./9.*m13*m35 + 1./
         9.*m13*m33 + 2./9.*m13*m16 - 5./9.*pow(m13,2) + 1./72.*m11*m66
          - 1./18.*m11*m56 + 1./72.*m11*m55 + 1./36.*m11*m35 + 1./72.*
         m11*m33 + 1./36.*m11*m16 + 1./9.*m11*m13 + 1./144.*pow(m11,2)
          );

      M13p6x23RV +=  + hhV(1,m16,m36,m66,m13,L,mu2) * ( 1./36.*pow(
         m66,2) - 1./108.*pow(m66,2)*RS645pe - 1./18.*m36*m66 + 1./54.*
         m36*m66*RS645pe + 1./36.*pow(m36,2) - 1./108.*pow(m36,2)*
         RS645pe - 1./36.*m33*m66 + 1./108.*m33*m66*RS645pe + 1./36.*
         m33*m36 - 1./108.*m33*m36*RS645pe + 1./144.*pow(m33,2) - 1./
         432.*pow(m33,2)*RS645pe - 1./18.*m16*m66 + 1./54.*m16*m66*
         RS645pe + 1./18.*m16*m36 - 1./54.*m16*m36*RS645pe + 1./36.*m16
         *m33 - 1./108.*m16*m33*RS645pe + 1./36.*pow(m16,2) - 1./108.*
         pow(m16,2)*RS645pe - 2./9.*m13*m66 + 2./27.*m13*m66*RS645pe + 
         2./9.*m13*m36 - 2./27.*m13*m36*RS645pe + 1./9.*m13*m33 - 1./27.
         *m13*m33*RS645pe + 2./9.*m13*m16 - 2./27.*m13*m16*RS645pe - 5./
         9.*pow(m13,2) + 5./27.*pow(m13,2)*RS645pe - 1./36.*m11*m66 + 1.
         /108.*m11*m66*RS645pe + 1./36.*m11*m36 - 1./108.*m11*m36*
         RS645pe + 1./72.*m11*m33 - 1./216.*m11*m33*RS645pe + 1./36.*
         m11*m16 - 1./108.*m11*m16*RS645pe + 1./9.*m11*m13 - 1./27.*m11
         *m13*RS645pe );

      M13p6x23RV +=  + hhV(1,m16,m36,m66,m13,L,mu2) * ( 1./144.*pow(
         m11,2) - 1./432.*pow(m11,2)*RS645pe );

      M13p6x23RV +=  + hhV(1,m16,m36,mp2,m13,L,mu2) * (  - 1./54.*pow(
         mp2,2)*RSp45613e - 1./108.*pow(mp2,2)*RSp45633e + 1./27.*pow(
         mp2,2)*RSp453e - 1./108.*pow(mp2,2)*RSp45611e + 1./27.*pow(
         mp2,2)*RSp451e - 1./27.*pow(mp2,2)*RSp456e - 1./54.*m66*mp2*
         RSp45613e - 1./108.*m66*mp2*RSp45633e - 1./108.*m66*mp2*
         RSp45611e + 1./27.*m66*mp2*RSp456e - 1./216.*pow(m66,2)*
         RSp45613e - 1./432.*pow(m66,2)*RSp45633e - 1./108.*pow(m66,2)*
         RSp453e - 1./432.*pow(m66,2)*RSp45611e - 1./108.*pow(m66,2)*
         RSp451e - 1./108.*pow(m66,2)*RSp456e + 1./54.*m36*mp2*
         RSp45613e + 1./27.*m36*mp2*RSp45633e - 5./54.*m36*mp2*RSp453e
          - 1./54.*m36*mp2*RSp45611e + 1./54.*m36*mp2*RSp451e + 1./27.*
         m36*mp2*RSp456e + 1./108.*m36*m66*RSp45613e + 1./54.*m36*m66*
         RSp45633e + 1./36.*m36*m66*RSp453e - 1./108.*m36*m66*RSp45611e
          - 1./36.*m36*m66*RSp451e - 1./54.*m36*m66*RSp456e + 1./27.*
         pow(m36,2)*RSp45613e - 1./27.*pow(m36,2)*RSp45633e + 1./27.*
         pow(m36,2)*RSp453e );

      M13p6x23RV +=  + hhV(1,m16,m36,mp2,m13,L,mu2) * (  - 1./108.*pow(
         m36,2)*RSp45611e - 1./54.*pow(m36,2)*RSp451e - 1./108.*pow(
         m36,2)*RSp456e - 1./36.*m33*mp2*RSp45613e - 1./54.*m33*mp2*
         RSp45633e + 1./36.*m33*mp2*RSp453e - 1./108.*m33*mp2*RSp45611e
          + 1./108.*m33*mp2*RSp451e + 1./54.*m33*mp2*RSp456e - 1./72.*
         m33*m66*RSp45613e - 1./108.*m33*m66*RSp45633e - 5./216.*m33*
         m66*RSp453e - 1./216.*m33*m66*RSp45611e - 1./72.*m33*m66*
         RSp451e - 1./108.*m33*m66*RSp456e + 1./27.*m33*m36*RSp45633e
          - 1./108.*m33*m36*RSp45611e - 1./54.*m33*m36*RSp451e - 1./108.
         *m33*m36*RSp456e - 1./108.*pow(m33,2)*RSp45613e - 1./108.*pow(
         m33,2)*RSp45633e - 1./108.*pow(m33,2)*RSp453e - 1./432.*pow(
         m33,2)*RSp45611e - 1./216.*pow(m33,2)*RSp451e - 1./432.*pow(
         m33,2)*RSp456e + 1./54.*m16*mp2*RSp45613e - 1./54.*m16*mp2*
         RSp45633e + 1./54.*m16*mp2*RSp453e + 1./27.*m16*mp2*RSp45611e
          - 5./54.*m16*mp2*RSp451e + 1./27.*m16*mp2*RSp456e + 1./108.*
         m16*m66*RSp45613e );

      M13p6x23RV +=  + hhV(1,m16,m36,mp2,m13,L,mu2) * (  - 1./108.*m16*
         m66*RSp45633e - 1./36.*m16*m66*RSp453e + 1./54.*m16*m66*
         RSp45611e + 1./36.*m16*m66*RSp451e - 1./54.*m16*m66*RSp456e - 
         5./54.*m16*m36*RSp45613e + 1./27.*m16*m36*RSp45633e + 1./54.*
         m16*m36*RSp453e + 1./27.*m16*m36*RSp45611e + 1./54.*m16*m36*
         RSp451e - 1./54.*m16*m36*RSp456e + 1./36.*m16*m33*RSp45613e - 
         1./54.*m16*m33*RSp45633e - 1./36.*m16*m33*RSp453e + 1./54.*m16
         *m33*RSp45611e + 1./108.*m16*m33*RSp451e - 1./108.*m16*m33*
         RSp456e + 1./27.*pow(m16,2)*RSp45613e - 1./108.*pow(m16,2)*
         RSp45633e - 1./54.*pow(m16,2)*RSp453e - 1./27.*pow(m16,2)*
         RSp45611e + 1./27.*pow(m16,2)*RSp451e - 1./108.*pow(m16,2)*
         RSp456e + 2./27.*m13*mp2*RSp45613e + 1./27.*m13*mp2*RSp45633e
          - 4./27.*m13*mp2*RSp453e + 1./27.*m13*mp2*RSp45611e - 4./27.*
         m13*mp2*RSp451e + 4./27.*m13*mp2*RSp456e + 1./27.*m13*m66*
         RSp45613e + 1./54.*m13*m66*RSp45633e + 1./54.*m13*m66*
         RSp45611e );

      M13p6x23RV +=  + hhV(1,m16,m36,mp2,m13,L,mu2) * (  - 2./27.*m13*
         m66*RSp456e - 1./27.*m13*m36*RSp45613e - 2./27.*m13*m36*
         RSp45633e + 5./27.*m13*m36*RSp453e + 1./27.*m13*m36*RSp45611e
          - 1./27.*m13*m36*RSp451e - 2./27.*m13*m36*RSp456e + 1./18.*
         m13*m33*RSp45613e + 1./27.*m13*m33*RSp45633e - 1./18.*m13*m33*
         RSp453e + 1./54.*m13*m33*RSp45611e - 1./54.*m13*m33*RSp451e - 
         1./27.*m13*m33*RSp456e + 5./108.*m13*m16*RSp45613e + 1./27.*
         m13*m16*RSp45633e - 13./108.*m13*m16*RSp453e - 17./108.*m13*
         m16*RSp45611e + 29./108.*m13*m16*RSp451e - 2./27.*m13*m16*
         RSp456e + 7./27.*pow(m13,2)*RSp45613e - 1./27.*pow(m13,2)*
         RSp45633e - 5./27.*pow(m13,2)*RSp453e - 1./27.*pow(m13,2)*
         RSp45611e - 5./27.*pow(m13,2)*RSp451e + 5./27.*pow(m13,2)*
         RSp456e - 1./36.*m11*mp2*RSp45613e - 1./108.*m11*mp2*RSp45633e
          + 1./108.*m11*mp2*RSp453e - 1./54.*m11*mp2*RSp45611e + 1./36.
         *m11*mp2*RSp451e + 1./54.*m11*mp2*RSp456e - 1./72.*m11*m66*
         RSp45613e );

      M13p6x23RV +=  + hhV(1,m16,m36,mp2,m13,L,mu2) * (  - 1./216.*m11*
         m66*RSp45633e - 1./72.*m11*m66*RSp453e - 1./108.*m11*m66*
         RSp45611e - 5./216.*m11*m66*RSp451e - 1./108.*m11*m66*RSp456e
          + 1./36.*m11*m36*RSp45613e + 1./54.*m11*m36*RSp45633e + 1./
         108.*m11*m36*RSp453e - 1./54.*m11*m36*RSp45611e - 1./36.*m11*
         m36*RSp451e - 1./108.*m11*m36*RSp456e - 5./216.*m11*m33*
         RSp45613e - 1./108.*m11*m33*RSp45633e - 1./72.*m11*m33*RSp453e
          - 1./108.*m11*m33*RSp45611e - 1./72.*m11*m33*RSp451e - 1./216.
         *m11*m33*RSp456e - 1./108.*m11*m16*RSp45633e - 1./54.*m11*m16*
         RSp453e + 1./27.*m11*m16*RSp45611e - 1./108.*m11*m16*RSp456e
          + 1./18.*m11*m13*RSp45613e + 1./54.*m11*m13*RSp45633e - 1./54.
         *m11*m13*RSp453e + 1./27.*m11*m13*RSp45611e - 1./18.*m11*m13*
         RSp451e - 1./27.*m11*m13*RSp456e - 1./108.*pow(m11,2)*
         RSp45613e - 1./432.*pow(m11,2)*RSp45633e - 1./216.*pow(m11,2)*
         RSp453e - 1./108.*pow(m11,2)*RSp45611e - 1./108.*pow(m11,2)*
         RSp451e );

      M13p6x23RV +=  + hhV(1,m16,m36,mp2,m13,L,mu2) * (  - 1./432.*pow(
         m11,2)*RSp456e );

      M13p6x23RV +=  + hhV(1,m16,m36,me2,m13,L,mu2) * (  - 1./54.*pow(
         me2,2)*RSe45613p - 1./108.*pow(me2,2)*RSe45633p + 1./27.*pow(
         me2,2)*RSe453p - 1./108.*pow(me2,2)*RSe45611p + 1./27.*pow(
         me2,2)*RSe451p - 1./27.*pow(me2,2)*RSe456p - 1./54.*m66*me2*
         RSe45613p - 1./108.*m66*me2*RSe45633p - 1./108.*m66*me2*
         RSe45611p + 1./27.*m66*me2*RSe456p - 1./216.*pow(m66,2)*
         RSe45613p - 1./432.*pow(m66,2)*RSe45633p - 1./108.*pow(m66,2)*
         RSe453p - 1./432.*pow(m66,2)*RSe45611p - 1./108.*pow(m66,2)*
         RSe451p - 1./108.*pow(m66,2)*RSe456p + 1./54.*m36*me2*
         RSe45613p + 1./27.*m36*me2*RSe45633p - 5./54.*m36*me2*RSe453p
          - 1./54.*m36*me2*RSe45611p + 1./54.*m36*me2*RSe451p + 1./27.*
         m36*me2*RSe456p + 1./108.*m36*m66*RSe45613p + 1./54.*m36*m66*
         RSe45633p + 1./36.*m36*m66*RSe453p - 1./108.*m36*m66*RSe45611p
          - 1./36.*m36*m66*RSe451p - 1./54.*m36*m66*RSe456p + 1./27.*
         pow(m36,2)*RSe45613p - 1./27.*pow(m36,2)*RSe45633p + 1./27.*
         pow(m36,2)*RSe453p );

      M13p6x23RV +=  + hhV(1,m16,m36,me2,m13,L,mu2) * (  - 1./108.*pow(
         m36,2)*RSe45611p - 1./54.*pow(m36,2)*RSe451p - 1./108.*pow(
         m36,2)*RSe456p - 1./36.*m33*me2*RSe45613p - 1./54.*m33*me2*
         RSe45633p + 1./36.*m33*me2*RSe453p - 1./108.*m33*me2*RSe45611p
          + 1./108.*m33*me2*RSe451p + 1./54.*m33*me2*RSe456p - 1./72.*
         m33*m66*RSe45613p - 1./108.*m33*m66*RSe45633p - 5./216.*m33*
         m66*RSe453p - 1./216.*m33*m66*RSe45611p - 1./72.*m33*m66*
         RSe451p - 1./108.*m33*m66*RSe456p + 1./27.*m33*m36*RSe45633p
          - 1./108.*m33*m36*RSe45611p - 1./54.*m33*m36*RSe451p - 1./108.
         *m33*m36*RSe456p - 1./108.*pow(m33,2)*RSe45613p - 1./108.*pow(
         m33,2)*RSe45633p - 1./108.*pow(m33,2)*RSe453p - 1./432.*pow(
         m33,2)*RSe45611p - 1./216.*pow(m33,2)*RSe451p - 1./432.*pow(
         m33,2)*RSe456p + 1./54.*m16*me2*RSe45613p - 1./54.*m16*me2*
         RSe45633p + 1./54.*m16*me2*RSe453p + 1./27.*m16*me2*RSe45611p
          - 5./54.*m16*me2*RSe451p + 1./27.*m16*me2*RSe456p + 1./108.*
         m16*m66*RSe45613p );

      M13p6x23RV +=  + hhV(1,m16,m36,me2,m13,L,mu2) * (  - 1./108.*m16*
         m66*RSe45633p - 1./36.*m16*m66*RSe453p + 1./54.*m16*m66*
         RSe45611p + 1./36.*m16*m66*RSe451p - 1./54.*m16*m66*RSe456p - 
         5./54.*m16*m36*RSe45613p + 1./27.*m16*m36*RSe45633p + 1./54.*
         m16*m36*RSe453p + 1./27.*m16*m36*RSe45611p + 1./54.*m16*m36*
         RSe451p - 1./54.*m16*m36*RSe456p + 1./36.*m16*m33*RSe45613p - 
         1./54.*m16*m33*RSe45633p - 1./36.*m16*m33*RSe453p + 1./54.*m16
         *m33*RSe45611p + 1./108.*m16*m33*RSe451p - 1./108.*m16*m33*
         RSe456p + 1./27.*pow(m16,2)*RSe45613p - 1./108.*pow(m16,2)*
         RSe45633p - 1./54.*pow(m16,2)*RSe453p - 1./27.*pow(m16,2)*
         RSe45611p + 1./27.*pow(m16,2)*RSe451p - 1./108.*pow(m16,2)*
         RSe456p + 2./27.*m13*me2*RSe45613p + 1./27.*m13*me2*RSe45633p
          - 4./27.*m13*me2*RSe453p + 1./27.*m13*me2*RSe45611p - 4./27.*
         m13*me2*RSe451p + 4./27.*m13*me2*RSe456p + 1./27.*m13*m66*
         RSe45613p + 1./54.*m13*m66*RSe45633p + 1./54.*m13*m66*
         RSe45611p );

      M13p6x23RV +=  + hhV(1,m16,m36,me2,m13,L,mu2) * (  - 2./27.*m13*
         m66*RSe456p - 1./27.*m13*m36*RSe45613p - 2./27.*m13*m36*
         RSe45633p + 5./27.*m13*m36*RSe453p + 1./27.*m13*m36*RSe45611p
          - 1./27.*m13*m36*RSe451p - 2./27.*m13*m36*RSe456p + 1./18.*
         m13*m33*RSe45613p + 1./27.*m13*m33*RSe45633p - 1./18.*m13*m33*
         RSe453p + 1./54.*m13*m33*RSe45611p - 1./54.*m13*m33*RSe451p - 
         1./27.*m13*m33*RSe456p + 5./108.*m13*m16*RSe45613p + 1./27.*
         m13*m16*RSe45633p - 13./108.*m13*m16*RSe453p - 17./108.*m13*
         m16*RSe45611p + 29./108.*m13*m16*RSe451p - 2./27.*m13*m16*
         RSe456p + 7./27.*pow(m13,2)*RSe45613p - 1./27.*pow(m13,2)*
         RSe45633p - 5./27.*pow(m13,2)*RSe453p - 1./27.*pow(m13,2)*
         RSe45611p - 5./27.*pow(m13,2)*RSe451p + 5./27.*pow(m13,2)*
         RSe456p - 1./36.*m11*me2*RSe45613p - 1./108.*m11*me2*RSe45633p
          + 1./108.*m11*me2*RSe453p - 1./54.*m11*me2*RSe45611p + 1./36.
         *m11*me2*RSe451p + 1./54.*m11*me2*RSe456p - 1./72.*m11*m66*
         RSe45613p );

      M13p6x23RV +=  + hhV(1,m16,m36,me2,m13,L,mu2) * (  - 1./216.*m11*
         m66*RSe45633p - 1./72.*m11*m66*RSe453p - 1./108.*m11*m66*
         RSe45611p - 5./216.*m11*m66*RSe451p - 1./108.*m11*m66*RSe456p
          + 1./36.*m11*m36*RSe45613p + 1./54.*m11*m36*RSe45633p + 1./
         108.*m11*m36*RSe453p - 1./54.*m11*m36*RSe45611p - 1./36.*m11*
         m36*RSe451p - 1./108.*m11*m36*RSe456p - 5./216.*m11*m33*
         RSe45613p - 1./108.*m11*m33*RSe45633p - 1./72.*m11*m33*RSe453p
          - 1./108.*m11*m33*RSe45611p - 1./72.*m11*m33*RSe451p - 1./216.
         *m11*m33*RSe456p - 1./108.*m11*m16*RSe45633p - 1./54.*m11*m16*
         RSe453p + 1./27.*m11*m16*RSe45611p - 1./108.*m11*m16*RSe456p
          + 1./18.*m11*m13*RSe45613p + 1./54.*m11*m13*RSe45633p - 1./54.
         *m11*m13*RSe453p + 1./27.*m11*m13*RSe45611p - 1./18.*m11*m13*
         RSe451p - 1./27.*m11*m13*RSe456p - 1./108.*pow(m11,2)*
         RSe45613p - 1./432.*pow(m11,2)*RSe45633p - 1./216.*pow(m11,2)*
         RSe453p - 1./108.*pow(m11,2)*RSe45611p - 1./108.*pow(m11,2)*
         RSe451p );

      M13p6x23RV +=  + hhV(1,m16,m36,me2,m13,L,mu2) * (  - 1./432.*pow(
         m11,2)*RSe456p );

      M13p6x23RV +=  + hhV(1,m33,m13,m33,m13,L,mu2) * (  - 1./4.*m13*
         m33 );

      M13p6x23RV +=  + hhV(1,m33,m14,m34,m13,L,mu2) * (  - 1./12.*m13*
         m33*RS34561pe + 1./12.*m13*m33*RS356pe );

      M13p6x23RV +=  + hhV(1,m33,m15,m35,m13,L,mu2) * (  - 1./12.*m13*
         m33*RS34561pe + 1./12.*m13*m33*RS346pe );

      M13p6x23RV +=  + hhV(1,m33,m16,m36,m13,L,mu2) * (  - 1./12.*m13*
         m33*RS34561pe + 1./12.*m13*m33*RS345pe );

      M13p6x23RV +=  + hhV(1,m34,m11,m14,m13,L,mu2) * ( 1./12.*m13*m34*
         RS14563pe - 1./12.*m13*m34*RS156pe );

      M13p6x23RV +=  + hhV(1,m34,m14,m33,m13,L,mu2) * ( 1./12.*m13*m34*
         RS34561pe - 1./12.*m13*m34*RC3456pe + 1./12.*m13*m34*RS356pe )
         ;

      M13p6x23RV +=  + hhV(1,m34,m14,mp2,m13,L,mu2) * ( 1./12.*m13*m34*
         RSp45613e - 1./12.*m13*m34*RSp45633e + 1./12.*m13*m34*RSp563e
          - 1./12.*m13*m34*RSp561e );

      M13p6x23RV +=  + hhV(1,m34,m14,me2,m13,L,mu2) * ( 1./12.*m13*m34*
         RSe45613p - 1./12.*m13*m34*RSe45633p + 1./12.*m13*m34*RSe563p
          - 1./12.*m13*m34*RSe561p );

      M13p6x23RV +=  + hhV(1,m35,m11,m15,m13,L,mu2) * ( 1./12.*m13*m35*
         RS14563pe - 1./12.*m13*m35*RS146pe );

      M13p6x23RV +=  + hhV(1,m35,m15,m33,m13,L,mu2) * ( 1./12.*m13*m35*
         RS34561pe - 1./12.*m13*m35*RC3456pe + 1./12.*m13*m35*RS346pe )
         ;

      M13p6x23RV +=  + hhV(1,m35,m15,mp2,m13,L,mu2) * ( 1./12.*m13*m35*
         RSp45613e - 1./12.*m13*m35*RSp45633e + 1./12.*m13*m35*RSp463e
          - 1./12.*m13*m35*RSp461e );

      M13p6x23RV +=  + hhV(1,m35,m15,me2,m13,L,mu2) * ( 1./12.*m13*m35*
         RSe45613p - 1./12.*m13*m35*RSe45633p + 1./12.*m13*m35*RSe463p
          - 1./12.*m13*m35*RSe461p );

      M13p6x23RV +=  + hhV(1,m36,m11,m16,m13,L,mu2) * ( 1./12.*m13*m36*
         RS14563pe - 1./12.*m13*m36*RS145pe );

      M13p6x23RV +=  + hhV(1,m36,m16,m33,m13,L,mu2) * ( 1./12.*m13*m36*
         RS34561pe - 1./12.*m13*m36*RC3456pe + 1./12.*m13*m36*RS345pe )
         ;

      M13p6x23RV +=  + hhV(1,m36,m16,mp2,m13,L,mu2) * ( 1./12.*m13*m36*
         RSp45613e - 1./12.*m13*m36*RSp45633e + 1./12.*m13*m36*RSp453e
          - 1./12.*m13*m36*RSp451e );

      M13p6x23RV +=  + hhV(1,m36,m16,me2,m13,L,mu2) * ( 1./12.*m13*m36*
         RSe45613p - 1./12.*m13*m36*RSe45633p + 1./12.*m13*m36*RSe453p
          - 1./12.*m13*m36*RSe451p );

      M13p6x23RV +=  + hhV(1,m44,m14,m34,m13,L,mu2) * ( 1./4.*m13*m44
          - 1./12.*m13*m44*RS456pe );

      M13p6x23RV +=  + hhV(1,m45,m14,m35,m13,L,mu2) * ( 1./4.*m13*m45 )
         ;

      M13p6x23RV +=  + hhV(1,m45,m15,m34,m13,L,mu2) * ( 1./4.*m13*m45 )
         ;

      M13p6x23RV +=  + hhV(1,m46,m14,m36,m13,L,mu2) * ( 1./4.*m13*m46 )
         ;

      M13p6x23RV +=  + hhV(1,m46,m16,m34,m13,L,mu2) * ( 1./4.*m13*m46 )
         ;

      M13p6x23RV +=  + hhV(1,m55,m15,m35,m13,L,mu2) * ( 1./4.*m13*m55
          - 1./12.*m13*m55*RS546pe );

      M13p6x23RV +=  + hhV(1,m56,m15,m36,m13,L,mu2) * ( 1./4.*m13*m56 )
         ;

      M13p6x23RV +=  + hhV(1,m56,m16,m35,m13,L,mu2) * ( 1./4.*m13*m56 )
         ;

      M13p6x23RV +=  + hhV(1,m66,m16,m36,m13,L,mu2) * ( 1./4.*m13*m66
          - 1./12.*m13*m66*RS645pe );

      M13p6x23RV +=  + hhV(1,mp2,m14,m34,m13,L,mu2) * (  - 1./12.*m13*
         mp2*RSp45613e + 1./12.*m13*mp2*RSp563e + 1./12.*m13*mp2*
         RSp561e - 1./12.*m13*mp2*RSp564e );

      M13p6x23RV +=  + hhV(1,mp2,m15,m35,m13,L,mu2) * (  - 1./12.*m13*
         mp2*RSp45613e + 1./12.*m13*mp2*RSp463e + 1./12.*m13*mp2*
         RSp461e - 1./12.*m13*mp2*RSp465e );

      M13p6x23RV +=  + hhV(1,mp2,m16,m36,m13,L,mu2) * (  - 1./12.*m13*
         mp2*RSp45613e + 1./12.*m13*mp2*RSp453e + 1./12.*m13*mp2*
         RSp451e - 1./12.*m13*mp2*RSp456e );

      M13p6x23RV +=  + hhV(1,me2,m14,m34,m13,L,mu2) * (  - 1./12.*m13*
         me2*RSe45613p + 1./12.*m13*me2*RSe563p + 1./12.*m13*me2*
         RSe561p - 1./12.*m13*me2*RSe564p );

      M13p6x23RV +=  + hhV(1,me2,m15,m35,m13,L,mu2) * (  - 1./12.*m13*
         me2*RSe45613p + 1./12.*m13*me2*RSe463p + 1./12.*m13*me2*
         RSe461p - 1./12.*m13*me2*RSe465p );

      M13p6x23RV +=  + hhV(1,me2,m16,m36,m13,L,mu2) * (  - 1./12.*m13*
         me2*RSe45613p + 1./12.*m13*me2*RSe453p + 1./12.*m13*me2*
         RSe451p - 1./12.*m13*me2*RSe456p );

      M13p6x23RV +=  + hhV(2,m11,m11,m13,m13,L,mu2) * (  - 1./162.*m33*
         pow(RD1456ep,2) + 1./162.*pow(m33,2)*RD1456ep*RS14563pe + 1./
         324.*pow(m33,2)*RD1456ep*RC1456pe + 1./81.*m13*pow(RD1456ep,2)
          - 1./81.*m13*m33*RD1456ep*RC1456pe + 16./81.*pow(m13,2)*
         RD1456ep*RS14563pe - 8./81.*pow(m13,2)*RD1456ep*RC1456pe - 1./
         162.*m11*pow(RD1456ep,2) + 2./81.*m11*m33*RD1456ep*RS14563pe
          + 1./162.*m11*m33*RD1456ep*RC1456pe - 2./81.*m11*m13*RD1456ep
         *RS14563pe - 1./81.*m11*m13*RD1456ep*RC1456pe + 1./54.*pow(
         m11,2)*RD1456ep*RS14563pe + 1./324.*pow(m11,2)*RD1456ep*
         RC1456pe );

      M13p6x23RV +=  + hhV(2,m11,m13,m33,m13,L,mu2) * ( 1./81.*m33*
         RD1456ep*RD3456ep + 1./81.*pow(m33,2)*RD1456ep*RC3456pe - 2./
         81.*m13*RD1456ep*RD3456ep + 4./81.*m13*m33*RD1456ep*RS34561pe
          - 4./81.*m13*m33*RD1456ep*RC3456pe - 8./81.*pow(m13,2)*
         RD1456ep*RS34561pe + 4./81.*pow(m13,2)*RD1456ep*RC3456pe + 1./
         81.*m11*RD1456ep*RD3456ep + 2./81.*m11*m33*RD1456ep*RS34561pe
          + 2./81.*m11*m33*RD1456ep*RC3456pe - 4./81.*m11*m13*RD1456ep*
         RC3456pe + 2./81.*pow(m11,2)*RD1456ep*RS34561pe + 1./81.*pow(
         m11,2)*RD1456ep*RC3456pe );

      M13p6x23RV +=  + hhV(2,m11,m13,mp2,m13,L,mu2) * (  - 1./162.*pow(
         mp2,2)*RD1456ep*RSp45613e + 1./324.*pow(mp2,2)*RD1456ep*
         RSp45633e + 1./324.*pow(mp2,2)*RSp45611e*RD1456ep + 1./162.*
         m33*mp2*RD1456ep*RSp45633e - 1./162.*m33*mp2*RSp45611e*
         RD1456ep + 1./162.*pow(m33,2)*RD1456ep*RSp45613e + 1./324.*
         pow(m33,2)*RD1456ep*RSp45633e + 1./324.*pow(m33,2)*RSp45611e*
         RD1456ep + 4./81.*m13*mp2*RD1456ep*RSp45613e - 2./81.*m13*mp2*
         RD1456ep*RSp45633e - 2./81.*m13*mp2*RSp45611e*RD1456ep - 2./81.
         *m13*m33*RD1456ep*RSp45633e + 2./81.*m13*m33*RSp45611e*
         RD1456ep - 8./81.*pow(m13,2)*RD1456ep*RSp45613e + 4./81.*pow(
         m13,2)*RD1456ep*RSp45633e + 4./81.*pow(m13,2)*RSp45611e*
         RD1456ep + 1./81.*m11*mp2*RD1456ep*RSp45633e - 1./81.*m11*mp2*
         RSp45611e*RD1456ep + 2./81.*m11*m33*RD1456ep*RSp45613e + 1./81.
         *m11*m33*RD1456ep*RSp45633e + 1./81.*m11*m33*RSp45611e*
         RD1456ep - 4./81.*m11*m13*RD1456ep*RSp45633e + 4./81.*m11*m13*
         RSp45611e*RD1456ep );

      M13p6x23RV +=  + hhV(2,m11,m13,mp2,m13,L,mu2) * ( 2./81.*pow(
         m11,2)*RD1456ep*RSp45613e + 1./81.*pow(m11,2)*RD1456ep*
         RSp45633e + 1./81.*pow(m11,2)*RSp45611e*RD1456ep );

      M13p6x23RV +=  + hhV(2,m11,m13,me2,m13,L,mu2) * (  - 1./162.*pow(
         me2,2)*RD1456ep*RSe45613p + 1./324.*pow(me2,2)*RD1456ep*
         RSe45633p + 1./324.*pow(me2,2)*RSe45611p*RD1456ep + 1./162.*
         m33*me2*RD1456ep*RSe45633p - 1./162.*m33*me2*RSe45611p*
         RD1456ep + 1./162.*pow(m33,2)*RD1456ep*RSe45613p + 1./324.*
         pow(m33,2)*RD1456ep*RSe45633p + 1./324.*pow(m33,2)*RSe45611p*
         RD1456ep + 4./81.*m13*me2*RD1456ep*RSe45613p - 2./81.*m13*me2*
         RD1456ep*RSe45633p - 2./81.*m13*me2*RSe45611p*RD1456ep - 2./81.
         *m13*m33*RD1456ep*RSe45633p + 2./81.*m13*m33*RSe45611p*
         RD1456ep - 8./81.*pow(m13,2)*RD1456ep*RSe45613p + 4./81.*pow(
         m13,2)*RD1456ep*RSe45633p + 4./81.*pow(m13,2)*RSe45611p*
         RD1456ep + 1./81.*m11*me2*RD1456ep*RSe45633p - 1./81.*m11*me2*
         RSe45611p*RD1456ep + 2./81.*m11*m33*RD1456ep*RSe45613p + 1./81.
         *m11*m33*RD1456ep*RSe45633p + 1./81.*m11*m33*RSe45611p*
         RD1456ep - 4./81.*m11*m13*RD1456ep*RSe45633p + 4./81.*m11*m13*
         RSe45611p*RD1456ep );

      M13p6x23RV +=  + hhV(2,m11,m13,me2,m13,L,mu2) * ( 2./81.*pow(
         m11,2)*RD1456ep*RSe45613p + 1./81.*pow(m11,2)*RD1456ep*
         RSe45633p + 1./81.*pow(m11,2)*RSe45611p*RD1456ep );

      M13p6x23RV +=  + hhV(2,m11,m14,m34,m13,L,mu2) * (  - 1./432.*pow(
         m44,2)*RD1456ep - 1./108.*m34*m44*RD1456ep - 1./108.*pow(
         m34,2)*RD1456ep - 1./216.*m33*m44*RD1456ep - 1./108.*m33*m34*
         RD1456ep - 1./432.*pow(m33,2)*RD1456ep + 1./54.*m14*m44*
         RD1456ep + 1./27.*m14*m34*RD1456ep + 1./54.*m14*m33*RD1456ep
          - 1./27.*pow(m14,2)*RD1456ep + 1./54.*m13*m44*RD1456ep + 1./
         27.*m13*m34*RD1456ep + 1./54.*m13*m33*RD1456ep - 2./27.*m13*
         m14*RD1456ep - 1./27.*pow(m13,2)*RD1456ep - 1./54.*m11*m44*
         RD1456ep - 1./27.*m11*m34*RD1456ep - 1./54.*m11*m33*RD1456ep
          + 2./27.*m11*m14*RD1456ep + 2./27.*m11*m13*RD1456ep - 1./27.*
         pow(m11,2)*RD1456ep );

      M13p6x23RV +=  + hhV(2,m11,m15,m35,m13,L,mu2) * (  - 1./432.*pow(
         m55,2)*RD1456ep - 1./108.*m35*m55*RD1456ep - 1./108.*pow(
         m35,2)*RD1456ep - 1./216.*m33*m55*RD1456ep - 1./108.*m33*m35*
         RD1456ep - 1./432.*pow(m33,2)*RD1456ep + 1./54.*m15*m55*
         RD1456ep + 1./27.*m15*m35*RD1456ep + 1./54.*m15*m33*RD1456ep
          - 1./27.*pow(m15,2)*RD1456ep + 1./54.*m13*m55*RD1456ep + 1./
         27.*m13*m35*RD1456ep + 1./54.*m13*m33*RD1456ep - 2./27.*m13*
         m15*RD1456ep - 1./27.*pow(m13,2)*RD1456ep - 1./54.*m11*m55*
         RD1456ep - 1./27.*m11*m35*RD1456ep - 1./54.*m11*m33*RD1456ep
          + 2./27.*m11*m15*RD1456ep + 2./27.*m11*m13*RD1456ep - 1./27.*
         pow(m11,2)*RD1456ep );

      M13p6x23RV +=  + hhV(2,m11,m16,m36,m13,L,mu2) * (  - 1./432.*pow(
         m66,2)*RD1456ep - 1./108.*m36*m66*RD1456ep - 1./108.*pow(
         m36,2)*RD1456ep - 1./216.*m33*m66*RD1456ep - 1./108.*m33*m36*
         RD1456ep - 1./432.*pow(m33,2)*RD1456ep + 1./54.*m16*m66*
         RD1456ep + 1./27.*m16*m36*RD1456ep + 1./54.*m16*m33*RD1456ep
          - 1./27.*pow(m16,2)*RD1456ep + 1./54.*m13*m66*RD1456ep + 1./
         27.*m13*m36*RD1456ep + 1./54.*m13*m33*RD1456ep - 2./27.*m13*
         m16*RD1456ep - 1./27.*pow(m13,2)*RD1456ep - 1./54.*m11*m66*
         RD1456ep - 1./27.*m11*m36*RD1456ep - 1./54.*m11*m33*RD1456ep
          + 2./27.*m11*m16*RD1456ep + 2./27.*m11*m13*RD1456ep - 1./27.*
         pow(m11,2)*RD1456ep );

      M13p6x23RV +=  + hhV(3,m13,m11,m11,m13,L,mu2) * (  - 1./18.*pow(
         m13,2)*RD1456ep*RS14563pe + 1./36.*pow(m13,2)*RD1456ep*
         RC1456pe );

      M13p6x23RV +=  + hhV(3,m13,m11,m33,m13,L,mu2) * (  - 1./18.*pow(
         m13,2)*RD1456ep*RS34561pe + 1./36.*pow(m13,2)*RD1456ep*
         RC3456pe );

      M13p6x23RV +=  + hhV(3,m13,m11,mp2,m13,L,mu2) * (  - 1./18.*pow(
         m13,2)*RD1456ep*RSp45613e + 1./36.*pow(m13,2)*RD1456ep*
         RSp45633e + 1./36.*pow(m13,2)*RSp45611e*RD1456ep );

      M13p6x23RV +=  + hhV(3,m13,m11,me2,m13,L,mu2) * (  - 1./18.*pow(
         m13,2)*RD1456ep*RSe45613p + 1./36.*pow(m13,2)*RD1456ep*
         RSe45633p + 1./36.*pow(m13,2)*RSe45611p*RD1456ep );

      M13p6x23RV +=  + hhV(3,m13,m33,m33,m13,L,mu2) * (  - 1./162.*m33*
         pow(RD3456ep,2) + 1./54.*pow(m33,2)*RD3456ep*RS34561pe + 1./
         324.*pow(m33,2)*RD3456ep*RC3456pe - 5./216.*m13*pow(
         RD3456ep,2) + 4./81.*m13*m33*RD3456ep*RS34561pe + 2./81.*m13*
         m33*RD3456ep*RC3456pe - 25./162.*pow(m13,2)*RD3456ep*RS34561pe
          + 25./324.*pow(m13,2)*RD3456ep*RC3456pe - 1./162.*m11*pow(
         RD3456ep,2) + 2./81.*m11*m33*RD3456ep*RS34561pe + 1./162.*m11*
         m33*RD3456ep*RC3456pe + 2./81.*m11*m13*RD3456ep*RC3456pe + 1./
         162.*pow(m11,2)*RD3456ep*RS34561pe + 1./324.*pow(m11,2)*
         RD3456ep*RC3456pe );

      M13p6x23RV +=  + hhV(3,m13,m33,mp2,m13,L,mu2) * (  - 1./162.*pow(
         mp2,2)*RD3456ep*RSp45613e + 1./324.*pow(mp2,2)*RSp45633e*
         RD3456ep + 1./324.*pow(mp2,2)*RSp45611e*RD3456ep - 1./81.*m33*
         mp2*RSp45633e*RD3456ep + 1./81.*m33*mp2*RSp45611e*RD3456ep + 2.
         /81.*pow(m33,2)*RD3456ep*RSp45613e + 1./81.*pow(m33,2)*
         RSp45633e*RD3456ep + 1./81.*pow(m33,2)*RSp45611e*RD3456ep + 4./
         81.*m13*mp2*RD3456ep*RSp45613e - 2./81.*m13*mp2*RSp45633e*
         RD3456ep - 2./81.*m13*mp2*RSp45611e*RD3456ep + 4./81.*m13*m33*
         RSp45633e*RD3456ep - 4./81.*m13*m33*RSp45611e*RD3456ep - 25./
         162.*pow(m13,2)*RD3456ep*RSp45613e + 25./324.*pow(m13,2)*
         RSp45633e*RD3456ep + 25./324.*pow(m13,2)*RSp45611e*RD3456ep - 
         1./162.*m11*mp2*RSp45633e*RD3456ep + 1./162.*m11*mp2*RSp45611e
         *RD3456ep + 2./81.*m11*m33*RD3456ep*RSp45613e + 1./81.*m11*m33
         *RSp45633e*RD3456ep + 1./81.*m11*m33*RSp45611e*RD3456ep + 2./
         81.*m11*m13*RSp45633e*RD3456ep - 2./81.*m11*m13*RSp45611e*
         RD3456ep );

      M13p6x23RV +=  + hhV(3,m13,m33,mp2,m13,L,mu2) * ( 1./162.*pow(
         m11,2)*RD3456ep*RSp45613e + 1./324.*pow(m11,2)*RSp45633e*
         RD3456ep + 1./324.*pow(m11,2)*RSp45611e*RD3456ep );

      M13p6x23RV +=  + hhV(3,m13,m33,me2,m13,L,mu2) * (  - 1./162.*pow(
         me2,2)*RD3456ep*RSe45613p + 1./324.*pow(me2,2)*RSe45633p*
         RD3456ep + 1./324.*pow(me2,2)*RSe45611p*RD3456ep - 1./81.*m33*
         me2*RSe45633p*RD3456ep + 1./81.*m33*me2*RSe45611p*RD3456ep + 2.
         /81.*pow(m33,2)*RD3456ep*RSe45613p + 1./81.*pow(m33,2)*
         RSe45633p*RD3456ep + 1./81.*pow(m33,2)*RSe45611p*RD3456ep + 4./
         81.*m13*me2*RD3456ep*RSe45613p - 2./81.*m13*me2*RSe45633p*
         RD3456ep - 2./81.*m13*me2*RSe45611p*RD3456ep + 4./81.*m13*m33*
         RSe45633p*RD3456ep - 4./81.*m13*m33*RSe45611p*RD3456ep - 25./
         162.*pow(m13,2)*RD3456ep*RSe45613p + 25./324.*pow(m13,2)*
         RSe45633p*RD3456ep + 25./324.*pow(m13,2)*RSe45611p*RD3456ep - 
         1./162.*m11*me2*RSe45633p*RD3456ep + 1./162.*m11*me2*RSe45611p
         *RD3456ep + 2./81.*m11*m33*RD3456ep*RSe45613p + 1./81.*m11*m33
         *RSe45633p*RD3456ep + 1./81.*m11*m33*RSe45611p*RD3456ep + 2./
         81.*m11*m13*RSe45633p*RD3456ep - 2./81.*m11*m13*RSe45611p*
         RD3456ep );

      M13p6x23RV +=  + hhV(3,m13,m33,me2,m13,L,mu2) * ( 1./162.*pow(
         m11,2)*RD3456ep*RSe45613p + 1./324.*pow(m11,2)*RSe45633p*
         RD3456ep + 1./324.*pow(m11,2)*RSe45611p*RD3456ep );

      M13p6x23RV +=  + hhV(3,m14,m11,m34,m13,L,mu2) * (  - 1./12.*m13*
         m14*RD1456ep );

      M13p6x23RV +=  + hhV(3,m14,m33,m34,m13,L,mu2) * (  - 1./432.*pow(
         m44,2)*RD3456ep + 1./54.*m34*m44*RD3456ep - 1./27.*pow(m34,2)*
         RD3456ep - 1./54.*m33*m44*RD3456ep + 2./27.*m33*m34*RD3456ep
          - 1./27.*pow(m33,2)*RD3456ep - 1./108.*m14*m44*RD3456ep + 1./
         27.*m14*m34*RD3456ep - 1./27.*m14*m33*RD3456ep - 1./108.*pow(
         m14,2)*RD3456ep - 1./27.*m13*m44*RD3456ep + 4./27.*m13*m34*
         RD3456ep - 4./27.*m13*m33*RD3456ep - 2./27.*m13*m14*RD3456ep
          + 5./27.*pow(m13,2)*RD3456ep - 1./216.*m11*m44*RD3456ep + 1./
         54.*m11*m34*RD3456ep - 1./54.*m11*m33*RD3456ep - 1./108.*m11*
         m14*RD3456ep - 1./27.*m11*m13*RD3456ep - 1./432.*pow(m11,2)*
         RD3456ep );

      M13p6x23RV +=  + hhV(3,m15,m11,m35,m13,L,mu2) * (  - 1./12.*m13*
         m15*RD1456ep );

      M13p6x23RV +=  + hhV(3,m15,m33,m35,m13,L,mu2) * (  - 1./432.*pow(
         m55,2)*RD3456ep + 1./54.*m35*m55*RD3456ep - 1./27.*pow(m35,2)*
         RD3456ep - 1./54.*m33*m55*RD3456ep + 2./27.*m33*m35*RD3456ep
          - 1./27.*pow(m33,2)*RD3456ep - 1./108.*m15*m55*RD3456ep + 1./
         27.*m15*m35*RD3456ep - 1./27.*m15*m33*RD3456ep - 1./108.*pow(
         m15,2)*RD3456ep - 1./27.*m13*m55*RD3456ep + 4./27.*m13*m35*
         RD3456ep - 4./27.*m13*m33*RD3456ep - 2./27.*m13*m15*RD3456ep
          + 5./27.*pow(m13,2)*RD3456ep - 1./216.*m11*m55*RD3456ep + 1./
         54.*m11*m35*RD3456ep - 1./54.*m11*m33*RD3456ep - 1./108.*m11*
         m15*RD3456ep - 1./27.*m11*m13*RD3456ep - 1./432.*pow(m11,2)*
         RD3456ep );

      M13p6x23RV +=  + hhV(3,m16,m11,m36,m13,L,mu2) * (  - 1./12.*m13*
         m16*RD1456ep );

      M13p6x23RV +=  + hhV(3,m16,m33,m36,m13,L,mu2) * (  - 1./432.*pow(
         m66,2)*RD3456ep + 1./54.*m36*m66*RD3456ep - 1./27.*pow(m36,2)*
         RD3456ep - 1./54.*m33*m66*RD3456ep + 2./27.*m33*m36*RD3456ep
          - 1./27.*pow(m33,2)*RD3456ep - 1./108.*m16*m66*RD3456ep + 1./
         27.*m16*m36*RD3456ep - 1./27.*m16*m33*RD3456ep - 1./108.*pow(
         m16,2)*RD3456ep - 1./27.*m13*m66*RD3456ep + 4./27.*m13*m36*
         RD3456ep - 4./27.*m13*m33*RD3456ep - 2./27.*m13*m16*RD3456ep
          + 5./27.*pow(m13,2)*RD3456ep - 1./216.*m11*m66*RD3456ep + 1./
         54.*m11*m36*RD3456ep - 1./54.*m11*m33*RD3456ep - 1./108.*m11*
         m16*RD3456ep - 1./27.*m11*m13*RD3456ep - 1./432.*pow(m11,2)*
         RD3456ep );

      M13p6x23RV +=  + hhV(4,m11,m13,m33,m13,L,mu2) * ( 1./81.*m33*
         RD1456ep*RD3456ep + 2./81.*pow(m33,2)*RD3456ep*RS14563pe + 1./
         81.*pow(m33,2)*RC1456pe*RD3456ep - 2./81.*m13*RD1456ep*
         RD3456ep - 4./81.*m13*m33*RC1456pe*RD3456ep - 8./81.*pow(
         m13,2)*RD3456ep*RS14563pe + 4./81.*pow(m13,2)*RC1456pe*
         RD3456ep + 1./81.*m11*RD1456ep*RD3456ep + 2./81.*m11*m33*
         RD3456ep*RS14563pe + 2./81.*m11*m33*RC1456pe*RD3456ep + 4./81.
         *m11*m13*RD3456ep*RS14563pe - 4./81.*m11*m13*RC1456pe*RD3456ep
          + 1./81.*pow(m11,2)*RC1456pe*RD3456ep );

      M13p6x23RV +=  + hhV(4,m13,m11,m33,m13,L,mu2) * (  - 1./18.*pow(
         m13,2)*RD3456ep*RS14563pe + 1./36.*pow(m13,2)*RC1456pe*
         RD3456ep );

      M13p6x23RV +=  + hhV(4,m13,m33,m33,m13,L,mu2) * (  - 1./648.*m13*
         pow(RD3456ep,2) );

      M13p6x23RV +=  + hhV(4,m34,m14,m33,m13,L,mu2) * (  - 1./12.*m13*
         m34*RD3456ep );

      M13p6x23RV +=  + hhV(4,m35,m15,m33,m13,L,mu2) * (  - 1./12.*m13*
         m35*RD3456ep );

      M13p6x23RV +=  + hhV(4,m36,m16,m33,m13,L,mu2) * (  - 1./12.*m13*
         m36*RD3456ep );

      M13p6x23RV +=  + hhV(5,m11,m11,m13,m13,L,mu2) * ( 1./648.*pow(
         m33,2)*pow(RD1456ep,2) - 1./162.*m13*m33*pow(RD1456ep,2) - 4./
         81.*pow(m13,2)*pow(RD1456ep,2) + 1./324.*m11*m33*pow(
         RD1456ep,2) - 1./162.*m11*m13*pow(RD1456ep,2) + 1./648.*pow(
         m11,2)*pow(RD1456ep,2) );

      M13p6x23RV +=  + hhV(6,m11,m13,m33,m13,L,mu2) * ( 1./81.*pow(
         m33,2)*RD1456ep*RD3456ep - 4./81.*m13*m33*RD1456ep*RD3456ep + 
         4./81.*pow(m13,2)*RD1456ep*RD3456ep + 2./81.*m11*m33*RD1456ep*
         RD3456ep - 4./81.*m11*m13*RD1456ep*RD3456ep + 1./81.*pow(
         m11,2)*RD1456ep*RD3456ep );

      M13p6x23RV +=  + hhV(7,m13,m11,m11,m13,L,mu2) * ( 1./72.*pow(
         m13,2)*pow(RD1456ep,2) );

      M13p6x23RV +=  + hhV(7,m13,m11,m33,m13,L,mu2) * ( 1./36.*pow(
         m13,2)*RD1456ep*RD3456ep );

      M13p6x23RV +=  + hhV(7,m13,m33,m33,m13,L,mu2) * ( 1./648.*pow(
         m33,2)*pow(RD3456ep,2) + 1./81.*m13*m33*pow(RD3456ep,2) + 25./
         648.*pow(m13,2)*pow(RD3456ep,2) + 1./324.*m11*m33*pow(
         RD3456ep,2) + 1./81.*m11*m13*pow(RD3456ep,2) + 1./648.*pow(
         m11,2)*pow(RD3456ep,2) );

      M13p6x23RV +=  + hh1V(1,m11,m11,m13,m13,L,mu2) * ( 4./27.*m13*
         RD1456ep*RS14563pe - 2./27.*m13*RD1456ep*RC1456pe + 1./6.*m13*
         m33 + 2./27.*m13*m33*pow(RS14563pe,2) + 1./27.*m13*m33*pow(
         RC1456pe,2) + 5./3.*pow(m13,2) + 16./27.*pow(m13,2)*pow(
         RS14563pe,2) - 16./27.*pow(m13,2)*RC1456pe*RS14563pe + 4./27.*
         pow(m13,2)*pow(RC1456pe,2) + 1./6.*m11*m13 - 2./9.*m11*m13*
         pow(RS14563pe,2) + 4./27.*m11*m13*RC1456pe*RS14563pe + 1./27.*
         m11*m13*pow(RC1456pe,2) );

      M13p6x23RV +=  + hh1V(1,m11,m14,m34,m13,L,mu2) * ( 1./18.*m13*m44
         *RS14563pe - 1./18.*m13*m44*RS156pe + 1./9.*m13*m34*RS14563pe
          - 1./9.*m13*m34*RS156pe + 1./18.*m13*m33*RS14563pe - 1./18.*
         m13*m33*RS156pe - 2./9.*m13*m14*RS14563pe + 2./9.*m13*m14*
         RS156pe + 4./9.*pow(m13,2)*RS14563pe - 4./9.*pow(m13,2)*
         RS156pe + 2./9.*m11*m13*RS14563pe - 2./9.*m11*m13*RS156pe );

      M13p6x23RV +=  + hh1V(1,m11,m15,m35,m13,L,mu2) * ( 1./18.*m13*m55
         *RS14563pe - 1./18.*m13*m55*RS146pe + 1./9.*m13*m35*RS14563pe
          - 1./9.*m13*m35*RS146pe + 1./18.*m13*m33*RS14563pe - 1./18.*
         m13*m33*RS146pe - 2./9.*m13*m15*RS14563pe + 2./9.*m13*m15*
         RS146pe + 4./9.*pow(m13,2)*RS14563pe - 4./9.*pow(m13,2)*
         RS146pe + 2./9.*m11*m13*RS14563pe - 2./9.*m11*m13*RS146pe );

      M13p6x23RV +=  + hh1V(1,m11,m16,m36,m13,L,mu2) * ( 1./18.*m13*m66
         *RS14563pe - 1./18.*m13*m66*RS145pe + 1./9.*m13*m36*RS14563pe
          - 1./9.*m13*m36*RS145pe + 1./18.*m13*m33*RS14563pe - 1./18.*
         m13*m33*RS145pe - 2./9.*m13*m16*RS14563pe + 2./9.*m13*m16*
         RS145pe + 4./9.*pow(m13,2)*RS14563pe - 4./9.*pow(m13,2)*
         RS145pe + 2./9.*m11*m13*RS14563pe - 2./9.*m11*m13*RS145pe );

      M13p6x23RV +=  + hh1V(1,m13,m11,m33,m13,L,mu2) * (  - 2./27.*m13*
         RD3456ep*RS14563pe + 1./27.*m13*RC1456pe*RD3456ep - 2./27.*m13
         *RD1456ep*RS34561pe + 1./27.*m13*RD1456ep*RC3456pe + 2./27.*
         m13*m33*RS14563pe*RS34561pe - 2./27.*m13*m33*RC1456pe*
         RS34561pe + 2./27.*m13*m33*RC1456pe*RC3456pe - 16./27.*pow(
         m13,2)*RS14563pe*RS34561pe + 8./27.*pow(m13,2)*RC3456pe*
         RS14563pe + 8./27.*pow(m13,2)*RC1456pe*RS34561pe - 4./27.*pow(
         m13,2)*RC1456pe*RC3456pe + 2./27.*m11*m13*RS14563pe*RS34561pe
          - 2./27.*m11*m13*RC3456pe*RS14563pe + 2./27.*m11*m13*RC1456pe
         *RC3456pe );

      M13p6x23RV +=  + hh1V(1,m13,m11,mp2,m13,L,mu2) * (  - 2./27.*m13*
         RD1456ep*RSp45613e + 1./27.*m13*RD1456ep*RSp45633e + 1./27.*
         m13*RSp45611e*RD1456ep + 4./27.*m13*mp2*RSp45613e*RS14563pe - 
         2./27.*m13*mp2*RSp45633e*RS14563pe - 2./27.*m13*mp2*RC1456pe*
         RSp45613e + 1./27.*m13*mp2*RC1456pe*RSp45633e - 2./27.*m13*mp2
         *RSp45611e*RS14563pe + 1./27.*m13*mp2*RSp45611e*RC1456pe - 2./
         27.*m13*m33*RSp45613e*RS14563pe + 2./27.*m13*m33*RSp45633e*
         RS14563pe + 1./27.*m13*m33*RC1456pe*RSp45633e - 1./27.*m13*m33
         *RSp45611e*RC1456pe - 16./27.*pow(m13,2)*RSp45613e*RS14563pe
          + 8./27.*pow(m13,2)*RSp45633e*RS14563pe + 8./27.*pow(m13,2)*
         RC1456pe*RSp45613e - 4./27.*pow(m13,2)*RC1456pe*RSp45633e + 8./
         27.*pow(m13,2)*RSp45611e*RS14563pe - 4./27.*pow(m13,2)*
         RSp45611e*RC1456pe + 2./27.*m11*m13*RSp45613e*RS14563pe - 2./
         27.*m11*m13*RSp45633e*RS14563pe + 2./27.*m11*m13*RC1456pe*
         RSp45633e - 2./27.*m11*m13*RSp45611e*RC1456pe );

      M13p6x23RV +=  + hh1V(1,m13,m11,me2,m13,L,mu2) * (  - 2./27.*m13*
         RD1456ep*RSe45613p + 1./27.*m13*RD1456ep*RSe45633p + 1./27.*
         m13*RSe45611p*RD1456ep + 4./27.*m13*me2*RSe45613p*RS14563pe - 
         2./27.*m13*me2*RSe45633p*RS14563pe - 2./27.*m13*me2*RC1456pe*
         RSe45613p + 1./27.*m13*me2*RC1456pe*RSe45633p - 2./27.*m13*me2
         *RSe45611p*RS14563pe + 1./27.*m13*me2*RSe45611p*RC1456pe - 2./
         27.*m13*m33*RSe45613p*RS14563pe + 2./27.*m13*m33*RSe45633p*
         RS14563pe + 1./27.*m13*m33*RC1456pe*RSe45633p - 1./27.*m13*m33
         *RSe45611p*RC1456pe - 16./27.*pow(m13,2)*RSe45613p*RS14563pe
          + 8./27.*pow(m13,2)*RSe45633p*RS14563pe + 8./27.*pow(m13,2)*
         RC1456pe*RSe45613p - 4./27.*pow(m13,2)*RC1456pe*RSe45633p + 8./
         27.*pow(m13,2)*RSe45611p*RS14563pe - 4./27.*pow(m13,2)*
         RSe45611p*RC1456pe + 2./27.*m11*m13*RSe45613p*RS14563pe - 2./
         27.*m11*m13*RSe45633p*RS14563pe + 2./27.*m11*m13*RC1456pe*
         RSe45633p - 2./27.*m11*m13*RSe45611p*RC1456pe );

      M13p6x23RV +=  + hh1V(1,m13,m33,m33,m13,L,mu2) * (  - 2./27.*m13*
         RD3456ep*RS34561pe + 1./27.*m13*RD3456ep*RC3456pe - 1./12.*m13
         *m33 + 1./9.*m13*m33*pow(RS34561pe,2) - 2./27.*m13*m33*
         RC3456pe*RS34561pe - 1./54.*m13*m33*pow(RC3456pe,2) - 5./6.*
         pow(m13,2) - 8./27.*pow(m13,2)*pow(RS34561pe,2) + 8./27.*pow(
         m13,2)*RC3456pe*RS34561pe - 2./27.*pow(m13,2)*pow(RC3456pe,2)
          - 1./12.*m11*m13 - 1./27.*m11*m13*pow(RS34561pe,2) - 1./54.*
         m11*m13*pow(RC3456pe,2) );

      M13p6x23RV +=  + hh1V(1,m13,m33,mp2,m13,L,mu2) * (  - 2./27.*m13*
         RD3456ep*RSp45613e + 1./27.*m13*RSp45633e*RD3456ep + 1./27.*
         m13*RSp45611e*RD3456ep + 4./27.*m13*mp2*RSp45613e*RS34561pe - 
         2./27.*m13*mp2*RC3456pe*RSp45613e - 2./27.*m13*mp2*RSp45633e*
         RS34561pe + 1./27.*m13*mp2*RSp45633e*RC3456pe - 2./27.*m13*mp2
         *RSp45611e*RS34561pe + 1./27.*m13*mp2*RSp45611e*RC3456pe + 2./
         27.*m13*m33*RSp45613e*RS34561pe - 2./27.*m13*m33*RSp45633e*
         RC3456pe - 2./27.*m13*m33*RSp45611e*RS34561pe + 2./27.*m13*m33
         *RSp45611e*RC3456pe - 16./27.*pow(m13,2)*RSp45613e*RS34561pe
          + 8./27.*pow(m13,2)*RC3456pe*RSp45613e + 8./27.*pow(m13,2)*
         RSp45633e*RS34561pe - 4./27.*pow(m13,2)*RSp45633e*RC3456pe + 8.
         /27.*pow(m13,2)*RSp45611e*RS34561pe - 4./27.*pow(m13,2)*
         RSp45611e*RC3456pe - 2./27.*m11*m13*RSp45613e*RS34561pe - 1./
         27.*m11*m13*RSp45633e*RC3456pe + 2./27.*m11*m13*RSp45611e*
         RS34561pe + 1./27.*m11*m13*RSp45611e*RC3456pe );

      M13p6x23RV +=  + hh1V(1,m13,m33,me2,m13,L,mu2) * (  - 2./27.*m13*
         RD3456ep*RSe45613p + 1./27.*m13*RSe45633p*RD3456ep + 1./27.*
         m13*RSe45611p*RD3456ep + 4./27.*m13*me2*RSe45613p*RS34561pe - 
         2./27.*m13*me2*RC3456pe*RSe45613p - 2./27.*m13*me2*RSe45633p*
         RS34561pe + 1./27.*m13*me2*RSe45633p*RC3456pe - 2./27.*m13*me2
         *RSe45611p*RS34561pe + 1./27.*m13*me2*RSe45611p*RC3456pe + 2./
         27.*m13*m33*RSe45613p*RS34561pe - 2./27.*m13*m33*RSe45633p*
         RC3456pe - 2./27.*m13*m33*RSe45611p*RS34561pe + 2./27.*m13*m33
         *RSe45611p*RC3456pe - 16./27.*pow(m13,2)*RSe45613p*RS34561pe
          + 8./27.*pow(m13,2)*RC3456pe*RSe45613p + 8./27.*pow(m13,2)*
         RSe45633p*RS34561pe - 4./27.*pow(m13,2)*RSe45633p*RC3456pe + 8.
         /27.*pow(m13,2)*RSe45611p*RS34561pe - 4./27.*pow(m13,2)*
         RSe45611p*RC3456pe - 2./27.*m11*m13*RSe45613p*RS34561pe - 1./
         27.*m11*m13*RSe45633p*RC3456pe + 2./27.*m11*m13*RSe45611p*
         RS34561pe + 1./27.*m11*m13*RSe45611p*RC3456pe );

      M13p6x23RV +=  + hh1V(1,m13,mp2,mp2,m13,L,mu2) * ( 4./27.*m13*mp2
         *pow(RSp45613e,2) - 4./27.*m13*mp2*RSp45633e*RSp45613e + 1./27.
         *m13*mp2*pow(RSp45633e,2) - 4./27.*m13*mp2*RSp45611e*RSp45613e
          + 2./27.*m13*mp2*RSp45611e*RSp45633e + 1./27.*m13*mp2*pow(
         RSp45611e,2) - 1./27.*m13*m33*pow(RSp45613e,2) + 2./27.*m13*
         m33*RSp45633e*RSp45613e - 1./18.*m13*m33*pow(RSp45633e,2) + 1./
         27.*m13*m33*RSp45611e*RSp45633e - 1./54.*m13*m33*pow(
         RSp45611e,2) - 8./27.*pow(m13,2)*pow(RSp45613e,2) + 8./27.*
         pow(m13,2)*RSp45633e*RSp45613e - 2./27.*pow(m13,2)*pow(
         RSp45633e,2) + 8./27.*pow(m13,2)*RSp45611e*RSp45613e - 4./27.*
         pow(m13,2)*RSp45611e*RSp45633e - 2./27.*pow(m13,2)*pow(
         RSp45611e,2) - 1./27.*m11*m13*pow(RSp45613e,2) - 1./54.*m11*
         m13*pow(RSp45633e,2) + 2./27.*m11*m13*RSp45611e*RSp45613e + 1./
         27.*m11*m13*RSp45611e*RSp45633e - 1./18.*m11*m13*pow(
         RSp45611e,2) );

      M13p6x23RV +=  + hh1V(1,m13,me2,mp2,m13,L,mu2) * ( 4./27.*m13*me2
         *RSp45613e*RSe45613p - 2./27.*m13*me2*RSe45633p*RSp45613e - 2./
         27.*m13*me2*RSp45633e*RSe45613p + 1./27.*m13*me2*RSp45633e*
         RSe45633p - 2./27.*m13*me2*RSe45611p*RSp45613e + 1./27.*m13*
         me2*RSe45611p*RSp45633e - 2./27.*m13*me2*RSp45611e*RSe45613p
          + 1./27.*m13*me2*RSp45611e*RSe45633p + 1./27.*m13*me2*
         RSp45611e*RSe45611p + 4./27.*m13*mp2*RSp45613e*RSe45613p - 2./
         27.*m13*mp2*RSe45633p*RSp45613e - 2./27.*m13*mp2*RSp45633e*
         RSe45613p + 1./27.*m13*mp2*RSp45633e*RSe45633p - 2./27.*m13*
         mp2*RSe45611p*RSp45613e + 1./27.*m13*mp2*RSe45611p*RSp45633e
          - 2./27.*m13*mp2*RSp45611e*RSe45613p + 1./27.*m13*mp2*
         RSp45611e*RSe45633p + 1./27.*m13*mp2*RSp45611e*RSe45611p - 2./
         27.*m13*m33*RSp45613e*RSe45613p + 2./27.*m13*m33*RSe45633p*
         RSp45613e + 2./27.*m13*m33*RSp45633e*RSe45613p - 1./9.*m13*m33
         *RSp45633e*RSe45633p + 1./27.*m13*m33*RSe45611p*RSp45633e + 1./
         27.*m13*m33*RSp45611e*RSe45633p );

      M13p6x23RV +=  + hh1V(1,m13,me2,mp2,m13,L,mu2) * (  - 1./27.*m13*
         m33*RSp45611e*RSe45611p - 16./27.*pow(m13,2)*RSp45613e*
         RSe45613p + 8./27.*pow(m13,2)*RSe45633p*RSp45613e + 8./27.*
         pow(m13,2)*RSp45633e*RSe45613p - 4./27.*pow(m13,2)*RSp45633e*
         RSe45633p + 8./27.*pow(m13,2)*RSe45611p*RSp45613e - 4./27.*
         pow(m13,2)*RSe45611p*RSp45633e + 8./27.*pow(m13,2)*RSp45611e*
         RSe45613p - 4./27.*pow(m13,2)*RSp45611e*RSe45633p - 4./27.*
         pow(m13,2)*RSp45611e*RSe45611p - 2./27.*m11*m13*RSp45613e*
         RSe45613p - 1./27.*m11*m13*RSp45633e*RSe45633p + 2./27.*m11*
         m13*RSe45611p*RSp45613e + 1./27.*m11*m13*RSe45611p*RSp45633e
          + 2./27.*m11*m13*RSp45611e*RSe45613p + 1./27.*m11*m13*
         RSp45611e*RSe45633p - 1./9.*m11*m13*RSp45611e*RSe45611p );

      M13p6x23RV +=  + hh1V(1,m13,me2,me2,m13,L,mu2) * ( 4./27.*m13*me2
         *pow(RSe45613p,2) - 4./27.*m13*me2*RSe45633p*RSe45613p + 1./27.
         *m13*me2*pow(RSe45633p,2) - 4./27.*m13*me2*RSe45611p*RSe45613p
          + 2./27.*m13*me2*RSe45611p*RSe45633p + 1./27.*m13*me2*pow(
         RSe45611p,2) - 1./27.*m13*m33*pow(RSe45613p,2) + 2./27.*m13*
         m33*RSe45633p*RSe45613p - 1./18.*m13*m33*pow(RSe45633p,2) + 1./
         27.*m13*m33*RSe45611p*RSe45633p - 1./54.*m13*m33*pow(
         RSe45611p,2) - 8./27.*pow(m13,2)*pow(RSe45613p,2) + 8./27.*
         pow(m13,2)*RSe45633p*RSe45613p - 2./27.*pow(m13,2)*pow(
         RSe45633p,2) + 8./27.*pow(m13,2)*RSe45611p*RSe45613p - 4./27.*
         pow(m13,2)*RSe45611p*RSe45633p - 2./27.*pow(m13,2)*pow(
         RSe45611p,2) - 1./27.*m11*m13*pow(RSe45613p,2) - 1./54.*m11*
         m13*pow(RSe45633p,2) + 2./27.*m11*m13*RSe45611p*RSe45613p + 1./
         27.*m11*m13*RSe45611p*RSe45633p - 1./18.*m11*m13*pow(
         RSe45611p,2) );

      M13p6x23RV +=  + hh1V(1,m14,m11,m34,m13,L,mu2) * (  - 1./9.*m13*
         RD1456ep - 1./18.*m13*m44*RC1456pe - 1./9.*m13*m44*RS156pe + 1.
         /3.*m13*m34*RS14563pe - 1./9.*m13*m34*RC1456pe - 1./9.*m13*m34
         *RS156pe - 1./18.*m13*m33*RS14563pe - 1./18.*m13*m33*RC1456pe
          - 1./18.*m13*m33*RS156pe - 1./3.*m13*m14*RS14563pe + 2./9.*
         m13*m14*RC1456pe - 1./9.*m13*m14*RS156pe + 2./9.*pow(m13,2)*
         RC1456pe - 4./9.*pow(m13,2)*RS156pe + 1./18.*m11*m13*RS14563pe
          - 2./9.*m11*m13*RC1456pe + 1./6.*m11*m13*RS156pe );

      M13p6x23RV +=  + hh1V(1,m14,m33,m34,m13,L,mu2) * ( 1./9.*m13*
         RD3456ep + 1./18.*m13*m44*RC3456pe + 1./9.*m13*m44*RS356pe + 1.
         /3.*m13*m34*RS34561pe - 2./9.*m13*m34*RC3456pe + 1./9.*m13*m34
         *RS356pe - 1./18.*m13*m33*RS34561pe + 2./9.*m13*m33*RC3456pe
          - 1./6.*m13*m33*RS356pe - 1./3.*m13*m14*RS34561pe + 1./9.*m13
         *m14*RC3456pe + 1./9.*m13*m14*RS356pe - 2./9.*pow(m13,2)*
         RC3456pe + 4./9.*pow(m13,2)*RS356pe + 1./18.*m11*m13*RS34561pe
          + 1./18.*m11*m13*RC3456pe + 1./18.*m11*m13*RS356pe );

      M13p6x23RV +=  + hh1V(1,m14,m34,m44,m13,L,mu2) * ( 1./3.*m13*m44
          - 1./9.*m13*m44*RS456pe - 1./3.*m13*m34 + 1./9.*m13*m34*
         RS456pe - 1./6.*m13*m33 + 1./18.*m13*m33*RS456pe - 1./3.*m13*
         m14 + 1./9.*m13*m14*RS456pe + 2./3.*pow(m13,2) - 2./9.*pow(
         m13,2)*RS456pe - 1./6.*m11*m13 + 1./18.*m11*m13*RS456pe );

      M13p6x23RV +=  + hh1V(1,m14,m34,mp2,m13,L,mu2) * (  - 1./9.*m13*
         mp2*RSp45613e + 1./9.*m13*mp2*RSp563e - 1./9.*m13*mp2*
         RSp45611e + 1./3.*m13*mp2*RSp561e - 2./9.*m13*mp2*RSp564e - 1./
         18.*m13*m44*RSp45613e + 1./18.*m13*m44*RSp563e - 1./18.*m13*
         m44*RSp45611e - 1./18.*m13*m44*RSp561e + 1./9.*m13*m44*RSp564e
          + 2./9.*m13*m34*RSp45613e - 2./9.*m13*m34*RSp563e - 1./9.*m13
         *m34*RSp45611e + 1./9.*m13*m34*RSp564e - 1./9.*m13*m33*
         RSp45613e + 1./9.*m13*m33*RSp563e - 1./18.*m13*m33*RSp45611e
          + 1./18.*m13*m33*RSp564e - 1./9.*m13*m14*RSp45613e + 1./9.*
         m13*m14*RSp563e + 2./9.*m13*m14*RSp45611e - 1./3.*m13*m14*
         RSp561e + 1./9.*m13*m14*RSp564e - 4./9.*pow(m13,2)*RSp45613e
          + 4./9.*pow(m13,2)*RSp563e + 2./9.*pow(m13,2)*RSp45611e - 2./
         9.*pow(m13,2)*RSp564e - 1./18.*m11*m13*RSp45613e + 1./18.*m11*
         m13*RSp563e - 1./9.*m11*m13*RSp45611e + 1./18.*m11*m13*RSp561e
          + 1./18.*m11*m13*RSp564e );

      M13p6x23RV +=  + hh1V(1,m14,m34,me2,m13,L,mu2) * (  - 1./9.*m13*
         me2*RSe45613p + 1./9.*m13*me2*RSe563p - 1./9.*m13*me2*
         RSe45611p + 1./3.*m13*me2*RSe561p - 2./9.*m13*me2*RSe564p - 1./
         18.*m13*m44*RSe45613p + 1./18.*m13*m44*RSe563p - 1./18.*m13*
         m44*RSe45611p - 1./18.*m13*m44*RSe561p + 1./9.*m13*m44*RSe564p
          + 2./9.*m13*m34*RSe45613p - 2./9.*m13*m34*RSe563p - 1./9.*m13
         *m34*RSe45611p + 1./9.*m13*m34*RSe564p - 1./9.*m13*m33*
         RSe45613p + 1./9.*m13*m33*RSe563p - 1./18.*m13*m33*RSe45611p
          + 1./18.*m13*m33*RSe564p - 1./9.*m13*m14*RSe45613p + 1./9.*
         m13*m14*RSe563p + 2./9.*m13*m14*RSe45611p - 1./3.*m13*m14*
         RSe561p + 1./9.*m13*m14*RSe564p - 4./9.*pow(m13,2)*RSe45613p
          + 4./9.*pow(m13,2)*RSe563p + 2./9.*pow(m13,2)*RSe45611p - 2./
         9.*pow(m13,2)*RSe564p - 1./18.*m11*m13*RSe45613p + 1./18.*m11*
         m13*RSe563p - 1./9.*m11*m13*RSe45611p + 1./18.*m11*m13*RSe561p
          + 1./18.*m11*m13*RSe564p );

      M13p6x23RV +=  + hh1V(1,m14,m35,m45,m13,L,mu2) * (  - 1./6.*m13*
         m55 + 2./3.*m13*m45 - 1./6.*m13*m44 - 1./3.*m13*m35 - 1./6.*
         m13*m33 - 1./3.*m13*m14 + 2./3.*pow(m13,2) - 1./6.*m11*m13 );

      M13p6x23RV +=  + hh1V(1,m14,m36,m46,m13,L,mu2) * (  - 1./6.*m13*
         m66 + 2./3.*m13*m46 - 1./6.*m13*m44 - 1./3.*m13*m36 - 1./6.*
         m13*m33 - 1./3.*m13*m14 + 2./3.*pow(m13,2) - 1./6.*m11*m13 );

      M13p6x23RV +=  + hh1V(1,m15,m11,m35,m13,L,mu2) * (  - 1./9.*m13*
         RD1456ep - 1./18.*m13*m55*RC1456pe - 1./9.*m13*m55*RS146pe + 1.
         /3.*m13*m35*RS14563pe - 1./9.*m13*m35*RC1456pe - 1./9.*m13*m35
         *RS146pe - 1./18.*m13*m33*RS14563pe - 1./18.*m13*m33*RC1456pe
          - 1./18.*m13*m33*RS146pe - 1./3.*m13*m15*RS14563pe + 2./9.*
         m13*m15*RC1456pe - 1./9.*m13*m15*RS146pe + 2./9.*pow(m13,2)*
         RC1456pe - 4./9.*pow(m13,2)*RS146pe + 1./18.*m11*m13*RS14563pe
          - 2./9.*m11*m13*RC1456pe + 1./6.*m11*m13*RS146pe );

      M13p6x23RV +=  + hh1V(1,m15,m33,m35,m13,L,mu2) * ( 1./9.*m13*
         RD3456ep + 1./18.*m13*m55*RC3456pe + 1./9.*m13*m55*RS346pe + 1.
         /3.*m13*m35*RS34561pe - 2./9.*m13*m35*RC3456pe + 1./9.*m13*m35
         *RS346pe - 1./18.*m13*m33*RS34561pe + 2./9.*m13*m33*RC3456pe
          - 1./6.*m13*m33*RS346pe - 1./3.*m13*m15*RS34561pe + 1./9.*m13
         *m15*RC3456pe + 1./9.*m13*m15*RS346pe - 2./9.*pow(m13,2)*
         RC3456pe + 4./9.*pow(m13,2)*RS346pe + 1./18.*m11*m13*RS34561pe
          + 1./18.*m11*m13*RC3456pe + 1./18.*m11*m13*RS346pe );

      M13p6x23RV +=  + hh1V(1,m15,m34,m45,m13,L,mu2) * (  - 1./6.*m13*
         m55 + 2./3.*m13*m45 - 1./6.*m13*m44 - 1./3.*m13*m34 - 1./6.*
         m13*m33 - 1./3.*m13*m15 + 2./3.*pow(m13,2) - 1./6.*m11*m13 );

      M13p6x23RV +=  + hh1V(1,m15,m35,m55,m13,L,mu2) * ( 1./3.*m13*m55
          - 1./9.*m13*m55*RS546pe - 1./3.*m13*m35 + 1./9.*m13*m35*
         RS546pe - 1./6.*m13*m33 + 1./18.*m13*m33*RS546pe - 1./3.*m13*
         m15 + 1./9.*m13*m15*RS546pe + 2./3.*pow(m13,2) - 2./9.*pow(
         m13,2)*RS546pe - 1./6.*m11*m13 + 1./18.*m11*m13*RS546pe );

      M13p6x23RV +=  + hh1V(1,m15,m35,mp2,m13,L,mu2) * (  - 1./9.*m13*
         mp2*RSp45613e + 1./9.*m13*mp2*RSp463e - 1./9.*m13*mp2*
         RSp45611e + 1./3.*m13*mp2*RSp461e - 2./9.*m13*mp2*RSp465e - 1./
         18.*m13*m55*RSp45613e + 1./18.*m13*m55*RSp463e - 1./18.*m13*
         m55*RSp45611e - 1./18.*m13*m55*RSp461e + 1./9.*m13*m55*RSp465e
          + 2./9.*m13*m35*RSp45613e - 2./9.*m13*m35*RSp463e - 1./9.*m13
         *m35*RSp45611e + 1./9.*m13*m35*RSp465e - 1./9.*m13*m33*
         RSp45613e + 1./9.*m13*m33*RSp463e - 1./18.*m13*m33*RSp45611e
          + 1./18.*m13*m33*RSp465e - 1./9.*m13*m15*RSp45613e + 1./9.*
         m13*m15*RSp463e + 2./9.*m13*m15*RSp45611e - 1./3.*m13*m15*
         RSp461e + 1./9.*m13*m15*RSp465e - 4./9.*pow(m13,2)*RSp45613e
          + 4./9.*pow(m13,2)*RSp463e + 2./9.*pow(m13,2)*RSp45611e - 2./
         9.*pow(m13,2)*RSp465e - 1./18.*m11*m13*RSp45613e + 1./18.*m11*
         m13*RSp463e - 1./9.*m11*m13*RSp45611e + 1./18.*m11*m13*RSp461e
          + 1./18.*m11*m13*RSp465e );

      M13p6x23RV +=  + hh1V(1,m15,m35,me2,m13,L,mu2) * (  - 1./9.*m13*
         me2*RSe45613p + 1./9.*m13*me2*RSe463p - 1./9.*m13*me2*
         RSe45611p + 1./3.*m13*me2*RSe461p - 2./9.*m13*me2*RSe465p - 1./
         18.*m13*m55*RSe45613p + 1./18.*m13*m55*RSe463p - 1./18.*m13*
         m55*RSe45611p - 1./18.*m13*m55*RSe461p + 1./9.*m13*m55*RSe465p
          + 2./9.*m13*m35*RSe45613p - 2./9.*m13*m35*RSe463p - 1./9.*m13
         *m35*RSe45611p + 1./9.*m13*m35*RSe465p - 1./9.*m13*m33*
         RSe45613p + 1./9.*m13*m33*RSe463p - 1./18.*m13*m33*RSe45611p
          + 1./18.*m13*m33*RSe465p - 1./9.*m13*m15*RSe45613p + 1./9.*
         m13*m15*RSe463p + 2./9.*m13*m15*RSe45611p - 1./3.*m13*m15*
         RSe461p + 1./9.*m13*m15*RSe465p - 4./9.*pow(m13,2)*RSe45613p
          + 4./9.*pow(m13,2)*RSe463p + 2./9.*pow(m13,2)*RSe45611p - 2./
         9.*pow(m13,2)*RSe465p - 1./18.*m11*m13*RSe45613p + 1./18.*m11*
         m13*RSe463p - 1./9.*m11*m13*RSe45611p + 1./18.*m11*m13*RSe461p
          + 1./18.*m11*m13*RSe465p );

      M13p6x23RV +=  + hh1V(1,m15,m36,m56,m13,L,mu2) * (  - 1./6.*m13*
         m66 + 2./3.*m13*m56 - 1./6.*m13*m55 - 1./3.*m13*m36 - 1./6.*
         m13*m33 - 1./3.*m13*m15 + 2./3.*pow(m13,2) - 1./6.*m11*m13 );

      M13p6x23RV +=  + hh1V(1,m16,m11,m36,m13,L,mu2) * (  - 1./9.*m13*
         RD1456ep - 1./18.*m13*m66*RC1456pe - 1./9.*m13*m66*RS145pe + 1.
         /3.*m13*m36*RS14563pe - 1./9.*m13*m36*RC1456pe - 1./9.*m13*m36
         *RS145pe - 1./18.*m13*m33*RS14563pe - 1./18.*m13*m33*RC1456pe
          - 1./18.*m13*m33*RS145pe - 1./3.*m13*m16*RS14563pe + 2./9.*
         m13*m16*RC1456pe - 1./9.*m13*m16*RS145pe + 2./9.*pow(m13,2)*
         RC1456pe - 4./9.*pow(m13,2)*RS145pe + 1./18.*m11*m13*RS14563pe
          - 2./9.*m11*m13*RC1456pe + 1./6.*m11*m13*RS145pe );

      M13p6x23RV +=  + hh1V(1,m16,m33,m36,m13,L,mu2) * ( 1./9.*m13*
         RD3456ep + 1./18.*m13*m66*RC3456pe + 1./9.*m13*m66*RS345pe + 1.
         /3.*m13*m36*RS34561pe - 2./9.*m13*m36*RC3456pe + 1./9.*m13*m36
         *RS345pe - 1./18.*m13*m33*RS34561pe + 2./9.*m13*m33*RC3456pe
          - 1./6.*m13*m33*RS345pe - 1./3.*m13*m16*RS34561pe + 1./9.*m13
         *m16*RC3456pe + 1./9.*m13*m16*RS345pe - 2./9.*pow(m13,2)*
         RC3456pe + 4./9.*pow(m13,2)*RS345pe + 1./18.*m11*m13*RS34561pe
          + 1./18.*m11*m13*RC3456pe + 1./18.*m11*m13*RS345pe );

      M13p6x23RV +=  + hh1V(1,m16,m34,m46,m13,L,mu2) * (  - 1./6.*m13*
         m66 + 2./3.*m13*m46 - 1./6.*m13*m44 - 1./3.*m13*m34 - 1./6.*
         m13*m33 - 1./3.*m13*m16 + 2./3.*pow(m13,2) - 1./6.*m11*m13 );

      M13p6x23RV +=  + hh1V(1,m16,m35,m56,m13,L,mu2) * (  - 1./6.*m13*
         m66 + 2./3.*m13*m56 - 1./6.*m13*m55 - 1./3.*m13*m35 - 1./6.*
         m13*m33 - 1./3.*m13*m16 + 2./3.*pow(m13,2) - 1./6.*m11*m13 );

      M13p6x23RV +=  + hh1V(1,m16,m36,m66,m13,L,mu2) * ( 1./3.*m13*m66
          - 1./9.*m13*m66*RS645pe - 1./3.*m13*m36 + 1./9.*m13*m36*
         RS645pe - 1./6.*m13*m33 + 1./18.*m13*m33*RS645pe - 1./3.*m13*
         m16 + 1./9.*m13*m16*RS645pe + 2./3.*pow(m13,2) - 2./9.*pow(
         m13,2)*RS645pe - 1./6.*m11*m13 + 1./18.*m11*m13*RS645pe );

      M13p6x23RV +=  + hh1V(1,m16,m36,mp2,m13,L,mu2) * (  - 1./9.*m13*
         mp2*RSp45613e + 1./9.*m13*mp2*RSp453e - 1./9.*m13*mp2*
         RSp45611e + 1./3.*m13*mp2*RSp451e - 2./9.*m13*mp2*RSp456e - 1./
         18.*m13*m66*RSp45613e + 1./18.*m13*m66*RSp453e - 1./18.*m13*
         m66*RSp45611e - 1./18.*m13*m66*RSp451e + 1./9.*m13*m66*RSp456e
          + 2./9.*m13*m36*RSp45613e - 2./9.*m13*m36*RSp453e - 1./9.*m13
         *m36*RSp45611e + 1./9.*m13*m36*RSp456e - 1./9.*m13*m33*
         RSp45613e + 1./9.*m13*m33*RSp453e - 1./18.*m13*m33*RSp45611e
          + 1./18.*m13*m33*RSp456e - 1./9.*m13*m16*RSp45613e + 1./9.*
         m13*m16*RSp453e + 2./9.*m13*m16*RSp45611e - 1./3.*m13*m16*
         RSp451e + 1./9.*m13*m16*RSp456e - 4./9.*pow(m13,2)*RSp45613e
          + 4./9.*pow(m13,2)*RSp453e + 2./9.*pow(m13,2)*RSp45611e - 2./
         9.*pow(m13,2)*RSp456e - 1./18.*m11*m13*RSp45613e + 1./18.*m11*
         m13*RSp453e - 1./9.*m11*m13*RSp45611e + 1./18.*m11*m13*RSp451e
          + 1./18.*m11*m13*RSp456e );

      M13p6x23RV +=  + hh1V(1,m16,m36,me2,m13,L,mu2) * (  - 1./9.*m13*
         me2*RSe45613p + 1./9.*m13*me2*RSe453p - 1./9.*m13*me2*
         RSe45611p + 1./3.*m13*me2*RSe451p - 2./9.*m13*me2*RSe456p - 1./
         18.*m13*m66*RSe45613p + 1./18.*m13*m66*RSe453p - 1./18.*m13*
         m66*RSe45611p - 1./18.*m13*m66*RSe451p + 1./9.*m13*m66*RSe456p
          + 2./9.*m13*m36*RSe45613p - 2./9.*m13*m36*RSe453p - 1./9.*m13
         *m36*RSe45611p + 1./9.*m13*m36*RSe456p - 1./9.*m13*m33*
         RSe45613p + 1./9.*m13*m33*RSe453p - 1./18.*m13*m33*RSe45611p
          + 1./18.*m13*m33*RSe456p - 1./9.*m13*m16*RSe45613p + 1./9.*
         m13*m16*RSe453p + 2./9.*m13*m16*RSe45611p - 1./3.*m13*m16*
         RSe451p + 1./9.*m13*m16*RSe456p - 4./9.*pow(m13,2)*RSe45613p
          + 4./9.*pow(m13,2)*RSe453p + 2./9.*pow(m13,2)*RSe45611p - 2./
         9.*pow(m13,2)*RSe456p - 1./18.*m11*m13*RSe45613p + 1./18.*m11*
         m13*RSe453p - 1./9.*m11*m13*RSe45611p + 1./18.*m11*m13*RSe451p
          + 1./18.*m11*m13*RSe456p );

      M13p6x23RV +=  + hh1V(1,m33,m14,m34,m13,L,mu2) * ( 1./9.*m13*
         RD3456ep + 1./18.*m13*m44*RS34561pe + 1./18.*m13*m44*RC3456pe
          + 1./18.*m13*m44*RS356pe + 1./9.*m13*m34*RS34561pe - 2./9.*
         m13*m34*RC3456pe + 1./3.*m13*m34*RS356pe + 1./6.*m13*m33*
         RS34561pe + 2./9.*m13*m33*RC3456pe - 7./18.*m13*m33*RS356pe - 
         2./9.*m13*m14*RS34561pe + 1./9.*m13*m14*RC3456pe + 4./9.*pow(
         m13,2)*RS34561pe - 2./9.*pow(m13,2)*RC3456pe + 1./9.*m11*m13*
         RS34561pe + 1./18.*m11*m13*RC3456pe );

      M13p6x23RV +=  + hh1V(1,m33,m15,m35,m13,L,mu2) * ( 1./9.*m13*
         RD3456ep + 1./18.*m13*m55*RS34561pe + 1./18.*m13*m55*RC3456pe
          + 1./18.*m13*m55*RS346pe + 1./9.*m13*m35*RS34561pe - 2./9.*
         m13*m35*RC3456pe + 1./3.*m13*m35*RS346pe + 1./6.*m13*m33*
         RS34561pe + 2./9.*m13*m33*RC3456pe - 7./18.*m13*m33*RS346pe - 
         2./9.*m13*m15*RS34561pe + 1./9.*m13*m15*RC3456pe + 4./9.*pow(
         m13,2)*RS34561pe - 2./9.*pow(m13,2)*RC3456pe + 1./9.*m11*m13*
         RS34561pe + 1./18.*m11*m13*RC3456pe );

      M13p6x23RV +=  + hh1V(1,m33,m16,m36,m13,L,mu2) * ( 1./9.*m13*
         RD3456ep + 1./18.*m13*m66*RS34561pe + 1./18.*m13*m66*RC3456pe
          + 1./18.*m13*m66*RS345pe + 1./9.*m13*m36*RS34561pe - 2./9.*
         m13*m36*RC3456pe + 1./3.*m13*m36*RS345pe + 1./6.*m13*m33*
         RS34561pe + 2./9.*m13*m33*RC3456pe - 7./18.*m13*m33*RS345pe - 
         2./9.*m13*m16*RS34561pe + 1./9.*m13*m16*RC3456pe + 4./9.*pow(
         m13,2)*RS34561pe - 2./9.*pow(m13,2)*RC3456pe + 1./9.*m11*m13*
         RS34561pe + 1./18.*m11*m13*RC3456pe );

      M13p6x23RV +=  + hh1V(1,m34,m14,m44,m13,L,mu2) * ( 1./3.*m13*m44
          - 1./9.*m13*m44*RS456pe - 1./3.*m13*m34 + 1./9.*m13*m34*
         RS456pe - 1./6.*m13*m33 + 1./18.*m13*m33*RS456pe - 1./3.*m13*
         m14 + 1./9.*m13*m14*RS456pe + 2./3.*pow(m13,2) - 2./9.*pow(
         m13,2)*RS456pe - 1./6.*m11*m13 + 1./18.*m11*m13*RS456pe );

      M13p6x23RV +=  + hh1V(1,m34,m14,mp2,m13,L,mu2) * (  - 1./9.*m13*
         mp2*RSp45613e - 1./9.*m13*mp2*RSp45633e + 1./3.*m13*mp2*
         RSp563e + 1./9.*m13*mp2*RSp561e - 2./9.*m13*mp2*RSp564e - 1./
         18.*m13*m44*RSp45613e - 1./18.*m13*m44*RSp45633e - 1./18.*m13*
         m44*RSp563e + 1./18.*m13*m44*RSp561e + 1./9.*m13*m44*RSp564e
          - 1./9.*m13*m34*RSp45613e + 2./9.*m13*m34*RSp45633e - 1./3.*
         m13*m34*RSp563e + 1./9.*m13*m34*RSp561e + 1./9.*m13*m34*
         RSp564e - 1./18.*m13*m33*RSp45613e - 1./9.*m13*m33*RSp45633e
          + 1./18.*m13*m33*RSp563e + 1./18.*m13*m33*RSp561e + 1./18.*
         m13*m33*RSp564e + 2./9.*m13*m14*RSp45613e - 1./9.*m13*m14*
         RSp45633e - 2./9.*m13*m14*RSp561e + 1./9.*m13*m14*RSp564e - 4./
         9.*pow(m13,2)*RSp45613e + 2./9.*pow(m13,2)*RSp45633e + 4./9.*
         pow(m13,2)*RSp561e - 2./9.*pow(m13,2)*RSp564e - 1./9.*m11*m13*
         RSp45613e - 1./18.*m11*m13*RSp45633e + 1./9.*m11*m13*RSp561e
          + 1./18.*m11*m13*RSp564e );

      M13p6x23RV +=  + hh1V(1,m34,m14,me2,m13,L,mu2) * (  - 1./9.*m13*
         me2*RSe45613p - 1./9.*m13*me2*RSe45633p + 1./3.*m13*me2*
         RSe563p + 1./9.*m13*me2*RSe561p - 2./9.*m13*me2*RSe564p - 1./
         18.*m13*m44*RSe45613p - 1./18.*m13*m44*RSe45633p - 1./18.*m13*
         m44*RSe563p + 1./18.*m13*m44*RSe561p + 1./9.*m13*m44*RSe564p
          - 1./9.*m13*m34*RSe45613p + 2./9.*m13*m34*RSe45633p - 1./3.*
         m13*m34*RSe563p + 1./9.*m13*m34*RSe561p + 1./9.*m13*m34*
         RSe564p - 1./18.*m13*m33*RSe45613p - 1./9.*m13*m33*RSe45633p
          + 1./18.*m13*m33*RSe563p + 1./18.*m13*m33*RSe561p + 1./18.*
         m13*m33*RSe564p + 2./9.*m13*m14*RSe45613p - 1./9.*m13*m14*
         RSe45633p - 2./9.*m13*m14*RSe561p + 1./9.*m13*m14*RSe564p - 4./
         9.*pow(m13,2)*RSe45613p + 2./9.*pow(m13,2)*RSe45633p + 4./9.*
         pow(m13,2)*RSe561p - 2./9.*pow(m13,2)*RSe564p - 1./9.*m11*m13*
         RSe45613p - 1./18.*m11*m13*RSe45633p + 1./9.*m11*m13*RSe561p
          + 1./18.*m11*m13*RSe564p );

      M13p6x23RV +=  + hh1V(1,m34,m15,m45,m13,L,mu2) * (  - 1./6.*m13*
         m55 + 2./3.*m13*m45 - 1./6.*m13*m44 - 1./3.*m13*m34 - 1./6.*
         m13*m33 - 1./3.*m13*m15 + 2./3.*pow(m13,2) - 1./6.*m11*m13 );

      M13p6x23RV +=  + hh1V(1,m34,m16,m46,m13,L,mu2) * (  - 1./6.*m13*
         m66 + 2./3.*m13*m46 - 1./6.*m13*m44 - 1./3.*m13*m34 - 1./6.*
         m13*m33 - 1./3.*m13*m16 + 2./3.*pow(m13,2) - 1./6.*m11*m13 );

      M13p6x23RV +=  + hh1V(1,m35,m14,m45,m13,L,mu2) * (  - 1./6.*m13*
         m55 + 2./3.*m13*m45 - 1./6.*m13*m44 - 1./3.*m13*m35 - 1./6.*
         m13*m33 - 1./3.*m13*m14 + 2./3.*pow(m13,2) - 1./6.*m11*m13 );

      M13p6x23RV +=  + hh1V(1,m35,m15,m55,m13,L,mu2) * ( 1./3.*m13*m55
          - 1./9.*m13*m55*RS546pe - 1./3.*m13*m35 + 1./9.*m13*m35*
         RS546pe - 1./6.*m13*m33 + 1./18.*m13*m33*RS546pe - 1./3.*m13*
         m15 + 1./9.*m13*m15*RS546pe + 2./3.*pow(m13,2) - 2./9.*pow(
         m13,2)*RS546pe - 1./6.*m11*m13 + 1./18.*m11*m13*RS546pe );

      M13p6x23RV +=  + hh1V(1,m35,m15,mp2,m13,L,mu2) * (  - 1./9.*m13*
         mp2*RSp45613e - 1./9.*m13*mp2*RSp45633e + 1./3.*m13*mp2*
         RSp463e + 1./9.*m13*mp2*RSp461e - 2./9.*m13*mp2*RSp465e - 1./
         18.*m13*m55*RSp45613e - 1./18.*m13*m55*RSp45633e - 1./18.*m13*
         m55*RSp463e + 1./18.*m13*m55*RSp461e + 1./9.*m13*m55*RSp465e
          - 1./9.*m13*m35*RSp45613e + 2./9.*m13*m35*RSp45633e - 1./3.*
         m13*m35*RSp463e + 1./9.*m13*m35*RSp461e + 1./9.*m13*m35*
         RSp465e - 1./18.*m13*m33*RSp45613e - 1./9.*m13*m33*RSp45633e
          + 1./18.*m13*m33*RSp463e + 1./18.*m13*m33*RSp461e + 1./18.*
         m13*m33*RSp465e + 2./9.*m13*m15*RSp45613e - 1./9.*m13*m15*
         RSp45633e - 2./9.*m13*m15*RSp461e + 1./9.*m13*m15*RSp465e - 4./
         9.*pow(m13,2)*RSp45613e + 2./9.*pow(m13,2)*RSp45633e + 4./9.*
         pow(m13,2)*RSp461e - 2./9.*pow(m13,2)*RSp465e - 1./9.*m11*m13*
         RSp45613e - 1./18.*m11*m13*RSp45633e + 1./9.*m11*m13*RSp461e
          + 1./18.*m11*m13*RSp465e );

      M13p6x23RV +=  + hh1V(1,m35,m15,me2,m13,L,mu2) * (  - 1./9.*m13*
         me2*RSe45613p - 1./9.*m13*me2*RSe45633p + 1./3.*m13*me2*
         RSe463p + 1./9.*m13*me2*RSe461p - 2./9.*m13*me2*RSe465p - 1./
         18.*m13*m55*RSe45613p - 1./18.*m13*m55*RSe45633p - 1./18.*m13*
         m55*RSe463p + 1./18.*m13*m55*RSe461p + 1./9.*m13*m55*RSe465p
          - 1./9.*m13*m35*RSe45613p + 2./9.*m13*m35*RSe45633p - 1./3.*
         m13*m35*RSe463p + 1./9.*m13*m35*RSe461p + 1./9.*m13*m35*
         RSe465p - 1./18.*m13*m33*RSe45613p - 1./9.*m13*m33*RSe45633p
          + 1./18.*m13*m33*RSe463p + 1./18.*m13*m33*RSe461p + 1./18.*
         m13*m33*RSe465p + 2./9.*m13*m15*RSe45613p - 1./9.*m13*m15*
         RSe45633p - 2./9.*m13*m15*RSe461p + 1./9.*m13*m15*RSe465p - 4./
         9.*pow(m13,2)*RSe45613p + 2./9.*pow(m13,2)*RSe45633p + 4./9.*
         pow(m13,2)*RSe461p - 2./9.*pow(m13,2)*RSe465p - 1./9.*m11*m13*
         RSe45613p - 1./18.*m11*m13*RSe45633p + 1./9.*m11*m13*RSe461p
          + 1./18.*m11*m13*RSe465p );

      M13p6x23RV +=  + hh1V(1,m35,m16,m56,m13,L,mu2) * (  - 1./6.*m13*
         m66 + 2./3.*m13*m56 - 1./6.*m13*m55 - 1./3.*m13*m35 - 1./6.*
         m13*m33 - 1./3.*m13*m16 + 2./3.*pow(m13,2) - 1./6.*m11*m13 );

      M13p6x23RV +=  + hh1V(1,m36,m14,m46,m13,L,mu2) * (  - 1./6.*m13*
         m66 + 2./3.*m13*m46 - 1./6.*m13*m44 - 1./3.*m13*m36 - 1./6.*
         m13*m33 - 1./3.*m13*m14 + 2./3.*pow(m13,2) - 1./6.*m11*m13 );

      M13p6x23RV +=  + hh1V(1,m36,m15,m56,m13,L,mu2) * (  - 1./6.*m13*
         m66 + 2./3.*m13*m56 - 1./6.*m13*m55 - 1./3.*m13*m36 - 1./6.*
         m13*m33 - 1./3.*m13*m15 + 2./3.*pow(m13,2) - 1./6.*m11*m13 );

      M13p6x23RV +=  + hh1V(1,m36,m16,m66,m13,L,mu2) * ( 1./3.*m13*m66
          - 1./9.*m13*m66*RS645pe - 1./3.*m13*m36 + 1./9.*m13*m36*
         RS645pe - 1./6.*m13*m33 + 1./18.*m13*m33*RS645pe - 1./3.*m13*
         m16 + 1./9.*m13*m16*RS645pe + 2./3.*pow(m13,2) - 2./9.*pow(
         m13,2)*RS645pe - 1./6.*m11*m13 + 1./18.*m11*m13*RS645pe );

      M13p6x23RV +=  + hh1V(1,m36,m16,mp2,m13,L,mu2) * (  - 1./9.*m13*
         mp2*RSp45613e - 1./9.*m13*mp2*RSp45633e + 1./3.*m13*mp2*
         RSp453e + 1./9.*m13*mp2*RSp451e - 2./9.*m13*mp2*RSp456e - 1./
         18.*m13*m66*RSp45613e - 1./18.*m13*m66*RSp45633e - 1./18.*m13*
         m66*RSp453e + 1./18.*m13*m66*RSp451e + 1./9.*m13*m66*RSp456e
          - 1./9.*m13*m36*RSp45613e + 2./9.*m13*m36*RSp45633e - 1./3.*
         m13*m36*RSp453e + 1./9.*m13*m36*RSp451e + 1./9.*m13*m36*
         RSp456e - 1./18.*m13*m33*RSp45613e - 1./9.*m13*m33*RSp45633e
          + 1./18.*m13*m33*RSp453e + 1./18.*m13*m33*RSp451e + 1./18.*
         m13*m33*RSp456e + 2./9.*m13*m16*RSp45613e - 1./9.*m13*m16*
         RSp45633e - 2./9.*m13*m16*RSp451e + 1./9.*m13*m16*RSp456e - 4./
         9.*pow(m13,2)*RSp45613e + 2./9.*pow(m13,2)*RSp45633e + 4./9.*
         pow(m13,2)*RSp451e - 2./9.*pow(m13,2)*RSp456e - 1./9.*m11*m13*
         RSp45613e - 1./18.*m11*m13*RSp45633e + 1./9.*m11*m13*RSp451e
          + 1./18.*m11*m13*RSp456e );

      M13p6x23RV +=  + hh1V(1,m36,m16,me2,m13,L,mu2) * (  - 1./9.*m13*
         me2*RSe45613p - 1./9.*m13*me2*RSe45633p + 1./3.*m13*me2*
         RSe453p + 1./9.*m13*me2*RSe451p - 2./9.*m13*me2*RSe456p - 1./
         18.*m13*m66*RSe45613p - 1./18.*m13*m66*RSe45633p - 1./18.*m13*
         m66*RSe453p + 1./18.*m13*m66*RSe451p + 1./9.*m13*m66*RSe456p
          - 1./9.*m13*m36*RSe45613p + 2./9.*m13*m36*RSe45633p - 1./3.*
         m13*m36*RSe453p + 1./9.*m13*m36*RSe451p + 1./9.*m13*m36*
         RSe456p - 1./18.*m13*m33*RSe45613p - 1./9.*m13*m33*RSe45633p
          + 1./18.*m13*m33*RSe453p + 1./18.*m13*m33*RSe451p + 1./18.*
         m13*m33*RSe456p + 2./9.*m13*m16*RSe45613p - 1./9.*m13*m16*
         RSe45633p - 2./9.*m13*m16*RSe451p + 1./9.*m13*m16*RSe456p - 4./
         9.*pow(m13,2)*RSe45613p + 2./9.*pow(m13,2)*RSe45633p + 4./9.*
         pow(m13,2)*RSe451p - 2./9.*pow(m13,2)*RSe456p - 1./9.*m11*m13*
         RSe45613p - 1./18.*m11*m13*RSe45633p + 1./9.*m11*m13*RSe451p
          + 1./18.*m11*m13*RSe456p );

      M13p6x23RV +=  + hh1V(2,m11,m11,m13,m13,L,mu2) * (  - 1./27.*m13*
         pow(RD1456ep,2) + 1./27.*m13*m33*RD1456ep*RC1456pe - 8./27.*
         pow(m13,2)*RD1456ep*RS14563pe + 4./27.*pow(m13,2)*RD1456ep*
         RC1456pe + 2./27.*m11*m13*RD1456ep*RS14563pe + 1./27.*m11*m13*
         RD1456ep*RC1456pe );

      M13p6x23RV +=  + hh1V(2,m33,m14,m34,m13,L,mu2) * ( 1./18.*m13*m44
         *RD3456ep - 2./9.*m13*m34*RD3456ep + 2./9.*m13*m33*RD3456ep + 
         1./9.*m13*m14*RD3456ep - 2./9.*pow(m13,2)*RD3456ep + 1./18.*
         m11*m13*RD3456ep );

      M13p6x23RV +=  + hh1V(2,m33,m15,m35,m13,L,mu2) * ( 1./18.*m13*m55
         *RD3456ep - 2./9.*m13*m35*RD3456ep + 2./9.*m13*m33*RD3456ep + 
         1./9.*m13*m15*RD3456ep - 2./9.*pow(m13,2)*RD3456ep + 1./18.*
         m11*m13*RD3456ep );

      M13p6x23RV +=  + hh1V(2,m33,m16,m36,m13,L,mu2) * ( 1./18.*m13*m66
         *RD3456ep - 2./9.*m13*m36*RD3456ep + 2./9.*m13*m33*RD3456ep + 
         1./9.*m13*m16*RD3456ep - 2./9.*pow(m13,2)*RD3456ep + 1./18.*
         m11*m13*RD3456ep );

      M13p6x23RV +=  + hh1V(3,m11,m11,m13,m13,L,mu2) * (  - 1./27.*m13*
         pow(RD1456ep,2) + 1./27.*m13*m33*RD1456ep*RC1456pe - 8./27.*
         pow(m13,2)*RD1456ep*RS14563pe + 4./27.*pow(m13,2)*RD1456ep*
         RC1456pe + 2./27.*m11*m13*RD1456ep*RS14563pe + 1./27.*m11*m13*
         RD1456ep*RC1456pe );

      M13p6x23RV +=  + hh1V(3,m13,m11,m33,m13,L,mu2) * ( 1./27.*m13*
         RD1456ep*RD3456ep - 2./27.*m13*m33*RD1456ep*RS34561pe + 2./27.
         *m13*m33*RD1456ep*RC3456pe + 8./27.*pow(m13,2)*RD1456ep*
         RS34561pe - 4./27.*pow(m13,2)*RD1456ep*RC3456pe + 2./27.*m11*
         m13*RD1456ep*RC3456pe );

      M13p6x23RV +=  + hh1V(3,m13,m11,mp2,m13,L,mu2) * (  - 2./27.*m13*
         mp2*RD1456ep*RSp45613e + 1./27.*m13*mp2*RD1456ep*RSp45633e + 1.
         /27.*m13*mp2*RSp45611e*RD1456ep + 1./27.*m13*m33*RD1456ep*
         RSp45633e - 1./27.*m13*m33*RSp45611e*RD1456ep + 8./27.*pow(
         m13,2)*RD1456ep*RSp45613e - 4./27.*pow(m13,2)*RD1456ep*
         RSp45633e - 4./27.*pow(m13,2)*RSp45611e*RD1456ep + 2./27.*m11*
         m13*RD1456ep*RSp45633e - 2./27.*m11*m13*RSp45611e*RD1456ep );

      M13p6x23RV +=  + hh1V(3,m13,m11,me2,m13,L,mu2) * (  - 2./27.*m13*
         me2*RD1456ep*RSe45613p + 1./27.*m13*me2*RD1456ep*RSe45633p + 1.
         /27.*m13*me2*RSe45611p*RD1456ep + 1./27.*m13*m33*RD1456ep*
         RSe45633p - 1./27.*m13*m33*RSe45611p*RD1456ep + 8./27.*pow(
         m13,2)*RD1456ep*RSe45613p - 4./27.*pow(m13,2)*RD1456ep*
         RSe45633p - 4./27.*pow(m13,2)*RSe45611p*RD1456ep + 2./27.*m11*
         m13*RD1456ep*RSe45633p - 2./27.*m11*m13*RSe45611p*RD1456ep );

      M13p6x23RV +=  + hh1V(3,m13,m33,m33,m13,L,mu2) * ( 23./648.*m13*
         pow(RD3456ep,2) - 2./27.*m13*m33*RD3456ep*RS34561pe - 1./27.*
         m13*m33*RD3456ep*RC3456pe + 8./27.*pow(m13,2)*RD3456ep*
         RS34561pe - 4./27.*pow(m13,2)*RD3456ep*RC3456pe - 1./27.*m11*
         m13*RD3456ep*RC3456pe );

      M13p6x23RV +=  + hh1V(3,m13,m33,mp2,m13,L,mu2) * (  - 2./27.*m13*
         mp2*RD3456ep*RSp45613e + 1./27.*m13*mp2*RSp45633e*RD3456ep + 1.
         /27.*m13*mp2*RSp45611e*RD3456ep - 2./27.*m13*m33*RSp45633e*
         RD3456ep + 2./27.*m13*m33*RSp45611e*RD3456ep + 8./27.*pow(
         m13,2)*RD3456ep*RSp45613e - 4./27.*pow(m13,2)*RSp45633e*
         RD3456ep - 4./27.*pow(m13,2)*RSp45611e*RD3456ep - 1./27.*m11*
         m13*RSp45633e*RD3456ep + 1./27.*m11*m13*RSp45611e*RD3456ep );

      M13p6x23RV +=  + hh1V(3,m13,m33,me2,m13,L,mu2) * (  - 2./27.*m13*
         me2*RD3456ep*RSe45613p + 1./27.*m13*me2*RSe45633p*RD3456ep + 1.
         /27.*m13*me2*RSe45611p*RD3456ep - 2./27.*m13*m33*RSe45633p*
         RD3456ep + 2./27.*m13*m33*RSe45611p*RD3456ep + 8./27.*pow(
         m13,2)*RD3456ep*RSe45613p - 4./27.*pow(m13,2)*RSe45633p*
         RD3456ep - 4./27.*pow(m13,2)*RSe45611p*RD3456ep - 1./27.*m11*
         m13*RSe45633p*RD3456ep + 1./27.*m11*m13*RSe45611p*RD3456ep );

      M13p6x23RV +=  + hh1V(3,m14,m11,m34,m13,L,mu2) * (  - 1./18.*m13*
         m44*RD1456ep - 1./9.*m13*m34*RD1456ep - 1./18.*m13*m33*
         RD1456ep + 2./9.*m13*m14*RD1456ep + 2./9.*pow(m13,2)*RD1456ep
          - 2./9.*m11*m13*RD1456ep );

      M13p6x23RV +=  + hh1V(3,m14,m33,m34,m13,L,mu2) * ( 1./18.*m13*m44
         *RD3456ep - 2./9.*m13*m34*RD3456ep + 2./9.*m13*m33*RD3456ep + 
         1./9.*m13*m14*RD3456ep - 2./9.*pow(m13,2)*RD3456ep + 1./18.*
         m11*m13*RD3456ep );

      M13p6x23RV +=  + hh1V(3,m15,m11,m35,m13,L,mu2) * (  - 1./18.*m13*
         m55*RD1456ep - 1./9.*m13*m35*RD1456ep - 1./18.*m13*m33*
         RD1456ep + 2./9.*m13*m15*RD1456ep + 2./9.*pow(m13,2)*RD1456ep
          - 2./9.*m11*m13*RD1456ep );

      M13p6x23RV +=  + hh1V(3,m15,m33,m35,m13,L,mu2) * ( 1./18.*m13*m55
         *RD3456ep - 2./9.*m13*m35*RD3456ep + 2./9.*m13*m33*RD3456ep + 
         1./9.*m13*m15*RD3456ep - 2./9.*pow(m13,2)*RD3456ep + 1./18.*
         m11*m13*RD3456ep );

      M13p6x23RV +=  + hh1V(3,m16,m11,m36,m13,L,mu2) * (  - 1./18.*m13*
         m66*RD1456ep - 1./9.*m13*m36*RD1456ep - 1./18.*m13*m33*
         RD1456ep + 2./9.*m13*m16*RD1456ep + 2./9.*pow(m13,2)*RD1456ep
          - 2./9.*m11*m13*RD1456ep );

      M13p6x23RV +=  + hh1V(3,m16,m33,m36,m13,L,mu2) * ( 1./18.*m13*m66
         *RD3456ep - 2./9.*m13*m36*RD3456ep + 2./9.*m13*m33*RD3456ep + 
         1./9.*m13*m16*RD3456ep - 2./9.*pow(m13,2)*RD3456ep + 1./18.*
         m11*m13*RD3456ep );

      M13p6x23RV +=  + hh1V(4,m13,m11,m33,m13,L,mu2) * ( 1./27.*m13*
         RD1456ep*RD3456ep + 2./27.*m13*m33*RC1456pe*RD3456ep + 8./27.*
         pow(m13,2)*RD3456ep*RS14563pe - 4./27.*pow(m13,2)*RC1456pe*
         RD3456ep - 2./27.*m11*m13*RD3456ep*RS14563pe + 2./27.*m11*m13*
         RC1456pe*RD3456ep );

      M13p6x23RV +=  + hh1V(4,m13,m33,m33,m13,L,mu2) * ( 1./648.*m13*
         pow(RD3456ep,2) );

      M13p6x23RV +=  + hh1V(5,m11,m11,m13,m13,L,mu2) * ( 1./27.*m13*m33
         *pow(RD1456ep,2) + 4./27.*pow(m13,2)*pow(RD1456ep,2) + 1./27.*
         m11*m13*pow(RD1456ep,2) );

      M13p6x23RV +=  + hh1V(7,m13,m11,m33,m13,L,mu2) * ( 2./27.*m13*m33
         *RD1456ep*RD3456ep - 4./27.*pow(m13,2)*RD1456ep*RD3456ep + 2./
         27.*m11*m13*RD1456ep*RD3456ep );

      M13p6x23RV +=  + hh1V(7,m13,m33,m33,m13,L,mu2) * (  - 1./54.*m13*
         m33*pow(RD3456ep,2) - 2./27.*pow(m13,2)*pow(RD3456ep,2) - 1./
         54.*m11*m13*pow(RD3456ep,2) );

      M13p6x23RV +=  + hh21V(1,m11,m11,m13,m13,L,mu2) * (  - 3./4.*pow(
         m13,2) );

      M13p6x23RV +=  + hh21V(1,m11,m14,m34,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RS14563pe + 1./4.*pow(m13,2)*RS156pe );

      M13p6x23RV +=  + hh21V(1,m11,m15,m35,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RS14563pe + 1./4.*pow(m13,2)*RS146pe );

      M13p6x23RV +=  + hh21V(1,m11,m16,m36,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RS14563pe + 1./4.*pow(m13,2)*RS145pe );

      M13p6x23RV +=  + hh21V(1,m13,m11,m11,m13,L,mu2) * ( 3./8.*pow(
         m13,2) + 1./6.*pow(m13,2)*pow(RS14563pe,2) - 1./6.*pow(m13,2)*
         RC1456pe*RS14563pe + 1./24.*pow(m13,2)*pow(RC1456pe,2) );

      M13p6x23RV +=  + hh21V(1,m13,m11,m33,m13,L,mu2) * ( 1./3.*pow(
         m13,2)*RS14563pe*RS34561pe - 1./6.*pow(m13,2)*RC3456pe*
         RS14563pe - 1./6.*pow(m13,2)*RC1456pe*RS34561pe + 1./12.*pow(
         m13,2)*RC1456pe*RC3456pe );

      M13p6x23RV +=  + hh21V(1,m13,m11,mp2,m13,L,mu2) * ( 1./3.*pow(
         m13,2)*RSp45613e*RS14563pe - 1./6.*pow(m13,2)*RSp45633e*
         RS14563pe - 1./6.*pow(m13,2)*RC1456pe*RSp45613e + 1./12.*pow(
         m13,2)*RC1456pe*RSp45633e - 1./6.*pow(m13,2)*RSp45611e*
         RS14563pe + 1./12.*pow(m13,2)*RSp45611e*RC1456pe );

      M13p6x23RV +=  + hh21V(1,m13,m11,me2,m13,L,mu2) * ( 1./3.*pow(
         m13,2)*RSe45613p*RS14563pe - 1./6.*pow(m13,2)*RSe45633p*
         RS14563pe - 1./6.*pow(m13,2)*RC1456pe*RSe45613p + 1./12.*pow(
         m13,2)*RC1456pe*RSe45633p - 1./6.*pow(m13,2)*RSe45611p*
         RS14563pe + 1./12.*pow(m13,2)*RSe45611p*RC1456pe );

      M13p6x23RV +=  + hh21V(1,m13,m13,m13,m13,L,mu2) * ( 3./4.*pow(
         m13,2) );

      M13p6x23RV +=  + hh21V(1,m13,m33,m33,m13,L,mu2) * ( 3./8.*pow(
         m13,2) + 1./6.*pow(m13,2)*pow(RS34561pe,2) - 1./6.*pow(m13,2)*
         RC3456pe*RS34561pe + 1./24.*pow(m13,2)*pow(RC3456pe,2) );

      M13p6x23RV +=  + hh21V(1,m13,m33,mp2,m13,L,mu2) * ( 1./3.*pow(
         m13,2)*RSp45613e*RS34561pe - 1./6.*pow(m13,2)*RC3456pe*
         RSp45613e - 1./6.*pow(m13,2)*RSp45633e*RS34561pe + 1./12.*pow(
         m13,2)*RSp45633e*RC3456pe - 1./6.*pow(m13,2)*RSp45611e*
         RS34561pe + 1./12.*pow(m13,2)*RSp45611e*RC3456pe );

      M13p6x23RV +=  + hh21V(1,m13,m33,me2,m13,L,mu2) * ( 1./3.*pow(
         m13,2)*RSe45613p*RS34561pe - 1./6.*pow(m13,2)*RC3456pe*
         RSe45613p - 1./6.*pow(m13,2)*RSe45633p*RS34561pe + 1./12.*pow(
         m13,2)*RSe45633p*RC3456pe - 1./6.*pow(m13,2)*RSe45611p*
         RS34561pe + 1./12.*pow(m13,2)*RSe45611p*RC3456pe );

      M13p6x23RV +=  + hh21V(1,m13,mp2,mp2,m13,L,mu2) * ( 1./6.*pow(
         m13,2)*pow(RSp45613e,2) - 1./6.*pow(m13,2)*RSp45633e*RSp45613e
          + 1./24.*pow(m13,2)*pow(RSp45633e,2) - 1./6.*pow(m13,2)*
         RSp45611e*RSp45613e + 1./12.*pow(m13,2)*RSp45611e*RSp45633e + 
         1./24.*pow(m13,2)*pow(RSp45611e,2) );

      M13p6x23RV +=  + hh21V(1,m13,me2,mp2,m13,L,mu2) * ( 1./3.*pow(
         m13,2)*RSp45613e*RSe45613p - 1./6.*pow(m13,2)*RSe45633p*
         RSp45613e - 1./6.*pow(m13,2)*RSp45633e*RSe45613p + 1./12.*pow(
         m13,2)*RSp45633e*RSe45633p - 1./6.*pow(m13,2)*RSe45611p*
         RSp45613e + 1./12.*pow(m13,2)*RSe45611p*RSp45633e - 1./6.*pow(
         m13,2)*RSp45611e*RSe45613p + 1./12.*pow(m13,2)*RSp45611e*
         RSe45633p + 1./12.*pow(m13,2)*RSp45611e*RSe45611p );

      M13p6x23RV +=  + hh21V(1,m13,me2,me2,m13,L,mu2) * ( 1./6.*pow(
         m13,2)*pow(RSe45613p,2) - 1./6.*pow(m13,2)*RSe45633p*RSe45613p
          + 1./24.*pow(m13,2)*pow(RSe45633p,2) - 1./6.*pow(m13,2)*
         RSe45611p*RSe45613p + 1./12.*pow(m13,2)*RSe45611p*RSe45633p + 
         1./24.*pow(m13,2)*pow(RSe45611p,2) );

      M13p6x23RV +=  + hh21V(1,m14,m11,m34,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RS14563pe - 1./4.*pow(m13,2)*RC1456pe + 1./4.*pow(
         m13,2)*RS156pe );

      M13p6x23RV +=  + hh21V(1,m14,m33,m34,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RS34561pe - 1./4.*pow(m13,2)*RS356pe );

      M13p6x23RV +=  + hh21V(1,m14,m34,mp2,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RSp45613e - 1./4.*pow(m13,2)*RSp563e - 1./4.*pow(m13,2)
         *RSp45611e + 1./4.*pow(m13,2)*RSp561e );

      M13p6x23RV +=  + hh21V(1,m14,m34,me2,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RSe45613p - 1./4.*pow(m13,2)*RSe563p - 1./4.*pow(m13,2)
         *RSe45611p + 1./4.*pow(m13,2)*RSe561p );

      M13p6x23RV +=  + hh21V(1,m15,m11,m35,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RS14563pe - 1./4.*pow(m13,2)*RC1456pe + 1./4.*pow(
         m13,2)*RS146pe );

      M13p6x23RV +=  + hh21V(1,m15,m33,m35,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RS34561pe - 1./4.*pow(m13,2)*RS346pe );

      M13p6x23RV +=  + hh21V(1,m15,m35,mp2,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RSp45613e - 1./4.*pow(m13,2)*RSp463e - 1./4.*pow(m13,2)
         *RSp45611e + 1./4.*pow(m13,2)*RSp461e );

      M13p6x23RV +=  + hh21V(1,m15,m35,me2,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RSe45613p - 1./4.*pow(m13,2)*RSe463p - 1./4.*pow(m13,2)
         *RSe45611p + 1./4.*pow(m13,2)*RSe461p );

      M13p6x23RV +=  + hh21V(1,m16,m11,m36,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RS14563pe - 1./4.*pow(m13,2)*RC1456pe + 1./4.*pow(
         m13,2)*RS145pe );

      M13p6x23RV +=  + hh21V(1,m16,m33,m36,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RS34561pe - 1./4.*pow(m13,2)*RS345pe );

      M13p6x23RV +=  + hh21V(1,m16,m36,mp2,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RSp45613e - 1./4.*pow(m13,2)*RSp453e - 1./4.*pow(m13,2)
         *RSp45611e + 1./4.*pow(m13,2)*RSp451e );

      M13p6x23RV +=  + hh21V(1,m16,m36,me2,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RSe45613p - 1./4.*pow(m13,2)*RSe453p - 1./4.*pow(m13,2)
         *RSe45611p + 1./4.*pow(m13,2)*RSe451p );

      M13p6x23RV +=  + hh21V(1,m33,m13,m33,m13,L,mu2) * (  - 3./4.*pow(
         m13,2) );

      M13p6x23RV +=  + hh21V(1,m33,m14,m34,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RS34561pe + 1./4.*pow(m13,2)*RS356pe );

      M13p6x23RV +=  + hh21V(1,m33,m15,m35,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RS34561pe + 1./4.*pow(m13,2)*RS346pe );

      M13p6x23RV +=  + hh21V(1,m33,m16,m36,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RS34561pe + 1./4.*pow(m13,2)*RS345pe );

      M13p6x23RV +=  + hh21V(1,m34,m11,m14,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RS14563pe - 1./4.*pow(m13,2)*RS156pe );

      M13p6x23RV +=  + hh21V(1,m34,m14,m33,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RS34561pe - 1./4.*pow(m13,2)*RC3456pe + 1./4.*pow(
         m13,2)*RS356pe );

      M13p6x23RV +=  + hh21V(1,m34,m14,mp2,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RSp45613e - 1./4.*pow(m13,2)*RSp45633e + 1./4.*pow(
         m13,2)*RSp563e - 1./4.*pow(m13,2)*RSp561e );

      M13p6x23RV +=  + hh21V(1,m34,m14,me2,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RSe45613p - 1./4.*pow(m13,2)*RSe45633p + 1./4.*pow(
         m13,2)*RSe563p - 1./4.*pow(m13,2)*RSe561p );

      M13p6x23RV +=  + hh21V(1,m35,m11,m15,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RS14563pe - 1./4.*pow(m13,2)*RS146pe );

      M13p6x23RV +=  + hh21V(1,m35,m15,m33,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RS34561pe - 1./4.*pow(m13,2)*RC3456pe + 1./4.*pow(
         m13,2)*RS346pe );

      M13p6x23RV +=  + hh21V(1,m35,m15,mp2,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RSp45613e - 1./4.*pow(m13,2)*RSp45633e + 1./4.*pow(
         m13,2)*RSp463e - 1./4.*pow(m13,2)*RSp461e );

      M13p6x23RV +=  + hh21V(1,m35,m15,me2,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RSe45613p - 1./4.*pow(m13,2)*RSe45633p + 1./4.*pow(
         m13,2)*RSe463p - 1./4.*pow(m13,2)*RSe461p );

      M13p6x23RV +=  + hh21V(1,m36,m11,m16,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RS14563pe - 1./4.*pow(m13,2)*RS145pe );

      M13p6x23RV +=  + hh21V(1,m36,m16,m33,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RS34561pe - 1./4.*pow(m13,2)*RC3456pe + 1./4.*pow(
         m13,2)*RS345pe );

      M13p6x23RV +=  + hh21V(1,m36,m16,mp2,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RSp45613e - 1./4.*pow(m13,2)*RSp45633e + 1./4.*pow(
         m13,2)*RSp453e - 1./4.*pow(m13,2)*RSp451e );

      M13p6x23RV +=  + hh21V(1,m36,m16,me2,m13,L,mu2) * ( 1./4.*pow(
         m13,2)*RSe45613p - 1./4.*pow(m13,2)*RSe45633p + 1./4.*pow(
         m13,2)*RSe453p - 1./4.*pow(m13,2)*RSe451p );

      M13p6x23RV +=  + hh21V(1,m44,m14,m34,m13,L,mu2) * ( 3./4.*pow(
         m13,2) - 1./4.*pow(m13,2)*RS456pe );

      M13p6x23RV +=  + hh21V(1,m45,m14,m35,m13,L,mu2) * ( 3./4.*pow(
         m13,2) );

      M13p6x23RV +=  + hh21V(1,m45,m15,m34,m13,L,mu2) * ( 3./4.*pow(
         m13,2) );

      M13p6x23RV +=  + hh21V(1,m46,m14,m36,m13,L,mu2) * ( 3./4.*pow(
         m13,2) );

      M13p6x23RV +=  + hh21V(1,m46,m16,m34,m13,L,mu2) * ( 3./4.*pow(
         m13,2) );

      M13p6x23RV +=  + hh21V(1,m55,m15,m35,m13,L,mu2) * ( 3./4.*pow(
         m13,2) - 1./4.*pow(m13,2)*RS546pe );

      M13p6x23RV +=  + hh21V(1,m56,m15,m36,m13,L,mu2) * ( 3./4.*pow(
         m13,2) );

      M13p6x23RV +=  + hh21V(1,m56,m16,m35,m13,L,mu2) * ( 3./4.*pow(
         m13,2) );

      M13p6x23RV +=  + hh21V(1,m66,m16,m36,m13,L,mu2) * ( 3./4.*pow(
         m13,2) - 1./4.*pow(m13,2)*RS645pe );

      M13p6x23RV +=  + hh21V(1,mp2,m14,m34,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RSp45613e + 1./4.*pow(m13,2)*RSp563e + 1./4.*pow(m13,2)
         *RSp561e - 1./4.*pow(m13,2)*RSp564e );

      M13p6x23RV +=  + hh21V(1,mp2,m15,m35,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RSp45613e + 1./4.*pow(m13,2)*RSp463e + 1./4.*pow(m13,2)
         *RSp461e - 1./4.*pow(m13,2)*RSp465e );

      M13p6x23RV +=  + hh21V(1,mp2,m16,m36,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RSp45613e + 1./4.*pow(m13,2)*RSp453e + 1./4.*pow(m13,2)
         *RSp451e - 1./4.*pow(m13,2)*RSp456e );

      M13p6x23RV +=  + hh21V(1,me2,m14,m34,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RSe45613p + 1./4.*pow(m13,2)*RSe563p + 1./4.*pow(m13,2)
         *RSe561p - 1./4.*pow(m13,2)*RSe564p );

      M13p6x23RV +=  + hh21V(1,me2,m15,m35,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RSe45613p + 1./4.*pow(m13,2)*RSe463p + 1./4.*pow(m13,2)
         *RSe461p - 1./4.*pow(m13,2)*RSe465p );

      M13p6x23RV +=  + hh21V(1,me2,m16,m36,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RSe45613p + 1./4.*pow(m13,2)*RSe453p + 1./4.*pow(m13,2)
         *RSe451p - 1./4.*pow(m13,2)*RSe456p );

      M13p6x23RV +=  + hh21V(3,m13,m11,m11,m13,L,mu2) * (  - 1./6.*pow(
         m13,2)*RD1456ep*RS14563pe + 1./12.*pow(m13,2)*RD1456ep*
         RC1456pe );

      M13p6x23RV +=  + hh21V(3,m13,m11,m33,m13,L,mu2) * (  - 1./6.*pow(
         m13,2)*RD1456ep*RS34561pe + 1./12.*pow(m13,2)*RD1456ep*
         RC3456pe );

      M13p6x23RV +=  + hh21V(3,m13,m11,mp2,m13,L,mu2) * (  - 1./6.*pow(
         m13,2)*RD1456ep*RSp45613e + 1./12.*pow(m13,2)*RD1456ep*
         RSp45633e + 1./12.*pow(m13,2)*RSp45611e*RD1456ep );

      M13p6x23RV +=  + hh21V(3,m13,m11,me2,m13,L,mu2) * (  - 1./6.*pow(
         m13,2)*RD1456ep*RSe45613p + 1./12.*pow(m13,2)*RD1456ep*
         RSe45633p + 1./12.*pow(m13,2)*RSe45611p*RD1456ep );

      M13p6x23RV +=  + hh21V(3,m13,m33,m33,m13,L,mu2) * (  - 1./6.*pow(
         m13,2)*RD3456ep*RS34561pe + 1./12.*pow(m13,2)*RD3456ep*
         RC3456pe );

      M13p6x23RV +=  + hh21V(3,m13,m33,mp2,m13,L,mu2) * (  - 1./6.*pow(
         m13,2)*RD3456ep*RSp45613e + 1./12.*pow(m13,2)*RSp45633e*
         RD3456ep + 1./12.*pow(m13,2)*RSp45611e*RD3456ep );

      M13p6x23RV +=  + hh21V(3,m13,m33,me2,m13,L,mu2) * (  - 1./6.*pow(
         m13,2)*RD3456ep*RSe45613p + 1./12.*pow(m13,2)*RSe45633p*
         RD3456ep + 1./12.*pow(m13,2)*RSe45611p*RD3456ep );

      M13p6x23RV +=  + hh21V(3,m14,m11,m34,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RD1456ep );

      M13p6x23RV +=  + hh21V(3,m15,m11,m35,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RD1456ep );

      M13p6x23RV +=  + hh21V(3,m16,m11,m36,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RD1456ep );

      M13p6x23RV +=  + hh21V(4,m13,m11,m33,m13,L,mu2) * (  - 1./6.*pow(
         m13,2)*RD3456ep*RS14563pe + 1./12.*pow(m13,2)*RC1456pe*
         RD3456ep );

      M13p6x23RV +=  + hh21V(4,m34,m14,m33,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RD3456ep );

      M13p6x23RV +=  + hh21V(4,m35,m15,m33,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RD3456ep );

      M13p6x23RV +=  + hh21V(4,m36,m16,m33,m13,L,mu2) * (  - 1./4.*pow(
         m13,2)*RD3456ep );

      M13p6x23RV +=  + hh21V(7,m13,m11,m11,m13,L,mu2) * ( 1./24.*pow(
         m13,2)*pow(RD1456ep,2) );

      M13p6x23RV +=  + hh21V(7,m13,m11,m33,m13,L,mu2) * ( 1./12.*pow(
         m13,2)*RD1456ep*RD3456ep );

      M13p6x23RV +=  + hh21V(7,m13,m33,m33,m13,L,mu2) * ( 1./24.*pow(
         m13,2)*pow(RD3456ep,2) );

      M13p6x23RV +=  + hh27V(1,m11,m11,m13,m13,L,mu2) * ( 3./4.*m13 );

      M13p6x23RV +=  + hh27V(1,m11,m14,m34,m13,L,mu2) * ( 1./4.*m13*
         RS14563pe - 1./4.*m13*RS156pe );

      M13p6x23RV +=  + hh27V(1,m11,m15,m35,m13,L,mu2) * ( 1./4.*m13*
         RS14563pe - 1./4.*m13*RS146pe );

      M13p6x23RV +=  + hh27V(1,m11,m16,m36,m13,L,mu2) * ( 1./4.*m13*
         RS14563pe - 1./4.*m13*RS145pe );

      M13p6x23RV +=  + hh27V(1,m13,m11,m11,m13,L,mu2) * (  - 3./8.*m13
          - 1./6.*m13*pow(RS14563pe,2) + 1./6.*m13*RC1456pe*RS14563pe
          - 1./24.*m13*pow(RC1456pe,2) );

      M13p6x23RV +=  + hh27V(1,m13,m11,m33,m13,L,mu2) * (  - 1./3.*m13*
         RS14563pe*RS34561pe + 1./6.*m13*RC3456pe*RS14563pe + 1./6.*m13
         *RC1456pe*RS34561pe - 1./12.*m13*RC1456pe*RC3456pe );

      M13p6x23RV +=  + hh27V(1,m13,m11,mp2,m13,L,mu2) * (  - 1./3.*m13*
         RSp45613e*RS14563pe + 1./6.*m13*RSp45633e*RS14563pe + 1./6.*
         m13*RC1456pe*RSp45613e - 1./12.*m13*RC1456pe*RSp45633e + 1./6.
         *m13*RSp45611e*RS14563pe - 1./12.*m13*RSp45611e*RC1456pe );

      M13p6x23RV +=  + hh27V(1,m13,m11,me2,m13,L,mu2) * (  - 1./3.*m13*
         RSe45613p*RS14563pe + 1./6.*m13*RSe45633p*RS14563pe + 1./6.*
         m13*RC1456pe*RSe45613p - 1./12.*m13*RC1456pe*RSe45633p + 1./6.
         *m13*RSe45611p*RS14563pe - 1./12.*m13*RSe45611p*RC1456pe );

      M13p6x23RV +=  + hh27V(1,m13,m13,m13,m13,L,mu2) * (  - 3./4.*m13
          );

      M13p6x23RV +=  + hh27V(1,m13,m33,m33,m13,L,mu2) * (  - 3./8.*m13
          - 1./6.*m13*pow(RS34561pe,2) + 1./6.*m13*RC3456pe*RS34561pe
          - 1./24.*m13*pow(RC3456pe,2) );

      M13p6x23RV +=  + hh27V(1,m13,m33,mp2,m13,L,mu2) * (  - 1./3.*m13*
         RSp45613e*RS34561pe + 1./6.*m13*RC3456pe*RSp45613e + 1./6.*m13
         *RSp45633e*RS34561pe - 1./12.*m13*RSp45633e*RC3456pe + 1./6.*
         m13*RSp45611e*RS34561pe - 1./12.*m13*RSp45611e*RC3456pe );

      M13p6x23RV +=  + hh27V(1,m13,m33,me2,m13,L,mu2) * (  - 1./3.*m13*
         RSe45613p*RS34561pe + 1./6.*m13*RC3456pe*RSe45613p + 1./6.*m13
         *RSe45633p*RS34561pe - 1./12.*m13*RSe45633p*RC3456pe + 1./6.*
         m13*RSe45611p*RS34561pe - 1./12.*m13*RSe45611p*RC3456pe );

      M13p6x23RV +=  + hh27V(1,m13,mp2,mp2,m13,L,mu2) * (  - 1./6.*m13*
         pow(RSp45613e,2) + 1./6.*m13*RSp45633e*RSp45613e - 1./24.*m13*
         pow(RSp45633e,2) + 1./6.*m13*RSp45611e*RSp45613e - 1./12.*m13*
         RSp45611e*RSp45633e - 1./24.*m13*pow(RSp45611e,2) );

      M13p6x23RV +=  + hh27V(1,m13,me2,mp2,m13,L,mu2) * (  - 1./3.*m13*
         RSp45613e*RSe45613p + 1./6.*m13*RSe45633p*RSp45613e + 1./6.*
         m13*RSp45633e*RSe45613p - 1./12.*m13*RSp45633e*RSe45633p + 1./
         6.*m13*RSe45611p*RSp45613e - 1./12.*m13*RSe45611p*RSp45633e + 
         1./6.*m13*RSp45611e*RSe45613p - 1./12.*m13*RSp45611e*RSe45633p
          - 1./12.*m13*RSp45611e*RSe45611p );

      M13p6x23RV +=  + hh27V(1,m13,me2,me2,m13,L,mu2) * (  - 1./6.*m13*
         pow(RSe45613p,2) + 1./6.*m13*RSe45633p*RSe45613p - 1./24.*m13*
         pow(RSe45633p,2) + 1./6.*m13*RSe45611p*RSe45613p - 1./12.*m13*
         RSe45611p*RSe45633p - 1./24.*m13*pow(RSe45611p,2) );

      M13p6x23RV +=  + hh27V(1,m14,m11,m34,m13,L,mu2) * (  - 1./4.*m13*
         RS14563pe + 1./4.*m13*RC1456pe - 1./4.*m13*RS156pe );

      M13p6x23RV +=  + hh27V(1,m14,m33,m34,m13,L,mu2) * (  - 1./4.*m13*
         RS34561pe + 1./4.*m13*RS356pe );

      M13p6x23RV +=  + hh27V(1,m14,m34,mp2,m13,L,mu2) * (  - 1./4.*m13*
         RSp45613e + 1./4.*m13*RSp563e + 1./4.*m13*RSp45611e - 1./4.*
         m13*RSp561e );

      M13p6x23RV +=  + hh27V(1,m14,m34,me2,m13,L,mu2) * (  - 1./4.*m13*
         RSe45613p + 1./4.*m13*RSe563p + 1./4.*m13*RSe45611p - 1./4.*
         m13*RSe561p );

      M13p6x23RV +=  + hh27V(1,m15,m11,m35,m13,L,mu2) * (  - 1./4.*m13*
         RS14563pe + 1./4.*m13*RC1456pe - 1./4.*m13*RS146pe );

      M13p6x23RV +=  + hh27V(1,m15,m33,m35,m13,L,mu2) * (  - 1./4.*m13*
         RS34561pe + 1./4.*m13*RS346pe );

      M13p6x23RV +=  + hh27V(1,m15,m35,mp2,m13,L,mu2) * (  - 1./4.*m13*
         RSp45613e + 1./4.*m13*RSp463e + 1./4.*m13*RSp45611e - 1./4.*
         m13*RSp461e );

      M13p6x23RV +=  + hh27V(1,m15,m35,me2,m13,L,mu2) * (  - 1./4.*m13*
         RSe45613p + 1./4.*m13*RSe463p + 1./4.*m13*RSe45611p - 1./4.*
         m13*RSe461p );

      M13p6x23RV +=  + hh27V(1,m16,m11,m36,m13,L,mu2) * (  - 1./4.*m13*
         RS14563pe + 1./4.*m13*RC1456pe - 1./4.*m13*RS145pe );

      M13p6x23RV +=  + hh27V(1,m16,m33,m36,m13,L,mu2) * (  - 1./4.*m13*
         RS34561pe + 1./4.*m13*RS345pe );

      M13p6x23RV +=  + hh27V(1,m16,m36,mp2,m13,L,mu2) * (  - 1./4.*m13*
         RSp45613e + 1./4.*m13*RSp453e + 1./4.*m13*RSp45611e - 1./4.*
         m13*RSp451e );

      M13p6x23RV +=  + hh27V(1,m16,m36,me2,m13,L,mu2) * (  - 1./4.*m13*
         RSe45613p + 1./4.*m13*RSe453p + 1./4.*m13*RSe45611p - 1./4.*
         m13*RSe451p );

      M13p6x23RV +=  + hh27V(1,m33,m13,m33,m13,L,mu2) * ( 3./4.*m13 );

      M13p6x23RV +=  + hh27V(1,m33,m14,m34,m13,L,mu2) * ( 1./4.*m13*
         RS34561pe - 1./4.*m13*RS356pe );

      M13p6x23RV +=  + hh27V(1,m33,m15,m35,m13,L,mu2) * ( 1./4.*m13*
         RS34561pe - 1./4.*m13*RS346pe );

      M13p6x23RV +=  + hh27V(1,m33,m16,m36,m13,L,mu2) * ( 1./4.*m13*
         RS34561pe - 1./4.*m13*RS345pe );

      M13p6x23RV +=  + hh27V(1,m34,m11,m14,m13,L,mu2) * (  - 1./4.*m13*
         RS14563pe + 1./4.*m13*RS156pe );

      M13p6x23RV +=  + hh27V(1,m34,m14,m33,m13,L,mu2) * (  - 1./4.*m13*
         RS34561pe + 1./4.*m13*RC3456pe - 1./4.*m13*RS356pe );

      M13p6x23RV +=  + hh27V(1,m34,m14,mp2,m13,L,mu2) * (  - 1./4.*m13*
         RSp45613e + 1./4.*m13*RSp45633e - 1./4.*m13*RSp563e + 1./4.*
         m13*RSp561e );

      M13p6x23RV +=  + hh27V(1,m34,m14,me2,m13,L,mu2) * (  - 1./4.*m13*
         RSe45613p + 1./4.*m13*RSe45633p - 1./4.*m13*RSe563p + 1./4.*
         m13*RSe561p );

      M13p6x23RV +=  + hh27V(1,m35,m11,m15,m13,L,mu2) * (  - 1./4.*m13*
         RS14563pe + 1./4.*m13*RS146pe );

      M13p6x23RV +=  + hh27V(1,m35,m15,m33,m13,L,mu2) * (  - 1./4.*m13*
         RS34561pe + 1./4.*m13*RC3456pe - 1./4.*m13*RS346pe );

      M13p6x23RV +=  + hh27V(1,m35,m15,mp2,m13,L,mu2) * (  - 1./4.*m13*
         RSp45613e + 1./4.*m13*RSp45633e - 1./4.*m13*RSp463e + 1./4.*
         m13*RSp461e );

      M13p6x23RV +=  + hh27V(1,m35,m15,me2,m13,L,mu2) * (  - 1./4.*m13*
         RSe45613p + 1./4.*m13*RSe45633p - 1./4.*m13*RSe463p + 1./4.*
         m13*RSe461p );

      M13p6x23RV +=  + hh27V(1,m36,m11,m16,m13,L,mu2) * (  - 1./4.*m13*
         RS14563pe + 1./4.*m13*RS145pe );

      M13p6x23RV +=  + hh27V(1,m36,m16,m33,m13,L,mu2) * (  - 1./4.*m13*
         RS34561pe + 1./4.*m13*RC3456pe - 1./4.*m13*RS345pe );

      M13p6x23RV +=  + hh27V(1,m36,m16,mp2,m13,L,mu2) * (  - 1./4.*m13*
         RSp45613e + 1./4.*m13*RSp45633e - 1./4.*m13*RSp453e + 1./4.*
         m13*RSp451e );

      M13p6x23RV +=  + hh27V(1,m36,m16,me2,m13,L,mu2) * (  - 1./4.*m13*
         RSe45613p + 1./4.*m13*RSe45633p - 1./4.*m13*RSe453p + 1./4.*
         m13*RSe451p );

      M13p6x23RV +=  + hh27V(1,m44,m14,m34,m13,L,mu2) * (  - 3./4.*m13
          + 1./4.*m13*RS456pe );

      M13p6x23RV +=  + hh27V(1,m45,m14,m35,m13,L,mu2) * (  - 3./4.*m13
          );

      M13p6x23RV +=  + hh27V(1,m45,m15,m34,m13,L,mu2) * (  - 3./4.*m13
          );

      M13p6x23RV +=  + hh27V(1,m46,m14,m36,m13,L,mu2) * (  - 3./4.*m13
          );

      M13p6x23RV +=  + hh27V(1,m46,m16,m34,m13,L,mu2) * (  - 3./4.*m13
          );

      M13p6x23RV +=  + hh27V(1,m55,m15,m35,m13,L,mu2) * (  - 3./4.*m13
          + 1./4.*m13*RS546pe );

      M13p6x23RV +=  + hh27V(1,m56,m15,m36,m13,L,mu2) * (  - 3./4.*m13
          );

      M13p6x23RV +=  + hh27V(1,m56,m16,m35,m13,L,mu2) * (  - 3./4.*m13
          );

      M13p6x23RV +=  + hh27V(1,m66,m16,m36,m13,L,mu2) * (  - 3./4.*m13
          + 1./4.*m13*RS645pe );

      M13p6x23RV +=  + hh27V(1,mp2,m14,m34,m13,L,mu2) * ( 1./4.*m13*
         RSp45613e - 1./4.*m13*RSp563e - 1./4.*m13*RSp561e + 1./4.*m13*
         RSp564e );

      M13p6x23RV +=  + hh27V(1,mp2,m15,m35,m13,L,mu2) * ( 1./4.*m13*
         RSp45613e - 1./4.*m13*RSp463e - 1./4.*m13*RSp461e + 1./4.*m13*
         RSp465e );

      M13p6x23RV +=  + hh27V(1,mp2,m16,m36,m13,L,mu2) * ( 1./4.*m13*
         RSp45613e - 1./4.*m13*RSp453e - 1./4.*m13*RSp451e + 1./4.*m13*
         RSp456e );

      M13p6x23RV +=  + hh27V(1,me2,m14,m34,m13,L,mu2) * ( 1./4.*m13*
         RSe45613p - 1./4.*m13*RSe563p - 1./4.*m13*RSe561p + 1./4.*m13*
         RSe564p );

      M13p6x23RV +=  + hh27V(1,me2,m15,m35,m13,L,mu2) * ( 1./4.*m13*
         RSe45613p - 1./4.*m13*RSe463p - 1./4.*m13*RSe461p + 1./4.*m13*
         RSe465p );

      M13p6x23RV +=  + hh27V(1,me2,m16,m36,m13,L,mu2) * ( 1./4.*m13*
         RSe45613p - 1./4.*m13*RSe453p - 1./4.*m13*RSe451p + 1./4.*m13*
         RSe456p );

      M13p6x23RV +=  + hh27V(3,m13,m11,m11,m13,L,mu2) * ( 1./6.*m13*
         RD1456ep*RS14563pe - 1./12.*m13*RD1456ep*RC1456pe );

      M13p6x23RV +=  + hh27V(3,m13,m11,m33,m13,L,mu2) * ( 1./6.*m13*
         RD1456ep*RS34561pe - 1./12.*m13*RD1456ep*RC3456pe );

      M13p6x23RV +=  + hh27V(3,m13,m11,mp2,m13,L,mu2) * ( 1./6.*m13*
         RD1456ep*RSp45613e - 1./12.*m13*RD1456ep*RSp45633e - 1./12.*
         m13*RSp45611e*RD1456ep );

      M13p6x23RV +=  + hh27V(3,m13,m11,me2,m13,L,mu2) * ( 1./6.*m13*
         RD1456ep*RSe45613p - 1./12.*m13*RD1456ep*RSe45633p - 1./12.*
         m13*RSe45611p*RD1456ep );

      M13p6x23RV +=  + hh27V(3,m13,m33,m33,m13,L,mu2) * ( 1./6.*m13*
         RD3456ep*RS34561pe - 1./12.*m13*RD3456ep*RC3456pe );

      M13p6x23RV +=  + hh27V(3,m13,m33,mp2,m13,L,mu2) * ( 1./6.*m13*
         RD3456ep*RSp45613e - 1./12.*m13*RSp45633e*RD3456ep - 1./12.*
         m13*RSp45611e*RD3456ep );

      M13p6x23RV +=  + hh27V(3,m13,m33,me2,m13,L,mu2) * ( 1./6.*m13*
         RD3456ep*RSe45613p - 1./12.*m13*RSe45633p*RD3456ep - 1./12.*
         m13*RSe45611p*RD3456ep );

      M13p6x23RV +=  + hh27V(3,m14,m11,m34,m13,L,mu2) * ( 1./4.*m13*
         RD1456ep );

      M13p6x23RV +=  + hh27V(3,m15,m11,m35,m13,L,mu2) * ( 1./4.*m13*
         RD1456ep );

      M13p6x23RV +=  + hh27V(3,m16,m11,m36,m13,L,mu2) * ( 1./4.*m13*
         RD1456ep );

      M13p6x23RV +=  + hh27V(4,m13,m11,m33,m13,L,mu2) * ( 1./6.*m13*
         RD3456ep*RS14563pe - 1./12.*m13*RC1456pe*RD3456ep );

      M13p6x23RV +=  + hh27V(4,m34,m14,m33,m13,L,mu2) * ( 1./4.*m13*
         RD3456ep );

      M13p6x23RV +=  + hh27V(4,m35,m15,m33,m13,L,mu2) * ( 1./4.*m13*
         RD3456ep );

      M13p6x23RV +=  + hh27V(4,m36,m16,m33,m13,L,mu2) * ( 1./4.*m13*
         RD3456ep );

      M13p6x23RV +=  + hh27V(7,m13,m11,m11,m13,L,mu2) * (  - 1./24.*m13
         *pow(RD1456ep,2) );

      M13p6x23RV +=  + hh27V(7,m13,m11,m33,m13,L,mu2) * (  - 1./12.*m13
         *RD1456ep*RD3456ep );

      M13p6x23RV +=  + hh27V(7,m13,m33,m33,m13,L,mu2) * (  - 1./24.*m13
         *pow(RD3456ep,2) );

  return M13p6x23RV/pow(f0,4);
}


//*********************************************************************
// decay constants
// 1 valence mass 1 sea mass *****************************************


double fv1s1nf3p4V(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 2) || (B0mq.size() != 2)){
    cout << "ERROR: fv1s1nf3p4V probably called with wrong number"
	 << "of quark masses\n";}
  // two masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m14 = (m11+m44)/2.;
  //double mu2 = mu0*mu0;
  double F12p4x11V =
       + AbV(m14,L) * ( 3./2. );

  return F12p4x11V/pow(f0,2);
}

double fv1s1nf3p6V(const quarkmassnf mass, const Linf Liin, const double L){
  return fv1s1nf3p6LV(mass,Liin,L)+fv1s1nf3p6RV(mass,L);
}

double fv1s1nf3p6LV(const quarkmassnf mass, const Linf Liin, const double L){
  double L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu;
  int nfLi;
  Liin.out(L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu,nfLi);
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if (mu0 != mu){
    cout << "ERROR: differing mu's in fv1s1nf3p6L\n";}
  if ( (nq != 2) || (B0mq.size() != 2)){
    cout << "ERROR: fv1s1nf3p6LV probably called with wrong number"
	 << "of quark masses\n";}
  // two masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m14 = (m11+m44)/2.;
  //double mu2 = mu0*mu0;
  double F12p6x11LV =
       + AbV(m11,L) * (  - 4*m44*L3r - 4*m44*L0r - 4./3.*m11*L5r + 8*
         m11*L3r - 10*m11*L2r - 4*m11*L1r + 8*m11*L0r );

      F12p6x11LV +=  + AbV(m14,L) * (  - 18*m44*L4r - 15*m44*L3r - 6*
         m44*L0r + 6*m11*L5r - 15*m11*L3r - 6*m11*L0r );

      F12p6x11LV +=  + AbV(m44,L) * ( 32*m44*L4r - 16*m44*L2r - 64*m44*
         L1r );

      F12p6x11LV +=  + BbV(m11,L) * ( 4./3.*m11*m44*L5r - 4*m11*m44*L3r
          - 4*m11*m44*L0r - 4./3.*pow(m11,2)*L5r + 4*pow(m11,2)*L3r + 4
         *pow(m11,2)*L0r );

      F12p6x11LV +=  + BbV(m14,L) * ( 6*pow(m44,2)*L8r + 36*pow(m44,2)*
         L6r - 3*pow(m44,2)*L5r - 18*pow(m44,2)*L4r + 12*m11*m44*L8r + 
         36*m11*m44*L6r - 6*m11*m44*L5r - 18*m11*m44*L4r + 6*pow(m11,2)
         *L8r - 3*pow(m11,2)*L5r );

      F12p6x11LV +=  + A23bV(m11,L) * (  - 4*L3r + 6*L2r + 12*L1r - 4*
         L0r );

      F12p6x11LV +=  + A23bV(m14,L) * ( 18*L3r + 36*L0r );

      F12p6x11LV +=  + A23bV(m44,L) * ( 48*L2r );

      F12p6x11LV +=  + B23bV(m11,L) * ( 4*m44*L3r + 4*m44*L0r - 4*m11*
         L3r - 4*m11*L0r );

  return F12p6x11LV/pow(f0,4);
}

double fv1s1nf3p6RV(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 2) || (B0mq.size() != 2)){
    cout << "ERROR: fv1s1nf3p6RV probably called with wrong number"
	 << "of quark masses\n";}
  // two masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m14 = (m11+m44)/2.;
  double mu2 = mu0*mu0;

  double F12p6x11RV =
       + Ab(m11,mu2)*BbV(m14,L) * (  - 1./4.*m44 - 1./4.*m11 );

      F12p6x11RV +=  + Bb(m14,mu2)*AbV(m11,L) * (  - 1./4.*m44 - 1./4.*
         m11 );

      F12p6x11RV +=  + AbV(m11,L)*BbV(m14,L) * (  - 1./4.*m44 - 1./4.*
         m11 );

      F12p6x11RV +=  + AbV(m14,L)*pi16 * (  - 3./2.*m44 - 3./8.*m11 );

      F12p6x11RV +=  + hhV(1,m11,m14,m14,m11,L,mu2) * (  - 1./8.*m44 );

      F12p6x11RV +=  + hhV(1,m44,m14,m14,m11,L,mu2) * (  - m44 );

      F12p6x11RV +=  + hhV(2,m11,m14,m14,m11,L,mu2) * (  - 5./36.*m11*
         m44 + 5./36.*pow(m11,2) );

      F12p6x11RV +=  + hh1V(2,m11,m14,m14,m11,L,mu2) * ( 1./72.*m11*m44
          - 1./72.*pow(m11,2) );

      F12p6x11RV +=  + hh1V(3,m14,m11,m14,m11,L,mu2) * ( 1./36.*m11*m44
          - 1./36.*pow(m11,2) );

      F12p6x11RV +=  + hh27V(1,m11,m14,m14,m11,L,mu2) * ( 3./8. );

      F12p6x11RV +=  + hh27V(1,m44,m14,m14,m11,L,mu2) * ( 3 );

      F12p6x11RV +=  + hh27V(2,m11,m14,m14,m11,L,mu2) * ( 3./8.*m44 - 3.
         /8.*m11 );

      F12p6x11RV +=  + hhdV(1,m11,m11,m11,m11,L,mu2) * ( 5./18.*pow(
         m11,2) );

      F12p6x11RV +=  + hhdV(1,m11,m14,m14,m11,L,mu2) * ( 1./8.*m11*m44
          - 1./2.*pow(m11,2) );

      F12p6x11RV +=  + hhdV(1,m44,m14,m14,m11,L,mu2) * ( m11*m44 );

      F12p6x11RV +=  + hhdV(2,m11,m11,m11,m11,L,mu2) * (  - 2./9.*pow(
         m11,2)*m44 + 2./9.*pow(m11,3) );

      F12p6x11RV +=  + hhdV(2,m11,m14,m14,m11,L,mu2) * ( 5./8.*pow(
         m11,2)*m44 - 5./8.*pow(m11,3) );

      F12p6x11RV +=  + hhdV(5,m11,m11,m11,m11,L,mu2) * ( 1./9.*pow(
         m11,2)*pow(m44,2) - 2./9.*pow(m11,3)*m44 + 1./9.*pow(m11,4) );

      F12p6x11RV +=  + hh1dV(2,m11,m14,m14,m11,L,mu2) * (  - pow(m11,2)
         *m44 + pow(m11,3) );

      F12p6x11RV +=  + hh21dV(1,m11,m14,m14,m11,L,mu2) * ( 3./8.*pow(
         m11,2) );

      F12p6x11RV +=  + hh21dV(1,m44,m14,m14,m11,L,mu2) * ( 3*pow(m11,2)
          );

      F12p6x11RV +=  + hh21dV(2,m11,m14,m14,m11,L,mu2) * ( 3./8.*pow(
         m11,2)*m44 - 3./8.*pow(m11,3) );

      F12p6x11RV +=  + hh27dV(1,m11,m14,m14,m11,L,mu2) * (  - 3./8.*m11
          );

      F12p6x11RV +=  + hh27dV(1,m44,m14,m14,m11,L,mu2) * (  - 3*m11 );

      F12p6x11RV +=  + hh27dV(2,m11,m14,m14,m11,L,mu2) * (  - 3./8.*m11
         *m44 + 3./8.*pow(m11,2) );

  return F12p6x11RV/pow(f0,4);
}


// 2 valence mass 1 sea mass *****************************************

double fv2s1nf3p4V(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 3) || (B0mq.size() != 3)){
    cout << "ERROR: fv1s1nf3p4V probably called with wrong number"
	 << "of quark masses\n";}
  // three masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m14 = (m11+m44)/2.;
  double m34 = (m33+m44)/2.;
  //double RN13 = RN(m11,m33);
  double RD14 = m11-m44;
  double RD34 = m33-m44;
  double RS143 = RS(m11,m44,m33);
  double RS341 = RS(m33,m44,m11);
  //double mu2 = mu0*mu0;
  double F13p4x21V =
       + AbV(m11,L) * (  - 1./12. + 1./6.*RS143 );

      F13p4x21V +=  + AbV(m14,L) * ( 3./4. );

      F13p4x21V +=  + AbV(m33,L) * (  - 1./12. + 1./6.*RS341 );

      F13p4x21V +=  + AbV(m34,L) * ( 3./4. );

      F13p4x21V +=  + BbV(m11,L) * (  - 1./12.*RD14 );

      F13p4x21V +=  + BbV(m33,L) * (  - 1./12.*RD34 );

  return F13p4x21V/pow(f0,2);
}

double fv2s1nf3p6V(const quarkmassnf mass, const Linf Liin, const double L){
  return fv2s1nf3p6LV(mass,Liin,L)+fv2s1nf3p6RV(mass,L);
}

double fv2s1nf3p6LV(const quarkmassnf mass, const Linf Liin, const double L){
  double L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu;
  int nfLi;
  Liin.out(L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu,nfLi);
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if (mu0 != mu){
    cout << "ERROR: differing mu's in fv2s1nf3p6LV\n";}
  if ( (nq != 3) || (B0mq.size() != 3)){
    cout << "ERROR: fv2s1nf3p6LV probably called with wrong number"
	 << "of quark masses\n";}
  // three masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m13 = (m11+m33)/2.;
  double m14 = (m11+m44)/2.;
  double m34 = (m33+m44)/2.;
  //double RN13 = RN(m11,m33);
  double RD14 = m11-m44;
  double RD34 = m33-m44;
  double RS143 = RS(m11,m44,m33);
  double RS341 = RS(m33,m44,m11);
  //double mu2 = mu0*mu0;
  double F13p6x21LV =
       + AbV(m11,L) * ( 5./3.*L3r*RD14 + 2./3.*L0r*RD14 - 2./3.*m44*L5r
          + m44*L4r - 2*m44*L4r*RS143 - 1./6.*m33*L5r + 1./3.*m33*L5r*
         RS143 + 1./2.*m11*L5r - m11*L5r*RS143 + 5./3.*m11*L3r + 2./3.*
         m11*L3r*RS143 + 2./3.*m11*L0r + 8./3.*m11*L0r*RS143 );

      F13p6x21LV +=  + AbV(m13,L) * (  - 5*m33*L2r - 2*m33*L1r - 5*m11*
         L2r - 2*m11*L1r );

      F13p6x21LV +=  + AbV(m14,L) * (  - 9*m44*L4r - 15./2.*m44*L3r - 3
         *m44*L0r + 3./2.*m33*L5r + 3./2.*m11*L5r - 15./2.*m11*L3r - 3*
         m11*L0r );

      F13p6x21LV +=  + AbV(m33,L) * ( 5./3.*L3r*RD34 + 2./3.*L0r*RD34
          + 2./3.*m44*L5r + 5*m44*L4r - 6*m44*L4r*RS341 - 4*m44*L4r*
         RS143 + 1./2.*m33*L5r - m33*L5r*RS341 - 4./3.*m33*L5r*RS143 + 
         5./3.*m33*L3r + 2./3.*m33*L3r*RS341 + 2./3.*m33*L0r + 8./3.*
         m33*L0r*RS341 - 1./6.*m11*L5r - m11*L5r*RS341 );

      F13p6x21LV +=  + AbV(m34,L) * (  - 9*m44*L4r - 15./2.*m44*L3r - 3
         *m44*L0r + 3./2.*m33*L5r - 15./2.*m33*L3r - 3*m33*L0r + 3./2.*
         m11*L5r );

      F13p6x21LV +=  + AbV(m44,L) * ( 32*m44*L4r - 16*m44*L2r - 64*m44*
         L1r );

      F13p6x21LV +=  + BbV(m11,m33,L) * (  - 8./3.*pow(m44,2)*L8r - 8*
         pow(m44,2)*L7r - 8*pow(m44,2)*L6r + 8*m33*m44*L7r + 8*m33*m44*
         L6r*RS143 + 8./3.*pow(m33,2)*L8r*RS143 + 8*m11*m44*L7r + 8*m11
         *m44*L6r*RS341 + 4./3.*m11*m44*L5r + 4*m11*m44*L4r - 4*m11*m44
         *L4r*RS341 - 4*m11*m44*L4r*RS143 - 8*m11*m33*L7r - 4./3.*m11*
         m33*L5r*RS143 + 8./3.*pow(m11,2)*L8r*RS341 - 4./3.*pow(m11,2)*
         L5r*RS341 );

      F13p6x21LV +=  + BbV(m11,L) * ( 3*m44*L4r*RD14 + 4./3.*pow(m44,2)
         *L8r + 4*pow(m44,2)*L7r + 4*pow(m44,2)*L6r - 1./6.*m33*L5r*
         RD14 + 1./2.*m11*L5r*RD14 + 5./3.*m11*L3r*RD14 + 2./3.*m11*L0r
         *RD14 - 8*m11*m44*L7r - 8*m11*m44*L6r + 8*m11*m44*L6r*RS143 - 
         2./3.*m11*m44*L5r + 2*m11*m44*L4r - 4*m11*m44*L4r*RS143 - 8./3.
         *pow(m11,2)*L8r + 8./3.*pow(m11,2)*L8r*RS143 + 4*pow(m11,2)*
         L7r + 4./3.*pow(m11,2)*L5r - 4./3.*pow(m11,2)*L5r*RS143 );

      F13p6x21LV +=  + BbV(m14,L) * ( 3*pow(m44,2)*L8r + 18*pow(m44,2)*
         L6r - 3./2.*pow(m44,2)*L5r - 9*pow(m44,2)*L4r + 6*m11*m44*L8r
          + 18*m11*m44*L6r - 3*m11*m44*L5r - 9*m11*m44*L4r + 3*pow(
         m11,2)*L8r - 3./2.*pow(m11,2)*L5r );

      F13p6x21LV +=  + BbV(m33,L) * ( 3*m44*L4r*RD34 + 4./3.*pow(m44,2)
         *L8r + 4*pow(m44,2)*L7r + 4*pow(m44,2)*L6r + 1./2.*m33*L5r*
         RD34 + 5./3.*m33*L3r*RD34 + 2./3.*m33*L0r*RD34 - 8*m33*m44*L7r
          - 8*m33*m44*L6r + 8*m33*m44*L6r*RS341 - 2./3.*m33*m44*L5r + 2
         *m33*m44*L4r - 4*m33*m44*L4r*RS341 - 8./3.*pow(m33,2)*L8r + 8./
         3.*pow(m33,2)*L8r*RS341 + 4*pow(m33,2)*L7r + 4./3.*pow(m33,2)*
         L5r - 4./3.*pow(m33,2)*L5r*RS341 - 1./6.*m11*L5r*RD34 );

      F13p6x21LV +=  + BbV(m34,L) * ( 3*pow(m44,2)*L8r + 18*pow(m44,2)*
         L6r - 3./2.*pow(m44,2)*L5r - 9*pow(m44,2)*L4r + 6*m33*m44*L8r
          + 18*m33*m44*L6r - 3*m33*m44*L5r - 9*m33*m44*L4r + 3*pow(
         m33,2)*L8r - 3./2.*pow(m33,2)*L5r );

      F13p6x21LV +=  + CbV(m11,L) * (  - 8*m11*m44*L6r*RD14 + 4*m11*m44
         *L4r*RD14 - 8./3.*pow(m11,2)*L8r*RD14 + 4./3.*pow(m11,2)*L5r*
         RD14 );

      F13p6x21LV +=  + CbV(m33,L) * (  - 8*m33*m44*L6r*RD34 + 4*m33*m44
         *L4r*RD34 - 8./3.*pow(m33,2)*L8r*RD34 + 4./3.*pow(m33,2)*L5r*
         RD34 );

      F13p6x21LV +=  + A23bV(m11,L) * (  - L3r - 2*L3r*RS143 - 2*L0r );

      F13p6x21LV +=  + A23bV(m13,L) * ( 6*L2r + 12*L1r );

      F13p6x21LV +=  + A23bV(m14,L) * ( 9*L3r + 18*L0r );

      F13p6x21LV +=  + A23bV(m33,L) * (  - L3r - 2*L3r*RS341 - 2*L0r );

      F13p6x21LV +=  + A23bV(m34,L) * ( 9*L3r + 18*L0r );

      F13p6x21LV +=  + A23bV(m44,L) * ( 48*L2r );

      F13p6x21LV +=  + B23bV(m11,L) * (  - L3r*RD14 - 2*L0r*RD14 );

      F13p6x21LV +=  + B23bV(m33,L) * (  - L3r*RD34 - 2*L0r*RD34 );

  return F13p6x21LV/pow(f0,4);
}

double fv2s1nf3p6RV(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 3) || (B0mq.size() != 3)){
    cout << "ERROR: fv2s1nf3p6RV probably called with wrong number"
	 << "of quark masses\n";}
  // three masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m13 = (m11+m33)/2.;
  double m14 = (m11+m44)/2.;
  double m34 = (m33+m44)/2.;
  double RN13 = RN(m11,m33);
  double RD14 = m11-m44;
  double RD34 = m33-m44;
  double RS143 = RS(m11,m44,m33);
  double RS341 = RS(m33,m44,m11);

  double mu2 = mu0*mu0;

  double F13p6x21RV =
       + Ab(m11,mu2)*AbV(m11,L) * ( 19./144. - 1./18.*RS341 - 1./36.*
         RS143 + 1./36.*pow(RS143,2) - 1./18.*m33*RN13*RS341 );

      F13p6x21RV +=  + Ab(m11,mu2)*AbV(m13,L) * (  - 241./648. - 1./324.
         *RS341 - 1./108.*RS143 + 1./162.*RS143*RS341 + 1./162.*pow(
         RS143,2) - 1./2.*m33*RN13 );

      F13p6x21RV +=  + Ab(m11,mu2)*AbV(m14,L) * ( 13./48. - 1./12.*
         RS143 + 1./6.*m44*RN13 + 1./3.*m33*RN13 );

      F13p6x21RV +=  + Ab(m11,mu2)*AbV(m33,L) * ( 1./144. - 1./72.*
         RS341 - 1./72.*RS143 + 1./36.*RS143*RS341 - 1./18.*m33*RN13*
         RS143 );

      F13p6x21RV +=  + Ab(m11,mu2)*AbV(m34,L) * ( 7./48. - 1./12.*RS143
          + 1./6.*m44*RN13 + 1./3.*m33*RN13 );

      F13p6x21RV +=  + Ab(m11,mu2)*AbV(m44,L) * (  - 4./9.*m44*RN13 );

      F13p6x21RV +=  + Ab(m11,mu2)*BbV(m11,m33,L) * (  - 1./18.*m11*
         RS341 );

      F13p6x21RV +=  + Ab(m11,mu2)*BbV(m11,L) * ( 1./144.*RD14 - 1./18.
         *RS341*RD14 - 1./72.*RS143*RD14 - 1./18.*m33*RN13*RS341*RD14
          + 11./36.*m11 - 1./18.*m11*RS143 );

      F13p6x21RV +=  + Ab(m11,mu2)*BbV(m14,L) * ( 1./24.*RD14 - 1./12.*
         m44 - 1./6.*m11 );

      F13p6x21RV +=  + Ab(m11,mu2)*BbV(m33,L) * ( 1./144.*RD34 - 1./72.
         *RS143*RD34 - 1./18.*m33*RN13*RS143*RD34 );

      F13p6x21RV +=  + Ab(m11,mu2)*CbV(m11,L) * ( 1./18.*m11*RD14 );

      F13p6x21RV +=  + Ab(m13,mu2)*AbV(m11,L) * (  - 25./648. - 1./324.
         *RS341 - 1./108.*RS143 + 1./162.*RS143*RS341 + 1./162.*pow(
         RS143,2) );

      F13p6x21RV +=  + Ab(m13,mu2)*AbV(m13,L) * ( 1./4. );

      F13p6x21RV +=  + Ab(m13,mu2)*AbV(m33,L) * (  - 133./648. - 1./108.
         *RS341 + 1./162.*pow(RS341,2) - 1./324.*RS143 + 1./162.*RS143*
         RS341 );

      F13p6x21RV +=  + Ab(m13,mu2)*BbV(m11,m33,L) * (  - 1./6.*m33 - 1./
         3.*m11 );

      F13p6x21RV +=  + Ab(m13,mu2)*BbV(m11,L) * ( 1./324.*RD14 - 1./324.
         *RS341*RD14 - 1./324.*RS143*RD14 );

      F13p6x21RV +=  + Ab(m13,mu2)*BbV(m33,L) * ( 1./324.*RD34 - 1./324.
         *RS341*RD34 - 1./324.*RS143*RD34 );

      F13p6x21RV +=  + Ab(m14,mu2)*AbV(m11,L) * (  - 1./16. - 1./12.*
         RS143 );

      F13p6x21RV +=  + Ab(m14,mu2)*AbV(m14,L) * (  - 9./16. );

      F13p6x21RV +=  + Ab(m14,mu2)*AbV(m33,L) * ( 7./48. - 1./12.*RS341
          );

      F13p6x21RV +=  + Ab(m14,mu2)*AbV(m34,L) * ( 67./144. + 7./72.*
         RS341 + 7./72.*RS143 );

      F13p6x21RV +=  + Ab(m14,mu2)*BbV(m11,m33,L) * ( 1./6.*m44 + 1./3.
         *m11 );

      F13p6x21RV +=  + Ab(m14,mu2)*BbV(m11,L) * ( 7./48.*RD14 - 1./6.*
         m44 - 1./3.*m11 );

      F13p6x21RV +=  + Ab(m14,mu2)*BbV(m33,L) * (  - 1./16.*RD34 );

      F13p6x21RV +=  + Ab(m33,mu2)*AbV(m11,L) * ( 1./144. + 1./24.*
         RS341 - 1./72.*RS143 + 1./36.*RS143*RS341 + 1./18.*m33*RN13*
         RS341 );

      F13p6x21RV +=  + Ab(m33,mu2)*AbV(m13,L) * ( 83./648. - 1./108.*
         RS341 + 1./162.*pow(RS341,2) - 1./324.*RS143 + 1./162.*RS143*
         RS341 + 1./2.*m33*RN13 );

      F13p6x21RV +=  + Ab(m33,mu2)*AbV(m14,L) * (  - 3./16. - 1./12.*
         RS341 - 1./6.*m44*RN13 - 1./3.*m33*RN13 );

      F13p6x21RV +=  + Ab(m33,mu2)*AbV(m33,L) * ( 19./144. - 1./36.*
         RS341 + 1./36.*pow(RS341,2) + 1./18.*m33*RN13*RS143 );

      F13p6x21RV +=  + Ab(m33,mu2)*AbV(m34,L) * (  - 1./16. - 1./12.*
         RS341 - 1./6.*m44*RN13 - 1./3.*m33*RN13 );

      F13p6x21RV +=  + Ab(m33,mu2)*AbV(m44,L) * ( 4./9.*m44*RN13 );

      F13p6x21RV +=  + Ab(m33,mu2)*BbV(m11,m33,L) * (  - 1./18.*m33*
         RS143 );

      F13p6x21RV +=  + Ab(m33,mu2)*BbV(m11,L) * ( 1./144.*RD14 + 1./24.
         *RS341*RD14 + 1./18.*m33*RN13*RS341*RD14 );

      F13p6x21RV +=  + Ab(m33,mu2)*BbV(m33,L) * ( 1./144.*RD34 - 1./72.
         *RS341*RD34 + 11./36.*m33 - 1./18.*m33*RS341 + 1./18.*m33*RN13
         *RS143*RD34 );

      F13p6x21RV +=  + Ab(m33,mu2)*BbV(m34,L) * ( 1./24.*RD34 - 1./12.*
         m44 - 1./6.*m33 );

      F13p6x21RV +=  + Ab(m33,mu2)*CbV(m33,L) * ( 1./18.*m33*RD34 );

      F13p6x21RV +=  + Ab(m34,mu2)*AbV(m11,L) * (  - 1./48. + 1./12.*
         RS143 );

      F13p6x21RV +=  + Ab(m34,mu2)*AbV(m14,L) * ( 67./144. + 7./72.*
         RS341 + 7./72.*RS143 );

      F13p6x21RV +=  + Ab(m34,mu2)*AbV(m33,L) * ( 5./48. - 1./12.*RS341
          - 1./6.*RS143 );

      F13p6x21RV +=  + Ab(m34,mu2)*AbV(m34,L) * (  - 9./16. );

      F13p6x21RV +=  + Ab(m34,mu2)*BbV(m11,m33,L) * ( 1./6.*m44 + 1./6.
         *m33 + 1./6.*m33*RS143 + 1./6.*m11 - 1./6.*m11*RS143 );

      F13p6x21RV +=  + Ab(m34,mu2)*BbV(m11,L) * (  - 1./16.*RD14 );

      F13p6x21RV +=  + Ab(m34,mu2)*BbV(m33,L) * ( 7./48.*RD34 - 1./6.*
         m44 - 1./3.*m33 );

      F13p6x21RV +=  + Ab(m44,mu2)*BbV(m11,m33,L) * (  - 4./9.*m44 );

      F13p6x21RV +=  + Ab(m44,mu2)*BbV(m11,L) * ( 2./9.*m44 );

      F13p6x21RV +=  + Ab(m44,mu2)*BbV(m33,L) * ( 2./9.*m44 );

      F13p6x21RV +=  + Bb(m11,mu2)*AbV(m11,L) * ( 1./144.*RD14 - 1./72.
         *RS143*RD14 + 11./36.*m11 - 1./18.*m11*RS143 );

      F13p6x21RV +=  + Bb(m11,mu2)*AbV(m13,L) * ( 1./324.*RD14 - 1./324.
         *RS341*RD14 - 1./324.*RS143*RD14 );

      F13p6x21RV +=  + Bb(m11,mu2)*AbV(m14,L) * ( 7./48.*RD14 - 1./6.*
         m44 - 1./3.*m11 );

      F13p6x21RV +=  + Bb(m11,mu2)*AbV(m33,L) * ( 1./144.*RD14 - 1./72.
         *RS341*RD14 );

      F13p6x21RV +=  + Bb(m11,mu2)*AbV(m34,L) * (  - 1./16.*RD14 );

      F13p6x21RV +=  + Bb(m11,mu2)*AbV(m44,L) * ( 2./9.*m44 );

      F13p6x21RV +=  + Bb(m11,mu2)*BbV(m11,m33,L) * (  - 1./18.*m11*
         RS341*RD14 );

      F13p6x21RV +=  + Bb(m11,mu2)*BbV(m11,L) * ( 1./144.*pow(RD14,2)
          + 1./9.*m11*RD14 - 1./9.*m11*RS143*RD14 );

      F13p6x21RV +=  + Bb(m11,mu2)*BbV(m33,L) * ( 1./144.*RD14*RD34 );

      F13p6x21RV +=  + Bb(m11,mu2)*CbV(m11,L) * ( 1./18.*m11*pow(
         RD14,2) );

      F13p6x21RV +=  + Bb(m14,mu2)*AbV(m11,L) * ( 1./24.*RD14 - 1./12.*
         m44 - 1./6.*m11 );

      F13p6x21RV +=  + Bb(m33,mu2)*AbV(m11,L) * ( 1./144.*RD34 - 1./72.
         *RS143*RD34 );

      F13p6x21RV +=  + Bb(m33,mu2)*AbV(m13,L) * ( 1./324.*RD34 - 1./324.
         *RS341*RD34 - 1./324.*RS143*RD34 );

      F13p6x21RV +=  + Bb(m33,mu2)*AbV(m14,L) * (  - 1./16.*RD34 );

      F13p6x21RV +=  + Bb(m33,mu2)*AbV(m33,L) * ( 1./144.*RD34 - 1./72.
         *RS341*RD34 + 11./36.*m33 - 1./18.*m33*RS341 );

      F13p6x21RV +=  + Bb(m33,mu2)*AbV(m34,L) * ( 7./48.*RD34 - 1./6.*
         m44 - 1./3.*m33 );

      F13p6x21RV +=  + Bb(m33,mu2)*AbV(m44,L) * ( 2./9.*m44 );

      F13p6x21RV +=  + Bb(m33,mu2)*BbV(m11,m33,L) * (  - 1./18.*m33*
         RS143*RD34 );

      F13p6x21RV +=  + Bb(m33,mu2)*BbV(m11,L) * ( 1./144.*RD14*RD34 );

      F13p6x21RV +=  + Bb(m33,mu2)*BbV(m33,L) * ( 1./144.*pow(RD34,2)
          + 1./9.*m33*RD34 - 1./9.*m33*RS341*RD34 );

      F13p6x21RV +=  + Bb(m33,mu2)*CbV(m33,L) * ( 1./18.*m33*pow(
         RD34,2) );

      F13p6x21RV +=  + Bb(m34,mu2)*AbV(m33,L) * ( 1./24.*RD34 - 1./12.*
         m44 - 1./6.*m33 );

      F13p6x21RV +=  + Cb(m11,mu2)*AbV(m11,L) * ( 1./18.*m11*RD14 );

      F13p6x21RV +=  + Cb(m11,mu2)*BbV(m11,L) * ( 1./18.*m11*pow(
         RD14,2) );

      F13p6x21RV +=  + Cb(m33,mu2)*AbV(m33,L) * ( 1./18.*m33*RD34 );

      F13p6x21RV +=  + Cb(m33,mu2)*BbV(m33,L) * ( 1./18.*m33*pow(
         RD34,2) );

      F13p6x21RV +=  + AbV(m11,L)*pi16 * (  - 1./144.*RD34 - 1./36.*
         RD14 + 1./72.*RS143*RD34 + 1./72.*RS143*RD14 + 1./16.*m44 - 1./
         8.*m44*RS143 + 5./288.*m33 + 1./48.*m33*RS341 - 1./36.*m33*
         RS143 - 1./24.*m33*RS143*RS341 - 1./72.*m33*pow(RS143,2) - 1./
         288.*m11 + 1./144.*m11*RS341 + 1./36.*m11*RS143 - 1./72.*m11*
         RS143*RS341 - 1./24.*m11*pow(RS143,2) );

      F13p6x21RV +=  + pow(AbV(m11,L),2) * ( 19./288. - 1./72.*RS143 + 
         1./72.*pow(RS143,2) );

      F13p6x21RV +=  + AbV(m11,L)*AbV(m13,L) * (  - 25./648. - 1./324.*
         RS341 - 1./108.*RS143 + 1./162.*RS143*RS341 + 1./162.*pow(
         RS143,2) );

      F13p6x21RV +=  + AbV(m11,L)*AbV(m14,L) * (  - 1./16. - 1./12.*
         RS143 );

      F13p6x21RV +=  + AbV(m11,L)*AbV(m33,L) * ( 1./144. - 1./72.*RS341
          - 1./72.*RS143 + 1./36.*RS143*RS341 );

      F13p6x21RV +=  + AbV(m11,L)*AbV(m34,L) * (  - 1./48. + 1./12.*
         RS143 );

      F13p6x21RV +=  + AbV(m11,L)*BbV(m11,m33,L) * (  - 1./18.*m11*
         RS341 );

      F13p6x21RV +=  + AbV(m11,L)*BbV(m11,L) * ( 1./144.*RD14 - 1./72.*
         RS143*RD14 + 11./36.*m11 - 1./18.*m11*RS143 );

      F13p6x21RV +=  + AbV(m11,L)*BbV(m14,L) * ( 1./24.*RD14 - 1./12.*
         m44 - 1./6.*m11 );

      F13p6x21RV +=  + AbV(m11,L)*BbV(m33,L) * ( 1./144.*RD34 - 1./72.*
         RS143*RD34 );

      F13p6x21RV +=  + AbV(m11,L)*CbV(m11,L) * ( 1./18.*m11*RD14 );

      F13p6x21RV +=  + AbV(m13,L)*pi16 * (  - 1./144.*m33 + 1./72.*m33*
         RS341 - 1./144.*m33*pow(RS341,2) + 1./72.*m33*RS143 - 1./72.*
         m33*RS143*RS341 - 1./144.*m33*pow(RS143,2) - 1./144.*m11 + 1./
         72.*m11*RS341 - 1./144.*m11*pow(RS341,2) + 1./72.*m11*RS143 - 
         1./72.*m11*RS143*RS341 - 1./144.*m11*pow(RS143,2) );

      F13p6x21RV +=  + pow(AbV(m13,L),2) * ( 1./8. );

      F13p6x21RV +=  + AbV(m13,L)*AbV(m33,L) * (  - 133./648. - 1./108.
         *RS341 + 1./162.*pow(RS341,2) - 1./324.*RS143 + 1./162.*RS143*
         RS341 );

      F13p6x21RV +=  + AbV(m13,L)*BbV(m11,m33,L) * (  - 1./6.*m33 - 1./
         3.*m11 );

      F13p6x21RV +=  + AbV(m13,L)*BbV(m11,L) * ( 1./324.*RD14 - 1./324.
         *RS341*RD14 - 1./324.*RS143*RD14 );

      F13p6x21RV +=  + AbV(m13,L)*BbV(m33,L) * ( 1./324.*RD34 - 1./324.
         *RS341*RD34 - 1./324.*RS143*RD34 );

      F13p6x21RV +=  + AbV(m14,L)*pi16 * ( 1./16.*RD34 - 21./32.*m44 - 
         1./32.*m44*RS341 - 1./32.*m44*RS143 - 11./32.*m33 + 1./8.*m11
          - 1./32.*m11*RS341 - 1./32.*m11*RS143 );

      F13p6x21RV +=  + pow(AbV(m14,L),2) * (  - 9./32. );

      F13p6x21RV +=  + AbV(m14,L)*AbV(m33,L) * ( 7./48. - 1./12.*RS341
          );

      F13p6x21RV +=  + AbV(m14,L)*AbV(m34,L) * ( 67./144. + 7./72.*
         RS341 + 7./72.*RS143 );

      F13p6x21RV +=  + AbV(m14,L)*BbV(m11,m33,L) * ( 1./6.*m44 + 1./3.*
         m11 );

      F13p6x21RV +=  + AbV(m14,L)*BbV(m11,L) * ( 7./48.*RD14 - 1./6.*
         m44 - 1./3.*m11 );

      F13p6x21RV +=  + AbV(m14,L)*BbV(m33,L) * (  - 1./16.*RD34 );

      F13p6x21RV +=  + AbV(m33,L)*pi16 * (  - 1./36.*RD34 - 1./144.*
         RD14 + 1./72.*RS341*RD34 + 1./72.*RS341*RD14 + 1./16.*m44 - 1./
         8.*m44*RS341 - 1./288.*m33 + 1./36.*m33*RS341 - 1./24.*m33*
         pow(RS341,2) + 1./144.*m33*RS143 - 1./72.*m33*RS143*RS341 + 5./
         288.*m11 - 1./36.*m11*RS341 - 1./72.*m11*pow(RS341,2) + 1./48.
         *m11*RS143 - 1./24.*m11*RS143*RS341 );

      F13p6x21RV +=  + pow(AbV(m33,L),2) * ( 19./288. - 1./72.*RS341 + 
         1./72.*pow(RS341,2) );

      F13p6x21RV +=  + AbV(m33,L)*AbV(m34,L) * ( 5./48. - 1./12.*RS341
          - 1./6.*RS143 );

      F13p6x21RV +=  + AbV(m33,L)*BbV(m11,m33,L) * (  - 1./18.*m33*
         RS143 );

      F13p6x21RV +=  + AbV(m33,L)*BbV(m11,L) * ( 1./144.*RD14 - 1./72.*
         RS341*RD14 );

      F13p6x21RV +=  + AbV(m33,L)*BbV(m33,L) * ( 1./144.*RD34 - 1./72.*
         RS341*RD34 + 11./36.*m33 - 1./18.*m33*RS341 );

      F13p6x21RV +=  + AbV(m33,L)*BbV(m34,L) * ( 1./24.*RD34 - 1./12.*
         m44 - 1./6.*m33 );

      F13p6x21RV +=  + AbV(m33,L)*CbV(m33,L) * ( 1./18.*m33*RD34 );

      F13p6x21RV +=  + AbV(m34,L)*pi16 * ( 1./16.*RD14 - 21./32.*m44 - 
         1./32.*m44*RS341 - 1./32.*m44*RS143 + 1./8.*m33 - 1./32.*m33*
         RS341 - 1./32.*m33*RS143 - 11./32.*m11 );

      F13p6x21RV +=  + pow(AbV(m34,L),2) * (  - 9./32. );

      F13p6x21RV +=  + AbV(m34,L)*BbV(m11,m33,L) * ( 1./6.*m44 + 1./6.*
         m33 + 1./6.*m33*RS143 + 1./6.*m11 - 1./6.*m11*RS143 );

      F13p6x21RV +=  + AbV(m34,L)*BbV(m11,L) * (  - 1./16.*RD14 );

      F13p6x21RV +=  + AbV(m34,L)*BbV(m33,L) * ( 7./48.*RD34 - 1./6.*
         m44 - 1./3.*m33 );

      F13p6x21RV +=  + AbV(m44,L)*BbV(m11,m33,L) * (  - 4./9.*m44 );

      F13p6x21RV +=  + AbV(m44,L)*BbV(m11,L) * ( 2./9.*m44 );

      F13p6x21RV +=  + AbV(m44,L)*BbV(m33,L) * ( 2./9.*m44 );

      F13p6x21RV +=  + BbV(m11,m33,L)*BbV(m11,L) * (  - 1./18.*m11*
         RS341*RD14 );

      F13p6x21RV +=  + BbV(m11,m33,L)*BbV(m33,L) * (  - 1./18.*m33*
         RS143*RD34 );

      F13p6x21RV +=  + BbV(m11,L)*pi16 * (  - 1./144.*RD14*RD34 - 1./
         144.*pow(RD14,2) + 1./16.*m44*RD14 + 5./288.*m33*RD14 + 1./48.
         *m33*RS341*RD14 + 1./144.*m33*RS143*RD14 - 1./288.*m11*RD14 + 
         1./144.*m11*RS341*RD14 + 1./48.*m11*RS143*RD14 );

      F13p6x21RV +=  + pow(BbV(m11,L),2) * ( 1./288.*pow(RD14,2) + 1./
         18.*m11*RD14 - 1./18.*m11*RS143*RD14 );

      F13p6x21RV +=  + BbV(m11,L)*BbV(m33,L) * ( 1./144.*RD14*RD34 );

      F13p6x21RV +=  + BbV(m11,L)*CbV(m11,L) * ( 1./18.*m11*pow(RD14,2)
          );

      F13p6x21RV +=  + BbV(m33,L)*pi16 * (  - 1./144.*pow(RD34,2) - 1./
         144.*RD14*RD34 + 1./16.*m44*RD34 - 1./288.*m33*RD34 + 1./48.*
         m33*RS341*RD34 + 1./144.*m33*RS143*RD34 + 5./288.*m11*RD34 + 1.
         /144.*m11*RS341*RD34 + 1./48.*m11*RS143*RD34 );

      F13p6x21RV +=  + pow(BbV(m33,L),2) * ( 1./288.*pow(RD34,2) + 1./
         18.*m33*RD34 - 1./18.*m33*RS341*RD34 );

      F13p6x21RV +=  + BbV(m33,L)*CbV(m33,L) * ( 1./18.*m33*pow(RD34,2)
          );

      F13p6x21RV +=  + A23bV(m11,L)*pi16 * ( 1./72. - 1./72.*RS341 - 1./
         24.*RS143 + 1./36.*RS143*RS341 + 1./36.*pow(RS143,2) );

      F13p6x21RV +=  + A23bV(m13,L)*pi16 * ( 1./24. - 1./12.*RS341 + 1./
         24.*pow(RS341,2) - 1./12.*RS143 + 1./12.*RS143*RS341 + 1./24.*
         pow(RS143,2) );

      F13p6x21RV +=  + A23bV(m14,L)*pi16 * (  - 3./16. + 3./16.*RS341
          + 3./16.*RS143 );

      F13p6x21RV +=  + A23bV(m33,L)*pi16 * ( 1./72. - 1./24.*RS341 + 1./
         36.*pow(RS341,2) - 1./72.*RS143 + 1./36.*RS143*RS341 );

      F13p6x21RV +=  + A23bV(m34,L)*pi16 * (  - 3./16. + 3./16.*RS341
          + 3./16.*RS143 );

      F13p6x21RV +=  + B23bV(m11,L)*pi16 * ( 1./72.*RD14 - 1./72.*RS341
         *RD14 - 1./72.*RS143*RD14 );

      F13p6x21RV +=  + B23bV(m33,L)*pi16 * ( 1./72.*RD34 - 1./72.*RS341
         *RD34 - 1./72.*RS143*RD34 );

      F13p6x21RV +=  + hhV(1,m11,m11,m13,m13,L,mu2) * ( 1./8.*m11 );

      F13p6x21RV +=  + hhV(1,m11,m14,m34,m13,L,mu2) * (  - 1./8.*m11 + 
         1./8.*m11*RS143 );

      F13p6x21RV +=  + hhV(1,m13,m11,m11,m13,L,mu2) * (  - 5./144.*m33
          + 1./72.*m33*RS143 - 1./72.*m33*pow(RS143,2) - 5./144.*m11 + 
         1./72.*m11*RS143 - 1./72.*m11*pow(RS143,2) );

      F13p6x21RV +=  + hhV(1,m13,m11,m33,m13,L,mu2) * (  - 1./144.*m33
          + 1./72.*m33*RS341 + 1./72.*m33*RS143 - 1./36.*m33*RS143*
         RS341 - 1./144.*m11 + 1./72.*m11*RS341 + 1./72.*m11*RS143 - 1./
         36.*m11*RS143*RS341 );

      F13p6x21RV +=  + hhV(1,m13,m13,m13,m13,L,mu2) * (  - 1./16.*m33
          - 1./16.*m11 );

      F13p6x21RV +=  + hhV(1,m13,m33,m33,m13,L,mu2) * (  - 5./144.*m33
          + 1./72.*m33*RS341 - 1./72.*m33*pow(RS341,2) - 5./144.*m11 + 
         1./72.*m11*RS341 - 1./72.*m11*pow(RS341,2) );

      F13p6x21RV +=  + hhV(1,m14,m11,m34,m13,L,mu2) * (  - 1./16.*m44*
         RS143 - 1./16.*m11*RS143 );

      F13p6x21RV +=  + hhV(1,m14,m33,m34,m13,L,mu2) * ( 1./16.*m44 - 1./
         16.*m44*RS341 + 1./16.*m11 - 1./16.*m11*RS341 );

      F13p6x21RV +=  + hhV(1,m33,m13,m33,m13,L,mu2) * ( 1./8.*m33 );

      F13p6x21RV +=  + hhV(1,m33,m14,m34,m13,L,mu2) * (  - 1./8.*m33 + 
         1./8.*m33*RS341 );

      F13p6x21RV +=  + hhV(1,m34,m11,m14,m13,L,mu2) * ( 1./16.*m44 - 1./
         16.*m44*RS143 + 1./16.*m33 - 1./16.*m33*RS143 );

      F13p6x21RV +=  + hhV(1,m34,m14,m33,m13,L,mu2) * (  - 1./16.*m44*
         RS341 - 1./16.*m33*RS341 );

      F13p6x21RV +=  + hhV(1,m44,m14,m34,m13,L,mu2) * (  - m44 );

      F13p6x21RV +=  + hhV(2,m11,m11,m13,m13,L,mu2) * (  - 1./1296.*
         pow(RD14,2) );

      F13p6x21RV +=  + hhV(3,m11,m11,m13,m13,L,mu2) * ( 1./1296.*pow(
         RD14,2) );

      F13p6x21RV +=  + hhV(3,m13,m11,m11,m13,L,mu2) * (  - 1./144.*m33*
         RD14 + 1./72.*m33*RS143*RD14 - 1./144.*m11*RD14 + 1./72.*m11*
         RS143*RD14 );

      F13p6x21RV +=  + hhV(3,m13,m11,m33,m13,L,mu2) * (  - 1./144.*m33*
         RD14 + 1./72.*m33*RS341*RD14 - 1./144.*m11*RD14 + 1./72.*m11*
         RS341*RD14 );

      F13p6x21RV +=  + hhV(3,m13,m33,m33,m13,L,mu2) * (  - 1./324.*pow(
         RD34,2) - 1./144.*m33*RD34 + 1./72.*m33*RS341*RD34 - 1./144.*
         m11*RD34 + 1./72.*m11*RS341*RD34 );

      F13p6x21RV +=  + hhV(3,m14,m11,m34,m13,L,mu2) * ( 1./16.*m44*RD14
          + 1./16.*m11*RD14 );

      F13p6x21RV +=  + hhV(4,m13,m11,m33,m13,L,mu2) * (  - 1./144.*m33*
         RD34 + 1./72.*m33*RS143*RD34 - 1./144.*m11*RD34 + 1./72.*m11*
         RS143*RD34 );

      F13p6x21RV +=  + hhV(4,m13,m33,m33,m13,L,mu2) * (  - 1./324.*pow(
         RD34,2) );

      F13p6x21RV +=  + hhV(4,m34,m14,m33,m13,L,mu2) * ( 1./16.*m44*RD34
          + 1./16.*m33*RD34 );

      F13p6x21RV +=  + hhV(7,m13,m11,m11,m13,L,mu2) * (  - 1./288.*m33*
         pow(RD14,2) - 1./288.*m11*pow(RD14,2) );

      F13p6x21RV +=  + hhV(7,m13,m11,m33,m13,L,mu2) * (  - 1./144.*m33*
         RD14*RD34 - 1./144.*m11*RD14*RD34 );

      F13p6x21RV +=  + hhV(7,m13,m33,m33,m13,L,mu2) * (  - 5./1296.*m33
         *pow(RD34,2) - 5./1296.*m11*pow(RD34,2) );

      F13p6x21RV +=  + hh1V(2,m33,m13,m33,m13,L,mu2) * ( 1./162.*pow(
         RD34,2) );

      F13p6x21RV +=  + hh1V(3,m13,m33,m33,m13,L,mu2) * ( 1./432.*pow(
         RD34,2) );

      F13p6x21RV +=  + hh1V(4,m13,m33,m33,m13,L,mu2) * ( 5./1296.*pow(
         RD34,2) );

      F13p6x21RV +=  + hh1V(4,m33,m13,m33,m13,L,mu2) * ( 1./162.*pow(
         RD34,2) );

      F13p6x21RV +=  + hh1V(6,m33,m13,m33,m13,L,mu2) * ( 1./1296.*m33*
         pow(RD34,2) + 1./1296.*m11*pow(RD34,2) );

      F13p6x21RV +=  + hh1V(7,m13,m33,m33,m13,L,mu2) * ( 1./2592.*m33*
         pow(RD34,2) + 1./2592.*m11*pow(RD34,2) );

      F13p6x21RV +=  + hh27V(1,m11,m11,m13,m13,L,mu2) * (  - 3./8. );

      F13p6x21RV +=  + hh27V(1,m11,m14,m34,m13,L,mu2) * ( 3./8. - 3./8.
         *RS143 );

      F13p6x21RV +=  + hh27V(1,m13,m11,m11,m13,L,mu2) * ( 5./24. - 1./
         12.*RS143 + 1./12.*pow(RS143,2) );

      F13p6x21RV +=  + hh27V(1,m13,m11,m33,m13,L,mu2) * ( 1./24. - 1./
         12.*RS341 - 1./12.*RS143 + 1./6.*RS143*RS341 );

      F13p6x21RV +=  + hh27V(1,m13,m13,m13,m13,L,mu2) * ( 3./8. );

      F13p6x21RV +=  + hh27V(1,m13,m33,m33,m13,L,mu2) * ( 5./24. - 1./
         12.*RS341 + 1./12.*pow(RS341,2) );

      F13p6x21RV +=  + hh27V(1,m14,m11,m34,m13,L,mu2) * ( 3./8.*RS143 )
         ;

      F13p6x21RV +=  + hh27V(1,m14,m33,m34,m13,L,mu2) * (  - 3./8. + 3./
         8.*RS341 );

      F13p6x21RV +=  + hh27V(1,m33,m13,m33,m13,L,mu2) * (  - 3./8. );

      F13p6x21RV +=  + hh27V(1,m33,m14,m34,m13,L,mu2) * ( 3./8. - 3./8.
         *RS341 );

      F13p6x21RV +=  + hh27V(1,m34,m11,m14,m13,L,mu2) * (  - 3./8. + 3./
         8.*RS143 );

      F13p6x21RV +=  + hh27V(1,m34,m14,m33,m13,L,mu2) * ( 3./8.*RS341 )
         ;

      F13p6x21RV +=  + hh27V(1,m44,m14,m34,m13,L,mu2) * ( 3 );

      F13p6x21RV +=  + hh27V(3,m13,m11,m11,m13,L,mu2) * ( 1./24.*RD14
          - 1./12.*RS143*RD14 );

      F13p6x21RV +=  + hh27V(3,m13,m11,m33,m13,L,mu2) * ( 1./24.*RD14
          - 1./12.*RS341*RD14 );

      F13p6x21RV +=  + hh27V(3,m13,m33,m33,m13,L,mu2) * ( 1./24.*RD34
          - 1./12.*RS341*RD34 );

      F13p6x21RV +=  + hh27V(3,m14,m11,m34,m13,L,mu2) * (  - 3./8.*RD14
          );

      F13p6x21RV +=  + hh27V(4,m13,m11,m33,m13,L,mu2) * ( 1./24.*RD34
          - 1./12.*RS143*RD34 );

      F13p6x21RV +=  + hh27V(4,m34,m14,m33,m13,L,mu2) * (  - 3./8.*RD34
          );

      F13p6x21RV +=  + hh27V(7,m13,m11,m11,m13,L,mu2) * ( 1./48.*pow(
         RD14,2) );

      F13p6x21RV +=  + hh27V(7,m13,m11,m33,m13,L,mu2) * ( 1./24.*RD14*
         RD34 );

      F13p6x21RV +=  + hh27V(7,m13,m33,m33,m13,L,mu2) * ( 1./48.*pow(
         RD34,2) );

      F13p6x21RV +=  + hhdV(1,m11,m11,m13,m13,L,mu2) * ( 1./648.*pow(
         RD14,2) - 1./162.*m33*RS143*RD14 - 5./72.*pow(m33,2) + 1./36.*
         pow(m33,2)*RS143 - 7./324.*pow(m33,2)*pow(RS143,2) - 1./81.*
         m11*RS143*RD14 - 29./144.*m11*m33 + 1./18.*m11*m33*RS143 - 5./
         162.*m11*m33*pow(RS143,2) - 19./144.*pow(m11,2) + 1./36.*pow(
         m11,2)*RS143 - 1./324.*pow(m11,2)*pow(RS143,2) );

      F13p6x21RV +=  + hhdV(1,m11,m13,m33,m13,L,mu2) * ( 1./324.*RD14*
         RD34 + 1./162.*m33*RS341*RD14 + 1./81.*m33*RS143*RD34 + 2./81.
         *pow(m33,2)*RS143*RS341 + 1./81.*m11*RS341*RD14 + 1./162.*m11*
         RS143*RD34 + 5./81.*m11*m33*RS143*RS341 + 2./81.*pow(m11,2)*
         RS143*RS341 );

      F13p6x21RV +=  + hhdV(1,m11,m14,m34,m13,L,mu2) * ( 1./8.*pow(
         m33,2) - 1./8.*pow(m33,2)*RS143 + 5./16.*m11*m33 - 5./16.*m11*
         m33*RS143 + 3./16.*pow(m11,2) - 3./16.*pow(m11,2)*RS143 );

      F13p6x21RV +=  + hhdV(1,m13,m11,m11,m13,L,mu2) * ( 5./288.*pow(
         m33,2) - 1./144.*pow(m33,2)*RS143 + 1./144.*pow(m33,2)*pow(
         RS143,2) + 5./144.*m11*m33 - 1./72.*m11*m33*RS143 + 1./72.*m11
         *m33*pow(RS143,2) + 5./288.*pow(m11,2) - 1./144.*pow(m11,2)*
         RS143 + 1./144.*pow(m11,2)*pow(RS143,2) );

      F13p6x21RV +=  + hhdV(1,m13,m11,m33,m13,L,mu2) * ( 1./288.*pow(
         m33,2) - 1./144.*pow(m33,2)*RS341 - 1./144.*pow(m33,2)*RS143
          + 1./72.*pow(m33,2)*RS143*RS341 + 1./144.*m11*m33 - 1./72.*
         m11*m33*RS341 - 1./72.*m11*m33*RS143 + 1./36.*m11*m33*RS143*
         RS341 + 1./288.*pow(m11,2) - 1./144.*pow(m11,2)*RS341 - 1./144.
         *pow(m11,2)*RS143 + 1./72.*pow(m11,2)*RS143*RS341 );

      F13p6x21RV +=  + hhdV(1,m13,m13,m13,m13,L,mu2) * ( 1./32.*pow(
         m33,2) + 1./16.*m11*m33 + 1./32.*pow(m11,2) );

      F13p6x21RV +=  + hhdV(1,m13,m33,m33,m13,L,mu2) * ( 1./648.*pow(
         RD34,2) - 1./108.*m33*RD34 + 1./162.*m33*RS341*RD34 + 25./288.
         *pow(m33,2) + 1./432.*pow(m33,2)*RS341 + 17./1296.*pow(m33,2)*
         pow(RS341,2) - 1./108.*m11*RD34 + 1./81.*m11*RS341*RD34 + 25./
         144.*m11*m33 - 1./72.*m11*m33*RS341 + 25./648.*m11*m33*pow(
         RS341,2) + 25./288.*pow(m11,2) - 7./432.*pow(m11,2)*RS341 + 41.
         /1296.*pow(m11,2)*pow(RS341,2) );

      F13p6x21RV +=  + hhdV(1,m14,m11,m34,m13,L,mu2) * ( 1./32.*m33*m44
         *RS143 + 1./32.*m11*m44*RS143 + 1./32.*m11*m33*RS143 + 1./32.*
         pow(m11,2)*RS143 );

      F13p6x21RV +=  + hhdV(1,m14,m33,m34,m13,L,mu2) * (  - 1./12.*m33*
         RD34 - 19./96.*m33*m44 + 1./32.*m33*m44*RS341 + 1./24.*pow(
         m33,2) - 5./24.*pow(m33,2)*RS341 - 1./12.*m11*RD34 - 19./96.*
         m11*m44 + 1./32.*m11*m44*RS341 - 11./96.*m11*m33 - 7./32.*m11*
         m33*RS341 - 5./32.*pow(m11,2) - 1./96.*pow(m11,2)*RS341 );

      F13p6x21RV +=  + hhdV(1,m33,m13,m33,m13,L,mu2) * (  - 1./16.*pow(
         m33,2) - 1./16.*m11*m33 );

      F13p6x21RV +=  + hhdV(1,m33,m14,m34,m13,L,mu2) * ( 1./16.*pow(
         m33,2) - 1./16.*pow(m33,2)*RS341 + 1./16.*m11*m33 - 1./16.*m11
         *m33*RS341 );

      F13p6x21RV +=  + hhdV(1,m34,m11,m14,m13,L,mu2) * (  - 1./32.*m33*
         m44 + 1./32.*m33*m44*RS143 - 1./32.*pow(m33,2) + 1./32.*pow(
         m33,2)*RS143 - 1./32.*m11*m44 + 1./32.*m11*m44*RS143 - 1./32.*
         m11*m33 + 1./32.*m11*m33*RS143 );

      F13p6x21RV +=  + hhdV(1,m34,m14,m33,m13,L,mu2) * ( 1./32.*m33*m44
         *RS341 + 1./32.*pow(m33,2)*RS341 + 1./32.*m11*m44*RS341 + 1./
         32.*m11*m33*RS341 );

      F13p6x21RV +=  + hhdV(1,m44,m14,m34,m13,L,mu2) * ( 1./2.*m33*m44
          + 1./2.*m11*m44 );

      F13p6x21RV +=  + hhdV(2,m11,m11,m13,m13,L,mu2) * (  - 1./72.*pow(
         m33,2)*RD14 + 1./36.*pow(m33,2)*RS143*RD14 - 1./36.*m11*m33*
         RD14 + 1./18.*m11*m33*RS143*RD14 - 1./72.*pow(m11,2)*RD14 + 1./
         36.*pow(m11,2)*RS143*RD14 );

      F13p6x21RV +=  + hhdV(3,m13,m11,m11,m13,L,mu2) * ( 1./288.*pow(
         m33,2)*RD14 - 1./144.*pow(m33,2)*RS143*RD14 + 1./144.*m11*m33*
         RD14 - 1./72.*m11*m33*RS143*RD14 + 1./288.*pow(m11,2)*RD14 - 1.
         /144.*pow(m11,2)*RS143*RD14 );

      F13p6x21RV +=  + hhdV(3,m13,m11,m33,m13,L,mu2) * ( 1./288.*pow(
         m33,2)*RD14 - 1./144.*pow(m33,2)*RS341*RD14 + 1./144.*m11*m33*
         RD14 - 1./72.*m11*m33*RS341*RD14 + 1./288.*pow(m11,2)*RD14 - 1.
         /144.*pow(m11,2)*RS341*RD14 );

      F13p6x21RV +=  + hhdV(3,m13,m33,m33,m13,L,mu2) * (  - 23./2592.*
         m33*pow(RD34,2) + 5./288.*pow(m33,2)*RD34 + 1./432.*pow(m33,2)
         *RS341*RD34 - 23./2592.*m11*pow(RD34,2) + 5./144.*m11*m33*RD34
          - 1./72.*m11*m33*RS341*RD34 + 5./288.*pow(m11,2)*RD34 - 7./
         432.*pow(m11,2)*RS341*RD34 );

      F13p6x21RV +=  + hhdV(3,m14,m11,m34,m13,L,mu2) * (  - 1./32.*m33*
         m44*RD14 - 1./32.*m11*m44*RD14 - 1./32.*m11*m33*RD14 - 1./32.*
         pow(m11,2)*RD14 );

      F13p6x21RV +=  + hhdV(4,m13,m11,m33,m13,L,mu2) * ( 1./288.*pow(
         m33,2)*RD34 - 1./144.*pow(m33,2)*RS143*RD34 + 1./144.*m11*m33*
         RD34 - 1./72.*m11*m33*RS143*RD34 + 1./288.*pow(m11,2)*RD34 - 1.
         /144.*pow(m11,2)*RS143*RD34 );

      F13p6x21RV +=  + hhdV(4,m13,m33,m33,m13,L,mu2) * (  - 1./2592.*
         m33*pow(RD34,2) - 1./2592.*m11*pow(RD34,2) );

      F13p6x21RV +=  + hhdV(4,m34,m14,m33,m13,L,mu2) * (  - 1./32.*m33*
         m44*RD34 - 1./32.*pow(m33,2)*RD34 - 1./32.*m11*m44*RD34 - 1./
         32.*m11*m33*RD34 );

      F13p6x21RV +=  + hhdV(5,m11,m11,m13,m13,L,mu2) * (  - 1./144.*
         pow(m33,2)*pow(RD14,2) - 1./72.*m11*m33*pow(RD14,2) - 1./144.*
         pow(m11,2)*pow(RD14,2) );

      F13p6x21RV +=  + hhdV(7,m13,m11,m11,m13,L,mu2) * ( 1./576.*pow(
         m33,2)*pow(RD14,2) + 1./288.*m11*m33*pow(RD14,2) + 1./576.*
         pow(m11,2)*pow(RD14,2) );

      F13p6x21RV +=  + hhdV(7,m13,m11,m33,m13,L,mu2) * ( 1./288.*pow(
         m33,2)*RD14*RD34 + 1./144.*m11*m33*RD14*RD34 + 1./288.*pow(
         m11,2)*RD14*RD34 );

      F13p6x21RV +=  + hhdV(7,m13,m33,m33,m13,L,mu2) * ( 5./576.*pow(
         m33,2)*pow(RD34,2) + 5./288.*m11*m33*pow(RD34,2) + 5./576.*
         pow(m11,2)*pow(RD34,2) );

      F13p6x21RV +=  + hh1dV(1,m11,m11,m13,m13,L,mu2) * (  - 1./54.*m33
         *RD14 + 1./27.*m33*RS143*RD14 + 5./18.*pow(m33,2) - 2./27.*
         pow(m33,2)*RS143 + 5./54.*pow(m33,2)*pow(RS143,2) - 1./54.*m11
         *RD14 + 1./27.*m11*RS143*RD14 + 5./9.*m11*m33 - 1./9.*m11*m33*
         RS143 + 1./9.*m11*m33*pow(RS143,2) + 5./18.*pow(m11,2) - 1./27.
         *pow(m11,2)*RS143 + 1./54.*pow(m11,2)*pow(RS143,2) );

      F13p6x21RV +=  + hh1dV(1,m11,m14,m34,m13,L,mu2) * (  - 1./4.*pow(
         m33,2) + 1./4.*pow(m33,2)*RS143 - 1./2.*m11*m33 + 1./2.*m11*
         m33*RS143 - 1./4.*pow(m11,2) + 1./4.*pow(m11,2)*RS143 );

      F13p6x21RV +=  + hh1dV(1,m13,m11,m33,m13,L,mu2) * ( 1./108.*m33*
         RD34 + 1./108.*m33*RD14 - 1./54.*m33*RS341*RD14 - 1./54.*m33*
         RS143*RD34 + 1./54.*pow(m33,2)*RS341 + 1./27.*pow(m33,2)*RS143
          - 1./18.*pow(m33,2)*RS143*RS341 + 1./108.*m11*RD34 + 1./108.*
         m11*RD14 - 1./54.*m11*RS341*RD14 - 1./54.*m11*RS143*RD34 + 1./
         18.*m11*m33*RS341 + 1./18.*m11*m33*RS143 - 1./9.*m11*m33*RS143
         *RS341 + 1./27.*pow(m11,2)*RS341 + 1./54.*pow(m11,2)*RS143 - 1.
         /18.*pow(m11,2)*RS143*RS341 );

      F13p6x21RV +=  + hh1dV(1,m13,m33,m33,m13,L,mu2) * ( 1./108.*m33*
         RD34 - 1./54.*m33*RS341*RD34 - 5./36.*pow(m33,2) + 1./54.*pow(
         m33,2)*RS341 - 1./108.*pow(m33,2)*pow(RS341,2) + 1./108.*m11*
         RD34 - 1./54.*m11*RS341*RD34 - 5./18.*m11*m33 + 1./18.*m11*m33
         *RS341 - 1./18.*m11*m33*pow(RS341,2) - 5./36.*pow(m11,2) + 1./
         27.*pow(m11,2)*RS341 - 5./108.*pow(m11,2)*pow(RS341,2) );

      F13p6x21RV +=  + hh1dV(1,m14,m11,m34,m13,L,mu2) * (  - 1./12.*m33
         *RD14 - 1./6.*m33*m44 - 1./4.*pow(m33,2) + 1./12.*pow(m33,2)*
         RS143 - 1./12.*m11*RD14 - 1./6.*m11*m44 - 1./3.*m11*m33 - 1./
         12.*pow(m11,2) - 1./12.*pow(m11,2)*RS143 );

      F13p6x21RV +=  + hh1dV(1,m14,m33,m34,m13,L,mu2) * ( 1./12.*m33*
         RD34 + 1./6.*m33*m44 + 1./12.*pow(m33,2) + 1./12.*pow(m33,2)*
         RS341 + 1./12.*m11*RD34 + 1./6.*m11*m44 + 1./3.*m11*m33 + 1./4.
         *pow(m11,2) - 1./12.*pow(m11,2)*RS341 );

      F13p6x21RV +=  + hh1dV(1,m33,m14,m34,m13,L,mu2) * ( 1./12.*m33*
         RD34 + 1./6.*m33*m44 - 1./6.*pow(m33,2) + 1./3.*pow(m33,2)*
         RS341 + 1./12.*m11*RD34 + 1./6.*m11*m44 - 1./6.*m11*m33 + 1./2.
         *m11*m33*RS341 + 1./6.*pow(m11,2)*RS341 );

      F13p6x21RV +=  + hh1dV(2,m11,m11,m13,m13,L,mu2) * (  - 1./108.*
         m33*pow(RD14,2) + 1./36.*pow(m33,2)*RD14 - 1./27.*pow(m33,2)*
         RS143*RD14 - 1./108.*m11*pow(RD14,2) + 1./18.*m11*m33*RD14 - 1.
         /18.*m11*m33*RS143*RD14 + 1./36.*pow(m11,2)*RD14 - 1./54.*pow(
         m11,2)*RS143*RD14 );

      F13p6x21RV +=  + hh1dV(3,m11,m11,m13,m13,L,mu2) * (  - 1./108.*
         m33*pow(RD14,2) + 1./36.*pow(m33,2)*RD14 - 1./27.*pow(m33,2)*
         RS143*RD14 - 1./108.*m11*pow(RD14,2) + 1./18.*m11*m33*RD14 - 1.
         /18.*m11*m33*RS143*RD14 + 1./36.*pow(m11,2)*RD14 - 1./54.*pow(
         m11,2)*RS143*RD14 );

      F13p6x21RV +=  + hh1dV(3,m13,m11,m33,m13,L,mu2) * ( 1./108.*m33*
         RD14*RD34 + 1./54.*pow(m33,2)*RS341*RD14 + 1./108.*m11*RD14*
         RD34 + 1./18.*m11*m33*RS341*RD14 + 1./27.*pow(m11,2)*RS341*
         RD14 );

      F13p6x21RV +=  + hh1dV(3,m13,m33,m33,m13,L,mu2) * ( 23./2592.*m33
         *pow(RD34,2) - 1./36.*pow(m33,2)*RD34 + 1./54.*pow(m33,2)*
         RS341*RD34 + 23./2592.*m11*pow(RD34,2) - 1./18.*m11*m33*RD34
          + 1./18.*m11*m33*RS341*RD34 - 1./36.*pow(m11,2)*RD34 + 1./27.
         *pow(m11,2)*RS341*RD34 );

      F13p6x21RV +=  + hh1dV(4,m13,m11,m33,m13,L,mu2) * ( 1./108.*m33*
         RD14*RD34 + 1./27.*pow(m33,2)*RS143*RD34 + 1./108.*m11*RD14*
         RD34 + 1./18.*m11*m33*RS143*RD34 + 1./54.*pow(m11,2)*RS143*
         RD34 );

      F13p6x21RV +=  + hh1dV(4,m13,m33,m33,m13,L,mu2) * ( 1./2592.*m33*
         pow(RD34,2) + 1./2592.*m11*pow(RD34,2) );

      F13p6x21RV +=  + hh1dV(5,m11,m11,m13,m13,L,mu2) * ( 1./36.*pow(
         m33,2)*pow(RD14,2) + 1./18.*m11*m33*pow(RD14,2) + 1./36.*pow(
         m11,2)*pow(RD14,2) );

      F13p6x21RV +=  + hh1dV(7,m13,m33,m33,m13,L,mu2) * (  - 1./72.*
         pow(m33,2)*pow(RD34,2) - 1./36.*m11*m33*pow(RD34,2) - 1./72.*
         pow(m11,2)*pow(RD34,2) );

      F13p6x21RV +=  + hh21dV(1,m11,m11,m13,m13,L,mu2) * (  - 3./32.*
         pow(m33,2) - 3./16.*m11*m33 - 3./32.*pow(m11,2) );

      F13p6x21RV +=  + hh21dV(1,m11,m14,m34,m13,L,mu2) * ( 3./32.*pow(
         m33,2) - 3./32.*pow(m33,2)*RS143 + 3./16.*m11*m33 - 3./16.*m11
         *m33*RS143 + 3./32.*pow(m11,2) - 3./32.*pow(m11,2)*RS143 );

      F13p6x21RV +=  + hh21dV(1,m13,m11,m11,m13,L,mu2) * ( 5./96.*pow(
         m33,2) - 1./48.*pow(m33,2)*RS143 + 1./48.*pow(m33,2)*pow(
         RS143,2) + 5./48.*m11*m33 - 1./24.*m11*m33*RS143 + 1./24.*m11*
         m33*pow(RS143,2) + 5./96.*pow(m11,2) - 1./48.*pow(m11,2)*RS143
          + 1./48.*pow(m11,2)*pow(RS143,2) );

      F13p6x21RV +=  + hh21dV(1,m13,m11,m33,m13,L,mu2) * ( 1./96.*pow(
         m33,2) - 1./48.*pow(m33,2)*RS341 - 1./48.*pow(m33,2)*RS143 + 1.
         /24.*pow(m33,2)*RS143*RS341 + 1./48.*m11*m33 - 1./24.*m11*m33*
         RS341 - 1./24.*m11*m33*RS143 + 1./12.*m11*m33*RS143*RS341 + 1./
         96.*pow(m11,2) - 1./48.*pow(m11,2)*RS341 - 1./48.*pow(m11,2)*
         RS143 + 1./24.*pow(m11,2)*RS143*RS341 );

      F13p6x21RV +=  + hh21dV(1,m13,m13,m13,m13,L,mu2) * ( 3./32.*pow(
         m33,2) + 3./16.*m11*m33 + 3./32.*pow(m11,2) );

      F13p6x21RV +=  + hh21dV(1,m13,m33,m33,m13,L,mu2) * ( 5./96.*pow(
         m33,2) - 1./48.*pow(m33,2)*RS341 + 1./48.*pow(m33,2)*pow(
         RS341,2) + 5./48.*m11*m33 - 1./24.*m11*m33*RS341 + 1./24.*m11*
         m33*pow(RS341,2) + 5./96.*pow(m11,2) - 1./48.*pow(m11,2)*RS341
          + 1./48.*pow(m11,2)*pow(RS341,2) );

      F13p6x21RV +=  + hh21dV(1,m14,m11,m34,m13,L,mu2) * ( 3./32.*pow(
         m33,2)*RS143 + 3./16.*m11*m33*RS143 + 3./32.*pow(m11,2)*RS143
          );

      F13p6x21RV +=  + hh21dV(1,m14,m33,m34,m13,L,mu2) * (  - 3./32.*
         pow(m33,2) + 3./32.*pow(m33,2)*RS341 - 3./16.*m11*m33 + 3./16.
         *m11*m33*RS341 - 3./32.*pow(m11,2) + 3./32.*pow(m11,2)*RS341 )
         ;

      F13p6x21RV +=  + hh21dV(1,m33,m13,m33,m13,L,mu2) * (  - 3./32.*
         pow(m33,2) - 3./16.*m11*m33 - 3./32.*pow(m11,2) );

      F13p6x21RV +=  + hh21dV(1,m33,m14,m34,m13,L,mu2) * ( 3./32.*pow(
         m33,2) - 3./32.*pow(m33,2)*RS341 + 3./16.*m11*m33 - 3./16.*m11
         *m33*RS341 + 3./32.*pow(m11,2) - 3./32.*pow(m11,2)*RS341 );

      F13p6x21RV +=  + hh21dV(1,m34,m11,m14,m13,L,mu2) * (  - 3./32.*
         pow(m33,2) + 3./32.*pow(m33,2)*RS143 - 3./16.*m11*m33 + 3./16.
         *m11*m33*RS143 - 3./32.*pow(m11,2) + 3./32.*pow(m11,2)*RS143 )
         ;

      F13p6x21RV +=  + hh21dV(1,m34,m14,m33,m13,L,mu2) * ( 3./32.*pow(
         m33,2)*RS341 + 3./16.*m11*m33*RS341 + 3./32.*pow(m11,2)*RS341
          );

      F13p6x21RV +=  + hh21dV(1,m44,m14,m34,m13,L,mu2) * ( 3./4.*pow(
         m33,2) + 3./2.*m11*m33 + 3./4.*pow(m11,2) );

      F13p6x21RV +=  + hh21dV(3,m13,m11,m11,m13,L,mu2) * ( 1./96.*pow(
         m33,2)*RD14 - 1./48.*pow(m33,2)*RS143*RD14 + 1./48.*m11*m33*
         RD14 - 1./24.*m11*m33*RS143*RD14 + 1./96.*pow(m11,2)*RD14 - 1./
         48.*pow(m11,2)*RS143*RD14 );

      F13p6x21RV +=  + hh21dV(3,m13,m11,m33,m13,L,mu2) * ( 1./96.*pow(
         m33,2)*RD14 - 1./48.*pow(m33,2)*RS341*RD14 + 1./48.*m11*m33*
         RD14 - 1./24.*m11*m33*RS341*RD14 + 1./96.*pow(m11,2)*RD14 - 1./
         48.*pow(m11,2)*RS341*RD14 );

      F13p6x21RV +=  + hh21dV(3,m13,m33,m33,m13,L,mu2) * ( 1./96.*pow(
         m33,2)*RD34 - 1./48.*pow(m33,2)*RS341*RD34 + 1./48.*m11*m33*
         RD34 - 1./24.*m11*m33*RS341*RD34 + 1./96.*pow(m11,2)*RD34 - 1./
         48.*pow(m11,2)*RS341*RD34 );

      F13p6x21RV +=  + hh21dV(3,m14,m11,m34,m13,L,mu2) * (  - 3./32.*
         pow(m33,2)*RD14 - 3./16.*m11*m33*RD14 - 3./32.*pow(m11,2)*RD14
          );

      F13p6x21RV +=  + hh21dV(4,m13,m11,m33,m13,L,mu2) * ( 1./96.*pow(
         m33,2)*RD34 - 1./48.*pow(m33,2)*RS143*RD34 + 1./48.*m11*m33*
         RD34 - 1./24.*m11*m33*RS143*RD34 + 1./96.*pow(m11,2)*RD34 - 1./
         48.*pow(m11,2)*RS143*RD34 );

      F13p6x21RV +=  + hh21dV(4,m34,m14,m33,m13,L,mu2) * (  - 3./32.*
         pow(m33,2)*RD34 - 3./16.*m11*m33*RD34 - 3./32.*pow(m11,2)*RD34
          );

      F13p6x21RV +=  + hh21dV(7,m13,m11,m11,m13,L,mu2) * ( 1./192.*pow(
         m33,2)*pow(RD14,2) + 1./96.*m11*m33*pow(RD14,2) + 1./192.*pow(
         m11,2)*pow(RD14,2) );

      F13p6x21RV +=  + hh21dV(7,m13,m11,m33,m13,L,mu2) * ( 1./96.*pow(
         m33,2)*RD14*RD34 + 1./48.*m11*m33*RD14*RD34 + 1./96.*pow(
         m11,2)*RD14*RD34 );

      F13p6x21RV +=  + hh21dV(7,m13,m33,m33,m13,L,mu2) * ( 1./192.*pow(
         m33,2)*pow(RD34,2) + 1./96.*m11*m33*pow(RD34,2) + 1./192.*pow(
         m11,2)*pow(RD34,2) );

      F13p6x21RV +=  + hh27dV(1,m11,m11,m13,m13,L,mu2) * ( 3./16.*m33
          + 3./16.*m11 );

      F13p6x21RV +=  + hh27dV(1,m11,m14,m34,m13,L,mu2) * (  - 3./16.*
         m33 + 3./16.*m33*RS143 - 3./16.*m11 + 3./16.*m11*RS143 );

      F13p6x21RV +=  + hh27dV(1,m13,m11,m11,m13,L,mu2) * (  - 5./48.*
         m33 + 1./24.*m33*RS143 - 1./24.*m33*pow(RS143,2) - 5./48.*m11
          + 1./24.*m11*RS143 - 1./24.*m11*pow(RS143,2) );

      F13p6x21RV +=  + hh27dV(1,m13,m11,m33,m13,L,mu2) * (  - 1./48.*
         m33 + 1./24.*m33*RS341 + 1./24.*m33*RS143 - 1./12.*m33*RS143*
         RS341 - 1./48.*m11 + 1./24.*m11*RS341 + 1./24.*m11*RS143 - 1./
         12.*m11*RS143*RS341 );

      F13p6x21RV +=  + hh27dV(1,m13,m13,m13,m13,L,mu2) * (  - 3./16.*
         m33 - 3./16.*m11 );

      F13p6x21RV +=  + hh27dV(1,m13,m33,m33,m13,L,mu2) * (  - 5./48.*
         m33 + 1./24.*m33*RS341 - 1./24.*m33*pow(RS341,2) - 5./48.*m11
          + 1./24.*m11*RS341 - 1./24.*m11*pow(RS341,2) );

      F13p6x21RV +=  + hh27dV(1,m14,m11,m34,m13,L,mu2) * (  - 3./16.*
         m33*RS143 - 3./16.*m11*RS143 );

      F13p6x21RV +=  + hh27dV(1,m14,m33,m34,m13,L,mu2) * ( 3./16.*m33
          - 3./16.*m33*RS341 + 3./16.*m11 - 3./16.*m11*RS341 );

      F13p6x21RV +=  + hh27dV(1,m33,m13,m33,m13,L,mu2) * ( 3./16.*m33
          + 3./16.*m11 );

      F13p6x21RV +=  + hh27dV(1,m33,m14,m34,m13,L,mu2) * (  - 3./16.*
         m33 + 3./16.*m33*RS341 - 3./16.*m11 + 3./16.*m11*RS341 );

      F13p6x21RV +=  + hh27dV(1,m34,m11,m14,m13,L,mu2) * ( 3./16.*m33
          - 3./16.*m33*RS143 + 3./16.*m11 - 3./16.*m11*RS143 );

      F13p6x21RV +=  + hh27dV(1,m34,m14,m33,m13,L,mu2) * (  - 3./16.*
         m33*RS341 - 3./16.*m11*RS341 );

      F13p6x21RV +=  + hh27dV(1,m44,m14,m34,m13,L,mu2) * (  - 3./2.*m33
          - 3./2.*m11 );

      F13p6x21RV +=  + hh27dV(3,m13,m11,m11,m13,L,mu2) * (  - 1./48.*
         m33*RD14 + 1./24.*m33*RS143*RD14 - 1./48.*m11*RD14 + 1./24.*
         m11*RS143*RD14 );

      F13p6x21RV +=  + hh27dV(3,m13,m11,m33,m13,L,mu2) * (  - 1./48.*
         m33*RD14 + 1./24.*m33*RS341*RD14 - 1./48.*m11*RD14 + 1./24.*
         m11*RS341*RD14 );

      F13p6x21RV +=  + hh27dV(3,m13,m33,m33,m13,L,mu2) * (  - 1./48.*
         m33*RD34 + 1./24.*m33*RS341*RD34 - 1./48.*m11*RD34 + 1./24.*
         m11*RS341*RD34 );

      F13p6x21RV +=  + hh27dV(3,m14,m11,m34,m13,L,mu2) * ( 3./16.*m33*
         RD14 + 3./16.*m11*RD14 );

      F13p6x21RV +=  + hh27dV(4,m13,m11,m33,m13,L,mu2) * (  - 1./48.*
         m33*RD34 + 1./24.*m33*RS143*RD34 - 1./48.*m11*RD34 + 1./24.*
         m11*RS143*RD34 );

      F13p6x21RV +=  + hh27dV(4,m34,m14,m33,m13,L,mu2) * ( 3./16.*m33*
         RD34 + 3./16.*m11*RD34 );

      F13p6x21RV +=  + hh27dV(7,m13,m11,m11,m13,L,mu2) * (  - 1./96.*
         m33*pow(RD14,2) - 1./96.*m11*pow(RD14,2) );

      F13p6x21RV +=  + hh27dV(7,m13,m11,m33,m13,L,mu2) * (  - 1./48.*
         m33*RD14*RD34 - 1./48.*m11*RD14*RD34 );

      F13p6x21RV +=  + hh27dV(7,m13,m33,m33,m13,L,mu2) * (  - 1./96.*
         m33*pow(RD34,2) - 1./96.*m11*pow(RD34,2) );

  return F13p6x21RV/pow(f0,4);
}

// 1 valence mass 2 sea mass *****************************************

double fv1s2nf3p4V(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 3) || (B0mq.size() != 3)){
    cout << "ERROR: fv1s2nf3p4V probably called with wrong number"
	 << "of quark masses\n";}
  // three masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m66 = 2.*B0mq[2];
  double m14 = (m11+m44)/2.;
  double m16 = (m11+m66)/2.;
  //double mu2 = mu0*mu0;

  double F12p4x12V =
       + AbV(m14,L) * ( 1 );

      F12p4x12V +=  + AbV(m16,L) * ( 1./2. );

  return F12p4x12V/pow(f0,2);
}

double fv1s2nf3p6V(const quarkmassnf mass, const Linf Liin, const double L){
  return fv1s2nf3p6LV(mass,Liin,L)+fv1s2nf3p6RV(mass,L);
}

double fv1s2nf3p6LV(const quarkmassnf mass, const Linf Liin, const double L){
  double L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu;
  int nfLi;
  Liin.out(L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu,nfLi);
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if (mu0 != mu){
    cout << "ERROR: differing mu's in fv1s2nf3p6L\n";}
  if ( (nq != 3) || (B0mq.size() != 3)){
    cout << "ERROR: fv1s2nf3p6L probably called with wrong number"
	 << "of quark masses\n";}
  // three masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m66 = 2.*B0mq[2];
  double m14 = (m11+m44)/2.;
  double m16 = (m11+m66)/2.;
  double m46 = (m44+m66)/2.;
  double me2 = (m44+2.*m66)/3.;
  //double mu2 = mu0*mu0;
  double RC146e = RC(m11,m44,m66,me2);
  double RSe61 = RS(me2,m66,m11);
  double RSe6611 = pow(RSe61,2);
  double RD146e = RD(m11,m44,m66,me2);

  double F12p6x12LV =
       + AbV(m11,L) * ( 4*L3r*RD146e + 4*L0r*RD146e - 4./3.*m11*L5r*
         RC146e + 4*m11*L3r*RC146e - 10*m11*L2r - 4*m11*L1r + 4*m11*L0r
         *RC146e );

      F12p6x12LV +=  + AbV(m14,L) * (  - 4*m66*L4r - 8*m44*L4r - 10*m44
         *L3r - 4*m44*L0r + 4*m11*L5r - 10*m11*L3r - 4*m11*L0r );

      F12p6x12LV +=  + AbV(m16,L) * (  - 2*m66*L4r - 5*m66*L3r - 2*m66*
         L0r - 4*m44*L4r + 2*m11*L5r - 5*m11*L3r - 2*m11*L0r );

      F12p6x12LV +=  + AbV(m44,L) * ( 12*m44*L4r - 6*m44*L2r - 24*m44*
         L1r );

      F12p6x12LV +=  + AbV(m46,L) * ( 8*m66*L4r - 4*m66*L2r - 16*m66*
         L1r + 8*m44*L4r - 4*m44*L2r - 16*m44*L1r );

      F12p6x12LV +=  + AbV(me2,L) * (  - 8*me2*L3r*RSe6611 - 2*me2*L2r
          - 8*me2*L1r - 8*me2*L0r*RSe6611 + 8./3.*m66*L4r + 4./3.*m44*
         L4r + 8./3.*m11*L5r*RSe6611 );

      F12p6x12LV +=  + BbV(m11,L) * (  - 4./3.*m11*L5r*RD146e + 4*m11*
         L3r*RD146e + 4*m11*L0r*RD146e );

      F12p6x12LV +=  + BbV(m14,L) * ( 8*m44*m66*L6r - 4*m44*m66*L4r + 4
         *pow(m44,2)*L8r + 16*pow(m44,2)*L6r - 2*pow(m44,2)*L5r - 8*
         pow(m44,2)*L4r + 8*m11*m66*L6r - 4*m11*m66*L4r + 8*m11*m44*L8r
          + 16*m11*m44*L6r - 4*m11*m44*L5r - 8*m11*m44*L4r + 4*pow(
         m11,2)*L8r - 2*pow(m11,2)*L5r );

      F12p6x12LV +=  + BbV(m16,L) * ( 2*pow(m66,2)*L8r + 4*pow(m66,2)*
         L6r - pow(m66,2)*L5r - 2*pow(m66,2)*L4r + 8*m44*m66*L6r - 4*
         m44*m66*L4r + 4*m11*m66*L8r + 4*m11*m66*L6r - 2*m11*m66*L5r - 
         2*m11*m66*L4r + 8*m11*m44*L6r - 4*m11*m44*L4r + 2*pow(m11,2)*
         L8r - pow(m11,2)*L5r );

      F12p6x12LV +=  + A23bV(m11,L) * (  - 4*L3r*RC146e + 6*L2r + 12*
         L1r - 4*L0r*RC146e );

      F12p6x12LV +=  + A23bV(m14,L) * ( 12*L3r + 24*L0r );

      F12p6x12LV +=  + A23bV(m16,L) * ( 6*L3r + 12*L0r );

      F12p6x12LV +=  + A23bV(m44,L) * ( 18*L2r );

      F12p6x12LV +=  + A23bV(m46,L) * ( 24*L2r );

      F12p6x12LV +=  + A23bV(me2,L) * ( 8*L3r*RSe6611 + 6*L2r + 8*L0r*
         RSe6611 );

      F12p6x12LV +=  + B23bV(m11,L) * (  - 4*L3r*RD146e - 4*L0r*RD146e
          );

  return F12p6x12LV/pow(f0,4);
}

double fv1s2nf3p6RV(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 3) || (B0mq.size() != 3)){
    cout << "ERROR: fv1s2nf3p6RV probably called with wrong number"
	 << "of quark masses\n";}
  // three masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m66 = 2.*B0mq[2];
  double m14 = (m11+m44)/2.;
  double m16 = (m11+m66)/2.;
  double m46 = (m44+m66)/2.;
  double me2 = (m44+2.*m66)/3.;
  double mu2 = mu0*mu0;
  double RC146e = RC(m11,m44,m66,me2);
  double RSe61 = RS(me2,m66,m11);
  double RSe6611 = pow(RSe61,2);
  double RS14e = RS(m11,m44,me2);
  double RS16e = RS(m11,m66,me2);
  //double RS144ee = RS14e*RS14e;
  //double RS166ee = RS16e*RS16e;
  double RD146e = RD(m11,m44,m66,me2);

  double F12p6x12RV =
       + Ab(m11,mu2)*BbV(m14,L) * ( 1./18.*RD146e - 1./9.*m44*RS16e - 2.
         /9.*m11*RS16e );

      F12p6x12RV +=  + Ab(m11,mu2)*BbV(m16,L) * ( 1./36.*RD146e - 1./18.
         *m66*RS14e - 1./9.*m11*RS14e );

      F12p6x12RV +=  + Ab(m14,mu2)*AbV(m14,L) * ( 5./54. - 5./27.*RSe61
          - 5./27.*RSe6611 - 5./27.*RS16e + 5./54.*RC146e );

      F12p6x12RV +=  + Ab(m16,mu2)*AbV(m16,L) * ( 5./108. + 5./27.*
         RSe61 - 5./54.*RSe6611 - 5./54.*RS14e + 5./108.*RC146e );

      F12p6x12RV +=  + Ab(me2,mu2)*BbV(m14,L) * (  - 1./36.*me2 - 1./9.
         *me2*RSe61 - 1./9.*me2*RSe6611 + 1./36.*m44 - 1./9.*m44*RSe61
          - 1./9.*m11*RSe61 + 1./9.*m11*RSe6611 );

      F12p6x12RV +=  + Ab(me2,mu2)*BbV(m16,L) * (  - 1./18.*me2 + 1./9.
         *me2*RSe61 - 1./18.*me2*RSe6611 + 1./18.*m66 + 1./9.*m66*RSe61
          + 1./9.*m11*RSe61 + 1./18.*m11*RSe6611 );

      F12p6x12RV +=  + Bb(m14,mu2)*AbV(m11,L) * ( 1./18.*RD146e - 1./9.
         *m44*RS16e - 2./9.*m11*RS16e );

      F12p6x12RV +=  + Bb(m14,mu2)*AbV(me2,L) * (  - 1./36.*me2 - 1./9.
         *me2*RSe61 - 1./9.*me2*RSe6611 + 1./36.*m44 - 1./9.*m44*RSe61
          - 1./9.*m11*RSe61 + 1./9.*m11*RSe6611 );

      F12p6x12RV +=  + Bb(m16,mu2)*AbV(m11,L) * ( 1./36.*RD146e - 1./18.
         *m66*RS14e - 1./9.*m11*RS14e );

      F12p6x12RV +=  + Bb(m16,mu2)*AbV(me2,L) * (  - 1./18.*me2 + 1./9.
         *me2*RSe61 - 1./18.*me2*RSe6611 + 1./18.*m66 + 1./9.*m66*RSe61
          + 1./9.*m11*RSe61 + 1./18.*m11*RSe6611 );

      F12p6x12RV +=  + AbV(m11,L)*pi16 * ( 1./8.*m11 - 1./12.*m11*RS16e
          - 1./24.*m11*RS14e );

      F12p6x12RV +=  + AbV(m11,L)*BbV(m14,L) * ( 1./18.*RD146e - 1./9.*
         m44*RS16e - 2./9.*m11*RS16e );

      F12p6x12RV +=  + AbV(m11,L)*BbV(m16,L) * ( 1./36.*RD146e - 1./18.
         *m66*RS14e - 1./9.*m11*RS14e );

      F12p6x12RV +=  + AbV(m14,L)*pi16 * ( 1./12.*RD146e - 1./24.*me2
          - 1./6.*me2*RSe61 - 1./6.*me2*RSe6611 - 1./4.*m66 - 7./12.*
         m44 - 1./12.*m44*RSe61 - 1./12.*m44*RSe6611 - 1./12.*m44*RS16e
          + 1./24.*m44*RC146e - 5./24.*m11 - 1./12.*m11*RSe61 - 1./12.*
         m11*RSe6611 - 1./4.*m11*RS16e + 1./8.*m11*RC146e );

      F12p6x12RV +=  + pow(AbV(m14,L),2) * ( 5./108. - 5./54.*RSe61 - 5.
         /54.*RSe6611 - 5./54.*RS16e + 5./108.*RC146e );

      F12p6x12RV +=  + AbV(m16,L)*pi16 * ( 1./24.*RD146e - 1./12.*me2
          + 1./6.*me2*RSe61 - 1./12.*me2*RSe6611 - 5./48.*m66 + 1./12.*
         m66*RSe61 - 1./24.*m66*RSe6611 - 1./24.*m66*RS14e + 1./48.*m66
         *RC146e - 1./4.*m44 - 5./48.*m11 + 1./12.*m11*RSe61 - 1./24.*
         m11*RSe6611 - 1./8.*m11*RS14e + 1./16.*m11*RC146e );

      F12p6x12RV +=  + pow(AbV(m16,L),2) * ( 5./216. + 5./54.*RSe61 - 5.
         /108.*RSe6611 - 5./108.*RS14e + 5./216.*RC146e );

      F12p6x12RV +=  + AbV(me2,L)*BbV(m14,L) * (  - 1./36.*me2 - 1./9.*
         me2*RSe61 - 1./9.*me2*RSe6611 + 1./36.*m44 - 1./9.*m44*RSe61
          - 1./9.*m11*RSe61 + 1./9.*m11*RSe6611 );

      F12p6x12RV +=  + AbV(me2,L)*BbV(m16,L) * (  - 1./18.*me2 + 1./9.*
         me2*RSe61 - 1./18.*me2*RSe6611 + 1./18.*m66 + 1./9.*m66*RSe61
          + 1./9.*m11*RSe61 + 1./18.*m11*RSe6611 );

      F12p6x12RV +=  + A23bV(m11,L)*pi16 * (  - 3./8. + 1./4.*RS16e + 1.
         /8.*RS14e );

      F12p6x12RV +=  + A23bV(m14,L)*pi16 * (  - 1./12. + 1./6.*RSe61 + 
         1./6.*RSe6611 + 1./6.*RS16e - 1./12.*RC146e );

      F12p6x12RV +=  + A23bV(m16,L)*pi16 * (  - 1./24. - 1./6.*RSe61 + 
         1./12.*RSe6611 + 1./12.*RS14e - 1./24.*RC146e );

      F12p6x12RV +=  + hhV(1,m11,m14,m14,m11,L,mu2) * ( 1./12.*RD146e
          - 1./6.*m11*RS16e + 1./12.*m11*RC146e );

      F12p6x12RV +=  + hhV(1,m11,m16,m16,m11,L,mu2) * ( 1./24.*RD146e
          - 1./12.*m11*RS14e + 1./24.*m11*RC146e );

      F12p6x12RV +=  + hhV(1,m44,m14,m14,m11,L,mu2) * (  - 3./8.*m44 );

      F12p6x12RV +=  + hhV(1,m46,m14,m16,m11,L,mu2) * (  - 1./4.*m66 - 
         1./4.*m44 );

      F12p6x12RV +=  + hhV(1,me2,m14,m14,m11,L,mu2) * (  - 1./24.*me2
          - 1./6.*me2*RSe61 - 1./6.*me2*RSe6611 );

      F12p6x12RV +=  + hhV(1,me2,m16,m16,m11,L,mu2) * (  - 1./12.*me2
          + 1./6.*me2*RSe61 - 1./12.*me2*RSe6611 );

      F12p6x12RV +=  + hhV(2,m11,m14,m14,m11,L,mu2) * ( 5./54.*m11*
         RD146e );

      F12p6x12RV +=  + hhV(2,m11,m16,m16,m11,L,mu2) * ( 5./108.*m11*
         RD146e );

      F12p6x12RV +=  + hh1V(2,m11,m14,m14,m11,L,mu2) * (  - 1./108.*m11
         *RD146e );

      F12p6x12RV +=  + hh1V(2,m11,m16,m16,m11,L,mu2) * (  - 1./216.*m11
         *RD146e );

      F12p6x12RV +=  + hh1V(3,m14,m11,m14,m11,L,mu2) * (  - 1./54.*m11*
         RD146e );

      F12p6x12RV +=  + hh1V(3,m16,m11,m16,m11,L,mu2) * (  - 1./108.*m11
         *RD146e );

      F12p6x12RV +=  + hh27V(1,m11,m14,m14,m11,L,mu2) * ( 1./2.*RS16e
          - 1./4.*RC146e );

      F12p6x12RV +=  + hh27V(1,m11,m16,m16,m11,L,mu2) * ( 1./4.*RS14e
          - 1./8.*RC146e );

      F12p6x12RV +=  + hh27V(1,m44,m14,m14,m11,L,mu2) * ( 9./8. );

      F12p6x12RV +=  + hh27V(1,m46,m14,m16,m11,L,mu2) * ( 3./2. );

      F12p6x12RV +=  + hh27V(1,me2,m14,m14,m11,L,mu2) * ( 1./8. + 1./2.
         *RSe61 + 1./2.*RSe6611 );

      F12p6x12RV +=  + hh27V(1,me2,m16,m16,m11,L,mu2) * ( 1./4. - 1./2.
         *RSe61 + 1./4.*RSe6611 );

      F12p6x12RV +=  + hh27V(2,m11,m14,m14,m11,L,mu2) * (  - 1./4.*
         RD146e );

      F12p6x12RV +=  + hh27V(2,m11,m16,m16,m11,L,mu2) * (  - 1./8.*
         RD146e );

      F12p6x12RV +=  + hhdV(1,m11,m11,m11,m11,L,mu2) * ( 1./6.*pow(
         m11,2) + 1./9.*pow(m11,2)*pow(RC146e,2) );

      F12p6x12RV +=  + hhdV(1,m11,m11,me2,m11,L,mu2) * (  - 4./9.*pow(
         m11,2)*RC146e*RSe6611 );

      F12p6x12RV +=  + hhdV(1,m11,m14,m14,m11,L,mu2) * (  - 11./36.*m11
         *RD146e - 2./9.*m11*m44*RS16e + 7./18.*pow(m11,2)*RS16e - 5./
         12.*pow(m11,2)*RC146e );

      F12p6x12RV +=  + hhdV(1,m11,m16,m16,m11,L,mu2) * (  - 11./72.*m11
         *RD146e - 1./9.*m11*m66*RS14e + 7./36.*pow(m11,2)*RS14e - 5./
         24.*pow(m11,2)*RC146e );

      F12p6x12RV +=  + hhdV(1,m11,me2,me2,m11,L,mu2) * ( 4./9.*pow(
         m11,2)*pow(RSe6611,2) );

      F12p6x12RV +=  + hhdV(1,m14,m14,me2,m11,L,mu2) * ( 1./54.*pow(
         me2,2) + 2./27.*pow(me2,2)*RSe61 + 2./27.*pow(me2,2)*RSe6611
          - 1./27.*m44*me2 - 2./27.*m44*me2*RSe61 + 1./54.*pow(m44,2)
          - 1./9.*m11*me2 - 8./27.*m11*me2*RSe61 - 4./27.*m11*me2*
         RSe6611 + 1./9.*m11*m44 + 2./27.*m11*m44*RSe61 - 4./9.*pow(
         m11,2)*RSe61 - 16./27.*pow(m11,2)*RSe6611 );

      F12p6x12RV +=  + hhdV(1,m16,m16,me2,m11,L,mu2) * ( 1./27.*pow(
         me2,2) - 2./27.*pow(me2,2)*RSe61 + 1./27.*pow(me2,2)*RSe6611
          - 2./27.*m66*me2 + 2./27.*m66*me2*RSe61 + 1./27.*pow(m66,2)
          - 2./9.*m11*me2 + 8./27.*m11*me2*RSe61 - 2./27.*m11*me2*
         RSe6611 + 2./9.*m11*m66 - 2./27.*m11*m66*RSe61 + 4./9.*pow(
         m11,2)*RSe61 - 8./27.*pow(m11,2)*RSe6611 );

      F12p6x12RV +=  + hhdV(1,m44,m14,m14,m11,L,mu2) * ( 3./8.*m11*m44
          );

      F12p6x12RV +=  + hhdV(1,m46,m14,m16,m11,L,mu2) * ( 1./4.*m11*m66
          + 1./4.*m11*m44 );

      F12p6x12RV +=  + hhdV(1,me2,m14,m14,m11,L,mu2) * ( 1./24.*m11*me2
          + 1./6.*m11*me2*RSe61 + 1./6.*m11*me2*RSe6611 );

      F12p6x12RV +=  + hhdV(1,me2,m16,m16,m11,L,mu2) * ( 1./12.*m11*me2
          - 1./6.*m11*me2*RSe61 + 1./12.*m11*me2*RSe6611 );

      F12p6x12RV +=  + hhdV(2,m11,m11,m11,m11,L,mu2) * ( 2./9.*pow(
         m11,2)*RD146e*RC146e );

      F12p6x12RV +=  + hhdV(2,m11,m11,me2,m11,L,mu2) * (  - 4./9.*pow(
         m11,2)*RD146e*RSe6611 );

      F12p6x12RV +=  + hhdV(2,m11,m14,m14,m11,L,mu2) * (  - 5./12.*pow(
         m11,2)*RD146e );

      F12p6x12RV +=  + hhdV(2,m11,m16,m16,m11,L,mu2) * (  - 5./24.*pow(
         m11,2)*RD146e );

      F12p6x12RV +=  + hhdV(5,m11,m11,m11,m11,L,mu2) * ( 1./9.*pow(
         m11,2)*pow(RD146e,2) );

      F12p6x12RV +=  + hh1dV(1,m11,m14,m14,m11,L,mu2) * ( 2./9.*m11*
         RD146e + 2./9.*m11*m44*RS16e - 8./9.*pow(m11,2)*RS16e + 2./3.*
         pow(m11,2)*RC146e );

      F12p6x12RV +=  + hh1dV(1,m11,m16,m16,m11,L,mu2) * ( 1./9.*m11*
         RD146e + 1./9.*m11*m66*RS14e - 4./9.*pow(m11,2)*RS14e + 1./3.*
         pow(m11,2)*RC146e );

      F12p6x12RV +=  + hh1dV(1,m14,m14,me2,m11,L,mu2) * ( 2./9.*m11*me2
          + 8./9.*m11*me2*RSe61 + 8./9.*m11*me2*RSe6611 - 2./9.*m11*m44
          - 4./9.*m11*m44*RSe61 + 8./9.*pow(m11,2)*RSe61 + 16./9.*pow(
         m11,2)*RSe6611 );

      F12p6x12RV +=  + hh1dV(1,m16,m16,me2,m11,L,mu2) * ( 4./9.*m11*me2
          - 8./9.*m11*me2*RSe61 + 4./9.*m11*me2*RSe6611 - 4./9.*m11*m66
          + 4./9.*m11*m66*RSe61 - 8./9.*pow(m11,2)*RSe61 + 8./9.*pow(
         m11,2)*RSe6611 );

      F12p6x12RV +=  + hh1dV(2,m11,m14,m14,m11,L,mu2) * ( 2./3.*pow(
         m11,2)*RD146e );

      F12p6x12RV +=  + hh1dV(2,m11,m16,m16,m11,L,mu2) * ( 1./3.*pow(
         m11,2)*RD146e );

      F12p6x12RV +=  + hh21dV(1,m11,m14,m14,m11,L,mu2) * ( 1./2.*pow(
         m11,2)*RS16e - 1./4.*pow(m11,2)*RC146e );

      F12p6x12RV +=  + hh21dV(1,m11,m16,m16,m11,L,mu2) * ( 1./4.*pow(
         m11,2)*RS14e - 1./8.*pow(m11,2)*RC146e );

      F12p6x12RV +=  + hh21dV(1,m44,m14,m14,m11,L,mu2) * ( 9./8.*pow(
         m11,2) );

      F12p6x12RV +=  + hh21dV(1,m46,m14,m16,m11,L,mu2) * ( 3./2.*pow(
         m11,2) );

      F12p6x12RV +=  + hh21dV(1,me2,m14,m14,m11,L,mu2) * ( 1./8.*pow(
         m11,2) + 1./2.*pow(m11,2)*RSe61 + 1./2.*pow(m11,2)*RSe6611 );

      F12p6x12RV +=  + hh21dV(1,me2,m16,m16,m11,L,mu2) * ( 1./4.*pow(
         m11,2) - 1./2.*pow(m11,2)*RSe61 + 1./4.*pow(m11,2)*RSe6611 );

      F12p6x12RV +=  + hh21dV(2,m11,m14,m14,m11,L,mu2) * (  - 1./4.*
         pow(m11,2)*RD146e );

      F12p6x12RV +=  + hh21dV(2,m11,m16,m16,m11,L,mu2) * (  - 1./8.*
         pow(m11,2)*RD146e );

      F12p6x12RV +=  + hh27dV(1,m11,m14,m14,m11,L,mu2) * (  - 1./2.*m11
         *RS16e + 1./4.*m11*RC146e );

      F12p6x12RV +=  + hh27dV(1,m11,m16,m16,m11,L,mu2) * (  - 1./4.*m11
         *RS14e + 1./8.*m11*RC146e );

      F12p6x12RV +=  + hh27dV(1,m44,m14,m14,m11,L,mu2) * (  - 9./8.*m11
          );

      F12p6x12RV +=  + hh27dV(1,m46,m14,m16,m11,L,mu2) * (  - 3./2.*m11
          );

      F12p6x12RV +=  + hh27dV(1,me2,m14,m14,m11,L,mu2) * (  - 1./8.*m11
          - 1./2.*m11*RSe61 - 1./2.*m11*RSe6611 );

      F12p6x12RV +=  + hh27dV(1,me2,m16,m16,m11,L,mu2) * (  - 1./4.*m11
          + 1./2.*m11*RSe61 - 1./4.*m11*RSe6611 );

      F12p6x12RV +=  + hh27dV(2,m11,m14,m14,m11,L,mu2) * ( 1./4.*m11*
         RD146e );

      F12p6x12RV +=  + hh27dV(2,m11,m16,m16,m11,L,mu2) * ( 1./8.*m11*
         RD146e );

  return F12p6x12RV/pow(f0,4);
}

// 2 valence mass 2 sea mass *****************************************

double fv2s2nf3p4V(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 4) || (B0mq.size() != 4)){
    cout << "ERROR: fv2s1nf3p4V probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m66 = 2.*B0mq[3];
  double me2 = (m44+2.*m66)/3.;

  double m14 = (m11+m44)/2;
  double m16 = (m11+m66)/2;
  double m34 = (m33+m44)/2;
  double m36 = (m33+m66)/2;

  double RC146e = RC(m11,m44,m66,me2);
  double RC346e = RC(m33,m44,m66,me2);
  double RD146e = RD(m11,m44,m66,me2);
  double RD346e = RD(m33,m44,m66,me2);
  double RS1463e = RS(m11,m44,m66,m33,me2);
  double RS3461e = RS(m33,m44,m66,m11,me2);
  double RSe6611 = RS(me2,m66,m66,m11,m11);
  double RSe6613 = RS(me2,m66,m66,m11,m33);
  double RSe6633 = RS(me2,m66,m66,m33,m33);
  //double mu2 = mu0*mu0;
  double F13p4x22V =
       + AbV(m11,L) * ( 1./6.*RS1463e - 1./12.*RC146e );

      F13p4x22V +=  + AbV(m14,L) * ( 1./2. );

      F13p4x22V +=  + AbV(m16,L) * ( 1./4. );

      F13p4x22V +=  + AbV(m33,L) * ( 1./6.*RS3461e - 1./12.*RC346e );

      F13p4x22V +=  + AbV(m34,L) * ( 1./2. );

      F13p4x22V +=  + AbV(m36,L) * ( 1./4. );

      F13p4x22V +=  + AbV(me2,L) * (  - 1./3.*RSe6613 + 1./6.*RSe6633
          + 1./6.*RSe6611 );

      F13p4x22V +=  + BbV(m11,L) * (  - 1./12.*RD146e );

      F13p4x22V +=  + BbV(m33,L) * (  - 1./12.*RD346e );

  return F13p4x22V/pow(f0,2);
}

double fv2s2nf3p6V(const quarkmassnf mass, const Linf Liin, const double L){
  return fv2s2nf3p6LV(mass,Liin,L)+fv2s2nf3p6RV(mass,L);
}

double fv2s2nf3p6LV(const quarkmassnf mass, const Linf Liin, const double L){
  double L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu;
  int nfLi;
  Liin.out(L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu,nfLi);
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if (mu0 != mu){
    cout << "ERROR: differing mu's in fv2s2nf3p6LV\n";}
  if ( (nq != 4) || (B0mq.size() != 4)){
    cout << "ERROR: fv2s2nf3p6LV probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m66 = 2.*B0mq[3];
  double m13 = (m11+m33)/2;
  double m14 = (m11+m44)/2;
  double m16 = (m11+m66)/2;
  double m34 = (m33+m44)/2;
  double m36 = (m33+m66)/2;
  double m46 = (m44+m66)/2;
  double me2 = (m44+2.*m66)/3.;

  //double RN13 = RN(m11,m33);
  //double RN1e = RN(m11,me2);
  //double RN3e = RN(m33,me2);

  //double RD13 = m11-m33;
  //double RD46 = m44-m66;

  double RD146e = RD(m11,m44,m66,me2);
  double RD346e = RD(m33,m44,m66,me2);

  double RC146e = RC(m11,m44,m66,me2);
  double RC346e = RC(m33,m44,m66,me2);

  double RS14e = RS(m11,m44,me2);
  double RS16e = RS(m11,m66,me2);
  double RS34e = RS(m33,m44,me2);
  double RS36e = RS(m33,m66,me2);
  double RSe61 = RS(me2,m66,m11);
  double RSe63 = RS(me2,m66,m33);

  double RS1463e = RS(m11,m44,m66,m33,me2);
  double RS3461e = RS(m33,m44,m66,m11,me2);
  double RSe6611 = RS(me2,m66,m66,m11,m11);
  double RSe6613 = RS(me2,m66,m66,m11,m33);
  double RSe6633 = RS(me2,m66,m66,m33,m33);

  //double mu2 = mu0*mu0;

  double F13p6x22LV =
       + AbV(m11,L) * ( 5./3.*L3r*RD146e + 2./3.*L0r*RD146e - 2./9.*m66
         *L5r*pow(RS14e,2) - 2./3.*m66*L4r*RS1463e - 4./9.*m66*L4r*pow(
         RS16e,2) - 2./9.*m66*L4r*pow(RS14e,2) + m66*L4r*RC146e - 4./9.
         *m44*L5r*pow(RS16e,2) - 4./3.*m44*L4r*RS1463e - 8./9.*m44*L4r*
         pow(RS16e,2) - 4./9.*m44*L4r*pow(RS14e,2) + 2*m44*L4r*RC146e
          + 1./3.*m33*L5r*RS1463e - 1./6.*m33*L5r*RC146e - m11*L5r*
         RS1463e + 1./2.*m11*L5r*RC146e + 2./3.*m11*L3r*RS1463e + 5./3.
         *m11*L3r*RC146e + 8./3.*m11*L0r*RS1463e + 2./3.*m11*L0r*RC146e
          );

      F13p6x22LV +=  + AbV(m13,L) * (  - 5*m33*L2r - 2*m33*L1r - 5*m11*
         L2r - 2*m11*L1r );

      F13p6x22LV +=  + AbV(m14,L) * (  - 2*m66*L4r - 4*m44*L4r - 5*m44*
         L3r - 2*m44*L0r + m33*L5r + m11*L5r - 5*m11*L3r - 2*m11*L0r );

      F13p6x22LV +=  + AbV(m16,L) * (  - m66*L4r - 5./2.*m66*L3r - m66*
         L0r - 2*m44*L4r + 1./2.*m33*L5r + 1./2.*m11*L5r - 5./2.*m11*
         L3r - m11*L0r );

      F13p6x22LV +=  + AbV(m33,L) * ( 5./3.*L3r*RD346e + 2./3.*L0r*
         RD346e - 2./9.*m66*L5r*pow(RS34e,2) + 4./9.*m66*L5r*RS14e*
         RS34e - 2*m66*L4r*RS3461e - 4./3.*m66*L4r*RS1463e - 4./9.*m66*
         L4r*pow(RS36e,2) - 2./9.*m66*L4r*pow(RS34e,2) + m66*L4r*RC346e
          + 8./9.*m66*L4r*RS16e*RS36e + 4./9.*m66*L4r*RS14e*RS34e - 4./
         9.*m44*L5r*pow(RS36e,2) + 8./9.*m44*L5r*RS16e*RS36e - 4*m44*
         L4r*RS3461e - 8./3.*m44*L4r*RS1463e - 8./9.*m44*L4r*pow(
         RS36e,2) - 4./9.*m44*L4r*pow(RS34e,2) + 2*m44*L4r*RC346e + 16./
         9.*m44*L4r*RS16e*RS36e + 8./9.*m44*L4r*RS14e*RS34e - m33*L5r*
         RS3461e - 4./3.*m33*L5r*RS1463e + 1./2.*m33*L5r*RC346e + 2./3.
         *m33*L3r*RS3461e + 5./3.*m33*L3r*RC346e + 8./3.*m33*L0r*
         RS3461e + 2./3.*m33*L0r*RC346e - m11*L5r*RS3461e - 1./6.*m11*
         L5r*RC346e );

      F13p6x22LV +=  + AbV(m34,L) * (  - 2*m66*L4r - 4*m44*L4r - 5*m44*
         L3r - 2*m44*L0r + m33*L5r - 5*m33*L3r - 2*m33*L0r + m11*L5r );

      F13p6x22LV +=  + AbV(m36,L) * (  - m66*L4r - 5./2.*m66*L3r - m66*
         L0r - 2*m44*L4r + 1./2.*m33*L5r - 5./2.*m33*L3r - m33*L0r + 1./
         2.*m11*L5r );

      F13p6x22LV +=  + AbV(m44,L) * ( 12*m44*L4r - 6*m44*L2r - 24*m44*
         L1r );

      F13p6x22LV +=  + AbV(m46,L) * ( 8*m66*L4r - 4*m66*L2r - 16*m66*
         L1r + 8*m44*L4r - 4*m44*L2r - 16*m44*L1r );

      F13p6x22LV +=  + AbV(me2,L) * (  - 4./3.*me2*L3r*RSe6613 - 10./3.
         *me2*L3r*RSe6633 - 10./3.*me2*L3r*RSe6611 - 2*me2*L2r - 8*me2*
         L1r - 16./3.*me2*L0r*RSe6613 - 4./3.*me2*L0r*RSe6633 - 4./3.*
         me2*L0r*RSe6611 - 8./9.*m66*L5r*pow(RSe63,2) + 8./9.*m66*L5r*
         RS34e*RSe63 + 16./9.*m66*L5r*RSe61*RSe63 - 8./9.*m66*L5r*RSe61
         *RS34e - 8./9.*m66*L5r*pow(RSe61,2) - 8./9.*m66*L5r*RS14e*
         RSe63 + 8./9.*m66*L5r*RS14e*RSe61 + 8./3.*m66*L4r + 4*m66*L4r*
         RSe6613 - 4./3.*m66*L4r*pow(RSe63,2) - 2*m66*L4r*RSe6633 - 8./
         9.*m66*L4r*RS36e*RSe63 + 8./9.*m66*L4r*RS34e*RSe63 + 8./3.*m66
         *L4r*RSe61*RSe63 + 8./9.*m66*L4r*RSe61*RS36e - 8./9.*m66*L4r*
         RSe61*RS34e - 4./3.*m66*L4r*pow(RSe61,2) - 2*m66*L4r*RSe6611
          + 8./9.*m66*L4r*RS16e*RSe63 - 8./9.*m66*L4r*RS16e*RSe61 - 8./
         9.*m66*L4r*RS14e*RSe63 + 8./9.*m66*L4r*RS14e*RSe61 - 4./9.*m44
         *L5r*pow(RSe63,2) - 8./9.*m44*L5r*RS36e*RSe63 + 8./9.*m44*L5r*
         RSe61*RSe63 + 8./9.*m44*L5r*RSe61*RS36e - 4./9.*m44*L5r*pow(
         RSe61,2) );

      F13p6x22LV +=  + AbV(me2,L) * ( 8./9.*m44*L5r*RS16e*RSe63 - 8./9.
         *m44*L5r*RS16e*RSe61 + 4./3.*m44*L4r + 8*m44*L4r*RSe6613 - 8./
         3.*m44*L4r*pow(RSe63,2) - 4*m44*L4r*RSe6633 - 16./9.*m44*L4r*
         RS36e*RSe63 + 16./9.*m44*L4r*RS34e*RSe63 + 16./3.*m44*L4r*
         RSe61*RSe63 + 16./9.*m44*L4r*RSe61*RS36e - 16./9.*m44*L4r*
         RSe61*RS34e - 8./3.*m44*L4r*pow(RSe61,2) - 4*m44*L4r*RSe6611
          + 16./9.*m44*L4r*RS16e*RSe63 - 16./9.*m44*L4r*RS16e*RSe61 - 
         16./9.*m44*L4r*RS14e*RSe63 + 16./9.*m44*L4r*RS14e*RSe61 + 2*
         m33*L5r*RSe6613 - m33*L5r*RSe6633 + 1./3.*m33*L5r*RSe6611 + 2*
         m11*L5r*RSe6613 + 1./3.*m11*L5r*RSe6633 - m11*L5r*RSe6611 );

      F13p6x22LV +=  + BbV(m11,m33,L) * (  - 8./9.*pow(m66,2)*L8r*RS14e
         *RS34e - 8./9.*pow(m66,2)*L7r*RS14e*RS34e - 8./9.*pow(m66,2)*
         L6r*RS14e*RS34e - 16./9.*m44*m66*L7r*RS16e*RS34e - 16./9.*m44*
         m66*L7r*RS14e*RS36e - 16./9.*m44*m66*L6r*RS16e*RS36e - 16./9.*
         m44*m66*L6r*RS14e*RS34e - 16./9.*pow(m44,2)*L8r*RS16e*RS36e - 
         32./9.*pow(m44,2)*L7r*RS16e*RS36e - 32./9.*pow(m44,2)*L6r*
         RS16e*RS36e + 8./3.*m33*m66*L7r*RS14e + 8./3.*m33*m66*L6r*
         RS1463e + 16./3.*m33*m44*L7r*RS16e + 16./3.*m33*m44*L6r*
         RS1463e + 8./3.*pow(m33,2)*L8r*RS1463e + 8./3.*m11*m66*L7r*
         RS34e + 8./3.*m11*m66*L6r*RS3461e + 4./9.*m11*m66*L5r*RS14e*
         RS34e - 4./3.*m11*m66*L4r*RS3461e - 4./3.*m11*m66*L4r*RS1463e
          + 8./9.*m11*m66*L4r*RS16e*RS36e + 4./9.*m11*m66*L4r*RS14e*
         RS34e + 16./3.*m11*m44*L7r*RS36e + 16./3.*m11*m44*L6r*RS3461e
          + 8./9.*m11*m44*L5r*RS16e*RS36e - 8./3.*m11*m44*L4r*RS3461e
          - 8./3.*m11*m44*L4r*RS1463e + 16./9.*m11*m44*L4r*RS16e*RS36e
          + 8./9.*m11*m44*L4r*RS14e*RS34e );

      F13p6x22LV +=  + BbV(m11,m33,L) * (  - 8*m11*m33*L7r - 4./3.*m11*
         m33*L5r*RS1463e + 8./3.*pow(m11,2)*L8r*RS3461e - 4./3.*pow(
         m11,2)*L5r*RS3461e );

      F13p6x22LV +=  + BbV(m11,me2,L) * ( 16./9.*pow(m66,2)*L8r*RS14e*
         RSe63 - 16./9.*pow(m66,2)*L8r*RS14e*RSe61 + 16./9.*pow(m66,2)*
         L7r*RS14e*RSe63 - 16./9.*pow(m66,2)*L7r*RS14e*RSe61 + 16./9.*
         pow(m66,2)*L6r*RS14e*RSe63 - 16./9.*pow(m66,2)*L6r*RS14e*RSe61
          + 32./9.*m44*m66*L7r*RS16e*RSe63 - 32./9.*m44*m66*L7r*RS16e*
         RSe61 - 16./9.*m44*m66*L7r*RS14e*RSe63 + 16./9.*m44*m66*L7r*
         RS14e*RSe61 - 16./9.*m44*m66*L6r*RS16e*RSe63 + 16./9.*m44*m66*
         L6r*RS16e*RSe61 + 32./9.*m44*m66*L6r*RS14e*RSe63 - 32./9.*m44*
         m66*L6r*RS14e*RSe61 - 16./9.*pow(m44,2)*L8r*RS16e*RSe63 + 16./
         9.*pow(m44,2)*L8r*RS16e*RSe61 - 32./9.*pow(m44,2)*L7r*RS16e*
         RSe63 + 32./9.*pow(m44,2)*L7r*RS16e*RSe61 - 32./9.*pow(m44,2)*
         L6r*RS16e*RSe63 + 32./9.*pow(m44,2)*L6r*RS16e*RSe61 - 16./3.*
         m11*m66*L7r*RSe63 + 16./3.*m11*m66*L7r*RSe61 - 16./3.*m11*m66*
         L6r*RSe6613 + 16./3.*m11*m66*L6r*RSe6611 - 8./9.*m11*m66*L5r*
         RS14e*RSe63 + 8./9.*m11*m66*L5r*RS14e*RSe61 + 8./3.*m11*m66*
         L4r*RSe6613 );

      F13p6x22LV +=  + BbV(m11,me2,L) * (  - 8./3.*m11*m66*L4r*RSe6611
          + 8./9.*m11*m66*L4r*RS16e*RSe63 - 8./9.*m11*m66*L4r*RS16e*
         RSe61 - 8./9.*m11*m66*L4r*RS14e*RSe63 + 8./9.*m11*m66*L4r*
         RS14e*RSe61 + 16./3.*m11*m44*L7r*RSe63 - 16./3.*m11*m44*L7r*
         RSe61 - 32./3.*m11*m44*L6r*RSe6613 + 32./3.*m11*m44*L6r*
         RSe6611 + 8./9.*m11*m44*L5r*RS16e*RSe63 - 8./9.*m11*m44*L5r*
         RS16e*RSe61 + 16./3.*m11*m44*L4r*RSe6613 - 16./3.*m11*m44*L4r*
         RSe6611 + 16./9.*m11*m44*L4r*RS16e*RSe63 - 16./9.*m11*m44*L4r*
         RS16e*RSe61 - 16./9.*m11*m44*L4r*RS14e*RSe63 + 16./9.*m11*m44*
         L4r*RS14e*RSe61 - 16./3.*pow(m11,2)*L8r*RSe6613 + 16./3.*pow(
         m11,2)*L8r*RSe6611 + 8./3.*pow(m11,2)*L5r*RSe6613 - 8./3.*pow(
         m11,2)*L5r*RSe6611 );

      F13p6x22LV +=  + BbV(m11,L) * ( m66*L4r*RD146e + 4./9.*pow(m66,2)
         *L8r*pow(RS14e,2) + 4./9.*pow(m66,2)*L7r*pow(RS14e,2) + 4./9.*
         pow(m66,2)*L6r*pow(RS14e,2) + 2*m44*L4r*RD146e + 16./9.*m44*
         m66*L7r*RS14e*RS16e + 8./9.*m44*m66*L6r*pow(RS16e,2) + 8./9.*
         m44*m66*L6r*pow(RS14e,2) + 8./9.*pow(m44,2)*L8r*pow(RS16e,2)
          + 16./9.*pow(m44,2)*L7r*pow(RS16e,2) + 16./9.*pow(m44,2)*L6r*
         pow(RS16e,2) - 1./6.*m33*L5r*RD146e + 1./2.*m11*L5r*RD146e + 5.
         /3.*m11*L3r*RD146e + 2./3.*m11*L0r*RD146e - 8./3.*m11*m66*L7r*
         RS14e + 8./3.*m11*m66*L6r*RS1463e - 8./3.*m11*m66*L6r*RC146e
          - 2./9.*m11*m66*L5r*pow(RS14e,2) - 4./3.*m11*m66*L4r*RS1463e
          - 4./9.*m11*m66*L4r*pow(RS16e,2) - 2./9.*m11*m66*L4r*pow(
         RS14e,2) + 4./3.*m11*m66*L4r*RC146e - 16./3.*m11*m44*L7r*RS16e
          + 16./3.*m11*m44*L6r*RS1463e - 16./3.*m11*m44*L6r*RC146e - 4./
         9.*m11*m44*L5r*pow(RS16e,2) - 8./3.*m11*m44*L4r*RS1463e - 8./9.
         *m11*m44*L4r*pow(RS16e,2) - 4./9.*m11*m44*L4r*pow(RS14e,2) + 8.
         /3.*m11*m44*L4r*RC146e );

      F13p6x22LV +=  + BbV(m11,L) * ( 8./3.*pow(m11,2)*L8r*RS1463e - 8./
         3.*pow(m11,2)*L8r*RC146e + 4*pow(m11,2)*L7r - 4./3.*pow(m11,2)
         *L5r*RS1463e + 4./3.*pow(m11,2)*L5r*RC146e );

      F13p6x22LV +=  + BbV(m14,L) * ( 4*m44*m66*L6r - 2*m44*m66*L4r + 2
         *pow(m44,2)*L8r + 8*pow(m44,2)*L6r - pow(m44,2)*L5r - 4*pow(
         m44,2)*L4r + 4*m11*m66*L6r - 2*m11*m66*L4r + 4*m11*m44*L8r + 8
         *m11*m44*L6r - 2*m11*m44*L5r - 4*m11*m44*L4r + 2*pow(m11,2)*
         L8r - pow(m11,2)*L5r );

      F13p6x22LV +=  + BbV(m16,L) * ( pow(m66,2)*L8r + 2*pow(m66,2)*L6r
          - 1./2.*pow(m66,2)*L5r - pow(m66,2)*L4r + 4*m44*m66*L6r - 2*
         m44*m66*L4r + 2*m11*m66*L8r + 2*m11*m66*L6r - m11*m66*L5r - 
         m11*m66*L4r + 4*m11*m44*L6r - 2*m11*m44*L4r + pow(m11,2)*L8r
          - 1./2.*pow(m11,2)*L5r );

      F13p6x22LV +=  + BbV(m33,me2,L) * (  - 16./9.*pow(m66,2)*L8r*
         RS34e*RSe63 + 16./9.*pow(m66,2)*L8r*RSe61*RS34e - 16./9.*pow(
         m66,2)*L7r*RS34e*RSe63 + 16./9.*pow(m66,2)*L7r*RSe61*RS34e - 
         16./9.*pow(m66,2)*L6r*RS34e*RSe63 + 16./9.*pow(m66,2)*L6r*
         RSe61*RS34e - 32./9.*m44*m66*L7r*RS36e*RSe63 + 16./9.*m44*m66*
         L7r*RS34e*RSe63 + 32./9.*m44*m66*L7r*RSe61*RS36e - 16./9.*m44*
         m66*L7r*RSe61*RS34e + 16./9.*m44*m66*L6r*RS36e*RSe63 - 32./9.*
         m44*m66*L6r*RS34e*RSe63 - 16./9.*m44*m66*L6r*RSe61*RS36e + 32./
         9.*m44*m66*L6r*RSe61*RS34e + 16./9.*pow(m44,2)*L8r*RS36e*RSe63
          - 16./9.*pow(m44,2)*L8r*RSe61*RS36e + 32./9.*pow(m44,2)*L7r*
         RS36e*RSe63 - 32./9.*pow(m44,2)*L7r*RSe61*RS36e + 32./9.*pow(
         m44,2)*L6r*RS36e*RSe63 - 32./9.*pow(m44,2)*L6r*RSe61*RS36e + 
         16./3.*m33*m66*L7r*RSe63 - 16./3.*m33*m66*L7r*RSe61 - 16./3.*
         m33*m66*L6r*RSe6613 + 16./3.*m33*m66*L6r*RSe6633 + 8./9.*m33*
         m66*L5r*RS34e*RSe63 - 8./9.*m33*m66*L5r*RSe61*RS34e + 8./3.*
         m33*m66*L4r*RSe6613 );

      F13p6x22LV +=  + BbV(m33,me2,L) * (  - 8./3.*m33*m66*L4r*RSe6633
          - 8./9.*m33*m66*L4r*RS36e*RSe63 + 8./9.*m33*m66*L4r*RS34e*
         RSe63 + 8./9.*m33*m66*L4r*RSe61*RS36e - 8./9.*m33*m66*L4r*
         RSe61*RS34e - 16./3.*m33*m44*L7r*RSe63 + 16./3.*m33*m44*L7r*
         RSe61 - 32./3.*m33*m44*L6r*RSe6613 + 32./3.*m33*m44*L6r*
         RSe6633 - 8./9.*m33*m44*L5r*RS36e*RSe63 + 8./9.*m33*m44*L5r*
         RSe61*RS36e + 16./3.*m33*m44*L4r*RSe6613 - 16./3.*m33*m44*L4r*
         RSe6633 - 16./9.*m33*m44*L4r*RS36e*RSe63 + 16./9.*m33*m44*L4r*
         RS34e*RSe63 + 16./9.*m33*m44*L4r*RSe61*RS36e - 16./9.*m33*m44*
         L4r*RSe61*RS34e - 16./3.*pow(m33,2)*L8r*RSe6613 + 16./3.*pow(
         m33,2)*L8r*RSe6633 + 8./3.*pow(m33,2)*L5r*RSe6613 - 8./3.*pow(
         m33,2)*L5r*RSe6633 );

      F13p6x22LV +=  + BbV(m33,L) * ( m66*L4r*RD346e + 4./9.*pow(m66,2)
         *L8r*pow(RS34e,2) + 4./9.*pow(m66,2)*L7r*pow(RS34e,2) + 4./9.*
         pow(m66,2)*L6r*pow(RS34e,2) + 2*m44*L4r*RD346e + 16./9.*m44*
         m66*L7r*RS34e*RS36e + 8./9.*m44*m66*L6r*pow(RS36e,2) + 8./9.*
         m44*m66*L6r*pow(RS34e,2) + 8./9.*pow(m44,2)*L8r*pow(RS36e,2)
          + 16./9.*pow(m44,2)*L7r*pow(RS36e,2) + 16./9.*pow(m44,2)*L6r*
         pow(RS36e,2) + 1./2.*m33*L5r*RD346e + 5./3.*m33*L3r*RD346e + 2.
         /3.*m33*L0r*RD346e - 8./3.*m33*m66*L7r*RS34e + 8./3.*m33*m66*
         L6r*RS3461e - 8./3.*m33*m66*L6r*RC346e - 2./9.*m33*m66*L5r*
         pow(RS34e,2) - 4./3.*m33*m66*L4r*RS3461e - 4./9.*m33*m66*L4r*
         pow(RS36e,2) - 2./9.*m33*m66*L4r*pow(RS34e,2) + 4./3.*m33*m66*
         L4r*RC346e - 16./3.*m33*m44*L7r*RS36e + 16./3.*m33*m44*L6r*
         RS3461e - 16./3.*m33*m44*L6r*RC346e - 4./9.*m33*m44*L5r*pow(
         RS36e,2) - 8./3.*m33*m44*L4r*RS3461e - 8./9.*m33*m44*L4r*pow(
         RS36e,2) - 4./9.*m33*m44*L4r*pow(RS34e,2) + 8./3.*m33*m44*L4r*
         RC346e );

      F13p6x22LV +=  + BbV(m33,L) * ( 8./3.*pow(m33,2)*L8r*RS3461e - 8./
         3.*pow(m33,2)*L8r*RC346e + 4*pow(m33,2)*L7r - 4./3.*pow(m33,2)
         *L5r*RS3461e + 4./3.*pow(m33,2)*L5r*RC346e - 1./6.*m11*L5r*
         RD346e );

      F13p6x22LV +=  + BbV(m34,L) * ( 4*m44*m66*L6r - 2*m44*m66*L4r + 2
         *pow(m44,2)*L8r + 8*pow(m44,2)*L6r - pow(m44,2)*L5r - 4*pow(
         m44,2)*L4r + 4*m33*m66*L6r - 2*m33*m66*L4r + 4*m33*m44*L8r + 8
         *m33*m44*L6r - 2*m33*m44*L5r - 4*m33*m44*L4r + 2*pow(m33,2)*
         L8r - pow(m33,2)*L5r );

      F13p6x22LV +=  + BbV(m36,L) * ( pow(m66,2)*L8r + 2*pow(m66,2)*L6r
          - 1./2.*pow(m66,2)*L5r - pow(m66,2)*L4r + 4*m44*m66*L6r - 2*
         m44*m66*L4r + 2*m33*m66*L8r + 2*m33*m66*L6r - m33*m66*L5r - 
         m33*m66*L4r + 4*m33*m44*L6r - 2*m33*m44*L4r + pow(m33,2)*L8r
          - 1./2.*pow(m33,2)*L5r );

      F13p6x22LV +=  + BbV(me2,L) * (  - 8./9.*m66*me2*L5r*pow(RSe63,2)
          + 16./9.*m66*me2*L5r*RSe61*RSe63 - 8./9.*m66*me2*L5r*pow(
         RSe61,2) - 4./3.*m66*me2*L4r*pow(RSe63,2) + 8./3.*m66*me2*L4r*
         RSe61*RSe63 - 4./3.*m66*me2*L4r*pow(RSe61,2) + 16./9.*pow(
         m66,2)*L8r*pow(RSe63,2) - 32./9.*pow(m66,2)*L8r*RSe61*RSe63 + 
         16./9.*pow(m66,2)*L8r*pow(RSe61,2) + 16./9.*pow(m66,2)*L7r*
         pow(RSe63,2) - 32./9.*pow(m66,2)*L7r*RSe61*RSe63 + 16./9.*pow(
         m66,2)*L7r*pow(RSe61,2) + 16./9.*pow(m66,2)*L6r*pow(RSe63,2)
          - 32./9.*pow(m66,2)*L6r*RSe61*RSe63 + 16./9.*pow(m66,2)*L6r*
         pow(RSe61,2) - 4./9.*m44*me2*L5r*pow(RSe63,2) + 8./9.*m44*me2*
         L5r*RSe61*RSe63 - 4./9.*m44*me2*L5r*pow(RSe61,2) - 8./3.*m44*
         me2*L4r*pow(RSe63,2) + 16./3.*m44*me2*L4r*RSe61*RSe63 - 8./3.*
         m44*me2*L4r*pow(RSe61,2) - 32./9.*m44*m66*L7r*pow(RSe63,2) + 
         64./9.*m44*m66*L7r*RSe61*RSe63 - 32./9.*m44*m66*L7r*pow(
         RSe61,2) + 40./9.*m44*m66*L6r*pow(RSe63,2) - 80./9.*m44*m66*
         L6r*RSe61*RSe63 );

      F13p6x22LV +=  + BbV(me2,L) * ( 40./9.*m44*m66*L6r*pow(RSe61,2)
          + 8./9.*pow(m44,2)*L8r*pow(RSe63,2) - 16./9.*pow(m44,2)*L8r*
         RSe61*RSe63 + 8./9.*pow(m44,2)*L8r*pow(RSe61,2) + 16./9.*pow(
         m44,2)*L7r*pow(RSe63,2) - 32./9.*pow(m44,2)*L7r*RSe61*RSe63 + 
         16./9.*pow(m44,2)*L7r*pow(RSe61,2) + 16./9.*pow(m44,2)*L6r*
         pow(RSe63,2) - 32./9.*pow(m44,2)*L6r*RSe61*RSe63 + 16./9.*pow(
         m44,2)*L6r*pow(RSe61,2) );

      F13p6x22LV +=  + CbV(m11,L) * (  - 8./3.*m11*m66*L6r*RD146e + 4./
         3.*m11*m66*L4r*RD146e - 16./3.*m11*m44*L6r*RD146e + 8./3.*m11*
         m44*L4r*RD146e - 8./3.*pow(m11,2)*L8r*RD146e + 4./3.*pow(
         m11,2)*L5r*RD146e );

      F13p6x22LV +=  + CbV(m33,L) * (  - 8./3.*m33*m66*L6r*RD346e + 4./
         3.*m33*m66*L4r*RD346e - 16./3.*m33*m44*L6r*RD346e + 8./3.*m33*
         m44*L4r*RD346e - 8./3.*pow(m33,2)*L8r*RD346e + 4./3.*pow(
         m33,2)*L5r*RD346e );

      F13p6x22LV +=  + A23bV(m11,L) * (  - 2*L3r*RS1463e - L3r*RC146e
          - 2*L0r*RC146e );

      F13p6x22LV +=  + A23bV(m13,L) * ( 6*L2r + 12*L1r );

      F13p6x22LV +=  + A23bV(m14,L) * ( 6*L3r + 12*L0r );

      F13p6x22LV +=  + A23bV(m16,L) * ( 3*L3r + 6*L0r );

      F13p6x22LV +=  + A23bV(m33,L) * (  - 2*L3r*RS3461e - L3r*RC346e
          - 2*L0r*RC346e );

      F13p6x22LV +=  + A23bV(m34,L) * ( 6*L3r + 12*L0r );

      F13p6x22LV +=  + A23bV(m36,L) * ( 3*L3r + 6*L0r );

      F13p6x22LV +=  + A23bV(m44,L) * ( 18*L2r );

      F13p6x22LV +=  + A23bV(m46,L) * ( 24*L2r );

      F13p6x22LV +=  + A23bV(me2,L) * ( 4*L3r*RSe6613 + 2*L3r*RSe6633
          + 2*L3r*RSe6611 + 6*L2r + 4*L0r*RSe6633 + 4*L0r*RSe6611 );

      F13p6x22LV +=  + B23bV(m11,L) * (  - L3r*RD146e - 2*L0r*RD146e );

      F13p6x22LV +=  + B23bV(m33,L) * (  - L3r*RD346e - 2*L0r*RD346e );

  return F13p6x22LV/pow(f0,4);
}


double fv2s2nf3p6RV(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 4) || (B0mq.size() != 4)){
    cout << "ERROR: fv2s2nf3p6RV probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m66 = 2.*B0mq[3];
  double m13 = (m11+m33)/2;
  double m14 = (m11+m44)/2;
  double m16 = (m11+m66)/2;
  double m34 = (m33+m44)/2;
  double m36 = (m33+m66)/2;
  double m46 = (m44+m66)/2;
  double me2 = (m44+2.*m66)/3.;

  double RN13 = RN(m11,m33);
  double RN1e = RN(m11,me2);
  double RN3e = RN(m33,me2);

  //double RD13 = m11-m33;
  //double RD14 = m11-m44;
  //double RD16 = m11-m66;
  //double RD34 = m33-m44;
  //double RD36 = m33-m66;
  //double RD46 = m44-m66;

  double RD146e = RD(m11,m44,m66,me2);
  double RD346e = RD(m33,m44,m66,me2);

  double RC146e = RC(m11,m44,m66,me2);
  double RC346e = RC(m33,m44,m66,me2);

  double RS14e = RS(m11,m44,me2);
  double RS16e = RS(m11,m66,me2);
  double RS34e = RS(m33,m44,me2);
  double RS36e = RS(m33,m66,me2);
  double RSe61 = RS(me2,m66,m11);
  double RSe63 = RS(me2,m66,m33);

  double RS1463e = RS(m11,m44,m66,m33,me2);
  double RS3461e = RS(m33,m44,m66,m11,me2);
  double RSe6611 = RS(me2,m66,m66,m11,m11);
  double RSe6613 = RS(me2,m66,m66,m11,m33);
  double RSe6633 = RS(me2,m66,m66,m33,m33);

  double mu2 = mu0*mu0;

  double F13p6x22RV =
       + Ab(m11,mu2)*AbV(m11,L) * ( 1./8. + 1./36.*pow(RS1463e,2) - 1./
         36.*RC146e*RS1463e + 1./144.*pow(RC146e,2) + 1./9.*m11*RN1e*
         RC146e*RSe6613 - 1./9.*m11*RN1e*RC146e*RSe6611 - 1./18.*m11*
         RN13*RC146e*RS3461e );

      F13p6x22RV +=  + Ab(m11,mu2)*AbV(m13,L) * (  - 1./24. + 1./162.*
         RS1463e*RS3461e + 1./162.*pow(RS1463e,2) - 1./81.*RSe6613*
         RS1463e + 1./162.*RSe6633*RS1463e - 1./324.*RC346e*RS1463e + 1.
         /162.*RSe6611*RS1463e - 1./324.*RC146e*RS3461e - 1./162.*
         RC146e*RS1463e + 1./162.*RC146e*RSe6613 - 1./324.*RC146e*
         RSe6633 + 1./648.*RC146e*RC346e - 1./324.*RC146e*RSe6611 + 1./
         648.*pow(RC146e,2) - 1./6.*m33*RN13 - 1./3.*m11*RN13 );

      F13p6x22RV +=  + Ab(m11,mu2)*AbV(m14,L) * (  - 1./18.*RS1463e - 5.
         /36.*RS16e + 7./72.*RC146e + 1./9.*m44*RN1e*RSe63 - 1./9.*m44*
         RN1e*RSe61 + 1./9.*m44*RN13*RS36e + 2./9.*m11*RN1e*RSe63 - 2./
         9.*m11*RN1e*RSe61 + 2./9.*m11*RN13*RS36e );

      F13p6x22RV +=  + Ab(m11,mu2)*AbV(m16,L) * (  - 1./36.*RS1463e - 5.
         /72.*RS14e + 7./144.*RC146e - 1./9.*m66*RN1e*RSe63 + 1./9.*m66
         *RN1e*RSe61 + 1./18.*m66*RN13*RS34e - 2./9.*m11*RN1e*RSe63 + 2.
         /9.*m11*RN1e*RSe61 + 1./9.*m11*RN13*RS34e );

      F13p6x22RV +=  + Ab(m11,mu2)*AbV(m33,L) * ( 1./36.*RS1463e*
         RS3461e - 1./72.*RC346e*RS1463e - 1./72.*RC146e*RS3461e + 1./
         144.*RC146e*RC346e - 1./18.*m33*RN13*RC346e*RS1463e );

      F13p6x22RV +=  + Ab(m11,mu2)*AbV(m34,L) * ( 1./18.*RS1463e + 1./
         36.*RS16e - 1./24.*RC146e + 1./9.*m44*RN13*RS16e + 1./9.*m33*
         RN13*RS1463e + 1./9.*m33*RN13*RS16e - 1./9.*m11*RN13*RS1463e
          + 1./9.*m11*RN13*RS16e );

      F13p6x22RV +=  + Ab(m11,mu2)*AbV(m36,L) * ( 1./36.*RS1463e + 1./
         72.*RS14e - 1./48.*RC146e + 1./18.*m66*RN13*RS14e + 1./18.*m33
         *RN13*RS1463e + 1./18.*m33*RN13*RS14e - 1./18.*m11*RN13*
         RS1463e + 1./18.*m11*RN13*RS14e );

      F13p6x22RV +=  + Ab(m11,mu2)*AbV(m44,L) * (  - 1./6.*m44*RN1e*
         RS16e*RSe63 + 1./6.*m44*RN1e*RS16e*RSe61 - 1./6.*m44*RN13*
         RS16e*RS36e );

      F13p6x22RV +=  + Ab(m11,mu2)*AbV(m46,L) * (  - 1./54.*pow(
         RS16e,2) + 1./27.*RS14e*RS16e - 1./54.*pow(RS14e,2) + 2./27.*
         m66*RN1e*RS16e*RSe63 - 2./27.*m66*RN1e*RS16e*RSe61 + 1./27.*
         m66*RN1e*RS14e*RSe63 - 1./27.*m66*RN1e*RS14e*RSe61 - 1./27.*
         m66*RN13*RS16e*RS34e - 1./27.*m66*RN13*RS14e*RS36e - 1./27.*
         m66*RN13*RS14e*RS34e + 1./27.*m44*RN1e*RS16e*RSe63 - 1./27.*
         m44*RN1e*RS16e*RSe61 - 1./27.*m44*RN1e*RS14e*RSe63 + 1./27.*
         m44*RN1e*RS14e*RSe61 - 1./27.*m44*RN13*RS16e*RS36e - 1./27.*
         m44*RN13*RS16e*RS34e - 1./27.*m44*RN13*RS14e*RS36e + 1./9.*m11
         *RN1e*RS16e*RSe63 - 1./9.*m11*RN1e*RS16e*RSe61 - 1./9.*m11*
         RN1e*RS14e*RSe63 + 1./9.*m11*RN1e*RS14e*RSe61 + 1./27.*m11*
         RN13*RS16e*RS36e - 1./27.*m11*RN13*RS16e*RS34e - 1./27.*m11*
         RN13*RS14e*RS36e + 1./27.*m11*RN13*RS14e*RS34e );

      F13p6x22RV +=  + Ab(m11,mu2)*AbV(me2,L) * (  - 1./18.*RSe6613*
         RS1463e + 1./36.*RSe6633*RS1463e + 1./36.*RSe6611*RS1463e + 1./
         36.*RC146e*RSe6613 - 1./72.*RC146e*RSe6633 - 1./72.*RC146e*
         RSe6611 + 2./27.*m66*RN1e*RS14e*RSe63 - 2./27.*m66*RN1e*RS14e*
         RSe61 - 1./27.*m66*RN13*RS14e*RS34e - 1./54.*m44*RN1e*RS16e*
         RSe63 + 1./54.*m44*RN1e*RS16e*RSe61 - 1./54.*m44*RN13*RS16e*
         RS36e + 1./9.*m33*RN13*RSe6633*RS1463e - 2./9.*m11*RN1e*
         RSe6611*RSe6613 + 2./9.*m11*RN1e*pow(RSe6611,2) + 1./9.*m11*
         RN13*RSe6611*RS3461e );

      F13p6x22RV +=  + Ab(m11,mu2)*BbV(m11,m33,L) * (  - 1./18.*m11*
         RC146e*RS3461e );

      F13p6x22RV +=  + Ab(m11,mu2)*BbV(m11,me2,L) * ( 1./9.*m11*RC146e*
         RSe6613 - 1./9.*m11*RC146e*RSe6611 );

      F13p6x22RV +=  + Ab(m11,mu2)*BbV(m11,L) * (  - 1./72.*RD146e*
         RS1463e + 1./144.*RD146e*RC146e + 1./4.*m11 - 1./18.*m11*
         RC146e*RS1463e + 1./18.*m11*pow(RC146e,2) + 1./9.*m11*RN1e*
         RD146e*RSe6613 - 1./9.*m11*RN1e*RD146e*RSe6611 - 1./18.*m11*
         RN13*RD146e*RS3461e );

      F13p6x22RV +=  + Ab(m11,mu2)*BbV(m14,L) * ( 1./36.*RD146e - 1./18.
         *m44*RS16e - 1./9.*m11*RS16e );

      F13p6x22RV +=  + Ab(m11,mu2)*BbV(m16,L) * ( 1./72.*RD146e - 1./36.
         *m66*RS14e - 1./18.*m11*RS14e );

      F13p6x22RV +=  + Ab(m11,mu2)*BbV(m33,L) * (  - 1./72.*RD346e*
         RS1463e + 1./144.*RC146e*RD346e - 1./18.*m33*RN13*RD346e*
         RS1463e );

      F13p6x22RV +=  + Ab(m11,mu2)*CbV(m11,L) * ( 1./18.*m11*RD146e*
         RC146e );

      F13p6x22RV +=  + Ab(m13,mu2)*AbV(m11,L) * (  - 1./24. + 1./162.*
         RS1463e*RS3461e + 1./162.*pow(RS1463e,2) - 1./81.*RSe6613*
         RS1463e + 1./162.*RSe6633*RS1463e - 1./324.*RC346e*RS1463e + 1.
         /162.*RSe6611*RS1463e - 1./324.*RC146e*RS3461e - 1./162.*
         RC146e*RS1463e + 1./162.*RC146e*RSe6613 - 1./324.*RC146e*
         RSe6633 + 1./648.*RC146e*RC346e - 1./324.*RC146e*RSe6611 + 1./
         648.*pow(RC146e,2) );

      F13p6x22RV +=  + Ab(m13,mu2)*AbV(m13,L) * ( 1./4. );

      F13p6x22RV +=  + Ab(m13,mu2)*AbV(m33,L) * (  - 5./24. + 1./162.*
         pow(RS3461e,2) + 1./162.*RS1463e*RS3461e - 1./81.*RSe6613*
         RS3461e + 1./162.*RSe6633*RS3461e - 1./162.*RC346e*RS3461e - 1.
         /324.*RC346e*RS1463e + 1./162.*RC346e*RSe6613 - 1./324.*RC346e
         *RSe6633 + 1./648.*pow(RC346e,2) + 1./162.*RSe6611*RS3461e - 1.
         /324.*RSe6611*RC346e - 1./324.*RC146e*RS3461e + 1./648.*RC146e
         *RC346e );

      F13p6x22RV +=  + Ab(m13,mu2)*AbV(me2,L) * (  - 1./81.*RSe6613*
         RS3461e - 1./81.*RSe6613*RS1463e + 2./81.*pow(RSe6613,2) + 1./
         162.*RSe6633*RS3461e + 1./162.*RSe6633*RS1463e - 2./81.*
         RSe6633*RSe6613 + 1./162.*pow(RSe6633,2) + 1./162.*RC346e*
         RSe6613 - 1./324.*RC346e*RSe6633 + 1./162.*RSe6611*RS3461e + 1.
         /162.*RSe6611*RS1463e - 2./81.*RSe6611*RSe6613 + 1./81.*
         RSe6611*RSe6633 - 1./324.*RSe6611*RC346e + 1./162.*pow(
         RSe6611,2) + 1./162.*RC146e*RSe6613 - 1./324.*RC146e*RSe6633
          - 1./324.*RC146e*RSe6611 );

      F13p6x22RV +=  + Ab(m13,mu2)*BbV(m11,m33,L) * (  - 1./6.*m33 - 1./
         3.*m11 );

      F13p6x22RV +=  + Ab(m13,mu2)*BbV(m11,L) * (  - 1./324.*RD146e*
         RS3461e - 1./324.*RD146e*RS1463e + 1./162.*RD146e*RSe6613 - 1./
         324.*RD146e*RSe6633 + 1./648.*RD146e*RC346e - 1./324.*RD146e*
         RSe6611 + 1./648.*RD146e*RC146e );

      F13p6x22RV +=  + Ab(m13,mu2)*BbV(m33,L) * (  - 1./324.*RD346e*
         RS3461e - 1./324.*RD346e*RS1463e + 1./162.*RD346e*RSe6613 - 1./
         324.*RD346e*RSe6633 + 1./648.*RD346e*RC346e - 1./324.*RSe6611*
         RD346e + 1./648.*RC146e*RD346e );

      F13p6x22RV +=  + Ab(m14,mu2)*AbV(m11,L) * (  - 1./18.*RS1463e - 5.
         /36.*RS16e + 7./72.*RC146e );

      F13p6x22RV +=  + Ab(m14,mu2)*AbV(m14,L) * (  - 1./4. );

      F13p6x22RV +=  + Ab(m14,mu2)*AbV(m16,L) * (  - 1./8. );

      F13p6x22RV +=  + Ab(m14,mu2)*AbV(m33,L) * (  - 1./18.*RS3461e + 5.
         /36.*RS36e - 1./24.*RC346e );

      F13p6x22RV +=  + Ab(m14,mu2)*AbV(m34,L) * ( 8./27. + 7./108.*
         RS3461e + 7./108.*RS1463e - 7./54.*RSe6613 - 5./108.*RSe63 + 1.
         /54.*RSe6633 - 5./108.*RS36e - 1./108.*RC346e - 5./108.*RSe61
          + 1./54.*RSe6611 - 5./108.*RS16e - 1./108.*RC146e );

      F13p6x22RV +=  + Ab(m14,mu2)*AbV(m36,L) * ( 1./8. );

      F13p6x22RV +=  + Ab(m14,mu2)*AbV(me2,L) * ( 1./9.*RSe6613 + 5./36.
         *RSe63 + 1./12.*RSe6633 - 5./36.*RSe61 - 7./36.*RSe6611 );

      F13p6x22RV +=  + Ab(m14,mu2)*BbV(m11,m33,L) * ( 1./9.*m44*RS36e
          + 2./9.*m11*RS36e );

      F13p6x22RV +=  + Ab(m14,mu2)*BbV(m11,me2,L) * ( 1./9.*m44*RSe63
          - 1./9.*m44*RSe61 + 2./9.*m11*RSe63 - 2./9.*m11*RSe61 );

      F13p6x22RV +=  + Ab(m14,mu2)*BbV(m11,L) * ( 7./72.*RD146e - 1./9.
         *m44*RS16e - 2./9.*m11*RS16e );

      F13p6x22RV +=  + Ab(m14,mu2)*BbV(m33,L) * (  - 1./24.*RD346e );

      F13p6x22RV +=  + Ab(m16,mu2)*AbV(m11,L) * (  - 1./36.*RS1463e - 5.
         /72.*RS14e + 7./144.*RC146e );

      F13p6x22RV +=  + Ab(m16,mu2)*AbV(m14,L) * (  - 1./8. );

      F13p6x22RV +=  + Ab(m16,mu2)*AbV(m16,L) * (  - 1./16. );

      F13p6x22RV +=  + Ab(m16,mu2)*AbV(m33,L) * (  - 1./36.*RS3461e + 5.
         /72.*RS34e - 1./48.*RC346e );

      F13p6x22RV +=  + Ab(m16,mu2)*AbV(m34,L) * ( 1./8. );

      F13p6x22RV +=  + Ab(m16,mu2)*AbV(m36,L) * ( 37./432. + 7./216.*
         RS3461e + 7./216.*RS1463e - 7./108.*RSe6613 + 5./108.*RSe63 + 
         1./108.*RSe6633 - 5./216.*RS34e - 1./216.*RC346e + 5./108.*
         RSe61 + 1./108.*RSe6611 - 5./216.*RS14e - 1./216.*RC146e );

      F13p6x22RV +=  + Ab(m16,mu2)*AbV(me2,L) * ( 1./18.*RSe6613 - 5./
         36.*RSe63 + 1./24.*RSe6633 + 5./36.*RSe61 - 7./72.*RSe6611 );

      F13p6x22RV +=  + Ab(m16,mu2)*BbV(m11,m33,L) * ( 1./18.*m66*RS34e
          + 1./9.*m11*RS34e );

      F13p6x22RV +=  + Ab(m16,mu2)*BbV(m11,me2,L) * (  - 1./9.*m66*
         RSe63 + 1./9.*m66*RSe61 - 2./9.*m11*RSe63 + 2./9.*m11*RSe61 );

      F13p6x22RV +=  + Ab(m16,mu2)*BbV(m11,L) * ( 7./144.*RD146e - 1./
         18.*m66*RS14e - 1./9.*m11*RS14e );

      F13p6x22RV +=  + Ab(m16,mu2)*BbV(m33,L) * (  - 1./48.*RD346e );

      F13p6x22RV +=  + Ab(m33,mu2)*AbV(m11,L) * ( 1./36.*RS1463e*
         RS3461e - 1./72.*RC346e*RS1463e - 1./72.*RC146e*RS3461e + 1./
         144.*RC146e*RC346e + 1./18.*m11*RN13*RC146e*RS3461e );

      F13p6x22RV +=  + Ab(m33,mu2)*AbV(m13,L) * (  - 5./24. + 1./162.*
         pow(RS3461e,2) + 1./162.*RS1463e*RS3461e - 1./81.*RSe6613*
         RS3461e + 1./162.*RSe6633*RS3461e - 1./162.*RC346e*RS3461e - 1.
         /324.*RC346e*RS1463e + 1./162.*RC346e*RSe6613 - 1./324.*RC346e
         *RSe6633 + 1./648.*pow(RC346e,2) + 1./162.*RSe6611*RS3461e - 1.
         /324.*RSe6611*RC346e - 1./324.*RC146e*RS3461e + 1./648.*RC146e
         *RC346e + 1./6.*m33*RN13 + 1./3.*m11*RN13 );

      F13p6x22RV +=  + Ab(m33,mu2)*AbV(m14,L) * (  - 1./18.*RS3461e + 5.
         /36.*RS36e - 1./24.*RC346e - 1./9.*m44*RN13*RS36e - 2./9.*m11*
         RN13*RS36e );

      F13p6x22RV +=  + Ab(m33,mu2)*AbV(m16,L) * (  - 1./36.*RS3461e + 5.
         /72.*RS34e - 1./48.*RC346e - 1./18.*m66*RN13*RS34e - 1./9.*m11
         *RN13*RS34e );

      F13p6x22RV +=  + Ab(m33,mu2)*AbV(m33,L) * ( 1./8. + 1./36.*pow(
         RS3461e,2) - 1./36.*RC346e*RS3461e + 1./144.*pow(RC346e,2) + 1.
         /9.*m33*RN3e*RC346e*RSe6613 - 1./9.*m33*RN3e*RC346e*RSe6633 + 
         1./18.*m33*RN13*RC346e*RS1463e );

      F13p6x22RV +=  + Ab(m33,mu2)*AbV(m34,L) * (  - 1./18.*RS3461e - 1.
         /9.*RS1463e - 5./36.*RS36e + 7./72.*RC346e + 1./9.*RS16e - 1./
         9.*m44*RN3e*RSe63 + 1./9.*m44*RN3e*RSe61 - 1./9.*m44*RN13*
         RS16e - 2./9.*m33*RN3e*RSe63 + 2./9.*m33*RN3e*RSe61 - 1./9.*
         m33*RN13*RS1463e - 1./9.*m33*RN13*RS16e + 1./9.*m11*RN13*
         RS1463e - 1./9.*m11*RN13*RS16e );

      F13p6x22RV +=  + Ab(m33,mu2)*AbV(m36,L) * (  - 1./36.*RS3461e - 1.
         /18.*RS1463e - 5./72.*RS34e + 7./144.*RC346e + 1./18.*RS14e + 
         1./9.*m66*RN3e*RSe63 - 1./9.*m66*RN3e*RSe61 - 1./18.*m66*RN13*
         RS14e + 2./9.*m33*RN3e*RSe63 - 2./9.*m33*RN3e*RSe61 - 1./18.*
         m33*RN13*RS1463e - 1./18.*m33*RN13*RS14e + 1./18.*m11*RN13*
         RS1463e - 1./18.*m11*RN13*RS14e );

      F13p6x22RV +=  + Ab(m33,mu2)*AbV(m44,L) * ( 1./6.*m44*RN3e*RS36e*
         RSe63 - 1./6.*m44*RN3e*RSe61*RS36e + 1./6.*m44*RN13*RS16e*
         RS36e );

      F13p6x22RV +=  + Ab(m33,mu2)*AbV(m46,L) * (  - 1./54.*pow(
         RS36e,2) + 1./27.*RS34e*RS36e - 1./54.*pow(RS34e,2) + 1./27.*
         RS16e*RS36e - 1./27.*RS16e*RS34e - 1./27.*RS14e*RS36e + 1./27.
         *RS14e*RS34e - 2./27.*m66*RN3e*RS36e*RSe63 - 1./27.*m66*RN3e*
         RS34e*RSe63 + 2./27.*m66*RN3e*RSe61*RS36e + 1./27.*m66*RN3e*
         RSe61*RS34e + 1./27.*m66*RN13*RS16e*RS34e + 1./27.*m66*RN13*
         RS14e*RS36e + 1./27.*m66*RN13*RS14e*RS34e - 1./27.*m44*RN3e*
         RS36e*RSe63 + 1./27.*m44*RN3e*RS34e*RSe63 + 1./27.*m44*RN3e*
         RSe61*RS36e - 1./27.*m44*RN3e*RSe61*RS34e + 1./27.*m44*RN13*
         RS16e*RS36e + 1./27.*m44*RN13*RS16e*RS34e + 1./27.*m44*RN13*
         RS14e*RS36e - 1./9.*m33*RN3e*RS36e*RSe63 + 1./9.*m33*RN3e*
         RS34e*RSe63 + 1./9.*m33*RN3e*RSe61*RS36e - 1./9.*m33*RN3e*
         RSe61*RS34e - 1./27.*m11*RN13*RS16e*RS36e + 1./27.*m11*RN13*
         RS16e*RS34e + 1./27.*m11*RN13*RS14e*RS36e - 1./27.*m11*RN13*
         RS14e*RS34e );

      F13p6x22RV +=  + Ab(m33,mu2)*AbV(me2,L) * (  - 1./18.*RSe6613*
         RS3461e + 1./36.*RSe6633*RS3461e + 1./36.*RC346e*RSe6613 - 1./
         72.*RC346e*RSe6633 + 1./36.*RSe6611*RS3461e - 1./72.*RSe6611*
         RC346e - 2./27.*m66*RN3e*RS34e*RSe63 + 2./27.*m66*RN3e*RSe61*
         RS34e + 1./27.*m66*RN13*RS14e*RS34e + 1./54.*m44*RN3e*RS36e*
         RSe63 - 1./54.*m44*RN3e*RSe61*RS36e + 1./54.*m44*RN13*RS16e*
         RS36e - 2./9.*m33*RN3e*RSe6633*RSe6613 + 2./9.*m33*RN3e*pow(
         RSe6633,2) - 1./9.*m33*RN13*RSe6633*RS1463e - 1./9.*m11*RN13*
         RSe6611*RS3461e );

      F13p6x22RV +=  + Ab(m33,mu2)*BbV(m11,m33,L) * (  - 1./18.*m33*
         RC346e*RS1463e );

      F13p6x22RV +=  + Ab(m33,mu2)*BbV(m11,L) * (  - 1./72.*RD146e*
         RS3461e + 1./144.*RD146e*RC346e + 1./18.*m11*RN13*RD146e*
         RS3461e );

      F13p6x22RV +=  + Ab(m33,mu2)*BbV(m33,me2,L) * ( 1./9.*m33*RC346e*
         RSe6613 - 1./9.*m33*RC346e*RSe6633 );

      F13p6x22RV +=  + Ab(m33,mu2)*BbV(m33,L) * (  - 1./72.*RD346e*
         RS3461e + 1./144.*RD346e*RC346e + 1./4.*m33 - 1./18.*m33*
         RC346e*RS3461e + 1./18.*m33*pow(RC346e,2) + 1./9.*m33*RN3e*
         RD346e*RSe6613 - 1./9.*m33*RN3e*RD346e*RSe6633 + 1./18.*m33*
         RN13*RD346e*RS1463e );

      F13p6x22RV +=  + Ab(m33,mu2)*BbV(m34,L) * ( 1./36.*RD346e - 1./18.
         *m44*RS36e - 1./9.*m33*RS36e );

      F13p6x22RV +=  + Ab(m33,mu2)*BbV(m36,L) * ( 1./72.*RD346e - 1./36.
         *m66*RS34e - 1./18.*m33*RS34e );

      F13p6x22RV +=  + Ab(m33,mu2)*CbV(m33,L) * ( 1./18.*m33*RD346e*
         RC346e );

      F13p6x22RV +=  + Ab(m34,mu2)*AbV(m11,L) * ( 1./18.*RS1463e + 1./
         36.*RS16e - 1./24.*RC146e );

      F13p6x22RV +=  + Ab(m34,mu2)*AbV(m14,L) * ( 8./27. + 7./108.*
         RS3461e + 7./108.*RS1463e - 7./54.*RSe6613 - 5./108.*RSe63 + 1.
         /54.*RSe6633 - 5./108.*RS36e - 1./108.*RC346e - 5./108.*RSe61
          + 1./54.*RSe6611 - 5./108.*RS16e - 1./108.*RC146e );

      F13p6x22RV +=  + Ab(m34,mu2)*AbV(m16,L) * ( 1./8. );

      F13p6x22RV +=  + Ab(m34,mu2)*AbV(m33,L) * (  - 1./18.*RS3461e - 1.
         /9.*RS1463e - 5./36.*RS36e + 7./72.*RC346e + 1./9.*RS16e );

      F13p6x22RV +=  + Ab(m34,mu2)*AbV(m34,L) * (  - 1./4. );

      F13p6x22RV +=  + Ab(m34,mu2)*AbV(m36,L) * (  - 1./8. );

      F13p6x22RV +=  + Ab(m34,mu2)*AbV(me2,L) * ( 1./9.*RSe6613 - 5./36.
         *RSe63 - 7./36.*RSe6633 + 5./36.*RSe61 + 1./12.*RSe6611 );

      F13p6x22RV +=  + Ab(m34,mu2)*BbV(m11,m33,L) * ( 1./9.*m44*RS16e
          + 1./9.*m33*RS1463e + 1./9.*m33*RS16e - 1./9.*m11*RS1463e + 1.
         /9.*m11*RS16e );

      F13p6x22RV +=  + Ab(m34,mu2)*BbV(m11,L) * (  - 1./24.*RD146e );

      F13p6x22RV +=  + Ab(m34,mu2)*BbV(m33,me2,L) * (  - 1./9.*m44*
         RSe63 + 1./9.*m44*RSe61 - 2./9.*m33*RSe63 + 2./9.*m33*RSe61 );

      F13p6x22RV +=  + Ab(m34,mu2)*BbV(m33,L) * ( 7./72.*RD346e - 1./9.
         *m44*RS36e - 2./9.*m33*RS36e );

      F13p6x22RV +=  + Ab(m36,mu2)*AbV(m11,L) * ( 1./36.*RS1463e + 1./
         72.*RS14e - 1./48.*RC146e );

      F13p6x22RV +=  + Ab(m36,mu2)*AbV(m14,L) * ( 1./8. );

      F13p6x22RV +=  + Ab(m36,mu2)*AbV(m16,L) * ( 37./432. + 7./216.*
         RS3461e + 7./216.*RS1463e - 7./108.*RSe6613 + 5./108.*RSe63 + 
         1./108.*RSe6633 - 5./216.*RS34e - 1./216.*RC346e + 5./108.*
         RSe61 + 1./108.*RSe6611 - 5./216.*RS14e - 1./216.*RC146e );

      F13p6x22RV +=  + Ab(m36,mu2)*AbV(m33,L) * (  - 1./36.*RS3461e - 1.
         /18.*RS1463e - 5./72.*RS34e + 7./144.*RC346e + 1./18.*RS14e );

      F13p6x22RV +=  + Ab(m36,mu2)*AbV(m34,L) * (  - 1./8. );

      F13p6x22RV +=  + Ab(m36,mu2)*AbV(m36,L) * (  - 1./16. );

      F13p6x22RV +=  + Ab(m36,mu2)*AbV(me2,L) * ( 1./18.*RSe6613 + 5./
         36.*RSe63 - 7./72.*RSe6633 - 5./36.*RSe61 + 1./24.*RSe6611 );

      F13p6x22RV +=  + Ab(m36,mu2)*BbV(m11,m33,L) * ( 1./18.*m66*RS14e
          + 1./18.*m33*RS1463e + 1./18.*m33*RS14e - 1./18.*m11*RS1463e
          + 1./18.*m11*RS14e );

      F13p6x22RV +=  + Ab(m36,mu2)*BbV(m11,L) * (  - 1./48.*RD146e );

      F13p6x22RV +=  + Ab(m36,mu2)*BbV(m33,me2,L) * ( 1./9.*m66*RSe63
          - 1./9.*m66*RSe61 + 2./9.*m33*RSe63 - 2./9.*m33*RSe61 );

      F13p6x22RV +=  + Ab(m36,mu2)*BbV(m33,L) * ( 7./144.*RD346e - 1./
         18.*m66*RS34e - 1./9.*m33*RS34e );

      F13p6x22RV +=  + Ab(m44,mu2)*BbV(m11,m33,L) * (  - 1./6.*m44*
         RS16e*RS36e );

      F13p6x22RV +=  + Ab(m44,mu2)*BbV(m11,me2,L) * (  - 1./6.*m44*
         RS16e*RSe63 + 1./6.*m44*RS16e*RSe61 );

      F13p6x22RV +=  + Ab(m44,mu2)*BbV(m11,L) * ( 1./12.*m44*pow(
         RS16e,2) );

      F13p6x22RV +=  + Ab(m44,mu2)*BbV(m33,me2,L) * ( 1./6.*m44*RS36e*
         RSe63 - 1./6.*m44*RSe61*RS36e );

      F13p6x22RV +=  + Ab(m44,mu2)*BbV(m33,L) * ( 1./12.*m44*pow(
         RS36e,2) );

      F13p6x22RV +=  + Ab(m44,mu2)*BbV(me2,L) * ( 1./12.*m44*pow(
         RSe63,2) - 1./6.*m44*RSe61*RSe63 + 1./12.*m44*pow(RSe61,2) );

      F13p6x22RV +=  + Ab(m46,mu2)*AbV(m11,L) * (  - 1./54.*pow(
         RS16e,2) + 1./27.*RS14e*RS16e - 1./54.*pow(RS14e,2) );

      F13p6x22RV +=  + Ab(m46,mu2)*AbV(m33,L) * (  - 1./54.*pow(
         RS36e,2) + 1./27.*RS34e*RS36e - 1./54.*pow(RS34e,2) + 1./27.*
         RS16e*RS36e - 1./27.*RS16e*RS34e - 1./27.*RS14e*RS36e + 1./27.
         *RS14e*RS34e );

      F13p6x22RV +=  + Ab(m46,mu2)*AbV(me2,L) * (  - 1./6.*pow(RSe63,2)
          - 1./9.*RS36e*RSe63 + 1./9.*RS34e*RSe63 + 1./3.*RSe61*RSe63
          + 1./9.*RSe61*RS36e - 1./9.*RSe61*RS34e - 1./6.*pow(RSe61,2)
          + 1./9.*RS16e*RSe63 - 1./9.*RS16e*RSe61 - 1./9.*RS14e*RSe63
          + 1./9.*RS14e*RSe61 );

      F13p6x22RV +=  + Ab(m46,mu2)*BbV(m11,m33,L) * (  - 1./27.*m66*
         RS16e*RS34e - 1./27.*m66*RS14e*RS36e - 1./27.*m66*RS14e*RS34e
          - 1./27.*m44*RS16e*RS36e - 1./27.*m44*RS16e*RS34e - 1./27.*
         m44*RS14e*RS36e + 1./27.*m11*RS16e*RS36e - 1./27.*m11*RS16e*
         RS34e - 1./27.*m11*RS14e*RS36e + 1./27.*m11*RS14e*RS34e );

      F13p6x22RV +=  + Ab(m46,mu2)*BbV(m11,me2,L) * ( 2./27.*m66*RS16e*
         RSe63 - 2./27.*m66*RS16e*RSe61 + 1./27.*m66*RS14e*RSe63 - 1./
         27.*m66*RS14e*RSe61 + 1./27.*m44*RS16e*RSe63 - 1./27.*m44*
         RS16e*RSe61 - 1./27.*m44*RS14e*RSe63 + 1./27.*m44*RS14e*RSe61
          + 1./9.*m11*RS16e*RSe63 - 1./9.*m11*RS16e*RSe61 - 1./9.*m11*
         RS14e*RSe63 + 1./9.*m11*RS14e*RSe61 );

      F13p6x22RV +=  + Ab(m46,mu2)*BbV(m11,L) * ( 1./27.*m66*RS14e*
         RS16e + 1./54.*m66*pow(RS14e,2) + 1./54.*m44*pow(RS16e,2) + 1./
         27.*m44*RS14e*RS16e - 1./54.*m11*pow(RS16e,2) + 1./27.*m11*
         RS14e*RS16e - 1./54.*m11*pow(RS14e,2) );

      F13p6x22RV +=  + Ab(m46,mu2)*BbV(m33,me2,L) * (  - 2./27.*m66*
         RS36e*RSe63 - 1./27.*m66*RS34e*RSe63 + 2./27.*m66*RSe61*RS36e
          + 1./27.*m66*RSe61*RS34e - 1./27.*m44*RS36e*RSe63 + 1./27.*
         m44*RS34e*RSe63 + 1./27.*m44*RSe61*RS36e - 1./27.*m44*RSe61*
         RS34e - 1./9.*m33*RS36e*RSe63 + 1./9.*m33*RS34e*RSe63 + 1./9.*
         m33*RSe61*RS36e - 1./9.*m33*RSe61*RS34e );

      F13p6x22RV +=  + Ab(m46,mu2)*BbV(m33,L) * ( 1./27.*m66*RS34e*
         RS36e + 1./54.*m66*pow(RS34e,2) + 1./54.*m44*pow(RS36e,2) + 1./
         27.*m44*RS34e*RS36e - 1./54.*m33*pow(RS36e,2) + 1./27.*m33*
         RS34e*RS36e - 1./54.*m33*pow(RS34e,2) );

      F13p6x22RV +=  + Ab(m46,mu2)*BbV(me2,L) * (  - 1./6.*me2*pow(
         RSe63,2) + 1./3.*me2*RSe61*RSe63 - 1./6.*me2*pow(RSe61,2) - 1./
         18.*m44*pow(RSe63,2) + 1./9.*m44*RSe61*RSe63 - 1./18.*m44*pow(
         RSe61,2) );

      F13p6x22RV +=  + Ab(me2,mu2)*AbV(m11,L) * (  - 1./18.*RSe6613*
         RS1463e + 1./36.*RSe6633*RS1463e + 1./36.*RSe6611*RS1463e + 1./
         36.*RC146e*RSe6613 - 1./72.*RC146e*RSe6633 - 1./72.*RC146e*
         RSe6611 - 1./9.*m11*RN1e*RC146e*RSe6613 + 1./9.*m11*RN1e*
         RC146e*RSe6611 );

      F13p6x22RV +=  + Ab(me2,mu2)*AbV(m13,L) * (  - 1./81.*RSe6613*
         RS3461e - 1./81.*RSe6613*RS1463e + 2./81.*pow(RSe6613,2) + 1./
         162.*RSe6633*RS3461e + 1./162.*RSe6633*RS1463e - 2./81.*
         RSe6633*RSe6613 + 1./162.*pow(RSe6633,2) + 1./162.*RC346e*
         RSe6613 - 1./324.*RC346e*RSe6633 + 1./162.*RSe6611*RS3461e + 1.
         /162.*RSe6611*RS1463e - 2./81.*RSe6611*RSe6613 + 1./81.*
         RSe6611*RSe6633 - 1./324.*RSe6611*RC346e + 1./162.*pow(
         RSe6611,2) + 1./162.*RC146e*RSe6613 - 1./324.*RC146e*RSe6633
          - 1./324.*RC146e*RSe6611 );

      F13p6x22RV +=  + Ab(me2,mu2)*AbV(m14,L) * ( 1./9.*RSe6613 + 5./36.
         *RSe63 + 1./12.*RSe6633 - 5./36.*RSe61 - 7./36.*RSe6611 - 1./9.
         *m44*RN1e*RSe63 + 1./9.*m44*RN1e*RSe61 - 2./9.*m11*RN1e*RSe63
          + 2./9.*m11*RN1e*RSe61 );

      F13p6x22RV +=  + Ab(me2,mu2)*AbV(m16,L) * ( 1./18.*RSe6613 - 5./
         36.*RSe63 + 1./24.*RSe6633 + 5./36.*RSe61 - 7./72.*RSe6611 + 1.
         /9.*m66*RN1e*RSe63 - 1./9.*m66*RN1e*RSe61 + 2./9.*m11*RN1e*
         RSe63 - 2./9.*m11*RN1e*RSe61 );

      F13p6x22RV +=  + Ab(me2,mu2)*AbV(m33,L) * (  - 1./18.*RSe6613*
         RS3461e + 1./36.*RSe6633*RS3461e + 1./36.*RC346e*RSe6613 - 1./
         72.*RC346e*RSe6633 + 1./36.*RSe6611*RS3461e - 1./72.*RSe6611*
         RC346e - 1./9.*m33*RN3e*RC346e*RSe6613 + 1./9.*m33*RN3e*RC346e
         *RSe6633 );

      F13p6x22RV +=  + Ab(me2,mu2)*AbV(m34,L) * ( 1./9.*RSe6613 - 5./36.
         *RSe63 - 7./36.*RSe6633 + 5./36.*RSe61 + 1./12.*RSe6611 + 1./9.
         *m44*RN3e*RSe63 - 1./9.*m44*RN3e*RSe61 + 2./9.*m33*RN3e*RSe63
          - 2./9.*m33*RN3e*RSe61 );

      F13p6x22RV +=  + Ab(me2,mu2)*AbV(m36,L) * ( 1./18.*RSe6613 + 5./
         36.*RSe63 - 7./72.*RSe6633 - 5./36.*RSe61 + 1./24.*RSe6611 - 1.
         /9.*m66*RN3e*RSe63 + 1./9.*m66*RN3e*RSe61 - 2./9.*m33*RN3e*
         RSe63 + 2./9.*m33*RN3e*RSe61 );

      F13p6x22RV +=  + Ab(me2,mu2)*AbV(m44,L) * (  - 1./6.*m44*RN3e*
         RS36e*RSe63 + 1./6.*m44*RN3e*RSe61*RS36e + 1./6.*m44*RN1e*
         RS16e*RSe63 - 1./6.*m44*RN1e*RS16e*RSe61 );

      F13p6x22RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./6.*pow(RSe63,2)
          - 1./9.*RS36e*RSe63 + 1./9.*RS34e*RSe63 + 1./3.*RSe61*RSe63
          + 1./9.*RSe61*RS36e - 1./9.*RSe61*RS34e - 1./6.*pow(RSe61,2)
          + 1./9.*RS16e*RSe63 - 1./9.*RS16e*RSe61 - 1./9.*RS14e*RSe63
          + 1./9.*RS14e*RSe61 + 2./27.*m66*RN3e*RS36e*RSe63 + 1./27.*
         m66*RN3e*RS34e*RSe63 - 2./27.*m66*RN3e*RSe61*RS36e - 1./27.*
         m66*RN3e*RSe61*RS34e - 2./27.*m66*RN1e*RS16e*RSe63 + 2./27.*
         m66*RN1e*RS16e*RSe61 - 1./27.*m66*RN1e*RS14e*RSe63 + 1./27.*
         m66*RN1e*RS14e*RSe61 + 1./27.*m44*RN3e*RS36e*RSe63 - 1./27.*
         m44*RN3e*RS34e*RSe63 - 1./27.*m44*RN3e*RSe61*RS36e + 1./27.*
         m44*RN3e*RSe61*RS34e - 1./27.*m44*RN1e*RS16e*RSe63 + 1./27.*
         m44*RN1e*RS16e*RSe61 + 1./27.*m44*RN1e*RS14e*RSe63 - 1./27.*
         m44*RN1e*RS14e*RSe61 + 1./9.*m33*RN3e*RS36e*RSe63 - 1./9.*m33*
         RN3e*RS34e*RSe63 - 1./9.*m33*RN3e*RSe61*RS36e + 1./9.*m33*RN3e
         *RSe61*RS34e - 1./9.*m11*RN1e*RS16e*RSe63 + 1./9.*m11*RN1e*
         RS16e*RSe61 );

      F13p6x22RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./9.*m11*RN1e*RS14e*
         RSe63 - 1./9.*m11*RN1e*RS14e*RSe61 );

      F13p6x22RV +=  + Ab(me2,mu2)*AbV(me2,L) * ( 1./9.*pow(RSe6613,2)
          - 1./9.*RSe6633*RSe6613 + 1./36.*pow(RSe6633,2) - 1./9.*
         RSe6611*RSe6613 + 1./18.*RSe6611*RSe6633 + 1./36.*pow(
         RSe6611,2) + 2./27.*m66*RN3e*RS34e*RSe63 - 2./27.*m66*RN3e*
         RSe61*RS34e - 2./27.*m66*RN1e*RS14e*RSe63 + 2./27.*m66*RN1e*
         RS14e*RSe61 - 1./54.*m44*RN3e*RS36e*RSe63 + 1./54.*m44*RN3e*
         RSe61*RS36e + 1./54.*m44*RN1e*RS16e*RSe63 - 1./54.*m44*RN1e*
         RS16e*RSe61 + 2./9.*m33*RN3e*RSe6633*RSe6613 - 2./9.*m33*RN3e*
         pow(RSe6633,2) + 2./9.*m11*RN1e*RSe6611*RSe6613 - 2./9.*m11*
         RN1e*pow(RSe6611,2) );

      F13p6x22RV +=  + Ab(me2,mu2)*BbV(m11,m33,L) * (  - 1./27.*m66*
         RS14e*RS34e - 1./54.*m44*RS16e*RS36e + 1./9.*m33*RSe6633*
         RS1463e + 1./9.*m11*RSe6611*RS3461e );

      F13p6x22RV +=  + Ab(me2,mu2)*BbV(m11,me2,L) * ( 2./27.*m66*RS14e*
         RSe63 - 2./27.*m66*RS14e*RSe61 - 1./54.*m44*RS16e*RSe63 + 1./
         54.*m44*RS16e*RSe61 - 2./9.*m11*RSe6611*RSe6613 + 2./9.*m11*
         pow(RSe6611,2) );

      F13p6x22RV +=  + Ab(me2,mu2)*BbV(m11,L) * ( 1./36.*RD146e*RSe6613
          - 1./72.*RD146e*RSe6633 - 1./72.*RD146e*RSe6611 + 1./54.*m66*
         pow(RS14e,2) + 1./108.*m44*pow(RS16e,2) + 1./9.*m11*RSe6611*
         RS1463e - 1./9.*m11*RC146e*RSe6611 - 1./9.*m11*RN1e*RD146e*
         RSe6613 + 1./9.*m11*RN1e*RD146e*RSe6611 );

      F13p6x22RV +=  + Ab(me2,mu2)*BbV(m14,L) * (  - 1./72.*me2 - 1./18.
         *me2*RSe61 - 1./18.*me2*RSe6611 + 1./72.*m44 - 1./18.*m44*
         RSe61 - 1./18.*m11*RSe61 + 1./18.*m11*RSe6611 );

      F13p6x22RV +=  + Ab(me2,mu2)*BbV(m16,L) * (  - 1./36.*me2 + 1./18.
         *me2*RSe61 - 1./36.*me2*RSe6611 + 1./36.*m66 + 1./18.*m66*
         RSe61 + 1./18.*m11*RSe61 + 1./36.*m11*RSe6611 );

      F13p6x22RV +=  + Ab(me2,mu2)*BbV(m33,me2,L) * (  - 2./27.*m66*
         RS34e*RSe63 + 2./27.*m66*RSe61*RS34e + 1./54.*m44*RS36e*RSe63
          - 1./54.*m44*RSe61*RS36e - 2./9.*m33*RSe6633*RSe6613 + 2./9.*
         m33*pow(RSe6633,2) );

      F13p6x22RV +=  + Ab(me2,mu2)*BbV(m33,L) * ( 1./36.*RD346e*RSe6613
          - 1./72.*RD346e*RSe6633 - 1./72.*RSe6611*RD346e + 1./54.*m66*
         pow(RS34e,2) + 1./108.*m44*pow(RS36e,2) + 1./9.*m33*RSe6633*
         RS3461e - 1./9.*m33*RC346e*RSe6633 - 1./9.*m33*RN3e*RD346e*
         RSe6613 + 1./9.*m33*RN3e*RD346e*RSe6633 );

      F13p6x22RV +=  + Ab(me2,mu2)*BbV(m34,L) * (  - 1./72.*me2 - 1./18.
         *me2*RSe63 - 1./18.*me2*RSe6633 + 1./72.*m44 - 1./18.*m44*
         RSe63 - 1./18.*m33*RSe63 + 1./18.*m33*RSe6633 );

      F13p6x22RV +=  + Ab(me2,mu2)*BbV(m36,L) * (  - 1./36.*me2 + 1./18.
         *me2*RSe63 - 1./36.*me2*RSe6633 + 1./36.*m66 + 1./18.*m66*
         RSe63 + 1./18.*m33*RSe63 + 1./36.*m33*RSe6633 );

      F13p6x22RV +=  + Ab(me2,mu2)*BbV(me2,L) * ( 2./27.*m66*pow(
         RSe63,2) - 4./27.*m66*RSe61*RSe63 + 2./27.*m66*pow(RSe61,2) + 
         1./108.*m44*pow(RSe63,2) - 1./54.*m44*RSe61*RSe63 + 1./108.*
         m44*pow(RSe61,2) );

      F13p6x22RV +=  + Ab(me2,mu2)*CbV(m11,L) * (  - 1./9.*m11*RD146e*
         RSe6611 );

      F13p6x22RV +=  + Ab(me2,mu2)*CbV(m33,L) * (  - 1./9.*m33*RD346e*
         RSe6633 );

      F13p6x22RV +=  + Bb(m11,mu2)*AbV(m11,L) * (  - 1./72.*RD146e*
         RS1463e + 1./144.*RD146e*RC146e + 1./4.*m11 - 1./18.*m11*
         RC146e*RS1463e + 1./18.*m11*pow(RC146e,2) );

      F13p6x22RV +=  + Bb(m11,mu2)*AbV(m13,L) * (  - 1./324.*RD146e*
         RS3461e - 1./324.*RD146e*RS1463e + 1./162.*RD146e*RSe6613 - 1./
         324.*RD146e*RSe6633 + 1./648.*RD146e*RC346e - 1./324.*RD146e*
         RSe6611 + 1./648.*RD146e*RC146e );

      F13p6x22RV +=  + Bb(m11,mu2)*AbV(m14,L) * ( 7./72.*RD146e - 1./9.
         *m44*RS16e - 2./9.*m11*RS16e );

      F13p6x22RV +=  + Bb(m11,mu2)*AbV(m16,L) * ( 7./144.*RD146e - 1./
         18.*m66*RS14e - 1./9.*m11*RS14e );

      F13p6x22RV +=  + Bb(m11,mu2)*AbV(m33,L) * (  - 1./72.*RD146e*
         RS3461e + 1./144.*RD146e*RC346e );

      F13p6x22RV +=  + Bb(m11,mu2)*AbV(m34,L) * (  - 1./24.*RD146e );

      F13p6x22RV +=  + Bb(m11,mu2)*AbV(m36,L) * (  - 1./48.*RD146e );

      F13p6x22RV +=  + Bb(m11,mu2)*AbV(m44,L) * ( 1./12.*m44*pow(
         RS16e,2) );

      F13p6x22RV +=  + Bb(m11,mu2)*AbV(m46,L) * ( 1./27.*m66*RS14e*
         RS16e + 1./54.*m66*pow(RS14e,2) + 1./54.*m44*pow(RS16e,2) + 1./
         27.*m44*RS14e*RS16e - 1./54.*m11*pow(RS16e,2) + 1./27.*m11*
         RS14e*RS16e - 1./54.*m11*pow(RS14e,2) );

      F13p6x22RV +=  + Bb(m11,mu2)*AbV(me2,L) * ( 1./36.*RD146e*RSe6613
          - 1./72.*RD146e*RSe6633 - 1./72.*RD146e*RSe6611 + 1./54.*m66*
         pow(RS14e,2) + 1./108.*m44*pow(RS16e,2) + 1./9.*m11*RSe6611*
         RS1463e - 1./9.*m11*RC146e*RSe6611 );

      F13p6x22RV +=  + Bb(m11,mu2)*BbV(m11,m33,L) * (  - 1./18.*m11*
         RD146e*RS3461e );

      F13p6x22RV +=  + Bb(m11,mu2)*BbV(m11,me2,L) * ( 1./9.*m11*RD146e*
         RSe6613 - 1./9.*m11*RD146e*RSe6611 );

      F13p6x22RV +=  + Bb(m11,mu2)*BbV(m11,L) * ( 1./144.*pow(RD146e,2)
          - 1./9.*m11*RD146e*RS1463e + 1./9.*m11*RD146e*RC146e );

      F13p6x22RV +=  + Bb(m11,mu2)*BbV(m33,L) * ( 1./144.*RD146e*RD346e
          );

      F13p6x22RV +=  + Bb(m11,mu2)*CbV(m11,L) * ( 1./18.*m11*pow(
         RD146e,2) );

      F13p6x22RV +=  + Bb(m14,mu2)*AbV(m11,L) * ( 1./36.*RD146e - 1./18.
         *m44*RS16e - 1./9.*m11*RS16e );

      F13p6x22RV +=  + Bb(m14,mu2)*AbV(me2,L) * (  - 1./72.*me2 - 1./18.
         *me2*RSe61 - 1./18.*me2*RSe6611 + 1./72.*m44 - 1./18.*m44*
         RSe61 - 1./18.*m11*RSe61 + 1./18.*m11*RSe6611 );

      F13p6x22RV +=  + Bb(m16,mu2)*AbV(m11,L) * ( 1./72.*RD146e - 1./36.
         *m66*RS14e - 1./18.*m11*RS14e );

      F13p6x22RV +=  + Bb(m16,mu2)*AbV(me2,L) * (  - 1./36.*me2 + 1./18.
         *me2*RSe61 - 1./36.*me2*RSe6611 + 1./36.*m66 + 1./18.*m66*
         RSe61 + 1./18.*m11*RSe61 + 1./36.*m11*RSe6611 );

      F13p6x22RV +=  + Bb(m33,mu2)*AbV(m11,L) * (  - 1./72.*RD346e*
         RS1463e + 1./144.*RC146e*RD346e );

      F13p6x22RV +=  + Bb(m33,mu2)*AbV(m13,L) * (  - 1./324.*RD346e*
         RS3461e - 1./324.*RD346e*RS1463e + 1./162.*RD346e*RSe6613 - 1./
         324.*RD346e*RSe6633 + 1./648.*RD346e*RC346e - 1./324.*RSe6611*
         RD346e + 1./648.*RC146e*RD346e );

      F13p6x22RV +=  + Bb(m33,mu2)*AbV(m14,L) * (  - 1./24.*RD346e );

      F13p6x22RV +=  + Bb(m33,mu2)*AbV(m16,L) * (  - 1./48.*RD346e );

      F13p6x22RV +=  + Bb(m33,mu2)*AbV(m33,L) * (  - 1./72.*RD346e*
         RS3461e + 1./144.*RD346e*RC346e + 1./4.*m33 - 1./18.*m33*
         RC346e*RS3461e + 1./18.*m33*pow(RC346e,2) );

      F13p6x22RV +=  + Bb(m33,mu2)*AbV(m34,L) * ( 7./72.*RD346e - 1./9.
         *m44*RS36e - 2./9.*m33*RS36e );

      F13p6x22RV +=  + Bb(m33,mu2)*AbV(m36,L) * ( 7./144.*RD346e - 1./
         18.*m66*RS34e - 1./9.*m33*RS34e );

      F13p6x22RV +=  + Bb(m33,mu2)*AbV(m44,L) * ( 1./12.*m44*pow(
         RS36e,2) );

      F13p6x22RV +=  + Bb(m33,mu2)*AbV(m46,L) * ( 1./27.*m66*RS34e*
         RS36e + 1./54.*m66*pow(RS34e,2) + 1./54.*m44*pow(RS36e,2) + 1./
         27.*m44*RS34e*RS36e - 1./54.*m33*pow(RS36e,2) + 1./27.*m33*
         RS34e*RS36e - 1./54.*m33*pow(RS34e,2) );

      F13p6x22RV +=  + Bb(m33,mu2)*AbV(me2,L) * ( 1./36.*RD346e*RSe6613
          - 1./72.*RD346e*RSe6633 - 1./72.*RSe6611*RD346e + 1./54.*m66*
         pow(RS34e,2) + 1./108.*m44*pow(RS36e,2) + 1./9.*m33*RSe6633*
         RS3461e - 1./9.*m33*RC346e*RSe6633 );

      F13p6x22RV +=  + Bb(m33,mu2)*BbV(m11,m33,L) * (  - 1./18.*m33*
         RD346e*RS1463e );

      F13p6x22RV +=  + Bb(m33,mu2)*BbV(m11,L) * ( 1./144.*RD146e*RD346e
          );

      F13p6x22RV +=  + Bb(m33,mu2)*BbV(m33,me2,L) * ( 1./9.*m33*RD346e*
         RSe6613 - 1./9.*m33*RD346e*RSe6633 );

      F13p6x22RV +=  + Bb(m33,mu2)*BbV(m33,L) * ( 1./144.*pow(RD346e,2)
          - 1./9.*m33*RD346e*RS3461e + 1./9.*m33*RD346e*RC346e );

      F13p6x22RV +=  + Bb(m33,mu2)*CbV(m33,L) * ( 1./18.*m33*pow(
         RD346e,2) );

      F13p6x22RV +=  + Bb(m34,mu2)*AbV(m33,L) * ( 1./36.*RD346e - 1./18.
         *m44*RS36e - 1./9.*m33*RS36e );

      F13p6x22RV +=  + Bb(m34,mu2)*AbV(me2,L) * (  - 1./72.*me2 - 1./18.
         *me2*RSe63 - 1./18.*me2*RSe6633 + 1./72.*m44 - 1./18.*m44*
         RSe63 - 1./18.*m33*RSe63 + 1./18.*m33*RSe6633 );

      F13p6x22RV +=  + Bb(m36,mu2)*AbV(m33,L) * ( 1./72.*RD346e - 1./36.
         *m66*RS34e - 1./18.*m33*RS34e );

      F13p6x22RV +=  + Bb(m36,mu2)*AbV(me2,L) * (  - 1./36.*me2 + 1./18.
         *me2*RSe63 - 1./36.*me2*RSe6633 + 1./36.*m66 + 1./18.*m66*
         RSe63 + 1./18.*m33*RSe63 + 1./36.*m33*RSe6633 );

      F13p6x22RV +=  + Bb(me2,mu2)*AbV(m44,L) * ( 1./12.*m44*pow(
         RSe63,2) - 1./6.*m44*RSe61*RSe63 + 1./12.*m44*pow(RSe61,2) );

      F13p6x22RV +=  + Bb(me2,mu2)*AbV(m46,L) * (  - 1./6.*me2*pow(
         RSe63,2) + 1./3.*me2*RSe61*RSe63 - 1./6.*me2*pow(RSe61,2) - 1./
         18.*m44*pow(RSe63,2) + 1./9.*m44*RSe61*RSe63 - 1./18.*m44*pow(
         RSe61,2) );

      F13p6x22RV +=  + Bb(me2,mu2)*AbV(me2,L) * ( 2./27.*m66*pow(
         RSe63,2) - 4./27.*m66*RSe61*RSe63 + 2./27.*m66*pow(RSe61,2) + 
         1./108.*m44*pow(RSe63,2) - 1./54.*m44*RSe61*RSe63 + 1./108.*
         m44*pow(RSe61,2) );

      F13p6x22RV +=  + Cb(m11,mu2)*AbV(m11,L) * ( 1./18.*m11*RD146e*
         RC146e );

      F13p6x22RV +=  + Cb(m11,mu2)*AbV(me2,L) * (  - 1./9.*m11*RD146e*
         RSe6611 );

      F13p6x22RV +=  + Cb(m11,mu2)*BbV(m11,L) * ( 1./18.*m11*pow(
         RD146e,2) );

      F13p6x22RV +=  + Cb(m33,mu2)*AbV(m33,L) * ( 1./18.*m33*RD346e*
         RC346e );

      F13p6x22RV +=  + Cb(m33,mu2)*AbV(me2,L) * (  - 1./9.*m33*RD346e*
         RSe6633 );

      F13p6x22RV +=  + Cb(m33,mu2)*BbV(m33,L) * ( 1./18.*m33*pow(
         RD346e,2) );

      F13p6x22RV +=  + AbV(m11,L)*pi16 * ( 1./72.*RD346e*RS1463e - 1./
         144.*RC146e*RD346e - 1./48.*RD146e + 1./72.*RD146e*RS1463e - 1.
         /144.*RD146e*RC146e + 1./18.*me2*RSe6613*RS1463e - 1./36.*me2*
         RSe6633*RS1463e - 1./36.*me2*RSe6611*RS1463e - 1./36.*me2*
         RC146e*RSe6613 + 1./72.*me2*RC146e*RSe6633 + 1./72.*me2*RC146e
         *RSe6611 - 1./24.*m66*RS1463e + 1./48.*m66*RC146e - 1./12.*m44
         *RS1463e + 1./24.*m44*RC146e - 1./16.*m33*RS1463e - 1./24.*m33
         *RS1463e*RS3461e - 1./72.*m33*pow(RS1463e,2) + 1./36.*m33*
         RSe6613*RS1463e - 1./72.*m33*RSe6633*RS1463e + 1./48.*m33*
         RC346e*RS1463e - 1./72.*m33*RSe6611*RS1463e + 1./32.*m33*
         RC146e + 1./48.*m33*RC146e*RS3461e + 1./72.*m33*RC146e*RS1463e
          - 1./72.*m33*RC146e*RSe6613 + 1./144.*m33*RC146e*RSe6633 - 1./
         96.*m33*RC146e*RC346e + 1./144.*m33*RC146e*RSe6611 - 1./288.*
         m33*pow(RC146e,2) + 1./16.*m11 - 1./48.*m11*RS1463e - 1./72.*
         m11*RS1463e*RS3461e - 1./24.*m11*pow(RS1463e,2) + 1./36.*m11*
         RSe6613*RS1463e );

      F13p6x22RV +=  + AbV(m11,L)*pi16 * (  - 1./72.*m11*RSe6633*
         RS1463e + 1./144.*m11*RC346e*RS1463e - 1./72.*m11*RSe6611*
         RS1463e - 1./24.*m11*RS16e - 1./48.*m11*RS14e + 1./96.*m11*
         RC146e + 1./144.*m11*RC146e*RS3461e + 1./24.*m11*RC146e*
         RS1463e - 1./72.*m11*RC146e*RSe6613 + 1./144.*m11*RC146e*
         RSe6633 - 1./288.*m11*RC146e*RC346e + 1./144.*m11*RC146e*
         RSe6611 - 1./96.*m11*pow(RC146e,2) );

      F13p6x22RV +=  + pow(AbV(m11,L),2) * ( 1./16. + 1./72.*pow(
         RS1463e,2) - 1./72.*RC146e*RS1463e + 1./288.*pow(RC146e,2) );

      F13p6x22RV +=  + AbV(m11,L)*AbV(m13,L) * (  - 1./24. + 1./162.*
         RS1463e*RS3461e + 1./162.*pow(RS1463e,2) - 1./81.*RSe6613*
         RS1463e + 1./162.*RSe6633*RS1463e - 1./324.*RC346e*RS1463e + 1.
         /162.*RSe6611*RS1463e - 1./324.*RC146e*RS3461e - 1./162.*
         RC146e*RS1463e + 1./162.*RC146e*RSe6613 - 1./324.*RC146e*
         RSe6633 + 1./648.*RC146e*RC346e - 1./324.*RC146e*RSe6611 + 1./
         648.*pow(RC146e,2) );

      F13p6x22RV +=  + AbV(m11,L)*AbV(m14,L) * (  - 1./18.*RS1463e - 5./
         36.*RS16e + 7./72.*RC146e );

      F13p6x22RV +=  + AbV(m11,L)*AbV(m16,L) * (  - 1./36.*RS1463e - 5./
         72.*RS14e + 7./144.*RC146e );

      F13p6x22RV +=  + AbV(m11,L)*AbV(m33,L) * ( 1./36.*RS1463e*RS3461e
          - 1./72.*RC346e*RS1463e - 1./72.*RC146e*RS3461e + 1./144.*
         RC146e*RC346e );

      F13p6x22RV +=  + AbV(m11,L)*AbV(m34,L) * ( 1./18.*RS1463e + 1./36.
         *RS16e - 1./24.*RC146e );

      F13p6x22RV +=  + AbV(m11,L)*AbV(m36,L) * ( 1./36.*RS1463e + 1./72.
         *RS14e - 1./48.*RC146e );

      F13p6x22RV +=  + AbV(m11,L)*AbV(m46,L) * (  - 1./54.*pow(RS16e,2)
          + 1./27.*RS14e*RS16e - 1./54.*pow(RS14e,2) );

      F13p6x22RV +=  + AbV(m11,L)*AbV(me2,L) * (  - 1./18.*RSe6613*
         RS1463e + 1./36.*RSe6633*RS1463e + 1./36.*RSe6611*RS1463e + 1./
         36.*RC146e*RSe6613 - 1./72.*RC146e*RSe6633 - 1./72.*RC146e*
         RSe6611 );

      F13p6x22RV +=  + AbV(m11,L)*BbV(m11,m33,L) * (  - 1./18.*m11*
         RC146e*RS3461e );

      F13p6x22RV +=  + AbV(m11,L)*BbV(m11,me2,L) * ( 1./9.*m11*RC146e*
         RSe6613 - 1./9.*m11*RC146e*RSe6611 );

      F13p6x22RV +=  + AbV(m11,L)*BbV(m11,L) * (  - 1./72.*RD146e*
         RS1463e + 1./144.*RD146e*RC146e + 1./4.*m11 - 1./18.*m11*
         RC146e*RS1463e + 1./18.*m11*pow(RC146e,2) );

      F13p6x22RV +=  + AbV(m11,L)*BbV(m14,L) * ( 1./36.*RD146e - 1./18.
         *m44*RS16e - 1./9.*m11*RS16e );

      F13p6x22RV +=  + AbV(m11,L)*BbV(m16,L) * ( 1./72.*RD146e - 1./36.
         *m66*RS14e - 1./18.*m11*RS14e );

      F13p6x22RV +=  + AbV(m11,L)*BbV(m33,L) * (  - 1./72.*RD346e*
         RS1463e + 1./144.*RC146e*RD346e );

      F13p6x22RV +=  + AbV(m11,L)*CbV(m11,L) * ( 1./18.*m11*RD146e*
         RC146e );

      F13p6x22RV +=  + AbV(m13,L)*pi16 * (  - 1./144.*m33*pow(
         RS3461e,2) - 1./72.*m33*RS1463e*RS3461e - 1./144.*m33*pow(
         RS1463e,2) + 1./36.*m33*RSe6613*RS3461e + 1./36.*m33*RSe6613*
         RS1463e - 1./36.*m33*pow(RSe6613,2) - 1./72.*m33*RSe6633*
         RS3461e - 1./72.*m33*RSe6633*RS1463e + 1./36.*m33*RSe6633*
         RSe6613 - 1./144.*m33*pow(RSe6633,2) + 1./144.*m33*RC346e*
         RS3461e + 1./144.*m33*RC346e*RS1463e - 1./72.*m33*RC346e*
         RSe6613 + 1./144.*m33*RC346e*RSe6633 - 1./576.*m33*pow(
         RC346e,2) - 1./72.*m33*RSe6611*RS3461e - 1./72.*m33*RSe6611*
         RS1463e + 1./36.*m33*RSe6611*RSe6613 - 1./72.*m33*RSe6611*
         RSe6633 + 1./144.*m33*RSe6611*RC346e - 1./144.*m33*pow(
         RSe6611,2) + 1./144.*m33*RC146e*RS3461e + 1./144.*m33*RC146e*
         RS1463e - 1./72.*m33*RC146e*RSe6613 + 1./144.*m33*RC146e*
         RSe6633 - 1./288.*m33*RC146e*RC346e + 1./144.*m33*RC146e*
         RSe6611 - 1./576.*m33*pow(RC146e,2) - 1./144.*m11*pow(
         RS3461e,2) );

      F13p6x22RV +=  + AbV(m13,L)*pi16 * (  - 1./72.*m11*RS1463e*
         RS3461e - 1./144.*m11*pow(RS1463e,2) + 1./36.*m11*RSe6613*
         RS3461e + 1./36.*m11*RSe6613*RS1463e - 1./36.*m11*pow(
         RSe6613,2) - 1./72.*m11*RSe6633*RS3461e - 1./72.*m11*RSe6633*
         RS1463e + 1./36.*m11*RSe6633*RSe6613 - 1./144.*m11*pow(
         RSe6633,2) + 1./144.*m11*RC346e*RS3461e + 1./144.*m11*RC346e*
         RS1463e - 1./72.*m11*RC346e*RSe6613 + 1./144.*m11*RC346e*
         RSe6633 - 1./576.*m11*pow(RC346e,2) - 1./72.*m11*RSe6611*
         RS3461e - 1./72.*m11*RSe6611*RS1463e + 1./36.*m11*RSe6611*
         RSe6613 - 1./72.*m11*RSe6611*RSe6633 + 1./144.*m11*RSe6611*
         RC346e - 1./144.*m11*pow(RSe6611,2) + 1./144.*m11*RC146e*
         RS3461e + 1./144.*m11*RC146e*RS1463e - 1./72.*m11*RC146e*
         RSe6613 + 1./144.*m11*RC146e*RSe6633 - 1./288.*m11*RC146e*
         RC346e + 1./144.*m11*RC146e*RSe6611 - 1./576.*m11*pow(
         RC146e,2) );

      F13p6x22RV +=  + pow(AbV(m13,L),2) * ( 1./8. );

      F13p6x22RV +=  + AbV(m13,L)*AbV(m33,L) * (  - 5./24. + 1./162.*
         pow(RS3461e,2) + 1./162.*RS1463e*RS3461e - 1./81.*RSe6613*
         RS3461e + 1./162.*RSe6633*RS3461e - 1./162.*RC346e*RS3461e - 1.
         /324.*RC346e*RS1463e + 1./162.*RC346e*RSe6613 - 1./324.*RC346e
         *RSe6633 + 1./648.*pow(RC346e,2) + 1./162.*RSe6611*RS3461e - 1.
         /324.*RSe6611*RC346e - 1./324.*RC146e*RS3461e + 1./648.*RC146e
         *RC346e );

      F13p6x22RV +=  + AbV(m13,L)*AbV(me2,L) * (  - 1./81.*RSe6613*
         RS3461e - 1./81.*RSe6613*RS1463e + 2./81.*pow(RSe6613,2) + 1./
         162.*RSe6633*RS3461e + 1./162.*RSe6633*RS1463e - 2./81.*
         RSe6633*RSe6613 + 1./162.*pow(RSe6633,2) + 1./162.*RC346e*
         RSe6613 - 1./324.*RC346e*RSe6633 + 1./162.*RSe6611*RS3461e + 1.
         /162.*RSe6611*RS1463e - 2./81.*RSe6611*RSe6613 + 1./81.*
         RSe6611*RSe6633 - 1./324.*RSe6611*RC346e + 1./162.*pow(
         RSe6611,2) + 1./162.*RC146e*RSe6613 - 1./324.*RC146e*RSe6633
          - 1./324.*RC146e*RSe6611 );

      F13p6x22RV +=  + AbV(m13,L)*BbV(m11,m33,L) * (  - 1./6.*m33 - 1./
         3.*m11 );

      F13p6x22RV +=  + AbV(m13,L)*BbV(m11,L) * (  - 1./324.*RD146e*
         RS3461e - 1./324.*RD146e*RS1463e + 1./162.*RD146e*RSe6613 - 1./
         324.*RD146e*RSe6633 + 1./648.*RD146e*RC346e - 1./324.*RD146e*
         RSe6611 + 1./648.*RD146e*RC146e );

      F13p6x22RV +=  + AbV(m13,L)*BbV(m33,L) * (  - 1./324.*RD346e*
         RS3461e - 1./324.*RD346e*RS1463e + 1./162.*RD346e*RSe6613 - 1./
         324.*RD346e*RSe6633 + 1./648.*RD346e*RC346e - 1./324.*RSe6611*
         RD346e + 1./648.*RC146e*RD346e );

      F13p6x22RV +=  + AbV(m14,L)*pi16 * ( 1./24.*RD346e - 1./48.*me2
          - 1./12.*me2*RSe63 - 1./12.*me2*RSe6633 - 1./8.*m66 - 7./24.*
         m44 - 1./48.*m44*RS3461e - 1./48.*m44*RS1463e + 1./24.*m44*
         RSe6613 - 1./48.*m44*RSe63 - 1./24.*m44*RSe6633 - 1./48.*m44*
         RS36e + 1./48.*m44*RC346e - 1./48.*m44*RSe61 - 1./24.*m44*
         RSe6611 - 1./48.*m44*RS16e + 1./48.*m44*RC146e - 1./6.*m33 - 1.
         /24.*m33*RSe63 - 1./24.*m33*RSe6633 - 1./8.*m33*RS36e + 1./16.
         *m33*RC346e + 1./16.*m11 - 1./48.*m11*RS3461e - 1./48.*m11*
         RS1463e + 1./24.*m11*RSe6613 + 1./48.*m11*RSe63 + 1./48.*m11*
         RS36e - 1./48.*m11*RSe61 - 1./24.*m11*RSe6611 - 1./48.*m11*
         RS16e + 1./48.*m11*RC146e );

      F13p6x22RV +=  + pow(AbV(m14,L),2) * (  - 1./8. );

      F13p6x22RV +=  + AbV(m14,L)*AbV(m16,L) * (  - 1./8. );

      F13p6x22RV +=  + AbV(m14,L)*AbV(m33,L) * (  - 1./18.*RS3461e + 5./
         36.*RS36e - 1./24.*RC346e );

      F13p6x22RV +=  + AbV(m14,L)*AbV(m34,L) * ( 8./27. + 7./108.*
         RS3461e + 7./108.*RS1463e - 7./54.*RSe6613 - 5./108.*RSe63 + 1.
         /54.*RSe6633 - 5./108.*RS36e - 1./108.*RC346e - 5./108.*RSe61
          + 1./54.*RSe6611 - 5./108.*RS16e - 1./108.*RC146e );

      F13p6x22RV +=  + AbV(m14,L)*AbV(m36,L) * ( 1./8. );

      F13p6x22RV +=  + AbV(m14,L)*AbV(me2,L) * ( 1./9.*RSe6613 + 5./36.
         *RSe63 + 1./12.*RSe6633 - 5./36.*RSe61 - 7./36.*RSe6611 );

      F13p6x22RV +=  + AbV(m14,L)*BbV(m11,m33,L) * ( 1./9.*m44*RS36e + 
         2./9.*m11*RS36e );

      F13p6x22RV +=  + AbV(m14,L)*BbV(m11,me2,L) * ( 1./9.*m44*RSe63 - 
         1./9.*m44*RSe61 + 2./9.*m11*RSe63 - 2./9.*m11*RSe61 );

      F13p6x22RV +=  + AbV(m14,L)*BbV(m11,L) * ( 7./72.*RD146e - 1./9.*
         m44*RS16e - 2./9.*m11*RS16e );

      F13p6x22RV +=  + AbV(m14,L)*BbV(m33,L) * (  - 1./24.*RD346e );

      F13p6x22RV +=  + AbV(m16,L)*pi16 * ( 1./48.*RD346e - 1./24.*me2
          + 1./12.*me2*RSe63 - 1./24.*me2*RSe6633 - 5./96.*m66 - 1./96.
         *m66*RS3461e - 1./96.*m66*RS1463e + 1./48.*m66*RSe6613 + 1./48.
         *m66*RSe63 - 1./48.*m66*RSe6633 - 1./96.*m66*RS34e + 1./96.*
         m66*RC346e + 1./48.*m66*RSe61 - 1./48.*m66*RSe6611 - 1./96.*
         m66*RS14e + 1./96.*m66*RC146e - 1./8.*m44 - 1./12.*m33 + 1./24.
         *m33*RSe63 - 1./48.*m33*RSe6633 - 1./16.*m33*RS34e + 1./32.*
         m33*RC346e + 1./32.*m11 - 1./96.*m11*RS3461e - 1./96.*m11*
         RS1463e + 1./48.*m11*RSe6613 - 1./48.*m11*RSe63 + 1./96.*m11*
         RS34e + 1./48.*m11*RSe61 - 1./48.*m11*RSe6611 - 1./96.*m11*
         RS14e + 1./96.*m11*RC146e );

      F13p6x22RV +=  + pow(AbV(m16,L),2) * (  - 1./32. );

      F13p6x22RV +=  + AbV(m16,L)*AbV(m33,L) * (  - 1./36.*RS3461e + 5./
         72.*RS34e - 1./48.*RC346e );

      F13p6x22RV +=  + AbV(m16,L)*AbV(m34,L) * ( 1./8. );

      F13p6x22RV +=  + AbV(m16,L)*AbV(m36,L) * ( 37./432. + 7./216.*
         RS3461e + 7./216.*RS1463e - 7./108.*RSe6613 + 5./108.*RSe63 + 
         1./108.*RSe6633 - 5./216.*RS34e - 1./216.*RC346e + 5./108.*
         RSe61 + 1./108.*RSe6611 - 5./216.*RS14e - 1./216.*RC146e );

      F13p6x22RV +=  + AbV(m16,L)*AbV(me2,L) * ( 1./18.*RSe6613 - 5./36.
         *RSe63 + 1./24.*RSe6633 + 5./36.*RSe61 - 7./72.*RSe6611 );

      F13p6x22RV +=  + AbV(m16,L)*BbV(m11,m33,L) * ( 1./18.*m66*RS34e
          + 1./9.*m11*RS34e );

      F13p6x22RV +=  + AbV(m16,L)*BbV(m11,me2,L) * (  - 1./9.*m66*RSe63
          + 1./9.*m66*RSe61 - 2./9.*m11*RSe63 + 2./9.*m11*RSe61 );

      F13p6x22RV +=  + AbV(m16,L)*BbV(m11,L) * ( 7./144.*RD146e - 1./18.
         *m66*RS14e - 1./9.*m11*RS14e );

      F13p6x22RV +=  + AbV(m16,L)*BbV(m33,L) * (  - 1./48.*RD346e );

      F13p6x22RV +=  + AbV(m33,L)*pi16 * (  - 1./48.*RD346e + 1./72.*
         RD346e*RS3461e - 1./144.*RD346e*RC346e + 1./72.*RD146e*RS3461e
          - 1./144.*RD146e*RC346e + 1./18.*me2*RSe6613*RS3461e - 1./36.
         *me2*RSe6633*RS3461e - 1./36.*me2*RC346e*RSe6613 + 1./72.*me2*
         RC346e*RSe6633 - 1./36.*me2*RSe6611*RS3461e + 1./72.*me2*
         RSe6611*RC346e - 1./24.*m66*RS3461e + 1./48.*m66*RC346e - 1./
         12.*m44*RS3461e + 1./24.*m44*RC346e + 1./16.*m33 - 1./48.*m33*
         RS3461e - 1./24.*m33*pow(RS3461e,2) - 1./72.*m33*RS1463e*
         RS3461e + 1./36.*m33*RSe6613*RS3461e - 1./72.*m33*RSe6633*
         RS3461e - 1./24.*m33*RS36e - 1./48.*m33*RS34e + 1./96.*m33*
         RC346e + 1./24.*m33*RC346e*RS3461e + 1./144.*m33*RC346e*
         RS1463e - 1./72.*m33*RC346e*RSe6613 + 1./144.*m33*RC346e*
         RSe6633 - 1./96.*m33*pow(RC346e,2) - 1./72.*m33*RSe6611*
         RS3461e + 1./144.*m33*RSe6611*RC346e + 1./144.*m33*RC146e*
         RS3461e - 1./288.*m33*RC146e*RC346e - 1./16.*m11*RS3461e - 1./
         72.*m11*pow(RS3461e,2) );

      F13p6x22RV +=  + AbV(m33,L)*pi16 * (  - 1./24.*m11*RS1463e*
         RS3461e + 1./36.*m11*RSe6613*RS3461e - 1./72.*m11*RSe6633*
         RS3461e + 1./32.*m11*RC346e + 1./72.*m11*RC346e*RS3461e + 1./
         48.*m11*RC346e*RS1463e - 1./72.*m11*RC346e*RSe6613 + 1./144.*
         m11*RC346e*RSe6633 - 1./288.*m11*pow(RC346e,2) - 1./72.*m11*
         RSe6611*RS3461e + 1./144.*m11*RSe6611*RC346e + 1./48.*m11*
         RC146e*RS3461e - 1./96.*m11*RC146e*RC346e );

      F13p6x22RV +=  + pow(AbV(m33,L),2) * ( 1./16. + 1./72.*pow(
         RS3461e,2) - 1./72.*RC346e*RS3461e + 1./288.*pow(RC346e,2) );

      F13p6x22RV +=  + AbV(m33,L)*AbV(m34,L) * (  - 1./18.*RS3461e - 1./
         9.*RS1463e - 5./36.*RS36e + 7./72.*RC346e + 1./9.*RS16e );

      F13p6x22RV +=  + AbV(m33,L)*AbV(m36,L) * (  - 1./36.*RS3461e - 1./
         18.*RS1463e - 5./72.*RS34e + 7./144.*RC346e + 1./18.*RS14e );

      F13p6x22RV +=  + AbV(m33,L)*AbV(m46,L) * (  - 1./54.*pow(RS36e,2)
          + 1./27.*RS34e*RS36e - 1./54.*pow(RS34e,2) + 1./27.*RS16e*
         RS36e - 1./27.*RS16e*RS34e - 1./27.*RS14e*RS36e + 1./27.*RS14e
         *RS34e );

      F13p6x22RV +=  + AbV(m33,L)*AbV(me2,L) * (  - 1./18.*RSe6613*
         RS3461e + 1./36.*RSe6633*RS3461e + 1./36.*RC346e*RSe6613 - 1./
         72.*RC346e*RSe6633 + 1./36.*RSe6611*RS3461e - 1./72.*RSe6611*
         RC346e );

      F13p6x22RV +=  + AbV(m33,L)*BbV(m11,m33,L) * (  - 1./18.*m33*
         RC346e*RS1463e );

      F13p6x22RV +=  + AbV(m33,L)*BbV(m11,L) * (  - 1./72.*RD146e*
         RS3461e + 1./144.*RD146e*RC346e );

      F13p6x22RV +=  + AbV(m33,L)*BbV(m33,me2,L) * ( 1./9.*m33*RC346e*
         RSe6613 - 1./9.*m33*RC346e*RSe6633 );

      F13p6x22RV +=  + AbV(m33,L)*BbV(m33,L) * (  - 1./72.*RD346e*
         RS3461e + 1./144.*RD346e*RC346e + 1./4.*m33 - 1./18.*m33*
         RC346e*RS3461e + 1./18.*m33*pow(RC346e,2) );

      F13p6x22RV +=  + AbV(m33,L)*BbV(m34,L) * ( 1./36.*RD346e - 1./18.
         *m44*RS36e - 1./9.*m33*RS36e );

      F13p6x22RV +=  + AbV(m33,L)*BbV(m36,L) * ( 1./72.*RD346e - 1./36.
         *m66*RS34e - 1./18.*m33*RS34e );

      F13p6x22RV +=  + AbV(m33,L)*CbV(m33,L) * ( 1./18.*m33*RD346e*
         RC346e );

      F13p6x22RV +=  + AbV(m34,L)*pi16 * ( 1./24.*RD146e - 1./48.*me2
          - 1./12.*me2*RSe61 - 1./12.*me2*RSe6611 - 1./8.*m66 - 7./24.*
         m44 - 1./48.*m44*RS3461e - 1./48.*m44*RS1463e + 1./24.*m44*
         RSe6613 - 1./48.*m44*RSe63 - 1./24.*m44*RSe6633 - 1./48.*m44*
         RS36e + 1./48.*m44*RC346e - 1./48.*m44*RSe61 - 1./24.*m44*
         RSe6611 - 1./48.*m44*RS16e + 1./48.*m44*RC146e + 1./16.*m33 - 
         1./48.*m33*RS3461e - 1./48.*m33*RS1463e + 1./24.*m33*RSe6613
          - 1./48.*m33*RSe63 - 1./24.*m33*RSe6633 - 1./48.*m33*RS36e + 
         1./48.*m33*RC346e + 1./48.*m33*RSe61 + 1./48.*m33*RS16e - 1./6.
         *m11 - 1./24.*m11*RSe61 - 1./24.*m11*RSe6611 - 1./8.*m11*RS16e
          + 1./16.*m11*RC146e );

      F13p6x22RV +=  + pow(AbV(m34,L),2) * (  - 1./8. );

      F13p6x22RV +=  + AbV(m34,L)*AbV(m36,L) * (  - 1./8. );

      F13p6x22RV +=  + AbV(m34,L)*AbV(me2,L) * ( 1./9.*RSe6613 - 5./36.
         *RSe63 - 7./36.*RSe6633 + 5./36.*RSe61 + 1./12.*RSe6611 );

      F13p6x22RV +=  + AbV(m34,L)*BbV(m11,m33,L) * ( 1./9.*m44*RS16e + 
         1./9.*m33*RS1463e + 1./9.*m33*RS16e - 1./9.*m11*RS1463e + 1./9.
         *m11*RS16e );

      F13p6x22RV +=  + AbV(m34,L)*BbV(m11,L) * (  - 1./24.*RD146e );

      F13p6x22RV +=  + AbV(m34,L)*BbV(m33,me2,L) * (  - 1./9.*m44*RSe63
          + 1./9.*m44*RSe61 - 2./9.*m33*RSe63 + 2./9.*m33*RSe61 );

      F13p6x22RV +=  + AbV(m34,L)*BbV(m33,L) * ( 7./72.*RD346e - 1./9.*
         m44*RS36e - 2./9.*m33*RS36e );

      F13p6x22RV +=  + AbV(m36,L)*pi16 * ( 1./48.*RD146e - 1./24.*me2
          + 1./12.*me2*RSe61 - 1./24.*me2*RSe6611 - 5./96.*m66 - 1./96.
         *m66*RS3461e - 1./96.*m66*RS1463e + 1./48.*m66*RSe6613 + 1./48.
         *m66*RSe63 - 1./48.*m66*RSe6633 - 1./96.*m66*RS34e + 1./96.*
         m66*RC346e + 1./48.*m66*RSe61 - 1./48.*m66*RSe6611 - 1./96.*
         m66*RS14e + 1./96.*m66*RC146e - 1./8.*m44 + 1./32.*m33 - 1./96.
         *m33*RS3461e - 1./96.*m33*RS1463e + 1./48.*m33*RSe6613 + 1./48.
         *m33*RSe63 - 1./48.*m33*RSe6633 - 1./96.*m33*RS34e + 1./96.*
         m33*RC346e - 1./48.*m33*RSe61 + 1./96.*m33*RS14e - 1./12.*m11
          + 1./24.*m11*RSe61 - 1./48.*m11*RSe6611 - 1./16.*m11*RS14e + 
         1./32.*m11*RC146e );

      F13p6x22RV +=  + pow(AbV(m36,L),2) * (  - 1./32. );

      F13p6x22RV +=  + AbV(m36,L)*AbV(me2,L) * ( 1./18.*RSe6613 + 5./36.
         *RSe63 - 7./72.*RSe6633 - 5./36.*RSe61 + 1./24.*RSe6611 );

      F13p6x22RV +=  + AbV(m36,L)*BbV(m11,m33,L) * ( 1./18.*m66*RS14e
          + 1./18.*m33*RS1463e + 1./18.*m33*RS14e - 1./18.*m11*RS1463e
          + 1./18.*m11*RS14e );

      F13p6x22RV +=  + AbV(m36,L)*BbV(m11,L) * (  - 1./48.*RD146e );

      F13p6x22RV +=  + AbV(m36,L)*BbV(m33,me2,L) * ( 1./9.*m66*RSe63 - 
         1./9.*m66*RSe61 + 2./9.*m33*RSe63 - 2./9.*m33*RSe61 );

      F13p6x22RV +=  + AbV(m36,L)*BbV(m33,L) * ( 7./144.*RD346e - 1./18.
         *m66*RS34e - 1./9.*m33*RS34e );

      F13p6x22RV +=  + AbV(m44,L)*BbV(m11,m33,L) * (  - 1./6.*m44*RS16e
         *RS36e );

      F13p6x22RV +=  + AbV(m44,L)*BbV(m11,me2,L) * (  - 1./6.*m44*RS16e
         *RSe63 + 1./6.*m44*RS16e*RSe61 );

      F13p6x22RV +=  + AbV(m44,L)*BbV(m11,L) * ( 1./12.*m44*pow(
         RS16e,2) );

      F13p6x22RV +=  + AbV(m44,L)*BbV(m33,me2,L) * ( 1./6.*m44*RS36e*
         RSe63 - 1./6.*m44*RSe61*RS36e );

      F13p6x22RV +=  + AbV(m44,L)*BbV(m33,L) * ( 1./12.*m44*pow(
         RS36e,2) );

      F13p6x22RV +=  + AbV(m44,L)*BbV(me2,L) * ( 1./12.*m44*pow(
         RSe63,2) - 1./6.*m44*RSe61*RSe63 + 1./12.*m44*pow(RSe61,2) );

      F13p6x22RV +=  + AbV(m46,L)*AbV(me2,L) * (  - 1./6.*pow(RSe63,2)
          - 1./9.*RS36e*RSe63 + 1./9.*RS34e*RSe63 + 1./3.*RSe61*RSe63
          + 1./9.*RSe61*RS36e - 1./9.*RSe61*RS34e - 1./6.*pow(RSe61,2)
          + 1./9.*RS16e*RSe63 - 1./9.*RS16e*RSe61 - 1./9.*RS14e*RSe63
          + 1./9.*RS14e*RSe61 );

      F13p6x22RV +=  + AbV(m46,L)*BbV(m11,m33,L) * (  - 1./27.*m66*
         RS16e*RS34e - 1./27.*m66*RS14e*RS36e - 1./27.*m66*RS14e*RS34e
          - 1./27.*m44*RS16e*RS36e - 1./27.*m44*RS16e*RS34e - 1./27.*
         m44*RS14e*RS36e + 1./27.*m11*RS16e*RS36e - 1./27.*m11*RS16e*
         RS34e - 1./27.*m11*RS14e*RS36e + 1./27.*m11*RS14e*RS34e );

      F13p6x22RV +=  + AbV(m46,L)*BbV(m11,me2,L) * ( 2./27.*m66*RS16e*
         RSe63 - 2./27.*m66*RS16e*RSe61 + 1./27.*m66*RS14e*RSe63 - 1./
         27.*m66*RS14e*RSe61 + 1./27.*m44*RS16e*RSe63 - 1./27.*m44*
         RS16e*RSe61 - 1./27.*m44*RS14e*RSe63 + 1./27.*m44*RS14e*RSe61
          + 1./9.*m11*RS16e*RSe63 - 1./9.*m11*RS16e*RSe61 - 1./9.*m11*
         RS14e*RSe63 + 1./9.*m11*RS14e*RSe61 );

      F13p6x22RV +=  + AbV(m46,L)*BbV(m11,L) * ( 1./27.*m66*RS14e*RS16e
          + 1./54.*m66*pow(RS14e,2) + 1./54.*m44*pow(RS16e,2) + 1./27.*
         m44*RS14e*RS16e - 1./54.*m11*pow(RS16e,2) + 1./27.*m11*RS14e*
         RS16e - 1./54.*m11*pow(RS14e,2) );

      F13p6x22RV +=  + AbV(m46,L)*BbV(m33,me2,L) * (  - 2./27.*m66*
         RS36e*RSe63 - 1./27.*m66*RS34e*RSe63 + 2./27.*m66*RSe61*RS36e
          + 1./27.*m66*RSe61*RS34e - 1./27.*m44*RS36e*RSe63 + 1./27.*
         m44*RS34e*RSe63 + 1./27.*m44*RSe61*RS36e - 1./27.*m44*RSe61*
         RS34e - 1./9.*m33*RS36e*RSe63 + 1./9.*m33*RS34e*RSe63 + 1./9.*
         m33*RSe61*RS36e - 1./9.*m33*RSe61*RS34e );

      F13p6x22RV +=  + AbV(m46,L)*BbV(m33,L) * ( 1./27.*m66*RS34e*RS36e
          + 1./54.*m66*pow(RS34e,2) + 1./54.*m44*pow(RS36e,2) + 1./27.*
         m44*RS34e*RS36e - 1./54.*m33*pow(RS36e,2) + 1./27.*m33*RS34e*
         RS36e - 1./54.*m33*pow(RS34e,2) );

      F13p6x22RV +=  + AbV(m46,L)*BbV(me2,L) * (  - 1./6.*me2*pow(
         RSe63,2) + 1./3.*me2*RSe61*RSe63 - 1./6.*me2*pow(RSe61,2) - 1./
         18.*m44*pow(RSe63,2) + 1./9.*m44*RSe61*RSe63 - 1./18.*m44*pow(
         RSe61,2) );

      F13p6x22RV +=  + AbV(me2,L)*pi16 * (  - 1./36.*RD346e*RSe6613 + 1.
         /72.*RD346e*RSe6633 + 1./72.*RSe6611*RD346e - 1./36.*RD146e*
         RSe6613 + 1./72.*RD146e*RSe6633 + 1./72.*RD146e*RSe6611 - 1./
         12.*me2*RSe6613 - 1./9.*me2*pow(RSe6613,2) + 1./24.*me2*
         RSe6633 + 1./9.*me2*RSe6633*RSe6613 - 1./36.*me2*pow(
         RSe6633,2) + 1./24.*me2*RSe6611 + 1./9.*me2*RSe6611*RSe6613 - 
         1./18.*me2*RSe6611*RSe6633 - 1./36.*me2*pow(RSe6611,2) + 1./12.
         *m66*RSe6613 - 1./24.*m66*RSe6633 - 1./24.*m66*RSe6611 + 1./6.
         *m44*RSe6613 - 1./12.*m44*RSe6633 - 1./12.*m44*RSe6611 + 1./8.
         *m33*RSe6613 + 1./12.*m33*RSe6613*RS3461e + 1./36.*m33*RSe6613
         *RS1463e - 1./18.*m33*pow(RSe6613,2) - 1./16.*m33*RSe6633 - 1./
         24.*m33*RSe6633*RS3461e - 1./72.*m33*RSe6633*RS1463e + 1./18.*
         m33*RSe6633*RSe6613 - 1./72.*m33*pow(RSe6633,2) - 1./24.*m33*
         RC346e*RSe6613 + 1./48.*m33*RC346e*RSe6633 - 1./16.*m33*
         RSe6611 - 1./24.*m33*RSe6611*RS3461e - 1./72.*m33*RSe6611*
         RS1463e );

      F13p6x22RV +=  + AbV(me2,L)*pi16 * ( 1./18.*m33*RSe6611*RSe6613
          - 1./36.*m33*RSe6611*RSe6633 + 1./48.*m33*RSe6611*RC346e - 1./
         72.*m33*pow(RSe6611,2) - 1./72.*m33*RC146e*RSe6613 + 1./144.*
         m33*RC146e*RSe6633 + 1./144.*m33*RC146e*RSe6611 + 1./8.*m11*
         RSe6613 + 1./36.*m11*RSe6613*RS3461e + 1./12.*m11*RSe6613*
         RS1463e - 1./18.*m11*pow(RSe6613,2) - 1./16.*m11*RSe6633 - 1./
         72.*m11*RSe6633*RS3461e - 1./24.*m11*RSe6633*RS1463e + 1./18.*
         m11*RSe6633*RSe6613 - 1./72.*m11*pow(RSe6633,2) - 1./72.*m11*
         RC346e*RSe6613 + 1./144.*m11*RC346e*RSe6633 - 1./16.*m11*
         RSe6611 - 1./72.*m11*RSe6611*RS3461e - 1./24.*m11*RSe6611*
         RS1463e + 1./18.*m11*RSe6611*RSe6613 - 1./36.*m11*RSe6611*
         RSe6633 + 1./144.*m11*RSe6611*RC346e - 1./72.*m11*pow(
         RSe6611,2) - 1./24.*m11*RC146e*RSe6613 + 1./48.*m11*RC146e*
         RSe6633 + 1./48.*m11*RC146e*RSe6611 );

      F13p6x22RV +=  + pow(AbV(me2,L),2) * ( 1./18.*pow(RSe6613,2) - 1./
         18.*RSe6633*RSe6613 + 1./72.*pow(RSe6633,2) - 1./18.*RSe6611*
         RSe6613 + 1./36.*RSe6611*RSe6633 + 1./72.*pow(RSe6611,2) );

      F13p6x22RV +=  + AbV(me2,L)*BbV(m11,m33,L) * (  - 1./27.*m66*
         RS14e*RS34e - 1./54.*m44*RS16e*RS36e + 1./9.*m33*RSe6633*
         RS1463e + 1./9.*m11*RSe6611*RS3461e );

      F13p6x22RV +=  + AbV(me2,L)*BbV(m11,me2,L) * ( 2./27.*m66*RS14e*
         RSe63 - 2./27.*m66*RS14e*RSe61 - 1./54.*m44*RS16e*RSe63 + 1./
         54.*m44*RS16e*RSe61 - 2./9.*m11*RSe6611*RSe6613 + 2./9.*m11*
         pow(RSe6611,2) );

      F13p6x22RV +=  + AbV(me2,L)*BbV(m11,L) * ( 1./36.*RD146e*RSe6613
          - 1./72.*RD146e*RSe6633 - 1./72.*RD146e*RSe6611 + 1./54.*m66*
         pow(RS14e,2) + 1./108.*m44*pow(RS16e,2) + 1./9.*m11*RSe6611*
         RS1463e - 1./9.*m11*RC146e*RSe6611 );

      F13p6x22RV +=  + AbV(me2,L)*BbV(m14,L) * (  - 1./72.*me2 - 1./18.
         *me2*RSe61 - 1./18.*me2*RSe6611 + 1./72.*m44 - 1./18.*m44*
         RSe61 - 1./18.*m11*RSe61 + 1./18.*m11*RSe6611 );

      F13p6x22RV +=  + AbV(me2,L)*BbV(m16,L) * (  - 1./36.*me2 + 1./18.
         *me2*RSe61 - 1./36.*me2*RSe6611 + 1./36.*m66 + 1./18.*m66*
         RSe61 + 1./18.*m11*RSe61 + 1./36.*m11*RSe6611 );

      F13p6x22RV +=  + AbV(me2,L)*BbV(m33,me2,L) * (  - 2./27.*m66*
         RS34e*RSe63 + 2./27.*m66*RSe61*RS34e + 1./54.*m44*RS36e*RSe63
          - 1./54.*m44*RSe61*RS36e - 2./9.*m33*RSe6633*RSe6613 + 2./9.*
         m33*pow(RSe6633,2) );

      F13p6x22RV +=  + AbV(me2,L)*BbV(m33,L) * ( 1./36.*RD346e*RSe6613
          - 1./72.*RD346e*RSe6633 - 1./72.*RSe6611*RD346e + 1./54.*m66*
         pow(RS34e,2) + 1./108.*m44*pow(RS36e,2) + 1./9.*m33*RSe6633*
         RS3461e - 1./9.*m33*RC346e*RSe6633 );

      F13p6x22RV +=  + AbV(me2,L)*BbV(m34,L) * (  - 1./72.*me2 - 1./18.
         *me2*RSe63 - 1./18.*me2*RSe6633 + 1./72.*m44 - 1./18.*m44*
         RSe63 - 1./18.*m33*RSe63 + 1./18.*m33*RSe6633 );

      F13p6x22RV +=  + AbV(me2,L)*BbV(m36,L) * (  - 1./36.*me2 + 1./18.
         *me2*RSe63 - 1./36.*me2*RSe6633 + 1./36.*m66 + 1./18.*m66*
         RSe63 + 1./18.*m33*RSe63 + 1./36.*m33*RSe6633 );

      F13p6x22RV +=  + AbV(me2,L)*BbV(me2,L) * ( 2./27.*m66*pow(
         RSe63,2) - 4./27.*m66*RSe61*RSe63 + 2./27.*m66*pow(RSe61,2) + 
         1./108.*m44*pow(RSe63,2) - 1./54.*m44*RSe61*RSe63 + 1./108.*
         m44*pow(RSe61,2) );

      F13p6x22RV +=  + AbV(me2,L)*CbV(m11,L) * (  - 1./9.*m11*RD146e*
         RSe6611 );

      F13p6x22RV +=  + AbV(me2,L)*CbV(m33,L) * (  - 1./9.*m33*RD346e*
         RSe6633 );

      F13p6x22RV +=  + BbV(m11,m33,L)*BbV(m11,L) * (  - 1./18.*m11*
         RD146e*RS3461e );

      F13p6x22RV +=  + BbV(m11,m33,L)*BbV(m33,L) * (  - 1./18.*m33*
         RD346e*RS1463e );

      F13p6x22RV +=  + BbV(m11,me2,L)*BbV(m11,L) * ( 1./9.*m11*RD146e*
         RSe6613 - 1./9.*m11*RD146e*RSe6611 );

      F13p6x22RV +=  + BbV(m11,L)*pi16 * (  - 1./144.*RD146e*RD346e - 1.
         /144.*pow(RD146e,2) - 1./36.*me2*RD146e*RSe6613 + 1./72.*me2*
         RD146e*RSe6633 + 1./72.*me2*RD146e*RSe6611 + 1./48.*m66*RD146e
          + 1./24.*m44*RD146e + 1./32.*m33*RD146e + 1./48.*m33*RD146e*
         RS3461e + 1./144.*m33*RD146e*RS1463e - 1./72.*m33*RD146e*
         RSe6613 + 1./144.*m33*RD146e*RSe6633 - 1./96.*m33*RD146e*
         RC346e + 1./144.*m33*RD146e*RSe6611 - 1./288.*m33*RD146e*
         RC146e + 1./96.*m11*RD146e + 1./144.*m11*RD146e*RS3461e + 1./
         48.*m11*RD146e*RS1463e - 1./72.*m11*RD146e*RSe6613 + 1./144.*
         m11*RD146e*RSe6633 - 1./288.*m11*RD146e*RC346e + 1./144.*m11*
         RD146e*RSe6611 - 1./96.*m11*RD146e*RC146e );

      F13p6x22RV +=  + pow(BbV(m11,L),2) * ( 1./288.*pow(RD146e,2) - 1./
         18.*m11*RD146e*RS1463e + 1./18.*m11*RD146e*RC146e );

      F13p6x22RV +=  + BbV(m11,L)*BbV(m33,L) * ( 1./144.*RD146e*RD346e
          );

      F13p6x22RV +=  + BbV(m11,L)*CbV(m11,L) * ( 1./18.*m11*pow(
         RD146e,2) );

      F13p6x22RV +=  + BbV(m33,me2,L)*BbV(m33,L) * ( 1./9.*m33*RD346e*
         RSe6613 - 1./9.*m33*RD346e*RSe6633 );

      F13p6x22RV +=  + BbV(m33,L)*pi16 * (  - 1./144.*pow(RD346e,2) - 1.
         /144.*RD146e*RD346e - 1./36.*me2*RD346e*RSe6613 + 1./72.*me2*
         RD346e*RSe6633 + 1./72.*me2*RSe6611*RD346e + 1./48.*m66*RD346e
          + 1./24.*m44*RD346e + 1./96.*m33*RD346e + 1./48.*m33*RD346e*
         RS3461e + 1./144.*m33*RD346e*RS1463e - 1./72.*m33*RD346e*
         RSe6613 + 1./144.*m33*RD346e*RSe6633 - 1./96.*m33*RD346e*
         RC346e + 1./144.*m33*RSe6611*RD346e - 1./288.*m33*RC146e*
         RD346e + 1./32.*m11*RD346e + 1./144.*m11*RD346e*RS3461e + 1./
         48.*m11*RD346e*RS1463e - 1./72.*m11*RD346e*RSe6613 + 1./144.*
         m11*RD346e*RSe6633 - 1./288.*m11*RD346e*RC346e + 1./144.*m11*
         RSe6611*RD346e - 1./96.*m11*RC146e*RD346e );

      F13p6x22RV +=  + pow(BbV(m33,L),2) * ( 1./288.*pow(RD346e,2) - 1./
         18.*m33*RD346e*RS3461e + 1./18.*m33*RD346e*RC346e );

      F13p6x22RV +=  + BbV(m33,L)*CbV(m33,L) * ( 1./18.*m33*pow(
         RD346e,2) );

      F13p6x22RV +=  + A23bV(m11,L)*pi16 * (  - 3./16. + 1./36.*RS1463e
         *RS3461e + 1./36.*pow(RS1463e,2) - 1./18.*RSe6613*RS1463e + 1./
         36.*RSe6633*RS1463e - 1./72.*RC346e*RS1463e + 1./36.*RSe6611*
         RS1463e + 1./8.*RS16e + 1./16.*RS14e - 1./72.*RC146e*RS3461e
          - 1./36.*RC146e*RS1463e + 1./36.*RC146e*RSe6613 - 1./72.*
         RC146e*RSe6633 + 1./144.*RC146e*RC346e - 1./72.*RC146e*RSe6611
          + 1./144.*pow(RC146e,2) );

      F13p6x22RV +=  + A23bV(m13,L)*pi16 * ( 1./24.*pow(RS3461e,2) + 1./
         12.*RS1463e*RS3461e + 1./24.*pow(RS1463e,2) - 1./6.*RSe6613*
         RS3461e - 1./6.*RSe6613*RS1463e + 1./6.*pow(RSe6613,2) + 1./12.
         *RSe6633*RS3461e + 1./12.*RSe6633*RS1463e - 1./6.*RSe6633*
         RSe6613 + 1./24.*pow(RSe6633,2) - 1./24.*RC346e*RS3461e - 1./
         24.*RC346e*RS1463e + 1./12.*RC346e*RSe6613 - 1./24.*RC346e*
         RSe6633 + 1./96.*pow(RC346e,2) + 1./12.*RSe6611*RS3461e + 1./
         12.*RSe6611*RS1463e - 1./6.*RSe6611*RSe6613 + 1./12.*RSe6611*
         RSe6633 - 1./24.*RSe6611*RC346e + 1./24.*pow(RSe6611,2) - 1./
         24.*RC146e*RS3461e - 1./24.*RC146e*RS1463e + 1./12.*RC146e*
         RSe6613 - 1./24.*RC146e*RSe6633 + 1./48.*RC146e*RC346e - 1./24.
         *RC146e*RSe6611 + 1./96.*pow(RC146e,2) );

      F13p6x22RV +=  + A23bV(m14,L)*pi16 * (  - 1./24. + 1./8.*RS3461e
          + 1./8.*RS1463e - 1./4.*RSe6613 - 1./24.*RSe63 + 1./12.*
         RSe6633 - 1./24.*RS36e - 1./24.*RC346e + 1./8.*RSe61 + 1./4.*
         RSe6611 + 1./8.*RS16e - 1./8.*RC146e );

      F13p6x22RV +=  + A23bV(m16,L)*pi16 * (  - 1./48. + 1./16.*RS3461e
          + 1./16.*RS1463e - 1./8.*RSe6613 + 1./24.*RSe63 + 1./24.*
         RSe6633 - 1./48.*RS34e - 1./48.*RC346e - 1./8.*RSe61 + 1./8.*
         RSe6611 + 1./16.*RS14e - 1./16.*RC146e );

      F13p6x22RV +=  + A23bV(m33,L)*pi16 * (  - 3./16. + 1./36.*pow(
         RS3461e,2) + 1./36.*RS1463e*RS3461e - 1./18.*RSe6613*RS3461e
          + 1./36.*RSe6633*RS3461e + 1./8.*RS36e + 1./16.*RS34e - 1./36.
         *RC346e*RS3461e - 1./72.*RC346e*RS1463e + 1./36.*RC346e*
         RSe6613 - 1./72.*RC346e*RSe6633 + 1./144.*pow(RC346e,2) + 1./
         36.*RSe6611*RS3461e - 1./72.*RSe6611*RC346e - 1./72.*RC146e*
         RS3461e + 1./144.*RC146e*RC346e );

      F13p6x22RV +=  + A23bV(m34,L)*pi16 * (  - 1./24. + 1./8.*RS3461e
          + 1./8.*RS1463e - 1./4.*RSe6613 + 1./8.*RSe63 + 1./4.*RSe6633
          + 1./8.*RS36e - 1./8.*RC346e - 1./24.*RSe61 + 1./12.*RSe6611
          - 1./24.*RS16e - 1./24.*RC146e );

      F13p6x22RV +=  + A23bV(m36,L)*pi16 * (  - 1./48. + 1./16.*RS3461e
          + 1./16.*RS1463e - 1./8.*RSe6613 - 1./8.*RSe63 + 1./8.*
         RSe6633 + 1./16.*RS34e - 1./16.*RC346e + 1./24.*RSe61 + 1./24.
         *RSe6611 - 1./48.*RS14e - 1./48.*RC146e );

      F13p6x22RV +=  + A23bV(me2,L)*pi16 * (  - 1./18.*RSe6613*RS3461e
          - 1./18.*RSe6613*RS1463e + 1./9.*pow(RSe6613,2) + 1./36.*
         RSe6633*RS3461e + 1./36.*RSe6633*RS1463e - 1./9.*RSe6633*
         RSe6613 + 1./36.*pow(RSe6633,2) + 1./36.*RC346e*RSe6613 - 1./
         72.*RC346e*RSe6633 + 1./36.*RSe6611*RS3461e + 1./36.*RSe6611*
         RS1463e - 1./9.*RSe6611*RSe6613 + 1./18.*RSe6611*RSe6633 - 1./
         72.*RSe6611*RC346e + 1./36.*pow(RSe6611,2) + 1./36.*RC146e*
         RSe6613 - 1./72.*RC146e*RSe6633 - 1./72.*RC146e*RSe6611 );

      F13p6x22RV +=  + B23bV(m11,L)*pi16 * (  - 1./72.*RD146e*RS3461e
          - 1./72.*RD146e*RS1463e + 1./36.*RD146e*RSe6613 - 1./72.*
         RD146e*RSe6633 + 1./144.*RD146e*RC346e - 1./72.*RD146e*RSe6611
          + 1./144.*RD146e*RC146e );

      F13p6x22RV +=  + B23bV(m33,L)*pi16 * (  - 1./72.*RD346e*RS3461e
          - 1./72.*RD346e*RS1463e + 1./36.*RD346e*RSe6613 - 1./72.*
         RD346e*RSe6633 + 1./144.*RD346e*RC346e - 1./72.*RSe6611*RD346e
          + 1./144.*RC146e*RD346e );

      F13p6x22RV +=  + hhV(1,m11,m11,m13,m13,L,mu2) * ( 1./8.*m11 );

      F13p6x22RV +=  + hhV(1,m11,m14,m34,m13,L,mu2) * ( 1./12.*m11*
         RS1463e - 1./12.*m11*RS16e );

      F13p6x22RV +=  + hhV(1,m11,m16,m36,m13,L,mu2) * ( 1./24.*m11*
         RS1463e - 1./24.*m11*RS14e );

      F13p6x22RV +=  + hhV(1,m13,m11,m11,m13,L,mu2) * (  - 1./32.*m33
          - 1./72.*m33*pow(RS1463e,2) + 1./72.*m33*RC146e*RS1463e - 1./
         288.*m33*pow(RC146e,2) - 1./32.*m11 - 1./72.*m11*pow(
         RS1463e,2) + 1./72.*m11*RC146e*RS1463e - 1./288.*m11*pow(
         RC146e,2) );

      F13p6x22RV +=  + hhV(1,m13,m11,m33,m13,L,mu2) * (  - 1./36.*m33*
         RS1463e*RS3461e + 1./72.*m33*RC346e*RS1463e + 1./72.*m33*
         RC146e*RS3461e - 1./144.*m33*RC146e*RC346e - 1./36.*m11*
         RS1463e*RS3461e + 1./72.*m11*RC346e*RS1463e + 1./72.*m11*
         RC146e*RS3461e - 1./144.*m11*RC146e*RC346e );

      F13p6x22RV +=  + hhV(1,m13,m11,me2,m13,L,mu2) * ( 1./18.*m33*
         RSe6613*RS1463e - 1./36.*m33*RSe6633*RS1463e - 1./36.*m33*
         RSe6611*RS1463e - 1./36.*m33*RC146e*RSe6613 + 1./72.*m33*
         RC146e*RSe6633 + 1./72.*m33*RC146e*RSe6611 + 1./18.*m11*
         RSe6613*RS1463e - 1./36.*m11*RSe6633*RS1463e - 1./36.*m11*
         RSe6611*RS1463e - 1./36.*m11*RC146e*RSe6613 + 1./72.*m11*
         RC146e*RSe6633 + 1./72.*m11*RC146e*RSe6611 );

      F13p6x22RV +=  + hhV(1,m13,m13,m13,m13,L,mu2) * (  - 1./16.*m33
          - 1./16.*m11 );

      F13p6x22RV +=  + hhV(1,m13,m33,m33,m13,L,mu2) * (  - 1./32.*m33
          - 1./72.*m33*pow(RS3461e,2) + 1./72.*m33*RC346e*RS3461e - 1./
         288.*m33*pow(RC346e,2) - 1./32.*m11 - 1./72.*m11*pow(
         RS3461e,2) + 1./72.*m11*RC346e*RS3461e - 1./288.*m11*pow(
         RC346e,2) );

      F13p6x22RV +=  + hhV(1,m13,m33,me2,m13,L,mu2) * ( 1./18.*m33*
         RSe6613*RS3461e - 1./36.*m33*RSe6633*RS3461e - 1./36.*m33*
         RC346e*RSe6613 + 1./72.*m33*RC346e*RSe6633 - 1./36.*m33*
         RSe6611*RS3461e + 1./72.*m33*RSe6611*RC346e + 1./18.*m11*
         RSe6613*RS3461e - 1./36.*m11*RSe6633*RS3461e - 1./36.*m11*
         RC346e*RSe6613 + 1./72.*m11*RC346e*RSe6633 - 1./36.*m11*
         RSe6611*RS3461e + 1./72.*m11*RSe6611*RC346e );

      F13p6x22RV +=  + hhV(1,m13,me2,me2,m13,L,mu2) * (  - 1./18.*m33*
         pow(RSe6613,2) + 1./18.*m33*RSe6633*RSe6613 - 1./72.*m33*pow(
         RSe6633,2) + 1./18.*m33*RSe6611*RSe6613 - 1./36.*m33*RSe6611*
         RSe6633 - 1./72.*m33*pow(RSe6611,2) - 1./18.*m11*pow(
         RSe6613,2) + 1./18.*m11*RSe6633*RSe6613 - 1./72.*m11*pow(
         RSe6633,2) + 1./18.*m11*RSe6611*RSe6613 - 1./36.*m11*RSe6611*
         RSe6633 - 1./72.*m11*pow(RSe6611,2) );

      F13p6x22RV +=  + hhV(1,m14,m11,m34,m13,L,mu2) * (  - 1./24.*m44*
         RS1463e - 1./24.*m44*RS16e + 1./24.*m44*RC146e - 1./24.*m11*
         RS1463e - 1./24.*m11*RS16e + 1./24.*m11*RC146e );

      F13p6x22RV +=  + hhV(1,m14,m33,m34,m13,L,mu2) * (  - 1./24.*m44*
         RS3461e + 1./24.*m44*RS36e - 1./24.*m11*RS3461e + 1./24.*m11*
         RS36e );

      F13p6x22RV +=  + hhV(1,m14,m34,me2,m13,L,mu2) * ( 1./12.*m44*
         RSe6613 + 1./24.*m44*RSe63 - 1./24.*m44*RSe61 - 1./12.*m44*
         RSe6611 + 1./12.*m11*RSe6613 + 1./24.*m11*RSe63 - 1./24.*m11*
         RSe61 - 1./12.*m11*RSe6611 );

      F13p6x22RV +=  + hhV(1,m16,m11,m36,m13,L,mu2) * (  - 1./48.*m66*
         RS1463e - 1./48.*m66*RS14e + 1./48.*m66*RC146e - 1./48.*m11*
         RS1463e - 1./48.*m11*RS14e + 1./48.*m11*RC146e );

      F13p6x22RV +=  + hhV(1,m16,m33,m36,m13,L,mu2) * (  - 1./48.*m66*
         RS3461e + 1./48.*m66*RS34e - 1./48.*m11*RS3461e + 1./48.*m11*
         RS34e );

      F13p6x22RV +=  + hhV(1,m16,m36,me2,m13,L,mu2) * ( 1./24.*m66*
         RSe6613 - 1./24.*m66*RSe63 + 1./24.*m66*RSe61 - 1./24.*m66*
         RSe6611 + 1./24.*m11*RSe6613 - 1./24.*m11*RSe63 + 1./24.*m11*
         RSe61 - 1./24.*m11*RSe6611 );

      F13p6x22RV +=  + hhV(1,m33,m13,m33,m13,L,mu2) * ( 1./8.*m33 );

      F13p6x22RV +=  + hhV(1,m33,m14,m34,m13,L,mu2) * ( 1./12.*m33*
         RS3461e - 1./12.*m33*RS36e );

      F13p6x22RV +=  + hhV(1,m33,m16,m36,m13,L,mu2) * ( 1./24.*m33*
         RS3461e - 1./24.*m33*RS34e );

      F13p6x22RV +=  + hhV(1,m34,m11,m14,m13,L,mu2) * (  - 1./24.*m44*
         RS1463e + 1./24.*m44*RS16e - 1./24.*m33*RS1463e + 1./24.*m33*
         RS16e );

      F13p6x22RV +=  + hhV(1,m34,m14,m33,m13,L,mu2) * (  - 1./24.*m44*
         RS3461e - 1./24.*m44*RS36e + 1./24.*m44*RC346e - 1./24.*m33*
         RS3461e - 1./24.*m33*RS36e + 1./24.*m33*RC346e );

      F13p6x22RV +=  + hhV(1,m34,m14,me2,m13,L,mu2) * ( 1./12.*m44*
         RSe6613 - 1./24.*m44*RSe63 - 1./12.*m44*RSe6633 + 1./24.*m44*
         RSe61 + 1./12.*m33*RSe6613 - 1./24.*m33*RSe63 - 1./12.*m33*
         RSe6633 + 1./24.*m33*RSe61 );

      F13p6x22RV +=  + hhV(1,m36,m11,m16,m13,L,mu2) * (  - 1./48.*m66*
         RS1463e + 1./48.*m66*RS14e - 1./48.*m33*RS1463e + 1./48.*m33*
         RS14e );

      F13p6x22RV +=  + hhV(1,m36,m16,m33,m13,L,mu2) * (  - 1./48.*m66*
         RS3461e - 1./48.*m66*RS34e + 1./48.*m66*RC346e - 1./48.*m33*
         RS3461e - 1./48.*m33*RS34e + 1./48.*m33*RC346e );

      F13p6x22RV +=  + hhV(1,m36,m16,me2,m13,L,mu2) * ( 1./24.*m66*
         RSe6613 + 1./24.*m66*RSe63 - 1./24.*m66*RSe6633 - 1./24.*m66*
         RSe61 + 1./24.*m33*RSe6613 + 1./24.*m33*RSe63 - 1./24.*m33*
         RSe6633 - 1./24.*m33*RSe61 );

      F13p6x22RV +=  + hhV(1,m44,m14,m34,m13,L,mu2) * (  - 3./8.*m44 );

      F13p6x22RV +=  + hhV(1,m46,m14,m36,m13,L,mu2) * (  - 1./8.*m66 - 
         1./8.*m44 );

      F13p6x22RV +=  + hhV(1,m46,m16,m34,m13,L,mu2) * (  - 1./8.*m66 - 
         1./8.*m44 );

      F13p6x22RV +=  + hhV(1,me2,m14,m34,m13,L,mu2) * (  - 1./24.*me2
          - 1./6.*me2*RSe6613 - 1./12.*me2*RSe63 - 1./12.*me2*RSe61 );

      F13p6x22RV +=  + hhV(1,me2,m16,m36,m13,L,mu2) * (  - 1./12.*me2
          - 1./12.*me2*RSe6613 + 1./12.*me2*RSe63 + 1./12.*me2*RSe61 );

      F13p6x22RV +=  + hhV(2,m11,m11,m13,m13,L,mu2) * (  - 1./1296.*
         pow(RD146e,2) );

      F13p6x22RV +=  + hhV(3,m11,m11,m13,m13,L,mu2) * ( 1./1296.*pow(
         RD146e,2) );

      F13p6x22RV +=  + hhV(3,m13,m11,m11,m13,L,mu2) * ( 1./72.*m33*
         RD146e*RS1463e - 1./144.*m33*RD146e*RC146e + 1./72.*m11*RD146e
         *RS1463e - 1./144.*m11*RD146e*RC146e );

      F13p6x22RV +=  + hhV(3,m13,m11,m33,m13,L,mu2) * ( 1./72.*m33*
         RD146e*RS3461e - 1./144.*m33*RD146e*RC346e + 1./72.*m11*RD146e
         *RS3461e - 1./144.*m11*RD146e*RC346e );

      F13p6x22RV +=  + hhV(3,m13,m11,me2,m13,L,mu2) * (  - 1./36.*m33*
         RD146e*RSe6613 + 1./72.*m33*RD146e*RSe6633 + 1./72.*m33*RD146e
         *RSe6611 - 1./36.*m11*RD146e*RSe6613 + 1./72.*m11*RD146e*
         RSe6633 + 1./72.*m11*RD146e*RSe6611 );

      F13p6x22RV +=  + hhV(3,m13,m33,m33,m13,L,mu2) * (  - 1./324.*pow(
         RD346e,2) + 1./72.*m33*RD346e*RS3461e - 1./144.*m33*RD346e*
         RC346e + 1./72.*m11*RD346e*RS3461e - 1./144.*m11*RD346e*RC346e
          );

      F13p6x22RV +=  + hhV(3,m13,m33,me2,m13,L,mu2) * (  - 1./36.*m33*
         RD346e*RSe6613 + 1./72.*m33*RD346e*RSe6633 + 1./72.*m33*
         RSe6611*RD346e - 1./36.*m11*RD346e*RSe6613 + 1./72.*m11*RD346e
         *RSe6633 + 1./72.*m11*RSe6611*RD346e );

      F13p6x22RV +=  + hhV(3,m14,m11,m34,m13,L,mu2) * ( 1./24.*m44*
         RD146e + 1./24.*m11*RD146e );

      F13p6x22RV +=  + hhV(3,m16,m11,m36,m13,L,mu2) * ( 1./48.*m66*
         RD146e + 1./48.*m11*RD146e );

      F13p6x22RV +=  + hhV(4,m13,m11,m33,m13,L,mu2) * ( 1./72.*m33*
         RD346e*RS1463e - 1./144.*m33*RC146e*RD346e + 1./72.*m11*RD346e
         *RS1463e - 1./144.*m11*RC146e*RD346e );

      F13p6x22RV +=  + hhV(4,m13,m33,m33,m13,L,mu2) * (  - 1./324.*pow(
         RD346e,2) );

      F13p6x22RV +=  + hhV(4,m34,m14,m33,m13,L,mu2) * ( 1./24.*m44*
         RD346e + 1./24.*m33*RD346e );

      F13p6x22RV +=  + hhV(4,m36,m16,m33,m13,L,mu2) * ( 1./48.*m66*
         RD346e + 1./48.*m33*RD346e );

      F13p6x22RV +=  + hhV(7,m13,m11,m11,m13,L,mu2) * (  - 1./288.*m33*
         pow(RD146e,2) - 1./288.*m11*pow(RD146e,2) );

      F13p6x22RV +=  + hhV(7,m13,m11,m33,m13,L,mu2) * (  - 1./144.*m33*
         RD146e*RD346e - 1./144.*m11*RD146e*RD346e );

      F13p6x22RV +=  + hhV(7,m13,m33,m33,m13,L,mu2) * (  - 5./1296.*m33
         *pow(RD346e,2) - 5./1296.*m11*pow(RD346e,2) );

      F13p6x22RV +=  + hh1V(2,m33,m13,m33,m13,L,mu2) * ( 1./162.*pow(
         RD346e,2) );

      F13p6x22RV +=  + hh1V(3,m13,m33,m33,m13,L,mu2) * ( 1./432.*pow(
         RD346e,2) );

      F13p6x22RV +=  + hh1V(4,m13,m33,m33,m13,L,mu2) * ( 5./1296.*pow(
         RD346e,2) );

      F13p6x22RV +=  + hh1V(4,m33,m13,m33,m13,L,mu2) * ( 1./162.*pow(
         RD346e,2) );

      F13p6x22RV +=  + hh1V(6,m33,m13,m33,m13,L,mu2) * ( 1./1296.*m33*
         pow(RD346e,2) + 1./1296.*m11*pow(RD346e,2) );

      F13p6x22RV +=  + hh1V(7,m13,m33,m33,m13,L,mu2) * ( 1./2592.*m33*
         pow(RD346e,2) + 1./2592.*m11*pow(RD346e,2) );

      F13p6x22RV +=  + hh27V(1,m11,m11,m13,m13,L,mu2) * (  - 3./8. );

      F13p6x22RV +=  + hh27V(1,m11,m14,m34,m13,L,mu2) * (  - 1./4.*
         RS1463e + 1./4.*RS16e );

      F13p6x22RV +=  + hh27V(1,m11,m16,m36,m13,L,mu2) * (  - 1./8.*
         RS1463e + 1./8.*RS14e );

      F13p6x22RV +=  + hh27V(1,m13,m11,m11,m13,L,mu2) * ( 3./16. + 1./
         12.*pow(RS1463e,2) - 1./12.*RC146e*RS1463e + 1./48.*pow(
         RC146e,2) );

      F13p6x22RV +=  + hh27V(1,m13,m11,m33,m13,L,mu2) * ( 1./6.*RS1463e
         *RS3461e - 1./12.*RC346e*RS1463e - 1./12.*RC146e*RS3461e + 1./
         24.*RC146e*RC346e );

      F13p6x22RV +=  + hh27V(1,m13,m11,me2,m13,L,mu2) * (  - 1./3.*
         RSe6613*RS1463e + 1./6.*RSe6633*RS1463e + 1./6.*RSe6611*
         RS1463e + 1./6.*RC146e*RSe6613 - 1./12.*RC146e*RSe6633 - 1./12.
         *RC146e*RSe6611 );

      F13p6x22RV +=  + hh27V(1,m13,m13,m13,m13,L,mu2) * ( 3./8. );

      F13p6x22RV +=  + hh27V(1,m13,m33,m33,m13,L,mu2) * ( 3./16. + 1./
         12.*pow(RS3461e,2) - 1./12.*RC346e*RS3461e + 1./48.*pow(
         RC346e,2) );

      F13p6x22RV +=  + hh27V(1,m13,m33,me2,m13,L,mu2) * (  - 1./3.*
         RSe6613*RS3461e + 1./6.*RSe6633*RS3461e + 1./6.*RC346e*RSe6613
          - 1./12.*RC346e*RSe6633 + 1./6.*RSe6611*RS3461e - 1./12.*
         RSe6611*RC346e );

      F13p6x22RV +=  + hh27V(1,m13,me2,me2,m13,L,mu2) * ( 1./3.*pow(
         RSe6613,2) - 1./3.*RSe6633*RSe6613 + 1./12.*pow(RSe6633,2) - 1.
         /3.*RSe6611*RSe6613 + 1./6.*RSe6611*RSe6633 + 1./12.*pow(
         RSe6611,2) );

      F13p6x22RV +=  + hh27V(1,m14,m11,m34,m13,L,mu2) * ( 1./4.*RS1463e
          + 1./4.*RS16e - 1./4.*RC146e );

      F13p6x22RV +=  + hh27V(1,m14,m33,m34,m13,L,mu2) * ( 1./4.*RS3461e
          - 1./4.*RS36e );

      F13p6x22RV +=  + hh27V(1,m14,m34,me2,m13,L,mu2) * (  - 1./2.*
         RSe6613 - 1./4.*RSe63 + 1./4.*RSe61 + 1./2.*RSe6611 );

      F13p6x22RV +=  + hh27V(1,m16,m11,m36,m13,L,mu2) * ( 1./8.*RS1463e
          + 1./8.*RS14e - 1./8.*RC146e );

      F13p6x22RV +=  + hh27V(1,m16,m33,m36,m13,L,mu2) * ( 1./8.*RS3461e
          - 1./8.*RS34e );

      F13p6x22RV +=  + hh27V(1,m16,m36,me2,m13,L,mu2) * (  - 1./4.*
         RSe6613 + 1./4.*RSe63 - 1./4.*RSe61 + 1./4.*RSe6611 );

      F13p6x22RV +=  + hh27V(1,m33,m13,m33,m13,L,mu2) * (  - 3./8. );

      F13p6x22RV +=  + hh27V(1,m33,m14,m34,m13,L,mu2) * (  - 1./4.*
         RS3461e + 1./4.*RS36e );

      F13p6x22RV +=  + hh27V(1,m33,m16,m36,m13,L,mu2) * (  - 1./8.*
         RS3461e + 1./8.*RS34e );

      F13p6x22RV +=  + hh27V(1,m34,m11,m14,m13,L,mu2) * ( 1./4.*RS1463e
          - 1./4.*RS16e );

      F13p6x22RV +=  + hh27V(1,m34,m14,m33,m13,L,mu2) * ( 1./4.*RS3461e
          + 1./4.*RS36e - 1./4.*RC346e );

      F13p6x22RV +=  + hh27V(1,m34,m14,me2,m13,L,mu2) * (  - 1./2.*
         RSe6613 + 1./4.*RSe63 + 1./2.*RSe6633 - 1./4.*RSe61 );

      F13p6x22RV +=  + hh27V(1,m36,m11,m16,m13,L,mu2) * ( 1./8.*RS1463e
          - 1./8.*RS14e );

      F13p6x22RV +=  + hh27V(1,m36,m16,m33,m13,L,mu2) * ( 1./8.*RS3461e
          + 1./8.*RS34e - 1./8.*RC346e );

      F13p6x22RV +=  + hh27V(1,m36,m16,me2,m13,L,mu2) * (  - 1./4.*
         RSe6613 - 1./4.*RSe63 + 1./4.*RSe6633 + 1./4.*RSe61 );

      F13p6x22RV +=  + hh27V(1,m44,m14,m34,m13,L,mu2) * ( 9./8. );

      F13p6x22RV +=  + hh27V(1,m46,m14,m36,m13,L,mu2) * ( 3./4. );

      F13p6x22RV +=  + hh27V(1,m46,m16,m34,m13,L,mu2) * ( 3./4. );

      F13p6x22RV +=  + hh27V(1,me2,m14,m34,m13,L,mu2) * ( 1./8. + 1./2.
         *RSe6613 + 1./4.*RSe63 + 1./4.*RSe61 );

      F13p6x22RV +=  + hh27V(1,me2,m16,m36,m13,L,mu2) * ( 1./4. + 1./4.
         *RSe6613 - 1./4.*RSe63 - 1./4.*RSe61 );

      F13p6x22RV +=  + hh27V(3,m13,m11,m11,m13,L,mu2) * (  - 1./12.*
         RD146e*RS1463e + 1./24.*RD146e*RC146e );

      F13p6x22RV +=  + hh27V(3,m13,m11,m33,m13,L,mu2) * (  - 1./12.*
         RD146e*RS3461e + 1./24.*RD146e*RC346e );

      F13p6x22RV +=  + hh27V(3,m13,m11,me2,m13,L,mu2) * ( 1./6.*RD146e*
         RSe6613 - 1./12.*RD146e*RSe6633 - 1./12.*RD146e*RSe6611 );

      F13p6x22RV +=  + hh27V(3,m13,m33,m33,m13,L,mu2) * (  - 1./12.*
         RD346e*RS3461e + 1./24.*RD346e*RC346e );

      F13p6x22RV +=  + hh27V(3,m13,m33,me2,m13,L,mu2) * ( 1./6.*RD346e*
         RSe6613 - 1./12.*RD346e*RSe6633 - 1./12.*RSe6611*RD346e );

      F13p6x22RV +=  + hh27V(3,m14,m11,m34,m13,L,mu2) * (  - 1./4.*
         RD146e );

      F13p6x22RV +=  + hh27V(3,m16,m11,m36,m13,L,mu2) * (  - 1./8.*
         RD146e );

      F13p6x22RV +=  + hh27V(4,m13,m11,m33,m13,L,mu2) * (  - 1./12.*
         RD346e*RS1463e + 1./24.*RC146e*RD346e );

      F13p6x22RV +=  + hh27V(4,m34,m14,m33,m13,L,mu2) * (  - 1./4.*
         RD346e );

      F13p6x22RV +=  + hh27V(4,m36,m16,m33,m13,L,mu2) * (  - 1./8.*
         RD346e );

      F13p6x22RV +=  + hh27V(7,m13,m11,m11,m13,L,mu2) * ( 1./48.*pow(
         RD146e,2) );

      F13p6x22RV +=  + hh27V(7,m13,m11,m33,m13,L,mu2) * ( 1./24.*RD146e
         *RD346e );

      F13p6x22RV +=  + hh27V(7,m13,m33,m33,m13,L,mu2) * ( 1./48.*pow(
         RD346e,2) );

      F13p6x22RV +=  + hhdV(1,m11,m11,m13,m13,L,mu2) * ( 1./648.*pow(
         RD146e,2) - 1./162.*m33*RD146e*RS1463e - 1./16.*pow(m33,2) - 7.
         /324.*pow(m33,2)*pow(RS1463e,2) + 1./36.*pow(m33,2)*RC146e*
         RS1463e - 1./144.*pow(m33,2)*pow(RC146e,2) - 1./81.*m11*RD146e
         *RS1463e - 3./16.*m11*m33 - 5./162.*m11*m33*pow(RS1463e,2) + 1.
         /18.*m11*m33*RC146e*RS1463e - 1./72.*m11*m33*pow(RC146e,2) - 1.
         /8.*pow(m11,2) - 1./324.*pow(m11,2)*pow(RS1463e,2) + 1./36.*
         pow(m11,2)*RC146e*RS1463e - 1./144.*pow(m11,2)*pow(RC146e,2) )
         ;

      F13p6x22RV +=  + hhdV(1,m11,m13,m33,m13,L,mu2) * ( 1./324.*RD146e
         *RD346e + 1./81.*m33*RD346e*RS1463e + 1./162.*m33*RD146e*
         RS3461e + 2./81.*pow(m33,2)*RS1463e*RS3461e + 1./162.*m11*
         RD346e*RS1463e + 1./81.*m11*RD146e*RS3461e + 5./81.*m11*m33*
         RS1463e*RS3461e + 2./81.*pow(m11,2)*RS1463e*RS3461e );

      F13p6x22RV +=  + hhdV(1,m11,m13,me2,m13,L,mu2) * ( 1./81.*me2*
         RD146e*RSe6613 - 1./162.*me2*RD146e*RSe6633 - 1./162.*me2*
         RD146e*RSe6611 - 1./81.*pow(me2,2)*RSe6613*RS1463e + 1./162.*
         pow(me2,2)*RSe6633*RS1463e + 1./162.*pow(me2,2)*RSe6611*
         RS1463e + 1./162.*pow(me2,2)*RC146e*RSe6613 - 1./324.*pow(
         me2,2)*RC146e*RSe6633 - 1./324.*pow(me2,2)*RC146e*RSe6611 - 2./
         81.*m33*RD146e*RSe6613 + 1./162.*m33*RD146e*RSe6633 + 1./54.*
         m33*RD146e*RSe6611 + 5./81.*m33*me2*RSe6613*RS1463e - 1./27.*
         m33*me2*RSe6633*RS1463e - 2./81.*m33*me2*RSe6611*RS1463e - 2./
         81.*m33*me2*RC146e*RSe6613 + 1./162.*m33*me2*RC146e*RSe6633 + 
         1./54.*m33*me2*RC146e*RSe6611 - 8./81.*pow(m33,2)*RSe6613*
         RS1463e + 5./162.*pow(m33,2)*RSe6633*RS1463e + 1./54.*pow(
         m33,2)*RSe6611*RS1463e + 1./54.*pow(m33,2)*RC146e*RSe6613 - 1./
         324.*pow(m33,2)*RC146e*RSe6633 - 1./36.*pow(m33,2)*RC146e*
         RSe6611 - 2./81.*m11*RD146e*RSe6613 + 2./81.*m11*RD146e*
         RSe6611 );

      F13p6x22RV +=  + hhdV(1,m11,m13,me2,m13,L,mu2) * ( 1./27.*m11*me2
         *RSe6613*RS1463e - 1./81.*m11*me2*RSe6633*RS1463e - 2./81.*m11
         *me2*RSe6611*RS1463e - 2./81.*m11*me2*RC146e*RSe6613 + 2./81.*
         m11*me2*RC146e*RSe6611 - 13./81.*m11*m33*RSe6613*RS1463e + 1./
         81.*m11*m33*RSe6633*RS1463e + 2./81.*m11*m33*RSe6611*RS1463e
          + 2./81.*m11*m33*RC146e*RSe6613 - 2./27.*m11*m33*RC146e*
         RSe6611 - 4./81.*pow(m11,2)*RSe6613*RS1463e - 4./81.*pow(
         m11,2)*RC146e*RSe6611 );

      F13p6x22RV +=  + hhdV(1,m11,m14,m34,m13,L,mu2) * (  - 1./12.*pow(
         m33,2)*RS1463e + 1./12.*pow(m33,2)*RS16e - 5./24.*m11*m33*
         RS1463e + 5./24.*m11*m33*RS16e - 1./8.*pow(m11,2)*RS1463e + 1./
         8.*pow(m11,2)*RS16e );

      F13p6x22RV +=  + hhdV(1,m11,m16,m36,m13,L,mu2) * (  - 1./24.*pow(
         m33,2)*RS1463e + 1./24.*pow(m33,2)*RS14e - 5./48.*m11*m33*
         RS1463e + 5./48.*m11*m33*RS14e - 1./16.*pow(m11,2)*RS1463e + 1.
         /16.*pow(m11,2)*RS14e );

      F13p6x22RV +=  + hhdV(1,m13,m11,m11,m13,L,mu2) * ( 1./64.*pow(
         m33,2) + 1./144.*pow(m33,2)*pow(RS1463e,2) - 1./144.*pow(
         m33,2)*RC146e*RS1463e + 1./576.*pow(m33,2)*pow(RC146e,2) + 1./
         32.*m11*m33 + 1./72.*m11*m33*pow(RS1463e,2) - 1./72.*m11*m33*
         RC146e*RS1463e + 1./288.*m11*m33*pow(RC146e,2) + 1./64.*pow(
         m11,2) + 1./144.*pow(m11,2)*pow(RS1463e,2) - 1./144.*pow(
         m11,2)*RC146e*RS1463e + 1./576.*pow(m11,2)*pow(RC146e,2) );

      F13p6x22RV +=  + hhdV(1,m13,m11,m33,m13,L,mu2) * ( 1./72.*pow(
         m33,2)*RS1463e*RS3461e - 1./144.*pow(m33,2)*RC346e*RS1463e - 1.
         /144.*pow(m33,2)*RC146e*RS3461e + 1./288.*pow(m33,2)*RC146e*
         RC346e + 1./36.*m11*m33*RS1463e*RS3461e - 1./72.*m11*m33*
         RC346e*RS1463e - 1./72.*m11*m33*RC146e*RS3461e + 1./144.*m11*
         m33*RC146e*RC346e + 1./72.*pow(m11,2)*RS1463e*RS3461e - 1./144.
         *pow(m11,2)*RC346e*RS1463e - 1./144.*pow(m11,2)*RC146e*RS3461e
          + 1./288.*pow(m11,2)*RC146e*RC346e );

      F13p6x22RV +=  + hhdV(1,m13,m11,me2,m13,L,mu2) * (  - 1./36.*pow(
         m33,2)*RSe6613*RS1463e + 1./72.*pow(m33,2)*RSe6633*RS1463e + 1.
         /72.*pow(m33,2)*RSe6611*RS1463e + 1./72.*pow(m33,2)*RC146e*
         RSe6613 - 1./144.*pow(m33,2)*RC146e*RSe6633 - 1./144.*pow(
         m33,2)*RC146e*RSe6611 - 1./18.*m11*m33*RSe6613*RS1463e + 1./36.
         *m11*m33*RSe6633*RS1463e + 1./36.*m11*m33*RSe6611*RS1463e + 1./
         36.*m11*m33*RC146e*RSe6613 - 1./72.*m11*m33*RC146e*RSe6633 - 1.
         /72.*m11*m33*RC146e*RSe6611 - 1./36.*pow(m11,2)*RSe6613*
         RS1463e + 1./72.*pow(m11,2)*RSe6633*RS1463e + 1./72.*pow(
         m11,2)*RSe6611*RS1463e + 1./72.*pow(m11,2)*RC146e*RSe6613 - 1./
         144.*pow(m11,2)*RC146e*RSe6633 - 1./144.*pow(m11,2)*RC146e*
         RSe6611 );

      F13p6x22RV +=  + hhdV(1,m13,m13,m13,m13,L,mu2) * ( 1./32.*pow(
         m33,2) + 1./16.*m11*m33 + 1./32.*pow(m11,2) );

      F13p6x22RV +=  + hhdV(1,m13,m33,m33,m13,L,mu2) * ( 1./648.*pow(
         RD346e,2) + 1./162.*m33*RD346e*RS3461e - 1./108.*m33*RD346e*
         RC346e + 5./64.*pow(m33,2) + 17./1296.*pow(m33,2)*pow(
         RS3461e,2) + 1./432.*pow(m33,2)*RC346e*RS3461e + 5./576.*pow(
         m33,2)*pow(RC346e,2) + 1./81.*m11*RD346e*RS3461e - 1./108.*m11
         *RD346e*RC346e + 5./32.*m11*m33 + 25./648.*m11*m33*pow(
         RS3461e,2) - 1./72.*m11*m33*RC346e*RS3461e + 5./288.*m11*m33*
         pow(RC346e,2) + 5./64.*pow(m11,2) + 41./1296.*pow(m11,2)*pow(
         RS3461e,2) - 7./432.*pow(m11,2)*RC346e*RS3461e + 5./576.*pow(
         m11,2)*pow(RC346e,2) );

      F13p6x22RV +=  + hhdV(1,m13,m33,me2,m13,L,mu2) * ( 1./81.*me2*
         RD346e*RSe6613 - 1./162.*me2*RD346e*RSe6633 - 1./162.*me2*
         RSe6611*RD346e - 1./81.*pow(me2,2)*RSe6613*RS3461e + 1./162.*
         pow(me2,2)*RSe6633*RS3461e + 1./162.*pow(me2,2)*RC346e*RSe6613
          - 1./324.*pow(me2,2)*RC346e*RSe6633 + 1./162.*pow(me2,2)*
         RSe6611*RS3461e - 1./324.*pow(me2,2)*RSe6611*RC346e - 2./81.*
         m33*RD346e*RSe6613 + 2./81.*m33*RD346e*RSe6633 + 1./27.*m33*
         me2*RSe6613*RS3461e - 2./81.*m33*me2*RSe6633*RS3461e - 2./81.*
         m33*me2*RC346e*RSe6613 + 2./81.*m33*me2*RC346e*RSe6633 - 1./81.
         *m33*me2*RSe6611*RS3461e - 25./324.*pow(m33,2)*RSe6613*RS3461e
          + 1./72.*pow(m33,2)*RSe6633*RS3461e + 1./72.*pow(m33,2)*
         RC346e*RSe6613 - 73./1296.*pow(m33,2)*RC346e*RSe6633 + 1./72.*
         pow(m33,2)*RSe6611*RS3461e - 1./144.*pow(m33,2)*RSe6611*RC346e
          - 2./81.*m11*RD346e*RSe6613 + 1./54.*m11*RD346e*RSe6633 + 1./
         162.*m11*RSe6611*RD346e + 5./81.*m11*me2*RSe6613*RS3461e - 2./
         81.*m11*me2*RSe6633*RS3461e );

      F13p6x22RV +=  + hhdV(1,m13,m33,me2,m13,L,mu2) * (  - 2./81.*m11*
         me2*RC346e*RSe6613 + 1./54.*m11*me2*RC346e*RSe6633 - 1./27.*
         m11*me2*RSe6611*RS3461e + 1./162.*m11*me2*RSe6611*RC346e - 35./
         162.*m11*m33*RSe6613*RS3461e + 17./324.*m11*m33*RSe6633*
         RS3461e + 17./324.*m11*m33*RC346e*RSe6613 - 19./216.*m11*m33*
         RC346e*RSe6633 + 13./324.*m11*m33*RSe6611*RS3461e - 1./72.*m11
         *m33*RSe6611*RC346e - 41./324.*pow(m11,2)*RSe6613*RS3461e + 7./
         216.*pow(m11,2)*RSe6633*RS3461e + 7./216.*pow(m11,2)*RC346e*
         RSe6613 - 5./144.*pow(m11,2)*RC346e*RSe6633 + 29./648.*pow(
         m11,2)*RSe6611*RS3461e - 13./1296.*pow(m11,2)*RSe6611*RC346e )
         ;

      F13p6x22RV +=  + hhdV(1,m13,me2,me2,m13,L,mu2) * ( 4./81.*pow(
         me2,2)*pow(RSe6613,2) - 4./81.*pow(me2,2)*RSe6633*RSe6613 + 1./
         81.*pow(me2,2)*pow(RSe6633,2) - 4./81.*pow(me2,2)*RSe6611*
         RSe6613 + 2./81.*pow(me2,2)*RSe6611*RSe6633 + 1./81.*pow(
         me2,2)*pow(RSe6611,2) - 10./81.*m33*me2*pow(RSe6613,2) + 4./27.
         *m33*me2*RSe6633*RSe6613 - 5./81.*m33*me2*pow(RSe6633,2) + 8./
         81.*m33*me2*RSe6611*RSe6613 - 2./81.*m33*me2*RSe6611*RSe6633
          - 1./27.*m33*me2*pow(RSe6611,2) + 41./324.*pow(m33,2)*pow(
         RSe6613,2) - 29./324.*pow(m33,2)*RSe6633*RSe6613 + 109./1296.*
         pow(m33,2)*pow(RSe6633,2) - 7./108.*pow(m33,2)*RSe6611*RSe6613
          + 13./648.*pow(m33,2)*RSe6611*RSe6633 + 5./144.*pow(m33,2)*
         pow(RSe6611,2) - 10./81.*m11*me2*pow(RSe6613,2) + 8./81.*m11*
         me2*RSe6633*RSe6613 - 1./27.*m11*me2*pow(RSe6633,2) + 4./27.*
         m11*me2*RSe6611*RSe6613 - 2./81.*m11*me2*RSe6611*RSe6633 - 5./
         81.*m11*me2*pow(RSe6611,2) + 5./18.*m11*m33*pow(RSe6613,2) - 
         25./162.*m11*m33*RSe6633*RSe6613 );

      F13p6x22RV +=  + hhdV(1,m13,me2,me2,m13,L,mu2) * ( 23./216.*m11*
         m33*pow(RSe6633,2) - 25./162.*m11*m33*RSe6611*RSe6613 + 13./
         324.*m11*m33*RSe6611*RSe6633 + 23./216.*m11*m33*pow(RSe6611,2)
          + 41./324.*pow(m11,2)*pow(RSe6613,2) - 7./108.*pow(m11,2)*
         RSe6633*RSe6613 + 5./144.*pow(m11,2)*pow(RSe6633,2) - 29./324.
         *pow(m11,2)*RSe6611*RSe6613 + 13./648.*pow(m11,2)*RSe6611*
         RSe6633 + 109./1296.*pow(m11,2)*pow(RSe6611,2) );

      F13p6x22RV +=  + hhdV(1,m14,m11,m34,m13,L,mu2) * ( 1./48.*m33*m44
         *RS1463e + 1./48.*m33*m44*RS16e - 1./48.*m33*m44*RC146e + 1./
         48.*m11*m44*RS1463e + 1./48.*m11*m44*RS16e - 1./48.*m11*m44*
         RC146e + 1./48.*m11*m33*RS1463e + 1./48.*m11*m33*RS16e - 1./48.
         *m11*m33*RC146e + 1./48.*pow(m11,2)*RS1463e + 1./48.*pow(
         m11,2)*RS16e - 1./48.*pow(m11,2)*RC146e );

      F13p6x22RV +=  + hhdV(1,m14,m33,m34,m13,L,mu2) * (  - 1./18.*m33*
         RD346e + 1./48.*m33*m44*RS3461e - 19./144.*m33*m44*RS36e - 5./
         36.*pow(m33,2)*RS3461e + 1./36.*pow(m33,2)*RS36e - 1./18.*m11*
         RD346e + 1./48.*m11*m44*RS3461e - 19./144.*m11*m44*RS36e - 7./
         48.*m11*m33*RS3461e - 11./144.*m11*m33*RS36e - 1./144.*pow(
         m11,2)*RS3461e - 5./48.*pow(m11,2)*RS36e );

      F13p6x22RV +=  + hhdV(1,m14,m34,me2,m13,L,mu2) * ( 1./54.*pow(
         me2,2) + 1./27.*pow(me2,2)*RSe6613 + 1./27.*pow(me2,2)*RSe63
          + 1./54.*pow(me2,2)*RSe6633 + 1./27.*pow(me2,2)*RSe61 + 1./54.
         *pow(me2,2)*RSe6611 - 1./27.*m44*me2 - 1./27.*m44*me2*RSe63 - 
         1./27.*m44*me2*RSe61 + 1./54.*pow(m44,2) - 1./18.*m33*me2 - 1./
         27.*m33*me2*RSe6613 - 5./54.*m33*me2*RSe63 - 1./27.*m33*me2*
         RSe6633 - 1./18.*m33*me2*RSe61 + 1./18.*m33*m44 - 1./24.*m33*
         m44*RSe6613 + 7./432.*m33*m44*RSe63 + 1./48.*m33*m44*RSe61 + 1.
         /24.*m33*m44*RSe6611 - 1./6.*pow(m33,2)*RSe6613 - 1./36.*pow(
         m33,2)*RSe63 + 1./54.*pow(m33,2)*RSe6633 - 1./12.*pow(m33,2)*
         RSe61 - 1./18.*m11*me2 - 1./27.*m11*me2*RSe6613 - 1./18.*m11*
         me2*RSe63 - 5./54.*m11*me2*RSe61 - 1./27.*m11*me2*RSe6611 + 1./
         18.*m11*m44 - 1./24.*m11*m44*RSe6613 - 1./48.*m11*m44*RSe63 + 
         25./432.*m11*m44*RSe61 + 1./24.*m11*m44*RSe6611 - 73./216.*m11
         *m33*RSe6613 - 19./144.*m11*m33*RSe63 - 13./144.*m11*m33*RSe61
          + 1./24.*m11*m33*RSe6611 );

      F13p6x22RV +=  + hhdV(1,m14,m34,me2,m13,L,mu2) * (  - 5./24.*pow(
         m11,2)*RSe6613 - 5./48.*pow(m11,2)*RSe63 - 1./144.*pow(m11,2)*
         RSe61 + 13./216.*pow(m11,2)*RSe6611 );

      F13p6x22RV +=  + hhdV(1,m16,m11,m36,m13,L,mu2) * ( 1./96.*m33*m66
         *RS1463e + 1./96.*m33*m66*RS14e - 1./96.*m33*m66*RC146e + 1./
         96.*m11*m66*RS1463e + 1./96.*m11*m66*RS14e - 1./96.*m11*m66*
         RC146e + 1./96.*m11*m33*RS1463e + 1./96.*m11*m33*RS14e - 1./96.
         *m11*m33*RC146e + 1./96.*pow(m11,2)*RS1463e + 1./96.*pow(
         m11,2)*RS14e - 1./96.*pow(m11,2)*RC146e );

      F13p6x22RV +=  + hhdV(1,m16,m33,m36,m13,L,mu2) * (  - 1./36.*m33*
         RD346e + 1./96.*m33*m66*RS3461e - 19./288.*m33*m66*RS34e - 5./
         72.*pow(m33,2)*RS3461e + 1./72.*pow(m33,2)*RS34e - 1./36.*m11*
         RD346e + 1./96.*m11*m66*RS3461e - 19./288.*m11*m66*RS34e - 7./
         96.*m11*m33*RS3461e - 11./288.*m11*m33*RS34e - 1./288.*pow(
         m11,2)*RS3461e - 5./96.*pow(m11,2)*RS34e );

      F13p6x22RV +=  + hhdV(1,m16,m36,me2,m13,L,mu2) * ( 1./27.*pow(
         me2,2) + 1./54.*pow(me2,2)*RSe6613 - 1./27.*pow(me2,2)*RSe63
          + 1./108.*pow(me2,2)*RSe6633 - 1./27.*pow(me2,2)*RSe61 + 1./
         108.*pow(me2,2)*RSe6611 - 2./27.*m66*me2 + 1./27.*m66*me2*
         RSe63 + 1./27.*m66*me2*RSe61 + 1./27.*pow(m66,2) - 1./9.*m33*
         me2 - 1./54.*m33*me2*RSe6613 + 5./54.*m33*me2*RSe63 - 1./54.*
         m33*me2*RSe6633 + 1./18.*m33*me2*RSe61 + 1./9.*m33*m66 - 1./48.
         *m33*m66*RSe6613 - 7./432.*m33*m66*RSe63 - 1./48.*m33*m66*
         RSe61 + 1./48.*m33*m66*RSe6611 - 1./12.*pow(m33,2)*RSe6613 + 1.
         /36.*pow(m33,2)*RSe63 + 1./108.*pow(m33,2)*RSe6633 + 1./12.*
         pow(m33,2)*RSe61 - 1./9.*m11*me2 - 1./54.*m11*me2*RSe6613 + 1./
         18.*m11*me2*RSe63 + 5./54.*m11*me2*RSe61 - 1./54.*m11*me2*
         RSe6611 + 1./9.*m11*m66 - 1./48.*m11*m66*RSe6613 + 1./48.*m11*
         m66*RSe63 - 25./432.*m11*m66*RSe61 + 1./48.*m11*m66*RSe6611 - 
         73./432.*m11*m33*RSe6613 + 19./144.*m11*m33*RSe63 + 13./144.*
         m11*m33*RSe61 );

      F13p6x22RV +=  + hhdV(1,m16,m36,me2,m13,L,mu2) * ( 1./48.*m11*m33
         *RSe6611 - 5./48.*pow(m11,2)*RSe6613 + 5./48.*pow(m11,2)*RSe63
          + 1./144.*pow(m11,2)*RSe61 + 13./432.*pow(m11,2)*RSe6611 );

      F13p6x22RV +=  + hhdV(1,m33,m13,m33,m13,L,mu2) * (  - 1./16.*pow(
         m33,2) - 1./16.*m11*m33 );

      F13p6x22RV +=  + hhdV(1,m33,m14,m34,m13,L,mu2) * (  - 1./24.*pow(
         m33,2)*RS3461e + 1./24.*pow(m33,2)*RS36e - 1./24.*m11*m33*
         RS3461e + 1./24.*m11*m33*RS36e );

      F13p6x22RV +=  + hhdV(1,m33,m16,m36,m13,L,mu2) * (  - 1./48.*pow(
         m33,2)*RS3461e + 1./48.*pow(m33,2)*RS34e - 1./48.*m11*m33*
         RS3461e + 1./48.*m11*m33*RS34e );

      F13p6x22RV +=  + hhdV(1,m34,m11,m14,m13,L,mu2) * ( 1./48.*m33*m44
         *RS1463e - 1./48.*m33*m44*RS16e + 1./48.*pow(m33,2)*RS1463e - 
         1./48.*pow(m33,2)*RS16e + 1./48.*m11*m44*RS1463e - 1./48.*m11*
         m44*RS16e + 1./48.*m11*m33*RS1463e - 1./48.*m11*m33*RS16e );

      F13p6x22RV +=  + hhdV(1,m34,m14,m33,m13,L,mu2) * ( 1./48.*m33*m44
         *RS3461e + 1./48.*m33*m44*RS36e - 1./48.*m33*m44*RC346e + 1./
         48.*pow(m33,2)*RS3461e + 1./48.*pow(m33,2)*RS36e - 1./48.*pow(
         m33,2)*RC346e + 1./48.*m11*m44*RS3461e + 1./48.*m11*m44*RS36e
          - 1./48.*m11*m44*RC346e + 1./48.*m11*m33*RS3461e + 1./48.*m11
         *m33*RS36e - 1./48.*m11*m33*RC346e );

      F13p6x22RV +=  + hhdV(1,m34,m14,me2,m13,L,mu2) * (  - 1./24.*m33*
         m44*RSe6613 + 1./48.*m33*m44*RSe63 + 1./24.*m33*m44*RSe6633 - 
         1./48.*m33*m44*RSe61 - 1./24.*pow(m33,2)*RSe6613 + 1./48.*pow(
         m33,2)*RSe63 + 1./24.*pow(m33,2)*RSe6633 - 1./48.*pow(m33,2)*
         RSe61 - 1./24.*m11*m44*RSe6613 + 1./48.*m11*m44*RSe63 + 1./24.
         *m11*m44*RSe6633 - 1./48.*m11*m44*RSe61 - 1./24.*m11*m33*
         RSe6613 + 1./48.*m11*m33*RSe63 + 1./24.*m11*m33*RSe6633 - 1./
         48.*m11*m33*RSe61 );

      F13p6x22RV +=  + hhdV(1,m36,m11,m16,m13,L,mu2) * ( 1./96.*m33*m66
         *RS1463e - 1./96.*m33*m66*RS14e + 1./96.*pow(m33,2)*RS1463e - 
         1./96.*pow(m33,2)*RS14e + 1./96.*m11*m66*RS1463e - 1./96.*m11*
         m66*RS14e + 1./96.*m11*m33*RS1463e - 1./96.*m11*m33*RS14e );

      F13p6x22RV +=  + hhdV(1,m36,m16,m33,m13,L,mu2) * ( 1./96.*m33*m66
         *RS3461e + 1./96.*m33*m66*RS34e - 1./96.*m33*m66*RC346e + 1./
         96.*pow(m33,2)*RS3461e + 1./96.*pow(m33,2)*RS34e - 1./96.*pow(
         m33,2)*RC346e + 1./96.*m11*m66*RS3461e + 1./96.*m11*m66*RS34e
          - 1./96.*m11*m66*RC346e + 1./96.*m11*m33*RS3461e + 1./96.*m11
         *m33*RS34e - 1./96.*m11*m33*RC346e );

      F13p6x22RV +=  + hhdV(1,m36,m16,me2,m13,L,mu2) * (  - 1./48.*m33*
         m66*RSe6613 - 1./48.*m33*m66*RSe63 + 1./48.*m33*m66*RSe6633 + 
         1./48.*m33*m66*RSe61 - 1./48.*pow(m33,2)*RSe6613 - 1./48.*pow(
         m33,2)*RSe63 + 1./48.*pow(m33,2)*RSe6633 + 1./48.*pow(m33,2)*
         RSe61 - 1./48.*m11*m66*RSe6613 - 1./48.*m11*m66*RSe63 + 1./48.
         *m11*m66*RSe6633 + 1./48.*m11*m66*RSe61 - 1./48.*m11*m33*
         RSe6613 - 1./48.*m11*m33*RSe63 + 1./48.*m11*m33*RSe6633 + 1./
         48.*m11*m33*RSe61 );

      F13p6x22RV +=  + hhdV(1,m44,m14,m34,m13,L,mu2) * ( 3./16.*m33*m44
          + 3./16.*m11*m44 );

      F13p6x22RV +=  + hhdV(1,m46,m14,m36,m13,L,mu2) * ( 1./16.*m33*m66
          + 1./16.*m33*m44 + 1./16.*m11*m66 + 1./16.*m11*m44 );

      F13p6x22RV +=  + hhdV(1,m46,m16,m34,m13,L,mu2) * ( 1./16.*m33*m66
          + 1./16.*m33*m44 + 1./16.*m11*m66 + 1./16.*m11*m44 );

      F13p6x22RV +=  + hhdV(1,me2,m14,m34,m13,L,mu2) * ( 1./48.*m33*me2
          + 1./12.*m33*me2*RSe6613 + 1./24.*m33*me2*RSe63 + 1./24.*m33*
         me2*RSe61 + 1./48.*m11*me2 + 1./12.*m11*me2*RSe6613 + 1./24.*
         m11*me2*RSe63 + 1./24.*m11*me2*RSe61 );

      F13p6x22RV +=  + hhdV(1,me2,m16,m36,m13,L,mu2) * ( 1./24.*m33*me2
          + 1./24.*m33*me2*RSe6613 - 1./24.*m33*me2*RSe63 - 1./24.*m33*
         me2*RSe61 + 1./24.*m11*me2 + 1./24.*m11*me2*RSe6613 - 1./24.*
         m11*me2*RSe63 - 1./24.*m11*me2*RSe61 );

      F13p6x22RV +=  + hhdV(2,m11,m11,m13,m13,L,mu2) * ( 1./36.*pow(
         m33,2)*RD146e*RS1463e - 1./72.*pow(m33,2)*RD146e*RC146e + 1./
         18.*m11*m33*RD146e*RS1463e - 1./36.*m11*m33*RD146e*RC146e + 1./
         36.*pow(m11,2)*RD146e*RS1463e - 1./72.*pow(m11,2)*RD146e*
         RC146e );

      F13p6x22RV +=  + hhdV(2,m11,m13,me2,m13,L,mu2) * ( 1./162.*pow(
         me2,2)*RD146e*RSe6613 - 1./324.*pow(me2,2)*RD146e*RSe6633 - 1./
         324.*pow(me2,2)*RD146e*RSe6611 - 2./81.*m33*me2*RD146e*RSe6613
          + 1./162.*m33*me2*RD146e*RSe6633 + 1./54.*m33*me2*RD146e*
         RSe6611 + 1./54.*pow(m33,2)*RD146e*RSe6613 - 1./324.*pow(
         m33,2)*RD146e*RSe6633 - 1./36.*pow(m33,2)*RD146e*RSe6611 - 2./
         81.*m11*me2*RD146e*RSe6613 + 2./81.*m11*me2*RD146e*RSe6611 + 2.
         /81.*m11*m33*RD146e*RSe6613 - 2./27.*m11*m33*RD146e*RSe6611 - 
         4./81.*pow(m11,2)*RD146e*RSe6611 );

      F13p6x22RV +=  + hhdV(3,m13,m11,m11,m13,L,mu2) * (  - 1./144.*
         pow(m33,2)*RD146e*RS1463e + 1./288.*pow(m33,2)*RD146e*RC146e
          - 1./72.*m11*m33*RD146e*RS1463e + 1./144.*m11*m33*RD146e*
         RC146e - 1./144.*pow(m11,2)*RD146e*RS1463e + 1./288.*pow(
         m11,2)*RD146e*RC146e );

      F13p6x22RV +=  + hhdV(3,m13,m11,m33,m13,L,mu2) * (  - 1./144.*
         pow(m33,2)*RD146e*RS3461e + 1./288.*pow(m33,2)*RD146e*RC346e
          - 1./72.*m11*m33*RD146e*RS3461e + 1./144.*m11*m33*RD146e*
         RC346e - 1./144.*pow(m11,2)*RD146e*RS3461e + 1./288.*pow(
         m11,2)*RD146e*RC346e );

      F13p6x22RV +=  + hhdV(3,m13,m11,me2,m13,L,mu2) * ( 1./72.*pow(
         m33,2)*RD146e*RSe6613 - 1./144.*pow(m33,2)*RD146e*RSe6633 - 1./
         144.*pow(m33,2)*RD146e*RSe6611 + 1./36.*m11*m33*RD146e*RSe6613
          - 1./72.*m11*m33*RD146e*RSe6633 - 1./72.*m11*m33*RD146e*
         RSe6611 + 1./72.*pow(m11,2)*RD146e*RSe6613 - 1./144.*pow(
         m11,2)*RD146e*RSe6633 - 1./144.*pow(m11,2)*RD146e*RSe6611 );

      F13p6x22RV +=  + hhdV(3,m13,m33,m33,m13,L,mu2) * (  - 23./2592.*
         m33*pow(RD346e,2) + 1./432.*pow(m33,2)*RD346e*RS3461e + 5./288.
         *pow(m33,2)*RD346e*RC346e - 23./2592.*m11*pow(RD346e,2) - 1./
         72.*m11*m33*RD346e*RS3461e + 5./144.*m11*m33*RD346e*RC346e - 7.
         /432.*pow(m11,2)*RD346e*RS3461e + 5./288.*pow(m11,2)*RD346e*
         RC346e );

      F13p6x22RV +=  + hhdV(3,m13,m33,me2,m13,L,mu2) * ( 1./162.*pow(
         me2,2)*RD346e*RSe6613 - 1./324.*pow(me2,2)*RD346e*RSe6633 - 1./
         324.*pow(me2,2)*RSe6611*RD346e - 2./81.*m33*me2*RD346e*RSe6613
          + 2./81.*m33*me2*RD346e*RSe6633 + 1./72.*pow(m33,2)*RD346e*
         RSe6613 - 73./1296.*pow(m33,2)*RD346e*RSe6633 - 1./144.*pow(
         m33,2)*RSe6611*RD346e - 2./81.*m11*me2*RD346e*RSe6613 + 1./54.
         *m11*me2*RD346e*RSe6633 + 1./162.*m11*me2*RSe6611*RD346e + 17./
         324.*m11*m33*RD346e*RSe6613 - 19./216.*m11*m33*RD346e*RSe6633
          - 1./72.*m11*m33*RSe6611*RD346e + 7./216.*pow(m11,2)*RD346e*
         RSe6613 - 5./144.*pow(m11,2)*RD346e*RSe6633 - 13./1296.*pow(
         m11,2)*RSe6611*RD346e );

      F13p6x22RV +=  + hhdV(3,m14,m11,m34,m13,L,mu2) * (  - 1./48.*m33*
         m44*RD146e - 1./48.*m11*m44*RD146e - 1./48.*m11*m33*RD146e - 1.
         /48.*pow(m11,2)*RD146e );

      F13p6x22RV +=  + hhdV(3,m16,m11,m36,m13,L,mu2) * (  - 1./96.*m33*
         m66*RD146e - 1./96.*m11*m66*RD146e - 1./96.*m11*m33*RD146e - 1.
         /96.*pow(m11,2)*RD146e );

      F13p6x22RV +=  + hhdV(4,m13,m11,m33,m13,L,mu2) * (  - 1./144.*
         pow(m33,2)*RD346e*RS1463e + 1./288.*pow(m33,2)*RC146e*RD346e
          - 1./72.*m11*m33*RD346e*RS1463e + 1./144.*m11*m33*RC146e*
         RD346e - 1./144.*pow(m11,2)*RD346e*RS1463e + 1./288.*pow(
         m11,2)*RC146e*RD346e );

      F13p6x22RV +=  + hhdV(4,m13,m33,m33,m13,L,mu2) * (  - 1./2592.*
         m33*pow(RD346e,2) - 1./2592.*m11*pow(RD346e,2) );

      F13p6x22RV +=  + hhdV(4,m34,m14,m33,m13,L,mu2) * (  - 1./48.*m33*
         m44*RD346e - 1./48.*pow(m33,2)*RD346e - 1./48.*m11*m44*RD346e
          - 1./48.*m11*m33*RD346e );

      F13p6x22RV +=  + hhdV(4,m36,m16,m33,m13,L,mu2) * (  - 1./96.*m33*
         m66*RD346e - 1./96.*pow(m33,2)*RD346e - 1./96.*m11*m66*RD346e
          - 1./96.*m11*m33*RD346e );

      F13p6x22RV +=  + hhdV(5,m11,m11,m13,m13,L,mu2) * (  - 1./144.*
         pow(m33,2)*pow(RD146e,2) - 1./72.*m11*m33*pow(RD146e,2) - 1./
         144.*pow(m11,2)*pow(RD146e,2) );

      F13p6x22RV +=  + hhdV(7,m13,m11,m11,m13,L,mu2) * ( 1./576.*pow(
         m33,2)*pow(RD146e,2) + 1./288.*m11*m33*pow(RD146e,2) + 1./576.
         *pow(m11,2)*pow(RD146e,2) );

      F13p6x22RV +=  + hhdV(7,m13,m11,m33,m13,L,mu2) * ( 1./288.*pow(
         m33,2)*RD146e*RD346e + 1./144.*m11*m33*RD146e*RD346e + 1./288.
         *pow(m11,2)*RD146e*RD346e );

      F13p6x22RV +=  + hhdV(7,m13,m33,m33,m13,L,mu2) * ( 5./576.*pow(
         m33,2)*pow(RD346e,2) + 5./288.*m11*m33*pow(RD346e,2) + 5./576.
         *pow(m11,2)*pow(RD346e,2) );

      F13p6x22RV +=  + hh1dV(1,m11,m11,m13,m13,L,mu2) * ( 1./27.*m33*
         RD146e*RS1463e - 1./54.*m33*RD146e*RC146e + 1./4.*pow(m33,2)
          + 5./54.*pow(m33,2)*pow(RS1463e,2) - 2./27.*pow(m33,2)*RC146e
         *RS1463e + 1./36.*pow(m33,2)*pow(RC146e,2) + 1./27.*m11*RD146e
         *RS1463e - 1./54.*m11*RD146e*RC146e + 1./2.*m11*m33 + 1./9.*
         m11*m33*pow(RS1463e,2) - 1./9.*m11*m33*RC146e*RS1463e + 1./18.
         *m11*m33*pow(RC146e,2) + 1./4.*pow(m11,2) + 1./54.*pow(m11,2)*
         pow(RS1463e,2) - 1./27.*pow(m11,2)*RC146e*RS1463e + 1./36.*
         pow(m11,2)*pow(RC146e,2) );

      F13p6x22RV +=  + hh1dV(1,m11,m14,m34,m13,L,mu2) * ( 1./6.*pow(
         m33,2)*RS1463e - 1./6.*pow(m33,2)*RS16e + 1./3.*m11*m33*
         RS1463e - 1./3.*m11*m33*RS16e + 1./6.*pow(m11,2)*RS1463e - 1./
         6.*pow(m11,2)*RS16e );

      F13p6x22RV +=  + hh1dV(1,m11,m16,m36,m13,L,mu2) * ( 1./12.*pow(
         m33,2)*RS1463e - 1./12.*pow(m33,2)*RS14e + 1./6.*m11*m33*
         RS1463e - 1./6.*m11*m33*RS14e + 1./12.*pow(m11,2)*RS1463e - 1./
         12.*pow(m11,2)*RS14e );

      F13p6x22RV +=  + hh1dV(1,m13,m11,m33,m13,L,mu2) * (  - 1./54.*m33
         *RD346e*RS1463e + 1./108.*m33*RC146e*RD346e - 1./54.*m33*
         RD146e*RS3461e + 1./108.*m33*RD146e*RC346e - 1./18.*pow(m33,2)
         *RS1463e*RS3461e + 1./27.*pow(m33,2)*RC346e*RS1463e + 1./54.*
         pow(m33,2)*RC146e*RS3461e - 1./54.*m11*RD346e*RS1463e + 1./108.
         *m11*RC146e*RD346e - 1./54.*m11*RD146e*RS3461e + 1./108.*m11*
         RD146e*RC346e - 1./9.*m11*m33*RS1463e*RS3461e + 1./18.*m11*m33
         *RC346e*RS1463e + 1./18.*m11*m33*RC146e*RS3461e - 1./18.*pow(
         m11,2)*RS1463e*RS3461e + 1./54.*pow(m11,2)*RC346e*RS1463e + 1./
         27.*pow(m11,2)*RC146e*RS3461e );

      F13p6x22RV +=  + hh1dV(1,m13,m11,me2,m13,L,mu2) * ( 1./27.*m33*
         RD146e*RSe6613 - 1./54.*m33*RD146e*RSe6633 - 1./54.*m33*RD146e
         *RSe6611 - 2./27.*m33*me2*RSe6613*RS1463e + 1./27.*m33*me2*
         RSe6633*RS1463e + 1./27.*m33*me2*RSe6611*RS1463e + 1./27.*m33*
         me2*RC146e*RSe6613 - 1./54.*m33*me2*RC146e*RSe6633 - 1./54.*
         m33*me2*RC146e*RSe6611 + 5./27.*pow(m33,2)*RSe6613*RS1463e - 1.
         /9.*pow(m33,2)*RSe6633*RS1463e - 2./27.*pow(m33,2)*RSe6611*
         RS1463e - 2./27.*pow(m33,2)*RC146e*RSe6613 + 1./54.*pow(m33,2)
         *RC146e*RSe6633 + 1./18.*pow(m33,2)*RC146e*RSe6611 + 1./27.*
         m11*RD146e*RSe6613 - 1./54.*m11*RD146e*RSe6633 - 1./54.*m11*
         RD146e*RSe6611 - 2./27.*m11*me2*RSe6613*RS1463e + 1./27.*m11*
         me2*RSe6633*RS1463e + 1./27.*m11*me2*RSe6611*RS1463e + 1./27.*
         m11*me2*RC146e*RSe6613 - 1./54.*m11*me2*RC146e*RSe6633 - 1./54.
         *m11*me2*RC146e*RSe6611 + 8./27.*m11*m33*RSe6613*RS1463e - 4./
         27.*m11*m33*RSe6633*RS1463e - 4./27.*m11*m33*RSe6611*RS1463e
          - 4./27.*m11*m33*RC146e*RSe6613 );

      F13p6x22RV +=  + hh1dV(1,m13,m11,me2,m13,L,mu2) * ( 1./54.*m11*
         m33*RC146e*RSe6633 + 7./54.*m11*m33*RC146e*RSe6611 + 1./9.*
         pow(m11,2)*RSe6613*RS1463e - 1./27.*pow(m11,2)*RSe6633*RS1463e
          - 2./27.*pow(m11,2)*RSe6611*RS1463e - 2./27.*pow(m11,2)*
         RC146e*RSe6613 + 2./27.*pow(m11,2)*RC146e*RSe6611 );

      F13p6x22RV +=  + hh1dV(1,m13,m33,m33,m13,L,mu2) * (  - 1./54.*m33
         *RD346e*RS3461e + 1./108.*m33*RD346e*RC346e - 1./8.*pow(m33,2)
          - 1./108.*pow(m33,2)*pow(RS3461e,2) + 1./54.*pow(m33,2)*
         RC346e*RS3461e - 1./72.*pow(m33,2)*pow(RC346e,2) - 1./54.*m11*
         RD346e*RS3461e + 1./108.*m11*RD346e*RC346e - 1./4.*m11*m33 - 1.
         /18.*m11*m33*pow(RS3461e,2) + 1./18.*m11*m33*RC346e*RS3461e - 
         1./36.*m11*m33*pow(RC346e,2) - 1./8.*pow(m11,2) - 5./108.*pow(
         m11,2)*pow(RS3461e,2) + 1./27.*pow(m11,2)*RC346e*RS3461e - 1./
         72.*pow(m11,2)*pow(RC346e,2) );

      F13p6x22RV +=  + hh1dV(1,m13,m33,me2,m13,L,mu2) * ( 1./27.*m33*
         RD346e*RSe6613 - 1./54.*m33*RD346e*RSe6633 - 1./54.*m33*
         RSe6611*RD346e - 2./27.*m33*me2*RSe6613*RS3461e + 1./27.*m33*
         me2*RSe6633*RS3461e + 1./27.*m33*me2*RC346e*RSe6613 - 1./54.*
         m33*me2*RC346e*RSe6633 + 1./27.*m33*me2*RSe6611*RS3461e - 1./
         54.*m33*me2*RSe6611*RC346e + 1./9.*pow(m33,2)*RSe6613*RS3461e
          - 2./27.*pow(m33,2)*RSe6633*RS3461e - 2./27.*pow(m33,2)*
         RC346e*RSe6613 + 2./27.*pow(m33,2)*RC346e*RSe6633 - 1./27.*
         pow(m33,2)*RSe6611*RS3461e + 1./27.*m11*RD346e*RSe6613 - 1./54.
         *m11*RD346e*RSe6633 - 1./54.*m11*RSe6611*RD346e - 2./27.*m11*
         me2*RSe6613*RS3461e + 1./27.*m11*me2*RSe6633*RS3461e + 1./27.*
         m11*me2*RC346e*RSe6613 - 1./54.*m11*me2*RC346e*RSe6633 + 1./27.
         *m11*me2*RSe6611*RS3461e - 1./54.*m11*me2*RSe6611*RC346e + 8./
         27.*m11*m33*RSe6613*RS3461e - 4./27.*m11*m33*RSe6633*RS3461e
          - 4./27.*m11*m33*RC346e*RSe6613 + 7./54.*m11*m33*RC346e*
         RSe6633 );

      F13p6x22RV +=  + hh1dV(1,m13,m33,me2,m13,L,mu2) * (  - 4./27.*m11
         *m33*RSe6611*RS3461e + 1./54.*m11*m33*RSe6611*RC346e + 5./27.*
         pow(m11,2)*RSe6613*RS3461e - 2./27.*pow(m11,2)*RSe6633*RS3461e
          - 2./27.*pow(m11,2)*RC346e*RSe6613 + 1./18.*pow(m11,2)*RC346e
         *RSe6633 - 1./9.*pow(m11,2)*RSe6611*RS3461e + 1./54.*pow(
         m11,2)*RSe6611*RC346e );

      F13p6x22RV +=  + hh1dV(1,m13,me2,me2,m13,L,mu2) * ( 4./27.*m33*
         me2*pow(RSe6613,2) - 4./27.*m33*me2*RSe6633*RSe6613 + 1./27.*
         m33*me2*pow(RSe6633,2) - 4./27.*m33*me2*RSe6611*RSe6613 + 2./
         27.*m33*me2*RSe6611*RSe6633 + 1./27.*m33*me2*pow(RSe6611,2) - 
         5./27.*pow(m33,2)*pow(RSe6613,2) + 2./9.*pow(m33,2)*RSe6633*
         RSe6613 - 5./54.*pow(m33,2)*pow(RSe6633,2) + 4./27.*pow(m33,2)
         *RSe6611*RSe6613 - 1./27.*pow(m33,2)*RSe6611*RSe6633 - 1./18.*
         pow(m33,2)*pow(RSe6611,2) + 4./27.*m11*me2*pow(RSe6613,2) - 4./
         27.*m11*me2*RSe6633*RSe6613 + 1./27.*m11*me2*pow(RSe6633,2) - 
         4./27.*m11*me2*RSe6611*RSe6613 + 2./27.*m11*me2*RSe6611*
         RSe6633 + 1./27.*m11*me2*pow(RSe6611,2) - 10./27.*m11*m33*pow(
         RSe6613,2) + 10./27.*m11*m33*RSe6633*RSe6613 - 4./27.*m11*m33*
         pow(RSe6633,2) + 10./27.*m11*m33*RSe6611*RSe6613 - 2./27.*m11*
         m33*RSe6611*RSe6633 - 4./27.*m11*m33*pow(RSe6611,2) - 5./27.*
         pow(m11,2)*pow(RSe6613,2) + 4./27.*pow(m11,2)*RSe6633*RSe6613
          - 1./18.*pow(m11,2)*pow(RSe6633,2) );

      F13p6x22RV +=  + hh1dV(1,m13,me2,me2,m13,L,mu2) * ( 2./9.*pow(
         m11,2)*RSe6611*RSe6613 - 1./27.*pow(m11,2)*RSe6611*RSe6633 - 5.
         /54.*pow(m11,2)*pow(RSe6611,2) );

      F13p6x22RV +=  + hh1dV(1,m14,m11,m34,m13,L,mu2) * (  - 1./18.*m33
         *RD146e - 1./9.*m33*m44*RS16e + 1./18.*pow(m33,2)*RS1463e - 1./
         6.*pow(m33,2)*RS16e - 1./18.*m11*RD146e - 1./9.*m11*m44*RS16e
          - 2./9.*m11*m33*RS16e - 1./18.*pow(m11,2)*RS1463e - 1./18.*
         pow(m11,2)*RS16e );

      F13p6x22RV +=  + hh1dV(1,m14,m33,m34,m13,L,mu2) * ( 1./18.*m33*
         RD346e + 1./9.*m33*m44*RS36e + 1./18.*pow(m33,2)*RS3461e + 1./
         18.*pow(m33,2)*RS36e + 1./18.*m11*RD346e + 1./9.*m11*m44*RS36e
          + 2./9.*m11*m33*RS36e - 1./18.*pow(m11,2)*RS3461e + 1./6.*
         pow(m11,2)*RS36e );

      F13p6x22RV +=  + hh1dV(1,m14,m34,me2,m13,L,mu2) * ( 1./18.*m33*
         me2 + 1./9.*m33*me2*RSe6613 + 1./18.*m33*me2*RSe63 + 1./6.*m33
         *me2*RSe61 + 1./9.*m33*me2*RSe6611 - 1./18.*m33*m44 - 1./9.*
         m33*m44*RSe61 + 2./9.*pow(m33,2)*RSe6613 + 1./9.*pow(m33,2)*
         RSe63 + 1./18.*m11*me2 + 1./9.*m11*me2*RSe6613 + 1./18.*m11*
         me2*RSe63 + 1./6.*m11*me2*RSe61 + 1./9.*m11*me2*RSe6611 - 1./
         18.*m11*m44 - 1./9.*m11*m44*RSe61 + 5./9.*m11*m33*RSe6613 + 5./
         18.*m11*m33*RSe63 - 1./18.*m11*m33*RSe61 - 1./9.*m11*m33*
         RSe6611 + 1./3.*pow(m11,2)*RSe6613 + 1./6.*pow(m11,2)*RSe63 - 
         1./18.*pow(m11,2)*RSe61 - 1./9.*pow(m11,2)*RSe6611 );

      F13p6x22RV +=  + hh1dV(1,m16,m11,m36,m13,L,mu2) * (  - 1./36.*m33
         *RD146e - 1./18.*m33*m66*RS14e + 1./36.*pow(m33,2)*RS1463e - 1.
         /12.*pow(m33,2)*RS14e - 1./36.*m11*RD146e - 1./18.*m11*m66*
         RS14e - 1./9.*m11*m33*RS14e - 1./36.*pow(m11,2)*RS1463e - 1./
         36.*pow(m11,2)*RS14e );

      F13p6x22RV +=  + hh1dV(1,m16,m33,m36,m13,L,mu2) * ( 1./36.*m33*
         RD346e + 1./18.*m33*m66*RS34e + 1./36.*pow(m33,2)*RS3461e + 1./
         36.*pow(m33,2)*RS34e + 1./36.*m11*RD346e + 1./18.*m11*m66*
         RS34e + 1./9.*m11*m33*RS34e - 1./36.*pow(m11,2)*RS3461e + 1./
         12.*pow(m11,2)*RS34e );

      F13p6x22RV +=  + hh1dV(1,m16,m36,me2,m13,L,mu2) * ( 1./9.*m33*me2
          + 1./18.*m33*me2*RSe6613 - 1./18.*m33*me2*RSe63 - 1./6.*m33*
         me2*RSe61 + 1./18.*m33*me2*RSe6611 - 1./9.*m33*m66 + 1./9.*m33
         *m66*RSe61 + 1./9.*pow(m33,2)*RSe6613 - 1./9.*pow(m33,2)*RSe63
          + 1./9.*m11*me2 + 1./18.*m11*me2*RSe6613 - 1./18.*m11*me2*
         RSe63 - 1./6.*m11*me2*RSe61 + 1./18.*m11*me2*RSe6611 - 1./9.*
         m11*m66 + 1./9.*m11*m66*RSe61 + 5./18.*m11*m33*RSe6613 - 5./18.
         *m11*m33*RSe63 + 1./18.*m11*m33*RSe61 - 1./18.*m11*m33*RSe6611
          + 1./6.*pow(m11,2)*RSe6613 - 1./6.*pow(m11,2)*RSe63 + 1./18.*
         pow(m11,2)*RSe61 - 1./18.*pow(m11,2)*RSe6611 );

      F13p6x22RV +=  + hh1dV(1,m33,m14,m34,m13,L,mu2) * ( 1./18.*m33*
         RD346e + 1./9.*m33*m44*RS36e + 2./9.*pow(m33,2)*RS3461e - 1./9.
         *pow(m33,2)*RS36e + 1./18.*m11*RD346e + 1./9.*m11*m44*RS36e + 
         1./3.*m11*m33*RS3461e - 1./9.*m11*m33*RS36e + 1./9.*pow(m11,2)
         *RS3461e );

      F13p6x22RV +=  + hh1dV(1,m33,m16,m36,m13,L,mu2) * ( 1./36.*m33*
         RD346e + 1./18.*m33*m66*RS34e + 1./9.*pow(m33,2)*RS3461e - 1./
         18.*pow(m33,2)*RS34e + 1./36.*m11*RD346e + 1./18.*m11*m66*
         RS34e + 1./6.*m11*m33*RS3461e - 1./18.*m11*m33*RS34e + 1./18.*
         pow(m11,2)*RS3461e );

      F13p6x22RV +=  + hh1dV(1,m34,m14,me2,m13,L,mu2) * ( 1./18.*m33*
         me2 + 1./9.*m33*me2*RSe6613 + 1./6.*m33*me2*RSe63 + 1./9.*m33*
         me2*RSe6633 + 1./18.*m33*me2*RSe61 - 1./18.*m33*m44 - 1./9.*
         m33*m44*RSe63 + 1./3.*pow(m33,2)*RSe6613 - 1./18.*pow(m33,2)*
         RSe63 - 1./9.*pow(m33,2)*RSe6633 + 1./6.*pow(m33,2)*RSe61 + 1./
         18.*m11*me2 + 1./9.*m11*me2*RSe6613 + 1./6.*m11*me2*RSe63 + 1./
         9.*m11*me2*RSe6633 + 1./18.*m11*me2*RSe61 - 1./18.*m11*m44 - 1.
         /9.*m11*m44*RSe63 + 5./9.*m11*m33*RSe6613 - 1./18.*m11*m33*
         RSe63 - 1./9.*m11*m33*RSe6633 + 5./18.*m11*m33*RSe61 + 2./9.*
         pow(m11,2)*RSe6613 + 1./9.*pow(m11,2)*RSe61 );

      F13p6x22RV +=  + hh1dV(1,m36,m16,me2,m13,L,mu2) * ( 1./9.*m33*me2
          + 1./18.*m33*me2*RSe6613 - 1./6.*m33*me2*RSe63 + 1./18.*m33*
         me2*RSe6633 - 1./18.*m33*me2*RSe61 - 1./9.*m33*m66 + 1./9.*m33
         *m66*RSe63 + 1./6.*pow(m33,2)*RSe6613 + 1./18.*pow(m33,2)*
         RSe63 - 1./18.*pow(m33,2)*RSe6633 - 1./6.*pow(m33,2)*RSe61 + 1.
         /9.*m11*me2 + 1./18.*m11*me2*RSe6613 - 1./6.*m11*me2*RSe63 + 1.
         /18.*m11*me2*RSe6633 - 1./18.*m11*me2*RSe61 - 1./9.*m11*m66 + 
         1./9.*m11*m66*RSe63 + 5./18.*m11*m33*RSe6613 + 1./18.*m11*m33*
         RSe63 - 1./18.*m11*m33*RSe6633 - 5./18.*m11*m33*RSe61 + 1./9.*
         pow(m11,2)*RSe6613 - 1./9.*pow(m11,2)*RSe61 );

      F13p6x22RV +=  + hh1dV(2,m11,m11,m13,m13,L,mu2) * (  - 1./108.*
         m33*pow(RD146e,2) - 1./27.*pow(m33,2)*RD146e*RS1463e + 1./36.*
         pow(m33,2)*RD146e*RC146e - 1./108.*m11*pow(RD146e,2) - 1./18.*
         m11*m33*RD146e*RS1463e + 1./18.*m11*m33*RD146e*RC146e - 1./54.
         *pow(m11,2)*RD146e*RS1463e + 1./36.*pow(m11,2)*RD146e*RC146e )
         ;

      F13p6x22RV +=  + hh1dV(3,m11,m11,m13,m13,L,mu2) * (  - 1./108.*
         m33*pow(RD146e,2) - 1./27.*pow(m33,2)*RD146e*RS1463e + 1./36.*
         pow(m33,2)*RD146e*RC146e - 1./108.*m11*pow(RD146e,2) - 1./18.*
         m11*m33*RD146e*RS1463e + 1./18.*m11*m33*RD146e*RC146e - 1./54.
         *pow(m11,2)*RD146e*RS1463e + 1./36.*pow(m11,2)*RD146e*RC146e )
         ;

      F13p6x22RV +=  + hh1dV(3,m13,m11,m33,m13,L,mu2) * ( 1./108.*m33*
         RD146e*RD346e + 1./54.*pow(m33,2)*RD146e*RS3461e + 1./108.*m11
         *RD146e*RD346e + 1./18.*m11*m33*RD146e*RS3461e + 1./27.*pow(
         m11,2)*RD146e*RS3461e );

      F13p6x22RV +=  + hh1dV(3,m13,m11,me2,m13,L,mu2) * ( 1./27.*m33*
         me2*RD146e*RSe6613 - 1./54.*m33*me2*RD146e*RSe6633 - 1./54.*
         m33*me2*RD146e*RSe6611 - 2./27.*pow(m33,2)*RD146e*RSe6613 + 1./
         54.*pow(m33,2)*RD146e*RSe6633 + 1./18.*pow(m33,2)*RD146e*
         RSe6611 + 1./27.*m11*me2*RD146e*RSe6613 - 1./54.*m11*me2*
         RD146e*RSe6633 - 1./54.*m11*me2*RD146e*RSe6611 - 4./27.*m11*
         m33*RD146e*RSe6613 + 1./54.*m11*m33*RD146e*RSe6633 + 7./54.*
         m11*m33*RD146e*RSe6611 - 2./27.*pow(m11,2)*RD146e*RSe6613 + 2./
         27.*pow(m11,2)*RD146e*RSe6611 );

      F13p6x22RV +=  + hh1dV(3,m13,m33,m33,m13,L,mu2) * ( 23./2592.*m33
         *pow(RD346e,2) + 1./54.*pow(m33,2)*RD346e*RS3461e - 1./36.*
         pow(m33,2)*RD346e*RC346e + 23./2592.*m11*pow(RD346e,2) + 1./18.
         *m11*m33*RD346e*RS3461e - 1./18.*m11*m33*RD346e*RC346e + 1./27.
         *pow(m11,2)*RD346e*RS3461e - 1./36.*pow(m11,2)*RD346e*RC346e )
         ;

      F13p6x22RV +=  + hh1dV(3,m13,m33,me2,m13,L,mu2) * ( 1./27.*m33*
         me2*RD346e*RSe6613 - 1./54.*m33*me2*RD346e*RSe6633 - 1./54.*
         m33*me2*RSe6611*RD346e - 2./27.*pow(m33,2)*RD346e*RSe6613 + 2./
         27.*pow(m33,2)*RD346e*RSe6633 + 1./27.*m11*me2*RD346e*RSe6613
          - 1./54.*m11*me2*RD346e*RSe6633 - 1./54.*m11*me2*RSe6611*
         RD346e - 4./27.*m11*m33*RD346e*RSe6613 + 7./54.*m11*m33*RD346e
         *RSe6633 + 1./54.*m11*m33*RSe6611*RD346e - 2./27.*pow(m11,2)*
         RD346e*RSe6613 + 1./18.*pow(m11,2)*RD346e*RSe6633 + 1./54.*
         pow(m11,2)*RSe6611*RD346e );

      F13p6x22RV +=  + hh1dV(4,m13,m11,m33,m13,L,mu2) * ( 1./108.*m33*
         RD146e*RD346e + 1./27.*pow(m33,2)*RD346e*RS1463e + 1./108.*m11
         *RD146e*RD346e + 1./18.*m11*m33*RD346e*RS1463e + 1./54.*pow(
         m11,2)*RD346e*RS1463e );

      F13p6x22RV +=  + hh1dV(4,m13,m33,m33,m13,L,mu2) * ( 1./2592.*m33*
         pow(RD346e,2) + 1./2592.*m11*pow(RD346e,2) );

      F13p6x22RV +=  + hh1dV(5,m11,m11,m13,m13,L,mu2) * ( 1./36.*pow(
         m33,2)*pow(RD146e,2) + 1./18.*m11*m33*pow(RD146e,2) + 1./36.*
         pow(m11,2)*pow(RD146e,2) );

      F13p6x22RV +=  + hh1dV(7,m13,m33,m33,m13,L,mu2) * (  - 1./72.*
         pow(m33,2)*pow(RD346e,2) - 1./36.*m11*m33*pow(RD346e,2) - 1./
         72.*pow(m11,2)*pow(RD346e,2) );

      F13p6x22RV +=  + hh21dV(1,m11,m11,m13,m13,L,mu2) * (  - 3./32.*
         pow(m33,2) - 3./16.*m11*m33 - 3./32.*pow(m11,2) );

      F13p6x22RV +=  + hh21dV(1,m11,m14,m34,m13,L,mu2) * (  - 1./16.*
         pow(m33,2)*RS1463e + 1./16.*pow(m33,2)*RS16e - 1./8.*m11*m33*
         RS1463e + 1./8.*m11*m33*RS16e - 1./16.*pow(m11,2)*RS1463e + 1./
         16.*pow(m11,2)*RS16e );

      F13p6x22RV +=  + hh21dV(1,m11,m16,m36,m13,L,mu2) * (  - 1./32.*
         pow(m33,2)*RS1463e + 1./32.*pow(m33,2)*RS14e - 1./16.*m11*m33*
         RS1463e + 1./16.*m11*m33*RS14e - 1./32.*pow(m11,2)*RS1463e + 1.
         /32.*pow(m11,2)*RS14e );

      F13p6x22RV +=  + hh21dV(1,m13,m11,m11,m13,L,mu2) * ( 3./64.*pow(
         m33,2) + 1./48.*pow(m33,2)*pow(RS1463e,2) - 1./48.*pow(m33,2)*
         RC146e*RS1463e + 1./192.*pow(m33,2)*pow(RC146e,2) + 3./32.*m11
         *m33 + 1./24.*m11*m33*pow(RS1463e,2) - 1./24.*m11*m33*RC146e*
         RS1463e + 1./96.*m11*m33*pow(RC146e,2) + 3./64.*pow(m11,2) + 1.
         /48.*pow(m11,2)*pow(RS1463e,2) - 1./48.*pow(m11,2)*RC146e*
         RS1463e + 1./192.*pow(m11,2)*pow(RC146e,2) );

      F13p6x22RV +=  + hh21dV(1,m13,m11,m33,m13,L,mu2) * ( 1./24.*pow(
         m33,2)*RS1463e*RS3461e - 1./48.*pow(m33,2)*RC346e*RS1463e - 1./
         48.*pow(m33,2)*RC146e*RS3461e + 1./96.*pow(m33,2)*RC146e*
         RC346e + 1./12.*m11*m33*RS1463e*RS3461e - 1./24.*m11*m33*
         RC346e*RS1463e - 1./24.*m11*m33*RC146e*RS3461e + 1./48.*m11*
         m33*RC146e*RC346e + 1./24.*pow(m11,2)*RS1463e*RS3461e - 1./48.
         *pow(m11,2)*RC346e*RS1463e - 1./48.*pow(m11,2)*RC146e*RS3461e
          + 1./96.*pow(m11,2)*RC146e*RC346e );

      F13p6x22RV +=  + hh21dV(1,m13,m11,me2,m13,L,mu2) * (  - 1./12.*
         pow(m33,2)*RSe6613*RS1463e + 1./24.*pow(m33,2)*RSe6633*RS1463e
          + 1./24.*pow(m33,2)*RSe6611*RS1463e + 1./24.*pow(m33,2)*
         RC146e*RSe6613 - 1./48.*pow(m33,2)*RC146e*RSe6633 - 1./48.*
         pow(m33,2)*RC146e*RSe6611 - 1./6.*m11*m33*RSe6613*RS1463e + 1./
         12.*m11*m33*RSe6633*RS1463e + 1./12.*m11*m33*RSe6611*RS1463e
          + 1./12.*m11*m33*RC146e*RSe6613 - 1./24.*m11*m33*RC146e*
         RSe6633 - 1./24.*m11*m33*RC146e*RSe6611 - 1./12.*pow(m11,2)*
         RSe6613*RS1463e + 1./24.*pow(m11,2)*RSe6633*RS1463e + 1./24.*
         pow(m11,2)*RSe6611*RS1463e + 1./24.*pow(m11,2)*RC146e*RSe6613
          - 1./48.*pow(m11,2)*RC146e*RSe6633 - 1./48.*pow(m11,2)*RC146e
         *RSe6611 );

      F13p6x22RV +=  + hh21dV(1,m13,m13,m13,m13,L,mu2) * ( 3./32.*pow(
         m33,2) + 3./16.*m11*m33 + 3./32.*pow(m11,2) );

      F13p6x22RV +=  + hh21dV(1,m13,m33,m33,m13,L,mu2) * ( 3./64.*pow(
         m33,2) + 1./48.*pow(m33,2)*pow(RS3461e,2) - 1./48.*pow(m33,2)*
         RC346e*RS3461e + 1./192.*pow(m33,2)*pow(RC346e,2) + 3./32.*m11
         *m33 + 1./24.*m11*m33*pow(RS3461e,2) - 1./24.*m11*m33*RC346e*
         RS3461e + 1./96.*m11*m33*pow(RC346e,2) + 3./64.*pow(m11,2) + 1.
         /48.*pow(m11,2)*pow(RS3461e,2) - 1./48.*pow(m11,2)*RC346e*
         RS3461e + 1./192.*pow(m11,2)*pow(RC346e,2) );

      F13p6x22RV +=  + hh21dV(1,m13,m33,me2,m13,L,mu2) * (  - 1./12.*
         pow(m33,2)*RSe6613*RS3461e + 1./24.*pow(m33,2)*RSe6633*RS3461e
          + 1./24.*pow(m33,2)*RC346e*RSe6613 - 1./48.*pow(m33,2)*RC346e
         *RSe6633 + 1./24.*pow(m33,2)*RSe6611*RS3461e - 1./48.*pow(
         m33,2)*RSe6611*RC346e - 1./6.*m11*m33*RSe6613*RS3461e + 1./12.
         *m11*m33*RSe6633*RS3461e + 1./12.*m11*m33*RC346e*RSe6613 - 1./
         24.*m11*m33*RC346e*RSe6633 + 1./12.*m11*m33*RSe6611*RS3461e - 
         1./24.*m11*m33*RSe6611*RC346e - 1./12.*pow(m11,2)*RSe6613*
         RS3461e + 1./24.*pow(m11,2)*RSe6633*RS3461e + 1./24.*pow(
         m11,2)*RC346e*RSe6613 - 1./48.*pow(m11,2)*RC346e*RSe6633 + 1./
         24.*pow(m11,2)*RSe6611*RS3461e - 1./48.*pow(m11,2)*RSe6611*
         RC346e );

      F13p6x22RV +=  + hh21dV(1,m13,me2,me2,m13,L,mu2) * ( 1./12.*pow(
         m33,2)*pow(RSe6613,2) - 1./12.*pow(m33,2)*RSe6633*RSe6613 + 1./
         48.*pow(m33,2)*pow(RSe6633,2) - 1./12.*pow(m33,2)*RSe6611*
         RSe6613 + 1./24.*pow(m33,2)*RSe6611*RSe6633 + 1./48.*pow(
         m33,2)*pow(RSe6611,2) + 1./6.*m11*m33*pow(RSe6613,2) - 1./6.*
         m11*m33*RSe6633*RSe6613 + 1./24.*m11*m33*pow(RSe6633,2) - 1./6.
         *m11*m33*RSe6611*RSe6613 + 1./12.*m11*m33*RSe6611*RSe6633 + 1./
         24.*m11*m33*pow(RSe6611,2) + 1./12.*pow(m11,2)*pow(RSe6613,2)
          - 1./12.*pow(m11,2)*RSe6633*RSe6613 + 1./48.*pow(m11,2)*pow(
         RSe6633,2) - 1./12.*pow(m11,2)*RSe6611*RSe6613 + 1./24.*pow(
         m11,2)*RSe6611*RSe6633 + 1./48.*pow(m11,2)*pow(RSe6611,2) );

      F13p6x22RV +=  + hh21dV(1,m14,m11,m34,m13,L,mu2) * ( 1./16.*pow(
         m33,2)*RS1463e + 1./16.*pow(m33,2)*RS16e - 1./16.*pow(m33,2)*
         RC146e + 1./8.*m11*m33*RS1463e + 1./8.*m11*m33*RS16e - 1./8.*
         m11*m33*RC146e + 1./16.*pow(m11,2)*RS1463e + 1./16.*pow(m11,2)
         *RS16e - 1./16.*pow(m11,2)*RC146e );

      F13p6x22RV +=  + hh21dV(1,m14,m33,m34,m13,L,mu2) * ( 1./16.*pow(
         m33,2)*RS3461e - 1./16.*pow(m33,2)*RS36e + 1./8.*m11*m33*
         RS3461e - 1./8.*m11*m33*RS36e + 1./16.*pow(m11,2)*RS3461e - 1./
         16.*pow(m11,2)*RS36e );

      F13p6x22RV +=  + hh21dV(1,m14,m34,me2,m13,L,mu2) * (  - 1./8.*
         pow(m33,2)*RSe6613 - 1./16.*pow(m33,2)*RSe63 + 1./16.*pow(
         m33,2)*RSe61 + 1./8.*pow(m33,2)*RSe6611 - 1./4.*m11*m33*
         RSe6613 - 1./8.*m11*m33*RSe63 + 1./8.*m11*m33*RSe61 + 1./4.*
         m11*m33*RSe6611 - 1./8.*pow(m11,2)*RSe6613 - 1./16.*pow(m11,2)
         *RSe63 + 1./16.*pow(m11,2)*RSe61 + 1./8.*pow(m11,2)*RSe6611 );

      F13p6x22RV +=  + hh21dV(1,m16,m11,m36,m13,L,mu2) * ( 1./32.*pow(
         m33,2)*RS1463e + 1./32.*pow(m33,2)*RS14e - 1./32.*pow(m33,2)*
         RC146e + 1./16.*m11*m33*RS1463e + 1./16.*m11*m33*RS14e - 1./16.
         *m11*m33*RC146e + 1./32.*pow(m11,2)*RS1463e + 1./32.*pow(
         m11,2)*RS14e - 1./32.*pow(m11,2)*RC146e );

      F13p6x22RV +=  + hh21dV(1,m16,m33,m36,m13,L,mu2) * ( 1./32.*pow(
         m33,2)*RS3461e - 1./32.*pow(m33,2)*RS34e + 1./16.*m11*m33*
         RS3461e - 1./16.*m11*m33*RS34e + 1./32.*pow(m11,2)*RS3461e - 1.
         /32.*pow(m11,2)*RS34e );

      F13p6x22RV +=  + hh21dV(1,m16,m36,me2,m13,L,mu2) * (  - 1./16.*
         pow(m33,2)*RSe6613 + 1./16.*pow(m33,2)*RSe63 - 1./16.*pow(
         m33,2)*RSe61 + 1./16.*pow(m33,2)*RSe6611 - 1./8.*m11*m33*
         RSe6613 + 1./8.*m11*m33*RSe63 - 1./8.*m11*m33*RSe61 + 1./8.*
         m11*m33*RSe6611 - 1./16.*pow(m11,2)*RSe6613 + 1./16.*pow(
         m11,2)*RSe63 - 1./16.*pow(m11,2)*RSe61 + 1./16.*pow(m11,2)*
         RSe6611 );

      F13p6x22RV +=  + hh21dV(1,m33,m13,m33,m13,L,mu2) * (  - 3./32.*
         pow(m33,2) - 3./16.*m11*m33 - 3./32.*pow(m11,2) );

      F13p6x22RV +=  + hh21dV(1,m33,m14,m34,m13,L,mu2) * (  - 1./16.*
         pow(m33,2)*RS3461e + 1./16.*pow(m33,2)*RS36e - 1./8.*m11*m33*
         RS3461e + 1./8.*m11*m33*RS36e - 1./16.*pow(m11,2)*RS3461e + 1./
         16.*pow(m11,2)*RS36e );

      F13p6x22RV +=  + hh21dV(1,m33,m16,m36,m13,L,mu2) * (  - 1./32.*
         pow(m33,2)*RS3461e + 1./32.*pow(m33,2)*RS34e - 1./16.*m11*m33*
         RS3461e + 1./16.*m11*m33*RS34e - 1./32.*pow(m11,2)*RS3461e + 1.
         /32.*pow(m11,2)*RS34e );

      F13p6x22RV +=  + hh21dV(1,m34,m11,m14,m13,L,mu2) * ( 1./16.*pow(
         m33,2)*RS1463e - 1./16.*pow(m33,2)*RS16e + 1./8.*m11*m33*
         RS1463e - 1./8.*m11*m33*RS16e + 1./16.*pow(m11,2)*RS1463e - 1./
         16.*pow(m11,2)*RS16e );

      F13p6x22RV +=  + hh21dV(1,m34,m14,m33,m13,L,mu2) * ( 1./16.*pow(
         m33,2)*RS3461e + 1./16.*pow(m33,2)*RS36e - 1./16.*pow(m33,2)*
         RC346e + 1./8.*m11*m33*RS3461e + 1./8.*m11*m33*RS36e - 1./8.*
         m11*m33*RC346e + 1./16.*pow(m11,2)*RS3461e + 1./16.*pow(m11,2)
         *RS36e - 1./16.*pow(m11,2)*RC346e );

      F13p6x22RV +=  + hh21dV(1,m34,m14,me2,m13,L,mu2) * (  - 1./8.*
         pow(m33,2)*RSe6613 + 1./16.*pow(m33,2)*RSe63 + 1./8.*pow(
         m33,2)*RSe6633 - 1./16.*pow(m33,2)*RSe61 - 1./4.*m11*m33*
         RSe6613 + 1./8.*m11*m33*RSe63 + 1./4.*m11*m33*RSe6633 - 1./8.*
         m11*m33*RSe61 - 1./8.*pow(m11,2)*RSe6613 + 1./16.*pow(m11,2)*
         RSe63 + 1./8.*pow(m11,2)*RSe6633 - 1./16.*pow(m11,2)*RSe61 );

      F13p6x22RV +=  + hh21dV(1,m36,m11,m16,m13,L,mu2) * ( 1./32.*pow(
         m33,2)*RS1463e - 1./32.*pow(m33,2)*RS14e + 1./16.*m11*m33*
         RS1463e - 1./16.*m11*m33*RS14e + 1./32.*pow(m11,2)*RS1463e - 1.
         /32.*pow(m11,2)*RS14e );

      F13p6x22RV +=  + hh21dV(1,m36,m16,m33,m13,L,mu2) * ( 1./32.*pow(
         m33,2)*RS3461e + 1./32.*pow(m33,2)*RS34e - 1./32.*pow(m33,2)*
         RC346e + 1./16.*m11*m33*RS3461e + 1./16.*m11*m33*RS34e - 1./16.
         *m11*m33*RC346e + 1./32.*pow(m11,2)*RS3461e + 1./32.*pow(
         m11,2)*RS34e - 1./32.*pow(m11,2)*RC346e );

      F13p6x22RV +=  + hh21dV(1,m36,m16,me2,m13,L,mu2) * (  - 1./16.*
         pow(m33,2)*RSe6613 - 1./16.*pow(m33,2)*RSe63 + 1./16.*pow(
         m33,2)*RSe6633 + 1./16.*pow(m33,2)*RSe61 - 1./8.*m11*m33*
         RSe6613 - 1./8.*m11*m33*RSe63 + 1./8.*m11*m33*RSe6633 + 1./8.*
         m11*m33*RSe61 - 1./16.*pow(m11,2)*RSe6613 - 1./16.*pow(m11,2)*
         RSe63 + 1./16.*pow(m11,2)*RSe6633 + 1./16.*pow(m11,2)*RSe61 );

      F13p6x22RV +=  + hh21dV(1,m44,m14,m34,m13,L,mu2) * ( 9./32.*pow(
         m33,2) + 9./16.*m11*m33 + 9./32.*pow(m11,2) );

      F13p6x22RV +=  + hh21dV(1,m46,m14,m36,m13,L,mu2) * ( 3./16.*pow(
         m33,2) + 3./8.*m11*m33 + 3./16.*pow(m11,2) );

      F13p6x22RV +=  + hh21dV(1,m46,m16,m34,m13,L,mu2) * ( 3./16.*pow(
         m33,2) + 3./8.*m11*m33 + 3./16.*pow(m11,2) );

      F13p6x22RV +=  + hh21dV(1,me2,m14,m34,m13,L,mu2) * ( 1./32.*pow(
         m33,2) + 1./8.*pow(m33,2)*RSe6613 + 1./16.*pow(m33,2)*RSe63 + 
         1./16.*pow(m33,2)*RSe61 + 1./16.*m11*m33 + 1./4.*m11*m33*
         RSe6613 + 1./8.*m11*m33*RSe63 + 1./8.*m11*m33*RSe61 + 1./32.*
         pow(m11,2) + 1./8.*pow(m11,2)*RSe6613 + 1./16.*pow(m11,2)*
         RSe63 + 1./16.*pow(m11,2)*RSe61 );

      F13p6x22RV +=  + hh21dV(1,me2,m16,m36,m13,L,mu2) * ( 1./16.*pow(
         m33,2) + 1./16.*pow(m33,2)*RSe6613 - 1./16.*pow(m33,2)*RSe63
          - 1./16.*pow(m33,2)*RSe61 + 1./8.*m11*m33 + 1./8.*m11*m33*
         RSe6613 - 1./8.*m11*m33*RSe63 - 1./8.*m11*m33*RSe61 + 1./16.*
         pow(m11,2) + 1./16.*pow(m11,2)*RSe6613 - 1./16.*pow(m11,2)*
         RSe63 - 1./16.*pow(m11,2)*RSe61 );

      F13p6x22RV +=  + hh21dV(3,m13,m11,m11,m13,L,mu2) * (  - 1./48.*
         pow(m33,2)*RD146e*RS1463e + 1./96.*pow(m33,2)*RD146e*RC146e - 
         1./24.*m11*m33*RD146e*RS1463e + 1./48.*m11*m33*RD146e*RC146e
          - 1./48.*pow(m11,2)*RD146e*RS1463e + 1./96.*pow(m11,2)*RD146e
         *RC146e );

      F13p6x22RV +=  + hh21dV(3,m13,m11,m33,m13,L,mu2) * (  - 1./48.*
         pow(m33,2)*RD146e*RS3461e + 1./96.*pow(m33,2)*RD146e*RC346e - 
         1./24.*m11*m33*RD146e*RS3461e + 1./48.*m11*m33*RD146e*RC346e
          - 1./48.*pow(m11,2)*RD146e*RS3461e + 1./96.*pow(m11,2)*RD146e
         *RC346e );

      F13p6x22RV +=  + hh21dV(3,m13,m11,me2,m13,L,mu2) * ( 1./24.*pow(
         m33,2)*RD146e*RSe6613 - 1./48.*pow(m33,2)*RD146e*RSe6633 - 1./
         48.*pow(m33,2)*RD146e*RSe6611 + 1./12.*m11*m33*RD146e*RSe6613
          - 1./24.*m11*m33*RD146e*RSe6633 - 1./24.*m11*m33*RD146e*
         RSe6611 + 1./24.*pow(m11,2)*RD146e*RSe6613 - 1./48.*pow(m11,2)
         *RD146e*RSe6633 - 1./48.*pow(m11,2)*RD146e*RSe6611 );

      F13p6x22RV +=  + hh21dV(3,m13,m33,m33,m13,L,mu2) * (  - 1./48.*
         pow(m33,2)*RD346e*RS3461e + 1./96.*pow(m33,2)*RD346e*RC346e - 
         1./24.*m11*m33*RD346e*RS3461e + 1./48.*m11*m33*RD346e*RC346e
          - 1./48.*pow(m11,2)*RD346e*RS3461e + 1./96.*pow(m11,2)*RD346e
         *RC346e );

      F13p6x22RV +=  + hh21dV(3,m13,m33,me2,m13,L,mu2) * ( 1./24.*pow(
         m33,2)*RD346e*RSe6613 - 1./48.*pow(m33,2)*RD346e*RSe6633 - 1./
         48.*pow(m33,2)*RSe6611*RD346e + 1./12.*m11*m33*RD346e*RSe6613
          - 1./24.*m11*m33*RD346e*RSe6633 - 1./24.*m11*m33*RSe6611*
         RD346e + 1./24.*pow(m11,2)*RD346e*RSe6613 - 1./48.*pow(m11,2)*
         RD346e*RSe6633 - 1./48.*pow(m11,2)*RSe6611*RD346e );

      F13p6x22RV +=  + hh21dV(3,m14,m11,m34,m13,L,mu2) * (  - 1./16.*
         pow(m33,2)*RD146e - 1./8.*m11*m33*RD146e - 1./16.*pow(m11,2)*
         RD146e );

      F13p6x22RV +=  + hh21dV(3,m16,m11,m36,m13,L,mu2) * (  - 1./32.*
         pow(m33,2)*RD146e - 1./16.*m11*m33*RD146e - 1./32.*pow(m11,2)*
         RD146e );

      F13p6x22RV +=  + hh21dV(4,m13,m11,m33,m13,L,mu2) * (  - 1./48.*
         pow(m33,2)*RD346e*RS1463e + 1./96.*pow(m33,2)*RC146e*RD346e - 
         1./24.*m11*m33*RD346e*RS1463e + 1./48.*m11*m33*RC146e*RD346e
          - 1./48.*pow(m11,2)*RD346e*RS1463e + 1./96.*pow(m11,2)*RC146e
         *RD346e );

      F13p6x22RV +=  + hh21dV(4,m34,m14,m33,m13,L,mu2) * (  - 1./16.*
         pow(m33,2)*RD346e - 1./8.*m11*m33*RD346e - 1./16.*pow(m11,2)*
         RD346e );

      F13p6x22RV +=  + hh21dV(4,m36,m16,m33,m13,L,mu2) * (  - 1./32.*
         pow(m33,2)*RD346e - 1./16.*m11*m33*RD346e - 1./32.*pow(m11,2)*
         RD346e );

      F13p6x22RV +=  + hh21dV(7,m13,m11,m11,m13,L,mu2) * ( 1./192.*pow(
         m33,2)*pow(RD146e,2) + 1./96.*m11*m33*pow(RD146e,2) + 1./192.*
         pow(m11,2)*pow(RD146e,2) );

      F13p6x22RV +=  + hh21dV(7,m13,m11,m33,m13,L,mu2) * ( 1./96.*pow(
         m33,2)*RD146e*RD346e + 1./48.*m11*m33*RD146e*RD346e + 1./96.*
         pow(m11,2)*RD146e*RD346e );

      F13p6x22RV +=  + hh21dV(7,m13,m33,m33,m13,L,mu2) * ( 1./192.*pow(
         m33,2)*pow(RD346e,2) + 1./96.*m11*m33*pow(RD346e,2) + 1./192.*
         pow(m11,2)*pow(RD346e,2) );

      F13p6x22RV +=  + hh27dV(1,m11,m11,m13,m13,L,mu2) * ( 3./16.*m33
          + 3./16.*m11 );

      F13p6x22RV +=  + hh27dV(1,m11,m14,m34,m13,L,mu2) * ( 1./8.*m33*
         RS1463e - 1./8.*m33*RS16e + 1./8.*m11*RS1463e - 1./8.*m11*
         RS16e );

      F13p6x22RV +=  + hh27dV(1,m11,m16,m36,m13,L,mu2) * ( 1./16.*m33*
         RS1463e - 1./16.*m33*RS14e + 1./16.*m11*RS1463e - 1./16.*m11*
         RS14e );

      F13p6x22RV +=  + hh27dV(1,m13,m11,m11,m13,L,mu2) * (  - 3./32.*
         m33 - 1./24.*m33*pow(RS1463e,2) + 1./24.*m33*RC146e*RS1463e - 
         1./96.*m33*pow(RC146e,2) - 3./32.*m11 - 1./24.*m11*pow(
         RS1463e,2) + 1./24.*m11*RC146e*RS1463e - 1./96.*m11*pow(
         RC146e,2) );

      F13p6x22RV +=  + hh27dV(1,m13,m11,m33,m13,L,mu2) * (  - 1./12.*
         m33*RS1463e*RS3461e + 1./24.*m33*RC346e*RS1463e + 1./24.*m33*
         RC146e*RS3461e - 1./48.*m33*RC146e*RC346e - 1./12.*m11*RS1463e
         *RS3461e + 1./24.*m11*RC346e*RS1463e + 1./24.*m11*RC146e*
         RS3461e - 1./48.*m11*RC146e*RC346e );

      F13p6x22RV +=  + hh27dV(1,m13,m11,me2,m13,L,mu2) * ( 1./6.*m33*
         RSe6613*RS1463e - 1./12.*m33*RSe6633*RS1463e - 1./12.*m33*
         RSe6611*RS1463e - 1./12.*m33*RC146e*RSe6613 + 1./24.*m33*
         RC146e*RSe6633 + 1./24.*m33*RC146e*RSe6611 + 1./6.*m11*RSe6613
         *RS1463e - 1./12.*m11*RSe6633*RS1463e - 1./12.*m11*RSe6611*
         RS1463e - 1./12.*m11*RC146e*RSe6613 + 1./24.*m11*RC146e*
         RSe6633 + 1./24.*m11*RC146e*RSe6611 );

      F13p6x22RV +=  + hh27dV(1,m13,m13,m13,m13,L,mu2) * (  - 3./16.*
         m33 - 3./16.*m11 );

      F13p6x22RV +=  + hh27dV(1,m13,m33,m33,m13,L,mu2) * (  - 3./32.*
         m33 - 1./24.*m33*pow(RS3461e,2) + 1./24.*m33*RC346e*RS3461e - 
         1./96.*m33*pow(RC346e,2) - 3./32.*m11 - 1./24.*m11*pow(
         RS3461e,2) + 1./24.*m11*RC346e*RS3461e - 1./96.*m11*pow(
         RC346e,2) );

      F13p6x22RV +=  + hh27dV(1,m13,m33,me2,m13,L,mu2) * ( 1./6.*m33*
         RSe6613*RS3461e - 1./12.*m33*RSe6633*RS3461e - 1./12.*m33*
         RC346e*RSe6613 + 1./24.*m33*RC346e*RSe6633 - 1./12.*m33*
         RSe6611*RS3461e + 1./24.*m33*RSe6611*RC346e + 1./6.*m11*
         RSe6613*RS3461e - 1./12.*m11*RSe6633*RS3461e - 1./12.*m11*
         RC346e*RSe6613 + 1./24.*m11*RC346e*RSe6633 - 1./12.*m11*
         RSe6611*RS3461e + 1./24.*m11*RSe6611*RC346e );

      F13p6x22RV +=  + hh27dV(1,m13,me2,me2,m13,L,mu2) * (  - 1./6.*m33
         *pow(RSe6613,2) + 1./6.*m33*RSe6633*RSe6613 - 1./24.*m33*pow(
         RSe6633,2) + 1./6.*m33*RSe6611*RSe6613 - 1./12.*m33*RSe6611*
         RSe6633 - 1./24.*m33*pow(RSe6611,2) - 1./6.*m11*pow(RSe6613,2)
          + 1./6.*m11*RSe6633*RSe6613 - 1./24.*m11*pow(RSe6633,2) + 1./
         6.*m11*RSe6611*RSe6613 - 1./12.*m11*RSe6611*RSe6633 - 1./24.*
         m11*pow(RSe6611,2) );

      F13p6x22RV +=  + hh27dV(1,m14,m11,m34,m13,L,mu2) * (  - 1./8.*m33
         *RS1463e - 1./8.*m33*RS16e + 1./8.*m33*RC146e - 1./8.*m11*
         RS1463e - 1./8.*m11*RS16e + 1./8.*m11*RC146e );

      F13p6x22RV +=  + hh27dV(1,m14,m33,m34,m13,L,mu2) * (  - 1./8.*m33
         *RS3461e + 1./8.*m33*RS36e - 1./8.*m11*RS3461e + 1./8.*m11*
         RS36e );

      F13p6x22RV +=  + hh27dV(1,m14,m34,me2,m13,L,mu2) * ( 1./4.*m33*
         RSe6613 + 1./8.*m33*RSe63 - 1./8.*m33*RSe61 - 1./4.*m33*
         RSe6611 + 1./4.*m11*RSe6613 + 1./8.*m11*RSe63 - 1./8.*m11*
         RSe61 - 1./4.*m11*RSe6611 );

      F13p6x22RV +=  + hh27dV(1,m16,m11,m36,m13,L,mu2) * (  - 1./16.*
         m33*RS1463e - 1./16.*m33*RS14e + 1./16.*m33*RC146e - 1./16.*
         m11*RS1463e - 1./16.*m11*RS14e + 1./16.*m11*RC146e );

      F13p6x22RV +=  + hh27dV(1,m16,m33,m36,m13,L,mu2) * (  - 1./16.*
         m33*RS3461e + 1./16.*m33*RS34e - 1./16.*m11*RS3461e + 1./16.*
         m11*RS34e );

      F13p6x22RV +=  + hh27dV(1,m16,m36,me2,m13,L,mu2) * ( 1./8.*m33*
         RSe6613 - 1./8.*m33*RSe63 + 1./8.*m33*RSe61 - 1./8.*m33*
         RSe6611 + 1./8.*m11*RSe6613 - 1./8.*m11*RSe63 + 1./8.*m11*
         RSe61 - 1./8.*m11*RSe6611 );

      F13p6x22RV +=  + hh27dV(1,m33,m13,m33,m13,L,mu2) * ( 3./16.*m33
          + 3./16.*m11 );

      F13p6x22RV +=  + hh27dV(1,m33,m14,m34,m13,L,mu2) * ( 1./8.*m33*
         RS3461e - 1./8.*m33*RS36e + 1./8.*m11*RS3461e - 1./8.*m11*
         RS36e );

      F13p6x22RV +=  + hh27dV(1,m33,m16,m36,m13,L,mu2) * ( 1./16.*m33*
         RS3461e - 1./16.*m33*RS34e + 1./16.*m11*RS3461e - 1./16.*m11*
         RS34e );

      F13p6x22RV +=  + hh27dV(1,m34,m11,m14,m13,L,mu2) * (  - 1./8.*m33
         *RS1463e + 1./8.*m33*RS16e - 1./8.*m11*RS1463e + 1./8.*m11*
         RS16e );

      F13p6x22RV +=  + hh27dV(1,m34,m14,m33,m13,L,mu2) * (  - 1./8.*m33
         *RS3461e - 1./8.*m33*RS36e + 1./8.*m33*RC346e - 1./8.*m11*
         RS3461e - 1./8.*m11*RS36e + 1./8.*m11*RC346e );

      F13p6x22RV +=  + hh27dV(1,m34,m14,me2,m13,L,mu2) * ( 1./4.*m33*
         RSe6613 - 1./8.*m33*RSe63 - 1./4.*m33*RSe6633 + 1./8.*m33*
         RSe61 + 1./4.*m11*RSe6613 - 1./8.*m11*RSe63 - 1./4.*m11*
         RSe6633 + 1./8.*m11*RSe61 );

      F13p6x22RV +=  + hh27dV(1,m36,m11,m16,m13,L,mu2) * (  - 1./16.*
         m33*RS1463e + 1./16.*m33*RS14e - 1./16.*m11*RS1463e + 1./16.*
         m11*RS14e );

      F13p6x22RV +=  + hh27dV(1,m36,m16,m33,m13,L,mu2) * (  - 1./16.*
         m33*RS3461e - 1./16.*m33*RS34e + 1./16.*m33*RC346e - 1./16.*
         m11*RS3461e - 1./16.*m11*RS34e + 1./16.*m11*RC346e );

      F13p6x22RV +=  + hh27dV(1,m36,m16,me2,m13,L,mu2) * ( 1./8.*m33*
         RSe6613 + 1./8.*m33*RSe63 - 1./8.*m33*RSe6633 - 1./8.*m33*
         RSe61 + 1./8.*m11*RSe6613 + 1./8.*m11*RSe63 - 1./8.*m11*
         RSe6633 - 1./8.*m11*RSe61 );

      F13p6x22RV +=  + hh27dV(1,m44,m14,m34,m13,L,mu2) * (  - 9./16.*
         m33 - 9./16.*m11 );

      F13p6x22RV +=  + hh27dV(1,m46,m14,m36,m13,L,mu2) * (  - 3./8.*m33
          - 3./8.*m11 );

      F13p6x22RV +=  + hh27dV(1,m46,m16,m34,m13,L,mu2) * (  - 3./8.*m33
          - 3./8.*m11 );

      F13p6x22RV +=  + hh27dV(1,me2,m14,m34,m13,L,mu2) * (  - 1./16.*
         m33 - 1./4.*m33*RSe6613 - 1./8.*m33*RSe63 - 1./8.*m33*RSe61 - 
         1./16.*m11 - 1./4.*m11*RSe6613 - 1./8.*m11*RSe63 - 1./8.*m11*
         RSe61 );

      F13p6x22RV +=  + hh27dV(1,me2,m16,m36,m13,L,mu2) * (  - 1./8.*m33
          - 1./8.*m33*RSe6613 + 1./8.*m33*RSe63 + 1./8.*m33*RSe61 - 1./
         8.*m11 - 1./8.*m11*RSe6613 + 1./8.*m11*RSe63 + 1./8.*m11*RSe61
          );

      F13p6x22RV +=  + hh27dV(3,m13,m11,m11,m13,L,mu2) * ( 1./24.*m33*
         RD146e*RS1463e - 1./48.*m33*RD146e*RC146e + 1./24.*m11*RD146e*
         RS1463e - 1./48.*m11*RD146e*RC146e );

      F13p6x22RV +=  + hh27dV(3,m13,m11,m33,m13,L,mu2) * ( 1./24.*m33*
         RD146e*RS3461e - 1./48.*m33*RD146e*RC346e + 1./24.*m11*RD146e*
         RS3461e - 1./48.*m11*RD146e*RC346e );

      F13p6x22RV +=  + hh27dV(3,m13,m11,me2,m13,L,mu2) * (  - 1./12.*
         m33*RD146e*RSe6613 + 1./24.*m33*RD146e*RSe6633 + 1./24.*m33*
         RD146e*RSe6611 - 1./12.*m11*RD146e*RSe6613 + 1./24.*m11*RD146e
         *RSe6633 + 1./24.*m11*RD146e*RSe6611 );

      F13p6x22RV +=  + hh27dV(3,m13,m33,m33,m13,L,mu2) * ( 1./24.*m33*
         RD346e*RS3461e - 1./48.*m33*RD346e*RC346e + 1./24.*m11*RD346e*
         RS3461e - 1./48.*m11*RD346e*RC346e );

      F13p6x22RV +=  + hh27dV(3,m13,m33,me2,m13,L,mu2) * (  - 1./12.*
         m33*RD346e*RSe6613 + 1./24.*m33*RD346e*RSe6633 + 1./24.*m33*
         RSe6611*RD346e - 1./12.*m11*RD346e*RSe6613 + 1./24.*m11*RD346e
         *RSe6633 + 1./24.*m11*RSe6611*RD346e );

      F13p6x22RV +=  + hh27dV(3,m14,m11,m34,m13,L,mu2) * ( 1./8.*m33*
         RD146e + 1./8.*m11*RD146e );

      F13p6x22RV +=  + hh27dV(3,m16,m11,m36,m13,L,mu2) * ( 1./16.*m33*
         RD146e + 1./16.*m11*RD146e );

      F13p6x22RV +=  + hh27dV(4,m13,m11,m33,m13,L,mu2) * ( 1./24.*m33*
         RD346e*RS1463e - 1./48.*m33*RC146e*RD346e + 1./24.*m11*RD346e*
         RS1463e - 1./48.*m11*RC146e*RD346e );

      F13p6x22RV +=  + hh27dV(4,m34,m14,m33,m13,L,mu2) * ( 1./8.*m33*
         RD346e + 1./8.*m11*RD346e );

      F13p6x22RV +=  + hh27dV(4,m36,m16,m33,m13,L,mu2) * ( 1./16.*m33*
         RD346e + 1./16.*m11*RD346e );

      F13p6x22RV +=  + hh27dV(7,m13,m11,m11,m13,L,mu2) * (  - 1./96.*
         m33*pow(RD146e,2) - 1./96.*m11*pow(RD146e,2) );

      F13p6x22RV +=  + hh27dV(7,m13,m11,m33,m13,L,mu2) * (  - 1./48.*
         m33*RD146e*RD346e - 1./48.*m11*RD146e*RD346e );

      F13p6x22RV +=  + hh27dV(7,m13,m33,m33,m13,L,mu2) * (  - 1./96.*
         m33*pow(RD346e,2) - 1./96.*m11*pow(RD346e,2) );

  return F13p6x22RV/pow(f0,4);
}

// 1 valence mass 3 sea mass *****************************************

double fv1s3nf3p4V(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 4) || (B0mq.size() != 4)){
    cout << "ERROR: fv2s3nf3p4R probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m55 = 2.*B0mq[2];
  double m66 = 2.*B0mq[3];
  double m14 = (m11+m44)/2;
  double m15 = (m11+m55)/2;
  double m16 = (m11+m66)/2;

  //double mu2 = mu0*mu0;

  double F12p4x13V =
       + AbV(m14,L) * ( 1./2. );

      F12p4x13V +=  + AbV(m15,L) * ( 1./2. );

      F12p4x13V +=  + AbV(m16,L) * ( 1./2. );

  return F12p4x13V/pow(f0,2);
}

double fv1s3nf3p6V(const quarkmassnf mass, const Linf Liin, const double L){
  return fv1s3nf3p6LV(mass,Liin,L)+fv1s3nf3p6RV(mass,L);
}

double fv1s3nf3p6LV(const quarkmassnf mass, const Linf Liin, const double L){
  double L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu;
  int nfLi;
  Liin.out(L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu,nfLi);
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if (mu0 != mu){
    cout << "ERROR: differing mu's in fv1s3nf3p6LV\n";}
  if ( (nq != 4) || (B0mq.size() != 4)){
    cout << "ERROR: fv1s3nf3p6LV probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m55 = 2.*B0mq[2];
  double m66 = 2.*B0mq[3];
  double m14 = (m11+m44)/2;
  double m15 = (m11+m55)/2;
  double m16 = (m11+m66)/2;
  double m45 = (m44+m55)/2;
  double m46 = (m44+m66)/2;
  double m56 = (m55+m66)/2;

  double xb1 = (m44+m55+m66)/3.;
  double msqr = sqrt(xb1*xb1-(m44*m55+m44*m66+m55*m66)/3.);
  double mp2 = xb1-msqr; 
  double me2 = xb1+msqr;
  //double mu2 = mu0*mu0;


  double RS456pe = RS(m44,m55,m66,mp2,me2);
  double RS546pe = RS(m55,m44,m66,mp2,me2);
  double RS645pe = RS(m66,m44,m55,mp2,me2);

  double RD1456ep = RD(m11,m44,m55,m66,me2,mp2);
  double RC1456pe = RC(m11,m44,m55,m66,mp2,me2);  
  double RSe45611p = RS(me2,m44,m55,m66,m11,m11,mp2);
  double RSp45611e = RS(mp2,m44,m55,m66,m11,m11,me2);

  double RSp564e = RS(mp2,m55,m66,m44,me2);
  double RSe564p = RS(me2,m55,m66,m44,mp2);
  double RSp465e = RS(mp2,m44,m66,m55,me2);
  double RSe465p = RS(me2,m44,m66,m55,mp2);
  double RSp456e = RS(mp2,m44,m55,m66,me2);
  double RSe456p = RS(me2,m44,m55,m66,mp2);
  double F12p6x13LV =
       + AbV(m11,L) * ( 4*L3r*RD1456ep + 4*L0r*RD1456ep - 4./3.*m11*L5r
         *RC1456pe + 4*m11*L3r*RC1456pe - 10*m11*L2r - 4*m11*L1r + 4*
         m11*L0r*RC1456pe );

      F12p6x13LV +=  + AbV(m14,L) * (  - 2*m66*L4r - 2*m55*L4r - 2*m44*
         L4r - 10*m14*L3r - 4*m14*L0r + 2*m11*L5r );

      F12p6x13LV +=  + AbV(m15,L) * (  - 2*m66*L4r - 2*m55*L4r - 2*m44*
         L4r - 10*m15*L3r - 4*m15*L0r + 2*m11*L5r );

      F12p6x13LV +=  + AbV(m16,L) * (  - 2*m66*L4r - 2*m55*L4r - 2*m44*
         L4r - 10*m16*L3r - 4*m16*L0r + 2*m11*L5r );

      F12p6x13LV +=  + AbV(m44,L) * ( 4*m44*L4r - 4./3.*m44*L4r*RS456pe
          - 2*m44*L2r + 2./3.*m44*L2r*RS456pe - 8*m44*L1r + 8./3.*m44*
         L1r*RS456pe );

      F12p6x13LV +=  + AbV(m45,L) * ( 4*m55*L4r - 4*m45*L2r - 16*m45*
         L1r + 4*m44*L4r );

      F12p6x13LV +=  + AbV(m46,L) * ( 4*m66*L4r - 4*m46*L2r - 16*m46*
         L1r + 4*m44*L4r );

      F12p6x13LV +=  + AbV(m55,L) * ( 4*m55*L4r - 4./3.*m55*L4r*RS546pe
          - 2*m55*L2r + 2./3.*m55*L2r*RS546pe - 8*m55*L1r + 8./3.*m55*
         L1r*RS546pe );

      F12p6x13LV +=  + AbV(m56,L) * ( 4*m66*L4r - 4*m56*L2r - 16*m56*
         L1r + 4*m55*L4r );

      F12p6x13LV +=  + AbV(m66,L) * ( 4*m66*L4r - 4./3.*m66*L4r*RS645pe
          - 2*m66*L2r + 2./3.*m66*L2r*RS645pe - 8*m66*L1r + 8./3.*m66*
         L1r*RS645pe );

      F12p6x13LV +=  + AbV(mp2,L) * ( 4*mp2*L3r*RSp45611e + 2./3.*mp2*
         L2r*RSp564e + 2./3.*mp2*L2r*RSp465e + 2./3.*mp2*L2r*RSp456e + 
         8./3.*mp2*L1r*RSp564e + 8./3.*mp2*L1r*RSp465e + 8./3.*mp2*L1r*
         RSp456e + 4*mp2*L0r*RSp45611e - 4./3.*m66*L4r*RSp456e - 4./3.*
         m55*L4r*RSp465e - 4./3.*m44*L4r*RSp564e - 4./3.*m11*L5r*
         RSp45611e );

      F12p6x13LV +=  + AbV(me2,L) * ( 4*me2*L3r*RSe45611p + 2./3.*me2*
         L2r*RSe564p + 2./3.*me2*L2r*RSe465p + 2./3.*me2*L2r*RSe456p + 
         8./3.*me2*L1r*RSe564p + 8./3.*me2*L1r*RSe465p + 8./3.*me2*L1r*
         RSe456p + 4*me2*L0r*RSe45611p - 4./3.*m66*L4r*RSe456p - 4./3.*
         m55*L4r*RSe465p - 4./3.*m44*L4r*RSe564p - 4./3.*m11*L5r*
         RSe45611p );

      F12p6x13LV +=  + BbV(m11,L) * (  - 4./3.*m11*L5r*RD1456ep + 4*m11
         *L3r*RD1456ep + 4*m11*L0r*RD1456ep );

      F12p6x13LV +=  + BbV(m14,L) * ( 4*m44*m66*L6r + 4*m44*m55*L6r + 2
         *pow(m44,2)*L8r + 4*pow(m44,2)*L6r - 4*m14*m66*L4r - 4*m14*m55
         *L4r - 2*m14*m44*L5r - 4*m14*m44*L4r + 4*m11*m66*L6r + 4*m11*
         m55*L6r + 4*m11*m44*L8r + 4*m11*m44*L6r - 2*m11*m14*L5r + 2*
         pow(m11,2)*L8r );

      F12p6x13LV +=  + BbV(m15,L) * ( 4*m55*m66*L6r + 2*pow(m55,2)*L8r
          + 4*pow(m55,2)*L6r + 4*m44*m55*L6r - 4*m15*m66*L4r - 2*m15*
         m55*L5r - 4*m15*m55*L4r - 4*m15*m44*L4r + 4*m11*m66*L6r + 4*
         m11*m55*L8r + 4*m11*m55*L6r + 4*m11*m44*L6r - 2*m11*m15*L5r + 
         2*pow(m11,2)*L8r );

      F12p6x13LV +=  + BbV(m16,L) * ( 2*pow(m66,2)*L8r + 4*pow(m66,2)*
         L6r + 4*m55*m66*L6r + 4*m44*m66*L6r - 2*m16*m66*L5r - 4*m16*
         m66*L4r - 4*m16*m55*L4r - 4*m16*m44*L4r + 4*m11*m66*L8r + 4*
         m11*m66*L6r + 4*m11*m55*L6r + 4*m11*m44*L6r - 2*m11*m16*L5r + 
         2*pow(m11,2)*L8r );

      F12p6x13LV +=  + A23bV(m11,L) * (  - 4*L3r*RC1456pe + 6*L2r + 12*
         L1r - 4*L0r*RC1456pe );

      F12p6x13LV +=  + A23bV(m14,L) * ( 6*L3r + 12*L0r );

      F12p6x13LV +=  + A23bV(m15,L) * ( 6*L3r + 12*L0r );

      F12p6x13LV +=  + A23bV(m16,L) * ( 6*L3r + 12*L0r );

      F12p6x13LV +=  + A23bV(m44,L) * ( 6*L2r - 2*L2r*RS456pe );

      F12p6x13LV +=  + A23bV(m45,L) * ( 12*L2r );

      F12p6x13LV +=  + A23bV(m46,L) * ( 12*L2r );

      F12p6x13LV +=  + A23bV(m55,L) * ( 6*L2r - 2*L2r*RS546pe );

      F12p6x13LV +=  + A23bV(m56,L) * ( 12*L2r );

      F12p6x13LV +=  + A23bV(m66,L) * ( 6*L2r - 2*L2r*RS645pe );

      F12p6x13LV +=  + A23bV(mp2,L) * (  - 4*L3r*RSp45611e - 2*L2r*
         RSp564e - 2*L2r*RSp465e - 2*L2r*RSp456e - 4*L0r*RSp45611e );

      F12p6x13LV +=  + A23bV(me2,L) * (  - 4*L3r*RSe45611p - 2*L2r*
         RSe564p - 2*L2r*RSe465p - 2*L2r*RSe456p - 4*L0r*RSe45611p );

      F12p6x13LV +=  + B23bV(m11,L) * (  - 4*L3r*RD1456ep - 4*L0r*
         RD1456ep );

  return F12p6x13LV/pow(f0,4);
}

double fv1s3nf3p6RV(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 4) || (B0mq.size() != 4)){
    cout << "ERROR: fv1s3nf3p6R probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m44 = 2.*B0mq[1];
  double m55 = 2.*B0mq[2];
  double m66 = 2.*B0mq[3];
  double m14 = (m11+m44)/2;
  double m15 = (m11+m55)/2;
  double m16 = (m11+m66)/2;
  double m45 = (m44+m55)/2;
  double m46 = (m44+m66)/2;
  double m56 = (m55+m66)/2;

  double xb1 = (m44+m55+m66)/3.;
  double msqr = sqrt(xb1*xb1-(m44*m55+m44*m66+m55*m66)/3.);
  double mp2 = xb1-msqr; 
  double me2 = xb1+msqr;
  double mu2 = mu0*mu0;

//  double RN1p = RN(m11,mp2);
//  double RN1e = RN(m11,me2);
//  double RNep = RN(me2,mp2);

  double RS14e = RS(m11,m44,me2);
  double RS16e = RS(m11,m66,me2);
  //double RS144ee = RS14e*RS14e;
  //double RS166ee = RS16e*RS16e;
  //double RSe6611 = RS(me2,m66,m66,m11,m11);
  double RSp456e = RS(mp2,m44,m55,m66,me2);
  double RSp465e = RS(mp2,m44,m66,m55,me2);
  double RSp564e = RS(mp2,m55,m66,m44,me2);
  double RSe456p = RS(me2,m44,m55,m66,mp2);
  double RSe465p = RS(me2,m44,m66,m55,mp2);
  double RSe564p = RS(me2,m55,m66,m44,mp2);
  double RSp451e = RS(mp2,m44,m55,m11,me2);
  double RSp461e = RS(mp2,m44,m66,m11,me2);
  double RSp561e = RS(mp2,m55,m66,m11,me2);
  double RSe451p = RS(me2,m44,m55,m11,mp2);
  double RSe461p = RS(me2,m44,m66,m11,mp2);
  double RSe561p = RS(me2,m55,m66,m11,mp2);
  double RS456pe = RS(m44,m55,m66,mp2,me2);
  double RS546pe = RS(m55,m44,m66,mp2,me2);
  double RS645pe = RS(m66,m44,m55,mp2,me2);
  double RS156pe = RS(m11,m55,m66,mp2,me2);
  double RS146pe = RS(m11,m44,m66,mp2,me2);
  double RS145pe = RS(m11,m44,m55,mp2,me2);

  double RD1456ep = RD(m11,m44,m55,m66,me2,mp2);
  double RC1456pe = RC(m11,m44,m55,m66,mp2,me2);  
  double RSe45611p = RS(me2,m44,m55,m66,m11,m11,mp2);
  double RSp45611e = RS(mp2,m44,m55,m66,m11,m11,me2);
//  double RS1456ppe = RS(m11,m44,m55,m66,mp2,mp2,me2);
//  double RS1456pee = RS(m11,m44,m55,m66,mp2,me2,me2);

  double F12p6x13RV =
       + Ab(m11,mu2)*BbV(m14,L) * ( 1./36.*RD1456ep - 1./72.*m44*
         RC1456pe - 1./36.*m44*RS156pe + 1./36.*m14*RC1456pe - 1./18.*
         m14*RS156pe - 1./72.*m11*RC1456pe - 1./12.*m11*RS156pe );

      F12p6x13RV +=  + Ab(m11,mu2)*BbV(m15,L) * ( 1./36.*RD1456ep - 1./
         72.*m55*RC1456pe - 1./36.*m55*RS146pe + 1./36.*m15*RC1456pe - 
         1./18.*m15*RS146pe - 1./72.*m11*RC1456pe - 1./12.*m11*RS146pe
          );

      F12p6x13RV +=  + Ab(m11,mu2)*BbV(m16,L) * ( 1./36.*RD1456ep - 1./
         72.*m66*RC1456pe - 1./36.*m66*RS145pe + 1./36.*m16*RC1456pe - 
         1./18.*m16*RS145pe - 1./72.*m11*RC1456pe - 1./12.*m11*RS145pe
          );

      F12p6x13RV +=  + Ab(m14,mu2)*AbV(m14,L) * ( 5./108.*RC1456pe + 5./
         108.*RSe45611p + 5./108.*RSp45611e - 5./54.*RS156pe + 5./108.*
         RS456pe - 5./54.*RSe561p - 5./54.*RSp561e + 5./108.*RSe564p + 
         5./108.*RSp564e );

      F12p6x13RV +=  + Ab(m14,mu2)*BbV(m14,L) * ( 1./12.*m44 - 1./6.*
         m14 + 1./12.*m11 );

      F12p6x13RV +=  + Ab(m14,mu2)*BbV(m15,L) * ( 1./24.*m55 + 1./24.*
         m44 - 1./12.*m15 - 1./12.*m14 + 1./12.*m11 );

      F12p6x13RV +=  + Ab(m14,mu2)*BbV(m16,L) * ( 1./24.*m66 + 1./24.*
         m44 - 1./12.*m16 - 1./12.*m14 + 1./12.*m11 );

      F12p6x13RV +=  + Ab(m15,mu2)*AbV(m15,L) * ( 5./108.*RC1456pe + 5./
         108.*RSe45611p + 5./108.*RSp45611e - 5./54.*RS146pe + 5./108.*
         RS546pe - 5./54.*RSe461p - 5./54.*RSp461e + 5./108.*RSe465p + 
         5./108.*RSp465e );

      F12p6x13RV +=  + Ab(m15,mu2)*BbV(m14,L) * ( 1./24.*m55 + 1./24.*
         m44 - 1./12.*m15 - 1./12.*m14 + 1./12.*m11 );

      F12p6x13RV +=  + Ab(m15,mu2)*BbV(m15,L) * ( 1./12.*m55 - 1./6.*
         m15 + 1./12.*m11 );

      F12p6x13RV +=  + Ab(m15,mu2)*BbV(m16,L) * ( 1./24.*m66 + 1./24.*
         m55 - 1./12.*m16 - 1./12.*m15 + 1./12.*m11 );

      F12p6x13RV +=  + Ab(m16,mu2)*AbV(m16,L) * ( 5./108.*RC1456pe + 5./
         108.*RSe45611p + 5./108.*RSp45611e - 5./54.*RS145pe + 5./108.*
         RS645pe - 5./54.*RSe451p - 5./54.*RSp451e + 5./108.*RSe456p + 
         5./108.*RSp456e );

      F12p6x13RV +=  + Ab(m16,mu2)*BbV(m14,L) * ( 1./24.*m66 + 1./24.*
         m44 - 1./12.*m16 - 1./12.*m14 + 1./12.*m11 );

      F12p6x13RV +=  + Ab(m16,mu2)*BbV(m15,L) * ( 1./24.*m66 + 1./24.*
         m55 - 1./12.*m16 - 1./12.*m15 + 1./12.*m11 );

      F12p6x13RV +=  + Ab(m16,mu2)*BbV(m16,L) * ( 1./12.*m66 - 1./6.*
         m16 + 1./12.*m11 );

      F12p6x13RV +=  + Ab(m44,mu2)*BbV(m14,L) * ( 1./24.*m44 - 1./72.*
         m44*RS456pe - 1./12.*m14 + 1./36.*m14*RS456pe + 1./24.*m11 - 1.
         /72.*m11*RS456pe );

      F12p6x13RV +=  + Ab(m45,mu2)*BbV(m14,L) * ( 1./24.*m55 - 1./12.*
         m45 + 1./12.*m44 - 1./12.*m14 + 1./24.*m11 );

      F12p6x13RV +=  + Ab(m45,mu2)*BbV(m15,L) * ( 1./12.*m55 - 1./12.*
         m45 + 1./24.*m44 - 1./12.*m15 + 1./24.*m11 );

      F12p6x13RV +=  + Ab(m46,mu2)*BbV(m14,L) * ( 1./24.*m66 - 1./12.*
         m46 + 1./12.*m44 - 1./12.*m14 + 1./24.*m11 );

      F12p6x13RV +=  + Ab(m46,mu2)*BbV(m16,L) * ( 1./12.*m66 - 1./12.*
         m46 + 1./24.*m44 - 1./12.*m16 + 1./24.*m11 );

      F12p6x13RV +=  + Ab(m55,mu2)*BbV(m15,L) * ( 1./24.*m55 - 1./72.*
         m55*RS546pe - 1./12.*m15 + 1./36.*m15*RS546pe + 1./24.*m11 - 1.
         /72.*m11*RS546pe );

      F12p6x13RV +=  + Ab(m56,mu2)*BbV(m15,L) * ( 1./24.*m66 - 1./12.*
         m56 + 1./12.*m55 - 1./12.*m15 + 1./24.*m11 );

      F12p6x13RV +=  + Ab(m56,mu2)*BbV(m16,L) * ( 1./12.*m66 - 1./12.*
         m56 + 1./24.*m55 - 1./12.*m16 + 1./24.*m11 );

      F12p6x13RV +=  + Ab(m66,mu2)*BbV(m16,L) * ( 1./24.*m66 - 1./72.*
         m66*RS645pe - 1./12.*m16 + 1./36.*m16*RS645pe + 1./24.*m11 - 1.
         /72.*m11*RS645pe );

      F12p6x13RV +=  + Ab(mp2,mu2)*BbV(m14,L) * ( 1./36.*mp2*RSp45611e
          - 1./18.*mp2*RSp561e + 1./36.*mp2*RSp564e - 1./72.*m44*
         RSp45611e - 1./36.*m44*RSp561e - 1./24.*m44*RSp564e + 1./36.*
         m14*RSp45611e - 1./18.*m14*RSp561e + 1./36.*m14*RSp564e - 1./
         24.*m11*RSp45611e - 1./36.*m11*RSp561e - 1./72.*m11*RSp564e );

      F12p6x13RV +=  + Ab(mp2,mu2)*BbV(m15,L) * ( 1./36.*mp2*RSp45611e
          - 1./18.*mp2*RSp461e + 1./36.*mp2*RSp465e - 1./72.*m55*
         RSp45611e - 1./36.*m55*RSp461e - 1./24.*m55*RSp465e + 1./36.*
         m15*RSp45611e - 1./18.*m15*RSp461e + 1./36.*m15*RSp465e - 1./
         24.*m11*RSp45611e - 1./36.*m11*RSp461e - 1./72.*m11*RSp465e );

      F12p6x13RV +=  + Ab(mp2,mu2)*BbV(m16,L) * ( 1./36.*mp2*RSp45611e
          - 1./18.*mp2*RSp451e + 1./36.*mp2*RSp456e - 1./72.*m66*
         RSp45611e - 1./36.*m66*RSp451e - 1./24.*m66*RSp456e + 1./36.*
         m16*RSp45611e - 1./18.*m16*RSp451e + 1./36.*m16*RSp456e - 1./
         24.*m11*RSp45611e - 1./36.*m11*RSp451e - 1./72.*m11*RSp456e );

      F12p6x13RV +=  + Ab(me2,mu2)*BbV(m14,L) * ( 1./36.*me2*RSe45611p
          - 1./18.*me2*RSe561p + 1./36.*me2*RSe564p - 1./72.*m44*
         RSe45611p - 1./36.*m44*RSe561p - 1./24.*m44*RSe564p + 1./36.*
         m14*RSe45611p - 1./18.*m14*RSe561p + 1./36.*m14*RSe564p - 1./
         24.*m11*RSe45611p - 1./36.*m11*RSe561p - 1./72.*m11*RSe564p );

      F12p6x13RV +=  + Ab(me2,mu2)*BbV(m15,L) * ( 1./36.*me2*RSe45611p
          - 1./18.*me2*RSe461p + 1./36.*me2*RSe465p - 1./72.*m55*
         RSe45611p - 1./36.*m55*RSe461p - 1./24.*m55*RSe465p + 1./36.*
         m15*RSe45611p - 1./18.*m15*RSe461p + 1./36.*m15*RSe465p - 1./
         24.*m11*RSe45611p - 1./36.*m11*RSe461p - 1./72.*m11*RSe465p );

      F12p6x13RV +=  + Ab(me2,mu2)*BbV(m16,L) * ( 1./36.*me2*RSe45611p
          - 1./18.*me2*RSe451p + 1./36.*me2*RSe456p - 1./72.*m66*
         RSe45611p - 1./36.*m66*RSe451p - 1./24.*m66*RSe456p + 1./36.*
         m16*RSe45611p - 1./18.*m16*RSe451p + 1./36.*m16*RSe456p - 1./
         24.*m11*RSe45611p - 1./36.*m11*RSe451p - 1./72.*m11*RSe456p );

      F12p6x13RV +=  + Bb(m11,mu2)*BbV(m14,L) * (  - 1./72.*m44*
         RD1456ep + 1./36.*m14*RD1456ep - 1./72.*m11*RD1456ep );

      F12p6x13RV +=  + Bb(m11,mu2)*BbV(m15,L) * (  - 1./72.*m55*
         RD1456ep + 1./36.*m15*RD1456ep - 1./72.*m11*RD1456ep );

      F12p6x13RV +=  + Bb(m11,mu2)*BbV(m16,L) * (  - 1./72.*m66*
         RD1456ep + 1./36.*m16*RD1456ep - 1./72.*m11*RD1456ep );

      F12p6x13RV +=  + Bb(m14,mu2)*AbV(m11,L) * ( 1./36.*RD1456ep - 1./
         72.*m44*RC1456pe - 1./36.*m44*RS156pe + 1./36.*m14*RC1456pe - 
         1./18.*m14*RS156pe - 1./72.*m11*RC1456pe - 1./12.*m11*RS156pe
          );

      F12p6x13RV +=  + Bb(m14,mu2)*AbV(m14,L) * ( 1./12.*m44 - 1./6.*
         m14 + 1./12.*m11 );

      F12p6x13RV +=  + Bb(m14,mu2)*AbV(m15,L) * ( 1./24.*m55 + 1./24.*
         m44 - 1./12.*m15 - 1./12.*m14 + 1./12.*m11 );

      F12p6x13RV +=  + Bb(m14,mu2)*AbV(m16,L) * ( 1./24.*m66 + 1./24.*
         m44 - 1./12.*m16 - 1./12.*m14 + 1./12.*m11 );

      F12p6x13RV +=  + Bb(m14,mu2)*AbV(m44,L) * ( 1./24.*m44 - 1./72.*
         m44*RS456pe - 1./12.*m14 + 1./36.*m14*RS456pe + 1./24.*m11 - 1.
         /72.*m11*RS456pe );

      F12p6x13RV +=  + Bb(m14,mu2)*AbV(m45,L) * ( 1./24.*m55 - 1./12.*
         m45 + 1./12.*m44 - 1./12.*m14 + 1./24.*m11 );

      F12p6x13RV +=  + Bb(m14,mu2)*AbV(m46,L) * ( 1./24.*m66 - 1./12.*
         m46 + 1./12.*m44 - 1./12.*m14 + 1./24.*m11 );

      F12p6x13RV +=  + Bb(m14,mu2)*AbV(mp2,L) * ( 1./36.*mp2*RSp45611e
          - 1./18.*mp2*RSp561e + 1./36.*mp2*RSp564e - 1./72.*m44*
         RSp45611e - 1./36.*m44*RSp561e - 1./24.*m44*RSp564e + 1./36.*
         m14*RSp45611e - 1./18.*m14*RSp561e + 1./36.*m14*RSp564e - 1./
         24.*m11*RSp45611e - 1./36.*m11*RSp561e - 1./72.*m11*RSp564e );

      F12p6x13RV +=  + Bb(m14,mu2)*AbV(me2,L) * ( 1./36.*me2*RSe45611p
          - 1./18.*me2*RSe561p + 1./36.*me2*RSe564p - 1./72.*m44*
         RSe45611p - 1./36.*m44*RSe561p - 1./24.*m44*RSe564p + 1./36.*
         m14*RSe45611p - 1./18.*m14*RSe561p + 1./36.*m14*RSe564p - 1./
         24.*m11*RSe45611p - 1./36.*m11*RSe561p - 1./72.*m11*RSe564p );

      F12p6x13RV +=  + Bb(m14,mu2)*BbV(m11,L) * (  - 1./72.*m44*
         RD1456ep + 1./36.*m14*RD1456ep - 1./72.*m11*RD1456ep );

      F12p6x13RV +=  + Bb(m15,mu2)*AbV(m11,L) * ( 1./36.*RD1456ep - 1./
         72.*m55*RC1456pe - 1./36.*m55*RS146pe + 1./36.*m15*RC1456pe - 
         1./18.*m15*RS146pe - 1./72.*m11*RC1456pe - 1./12.*m11*RS146pe
          );

      F12p6x13RV +=  + Bb(m15,mu2)*AbV(m14,L) * ( 1./24.*m55 + 1./24.*
         m44 - 1./12.*m15 - 1./12.*m14 + 1./12.*m11 );

      F12p6x13RV +=  + Bb(m15,mu2)*AbV(m15,L) * ( 1./12.*m55 - 1./6.*
         m15 + 1./12.*m11 );

      F12p6x13RV +=  + Bb(m15,mu2)*AbV(m16,L) * ( 1./24.*m66 + 1./24.*
         m55 - 1./12.*m16 - 1./12.*m15 + 1./12.*m11 );

      F12p6x13RV +=  + Bb(m15,mu2)*AbV(m45,L) * ( 1./12.*m55 - 1./12.*
         m45 + 1./24.*m44 - 1./12.*m15 + 1./24.*m11 );

      F12p6x13RV +=  + Bb(m15,mu2)*AbV(m55,L) * ( 1./24.*m55 - 1./72.*
         m55*RS546pe - 1./12.*m15 + 1./36.*m15*RS546pe + 1./24.*m11 - 1.
         /72.*m11*RS546pe );

      F12p6x13RV +=  + Bb(m15,mu2)*AbV(m56,L) * ( 1./24.*m66 - 1./12.*
         m56 + 1./12.*m55 - 1./12.*m15 + 1./24.*m11 );

      F12p6x13RV +=  + Bb(m15,mu2)*AbV(mp2,L) * ( 1./36.*mp2*RSp45611e
          - 1./18.*mp2*RSp461e + 1./36.*mp2*RSp465e - 1./72.*m55*
         RSp45611e - 1./36.*m55*RSp461e - 1./24.*m55*RSp465e + 1./36.*
         m15*RSp45611e - 1./18.*m15*RSp461e + 1./36.*m15*RSp465e - 1./
         24.*m11*RSp45611e - 1./36.*m11*RSp461e - 1./72.*m11*RSp465e );

      F12p6x13RV +=  + Bb(m15,mu2)*AbV(me2,L) * ( 1./36.*me2*RSe45611p
          - 1./18.*me2*RSe461p + 1./36.*me2*RSe465p - 1./72.*m55*
         RSe45611p - 1./36.*m55*RSe461p - 1./24.*m55*RSe465p + 1./36.*
         m15*RSe45611p - 1./18.*m15*RSe461p + 1./36.*m15*RSe465p - 1./
         24.*m11*RSe45611p - 1./36.*m11*RSe461p - 1./72.*m11*RSe465p );

      F12p6x13RV +=  + Bb(m15,mu2)*BbV(m11,L) * (  - 1./72.*m55*
         RD1456ep + 1./36.*m15*RD1456ep - 1./72.*m11*RD1456ep );

      F12p6x13RV +=  + Bb(m16,mu2)*AbV(m11,L) * ( 1./36.*RD1456ep - 1./
         72.*m66*RC1456pe - 1./36.*m66*RS145pe + 1./36.*m16*RC1456pe - 
         1./18.*m16*RS145pe - 1./72.*m11*RC1456pe - 1./12.*m11*RS145pe
          );

      F12p6x13RV +=  + Bb(m16,mu2)*AbV(m14,L) * ( 1./24.*m66 + 1./24.*
         m44 - 1./12.*m16 - 1./12.*m14 + 1./12.*m11 );

      F12p6x13RV +=  + Bb(m16,mu2)*AbV(m15,L) * ( 1./24.*m66 + 1./24.*
         m55 - 1./12.*m16 - 1./12.*m15 + 1./12.*m11 );

      F12p6x13RV +=  + Bb(m16,mu2)*AbV(m16,L) * ( 1./12.*m66 - 1./6.*
         m16 + 1./12.*m11 );

      F12p6x13RV +=  + Bb(m16,mu2)*AbV(m46,L) * ( 1./12.*m66 - 1./12.*
         m46 + 1./24.*m44 - 1./12.*m16 + 1./24.*m11 );

      F12p6x13RV +=  + Bb(m16,mu2)*AbV(m56,L) * ( 1./12.*m66 - 1./12.*
         m56 + 1./24.*m55 - 1./12.*m16 + 1./24.*m11 );

      F12p6x13RV +=  + Bb(m16,mu2)*AbV(m66,L) * ( 1./24.*m66 - 1./72.*
         m66*RS645pe - 1./12.*m16 + 1./36.*m16*RS645pe + 1./24.*m11 - 1.
         /72.*m11*RS645pe );

      F12p6x13RV +=  + Bb(m16,mu2)*AbV(mp2,L) * ( 1./36.*mp2*RSp45611e
          - 1./18.*mp2*RSp451e + 1./36.*mp2*RSp456e - 1./72.*m66*
         RSp45611e - 1./36.*m66*RSp451e - 1./24.*m66*RSp456e + 1./36.*
         m16*RSp45611e - 1./18.*m16*RSp451e + 1./36.*m16*RSp456e - 1./
         24.*m11*RSp45611e - 1./36.*m11*RSp451e - 1./72.*m11*RSp456e );

      F12p6x13RV +=  + Bb(m16,mu2)*AbV(me2,L) * ( 1./36.*me2*RSe45611p
          - 1./18.*me2*RSe451p + 1./36.*me2*RSe456p - 1./72.*m66*
         RSe45611p - 1./36.*m66*RSe451p - 1./24.*m66*RSe456p + 1./36.*
         m16*RSe45611p - 1./18.*m16*RSe451p + 1./36.*m16*RSe456p - 1./
         24.*m11*RSe45611p - 1./36.*m11*RSe451p - 1./72.*m11*RSe456p );

      F12p6x13RV +=  + Bb(m16,mu2)*BbV(m11,L) * (  - 1./72.*m66*
         RD1456ep + 1./36.*m16*RD1456ep - 1./72.*m11*RD1456ep );

      F12p6x13RV +=  + AbV(m11,L)*pi16 * ( 1./8.*m11 - 1./24.*m11*
         RS145pe - 1./24.*m11*RS146pe - 1./24.*m11*RS156pe );

      F12p6x13RV +=  + AbV(m11,L)*BbV(m14,L) * ( 1./36.*RD1456ep - 1./
         72.*m44*RC1456pe - 1./36.*m44*RS156pe + 1./36.*m14*RC1456pe - 
         1./18.*m14*RS156pe - 1./72.*m11*RC1456pe - 1./12.*m11*RS156pe
          );

      F12p6x13RV +=  + AbV(m11,L)*BbV(m15,L) * ( 1./36.*RD1456ep - 1./
         72.*m55*RC1456pe - 1./36.*m55*RS146pe + 1./36.*m15*RC1456pe - 
         1./18.*m15*RS146pe - 1./72.*m11*RC1456pe - 1./12.*m11*RS146pe
          );

      F12p6x13RV +=  + AbV(m11,L)*BbV(m16,L) * ( 1./36.*RD1456ep - 1./
         72.*m66*RC1456pe - 1./36.*m66*RS145pe + 1./36.*m16*RC1456pe - 
         1./18.*m16*RS145pe - 1./72.*m11*RC1456pe - 1./12.*m11*RS145pe
          );

      F12p6x13RV +=  + AbV(m14,L)*pi16 * ( 1./24.*RD1456ep + 1./24.*me2
         *RSe45611p - 1./12.*me2*RSe561p + 1./24.*me2*RSe564p + 1./24.*
         mp2*RSp45611e - 1./12.*mp2*RSp561e + 1./24.*mp2*RSp564e - 1./8.
         *m46 - 1./8.*m45 - 1./8.*m44 + 1./24.*m44*RS456pe - 1./8.*m16
          - 1./8.*m15 + 1./24.*m14*RC1456pe + 1./24.*m14*RSe45611p + 1./
         24.*m14*RSp45611e - 1./12.*m14*RS156pe + 1./24.*m14*RS456pe - 
         1./12.*m14*RSe561p - 1./12.*m14*RSp561e + 1./24.*m14*RSe564p
          + 1./24.*m14*RSp564e + 1./24.*m11*RC1456pe - 1./12.*m11*
         RS156pe );

      F12p6x13RV +=  + pow(AbV(m14,L),2) * ( 5./216.*RC1456pe + 5./216.
         *RSe45611p + 5./216.*RSp45611e - 5./108.*RS156pe + 5./216.*
         RS456pe - 5./108.*RSe561p - 5./108.*RSp561e + 5./216.*RSe564p
          + 5./216.*RSp564e );

      F12p6x13RV +=  + AbV(m14,L)*BbV(m14,L) * ( 1./12.*m44 - 1./6.*m14
          + 1./12.*m11 );

      F12p6x13RV +=  + AbV(m14,L)*BbV(m15,L) * ( 1./24.*m55 + 1./24.*
         m44 - 1./12.*m15 - 1./12.*m14 + 1./12.*m11 );

      F12p6x13RV +=  + AbV(m14,L)*BbV(m16,L) * ( 1./24.*m66 + 1./24.*
         m44 - 1./12.*m16 - 1./12.*m14 + 1./12.*m11 );

      F12p6x13RV +=  + AbV(m15,L)*pi16 * ( 1./24.*RD1456ep + 1./24.*me2
         *RSe45611p - 1./12.*me2*RSe461p + 1./24.*me2*RSe465p + 1./24.*
         mp2*RSp45611e - 1./12.*mp2*RSp461e + 1./24.*mp2*RSp465e - 1./8.
         *m56 - 1./8.*m55 + 1./24.*m55*RS546pe - 1./8.*m45 - 1./8.*m16
          + 1./24.*m15*RC1456pe + 1./24.*m15*RSe45611p + 1./24.*m15*
         RSp45611e - 1./12.*m15*RS146pe + 1./24.*m15*RS546pe - 1./12.*
         m15*RSe461p - 1./12.*m15*RSp461e + 1./24.*m15*RSe465p + 1./24.
         *m15*RSp465e - 1./8.*m14 + 1./24.*m11*RC1456pe - 1./12.*m11*
         RS146pe );

      F12p6x13RV +=  + pow(AbV(m15,L),2) * ( 5./216.*RC1456pe + 5./216.
         *RSe45611p + 5./216.*RSp45611e - 5./108.*RS146pe + 5./216.*
         RS546pe - 5./108.*RSe461p - 5./108.*RSp461e + 5./216.*RSe465p
          + 5./216.*RSp465e );

      F12p6x13RV +=  + AbV(m15,L)*BbV(m14,L) * ( 1./24.*m55 + 1./24.*
         m44 - 1./12.*m15 - 1./12.*m14 + 1./12.*m11 );

      F12p6x13RV +=  + AbV(m15,L)*BbV(m15,L) * ( 1./12.*m55 - 1./6.*m15
          + 1./12.*m11 );

      F12p6x13RV +=  + AbV(m15,L)*BbV(m16,L) * ( 1./24.*m66 + 1./24.*
         m55 - 1./12.*m16 - 1./12.*m15 + 1./12.*m11 );

      F12p6x13RV +=  + AbV(m16,L)*pi16 * ( 1./24.*RD1456ep + 1./24.*me2
         *RSe45611p - 1./12.*me2*RSe451p + 1./24.*me2*RSe456p + 1./24.*
         mp2*RSp45611e - 1./12.*mp2*RSp451e + 1./24.*mp2*RSp456e - 1./8.
         *m66 + 1./24.*m66*RS645pe - 1./8.*m56 - 1./8.*m46 + 1./24.*m16
         *RC1456pe + 1./24.*m16*RSe45611p + 1./24.*m16*RSp45611e - 1./
         12.*m16*RS145pe + 1./24.*m16*RS645pe - 1./12.*m16*RSe451p - 1./
         12.*m16*RSp451e + 1./24.*m16*RSe456p + 1./24.*m16*RSp456e - 1./
         8.*m15 - 1./8.*m14 + 1./24.*m11*RC1456pe - 1./12.*m11*RS145pe
          );

      F12p6x13RV +=  + pow(AbV(m16,L),2) * ( 5./216.*RC1456pe + 5./216.
         *RSe45611p + 5./216.*RSp45611e - 5./108.*RS145pe + 5./216.*
         RS645pe - 5./108.*RSe451p - 5./108.*RSp451e + 5./216.*RSe456p
          + 5./216.*RSp456e );

      F12p6x13RV +=  + AbV(m16,L)*BbV(m14,L) * ( 1./24.*m66 + 1./24.*
         m44 - 1./12.*m16 - 1./12.*m14 + 1./12.*m11 );

      F12p6x13RV +=  + AbV(m16,L)*BbV(m15,L) * ( 1./24.*m66 + 1./24.*
         m55 - 1./12.*m16 - 1./12.*m15 + 1./12.*m11 );

      F12p6x13RV +=  + AbV(m16,L)*BbV(m16,L) * ( 1./12.*m66 - 1./6.*m16
          + 1./12.*m11 );

      F12p6x13RV +=  + AbV(m44,L)*BbV(m14,L) * ( 1./24.*m44 - 1./72.*
         m44*RS456pe - 1./12.*m14 + 1./36.*m14*RS456pe + 1./24.*m11 - 1.
         /72.*m11*RS456pe );

      F12p6x13RV +=  + AbV(m45,L)*BbV(m14,L) * ( 1./24.*m55 - 1./12.*
         m45 + 1./12.*m44 - 1./12.*m14 + 1./24.*m11 );

      F12p6x13RV +=  + AbV(m45,L)*BbV(m15,L) * ( 1./12.*m55 - 1./12.*
         m45 + 1./24.*m44 - 1./12.*m15 + 1./24.*m11 );

      F12p6x13RV +=  + AbV(m46,L)*BbV(m14,L) * ( 1./24.*m66 - 1./12.*
         m46 + 1./12.*m44 - 1./12.*m14 + 1./24.*m11 );

      F12p6x13RV +=  + AbV(m46,L)*BbV(m16,L) * ( 1./12.*m66 - 1./12.*
         m46 + 1./24.*m44 - 1./12.*m16 + 1./24.*m11 );

      F12p6x13RV +=  + AbV(m55,L)*BbV(m15,L) * ( 1./24.*m55 - 1./72.*
         m55*RS546pe - 1./12.*m15 + 1./36.*m15*RS546pe + 1./24.*m11 - 1.
         /72.*m11*RS546pe );

      F12p6x13RV +=  + AbV(m56,L)*BbV(m15,L) * ( 1./24.*m66 - 1./12.*
         m56 + 1./12.*m55 - 1./12.*m15 + 1./24.*m11 );

      F12p6x13RV +=  + AbV(m56,L)*BbV(m16,L) * ( 1./12.*m66 - 1./12.*
         m56 + 1./24.*m55 - 1./12.*m16 + 1./24.*m11 );

      F12p6x13RV +=  + AbV(m66,L)*BbV(m16,L) * ( 1./24.*m66 - 1./72.*
         m66*RS645pe - 1./12.*m16 + 1./36.*m16*RS645pe + 1./24.*m11 - 1.
         /72.*m11*RS645pe );

      F12p6x13RV +=  + AbV(mp2,L)*pi16 * (  - 1./24.*mp2*RSp561e - 1./
         24.*mp2*RSp461e - 1./24.*mp2*RSp451e );

      F12p6x13RV +=  + AbV(mp2,L)*BbV(m14,L) * ( 1./36.*mp2*RSp45611e
          - 1./18.*mp2*RSp561e + 1./36.*mp2*RSp564e - 1./72.*m44*
         RSp45611e - 1./36.*m44*RSp561e - 1./24.*m44*RSp564e + 1./36.*
         m14*RSp45611e - 1./18.*m14*RSp561e + 1./36.*m14*RSp564e - 1./
         24.*m11*RSp45611e - 1./36.*m11*RSp561e - 1./72.*m11*RSp564e );

      F12p6x13RV +=  + AbV(mp2,L)*BbV(m15,L) * ( 1./36.*mp2*RSp45611e
          - 1./18.*mp2*RSp461e + 1./36.*mp2*RSp465e - 1./72.*m55*
         RSp45611e - 1./36.*m55*RSp461e - 1./24.*m55*RSp465e + 1./36.*
         m15*RSp45611e - 1./18.*m15*RSp461e + 1./36.*m15*RSp465e - 1./
         24.*m11*RSp45611e - 1./36.*m11*RSp461e - 1./72.*m11*RSp465e );

      F12p6x13RV +=  + AbV(mp2,L)*BbV(m16,L) * ( 1./36.*mp2*RSp45611e
          - 1./18.*mp2*RSp451e + 1./36.*mp2*RSp456e - 1./72.*m66*
         RSp45611e - 1./36.*m66*RSp451e - 1./24.*m66*RSp456e + 1./36.*
         m16*RSp45611e - 1./18.*m16*RSp451e + 1./36.*m16*RSp456e - 1./
         24.*m11*RSp45611e - 1./36.*m11*RSp451e - 1./72.*m11*RSp456e );

      F12p6x13RV +=  + AbV(me2,L)*pi16 * (  - 1./24.*me2*RSe561p - 1./
         24.*me2*RSe461p - 1./24.*me2*RSe451p );

      F12p6x13RV +=  + AbV(me2,L)*BbV(m14,L) * ( 1./36.*me2*RSe45611p
          - 1./18.*me2*RSe561p + 1./36.*me2*RSe564p - 1./72.*m44*
         RSe45611p - 1./36.*m44*RSe561p - 1./24.*m44*RSe564p + 1./36.*
         m14*RSe45611p - 1./18.*m14*RSe561p + 1./36.*m14*RSe564p - 1./
         24.*m11*RSe45611p - 1./36.*m11*RSe561p - 1./72.*m11*RSe564p );

      F12p6x13RV +=  + AbV(me2,L)*BbV(m15,L) * ( 1./36.*me2*RSe45611p
          - 1./18.*me2*RSe461p + 1./36.*me2*RSe465p - 1./72.*m55*
         RSe45611p - 1./36.*m55*RSe461p - 1./24.*m55*RSe465p + 1./36.*
         m15*RSe45611p - 1./18.*m15*RSe461p + 1./36.*m15*RSe465p - 1./
         24.*m11*RSe45611p - 1./36.*m11*RSe461p - 1./72.*m11*RSe465p );

      F12p6x13RV +=  + AbV(me2,L)*BbV(m16,L) * ( 1./36.*me2*RSe45611p
          - 1./18.*me2*RSe451p + 1./36.*me2*RSe456p - 1./72.*m66*
         RSe45611p - 1./36.*m66*RSe451p - 1./24.*m66*RSe456p + 1./36.*
         m16*RSe45611p - 1./18.*m16*RSe451p + 1./36.*m16*RSe456p - 1./
         24.*m11*RSe45611p - 1./36.*m11*RSe451p - 1./72.*m11*RSe456p );

      F12p6x13RV +=  + BbV(m11,L)*BbV(m14,L) * (  - 1./72.*m44*RD1456ep
          + 1./36.*m14*RD1456ep - 1./72.*m11*RD1456ep );

      F12p6x13RV +=  + BbV(m11,L)*BbV(m15,L) * (  - 1./72.*m55*RD1456ep
          + 1./36.*m15*RD1456ep - 1./72.*m11*RD1456ep );

      F12p6x13RV +=  + BbV(m11,L)*BbV(m16,L) * (  - 1./72.*m66*RD1456ep
          + 1./36.*m16*RD1456ep - 1./72.*m11*RD1456ep );

      F12p6x13RV +=  + A23bV(m11,L)*pi16 * (  - 3./8. + 1./8.*RS145pe
          + 1./8.*RS146pe + 1./8.*RS156pe );

      F12p6x13RV +=  + A23bV(m14,L)*pi16 * (  - 1./24.*RC1456pe - 1./24.
         *RSe45611p - 1./24.*RSp45611e + 1./12.*RS156pe - 1./24.*
         RS456pe + 1./12.*RSe561p + 1./12.*RSp561e - 1./24.*RSe564p - 1.
         /24.*RSp564e );

      F12p6x13RV +=  + A23bV(m15,L)*pi16 * (  - 1./24.*RC1456pe - 1./24.
         *RSe45611p - 1./24.*RSp45611e + 1./12.*RS146pe - 1./24.*
         RS546pe + 1./12.*RSe461p + 1./12.*RSp461e - 1./24.*RSe465p - 1.
         /24.*RSp465e );

      F12p6x13RV +=  + A23bV(m16,L)*pi16 * (  - 1./24.*RC1456pe - 1./24.
         *RSe45611p - 1./24.*RSp45611e + 1./12.*RS145pe - 1./24.*
         RS645pe + 1./12.*RSe451p + 1./12.*RSp451e - 1./24.*RSe456p - 1.
         /24.*RSp456e );

      F12p6x13RV +=  + A23bV(mp2,L)*pi16 * ( 1./8.*RSp561e + 1./8.*
         RSp461e + 1./8.*RSp451e );

      F12p6x13RV +=  + A23bV(me2,L)*pi16 * ( 1./8.*RSe561p + 1./8.*
         RSe461p + 1./8.*RSe451p );

      F12p6x13RV +=  + hhV(1,m11,m14,m14,m11,L,mu2) * ( 1./24.*RD1456ep
          + 1./24.*m11*RC1456pe - 1./12.*m11*RS156pe );

      F12p6x13RV +=  + hhV(1,m11,m15,m15,m11,L,mu2) * ( 1./24.*RD1456ep
          + 1./24.*m11*RC1456pe - 1./12.*m11*RS146pe );

      F12p6x13RV +=  + hhV(1,m11,m16,m16,m11,L,mu2) * ( 1./24.*RD1456ep
          + 1./24.*m11*RC1456pe - 1./12.*m11*RS145pe );

      F12p6x13RV +=  + hhV(1,m44,m14,m14,m11,L,mu2) * (  - 1./8.*m44 + 
         1./24.*m44*RS456pe );

      F12p6x13RV +=  + hhV(1,m45,m14,m15,m11,L,mu2) * (  - 1./4.*m45 );

      F12p6x13RV +=  + hhV(1,m46,m14,m16,m11,L,mu2) * (  - 1./4.*m46 );

      F12p6x13RV +=  + hhV(1,m55,m15,m15,m11,L,mu2) * (  - 1./8.*m55 + 
         1./24.*m55*RS546pe );

      F12p6x13RV +=  + hhV(1,m56,m15,m16,m11,L,mu2) * (  - 1./4.*m56 );

      F12p6x13RV +=  + hhV(1,m66,m16,m16,m11,L,mu2) * (  - 1./8.*m66 + 
         1./24.*m66*RS645pe );

      F12p6x13RV +=  + hhV(1,mp2,m14,m14,m11,L,mu2) * ( 1./24.*mp2*
         RSp45611e - 1./12.*mp2*RSp561e + 1./24.*mp2*RSp564e );

      F12p6x13RV +=  + hhV(1,mp2,m15,m15,m11,L,mu2) * ( 1./24.*mp2*
         RSp45611e - 1./12.*mp2*RSp461e + 1./24.*mp2*RSp465e );

      F12p6x13RV +=  + hhV(1,mp2,m16,m16,m11,L,mu2) * ( 1./24.*mp2*
         RSp45611e - 1./12.*mp2*RSp451e + 1./24.*mp2*RSp456e );

      F12p6x13RV +=  + hhV(1,me2,m14,m14,m11,L,mu2) * ( 1./24.*me2*
         RSe45611p - 1./12.*me2*RSe561p + 1./24.*me2*RSe564p );

      F12p6x13RV +=  + hhV(1,me2,m15,m15,m11,L,mu2) * ( 1./24.*me2*
         RSe45611p - 1./12.*me2*RSe461p + 1./24.*me2*RSe465p );

      F12p6x13RV +=  + hhV(1,me2,m16,m16,m11,L,mu2) * ( 1./24.*me2*
         RSe45611p - 1./12.*me2*RSe451p + 1./24.*me2*RSe456p );

      F12p6x13RV +=  + hhV(2,m11,m14,m14,m11,L,mu2) * (  - 1./54.*m44*
         RD1456ep + 1./27.*m14*RD1456ep + 1./36.*m11*RD1456ep );

      F12p6x13RV +=  + hhV(2,m11,m15,m15,m11,L,mu2) * (  - 1./54.*m55*
         RD1456ep + 1./27.*m15*RD1456ep + 1./36.*m11*RD1456ep );

      F12p6x13RV +=  + hhV(2,m11,m16,m16,m11,L,mu2) * (  - 1./54.*m66*
         RD1456ep + 1./27.*m16*RD1456ep + 1./36.*m11*RD1456ep );

      F12p6x13RV +=  + hh1V(2,m11,m14,m14,m11,L,mu2) * ( 1./54.*m44*
         RD1456ep - 1./27.*m14*RD1456ep + 1./72.*m11*RD1456ep );

      F12p6x13RV +=  + hh1V(2,m11,m15,m15,m11,L,mu2) * ( 1./54.*m55*
         RD1456ep - 1./27.*m15*RD1456ep + 1./72.*m11*RD1456ep );

      F12p6x13RV +=  + hh1V(2,m11,m16,m16,m11,L,mu2) * ( 1./54.*m66*
         RD1456ep - 1./27.*m16*RD1456ep + 1./72.*m11*RD1456ep );

      F12p6x13RV +=  + hh1V(3,m14,m11,m14,m11,L,mu2) * ( 1./27.*m44*
         RD1456ep - 2./27.*m14*RD1456ep + 1./36.*m11*RD1456ep );

      F12p6x13RV +=  + hh1V(3,m15,m11,m15,m11,L,mu2) * ( 1./27.*m55*
         RD1456ep - 2./27.*m15*RD1456ep + 1./36.*m11*RD1456ep );

      F12p6x13RV +=  + hh1V(3,m16,m11,m16,m11,L,mu2) * ( 1./27.*m66*
         RD1456ep - 2./27.*m16*RD1456ep + 1./36.*m11*RD1456ep );

      F12p6x13RV +=  + hh27V(1,m11,m14,m14,m11,L,mu2) * (  - 1./8.*
         RC1456pe + 1./4.*RS156pe );

      F12p6x13RV +=  + hh27V(1,m11,m15,m15,m11,L,mu2) * (  - 1./8.*
         RC1456pe + 1./4.*RS146pe );

      F12p6x13RV +=  + hh27V(1,m11,m16,m16,m11,L,mu2) * (  - 1./8.*
         RC1456pe + 1./4.*RS145pe );

      F12p6x13RV +=  + hh27V(1,m44,m14,m14,m11,L,mu2) * ( 3./8. - 1./8.
         *RS456pe );

      F12p6x13RV +=  + hh27V(1,m45,m14,m15,m11,L,mu2) * ( 3./4. );

      F12p6x13RV +=  + hh27V(1,m46,m14,m16,m11,L,mu2) * ( 3./4. );

      F12p6x13RV +=  + hh27V(1,m55,m15,m15,m11,L,mu2) * ( 3./8. - 1./8.
         *RS546pe );

      F12p6x13RV +=  + hh27V(1,m56,m15,m16,m11,L,mu2) * ( 3./4. );

      F12p6x13RV +=  + hh27V(1,m66,m16,m16,m11,L,mu2) * ( 3./8. - 1./8.
         *RS645pe );

      F12p6x13RV +=  + hh27V(1,mp2,m14,m14,m11,L,mu2) * (  - 1./8.*
         RSp45611e + 1./4.*RSp561e - 1./8.*RSp564e );

      F12p6x13RV +=  + hh27V(1,mp2,m15,m15,m11,L,mu2) * (  - 1./8.*
         RSp45611e + 1./4.*RSp461e - 1./8.*RSp465e );

      F12p6x13RV +=  + hh27V(1,mp2,m16,m16,m11,L,mu2) * (  - 1./8.*
         RSp45611e + 1./4.*RSp451e - 1./8.*RSp456e );

      F12p6x13RV +=  + hh27V(1,me2,m14,m14,m11,L,mu2) * (  - 1./8.*
         RSe45611p + 1./4.*RSe561p - 1./8.*RSe564p );

      F12p6x13RV +=  + hh27V(1,me2,m15,m15,m11,L,mu2) * (  - 1./8.*
         RSe45611p + 1./4.*RSe461p - 1./8.*RSe465p );

      F12p6x13RV +=  + hh27V(1,me2,m16,m16,m11,L,mu2) * (  - 1./8.*
         RSe45611p + 1./4.*RSe451p - 1./8.*RSe456p );

      F12p6x13RV +=  + hh27V(2,m11,m14,m14,m11,L,mu2) * (  - 1./8.*
         RD1456ep );

      F12p6x13RV +=  + hh27V(2,m11,m15,m15,m11,L,mu2) * (  - 1./8.*
         RD1456ep );

      F12p6x13RV +=  + hh27V(2,m11,m16,m16,m11,L,mu2) * (  - 1./8.*
         RD1456ep );

      F12p6x13RV +=  + hhdV(1,m11,m11,m11,m11,L,mu2) * ( 1./6.*pow(
         m11,2) + 1./9.*pow(m11,2)*pow(RC1456pe,2) );

      F12p6x13RV +=  + hhdV(1,m11,m11,mp2,m11,L,mu2) * ( 2./9.*pow(
         m11,2)*RSp45611e*RC1456pe );

      F12p6x13RV +=  + hhdV(1,m11,m11,me2,m11,L,mu2) * ( 2./9.*pow(
         m11,2)*RSe45611p*RC1456pe );

      F12p6x13RV +=  + hhdV(1,m11,m14,m14,m11,L,mu2) * (  - 1./54.*m44*
         RD1456ep - 1./216.*pow(m44,2)*RC1456pe - 1./108.*pow(m44,2)*
         RS156pe + 1./27.*m14*RD1456ep + 1./54.*m14*m44*RC1456pe - 1./
         54.*pow(m14,2)*RC1456pe + 1./27.*pow(m14,2)*RS156pe - 37./216.
         *m11*RD1456ep - 7./108.*m11*m44*RC1456pe - 1./27.*m11*m44*
         RS156pe + 7./54.*m11*m14*RC1456pe - 5./27.*m11*m14*RS156pe - 
         29./108.*pow(m11,2)*RC1456pe + 5./18.*pow(m11,2)*RS156pe );

      F12p6x13RV +=  + hhdV(1,m11,m15,m15,m11,L,mu2) * (  - 1./54.*m55*
         RD1456ep - 1./216.*pow(m55,2)*RC1456pe - 1./108.*pow(m55,2)*
         RS146pe + 1./27.*m15*RD1456ep + 1./54.*m15*m55*RC1456pe - 1./
         54.*pow(m15,2)*RC1456pe + 1./27.*pow(m15,2)*RS146pe - 37./216.
         *m11*RD1456ep - 7./108.*m11*m55*RC1456pe - 1./27.*m11*m55*
         RS146pe + 7./54.*m11*m15*RC1456pe - 5./27.*m11*m15*RS146pe - 
         29./108.*pow(m11,2)*RC1456pe + 5./18.*pow(m11,2)*RS146pe );

      F12p6x13RV +=  + hhdV(1,m11,m16,m16,m11,L,mu2) * (  - 1./54.*m66*
         RD1456ep - 1./216.*pow(m66,2)*RC1456pe - 1./108.*pow(m66,2)*
         RS145pe + 1./27.*m16*RD1456ep + 1./54.*m16*m66*RC1456pe - 1./
         54.*pow(m16,2)*RC1456pe + 1./27.*pow(m16,2)*RS145pe - 37./216.
         *m11*RD1456ep - 7./108.*m11*m66*RC1456pe - 1./27.*m11*m66*
         RS145pe + 7./54.*m11*m16*RC1456pe - 5./27.*m11*m16*RS145pe - 
         29./108.*pow(m11,2)*RC1456pe + 5./18.*pow(m11,2)*RS145pe );

      F12p6x13RV +=  + hhdV(1,m11,mp2,mp2,m11,L,mu2) * ( 1./9.*pow(
         m11,2)*pow(RSp45611e,2) );

      F12p6x13RV +=  + hhdV(1,m11,me2,mp2,m11,L,mu2) * ( 2./9.*pow(
         m11,2)*RSp45611e*RSe45611p );

      F12p6x13RV +=  + hhdV(1,m11,me2,me2,m11,L,mu2) * ( 1./9.*pow(
         m11,2)*pow(RSe45611p,2) );

      F12p6x13RV +=  + hhdV(1,m14,m14,m44,m11,L,mu2) * ( 1./72.*pow(
         m44,2) - 1./216.*pow(m44,2)*RS456pe - 1./18.*m14*m44 + 1./54.*
         m14*m44*RS456pe + 1./18.*pow(m14,2) - 1./54.*pow(m14,2)*
         RS456pe - 5./36.*m11*m44 + 5./108.*m11*m44*RS456pe + 5./18.*
         m11*m14 - 5./54.*m11*m14*RS456pe - 11./72.*pow(m11,2) + 11./
         216.*pow(m11,2)*RS456pe );

      F12p6x13RV +=  + hhdV(1,m14,m14,mp2,m11,L,mu2) * (  - 1./54.*pow(
         mp2,2)*RSp45611e + 1./27.*pow(mp2,2)*RSp561e - 1./54.*pow(
         mp2,2)*RSp564e - 1./54.*m44*mp2*RSp45611e + 1./54.*m44*mp2*
         RSp564e - 1./216.*pow(m44,2)*RSp45611e - 1./108.*pow(m44,2)*
         RSp561e - 1./216.*pow(m44,2)*RSp564e + 1./27.*m14*mp2*
         RSp45611e - 2./27.*m14*mp2*RSp561e + 1./27.*m14*mp2*RSp564e + 
         1./54.*m14*m44*RSp45611e - 1./54.*m14*m44*RSp564e - 1./54.*
         pow(m14,2)*RSp45611e + 1./27.*pow(m14,2)*RSp561e - 1./54.*pow(
         m14,2)*RSp564e + 1./54.*m11*mp2*RSp45611e - 1./9.*m11*mp2*
         RSp561e + 5./54.*m11*mp2*RSp564e + 1./108.*m11*m44*RSp45611e
          - 1./27.*m11*m44*RSp561e - 5./108.*m11*m44*RSp564e - 1./54.*
         m11*m14*RSp45611e + 1./9.*m11*m14*RSp561e - 5./54.*m11*m14*
         RSp564e + 35./216.*pow(m11,2)*RSp45611e - 31./108.*pow(m11,2)*
         RSp561e + 11./216.*pow(m11,2)*RSp564e );

      F12p6x13RV +=  + hhdV(1,m14,m14,me2,m11,L,mu2) * (  - 1./54.*pow(
         me2,2)*RSe45611p + 1./27.*pow(me2,2)*RSe561p - 1./54.*pow(
         me2,2)*RSe564p - 1./54.*m44*me2*RSe45611p + 1./54.*m44*me2*
         RSe564p - 1./216.*pow(m44,2)*RSe45611p - 1./108.*pow(m44,2)*
         RSe561p - 1./216.*pow(m44,2)*RSe564p + 1./27.*m14*me2*
         RSe45611p - 2./27.*m14*me2*RSe561p + 1./27.*m14*me2*RSe564p + 
         1./54.*m14*m44*RSe45611p - 1./54.*m14*m44*RSe564p - 1./54.*
         pow(m14,2)*RSe45611p + 1./27.*pow(m14,2)*RSe561p - 1./54.*pow(
         m14,2)*RSe564p + 1./54.*m11*me2*RSe45611p - 1./9.*m11*me2*
         RSe561p + 5./54.*m11*me2*RSe564p + 1./108.*m11*m44*RSe45611p
          - 1./27.*m11*m44*RSe561p - 5./108.*m11*m44*RSe564p - 1./54.*
         m11*m14*RSe45611p + 1./9.*m11*m14*RSe561p - 5./54.*m11*m14*
         RSe564p + 35./216.*pow(m11,2)*RSe45611p - 31./108.*pow(m11,2)*
         RSe561p + 11./216.*pow(m11,2)*RSe564p );

      F12p6x13RV +=  + hhdV(1,m14,m15,m45,m11,L,mu2) * ( 1./144.*pow(
         m55,2) - 1./18.*m45*m55 + 1./9.*pow(m45,2) + 1./72.*m44*m55 - 
         1./18.*m44*m45 + 1./144.*pow(m44,2) + 1./36.*m15*m55 - 1./9.*
         m15*m45 + 1./36.*m15*m44 + 1./36.*pow(m15,2) + 1./36.*m14*m55
          - 1./9.*m14*m45 + 1./36.*m14*m44 + 1./18.*m14*m15 + 1./36.*
         pow(m14,2) + 5./36.*m11*m55 - 5./9.*m11*m45 + 5./36.*m11*m44
          + 5./18.*m11*m15 + 5./18.*m11*m14 - 11./36.*pow(m11,2) );

      F12p6x13RV +=  + hhdV(1,m14,m16,m46,m11,L,mu2) * ( 1./144.*pow(
         m66,2) - 1./18.*m46*m66 + 1./9.*pow(m46,2) + 1./72.*m44*m66 - 
         1./18.*m44*m46 + 1./144.*pow(m44,2) + 1./36.*m16*m66 - 1./9.*
         m16*m46 + 1./36.*m16*m44 + 1./36.*pow(m16,2) + 1./36.*m14*m66
          - 1./9.*m14*m46 + 1./36.*m14*m44 + 1./18.*m14*m16 + 1./36.*
         pow(m14,2) + 5./36.*m11*m66 - 5./9.*m11*m46 + 5./36.*m11*m44
          + 5./18.*m11*m16 + 5./18.*m11*m14 - 11./36.*pow(m11,2) );

      F12p6x13RV +=  + hhdV(1,m15,m15,m55,m11,L,mu2) * ( 1./72.*pow(
         m55,2) - 1./216.*pow(m55,2)*RS546pe - 1./18.*m15*m55 + 1./54.*
         m15*m55*RS546pe + 1./18.*pow(m15,2) - 1./54.*pow(m15,2)*
         RS546pe - 5./36.*m11*m55 + 5./108.*m11*m55*RS546pe + 5./18.*
         m11*m15 - 5./54.*m11*m15*RS546pe - 11./72.*pow(m11,2) + 11./
         216.*pow(m11,2)*RS546pe );

      F12p6x13RV +=  + hhdV(1,m15,m15,mp2,m11,L,mu2) * (  - 1./54.*pow(
         mp2,2)*RSp45611e + 1./27.*pow(mp2,2)*RSp461e - 1./54.*pow(
         mp2,2)*RSp465e - 1./54.*m55*mp2*RSp45611e + 1./54.*m55*mp2*
         RSp465e - 1./216.*pow(m55,2)*RSp45611e - 1./108.*pow(m55,2)*
         RSp461e - 1./216.*pow(m55,2)*RSp465e + 1./27.*m15*mp2*
         RSp45611e - 2./27.*m15*mp2*RSp461e + 1./27.*m15*mp2*RSp465e + 
         1./54.*m15*m55*RSp45611e - 1./54.*m15*m55*RSp465e - 1./54.*
         pow(m15,2)*RSp45611e + 1./27.*pow(m15,2)*RSp461e - 1./54.*pow(
         m15,2)*RSp465e + 1./54.*m11*mp2*RSp45611e - 1./9.*m11*mp2*
         RSp461e + 5./54.*m11*mp2*RSp465e + 1./108.*m11*m55*RSp45611e
          - 1./27.*m11*m55*RSp461e - 5./108.*m11*m55*RSp465e - 1./54.*
         m11*m15*RSp45611e + 1./9.*m11*m15*RSp461e - 5./54.*m11*m15*
         RSp465e + 35./216.*pow(m11,2)*RSp45611e - 31./108.*pow(m11,2)*
         RSp461e + 11./216.*pow(m11,2)*RSp465e );

      F12p6x13RV +=  + hhdV(1,m15,m15,me2,m11,L,mu2) * (  - 1./54.*pow(
         me2,2)*RSe45611p + 1./27.*pow(me2,2)*RSe461p - 1./54.*pow(
         me2,2)*RSe465p - 1./54.*m55*me2*RSe45611p + 1./54.*m55*me2*
         RSe465p - 1./216.*pow(m55,2)*RSe45611p - 1./108.*pow(m55,2)*
         RSe461p - 1./216.*pow(m55,2)*RSe465p + 1./27.*m15*me2*
         RSe45611p - 2./27.*m15*me2*RSe461p + 1./27.*m15*me2*RSe465p + 
         1./54.*m15*m55*RSe45611p - 1./54.*m15*m55*RSe465p - 1./54.*
         pow(m15,2)*RSe45611p + 1./27.*pow(m15,2)*RSe461p - 1./54.*pow(
         m15,2)*RSe465p + 1./54.*m11*me2*RSe45611p - 1./9.*m11*me2*
         RSe461p + 5./54.*m11*me2*RSe465p + 1./108.*m11*m55*RSe45611p
          - 1./27.*m11*m55*RSe461p - 5./108.*m11*m55*RSe465p - 1./54.*
         m11*m15*RSe45611p + 1./9.*m11*m15*RSe461p - 5./54.*m11*m15*
         RSe465p + 35./216.*pow(m11,2)*RSe45611p - 31./108.*pow(m11,2)*
         RSe461p + 11./216.*pow(m11,2)*RSe465p );

      F12p6x13RV +=  + hhdV(1,m15,m16,m56,m11,L,mu2) * ( 1./144.*pow(
         m66,2) - 1./18.*m56*m66 + 1./9.*pow(m56,2) + 1./72.*m55*m66 - 
         1./18.*m55*m56 + 1./144.*pow(m55,2) + 1./36.*m16*m66 - 1./9.*
         m16*m56 + 1./36.*m16*m55 + 1./36.*pow(m16,2) + 1./36.*m15*m66
          - 1./9.*m15*m56 + 1./36.*m15*m55 + 1./18.*m15*m16 + 1./36.*
         pow(m15,2) + 5./36.*m11*m66 - 5./9.*m11*m56 + 5./36.*m11*m55
          + 5./18.*m11*m16 + 5./18.*m11*m15 - 11./36.*pow(m11,2) );

      F12p6x13RV +=  + hhdV(1,m16,m16,m66,m11,L,mu2) * ( 1./72.*pow(
         m66,2) - 1./216.*pow(m66,2)*RS645pe - 1./18.*m16*m66 + 1./54.*
         m16*m66*RS645pe + 1./18.*pow(m16,2) - 1./54.*pow(m16,2)*
         RS645pe - 5./36.*m11*m66 + 5./108.*m11*m66*RS645pe + 5./18.*
         m11*m16 - 5./54.*m11*m16*RS645pe - 11./72.*pow(m11,2) + 11./
         216.*pow(m11,2)*RS645pe );

      F12p6x13RV +=  + hhdV(1,m16,m16,mp2,m11,L,mu2) * (  - 1./54.*pow(
         mp2,2)*RSp45611e + 1./27.*pow(mp2,2)*RSp451e - 1./54.*pow(
         mp2,2)*RSp456e - 1./54.*m66*mp2*RSp45611e + 1./54.*m66*mp2*
         RSp456e - 1./216.*pow(m66,2)*RSp45611e - 1./108.*pow(m66,2)*
         RSp451e - 1./216.*pow(m66,2)*RSp456e + 1./27.*m16*mp2*
         RSp45611e - 2./27.*m16*mp2*RSp451e + 1./27.*m16*mp2*RSp456e + 
         1./54.*m16*m66*RSp45611e - 1./54.*m16*m66*RSp456e - 1./54.*
         pow(m16,2)*RSp45611e + 1./27.*pow(m16,2)*RSp451e - 1./54.*pow(
         m16,2)*RSp456e + 1./54.*m11*mp2*RSp45611e - 1./9.*m11*mp2*
         RSp451e + 5./54.*m11*mp2*RSp456e + 1./108.*m11*m66*RSp45611e
          - 1./27.*m11*m66*RSp451e - 5./108.*m11*m66*RSp456e - 1./54.*
         m11*m16*RSp45611e + 1./9.*m11*m16*RSp451e - 5./54.*m11*m16*
         RSp456e + 35./216.*pow(m11,2)*RSp45611e - 31./108.*pow(m11,2)*
         RSp451e + 11./216.*pow(m11,2)*RSp456e );

      F12p6x13RV +=  + hhdV(1,m16,m16,me2,m11,L,mu2) * (  - 1./54.*pow(
         me2,2)*RSe45611p + 1./27.*pow(me2,2)*RSe451p - 1./54.*pow(
         me2,2)*RSe456p - 1./54.*m66*me2*RSe45611p + 1./54.*m66*me2*
         RSe456p - 1./216.*pow(m66,2)*RSe45611p - 1./108.*pow(m66,2)*
         RSe451p - 1./216.*pow(m66,2)*RSe456p + 1./27.*m16*me2*
         RSe45611p - 2./27.*m16*me2*RSe451p + 1./27.*m16*me2*RSe456p + 
         1./54.*m16*m66*RSe45611p - 1./54.*m16*m66*RSe456p - 1./54.*
         pow(m16,2)*RSe45611p + 1./27.*pow(m16,2)*RSe451p - 1./54.*pow(
         m16,2)*RSe456p + 1./54.*m11*me2*RSe45611p - 1./9.*m11*me2*
         RSe451p + 5./54.*m11*me2*RSe456p + 1./108.*m11*m66*RSe45611p
          - 1./27.*m11*m66*RSe451p - 5./108.*m11*m66*RSe456p - 1./54.*
         m11*m16*RSe45611p + 1./9.*m11*m16*RSe451p - 5./54.*m11*m16*
         RSe456p + 35./216.*pow(m11,2)*RSe45611p - 31./108.*pow(m11,2)*
         RSe451p + 11./216.*pow(m11,2)*RSe456p );

      F12p6x13RV +=  + hhdV(1,m44,m14,m14,m11,L,mu2) * ( 1./8.*m11*m44
          - 1./24.*m11*m44*RS456pe );

      F12p6x13RV +=  + hhdV(1,m45,m14,m15,m11,L,mu2) * ( 1./4.*m11*m45
          );

      F12p6x13RV +=  + hhdV(1,m46,m14,m16,m11,L,mu2) * ( 1./4.*m11*m46
          );

      F12p6x13RV +=  + hhdV(1,m55,m15,m15,m11,L,mu2) * ( 1./8.*m11*m55
          - 1./24.*m11*m55*RS546pe );

      F12p6x13RV +=  + hhdV(1,m56,m15,m16,m11,L,mu2) * ( 1./4.*m11*m56
          );

      F12p6x13RV +=  + hhdV(1,m66,m16,m16,m11,L,mu2) * ( 1./8.*m11*m66
          - 1./24.*m11*m66*RS645pe );

      F12p6x13RV +=  + hhdV(1,mp2,m14,m14,m11,L,mu2) * (  - 1./24.*m11*
         mp2*RSp45611e + 1./12.*m11*mp2*RSp561e - 1./24.*m11*mp2*
         RSp564e );

      F12p6x13RV +=  + hhdV(1,mp2,m15,m15,m11,L,mu2) * (  - 1./24.*m11*
         mp2*RSp45611e + 1./12.*m11*mp2*RSp461e - 1./24.*m11*mp2*
         RSp465e );

      F12p6x13RV +=  + hhdV(1,mp2,m16,m16,m11,L,mu2) * (  - 1./24.*m11*
         mp2*RSp45611e + 1./12.*m11*mp2*RSp451e - 1./24.*m11*mp2*
         RSp456e );

      F12p6x13RV +=  + hhdV(1,me2,m14,m14,m11,L,mu2) * (  - 1./24.*m11*
         me2*RSe45611p + 1./12.*m11*me2*RSe561p - 1./24.*m11*me2*
         RSe564p );

      F12p6x13RV +=  + hhdV(1,me2,m15,m15,m11,L,mu2) * (  - 1./24.*m11*
         me2*RSe45611p + 1./12.*m11*me2*RSe461p - 1./24.*m11*me2*
         RSe465p );

      F12p6x13RV +=  + hhdV(1,me2,m16,m16,m11,L,mu2) * (  - 1./24.*m11*
         me2*RSe45611p + 1./12.*m11*me2*RSe451p - 1./24.*m11*me2*
         RSe456p );

      F12p6x13RV +=  + hhdV(2,m11,m11,m11,m11,L,mu2) * ( 2./9.*pow(
         m11,2)*RD1456ep*RC1456pe );

      F12p6x13RV +=  + hhdV(2,m11,m11,mp2,m11,L,mu2) * ( 2./9.*pow(
         m11,2)*RSp45611e*RD1456ep );

      F12p6x13RV +=  + hhdV(2,m11,m11,me2,m11,L,mu2) * ( 2./9.*pow(
         m11,2)*RSe45611p*RD1456ep );

      F12p6x13RV +=  + hhdV(2,m11,m14,m14,m11,L,mu2) * (  - 1./216.*
         pow(m44,2)*RD1456ep + 1./54.*m14*m44*RD1456ep - 1./54.*pow(
         m14,2)*RD1456ep - 7./108.*m11*m44*RD1456ep + 7./54.*m11*m14*
         RD1456ep - 29./108.*pow(m11,2)*RD1456ep );

      F12p6x13RV +=  + hhdV(2,m11,m15,m15,m11,L,mu2) * (  - 1./216.*
         pow(m55,2)*RD1456ep + 1./54.*m15*m55*RD1456ep - 1./54.*pow(
         m15,2)*RD1456ep - 7./108.*m11*m55*RD1456ep + 7./54.*m11*m15*
         RD1456ep - 29./108.*pow(m11,2)*RD1456ep );

      F12p6x13RV +=  + hhdV(2,m11,m16,m16,m11,L,mu2) * (  - 1./216.*
         pow(m66,2)*RD1456ep + 1./54.*m16*m66*RD1456ep - 1./54.*pow(
         m16,2)*RD1456ep - 7./108.*m11*m66*RD1456ep + 7./54.*m11*m16*
         RD1456ep - 29./108.*pow(m11,2)*RD1456ep );

      F12p6x13RV +=  + hhdV(5,m11,m11,m11,m11,L,mu2) * ( 1./9.*pow(
         m11,2)*pow(RD1456ep,2) );

      F12p6x13RV +=  + hh1dV(1,m11,m14,m14,m11,L,mu2) * ( 1./9.*m11*
         RD1456ep + 1./18.*m11*m44*RC1456pe - 1./9.*m11*m14*RC1456pe + 
         2./9.*m11*m14*RS156pe + 7./18.*pow(m11,2)*RC1456pe - 5./9.*
         pow(m11,2)*RS156pe );

      F12p6x13RV +=  + hh1dV(1,m11,m15,m15,m11,L,mu2) * ( 1./9.*m11*
         RD1456ep + 1./18.*m11*m55*RC1456pe - 1./9.*m11*m15*RC1456pe + 
         2./9.*m11*m15*RS146pe + 7./18.*pow(m11,2)*RC1456pe - 5./9.*
         pow(m11,2)*RS146pe );

      F12p6x13RV +=  + hh1dV(1,m11,m16,m16,m11,L,mu2) * ( 1./9.*m11*
         RD1456ep + 1./18.*m11*m66*RC1456pe - 1./9.*m11*m16*RC1456pe + 
         2./9.*m11*m16*RS145pe + 7./18.*pow(m11,2)*RC1456pe - 5./9.*
         pow(m11,2)*RS145pe );

      F12p6x13RV +=  + hh1dV(1,m14,m14,m44,m11,L,mu2) * ( 1./3.*m11*m44
          - 1./9.*m11*m44*RS456pe - 2./3.*m11*m14 + 2./9.*m11*m14*
         RS456pe + 1./3.*pow(m11,2) - 1./9.*pow(m11,2)*RS456pe );

      F12p6x13RV +=  + hh1dV(1,m14,m14,mp2,m11,L,mu2) * (  - 2./9.*m11*
         mp2*RSp45611e + 4./9.*m11*mp2*RSp561e - 2./9.*m11*mp2*RSp564e
          - 1./9.*m11*m44*RSp45611e + 1./9.*m11*m44*RSp564e + 2./9.*m11
         *m14*RSp45611e - 4./9.*m11*m14*RSp561e + 2./9.*m11*m14*RSp564e
          - 5./9.*pow(m11,2)*RSp45611e + 2./3.*pow(m11,2)*RSp561e - 1./
         9.*pow(m11,2)*RSp564e );

      F12p6x13RV +=  + hh1dV(1,m14,m14,me2,m11,L,mu2) * (  - 2./9.*m11*
         me2*RSe45611p + 4./9.*m11*me2*RSe561p - 2./9.*m11*me2*RSe564p
          - 1./9.*m11*m44*RSe45611p + 1./9.*m11*m44*RSe564p + 2./9.*m11
         *m14*RSe45611p - 4./9.*m11*m14*RSe561p + 2./9.*m11*m14*RSe564p
          - 5./9.*pow(m11,2)*RSe45611p + 2./3.*pow(m11,2)*RSe561p - 1./
         9.*pow(m11,2)*RSe564p );

      F12p6x13RV +=  + hh1dV(1,m14,m15,m45,m11,L,mu2) * (  - 1./6.*m11*
         m55 + 2./3.*m11*m45 - 1./6.*m11*m44 - 1./3.*m11*m15 - 1./3.*
         m11*m14 + 1./3.*pow(m11,2) );

      F12p6x13RV +=  + hh1dV(1,m14,m16,m46,m11,L,mu2) * (  - 1./6.*m11*
         m66 + 2./3.*m11*m46 - 1./6.*m11*m44 - 1./3.*m11*m16 - 1./3.*
         m11*m14 + 1./3.*pow(m11,2) );

      F12p6x13RV +=  + hh1dV(1,m15,m14,m45,m11,L,mu2) * (  - 1./6.*m11*
         m55 + 2./3.*m11*m45 - 1./6.*m11*m44 - 1./3.*m11*m15 - 1./3.*
         m11*m14 + 1./3.*pow(m11,2) );

      F12p6x13RV +=  + hh1dV(1,m15,m15,m55,m11,L,mu2) * ( 1./3.*m11*m55
          - 1./9.*m11*m55*RS546pe - 2./3.*m11*m15 + 2./9.*m11*m15*
         RS546pe + 1./3.*pow(m11,2) - 1./9.*pow(m11,2)*RS546pe );

      F12p6x13RV +=  + hh1dV(1,m15,m15,mp2,m11,L,mu2) * (  - 2./9.*m11*
         mp2*RSp45611e + 4./9.*m11*mp2*RSp461e - 2./9.*m11*mp2*RSp465e
          - 1./9.*m11*m55*RSp45611e + 1./9.*m11*m55*RSp465e + 2./9.*m11
         *m15*RSp45611e - 4./9.*m11*m15*RSp461e + 2./9.*m11*m15*RSp465e
          - 5./9.*pow(m11,2)*RSp45611e + 2./3.*pow(m11,2)*RSp461e - 1./
         9.*pow(m11,2)*RSp465e );

      F12p6x13RV +=  + hh1dV(1,m15,m15,me2,m11,L,mu2) * (  - 2./9.*m11*
         me2*RSe45611p + 4./9.*m11*me2*RSe461p - 2./9.*m11*me2*RSe465p
          - 1./9.*m11*m55*RSe45611p + 1./9.*m11*m55*RSe465p + 2./9.*m11
         *m15*RSe45611p - 4./9.*m11*m15*RSe461p + 2./9.*m11*m15*RSe465p
          - 5./9.*pow(m11,2)*RSe45611p + 2./3.*pow(m11,2)*RSe461p - 1./
         9.*pow(m11,2)*RSe465p );

      F12p6x13RV +=  + hh1dV(1,m15,m16,m56,m11,L,mu2) * (  - 1./6.*m11*
         m66 + 2./3.*m11*m56 - 1./6.*m11*m55 - 1./3.*m11*m16 - 1./3.*
         m11*m15 + 1./3.*pow(m11,2) );

      F12p6x13RV +=  + hh1dV(1,m16,m14,m46,m11,L,mu2) * (  - 1./6.*m11*
         m66 + 2./3.*m11*m46 - 1./6.*m11*m44 - 1./3.*m11*m16 - 1./3.*
         m11*m14 + 1./3.*pow(m11,2) );

      F12p6x13RV +=  + hh1dV(1,m16,m15,m56,m11,L,mu2) * (  - 1./6.*m11*
         m66 + 2./3.*m11*m56 - 1./6.*m11*m55 - 1./3.*m11*m16 - 1./3.*
         m11*m15 + 1./3.*pow(m11,2) );

      F12p6x13RV +=  + hh1dV(1,m16,m16,m66,m11,L,mu2) * ( 1./3.*m11*m66
          - 1./9.*m11*m66*RS645pe - 2./3.*m11*m16 + 2./9.*m11*m16*
         RS645pe + 1./3.*pow(m11,2) - 1./9.*pow(m11,2)*RS645pe );

      F12p6x13RV +=  + hh1dV(1,m16,m16,mp2,m11,L,mu2) * (  - 2./9.*m11*
         mp2*RSp45611e + 4./9.*m11*mp2*RSp451e - 2./9.*m11*mp2*RSp456e
          - 1./9.*m11*m66*RSp45611e + 1./9.*m11*m66*RSp456e + 2./9.*m11
         *m16*RSp45611e - 4./9.*m11*m16*RSp451e + 2./9.*m11*m16*RSp456e
          - 5./9.*pow(m11,2)*RSp45611e + 2./3.*pow(m11,2)*RSp451e - 1./
         9.*pow(m11,2)*RSp456e );

      F12p6x13RV +=  + hh1dV(1,m16,m16,me2,m11,L,mu2) * (  - 2./9.*m11*
         me2*RSe45611p + 4./9.*m11*me2*RSe451p - 2./9.*m11*me2*RSe456p
          - 1./9.*m11*m66*RSe45611p + 1./9.*m11*m66*RSe456p + 2./9.*m11
         *m16*RSe45611p - 4./9.*m11*m16*RSe451p + 2./9.*m11*m16*RSe456p
          - 5./9.*pow(m11,2)*RSe45611p + 2./3.*pow(m11,2)*RSe451p - 1./
         9.*pow(m11,2)*RSe456p );

      F12p6x13RV +=  + hh1dV(2,m11,m14,m14,m11,L,mu2) * ( 1./18.*m11*
         m44*RD1456ep - 1./9.*m11*m14*RD1456ep + 7./18.*pow(m11,2)*
         RD1456ep );

      F12p6x13RV +=  + hh1dV(2,m11,m15,m15,m11,L,mu2) * ( 1./18.*m11*
         m55*RD1456ep - 1./9.*m11*m15*RD1456ep + 7./18.*pow(m11,2)*
         RD1456ep );

      F12p6x13RV +=  + hh1dV(2,m11,m16,m16,m11,L,mu2) * ( 1./18.*m11*
         m66*RD1456ep - 1./9.*m11*m16*RD1456ep + 7./18.*pow(m11,2)*
         RD1456ep );

      F12p6x13RV +=  + hh21dV(1,m11,m14,m14,m11,L,mu2) * (  - 1./8.*
         pow(m11,2)*RC1456pe + 1./4.*pow(m11,2)*RS156pe );

      F12p6x13RV +=  + hh21dV(1,m11,m15,m15,m11,L,mu2) * (  - 1./8.*
         pow(m11,2)*RC1456pe + 1./4.*pow(m11,2)*RS146pe );

      F12p6x13RV +=  + hh21dV(1,m11,m16,m16,m11,L,mu2) * (  - 1./8.*
         pow(m11,2)*RC1456pe + 1./4.*pow(m11,2)*RS145pe );

      F12p6x13RV +=  + hh21dV(1,m44,m14,m14,m11,L,mu2) * ( 3./8.*pow(
         m11,2) - 1./8.*pow(m11,2)*RS456pe );

      F12p6x13RV +=  + hh21dV(1,m45,m14,m15,m11,L,mu2) * ( 3./4.*pow(
         m11,2) );

      F12p6x13RV +=  + hh21dV(1,m46,m14,m16,m11,L,mu2) * ( 3./4.*pow(
         m11,2) );

      F12p6x13RV +=  + hh21dV(1,m55,m15,m15,m11,L,mu2) * ( 3./8.*pow(
         m11,2) - 1./8.*pow(m11,2)*RS546pe );

      F12p6x13RV +=  + hh21dV(1,m56,m15,m16,m11,L,mu2) * ( 3./4.*pow(
         m11,2) );

      F12p6x13RV +=  + hh21dV(1,m66,m16,m16,m11,L,mu2) * ( 3./8.*pow(
         m11,2) - 1./8.*pow(m11,2)*RS645pe );

      F12p6x13RV +=  + hh21dV(1,mp2,m14,m14,m11,L,mu2) * (  - 1./8.*
         pow(m11,2)*RSp45611e + 1./4.*pow(m11,2)*RSp561e - 1./8.*pow(
         m11,2)*RSp564e );

      F12p6x13RV +=  + hh21dV(1,mp2,m15,m15,m11,L,mu2) * (  - 1./8.*
         pow(m11,2)*RSp45611e + 1./4.*pow(m11,2)*RSp461e - 1./8.*pow(
         m11,2)*RSp465e );

      F12p6x13RV +=  + hh21dV(1,mp2,m16,m16,m11,L,mu2) * (  - 1./8.*
         pow(m11,2)*RSp45611e + 1./4.*pow(m11,2)*RSp451e - 1./8.*pow(
         m11,2)*RSp456e );

      F12p6x13RV +=  + hh21dV(1,me2,m14,m14,m11,L,mu2) * (  - 1./8.*
         pow(m11,2)*RSe45611p + 1./4.*pow(m11,2)*RSe561p - 1./8.*pow(
         m11,2)*RSe564p );

      F12p6x13RV +=  + hh21dV(1,me2,m15,m15,m11,L,mu2) * (  - 1./8.*
         pow(m11,2)*RSe45611p + 1./4.*pow(m11,2)*RSe461p - 1./8.*pow(
         m11,2)*RSe465p );

      F12p6x13RV +=  + hh21dV(1,me2,m16,m16,m11,L,mu2) * (  - 1./8.*
         pow(m11,2)*RSe45611p + 1./4.*pow(m11,2)*RSe451p - 1./8.*pow(
         m11,2)*RSe456p );

      F12p6x13RV +=  + hh21dV(2,m11,m14,m14,m11,L,mu2) * (  - 1./8.*
         pow(m11,2)*RD1456ep );

      F12p6x13RV +=  + hh21dV(2,m11,m15,m15,m11,L,mu2) * (  - 1./8.*
         pow(m11,2)*RD1456ep );

      F12p6x13RV +=  + hh21dV(2,m11,m16,m16,m11,L,mu2) * (  - 1./8.*
         pow(m11,2)*RD1456ep );

      F12p6x13RV +=  + hh27dV(1,m11,m14,m14,m11,L,mu2) * ( 1./8.*m11*
         RC1456pe - 1./4.*m11*RS156pe );

      F12p6x13RV +=  + hh27dV(1,m11,m15,m15,m11,L,mu2) * ( 1./8.*m11*
         RC1456pe - 1./4.*m11*RS146pe );

      F12p6x13RV +=  + hh27dV(1,m11,m16,m16,m11,L,mu2) * ( 1./8.*m11*
         RC1456pe - 1./4.*m11*RS145pe );

      F12p6x13RV +=  + hh27dV(1,m44,m14,m14,m11,L,mu2) * (  - 3./8.*m11
          + 1./8.*m11*RS456pe );

      F12p6x13RV +=  + hh27dV(1,m45,m14,m15,m11,L,mu2) * (  - 3./4.*m11
          );

      F12p6x13RV +=  + hh27dV(1,m46,m14,m16,m11,L,mu2) * (  - 3./4.*m11
          );

      F12p6x13RV +=  + hh27dV(1,m55,m15,m15,m11,L,mu2) * (  - 3./8.*m11
          + 1./8.*m11*RS546pe );

      F12p6x13RV +=  + hh27dV(1,m56,m15,m16,m11,L,mu2) * (  - 3./4.*m11
          );

      F12p6x13RV +=  + hh27dV(1,m66,m16,m16,m11,L,mu2) * (  - 3./8.*m11
          + 1./8.*m11*RS645pe );

      F12p6x13RV +=  + hh27dV(1,mp2,m14,m14,m11,L,mu2) * ( 1./8.*m11*
         RSp45611e - 1./4.*m11*RSp561e + 1./8.*m11*RSp564e );

      F12p6x13RV +=  + hh27dV(1,mp2,m15,m15,m11,L,mu2) * ( 1./8.*m11*
         RSp45611e - 1./4.*m11*RSp461e + 1./8.*m11*RSp465e );

      F12p6x13RV +=  + hh27dV(1,mp2,m16,m16,m11,L,mu2) * ( 1./8.*m11*
         RSp45611e - 1./4.*m11*RSp451e + 1./8.*m11*RSp456e );

      F12p6x13RV +=  + hh27dV(1,me2,m14,m14,m11,L,mu2) * ( 1./8.*m11*
         RSe45611p - 1./4.*m11*RSe561p + 1./8.*m11*RSe564p );

      F12p6x13RV +=  + hh27dV(1,me2,m15,m15,m11,L,mu2) * ( 1./8.*m11*
         RSe45611p - 1./4.*m11*RSe461p + 1./8.*m11*RSe465p );

      F12p6x13RV +=  + hh27dV(1,me2,m16,m16,m11,L,mu2) * ( 1./8.*m11*
         RSe45611p - 1./4.*m11*RSe451p + 1./8.*m11*RSe456p );

      F12p6x13RV +=  + hh27dV(2,m11,m14,m14,m11,L,mu2) * ( 1./8.*m11*
         RD1456ep );

      F12p6x13RV +=  + hh27dV(2,m11,m15,m15,m11,L,mu2) * ( 1./8.*m11*
         RD1456ep );

      F12p6x13RV +=  + hh27dV(2,m11,m16,m16,m11,L,mu2) * ( 1./8.*m11*
         RD1456ep );

  return F12p6x13RV/pow(f0,4);
}
// 2 valence mass 3 sea mass *****************************************

double fv2s3nf3p4V(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 5) || (B0mq.size() != 5)){
    cout << "ERROR: fv2s3nf3p4V probably called with wrong number"
	 << "of quark masses\n";}
  // five masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m55 = 2.*B0mq[3];
  double m66 = 2.*B0mq[4];
  double m14 = (m11+m44)/2;
  double m15 = (m11+m55)/2;
  double m16 = (m11+m66)/2;
  double m34 = (m33+m44)/2;
  double m35 = (m33+m55)/2;
  double m36 = (m33+m66)/2;

  double xb1 = (m44+m55+m66)/3.;
  double msqr = sqrt(xb1*xb1-(m44*m55+m44*m66+m55*m66)/3.);
  double mp2 = xb1-msqr; 
  double me2 = xb1+msqr;
  //double mu2 = mu0*mu0;
  double RSe45611p = RS(me2,m44,m55,m66,m11,m11,mp2);
  double RSp45611e = RS(mp2,m44,m55,m66,m11,m11,me2);
  double RSe45613p = RS(me2,m44,m55,m66,m11,m33,mp2);
  double RSp45613e = RS(mp2,m44,m55,m66,m11,m33,me2);
  double RSe45633p = RS(me2,m44,m55,m66,m33,m33,mp2);
  double RSp45633e = RS(mp2,m44,m55,m66,m33,m33,me2);
  double RS14563pe = RS(m11,m44,m55,m66,m33,mp2,me2);
  double RS34561pe = RS(m33,m44,m55,m66,m11,mp2,me2);

  double RC1456pe = RC(m11,m44,m55,m66,mp2,me2);
  double RC3456pe = RC(m33,m44,m55,m66,mp2,me2);
  double RD1456ep = RD(m11,m44,m55,m66,me2,mp2);
  double RD3456ep = RD(m33,m44,m55,m66,me2,mp2);

  double F13p4x23V =
       + AbV(m11,L) * ( 1./6.*RS14563pe - 1./12.*RC1456pe );

      F13p4x23V +=  + AbV(m14,L) * ( 1./4. );

      F13p4x23V +=  + AbV(m15,L) * ( 1./4. );

      F13p4x23V +=  + AbV(m16,L) * ( 1./4. );

      F13p4x23V +=  + AbV(m33,L) * ( 1./6.*RS34561pe - 1./12.*RC3456pe
          );

      F13p4x23V +=  + AbV(m34,L) * ( 1./4. );

      F13p4x23V +=  + AbV(m35,L) * ( 1./4. );

      F13p4x23V +=  + AbV(m36,L) * ( 1./4. );

      F13p4x23V +=  + AbV(mp2,L) * ( 1./6.*RSp45613e - 1./12.*RSp45633e
          - 1./12.*RSp45611e );

      F13p4x23V +=  + AbV(me2,L) * ( 1./6.*RSe45613p - 1./12.*RSe45633p
          - 1./12.*RSe45611p );

      F13p4x23V +=  + BbV(m11,L) * (  - 1./12.*RD1456ep );

      F13p4x23V +=  + BbV(m33,L) * (  - 1./12.*RD3456ep );

  return F13p4x23V/pow(f0,2);
}

double fv2s3nf3p6V(const quarkmassnf mass, const Linf Liin, const double L){
  return fv2s3nf3p6LV(mass,Liin,L)+fv2s3nf3p6RV(mass,L);
}

double fv2s3nf3p6LV(const quarkmassnf mass, const Linf Liin, const double L){
  double L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu;
  int nfLi;
  Liin.out(L0r,L1r,L2r,L3r,L4r,L5r,L6r,L7r,L8r,L9r,L10r,L11r,H1r,H2r,mu,nfLi);
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if (mu0 != mu){
    cout << "ERROR: differing mu's in fv2s3nf3p6LV\n";}
  if ( (nq != 5) || (B0mq.size() != 5)){
    cout << "ERROR: fv2s3nf3p6LV probably called with wrong number"
	 << "of quark masses\n";}
  // four masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m55 = 2.*B0mq[3];
  double m66 = 2.*B0mq[4];
  double m13 = (m11+m33)/2;
  double m14 = (m11+m44)/2;
  double m15 = (m11+m55)/2;
  double m16 = (m11+m66)/2;
  double m34 = (m33+m44)/2;
  double m35 = (m33+m55)/2;
  double m36 = (m33+m66)/2;
  double m45 = (m44+m55)/2;
  double m46 = (m44+m66)/2;
  double m56 = (m55+m66)/2;

  double xb1 = (m44+m55+m66)/3.;
  double msqr = sqrt(xb1*xb1-(m44*m55+m44*m66+m55*m66)/3.);
  double mp2 = xb1-msqr; 
  double me2 = xb1+msqr;
  //double mu2 = mu0*mu0;

  //double RN13 = RN(m11,m33);
  //double RN1p = RN(m11,mp2);
  //double RN1e = RN(m11,me2);
  //double RN3p = RN(m33,mp2);
  //double RN3e = RN(m33,me2);
  //double RNep = RN(me2,mp2);

  double RSp456e = RS(mp2,m44,m55,m66,me2);
  double RSp465e = RS(mp2,m44,m66,m55,me2);
  double RSp564e = RS(mp2,m55,m66,m44,me2);
  double RSe456p = RS(me2,m44,m55,m66,mp2);
  double RSe465p = RS(me2,m44,m66,m55,mp2);
  double RSe564p = RS(me2,m55,m66,m44,mp2);
  double RSp451e = RS(mp2,m44,m55,m11,me2);
  double RSp461e = RS(mp2,m44,m66,m11,me2);
  double RSp561e = RS(mp2,m55,m66,m11,me2);
  double RSe451p = RS(me2,m44,m55,m11,mp2);
  double RSe461p = RS(me2,m44,m66,m11,mp2);
  double RSe561p = RS(me2,m55,m66,m11,mp2);
  double RSp453e = RS(mp2,m44,m55,m33,me2);
  double RSp463e = RS(mp2,m44,m66,m33,me2);
  double RSp563e = RS(mp2,m55,m66,m33,me2);
  double RSe453p = RS(me2,m44,m55,m33,mp2);
  double RSe463p = RS(me2,m44,m66,m33,mp2);
  double RSe563p = RS(me2,m55,m66,m33,mp2);
  double RS456pe = RS(m44,m55,m66,mp2,me2);
  double RS546pe = RS(m55,m44,m66,mp2,me2);
  double RS645pe = RS(m66,m44,m55,mp2,me2);

  double RS156pe = RS(m11,m55,m66,mp2,me2);
  double RS146pe = RS(m11,m44,m66,mp2,me2);
  double RS145pe = RS(m11,m44,m55,mp2,me2);
  double RS356pe = RS(m33,m55,m66,mp2,me2);
  double RS346pe = RS(m33,m44,m66,mp2,me2);
  double RS345pe = RS(m33,m44,m55,mp2,me2);

  double RD1456ep = RD(m11,m44,m55,m66,me2,mp2);
  double RD3456ep = RD(m33,m44,m55,m66,me2,mp2);

  //double RS1456ppe = RS(m11,m44,m55,m66,mp2,mp2,me2);
  //double RS1456pee = RS(m11,m44,m55,m66,mp2,me2,me2);
  double RSe45611p = RS(me2,m44,m55,m66,m11,m11,mp2);
  double RSp45611e = RS(mp2,m44,m55,m66,m11,m11,me2);
  double RSe45633p = RS(me2,m44,m55,m66,m33,m33,mp2);
  double RSp45633e = RS(mp2,m44,m55,m66,m33,m33,me2);
  //double RS3456ppe = RS(m33,m44,m55,m66,mp2,mp2,me2);
  //double RS3456pee = RS(m33,m44,m55,m66,mp2,me2,me2);
  double RSp45613e = RS(mp2,m44,m55,m66,m11,m33,me2);
  double RSe45613p = RS(me2,m44,m55,m66,m11,m33,mp2);
  double RS14563pe = RS(m11,m44,m55,m66,m33,mp2,me2);
  double RS34561pe = RS(m33,m44,m55,m66,m11,mp2,me2);

  double RC1456pe = RC(m11,m44,m55,m66,mp2,me2);
  double RC3456pe = RC(m33,m44,m55,m66,mp2,me2);

  double F13p6x23LV =
       + AbV(m11,L) * ( 5./3.*L3r*RD1456ep + 2./3.*L0r*RD1456ep - 2./9.
         *m66*L5r*pow(RS145pe,2) - 2./3.*m66*L4r*RS14563pe + m66*L4r*
         RC1456pe - 2./9.*m66*L4r*pow(RS145pe,2) - 2./9.*m66*L4r*pow(
         RS146pe,2) - 2./9.*m66*L4r*pow(RS156pe,2) - 2./9.*m55*L5r*pow(
         RS146pe,2) - 2./3.*m55*L4r*RS14563pe + m55*L4r*RC1456pe - 2./9.
         *m55*L4r*pow(RS145pe,2) - 2./9.*m55*L4r*pow(RS146pe,2) - 2./9.
         *m55*L4r*pow(RS156pe,2) - 2./9.*m44*L5r*pow(RS156pe,2) - 2./3.
         *m44*L4r*RS14563pe + m44*L4r*RC1456pe - 2./9.*m44*L4r*pow(
         RS145pe,2) - 2./9.*m44*L4r*pow(RS146pe,2) - 2./9.*m44*L4r*pow(
         RS156pe,2) + 1./3.*m33*L5r*RS14563pe - 1./6.*m33*L5r*RC1456pe
          - m11*L5r*RS14563pe + 1./2.*m11*L5r*RC1456pe + 2./3.*m11*L3r*
         RS14563pe + 5./3.*m11*L3r*RC1456pe + 8./3.*m11*L0r*RS14563pe
          + 2./3.*m11*L0r*RC1456pe );

      F13p6x23LV +=  + AbV(m13,L) * (  - 10*m13*L2r - 4*m13*L1r );

      F13p6x23LV +=  + AbV(m14,L) * (  - m66*L4r - m55*L4r - m44*L4r + 
         1./2.*m33*L5r - 5*m14*L3r - 2*m14*L0r + 1./2.*m11*L5r );

      F13p6x23LV +=  + AbV(m15,L) * (  - m66*L4r - m55*L4r - m44*L4r + 
         1./2.*m33*L5r - 5*m15*L3r - 2*m15*L0r + 1./2.*m11*L5r );

      F13p6x23LV +=  + AbV(m16,L) * (  - m66*L4r - m55*L4r - m44*L4r + 
         1./2.*m33*L5r - 5*m16*L3r - 2*m16*L0r + 1./2.*m11*L5r );

      F13p6x23LV +=  + AbV(m33,L) * ( 5./3.*L3r*RD3456ep + 2./3.*L0r*
         RD3456ep - 2./9.*m66*L5r*pow(RS345pe,2) + 4./9.*m66*L5r*
         RS145pe*RS345pe - 2*m66*L4r*RS34561pe - 4./3.*m66*L4r*
         RS14563pe + m66*L4r*RC3456pe - 2./9.*m66*L4r*pow(RS345pe,2) - 
         2./9.*m66*L4r*pow(RS346pe,2) - 2./9.*m66*L4r*pow(RS356pe,2) + 
         4./9.*m66*L4r*RS145pe*RS345pe + 4./9.*m66*L4r*RS146pe*RS346pe
          + 4./9.*m66*L4r*RS156pe*RS356pe - 2./9.*m55*L5r*pow(
         RS346pe,2) + 4./9.*m55*L5r*RS146pe*RS346pe - 2*m55*L4r*
         RS34561pe - 4./3.*m55*L4r*RS14563pe + m55*L4r*RC3456pe - 2./9.
         *m55*L4r*pow(RS345pe,2) - 2./9.*m55*L4r*pow(RS346pe,2) - 2./9.
         *m55*L4r*pow(RS356pe,2) + 4./9.*m55*L4r*RS145pe*RS345pe + 4./9.
         *m55*L4r*RS146pe*RS346pe + 4./9.*m55*L4r*RS156pe*RS356pe - 2./
         9.*m44*L5r*pow(RS356pe,2) + 4./9.*m44*L5r*RS156pe*RS356pe - 2*
         m44*L4r*RS34561pe - 4./3.*m44*L4r*RS14563pe + m44*L4r*RC3456pe
          - 2./9.*m44*L4r*pow(RS345pe,2) - 2./9.*m44*L4r*pow(RS346pe,2)
          - 2./9.*m44*L4r*pow(RS356pe,2) );

      F13p6x23LV +=  + AbV(m33,L) * ( 4./9.*m44*L4r*RS145pe*RS345pe + 4.
         /9.*m44*L4r*RS146pe*RS346pe + 4./9.*m44*L4r*RS156pe*RS356pe - 
         m33*L5r*RS34561pe - 4./3.*m33*L5r*RS14563pe + 1./2.*m33*L5r*
         RC3456pe + 2./3.*m33*L3r*RS34561pe + 5./3.*m33*L3r*RC3456pe + 
         8./3.*m33*L0r*RS34561pe + 2./3.*m33*L0r*RC3456pe - m11*L5r*
         RS34561pe - 1./6.*m11*L5r*RC3456pe );

      F13p6x23LV +=  + AbV(m34,L) * (  - m66*L4r - m55*L4r - m44*L4r - 
         5*m34*L3r - 2*m34*L0r + 1./2.*m33*L5r + 1./2.*m11*L5r );

      F13p6x23LV +=  + AbV(m35,L) * (  - m66*L4r - m55*L4r - m44*L4r - 
         5*m35*L3r - 2*m35*L0r + 1./2.*m33*L5r + 1./2.*m11*L5r );

      F13p6x23LV +=  + AbV(m36,L) * (  - m66*L4r - m55*L4r - m44*L4r - 
         5*m36*L3r - 2*m36*L0r + 1./2.*m33*L5r + 1./2.*m11*L5r );

      F13p6x23LV +=  + AbV(m44,L) * ( 4*m44*L4r - 4./3.*m44*L4r*RS456pe
          - 2*m44*L2r + 2./3.*m44*L2r*RS456pe - 8*m44*L1r + 8./3.*m44*
         L1r*RS456pe );

      F13p6x23LV +=  + AbV(m45,L) * ( 4*m55*L4r - 4*m45*L2r - 16*m45*
         L1r + 4*m44*L4r );

      F13p6x23LV +=  + AbV(m46,L) * ( 4*m66*L4r - 4*m46*L2r - 16*m46*
         L1r + 4*m44*L4r );

      F13p6x23LV +=  + AbV(m55,L) * ( 4*m55*L4r - 4./3.*m55*L4r*RS546pe
          - 2*m55*L2r + 2./3.*m55*L2r*RS546pe - 8*m55*L1r + 8./3.*m55*
         L1r*RS546pe );

      F13p6x23LV +=  + AbV(m56,L) * ( 4*m66*L4r - 4*m56*L2r - 16*m56*
         L1r + 4*m55*L4r );

      F13p6x23LV +=  + AbV(m66,L) * ( 4*m66*L4r - 4./3.*m66*L4r*RS645pe
          - 2*m66*L2r + 2./3.*m66*L2r*RS645pe - 8*m66*L1r + 8./3.*m66*
         L1r*RS645pe );

      F13p6x23LV +=  + AbV(mp2,L) * ( 2./3.*mp2*L3r*RSp45613e + 5./3.*
         mp2*L3r*RSp45633e + 5./3.*mp2*L3r*RSp45611e + 2./3.*mp2*L2r*
         RSp564e + 2./3.*mp2*L2r*RSp465e + 2./3.*mp2*L2r*RSp456e + 8./3.
         *mp2*L1r*RSp564e + 8./3.*mp2*L1r*RSp465e + 8./3.*mp2*L1r*
         RSp456e + 8./3.*mp2*L0r*RSp45613e + 2./3.*mp2*L0r*RSp45633e + 
         2./3.*mp2*L0r*RSp45611e - 4./9.*m66*L5r*RSp453e*RS345pe + 4./
         27.*m66*L5r*RSp453e*RSe453p - 2./9.*m66*L5r*pow(RSp453e,2) + 4.
         /9.*m66*L5r*RS145pe*RSp453e - 4./27.*m66*L5r*RSe451p*RSp453e
          + 4./9.*m66*L5r*RSp451e*RS345pe - 4./27.*m66*L5r*RSp451e*
         RSe453p + 4./9.*m66*L5r*RSp451e*RSp453e - 4./9.*m66*L5r*
         RSp451e*RS145pe + 4./27.*m66*L5r*RSp451e*RSe451p - 2./9.*m66*
         L5r*pow(RSp451e,2) - 2*m66*L4r*RSp45613e + m66*L4r*RSp45633e
          - 4./9.*m66*L4r*RSp563e*RS356pe + 4./27.*m66*L4r*RSp563e*
         RSe563p - 2./9.*m66*L4r*pow(RSp563e,2) - 4./9.*m66*L4r*RSp463e
         *RS346pe + 4./27.*m66*L4r*RSp463e*RSe463p - 2./9.*m66*L4r*pow(
         RSp463e,2) );

      F13p6x23LV +=  + AbV(mp2,L) * (  - 4./9.*m66*L4r*RSp453e*RS345pe
          + 4./27.*m66*L4r*RSp453e*RSe453p - 2./9.*m66*L4r*pow(
         RSp453e,2) + m66*L4r*RSp45611e + 4./9.*m66*L4r*RS145pe*RSp453e
          + 4./9.*m66*L4r*RS146pe*RSp463e + 4./9.*m66*L4r*RS156pe*
         RSp563e - 4./27.*m66*L4r*RSe561p*RSp563e - 4./27.*m66*L4r*
         RSe461p*RSp463e - 4./27.*m66*L4r*RSe451p*RSp453e + 4./9.*m66*
         L4r*RSp561e*RS356pe - 4./27.*m66*L4r*RSp561e*RSe563p + 4./9.*
         m66*L4r*RSp561e*RSp563e - 4./9.*m66*L4r*RSp561e*RS156pe + 4./
         27.*m66*L4r*RSp561e*RSe561p - 2./9.*m66*L4r*pow(RSp561e,2) + 4.
         /9.*m66*L4r*RSp461e*RS346pe - 4./27.*m66*L4r*RSp461e*RSe463p
          + 4./9.*m66*L4r*RSp461e*RSp463e - 4./9.*m66*L4r*RSp461e*
         RS146pe + 4./27.*m66*L4r*RSp461e*RSe461p - 2./9.*m66*L4r*pow(
         RSp461e,2) + 4./9.*m66*L4r*RSp451e*RS345pe - 4./27.*m66*L4r*
         RSp451e*RSe453p + 4./9.*m66*L4r*RSp451e*RSp453e - 4./9.*m66*
         L4r*RSp451e*RS145pe + 4./27.*m66*L4r*RSp451e*RSe451p - 2./9.*
         m66*L4r*pow(RSp451e,2) );

      F13p6x23LV +=  + AbV(mp2,L) * (  - 4./3.*m66*L4r*RSp456e - 4./9.*
         m55*L5r*RSp463e*RS346pe + 4./27.*m55*L5r*RSp463e*RSe463p - 2./
         9.*m55*L5r*pow(RSp463e,2) + 4./9.*m55*L5r*RS146pe*RSp463e - 4./
         27.*m55*L5r*RSe461p*RSp463e + 4./9.*m55*L5r*RSp461e*RS346pe - 
         4./27.*m55*L5r*RSp461e*RSe463p + 4./9.*m55*L5r*RSp461e*RSp463e
          - 4./9.*m55*L5r*RSp461e*RS146pe + 4./27.*m55*L5r*RSp461e*
         RSe461p - 2./9.*m55*L5r*pow(RSp461e,2) - 2*m55*L4r*RSp45613e
          + m55*L4r*RSp45633e - 4./9.*m55*L4r*RSp563e*RS356pe + 4./27.*
         m55*L4r*RSp563e*RSe563p - 2./9.*m55*L4r*pow(RSp563e,2) - 4./9.
         *m55*L4r*RSp463e*RS346pe + 4./27.*m55*L4r*RSp463e*RSe463p - 2./
         9.*m55*L4r*pow(RSp463e,2) - 4./9.*m55*L4r*RSp453e*RS345pe + 4./
         27.*m55*L4r*RSp453e*RSe453p - 2./9.*m55*L4r*pow(RSp453e,2) + 
         m55*L4r*RSp45611e + 4./9.*m55*L4r*RS145pe*RSp453e + 4./9.*m55*
         L4r*RS146pe*RSp463e + 4./9.*m55*L4r*RS156pe*RSp563e - 4./27.*
         m55*L4r*RSe561p*RSp563e - 4./27.*m55*L4r*RSe461p*RSp463e - 4./
         27.*m55*L4r*RSe451p*RSp453e );

      F13p6x23LV +=  + AbV(mp2,L) * ( 4./9.*m55*L4r*RSp561e*RS356pe - 4.
         /27.*m55*L4r*RSp561e*RSe563p + 4./9.*m55*L4r*RSp561e*RSp563e
          - 4./9.*m55*L4r*RSp561e*RS156pe + 4./27.*m55*L4r*RSp561e*
         RSe561p - 2./9.*m55*L4r*pow(RSp561e,2) + 4./9.*m55*L4r*RSp461e
         *RS346pe - 4./27.*m55*L4r*RSp461e*RSe463p + 4./9.*m55*L4r*
         RSp461e*RSp463e - 4./9.*m55*L4r*RSp461e*RS146pe + 4./27.*m55*
         L4r*RSp461e*RSe461p - 2./9.*m55*L4r*pow(RSp461e,2) + 4./9.*m55
         *L4r*RSp451e*RS345pe - 4./27.*m55*L4r*RSp451e*RSe453p + 4./9.*
         m55*L4r*RSp451e*RSp453e - 4./9.*m55*L4r*RSp451e*RS145pe + 4./
         27.*m55*L4r*RSp451e*RSe451p - 2./9.*m55*L4r*pow(RSp451e,2) - 4.
         /3.*m55*L4r*RSp465e - 4./9.*m44*L5r*RSp563e*RS356pe + 4./27.*
         m44*L5r*RSp563e*RSe563p - 2./9.*m44*L5r*pow(RSp563e,2) + 4./9.
         *m44*L5r*RS156pe*RSp563e - 4./27.*m44*L5r*RSe561p*RSp563e + 4./
         9.*m44*L5r*RSp561e*RS356pe - 4./27.*m44*L5r*RSp561e*RSe563p + 
         4./9.*m44*L5r*RSp561e*RSp563e - 4./9.*m44*L5r*RSp561e*RS156pe
          + 4./27.*m44*L5r*RSp561e*RSe561p );

      F13p6x23LV +=  + AbV(mp2,L) * (  - 2./9.*m44*L5r*pow(RSp561e,2)
          - 2*m44*L4r*RSp45613e + m44*L4r*RSp45633e - 4./9.*m44*L4r*
         RSp563e*RS356pe + 4./27.*m44*L4r*RSp563e*RSe563p - 2./9.*m44*
         L4r*pow(RSp563e,2) - 4./9.*m44*L4r*RSp463e*RS346pe + 4./27.*
         m44*L4r*RSp463e*RSe463p - 2./9.*m44*L4r*pow(RSp463e,2) - 4./9.
         *m44*L4r*RSp453e*RS345pe + 4./27.*m44*L4r*RSp453e*RSe453p - 2./
         9.*m44*L4r*pow(RSp453e,2) + m44*L4r*RSp45611e + 4./9.*m44*L4r*
         RS145pe*RSp453e + 4./9.*m44*L4r*RS146pe*RSp463e + 4./9.*m44*
         L4r*RS156pe*RSp563e - 4./27.*m44*L4r*RSe561p*RSp563e - 4./27.*
         m44*L4r*RSe461p*RSp463e - 4./27.*m44*L4r*RSe451p*RSp453e + 4./
         9.*m44*L4r*RSp561e*RS356pe - 4./27.*m44*L4r*RSp561e*RSe563p + 
         4./9.*m44*L4r*RSp561e*RSp563e - 4./9.*m44*L4r*RSp561e*RS156pe
          + 4./27.*m44*L4r*RSp561e*RSe561p - 2./9.*m44*L4r*pow(
         RSp561e,2) + 4./9.*m44*L4r*RSp461e*RS346pe - 4./27.*m44*L4r*
         RSp461e*RSe463p + 4./9.*m44*L4r*RSp461e*RSp463e - 4./9.*m44*
         L4r*RSp461e*RS146pe );

      F13p6x23LV +=  + AbV(mp2,L) * ( 4./27.*m44*L4r*RSp461e*RSe461p - 
         2./9.*m44*L4r*pow(RSp461e,2) + 4./9.*m44*L4r*RSp451e*RS345pe
          - 4./27.*m44*L4r*RSp451e*RSe453p + 4./9.*m44*L4r*RSp451e*
         RSp453e - 4./9.*m44*L4r*RSp451e*RS145pe + 4./27.*m44*L4r*
         RSp451e*RSe451p - 2./9.*m44*L4r*pow(RSp451e,2) - 4./3.*m44*L4r
         *RSp564e - m33*L5r*RSp45613e + 1./2.*m33*L5r*RSp45633e - 1./6.
         *m33*L5r*RSp45611e - m11*L5r*RSp45613e - 1./6.*m11*L5r*
         RSp45633e + 1./2.*m11*L5r*RSp45611e );

      F13p6x23LV +=  + AbV(me2,L) * ( 2./3.*me2*L3r*RSe45613p + 5./3.*
         me2*L3r*RSe45633p + 5./3.*me2*L3r*RSe45611p + 2./3.*me2*L2r*
         RSe564p + 2./3.*me2*L2r*RSe465p + 2./3.*me2*L2r*RSe456p + 8./3.
         *me2*L1r*RSe564p + 8./3.*me2*L1r*RSe465p + 8./3.*me2*L1r*
         RSe456p + 8./3.*me2*L0r*RSe45613p + 2./3.*me2*L0r*RSe45633p + 
         2./3.*me2*L0r*RSe45611p - 4./9.*m66*L5r*RSe453p*RS345pe - 2./9.
         *m66*L5r*pow(RSe453p,2) - 16./27.*m66*L5r*RSp453e*RSe453p + 4./
         9.*m66*L5r*RS145pe*RSe453p + 4./9.*m66*L5r*RSe451p*RS345pe + 4.
         /9.*m66*L5r*RSe451p*RSe453p + 16./27.*m66*L5r*RSe451p*RSp453e
          - 4./9.*m66*L5r*RSe451p*RS145pe - 2./9.*m66*L5r*pow(
         RSe451p,2) + 16./27.*m66*L5r*RSp451e*RSe453p - 16./27.*m66*L5r
         *RSp451e*RSe451p - 2*m66*L4r*RSe45613p + m66*L4r*RSe45633p - 4.
         /9.*m66*L4r*RSe563p*RS356pe - 2./9.*m66*L4r*pow(RSe563p,2) - 4.
         /9.*m66*L4r*RSe463p*RS346pe - 2./9.*m66*L4r*pow(RSe463p,2) - 4.
         /9.*m66*L4r*RSe453p*RS345pe - 2./9.*m66*L4r*pow(RSe453p,2) - 
         16./27.*m66*L4r*RSp563e*RSe563p );

      F13p6x23LV +=  + AbV(me2,L) * (  - 16./27.*m66*L4r*RSp463e*
         RSe463p - 16./27.*m66*L4r*RSp453e*RSe453p + m66*L4r*RSe45611p
          + 4./9.*m66*L4r*RS145pe*RSe453p + 4./9.*m66*L4r*RS146pe*
         RSe463p + 4./9.*m66*L4r*RS156pe*RSe563p + 4./9.*m66*L4r*
         RSe561p*RS356pe + 4./9.*m66*L4r*RSe561p*RSe563p + 16./27.*m66*
         L4r*RSe561p*RSp563e - 4./9.*m66*L4r*RSe561p*RS156pe - 2./9.*
         m66*L4r*pow(RSe561p,2) + 4./9.*m66*L4r*RSe461p*RS346pe + 4./9.
         *m66*L4r*RSe461p*RSe463p + 16./27.*m66*L4r*RSe461p*RSp463e - 4.
         /9.*m66*L4r*RSe461p*RS146pe - 2./9.*m66*L4r*pow(RSe461p,2) + 4.
         /9.*m66*L4r*RSe451p*RS345pe + 4./9.*m66*L4r*RSe451p*RSe453p + 
         16./27.*m66*L4r*RSe451p*RSp453e - 4./9.*m66*L4r*RSe451p*
         RS145pe - 2./9.*m66*L4r*pow(RSe451p,2) + 16./27.*m66*L4r*
         RSp561e*RSe563p - 16./27.*m66*L4r*RSp561e*RSe561p + 16./27.*
         m66*L4r*RSp461e*RSe463p - 16./27.*m66*L4r*RSp461e*RSe461p + 16.
         /27.*m66*L4r*RSp451e*RSe453p - 16./27.*m66*L4r*RSp451e*RSe451p
          - 4./3.*m66*L4r*RSe456p );

      F13p6x23LV +=  + AbV(me2,L) * (  - 4./9.*m55*L5r*RSe463p*RS346pe
          - 2./9.*m55*L5r*pow(RSe463p,2) - 16./27.*m55*L5r*RSp463e*
         RSe463p + 4./9.*m55*L5r*RS146pe*RSe463p + 4./9.*m55*L5r*
         RSe461p*RS346pe + 4./9.*m55*L5r*RSe461p*RSe463p + 16./27.*m55*
         L5r*RSe461p*RSp463e - 4./9.*m55*L5r*RSe461p*RS146pe - 2./9.*
         m55*L5r*pow(RSe461p,2) + 16./27.*m55*L5r*RSp461e*RSe463p - 16./
         27.*m55*L5r*RSp461e*RSe461p - 2*m55*L4r*RSe45613p + m55*L4r*
         RSe45633p - 4./9.*m55*L4r*RSe563p*RS356pe - 2./9.*m55*L4r*pow(
         RSe563p,2) - 4./9.*m55*L4r*RSe463p*RS346pe - 2./9.*m55*L4r*
         pow(RSe463p,2) - 4./9.*m55*L4r*RSe453p*RS345pe - 2./9.*m55*L4r
         *pow(RSe453p,2) - 16./27.*m55*L4r*RSp563e*RSe563p - 16./27.*
         m55*L4r*RSp463e*RSe463p - 16./27.*m55*L4r*RSp453e*RSe453p + 
         m55*L4r*RSe45611p + 4./9.*m55*L4r*RS145pe*RSe453p + 4./9.*m55*
         L4r*RS146pe*RSe463p + 4./9.*m55*L4r*RS156pe*RSe563p + 4./9.*
         m55*L4r*RSe561p*RS356pe + 4./9.*m55*L4r*RSe561p*RSe563p + 16./
         27.*m55*L4r*RSe561p*RSp563e );

      F13p6x23LV +=  + AbV(me2,L) * (  - 4./9.*m55*L4r*RSe561p*RS156pe
          - 2./9.*m55*L4r*pow(RSe561p,2) + 4./9.*m55*L4r*RSe461p*
         RS346pe + 4./9.*m55*L4r*RSe461p*RSe463p + 16./27.*m55*L4r*
         RSe461p*RSp463e - 4./9.*m55*L4r*RSe461p*RS146pe - 2./9.*m55*
         L4r*pow(RSe461p,2) + 4./9.*m55*L4r*RSe451p*RS345pe + 4./9.*m55
         *L4r*RSe451p*RSe453p + 16./27.*m55*L4r*RSe451p*RSp453e - 4./9.
         *m55*L4r*RSe451p*RS145pe - 2./9.*m55*L4r*pow(RSe451p,2) + 16./
         27.*m55*L4r*RSp561e*RSe563p - 16./27.*m55*L4r*RSp561e*RSe561p
          + 16./27.*m55*L4r*RSp461e*RSe463p - 16./27.*m55*L4r*RSp461e*
         RSe461p + 16./27.*m55*L4r*RSp451e*RSe453p - 16./27.*m55*L4r*
         RSp451e*RSe451p - 4./3.*m55*L4r*RSe465p - 4./9.*m44*L5r*
         RSe563p*RS356pe - 2./9.*m44*L5r*pow(RSe563p,2) - 16./27.*m44*
         L5r*RSp563e*RSe563p + 4./9.*m44*L5r*RS156pe*RSe563p + 4./9.*
         m44*L5r*RSe561p*RS356pe + 4./9.*m44*L5r*RSe561p*RSe563p + 16./
         27.*m44*L5r*RSe561p*RSp563e - 4./9.*m44*L5r*RSe561p*RS156pe - 
         2./9.*m44*L5r*pow(RSe561p,2) );

      F13p6x23LV +=  + AbV(me2,L) * ( 16./27.*m44*L5r*RSp561e*RSe563p
          - 16./27.*m44*L5r*RSp561e*RSe561p - 2*m44*L4r*RSe45613p + m44
         *L4r*RSe45633p - 4./9.*m44*L4r*RSe563p*RS356pe - 2./9.*m44*L4r
         *pow(RSe563p,2) - 4./9.*m44*L4r*RSe463p*RS346pe - 2./9.*m44*
         L4r*pow(RSe463p,2) - 4./9.*m44*L4r*RSe453p*RS345pe - 2./9.*m44
         *L4r*pow(RSe453p,2) - 16./27.*m44*L4r*RSp563e*RSe563p - 16./27.
         *m44*L4r*RSp463e*RSe463p - 16./27.*m44*L4r*RSp453e*RSe453p + 
         m44*L4r*RSe45611p + 4./9.*m44*L4r*RS145pe*RSe453p + 4./9.*m44*
         L4r*RS146pe*RSe463p + 4./9.*m44*L4r*RS156pe*RSe563p + 4./9.*
         m44*L4r*RSe561p*RS356pe + 4./9.*m44*L4r*RSe561p*RSe563p + 16./
         27.*m44*L4r*RSe561p*RSp563e - 4./9.*m44*L4r*RSe561p*RS156pe - 
         2./9.*m44*L4r*pow(RSe561p,2) + 4./9.*m44*L4r*RSe461p*RS346pe
          + 4./9.*m44*L4r*RSe461p*RSe463p + 16./27.*m44*L4r*RSe461p*
         RSp463e - 4./9.*m44*L4r*RSe461p*RS146pe - 2./9.*m44*L4r*pow(
         RSe461p,2) + 4./9.*m44*L4r*RSe451p*RS345pe + 4./9.*m44*L4r*
         RSe451p*RSe453p );

      F13p6x23LV +=  + AbV(me2,L) * ( 16./27.*m44*L4r*RSe451p*RSp453e
          - 4./9.*m44*L4r*RSe451p*RS145pe - 2./9.*m44*L4r*pow(
         RSe451p,2) + 16./27.*m44*L4r*RSp561e*RSe563p - 16./27.*m44*L4r
         *RSp561e*RSe561p + 16./27.*m44*L4r*RSp461e*RSe463p - 16./27.*
         m44*L4r*RSp461e*RSe461p + 16./27.*m44*L4r*RSp451e*RSe453p - 16.
         /27.*m44*L4r*RSp451e*RSe451p - 4./3.*m44*L4r*RSe564p - m33*L5r
         *RSe45613p + 1./2.*m33*L5r*RSe45633p - 1./6.*m33*L5r*RSe45611p
          - m11*L5r*RSe45613p - 1./6.*m11*L5r*RSe45633p + 1./2.*m11*L5r
         *RSe45611p );

      F13p6x23LV +=  + BbV(m11,m33,L) * (  - 8./9.*pow(m66,2)*L8r*
         RS145pe*RS345pe - 8./9.*pow(m66,2)*L7r*RS145pe*RS345pe - 8./9.
         *pow(m66,2)*L6r*RS145pe*RS345pe - 8./9.*m55*m66*L7r*RS145pe*
         RS346pe - 8./9.*m55*m66*L7r*RS146pe*RS345pe - 8./9.*m55*m66*
         L6r*RS145pe*RS345pe - 8./9.*m55*m66*L6r*RS146pe*RS346pe - 8./9.
         *pow(m55,2)*L8r*RS146pe*RS346pe - 8./9.*pow(m55,2)*L7r*RS146pe
         *RS346pe - 8./9.*pow(m55,2)*L6r*RS146pe*RS346pe - 8./9.*m44*
         m66*L7r*RS145pe*RS356pe - 8./9.*m44*m66*L7r*RS156pe*RS345pe - 
         8./9.*m44*m66*L6r*RS145pe*RS345pe - 8./9.*m44*m66*L6r*RS156pe*
         RS356pe - 8./9.*m44*m55*L7r*RS146pe*RS356pe - 8./9.*m44*m55*
         L7r*RS156pe*RS346pe - 8./9.*m44*m55*L6r*RS146pe*RS346pe - 8./9.
         *m44*m55*L6r*RS156pe*RS356pe - 8./9.*pow(m44,2)*L8r*RS156pe*
         RS356pe - 8./9.*pow(m44,2)*L7r*RS156pe*RS356pe - 8./9.*pow(
         m44,2)*L6r*RS156pe*RS356pe + 8./3.*m33*m66*L7r*RS145pe + 8./3.
         *m33*m66*L6r*RS14563pe + 8./3.*m33*m55*L7r*RS146pe + 8./3.*m33
         *m55*L6r*RS14563pe );

      F13p6x23LV +=  + BbV(m11,m33,L) * ( 8./3.*m33*m44*L7r*RS156pe + 8.
         /3.*m33*m44*L6r*RS14563pe + 8./3.*pow(m33,2)*L8r*RS14563pe + 8.
         /3.*m11*m66*L7r*RS345pe + 8./3.*m11*m66*L6r*RS34561pe + 4./9.*
         m11*m66*L5r*RS145pe*RS345pe - 4./3.*m11*m66*L4r*RS34561pe - 4./
         3.*m11*m66*L4r*RS14563pe + 4./9.*m11*m66*L4r*RS145pe*RS345pe
          + 4./9.*m11*m66*L4r*RS146pe*RS346pe + 4./9.*m11*m66*L4r*
         RS156pe*RS356pe + 8./3.*m11*m55*L7r*RS346pe + 8./3.*m11*m55*
         L6r*RS34561pe + 4./9.*m11*m55*L5r*RS146pe*RS346pe - 4./3.*m11*
         m55*L4r*RS34561pe - 4./3.*m11*m55*L4r*RS14563pe + 4./9.*m11*
         m55*L4r*RS145pe*RS345pe + 4./9.*m11*m55*L4r*RS146pe*RS346pe + 
         4./9.*m11*m55*L4r*RS156pe*RS356pe + 8./3.*m11*m44*L7r*RS356pe
          + 8./3.*m11*m44*L6r*RS34561pe + 4./9.*m11*m44*L5r*RS156pe*
         RS356pe - 4./3.*m11*m44*L4r*RS34561pe - 4./3.*m11*m44*L4r*
         RS14563pe + 4./9.*m11*m44*L4r*RS145pe*RS345pe + 4./9.*m11*m44*
         L4r*RS146pe*RS346pe + 4./9.*m11*m44*L4r*RS156pe*RS356pe - 8*
         m11*m33*L7r );

      F13p6x23LV +=  + BbV(m11,m33,L) * (  - 4./3.*m11*m33*L5r*
         RS14563pe + 8./3.*pow(m11,2)*L8r*RS34561pe - 4./3.*pow(m11,2)*
         L5r*RS34561pe );

      F13p6x23LV +=  + BbV(m11,mp2,L) * (  - 8./9.*pow(m66,2)*L8r*
         RS145pe*RSp453e + 8./9.*pow(m66,2)*L8r*RSp451e*RS145pe - 8./9.
         *pow(m66,2)*L7r*RS145pe*RSp453e + 8./9.*pow(m66,2)*L7r*RSp451e
         *RS145pe - 8./9.*pow(m66,2)*L6r*RS145pe*RSp453e + 8./9.*pow(
         m66,2)*L6r*RSp451e*RS145pe - 8./9.*m55*m66*L7r*RS145pe*RSp463e
          - 8./9.*m55*m66*L7r*RS146pe*RSp453e + 8./9.*m55*m66*L7r*
         RSp461e*RS145pe + 8./9.*m55*m66*L7r*RSp451e*RS146pe - 8./9.*
         m55*m66*L6r*RS145pe*RSp453e - 8./9.*m55*m66*L6r*RS146pe*
         RSp463e + 8./9.*m55*m66*L6r*RSp461e*RS146pe + 8./9.*m55*m66*
         L6r*RSp451e*RS145pe - 8./9.*pow(m55,2)*L8r*RS146pe*RSp463e + 8.
         /9.*pow(m55,2)*L8r*RSp461e*RS146pe - 8./9.*pow(m55,2)*L7r*
         RS146pe*RSp463e + 8./9.*pow(m55,2)*L7r*RSp461e*RS146pe - 8./9.
         *pow(m55,2)*L6r*RS146pe*RSp463e + 8./9.*pow(m55,2)*L6r*RSp461e
         *RS146pe - 8./9.*m44*m66*L7r*RS145pe*RSp563e - 8./9.*m44*m66*
         L7r*RS156pe*RSp453e + 8./9.*m44*m66*L7r*RSp561e*RS145pe + 8./9.
         *m44*m66*L7r*RSp451e*RS156pe );

      F13p6x23LV +=  + BbV(m11,mp2,L) * (  - 8./9.*m44*m66*L6r*RS145pe*
         RSp453e - 8./9.*m44*m66*L6r*RS156pe*RSp563e + 8./9.*m44*m66*
         L6r*RSp561e*RS156pe + 8./9.*m44*m66*L6r*RSp451e*RS145pe - 8./9.
         *m44*m55*L7r*RS146pe*RSp563e - 8./9.*m44*m55*L7r*RS156pe*
         RSp463e + 8./9.*m44*m55*L7r*RSp561e*RS146pe + 8./9.*m44*m55*
         L7r*RSp461e*RS156pe - 8./9.*m44*m55*L6r*RS146pe*RSp463e - 8./9.
         *m44*m55*L6r*RS156pe*RSp563e + 8./9.*m44*m55*L6r*RSp561e*
         RS156pe + 8./9.*m44*m55*L6r*RSp461e*RS146pe - 8./9.*pow(m44,2)
         *L8r*RS156pe*RSp563e + 8./9.*pow(m44,2)*L8r*RSp561e*RS156pe - 
         8./9.*pow(m44,2)*L7r*RS156pe*RSp563e + 8./9.*pow(m44,2)*L7r*
         RSp561e*RS156pe - 8./9.*pow(m44,2)*L6r*RS156pe*RSp563e + 8./9.
         *pow(m44,2)*L6r*RSp561e*RS156pe + 8./3.*m11*m66*L7r*RSp453e - 
         8./3.*m11*m66*L7r*RSp451e + 8./3.*m11*m66*L6r*RSp45613e - 8./3.
         *m11*m66*L6r*RSp45611e + 4./9.*m11*m66*L5r*RS145pe*RSp453e - 4.
         /9.*m11*m66*L5r*RSp451e*RS145pe - 4./3.*m11*m66*L4r*RSp45613e
          + 4./3.*m11*m66*L4r*RSp45611e );

      F13p6x23LV +=  + BbV(m11,mp2,L) * ( 4./9.*m11*m66*L4r*RS145pe*
         RSp453e + 4./9.*m11*m66*L4r*RS146pe*RSp463e + 4./9.*m11*m66*
         L4r*RS156pe*RSp563e - 4./9.*m11*m66*L4r*RSp561e*RS156pe - 4./9.
         *m11*m66*L4r*RSp461e*RS146pe - 4./9.*m11*m66*L4r*RSp451e*
         RS145pe + 8./3.*m11*m55*L7r*RSp463e - 8./3.*m11*m55*L7r*
         RSp461e + 8./3.*m11*m55*L6r*RSp45613e - 8./3.*m11*m55*L6r*
         RSp45611e + 4./9.*m11*m55*L5r*RS146pe*RSp463e - 4./9.*m11*m55*
         L5r*RSp461e*RS146pe - 4./3.*m11*m55*L4r*RSp45613e + 4./3.*m11*
         m55*L4r*RSp45611e + 4./9.*m11*m55*L4r*RS145pe*RSp453e + 4./9.*
         m11*m55*L4r*RS146pe*RSp463e + 4./9.*m11*m55*L4r*RS156pe*
         RSp563e - 4./9.*m11*m55*L4r*RSp561e*RS156pe - 4./9.*m11*m55*
         L4r*RSp461e*RS146pe - 4./9.*m11*m55*L4r*RSp451e*RS145pe + 8./3.
         *m11*m44*L7r*RSp563e - 8./3.*m11*m44*L7r*RSp561e + 8./3.*m11*
         m44*L6r*RSp45613e - 8./3.*m11*m44*L6r*RSp45611e + 4./9.*m11*
         m44*L5r*RS156pe*RSp563e - 4./9.*m11*m44*L5r*RSp561e*RS156pe - 
         4./3.*m11*m44*L4r*RSp45613e );

      F13p6x23LV +=  + BbV(m11,mp2,L) * ( 4./3.*m11*m44*L4r*RSp45611e
          + 4./9.*m11*m44*L4r*RS145pe*RSp453e + 4./9.*m11*m44*L4r*
         RS146pe*RSp463e + 4./9.*m11*m44*L4r*RS156pe*RSp563e - 4./9.*
         m11*m44*L4r*RSp561e*RS156pe - 4./9.*m11*m44*L4r*RSp461e*
         RS146pe - 4./9.*m11*m44*L4r*RSp451e*RS145pe + 8./3.*pow(m11,2)
         *L8r*RSp45613e - 8./3.*pow(m11,2)*L8r*RSp45611e - 4./3.*pow(
         m11,2)*L5r*RSp45613e + 4./3.*pow(m11,2)*L5r*RSp45611e );

      F13p6x23LV +=  + BbV(m11,me2,L) * (  - 8./9.*pow(m66,2)*L8r*
         RS145pe*RSe453p + 8./9.*pow(m66,2)*L8r*RSe451p*RS145pe - 8./9.
         *pow(m66,2)*L7r*RS145pe*RSe453p + 8./9.*pow(m66,2)*L7r*RSe451p
         *RS145pe - 8./9.*pow(m66,2)*L6r*RS145pe*RSe453p + 8./9.*pow(
         m66,2)*L6r*RSe451p*RS145pe - 8./9.*m55*m66*L7r*RS145pe*RSe463p
          - 8./9.*m55*m66*L7r*RS146pe*RSe453p + 8./9.*m55*m66*L7r*
         RSe461p*RS145pe + 8./9.*m55*m66*L7r*RSe451p*RS146pe - 8./9.*
         m55*m66*L6r*RS145pe*RSe453p - 8./9.*m55*m66*L6r*RS146pe*
         RSe463p + 8./9.*m55*m66*L6r*RSe461p*RS146pe + 8./9.*m55*m66*
         L6r*RSe451p*RS145pe - 8./9.*pow(m55,2)*L8r*RS146pe*RSe463p + 8.
         /9.*pow(m55,2)*L8r*RSe461p*RS146pe - 8./9.*pow(m55,2)*L7r*
         RS146pe*RSe463p + 8./9.*pow(m55,2)*L7r*RSe461p*RS146pe - 8./9.
         *pow(m55,2)*L6r*RS146pe*RSe463p + 8./9.*pow(m55,2)*L6r*RSe461p
         *RS146pe - 8./9.*m44*m66*L7r*RS145pe*RSe563p - 8./9.*m44*m66*
         L7r*RS156pe*RSe453p + 8./9.*m44*m66*L7r*RSe561p*RS145pe + 8./9.
         *m44*m66*L7r*RSe451p*RS156pe );

      F13p6x23LV +=  + BbV(m11,me2,L) * (  - 8./9.*m44*m66*L6r*RS145pe*
         RSe453p - 8./9.*m44*m66*L6r*RS156pe*RSe563p + 8./9.*m44*m66*
         L6r*RSe561p*RS156pe + 8./9.*m44*m66*L6r*RSe451p*RS145pe - 8./9.
         *m44*m55*L7r*RS146pe*RSe563p - 8./9.*m44*m55*L7r*RS156pe*
         RSe463p + 8./9.*m44*m55*L7r*RSe561p*RS146pe + 8./9.*m44*m55*
         L7r*RSe461p*RS156pe - 8./9.*m44*m55*L6r*RS146pe*RSe463p - 8./9.
         *m44*m55*L6r*RS156pe*RSe563p + 8./9.*m44*m55*L6r*RSe561p*
         RS156pe + 8./9.*m44*m55*L6r*RSe461p*RS146pe - 8./9.*pow(m44,2)
         *L8r*RS156pe*RSe563p + 8./9.*pow(m44,2)*L8r*RSe561p*RS156pe - 
         8./9.*pow(m44,2)*L7r*RS156pe*RSe563p + 8./9.*pow(m44,2)*L7r*
         RSe561p*RS156pe - 8./9.*pow(m44,2)*L6r*RS156pe*RSe563p + 8./9.
         *pow(m44,2)*L6r*RSe561p*RS156pe + 8./3.*m11*m66*L7r*RSe453p - 
         8./3.*m11*m66*L7r*RSe451p + 8./3.*m11*m66*L6r*RSe45613p - 8./3.
         *m11*m66*L6r*RSe45611p + 4./9.*m11*m66*L5r*RS145pe*RSe453p - 4.
         /9.*m11*m66*L5r*RSe451p*RS145pe - 4./3.*m11*m66*L4r*RSe45613p
          + 4./3.*m11*m66*L4r*RSe45611p );

      F13p6x23LV +=  + BbV(m11,me2,L) * ( 4./9.*m11*m66*L4r*RS145pe*
         RSe453p + 4./9.*m11*m66*L4r*RS146pe*RSe463p + 4./9.*m11*m66*
         L4r*RS156pe*RSe563p - 4./9.*m11*m66*L4r*RSe561p*RS156pe - 4./9.
         *m11*m66*L4r*RSe461p*RS146pe - 4./9.*m11*m66*L4r*RSe451p*
         RS145pe + 8./3.*m11*m55*L7r*RSe463p - 8./3.*m11*m55*L7r*
         RSe461p + 8./3.*m11*m55*L6r*RSe45613p - 8./3.*m11*m55*L6r*
         RSe45611p + 4./9.*m11*m55*L5r*RS146pe*RSe463p - 4./9.*m11*m55*
         L5r*RSe461p*RS146pe - 4./3.*m11*m55*L4r*RSe45613p + 4./3.*m11*
         m55*L4r*RSe45611p + 4./9.*m11*m55*L4r*RS145pe*RSe453p + 4./9.*
         m11*m55*L4r*RS146pe*RSe463p + 4./9.*m11*m55*L4r*RS156pe*
         RSe563p - 4./9.*m11*m55*L4r*RSe561p*RS156pe - 4./9.*m11*m55*
         L4r*RSe461p*RS146pe - 4./9.*m11*m55*L4r*RSe451p*RS145pe + 8./3.
         *m11*m44*L7r*RSe563p - 8./3.*m11*m44*L7r*RSe561p + 8./3.*m11*
         m44*L6r*RSe45613p - 8./3.*m11*m44*L6r*RSe45611p + 4./9.*m11*
         m44*L5r*RS156pe*RSe563p - 4./9.*m11*m44*L5r*RSe561p*RS156pe - 
         4./3.*m11*m44*L4r*RSe45613p );

      F13p6x23LV +=  + BbV(m11,me2,L) * ( 4./3.*m11*m44*L4r*RSe45611p
          + 4./9.*m11*m44*L4r*RS145pe*RSe453p + 4./9.*m11*m44*L4r*
         RS146pe*RSe463p + 4./9.*m11*m44*L4r*RS156pe*RSe563p - 4./9.*
         m11*m44*L4r*RSe561p*RS156pe - 4./9.*m11*m44*L4r*RSe461p*
         RS146pe - 4./9.*m11*m44*L4r*RSe451p*RS145pe + 8./3.*pow(m11,2)
         *L8r*RSe45613p - 8./3.*pow(m11,2)*L8r*RSe45611p - 4./3.*pow(
         m11,2)*L5r*RSe45613p + 4./3.*pow(m11,2)*L5r*RSe45611p );

      F13p6x23LV +=  + BbV(m11,L) * ( m66*L4r*RD1456ep + 4./9.*pow(
         m66,2)*L8r*pow(RS145pe,2) + 4./9.*pow(m66,2)*L7r*pow(
         RS145pe,2) + 4./9.*pow(m66,2)*L6r*pow(RS145pe,2) + m55*L4r*
         RD1456ep + 8./9.*m55*m66*L7r*RS146pe*RS145pe + 4./9.*m55*m66*
         L6r*pow(RS145pe,2) + 4./9.*m55*m66*L6r*pow(RS146pe,2) + 4./9.*
         pow(m55,2)*L8r*pow(RS146pe,2) + 4./9.*pow(m55,2)*L7r*pow(
         RS146pe,2) + 4./9.*pow(m55,2)*L6r*pow(RS146pe,2) + m44*L4r*
         RD1456ep + 8./9.*m44*m66*L7r*RS156pe*RS145pe + 4./9.*m44*m66*
         L6r*pow(RS145pe,2) + 4./9.*m44*m66*L6r*pow(RS156pe,2) + 8./9.*
         m44*m55*L7r*RS156pe*RS146pe + 4./9.*m44*m55*L6r*pow(RS146pe,2)
          + 4./9.*m44*m55*L6r*pow(RS156pe,2) + 4./9.*pow(m44,2)*L8r*
         pow(RS156pe,2) + 4./9.*pow(m44,2)*L7r*pow(RS156pe,2) + 4./9.*
         pow(m44,2)*L6r*pow(RS156pe,2) - 1./6.*m33*L5r*RD1456ep + 1./2.
         *m11*L5r*RD1456ep + 5./3.*m11*L3r*RD1456ep + 2./3.*m11*L0r*
         RD1456ep - 8./3.*m11*m66*L7r*RS145pe + 8./3.*m11*m66*L6r*
         RS14563pe );

      F13p6x23LV +=  + BbV(m11,L) * (  - 8./3.*m11*m66*L6r*RC1456pe - 2.
         /9.*m11*m66*L5r*pow(RS145pe,2) - 4./3.*m11*m66*L4r*RS14563pe
          + 4./3.*m11*m66*L4r*RC1456pe - 2./9.*m11*m66*L4r*pow(
         RS145pe,2) - 2./9.*m11*m66*L4r*pow(RS146pe,2) - 2./9.*m11*m66*
         L4r*pow(RS156pe,2) - 8./3.*m11*m55*L7r*RS146pe + 8./3.*m11*m55
         *L6r*RS14563pe - 8./3.*m11*m55*L6r*RC1456pe - 2./9.*m11*m55*
         L5r*pow(RS146pe,2) - 4./3.*m11*m55*L4r*RS14563pe + 4./3.*m11*
         m55*L4r*RC1456pe - 2./9.*m11*m55*L4r*pow(RS145pe,2) - 2./9.*
         m11*m55*L4r*pow(RS146pe,2) - 2./9.*m11*m55*L4r*pow(RS156pe,2)
          - 8./3.*m11*m44*L7r*RS156pe + 8./3.*m11*m44*L6r*RS14563pe - 8.
         /3.*m11*m44*L6r*RC1456pe - 2./9.*m11*m44*L5r*pow(RS156pe,2) - 
         4./3.*m11*m44*L4r*RS14563pe + 4./3.*m11*m44*L4r*RC1456pe - 2./
         9.*m11*m44*L4r*pow(RS145pe,2) - 2./9.*m11*m44*L4r*pow(
         RS146pe,2) - 2./9.*m11*m44*L4r*pow(RS156pe,2) + 8./3.*pow(
         m11,2)*L8r*RS14563pe - 8./3.*pow(m11,2)*L8r*RC1456pe + 4*pow(
         m11,2)*L7r );

      F13p6x23LV +=  + BbV(m11,L) * (  - 4./3.*pow(m11,2)*L5r*RS14563pe
          + 4./3.*pow(m11,2)*L5r*RC1456pe );

      F13p6x23LV +=  + BbV(m14,L) * ( 2*m44*m66*L6r + 2*m44*m55*L6r + 
         pow(m44,2)*L8r + 2*pow(m44,2)*L6r - 2*m14*m66*L4r - 2*m14*m55*
         L4r - m14*m44*L5r - 2*m14*m44*L4r + 2*m11*m66*L6r + 2*m11*m55*
         L6r + 2*m11*m44*L8r + 2*m11*m44*L6r - m11*m14*L5r + pow(m11,2)
         *L8r );

      F13p6x23LV +=  + BbV(m15,L) * ( 2*m55*m66*L6r + pow(m55,2)*L8r + 
         2*pow(m55,2)*L6r + 2*m44*m55*L6r - 2*m15*m66*L4r - m15*m55*L5r
          - 2*m15*m55*L4r - 2*m15*m44*L4r + 2*m11*m66*L6r + 2*m11*m55*
         L8r + 2*m11*m55*L6r + 2*m11*m44*L6r - m11*m15*L5r + pow(m11,2)
         *L8r );

      F13p6x23LV +=  + BbV(m16,L) * ( pow(m66,2)*L8r + 2*pow(m66,2)*L6r
          + 2*m55*m66*L6r + 2*m44*m66*L6r - m16*m66*L5r - 2*m16*m66*L4r
          - 2*m16*m55*L4r - 2*m16*m44*L4r + 2*m11*m66*L8r + 2*m11*m66*
         L6r + 2*m11*m55*L6r + 2*m11*m44*L6r - m11*m16*L5r + pow(m11,2)
         *L8r );

      F13p6x23LV +=  + BbV(m33,mp2,L) * ( 8./9.*pow(m66,2)*L8r*RSp453e*
         RS345pe - 8./9.*pow(m66,2)*L8r*RSp451e*RS345pe + 8./9.*pow(
         m66,2)*L7r*RSp453e*RS345pe - 8./9.*pow(m66,2)*L7r*RSp451e*
         RS345pe + 8./9.*pow(m66,2)*L6r*RSp453e*RS345pe - 8./9.*pow(
         m66,2)*L6r*RSp451e*RS345pe + 8./9.*m55*m66*L7r*RSp463e*RS345pe
          + 8./9.*m55*m66*L7r*RSp453e*RS346pe - 8./9.*m55*m66*L7r*
         RSp461e*RS345pe - 8./9.*m55*m66*L7r*RSp451e*RS346pe + 8./9.*
         m55*m66*L6r*RSp463e*RS346pe + 8./9.*m55*m66*L6r*RSp453e*
         RS345pe - 8./9.*m55*m66*L6r*RSp461e*RS346pe - 8./9.*m55*m66*
         L6r*RSp451e*RS345pe + 8./9.*pow(m55,2)*L8r*RSp463e*RS346pe - 8.
         /9.*pow(m55,2)*L8r*RSp461e*RS346pe + 8./9.*pow(m55,2)*L7r*
         RSp463e*RS346pe - 8./9.*pow(m55,2)*L7r*RSp461e*RS346pe + 8./9.
         *pow(m55,2)*L6r*RSp463e*RS346pe - 8./9.*pow(m55,2)*L6r*RSp461e
         *RS346pe + 8./9.*m44*m66*L7r*RSp563e*RS345pe + 8./9.*m44*m66*
         L7r*RSp453e*RS356pe - 8./9.*m44*m66*L7r*RSp561e*RS345pe - 8./9.
         *m44*m66*L7r*RSp451e*RS356pe );

      F13p6x23LV +=  + BbV(m33,mp2,L) * ( 8./9.*m44*m66*L6r*RSp563e*
         RS356pe + 8./9.*m44*m66*L6r*RSp453e*RS345pe - 8./9.*m44*m66*
         L6r*RSp561e*RS356pe - 8./9.*m44*m66*L6r*RSp451e*RS345pe + 8./9.
         *m44*m55*L7r*RSp563e*RS346pe + 8./9.*m44*m55*L7r*RSp463e*
         RS356pe - 8./9.*m44*m55*L7r*RSp561e*RS346pe - 8./9.*m44*m55*
         L7r*RSp461e*RS356pe + 8./9.*m44*m55*L6r*RSp563e*RS356pe + 8./9.
         *m44*m55*L6r*RSp463e*RS346pe - 8./9.*m44*m55*L6r*RSp561e*
         RS356pe - 8./9.*m44*m55*L6r*RSp461e*RS346pe + 8./9.*pow(m44,2)
         *L8r*RSp563e*RS356pe - 8./9.*pow(m44,2)*L8r*RSp561e*RS356pe + 
         8./9.*pow(m44,2)*L7r*RSp563e*RS356pe - 8./9.*pow(m44,2)*L7r*
         RSp561e*RS356pe + 8./9.*pow(m44,2)*L6r*RSp563e*RS356pe - 8./9.
         *pow(m44,2)*L6r*RSp561e*RS356pe - 8./3.*m33*m66*L7r*RSp453e + 
         8./3.*m33*m66*L7r*RSp451e + 8./3.*m33*m66*L6r*RSp45613e - 8./3.
         *m33*m66*L6r*RSp45633e - 4./9.*m33*m66*L5r*RSp453e*RS345pe + 4.
         /9.*m33*m66*L5r*RSp451e*RS345pe - 4./3.*m33*m66*L4r*RSp45613e
          + 4./3.*m33*m66*L4r*RSp45633e );

      F13p6x23LV +=  + BbV(m33,mp2,L) * (  - 4./9.*m33*m66*L4r*RSp563e*
         RS356pe - 4./9.*m33*m66*L4r*RSp463e*RS346pe - 4./9.*m33*m66*
         L4r*RSp453e*RS345pe + 4./9.*m33*m66*L4r*RSp561e*RS356pe + 4./9.
         *m33*m66*L4r*RSp461e*RS346pe + 4./9.*m33*m66*L4r*RSp451e*
         RS345pe - 8./3.*m33*m55*L7r*RSp463e + 8./3.*m33*m55*L7r*
         RSp461e + 8./3.*m33*m55*L6r*RSp45613e - 8./3.*m33*m55*L6r*
         RSp45633e - 4./9.*m33*m55*L5r*RSp463e*RS346pe + 4./9.*m33*m55*
         L5r*RSp461e*RS346pe - 4./3.*m33*m55*L4r*RSp45613e + 4./3.*m33*
         m55*L4r*RSp45633e - 4./9.*m33*m55*L4r*RSp563e*RS356pe - 4./9.*
         m33*m55*L4r*RSp463e*RS346pe - 4./9.*m33*m55*L4r*RSp453e*
         RS345pe + 4./9.*m33*m55*L4r*RSp561e*RS356pe + 4./9.*m33*m55*
         L4r*RSp461e*RS346pe + 4./9.*m33*m55*L4r*RSp451e*RS345pe - 8./3.
         *m33*m44*L7r*RSp563e + 8./3.*m33*m44*L7r*RSp561e + 8./3.*m33*
         m44*L6r*RSp45613e - 8./3.*m33*m44*L6r*RSp45633e - 4./9.*m33*
         m44*L5r*RSp563e*RS356pe + 4./9.*m33*m44*L5r*RSp561e*RS356pe - 
         4./3.*m33*m44*L4r*RSp45613e );

      F13p6x23LV +=  + BbV(m33,mp2,L) * ( 4./3.*m33*m44*L4r*RSp45633e
          - 4./9.*m33*m44*L4r*RSp563e*RS356pe - 4./9.*m33*m44*L4r*
         RSp463e*RS346pe - 4./9.*m33*m44*L4r*RSp453e*RS345pe + 4./9.*
         m33*m44*L4r*RSp561e*RS356pe + 4./9.*m33*m44*L4r*RSp461e*
         RS346pe + 4./9.*m33*m44*L4r*RSp451e*RS345pe + 8./3.*pow(m33,2)
         *L8r*RSp45613e - 8./3.*pow(m33,2)*L8r*RSp45633e - 4./3.*pow(
         m33,2)*L5r*RSp45613e + 4./3.*pow(m33,2)*L5r*RSp45633e );

      F13p6x23LV +=  + BbV(m33,me2,L) * ( 8./9.*pow(m66,2)*L8r*RSe453p*
         RS345pe - 8./9.*pow(m66,2)*L8r*RSe451p*RS345pe + 8./9.*pow(
         m66,2)*L7r*RSe453p*RS345pe - 8./9.*pow(m66,2)*L7r*RSe451p*
         RS345pe + 8./9.*pow(m66,2)*L6r*RSe453p*RS345pe - 8./9.*pow(
         m66,2)*L6r*RSe451p*RS345pe + 8./9.*m55*m66*L7r*RSe463p*RS345pe
          + 8./9.*m55*m66*L7r*RSe453p*RS346pe - 8./9.*m55*m66*L7r*
         RSe461p*RS345pe - 8./9.*m55*m66*L7r*RSe451p*RS346pe + 8./9.*
         m55*m66*L6r*RSe463p*RS346pe + 8./9.*m55*m66*L6r*RSe453p*
         RS345pe - 8./9.*m55*m66*L6r*RSe461p*RS346pe - 8./9.*m55*m66*
         L6r*RSe451p*RS345pe + 8./9.*pow(m55,2)*L8r*RSe463p*RS346pe - 8.
         /9.*pow(m55,2)*L8r*RSe461p*RS346pe + 8./9.*pow(m55,2)*L7r*
         RSe463p*RS346pe - 8./9.*pow(m55,2)*L7r*RSe461p*RS346pe + 8./9.
         *pow(m55,2)*L6r*RSe463p*RS346pe - 8./9.*pow(m55,2)*L6r*RSe461p
         *RS346pe + 8./9.*m44*m66*L7r*RSe563p*RS345pe + 8./9.*m44*m66*
         L7r*RSe453p*RS356pe - 8./9.*m44*m66*L7r*RSe561p*RS345pe - 8./9.
         *m44*m66*L7r*RSe451p*RS356pe );

      F13p6x23LV +=  + BbV(m33,me2,L) * ( 8./9.*m44*m66*L6r*RSe563p*
         RS356pe + 8./9.*m44*m66*L6r*RSe453p*RS345pe - 8./9.*m44*m66*
         L6r*RSe561p*RS356pe - 8./9.*m44*m66*L6r*RSe451p*RS345pe + 8./9.
         *m44*m55*L7r*RSe563p*RS346pe + 8./9.*m44*m55*L7r*RSe463p*
         RS356pe - 8./9.*m44*m55*L7r*RSe561p*RS346pe - 8./9.*m44*m55*
         L7r*RSe461p*RS356pe + 8./9.*m44*m55*L6r*RSe563p*RS356pe + 8./9.
         *m44*m55*L6r*RSe463p*RS346pe - 8./9.*m44*m55*L6r*RSe561p*
         RS356pe - 8./9.*m44*m55*L6r*RSe461p*RS346pe + 8./9.*pow(m44,2)
         *L8r*RSe563p*RS356pe - 8./9.*pow(m44,2)*L8r*RSe561p*RS356pe + 
         8./9.*pow(m44,2)*L7r*RSe563p*RS356pe - 8./9.*pow(m44,2)*L7r*
         RSe561p*RS356pe + 8./9.*pow(m44,2)*L6r*RSe563p*RS356pe - 8./9.
         *pow(m44,2)*L6r*RSe561p*RS356pe - 8./3.*m33*m66*L7r*RSe453p + 
         8./3.*m33*m66*L7r*RSe451p + 8./3.*m33*m66*L6r*RSe45613p - 8./3.
         *m33*m66*L6r*RSe45633p - 4./9.*m33*m66*L5r*RSe453p*RS345pe + 4.
         /9.*m33*m66*L5r*RSe451p*RS345pe - 4./3.*m33*m66*L4r*RSe45613p
          + 4./3.*m33*m66*L4r*RSe45633p );

      F13p6x23LV +=  + BbV(m33,me2,L) * (  - 4./9.*m33*m66*L4r*RSe563p*
         RS356pe - 4./9.*m33*m66*L4r*RSe463p*RS346pe - 4./9.*m33*m66*
         L4r*RSe453p*RS345pe + 4./9.*m33*m66*L4r*RSe561p*RS356pe + 4./9.
         *m33*m66*L4r*RSe461p*RS346pe + 4./9.*m33*m66*L4r*RSe451p*
         RS345pe - 8./3.*m33*m55*L7r*RSe463p + 8./3.*m33*m55*L7r*
         RSe461p + 8./3.*m33*m55*L6r*RSe45613p - 8./3.*m33*m55*L6r*
         RSe45633p - 4./9.*m33*m55*L5r*RSe463p*RS346pe + 4./9.*m33*m55*
         L5r*RSe461p*RS346pe - 4./3.*m33*m55*L4r*RSe45613p + 4./3.*m33*
         m55*L4r*RSe45633p - 4./9.*m33*m55*L4r*RSe563p*RS356pe - 4./9.*
         m33*m55*L4r*RSe463p*RS346pe - 4./9.*m33*m55*L4r*RSe453p*
         RS345pe + 4./9.*m33*m55*L4r*RSe561p*RS356pe + 4./9.*m33*m55*
         L4r*RSe461p*RS346pe + 4./9.*m33*m55*L4r*RSe451p*RS345pe - 8./3.
         *m33*m44*L7r*RSe563p + 8./3.*m33*m44*L7r*RSe561p + 8./3.*m33*
         m44*L6r*RSe45613p - 8./3.*m33*m44*L6r*RSe45633p - 4./9.*m33*
         m44*L5r*RSe563p*RS356pe + 4./9.*m33*m44*L5r*RSe561p*RS356pe - 
         4./3.*m33*m44*L4r*RSe45613p );

      F13p6x23LV +=  + BbV(m33,me2,L) * ( 4./3.*m33*m44*L4r*RSe45633p
          - 4./9.*m33*m44*L4r*RSe563p*RS356pe - 4./9.*m33*m44*L4r*
         RSe463p*RS346pe - 4./9.*m33*m44*L4r*RSe453p*RS345pe + 4./9.*
         m33*m44*L4r*RSe561p*RS356pe + 4./9.*m33*m44*L4r*RSe461p*
         RS346pe + 4./9.*m33*m44*L4r*RSe451p*RS345pe + 8./3.*pow(m33,2)
         *L8r*RSe45613p - 8./3.*pow(m33,2)*L8r*RSe45633p - 4./3.*pow(
         m33,2)*L5r*RSe45613p + 4./3.*pow(m33,2)*L5r*RSe45633p );

      F13p6x23LV +=  + BbV(m33,L) * ( m66*L4r*RD3456ep + 4./9.*pow(
         m66,2)*L8r*pow(RS345pe,2) + 4./9.*pow(m66,2)*L7r*pow(
         RS345pe,2) + 4./9.*pow(m66,2)*L6r*pow(RS345pe,2) + m55*L4r*
         RD3456ep + 8./9.*m55*m66*L7r*RS346pe*RS345pe + 4./9.*m55*m66*
         L6r*pow(RS345pe,2) + 4./9.*m55*m66*L6r*pow(RS346pe,2) + 4./9.*
         pow(m55,2)*L8r*pow(RS346pe,2) + 4./9.*pow(m55,2)*L7r*pow(
         RS346pe,2) + 4./9.*pow(m55,2)*L6r*pow(RS346pe,2) + m44*L4r*
         RD3456ep + 8./9.*m44*m66*L7r*RS356pe*RS345pe + 4./9.*m44*m66*
         L6r*pow(RS345pe,2) + 4./9.*m44*m66*L6r*pow(RS356pe,2) + 8./9.*
         m44*m55*L7r*RS356pe*RS346pe + 4./9.*m44*m55*L6r*pow(RS346pe,2)
          + 4./9.*m44*m55*L6r*pow(RS356pe,2) + 4./9.*pow(m44,2)*L8r*
         pow(RS356pe,2) + 4./9.*pow(m44,2)*L7r*pow(RS356pe,2) + 4./9.*
         pow(m44,2)*L6r*pow(RS356pe,2) + 1./2.*m33*L5r*RD3456ep + 5./3.
         *m33*L3r*RD3456ep + 2./3.*m33*L0r*RD3456ep - 8./3.*m33*m66*L7r
         *RS345pe + 8./3.*m33*m66*L6r*RS34561pe - 8./3.*m33*m66*L6r*
         RC3456pe );

      F13p6x23LV +=  + BbV(m33,L) * (  - 2./9.*m33*m66*L5r*pow(
         RS345pe,2) - 4./3.*m33*m66*L4r*RS34561pe + 4./3.*m33*m66*L4r*
         RC3456pe - 2./9.*m33*m66*L4r*pow(RS345pe,2) - 2./9.*m33*m66*
         L4r*pow(RS346pe,2) - 2./9.*m33*m66*L4r*pow(RS356pe,2) - 8./3.*
         m33*m55*L7r*RS346pe + 8./3.*m33*m55*L6r*RS34561pe - 8./3.*m33*
         m55*L6r*RC3456pe - 2./9.*m33*m55*L5r*pow(RS346pe,2) - 4./3.*
         m33*m55*L4r*RS34561pe + 4./3.*m33*m55*L4r*RC3456pe - 2./9.*m33
         *m55*L4r*pow(RS345pe,2) - 2./9.*m33*m55*L4r*pow(RS346pe,2) - 2.
         /9.*m33*m55*L4r*pow(RS356pe,2) - 8./3.*m33*m44*L7r*RS356pe + 8.
         /3.*m33*m44*L6r*RS34561pe - 8./3.*m33*m44*L6r*RC3456pe - 2./9.
         *m33*m44*L5r*pow(RS356pe,2) - 4./3.*m33*m44*L4r*RS34561pe + 4./
         3.*m33*m44*L4r*RC3456pe - 2./9.*m33*m44*L4r*pow(RS345pe,2) - 2.
         /9.*m33*m44*L4r*pow(RS346pe,2) - 2./9.*m33*m44*L4r*pow(
         RS356pe,2) + 8./3.*pow(m33,2)*L8r*RS34561pe - 8./3.*pow(m33,2)
         *L8r*RC3456pe + 4*pow(m33,2)*L7r - 4./3.*pow(m33,2)*L5r*
         RS34561pe );

      F13p6x23LV +=  + BbV(m33,L) * ( 4./3.*pow(m33,2)*L5r*RC3456pe - 1.
         /6.*m11*L5r*RD3456ep );

      F13p6x23LV +=  + BbV(m34,L) * ( 2*m44*m66*L6r + 2*m44*m55*L6r + 
         pow(m44,2)*L8r + 2*pow(m44,2)*L6r - 2*m34*m66*L4r - 2*m34*m55*
         L4r - m34*m44*L5r - 2*m34*m44*L4r + 2*m33*m66*L6r + 2*m33*m55*
         L6r + 2*m33*m44*L8r + 2*m33*m44*L6r - m33*m34*L5r + pow(m33,2)
         *L8r );

      F13p6x23LV +=  + BbV(m35,L) * ( 2*m55*m66*L6r + pow(m55,2)*L8r + 
         2*pow(m55,2)*L6r + 2*m44*m55*L6r - 2*m35*m66*L4r - m35*m55*L5r
          - 2*m35*m55*L4r - 2*m35*m44*L4r + 2*m33*m66*L6r + 2*m33*m55*
         L8r + 2*m33*m55*L6r + 2*m33*m44*L6r - m33*m35*L5r + pow(m33,2)
         *L8r );

      F13p6x23LV +=  + BbV(m36,L) * ( pow(m66,2)*L8r + 2*pow(m66,2)*L6r
          + 2*m55*m66*L6r + 2*m44*m66*L6r - m36*m66*L5r - 2*m36*m66*L4r
          - 2*m36*m55*L4r - 2*m36*m44*L4r + 2*m33*m66*L8r + 2*m33*m66*
         L6r + 2*m33*m55*L6r + 2*m33*m44*L6r - m33*m36*L5r + pow(m33,2)
         *L8r );

      F13p6x23LV +=  + BbV(mp2,me2,L) * (  - 16./27.*m66*mp2*L5r*
         RSp453e*RSe453p + 16./27.*m66*mp2*L5r*RSe451p*RSp453e + 16./27.
         *m66*mp2*L5r*RSp451e*RSe453p - 16./27.*m66*mp2*L5r*RSp451e*
         RSe451p - 16./27.*m66*mp2*L4r*RSp563e*RSe563p - 16./27.*m66*
         mp2*L4r*RSp463e*RSe463p - 16./27.*m66*mp2*L4r*RSp453e*RSe453p
          + 16./27.*m66*mp2*L4r*RSe561p*RSp563e + 16./27.*m66*mp2*L4r*
         RSe461p*RSp463e + 16./27.*m66*mp2*L4r*RSe451p*RSp453e + 16./27.
         *m66*mp2*L4r*RSp561e*RSe563p - 16./27.*m66*mp2*L4r*RSp561e*
         RSe561p + 16./27.*m66*mp2*L4r*RSp461e*RSe463p - 16./27.*m66*
         mp2*L4r*RSp461e*RSe461p + 16./27.*m66*mp2*L4r*RSp451e*RSe453p
          - 16./27.*m66*mp2*L4r*RSp451e*RSe451p + 32./27.*pow(m66,2)*
         L8r*RSp453e*RSe453p - 32./27.*pow(m66,2)*L8r*RSe451p*RSp453e
          - 32./27.*pow(m66,2)*L8r*RSp451e*RSe453p + 32./27.*pow(m66,2)
         *L8r*RSp451e*RSe451p + 32./27.*pow(m66,2)*L7r*RSp453e*RSe453p
          - 32./27.*pow(m66,2)*L7r*RSe451p*RSp453e - 32./27.*pow(m66,2)
         *L7r*RSp451e*RSe453p );

      F13p6x23LV +=  + BbV(mp2,me2,L) * ( 32./27.*pow(m66,2)*L7r*
         RSp451e*RSe451p + 32./27.*pow(m66,2)*L6r*RSp453e*RSe453p - 32./
         27.*pow(m66,2)*L6r*RSe451p*RSp453e - 32./27.*pow(m66,2)*L6r*
         RSp451e*RSe453p + 32./27.*pow(m66,2)*L6r*RSp451e*RSe451p - 16./
         27.*m55*mp2*L5r*RSp463e*RSe463p + 16./27.*m55*mp2*L5r*RSe461p*
         RSp463e + 16./27.*m55*mp2*L5r*RSp461e*RSe463p - 16./27.*m55*
         mp2*L5r*RSp461e*RSe461p - 16./27.*m55*mp2*L4r*RSp563e*RSe563p
          - 16./27.*m55*mp2*L4r*RSp463e*RSe463p - 16./27.*m55*mp2*L4r*
         RSp453e*RSe453p + 16./27.*m55*mp2*L4r*RSe561p*RSp563e + 16./27.
         *m55*mp2*L4r*RSe461p*RSp463e + 16./27.*m55*mp2*L4r*RSe451p*
         RSp453e + 16./27.*m55*mp2*L4r*RSp561e*RSe563p - 16./27.*m55*
         mp2*L4r*RSp561e*RSe561p + 16./27.*m55*mp2*L4r*RSp461e*RSe463p
          - 16./27.*m55*mp2*L4r*RSp461e*RSe461p + 16./27.*m55*mp2*L4r*
         RSp451e*RSe453p - 16./27.*m55*mp2*L4r*RSp451e*RSe451p + 32./27.
         *m55*m66*L7r*RSp463e*RSe453p + 32./27.*m55*m66*L7r*RSp453e*
         RSe463p );

      F13p6x23LV +=  + BbV(mp2,me2,L) * (  - 32./27.*m55*m66*L7r*
         RSe461p*RSp453e - 32./27.*m55*m66*L7r*RSe451p*RSp463e - 32./27.
         *m55*m66*L7r*RSp461e*RSe453p + 32./27.*m55*m66*L7r*RSp461e*
         RSe451p - 32./27.*m55*m66*L7r*RSp451e*RSe463p + 32./27.*m55*
         m66*L7r*RSp451e*RSe461p + 32./27.*m55*m66*L6r*RSp463e*RSe463p
          + 32./27.*m55*m66*L6r*RSp453e*RSe453p - 32./27.*m55*m66*L6r*
         RSe461p*RSp463e - 32./27.*m55*m66*L6r*RSe451p*RSp453e - 32./27.
         *m55*m66*L6r*RSp461e*RSe463p + 32./27.*m55*m66*L6r*RSp461e*
         RSe461p - 32./27.*m55*m66*L6r*RSp451e*RSe453p + 32./27.*m55*
         m66*L6r*RSp451e*RSe451p + 32./27.*pow(m55,2)*L8r*RSp463e*
         RSe463p - 32./27.*pow(m55,2)*L8r*RSe461p*RSp463e - 32./27.*
         pow(m55,2)*L8r*RSp461e*RSe463p + 32./27.*pow(m55,2)*L8r*
         RSp461e*RSe461p + 32./27.*pow(m55,2)*L7r*RSp463e*RSe463p - 32./
         27.*pow(m55,2)*L7r*RSe461p*RSp463e - 32./27.*pow(m55,2)*L7r*
         RSp461e*RSe463p + 32./27.*pow(m55,2)*L7r*RSp461e*RSe461p + 32./
         27.*pow(m55,2)*L6r*RSp463e*RSe463p );

      F13p6x23LV +=  + BbV(mp2,me2,L) * (  - 32./27.*pow(m55,2)*L6r*
         RSe461p*RSp463e - 32./27.*pow(m55,2)*L6r*RSp461e*RSe463p + 32./
         27.*pow(m55,2)*L6r*RSp461e*RSe461p - 16./27.*m44*mp2*L5r*
         RSp563e*RSe563p + 16./27.*m44*mp2*L5r*RSe561p*RSp563e + 16./27.
         *m44*mp2*L5r*RSp561e*RSe563p - 16./27.*m44*mp2*L5r*RSp561e*
         RSe561p - 16./27.*m44*mp2*L4r*RSp563e*RSe563p - 16./27.*m44*
         mp2*L4r*RSp463e*RSe463p - 16./27.*m44*mp2*L4r*RSp453e*RSe453p
          + 16./27.*m44*mp2*L4r*RSe561p*RSp563e + 16./27.*m44*mp2*L4r*
         RSe461p*RSp463e + 16./27.*m44*mp2*L4r*RSe451p*RSp453e + 16./27.
         *m44*mp2*L4r*RSp561e*RSe563p - 16./27.*m44*mp2*L4r*RSp561e*
         RSe561p + 16./27.*m44*mp2*L4r*RSp461e*RSe463p - 16./27.*m44*
         mp2*L4r*RSp461e*RSe461p + 16./27.*m44*mp2*L4r*RSp451e*RSe453p
          - 16./27.*m44*mp2*L4r*RSp451e*RSe451p + 32./27.*m44*m66*L7r*
         RSp563e*RSe453p + 32./27.*m44*m66*L7r*RSp453e*RSe563p - 32./27.
         *m44*m66*L7r*RSe561p*RSp453e - 32./27.*m44*m66*L7r*RSe451p*
         RSp563e );

      F13p6x23LV +=  + BbV(mp2,me2,L) * (  - 32./27.*m44*m66*L7r*
         RSp561e*RSe453p + 32./27.*m44*m66*L7r*RSp561e*RSe451p - 32./27.
         *m44*m66*L7r*RSp451e*RSe563p + 32./27.*m44*m66*L7r*RSp451e*
         RSe561p + 32./27.*m44*m66*L6r*RSp563e*RSe563p + 32./27.*m44*
         m66*L6r*RSp453e*RSe453p - 32./27.*m44*m66*L6r*RSe561p*RSp563e
          - 32./27.*m44*m66*L6r*RSe451p*RSp453e - 32./27.*m44*m66*L6r*
         RSp561e*RSe563p + 32./27.*m44*m66*L6r*RSp561e*RSe561p - 32./27.
         *m44*m66*L6r*RSp451e*RSe453p + 32./27.*m44*m66*L6r*RSp451e*
         RSe451p + 32./27.*m44*m55*L7r*RSp563e*RSe463p + 32./27.*m44*
         m55*L7r*RSp463e*RSe563p - 32./27.*m44*m55*L7r*RSe561p*RSp463e
          - 32./27.*m44*m55*L7r*RSe461p*RSp563e - 32./27.*m44*m55*L7r*
         RSp561e*RSe463p + 32./27.*m44*m55*L7r*RSp561e*RSe461p - 32./27.
         *m44*m55*L7r*RSp461e*RSe563p + 32./27.*m44*m55*L7r*RSp461e*
         RSe561p + 32./27.*m44*m55*L6r*RSp563e*RSe563p + 32./27.*m44*
         m55*L6r*RSp463e*RSe463p - 32./27.*m44*m55*L6r*RSe561p*RSp563e
          - 32./27.*m44*m55*L6r*RSe461p*RSp463e );

      F13p6x23LV +=  + BbV(mp2,me2,L) * (  - 32./27.*m44*m55*L6r*
         RSp561e*RSe563p + 32./27.*m44*m55*L6r*RSp561e*RSe561p - 32./27.
         *m44*m55*L6r*RSp461e*RSe463p + 32./27.*m44*m55*L6r*RSp461e*
         RSe461p + 32./27.*pow(m44,2)*L8r*RSp563e*RSe563p - 32./27.*
         pow(m44,2)*L8r*RSe561p*RSp563e - 32./27.*pow(m44,2)*L8r*
         RSp561e*RSe563p + 32./27.*pow(m44,2)*L8r*RSp561e*RSe561p + 32./
         27.*pow(m44,2)*L7r*RSp563e*RSe563p - 32./27.*pow(m44,2)*L7r*
         RSe561p*RSp563e - 32./27.*pow(m44,2)*L7r*RSp561e*RSe563p + 32./
         27.*pow(m44,2)*L7r*RSp561e*RSe561p + 32./27.*pow(m44,2)*L6r*
         RSp563e*RSe563p - 32./27.*pow(m44,2)*L6r*RSe561p*RSp563e - 32./
         27.*pow(m44,2)*L6r*RSp561e*RSe563p + 32./27.*pow(m44,2)*L6r*
         RSp561e*RSe561p );

      F13p6x23LV +=  + BbV(mp2,L) * (  - 2./9.*m66*mp2*L5r*pow(
         RSp453e,2) + 4./9.*m66*mp2*L5r*RSp451e*RSp453e - 2./9.*m66*mp2
         *L5r*pow(RSp451e,2) - 2./9.*m66*mp2*L4r*pow(RSp563e,2) - 2./9.
         *m66*mp2*L4r*pow(RSp463e,2) - 2./9.*m66*mp2*L4r*pow(RSp453e,2)
          + 4./9.*m66*mp2*L4r*RSp561e*RSp563e - 2./9.*m66*mp2*L4r*pow(
         RSp561e,2) + 4./9.*m66*mp2*L4r*RSp461e*RSp463e - 2./9.*m66*mp2
         *L4r*pow(RSp461e,2) + 4./9.*m66*mp2*L4r*RSp451e*RSp453e - 2./9.
         *m66*mp2*L4r*pow(RSp451e,2) + 4./9.*pow(m66,2)*L8r*pow(
         RSp453e,2) - 8./9.*pow(m66,2)*L8r*RSp451e*RSp453e + 4./9.*pow(
         m66,2)*L8r*pow(RSp451e,2) + 4./9.*pow(m66,2)*L7r*pow(
         RSp453e,2) - 8./9.*pow(m66,2)*L7r*RSp451e*RSp453e + 4./9.*pow(
         m66,2)*L7r*pow(RSp451e,2) + 4./9.*pow(m66,2)*L6r*pow(
         RSp453e,2) - 8./9.*pow(m66,2)*L6r*RSp451e*RSp453e + 4./9.*pow(
         m66,2)*L6r*pow(RSp451e,2) - 2./9.*m55*mp2*L5r*pow(RSp463e,2)
          + 4./9.*m55*mp2*L5r*RSp461e*RSp463e - 2./9.*m55*mp2*L5r*pow(
         RSp461e,2) );

      F13p6x23LV +=  + BbV(mp2,L) * (  - 2./9.*m55*mp2*L4r*pow(
         RSp563e,2) - 2./9.*m55*mp2*L4r*pow(RSp463e,2) - 2./9.*m55*mp2*
         L4r*pow(RSp453e,2) + 4./9.*m55*mp2*L4r*RSp561e*RSp563e - 2./9.
         *m55*mp2*L4r*pow(RSp561e,2) + 4./9.*m55*mp2*L4r*RSp461e*
         RSp463e - 2./9.*m55*mp2*L4r*pow(RSp461e,2) + 4./9.*m55*mp2*L4r
         *RSp451e*RSp453e - 2./9.*m55*mp2*L4r*pow(RSp451e,2) + 8./9.*
         m55*m66*L7r*RSp453e*RSp463e - 8./9.*m55*m66*L7r*RSp461e*
         RSp453e - 8./9.*m55*m66*L7r*RSp451e*RSp463e + 8./9.*m55*m66*
         L7r*RSp451e*RSp461e + 4./9.*m55*m66*L6r*pow(RSp463e,2) + 4./9.
         *m55*m66*L6r*pow(RSp453e,2) - 8./9.*m55*m66*L6r*RSp461e*
         RSp463e + 4./9.*m55*m66*L6r*pow(RSp461e,2) - 8./9.*m55*m66*L6r
         *RSp451e*RSp453e + 4./9.*m55*m66*L6r*pow(RSp451e,2) + 4./9.*
         pow(m55,2)*L8r*pow(RSp463e,2) - 8./9.*pow(m55,2)*L8r*RSp461e*
         RSp463e + 4./9.*pow(m55,2)*L8r*pow(RSp461e,2) + 4./9.*pow(
         m55,2)*L7r*pow(RSp463e,2) - 8./9.*pow(m55,2)*L7r*RSp461e*
         RSp463e );

      F13p6x23LV +=  + BbV(mp2,L) * ( 4./9.*pow(m55,2)*L7r*pow(
         RSp461e,2) + 4./9.*pow(m55,2)*L6r*pow(RSp463e,2) - 8./9.*pow(
         m55,2)*L6r*RSp461e*RSp463e + 4./9.*pow(m55,2)*L6r*pow(
         RSp461e,2) - 2./9.*m44*mp2*L5r*pow(RSp563e,2) + 4./9.*m44*mp2*
         L5r*RSp561e*RSp563e - 2./9.*m44*mp2*L5r*pow(RSp561e,2) - 2./9.
         *m44*mp2*L4r*pow(RSp563e,2) - 2./9.*m44*mp2*L4r*pow(RSp463e,2)
          - 2./9.*m44*mp2*L4r*pow(RSp453e,2) + 4./9.*m44*mp2*L4r*
         RSp561e*RSp563e - 2./9.*m44*mp2*L4r*pow(RSp561e,2) + 4./9.*m44
         *mp2*L4r*RSp461e*RSp463e - 2./9.*m44*mp2*L4r*pow(RSp461e,2) + 
         4./9.*m44*mp2*L4r*RSp451e*RSp453e - 2./9.*m44*mp2*L4r*pow(
         RSp451e,2) + 8./9.*m44*m66*L7r*RSp453e*RSp563e - 8./9.*m44*m66
         *L7r*RSp561e*RSp453e - 8./9.*m44*m66*L7r*RSp451e*RSp563e + 8./
         9.*m44*m66*L7r*RSp451e*RSp561e + 4./9.*m44*m66*L6r*pow(
         RSp563e,2) + 4./9.*m44*m66*L6r*pow(RSp453e,2) - 8./9.*m44*m66*
         L6r*RSp561e*RSp563e + 4./9.*m44*m66*L6r*pow(RSp561e,2) - 8./9.
         *m44*m66*L6r*RSp451e*RSp453e );

      F13p6x23LV +=  + BbV(mp2,L) * ( 4./9.*m44*m66*L6r*pow(RSp451e,2)
          + 8./9.*m44*m55*L7r*RSp463e*RSp563e - 8./9.*m44*m55*L7r*
         RSp561e*RSp463e - 8./9.*m44*m55*L7r*RSp461e*RSp563e + 8./9.*
         m44*m55*L7r*RSp461e*RSp561e + 4./9.*m44*m55*L6r*pow(RSp563e,2)
          + 4./9.*m44*m55*L6r*pow(RSp463e,2) - 8./9.*m44*m55*L6r*
         RSp561e*RSp563e + 4./9.*m44*m55*L6r*pow(RSp561e,2) - 8./9.*m44
         *m55*L6r*RSp461e*RSp463e + 4./9.*m44*m55*L6r*pow(RSp461e,2) + 
         4./9.*pow(m44,2)*L8r*pow(RSp563e,2) - 8./9.*pow(m44,2)*L8r*
         RSp561e*RSp563e + 4./9.*pow(m44,2)*L8r*pow(RSp561e,2) + 4./9.*
         pow(m44,2)*L7r*pow(RSp563e,2) - 8./9.*pow(m44,2)*L7r*RSp561e*
         RSp563e + 4./9.*pow(m44,2)*L7r*pow(RSp561e,2) + 4./9.*pow(
         m44,2)*L6r*pow(RSp563e,2) - 8./9.*pow(m44,2)*L6r*RSp561e*
         RSp563e + 4./9.*pow(m44,2)*L6r*pow(RSp561e,2) );

      F13p6x23LV +=  + BbV(me2,mp2,L) * ( 4./27.*m66*me2*L5r*RSp453e*
         RSe453p - 4./27.*m66*me2*L5r*RSe451p*RSp453e - 4./27.*m66*me2*
         L5r*RSp451e*RSe453p + 4./27.*m66*me2*L5r*RSp451e*RSe451p + 4./
         27.*m66*me2*L4r*RSp563e*RSe563p + 4./27.*m66*me2*L4r*RSp463e*
         RSe463p + 4./27.*m66*me2*L4r*RSp453e*RSe453p - 4./27.*m66*me2*
         L4r*RSe561p*RSp563e - 4./27.*m66*me2*L4r*RSe461p*RSp463e - 4./
         27.*m66*me2*L4r*RSe451p*RSp453e - 4./27.*m66*me2*L4r*RSp561e*
         RSe563p + 4./27.*m66*me2*L4r*RSp561e*RSe561p - 4./27.*m66*me2*
         L4r*RSp461e*RSe463p + 4./27.*m66*me2*L4r*RSp461e*RSe461p - 4./
         27.*m66*me2*L4r*RSp451e*RSe453p + 4./27.*m66*me2*L4r*RSp451e*
         RSe451p - 8./27.*pow(m66,2)*L8r*RSp453e*RSe453p + 8./27.*pow(
         m66,2)*L8r*RSe451p*RSp453e + 8./27.*pow(m66,2)*L8r*RSp451e*
         RSe453p - 8./27.*pow(m66,2)*L8r*RSp451e*RSe451p - 8./27.*pow(
         m66,2)*L7r*RSp453e*RSe453p + 8./27.*pow(m66,2)*L7r*RSe451p*
         RSp453e + 8./27.*pow(m66,2)*L7r*RSp451e*RSe453p - 8./27.*pow(
         m66,2)*L7r*RSp451e*RSe451p );

      F13p6x23LV +=  + BbV(me2,mp2,L) * (  - 8./27.*pow(m66,2)*L6r*
         RSp453e*RSe453p + 8./27.*pow(m66,2)*L6r*RSe451p*RSp453e + 8./
         27.*pow(m66,2)*L6r*RSp451e*RSe453p - 8./27.*pow(m66,2)*L6r*
         RSp451e*RSe451p + 4./27.*m55*me2*L5r*RSp463e*RSe463p - 4./27.*
         m55*me2*L5r*RSe461p*RSp463e - 4./27.*m55*me2*L5r*RSp461e*
         RSe463p + 4./27.*m55*me2*L5r*RSp461e*RSe461p + 4./27.*m55*me2*
         L4r*RSp563e*RSe563p + 4./27.*m55*me2*L4r*RSp463e*RSe463p + 4./
         27.*m55*me2*L4r*RSp453e*RSe453p - 4./27.*m55*me2*L4r*RSe561p*
         RSp563e - 4./27.*m55*me2*L4r*RSe461p*RSp463e - 4./27.*m55*me2*
         L4r*RSe451p*RSp453e - 4./27.*m55*me2*L4r*RSp561e*RSe563p + 4./
         27.*m55*me2*L4r*RSp561e*RSe561p - 4./27.*m55*me2*L4r*RSp461e*
         RSe463p + 4./27.*m55*me2*L4r*RSp461e*RSe461p - 4./27.*m55*me2*
         L4r*RSp451e*RSe453p + 4./27.*m55*me2*L4r*RSp451e*RSe451p - 8./
         27.*m55*m66*L7r*RSp463e*RSe453p - 8./27.*m55*m66*L7r*RSp453e*
         RSe463p + 8./27.*m55*m66*L7r*RSe461p*RSp453e + 8./27.*m55*m66*
         L7r*RSe451p*RSp463e );

      F13p6x23LV +=  + BbV(me2,mp2,L) * ( 8./27.*m55*m66*L7r*RSp461e*
         RSe453p - 8./27.*m55*m66*L7r*RSp461e*RSe451p + 8./27.*m55*m66*
         L7r*RSp451e*RSe463p - 8./27.*m55*m66*L7r*RSp451e*RSe461p - 8./
         27.*m55*m66*L6r*RSp463e*RSe463p - 8./27.*m55*m66*L6r*RSp453e*
         RSe453p + 8./27.*m55*m66*L6r*RSe461p*RSp463e + 8./27.*m55*m66*
         L6r*RSe451p*RSp453e + 8./27.*m55*m66*L6r*RSp461e*RSe463p - 8./
         27.*m55*m66*L6r*RSp461e*RSe461p + 8./27.*m55*m66*L6r*RSp451e*
         RSe453p - 8./27.*m55*m66*L6r*RSp451e*RSe451p - 8./27.*pow(
         m55,2)*L8r*RSp463e*RSe463p + 8./27.*pow(m55,2)*L8r*RSe461p*
         RSp463e + 8./27.*pow(m55,2)*L8r*RSp461e*RSe463p - 8./27.*pow(
         m55,2)*L8r*RSp461e*RSe461p - 8./27.*pow(m55,2)*L7r*RSp463e*
         RSe463p + 8./27.*pow(m55,2)*L7r*RSe461p*RSp463e + 8./27.*pow(
         m55,2)*L7r*RSp461e*RSe463p - 8./27.*pow(m55,2)*L7r*RSp461e*
         RSe461p - 8./27.*pow(m55,2)*L6r*RSp463e*RSe463p + 8./27.*pow(
         m55,2)*L6r*RSe461p*RSp463e + 8./27.*pow(m55,2)*L6r*RSp461e*
         RSe463p );

      F13p6x23LV +=  + BbV(me2,mp2,L) * (  - 8./27.*pow(m55,2)*L6r*
         RSp461e*RSe461p + 4./27.*m44*me2*L5r*RSp563e*RSe563p - 4./27.*
         m44*me2*L5r*RSe561p*RSp563e - 4./27.*m44*me2*L5r*RSp561e*
         RSe563p + 4./27.*m44*me2*L5r*RSp561e*RSe561p + 4./27.*m44*me2*
         L4r*RSp563e*RSe563p + 4./27.*m44*me2*L4r*RSp463e*RSe463p + 4./
         27.*m44*me2*L4r*RSp453e*RSe453p - 4./27.*m44*me2*L4r*RSe561p*
         RSp563e - 4./27.*m44*me2*L4r*RSe461p*RSp463e - 4./27.*m44*me2*
         L4r*RSe451p*RSp453e - 4./27.*m44*me2*L4r*RSp561e*RSe563p + 4./
         27.*m44*me2*L4r*RSp561e*RSe561p - 4./27.*m44*me2*L4r*RSp461e*
         RSe463p + 4./27.*m44*me2*L4r*RSp461e*RSe461p - 4./27.*m44*me2*
         L4r*RSp451e*RSe453p + 4./27.*m44*me2*L4r*RSp451e*RSe451p - 8./
         27.*m44*m66*L7r*RSp563e*RSe453p - 8./27.*m44*m66*L7r*RSp453e*
         RSe563p + 8./27.*m44*m66*L7r*RSe561p*RSp453e + 8./27.*m44*m66*
         L7r*RSe451p*RSp563e + 8./27.*m44*m66*L7r*RSp561e*RSe453p - 8./
         27.*m44*m66*L7r*RSp561e*RSe451p + 8./27.*m44*m66*L7r*RSp451e*
         RSe563p );

      F13p6x23LV +=  + BbV(me2,mp2,L) * (  - 8./27.*m44*m66*L7r*RSp451e
         *RSe561p - 8./27.*m44*m66*L6r*RSp563e*RSe563p - 8./27.*m44*m66
         *L6r*RSp453e*RSe453p + 8./27.*m44*m66*L6r*RSe561p*RSp563e + 8./
         27.*m44*m66*L6r*RSe451p*RSp453e + 8./27.*m44*m66*L6r*RSp561e*
         RSe563p - 8./27.*m44*m66*L6r*RSp561e*RSe561p + 8./27.*m44*m66*
         L6r*RSp451e*RSe453p - 8./27.*m44*m66*L6r*RSp451e*RSe451p - 8./
         27.*m44*m55*L7r*RSp563e*RSe463p - 8./27.*m44*m55*L7r*RSp463e*
         RSe563p + 8./27.*m44*m55*L7r*RSe561p*RSp463e + 8./27.*m44*m55*
         L7r*RSe461p*RSp563e + 8./27.*m44*m55*L7r*RSp561e*RSe463p - 8./
         27.*m44*m55*L7r*RSp561e*RSe461p + 8./27.*m44*m55*L7r*RSp461e*
         RSe563p - 8./27.*m44*m55*L7r*RSp461e*RSe561p - 8./27.*m44*m55*
         L6r*RSp563e*RSe563p - 8./27.*m44*m55*L6r*RSp463e*RSe463p + 8./
         27.*m44*m55*L6r*RSe561p*RSp563e + 8./27.*m44*m55*L6r*RSe461p*
         RSp463e + 8./27.*m44*m55*L6r*RSp561e*RSe563p - 8./27.*m44*m55*
         L6r*RSp561e*RSe561p + 8./27.*m44*m55*L6r*RSp461e*RSe463p - 8./
         27.*m44*m55*L6r*RSp461e*RSe461p );

      F13p6x23LV +=  + BbV(me2,mp2,L) * (  - 8./27.*pow(m44,2)*L8r*
         RSp563e*RSe563p + 8./27.*pow(m44,2)*L8r*RSe561p*RSp563e + 8./
         27.*pow(m44,2)*L8r*RSp561e*RSe563p - 8./27.*pow(m44,2)*L8r*
         RSp561e*RSe561p - 8./27.*pow(m44,2)*L7r*RSp563e*RSe563p + 8./
         27.*pow(m44,2)*L7r*RSe561p*RSp563e + 8./27.*pow(m44,2)*L7r*
         RSp561e*RSe563p - 8./27.*pow(m44,2)*L7r*RSp561e*RSe561p - 8./
         27.*pow(m44,2)*L6r*RSp563e*RSe563p + 8./27.*pow(m44,2)*L6r*
         RSe561p*RSp563e + 8./27.*pow(m44,2)*L6r*RSp561e*RSe563p - 8./
         27.*pow(m44,2)*L6r*RSp561e*RSe561p );

      F13p6x23LV +=  + BbV(me2,L) * (  - 2./9.*m66*me2*L5r*pow(
         RSe453p,2) + 4./9.*m66*me2*L5r*RSe451p*RSe453p - 2./9.*m66*me2
         *L5r*pow(RSe451p,2) - 2./9.*m66*me2*L4r*pow(RSe563p,2) - 2./9.
         *m66*me2*L4r*pow(RSe463p,2) - 2./9.*m66*me2*L4r*pow(RSe453p,2)
          + 4./9.*m66*me2*L4r*RSe561p*RSe563p - 2./9.*m66*me2*L4r*pow(
         RSe561p,2) + 4./9.*m66*me2*L4r*RSe461p*RSe463p - 2./9.*m66*me2
         *L4r*pow(RSe461p,2) + 4./9.*m66*me2*L4r*RSe451p*RSe453p - 2./9.
         *m66*me2*L4r*pow(RSe451p,2) + 4./9.*pow(m66,2)*L8r*pow(
         RSe453p,2) - 8./9.*pow(m66,2)*L8r*RSe451p*RSe453p + 4./9.*pow(
         m66,2)*L8r*pow(RSe451p,2) + 4./9.*pow(m66,2)*L7r*pow(
         RSe453p,2) - 8./9.*pow(m66,2)*L7r*RSe451p*RSe453p + 4./9.*pow(
         m66,2)*L7r*pow(RSe451p,2) + 4./9.*pow(m66,2)*L6r*pow(
         RSe453p,2) - 8./9.*pow(m66,2)*L6r*RSe451p*RSe453p + 4./9.*pow(
         m66,2)*L6r*pow(RSe451p,2) - 2./9.*m55*me2*L5r*pow(RSe463p,2)
          + 4./9.*m55*me2*L5r*RSe461p*RSe463p - 2./9.*m55*me2*L5r*pow(
         RSe461p,2) );

      F13p6x23LV +=  + BbV(me2,L) * (  - 2./9.*m55*me2*L4r*pow(
         RSe563p,2) - 2./9.*m55*me2*L4r*pow(RSe463p,2) - 2./9.*m55*me2*
         L4r*pow(RSe453p,2) + 4./9.*m55*me2*L4r*RSe561p*RSe563p - 2./9.
         *m55*me2*L4r*pow(RSe561p,2) + 4./9.*m55*me2*L4r*RSe461p*
         RSe463p - 2./9.*m55*me2*L4r*pow(RSe461p,2) + 4./9.*m55*me2*L4r
         *RSe451p*RSe453p - 2./9.*m55*me2*L4r*pow(RSe451p,2) + 8./9.*
         m55*m66*L7r*RSe453p*RSe463p - 8./9.*m55*m66*L7r*RSe461p*
         RSe453p - 8./9.*m55*m66*L7r*RSe451p*RSe463p + 8./9.*m55*m66*
         L7r*RSe451p*RSe461p + 4./9.*m55*m66*L6r*pow(RSe463p,2) + 4./9.
         *m55*m66*L6r*pow(RSe453p,2) - 8./9.*m55*m66*L6r*RSe461p*
         RSe463p + 4./9.*m55*m66*L6r*pow(RSe461p,2) - 8./9.*m55*m66*L6r
         *RSe451p*RSe453p + 4./9.*m55*m66*L6r*pow(RSe451p,2) + 4./9.*
         pow(m55,2)*L8r*pow(RSe463p,2) - 8./9.*pow(m55,2)*L8r*RSe461p*
         RSe463p + 4./9.*pow(m55,2)*L8r*pow(RSe461p,2) + 4./9.*pow(
         m55,2)*L7r*pow(RSe463p,2) - 8./9.*pow(m55,2)*L7r*RSe461p*
         RSe463p );

      F13p6x23LV +=  + BbV(me2,L) * ( 4./9.*pow(m55,2)*L7r*pow(
         RSe461p,2) + 4./9.*pow(m55,2)*L6r*pow(RSe463p,2) - 8./9.*pow(
         m55,2)*L6r*RSe461p*RSe463p + 4./9.*pow(m55,2)*L6r*pow(
         RSe461p,2) - 2./9.*m44*me2*L5r*pow(RSe563p,2) + 4./9.*m44*me2*
         L5r*RSe561p*RSe563p - 2./9.*m44*me2*L5r*pow(RSe561p,2) - 2./9.
         *m44*me2*L4r*pow(RSe563p,2) - 2./9.*m44*me2*L4r*pow(RSe463p,2)
          - 2./9.*m44*me2*L4r*pow(RSe453p,2) + 4./9.*m44*me2*L4r*
         RSe561p*RSe563p - 2./9.*m44*me2*L4r*pow(RSe561p,2) + 4./9.*m44
         *me2*L4r*RSe461p*RSe463p - 2./9.*m44*me2*L4r*pow(RSe461p,2) + 
         4./9.*m44*me2*L4r*RSe451p*RSe453p - 2./9.*m44*me2*L4r*pow(
         RSe451p,2) + 8./9.*m44*m66*L7r*RSe453p*RSe563p - 8./9.*m44*m66
         *L7r*RSe561p*RSe453p - 8./9.*m44*m66*L7r*RSe451p*RSe563p + 8./
         9.*m44*m66*L7r*RSe451p*RSe561p + 4./9.*m44*m66*L6r*pow(
         RSe563p,2) + 4./9.*m44*m66*L6r*pow(RSe453p,2) - 8./9.*m44*m66*
         L6r*RSe561p*RSe563p + 4./9.*m44*m66*L6r*pow(RSe561p,2) - 8./9.
         *m44*m66*L6r*RSe451p*RSe453p );

      F13p6x23LV +=  + BbV(me2,L) * ( 4./9.*m44*m66*L6r*pow(RSe451p,2)
          + 8./9.*m44*m55*L7r*RSe463p*RSe563p - 8./9.*m44*m55*L7r*
         RSe561p*RSe463p - 8./9.*m44*m55*L7r*RSe461p*RSe563p + 8./9.*
         m44*m55*L7r*RSe461p*RSe561p + 4./9.*m44*m55*L6r*pow(RSe563p,2)
          + 4./9.*m44*m55*L6r*pow(RSe463p,2) - 8./9.*m44*m55*L6r*
         RSe561p*RSe563p + 4./9.*m44*m55*L6r*pow(RSe561p,2) - 8./9.*m44
         *m55*L6r*RSe461p*RSe463p + 4./9.*m44*m55*L6r*pow(RSe461p,2) + 
         4./9.*pow(m44,2)*L8r*pow(RSe563p,2) - 8./9.*pow(m44,2)*L8r*
         RSe561p*RSe563p + 4./9.*pow(m44,2)*L8r*pow(RSe561p,2) + 4./9.*
         pow(m44,2)*L7r*pow(RSe563p,2) - 8./9.*pow(m44,2)*L7r*RSe561p*
         RSe563p + 4./9.*pow(m44,2)*L7r*pow(RSe561p,2) + 4./9.*pow(
         m44,2)*L6r*pow(RSe563p,2) - 8./9.*pow(m44,2)*L6r*RSe561p*
         RSe563p + 4./9.*pow(m44,2)*L6r*pow(RSe561p,2) );

      F13p6x23LV +=  + CbV(m11,L) * (  - 8./3.*m11*m66*L6r*RD1456ep + 4.
         /3.*m11*m66*L4r*RD1456ep - 8./3.*m11*m55*L6r*RD1456ep + 4./3.*
         m11*m55*L4r*RD1456ep - 8./3.*m11*m44*L6r*RD1456ep + 4./3.*m11*
         m44*L4r*RD1456ep - 8./3.*pow(m11,2)*L8r*RD1456ep + 4./3.*pow(
         m11,2)*L5r*RD1456ep );

      F13p6x23LV +=  + CbV(m33,L) * (  - 8./3.*m33*m66*L6r*RD3456ep + 4.
         /3.*m33*m66*L4r*RD3456ep - 8./3.*m33*m55*L6r*RD3456ep + 4./3.*
         m33*m55*L4r*RD3456ep - 8./3.*m33*m44*L6r*RD3456ep + 4./3.*m33*
         m44*L4r*RD3456ep - 8./3.*pow(m33,2)*L8r*RD3456ep + 4./3.*pow(
         m33,2)*L5r*RD3456ep );

      F13p6x23LV +=  + A23bV(m11,L) * (  - 2*L3r*RS14563pe - L3r*
         RC1456pe - 2*L0r*RC1456pe );

      F13p6x23LV +=  + A23bV(m13,L) * ( 6*L2r + 12*L1r );

      F13p6x23LV +=  + A23bV(m14,L) * ( 3*L3r + 6*L0r );

      F13p6x23LV +=  + A23bV(m15,L) * ( 3*L3r + 6*L0r );

      F13p6x23LV +=  + A23bV(m16,L) * ( 3*L3r + 6*L0r );

      F13p6x23LV +=  + A23bV(m33,L) * (  - 2*L3r*RS34561pe - L3r*
         RC3456pe - 2*L0r*RC3456pe );

      F13p6x23LV +=  + A23bV(m34,L) * ( 3*L3r + 6*L0r );

      F13p6x23LV +=  + A23bV(m35,L) * ( 3*L3r + 6*L0r );

      F13p6x23LV +=  + A23bV(m36,L) * ( 3*L3r + 6*L0r );

      F13p6x23LV +=  + A23bV(m44,L) * ( 6*L2r - 2*L2r*RS456pe );

      F13p6x23LV +=  + A23bV(m45,L) * ( 12*L2r );

      F13p6x23LV +=  + A23bV(m46,L) * ( 12*L2r );

      F13p6x23LV +=  + A23bV(m55,L) * ( 6*L2r - 2*L2r*RS546pe );

      F13p6x23LV +=  + A23bV(m56,L) * ( 12*L2r );

      F13p6x23LV +=  + A23bV(m66,L) * ( 6*L2r - 2*L2r*RS645pe );

      F13p6x23LV +=  + A23bV(mp2,L) * (  - 2*L3r*RSp45613e - L3r*
         RSp45633e - L3r*RSp45611e - 2*L2r*RSp564e - 2*L2r*RSp465e - 2*
         L2r*RSp456e - 2*L0r*RSp45633e - 2*L0r*RSp45611e );

      F13p6x23LV +=  + A23bV(me2,L) * (  - 2*L3r*RSe45613p - L3r*
         RSe45633p - L3r*RSe45611p - 2*L2r*RSe564p - 2*L2r*RSe465p - 2*
         L2r*RSe456p - 2*L0r*RSe45633p - 2*L0r*RSe45611p );

      F13p6x23LV +=  + B23bV(m11,L) * (  - L3r*RD1456ep - 2*L0r*
         RD1456ep );

      F13p6x23LV +=  + B23bV(m33,L) * (  - L3r*RD3456ep - 2*L0r*
         RD3456ep );

  return F13p6x23LV/pow(f0,4);
}

double fv2s3nf3p6RV(const quarkmassnf mass, const double L){
  vector<double> B0mq;
  double f0,mu0;
  int nq;
  mass.out(B0mq,f0,mu0,nq);
  if ( (nq != 5) || (B0mq.size() != 5)){
    cout << "ERROR: fv2s3nf3p6RV probably called with wrong number"
	 << "of quark masses\n";}
  // two masses: valence and sea quark mass
  double m11 = 2.*B0mq[0];
  double m33 = 2.*B0mq[1];
  double m44 = 2.*B0mq[2];
  double m55 = 2.*B0mq[3];
  double m66 = 2.*B0mq[4];
  double m13 = (m11+m33)/2;
  double m14 = (m11+m44)/2;
  double m15 = (m11+m55)/2;
  double m16 = (m11+m66)/2;
  double m34 = (m33+m44)/2;
  double m35 = (m33+m55)/2;
  double m36 = (m33+m66)/2;
  double m45 = (m44+m55)/2;
  double m46 = (m44+m66)/2;
  double m56 = (m55+m66)/2;

  double xb1 = (m44+m55+m66)/3.;
  double msqr = sqrt(xb1*xb1-(m44*m55+m44*m66+m55*m66)/3.);
  double mp2 = xb1-msqr; 
  double me2 = xb1+msqr;
  double mu2 = mu0*mu0;

  double RN13 = RN(m11,m33);
  double RN1p = RN(m11,mp2);
  double RN1e = RN(m11,me2);
  double RN3p = RN(m33,mp2);
  double RN3e = RN(m33,me2);
  //double RNep = RN(me2,mp2);

  double RSp456e = RS(mp2,m44,m55,m66,me2);
  double RSp465e = RS(mp2,m44,m66,m55,me2);
  double RSp564e = RS(mp2,m55,m66,m44,me2);
  double RSe456p = RS(me2,m44,m55,m66,mp2);
  double RSe465p = RS(me2,m44,m66,m55,mp2);
  double RSe564p = RS(me2,m55,m66,m44,mp2);
  double RSp451e = RS(mp2,m44,m55,m11,me2);
  double RSp461e = RS(mp2,m44,m66,m11,me2);
  double RSp561e = RS(mp2,m55,m66,m11,me2);
  double RSe451p = RS(me2,m44,m55,m11,mp2);
  double RSe461p = RS(me2,m44,m66,m11,mp2);
  double RSe561p = RS(me2,m55,m66,m11,mp2);
  double RSp453e = RS(mp2,m44,m55,m33,me2);
  double RSp463e = RS(mp2,m44,m66,m33,me2);
  double RSp563e = RS(mp2,m55,m66,m33,me2);
  double RSe453p = RS(me2,m44,m55,m33,mp2);
  double RSe463p = RS(me2,m44,m66,m33,mp2);
  double RSe563p = RS(me2,m55,m66,m33,mp2);
  double RS456pe = RS(m44,m55,m66,mp2,me2);
  double RS546pe = RS(m55,m44,m66,mp2,me2);
  double RS645pe = RS(m66,m44,m55,mp2,me2);
  double RS156pe = RS(m11,m55,m66,mp2,me2);
  double RS146pe = RS(m11,m44,m66,mp2,me2);
  double RS145pe = RS(m11,m44,m55,mp2,me2);
  double RS356pe = RS(m33,m55,m66,mp2,me2);
  double RS346pe = RS(m33,m44,m66,mp2,me2);
  double RS345pe = RS(m33,m44,m55,mp2,me2);

  double RD1456ep = RD(m11,m44,m55,m66,me2,mp2);
  double RD3456ep = RD(m33,m44,m55,m66,me2,mp2);
  //  double RC1456pe = RC(m11,m44,m55,m66,mp2,me2);  
  double RSe45611p = RS(me2,m44,m55,m66,m11,m11,mp2);
  double RSp45611e = RS(mp2,m44,m55,m66,m11,m11,me2);
  //double RS1456ppe = RS(m11,m44,m55,m66,mp2,mp2,me2);
  //double RS1456pee = RS(m11,m44,m55,m66,mp2,me2,me2);
  double RSe45633p = RS(me2,m44,m55,m66,m33,m33,mp2);
  double RSp45633e = RS(mp2,m44,m55,m66,m33,m33,me2);
  //double RS3456ppe = RS(m33,m44,m55,m66,mp2,mp2,me2);
  //double RS3456pee = RS(m33,m44,m55,m66,mp2,me2,me2);
  double RSe45613p = RS(me2,m44,m55,m66,m11,m33,mp2);
  double RSp45613e = RS(mp2,m44,m55,m66,m11,m33,me2);
  double RS14563pe = RS(m11,m44,m55,m66,m33,mp2,me2);
  double RS34561pe = RS(m33,m44,m55,m66,m11,mp2,me2);

  double RC1456pe = RC(m11,m44,m55,m66,mp2,me2);
  double RC3456pe = RC(m33,m44,m55,m66,mp2,me2);

  double F13p6x23RV =
       + Ab(m11,mu2)*AbV(m11,L) * ( 1./8. + 1./36.*pow(RS14563pe,2) - 1.
         /36.*RC1456pe*RS14563pe + 1./144.*pow(RC1456pe,2) - 1./18.*m11
         *RN1p*RC1456pe*RSp45613e + 1./18.*m11*RN1p*RSp45611e*RC1456pe
          - 1./18.*m11*RN1e*RC1456pe*RSe45613p + 1./18.*m11*RN1e*
         RSe45611p*RC1456pe - 1./18.*m11*RN13*RC1456pe*RS34561pe );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m13,L) * (  - 1./24. + 1./162.*
         RS14563pe*RS34561pe + 1./162.*pow(RS14563pe,2) + 1./162.*
         RSe45613p*RS14563pe + 1./162.*RSp45613e*RS14563pe - 1./324.*
         RC3456pe*RS14563pe - 1./324.*RSe45633p*RS14563pe - 1./324.*
         RSp45633e*RS14563pe - 1./324.*RC1456pe*RS34561pe - 1./162.*
         RC1456pe*RS14563pe - 1./324.*RC1456pe*RSe45613p - 1./324.*
         RC1456pe*RSp45613e + 1./648.*RC1456pe*RC3456pe + 1./648.*
         RC1456pe*RSe45633p + 1./648.*RC1456pe*RSp45633e + 1./648.*pow(
         RC1456pe,2) - 1./324.*RSe45611p*RS14563pe + 1./648.*RSe45611p*
         RC1456pe - 1./324.*RSp45611e*RS14563pe + 1./648.*RSp45611e*
         RC1456pe - 1./12.*m33*RN13 - 1./6.*m13*RN13 - 1./4.*m11*RN13 )
         ;

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m14,L) * (  - 1./36.*RS14563pe
          + 7./144.*RC1456pe - 5./72.*RS156pe + 1./36.*m44*RN1p*
         RSp45613e + 1./36.*m44*RN1p*RSp563e - 1./36.*m44*RN1p*
         RSp45611e - 1./36.*m44*RN1p*RSp561e + 1./36.*m44*RN1e*
         RSe45613p + 1./36.*m44*RN1e*RSe563p - 1./36.*m44*RN1e*
         RSe45611p - 1./36.*m44*RN1e*RSe561p + 1./36.*m44*RN13*
         RS34561pe + 1./36.*m44*RN13*RS356pe - 1./18.*m14*RN1p*
         RSp45613e + 1./18.*m14*RN1p*RSp563e + 1./18.*m14*RN1p*
         RSp45611e - 1./18.*m14*RN1p*RSp561e - 1./18.*m14*RN1e*
         RSe45613p + 1./18.*m14*RN1e*RSe563p + 1./18.*m14*RN1e*
         RSe45611p - 1./18.*m14*RN1e*RSe561p - 1./18.*m14*RN13*
         RS34561pe + 1./18.*m14*RN13*RS356pe + 1./36.*m11*RN1p*
         RSp45613e + 1./12.*m11*RN1p*RSp563e - 1./36.*m11*RN1p*
         RSp45611e - 1./12.*m11*RN1p*RSp561e + 1./36.*m11*RN1e*
         RSe45613p + 1./12.*m11*RN1e*RSe563p - 1./36.*m11*RN1e*
         RSe45611p );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m14,L) * (  - 1./12.*m11*RN1e*
         RSe561p + 1./36.*m11*RN13*RS34561pe + 1./12.*m11*RN13*RS356pe
          );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m15,L) * (  - 1./36.*RS14563pe
          + 7./144.*RC1456pe - 5./72.*RS146pe + 1./36.*m55*RN1p*
         RSp45613e + 1./36.*m55*RN1p*RSp463e - 1./36.*m55*RN1p*
         RSp45611e - 1./36.*m55*RN1p*RSp461e + 1./36.*m55*RN1e*
         RSe45613p + 1./36.*m55*RN1e*RSe463p - 1./36.*m55*RN1e*
         RSe45611p - 1./36.*m55*RN1e*RSe461p + 1./36.*m55*RN13*
         RS34561pe + 1./36.*m55*RN13*RS346pe - 1./18.*m15*RN1p*
         RSp45613e + 1./18.*m15*RN1p*RSp463e + 1./18.*m15*RN1p*
         RSp45611e - 1./18.*m15*RN1p*RSp461e - 1./18.*m15*RN1e*
         RSe45613p + 1./18.*m15*RN1e*RSe463p + 1./18.*m15*RN1e*
         RSe45611p - 1./18.*m15*RN1e*RSe461p - 1./18.*m15*RN13*
         RS34561pe + 1./18.*m15*RN13*RS346pe + 1./36.*m11*RN1p*
         RSp45613e + 1./12.*m11*RN1p*RSp463e - 1./36.*m11*RN1p*
         RSp45611e - 1./12.*m11*RN1p*RSp461e + 1./36.*m11*RN1e*
         RSe45613p + 1./12.*m11*RN1e*RSe463p - 1./36.*m11*RN1e*
         RSe45611p );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m15,L) * (  - 1./12.*m11*RN1e*
         RSe461p + 1./36.*m11*RN13*RS34561pe + 1./12.*m11*RN13*RS346pe
          );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m16,L) * (  - 1./36.*RS14563pe
          + 7./144.*RC1456pe - 5./72.*RS145pe + 1./36.*m66*RN1p*
         RSp45613e + 1./36.*m66*RN1p*RSp453e - 1./36.*m66*RN1p*
         RSp45611e - 1./36.*m66*RN1p*RSp451e + 1./36.*m66*RN1e*
         RSe45613p + 1./36.*m66*RN1e*RSe453p - 1./36.*m66*RN1e*
         RSe45611p - 1./36.*m66*RN1e*RSe451p + 1./36.*m66*RN13*
         RS34561pe + 1./36.*m66*RN13*RS345pe - 1./18.*m16*RN1p*
         RSp45613e + 1./18.*m16*RN1p*RSp453e + 1./18.*m16*RN1p*
         RSp45611e - 1./18.*m16*RN1p*RSp451e - 1./18.*m16*RN1e*
         RSe45613p + 1./18.*m16*RN1e*RSe453p + 1./18.*m16*RN1e*
         RSe45611p - 1./18.*m16*RN1e*RSe451p - 1./18.*m16*RN13*
         RS34561pe + 1./18.*m16*RN13*RS345pe + 1./36.*m11*RN1p*
         RSp45613e + 1./12.*m11*RN1p*RSp453e - 1./36.*m11*RN1p*
         RSp45611e - 1./12.*m11*RN1p*RSp451e + 1./36.*m11*RN1e*
         RSe45613p + 1./12.*m11*RN1e*RSe453p - 1./36.*m11*RN1e*
         RSe45611p );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m16,L) * (  - 1./12.*m11*RN1e*
         RSe451p + 1./36.*m11*RN13*RS34561pe + 1./12.*m11*RN13*RS345pe
          );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m33,L) * ( 1./36.*RS14563pe*
         RS34561pe - 1./72.*RC3456pe*RS14563pe - 1./72.*RC1456pe*
         RS34561pe + 1./144.*RC1456pe*RC3456pe - 1./18.*m33*RN13*
         RC3456pe*RS14563pe );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m34,L) * ( 1./36.*RS14563pe - 1./
         48.*RC1456pe + 1./72.*RS156pe + 1./36.*m44*RN13*RS14563pe + 1./
         36.*m44*RN13*RS156pe - 1./18.*m34*RN13*RS14563pe + 1./18.*m34*
         RN13*RS156pe + 1./12.*m33*RN13*RS14563pe + 1./36.*m33*RN13*
         RS156pe - 1./18.*m11*RN13*RS14563pe + 1./18.*m11*RN13*RS156pe
          );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m35,L) * ( 1./36.*RS14563pe - 1./
         48.*RC1456pe + 1./72.*RS146pe + 1./36.*m55*RN13*RS14563pe + 1./
         36.*m55*RN13*RS146pe - 1./18.*m35*RN13*RS14563pe + 1./18.*m35*
         RN13*RS146pe + 1./12.*m33*RN13*RS14563pe + 1./36.*m33*RN13*
         RS146pe - 1./18.*m11*RN13*RS14563pe + 1./18.*m11*RN13*RS146pe
          );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m36,L) * ( 1./36.*RS14563pe - 1./
         48.*RC1456pe + 1./72.*RS145pe + 1./36.*m66*RN13*RS14563pe + 1./
         36.*m66*RN13*RS145pe - 1./18.*m36*RN13*RS14563pe + 1./18.*m36*
         RN13*RS145pe + 1./12.*m33*RN13*RS14563pe + 1./36.*m33*RN13*
         RS145pe - 1./18.*m11*RN13*RS14563pe + 1./18.*m11*RN13*RS145pe
          );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m44,L) * (  - 1./18.*m44*RN1p*
         RS156pe*RSp563e + 1./54.*m44*RN1p*RS456pe*RS156pe*RSp563e + 1./
         18.*m44*RN1p*RSp561e*RS156pe - 1./54.*m44*RN1p*RSp561e*RS456pe
         *RS156pe - 1./18.*m44*RN1e*RS156pe*RSe563p + 1./54.*m44*RN1e*
         RS456pe*RS156pe*RSe563p + 1./18.*m44*RN1e*RSe561p*RS156pe - 1./
         54.*m44*RN1e*RSe561p*RS456pe*RS156pe - 1./18.*m44*RN13*RS156pe
         *RS356pe + 1./54.*m44*RN13*RS456pe*RS156pe*RS356pe );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * (  - 1./108.*pow(
         RS146pe,2) + 1./54.*RS156pe*RS146pe - 1./108.*pow(RS156pe,2)
          - 1./108.*m55*RN1p*RS146pe*RSp563e - 1./36.*m55*RN1p*RS146pe*
         RSp463e - 1./108.*m55*RN1p*RS156pe*RSp563e - 1./108.*m55*RN1p*
         RS156pe*RSp463e + 1./108.*m55*RN1p*RSp561e*RS146pe + 1./108.*
         m55*RN1p*RSp561e*RS156pe + 1./36.*m55*RN1p*RSp461e*RS146pe + 1.
         /108.*m55*RN1p*RSp461e*RS156pe - 1./108.*m55*RN1e*RS146pe*
         RSe563p - 1./36.*m55*RN1e*RS146pe*RSe463p - 1./108.*m55*RN1e*
         RS156pe*RSe563p - 1./108.*m55*RN1e*RS156pe*RSe463p + 1./108.*
         m55*RN1e*RSe561p*RS146pe + 1./108.*m55*RN1e*RSe561p*RS156pe + 
         1./36.*m55*RN1e*RSe461p*RS146pe + 1./108.*m55*RN1e*RSe461p*
         RS156pe - 1./36.*m55*RN13*RS146pe*RS346pe - 1./108.*m55*RN13*
         RS146pe*RS356pe - 1./108.*m55*RN13*RS156pe*RS346pe - 1./108.*
         m55*RN13*RS156pe*RS356pe - 1./54.*m45*RN1p*RS146pe*RSp563e + 1.
         /54.*m45*RN1p*RS146pe*RSp463e + 1./54.*m45*RN1p*RS156pe*
         RSp563e );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * (  - 1./54.*m45*RN1p*
         RS156pe*RSp463e + 1./54.*m45*RN1p*RSp561e*RS146pe - 1./54.*m45
         *RN1p*RSp561e*RS156pe - 1./54.*m45*RN1p*RSp461e*RS146pe + 1./
         54.*m45*RN1p*RSp461e*RS156pe - 1./54.*m45*RN1e*RS146pe*RSe563p
          + 1./54.*m45*RN1e*RS146pe*RSe463p + 1./54.*m45*RN1e*RS156pe*
         RSe563p - 1./54.*m45*RN1e*RS156pe*RSe463p + 1./54.*m45*RN1e*
         RSe561p*RS146pe - 1./54.*m45*RN1e*RSe561p*RS156pe - 1./54.*m45
         *RN1e*RSe461p*RS146pe + 1./54.*m45*RN1e*RSe461p*RS156pe + 1./
         54.*m45*RN13*RS146pe*RS346pe - 1./54.*m45*RN13*RS146pe*RS356pe
          - 1./54.*m45*RN13*RS156pe*RS346pe + 1./54.*m45*RN13*RS156pe*
         RS356pe - 1./108.*m44*RN1p*RS146pe*RSp563e - 1./108.*m44*RN1p*
         RS146pe*RSp463e - 1./36.*m44*RN1p*RS156pe*RSp563e - 1./108.*
         m44*RN1p*RS156pe*RSp463e + 1./108.*m44*RN1p*RSp561e*RS146pe + 
         1./36.*m44*RN1p*RSp561e*RS156pe + 1./108.*m44*RN1p*RSp461e*
         RS146pe + 1./108.*m44*RN1p*RSp461e*RS156pe - 1./108.*m44*RN1e*
         RS146pe*RSe563p );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * (  - 1./108.*m44*RN1e*
         RS146pe*RSe463p - 1./36.*m44*RN1e*RS156pe*RSe563p - 1./108.*
         m44*RN1e*RS156pe*RSe463p + 1./108.*m44*RN1e*RSe561p*RS146pe + 
         1./36.*m44*RN1e*RSe561p*RS156pe + 1./108.*m44*RN1e*RSe461p*
         RS146pe + 1./108.*m44*RN1e*RSe461p*RS156pe - 1./108.*m44*RN13*
         RS146pe*RS346pe - 1./108.*m44*RN13*RS146pe*RS356pe - 1./108.*
         m44*RN13*RS156pe*RS346pe - 1./36.*m44*RN13*RS156pe*RS356pe - 1.
         /54.*m11*RN1p*RS146pe*RSp563e + 1./54.*m11*RN1p*RS146pe*
         RSp463e + 1./54.*m11*RN1p*RS156pe*RSp563e - 1./54.*m11*RN1p*
         RS156pe*RSp463e + 1./54.*m11*RN1p*RSp561e*RS146pe - 1./54.*m11
         *RN1p*RSp561e*RS156pe - 1./54.*m11*RN1p*RSp461e*RS146pe + 1./
         54.*m11*RN1p*RSp461e*RS156pe - 1./54.*m11*RN1e*RS146pe*RSe563p
          + 1./54.*m11*RN1e*RS146pe*RSe463p + 1./54.*m11*RN1e*RS156pe*
         RSe563p - 1./54.*m11*RN1e*RS156pe*RSe463p + 1./54.*m11*RN1e*
         RSe561p*RS146pe - 1./54.*m11*RN1e*RSe561p*RS156pe - 1./54.*m11
         *RN1e*RSe461p*RS146pe );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m45,L) * ( 1./54.*m11*RN1e*
         RSe461p*RS156pe + 1./54.*m11*RN13*RS146pe*RS346pe - 1./54.*m11
         *RN13*RS146pe*RS356pe - 1./54.*m11*RN13*RS156pe*RS346pe + 1./
         54.*m11*RN13*RS156pe*RS356pe );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * (  - 1./108.*pow(
         RS145pe,2) + 1./54.*RS156pe*RS145pe - 1./108.*pow(RS156pe,2)
          - 1./108.*m66*RN1p*RS145pe*RSp563e - 1./36.*m66*RN1p*RS145pe*
         RSp453e - 1./108.*m66*RN1p*RS156pe*RSp563e - 1./108.*m66*RN1p*
         RS156pe*RSp453e + 1./108.*m66*RN1p*RSp561e*RS145pe + 1./108.*
         m66*RN1p*RSp561e*RS156pe + 1./36.*m66*RN1p*RSp451e*RS145pe + 1.
         /108.*m66*RN1p*RSp451e*RS156pe - 1./108.*m66*RN1e*RS145pe*
         RSe563p - 1./36.*m66*RN1e*RS145pe*RSe453p - 1./108.*m66*RN1e*
         RS156pe*RSe563p - 1./108.*m66*RN1e*RS156pe*RSe453p + 1./108.*
         m66*RN1e*RSe561p*RS145pe + 1./108.*m66*RN1e*RSe561p*RS156pe + 
         1./36.*m66*RN1e*RSe451p*RS145pe + 1./108.*m66*RN1e*RSe451p*
         RS156pe - 1./36.*m66*RN13*RS145pe*RS345pe - 1./108.*m66*RN13*
         RS145pe*RS356pe - 1./108.*m66*RN13*RS156pe*RS345pe - 1./108.*
         m66*RN13*RS156pe*RS356pe - 1./54.*m46*RN1p*RS145pe*RSp563e + 1.
         /54.*m46*RN1p*RS145pe*RSp453e + 1./54.*m46*RN1p*RS156pe*
         RSp563e );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * (  - 1./54.*m46*RN1p*
         RS156pe*RSp453e + 1./54.*m46*RN1p*RSp561e*RS145pe - 1./54.*m46
         *RN1p*RSp561e*RS156pe - 1./54.*m46*RN1p*RSp451e*RS145pe + 1./
         54.*m46*RN1p*RSp451e*RS156pe - 1./54.*m46*RN1e*RS145pe*RSe563p
          + 1./54.*m46*RN1e*RS145pe*RSe453p + 1./54.*m46*RN1e*RS156pe*
         RSe563p - 1./54.*m46*RN1e*RS156pe*RSe453p + 1./54.*m46*RN1e*
         RSe561p*RS145pe - 1./54.*m46*RN1e*RSe561p*RS156pe - 1./54.*m46
         *RN1e*RSe451p*RS145pe + 1./54.*m46*RN1e*RSe451p*RS156pe + 1./
         54.*m46*RN13*RS145pe*RS345pe - 1./54.*m46*RN13*RS145pe*RS356pe
          - 1./54.*m46*RN13*RS156pe*RS345pe + 1./54.*m46*RN13*RS156pe*
         RS356pe - 1./108.*m44*RN1p*RS145pe*RSp563e - 1./108.*m44*RN1p*
         RS145pe*RSp453e - 1./36.*m44*RN1p*RS156pe*RSp563e - 1./108.*
         m44*RN1p*RS156pe*RSp453e + 1./108.*m44*RN1p*RSp561e*RS145pe + 
         1./36.*m44*RN1p*RSp561e*RS156pe + 1./108.*m44*RN1p*RSp451e*
         RS145pe + 1./108.*m44*RN1p*RSp451e*RS156pe - 1./108.*m44*RN1e*
         RS145pe*RSe563p );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * (  - 1./108.*m44*RN1e*
         RS145pe*RSe453p - 1./36.*m44*RN1e*RS156pe*RSe563p - 1./108.*
         m44*RN1e*RS156pe*RSe453p + 1./108.*m44*RN1e*RSe561p*RS145pe + 
         1./36.*m44*RN1e*RSe561p*RS156pe + 1./108.*m44*RN1e*RSe451p*
         RS145pe + 1./108.*m44*RN1e*RSe451p*RS156pe - 1./108.*m44*RN13*
         RS145pe*RS345pe - 1./108.*m44*RN13*RS145pe*RS356pe - 1./108.*
         m44*RN13*RS156pe*RS345pe - 1./36.*m44*RN13*RS156pe*RS356pe - 1.
         /54.*m11*RN1p*RS145pe*RSp563e + 1./54.*m11*RN1p*RS145pe*
         RSp453e + 1./54.*m11*RN1p*RS156pe*RSp563e - 1./54.*m11*RN1p*
         RS156pe*RSp453e + 1./54.*m11*RN1p*RSp561e*RS145pe - 1./54.*m11
         *RN1p*RSp561e*RS156pe - 1./54.*m11*RN1p*RSp451e*RS145pe + 1./
         54.*m11*RN1p*RSp451e*RS156pe - 1./54.*m11*RN1e*RS145pe*RSe563p
          + 1./54.*m11*RN1e*RS145pe*RSe453p + 1./54.*m11*RN1e*RS156pe*
         RSe563p - 1./54.*m11*RN1e*RS156pe*RSe453p + 1./54.*m11*RN1e*
         RSe561p*RS145pe - 1./54.*m11*RN1e*RSe561p*RS156pe - 1./54.*m11
         *RN1e*RSe451p*RS145pe );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m46,L) * ( 1./54.*m11*RN1e*
         RSe451p*RS156pe + 1./54.*m11*RN13*RS145pe*RS345pe - 1./54.*m11
         *RN13*RS145pe*RS356pe - 1./54.*m11*RN13*RS156pe*RS345pe + 1./
         54.*m11*RN13*RS156pe*RS356pe );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m55,L) * (  - 1./18.*m55*RN1p*
         RS146pe*RSp463e + 1./54.*m55*RN1p*RS546pe*RS146pe*RSp463e + 1./
         18.*m55*RN1p*RSp461e*RS146pe - 1./54.*m55*RN1p*RSp461e*RS546pe
         *RS146pe - 1./18.*m55*RN1e*RS146pe*RSe463p + 1./54.*m55*RN1e*
         RS546pe*RS146pe*RSe463p + 1./18.*m55*RN1e*RSe461p*RS146pe - 1./
         54.*m55*RN1e*RSe461p*RS546pe*RS146pe - 1./18.*m55*RN13*RS146pe
         *RS346pe + 1./54.*m55*RN13*RS546pe*RS146pe*RS346pe );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * (  - 1./108.*pow(
         RS145pe,2) + 1./54.*RS146pe*RS145pe - 1./108.*pow(RS146pe,2)
          - 1./108.*m66*RN1p*RS145pe*RSp463e - 1./36.*m66*RN1p*RS145pe*
         RSp453e - 1./108.*m66*RN1p*RS146pe*RSp463e - 1./108.*m66*RN1p*
         RS146pe*RSp453e + 1./108.*m66*RN1p*RSp461e*RS145pe + 1./108.*
         m66*RN1p*RSp461e*RS146pe + 1./36.*m66*RN1p*RSp451e*RS145pe + 1.
         /108.*m66*RN1p*RSp451e*RS146pe - 1./108.*m66*RN1e*RS145pe*
         RSe463p - 1./36.*m66*RN1e*RS145pe*RSe453p - 1./108.*m66*RN1e*
         RS146pe*RSe463p - 1./108.*m66*RN1e*RS146pe*RSe453p + 1./108.*
         m66*RN1e*RSe461p*RS145pe + 1./108.*m66*RN1e*RSe461p*RS146pe + 
         1./36.*m66*RN1e*RSe451p*RS145pe + 1./108.*m66*RN1e*RSe451p*
         RS146pe - 1./36.*m66*RN13*RS145pe*RS345pe - 1./108.*m66*RN13*
         RS145pe*RS346pe - 1./108.*m66*RN13*RS146pe*RS345pe - 1./108.*
         m66*RN13*RS146pe*RS346pe - 1./54.*m56*RN1p*RS145pe*RSp463e + 1.
         /54.*m56*RN1p*RS145pe*RSp453e + 1./54.*m56*RN1p*RS146pe*
         RSp463e );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * (  - 1./54.*m56*RN1p*
         RS146pe*RSp453e + 1./54.*m56*RN1p*RSp461e*RS145pe - 1./54.*m56
         *RN1p*RSp461e*RS146pe - 1./54.*m56*RN1p*RSp451e*RS145pe + 1./
         54.*m56*RN1p*RSp451e*RS146pe - 1./54.*m56*RN1e*RS145pe*RSe463p
          + 1./54.*m56*RN1e*RS145pe*RSe453p + 1./54.*m56*RN1e*RS146pe*
         RSe463p - 1./54.*m56*RN1e*RS146pe*RSe453p + 1./54.*m56*RN1e*
         RSe461p*RS145pe - 1./54.*m56*RN1e*RSe461p*RS146pe - 1./54.*m56
         *RN1e*RSe451p*RS145pe + 1./54.*m56*RN1e*RSe451p*RS146pe + 1./
         54.*m56*RN13*RS145pe*RS345pe - 1./54.*m56*RN13*RS145pe*RS346pe
          - 1./54.*m56*RN13*RS146pe*RS345pe + 1./54.*m56*RN13*RS146pe*
         RS346pe - 1./108.*m55*RN1p*RS145pe*RSp463e - 1./108.*m55*RN1p*
         RS145pe*RSp453e - 1./36.*m55*RN1p*RS146pe*RSp463e - 1./108.*
         m55*RN1p*RS146pe*RSp453e + 1./108.*m55*RN1p*RSp461e*RS145pe + 
         1./36.*m55*RN1p*RSp461e*RS146pe + 1./108.*m55*RN1p*RSp451e*
         RS145pe + 1./108.*m55*RN1p*RSp451e*RS146pe - 1./108.*m55*RN1e*
         RS145pe*RSe463p );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * (  - 1./108.*m55*RN1e*
         RS145pe*RSe453p - 1./36.*m55*RN1e*RS146pe*RSe463p - 1./108.*
         m55*RN1e*RS146pe*RSe453p + 1./108.*m55*RN1e*RSe461p*RS145pe + 
         1./36.*m55*RN1e*RSe461p*RS146pe + 1./108.*m55*RN1e*RSe451p*
         RS145pe + 1./108.*m55*RN1e*RSe451p*RS146pe - 1./108.*m55*RN13*
         RS145pe*RS345pe - 1./108.*m55*RN13*RS145pe*RS346pe - 1./108.*
         m55*RN13*RS146pe*RS345pe - 1./36.*m55*RN13*RS146pe*RS346pe - 1.
         /54.*m11*RN1p*RS145pe*RSp463e + 1./54.*m11*RN1p*RS145pe*
         RSp453e + 1./54.*m11*RN1p*RS146pe*RSp463e - 1./54.*m11*RN1p*
         RS146pe*RSp453e + 1./54.*m11*RN1p*RSp461e*RS145pe - 1./54.*m11
         *RN1p*RSp461e*RS146pe - 1./54.*m11*RN1p*RSp451e*RS145pe + 1./
         54.*m11*RN1p*RSp451e*RS146pe - 1./54.*m11*RN1e*RS145pe*RSe463p
          + 1./54.*m11*RN1e*RS145pe*RSe453p + 1./54.*m11*RN1e*RS146pe*
         RSe463p - 1./54.*m11*RN1e*RS146pe*RSe453p + 1./54.*m11*RN1e*
         RSe461p*RS145pe - 1./54.*m11*RN1e*RSe461p*RS146pe - 1./54.*m11
         *RN1e*RSe451p*RS145pe );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m56,L) * ( 1./54.*m11*RN1e*
         RSe451p*RS146pe + 1./54.*m11*RN13*RS145pe*RS345pe - 1./54.*m11
         *RN13*RS145pe*RS346pe - 1./54.*m11*RN13*RS146pe*RS345pe + 1./
         54.*m11*RN13*RS146pe*RS346pe );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(m66,L) * (  - 1./18.*m66*RN1p*
         RS145pe*RSp453e + 1./54.*m66*RN1p*RS645pe*RS145pe*RSp453e + 1./
         18.*m66*RN1p*RSp451e*RS145pe - 1./54.*m66*RN1p*RSp451e*RS645pe
         *RS145pe - 1./18.*m66*RN1e*RS145pe*RSe453p + 1./54.*m66*RN1e*
         RS645pe*RS145pe*RSe453p + 1./18.*m66*RN1e*RSe451p*RS145pe - 1./
         54.*m66*RN1e*RSe451p*RS645pe*RS145pe - 1./18.*m66*RN13*RS145pe
         *RS345pe + 1./54.*m66*RN13*RS645pe*RS145pe*RS345pe );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(mp2,L) * ( 1./36.*RSp45613e*
         RS14563pe - 1./72.*RSp45633e*RS14563pe - 1./72.*RC1456pe*
         RSp45613e + 1./144.*RC1456pe*RSp45633e - 1./72.*RSp45611e*
         RS14563pe + 1./144.*RSp45611e*RC1456pe + 1./54.*m66*RN1p*
         RSp456e*RS145pe*RSp453e - 1./54.*m66*RN1p*RSp456e*RSp451e*
         RS145pe + 1./54.*m66*RN1e*RSp456e*RS145pe*RSe453p - 1./54.*m66
         *RN1e*RSp456e*RSe451p*RS145pe + 1./54.*m66*RN13*RSp456e*
         RS145pe*RS345pe + 1./54.*m55*RN1p*RSp465e*RS146pe*RSp463e - 1./
         54.*m55*RN1p*RSp465e*RSp461e*RS146pe + 1./54.*m55*RN1e*RSp465e
         *RS146pe*RSe463p - 1./54.*m55*RN1e*RSp465e*RSe461p*RS146pe + 1.
         /54.*m55*RN13*RSp465e*RS146pe*RS346pe + 1./54.*m44*RN1p*
         RSp564e*RS156pe*RSp563e - 1./54.*m44*RN1p*RSp564e*RSp561e*
         RS156pe + 1./54.*m44*RN1e*RSp564e*RS156pe*RSe563p - 1./54.*m44
         *RN1e*RSp564e*RSe561p*RS156pe + 1./54.*m44*RN13*RSp564e*
         RS156pe*RS356pe - 1./18.*m33*RN13*RSp45633e*RS14563pe - 1./18.
         *m11*RN1p*RSp45611e*RSp45613e );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(mp2,L) * ( 1./18.*m11*RN1p*pow(
         RSp45611e,2) - 1./18.*m11*RN1e*RSp45611e*RSe45613p + 1./18.*
         m11*RN1e*RSp45611e*RSe45611p - 1./18.*m11*RN13*RSp45611e*
         RS34561pe );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(me2,L) * ( 1./36.*RSe45613p*
         RS14563pe - 1./72.*RSe45633p*RS14563pe - 1./72.*RC1456pe*
         RSe45613p + 1./144.*RC1456pe*RSe45633p - 1./72.*RSe45611p*
         RS14563pe + 1./144.*RSe45611p*RC1456pe + 1./54.*m66*RN1p*
         RSe456p*RS145pe*RSp453e - 1./54.*m66*RN1p*RSe456p*RSp451e*
         RS145pe + 1./54.*m66*RN1e*RSe456p*RS145pe*RSe453p - 1./54.*m66
         *RN1e*RSe456p*RSe451p*RS145pe + 1./54.*m66*RN13*RSe456p*
         RS145pe*RS345pe + 1./54.*m55*RN1p*RSe465p*RS146pe*RSp463e - 1./
         54.*m55*RN1p*RSe465p*RSp461e*RS146pe + 1./54.*m55*RN1e*RSe465p
         *RS146pe*RSe463p - 1./54.*m55*RN1e*RSe465p*RSe461p*RS146pe + 1.
         /54.*m55*RN13*RSe465p*RS146pe*RS346pe + 1./54.*m44*RN1p*
         RSe564p*RS156pe*RSp563e - 1./54.*m44*RN1p*RSe564p*RSp561e*
         RS156pe + 1./54.*m44*RN1e*RSe564p*RS156pe*RSe563p - 1./54.*m44
         *RN1e*RSe564p*RSe561p*RS156pe + 1./54.*m44*RN13*RSe564p*
         RS156pe*RS356pe - 1./18.*m33*RN13*RSe45633p*RS14563pe - 1./18.
         *m11*RN1p*RSe45611p*RSp45613e );

      F13p6x23RV +=  + Ab(m11,mu2)*AbV(me2,L) * ( 1./18.*m11*RN1p*
         RSp45611e*RSe45611p - 1./18.*m11*RN1e*RSe45611p*RSe45613p + 1./
         18.*m11*RN1e*pow(RSe45611p,2) - 1./18.*m11*RN13*RSe45611p*
         RS34561pe );

      F13p6x23RV +=  + Ab(m11,mu2)*BbV(m11,m33,L) * (  - 1./18.*m11*
         RC1456pe*RS34561pe );

      F13p6x23RV +=  + Ab(m11,mu2)*BbV(m11,mp2,L) * (  - 1./18.*m11*
         RC1456pe*RSp45613e + 1./18.*m11*RSp45611e*RC1456pe );

      F13p6x23RV +=  + Ab(m11,mu2)*BbV(m11,me2,L) * (  - 1./18.*m11*
         RC1456pe*RSe45613p + 1./18.*m11*RSe45611p*RC1456pe );

      F13p6x23RV +=  + Ab(m11,mu2)*BbV(m11,L) * (  - 1./72.*RD1456ep*
         RS14563pe + 1./144.*RD1456ep*RC1456pe + 1./4.*m11 - 1./18.*m11
         *RC1456pe*RS14563pe + 1./18.*m11*pow(RC1456pe,2) - 1./18.*m11*
         RN1p*RD1456ep*RSp45613e + 1./18.*m11*RN1p*RSp45611e*RD1456ep
          - 1./18.*m11*RN1e*RD1456ep*RSe45613p + 1./18.*m11*RN1e*
         RSe45611p*RD1456ep - 1./18.*m11*RN13*RD1456ep*RS34561pe );

      F13p6x23RV +=  + Ab(m11,mu2)*BbV(m14,L) * ( 1./72.*RD1456ep - 1./
         144.*m44*RC1456pe - 1./72.*m44*RS156pe + 1./72.*m14*RC1456pe
          - 1./36.*m14*RS156pe - 1./144.*m11*RC1456pe - 1./24.*m11*
         RS156pe );

      F13p6x23RV +=  + Ab(m11,mu2)*BbV(m15,L) * ( 1./72.*RD1456ep - 1./
         144.*m55*RC1456pe - 1./72.*m55*RS146pe + 1./72.*m15*RC1456pe
          - 1./36.*m15*RS146pe - 1./144.*m11*RC1456pe - 1./24.*m11*
         RS146pe );

      F13p6x23RV +=  + Ab(m11,mu2)*BbV(m16,L) * ( 1./72.*RD1456ep - 1./
         144.*m66*RC1456pe - 1./72.*m66*RS145pe + 1./72.*m16*RC1456pe
          - 1./36.*m16*RS145pe - 1./144.*m11*RC1456pe - 1./24.*m11*
         RS145pe );

      F13p6x23RV +=  + Ab(m11,mu2)*BbV(m33,L) * (  - 1./72.*RD3456ep*
         RS14563pe + 1./144.*RC1456pe*RD3456ep - 1./18.*m33*RN13*
         RD3456ep*RS14563pe );

      F13p6x23RV +=  + Ab(m11,mu2)*CbV(m11,L) * ( 1./18.*m11*RD1456ep*
         RC1456pe );

      F13p6x23RV +=  + Ab(m13,mu2)*AbV(m11,L) * (  - 1./24. + 1./162.*
         RS14563pe*RS34561pe + 1./162.*pow(RS14563pe,2) + 1./162.*
         RSe45613p*RS14563pe + 1./162.*RSp45613e*RS14563pe - 1./324.*
         RC3456pe*RS14563pe - 1./324.*RSe45633p*RS14563pe - 1./324.*
         RSp45633e*RS14563pe - 1./324.*RC1456pe*RS34561pe - 1./162.*
         RC1456pe*RS14563pe - 1./324.*RC1456pe*RSe45613p - 1./324.*
         RC1456pe*RSp45613e + 1./648.*RC1456pe*RC3456pe + 1./648.*
         RC1456pe*RSe45633p + 1./648.*RC1456pe*RSp45633e + 1./648.*pow(
         RC1456pe,2) - 1./324.*RSe45611p*RS14563pe + 1./648.*RSe45611p*
         RC1456pe - 1./324.*RSp45611e*RS14563pe + 1./648.*RSp45611e*
         RC1456pe );

      F13p6x23RV +=  + Ab(m13,mu2)*AbV(m13,L) * ( 1./4. );

      F13p6x23RV +=  + Ab(m13,mu2)*AbV(m33,L) * (  - 5./24. + 1./162.*
         pow(RS34561pe,2) + 1./162.*RS14563pe*RS34561pe + 1./162.*
         RSe45613p*RS34561pe + 1./162.*RSp45613e*RS34561pe - 1./162.*
         RC3456pe*RS34561pe - 1./324.*RC3456pe*RS14563pe - 1./324.*
         RC3456pe*RSe45613p - 1./324.*RC3456pe*RSp45613e + 1./648.*pow(
         RC3456pe,2) - 1./324.*RSe45633p*RS34561pe + 1./648.*RSe45633p*
         RC3456pe - 1./324.*RSp45633e*RS34561pe + 1./648.*RSp45633e*
         RC3456pe - 1./324.*RC1456pe*RS34561pe + 1./648.*RC1456pe*
         RC3456pe - 1./324.*RSe45611p*RS34561pe + 1./648.*RSe45611p*
         RC3456pe - 1./324.*RSp45611e*RS34561pe + 1./648.*RSp45611e*
         RC3456pe );

      F13p6x23RV +=  + Ab(m13,mu2)*AbV(mp2,L) * ( 1./162.*RSp45613e*
         RS34561pe + 1./162.*RSp45613e*RS14563pe + 1./162.*RSp45613e*
         RSe45613p + 1./162.*pow(RSp45613e,2) - 1./324.*RC3456pe*
         RSp45613e - 1./324.*RSe45633p*RSp45613e - 1./324.*RSp45633e*
         RS34561pe - 1./324.*RSp45633e*RS14563pe - 1./324.*RSp45633e*
         RSe45613p - 1./162.*RSp45633e*RSp45613e + 1./648.*RSp45633e*
         RC3456pe + 1./648.*RSp45633e*RSe45633p + 1./648.*pow(
         RSp45633e,2) - 1./324.*RC1456pe*RSp45613e + 1./648.*RC1456pe*
         RSp45633e - 1./324.*RSe45611p*RSp45613e + 1./648.*RSe45611p*
         RSp45633e - 1./324.*RSp45611e*RS34561pe - 1./324.*RSp45611e*
         RS14563pe - 1./324.*RSp45611e*RSe45613p - 1./162.*RSp45611e*
         RSp45613e + 1./648.*RSp45611e*RC3456pe + 1./648.*RSp45611e*
         RSe45633p + 1./324.*RSp45611e*RSp45633e + 1./648.*RSp45611e*
         RC1456pe + 1./648.*RSp45611e*RSe45611p + 1./648.*pow(
         RSp45611e,2) );

      F13p6x23RV +=  + Ab(m13,mu2)*AbV(me2,L) * ( 1./162.*RSe45613p*
         RS34561pe + 1./162.*RSe45613p*RS14563pe + 1./162.*pow(
         RSe45613p,2) + 1./162.*RSp45613e*RSe45613p - 1./324.*RC3456pe*
         RSe45613p - 1./324.*RSe45633p*RS34561pe - 1./324.*RSe45633p*
         RS14563pe - 1./162.*RSe45633p*RSe45613p - 1./324.*RSe45633p*
         RSp45613e + 1./648.*RSe45633p*RC3456pe + 1./648.*pow(
         RSe45633p,2) - 1./324.*RSp45633e*RSe45613p + 1./648.*RSp45633e
         *RSe45633p - 1./324.*RC1456pe*RSe45613p + 1./648.*RC1456pe*
         RSe45633p - 1./324.*RSe45611p*RS34561pe - 1./324.*RSe45611p*
         RS14563pe - 1./162.*RSe45611p*RSe45613p - 1./324.*RSe45611p*
         RSp45613e + 1./648.*RSe45611p*RC3456pe + 1./324.*RSe45611p*
         RSe45633p + 1./648.*RSe45611p*RSp45633e + 1./648.*RSe45611p*
         RC1456pe + 1./648.*pow(RSe45611p,2) - 1./324.*RSp45611e*
         RSe45613p + 1./648.*RSp45611e*RSe45633p + 1./648.*RSp45611e*
         RSe45611p );

      F13p6x23RV +=  + Ab(m13,mu2)*BbV(m11,m33,L) * (  - 1./12.*m33 - 1.
         /6.*m13 - 1./4.*m11 );

      F13p6x23RV +=  + Ab(m13,mu2)*BbV(m11,L) * (  - 1./324.*RD1456ep*
         RS34561pe - 1./324.*RD1456ep*RS14563pe - 1./324.*RD1456ep*
         RSe45613p - 1./324.*RD1456ep*RSp45613e + 1./648.*RD1456ep*
         RC3456pe + 1./648.*RD1456ep*RSe45633p + 1./648.*RD1456ep*
         RSp45633e + 1./648.*RD1456ep*RC1456pe + 1./648.*RSe45611p*
         RD1456ep + 1./648.*RSp45611e*RD1456ep );

      F13p6x23RV +=  + Ab(m13,mu2)*BbV(m33,L) * (  - 1./324.*RD3456ep*
         RS34561pe - 1./324.*RD3456ep*RS14563pe - 1./324.*RD3456ep*
         RSe45613p - 1./324.*RD3456ep*RSp45613e + 1./648.*RD3456ep*
         RC3456pe + 1./648.*RSe45633p*RD3456ep + 1./648.*RSp45633e*
         RD3456ep + 1./648.*RC1456pe*RD3456ep + 1./648.*RSe45611p*
         RD3456ep + 1./648.*RSp45611e*RD3456ep );

      F13p6x23RV +=  + Ab(m14,mu2)*AbV(m11,L) * (  - 1./36.*RS14563pe
          + 7./144.*RC1456pe - 5./72.*RS156pe );

      F13p6x23RV +=  + Ab(m14,mu2)*AbV(m14,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m14,mu2)*AbV(m15,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m14,mu2)*AbV(m16,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m14,mu2)*AbV(m33,L) * (  - 1./36.*RS34561pe
          - 1./48.*RC3456pe + 5./72.*RS356pe );

      F13p6x23RV +=  + Ab(m14,mu2)*AbV(m34,L) * ( 1./16. + 7./216.*
         RS34561pe + 7./216.*RS14563pe + 7./216.*RSe45613p + 7./216.*
         RSp45613e - 1./216.*RC3456pe - 1./216.*RSe45633p - 1./216.*
         RSp45633e - 5./216.*RS356pe - 5./216.*RSe563p - 5./216.*
         RSp563e - 1./216.*RC1456pe - 1./216.*RSe45611p - 1./216.*
         RSp45611e - 5./216.*RS156pe + 5./216.*RS456pe - 5./216.*
         RSe561p - 5./216.*RSp561e + 5./216.*RSe564p + 5./216.*RSp564e
          );

      F13p6x23RV +=  + Ab(m14,mu2)*AbV(m35,L) * ( 1./16. );

      F13p6x23RV +=  + Ab(m14,mu2)*AbV(m36,L) * ( 1./16. );

      F13p6x23RV +=  + Ab(m14,mu2)*AbV(mp2,L) * (  - 1./36.*RSp45613e
          - 1./48.*RSp45633e + 5./72.*RSp563e + 7./144.*RSp45611e - 5./
         72.*RSp561e );

      F13p6x23RV +=  + Ab(m14,mu2)*AbV(me2,L) * (  - 1./36.*RSe45613p
          - 1./48.*RSe45633p + 5./72.*RSe563p + 7./144.*RSe45611p - 5./
         72.*RSe561p );

      F13p6x23RV +=  + Ab(m14,mu2)*BbV(m11,m33,L) * ( 1./36.*m44*
         RS34561pe + 1./36.*m44*RS356pe - 1./18.*m14*RS34561pe + 1./18.
         *m14*RS356pe + 1./36.*m11*RS34561pe + 1./12.*m11*RS356pe );

      F13p6x23RV +=  + Ab(m14,mu2)*BbV(m11,mp2,L) * ( 1./36.*m44*
         RSp45613e + 1./36.*m44*RSp563e - 1./36.*m44*RSp45611e - 1./36.
         *m44*RSp561e - 1./18.*m14*RSp45613e + 1./18.*m14*RSp563e + 1./
         18.*m14*RSp45611e - 1./18.*m14*RSp561e + 1./36.*m11*RSp45613e
          + 1./12.*m11*RSp563e - 1./36.*m11*RSp45611e - 1./12.*m11*
         RSp561e );

      F13p6x23RV +=  + Ab(m14,mu2)*BbV(m11,me2,L) * ( 1./36.*m44*
         RSe45613p + 1./36.*m44*RSe563p - 1./36.*m44*RSe45611p - 1./36.
         *m44*RSe561p - 1./18.*m14*RSe45613p + 1./18.*m14*RSe563p + 1./
         18.*m14*RSe45611p - 1./18.*m14*RSe561p + 1./36.*m11*RSe45613p
          + 1./12.*m11*RSe563p - 1./36.*m11*RSe45611p - 1./12.*m11*
         RSe561p );

      F13p6x23RV +=  + Ab(m14,mu2)*BbV(m11,L) * ( 7./144.*RD1456ep + 1./
         36.*m44*RS14563pe - 1./36.*m44*RC1456pe - 1./36.*m44*RS156pe
          - 1./18.*m14*RS14563pe + 1./18.*m14*RC1456pe - 1./18.*m14*
         RS156pe + 1./36.*m11*RS14563pe - 1./36.*m11*RC1456pe - 1./12.*
         m11*RS156pe );

      F13p6x23RV +=  + Ab(m14,mu2)*BbV(m14,L) * ( 1./24.*m44 - 1./12.*
         m14 + 1./24.*m11 );

      F13p6x23RV +=  + Ab(m14,mu2)*BbV(m15,L) * ( 1./48.*m55 + 1./48.*
         m44 - 1./24.*m15 - 1./24.*m14 + 1./24.*m11 );

      F13p6x23RV +=  + Ab(m14,mu2)*BbV(m16,L) * ( 1./48.*m66 + 1./48.*
         m44 - 1./24.*m16 - 1./24.*m14 + 1./24.*m11 );

      F13p6x23RV +=  + Ab(m14,mu2)*BbV(m33,L) * (  - 1./48.*RD3456ep );

      F13p6x23RV +=  + Ab(m14,mu2)*CbV(m11,L) * (  - 1./36.*m44*
         RD1456ep + 1./18.*m14*RD1456ep - 1./36.*m11*RD1456ep );

      F13p6x23RV +=  + Ab(m15,mu2)*AbV(m11,L) * (  - 1./36.*RS14563pe
          + 7./144.*RC1456pe - 5./72.*RS146pe );

      F13p6x23RV +=  + Ab(m15,mu2)*AbV(m14,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m15,mu2)*AbV(m15,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m15,mu2)*AbV(m16,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m15,mu2)*AbV(m33,L) * (  - 1./36.*RS34561pe
          - 1./48.*RC3456pe + 5./72.*RS346pe );

      F13p6x23RV +=  + Ab(m15,mu2)*AbV(m34,L) * ( 1./16. );

      F13p6x23RV +=  + Ab(m15,mu2)*AbV(m35,L) * ( 1./16. + 7./216.*
         RS34561pe + 7./216.*RS14563pe + 7./216.*RSe45613p + 7./216.*
         RSp45613e - 1./216.*RC3456pe - 1./216.*RSe45633p - 1./216.*
         RSp45633e - 5./216.*RS346pe - 5./216.*RSe463p - 5./216.*
         RSp463e - 1./216.*RC1456pe - 1./216.*RSe45611p - 1./216.*
         RSp45611e - 5./216.*RS146pe + 5./216.*RS546pe - 5./216.*
         RSe461p - 5./216.*RSp461e + 5./216.*RSe465p + 5./216.*RSp465e
          );

      F13p6x23RV +=  + Ab(m15,mu2)*AbV(m36,L) * ( 1./16. );

      F13p6x23RV +=  + Ab(m15,mu2)*AbV(mp2,L) * (  - 1./36.*RSp45613e
          - 1./48.*RSp45633e + 5./72.*RSp463e + 7./144.*RSp45611e - 5./
         72.*RSp461e );

      F13p6x23RV +=  + Ab(m15,mu2)*AbV(me2,L) * (  - 1./36.*RSe45613p
          - 1./48.*RSe45633p + 5./72.*RSe463p + 7./144.*RSe45611p - 5./
         72.*RSe461p );

      F13p6x23RV +=  + Ab(m15,mu2)*BbV(m11,m33,L) * ( 1./36.*m55*
         RS34561pe + 1./36.*m55*RS346pe - 1./18.*m15*RS34561pe + 1./18.
         *m15*RS346pe + 1./36.*m11*RS34561pe + 1./12.*m11*RS346pe );

      F13p6x23RV +=  + Ab(m15,mu2)*BbV(m11,mp2,L) * ( 1./36.*m55*
         RSp45613e + 1./36.*m55*RSp463e - 1./36.*m55*RSp45611e - 1./36.
         *m55*RSp461e - 1./18.*m15*RSp45613e + 1./18.*m15*RSp463e + 1./
         18.*m15*RSp45611e - 1./18.*m15*RSp461e + 1./36.*m11*RSp45613e
          + 1./12.*m11*RSp463e - 1./36.*m11*RSp45611e - 1./12.*m11*
         RSp461e );

      F13p6x23RV +=  + Ab(m15,mu2)*BbV(m11,me2,L) * ( 1./36.*m55*
         RSe45613p + 1./36.*m55*RSe463p - 1./36.*m55*RSe45611p - 1./36.
         *m55*RSe461p - 1./18.*m15*RSe45613p + 1./18.*m15*RSe463p + 1./
         18.*m15*RSe45611p - 1./18.*m15*RSe461p + 1./36.*m11*RSe45613p
          + 1./12.*m11*RSe463p - 1./36.*m11*RSe45611p - 1./12.*m11*
         RSe461p );

      F13p6x23RV +=  + Ab(m15,mu2)*BbV(m11,L) * ( 7./144.*RD1456ep + 1./
         36.*m55*RS14563pe - 1./36.*m55*RC1456pe - 1./36.*m55*RS146pe
          - 1./18.*m15*RS14563pe + 1./18.*m15*RC1456pe - 1./18.*m15*
         RS146pe + 1./36.*m11*RS14563pe - 1./36.*m11*RC1456pe - 1./12.*
         m11*RS146pe );

      F13p6x23RV +=  + Ab(m15,mu2)*BbV(m14,L) * ( 1./48.*m55 + 1./48.*
         m44 - 1./24.*m15 - 1./24.*m14 + 1./24.*m11 );

      F13p6x23RV +=  + Ab(m15,mu2)*BbV(m15,L) * ( 1./24.*m55 - 1./12.*
         m15 + 1./24.*m11 );

      F13p6x23RV +=  + Ab(m15,mu2)*BbV(m16,L) * ( 1./48.*m66 + 1./48.*
         m55 - 1./24.*m16 - 1./24.*m15 + 1./24.*m11 );

      F13p6x23RV +=  + Ab(m15,mu2)*BbV(m33,L) * (  - 1./48.*RD3456ep );

      F13p6x23RV +=  + Ab(m15,mu2)*CbV(m11,L) * (  - 1./36.*m55*
         RD1456ep + 1./18.*m15*RD1456ep - 1./36.*m11*RD1456ep );

      F13p6x23RV +=  + Ab(m16,mu2)*AbV(m11,L) * (  - 1./36.*RS14563pe
          + 7./144.*RC1456pe - 5./72.*RS145pe );

      F13p6x23RV +=  + Ab(m16,mu2)*AbV(m14,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m16,mu2)*AbV(m15,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m16,mu2)*AbV(m16,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m16,mu2)*AbV(m33,L) * (  - 1./36.*RS34561pe
          - 1./48.*RC3456pe + 5./72.*RS345pe );

      F13p6x23RV +=  + Ab(m16,mu2)*AbV(m34,L) * ( 1./16. );

      F13p6x23RV +=  + Ab(m16,mu2)*AbV(m35,L) * ( 1./16. );

      F13p6x23RV +=  + Ab(m16,mu2)*AbV(m36,L) * ( 1./16. + 7./216.*
         RS34561pe + 7./216.*RS14563pe + 7./216.*RSe45613p + 7./216.*
         RSp45613e - 1./216.*RC3456pe - 1./216.*RSe45633p - 1./216.*
         RSp45633e - 5./216.*RS345pe - 5./216.*RSe453p - 5./216.*
         RSp453e - 1./216.*RC1456pe - 1./216.*RSe45611p - 1./216.*
         RSp45611e - 5./216.*RS145pe + 5./216.*RS645pe - 5./216.*
         RSe451p - 5./216.*RSp451e + 5./216.*RSe456p + 5./216.*RSp456e
          );

      F13p6x23RV +=  + Ab(m16,mu2)*AbV(mp2,L) * (  - 1./36.*RSp45613e
          - 1./48.*RSp45633e + 5./72.*RSp453e + 7./144.*RSp45611e - 5./
         72.*RSp451e );

      F13p6x23RV +=  + Ab(m16,mu2)*AbV(me2,L) * (  - 1./36.*RSe45613p
          - 1./48.*RSe45633p + 5./72.*RSe453p + 7./144.*RSe45611p - 5./
         72.*RSe451p );

      F13p6x23RV +=  + Ab(m16,mu2)*BbV(m11,m33,L) * ( 1./36.*m66*
         RS34561pe + 1./36.*m66*RS345pe - 1./18.*m16*RS34561pe + 1./18.
         *m16*RS345pe + 1./36.*m11*RS34561pe + 1./12.*m11*RS345pe );

      F13p6x23RV +=  + Ab(m16,mu2)*BbV(m11,mp2,L) * ( 1./36.*m66*
         RSp45613e + 1./36.*m66*RSp453e - 1./36.*m66*RSp45611e - 1./36.
         *m66*RSp451e - 1./18.*m16*RSp45613e + 1./18.*m16*RSp453e + 1./
         18.*m16*RSp45611e - 1./18.*m16*RSp451e + 1./36.*m11*RSp45613e
          + 1./12.*m11*RSp453e - 1./36.*m11*RSp45611e - 1./12.*m11*
         RSp451e );

      F13p6x23RV +=  + Ab(m16,mu2)*BbV(m11,me2,L) * ( 1./36.*m66*
         RSe45613p + 1./36.*m66*RSe453p - 1./36.*m66*RSe45611p - 1./36.
         *m66*RSe451p - 1./18.*m16*RSe45613p + 1./18.*m16*RSe453p + 1./
         18.*m16*RSe45611p - 1./18.*m16*RSe451p + 1./36.*m11*RSe45613p
          + 1./12.*m11*RSe453p - 1./36.*m11*RSe45611p - 1./12.*m11*
         RSe451p );

      F13p6x23RV +=  + Ab(m16,mu2)*BbV(m11,L) * ( 7./144.*RD1456ep + 1./
         36.*m66*RS14563pe - 1./36.*m66*RC1456pe - 1./36.*m66*RS145pe
          - 1./18.*m16*RS14563pe + 1./18.*m16*RC1456pe - 1./18.*m16*
         RS145pe + 1./36.*m11*RS14563pe - 1./36.*m11*RC1456pe - 1./12.*
         m11*RS145pe );

      F13p6x23RV +=  + Ab(m16,mu2)*BbV(m14,L) * ( 1./48.*m66 + 1./48.*
         m44 - 1./24.*m16 - 1./24.*m14 + 1./24.*m11 );

      F13p6x23RV +=  + Ab(m16,mu2)*BbV(m15,L) * ( 1./48.*m66 + 1./48.*
         m55 - 1./24.*m16 - 1./24.*m15 + 1./24.*m11 );

      F13p6x23RV +=  + Ab(m16,mu2)*BbV(m16,L) * ( 1./24.*m66 - 1./12.*
         m16 + 1./24.*m11 );

      F13p6x23RV +=  + Ab(m16,mu2)*BbV(m33,L) * (  - 1./48.*RD3456ep );

      F13p6x23RV +=  + Ab(m16,mu2)*CbV(m11,L) * (  - 1./36.*m66*
         RD1456ep + 1./18.*m16*RD1456ep - 1./36.*m11*RD1456ep );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m11,L) * ( 1./36.*RS14563pe*
         RS34561pe - 1./72.*RC3456pe*RS14563pe - 1./72.*RC1456pe*
         RS34561pe + 1./144.*RC1456pe*RC3456pe + 1./18.*m11*RN13*
         RC1456pe*RS34561pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m13,L) * (  - 5./24. + 1./162.*
         pow(RS34561pe,2) + 1./162.*RS14563pe*RS34561pe + 1./162.*
         RSe45613p*RS34561pe + 1./162.*RSp45613e*RS34561pe - 1./162.*
         RC3456pe*RS34561pe - 1./324.*RC3456pe*RS14563pe - 1./324.*
         RC3456pe*RSe45613p - 1./324.*RC3456pe*RSp45613e + 1./648.*pow(
         RC3456pe,2) - 1./324.*RSe45633p*RS34561pe + 1./648.*RSe45633p*
         RC3456pe - 1./324.*RSp45633e*RS34561pe + 1./648.*RSp45633e*
         RC3456pe - 1./324.*RC1456pe*RS34561pe + 1./648.*RC1456pe*
         RC3456pe - 1./324.*RSe45611p*RS34561pe + 1./648.*RSe45611p*
         RC3456pe - 1./324.*RSp45611e*RS34561pe + 1./648.*RSp45611e*
         RC3456pe + 1./12.*m33*RN13 + 1./6.*m13*RN13 + 1./4.*m11*RN13 )
         ;

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m14,L) * (  - 1./36.*RS34561pe
          - 1./48.*RC3456pe + 5./72.*RS356pe - 1./36.*m44*RN13*
         RS34561pe - 1./36.*m44*RN13*RS356pe + 1./18.*m14*RN13*
         RS34561pe - 1./18.*m14*RN13*RS356pe - 1./36.*m11*RN13*
         RS34561pe - 1./12.*m11*RN13*RS356pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m15,L) * (  - 1./36.*RS34561pe
          - 1./48.*RC3456pe + 5./72.*RS346pe - 1./36.*m55*RN13*
         RS34561pe - 1./36.*m55*RN13*RS346pe + 1./18.*m15*RN13*
         RS34561pe - 1./18.*m15*RN13*RS346pe - 1./36.*m11*RN13*
         RS34561pe - 1./12.*m11*RN13*RS346pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m16,L) * (  - 1./36.*RS34561pe
          - 1./48.*RC3456pe + 5./72.*RS345pe - 1./36.*m66*RN13*
         RS34561pe - 1./36.*m66*RN13*RS345pe + 1./18.*m16*RN13*
         RS34561pe - 1./18.*m16*RN13*RS345pe - 1./36.*m11*RN13*
         RS34561pe - 1./12.*m11*RN13*RS345pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m33,L) * ( 1./8. + 1./36.*pow(
         RS34561pe,2) - 1./36.*RC3456pe*RS34561pe + 1./144.*pow(
         RC3456pe,2) - 1./18.*m33*RN3p*RC3456pe*RSp45613e + 1./18.*m33*
         RN3p*RSp45633e*RC3456pe - 1./18.*m33*RN3e*RC3456pe*RSe45613p
          + 1./18.*m33*RN3e*RSe45633p*RC3456pe + 1./18.*m33*RN13*
         RC3456pe*RS14563pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m34,L) * (  - 1./36.*RS34561pe
          - 1./18.*RS14563pe + 7./144.*RC3456pe - 5./72.*RS356pe + 1./
         18.*RS156pe + 1./36.*m44*RN3p*RSp45613e - 1./36.*m44*RN3p*
         RSp45633e - 1./36.*m44*RN3p*RSp563e + 1./36.*m44*RN3p*RSp561e
          + 1./36.*m44*RN3e*RSe45613p - 1./36.*m44*RN3e*RSe45633p - 1./
         36.*m44*RN3e*RSe563p + 1./36.*m44*RN3e*RSe561p - 1./36.*m44*
         RN13*RS14563pe - 1./36.*m44*RN13*RS156pe - 1./18.*m34*RN3p*
         RSp45613e + 1./18.*m34*RN3p*RSp45633e - 1./18.*m34*RN3p*
         RSp563e + 1./18.*m34*RN3p*RSp561e - 1./18.*m34*RN3e*RSe45613p
          + 1./18.*m34*RN3e*RSe45633p - 1./18.*m34*RN3e*RSe563p + 1./18.
         *m34*RN3e*RSe561p + 1./18.*m34*RN13*RS14563pe - 1./18.*m34*
         RN13*RS156pe + 1./36.*m33*RN3p*RSp45613e - 1./36.*m33*RN3p*
         RSp45633e - 1./12.*m33*RN3p*RSp563e + 1./12.*m33*RN3p*RSp561e
          + 1./36.*m33*RN3e*RSe45613p - 1./36.*m33*RN3e*RSe45633p - 1./
         12.*m33*RN3e*RSe563p + 1./12.*m33*RN3e*RSe561p - 1./12.*m33*
         RN13*RS14563pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m34,L) * (  - 1./36.*m33*RN13*
         RS156pe + 1./18.*m11*RN13*RS14563pe - 1./18.*m11*RN13*RS156pe
          );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m35,L) * (  - 1./36.*RS34561pe
          - 1./18.*RS14563pe + 7./144.*RC3456pe - 5./72.*RS346pe + 1./
         18.*RS146pe + 1./36.*m55*RN3p*RSp45613e - 1./36.*m55*RN3p*
         RSp45633e - 1./36.*m55*RN3p*RSp463e + 1./36.*m55*RN3p*RSp461e
          + 1./36.*m55*RN3e*RSe45613p - 1./36.*m55*RN3e*RSe45633p - 1./
         36.*m55*RN3e*RSe463p + 1./36.*m55*RN3e*RSe461p - 1./36.*m55*
         RN13*RS14563pe - 1./36.*m55*RN13*RS146pe - 1./18.*m35*RN3p*
         RSp45613e + 1./18.*m35*RN3p*RSp45633e - 1./18.*m35*RN3p*
         RSp463e + 1./18.*m35*RN3p*RSp461e - 1./18.*m35*RN3e*RSe45613p
          + 1./18.*m35*RN3e*RSe45633p - 1./18.*m35*RN3e*RSe463p + 1./18.
         *m35*RN3e*RSe461p + 1./18.*m35*RN13*RS14563pe - 1./18.*m35*
         RN13*RS146pe + 1./36.*m33*RN3p*RSp45613e - 1./36.*m33*RN3p*
         RSp45633e - 1./12.*m33*RN3p*RSp463e + 1./12.*m33*RN3p*RSp461e
          + 1./36.*m33*RN3e*RSe45613p - 1./36.*m33*RN3e*RSe45633p - 1./
         12.*m33*RN3e*RSe463p + 1./12.*m33*RN3e*RSe461p - 1./12.*m33*
         RN13*RS14563pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m35,L) * (  - 1./36.*m33*RN13*
         RS146pe + 1./18.*m11*RN13*RS14563pe - 1./18.*m11*RN13*RS146pe
          );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m36,L) * (  - 1./36.*RS34561pe
          - 1./18.*RS14563pe + 7./144.*RC3456pe - 5./72.*RS345pe + 1./
         18.*RS145pe + 1./36.*m66*RN3p*RSp45613e - 1./36.*m66*RN3p*
         RSp45633e - 1./36.*m66*RN3p*RSp453e + 1./36.*m66*RN3p*RSp451e
          + 1./36.*m66*RN3e*RSe45613p - 1./36.*m66*RN3e*RSe45633p - 1./
         36.*m66*RN3e*RSe453p + 1./36.*m66*RN3e*RSe451p - 1./36.*m66*
         RN13*RS14563pe - 1./36.*m66*RN13*RS145pe - 1./18.*m36*RN3p*
         RSp45613e + 1./18.*m36*RN3p*RSp45633e - 1./18.*m36*RN3p*
         RSp453e + 1./18.*m36*RN3p*RSp451e - 1./18.*m36*RN3e*RSe45613p
          + 1./18.*m36*RN3e*RSe45633p - 1./18.*m36*RN3e*RSe453p + 1./18.
         *m36*RN3e*RSe451p + 1./18.*m36*RN13*RS14563pe - 1./18.*m36*
         RN13*RS145pe + 1./36.*m33*RN3p*RSp45613e - 1./36.*m33*RN3p*
         RSp45633e - 1./12.*m33*RN3p*RSp453e + 1./12.*m33*RN3p*RSp451e
          + 1./36.*m33*RN3e*RSe45613p - 1./36.*m33*RN3e*RSe45633p - 1./
         12.*m33*RN3e*RSe453p + 1./12.*m33*RN3e*RSe451p - 1./12.*m33*
         RN13*RS14563pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m36,L) * (  - 1./36.*m33*RN13*
         RS145pe + 1./18.*m11*RN13*RS14563pe - 1./18.*m11*RN13*RS145pe
          );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m44,L) * ( 1./18.*m44*RN3p*
         RSp563e*RS356pe - 1./54.*m44*RN3p*RS456pe*RSp563e*RS356pe - 1./
         18.*m44*RN3p*RSp561e*RS356pe + 1./54.*m44*RN3p*RSp561e*RS456pe
         *RS356pe + 1./18.*m44*RN3e*RSe563p*RS356pe - 1./54.*m44*RN3e*
         RS456pe*RSe563p*RS356pe - 1./18.*m44*RN3e*RSe561p*RS356pe + 1./
         54.*m44*RN3e*RSe561p*RS456pe*RS356pe + 1./18.*m44*RN13*RS156pe
         *RS356pe - 1./54.*m44*RN13*RS456pe*RS156pe*RS356pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * (  - 1./108.*pow(
         RS346pe,2) + 1./54.*RS356pe*RS346pe - 1./108.*pow(RS356pe,2)
          + 1./54.*RS146pe*RS346pe - 1./54.*RS146pe*RS356pe - 1./54.*
         RS156pe*RS346pe + 1./54.*RS156pe*RS356pe + 1./108.*m55*RN3p*
         RSp563e*RS346pe + 1./108.*m55*RN3p*RSp563e*RS356pe + 1./36.*
         m55*RN3p*RSp463e*RS346pe + 1./108.*m55*RN3p*RSp463e*RS356pe - 
         1./108.*m55*RN3p*RSp561e*RS346pe - 1./108.*m55*RN3p*RSp561e*
         RS356pe - 1./36.*m55*RN3p*RSp461e*RS346pe - 1./108.*m55*RN3p*
         RSp461e*RS356pe + 1./108.*m55*RN3e*RSe563p*RS346pe + 1./108.*
         m55*RN3e*RSe563p*RS356pe + 1./36.*m55*RN3e*RSe463p*RS346pe + 1.
         /108.*m55*RN3e*RSe463p*RS356pe - 1./108.*m55*RN3e*RSe561p*
         RS346pe - 1./108.*m55*RN3e*RSe561p*RS356pe - 1./36.*m55*RN3e*
         RSe461p*RS346pe - 1./108.*m55*RN3e*RSe461p*RS356pe + 1./36.*
         m55*RN13*RS146pe*RS346pe + 1./108.*m55*RN13*RS146pe*RS356pe + 
         1./108.*m55*RN13*RS156pe*RS346pe + 1./108.*m55*RN13*RS156pe*
         RS356pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * ( 1./54.*m45*RN3p*
         RSp563e*RS346pe - 1./54.*m45*RN3p*RSp563e*RS356pe - 1./54.*m45
         *RN3p*RSp463e*RS346pe + 1./54.*m45*RN3p*RSp463e*RS356pe - 1./
         54.*m45*RN3p*RSp561e*RS346pe + 1./54.*m45*RN3p*RSp561e*RS356pe
          + 1./54.*m45*RN3p*RSp461e*RS346pe - 1./54.*m45*RN3p*RSp461e*
         RS356pe + 1./54.*m45*RN3e*RSe563p*RS346pe - 1./54.*m45*RN3e*
         RSe563p*RS356pe - 1./54.*m45*RN3e*RSe463p*RS346pe + 1./54.*m45
         *RN3e*RSe463p*RS356pe - 1./54.*m45*RN3e*RSe561p*RS346pe + 1./
         54.*m45*RN3e*RSe561p*RS356pe + 1./54.*m45*RN3e*RSe461p*RS346pe
          - 1./54.*m45*RN3e*RSe461p*RS356pe - 1./54.*m45*RN13*RS146pe*
         RS346pe + 1./54.*m45*RN13*RS146pe*RS356pe + 1./54.*m45*RN13*
         RS156pe*RS346pe - 1./54.*m45*RN13*RS156pe*RS356pe + 1./108.*
         m44*RN3p*RSp563e*RS346pe + 1./36.*m44*RN3p*RSp563e*RS356pe + 1.
         /108.*m44*RN3p*RSp463e*RS346pe + 1./108.*m44*RN3p*RSp463e*
         RS356pe - 1./108.*m44*RN3p*RSp561e*RS346pe - 1./36.*m44*RN3p*
         RSp561e*RS356pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * (  - 1./108.*m44*RN3p*
         RSp461e*RS346pe - 1./108.*m44*RN3p*RSp461e*RS356pe + 1./108.*
         m44*RN3e*RSe563p*RS346pe + 1./36.*m44*RN3e*RSe563p*RS356pe + 1.
         /108.*m44*RN3e*RSe463p*RS346pe + 1./108.*m44*RN3e*RSe463p*
         RS356pe - 1./108.*m44*RN3e*RSe561p*RS346pe - 1./36.*m44*RN3e*
         RSe561p*RS356pe - 1./108.*m44*RN3e*RSe461p*RS346pe - 1./108.*
         m44*RN3e*RSe461p*RS356pe + 1./108.*m44*RN13*RS146pe*RS346pe + 
         1./108.*m44*RN13*RS146pe*RS356pe + 1./108.*m44*RN13*RS156pe*
         RS346pe + 1./36.*m44*RN13*RS156pe*RS356pe + 1./54.*m33*RN3p*
         RSp563e*RS346pe - 1./54.*m33*RN3p*RSp563e*RS356pe - 1./54.*m33
         *RN3p*RSp463e*RS346pe + 1./54.*m33*RN3p*RSp463e*RS356pe - 1./
         54.*m33*RN3p*RSp561e*RS346pe + 1./54.*m33*RN3p*RSp561e*RS356pe
          + 1./54.*m33*RN3p*RSp461e*RS346pe - 1./54.*m33*RN3p*RSp461e*
         RS356pe + 1./54.*m33*RN3e*RSe563p*RS346pe - 1./54.*m33*RN3e*
         RSe563p*RS356pe - 1./54.*m33*RN3e*RSe463p*RS346pe + 1./54.*m33
         *RN3e*RSe463p*RS356pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m45,L) * (  - 1./54.*m33*RN3e*
         RSe561p*RS346pe + 1./54.*m33*RN3e*RSe561p*RS356pe + 1./54.*m33
         *RN3e*RSe461p*RS346pe - 1./54.*m33*RN3e*RSe461p*RS356pe - 1./
         54.*m11*RN13*RS146pe*RS346pe + 1./54.*m11*RN13*RS146pe*RS356pe
          + 1./54.*m11*RN13*RS156pe*RS346pe - 1./54.*m11*RN13*RS156pe*
         RS356pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * (  - 1./108.*pow(
         RS345pe,2) + 1./54.*RS356pe*RS345pe - 1./108.*pow(RS356pe,2)
          + 1./54.*RS145pe*RS345pe - 1./54.*RS145pe*RS356pe - 1./54.*
         RS156pe*RS345pe + 1./54.*RS156pe*RS356pe + 1./108.*m66*RN3p*
         RSp563e*RS345pe + 1./108.*m66*RN3p*RSp563e*RS356pe + 1./36.*
         m66*RN3p*RSp453e*RS345pe + 1./108.*m66*RN3p*RSp453e*RS356pe - 
         1./108.*m66*RN3p*RSp561e*RS345pe - 1./108.*m66*RN3p*RSp561e*
         RS356pe - 1./36.*m66*RN3p*RSp451e*RS345pe - 1./108.*m66*RN3p*
         RSp451e*RS356pe + 1./108.*m66*RN3e*RSe563p*RS345pe + 1./108.*
         m66*RN3e*RSe563p*RS356pe + 1./36.*m66*RN3e*RSe453p*RS345pe + 1.
         /108.*m66*RN3e*RSe453p*RS356pe - 1./108.*m66*RN3e*RSe561p*
         RS345pe - 1./108.*m66*RN3e*RSe561p*RS356pe - 1./36.*m66*RN3e*
         RSe451p*RS345pe - 1./108.*m66*RN3e*RSe451p*RS356pe + 1./36.*
         m66*RN13*RS145pe*RS345pe + 1./108.*m66*RN13*RS145pe*RS356pe + 
         1./108.*m66*RN13*RS156pe*RS345pe + 1./108.*m66*RN13*RS156pe*
         RS356pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * ( 1./54.*m46*RN3p*
         RSp563e*RS345pe - 1./54.*m46*RN3p*RSp563e*RS356pe - 1./54.*m46
         *RN3p*RSp453e*RS345pe + 1./54.*m46*RN3p*RSp453e*RS356pe - 1./
         54.*m46*RN3p*RSp561e*RS345pe + 1./54.*m46*RN3p*RSp561e*RS356pe
          + 1./54.*m46*RN3p*RSp451e*RS345pe - 1./54.*m46*RN3p*RSp451e*
         RS356pe + 1./54.*m46*RN3e*RSe563p*RS345pe - 1./54.*m46*RN3e*
         RSe563p*RS356pe - 1./54.*m46*RN3e*RSe453p*RS345pe + 1./54.*m46
         *RN3e*RSe453p*RS356pe - 1./54.*m46*RN3e*RSe561p*RS345pe + 1./
         54.*m46*RN3e*RSe561p*RS356pe + 1./54.*m46*RN3e*RSe451p*RS345pe
          - 1./54.*m46*RN3e*RSe451p*RS356pe - 1./54.*m46*RN13*RS145pe*
         RS345pe + 1./54.*m46*RN13*RS145pe*RS356pe + 1./54.*m46*RN13*
         RS156pe*RS345pe - 1./54.*m46*RN13*RS156pe*RS356pe + 1./108.*
         m44*RN3p*RSp563e*RS345pe + 1./36.*m44*RN3p*RSp563e*RS356pe + 1.
         /108.*m44*RN3p*RSp453e*RS345pe + 1./108.*m44*RN3p*RSp453e*
         RS356pe - 1./108.*m44*RN3p*RSp561e*RS345pe - 1./36.*m44*RN3p*
         RSp561e*RS356pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * (  - 1./108.*m44*RN3p*
         RSp451e*RS345pe - 1./108.*m44*RN3p*RSp451e*RS356pe + 1./108.*
         m44*RN3e*RSe563p*RS345pe + 1./36.*m44*RN3e*RSe563p*RS356pe + 1.
         /108.*m44*RN3e*RSe453p*RS345pe + 1./108.*m44*RN3e*RSe453p*
         RS356pe - 1./108.*m44*RN3e*RSe561p*RS345pe - 1./36.*m44*RN3e*
         RSe561p*RS356pe - 1./108.*m44*RN3e*RSe451p*RS345pe - 1./108.*
         m44*RN3e*RSe451p*RS356pe + 1./108.*m44*RN13*RS145pe*RS345pe + 
         1./108.*m44*RN13*RS145pe*RS356pe + 1./108.*m44*RN13*RS156pe*
         RS345pe + 1./36.*m44*RN13*RS156pe*RS356pe + 1./54.*m33*RN3p*
         RSp563e*RS345pe - 1./54.*m33*RN3p*RSp563e*RS356pe - 1./54.*m33
         *RN3p*RSp453e*RS345pe + 1./54.*m33*RN3p*RSp453e*RS356pe - 1./
         54.*m33*RN3p*RSp561e*RS345pe + 1./54.*m33*RN3p*RSp561e*RS356pe
          + 1./54.*m33*RN3p*RSp451e*RS345pe - 1./54.*m33*RN3p*RSp451e*
         RS356pe + 1./54.*m33*RN3e*RSe563p*RS345pe - 1./54.*m33*RN3e*
         RSe563p*RS356pe - 1./54.*m33*RN3e*RSe453p*RS345pe + 1./54.*m33
         *RN3e*RSe453p*RS356pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m46,L) * (  - 1./54.*m33*RN3e*
         RSe561p*RS345pe + 1./54.*m33*RN3e*RSe561p*RS356pe + 1./54.*m33
         *RN3e*RSe451p*RS345pe - 1./54.*m33*RN3e*RSe451p*RS356pe - 1./
         54.*m11*RN13*RS145pe*RS345pe + 1./54.*m11*RN13*RS145pe*RS356pe
          + 1./54.*m11*RN13*RS156pe*RS345pe - 1./54.*m11*RN13*RS156pe*
         RS356pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m55,L) * ( 1./18.*m55*RN3p*
         RSp463e*RS346pe - 1./54.*m55*RN3p*RS546pe*RSp463e*RS346pe - 1./
         18.*m55*RN3p*RSp461e*RS346pe + 1./54.*m55*RN3p*RSp461e*RS546pe
         *RS346pe + 1./18.*m55*RN3e*RSe463p*RS346pe - 1./54.*m55*RN3e*
         RS546pe*RSe463p*RS346pe - 1./18.*m55*RN3e*RSe461p*RS346pe + 1./
         54.*m55*RN3e*RSe461p*RS546pe*RS346pe + 1./18.*m55*RN13*RS146pe
         *RS346pe - 1./54.*m55*RN13*RS546pe*RS146pe*RS346pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * (  - 1./108.*pow(
         RS345pe,2) + 1./54.*RS346pe*RS345pe - 1./108.*pow(RS346pe,2)
          + 1./54.*RS145pe*RS345pe - 1./54.*RS145pe*RS346pe - 1./54.*
         RS146pe*RS345pe + 1./54.*RS146pe*RS346pe + 1./108.*m66*RN3p*
         RSp463e*RS345pe + 1./108.*m66*RN3p*RSp463e*RS346pe + 1./36.*
         m66*RN3p*RSp453e*RS345pe + 1./108.*m66*RN3p*RSp453e*RS346pe - 
         1./108.*m66*RN3p*RSp461e*RS345pe - 1./108.*m66*RN3p*RSp461e*
         RS346pe - 1./36.*m66*RN3p*RSp451e*RS345pe - 1./108.*m66*RN3p*
         RSp451e*RS346pe + 1./108.*m66*RN3e*RSe463p*RS345pe + 1./108.*
         m66*RN3e*RSe463p*RS346pe + 1./36.*m66*RN3e*RSe453p*RS345pe + 1.
         /108.*m66*RN3e*RSe453p*RS346pe - 1./108.*m66*RN3e*RSe461p*
         RS345pe - 1./108.*m66*RN3e*RSe461p*RS346pe - 1./36.*m66*RN3e*
         RSe451p*RS345pe - 1./108.*m66*RN3e*RSe451p*RS346pe + 1./36.*
         m66*RN13*RS145pe*RS345pe + 1./108.*m66*RN13*RS145pe*RS346pe + 
         1./108.*m66*RN13*RS146pe*RS345pe + 1./108.*m66*RN13*RS146pe*
         RS346pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * ( 1./54.*m56*RN3p*
         RSp463e*RS345pe - 1./54.*m56*RN3p*RSp463e*RS346pe - 1./54.*m56
         *RN3p*RSp453e*RS345pe + 1./54.*m56*RN3p*RSp453e*RS346pe - 1./
         54.*m56*RN3p*RSp461e*RS345pe + 1./54.*m56*RN3p*RSp461e*RS346pe
          + 1./54.*m56*RN3p*RSp451e*RS345pe - 1./54.*m56*RN3p*RSp451e*
         RS346pe + 1./54.*m56*RN3e*RSe463p*RS345pe - 1./54.*m56*RN3e*
         RSe463p*RS346pe - 1./54.*m56*RN3e*RSe453p*RS345pe + 1./54.*m56
         *RN3e*RSe453p*RS346pe - 1./54.*m56*RN3e*RSe461p*RS345pe + 1./
         54.*m56*RN3e*RSe461p*RS346pe + 1./54.*m56*RN3e*RSe451p*RS345pe
          - 1./54.*m56*RN3e*RSe451p*RS346pe - 1./54.*m56*RN13*RS145pe*
         RS345pe + 1./54.*m56*RN13*RS145pe*RS346pe + 1./54.*m56*RN13*
         RS146pe*RS345pe - 1./54.*m56*RN13*RS146pe*RS346pe + 1./108.*
         m55*RN3p*RSp463e*RS345pe + 1./36.*m55*RN3p*RSp463e*RS346pe + 1.
         /108.*m55*RN3p*RSp453e*RS345pe + 1./108.*m55*RN3p*RSp453e*
         RS346pe - 1./108.*m55*RN3p*RSp461e*RS345pe - 1./36.*m55*RN3p*
         RSp461e*RS346pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * (  - 1./108.*m55*RN3p*
         RSp451e*RS345pe - 1./108.*m55*RN3p*RSp451e*RS346pe + 1./108.*
         m55*RN3e*RSe463p*RS345pe + 1./36.*m55*RN3e*RSe463p*RS346pe + 1.
         /108.*m55*RN3e*RSe453p*RS345pe + 1./108.*m55*RN3e*RSe453p*
         RS346pe - 1./108.*m55*RN3e*RSe461p*RS345pe - 1./36.*m55*RN3e*
         RSe461p*RS346pe - 1./108.*m55*RN3e*RSe451p*RS345pe - 1./108.*
         m55*RN3e*RSe451p*RS346pe + 1./108.*m55*RN13*RS145pe*RS345pe + 
         1./108.*m55*RN13*RS145pe*RS346pe + 1./108.*m55*RN13*RS146pe*
         RS345pe + 1./36.*m55*RN13*RS146pe*RS346pe + 1./54.*m33*RN3p*
         RSp463e*RS345pe - 1./54.*m33*RN3p*RSp463e*RS346pe - 1./54.*m33
         *RN3p*RSp453e*RS345pe + 1./54.*m33*RN3p*RSp453e*RS346pe - 1./
         54.*m33*RN3p*RSp461e*RS345pe + 1./54.*m33*RN3p*RSp461e*RS346pe
          + 1./54.*m33*RN3p*RSp451e*RS345pe - 1./54.*m33*RN3p*RSp451e*
         RS346pe + 1./54.*m33*RN3e*RSe463p*RS345pe - 1./54.*m33*RN3e*
         RSe463p*RS346pe - 1./54.*m33*RN3e*RSe453p*RS345pe + 1./54.*m33
         *RN3e*RSe453p*RS346pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m56,L) * (  - 1./54.*m33*RN3e*
         RSe461p*RS345pe + 1./54.*m33*RN3e*RSe461p*RS346pe + 1./54.*m33
         *RN3e*RSe451p*RS345pe - 1./54.*m33*RN3e*RSe451p*RS346pe - 1./
         54.*m11*RN13*RS145pe*RS345pe + 1./54.*m11*RN13*RS145pe*RS346pe
          + 1./54.*m11*RN13*RS146pe*RS345pe - 1./54.*m11*RN13*RS146pe*
         RS346pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(m66,L) * ( 1./18.*m66*RN3p*
         RSp453e*RS345pe - 1./54.*m66*RN3p*RS645pe*RSp453e*RS345pe - 1./
         18.*m66*RN3p*RSp451e*RS345pe + 1./54.*m66*RN3p*RSp451e*RS645pe
         *RS345pe + 1./18.*m66*RN3e*RSe453p*RS345pe - 1./54.*m66*RN3e*
         RS645pe*RSe453p*RS345pe - 1./18.*m66*RN3e*RSe451p*RS345pe + 1./
         54.*m66*RN3e*RSe451p*RS645pe*RS345pe + 1./18.*m66*RN13*RS145pe
         *RS345pe - 1./54.*m66*RN13*RS645pe*RS145pe*RS345pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(mp2,L) * ( 1./36.*RSp45613e*
         RS34561pe - 1./72.*RC3456pe*RSp45613e - 1./72.*RSp45633e*
         RS34561pe + 1./144.*RSp45633e*RC3456pe - 1./72.*RSp45611e*
         RS34561pe + 1./144.*RSp45611e*RC3456pe - 1./54.*m66*RN3p*
         RSp456e*RSp453e*RS345pe + 1./54.*m66*RN3p*RSp456e*RSp451e*
         RS345pe - 1./54.*m66*RN3e*RSp456e*RSe453p*RS345pe + 1./54.*m66
         *RN3e*RSp456e*RSe451p*RS345pe - 1./54.*m66*RN13*RSp456e*
         RS145pe*RS345pe - 1./54.*m55*RN3p*RSp465e*RSp463e*RS346pe + 1./
         54.*m55*RN3p*RSp465e*RSp461e*RS346pe - 1./54.*m55*RN3e*RSp465e
         *RSe463p*RS346pe + 1./54.*m55*RN3e*RSp465e*RSe461p*RS346pe - 1.
         /54.*m55*RN13*RSp465e*RS146pe*RS346pe - 1./54.*m44*RN3p*
         RSp564e*RSp563e*RS356pe + 1./54.*m44*RN3p*RSp564e*RSp561e*
         RS356pe - 1./54.*m44*RN3e*RSp564e*RSe563p*RS356pe + 1./54.*m44
         *RN3e*RSp564e*RSe561p*RS356pe - 1./54.*m44*RN13*RSp564e*
         RS156pe*RS356pe - 1./18.*m33*RN3p*RSp45633e*RSp45613e + 1./18.
         *m33*RN3p*pow(RSp45633e,2) );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(mp2,L) * (  - 1./18.*m33*RN3e*
         RSp45633e*RSe45613p + 1./18.*m33*RN3e*RSp45633e*RSe45633p + 1./
         18.*m33*RN13*RSp45633e*RS14563pe + 1./18.*m11*RN13*RSp45611e*
         RS34561pe );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(me2,L) * ( 1./36.*RSe45613p*
         RS34561pe - 1./72.*RC3456pe*RSe45613p - 1./72.*RSe45633p*
         RS34561pe + 1./144.*RSe45633p*RC3456pe - 1./72.*RSe45611p*
         RS34561pe + 1./144.*RSe45611p*RC3456pe - 1./54.*m66*RN3p*
         RSe456p*RSp453e*RS345pe + 1./54.*m66*RN3p*RSe456p*RSp451e*
         RS345pe - 1./54.*m66*RN3e*RSe456p*RSe453p*RS345pe + 1./54.*m66
         *RN3e*RSe456p*RSe451p*RS345pe - 1./54.*m66*RN13*RSe456p*
         RS145pe*RS345pe - 1./54.*m55*RN3p*RSe465p*RSp463e*RS346pe + 1./
         54.*m55*RN3p*RSe465p*RSp461e*RS346pe - 1./54.*m55*RN3e*RSe465p
         *RSe463p*RS346pe + 1./54.*m55*RN3e*RSe465p*RSe461p*RS346pe - 1.
         /54.*m55*RN13*RSe465p*RS146pe*RS346pe - 1./54.*m44*RN3p*
         RSe564p*RSp563e*RS356pe + 1./54.*m44*RN3p*RSe564p*RSp561e*
         RS356pe - 1./54.*m44*RN3e*RSe564p*RSe563p*RS356pe + 1./54.*m44
         *RN3e*RSe564p*RSe561p*RS356pe - 1./54.*m44*RN13*RSe564p*
         RS156pe*RS356pe - 1./18.*m33*RN3p*RSe45633p*RSp45613e + 1./18.
         *m33*RN3p*RSp45633e*RSe45633p );

      F13p6x23RV +=  + Ab(m33,mu2)*AbV(me2,L) * (  - 1./18.*m33*RN3e*
         RSe45633p*RSe45613p + 1./18.*m33*RN3e*pow(RSe45633p,2) + 1./18.
         *m33*RN13*RSe45633p*RS14563pe + 1./18.*m11*RN13*RSe45611p*
         RS34561pe );

      F13p6x23RV +=  + Ab(m33,mu2)*BbV(m11,m33,L) * (  - 1./18.*m33*
         RC3456pe*RS14563pe );

      F13p6x23RV +=  + Ab(m33,mu2)*BbV(m11,L) * (  - 1./72.*RD1456ep*
         RS34561pe + 1./144.*RD1456ep*RC3456pe + 1./18.*m11*RN13*
         RD1456ep*RS34561pe );

      F13p6x23RV +=  + Ab(m33,mu2)*BbV(m33,mp2,L) * (  - 1./18.*m33*
         RC3456pe*RSp45613e + 1./18.*m33*RSp45633e*RC3456pe );

      F13p6x23RV +=  + Ab(m33,mu2)*BbV(m33,me2,L) * (  - 1./18.*m33*
         RC3456pe*RSe45613p + 1./18.*m33*RSe45633p*RC3456pe );

      F13p6x23RV +=  + Ab(m33,mu2)*BbV(m33,L) * (  - 1./72.*RD3456ep*
         RS34561pe + 1./144.*RD3456ep*RC3456pe + 1./4.*m33 - 1./18.*m33
         *RC3456pe*RS34561pe + 1./18.*m33*pow(RC3456pe,2) - 1./18.*m33*
         RN3p*RD3456ep*RSp45613e + 1./18.*m33*RN3p*RSp45633e*RD3456ep
          - 1./18.*m33*RN3e*RD3456ep*RSe45613p + 1./18.*m33*RN3e*
         RSe45633p*RD3456ep + 1./18.*m33*RN13*RD3456ep*RS14563pe );

      F13p6x23RV +=  + Ab(m33,mu2)*BbV(m34,L) * ( 1./72.*RD3456ep - 1./
         144.*m44*RC3456pe - 1./72.*m44*RS356pe + 1./72.*m34*RC3456pe
          - 1./36.*m34*RS356pe - 1./144.*m33*RC3456pe - 1./24.*m33*
         RS356pe );

      F13p6x23RV +=  + Ab(m33,mu2)*BbV(m35,L) * ( 1./72.*RD3456ep - 1./
         144.*m55*RC3456pe - 1./72.*m55*RS346pe + 1./72.*m35*RC3456pe
          - 1./36.*m35*RS346pe - 1./144.*m33*RC3456pe - 1./24.*m33*
         RS346pe );

      F13p6x23RV +=  + Ab(m33,mu2)*BbV(m36,L) * ( 1./72.*RD3456ep - 1./
         144.*m66*RC3456pe - 1./72.*m66*RS345pe + 1./72.*m36*RC3456pe
          - 1./36.*m36*RS345pe - 1./144.*m33*RC3456pe - 1./24.*m33*
         RS345pe );

      F13p6x23RV +=  + Ab(m33,mu2)*CbV(m33,L) * ( 1./18.*m33*RD3456ep*
         RC3456pe );

      F13p6x23RV +=  + Ab(m34,mu2)*AbV(m11,L) * ( 1./36.*RS14563pe - 1./
         48.*RC1456pe + 1./72.*RS156pe );

      F13p6x23RV +=  + Ab(m34,mu2)*AbV(m14,L) * ( 1./16. + 7./216.*
         RS34561pe + 7./216.*RS14563pe + 7./216.*RSe45613p + 7./216.*
         RSp45613e - 1./216.*RC3456pe - 1./216.*RSe45633p - 1./216.*
         RSp45633e - 5./216.*RS356pe - 5./216.*RSe563p - 5./216.*
         RSp563e - 1./216.*RC1456pe - 1./216.*RSe45611p - 1./216.*
         RSp45611e - 5./216.*RS156pe + 5./216.*RS456pe - 5./216.*
         RSe561p - 5./216.*RSp561e + 5./216.*RSe564p + 5./216.*RSp564e
          );

      F13p6x23RV +=  + Ab(m34,mu2)*AbV(m15,L) * ( 1./16. );

      F13p6x23RV +=  + Ab(m34,mu2)*AbV(m16,L) * ( 1./16. );

      F13p6x23RV +=  + Ab(m34,mu2)*AbV(m33,L) * (  - 1./36.*RS34561pe
          - 1./18.*RS14563pe + 7./144.*RC3456pe - 5./72.*RS356pe + 1./
         18.*RS156pe );

      F13p6x23RV +=  + Ab(m34,mu2)*AbV(m34,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m34,mu2)*AbV(m35,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m34,mu2)*AbV(m36,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m34,mu2)*AbV(mp2,L) * (  - 1./36.*RSp45613e
          + 7./144.*RSp45633e - 5./72.*RSp563e - 1./48.*RSp45611e + 5./
         72.*RSp561e );

      F13p6x23RV +=  + Ab(m34,mu2)*AbV(me2,L) * (  - 1./36.*RSe45613p
          + 7./144.*RSe45633p - 5./72.*RSe563p - 1./48.*RSe45611p + 5./
         72.*RSe561p );

      F13p6x23RV +=  + Ab(m34,mu2)*BbV(m11,m33,L) * ( 1./36.*m44*
         RS14563pe + 1./36.*m44*RS156pe - 1./18.*m34*RS14563pe + 1./18.
         *m34*RS156pe + 1./12.*m33*RS14563pe + 1./36.*m33*RS156pe - 1./
         18.*m11*RS14563pe + 1./18.*m11*RS156pe );

      F13p6x23RV +=  + Ab(m34,mu2)*BbV(m11,L) * (  - 1./48.*RD1456ep );

      F13p6x23RV +=  + Ab(m34,mu2)*BbV(m33,mp2,L) * ( 1./36.*m44*
         RSp45613e - 1./36.*m44*RSp45633e - 1./36.*m44*RSp563e + 1./36.
         *m44*RSp561e - 1./18.*m34*RSp45613e + 1./18.*m34*RSp45633e - 1.
         /18.*m34*RSp563e + 1./18.*m34*RSp561e + 1./36.*m33*RSp45613e
          - 1./36.*m33*RSp45633e - 1./12.*m33*RSp563e + 1./12.*m33*
         RSp561e );

      F13p6x23RV +=  + Ab(m34,mu2)*BbV(m33,me2,L) * ( 1./36.*m44*
         RSe45613p - 1./36.*m44*RSe45633p - 1./36.*m44*RSe563p + 1./36.
         *m44*RSe561p - 1./18.*m34*RSe45613p + 1./18.*m34*RSe45633p - 1.
         /18.*m34*RSe563p + 1./18.*m34*RSe561p + 1./36.*m33*RSe45613p
          - 1./36.*m33*RSe45633p - 1./12.*m33*RSe563p + 1./12.*m33*
         RSe561p );

      F13p6x23RV +=  + Ab(m34,mu2)*BbV(m33,L) * ( 7./144.*RD3456ep + 1./
         36.*m44*RS34561pe - 1./36.*m44*RC3456pe - 1./36.*m44*RS356pe
          - 1./18.*m34*RS34561pe + 1./18.*m34*RC3456pe - 1./18.*m34*
         RS356pe + 1./36.*m33*RS34561pe - 1./36.*m33*RC3456pe - 1./12.*
         m33*RS356pe );

      F13p6x23RV +=  + Ab(m34,mu2)*BbV(m34,L) * ( 1./24.*m44 - 1./12.*
         m34 + 1./24.*m33 );

      F13p6x23RV +=  + Ab(m34,mu2)*BbV(m35,L) * ( 1./48.*m55 + 1./48.*
         m44 - 1./24.*m35 - 1./24.*m34 + 1./24.*m33 );

      F13p6x23RV +=  + Ab(m34,mu2)*BbV(m36,L) * ( 1./48.*m66 + 1./48.*
         m44 - 1./24.*m36 - 1./24.*m34 + 1./24.*m33 );

      F13p6x23RV +=  + Ab(m34,mu2)*CbV(m33,L) * (  - 1./36.*m44*
         RD3456ep + 1./18.*m34*RD3456ep - 1./36.*m33*RD3456ep );

      F13p6x23RV +=  + Ab(m35,mu2)*AbV(m11,L) * ( 1./36.*RS14563pe - 1./
         48.*RC1456pe + 1./72.*RS146pe );

      F13p6x23RV +=  + Ab(m35,mu2)*AbV(m14,L) * ( 1./16. );

      F13p6x23RV +=  + Ab(m35,mu2)*AbV(m15,L) * ( 1./16. + 7./216.*
         RS34561pe + 7./216.*RS14563pe + 7./216.*RSe45613p + 7./216.*
         RSp45613e - 1./216.*RC3456pe - 1./216.*RSe45633p - 1./216.*
         RSp45633e - 5./216.*RS346pe - 5./216.*RSe463p - 5./216.*
         RSp463e - 1./216.*RC1456pe - 1./216.*RSe45611p - 1./216.*
         RSp45611e - 5./216.*RS146pe + 5./216.*RS546pe - 5./216.*
         RSe461p - 5./216.*RSp461e + 5./216.*RSe465p + 5./216.*RSp465e
          );

      F13p6x23RV +=  + Ab(m35,mu2)*AbV(m16,L) * ( 1./16. );

      F13p6x23RV +=  + Ab(m35,mu2)*AbV(m33,L) * (  - 1./36.*RS34561pe
          - 1./18.*RS14563pe + 7./144.*RC3456pe - 5./72.*RS346pe + 1./
         18.*RS146pe );

      F13p6x23RV +=  + Ab(m35,mu2)*AbV(m34,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m35,mu2)*AbV(m35,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m35,mu2)*AbV(m36,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m35,mu2)*AbV(mp2,L) * (  - 1./36.*RSp45613e
          + 7./144.*RSp45633e - 5./72.*RSp463e - 1./48.*RSp45611e + 5./
         72.*RSp461e );

      F13p6x23RV +=  + Ab(m35,mu2)*AbV(me2,L) * (  - 1./36.*RSe45613p
          + 7./144.*RSe45633p - 5./72.*RSe463p - 1./48.*RSe45611p + 5./
         72.*RSe461p );

      F13p6x23RV +=  + Ab(m35,mu2)*BbV(m11,m33,L) * ( 1./36.*m55*
         RS14563pe + 1./36.*m55*RS146pe - 1./18.*m35*RS14563pe + 1./18.
         *m35*RS146pe + 1./12.*m33*RS14563pe + 1./36.*m33*RS146pe - 1./
         18.*m11*RS14563pe + 1./18.*m11*RS146pe );

      F13p6x23RV +=  + Ab(m35,mu2)*BbV(m11,L) * (  - 1./48.*RD1456ep );

      F13p6x23RV +=  + Ab(m35,mu2)*BbV(m33,mp2,L) * ( 1./36.*m55*
         RSp45613e - 1./36.*m55*RSp45633e - 1./36.*m55*RSp463e + 1./36.
         *m55*RSp461e - 1./18.*m35*RSp45613e + 1./18.*m35*RSp45633e - 1.
         /18.*m35*RSp463e + 1./18.*m35*RSp461e + 1./36.*m33*RSp45613e
          - 1./36.*m33*RSp45633e - 1./12.*m33*RSp463e + 1./12.*m33*
         RSp461e );

      F13p6x23RV +=  + Ab(m35,mu2)*BbV(m33,me2,L) * ( 1./36.*m55*
         RSe45613p - 1./36.*m55*RSe45633p - 1./36.*m55*RSe463p + 1./36.
         *m55*RSe461p - 1./18.*m35*RSe45613p + 1./18.*m35*RSe45633p - 1.
         /18.*m35*RSe463p + 1./18.*m35*RSe461p + 1./36.*m33*RSe45613p
          - 1./36.*m33*RSe45633p - 1./12.*m33*RSe463p + 1./12.*m33*
         RSe461p );

      F13p6x23RV +=  + Ab(m35,mu2)*BbV(m33,L) * ( 7./144.*RD3456ep + 1./
         36.*m55*RS34561pe - 1./36.*m55*RC3456pe - 1./36.*m55*RS346pe
          - 1./18.*m35*RS34561pe + 1./18.*m35*RC3456pe - 1./18.*m35*
         RS346pe + 1./36.*m33*RS34561pe - 1./36.*m33*RC3456pe - 1./12.*
         m33*RS346pe );

      F13p6x23RV +=  + Ab(m35,mu2)*BbV(m34,L) * ( 1./48.*m55 + 1./48.*
         m44 - 1./24.*m35 - 1./24.*m34 + 1./24.*m33 );

      F13p6x23RV +=  + Ab(m35,mu2)*BbV(m35,L) * ( 1./24.*m55 - 1./12.*
         m35 + 1./24.*m33 );

      F13p6x23RV +=  + Ab(m35,mu2)*BbV(m36,L) * ( 1./48.*m66 + 1./48.*
         m55 - 1./24.*m36 - 1./24.*m35 + 1./24.*m33 );

      F13p6x23RV +=  + Ab(m35,mu2)*CbV(m33,L) * (  - 1./36.*m55*
         RD3456ep + 1./18.*m35*RD3456ep - 1./36.*m33*RD3456ep );

      F13p6x23RV +=  + Ab(m36,mu2)*AbV(m11,L) * ( 1./36.*RS14563pe - 1./
         48.*RC1456pe + 1./72.*RS145pe );

      F13p6x23RV +=  + Ab(m36,mu2)*AbV(m14,L) * ( 1./16. );

      F13p6x23RV +=  + Ab(m36,mu2)*AbV(m15,L) * ( 1./16. );

      F13p6x23RV +=  + Ab(m36,mu2)*AbV(m16,L) * ( 1./16. + 7./216.*
         RS34561pe + 7./216.*RS14563pe + 7./216.*RSe45613p + 7./216.*
         RSp45613e - 1./216.*RC3456pe - 1./216.*RSe45633p - 1./216.*
         RSp45633e - 5./216.*RS345pe - 5./216.*RSe453p - 5./216.*
         RSp453e - 1./216.*RC1456pe - 1./216.*RSe45611p - 1./216.*
         RSp45611e - 5./216.*RS145pe + 5./216.*RS645pe - 5./216.*
         RSe451p - 5./216.*RSp451e + 5./216.*RSe456p + 5./216.*RSp456e
          );

      F13p6x23RV +=  + Ab(m36,mu2)*AbV(m33,L) * (  - 1./36.*RS34561pe
          - 1./18.*RS14563pe + 7./144.*RC3456pe - 5./72.*RS345pe + 1./
         18.*RS145pe );

      F13p6x23RV +=  + Ab(m36,mu2)*AbV(m34,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m36,mu2)*AbV(m35,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m36,mu2)*AbV(m36,L) * (  - 1./16. );

      F13p6x23RV +=  + Ab(m36,mu2)*AbV(mp2,L) * (  - 1./36.*RSp45613e
          + 7./144.*RSp45633e - 5./72.*RSp453e - 1./48.*RSp45611e + 5./
         72.*RSp451e );

      F13p6x23RV +=  + Ab(m36,mu2)*AbV(me2,L) * (  - 1./36.*RSe45613p
          + 7./144.*RSe45633p - 5./72.*RSe453p - 1./48.*RSe45611p + 5./
         72.*RSe451p );

      F13p6x23RV +=  + Ab(m36,mu2)*BbV(m11,m33,L) * ( 1./36.*m66*
         RS14563pe + 1./36.*m66*RS145pe - 1./18.*m36*RS14563pe + 1./18.
         *m36*RS145pe + 1./12.*m33*RS14563pe + 1./36.*m33*RS145pe - 1./
         18.*m11*RS14563pe + 1./18.*m11*RS145pe );

      F13p6x23RV +=  + Ab(m36,mu2)*BbV(m11,L) * (  - 1./48.*RD1456ep );

      F13p6x23RV +=  + Ab(m36,mu2)*BbV(m33,mp2,L) * ( 1./36.*m66*
         RSp45613e - 1./36.*m66*RSp45633e - 1./36.*m66*RSp453e + 1./36.
         *m66*RSp451e - 1./18.*m36*RSp45613e + 1./18.*m36*RSp45633e - 1.
         /18.*m36*RSp453e + 1./18.*m36*RSp451e + 1./36.*m33*RSp45613e
          - 1./36.*m33*RSp45633e - 1./12.*m33*RSp453e + 1./12.*m33*
         RSp451e );

      F13p6x23RV +=  + Ab(m36,mu2)*BbV(m33,me2,L) * ( 1./36.*m66*
         RSe45613p - 1./36.*m66*RSe45633p - 1./36.*m66*RSe453p + 1./36.
         *m66*RSe451p - 1./18.*m36*RSe45613p + 1./18.*m36*RSe45633p - 1.
         /18.*m36*RSe453p + 1./18.*m36*RSe451p + 1./36.*m33*RSe45613p
          - 1./36.*m33*RSe45633p - 1./12.*m33*RSe453p + 1./12.*m33*
         RSe451p );

      F13p6x23RV +=  + Ab(m36,mu2)*BbV(m33,L) * ( 7./144.*RD3456ep + 1./
         36.*m66*RS34561pe - 1./36.*m66*RC3456pe - 1./36.*m66*RS345pe
          - 1./18.*m36*RS34561pe + 1./18.*m36*RC3456pe - 1./18.*m36*
         RS345pe + 1./36.*m33*RS34561pe - 1./36.*m33*RC3456pe - 1./12.*
         m33*RS345pe );

      F13p6x23RV +=  + Ab(m36,mu2)*BbV(m34,L) * ( 1./48.*m66 + 1./48.*
         m44 - 1./24.*m36 - 1./24.*m34 + 1./24.*m33 );

      F13p6x23RV +=  + Ab(m36,mu2)*BbV(m35,L) * ( 1./48.*m66 + 1./48.*
         m55 - 1./24.*m36 - 1./24.*m35 + 1./24.*m33 );

      F13p6x23RV +=  + Ab(m36,mu2)*BbV(m36,L) * ( 1./24.*m66 - 1./12.*
         m36 + 1./24.*m33 );

      F13p6x23RV +=  + Ab(m36,mu2)*CbV(m33,L) * (  - 1./36.*m66*
         RD3456ep + 1./18.*m36*RD3456ep - 1./36.*m33*RD3456ep );

      F13p6x23RV +=  + Ab(m44,mu2)*BbV(m11,m33,L) * (  - 1./18.*m44*
         RS156pe*RS356pe + 1./54.*m44*RS456pe*RS156pe*RS356pe );

      F13p6x23RV +=  + Ab(m44,mu2)*BbV(m11,mp2,L) * (  - 1./18.*m44*
         RS156pe*RSp563e + 1./54.*m44*RS456pe*RS156pe*RSp563e + 1./18.*
         m44*RSp561e*RS156pe - 1./54.*m44*RSp561e*RS456pe*RS156pe );

      F13p6x23RV +=  + Ab(m44,mu2)*BbV(m11,me2,L) * (  - 1./18.*m44*
         RS156pe*RSe563p + 1./54.*m44*RS456pe*RS156pe*RSe563p + 1./18.*
         m44*RSe561p*RS156pe - 1./54.*m44*RSe561p*RS456pe*RS156pe );

      F13p6x23RV +=  + Ab(m44,mu2)*BbV(m11,L) * ( 1./36.*m44*pow(
         RS156pe,2) - 1./108.*m44*RS456pe*pow(RS156pe,2) );

      F13p6x23RV +=  + Ab(m44,mu2)*BbV(m14,L) * ( 1./48.*m44 - 1./144.*
         m44*RS456pe - 1./24.*m14 + 1./72.*m14*RS456pe + 1./48.*m11 - 1.
         /144.*m11*RS456pe );

      F13p6x23RV +=  + Ab(m44,mu2)*BbV(m33,mp2,L) * ( 1./18.*m44*
         RSp563e*RS356pe - 1./54.*m44*RS456pe*RSp563e*RS356pe - 1./18.*
         m44*RSp561e*RS356pe + 1./54.*m44*RSp561e*RS456pe*RS356pe );

      F13p6x23RV +=  + Ab(m44,mu2)*BbV(m33,me2,L) * ( 1./18.*m44*
         RSe563p*RS356pe - 1./54.*m44*RS456pe*RSe563p*RS356pe - 1./18.*
         m44*RSe561p*RS356pe + 1./54.*m44*RSe561p*RS456pe*RS356pe );

      F13p6x23RV +=  + Ab(m44,mu2)*BbV(m33,L) * ( 1./36.*m44*pow(
         RS356pe,2) - 1./108.*m44*RS456pe*pow(RS356pe,2) );

      F13p6x23RV +=  + Ab(m44,mu2)*BbV(m34,L) * ( 1./48.*m44 - 1./144.*
         m44*RS456pe - 1./24.*m34 + 1./72.*m34*RS456pe + 1./48.*m33 - 1.
         /144.*m33*RS456pe );

      F13p6x23RV +=  + Ab(m44,mu2)*BbV(mp2,me2,L) * ( 2./27.*m44*
         RSp563e*RSe563p - 2./81.*m44*RS456pe*RSp563e*RSe563p - 2./27.*
         m44*RSe561p*RSp563e + 2./81.*m44*RSe561p*RS456pe*RSp563e - 2./
         27.*m44*RSp561e*RSe563p + 2./81.*m44*RSp561e*RS456pe*RSe563p
          + 2./27.*m44*RSp561e*RSe561p - 2./81.*m44*RSp561e*RSe561p*
         RS456pe );

      F13p6x23RV +=  + Ab(m44,mu2)*BbV(mp2,L) * ( 1./36.*m44*pow(
         RSp563e,2) - 1./108.*m44*RS456pe*pow(RSp563e,2) - 1./18.*m44*
         RSp561e*RSp563e + 1./54.*m44*RSp561e*RS456pe*RSp563e + 1./36.*
         m44*pow(RSp561e,2) - 1./108.*m44*pow(RSp561e,2)*RS456pe );

      F13p6x23RV +=  + Ab(m44,mu2)*BbV(me2,mp2,L) * (  - 1./54.*m44*
         RSp563e*RSe563p + 1./162.*m44*RS456pe*RSp563e*RSe563p + 1./54.
         *m44*RSe561p*RSp563e - 1./162.*m44*RSe561p*RS456pe*RSp563e + 1.
         /54.*m44*RSp561e*RSe563p - 1./162.*m44*RSp561e*RS456pe*RSe563p
          - 1./54.*m44*RSp561e*RSe561p + 1./162.*m44*RSp561e*RSe561p*
         RS456pe );

      F13p6x23RV +=  + Ab(m44,mu2)*BbV(me2,L) * ( 1./36.*m44*pow(
         RSe563p,2) - 1./108.*m44*RS456pe*pow(RSe563p,2) - 1./18.*m44*
         RSe561p*RSe563p + 1./54.*m44*RSe561p*RS456pe*RSe563p + 1./36.*
         m44*pow(RSe561p,2) - 1./108.*m44*pow(RSe561p,2)*RS456pe );

      F13p6x23RV +=  + Ab(m45,mu2)*AbV(m11,L) * (  - 1./108.*pow(
         RS146pe,2) + 1./54.*RS156pe*RS146pe - 1./108.*pow(RS156pe,2) )
         ;

      F13p6x23RV +=  + Ab(m45,mu2)*AbV(m33,L) * (  - 1./108.*pow(
         RS346pe,2) + 1./54.*RS356pe*RS346pe - 1./108.*pow(RS356pe,2)
          + 1./54.*RS146pe*RS346pe - 1./54.*RS146pe*RS356pe - 1./54.*
         RS156pe*RS346pe + 1./54.*RS156pe*RS356pe );

      F13p6x23RV +=  + Ab(m45,mu2)*AbV(mp2,L) * ( 1./54.*RSp563e*
         RS346pe - 1./54.*RSp563e*RS356pe + 1./162.*RSp563e*RSe563p - 1.
         /162.*RSp563e*RSe463p - 1./108.*pow(RSp563e,2) - 1./54.*
         RSp463e*RS346pe + 1./54.*RSp463e*RS356pe - 1./162.*RSp463e*
         RSe563p + 1./162.*RSp463e*RSe463p + 1./54.*RSp463e*RSp563e - 1.
         /108.*pow(RSp463e,2) - 1./54.*RS146pe*RSp563e + 1./54.*RS146pe
         *RSp463e + 1./54.*RS156pe*RSp563e - 1./54.*RS156pe*RSp463e - 1.
         /162.*RSe561p*RSp563e + 1./162.*RSe561p*RSp463e + 1./162.*
         RSe461p*RSp563e - 1./162.*RSe461p*RSp463e - 1./54.*RSp561e*
         RS346pe + 1./54.*RSp561e*RS356pe - 1./162.*RSp561e*RSe563p + 1.
         /162.*RSp561e*RSe463p + 1./54.*RSp561e*RSp563e - 1./54.*
         RSp561e*RSp463e + 1./54.*RSp561e*RS146pe - 1./54.*RSp561e*
         RS156pe + 1./162.*RSp561e*RSe561p - 1./162.*RSp561e*RSe461p - 
         1./108.*pow(RSp561e,2) + 1./54.*RSp461e*RS346pe - 1./54.*
         RSp461e*RS356pe + 1./162.*RSp461e*RSe563p - 1./162.*RSp461e*
         RSe463p );

      F13p6x23RV +=  + Ab(m45,mu2)*AbV(mp2,L) * (  - 1./54.*RSp461e*
         RSp563e + 1./54.*RSp461e*RSp463e - 1./54.*RSp461e*RS146pe + 1./
         54.*RSp461e*RS156pe - 1./162.*RSp461e*RSe561p + 1./162.*
         RSp461e*RSe461p + 1./54.*RSp461e*RSp561e - 1./108.*pow(
         RSp461e,2) );

      F13p6x23RV +=  + Ab(m45,mu2)*AbV(me2,L) * ( 1./54.*RSe563p*
         RS346pe - 1./54.*RSe563p*RS356pe - 1./108.*pow(RSe563p,2) - 1./
         54.*RSe463p*RS346pe + 1./54.*RSe463p*RS356pe + 1./54.*RSe463p*
         RSe563p - 1./108.*pow(RSe463p,2) - 2./81.*RSp563e*RSe563p + 2./
         81.*RSp563e*RSe463p + 2./81.*RSp463e*RSe563p - 2./81.*RSp463e*
         RSe463p - 1./54.*RS146pe*RSe563p + 1./54.*RS146pe*RSe463p + 1./
         54.*RS156pe*RSe563p - 1./54.*RS156pe*RSe463p - 1./54.*RSe561p*
         RS346pe + 1./54.*RSe561p*RS356pe + 1./54.*RSe561p*RSe563p - 1./
         54.*RSe561p*RSe463p + 2./81.*RSe561p*RSp563e - 2./81.*RSe561p*
         RSp463e + 1./54.*RSe561p*RS146pe - 1./54.*RSe561p*RS156pe - 1./
         108.*pow(RSe561p,2) + 1./54.*RSe461p*RS346pe - 1./54.*RSe461p*
         RS356pe - 1./54.*RSe461p*RSe563p + 1./54.*RSe461p*RSe463p - 2./
         81.*RSe461p*RSp563e + 2./81.*RSe461p*RSp463e - 1./54.*RSe461p*
         RS146pe + 1./54.*RSe461p*RS156pe + 1./54.*RSe461p*RSe561p - 1./
         108.*pow(RSe461p,2) + 2./81.*RSp561e*RSe563p - 2./81.*RSp561e*
         RSe463p );

      F13p6x23RV +=  + Ab(m45,mu2)*AbV(me2,L) * (  - 2./81.*RSp561e*
         RSe561p + 2./81.*RSp561e*RSe461p - 2./81.*RSp461e*RSe563p + 2./
         81.*RSp461e*RSe463p + 2./81.*RSp461e*RSe561p - 2./81.*RSp461e*
         RSe461p );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,m33,L) * (  - 1./36.*m55*
         RS146pe*RS346pe - 1./108.*m55*RS146pe*RS356pe - 1./108.*m55*
         RS156pe*RS346pe - 1./108.*m55*RS156pe*RS356pe + 1./54.*m45*
         RS146pe*RS346pe - 1./54.*m45*RS146pe*RS356pe - 1./54.*m45*
         RS156pe*RS346pe + 1./54.*m45*RS156pe*RS356pe - 1./108.*m44*
         RS146pe*RS346pe - 1./108.*m44*RS146pe*RS356pe - 1./108.*m44*
         RS156pe*RS346pe - 1./36.*m44*RS156pe*RS356pe + 1./54.*m11*
         RS146pe*RS346pe - 1./54.*m11*RS146pe*RS356pe - 1./54.*m11*
         RS156pe*RS346pe + 1./54.*m11*RS156pe*RS356pe );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,mp2,L) * (  - 1./108.*m55*
         RS146pe*RSp563e - 1./36.*m55*RS146pe*RSp463e - 1./108.*m55*
         RS156pe*RSp563e - 1./108.*m55*RS156pe*RSp463e + 1./108.*m55*
         RSp561e*RS146pe + 1./108.*m55*RSp561e*RS156pe + 1./36.*m55*
         RSp461e*RS146pe + 1./108.*m55*RSp461e*RS156pe - 1./54.*m45*
         RS146pe*RSp563e + 1./54.*m45*RS146pe*RSp463e + 1./54.*m45*
         RS156pe*RSp563e - 1./54.*m45*RS156pe*RSp463e + 1./54.*m45*
         RSp561e*RS146pe - 1./54.*m45*RSp561e*RS156pe - 1./54.*m45*
         RSp461e*RS146pe + 1./54.*m45*RSp461e*RS156pe - 1./108.*m44*
         RS146pe*RSp563e - 1./108.*m44*RS146pe*RSp463e - 1./36.*m44*
         RS156pe*RSp563e - 1./108.*m44*RS156pe*RSp463e + 1./108.*m44*
         RSp561e*RS146pe + 1./36.*m44*RSp561e*RS156pe + 1./108.*m44*
         RSp461e*RS146pe + 1./108.*m44*RSp461e*RS156pe - 1./54.*m11*
         RS146pe*RSp563e + 1./54.*m11*RS146pe*RSp463e + 1./54.*m11*
         RS156pe*RSp563e - 1./54.*m11*RS156pe*RSp463e + 1./54.*m11*
         RSp561e*RS146pe );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,mp2,L) * (  - 1./54.*m11*
         RSp561e*RS156pe - 1./54.*m11*RSp461e*RS146pe + 1./54.*m11*
         RSp461e*RS156pe );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,me2,L) * (  - 1./108.*m55*
         RS146pe*RSe563p - 1./36.*m55*RS146pe*RSe463p - 1./108.*m55*
         RS156pe*RSe563p - 1./108.*m55*RS156pe*RSe463p + 1./108.*m55*
         RSe561p*RS146pe + 1./108.*m55*RSe561p*RS156pe + 1./36.*m55*
         RSe461p*RS146pe + 1./108.*m55*RSe461p*RS156pe - 1./54.*m45*
         RS146pe*RSe563p + 1./54.*m45*RS146pe*RSe463p + 1./54.*m45*
         RS156pe*RSe563p - 1./54.*m45*RS156pe*RSe463p + 1./54.*m45*
         RSe561p*RS146pe - 1./54.*m45*RSe561p*RS156pe - 1./54.*m45*
         RSe461p*RS146pe + 1./54.*m45*RSe461p*RS156pe - 1./108.*m44*
         RS146pe*RSe563p - 1./108.*m44*RS146pe*RSe463p - 1./36.*m44*
         RS156pe*RSe563p - 1./108.*m44*RS156pe*RSe463p + 1./108.*m44*
         RSe561p*RS146pe + 1./36.*m44*RSe561p*RS156pe + 1./108.*m44*
         RSe461p*RS146pe + 1./108.*m44*RSe461p*RS156pe - 1./54.*m11*
         RS146pe*RSe563p + 1./54.*m11*RS146pe*RSe463p + 1./54.*m11*
         RS156pe*RSe563p - 1./54.*m11*RS156pe*RSe463p + 1./54.*m11*
         RSe561p*RS146pe );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,me2,L) * (  - 1./54.*m11*
         RSe561p*RS156pe - 1./54.*m11*RSe461p*RS146pe + 1./54.*m11*
         RSe461p*RS156pe );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m11,L) * ( 1./72.*m55*pow(
         RS146pe,2) + 1./108.*m55*RS156pe*RS146pe + 1./216.*m55*pow(
         RS156pe,2) - 1./108.*m45*pow(RS146pe,2) + 1./54.*m45*RS156pe*
         RS146pe - 1./108.*m45*pow(RS156pe,2) + 1./216.*m44*pow(
         RS146pe,2) + 1./108.*m44*RS156pe*RS146pe + 1./72.*m44*pow(
         RS156pe,2) - 1./108.*m11*pow(RS146pe,2) + 1./54.*m11*RS156pe*
         RS146pe - 1./108.*m11*pow(RS156pe,2) );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m14,L) * ( 1./48.*m55 - 1./24.*
         m45 + 1./24.*m44 - 1./24.*m14 + 1./48.*m11 );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m15,L) * ( 1./24.*m55 - 1./24.*
         m45 + 1./48.*m44 - 1./24.*m15 + 1./48.*m11 );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,mp2,L) * ( 1./108.*m55*
         RSp563e*RS346pe + 1./108.*m55*RSp563e*RS356pe + 1./36.*m55*
         RSp463e*RS346pe + 1./108.*m55*RSp463e*RS356pe - 1./108.*m55*
         RSp561e*RS346pe - 1./108.*m55*RSp561e*RS356pe - 1./36.*m55*
         RSp461e*RS346pe - 1./108.*m55*RSp461e*RS356pe + 1./54.*m45*
         RSp563e*RS346pe - 1./54.*m45*RSp563e*RS356pe - 1./54.*m45*
         RSp463e*RS346pe + 1./54.*m45*RSp463e*RS356pe - 1./54.*m45*
         RSp561e*RS346pe + 1./54.*m45*RSp561e*RS356pe + 1./54.*m45*
         RSp461e*RS346pe - 1./54.*m45*RSp461e*RS356pe + 1./108.*m44*
         RSp563e*RS346pe + 1./36.*m44*RSp563e*RS356pe + 1./108.*m44*
         RSp463e*RS346pe + 1./108.*m44*RSp463e*RS356pe - 1./108.*m44*
         RSp561e*RS346pe - 1./36.*m44*RSp561e*RS356pe - 1./108.*m44*
         RSp461e*RS346pe - 1./108.*m44*RSp461e*RS356pe + 1./54.*m33*
         RSp563e*RS346pe - 1./54.*m33*RSp563e*RS356pe - 1./54.*m33*
         RSp463e*RS346pe + 1./54.*m33*RSp463e*RS356pe - 1./54.*m33*
         RSp561e*RS346pe );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,mp2,L) * ( 1./54.*m33*
         RSp561e*RS356pe + 1./54.*m33*RSp461e*RS346pe - 1./54.*m33*
         RSp461e*RS356pe );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,me2,L) * ( 1./108.*m55*
         RSe563p*RS346pe + 1./108.*m55*RSe563p*RS356pe + 1./36.*m55*
         RSe463p*RS346pe + 1./108.*m55*RSe463p*RS356pe - 1./108.*m55*
         RSe561p*RS346pe - 1./108.*m55*RSe561p*RS356pe - 1./36.*m55*
         RSe461p*RS346pe - 1./108.*m55*RSe461p*RS356pe + 1./54.*m45*
         RSe563p*RS346pe - 1./54.*m45*RSe563p*RS356pe - 1./54.*m45*
         RSe463p*RS346pe + 1./54.*m45*RSe463p*RS356pe - 1./54.*m45*
         RSe561p*RS346pe + 1./54.*m45*RSe561p*RS356pe + 1./54.*m45*
         RSe461p*RS346pe - 1./54.*m45*RSe461p*RS356pe + 1./108.*m44*
         RSe563p*RS346pe + 1./36.*m44*RSe563p*RS356pe + 1./108.*m44*
         RSe463p*RS346pe + 1./108.*m44*RSe463p*RS356pe - 1./108.*m44*
         RSe561p*RS346pe - 1./36.*m44*RSe561p*RS356pe - 1./108.*m44*
         RSe461p*RS346pe - 1./108.*m44*RSe461p*RS356pe + 1./54.*m33*
         RSe563p*RS346pe - 1./54.*m33*RSe563p*RS356pe - 1./54.*m33*
         RSe463p*RS346pe + 1./54.*m33*RSe463p*RS356pe - 1./54.*m33*
         RSe561p*RS346pe );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,me2,L) * ( 1./54.*m33*
         RSe561p*RS356pe + 1./54.*m33*RSe461p*RS346pe - 1./54.*m33*
         RSe461p*RS356pe );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m33,L) * ( 1./72.*m55*pow(
         RS346pe,2) + 1./108.*m55*RS356pe*RS346pe + 1./216.*m55*pow(
         RS356pe,2) - 1./108.*m45*pow(RS346pe,2) + 1./54.*m45*RS356pe*
         RS346pe - 1./108.*m45*pow(RS356pe,2) + 1./216.*m44*pow(
         RS346pe,2) + 1./108.*m44*RS356pe*RS346pe + 1./72.*m44*pow(
         RS356pe,2) - 1./108.*m33*pow(RS346pe,2) + 1./54.*m33*RS356pe*
         RS346pe - 1./108.*m33*pow(RS356pe,2) );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m34,L) * ( 1./48.*m55 - 1./24.*
         m45 + 1./24.*m44 - 1./24.*m34 + 1./48.*m33 );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(m35,L) * ( 1./24.*m55 - 1./24.*
         m45 + 1./48.*m44 - 1./24.*m35 + 1./48.*m33 );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(mp2,me2,L) * (  - 2./81.*mp2*
         RSp563e*RSe563p + 2./81.*mp2*RSp563e*RSe463p + 2./81.*mp2*
         RSp463e*RSe563p - 2./81.*mp2*RSp463e*RSe463p + 2./81.*mp2*
         RSe561p*RSp563e - 2./81.*mp2*RSe561p*RSp463e - 2./81.*mp2*
         RSe461p*RSp563e + 2./81.*mp2*RSe461p*RSp463e + 2./81.*mp2*
         RSp561e*RSe563p - 2./81.*mp2*RSp561e*RSe463p - 2./81.*mp2*
         RSp561e*RSe561p + 2./81.*mp2*RSp561e*RSe461p - 2./81.*mp2*
         RSp461e*RSe563p + 2./81.*mp2*RSp461e*RSe463p + 2./81.*mp2*
         RSp461e*RSe561p - 2./81.*mp2*RSp461e*RSe461p + 1./81.*m55*
         RSp563e*RSe563p + 1./81.*m55*RSp563e*RSe463p + 1./81.*m55*
         RSp463e*RSe563p + 1./27.*m55*RSp463e*RSe463p - 1./81.*m55*
         RSe561p*RSp563e - 1./81.*m55*RSe561p*RSp463e - 1./81.*m55*
         RSe461p*RSp563e - 1./27.*m55*RSe461p*RSp463e - 1./81.*m55*
         RSp561e*RSe563p - 1./81.*m55*RSp561e*RSe463p + 1./81.*m55*
         RSp561e*RSe561p + 1./81.*m55*RSp561e*RSe461p - 1./81.*m55*
         RSp461e*RSe563p );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(mp2,me2,L) * (  - 1./27.*m55*
         RSp461e*RSe463p + 1./81.*m55*RSp461e*RSe561p + 1./27.*m55*
         RSp461e*RSe461p - 2./81.*m45*RSp563e*RSe563p + 2./81.*m45*
         RSp563e*RSe463p + 2./81.*m45*RSp463e*RSe563p - 2./81.*m45*
         RSp463e*RSe463p + 2./81.*m45*RSe561p*RSp563e - 2./81.*m45*
         RSe561p*RSp463e - 2./81.*m45*RSe461p*RSp563e + 2./81.*m45*
         RSe461p*RSp463e + 2./81.*m45*RSp561e*RSe563p - 2./81.*m45*
         RSp561e*RSe463p - 2./81.*m45*RSp561e*RSe561p + 2./81.*m45*
         RSp561e*RSe461p - 2./81.*m45*RSp461e*RSe563p + 2./81.*m45*
         RSp461e*RSe463p + 2./81.*m45*RSp461e*RSe561p - 2./81.*m45*
         RSp461e*RSe461p + 1./27.*m44*RSp563e*RSe563p + 1./81.*m44*
         RSp563e*RSe463p + 1./81.*m44*RSp463e*RSe563p + 1./81.*m44*
         RSp463e*RSe463p - 1./27.*m44*RSe561p*RSp563e - 1./81.*m44*
         RSe561p*RSp463e - 1./81.*m44*RSe461p*RSp563e - 1./81.*m44*
         RSe461p*RSp463e - 1./27.*m44*RSp561e*RSe563p - 1./81.*m44*
         RSp561e*RSe463p );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(mp2,me2,L) * ( 1./27.*m44*
         RSp561e*RSe561p + 1./81.*m44*RSp561e*RSe461p - 1./81.*m44*
         RSp461e*RSe563p - 1./81.*m44*RSp461e*RSe463p + 1./81.*m44*
         RSp461e*RSe561p + 1./81.*m44*RSp461e*RSe461p );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(mp2,L) * (  - 1./108.*mp2*pow(
         RSp563e,2) + 1./54.*mp2*RSp463e*RSp563e - 1./108.*mp2*pow(
         RSp463e,2) + 1./54.*mp2*RSp561e*RSp563e - 1./54.*mp2*RSp561e*
         RSp463e - 1./108.*mp2*pow(RSp561e,2) - 1./54.*mp2*RSp461e*
         RSp563e + 1./54.*mp2*RSp461e*RSp463e + 1./54.*mp2*RSp461e*
         RSp561e - 1./108.*mp2*pow(RSp461e,2) + 1./216.*m55*pow(
         RSp563e,2) + 1./108.*m55*RSp463e*RSp563e + 1./72.*m55*pow(
         RSp463e,2) - 1./108.*m55*RSp561e*RSp563e - 1./108.*m55*RSp561e
         *RSp463e + 1./216.*m55*pow(RSp561e,2) - 1./108.*m55*RSp461e*
         RSp563e - 1./36.*m55*RSp461e*RSp463e + 1./108.*m55*RSp461e*
         RSp561e + 1./72.*m55*pow(RSp461e,2) - 1./108.*m45*pow(
         RSp563e,2) + 1./54.*m45*RSp463e*RSp563e - 1./108.*m45*pow(
         RSp463e,2) + 1./54.*m45*RSp561e*RSp563e - 1./54.*m45*RSp561e*
         RSp463e - 1./108.*m45*pow(RSp561e,2) - 1./54.*m45*RSp461e*
         RSp563e + 1./54.*m45*RSp461e*RSp463e + 1./54.*m45*RSp461e*
         RSp561e );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(mp2,L) * (  - 1./108.*m45*pow(
         RSp461e,2) + 1./72.*m44*pow(RSp563e,2) + 1./108.*m44*RSp463e*
         RSp563e + 1./216.*m44*pow(RSp463e,2) - 1./36.*m44*RSp561e*
         RSp563e - 1./108.*m44*RSp561e*RSp463e + 1./72.*m44*pow(
         RSp561e,2) - 1./108.*m44*RSp461e*RSp563e - 1./108.*m44*RSp461e
         *RSp463e + 1./108.*m44*RSp461e*RSp561e + 1./216.*m44*pow(
         RSp461e,2) );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * ( 1./162.*me2*
         RSp563e*RSe563p - 1./162.*me2*RSp563e*RSe463p - 1./162.*me2*
         RSp463e*RSe563p + 1./162.*me2*RSp463e*RSe463p - 1./162.*me2*
         RSe561p*RSp563e + 1./162.*me2*RSe561p*RSp463e + 1./162.*me2*
         RSe461p*RSp563e - 1./162.*me2*RSe461p*RSp463e - 1./162.*me2*
         RSp561e*RSe563p + 1./162.*me2*RSp561e*RSe463p + 1./162.*me2*
         RSp561e*RSe561p - 1./162.*me2*RSp561e*RSe461p + 1./162.*me2*
         RSp461e*RSe563p - 1./162.*me2*RSp461e*RSe463p - 1./162.*me2*
         RSp461e*RSe561p + 1./162.*me2*RSp461e*RSe461p - 1./324.*m55*
         RSp563e*RSe563p - 1./324.*m55*RSp563e*RSe463p - 1./324.*m55*
         RSp463e*RSe563p - 1./108.*m55*RSp463e*RSe463p + 1./324.*m55*
         RSe561p*RSp563e + 1./324.*m55*RSe561p*RSp463e + 1./324.*m55*
         RSe461p*RSp563e + 1./108.*m55*RSe461p*RSp463e + 1./324.*m55*
         RSp561e*RSe563p + 1./324.*m55*RSp561e*RSe463p - 1./324.*m55*
         RSp561e*RSe561p - 1./324.*m55*RSp561e*RSe461p + 1./324.*m55*
         RSp461e*RSe563p );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * ( 1./108.*m55*
         RSp461e*RSe463p - 1./324.*m55*RSp461e*RSe561p - 1./108.*m55*
         RSp461e*RSe461p + 1./162.*m45*RSp563e*RSe563p - 1./162.*m45*
         RSp563e*RSe463p - 1./162.*m45*RSp463e*RSe563p + 1./162.*m45*
         RSp463e*RSe463p - 1./162.*m45*RSe561p*RSp563e + 1./162.*m45*
         RSe561p*RSp463e + 1./162.*m45*RSe461p*RSp563e - 1./162.*m45*
         RSe461p*RSp463e - 1./162.*m45*RSp561e*RSe563p + 1./162.*m45*
         RSp561e*RSe463p + 1./162.*m45*RSp561e*RSe561p - 1./162.*m45*
         RSp561e*RSe461p + 1./162.*m45*RSp461e*RSe563p - 1./162.*m45*
         RSp461e*RSe463p - 1./162.*m45*RSp461e*RSe561p + 1./162.*m45*
         RSp461e*RSe461p - 1./108.*m44*RSp563e*RSe563p - 1./324.*m44*
         RSp563e*RSe463p - 1./324.*m44*RSp463e*RSe563p - 1./324.*m44*
         RSp463e*RSe463p + 1./108.*m44*RSe561p*RSp563e + 1./324.*m44*
         RSe561p*RSp463e + 1./324.*m44*RSe461p*RSp563e + 1./324.*m44*
         RSe461p*RSp463e + 1./108.*m44*RSp561e*RSe563p + 1./324.*m44*
         RSp561e*RSe463p );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,mp2,L) * (  - 1./108.*m44*
         RSp561e*RSe561p - 1./324.*m44*RSp561e*RSe461p + 1./324.*m44*
         RSp461e*RSe563p + 1./324.*m44*RSp461e*RSe463p - 1./324.*m44*
         RSp461e*RSe561p - 1./324.*m44*RSp461e*RSe461p );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,L) * (  - 1./108.*me2*pow(
         RSe563p,2) + 1./54.*me2*RSe463p*RSe563p - 1./108.*me2*pow(
         RSe463p,2) + 1./54.*me2*RSe561p*RSe563p - 1./54.*me2*RSe561p*
         RSe463p - 1./108.*me2*pow(RSe561p,2) - 1./54.*me2*RSe461p*
         RSe563p + 1./54.*me2*RSe461p*RSe463p + 1./54.*me2*RSe461p*
         RSe561p - 1./108.*me2*pow(RSe461p,2) + 1./216.*m55*pow(
         RSe563p,2) + 1./108.*m55*RSe463p*RSe563p + 1./72.*m55*pow(
         RSe463p,2) - 1./108.*m55*RSe561p*RSe563p - 1./108.*m55*RSe561p
         *RSe463p + 1./216.*m55*pow(RSe561p,2) - 1./108.*m55*RSe461p*
         RSe563p - 1./36.*m55*RSe461p*RSe463p + 1./108.*m55*RSe461p*
         RSe561p + 1./72.*m55*pow(RSe461p,2) - 1./108.*m45*pow(
         RSe563p,2) + 1./54.*m45*RSe463p*RSe563p - 1./108.*m45*pow(
         RSe463p,2) + 1./54.*m45*RSe561p*RSe563p - 1./54.*m45*RSe561p*
         RSe463p - 1./108.*m45*pow(RSe561p,2) - 1./54.*m45*RSe461p*
         RSe563p + 1./54.*m45*RSe461p*RSe463p + 1./54.*m45*RSe461p*
         RSe561p );

      F13p6x23RV +=  + Ab(m45,mu2)*BbV(me2,L) * (  - 1./108.*m45*pow(
         RSe461p,2) + 1./72.*m44*pow(RSe563p,2) + 1./108.*m44*RSe463p*
         RSe563p + 1./216.*m44*pow(RSe463p,2) - 1./36.*m44*RSe561p*
         RSe563p - 1./108.*m44*RSe561p*RSe463p + 1./72.*m44*pow(
         RSe561p,2) - 1./108.*m44*RSe461p*RSe563p - 1./108.*m44*RSe461p
         *RSe463p + 1./108.*m44*RSe461p*RSe561p + 1./216.*m44*pow(
         RSe461p,2) );

      F13p6x23RV +=  + Ab(m46,mu2)*AbV(m11,L) * (  - 1./108.*pow(
         RS145pe,2) + 1./54.*RS156pe*RS145pe - 1./108.*pow(RS156pe,2) )
         ;

      F13p6x23RV +=  + Ab(m46,mu2)*AbV(m33,L) * (  - 1./108.*pow(
         RS345pe,2) + 1./54.*RS356pe*RS345pe - 1./108.*pow(RS356pe,2)
          + 1./54.*RS145pe*RS345pe - 1./54.*RS145pe*RS356pe - 1./54.*
         RS156pe*RS345pe + 1./54.*RS156pe*RS356pe );

      F13p6x23RV +=  + Ab(m46,mu2)*AbV(mp2,L) * ( 1./54.*RSp563e*
         RS345pe - 1./54.*RSp563e*RS356pe + 1./162.*RSp563e*RSe563p - 1.
         /162.*RSp563e*RSe453p - 1./108.*pow(RSp563e,2) - 1./54.*
         RSp453e*RS345pe + 1./54.*RSp453e*RS356pe - 1./162.*RSp453e*
         RSe563p + 1./162.*RSp453e*RSe453p + 1./54.*RSp453e*RSp563e - 1.
         /108.*pow(RSp453e,2) - 1./54.*RS145pe*RSp563e + 1./54.*RS145pe
         *RSp453e + 1./54.*RS156pe*RSp563e - 1./54.*RS156pe*RSp453e - 1.
         /162.*RSe561p*RSp563e + 1./162.*RSe561p*RSp453e + 1./162.*
         RSe451p*RSp563e - 1./162.*RSe451p*RSp453e - 1./54.*RSp561e*
         RS345pe + 1./54.*RSp561e*RS356pe - 1./162.*RSp561e*RSe563p + 1.
         /162.*RSp561e*RSe453p + 1./54.*RSp561e*RSp563e - 1./54.*
         RSp561e*RSp453e + 1./54.*RSp561e*RS145pe - 1./54.*RSp561e*
         RS156pe + 1./162.*RSp561e*RSe561p - 1./162.*RSp561e*RSe451p - 
         1./108.*pow(RSp561e,2) + 1./54.*RSp451e*RS345pe - 1./54.*
         RSp451e*RS356pe + 1./162.*RSp451e*RSe563p - 1./162.*RSp451e*
         RSe453p );

      F13p6x23RV +=  + Ab(m46,mu2)*AbV(mp2,L) * (  - 1./54.*RSp451e*
         RSp563e + 1./54.*RSp451e*RSp453e - 1./54.*RSp451e*RS145pe + 1./
         54.*RSp451e*RS156pe - 1./162.*RSp451e*RSe561p + 1./162.*
         RSp451e*RSe451p + 1./54.*RSp451e*RSp561e - 1./108.*pow(
         RSp451e,2) );

      F13p6x23RV +=  + Ab(m46,mu2)*AbV(me2,L) * ( 1./54.*RSe563p*
         RS345pe - 1./54.*RSe563p*RS356pe - 1./108.*pow(RSe563p,2) - 1./
         54.*RSe453p*RS345pe + 1./54.*RSe453p*RS356pe + 1./54.*RSe453p*
         RSe563p - 1./108.*pow(RSe453p,2) - 2./81.*RSp563e*RSe563p + 2./
         81.*RSp563e*RSe453p + 2./81.*RSp453e*RSe563p - 2./81.*RSp453e*
         RSe453p - 1./54.*RS145pe*RSe563p + 1./54.*RS145pe*RSe453p + 1./
         54.*RS156pe*RSe563p - 1./54.*RS156pe*RSe453p - 1./54.*RSe561p*
         RS345pe + 1./54.*RSe561p*RS356pe + 1./54.*RSe561p*RSe563p - 1./
         54.*RSe561p*RSe453p + 2./81.*RSe561p*RSp563e - 2./81.*RSe561p*
         RSp453e + 1./54.*RSe561p*RS145pe - 1./54.*RSe561p*RS156pe - 1./
         108.*pow(RSe561p,2) + 1./54.*RSe451p*RS345pe - 1./54.*RSe451p*
         RS356pe - 1./54.*RSe451p*RSe563p + 1./54.*RSe451p*RSe453p - 2./
         81.*RSe451p*RSp563e + 2./81.*RSe451p*RSp453e - 1./54.*RSe451p*
         RS145pe + 1./54.*RSe451p*RS156pe + 1./54.*RSe451p*RSe561p - 1./
         108.*pow(RSe451p,2) + 2./81.*RSp561e*RSe563p - 2./81.*RSp561e*
         RSe453p );

      F13p6x23RV +=  + Ab(m46,mu2)*AbV(me2,L) * (  - 2./81.*RSp561e*
         RSe561p + 2./81.*RSp561e*RSe451p - 2./81.*RSp451e*RSe563p + 2./
         81.*RSp451e*RSe453p + 2./81.*RSp451e*RSe561p - 2./81.*RSp451e*
         RSe451p );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,m33,L) * (  - 1./36.*m66*
         RS145pe*RS345pe - 1./108.*m66*RS145pe*RS356pe - 1./108.*m66*
         RS156pe*RS345pe - 1./108.*m66*RS156pe*RS356pe + 1./54.*m46*
         RS145pe*RS345pe - 1./54.*m46*RS145pe*RS356pe - 1./54.*m46*
         RS156pe*RS345pe + 1./54.*m46*RS156pe*RS356pe - 1./108.*m44*
         RS145pe*RS345pe - 1./108.*m44*RS145pe*RS356pe - 1./108.*m44*
         RS156pe*RS345pe - 1./36.*m44*RS156pe*RS356pe + 1./54.*m11*
         RS145pe*RS345pe - 1./54.*m11*RS145pe*RS356pe - 1./54.*m11*
         RS156pe*RS345pe + 1./54.*m11*RS156pe*RS356pe );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,mp2,L) * (  - 1./108.*m66*
         RS145pe*RSp563e - 1./36.*m66*RS145pe*RSp453e - 1./108.*m66*
         RS156pe*RSp563e - 1./108.*m66*RS156pe*RSp453e + 1./108.*m66*
         RSp561e*RS145pe + 1./108.*m66*RSp561e*RS156pe + 1./36.*m66*
         RSp451e*RS145pe + 1./108.*m66*RSp451e*RS156pe - 1./54.*m46*
         RS145pe*RSp563e + 1./54.*m46*RS145pe*RSp453e + 1./54.*m46*
         RS156pe*RSp563e - 1./54.*m46*RS156pe*RSp453e + 1./54.*m46*
         RSp561e*RS145pe - 1./54.*m46*RSp561e*RS156pe - 1./54.*m46*
         RSp451e*RS145pe + 1./54.*m46*RSp451e*RS156pe - 1./108.*m44*
         RS145pe*RSp563e - 1./108.*m44*RS145pe*RSp453e - 1./36.*m44*
         RS156pe*RSp563e - 1./108.*m44*RS156pe*RSp453e + 1./108.*m44*
         RSp561e*RS145pe + 1./36.*m44*RSp561e*RS156pe + 1./108.*m44*
         RSp451e*RS145pe + 1./108.*m44*RSp451e*RS156pe - 1./54.*m11*
         RS145pe*RSp563e + 1./54.*m11*RS145pe*RSp453e + 1./54.*m11*
         RS156pe*RSp563e - 1./54.*m11*RS156pe*RSp453e + 1./54.*m11*
         RSp561e*RS145pe );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,mp2,L) * (  - 1./54.*m11*
         RSp561e*RS156pe - 1./54.*m11*RSp451e*RS145pe + 1./54.*m11*
         RSp451e*RS156pe );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,me2,L) * (  - 1./108.*m66*
         RS145pe*RSe563p - 1./36.*m66*RS145pe*RSe453p - 1./108.*m66*
         RS156pe*RSe563p - 1./108.*m66*RS156pe*RSe453p + 1./108.*m66*
         RSe561p*RS145pe + 1./108.*m66*RSe561p*RS156pe + 1./36.*m66*
         RSe451p*RS145pe + 1./108.*m66*RSe451p*RS156pe - 1./54.*m46*
         RS145pe*RSe563p + 1./54.*m46*RS145pe*RSe453p + 1./54.*m46*
         RS156pe*RSe563p - 1./54.*m46*RS156pe*RSe453p + 1./54.*m46*
         RSe561p*RS145pe - 1./54.*m46*RSe561p*RS156pe - 1./54.*m46*
         RSe451p*RS145pe + 1./54.*m46*RSe451p*RS156pe - 1./108.*m44*
         RS145pe*RSe563p - 1./108.*m44*RS145pe*RSe453p - 1./36.*m44*
         RS156pe*RSe563p - 1./108.*m44*RS156pe*RSe453p + 1./108.*m44*
         RSe561p*RS145pe + 1./36.*m44*RSe561p*RS156pe + 1./108.*m44*
         RSe451p*RS145pe + 1./108.*m44*RSe451p*RS156pe - 1./54.*m11*
         RS145pe*RSe563p + 1./54.*m11*RS145pe*RSe453p + 1./54.*m11*
         RS156pe*RSe563p - 1./54.*m11*RS156pe*RSe453p + 1./54.*m11*
         RSe561p*RS145pe );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,me2,L) * (  - 1./54.*m11*
         RSe561p*RS156pe - 1./54.*m11*RSe451p*RS145pe + 1./54.*m11*
         RSe451p*RS156pe );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m11,L) * ( 1./72.*m66*pow(
         RS145pe,2) + 1./108.*m66*RS156pe*RS145pe + 1./216.*m66*pow(
         RS156pe,2) - 1./108.*m46*pow(RS145pe,2) + 1./54.*m46*RS156pe*
         RS145pe - 1./108.*m46*pow(RS156pe,2) + 1./216.*m44*pow(
         RS145pe,2) + 1./108.*m44*RS156pe*RS145pe + 1./72.*m44*pow(
         RS156pe,2) - 1./108.*m11*pow(RS145pe,2) + 1./54.*m11*RS156pe*
         RS145pe - 1./108.*m11*pow(RS156pe,2) );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m14,L) * ( 1./48.*m66 - 1./24.*
         m46 + 1./24.*m44 - 1./24.*m14 + 1./48.*m11 );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m16,L) * ( 1./24.*m66 - 1./24.*
         m46 + 1./48.*m44 - 1./24.*m16 + 1./48.*m11 );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,mp2,L) * ( 1./108.*m66*
         RSp563e*RS345pe + 1./108.*m66*RSp563e*RS356pe + 1./36.*m66*
         RSp453e*RS345pe + 1./108.*m66*RSp453e*RS356pe - 1./108.*m66*
         RSp561e*RS345pe - 1./108.*m66*RSp561e*RS356pe - 1./36.*m66*
         RSp451e*RS345pe - 1./108.*m66*RSp451e*RS356pe + 1./54.*m46*
         RSp563e*RS345pe - 1./54.*m46*RSp563e*RS356pe - 1./54.*m46*
         RSp453e*RS345pe + 1./54.*m46*RSp453e*RS356pe - 1./54.*m46*
         RSp561e*RS345pe + 1./54.*m46*RSp561e*RS356pe + 1./54.*m46*
         RSp451e*RS345pe - 1./54.*m46*RSp451e*RS356pe + 1./108.*m44*
         RSp563e*RS345pe + 1./36.*m44*RSp563e*RS356pe + 1./108.*m44*
         RSp453e*RS345pe + 1./108.*m44*RSp453e*RS356pe - 1./108.*m44*
         RSp561e*RS345pe - 1./36.*m44*RSp561e*RS356pe - 1./108.*m44*
         RSp451e*RS345pe - 1./108.*m44*RSp451e*RS356pe + 1./54.*m33*
         RSp563e*RS345pe - 1./54.*m33*RSp563e*RS356pe - 1./54.*m33*
         RSp453e*RS345pe + 1./54.*m33*RSp453e*RS356pe - 1./54.*m33*
         RSp561e*RS345pe );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,mp2,L) * ( 1./54.*m33*
         RSp561e*RS356pe + 1./54.*m33*RSp451e*RS345pe - 1./54.*m33*
         RSp451e*RS356pe );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,me2,L) * ( 1./108.*m66*
         RSe563p*RS345pe + 1./108.*m66*RSe563p*RS356pe + 1./36.*m66*
         RSe453p*RS345pe + 1./108.*m66*RSe453p*RS356pe - 1./108.*m66*
         RSe561p*RS345pe - 1./108.*m66*RSe561p*RS356pe - 1./36.*m66*
         RSe451p*RS345pe - 1./108.*m66*RSe451p*RS356pe + 1./54.*m46*
         RSe563p*RS345pe - 1./54.*m46*RSe563p*RS356pe - 1./54.*m46*
         RSe453p*RS345pe + 1./54.*m46*RSe453p*RS356pe - 1./54.*m46*
         RSe561p*RS345pe + 1./54.*m46*RSe561p*RS356pe + 1./54.*m46*
         RSe451p*RS345pe - 1./54.*m46*RSe451p*RS356pe + 1./108.*m44*
         RSe563p*RS345pe + 1./36.*m44*RSe563p*RS356pe + 1./108.*m44*
         RSe453p*RS345pe + 1./108.*m44*RSe453p*RS356pe - 1./108.*m44*
         RSe561p*RS345pe - 1./36.*m44*RSe561p*RS356pe - 1./108.*m44*
         RSe451p*RS345pe - 1./108.*m44*RSe451p*RS356pe + 1./54.*m33*
         RSe563p*RS345pe - 1./54.*m33*RSe563p*RS356pe - 1./54.*m33*
         RSe453p*RS345pe + 1./54.*m33*RSe453p*RS356pe - 1./54.*m33*
         RSe561p*RS345pe );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,me2,L) * ( 1./54.*m33*
         RSe561p*RS356pe + 1./54.*m33*RSe451p*RS345pe - 1./54.*m33*
         RSe451p*RS356pe );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m33,L) * ( 1./72.*m66*pow(
         RS345pe,2) + 1./108.*m66*RS356pe*RS345pe + 1./216.*m66*pow(
         RS356pe,2) - 1./108.*m46*pow(RS345pe,2) + 1./54.*m46*RS356pe*
         RS345pe - 1./108.*m46*pow(RS356pe,2) + 1./216.*m44*pow(
         RS345pe,2) + 1./108.*m44*RS356pe*RS345pe + 1./72.*m44*pow(
         RS356pe,2) - 1./108.*m33*pow(RS345pe,2) + 1./54.*m33*RS356pe*
         RS345pe - 1./108.*m33*pow(RS356pe,2) );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m34,L) * ( 1./48.*m66 - 1./24.*
         m46 + 1./24.*m44 - 1./24.*m34 + 1./48.*m33 );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(m36,L) * ( 1./24.*m66 - 1./24.*
         m46 + 1./48.*m44 - 1./24.*m36 + 1./48.*m33 );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(mp2,me2,L) * (  - 2./81.*mp2*
         RSp563e*RSe563p + 2./81.*mp2*RSp563e*RSe453p + 2./81.*mp2*
         RSp453e*RSe563p - 2./81.*mp2*RSp453e*RSe453p + 2./81.*mp2*
         RSe561p*RSp563e - 2./81.*mp2*RSe561p*RSp453e - 2./81.*mp2*
         RSe451p*RSp563e + 2./81.*mp2*RSe451p*RSp453e + 2./81.*mp2*
         RSp561e*RSe563p - 2./81.*mp2*RSp561e*RSe453p - 2./81.*mp2*
         RSp561e*RSe561p + 2./81.*mp2*RSp561e*RSe451p - 2./81.*mp2*
         RSp451e*RSe563p + 2./81.*mp2*RSp451e*RSe453p + 2./81.*mp2*
         RSp451e*RSe561p - 2./81.*mp2*RSp451e*RSe451p + 1./81.*m66*
         RSp563e*RSe563p + 1./81.*m66*RSp563e*RSe453p + 1./81.*m66*
         RSp453e*RSe563p + 1./27.*m66*RSp453e*RSe453p - 1./81.*m66*
         RSe561p*RSp563e - 1./81.*m66*RSe561p*RSp453e - 1./81.*m66*
         RSe451p*RSp563e - 1./27.*m66*RSe451p*RSp453e - 1./81.*m66*
         RSp561e*RSe563p - 1./81.*m66*RSp561e*RSe453p + 1./81.*m66*
         RSp561e*RSe561p + 1./81.*m66*RSp561e*RSe451p - 1./81.*m66*
         RSp451e*RSe563p );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(mp2,me2,L) * (  - 1./27.*m66*
         RSp451e*RSe453p + 1./81.*m66*RSp451e*RSe561p + 1./27.*m66*
         RSp451e*RSe451p - 2./81.*m46*RSp563e*RSe563p + 2./81.*m46*
         RSp563e*RSe453p + 2./81.*m46*RSp453e*RSe563p - 2./81.*m46*
         RSp453e*RSe453p + 2./81.*m46*RSe561p*RSp563e - 2./81.*m46*
         RSe561p*RSp453e - 2./81.*m46*RSe451p*RSp563e + 2./81.*m46*
         RSe451p*RSp453e + 2./81.*m46*RSp561e*RSe563p - 2./81.*m46*
         RSp561e*RSe453p - 2./81.*m46*RSp561e*RSe561p + 2./81.*m46*
         RSp561e*RSe451p - 2./81.*m46*RSp451e*RSe563p + 2./81.*m46*
         RSp451e*RSe453p + 2./81.*m46*RSp451e*RSe561p - 2./81.*m46*
         RSp451e*RSe451p + 1./27.*m44*RSp563e*RSe563p + 1./81.*m44*
         RSp563e*RSe453p + 1./81.*m44*RSp453e*RSe563p + 1./81.*m44*
         RSp453e*RSe453p - 1./27.*m44*RSe561p*RSp563e - 1./81.*m44*
         RSe561p*RSp453e - 1./81.*m44*RSe451p*RSp563e - 1./81.*m44*
         RSe451p*RSp453e - 1./27.*m44*RSp561e*RSe563p - 1./81.*m44*
         RSp561e*RSe453p );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(mp2,me2,L) * ( 1./27.*m44*
         RSp561e*RSe561p + 1./81.*m44*RSp561e*RSe451p - 1./81.*m44*
         RSp451e*RSe563p - 1./81.*m44*RSp451e*RSe453p + 1./81.*m44*
         RSp451e*RSe561p + 1./81.*m44*RSp451e*RSe451p );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(mp2,L) * (  - 1./108.*mp2*pow(
         RSp563e,2) + 1./54.*mp2*RSp453e*RSp563e - 1./108.*mp2*pow(
         RSp453e,2) + 1./54.*mp2*RSp561e*RSp563e - 1./54.*mp2*RSp561e*
         RSp453e - 1./108.*mp2*pow(RSp561e,2) - 1./54.*mp2*RSp451e*
         RSp563e + 1./54.*mp2*RSp451e*RSp453e + 1./54.*mp2*RSp451e*
         RSp561e - 1./108.*mp2*pow(RSp451e,2) + 1./216.*m66*pow(
         RSp563e,2) + 1./108.*m66*RSp453e*RSp563e + 1./72.*m66*pow(
         RSp453e,2) - 1./108.*m66*RSp561e*RSp563e - 1./108.*m66*RSp561e
         *RSp453e + 1./216.*m66*pow(RSp561e,2) - 1./108.*m66*RSp451e*
         RSp563e - 1./36.*m66*RSp451e*RSp453e + 1./108.*m66*RSp451e*
         RSp561e + 1./72.*m66*pow(RSp451e,2) - 1./108.*m46*pow(
         RSp563e,2) + 1./54.*m46*RSp453e*RSp563e - 1./108.*m46*pow(
         RSp453e,2) + 1./54.*m46*RSp561e*RSp563e - 1./54.*m46*RSp561e*
         RSp453e - 1./108.*m46*pow(RSp561e,2) - 1./54.*m46*RSp451e*
         RSp563e + 1./54.*m46*RSp451e*RSp453e + 1./54.*m46*RSp451e*
         RSp561e );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(mp2,L) * (  - 1./108.*m46*pow(
         RSp451e,2) + 1./72.*m44*pow(RSp563e,2) + 1./108.*m44*RSp453e*
         RSp563e + 1./216.*m44*pow(RSp453e,2) - 1./36.*m44*RSp561e*
         RSp563e - 1./108.*m44*RSp561e*RSp453e + 1./72.*m44*pow(
         RSp561e,2) - 1./108.*m44*RSp451e*RSp563e - 1./108.*m44*RSp451e
         *RSp453e + 1./108.*m44*RSp451e*RSp561e + 1./216.*m44*pow(
         RSp451e,2) );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * ( 1./162.*me2*
         RSp563e*RSe563p - 1./162.*me2*RSp563e*RSe453p - 1./162.*me2*
         RSp453e*RSe563p + 1./162.*me2*RSp453e*RSe453p - 1./162.*me2*
         RSe561p*RSp563e + 1./162.*me2*RSe561p*RSp453e + 1./162.*me2*
         RSe451p*RSp563e - 1./162.*me2*RSe451p*RSp453e - 1./162.*me2*
         RSp561e*RSe563p + 1./162.*me2*RSp561e*RSe453p + 1./162.*me2*
         RSp561e*RSe561p - 1./162.*me2*RSp561e*RSe451p + 1./162.*me2*
         RSp451e*RSe563p - 1./162.*me2*RSp451e*RSe453p - 1./162.*me2*
         RSp451e*RSe561p + 1./162.*me2*RSp451e*RSe451p - 1./324.*m66*
         RSp563e*RSe563p - 1./324.*m66*RSp563e*RSe453p - 1./324.*m66*
         RSp453e*RSe563p - 1./108.*m66*RSp453e*RSe453p + 1./324.*m66*
         RSe561p*RSp563e + 1./324.*m66*RSe561p*RSp453e + 1./324.*m66*
         RSe451p*RSp563e + 1./108.*m66*RSe451p*RSp453e + 1./324.*m66*
         RSp561e*RSe563p + 1./324.*m66*RSp561e*RSe453p - 1./324.*m66*
         RSp561e*RSe561p - 1./324.*m66*RSp561e*RSe451p + 1./324.*m66*
         RSp451e*RSe563p );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * ( 1./108.*m66*
         RSp451e*RSe453p - 1./324.*m66*RSp451e*RSe561p - 1./108.*m66*
         RSp451e*RSe451p + 1./162.*m46*RSp563e*RSe563p - 1./162.*m46*
         RSp563e*RSe453p - 1./162.*m46*RSp453e*RSe563p + 1./162.*m46*
         RSp453e*RSe453p - 1./162.*m46*RSe561p*RSp563e + 1./162.*m46*
         RSe561p*RSp453e + 1./162.*m46*RSe451p*RSp563e - 1./162.*m46*
         RSe451p*RSp453e - 1./162.*m46*RSp561e*RSe563p + 1./162.*m46*
         RSp561e*RSe453p + 1./162.*m46*RSp561e*RSe561p - 1./162.*m46*
         RSp561e*RSe451p + 1./162.*m46*RSp451e*RSe563p - 1./162.*m46*
         RSp451e*RSe453p - 1./162.*m46*RSp451e*RSe561p + 1./162.*m46*
         RSp451e*RSe451p - 1./108.*m44*RSp563e*RSe563p - 1./324.*m44*
         RSp563e*RSe453p - 1./324.*m44*RSp453e*RSe563p - 1./324.*m44*
         RSp453e*RSe453p + 1./108.*m44*RSe561p*RSp563e + 1./324.*m44*
         RSe561p*RSp453e + 1./324.*m44*RSe451p*RSp563e + 1./324.*m44*
         RSe451p*RSp453e + 1./108.*m44*RSp561e*RSe563p + 1./324.*m44*
         RSp561e*RSe453p );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,mp2,L) * (  - 1./108.*m44*
         RSp561e*RSe561p - 1./324.*m44*RSp561e*RSe451p + 1./324.*m44*
         RSp451e*RSe563p + 1./324.*m44*RSp451e*RSe453p - 1./324.*m44*
         RSp451e*RSe561p - 1./324.*m44*RSp451e*RSe451p );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,L) * (  - 1./108.*me2*pow(
         RSe563p,2) + 1./54.*me2*RSe453p*RSe563p - 1./108.*me2*pow(
         RSe453p,2) + 1./54.*me2*RSe561p*RSe563p - 1./54.*me2*RSe561p*
         RSe453p - 1./108.*me2*pow(RSe561p,2) - 1./54.*me2*RSe451p*
         RSe563p + 1./54.*me2*RSe451p*RSe453p + 1./54.*me2*RSe451p*
         RSe561p - 1./108.*me2*pow(RSe451p,2) + 1./216.*m66*pow(
         RSe563p,2) + 1./108.*m66*RSe453p*RSe563p + 1./72.*m66*pow(
         RSe453p,2) - 1./108.*m66*RSe561p*RSe563p - 1./108.*m66*RSe561p
         *RSe453p + 1./216.*m66*pow(RSe561p,2) - 1./108.*m66*RSe451p*
         RSe563p - 1./36.*m66*RSe451p*RSe453p + 1./108.*m66*RSe451p*
         RSe561p + 1./72.*m66*pow(RSe451p,2) - 1./108.*m46*pow(
         RSe563p,2) + 1./54.*m46*RSe453p*RSe563p - 1./108.*m46*pow(
         RSe453p,2) + 1./54.*m46*RSe561p*RSe563p - 1./54.*m46*RSe561p*
         RSe453p - 1./108.*m46*pow(RSe561p,2) - 1./54.*m46*RSe451p*
         RSe563p + 1./54.*m46*RSe451p*RSe453p + 1./54.*m46*RSe451p*
         RSe561p );

      F13p6x23RV +=  + Ab(m46,mu2)*BbV(me2,L) * (  - 1./108.*m46*pow(
         RSe451p,2) + 1./72.*m44*pow(RSe563p,2) + 1./108.*m44*RSe453p*
         RSe563p + 1./216.*m44*pow(RSe453p,2) - 1./36.*m44*RSe561p*
         RSe563p - 1./108.*m44*RSe561p*RSe453p + 1./72.*m44*pow(
         RSe561p,2) - 1./108.*m44*RSe451p*RSe563p - 1./108.*m44*RSe451p
         *RSe453p + 1./108.*m44*RSe451p*RSe561p + 1./216.*m44*pow(
         RSe451p,2) );

      F13p6x23RV +=  + Ab(m55,mu2)*BbV(m11,m33,L) * (  - 1./18.*m55*
         RS146pe*RS346pe + 1./54.*m55*RS546pe*RS146pe*RS346pe );

      F13p6x23RV +=  + Ab(m55,mu2)*BbV(m11,mp2,L) * (  - 1./18.*m55*
         RS146pe*RSp463e + 1./54.*m55*RS546pe*RS146pe*RSp463e + 1./18.*
         m55*RSp461e*RS146pe - 1./54.*m55*RSp461e*RS546pe*RS146pe );

      F13p6x23RV +=  + Ab(m55,mu2)*BbV(m11,me2,L) * (  - 1./18.*m55*
         RS146pe*RSe463p + 1./54.*m55*RS546pe*RS146pe*RSe463p + 1./18.*
         m55*RSe461p*RS146pe - 1./54.*m55*RSe461p*RS546pe*RS146pe );

      F13p6x23RV +=  + Ab(m55,mu2)*BbV(m11,L) * ( 1./36.*m55*pow(
         RS146pe,2) - 1./108.*m55*RS546pe*pow(RS146pe,2) );

      F13p6x23RV +=  + Ab(m55,mu2)*BbV(m15,L) * ( 1./48.*m55 - 1./144.*
         m55*RS546pe - 1./24.*m15 + 1./72.*m15*RS546pe + 1./48.*m11 - 1.
         /144.*m11*RS546pe );

      F13p6x23RV +=  + Ab(m55,mu2)*BbV(m33,mp2,L) * ( 1./18.*m55*
         RSp463e*RS346pe - 1./54.*m55*RS546pe*RSp463e*RS346pe - 1./18.*
         m55*RSp461e*RS346pe + 1./54.*m55*RSp461e*RS546pe*RS346pe );

      F13p6x23RV +=  + Ab(m55,mu2)*BbV(m33,me2,L) * ( 1./18.*m55*
         RSe463p*RS346pe - 1./54.*m55*RS546pe*RSe463p*RS346pe - 1./18.*
         m55*RSe461p*RS346pe + 1./54.*m55*RSe461p*RS546pe*RS346pe );

      F13p6x23RV +=  + Ab(m55,mu2)*BbV(m33,L) * ( 1./36.*m55*pow(
         RS346pe,2) - 1./108.*m55*RS546pe*pow(RS346pe,2) );

      F13p6x23RV +=  + Ab(m55,mu2)*BbV(m35,L) * ( 1./48.*m55 - 1./144.*
         m55*RS546pe - 1./24.*m35 + 1./72.*m35*RS546pe + 1./48.*m33 - 1.
         /144.*m33*RS546pe );

      F13p6x23RV +=  + Ab(m55,mu2)*BbV(mp2,me2,L) * ( 2./27.*m55*
         RSp463e*RSe463p - 2./81.*m55*RS546pe*RSp463e*RSe463p - 2./27.*
         m55*RSe461p*RSp463e + 2./81.*m55*RSe461p*RS546pe*RSp463e - 2./
         27.*m55*RSp461e*RSe463p + 2./81.*m55*RSp461e*RS546pe*RSe463p
          + 2./27.*m55*RSp461e*RSe461p - 2./81.*m55*RSp461e*RSe461p*
         RS546pe );

      F13p6x23RV +=  + Ab(m55,mu2)*BbV(mp2,L) * ( 1./36.*m55*pow(
         RSp463e,2) - 1./108.*m55*RS546pe*pow(RSp463e,2) - 1./18.*m55*
         RSp461e*RSp463e + 1./54.*m55*RSp461e*RS546pe*RSp463e + 1./36.*
         m55*pow(RSp461e,2) - 1./108.*m55*pow(RSp461e,2)*RS546pe );

      F13p6x23RV +=  + Ab(m55,mu2)*BbV(me2,mp2,L) * (  - 1./54.*m55*
         RSp463e*RSe463p + 1./162.*m55*RS546pe*RSp463e*RSe463p + 1./54.
         *m55*RSe461p*RSp463e - 1./162.*m55*RSe461p*RS546pe*RSp463e + 1.
         /54.*m55*RSp461e*RSe463p - 1./162.*m55*RSp461e*RS546pe*RSe463p
          - 1./54.*m55*RSp461e*RSe461p + 1./162.*m55*RSp461e*RSe461p*
         RS546pe );

      F13p6x23RV +=  + Ab(m55,mu2)*BbV(me2,L) * ( 1./36.*m55*pow(
         RSe463p,2) - 1./108.*m55*RS546pe*pow(RSe463p,2) - 1./18.*m55*
         RSe461p*RSe463p + 1./54.*m55*RSe461p*RS546pe*RSe463p + 1./36.*
         m55*pow(RSe461p,2) - 1./108.*m55*pow(RSe461p,2)*RS546pe );

      F13p6x23RV +=  + Ab(m56,mu2)*AbV(m11,L) * (  - 1./108.*pow(
         RS145pe,2) + 1./54.*RS146pe*RS145pe - 1./108.*pow(RS146pe,2) )
         ;

      F13p6x23RV +=  + Ab(m56,mu2)*AbV(m33,L) * (  - 1./108.*pow(
         RS345pe,2) + 1./54.*RS346pe*RS345pe - 1./108.*pow(RS346pe,2)
          + 1./54.*RS145pe*RS345pe - 1./54.*RS145pe*RS346pe - 1./54.*
         RS146pe*RS345pe + 1./54.*RS146pe*RS346pe );

      F13p6x23RV +=  + Ab(m56,mu2)*AbV(mp2,L) * ( 1./54.*RSp463e*
         RS345pe - 1./54.*RSp463e*RS346pe + 1./162.*RSp463e*RSe463p - 1.
         /162.*RSp463e*RSe453p - 1./108.*pow(RSp463e,2) - 1./54.*
         RSp453e*RS345pe + 1./54.*RSp453e*RS346pe - 1./162.*RSp453e*
         RSe463p + 1./162.*RSp453e*RSe453p + 1./54.*RSp453e*RSp463e - 1.
         /108.*pow(RSp453e,2) - 1./54.*RS145pe*RSp463e + 1./54.*RS145pe
         *RSp453e + 1./54.*RS146pe*RSp463e - 1./54.*RS146pe*RSp453e - 1.
         /162.*RSe461p*RSp463e + 1./162.*RSe461p*RSp453e + 1./162.*
         RSe451p*RSp463e - 1./162.*RSe451p*RSp453e - 1./54.*RSp461e*
         RS345pe + 1./54.*RSp461e*RS346pe - 1./162.*RSp461e*RSe463p + 1.
         /162.*RSp461e*RSe453p + 1./54.*RSp461e*RSp463e - 1./54.*
         RSp461e*RSp453e + 1./54.*RSp461e*RS145pe - 1./54.*RSp461e*
         RS146pe + 1./162.*RSp461e*RSe461p - 1./162.*RSp461e*RSe451p - 
         1./108.*pow(RSp461e,2) + 1./54.*RSp451e*RS345pe - 1./54.*
         RSp451e*RS346pe + 1./162.*RSp451e*RSe463p - 1./162.*RSp451e*
         RSe453p );

      F13p6x23RV +=  + Ab(m56,mu2)*AbV(mp2,L) * (  - 1./54.*RSp451e*
         RSp463e + 1./54.*RSp451e*RSp453e - 1./54.*RSp451e*RS145pe + 1./
         54.*RSp451e*RS146pe - 1./162.*RSp451e*RSe461p + 1./162.*
         RSp451e*RSe451p + 1./54.*RSp451e*RSp461e - 1./108.*pow(
         RSp451e,2) );

      F13p6x23RV +=  + Ab(m56,mu2)*AbV(me2,L) * ( 1./54.*RSe463p*
         RS345pe - 1./54.*RSe463p*RS346pe - 1./108.*pow(RSe463p,2) - 1./
         54.*RSe453p*RS345pe + 1./54.*RSe453p*RS346pe + 1./54.*RSe453p*
         RSe463p - 1./108.*pow(RSe453p,2) - 2./81.*RSp463e*RSe463p + 2./
         81.*RSp463e*RSe453p + 2./81.*RSp453e*RSe463p - 2./81.*RSp453e*
         RSe453p - 1./54.*RS145pe*RSe463p + 1./54.*RS145pe*RSe453p + 1./
         54.*RS146pe*RSe463p - 1./54.*RS146pe*RSe453p - 1./54.*RSe461p*
         RS345pe + 1./54.*RSe461p*RS346pe + 1./54.*RSe461p*RSe463p - 1./
         54.*RSe461p*RSe453p + 2./81.*RSe461p*RSp463e - 2./81.*RSe461p*
         RSp453e + 1./54.*RSe461p*RS145pe - 1./54.*RSe461p*RS146pe - 1./
         108.*pow(RSe461p,2) + 1./54.*RSe451p*RS345pe - 1./54.*RSe451p*
         RS346pe - 1./54.*RSe451p*RSe463p + 1./54.*RSe451p*RSe453p - 2./
         81.*RSe451p*RSp463e + 2./81.*RSe451p*RSp453e - 1./54.*RSe451p*
         RS145pe + 1./54.*RSe451p*RS146pe + 1./54.*RSe451p*RSe461p - 1./
         108.*pow(RSe451p,2) + 2./81.*RSp461e*RSe463p - 2./81.*RSp461e*
         RSe453p );

      F13p6x23RV +=  + Ab(m56,mu2)*AbV(me2,L) * (  - 2./81.*RSp461e*
         RSe461p + 2./81.*RSp461e*RSe451p - 2./81.*RSp451e*RSe463p + 2./
         81.*RSp451e*RSe453p + 2./81.*RSp451e*RSe461p - 2./81.*RSp451e*
         RSe451p );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,m33,L) * (  - 1./36.*m66*
         RS145pe*RS345pe - 1./108.*m66*RS145pe*RS346pe - 1./108.*m66*
         RS146pe*RS345pe - 1./108.*m66*RS146pe*RS346pe + 1./54.*m56*
         RS145pe*RS345pe - 1./54.*m56*RS145pe*RS346pe - 1./54.*m56*
         RS146pe*RS345pe + 1./54.*m56*RS146pe*RS346pe - 1./108.*m55*
         RS145pe*RS345pe - 1./108.*m55*RS145pe*RS346pe - 1./108.*m55*
         RS146pe*RS345pe - 1./36.*m55*RS146pe*RS346pe + 1./54.*m11*
         RS145pe*RS345pe - 1./54.*m11*RS145pe*RS346pe - 1./54.*m11*
         RS146pe*RS345pe + 1./54.*m11*RS146pe*RS346pe );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,mp2,L) * (  - 1./108.*m66*
         RS145pe*RSp463e - 1./36.*m66*RS145pe*RSp453e - 1./108.*m66*
         RS146pe*RSp463e - 1./108.*m66*RS146pe*RSp453e + 1./108.*m66*
         RSp461e*RS145pe + 1./108.*m66*RSp461e*RS146pe + 1./36.*m66*
         RSp451e*RS145pe + 1./108.*m66*RSp451e*RS146pe - 1./54.*m56*
         RS145pe*RSp463e + 1./54.*m56*RS145pe*RSp453e + 1./54.*m56*
         RS146pe*RSp463e - 1./54.*m56*RS146pe*RSp453e + 1./54.*m56*
         RSp461e*RS145pe - 1./54.*m56*RSp461e*RS146pe - 1./54.*m56*
         RSp451e*RS145pe + 1./54.*m56*RSp451e*RS146pe - 1./108.*m55*
         RS145pe*RSp463e - 1./108.*m55*RS145pe*RSp453e - 1./36.*m55*
         RS146pe*RSp463e - 1./108.*m55*RS146pe*RSp453e + 1./108.*m55*
         RSp461e*RS145pe + 1./36.*m55*RSp461e*RS146pe + 1./108.*m55*
         RSp451e*RS145pe + 1./108.*m55*RSp451e*RS146pe - 1./54.*m11*
         RS145pe*RSp463e + 1./54.*m11*RS145pe*RSp453e + 1./54.*m11*
         RS146pe*RSp463e - 1./54.*m11*RS146pe*RSp453e + 1./54.*m11*
         RSp461e*RS145pe );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,mp2,L) * (  - 1./54.*m11*
         RSp461e*RS146pe - 1./54.*m11*RSp451e*RS145pe + 1./54.*m11*
         RSp451e*RS146pe );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,me2,L) * (  - 1./108.*m66*
         RS145pe*RSe463p - 1./36.*m66*RS145pe*RSe453p - 1./108.*m66*
         RS146pe*RSe463p - 1./108.*m66*RS146pe*RSe453p + 1./108.*m66*
         RSe461p*RS145pe + 1./108.*m66*RSe461p*RS146pe + 1./36.*m66*
         RSe451p*RS145pe + 1./108.*m66*RSe451p*RS146pe - 1./54.*m56*
         RS145pe*RSe463p + 1./54.*m56*RS145pe*RSe453p + 1./54.*m56*
         RS146pe*RSe463p - 1./54.*m56*RS146pe*RSe453p + 1./54.*m56*
         RSe461p*RS145pe - 1./54.*m56*RSe461p*RS146pe - 1./54.*m56*
         RSe451p*RS145pe + 1./54.*m56*RSe451p*RS146pe - 1./108.*m55*
         RS145pe*RSe463p - 1./108.*m55*RS145pe*RSe453p - 1./36.*m55*
         RS146pe*RSe463p - 1./108.*m55*RS146pe*RSe453p + 1./108.*m55*
         RSe461p*RS145pe + 1./36.*m55*RSe461p*RS146pe + 1./108.*m55*
         RSe451p*RS145pe + 1./108.*m55*RSe451p*RS146pe - 1./54.*m11*
         RS145pe*RSe463p + 1./54.*m11*RS145pe*RSe453p + 1./54.*m11*
         RS146pe*RSe463p - 1./54.*m11*RS146pe*RSe453p + 1./54.*m11*
         RSe461p*RS145pe );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,me2,L) * (  - 1./54.*m11*
         RSe461p*RS146pe - 1./54.*m11*RSe451p*RS145pe + 1./54.*m11*
         RSe451p*RS146pe );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m11,L) * ( 1./72.*m66*pow(
         RS145pe,2) + 1./108.*m66*RS146pe*RS145pe + 1./216.*m66*pow(
         RS146pe,2) - 1./108.*m56*pow(RS145pe,2) + 1./54.*m56*RS146pe*
         RS145pe - 1./108.*m56*pow(RS146pe,2) + 1./216.*m55*pow(
         RS145pe,2) + 1./108.*m55*RS146pe*RS145pe + 1./72.*m55*pow(
         RS146pe,2) - 1./108.*m11*pow(RS145pe,2) + 1./54.*m11*RS146pe*
         RS145pe - 1./108.*m11*pow(RS146pe,2) );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m15,L) * ( 1./48.*m66 - 1./24.*
         m56 + 1./24.*m55 - 1./24.*m15 + 1./48.*m11 );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m16,L) * ( 1./24.*m66 - 1./24.*
         m56 + 1./48.*m55 - 1./24.*m16 + 1./48.*m11 );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,mp2,L) * ( 1./108.*m66*
         RSp463e*RS345pe + 1./108.*m66*RSp463e*RS346pe + 1./36.*m66*
         RSp453e*RS345pe + 1./108.*m66*RSp453e*RS346pe - 1./108.*m66*
         RSp461e*RS345pe - 1./108.*m66*RSp461e*RS346pe - 1./36.*m66*
         RSp451e*RS345pe - 1./108.*m66*RSp451e*RS346pe + 1./54.*m56*
         RSp463e*RS345pe - 1./54.*m56*RSp463e*RS346pe - 1./54.*m56*
         RSp453e*RS345pe + 1./54.*m56*RSp453e*RS346pe - 1./54.*m56*
         RSp461e*RS345pe + 1./54.*m56*RSp461e*RS346pe + 1./54.*m56*
         RSp451e*RS345pe - 1./54.*m56*RSp451e*RS346pe + 1./108.*m55*
         RSp463e*RS345pe + 1./36.*m55*RSp463e*RS346pe + 1./108.*m55*
         RSp453e*RS345pe + 1./108.*m55*RSp453e*RS346pe - 1./108.*m55*
         RSp461e*RS345pe - 1./36.*m55*RSp461e*RS346pe - 1./108.*m55*
         RSp451e*RS345pe - 1./108.*m55*RSp451e*RS346pe + 1./54.*m33*
         RSp463e*RS345pe - 1./54.*m33*RSp463e*RS346pe - 1./54.*m33*
         RSp453e*RS345pe + 1./54.*m33*RSp453e*RS346pe - 1./54.*m33*
         RSp461e*RS345pe );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,mp2,L) * ( 1./54.*m33*
         RSp461e*RS346pe + 1./54.*m33*RSp451e*RS345pe - 1./54.*m33*
         RSp451e*RS346pe );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,me2,L) * ( 1./108.*m66*
         RSe463p*RS345pe + 1./108.*m66*RSe463p*RS346pe + 1./36.*m66*
         RSe453p*RS345pe + 1./108.*m66*RSe453p*RS346pe - 1./108.*m66*
         RSe461p*RS345pe - 1./108.*m66*RSe461p*RS346pe - 1./36.*m66*
         RSe451p*RS345pe - 1./108.*m66*RSe451p*RS346pe + 1./54.*m56*
         RSe463p*RS345pe - 1./54.*m56*RSe463p*RS346pe - 1./54.*m56*
         RSe453p*RS345pe + 1./54.*m56*RSe453p*RS346pe - 1./54.*m56*
         RSe461p*RS345pe + 1./54.*m56*RSe461p*RS346pe + 1./54.*m56*
         RSe451p*RS345pe - 1./54.*m56*RSe451p*RS346pe + 1./108.*m55*
         RSe463p*RS345pe + 1./36.*m55*RSe463p*RS346pe + 1./108.*m55*
         RSe453p*RS345pe + 1./108.*m55*RSe453p*RS346pe - 1./108.*m55*
         RSe461p*RS345pe - 1./36.*m55*RSe461p*RS346pe - 1./108.*m55*
         RSe451p*RS345pe - 1./108.*m55*RSe451p*RS346pe + 1./54.*m33*
         RSe463p*RS345pe - 1./54.*m33*RSe463p*RS346pe - 1./54.*m33*
         RSe453p*RS345pe + 1./54.*m33*RSe453p*RS346pe - 1./54.*m33*
         RSe461p*RS345pe );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,me2,L) * ( 1./54.*m33*
         RSe461p*RS346pe + 1./54.*m33*RSe451p*RS345pe - 1./54.*m33*
         RSe451p*RS346pe );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m33,L) * ( 1./72.*m66*pow(
         RS345pe,2) + 1./108.*m66*RS346pe*RS345pe + 1./216.*m66*pow(
         RS346pe,2) - 1./108.*m56*pow(RS345pe,2) + 1./54.*m56*RS346pe*
         RS345pe - 1./108.*m56*pow(RS346pe,2) + 1./216.*m55*pow(
         RS345pe,2) + 1./108.*m55*RS346pe*RS345pe + 1./72.*m55*pow(
         RS346pe,2) - 1./108.*m33*pow(RS345pe,2) + 1./54.*m33*RS346pe*
         RS345pe - 1./108.*m33*pow(RS346pe,2) );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m35,L) * ( 1./48.*m66 - 1./24.*
         m56 + 1./24.*m55 - 1./24.*m35 + 1./48.*m33 );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(m36,L) * ( 1./24.*m66 - 1./24.*
         m56 + 1./48.*m55 - 1./24.*m36 + 1./48.*m33 );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(mp2,me2,L) * (  - 2./81.*mp2*
         RSp463e*RSe463p + 2./81.*mp2*RSp463e*RSe453p + 2./81.*mp2*
         RSp453e*RSe463p - 2./81.*mp2*RSp453e*RSe453p + 2./81.*mp2*
         RSe461p*RSp463e - 2./81.*mp2*RSe461p*RSp453e - 2./81.*mp2*
         RSe451p*RSp463e + 2./81.*mp2*RSe451p*RSp453e + 2./81.*mp2*
         RSp461e*RSe463p - 2./81.*mp2*RSp461e*RSe453p - 2./81.*mp2*
         RSp461e*RSe461p + 2./81.*mp2*RSp461e*RSe451p - 2./81.*mp2*
         RSp451e*RSe463p + 2./81.*mp2*RSp451e*RSe453p + 2./81.*mp2*
         RSp451e*RSe461p - 2./81.*mp2*RSp451e*RSe451p + 1./81.*m66*
         RSp463e*RSe463p + 1./81.*m66*RSp463e*RSe453p + 1./81.*m66*
         RSp453e*RSe463p + 1./27.*m66*RSp453e*RSe453p - 1./81.*m66*
         RSe461p*RSp463e - 1./81.*m66*RSe461p*RSp453e - 1./81.*m66*
         RSe451p*RSp463e - 1./27.*m66*RSe451p*RSp453e - 1./81.*m66*
         RSp461e*RSe463p - 1./81.*m66*RSp461e*RSe453p + 1./81.*m66*
         RSp461e*RSe461p + 1./81.*m66*RSp461e*RSe451p - 1./81.*m66*
         RSp451e*RSe463p );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(mp2,me2,L) * (  - 1./27.*m66*
         RSp451e*RSe453p + 1./81.*m66*RSp451e*RSe461p + 1./27.*m66*
         RSp451e*RSe451p - 2./81.*m56*RSp463e*RSe463p + 2./81.*m56*
         RSp463e*RSe453p + 2./81.*m56*RSp453e*RSe463p - 2./81.*m56*
         RSp453e*RSe453p + 2./81.*m56*RSe461p*RSp463e - 2./81.*m56*
         RSe461p*RSp453e - 2./81.*m56*RSe451p*RSp463e + 2./81.*m56*
         RSe451p*RSp453e + 2./81.*m56*RSp461e*RSe463p - 2./81.*m56*
         RSp461e*RSe453p - 2./81.*m56*RSp461e*RSe461p + 2./81.*m56*
         RSp461e*RSe451p - 2./81.*m56*RSp451e*RSe463p + 2./81.*m56*
         RSp451e*RSe453p + 2./81.*m56*RSp451e*RSe461p - 2./81.*m56*
         RSp451e*RSe451p + 1./27.*m55*RSp463e*RSe463p + 1./81.*m55*
         RSp463e*RSe453p + 1./81.*m55*RSp453e*RSe463p + 1./81.*m55*
         RSp453e*RSe453p - 1./27.*m55*RSe461p*RSp463e - 1./81.*m55*
         RSe461p*RSp453e - 1./81.*m55*RSe451p*RSp463e - 1./81.*m55*
         RSe451p*RSp453e - 1./27.*m55*RSp461e*RSe463p - 1./81.*m55*
         RSp461e*RSe453p );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(mp2,me2,L) * ( 1./27.*m55*
         RSp461e*RSe461p + 1./81.*m55*RSp461e*RSe451p - 1./81.*m55*
         RSp451e*RSe463p - 1./81.*m55*RSp451e*RSe453p + 1./81.*m55*
         RSp451e*RSe461p + 1./81.*m55*RSp451e*RSe451p );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(mp2,L) * (  - 1./108.*mp2*pow(
         RSp463e,2) + 1./54.*mp2*RSp453e*RSp463e - 1./108.*mp2*pow(
         RSp453e,2) + 1./54.*mp2*RSp461e*RSp463e - 1./54.*mp2*RSp461e*
         RSp453e - 1./108.*mp2*pow(RSp461e,2) - 1./54.*mp2*RSp451e*
         RSp463e + 1./54.*mp2*RSp451e*RSp453e + 1./54.*mp2*RSp451e*
         RSp461e - 1./108.*mp2*pow(RSp451e,2) + 1./216.*m66*pow(
         RSp463e,2) + 1./108.*m66*RSp453e*RSp463e + 1./72.*m66*pow(
         RSp453e,2) - 1./108.*m66*RSp461e*RSp463e - 1./108.*m66*RSp461e
         *RSp453e + 1./216.*m66*pow(RSp461e,2) - 1./108.*m66*RSp451e*
         RSp463e - 1./36.*m66*RSp451e*RSp453e + 1./108.*m66*RSp451e*
         RSp461e + 1./72.*m66*pow(RSp451e,2) - 1./108.*m56*pow(
         RSp463e,2) + 1./54.*m56*RSp453e*RSp463e - 1./108.*m56*pow(
         RSp453e,2) + 1./54.*m56*RSp461e*RSp463e - 1./54.*m56*RSp461e*
         RSp453e - 1./108.*m56*pow(RSp461e,2) - 1./54.*m56*RSp451e*
         RSp463e + 1./54.*m56*RSp451e*RSp453e + 1./54.*m56*RSp451e*
         RSp461e );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(mp2,L) * (  - 1./108.*m56*pow(
         RSp451e,2) + 1./72.*m55*pow(RSp463e,2) + 1./108.*m55*RSp453e*
         RSp463e + 1./216.*m55*pow(RSp453e,2) - 1./36.*m55*RSp461e*
         RSp463e - 1./108.*m55*RSp461e*RSp453e + 1./72.*m55*pow(
         RSp461e,2) - 1./108.*m55*RSp451e*RSp463e - 1./108.*m55*RSp451e
         *RSp453e + 1./108.*m55*RSp451e*RSp461e + 1./216.*m55*pow(
         RSp451e,2) );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * ( 1./162.*me2*
         RSp463e*RSe463p - 1./162.*me2*RSp463e*RSe453p - 1./162.*me2*
         RSp453e*RSe463p + 1./162.*me2*RSp453e*RSe453p - 1./162.*me2*
         RSe461p*RSp463e + 1./162.*me2*RSe461p*RSp453e + 1./162.*me2*
         RSe451p*RSp463e - 1./162.*me2*RSe451p*RSp453e - 1./162.*me2*
         RSp461e*RSe463p + 1./162.*me2*RSp461e*RSe453p + 1./162.*me2*
         RSp461e*RSe461p - 1./162.*me2*RSp461e*RSe451p + 1./162.*me2*
         RSp451e*RSe463p - 1./162.*me2*RSp451e*RSe453p - 1./162.*me2*
         RSp451e*RSe461p + 1./162.*me2*RSp451e*RSe451p - 1./324.*m66*
         RSp463e*RSe463p - 1./324.*m66*RSp463e*RSe453p - 1./324.*m66*
         RSp453e*RSe463p - 1./108.*m66*RSp453e*RSe453p + 1./324.*m66*
         RSe461p*RSp463e + 1./324.*m66*RSe461p*RSp453e + 1./324.*m66*
         RSe451p*RSp463e + 1./108.*m66*RSe451p*RSp453e + 1./324.*m66*
         RSp461e*RSe463p + 1./324.*m66*RSp461e*RSe453p - 1./324.*m66*
         RSp461e*RSe461p - 1./324.*m66*RSp461e*RSe451p + 1./324.*m66*
         RSp451e*RSe463p );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * ( 1./108.*m66*
         RSp451e*RSe453p - 1./324.*m66*RSp451e*RSe461p - 1./108.*m66*
         RSp451e*RSe451p + 1./162.*m56*RSp463e*RSe463p - 1./162.*m56*
         RSp463e*RSe453p - 1./162.*m56*RSp453e*RSe463p + 1./162.*m56*
         RSp453e*RSe453p - 1./162.*m56*RSe461p*RSp463e + 1./162.*m56*
         RSe461p*RSp453e + 1./162.*m56*RSe451p*RSp463e - 1./162.*m56*
         RSe451p*RSp453e - 1./162.*m56*RSp461e*RSe463p + 1./162.*m56*
         RSp461e*RSe453p + 1./162.*m56*RSp461e*RSe461p - 1./162.*m56*
         RSp461e*RSe451p + 1./162.*m56*RSp451e*RSe463p - 1./162.*m56*
         RSp451e*RSe453p - 1./162.*m56*RSp451e*RSe461p + 1./162.*m56*
         RSp451e*RSe451p - 1./108.*m55*RSp463e*RSe463p - 1./324.*m55*
         RSp463e*RSe453p - 1./324.*m55*RSp453e*RSe463p - 1./324.*m55*
         RSp453e*RSe453p + 1./108.*m55*RSe461p*RSp463e + 1./324.*m55*
         RSe461p*RSp453e + 1./324.*m55*RSe451p*RSp463e + 1./324.*m55*
         RSe451p*RSp453e + 1./108.*m55*RSp461e*RSe463p + 1./324.*m55*
         RSp461e*RSe453p );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,mp2,L) * (  - 1./108.*m55*
         RSp461e*RSe461p - 1./324.*m55*RSp461e*RSe451p + 1./324.*m55*
         RSp451e*RSe463p + 1./324.*m55*RSp451e*RSe453p - 1./324.*m55*
         RSp451e*RSe461p - 1./324.*m55*RSp451e*RSe451p );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,L) * (  - 1./108.*me2*pow(
         RSe463p,2) + 1./54.*me2*RSe453p*RSe463p - 1./108.*me2*pow(
         RSe453p,2) + 1./54.*me2*RSe461p*RSe463p - 1./54.*me2*RSe461p*
         RSe453p - 1./108.*me2*pow(RSe461p,2) - 1./54.*me2*RSe451p*
         RSe463p + 1./54.*me2*RSe451p*RSe453p + 1./54.*me2*RSe451p*
         RSe461p - 1./108.*me2*pow(RSe451p,2) + 1./216.*m66*pow(
         RSe463p,2) + 1./108.*m66*RSe453p*RSe463p + 1./72.*m66*pow(
         RSe453p,2) - 1./108.*m66*RSe461p*RSe463p - 1./108.*m66*RSe461p
         *RSe453p + 1./216.*m66*pow(RSe461p,2) - 1./108.*m66*RSe451p*
         RSe463p - 1./36.*m66*RSe451p*RSe453p + 1./108.*m66*RSe451p*
         RSe461p + 1./72.*m66*pow(RSe451p,2) - 1./108.*m56*pow(
         RSe463p,2) + 1./54.*m56*RSe453p*RSe463p - 1./108.*m56*pow(
         RSe453p,2) + 1./54.*m56*RSe461p*RSe463p - 1./54.*m56*RSe461p*
         RSe453p - 1./108.*m56*pow(RSe461p,2) - 1./54.*m56*RSe451p*
         RSe463p + 1./54.*m56*RSe451p*RSe453p + 1./54.*m56*RSe451p*
         RSe461p );

      F13p6x23RV +=  + Ab(m56,mu2)*BbV(me2,L) * (  - 1./108.*m56*pow(
         RSe451p,2) + 1./72.*m55*pow(RSe463p,2) + 1./108.*m55*RSe453p*
         RSe463p + 1./216.*m55*pow(RSe453p,2) - 1./36.*m55*RSe461p*
         RSe463p - 1./108.*m55*RSe461p*RSe453p + 1./72.*m55*pow(
         RSe461p,2) - 1./108.*m55*RSe451p*RSe463p - 1./108.*m55*RSe451p
         *RSe453p + 1./108.*m55*RSe451p*RSe461p + 1./216.*m55*pow(
         RSe451p,2) );

      F13p6x23RV +=  + Ab(m66,mu2)*BbV(m11,m33,L) * (  - 1./18.*m66*
         RS145pe*RS345pe + 1./54.*m66*RS645pe*RS145pe*RS345pe );

      F13p6x23RV +=  + Ab(m66,mu2)*BbV(m11,mp2,L) * (  - 1./18.*m66*
         RS145pe*RSp453e + 1./54.*m66*RS645pe*RS145pe*RSp453e + 1./18.*
         m66*RSp451e*RS145pe - 1./54.*m66*RSp451e*RS645pe*RS145pe );

      F13p6x23RV +=  + Ab(m66,mu2)*BbV(m11,me2,L) * (  - 1./18.*m66*
         RS145pe*RSe453p + 1./54.*m66*RS645pe*RS145pe*RSe453p + 1./18.*
         m66*RSe451p*RS145pe - 1./54.*m66*RSe451p*RS645pe*RS145pe );

      F13p6x23RV +=  + Ab(m66,mu2)*BbV(m11,L) * ( 1./36.*m66*pow(
         RS145pe,2) - 1./108.*m66*RS645pe*pow(RS145pe,2) );

      F13p6x23RV +=  + Ab(m66,mu2)*BbV(m16,L) * ( 1./48.*m66 - 1./144.*
         m66*RS645pe - 1./24.*m16 + 1./72.*m16*RS645pe + 1./48.*m11 - 1.
         /144.*m11*RS645pe );

      F13p6x23RV +=  + Ab(m66,mu2)*BbV(m33,mp2,L) * ( 1./18.*m66*
         RSp453e*RS345pe - 1./54.*m66*RS645pe*RSp453e*RS345pe - 1./18.*
         m66*RSp451e*RS345pe + 1./54.*m66*RSp451e*RS645pe*RS345pe );

      F13p6x23RV +=  + Ab(m66,mu2)*BbV(m33,me2,L) * ( 1./18.*m66*
         RSe453p*RS345pe - 1./54.*m66*RS645pe*RSe453p*RS345pe - 1./18.*
         m66*RSe451p*RS345pe + 1./54.*m66*RSe451p*RS645pe*RS345pe );

      F13p6x23RV +=  + Ab(m66,mu2)*BbV(m33,L) * ( 1./36.*m66*pow(
         RS345pe,2) - 1./108.*m66*RS645pe*pow(RS345pe,2) );

      F13p6x23RV +=  + Ab(m66,mu2)*BbV(m36,L) * ( 1./48.*m66 - 1./144.*
         m66*RS645pe - 1./24.*m36 + 1./72.*m36*RS645pe + 1./48.*m33 - 1.
         /144.*m33*RS645pe );

      F13p6x23RV +=  + Ab(m66,mu2)*BbV(mp2,me2,L) * ( 2./27.*m66*
         RSp453e*RSe453p - 2./81.*m66*RS645pe*RSp453e*RSe453p - 2./27.*
         m66*RSe451p*RSp453e + 2./81.*m66*RSe451p*RS645pe*RSp453e - 2./
         27.*m66*RSp451e*RSe453p + 2./81.*m66*RSp451e*RS645pe*RSe453p
          + 2./27.*m66*RSp451e*RSe451p - 2./81.*m66*RSp451e*RSe451p*
         RS645pe );

      F13p6x23RV +=  + Ab(m66,mu2)*BbV(mp2,L) * ( 1./36.*m66*pow(
         RSp453e,2) - 1./108.*m66*RS645pe*pow(RSp453e,2) - 1./18.*m66*
         RSp451e*RSp453e + 1./54.*m66*RSp451e*RS645pe*RSp453e + 1./36.*
         m66*pow(RSp451e,2) - 1./108.*m66*pow(RSp451e,2)*RS645pe );

      F13p6x23RV +=  + Ab(m66,mu2)*BbV(me2,mp2,L) * (  - 1./54.*m66*
         RSp453e*RSe453p + 1./162.*m66*RS645pe*RSp453e*RSe453p + 1./54.
         *m66*RSe451p*RSp453e - 1./162.*m66*RSe451p*RS645pe*RSp453e + 1.
         /54.*m66*RSp451e*RSe453p - 1./162.*m66*RSp451e*RS645pe*RSe453p
          - 1./54.*m66*RSp451e*RSe451p + 1./162.*m66*RSp451e*RSe451p*
         RS645pe );

      F13p6x23RV +=  + Ab(m66,mu2)*BbV(me2,L) * ( 1./36.*m66*pow(
         RSe453p,2) - 1./108.*m66*RS645pe*pow(RSe453p,2) - 1./18.*m66*
         RSe451p*RSe453p + 1./54.*m66*RSe451p*RS645pe*RSe453p + 1./36.*
         m66*pow(RSe451p,2) - 1./108.*m66*pow(RSe451p,2)*RS645pe );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m11,L) * ( 1./36.*RSp45613e*
         RS14563pe - 1./72.*RSp45633e*RS14563pe - 1./72.*RC1456pe*
         RSp45613e + 1./144.*RC1456pe*RSp45633e - 1./72.*RSp45611e*
         RS14563pe + 1./144.*RSp45611e*RC1456pe + 1./18.*m11*RN1p*
         RC1456pe*RSp45613e - 1./18.*m11*RN1p*RSp45611e*RC1456pe );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m13,L) * ( 1./162.*RSp45613e*
         RS34561pe + 1./162.*RSp45613e*RS14563pe + 1./162.*RSp45613e*
         RSe45613p + 1./162.*pow(RSp45613e,2) - 1./324.*RC3456pe*
         RSp45613e - 1./324.*RSe45633p*RSp45613e - 1./324.*RSp45633e*
         RS34561pe - 1./324.*RSp45633e*RS14563pe - 1./324.*RSp45633e*
         RSe45613p - 1./162.*RSp45633e*RSp45613e + 1./648.*RSp45633e*
         RC3456pe + 1./648.*RSp45633e*RSe45633p + 1./648.*pow(
         RSp45633e,2) - 1./324.*RC1456pe*RSp45613e + 1./648.*RC1456pe*
         RSp45633e - 1./324.*RSe45611p*RSp45613e + 1./648.*RSe45611p*
         RSp45633e - 1./324.*RSp45611e*RS34561pe - 1./324.*RSp45611e*
         RS14563pe - 1./324.*RSp45611e*RSe45613p - 1./162.*RSp45611e*
         RSp45613e + 1./648.*RSp45611e*RC3456pe + 1./648.*RSp45611e*
         RSe45633p + 1./324.*RSp45611e*RSp45633e + 1./648.*RSp45611e*
         RC1456pe + 1./648.*RSp45611e*RSe45611p + 1./648.*pow(
         RSp45611e,2) );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m14,L) * (  - 1./36.*RSp45613e
          - 1./48.*RSp45633e + 5./72.*RSp563e + 7./144.*RSp45611e - 5./
         72.*RSp561e - 1./36.*m44*RN1p*RSp45613e - 1./36.*m44*RN1p*
         RSp563e + 1./36.*m44*RN1p*RSp45611e + 1./36.*m44*RN1p*RSp561e
          + 1./18.*m14*RN1p*RSp45613e - 1./18.*m14*RN1p*RSp563e - 1./18.
         *m14*RN1p*RSp45611e + 1./18.*m14*RN1p*RSp561e - 1./36.*m11*
         RN1p*RSp45613e - 1./12.*m11*RN1p*RSp563e + 1./36.*m11*RN1p*
         RSp45611e + 1./12.*m11*RN1p*RSp561e );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m15,L) * (  - 1./36.*RSp45613e
          - 1./48.*RSp45633e + 5./72.*RSp463e + 7./144.*RSp45611e - 5./
         72.*RSp461e - 1./36.*m55*RN1p*RSp45613e - 1./36.*m55*RN1p*
         RSp463e + 1./36.*m55*RN1p*RSp45611e + 1./36.*m55*RN1p*RSp461e
          + 1./18.*m15*RN1p*RSp45613e - 1./18.*m15*RN1p*RSp463e - 1./18.
         *m15*RN1p*RSp45611e + 1./18.*m15*RN1p*RSp461e - 1./36.*m11*
         RN1p*RSp45613e - 1./12.*m11*RN1p*RSp463e + 1./36.*m11*RN1p*
         RSp45611e + 1./12.*m11*RN1p*RSp461e );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m16,L) * (  - 1./36.*RSp45613e
          - 1./48.*RSp45633e + 5./72.*RSp453e + 7./144.*RSp45611e - 5./
         72.*RSp451e - 1./36.*m66*RN1p*RSp45613e - 1./36.*m66*RN1p*
         RSp453e + 1./36.*m66*RN1p*RSp45611e + 1./36.*m66*RN1p*RSp451e
          + 1./18.*m16*RN1p*RSp45613e - 1./18.*m16*RN1p*RSp453e - 1./18.
         *m16*RN1p*RSp45611e + 1./18.*m16*RN1p*RSp451e - 1./36.*m11*
         RN1p*RSp45613e - 1./12.*m11*RN1p*RSp453e + 1./36.*m11*RN1p*
         RSp45611e + 1./12.*m11*RN1p*RSp451e );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m33,L) * ( 1./36.*RSp45613e*
         RS34561pe - 1./72.*RC3456pe*RSp45613e - 1./72.*RSp45633e*
         RS34561pe + 1./144.*RSp45633e*RC3456pe - 1./72.*RSp45611e*
         RS34561pe + 1./144.*RSp45611e*RC3456pe + 1./18.*m33*RN3p*
         RC3456pe*RSp45613e - 1./18.*m33*RN3p*RSp45633e*RC3456pe );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m34,L) * (  - 1./36.*RSp45613e
          + 7./144.*RSp45633e - 5./72.*RSp563e - 1./48.*RSp45611e + 5./
         72.*RSp561e - 1./36.*m44*RN3p*RSp45613e + 1./36.*m44*RN3p*
         RSp45633e + 1./36.*m44*RN3p*RSp563e - 1./36.*m44*RN3p*RSp561e
          + 1./18.*m34*RN3p*RSp45613e - 1./18.*m34*RN3p*RSp45633e + 1./
         18.*m34*RN3p*RSp563e - 1./18.*m34*RN3p*RSp561e - 1./36.*m33*
         RN3p*RSp45613e + 1./36.*m33*RN3p*RSp45633e + 1./12.*m33*RN3p*
         RSp563e - 1./12.*m33*RN3p*RSp561e );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m35,L) * (  - 1./36.*RSp45613e
          + 7./144.*RSp45633e - 5./72.*RSp463e - 1./48.*RSp45611e + 5./
         72.*RSp461e - 1./36.*m55*RN3p*RSp45613e + 1./36.*m55*RN3p*
         RSp45633e + 1./36.*m55*RN3p*RSp463e - 1./36.*m55*RN3p*RSp461e
          + 1./18.*m35*RN3p*RSp45613e - 1./18.*m35*RN3p*RSp45633e + 1./
         18.*m35*RN3p*RSp463e - 1./18.*m35*RN3p*RSp461e - 1./36.*m33*
         RN3p*RSp45613e + 1./36.*m33*RN3p*RSp45633e + 1./12.*m33*RN3p*
         RSp463e - 1./12.*m33*RN3p*RSp461e );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m36,L) * (  - 1./36.*RSp45613e
          + 7./144.*RSp45633e - 5./72.*RSp453e - 1./48.*RSp45611e + 5./
         72.*RSp451e - 1./36.*m66*RN3p*RSp45613e + 1./36.*m66*RN3p*
         RSp45633e + 1./36.*m66*RN3p*RSp453e - 1./36.*m66*RN3p*RSp451e
          + 1./18.*m36*RN3p*RSp45613e - 1./18.*m36*RN3p*RSp45633e + 1./
         18.*m36*RN3p*RSp453e - 1./18.*m36*RN3p*RSp451e - 1./36.*m33*
         RN3p*RSp45613e + 1./36.*m33*RN3p*RSp45633e + 1./12.*m33*RN3p*
         RSp453e - 1./12.*m33*RN3p*RSp451e );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m44,L) * (  - 1./18.*m44*RN3p*
         RSp563e*RS356pe + 1./54.*m44*RN3p*RS456pe*RSp563e*RS356pe + 1./
         18.*m44*RN3p*RSp561e*RS356pe - 1./54.*m44*RN3p*RSp561e*RS456pe
         *RS356pe + 1./18.*m44*RN1p*RS156pe*RSp563e - 1./54.*m44*RN1p*
         RS456pe*RS156pe*RSp563e - 1./18.*m44*RN1p*RSp561e*RS156pe + 1./
         54.*m44*RN1p*RSp561e*RS456pe*RS156pe + 1./18.*RN(mp2,me2)*m44*
         RSp563e*RSe563p - 1./54.*RN(mp2,me2)*m44*RS456pe*RSp563e*
         RSe563p - 1./18.*RN(mp2,me2)*m44*RSe561p*RSp563e + 1./54.*RN(
         mp2,me2)*m44*RSe561p*RS456pe*RSp563e - 1./18.*RN(mp2,me2)*m44*
         RSp561e*RSe563p + 1./54.*RN(mp2,me2)*m44*RSp561e*RS456pe*
         RSe563p + 1./18.*RN(mp2,me2)*m44*RSp561e*RSe561p - 1./54.*RN(
         mp2,me2)*m44*RSp561e*RSe561p*RS456pe );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./54.*RSp563e*
         RS346pe - 1./54.*RSp563e*RS356pe + 1./162.*RSp563e*RSe563p - 1.
         /162.*RSp563e*RSe463p - 1./108.*pow(RSp563e,2) - 1./54.*
         RSp463e*RS346pe + 1./54.*RSp463e*RS356pe - 1./162.*RSp463e*
         RSe563p + 1./162.*RSp463e*RSe463p + 1./54.*RSp463e*RSp563e - 1.
         /108.*pow(RSp463e,2) - 1./54.*RS146pe*RSp563e + 1./54.*RS146pe
         *RSp463e + 1./54.*RS156pe*RSp563e - 1./54.*RS156pe*RSp463e - 1.
         /162.*RSe561p*RSp563e + 1./162.*RSe561p*RSp463e + 1./162.*
         RSe461p*RSp563e - 1./162.*RSe461p*RSp463e - 1./54.*RSp561e*
         RS346pe + 1./54.*RSp561e*RS356pe - 1./162.*RSp561e*RSe563p + 1.
         /162.*RSp561e*RSe463p + 1./54.*RSp561e*RSp563e - 1./54.*
         RSp561e*RSp463e + 1./54.*RSp561e*RS146pe - 1./54.*RSp561e*
         RS156pe + 1./162.*RSp561e*RSe561p - 1./162.*RSp561e*RSe461p - 
         1./108.*pow(RSp561e,2) + 1./54.*RSp461e*RS346pe - 1./54.*
         RSp461e*RS356pe + 1./162.*RSp461e*RSe563p - 1./162.*RSp461e*
         RSe463p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./54.*RSp461e*
         RSp563e + 1./54.*RSp461e*RSp463e - 1./54.*RSp461e*RS146pe + 1./
         54.*RSp461e*RS156pe - 1./162.*RSp461e*RSe561p + 1./162.*
         RSp461e*RSe461p + 1./54.*RSp461e*RSp561e - 1./108.*pow(
         RSp461e,2) - 1./108.*m55*RN3p*RSp563e*RS346pe - 1./108.*m55*
         RN3p*RSp563e*RS356pe - 1./36.*m55*RN3p*RSp463e*RS346pe - 1./
         108.*m55*RN3p*RSp463e*RS356pe + 1./108.*m55*RN3p*RSp561e*
         RS346pe + 1./108.*m55*RN3p*RSp561e*RS356pe + 1./36.*m55*RN3p*
         RSp461e*RS346pe + 1./108.*m55*RN3p*RSp461e*RS356pe + 1./108.*
         m55*RN1p*RS146pe*RSp563e + 1./36.*m55*RN1p*RS146pe*RSp463e + 1.
         /108.*m55*RN1p*RS156pe*RSp563e + 1./108.*m55*RN1p*RS156pe*
         RSp463e - 1./108.*m55*RN1p*RSp561e*RS146pe - 1./108.*m55*RN1p*
         RSp561e*RS156pe - 1./36.*m55*RN1p*RSp461e*RS146pe - 1./108.*
         m55*RN1p*RSp461e*RS156pe - 1./54.*m45*RN3p*RSp563e*RS346pe + 1.
         /54.*m45*RN3p*RSp563e*RS356pe + 1./54.*m45*RN3p*RSp463e*
         RS346pe );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./54.*m45*RN3p*
         RSp463e*RS356pe + 1./54.*m45*RN3p*RSp561e*RS346pe - 1./54.*m45
         *RN3p*RSp561e*RS356pe - 1./54.*m45*RN3p*RSp461e*RS346pe + 1./
         54.*m45*RN3p*RSp461e*RS356pe + 1./54.*m45*RN1p*RS146pe*RSp563e
          - 1./54.*m45*RN1p*RS146pe*RSp463e - 1./54.*m45*RN1p*RS156pe*
         RSp563e + 1./54.*m45*RN1p*RS156pe*RSp463e - 1./54.*m45*RN1p*
         RSp561e*RS146pe + 1./54.*m45*RN1p*RSp561e*RS156pe + 1./54.*m45
         *RN1p*RSp461e*RS146pe - 1./54.*m45*RN1p*RSp461e*RS156pe - 1./
         108.*m44*RN3p*RSp563e*RS346pe - 1./36.*m44*RN3p*RSp563e*
         RS356pe - 1./108.*m44*RN3p*RSp463e*RS346pe - 1./108.*m44*RN3p*
         RSp463e*RS356pe + 1./108.*m44*RN3p*RSp561e*RS346pe + 1./36.*
         m44*RN3p*RSp561e*RS356pe + 1./108.*m44*RN3p*RSp461e*RS346pe + 
         1./108.*m44*RN3p*RSp461e*RS356pe + 1./108.*m44*RN1p*RS146pe*
         RSp563e + 1./108.*m44*RN1p*RS146pe*RSp463e + 1./36.*m44*RN1p*
         RS156pe*RSp563e + 1./108.*m44*RN1p*RS156pe*RSp463e - 1./108.*
         m44*RN1p*RSp561e*RS146pe );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./36.*m44*RN1p*
         RSp561e*RS156pe - 1./108.*m44*RN1p*RSp461e*RS146pe - 1./108.*
         m44*RN1p*RSp461e*RS156pe - 1./54.*m33*RN3p*RSp563e*RS346pe + 1.
         /54.*m33*RN3p*RSp563e*RS356pe + 1./54.*m33*RN3p*RSp463e*
         RS346pe - 1./54.*m33*RN3p*RSp463e*RS356pe + 1./54.*m33*RN3p*
         RSp561e*RS346pe - 1./54.*m33*RN3p*RSp561e*RS356pe - 1./54.*m33
         *RN3p*RSp461e*RS346pe + 1./54.*m33*RN3p*RSp461e*RS356pe + 1./
         54.*m11*RN1p*RS146pe*RSp563e - 1./54.*m11*RN1p*RS146pe*RSp463e
          - 1./54.*m11*RN1p*RS156pe*RSp563e + 1./54.*m11*RN1p*RS156pe*
         RSp463e - 1./54.*m11*RN1p*RSp561e*RS146pe + 1./54.*m11*RN1p*
         RSp561e*RS156pe + 1./54.*m11*RN1p*RSp461e*RS146pe - 1./54.*m11
         *RN1p*RSp461e*RS156pe + 1./162.*RN(mp2,me2)*me2*RSp563e*
         RSe563p - 1./162.*RN(mp2,me2)*me2*RSp563e*RSe463p - 1./162.*
         RN(mp2,me2)*me2*RSp463e*RSe563p + 1./162.*RN(mp2,me2)*me2*
         RSp463e*RSe463p - 1./162.*RN(mp2,me2)*me2*RSe561p*RSp563e + 1./
         162.*RN(mp2,me2)*me2*RSe561p*RSp463e );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./162.*RN(mp2,me2)*
         me2*RSe461p*RSp563e - 1./162.*RN(mp2,me2)*me2*RSe461p*RSp463e
          - 1./162.*RN(mp2,me2)*me2*RSp561e*RSe563p + 1./162.*RN(mp2,
         me2)*me2*RSp561e*RSe463p + 1./162.*RN(mp2,me2)*me2*RSp561e*
         RSe561p - 1./162.*RN(mp2,me2)*me2*RSp561e*RSe461p + 1./162.*
         RN(mp2,me2)*me2*RSp461e*RSe563p - 1./162.*RN(mp2,me2)*me2*
         RSp461e*RSe463p - 1./162.*RN(mp2,me2)*me2*RSp461e*RSe561p + 1./
         162.*RN(mp2,me2)*me2*RSp461e*RSe461p - 2./81.*RN(mp2,me2)*mp2*
         RSp563e*RSe563p + 2./81.*RN(mp2,me2)*mp2*RSp563e*RSe463p + 2./
         81.*RN(mp2,me2)*mp2*RSp463e*RSe563p - 2./81.*RN(mp2,me2)*mp2*
         RSp463e*RSe463p + 2./81.*RN(mp2,me2)*mp2*RSe561p*RSp563e - 2./
         81.*RN(mp2,me2)*mp2*RSe561p*RSp463e - 2./81.*RN(mp2,me2)*mp2*
         RSe461p*RSp563e + 2./81.*RN(mp2,me2)*mp2*RSe461p*RSp463e + 2./
         81.*RN(mp2,me2)*mp2*RSp561e*RSe563p - 2./81.*RN(mp2,me2)*mp2*
         RSp561e*RSe463p - 2./81.*RN(mp2,me2)*mp2*RSp561e*RSe561p + 2./
         81.*RN(mp2,me2)*mp2*RSp561e*RSe461p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 2./81.*RN(mp2,me2)
         *mp2*RSp461e*RSe563p + 2./81.*RN(mp2,me2)*mp2*RSp461e*RSe463p
          + 2./81.*RN(mp2,me2)*mp2*RSp461e*RSe561p - 2./81.*RN(mp2,me2)
         *mp2*RSp461e*RSe461p + 1./108.*RN(mp2,me2)*m55*RSp563e*RSe563p
          + 1./108.*RN(mp2,me2)*m55*RSp563e*RSe463p + 1./108.*RN(mp2,
         me2)*m55*RSp463e*RSe563p + 1./36.*RN(mp2,me2)*m55*RSp463e*
         RSe463p - 1./108.*RN(mp2,me2)*m55*RSe561p*RSp563e - 1./108.*
         RN(mp2,me2)*m55*RSe561p*RSp463e - 1./108.*RN(mp2,me2)*m55*
         RSe461p*RSp563e - 1./36.*RN(mp2,me2)*m55*RSe461p*RSp463e - 1./
         108.*RN(mp2,me2)*m55*RSp561e*RSe563p - 1./108.*RN(mp2,me2)*m55
         *RSp561e*RSe463p + 1./108.*RN(mp2,me2)*m55*RSp561e*RSe561p + 1.
         /108.*RN(mp2,me2)*m55*RSp561e*RSe461p - 1./108.*RN(mp2,me2)*
         m55*RSp461e*RSe563p - 1./36.*RN(mp2,me2)*m55*RSp461e*RSe463p
          + 1./108.*RN(mp2,me2)*m55*RSp461e*RSe561p + 1./36.*RN(mp2,me2
         )*m55*RSp461e*RSe461p - 1./54.*RN(mp2,me2)*m45*RSp563e*RSe563p
          + 1./54.*RN(mp2,me2)*m45*RSp563e*RSe463p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * ( 1./54.*RN(mp2,me2)*
         m45*RSp463e*RSe563p - 1./54.*RN(mp2,me2)*m45*RSp463e*RSe463p
          + 1./54.*RN(mp2,me2)*m45*RSe561p*RSp563e - 1./54.*RN(mp2,me2)
         *m45*RSe561p*RSp463e - 1./54.*RN(mp2,me2)*m45*RSe461p*RSp563e
          + 1./54.*RN(mp2,me2)*m45*RSe461p*RSp463e + 1./54.*RN(mp2,me2)
         *m45*RSp561e*RSe563p - 1./54.*RN(mp2,me2)*m45*RSp561e*RSe463p
          - 1./54.*RN(mp2,me2)*m45*RSp561e*RSe561p + 1./54.*RN(mp2,me2)
         *m45*RSp561e*RSe461p - 1./54.*RN(mp2,me2)*m45*RSp461e*RSe563p
          + 1./54.*RN(mp2,me2)*m45*RSp461e*RSe463p + 1./54.*RN(mp2,me2)
         *m45*RSp461e*RSe561p - 1./54.*RN(mp2,me2)*m45*RSp461e*RSe461p
          + 1./36.*RN(mp2,me2)*m44*RSp563e*RSe563p + 1./108.*RN(mp2,me2
         )*m44*RSp563e*RSe463p + 1./108.*RN(mp2,me2)*m44*RSp463e*
         RSe563p + 1./108.*RN(mp2,me2)*m44*RSp463e*RSe463p - 1./36.*RN(
         mp2,me2)*m44*RSe561p*RSp563e - 1./108.*RN(mp2,me2)*m44*RSe561p
         *RSp463e - 1./108.*RN(mp2,me2)*m44*RSe461p*RSp563e - 1./108.*
         RN(mp2,me2)*m44*RSe461p*RSp463e );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m45,L) * (  - 1./36.*RN(mp2,me2)
         *m44*RSp561e*RSe563p - 1./108.*RN(mp2,me2)*m44*RSp561e*RSe463p
          + 1./36.*RN(mp2,me2)*m44*RSp561e*RSe561p + 1./108.*RN(mp2,me2
         )*m44*RSp561e*RSe461p - 1./108.*RN(mp2,me2)*m44*RSp461e*
         RSe563p - 1./108.*RN(mp2,me2)*m44*RSp461e*RSe463p + 1./108.*
         RN(mp2,me2)*m44*RSp461e*RSe561p + 1./108.*RN(mp2,me2)*m44*
         RSp461e*RSe461p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./54.*RSp563e*
         RS345pe - 1./54.*RSp563e*RS356pe + 1./162.*RSp563e*RSe563p - 1.
         /162.*RSp563e*RSe453p - 1./108.*pow(RSp563e,2) - 1./54.*
         RSp453e*RS345pe + 1./54.*RSp453e*RS356pe - 1./162.*RSp453e*
         RSe563p + 1./162.*RSp453e*RSe453p + 1./54.*RSp453e*RSp563e - 1.
         /108.*pow(RSp453e,2) - 1./54.*RS145pe*RSp563e + 1./54.*RS145pe
         *RSp453e + 1./54.*RS156pe*RSp563e - 1./54.*RS156pe*RSp453e - 1.
         /162.*RSe561p*RSp563e + 1./162.*RSe561p*RSp453e + 1./162.*
         RSe451p*RSp563e - 1./162.*RSe451p*RSp453e - 1./54.*RSp561e*
         RS345pe + 1./54.*RSp561e*RS356pe - 1./162.*RSp561e*RSe563p + 1.
         /162.*RSp561e*RSe453p + 1./54.*RSp561e*RSp563e - 1./54.*
         RSp561e*RSp453e + 1./54.*RSp561e*RS145pe - 1./54.*RSp561e*
         RS156pe + 1./162.*RSp561e*RSe561p - 1./162.*RSp561e*RSe451p - 
         1./108.*pow(RSp561e,2) + 1./54.*RSp451e*RS345pe - 1./54.*
         RSp451e*RS356pe + 1./162.*RSp451e*RSe563p - 1./162.*RSp451e*
         RSe453p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./54.*RSp451e*
         RSp563e + 1./54.*RSp451e*RSp453e - 1./54.*RSp451e*RS145pe + 1./
         54.*RSp451e*RS156pe - 1./162.*RSp451e*RSe561p + 1./162.*
         RSp451e*RSe451p + 1./54.*RSp451e*RSp561e - 1./108.*pow(
         RSp451e,2) - 1./108.*m66*RN3p*RSp563e*RS345pe - 1./108.*m66*
         RN3p*RSp563e*RS356pe - 1./36.*m66*RN3p*RSp453e*RS345pe - 1./
         108.*m66*RN3p*RSp453e*RS356pe + 1./108.*m66*RN3p*RSp561e*
         RS345pe + 1./108.*m66*RN3p*RSp561e*RS356pe + 1./36.*m66*RN3p*
         RSp451e*RS345pe + 1./108.*m66*RN3p*RSp451e*RS356pe + 1./108.*
         m66*RN1p*RS145pe*RSp563e + 1./36.*m66*RN1p*RS145pe*RSp453e + 1.
         /108.*m66*RN1p*RS156pe*RSp563e + 1./108.*m66*RN1p*RS156pe*
         RSp453e - 1./108.*m66*RN1p*RSp561e*RS145pe - 1./108.*m66*RN1p*
         RSp561e*RS156pe - 1./36.*m66*RN1p*RSp451e*RS145pe - 1./108.*
         m66*RN1p*RSp451e*RS156pe - 1./54.*m46*RN3p*RSp563e*RS345pe + 1.
         /54.*m46*RN3p*RSp563e*RS356pe + 1./54.*m46*RN3p*RSp453e*
         RS345pe );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./54.*m46*RN3p*
         RSp453e*RS356pe + 1./54.*m46*RN3p*RSp561e*RS345pe - 1./54.*m46
         *RN3p*RSp561e*RS356pe - 1./54.*m46*RN3p*RSp451e*RS345pe + 1./
         54.*m46*RN3p*RSp451e*RS356pe + 1./54.*m46*RN1p*RS145pe*RSp563e
          - 1./54.*m46*RN1p*RS145pe*RSp453e - 1./54.*m46*RN1p*RS156pe*
         RSp563e + 1./54.*m46*RN1p*RS156pe*RSp453e - 1./54.*m46*RN1p*
         RSp561e*RS145pe + 1./54.*m46*RN1p*RSp561e*RS156pe + 1./54.*m46
         *RN1p*RSp451e*RS145pe - 1./54.*m46*RN1p*RSp451e*RS156pe - 1./
         108.*m44*RN3p*RSp563e*RS345pe - 1./36.*m44*RN3p*RSp563e*
         RS356pe - 1./108.*m44*RN3p*RSp453e*RS345pe - 1./108.*m44*RN3p*
         RSp453e*RS356pe + 1./108.*m44*RN3p*RSp561e*RS345pe + 1./36.*
         m44*RN3p*RSp561e*RS356pe + 1./108.*m44*RN3p*RSp451e*RS345pe + 
         1./108.*m44*RN3p*RSp451e*RS356pe + 1./108.*m44*RN1p*RS145pe*
         RSp563e + 1./108.*m44*RN1p*RS145pe*RSp453e + 1./36.*m44*RN1p*
         RS156pe*RSp563e + 1./108.*m44*RN1p*RS156pe*RSp453e - 1./108.*
         m44*RN1p*RSp561e*RS145pe );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./36.*m44*RN1p*
         RSp561e*RS156pe - 1./108.*m44*RN1p*RSp451e*RS145pe - 1./108.*
         m44*RN1p*RSp451e*RS156pe - 1./54.*m33*RN3p*RSp563e*RS345pe + 1.
         /54.*m33*RN3p*RSp563e*RS356pe + 1./54.*m33*RN3p*RSp453e*
         RS345pe - 1./54.*m33*RN3p*RSp453e*RS356pe + 1./54.*m33*RN3p*
         RSp561e*RS345pe - 1./54.*m33*RN3p*RSp561e*RS356pe - 1./54.*m33
         *RN3p*RSp451e*RS345pe + 1./54.*m33*RN3p*RSp451e*RS356pe + 1./
         54.*m11*RN1p*RS145pe*RSp563e - 1./54.*m11*RN1p*RS145pe*RSp453e
          - 1./54.*m11*RN1p*RS156pe*RSp563e + 1./54.*m11*RN1p*RS156pe*
         RSp453e - 1./54.*m11*RN1p*RSp561e*RS145pe + 1./54.*m11*RN1p*
         RSp561e*RS156pe + 1./54.*m11*RN1p*RSp451e*RS145pe - 1./54.*m11
         *RN1p*RSp451e*RS156pe + 1./162.*RN(mp2,me2)*me2*RSp563e*
         RSe563p - 1./162.*RN(mp2,me2)*me2*RSp563e*RSe453p - 1./162.*
         RN(mp2,me2)*me2*RSp453e*RSe563p + 1./162.*RN(mp2,me2)*me2*
         RSp453e*RSe453p - 1./162.*RN(mp2,me2)*me2*RSe561p*RSp563e + 1./
         162.*RN(mp2,me2)*me2*RSe561p*RSp453e );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./162.*RN(mp2,me2)*
         me2*RSe451p*RSp563e - 1./162.*RN(mp2,me2)*me2*RSe451p*RSp453e
          - 1./162.*RN(mp2,me2)*me2*RSp561e*RSe563p + 1./162.*RN(mp2,
         me2)*me2*RSp561e*RSe453p + 1./162.*RN(mp2,me2)*me2*RSp561e*
         RSe561p - 1./162.*RN(mp2,me2)*me2*RSp561e*RSe451p + 1./162.*
         RN(mp2,me2)*me2*RSp451e*RSe563p - 1./162.*RN(mp2,me2)*me2*
         RSp451e*RSe453p - 1./162.*RN(mp2,me2)*me2*RSp451e*RSe561p + 1./
         162.*RN(mp2,me2)*me2*RSp451e*RSe451p - 2./81.*RN(mp2,me2)*mp2*
         RSp563e*RSe563p + 2./81.*RN(mp2,me2)*mp2*RSp563e*RSe453p + 2./
         81.*RN(mp2,me2)*mp2*RSp453e*RSe563p - 2./81.*RN(mp2,me2)*mp2*
         RSp453e*RSe453p + 2./81.*RN(mp2,me2)*mp2*RSe561p*RSp563e - 2./
         81.*RN(mp2,me2)*mp2*RSe561p*RSp453e - 2./81.*RN(mp2,me2)*mp2*
         RSe451p*RSp563e + 2./81.*RN(mp2,me2)*mp2*RSe451p*RSp453e + 2./
         81.*RN(mp2,me2)*mp2*RSp561e*RSe563p - 2./81.*RN(mp2,me2)*mp2*
         RSp561e*RSe453p - 2./81.*RN(mp2,me2)*mp2*RSp561e*RSe561p + 2./
         81.*RN(mp2,me2)*mp2*RSp561e*RSe451p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 2./81.*RN(mp2,me2)
         *mp2*RSp451e*RSe563p + 2./81.*RN(mp2,me2)*mp2*RSp451e*RSe453p
          + 2./81.*RN(mp2,me2)*mp2*RSp451e*RSe561p - 2./81.*RN(mp2,me2)
         *mp2*RSp451e*RSe451p + 1./108.*RN(mp2,me2)*m66*RSp563e*RSe563p
          + 1./108.*RN(mp2,me2)*m66*RSp563e*RSe453p + 1./108.*RN(mp2,
         me2)*m66*RSp453e*RSe563p + 1./36.*RN(mp2,me2)*m66*RSp453e*
         RSe453p - 1./108.*RN(mp2,me2)*m66*RSe561p*RSp563e - 1./108.*
         RN(mp2,me2)*m66*RSe561p*RSp453e - 1./108.*RN(mp2,me2)*m66*
         RSe451p*RSp563e - 1./36.*RN(mp2,me2)*m66*RSe451p*RSp453e - 1./
         108.*RN(mp2,me2)*m66*RSp561e*RSe563p - 1./108.*RN(mp2,me2)*m66
         *RSp561e*RSe453p + 1./108.*RN(mp2,me2)*m66*RSp561e*RSe561p + 1.
         /108.*RN(mp2,me2)*m66*RSp561e*RSe451p - 1./108.*RN(mp2,me2)*
         m66*RSp451e*RSe563p - 1./36.*RN(mp2,me2)*m66*RSp451e*RSe453p
          + 1./108.*RN(mp2,me2)*m66*RSp451e*RSe561p + 1./36.*RN(mp2,me2
         )*m66*RSp451e*RSe451p - 1./54.*RN(mp2,me2)*m46*RSp563e*RSe563p
          + 1./54.*RN(mp2,me2)*m46*RSp563e*RSe453p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * ( 1./54.*RN(mp2,me2)*
         m46*RSp453e*RSe563p - 1./54.*RN(mp2,me2)*m46*RSp453e*RSe453p
          + 1./54.*RN(mp2,me2)*m46*RSe561p*RSp563e - 1./54.*RN(mp2,me2)
         *m46*RSe561p*RSp453e - 1./54.*RN(mp2,me2)*m46*RSe451p*RSp563e
          + 1./54.*RN(mp2,me2)*m46*RSe451p*RSp453e + 1./54.*RN(mp2,me2)
         *m46*RSp561e*RSe563p - 1./54.*RN(mp2,me2)*m46*RSp561e*RSe453p
          - 1./54.*RN(mp2,me2)*m46*RSp561e*RSe561p + 1./54.*RN(mp2,me2)
         *m46*RSp561e*RSe451p - 1./54.*RN(mp2,me2)*m46*RSp451e*RSe563p
          + 1./54.*RN(mp2,me2)*m46*RSp451e*RSe453p + 1./54.*RN(mp2,me2)
         *m46*RSp451e*RSe561p - 1./54.*RN(mp2,me2)*m46*RSp451e*RSe451p
          + 1./36.*RN(mp2,me2)*m44*RSp563e*RSe563p + 1./108.*RN(mp2,me2
         )*m44*RSp563e*RSe453p + 1./108.*RN(mp2,me2)*m44*RSp453e*
         RSe563p + 1./108.*RN(mp2,me2)*m44*RSp453e*RSe453p - 1./36.*RN(
         mp2,me2)*m44*RSe561p*RSp563e - 1./108.*RN(mp2,me2)*m44*RSe561p
         *RSp453e - 1./108.*RN(mp2,me2)*m44*RSe451p*RSp563e - 1./108.*
         RN(mp2,me2)*m44*RSe451p*RSp453e );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m46,L) * (  - 1./36.*RN(mp2,me2)
         *m44*RSp561e*RSe563p - 1./108.*RN(mp2,me2)*m44*RSp561e*RSe453p
          + 1./36.*RN(mp2,me2)*m44*RSp561e*RSe561p + 1./108.*RN(mp2,me2
         )*m44*RSp561e*RSe451p - 1./108.*RN(mp2,me2)*m44*RSp451e*
         RSe563p - 1./108.*RN(mp2,me2)*m44*RSp451e*RSe453p + 1./108.*
         RN(mp2,me2)*m44*RSp451e*RSe561p + 1./108.*RN(mp2,me2)*m44*
         RSp451e*RSe451p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m55,L) * (  - 1./18.*m55*RN3p*
         RSp463e*RS346pe + 1./54.*m55*RN3p*RS546pe*RSp463e*RS346pe + 1./
         18.*m55*RN3p*RSp461e*RS346pe - 1./54.*m55*RN3p*RSp461e*RS546pe
         *RS346pe + 1./18.*m55*RN1p*RS146pe*RSp463e - 1./54.*m55*RN1p*
         RS546pe*RS146pe*RSp463e - 1./18.*m55*RN1p*RSp461e*RS146pe + 1./
         54.*m55*RN1p*RSp461e*RS546pe*RS146pe + 1./18.*RN(mp2,me2)*m55*
         RSp463e*RSe463p - 1./54.*RN(mp2,me2)*m55*RS546pe*RSp463e*
         RSe463p - 1./18.*RN(mp2,me2)*m55*RSe461p*RSp463e + 1./54.*RN(
         mp2,me2)*m55*RSe461p*RS546pe*RSp463e - 1./18.*RN(mp2,me2)*m55*
         RSp461e*RSe463p + 1./54.*RN(mp2,me2)*m55*RSp461e*RS546pe*
         RSe463p + 1./18.*RN(mp2,me2)*m55*RSp461e*RSe461p - 1./54.*RN(
         mp2,me2)*m55*RSp461e*RSe461p*RS546pe );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./54.*RSp463e*
         RS345pe - 1./54.*RSp463e*RS346pe + 1./162.*RSp463e*RSe463p - 1.
         /162.*RSp463e*RSe453p - 1./108.*pow(RSp463e,2) - 1./54.*
         RSp453e*RS345pe + 1./54.*RSp453e*RS346pe - 1./162.*RSp453e*
         RSe463p + 1./162.*RSp453e*RSe453p + 1./54.*RSp453e*RSp463e - 1.
         /108.*pow(RSp453e,2) - 1./54.*RS145pe*RSp463e + 1./54.*RS145pe
         *RSp453e + 1./54.*RS146pe*RSp463e - 1./54.*RS146pe*RSp453e - 1.
         /162.*RSe461p*RSp463e + 1./162.*RSe461p*RSp453e + 1./162.*
         RSe451p*RSp463e - 1./162.*RSe451p*RSp453e - 1./54.*RSp461e*
         RS345pe + 1./54.*RSp461e*RS346pe - 1./162.*RSp461e*RSe463p + 1.
         /162.*RSp461e*RSe453p + 1./54.*RSp461e*RSp463e - 1./54.*
         RSp461e*RSp453e + 1./54.*RSp461e*RS145pe - 1./54.*RSp461e*
         RS146pe + 1./162.*RSp461e*RSe461p - 1./162.*RSp461e*RSe451p - 
         1./108.*pow(RSp461e,2) + 1./54.*RSp451e*RS345pe - 1./54.*
         RSp451e*RS346pe + 1./162.*RSp451e*RSe463p - 1./162.*RSp451e*
         RSe453p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./54.*RSp451e*
         RSp463e + 1./54.*RSp451e*RSp453e - 1./54.*RSp451e*RS145pe + 1./
         54.*RSp451e*RS146pe - 1./162.*RSp451e*RSe461p + 1./162.*
         RSp451e*RSe451p + 1./54.*RSp451e*RSp461e - 1./108.*pow(
         RSp451e,2) - 1./108.*m66*RN3p*RSp463e*RS345pe - 1./108.*m66*
         RN3p*RSp463e*RS346pe - 1./36.*m66*RN3p*RSp453e*RS345pe - 1./
         108.*m66*RN3p*RSp453e*RS346pe + 1./108.*m66*RN3p*RSp461e*
         RS345pe + 1./108.*m66*RN3p*RSp461e*RS346pe + 1./36.*m66*RN3p*
         RSp451e*RS345pe + 1./108.*m66*RN3p*RSp451e*RS346pe + 1./108.*
         m66*RN1p*RS145pe*RSp463e + 1./36.*m66*RN1p*RS145pe*RSp453e + 1.
         /108.*m66*RN1p*RS146pe*RSp463e + 1./108.*m66*RN1p*RS146pe*
         RSp453e - 1./108.*m66*RN1p*RSp461e*RS145pe - 1./108.*m66*RN1p*
         RSp461e*RS146pe - 1./36.*m66*RN1p*RSp451e*RS145pe - 1./108.*
         m66*RN1p*RSp451e*RS146pe - 1./54.*m56*RN3p*RSp463e*RS345pe + 1.
         /54.*m56*RN3p*RSp463e*RS346pe + 1./54.*m56*RN3p*RSp453e*
         RS345pe );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./54.*m56*RN3p*
         RSp453e*RS346pe + 1./54.*m56*RN3p*RSp461e*RS345pe - 1./54.*m56
         *RN3p*RSp461e*RS346pe - 1./54.*m56*RN3p*RSp451e*RS345pe + 1./
         54.*m56*RN3p*RSp451e*RS346pe + 1./54.*m56*RN1p*RS145pe*RSp463e
          - 1./54.*m56*RN1p*RS145pe*RSp453e - 1./54.*m56*RN1p*RS146pe*
         RSp463e + 1./54.*m56*RN1p*RS146pe*RSp453e - 1./54.*m56*RN1p*
         RSp461e*RS145pe + 1./54.*m56*RN1p*RSp461e*RS146pe + 1./54.*m56
         *RN1p*RSp451e*RS145pe - 1./54.*m56*RN1p*RSp451e*RS146pe - 1./
         108.*m55*RN3p*RSp463e*RS345pe - 1./36.*m55*RN3p*RSp463e*
         RS346pe - 1./108.*m55*RN3p*RSp453e*RS345pe - 1./108.*m55*RN3p*
         RSp453e*RS346pe + 1./108.*m55*RN3p*RSp461e*RS345pe + 1./36.*
         m55*RN3p*RSp461e*RS346pe + 1./108.*m55*RN3p*RSp451e*RS345pe + 
         1./108.*m55*RN3p*RSp451e*RS346pe + 1./108.*m55*RN1p*RS145pe*
         RSp463e + 1./108.*m55*RN1p*RS145pe*RSp453e + 1./36.*m55*RN1p*
         RS146pe*RSp463e + 1./108.*m55*RN1p*RS146pe*RSp453e - 1./108.*
         m55*RN1p*RSp461e*RS145pe );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./36.*m55*RN1p*
         RSp461e*RS146pe - 1./108.*m55*RN1p*RSp451e*RS145pe - 1./108.*
         m55*RN1p*RSp451e*RS146pe - 1./54.*m33*RN3p*RSp463e*RS345pe + 1.
         /54.*m33*RN3p*RSp463e*RS346pe + 1./54.*m33*RN3p*RSp453e*
         RS345pe - 1./54.*m33*RN3p*RSp453e*RS346pe + 1./54.*m33*RN3p*
         RSp461e*RS345pe - 1./54.*m33*RN3p*RSp461e*RS346pe - 1./54.*m33
         *RN3p*RSp451e*RS345pe + 1./54.*m33*RN3p*RSp451e*RS346pe + 1./
         54.*m11*RN1p*RS145pe*RSp463e - 1./54.*m11*RN1p*RS145pe*RSp453e
          - 1./54.*m11*RN1p*RS146pe*RSp463e + 1./54.*m11*RN1p*RS146pe*
         RSp453e - 1./54.*m11*RN1p*RSp461e*RS145pe + 1./54.*m11*RN1p*
         RSp461e*RS146pe + 1./54.*m11*RN1p*RSp451e*RS145pe - 1./54.*m11
         *RN1p*RSp451e*RS146pe + 1./162.*RN(mp2,me2)*me2*RSp463e*
         RSe463p - 1./162.*RN(mp2,me2)*me2*RSp463e*RSe453p - 1./162.*
         RN(mp2,me2)*me2*RSp453e*RSe463p + 1./162.*RN(mp2,me2)*me2*
         RSp453e*RSe453p - 1./162.*RN(mp2,me2)*me2*RSe461p*RSp463e + 1./
         162.*RN(mp2,me2)*me2*RSe461p*RSp453e );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./162.*RN(mp2,me2)*
         me2*RSe451p*RSp463e - 1./162.*RN(mp2,me2)*me2*RSe451p*RSp453e
          - 1./162.*RN(mp2,me2)*me2*RSp461e*RSe463p + 1./162.*RN(mp2,
         me2)*me2*RSp461e*RSe453p + 1./162.*RN(mp2,me2)*me2*RSp461e*
         RSe461p - 1./162.*RN(mp2,me2)*me2*RSp461e*RSe451p + 1./162.*
         RN(mp2,me2)*me2*RSp451e*RSe463p - 1./162.*RN(mp2,me2)*me2*
         RSp451e*RSe453p - 1./162.*RN(mp2,me2)*me2*RSp451e*RSe461p + 1./
         162.*RN(mp2,me2)*me2*RSp451e*RSe451p - 2./81.*RN(mp2,me2)*mp2*
         RSp463e*RSe463p + 2./81.*RN(mp2,me2)*mp2*RSp463e*RSe453p + 2./
         81.*RN(mp2,me2)*mp2*RSp453e*RSe463p - 2./81.*RN(mp2,me2)*mp2*
         RSp453e*RSe453p + 2./81.*RN(mp2,me2)*mp2*RSe461p*RSp463e - 2./
         81.*RN(mp2,me2)*mp2*RSe461p*RSp453e - 2./81.*RN(mp2,me2)*mp2*
         RSe451p*RSp463e + 2./81.*RN(mp2,me2)*mp2*RSe451p*RSp453e + 2./
         81.*RN(mp2,me2)*mp2*RSp461e*RSe463p - 2./81.*RN(mp2,me2)*mp2*
         RSp461e*RSe453p - 2./81.*RN(mp2,me2)*mp2*RSp461e*RSe461p + 2./
         81.*RN(mp2,me2)*mp2*RSp461e*RSe451p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 2./81.*RN(mp2,me2)
         *mp2*RSp451e*RSe463p + 2./81.*RN(mp2,me2)*mp2*RSp451e*RSe453p
          + 2./81.*RN(mp2,me2)*mp2*RSp451e*RSe461p - 2./81.*RN(mp2,me2)
         *mp2*RSp451e*RSe451p + 1./108.*RN(mp2,me2)*m66*RSp463e*RSe463p
          + 1./108.*RN(mp2,me2)*m66*RSp463e*RSe453p + 1./108.*RN(mp2,
         me2)*m66*RSp453e*RSe463p + 1./36.*RN(mp2,me2)*m66*RSp453e*
         RSe453p - 1./108.*RN(mp2,me2)*m66*RSe461p*RSp463e - 1./108.*
         RN(mp2,me2)*m66*RSe461p*RSp453e - 1./108.*RN(mp2,me2)*m66*
         RSe451p*RSp463e - 1./36.*RN(mp2,me2)*m66*RSe451p*RSp453e - 1./
         108.*RN(mp2,me2)*m66*RSp461e*RSe463p - 1./108.*RN(mp2,me2)*m66
         *RSp461e*RSe453p + 1./108.*RN(mp2,me2)*m66*RSp461e*RSe461p + 1.
         /108.*RN(mp2,me2)*m66*RSp461e*RSe451p - 1./108.*RN(mp2,me2)*
         m66*RSp451e*RSe463p - 1./36.*RN(mp2,me2)*m66*RSp451e*RSe453p
          + 1./108.*RN(mp2,me2)*m66*RSp451e*RSe461p + 1./36.*RN(mp2,me2
         )*m66*RSp451e*RSe451p - 1./54.*RN(mp2,me2)*m56*RSp463e*RSe463p
          + 1./54.*RN(mp2,me2)*m56*RSp463e*RSe453p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * ( 1./54.*RN(mp2,me2)*
         m56*RSp453e*RSe463p - 1./54.*RN(mp2,me2)*m56*RSp453e*RSe453p
          + 1./54.*RN(mp2,me2)*m56*RSe461p*RSp463e - 1./54.*RN(mp2,me2)
         *m56*RSe461p*RSp453e - 1./54.*RN(mp2,me2)*m56*RSe451p*RSp463e
          + 1./54.*RN(mp2,me2)*m56*RSe451p*RSp453e + 1./54.*RN(mp2,me2)
         *m56*RSp461e*RSe463p - 1./54.*RN(mp2,me2)*m56*RSp461e*RSe453p
          - 1./54.*RN(mp2,me2)*m56*RSp461e*RSe461p + 1./54.*RN(mp2,me2)
         *m56*RSp461e*RSe451p - 1./54.*RN(mp2,me2)*m56*RSp451e*RSe463p
          + 1./54.*RN(mp2,me2)*m56*RSp451e*RSe453p + 1./54.*RN(mp2,me2)
         *m56*RSp451e*RSe461p - 1./54.*RN(mp2,me2)*m56*RSp451e*RSe451p
          + 1./36.*RN(mp2,me2)*m55*RSp463e*RSe463p + 1./108.*RN(mp2,me2
         )*m55*RSp463e*RSe453p + 1./108.*RN(mp2,me2)*m55*RSp453e*
         RSe463p + 1./108.*RN(mp2,me2)*m55*RSp453e*RSe453p - 1./36.*RN(
         mp2,me2)*m55*RSe461p*RSp463e - 1./108.*RN(mp2,me2)*m55*RSe461p
         *RSp453e - 1./108.*RN(mp2,me2)*m55*RSe451p*RSp463e - 1./108.*
         RN(mp2,me2)*m55*RSe451p*RSp453e );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m56,L) * (  - 1./36.*RN(mp2,me2)
         *m55*RSp461e*RSe463p - 1./108.*RN(mp2,me2)*m55*RSp461e*RSe453p
          + 1./36.*RN(mp2,me2)*m55*RSp461e*RSe461p + 1./108.*RN(mp2,me2
         )*m55*RSp461e*RSe451p - 1./108.*RN(mp2,me2)*m55*RSp451e*
         RSe463p - 1./108.*RN(mp2,me2)*m55*RSp451e*RSe453p + 1./108.*
         RN(mp2,me2)*m55*RSp451e*RSe461p + 1./108.*RN(mp2,me2)*m55*
         RSp451e*RSe451p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(m66,L) * (  - 1./18.*m66*RN3p*
         RSp453e*RS345pe + 1./54.*m66*RN3p*RS645pe*RSp453e*RS345pe + 1./
         18.*m66*RN3p*RSp451e*RS345pe - 1./54.*m66*RN3p*RSp451e*RS645pe
         *RS345pe + 1./18.*m66*RN1p*RS145pe*RSp453e - 1./54.*m66*RN1p*
         RS645pe*RS145pe*RSp453e - 1./18.*m66*RN1p*RSp451e*RS145pe + 1./
         54.*m66*RN1p*RSp451e*RS645pe*RS145pe + 1./18.*RN(mp2,me2)*m66*
         RSp453e*RSe453p - 1./54.*RN(mp2,me2)*m66*RS645pe*RSp453e*
         RSe453p - 1./18.*RN(mp2,me2)*m66*RSe451p*RSp453e + 1./54.*RN(
         mp2,me2)*m66*RSe451p*RS645pe*RSp453e - 1./18.*RN(mp2,me2)*m66*
         RSp451e*RSe453p + 1./54.*RN(mp2,me2)*m66*RSp451e*RS645pe*
         RSe453p + 1./18.*RN(mp2,me2)*m66*RSp451e*RSe451p - 1./54.*RN(
         mp2,me2)*m66*RSp451e*RSe451p*RS645pe );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(mp2,L) * ( 1./36.*pow(
         RSp45613e,2) - 1./36.*RSp45633e*RSp45613e + 1./144.*pow(
         RSp45633e,2) - 1./36.*RSp45611e*RSp45613e + 1./72.*RSp45611e*
         RSp45633e + 1./144.*pow(RSp45611e,2) + 1./54.*m66*RN3p*RSp456e
         *RSp453e*RS345pe - 1./54.*m66*RN3p*RSp456e*RSp451e*RS345pe - 1.
         /54.*m66*RN1p*RSp456e*RS145pe*RSp453e + 1./54.*m66*RN1p*
         RSp456e*RSp451e*RS145pe + 1./54.*m55*RN3p*RSp465e*RSp463e*
         RS346pe - 1./54.*m55*RN3p*RSp465e*RSp461e*RS346pe - 1./54.*m55
         *RN1p*RSp465e*RS146pe*RSp463e + 1./54.*m55*RN1p*RSp465e*
         RSp461e*RS146pe + 1./54.*m44*RN3p*RSp564e*RSp563e*RS356pe - 1./
         54.*m44*RN3p*RSp564e*RSp561e*RS356pe - 1./54.*m44*RN1p*RSp564e
         *RS156pe*RSp563e + 1./54.*m44*RN1p*RSp564e*RSp561e*RS156pe + 1.
         /18.*m33*RN3p*RSp45633e*RSp45613e - 1./18.*m33*RN3p*pow(
         RSp45633e,2) + 1./18.*m11*RN1p*RSp45611e*RSp45613e - 1./18.*
         m11*RN1p*pow(RSp45611e,2) - 1./54.*RN(mp2,me2)*m66*RSp456e*
         RSp453e*RSe453p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(mp2,L) * ( 1./54.*RN(mp2,me2)*
         m66*RSp456e*RSe451p*RSp453e + 1./54.*RN(mp2,me2)*m66*RSp456e*
         RSp451e*RSe453p - 1./54.*RN(mp2,me2)*m66*RSp456e*RSp451e*
         RSe451p - 1./54.*RN(mp2,me2)*m55*RSp465e*RSp463e*RSe463p + 1./
         54.*RN(mp2,me2)*m55*RSp465e*RSe461p*RSp463e + 1./54.*RN(mp2,
         me2)*m55*RSp465e*RSp461e*RSe463p - 1./54.*RN(mp2,me2)*m55*
         RSp465e*RSp461e*RSe461p - 1./54.*RN(mp2,me2)*m44*RSp564e*
         RSp563e*RSe563p + 1./54.*RN(mp2,me2)*m44*RSp564e*RSe561p*
         RSp563e + 1./54.*RN(mp2,me2)*m44*RSp564e*RSp561e*RSe563p - 1./
         54.*RN(mp2,me2)*m44*RSp564e*RSp561e*RSe561p );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(me2,L) * ( 1./36.*RSp45613e*
         RSe45613p - 1./72.*RSe45633p*RSp45613e - 1./72.*RSp45633e*
         RSe45613p + 1./144.*RSp45633e*RSe45633p - 1./72.*RSe45611p*
         RSp45613e + 1./144.*RSe45611p*RSp45633e - 1./72.*RSp45611e*
         RSe45613p + 1./144.*RSp45611e*RSe45633p + 1./144.*RSp45611e*
         RSe45611p + 1./54.*m66*RN3p*RSe456p*RSp453e*RS345pe - 1./54.*
         m66*RN3p*RSe456p*RSp451e*RS345pe - 1./54.*m66*RN1p*RSe456p*
         RS145pe*RSp453e + 1./54.*m66*RN1p*RSe456p*RSp451e*RS145pe + 1./
         54.*m55*RN3p*RSe465p*RSp463e*RS346pe - 1./54.*m55*RN3p*RSe465p
         *RSp461e*RS346pe - 1./54.*m55*RN1p*RSe465p*RS146pe*RSp463e + 1.
         /54.*m55*RN1p*RSe465p*RSp461e*RS146pe + 1./54.*m44*RN3p*
         RSe564p*RSp563e*RS356pe - 1./54.*m44*RN3p*RSe564p*RSp561e*
         RS356pe - 1./54.*m44*RN1p*RSe564p*RS156pe*RSp563e + 1./54.*m44
         *RN1p*RSe564p*RSp561e*RS156pe + 1./18.*m33*RN3p*RSe45633p*
         RSp45613e - 1./18.*m33*RN3p*RSp45633e*RSe45633p + 1./18.*m11*
         RN1p*RSe45611p*RSp45613e );

      F13p6x23RV +=  + Ab(mp2,mu2)*AbV(me2,L) * (  - 1./18.*m11*RN1p*
         RSp45611e*RSe45611p - 1./54.*RN(mp2,me2)*m66*RSe456p*RSp453e*
         RSe453p + 1./54.*RN(mp2,me2)*m66*RSe456p*RSe451p*RSp453e + 1./
         54.*RN(mp2,me2)*m66*RSe456p*RSp451e*RSe453p - 1./54.*RN(mp2,
         me2)*m66*RSe456p*RSp451e*RSe451p - 1./54.*RN(mp2,me2)*m55*
         RSe465p*RSp463e*RSe463p + 1./54.*RN(mp2,me2)*m55*RSe465p*
         RSe461p*RSp463e + 1./54.*RN(mp2,me2)*m55*RSe465p*RSp461e*
         RSe463p - 1./54.*RN(mp2,me2)*m55*RSe465p*RSp461e*RSe461p - 1./
         54.*RN(mp2,me2)*m44*RSe564p*RSp563e*RSe563p + 1./54.*RN(mp2,
         me2)*m44*RSe564p*RSe561p*RSp563e + 1./54.*RN(mp2,me2)*m44*
         RSe564p*RSp561e*RSe563p - 1./54.*RN(mp2,me2)*m44*RSe564p*
         RSp561e*RSe561p );

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(m11,m33,L) * ( 1./54.*m66*
         RSp456e*RS145pe*RS345pe + 1./54.*m55*RSp465e*RS146pe*RS346pe
          + 1./54.*m44*RSp564e*RS156pe*RS356pe - 1./18.*m33*RSp45633e*
         RS14563pe - 1./18.*m11*RSp45611e*RS34561pe );

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(m11,mp2,L) * ( 1./54.*m66*
         RSp456e*RS145pe*RSp453e - 1./54.*m66*RSp456e*RSp451e*RS145pe
          + 1./54.*m55*RSp465e*RS146pe*RSp463e - 1./54.*m55*RSp465e*
         RSp461e*RS146pe + 1./54.*m44*RSp564e*RS156pe*RSp563e - 1./54.*
         m44*RSp564e*RSp561e*RS156pe - 1./18.*m11*RSp45611e*RSp45613e
          + 1./18.*m11*pow(RSp45611e,2) );

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(m11,me2,L) * ( 1./54.*m66*
         RSp456e*RS145pe*RSe453p - 1./54.*m66*RSp456e*RSe451p*RS145pe
          + 1./54.*m55*RSp465e*RS146pe*RSe463p - 1./54.*m55*RSp465e*
         RSe461p*RS146pe + 1./54.*m44*RSp564e*RS156pe*RSe563p - 1./54.*
         m44*RSp564e*RSe561p*RS156pe - 1./18.*m11*RSp45611e*RSe45613p
          + 1./18.*m11*RSp45611e*RSe45611p );

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(m11,L) * (  - 1./72.*RD1456ep*
         RSp45613e + 1./144.*RD1456ep*RSp45633e + 1./144.*RSp45611e*
         RD1456ep - 1./108.*m66*RSp456e*pow(RS145pe,2) - 1./108.*m55*
         RSp465e*pow(RS146pe,2) - 1./108.*m44*RSp564e*pow(RS156pe,2) - 
         1./18.*m11*RSp45611e*RS14563pe + 1./18.*m11*RSp45611e*RC1456pe
          + 1./18.*m11*RN1p*RD1456ep*RSp45613e - 1./18.*m11*RN1p*
         RSp45611e*RD1456ep );

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(m14,L) * ( 1./72.*mp2*RSp45611e
          - 1./36.*mp2*RSp561e + 1./72.*mp2*RSp564e - 1./144.*m44*
         RSp45611e - 1./72.*m44*RSp561e - 1./48.*m44*RSp564e + 1./72.*
         m14*RSp45611e - 1./36.*m14*RSp561e + 1./72.*m14*RSp564e - 1./
         48.*m11*RSp45611e - 1./72.*m11*RSp561e - 1./144.*m11*RSp564e )
         ;

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(m15,L) * ( 1./72.*mp2*RSp45611e
          - 1./36.*mp2*RSp461e + 1./72.*mp2*RSp465e - 1./144.*m55*
         RSp45611e - 1./72.*m55*RSp461e - 1./48.*m55*RSp465e + 1./72.*
         m15*RSp45611e - 1./36.*m15*RSp461e + 1./72.*m15*RSp465e - 1./
         48.*m11*RSp45611e - 1./72.*m11*RSp461e - 1./144.*m11*RSp465e )
         ;

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(m16,L) * ( 1./72.*mp2*RSp45611e
          - 1./36.*mp2*RSp451e + 1./72.*mp2*RSp456e - 1./144.*m66*
         RSp45611e - 1./72.*m66*RSp451e - 1./48.*m66*RSp456e + 1./72.*
         m16*RSp45611e - 1./36.*m16*RSp451e + 1./72.*m16*RSp456e - 1./
         48.*m11*RSp45611e - 1./72.*m11*RSp451e - 1./144.*m11*RSp456e )
         ;

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(m33,mp2,L) * (  - 1./54.*m66*
         RSp456e*RSp453e*RS345pe + 1./54.*m66*RSp456e*RSp451e*RS345pe
          - 1./54.*m55*RSp465e*RSp463e*RS346pe + 1./54.*m55*RSp465e*
         RSp461e*RS346pe - 1./54.*m44*RSp564e*RSp563e*RS356pe + 1./54.*
         m44*RSp564e*RSp561e*RS356pe - 1./18.*m33*RSp45633e*RSp45613e
          + 1./18.*m33*pow(RSp45633e,2) );

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(m33,me2,L) * (  - 1./54.*m66*
         RSp456e*RSe453p*RS345pe + 1./54.*m66*RSp456e*RSe451p*RS345pe
          - 1./54.*m55*RSp465e*RSe463p*RS346pe + 1./54.*m55*RSp465e*
         RSe461p*RS346pe - 1./54.*m44*RSp564e*RSe563p*RS356pe + 1./54.*
         m44*RSp564e*RSe561p*RS356pe - 1./18.*m33*RSp45633e*RSe45613p
          + 1./18.*m33*RSp45633e*RSe45633p );

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(m33,L) * (  - 1./72.*RD3456ep*
         RSp45613e + 1./144.*RSp45633e*RD3456ep + 1./144.*RSp45611e*
         RD3456ep - 1./108.*m66*RSp456e*pow(RS345pe,2) - 1./108.*m55*
         RSp465e*pow(RS346pe,2) - 1./108.*m44*RSp564e*pow(RS356pe,2) - 
         1./18.*m33*RSp45633e*RS34561pe + 1./18.*m33*RSp45633e*RC3456pe
          + 1./18.*m33*RN3p*RD3456ep*RSp45613e - 1./18.*m33*RN3p*
         RSp45633e*RD3456ep );

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(m34,L) * ( 1./72.*mp2*RSp45633e
          - 1./36.*mp2*RSp563e + 1./72.*mp2*RSp564e - 1./144.*m44*
         RSp45633e - 1./72.*m44*RSp563e - 1./48.*m44*RSp564e + 1./72.*
         m34*RSp45633e - 1./36.*m34*RSp563e + 1./72.*m34*RSp564e - 1./
         48.*m33*RSp45633e - 1./72.*m33*RSp563e - 1./144.*m33*RSp564e )
         ;

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(m35,L) * ( 1./72.*mp2*RSp45633e
          - 1./36.*mp2*RSp463e + 1./72.*mp2*RSp465e - 1./144.*m55*
         RSp45633e - 1./72.*m55*RSp463e - 1./48.*m55*RSp465e + 1./72.*
         m35*RSp45633e - 1./36.*m35*RSp463e + 1./72.*m35*RSp465e - 1./
         48.*m33*RSp45633e - 1./72.*m33*RSp463e - 1./144.*m33*RSp465e )
         ;

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(m36,L) * ( 1./72.*mp2*RSp45633e
          - 1./36.*mp2*RSp453e + 1./72.*mp2*RSp456e - 1./144.*m66*
         RSp45633e - 1./72.*m66*RSp453e - 1./48.*m66*RSp456e + 1./72.*
         m36*RSp45633e - 1./36.*m36*RSp453e + 1./72.*m36*RSp456e - 1./
         48.*m33*RSp45633e - 1./72.*m33*RSp453e - 1./144.*m33*RSp456e )
         ;

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(mp2,me2,L) * (  - 2./81.*m66*
         RSp456e*RSp453e*RSe453p + 2./81.*m66*RSp456e*RSe451p*RSp453e
          + 2./81.*m66*RSp456e*RSp451e*RSe453p - 2./81.*m66*RSp456e*
         RSp451e*RSe451p - 2./81.*m55*RSp465e*RSp463e*RSe463p + 2./81.*
         m55*RSp465e*RSe461p*RSp463e + 2./81.*m55*RSp465e*RSp461e*
         RSe463p - 2./81.*m55*RSp465e*RSp461e*RSe461p - 2./81.*m44*
         RSp564e*RSp563e*RSe563p + 2./81.*m44*RSp564e*RSe561p*RSp563e
          + 2./81.*m44*RSp564e*RSp561e*RSe563p - 2./81.*m44*RSp564e*
         RSp561e*RSe561p );

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(mp2,L) * (  - 1./108.*m66*
         RSp456e*pow(RSp453e,2) + 1./54.*m66*RSp456e*RSp451e*RSp453e - 
         1./108.*m66*RSp456e*pow(RSp451e,2) - 1./108.*m55*RSp465e*pow(
         RSp463e,2) + 1./54.*m55*RSp465e*RSp461e*RSp463e - 1./108.*m55*
         RSp465e*pow(RSp461e,2) - 1./108.*m44*RSp564e*pow(RSp563e,2) + 
         1./54.*m44*RSp564e*RSp561e*RSp563e - 1./108.*m44*RSp564e*pow(
         RSp561e,2) );

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(me2,mp2,L) * ( 1./162.*m66*
         RSp456e*RSp453e*RSe453p - 1./162.*m66*RSp456e*RSe451p*RSp453e
          - 1./162.*m66*RSp456e*RSp451e*RSe453p + 1./162.*m66*RSp456e*
         RSp451e*RSe451p + 1./162.*m55*RSp465e*RSp463e*RSe463p - 1./162.
         *m55*RSp465e*RSe461p*RSp463e - 1./162.*m55*RSp465e*RSp461e*
         RSe463p + 1./162.*m55*RSp465e*RSp461e*RSe461p + 1./162.*m44*
         RSp564e*RSp563e*RSe563p - 1./162.*m44*RSp564e*RSe561p*RSp563e
          - 1./162.*m44*RSp564e*RSp561e*RSe563p + 1./162.*m44*RSp564e*
         RSp561e*RSe561p );

      F13p6x23RV +=  + Ab(mp2,mu2)*BbV(me2,L) * (  - 1./108.*m66*
         RSp456e*pow(RSe453p,2) + 1./54.*m66*RSp456e*RSe451p*RSe453p - 
         1./108.*m66*RSp456e*pow(RSe451p,2) - 1./108.*m55*RSp465e*pow(
         RSe463p,2) + 1./54.*m55*RSp465e*RSe461p*RSe463p - 1./108.*m55*
         RSp465e*pow(RSe461p,2) - 1./108.*m44*RSp564e*pow(RSe563p,2) + 
         1./54.*m44*RSp564e*RSe561p*RSe563p - 1./108.*m44*RSp564e*pow(
         RSe561p,2) );

      F13p6x23RV +=  + Ab(mp2,mu2)*CbV(m11,L) * ( 1./18.*m11*RSp45611e*
         RD1456ep );

      F13p6x23RV +=  + Ab(mp2,mu2)*CbV(m33,L) * ( 1./18.*m33*RSp45633e*
         RD3456ep );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m11,L) * ( 1./36.*RSe45613p*
         RS14563pe - 1./72.*RSe45633p*RS14563pe - 1./72.*RC1456pe*
         RSe45613p + 1./144.*RC1456pe*RSe45633p - 1./72.*RSe45611p*
         RS14563pe + 1./144.*RSe45611p*RC1456pe + 1./18.*m11*RN1e*
         RC1456pe*RSe45613p - 1./18.*m11*RN1e*RSe45611p*RC1456pe );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m13,L) * ( 1./162.*RSe45613p*
         RS34561pe + 1./162.*RSe45613p*RS14563pe + 1./162.*pow(
         RSe45613p,2) + 1./162.*RSp45613e*RSe45613p - 1./324.*RC3456pe*
         RSe45613p - 1./324.*RSe45633p*RS34561pe - 1./324.*RSe45633p*
         RS14563pe - 1./162.*RSe45633p*RSe45613p - 1./324.*RSe45633p*
         RSp45613e + 1./648.*RSe45633p*RC3456pe + 1./648.*pow(
         RSe45633p,2) - 1./324.*RSp45633e*RSe45613p + 1./648.*RSp45633e
         *RSe45633p - 1./324.*RC1456pe*RSe45613p + 1./648.*RC1456pe*
         RSe45633p - 1./324.*RSe45611p*RS34561pe - 1./324.*RSe45611p*
         RS14563pe - 1./162.*RSe45611p*RSe45613p - 1./324.*RSe45611p*
         RSp45613e + 1./648.*RSe45611p*RC3456pe + 1./324.*RSe45611p*
         RSe45633p + 1./648.*RSe45611p*RSp45633e + 1./648.*RSe45611p*
         RC1456pe + 1./648.*pow(RSe45611p,2) - 1./324.*RSp45611e*
         RSe45613p + 1./648.*RSp45611e*RSe45633p + 1./648.*RSp45611e*
         RSe45611p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m14,L) * (  - 1./36.*RSe45613p
          - 1./48.*RSe45633p + 5./72.*RSe563p + 7./144.*RSe45611p - 5./
         72.*RSe561p - 1./36.*m44*RN1e*RSe45613p - 1./36.*m44*RN1e*
         RSe563p + 1./36.*m44*RN1e*RSe45611p + 1./36.*m44*RN1e*RSe561p
          + 1./18.*m14*RN1e*RSe45613p - 1./18.*m14*RN1e*RSe563p - 1./18.
         *m14*RN1e*RSe45611p + 1./18.*m14*RN1e*RSe561p - 1./36.*m11*
         RN1e*RSe45613p - 1./12.*m11*RN1e*RSe563p + 1./36.*m11*RN1e*
         RSe45611p + 1./12.*m11*RN1e*RSe561p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m15,L) * (  - 1./36.*RSe45613p
          - 1./48.*RSe45633p + 5./72.*RSe463p + 7./144.*RSe45611p - 5./
         72.*RSe461p - 1./36.*m55*RN1e*RSe45613p - 1./36.*m55*RN1e*
         RSe463p + 1./36.*m55*RN1e*RSe45611p + 1./36.*m55*RN1e*RSe461p
          + 1./18.*m15*RN1e*RSe45613p - 1./18.*m15*RN1e*RSe463p - 1./18.
         *m15*RN1e*RSe45611p + 1./18.*m15*RN1e*RSe461p - 1./36.*m11*
         RN1e*RSe45613p - 1./12.*m11*RN1e*RSe463p + 1./36.*m11*RN1e*
         RSe45611p + 1./12.*m11*RN1e*RSe461p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m16,L) * (  - 1./36.*RSe45613p
          - 1./48.*RSe45633p + 5./72.*RSe453p + 7./144.*RSe45611p - 5./
         72.*RSe451p - 1./36.*m66*RN1e*RSe45613p - 1./36.*m66*RN1e*
         RSe453p + 1./36.*m66*RN1e*RSe45611p + 1./36.*m66*RN1e*RSe451p
          + 1./18.*m16*RN1e*RSe45613p - 1./18.*m16*RN1e*RSe453p - 1./18.
         *m16*RN1e*RSe45611p + 1./18.*m16*RN1e*RSe451p - 1./36.*m11*
         RN1e*RSe45613p - 1./12.*m11*RN1e*RSe453p + 1./36.*m11*RN1e*
         RSe45611p + 1./12.*m11*RN1e*RSe451p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m33,L) * ( 1./36.*RSe45613p*
         RS34561pe - 1./72.*RC3456pe*RSe45613p - 1./72.*RSe45633p*
         RS34561pe + 1./144.*RSe45633p*RC3456pe - 1./72.*RSe45611p*
         RS34561pe + 1./144.*RSe45611p*RC3456pe + 1./18.*m33*RN3e*
         RC3456pe*RSe45613p - 1./18.*m33*RN3e*RSe45633p*RC3456pe );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m34,L) * (  - 1./36.*RSe45613p
          + 7./144.*RSe45633p - 5./72.*RSe563p - 1./48.*RSe45611p + 5./
         72.*RSe561p - 1./36.*m44*RN3e*RSe45613p + 1./36.*m44*RN3e*
         RSe45633p + 1./36.*m44*RN3e*RSe563p - 1./36.*m44*RN3e*RSe561p
          + 1./18.*m34*RN3e*RSe45613p - 1./18.*m34*RN3e*RSe45633p + 1./
         18.*m34*RN3e*RSe563p - 1./18.*m34*RN3e*RSe561p - 1./36.*m33*
         RN3e*RSe45613p + 1./36.*m33*RN3e*RSe45633p + 1./12.*m33*RN3e*
         RSe563p - 1./12.*m33*RN3e*RSe561p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m35,L) * (  - 1./36.*RSe45613p
          + 7./144.*RSe45633p - 5./72.*RSe463p - 1./48.*RSe45611p + 5./
         72.*RSe461p - 1./36.*m55*RN3e*RSe45613p + 1./36.*m55*RN3e*
         RSe45633p + 1./36.*m55*RN3e*RSe463p - 1./36.*m55*RN3e*RSe461p
          + 1./18.*m35*RN3e*RSe45613p - 1./18.*m35*RN3e*RSe45633p + 1./
         18.*m35*RN3e*RSe463p - 1./18.*m35*RN3e*RSe461p - 1./36.*m33*
         RN3e*RSe45613p + 1./36.*m33*RN3e*RSe45633p + 1./12.*m33*RN3e*
         RSe463p - 1./12.*m33*RN3e*RSe461p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m36,L) * (  - 1./36.*RSe45613p
          + 7./144.*RSe45633p - 5./72.*RSe453p - 1./48.*RSe45611p + 5./
         72.*RSe451p - 1./36.*m66*RN3e*RSe45613p + 1./36.*m66*RN3e*
         RSe45633p + 1./36.*m66*RN3e*RSe453p - 1./36.*m66*RN3e*RSe451p
          + 1./18.*m36*RN3e*RSe45613p - 1./18.*m36*RN3e*RSe45633p + 1./
         18.*m36*RN3e*RSe453p - 1./18.*m36*RN3e*RSe451p - 1./36.*m33*
         RN3e*RSe45613p + 1./36.*m33*RN3e*RSe45633p + 1./12.*m33*RN3e*
         RSe453p - 1./12.*m33*RN3e*RSe451p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m44,L) * (  - 1./18.*m44*RN3e*
         RSe563p*RS356pe + 1./54.*m44*RN3e*RS456pe*RSe563p*RS356pe + 1./
         18.*m44*RN3e*RSe561p*RS356pe - 1./54.*m44*RN3e*RSe561p*RS456pe
         *RS356pe + 1./18.*m44*RN1e*RS156pe*RSe563p - 1./54.*m44*RN1e*
         RS456pe*RS156pe*RSe563p - 1./18.*m44*RN1e*RSe561p*RS156pe + 1./
         54.*m44*RN1e*RSe561p*RS456pe*RS156pe - 1./18.*RN(mp2,me2)*m44*
         RSp563e*RSe563p + 1./54.*RN(mp2,me2)*m44*RS456pe*RSp563e*
         RSe563p + 1./18.*RN(mp2,me2)*m44*RSe561p*RSp563e - 1./54.*RN(
         mp2,me2)*m44*RSe561p*RS456pe*RSp563e + 1./18.*RN(mp2,me2)*m44*
         RSp561e*RSe563p - 1./54.*RN(mp2,me2)*m44*RSp561e*RS456pe*
         RSe563p - 1./18.*RN(mp2,me2)*m44*RSp561e*RSe561p + 1./54.*RN(
         mp2,me2)*m44*RSp561e*RSe561p*RS456pe );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./54.*RSe563p*
         RS346pe - 1./54.*RSe563p*RS356pe - 1./108.*pow(RSe563p,2) - 1./
         54.*RSe463p*RS346pe + 1./54.*RSe463p*RS356pe + 1./54.*RSe463p*
         RSe563p - 1./108.*pow(RSe463p,2) - 2./81.*RSp563e*RSe563p + 2./
         81.*RSp563e*RSe463p + 2./81.*RSp463e*RSe563p - 2./81.*RSp463e*
         RSe463p - 1./54.*RS146pe*RSe563p + 1./54.*RS146pe*RSe463p + 1./
         54.*RS156pe*RSe563p - 1./54.*RS156pe*RSe463p - 1./54.*RSe561p*
         RS346pe + 1./54.*RSe561p*RS356pe + 1./54.*RSe561p*RSe563p - 1./
         54.*RSe561p*RSe463p + 2./81.*RSe561p*RSp563e - 2./81.*RSe561p*
         RSp463e + 1./54.*RSe561p*RS146pe - 1./54.*RSe561p*RS156pe - 1./
         108.*pow(RSe561p,2) + 1./54.*RSe461p*RS346pe - 1./54.*RSe461p*
         RS356pe - 1./54.*RSe461p*RSe563p + 1./54.*RSe461p*RSe463p - 2./
         81.*RSe461p*RSp563e + 2./81.*RSe461p*RSp463e - 1./54.*RSe461p*
         RS146pe + 1./54.*RSe461p*RS156pe + 1./54.*RSe461p*RSe561p - 1./
         108.*pow(RSe461p,2) + 2./81.*RSp561e*RSe563p - 2./81.*RSp561e*
         RSe463p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 2./81.*RSp561e*
         RSe561p + 2./81.*RSp561e*RSe461p - 2./81.*RSp461e*RSe563p + 2./
         81.*RSp461e*RSe463p + 2./81.*RSp461e*RSe561p - 2./81.*RSp461e*
         RSe461p - 1./108.*m55*RN3e*RSe563p*RS346pe - 1./108.*m55*RN3e*
         RSe563p*RS356pe - 1./36.*m55*RN3e*RSe463p*RS346pe - 1./108.*
         m55*RN3e*RSe463p*RS356pe + 1./108.*m55*RN3e*RSe561p*RS346pe + 
         1./108.*m55*RN3e*RSe561p*RS356pe + 1./36.*m55*RN3e*RSe461p*
         RS346pe + 1./108.*m55*RN3e*RSe461p*RS356pe + 1./108.*m55*RN1e*
         RS146pe*RSe563p + 1./36.*m55*RN1e*RS146pe*RSe463p + 1./108.*
         m55*RN1e*RS156pe*RSe563p + 1./108.*m55*RN1e*RS156pe*RSe463p - 
         1./108.*m55*RN1e*RSe561p*RS146pe - 1./108.*m55*RN1e*RSe561p*
         RS156pe - 1./36.*m55*RN1e*RSe461p*RS146pe - 1./108.*m55*RN1e*
         RSe461p*RS156pe - 1./54.*m45*RN3e*RSe563p*RS346pe + 1./54.*m45
         *RN3e*RSe563p*RS356pe + 1./54.*m45*RN3e*RSe463p*RS346pe - 1./
         54.*m45*RN3e*RSe463p*RS356pe + 1./54.*m45*RN3e*RSe561p*RS346pe
          - 1./54.*m45*RN3e*RSe561p*RS356pe );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./54.*m45*RN3e*
         RSe461p*RS346pe + 1./54.*m45*RN3e*RSe461p*RS356pe + 1./54.*m45
         *RN1e*RS146pe*RSe563p - 1./54.*m45*RN1e*RS146pe*RSe463p - 1./
         54.*m45*RN1e*RS156pe*RSe563p + 1./54.*m45*RN1e*RS156pe*RSe463p
          - 1./54.*m45*RN1e*RSe561p*RS146pe + 1./54.*m45*RN1e*RSe561p*
         RS156pe + 1./54.*m45*RN1e*RSe461p*RS146pe - 1./54.*m45*RN1e*
         RSe461p*RS156pe - 1./108.*m44*RN3e*RSe563p*RS346pe - 1./36.*
         m44*RN3e*RSe563p*RS356pe - 1./108.*m44*RN3e*RSe463p*RS346pe - 
         1./108.*m44*RN3e*RSe463p*RS356pe + 1./108.*m44*RN3e*RSe561p*
         RS346pe + 1./36.*m44*RN3e*RSe561p*RS356pe + 1./108.*m44*RN3e*
         RSe461p*RS346pe + 1./108.*m44*RN3e*RSe461p*RS356pe + 1./108.*
         m44*RN1e*RS146pe*RSe563p + 1./108.*m44*RN1e*RS146pe*RSe463p + 
         1./36.*m44*RN1e*RS156pe*RSe563p + 1./108.*m44*RN1e*RS156pe*
         RSe463p - 1./108.*m44*RN1e*RSe561p*RS146pe - 1./36.*m44*RN1e*
         RSe561p*RS156pe - 1./108.*m44*RN1e*RSe461p*RS146pe - 1./108.*
         m44*RN1e*RSe461p*RS156pe );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./54.*m33*RN3e*
         RSe563p*RS346pe + 1./54.*m33*RN3e*RSe563p*RS356pe + 1./54.*m33
         *RN3e*RSe463p*RS346pe - 1./54.*m33*RN3e*RSe463p*RS356pe + 1./
         54.*m33*RN3e*RSe561p*RS346pe - 1./54.*m33*RN3e*RSe561p*RS356pe
          - 1./54.*m33*RN3e*RSe461p*RS346pe + 1./54.*m33*RN3e*RSe461p*
         RS356pe + 1./54.*m11*RN1e*RS146pe*RSe563p - 1./54.*m11*RN1e*
         RS146pe*RSe463p - 1./54.*m11*RN1e*RS156pe*RSe563p + 1./54.*m11
         *RN1e*RS156pe*RSe463p - 1./54.*m11*RN1e*RSe561p*RS146pe + 1./
         54.*m11*RN1e*RSe561p*RS156pe + 1./54.*m11*RN1e*RSe461p*RS146pe
          - 1./54.*m11*RN1e*RSe461p*RS156pe - 1./162.*RN(mp2,me2)*me2*
         RSp563e*RSe563p + 1./162.*RN(mp2,me2)*me2*RSp563e*RSe463p + 1./
         162.*RN(mp2,me2)*me2*RSp463e*RSe563p - 1./162.*RN(mp2,me2)*me2
         *RSp463e*RSe463p + 1./162.*RN(mp2,me2)*me2*RSe561p*RSp563e - 1.
         /162.*RN(mp2,me2)*me2*RSe561p*RSp463e - 1./162.*RN(mp2,me2)*
         me2*RSe461p*RSp563e + 1./162.*RN(mp2,me2)*me2*RSe461p*RSp463e
          + 1./162.*RN(mp2,me2)*me2*RSp561e*RSe563p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./162.*RN(mp2,me2
         )*me2*RSp561e*RSe463p - 1./162.*RN(mp2,me2)*me2*RSp561e*
         RSe561p + 1./162.*RN(mp2,me2)*me2*RSp561e*RSe461p - 1./162.*
         RN(mp2,me2)*me2*RSp461e*RSe563p + 1./162.*RN(mp2,me2)*me2*
         RSp461e*RSe463p + 1./162.*RN(mp2,me2)*me2*RSp461e*RSe561p - 1./
         162.*RN(mp2,me2)*me2*RSp461e*RSe461p + 2./81.*RN(mp2,me2)*mp2*
         RSp563e*RSe563p - 2./81.*RN(mp2,me2)*mp2*RSp563e*RSe463p - 2./
         81.*RN(mp2,me2)*mp2*RSp463e*RSe563p + 2./81.*RN(mp2,me2)*mp2*
         RSp463e*RSe463p - 2./81.*RN(mp2,me2)*mp2*RSe561p*RSp563e + 2./
         81.*RN(mp2,me2)*mp2*RSe561p*RSp463e + 2./81.*RN(mp2,me2)*mp2*
         RSe461p*RSp563e - 2./81.*RN(mp2,me2)*mp2*RSe461p*RSp463e - 2./
         81.*RN(mp2,me2)*mp2*RSp561e*RSe563p + 2./81.*RN(mp2,me2)*mp2*
         RSp561e*RSe463p + 2./81.*RN(mp2,me2)*mp2*RSp561e*RSe561p - 2./
         81.*RN(mp2,me2)*mp2*RSp561e*RSe461p + 2./81.*RN(mp2,me2)*mp2*
         RSp461e*RSe563p - 2./81.*RN(mp2,me2)*mp2*RSp461e*RSe463p - 2./
         81.*RN(mp2,me2)*mp2*RSp461e*RSe561p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 2./81.*RN(mp2,me2)*
         mp2*RSp461e*RSe461p - 1./108.*RN(mp2,me2)*m55*RSp563e*RSe563p
          - 1./108.*RN(mp2,me2)*m55*RSp563e*RSe463p - 1./108.*RN(mp2,
         me2)*m55*RSp463e*RSe563p - 1./36.*RN(mp2,me2)*m55*RSp463e*
         RSe463p + 1./108.*RN(mp2,me2)*m55*RSe561p*RSp563e + 1./108.*
         RN(mp2,me2)*m55*RSe561p*RSp463e + 1./108.*RN(mp2,me2)*m55*
         RSe461p*RSp563e + 1./36.*RN(mp2,me2)*m55*RSe461p*RSp463e + 1./
         108.*RN(mp2,me2)*m55*RSp561e*RSe563p + 1./108.*RN(mp2,me2)*m55
         *RSp561e*RSe463p - 1./108.*RN(mp2,me2)*m55*RSp561e*RSe561p - 1.
         /108.*RN(mp2,me2)*m55*RSp561e*RSe461p + 1./108.*RN(mp2,me2)*
         m55*RSp461e*RSe563p + 1./36.*RN(mp2,me2)*m55*RSp461e*RSe463p
          - 1./108.*RN(mp2,me2)*m55*RSp461e*RSe561p - 1./36.*RN(mp2,me2
         )*m55*RSp461e*RSe461p + 1./54.*RN(mp2,me2)*m45*RSp563e*RSe563p
          - 1./54.*RN(mp2,me2)*m45*RSp563e*RSe463p - 1./54.*RN(mp2,me2)
         *m45*RSp463e*RSe563p + 1./54.*RN(mp2,me2)*m45*RSp463e*RSe463p
          - 1./54.*RN(mp2,me2)*m45*RSe561p*RSp563e );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * ( 1./54.*RN(mp2,me2)*
         m45*RSe561p*RSp463e + 1./54.*RN(mp2,me2)*m45*RSe461p*RSp563e
          - 1./54.*RN(mp2,me2)*m45*RSe461p*RSp463e - 1./54.*RN(mp2,me2)
         *m45*RSp561e*RSe563p + 1./54.*RN(mp2,me2)*m45*RSp561e*RSe463p
          + 1./54.*RN(mp2,me2)*m45*RSp561e*RSe561p - 1./54.*RN(mp2,me2)
         *m45*RSp561e*RSe461p + 1./54.*RN(mp2,me2)*m45*RSp461e*RSe563p
          - 1./54.*RN(mp2,me2)*m45*RSp461e*RSe463p - 1./54.*RN(mp2,me2)
         *m45*RSp461e*RSe561p + 1./54.*RN(mp2,me2)*m45*RSp461e*RSe461p
          - 1./36.*RN(mp2,me2)*m44*RSp563e*RSe563p - 1./108.*RN(mp2,me2
         )*m44*RSp563e*RSe463p - 1./108.*RN(mp2,me2)*m44*RSp463e*
         RSe563p - 1./108.*RN(mp2,me2)*m44*RSp463e*RSe463p + 1./36.*RN(
         mp2,me2)*m44*RSe561p*RSp563e + 1./108.*RN(mp2,me2)*m44*RSe561p
         *RSp463e + 1./108.*RN(mp2,me2)*m44*RSe461p*RSp563e + 1./108.*
         RN(mp2,me2)*m44*RSe461p*RSp463e + 1./36.*RN(mp2,me2)*m44*
         RSp561e*RSe563p + 1./108.*RN(mp2,me2)*m44*RSp561e*RSe463p - 1./
         36.*RN(mp2,me2)*m44*RSp561e*RSe561p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m45,L) * (  - 1./108.*RN(mp2,me2
         )*m44*RSp561e*RSe461p + 1./108.*RN(mp2,me2)*m44*RSp461e*
         RSe563p + 1./108.*RN(mp2,me2)*m44*RSp461e*RSe463p - 1./108.*
         RN(mp2,me2)*m44*RSp461e*RSe561p - 1./108.*RN(mp2,me2)*m44*
         RSp461e*RSe461p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./54.*RSe563p*
         RS345pe - 1./54.*RSe563p*RS356pe - 1./108.*pow(RSe563p,2) - 1./
         54.*RSe453p*RS345pe + 1./54.*RSe453p*RS356pe + 1./54.*RSe453p*
         RSe563p - 1./108.*pow(RSe453p,2) - 2./81.*RSp563e*RSe563p + 2./
         81.*RSp563e*RSe453p + 2./81.*RSp453e*RSe563p - 2./81.*RSp453e*
         RSe453p - 1./54.*RS145pe*RSe563p + 1./54.*RS145pe*RSe453p + 1./
         54.*RS156pe*RSe563p - 1./54.*RS156pe*RSe453p - 1./54.*RSe561p*
         RS345pe + 1./54.*RSe561p*RS356pe + 1./54.*RSe561p*RSe563p - 1./
         54.*RSe561p*RSe453p + 2./81.*RSe561p*RSp563e - 2./81.*RSe561p*
         RSp453e + 1./54.*RSe561p*RS145pe - 1./54.*RSe561p*RS156pe - 1./
         108.*pow(RSe561p,2) + 1./54.*RSe451p*RS345pe - 1./54.*RSe451p*
         RS356pe - 1./54.*RSe451p*RSe563p + 1./54.*RSe451p*RSe453p - 2./
         81.*RSe451p*RSp563e + 2./81.*RSe451p*RSp453e - 1./54.*RSe451p*
         RS145pe + 1./54.*RSe451p*RS156pe + 1./54.*RSe451p*RSe561p - 1./
         108.*pow(RSe451p,2) + 2./81.*RSp561e*RSe563p - 2./81.*RSp561e*
         RSe453p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 2./81.*RSp561e*
         RSe561p + 2./81.*RSp561e*RSe451p - 2./81.*RSp451e*RSe563p + 2./
         81.*RSp451e*RSe453p + 2./81.*RSp451e*RSe561p - 2./81.*RSp451e*
         RSe451p - 1./108.*m66*RN3e*RSe563p*RS345pe - 1./108.*m66*RN3e*
         RSe563p*RS356pe - 1./36.*m66*RN3e*RSe453p*RS345pe - 1./108.*
         m66*RN3e*RSe453p*RS356pe + 1./108.*m66*RN3e*RSe561p*RS345pe + 
         1./108.*m66*RN3e*RSe561p*RS356pe + 1./36.*m66*RN3e*RSe451p*
         RS345pe + 1./108.*m66*RN3e*RSe451p*RS356pe + 1./108.*m66*RN1e*
         RS145pe*RSe563p + 1./36.*m66*RN1e*RS145pe*RSe453p + 1./108.*
         m66*RN1e*RS156pe*RSe563p + 1./108.*m66*RN1e*RS156pe*RSe453p - 
         1./108.*m66*RN1e*RSe561p*RS145pe - 1./108.*m66*RN1e*RSe561p*
         RS156pe - 1./36.*m66*RN1e*RSe451p*RS145pe - 1./108.*m66*RN1e*
         RSe451p*RS156pe - 1./54.*m46*RN3e*RSe563p*RS345pe + 1./54.*m46
         *RN3e*RSe563p*RS356pe + 1./54.*m46*RN3e*RSe453p*RS345pe - 1./
         54.*m46*RN3e*RSe453p*RS356pe + 1./54.*m46*RN3e*RSe561p*RS345pe
          - 1./54.*m46*RN3e*RSe561p*RS356pe );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./54.*m46*RN3e*
         RSe451p*RS345pe + 1./54.*m46*RN3e*RSe451p*RS356pe + 1./54.*m46
         *RN1e*RS145pe*RSe563p - 1./54.*m46*RN1e*RS145pe*RSe453p - 1./
         54.*m46*RN1e*RS156pe*RSe563p + 1./54.*m46*RN1e*RS156pe*RSe453p
          - 1./54.*m46*RN1e*RSe561p*RS145pe + 1./54.*m46*RN1e*RSe561p*
         RS156pe + 1./54.*m46*RN1e*RSe451p*RS145pe - 1./54.*m46*RN1e*
         RSe451p*RS156pe - 1./108.*m44*RN3e*RSe563p*RS345pe - 1./36.*
         m44*RN3e*RSe563p*RS356pe - 1./108.*m44*RN3e*RSe453p*RS345pe - 
         1./108.*m44*RN3e*RSe453p*RS356pe + 1./108.*m44*RN3e*RSe561p*
         RS345pe + 1./36.*m44*RN3e*RSe561p*RS356pe + 1./108.*m44*RN3e*
         RSe451p*RS345pe + 1./108.*m44*RN3e*RSe451p*RS356pe + 1./108.*
         m44*RN1e*RS145pe*RSe563p + 1./108.*m44*RN1e*RS145pe*RSe453p + 
         1./36.*m44*RN1e*RS156pe*RSe563p + 1./108.*m44*RN1e*RS156pe*
         RSe453p - 1./108.*m44*RN1e*RSe561p*RS145pe - 1./36.*m44*RN1e*
         RSe561p*RS156pe - 1./108.*m44*RN1e*RSe451p*RS145pe - 1./108.*
         m44*RN1e*RSe451p*RS156pe );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./54.*m33*RN3e*
         RSe563p*RS345pe + 1./54.*m33*RN3e*RSe563p*RS356pe + 1./54.*m33
         *RN3e*RSe453p*RS345pe - 1./54.*m33*RN3e*RSe453p*RS356pe + 1./
         54.*m33*RN3e*RSe561p*RS345pe - 1./54.*m33*RN3e*RSe561p*RS356pe
          - 1./54.*m33*RN3e*RSe451p*RS345pe + 1./54.*m33*RN3e*RSe451p*
         RS356pe + 1./54.*m11*RN1e*RS145pe*RSe563p - 1./54.*m11*RN1e*
         RS145pe*RSe453p - 1./54.*m11*RN1e*RS156pe*RSe563p + 1./54.*m11
         *RN1e*RS156pe*RSe453p - 1./54.*m11*RN1e*RSe561p*RS145pe + 1./
         54.*m11*RN1e*RSe561p*RS156pe + 1./54.*m11*RN1e*RSe451p*RS145pe
          - 1./54.*m11*RN1e*RSe451p*RS156pe - 1./162.*RN(mp2,me2)*me2*
         RSp563e*RSe563p + 1./162.*RN(mp2,me2)*me2*RSp563e*RSe453p + 1./
         162.*RN(mp2,me2)*me2*RSp453e*RSe563p - 1./162.*RN(mp2,me2)*me2
         *RSp453e*RSe453p + 1./162.*RN(mp2,me2)*me2*RSe561p*RSp563e - 1.
         /162.*RN(mp2,me2)*me2*RSe561p*RSp453e - 1./162.*RN(mp2,me2)*
         me2*RSe451p*RSp563e + 1./162.*RN(mp2,me2)*me2*RSe451p*RSp453e
          + 1./162.*RN(mp2,me2)*me2*RSp561e*RSe563p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./162.*RN(mp2,me2
         )*me2*RSp561e*RSe453p - 1./162.*RN(mp2,me2)*me2*RSp561e*
         RSe561p + 1./162.*RN(mp2,me2)*me2*RSp561e*RSe451p - 1./162.*
         RN(mp2,me2)*me2*RSp451e*RSe563p + 1./162.*RN(mp2,me2)*me2*
         RSp451e*RSe453p + 1./162.*RN(mp2,me2)*me2*RSp451e*RSe561p - 1./
         162.*RN(mp2,me2)*me2*RSp451e*RSe451p + 2./81.*RN(mp2,me2)*mp2*
         RSp563e*RSe563p - 2./81.*RN(mp2,me2)*mp2*RSp563e*RSe453p - 2./
         81.*RN(mp2,me2)*mp2*RSp453e*RSe563p + 2./81.*RN(mp2,me2)*mp2*
         RSp453e*RSe453p - 2./81.*RN(mp2,me2)*mp2*RSe561p*RSp563e + 2./
         81.*RN(mp2,me2)*mp2*RSe561p*RSp453e + 2./81.*RN(mp2,me2)*mp2*
         RSe451p*RSp563e - 2./81.*RN(mp2,me2)*mp2*RSe451p*RSp453e - 2./
         81.*RN(mp2,me2)*mp2*RSp561e*RSe563p + 2./81.*RN(mp2,me2)*mp2*
         RSp561e*RSe453p + 2./81.*RN(mp2,me2)*mp2*RSp561e*RSe561p - 2./
         81.*RN(mp2,me2)*mp2*RSp561e*RSe451p + 2./81.*RN(mp2,me2)*mp2*
         RSp451e*RSe563p - 2./81.*RN(mp2,me2)*mp2*RSp451e*RSe453p - 2./
         81.*RN(mp2,me2)*mp2*RSp451e*RSe561p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 2./81.*RN(mp2,me2)*
         mp2*RSp451e*RSe451p - 1./108.*RN(mp2,me2)*m66*RSp563e*RSe563p
          - 1./108.*RN(mp2,me2)*m66*RSp563e*RSe453p - 1./108.*RN(mp2,
         me2)*m66*RSp453e*RSe563p - 1./36.*RN(mp2,me2)*m66*RSp453e*
         RSe453p + 1./108.*RN(mp2,me2)*m66*RSe561p*RSp563e + 1./108.*
         RN(mp2,me2)*m66*RSe561p*RSp453e + 1./108.*RN(mp2,me2)*m66*
         RSe451p*RSp563e + 1./36.*RN(mp2,me2)*m66*RSe451p*RSp453e + 1./
         108.*RN(mp2,me2)*m66*RSp561e*RSe563p + 1./108.*RN(mp2,me2)*m66
         *RSp561e*RSe453p - 1./108.*RN(mp2,me2)*m66*RSp561e*RSe561p - 1.
         /108.*RN(mp2,me2)*m66*RSp561e*RSe451p + 1./108.*RN(mp2,me2)*
         m66*RSp451e*RSe563p + 1./36.*RN(mp2,me2)*m66*RSp451e*RSe453p
          - 1./108.*RN(mp2,me2)*m66*RSp451e*RSe561p - 1./36.*RN(mp2,me2
         )*m66*RSp451e*RSe451p + 1./54.*RN(mp2,me2)*m46*RSp563e*RSe563p
          - 1./54.*RN(mp2,me2)*m46*RSp563e*RSe453p - 1./54.*RN(mp2,me2)
         *m46*RSp453e*RSe563p + 1./54.*RN(mp2,me2)*m46*RSp453e*RSe453p
          - 1./54.*RN(mp2,me2)*m46*RSe561p*RSp563e );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * ( 1./54.*RN(mp2,me2)*
         m46*RSe561p*RSp453e + 1./54.*RN(mp2,me2)*m46*RSe451p*RSp563e
          - 1./54.*RN(mp2,me2)*m46*RSe451p*RSp453e - 1./54.*RN(mp2,me2)
         *m46*RSp561e*RSe563p + 1./54.*RN(mp2,me2)*m46*RSp561e*RSe453p
          + 1./54.*RN(mp2,me2)*m46*RSp561e*RSe561p - 1./54.*RN(mp2,me2)
         *m46*RSp561e*RSe451p + 1./54.*RN(mp2,me2)*m46*RSp451e*RSe563p
          - 1./54.*RN(mp2,me2)*m46*RSp451e*RSe453p - 1./54.*RN(mp2,me2)
         *m46*RSp451e*RSe561p + 1./54.*RN(mp2,me2)*m46*RSp451e*RSe451p
          - 1./36.*RN(mp2,me2)*m44*RSp563e*RSe563p - 1./108.*RN(mp2,me2
         )*m44*RSp563e*RSe453p - 1./108.*RN(mp2,me2)*m44*RSp453e*
         RSe563p - 1./108.*RN(mp2,me2)*m44*RSp453e*RSe453p + 1./36.*RN(
         mp2,me2)*m44*RSe561p*RSp563e + 1./108.*RN(mp2,me2)*m44*RSe561p
         *RSp453e + 1./108.*RN(mp2,me2)*m44*RSe451p*RSp563e + 1./108.*
         RN(mp2,me2)*m44*RSe451p*RSp453e + 1./36.*RN(mp2,me2)*m44*
         RSp561e*RSe563p + 1./108.*RN(mp2,me2)*m44*RSp561e*RSe453p - 1./
         36.*RN(mp2,me2)*m44*RSp561e*RSe561p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m46,L) * (  - 1./108.*RN(mp2,me2
         )*m44*RSp561e*RSe451p + 1./108.*RN(mp2,me2)*m44*RSp451e*
         RSe563p + 1./108.*RN(mp2,me2)*m44*RSp451e*RSe453p - 1./108.*
         RN(mp2,me2)*m44*RSp451e*RSe561p - 1./108.*RN(mp2,me2)*m44*
         RSp451e*RSe451p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m55,L) * (  - 1./18.*m55*RN3e*
         RSe463p*RS346pe + 1./54.*m55*RN3e*RS546pe*RSe463p*RS346pe + 1./
         18.*m55*RN3e*RSe461p*RS346pe - 1./54.*m55*RN3e*RSe461p*RS546pe
         *RS346pe + 1./18.*m55*RN1e*RS146pe*RSe463p - 1./54.*m55*RN1e*
         RS546pe*RS146pe*RSe463p - 1./18.*m55*RN1e*RSe461p*RS146pe + 1./
         54.*m55*RN1e*RSe461p*RS546pe*RS146pe - 1./18.*RN(mp2,me2)*m55*
         RSp463e*RSe463p + 1./54.*RN(mp2,me2)*m55*RS546pe*RSp463e*
         RSe463p + 1./18.*RN(mp2,me2)*m55*RSe461p*RSp463e - 1./54.*RN(
         mp2,me2)*m55*RSe461p*RS546pe*RSp463e + 1./18.*RN(mp2,me2)*m55*
         RSp461e*RSe463p - 1./54.*RN(mp2,me2)*m55*RSp461e*RS546pe*
         RSe463p - 1./18.*RN(mp2,me2)*m55*RSp461e*RSe461p + 1./54.*RN(
         mp2,me2)*m55*RSp461e*RSe461p*RS546pe );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./54.*RSe463p*
         RS345pe - 1./54.*RSe463p*RS346pe - 1./108.*pow(RSe463p,2) - 1./
         54.*RSe453p*RS345pe + 1./54.*RSe453p*RS346pe + 1./54.*RSe453p*
         RSe463p - 1./108.*pow(RSe453p,2) - 2./81.*RSp463e*RSe463p + 2./
         81.*RSp463e*RSe453p + 2./81.*RSp453e*RSe463p - 2./81.*RSp453e*
         RSe453p - 1./54.*RS145pe*RSe463p + 1./54.*RS145pe*RSe453p + 1./
         54.*RS146pe*RSe463p - 1./54.*RS146pe*RSe453p - 1./54.*RSe461p*
         RS345pe + 1./54.*RSe461p*RS346pe + 1./54.*RSe461p*RSe463p - 1./
         54.*RSe461p*RSe453p + 2./81.*RSe461p*RSp463e - 2./81.*RSe461p*
         RSp453e + 1./54.*RSe461p*RS145pe - 1./54.*RSe461p*RS146pe - 1./
         108.*pow(RSe461p,2) + 1./54.*RSe451p*RS345pe - 1./54.*RSe451p*
         RS346pe - 1./54.*RSe451p*RSe463p + 1./54.*RSe451p*RSe453p - 2./
         81.*RSe451p*RSp463e + 2./81.*RSe451p*RSp453e - 1./54.*RSe451p*
         RS145pe + 1./54.*RSe451p*RS146pe + 1./54.*RSe451p*RSe461p - 1./
         108.*pow(RSe451p,2) + 2./81.*RSp461e*RSe463p - 2./81.*RSp461e*
         RSe453p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 2./81.*RSp461e*
         RSe461p + 2./81.*RSp461e*RSe451p - 2./81.*RSp451e*RSe463p + 2./
         81.*RSp451e*RSe453p + 2./81.*RSp451e*RSe461p - 2./81.*RSp451e*
         RSe451p - 1./108.*m66*RN3e*RSe463p*RS345pe - 1./108.*m66*RN3e*
         RSe463p*RS346pe - 1./36.*m66*RN3e*RSe453p*RS345pe - 1./108.*
         m66*RN3e*RSe453p*RS346pe + 1./108.*m66*RN3e*RSe461p*RS345pe + 
         1./108.*m66*RN3e*RSe461p*RS346pe + 1./36.*m66*RN3e*RSe451p*
         RS345pe + 1./108.*m66*RN3e*RSe451p*RS346pe + 1./108.*m66*RN1e*
         RS145pe*RSe463p + 1./36.*m66*RN1e*RS145pe*RSe453p + 1./108.*
         m66*RN1e*RS146pe*RSe463p + 1./108.*m66*RN1e*RS146pe*RSe453p - 
         1./108.*m66*RN1e*RSe461p*RS145pe - 1./108.*m66*RN1e*RSe461p*
         RS146pe - 1./36.*m66*RN1e*RSe451p*RS145pe - 1./108.*m66*RN1e*
         RSe451p*RS146pe - 1./54.*m56*RN3e*RSe463p*RS345pe + 1./54.*m56
         *RN3e*RSe463p*RS346pe + 1./54.*m56*RN3e*RSe453p*RS345pe - 1./
         54.*m56*RN3e*RSe453p*RS346pe + 1./54.*m56*RN3e*RSe461p*RS345pe
          - 1./54.*m56*RN3e*RSe461p*RS346pe );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./54.*m56*RN3e*
         RSe451p*RS345pe + 1./54.*m56*RN3e*RSe451p*RS346pe + 1./54.*m56
         *RN1e*RS145pe*RSe463p - 1./54.*m56*RN1e*RS145pe*RSe453p - 1./
         54.*m56*RN1e*RS146pe*RSe463p + 1./54.*m56*RN1e*RS146pe*RSe453p
          - 1./54.*m56*RN1e*RSe461p*RS145pe + 1./54.*m56*RN1e*RSe461p*
         RS146pe + 1./54.*m56*RN1e*RSe451p*RS145pe - 1./54.*m56*RN1e*
         RSe451p*RS146pe - 1./108.*m55*RN3e*RSe463p*RS345pe - 1./36.*
         m55*RN3e*RSe463p*RS346pe - 1./108.*m55*RN3e*RSe453p*RS345pe - 
         1./108.*m55*RN3e*RSe453p*RS346pe + 1./108.*m55*RN3e*RSe461p*
         RS345pe + 1./36.*m55*RN3e*RSe461p*RS346pe + 1./108.*m55*RN3e*
         RSe451p*RS345pe + 1./108.*m55*RN3e*RSe451p*RS346pe + 1./108.*
         m55*RN1e*RS145pe*RSe463p + 1./108.*m55*RN1e*RS145pe*RSe453p + 
         1./36.*m55*RN1e*RS146pe*RSe463p + 1./108.*m55*RN1e*RS146pe*
         RSe453p - 1./108.*m55*RN1e*RSe461p*RS145pe - 1./36.*m55*RN1e*
         RSe461p*RS146pe - 1./108.*m55*RN1e*RSe451p*RS145pe - 1./108.*
         m55*RN1e*RSe451p*RS146pe );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./54.*m33*RN3e*
         RSe463p*RS345pe + 1./54.*m33*RN3e*RSe463p*RS346pe + 1./54.*m33
         *RN3e*RSe453p*RS345pe - 1./54.*m33*RN3e*RSe453p*RS346pe + 1./
         54.*m33*RN3e*RSe461p*RS345pe - 1./54.*m33*RN3e*RSe461p*RS346pe
          - 1./54.*m33*RN3e*RSe451p*RS345pe + 1./54.*m33*RN3e*RSe451p*
         RS346pe + 1./54.*m11*RN1e*RS145pe*RSe463p - 1./54.*m11*RN1e*
         RS145pe*RSe453p - 1./54.*m11*RN1e*RS146pe*RSe463p + 1./54.*m11
         *RN1e*RS146pe*RSe453p - 1./54.*m11*RN1e*RSe461p*RS145pe + 1./
         54.*m11*RN1e*RSe461p*RS146pe + 1./54.*m11*RN1e*RSe451p*RS145pe
          - 1./54.*m11*RN1e*RSe451p*RS146pe - 1./162.*RN(mp2,me2)*me2*
         RSp463e*RSe463p + 1./162.*RN(mp2,me2)*me2*RSp463e*RSe453p + 1./
         162.*RN(mp2,me2)*me2*RSp453e*RSe463p - 1./162.*RN(mp2,me2)*me2
         *RSp453e*RSe453p + 1./162.*RN(mp2,me2)*me2*RSe461p*RSp463e - 1.
         /162.*RN(mp2,me2)*me2*RSe461p*RSp453e - 1./162.*RN(mp2,me2)*
         me2*RSe451p*RSp463e + 1./162.*RN(mp2,me2)*me2*RSe451p*RSp453e
          + 1./162.*RN(mp2,me2)*me2*RSp461e*RSe463p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./162.*RN(mp2,me2
         )*me2*RSp461e*RSe453p - 1./162.*RN(mp2,me2)*me2*RSp461e*
         RSe461p + 1./162.*RN(mp2,me2)*me2*RSp461e*RSe451p - 1./162.*
         RN(mp2,me2)*me2*RSp451e*RSe463p + 1./162.*RN(mp2,me2)*me2*
         RSp451e*RSe453p + 1./162.*RN(mp2,me2)*me2*RSp451e*RSe461p - 1./
         162.*RN(mp2,me2)*me2*RSp451e*RSe451p + 2./81.*RN(mp2,me2)*mp2*
         RSp463e*RSe463p - 2./81.*RN(mp2,me2)*mp2*RSp463e*RSe453p - 2./
         81.*RN(mp2,me2)*mp2*RSp453e*RSe463p + 2./81.*RN(mp2,me2)*mp2*
         RSp453e*RSe453p - 2./81.*RN(mp2,me2)*mp2*RSe461p*RSp463e + 2./
         81.*RN(mp2,me2)*mp2*RSe461p*RSp453e + 2./81.*RN(mp2,me2)*mp2*
         RSe451p*RSp463e - 2./81.*RN(mp2,me2)*mp2*RSe451p*RSp453e - 2./
         81.*RN(mp2,me2)*mp2*RSp461e*RSe463p + 2./81.*RN(mp2,me2)*mp2*
         RSp461e*RSe453p + 2./81.*RN(mp2,me2)*mp2*RSp461e*RSe461p - 2./
         81.*RN(mp2,me2)*mp2*RSp461e*RSe451p + 2./81.*RN(mp2,me2)*mp2*
         RSp451e*RSe463p - 2./81.*RN(mp2,me2)*mp2*RSp451e*RSe453p - 2./
         81.*RN(mp2,me2)*mp2*RSp451e*RSe461p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 2./81.*RN(mp2,me2)*
         mp2*RSp451e*RSe451p - 1./108.*RN(mp2,me2)*m66*RSp463e*RSe463p
          - 1./108.*RN(mp2,me2)*m66*RSp463e*RSe453p - 1./108.*RN(mp2,
         me2)*m66*RSp453e*RSe463p - 1./36.*RN(mp2,me2)*m66*RSp453e*
         RSe453p + 1./108.*RN(mp2,me2)*m66*RSe461p*RSp463e + 1./108.*
         RN(mp2,me2)*m66*RSe461p*RSp453e + 1./108.*RN(mp2,me2)*m66*
         RSe451p*RSp463e + 1./36.*RN(mp2,me2)*m66*RSe451p*RSp453e + 1./
         108.*RN(mp2,me2)*m66*RSp461e*RSe463p + 1./108.*RN(mp2,me2)*m66
         *RSp461e*RSe453p - 1./108.*RN(mp2,me2)*m66*RSp461e*RSe461p - 1.
         /108.*RN(mp2,me2)*m66*RSp461e*RSe451p + 1./108.*RN(mp2,me2)*
         m66*RSp451e*RSe463p + 1./36.*RN(mp2,me2)*m66*RSp451e*RSe453p
          - 1./108.*RN(mp2,me2)*m66*RSp451e*RSe461p - 1./36.*RN(mp2,me2
         )*m66*RSp451e*RSe451p + 1./54.*RN(mp2,me2)*m56*RSp463e*RSe463p
          - 1./54.*RN(mp2,me2)*m56*RSp463e*RSe453p - 1./54.*RN(mp2,me2)
         *m56*RSp453e*RSe463p + 1./54.*RN(mp2,me2)*m56*RSp453e*RSe453p
          - 1./54.*RN(mp2,me2)*m56*RSe461p*RSp463e );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * ( 1./54.*RN(mp2,me2)*
         m56*RSe461p*RSp453e + 1./54.*RN(mp2,me2)*m56*RSe451p*RSp463e
          - 1./54.*RN(mp2,me2)*m56*RSe451p*RSp453e - 1./54.*RN(mp2,me2)
         *m56*RSp461e*RSe463p + 1./54.*RN(mp2,me2)*m56*RSp461e*RSe453p
          + 1./54.*RN(mp2,me2)*m56*RSp461e*RSe461p - 1./54.*RN(mp2,me2)
         *m56*RSp461e*RSe451p + 1./54.*RN(mp2,me2)*m56*RSp451e*RSe463p
          - 1./54.*RN(mp2,me2)*m56*RSp451e*RSe453p - 1./54.*RN(mp2,me2)
         *m56*RSp451e*RSe461p + 1./54.*RN(mp2,me2)*m56*RSp451e*RSe451p
          - 1./36.*RN(mp2,me2)*m55*RSp463e*RSe463p - 1./108.*RN(mp2,me2
         )*m55*RSp463e*RSe453p - 1./108.*RN(mp2,me2)*m55*RSp453e*
         RSe463p - 1./108.*RN(mp2,me2)*m55*RSp453e*RSe453p + 1./36.*RN(
         mp2,me2)*m55*RSe461p*RSp463e + 1./108.*RN(mp2,me2)*m55*RSe461p
         *RSp453e + 1./108.*RN(mp2,me2)*m55*RSe451p*RSp463e + 1./108.*
         RN(mp2,me2)*m55*RSe451p*RSp453e + 1./36.*RN(mp2,me2)*m55*
         RSp461e*RSe463p + 1./108.*RN(mp2,me2)*m55*RSp461e*RSe453p - 1./
         36.*RN(mp2,me2)*m55*RSp461e*RSe461p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m56,L) * (  - 1./108.*RN(mp2,me2
         )*m55*RSp461e*RSe451p + 1./108.*RN(mp2,me2)*m55*RSp451e*
         RSe463p + 1./108.*RN(mp2,me2)*m55*RSp451e*RSe453p - 1./108.*
         RN(mp2,me2)*m55*RSp451e*RSe461p - 1./108.*RN(mp2,me2)*m55*
         RSp451e*RSe451p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(m66,L) * (  - 1./18.*m66*RN3e*
         RSe453p*RS345pe + 1./54.*m66*RN3e*RS645pe*RSe453p*RS345pe + 1./
         18.*m66*RN3e*RSe451p*RS345pe - 1./54.*m66*RN3e*RSe451p*RS645pe
         *RS345pe + 1./18.*m66*RN1e*RS145pe*RSe453p - 1./54.*m66*RN1e*
         RS645pe*RS145pe*RSe453p - 1./18.*m66*RN1e*RSe451p*RS145pe + 1./
         54.*m66*RN1e*RSe451p*RS645pe*RS145pe - 1./18.*RN(mp2,me2)*m66*
         RSp453e*RSe453p + 1./54.*RN(mp2,me2)*m66*RS645pe*RSp453e*
         RSe453p + 1./18.*RN(mp2,me2)*m66*RSe451p*RSp453e - 1./54.*RN(
         mp2,me2)*m66*RSe451p*RS645pe*RSp453e + 1./18.*RN(mp2,me2)*m66*
         RSp451e*RSe453p - 1./54.*RN(mp2,me2)*m66*RSp451e*RS645pe*
         RSe453p - 1./18.*RN(mp2,me2)*m66*RSp451e*RSe451p + 1./54.*RN(
         mp2,me2)*m66*RSp451e*RSe451p*RS645pe );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(mp2,L) * ( 1./36.*RSp45613e*
         RSe45613p - 1./72.*RSe45633p*RSp45613e - 1./72.*RSp45633e*
         RSe45613p + 1./144.*RSp45633e*RSe45633p - 1./72.*RSe45611p*
         RSp45613e + 1./144.*RSe45611p*RSp45633e - 1./72.*RSp45611e*
         RSe45613p + 1./144.*RSp45611e*RSe45633p + 1./144.*RSp45611e*
         RSe45611p + 1./54.*m66*RN3e*RSp456e*RSe453p*RS345pe - 1./54.*
         m66*RN3e*RSp456e*RSe451p*RS345pe - 1./54.*m66*RN1e*RSp456e*
         RS145pe*RSe453p + 1./54.*m66*RN1e*RSp456e*RSe451p*RS145pe + 1./
         54.*m55*RN3e*RSp465e*RSe463p*RS346pe - 1./54.*m55*RN3e*RSp465e
         *RSe461p*RS346pe - 1./54.*m55*RN1e*RSp465e*RS146pe*RSe463p + 1.
         /54.*m55*RN1e*RSp465e*RSe461p*RS146pe + 1./54.*m44*RN3e*
         RSp564e*RSe563p*RS356pe - 1./54.*m44*RN3e*RSp564e*RSe561p*
         RS356pe - 1./54.*m44*RN1e*RSp564e*RS156pe*RSe563p + 1./54.*m44
         *RN1e*RSp564e*RSe561p*RS156pe + 1./18.*m33*RN3e*RSp45633e*
         RSe45613p - 1./18.*m33*RN3e*RSp45633e*RSe45633p + 1./18.*m11*
         RN1e*RSp45611e*RSe45613p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(mp2,L) * (  - 1./18.*m11*RN1e*
         RSp45611e*RSe45611p + 1./54.*RN(mp2,me2)*m66*RSp456e*RSp453e*
         RSe453p - 1./54.*RN(mp2,me2)*m66*RSp456e*RSe451p*RSp453e - 1./
         54.*RN(mp2,me2)*m66*RSp456e*RSp451e*RSe453p + 1./54.*RN(mp2,
         me2)*m66*RSp456e*RSp451e*RSe451p + 1./54.*RN(mp2,me2)*m55*
         RSp465e*RSp463e*RSe463p - 1./54.*RN(mp2,me2)*m55*RSp465e*
         RSe461p*RSp463e - 1./54.*RN(mp2,me2)*m55*RSp465e*RSp461e*
         RSe463p + 1./54.*RN(mp2,me2)*m55*RSp465e*RSp461e*RSe461p + 1./
         54.*RN(mp2,me2)*m44*RSp564e*RSp563e*RSe563p - 1./54.*RN(mp2,
         me2)*m44*RSp564e*RSe561p*RSp563e - 1./54.*RN(mp2,me2)*m44*
         RSp564e*RSp561e*RSe563p + 1./54.*RN(mp2,me2)*m44*RSp564e*
         RSp561e*RSe561p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(me2,L) * ( 1./36.*pow(
         RSe45613p,2) - 1./36.*RSe45633p*RSe45613p + 1./144.*pow(
         RSe45633p,2) - 1./36.*RSe45611p*RSe45613p + 1./72.*RSe45611p*
         RSe45633p + 1./144.*pow(RSe45611p,2) + 1./54.*m66*RN3e*RSe456p
         *RSe453p*RS345pe - 1./54.*m66*RN3e*RSe456p*RSe451p*RS345pe - 1.
         /54.*m66*RN1e*RSe456p*RS145pe*RSe453p + 1./54.*m66*RN1e*
         RSe456p*RSe451p*RS145pe + 1./54.*m55*RN3e*RSe465p*RSe463p*
         RS346pe - 1./54.*m55*RN3e*RSe465p*RSe461p*RS346pe - 1./54.*m55
         *RN1e*RSe465p*RS146pe*RSe463p + 1./54.*m55*RN1e*RSe465p*
         RSe461p*RS146pe + 1./54.*m44*RN3e*RSe564p*RSe563p*RS356pe - 1./
         54.*m44*RN3e*RSe564p*RSe561p*RS356pe - 1./54.*m44*RN1e*RSe564p
         *RS156pe*RSe563p + 1./54.*m44*RN1e*RSe564p*RSe561p*RS156pe + 1.
         /18.*m33*RN3e*RSe45633p*RSe45613p - 1./18.*m33*RN3e*pow(
         RSe45633p,2) + 1./18.*m11*RN1e*RSe45611p*RSe45613p - 1./18.*
         m11*RN1e*pow(RSe45611p,2) + 1./54.*RN(mp2,me2)*m66*RSe456p*
         RSp453e*RSe453p );

      F13p6x23RV +=  + Ab(me2,mu2)*AbV(me2,L) * (  - 1./54.*RN(mp2,me2)
         *m66*RSe456p*RSe451p*RSp453e - 1./54.*RN(mp2,me2)*m66*RSe456p*
         RSp451e*RSe453p + 1./54.*RN(mp2,me2)*m66*RSe456p*RSp451e*
         RSe451p + 1./54.*RN(mp2,me2)*m55*RSe465p*RSp463e*RSe463p - 1./
         54.*RN(mp2,me2)*m55*RSe465p*RSe461p*RSp463e - 1./54.*RN(mp2,
         me2)*m55*RSe465p*RSp461e*RSe463p + 1./54.*RN(mp2,me2)*m55*
         RSe465p*RSp461e*RSe461p + 1./54.*RN(mp2,me2)*m44*RSe564p*
         RSp563e*RSe563p - 1./54.*RN(mp2,me2)*m44*RSe564p*RSe561p*
         RSp563e - 1./54.*RN(mp2,me2)*m44*RSe564p*RSp561e*RSe563p + 1./
         54.*RN(mp2,me2)*m44*RSe564p*RSp561e*RSe561p );

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(m11,m33,L) * ( 1./54.*m66*
         RSe456p*RS145pe*RS345pe + 1./54.*m55*RSe465p*RS146pe*RS346pe
          + 1./54.*m44*RSe564p*RS156pe*RS356pe - 1./18.*m33*RSe45633p*
         RS14563pe - 1./18.*m11*RSe45611p*RS34561pe );

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(m11,mp2,L) * ( 1./54.*m66*
         RSe456p*RS145pe*RSp453e - 1./54.*m66*RSe456p*RSp451e*RS145pe
          + 1./54.*m55*RSe465p*RS146pe*RSp463e - 1./54.*m55*RSe465p*
         RSp461e*RS146pe + 1./54.*m44*RSe564p*RS156pe*RSp563e - 1./54.*
         m44*RSe564p*RSp561e*RS156pe - 1./18.*m11*RSe45611p*RSp45613e
          + 1./18.*m11*RSp45611e*RSe45611p );

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(m11,me2,L) * ( 1./54.*m66*
         RSe456p*RS145pe*RSe453p - 1./54.*m66*RSe456p*RSe451p*RS145pe
          + 1./54.*m55*RSe465p*RS146pe*RSe463p - 1./54.*m55*RSe465p*
         RSe461p*RS146pe + 1./54.*m44*RSe564p*RS156pe*RSe563p - 1./54.*
         m44*RSe564p*RSe561p*RS156pe - 1./18.*m11*RSe45611p*RSe45613p
          + 1./18.*m11*pow(RSe45611p,2) );

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(m11,L) * (  - 1./72.*RD1456ep*
         RSe45613p + 1./144.*RD1456ep*RSe45633p + 1./144.*RSe45611p*
         RD1456ep - 1./108.*m66*RSe456p*pow(RS145pe,2) - 1./108.*m55*
         RSe465p*pow(RS146pe,2) - 1./108.*m44*RSe564p*pow(RS156pe,2) - 
         1./18.*m11*RSe45611p*RS14563pe + 1./18.*m11*RSe45611p*RC1456pe
          + 1./18.*m11*RN1e*RD1456ep*RSe45613p - 1./18.*m11*RN1e*
         RSe45611p*RD1456ep );

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(m14,L) * ( 1./72.*me2*RSe45611p
          - 1./36.*me2*RSe561p + 1./72.*me2*RSe564p - 1./144.*m44*
         RSe45611p - 1./72.*m44*RSe561p - 1./48.*m44*RSe564p + 1./72.*
         m14*RSe45611p - 1./36.*m14*RSe561p + 1./72.*m14*RSe564p - 1./
         48.*m11*RSe45611p - 1./72.*m11*RSe561p - 1./144.*m11*RSe564p )
         ;

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(m15,L) * ( 1./72.*me2*RSe45611p
          - 1./36.*me2*RSe461p + 1./72.*me2*RSe465p - 1./144.*m55*
         RSe45611p - 1./72.*m55*RSe461p - 1./48.*m55*RSe465p + 1./72.*
         m15*RSe45611p - 1./36.*m15*RSe461p + 1./72.*m15*RSe465p - 1./
         48.*m11*RSe45611p - 1./72.*m11*RSe461p - 1./144.*m11*RSe465p )
         ;

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(m16,L) * ( 1./72.*me2*RSe45611p
          - 1./36.*me2*RSe451p + 1./72.*me2*RSe456p - 1./144.*m66*
         RSe45611p - 1./72.*m66*RSe451p - 1./48.*m66*RSe456p + 1./72.*
         m16*RSe45611p - 1./36.*m16*RSe451p + 1./72.*m16*RSe456p - 1./
         48.*m11*RSe45611p - 1./72.*m11*RSe451p - 1./144.*m11*RSe456p )
         ;

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(m33,mp2,L) * (  - 1./54.*m66*
         RSe456p*RSp453e*RS345pe + 1./54.*m66*RSe456p*RSp451e*RS345pe
          - 1./54.*m55*RSe465p*RSp463e*RS346pe + 1./54.*m55*RSe465p*
         RSp461e*RS346pe - 1./54.*m44*RSe564p*RSp563e*RS356pe + 1./54.*
         m44*RSe564p*RSp561e*RS356pe - 1./18.*m33*RSe45633p*RSp45613e
          + 1./18.*m33*RSp45633e*RSe45633p );

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(m33,me2,L) * (  - 1./54.*m66*
         RSe456p*RSe453p*RS345pe + 1./54.*m66*RSe456p*RSe451p*RS345pe
          - 1./54.*m55*RSe465p*RSe463p*RS346pe + 1./54.*m55*RSe465p*
         RSe461p*RS346pe - 1./54.*m44*RSe564p*RSe563p*RS356pe + 1./54.*
         m44*RSe564p*RSe561p*RS356pe - 1./18.*m33*RSe45633p*RSe45613p
          + 1./18.*m33*pow(RSe45633p,2) );

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(m33,L) * (  - 1./72.*RD3456ep*
         RSe45613p + 1./144.*RSe45633p*RD3456ep + 1./144.*RSe45611p*
         RD3456ep - 1./108.*m66*RSe456p*pow(RS345pe,2) - 1./108.*m55*
         RSe465p*pow(RS346pe,2) - 1./108.*m44*RSe564p*pow(RS356pe,2) - 
         1./18.*m33*RSe45633p*RS34561pe + 1./18.*m33*RSe45633p*RC3456pe
          + 1./18.*m33*RN3e*RD3456ep*RSe45613p - 1./18.*m33*RN3e*
         RSe45633p*RD3456ep );

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(m34,L) * ( 1./72.*me2*RSe45633p
          - 1./36.*me2*RSe563p + 1./72.*me2*RSe564p - 1./144.*m44*
         RSe45633p - 1./72.*m44*RSe563p - 1./48.*m44*RSe564p + 1./72.*
         m34*RSe45633p - 1./36.*m34*RSe563p + 1./72.*m34*RSe564p - 1./
         48.*m33*RSe45633p - 1./72.*m33*RSe563p - 1./144.*m33*RSe564p )
         ;

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(m35,L) * ( 1./72.*me2*RSe45633p
          - 1./36.*me2*RSe463p + 1./72.*me2*RSe465p - 1./144.*m55*
         RSe45633p - 1./72.*m55*RSe463p - 1./48.*m55*RSe465p + 1./72.*
         m35*RSe45633p - 1./36.*m35*RSe463p + 1./72.*m35*RSe465p - 1./
         48.*m33*RSe45633p - 1./72.*m33*RSe463p - 1./144.*m33*RSe465p )
         ;

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(m36,L) * ( 1./72.*me2*RSe45633p
          - 1./36.*me2*RSe453p + 1./72.*me2*RSe456p - 1./144.*m66*
         RSe45633p - 1./72.*m66*RSe453p - 1./48.*m66*RSe456p + 1./72.*
         m36*RSe45633p - 1./36.*m36*RSe453p + 1./72.*m36*RSe456p - 1./
         48.*m33*RSe45633p - 1./72.*m33*RSe453p - 1./144.*m33*RSe456p )
         ;

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(mp2,me2,L) * (  - 2./81.*m66*
         RSe456p*RSp453e*RSe453p + 2./81.*m66*RSe456p*RSe451p*RSp453e
          + 2./81.*m66*RSe456p*RSp451e*RSe453p - 2./81.*m66*RSe456p*
         RSp451e*RSe451p - 2./81.*m55*RSe465p*RSp463e*RSe463p + 2./81.*
         m55*RSe465p*RSe461p*RSp463e + 2./81.*m55*RSe465p*RSp461e*
         RSe463p - 2./81.*m55*RSe465p*RSp461e*RSe461p - 2./81.*m44*
         RSe564p*RSp563e*RSe563p + 2./81.*m44*RSe564p*RSe561p*RSp563e
          + 2./81.*m44*RSe564p*RSp561e*RSe563p - 2./81.*m44*RSe564p*
         RSp561e*RSe561p );

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(mp2,L) * (  - 1./108.*m66*
         RSe456p*pow(RSp453e,2) + 1./54.*m66*RSe456p*RSp451e*RSp453e - 
         1./108.*m66*RSe456p*pow(RSp451e,2) - 1./108.*m55*RSe465p*pow(
         RSp463e,2) + 1./54.*m55*RSe465p*RSp461e*RSp463e - 1./108.*m55*
         RSe465p*pow(RSp461e,2) - 1./108.*m44*RSe564p*pow(RSp563e,2) + 
         1./54.*m44*RSe564p*RSp561e*RSp563e - 1./108.*m44*RSe564p*pow(
         RSp561e,2) );

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(me2,mp2,L) * ( 1./162.*m66*
         RSe456p*RSp453e*RSe453p - 1./162.*m66*RSe456p*RSe451p*RSp453e
          - 1./162.*m66*RSe456p*RSp451e*RSe453p + 1./162.*m66*RSe456p*
         RSp451e*RSe451p + 1./162.*m55*RSe465p*RSp463e*RSe463p - 1./162.
         *m55*RSe465p*RSe461p*RSp463e - 1./162.*m55*RSe465p*RSp461e*
         RSe463p + 1./162.*m55*RSe465p*RSp461e*RSe461p + 1./162.*m44*
         RSe564p*RSp563e*RSe563p - 1./162.*m44*RSe564p*RSe561p*RSp563e
          - 1./162.*m44*RSe564p*RSp561e*RSe563p + 1./162.*m44*RSe564p*
         RSp561e*RSe561p );

      F13p6x23RV +=  + Ab(me2,mu2)*BbV(me2,L) * (  - 1./108.*m66*
         RSe456p*pow(RSe453p,2) + 1./54.*m66*RSe456p*RSe451p*RSe453p - 
         1./108.*m66*RSe456p*pow(RSe451p,2) - 1./108.*m55*RSe465p*pow(
         RSe463p,2) + 1./54.*m55*RSe465p*RSe461p*RSe463p - 1./108.*m55*
         RSe465p*pow(RSe461p,2) - 1./108.*m44*RSe564p*pow(RSe563p,2) + 
         1./54.*m44*RSe564p*RSe561p*RSe563p - 1./108.*m44*RSe564p*pow(
         RSe561p,2) );

      F13p6x23RV +=  + Ab(me2,mu2)*CbV(m11,L) * ( 1./18.*m11*RSe45611p*
         RD1456ep );

      F13p6x23RV +=  + Ab(me2,mu2)*CbV(m33,L) * ( 1./18.*m33*RSe45633p*
         RD3456ep );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m11,L) * (  - 1./72.*RD1456ep*
         RS14563pe + 1./144.*RD1456ep*RC1456pe + 1./4.*m11 - 1./18.*m11
         *RC1456pe*RS14563pe + 1./18.*m11*pow(RC1456pe,2) );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m13,L) * (  - 1./324.*RD1456ep*
         RS34561pe - 1./324.*RD1456ep*RS14563pe - 1./324.*RD1456ep*
         RSe45613p - 1./324.*RD1456ep*RSp45613e + 1./648.*RD1456ep*
         RC3456pe + 1./648.*RD1456ep*RSe45633p + 1./648.*RD1456ep*
         RSp45633e + 1./648.*RD1456ep*RC1456pe + 1./648.*RSe45611p*
         RD1456ep + 1./648.*RSp45611e*RD1456ep );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m14,L) * ( 7./144.*RD1456ep + 1./
         36.*m44*RS14563pe - 1./36.*m44*RC1456pe - 1./36.*m44*RS156pe
          - 1./18.*m14*RS14563pe + 1./18.*m14*RC1456pe - 1./18.*m14*
         RS156pe + 1./36.*m11*RS14563pe - 1./36.*m11*RC1456pe - 1./12.*
         m11*RS156pe );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m15,L) * ( 7./144.*RD1456ep + 1./
         36.*m55*RS14563pe - 1./36.*m55*RC1456pe - 1./36.*m55*RS146pe
          - 1./18.*m15*RS14563pe + 1./18.*m15*RC1456pe - 1./18.*m15*
         RS146pe + 1./36.*m11*RS14563pe - 1./36.*m11*RC1456pe - 1./12.*
         m11*RS146pe );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m16,L) * ( 7./144.*RD1456ep + 1./
         36.*m66*RS14563pe - 1./36.*m66*RC1456pe - 1./36.*m66*RS145pe
          - 1./18.*m16*RS14563pe + 1./18.*m16*RC1456pe - 1./18.*m16*
         RS145pe + 1./36.*m11*RS14563pe - 1./36.*m11*RC1456pe - 1./12.*
         m11*RS145pe );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m33,L) * (  - 1./72.*RD1456ep*
         RS34561pe + 1./144.*RD1456ep*RC3456pe );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m34,L) * (  - 1./48.*RD1456ep );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m35,L) * (  - 1./48.*RD1456ep );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m36,L) * (  - 1./48.*RD1456ep );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m44,L) * ( 1./36.*m44*pow(
         RS156pe,2) - 1./108.*m44*RS456pe*pow(RS156pe,2) );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m45,L) * ( 1./72.*m55*pow(
         RS146pe,2) + 1./108.*m55*RS156pe*RS146pe + 1./216.*m55*pow(
         RS156pe,2) - 1./108.*m45*pow(RS146pe,2) + 1./54.*m45*RS156pe*
         RS146pe - 1./108.*m45*pow(RS156pe,2) + 1./216.*m44*pow(
         RS146pe,2) + 1./108.*m44*RS156pe*RS146pe + 1./72.*m44*pow(
         RS156pe,2) - 1./108.*m11*pow(RS146pe,2) + 1./54.*m11*RS156pe*
         RS146pe - 1./108.*m11*pow(RS156pe,2) );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m46,L) * ( 1./72.*m66*pow(
         RS145pe,2) + 1./108.*m66*RS156pe*RS145pe + 1./216.*m66*pow(
         RS156pe,2) - 1./108.*m46*pow(RS145pe,2) + 1./54.*m46*RS156pe*
         RS145pe - 1./108.*m46*pow(RS156pe,2) + 1./216.*m44*pow(
         RS145pe,2) + 1./108.*m44*RS156pe*RS145pe + 1./72.*m44*pow(
         RS156pe,2) - 1./108.*m11*pow(RS145pe,2) + 1./54.*m11*RS156pe*
         RS145pe - 1./108.*m11*pow(RS156pe,2) );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m55,L) * ( 1./36.*m55*pow(
         RS146pe,2) - 1./108.*m55*RS546pe*pow(RS146pe,2) );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m56,L) * ( 1./72.*m66*pow(
         RS145pe,2) + 1./108.*m66*RS146pe*RS145pe + 1./216.*m66*pow(
         RS146pe,2) - 1./108.*m56*pow(RS145pe,2) + 1./54.*m56*RS146pe*
         RS145pe - 1./108.*m56*pow(RS146pe,2) + 1./216.*m55*pow(
         RS145pe,2) + 1./108.*m55*RS146pe*RS145pe + 1./72.*m55*pow(
         RS146pe,2) - 1./108.*m11*pow(RS145pe,2) + 1./54.*m11*RS146pe*
         RS145pe - 1./108.*m11*pow(RS146pe,2) );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(m66,L) * ( 1./36.*m66*pow(
         RS145pe,2) - 1./108.*m66*RS645pe*pow(RS145pe,2) );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(mp2,L) * (  - 1./72.*RD1456ep*
         RSp45613e + 1./144.*RD1456ep*RSp45633e + 1./144.*RSp45611e*
         RD1456ep - 1./108.*m66*RSp456e*pow(RS145pe,2) - 1./108.*m55*
         RSp465e*pow(RS146pe,2) - 1./108.*m44*RSp564e*pow(RS156pe,2) - 
         1./18.*m11*RSp45611e*RS14563pe + 1./18.*m11*RSp45611e*RC1456pe
          );

      F13p6x23RV +=  + Bb(m11,mu2)*AbV(me2,L) * (  - 1./72.*RD1456ep*
         RSe45613p + 1./144.*RD1456ep*RSe45633p + 1./144.*RSe45611p*
         RD1456ep - 1./108.*m66*RSe456p*pow(RS145pe,2) - 1./108.*m55*
         RSe465p*pow(RS146pe,2) - 1./108.*m44*RSe564p*pow(RS156pe,2) - 
         1./18.*m11*RSe45611p*RS14563pe + 1./18.*m11*RSe45611p*RC1456pe
          );

      F13p6x23RV +=  + Bb(m11,mu2)*BbV(m11,m33,L) * (  - 1./18.*m11*
         RD1456ep*RS34561pe );

      F13p6x23RV +=  + Bb(m11,mu2)*BbV(m11,mp2,L) * (  - 1./18.*m11*
         RD1456ep*RSp45613e + 1./18.*m11*RSp45611e*RD1456ep );

      F13p6x23RV +=  + Bb(m11,mu2)*BbV(m11,me2,L) * (  - 1./18.*m11*
         RD1456ep*RSe45613p + 1./18.*m11*RSe45611p*RD1456ep );

      F13p6x23RV +=  + Bb(m11,mu2)*BbV(m11,L) * ( 1./144.*pow(
         RD1456ep,2) - 1./9.*m11*RD1456ep*RS14563pe + 1./9.*m11*
         RD1456ep*RC1456pe );

      F13p6x23RV +=  + Bb(m11,mu2)*BbV(m14,L) * (  - 1./144.*m44*
         RD1456ep + 1./72.*m14*RD1456ep - 1./144.*m11*RD1456ep );

      F13p6x23RV +=  + Bb(m11,mu2)*BbV(m15,L) * (  - 1./144.*m55*
         RD1456ep + 1./72.*m15*RD1456ep - 1./144.*m11*RD1456ep );

      F13p6x23RV +=  + Bb(m11,mu2)*BbV(m16,L) * (  - 1./144.*m66*
         RD1456ep + 1./72.*m16*RD1456ep - 1./144.*m11*RD1456ep );

      F13p6x23RV +=  + Bb(m11,mu2)*BbV(m33,L) * ( 1./144.*RD1456ep*
         RD3456ep );

      F13p6x23RV +=  + Bb(m11,mu2)*CbV(m11,L) * ( 1./18.*m11*pow(
         RD1456ep,2) );

      F13p6x23RV +=  + Bb(m14,mu2)*AbV(m11,L) * ( 1./72.*RD1456ep - 1./
         144.*m44*RC1456pe - 1./72.*m44*RS156pe + 1./72.*m14*RC1456pe
          - 1./36.*m14*RS156pe - 1./144.*m11*RC1456pe - 1./24.*m11*
         RS156pe );

      F13p6x23RV +=  + Bb(m14,mu2)*AbV(m14,L) * ( 1./24.*m44 - 1./12.*
         m14 + 1./24.*m11 );

      F13p6x23RV +=  + Bb(m14,mu2)*AbV(m15,L) * ( 1./48.*m55 + 1./48.*
         m44 - 1./24.*m15 - 1./24.*m14 + 1./24.*m11 );

      F13p6x23RV +=  + Bb(m14,mu2)*AbV(m16,L) * ( 1./48.*m66 + 1./48.*
         m44 - 1./24.*m16 - 1./24.*m14 + 1./24.*m11 );

      F13p6x23RV +=  + Bb(m14,mu2)*AbV(m44,L) * ( 1./48.*m44 - 1./144.*
         m44*RS456pe - 1./24.*m14 + 1./72.*m14*RS456pe + 1./48.*m11 - 1.
         /144.*m11*RS456pe );

      F13p6x23RV +=  + Bb(m14,mu2)*AbV(m45,L) * ( 1./48.*m55 - 1./24.*
         m45 + 1./24.*m44 - 1./24.*m14 + 1./48.*m11 );

      F13p6x23RV +=  + Bb(m14,mu2)*AbV(m46,L) * ( 1./48.*m66 - 1./24.*
         m46 + 1./24.*m44 - 1./24.*m14 + 1./48.*m11 );

      F13p6x23RV +=  + Bb(m14,mu2)*AbV(mp2,L) * ( 1./72.*mp2*RSp45611e
          - 1./36.*mp2*RSp561e + 1./72.*mp2*RSp564e - 1./144.*m44*
         RSp45611e - 1./72.*m44*RSp561e - 1./48.*m44*RSp564e + 1./72.*
         m14*RSp45611e - 1./36.*m14*RSp561e + 1./72.*m14*RSp564e - 1./
         48.*m11*RSp45611e - 1./72.*m11*RSp561e - 1./144.*m11*RSp564e )
         ;

      F13p6x23RV +=  + Bb(m14,mu2)*AbV(me2,L) * ( 1./72.*me2*RSe45611p
          - 1./36.*me2*RSe561p + 1./72.*me2*RSe564p - 1./144.*m44*
         RSe45611p - 1./72.*m44*RSe561p - 1./48.*m44*RSe564p + 1./72.*
         m14*RSe45611p - 1./36.*m14*RSe561p + 1./72.*m14*RSe564p - 1./
         48.*m11*RSe45611p - 1./72.*m11*RSe561p - 1./144.*m11*RSe564p )
         ;

      F13p6x23RV +=  + Bb(m14,mu2)*BbV(m11,L) * (  - 1./144.*m44*
         RD1456ep + 1./72.*m14*RD1456ep - 1./144.*m11*RD1456ep );

      F13p6x23RV +=  + Bb(m15,mu2)*AbV(m11,L) * ( 1./72.*RD1456ep - 1./
         144.*m55*RC1456pe - 1./72.*m55*RS146pe + 1./72.*m15*RC1456pe
          - 1./36.*m15*RS146pe - 1./144.*m11*RC1456pe - 1./24.*m11*
         RS146pe );

      F13p6x23RV +=  + Bb(m15,mu2)*AbV(m14,L) * ( 1./48.*m55 + 1./48.*
         m44 - 1./24.*m15 - 1./24.*m14 + 1./24.*m11 );

      F13p6x23RV +=  + Bb(m15,mu2)*AbV(m15,L) * ( 1./24.*m55 - 1./12.*
         m15 + 1./24.*m11 );

      F13p6x23RV +=  + Bb(m15,mu2)*AbV(m16,L) * ( 1./48.*m66 + 1./48.*
         m55 - 1./24.*m16 - 1./24.*m15 + 1./24.*m11 );

      F13p6x23RV +=  + Bb(m15,mu2)*AbV(m45,L) * ( 1./24.*m55 - 1./24.*
         m45 + 1./48.*m44 - 1./24.*m15 + 1./48.*m11 );

      F13p6x23RV +=  + Bb(m15,mu2)*AbV(m55,L) * ( 1./48.*m55 - 1./144.*
         m55*RS546pe - 1./24.*m15 + 1./72.*m15*RS546pe + 1./48.*m11 - 1.
         /144.*m11*RS546pe );

      F13p6x23RV +=  + Bb(m15,mu2)*AbV(m56,L) * ( 1./48.*m66 - 1./24.*
         m56 + 1./24.*m55 - 1./24.*m15 + 1./48.*m11 );

      F13p6x23RV +=  + Bb(m15,mu2)*AbV(mp2,L) * ( 1./72.*mp2*RSp45611e
          - 1./36.*mp2*RSp461e + 1./72.*mp2*RSp465e - 1./144.*m55*
         RSp45611e - 1./72.*m55*RSp461e - 1./48.*m55*RSp465e + 1./72.*
         m15*RSp45611e - 1./36.*m15*RSp461e + 1./72.*m15*RSp465e - 1./
         48.*m11*RSp45611e - 1./72.*m11*RSp461e - 1./144.*m11*RSp465e )
         ;

      F13p6x23RV +=  + Bb(m15,mu2)*AbV(me2,L) * ( 1./72.*me2*RSe45611p
          - 1./36.*me2*RSe461p + 1./72.*me2*RSe465p - 1./144.*m55*
         RSe45611p - 1./72.*m55*RSe461p - 1./48.*m55*RSe465p + 1./72.*
         m15*RSe45611p - 1./36.*m15*RSe461p + 1./72.*m15*RSe465p - 1./
         48.*m11*RSe45611p - 1./72.*m11*RSe461p - 1./144.*m11*RSe465p )
         ;

      F13p6x23RV +=  + Bb(m15,mu2)*BbV(m11,L) * (  - 1./144.*m55*
         RD1456ep + 1./72.*m15*RD1456ep - 1./144.*m11*RD1456ep );

      F13p6x23RV +=  + Bb(m16,mu2)*AbV(m11,L) * ( 1./72.*RD1456ep - 1./
         144.*m66*RC1456pe - 1./72.*m66*RS145pe + 1./72.*m16*RC1456pe
          - 1./36.*m16*RS145pe - 1./144.*m11*RC1456pe - 1./24.*m11*
         RS145pe );

      F13p6x23RV +=  + Bb(m16,mu2)*AbV(m14,L) * ( 1./48.*m66 + 1./48.*
         m44 - 1./24.*m16 - 1./24.*m14 + 1./24.*m11 );

      F13p6x23RV +=  + Bb(m16,mu2)*AbV(m15,L) * ( 1./48.*m66 + 1./48.*
         m55 - 1./24.*m16 - 1./24.*m15 + 1./24.*m11 );

      F13p6x23RV +=  + Bb(m16,mu2)*AbV(m16,L) * ( 1./24.*m66 - 1./12.*
         m16 + 1./24.*m11 );

      F13p6x23RV +=  + Bb(m16,mu2)*AbV(m46,L) * ( 1./24.*m66 - 1./24.*
         m46 + 1./48.*m44 - 1./24.*m16 + 1./48.*m11 );

      F13p6x23RV +=  + Bb(m16,mu2)*AbV(m56,L) * ( 1./24.*m66 - 1./24.*
         m56 + 1./48.*m55 - 1./24.*m16 + 1./48.*m11 );

      F13p6x23RV +=  + Bb(m16,mu2)*AbV(m66,L) * ( 1./48.*m66 - 1./144.*
         m66*RS645pe - 1./24.*m16 + 1./72.*m16*RS645pe + 1./48.*m11 - 1.
         /144.*m11*RS645pe );

      F13p6x23RV +=  + Bb(m16,mu2)*AbV(mp2,L) * ( 1./72.*mp2*RSp45611e
          - 1./36.*mp2*RSp451e + 1./72.*mp2*RSp456e - 1./144.*m66*
         RSp45611e - 1./72.*m66*RSp451e - 1./48.*m66*RSp456e + 1./72.*
         m16*RSp45611e - 1./36.*m16*RSp451e + 1./72.*m16*RSp456e - 1./
         48.*m11*RSp45611e - 1./72.*m11*RSp451e - 1./144.*m11*RSp456e )
         ;

      F13p6x23RV +=  + Bb(m16,mu2)*AbV(me2,L) * ( 1./72.*me2*RSe45611p
          - 1./36.*me2*RSe451p + 1./72.*me2*RSe456p - 1./144.*m66*
         RSe45611p - 1./72.*m66*RSe451p - 1./48.*m66*RSe456p + 1./72.*
         m16*RSe45611p - 1./36.*m16*RSe451p + 1./72.*m16*RSe456p - 1./
         48.*m11*RSe45611p - 1./72.*m11*RSe451p - 1./144.*m11*RSe456p )
         ;

      F13p6x23RV +=  + Bb(m16,mu2)*BbV(m11,L) * (  - 1./144.*m66*
         RD1456ep + 1./72.*m16*RD1456ep - 1./144.*m11*RD1456ep );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m11,L) * (  - 1./72.*RD3456ep*
         RS14563pe + 1./144.*RC1456pe*RD3456ep );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m13,L) * (  - 1./324.*RD3456ep*
         RS34561pe - 1./324.*RD3456ep*RS14563pe - 1./324.*RD3456ep*
         RSe45613p - 1./324.*RD3456ep*RSp45613e + 1./648.*RD3456ep*
         RC3456pe + 1./648.*RSe45633p*RD3456ep + 1./648.*RSp45633e*
         RD3456ep + 1./648.*RC1456pe*RD3456ep + 1./648.*RSe45611p*
         RD3456ep + 1./648.*RSp45611e*RD3456ep );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m14,L) * (  - 1./48.*RD3456ep );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m15,L) * (  - 1./48.*RD3456ep );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m16,L) * (  - 1./48.*RD3456ep );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m33,L) * (  - 1./72.*RD3456ep*
         RS34561pe + 1./144.*RD3456ep*RC3456pe + 1./4.*m33 - 1./18.*m33
         *RC3456pe*RS34561pe + 1./18.*m33*pow(RC3456pe,2) );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m34,L) * ( 7./144.*RD3456ep + 1./
         36.*m44*RS34561pe - 1./36.*m44*RC3456pe - 1./36.*m44*RS356pe
          - 1./18.*m34*RS34561pe + 1./18.*m34*RC3456pe - 1./18.*m34*
         RS356pe + 1./36.*m33*RS34561pe - 1./36.*m33*RC3456pe - 1./12.*
         m33*RS356pe );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m35,L) * ( 7./144.*RD3456ep + 1./
         36.*m55*RS34561pe - 1./36.*m55*RC3456pe - 1./36.*m55*RS346pe
          - 1./18.*m35*RS34561pe + 1./18.*m35*RC3456pe - 1./18.*m35*
         RS346pe + 1./36.*m33*RS34561pe - 1./36.*m33*RC3456pe - 1./12.*
         m33*RS346pe );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m36,L) * ( 7./144.*RD3456ep + 1./
         36.*m66*RS34561pe - 1./36.*m66*RC3456pe - 1./36.*m66*RS345pe
          - 1./18.*m36*RS34561pe + 1./18.*m36*RC3456pe - 1./18.*m36*
         RS345pe + 1./36.*m33*RS34561pe - 1./36.*m33*RC3456pe - 1./12.*
         m33*RS345pe );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m44,L) * ( 1./36.*m44*pow(
         RS356pe,2) - 1./108.*m44*RS456pe*pow(RS356pe,2) );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m45,L) * ( 1./72.*m55*pow(
         RS346pe,2) + 1./108.*m55*RS356pe*RS346pe + 1./216.*m55*pow(
         RS356pe,2) - 1./108.*m45*pow(RS346pe,2) + 1./54.*m45*RS356pe*
         RS346pe - 1./108.*m45*pow(RS356pe,2) + 1./216.*m44*pow(
         RS346pe,2) + 1./108.*m44*RS356pe*RS346pe + 1./72.*m44*pow(
         RS356pe,2) - 1./108.*m33*pow(RS346pe,2) + 1./54.*m33*RS356pe*
         RS346pe - 1./108.*m33*pow(RS356pe,2) );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m46,L) * ( 1./72.*m66*pow(
         RS345pe,2) + 1./108.*m66*RS356pe*RS345pe + 1./216.*m66*pow(
         RS356pe,2) - 1./108.*m46*pow(RS345pe,2) + 1./54.*m46*RS356pe*
         RS345pe - 1./108.*m46*pow(RS356pe,2) + 1./216.*m44*pow(
         RS345pe,2) + 1./108.*m44*RS356pe*RS345pe + 1./72.*m44*pow(
         RS356pe,2) - 1./108.*m33*pow(RS345pe,2) + 1./54.*m33*RS356pe*
         RS345pe - 1./108.*m33*pow(RS356pe,2) );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m55,L) * ( 1./36.*m55*pow(
         RS346pe,2) - 1./108.*m55*RS546pe*pow(RS346pe,2) );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m56,L) * ( 1./72.*m66*pow(
         RS345pe,2) + 1./108.*m66*RS346pe*RS345pe + 1./216.*m66*pow(
         RS346pe,2) - 1./108.*m56*pow(RS345pe,2) + 1./54.*m56*RS346pe*
         RS345pe - 1./108.*m56*pow(RS346pe,2) + 1./216.*m55*pow(
         RS345pe,2) + 1./108.*m55*RS346pe*RS345pe + 1./72.*m55*pow(
         RS346pe,2) - 1./108.*m33*pow(RS345pe,2) + 1./54.*m33*RS346pe*
         RS345pe - 1./108.*m33*pow(RS346pe,2) );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(m66,L) * ( 1./36.*m66*pow(
         RS345pe,2) - 1./108.*m66*RS645pe*pow(RS345pe,2) );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(mp2,L) * (  - 1./72.*RD3456ep*
         RSp45613e + 1./144.*RSp45633e*RD3456ep + 1./144.*RSp45611e*
         RD3456ep - 1./108.*m66*RSp456e*pow(RS345pe,2) - 1./108.*m55*
         RSp465e*pow(RS346pe,2) - 1./108.*m44*RSp564e*pow(RS356pe,2) - 
         1./18.*m33*RSp45633e*RS34561pe + 1./18.*m33*RSp45633e*RC3456pe
          );

      F13p6x23RV +=  + Bb(m33,mu2)*AbV(me2,L) * (  - 1./72.*RD3456ep*
         RSe45613p + 1./144.*RSe45633p*RD3456ep + 1./144.*RSe45611p*
         RD3456ep - 1./108.*m66*RSe456p*pow(RS345pe,2) - 1./108.*m55*
         RSe465p*pow(RS346pe,2) - 1./108.*m44*RSe564p*pow(RS356pe,2) - 
         1./18.*m33*RSe45633p*RS34561pe + 1./18.*m33*RSe45633p*RC3456pe
          );

      F13p6x23RV +=  + Bb(m33,mu2)*BbV(m11,m33,L) * (  - 1./18.*m33*
         RD3456ep*RS14563pe );

      F13p6x23RV +=  + Bb(m33,mu2)*BbV(m11,L) * ( 1./144.*RD1456ep*
         RD3456ep );

      F13p6x23RV +=  + Bb(m33,mu2)*BbV(m33,mp2,L) * (  - 1./18.*m33*
         RD3456ep*RSp45613e + 1./18.*m33*RSp45633e*RD3456ep );

      F13p6x23RV +=  + Bb(m33,mu2)*BbV(m33,me2,L) * (  - 1./18.*m33*
         RD3456ep*RSe45613p + 1./18.*m33*RSe45633p*RD3456ep );

      F13p6x23RV +=  + Bb(m33,mu2)*BbV(m33,L) * ( 1./144.*pow(
         RD3456ep,2) - 1./9.*m33*RD3456ep*RS34561pe + 1./9.*m33*
         RD3456ep*RC3456pe );

      F13p6x23RV +=  + Bb(m33,mu2)*BbV(m34,L) * (  - 1./144.*m44*
         RD3456ep + 1./72.*m34*RD3456ep - 1./144.*m33*RD3456ep );

      F13p6x23RV +=  + Bb(m33,mu2)*BbV(m35,L) * (  - 1./144.*m55*
         RD3456ep + 1./72.*m35*RD3456ep - 1./144.*m33*RD3456ep );

      F13p6x23RV +=  + Bb(m33,mu2)*BbV(m36,L) * (  - 1./144.*m66*
         RD3456ep + 1./72.*m36*RD3456ep - 1./144.*m33*RD3456ep );

      F13p6x23RV +=  + Bb(m33,mu2)*CbV(m33,L) * ( 1./18.*m33*pow(
         RD3456ep,2) );

      F13p6x23RV +=  + Bb(m34,mu2)*AbV(m33,L) * ( 1./72.*RD3456ep - 1./
         144.*m44*RC3456pe - 1./72.*m44*RS356pe + 1./72.*m34*RC3456pe
          - 1./36.*m34*RS356pe - 1./144.*m33*RC3456pe - 1./24.*m33*
         RS356pe );

      F13p6x23RV +=  + Bb(m34,mu2)*AbV(m34,L) * ( 1./24.*m44 - 1./12.*
         m34 + 1./24.*m33 );

      F13p6x23RV +=  + Bb(m34,mu2)*AbV(m35,L) * ( 1./48.*m55 + 1./48.*
         m44 - 1./24.*m35 - 1./24.*m34 + 1./24.*m33 );

      F13p6x23RV +=  + Bb(m34,mu2)*AbV(m36,L) * ( 1./48.*m66 + 1./48.*
         m44 - 1./24.*m36 - 1./24.*m34 + 1./24.*m33 );

      F13p6x23RV +=  + Bb(m34,mu2)*AbV(m44,L) * ( 1./48.*m44 - 1./144.*
         m44*RS456pe - 1./24.*m34 + 1./72.*m34*RS456pe + 1./48.*m33 - 1.
         /144.*m33*RS456pe );

      F13p6x23RV +=  + Bb(m34,mu2)*AbV(m45,L) * ( 1./48.*m55 - 1./24.*
         m45 + 1./24.*m44 - 1./24.*m34 + 1./48.*m33 );

      F13p6x23RV +=  + Bb(m34,mu2)*AbV(m46,L) * ( 1./48.*m66 - 1./24.*
         m46 + 1./24.*m44 - 1./24.*m34 + 1./48.*m33 );

      F13p6x23RV +=  + Bb(m34,mu2)*AbV(mp2,L) * ( 1./72.*mp2*RSp45633e
          - 1./36.*mp2*RSp563e + 1./72.*mp2*RSp564e - 1./144.*m44*
         RSp45633e - 1./72.*m44*RSp563e - 1./48.*m44*RSp564e + 1./72.*
         m34*RSp45633e - 1./36.*m34*RSp563e + 1./72.*m34*RSp564e - 1./
         48.*m33*RSp45633e - 1./72.*m33*RSp563e - 1./144.*m33*RSp564e )
         ;

      F13p6x23RV +=  + Bb(m34,mu2)*AbV(me2,L) * ( 1./72.*me2*RSe45633p
          - 1./36.*me2*RSe563p + 1./72.*me2*RSe564p - 1./144.*m44*
         RSe45633p - 1./72.*m44*RSe563p - 1./48.*m44*RSe564p + 1./72.*
         m34*RSe45633p - 1./36.*m34*RSe563p + 1./72.*m34*RSe564p - 1./
         48.*m33*RSe45633p - 1./72.*m33*RSe563p - 1./144.*m33*RSe564p )
         ;

      F13p6x23RV +=  + Bb(m34,mu2)*BbV(m33,L) * (  - 1./144.*m44*
         RD3456ep + 1./72.*m34*RD3456ep - 1./144.*m33*RD3456ep );

      F13p6x23RV +=  + Bb(m35,mu2)*AbV(m33,L) * ( 1./72.*RD3456ep - 1./
         144.*m55*RC3456pe - 1./72.*m55*RS346pe + 1./72.*m35*RC3456pe
          - 1./36.*m35*RS346pe - 1./144.*m33*RC3456pe - 1./24.*m33*
         RS346pe );

      F13p6x23RV +=  + Bb(m35,mu2)*AbV(m34,L) * ( 1./48.*m55 + 1./48.*
         m44 - 1./24.*m35 - 1./24.*m34 + 1./24.*m33 );

      F13p6x23RV +=  + Bb(m35,mu2)*AbV(m35,L) * ( 1./24.*m55 - 1./12.*
         m35 + 1./24.*m33 );

      F13p6x23RV +=  + Bb(m35,mu2)*AbV(m36,L) * ( 1./48.*m66 + 1./48.*
         m55 - 1./24.*m36 - 1./24.*m35 + 1./24.*m33 );

      F13p6x23RV +=  + Bb(m35,mu2)*AbV(m45,L) * ( 1./24.*m55 - 1./24.*
         m45 + 1./48.*m44 - 1./24.*m35 + 1./48.*m33 );

      F13p6x23RV +=  + Bb(m35,mu2)*AbV(m55,L) * ( 1./48.*m55 - 1./144.*
         m55*RS546pe - 1./24.*m35 + 1./72.*m35*RS546pe + 1./48.*m33 - 1.
         /144.*m33*RS546pe );

      F13p6x23RV +=  + Bb(m35,mu2)*AbV(m56,L) * ( 1./48.*m66 - 1./24.*
         m56 + 1./24.*m55 - 1./24.*m35 + 1./48.*m33 );

      F13p6x23RV +=  + Bb(m35,mu2)*AbV(mp2,L) * ( 1./72.*mp2*RSp45633e
          - 1./36.*mp2*RSp463e + 1./72.*mp2*RSp465e - 1./144.*m55*
         RSp45633e - 1./72.*m55*RSp463e - 1./48.*m55*RSp465e + 1./72.*
         m35*RSp45633e - 1./36.*m35*RSp463e + 1./72.*m35*RSp465e - 1./
         48.*m33*RSp45633e - 1./72.*m33*RSp463e - 1./144.*m33*RSp465e )
         ;

      F13p6x23RV +=  + Bb(m35,mu2)*AbV(me2,L) * ( 1./72.*me2*RSe45633p
          - 1./36.*me2*RSe463p + 1./72.*me2*RSe465p - 1./144.*m55*
         RSe45633p - 1./72.*m55*RSe463p - 1./48.*m55*RSe465p + 1./72.*
         m35*RSe45633p - 1./36.*m35*RSe463p + 1./72.*m35*RSe465p - 1./
         48.*m33*RSe45633p - 1./72.*m33*RSe463p - 1./144.*m33*RSe465p )
         ;

      F13p6x23RV +=  + Bb(m35,mu2)*BbV(m33,L) * (  - 1./144.*m55*
         RD3456ep + 1./72.*m35*RD3456ep - 1./144.*m33*RD3456ep );

      F13p6x23RV +=  + Bb(m36,mu2)*AbV(m33,L) * ( 1./72.*RD3456ep - 1./
         144.*m66*RC3456pe - 1./72.*m66*RS345pe + 1./72.*m36*RC3456pe
          - 1./36.*m36*RS345pe - 1./144.*m33*RC3456pe - 1./24.*m33*
         RS345pe );

      F13p6x23RV +=  + Bb(m36,mu2)*AbV(m34,L) * ( 1./48.*m66 + 1./48.*
         m44 - 1./24.*m36 - 1./24.*m34 + 1./24.*m33 );

      F13p6x23RV +=  + Bb(m36,mu2)*AbV(m35,L) * ( 1./48.*m66 + 1./48.*
         m55 - 1./24.*m36 - 1./24.*m35 + 1./24.*m33 );

      F13p6x23RV +=  + Bb(m36,mu2)*AbV(m36,L) * ( 1./24.*m66 - 1./12.*
         m36 + 1./24.*m33 );

      F13p6x23RV +=  + Bb(m36,mu2)*AbV(m46,L) * ( 1./24.*m66 - 1./24.*
         m46 + 1./48.*m44 - 1./24.*m36 + 1./48.*m33 );

      F13p6x23RV +=  + Bb(m36,mu2)*AbV(m56,L) * ( 1./24.*m66 - 1./24.*
         m56 + 1./48.*m55 - 1./24.*m36 + 1./48.*m33 );

      F13p6x23RV +=  + Bb(m36,mu2)*AbV(m66,L) * ( 1./48.*m66 - 1./144.*
         m66*RS645pe - 1./24.*m36 + 1./72.*m36*RS645pe + 1./48.*m33 - 1.
         /144.*m33*RS645pe );

      F13p6x23RV +=  + Bb(m36,mu2)*AbV(mp2,L) * ( 1./72.*mp2*RSp45633e
          - 1./36.*mp2*RSp453e + 1./72.*mp2*RSp456e - 1./144.*m66*
         RSp45633e - 1./72.*m66*RSp453e - 1./48.*m66*RSp456e + 1./72.*
         m36*RSp45633e - 1./36.*m36*RSp453e + 1./72.*m36*RSp456e - 1./
         48.*m33*RSp45633e - 1./72.*m33*RSp453e - 1./144.*m33*RSp456e )
         ;

      F13p6x23RV +=  + Bb(m36,mu2)*AbV(me2,L) * ( 1./72.*me2*RSe45633p
          - 1./36.*me2*RSe453p + 1./72.*me2*RSe456p - 1./144.*m66*
         RSe45633p - 1./72.*m66*RSe453p - 1./48.*m66*RSe456p + 1./72.*
         m36*RSe45633p - 1./36.*m36*RSe453p + 1./72.*m36*RSe456p - 1./
         48.*m33*RSe45633p - 1./72.*m33*RSe453p - 1./144.*m33*RSe456p )
         ;

      F13p6x23RV +=  + Bb(m36,mu2)*BbV(m33,L) * (  - 1./144.*m66*
         RD3456ep + 1./72.*m36*RD3456ep - 1./144.*m33*RD3456ep );

      F13p6x23RV +=  + Bb(mp2,mu2)*AbV(m44,L) * ( 1./36.*m44*pow(
         RSp563e,2) - 1./108.*m44*RS456pe*pow(RSp563e,2) - 1./18.*m44*
         RSp561e*RSp563e + 1./54.*m44*RSp561e*RS456pe*RSp563e + 1./36.*
         m44*pow(RSp561e,2) - 1./108.*m44*pow(RSp561e,2)*RS456pe );

      F13p6x23RV +=  + Bb(mp2,mu2)*AbV(m45,L) * (  - 1./108.*mp2*pow(
         RSp563e,2) + 1./54.*mp2*RSp463e*RSp563e - 1./108.*mp2*pow(
         RSp463e,2) + 1./54.*mp2*RSp561e*RSp563e - 1./54.*mp2*RSp561e*
         RSp463e - 1./108.*mp2*pow(RSp561e,2) - 1./54.*mp2*RSp461e*
         RSp563e + 1./54.*mp2*RSp461e*RSp463e + 1./54.*mp2*RSp461e*
         RSp561e - 1./108.*mp2*pow(RSp461e,2) + 1./216.*m55*pow(
         RSp563e,2) + 1./108.*m55*RSp463e*RSp563e + 1./72.*m55*pow(
         RSp463e,2) - 1./108.*m55*RSp561e*RSp563e - 1./108.*m55*RSp561e
         *RSp463e + 1./216.*m55*pow(RSp561e,2) - 1./108.*m55*RSp461e*
         RSp563e - 1./36.*m55*RSp461e*RSp463e + 1./108.*m55*RSp461e*
         RSp561e + 1./72.*m55*pow(RSp461e,2) - 1./108.*m45*pow(
         RSp563e,2) + 1./54.*m45*RSp463e*RSp563e - 1./108.*m45*pow(
         RSp463e,2) + 1./54.*m45*RSp561e*RSp563e - 1./54.*m45*RSp561e*
         RSp463e - 1./108.*m45*pow(RSp561e,2) - 1./54.*m45*RSp461e*
         RSp563e + 1./54.*m45*RSp461e*RSp463e + 1./54.*m45*RSp461e*
         RSp561e );

      F13p6x23RV +=  + Bb(mp2,mu2)*AbV(m45,L) * (  - 1./108.*m45*pow(
         RSp461e,2) + 1./72.*m44*pow(RSp563e,2) + 1./108.*m44*RSp463e*
         RSp563e + 1./216.*m44*pow(RSp463e,2) - 1./36.*m44*RSp561e*
         RSp563e - 1./108.*m44*RSp561e*RSp463e + 1./72.*m44*pow(
         RSp561e,2) - 1./108.*m44*RSp461e*RSp563e - 1./108.*m44*RSp461e
         *RSp463e + 1./108.*m44*RSp461e*RSp561e + 1./216.*m44*pow(
         RSp461e,2) );

      F13p6x23RV +=  + Bb(mp2,mu2)*AbV(m46,L) * (  - 1./108.*mp2*pow(
         RSp563e,2) + 1./54.*mp2*RSp453e*RSp563e - 1./108.*mp2*pow(
         RSp453e,2) + 1./54.*mp2*RSp561e*RSp563e - 1./54.*mp2*RSp561e*
         RSp453e - 1./108.*mp2*pow(RSp561e,2) - 1./54.*mp2*RSp451e*
         RSp563e + 1./54.*mp2*RSp451e*RSp453e + 1./54.*mp2*RSp451e*
         RSp561e - 1./108.*mp2*pow(RSp451e,2) + 1./216.*m66*pow(
         RSp563e,2) + 1./108.*m66*RSp453e*RSp563e + 1./72.*m66*pow(
         RSp453e,2) - 1./108.*m66*RSp561e*RSp563e - 1./108.*m66*RSp561e
         *RSp453e + 1./216.*m66*pow(RSp561e,2) - 1./108.*m66*RSp451e*
         RSp563e - 1./36.*m66*RSp451e*RSp453e + 1./108.*m66*RSp451e*
         RSp561e + 1./72.*m66*pow(RSp451e,2) - 1./108.*m46*pow(
         RSp563e,2) + 1./54.*m46*RSp453e*RSp563e - 1./108.*m46*pow(
         RSp453e,2) + 1./54.*m46*RSp561e*RSp563e - 1./54.*m46*RSp561e*
         RSp453e - 1./108.*m46*pow(RSp561e,2) - 1./54.*m46*RSp451e*
         RSp563e + 1./54.*m46*RSp451e*RSp453e + 1./54.*m46*RSp451e*
         RSp561e );

      F13p6x23RV +=  + Bb(mp2,mu2)*AbV(m46,L) * (  - 1./108.*m46*pow(
         RSp451e,2) + 1./72.*m44*pow(RSp563e,2) + 1./108.*m44*RSp453e*
         RSp563e + 1./216.*m44*pow(RSp453e,2) - 1./36.*m44*RSp561e*
         RSp563e - 1./108.*m44*RSp561e*RSp453e + 1./72.*m44*pow(
         RSp561e,2) - 1./108.*m44*RSp451e*RSp563e - 1./108.*m44*RSp451e
         *RSp453e + 1./108.*m44*RSp451e*RSp561e + 1./216.*m44*pow(
         RSp451e,2) );

      F13p6x23RV +=  + Bb(mp2,mu2)*AbV(m55,L) * ( 1./36.*m55*pow(
         RSp463e,2) - 1./108.*m55*RS546pe*pow(RSp463e,2) - 1./18.*m55*
         RSp461e*RSp463e + 1./54.*m55*RSp461e*RS546pe*RSp463e + 1./36.*
         m55*pow(RSp461e,2) - 1./108.*m55*pow(RSp461e,2)*RS546pe );

      F13p6x23RV +=  + Bb(mp2,mu2)*AbV(m56,L) * (  - 1./108.*mp2*pow(
         RSp463e,2) + 1./54.*mp2*RSp453e*RSp463e - 1./108.*mp2*pow(
         RSp453e,2) + 1./54.*mp2*RSp461e*RSp463e - 1./54.*mp2*RSp461e*
         RSp453e - 1./108.*mp2*pow(RSp461e,2) - 1./54.*mp2*RSp451e*
         RSp463e + 1./54.*mp2*RSp451e*RSp453e + 1./54.*mp2*RSp451e*
         RSp461e - 1./108.*mp2*pow(RSp451e,2) + 1./216.*m66*pow(
         RSp463e,2) + 1./108.*m66*RSp453e*RSp463e + 1./72.*m66*pow(
         RSp453e,2) - 1./108.*m66*RSp461e*RSp463e - 1./108.*m66*RSp461e
         *RSp453e + 1./216.*m66*pow(RSp461e,2) - 1./108.*m66*RSp451e*
         RSp463e - 1./36.*m66*RSp451e*RSp453e + 1./108.*m66*RSp451e*
         RSp461e + 1./72.*m66*pow(RSp451e,2) - 1./108.*m56*pow(
         RSp463e,2) + 1./54.*m56*RSp453e*RSp463e - 1./108.*m56*pow(
         RSp453e,2) + 1./54.*m56*RSp461e*RSp463e - 1./54.*m56*RSp461e*
         RSp453e - 1./108.*m56*pow(RSp461e,2) - 1./54.*m56*RSp451e*
         RSp463e + 1./54.*m56*RSp451e*RSp453e + 1./54.*m56*RSp451e*
         RSp461e );

      F13p6x23RV +=  + Bb(mp2,mu2)*AbV(m56,L) * (  - 1./108.*m56*pow(
         RSp451e,2) + 1./72.*m55*pow(RSp463e,2) + 1./108.*m55*RSp453e*
         RSp463e + 1./216.*m55*pow(RSp453e,2) - 1./36.*m55*RSp461e*
         RSp463e - 1./108.*m55*RSp461e*RSp453e + 1./72.*m55*pow(
         RSp461e,2) - 1./108.*m55*RSp451e*RSp463e - 1./108.*m55*RSp451e
         *RSp453e + 1./108.*m55*RSp451e*RSp461e + 1./216.*m55*pow(
         RSp451e,2) );

      F13p6x23RV +=  + Bb(mp2,mu2)*AbV(m66,L) * ( 1./36.*m66*pow(
         RSp453e,2) - 1./108.*m66*RS645pe*pow(RSp453e,2) - 1./18.*m66*
         RSp451e*RSp453e + 1./54.*m66*RSp451e*RS645pe*RSp453e + 1./36.*
         m66*pow(RSp451e,2) - 1./108.*m66*pow(RSp451e,2)*RS645pe );

      F13p6x23RV +=  + Bb(mp2,mu2)*AbV(mp2,L) * (  - 1./108.*m66*
         RSp456e*pow(RSp453e,2) + 1./54.*m66*RSp456e*RSp451e*RSp453e - 
         1./108.*m66*RSp456e*pow(RSp451e,2) - 1./108.*m55*RSp465e*pow(
         RSp463e,2) + 1./54.*m55*RSp465e*RSp461e*RSp463e - 1./108.*m55*
         RSp465e*pow(RSp461e,2) - 1./108.*m44*RSp564e*pow(RSp563e,2) + 
         1./54.*m44*RSp564e*RSp561e*RSp563e - 1./108.*m44*RSp564e*pow(
         RSp561e,2) );

      F13p6x23RV +=  + Bb(mp2,mu2)*AbV(me2,L) * (  - 1./108.*m66*
         RSe456p*pow(RSp453e,2) + 1./54.*m66*RSe456p*RSp451e*RSp453e - 
         1./108.*m66*RSe456p*pow(RSp451e,2) - 1./108.*m55*RSe465p*pow(
         RSp463e,2) + 1./54.*m55*RSe465p*RSp461e*RSp463e - 1./108.*m55*
         RSe465p*pow(RSp461e,2) - 1./108.*m44*RSe564p*pow(RSp563e,2) + 
         1./54.*m44*RSe564p*RSp561e*RSp563e - 1./108.*m44*RSe564p*pow(
         RSp561e,2) );

      F13p6x23RV +=  + Bb(me2,mu2)*AbV(m44,L) * ( 1./36.*m44*pow(
         RSe563p,2) - 1./108.*m44*RS456pe*pow(RSe563p,2) - 1./18.*m44*
         RSe561p*RSe563p + 1./54.*m44*RSe561p*RS456pe*RSe563p + 1./36.*
         m44*pow(RSe561p,2) - 1./108.*m44*pow(RSe561p,2)*RS456pe );

      F13p6x23RV +=  + Bb(me2,mu2)*AbV(m45,L) * (  - 1./108.*me2*pow(
         RSe563p,2) + 1./54.*me2*RSe463p*RSe563p - 1./108.*me2*pow(
         RSe463p,2) + 1./54.*me2*RSe561p*RSe563p - 1./54.*me2*RSe561p*
         RSe463p - 1./108.*me2*pow(RSe561p,2) - 1./54.*me2*RSe461p*
         RSe563p + 1./54.*me2*RSe461p*RSe463p + 1./54.*me2*RSe461p*
         RSe561p - 1./108.*me2*pow(RSe461p,2) + 1./216.*m55*pow(
         RSe563p,2) + 1./108.*m55*RSe463p*RSe563p + 1./72.*m55*pow(
         RSe463p,2) - 1./108.*m55*RSe561p*RSe563p - 1./108.*m55*RSe561p
         *RSe463p + 1./216.*m55*pow(RSe561p,2) - 1./108.*m55*RSe461p*
         RSe563p - 1./36.*m55*RSe461p*RSe463p + 1./108.*m55*RSe461p*
         RSe561p + 1./72.*m55*pow(RSe461p,2) - 1./108.*m45*pow(
         RSe563p,2) + 1./54.*m45*RSe463p*RSe563p - 1./108.*m45*pow(
         RSe463p,2) + 1./54.*m45*RSe561p*RSe563p - 1./54.*m45*RSe561p*
         RSe463p - 1./108.*m45*pow(RSe561p,2) - 1./54.*m45*RSe461p*
         RSe563p + 1./54.*m45*RSe461p*RSe463p + 1./54.*m45*RSe461p*
         RSe561p );

      F13p6x23RV +=  + Bb(me2,mu2)*AbV(m45,L) * (  - 1./108.*m45*pow(
         RSe461p,2) + 1./72.*m44*pow(RSe563p,2) + 1./108.*m44*RSe463p*
         RSe563p + 1./216.*m44*pow(RSe463p,2) - 1./36.*m44*RSe561p*
         RSe563p - 1./108.*m44*RSe561p*RSe463p + 1./72.*m44*pow(
         RSe561p,2) - 1./108.*m44*RSe461p*RSe563p - 1./108.*m44*RSe461p
         *RSe463p + 1./108.*m44*RSe461p*RSe561p + 1./216.*m44*pow(
         RSe461p,2) );

      F13p6x23RV +=  + Bb(me2,mu2)*AbV(m46,L) * (  - 1./108.*me2*pow(
         RSe563p,2) + 1./54.*me2*RSe453p*RSe563p - 1./108.*me2*pow(
         RSe453p,2) + 1./54.*me2*RSe561p*RSe563p - 1./54.*me2*RSe561p*
         RSe453p - 1./108.*me2*pow(RSe561p,2) - 1./54.*me2*RSe451p*
         RSe563p + 1./54.*me2*RSe451p*RSe453p + 1./54.*me2*RSe451p*
         RSe561p - 1./108.*me2*pow(RSe451p,2) + 1./216.*m66*pow(
         RSe563p,2) + 1./108.*m66*RSe453p*RSe563p + 1./72.*m66*pow(
         RSe453p,2) - 1./108.*m66*RSe561p*RSe563p - 1./108.*m66*RSe561p
         *RSe453p + 1./216.*m66*pow(RSe561p,2) - 1./108.*m66*RSe451p*
         RSe563p - 1./36.*m66*RSe451p*RSe453p + 1./108.*m66*RSe451p*
         RSe561p + 1./72.*m66*pow(RSe451p,2) - 1./108.*m46*pow(
         RSe563p,2) + 1./54.*m46*RSe453p*RSe563p - 1./108.*m46*pow(
         RSe453p,2) + 1./54.*m46*RSe561p*RSe563p - 1./54.*m46*RSe561p*
         RSe453p - 1./108.*m46*pow(RSe561p,2) - 1./54.*m46*RSe451p*
         RSe563p + 1./54.*m46*RSe451p*RSe453p + 1./54.*m46*RSe451p*
         RSe561p );

      F13p6x23RV +=  + Bb(me2,mu2)*AbV(m46,L) * (  - 1./108.*m46*pow(
         RSe451p,2) + 1./72.*m44*pow(RSe563p,2) + 1./108.*m44*RSe453p*
         RSe563p + 1./216.*m44*pow(RSe453p,2) - 1./36.*m44*RSe561p*
         RSe563p - 1./108.*m44*RSe561p*RSe453p + 1./72.*m44*pow(
         RSe561p,2) - 1./108.*m44*RSe451p*RSe563p - 1./108.*m44*RSe451p
         *RSe453p + 1./108.*m44*RSe451p*RSe561p + 1./216.*m44*pow(
         RSe451p,2) );

      F13p6x23RV +=  + Bb(me2,mu2)*AbV(m55,L) * ( 1./36.*m55*pow(
         RSe463p,2) - 1./108.*m55*RS546pe*pow(RSe463p,2) - 1./18.*m55*
         RSe461p*RSe463p + 1./54.*m55*RSe461p*RS546pe*RSe463p + 1./36.*
         m55*pow(RSe461p,2) - 1./108.*m55*pow(RSe461p,2)*RS546pe );

      F13p6x23RV +=  + Bb(me2,mu2)*AbV(m56,L) * (  - 1./108.*me2*pow(
         RSe463p,2) + 1./54.*me2*RSe453p*RSe463p - 1./108.*me2*pow(
         RSe453p,2) + 1./54.*me2*RSe461p*RSe463p - 1./54.*me2*RSe461p*
         RSe453p - 1./108.*me2*pow(RSe461p,2) - 1./54.*me2*RSe451p*
         RSe463p + 1./54.*me2*RSe451p*RSe453p + 1./54.*me2*RSe451p*
         RSe461p - 1./108.*me2*pow(RSe451p,2) + 1./216.*m66*pow(
         RSe463p,2) + 1./108.*m66*RSe453p*RSe463p + 1./72.*m66*pow(
         RSe453p,2) - 1./108.*m66*RSe461p*RSe463p - 1./108.*m66*RSe461p
         *RSe453p + 1./216.*m66*pow(RSe461p,2) - 1./108.*m66*RSe451p*
         RSe463p - 1./36.*m66*RSe451p*RSe453p + 1./108.*m66*RSe451p*
         RSe461p + 1./72.*m66*pow(RSe451p,2) - 1./108.*m56*pow(
         RSe463p,2) + 1./54.*m56*RSe453p*RSe463p - 1./108.*m56*pow(
         RSe453p,2) + 1./54.*m56*RSe461p*RSe463p - 1./54.*m56*RSe461p*
         RSe453p - 1./108.*m56*pow(RSe461p,2) - 1./54.*m56*RSe451p*
         RSe463p + 1./54.*m56*RSe451p*RSe453p + 1./54.*m56*RSe451p*
         RSe461p );

      F13p6x23RV +=  + Bb(me2,mu2)*AbV(m56,L) * (  - 1./108.*m56*pow(
         RSe451p,2) + 1./72.*m55*pow(RSe463p,2) + 1./108.*m55*RSe453p*
         RSe463p + 1./216.*m55*pow(RSe453p,2) - 1./36.*m55*RSe461p*
         RSe463p - 1./108.*m55*RSe461p*RSe453p + 1./72.*m55*pow(
         RSe461p,2) - 1./108.*m55*RSe451p*RSe463p - 1./108.*m55*RSe451p
         *RSe453p + 1./108.*m55*RSe451p*RSe461p + 1./216.*m55*pow(
         RSe451p,2) );

      F13p6x23RV +=  + Bb(me2,mu2)*AbV(m66,L) * ( 1./36.*m66*pow(
         RSe453p,2) - 1./108.*m66*RS645pe*pow(RSe453p,2) - 1./18.*m66*
         RSe451p*RSe453p + 1./54.*m66*RSe451p*RS645pe*RSe453p + 1./36.*
         m66*pow(RSe451p,2) - 1./108.*m66*pow(RSe451p,2)*RS645pe );

      F13p6x23RV +=  + Bb(me2,mu2)*AbV(mp2,L) * (  - 1./108.*m66*
         RSp456e*pow(RSe453p,2) + 1./54.*m66*RSp456e*RSe451p*RSe453p - 
         1./108.*m66*RSp456e*pow(RSe451p,2) - 1./108.*m55*RSp465e*pow(
         RSe463p,2) + 1./54.*m55*RSp465e*RSe461p*RSe463p - 1./108.*m55*
         RSp465e*pow(RSe461p,2) - 1./108.*m44*RSp564e*pow(RSe563p,2) + 
         1./54.*m44*RSp564e*RSe561p*RSe563p - 1./108.*m44*RSp564e*pow(
         RSe561p,2) );

      F13p6x23RV +=  + Bb(me2,mu2)*AbV(me2,L) * (  - 1./108.*m66*
         RSe456p*pow(RSe453p,2) + 1./54.*m66*RSe456p*RSe451p*RSe453p - 
         1./108.*m66*RSe456p*pow(RSe451p,2) - 1./108.*m55*RSe465p*pow(
         RSe463p,2) + 1./54.*m55*RSe465p*RSe461p*RSe463p - 1./108.*m55*
         RSe465p*pow(RSe461p,2) - 1./108.*m44*RSe564p*pow(RSe563p,2) + 
         1./54.*m44*RSe564p*RSe561p*RSe563p - 1./108.*m44*RSe564p*pow(
         RSe561p,2) );

      F13p6x23RV +=  + Cb(m11,mu2)*AbV(m11,L) * ( 1./18.*m11*RD1456ep*
         RC1456pe );

      F13p6x23RV +=  + Cb(m11,mu2)*AbV(m14,L) * (  - 1./36.*m44*
         RD1456ep + 1./18.*m14*RD1456ep - 1./36.*m11*RD1456ep );

      F13p6x23RV +=  + Cb(m11,mu2)*AbV(m15,L) * (  - 1./36.*m55*
         RD1456ep + 1./18.*m15*RD1456ep - 1./36.*m11*RD1456ep );

      F13p6x23RV +=  + Cb(m11,mu2)*AbV(m16,L) * (  - 1./36.*m66*
         RD1456ep + 1./18.*m16*RD1456ep - 1./36.*m11*RD1456ep );

      F13p6x23RV +=  + Cb(m11,mu2)*AbV(mp2,L) * ( 1./18.*m11*RSp45611e*
         RD1456ep );

      F13p6x23RV +=  + Cb(m11,mu2)*AbV(me2,L) * ( 1./18.*m11*RSe45611p*
         RD1456ep );

      F13p6x23RV +=  + Cb(m11,mu2)*BbV(m11,L) * ( 1./18.*m11*pow(
         RD1456ep,2) );

      F13p6x23RV +=  + Cb(m33,mu2)*AbV(m33,L) * ( 1./18.*m33*RD3456ep*
         RC3456pe );

      F13p6x23RV +=  + Cb(m33,mu2)*AbV(m34,L) * (  - 1./36.*m44*
         RD3456ep + 1./18.*m34*RD3456ep - 1./36.*m33*RD3456ep );

      F13p6x23RV +=  + Cb(m33,mu2)*AbV(m35,L) * (  - 1./36.*m55*
         RD3456ep + 1./18.*m35*RD3456ep - 1./36.*m33*RD3456ep );

      F13p6x23RV +=  + Cb(m33,mu2)*AbV(m36,L) * (  - 1./36.*m66*
         RD3456ep + 1./18.*m36*RD3456ep - 1./36.*m33*RD3456ep );

      F13p6x23RV +=  + Cb(m33,mu2)*AbV(mp2,L) * ( 1./18.*m33*RSp45633e*
         RD3456ep );

      F13p6x23RV +=  + Cb(m33,mu2)*AbV(me2,L) * ( 1./18.*m33*RSe45633p*
         RD3456ep );

      F13p6x23RV +=  + Cb(m33,mu2)*BbV(m33,L) * ( 1./18.*m33*pow(
         RD3456ep,2) );

      F13p6x23RV +=  + AbV(m11,L)*pi16 * ( 1./72.*RD3456ep*RS14563pe - 
         1./144.*RC1456pe*RD3456ep - 1./48.*RD1456ep + 1./72.*RD1456ep*
         RS14563pe - 1./144.*RD1456ep*RC1456pe - 1./36.*me2*RSe45613p*
         RS14563pe + 1./72.*me2*RSe45633p*RS14563pe + 1./72.*me2*
         RC1456pe*RSe45613p - 1./144.*me2*RC1456pe*RSe45633p + 1./72.*
         me2*RSe45611p*RS14563pe - 1./144.*me2*RSe45611p*RC1456pe - 1./
         36.*mp2*RSp45613e*RS14563pe + 1./72.*mp2*RSp45633e*RS14563pe
          + 1./72.*mp2*RC1456pe*RSp45613e - 1./144.*mp2*RC1456pe*
         RSp45633e + 1./72.*mp2*RSp45611e*RS14563pe - 1./144.*mp2*
         RSp45611e*RC1456pe - 1./24.*m36*RS14563pe + 1./48.*m36*
         RC1456pe - 1./24.*m35*RS14563pe + 1./48.*m35*RC1456pe - 1./24.
         *m34*RS14563pe + 1./48.*m34*RC1456pe - 1./36.*m33*RS14563pe*
         RS34561pe + 1./72.*m33*RC3456pe*RS14563pe + 1./72.*m33*
         RC1456pe*RS34561pe - 1./144.*m33*RC1456pe*RC3456pe - 1./24.*
         m16*RS14563pe + 1./48.*m16*RC1456pe - 1./24.*m15*RS14563pe + 1.
         /48.*m15*RC1456pe );

      F13p6x23RV +=  + AbV(m11,L)*pi16 * (  - 1./24.*m14*RS14563pe + 1./
         48.*m14*RC1456pe - 1./36.*m13*RS14563pe*RS34561pe - 1./36.*m13
         *pow(RS14563pe,2) - 1./36.*m13*RSe45613p*RS14563pe - 1./36.*
         m13*RSp45613e*RS14563pe + 1./72.*m13*RC3456pe*RS14563pe + 1./
         72.*m13*RSe45633p*RS14563pe + 1./72.*m13*RSp45633e*RS14563pe
          + 1./72.*m13*RC1456pe*RS34561pe + 1./36.*m13*RC1456pe*
         RS14563pe + 1./72.*m13*RC1456pe*RSe45613p + 1./72.*m13*
         RC1456pe*RSp45613e - 1./144.*m13*RC1456pe*RC3456pe - 1./144.*
         m13*RC1456pe*RSe45633p - 1./144.*m13*RC1456pe*RSp45633e - 1./
         144.*m13*pow(RC1456pe,2) + 1./72.*m13*RSe45611p*RS14563pe - 1./
         144.*m13*RSe45611p*RC1456pe + 1./72.*m13*RSp45611e*RS14563pe
          - 1./144.*m13*RSp45611e*RC1456pe + 1./16.*m11 + 1./24.*m11*
         RS14563pe - 1./36.*m11*pow(RS14563pe,2) - 1./48.*m11*RC1456pe
          + 1./36.*m11*RC1456pe*RS14563pe - 1./144.*m11*pow(RC1456pe,2)
          - 1./48.*m11*RS145pe - 1./48.*m11*RS146pe - 1./48.*m11*
         RS156pe );

      F13p6x23RV +=  + pow(AbV(m11,L),2) * ( 1./16. + 1./72.*pow(
         RS14563pe,2) - 1./72.*RC1456pe*RS14563pe + 1./288.*pow(
         RC1456pe,2) );

      F13p6x23RV +=  + AbV(m11,L)*AbV(m13,L) * (  - 1./24. + 1./162.*
         RS14563pe*RS34561pe + 1./162.*pow(RS14563pe,2) + 1./162.*
         RSe45613p*RS14563pe + 1./162.*RSp45613e*RS14563pe - 1./324.*
         RC3456pe*RS14563pe - 1./324.*RSe45633p*RS14563pe - 1./324.*
         RSp45633e*RS14563pe - 1./324.*RC1456pe*RS34561pe - 1./162.*
         RC1456pe*RS14563pe - 1./324.*RC1456pe*RSe45613p - 1./324.*
         RC1456pe*RSp45613e + 1./648.*RC1456pe*RC3456pe + 1./648.*
         RC1456pe*RSe45633p + 1./648.*RC1456pe*RSp45633e + 1./648.*pow(
         RC1456pe,2) - 1./324.*RSe45611p*RS14563pe + 1./648.*RSe45611p*
         RC1456pe - 1./324.*RSp45611e*RS14563pe + 1./648.*RSp45611e*
         RC1456pe );

      F13p6x23RV +=  + AbV(m11,L)*AbV(m14,L) * (  - 1./36.*RS14563pe + 
         7./144.*RC1456pe - 5./72.*RS156pe );

      F13p6x23RV +=  + AbV(m11,L)*AbV(m15,L) * (  - 1./36.*RS14563pe + 
         7./144.*RC1456pe - 5./72.*RS146pe );

      F13p6x23RV +=  + AbV(m11,L)*AbV(m16,L) * (  - 1./36.*RS14563pe + 
         7./144.*RC1456pe - 5./72.*RS145pe );

      F13p6x23RV +=  + AbV(m11,L)*AbV(m33,L) * ( 1./36.*RS14563pe*
         RS34561pe - 1./72.*RC3456pe*RS14563pe - 1./72.*RC1456pe*
         RS34561pe + 1./144.*RC1456pe*RC3456pe );

      F13p6x23RV +=  + AbV(m11,L)*AbV(m34,L) * ( 1./36.*RS14563pe - 1./
         48.*RC1456pe + 1./72.*RS156pe );

      F13p6x23RV +=  + AbV(m11,L)*AbV(m35,L) * ( 1./36.*RS14563pe - 1./
         48.*RC1456pe + 1./72.*RS146pe );

      F13p6x23RV +=  + AbV(m11,L)*AbV(m36,L) * ( 1./36.*RS14563pe - 1./
         48.*RC1456pe + 1./72.*RS145pe );

      F13p6x23RV +=  + AbV(m11,L)*AbV(m45,L) * (  - 1./108.*pow(
         RS146pe,2) + 1./54.*RS156pe*RS146pe - 1./108.*pow(RS156pe,2) )
         ;

      F13p6x23RV +=  + AbV(m11,L)*AbV(m46,L) * (  - 1./108.*pow(
         RS145pe,2) + 1./54.*RS156pe*RS145pe - 1./108.*pow(RS156pe,2) )
         ;

      F13p6x23RV +=  + AbV(m11,L)*AbV(m56,L) * (  - 1./108.*pow(
         RS145pe,2) + 1./54.*RS146pe*RS145pe - 1./108.*pow(RS146pe,2) )
         ;

      F13p6x23RV +=  + AbV(m11,L)*AbV(mp2,L) * ( 1./36.*RSp45613e*
         RS14563pe - 1./72.*RSp45633e*RS14563pe - 1./72.*RC1456pe*
         RSp45613e + 1./144.*RC1456pe*RSp45633e - 1./72.*RSp45611e*
         RS14563pe + 1./144.*RSp45611e*RC1456pe );

      F13p6x23RV +=  + AbV(m11,L)*AbV(me2,L) * ( 1./36.*RSe45613p*
         RS14563pe - 1./72.*RSe45633p*RS14563pe - 1./72.*RC1456pe*
         RSe45613p + 1./144.*RC1456pe*RSe45633p - 1./72.*RSe45611p*
         RS14563pe + 1./144.*RSe45611p*RC1456pe );

      F13p6x23RV +=  + AbV(m11,L)*BbV(m11,m33,L) * (  - 1./18.*m11*
         RC1456pe*RS34561pe );

      F13p6x23RV +=  + AbV(m11,L)*BbV(m11,mp2,L) * (  - 1./18.*m11*
         RC1456pe*RSp45613e + 1./18.*m11*RSp45611e*RC1456pe );

      F13p6x23RV +=  + AbV(m11,L)*BbV(m11,me2,L) * (  - 1./18.*m11*
         RC1456pe*RSe45613p + 1./18.*m11*RSe45611p*RC1456pe );

      F13p6x23RV +=  + AbV(m11,L)*BbV(m11,L) * (  - 1./72.*RD1456ep*
         RS14563pe + 1./144.*RD1456ep*RC1456pe + 1./4.*m11 - 1./18.*m11
         *RC1456pe*RS14563pe + 1./18.*m11*pow(RC1456pe,2) );

      F13p6x23RV +=  + AbV(m11,L)*BbV(m14,L) * ( 1./72.*RD1456ep - 1./
         144.*m44*RC1456pe - 1./72.*m44*RS156pe + 1./72.*m14*RC1456pe
          - 1./36.*m14*RS156pe - 1./144.*m11*RC1456pe - 1./24.*m11*
         RS156pe );

      F13p6x23RV +=  + AbV(m11,L)*BbV(m15,L) * ( 1./72.*RD1456ep - 1./
         144.*m55*RC1456pe - 1./72.*m55*RS146pe + 1./72.*m15*RC1456pe
          - 1./36.*m15*RS146pe - 1./144.*m11*RC1456pe - 1./24.*m11*
         RS146pe );

      F13p6x23RV +=  + AbV(m11,L)*BbV(m16,L) * ( 1./72.*RD1456ep - 1./
         144.*m66*RC1456pe - 1./72.*m66*RS145pe + 1./72.*m16*RC1456pe
          - 1./36.*m16*RS145pe - 1./144.*m11*RC1456pe - 1./24.*m11*
         RS145pe );

      F13p6x23RV +=  + AbV(m11,L)*BbV(m33,L) * (  - 1./72.*RD3456ep*
         RS14563pe + 1./144.*RC1456pe*RD3456ep );

      F13p6x23RV +=  + AbV(m11,L)*CbV(m11,L) * ( 1./18.*m11*RD1456ep*
         RC1456pe );

      F13p6x23RV +=  + AbV(m13,L)*pi16 * ( 1./8.*m33 - 1./4.*m13 - 1./
         72.*m13*pow(RS34561pe,2) - 1./36.*m13*RS14563pe*RS34561pe - 1./
         72.*m13*pow(RS14563pe,2) - 1./36.*m13*RSe45613p*RS34561pe - 1./
         36.*m13*RSe45613p*RS14563pe - 1./72.*m13*pow(RSe45613p,2) - 1./
         36.*m13*RSp45613e*RS34561pe - 1./36.*m13*RSp45613e*RS14563pe
          - 1./36.*m13*RSp45613e*RSe45613p - 1./72.*m13*pow(
         RSp45613e,2) + 1./72.*m13*RC3456pe*RS34561pe + 1./72.*m13*
         RC3456pe*RS14563pe + 1./72.*m13*RC3456pe*RSe45613p + 1./72.*
         m13*RC3456pe*RSp45613e - 1./288.*m13*pow(RC3456pe,2) + 1./72.*
         m13*RSe45633p*RS34561pe + 1./72.*m13*RSe45633p*RS14563pe + 1./
         72.*m13*RSe45633p*RSe45613p + 1./72.*m13*RSe45633p*RSp45613e
          - 1./144.*m13*RSe45633p*RC3456pe - 1./288.*m13*pow(
         RSe45633p,2) + 1./72.*m13*RSp45633e*RS34561pe + 1./72.*m13*
         RSp45633e*RS14563pe + 1./72.*m13*RSp45633e*RSe45613p + 1./72.*
         m13*RSp45633e*RSp45613e - 1./144.*m13*RSp45633e*RC3456pe - 1./
         144.*m13*RSp45633e*RSe45633p );

      F13p6x23RV +=  + AbV(m13,L)*pi16 * (  - 1./288.*m13*pow(
         RSp45633e,2) + 1./72.*m13*RC1456pe*RS34561pe + 1./72.*m13*
         RC1456pe*RS14563pe + 1./72.*m13*RC1456pe*RSe45613p + 1./72.*
         m13*RC1456pe*RSp45613e - 1./144.*m13*RC1456pe*RC3456pe - 1./
         144.*m13*RC1456pe*RSe45633p - 1./144.*m13*RC1456pe*RSp45633e
          - 1./288.*m13*pow(RC1456pe,2) + 1./72.*m13*RSe45611p*
         RS34561pe + 1./72.*m13*RSe45611p*RS14563pe + 1./72.*m13*
         RSe45611p*RSe45613p + 1./72.*m13*RSe45611p*RSp45613e - 1./144.
         *m13*RSe45611p*RC3456pe - 1./144.*m13*RSe45611p*RSe45633p - 1./
         144.*m13*RSe45611p*RSp45633e - 1./144.*m13*RSe45611p*RC1456pe
          - 1./288.*m13*pow(RSe45611p,2) + 1./72.*m13*RSp45611e*
         RS34561pe + 1./72.*m13*RSp45611e*RS14563pe + 1./72.*m13*
         RSp45611e*RSe45613p + 1./72.*m13*RSp45611e*RSp45613e - 1./144.
         *m13*RSp45611e*RC3456pe - 1./144.*m13*RSp45611e*RSe45633p - 1./
         144.*m13*RSp45611e*RSp45633e - 1./144.*m13*RSp45611e*RC1456pe
          - 1./144.*m13*RSp45611e*RSe45611p );

      F13p6x23RV +=  + AbV(m13,L)*pi16 * (  - 1./288.*m13*pow(
         RSp45611e,2) + 1./8.*m11 );

      F13p6x23RV +=  + pow(AbV(m13,L),2) * ( 1./8. );

      F13p6x23RV +=  + AbV(m13,L)*AbV(m33,L) * (  - 5./24. + 1./162.*
         pow(RS34561pe,2) + 1./162.*RS14563pe*RS34561pe + 1./162.*
         RSe45613p*RS34561pe + 1./162.*RSp45613e*RS34561pe - 1./162.*
         RC3456pe*RS34561pe - 1./324.*RC3456pe*RS14563pe - 1./324.*
         RC3456pe*RSe45613p - 1./324.*RC3456pe*RSp45613e + 1./648.*pow(
         RC3456pe,2) - 1./324.*RSe45633p*RS34561pe + 1./648.*RSe45633p*
         RC3456pe - 1./324.*RSp45633e*RS34561pe + 1./648.*RSp45633e*
         RC3456pe - 1./324.*RC1456pe*RS34561pe + 1./648.*RC1456pe*
         RC3456pe - 1./324.*RSe45611p*RS34561pe + 1./648.*RSe45611p*
         RC3456pe - 1./324.*RSp45611e*RS34561pe + 1./648.*RSp45611e*
         RC3456pe );

      F13p6x23RV +=  + AbV(m13,L)*AbV(mp2,L) * ( 1./162.*RSp45613e*
         RS34561pe + 1./162.*RSp45613e*RS14563pe + 1./162.*RSp45613e*
         RSe45613p + 1./162.*pow(RSp45613e,2) - 1./324.*RC3456pe*
         RSp45613e - 1./324.*RSe45633p*RSp45613e - 1./324.*RSp45633e*
         RS34561pe - 1./324.*RSp45633e*RS14563pe - 1./324.*RSp45633e*
         RSe45613p - 1./162.*RSp45633e*RSp45613e + 1./648.*RSp45633e*
         RC3456pe + 1./648.*RSp45633e*RSe45633p + 1./648.*pow(
         RSp45633e,2) - 1./324.*RC1456pe*RSp45613e + 1./648.*RC1456pe*
         RSp45633e - 1./324.*RSe45611p*RSp45613e + 1./648.*RSe45611p*
         RSp45633e - 1./324.*RSp45611e*RS34561pe - 1./324.*RSp45611e*
         RS14563pe - 1./324.*RSp45611e*RSe45613p - 1./162.*RSp45611e*
         RSp45613e + 1./648.*RSp45611e*RC3456pe + 1./648.*RSp45611e*
         RSe45633p + 1./324.*RSp45611e*RSp45633e + 1./648.*RSp45611e*
         RC1456pe + 1./648.*RSp45611e*RSe45611p + 1./648.*pow(
         RSp45611e,2) );

      F13p6x23RV +=  + AbV(m13,L)*AbV(me2,L) * ( 1./162.*RSe45613p*
         RS34561pe + 1./162.*RSe45613p*RS14563pe + 1./162.*pow(
         RSe45613p,2) + 1./162.*RSp45613e*RSe45613p - 1./324.*RC3456pe*
         RSe45613p - 1./324.*RSe45633p*RS34561pe - 1./324.*RSe45633p*
         RS14563pe - 1./162.*RSe45633p*RSe45613p - 1./324.*RSe45633p*
         RSp45613e + 1./648.*RSe45633p*RC3456pe + 1./648.*pow(
         RSe45633p,2) - 1./324.*RSp45633e*RSe45613p + 1./648.*RSp45633e
         *RSe45633p - 1./324.*RC1456pe*RSe45613p + 1./648.*RC1456pe*
         RSe45633p - 1./324.*RSe45611p*RS34561pe - 1./324.*RSe45611p*
         RS14563pe - 1./162.*RSe45611p*RSe45613p - 1./324.*RSe45611p*
         RSp45613e + 1./648.*RSe45611p*RC3456pe + 1./324.*RSe45611p*
         RSe45633p + 1./648.*RSe45611p*RSp45633e + 1./648.*RSe45611p*
         RC1456pe + 1./648.*pow(RSe45611p,2) - 1./324.*RSp45611e*
         RSe45613p + 1./648.*RSp45611e*RSe45633p + 1./648.*RSp45611e*
         RSe45611p );

      F13p6x23RV +=  + AbV(m13,L)*BbV(m11,m33,L) * (  - 1./12.*m33 - 1./
         6.*m13 - 1./4.*m11 );

      F13p6x23RV +=  + AbV(m13,L)*BbV(m11,L) * (  - 1./324.*RD1456ep*
         RS34561pe - 1./324.*RD1456ep*RS14563pe - 1./324.*RD1456ep*
         RSe45613p - 1./324.*RD1456ep*RSp45613e + 1./648.*RD1456ep*
         RC3456pe + 1./648.*RD1456ep*RSe45633p + 1./648.*RD1456ep*
         RSp45633e + 1./648.*RD1456ep*RC1456pe + 1./648.*RSe45611p*
         RD1456ep + 1./648.*RSp45611e*RD1456ep );

      F13p6x23RV +=  + AbV(m13,L)*BbV(m33,L) * (  - 1./324.*RD3456ep*
         RS34561pe - 1./324.*RD3456ep*RS14563pe - 1./324.*RD3456ep*
         RSe45613p - 1./324.*RD3456ep*RSp45613e + 1./648.*RD3456ep*
         RC3456pe + 1./648.*RSe45633p*RD3456ep + 1./648.*RSp45633e*
         RD3456ep + 1./648.*RC1456pe*RD3456ep + 1./648.*RSe45611p*
         RD3456ep + 1./648.*RSp45611e*RD3456ep );

      F13p6x23RV +=  + AbV(m14,L)*pi16 * ( 1./48.*RD3456ep + 1./48.*me2
         *RSe45633p - 1./24.*me2*RSe563p + 1./48.*me2*RSe564p + 1./48.*
         mp2*RSp45633e - 1./24.*mp2*RSp563e + 1./48.*mp2*RSp564e - 1./
         16.*m46 - 1./16.*m45 - 1./16.*m44 + 1./48.*m44*RS456pe - 1./16.
         *m36 - 1./16.*m35 - 1./16.*m34 + 1./48.*m34*RC3456pe + 1./48.*
         m34*RSe45633p + 1./48.*m34*RSp45633e - 1./24.*m34*RS356pe - 1./
         24.*m34*RSe563p - 1./24.*m34*RSp563e + 1./48.*m34*RS456pe + 1./
         48.*m34*RSe564p + 1./48.*m34*RSp564e + 1./48.*m33*RC3456pe - 1.
         /24.*m33*RS356pe + 1./16.*m14 - 1./48.*m14*RS34561pe - 1./48.*
         m14*RS14563pe - 1./48.*m14*RSe45613p - 1./48.*m14*RSp45613e + 
         1./48.*m14*RS356pe + 1./48.*m14*RSe563p + 1./48.*m14*RSp563e
          + 1./48.*m14*RC1456pe + 1./48.*m14*RSe45611p + 1./48.*m14*
         RSp45611e - 1./48.*m14*RS156pe - 1./48.*m14*RSe561p - 1./48.*
         m14*RSp561e );

      F13p6x23RV +=  + pow(AbV(m14,L),2) * (  - 1./32. );

      F13p6x23RV +=  + AbV(m14,L)*AbV(m15,L) * (  - 1./16. );

      F13p6x23RV +=  + AbV(m14,L)*AbV(m16,L) * (  - 1./16. );

      F13p6x23RV +=  + AbV(m14,L)*AbV(m33,L) * (  - 1./36.*RS34561pe - 
         1./48.*RC3456pe + 5./72.*RS356pe );

      F13p6x23RV +=  + AbV(m14,L)*AbV(m34,L) * ( 1./16. + 7./216.*
         RS34561pe + 7./216.*RS14563pe + 7./216.*RSe45613p + 7./216.*
         RSp45613e - 1./216.*RC3456pe - 1./216.*RSe45633p - 1./216.*
         RSp45633e - 5./216.*RS356pe - 5./216.*RSe563p - 5./216.*
         RSp563e - 1./216.*RC1456pe - 1./216.*RSe45611p - 1./216.*
         RSp45611e - 5./216.*RS156pe + 5./216.*RS456pe - 5./216.*
         RSe561p - 5./216.*RSp561e + 5./216.*RSe564p + 5./216.*RSp564e
          );

      F13p6x23RV +=  + AbV(m14,L)*AbV(m35,L) * ( 1./16. );

      F13p6x23RV +=  + AbV(m14,L)*AbV(m36,L) * ( 1./16. );

      F13p6x23RV +=  + AbV(m14,L)*AbV(mp2,L) * (  - 1./36.*RSp45613e - 
         1./48.*RSp45633e + 5./72.*RSp563e + 7./144.*RSp45611e - 5./72.
         *RSp561e );

      F13p6x23RV +=  + AbV(m14,L)*AbV(me2,L) * (  - 1./36.*RSe45613p - 
         1./48.*RSe45633p + 5./72.*RSe563p + 7./144.*RSe45611p - 5./72.
         *RSe561p );

      F13p6x23RV +=  + AbV(m14,L)*BbV(m11,m33,L) * ( 1./36.*m44*
         RS34561pe + 1./36.*m44*RS356pe - 1./18.*m14*RS34561pe + 1./18.
         *m14*RS356pe + 1./36.*m11*RS34561pe + 1./12.*m11*RS356pe );

      F13p6x23RV +=  + AbV(m14,L)*BbV(m11,mp2,L) * ( 1./36.*m44*
         RSp45613e + 1./36.*m44*RSp563e - 1./36.*m44*RSp45611e - 1./36.
         *m44*RSp561e - 1./18.*m14*RSp45613e + 1./18.*m14*RSp563e + 1./
         18.*m14*RSp45611e - 1./18.*m14*RSp561e + 1./36.*m11*RSp45613e
          + 1./12.*m11*RSp563e - 1./36.*m11*RSp45611e - 1./12.*m11*
         RSp561e );

      F13p6x23RV +=  + AbV(m14,L)*BbV(m11,me2,L) * ( 1./36.*m44*
         RSe45613p + 1./36.*m44*RSe563p - 1./36.*m44*RSe45611p - 1./36.
         *m44*RSe561p - 1./18.*m14*RSe45613p + 1./18.*m14*RSe563p + 1./
         18.*m14*RSe45611p - 1./18.*m14*RSe561p + 1./36.*m11*RSe45613p
          + 1./12.*m11*RSe563p - 1./36.*m11*RSe45611p - 1./12.*m11*
         RSe561p );

      F13p6x23RV +=  + AbV(m14,L)*BbV(m11,L) * ( 7./144.*RD1456ep + 1./
         36.*m44*RS14563pe - 1./36.*m44*RC1456pe - 1./36.*m44*RS156pe
          - 1./18.*m14*RS14563pe + 1./18.*m14*RC1456pe - 1./18.*m14*
         RS156pe + 1./36.*m11*RS14563pe - 1./36.*m11*RC1456pe - 1./12.*
         m11*RS156pe );

      F13p6x23RV +=  + AbV(m14,L)*BbV(m14,L) * ( 1./24.*m44 - 1./12.*
         m14 + 1./24.*m11 );

      F13p6x23RV +=  + AbV(m14,L)*BbV(m15,L) * ( 1./48.*m55 + 1./48.*
         m44 - 1./24.*m15 - 1./24.*m14 + 1./24.*m11 );

      F13p6x23RV +=  + AbV(m14,L)*BbV(m16,L) * ( 1./48.*m66 + 1./48.*
         m44 - 1./24.*m16 - 1./24.*m14 + 1./24.*m11 );

      F13p6x23RV +=  + AbV(m14,L)*BbV(m33,L) * (  - 1./48.*RD3456ep );

      F13p6x23RV +=  + AbV(m14,L)*CbV(m11,L) * (  - 1./36.*m44*RD1456ep
          + 1./18.*m14*RD1456ep - 1./36.*m11*RD1456ep );

      F13p6x23RV +=  + AbV(m15,L)*pi16 * ( 1./48.*RD3456ep + 1./48.*me2
         *RSe45633p - 1./24.*me2*RSe463p + 1./48.*me2*RSe465p + 1./48.*
         mp2*RSp45633e - 1./24.*mp2*RSp463e + 1./48.*mp2*RSp465e - 1./
         16.*m56 - 1./16.*m55 + 1./48.*m55*RS546pe - 1./16.*m45 - 1./16.
         *m36 - 1./16.*m35 + 1./48.*m35*RC3456pe + 1./48.*m35*RSe45633p
          + 1./48.*m35*RSp45633e - 1./24.*m35*RS346pe - 1./24.*m35*
         RSe463p - 1./24.*m35*RSp463e + 1./48.*m35*RS546pe + 1./48.*m35
         *RSe465p + 1./48.*m35*RSp465e - 1./16.*m34 + 1./48.*m33*
         RC3456pe - 1./24.*m33*RS346pe + 1./16.*m15 - 1./48.*m15*
         RS34561pe - 1./48.*m15*RS14563pe - 1./48.*m15*RSe45613p - 1./
         48.*m15*RSp45613e + 1./48.*m15*RS346pe + 1./48.*m15*RSe463p + 
         1./48.*m15*RSp463e + 1./48.*m15*RC1456pe + 1./48.*m15*
         RSe45611p + 1./48.*m15*RSp45611e - 1./48.*m15*RS146pe - 1./48.
         *m15*RSe461p - 1./48.*m15*RSp461e );

      F13p6x23RV +=  + pow(AbV(m15,L),2) * (  - 1./32. );

      F13p6x23RV +=  + AbV(m15,L)*AbV(m16,L) * (  - 1./16. );

      F13p6x23RV +=  + AbV(m15,L)*AbV(m33,L) * (  - 1./36.*RS34561pe - 
         1./48.*RC3456pe + 5./72.*RS346pe );

      F13p6x23RV +=  + AbV(m15,L)*AbV(m34,L) * ( 1./16. );

      F13p6x23RV +=  + AbV(m15,L)*AbV(m35,L) * ( 1./16. + 7./216.*
         RS34561pe + 7./216.*RS14563pe + 7./216.*RSe45613p + 7./216.*
         RSp45613e - 1./216.*RC3456pe - 1./216.*RSe45633p - 1./216.*
         RSp45633e - 5./216.*RS346pe - 5./216.*RSe463p - 5./216.*
         RSp463e - 1./216.*RC1456pe - 1./216.*RSe45611p - 1./216.*
         RSp45611e - 5./216.*RS146pe + 5./216.*RS546pe - 5./216.*
         RSe461p - 5./216.*RSp461e + 5./216.*RSe465p + 5./216.*RSp465e
          );

      F13p6x23RV +=  + AbV(m15,L)*AbV(m36,L) * ( 1./16. );

      F13p6x23RV +=  + AbV(m15,L)*AbV(mp2,L) * (  - 1./36.*RSp45613e - 
         1./48.*RSp45633e + 5./72.*RSp463e + 7./144.*RSp45611e - 5./72.
         *RSp461e );

      F13p6x23RV +=  + AbV(m15,L)*AbV(me2,L) * (  - 1./36.*RSe45613p - 
         1./48.*RSe45633p + 5./72.*RSe463p + 7./144.*RSe45611p - 5./72.
         *RSe461p );

      F13p6x23RV +=  + AbV(m15,L)*BbV(m11,m33,L) * ( 1./36.*m55*
         RS34561pe + 1./36.*m55*RS346pe - 1./18.*m15*RS34561pe + 1./18.
         *m15*RS346pe + 1./36.*m11*RS34561pe + 1./12.*m11*RS346pe );

      F13p6x23RV +=  + AbV(m15,L)*BbV(m11,mp2,L) * ( 1./36.*m55*
         RSp45613e + 1./36.*m55*RSp463e - 1./36.*m55*RSp45611e - 1./36.
         *m55*RSp461e - 1./18.*m15*RSp45613e + 1./18.*m15*RSp463e + 1./
         18.*m15*RSp45611e - 1./18.*m15*RSp461e + 1./36.*m11*RSp45613e
          + 1./12.*m11*RSp463e - 1./36.*m11*RSp45611e - 1./12.*m11*
         RSp461e );

      F13p6x23RV +=  + AbV(m15,L)*BbV(m11,me2,L) * ( 1./36.*m55*
         RSe45613p + 1./36.*m55*RSe463p - 1./36.*m55*RSe45611p - 1./36.
         *m55*RSe461p - 1./18.*m15*RSe45613p + 1./18.*m15*RSe463p + 1./
         18.*m15*RSe45611p - 1./18.*m15*RSe461p + 1./36.*m11*RSe45613p
          + 1./12.*m11*RSe463p - 1./36.*m11*RSe45611p - 1./12.*m11*
         RSe461p );

      F13p6x23RV +=  + AbV(m15,L)*BbV(m11,L) * ( 7./144.*RD1456ep + 1./
         36.*m55*RS14563pe - 1./36.*m55*RC1456pe - 1./36.*m55*RS146pe
          - 1./18.*m15*RS14563pe + 1./18.*m15*RC1456pe - 1./18.*m15*
         RS146pe + 1./36.*m11*RS14563pe - 1./36.*m11*RC1456pe - 1./12.*
         m11*RS146pe );

      F13p6x23RV +=  + AbV(m15,L)*BbV(m14,L) * ( 1./48.*m55 + 1./48.*
         m44 - 1./24.*m15 - 1./24.*m14 + 1./24.*m11 );

      F13p6x23RV +=  + AbV(m15,L)*BbV(m15,L) * ( 1./24.*m55 - 1./12.*
         m15 + 1./24.*m11 );

      F13p6x23RV +=  + AbV(m15,L)*BbV(m16,L) * ( 1./48.*m66 + 1./48.*
         m55 - 1./24.*m16 - 1./24.*m15 + 1./24.*m11 );

      F13p6x23RV +=  + AbV(m15,L)*BbV(m33,L) * (  - 1./48.*RD3456ep );

      F13p6x23RV +=  + AbV(m15,L)*CbV(m11,L) * (  - 1./36.*m55*RD1456ep
          + 1./18.*m15*RD1456ep - 1./36.*m11*RD1456ep );

      F13p6x23RV +=  + AbV(m16,L)*pi16 * ( 1./48.*RD3456ep + 1./48.*me2
         *RSe45633p - 1./24.*me2*RSe453p + 1./48.*me2*RSe456p + 1./48.*
         mp2*RSp45633e - 1./24.*mp2*RSp453e + 1./48.*mp2*RSp456e - 1./
         16.*m66 + 1./48.*m66*RS645pe - 1./16.*m56 - 1./16.*m46 - 1./16.
         *m36 + 1./48.*m36*RC3456pe + 1./48.*m36*RSe45633p + 1./48.*m36
         *RSp45633e - 1./24.*m36*RS345pe - 1./24.*m36*RSe453p - 1./24.*
         m36*RSp453e + 1./48.*m36*RS645pe + 1./48.*m36*RSe456p + 1./48.
         *m36*RSp456e - 1./16.*m35 - 1./16.*m34 + 1./48.*m33*RC3456pe
          - 1./24.*m33*RS345pe + 1./16.*m16 - 1./48.*m16*RS34561pe - 1./
         48.*m16*RS14563pe - 1./48.*m16*RSe45613p - 1./48.*m16*
         RSp45613e + 1./48.*m16*RS345pe + 1./48.*m16*RSe453p + 1./48.*
         m16*RSp453e + 1./48.*m16*RC1456pe + 1./48.*m16*RSe45611p + 1./
         48.*m16*RSp45611e - 1./48.*m16*RS145pe - 1./48.*m16*RSe451p - 
         1./48.*m16*RSp451e );

      F13p6x23RV +=  + pow(AbV(m16,L),2) * (  - 1./32. );

      F13p6x23RV +=  + AbV(m16,L)*AbV(m33,L) * (  - 1./36.*RS34561pe - 
         1./48.*RC3456pe + 5./72.*RS345pe );

      F13p6x23RV +=  + AbV(m16,L)*AbV(m34,L) * ( 1./16. );

      F13p6x23RV +=  + AbV(m16,L)*AbV(m35,L) * ( 1./16. );

      F13p6x23RV +=  + AbV(m16,L)*AbV(m36,L) * ( 1./16. + 7./216.*
         RS34561pe + 7./216.*RS14563pe + 7./216.*RSe45613p + 7./216.*
         RSp45613e - 1./216.*RC3456pe - 1./216.*RSe45633p - 1./216.*
         RSp45633e - 5./216.*RS345pe - 5./216.*RSe453p - 5./216.*
         RSp453e - 1./216.*RC1456pe - 1./216.*RSe45611p - 1./216.*
         RSp45611e - 5./216.*RS145pe + 5./216.*RS645pe - 5./216.*
         RSe451p - 5./216.*RSp451e + 5./216.*RSe456p + 5./216.*RSp456e
          );

      F13p6x23RV +=  + AbV(m16,L)*AbV(mp2,L) * (  - 1./36.*RSp45613e - 
         1./48.*RSp45633e + 5./72.*RSp453e + 7./144.*RSp45611e - 5./72.
         *RSp451e );

      F13p6x23RV +=  + AbV(m16,L)*AbV(me2,L) * (  - 1./36.*RSe45613p - 
         1./48.*RSe45633p + 5./72.*RSe453p + 7./144.*RSe45611p - 5./72.
         *RSe451p );

      F13p6x23RV +=  + AbV(m16,L)*BbV(m11,m33,L) * ( 1./36.*m66*
         RS34561pe + 1./36.*m66*RS345pe - 1./18.*m16*RS34561pe + 1./18.
         *m16*RS345pe + 1./36.*m11*RS34561pe + 1./12.*m11*RS345pe );

      F13p6x23RV +=  + AbV(m16,L)*BbV(m11,mp2,L) * ( 1./36.*m66*
         RSp45613e + 1./36.*m66*RSp453e - 1./36.*m66*RSp45611e - 1./36.
         *m66*RSp451e - 1./18.*m16*RSp45613e + 1./18.*m16*RSp453e + 1./
         18.*m16*RSp45611e - 1./18.*m16*RSp451e + 1./36.*m11*RSp45613e
          + 1./12.*m11*RSp453e - 1./36.*m11*RSp45611e - 1./12.*m11*
         RSp451e );

      F13p6x23RV +=  + AbV(m16,L)*BbV(m11,me2,L) * ( 1./36.*m66*
         RSe45613p + 1./36.*m66*RSe453p - 1./36.*m66*RSe45611p - 1./36.
         *m66*RSe451p - 1./18.*m16*RSe45613p + 1./18.*m16*RSe453p + 1./
         18.*m16*RSe45611p - 1./18.*m16*RSe451p + 1./36.*m11*RSe45613p
          + 1./12.*m11*RSe453p - 1./36.*m11*RSe45611p - 1./12.*m11*
         RSe451p );

      F13p6x23RV +=  + AbV(m16,L)*BbV(m11,L) * ( 7./144.*RD1456ep + 1./
         36.*m66*RS14563pe - 1./36.*m66*RC1456pe - 1./36.*m66*RS145pe
          - 1./18.*m16*RS14563pe + 1./18.*m16*RC1456pe - 1./18.*m16*
         RS145pe + 1./36.*m11*RS14563pe - 1./36.*m11*RC1456pe - 1./12.*
         m11*RS145pe );

      F13p6x23RV +=  + AbV(m16,L)*BbV(m14,L) * ( 1./48.*m66 + 1./48.*
         m44 - 1./24.*m16 - 1./24.*m14 + 1./24.*m11 );

      F13p6x23RV +=  + AbV(m16,L)*BbV(m15,L) * ( 1./48.*m66 + 1./48.*
         m55 - 1./24.*m16 - 1./24.*m15 + 1./24.*m11 );

      F13p6x23RV +=  + AbV(m16,L)*BbV(m16,L) * ( 1./24.*m66 - 1./12.*
         m16 + 1./24.*m11 );

      F13p6x23RV +=  + AbV(m16,L)*BbV(m33,L) * (  - 1./48.*RD3456ep );

      F13p6x23RV +=  + AbV(m16,L)*CbV(m11,L) * (  - 1./36.*m66*RD1456ep
          + 1./18.*m16*RD1456ep - 1./36.*m11*RD1456ep );

      F13p6x23RV +=  + AbV(m33,L)*pi16 * (  - 1./48.*RD3456ep + 1./72.*
         RD3456ep*RS34561pe - 1./144.*RD3456ep*RC3456pe + 1./72.*
         RD1456ep*RS34561pe - 1./144.*RD1456ep*RC3456pe - 1./36.*me2*
         RSe45613p*RS34561pe + 1./72.*me2*RC3456pe*RSe45613p + 1./72.*
         me2*RSe45633p*RS34561pe - 1./144.*me2*RSe45633p*RC3456pe + 1./
         72.*me2*RSe45611p*RS34561pe - 1./144.*me2*RSe45611p*RC3456pe
          - 1./36.*mp2*RSp45613e*RS34561pe + 1./72.*mp2*RC3456pe*
         RSp45613e + 1./72.*mp2*RSp45633e*RS34561pe - 1./144.*mp2*
         RSp45633e*RC3456pe + 1./72.*mp2*RSp45611e*RS34561pe - 1./144.*
         mp2*RSp45611e*RC3456pe - 1./24.*m36*RS34561pe + 1./48.*m36*
         RC3456pe - 1./24.*m35*RS34561pe + 1./48.*m35*RC3456pe - 1./24.
         *m34*RS34561pe + 1./48.*m34*RC3456pe + 1./16.*m33 + 1./24.*m33
         *RS34561pe - 1./36.*m33*pow(RS34561pe,2) - 1./48.*m33*RC3456pe
          + 1./36.*m33*RC3456pe*RS34561pe - 1./144.*m33*pow(RC3456pe,2)
          - 1./48.*m33*RS345pe - 1./48.*m33*RS346pe - 1./48.*m33*
         RS356pe );

      F13p6x23RV +=  + AbV(m33,L)*pi16 * (  - 1./24.*m16*RS34561pe + 1./
         48.*m16*RC3456pe - 1./24.*m15*RS34561pe + 1./48.*m15*RC3456pe
          - 1./24.*m14*RS34561pe + 1./48.*m14*RC3456pe - 1./36.*m13*
         pow(RS34561pe,2) - 1./36.*m13*RS14563pe*RS34561pe - 1./36.*m13
         *RSe45613p*RS34561pe - 1./36.*m13*RSp45613e*RS34561pe + 1./36.
         *m13*RC3456pe*RS34561pe + 1./72.*m13*RC3456pe*RS14563pe + 1./
         72.*m13*RC3456pe*RSe45613p + 1./72.*m13*RC3456pe*RSp45613e - 1.
         /144.*m13*pow(RC3456pe,2) + 1./72.*m13*RSe45633p*RS34561pe - 1.
         /144.*m13*RSe45633p*RC3456pe + 1./72.*m13*RSp45633e*RS34561pe
          - 1./144.*m13*RSp45633e*RC3456pe + 1./72.*m13*RC1456pe*
         RS34561pe - 1./144.*m13*RC1456pe*RC3456pe + 1./72.*m13*
         RSe45611p*RS34561pe - 1./144.*m13*RSe45611p*RC3456pe + 1./72.*
         m13*RSp45611e*RS34561pe - 1./144.*m13*RSp45611e*RC3456pe - 1./
         36.*m11*RS14563pe*RS34561pe + 1./72.*m11*RC3456pe*RS14563pe + 
         1./72.*m11*RC1456pe*RS34561pe - 1./144.*m11*RC1456pe*RC3456pe
          );

      F13p6x23RV +=  + pow(AbV(m33,L),2) * ( 1./16. + 1./72.*pow(
         RS34561pe,2) - 1./72.*RC3456pe*RS34561pe + 1./288.*pow(
         RC3456pe,2) );

      F13p6x23RV +=  + AbV(m33,L)*AbV(m34,L) * (  - 1./36.*RS34561pe - 
         1./18.*RS14563pe + 7./144.*RC3456pe - 5./72.*RS356pe + 1./18.*
         RS156pe );

      F13p6x23RV +=  + AbV(m33,L)*AbV(m35,L) * (  - 1./36.*RS34561pe - 
         1./18.*RS14563pe + 7./144.*RC3456pe - 5./72.*RS346pe + 1./18.*
         RS146pe );

      F13p6x23RV +=  + AbV(m33,L)*AbV(m36,L) * (  - 1./36.*RS34561pe - 
         1./18.*RS14563pe + 7./144.*RC3456pe - 5./72.*RS345pe + 1./18.*
         RS145pe );

      F13p6x23RV +=  + AbV(m33,L)*AbV(m45,L) * (  - 1./108.*pow(
         RS346pe,2) + 1./54.*RS356pe*RS346pe - 1./108.*pow(RS356pe,2)
          + 1./54.*RS146pe*RS346pe - 1./54.*RS146pe*RS356pe - 1./54.*
         RS156pe*RS346pe + 1./54.*RS156pe*RS356pe );

      F13p6x23RV +=  + AbV(m33,L)*AbV(m46,L) * (  - 1./108.*pow(
         RS345pe,2) + 1./54.*RS356pe*RS345pe - 1./108.*pow(RS356pe,2)
          + 1./54.*RS145pe*RS345pe - 1./54.*RS145pe*RS356pe - 1./54.*
         RS156pe*RS345pe + 1./54.*RS156pe*RS356pe );

      F13p6x23RV +=  + AbV(m33,L)*AbV(m56,L) * (  - 1./108.*pow(
         RS345pe,2) + 1./54.*RS346pe*RS345pe - 1./108.*pow(RS346pe,2)
          + 1./54.*RS145pe*RS345pe - 1./54.*RS145pe*RS346pe - 1./54.*
         RS146pe*RS345pe + 1./54.*RS146pe*RS346pe );

      F13p6x23RV +=  + AbV(m33,L)*AbV(mp2,L) * ( 1./36.*RSp45613e*
         RS34561pe - 1./72.*RC3456pe*RSp45613e - 1./72.*RSp45633e*
         RS34561pe + 1./144.*RSp45633e*RC3456pe - 1./72.*RSp45611e*
         RS34561pe + 1./144.*RSp45611e*RC3456pe );

      F13p6x23RV +=  + AbV(m33,L)*AbV(me2,L) * ( 1./36.*RSe45613p*
         RS34561pe - 1./72.*RC3456pe*RSe45613p - 1./72.*RSe45633p*
         RS34561pe + 1./144.*RSe45633p*RC3456pe - 1./72.*RSe45611p*
         RS34561pe + 1./144.*RSe45611p*RC3456pe );

      F13p6x23RV +=  + AbV(m33,L)*BbV(m11,m33,L) * (  - 1./18.*m33*
         RC3456pe*RS14563pe );

      F13p6x23RV +=  + AbV(m33,L)*BbV(m11,L) * (  - 1./72.*RD1456ep*
         RS34561pe + 1./144.*RD1456ep*RC3456pe );

      F13p6x23RV +=  + AbV(m33,L)*BbV(m33,mp2,L) * (  - 1./18.*m33*
         RC3456pe*RSp45613e + 1./18.*m33*RSp45633e*RC3456pe );

      F13p6x23RV +=  + AbV(m33,L)*BbV(m33,me2,L) * (  - 1./18.*m33*
         RC3456pe*RSe45613p + 1./18.*m33*RSe45633p*RC3456pe );

      F13p6x23RV +=  + AbV(m33,L)*BbV(m33,L) * (  - 1./72.*RD3456ep*
         RS34561pe + 1./144.*RD3456ep*RC3456pe + 1./4.*m33 - 1./18.*m33
         *RC3456pe*RS34561pe + 1./18.*m33*pow(RC3456pe,2) );

      F13p6x23RV +=  + AbV(m33,L)*BbV(m34,L) * ( 1./72.*RD3456ep - 1./
         144.*m44*RC3456pe - 1./72.*m44*RS356pe + 1./72.*m34*RC3456pe
          - 1./36.*m34*RS356pe - 1./144.*m33*RC3456pe - 1./24.*m33*
         RS356pe );

      F13p6x23RV +=  + AbV(m33,L)*BbV(m35,L) * ( 1./72.*RD3456ep - 1./
         144.*m55*RC3456pe - 1./72.*m55*RS346pe + 1./72.*m35*RC3456pe
          - 1./36.*m35*RS346pe - 1./144.*m33*RC3456pe - 1./24.*m33*
         RS346pe );

      F13p6x23RV +=  + AbV(m33,L)*BbV(m36,L) * ( 1./72.*RD3456ep - 1./
         144.*m66*RC3456pe - 1./72.*m66*RS345pe + 1./72.*m36*RC3456pe
          - 1./36.*m36*RS345pe - 1./144.*m33*RC3456pe - 1./24.*m33*
         RS345pe );

      F13p6x23RV +=  + AbV(m33,L)*CbV(m33,L) * ( 1./18.*m33*RD3456ep*
         RC3456pe );

      F13p6x23RV +=  + AbV(m34,L)*pi16 * ( 1./48.*RD1456ep + 1./48.*me2
         *RSe45611p - 1./24.*me2*RSe561p + 1./48.*me2*RSe564p + 1./48.*
         mp2*RSp45611e - 1./24.*mp2*RSp561e + 1./48.*mp2*RSp564e - 1./
         16.*m46 - 1./16.*m45 - 1./16.*m44 + 1./48.*m44*RS456pe + 1./16.
         *m34 - 1./48.*m34*RS34561pe - 1./48.*m34*RS14563pe - 1./48.*
         m34*RSe45613p - 1./48.*m34*RSp45613e + 1./48.*m34*RC3456pe + 1.
         /48.*m34*RSe45633p + 1./48.*m34*RSp45633e - 1./48.*m34*RS356pe
          - 1./48.*m34*RSe563p - 1./48.*m34*RSp563e + 1./48.*m34*
         RS156pe + 1./48.*m34*RSe561p + 1./48.*m34*RSp561e - 1./16.*m16
          - 1./16.*m15 - 1./16.*m14 + 1./48.*m14*RC1456pe + 1./48.*m14*
         RSe45611p + 1./48.*m14*RSp45611e - 1./24.*m14*RS156pe + 1./48.
         *m14*RS456pe - 1./24.*m14*RSe561p - 1./24.*m14*RSp561e + 1./48.
         *m14*RSe564p + 1./48.*m14*RSp564e + 1./48.*m11*RC1456pe - 1./
         24.*m11*RS156pe );

      F13p6x23RV +=  + pow(AbV(m34,L),2) * (  - 1./32. );

      F13p6x23RV +=  + AbV(m34,L)*AbV(m35,L) * (  - 1./16. );

      F13p6x23RV +=  + AbV(m34,L)*AbV(m36,L) * (  - 1./16. );

      F13p6x23RV +=  + AbV(m34,L)*AbV(mp2,L) * (  - 1./36.*RSp45613e + 
         7./144.*RSp45633e - 5./72.*RSp563e - 1./48.*RSp45611e + 5./72.
         *RSp561e );

      F13p6x23RV +=  + AbV(m34,L)*AbV(me2,L) * (  - 1./36.*RSe45613p + 
         7./144.*RSe45633p - 5./72.*RSe563p - 1./48.*RSe45611p + 5./72.
         *RSe561p );

      F13p6x23RV +=  + AbV(m34,L)*BbV(m11,m33,L) * ( 1./36.*m44*
         RS14563pe + 1./36.*m44*RS156pe - 1./18.*m34*RS14563pe + 1./18.
         *m34*RS156pe + 1./12.*m33*RS14563pe + 1./36.*m33*RS156pe - 1./
         18.*m11*RS14563pe + 1./18.*m11*RS156pe );

      F13p6x23RV +=  + AbV(m34,L)*BbV(m11,L) * (  - 1./48.*RD1456ep );

      F13p6x23RV +=  + AbV(m34,L)*BbV(m33,mp2,L) * ( 1./36.*m44*
         RSp45613e - 1./36.*m44*RSp45633e - 1./36.*m44*RSp563e + 1./36.
         *m44*RSp561e - 1./18.*m34*RSp45613e + 1./18.*m34*RSp45633e - 1.
         /18.*m34*RSp563e + 1./18.*m34*RSp561e + 1./36.*m33*RSp45613e
          - 1./36.*m33*RSp45633e - 1./12.*m33*RSp563e + 1./12.*m33*
         RSp561e );

      F13p6x23RV +=  + AbV(m34,L)*BbV(m33,me2,L) * ( 1./36.*m44*
         RSe45613p - 1./36.*m44*RSe45633p - 1./36.*m44*RSe563p + 1./36.
         *m44*RSe561p - 1./18.*m34*RSe45613p + 1./18.*m34*RSe45633p - 1.
         /18.*m34*RSe563p + 1./18.*m34*RSe561p + 1./36.*m33*RSe45613p
          - 1./36.*m33*RSe45633p - 1./12.*m33*RSe563p + 1./12.*m33*
         RSe561p );

      F13p6x23RV +=  + AbV(m34,L)*BbV(m33,L) * ( 7./144.*RD3456ep + 1./
         36.*m44*RS34561pe - 1./36.*m44*RC3456pe - 1./36.*m44*RS356pe
          - 1./18.*m34*RS34561pe + 1./18.*m34*RC3456pe - 1./18.*m34*
         RS356pe + 1./36.*m33*RS34561pe - 1./36.*m33*RC3456pe - 1./12.*
         m33*RS356pe );

      F13p6x23RV +=  + AbV(m34,L)*BbV(m34,L) * ( 1./24.*m44 - 1./12.*
         m34 + 1./24.*m33 );

      F13p6x23RV +=  + AbV(m34,L)*BbV(m35,L) * ( 1./48.*m55 + 1./48.*
         m44 - 1./24.*m35 - 1./24.*m34 + 1./24.*m33 );

      F13p6x23RV +=  + AbV(m34,L)*BbV(m36,L) * ( 1./48.*m66 + 1./48.*
         m44 - 1./24.*m36 - 1./24.*m34 + 1./24.*m33 );

      F13p6x23RV +=  + AbV(m34,L)*CbV(m33,L) * (  - 1./36.*m44*RD3456ep
          + 1./18.*m34*RD3456ep - 1./36.*m33*RD3456ep );

      F13p6x23RV +=  + AbV(m35,L)*pi16 * ( 1./48.*RD1456ep + 1./48.*me2
         *RSe45611p - 1./24.*me2*RSe461p + 1./48.*me2*RSe465p + 1./48.*
         mp2*RSp45611e - 1./24.*mp2*RSp461e + 1./48.*mp2*RSp465e - 1./
         16.*m56 - 1./16.*m55 + 1./48.*m55*RS546pe - 1./16.*m45 + 1./16.
         *m35 - 1./48.*m35*RS34561pe - 1./48.*m35*RS14563pe - 1./48.*
         m35*RSe45613p - 1./48.*m35*RSp45613e + 1./48.*m35*RC3456pe + 1.
         /48.*m35*RSe45633p + 1./48.*m35*RSp45633e - 1./48.*m35*RS346pe
          - 1./48.*m35*RSe463p - 1./48.*m35*RSp463e + 1./48.*m35*
         RS146pe + 1./48.*m35*RSe461p + 1./48.*m35*RSp461e - 1./16.*m16
          - 1./16.*m15 + 1./48.*m15*RC1456pe + 1./48.*m15*RSe45611p + 1.
         /48.*m15*RSp45611e - 1./24.*m15*RS146pe + 1./48.*m15*RS546pe
          - 1./24.*m15*RSe461p - 1./24.*m15*RSp461e + 1./48.*m15*
         RSe465p + 1./48.*m15*RSp465e - 1./16.*m14 + 1./48.*m11*
         RC1456pe - 1./24.*m11*RS146pe );

      F13p6x23RV +=  + pow(AbV(m35,L),2) * (  - 1./32. );

      F13p6x23RV +=  + AbV(m35,L)*AbV(m36,L) * (  - 1./16. );

      F13p6x23RV +=  + AbV(m35,L)*AbV(mp2,L) * (  - 1./36.*RSp45613e + 
         7./144.*RSp45633e - 5./72.*RSp463e - 1./48.*RSp45611e + 5./72.
         *RSp461e );

      F13p6x23RV +=  + AbV(m35,L)*AbV(me2,L) * (  - 1./36.*RSe45613p + 
         7./144.*RSe45633p - 5./72.*RSe463p - 1./48.*RSe45611p + 5./72.
         *RSe461p );

      F13p6x23RV +=  + AbV(m35,L)*BbV(m11,m33,L) * ( 1./36.*m55*
         RS14563pe + 1./36.*m55*RS146pe - 1./18.*m35*RS14563pe + 1./18.
         *m35*RS146pe + 1./12.*m33*RS14563pe + 1./36.*m33*RS146pe - 1./
         18.*m11*RS14563pe + 1./18.*m11*RS146pe );

      F13p6x23RV +=  + AbV(m35,L)*BbV(m11,L) * (  - 1./48.*RD1456ep );

      F13p6x23RV +=  + AbV(m35,L)*BbV(m33,mp2,L) * ( 1./36.*m55*
         RSp45613e - 1./36.*m55*RSp45633e - 1./36.*m55*RSp463e + 1./36.
         *m55*RSp461e - 1./18.*m35*RSp45613e + 1./18.*m35*RSp45633e - 1.
         /18.*m35*RSp463e + 1./18.*m35*RSp461e + 1./36.*m33*RSp45613e
          - 1./36.*m33*RSp45633e - 1./12.*m33*RSp463e + 1./12.*m33*
         RSp461e );

      F13p6x23RV +=  + AbV(m35,L)*BbV(m33,me2,L) * ( 1./36.*m55*
         RSe45613p - 1./36.*m55*RSe45633p - 1./36.*m55*RSe463p + 1./36.
         *m55*RSe461p - 1./18.*m35*RSe45613p + 1./18.*m35*RSe45633p - 1.
         /18.*m35*RSe463p + 1./18.*m35*RSe461p + 1./36.*m33*RSe45613p
          - 1./36.*m33*RSe45633p - 1./12.*m33*RSe463p + 1./12.*m33*
         RSe461p );

      F13p6x23RV +=  + AbV(m35,L)*BbV(m33,L) * ( 7./144.*RD3456ep + 1./
         36.*m55*RS34561pe - 1./36.*m55*RC3456pe - 1./36.*m55*RS346pe
          - 1./18.*m35*RS34561pe + 1./18.*m35*RC3456pe - 1./18.*m35*
         RS346pe + 1./36.*m33*RS34561pe - 1./36.*m33*RC3456pe - 1./12.*
         m33*RS346pe );

      F13p6x23RV +=  + AbV(m35,L)*BbV(m34,L) * ( 1./48.*m55 + 1./48.*
         m44 - 1./24.*m35 - 1./24.*m34 + 1./24.*m33 );

      F13p6x23RV +=  + AbV(m35,L)*BbV(m35,L) * ( 1./24.*m55 - 1./12.*
         m35 + 1./24.*m33 );

      F13p6x23RV +=  + AbV(m35,L)*BbV(m36,L) * ( 1./48.*m66 + 1./48.*
         m55 - 1./24.*m36 - 1./24.*m35 + 1./24.*m33 );

      F13p6x23RV +=  + AbV(m35,L)*CbV(m33,L) * (  - 1./36.*m55*RD3456ep
          + 1./18.*m35*RD3456ep - 1./36.*m33*RD3456ep );

      F13p6x23RV +=  + AbV(m36,L)*pi16 * ( 1./48.*RD1456ep + 1./48.*me2
         *RSe45611p - 1./24.*me2*RSe451p + 1./48.*me2*RSe456p + 1./48.*
         mp2*RSp45611e - 1./24.*mp2*RSp451e + 1./48.*mp2*RSp456e - 1./
         16.*m66 + 1./48.*m66*RS645pe - 1./16.*m56 - 1./16.*m46 + 1./16.
         *m36 - 1./48.*m36*RS34561pe - 1./48.*m36*RS14563pe - 1./48.*
         m36*RSe45613p - 1./48.*m36*RSp45613e + 1./48.*m36*RC3456pe + 1.
         /48.*m36*RSe45633p + 1./48.*m36*RSp45633e - 1./48.*m36*RS345pe
          - 1./48.*m36*RSe453p - 1./48.*m36*RSp453e + 1./48.*m36*
         RS145pe + 1./48.*m36*RSe451p + 1./48.*m36*RSp451e - 1./16.*m16
          + 1./48.*m16*RC1456pe + 1./48.*m16*RSe45611p + 1./48.*m16*
         RSp45611e - 1./24.*m16*RS145pe + 1./48.*m16*RS645pe - 1./24.*
         m16*RSe451p - 1./24.*m16*RSp451e + 1./48.*m16*RSe456p + 1./48.
         *m16*RSp456e - 1./16.*m15 - 1./16.*m14 + 1./48.*m11*RC1456pe
          - 1./24.*m11*RS145pe );

      F13p6x23RV +=  + pow(AbV(m36,L),2) * (  - 1./32. );

      F13p6x23RV +=  + AbV(m36,L)*AbV(mp2,L) * (  - 1./36.*RSp45613e + 
         7./144.*RSp45633e - 5./72.*RSp453e - 1./48.*RSp45611e + 5./72.
         *RSp451e );

      F13p6x23RV +=  + AbV(m36,L)*AbV(me2,L) * (  - 1./36.*RSe45613p + 
         7./144.*RSe45633p - 5./72.*RSe453p - 1./48.*RSe45611p + 5./72.
         *RSe451p );

      F13p6x23RV +=  + AbV(m36,L)*BbV(m11,m33,L) * ( 1./36.*m66*
         RS14563pe + 1./36.*m66*RS145pe - 1./18.*m36*RS14563pe + 1./18.
         *m36*RS145pe + 1./12.*m33*RS14563pe + 1./36.*m33*RS145pe - 1./
         18.*m11*RS14563pe + 1./18.*m11*RS145pe );

      F13p6x23RV +=  + AbV(m36,L)*BbV(m11,L) * (  - 1./48.*RD1456ep );

      F13p6x23RV +=  + AbV(m36,L)*BbV(m33,mp2,L) * ( 1./36.*m66*
         RSp45613e - 1./36.*m66*RSp45633e - 1./36.*m66*RSp453e + 1./36.
         *m66*RSp451e - 1./18.*m36*RSp45613e + 1./18.*m36*RSp45633e - 1.
         /18.*m36*RSp453e + 1./18.*m36*RSp451e + 1./36.*m33*RSp45613e
          - 1./36.*m33*RSp45633e - 1./12.*m33*RSp453e + 1./12.*m33*
         RSp451e );

      F13p6x23RV +=  + AbV(m36,L)*BbV(m33,me2,L) * ( 1./36.*m66*
         RSe45613p - 1./36.*m66*RSe45633p - 1./36.*m66*RSe453p + 1./36.
         *m66*RSe451p - 1./18.*m36*RSe45613p + 1./18.*m36*RSe45633p - 1.
         /18.*m36*RSe453p + 1./18.*m36*RSe451p + 1./36.*m33*RSe45613p
          - 1./36.*m33*RSe45633p - 1./12.*m33*RSe453p + 1./12.*m33*
         RSe451p );

      F13p6x23RV +=  + AbV(m36,L)*BbV(m33,L) * ( 7./144.*RD3456ep + 1./
         36.*m66*RS34561pe - 1./36.*m66*RC3456pe - 1./36.*m66*RS345pe
          - 1./18.*m36*RS34561pe + 1./18.*m36*RC3456pe - 1./18.*m36*
         RS345pe + 1./36.*m33*RS34561pe - 1./36.*m33*RC3456pe - 1./12.*
         m33*RS345pe );

      F13p6x23RV +=  + AbV(m36,L)*BbV(m34,L) * ( 1./48.*m66 + 1./48.*
         m44 - 1./24.*m36 - 1./24.*m34 + 1./24.*m33 );

      F13p6x23RV +=  + AbV(m36,L)*BbV(m35,L) * ( 1./48.*m66 + 1./48.*
         m55 - 1./24.*m36 - 1./24.*m35 + 1./24.*m33 );

      F13p6x23RV +=  + AbV(m36,L)*BbV(m36,L) * ( 1./24.*m66 - 1./12.*
         m36 + 1./24.*m33 );

      F13p6x23RV +=  + AbV(m36,L)*CbV(m33,L) * (  - 1./36.*m66*RD3456ep
          + 1./18.*m36*RD3456ep - 1./36.*m33*RD3456ep );

      F13p6x23RV +=  + AbV(m44,L)*BbV(m11,m33,L) * (  - 1./18.*m44*
         RS156pe*RS356pe + 1./54.*m44*RS456pe*RS156pe*RS356pe );

      F13p6x23RV +=  + AbV(m44,L)*BbV(m11,mp2,L) * (  - 1./18.*m44*
         RS156pe*RSp563e + 1./54.*m44*RS456pe*RS156pe*RSp563e + 1./18.*
         m44*RSp561e*RS156pe - 1./54.*m44*RSp561e*RS456pe*RS156pe );

      F13p6x23RV +=  + AbV(m44,L)*BbV(m11,me2,L) * (  - 1./18.*m44*
         RS156pe*RSe563p + 1./54.*m44*RS456pe*RS156pe*RSe563p + 1./18.*
         m44*RSe561p*RS156pe - 1./54.*m44*RSe561p*RS456pe*RS156pe );

      F13p6x23RV +=  + AbV(m44,L)*BbV(m11,L) * ( 1./36.*m44*pow(
         RS156pe,2) - 1./108.*m44*RS456pe*pow(RS156pe,2) );

      F13p6x23RV +=  + AbV(m44,L)*BbV(m14,L) * ( 1./48.*m44 - 1./144.*
         m44*RS456pe - 1./24.*m14 + 1./72.*m14*RS456pe + 1./48.*m11 - 1.
         /144.*m11*RS456pe );

      F13p6x23RV +=  + AbV(m44,L)*BbV(m33,mp2,L) * ( 1./18.*m44*RSp563e
         *RS356pe - 1./54.*m44*RS456pe*RSp563e*RS356pe - 1./18.*m44*
         RSp561e*RS356pe + 1./54.*m44*RSp561e*RS456pe*RS356pe );

      F13p6x23RV +=  + AbV(m44,L)*BbV(m33,me2,L) * ( 1./18.*m44*RSe563p
         *RS356pe - 1./54.*m44*RS456pe*RSe563p*RS356pe - 1./18.*m44*
         RSe561p*RS356pe + 1./54.*m44*RSe561p*RS456pe*RS356pe );

      F13p6x23RV +=  + AbV(m44,L)*BbV(m33,L) * ( 1./36.*m44*pow(
         RS356pe,2) - 1./108.*m44*RS456pe*pow(RS356pe,2) );

      F13p6x23RV +=  + AbV(m44,L)*BbV(m34,L) * ( 1./48.*m44 - 1./144.*
         m44*RS456pe - 1./24.*m34 + 1./72.*m34*RS456pe + 1./48.*m33 - 1.
         /144.*m33*RS456pe );

      F13p6x23RV +=  + AbV(m44,L)*BbV(mp2,me2,L) * ( 2./27.*m44*RSp563e
         *RSe563p - 2./81.*m44*RS456pe*RSp563e*RSe563p - 2./27.*m44*
         RSe561p*RSp563e + 2./81.*m44*RSe561p*RS456pe*RSp563e - 2./27.*
         m44*RSp561e*RSe563p + 2./81.*m44*RSp561e*RS456pe*RSe563p + 2./
         27.*m44*RSp561e*RSe561p - 2./81.*m44*RSp561e*RSe561p*RS456pe )
         ;

      F13p6x23RV +=  + AbV(m44,L)*BbV(mp2,L) * ( 1./36.*m44*pow(
         RSp563e,2) - 1./108.*m44*RS456pe*pow(RSp563e,2) - 1./18.*m44*
         RSp561e*RSp563e + 1./54.*m44*RSp561e*RS456pe*RSp563e + 1./36.*
         m44*pow(RSp561e,2) - 1./108.*m44*pow(RSp561e,2)*RS456pe );

      F13p6x23RV +=  + AbV(m44,L)*BbV(me2,mp2,L) * (  - 1./54.*m44*
         RSp563e*RSe563p + 1./162.*m44*RS456pe*RSp563e*RSe563p + 1./54.
         *m44*RSe561p*RSp563e - 1./162.*m44*RSe561p*RS456pe*RSp563e + 1.
         /54.*m44*RSp561e*RSe563p - 1./162.*m44*RSp561e*RS456pe*RSe563p
          - 1./54.*m44*RSp561e*RSe561p + 1./162.*m44*RSp561e*RSe561p*
         RS456pe );

      F13p6x23RV +=  + AbV(m44,L)*BbV(me2,L) * ( 1./36.*m44*pow(
         RSe563p,2) - 1./108.*m44*RS456pe*pow(RSe563p,2) - 1./18.*m44*
         RSe561p*RSe563p + 1./54.*m44*RSe561p*RS456pe*RSe563p + 1./36.*
         m44*pow(RSe561p,2) - 1./108.*m44*pow(RSe561p,2)*RS456pe );

      F13p6x23RV +=  + AbV(m45,L)*AbV(mp2,L) * ( 1./54.*RSp563e*RS346pe
          - 1./54.*RSp563e*RS356pe + 1./162.*RSp563e*RSe563p - 1./162.*
         RSp563e*RSe463p - 1./108.*pow(RSp563e,2) - 1./54.*RSp463e*
         RS346pe + 1./54.*RSp463e*RS356pe - 1./162.*RSp463e*RSe563p + 1.
         /162.*RSp463e*RSe463p + 1./54.*RSp463e*RSp563e - 1./108.*pow(
         RSp463e,2) - 1./54.*RS146pe*RSp563e + 1./54.*RS146pe*RSp463e
          + 1./54.*RS156pe*RSp563e - 1./54.*RS156pe*RSp463e - 1./162.*
         RSe561p*RSp563e + 1./162.*RSe561p*RSp463e + 1./162.*RSe461p*
         RSp563e - 1./162.*RSe461p*RSp463e - 1./54.*RSp561e*RS346pe + 1.
         /54.*RSp561e*RS356pe - 1./162.*RSp561e*RSe563p + 1./162.*
         RSp561e*RSe463p + 1./54.*RSp561e*RSp563e - 1./54.*RSp561e*
         RSp463e + 1./54.*RSp561e*RS146pe - 1./54.*RSp561e*RS156pe + 1./
         162.*RSp561e*RSe561p - 1./162.*RSp561e*RSe461p - 1./108.*pow(
         RSp561e,2) + 1./54.*RSp461e*RS346pe - 1./54.*RSp461e*RS356pe
          + 1./162.*RSp461e*RSe563p - 1./162.*RSp461e*RSe463p - 1./54.*
         RSp461e*RSp563e );

      F13p6x23RV +=  + AbV(m45,L)*AbV(mp2,L) * ( 1./54.*RSp461e*RSp463e
          - 1./54.*RSp461e*RS146pe + 1./54.*RSp461e*RS156pe - 1./162.*
         RSp461e*RSe561p + 1./162.*RSp461e*RSe461p + 1./54.*RSp461e*
         RSp561e - 1./108.*pow(RSp461e,2) );

      F13p6x23RV +=  + AbV(m45,L)*AbV(me2,L) * ( 1./54.*RSe563p*RS346pe
          - 1./54.*RSe563p*RS356pe - 1./108.*pow(RSe563p,2) - 1./54.*
         RSe463p*RS346pe + 1./54.*RSe463p*RS356pe + 1./54.*RSe463p*
         RSe563p - 1./108.*pow(RSe463p,2) - 2./81.*RSp563e*RSe563p + 2./
         81.*RSp563e*RSe463p + 2./81.*RSp463e*RSe563p - 2./81.*RSp463e*
         RSe463p - 1./54.*RS146pe*RSe563p + 1./54.*RS146pe*RSe463p + 1./
         54.*RS156pe*RSe563p - 1./54.*RS156pe*RSe463p - 1./54.*RSe561p*
         RS346pe + 1./54.*RSe561p*RS356pe + 1./54.*RSe561p*RSe563p - 1./
         54.*RSe561p*RSe463p + 2./81.*RSe561p*RSp563e - 2./81.*RSe561p*
         RSp463e + 1./54.*RSe561p*RS146pe - 1./54.*RSe561p*RS156pe - 1./
         108.*pow(RSe561p,2) + 1./54.*RSe461p*RS346pe - 1./54.*RSe461p*
         RS356pe - 1./54.*RSe461p*RSe563p + 1./54.*RSe461p*RSe463p - 2./
         81.*RSe461p*RSp563e + 2./81.*RSe461p*RSp463e - 1./54.*RSe461p*
         RS146pe + 1./54.*RSe461p*RS156pe + 1./54.*RSe461p*RSe561p - 1./
         108.*pow(RSe461p,2) + 2./81.*RSp561e*RSe563p - 2./81.*RSp561e*
         RSe463p );

      F13p6x23RV +=  + AbV(m45,L)*AbV(me2,L) * (  - 2./81.*RSp561e*
         RSe561p + 2./81.*RSp561e*RSe461p - 2./81.*RSp461e*RSe563p + 2./
         81.*RSp461e*RSe463p + 2./81.*RSp461e*RSe561p - 2./81.*RSp461e*
         RSe461p );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m11,m33,L) * (  - 1./36.*m55*
         RS146pe*RS346pe - 1./108.*m55*RS146pe*RS356pe - 1./108.*m55*
         RS156pe*RS346pe - 1./108.*m55*RS156pe*RS356pe + 1./54.*m45*
         RS146pe*RS346pe - 1./54.*m45*RS146pe*RS356pe - 1./54.*m45*
         RS156pe*RS346pe + 1./54.*m45*RS156pe*RS356pe - 1./108.*m44*
         RS146pe*RS346pe - 1./108.*m44*RS146pe*RS356pe - 1./108.*m44*
         RS156pe*RS346pe - 1./36.*m44*RS156pe*RS356pe + 1./54.*m11*
         RS146pe*RS346pe - 1./54.*m11*RS146pe*RS356pe - 1./54.*m11*
         RS156pe*RS346pe + 1./54.*m11*RS156pe*RS356pe );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m11,mp2,L) * (  - 1./108.*m55*
         RS146pe*RSp563e - 1./36.*m55*RS146pe*RSp463e - 1./108.*m55*
         RS156pe*RSp563e - 1./108.*m55*RS156pe*RSp463e + 1./108.*m55*
         RSp561e*RS146pe + 1./108.*m55*RSp561e*RS156pe + 1./36.*m55*
         RSp461e*RS146pe + 1./108.*m55*RSp461e*RS156pe - 1./54.*m45*
         RS146pe*RSp563e + 1./54.*m45*RS146pe*RSp463e + 1./54.*m45*
         RS156pe*RSp563e - 1./54.*m45*RS156pe*RSp463e + 1./54.*m45*
         RSp561e*RS146pe - 1./54.*m45*RSp561e*RS156pe - 1./54.*m45*
         RSp461e*RS146pe + 1./54.*m45*RSp461e*RS156pe - 1./108.*m44*
         RS146pe*RSp563e - 1./108.*m44*RS146pe*RSp463e - 1./36.*m44*
         RS156pe*RSp563e - 1./108.*m44*RS156pe*RSp463e + 1./108.*m44*
         RSp561e*RS146pe + 1./36.*m44*RSp561e*RS156pe + 1./108.*m44*
         RSp461e*RS146pe + 1./108.*m44*RSp461e*RS156pe - 1./54.*m11*
         RS146pe*RSp563e + 1./54.*m11*RS146pe*RSp463e + 1./54.*m11*
         RS156pe*RSp563e - 1./54.*m11*RS156pe*RSp463e + 1./54.*m11*
         RSp561e*RS146pe );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m11,mp2,L) * (  - 1./54.*m11*
         RSp561e*RS156pe - 1./54.*m11*RSp461e*RS146pe + 1./54.*m11*
         RSp461e*RS156pe );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m11,me2,L) * (  - 1./108.*m55*
         RS146pe*RSe563p - 1./36.*m55*RS146pe*RSe463p - 1./108.*m55*
         RS156pe*RSe563p - 1./108.*m55*RS156pe*RSe463p + 1./108.*m55*
         RSe561p*RS146pe + 1./108.*m55*RSe561p*RS156pe + 1./36.*m55*
         RSe461p*RS146pe + 1./108.*m55*RSe461p*RS156pe - 1./54.*m45*
         RS146pe*RSe563p + 1./54.*m45*RS146pe*RSe463p + 1./54.*m45*
         RS156pe*RSe563p - 1./54.*m45*RS156pe*RSe463p + 1./54.*m45*
         RSe561p*RS146pe - 1./54.*m45*RSe561p*RS156pe - 1./54.*m45*
         RSe461p*RS146pe + 1./54.*m45*RSe461p*RS156pe - 1./108.*m44*
         RS146pe*RSe563p - 1./108.*m44*RS146pe*RSe463p - 1./36.*m44*
         RS156pe*RSe563p - 1./108.*m44*RS156pe*RSe463p + 1./108.*m44*
         RSe561p*RS146pe + 1./36.*m44*RSe561p*RS156pe + 1./108.*m44*
         RSe461p*RS146pe + 1./108.*m44*RSe461p*RS156pe - 1./54.*m11*
         RS146pe*RSe563p + 1./54.*m11*RS146pe*RSe463p + 1./54.*m11*
         RS156pe*RSe563p - 1./54.*m11*RS156pe*RSe463p + 1./54.*m11*
         RSe561p*RS146pe );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m11,me2,L) * (  - 1./54.*m11*
         RSe561p*RS156pe - 1./54.*m11*RSe461p*RS146pe + 1./54.*m11*
         RSe461p*RS156pe );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m11,L) * ( 1./72.*m55*pow(
         RS146pe,2) + 1./108.*m55*RS156pe*RS146pe + 1./216.*m55*pow(
         RS156pe,2) - 1./108.*m45*pow(RS146pe,2) + 1./54.*m45*RS156pe*
         RS146pe - 1./108.*m45*pow(RS156pe,2) + 1./216.*m44*pow(
         RS146pe,2) + 1./108.*m44*RS156pe*RS146pe + 1./72.*m44*pow(
         RS156pe,2) - 1./108.*m11*pow(RS146pe,2) + 1./54.*m11*RS156pe*
         RS146pe - 1./108.*m11*pow(RS156pe,2) );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m14,L) * ( 1./48.*m55 - 1./24.*
         m45 + 1./24.*m44 - 1./24.*m14 + 1./48.*m11 );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m15,L) * ( 1./24.*m55 - 1./24.*
         m45 + 1./48.*m44 - 1./24.*m15 + 1./48.*m11 );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m33,mp2,L) * ( 1./108.*m55*
         RSp563e*RS346pe + 1./108.*m55*RSp563e*RS356pe + 1./36.*m55*
         RSp463e*RS346pe + 1./108.*m55*RSp463e*RS356pe - 1./108.*m55*
         RSp561e*RS346pe - 1./108.*m55*RSp561e*RS356pe - 1./36.*m55*
         RSp461e*RS346pe - 1./108.*m55*RSp461e*RS356pe + 1./54.*m45*
         RSp563e*RS346pe - 1./54.*m45*RSp563e*RS356pe - 1./54.*m45*
         RSp463e*RS346pe + 1./54.*m45*RSp463e*RS356pe - 1./54.*m45*
         RSp561e*RS346pe + 1./54.*m45*RSp561e*RS356pe + 1./54.*m45*
         RSp461e*RS346pe - 1./54.*m45*RSp461e*RS356pe + 1./108.*m44*
         RSp563e*RS346pe + 1./36.*m44*RSp563e*RS356pe + 1./108.*m44*
         RSp463e*RS346pe + 1./108.*m44*RSp463e*RS356pe - 1./108.*m44*
         RSp561e*RS346pe - 1./36.*m44*RSp561e*RS356pe - 1./108.*m44*
         RSp461e*RS346pe - 1./108.*m44*RSp461e*RS356pe + 1./54.*m33*
         RSp563e*RS346pe - 1./54.*m33*RSp563e*RS356pe - 1./54.*m33*
         RSp463e*RS346pe + 1./54.*m33*RSp463e*RS356pe - 1./54.*m33*
         RSp561e*RS346pe );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m33,mp2,L) * ( 1./54.*m33*RSp561e
         *RS356pe + 1./54.*m33*RSp461e*RS346pe - 1./54.*m33*RSp461e*
         RS356pe );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m33,me2,L) * ( 1./108.*m55*
         RSe563p*RS346pe + 1./108.*m55*RSe563p*RS356pe + 1./36.*m55*
         RSe463p*RS346pe + 1./108.*m55*RSe463p*RS356pe - 1./108.*m55*
         RSe561p*RS346pe - 1./108.*m55*RSe561p*RS356pe - 1./36.*m55*
         RSe461p*RS346pe - 1./108.*m55*RSe461p*RS356pe + 1./54.*m45*
         RSe563p*RS346pe - 1./54.*m45*RSe563p*RS356pe - 1./54.*m45*
         RSe463p*RS346pe + 1./54.*m45*RSe463p*RS356pe - 1./54.*m45*
         RSe561p*RS346pe + 1./54.*m45*RSe561p*RS356pe + 1./54.*m45*
         RSe461p*RS346pe - 1./54.*m45*RSe461p*RS356pe + 1./108.*m44*
         RSe563p*RS346pe + 1./36.*m44*RSe563p*RS356pe + 1./108.*m44*
         RSe463p*RS346pe + 1./108.*m44*RSe463p*RS356pe - 1./108.*m44*
         RSe561p*RS346pe - 1./36.*m44*RSe561p*RS356pe - 1./108.*m44*
         RSe461p*RS346pe - 1./108.*m44*RSe461p*RS356pe + 1./54.*m33*
         RSe563p*RS346pe - 1./54.*m33*RSe563p*RS356pe - 1./54.*m33*
         RSe463p*RS346pe + 1./54.*m33*RSe463p*RS356pe - 1./54.*m33*
         RSe561p*RS346pe );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m33,me2,L) * ( 1./54.*m33*RSe561p
         *RS356pe + 1./54.*m33*RSe461p*RS346pe - 1./54.*m33*RSe461p*
         RS356pe );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m33,L) * ( 1./72.*m55*pow(
         RS346pe,2) + 1./108.*m55*RS356pe*RS346pe + 1./216.*m55*pow(
         RS356pe,2) - 1./108.*m45*pow(RS346pe,2) + 1./54.*m45*RS356pe*
         RS346pe - 1./108.*m45*pow(RS356pe,2) + 1./216.*m44*pow(
         RS346pe,2) + 1./108.*m44*RS356pe*RS346pe + 1./72.*m44*pow(
         RS356pe,2) - 1./108.*m33*pow(RS346pe,2) + 1./54.*m33*RS356pe*
         RS346pe - 1./108.*m33*pow(RS356pe,2) );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m34,L) * ( 1./48.*m55 - 1./24.*
         m45 + 1./24.*m44 - 1./24.*m34 + 1./48.*m33 );

      F13p6x23RV +=  + AbV(m45,L)*BbV(m35,L) * ( 1./24.*m55 - 1./24.*
         m45 + 1./48.*m44 - 1./24.*m35 + 1./48.*m33 );

      F13p6x23RV +=  + AbV(m45,L)*BbV(mp2,me2,L) * (  - 2./81.*mp2*
         RSp563e*RSe563p + 2./81.*mp2*RSp563e*RSe463p + 2./81.*mp2*
         RSp463e*RSe563p - 2./81.*mp2*RSp463e*RSe463p + 2./81.*mp2*
         RSe561p*RSp563e - 2./81.*mp2*RSe561p*RSp463e - 2./81.*mp2*
         RSe461p*RSp563e + 2./81.*mp2*RSe461p*RSp463e + 2./81.*mp2*
         RSp561e*RSe563p - 2./81.*mp2*RSp561e*RSe463p - 2./81.*mp2*
         RSp561e*RSe561p + 2./81.*mp2*RSp561e*RSe461p - 2./81.*mp2*
         RSp461e*RSe563p + 2./81.*mp2*RSp461e*RSe463p + 2./81.*mp2*
         RSp461e*RSe561p - 2./81.*mp2*RSp461e*RSe461p + 1./81.*m55*
         RSp563e*RSe563p + 1./81.*m55*RSp563e*RSe463p + 1./81.*m55*
         RSp463e*RSe563p + 1./27.*m55*RSp463e*RSe463p - 1./81.*m55*
         RSe561p*RSp563e - 1./81.*m55*RSe561p*RSp463e - 1./81.*m55*
         RSe461p*RSp563e - 1./27.*m55*RSe461p*RSp463e - 1./81.*m55*
         RSp561e*RSe563p - 1./81.*m55*RSp561e*RSe463p + 1./81.*m55*
         RSp561e*RSe561p + 1./81.*m55*RSp561e*RSe461p - 1./81.*m55*
         RSp461e*RSe563p );

      F13p6x23RV +=  + AbV(m45,L)*BbV(mp2,me2,L) * (  - 1./27.*m55*
         RSp461e*RSe463p + 1./81.*m55*RSp461e*RSe561p + 1./27.*m55*
         RSp461e*RSe461p - 2./81.*m45*RSp563e*RSe563p + 2./81.*m45*
         RSp563e*RSe463p + 2./81.*m45*RSp463e*RSe563p - 2./81.*m45*
         RSp463e*RSe463p + 2./81.*m45*RSe561p*RSp563e - 2./81.*m45*
         RSe561p*RSp463e - 2./81.*m45*RSe461p*RSp563e + 2./81.*m45*
         RSe461p*RSp463e + 2./81.*m45*RSp561e*RSe563p - 2./81.*m45*
         RSp561e*RSe463p - 2./81.*m45*RSp561e*RSe561p + 2./81.*m45*
         RSp561e*RSe461p - 2./81.*m45*RSp461e*RSe563p + 2./81.*m45*
         RSp461e*RSe463p + 2./81.*m45*RSp461e*RSe561p - 2./81.*m45*
         RSp461e*RSe461p + 1./27.*m44*RSp563e*RSe563p + 1./81.*m44*
         RSp563e*RSe463p + 1./81.*m44*RSp463e*RSe563p + 1./81.*m44*
         RSp463e*RSe463p - 1./27.*m44*RSe561p*RSp563e - 1./81.*m44*
         RSe561p*RSp463e - 1./81.*m44*RSe461p*RSp563e - 1./81.*m44*
         RSe461p*RSp463e - 1./27.*m44*RSp561e*RSe563p - 1./81.*m44*
         RSp561e*RSe463p );

      F13p6x23RV +=  + AbV(m45,L)*BbV(mp2,me2,L) * ( 1./27.*m44*RSp561e
         *RSe561p + 1./81.*m44*RSp561e*RSe461p - 1./81.*m44*RSp461e*
         RSe563p - 1./81.*m44*RSp461e*RSe463p + 1./81.*m44*RSp461e*
         RSe561p + 1./81.*m44*RSp461e*RSe461p );

      F13p6x23RV +=  + AbV(m45,L)*BbV(mp2,L) * (  - 1./108.*mp2*pow(
         RSp563e,2) + 1./54.*mp2*RSp463e*RSp563e - 1./108.*mp2*pow(
         RSp463e,2) + 1./54.*mp2*RSp561e*RSp563e - 1./54.*mp2*RSp561e*
         RSp463e - 1./108.*mp2*pow(RSp561e,2) - 1./54.*mp2*RSp461e*
         RSp563e + 1./54.*mp2*RSp461e*RSp463e + 1./54.*mp2*RSp461e*
         RSp561e - 1./108.*mp2*pow(RSp461e,2) + 1./216.*m55*pow(
         RSp563e,2) + 1./108.*m55*RSp463e*RSp563e + 1./72.*m55*pow(
         RSp463e,2) - 1./108.*m55*RSp561e*RSp563e - 1./108.*m55*RSp561e
         *RSp463e + 1./216.*m55*pow(RSp561e,2) - 1./108.*m55*RSp461e*
         RSp563e - 1./36.*m55*RSp461e*RSp463e + 1./108.*m55*RSp461e*
         RSp561e + 1./72.*m55*pow(RSp461e,2) - 1./108.*m45*pow(
         RSp563e,2) + 1./54.*m45*RSp463e*RSp563e - 1./108.*m45*pow(
         RSp463e,2) + 1./54.*m45*RSp561e*RSp563e - 1./54.*m45*RSp561e*
         RSp463e - 1./108.*m45*pow(RSp561e,2) - 1./54.*m45*RSp461e*
         RSp563e + 1./54.*m45*RSp461e*RSp463e + 1./54.*m45*RSp461e*
         RSp561e );

      F13p6x23RV +=  + AbV(m45,L)*BbV(mp2,L) * (  - 1./108.*m45*pow(
         RSp461e,2) + 1./72.*m44*pow(RSp563e,2) + 1./108.*m44*RSp463e*
         RSp563e + 1./216.*m44*pow(RSp463e,2) - 1./36.*m44*RSp561e*
         RSp563e - 1./108.*m44*RSp561e*RSp463e + 1./72.*m44*pow(
         RSp561e,2) - 1./108.*m44*RSp461e*RSp563e - 1./108.*m44*RSp461e
         *RSp463e + 1./108.*m44*RSp461e*RSp561e + 1./216.*m44*pow(
         RSp461e,2) );

      F13p6x23RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * ( 1./162.*me2*
         RSp563e*RSe563p - 1./162.*me2*RSp563e*RSe463p - 1./162.*me2*
         RSp463e*RSe563p + 1./162.*me2*RSp463e*RSe463p - 1./162.*me2*
         RSe561p*RSp563e + 1./162.*me2*RSe561p*RSp463e + 1./162.*me2*
         RSe461p*RSp563e - 1./162.*me2*RSe461p*RSp463e - 1./162.*me2*
         RSp561e*RSe563p + 1./162.*me2*RSp561e*RSe463p + 1./162.*me2*
         RSp561e*RSe561p - 1./162.*me2*RSp561e*RSe461p + 1./162.*me2*
         RSp461e*RSe563p - 1./162.*me2*RSp461e*RSe463p - 1./162.*me2*
         RSp461e*RSe561p + 1./162.*me2*RSp461e*RSe461p - 1./324.*m55*
         RSp563e*RSe563p - 1./324.*m55*RSp563e*RSe463p - 1./324.*m55*
         RSp463e*RSe563p - 1./108.*m55*RSp463e*RSe463p + 1./324.*m55*
         RSe561p*RSp563e + 1./324.*m55*RSe561p*RSp463e + 1./324.*m55*
         RSe461p*RSp563e + 1./108.*m55*RSe461p*RSp463e + 1./324.*m55*
         RSp561e*RSe563p + 1./324.*m55*RSp561e*RSe463p - 1./324.*m55*
         RSp561e*RSe561p - 1./324.*m55*RSp561e*RSe461p + 1./324.*m55*
         RSp461e*RSe563p );

      F13p6x23RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * ( 1./108.*m55*
         RSp461e*RSe463p - 1./324.*m55*RSp461e*RSe561p - 1./108.*m55*
         RSp461e*RSe461p + 1./162.*m45*RSp563e*RSe563p - 1./162.*m45*
         RSp563e*RSe463p - 1./162.*m45*RSp463e*RSe563p + 1./162.*m45*
         RSp463e*RSe463p - 1./162.*m45*RSe561p*RSp563e + 1./162.*m45*
         RSe561p*RSp463e + 1./162.*m45*RSe461p*RSp563e - 1./162.*m45*
         RSe461p*RSp463e - 1./162.*m45*RSp561e*RSe563p + 1./162.*m45*
         RSp561e*RSe463p + 1./162.*m45*RSp561e*RSe561p - 1./162.*m45*
         RSp561e*RSe461p + 1./162.*m45*RSp461e*RSe563p - 1./162.*m45*
         RSp461e*RSe463p - 1./162.*m45*RSp461e*RSe561p + 1./162.*m45*
         RSp461e*RSe461p - 1./108.*m44*RSp563e*RSe563p - 1./324.*m44*
         RSp563e*RSe463p - 1./324.*m44*RSp463e*RSe563p - 1./324.*m44*
         RSp463e*RSe463p + 1./108.*m44*RSe561p*RSp563e + 1./324.*m44*
         RSe561p*RSp463e + 1./324.*m44*RSe461p*RSp563e + 1./324.*m44*
         RSe461p*RSp463e + 1./108.*m44*RSp561e*RSe563p + 1./324.*m44*
         RSp561e*RSe463p );

      F13p6x23RV +=  + AbV(m45,L)*BbV(me2,mp2,L) * (  - 1./108.*m44*
         RSp561e*RSe561p - 1./324.*m44*RSp561e*RSe461p + 1./324.*m44*
         RSp461e*RSe563p + 1./324.*m44*RSp461e*RSe463p - 1./324.*m44*
         RSp461e*RSe561p - 1./324.*m44*RSp461e*RSe461p );

      F13p6x23RV +=  + AbV(m45,L)*BbV(me2,L) * (  - 1./108.*me2*pow(
         RSe563p,2) + 1./54.*me2*RSe463p*RSe563p - 1./108.*me2*pow(
         RSe463p,2) + 1./54.*me2*RSe561p*RSe563p - 1./54.*me2*RSe561p*
         RSe463p - 1./108.*me2*pow(RSe561p,2) - 1./54.*me2*RSe461p*
         RSe563p + 1./54.*me2*RSe461p*RSe463p + 1./54.*me2*RSe461p*
         RSe561p - 1./108.*me2*pow(RSe461p,2) + 1./216.*m55*pow(
         RSe563p,2) + 1./108.*m55*RSe463p*RSe563p + 1./72.*m55*pow(
         RSe463p,2) - 1./108.*m55*RSe561p*RSe563p - 1./108.*m55*RSe561p
         *RSe463p + 1./216.*m55*pow(RSe561p,2) - 1./108.*m55*RSe461p*
         RSe563p - 1./36.*m55*RSe461p*RSe463p + 1./108.*m55*RSe461p*
         RSe561p + 1./72.*m55*pow(RSe461p,2) - 1./108.*m45*pow(
         RSe563p,2) + 1./54.*m45*RSe463p*RSe563p - 1./108.*m45*pow(
         RSe463p,2) + 1./54.*m45*RSe561p*RSe563p - 1./54.*m45*RSe561p*
         RSe463p - 1./108.*m45*pow(RSe561p,2) - 1./54.*m45*RSe461p*
         RSe563p + 1./54.*m45*RSe461p*RSe463p + 1./54.*m45*RSe461p*
         RSe561p );

      F13p6x23RV +=  + AbV(m45,L)*BbV(me2,L) * (  - 1./108.*m45*pow(
         RSe461p,2) + 1./72.*m44*pow(RSe563p,2) + 1./108.*m44*RSe463p*
         RSe563p + 1./216.*m44*pow(RSe463p,2) - 1./36.*m44*RSe561p*
         RSe563p - 1./108.*m44*RSe561p*RSe463p + 1./72.*m44*pow(
         RSe561p,2) - 1./108.*m44*RSe461p*RSe563p - 1./108.*m44*RSe461p
         *RSe463p + 1./108.*m44*RSe461p*RSe561p + 1./216.*m44*pow(
         RSe461p,2) );

      F13p6x23RV +=  + AbV(m46,L)*AbV(mp2,L) * ( 1./54.*RSp563e*RS345pe
          - 1./54.*RSp563e*RS356pe + 1./162.*RSp563e*RSe563p - 1./162.*
         RSp563e*RSe453p - 1./108.*pow(RSp563e,2) - 1./54.*RSp453e*
         RS345pe + 1./54.*RSp453e*RS356pe - 1./162.*RSp453e*RSe563p + 1.
         /162.*RSp453e*RSe453p + 1./54.*RSp453e*RSp563e - 1./108.*pow(
         RSp453e,2) - 1./54.*RS145pe*RSp563e + 1./54.*RS145pe*RSp453e
          + 1./54.*RS156pe*RSp563e - 1./54.*RS156pe*RSp453e - 1./162.*
         RSe561p*RSp563e + 1./162.*RSe561p*RSp453e + 1./162.*RSe451p*
         RSp563e - 1./162.*RSe451p*RSp453e - 1./54.*RSp561e*RS345pe + 1.
         /54.*RSp561e*RS356pe - 1./162.*RSp561e*RSe563p + 1./162.*
         RSp561e*RSe453p + 1./54.*RSp561e*RSp563e - 1./54.*RSp561e*
         RSp453e + 1./54.*RSp561e*RS145pe - 1./54.*RSp561e*RS156pe + 1./
         162.*RSp561e*RSe561p - 1./162.*RSp561e*RSe451p - 1./108.*pow(
         RSp561e,2) + 1./54.*RSp451e*RS345pe - 1./54.*RSp451e*RS356pe
          + 1./162.*RSp451e*RSe563p - 1./162.*RSp451e*RSe453p - 1./54.*
         RSp451e*RSp563e );

      F13p6x23RV +=  + AbV(m46,L)*AbV(mp2,L) * ( 1./54.*RSp451e*RSp453e
          - 1./54.*RSp451e*RS145pe + 1./54.*RSp451e*RS156pe - 1./162.*
         RSp451e*RSe561p + 1./162.*RSp451e*RSe451p + 1./54.*RSp451e*
         RSp561e - 1./108.*pow(RSp451e,2) );

      F13p6x23RV +=  + AbV(m46,L)*AbV(me2,L) * ( 1./54.*RSe563p*RS345pe
          - 1./54.*RSe563p*RS356pe - 1./108.*pow(RSe563p,2) - 1./54.*
         RSe453p*RS345pe + 1./54.*RSe453p*RS356pe + 1./54.*RSe453p*
         RSe563p - 1./108.*pow(RSe453p,2) - 2./81.*RSp563e*RSe563p + 2./
         81.*RSp563e*RSe453p + 2./81.*RSp453e*RSe563p - 2./81.*RSp453e*
         RSe453p - 1./54.*RS145pe*RSe563p + 1./54.*RS145pe*RSe453p + 1./
         54.*RS156pe*RSe563p - 1./54.*RS156pe*RSe453p - 1./54.*RSe561p*
         RS345pe + 1./54.*RSe561p*RS356pe + 1./54.*RSe561p*RSe563p - 1./
         54.*RSe561p*RSe453p + 2./81.*RSe561p*RSp563e - 2./81.*RSe561p*
         RSp453e + 1./54.*RSe561p*RS145pe - 1./54.*RSe561p*RS156pe - 1./
         108.*pow(RSe561p,2) + 1./54.*RSe451p*RS345pe - 1./54.*RSe451p*
         RS356pe - 1./54.*RSe451p*RSe563p + 1./54.*RSe451p*RSe453p - 2./
         81.*RSe451p*RSp563e + 2./81.*RSe451p*RSp453e - 1./54.*RSe451p*
         RS145pe + 1./54.*RSe451p*RS156pe + 1./54.*RSe451p*RSe561p - 1./
         108.*pow(RSe451p,2) + 2./81.*RSp561e*RSe563p - 2./81.*RSp561e*
         RSe453p );

      F13p6x23RV +=  + AbV(m46,L)*AbV(me2,L) * (  - 2./81.*RSp561e*
         RSe561p + 2./81.*RSp561e*RSe451p - 2./81.*RSp451e*RSe563p + 2./
         81.*RSp451e*RSe453p + 2./81.*RSp451e*RSe561p - 2./81.*RSp451e*
         RSe451p );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m11,m33,L) * (  - 1./36.*m66*
         RS145pe*RS345pe - 1./108.*m66*RS145pe*RS356pe - 1./108.*m66*
         RS156pe*RS345pe - 1./108.*m66*RS156pe*RS356pe + 1./54.*m46*
         RS145pe*RS345pe - 1./54.*m46*RS145pe*RS356pe - 1./54.*m46*
         RS156pe*RS345pe + 1./54.*m46*RS156pe*RS356pe - 1./108.*m44*
         RS145pe*RS345pe - 1./108.*m44*RS145pe*RS356pe - 1./108.*m44*
         RS156pe*RS345pe - 1./36.*m44*RS156pe*RS356pe + 1./54.*m11*
         RS145pe*RS345pe - 1./54.*m11*RS145pe*RS356pe - 1./54.*m11*
         RS156pe*RS345pe + 1./54.*m11*RS156pe*RS356pe );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m11,mp2,L) * (  - 1./108.*m66*
         RS145pe*RSp563e - 1./36.*m66*RS145pe*RSp453e - 1./108.*m66*
         RS156pe*RSp563e - 1./108.*m66*RS156pe*RSp453e + 1./108.*m66*
         RSp561e*RS145pe + 1./108.*m66*RSp561e*RS156pe + 1./36.*m66*
         RSp451e*RS145pe + 1./108.*m66*RSp451e*RS156pe - 1./54.*m46*
         RS145pe*RSp563e + 1./54.*m46*RS145pe*RSp453e + 1./54.*m46*
         RS156pe*RSp563e - 1./54.*m46*RS156pe*RSp453e + 1./54.*m46*
         RSp561e*RS145pe - 1./54.*m46*RSp561e*RS156pe - 1./54.*m46*
         RSp451e*RS145pe + 1./54.*m46*RSp451e*RS156pe - 1./108.*m44*
         RS145pe*RSp563e - 1./108.*m44*RS145pe*RSp453e - 1./36.*m44*
         RS156pe*RSp563e - 1./108.*m44*RS156pe*RSp453e + 1./108.*m44*
         RSp561e*RS145pe + 1./36.*m44*RSp561e*RS156pe + 1./108.*m44*
         RSp451e*RS145pe + 1./108.*m44*RSp451e*RS156pe - 1./54.*m11*
         RS145pe*RSp563e + 1./54.*m11*RS145pe*RSp453e + 1./54.*m11*
         RS156pe*RSp563e - 1./54.*m11*RS156pe*RSp453e + 1./54.*m11*
         RSp561e*RS145pe );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m11,mp2,L) * (  - 1./54.*m11*
         RSp561e*RS156pe - 1./54.*m11*RSp451e*RS145pe + 1./54.*m11*
         RSp451e*RS156pe );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m11,me2,L) * (  - 1./108.*m66*
         RS145pe*RSe563p - 1./36.*m66*RS145pe*RSe453p - 1./108.*m66*
         RS156pe*RSe563p - 1./108.*m66*RS156pe*RSe453p + 1./108.*m66*
         RSe561p*RS145pe + 1./108.*m66*RSe561p*RS156pe + 1./36.*m66*
         RSe451p*RS145pe + 1./108.*m66*RSe451p*RS156pe - 1./54.*m46*
         RS145pe*RSe563p + 1./54.*m46*RS145pe*RSe453p + 1./54.*m46*
         RS156pe*RSe563p - 1./54.*m46*RS156pe*RSe453p + 1./54.*m46*
         RSe561p*RS145pe - 1./54.*m46*RSe561p*RS156pe - 1./54.*m46*
         RSe451p*RS145pe + 1./54.*m46*RSe451p*RS156pe - 1./108.*m44*
         RS145pe*RSe563p - 1./108.*m44*RS145pe*RSe453p - 1./36.*m44*
         RS156pe*RSe563p - 1./108.*m44*RS156pe*RSe453p + 1./108.*m44*
         RSe561p*RS145pe + 1./36.*m44*RSe561p*RS156pe + 1./108.*m44*
         RSe451p*RS145pe + 1./108.*m44*RSe451p*RS156pe - 1./54.*m11*
         RS145pe*RSe563p + 1./54.*m11*RS145pe*RSe453p + 1./54.*m11*
         RS156pe*RSe563p - 1./54.*m11*RS156pe*RSe453p + 1./54.*m11*
         RSe561p*RS145pe );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m11,me2,L) * (  - 1./54.*m11*
         RSe561p*RS156pe - 1./54.*m11*RSe451p*RS145pe + 1./54.*m11*
         RSe451p*RS156pe );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m11,L) * ( 1./72.*m66*pow(
         RS145pe,2) + 1./108.*m66*RS156pe*RS145pe + 1./216.*m66*pow(
         RS156pe,2) - 1./108.*m46*pow(RS145pe,2) + 1./54.*m46*RS156pe*
         RS145pe - 1./108.*m46*pow(RS156pe,2) + 1./216.*m44*pow(
         RS145pe,2) + 1./108.*m44*RS156pe*RS145pe + 1./72.*m44*pow(
         RS156pe,2) - 1./108.*m11*pow(RS145pe,2) + 1./54.*m11*RS156pe*
         RS145pe - 1./108.*m11*pow(RS156pe,2) );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m14,L) * ( 1./48.*m66 - 1./24.*
         m46 + 1./24.*m44 - 1./24.*m14 + 1./48.*m11 );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m16,L) * ( 1./24.*m66 - 1./24.*
         m46 + 1./48.*m44 - 1./24.*m16 + 1./48.*m11 );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m33,mp2,L) * ( 1./108.*m66*
         RSp563e*RS345pe + 1./108.*m66*RSp563e*RS356pe + 1./36.*m66*
         RSp453e*RS345pe + 1./108.*m66*RSp453e*RS356pe - 1./108.*m66*
         RSp561e*RS345pe - 1./108.*m66*RSp561e*RS356pe - 1./36.*m66*
         RSp451e*RS345pe - 1./108.*m66*RSp451e*RS356pe + 1./54.*m46*
         RSp563e*RS345pe - 1./54.*m46*RSp563e*RS356pe - 1./54.*m46*
         RSp453e*RS345pe + 1./54.*m46*RSp453e*RS356pe - 1./54.*m46*
         RSp561e*RS345pe + 1./54.*m46*RSp561e*RS356pe + 1./54.*m46*
         RSp451e*RS345pe - 1./54.*m46*RSp451e*RS356pe + 1./108.*m44*
         RSp563e*RS345pe + 1./36.*m44*RSp563e*RS356pe + 1./108.*m44*
         RSp453e*RS345pe + 1./108.*m44*RSp453e*RS356pe - 1./108.*m44*
         RSp561e*RS345pe - 1./36.*m44*RSp561e*RS356pe - 1./108.*m44*
         RSp451e*RS345pe - 1./108.*m44*RSp451e*RS356pe + 1./54.*m33*
         RSp563e*RS345pe - 1./54.*m33*RSp563e*RS356pe - 1./54.*m33*
         RSp453e*RS345pe + 1./54.*m33*RSp453e*RS356pe - 1./54.*m33*
         RSp561e*RS345pe );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m33,mp2,L) * ( 1./54.*m33*RSp561e
         *RS356pe + 1./54.*m33*RSp451e*RS345pe - 1./54.*m33*RSp451e*
         RS356pe );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m33,me2,L) * ( 1./108.*m66*
         RSe563p*RS345pe + 1./108.*m66*RSe563p*RS356pe + 1./36.*m66*
         RSe453p*RS345pe + 1./108.*m66*RSe453p*RS356pe - 1./108.*m66*
         RSe561p*RS345pe - 1./108.*m66*RSe561p*RS356pe - 1./36.*m66*
         RSe451p*RS345pe - 1./108.*m66*RSe451p*RS356pe + 1./54.*m46*
         RSe563p*RS345pe - 1./54.*m46*RSe563p*RS356pe - 1./54.*m46*
         RSe453p*RS345pe + 1./54.*m46*RSe453p*RS356pe - 1./54.*m46*
         RSe561p*RS345pe + 1./54.*m46*RSe561p*RS356pe + 1./54.*m46*
         RSe451p*RS345pe - 1./54.*m46*RSe451p*RS356pe + 1./108.*m44*
         RSe563p*RS345pe + 1./36.*m44*RSe563p*RS356pe + 1./108.*m44*
         RSe453p*RS345pe + 1./108.*m44*RSe453p*RS356pe - 1./108.*m44*
         RSe561p*RS345pe - 1./36.*m44*RSe561p*RS356pe - 1./108.*m44*
         RSe451p*RS345pe - 1./108.*m44*RSe451p*RS356pe + 1./54.*m33*
         RSe563p*RS345pe - 1./54.*m33*RSe563p*RS356pe - 1./54.*m33*
         RSe453p*RS345pe + 1./54.*m33*RSe453p*RS356pe - 1./54.*m33*
         RSe561p*RS345pe );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m33,me2,L) * ( 1./54.*m33*RSe561p
         *RS356pe + 1./54.*m33*RSe451p*RS345pe - 1./54.*m33*RSe451p*
         RS356pe );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m33,L) * ( 1./72.*m66*pow(
         RS345pe,2) + 1./108.*m66*RS356pe*RS345pe + 1./216.*m66*pow(
         RS356pe,2) - 1./108.*m46*pow(RS345pe,2) + 1./54.*m46*RS356pe*
         RS345pe - 1./108.*m46*pow(RS356pe,2) + 1./216.*m44*pow(
         RS345pe,2) + 1./108.*m44*RS356pe*RS345pe + 1./72.*m44*pow(
         RS356pe,2) - 1./108.*m33*pow(RS345pe,2) + 1./54.*m33*RS356pe*
         RS345pe - 1./108.*m33*pow(RS356pe,2) );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m34,L) * ( 1./48.*m66 - 1./24.*
         m46 + 1./24.*m44 - 1./24.*m34 + 1./48.*m33 );

      F13p6x23RV +=  + AbV(m46,L)*BbV(m36,L) * ( 1./24.*m66 - 1./24.*
         m46 + 1./48.*m44 - 1./24.*m36 + 1./48.*m33 );

      F13p6x23RV +=  + AbV(m46,L)*BbV(mp2,me2,L) * (  - 2./81.*mp2*
         RSp563e*RSe563p + 2./81.*mp2*RSp563e*RSe453p + 2./81.*mp2*
         RSp453e*RSe563p - 2./81.*mp2*RSp453e*RSe453p + 2./81.*mp2*
         RSe561p*RSp563e - 2./81.*mp2*RSe561p*RSp453e - 2./81.*mp2*
         RSe451p*RSp563e + 2./81.*mp2*RSe451p*RSp453e + 2./81.*mp2*
         RSp561e*RSe563p - 2./81.*mp2*RSp561e*RSe453p - 2./81.*mp2*
         RSp561e*RSe561p + 2./81.*mp2*RSp561e*RSe451p - 2./81.*mp2*
         RSp451e*RSe563p + 2./81.*mp2*RSp451e*RSe453p + 2./81.*mp2*
         RSp451e*RSe561p - 2./81.*mp2*RSp451e*RSe451p + 1./81.*m66*
         RSp563e*RSe563p + 1./81.*m66*RSp563e*RSe453p + 1./81.*m66*
         RSp453e*RSe563p + 1./27.*m66*RSp453e*RSe453p - 1./81.*m66*
         RSe561p*RSp563e - 1./81.*m66*RSe561p*RSp453e - 1./81.*m66*
         RSe451p*RSp563e - 1./27.*m66*RSe451p*RSp453e - 1./81.*m66*
         RSp561e*RSe563p - 1./81.*m66*RSp561e*RSe453p + 1./81.*m66*
         RSp561e*RSe561p + 1./81.*m66*RSp561e*RSe451p - 1./81.*m66*
         RSp451e*RSe563p );

      F13p6x23RV +=  + AbV(m46,L)*BbV(mp2,me2,L) * (  - 1./27.*m66*
         RSp451e*RSe453p + 1./81.*m66*RSp451e*RSe561p + 1./27.*m66*
         RSp451e*RSe451p - 2./81.*m46*RSp563e*RSe563p + 2./81.*m46*
         RSp563e*RSe453p + 2./81.*m46*RSp453e*RSe563p - 2./81.*m46*
         RSp453e*RSe453p + 2./81.*m46*RSe561p*RSp563e - 2./81.*m46*
         RSe561p*RSp453e - 2./81.*m46*RSe451p*RSp563e + 2./81.*m46*
         RSe451p*RSp453e + 2./81.*m46*RSp561e*RSe563p - 2./81.*m46*
         RSp561e*RSe453p - 2./81.*m46*RSp561e*RSe561p + 2./81.*m46*
         RSp561e*RSe451p - 2./81.*m46*RSp451e*RSe563p + 2./81.*m46*
         RSp451e*RSe453p + 2./81.*m46*RSp451e*RSe561p - 2./81.*m46*
         RSp451e*RSe451p + 1./27.*m44*RSp563e*RSe563p + 1./81.*m44*
         RSp563e*RSe453p + 1./81.*m44*RSp453e*RSe563p + 1./81.*m44*
         RSp453e*RSe453p - 1./27.*m44*RSe561p*RSp563e - 1./81.*m44*
         RSe561p*RSp453e - 1./81.*m44*RSe451p*RSp563e - 1./81.*m44*
         RSe451p*RSp453e - 1./27.*m44*RSp561e*RSe563p - 1./81.*m44*
         RSp561e*RSe453p );

      F13p6x23RV +=  + AbV(m46,L)*BbV(mp2,me2,L) * ( 1./27.*m44*RSp561e
         *RSe561p + 1./81.*m44*RSp561e*RSe451p - 1./81.*m44*RSp451e*
         RSe563p - 1./81.*m44*RSp451e*RSe453p + 1./81.*m44*RSp451e*
         RSe561p + 1./81.*m44*RSp451e*RSe451p );

      F13p6x23RV +=  + AbV(m46,L)*BbV(mp2,L) * (  - 1./108.*mp2*pow(
         RSp563e,2) + 1./54.*mp2*RSp453e*RSp563e - 1./108.*mp2*pow(
         RSp453e,2) + 1./54.*mp2*RSp561e*RSp563e - 1./54.*mp2*RSp561e*
         RSp453e - 1./108.*mp2*pow(RSp561e,2) - 1./54.*mp2*RSp451e*
         RSp563e + 1./54.*mp2*RSp451e*RSp453e + 1./54.*mp2*RSp451e*
         RSp561e - 1./108.*mp2*pow(RSp451e,2) + 1./216.*m66*pow(
         RSp563e,2) + 1./108.*m66*RSp453e*RSp563e + 1./72.*m66*pow(
         RSp453e,2) - 1./108.*m66*RSp561e*RSp563e - 1./108.*m66*RSp561e
         *RSp453e + 1./216.*m66*pow(RSp561e,2) - 1./108.*m66*RSp451e*
         RSp563e - 1./36.*m66*RSp451e*RSp453e + 1./108.*m66*RSp451e*
         RSp561e + 1./72.*m66*pow(RSp451e,2) - 1./108.*m46*pow(
         RSp563e,2) + 1./54.*m46*RSp453e*RSp563e - 1./108.*m46*pow(
         RSp453e,2) + 1./54.*m46*RSp561e*RSp563e - 1./54.*m46*RSp561e*
         RSp453e - 1./108.*m46*pow(RSp561e,2) - 1./54.*m46*RSp451e*
         RSp563e + 1./54.*m46*RSp451e*RSp453e + 1./54.*m46*RSp451e*
         RSp561e );

      F13p6x23RV +=  + AbV(m46,L)*BbV(mp2,L) * (  - 1./108.*m46*pow(
         RSp451e,2) + 1./72.*m44*pow(RSp563e,2) + 1./108.*m44*RSp453e*
         RSp563e + 1./216.*m44*pow(RSp453e,2) - 1./36.*m44*RSp561e*
         RSp563e - 1./108.*m44*RSp561e*RSp453e + 1./72.*m44*pow(
         RSp561e,2) - 1./108.*m44*RSp451e*RSp563e - 1./108.*m44*RSp451e
         *RSp453e + 1./108.*m44*RSp451e*RSp561e + 1./216.*m44*pow(
         RSp451e,2) );

      F13p6x23RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * ( 1./162.*me2*
         RSp563e*RSe563p - 1./162.*me2*RSp563e*RSe453p - 1./162.*me2*
         RSp453e*RSe563p + 1./162.*me2*RSp453e*RSe453p - 1./162.*me2*
         RSe561p*RSp563e + 1./162.*me2*RSe561p*RSp453e + 1./162.*me2*
         RSe451p*RSp563e - 1./162.*me2*RSe451p*RSp453e - 1./162.*me2*
         RSp561e*RSe563p + 1./162.*me2*RSp561e*RSe453p + 1./162.*me2*
         RSp561e*RSe561p - 1./162.*me2*RSp561e*RSe451p + 1./162.*me2*
         RSp451e*RSe563p - 1./162.*me2*RSp451e*RSe453p - 1./162.*me2*
         RSp451e*RSe561p + 1./162.*me2*RSp451e*RSe451p - 1./324.*m66*
         RSp563e*RSe563p - 1./324.*m66*RSp563e*RSe453p - 1./324.*m66*
         RSp453e*RSe563p - 1./108.*m66*RSp453e*RSe453p + 1./324.*m66*
         RSe561p*RSp563e + 1./324.*m66*RSe561p*RSp453e + 1./324.*m66*
         RSe451p*RSp563e + 1./108.*m66*RSe451p*RSp453e + 1./324.*m66*
         RSp561e*RSe563p + 1./324.*m66*RSp561e*RSe453p - 1./324.*m66*
         RSp561e*RSe561p - 1./324.*m66*RSp561e*RSe451p + 1./324.*m66*
         RSp451e*RSe563p );

      F13p6x23RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * ( 1./108.*m66*
         RSp451e*RSe453p - 1./324.*m66*RSp451e*RSe561p - 1./108.*m66*
         RSp451e*RSe451p + 1./162.*m46*RSp563e*RSe563p - 1./162.*m46*
         RSp563e*RSe453p - 1./162.*m46*RSp453e*RSe563p + 1./162.*m46*
         RSp453e*RSe453p - 1./162.*m46*RSe561p*RSp563e + 1./162.*m46*
         RSe561p*RSp453e + 1./162.*m46*RSe451p*RSp563e - 1./162.*m46*
         RSe451p*RSp453e - 1./162.*m46*RSp561e*RSe563p + 1./162.*m46*
         RSp561e*RSe453p + 1./162.*m46*RSp561e*RSe561p - 1./162.*m46*
         RSp561e*RSe451p + 1./162.*m46*RSp451e*RSe563p - 1./162.*m46*
         RSp451e*RSe453p - 1./162.*m46*RSp451e*RSe561p + 1./162.*m46*
         RSp451e*RSe451p - 1./108.*m44*RSp563e*RSe563p - 1./324.*m44*
         RSp563e*RSe453p - 1./324.*m44*RSp453e*RSe563p - 1./324.*m44*
         RSp453e*RSe453p + 1./108.*m44*RSe561p*RSp563e + 1./324.*m44*
         RSe561p*RSp453e + 1./324.*m44*RSe451p*RSp563e + 1./324.*m44*
         RSe451p*RSp453e + 1./108.*m44*RSp561e*RSe563p + 1./324.*m44*
         RSp561e*RSe453p );

      F13p6x23RV +=  + AbV(m46,L)*BbV(me2,mp2,L) * (  - 1./108.*m44*
         RSp561e*RSe561p - 1./324.*m44*RSp561e*RSe451p + 1./324.*m44*
         RSp451e*RSe563p + 1./324.*m44*RSp451e*RSe453p - 1./324.*m44*
         RSp451e*RSe561p - 1./324.*m44*RSp451e*RSe451p );

      F13p6x23RV +=  + AbV(m46,L)*BbV(me2,L) * (  - 1./108.*me2*pow(
         RSe563p,2) + 1./54.*me2*RSe453p*RSe563p - 1./108.*me2*pow(
         RSe453p,2) + 1./54.*me2*RSe561p*RSe563p - 1./54.*me2*RSe561p*
         RSe453p - 1./108.*me2*pow(RSe561p,2) - 1./54.*me2*RSe451p*
         RSe563p + 1./54.*me2*RSe451p*RSe453p + 1./54.*me2*RSe451p*
         RSe561p - 1./108.*me2*pow(RSe451p,2) + 1./216.*m66*pow(
         RSe563p,2) + 1./108.*m66*RSe453p*RSe563p + 1./72.*m66*pow(
         RSe453p,2) - 1./108.*m66*RSe561p*RSe563p - 1./108.*m66*RSe561p
         *RSe453p + 1./216.*m66*pow(RSe561p,2) - 1./108.*m66*RSe451p*
         RSe563p - 1./36.*m66*RSe451p*RSe453p + 1./108.*m66*RSe451p*
         RSe561p + 1./72.*m66*pow(RSe451p,2) - 1./108.*m46*pow(
         RSe563p,2) + 1./54.*m46*RSe453p*RSe563p - 1./108.*m46*pow(
         RSe453p,2) + 1./54.*m46*RSe561p*RSe563p - 1./54.*m46*RSe561p*
         RSe453p - 1./108.*m46*pow(RSe561p,2) - 1./54.*m46*RSe451p*
         RSe563p + 1./54.*m46*RSe451p*RSe453p + 1./54.*m46*RSe451p*
         RSe561p );

      F13p6x23RV +=  + AbV(m46,L)*BbV(me2,L) * (  - 1./108.*m46*pow(
         RSe451p,2) + 1./72.*m44*pow(RSe563p,2) + 1./108.*m44*RSe453p*
         RSe563p + 1./216.*m44*pow(RSe453p,2) - 1./36.*m44*RSe561p*
         RSe563p - 1./108.*m44*RSe561p*RSe453p + 1./72.*m44*pow(
         RSe561p,2) - 1./108.*m44*RSe451p*RSe563p - 1./108.*m44*RSe451p
         *RSe453p + 1./108.*m44*RSe451p*RSe561p + 1./216.*m44*pow(
         RSe451p,2) );

      F13p6x23RV +=  + AbV(m55,L)*BbV(m11,m33,L) * (  - 1./18.*m55*
         RS146pe*RS346pe + 1./54.*m55*RS546pe*RS146pe*RS346pe );

      F13p6x23RV +=  + AbV(m55,L)*BbV(m11,mp2,L) * (  - 1./18.*m55*
         RS146pe*RSp463e + 1./54.*m55*RS546pe*RS146pe*RSp463e + 1./18.*
         m55*RSp461e*RS146pe - 1./54.*m55*RSp461e*RS546pe*RS146pe );

      F13p6x23RV +=  + AbV(m55,L)*BbV(m11,me2,L) * (  - 1./18.*m55*
         RS146pe*RSe463p + 1./54.*m55*RS546pe*RS146pe*RSe463p + 1./18.*
         m55*RSe461p*RS146pe - 1./54.*m55*RSe461p*RS546pe*RS146pe );

      F13p6x23RV +=  + AbV(m55,L)*BbV(m11,L) * ( 1./36.*m55*pow(
         RS146pe,2) - 1./108.*m55*RS546pe*pow(RS146pe,2) );

      F13p6x23RV +=  + AbV(m55,L)*BbV(m15,L) * ( 1./48.*m55 - 1./144.*
         m55*RS546pe - 1./24.*m15 + 1./72.*m15*RS546pe + 1./48.*m11 - 1.
         /144.*m11*RS546pe );

      F13p6x23RV +=  + AbV(m55,L)*BbV(m33,mp2,L) * ( 1./18.*m55*RSp463e
         *RS346pe - 1./54.*m55*RS546pe*RSp463e*RS346pe - 1./18.*m55*
         RSp461e*RS346pe + 1./54.*m55*RSp461e*RS546pe*RS346pe );

      F13p6x23RV +=  + AbV(m55,L)*BbV(m33,me2,L) * ( 1./18.*m55*RSe463p
         *RS346pe - 1./54.*m55*RS546pe*RSe463p*RS346pe - 1./18.*m55*
         RSe461p*RS346pe + 1./54.*m55*RSe461p*RS546pe*RS346pe );

      F13p6x23RV +=  + AbV(m55,L)*BbV(m33,L) * ( 1./36.*m55*pow(
         RS346pe,2) - 1./108.*m55*RS546pe*pow(RS346pe,2) );

      F13p6x23RV +=  + AbV(m55,L)*BbV(m35,L) * ( 1./48.*m55 - 1./144.*
         m55*RS546pe - 1./24.*m35 + 1./72.*m35*RS546pe + 1./48.*m33 - 1.
         /144.*m33*RS546pe );

      F13p6x23RV +=  + AbV(m55,L)*BbV(mp2,me2,L) * ( 2./27.*m55*RSp463e
         *RSe463p - 2./81.*m55*RS546pe*RSp463e*RSe463p - 2./27.*m55*
         RSe461p*RSp463e + 2./81.*m55*RSe461p*RS546pe*RSp463e - 2./27.*
         m55*RSp461e*RSe463p + 2./81.*m55*RSp461e*RS546pe*RSe463p + 2./
         27.*m55*RSp461e*RSe461p - 2./81.*m55*RSp461e*RSe461p*RS546pe )
         ;

      F13p6x23RV +=  + AbV(m55,L)*BbV(mp2,L) * ( 1./36.*m55*pow(
         RSp463e,2) - 1./108.*m55*RS546pe*pow(RSp463e,2) - 1./18.*m55*
         RSp461e*RSp463e + 1./54.*m55*RSp461e*RS546pe*RSp463e + 1./36.*
         m55*pow(RSp461e,2) - 1./108.*m55*pow(RSp461e,2)*RS546pe );

      F13p6x23RV +=  + AbV(m55,L)*BbV(me2,mp2,L) * (  - 1./54.*m55*
         RSp463e*RSe463p + 1./162.*m55*RS546pe*RSp463e*RSe463p + 1./54.
         *m55*RSe461p*RSp463e - 1./162.*m55*RSe461p*RS546pe*RSp463e + 1.
         /54.*m55*RSp461e*RSe463p - 1./162.*m55*RSp461e*RS546pe*RSe463p
          - 1./54.*m55*RSp461e*RSe461p + 1./162.*m55*RSp461e*RSe461p*
         RS546pe );

      F13p6x23RV +=  + AbV(m55,L)*BbV(me2,L) * ( 1./36.*m55*pow(
         RSe463p,2) - 1./108.*m55*RS546pe*pow(RSe463p,2) - 1./18.*m55*
         RSe461p*RSe463p + 1./54.*m55*RSe461p*RS546pe*RSe463p + 1./36.*
         m55*pow(RSe461p,2) - 1./108.*m55*pow(RSe461p,2)*RS546pe );

      F13p6x23RV +=  + AbV(m56,L)*AbV(mp2,L) * ( 1./54.*RSp463e*RS345pe
          - 1./54.*RSp463e*RS346pe + 1./162.*RSp463e*RSe463p - 1./162.*
         RSp463e*RSe453p - 1./108.*pow(RSp463e,2) - 1./54.*RSp453e*
         RS345pe + 1./54.*RSp453e*RS346pe - 1./162.*RSp453e*RSe463p + 1.
         /162.*RSp453e*RSe453p + 1./54.*RSp453e*RSp463e - 1./108.*pow(
         RSp453e,2) - 1./54.*RS145pe*RSp463e + 1./54.*RS145pe*RSp453e
          + 1./54.*RS146pe*RSp463e - 1./54.*RS146pe*RSp453e - 1./162.*
         RSe461p*RSp463e + 1./162.*RSe461p*RSp453e + 1./162.*RSe451p*
         RSp463e - 1./162.*RSe451p*RSp453e - 1./54.*RSp461e*RS345pe + 1.
         /54.*RSp461e*RS346pe - 1./162.*RSp461e*RSe463p + 1./162.*
         RSp461e*RSe453p + 1./54.*RSp461e*RSp463e - 1./54.*RSp461e*
         RSp453e + 1./54.*RSp461e*RS145pe - 1./54.*RSp461e*RS146pe + 1./
         162.*RSp461e*RSe461p - 1./162.*RSp461e*RSe451p - 1./108.*pow(
         RSp461e,2) + 1./54.*RSp451e*RS345pe - 1./54.*RSp451e*RS346pe
          + 1./162.*RSp451e*RSe463p - 1./162.*RSp451e*RSe453p - 1./54.*
         RSp451e*RSp463e );

      F13p6x23RV +=  + AbV(m56,L)*AbV(mp2,L) * ( 1./54.*RSp451e*RSp453e
          - 1./54.*RSp451e*RS145pe + 1./54.*RSp451e*RS146pe - 1./162.*
         RSp451e*RSe461p + 1./162.*RSp451e*RSe451p + 1./54.*RSp451e*
         RSp461e - 1./108.*pow(RSp451e,2) );

      F13p6x23RV +=  + AbV(m56,L)*AbV(me2,L) * ( 1./54.*RSe463p*RS345pe
          - 1./54.*RSe463p*RS346pe - 1./108.*pow(RSe463p,2) - 1./54.*
         RSe453p*RS345pe + 1./54.*RSe453p*RS346pe + 1./54.*RSe453p*
         RSe463p - 1./108.*pow(RSe453p,2) - 2./81.*RSp463e*RSe463p + 2./
         81.*RSp463e*RSe453p + 2./81.*RSp453e*RSe463p - 2./81.*RSp453e*
         RSe453p - 1./54.*RS145pe*RSe463p + 1./54.*RS145pe*RSe453p + 1./
         54.*RS146pe*RSe463p - 1./54.*RS146pe*RSe453p - 1./54.*RSe461p*
         RS345pe + 1./54.*RSe461p*RS346pe + 1./54.*RSe461p*RSe463p - 1./
         54.*RSe461p*RSe453p + 2./81.*RSe461p*RSp463e - 2./81.*RSe461p*
         RSp453e + 1./54.*RSe461p*RS145pe - 1./54.*RSe461p*RS146pe - 1./
         108.*pow(RSe461p,2) + 1./54.*RSe451p*RS345pe - 1./54.*RSe451p*
         RS346pe - 1./54.*RSe451p*RSe463p + 1./54.*RSe451p*RSe453p - 2./
         81.*RSe451p*RSp463e + 2./81.*RSe451p*RSp453e - 1./54.*RSe451p*
         RS145pe + 1./54.*RSe451p*RS146pe + 1./54.*RSe451p*RSe461p - 1./
         108.*pow(RSe451p,2) + 2./81.*RSp461e*RSe463p - 2./81.*RSp461e*
         RSe453p );

      F13p6x23RV +=  + AbV(m56,L)*AbV(me2,L) * (  - 2./81.*RSp461e*
         RSe461p + 2./81.*RSp461e*RSe451p - 2./81.*RSp451e*RSe463p + 2./
         81.*RSp451e*RSe453p + 2./81.*RSp451e*RSe461p - 2./81.*RSp451e*
         RSe451p );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m11,m33,L) * (  - 1./36.*m66*
         RS145pe*RS345pe - 1./108.*m66*RS145pe*RS346pe - 1./108.*m66*
         RS146pe*RS345pe - 1./108.*m66*RS146pe*RS346pe + 1./54.*m56*
         RS145pe*RS345pe - 1./54.*m56*RS145pe*RS346pe - 1./54.*m56*
         RS146pe*RS345pe + 1./54.*m56*RS146pe*RS346pe - 1./108.*m55*
         RS145pe*RS345pe - 1./108.*m55*RS145pe*RS346pe - 1./108.*m55*
         RS146pe*RS345pe - 1./36.*m55*RS146pe*RS346pe + 1./54.*m11*
         RS145pe*RS345pe - 1./54.*m11*RS145pe*RS346pe - 1./54.*m11*
         RS146pe*RS345pe + 1./54.*m11*RS146pe*RS346pe );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m11,mp2,L) * (  - 1./108.*m66*
         RS145pe*RSp463e - 1./36.*m66*RS145pe*RSp453e - 1./108.*m66*
         RS146pe*RSp463e - 1./108.*m66*RS146pe*RSp453e + 1./108.*m66*
         RSp461e*RS145pe + 1./108.*m66*RSp461e*RS146pe + 1./36.*m66*
         RSp451e*RS145pe + 1./108.*m66*RSp451e*RS146pe - 1./54.*m56*
         RS145pe*RSp463e + 1./54.*m56*RS145pe*RSp453e + 1./54.*m56*
         RS146pe*RSp463e - 1./54.*m56*RS146pe*RSp453e + 1./54.*m56*
         RSp461e*RS145pe - 1./54.*m56*RSp461e*RS146pe - 1./54.*m56*
         RSp451e*RS145pe + 1./54.*m56*RSp451e*RS146pe - 1./108.*m55*
         RS145pe*RSp463e - 1./108.*m55*RS145pe*RSp453e - 1./36.*m55*
         RS146pe*RSp463e - 1./108.*m55*RS146pe*RSp453e + 1./108.*m55*
         RSp461e*RS145pe + 1./36.*m55*RSp461e*RS146pe + 1./108.*m55*
         RSp451e*RS145pe + 1./108.*m55*RSp451e*RS146pe - 1./54.*m11*
         RS145pe*RSp463e + 1./54.*m11*RS145pe*RSp453e + 1./54.*m11*
         RS146pe*RSp463e - 1./54.*m11*RS146pe*RSp453e + 1./54.*m11*
         RSp461e*RS145pe );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m11,mp2,L) * (  - 1./54.*m11*
         RSp461e*RS146pe - 1./54.*m11*RSp451e*RS145pe + 1./54.*m11*
         RSp451e*RS146pe );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m11,me2,L) * (  - 1./108.*m66*
         RS145pe*RSe463p - 1./36.*m66*RS145pe*RSe453p - 1./108.*m66*
         RS146pe*RSe463p - 1./108.*m66*RS146pe*RSe453p + 1./108.*m66*
         RSe461p*RS145pe + 1./108.*m66*RSe461p*RS146pe + 1./36.*m66*
         RSe451p*RS145pe + 1./108.*m66*RSe451p*RS146pe - 1./54.*m56*
         RS145pe*RSe463p + 1./54.*m56*RS145pe*RSe453p + 1./54.*m56*
         RS146pe*RSe463p - 1./54.*m56*RS146pe*RSe453p + 1./54.*m56*
         RSe461p*RS145pe - 1./54.*m56*RSe461p*RS146pe - 1./54.*m56*
         RSe451p*RS145pe + 1./54.*m56*RSe451p*RS146pe - 1./108.*m55*
         RS145pe*RSe463p - 1./108.*m55*RS145pe*RSe453p - 1./36.*m55*
         RS146pe*RSe463p - 1./108.*m55*RS146pe*RSe453p + 1./108.*m55*
         RSe461p*RS145pe + 1./36.*m55*RSe461p*RS146pe + 1./108.*m55*
         RSe451p*RS145pe + 1./108.*m55*RSe451p*RS146pe - 1./54.*m11*
         RS145pe*RSe463p + 1./54.*m11*RS145pe*RSe453p + 1./54.*m11*
         RS146pe*RSe463p - 1./54.*m11*RS146pe*RSe453p + 1./54.*m11*
         RSe461p*RS145pe );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m11,me2,L) * (  - 1./54.*m11*
         RSe461p*RS146pe - 1./54.*m11*RSe451p*RS145pe + 1./54.*m11*
         RSe451p*RS146pe );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m11,L) * ( 1./72.*m66*pow(
         RS145pe,2) + 1./108.*m66*RS146pe*RS145pe + 1./216.*m66*pow(
         RS146pe,2) - 1./108.*m56*pow(RS145pe,2) + 1./54.*m56*RS146pe*
         RS145pe - 1./108.*m56*pow(RS146pe,2) + 1./216.*m55*pow(
         RS145pe,2) + 1./108.*m55*RS146pe*RS145pe + 1./72.*m55*pow(
         RS146pe,2) - 1./108.*m11*pow(RS145pe,2) + 1./54.*m11*RS146pe*
         RS145pe - 1./108.*m11*pow(RS146pe,2) );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m15,L) * ( 1./48.*m66 - 1./24.*
         m56 + 1./24.*m55 - 1./24.*m15 + 1./48.*m11 );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m16,L) * ( 1./24.*m66 - 1./24.*
         m56 + 1./48.*m55 - 1./24.*m16 + 1./48.*m11 );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m33,mp2,L) * ( 1./108.*m66*
         RSp463e*RS345pe + 1./108.*m66*RSp463e*RS346pe + 1./36.*m66*
         RSp453e*RS345pe + 1./108.*m66*RSp453e*RS346pe - 1./108.*m66*
         RSp461e*RS345pe - 1./108.*m66*RSp461e*RS346pe - 1./36.*m66*
         RSp451e*RS345pe - 1./108.*m66*RSp451e*RS346pe + 1./54.*m56*
         RSp463e*RS345pe - 1./54.*m56*RSp463e*RS346pe - 1./54.*m56*
         RSp453e*RS345pe + 1./54.*m56*RSp453e*RS346pe - 1./54.*m56*
         RSp461e*RS345pe + 1./54.*m56*RSp461e*RS346pe + 1./54.*m56*
         RSp451e*RS345pe - 1./54.*m56*RSp451e*RS346pe + 1./108.*m55*
         RSp463e*RS345pe + 1./36.*m55*RSp463e*RS346pe + 1./108.*m55*
         RSp453e*RS345pe + 1./108.*m55*RSp453e*RS346pe - 1./108.*m55*
         RSp461e*RS345pe - 1./36.*m55*RSp461e*RS346pe - 1./108.*m55*
         RSp451e*RS345pe - 1./108.*m55*RSp451e*RS346pe + 1./54.*m33*
         RSp463e*RS345pe - 1./54.*m33*RSp463e*RS346pe - 1./54.*m33*
         RSp453e*RS345pe + 1./54.*m33*RSp453e*RS346pe - 1./54.*m33*
         RSp461e*RS345pe );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m33,mp2,L) * ( 1./54.*m33*RSp461e
         *RS346pe + 1./54.*m33*RSp451e*RS345pe - 1./54.*m33*RSp451e*
         RS346pe );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m33,me2,L) * ( 1./108.*m66*
         RSe463p*RS345pe + 1./108.*m66*RSe463p*RS346pe + 1./36.*m66*
         RSe453p*RS345pe + 1./108.*m66*RSe453p*RS346pe - 1./108.*m66*
         RSe461p*RS345pe - 1./108.*m66*RSe461p*RS346pe - 1./36.*m66*
         RSe451p*RS345pe - 1./108.*m66*RSe451p*RS346pe + 1./54.*m56*
         RSe463p*RS345pe - 1./54.*m56*RSe463p*RS346pe - 1./54.*m56*
         RSe453p*RS345pe + 1./54.*m56*RSe453p*RS346pe - 1./54.*m56*
         RSe461p*RS345pe + 1./54.*m56*RSe461p*RS346pe + 1./54.*m56*
         RSe451p*RS345pe - 1./54.*m56*RSe451p*RS346pe + 1./108.*m55*
         RSe463p*RS345pe + 1./36.*m55*RSe463p*RS346pe + 1./108.*m55*
         RSe453p*RS345pe + 1./108.*m55*RSe453p*RS346pe - 1./108.*m55*
         RSe461p*RS345pe - 1./36.*m55*RSe461p*RS346pe - 1./108.*m55*
         RSe451p*RS345pe - 1./108.*m55*RSe451p*RS346pe + 1./54.*m33*
         RSe463p*RS345pe - 1./54.*m33*RSe463p*RS346pe - 1./54.*m33*
         RSe453p*RS345pe + 1./54.*m33*RSe453p*RS346pe - 1./54.*m33*
         RSe461p*RS345pe );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m33,me2,L) * ( 1./54.*m33*RSe461p
         *RS346pe + 1./54.*m33*RSe451p*RS345pe - 1./54.*m33*RSe451p*
         RS346pe );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m33,L) * ( 1./72.*m66*pow(
         RS345pe,2) + 1./108.*m66*RS346pe*RS345pe + 1./216.*m66*pow(
         RS346pe,2) - 1./108.*m56*pow(RS345pe,2) + 1./54.*m56*RS346pe*
         RS345pe - 1./108.*m56*pow(RS346pe,2) + 1./216.*m55*pow(
         RS345pe,2) + 1./108.*m55*RS346pe*RS345pe + 1./72.*m55*pow(
         RS346pe,2) - 1./108.*m33*pow(RS345pe,2) + 1./54.*m33*RS346pe*
         RS345pe - 1./108.*m33*pow(RS346pe,2) );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m35,L) * ( 1./48.*m66 - 1./24.*
         m56 + 1./24.*m55 - 1./24.*m35 + 1./48.*m33 );

      F13p6x23RV +=  + AbV(m56,L)*BbV(m36,L) * ( 1./24.*m66 - 1./24.*
         m56 + 1./48.*m55 - 1./24.*m36 + 1./48.*m33 );

      F13p6x23RV +=  + AbV(m56,L)*BbV(mp2,me2,L) * (  - 2./81.*mp2*
         RSp463e*RSe463p + 2./81.*mp2*RSp463e*RSe453p + 2./81.*mp2*
         RSp453e*RSe463p - 2./81.*mp2*RSp453e*RSe453p + 2./81.*mp2*
         RSe461p*RSp463e - 2./81.*mp2*RSe461p*RSp453e - 2./81.*mp2*
         RSe451p*RSp463e + 2./81.*mp2*RSe451p*RSp453e + 2./81.*mp2*
         RSp461e*RSe463p - 2./81.*mp2*RSp461e*RSe453p - 2./81.*mp2*
         RSp461e*RSe461p + 2./81.*mp2*RSp461e*RSe451p - 2./81.*mp2*
         RSp451e*RSe463p + 2./81.*mp2*RSp451e*RSe453p + 2./81.*mp2*
         RSp451e*RSe461p - 2./81.*mp2*RSp451e*RSe451p + 1./81.*m66*
         RSp463e*RSe463p + 1./81.*m66*RSp463e*RSe453p + 1./81.*m66*
         RSp453e*RSe463p + 1./27.*m66*RSp453e*RSe453p - 1./81.*m66*
         RSe461p*RSp463e - 1./81.*m66*RSe461p*RSp453e - 1./81.*m66*
         RSe451p*RSp463e - 1./27.*m66*RSe451p*RSp453e - 1./81.*m66*
         RSp461e*RSe463p - 1./81.*m66*RSp461e*RSe453p + 1./81.*m66*
         RSp461e*RSe461p + 1./81.*m66*RSp461e*RSe451p - 1./81.*m66*
         RSp451e*RSe463p );

      F13p6x23RV +=  + AbV(m56,L)*BbV(mp2,me2,L) * (  - 1./27.*m66*
         RSp451e*RSe453p + 1./81.*m66*RSp451e*RSe461p + 1./27.*m66*
         RSp451e*RSe451p - 2./81.*m56*RSp463e*RSe463p + 2./81.*m56*
         RSp463e*RSe453p + 2./81.*m56*RSp453e*RSe463p - 2./81.*m56*
         RSp453e*RSe453p + 2./81.*m56*RSe461p*RSp463e - 2./81.*m56*
         RSe461p*RSp453e - 2./81.*m56*RSe451p*RSp463e + 2./81.*m56*
         RSe451p*RSp453e + 2./81.*m56*RSp461e*RSe463p - 2./81.*m56*
         RSp461e*RSe453p - 2./81.*m56*RSp461e*RSe461p + 2./81.*m56*
         RSp461e*RSe451p - 2./81.*m56*RSp451e*RSe463p + 2./81.*m56*
         RSp451e*RSe453p + 2./81.*m56*RSp451e*RSe461p - 2./81.*m56*
         RSp451e*RSe451p + 1./27.*m55*RSp463e*RSe463p + 1./81.*m55*
         RSp463e*RSe453p + 1./81.*m55*RSp453e*RSe463p + 1./81.*m55*
         RSp453e*RSe453p - 1./27.*m55*RSe461p*RSp463e - 1./81.*m55*
         RSe461p*RSp453e - 1./81.*m55*RSe451p*RSp463e - 1./81.*m55*
         RSe451p*RSp453e - 1./27.*m55*RSp461e*RSe463p - 1./81.*m55*
         RSp461e*RSe453p );

      F13p6x23RV +=  + AbV(m56,L)*BbV(mp2,me2,L) * ( 1./27.*m55*RSp461e
         *RSe461p + 1./81.*m55*RSp461e*RSe451p - 1./81.*m55*RSp451e*
         RSe463p - 1./81.*m55*RSp451e*RSe453p + 1./81.*m55*RSp451e*
         RSe461p + 1./81.*m55*RSp451e*RSe451p );

      F13p6x23RV +=  + AbV(m56,L)*BbV(mp2,L) * (  - 1./108.*mp2*pow(
         RSp463e,2) + 1./54.*mp2*RSp453e*RSp463e - 1./108.*mp2*pow(
         RSp453e,2) + 1./54.*mp2*RSp461e*RSp463e - 1./54.*mp2*RSp461e*
         RSp453e - 1./108.*mp2*pow(RSp461e,2) - 1./54.*mp2*RSp451e*
         RSp463e + 1./54.*mp2*RSp451e*RSp453e + 1./54.*mp2*RSp451e*
         RSp461e - 1./108.*mp2*pow(RSp451e,2) + 1./216.*m66*pow(
         RSp463e,2) + 1./108.*m66*RSp453e*RSp463e + 1./72.*m66*pow(
         RSp453e,2) - 1./108.*m66*RSp461e*RSp463e - 1./108.*m66*RSp461e
         *RSp453e + 1./216.*m66*pow(RSp461e,2) - 1./108.*m66*RSp451e*
         RSp463e - 1./36.*m66*RSp451e*RSp453e + 1./108.*m66*RSp451e*
         RSp461e + 1./72.*m66*pow(RSp451e,2) - 1./108.*m56*pow(
         RSp463e,2) + 1./54.*m56*RSp453e*RSp463e - 1./108.*m56*pow(
         RSp453e,2) + 1./54.*m56*RSp461e*RSp463e - 1./54.*m56*RSp461e*
         RSp453e - 1./108.*m56*pow(RSp461e,2) - 1./54.*m56*RSp451e*
         RSp463e + 1./54.*m56*RSp451e*RSp453e + 1./54.*m56*RSp451e*
         RSp461e );

      F13p6x23RV +=  + AbV(m56,L)*BbV(mp2,L) * (  - 1./108.*m56*pow(
         RSp451e,2) + 1./72.*m55*pow(RSp463e,2) + 1./108.*m55*RSp453e*
         RSp463e + 1./216.*m55*pow(RSp453e,2) - 1./36.*m55*RSp461e*
         RSp463e - 1./108.*m55*RSp461e*RSp453e + 1./72.*m55*pow(
         RSp461e,2) - 1./108.*m55*RSp451e*RSp463e - 1./108.*m55*RSp451e
         *RSp453e + 1./108.*m55*RSp451e*RSp461e + 1./216.*m55*pow(
         RSp451e,2) );

      F13p6x23RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * ( 1./162.*me2*
         RSp463e*RSe463p - 1./162.*me2*RSp463e*RSe453p - 1./162.*me2*
         RSp453e*RSe463p + 1./162.*me2*RSp453e*RSe453p - 1./162.*me2*
         RSe461p*RSp463e + 1./162.*me2*RSe461p*RSp453e + 1./162.*me2*
         RSe451p*RSp463e - 1./162.*me2*RSe451p*RSp453e - 1./162.*me2*
         RSp461e*RSe463p + 1./162.*me2*RSp461e*RSe453p + 1./162.*me2*
         RSp461e*RSe461p - 1./162.*me2*RSp461e*RSe451p + 1./162.*me2*
         RSp451e*RSe463p - 1./162.*me2*RSp451e*RSe453p - 1./162.*me2*
         RSp451e*RSe461p + 1./162.*me2*RSp451e*RSe451p - 1./324.*m66*
         RSp463e*RSe463p - 1./324.*m66*RSp463e*RSe453p - 1./324.*m66*
         RSp453e*RSe463p - 1./108.*m66*RSp453e*RSe453p + 1./324.*m66*
         RSe461p*RSp463e + 1./324.*m66*RSe461p*RSp453e + 1./324.*m66*
         RSe451p*RSp463e + 1./108.*m66*RSe451p*RSp453e + 1./324.*m66*
         RSp461e*RSe463p + 1./324.*m66*RSp461e*RSe453p - 1./324.*m66*
         RSp461e*RSe461p - 1./324.*m66*RSp461e*RSe451p + 1./324.*m66*
         RSp451e*RSe463p );

      F13p6x23RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * ( 1./108.*m66*
         RSp451e*RSe453p - 1./324.*m66*RSp451e*RSe461p - 1./108.*m66*
         RSp451e*RSe451p + 1./162.*m56*RSp463e*RSe463p - 1./162.*m56*
         RSp463e*RSe453p - 1./162.*m56*RSp453e*RSe463p + 1./162.*m56*
         RSp453e*RSe453p - 1./162.*m56*RSe461p*RSp463e + 1./162.*m56*
         RSe461p*RSp453e + 1./162.*m56*RSe451p*RSp463e - 1./162.*m56*
         RSe451p*RSp453e - 1./162.*m56*RSp461e*RSe463p + 1./162.*m56*
         RSp461e*RSe453p + 1./162.*m56*RSp461e*RSe461p - 1./162.*m56*
         RSp461e*RSe451p + 1./162.*m56*RSp451e*RSe463p - 1./162.*m56*
         RSp451e*RSe453p - 1./162.*m56*RSp451e*RSe461p + 1./162.*m56*
         RSp451e*RSe451p - 1./108.*m55*RSp463e*RSe463p - 1./324.*m55*
         RSp463e*RSe453p - 1./324.*m55*RSp453e*RSe463p - 1./324.*m55*
         RSp453e*RSe453p + 1./108.*m55*RSe461p*RSp463e + 1./324.*m55*
         RSe461p*RSp453e + 1./324.*m55*RSe451p*RSp463e + 1./324.*m55*
         RSe451p*RSp453e + 1./108.*m55*RSp461e*RSe463p + 1./324.*m55*
         RSp461e*RSe453p );

      F13p6x23RV +=  + AbV(m56,L)*BbV(me2,mp2,L) * (  - 1./108.*m55*
         RSp461e*RSe461p - 1./324.*m55*RSp461e*RSe451p + 1./324.*m55*
         RSp451e*RSe463p + 1./324.*m55*RSp451e*RSe453p - 1./324.*m55*
         RSp451e*RSe461p - 1./324.*m55*RSp451e*RSe451p );

      F13p6x23RV +=  + AbV(m56,L)*BbV(me2,L) * (  - 1./108.*me2*pow(
         RSe463p,2) + 1./54.*me2*RSe453p*RSe463p - 1./108.*me2*pow(
         RSe453p,2) + 1./54.*me2*RSe461p*RSe463p - 1./54.*me2*RSe461p*
         RSe453p - 1./108.*me2*pow(RSe461p,2) - 1./54.*me2*RSe451p*
         RSe463p + 1./54.*me2*RSe451p*RSe453p + 1./54.*me2*RSe451p*
         RSe461p - 1./108.*me2*pow(RSe451p,2) + 1./216.*m66*pow(
         RSe463p,2) + 1./108.*m66*RSe453p*RSe463p + 1./72.*m66*pow(
         RSe453p,2) - 1./108.*m66*RSe461p*RSe463p - 1./108.*m66*RSe461p
         *RSe453p + 1./216.*m66*pow(RSe461p,2) - 1./108.*m66*RSe451p*
         RSe463p - 1./36.*m66*RSe451p*RSe453p + 1./108.*m66*RSe451p*
         RSe461p + 1./72.*m66*pow(RSe451p,2) - 1./108.*m56*pow(
         RSe463p,2) + 1./54.*m56*RSe453p*RSe463p - 1./108.*m56*pow(
         RSe453p,2) + 1./54.*m56*RSe461p*RSe463p - 1./54.*m56*RSe461p*
         RSe453p - 1./108.*m56*pow(RSe461p,2) - 1./54.*m56*RSe451p*
         RSe463p + 1./54.*m56*RSe451p*RSe453p + 1./54.*m56*RSe451p*
         RSe461p );

      F13p6x23RV +=  + AbV(m56,L)*BbV(me2,L) * (  - 1./108.*m56*pow(
         RSe451p,2) + 1./72.*m55*pow(RSe463p,2) + 1./108.*m55*RSe453p*
         RSe463p + 1./216.*m55*pow(RSe453p,2) - 1./36.*m55*RSe461p*
         RSe463p - 1./108.*m55*RSe461p*RSe453p + 1./72.*m55*pow(
         RSe461p,2) - 1./108.*m55*RSe451p*RSe463p - 1./108.*m55*RSe451p
         *RSe453p + 1./108.*m55*RSe451p*RSe461p + 1./216.*m55*pow(
         RSe451p,2) );

      F13p6x23RV +=  + AbV(m66,L)*BbV(m11,m33,L) * (  - 1./18.*m66*
         RS145pe*RS345pe + 1./54.*m66*RS645pe*RS145pe*RS345pe );

      F13p6x23RV +=  + AbV(m66,L)*BbV(m11,mp2,L) * (  - 1./18.*m66*
         RS145pe*RSp453e + 1./54.*m66*RS645pe*RS145pe*RSp453e + 1./18.*
         m66*RSp451e*RS145pe - 1./54.*m66*RSp451e*RS645pe*RS145pe );

      F13p6x23RV +=  + AbV(m66,L)*BbV(m11,me2,L) * (  - 1./18.*m66*
         RS145pe*RSe453p + 1./54.*m66*RS645pe*RS145pe*RSe453p + 1./18.*
         m66*RSe451p*RS145pe - 1./54.*m66*RSe451p*RS645pe*RS145pe );

      F13p6x23RV +=  + AbV(m66,L)*BbV(m11,L) * ( 1./36.*m66*pow(
         RS145pe,2) - 1./108.*m66*RS645pe*pow(RS145pe,2) );

      F13p6x23RV +=  + AbV(m66,L)*BbV(m16,L) * ( 1./48.*m66 - 1./144.*
         m66*RS645pe - 1./24.*m16 + 1./72.*m16*RS645pe + 1./48.*m11 - 1.
         /144.*m11*RS645pe );

      F13p6x23RV +=  + AbV(m66,L)*BbV(m33,mp2,L) * ( 1./18.*m66*RSp453e
         *RS345pe - 1./54.*m66*RS645pe*RSp453e*RS345pe - 1./18.*m66*
         RSp451e*RS345pe + 1./54.*m66*RSp451e*RS645pe*RS345pe );

      F13p6x23RV +=  + AbV(m66,L)*BbV(m33,me2,L) * ( 1./18.*m66*RSe453p
         *RS345pe - 1./54.*m66*RS645pe*RSe453p*RS345pe - 1./18.*m66*
         RSe451p*RS345pe + 1./54.*m66*RSe451p*RS645pe*RS345pe );

      F13p6x23RV +=  + AbV(m66,L)*BbV(m33,L) * ( 1./36.*m66*pow(
         RS345pe,2) - 1./108.*m66*RS645pe*pow(RS345pe,2) );

      F13p6x23RV +=  + AbV(m66,L)*BbV(m36,L) * ( 1./48.*m66 - 1./144.*
         m66*RS645pe - 1./24.*m36 + 1./72.*m36*RS645pe + 1./48.*m33 - 1.
         /144.*m33*RS645pe );

      F13p6x23RV +=  + AbV(m66,L)*BbV(mp2,me2,L) * ( 2./27.*m66*RSp453e
         *RSe453p - 2./81.*m66*RS645pe*RSp453e*RSe453p - 2./27.*m66*
         RSe451p*RSp453e + 2./81.*m66*RSe451p*RS645pe*RSp453e - 2./27.*
         m66*RSp451e*RSe453p + 2./81.*m66*RSp451e*RS645pe*RSe453p + 2./
         27.*m66*RSp451e*RSe451p - 2./81.*m66*RSp451e*RSe451p*RS645pe )
         ;

      F13p6x23RV +=  + AbV(m66,L)*BbV(mp2,L) * ( 1./36.*m66*pow(
         RSp453e,2) - 1./108.*m66*RS645pe*pow(RSp453e,2) - 1./18.*m66*
         RSp451e*RSp453e + 1./54.*m66*RSp451e*RS645pe*RSp453e + 1./36.*
         m66*pow(RSp451e,2) - 1./108.*m66*pow(RSp451e,2)*RS645pe );

      F13p6x23RV +=  + AbV(m66,L)*BbV(me2,mp2,L) * (  - 1./54.*m66*
         RSp453e*RSe453p + 1./162.*m66*RS645pe*RSp453e*RSe453p + 1./54.
         *m66*RSe451p*RSp453e - 1./162.*m66*RSe451p*RS645pe*RSp453e + 1.
         /54.*m66*RSp451e*RSe453p - 1./162.*m66*RSp451e*RS645pe*RSe453p
          - 1./54.*m66*RSp451e*RSe451p + 1./162.*m66*RSp451e*RSe451p*
         RS645pe );

      F13p6x23RV +=  + AbV(m66,L)*BbV(me2,L) * ( 1./36.*m66*pow(
         RSe453p,2) - 1./108.*m66*RS645pe*pow(RSe453p,2) - 1./18.*m66*
         RSe451p*RSe453p + 1./54.*m66*RSe451p*RS645pe*RSe453p + 1./36.*
         m66*pow(RSe451p,2) - 1./108.*m66*pow(RSe451p,2)*RS645pe );

      F13p6x23RV +=  + AbV(mp2,L)*pi16 * ( 1./72.*RD3456ep*RSp45613e - 
         1./144.*RSp45633e*RD3456ep + 1./72.*RD1456ep*RSp45613e - 1./
         144.*RD1456ep*RSp45633e - 1./144.*RSp45611e*RD3456ep - 1./144.
         *RSp45611e*RD1456ep - 1./36.*me2*RSp45613e*RSe45613p + 1./72.*
         me2*RSe45633p*RSp45613e + 1./72.*me2*RSp45633e*RSe45613p - 1./
         144.*me2*RSp45633e*RSe45633p + 1./72.*me2*RSe45611p*RSp45613e
          - 1./144.*me2*RSe45611p*RSp45633e + 1./72.*me2*RSp45611e*
         RSe45613p - 1./144.*me2*RSp45611e*RSe45633p - 1./144.*me2*
         RSp45611e*RSe45611p + 1./24.*mp2*RSp45613e - 1./36.*mp2*pow(
         RSp45613e,2) - 1./48.*mp2*RSp45633e + 1./36.*mp2*RSp45633e*
         RSp45613e - 1./144.*mp2*pow(RSp45633e,2) - 1./48.*mp2*RSp563e
          - 1./48.*mp2*RSp463e - 1./48.*mp2*RSp453e - 1./48.*mp2*
         RSp45611e + 1./36.*mp2*RSp45611e*RSp45613e - 1./72.*mp2*
         RSp45611e*RSp45633e - 1./144.*mp2*pow(RSp45611e,2) - 1./48.*
         mp2*RSp561e - 1./48.*mp2*RSp461e - 1./48.*mp2*RSp451e - 1./24.
         *m36*RSp45613e );

      F13p6x23RV +=  + AbV(mp2,L)*pi16 * ( 1./48.*m36*RSp45633e + 1./48.
         *m36*RSp45611e - 1./24.*m35*RSp45613e + 1./48.*m35*RSp45633e
          + 1./48.*m35*RSp45611e - 1./24.*m34*RSp45613e + 1./48.*m34*
         RSp45633e + 1./48.*m34*RSp45611e - 1./36.*m33*RSp45613e*
         RS34561pe + 1./72.*m33*RC3456pe*RSp45613e + 1./72.*m33*
         RSp45633e*RS34561pe - 1./144.*m33*RSp45633e*RC3456pe + 1./72.*
         m33*RSp45611e*RS34561pe - 1./144.*m33*RSp45611e*RC3456pe - 1./
         24.*m16*RSp45613e + 1./48.*m16*RSp45633e + 1./48.*m16*
         RSp45611e - 1./24.*m15*RSp45613e + 1./48.*m15*RSp45633e + 1./
         48.*m15*RSp45611e - 1./24.*m14*RSp45613e + 1./48.*m14*
         RSp45633e + 1./48.*m14*RSp45611e - 1./36.*m13*RSp45613e*
         RS34561pe - 1./36.*m13*RSp45613e*RS14563pe - 1./36.*m13*
         RSp45613e*RSe45613p - 1./36.*m13*pow(RSp45613e,2) + 1./72.*m13
         *RC3456pe*RSp45613e + 1./72.*m13*RSe45633p*RSp45613e + 1./72.*
         m13*RSp45633e*RS34561pe + 1./72.*m13*RSp45633e*RS14563pe + 1./
         72.*m13*RSp45633e*RSe45613p );

      F13p6x23RV +=  + AbV(mp2,L)*pi16 * ( 1./36.*m13*RSp45633e*
         RSp45613e - 1./144.*m13*RSp45633e*RC3456pe - 1./144.*m13*
         RSp45633e*RSe45633p - 1./144.*m13*pow(RSp45633e,2) + 1./72.*
         m13*RC1456pe*RSp45613e - 1./144.*m13*RC1456pe*RSp45633e + 1./
         72.*m13*RSe45611p*RSp45613e - 1./144.*m13*RSe45611p*RSp45633e
          + 1./72.*m13*RSp45611e*RS34561pe + 1./72.*m13*RSp45611e*
         RS14563pe + 1./72.*m13*RSp45611e*RSe45613p + 1./36.*m13*
         RSp45611e*RSp45613e - 1./144.*m13*RSp45611e*RC3456pe - 1./144.
         *m13*RSp45611e*RSe45633p - 1./72.*m13*RSp45611e*RSp45633e - 1./
         144.*m13*RSp45611e*RC1456pe - 1./144.*m13*RSp45611e*RSe45611p
          - 1./144.*m13*pow(RSp45611e,2) - 1./36.*m11*RSp45613e*
         RS14563pe + 1./72.*m11*RSp45633e*RS14563pe + 1./72.*m11*
         RC1456pe*RSp45613e - 1./144.*m11*RC1456pe*RSp45633e + 1./72.*
         m11*RSp45611e*RS14563pe - 1./144.*m11*RSp45611e*RC1456pe );

      F13p6x23RV +=  + pow(AbV(mp2,L),2) * ( 1./72.*pow(RSp45613e,2) - 
         1./72.*RSp45633e*RSp45613e + 1./288.*pow(RSp45633e,2) - 1./72.
         *RSp45611e*RSp45613e + 1./144.*RSp45611e*RSp45633e + 1./288.*
         pow(RSp45611e,2) );

      F13p6x23RV +=  + AbV(mp2,L)*AbV(me2,L) * ( 1./36.*RSp45613e*
         RSe45613p - 1./72.*RSe45633p*RSp45613e - 1./72.*RSp45633e*
         RSe45613p + 1./144.*RSp45633e*RSe45633p - 1./72.*RSe45611p*
         RSp45613e + 1./144.*RSe45611p*RSp45633e - 1./72.*RSp45611e*
         RSe45613p + 1./144.*RSp45611e*RSe45633p + 1./144.*RSp45611e*
         RSe45611p );

      F13p6x23RV +=  + AbV(mp2,L)*BbV(m11,m33,L) * ( 1./54.*m66*RSp456e
         *RS145pe*RS345pe + 1./54.*m55*RSp465e*RS146pe*RS346pe + 1./54.
         *m44*RSp564e*RS156pe*RS356pe - 1./18.*m33*RSp45633e*RS14563pe
          - 1./18.*m11*RSp45611e*RS34561pe );

      F13p6x23RV +=  + AbV(mp2,L)*BbV(m11,mp2,L) * ( 1./54.*m66*RSp456e
         *RS145pe*RSp453e - 1./54.*m66*RSp456e*RSp451e*RS145pe + 1./54.
         *m55*RSp465e*RS146pe*RSp463e - 1./54.*m55*RSp465e*RSp461e*
         RS146pe + 1./54.*m44*RSp564e*RS156pe*RSp563e - 1./54.*m44*
         RSp564e*RSp561e*RS156pe - 1./18.*m11*RSp45611e*RSp45613e + 1./
         18.*m11*pow(RSp45611e,2) );

      F13p6x23RV +=  + AbV(mp2,L)*BbV(m11,me2,L) * ( 1./54.*m66*RSp456e
         *RS145pe*RSe453p - 1./54.*m66*RSp456e*RSe451p*RS145pe + 1./54.
         *m55*RSp465e*RS146pe*RSe463p - 1./54.*m55*RSp465e*RSe461p*
         RS146pe + 1./54.*m44*RSp564e*RS156pe*RSe563p - 1./54.*m44*
         RSp564e*RSe561p*RS156pe - 1./18.*m11*RSp45611e*RSe45613p + 1./
         18.*m11*RSp45611e*RSe45611p );

      F13p6x23RV +=  + AbV(mp2,L)*BbV(m11,L) * (  - 1./72.*RD1456ep*
         RSp45613e + 1./144.*RD1456ep*RSp45633e + 1./144.*RSp45611e*
         RD1456ep - 1./108.*m66*RSp456e*pow(RS145pe,2) - 1./108.*m55*
         RSp465e*pow(RS146pe,2) - 1./108.*m44*RSp564e*pow(RS156pe,2) - 
         1./18.*m11*RSp45611e*RS14563pe + 1./18.*m11*RSp45611e*RC1456pe
          );

      F13p6x23RV +=  + AbV(mp2,L)*BbV(m14,L) * ( 1./72.*mp2*RSp45611e
          - 1./36.*mp2*RSp561e + 1./72.*mp2*RSp564e - 1./144.*m44*
         RSp45611e - 1./72.*m44*RSp561e - 1./48.*m44*RSp564e + 1./72.*
         m14*RSp45611e - 1./36.*m14*RSp561e + 1./72.*m14*RSp564e - 1./
         48.*m11*RSp45611e - 1./72.*m11*RSp561e - 1./144.*m11*RSp564e )
         ;

      F13p6x23RV +=  + AbV(mp2,L)*BbV(m15,L) * ( 1./72.*mp2*RSp45611e
          - 1./36.*mp2*RSp461e + 1./72.*mp2*RSp465e - 1./144.*m55*
         RSp45611e - 1./72.*m55*RSp461e - 1./48.*m55*RSp465e + 1./72.*
         m15*RSp45611e - 1./36.*m15*RSp461e + 1./72.*m15*RSp465e - 1./
         48.*m11*RSp45611e - 1./72.*m11*RSp461e - 1./144.*m11*RSp465e )
         ;

      F13p6x23RV +=  + AbV(mp2,L)*BbV(m16,L) * ( 1./72.*mp2*RSp45611e
          - 1./36.*mp2*RSp451e + 1./72.*mp2*RSp456e - 1./144.*m66*
         RSp45611e - 1./72.*m66*RSp451e - 1./48.*m66*RSp456e + 1./72.*
         m16*RSp45611e - 1./36.*m16*RSp451e + 1./72.*m16*RSp456e - 1./
         48.*m11*RSp45611e - 1./72.*m11*RSp451e - 1./144.*m11*RSp456e )
         ;

      F13p6x23RV +=  + AbV(mp2,L)*BbV(m33,mp2,L) * (  - 1./54.*m66*
         RSp456e*RSp453e*RS345pe + 1./54.*m66*RSp456e*RSp451e*RS345pe
          - 1./54.*m55*RSp465e*RSp463e*RS346pe + 1./54.*m55*RSp465e*
         RSp461e*RS346pe - 1./54.*m44*RSp564e*RSp563e*RS356pe + 1./54.*
         m44*RSp564e*RSp561e*RS356pe - 1./18.*m33*RSp45633e*RSp45613e
          + 1./18.*m33*pow(RSp45633e,2) );

      F13p6x23RV +=  + AbV(mp2,L)*BbV(m33,me2,L) * (  - 1./54.*m66*
         RSp456e*RSe453p*RS345pe + 1./54.*m66*RSp456e*RSe451p*RS345pe
          - 1./54.*m55*RSp465e*RSe463p*RS346pe + 1./54.*m55*RSp465e*
         RSe461p*RS346pe - 1./54.*m44*RSp564e*RSe563p*RS356pe + 1./54.*
         m44*RSp564e*RSe561p*RS356pe - 1./18.*m33*RSp45633e*RSe45613p
          + 1./18.*m33*RSp45633e*RSe45633p );

      F13p6x23RV +=  + AbV(mp2,L)*BbV(m33,L) * (  - 1./72.*RD3456ep*
         RSp45613e + 1./144.*RSp45633e*RD3456ep + 1./144.*RSp45611e*
         RD3456ep - 1./108.*m66*RSp456e*pow(RS345pe,2) - 1./108.*m55*
         RSp465e*pow(RS346pe,2) - 1./108.*m44*RSp564e*pow(RS356pe,2) - 
         1./18.*m33*RSp45633e*RS34561pe + 1./18.*m33*RSp45633e*RC3456pe
          );

      F13p6x23RV +=  + AbV(mp2,L)*BbV(m34,L) * ( 1./72.*mp2*RSp45633e
          - 1./36.*mp2*RSp563e + 1./72.*mp2*RSp564e - 1./144.*m44*
         RSp45633e - 1./72.*m44*RSp563e - 1./48.*m44*RSp564e + 1./72.*
         m34*RSp45633e - 1./36.*m34*RSp563e + 1./72.*m34*RSp564e - 1./
         48.*m33*RSp45633e - 1./72.*m33*RSp563e - 1./144.*m33*RSp564e )
         ;

      F13p6x23RV +=  + AbV(mp2,L)*BbV(m35,L) * ( 1./72.*mp2*RSp45633e
          - 1./36.*mp2*RSp463e + 1./72.*mp2*RSp465e - 1./144.*m55*
         RSp45633e - 1./72.*m55*RSp463e - 1./48.*m55*RSp465e + 1./72.*
         m35*RSp45633e - 1./36.*m35*RSp463e + 1./72.*m35*RSp465e - 1./
         48.*m33*RSp45633e - 1./72.*m33*RSp463e - 1./144.*m33*RSp465e )
         ;

      F13p6x23RV +=  + AbV(mp2,L)*BbV(m36,L) * ( 1./72.*mp2*RSp45633e
          - 1./36.*mp2*RSp453e + 1./72.*mp2*RSp456e - 1./144.*m66*
         RSp45633e - 1./72.*m66*RSp453e - 1./48.*m66*RSp456e + 1./72.*
         m36*RSp45633e - 1./36.*m36*RSp453e + 1./72.*m36*RSp456e - 1./
         48.*m33*RSp45633e - 1./72.*m33*RSp453e - 1./144.*m33*RSp456e )
         ;

      F13p6x23RV +=  + AbV(mp2,L)*BbV(mp2,me2,L) * (  - 2./81.*m66*
         RSp456e*RSp453e*RSe453p + 2./81.*m66*RSp456e*RSe451p*RSp453e
          + 2./81.*m66*RSp456e*RSp451e*RSe453p - 2./81.*m66*RSp456e*
         RSp451e*RSe451p - 2./81.*m55*RSp465e*RSp463e*RSe463p + 2./81.*
         m55*RSp465e*RSe461p*RSp463e + 2./81.*m55*RSp465e*RSp461e*
         RSe463p - 2./81.*m55*RSp465e*RSp461e*RSe461p - 2./81.*m44*
         RSp564e*RSp563e*RSe563p + 2./81.*m44*RSp564e*RSe561p*RSp563e
          + 2./81.*m44*RSp564e*RSp561e*RSe563p - 2./81.*m44*RSp564e*
         RSp561e*RSe561p );

      F13p6x23RV +=  + AbV(mp2,L)*BbV(mp2,L) * (  - 1./108.*m66*RSp456e
         *pow(RSp453e,2) + 1./54.*m66*RSp456e*RSp451e*RSp453e - 1./108.
         *m66*RSp456e*pow(RSp451e,2) - 1./108.*m55*RSp465e*pow(
         RSp463e,2) + 1./54.*m55*RSp465e*RSp461e*RSp463e - 1./108.*m55*
         RSp465e*pow(RSp461e,2) - 1./108.*m44*RSp564e*pow(RSp563e,2) + 
         1./54.*m44*RSp564e*RSp561e*RSp563e - 1./108.*m44*RSp564e*pow(
         RSp561e,2) );

      F13p6x23RV +=  + AbV(mp2,L)*BbV(me2,mp2,L) * ( 1./162.*m66*
         RSp456e*RSp453e*RSe453p - 1./162.*m66*RSp456e*RSe451p*RSp453e
          - 1./162.*m66*RSp456e*RSp451e*RSe453p + 1./162.*m66*RSp456e*
         RSp451e*RSe451p + 1./162.*m55*RSp465e*RSp463e*RSe463p - 1./162.
         *m55*RSp465e*RSe461p*RSp463e - 1./162.*m55*RSp465e*RSp461e*
         RSe463p + 1./162.*m55*RSp465e*RSp461e*RSe461p + 1./162.*m44*
         RSp564e*RSp563e*RSe563p - 1./162.*m44*RSp564e*RSe561p*RSp563e
          - 1./162.*m44*RSp564e*RSp561e*RSe563p + 1./162.*m44*RSp564e*
         RSp561e*RSe561p );

      F13p6x23RV +=  + AbV(mp2,L)*BbV(me2,L) * (  - 1./108.*m66*RSp456e
         *pow(RSe453p,2) + 1./54.*m66*RSp456e*RSe451p*RSe453p - 1./108.
         *m66*RSp456e*pow(RSe451p,2) - 1./108.*m55*RSp465e*pow(
         RSe463p,2) + 1./54.*m55*RSp465e*RSe461p*RSe463p - 1./108.*m55*
         RSp465e*pow(RSe461p,2) - 1./108.*m44*RSp564e*pow(RSe563p,2) + 
         1./54.*m44*RSp564e*RSe561p*RSe563p - 1./108.*m44*RSp564e*pow(
         RSe561p,2) );

      F13p6x23RV +=  + AbV(mp2,L)*CbV(m11,L) * ( 1./18.*m11*RSp45611e*
         RD1456ep );

      F13p6x23RV +=  + AbV(mp2,L)*CbV(m33,L) * ( 1./18.*m33*RSp45633e*
         RD3456ep );

      F13p6x23RV +=  + AbV(me2,L)*pi16 * ( 1./72.*RD3456ep*RSe45613p - 
         1./144.*RSe45633p*RD3456ep + 1./72.*RD1456ep*RSe45613p - 1./
         144.*RD1456ep*RSe45633p - 1./144.*RSe45611p*RD3456ep - 1./144.
         *RSe45611p*RD1456ep + 1./24.*me2*RSe45613p - 1./36.*me2*pow(
         RSe45613p,2) - 1./48.*me2*RSe45633p + 1./36.*me2*RSe45633p*
         RSe45613p - 1./144.*me2*pow(RSe45633p,2) - 1./48.*me2*RSe563p
          - 1./48.*me2*RSe463p - 1./48.*me2*RSe453p - 1./48.*me2*
         RSe45611p + 1./36.*me2*RSe45611p*RSe45613p - 1./72.*me2*
         RSe45611p*RSe45633p - 1./144.*me2*pow(RSe45611p,2) - 1./48.*
         me2*RSe561p - 1./48.*me2*RSe461p - 1./48.*me2*RSe451p - 1./36.
         *mp2*RSp45613e*RSe45613p + 1./72.*mp2*RSe45633p*RSp45613e + 1./
         72.*mp2*RSp45633e*RSe45613p - 1./144.*mp2*RSp45633e*RSe45633p
          + 1./72.*mp2*RSe45611p*RSp45613e - 1./144.*mp2*RSe45611p*
         RSp45633e + 1./72.*mp2*RSp45611e*RSe45613p - 1./144.*mp2*
         RSp45611e*RSe45633p - 1./144.*mp2*RSp45611e*RSe45611p - 1./24.
         *m36*RSe45613p );

      F13p6x23RV +=  + AbV(me2,L)*pi16 * ( 1./48.*m36*RSe45633p + 1./48.
         *m36*RSe45611p - 1./24.*m35*RSe45613p + 1./48.*m35*RSe45633p
          + 1./48.*m35*RSe45611p - 1./24.*m34*RSe45613p + 1./48.*m34*
         RSe45633p + 1./48.*m34*RSe45611p - 1./36.*m33*RSe45613p*
         RS34561pe + 1./72.*m33*RC3456pe*RSe45613p + 1./72.*m33*
         RSe45633p*RS34561pe - 1./144.*m33*RSe45633p*RC3456pe + 1./72.*
         m33*RSe45611p*RS34561pe - 1./144.*m33*RSe45611p*RC3456pe - 1./
         24.*m16*RSe45613p + 1./48.*m16*RSe45633p + 1./48.*m16*
         RSe45611p - 1./24.*m15*RSe45613p + 1./48.*m15*RSe45633p + 1./
         48.*m15*RSe45611p - 1./24.*m14*RSe45613p + 1./48.*m14*
         RSe45633p + 1./48.*m14*RSe45611p - 1./36.*m13*RSe45613p*
         RS34561pe - 1./36.*m13*RSe45613p*RS14563pe - 1./36.*m13*pow(
         RSe45613p,2) - 1./36.*m13*RSp45613e*RSe45613p + 1./72.*m13*
         RC3456pe*RSe45613p + 1./72.*m13*RSe45633p*RS34561pe + 1./72.*
         m13*RSe45633p*RS14563pe + 1./36.*m13*RSe45633p*RSe45613p + 1./
         72.*m13*RSe45633p*RSp45613e );

      F13p6x23RV +=  + AbV(me2,L)*pi16 * (  - 1./144.*m13*RSe45633p*
         RC3456pe - 1./144.*m13*pow(RSe45633p,2) + 1./72.*m13*RSp45633e
         *RSe45613p - 1./144.*m13*RSp45633e*RSe45633p + 1./72.*m13*
         RC1456pe*RSe45613p - 1./144.*m13*RC1456pe*RSe45633p + 1./72.*
         m13*RSe45611p*RS34561pe + 1./72.*m13*RSe45611p*RS14563pe + 1./
         36.*m13*RSe45611p*RSe45613p + 1./72.*m13*RSe45611p*RSp45613e
          - 1./144.*m13*RSe45611p*RC3456pe - 1./72.*m13*RSe45611p*
         RSe45633p - 1./144.*m13*RSe45611p*RSp45633e - 1./144.*m13*
         RSe45611p*RC1456pe - 1./144.*m13*pow(RSe45611p,2) + 1./72.*m13
         *RSp45611e*RSe45613p - 1./144.*m13*RSp45611e*RSe45633p - 1./
         144.*m13*RSp45611e*RSe45611p - 1./36.*m11*RSe45613p*RS14563pe
          + 1./72.*m11*RSe45633p*RS14563pe + 1./72.*m11*RC1456pe*
         RSe45613p - 1./144.*m11*RC1456pe*RSe45633p + 1./72.*m11*
         RSe45611p*RS14563pe - 1./144.*m11*RSe45611p*RC1456pe );

      F13p6x23RV +=  + pow(AbV(me2,L),2) * ( 1./72.*pow(RSe45613p,2) - 
         1./72.*RSe45633p*RSe45613p + 1./288.*pow(RSe45633p,2) - 1./72.
         *RSe45611p*RSe45613p + 1./144.*RSe45611p*RSe45633p + 1./288.*
         pow(RSe45611p,2) );

      F13p6x23RV +=  + AbV(me2,L)*BbV(m11,m33,L) * ( 1./54.*m66*RSe456p
         *RS145pe*RS345pe + 1./54.*m55*RSe465p*RS146pe*RS346pe + 1./54.
         *m44*RSe564p*RS156pe*RS356pe - 1./18.*m33*RSe45633p*RS14563pe
          - 1./18.*m11*RSe45611p*RS34561pe );

      F13p6x23RV +=  + AbV(me2,L)*BbV(m11,mp2,L) * ( 1./54.*m66*RSe456p
         *RS145pe*RSp453e - 1./54.*m66*RSe456p*RSp451e*RS145pe + 1./54.
         *m55*RSe465p*RS146pe*RSp463e - 1./54.*m55*RSe465p*RSp461e*
         RS146pe + 1./54.*m44*RSe564p*RS156pe*RSp563e - 1./54.*m44*
         RSe564p*RSp561e*RS156pe - 1./18.*m11*RSe45611p*RSp45613e + 1./
         18.*m11*RSp45611e*RSe45611p );

      F13p6x23RV +=  + AbV(me2,L)*BbV(m11,me2,L) * ( 1./54.*m66*RSe456p
         *RS145pe*RSe453p - 1./54.*m66*RSe456p*RSe451p*RS145pe + 1./54.
         *m55*RSe465p*RS146pe*RSe463p - 1./54.*m55*RSe465p*RSe461p*
         RS146pe + 1./54.*m44*RSe564p*RS156pe*RSe563p - 1./54.*m44*
         RSe564p*RSe561p*RS156pe - 1./18.*m11*RSe45611p*RSe45613p + 1./
         18.*m11*pow(RSe45611p,2) );

      F13p6x23RV +=  + AbV(me2,L)*BbV(m11,L) * (  - 1./72.*RD1456ep*
         RSe45613p + 1./144.*RD1456ep*RSe45633p + 1./144.*RSe45611p*
         RD1456ep - 1./108.*m66*RSe456p*pow(RS145pe,2) - 1./108.*m55*
         RSe465p*pow(RS146pe,2) - 1./108.*m44*RSe564p*pow(RS156pe,2) - 
         1./18.*m11*RSe45611p*RS14563pe + 1./18.*m11*RSe45611p*RC1456pe
          );

      F13p6x23RV +=  + AbV(me2,L)*BbV(m14,L) * ( 1./72.*me2*RSe45611p
          - 1./36.*me2*RSe561p + 1./72.*me2*RSe564p - 1./144.*m44*
         RSe45611p - 1./72.*m44*RSe561p - 1./48.*m44*RSe564p + 1./72.*
         m14*RSe45611p - 1./36.*m14*RSe561p + 1./72.*m14*RSe564p - 1./
         48.*m11*RSe45611p - 1./72.*m11*RSe561p - 1./144.*m11*RSe564p )
         ;

      F13p6x23RV +=  + AbV(me2,L)*BbV(m15,L) * ( 1./72.*me2*RSe45611p
          - 1./36.*me2*RSe461p + 1./72.*me2*RSe465p - 1./144.*m55*
         RSe45611p - 1./72.*m55*RSe461p - 1./48.*m55*RSe465p + 1./72.*
         m15*RSe45611p - 1./36.*m15*RSe461p + 1./72.*m15*RSe465p - 1./
         48.*m11*RSe45611p - 1./72.*m11*RSe461p - 1./144.*m11*RSe465p )
         ;

      F13p6x23RV +=  + AbV(me2,L)*BbV(m16,L) * ( 1./72.*me2*RSe45611p
          - 1./36.*me2*RSe451p + 1./72.*me2*RSe456p - 1./144.*m66*
         RSe45611p - 1./72.*m66*RSe451p - 1./48.*m66*RSe456p + 1./72.*
         m16*RSe45611p - 1./36.*m16*RSe451p + 1./72.*m16*RSe456p - 1./
         48.*m11*RSe45611p - 1./72.*m11*RSe451p - 1./144.*m11*RSe456p )
         ;

      F13p6x23RV +=  + AbV(me2,L)*BbV(m33,mp2,L) * (  - 1./54.*m66*
         RSe456p*RSp453e*RS345pe + 1./54.*m66*RSe456p*RSp451e*RS345pe
          - 1./54.*m55*RSe465p*RSp463e*RS346pe + 1./54.*m55*RSe465p*
         RSp461e*RS346pe - 1./54.*m44*RSe564p*RSp563e*RS356pe + 1./54.*
         m44*RSe564p*RSp561e*RS356pe - 1./18.*m33*RSe45633p*RSp45613e
          + 1./18.*m33*RSp45633e*RSe45633p );

      F13p6x23RV +=  + AbV(me2,L)*BbV(m33,me2,L) * (  - 1./54.*m66*
         RSe456p*RSe453p*RS345pe + 1./54.*m66*RSe456p*RSe451p*RS345pe
          - 1./54.*m55*RSe465p*RSe463p*RS346pe + 1./54.*m55*RSe465p*
         RSe461p*RS346pe - 1./54.*m44*RSe564p*RSe563p*RS356pe + 1./54.*
         m44*RSe564p*RSe561p*RS356pe - 1./18.*m33*RSe45633p*RSe45613p
          + 1./18.*m33*pow(RSe45633p,2) );

      F13p6x23RV +=  + AbV(me2,L)*BbV(m33,L) * (  - 1./72.*RD3456ep*
         RSe45613p + 1./144.*RSe45633p*RD3456ep + 1./144.*RSe45611p*
         RD3456ep - 1./108.*m66*RSe456p*pow(RS345pe,2) - 1./108.*m55*
         RSe465p*pow(RS346pe,2) - 1./108.*m44*RSe564p*pow(RS356pe,2) - 
         1./18.*m33*RSe45633p*RS34561pe + 1./18.*m33*RSe45633p*RC3456pe
          );

      F13p6x23RV +=  + AbV(me2,L)*BbV(m34,L) * ( 1./72.*me2*RSe45633p
          - 1./36.*me2*RSe563p + 1./72.*me2*RSe564p - 1./144.*m44*
         RSe45633p - 1./72.*m44*RSe563p - 1./48.*m44*RSe564p + 1./72.*
         m34*RSe45633p - 1./36.*m34*RSe563p + 1./72.*m34*RSe564p - 1./
         48.*m33*RSe45633p - 1./72.*m33*RSe563p - 1./144.*m33*RSe564p )
         ;

      F13p6x23RV +=  + AbV(me2,L)*BbV(m35,L) * ( 1./72.*me2*RSe45633p
          - 1./36.*me2*RSe463p + 1./72.*me2*RSe465p - 1./144.*m55*
         RSe45633p - 1./72.*m55*RSe463p - 1./48.*m55*RSe465p + 1./72.*
         m35*RSe45633p - 1./36.*m35*RSe463p + 1./72.*m35*RSe465p - 1./
         48.*m33*RSe45633p - 1./72.*m33*RSe463p - 1./144.*m33*RSe465p )
         ;

      F13p6x23RV +=  + AbV(me2,L)*BbV(m36,L) * ( 1./72.*me2*RSe45633p
          - 1./36.*me2*RSe453p + 1./72.*me2*RSe456p - 1./144.*m66*
         RSe45633p - 1./72.*m66*RSe453p - 1./48.*m66*RSe456p + 1./72.*
         m36*RSe45633p - 1./36.*m36*RSe453p + 1./72.*m36*RSe456p - 1./
         48.*m33*RSe45633p - 1./72.*m33*RSe453p - 1./144.*m33*RSe456p )
         ;

      F13p6x23RV +=  + AbV(me2,L)*BbV(mp2,me2,L) * (  - 2./81.*m66*
         RSe456p*RSp453e*RSe453p + 2./81.*m66*RSe456p*RSe451p*RSp453e
          + 2./81.*m66*RSe456p*RSp451e*RSe453p - 2./81.*m66*RSe456p*
         RSp451e*RSe451p - 2./81.*m55*RSe465p*RSp463e*RSe463p + 2./81.*
         m55*RSe465p*RSe461p*RSp463e + 2./81.*m55*RSe465p*RSp461e*
         RSe463p - 2./81.*m55*RSe465p*RSp461e*RSe461p - 2./81.*m44*
         RSe564p*RSp563e*RSe563p + 2./81.*m44*RSe564p*RSe561p*RSp563e
          + 2./81.*m44*RSe564p*RSp561e*RSe563p - 2./81.*m44*RSe564p*
         RSp561e*RSe561p );

      F13p6x23RV +=  + AbV(me2,L)*BbV(mp2,L) * (  - 1./108.*m66*RSe456p
         *pow(RSp453e,2) + 1./54.*m66*RSe456p*RSp451e*RSp453e - 1./108.
         *m66*RSe456p*pow(RSp451e,2) - 1./108.*m55*RSe465p*pow(
         RSp463e,2) + 1./54.*m55*RSe465p*RSp461e*RSp463e - 1./108.*m55*
         RSe465p*pow(RSp461e,2) - 1./108.*m44*RSe564p*pow(RSp563e,2) + 
         1./54.*m44*RSe564p*RSp561e*RSp563e - 1./108.*m44*RSe564p*pow(
         RSp561e,2) );

      F13p6x23RV +=  + AbV(me2,L)*BbV(me2,mp2,L) * ( 1./162.*m66*
         RSe456p*RSp453e*RSe453p - 1./162.*m66*RSe456p*RSe451p*RSp453e
          - 1./162.*m66*RSe456p*RSp451e*RSe453p + 1./162.*m66*RSe456p*
         RSp451e*RSe451p + 1./162.*m55*RSe465p*RSp463e*RSe463p - 1./162.
         *m55*RSe465p*RSe461p*RSp463e - 1./162.*m55*RSe465p*RSp461e*
         RSe463p + 1./162.*m55*RSe465p*RSp461e*RSe461p + 1./162.*m44*
         RSe564p*RSp563e*RSe563p - 1./162.*m44*RSe564p*RSe561p*RSp563e
          - 1./162.*m44*RSe564p*RSp561e*RSe563p + 1./162.*m44*RSe564p*
         RSp561e*RSe561p );

      F13p6x23RV +=  + AbV(me2,L)*BbV(me2,L) * (  - 1./108.*m66*RSe456p
         *pow(RSe453p,2) + 1./54.*m66*RSe456p*RSe451p*RSe453p - 1./108.
         *m66*RSe456p*pow(RSe451p,2) - 1./108.*m55*RSe465p*pow(
         RSe463p,2) + 1./54.*m55*RSe465p*RSe461p*RSe463p - 1./108.*m55*
         RSe465p*pow(RSe461p,2) - 1./108.*m44*RSe564p*pow(RSe563p,2) + 
         1./54.*m44*RSe564p*RSe561p*RSe563p - 1./108.*m44*RSe564p*pow(
         RSe561p,2) );

      F13p6x23RV +=  + AbV(me2,L)*CbV(m11,L) * ( 1./18.*m11*RSe45611p*
         RD1456ep );

      F13p6x23RV +=  + AbV(me2,L)*CbV(m33,L) * ( 1./18.*m33*RSe45633p*
         RD3456ep );

      F13p6x23RV +=  + BbV(m11,m33,L)*BbV(m11,L) * (  - 1./18.*m11*
         RD1456ep*RS34561pe );

      F13p6x23RV +=  + BbV(m11,m33,L)*BbV(m33,L) * (  - 1./18.*m33*
         RD3456ep*RS14563pe );

      F13p6x23RV +=  + BbV(m11,mp2,L)*BbV(m11,L) * (  - 1./18.*m11*
         RD1456ep*RSp45613e + 1./18.*m11*RSp45611e*RD1456ep );

      F13p6x23RV +=  + BbV(m11,me2,L)*BbV(m11,L) * (  - 1./18.*m11*
         RD1456ep*RSe45613p + 1./18.*m11*RSe45611p*RD1456ep );

      F13p6x23RV +=  + BbV(m11,L)*pi16 * (  - 1./144.*RD1456ep*RD3456ep
          - 1./144.*pow(RD1456ep,2) + 1./72.*me2*RD1456ep*RSe45613p - 1.
         /144.*me2*RD1456ep*RSe45633p - 1./144.*me2*RSe45611p*RD1456ep
          + 1./72.*mp2*RD1456ep*RSp45613e - 1./144.*mp2*RD1456ep*
         RSp45633e - 1./144.*mp2*RSp45611e*RD1456ep + 1./48.*m36*
         RD1456ep + 1./48.*m35*RD1456ep + 1./48.*m34*RD1456ep + 1./72.*
         m33*RD1456ep*RS34561pe - 1./144.*m33*RD1456ep*RC3456pe + 1./48.
         *m16*RD1456ep + 1./48.*m15*RD1456ep + 1./48.*m14*RD1456ep + 1./
         72.*m13*RD1456ep*RS34561pe + 1./72.*m13*RD1456ep*RS14563pe + 1.
         /72.*m13*RD1456ep*RSe45613p + 1./72.*m13*RD1456ep*RSp45613e - 
         1./144.*m13*RD1456ep*RC3456pe - 1./144.*m13*RD1456ep*RSe45633p
          - 1./144.*m13*RD1456ep*RSp45633e - 1./144.*m13*RD1456ep*
         RC1456pe - 1./144.*m13*RSe45611p*RD1456ep - 1./144.*m13*
         RSp45611e*RD1456ep - 1./48.*m11*RD1456ep + 1./72.*m11*RD1456ep
         *RS14563pe - 1./144.*m11*RD1456ep*RC1456pe );

      F13p6x23RV +=  + pow(BbV(m11,L),2) * ( 1./288.*pow(RD1456ep,2) - 
         1./18.*m11*RD1456ep*RS14563pe + 1./18.*m11*RD1456ep*RC1456pe )
         ;

      F13p6x23RV +=  + BbV(m11,L)*BbV(m14,L) * (  - 1./144.*m44*
         RD1456ep + 1./72.*m14*RD1456ep - 1./144.*m11*RD1456ep );

      F13p6x23RV +=  + BbV(m11,L)*BbV(m15,L) * (  - 1./144.*m55*
         RD1456ep + 1./72.*m15*RD1456ep - 1./144.*m11*RD1456ep );

      F13p6x23RV +=  + BbV(m11,L)*BbV(m16,L) * (  - 1./144.*m66*
         RD1456ep + 1./72.*m16*RD1456ep - 1./144.*m11*RD1456ep );

      F13p6x23RV +=  + BbV(m11,L)*BbV(m33,L) * ( 1./144.*RD1456ep*
         RD3456ep );

      F13p6x23RV +=  + BbV(m11,L)*CbV(m11,L) * ( 1./18.*m11*pow(
         RD1456ep,2) );

      F13p6x23RV +=  + BbV(m33,mp2,L)*BbV(m33,L) * (  - 1./18.*m33*
         RD3456ep*RSp45613e + 1./18.*m33*RSp45633e*RD3456ep );

      F13p6x23RV +=  + BbV(m33,me2,L)*BbV(m33,L) * (  - 1./18.*m33*
         RD3456ep*RSe45613p + 1./18.*m33*RSe45633p*RD3456ep );

      F13p6x23RV +=  + BbV(m33,L)*pi16 * (  - 1./144.*pow(RD3456ep,2)
          - 1./144.*RD1456ep*RD3456ep + 1./72.*me2*RD3456ep*RSe45613p
          - 1./144.*me2*RSe45633p*RD3456ep - 1./144.*me2*RSe45611p*
         RD3456ep + 1./72.*mp2*RD3456ep*RSp45613e - 1./144.*mp2*
         RSp45633e*RD3456ep - 1./144.*mp2*RSp45611e*RD3456ep + 1./48.*
         m36*RD3456ep + 1./48.*m35*RD3456ep + 1./48.*m34*RD3456ep - 1./
         48.*m33*RD3456ep + 1./72.*m33*RD3456ep*RS34561pe - 1./144.*m33
         *RD3456ep*RC3456pe + 1./48.*m16*RD3456ep + 1./48.*m15*RD3456ep
          + 1./48.*m14*RD3456ep + 1./72.*m13*RD3456ep*RS34561pe + 1./72.
         *m13*RD3456ep*RS14563pe + 1./72.*m13*RD3456ep*RSe45613p + 1./
         72.*m13*RD3456ep*RSp45613e - 1./144.*m13*RD3456ep*RC3456pe - 1.
         /144.*m13*RSe45633p*RD3456ep - 1./144.*m13*RSp45633e*RD3456ep
          - 1./144.*m13*RC1456pe*RD3456ep - 1./144.*m13*RSe45611p*
         RD3456ep - 1./144.*m13*RSp45611e*RD3456ep + 1./72.*m11*
         RD3456ep*RS14563pe - 1./144.*m11*RC1456pe*RD3456ep );

      F13p6x23RV +=  + pow(BbV(m33,L),2) * ( 1./288.*pow(RD3456ep,2) - 
         1./18.*m33*RD3456ep*RS34561pe + 1./18.*m33*RD3456ep*RC3456pe )
         ;

      F13p6x23RV +=  + BbV(m33,L)*BbV(m34,L) * (  - 1./144.*m44*
         RD3456ep + 1./72.*m34*RD3456ep - 1./144.*m33*RD3456ep );

      F13p6x23RV +=  + BbV(m33,L)*BbV(m35,L) * (  - 1./144.*m55*
         RD3456ep + 1./72.*m35*RD3456ep - 1./144.*m33*RD3456ep );

      F13p6x23RV +=  + BbV(m33,L)*BbV(m36,L) * (  - 1./144.*m66*
         RD3456ep + 1./72.*m36*RD3456ep - 1./144.*m33*RD3456ep );

      F13p6x23RV +=  + BbV(m33,L)*CbV(m33,L) * ( 1./18.*m33*pow(
         RD3456ep,2) );

      F13p6x23RV +=  + A23bV(m11,L)*pi16 * (  - 3./16. + 1./36.*
         RS14563pe*RS34561pe + 1./36.*pow(RS14563pe,2) + 1./36.*
         RSe45613p*RS14563pe + 1./36.*RSp45613e*RS14563pe - 1./72.*
         RC3456pe*RS14563pe - 1./72.*RSe45633p*RS14563pe - 1./72.*
         RSp45633e*RS14563pe - 1./72.*RC1456pe*RS34561pe - 1./36.*
         RC1456pe*RS14563pe - 1./72.*RC1456pe*RSe45613p - 1./72.*
         RC1456pe*RSp45613e + 1./144.*RC1456pe*RC3456pe + 1./144.*
         RC1456pe*RSe45633p + 1./144.*RC1456pe*RSp45633e + 1./144.*pow(
         RC1456pe,2) - 1./72.*RSe45611p*RS14563pe + 1./144.*RSe45611p*
         RC1456pe - 1./72.*RSp45611e*RS14563pe + 1./144.*RSp45611e*
         RC1456pe + 1./16.*RS145pe + 1./16.*RS146pe + 1./16.*RS156pe );

      F13p6x23RV +=  + A23bV(m13,L)*pi16 * ( 1./24.*pow(RS34561pe,2) + 
         1./12.*RS14563pe*RS34561pe + 1./24.*pow(RS14563pe,2) + 1./12.*
         RSe45613p*RS34561pe + 1./12.*RSe45613p*RS14563pe + 1./24.*pow(
         RSe45613p,2) + 1./12.*RSp45613e*RS34561pe + 1./12.*RSp45613e*
         RS14563pe + 1./12.*RSp45613e*RSe45613p + 1./24.*pow(
         RSp45613e,2) - 1./24.*RC3456pe*RS34561pe - 1./24.*RC3456pe*
         RS14563pe - 1./24.*RC3456pe*RSe45613p - 1./24.*RC3456pe*
         RSp45613e + 1./96.*pow(RC3456pe,2) - 1./24.*RSe45633p*
         RS34561pe - 1./24.*RSe45633p*RS14563pe - 1./24.*RSe45633p*
         RSe45613p - 1./24.*RSe45633p*RSp45613e + 1./48.*RSe45633p*
         RC3456pe + 1./96.*pow(RSe45633p,2) - 1./24.*RSp45633e*
         RS34561pe - 1./24.*RSp45633e*RS14563pe - 1./24.*RSp45633e*
         RSe45613p - 1./24.*RSp45633e*RSp45613e + 1./48.*RSp45633e*
         RC3456pe + 1./48.*RSp45633e*RSe45633p + 1./96.*pow(
         RSp45633e,2) - 1./24.*RC1456pe*RS34561pe - 1./24.*RC1456pe*
         RS14563pe );

      F13p6x23RV +=  + A23bV(m13,L)*pi16 * (  - 1./24.*RC1456pe*
         RSe45613p - 1./24.*RC1456pe*RSp45613e + 1./48.*RC1456pe*
         RC3456pe + 1./48.*RC1456pe*RSe45633p + 1./48.*RC1456pe*
         RSp45633e + 1./96.*pow(RC1456pe,2) - 1./24.*RSe45611p*
         RS34561pe - 1./24.*RSe45611p*RS14563pe - 1./24.*RSe45611p*
         RSe45613p - 1./24.*RSe45611p*RSp45613e + 1./48.*RSe45611p*
         RC3456pe + 1./48.*RSe45611p*RSe45633p + 1./48.*RSe45611p*
         RSp45633e + 1./48.*RSe45611p*RC1456pe + 1./96.*pow(
         RSe45611p,2) - 1./24.*RSp45611e*RS34561pe - 1./24.*RSp45611e*
         RS14563pe - 1./24.*RSp45611e*RSe45613p - 1./24.*RSp45611e*
         RSp45613e + 1./48.*RSp45611e*RC3456pe + 1./48.*RSp45611e*
         RSe45633p + 1./48.*RSp45611e*RSp45633e + 1./48.*RSp45611e*
         RC1456pe + 1./48.*RSp45611e*RSe45611p + 1./96.*pow(
         RSp45611e,2) );

      F13p6x23RV +=  + A23bV(m14,L)*pi16 * ( 1./16.*RS34561pe + 1./16.*
         RS14563pe + 1./16.*RSe45613p + 1./16.*RSp45613e - 1./48.*
         RC3456pe - 1./48.*RSe45633p - 1./48.*RSp45633e - 1./48.*
         RS356pe - 1./48.*RSe563p - 1./48.*RSp563e - 1./16.*RC1456pe - 
         1./16.*RSe45611p - 1./16.*RSp45611e + 1./16.*RS156pe - 1./48.*
         RS456pe + 1./16.*RSe561p + 1./16.*RSp561e - 1./48.*RSe564p - 1.
         /48.*RSp564e );

      F13p6x23RV +=  + A23bV(m15,L)*pi16 * ( 1./16.*RS34561pe + 1./16.*
         RS14563pe + 1./16.*RSe45613p + 1./16.*RSp45613e - 1./48.*
         RC3456pe - 1./48.*RSe45633p - 1./48.*RSp45633e - 1./48.*
         RS346pe - 1./48.*RSe463p - 1./48.*RSp463e - 1./16.*RC1456pe - 
         1./16.*RSe45611p - 1./16.*RSp45611e + 1./16.*RS146pe - 1./48.*
         RS546pe + 1./16.*RSe461p + 1./16.*RSp461e - 1./48.*RSe465p - 1.
         /48.*RSp465e );

      F13p6x23RV +=  + A23bV(m16,L)*pi16 * ( 1./16.*RS34561pe + 1./16.*
         RS14563pe + 1./16.*RSe45613p + 1./16.*RSp45613e - 1./48.*
         RC3456pe - 1./48.*RSe45633p - 1./48.*RSp45633e - 1./48.*
         RS345pe - 1./48.*RSe453p - 1./48.*RSp453e - 1./16.*RC1456pe - 
         1./16.*RSe45611p - 1./16.*RSp45611e + 1./16.*RS145pe - 1./48.*
         RS645pe + 1./16.*RSe451p + 1./16.*RSp451e - 1./48.*RSe456p - 1.
         /48.*RSp456e );

      F13p6x23RV +=  + A23bV(m33,L)*pi16 * (  - 3./16. + 1./36.*pow(
         RS34561pe,2) + 1./36.*RS14563pe*RS34561pe + 1./36.*RSe45613p*
         RS34561pe + 1./36.*RSp45613e*RS34561pe - 1./36.*RC3456pe*
         RS34561pe - 1./72.*RC3456pe*RS14563pe - 1./72.*RC3456pe*
         RSe45613p - 1./72.*RC3456pe*RSp45613e + 1./144.*pow(
         RC3456pe,2) - 1./72.*RSe45633p*RS34561pe + 1./144.*RSe45633p*
         RC3456pe - 1./72.*RSp45633e*RS34561pe + 1./144.*RSp45633e*
         RC3456pe + 1./16.*RS345pe + 1./16.*RS346pe + 1./16.*RS356pe - 
         1./72.*RC1456pe*RS34561pe + 1./144.*RC1456pe*RC3456pe - 1./72.
         *RSe45611p*RS34561pe + 1./144.*RSe45611p*RC3456pe - 1./72.*
         RSp45611e*RS34561pe + 1./144.*RSp45611e*RC3456pe );

      F13p6x23RV +=  + A23bV(m34,L)*pi16 * ( 1./16.*RS34561pe + 1./16.*
         RS14563pe + 1./16.*RSe45613p + 1./16.*RSp45613e - 1./16.*
         RC3456pe - 1./16.*RSe45633p - 1./16.*RSp45633e + 1./16.*
         RS356pe + 1./16.*RSe563p + 1./16.*RSp563e - 1./48.*RC1456pe - 
         1./48.*RSe45611p - 1./48.*RSp45611e - 1./48.*RS156pe - 1./48.*
         RS456pe - 1./48.*RSe561p - 1./48.*RSp561e - 1./48.*RSe564p - 1.
         /48.*RSp564e );

      F13p6x23RV +=  + A23bV(m35,L)*pi16 * ( 1./16.*RS34561pe + 1./16.*
         RS14563pe + 1./16.*RSe45613p + 1./16.*RSp45613e - 1./16.*
         RC3456pe - 1./16.*RSe45633p - 1./16.*RSp45633e + 1./16.*
         RS346pe + 1./16.*RSe463p + 1./16.*RSp463e - 1./48.*RC1456pe - 
         1./48.*RSe45611p - 1./48.*RSp45611e - 1./48.*RS146pe - 1./48.*
         RS546pe - 1./48.*RSe461p - 1./48.*RSp461e - 1./48.*RSe465p - 1.
         /48.*RSp465e );

      F13p6x23RV +=  + A23bV(m36,L)*pi16 * ( 1./16.*RS34561pe + 1./16.*
         RS14563pe + 1./16.*RSe45613p + 1./16.*RSp45613e - 1./16.*
         RC3456pe - 1./16.*RSe45633p - 1./16.*RSp45633e + 1./16.*
         RS345pe + 1./16.*RSe453p + 1./16.*RSp453e - 1./48.*RC1456pe - 
         1./48.*RSe45611p - 1./48.*RSp45611e - 1./48.*RS145pe - 1./48.*
         RS645pe - 1./48.*RSe451p - 1./48.*RSp451e - 1./48.*RSe456p - 1.
         /48.*RSp456e );

      F13p6x23RV +=  + A23bV(mp2,L)*pi16 * ( 1./36.*RSp45613e*RS34561pe
          + 1./36.*RSp45613e*RS14563pe + 1./36.*RSp45613e*RSe45613p + 1.
         /36.*pow(RSp45613e,2) - 1./72.*RC3456pe*RSp45613e - 1./72.*
         RSe45633p*RSp45613e - 1./72.*RSp45633e*RS34561pe - 1./72.*
         RSp45633e*RS14563pe - 1./72.*RSp45633e*RSe45613p - 1./36.*
         RSp45633e*RSp45613e + 1./144.*RSp45633e*RC3456pe + 1./144.*
         RSp45633e*RSe45633p + 1./144.*pow(RSp45633e,2) + 1./16.*
         RSp563e + 1./16.*RSp463e + 1./16.*RSp453e - 1./72.*RC1456pe*
         RSp45613e + 1./144.*RC1456pe*RSp45633e - 1./72.*RSe45611p*
         RSp45613e + 1./144.*RSe45611p*RSp45633e - 1./72.*RSp45611e*
         RS34561pe - 1./72.*RSp45611e*RS14563pe - 1./72.*RSp45611e*
         RSe45613p - 1./36.*RSp45611e*RSp45613e + 1./144.*RSp45611e*
         RC3456pe + 1./144.*RSp45611e*RSe45633p + 1./72.*RSp45611e*
         RSp45633e + 1./144.*RSp45611e*RC1456pe + 1./144.*RSp45611e*
         RSe45611p + 1./144.*pow(RSp45611e,2) + 1./16.*RSp561e + 1./16.
         *RSp461e );

      F13p6x23RV +=  + A23bV(mp2,L)*pi16 * ( 1./16.*RSp451e );

      F13p6x23RV +=  + A23bV(me2,L)*pi16 * ( 1./36.*RSe45613p*RS34561pe
          + 1./36.*RSe45613p*RS14563pe + 1./36.*pow(RSe45613p,2) + 1./
         36.*RSp45613e*RSe45613p - 1./72.*RC3456pe*RSe45613p - 1./72.*
         RSe45633p*RS34561pe - 1./72.*RSe45633p*RS14563pe - 1./36.*
         RSe45633p*RSe45613p - 1./72.*RSe45633p*RSp45613e + 1./144.*
         RSe45633p*RC3456pe + 1./144.*pow(RSe45633p,2) - 1./72.*
         RSp45633e*RSe45613p + 1./144.*RSp45633e*RSe45633p + 1./16.*
         RSe563p + 1./16.*RSe463p + 1./16.*RSe453p - 1./72.*RC1456pe*
         RSe45613p + 1./144.*RC1456pe*RSe45633p - 1./72.*RSe45611p*
         RS34561pe - 1./72.*RSe45611p*RS14563pe - 1./36.*RSe45611p*
         RSe45613p - 1./72.*RSe45611p*RSp45613e + 1./144.*RSe45611p*
         RC3456pe + 1./72.*RSe45611p*RSe45633p + 1./144.*RSe45611p*
         RSp45633e + 1./144.*RSe45611p*RC1456pe + 1./144.*pow(
         RSe45611p,2) - 1./72.*RSp45611e*RSe45613p + 1./144.*RSp45611e*
         RSe45633p + 1./144.*RSp45611e*RSe45611p + 1./16.*RSe561p + 1./
         16.*RSe461p );

      F13p6x23RV +=  + A23bV(me2,L)*pi16 * ( 1./16.*RSe451p );

      F13p6x23RV +=  + B23bV(m11,L)*pi16 * (  - 1./72.*RD1456ep*
         RS34561pe - 1./72.*RD1456ep*RS14563pe - 1./72.*RD1456ep*
         RSe45613p - 1./72.*RD1456ep*RSp45613e + 1./144.*RD1456ep*
         RC3456pe + 1./144.*RD1456ep*RSe45633p + 1./144.*RD1456ep*
         RSp45633e + 1./144.*RD1456ep*RC1456pe + 1./144.*RSe45611p*
         RD1456ep + 1./144.*RSp45611e*RD1456ep );

      F13p6x23RV +=  + B23bV(m33,L)*pi16 * (  - 1./72.*RD3456ep*
         RS34561pe - 1./72.*RD3456ep*RS14563pe - 1./72.*RD3456ep*
         RSe45613p - 1./72.*RD3456ep*RSp45613e + 1./144.*RD3456ep*
         RC3456pe + 1./144.*RSe45633p*RD3456ep + 1./144.*RSp45633e*
         RD3456ep + 1./144.*RC1456pe*RD3456ep + 1./144.*RSe45611p*
         RD3456ep + 1./144.*RSp45611e*RD3456ep );

      F13p6x23RV +=  + hhV(1,m11,m11,m13,m13,L,mu2) * ( 1./8.*m11 );

      F13p6x23RV +=  + hhV(1,m11,m14,m34,m13,L,mu2) * ( 1./24.*m11*
         RS14563pe - 1./24.*m11*RS156pe );

      F13p6x23RV +=  + hhV(1,m11,m15,m35,m13,L,mu2) * ( 1./24.*m11*
         RS14563pe - 1./24.*m11*RS146pe );

      F13p6x23RV +=  + hhV(1,m11,m16,m36,m13,L,mu2) * ( 1./24.*m11*
         RS14563pe - 1./24.*m11*RS145pe );

      F13p6x23RV +=  + hhV(1,m13,m11,m11,m13,L,mu2) * (  - 1./16.*m13
          - 1./36.*m13*pow(RS14563pe,2) + 1./36.*m13*RC1456pe*RS14563pe
          - 1./144.*m13*pow(RC1456pe,2) );

      F13p6x23RV +=  + hhV(1,m13,m11,m33,m13,L,mu2) * (  - 1./18.*m13*
         RS14563pe*RS34561pe + 1./36.*m13*RC3456pe*RS14563pe + 1./36.*
         m13*RC1456pe*RS34561pe - 1./72.*m13*RC1456pe*RC3456pe );

      F13p6x23RV +=  + hhV(1,m13,m11,mp2,m13,L,mu2) * (  - 1./18.*m13*
         RSp45613e*RS14563pe + 1./36.*m13*RSp45633e*RS14563pe + 1./36.*
         m13*RC1456pe*RSp45613e - 1./72.*m13*RC1456pe*RSp45633e + 1./36.
         *m13*RSp45611e*RS14563pe - 1./72.*m13*RSp45611e*RC1456pe );

      F13p6x23RV +=  + hhV(1,m13,m11,me2,m13,L,mu2) * (  - 1./18.*m13*
         RSe45613p*RS14563pe + 1./36.*m13*RSe45633p*RS14563pe + 1./36.*
         m13*RC1456pe*RSe45613p - 1./72.*m13*RC1456pe*RSe45633p + 1./36.
         *m13*RSe45611p*RS14563pe - 1./72.*m13*RSe45611p*RC1456pe );

      F13p6x23RV +=  + hhV(1,m13,m13,m13,m13,L,mu2) * (  - 1./8.*m13 );

      F13p6x23RV +=  + hhV(1,m13,m33,m33,m13,L,mu2) * (  - 1./16.*m13
          - 1./36.*m13*pow(RS34561pe,2) + 1./36.*m13*RC3456pe*RS34561pe
          - 1./144.*m13*pow(RC3456pe,2) );

      F13p6x23RV +=  + hhV(1,m13,m33,mp2,m13,L,mu2) * (  - 1./18.*m13*
         RSp45613e*RS34561pe + 1./36.*m13*RC3456pe*RSp45613e + 1./36.*
         m13*RSp45633e*RS34561pe - 1./72.*m13*RSp45633e*RC3456pe + 1./
         36.*m13*RSp45611e*RS34561pe - 1./72.*m13*RSp45611e*RC3456pe );

      F13p6x23RV +=  + hhV(1,m13,m33,me2,m13,L,mu2) * (  - 1./18.*m13*
         RSe45613p*RS34561pe + 1./36.*m13*RC3456pe*RSe45613p + 1./36.*
         m13*RSe45633p*RS34561pe - 1./72.*m13*RSe45633p*RC3456pe + 1./
         36.*m13*RSe45611p*RS34561pe - 1./72.*m13*RSe45611p*RC3456pe );

      F13p6x23RV +=  + hhV(1,m13,mp2,mp2,m13,L,mu2) * (  - 1./36.*m13*
         pow(RSp45613e,2) + 1./36.*m13*RSp45633e*RSp45613e - 1./144.*
         m13*pow(RSp45633e,2) + 1./36.*m13*RSp45611e*RSp45613e - 1./72.
         *m13*RSp45611e*RSp45633e - 1./144.*m13*pow(RSp45611e,2) );

      F13p6x23RV +=  + hhV(1,m13,mp2,me2,m13,L,mu2) * ( 2./81.*me2*
         RSp45613e*RSe45613p - 1./81.*me2*RSe45633p*RSp45613e - 1./81.*
         me2*RSp45633e*RSe45613p + 1./162.*me2*RSp45633e*RSe45633p - 1./
         81.*me2*RSe45611p*RSp45613e + 1./162.*me2*RSe45611p*RSp45633e
          - 1./81.*me2*RSp45611e*RSe45613p + 1./162.*me2*RSp45611e*
         RSe45633p + 1./162.*me2*RSp45611e*RSe45611p + 2./81.*mp2*
         RSp45613e*RSe45613p - 1./81.*mp2*RSe45633p*RSp45613e - 1./81.*
         mp2*RSp45633e*RSe45613p + 1./162.*mp2*RSp45633e*RSe45633p - 1./
         81.*mp2*RSe45611p*RSp45613e + 1./162.*mp2*RSe45611p*RSp45633e
          - 1./81.*mp2*RSp45611e*RSe45613p + 1./162.*mp2*RSp45611e*
         RSe45633p + 1./162.*mp2*RSp45611e*RSe45611p - 1./81.*m33*
         RSp45613e*RSe45613p + 1./81.*m33*RSe45633p*RSp45613e + 1./81.*
         m33*RSp45633e*RSe45613p - 1./54.*m33*RSp45633e*RSe45633p + 1./
         162.*m33*RSe45611p*RSp45633e + 1./162.*m33*RSp45611e*RSe45633p
          - 1./162.*m33*RSp45611e*RSe45611p - 17./81.*m13*RSp45613e*
         RSe45613p );

      F13p6x23RV +=  + hhV(1,m13,mp2,me2,m13,L,mu2) * ( 17./162.*m13*
         RSe45633p*RSp45613e + 17./162.*m13*RSp45633e*RSe45613p - 17./
         324.*m13*RSp45633e*RSe45633p + 17./162.*m13*RSe45611p*
         RSp45613e - 17./324.*m13*RSe45611p*RSp45633e + 17./162.*m13*
         RSp45611e*RSe45613p - 17./324.*m13*RSp45611e*RSe45633p - 17./
         324.*m13*RSp45611e*RSe45611p - 1./81.*m11*RSp45613e*RSe45613p
          - 1./162.*m11*RSp45633e*RSe45633p + 1./81.*m11*RSe45611p*
         RSp45613e + 1./162.*m11*RSe45611p*RSp45633e + 1./81.*m11*
         RSp45611e*RSe45613p + 1./162.*m11*RSp45611e*RSe45633p - 1./54.
         *m11*RSp45611e*RSe45611p );

      F13p6x23RV +=  + hhV(1,m13,me2,mp2,m13,L,mu2) * (  - 2./81.*me2*
         RSp45613e*RSe45613p + 1./81.*me2*RSe45633p*RSp45613e + 1./81.*
         me2*RSp45633e*RSe45613p - 1./162.*me2*RSp45633e*RSe45633p + 1./
         81.*me2*RSe45611p*RSp45613e - 1./162.*me2*RSe45611p*RSp45633e
          + 1./81.*me2*RSp45611e*RSe45613p - 1./162.*me2*RSp45611e*
         RSe45633p - 1./162.*me2*RSp45611e*RSe45611p - 2./81.*mp2*
         RSp45613e*RSe45613p + 1./81.*mp2*RSe45633p*RSp45613e + 1./81.*
         mp2*RSp45633e*RSe45613p - 1./162.*mp2*RSp45633e*RSe45633p + 1./
         81.*mp2*RSe45611p*RSp45613e - 1./162.*mp2*RSe45611p*RSp45633e
          + 1./81.*mp2*RSp45611e*RSe45613p - 1./162.*mp2*RSp45611e*
         RSe45633p - 1./162.*mp2*RSp45611e*RSe45611p + 1./81.*m33*
         RSp45613e*RSe45613p - 1./81.*m33*RSe45633p*RSp45613e - 1./81.*
         m33*RSp45633e*RSe45613p + 1./54.*m33*RSp45633e*RSe45633p - 1./
         162.*m33*RSe45611p*RSp45633e - 1./162.*m33*RSp45611e*RSe45633p
          + 1./162.*m33*RSp45611e*RSe45611p + 25./162.*m13*RSp45613e*
         RSe45613p );

      F13p6x23RV +=  + hhV(1,m13,me2,mp2,m13,L,mu2) * (  - 25./324.*m13
         *RSe45633p*RSp45613e - 25./324.*m13*RSp45633e*RSe45613p + 25./
         648.*m13*RSp45633e*RSe45633p - 25./324.*m13*RSe45611p*
         RSp45613e + 25./648.*m13*RSe45611p*RSp45633e - 25./324.*m13*
         RSp45611e*RSe45613p + 25./648.*m13*RSp45611e*RSe45633p + 25./
         648.*m13*RSp45611e*RSe45611p + 1./81.*m11*RSp45613e*RSe45613p
          + 1./162.*m11*RSp45633e*RSe45633p - 1./81.*m11*RSe45611p*
         RSp45613e - 1./162.*m11*RSe45611p*RSp45633e - 1./81.*m11*
         RSp45611e*RSe45613p - 1./162.*m11*RSp45611e*RSe45633p + 1./54.
         *m11*RSp45611e*RSe45611p );

      F13p6x23RV +=  + hhV(1,m13,me2,me2,m13,L,mu2) * (  - 1./36.*m13*
         pow(RSe45613p,2) + 1./36.*m13*RSe45633p*RSe45613p - 1./144.*
         m13*pow(RSe45633p,2) + 1./36.*m13*RSe45611p*RSe45613p - 1./72.
         *m13*RSe45611p*RSe45633p - 1./144.*m13*pow(RSe45611p,2) );

      F13p6x23RV +=  + hhV(1,m14,m11,m34,m13,L,mu2) * (  - 1./24.*m14*
         RS14563pe + 1./24.*m14*RC1456pe - 1./24.*m14*RS156pe );

      F13p6x23RV +=  + hhV(1,m14,m33,m34,m13,L,mu2) * (  - 1./24.*m14*
         RS34561pe + 1./24.*m14*RS356pe );

      F13p6x23RV +=  + hhV(1,m14,m34,mp2,m13,L,mu2) * (  - 1./24.*m14*
         RSp45613e + 1./24.*m14*RSp563e + 1./24.*m14*RSp45611e - 1./24.
         *m14*RSp561e );

      F13p6x23RV +=  + hhV(1,m14,m34,me2,m13,L,mu2) * (  - 1./24.*m14*
         RSe45613p + 1./24.*m14*RSe563p + 1./24.*m14*RSe45611p - 1./24.
         *m14*RSe561p );

      F13p6x23RV +=  + hhV(1,m15,m11,m35,m13,L,mu2) * (  - 1./24.*m15*
         RS14563pe + 1./24.*m15*RC1456pe - 1./24.*m15*RS146pe );

      F13p6x23RV +=  + hhV(1,m15,m33,m35,m13,L,mu2) * (  - 1./24.*m15*
         RS34561pe + 1./24.*m15*RS346pe );

      F13p6x23RV +=  + hhV(1,m15,m35,mp2,m13,L,mu2) * (  - 1./24.*m15*
         RSp45613e + 1./24.*m15*RSp463e + 1./24.*m15*RSp45611e - 1./24.
         *m15*RSp461e );

      F13p6x23RV +=  + hhV(1,m15,m35,me2,m13,L,mu2) * (  - 1./24.*m15*
         RSe45613p + 1./24.*m15*RSe463p + 1./24.*m15*RSe45611p - 1./24.
         *m15*RSe461p );

      F13p6x23RV +=  + hhV(1,m16,m11,m36,m13,L,mu2) * (  - 1./24.*m16*
         RS14563pe + 1./24.*m16*RC1456pe - 1./24.*m16*RS145pe );

      F13p6x23RV +=  + hhV(1,m16,m33,m36,m13,L,mu2) * (  - 1./24.*m16*
         RS34561pe + 1./24.*m16*RS345pe );

      F13p6x23RV +=  + hhV(1,m16,m36,mp2,m13,L,mu2) * (  - 1./24.*m16*
         RSp45613e + 1./24.*m16*RSp453e + 1./24.*m16*RSp45611e - 1./24.
         *m16*RSp451e );

      F13p6x23RV +=  + hhV(1,m16,m36,me2,m13,L,mu2) * (  - 1./24.*m16*
         RSe45613p + 1./24.*m16*RSe453p + 1./24.*m16*RSe45611p - 1./24.
         *m16*RSe451p );

      F13p6x23RV +=  + hhV(1,m33,m13,m33,m13,L,mu2) * ( 1./8.*m33 );

      F13p6x23RV +=  + hhV(1,m33,m14,m34,m13,L,mu2) * ( 1./24.*m33*
         RS34561pe - 1./24.*m33*RS356pe );

      F13p6x23RV +=  + hhV(1,m33,m15,m35,m13,L,mu2) * ( 1./24.*m33*
         RS34561pe - 1./24.*m33*RS346pe );

      F13p6x23RV +=  + hhV(1,m33,m16,m36,m13,L,mu2) * ( 1./24.*m33*
         RS34561pe - 1./24.*m33*RS345pe );

      F13p6x23RV +=  + hhV(1,m34,m11,m14,m13,L,mu2) * (  - 1./24.*m34*
         RS14563pe + 1./24.*m34*RS156pe );

      F13p6x23RV +=  + hhV(1,m34,m14,m33,m13,L,mu2) * (  - 1./24.*m34*
         RS34561pe + 1./24.*m34*RC3456pe - 1./24.*m34*RS356pe );

      F13p6x23RV +=  + hhV(1,m34,m14,mp2,m13,L,mu2) * (  - 1./24.*m34*
         RSp45613e + 1./24.*m34*RSp45633e - 1./24.*m34*RSp563e + 1./24.
         *m34*RSp561e );

      F13p6x23RV +=  + hhV(1,m34,m14,me2,m13,L,mu2) * (  - 1./24.*m34*
         RSe45613p + 1./24.*m34*RSe45633p - 1./24.*m34*RSe563p + 1./24.
         *m34*RSe561p );

      F13p6x23RV +=  + hhV(1,m35,m11,m15,m13,L,mu2) * (  - 1./24.*m35*
         RS14563pe + 1./24.*m35*RS146pe );

      F13p6x23RV +=  + hhV(1,m35,m15,m33,m13,L,mu2) * (  - 1./24.*m35*
         RS34561pe + 1./24.*m35*RC3456pe - 1./24.*m35*RS346pe );

      F13p6x23RV +=  + hhV(1,m35,m15,mp2,m13,L,mu2) * (  - 1./24.*m35*
         RSp45613e + 1./24.*m35*RSp45633e - 1./24.*m35*RSp463e + 1./24.
         *m35*RSp461e );

      F13p6x23RV +=  + hhV(1,m35,m15,me2,m13,L,mu2) * (  - 1./24.*m35*
         RSe45613p + 1./24.*m35*RSe45633p - 1./24.*m35*RSe463p + 1./24.
         *m35*RSe461p );

      F13p6x23RV +=  + hhV(1,m36,m11,m16,m13,L,mu2) * (  - 1./24.*m36*
         RS14563pe + 1./24.*m36*RS145pe );

      F13p6x23RV +=  + hhV(1,m36,m16,m33,m13,L,mu2) * (  - 1./24.*m36*
         RS34561pe + 1./24.*m36*RC3456pe - 1./24.*m36*RS345pe );

      F13p6x23RV +=  + hhV(1,m36,m16,mp2,m13,L,mu2) * (  - 1./24.*m36*
         RSp45613e + 1./24.*m36*RSp45633e - 1./24.*m36*RSp453e + 1./24.
         *m36*RSp451e );

      F13p6x23RV +=  + hhV(1,m36,m16,me2,m13,L,mu2) * (  - 1./24.*m36*
         RSe45613p + 1./24.*m36*RSe45633p - 1./24.*m36*RSe453p + 1./24.
         *m36*RSe451p );

      F13p6x23RV +=  + hhV(1,m44,m14,m34,m13,L,mu2) * (  - 1./8.*m44 + 
         1./24.*m44*RS456pe );

      F13p6x23RV +=  + hhV(1,m45,m14,m35,m13,L,mu2) * (  - 1./8.*m45 );

      F13p6x23RV +=  + hhV(1,m45,m15,m34,m13,L,mu2) * (  - 1./8.*m45 );

      F13p6x23RV +=  + hhV(1,m46,m14,m36,m13,L,mu2) * (  - 1./8.*m46 );

      F13p6x23RV +=  + hhV(1,m46,m16,m34,m13,L,mu2) * (  - 1./8.*m46 );

      F13p6x23RV +=  + hhV(1,m55,m15,m35,m13,L,mu2) * (  - 1./8.*m55 + 
         1./24.*m55*RS546pe );

      F13p6x23RV +=  + hhV(1,m56,m15,m36,m13,L,mu2) * (  - 1./8.*m56 );

      F13p6x23RV +=  + hhV(1,m56,m16,m35,m13,L,mu2) * (  - 1./8.*m56 );

      F13p6x23RV +=  + hhV(1,m66,m16,m36,m13,L,mu2) * (  - 1./8.*m66 + 
         1./24.*m66*RS645pe );

      F13p6x23RV +=  + hhV(1,mp2,m14,m34,m13,L,mu2) * ( 1./24.*mp2*
         RSp45613e - 1./24.*mp2*RSp563e - 1./24.*mp2*RSp561e + 1./24.*
         mp2*RSp564e );

      F13p6x23RV +=  + hhV(1,mp2,m15,m35,m13,L,mu2) * ( 1./24.*mp2*
         RSp45613e - 1./24.*mp2*RSp463e - 1./24.*mp2*RSp461e + 1./24.*
         mp2*RSp465e );

      F13p6x23RV +=  + hhV(1,mp2,m16,m36,m13,L,mu2) * ( 1./24.*mp2*
         RSp45613e - 1./24.*mp2*RSp453e - 1./24.*mp2*RSp451e + 1./24.*
         mp2*RSp456e );

      F13p6x23RV +=  + hhV(1,me2,m14,m34,m13,L,mu2) * ( 1./24.*me2*
         RSe45613p - 1./24.*me2*RSe563p - 1./24.*me2*RSe561p + 1./24.*
         me2*RSe564p );

      F13p6x23RV +=  + hhV(1,me2,m15,m35,m13,L,mu2) * ( 1./24.*me2*
         RSe45613p - 1./24.*me2*RSe463p - 1./24.*me2*RSe461p + 1./24.*
         me2*RSe465p );

      F13p6x23RV +=  + hhV(1,me2,m16,m36,m13,L,mu2) * ( 1./24.*me2*
         RSe45613p - 1./24.*me2*RSe453p - 1./24.*me2*RSe451p + 1./24.*
         me2*RSe456p );

      F13p6x23RV +=  + hhV(2,m11,m11,m13,m13,L,mu2) * (  - 1./1296.*
         pow(RD1456ep,2) );

      F13p6x23RV +=  + hhV(3,m11,m11,m13,m13,L,mu2) * ( 1./1296.*pow(
         RD1456ep,2) );

      F13p6x23RV +=  + hhV(3,m13,m11,m11,m13,L,mu2) * ( 1./36.*m13*
         RD1456ep*RS14563pe - 1./72.*m13*RD1456ep*RC1456pe );

      F13p6x23RV +=  + hhV(3,m13,m11,m33,m13,L,mu2) * ( 1./36.*m13*
         RD1456ep*RS34561pe - 1./72.*m13*RD1456ep*RC3456pe );

      F13p6x23RV +=  + hhV(3,m13,m11,mp2,m13,L,mu2) * ( 1./36.*m13*
         RD1456ep*RSp45613e - 1./72.*m13*RD1456ep*RSp45633e - 1./72.*
         m13*RSp45611e*RD1456ep );

      F13p6x23RV +=  + hhV(3,m13,m11,me2,m13,L,mu2) * ( 1./36.*m13*
         RD1456ep*RSe45613p - 1./72.*m13*RD1456ep*RSe45633p - 1./72.*
         m13*RSe45611p*RD1456ep );

      F13p6x23RV +=  + hhV(3,m13,m33,m33,m13,L,mu2) * (  - 1./324.*pow(
         RD3456ep,2) + 1./36.*m13*RD3456ep*RS34561pe - 1./72.*m13*
         RD3456ep*RC3456pe );

      F13p6x23RV +=  + hhV(3,m13,m33,mp2,m13,L,mu2) * ( 1./36.*m13*
         RD3456ep*RSp45613e - 1./72.*m13*RSp45633e*RD3456ep - 1./72.*
         m13*RSp45611e*RD3456ep );

      F13p6x23RV +=  + hhV(3,m13,m33,me2,m13,L,mu2) * ( 1./36.*m13*
         RD3456ep*RSe45613p - 1./72.*m13*RSe45633p*RD3456ep - 1./72.*
         m13*RSe45611p*RD3456ep );

      F13p6x23RV +=  + hhV(3,m14,m11,m34,m13,L,mu2) * ( 1./24.*m14*
         RD1456ep );

      F13p6x23RV +=  + hhV(3,m15,m11,m35,m13,L,mu2) * ( 1./24.*m15*
         RD1456ep );

      F13p6x23RV +=  + hhV(3,m16,m11,m36,m13,L,mu2) * ( 1./24.*m16*
         RD1456ep );

      F13p6x23RV +=  + hhV(4,m13,m11,m33,m13,L,mu2) * ( 1./36.*m13*
         RD3456ep*RS14563pe - 1./72.*m13*RC1456pe*RD3456ep );

      F13p6x23RV +=  + hhV(4,m13,m33,m33,m13,L,mu2) * (  - 1./324.*pow(
         RD3456ep,2) );

      F13p6x23RV +=  + hhV(4,m34,m14,m33,m13,L,mu2) * ( 1./24.*m34*
         RD3456ep );

      F13p6x23RV +=  + hhV(4,m35,m15,m33,m13,L,mu2) * ( 1./24.*m35*
         RD3456ep );

      F13p6x23RV +=  + hhV(4,m36,m16,m33,m13,L,mu2) * ( 1./24.*m36*
         RD3456ep );

      F13p6x23RV +=  + hhV(7,m13,m11,m11,m13,L,mu2) * (  - 1./144.*m13*
         pow(RD1456ep,2) );

      F13p6x23RV +=  + hhV(7,m13,m11,m33,m13,L,mu2) * (  - 1./72.*m13*
         RD1456ep*RD3456ep );

      F13p6x23RV +=  + hhV(7,m13,m33,m33,m13,L,mu2) * ( 1./324.*m33*
         pow(RD3456ep,2) - 1./72.*m13*pow(RD3456ep,2) + 1./324.*m11*
         pow(RD3456ep,2) );

      F13p6x23RV +=  + hh1V(1,m13,mp2,me2,m13,L,mu2) * (  - 2./27.*me2*
         RSp45613e*RSe45613p + 1./27.*me2*RSe45633p*RSp45613e + 1./27.*
         me2*RSp45633e*RSe45613p - 1./54.*me2*RSp45633e*RSe45633p + 1./
         27.*me2*RSe45611p*RSp45613e - 1./54.*me2*RSe45611p*RSp45633e
          + 1./27.*me2*RSp45611e*RSe45613p - 1./54.*me2*RSp45611e*
         RSe45633p - 1./54.*me2*RSp45611e*RSe45611p - 2./27.*mp2*
         RSp45613e*RSe45613p + 1./27.*mp2*RSe45633p*RSp45613e + 1./27.*
         mp2*RSp45633e*RSe45613p - 1./54.*mp2*RSp45633e*RSe45633p + 1./
         27.*mp2*RSe45611p*RSp45613e - 1./54.*mp2*RSe45611p*RSp45633e
          + 1./27.*mp2*RSp45611e*RSe45613p - 1./54.*mp2*RSp45611e*
         RSe45633p - 1./54.*mp2*RSp45611e*RSe45611p + 1./27.*m33*
         RSp45613e*RSe45613p - 1./27.*m33*RSe45633p*RSp45613e - 1./27.*
         m33*RSp45633e*RSe45613p + 1./18.*m33*RSp45633e*RSe45633p - 1./
         54.*m33*RSe45611p*RSp45633e - 1./54.*m33*RSp45611e*RSe45633p
          + 1./54.*m33*RSp45611e*RSe45611p + 4./9.*m13*RSp45613e*
         RSe45613p );

      F13p6x23RV +=  + hh1V(1,m13,mp2,me2,m13,L,mu2) * (  - 2./9.*m13*
         RSe45633p*RSp45613e - 2./9.*m13*RSp45633e*RSe45613p + 1./9.*
         m13*RSp45633e*RSe45633p - 2./9.*m13*RSe45611p*RSp45613e + 1./9.
         *m13*RSe45611p*RSp45633e - 2./9.*m13*RSp45611e*RSe45613p + 1./
         9.*m13*RSp45611e*RSe45633p + 1./9.*m13*RSp45611e*RSe45611p + 1.
         /27.*m11*RSp45613e*RSe45613p + 1./54.*m11*RSp45633e*RSe45633p
          - 1./27.*m11*RSe45611p*RSp45613e - 1./54.*m11*RSe45611p*
         RSp45633e - 1./27.*m11*RSp45611e*RSe45613p - 1./54.*m11*
         RSp45611e*RSe45633p + 1./18.*m11*RSp45611e*RSe45611p );

      F13p6x23RV +=  + hh1V(1,m13,me2,mp2,m13,L,mu2) * ( 2./27.*me2*
         RSp45613e*RSe45613p - 1./27.*me2*RSe45633p*RSp45613e - 1./27.*
         me2*RSp45633e*RSe45613p + 1./54.*me2*RSp45633e*RSe45633p - 1./
         27.*me2*RSe45611p*RSp45613e + 1./54.*me2*RSe45611p*RSp45633e
          - 1./27.*me2*RSp45611e*RSe45613p + 1./54.*me2*RSp45611e*
         RSe45633p + 1./54.*me2*RSp45611e*RSe45611p + 2./27.*mp2*
         RSp45613e*RSe45613p - 1./27.*mp2*RSe45633p*RSp45613e - 1./27.*
         mp2*RSp45633e*RSe45613p + 1./54.*mp2*RSp45633e*RSe45633p - 1./
         27.*mp2*RSe45611p*RSp45613e + 1./54.*mp2*RSe45611p*RSp45633e
          - 1./27.*mp2*RSp45611e*RSe45613p + 1./54.*mp2*RSp45611e*
         RSe45633p + 1./54.*mp2*RSp45611e*RSe45611p - 1./27.*m33*
         RSp45613e*RSe45613p + 1./27.*m33*RSe45633p*RSp45613e + 1./27.*
         m33*RSp45633e*RSe45613p - 1./18.*m33*RSp45633e*RSe45633p + 1./
         54.*m33*RSe45611p*RSp45633e + 1./54.*m33*RSp45611e*RSe45633p
          - 1./54.*m33*RSp45611e*RSe45611p - 4./9.*m13*RSp45613e*
         RSe45613p );

      F13p6x23RV +=  + hh1V(1,m13,me2,mp2,m13,L,mu2) * ( 2./9.*m13*
         RSe45633p*RSp45613e + 2./9.*m13*RSp45633e*RSe45613p - 1./9.*
         m13*RSp45633e*RSe45633p + 2./9.*m13*RSe45611p*RSp45613e - 1./9.
         *m13*RSe45611p*RSp45633e + 2./9.*m13*RSp45611e*RSe45613p - 1./
         9.*m13*RSp45611e*RSe45633p - 1./9.*m13*RSp45611e*RSe45611p - 1.
         /27.*m11*RSp45613e*RSe45613p - 1./54.*m11*RSp45633e*RSe45633p
          + 1./27.*m11*RSe45611p*RSp45613e + 1./54.*m11*RSe45611p*
         RSp45633e + 1./27.*m11*RSp45611e*RSe45613p + 1./54.*m11*
         RSp45611e*RSe45633p - 1./18.*m11*RSp45611e*RSe45611p );

      F13p6x23RV +=  + hh1V(2,m33,m13,m33,m13,L,mu2) * ( 1./162.*pow(
         RD3456ep,2) );

      F13p6x23RV +=  + hh1V(3,m13,m33,m33,m13,L,mu2) * ( 1./432.*pow(
         RD3456ep,2) );

      F13p6x23RV +=  + hh1V(4,m13,m33,m33,m13,L,mu2) * ( 5./1296.*pow(
         RD3456ep,2) );

      F13p6x23RV +=  + hh1V(4,m33,m13,m33,m13,L,mu2) * ( 1./162.*pow(
         RD3456ep,2) );

      F13p6x23RV +=  + hh1V(6,m33,m13,m33,m13,L,mu2) * (  - 1./162.*m33
         *pow(RD3456ep,2) + 1./72.*m13*pow(RD3456ep,2) - 1./162.*m11*
         pow(RD3456ep,2) );

      F13p6x23RV +=  + hh1V(7,m13,m33,m33,m13,L,mu2) * (  - 1./324.*m33
         *pow(RD3456ep,2) + 1./144.*m13*pow(RD3456ep,2) - 1./324.*m11*
         pow(RD3456ep,2) );

      F13p6x23RV +=  + hh21V(1,m13,mp2,me2,m13,L,mu2) * (  - 1./3.*m13*
         RSp45613e*RSe45613p + 1./6.*m13*RSe45633p*RSp45613e + 1./6.*
         m13*RSp45633e*RSe45613p - 1./12.*m13*RSp45633e*RSe45633p + 1./
         6.*m13*RSe45611p*RSp45613e - 1./12.*m13*RSe45611p*RSp45633e + 
         1./6.*m13*RSp45611e*RSe45613p - 1./12.*m13*RSp45611e*RSe45633p
          - 1./12.*m13*RSp45611e*RSe45611p );

      F13p6x23RV +=  + hh21V(1,m13,me2,mp2,m13,L,mu2) * ( 1./3.*m13*
         RSp45613e*RSe45613p - 1./6.*m13*RSe45633p*RSp45613e - 1./6.*
         m13*RSp45633e*RSe45613p + 1./12.*m13*RSp45633e*RSe45633p - 1./
         6.*m13*RSe45611p*RSp45613e + 1./12.*m13*RSe45611p*RSp45633e - 
         1./6.*m13*RSp45611e*RSe45613p + 1./12.*m13*RSp45611e*RSe45633p
          + 1./12.*m13*RSp45611e*RSe45611p );

      F13p6x23RV +=  + hh27V(1,m11,m11,m13,m13,L,mu2) * (  - 3./8. );

      F13p6x23RV +=  + hh27V(1,m11,m14,m34,m13,L,mu2) * (  - 1./8.*
         RS14563pe + 1./8.*RS156pe );

      F13p6x23RV +=  + hh27V(1,m11,m15,m35,m13,L,mu2) * (  - 1./8.*
         RS14563pe + 1./8.*RS146pe );

      F13p6x23RV +=  + hh27V(1,m11,m16,m36,m13,L,mu2) * (  - 1./8.*
         RS14563pe + 1./8.*RS145pe );

      F13p6x23RV +=  + hh27V(1,m13,m11,m11,m13,L,mu2) * ( 3./16. + 1./
         12.*pow(RS14563pe,2) - 1./12.*RC1456pe*RS14563pe + 1./48.*pow(
         RC1456pe,2) );

      F13p6x23RV +=  + hh27V(1,m13,m11,m33,m13,L,mu2) * ( 1./6.*
         RS14563pe*RS34561pe - 1./12.*RC3456pe*RS14563pe - 1./12.*
         RC1456pe*RS34561pe + 1./24.*RC1456pe*RC3456pe );

      F13p6x23RV +=  + hh27V(1,m13,m11,mp2,m13,L,mu2) * ( 1./6.*
         RSp45613e*RS14563pe - 1./12.*RSp45633e*RS14563pe - 1./12.*
         RC1456pe*RSp45613e + 1./24.*RC1456pe*RSp45633e - 1./12.*
         RSp45611e*RS14563pe + 1./24.*RSp45611e*RC1456pe );

      F13p6x23RV +=  + hh27V(1,m13,m11,me2,m13,L,mu2) * ( 1./6.*
         RSe45613p*RS14563pe - 1./12.*RSe45633p*RS14563pe - 1./12.*
         RC1456pe*RSe45613p + 1./24.*RC1456pe*RSe45633p - 1./12.*
         RSe45611p*RS14563pe + 1./24.*RSe45611p*RC1456pe );

      F13p6x23RV +=  + hh27V(1,m13,m13,m13,m13,L,mu2) * ( 3./8. );

      F13p6x23RV +=  + hh27V(1,m13,m33,m33,m13,L,mu2) * ( 3./16. + 1./
         12.*pow(RS34561pe,2) - 1./12.*RC3456pe*RS34561pe + 1./48.*pow(
         RC3456pe,2) );

      F13p6x23RV +=  + hh27V(1,m13,m33,mp2,m13,L,mu2) * ( 1./6.*
         RSp45613e*RS34561pe - 1./12.*RC3456pe*RSp45613e - 1./12.*
         RSp45633e*RS34561pe + 1./24.*RSp45633e*RC3456pe - 1./12.*
         RSp45611e*RS34561pe + 1./24.*RSp45611e*RC3456pe );

      F13p6x23RV +=  + hh27V(1,m13,m33,me2,m13,L,mu2) * ( 1./6.*
         RSe45613p*RS34561pe - 1./12.*RC3456pe*RSe45613p - 1./12.*
         RSe45633p*RS34561pe + 1./24.*RSe45633p*RC3456pe - 1./12.*
         RSe45611p*RS34561pe + 1./24.*RSe45611p*RC3456pe );

      F13p6x23RV +=  + hh27V(1,m13,mp2,mp2,m13,L,mu2) * ( 1./12.*pow(
         RSp45613e,2) - 1./12.*RSp45633e*RSp45613e + 1./48.*pow(
         RSp45633e,2) - 1./12.*RSp45611e*RSp45613e + 1./24.*RSp45611e*
         RSp45633e + 1./48.*pow(RSp45611e,2) );

      F13p6x23RV +=  + hh27V(1,m13,mp2,me2,m13,L,mu2) * ( 1./3.*
         RSp45613e*RSe45613p - 1./6.*RSe45633p*RSp45613e - 1./6.*
         RSp45633e*RSe45613p + 1./12.*RSp45633e*RSe45633p - 1./6.*
         RSe45611p*RSp45613e + 1./12.*RSe45611p*RSp45633e - 1./6.*
         RSp45611e*RSe45613p + 1./12.*RSp45611e*RSe45633p + 1./12.*
         RSp45611e*RSe45611p );

      F13p6x23RV +=  + hh27V(1,m13,me2,mp2,m13,L,mu2) * (  - 1./6.*
         RSp45613e*RSe45613p + 1./12.*RSe45633p*RSp45613e + 1./12.*
         RSp45633e*RSe45613p - 1./24.*RSp45633e*RSe45633p + 1./12.*
         RSe45611p*RSp45613e - 1./24.*RSe45611p*RSp45633e + 1./12.*
         RSp45611e*RSe45613p - 1./24.*RSp45611e*RSe45633p - 1./24.*
         RSp45611e*RSe45611p );

      F13p6x23RV +=  + hh27V(1,m13,me2,me2,m13,L,mu2) * ( 1./12.*pow(
         RSe45613p,2) - 1./12.*RSe45633p*RSe45613p + 1./48.*pow(
         RSe45633p,2) - 1./12.*RSe45611p*RSe45613p + 1./24.*RSe45611p*
         RSe45633p + 1./48.*pow(RSe45611p,2) );

      F13p6x23RV +=  + hh27V(1,m14,m11,m34,m13,L,mu2) * ( 1./8.*
         RS14563pe - 1./8.*RC1456pe + 1./8.*RS156pe );

      F13p6x23RV +=  + hh27V(1,m14,m33,m34,m13,L,mu2) * ( 1./8.*
         RS34561pe - 1./8.*RS356pe );

      F13p6x23RV +=  + hh27V(1,m14,m34,mp2,m13,L,mu2) * ( 1./8.*
         RSp45613e - 1./8.*RSp563e - 1./8.*RSp45611e + 1./8.*RSp561e );

      F13p6x23RV +=  + hh27V(1,m14,m34,me2,m13,L,mu2) * ( 1./8.*
         RSe45613p - 1./8.*RSe563p - 1./8.*RSe45611p + 1./8.*RSe561p );

      F13p6x23RV +=  + hh27V(1,m15,m11,m35,m13,L,mu2) * ( 1./8.*
         RS14563pe - 1./8.*RC1456pe + 1./8.*RS146pe );

      F13p6x23RV +=  + hh27V(1,m15,m33,m35,m13,L,mu2) * ( 1./8.*
         RS34561pe - 1./8.*RS346pe );

      F13p6x23RV +=  + hh27V(1,m15,m35,mp2,m13,L,mu2) * ( 1./8.*
         RSp45613e - 1./8.*RSp463e - 1./8.*RSp45611e + 1./8.*RSp461e );

      F13p6x23RV +=  + hh27V(1,m15,m35,me2,m13,L,mu2) * ( 1./8.*
         RSe45613p - 1./8.*RSe463p - 1./8.*RSe45611p + 1./8.*RSe461p );

      F13p6x23RV +=  + hh27V(1,m16,m11,m36,m13,L,mu2) * ( 1./8.*
         RS14563pe - 1./8.*RC1456pe + 1./8.*RS145pe );

      F13p6x23RV +=  + hh27V(1,m16,m33,m36,m13,L,mu2) * ( 1./8.*
         RS34561pe - 1./8.*RS345pe );

      F13p6x23RV +=  + hh27V(1,m16,m36,mp2,m13,L,mu2) * ( 1./8.*
         RSp45613e - 1./8.*RSp453e - 1./8.*RSp45611e + 1./8.*RSp451e );

      F13p6x23RV +=  + hh27V(1,m16,m36,me2,m13,L,mu2) * ( 1./8.*
         RSe45613p - 1./8.*RSe453p - 1./8.*RSe45611p + 1./8.*RSe451p );

      F13p6x23RV +=  + hh27V(1,m33,m13,m33,m13,L,mu2) * (  - 3./8. );

      F13p6x23RV +=  + hh27V(1,m33,m14,m34,m13,L,mu2) * (  - 1./8.*
         RS34561pe + 1./8.*RS356pe );

      F13p6x23RV +=  + hh27V(1,m33,m15,m35,m13,L,mu2) * (  - 1./8.*
         RS34561pe + 1./8.*RS346pe );

      F13p6x23RV +=  + hh27V(1,m33,m16,m36,m13,L,mu2) * (  - 1./8.*
         RS34561pe + 1./8.*RS345pe );

      F13p6x23RV +=  + hh27V(1,m34,m11,m14,m13,L,mu2) * ( 1./8.*
         RS14563pe - 1./8.*RS156pe );

      F13p6x23RV +=  + hh27V(1,m34,m14,m33,m13,L,mu2) * ( 1./8.*
         RS34561pe - 1./8.*RC3456pe + 1./8.*RS356pe );

      F13p6x23RV +=  + hh27V(1,m34,m14,mp2,m13,L,mu2) * ( 1./8.*
         RSp45613e - 1./8.*RSp45633e + 1./8.*RSp563e - 1./8.*RSp561e );

      F13p6x23RV +=  + hh27V(1,m34,m14,me2,m13,L,mu2) * ( 1./8.*
         RSe45613p - 1./8.*RSe45633p + 1./8.*RSe563p - 1./8.*RSe561p );

      F13p6x23RV +=  + hh27V(1,m35,m11,m15,m13,L,mu2) * ( 1./8.*
         RS14563pe - 1./8.*RS146pe );

      F13p6x23RV +=  + hh27V(1,m35,m15,m33,m13,L,mu2) * ( 1./8.*
         RS34561pe - 1./8.*RC3456pe + 1./8.*RS346pe );

      F13p6x23RV +=  + hh27V(1,m35,m15,mp2,m13,L,mu2) * ( 1./8.*
         RSp45613e - 1./8.*RSp45633e + 1./8.*RSp463e - 1./8.*RSp461e );

      F13p6x23RV +=  + hh27V(1,m35,m15,me2,m13,L,mu2) * ( 1./8.*
         RSe45613p - 1./8.*RSe45633p + 1./8.*RSe463p - 1./8.*RSe461p );

      F13p6x23RV +=  + hh27V(1,m36,m11,m16,m13,L,mu2) * ( 1./8.*
         RS14563pe - 1./8.*RS145pe );

      F13p6x23RV +=  + hh27V(1,m36,m16,m33,m13,L,mu2) * ( 1./8.*
         RS34561pe - 1./8.*RC3456pe + 1./8.*RS345pe );

      F13p6x23RV +=  + hh27V(1,m36,m16,mp2,m13,L,mu2) * ( 1./8.*
         RSp45613e - 1./8.*RSp45633e + 1./8.*RSp453e - 1./8.*RSp451e );

      F13p6x23RV +=  + hh27V(1,m36,m16,me2,m13,L,mu2) * ( 1./8.*
         RSe45613p - 1./8.*RSe45633p + 1./8.*RSe453p - 1./8.*RSe451p );

      F13p6x23RV +=  + hh27V(1,m44,m14,m34,m13,L,mu2) * ( 3./8. - 1./8.
         *RS456pe );

      F13p6x23RV +=  + hh27V(1,m45,m14,m35,m13,L,mu2) * ( 3./8. );

      F13p6x23RV +=  + hh27V(1,m45,m15,m34,m13,L,mu2) * ( 3./8. );

      F13p6x23RV +=  + hh27V(1,m46,m14,m36,m13,L,mu2) * ( 3./8. );

      F13p6x23RV +=  + hh27V(1,m46,m16,m34,m13,L,mu2) * ( 3./8. );

      F13p6x23RV +=  + hh27V(1,m55,m15,m35,m13,L,mu2) * ( 3./8. - 1./8.
         *RS546pe );

      F13p6x23RV +=  + hh27V(1,m56,m15,m36,m13,L,mu2) * ( 3./8. );

      F13p6x23RV +=  + hh27V(1,m56,m16,m35,m13,L,mu2) * ( 3./8. );

      F13p6x23RV +=  + hh27V(1,m66,m16,m36,m13,L,mu2) * ( 3./8. - 1./8.
         *RS645pe );

      F13p6x23RV +=  + hh27V(1,mp2,m14,m34,m13,L,mu2) * (  - 1./8.*
         RSp45613e + 1./8.*RSp563e + 1./8.*RSp561e - 1./8.*RSp564e );

      F13p6x23RV +=  + hh27V(1,mp2,m15,m35,m13,L,mu2) * (  - 1./8.*
         RSp45613e + 1./8.*RSp463e + 1./8.*RSp461e - 1./8.*RSp465e );

      F13p6x23RV +=  + hh27V(1,mp2,m16,m36,m13,L,mu2) * (  - 1./8.*
         RSp45613e + 1./8.*RSp453e + 1./8.*RSp451e - 1./8.*RSp456e );

      F13p6x23RV +=  + hh27V(1,me2,m14,m34,m13,L,mu2) * (  - 1./8.*
         RSe45613p + 1./8.*RSe563p + 1./8.*RSe561p - 1./8.*RSe564p );

      F13p6x23RV +=  + hh27V(1,me2,m15,m35,m13,L,mu2) * (  - 1./8.*
         RSe45613p + 1./8.*RSe463p + 1./8.*RSe461p - 1./8.*RSe465p );

      F13p6x23RV +=  + hh27V(1,me2,m16,m36,m13,L,mu2) * (  - 1./8.*
         RSe45613p + 1./8.*RSe453p + 1./8.*RSe451p - 1./8.*RSe456p );

      F13p6x23RV +=  + hh27V(3,m13,m11,m11,m13,L,mu2) * (  - 1./12.*
         RD1456ep*RS14563pe + 1./24.*RD1456ep*RC1456pe );

      F13p6x23RV +=  + hh27V(3,m13,m11,m33,m13,L,mu2) * (  - 1./12.*
         RD1456ep*RS34561pe + 1./24.*RD1456ep*RC3456pe );

      F13p6x23RV +=  + hh27V(3,m13,m11,mp2,m13,L,mu2) * (  - 1./12.*
         RD1456ep*RSp45613e + 1./24.*RD1456ep*RSp45633e + 1./24.*
         RSp45611e*RD1456ep );

      F13p6x23RV +=  + hh27V(3,m13,m11,me2,m13,L,mu2) * (  - 1./12.*
         RD1456ep*RSe45613p + 1./24.*RD1456ep*RSe45633p + 1./24.*
         RSe45611p*RD1456ep );

      F13p6x23RV +=  + hh27V(3,m13,m33,m33,m13,L,mu2) * (  - 1./12.*
         RD3456ep*RS34561pe + 1./24.*RD3456ep*RC3456pe );

      F13p6x23RV +=  + hh27V(3,m13,m33,mp2,m13,L,mu2) * (  - 1./12.*
         RD3456ep*RSp45613e + 1./24.*RSp45633e*RD3456ep + 1./24.*
         RSp45611e*RD3456ep );

      F13p6x23RV +=  + hh27V(3,m13,m33,me2,m13,L,mu2) * (  - 1./12.*
         RD3456ep*RSe45613p + 1./24.*RSe45633p*RD3456ep + 1./24.*
         RSe45611p*RD3456ep );

      F13p6x23RV +=  + hh27V(3,m14,m11,m34,m13,L,mu2) * (  - 1./8.*
         RD1456ep );

      F13p6x23RV +=  + hh27V(3,m15,m11,m35,m13,L,mu2) * (  - 1./8.*
         RD1456ep );

      F13p6x23RV +=  + hh27V(3,m16,m11,m36,m13,L,mu2) * (  - 1./8.*
         RD1456ep );

      F13p6x23RV +=  + hh27V(4,m13,m11,m33,m13,L,mu2) * (  - 1./12.*
         RD3456ep*RS14563pe + 1./24.*RC1456pe*RD3456ep );

      F13p6x23RV +=  + hh27V(4,m34,m14,m33,m13,L,mu2) * (  - 1./8.*
         RD3456ep );

      F13p6x23RV +=  + hh27V(4,m35,m15,m33,m13,L,mu2) * (  - 1./8.*
         RD3456ep );

      F13p6x23RV +=  + hh27V(4,m36,m16,m33,m13,L,mu2) * (  - 1./8.*
         RD3456ep );

      F13p6x23RV +=  + hh27V(7,m13,m11,m11,m13,L,mu2) * ( 1./48.*pow(
         RD1456ep,2) );

      F13p6x23RV +=  + hh27V(7,m13,m11,m33,m13,L,mu2) * ( 1./24.*
         RD1456ep*RD3456ep );

      F13p6x23RV +=  + hh27V(7,m13,m33,m33,m13,L,mu2) * ( 1./48.*pow(
         RD3456ep,2) );

      F13p6x23RV +=  + hhdV(1,m11,m11,m13,m13,L,mu2) * ( 1./648.*pow(
         RD1456ep,2) - 1./324.*m33*RD1456ep*RC1456pe + 1./288.*pow(
         m33,2) + 1./162.*pow(m33,2)*pow(RS14563pe,2) + 1./324.*pow(
         m33,2)*RC1456pe*RS14563pe + 1./1296.*pow(m33,2)*pow(
         RC1456pe,2) - 1./81.*m13*RD1456ep*RS14563pe + 1./162.*m13*
         RD1456ep*RC1456pe - 1./72.*m13*m33 - 1./162.*m13*m33*pow(
         RS14563pe,2) - 1./324.*m13*m33*pow(RC1456pe,2) - 17./72.*pow(
         m13,2) - 8./81.*pow(m13,2)*pow(RS14563pe,2) + 8./81.*pow(
         m13,2)*RC1456pe*RS14563pe - 2./81.*pow(m13,2)*pow(RC1456pe,2)
          - 1./162.*m11*RD1456ep*RS14563pe - 1./324.*m11*RD1456ep*
         RC1456pe + 1./144.*m11*m33 + 1./81.*m11*m33*pow(RS14563pe,2)
          + 1./81.*m11*m33*RC1456pe*RS14563pe + 1./648.*m11*m33*pow(
         RC1456pe,2) - 5./36.*m11*m13 + 1./54.*m11*m13*pow(RS14563pe,2)
          - 1./81.*m11*m13*RC1456pe*RS14563pe - 1./324.*m11*m13*pow(
         RC1456pe,2) + 1./288.*pow(m11,2) + 1./81.*pow(m11,2)*pow(
         RS14563pe,2) );

      F13p6x23RV +=  + hhdV(1,m11,m11,m13,m13,L,mu2) * ( 1./108.*pow(
         m11,2)*RC1456pe*RS14563pe + 1./1296.*pow(m11,2)*pow(
         RC1456pe,2) );

      F13p6x23RV +=  + hhdV(1,m11,m13,m33,m13,L,mu2) * ( 1./324.*
         RD1456ep*RD3456ep + 1./162.*m33*RC1456pe*RD3456ep - 1./162.*
         m33*RD1456ep*RS34561pe + 1./162.*m33*RD1456ep*RC3456pe + 1./81.
         *pow(m33,2)*RS14563pe*RS34561pe + 1./81.*pow(m33,2)*RC3456pe*
         RS14563pe + 1./162.*pow(m33,2)*RC1456pe*RC3456pe + 2./81.*m13*
         RD3456ep*RS14563pe - 1./81.*m13*RC1456pe*RD3456ep + 2./81.*m13
         *RD1456ep*RS34561pe - 1./81.*m13*RD1456ep*RC3456pe - 2./81.*
         m13*m33*RS14563pe*RS34561pe + 2./81.*m13*m33*RC1456pe*
         RS34561pe - 2./81.*m13*m33*RC1456pe*RC3456pe + 8./81.*pow(
         m13,2)*RS14563pe*RS34561pe - 4./81.*pow(m13,2)*RC3456pe*
         RS14563pe - 4./81.*pow(m13,2)*RC1456pe*RS34561pe + 2./81.*pow(
         m13,2)*RC1456pe*RC3456pe - 1./162.*m11*RD3456ep*RS14563pe + 1./
         162.*m11*RC1456pe*RD3456ep + 1./162.*m11*RD1456ep*RC3456pe + 1.
         /27.*m11*m33*RS14563pe*RS34561pe + 1./81.*m11*m33*RC3456pe*
         RS14563pe + 1./81.*m11*m33*RC1456pe*RS34561pe + 1./81.*m11*m33
         *RC1456pe*RC3456pe );

      F13p6x23RV +=  + hhdV(1,m11,m13,m33,m13,L,mu2) * (  - 2./81.*m11*
         m13*RS14563pe*RS34561pe + 2./81.*m11*m13*RC3456pe*RS14563pe - 
         2./81.*m11*m13*RC1456pe*RC3456pe + 1./81.*pow(m11,2)*RS14563pe
         *RS34561pe + 1./81.*pow(m11,2)*RC1456pe*RS34561pe + 1./162.*
         pow(m11,2)*RC1456pe*RC3456pe );

      F13p6x23RV +=  + hhdV(1,m11,m13,mp2,m13,L,mu2) * (  - 1./162.*mp2
         *RD1456ep*RSp45613e + 1./324.*mp2*RD1456ep*RSp45633e + 1./324.
         *mp2*RSp45611e*RD1456ep + 1./162.*pow(mp2,2)*RSp45613e*
         RS14563pe - 1./324.*pow(mp2,2)*RSp45633e*RS14563pe - 1./324.*
         pow(mp2,2)*RC1456pe*RSp45613e + 1./648.*pow(mp2,2)*RC1456pe*
         RSp45633e - 1./324.*pow(mp2,2)*RSp45611e*RS14563pe + 1./648.*
         pow(mp2,2)*RSp45611e*RC1456pe + 1./324.*m33*RD1456ep*RSp45633e
          - 1./324.*m33*RSp45611e*RD1456ep - 1./162.*m33*mp2*RSp45613e*
         RS14563pe + 1./162.*m33*mp2*RSp45633e*RS14563pe + 1./324.*m33*
         mp2*RC1456pe*RSp45633e - 1./324.*m33*mp2*RSp45611e*RC1456pe + 
         1./81.*pow(m33,2)*RSp45613e*RS14563pe + 1./108.*pow(m33,2)*
         RSp45633e*RS14563pe + 1./324.*pow(m33,2)*RC1456pe*RSp45613e + 
         1./648.*pow(m33,2)*RC1456pe*RSp45633e + 1./324.*pow(m33,2)*
         RSp45611e*RS14563pe + 1./648.*pow(m33,2)*RSp45611e*RC1456pe + 
         2./81.*m13*RD1456ep*RSp45613e - 1./81.*m13*RD1456ep*RSp45633e
          - 1./81.*m13*RSp45611e*RD1456ep );

      F13p6x23RV +=  + hhdV(1,m11,m13,mp2,m13,L,mu2) * (  - 4./81.*m13*
         mp2*RSp45613e*RS14563pe + 2./81.*m13*mp2*RSp45633e*RS14563pe
          + 2./81.*m13*mp2*RC1456pe*RSp45613e - 1./81.*m13*mp2*RC1456pe
         *RSp45633e + 2./81.*m13*mp2*RSp45611e*RS14563pe - 1./81.*m13*
         mp2*RSp45611e*RC1456pe + 2./81.*m13*m33*RSp45613e*RS14563pe - 
         2./81.*m13*m33*RSp45633e*RS14563pe - 1./81.*m13*m33*RC1456pe*
         RSp45633e + 1./81.*m13*m33*RSp45611e*RC1456pe + 8./81.*pow(
         m13,2)*RSp45613e*RS14563pe - 4./81.*pow(m13,2)*RSp45633e*
         RS14563pe - 4./81.*pow(m13,2)*RC1456pe*RSp45613e + 2./81.*pow(
         m13,2)*RC1456pe*RSp45633e - 4./81.*pow(m13,2)*RSp45611e*
         RS14563pe + 2./81.*pow(m13,2)*RSp45611e*RC1456pe + 1./162.*m11
         *RD1456ep*RSp45633e - 1./162.*m11*RSp45611e*RD1456ep + 1./162.
         *m11*mp2*RSp45613e*RS14563pe - 1./162.*m11*mp2*RSp45633e*
         RS14563pe + 1./162.*m11*mp2*RC1456pe*RSp45633e - 1./162.*m11*
         mp2*RSp45611e*RC1456pe + 5./162.*m11*m33*RSp45613e*RS14563pe
          + 1./54.*m11*m33*RSp45633e*RS14563pe );

      F13p6x23RV +=  + hhdV(1,m11,m13,mp2,m13,L,mu2) * ( 1./81.*m11*m33
         *RC1456pe*RSp45613e + 1./162.*m11*m33*RC1456pe*RSp45633e + 1./
         81.*m11*m33*RSp45611e*RS14563pe + 1./162.*m11*m33*RSp45611e*
         RC1456pe - 2./81.*m11*m13*RSp45613e*RS14563pe + 2./81.*m11*m13
         *RSp45633e*RS14563pe - 2./81.*m11*m13*RC1456pe*RSp45633e + 2./
         81.*m11*m13*RSp45611e*RC1456pe + 1./81.*pow(m11,2)*RSp45613e*
         RS14563pe + 1./81.*pow(m11,2)*RC1456pe*RSp45613e + 1./162.*
         pow(m11,2)*RC1456pe*RSp45633e + 1./81.*pow(m11,2)*RSp45611e*
         RS14563pe + 1./162.*pow(m11,2)*RSp45611e*RC1456pe );

      F13p6x23RV +=  + hhdV(1,m11,m13,me2,m13,L,mu2) * (  - 1./162.*me2
         *RD1456ep*RSe45613p + 1./324.*me2*RD1456ep*RSe45633p + 1./324.
         *me2*RSe45611p*RD1456ep + 1./162.*pow(me2,2)*RSe45613p*
         RS14563pe - 1./324.*pow(me2,2)*RSe45633p*RS14563pe - 1./324.*
         pow(me2,2)*RC1456pe*RSe45613p + 1./648.*pow(me2,2)*RC1456pe*
         RSe45633p - 1./324.*pow(me2,2)*RSe45611p*RS14563pe + 1./648.*
         pow(me2,2)*RSe45611p*RC1456pe + 1./324.*m33*RD1456ep*RSe45633p
          - 1./324.*m33*RSe45611p*RD1456ep - 1./162.*m33*me2*RSe45613p*
         RS14563pe + 1./162.*m33*me2*RSe45633p*RS14563pe + 1./324.*m33*
         me2*RC1456pe*RSe45633p - 1./324.*m33*me2*RSe45611p*RC1456pe + 
         1./81.*pow(m33,2)*RSe45613p*RS14563pe + 1./108.*pow(m33,2)*
         RSe45633p*RS14563pe + 1./324.*pow(m33,2)*RC1456pe*RSe45613p + 
         1./648.*pow(m33,2)*RC1456pe*RSe45633p + 1./324.*pow(m33,2)*
         RSe45611p*RS14563pe + 1./648.*pow(m33,2)*RSe45611p*RC1456pe + 
         2./81.*m13*RD1456ep*RSe45613p - 1./81.*m13*RD1456ep*RSe45633p
          - 1./81.*m13*RSe45611p*RD1456ep );

      F13p6x23RV +=  + hhdV(1,m11,m13,me2,m13,L,mu2) * (  - 4./81.*m13*
         me2*RSe45613p*RS14563pe + 2./81.*m13*me2*RSe45633p*RS14563pe
          + 2./81.*m13*me2*RC1456pe*RSe45613p - 1./81.*m13*me2*RC1456pe
         *RSe45633p + 2./81.*m13*me2*RSe45611p*RS14563pe - 1./81.*m13*
         me2*RSe45611p*RC1456pe + 2./81.*m13*m33*RSe45613p*RS14563pe - 
         2./81.*m13*m33*RSe45633p*RS14563pe - 1./81.*m13*m33*RC1456pe*
         RSe45633p + 1./81.*m13*m33*RSe45611p*RC1456pe + 8./81.*pow(
         m13,2)*RSe45613p*RS14563pe - 4./81.*pow(m13,2)*RSe45633p*
         RS14563pe - 4./81.*pow(m13,2)*RC1456pe*RSe45613p + 2./81.*pow(
         m13,2)*RC1456pe*RSe45633p - 4./81.*pow(m13,2)*RSe45611p*
         RS14563pe + 2./81.*pow(m13,2)*RSe45611p*RC1456pe + 1./162.*m11
         *RD1456ep*RSe45633p - 1./162.*m11*RSe45611p*RD1456ep + 1./162.
         *m11*me2*RSe45613p*RS14563pe - 1./162.*m11*me2*RSe45633p*
         RS14563pe + 1./162.*m11*me2*RC1456pe*RSe45633p - 1./162.*m11*
         me2*RSe45611p*RC1456pe + 5./162.*m11*m33*RSe45613p*RS14563pe
          + 1./54.*m11*m33*RSe45633p*RS14563pe );

      F13p6x23RV +=  + hhdV(1,m11,m13,me2,m13,L,mu2) * ( 1./81.*m11*m33
         *RC1456pe*RSe45613p + 1./162.*m11*m33*RC1456pe*RSe45633p + 1./
         81.*m11*m33*RSe45611p*RS14563pe + 1./162.*m11*m33*RSe45611p*
         RC1456pe - 2./81.*m11*m13*RSe45613p*RS14563pe + 2./81.*m11*m13
         *RSe45633p*RS14563pe - 2./81.*m11*m13*RC1456pe*RSe45633p + 2./
         81.*m11*m13*RSe45611p*RC1456pe + 1./81.*pow(m11,2)*RSe45613p*
         RS14563pe + 1./81.*pow(m11,2)*RC1456pe*RSe45613p + 1./162.*
         pow(m11,2)*RC1456pe*RSe45633p + 1./81.*pow(m11,2)*RSe45611p*
         RS14563pe + 1./162.*pow(m11,2)*RSe45611p*RC1456pe );

      F13p6x23RV +=  + hhdV(1,m11,m14,m34,m13,L,mu2) * (  - 1./216.*m44
         *RD1456ep - 1./432.*pow(m44,2)*RS14563pe - 1./864.*pow(m44,2)*
         RC1456pe - 1./216.*pow(m44,2)*RS156pe - 1./108.*m34*RD1456ep
          + 1./216.*m34*m44*RS14563pe - 1./216.*m34*m44*RC1456pe - 1./
         72.*m34*m44*RS156pe + 1./54.*pow(m34,2)*RS14563pe - 1./216.*
         pow(m34,2)*RC1456pe - 1./108.*pow(m34,2)*RS156pe - 1./216.*m33
         *RD1456ep - 1./144.*m33*m44*RS14563pe - 1./432.*m33*m44*
         RC1456pe - 1./144.*m33*m44*RS156pe - 1./216.*m33*m34*RC1456pe
          - 1./108.*m33*m34*RS156pe - 1./216.*pow(m33,2)*RS14563pe - 1./
         864.*pow(m33,2)*RC1456pe - 1./432.*pow(m33,2)*RS156pe + 1./54.
         *m14*RD1456ep + 1./216.*m14*m44*RS14563pe + 1./108.*m14*m44*
         RC1456pe + 1./72.*m14*m44*RS156pe - 5./108.*m14*m34*RS14563pe
          + 1./54.*m14*m34*RC1456pe + 1./108.*m14*m34*RS156pe + 1./72.*
         m14*m33*RS14563pe + 1./108.*m14*m33*RC1456pe + 1./216.*m14*m33
         *RS156pe + 1./54.*pow(m14,2)*RS14563pe - 1./54.*pow(m14,2)*
         RC1456pe );

      F13p6x23RV +=  + hhdV(1,m11,m14,m34,m13,L,mu2) * ( 1./54.*pow(
         m14,2)*RS156pe + 1./54.*m13*RD1456ep - 1./108.*m13*m44*
         RS14563pe + 1./108.*m13*m44*RC1456pe + 1./36.*m13*m44*RS156pe
          - 2./27.*m13*m34*RS14563pe + 1./54.*m13*m34*RC1456pe + 1./27.
         *m13*m34*RS156pe + 1./108.*m13*m33*RC1456pe + 1./54.*m13*m33*
         RS156pe + 5./54.*m13*m14*RS14563pe - 1./27.*m13*m14*RC1456pe
          - 1./54.*m13*m14*RS156pe - 5./54.*pow(m13,2)*RS14563pe - 1./
         54.*pow(m13,2)*RC1456pe + 7./54.*pow(m13,2)*RS156pe - 1./54.*
         m11*RD1456ep - 7./432.*m11*m44*RS14563pe - 1./108.*m11*m44*
         RC1456pe - 5./432.*m11*m44*RS156pe + 5./216.*m11*m34*RS14563pe
          - 1./54.*m11*m34*RC1456pe - 1./216.*m11*m34*RS156pe - 11./432.
         *m11*m33*RS14563pe - 1./108.*m11*m33*RC1456pe - 1./432.*m11*
         m33*RS156pe + 1./108.*m11*m14*RS14563pe + 1./27.*m11*m14*
         RC1456pe - 5./108.*m11*m14*RS156pe - 19./216.*m11*m13*
         RS14563pe + 1./27.*m11*m13*RC1456pe + 11./216.*m11*m13*RS156pe
          - 1./36.*pow(m11,2)*RS14563pe );

      F13p6x23RV +=  + hhdV(1,m11,m14,m34,m13,L,mu2) * (  - 1./54.*pow(
         m11,2)*RC1456pe + 1./36.*pow(m11,2)*RS156pe );

      F13p6x23RV +=  + hhdV(1,m11,m15,m35,m13,L,mu2) * (  - 1./216.*m55
         *RD1456ep - 1./432.*pow(m55,2)*RS14563pe - 1./864.*pow(m55,2)*
         RC1456pe - 1./216.*pow(m55,2)*RS146pe - 1./108.*m35*RD1456ep
          + 1./216.*m35*m55*RS14563pe - 1./216.*m35*m55*RC1456pe - 1./
         72.*m35*m55*RS146pe + 1./54.*pow(m35,2)*RS14563pe - 1./216.*
         pow(m35,2)*RC1456pe - 1./108.*pow(m35,2)*RS146pe - 1./216.*m33
         *RD1456ep - 1./144.*m33*m55*RS14563pe - 1./432.*m33*m55*
         RC1456pe - 1./144.*m33*m55*RS146pe - 1./216.*m33*m35*RC1456pe
          - 1./108.*m33*m35*RS146pe - 1./216.*pow(m33,2)*RS14563pe - 1./
         864.*pow(m33,2)*RC1456pe - 1./432.*pow(m33,2)*RS146pe + 1./54.
         *m15*RD1456ep + 1./216.*m15*m55*RS14563pe + 1./108.*m15*m55*
         RC1456pe + 1./72.*m15*m55*RS146pe - 5./108.*m15*m35*RS14563pe
          + 1./54.*m15*m35*RC1456pe + 1./108.*m15*m35*RS146pe + 1./72.*
         m15*m33*RS14563pe + 1./108.*m15*m33*RC1456pe + 1./216.*m15*m33
         *RS146pe + 1./54.*pow(m15,2)*RS14563pe - 1./54.*pow(m15,2)*
         RC1456pe );

      F13p6x23RV +=  + hhdV(1,m11,m15,m35,m13,L,mu2) * ( 1./54.*pow(
         m15,2)*RS146pe + 1./54.*m13*RD1456ep - 1./108.*m13*m55*
         RS14563pe + 1./108.*m13*m55*RC1456pe + 1./36.*m13*m55*RS146pe
          - 2./27.*m13*m35*RS14563pe + 1./54.*m13*m35*RC1456pe + 1./27.
         *m13*m35*RS146pe + 1./108.*m13*m33*RC1456pe + 1./54.*m13*m33*
         RS146pe + 5./54.*m13*m15*RS14563pe - 1./27.*m13*m15*RC1456pe
          - 1./54.*m13*m15*RS146pe - 5./54.*pow(m13,2)*RS14563pe - 1./
         54.*pow(m13,2)*RC1456pe + 7./54.*pow(m13,2)*RS146pe - 1./54.*
         m11*RD1456ep - 7./432.*m11*m55*RS14563pe - 1./108.*m11*m55*
         RC1456pe - 5./432.*m11*m55*RS146pe + 5./216.*m11*m35*RS14563pe
          - 1./54.*m11*m35*RC1456pe - 1./216.*m11*m35*RS146pe - 11./432.
         *m11*m33*RS14563pe - 1./108.*m11*m33*RC1456pe - 1./432.*m11*
         m33*RS146pe + 1./108.*m11*m15*RS14563pe + 1./27.*m11*m15*
         RC1456pe - 5./108.*m11*m15*RS146pe - 19./216.*m11*m13*
         RS14563pe + 1./27.*m11*m13*RC1456pe + 11./216.*m11*m13*RS146pe
          - 1./36.*pow(m11,2)*RS14563pe );

      F13p6x23RV +=  + hhdV(1,m11,m15,m35,m13,L,mu2) * (  - 1./54.*pow(
         m11,2)*RC1456pe + 1./36.*pow(m11,2)*RS146pe );

      F13p6x23RV +=  + hhdV(1,m11,m16,m36,m13,L,mu2) * (  - 1./216.*m66
         *RD1456ep - 1./432.*pow(m66,2)*RS14563pe - 1./864.*pow(m66,2)*
         RC1456pe - 1./216.*pow(m66,2)*RS145pe - 1./108.*m36*RD1456ep
          + 1./216.*m36*m66*RS14563pe - 1./216.*m36*m66*RC1456pe - 1./
         72.*m36*m66*RS145pe + 1./54.*pow(m36,2)*RS14563pe - 1./216.*
         pow(m36,2)*RC1456pe - 1./108.*pow(m36,2)*RS145pe - 1./216.*m33
         *RD1456ep - 1./144.*m33*m66*RS14563pe - 1./432.*m33*m66*
         RC1456pe - 1./144.*m33*m66*RS145pe - 1./216.*m33*m36*RC1456pe
          - 1./108.*m33*m36*RS145pe - 1./216.*pow(m33,2)*RS14563pe - 1./
         864.*pow(m33,2)*RC1456pe - 1./432.*pow(m33,2)*RS145pe + 1./54.
         *m16*RD1456ep + 1./216.*m16*m66*RS14563pe + 1./108.*m16*m66*
         RC1456pe + 1./72.*m16*m66*RS145pe - 5./108.*m16*m36*RS14563pe
          + 1./54.*m16*m36*RC1456pe + 1./108.*m16*m36*RS145pe + 1./72.*
         m16*m33*RS14563pe + 1./108.*m16*m33*RC1456pe + 1./216.*m16*m33
         *RS145pe + 1./54.*pow(m16,2)*RS14563pe - 1./54.*pow(m16,2)*
         RC1456pe );

      F13p6x23RV +=  + hhdV(1,m11,m16,m36,m13,L,mu2) * ( 1./54.*pow(
         m16,2)*RS145pe + 1./54.*m13*RD1456ep - 1./108.*m13*m66*
         RS14563pe + 1./108.*m13*m66*RC1456pe + 1./36.*m13*m66*RS145pe
          - 2./27.*m13*m36*RS14563pe + 1./54.*m13*m36*RC1456pe + 1./27.
         *m13*m36*RS145pe + 1./108.*m13*m33*RC1456pe + 1./54.*m13*m33*
         RS145pe + 5./54.*m13*m16*RS14563pe - 1./27.*m13*m16*RC1456pe
          - 1./54.*m13*m16*RS145pe - 5./54.*pow(m13,2)*RS14563pe - 1./
         54.*pow(m13,2)*RC1456pe + 7./54.*pow(m13,2)*RS145pe - 1./54.*
         m11*RD1456ep - 7./432.*m11*m66*RS14563pe - 1./108.*m11*m66*
         RC1456pe - 5./432.*m11*m66*RS145pe + 5./216.*m11*m36*RS14563pe
          - 1./54.*m11*m36*RC1456pe - 1./216.*m11*m36*RS145pe - 11./432.
         *m11*m33*RS14563pe - 1./108.*m11*m33*RC1456pe - 1./432.*m11*
         m33*RS145pe + 1./108.*m11*m16*RS14563pe + 1./27.*m11*m16*
         RC1456pe - 5./108.*m11*m16*RS145pe - 19./216.*m11*m13*
         RS14563pe + 1./27.*m11*m13*RC1456pe + 11./216.*m11*m13*RS145pe
          - 1./36.*pow(m11,2)*RS14563pe );

      F13p6x23RV +=  + hhdV(1,m11,m16,m36,m13,L,mu2) * (  - 1./54.*pow(
         m11,2)*RC1456pe + 1./36.*pow(m11,2)*RS145pe );

      F13p6x23RV +=  + hhdV(1,m13,m11,m11,m13,L,mu2) * ( 1./16.*pow(
         m13,2) + 1./36.*pow(m13,2)*pow(RS14563pe,2) - 1./36.*pow(
         m13,2)*RC1456pe*RS14563pe + 1./144.*pow(m13,2)*pow(RC1456pe,2)
          );

      F13p6x23RV +=  + hhdV(1,m13,m11,m33,m13,L,mu2) * ( 1./18.*pow(
         m13,2)*RS14563pe*RS34561pe - 1./36.*pow(m13,2)*RC3456pe*
         RS14563pe - 1./36.*pow(m13,2)*RC1456pe*RS34561pe + 1./72.*pow(
         m13,2)*RC1456pe*RC3456pe );

      F13p6x23RV +=  + hhdV(1,m13,m11,mp2,m13,L,mu2) * ( 1./18.*pow(
         m13,2)*RSp45613e*RS14563pe - 1./36.*pow(m13,2)*RSp45633e*
         RS14563pe - 1./36.*pow(m13,2)*RC1456pe*RSp45613e + 1./72.*pow(
         m13,2)*RC1456pe*RSp45633e - 1./36.*pow(m13,2)*RSp45611e*
         RS14563pe + 1./72.*pow(m13,2)*RSp45611e*RC1456pe );

      F13p6x23RV +=  + hhdV(1,m13,m11,me2,m13,L,mu2) * ( 1./18.*pow(
         m13,2)*RSe45613p*RS14563pe - 1./36.*pow(m13,2)*RSe45633p*
         RS14563pe - 1./36.*pow(m13,2)*RC1456pe*RSe45613p + 1./72.*pow(
         m13,2)*RC1456pe*RSe45633p - 1./36.*pow(m13,2)*RSe45611p*
         RS14563pe + 1./72.*pow(m13,2)*RSe45611p*RC1456pe );

      F13p6x23RV +=  + hhdV(1,m13,m13,m13,m13,L,mu2) * ( 1./72.*pow(
         m33,2) + 5./72.*pow(m13,2) + 1./36.*m11*m33 + 1./72.*pow(
         m11,2) );

      F13p6x23RV +=  + hhdV(1,m13,m33,m33,m13,L,mu2) * ( 1./648.*pow(
         RD3456ep,2) - 1./162.*m33*RD3456ep*RS34561pe - 1./324.*m33*
         RD3456ep*RC3456pe + 1./288.*pow(m33,2) + 1./81.*pow(m33,2)*
         pow(RS34561pe,2) + 1./108.*pow(m33,2)*RC3456pe*RS34561pe + 1./
         1296.*pow(m33,2)*pow(RC3456pe,2) + 2./81.*m13*RD3456ep*
         RS34561pe - 1./81.*m13*RD3456ep*RC3456pe + 1./36.*m13*m33 - 1./
         27.*m13*m33*pow(RS34561pe,2) + 2./81.*m13*m33*RC3456pe*
         RS34561pe + 1./162.*m13*m33*pow(RC3456pe,2) + 35./144.*pow(
         m13,2) + 25./324.*pow(m13,2)*pow(RS34561pe,2) - 25./324.*pow(
         m13,2)*RC3456pe*RS34561pe + 25./1296.*pow(m13,2)*pow(
         RC3456pe,2) - 1./324.*m11*RD3456ep*RC3456pe + 1./144.*m11*m33
          + 1./81.*m11*m33*pow(RS34561pe,2) + 1./81.*m11*m33*RC3456pe*
         RS34561pe + 1./648.*m11*m33*pow(RC3456pe,2) + 1./36.*m11*m13
          + 1./81.*m11*m13*pow(RS34561pe,2) + 1./162.*m11*m13*pow(
         RC3456pe,2) + 1./288.*pow(m11,2) + 1./162.*pow(m11,2)*pow(
         RS34561pe,2) );

      F13p6x23RV +=  + hhdV(1,m13,m33,m33,m13,L,mu2) * ( 1./324.*pow(
         m11,2)*RC3456pe*RS34561pe + 1./1296.*pow(m11,2)*pow(
         RC3456pe,2) );

      F13p6x23RV +=  + hhdV(1,m13,m33,mp2,m13,L,mu2) * (  - 1./162.*mp2
         *RD3456ep*RSp45613e + 1./324.*mp2*RSp45633e*RD3456ep + 1./324.
         *mp2*RSp45611e*RD3456ep + 1./162.*pow(mp2,2)*RSp45613e*
         RS34561pe - 1./324.*pow(mp2,2)*RC3456pe*RSp45613e - 1./324.*
         pow(mp2,2)*RSp45633e*RS34561pe + 1./648.*pow(mp2,2)*RSp45633e*
         RC3456pe - 1./324.*pow(mp2,2)*RSp45611e*RS34561pe + 1./648.*
         pow(mp2,2)*RSp45611e*RC3456pe - 1./162.*m33*RSp45633e*RD3456ep
          + 1./162.*m33*RSp45611e*RD3456ep + 1./162.*m33*mp2*RSp45613e*
         RS34561pe - 1./162.*m33*mp2*RSp45633e*RC3456pe - 1./162.*m33*
         mp2*RSp45611e*RS34561pe + 1./162.*m33*mp2*RSp45611e*RC3456pe
          + 1./81.*pow(m33,2)*RSp45613e*RS34561pe + 1./81.*pow(m33,2)*
         RC3456pe*RSp45613e + 1./81.*pow(m33,2)*RSp45633e*RS34561pe + 1.
         /162.*pow(m33,2)*RSp45633e*RC3456pe + 1./162.*pow(m33,2)*
         RSp45611e*RC3456pe + 2./81.*m13*RD3456ep*RSp45613e - 1./81.*
         m13*RSp45633e*RD3456ep - 1./81.*m13*RSp45611e*RD3456ep - 4./81.
         *m13*mp2*RSp45613e*RS34561pe );

      F13p6x23RV +=  + hhdV(1,m13,m33,mp2,m13,L,mu2) * ( 2./81.*m13*mp2
         *RC3456pe*RSp45613e + 2./81.*m13*mp2*RSp45633e*RS34561pe - 1./
         81.*m13*mp2*RSp45633e*RC3456pe + 2./81.*m13*mp2*RSp45611e*
         RS34561pe - 1./81.*m13*mp2*RSp45611e*RC3456pe - 2./81.*m13*m33
         *RSp45613e*RS34561pe + 2./81.*m13*m33*RSp45633e*RC3456pe + 2./
         81.*m13*m33*RSp45611e*RS34561pe - 2./81.*m13*m33*RSp45611e*
         RC3456pe + 25./162.*pow(m13,2)*RSp45613e*RS34561pe - 25./324.*
         pow(m13,2)*RC3456pe*RSp45613e - 25./324.*pow(m13,2)*RSp45633e*
         RS34561pe + 25./648.*pow(m13,2)*RSp45633e*RC3456pe - 25./324.*
         pow(m13,2)*RSp45611e*RS34561pe + 25./648.*pow(m13,2)*RSp45611e
         *RC3456pe - 1./324.*m11*RSp45633e*RD3456ep + 1./324.*m11*
         RSp45611e*RD3456ep - 1./162.*m11*mp2*RSp45613e*RS34561pe - 1./
         324.*m11*mp2*RSp45633e*RC3456pe + 1./162.*m11*mp2*RSp45611e*
         RS34561pe + 1./324.*m11*mp2*RSp45611e*RC3456pe + 5./162.*m11*
         m33*RSp45613e*RS34561pe + 1./81.*m11*m33*RC3456pe*RSp45613e + 
         1./81.*m11*m33*RSp45633e*RS34561pe );

      F13p6x23RV +=  + hhdV(1,m13,m33,mp2,m13,L,mu2) * ( 1./162.*m11*
         m33*RSp45633e*RC3456pe + 1./54.*m11*m33*RSp45611e*RS34561pe + 
         1./162.*m11*m33*RSp45611e*RC3456pe + 2./81.*m11*m13*RSp45613e*
         RS34561pe + 1./81.*m11*m13*RSp45633e*RC3456pe - 2./81.*m11*m13
         *RSp45611e*RS34561pe - 1./81.*m11*m13*RSp45611e*RC3456pe + 1./
         81.*pow(m11,2)*RSp45613e*RS34561pe + 1./324.*pow(m11,2)*
         RC3456pe*RSp45613e + 1./324.*pow(m11,2)*RSp45633e*RS34561pe + 
         1./648.*pow(m11,2)*RSp45633e*RC3456pe + 1./108.*pow(m11,2)*
         RSp45611e*RS34561pe + 1./648.*pow(m11,2)*RSp45611e*RC3456pe );

      F13p6x23RV +=  + hhdV(1,m13,m33,me2,m13,L,mu2) * (  - 1./162.*me2
         *RD3456ep*RSe45613p + 1./324.*me2*RSe45633p*RD3456ep + 1./324.
         *me2*RSe45611p*RD3456ep + 1./162.*pow(me2,2)*RSe45613p*
         RS34561pe - 1./324.*pow(me2,2)*RC3456pe*RSe45613p - 1./324.*
         pow(me2,2)*RSe45633p*RS34561pe + 1./648.*pow(me2,2)*RSe45633p*
         RC3456pe - 1./324.*pow(me2,2)*RSe45611p*RS34561pe + 1./648.*
         pow(me2,2)*RSe45611p*RC3456pe - 1./162.*m33*RSe45633p*RD3456ep
          + 1./162.*m33*RSe45611p*RD3456ep + 1./162.*m33*me2*RSe45613p*
         RS34561pe - 1./162.*m33*me2*RSe45633p*RC3456pe - 1./162.*m33*
         me2*RSe45611p*RS34561pe + 1./162.*m33*me2*RSe45611p*RC3456pe
          + 1./81.*pow(m33,2)*RSe45613p*RS34561pe + 1./81.*pow(m33,2)*
         RC3456pe*RSe45613p + 1./81.*pow(m33,2)*RSe45633p*RS34561pe + 1.
         /162.*pow(m33,2)*RSe45633p*RC3456pe + 1./162.*pow(m33,2)*
         RSe45611p*RC3456pe + 2./81.*m13*RD3456ep*RSe45613p - 1./81.*
         m13*RSe45633p*RD3456ep - 1./81.*m13*RSe45611p*RD3456ep - 4./81.
         *m13*me2*RSe45613p*RS34561pe );

      F13p6x23RV +=  + hhdV(1,m13,m33,me2,m13,L,mu2) * ( 2./81.*m13*me2
         *RC3456pe*RSe45613p + 2./81.*m13*me2*RSe45633p*RS34561pe - 1./
         81.*m13*me2*RSe45633p*RC3456pe + 2./81.*m13*me2*RSe45611p*
         RS34561pe - 1./81.*m13*me2*RSe45611p*RC3456pe - 2./81.*m13*m33
         *RSe45613p*RS34561pe + 2./81.*m13*m33*RSe45633p*RC3456pe + 2./
         81.*m13*m33*RSe45611p*RS34561pe - 2./81.*m13*m33*RSe45611p*
         RC3456pe + 25./162.*pow(m13,2)*RSe45613p*RS34561pe - 25./324.*
         pow(m13,2)*RC3456pe*RSe45613p - 25./324.*pow(m13,2)*RSe45633p*
         RS34561pe + 25./648.*pow(m13,2)*RSe45633p*RC3456pe - 25./324.*
         pow(m13,2)*RSe45611p*RS34561pe + 25./648.*pow(m13,2)*RSe45611p
         *RC3456pe - 1./324.*m11*RSe45633p*RD3456ep + 1./324.*m11*
         RSe45611p*RD3456ep - 1./162.*m11*me2*RSe45613p*RS34561pe - 1./
         324.*m11*me2*RSe45633p*RC3456pe + 1./162.*m11*me2*RSe45611p*
         RS34561pe + 1./324.*m11*me2*RSe45611p*RC3456pe + 5./162.*m11*
         m33*RSe45613p*RS34561pe + 1./81.*m11*m33*RC3456pe*RSe45613p + 
         1./81.*m11*m33*RSe45633p*RS34561pe );

      F13p6x23RV +=  + hhdV(1,m13,m33,me2,m13,L,mu2) * ( 1./162.*m11*
         m33*RSe45633p*RC3456pe + 1./54.*m11*m33*RSe45611p*RS34561pe + 
         1./162.*m11*m33*RSe45611p*RC3456pe + 2./81.*m11*m13*RSe45613p*
         RS34561pe + 1./81.*m11*m13*RSe45633p*RC3456pe - 2./81.*m11*m13
         *RSe45611p*RS34561pe - 1./81.*m11*m13*RSe45611p*RC3456pe + 1./
         81.*pow(m11,2)*RSe45613p*RS34561pe + 1./324.*pow(m11,2)*
         RC3456pe*RSe45613p + 1./324.*pow(m11,2)*RSe45633p*RS34561pe + 
         1./648.*pow(m11,2)*RSe45633p*RC3456pe + 1./108.*pow(m11,2)*
         RSe45611p*RS34561pe + 1./648.*pow(m11,2)*RSe45611p*RC3456pe );

      F13p6x23RV +=  + hhdV(1,m13,mp2,mp2,m13,L,mu2) * ( 1./81.*pow(
         mp2,2)*pow(RSp45613e,2) - 1./81.*pow(mp2,2)*RSp45633e*
         RSp45613e + 1./324.*pow(mp2,2)*pow(RSp45633e,2) - 1./81.*pow(
         mp2,2)*RSp45611e*RSp45613e + 1./162.*pow(mp2,2)*RSp45611e*
         RSp45633e + 1./324.*pow(mp2,2)*pow(RSp45611e,2) - 1./162.*m33*
         mp2*pow(RSp45613e,2) + 1./81.*m33*mp2*RSp45633e*RSp45613e - 1./
         108.*m33*mp2*pow(RSp45633e,2) + 1./162.*m33*mp2*RSp45611e*
         RSp45633e - 1./324.*m33*mp2*pow(RSp45611e,2) + 1./162.*pow(
         m33,2)*pow(RSp45613e,2) + 1./108.*pow(m33,2)*RSp45633e*
         RSp45613e + 1./144.*pow(m33,2)*pow(RSp45633e,2) + 1./324.*pow(
         m33,2)*RSp45611e*RSp45613e + 1./648.*pow(m33,2)*RSp45611e*
         RSp45633e + 1./1296.*pow(m33,2)*pow(RSp45611e,2) - 4./81.*m13*
         mp2*pow(RSp45613e,2) + 4./81.*m13*mp2*RSp45633e*RSp45613e - 1./
         81.*m13*mp2*pow(RSp45633e,2) + 4./81.*m13*mp2*RSp45611e*
         RSp45613e - 2./81.*m13*mp2*RSp45611e*RSp45633e - 1./81.*m13*
         mp2*pow(RSp45611e,2) );

      F13p6x23RV +=  + hhdV(1,m13,mp2,mp2,m13,L,mu2) * ( 1./81.*m13*m33
         *pow(RSp45613e,2) - 2./81.*m13*m33*RSp45633e*RSp45613e + 1./54.
         *m13*m33*pow(RSp45633e,2) - 1./81.*m13*m33*RSp45611e*RSp45633e
          + 1./162.*m13*m33*pow(RSp45611e,2) + 25./324.*pow(m13,2)*pow(
         RSp45613e,2) - 25./324.*pow(m13,2)*RSp45633e*RSp45613e + 25./
         1296.*pow(m13,2)*pow(RSp45633e,2) - 25./324.*pow(m13,2)*
         RSp45611e*RSp45613e + 25./648.*pow(m13,2)*RSp45611e*RSp45633e
          + 25./1296.*pow(m13,2)*pow(RSp45611e,2) - 1./162.*m11*mp2*
         pow(RSp45613e,2) - 1./324.*m11*mp2*pow(RSp45633e,2) + 1./81.*
         m11*mp2*RSp45611e*RSp45613e + 1./162.*m11*mp2*RSp45611e*
         RSp45633e - 1./108.*m11*mp2*pow(RSp45611e,2) + 1./54.*m11*m33*
         pow(RSp45613e,2) + 1./81.*m11*m33*RSp45633e*RSp45613e + 1./216.
         *m11*m33*pow(RSp45633e,2) + 1./81.*m11*m33*RSp45611e*RSp45613e
          + 1./324.*m11*m33*RSp45611e*RSp45633e + 1./216.*m11*m33*pow(
         RSp45611e,2) + 1./81.*m11*m13*pow(RSp45613e,2) + 1./162.*m11*
         m13*pow(RSp45633e,2) );

      F13p6x23RV +=  + hhdV(1,m13,mp2,mp2,m13,L,mu2) * (  - 2./81.*m11*
         m13*RSp45611e*RSp45613e - 1./81.*m11*m13*RSp45611e*RSp45633e
          + 1./54.*m11*m13*pow(RSp45611e,2) + 1./162.*pow(m11,2)*pow(
         RSp45613e,2) + 1./324.*pow(m11,2)*RSp45633e*RSp45613e + 1./
         1296.*pow(m11,2)*pow(RSp45633e,2) + 1./108.*pow(m11,2)*
         RSp45611e*RSp45613e + 1./648.*pow(m11,2)*RSp45611e*RSp45633e
          + 1./144.*pow(m11,2)*pow(RSp45611e,2) );

      F13p6x23RV +=  + hhdV(1,m13,me2,mp2,m13,L,mu2) * ( 1./162.*pow(
         me2,2)*RSp45613e*RSe45613p - 1./324.*pow(me2,2)*RSe45633p*
         RSp45613e - 1./324.*pow(me2,2)*RSp45633e*RSe45613p + 1./648.*
         pow(me2,2)*RSp45633e*RSe45633p - 1./324.*pow(me2,2)*RSe45611p*
         RSp45613e + 1./648.*pow(me2,2)*RSe45611p*RSp45633e - 1./324.*
         pow(me2,2)*RSp45611e*RSe45613p + 1./648.*pow(me2,2)*RSp45611e*
         RSe45633p + 1./648.*pow(me2,2)*RSp45611e*RSe45611p + 1./81.*
         mp2*me2*RSp45613e*RSe45613p - 1./162.*mp2*me2*RSe45633p*
         RSp45613e - 1./162.*mp2*me2*RSp45633e*RSe45613p + 1./324.*mp2*
         me2*RSp45633e*RSe45633p - 1./162.*mp2*me2*RSe45611p*RSp45613e
          + 1./324.*mp2*me2*RSe45611p*RSp45633e - 1./162.*mp2*me2*
         RSp45611e*RSe45613p + 1./324.*mp2*me2*RSp45611e*RSe45633p + 1./
         324.*mp2*me2*RSp45611e*RSe45611p + 1./162.*pow(mp2,2)*
         RSp45613e*RSe45613p - 1./324.*pow(mp2,2)*RSe45633p*RSp45613e
          - 1./324.*pow(mp2,2)*RSp45633e*RSe45613p + 1./648.*pow(mp2,2)
         *RSp45633e*RSe45633p );

      F13p6x23RV +=  + hhdV(1,m13,me2,mp2,m13,L,mu2) * (  - 1./324.*
         pow(mp2,2)*RSe45611p*RSp45613e + 1./648.*pow(mp2,2)*RSe45611p*
         RSp45633e - 1./324.*pow(mp2,2)*RSp45611e*RSe45613p + 1./648.*
         pow(mp2,2)*RSp45611e*RSe45633p + 1./648.*pow(mp2,2)*RSp45611e*
         RSe45611p - 1./162.*m33*me2*RSp45613e*RSe45613p + 1./162.*m33*
         me2*RSe45633p*RSp45613e + 1./162.*m33*me2*RSp45633e*RSe45613p
          - 1./108.*m33*me2*RSp45633e*RSe45633p + 1./324.*m33*me2*
         RSe45611p*RSp45633e + 1./324.*m33*me2*RSp45611e*RSe45633p - 1./
         324.*m33*me2*RSp45611e*RSe45611p - 1./162.*m33*mp2*RSp45613e*
         RSe45613p + 1./162.*m33*mp2*RSe45633p*RSp45613e + 1./162.*m33*
         mp2*RSp45633e*RSe45613p - 1./108.*m33*mp2*RSp45633e*RSe45633p
          + 1./324.*m33*mp2*RSe45611p*RSp45633e + 1./324.*m33*mp2*
         RSp45611e*RSe45633p - 1./324.*m33*mp2*RSp45611e*RSe45611p + 1./
         81.*pow(m33,2)*RSp45613e*RSe45613p + 1./108.*pow(m33,2)*
         RSe45633p*RSp45613e + 1./108.*pow(m33,2)*RSp45633e*RSe45613p
          + 1./72.*pow(m33,2)*RSp45633e*RSe45633p );

      F13p6x23RV +=  + hhdV(1,m13,me2,mp2,m13,L,mu2) * ( 1./324.*pow(
         m33,2)*RSe45611p*RSp45613e + 1./648.*pow(m33,2)*RSe45611p*
         RSp45633e + 1./324.*pow(m33,2)*RSp45611e*RSe45613p + 1./648.*
         pow(m33,2)*RSp45611e*RSe45633p + 1./648.*pow(m33,2)*RSp45611e*
         RSe45611p - 4./81.*m13*me2*RSp45613e*RSe45613p + 2./81.*m13*
         me2*RSe45633p*RSp45613e + 2./81.*m13*me2*RSp45633e*RSe45613p
          - 1./81.*m13*me2*RSp45633e*RSe45633p + 2./81.*m13*me2*
         RSe45611p*RSp45613e - 1./81.*m13*me2*RSe45611p*RSp45633e + 2./
         81.*m13*me2*RSp45611e*RSe45613p - 1./81.*m13*me2*RSp45611e*
         RSe45633p - 1./81.*m13*me2*RSp45611e*RSe45611p - 4./81.*m13*
         mp2*RSp45613e*RSe45613p + 2./81.*m13*mp2*RSe45633p*RSp45613e
          + 2./81.*m13*mp2*RSp45633e*RSe45613p - 1./81.*m13*mp2*
         RSp45633e*RSe45633p + 2./81.*m13*mp2*RSe45611p*RSp45613e - 1./
         81.*m13*mp2*RSe45611p*RSp45633e + 2./81.*m13*mp2*RSp45611e*
         RSe45613p - 1./81.*m13*mp2*RSp45611e*RSe45633p - 1./81.*m13*
         mp2*RSp45611e*RSe45611p );

      F13p6x23RV +=  + hhdV(1,m13,me2,mp2,m13,L,mu2) * ( 2./81.*m13*m33
         *RSp45613e*RSe45613p - 2./81.*m13*m33*RSe45633p*RSp45613e - 2./
         81.*m13*m33*RSp45633e*RSe45613p + 1./27.*m13*m33*RSp45633e*
         RSe45633p - 1./81.*m13*m33*RSe45611p*RSp45633e - 1./81.*m13*
         m33*RSp45611e*RSe45633p + 1./81.*m13*m33*RSp45611e*RSe45611p
          + 25./162.*pow(m13,2)*RSp45613e*RSe45613p - 25./324.*pow(
         m13,2)*RSe45633p*RSp45613e - 25./324.*pow(m13,2)*RSp45633e*
         RSe45613p + 25./648.*pow(m13,2)*RSp45633e*RSe45633p - 25./324.
         *pow(m13,2)*RSe45611p*RSp45613e + 25./648.*pow(m13,2)*
         RSe45611p*RSp45633e - 25./324.*pow(m13,2)*RSp45611e*RSe45613p
          + 25./648.*pow(m13,2)*RSp45611e*RSe45633p + 25./648.*pow(
         m13,2)*RSp45611e*RSe45611p - 1./162.*m11*me2*RSp45613e*
         RSe45613p - 1./324.*m11*me2*RSp45633e*RSe45633p + 1./162.*m11*
         me2*RSe45611p*RSp45613e + 1./324.*m11*me2*RSe45611p*RSp45633e
          + 1./162.*m11*me2*RSp45611e*RSe45613p + 1./324.*m11*me2*
         RSp45611e*RSe45633p );

      F13p6x23RV +=  + hhdV(1,m13,me2,mp2,m13,L,mu2) * (  - 1./108.*m11
         *me2*RSp45611e*RSe45611p - 1./162.*m11*mp2*RSp45613e*RSe45613p
          - 1./324.*m11*mp2*RSp45633e*RSe45633p + 1./162.*m11*mp2*
         RSe45611p*RSp45613e + 1./324.*m11*mp2*RSe45611p*RSp45633e + 1./
         162.*m11*mp2*RSp45611e*RSe45613p + 1./324.*m11*mp2*RSp45611e*
         RSe45633p - 1./108.*m11*mp2*RSp45611e*RSe45611p + 1./27.*m11*
         m33*RSp45613e*RSe45613p + 1./81.*m11*m33*RSe45633p*RSp45613e
          + 1./81.*m11*m33*RSp45633e*RSe45613p + 1./108.*m11*m33*
         RSp45633e*RSe45633p + 1./81.*m11*m33*RSe45611p*RSp45613e + 1./
         324.*m11*m33*RSe45611p*RSp45633e + 1./81.*m11*m33*RSp45611e*
         RSe45613p + 1./324.*m11*m33*RSp45611e*RSe45633p + 1./108.*m11*
         m33*RSp45611e*RSe45611p + 2./81.*m11*m13*RSp45613e*RSe45613p
          + 1./81.*m11*m13*RSp45633e*RSe45633p - 2./81.*m11*m13*
         RSe45611p*RSp45613e - 1./81.*m11*m13*RSe45611p*RSp45633e - 2./
         81.*m11*m13*RSp45611e*RSe45613p - 1./81.*m11*m13*RSp45611e*
         RSe45633p );

      F13p6x23RV +=  + hhdV(1,m13,me2,mp2,m13,L,mu2) * ( 1./27.*m11*m13
         *RSp45611e*RSe45611p + 1./81.*pow(m11,2)*RSp45613e*RSe45613p
          + 1./324.*pow(m11,2)*RSe45633p*RSp45613e + 1./324.*pow(m11,2)
         *RSp45633e*RSe45613p + 1./648.*pow(m11,2)*RSp45633e*RSe45633p
          + 1./108.*pow(m11,2)*RSe45611p*RSp45613e + 1./648.*pow(m11,2)
         *RSe45611p*RSp45633e + 1./108.*pow(m11,2)*RSp45611e*RSe45613p
          + 1./648.*pow(m11,2)*RSp45611e*RSe45633p + 1./72.*pow(m11,2)*
         RSp45611e*RSe45611p );

      F13p6x23RV +=  + hhdV(1,m13,me2,me2,m13,L,mu2) * ( 1./81.*pow(
         me2,2)*pow(RSe45613p,2) - 1./81.*pow(me2,2)*RSe45633p*
         RSe45613p + 1./324.*pow(me2,2)*pow(RSe45633p,2) - 1./81.*pow(
         me2,2)*RSe45611p*RSe45613p + 1./162.*pow(me2,2)*RSe45611p*
         RSe45633p + 1./324.*pow(me2,2)*pow(RSe45611p,2) - 1./162.*m33*
         me2*pow(RSe45613p,2) + 1./81.*m33*me2*RSe45633p*RSe45613p - 1./
         108.*m33*me2*pow(RSe45633p,2) + 1./162.*m33*me2*RSe45611p*
         RSe45633p - 1./324.*m33*me2*pow(RSe45611p,2) + 1./162.*pow(
         m33,2)*pow(RSe45613p,2) + 1./108.*pow(m33,2)*RSe45633p*
         RSe45613p + 1./144.*pow(m33,2)*pow(RSe45633p,2) + 1./324.*pow(
         m33,2)*RSe45611p*RSe45613p + 1./648.*pow(m33,2)*RSe45611p*
         RSe45633p + 1./1296.*pow(m33,2)*pow(RSe45611p,2) - 4./81.*m13*
         me2*pow(RSe45613p,2) + 4./81.*m13*me2*RSe45633p*RSe45613p - 1./
         81.*m13*me2*pow(RSe45633p,2) + 4./81.*m13*me2*RSe45611p*
         RSe45613p - 2./81.*m13*me2*RSe45611p*RSe45633p - 1./81.*m13*
         me2*pow(RSe45611p,2) );

      F13p6x23RV +=  + hhdV(1,m13,me2,me2,m13,L,mu2) * ( 1./81.*m13*m33
         *pow(RSe45613p,2) - 2./81.*m13*m33*RSe45633p*RSe45613p + 1./54.
         *m13*m33*pow(RSe45633p,2) - 1./81.*m13*m33*RSe45611p*RSe45633p
          + 1./162.*m13*m33*pow(RSe45611p,2) + 25./324.*pow(m13,2)*pow(
         RSe45613p,2) - 25./324.*pow(m13,2)*RSe45633p*RSe45613p + 25./
         1296.*pow(m13,2)*pow(RSe45633p,2) - 25./324.*pow(m13,2)*
         RSe45611p*RSe45613p + 25./648.*pow(m13,2)*RSe45611p*RSe45633p
          + 25./1296.*pow(m13,2)*pow(RSe45611p,2) - 1./162.*m11*me2*
         pow(RSe45613p,2) - 1./324.*m11*me2*pow(RSe45633p,2) + 1./81.*
         m11*me2*RSe45611p*RSe45613p + 1./162.*m11*me2*RSe45611p*
         RSe45633p - 1./108.*m11*me2*pow(RSe45611p,2) + 1./54.*m11*m33*
         pow(RSe45613p,2) + 1./81.*m11*m33*RSe45633p*RSe45613p + 1./216.
         *m11*m33*pow(RSe45633p,2) + 1./81.*m11*m33*RSe45611p*RSe45613p
          + 1./324.*m11*m33*RSe45611p*RSe45633p + 1./216.*m11*m33*pow(
         RSe45611p,2) + 1./81.*m11*m13*pow(RSe45613p,2) + 1./162.*m11*
         m13*pow(RSe45633p,2) );

      F13p6x23RV +=  + hhdV(1,m13,me2,me2,m13,L,mu2) * (  - 2./81.*m11*
         m13*RSe45611p*RSe45613p - 1./81.*m11*m13*RSe45611p*RSe45633p
          + 1./54.*m11*m13*pow(RSe45611p,2) + 1./162.*pow(m11,2)*pow(
         RSe45613p,2) + 1./324.*pow(m11,2)*RSe45633p*RSe45613p + 1./
         1296.*pow(m11,2)*pow(RSe45633p,2) + 1./108.*pow(m11,2)*
         RSe45611p*RSe45613p + 1./648.*pow(m11,2)*RSe45611p*RSe45633p
          + 1./144.*pow(m11,2)*pow(RSe45611p,2) );

      F13p6x23RV +=  + hhdV(1,m14,m11,m34,m13,L,mu2) * ( 1./24.*m13*m14
         *RS14563pe - 1./24.*m13*m14*RC1456pe + 1./24.*m13*m14*RS156pe
          );

      F13p6x23RV +=  + hhdV(1,m14,m33,m34,m13,L,mu2) * (  - 1./216.*m44
         *RD3456ep - 1./432.*pow(m44,2)*RS34561pe - 1./864.*pow(m44,2)*
         RC3456pe - 1./216.*pow(m44,2)*RS356pe + 1./54.*m34*RD3456ep + 
         1./216.*m34*m44*RS34561pe + 1./108.*m34*m44*RC3456pe + 1./72.*
         m34*m44*RS356pe + 1./54.*pow(m34,2)*RS34561pe - 1./54.*pow(
         m34,2)*RC3456pe + 1./54.*pow(m34,2)*RS356pe - 1./54.*m33*
         RD3456ep - 7./432.*m33*m44*RS34561pe - 1./108.*m33*m44*
         RC3456pe - 5./432.*m33*m44*RS356pe + 1./108.*m33*m34*RS34561pe
          + 1./27.*m33*m34*RC3456pe - 5./108.*m33*m34*RS356pe - 1./36.*
         pow(m33,2)*RS34561pe - 1./54.*pow(m33,2)*RC3456pe + 1./36.*
         pow(m33,2)*RS356pe - 1./108.*m14*RD3456ep + 1./216.*m14*m44*
         RS34561pe - 1./216.*m14*m44*RC3456pe - 1./72.*m14*m44*RS356pe
          - 5./108.*m14*m34*RS34561pe + 1./54.*m14*m34*RC3456pe + 1./
         108.*m14*m34*RS356pe + 5./216.*m14*m33*RS34561pe - 1./54.*m14*
         m33*RC3456pe - 1./216.*m14*m33*RS356pe + 1./54.*pow(m14,2)*
         RS34561pe );

      F13p6x23RV +=  + hhdV(1,m14,m33,m34,m13,L,mu2) * (  - 1./216.*
         pow(m14,2)*RC3456pe - 1./108.*pow(m14,2)*RS356pe - 1./27.*m13*
         RD3456ep - 1./108.*m13*m44*RS34561pe - 1./54.*m13*m44*RC3456pe
          - 1./36.*m13*m44*RS356pe - 2./27.*m13*m34*RS34561pe + 2./27.*
         m13*m34*RC3456pe - 2./27.*m13*m34*RS356pe - 1./54.*m13*m33*
         RS34561pe - 2./27.*m13*m33*RC3456pe + 5./54.*m13*m33*RS356pe
          + 29./216.*m13*m14*RS34561pe - 1./27.*m13*m14*RC3456pe - 13./
         216.*m13*m14*RS356pe - 5./54.*pow(m13,2)*RS34561pe + 5./54.*
         pow(m13,2)*RC3456pe - 5./54.*pow(m13,2)*RS356pe - 1./216.*m11*
         RD3456ep - 1./144.*m11*m44*RS34561pe - 1./432.*m11*m44*
         RC3456pe - 1./144.*m11*m44*RS356pe + 1./72.*m11*m34*RS34561pe
          + 1./108.*m11*m34*RC3456pe + 1./216.*m11*m34*RS356pe - 11./
         432.*m11*m33*RS34561pe - 1./108.*m11*m33*RC3456pe - 1./432.*
         m11*m33*RS356pe - 1./216.*m11*m14*RC3456pe - 1./108.*m11*m14*
         RS356pe - 1./36.*m11*m13*RS34561pe - 1./54.*m11*m13*RC3456pe
          - 1./108.*m11*m13*RS356pe );

      F13p6x23RV +=  + hhdV(1,m14,m33,m34,m13,L,mu2) * (  - 1./216.*
         pow(m11,2)*RS34561pe - 1./864.*pow(m11,2)*RC3456pe - 1./432.*
         pow(m11,2)*RS356pe );

      F13p6x23RV +=  + hhdV(1,m14,m34,m44,m13,L,mu2) * ( 1./72.*pow(
         m44,2) - 1./216.*pow(m44,2)*RS456pe - 1./36.*m34*m44 + 1./108.
         *m34*m44*RS456pe + 1./72.*pow(m34,2) - 1./216.*pow(m34,2)*
         RS456pe - 1./72.*m33*m44 + 1./216.*m33*m44*RS456pe + 1./72.*
         m33*m34 - 1./216.*m33*m34*RS456pe + 1./288.*pow(m33,2) - 1./
         864.*pow(m33,2)*RS456pe - 1./36.*m14*m44 + 1./108.*m14*m44*
         RS456pe + 1./36.*m14*m34 - 1./108.*m14*m34*RS456pe + 1./72.*
         m14*m33 - 1./216.*m14*m33*RS456pe + 1./72.*pow(m14,2) - 1./216.
         *pow(m14,2)*RS456pe - 1./9.*m13*m44 + 1./27.*m13*m44*RS456pe
          + 1./9.*m13*m34 - 1./27.*m13*m34*RS456pe + 1./18.*m13*m33 - 1.
         /54.*m13*m33*RS456pe + 1./9.*m13*m14 - 1./27.*m13*m14*RS456pe
          - 5./18.*pow(m13,2) + 5./54.*pow(m13,2)*RS456pe - 1./72.*m11*
         m44 + 1./216.*m11*m44*RS456pe + 1./72.*m11*m34 - 1./216.*m11*
         m34*RS456pe + 1./144.*m11*m33 - 1./432.*m11*m33*RS456pe + 1./
         72.*m11*m14 - 1./216.*m11*m14*RS456pe + 1./18.*m11*m13 - 1./54.
         *m11*m13*RS456pe );

      F13p6x23RV +=  + hhdV(1,m14,m34,m44,m13,L,mu2) * ( 1./288.*pow(
         m11,2) - 1./864.*pow(m11,2)*RS456pe );

      F13p6x23RV +=  + hhdV(1,m14,m34,mp2,m13,L,mu2) * (  - 1./108.*
         pow(mp2,2)*RSp45613e - 1./216.*pow(mp2,2)*RSp45633e + 1./54.*
         pow(mp2,2)*RSp563e - 1./216.*pow(mp2,2)*RSp45611e + 1./54.*
         pow(mp2,2)*RSp561e - 1./54.*pow(mp2,2)*RSp564e - 1./108.*m44*
         mp2*RSp45613e - 1./216.*m44*mp2*RSp45633e - 1./216.*m44*mp2*
         RSp45611e + 1./54.*m44*mp2*RSp564e - 1./432.*pow(m44,2)*
         RSp45613e - 1./864.*pow(m44,2)*RSp45633e - 1./216.*pow(m44,2)*
         RSp563e - 1./864.*pow(m44,2)*RSp45611e - 1./216.*pow(m44,2)*
         RSp561e - 1./216.*pow(m44,2)*RSp564e + 1./108.*m34*mp2*
         RSp45613e + 1./54.*m34*mp2*RSp45633e - 5./108.*m34*mp2*RSp563e
          - 1./108.*m34*mp2*RSp45611e + 1./108.*m34*mp2*RSp561e + 1./54.
         *m34*mp2*RSp564e + 1./216.*m34*m44*RSp45613e + 1./108.*m34*m44
         *RSp45633e + 1./72.*m34*m44*RSp563e - 1./216.*m34*m44*
         RSp45611e - 1./72.*m34*m44*RSp561e - 1./108.*m34*m44*RSp564e
          + 1./54.*pow(m34,2)*RSp45613e - 1./54.*pow(m34,2)*RSp45633e
          + 1./54.*pow(m34,2)*RSp563e );

      F13p6x23RV +=  + hhdV(1,m14,m34,mp2,m13,L,mu2) * (  - 1./216.*
         pow(m34,2)*RSp45611e - 1./108.*pow(m34,2)*RSp561e - 1./216.*
         pow(m34,2)*RSp564e - 1./72.*m33*mp2*RSp45613e - 1./108.*m33*
         mp2*RSp45633e + 1./72.*m33*mp2*RSp563e - 1./216.*m33*mp2*
         RSp45611e + 1./216.*m33*mp2*RSp561e + 1./108.*m33*mp2*RSp564e
          - 1./144.*m33*m44*RSp45613e - 1./216.*m33*m44*RSp45633e - 5./
         432.*m33*m44*RSp563e - 1./432.*m33*m44*RSp45611e - 1./144.*m33
         *m44*RSp561e - 1./216.*m33*m44*RSp564e + 1./54.*m33*m34*
         RSp45633e - 1./216.*m33*m34*RSp45611e - 1./108.*m33*m34*
         RSp561e - 1./216.*m33*m34*RSp564e - 1./216.*pow(m33,2)*
         RSp45613e - 1./216.*pow(m33,2)*RSp45633e - 1./216.*pow(m33,2)*
         RSp563e - 1./864.*pow(m33,2)*RSp45611e - 1./432.*pow(m33,2)*
         RSp561e - 1./864.*pow(m33,2)*RSp564e + 1./108.*m14*mp2*
         RSp45613e - 1./108.*m14*mp2*RSp45633e + 1./108.*m14*mp2*
         RSp563e + 1./54.*m14*mp2*RSp45611e - 5./108.*m14*mp2*RSp561e
          + 1./54.*m14*mp2*RSp564e );

      F13p6x23RV +=  + hhdV(1,m14,m34,mp2,m13,L,mu2) * ( 1./216.*m14*
         m44*RSp45613e - 1./216.*m14*m44*RSp45633e - 1./72.*m14*m44*
         RSp563e + 1./108.*m14*m44*RSp45611e + 1./72.*m14*m44*RSp561e
          - 1./108.*m14*m44*RSp564e - 5./108.*m14*m34*RSp45613e + 1./54.
         *m14*m34*RSp45633e + 1./108.*m14*m34*RSp563e + 1./54.*m14*m34*
         RSp45611e + 1./108.*m14*m34*RSp561e - 1./108.*m14*m34*RSp564e
          + 1./72.*m14*m33*RSp45613e - 1./108.*m14*m33*RSp45633e - 1./
         72.*m14*m33*RSp563e + 1./108.*m14*m33*RSp45611e + 1./216.*m14*
         m33*RSp561e - 1./216.*m14*m33*RSp564e + 1./54.*pow(m14,2)*
         RSp45613e - 1./216.*pow(m14,2)*RSp45633e - 1./108.*pow(m14,2)*
         RSp563e - 1./54.*pow(m14,2)*RSp45611e + 1./54.*pow(m14,2)*
         RSp561e - 1./216.*pow(m14,2)*RSp564e + 1./27.*m13*mp2*
         RSp45613e + 1./54.*m13*mp2*RSp45633e - 2./27.*m13*mp2*RSp563e
          + 1./54.*m13*mp2*RSp45611e - 2./27.*m13*mp2*RSp561e + 2./27.*
         m13*mp2*RSp564e + 1./54.*m13*m44*RSp45613e + 1./108.*m13*m44*
         RSp45633e );

      F13p6x23RV +=  + hhdV(1,m14,m34,mp2,m13,L,mu2) * ( 1./108.*m13*
         m44*RSp45611e - 1./27.*m13*m44*RSp564e - 1./54.*m13*m34*
         RSp45613e - 1./27.*m13*m34*RSp45633e + 5./54.*m13*m34*RSp563e
          + 1./54.*m13*m34*RSp45611e - 1./54.*m13*m34*RSp561e - 1./27.*
         m13*m34*RSp564e + 1./36.*m13*m33*RSp45613e + 1./54.*m13*m33*
         RSp45633e - 1./36.*m13*m33*RSp563e + 1./108.*m13*m33*RSp45611e
          - 1./108.*m13*m33*RSp561e - 1./54.*m13*m33*RSp564e + 5./216.*
         m13*m14*RSp45613e + 1./54.*m13*m14*RSp45633e - 13./216.*m13*
         m14*RSp563e - 17./216.*m13*m14*RSp45611e + 29./216.*m13*m14*
         RSp561e - 1./27.*m13*m14*RSp564e + 7./54.*pow(m13,2)*RSp45613e
          - 1./54.*pow(m13,2)*RSp45633e - 5./54.*pow(m13,2)*RSp563e - 1.
         /54.*pow(m13,2)*RSp45611e - 5./54.*pow(m13,2)*RSp561e + 5./54.
         *pow(m13,2)*RSp564e - 1./72.*m11*mp2*RSp45613e - 1./216.*m11*
         mp2*RSp45633e + 1./216.*m11*mp2*RSp563e - 1./108.*m11*mp2*
         RSp45611e + 1./72.*m11*mp2*RSp561e + 1./108.*m11*mp2*RSp564e
          - 1./144.*m11*m44*RSp45613e );

      F13p6x23RV +=  + hhdV(1,m14,m34,mp2,m13,L,mu2) * (  - 1./432.*m11
         *m44*RSp45633e - 1./144.*m11*m44*RSp563e - 1./216.*m11*m44*
         RSp45611e - 5./432.*m11*m44*RSp561e - 1./216.*m11*m44*RSp564e
          + 1./72.*m11*m34*RSp45613e + 1./108.*m11*m34*RSp45633e + 1./
         216.*m11*m34*RSp563e - 1./108.*m11*m34*RSp45611e - 1./72.*m11*
         m34*RSp561e - 1./216.*m11*m34*RSp564e - 5./432.*m11*m33*
         RSp45613e - 1./216.*m11*m33*RSp45633e - 1./144.*m11*m33*
         RSp563e - 1./216.*m11*m33*RSp45611e - 1./144.*m11*m33*RSp561e
          - 1./432.*m11*m33*RSp564e - 1./216.*m11*m14*RSp45633e - 1./
         108.*m11*m14*RSp563e + 1./54.*m11*m14*RSp45611e - 1./216.*m11*
         m14*RSp564e + 1./36.*m11*m13*RSp45613e + 1./108.*m11*m13*
         RSp45633e - 1./108.*m11*m13*RSp563e + 1./54.*m11*m13*RSp45611e
          - 1./36.*m11*m13*RSp561e - 1./54.*m11*m13*RSp564e - 1./216.*
         pow(m11,2)*RSp45613e - 1./864.*pow(m11,2)*RSp45633e - 1./432.*
         pow(m11,2)*RSp563e - 1./216.*pow(m11,2)*RSp45611e - 1./216.*
         pow(m11,2)*RSp561e );

      F13p6x23RV +=  + hhdV(1,m14,m34,mp2,m13,L,mu2) * (  - 1./864.*
         pow(m11,2)*RSp564e );

      F13p6x23RV +=  + hhdV(1,m14,m34,me2,m13,L,mu2) * (  - 1./108.*
         pow(me2,2)*RSe45613p - 1./216.*pow(me2,2)*RSe45633p + 1./54.*
         pow(me2,2)*RSe563p - 1./216.*pow(me2,2)*RSe45611p + 1./54.*
         pow(me2,2)*RSe561p - 1./54.*pow(me2,2)*RSe564p - 1./108.*m44*
         me2*RSe45613p - 1./216.*m44*me2*RSe45633p - 1./216.*m44*me2*
         RSe45611p + 1./54.*m44*me2*RSe564p - 1./432.*pow(m44,2)*
         RSe45613p - 1./864.*pow(m44,2)*RSe45633p - 1./216.*pow(m44,2)*
         RSe563p - 1./864.*pow(m44,2)*RSe45611p - 1./216.*pow(m44,2)*
         RSe561p - 1./216.*pow(m44,2)*RSe564p + 1./108.*m34*me2*
         RSe45613p + 1./54.*m34*me2*RSe45633p - 5./108.*m34*me2*RSe563p
          - 1./108.*m34*me2*RSe45611p + 1./108.*m34*me2*RSe561p + 1./54.
         *m34*me2*RSe564p + 1./216.*m34*m44*RSe45613p + 1./108.*m34*m44
         *RSe45633p + 1./72.*m34*m44*RSe563p - 1./216.*m34*m44*
         RSe45611p - 1./72.*m34*m44*RSe561p - 1./108.*m34*m44*RSe564p
          + 1./54.*pow(m34,2)*RSe45613p - 1./54.*pow(m34,2)*RSe45633p
          + 1./54.*pow(m34,2)*RSe563p );

      F13p6x23RV +=  + hhdV(1,m14,m34,me2,m13,L,mu2) * (  - 1./216.*
         pow(m34,2)*RSe45611p - 1./108.*pow(m34,2)*RSe561p - 1./216.*
         pow(m34,2)*RSe564p - 1./72.*m33*me2*RSe45613p - 1./108.*m33*
         me2*RSe45633p + 1./72.*m33*me2*RSe563p - 1./216.*m33*me2*
         RSe45611p + 1./216.*m33*me2*RSe561p + 1./108.*m33*me2*RSe564p
          - 1./144.*m33*m44*RSe45613p - 1./216.*m33*m44*RSe45633p - 5./
         432.*m33*m44*RSe563p - 1./432.*m33*m44*RSe45611p - 1./144.*m33
         *m44*RSe561p - 1./216.*m33*m44*RSe564p + 1./54.*m33*m34*
         RSe45633p - 1./216.*m33*m34*RSe45611p - 1./108.*m33*m34*
         RSe561p - 1./216.*m33*m34*RSe564p - 1./216.*pow(m33,2)*
         RSe45613p - 1./216.*pow(m33,2)*RSe45633p - 1./216.*pow(m33,2)*
         RSe563p - 1./864.*pow(m33,2)*RSe45611p - 1./432.*pow(m33,2)*
         RSe561p - 1./864.*pow(m33,2)*RSe564p + 1./108.*m14*me2*
         RSe45613p - 1./108.*m14*me2*RSe45633p + 1./108.*m14*me2*
         RSe563p + 1./54.*m14*me2*RSe45611p - 5./108.*m14*me2*RSe561p
          + 1./54.*m14*me2*RSe564p );

      F13p6x23RV +=  + hhdV(1,m14,m34,me2,m13,L,mu2) * ( 1./216.*m14*
         m44*RSe45613p - 1./216.*m14*m44*RSe45633p - 1./72.*m14*m44*
         RSe563p + 1./108.*m14*m44*RSe45611p + 1./72.*m14*m44*RSe561p
          - 1./108.*m14*m44*RSe564p - 5./108.*m14*m34*RSe45613p + 1./54.
         *m14*m34*RSe45633p + 1./108.*m14*m34*RSe563p + 1./54.*m14*m34*
         RSe45611p + 1./108.*m14*m34*RSe561p - 1./108.*m14*m34*RSe564p
          + 1./72.*m14*m33*RSe45613p - 1./108.*m14*m33*RSe45633p - 1./
         72.*m14*m33*RSe563p + 1./108.*m14*m33*RSe45611p + 1./216.*m14*
         m33*RSe561p - 1./216.*m14*m33*RSe564p + 1./54.*pow(m14,2)*
         RSe45613p - 1./216.*pow(m14,2)*RSe45633p - 1./108.*pow(m14,2)*
         RSe563p - 1./54.*pow(m14,2)*RSe45611p + 1./54.*pow(m14,2)*
         RSe561p - 1./216.*pow(m14,2)*RSe564p + 1./27.*m13*me2*
         RSe45613p + 1./54.*m13*me2*RSe45633p - 2./27.*m13*me2*RSe563p
          + 1./54.*m13*me2*RSe45611p - 2./27.*m13*me2*RSe561p + 2./27.*
         m13*me2*RSe564p + 1./54.*m13*m44*RSe45613p + 1./108.*m13*m44*
         RSe45633p );

      F13p6x23RV +=  + hhdV(1,m14,m34,me2,m13,L,mu2) * ( 1./108.*m13*
         m44*RSe45611p - 1./27.*m13*m44*RSe564p - 1./54.*m13*m34*
         RSe45613p - 1./27.*m13*m34*RSe45633p + 5./54.*m13*m34*RSe563p
          + 1./54.*m13*m34*RSe45611p - 1./54.*m13*m34*RSe561p - 1./27.*
         m13*m34*RSe564p + 1./36.*m13*m33*RSe45613p + 1./54.*m13*m33*
         RSe45633p - 1./36.*m13*m33*RSe563p + 1./108.*m13*m33*RSe45611p
          - 1./108.*m13*m33*RSe561p - 1./54.*m13*m33*RSe564p + 5./216.*
         m13*m14*RSe45613p + 1./54.*m13*m14*RSe45633p - 13./216.*m13*
         m14*RSe563p - 17./216.*m13*m14*RSe45611p + 29./216.*m13*m14*
         RSe561p - 1./27.*m13*m14*RSe564p + 7./54.*pow(m13,2)*RSe45613p
          - 1./54.*pow(m13,2)*RSe45633p - 5./54.*pow(m13,2)*RSe563p - 1.
         /54.*pow(m13,2)*RSe45611p - 5./54.*pow(m13,2)*RSe561p + 5./54.
         *pow(m13,2)*RSe564p - 1./72.*m11*me2*RSe45613p - 1./216.*m11*
         me2*RSe45633p + 1./216.*m11*me2*RSe563p - 1./108.*m11*me2*
         RSe45611p + 1./72.*m11*me2*RSe561p + 1./108.*m11*me2*RSe564p
          - 1./144.*m11*m44*RSe45613p );

      F13p6x23RV +=  + hhdV(1,m14,m34,me2,m13,L,mu2) * (  - 1./432.*m11
         *m44*RSe45633p - 1./144.*m11*m44*RSe563p - 1./216.*m11*m44*
         RSe45611p - 5./432.*m11*m44*RSe561p - 1./216.*m11*m44*RSe564p
          + 1./72.*m11*m34*RSe45613p + 1./108.*m11*m34*RSe45633p + 1./
         216.*m11*m34*RSe563p - 1./108.*m11*m34*RSe45611p - 1./72.*m11*
         m34*RSe561p - 1./216.*m11*m34*RSe564p - 5./432.*m11*m33*
         RSe45613p - 1./216.*m11*m33*RSe45633p - 1./144.*m11*m33*
         RSe563p - 1./216.*m11*m33*RSe45611p - 1./144.*m11*m33*RSe561p
          - 1./432.*m11*m33*RSe564p - 1./216.*m11*m14*RSe45633p - 1./
         108.*m11*m14*RSe563p + 1./54.*m11*m14*RSe45611p - 1./216.*m11*
         m14*RSe564p + 1./36.*m11*m13*RSe45613p + 1./108.*m11*m13*
         RSe45633p - 1./108.*m11*m13*RSe563p + 1./54.*m11*m13*RSe45611p
          - 1./36.*m11*m13*RSe561p - 1./54.*m11*m13*RSe564p - 1./216.*
         pow(m11,2)*RSe45613p - 1./864.*pow(m11,2)*RSe45633p - 1./432.*
         pow(m11,2)*RSe563p - 1./216.*pow(m11,2)*RSe45611p - 1./216.*
         pow(m11,2)*RSe561p );

      F13p6x23RV +=  + hhdV(1,m14,m34,me2,m13,L,mu2) * (  - 1./864.*
         pow(m11,2)*RSe564p );

      F13p6x23RV +=  + hhdV(1,m14,m35,m45,m13,L,mu2) * ( 1./288.*pow(
         m55,2) - 1./36.*m45*m55 + 1./18.*pow(m45,2) + 1./144.*m44*m55
          - 1./36.*m44*m45 + 1./288.*pow(m44,2) + 1./72.*m35*m55 - 1./
         18.*m35*m45 + 1./72.*m35*m44 + 1./72.*pow(m35,2) + 1./144.*m33
         *m55 - 1./36.*m33*m45 + 1./144.*m33*m44 + 1./72.*m33*m35 + 1./
         288.*pow(m33,2) + 1./72.*m14*m55 - 1./18.*m14*m45 + 1./72.*m14
         *m44 + 1./36.*m14*m35 + 1./72.*m14*m33 + 1./72.*pow(m14,2) + 1.
         /18.*m13*m55 - 2./9.*m13*m45 + 1./18.*m13*m44 + 1./9.*m13*m35
          + 1./18.*m13*m33 + 1./9.*m13*m14 - 5./18.*pow(m13,2) + 1./144.
         *m11*m55 - 1./36.*m11*m45 + 1./144.*m11*m44 + 1./72.*m11*m35
          + 1./144.*m11*m33 + 1./72.*m11*m14 + 1./18.*m11*m13 + 1./288.
         *pow(m11,2) );

      F13p6x23RV +=  + hhdV(1,m14,m36,m46,m13,L,mu2) * ( 1./288.*pow(
         m66,2) - 1./36.*m46*m66 + 1./18.*pow(m46,2) + 1./144.*m44*m66
          - 1./36.*m44*m46 + 1./288.*pow(m44,2) + 1./72.*m36*m66 - 1./
         18.*m36*m46 + 1./72.*m36*m44 + 1./72.*pow(m36,2) + 1./144.*m33
         *m66 - 1./36.*m33*m46 + 1./144.*m33*m44 + 1./72.*m33*m36 + 1./
         288.*pow(m33,2) + 1./72.*m14*m66 - 1./18.*m14*m46 + 1./72.*m14
         *m44 + 1./36.*m14*m36 + 1./72.*m14*m33 + 1./72.*pow(m14,2) + 1.
         /18.*m13*m66 - 2./9.*m13*m46 + 1./18.*m13*m44 + 1./9.*m13*m36
          + 1./18.*m13*m33 + 1./9.*m13*m14 - 5./18.*pow(m13,2) + 1./144.
         *m11*m66 - 1./36.*m11*m46 + 1./144.*m11*m44 + 1./72.*m11*m36
          + 1./144.*m11*m33 + 1./72.*m11*m14 + 1./18.*m11*m13 + 1./288.
         *pow(m11,2) );

      F13p6x23RV +=  + hhdV(1,m15,m11,m35,m13,L,mu2) * ( 1./24.*m13*m15
         *RS14563pe - 1./24.*m13*m15*RC1456pe + 1./24.*m13*m15*RS146pe
          );

      F13p6x23RV +=  + hhdV(1,m15,m33,m35,m13,L,mu2) * (  - 1./216.*m55
         *RD3456ep - 1./432.*pow(m55,2)*RS34561pe - 1./864.*pow(m55,2)*
         RC3456pe - 1./216.*pow(m55,2)*RS346pe + 1./54.*m35*RD3456ep + 
         1./216.*m35*m55*RS34561pe + 1./108.*m35*m55*RC3456pe + 1./72.*
         m35*m55*RS346pe + 1./54.*pow(m35,2)*RS34561pe - 1./54.*pow(
         m35,2)*RC3456pe + 1./54.*pow(m35,2)*RS346pe - 1./54.*m33*
         RD3456ep - 7./432.*m33*m55*RS34561pe - 1./108.*m33*m55*
         RC3456pe - 5./432.*m33*m55*RS346pe + 1./108.*m33*m35*RS34561pe
          + 1./27.*m33*m35*RC3456pe - 5./108.*m33*m35*RS346pe - 1./36.*
         pow(m33,2)*RS34561pe - 1./54.*pow(m33,2)*RC3456pe + 1./36.*
         pow(m33,2)*RS346pe - 1./108.*m15*RD3456ep + 1./216.*m15*m55*
         RS34561pe - 1./216.*m15*m55*RC3456pe - 1./72.*m15*m55*RS346pe
          - 5./108.*m15*m35*RS34561pe + 1./54.*m15*m35*RC3456pe + 1./
         108.*m15*m35*RS346pe + 5./216.*m15*m33*RS34561pe - 1./54.*m15*
         m33*RC3456pe - 1./216.*m15*m33*RS346pe + 1./54.*pow(m15,2)*
         RS34561pe );

      F13p6x23RV +=  + hhdV(1,m15,m33,m35,m13,L,mu2) * (  - 1./216.*
         pow(m15,2)*RC3456pe - 1./108.*pow(m15,2)*RS346pe - 1./27.*m13*
         RD3456ep - 1./108.*m13*m55*RS34561pe - 1./54.*m13*m55*RC3456pe
          - 1./36.*m13*m55*RS346pe - 2./27.*m13*m35*RS34561pe + 2./27.*
         m13*m35*RC3456pe - 2./27.*m13*m35*RS346pe - 1./54.*m13*m33*
         RS34561pe - 2./27.*m13*m33*RC3456pe + 5./54.*m13*m33*RS346pe
          + 29./216.*m13*m15*RS34561pe - 1./27.*m13*m15*RC3456pe - 13./
         216.*m13*m15*RS346pe - 5./54.*pow(m13,2)*RS34561pe + 5./54.*
         pow(m13,2)*RC3456pe - 5./54.*pow(m13,2)*RS346pe - 1./216.*m11*
         RD3456ep - 1./144.*m11*m55*RS34561pe - 1./432.*m11*m55*
         RC3456pe - 1./144.*m11*m55*RS346pe + 1./72.*m11*m35*RS34561pe
          + 1./108.*m11*m35*RC3456pe + 1./216.*m11*m35*RS346pe - 11./
         432.*m11*m33*RS34561pe - 1./108.*m11*m33*RC3456pe - 1./432.*
         m11*m33*RS346pe - 1./216.*m11*m15*RC3456pe - 1./108.*m11*m15*
         RS346pe - 1./36.*m11*m13*RS34561pe - 1./54.*m11*m13*RC3456pe
          - 1./108.*m11*m13*RS346pe );

      F13p6x23RV +=  + hhdV(1,m15,m33,m35,m13,L,mu2) * (  - 1./216.*
         pow(m11,2)*RS34561pe - 1./864.*pow(m11,2)*RC3456pe - 1./432.*
         pow(m11,2)*RS346pe );

      F13p6x23RV +=  + hhdV(1,m15,m34,m45,m13,L,mu2) * ( 1./288.*pow(
         m55,2) - 1./36.*m45*m55 + 1./18.*pow(m45,2) + 1./144.*m44*m55
          - 1./36.*m44*m45 + 1./288.*pow(m44,2) + 1./72.*m34*m55 - 1./
         18.*m34*m45 + 1./72.*m34*m44 + 1./72.*pow(m34,2) + 1./144.*m33
         *m55 - 1./36.*m33*m45 + 1./144.*m33*m44 + 1./72.*m33*m34 + 1./
         288.*pow(m33,2) + 1./72.*m15*m55 - 1./18.*m15*m45 + 1./72.*m15
         *m44 + 1./36.*m15*m34 + 1./72.*m15*m33 + 1./72.*pow(m15,2) + 1.
         /18.*m13*m55 - 2./9.*m13*m45 + 1./18.*m13*m44 + 1./9.*m13*m34
          + 1./18.*m13*m33 + 1./9.*m13*m15 - 5./18.*pow(m13,2) + 1./144.
         *m11*m55 - 1./36.*m11*m45 + 1./144.*m11*m44 + 1./72.*m11*m34
          + 1./144.*m11*m33 + 1./72.*m11*m15 + 1./18.*m11*m13 + 1./288.
         *pow(m11,2) );

      F13p6x23RV +=  + hhdV(1,m15,m35,m55,m13,L,mu2) * ( 1./72.*pow(
         m55,2) - 1./216.*pow(m55,2)*RS546pe - 1./36.*m35*m55 + 1./108.
         *m35*m55*RS546pe + 1./72.*pow(m35,2) - 1./216.*pow(m35,2)*
         RS546pe - 1./72.*m33*m55 + 1./216.*m33*m55*RS546pe + 1./72.*
         m33*m35 - 1./216.*m33*m35*RS546pe + 1./288.*pow(m33,2) - 1./
         864.*pow(m33,2)*RS546pe - 1./36.*m15*m55 + 1./108.*m15*m55*
         RS546pe + 1./36.*m15*m35 - 1./108.*m15*m35*RS546pe + 1./72.*
         m15*m33 - 1./216.*m15*m33*RS546pe + 1./72.*pow(m15,2) - 1./216.
         *pow(m15,2)*RS546pe - 1./9.*m13*m55 + 1./27.*m13*m55*RS546pe
          + 1./9.*m13*m35 - 1./27.*m13*m35*RS546pe + 1./18.*m13*m33 - 1.
         /54.*m13*m33*RS546pe + 1./9.*m13*m15 - 1./27.*m13*m15*RS546pe
          - 5./18.*pow(m13,2) + 5./54.*pow(m13,2)*RS546pe - 1./72.*m11*
         m55 + 1./216.*m11*m55*RS546pe + 1./72.*m11*m35 - 1./216.*m11*
         m35*RS546pe + 1./144.*m11*m33 - 1./432.*m11*m33*RS546pe + 1./
         72.*m11*m15 - 1./216.*m11*m15*RS546pe + 1./18.*m11*m13 - 1./54.
         *m11*m13*RS546pe );

      F13p6x23RV +=  + hhdV(1,m15,m35,m55,m13,L,mu2) * ( 1./288.*pow(
         m11,2) - 1./864.*pow(m11,2)*RS546pe );

      F13p6x23RV +=  + hhdV(1,m15,m35,mp2,m13,L,mu2) * (  - 1./108.*
         pow(mp2,2)*RSp45613e - 1./216.*pow(mp2,2)*RSp45633e + 1./54.*
         pow(mp2,2)*RSp463e - 1./216.*pow(mp2,2)*RSp45611e + 1./54.*
         pow(mp2,2)*RSp461e - 1./54.*pow(mp2,2)*RSp465e - 1./108.*m55*
         mp2*RSp45613e - 1./216.*m55*mp2*RSp45633e - 1./216.*m55*mp2*
         RSp45611e + 1./54.*m55*mp2*RSp465e - 1./432.*pow(m55,2)*
         RSp45613e - 1./864.*pow(m55,2)*RSp45633e - 1./216.*pow(m55,2)*
         RSp463e - 1./864.*pow(m55,2)*RSp45611e - 1./216.*pow(m55,2)*
         RSp461e - 1./216.*pow(m55,2)*RSp465e + 1./108.*m35*mp2*
         RSp45613e + 1./54.*m35*mp2*RSp45633e - 5./108.*m35*mp2*RSp463e
          - 1./108.*m35*mp2*RSp45611e + 1./108.*m35*mp2*RSp461e + 1./54.
         *m35*mp2*RSp465e + 1./216.*m35*m55*RSp45613e + 1./108.*m35*m55
         *RSp45633e + 1./72.*m35*m55*RSp463e - 1./216.*m35*m55*
         RSp45611e - 1./72.*m35*m55*RSp461e - 1./108.*m35*m55*RSp465e
          + 1./54.*pow(m35,2)*RSp45613e - 1./54.*pow(m35,2)*RSp45633e
          + 1./54.*pow(m35,2)*RSp463e );

      F13p6x23RV +=  + hhdV(1,m15,m35,mp2,m13,L,mu2) * (  - 1./216.*
         pow(m35,2)*RSp45611e - 1./108.*pow(m35,2)*RSp461e - 1./216.*
         pow(m35,2)*RSp465e - 1./72.*m33*mp2*RSp45613e - 1./108.*m33*
         mp2*RSp45633e + 1./72.*m33*mp2*RSp463e - 1./216.*m33*mp2*
         RSp45611e + 1./216.*m33*mp2*RSp461e + 1./108.*m33*mp2*RSp465e
          - 1./144.*m33*m55*RSp45613e - 1./216.*m33*m55*RSp45633e - 5./
         432.*m33*m55*RSp463e - 1./432.*m33*m55*RSp45611e - 1./144.*m33
         *m55*RSp461e - 1./216.*m33*m55*RSp465e + 1./54.*m33*m35*
         RSp45633e - 1./216.*m33*m35*RSp45611e - 1./108.*m33*m35*
         RSp461e - 1./216.*m33*m35*RSp465e - 1./216.*pow(m33,2)*
         RSp45613e - 1./216.*pow(m33,2)*RSp45633e - 1./216.*pow(m33,2)*
         RSp463e - 1./864.*pow(m33,2)*RSp45611e - 1./432.*pow(m33,2)*
         RSp461e - 1./864.*pow(m33,2)*RSp465e + 1./108.*m15*mp2*
         RSp45613e - 1./108.*m15*mp2*RSp45633e + 1./108.*m15*mp2*
         RSp463e + 1./54.*m15*mp2*RSp45611e - 5./108.*m15*mp2*RSp461e
          + 1./54.*m15*mp2*RSp465e );

      F13p6x23RV +=  + hhdV(1,m15,m35,mp2,m13,L,mu2) * ( 1./216.*m15*
         m55*RSp45613e - 1./216.*m15*m55*RSp45633e - 1./72.*m15*m55*
         RSp463e + 1./108.*m15*m55*RSp45611e + 1./72.*m15*m55*RSp461e
          - 1./108.*m15*m55*RSp465e - 5./108.*m15*m35*RSp45613e + 1./54.
         *m15*m35*RSp45633e + 1./108.*m15*m35*RSp463e + 1./54.*m15*m35*
         RSp45611e + 1./108.*m15*m35*RSp461e - 1./108.*m15*m35*RSp465e
          + 1./72.*m15*m33*RSp45613e - 1./108.*m15*m33*RSp45633e - 1./
         72.*m15*m33*RSp463e + 1./108.*m15*m33*RSp45611e + 1./216.*m15*
         m33*RSp461e - 1./216.*m15*m33*RSp465e + 1./54.*pow(m15,2)*
         RSp45613e - 1./216.*pow(m15,2)*RSp45633e - 1./108.*pow(m15,2)*
         RSp463e - 1./54.*pow(m15,2)*RSp45611e + 1./54.*pow(m15,2)*
         RSp461e - 1./216.*pow(m15,2)*RSp465e + 1./27.*m13*mp2*
         RSp45613e + 1./54.*m13*mp2*RSp45633e - 2./27.*m13*mp2*RSp463e
          + 1./54.*m13*mp2*RSp45611e - 2./27.*m13*mp2*RSp461e + 2./27.*
         m13*mp2*RSp465e + 1./54.*m13*m55*RSp45613e + 1./108.*m13*m55*
         RSp45633e );

      F13p6x23RV +=  + hhdV(1,m15,m35,mp2,m13,L,mu2) * ( 1./108.*m13*
         m55*RSp45611e - 1./27.*m13*m55*RSp465e - 1./54.*m13*m35*
         RSp45613e - 1./27.*m13*m35*RSp45633e + 5./54.*m13*m35*RSp463e
          + 1./54.*m13*m35*RSp45611e - 1./54.*m13*m35*RSp461e - 1./27.*
         m13*m35*RSp465e + 1./36.*m13*m33*RSp45613e + 1./54.*m13*m33*
         RSp45633e - 1./36.*m13*m33*RSp463e + 1./108.*m13*m33*RSp45611e
          - 1./108.*m13*m33*RSp461e - 1./54.*m13*m33*RSp465e + 5./216.*
         m13*m15*RSp45613e + 1./54.*m13*m15*RSp45633e - 13./216.*m13*
         m15*RSp463e - 17./216.*m13*m15*RSp45611e + 29./216.*m13*m15*
         RSp461e - 1./27.*m13*m15*RSp465e + 7./54.*pow(m13,2)*RSp45613e
          - 1./54.*pow(m13,2)*RSp45633e - 5./54.*pow(m13,2)*RSp463e - 1.
         /54.*pow(m13,2)*RSp45611e - 5./54.*pow(m13,2)*RSp461e + 5./54.
         *pow(m13,2)*RSp465e - 1./72.*m11*mp2*RSp45613e - 1./216.*m11*
         mp2*RSp45633e + 1./216.*m11*mp2*RSp463e - 1./108.*m11*mp2*
         RSp45611e + 1./72.*m11*mp2*RSp461e + 1./108.*m11*mp2*RSp465e
          - 1./144.*m11*m55*RSp45613e );

      F13p6x23RV +=  + hhdV(1,m15,m35,mp2,m13,L,mu2) * (  - 1./432.*m11
         *m55*RSp45633e - 1./144.*m11*m55*RSp463e - 1./216.*m11*m55*
         RSp45611e - 5./432.*m11*m55*RSp461e - 1./216.*m11*m55*RSp465e
          + 1./72.*m11*m35*RSp45613e + 1./108.*m11*m35*RSp45633e + 1./
         216.*m11*m35*RSp463e - 1./108.*m11*m35*RSp45611e - 1./72.*m11*
         m35*RSp461e - 1./216.*m11*m35*RSp465e - 5./432.*m11*m33*
         RSp45613e - 1./216.*m11*m33*RSp45633e - 1./144.*m11*m33*
         RSp463e - 1./216.*m11*m33*RSp45611e - 1./144.*m11*m33*RSp461e
          - 1./432.*m11*m33*RSp465e - 1./216.*m11*m15*RSp45633e - 1./
         108.*m11*m15*RSp463e + 1./54.*m11*m15*RSp45611e - 1./216.*m11*
         m15*RSp465e + 1./36.*m11*m13*RSp45613e + 1./108.*m11*m13*
         RSp45633e - 1./108.*m11*m13*RSp463e + 1./54.*m11*m13*RSp45611e
          - 1./36.*m11*m13*RSp461e - 1./54.*m11*m13*RSp465e - 1./216.*
         pow(m11,2)*RSp45613e - 1./864.*pow(m11,2)*RSp45633e - 1./432.*
         pow(m11,2)*RSp463e - 1./216.*pow(m11,2)*RSp45611e - 1./216.*
         pow(m11,2)*RSp461e );

      F13p6x23RV +=  + hhdV(1,m15,m35,mp2,m13,L,mu2) * (  - 1./864.*
         pow(m11,2)*RSp465e );

      F13p6x23RV +=  + hhdV(1,m15,m35,me2,m13,L,mu2) * (  - 1./108.*
         pow(me2,2)*RSe45613p - 1./216.*pow(me2,2)*RSe45633p + 1./54.*
         pow(me2,2)*RSe463p - 1./216.*pow(me2,2)*RSe45611p + 1./54.*
         pow(me2,2)*RSe461p - 1./54.*pow(me2,2)*RSe465p - 1./108.*m55*
         me2*RSe45613p - 1./216.*m55*me2*RSe45633p - 1./216.*m55*me2*
         RSe45611p + 1./54.*m55*me2*RSe465p - 1./432.*pow(m55,2)*
         RSe45613p - 1./864.*pow(m55,2)*RSe45633p - 1./216.*pow(m55,2)*
         RSe463p - 1./864.*pow(m55,2)*RSe45611p - 1./216.*pow(m55,2)*
         RSe461p - 1./216.*pow(m55,2)*RSe465p + 1./108.*m35*me2*
         RSe45613p + 1./54.*m35*me2*RSe45633p - 5./108.*m35*me2*RSe463p
          - 1./108.*m35*me2*RSe45611p + 1./108.*m35*me2*RSe461p + 1./54.
         *m35*me2*RSe465p + 1./216.*m35*m55*RSe45613p + 1./108.*m35*m55
         *RSe45633p + 1./72.*m35*m55*RSe463p - 1./216.*m35*m55*
         RSe45611p - 1./72.*m35*m55*RSe461p - 1./108.*m35*m55*RSe465p
          + 1./54.*pow(m35,2)*RSe45613p - 1./54.*pow(m35,2)*RSe45633p
          + 1./54.*pow(m35,2)*RSe463p );

      F13p6x23RV +=  + hhdV(1,m15,m35,me2,m13,L,mu2) * (  - 1./216.*
         pow(m35,2)*RSe45611p - 1./108.*pow(m35,2)*RSe461p - 1./216.*
         pow(m35,2)*RSe465p - 1./72.*m33*me2*RSe45613p - 1./108.*m33*
         me2*RSe45633p + 1./72.*m33*me2*RSe463p - 1./216.*m33*me2*
         RSe45611p + 1./216.*m33*me2*RSe461p + 1./108.*m33*me2*RSe465p
          - 1./144.*m33*m55*RSe45613p - 1./216.*m33*m55*RSe45633p - 5./
         432.*m33*m55*RSe463p - 1./432.*m33*m55*RSe45611p - 1./144.*m33
         *m55*RSe461p - 1./216.*m33*m55*RSe465p + 1./54.*m33*m35*
         RSe45633p - 1./216.*m33*m35*RSe45611p - 1./108.*m33*m35*
         RSe461p - 1./216.*m33*m35*RSe465p - 1./216.*pow(m33,2)*
         RSe45613p - 1./216.*pow(m33,2)*RSe45633p - 1./216.*pow(m33,2)*
         RSe463p - 1./864.*pow(m33,2)*RSe45611p - 1./432.*pow(m33,2)*
         RSe461p - 1./864.*pow(m33,2)*RSe465p + 1./108.*m15*me2*
         RSe45613p - 1./108.*m15*me2*RSe45633p + 1./108.*m15*me2*
         RSe463p + 1./54.*m15*me2*RSe45611p - 5./108.*m15*me2*RSe461p
          + 1./54.*m15*me2*RSe465p );

      F13p6x23RV +=  + hhdV(1,m15,m35,me2,m13,L,mu2) * ( 1./216.*m15*
         m55*RSe45613p - 1./216.*m15*m55*RSe45633p - 1./72.*m15*m55*
         RSe463p + 1./108.*m15*m55*RSe45611p + 1./72.*m15*m55*RSe461p
          - 1./108.*m15*m55*RSe465p - 5./108.*m15*m35*RSe45613p + 1./54.
         *m15*m35*RSe45633p + 1./108.*m15*m35*RSe463p + 1./54.*m15*m35*
         RSe45611p + 1./108.*m15*m35*RSe461p - 1./108.*m15*m35*RSe465p
          + 1./72.*m15*m33*RSe45613p - 1./108.*m15*m33*RSe45633p - 1./
         72.*m15*m33*RSe463p + 1./108.*m15*m33*RSe45611p + 1./216.*m15*
         m33*RSe461p - 1./216.*m15*m33*RSe465p + 1./54.*pow(m15,2)*
         RSe45613p - 1./216.*pow(m15,2)*RSe45633p - 1./108.*pow(m15,2)*
         RSe463p - 1./54.*pow(m15,2)*RSe45611p + 1./54.*pow(m15,2)*
         RSe461p - 1./216.*pow(m15,2)*RSe465p + 1./27.*m13*me2*
         RSe45613p + 1./54.*m13*me2*RSe45633p - 2./27.*m13*me2*RSe463p
          + 1./54.*m13*me2*RSe45611p - 2./27.*m13*me2*RSe461p + 2./27.*
         m13*me2*RSe465p + 1./54.*m13*m55*RSe45613p + 1./108.*m13*m55*
         RSe45633p );

      F13p6x23RV +=  + hhdV(1,m15,m35,me2,m13,L,mu2) * ( 1./108.*m13*
         m55*RSe45611p - 1./27.*m13*m55*RSe465p - 1./54.*m13*m35*
         RSe45613p - 1./27.*m13*m35*RSe45633p + 5./54.*m13*m35*RSe463p
          + 1./54.*m13*m35*RSe45611p - 1./54.*m13*m35*RSe461p - 1./27.*
         m13*m35*RSe465p + 1./36.*m13*m33*RSe45613p + 1./54.*m13*m33*
         RSe45633p - 1./36.*m13*m33*RSe463p + 1./108.*m13*m33*RSe45611p
          - 1./108.*m13*m33*RSe461p - 1./54.*m13*m33*RSe465p + 5./216.*
         m13*m15*RSe45613p + 1./54.*m13*m15*RSe45633p - 13./216.*m13*
         m15*RSe463p - 17./216.*m13*m15*RSe45611p + 29./216.*m13*m15*
         RSe461p - 1./27.*m13*m15*RSe465p + 7./54.*pow(m13,2)*RSe45613p
          - 1./54.*pow(m13,2)*RSe45633p - 5./54.*pow(m13,2)*RSe463p - 1.
         /54.*pow(m13,2)*RSe45611p - 5./54.*pow(m13,2)*RSe461p + 5./54.
         *pow(m13,2)*RSe465p - 1./72.*m11*me2*RSe45613p - 1./216.*m11*
         me2*RSe45633p + 1./216.*m11*me2*RSe463p - 1./108.*m11*me2*
         RSe45611p + 1./72.*m11*me2*RSe461p + 1./108.*m11*me2*RSe465p
          - 1./144.*m11*m55*RSe45613p );

      F13p6x23RV +=  + hhdV(1,m15,m35,me2,m13,L,mu2) * (  - 1./432.*m11
         *m55*RSe45633p - 1./144.*m11*m55*RSe463p - 1./216.*m11*m55*
         RSe45611p - 5./432.*m11*m55*RSe461p - 1./216.*m11*m55*RSe465p
          + 1./72.*m11*m35*RSe45613p + 1./108.*m11*m35*RSe45633p + 1./
         216.*m11*m35*RSe463p - 1./108.*m11*m35*RSe45611p - 1./72.*m11*
         m35*RSe461p - 1./216.*m11*m35*RSe465p - 5./432.*m11*m33*
         RSe45613p - 1./216.*m11*m33*RSe45633p - 1./144.*m11*m33*
         RSe463p - 1./216.*m11*m33*RSe45611p - 1./144.*m11*m33*RSe461p
          - 1./432.*m11*m33*RSe465p - 1./216.*m11*m15*RSe45633p - 1./
         108.*m11*m15*RSe463p + 1./54.*m11*m15*RSe45611p - 1./216.*m11*
         m15*RSe465p + 1./36.*m11*m13*RSe45613p + 1./108.*m11*m13*
         RSe45633p - 1./108.*m11*m13*RSe463p + 1./54.*m11*m13*RSe45611p
          - 1./36.*m11*m13*RSe461p - 1./54.*m11*m13*RSe465p - 1./216.*
         pow(m11,2)*RSe45613p - 1./864.*pow(m11,2)*RSe45633p - 1./432.*
         pow(m11,2)*RSe463p - 1./216.*pow(m11,2)*RSe45611p - 1./216.*
         pow(m11,2)*RSe461p );

      F13p6x23RV +=  + hhdV(1,m15,m35,me2,m13,L,mu2) * (  - 1./864.*
         pow(m11,2)*RSe465p );

      F13p6x23RV +=  + hhdV(1,m15,m36,m56,m13,L,mu2) * ( 1./288.*pow(
         m66,2) - 1./36.*m56*m66 + 1./18.*pow(m56,2) + 1./144.*m55*m66
          - 1./36.*m55*m56 + 1./288.*pow(m55,2) + 1./72.*m36*m66 - 1./
         18.*m36*m56 + 1./72.*m36*m55 + 1./72.*pow(m36,2) + 1./144.*m33
         *m66 - 1./36.*m33*m56 + 1./144.*m33*m55 + 1./72.*m33*m36 + 1./
         288.*pow(m33,2) + 1./72.*m15*m66 - 1./18.*m15*m56 + 1./72.*m15
         *m55 + 1./36.*m15*m36 + 1./72.*m15*m33 + 1./72.*pow(m15,2) + 1.
         /18.*m13*m66 - 2./9.*m13*m56 + 1./18.*m13*m55 + 1./9.*m13*m36
          + 1./18.*m13*m33 + 1./9.*m13*m15 - 5./18.*pow(m13,2) + 1./144.
         *m11*m66 - 1./36.*m11*m56 + 1./144.*m11*m55 + 1./72.*m11*m36
          + 1./144.*m11*m33 + 1./72.*m11*m15 + 1./18.*m11*m13 + 1./288.
         *pow(m11,2) );

      F13p6x23RV +=  + hhdV(1,m16,m11,m36,m13,L,mu2) * ( 1./24.*m13*m16
         *RS14563pe - 1./24.*m13*m16*RC1456pe + 1./24.*m13*m16*RS145pe
          );

      F13p6x23RV +=  + hhdV(1,m16,m33,m36,m13,L,mu2) * (  - 1./216.*m66
         *RD3456ep - 1./432.*pow(m66,2)*RS34561pe - 1./864.*pow(m66,2)*
         RC3456pe - 1./216.*pow(m66,2)*RS345pe + 1./54.*m36*RD3456ep + 
         1./216.*m36*m66*RS34561pe + 1./108.*m36*m66*RC3456pe + 1./72.*
         m36*m66*RS345pe + 1./54.*pow(m36,2)*RS34561pe - 1./54.*pow(
         m36,2)*RC3456pe + 1./54.*pow(m36,2)*RS345pe - 1./54.*m33*
         RD3456ep - 7./432.*m33*m66*RS34561pe - 1./108.*m33*m66*
         RC3456pe - 5./432.*m33*m66*RS345pe + 1./108.*m33*m36*RS34561pe
          + 1./27.*m33*m36*RC3456pe - 5./108.*m33*m36*RS345pe - 1./36.*
         pow(m33,2)*RS34561pe - 1./54.*pow(m33,2)*RC3456pe + 1./36.*
         pow(m33,2)*RS345pe - 1./108.*m16*RD3456ep + 1./216.*m16*m66*
         RS34561pe - 1./216.*m16*m66*RC3456pe - 1./72.*m16*m66*RS345pe
          - 5./108.*m16*m36*RS34561pe + 1./54.*m16*m36*RC3456pe + 1./
         108.*m16*m36*RS345pe + 5./216.*m16*m33*RS34561pe - 1./54.*m16*
         m33*RC3456pe - 1./216.*m16*m33*RS345pe + 1./54.*pow(m16,2)*
         RS34561pe );

      F13p6x23RV +=  + hhdV(1,m16,m33,m36,m13,L,mu2) * (  - 1./216.*
         pow(m16,2)*RC3456pe - 1./108.*pow(m16,2)*RS345pe - 1./27.*m13*
         RD3456ep - 1./108.*m13*m66*RS34561pe - 1./54.*m13*m66*RC3456pe
          - 1./36.*m13*m66*RS345pe - 2./27.*m13*m36*RS34561pe + 2./27.*
         m13*m36*RC3456pe - 2./27.*m13*m36*RS345pe - 1./54.*m13*m33*
         RS34561pe - 2./27.*m13*m33*RC3456pe + 5./54.*m13*m33*RS345pe
          + 29./216.*m13*m16*RS34561pe - 1./27.*m13*m16*RC3456pe - 13./
         216.*m13*m16*RS345pe - 5./54.*pow(m13,2)*RS34561pe + 5./54.*
         pow(m13,2)*RC3456pe - 5./54.*pow(m13,2)*RS345pe - 1./216.*m11*
         RD3456ep - 1./144.*m11*m66*RS34561pe - 1./432.*m11*m66*
         RC3456pe - 1./144.*m11*m66*RS345pe + 1./72.*m11*m36*RS34561pe
          + 1./108.*m11*m36*RC3456pe + 1./216.*m11*m36*RS345pe - 11./
         432.*m11*m33*RS34561pe - 1./108.*m11*m33*RC3456pe - 1./432.*
         m11*m33*RS345pe - 1./216.*m11*m16*RC3456pe - 1./108.*m11*m16*
         RS345pe - 1./36.*m11*m13*RS34561pe - 1./54.*m11*m13*RC3456pe
          - 1./108.*m11*m13*RS345pe );

      F13p6x23RV +=  + hhdV(1,m16,m33,m36,m13,L,mu2) * (  - 1./216.*
         pow(m11,2)*RS34561pe - 1./864.*pow(m11,2)*RC3456pe - 1./432.*
         pow(m11,2)*RS345pe );

      F13p6x23RV +=  + hhdV(1,m16,m34,m46,m13,L,mu2) * ( 1./288.*pow(
         m66,2) - 1./36.*m46*m66 + 1./18.*pow(m46,2) + 1./144.*m44*m66
          - 1./36.*m44*m46 + 1./288.*pow(m44,2) + 1./72.*m34*m66 - 1./
         18.*m34*m46 + 1./72.*m34*m44 + 1./72.*pow(m34,2) + 1./144.*m33
         *m66 - 1./36.*m33*m46 + 1./144.*m33*m44 + 1./72.*m33*m34 + 1./
         288.*pow(m33,2) + 1./72.*m16*m66 - 1./18.*m16*m46 + 1./72.*m16
         *m44 + 1./36.*m16*m34 + 1./72.*m16*m33 + 1./72.*pow(m16,2) + 1.
         /18.*m13*m66 - 2./9.*m13*m46 + 1./18.*m13*m44 + 1./9.*m13*m34
          + 1./18.*m13*m33 + 1./9.*m13*m16 - 5./18.*pow(m13,2) + 1./144.
         *m11*m66 - 1./36.*m11*m46 + 1./144.*m11*m44 + 1./72.*m11*m34
          + 1./144.*m11*m33 + 1./72.*m11*m16 + 1./18.*m11*m13 + 1./288.
         *pow(m11,2) );

      F13p6x23RV +=  + hhdV(1,m16,m35,m56,m13,L,mu2) * ( 1./288.*pow(
         m66,2) - 1./36.*m56*m66 + 1./18.*pow(m56,2) + 1./144.*m55*m66
          - 1./36.*m55*m56 + 1./288.*pow(m55,2) + 1./72.*m35*m66 - 1./
         18.*m35*m56 + 1./72.*m35*m55 + 1./72.*pow(m35,2) + 1./144.*m33
         *m66 - 1./36.*m33*m56 + 1./144.*m33*m55 + 1./72.*m33*m35 + 1./
         288.*pow(m33,2) + 1./72.*m16*m66 - 1./18.*m16*m56 + 1./72.*m16
         *m55 + 1./36.*m16*m35 + 1./72.*m16*m33 + 1./72.*pow(m16,2) + 1.
         /18.*m13*m66 - 2./9.*m13*m56 + 1./18.*m13*m55 + 1./9.*m13*m35
          + 1./18.*m13*m33 + 1./9.*m13*m16 - 5./18.*pow(m13,2) + 1./144.
         *m11*m66 - 1./36.*m11*m56 + 1./144.*m11*m55 + 1./72.*m11*m35
          + 1./144.*m11*m33 + 1./72.*m11*m16 + 1./18.*m11*m13 + 1./288.
         *pow(m11,2) );

      F13p6x23RV +=  + hhdV(1,m16,m36,m66,m13,L,mu2) * ( 1./72.*pow(
         m66,2) - 1./216.*pow(m66,2)*RS645pe - 1./36.*m36*m66 + 1./108.
         *m36*m66*RS645pe + 1./72.*pow(m36,2) - 1./216.*pow(m36,2)*
         RS645pe - 1./72.*m33*m66 + 1./216.*m33*m66*RS645pe + 1./72.*
         m33*m36 - 1./216.*m33*m36*RS645pe + 1./288.*pow(m33,2) - 1./
         864.*pow(m33,2)*RS645pe - 1./36.*m16*m66 + 1./108.*m16*m66*
         RS645pe + 1./36.*m16*m36 - 1./108.*m16*m36*RS645pe + 1./72.*
         m16*m33 - 1./216.*m16*m33*RS645pe + 1./72.*pow(m16,2) - 1./216.
         *pow(m16,2)*RS645pe - 1./9.*m13*m66 + 1./27.*m13*m66*RS645pe
          + 1./9.*m13*m36 - 1./27.*m13*m36*RS645pe + 1./18.*m13*m33 - 1.
         /54.*m13*m33*RS645pe + 1./9.*m13*m16 - 1./27.*m13*m16*RS645pe
          - 5./18.*pow(m13,2) + 5./54.*pow(m13,2)*RS645pe - 1./72.*m11*
         m66 + 1./216.*m11*m66*RS645pe + 1./72.*m11*m36 - 1./216.*m11*
         m36*RS645pe + 1./144.*m11*m33 - 1./432.*m11*m33*RS645pe + 1./
         72.*m11*m16 - 1./216.*m11*m16*RS645pe + 1./18.*m11*m13 - 1./54.
         *m11*m13*RS645pe );

      F13p6x23RV +=  + hhdV(1,m16,m36,m66,m13,L,mu2) * ( 1./288.*pow(
         m11,2) - 1./864.*pow(m11,2)*RS645pe );

      F13p6x23RV +=  + hhdV(1,m16,m36,mp2,m13,L,mu2) * (  - 1./108.*
         pow(mp2,2)*RSp45613e - 1./216.*pow(mp2,2)*RSp45633e + 1./54.*
         pow(mp2,2)*RSp453e - 1./216.*pow(mp2,2)*RSp45611e + 1./54.*
         pow(mp2,2)*RSp451e - 1./54.*pow(mp2,2)*RSp456e - 1./108.*m66*
         mp2*RSp45613e - 1./216.*m66*mp2*RSp45633e - 1./216.*m66*mp2*
         RSp45611e + 1./54.*m66*mp2*RSp456e - 1./432.*pow(m66,2)*
         RSp45613e - 1./864.*pow(m66,2)*RSp45633e - 1./216.*pow(m66,2)*
         RSp453e - 1./864.*pow(m66,2)*RSp45611e - 1./216.*pow(m66,2)*
         RSp451e - 1./216.*pow(m66,2)*RSp456e + 1./108.*m36*mp2*
         RSp45613e + 1./54.*m36*mp2*RSp45633e - 5./108.*m36*mp2*RSp453e
          - 1./108.*m36*mp2*RSp45611e + 1./108.*m36*mp2*RSp451e + 1./54.
         *m36*mp2*RSp456e + 1./216.*m36*m66*RSp45613e + 1./108.*m36*m66
         *RSp45633e + 1./72.*m36*m66*RSp453e - 1./216.*m36*m66*
         RSp45611e - 1./72.*m36*m66*RSp451e - 1./108.*m36*m66*RSp456e
          + 1./54.*pow(m36,2)*RSp45613e - 1./54.*pow(m36,2)*RSp45633e
          + 1./54.*pow(m36,2)*RSp453e );

      F13p6x23RV +=  + hhdV(1,m16,m36,mp2,m13,L,mu2) * (  - 1./216.*
         pow(m36,2)*RSp45611e - 1./108.*pow(m36,2)*RSp451e - 1./216.*
         pow(m36,2)*RSp456e - 1./72.*m33*mp2*RSp45613e - 1./108.*m33*
         mp2*RSp45633e + 1./72.*m33*mp2*RSp453e - 1./216.*m33*mp2*
         RSp45611e + 1./216.*m33*mp2*RSp451e + 1./108.*m33*mp2*RSp456e
          - 1./144.*m33*m66*RSp45613e - 1./216.*m33*m66*RSp45633e - 5./
         432.*m33*m66*RSp453e - 1./432.*m33*m66*RSp45611e - 1./144.*m33
         *m66*RSp451e - 1./216.*m33*m66*RSp456e + 1./54.*m33*m36*
         RSp45633e - 1./216.*m33*m36*RSp45611e - 1./108.*m33*m36*
         RSp451e - 1./216.*m33*m36*RSp456e - 1./216.*pow(m33,2)*
         RSp45613e - 1./216.*pow(m33,2)*RSp45633e - 1./216.*pow(m33,2)*
         RSp453e - 1./864.*pow(m33,2)*RSp45611e - 1./432.*pow(m33,2)*
         RSp451e - 1./864.*pow(m33,2)*RSp456e + 1./108.*m16*mp2*
         RSp45613e - 1./108.*m16*mp2*RSp45633e + 1./108.*m16*mp2*
         RSp453e + 1./54.*m16*mp2*RSp45611e - 5./108.*m16*mp2*RSp451e
          + 1./54.*m16*mp2*RSp456e );

      F13p6x23RV +=  + hhdV(1,m16,m36,mp2,m13,L,mu2) * ( 1./216.*m16*
         m66*RSp45613e - 1./216.*m16*m66*RSp45633e - 1./72.*m16*m66*
         RSp453e + 1./108.*m16*m66*RSp45611e + 1./72.*m16*m66*RSp451e
          - 1./108.*m16*m66*RSp456e - 5./108.*m16*m36*RSp45613e + 1./54.
         *m16*m36*RSp45633e + 1./108.*m16*m36*RSp453e + 1./54.*m16*m36*
         RSp45611e + 1./108.*m16*m36*RSp451e - 1./108.*m16*m36*RSp456e
          + 1./72.*m16*m33*RSp45613e - 1./108.*m16*m33*RSp45633e - 1./
         72.*m16*m33*RSp453e + 1./108.*m16*m33*RSp45611e + 1./216.*m16*
         m33*RSp451e - 1./216.*m16*m33*RSp456e + 1./54.*pow(m16,2)*
         RSp45613e - 1./216.*pow(m16,2)*RSp45633e - 1./108.*pow(m16,2)*
         RSp453e - 1./54.*pow(m16,2)*RSp45611e + 1./54.*pow(m16,2)*
         RSp451e - 1./216.*pow(m16,2)*RSp456e + 1./27.*m13*mp2*
         RSp45613e + 1./54.*m13*mp2*RSp45633e - 2./27.*m13*mp2*RSp453e
          + 1./54.*m13*mp2*RSp45611e - 2./27.*m13*mp2*RSp451e + 2./27.*
         m13*mp2*RSp456e + 1./54.*m13*m66*RSp45613e + 1./108.*m13*m66*
         RSp45633e );

      F13p6x23RV +=  + hhdV(1,m16,m36,mp2,m13,L,mu2) * ( 1./108.*m13*
         m66*RSp45611e - 1./27.*m13*m66*RSp456e - 1./54.*m13*m36*
         RSp45613e - 1./27.*m13*m36*RSp45633e + 5./54.*m13*m36*RSp453e
          + 1./54.*m13*m36*RSp45611e - 1./54.*m13*m36*RSp451e - 1./27.*
         m13*m36*RSp456e + 1./36.*m13*m33*RSp45613e + 1./54.*m13*m33*
         RSp45633e - 1./36.*m13*m33*RSp453e + 1./108.*m13*m33*RSp45611e
          - 1./108.*m13*m33*RSp451e - 1./54.*m13*m33*RSp456e + 5./216.*
         m13*m16*RSp45613e + 1./54.*m13*m16*RSp45633e - 13./216.*m13*
         m16*RSp453e - 17./216.*m13*m16*RSp45611e + 29./216.*m13*m16*
         RSp451e - 1./27.*m13*m16*RSp456e + 7./54.*pow(m13,2)*RSp45613e
          - 1./54.*pow(m13,2)*RSp45633e - 5./54.*pow(m13,2)*RSp453e - 1.
         /54.*pow(m13,2)*RSp45611e - 5./54.*pow(m13,2)*RSp451e + 5./54.
         *pow(m13,2)*RSp456e - 1./72.*m11*mp2*RSp45613e - 1./216.*m11*
         mp2*RSp45633e + 1./216.*m11*mp2*RSp453e - 1./108.*m11*mp2*
         RSp45611e + 1./72.*m11*mp2*RSp451e + 1./108.*m11*mp2*RSp456e
          - 1./144.*m11*m66*RSp45613e );

      F13p6x23RV +=  + hhdV(1,m16,m36,mp2,m13,L,mu2) * (  - 1./432.*m11
         *m66*RSp45633e - 1./144.*m11*m66*RSp453e - 1./216.*m11*m66*
         RSp45611e - 5./432.*m11*m66*RSp451e - 1./216.*m11*m66*RSp456e
          + 1./72.*m11*m36*RSp45613e + 1./108.*m11*m36*RSp45633e + 1./
         216.*m11*m36*RSp453e - 1./108.*m11*m36*RSp45611e - 1./72.*m11*
         m36*RSp451e - 1./216.*m11*m36*RSp456e - 5./432.*m11*m33*
         RSp45613e - 1./216.*m11*m33*RSp45633e - 1./144.*m11*m33*
         RSp453e - 1./216.*m11*m33*RSp45611e - 1./144.*m11*m33*RSp451e
          - 1./432.*m11*m33*RSp456e - 1./216.*m11*m16*RSp45633e - 1./
         108.*m11*m16*RSp453e + 1./54.*m11*m16*RSp45611e - 1./216.*m11*
         m16*RSp456e + 1./36.*m11*m13*RSp45613e + 1./108.*m11*m13*
         RSp45633e - 1./108.*m11*m13*RSp453e + 1./54.*m11*m13*RSp45611e
          - 1./36.*m11*m13*RSp451e - 1./54.*m11*m13*RSp456e - 1./216.*
         pow(m11,2)*RSp45613e - 1./864.*pow(m11,2)*RSp45633e - 1./432.*
         pow(m11,2)*RSp453e - 1./216.*pow(m11,2)*RSp45611e - 1./216.*
         pow(m11,2)*RSp451e );

      F13p6x23RV +=  + hhdV(1,m16,m36,mp2,m13,L,mu2) * (  - 1./864.*
         pow(m11,2)*RSp456e );

      F13p6x23RV +=  + hhdV(1,m16,m36,me2,m13,L,mu2) * (  - 1./108.*
         pow(me2,2)*RSe45613p - 1./216.*pow(me2,2)*RSe45633p + 1./54.*
         pow(me2,2)*RSe453p - 1./216.*pow(me2,2)*RSe45611p + 1./54.*
         pow(me2,2)*RSe451p - 1./54.*pow(me2,2)*RSe456p - 1./108.*m66*
         me2*RSe45613p - 1./216.*m66*me2*RSe45633p - 1./216.*m66*me2*
         RSe45611p + 1./54.*m66*me2*RSe456p - 1./432.*pow(m66,2)*
         RSe45613p - 1./864.*pow(m66,2)*RSe45633p - 1./216.*pow(m66,2)*
         RSe453p - 1./864.*pow(m66,2)*RSe45611p - 1./216.*pow(m66,2)*
         RSe451p - 1./216.*pow(m66,2)*RSe456p + 1./108.*m36*me2*
         RSe45613p + 1./54.*m36*me2*RSe45633p - 5./108.*m36*me2*RSe453p
          - 1./108.*m36*me2*RSe45611p + 1./108.*m36*me2*RSe451p + 1./54.
         *m36*me2*RSe456p + 1./216.*m36*m66*RSe45613p + 1./108.*m36*m66
         *RSe45633p + 1./72.*m36*m66*RSe453p - 1./216.*m36*m66*
         RSe45611p - 1./72.*m36*m66*RSe451p - 1./108.*m36*m66*RSe456p
          + 1./54.*pow(m36,2)*RSe45613p - 1./54.*pow(m36,2)*RSe45633p
          + 1./54.*pow(m36,2)*RSe453p );

      F13p6x23RV +=  + hhdV(1,m16,m36,me2,m13,L,mu2) * (  - 1./216.*
         pow(m36,2)*RSe45611p - 1./108.*pow(m36,2)*RSe451p - 1./216.*
         pow(m36,2)*RSe456p - 1./72.*m33*me2*RSe45613p - 1./108.*m33*
         me2*RSe45633p + 1./72.*m33*me2*RSe453p - 1./216.*m33*me2*
         RSe45611p + 1./216.*m33*me2*RSe451p + 1./108.*m33*me2*RSe456p
          - 1./144.*m33*m66*RSe45613p - 1./216.*m33*m66*RSe45633p - 5./
         432.*m33*m66*RSe453p - 1./432.*m33*m66*RSe45611p - 1./144.*m33
         *m66*RSe451p - 1./216.*m33*m66*RSe456p + 1./54.*m33*m36*
         RSe45633p - 1./216.*m33*m36*RSe45611p - 1./108.*m33*m36*
         RSe451p - 1./216.*m33*m36*RSe456p - 1./216.*pow(m33,2)*
         RSe45613p - 1./216.*pow(m33,2)*RSe45633p - 1./216.*pow(m33,2)*
         RSe453p - 1./864.*pow(m33,2)*RSe45611p - 1./432.*pow(m33,2)*
         RSe451p - 1./864.*pow(m33,2)*RSe456p + 1./108.*m16*me2*
         RSe45613p - 1./108.*m16*me2*RSe45633p + 1./108.*m16*me2*
         RSe453p + 1./54.*m16*me2*RSe45611p - 5./108.*m16*me2*RSe451p
          + 1./54.*m16*me2*RSe456p );

      F13p6x23RV +=  + hhdV(1,m16,m36,me2,m13,L,mu2) * ( 1./216.*m16*
         m66*RSe45613p - 1./216.*m16*m66*RSe45633p - 1./72.*m16*m66*
         RSe453p + 1./108.*m16*m66*RSe45611p + 1./72.*m16*m66*RSe451p
          - 1./108.*m16*m66*RSe456p - 5./108.*m16*m36*RSe45613p + 1./54.
         *m16*m36*RSe45633p + 1./108.*m16*m36*RSe453p + 1./54.*m16*m36*
         RSe45611p + 1./108.*m16*m36*RSe451p - 1./108.*m16*m36*RSe456p
          + 1./72.*m16*m33*RSe45613p - 1./108.*m16*m33*RSe45633p - 1./
         72.*m16*m33*RSe453p + 1./108.*m16*m33*RSe45611p + 1./216.*m16*
         m33*RSe451p - 1./216.*m16*m33*RSe456p + 1./54.*pow(m16,2)*
         RSe45613p - 1./216.*pow(m16,2)*RSe45633p - 1./108.*pow(m16,2)*
         RSe453p - 1./54.*pow(m16,2)*RSe45611p + 1./54.*pow(m16,2)*
         RSe451p - 1./216.*pow(m16,2)*RSe456p + 1./27.*m13*me2*
         RSe45613p + 1./54.*m13*me2*RSe45633p - 2./27.*m13*me2*RSe453p
          + 1./54.*m13*me2*RSe45611p - 2./27.*m13*me2*RSe451p + 2./27.*
         m13*me2*RSe456p + 1./54.*m13*m66*RSe45613p + 1./108.*m13*m66*
         RSe45633p );

      F13p6x23RV +=  + hhdV(1,m16,m36,me2,m13,L,mu2) * ( 1./108.*m13*
         m66*RSe45611p - 1./27.*m13*m66*RSe456p - 1./54.*m13*m36*
         RSe45613p - 1./27.*m13*m36*RSe45633p + 5./54.*m13*m36*RSe453p
          + 1./54.*m13*m36*RSe45611p - 1./54.*m13*m36*RSe451p - 1./27.*
         m13*m36*RSe456p + 1./36.*m13*m33*RSe45613p + 1./54.*m13*m33*
         RSe45633p - 1./36.*m13*m33*RSe453p + 1./108.*m13*m33*RSe45611p
          - 1./108.*m13*m33*RSe451p - 1./54.*m13*m33*RSe456p + 5./216.*
         m13*m16*RSe45613p + 1./54.*m13*m16*RSe45633p - 13./216.*m13*
         m16*RSe453p - 17./216.*m13*m16*RSe45611p + 29./216.*m13*m16*
         RSe451p - 1./27.*m13*m16*RSe456p + 7./54.*pow(m13,2)*RSe45613p
          - 1./54.*pow(m13,2)*RSe45633p - 5./54.*pow(m13,2)*RSe453p - 1.
         /54.*pow(m13,2)*RSe45611p - 5./54.*pow(m13,2)*RSe451p + 5./54.
         *pow(m13,2)*RSe456p - 1./72.*m11*me2*RSe45613p - 1./216.*m11*
         me2*RSe45633p + 1./216.*m11*me2*RSe453p - 1./108.*m11*me2*
         RSe45611p + 1./72.*m11*me2*RSe451p + 1./108.*m11*me2*RSe456p
          - 1./144.*m11*m66*RSe45613p );

      F13p6x23RV +=  + hhdV(1,m16,m36,me2,m13,L,mu2) * (  - 1./432.*m11
         *m66*RSe45633p - 1./144.*m11*m66*RSe453p - 1./216.*m11*m66*
         RSe45611p - 5./432.*m11*m66*RSe451p - 1./216.*m11*m66*RSe456p
          + 1./72.*m11*m36*RSe45613p + 1./108.*m11*m36*RSe45633p + 1./
         216.*m11*m36*RSe453p - 1./108.*m11*m36*RSe45611p - 1./72.*m11*
         m36*RSe451p - 1./216.*m11*m36*RSe456p - 5./432.*m11*m33*
         RSe45613p - 1./216.*m11*m33*RSe45633p - 1./144.*m11*m33*
         RSe453p - 1./216.*m11*m33*RSe45611p - 1./144.*m11*m33*RSe451p
          - 1./432.*m11*m33*RSe456p - 1./216.*m11*m16*RSe45633p - 1./
         108.*m11*m16*RSe453p + 1./54.*m11*m16*RSe45611p - 1./216.*m11*
         m16*RSe456p + 1./36.*m11*m13*RSe45613p + 1./108.*m11*m13*
         RSe45633p - 1./108.*m11*m13*RSe453p + 1./54.*m11*m13*RSe45611p
          - 1./36.*m11*m13*RSe451p - 1./54.*m11*m13*RSe456p - 1./216.*
         pow(m11,2)*RSe45613p - 1./864.*pow(m11,2)*RSe45633p - 1./432.*
         pow(m11,2)*RSe453p - 1./216.*pow(m11,2)*RSe45611p - 1./216.*
         pow(m11,2)*RSe451p );

      F13p6x23RV +=  + hhdV(1,m16,m36,me2,m13,L,mu2) * (  - 1./864.*
         pow(m11,2)*RSe456p );

      F13p6x23RV +=  + hhdV(1,m33,m13,m33,m13,L,mu2) * (  - 1./8.*m13*
         m33 );

      F13p6x23RV +=  + hhdV(1,m33,m14,m34,m13,L,mu2) * (  - 1./24.*m13*
         m33*RS34561pe + 1./24.*m13*m33*RS356pe );

      F13p6x23RV +=  + hhdV(1,m33,m15,m35,m13,L,mu2) * (  - 1./24.*m13*
         m33*RS34561pe + 1./24.*m13*m33*RS346pe );

      F13p6x23RV +=  + hhdV(1,m33,m16,m36,m13,L,mu2) * (  - 1./24.*m13*
         m33*RS34561pe + 1./24.*m13*m33*RS345pe );

      F13p6x23RV +=  + hhdV(1,m34,m11,m14,m13,L,mu2) * ( 1./24.*m13*m34
         *RS14563pe - 1./24.*m13*m34*RS156pe );

      F13p6x23RV +=  + hhdV(1,m34,m14,m33,m13,L,mu2) * ( 1./24.*m13*m34
         *RS34561pe - 1./24.*m13*m34*RC3456pe + 1./24.*m13*m34*RS356pe
          );

      F13p6x23RV +=  + hhdV(1,m34,m14,mp2,m13,L,mu2) * ( 1./24.*m13*m34
         *RSp45613e - 1./24.*m13*m34*RSp45633e + 1./24.*m13*m34*RSp563e
          - 1./24.*m13*m34*RSp561e );

      F13p6x23RV +=  + hhdV(1,m34,m14,me2,m13,L,mu2) * ( 1./24.*m13*m34
         *RSe45613p - 1./24.*m13*m34*RSe45633p + 1./24.*m13*m34*RSe563p
          - 1./24.*m13*m34*RSe561p );

      F13p6x23RV +=  + hhdV(1,m35,m11,m15,m13,L,mu2) * ( 1./24.*m13*m35
         *RS14563pe - 1./24.*m13*m35*RS146pe );

      F13p6x23RV +=  + hhdV(1,m35,m15,m33,m13,L,mu2) * ( 1./24.*m13*m35
         *RS34561pe - 1./24.*m13*m35*RC3456pe + 1./24.*m13*m35*RS346pe
          );

      F13p6x23RV +=  + hhdV(1,m35,m15,mp2,m13,L,mu2) * ( 1./24.*m13*m35
         *RSp45613e - 1./24.*m13*m35*RSp45633e + 1./24.*m13*m35*RSp463e
          - 1./24.*m13*m35*RSp461e );

      F13p6x23RV +=  + hhdV(1,m35,m15,me2,m13,L,mu2) * ( 1./24.*m13*m35
         *RSe45613p - 1./24.*m13*m35*RSe45633p + 1./24.*m13*m35*RSe463p
          - 1./24.*m13*m35*RSe461p );

      F13p6x23RV +=  + hhdV(1,m36,m11,m16,m13,L,mu2) * ( 1./24.*m13*m36
         *RS14563pe - 1./24.*m13*m36*RS145pe );

      F13p6x23RV +=  + hhdV(1,m36,m16,m33,m13,L,mu2) * ( 1./24.*m13*m36
         *RS34561pe - 1./24.*m13*m36*RC3456pe + 1./24.*m13*m36*RS345pe
          );

      F13p6x23RV +=  + hhdV(1,m36,m16,mp2,m13,L,mu2) * ( 1./24.*m13*m36
         *RSp45613e - 1./24.*m13*m36*RSp45633e + 1./24.*m13*m36*RSp453e
          - 1./24.*m13*m36*RSp451e );

      F13p6x23RV +=  + hhdV(1,m36,m16,me2,m13,L,mu2) * ( 1./24.*m13*m36
         *RSe45613p - 1./24.*m13*m36*RSe45633p + 1./24.*m13*m36*RSe453p
          - 1./24.*m13*m36*RSe451p );

      F13p6x23RV +=  + hhdV(1,m44,m14,m34,m13,L,mu2) * ( 1./8.*m13*m44
          - 1./24.*m13*m44*RS456pe );

      F13p6x23RV +=  + hhdV(1,m45,m14,m35,m13,L,mu2) * ( 1./8.*m13*m45
          );

      F13p6x23RV +=  + hhdV(1,m45,m15,m34,m13,L,mu2) * ( 1./8.*m13*m45
          );

      F13p6x23RV +=  + hhdV(1,m46,m14,m36,m13,L,mu2) * ( 1./8.*m13*m46
          );

      F13p6x23RV +=  + hhdV(1,m46,m16,m34,m13,L,mu2) * ( 1./8.*m13*m46
          );

      F13p6x23RV +=  + hhdV(1,m55,m15,m35,m13,L,mu2) * ( 1./8.*m13*m55
          - 1./24.*m13*m55*RS546pe );

      F13p6x23RV +=  + hhdV(1,m56,m15,m36,m13,L,mu2) * ( 1./8.*m13*m56
          );

      F13p6x23RV +=  + hhdV(1,m56,m16,m35,m13,L,mu2) * ( 1./8.*m13*m56
          );

      F13p6x23RV +=  + hhdV(1,m66,m16,m36,m13,L,mu2) * ( 1./8.*m13*m66
          - 1./24.*m13*m66*RS645pe );

      F13p6x23RV +=  + hhdV(1,mp2,m14,m34,m13,L,mu2) * (  - 1./24.*m13*
         mp2*RSp45613e + 1./24.*m13*mp2*RSp563e + 1./24.*m13*mp2*
         RSp561e - 1./24.*m13*mp2*RSp564e );

      F13p6x23RV +=  + hhdV(1,mp2,m15,m35,m13,L,mu2) * (  - 1./24.*m13*
         mp2*RSp45613e + 1./24.*m13*mp2*RSp463e + 1./24.*m13*mp2*
         RSp461e - 1./24.*m13*mp2*RSp465e );

      F13p6x23RV +=  + hhdV(1,mp2,m16,m36,m13,L,mu2) * (  - 1./24.*m13*
         mp2*RSp45613e + 1./24.*m13*mp2*RSp453e + 1./24.*m13*mp2*
         RSp451e - 1./24.*m13*mp2*RSp456e );

      F13p6x23RV +=  + hhdV(1,me2,m14,m34,m13,L,mu2) * (  - 1./24.*m13*
         me2*RSe45613p + 1./24.*m13*me2*RSe563p + 1./24.*m13*me2*
         RSe561p - 1./24.*m13*me2*RSe564p );

      F13p6x23RV +=  + hhdV(1,me2,m15,m35,m13,L,mu2) * (  - 1./24.*m13*
         me2*RSe45613p + 1./24.*m13*me2*RSe463p + 1./24.*m13*me2*
         RSe461p - 1./24.*m13*me2*RSe465p );

      F13p6x23RV +=  + hhdV(1,me2,m16,m36,m13,L,mu2) * (  - 1./24.*m13*
         me2*RSe45613p + 1./24.*m13*me2*RSe453p + 1./24.*m13*me2*
         RSe451p - 1./24.*m13*me2*RSe456p );

      F13p6x23RV +=  + hhdV(2,m11,m11,m13,m13,L,mu2) * (  - 1./324.*m33
         *pow(RD1456ep,2) + 1./324.*pow(m33,2)*RD1456ep*RS14563pe + 1./
         648.*pow(m33,2)*RD1456ep*RC1456pe + 1./162.*m13*pow(
         RD1456ep,2) - 1./162.*m13*m33*RD1456ep*RC1456pe + 8./81.*pow(
         m13,2)*RD1456ep*RS14563pe - 4./81.*pow(m13,2)*RD1456ep*
         RC1456pe - 1./324.*m11*pow(RD1456ep,2) + 1./81.*m11*m33*
         RD1456ep*RS14563pe + 1./324.*m11*m33*RD1456ep*RC1456pe - 1./81.
         *m11*m13*RD1456ep*RS14563pe - 1./162.*m11*m13*RD1456ep*
         RC1456pe + 1./108.*pow(m11,2)*RD1456ep*RS14563pe + 1./648.*
         pow(m11,2)*RD1456ep*RC1456pe );

      F13p6x23RV +=  + hhdV(2,m11,m13,m33,m13,L,mu2) * ( 1./162.*m33*
         RD1456ep*RD3456ep + 1./162.*pow(m33,2)*RD1456ep*RC3456pe - 1./
         81.*m13*RD1456ep*RD3456ep + 2./81.*m13*m33*RD1456ep*RS34561pe
          - 2./81.*m13*m33*RD1456ep*RC3456pe - 4./81.*pow(m13,2)*
         RD1456ep*RS34561pe + 2./81.*pow(m13,2)*RD1456ep*RC3456pe + 1./
         162.*m11*RD1456ep*RD3456ep + 1./81.*m11*m33*RD1456ep*RS34561pe
          + 1./81.*m11*m33*RD1456ep*RC3456pe - 2./81.*m11*m13*RD1456ep*
         RC3456pe + 1./81.*pow(m11,2)*RD1456ep*RS34561pe + 1./162.*pow(
         m11,2)*RD1456ep*RC3456pe );

      F13p6x23RV +=  + hhdV(2,m11,m13,mp2,m13,L,mu2) * (  - 1./324.*
         pow(mp2,2)*RD1456ep*RSp45613e + 1./648.*pow(mp2,2)*RD1456ep*
         RSp45633e + 1./648.*pow(mp2,2)*RSp45611e*RD1456ep + 1./324.*
         m33*mp2*RD1456ep*RSp45633e - 1./324.*m33*mp2*RSp45611e*
         RD1456ep + 1./324.*pow(m33,2)*RD1456ep*RSp45613e + 1./648.*
         pow(m33,2)*RD1456ep*RSp45633e + 1./648.*pow(m33,2)*RSp45611e*
         RD1456ep + 2./81.*m13*mp2*RD1456ep*RSp45613e - 1./81.*m13*mp2*
         RD1456ep*RSp45633e - 1./81.*m13*mp2*RSp45611e*RD1456ep - 1./81.
         *m13*m33*RD1456ep*RSp45633e + 1./81.*m13*m33*RSp45611e*
         RD1456ep - 4./81.*pow(m13,2)*RD1456ep*RSp45613e + 2./81.*pow(
         m13,2)*RD1456ep*RSp45633e + 2./81.*pow(m13,2)*RSp45611e*
         RD1456ep + 1./162.*m11*mp2*RD1456ep*RSp45633e - 1./162.*m11*
         mp2*RSp45611e*RD1456ep + 1./81.*m11*m33*RD1456ep*RSp45613e + 1.
         /162.*m11*m33*RD1456ep*RSp45633e + 1./162.*m11*m33*RSp45611e*
         RD1456ep - 2./81.*m11*m13*RD1456ep*RSp45633e + 2./81.*m11*m13*
         RSp45611e*RD1456ep );

      F13p6x23RV +=  + hhdV(2,m11,m13,mp2,m13,L,mu2) * ( 1./81.*pow(
         m11,2)*RD1456ep*RSp45613e + 1./162.*pow(m11,2)*RD1456ep*
         RSp45633e + 1./162.*pow(m11,2)*RSp45611e*RD1456ep );

      F13p6x23RV +=  + hhdV(2,m11,m13,me2,m13,L,mu2) * (  - 1./324.*
         pow(me2,2)*RD1456ep*RSe45613p + 1./648.*pow(me2,2)*RD1456ep*
         RSe45633p + 1./648.*pow(me2,2)*RSe45611p*RD1456ep + 1./324.*
         m33*me2*RD1456ep*RSe45633p - 1./324.*m33*me2*RSe45611p*
         RD1456ep + 1./324.*pow(m33,2)*RD1456ep*RSe45613p + 1./648.*
         pow(m33,2)*RD1456ep*RSe45633p + 1./648.*pow(m33,2)*RSe45611p*
         RD1456ep + 2./81.*m13*me2*RD1456ep*RSe45613p - 1./81.*m13*me2*
         RD1456ep*RSe45633p - 1./81.*m13*me2*RSe45611p*RD1456ep - 1./81.
         *m13*m33*RD1456ep*RSe45633p + 1./81.*m13*m33*RSe45611p*
         RD1456ep - 4./81.*pow(m13,2)*RD1456ep*RSe45613p + 2./81.*pow(
         m13,2)*RD1456ep*RSe45633p + 2./81.*pow(m13,2)*RSe45611p*
         RD1456ep + 1./162.*m11*me2*RD1456ep*RSe45633p - 1./162.*m11*
         me2*RSe45611p*RD1456ep + 1./81.*m11*m33*RD1456ep*RSe45613p + 1.
         /162.*m11*m33*RD1456ep*RSe45633p + 1./162.*m11*m33*RSe45611p*
         RD1456ep - 2./81.*m11*m13*RD1456ep*RSe45633p + 2./81.*m11*m13*
         RSe45611p*RD1456ep );

      F13p6x23RV +=  + hhdV(2,m11,m13,me2,m13,L,mu2) * ( 1./81.*pow(
         m11,2)*RD1456ep*RSe45613p + 1./162.*pow(m11,2)*RD1456ep*
         RSe45633p + 1./162.*pow(m11,2)*RSe45611p*RD1456ep );

      F13p6x23RV +=  + hhdV(2,m11,m14,m34,m13,L,mu2) * (  - 1./864.*
         pow(m44,2)*RD1456ep - 1./216.*m34*m44*RD1456ep - 1./216.*pow(
         m34,2)*RD1456ep - 1./432.*m33*m44*RD1456ep - 1./216.*m33*m34*
         RD1456ep - 1./864.*pow(m33,2)*RD1456ep + 1./108.*m14*m44*
         RD1456ep + 1./54.*m14*m34*RD1456ep + 1./108.*m14*m33*RD1456ep
          - 1./54.*pow(m14,2)*RD1456ep + 1./108.*m13*m44*RD1456ep + 1./
         54.*m13*m34*RD1456ep + 1./108.*m13*m33*RD1456ep - 1./27.*m13*
         m14*RD1456ep - 1./54.*pow(m13,2)*RD1456ep - 1./108.*m11*m44*
         RD1456ep - 1./54.*m11*m34*RD1456ep - 1./108.*m11*m33*RD1456ep
          + 1./27.*m11*m14*RD1456ep + 1./27.*m11*m13*RD1456ep - 1./54.*
         pow(m11,2)*RD1456ep );

      F13p6x23RV +=  + hhdV(2,m11,m15,m35,m13,L,mu2) * (  - 1./864.*
         pow(m55,2)*RD1456ep - 1./216.*m35*m55*RD1456ep - 1./216.*pow(
         m35,2)*RD1456ep - 1./432.*m33*m55*RD1456ep - 1./216.*m33*m35*
         RD1456ep - 1./864.*pow(m33,2)*RD1456ep + 1./108.*m15*m55*
         RD1456ep + 1./54.*m15*m35*RD1456ep + 1./108.*m15*m33*RD1456ep
          - 1./54.*pow(m15,2)*RD1456ep + 1./108.*m13*m55*RD1456ep + 1./
         54.*m13*m35*RD1456ep + 1./108.*m13*m33*RD1456ep - 1./27.*m13*
         m15*RD1456ep - 1./54.*pow(m13,2)*RD1456ep - 1./108.*m11*m55*
         RD1456ep - 1./54.*m11*m35*RD1456ep - 1./108.*m11*m33*RD1456ep
          + 1./27.*m11*m15*RD1456ep + 1./27.*m11*m13*RD1456ep - 1./54.*
         pow(m11,2)*RD1456ep );

      F13p6x23RV +=  + hhdV(2,m11,m16,m36,m13,L,mu2) * (  - 1./864.*
         pow(m66,2)*RD1456ep - 1./216.*m36*m66*RD1456ep - 1./216.*pow(
         m36,2)*RD1456ep - 1./432.*m33*m66*RD1456ep - 1./216.*m33*m36*
         RD1456ep - 1./864.*pow(m33,2)*RD1456ep + 1./108.*m16*m66*
         RD1456ep + 1./54.*m16*m36*RD1456ep + 1./108.*m16*m33*RD1456ep
          - 1./54.*pow(m16,2)*RD1456ep + 1./108.*m13*m66*RD1456ep + 1./
         54.*m13*m36*RD1456ep + 1./108.*m13*m33*RD1456ep - 1./27.*m13*
         m16*RD1456ep - 1./54.*pow(m13,2)*RD1456ep - 1./108.*m11*m66*
         RD1456ep - 1./54.*m11*m36*RD1456ep - 1./108.*m11*m33*RD1456ep
          + 1./27.*m11*m16*RD1456ep + 1./27.*m11*m13*RD1456ep - 1./54.*
         pow(m11,2)*RD1456ep );

      F13p6x23RV +=  + hhdV(3,m13,m11,m11,m13,L,mu2) * (  - 1./36.*pow(
         m13,2)*RD1456ep*RS14563pe + 1./72.*pow(m13,2)*RD1456ep*
         RC1456pe );

      F13p6x23RV +=  + hhdV(3,m13,m11,m33,m13,L,mu2) * (  - 1./36.*pow(
         m13,2)*RD1456ep*RS34561pe + 1./72.*pow(m13,2)*RD1456ep*
         RC3456pe );

      F13p6x23RV +=  + hhdV(3,m13,m11,mp2,m13,L,mu2) * (  - 1./36.*pow(
         m13,2)*RD1456ep*RSp45613e + 1./72.*pow(m13,2)*RD1456ep*
         RSp45633e + 1./72.*pow(m13,2)*RSp45611e*RD1456ep );

      F13p6x23RV +=  + hhdV(3,m13,m11,me2,m13,L,mu2) * (  - 1./36.*pow(
         m13,2)*RD1456ep*RSe45613p + 1./72.*pow(m13,2)*RD1456ep*
         RSe45633p + 1./72.*pow(m13,2)*RSe45611p*RD1456ep );

      F13p6x23RV +=  + hhdV(3,m13,m33,m33,m13,L,mu2) * (  - 1./324.*m33
         *pow(RD3456ep,2) + 1./108.*pow(m33,2)*RD3456ep*RS34561pe + 1./
         648.*pow(m33,2)*RD3456ep*RC3456pe - 5./432.*m13*pow(
         RD3456ep,2) + 2./81.*m13*m33*RD3456ep*RS34561pe + 1./81.*m13*
         m33*RD3456ep*RC3456pe - 25./324.*pow(m13,2)*RD3456ep*RS34561pe
          + 25./648.*pow(m13,2)*RD3456ep*RC3456pe - 1./324.*m11*pow(
         RD3456ep,2) + 1./81.*m11*m33*RD3456ep*RS34561pe + 1./324.*m11*
         m33*RD3456ep*RC3456pe + 1./81.*m11*m13*RD3456ep*RC3456pe + 1./
         324.*pow(m11,2)*RD3456ep*RS34561pe + 1./648.*pow(m11,2)*
         RD3456ep*RC3456pe );

      F13p6x23RV +=  + hhdV(3,m13,m33,mp2,m13,L,mu2) * (  - 1./324.*
         pow(mp2,2)*RD3456ep*RSp45613e + 1./648.*pow(mp2,2)*RSp45633e*
         RD3456ep + 1./648.*pow(mp2,2)*RSp45611e*RD3456ep - 1./162.*m33
         *mp2*RSp45633e*RD3456ep + 1./162.*m33*mp2*RSp45611e*RD3456ep
          + 1./81.*pow(m33,2)*RD3456ep*RSp45613e + 1./162.*pow(m33,2)*
         RSp45633e*RD3456ep + 1./162.*pow(m33,2)*RSp45611e*RD3456ep + 2.
         /81.*m13*mp2*RD3456ep*RSp45613e - 1./81.*m13*mp2*RSp45633e*
         RD3456ep - 1./81.*m13*mp2*RSp45611e*RD3456ep + 2./81.*m13*m33*
         RSp45633e*RD3456ep - 2./81.*m13*m33*RSp45611e*RD3456ep - 25./
         324.*pow(m13,2)*RD3456ep*RSp45613e + 25./648.*pow(m13,2)*
         RSp45633e*RD3456ep + 25./648.*pow(m13,2)*RSp45611e*RD3456ep - 
         1./324.*m11*mp2*RSp45633e*RD3456ep + 1./324.*m11*mp2*RSp45611e
         *RD3456ep + 1./81.*m11*m33*RD3456ep*RSp45613e + 1./162.*m11*
         m33*RSp45633e*RD3456ep + 1./162.*m11*m33*RSp45611e*RD3456ep + 
         1./81.*m11*m13*RSp45633e*RD3456ep - 1./81.*m11*m13*RSp45611e*
         RD3456ep );

      F13p6x23RV +=  + hhdV(3,m13,m33,mp2,m13,L,mu2) * ( 1./324.*pow(
         m11,2)*RD3456ep*RSp45613e + 1./648.*pow(m11,2)*RSp45633e*
         RD3456ep + 1./648.*pow(m11,2)*RSp45611e*RD3456ep );

      F13p6x23RV +=  + hhdV(3,m13,m33,me2,m13,L,mu2) * (  - 1./324.*
         pow(me2,2)*RD3456ep*RSe45613p + 1./648.*pow(me2,2)*RSe45633p*
         RD3456ep + 1./648.*pow(me2,2)*RSe45611p*RD3456ep - 1./162.*m33
         *me2*RSe45633p*RD3456ep + 1./162.*m33*me2*RSe45611p*RD3456ep
          + 1./81.*pow(m33,2)*RD3456ep*RSe45613p + 1./162.*pow(m33,2)*
         RSe45633p*RD3456ep + 1./162.*pow(m33,2)*RSe45611p*RD3456ep + 2.
         /81.*m13*me2*RD3456ep*RSe45613p - 1./81.*m13*me2*RSe45633p*
         RD3456ep - 1./81.*m13*me2*RSe45611p*RD3456ep + 2./81.*m13*m33*
         RSe45633p*RD3456ep - 2./81.*m13*m33*RSe45611p*RD3456ep - 25./
         324.*pow(m13,2)*RD3456ep*RSe45613p + 25./648.*pow(m13,2)*
         RSe45633p*RD3456ep + 25./648.*pow(m13,2)*RSe45611p*RD3456ep - 
         1./324.*m11*me2*RSe45633p*RD3456ep + 1./324.*m11*me2*RSe45611p
         *RD3456ep + 1./81.*m11*m33*RD3456ep*RSe45613p + 1./162.*m11*
         m33*RSe45633p*RD3456ep + 1./162.*m11*m33*RSe45611p*RD3456ep + 
         1./81.*m11*m13*RSe45633p*RD3456ep - 1./81.*m11*m13*RSe45611p*
         RD3456ep );

      F13p6x23RV +=  + hhdV(3,m13,m33,me2,m13,L,mu2) * ( 1./324.*pow(
         m11,2)*RD3456ep*RSe45613p + 1./648.*pow(m11,2)*RSe45633p*
         RD3456ep + 1./648.*pow(m11,2)*RSe45611p*RD3456ep );

      F13p6x23RV +=  + hhdV(3,m14,m11,m34,m13,L,mu2) * (  - 1./24.*m13*
         m14*RD1456ep );

      F13p6x23RV +=  + hhdV(3,m14,m33,m34,m13,L,mu2) * (  - 1./864.*
         pow(m44,2)*RD3456ep + 1./108.*m34*m44*RD3456ep - 1./54.*pow(
         m34,2)*RD3456ep - 1./108.*m33*m44*RD3456ep + 1./27.*m33*m34*
         RD3456ep - 1./54.*pow(m33,2)*RD3456ep - 1./216.*m14*m44*
         RD3456ep + 1./54.*m14*m34*RD3456ep - 1./54.*m14*m33*RD3456ep
          - 1./216.*pow(m14,2)*RD3456ep - 1./54.*m13*m44*RD3456ep + 2./
         27.*m13*m34*RD3456ep - 2./27.*m13*m33*RD3456ep - 1./27.*m13*
         m14*RD3456ep + 5./54.*pow(m13,2)*RD3456ep - 1./432.*m11*m44*
         RD3456ep + 1./108.*m11*m34*RD3456ep - 1./108.*m11*m33*RD3456ep
          - 1./216.*m11*m14*RD3456ep - 1./54.*m11*m13*RD3456ep - 1./864.
         *pow(m11,2)*RD3456ep );

      F13p6x23RV +=  + hhdV(3,m15,m11,m35,m13,L,mu2) * (  - 1./24.*m13*
         m15*RD1456ep );

      F13p6x23RV +=  + hhdV(3,m15,m33,m35,m13,L,mu2) * (  - 1./864.*
         pow(m55,2)*RD3456ep + 1./108.*m35*m55*RD3456ep - 1./54.*pow(
         m35,2)*RD3456ep - 1./108.*m33*m55*RD3456ep + 1./27.*m33*m35*
         RD3456ep - 1./54.*pow(m33,2)*RD3456ep - 1./216.*m15*m55*
         RD3456ep + 1./54.*m15*m35*RD3456ep - 1./54.*m15*m33*RD3456ep
          - 1./216.*pow(m15,2)*RD3456ep - 1./54.*m13*m55*RD3456ep + 2./
         27.*m13*m35*RD3456ep - 2./27.*m13*m33*RD3456ep - 1./27.*m13*
         m15*RD3456ep + 5./54.*pow(m13,2)*RD3456ep - 1./432.*m11*m55*
         RD3456ep + 1./108.*m11*m35*RD3456ep - 1./108.*m11*m33*RD3456ep
          - 1./216.*m11*m15*RD3456ep - 1./54.*m11*m13*RD3456ep - 1./864.
         *pow(m11,2)*RD3456ep );

      F13p6x23RV +=  + hhdV(3,m16,m11,m36,m13,L,mu2) * (  - 1./24.*m13*
         m16*RD1456ep );

      F13p6x23RV +=  + hhdV(3,m16,m33,m36,m13,L,mu2) * (  - 1./864.*
         pow(m66,2)*RD3456ep + 1./108.*m36*m66*RD3456ep - 1./54.*pow(
         m36,2)*RD3456ep - 1./108.*m33*m66*RD3456ep + 1./27.*m33*m36*
         RD3456ep - 1./54.*pow(m33,2)*RD3456ep - 1./216.*m16*m66*
         RD3456ep + 1./54.*m16*m36*RD3456ep - 1./54.*m16*m33*RD3456ep
          - 1./216.*pow(m16,2)*RD3456ep - 1./54.*m13*m66*RD3456ep + 2./
         27.*m13*m36*RD3456ep - 2./27.*m13*m33*RD3456ep - 1./27.*m13*
         m16*RD3456ep + 5./54.*pow(m13,2)*RD3456ep - 1./432.*m11*m66*
         RD3456ep + 1./108.*m11*m36*RD3456ep - 1./108.*m11*m33*RD3456ep
          - 1./216.*m11*m16*RD3456ep - 1./54.*m11*m13*RD3456ep - 1./864.
         *pow(m11,2)*RD3456ep );

      F13p6x23RV +=  + hhdV(4,m11,m13,m33,m13,L,mu2) * ( 1./162.*m33*
         RD1456ep*RD3456ep + 1./81.*pow(m33,2)*RD3456ep*RS14563pe + 1./
         162.*pow(m33,2)*RC1456pe*RD3456ep - 1./81.*m13*RD1456ep*
         RD3456ep - 2./81.*m13*m33*RC1456pe*RD3456ep - 4./81.*pow(
         m13,2)*RD3456ep*RS14563pe + 2./81.*pow(m13,2)*RC1456pe*
         RD3456ep + 1./162.*m11*RD1456ep*RD3456ep + 1./81.*m11*m33*
         RD3456ep*RS14563pe + 1./81.*m11*m33*RC1456pe*RD3456ep + 2./81.
         *m11*m13*RD3456ep*RS14563pe - 2./81.*m11*m13*RC1456pe*RD3456ep
          + 1./162.*pow(m11,2)*RC1456pe*RD3456ep );

      F13p6x23RV +=  + hhdV(4,m13,m11,m33,m13,L,mu2) * (  - 1./36.*pow(
         m13,2)*RD3456ep*RS14563pe + 1./72.*pow(m13,2)*RC1456pe*
         RD3456ep );

      F13p6x23RV +=  + hhdV(4,m13,m33,m33,m13,L,mu2) * (  - 1./1296.*
         m13*pow(RD3456ep,2) );

      F13p6x23RV +=  + hhdV(4,m34,m14,m33,m13,L,mu2) * (  - 1./24.*m13*
         m34*RD3456ep );

      F13p6x23RV +=  + hhdV(4,m35,m15,m33,m13,L,mu2) * (  - 1./24.*m13*
         m35*RD3456ep );

      F13p6x23RV +=  + hhdV(4,m36,m16,m33,m13,L,mu2) * (  - 1./24.*m13*
         m36*RD3456ep );

      F13p6x23RV +=  + hhdV(5,m11,m11,m13,m13,L,mu2) * ( 1./1296.*pow(
         m33,2)*pow(RD1456ep,2) - 1./324.*m13*m33*pow(RD1456ep,2) - 2./
         81.*pow(m13,2)*pow(RD1456ep,2) + 1./648.*m11*m33*pow(
         RD1456ep,2) - 1./324.*m11*m13*pow(RD1456ep,2) + 1./1296.*pow(
         m11,2)*pow(RD1456ep,2) );

      F13p6x23RV +=  + hhdV(6,m11,m13,m33,m13,L,mu2) * ( 1./162.*pow(
         m33,2)*RD1456ep*RD3456ep - 2./81.*m13*m33*RD1456ep*RD3456ep + 
         2./81.*pow(m13,2)*RD1456ep*RD3456ep + 1./81.*m11*m33*RD1456ep*
         RD3456ep - 2./81.*m11*m13*RD1456ep*RD3456ep + 1./162.*pow(
         m11,2)*RD1456ep*RD3456ep );

      F13p6x23RV +=  + hhdV(7,m13,m11,m11,m13,L,mu2) * ( 1./144.*pow(
         m13,2)*pow(RD1456ep,2) );

      F13p6x23RV +=  + hhdV(7,m13,m11,m33,m13,L,mu2) * ( 1./72.*pow(
         m13,2)*RD1456ep*RD3456ep );

      F13p6x23RV +=  + hhdV(7,m13,m33,m33,m13,L,mu2) * ( 1./1296.*pow(
         m33,2)*pow(RD3456ep,2) + 1./162.*m13*m33*pow(RD3456ep,2) + 25./
         1296.*pow(m13,2)*pow(RD3456ep,2) + 1./648.*m11*m33*pow(
         RD3456ep,2) + 1./162.*m11*m13*pow(RD3456ep,2) + 1./1296.*pow(
         m11,2)*pow(RD3456ep,2) );

      F13p6x23RV +=  + hh1dV(1,m11,m11,m13,m13,L,mu2) * ( 2./27.*m13*
         RD1456ep*RS14563pe - 1./27.*m13*RD1456ep*RC1456pe + 1./12.*m13
         *m33 + 1./27.*m13*m33*pow(RS14563pe,2) + 1./54.*m13*m33*pow(
         RC1456pe,2) + 5./6.*pow(m13,2) + 8./27.*pow(m13,2)*pow(
         RS14563pe,2) - 8./27.*pow(m13,2)*RC1456pe*RS14563pe + 2./27.*
         pow(m13,2)*pow(RC1456pe,2) + 1./12.*m11*m13 - 1./9.*m11*m13*
         pow(RS14563pe,2) + 2./27.*m11*m13*RC1456pe*RS14563pe + 1./54.*
         m11*m13*pow(RC1456pe,2) );

      F13p6x23RV +=  + hh1dV(1,m11,m14,m34,m13,L,mu2) * ( 1./36.*m13*
         m44*RS14563pe - 1./36.*m13*m44*RS156pe + 1./18.*m13*m34*
         RS14563pe - 1./18.*m13*m34*RS156pe + 1./36.*m13*m33*RS14563pe
          - 1./36.*m13*m33*RS156pe - 1./9.*m13*m14*RS14563pe + 1./9.*
         m13*m14*RS156pe + 2./9.*pow(m13,2)*RS14563pe - 2./9.*pow(
         m13,2)*RS156pe + 1./9.*m11*m13*RS14563pe - 1./9.*m11*m13*
         RS156pe );

      F13p6x23RV +=  + hh1dV(1,m11,m15,m35,m13,L,mu2) * ( 1./36.*m13*
         m55*RS14563pe - 1./36.*m13*m55*RS146pe + 1./18.*m13*m35*
         RS14563pe - 1./18.*m13*m35*RS146pe + 1./36.*m13*m33*RS14563pe
          - 1./36.*m13*m33*RS146pe - 1./9.*m13*m15*RS14563pe + 1./9.*
         m13*m15*RS146pe + 2./9.*pow(m13,2)*RS14563pe - 2./9.*pow(
         m13,2)*RS146pe + 1./9.*m11*m13*RS14563pe - 1./9.*m11*m13*
         RS146pe );

      F13p6x23RV +=  + hh1dV(1,m11,m16,m36,m13,L,mu2) * ( 1./36.*m13*
         m66*RS14563pe - 1./36.*m13*m66*RS145pe + 1./18.*m13*m36*
         RS14563pe - 1./18.*m13*m36*RS145pe + 1./36.*m13*m33*RS14563pe
          - 1./36.*m13*m33*RS145pe - 1./9.*m13*m16*RS14563pe + 1./9.*
         m13*m16*RS145pe + 2./9.*pow(m13,2)*RS14563pe - 2./9.*pow(
         m13,2)*RS145pe + 1./9.*m11*m13*RS14563pe - 1./9.*m11*m13*
         RS145pe );

      F13p6x23RV +=  + hh1dV(1,m13,m11,m33,m13,L,mu2) * (  - 1./27.*m13
         *RD3456ep*RS14563pe + 1./54.*m13*RC1456pe*RD3456ep - 1./27.*
         m13*RD1456ep*RS34561pe + 1./54.*m13*RD1456ep*RC3456pe + 1./27.
         *m13*m33*RS14563pe*RS34561pe - 1./27.*m13*m33*RC1456pe*
         RS34561pe + 1./27.*m13*m33*RC1456pe*RC3456pe - 8./27.*pow(
         m13,2)*RS14563pe*RS34561pe + 4./27.*pow(m13,2)*RC3456pe*
         RS14563pe + 4./27.*pow(m13,2)*RC1456pe*RS34561pe - 2./27.*pow(
         m13,2)*RC1456pe*RC3456pe + 1./27.*m11*m13*RS14563pe*RS34561pe
          - 1./27.*m11*m13*RC3456pe*RS14563pe + 1./27.*m11*m13*RC1456pe
         *RC3456pe );

      F13p6x23RV +=  + hh1dV(1,m13,m11,mp2,m13,L,mu2) * (  - 1./27.*m13
         *RD1456ep*RSp45613e + 1./54.*m13*RD1456ep*RSp45633e + 1./54.*
         m13*RSp45611e*RD1456ep + 2./27.*m13*mp2*RSp45613e*RS14563pe - 
         1./27.*m13*mp2*RSp45633e*RS14563pe - 1./27.*m13*mp2*RC1456pe*
         RSp45613e + 1./54.*m13*mp2*RC1456pe*RSp45633e - 1./27.*m13*mp2
         *RSp45611e*RS14563pe + 1./54.*m13*mp2*RSp45611e*RC1456pe - 1./
         27.*m13*m33*RSp45613e*RS14563pe + 1./27.*m13*m33*RSp45633e*
         RS14563pe + 1./54.*m13*m33*RC1456pe*RSp45633e - 1./54.*m13*m33
         *RSp45611e*RC1456pe - 8./27.*pow(m13,2)*RSp45613e*RS14563pe + 
         4./27.*pow(m13,2)*RSp45633e*RS14563pe + 4./27.*pow(m13,2)*
         RC1456pe*RSp45613e - 2./27.*pow(m13,2)*RC1456pe*RSp45633e + 4./
         27.*pow(m13,2)*RSp45611e*RS14563pe - 2./27.*pow(m13,2)*
         RSp45611e*RC1456pe + 1./27.*m11*m13*RSp45613e*RS14563pe - 1./
         27.*m11*m13*RSp45633e*RS14563pe + 1./27.*m11*m13*RC1456pe*
         RSp45633e - 1./27.*m11*m13*RSp45611e*RC1456pe );

      F13p6x23RV +=  + hh1dV(1,m13,m11,me2,m13,L,mu2) * (  - 1./27.*m13
         *RD1456ep*RSe45613p + 1./54.*m13*RD1456ep*RSe45633p + 1./54.*
         m13*RSe45611p*RD1456ep + 2./27.*m13*me2*RSe45613p*RS14563pe - 
         1./27.*m13*me2*RSe45633p*RS14563pe - 1./27.*m13*me2*RC1456pe*
         RSe45613p + 1./54.*m13*me2*RC1456pe*RSe45633p - 1./27.*m13*me2
         *RSe45611p*RS14563pe + 1./54.*m13*me2*RSe45611p*RC1456pe - 1./
         27.*m13*m33*RSe45613p*RS14563pe + 1./27.*m13*m33*RSe45633p*
         RS14563pe + 1./54.*m13*m33*RC1456pe*RSe45633p - 1./54.*m13*m33
         *RSe45611p*RC1456pe - 8./27.*pow(m13,2)*RSe45613p*RS14563pe + 
         4./27.*pow(m13,2)*RSe45633p*RS14563pe + 4./27.*pow(m13,2)*
         RC1456pe*RSe45613p - 2./27.*pow(m13,2)*RC1456pe*RSe45633p + 4./
         27.*pow(m13,2)*RSe45611p*RS14563pe - 2./27.*pow(m13,2)*
         RSe45611p*RC1456pe + 1./27.*m11*m13*RSe45613p*RS14563pe - 1./
         27.*m11*m13*RSe45633p*RS14563pe + 1./27.*m11*m13*RC1456pe*
         RSe45633p - 1./27.*m11*m13*RSe45611p*RC1456pe );

      F13p6x23RV +=  + hh1dV(1,m13,m33,m33,m13,L,mu2) * (  - 1./27.*m13
         *RD3456ep*RS34561pe + 1./54.*m13*RD3456ep*RC3456pe - 1./24.*
         m13*m33 + 1./18.*m13*m33*pow(RS34561pe,2) - 1./27.*m13*m33*
         RC3456pe*RS34561pe - 1./108.*m13*m33*pow(RC3456pe,2) - 5./12.*
         pow(m13,2) - 4./27.*pow(m13,2)*pow(RS34561pe,2) + 4./27.*pow(
         m13,2)*RC3456pe*RS34561pe - 1./27.*pow(m13,2)*pow(RC3456pe,2)
          - 1./24.*m11*m13 - 1./54.*m11*m13*pow(RS34561pe,2) - 1./108.*
         m11*m13*pow(RC3456pe,2) );

      F13p6x23RV +=  + hh1dV(1,m13,m33,mp2,m13,L,mu2) * (  - 1./27.*m13
         *RD3456ep*RSp45613e + 1./54.*m13*RSp45633e*RD3456ep + 1./54.*
         m13*RSp45611e*RD3456ep + 2./27.*m13*mp2*RSp45613e*RS34561pe - 
         1./27.*m13*mp2*RC3456pe*RSp45613e - 1./27.*m13*mp2*RSp45633e*
         RS34561pe + 1./54.*m13*mp2*RSp45633e*RC3456pe - 1./27.*m13*mp2
         *RSp45611e*RS34561pe + 1./54.*m13*mp2*RSp45611e*RC3456pe + 1./
         27.*m13*m33*RSp45613e*RS34561pe - 1./27.*m13*m33*RSp45633e*
         RC3456pe - 1./27.*m13*m33*RSp45611e*RS34561pe + 1./27.*m13*m33
         *RSp45611e*RC3456pe - 8./27.*pow(m13,2)*RSp45613e*RS34561pe + 
         4./27.*pow(m13,2)*RC3456pe*RSp45613e + 4./27.*pow(m13,2)*
         RSp45633e*RS34561pe - 2./27.*pow(m13,2)*RSp45633e*RC3456pe + 4.
         /27.*pow(m13,2)*RSp45611e*RS34561pe - 2./27.*pow(m13,2)*
         RSp45611e*RC3456pe - 1./27.*m11*m13*RSp45613e*RS34561pe - 1./
         54.*m11*m13*RSp45633e*RC3456pe + 1./27.*m11*m13*RSp45611e*
         RS34561pe + 1./54.*m11*m13*RSp45611e*RC3456pe );

      F13p6x23RV +=  + hh1dV(1,m13,m33,me2,m13,L,mu2) * (  - 1./27.*m13
         *RD3456ep*RSe45613p + 1./54.*m13*RSe45633p*RD3456ep + 1./54.*
         m13*RSe45611p*RD3456ep + 2./27.*m13*me2*RSe45613p*RS34561pe - 
         1./27.*m13*me2*RC3456pe*RSe45613p - 1./27.*m13*me2*RSe45633p*
         RS34561pe + 1./54.*m13*me2*RSe45633p*RC3456pe - 1./27.*m13*me2
         *RSe45611p*RS34561pe + 1./54.*m13*me2*RSe45611p*RC3456pe + 1./
         27.*m13*m33*RSe45613p*RS34561pe - 1./27.*m13*m33*RSe45633p*
         RC3456pe - 1./27.*m13*m33*RSe45611p*RS34561pe + 1./27.*m13*m33
         *RSe45611p*RC3456pe - 8./27.*pow(m13,2)*RSe45613p*RS34561pe + 
         4./27.*pow(m13,2)*RC3456pe*RSe45613p + 4./27.*pow(m13,2)*
         RSe45633p*RS34561pe - 2./27.*pow(m13,2)*RSe45633p*RC3456pe + 4.
         /27.*pow(m13,2)*RSe45611p*RS34561pe - 2./27.*pow(m13,2)*
         RSe45611p*RC3456pe - 1./27.*m11*m13*RSe45613p*RS34561pe - 1./
         54.*m11*m13*RSe45633p*RC3456pe + 1./27.*m11*m13*RSe45611p*
         RS34561pe + 1./54.*m11*m13*RSe45611p*RC3456pe );

      F13p6x23RV +=  + hh1dV(1,m13,mp2,mp2,m13,L,mu2) * ( 2./27.*m13*
         mp2*pow(RSp45613e,2) - 2./27.*m13*mp2*RSp45633e*RSp45613e + 1./
         54.*m13*mp2*pow(RSp45633e,2) - 2./27.*m13*mp2*RSp45611e*
         RSp45613e + 1./27.*m13*mp2*RSp45611e*RSp45633e + 1./54.*m13*
         mp2*pow(RSp45611e,2) - 1./54.*m13*m33*pow(RSp45613e,2) + 1./27.
         *m13*m33*RSp45633e*RSp45613e - 1./36.*m13*m33*pow(RSp45633e,2)
          + 1./54.*m13*m33*RSp45611e*RSp45633e - 1./108.*m13*m33*pow(
         RSp45611e,2) - 4./27.*pow(m13,2)*pow(RSp45613e,2) + 4./27.*
         pow(m13,2)*RSp45633e*RSp45613e - 1./27.*pow(m13,2)*pow(
         RSp45633e,2) + 4./27.*pow(m13,2)*RSp45611e*RSp45613e - 2./27.*
         pow(m13,2)*RSp45611e*RSp45633e - 1./27.*pow(m13,2)*pow(
         RSp45611e,2) - 1./54.*m11*m13*pow(RSp45613e,2) - 1./108.*m11*
         m13*pow(RSp45633e,2) + 1./27.*m11*m13*RSp45611e*RSp45613e + 1./
         54.*m11*m13*RSp45611e*RSp45633e - 1./36.*m11*m13*pow(
         RSp45611e,2) );

      F13p6x23RV +=  + hh1dV(1,m13,me2,mp2,m13,L,mu2) * ( 2./27.*m13*
         me2*RSp45613e*RSe45613p - 1./27.*m13*me2*RSe45633p*RSp45613e
          - 1./27.*m13*me2*RSp45633e*RSe45613p + 1./54.*m13*me2*
         RSp45633e*RSe45633p - 1./27.*m13*me2*RSe45611p*RSp45613e + 1./
         54.*m13*me2*RSe45611p*RSp45633e - 1./27.*m13*me2*RSp45611e*
         RSe45613p + 1./54.*m13*me2*RSp45611e*RSe45633p + 1./54.*m13*
         me2*RSp45611e*RSe45611p + 2./27.*m13*mp2*RSp45613e*RSe45613p
          - 1./27.*m13*mp2*RSe45633p*RSp45613e - 1./27.*m13*mp2*
         RSp45633e*RSe45613p + 1./54.*m13*mp2*RSp45633e*RSe45633p - 1./
         27.*m13*mp2*RSe45611p*RSp45613e + 1./54.*m13*mp2*RSe45611p*
         RSp45633e - 1./27.*m13*mp2*RSp45611e*RSe45613p + 1./54.*m13*
         mp2*RSp45611e*RSe45633p + 1./54.*m13*mp2*RSp45611e*RSe45611p
          - 1./27.*m13*m33*RSp45613e*RSe45613p + 1./27.*m13*m33*
         RSe45633p*RSp45613e + 1./27.*m13*m33*RSp45633e*RSe45613p - 1./
         18.*m13*m33*RSp45633e*RSe45633p + 1./54.*m13*m33*RSe45611p*
         RSp45633e );

      F13p6x23RV +=  + hh1dV(1,m13,me2,mp2,m13,L,mu2) * ( 1./54.*m13*
         m33*RSp45611e*RSe45633p - 1./54.*m13*m33*RSp45611e*RSe45611p
          - 8./27.*pow(m13,2)*RSp45613e*RSe45613p + 4./27.*pow(m13,2)*
         RSe45633p*RSp45613e + 4./27.*pow(m13,2)*RSp45633e*RSe45613p - 
         2./27.*pow(m13,2)*RSp45633e*RSe45633p + 4./27.*pow(m13,2)*
         RSe45611p*RSp45613e - 2./27.*pow(m13,2)*RSe45611p*RSp45633e + 
         4./27.*pow(m13,2)*RSp45611e*RSe45613p - 2./27.*pow(m13,2)*
         RSp45611e*RSe45633p - 2./27.*pow(m13,2)*RSp45611e*RSe45611p - 
         1./27.*m11*m13*RSp45613e*RSe45613p - 1./54.*m11*m13*RSp45633e*
         RSe45633p + 1./27.*m11*m13*RSe45611p*RSp45613e + 1./54.*m11*
         m13*RSe45611p*RSp45633e + 1./27.*m11*m13*RSp45611e*RSe45613p
          + 1./54.*m11*m13*RSp45611e*RSe45633p - 1./18.*m11*m13*
         RSp45611e*RSe45611p );

      F13p6x23RV +=  + hh1dV(1,m13,me2,me2,m13,L,mu2) * ( 2./27.*m13*
         me2*pow(RSe45613p,2) - 2./27.*m13*me2*RSe45633p*RSe45613p + 1./
         54.*m13*me2*pow(RSe45633p,2) - 2./27.*m13*me2*RSe45611p*
         RSe45613p + 1./27.*m13*me2*RSe45611p*RSe45633p + 1./54.*m13*
         me2*pow(RSe45611p,2) - 1./54.*m13*m33*pow(RSe45613p,2) + 1./27.
         *m13*m33*RSe45633p*RSe45613p - 1./36.*m13*m33*pow(RSe45633p,2)
          + 1./54.*m13*m33*RSe45611p*RSe45633p - 1./108.*m13*m33*pow(
         RSe45611p,2) - 4./27.*pow(m13,2)*pow(RSe45613p,2) + 4./27.*
         pow(m13,2)*RSe45633p*RSe45613p - 1./27.*pow(m13,2)*pow(
         RSe45633p,2) + 4./27.*pow(m13,2)*RSe45611p*RSe45613p - 2./27.*
         pow(m13,2)*RSe45611p*RSe45633p - 1./27.*pow(m13,2)*pow(
         RSe45611p,2) - 1./54.*m11*m13*pow(RSe45613p,2) - 1./108.*m11*
         m13*pow(RSe45633p,2) + 1./27.*m11*m13*RSe45611p*RSe45613p + 1./
         54.*m11*m13*RSe45611p*RSe45633p - 1./36.*m11*m13*pow(
         RSe45611p,2) );

      F13p6x23RV +=  + hh1dV(1,m14,m11,m34,m13,L,mu2) * (  - 1./18.*m13
         *RD1456ep - 1./36.*m13*m44*RC1456pe - 1./18.*m13*m44*RS156pe
          + 1./6.*m13*m34*RS14563pe - 1./18.*m13*m34*RC1456pe - 1./18.*
         m13*m34*RS156pe - 1./36.*m13*m33*RS14563pe - 1./36.*m13*m33*
         RC1456pe - 1./36.*m13*m33*RS156pe - 1./6.*m13*m14*RS14563pe + 
         1./9.*m13*m14*RC1456pe - 1./18.*m13*m14*RS156pe + 1./9.*pow(
         m13,2)*RC1456pe - 2./9.*pow(m13,2)*RS156pe + 1./36.*m11*m13*
         RS14563pe - 1./9.*m11*m13*RC1456pe + 1./12.*m11*m13*RS156pe );

      F13p6x23RV +=  + hh1dV(1,m14,m33,m34,m13,L,mu2) * ( 1./18.*m13*
         RD3456ep + 1./36.*m13*m44*RC3456pe + 1./18.*m13*m44*RS356pe + 
         1./6.*m13*m34*RS34561pe - 1./9.*m13*m34*RC3456pe + 1./18.*m13*
         m34*RS356pe - 1./36.*m13*m33*RS34561pe + 1./9.*m13*m33*
         RC3456pe - 1./12.*m13*m33*RS356pe - 1./6.*m13*m14*RS34561pe + 
         1./18.*m13*m14*RC3456pe + 1./18.*m13*m14*RS356pe - 1./9.*pow(
         m13,2)*RC3456pe + 2./9.*pow(m13,2)*RS356pe + 1./36.*m11*m13*
         RS34561pe + 1./36.*m11*m13*RC3456pe + 1./36.*m11*m13*RS356pe )
         ;

      F13p6x23RV +=  + hh1dV(1,m14,m34,m44,m13,L,mu2) * ( 1./6.*m13*m44
          - 1./18.*m13*m44*RS456pe - 1./6.*m13*m34 + 1./18.*m13*m34*
         RS456pe - 1./12.*m13*m33 + 1./36.*m13*m33*RS456pe - 1./6.*m13*
         m14 + 1./18.*m13*m14*RS456pe + 1./3.*pow(m13,2) - 1./9.*pow(
         m13,2)*RS456pe - 1./12.*m11*m13 + 1./36.*m11*m13*RS456pe );

      F13p6x23RV +=  + hh1dV(1,m14,m34,mp2,m13,L,mu2) * (  - 1./18.*m13
         *mp2*RSp45613e + 1./18.*m13*mp2*RSp563e - 1./18.*m13*mp2*
         RSp45611e + 1./6.*m13*mp2*RSp561e - 1./9.*m13*mp2*RSp564e - 1./
         36.*m13*m44*RSp45613e + 1./36.*m13*m44*RSp563e - 1./36.*m13*
         m44*RSp45611e - 1./36.*m13*m44*RSp561e + 1./18.*m13*m44*
         RSp564e + 1./9.*m13*m34*RSp45613e - 1./9.*m13*m34*RSp563e - 1./
         18.*m13*m34*RSp45611e + 1./18.*m13*m34*RSp564e - 1./18.*m13*
         m33*RSp45613e + 1./18.*m13*m33*RSp563e - 1./36.*m13*m33*
         RSp45611e + 1./36.*m13*m33*RSp564e - 1./18.*m13*m14*RSp45613e
          + 1./18.*m13*m14*RSp563e + 1./9.*m13*m14*RSp45611e - 1./6.*
         m13*m14*RSp561e + 1./18.*m13*m14*RSp564e - 2./9.*pow(m13,2)*
         RSp45613e + 2./9.*pow(m13,2)*RSp563e + 1./9.*pow(m13,2)*
         RSp45611e - 1./9.*pow(m13,2)*RSp564e - 1./36.*m11*m13*
         RSp45613e + 1./36.*m11*m13*RSp563e - 1./18.*m11*m13*RSp45611e
          + 1./36.*m11*m13*RSp561e + 1./36.*m11*m13*RSp564e );

      F13p6x23RV +=  + hh1dV(1,m14,m34,me2,m13,L,mu2) * (  - 1./18.*m13
         *me2*RSe45613p + 1./18.*m13*me2*RSe563p - 1./18.*m13*me2*
         RSe45611p + 1./6.*m13*me2*RSe561p - 1./9.*m13*me2*RSe564p - 1./
         36.*m13*m44*RSe45613p + 1./36.*m13*m44*RSe563p - 1./36.*m13*
         m44*RSe45611p - 1./36.*m13*m44*RSe561p + 1./18.*m13*m44*
         RSe564p + 1./9.*m13*m34*RSe45613p - 1./9.*m13*m34*RSe563p - 1./
         18.*m13*m34*RSe45611p + 1./18.*m13*m34*RSe564p - 1./18.*m13*
         m33*RSe45613p + 1./18.*m13*m33*RSe563p - 1./36.*m13*m33*
         RSe45611p + 1./36.*m13*m33*RSe564p - 1./18.*m13*m14*RSe45613p
          + 1./18.*m13*m14*RSe563p + 1./9.*m13*m14*RSe45611p - 1./6.*
         m13*m14*RSe561p + 1./18.*m13*m14*RSe564p - 2./9.*pow(m13,2)*
         RSe45613p + 2./9.*pow(m13,2)*RSe563p + 1./9.*pow(m13,2)*
         RSe45611p - 1./9.*pow(m13,2)*RSe564p - 1./36.*m11*m13*
         RSe45613p + 1./36.*m11*m13*RSe563p - 1./18.*m11*m13*RSe45611p
          + 1./36.*m11*m13*RSe561p + 1./36.*m11*m13*RSe564p );

      F13p6x23RV +=  + hh1dV(1,m14,m35,m45,m13,L,mu2) * (  - 1./12.*m13
         *m55 + 1./3.*m13*m45 - 1./12.*m13*m44 - 1./6.*m13*m35 - 1./12.
         *m13*m33 - 1./6.*m13*m14 + 1./3.*pow(m13,2) - 1./12.*m11*m13 )
         ;

      F13p6x23RV +=  + hh1dV(1,m14,m36,m46,m13,L,mu2) * (  - 1./12.*m13
         *m66 + 1./3.*m13*m46 - 1./12.*m13*m44 - 1./6.*m13*m36 - 1./12.
         *m13*m33 - 1./6.*m13*m14 + 1./3.*pow(m13,2) - 1./12.*m11*m13 )
         ;

      F13p6x23RV +=  + hh1dV(1,m15,m11,m35,m13,L,mu2) * (  - 1./18.*m13
         *RD1456ep - 1./36.*m13*m55*RC1456pe - 1./18.*m13*m55*RS146pe
          + 1./6.*m13*m35*RS14563pe - 1./18.*m13*m35*RC1456pe - 1./18.*
         m13*m35*RS146pe - 1./36.*m13*m33*RS14563pe - 1./36.*m13*m33*
         RC1456pe - 1./36.*m13*m33*RS146pe - 1./6.*m13*m15*RS14563pe + 
         1./9.*m13*m15*RC1456pe - 1./18.*m13*m15*RS146pe + 1./9.*pow(
         m13,2)*RC1456pe - 2./9.*pow(m13,2)*RS146pe + 1./36.*m11*m13*
         RS14563pe - 1./9.*m11*m13*RC1456pe + 1./12.*m11*m13*RS146pe );

      F13p6x23RV +=  + hh1dV(1,m15,m33,m35,m13,L,mu2) * ( 1./18.*m13*
         RD3456ep + 1./36.*m13*m55*RC3456pe + 1./18.*m13*m55*RS346pe + 
         1./6.*m13*m35*RS34561pe - 1./9.*m13*m35*RC3456pe + 1./18.*m13*
         m35*RS346pe - 1./36.*m13*m33*RS34561pe + 1./9.*m13*m33*
         RC3456pe - 1./12.*m13*m33*RS346pe - 1./6.*m13*m15*RS34561pe + 
         1./18.*m13*m15*RC3456pe + 1./18.*m13*m15*RS346pe - 1./9.*pow(
         m13,2)*RC3456pe + 2./9.*pow(m13,2)*RS346pe + 1./36.*m11*m13*
         RS34561pe + 1./36.*m11*m13*RC3456pe + 1./36.*m11*m13*RS346pe )
         ;

      F13p6x23RV +=  + hh1dV(1,m15,m34,m45,m13,L,mu2) * (  - 1./12.*m13
         *m55 + 1./3.*m13*m45 - 1./12.*m13*m44 - 1./6.*m13*m34 - 1./12.
         *m13*m33 - 1./6.*m13*m15 + 1./3.*pow(m13,2) - 1./12.*m11*m13 )
         ;

      F13p6x23RV +=  + hh1dV(1,m15,m35,m55,m13,L,mu2) * ( 1./6.*m13*m55
          - 1./18.*m13*m55*RS546pe - 1./6.*m13*m35 + 1./18.*m13*m35*
         RS546pe - 1./12.*m13*m33 + 1./36.*m13*m33*RS546pe - 1./6.*m13*
         m15 + 1./18.*m13*m15*RS546pe + 1./3.*pow(m13,2) - 1./9.*pow(
         m13,2)*RS546pe - 1./12.*m11*m13 + 1./36.*m11*m13*RS546pe );

      F13p6x23RV +=  + hh1dV(1,m15,m35,mp2,m13,L,mu2) * (  - 1./18.*m13
         *mp2*RSp45613e + 1./18.*m13*mp2*RSp463e - 1./18.*m13*mp2*
         RSp45611e + 1./6.*m13*mp2*RSp461e - 1./9.*m13*mp2*RSp465e - 1./
         36.*m13*m55*RSp45613e + 1./36.*m13*m55*RSp463e - 1./36.*m13*
         m55*RSp45611e - 1./36.*m13*m55*RSp461e + 1./18.*m13*m55*
         RSp465e + 1./9.*m13*m35*RSp45613e - 1./9.*m13*m35*RSp463e - 1./
         18.*m13*m35*RSp45611e + 1./18.*m13*m35*RSp465e - 1./18.*m13*
         m33*RSp45613e + 1./18.*m13*m33*RSp463e - 1./36.*m13*m33*
         RSp45611e + 1./36.*m13*m33*RSp465e - 1./18.*m13*m15*RSp45613e
          + 1./18.*m13*m15*RSp463e + 1./9.*m13*m15*RSp45611e - 1./6.*
         m13*m15*RSp461e + 1./18.*m13*m15*RSp465e - 2./9.*pow(m13,2)*
         RSp45613e + 2./9.*pow(m13,2)*RSp463e + 1./9.*pow(m13,2)*
         RSp45611e - 1./9.*pow(m13,2)*RSp465e - 1./36.*m11*m13*
         RSp45613e + 1./36.*m11*m13*RSp463e - 1./18.*m11*m13*RSp45611e
          + 1./36.*m11*m13*RSp461e + 1./36.*m11*m13*RSp465e );

      F13p6x23RV +=  + hh1dV(1,m15,m35,me2,m13,L,mu2) * (  - 1./18.*m13
         *me2*RSe45613p + 1./18.*m13*me2*RSe463p - 1./18.*m13*me2*
         RSe45611p + 1./6.*m13*me2*RSe461p - 1./9.*m13*me2*RSe465p - 1./
         36.*m13*m55*RSe45613p + 1./36.*m13*m55*RSe463p - 1./36.*m13*
         m55*RSe45611p - 1./36.*m13*m55*RSe461p + 1./18.*m13*m55*
         RSe465p + 1./9.*m13*m35*RSe45613p - 1./9.*m13*m35*RSe463p - 1./
         18.*m13*m35*RSe45611p + 1./18.*m13*m35*RSe465p - 1./18.*m13*
         m33*RSe45613p + 1./18.*m13*m33*RSe463p - 1./36.*m13*m33*
         RSe45611p + 1./36.*m13*m33*RSe465p - 1./18.*m13*m15*RSe45613p
          + 1./18.*m13*m15*RSe463p + 1./9.*m13*m15*RSe45611p - 1./6.*
         m13*m15*RSe461p + 1./18.*m13*m15*RSe465p - 2./9.*pow(m13,2)*
         RSe45613p + 2./9.*pow(m13,2)*RSe463p + 1./9.*pow(m13,2)*
         RSe45611p - 1./9.*pow(m13,2)*RSe465p - 1./36.*m11*m13*
         RSe45613p + 1./36.*m11*m13*RSe463p - 1./18.*m11*m13*RSe45611p
          + 1./36.*m11*m13*RSe461p + 1./36.*m11*m13*RSe465p );

      F13p6x23RV +=  + hh1dV(1,m15,m36,m56,m13,L,mu2) * (  - 1./12.*m13
         *m66 + 1./3.*m13*m56 - 1./12.*m13*m55 - 1./6.*m13*m36 - 1./12.
         *m13*m33 - 1./6.*m13*m15 + 1./3.*pow(m13,2) - 1./12.*m11*m13 )
         ;

      F13p6x23RV +=  + hh1dV(1,m16,m11,m36,m13,L,mu2) * (  - 1./18.*m13
         *RD1456ep - 1./36.*m13*m66*RC1456pe - 1./18.*m13*m66*RS145pe
          + 1./6.*m13*m36*RS14563pe - 1./18.*m13*m36*RC1456pe - 1./18.*
         m13*m36*RS145pe - 1./36.*m13*m33*RS14563pe - 1./36.*m13*m33*
         RC1456pe - 1./36.*m13*m33*RS145pe - 1./6.*m13*m16*RS14563pe + 
         1./9.*m13*m16*RC1456pe - 1./18.*m13*m16*RS145pe + 1./9.*pow(
         m13,2)*RC1456pe - 2./9.*pow(m13,2)*RS145pe + 1./36.*m11*m13*
         RS14563pe - 1./9.*m11*m13*RC1456pe + 1./12.*m11*m13*RS145pe );

      F13p6x23RV +=  + hh1dV(1,m16,m33,m36,m13,L,mu2) * ( 1./18.*m13*
         RD3456ep + 1./36.*m13*m66*RC3456pe + 1./18.*m13*m66*RS345pe + 
         1./6.*m13*m36*RS34561pe - 1./9.*m13*m36*RC3456pe + 1./18.*m13*
         m36*RS345pe - 1./36.*m13*m33*RS34561pe + 1./9.*m13*m33*
         RC3456pe - 1./12.*m13*m33*RS345pe - 1./6.*m13*m16*RS34561pe + 
         1./18.*m13*m16*RC3456pe + 1./18.*m13*m16*RS345pe - 1./9.*pow(
         m13,2)*RC3456pe + 2./9.*pow(m13,2)*RS345pe + 1./36.*m11*m13*
         RS34561pe + 1./36.*m11*m13*RC3456pe + 1./36.*m11*m13*RS345pe )
         ;

      F13p6x23RV +=  + hh1dV(1,m16,m34,m46,m13,L,mu2) * (  - 1./12.*m13
         *m66 + 1./3.*m13*m46 - 1./12.*m13*m44 - 1./6.*m13*m34 - 1./12.
         *m13*m33 - 1./6.*m13*m16 + 1./3.*pow(m13,2) - 1./12.*m11*m13 )
         ;

      F13p6x23RV +=  + hh1dV(1,m16,m35,m56,m13,L,mu2) * (  - 1./12.*m13
         *m66 + 1./3.*m13*m56 - 1./12.*m13*m55 - 1./6.*m13*m35 - 1./12.
         *m13*m33 - 1./6.*m13*m16 + 1./3.*pow(m13,2) - 1./12.*m11*m13 )
         ;

      F13p6x23RV +=  + hh1dV(1,m16,m36,m66,m13,L,mu2) * ( 1./6.*m13*m66
          - 1./18.*m13*m66*RS645pe - 1./6.*m13*m36 + 1./18.*m13*m36*
         RS645pe - 1./12.*m13*m33 + 1./36.*m13*m33*RS645pe - 1./6.*m13*
         m16 + 1./18.*m13*m16*RS645pe + 1./3.*pow(m13,2) - 1./9.*pow(
         m13,2)*RS645pe - 1./12.*m11*m13 + 1./36.*m11*m13*RS645pe );

      F13p6x23RV +=  + hh1dV(1,m16,m36,mp2,m13,L,mu2) * (  - 1./18.*m13
         *mp2*RSp45613e + 1./18.*m13*mp2*RSp453e - 1./18.*m13*mp2*
         RSp45611e + 1./6.*m13*mp2*RSp451e - 1./9.*m13*mp2*RSp456e - 1./
         36.*m13*m66*RSp45613e + 1./36.*m13*m66*RSp453e - 1./36.*m13*
         m66*RSp45611e - 1./36.*m13*m66*RSp451e + 1./18.*m13*m66*
         RSp456e + 1./9.*m13*m36*RSp45613e - 1./9.*m13*m36*RSp453e - 1./
         18.*m13*m36*RSp45611e + 1./18.*m13*m36*RSp456e - 1./18.*m13*
         m33*RSp45613e + 1./18.*m13*m33*RSp453e - 1./36.*m13*m33*
         RSp45611e + 1./36.*m13*m33*RSp456e - 1./18.*m13*m16*RSp45613e
          + 1./18.*m13*m16*RSp453e + 1./9.*m13*m16*RSp45611e - 1./6.*
         m13*m16*RSp451e + 1./18.*m13*m16*RSp456e - 2./9.*pow(m13,2)*
         RSp45613e + 2./9.*pow(m13,2)*RSp453e + 1./9.*pow(m13,2)*
         RSp45611e - 1./9.*pow(m13,2)*RSp456e - 1./36.*m11*m13*
         RSp45613e + 1./36.*m11*m13*RSp453e - 1./18.*m11*m13*RSp45611e
          + 1./36.*m11*m13*RSp451e + 1./36.*m11*m13*RSp456e );

      F13p6x23RV +=  + hh1dV(1,m16,m36,me2,m13,L,mu2) * (  - 1./18.*m13
         *me2*RSe45613p + 1./18.*m13*me2*RSe453p - 1./18.*m13*me2*
         RSe45611p + 1./6.*m13*me2*RSe451p - 1./9.*m13*me2*RSe456p - 1./
         36.*m13*m66*RSe45613p + 1./36.*m13*m66*RSe453p - 1./36.*m13*
         m66*RSe45611p - 1./36.*m13*m66*RSe451p + 1./18.*m13*m66*
         RSe456p + 1./9.*m13*m36*RSe45613p - 1./9.*m13*m36*RSe453p - 1./
         18.*m13*m36*RSe45611p + 1./18.*m13*m36*RSe456p - 1./18.*m13*
         m33*RSe45613p + 1./18.*m13*m33*RSe453p - 1./36.*m13*m33*
         RSe45611p + 1./36.*m13*m33*RSe456p - 1./18.*m13*m16*RSe45613p
          + 1./18.*m13*m16*RSe453p + 1./9.*m13*m16*RSe45611p - 1./6.*
         m13*m16*RSe451p + 1./18.*m13*m16*RSe456p - 2./9.*pow(m13,2)*
         RSe45613p + 2./9.*pow(m13,2)*RSe453p + 1./9.*pow(m13,2)*
         RSe45611p - 1./9.*pow(m13,2)*RSe456p - 1./36.*m11*m13*
         RSe45613p + 1./36.*m11*m13*RSe453p - 1./18.*m11*m13*RSe45611p
          + 1./36.*m11*m13*RSe451p + 1./36.*m11*m13*RSe456p );

      F13p6x23RV +=  + hh1dV(1,m33,m14,m34,m13,L,mu2) * ( 1./18.*m13*
         RD3456ep + 1./36.*m13*m44*RS34561pe + 1./36.*m13*m44*RC3456pe
          + 1./36.*m13*m44*RS356pe + 1./18.*m13*m34*RS34561pe - 1./9.*
         m13*m34*RC3456pe + 1./6.*m13*m34*RS356pe + 1./12.*m13*m33*
         RS34561pe + 1./9.*m13*m33*RC3456pe - 7./36.*m13*m33*RS356pe - 
         1./9.*m13*m14*RS34561pe + 1./18.*m13*m14*RC3456pe + 2./9.*pow(
         m13,2)*RS34561pe - 1./9.*pow(m13,2)*RC3456pe + 1./18.*m11*m13*
         RS34561pe + 1./36.*m11*m13*RC3456pe );

      F13p6x23RV +=  + hh1dV(1,m33,m15,m35,m13,L,mu2) * ( 1./18.*m13*
         RD3456ep + 1./36.*m13*m55*RS34561pe + 1./36.*m13*m55*RC3456pe
          + 1./36.*m13*m55*RS346pe + 1./18.*m13*m35*RS34561pe - 1./9.*
         m13*m35*RC3456pe + 1./6.*m13*m35*RS346pe + 1./12.*m13*m33*
         RS34561pe + 1./9.*m13*m33*RC3456pe - 7./36.*m13*m33*RS346pe - 
         1./9.*m13*m15*RS34561pe + 1./18.*m13*m15*RC3456pe + 2./9.*pow(
         m13,2)*RS34561pe - 1./9.*pow(m13,2)*RC3456pe + 1./18.*m11*m13*
         RS34561pe + 1./36.*m11*m13*RC3456pe );

      F13p6x23RV +=  + hh1dV(1,m33,m16,m36,m13,L,mu2) * ( 1./18.*m13*
         RD3456ep + 1./36.*m13*m66*RS34561pe + 1./36.*m13*m66*RC3456pe
          + 1./36.*m13*m66*RS345pe + 1./18.*m13*m36*RS34561pe - 1./9.*
         m13*m36*RC3456pe + 1./6.*m13*m36*RS345pe + 1./12.*m13*m33*
         RS34561pe + 1./9.*m13*m33*RC3456pe - 7./36.*m13*m33*RS345pe - 
         1./9.*m13*m16*RS34561pe + 1./18.*m13*m16*RC3456pe + 2./9.*pow(
         m13,2)*RS34561pe - 1./9.*pow(m13,2)*RC3456pe + 1./18.*m11*m13*
         RS34561pe + 1./36.*m11*m13*RC3456pe );

      F13p6x23RV +=  + hh1dV(1,m34,m14,m44,m13,L,mu2) * ( 1./6.*m13*m44
          - 1./18.*m13*m44*RS456pe - 1./6.*m13*m34 + 1./18.*m13*m34*
         RS456pe - 1./12.*m13*m33 + 1./36.*m13*m33*RS456pe - 1./6.*m13*
         m14 + 1./18.*m13*m14*RS456pe + 1./3.*pow(m13,2) - 1./9.*pow(
         m13,2)*RS456pe - 1./12.*m11*m13 + 1./36.*m11*m13*RS456pe );

      F13p6x23RV +=  + hh1dV(1,m34,m14,mp2,m13,L,mu2) * (  - 1./18.*m13
         *mp2*RSp45613e - 1./18.*m13*mp2*RSp45633e + 1./6.*m13*mp2*
         RSp563e + 1./18.*m13*mp2*RSp561e - 1./9.*m13*mp2*RSp564e - 1./
         36.*m13*m44*RSp45613e - 1./36.*m13*m44*RSp45633e - 1./36.*m13*
         m44*RSp563e + 1./36.*m13*m44*RSp561e + 1./18.*m13*m44*RSp564e
          - 1./18.*m13*m34*RSp45613e + 1./9.*m13*m34*RSp45633e - 1./6.*
         m13*m34*RSp563e + 1./18.*m13*m34*RSp561e + 1./18.*m13*m34*
         RSp564e - 1./36.*m13*m33*RSp45613e - 1./18.*m13*m33*RSp45633e
          + 1./36.*m13*m33*RSp563e + 1./36.*m13*m33*RSp561e + 1./36.*
         m13*m33*RSp564e + 1./9.*m13*m14*RSp45613e - 1./18.*m13*m14*
         RSp45633e - 1./9.*m13*m14*RSp561e + 1./18.*m13*m14*RSp564e - 2.
         /9.*pow(m13,2)*RSp45613e + 1./9.*pow(m13,2)*RSp45633e + 2./9.*
         pow(m13,2)*RSp561e - 1./9.*pow(m13,2)*RSp564e - 1./18.*m11*m13
         *RSp45613e - 1./36.*m11*m13*RSp45633e + 1./18.*m11*m13*RSp561e
          + 1./36.*m11*m13*RSp564e );

      F13p6x23RV +=  + hh1dV(1,m34,m14,me2,m13,L,mu2) * (  - 1./18.*m13
         *me2*RSe45613p - 1./18.*m13*me2*RSe45633p + 1./6.*m13*me2*
         RSe563p + 1./18.*m13*me2*RSe561p - 1./9.*m13*me2*RSe564p - 1./
         36.*m13*m44*RSe45613p - 1./36.*m13*m44*RSe45633p - 1./36.*m13*
         m44*RSe563p + 1./36.*m13*m44*RSe561p + 1./18.*m13*m44*RSe564p
          - 1./18.*m13*m34*RSe45613p + 1./9.*m13*m34*RSe45633p - 1./6.*
         m13*m34*RSe563p + 1./18.*m13*m34*RSe561p + 1./18.*m13*m34*
         RSe564p - 1./36.*m13*m33*RSe45613p - 1./18.*m13*m33*RSe45633p
          + 1./36.*m13*m33*RSe563p + 1./36.*m13*m33*RSe561p + 1./36.*
         m13*m33*RSe564p + 1./9.*m13*m14*RSe45613p - 1./18.*m13*m14*
         RSe45633p - 1./9.*m13*m14*RSe561p + 1./18.*m13*m14*RSe564p - 2.
         /9.*pow(m13,2)*RSe45613p + 1./9.*pow(m13,2)*RSe45633p + 2./9.*
         pow(m13,2)*RSe561p - 1./9.*pow(m13,2)*RSe564p - 1./18.*m11*m13
         *RSe45613p - 1./36.*m11*m13*RSe45633p + 1./18.*m11*m13*RSe561p
          + 1./36.*m11*m13*RSe564p );

      F13p6x23RV +=  + hh1dV(1,m34,m15,m45,m13,L,mu2) * (  - 1./12.*m13
         *m55 + 1./3.*m13*m45 - 1./12.*m13*m44 - 1./6.*m13*m34 - 1./12.
         *m13*m33 - 1./6.*m13*m15 + 1./3.*pow(m13,2) - 1./12.*m11*m13 )
         ;

      F13p6x23RV +=  + hh1dV(1,m34,m16,m46,m13,L,mu2) * (  - 1./12.*m13
         *m66 + 1./3.*m13*m46 - 1./12.*m13*m44 - 1./6.*m13*m34 - 1./12.
         *m13*m33 - 1./6.*m13*m16 + 1./3.*pow(m13,2) - 1./12.*m11*m13 )
         ;

      F13p6x23RV +=  + hh1dV(1,m35,m14,m45,m13,L,mu2) * (  - 1./12.*m13
         *m55 + 1./3.*m13*m45 - 1./12.*m13*m44 - 1./6.*m13*m35 - 1./12.
         *m13*m33 - 1./6.*m13*m14 + 1./3.*pow(m13,2) - 1./12.*m11*m13 )
         ;

      F13p6x23RV +=  + hh1dV(1,m35,m15,m55,m13,L,mu2) * ( 1./6.*m13*m55
          - 1./18.*m13*m55*RS546pe - 1./6.*m13*m35 + 1./18.*m13*m35*
         RS546pe - 1./12.*m13*m33 + 1./36.*m13*m33*RS546pe - 1./6.*m13*
         m15 + 1./18.*m13*m15*RS546pe + 1./3.*pow(m13,2) - 1./9.*pow(
         m13,2)*RS546pe - 1./12.*m11*m13 + 1./36.*m11*m13*RS546pe );

      F13p6x23RV +=  + hh1dV(1,m35,m15,mp2,m13,L,mu2) * (  - 1./18.*m13
         *mp2*RSp45613e - 1./18.*m13*mp2*RSp45633e + 1./6.*m13*mp2*
         RSp463e + 1./18.*m13*mp2*RSp461e - 1./9.*m13*mp2*RSp465e - 1./
         36.*m13*m55*RSp45613e - 1./36.*m13*m55*RSp45633e - 1./36.*m13*
         m55*RSp463e + 1./36.*m13*m55*RSp461e + 1./18.*m13*m55*RSp465e
          - 1./18.*m13*m35*RSp45613e + 1./9.*m13*m35*RSp45633e - 1./6.*
         m13*m35*RSp463e + 1./18.*m13*m35*RSp461e + 1./18.*m13*m35*
         RSp465e - 1./36.*m13*m33*RSp45613e - 1./18.*m13*m33*RSp45633e
          + 1./36.*m13*m33*RSp463e + 1./36.*m13*m33*RSp461e + 1./36.*
         m13*m33*RSp465e + 1./9.*m13*m15*RSp45613e - 1./18.*m13*m15*
         RSp45633e - 1./9.*m13*m15*RSp461e + 1./18.*m13*m15*RSp465e - 2.
         /9.*pow(m13,2)*RSp45613e + 1./9.*pow(m13,2)*RSp45633e + 2./9.*
         pow(m13,2)*RSp461e - 1./9.*pow(m13,2)*RSp465e - 1./18.*m11*m13
         *RSp45613e - 1./36.*m11*m13*RSp45633e + 1./18.*m11*m13*RSp461e
          + 1./36.*m11*m13*RSp465e );

      F13p6x23RV +=  + hh1dV(1,m35,m15,me2,m13,L,mu2) * (  - 1./18.*m13
         *me2*RSe45613p - 1./18.*m13*me2*RSe45633p + 1./6.*m13*me2*
         RSe463p + 1./18.*m13*me2*RSe461p - 1./9.*m13*me2*RSe465p - 1./
         36.*m13*m55*RSe45613p - 1./36.*m13*m55*RSe45633p - 1./36.*m13*
         m55*RSe463p + 1./36.*m13*m55*RSe461p + 1./18.*m13*m55*RSe465p
          - 1./18.*m13*m35*RSe45613p + 1./9.*m13*m35*RSe45633p - 1./6.*
         m13*m35*RSe463p + 1./18.*m13*m35*RSe461p + 1./18.*m13*m35*
         RSe465p - 1./36.*m13*m33*RSe45613p - 1./18.*m13*m33*RSe45633p
          + 1./36.*m13*m33*RSe463p + 1./36.*m13*m33*RSe461p + 1./36.*
         m13*m33*RSe465p + 1./9.*m13*m15*RSe45613p - 1./18.*m13*m15*
         RSe45633p - 1./9.*m13*m15*RSe461p + 1./18.*m13*m15*RSe465p - 2.
         /9.*pow(m13,2)*RSe45613p + 1./9.*pow(m13,2)*RSe45633p + 2./9.*
         pow(m13,2)*RSe461p - 1./9.*pow(m13,2)*RSe465p - 1./18.*m11*m13
         *RSe45613p - 1./36.*m11*m13*RSe45633p + 1./18.*m11*m13*RSe461p
          + 1./36.*m11*m13*RSe465p );

      F13p6x23RV +=  + hh1dV(1,m35,m16,m56,m13,L,mu2) * (  - 1./12.*m13
         *m66 + 1./3.*m13*m56 - 1./12.*m13*m55 - 1./6.*m13*m35 - 1./12.
         *m13*m33 - 1./6.*m13*m16 + 1./3.*pow(m13,2) - 1./12.*m11*m13 )
         ;

      F13p6x23RV +=  + hh1dV(1,m36,m14,m46,m13,L,mu2) * (  - 1./12.*m13
         *m66 + 1./3.*m13*m46 - 1./12.*m13*m44 - 1./6.*m13*m36 - 1./12.
         *m13*m33 - 1./6.*m13*m14 + 1./3.*pow(m13,2) - 1./12.*m11*m13 )
         ;

      F13p6x23RV +=  + hh1dV(1,m36,m15,m56,m13,L,mu2) * (  - 1./12.*m13
         *m66 + 1./3.*m13*m56 - 1./12.*m13*m55 - 1./6.*m13*m36 - 1./12.
         *m13*m33 - 1./6.*m13*m15 + 1./3.*pow(m13,2) - 1./12.*m11*m13 )
         ;

      F13p6x23RV +=  + hh1dV(1,m36,m16,m66,m13,L,mu2) * ( 1./6.*m13*m66
          - 1./18.*m13*m66*RS645pe - 1./6.*m13*m36 + 1./18.*m13*m36*
         RS645pe - 1./12.*m13*m33 + 1./36.*m13*m33*RS645pe - 1./6.*m13*
         m16 + 1./18.*m13*m16*RS645pe + 1./3.*pow(m13,2) - 1./9.*pow(
         m13,2)*RS645pe - 1./12.*m11*m13 + 1./36.*m11*m13*RS645pe );

      F13p6x23RV +=  + hh1dV(1,m36,m16,mp2,m13,L,mu2) * (  - 1./18.*m13
         *mp2*RSp45613e - 1./18.*m13*mp2*RSp45633e + 1./6.*m13*mp2*
         RSp453e + 1./18.*m13*mp2*RSp451e - 1./9.*m13*mp2*RSp456e - 1./
         36.*m13*m66*RSp45613e - 1./36.*m13*m66*RSp45633e - 1./36.*m13*
         m66*RSp453e + 1./36.*m13*m66*RSp451e + 1./18.*m13*m66*RSp456e
          - 1./18.*m13*m36*RSp45613e + 1./9.*m13*m36*RSp45633e - 1./6.*
         m13*m36*RSp453e + 1./18.*m13*m36*RSp451e + 1./18.*m13*m36*
         RSp456e - 1./36.*m13*m33*RSp45613e - 1./18.*m13*m33*RSp45633e
          + 1./36.*m13*m33*RSp453e + 1./36.*m13*m33*RSp451e + 1./36.*
         m13*m33*RSp456e + 1./9.*m13*m16*RSp45613e - 1./18.*m13*m16*
         RSp45633e - 1./9.*m13*m16*RSp451e + 1./18.*m13*m16*RSp456e - 2.
         /9.*pow(m13,2)*RSp45613e + 1./9.*pow(m13,2)*RSp45633e + 2./9.*
         pow(m13,2)*RSp451e - 1./9.*pow(m13,2)*RSp456e - 1./18.*m11*m13
         *RSp45613e - 1./36.*m11*m13*RSp45633e + 1./18.*m11*m13*RSp451e
          + 1./36.*m11*m13*RSp456e );

      F13p6x23RV +=  + hh1dV(1,m36,m16,me2,m13,L,mu2) * (  - 1./18.*m13
         *me2*RSe45613p - 1./18.*m13*me2*RSe45633p + 1./6.*m13*me2*
         RSe453p + 1./18.*m13*me2*RSe451p - 1./9.*m13*me2*RSe456p - 1./
         36.*m13*m66*RSe45613p - 1./36.*m13*m66*RSe45633p - 1./36.*m13*
         m66*RSe453p + 1./36.*m13*m66*RSe451p + 1./18.*m13*m66*RSe456p
          - 1./18.*m13*m36*RSe45613p + 1./9.*m13*m36*RSe45633p - 1./6.*
         m13*m36*RSe453p + 1./18.*m13*m36*RSe451p + 1./18.*m13*m36*
         RSe456p - 1./36.*m13*m33*RSe45613p - 1./18.*m13*m33*RSe45633p
          + 1./36.*m13*m33*RSe453p + 1./36.*m13*m33*RSe451p + 1./36.*
         m13*m33*RSe456p + 1./9.*m13*m16*RSe45613p - 1./18.*m13*m16*
         RSe45633p - 1./9.*m13*m16*RSe451p + 1./18.*m13*m16*RSe456p - 2.
         /9.*pow(m13,2)*RSe45613p + 1./9.*pow(m13,2)*RSe45633p + 2./9.*
         pow(m13,2)*RSe451p - 1./9.*pow(m13,2)*RSe456p - 1./18.*m11*m13
         *RSe45613p - 1./36.*m11*m13*RSe45633p + 1./18.*m11*m13*RSe451p
          + 1./36.*m11*m13*RSe456p );

      F13p6x23RV +=  + hh1dV(2,m11,m11,m13,m13,L,mu2) * (  - 1./54.*m13
         *pow(RD1456ep,2) + 1./54.*m13*m33*RD1456ep*RC1456pe - 4./27.*
         pow(m13,2)*RD1456ep*RS14563pe + 2./27.*pow(m13,2)*RD1456ep*
         RC1456pe + 1./27.*m11*m13*RD1456ep*RS14563pe + 1./54.*m11*m13*
         RD1456ep*RC1456pe );

      F13p6x23RV +=  + hh1dV(2,m33,m14,m34,m13,L,mu2) * ( 1./36.*m13*
         m44*RD3456ep - 1./9.*m13*m34*RD3456ep + 1./9.*m13*m33*RD3456ep
          + 1./18.*m13*m14*RD3456ep - 1./9.*pow(m13,2)*RD3456ep + 1./36.
         *m11*m13*RD3456ep );

      F13p6x23RV +=  + hh1dV(2,m33,m15,m35,m13,L,mu2) * ( 1./36.*m13*
         m55*RD3456ep - 1./9.*m13*m35*RD3456ep + 1./9.*m13*m33*RD3456ep
          + 1./18.*m13*m15*RD3456ep - 1./9.*pow(m13,2)*RD3456ep + 1./36.
         *m11*m13*RD3456ep );

      F13p6x23RV +=  + hh1dV(2,m33,m16,m36,m13,L,mu2) * ( 1./36.*m13*
         m66*RD3456ep - 1./9.*m13*m36*RD3456ep + 1./9.*m13*m33*RD3456ep
          + 1./18.*m13*m16*RD3456ep - 1./9.*pow(m13,2)*RD3456ep + 1./36.
         *m11*m13*RD3456ep );

      F13p6x23RV +=  + hh1dV(3,m11,m11,m13,m13,L,mu2) * (  - 1./54.*m13
         *pow(RD1456ep,2) + 1./54.*m13*m33*RD1456ep*RC1456pe - 4./27.*
         pow(m13,2)*RD1456ep*RS14563pe + 2./27.*pow(m13,2)*RD1456ep*
         RC1456pe + 1./27.*m11*m13*RD1456ep*RS14563pe + 1./54.*m11*m13*
         RD1456ep*RC1456pe );

      F13p6x23RV +=  + hh1dV(3,m13,m11,m33,m13,L,mu2) * ( 1./54.*m13*
         RD1456ep*RD3456ep - 1./27.*m13*m33*RD1456ep*RS34561pe + 1./27.
         *m13*m33*RD1456ep*RC3456pe + 4./27.*pow(m13,2)*RD1456ep*
         RS34561pe - 2./27.*pow(m13,2)*RD1456ep*RC3456pe + 1./27.*m11*
         m13*RD1456ep*RC3456pe );

      F13p6x23RV +=  + hh1dV(3,m13,m11,mp2,m13,L,mu2) * (  - 1./27.*m13
         *mp2*RD1456ep*RSp45613e + 1./54.*m13*mp2*RD1456ep*RSp45633e + 
         1./54.*m13*mp2*RSp45611e*RD1456ep + 1./54.*m13*m33*RD1456ep*
         RSp45633e - 1./54.*m13*m33*RSp45611e*RD1456ep + 4./27.*pow(
         m13,2)*RD1456ep*RSp45613e - 2./27.*pow(m13,2)*RD1456ep*
         RSp45633e - 2./27.*pow(m13,2)*RSp45611e*RD1456ep + 1./27.*m11*
         m13*RD1456ep*RSp45633e - 1./27.*m11*m13*RSp45611e*RD1456ep );

      F13p6x23RV +=  + hh1dV(3,m13,m11,me2,m13,L,mu2) * (  - 1./27.*m13
         *me2*RD1456ep*RSe45613p + 1./54.*m13*me2*RD1456ep*RSe45633p + 
         1./54.*m13*me2*RSe45611p*RD1456ep + 1./54.*m13*m33*RD1456ep*
         RSe45633p - 1./54.*m13*m33*RSe45611p*RD1456ep + 4./27.*pow(
         m13,2)*RD1456ep*RSe45613p - 2./27.*pow(m13,2)*RD1456ep*
         RSe45633p - 2./27.*pow(m13,2)*RSe45611p*RD1456ep + 1./27.*m11*
         m13*RD1456ep*RSe45633p - 1./27.*m11*m13*RSe45611p*RD1456ep );

      F13p6x23RV +=  + hh1dV(3,m13,m33,m33,m13,L,mu2) * ( 23./1296.*m13
         *pow(RD3456ep,2) - 1./27.*m13*m33*RD3456ep*RS34561pe - 1./54.*
         m13*m33*RD3456ep*RC3456pe + 4./27.*pow(m13,2)*RD3456ep*
         RS34561pe - 2./27.*pow(m13,2)*RD3456ep*RC3456pe - 1./54.*m11*
         m13*RD3456ep*RC3456pe );

      F13p6x23RV +=  + hh1dV(3,m13,m33,mp2,m13,L,mu2) * (  - 1./27.*m13
         *mp2*RD3456ep*RSp45613e + 1./54.*m13*mp2*RSp45633e*RD3456ep + 
         1./54.*m13*mp2*RSp45611e*RD3456ep - 1./27.*m13*m33*RSp45633e*
         RD3456ep + 1./27.*m13*m33*RSp45611e*RD3456ep + 4./27.*pow(
         m13,2)*RD3456ep*RSp45613e - 2./27.*pow(m13,2)*RSp45633e*
         RD3456ep - 2./27.*pow(m13,2)*RSp45611e*RD3456ep - 1./54.*m11*
         m13*RSp45633e*RD3456ep + 1./54.*m11*m13*RSp45611e*RD3456ep );

      F13p6x23RV +=  + hh1dV(3,m13,m33,me2,m13,L,mu2) * (  - 1./27.*m13
         *me2*RD3456ep*RSe45613p + 1./54.*m13*me2*RSe45633p*RD3456ep + 
         1./54.*m13*me2*RSe45611p*RD3456ep - 1./27.*m13*m33*RSe45633p*
         RD3456ep + 1./27.*m13*m33*RSe45611p*RD3456ep + 4./27.*pow(
         m13,2)*RD3456ep*RSe45613p - 2./27.*pow(m13,2)*RSe45633p*
         RD3456ep - 2./27.*pow(m13,2)*RSe45611p*RD3456ep - 1./54.*m11*
         m13*RSe45633p*RD3456ep + 1./54.*m11*m13*RSe45611p*RD3456ep );

      F13p6x23RV +=  + hh1dV(3,m14,m11,m34,m13,L,mu2) * (  - 1./36.*m13
         *m44*RD1456ep - 1./18.*m13*m34*RD1456ep - 1./36.*m13*m33*
         RD1456ep + 1./9.*m13*m14*RD1456ep + 1./9.*pow(m13,2)*RD1456ep
          - 1./9.*m11*m13*RD1456ep );

      F13p6x23RV +=  + hh1dV(3,m14,m33,m34,m13,L,mu2) * ( 1./36.*m13*
         m44*RD3456ep - 1./9.*m13*m34*RD3456ep + 1./9.*m13*m33*RD3456ep
          + 1./18.*m13*m14*RD3456ep - 1./9.*pow(m13,2)*RD3456ep + 1./36.
         *m11*m13*RD3456ep );

      F13p6x23RV +=  + hh1dV(3,m15,m11,m35,m13,L,mu2) * (  - 1./36.*m13
         *m55*RD1456ep - 1./18.*m13*m35*RD1456ep - 1./36.*m13*m33*
         RD1456ep + 1./9.*m13*m15*RD1456ep + 1./9.*pow(m13,2)*RD1456ep
          - 1./9.*m11*m13*RD1456ep );

      F13p6x23RV +=  + hh1dV(3,m15,m33,m35,m13,L,mu2) * ( 1./36.*m13*
         m55*RD3456ep - 1./9.*m13*m35*RD3456ep + 1./9.*m13*m33*RD3456ep
          + 1./18.*m13*m15*RD3456ep - 1./9.*pow(m13,2)*RD3456ep + 1./36.
         *m11*m13*RD3456ep );

      F13p6x23RV +=  + hh1dV(3,m16,m11,m36,m13,L,mu2) * (  - 1./36.*m13
         *m66*RD1456ep - 1./18.*m13*m36*RD1456ep - 1./36.*m13*m33*
         RD1456ep + 1./9.*m13*m16*RD1456ep + 1./9.*pow(m13,2)*RD1456ep
          - 1./9.*m11*m13*RD1456ep );

      F13p6x23RV +=  + hh1dV(3,m16,m33,m36,m13,L,mu2) * ( 1./36.*m13*
         m66*RD3456ep - 1./9.*m13*m36*RD3456ep + 1./9.*m13*m33*RD3456ep
          + 1./18.*m13*m16*RD3456ep - 1./9.*pow(m13,2)*RD3456ep + 1./36.
         *m11*m13*RD3456ep );

      F13p6x23RV +=  + hh1dV(4,m13,m11,m33,m13,L,mu2) * ( 1./54.*m13*
         RD1456ep*RD3456ep + 1./27.*m13*m33*RC1456pe*RD3456ep + 4./27.*
         pow(m13,2)*RD3456ep*RS14563pe - 2./27.*pow(m13,2)*RC1456pe*
         RD3456ep - 1./27.*m11*m13*RD3456ep*RS14563pe + 1./27.*m11*m13*
         RC1456pe*RD3456ep );

      F13p6x23RV +=  + hh1dV(4,m13,m33,m33,m13,L,mu2) * ( 1./1296.*m13*
         pow(RD3456ep,2) );

      F13p6x23RV +=  + hh1dV(5,m11,m11,m13,m13,L,mu2) * ( 1./54.*m13*
         m33*pow(RD1456ep,2) + 2./27.*pow(m13,2)*pow(RD1456ep,2) + 1./
         54.*m11*m13*pow(RD1456ep,2) );

      F13p6x23RV +=  + hh1dV(7,m13,m11,m33,m13,L,mu2) * ( 1./27.*m13*
         m33*RD1456ep*RD3456ep - 2./27.*pow(m13,2)*RD1456ep*RD3456ep + 
         1./27.*m11*m13*RD1456ep*RD3456ep );

      F13p6x23RV +=  + hh1dV(7,m13,m33,m33,m13,L,mu2) * (  - 1./108.*
         m13*m33*pow(RD3456ep,2) - 1./27.*pow(m13,2)*pow(RD3456ep,2) - 
         1./108.*m11*m13*pow(RD3456ep,2) );

      F13p6x23RV +=  + hh21dV(1,m11,m11,m13,m13,L,mu2) * (  - 3./8.*
         pow(m13,2) );

      F13p6x23RV +=  + hh21dV(1,m11,m14,m34,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RS14563pe + 1./8.*pow(m13,2)*RS156pe );

      F13p6x23RV +=  + hh21dV(1,m11,m15,m35,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RS14563pe + 1./8.*pow(m13,2)*RS146pe );

      F13p6x23RV +=  + hh21dV(1,m11,m16,m36,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RS14563pe + 1./8.*pow(m13,2)*RS145pe );

      F13p6x23RV +=  + hh21dV(1,m13,m11,m11,m13,L,mu2) * ( 3./16.*pow(
         m13,2) + 1./12.*pow(m13,2)*pow(RS14563pe,2) - 1./12.*pow(
         m13,2)*RC1456pe*RS14563pe + 1./48.*pow(m13,2)*pow(RC1456pe,2)
          );

      F13p6x23RV +=  + hh21dV(1,m13,m11,m33,m13,L,mu2) * ( 1./6.*pow(
         m13,2)*RS14563pe*RS34561pe - 1./12.*pow(m13,2)*RC3456pe*
         RS14563pe - 1./12.*pow(m13,2)*RC1456pe*RS34561pe + 1./24.*pow(
         m13,2)*RC1456pe*RC3456pe );

      F13p6x23RV +=  + hh21dV(1,m13,m11,mp2,m13,L,mu2) * ( 1./6.*pow(
         m13,2)*RSp45613e*RS14563pe - 1./12.*pow(m13,2)*RSp45633e*
         RS14563pe - 1./12.*pow(m13,2)*RC1456pe*RSp45613e + 1./24.*pow(
         m13,2)*RC1456pe*RSp45633e - 1./12.*pow(m13,2)*RSp45611e*
         RS14563pe + 1./24.*pow(m13,2)*RSp45611e*RC1456pe );

      F13p6x23RV +=  + hh21dV(1,m13,m11,me2,m13,L,mu2) * ( 1./6.*pow(
         m13,2)*RSe45613p*RS14563pe - 1./12.*pow(m13,2)*RSe45633p*
         RS14563pe - 1./12.*pow(m13,2)*RC1456pe*RSe45613p + 1./24.*pow(
         m13,2)*RC1456pe*RSe45633p - 1./12.*pow(m13,2)*RSe45611p*
         RS14563pe + 1./24.*pow(m13,2)*RSe45611p*RC1456pe );

      F13p6x23RV +=  + hh21dV(1,m13,m13,m13,m13,L,mu2) * ( 3./8.*pow(
         m13,2) );

      F13p6x23RV +=  + hh21dV(1,m13,m33,m33,m13,L,mu2) * ( 3./16.*pow(
         m13,2) + 1./12.*pow(m13,2)*pow(RS34561pe,2) - 1./12.*pow(
         m13,2)*RC3456pe*RS34561pe + 1./48.*pow(m13,2)*pow(RC3456pe,2)
          );

      F13p6x23RV +=  + hh21dV(1,m13,m33,mp2,m13,L,mu2) * ( 1./6.*pow(
         m13,2)*RSp45613e*RS34561pe - 1./12.*pow(m13,2)*RC3456pe*
         RSp45613e - 1./12.*pow(m13,2)*RSp45633e*RS34561pe + 1./24.*
         pow(m13,2)*RSp45633e*RC3456pe - 1./12.*pow(m13,2)*RSp45611e*
         RS34561pe + 1./24.*pow(m13,2)*RSp45611e*RC3456pe );

      F13p6x23RV +=  + hh21dV(1,m13,m33,me2,m13,L,mu2) * ( 1./6.*pow(
         m13,2)*RSe45613p*RS34561pe - 1./12.*pow(m13,2)*RC3456pe*
         RSe45613p - 1./12.*pow(m13,2)*RSe45633p*RS34561pe + 1./24.*
         pow(m13,2)*RSe45633p*RC3456pe - 1./12.*pow(m13,2)*RSe45611p*
         RS34561pe + 1./24.*pow(m13,2)*RSe45611p*RC3456pe );

      F13p6x23RV +=  + hh21dV(1,m13,mp2,mp2,m13,L,mu2) * ( 1./12.*pow(
         m13,2)*pow(RSp45613e,2) - 1./12.*pow(m13,2)*RSp45633e*
         RSp45613e + 1./48.*pow(m13,2)*pow(RSp45633e,2) - 1./12.*pow(
         m13,2)*RSp45611e*RSp45613e + 1./24.*pow(m13,2)*RSp45611e*
         RSp45633e + 1./48.*pow(m13,2)*pow(RSp45611e,2) );

      F13p6x23RV +=  + hh21dV(1,m13,me2,mp2,m13,L,mu2) * ( 1./6.*pow(
         m13,2)*RSp45613e*RSe45613p - 1./12.*pow(m13,2)*RSe45633p*
         RSp45613e - 1./12.*pow(m13,2)*RSp45633e*RSe45613p + 1./24.*
         pow(m13,2)*RSp45633e*RSe45633p - 1./12.*pow(m13,2)*RSe45611p*
         RSp45613e + 1./24.*pow(m13,2)*RSe45611p*RSp45633e - 1./12.*
         pow(m13,2)*RSp45611e*RSe45613p + 1./24.*pow(m13,2)*RSp45611e*
         RSe45633p + 1./24.*pow(m13,2)*RSp45611e*RSe45611p );

      F13p6x23RV +=  + hh21dV(1,m13,me2,me2,m13,L,mu2) * ( 1./12.*pow(
         m13,2)*pow(RSe45613p,2) - 1./12.*pow(m13,2)*RSe45633p*
         RSe45613p + 1./48.*pow(m13,2)*pow(RSe45633p,2) - 1./12.*pow(
         m13,2)*RSe45611p*RSe45613p + 1./24.*pow(m13,2)*RSe45611p*
         RSe45633p + 1./48.*pow(m13,2)*pow(RSe45611p,2) );

      F13p6x23RV +=  + hh21dV(1,m14,m11,m34,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RS14563pe - 1./8.*pow(m13,2)*RC1456pe + 1./8.*pow(
         m13,2)*RS156pe );

      F13p6x23RV +=  + hh21dV(1,m14,m33,m34,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RS34561pe - 1./8.*pow(m13,2)*RS356pe );

      F13p6x23RV +=  + hh21dV(1,m14,m34,mp2,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RSp45613e - 1./8.*pow(m13,2)*RSp563e - 1./8.*pow(m13,2)
         *RSp45611e + 1./8.*pow(m13,2)*RSp561e );

      F13p6x23RV +=  + hh21dV(1,m14,m34,me2,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RSe45613p - 1./8.*pow(m13,2)*RSe563p - 1./8.*pow(m13,2)
         *RSe45611p + 1./8.*pow(m13,2)*RSe561p );

      F13p6x23RV +=  + hh21dV(1,m15,m11,m35,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RS14563pe - 1./8.*pow(m13,2)*RC1456pe + 1./8.*pow(
         m13,2)*RS146pe );

      F13p6x23RV +=  + hh21dV(1,m15,m33,m35,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RS34561pe - 1./8.*pow(m13,2)*RS346pe );

      F13p6x23RV +=  + hh21dV(1,m15,m35,mp2,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RSp45613e - 1./8.*pow(m13,2)*RSp463e - 1./8.*pow(m13,2)
         *RSp45611e + 1./8.*pow(m13,2)*RSp461e );

      F13p6x23RV +=  + hh21dV(1,m15,m35,me2,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RSe45613p - 1./8.*pow(m13,2)*RSe463p - 1./8.*pow(m13,2)
         *RSe45611p + 1./8.*pow(m13,2)*RSe461p );

      F13p6x23RV +=  + hh21dV(1,m16,m11,m36,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RS14563pe - 1./8.*pow(m13,2)*RC1456pe + 1./8.*pow(
         m13,2)*RS145pe );

      F13p6x23RV +=  + hh21dV(1,m16,m33,m36,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RS34561pe - 1./8.*pow(m13,2)*RS345pe );

      F13p6x23RV +=  + hh21dV(1,m16,m36,mp2,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RSp45613e - 1./8.*pow(m13,2)*RSp453e - 1./8.*pow(m13,2)
         *RSp45611e + 1./8.*pow(m13,2)*RSp451e );

      F13p6x23RV +=  + hh21dV(1,m16,m36,me2,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RSe45613p - 1./8.*pow(m13,2)*RSe453p - 1./8.*pow(m13,2)
         *RSe45611p + 1./8.*pow(m13,2)*RSe451p );

      F13p6x23RV +=  + hh21dV(1,m33,m13,m33,m13,L,mu2) * (  - 3./8.*
         pow(m13,2) );

      F13p6x23RV +=  + hh21dV(1,m33,m14,m34,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RS34561pe + 1./8.*pow(m13,2)*RS356pe );

      F13p6x23RV +=  + hh21dV(1,m33,m15,m35,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RS34561pe + 1./8.*pow(m13,2)*RS346pe );

      F13p6x23RV +=  + hh21dV(1,m33,m16,m36,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RS34561pe + 1./8.*pow(m13,2)*RS345pe );

      F13p6x23RV +=  + hh21dV(1,m34,m11,m14,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RS14563pe - 1./8.*pow(m13,2)*RS156pe );

      F13p6x23RV +=  + hh21dV(1,m34,m14,m33,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RS34561pe - 1./8.*pow(m13,2)*RC3456pe + 1./8.*pow(
         m13,2)*RS356pe );

      F13p6x23RV +=  + hh21dV(1,m34,m14,mp2,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RSp45613e - 1./8.*pow(m13,2)*RSp45633e + 1./8.*pow(
         m13,2)*RSp563e - 1./8.*pow(m13,2)*RSp561e );

      F13p6x23RV +=  + hh21dV(1,m34,m14,me2,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RSe45613p - 1./8.*pow(m13,2)*RSe45633p + 1./8.*pow(
         m13,2)*RSe563p - 1./8.*pow(m13,2)*RSe561p );

      F13p6x23RV +=  + hh21dV(1,m35,m11,m15,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RS14563pe - 1./8.*pow(m13,2)*RS146pe );

      F13p6x23RV +=  + hh21dV(1,m35,m15,m33,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RS34561pe - 1./8.*pow(m13,2)*RC3456pe + 1./8.*pow(
         m13,2)*RS346pe );

      F13p6x23RV +=  + hh21dV(1,m35,m15,mp2,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RSp45613e - 1./8.*pow(m13,2)*RSp45633e + 1./8.*pow(
         m13,2)*RSp463e - 1./8.*pow(m13,2)*RSp461e );

      F13p6x23RV +=  + hh21dV(1,m35,m15,me2,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RSe45613p - 1./8.*pow(m13,2)*RSe45633p + 1./8.*pow(
         m13,2)*RSe463p - 1./8.*pow(m13,2)*RSe461p );

      F13p6x23RV +=  + hh21dV(1,m36,m11,m16,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RS14563pe - 1./8.*pow(m13,2)*RS145pe );

      F13p6x23RV +=  + hh21dV(1,m36,m16,m33,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RS34561pe - 1./8.*pow(m13,2)*RC3456pe + 1./8.*pow(
         m13,2)*RS345pe );

      F13p6x23RV +=  + hh21dV(1,m36,m16,mp2,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RSp45613e - 1./8.*pow(m13,2)*RSp45633e + 1./8.*pow(
         m13,2)*RSp453e - 1./8.*pow(m13,2)*RSp451e );

      F13p6x23RV +=  + hh21dV(1,m36,m16,me2,m13,L,mu2) * ( 1./8.*pow(
         m13,2)*RSe45613p - 1./8.*pow(m13,2)*RSe45633p + 1./8.*pow(
         m13,2)*RSe453p - 1./8.*pow(m13,2)*RSe451p );

      F13p6x23RV +=  + hh21dV(1,m44,m14,m34,m13,L,mu2) * ( 3./8.*pow(
         m13,2) - 1./8.*pow(m13,2)*RS456pe );

      F13p6x23RV +=  + hh21dV(1,m45,m14,m35,m13,L,mu2) * ( 3./8.*pow(
         m13,2) );

      F13p6x23RV +=  + hh21dV(1,m45,m15,m34,m13,L,mu2) * ( 3./8.*pow(
         m13,2) );

      F13p6x23RV +=  + hh21dV(1,m46,m14,m36,m13,L,mu2) * ( 3./8.*pow(
         m13,2) );

      F13p6x23RV +=  + hh21dV(1,m46,m16,m34,m13,L,mu2) * ( 3./8.*pow(
         m13,2) );

      F13p6x23RV +=  + hh21dV(1,m55,m15,m35,m13,L,mu2) * ( 3./8.*pow(
         m13,2) - 1./8.*pow(m13,2)*RS546pe );

      F13p6x23RV +=  + hh21dV(1,m56,m15,m36,m13,L,mu2) * ( 3./8.*pow(
         m13,2) );

      F13p6x23RV +=  + hh21dV(1,m56,m16,m35,m13,L,mu2) * ( 3./8.*pow(
         m13,2) );

      F13p6x23RV +=  + hh21dV(1,m66,m16,m36,m13,L,mu2) * ( 3./8.*pow(
         m13,2) - 1./8.*pow(m13,2)*RS645pe );

      F13p6x23RV +=  + hh21dV(1,mp2,m14,m34,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RSp45613e + 1./8.*pow(m13,2)*RSp563e + 1./8.*pow(
         m13,2)*RSp561e - 1./8.*pow(m13,2)*RSp564e );

      F13p6x23RV +=  + hh21dV(1,mp2,m15,m35,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RSp45613e + 1./8.*pow(m13,2)*RSp463e + 1./8.*pow(
         m13,2)*RSp461e - 1./8.*pow(m13,2)*RSp465e );

      F13p6x23RV +=  + hh21dV(1,mp2,m16,m36,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RSp45613e + 1./8.*pow(m13,2)*RSp453e + 1./8.*pow(
         m13,2)*RSp451e - 1./8.*pow(m13,2)*RSp456e );

      F13p6x23RV +=  + hh21dV(1,me2,m14,m34,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RSe45613p + 1./8.*pow(m13,2)*RSe563p + 1./8.*pow(
         m13,2)*RSe561p - 1./8.*pow(m13,2)*RSe564p );

      F13p6x23RV +=  + hh21dV(1,me2,m15,m35,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RSe45613p + 1./8.*pow(m13,2)*RSe463p + 1./8.*pow(
         m13,2)*RSe461p - 1./8.*pow(m13,2)*RSe465p );

      F13p6x23RV +=  + hh21dV(1,me2,m16,m36,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RSe45613p + 1./8.*pow(m13,2)*RSe453p + 1./8.*pow(
         m13,2)*RSe451p - 1./8.*pow(m13,2)*RSe456p );

      F13p6x23RV +=  + hh21dV(3,m13,m11,m11,m13,L,mu2) * (  - 1./12.*
         pow(m13,2)*RD1456ep*RS14563pe + 1./24.*pow(m13,2)*RD1456ep*
         RC1456pe );

      F13p6x23RV +=  + hh21dV(3,m13,m11,m33,m13,L,mu2) * (  - 1./12.*
         pow(m13,2)*RD1456ep*RS34561pe + 1./24.*pow(m13,2)*RD1456ep*
         RC3456pe );

      F13p6x23RV +=  + hh21dV(3,m13,m11,mp2,m13,L,mu2) * (  - 1./12.*
         pow(m13,2)*RD1456ep*RSp45613e + 1./24.*pow(m13,2)*RD1456ep*
         RSp45633e + 1./24.*pow(m13,2)*RSp45611e*RD1456ep );

      F13p6x23RV +=  + hh21dV(3,m13,m11,me2,m13,L,mu2) * (  - 1./12.*
         pow(m13,2)*RD1456ep*RSe45613p + 1./24.*pow(m13,2)*RD1456ep*
         RSe45633p + 1./24.*pow(m13,2)*RSe45611p*RD1456ep );

      F13p6x23RV +=  + hh21dV(3,m13,m33,m33,m13,L,mu2) * (  - 1./12.*
         pow(m13,2)*RD3456ep*RS34561pe + 1./24.*pow(m13,2)*RD3456ep*
         RC3456pe );

      F13p6x23RV +=  + hh21dV(3,m13,m33,mp2,m13,L,mu2) * (  - 1./12.*
         pow(m13,2)*RD3456ep*RSp45613e + 1./24.*pow(m13,2)*RSp45633e*
         RD3456ep + 1./24.*pow(m13,2)*RSp45611e*RD3456ep );

      F13p6x23RV +=  + hh21dV(3,m13,m33,me2,m13,L,mu2) * (  - 1./12.*
         pow(m13,2)*RD3456ep*RSe45613p + 1./24.*pow(m13,2)*RSe45633p*
         RD3456ep + 1./24.*pow(m13,2)*RSe45611p*RD3456ep );

      F13p6x23RV +=  + hh21dV(3,m14,m11,m34,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RD1456ep );

      F13p6x23RV +=  + hh21dV(3,m15,m11,m35,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RD1456ep );

      F13p6x23RV +=  + hh21dV(3,m16,m11,m36,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RD1456ep );

      F13p6x23RV +=  + hh21dV(4,m13,m11,m33,m13,L,mu2) * (  - 1./12.*
         pow(m13,2)*RD3456ep*RS14563pe + 1./24.*pow(m13,2)*RC1456pe*
         RD3456ep );

      F13p6x23RV +=  + hh21dV(4,m34,m14,m33,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RD3456ep );

      F13p6x23RV +=  + hh21dV(4,m35,m15,m33,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RD3456ep );

      F13p6x23RV +=  + hh21dV(4,m36,m16,m33,m13,L,mu2) * (  - 1./8.*
         pow(m13,2)*RD3456ep );

      F13p6x23RV +=  + hh21dV(7,m13,m11,m11,m13,L,mu2) * ( 1./48.*pow(
         m13,2)*pow(RD1456ep,2) );

      F13p6x23RV +=  + hh21dV(7,m13,m11,m33,m13,L,mu2) * ( 1./24.*pow(
         m13,2)*RD1456ep*RD3456ep );

      F13p6x23RV +=  + hh21dV(7,m13,m33,m33,m13,L,mu2) * ( 1./48.*pow(
         m13,2)*pow(RD3456ep,2) );

      F13p6x23RV +=  + hh27dV(1,m11,m11,m13,m13,L,mu2) * ( 3./8.*m13 );

      F13p6x23RV +=  + hh27dV(1,m11,m14,m34,m13,L,mu2) * ( 1./8.*m13*
         RS14563pe - 1./8.*m13*RS156pe );

      F13p6x23RV +=  + hh27dV(1,m11,m15,m35,m13,L,mu2) * ( 1./8.*m13*
         RS14563pe - 1./8.*m13*RS146pe );

      F13p6x23RV +=  + hh27dV(1,m11,m16,m36,m13,L,mu2) * ( 1./8.*m13*
         RS14563pe - 1./8.*m13*RS145pe );

      F13p6x23RV +=  + hh27dV(1,m13,m11,m11,m13,L,mu2) * (  - 3./16.*
         m13 - 1./12.*m13*pow(RS14563pe,2) + 1./12.*m13*RC1456pe*
         RS14563pe - 1./48.*m13*pow(RC1456pe,2) );

      F13p6x23RV +=  + hh27dV(1,m13,m11,m33,m13,L,mu2) * (  - 1./6.*m13
         *RS14563pe*RS34561pe + 1./12.*m13*RC3456pe*RS14563pe + 1./12.*
         m13*RC1456pe*RS34561pe - 1./24.*m13*RC1456pe*RC3456pe );

      F13p6x23RV +=  + hh27dV(1,m13,m11,mp2,m13,L,mu2) * (  - 1./6.*m13
         *RSp45613e*RS14563pe + 1./12.*m13*RSp45633e*RS14563pe + 1./12.
         *m13*RC1456pe*RSp45613e - 1./24.*m13*RC1456pe*RSp45633e + 1./
         12.*m13*RSp45611e*RS14563pe - 1./24.*m13*RSp45611e*RC1456pe );

      F13p6x23RV +=  + hh27dV(1,m13,m11,me2,m13,L,mu2) * (  - 1./6.*m13
         *RSe45613p*RS14563pe + 1./12.*m13*RSe45633p*RS14563pe + 1./12.
         *m13*RC1456pe*RSe45613p - 1./24.*m13*RC1456pe*RSe45633p + 1./
         12.*m13*RSe45611p*RS14563pe - 1./24.*m13*RSe45611p*RC1456pe );

      F13p6x23RV +=  + hh27dV(1,m13,m13,m13,m13,L,mu2) * (  - 3./8.*m13
          );

      F13p6x23RV +=  + hh27dV(1,m13,m33,m33,m13,L,mu2) * (  - 3./16.*
         m13 - 1./12.*m13*pow(RS34561pe,2) + 1./12.*m13*RC3456pe*
         RS34561pe - 1./48.*m13*pow(RC3456pe,2) );

      F13p6x23RV +=  + hh27dV(1,m13,m33,mp2,m13,L,mu2) * (  - 1./6.*m13
         *RSp45613e*RS34561pe + 1./12.*m13*RC3456pe*RSp45613e + 1./12.*
         m13*RSp45633e*RS34561pe - 1./24.*m13*RSp45633e*RC3456pe + 1./
         12.*m13*RSp45611e*RS34561pe - 1./24.*m13*RSp45611e*RC3456pe );

      F13p6x23RV +=  + hh27dV(1,m13,m33,me2,m13,L,mu2) * (  - 1./6.*m13
         *RSe45613p*RS34561pe + 1./12.*m13*RC3456pe*RSe45613p + 1./12.*
         m13*RSe45633p*RS34561pe - 1./24.*m13*RSe45633p*RC3456pe + 1./
         12.*m13*RSe45611p*RS34561pe - 1./24.*m13*RSe45611p*RC3456pe );

      F13p6x23RV +=  + hh27dV(1,m13,mp2,mp2,m13,L,mu2) * (  - 1./12.*
         m13*pow(RSp45613e,2) + 1./12.*m13*RSp45633e*RSp45613e - 1./48.
         *m13*pow(RSp45633e,2) + 1./12.*m13*RSp45611e*RSp45613e - 1./24.
         *m13*RSp45611e*RSp45633e - 1./48.*m13*pow(RSp45611e,2) );

      F13p6x23RV +=  + hh27dV(1,m13,me2,mp2,m13,L,mu2) * (  - 1./6.*m13
         *RSp45613e*RSe45613p + 1./12.*m13*RSe45633p*RSp45613e + 1./12.
         *m13*RSp45633e*RSe45613p - 1./24.*m13*RSp45633e*RSe45633p + 1./
         12.*m13*RSe45611p*RSp45613e - 1./24.*m13*RSe45611p*RSp45633e
          + 1./12.*m13*RSp45611e*RSe45613p - 1./24.*m13*RSp45611e*
         RSe45633p - 1./24.*m13*RSp45611e*RSe45611p );

      F13p6x23RV +=  + hh27dV(1,m13,me2,me2,m13,L,mu2) * (  - 1./12.*
         m13*pow(RSe45613p,2) + 1./12.*m13*RSe45633p*RSe45613p - 1./48.
         *m13*pow(RSe45633p,2) + 1./12.*m13*RSe45611p*RSe45613p - 1./24.
         *m13*RSe45611p*RSe45633p - 1./48.*m13*pow(RSe45611p,2) );

      F13p6x23RV +=  + hh27dV(1,m14,m11,m34,m13,L,mu2) * (  - 1./8.*m13
         *RS14563pe + 1./8.*m13*RC1456pe - 1./8.*m13*RS156pe );

      F13p6x23RV +=  + hh27dV(1,m14,m33,m34,m13,L,mu2) * (  - 1./8.*m13
         *RS34561pe + 1./8.*m13*RS356pe );

      F13p6x23RV +=  + hh27dV(1,m14,m34,mp2,m13,L,mu2) * (  - 1./8.*m13
         *RSp45613e + 1./8.*m13*RSp563e + 1./8.*m13*RSp45611e - 1./8.*
         m13*RSp561e );

      F13p6x23RV +=  + hh27dV(1,m14,m34,me2,m13,L,mu2) * (  - 1./8.*m13
         *RSe45613p + 1./8.*m13*RSe563p + 1./8.*m13*RSe45611p - 1./8.*
         m13*RSe561p );

      F13p6x23RV +=  + hh27dV(1,m15,m11,m35,m13,L,mu2) * (  - 1./8.*m13
         *RS14563pe + 1./8.*m13*RC1456pe - 1./8.*m13*RS146pe );

      F13p6x23RV +=  + hh27dV(1,m15,m33,m35,m13,L,mu2) * (  - 1./8.*m13
         *RS34561pe + 1./8.*m13*RS346pe );

      F13p6x23RV +=  + hh27dV(1,m15,m35,mp2,m13,L,mu2) * (  - 1./8.*m13
         *RSp45613e + 1./8.*m13*RSp463e + 1./8.*m13*RSp45611e - 1./8.*
         m13*RSp461e );

      F13p6x23RV +=  + hh27dV(1,m15,m35,me2,m13,L,mu2) * (  - 1./8.*m13
         *RSe45613p + 1./8.*m13*RSe463p + 1./8.*m13*RSe45611p - 1./8.*
         m13*RSe461p );

      F13p6x23RV +=  + hh27dV(1,m16,m11,m36,m13,L,mu2) * (  - 1./8.*m13
         *RS14563pe + 1./8.*m13*RC1456pe - 1./8.*m13*RS145pe );

      F13p6x23RV +=  + hh27dV(1,m16,m33,m36,m13,L,mu2) * (  - 1./8.*m13
         *RS34561pe + 1./8.*m13*RS345pe );

      F13p6x23RV +=  + hh27dV(1,m16,m36,mp2,m13,L,mu2) * (  - 1./8.*m13
         *RSp45613e + 1./8.*m13*RSp453e + 1./8.*m13*RSp45611e - 1./8.*
         m13*RSp451e );

      F13p6x23RV +=  + hh27dV(1,m16,m36,me2,m13,L,mu2) * (  - 1./8.*m13
         *RSe45613p + 1./8.*m13*RSe453p + 1./8.*m13*RSe45611p - 1./8.*
         m13*RSe451p );

      F13p6x23RV +=  + hh27dV(1,m33,m13,m33,m13,L,mu2) * ( 3./8.*m13 );

      F13p6x23RV +=  + hh27dV(1,m33,m14,m34,m13,L,mu2) * ( 1./8.*m13*
         RS34561pe - 1./8.*m13*RS356pe );

      F13p6x23RV +=  + hh27dV(1,m33,m15,m35,m13,L,mu2) * ( 1./8.*m13*
         RS34561pe - 1./8.*m13*RS346pe );

      F13p6x23RV +=  + hh27dV(1,m33,m16,m36,m13,L,mu2) * ( 1./8.*m13*
         RS34561pe - 1./8.*m13*RS345pe );

      F13p6x23RV +=  + hh27dV(1,m34,m11,m14,m13,L,mu2) * (  - 1./8.*m13
         *RS14563pe + 1./8.*m13*RS156pe );

      F13p6x23RV +=  + hh27dV(1,m34,m14,m33,m13,L,mu2) * (  - 1./8.*m13
         *RS34561pe + 1./8.*m13*RC3456pe - 1./8.*m13*RS356pe );

      F13p6x23RV +=  + hh27dV(1,m34,m14,mp2,m13,L,mu2) * (  - 1./8.*m13
         *RSp45613e + 1./8.*m13*RSp45633e - 1./8.*m13*RSp563e + 1./8.*
         m13*RSp561e );

      F13p6x23RV +=  + hh27dV(1,m34,m14,me2,m13,L,mu2) * (  - 1./8.*m13
         *RSe45613p + 1./8.*m13*RSe45633p - 1./8.*m13*RSe563p + 1./8.*
         m13*RSe561p );

      F13p6x23RV +=  + hh27dV(1,m35,m11,m15,m13,L,mu2) * (  - 1./8.*m13
         *RS14563pe + 1./8.*m13*RS146pe );

      F13p6x23RV +=  + hh27dV(1,m35,m15,m33,m13,L,mu2) * (  - 1./8.*m13
         *RS34561pe + 1./8.*m13*RC3456pe - 1./8.*m13*RS346pe );

      F13p6x23RV +=  + hh27dV(1,m35,m15,mp2,m13,L,mu2) * (  - 1./8.*m13
         *RSp45613e + 1./8.*m13*RSp45633e - 1./8.*m13*RSp463e + 1./8.*
         m13*RSp461e );

      F13p6x23RV +=  + hh27dV(1,m35,m15,me2,m13,L,mu2) * (  - 1./8.*m13
         *RSe45613p + 1./8.*m13*RSe45633p - 1./8.*m13*RSe463p + 1./8.*
         m13*RSe461p );

      F13p6x23RV +=  + hh27dV(1,m36,m11,m16,m13,L,mu2) * (  - 1./8.*m13
         *RS14563pe + 1./8.*m13*RS145pe );

      F13p6x23RV +=  + hh27dV(1,m36,m16,m33,m13,L,mu2) * (  - 1./8.*m13
         *RS34561pe + 1./8.*m13*RC3456pe - 1./8.*m13*RS345pe );

      F13p6x23RV +=  + hh27dV(1,m36,m16,mp2,m13,L,mu2) * (  - 1./8.*m13
         *RSp45613e + 1./8.*m13*RSp45633e - 1./8.*m13*RSp453e + 1./8.*
         m13*RSp451e );

      F13p6x23RV +=  + hh27dV(1,m36,m16,me2,m13,L,mu2) * (  - 1./8.*m13
         *RSe45613p + 1./8.*m13*RSe45633p - 1./8.*m13*RSe453p + 1./8.*
         m13*RSe451p );

      F13p6x23RV +=  + hh27dV(1,m44,m14,m34,m13,L,mu2) * (  - 3./8.*m13
          + 1./8.*m13*RS456pe );

      F13p6x23RV +=  + hh27dV(1,m45,m14,m35,m13,L,mu2) * (  - 3./8.*m13
          );

      F13p6x23RV +=  + hh27dV(1,m45,m15,m34,m13,L,mu2) * (  - 3./8.*m13
          );

      F13p6x23RV +=  + hh27dV(1,m46,m14,m36,m13,L,mu2) * (  - 3./8.*m13
          );

      F13p6x23RV +=  + hh27dV(1,m46,m16,m34,m13,L,mu2) * (  - 3./8.*m13
          );

      F13p6x23RV +=  + hh27dV(1,m55,m15,m35,m13,L,mu2) * (  - 3./8.*m13
          + 1./8.*m13*RS546pe );

      F13p6x23RV +=  + hh27dV(1,m56,m15,m36,m13,L,mu2) * (  - 3./8.*m13
          );

      F13p6x23RV +=  + hh27dV(1,m56,m16,m35,m13,L,mu2) * (  - 3./8.*m13
          );

      F13p6x23RV +=  + hh27dV(1,m66,m16,m36,m13,L,mu2) * (  - 3./8.*m13
          + 1./8.*m13*RS645pe );

      F13p6x23RV +=  + hh27dV(1,mp2,m14,m34,m13,L,mu2) * ( 1./8.*m13*
         RSp45613e - 1./8.*m13*RSp563e - 1./8.*m13*RSp561e + 1./8.*m13*
         RSp564e );

      F13p6x23RV +=  + hh27dV(1,mp2,m15,m35,m13,L,mu2) * ( 1./8.*m13*
         RSp45613e - 1./8.*m13*RSp463e - 1./8.*m13*RSp461e + 1./8.*m13*
         RSp465e );

      F13p6x23RV +=  + hh27dV(1,mp2,m16,m36,m13,L,mu2) * ( 1./8.*m13*
         RSp45613e - 1./8.*m13*RSp453e - 1./8.*m13*RSp451e + 1./8.*m13*
         RSp456e );

      F13p6x23RV +=  + hh27dV(1,me2,m14,m34,m13,L,mu2) * ( 1./8.*m13*
         RSe45613p - 1./8.*m13*RSe563p - 1./8.*m13*RSe561p + 1./8.*m13*
         RSe564p );

      F13p6x23RV +=  + hh27dV(1,me2,m15,m35,m13,L,mu2) * ( 1./8.*m13*
         RSe45613p - 1./8.*m13*RSe463p - 1./8.*m13*RSe461p + 1./8.*m13*
         RSe465p );

      F13p6x23RV +=  + hh27dV(1,me2,m16,m36,m13,L,mu2) * ( 1./8.*m13*
         RSe45613p - 1./8.*m13*RSe453p - 1./8.*m13*RSe451p + 1./8.*m13*
         RSe456p );

      F13p6x23RV +=  + hh27dV(3,m13,m11,m11,m13,L,mu2) * ( 1./12.*m13*
         RD1456ep*RS14563pe - 1./24.*m13*RD1456ep*RC1456pe );

      F13p6x23RV +=  + hh27dV(3,m13,m11,m33,m13,L,mu2) * ( 1./12.*m13*
         RD1456ep*RS34561pe - 1./24.*m13*RD1456ep*RC3456pe );

      F13p6x23RV +=  + hh27dV(3,m13,m11,mp2,m13,L,mu2) * ( 1./12.*m13*
         RD1456ep*RSp45613e - 1./24.*m13*RD1456ep*RSp45633e - 1./24.*
         m13*RSp45611e*RD1456ep );

      F13p6x23RV +=  + hh27dV(3,m13,m11,me2,m13,L,mu2) * ( 1./12.*m13*
         RD1456ep*RSe45613p - 1./24.*m13*RD1456ep*RSe45633p - 1./24.*
         m13*RSe45611p*RD1456ep );

      F13p6x23RV +=  + hh27dV(3,m13,m33,m33,m13,L,mu2) * ( 1./12.*m13*
         RD3456ep*RS34561pe - 1./24.*m13*RD3456ep*RC3456pe );

      F13p6x23RV +=  + hh27dV(3,m13,m33,mp2,m13,L,mu2) * ( 1./12.*m13*
         RD3456ep*RSp45613e - 1./24.*m13*RSp45633e*RD3456ep - 1./24.*
         m13*RSp45611e*RD3456ep );

      F13p6x23RV +=  + hh27dV(3,m13,m33,me2,m13,L,mu2) * ( 1./12.*m13*
         RD3456ep*RSe45613p - 1./24.*m13*RSe45633p*RD3456ep - 1./24.*
         m13*RSe45611p*RD3456ep );

      F13p6x23RV +=  + hh27dV(3,m14,m11,m34,m13,L,mu2) * ( 1./8.*m13*
         RD1456ep );

      F13p6x23RV +=  + hh27dV(3,m15,m11,m35,m13,L,mu2) * ( 1./8.*m13*
         RD1456ep );

      F13p6x23RV +=  + hh27dV(3,m16,m11,m36,m13,L,mu2) * ( 1./8.*m13*
         RD1456ep );

      F13p6x23RV +=  + hh27dV(4,m13,m11,m33,m13,L,mu2) * ( 1./12.*m13*
         RD3456ep*RS14563pe - 1./24.*m13*RC1456pe*RD3456ep );

      F13p6x23RV +=  + hh27dV(4,m34,m14,m33,m13,L,mu2) * ( 1./8.*m13*
         RD3456ep );

      F13p6x23RV +=  + hh27dV(4,m35,m15,m33,m13,L,mu2) * ( 1./8.*m13*
         RD3456ep );

      F13p6x23RV +=  + hh27dV(4,m36,m16,m33,m13,L,mu2) * ( 1./8.*m13*
         RD3456ep );

      F13p6x23RV +=  + hh27dV(7,m13,m11,m11,m13,L,mu2) * (  - 1./48.*
         m13*pow(RD1456ep,2) );

      F13p6x23RV +=  + hh27dV(7,m13,m11,m33,m13,L,mu2) * (  - 1./24.*
         m13*RD1456ep*RD3456ep );

      F13p6x23RV +=  + hh27dV(7,m13,m33,m33,m13,L,mu2) * (  - 1./48.*
         m13*pow(RD3456ep,2) );

  return F13p6x23RV/pow(f0,4);
}

